//==========================================================================
//
//      flash_program_buf.c
//
//      Flash programming
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    lagarwal
// Contributors:
// Date:         2002-03-08
// Purpose:      
// Description:  contains routine to write flash device
//              
//####DESCRIPTIONEND####
//
//==========================================================================

#include "flash.h"

#include <pkgconf/hal.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/hal/hal_cache.h>

extern void printf(char *fmt, ...);

extern cyg_uint32 silicon_rev;

//
// CAUTION!  This code must be copied to RAM before execution.  Therefore,
// it must not contain any code which might be position dependent!
//

int
flash_program_buf(volatile unsigned char *addr, unsigned char *data, int len)
{
    volatile unsigned char *ROM;
    volatile unsigned char *BA;
    unsigned char stat = 0;
    int timeout = 5000000;
    int i, wc;

#ifdef CYGHWR_IO_FLASH_DEVICE_NOT_IN_RAM
    int cache_on;

    HAL_DCACHE_IS_ENABLED(cache_on);
    if (cache_on) {
        HAL_DCACHE_SYNC();
        HAL_DCACHE_DISABLE();
    }
#endif

    ROM = (unsigned char *)FLASH_BASE;

    BA  = (unsigned char *)((unsigned long)addr & ~(FLASH_BLOCK_SIZE -1));

    // Clear any error conditions
    ROM[0] = FLASH_Clear_Status;

    wc = 32;
    while (len >= wc) {
        len -= wc;

        BA[0] = FLASH_Write_Buffer;
        timeout = 5000000;
        while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready) {
            if (--timeout == 0) {
				printf("stat = 0x%02x\n", stat);
                goto bad;
            }
            BA[0] = FLASH_Write_Buffer;
        }
        BA[0] = wc-1;  // Count is 0..N-1

		// writing to reverse byte order is not working. So, try writing in order
        if(((silicon_rev & 0xF0) >> 4))
        {
            // B0 rev. onwards silicon have slowport write endianness fix.
            for (i = 0;  i < wc;  i++)
			    addr[i] = data[i];
        }
        else
        {
	        for (i = 0;  i < wc;  i++)
			    addr[(i^3)] = data[i];
        }
		addr += 32;
		data += 32;

        BA[0] = FLASH_Confirm;

		ROM[0] = FLASH_Read_Status;
		timeout = 5000000;
		while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready) {
			if (--timeout == 0) {
				printf("stat = 0x%02x\n", stat);
				goto bad;
			}
		}
    }

    while (len > 0) {
        ROM[0] = FLASH_Program;

        if(((silicon_rev & 0xF0) >> 4))
        {
            // B0 rev. onwards silicon have slowport write endianness fix.
            *addr = *data;
        }
        else
        {
	        *((unsigned char *)((int)addr^3)) = *data;
        }
		addr++;
		data++;

        timeout = 5000000;
        while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready) {
            if (--timeout == 0) {
				printf("stat = 0x%02x\n", stat);
                goto bad;
            }
        }
        --len;
    }
    // Restore ROM to "normal" mode
bad:
    ROM[0] = FLASH_Reset;            

#ifdef CYGHWR_IO_FLASH_DEVICE_NOT_IN_RAM
    if (cache_on) {
        HAL_DCACHE_ENABLE();
    }
#endif

    return stat;
}
