/* post.h
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXDP2400
subsystem: BootMonitor
author: gvaddadi
revisions:
--------------------------------------------------------------------------
*/
#include <cyg/hal/hal_ixdp2400.h>       // Platform specific hardware definitions

//I2C defines -start
#define CLOCK 0x80
#define DATA  0x40

#define I2C_WRITE_MODE 		0x00
#define I2C_READ_MODE		0x01
#define DELAY_I2C			50
#define I2C_INIT_BIT		0x04 

//I2C defines -end

#define SCRATCH_TEST_END_ADDR	SCRATCHPAD_BASE+ 0x100 //256 bytes
#define SDRAM_TEST_END_ADDR		0x100000 //1MB

//macro to initialise the UART	
.MACRO  START_UART reg0, reg1
	ldr	\reg1, =IXP2400_UART_ADDR				/* base address of UART */
	
	ldr \reg0, =0x80808080						/* enable access to divisor registers */
	str	\reg0, [\reg1, #UART_LineControl]
	
	ldr \reg0, =DEFAULT_BAUD_DIVISOR
	str	\reg0, [\reg1, #UART_DivisorLatchLSB]
	ldr	\reg0, =0x00000000
	str	\reg0, [\reg1, #UART_DivisorLatchMSB]
	
	ldr	\reg0, =0x03030303                  	/* 8 data, 1 stop bit, no parity */
	str	\reg0, [\reg1, #UART_LineControl]       /* also disables access to divisor regs */
	
	ldr	\reg0, =0x40404040
	str	\reg0, [\reg1, #UART_InterruptEnable]   /* no IRQs just yet */
	
	ldr	\reg0, =0x01010101
	str	\reg0, [\reg1, #UART_FIFOControl]       /* set bit 0 */
	
	ldr	\reg0, =0x07070707
	str	\reg0, [\reg1, #UART_FIFOControl]       /* turn FIFOs on */
	
	ldr	\reg0, =0x00000000                      /* clear the THRE bit for TI UARTs */
	str	\reg0, [\reg1, #UART_LineStatus]
.ENDM

//macro for uart Tx
.MACRO UARTTx reg0, reg1, reg2
	ldr \reg1, =IXP2400_UART_ADDR                /* base address of UART */
30:
	ldr \reg2, [\reg1, #UART_LineStatus]
    tst \reg2, #0x20
    beq 30b                                      /*  and wait for the all clear */
    str \reg0, [\reg1, #UART_Transmit]           /* Tx character in reg0 */
.ENDM

/* ----------- I2C protocol driver macros-begin ----------- */

// Macro to write to GPIO registers
.macro GPIO_REG_WRITE	reg0, reg1, greg, data
	ldr		\reg1, =\greg	//load the GPIO register addr 
	ldr		\reg0, =\data	//load the value to be written
	str		\reg0, [\reg1]	//write to the register
.endm

//Macro to read the level on a particuler GPIO pin
.macro GPIO_REG_READ  reg0, reg2, gpioreg, bitmsk	
	ldr		\reg0, =\gpioreg	//load the GPIO register to be read
	ldr		\reg2, [\reg0]		//read the register value
	ldr		\reg0, =\bitmsk		//load the bit mask
	and		\reg2, \reg2, \reg0	//store the bit value
.endm

//macro that generates delay
.macro I2C_DELAY ntimes, reg8
	ldr	\reg8, =\ntimes
10:
	subs	\reg8, \reg8, #1
	bpl		10b
.endm

//macro to generate START condition in a i2c transaction
.macro I2C_START reg0, reg1
	//SDA=1, SCL=1
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, DATA
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, CLOCK
	I2C_DELAY DELAY_I2C, r8

	//SCL=1, SDA=0 . since SCL is alreday 1 , i am setting SDA to 0
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, DATA
	I2C_DELAY DELAY_I2C, r8

	//SCL=0;SDA=0 since SDA is already low, i am setting SCL to 0
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, CLOCK
	I2C_DELAY DELAY_I2C, r8
.endm

//macro to generate a STOP condition in a i2c transaction
.macro I2C_STOP reg0, reg1
	//SCL=0, SDA=0
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, DATA
	I2C_DELAY DELAY_I2C, r8

	//SCL=1, SDA=0
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, CLOCK
	I2C_DELAY DELAY_I2C, r8

	//SCL=1,SDA=1
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, DATA
	I2C_DELAY DELAY_I2C, r8
.endm

.macro I2C_WRITEBIT reg0, reg1, reg2
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, CLOCK
	I2C_DELAY DELAY_I2C, r8

	tst \reg2, #0x80
	beq 12f
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, DATA
	b 13f

12:
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, DATA

13:
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, CLOCK
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, CLOCK
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, DATA
.endm

//macro to put 8 bits on the i2c bus
.macro I2C_WRITEBYTE data, reg0, reg1, reg2
	//load data to be written into reg2
	ldr \reg2, =\data
	ldr r7, =0x0	// used by for loop

14:
	I2C_WRITEBIT \reg0, \reg1, \reg2
	mov		\reg2, \reg2, LSL #1
	add		r7, r7, #1
	cmp		r7, #8
	bne		14b
	
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, DATA
.endm

.macro I2C_READBIT reg0, reg1, reg2
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, CLOCK
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDCR, CLOCK
	I2C_DELAY DELAY_I2C, r8
	GPIO_REG_READ \reg0, \reg1, GPIO_PLR, DATA

	tst \reg1, #DATA
	bne 10f
	ldr \reg2, =0x0
	b 11f

10:
	ldr \reg2, =0x1

11:
	GPIO_REG_WRITE \reg0, \reg1, GPIO_PDSR, CLOCK
	I2C_DELAY DELAY_I2C, r8
.endm

//macro to get 8 bits from the i2c bus
.macro I2C_READBYTE reg0, reg1, reg2, reg4
	//initilaize r4 with zero, this will contain the read byte
	ldr		\reg4, =0x0
	ldr		r7, =0x0	// use by for loop
228:
	I2C_READBIT \reg0, \reg1, \reg2
	orr		\reg4, \reg4, \reg2
	mov		\reg4, \reg4, LSL #1
	add		r7, r7, #1
	cmp		r7, #7
	bne		228b

	I2C_READBIT \reg0, \reg1, \reg2
	orr		\reg4, \reg4, \reg2
.endm

.macro I2C_DEVICE_BUSY slave, reg0, reg1, reg2
	ldr r6, =0x0	// use by while loop
	ldr \reg2, =0x1	// set as busy
15:
	I2C_START \reg0, \reg1
	I2C_WRITEBYTE \slave, \reg0, \reg1, \reg2
	I2C_READBIT \reg0, \reg1, \reg2
	cmp \reg2, #0x0	// check if busy
	beq 17f			// branch if not busy

	I2C_STOP \reg0, \reg1
	add r6, r6, #1
	cmp r6, #4
	bne 15b
17:
.endm

//macro that implements the i2c protocol to read a byte from the i2c device
.macro I2C_READ slave, addr, reg4
	I2C_DEVICE_BUSY \slave, r0, r1, r2
	cmp r2, #0x0
	bne 18f

	I2C_WRITEBYTE \addr, r0, r1, r2
	I2C_READBIT	r0, r1, \reg4
	I2C_DELAY DELAY_I2C, r8

	I2C_START r0, r1
	I2C_WRITEBYTE \slave|I2C_READ_MODE, r0, r1, r2
	I2C_READBIT r0, r1, \reg4

	I2C_READBYTE r0, r1, r2, \reg4
	ldr r2, =0x80
	I2C_WRITEBIT r0, r1, r2
	I2C_STOP r0, r1
	b 18f

	.ltorg
18:
.endm

.macro I2C_WRITE slave, addr, data
	I2C_DEVICE_BUSY \slave, r0, r1, r2
	cmp r2, #0x0
	bne 19f

	I2C_WRITEBYTE \addr, r0, r1, r2
	I2C_READBIT	r0, r1, r4
	I2C_DELAY DELAY_I2C, r8

	I2C_WRITEBYTE \data, r0, r1, r2
	I2C_READBIT	r0, r1, r4
	I2C_STOP r0, r1
	b 19f

	.ltorg
19:
.endm

.macro I2C_INIT0
	/* initially at power up SDA has pull down because it is used as strap pin.
	 * the PCI_ARB strap pin which is GPIO 2 should be driven high inorder to 
     * have a pull up on SDA line. This is required by this board design
     */
    
    /*configure GPIO 6 as output*/
	GPIO_REG_WRITE r0,r1, GPIO_PDSR, I2C_INIT_BIT

	/*drive this pin high*/
	GPIO_REG_WRITE r0,r1, GPIO_POSR, I2C_INIT_BIT

	/*configure pin 7 and 6 to be output*/
	GPIO_REG_WRITE r0,r1, GPIO_PDSR, (CLOCK|DATA)

	/*make the output level on pins gpio 6 and 7 low*/
	GPIO_REG_WRITE r0,r1, GPIO_POCR, (CLOCK|DATA)

	/*make pins 6 and 7 as input during init*/
	GPIO_REG_WRITE r0,r1, GPIO_PDCR, CLOCK

	I2C_DELAY 10, r8
	
	GPIO_REG_WRITE r0,r1, GPIO_PDCR, DATA
	
	I2C_DELAY 0x1000, r8

.endm

//End of macros for I2C


//macro that does the POST		
.macro POST
	ldr	r0, =SLAVE_POST_FLAG_LOC
	ldr	r1, =0x0
	str	r1, [r0]

	START_UART  r0, r1

	/* ---------- XScale Scratchpad test ---------- */
	/* -------------------------------------------- */
	bl	PrintString
        .ascii  "\n"
        .ascii  " Testing Xscale Scratchpad memory... \0"
        .align 2
	
	ldr r5, =0x5A5A5A5A       //pattern 
	ldr r8, =SCRATCHPAD_BASE  //load scratch base
    ldr r7, =SCRATCH_TEST_END_ADDR 

10:
	//initialize entire memory to be tetsed with the pattern
    str r5, [r8], #0x4
    cmp r8, r7
    bne 10b

	// Check for pattern going from START to END address, while loading inverse pattern
	ldr r8, =SCRATCHPAD_BASE  //load scratch base
112:
    ldr r6, [r8]        //read back the value at addr in r4 to r6 and increment the addrin r4
	cmp r6,r5           //compare the read and written values
	blne 223f           //scratch_err
    ldr r6, = 0xA5A5A5A5
    str r6, [r8], #0x4  //store the inverse pattern and increment addr
    cmp r8, r7
	bne 112b

	// Check for inverse pattern going from END to START
	ldr r7, =SCRATCHPAD_BASE  //load scratch base
	ldr r5, =0xA5A5A5A5	// inverse pattern
113:
	sub r8,r8, #0x4		//load the last addr
    ldr r6,[r8]			//read test location 
    cmp r6, r5			//compare with inverse pattern
    blne 223f			//error 
    cmp r8, r7
    bne 113b

	HEX_DISPLAY 'X', 'S', 'C', 'P'
	bl	PrintString
		.ascii  "PASS\n\0"
		.align 2
	b 224f

223:
	mov r11, lr
	HEX_DISPLAY 'X', 'S', 'C', 'F'
	bl	PrintString
		.ascii  "\n"
		.ascii  " Xscale Scratch Test failed at address 0x\0"
		.align 2
	mov r0, r8         //move the failed addr from r4 to r0 to print
	bl	PrintHex32 //prints the addr in r0

	bl	PrintString
		.ascii  "\n Expected Value: 0x\0"
		.align 2
	mov r0, r5         //move the expected value from r1 to ro to print
	bl	PrintHex32 //prints the addr in r0
 

	bl	PrintString
		.ascii  "\n Actual Value:   0x\0"
		.align 2
	mov r0, r6         //move the actual value from r2 to ro to print
	bl	PrintHex32 //prints the addr in r0
	bl	PrintString
		.ascii  "\n\0"
		.align 2
	mov pc, r11

224:
	/* ---------- I2C TEST ---------- */
	/* ------------------------------ */
	I2C_INIT0

	bl	PrintString
	    .ascii  " Testing I2C Interface... \0"
	    .align 2

	I2C_DEVICE_BUSY 0xa0, r0, r1, r2
	cmp r2, #0x0
	bne 1224f

	I2C_STOP r0, r1

	HEX_DISPLAY 'I', '2', 'C', 'P'
	bl	PrintString
	    .ascii  "PASS\n\0"
	    .align 2

	b 225f

1224:
	HEX_DISPLAY 'I', '2', 'C', 'F'
	bl	PrintString
	    .ascii  "FAIL\n\0"
	    .align 2

225:		
	/* ---------- SDRAM TEST ----------- */
	/* --------------------------------- */
sdram_post:
//#define DEBUG
#ifdef DEBUG
	bl	PrintString
		.ascii  "DU Control: 0x\0"
		.align 2
	ldr r1, =DU_CONTROL
	ldr r0, [r1]
	bl	PrintHex32
	bl	PrintString
		.ascii  "\n\0"
		.align 2
	bl	PrintString
		.ascii  "CCR: 0x\0"
		.align 2
	ldr r1, =CCR
	ldr r0, [r1]
	bl	PrintHex32
	bl	PrintString
		.ascii  "\n\0"
		.align 2
#endif
	cmp r10, #0		// if single boot, slave skips SDRAM POST
	beq calc_dram_size

	bl	PrintString
		.ascii  " Testing SDRAM memory... \0"
		.align 2

	ldr r5, =0x5A5A5A5A       //pattern 
	ldr r8, =0x0              //load SDRAM base addr
	ldr r7, =SDRAM_TEST_END_ADDR 


110:
	//initialize entire memory to be tested with the pattern
	str r5, [r8], #0x4
	cmp r8, r7
	bne 110b


	// Check for pattern going from START to END address, while loading inverse pattern
	ldr r8, =0x0            //load SDRAM BASE
111: 
	ldr r6, [r8]        //read back the value at addr in r4 to r6 and increment the addrin r4
	cmp r6, r5          //compare the read and written values
	bne 226f           //sdram_err
	ldr r6, =0xA5A5A5A5
	str r6, [r8], #0x4  //store the inverse pattern and increment addr
	cmp r8, r7
	bne 111b

	// Check for inverse pattern going from END to START
	ldr r7, =0x0            //load SDRAM BASE
	ldr r5, =0xA5A5A5A5	// inverse pattern
113:
	sub r8,r8, #0x4		//load the last addr
	ldr r6,[r8]			//read test location 
	cmp r6, r5			//compare with inverse pattern
	bne 226f			//error 
	cmp r8, r7
	bne 113b

	HEX_DISPLAY 'D', 'D', 'R', 'P'
	bl	PrintString
		.ascii  "PASS\n\0"
		.align 2
	b 227f

226:
	HEX_DISPLAY 'D', 'D', 'R', 'F'
	bl	PrintString
		.ascii  " \nSDRAM Test failed at address 0x\0"
		.align 2
	mov	r0, r8
	bl	PrintHex32 //prints the addr in r0

	bl	PrintString
		.ascii  "\n Expected Value: 0x\0"
		.align 2
	mov     r0, r5
	bl	PrintHex32

	bl	PrintString
		.ascii  "\n Actual Value:   0x\0"
		.align 2
	mov     r0, r6
	bl	PrintHex32	
	bl	PrintString
		.ascii  "\n\0"
		.align 2
hang_loop:
	b hang_loop

227:
	b calc_dram_size
	.ltorg

	/* ---------- CALCULATE SDRAM ----------- */
	/* -------------------------------------- */
calc_dram_size:
	I2C_READ 0xa0, 0x5, r5
	I2C_READ 0xa0, 0x1F, r6

	ldr r7, =0x0
	ldr r1, =0x1
50:
	and r0, r6, r1
	cmp r0, #0x0
	bne 55f

	mov	r1, r1, LSL #1
	add r7, r7, #0x1
	cmp r7, #0x8
	bne 50b

55:
	ldr r8, =0x400000	// 1 << 22
60:
	cmp r7, #0x0
	beq 65f
	mov r8, r8, LSL #1
	sub r7, r7, #0x1
	b 60b
65:
	cmp r5, #0x1
	beq 70f
	add r8, r8, r8
	sub r5, r5, #0x1
	b 65b
70:
	b post_done

PrintString:                            
115:
	ldrb    r0, [lr], #1
	//the following chunk enables LF to CR+LF translation
	teq     r0, #0x0A               // LF 
	bne     120f
	mov     r0, #0x0D               // CR             
	UARTTx  r0, r1, r2
	mov     r0, #0x0A               // LF             

120:
	teq     r0, #0                  // test for the string termination character (0) 
	beq     130f
	UARTTx  r0, r1, r2
	b       115b
130:
	add     lr, lr, #3             // round up to next word boundary 
	bic     pc, lr, #3

	/*
	* Print 32-bit hex value subroutine
	* in:  r0 = value to print in hex
	*/

PrintHex32:
	mov     r2, lr
	mov     r1, r0, LSR #28
	bl	PrintHexDigit
	mov     r1, r0, LSR #24
	bl	PrintHexDigit
	mov     r1, r0, LSR #20
	bl	PrintHexDigit
	mov     r1, r0, LSR #16
	bl	PrintHexDigit
	mov     r1, r0, LSR #12
	bl	PrintHexDigit
	mov     r1, r0, LSR #04
	mov     r1, r1, LSR #04
	bl	PrintHexDigit
	mov     r1, r0, LSR #04
	bl	PrintHexDigit
	mov     r1, r0
	bl	PrintHexDigit
	mov     pc, r2

PrintHexDigit:
	and     r1, r1, #0x0F
	cmp     r1, #10
	addcc   r1, r1, #'0'
	addcs   r1, r1, #'A'-10
	UARTTx  r1, r3, r4
	mov     pc, lr

post_done:
#ifdef DEBUG
	bl	PrintString
		.ascii  "DDR Size: 0x\0"
		.align 2
	mov r0, r8
	bl	PrintHex32
	bl	PrintString
		.ascii  "\n\0"
		.align 2
#endif
	bl	PrintString
		.ascii  "Scrubbing SDRAM\n\0"
		.align 2

	ldr	r0, =SLAVE_POST_FLAG_LOC
	ldr	r1, =POST_DONE
	str	r1, [r0]
.endm	// end of POST macro
