/* ixdp2400_pci.c
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXDP2400
subsystem: BootMonitor
author: lagarwal
revisions:
--------------------------------------------------------------------------
*/

#include <pkgconf/hal.h>
#include <pkgconf/system.h>
#include CYGBLD_HAL_PLATFORM_H
#include CYGHWR_MEMORY_LAYOUT_H

#include <cyg/infra/cyg_type.h>         // base types
#include <cyg/infra/cyg_trac.h>         // tracing macros
#include <cyg/infra/cyg_ass.h>          // assertion macros

#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_if.h>             // calling interface API
#include <cyg/hal/hal_arch.h>           // Register state info
#include <cyg/hal/hal_intr.h>           // Interrupt names
#include <cyg/hal/hal_cache.h>
#include <cyg/hal/hal_ixdp2400.h>        // Hardware definitions
#include <cyg/io/pci_hw.h>
#include <cyg/io/pci.h>
#include <cyg/hal/cfg_utility.h>
#include <cyg/hal/ntb_srom_util.h>
#include <redboot.h>
#include <flash_config.h>

// define this to leave Slave in reset (in single flash mode)
//#define DONT_INIT_SLAVE

externC cyg_uint32 ext_osc_freq;
cyg_pci_device slave_dev_info;
extern struct board_config *board_config_data;
extern struct _config config;
externC char __rom_data_start[], __ram_data_start[], __ram_data_end[];

void ntb_init(void)
{
    cyg_pci_device_id dev_id = CYG_PCI_NULL_DEVID;

    if(!cyg_pci_find_device(NTB_VENDOR_ID, NTB_DEVICE_ID, &dev_id))
    {
        printf("Couldn't find 21555\n");
        return;
    }

    if(strap_options_val() & CFG_PCI_BOOT_HOST)
	{
#if 0
        // check to see if SROM is blank
        if(read_srom_byte(0) == 0xFF)
        {
            char srom_data[] = {0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x86, 0x80, 0x00, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00,
                                0x00, 0x00, 0x00, 0xF0, 0xFF, 0xFF, 0x00, 0x00, 0xF0, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
                                0xF0, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xF1, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                                0x00, 0x08, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                                0x00, 0x00, 0x00};

            printf("Found blank 21555 SROM. Going to program it..\n");

            // if blank then program default data
            // default data program 1M of DS_IOMEM1 and DS_MEM3 BAR and 10M of DS_MEM3 BAR
            write_srom((char *)srom_data, sizeof(srom_data));
        }
#endif

		printf("Mapping CSR for mapping from host\n");
        // master's CSR and SDRAM will be mapped using downstream BAR 1 and 2 respectively
        cyg_pci_write_config_uint32(dev_id, DS_IOMEM_1_TBASE, (*(PCI_CSR_BAR_REG) & ~0xF));
//#define MAP_SRAM
#ifdef MAP_SRAM
		printf("Mapping SRAM for mapping from host\n");
		cyg_pci_write_config_uint32(dev_id, DS_MEM_2_TBASE, (*(PCI_SRAM_BAR_REG) & ~0xF));
		
//#define TEST_MAP_SRAM
#ifdef TEST_MAP_SRAM
		printf("Init SRAM for testing\n");
		*((volatile cyg_uint32 *)(0x80000000)) = 0x12345678;
		*((volatile cyg_uint32 *)(0x80000004)) = 0x12345678;
#endif
#else
		printf("Mapping SDRAM for mapping from host\n");
        cyg_pci_write_config_uint32(dev_id, DS_MEM_2_TBASE, (*(PCI_DRAM_BAR_REG) & ~0xF));
#endif
//#define TEST_21555_PRI
#ifdef TEST_21555_PRI
#define NTB_PRI_COMMAND			0x44
#define DSTREAM_MEM_1_PRI_BAR	0x18
#define DSTREAM_MEM_2_PRI_BAR	0x1C
#define DSTREAM_MEM_3_PRI_BAR	0x20
#define DSTREAM_MEM_1_SEC_BAR	0x58
#define DSTREAM_MEM_2_SEC_BAR	0x5C
#define DSTREAM_MEM_3_SEC_BAR	0x60
#define CONFIG_CSR				0x92
#define UPSTREAM_CONFIG_ADDR	0x88
#define UPSTREAM_CONFIG_DATA	0x8C
#define NTB_PRI_IDSEL			17

		// enable IO and Mem transactions from primary side
		cyg_pci_write_config_uint16(dev_id, NTB_PRI_COMMAND, CYG_PCI_CFG_COMMAND_IO |
			CYG_PCI_CFG_COMMAND_MEMORY | CYG_PCI_CFG_COMMAND_MASTER);
		// enable upstream config cycles and self addressing
		cyg_pci_write_config_uint16(dev_id, CONFIG_CSR, (3 << 9));

		// assign PCI address to downstream BARs
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_ADDR, ((1 << NTB_PRI_IDSEL) | DSTREAM_MEM_1_PRI_BAR));
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_DATA, 0x80000000);
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_ADDR, ((1 << NTB_PRI_IDSEL) | DSTREAM_MEM_3_PRI_BAR));
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_DATA, 0x80100000);
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_ADDR, ((1 << NTB_PRI_IDSEL) | DSTREAM_MEM_2_PRI_BAR));
		cyg_pci_write_config_uint32(dev_id, UPSTREAM_CONFIG_DATA, 0x81000000);
#endif
    }
    else
    {
		printf("Mapping CSR for mapping from host\n");
        // slave's CSR will be mapped using downstream BAR3
        cyg_pci_write_config_uint32(dev_id, DS_MEM_3_TBASE, (*(PCI_CSR_BAR_REG) & ~0xF));
    }
}

void slave_ixp_init(void)
{
	int i, temp;
	int slave_sdram_passed = 1;

	/* PCI unit does a byte swapping by default. But we don't want to swap bytes when copying into slave's memory.
	So set these bits so that PCI unit doesn't do byte swapping. */
	*(PCI_CONTROL_REG) |= (PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);

	// SDRAM, SRAM and CSR bars are mapped in master's pci mem space.
	// first BAR is CSR, second is SRAM and third is SDRAM
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_CONTROL_FRM_PCI)) |= (PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);

	// check to see if slave has flash or not
	if((*(volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + STRAP_OPTIONS_FRM_PCI) & CFG_PROM_BOOT))
	{
		printf("Dual flash system detected\n");
	}
	else
	{
		// slave doesn't have flash. So, start slave init
		printf("Single flash system detected\n");
		
		// enable outbound pci int. from xscale
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_OUT_INT_MASK_FRM_PCI)) &= XSIM;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_OUT_INT_MASK_FRM_PCI));
#endif
		
		// set the upper bits for pci mem and io transactions
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_ADDR_EXT_FRM_PCI)) = 0x0;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_ADDR_EXT_FRM_PCI));
#endif
		
		// slave pci init is complete. So start accepting pci transactions
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI)) |= INIT_COMP;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI));
#endif
		
		// now I need to config SDRAM/SRAM also. So, that I can download image in SDRAM.
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RCOMP_IO_CONFIG_FRM_PCI)) = DDR_RCOMP_IO_CONFIG_VAL;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RCOMP_IO_CONFIG_FRM_PCI));
#endif

		// setup operating freq. for SDRAM/SRAM interface
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI));
		temp &= ~(0xF00FF);
		if(ext_osc_freq == 100000000)
		{
			temp |= ((CCR_150MHZ_DIVISOR << 16) | (CCR_150MHZ_DIVISOR << 4) | CCR_150MHZ_DIVISOR);
		}
		else
		{
			temp |= ((CCR_150MHZ_DIVISOR << 16) | (CCR_200MHZ_DIVISOR << 4) | CCR_200MHZ_DIVISOR);
		}
#else
		temp = *CCR_REG;
#endif
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI)) = temp;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI));
#endif
		hal_delay_us(1000); // 1ms

		// now config SDRAM interface
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RDDLYSEL_RECEN_FRM_PCI)) = DDR_RDDLYSEL_RECEN_VAL;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RDDLYSEL_RECEN_FRM_PCI));
#endif
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DLL_FRM_PCI)) = DDR_RX_DLL_VAL;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DLL_FRM_PCI));
#endif
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DESKEW_FRM_PCI)) = DDR_RX_DESKEW_VAL;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DESKEW_FRM_PCI));
#endif
        // cr0_dstrengthsel
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DSTRENGTHSEL_FRM_PCI)) = 0x036db6db;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DSTRENGTHSEL_FRM_PCI));
#endif
        
        // cr0_ddqrcomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DDQRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DDQRCOMP_FRM_PCI));
#endif
        
        // cr0_dctlrcomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCTLRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCTLRCOMP_FRM_PCI));
#endif
        
        // cr0_drcvrcomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DRCVRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DRCVRCOMP_FRM_PCI));
#endif
        
        // cr0_dckercomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKERCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKERCOMP_FRM_PCI));
#endif
        
        // cr0_dcsrcomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCSRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCSRCOMP_FRM_PCI));
#endif
        
        // cr0_dckrcomp
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKRCOMP_FRM_PCI));
#endif
        
        // cr0_dx8x16ckecscksel
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DX8X16CKECSCKSEL_FRM_PCI)) = 0x11111111;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DX8X16CKECSCKSEL_FRM_PCI));
#endif
        
        // cr0_rcompprd
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_RCOMPPRD_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_RCOMPPRD_FRM_PCI));
#endif
        
        // cr0_digfil
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DIGFIL_FRM_PCI)) = 0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DIGFIL_FRM_PCI));
#endif
        
        /* Program DDR DQ/DQS pull-up and pull-down Slew Lookup table registers */
        /* (cr0_ddqpslew0 - cr0_ddqpslew3) and (cr0_ddqnslew0 - cr0_ddqnslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x150); i <= (DRAM_CH0_BASE_FRM_PCI + 0x188); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program MA/BA/RAS#/CAS#/WE# pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dctlpslew0 - cr0_dctlpslew3) and (cr0_dctlnslew0 - cr0_dctlnslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x198); i <= (DRAM_CH0_BASE_FRM_PCI + 0x1D0); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program RCV pull-up and pull-down Slew Lookup table registers */
        /* (cr0_drcvpslew0 - cr0_drcvpslew3) and (cr0_drcvnslew0 - cr0_drcvnslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x1E0); i <= (DRAM_CH0_BASE_FRM_PCI + 0x218); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CKE x8 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dckex8pslew0 - cr0_dckex8pslew3) and (cr0_dckex8nslew0 - cr0_dckex8nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x230); i <= (DRAM_CH0_BASE_FRM_PCI + 0x268); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CKE x16 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dckex16pslew0 - cr0_dckex16pslew3) and (cr0_dckex16nslew0 - cr0_dckex16nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x270); i <= (DRAM_CH0_BASE_FRM_PCI + 0x2A8); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CS# x8 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dcsx8pslew0 - cr0_dcsx8pslew3) and (cr0_dcsx8nslew0 - cr0_dcsx8nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x2B8); i <= (DRAM_CH0_BASE_FRM_PCI + 0x2F0); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CS# x16 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dcsx16pslew0 - cr0_dcsx16pslew3) and (cr0_dcsx16nslew0 - cr0_dcsx16nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x2F8); i <= (DRAM_CH0_BASE_FRM_PCI + 0x330); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CK CK# x8 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dckx8pslew0 - cr0_dckx8pslew3) and (cr0_dckx8nslew0 - cr0_dckx8nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x340); i <= (DRAM_CH0_BASE_FRM_PCI + 0x378); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Program CK CK# x16 pull-up and pull-down Slew Lookup table registers */
        /* (cr0_dckx16pslew0 - cr0_dckx16pslew3) and (cr0_dckx16nslew0 - cr0_dckx16nslew3) */
        for(i = (DRAM_CH0_BASE_FRM_PCI + 0x380); i <= (DRAM_CH0_BASE_FRM_PCI + 0x3B8); i += 8)
        {
            *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
            temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
        }
        
        /* Flag Slew Programmed register as done */
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_SLEWPROGRAMMED_FRM_PCI)) = 0x1;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_SLEWPROGRAMMED_FRM_PCI));
#endif
        
        /* Force SM Rcomp */
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI)) = 0x1;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI));
#endif
        hal_delay_us(1000);
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI)) = 0x0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI));
#endif
        
        /* cr0_ovrrideh: turn off h override, 0x0 */
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEH_FRM_PCI)) = 0x0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEH_FRM_PCI));
#endif
        
        /* cr0_ovrridev: turn off v override, 0x0 */
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEV_FRM_PCI)) = 0x0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEV_FRM_PCI));
#endif
        
        /* De-Select Test Mode (cr0_jt_config)
        // User Overide RCOMP settings */
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_JT_CONFIG_FRM_PCI)) = 0x0;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_JT_CONFIG_FRM_PCI));
#endif
        
        hal_delay_us(1000); // 1ms
        
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_CONTROL_FRM_PCI)) = *DU_CONTROL_REG;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_CONTROL_FRM_PCI));
#endif
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = CKE;
#ifdef A0_REV
        temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI));
#endif
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL);
        while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & PRECHARGE)
            hal_delay_us(1000); // 1ms
        
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | EXT_LD_MODE);
        while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
            hal_delay_us(1000); // 1ms
        
        *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | RESET_DLL);
        while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
			hal_delay_us(1000); // 1ms

		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL);
		while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & PRECHARGE)
			hal_delay_us(1000); // 1ms

		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (REFRESH | SIDE1 | SIDE0);
		while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & REFRESH)
			hal_delay_us(1000); // 1ms

		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (REFRESH | SIDE1 | SIDE0);
		while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & REFRESH)
			hal_delay_us(1000); // 1ms
		
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | LOAD_MODE_NORMAL);
		while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
			hal_delay_us(1000); // 1ms

		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_ECC_TEST_FRM_PCI)) |= DISABLE_CHK;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + DU_ECC_TEST_FRM_PCI));
#endif
		*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + MISC_CONTROL_FRM_PCI)) |= FLASH_ALIAS_DISABLE;
#ifdef A0_REV
		temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + MISC_CONTROL_FRM_PCI));
#endif

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;
		// check slave's SDRAM
		for(i = 0; i < 0x100000; i = i + 4)
		{
			*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i)) = 0x5A5A5A5A;
			
			// read it back
			temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
			if(temp != 0x5A5A5A5A)
			{
				printf("Slave SDRAM POST failed. addr = 0x%08X, actual = 0x%08X, expected = 0x5A5A5A5A\n", i, temp);
				slave_sdram_passed = 0;
				break;
			}
		}
		
		if(slave_sdram_passed)
		{
			cyg_uint32 sdram_data_end, sdram_data_start, flash_data_start;

			sdram_data_end = (cyg_uint32)&__ram_data_end;
			sdram_data_start = (cyg_uint32)&__ram_data_start;
			flash_data_start = (cyg_uint32)&__rom_data_start;

			// now download BM to slave's SDRAM.
			printf("Slave SDRAM POST passed\n");
			
			*(SP_FRM_REG) = USE_32_BIT_DATA;
			// copy the text section
			for(i = 0; i < CYGMEM_REGION_rom_SIZE; i = i + 4)
			{
				*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i)) = *((volatile cyg_uint32 *)(SLOW_PORT_BASE + i));
#ifdef A0_REV
				// read it back to prevent PCI Mem bug
				temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
#endif
			}
			// copy the data section
			for(i = 0; i < (sdram_data_end - sdram_data_start); i = i + 4)
			{
				*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + sdram_data_start + i)) = *((volatile cyg_uint32 *)(SLOW_PORT_BASE + flash_data_start + i));
#ifdef A0_REV				
				// read it back to prevent PCI Mem bug
				temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + sdram_data_start + i));
#endif
			}

			// scrub only 1M
			for(i = sdram_data_end; i < 0x100000; i = i + 4)
			{
				*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i)) = 0;
#ifdef A0_REV
				// read it back to prevent PCI Mem bug
				temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
#endif
			}
			*(SP_FRM_REG) = USE_8_BIT_DATA;

			// copy flash config
			for(i = 0; i < MAX_CONFIG_DATA; i = i + 4)
			{
				*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + (cyg_uint32)&config + i)) = *((volatile cyg_uint32 *)((cyg_uint32)&config + i));
#ifdef A0_REV
				// read it back to prevent PCI Mem bug
				temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + (cyg_uint32)&config + i));
#endif
			}
			
			*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + SLAVE_POST_FLAG_LOC)) = 0;
			
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
			// final step - take slave out of reset and I am done with slave.
			*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI))	&= ~(RESET_XSCALE);
		}
				
	}
	
	// now wait to see if slave completes its POST
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;

	i = 0;
	printf("Waiting for slave to complete POST.");
	while(*((volatile cyg_uint32 *)(slave_dev_info.base_map[SDRAM_BAR] + SLAVE_POST_FLAG_LOC)) != POST_DONE)
	{
		// wait for 2s max
		if(i > 4)
		{
			printf("\nSlave POST failed");
			break;
		}
		// wait for 500ms
		hal_delay_us(500000);
		printf(".");
		i++;
	}
	printf("\n");

	// copy config data to slave's SRAM
	if(board_config_data->config_valid == CONFIG_DATA_VALID)
	{
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		for(i = 0; i < sizeof(struct board_config); i = i + 4)
		{
			*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + i)) = *((volatile cyg_uint32 *)((cyg_uint32)board_config_data + i));
#ifdef A0_REV
			// read it back to prevent PCI Mem bug
			temp = *((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + i));
#endif
		}
	}
#ifdef __ARMEB__
	/* 82559 is LE. So, now I need to clear byte swapping bits so that when 82559 talks to IXP2400
	pci unit does byte swapping */
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	*((volatile cyg_uint32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_CONTROL_FRM_PCI)) &= ~(PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);
	*(PCI_CONTROL_REG) &= ~(PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);
#endif
	
	return;
}

CYG_PCI_ADDRESS64 force_pci_alloc_mem(CYG_PCI_ADDRESS64 *base, int *addr_ext, cyg_uint32 size)
{
    CYG_PCI_ADDRESS64 aligned_addr;

    if(size >= 0x8000000)
    {
        (*addr_ext)++;
        aligned_addr = (*addr_ext) << 29;
    }
    else
    {
        aligned_addr = (*base + size - 1) & ~(size - 1);
        // Update the resource pointer and return values.
        *base = aligned_addr + size;
    }

    return aligned_addr;
}

void cyg_hal_plf_pci_init(void)
{
    cyg_pci_device_id dev_id = CYG_PCI_NULL_DEVID;
	cyg_pci_device_id master_dev_id;
	cyg_uint8 next_bus = SECONDARY_BUS_NUM;
	int slave_found = 0;
	int sram_bar_mask_array[] = {0x1FC0000, 0x3FC0000, 0x7FC0000, 0xFFC0000};
	int dram_bar_mask_array[] = {0x7F00000, 0xFF00000, 0x1FF00000, 0x3FF00000};
	int sram_bar_size, dram_bar_size, sram_bar, dram_bar;

	display_led('P', 'C', 'I', 'I');

	// if there is no flash then pci init. is done by master. so don't do it again
	if(strap_options_val() & CFG_PROM_BOOT)
	{
		/* set the upper bits for pci mem and io transactions */
		*(PCI_ADDR_EXT_REG) = 0x0;
				
		/* Setup SRAM Base Address Mask Reg */
		sram_bar_size = (strap_options_val() & 0xC0) >> 6;
		*(SRAM_BASE_ADDR_MASK_REG) = sram_bar_mask_array[sram_bar_size];
		sram_bar = 0x2000000 * (1 << sram_bar_size);

		/* Setup SDRAM Base Address Mask Reg */
		dram_bar_size = (strap_options_val() & 0x30) >> 4;
		*(DRAM_BASE_ADDR_MASK_REG) = dram_bar_mask_array[dram_bar_size];
		dram_bar = 0x8000000 * (1 << dram_bar_size);

		if(strap_options_val() & CFG_PCI_BOOT_HOST)
		{
            CYG_PCI_ADDRESS64 pci_base = HAL_PCI_ALLOC_BASE_MEMORY;
            int addr_ext = 0;

			*(PCI_CMD_STAT_REG) = PCI_CMD_STAT_VAL;
			*(PCI_CSR_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, 0x100000);
			*(PCI_SRAM_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, sram_bar);
			*(PCI_DRAM_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, dram_bar);
			cyg_pci_set_memory_base(pci_base, addr_ext);
		}

		/* enable outbound pci int. from xscale */
		*(PCI_OUT_INT_MASK_REG) &= XSIM;

		*(IXP_RESET0_REG) |= INIT_COMP;

		// if this is master npu then init bus
		if(strap_options_val() & CFG_PCI_BOOT_HOST)
		{
			// Initialize Primary PCI bus (bus 0)
			cyg_pci_configure_bus(0, &next_bus);

            // prepare dev_id so that we skip slave
			master_dev_id = CYG_PCI_DEV_MAKE_ID(0, CYG_PCI_DEV_MAKE_DEVFN(6, 0));
			cyg_pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &master_dev_id);

			// first npu would be slave
			dev_id = CYG_PCI_NULL_DEVID;
			// configure slave npu
			if(cyg_pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &dev_id))
			{
				if(dev_id != master_dev_id)
				{
					cyg_pci_get_device_info(dev_id, &slave_dev_info);
					slave_found = 1;
				}
			}
			
			if(slave_found)
			{
#ifndef DONT_INIT_SLAVE
				slave_ixp_init();
#else
                printf("Warning - Slave will remain in reset in single flash system\n");
#endif
			}
			else
			{
				printf("Can't find slave NPU\n");
			}
		}
		else
		{
			int i = 0;

			printf("Waiting for master to copy config data.");
			// wait till master copies the config data and assigns BAR
			while(board_config_data->config_valid != CONFIG_DATA_VALID)
			{
				if(i > 30)
				{
					printf("\nMaster hasn't copied the config data");
					break;
				}
				// wait for 1 sec.
				hal_delay_us(1000000);
				printf(".");
				i++;		
			}
			printf("\n");
		}
	}

    // setup non transparent bridge
    ntb_init();

}

// Use "naked" attribute to suppress C prologue/epilogue
static void __attribute__ ((naked)) __pci_abort_handler(void) 
{
    asm ( "subs	pc, lr, #4\n" );
}

static cyg_uint32 orig_abort_vec;

static inline cyg_uint32 *pci_config_setup(cyg_uint32 bus,
					   cyg_uint32 devfn,
					   cyg_uint32 offset)
{
	cyg_uint32 *paddr;
    cyg_uint32 dev = CYG_PCI_DEV_GET_DEV(devfn);
    cyg_uint32 fn  = CYG_PCI_DEV_GET_FN(devfn);
	
	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (cyg_uint32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
	}
	else
	{
		paddr = (cyg_uint32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
	}

    /* Offsets must be dword-aligned */
    offset &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{ /* Type 0 config */
		(cyg_uint32)paddr |= ((1 << (dev + 16)) | (fn << 8) | offset | 0);
	}
	else
	{ /* Type 1 config */
		(cyg_uint32)paddr |= ((bus << 16) | (dev << 11) | (fn << 8) | offset | 0);
	}

	orig_abort_vec = ((volatile cyg_uint32 *)0x20)[4];
    ((volatile unsigned *)0x20)[4] = (unsigned)__pci_abort_handler;
    HAL_ICACHE_SYNC();

    return paddr;
}

static int clear_master_aborts(void)
{
	cyg_uint32 temp;
	int err = 0;

	temp = *(PCI_CONTROL_REG);
	if(temp & 0x120)
	{
		err = 1;
		*(PCI_CONTROL_REG) = temp;
	}
	temp = *(PCI_CMD_STAT_REG);
	if(temp & 0x20000000)
	{
		err = 1;
#ifdef A0_REV
		while(temp & 0x20000000)
		{
			*(PCI_CMD_STAT_REG) = temp;
			temp = *(PCI_CMD_STAT_REG);
		}
#else
		*(PCI_CMD_STAT_REG) = temp;
#endif
	}

	((volatile unsigned *)0x20)[4] = orig_abort_vec;
    HAL_ICACHE_SYNC();

	return err;
}

cyg_uint32 cyg_hal_plf_pci_cfg_read_dword (cyg_uint32 bus,
					   cyg_uint32 devfn,
					   cyg_uint32 offset)
{
    cyg_uint32 *pdata, config_data;

    pdata = pci_config_setup(bus, devfn, offset);

    config_data = *pdata;

	if(clear_master_aborts())
		return 0xFFFFFFFF;
	else
		return config_data;
}


void cyg_hal_plf_pci_cfg_write_dword (cyg_uint32 bus,
				      cyg_uint32 devfn,
				      cyg_uint32 offset,
				      cyg_uint32 data)
{
    cyg_uint32 *pdata;

    pdata = pci_config_setup(bus, devfn, offset);

    *pdata = data;

	clear_master_aborts();
}


cyg_uint16 cyg_hal_plf_pci_cfg_read_word (cyg_uint32 bus,
					  cyg_uint32 devfn,
					  cyg_uint32 offset)
{
    cyg_uint32 *pdata;
    cyg_uint16 config_data;

	pdata = pci_config_setup(bus, devfn, offset);

    config_data = (cyg_uint16)(((*pdata) >> ((offset % 0x4) * 8)) & 0xffff);

	if(clear_master_aborts())
		return 0xFFFF;
	else
		return config_data;
}

void cyg_hal_plf_pci_cfg_write_word (cyg_uint32 bus,
				     cyg_uint32 devfn,
				     cyg_uint32 offset,
				     cyg_uint16 data)
{
    cyg_uint32 *pdata, mask, temp;

    pdata = pci_config_setup(bus, devfn, offset);

    mask = ~(0x0000ffff << ((offset % 0x4) * 8));

    temp = (cyg_uint32)(((cyg_uint32)data) << ((offset % 0x4) * 8));
    *pdata = (*pdata & mask) | temp; 

	clear_master_aborts();
}

cyg_uint8 cyg_hal_plf_pci_cfg_read_byte (cyg_uint32 bus,
					 cyg_uint32 devfn,
					 cyg_uint32 offset)
{
    cyg_uint32 *pdata;
    cyg_uint8 config_data;

    pdata = pci_config_setup(bus, devfn, offset);

    config_data = (cyg_uint8)(((*pdata) >> ((offset % 0x4) * 8)) & 0xff);

	if(clear_master_aborts())
		return 0xFF;
	else
		return config_data;
}


void cyg_hal_plf_pci_cfg_write_byte (cyg_uint32 bus,
				     cyg_uint32 devfn,
				     cyg_uint32 offset,
				     cyg_uint8 data)
{
    cyg_uint32 *pdata, mask, temp;

    pdata = pci_config_setup(bus, devfn, offset);

    mask = ~(0x000000ff << ((offset % 0x4) * 8));
    temp = (cyg_uint32)(((cyg_uint32)data) << ((offset % 0x4) * 8));
    *pdata = (*pdata & mask) | temp; 

	clear_master_aborts();
}
