/* cfg_utility.c
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXDP2400
subsystem: BootMonitor
author: lagarwal
revisions:
--------------------------------------------------------------------------
*/

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/cfg_utility.h>
#include <cyg/io/i2c.h>
#include <redboot.h>
#include <cyg/hal/hal_ixdp2400.h> 

extern int passwd_input;

static cmd_fun do_cfg;

RedBoot_cmd("cfg", 
            "Manage config PROM", 
            "{cmds}",
            do_cfg
    );

// read PROM function
local_cmd_entry("read",
                "Read config PROM",
                "-n <device id>",
                cmd_cfg_rd,
                CFG_cmds
    );

// write PROM function
local_cmd_entry("write",
                "Write config PROM",
                "-n <device id>",
                cmd_cfg_wr,
                CFG_cmds
    );

// update test status and date fields
local_cmd_entry("teststatus",
                "Update Test Status and Date",
                "-n <device id>",
                cmd_cfg_ts,
                CFG_cmds
    );

// Define table boundaries
CYG_HAL_TABLE_BEGIN( __CFG_cmds_TAB__, CFG_cmds);
CYG_HAL_TABLE_END( __CFG_cmds_TAB_END__, CFG_cmds);

extern struct cmd __CFG_cmds_TAB__[], __CFG_cmds_TAB_END__;

/*These Global variables are used for checksum computation by the routine and
 *need to be initialized to r=55665, c1=52845 and c2= 22719 */
unsigned short r;
unsigned short c1 = 52845;
unsigned short c2 = 22719;

static void
cfg_usage(char *why)
{
    printf("*** invalid 'cfg' command: %s\n", why);
    cmd_usage(__CFG_cmds_TAB__, &__CFG_cmds_TAB_END__, "cfg ");
}

static void
do_cfg(int argc, char *argv[])
{
    struct cmd *cmd;

    if (argc < 2) {
        cfg_usage("Too few arguments");
        return;
    }
    if ((cmd = cmd_search(__CFG_cmds_TAB__, &__CFG_cmds_TAB_END__, 
                          argv[1])) != (struct cmd *)0) {
        (cmd->fun)(argc, argv);
        return;
    }
    cfg_usage("Unrecognized command");
}

/**************************************************************************
Function:get_man_id()
This function gives the Manufacturer's ID to be stored on the config prom

Arguments: manid -Ptr to store the manufacturer's ID, 
		   user -1 take input from user 
				 0  default value
Returns: none
***************************************************************************/
void get_man_id(unsigned char *manid, unsigned char *curr_manid,int user)
{
	unsigned char temp[CFG_MAN_ID_LEN + 1];

	if(user)	
	{
		memset(temp, '\0', CFG_MAN_ID_LEN + 1);
		memcpy(temp, curr_manid, CFG_MAN_ID_LEN);
        printf("Current Manufcaturer's ID is %s\nEnter new Value: ",temp);
                
		gets((char *)temp, (CFG_MAN_ID_LEN + 1), 0);
		if(*temp == '\0')
            memcpy((char *)manid, (const char*)curr_manid, CFG_MAN_ID_LEN);
		else
			memcpy((char *)manid, (const char*)temp, CFG_MAN_ID_LEN);
	}		
	else
		memcpy((char*)manid, DEFAULT_MANID, CFG_MAN_ID_LEN);
    
	return ;
}


/**************************************************************************
Function:get_dev_id()
This function gives the Device ID to be stored on the config prom

Arguments: devid -Ptr to store the Device ID, 
		   user -1 take input from user 
				 0  default value
Returns: none
***************************************************************************/

void get_dev_id(unsigned char *devid, unsigned char *curr_devid,int user, int id)
{
	char c[2];
    unsigned long num;
    if(user)
	{
		printf("Current Device ID is %d\nEnter Device ID: ", *curr_devid);
                
		gets(c,2, 0);
        parse_num(c,&num,0,0);
		if(*c=='\0')
            *devid = *curr_devid;
        else
            *devid = (unsigned char)num;
	}
	else
    {
        *devid = (unsigned char)id;
    }

	return;
}

void get_op_freq(unsigned int *freq, unsigned int *curr_freq, int user)
{
	char c[5];
    unsigned long num;

    if(user)
	{
		printf("Current operating freq. is %d MHZ\nEnter new operating freq. (in MHZ): ", *curr_freq);
                
		gets(c, 5, 0);
        parse_num(c, &num, 0, 0);
		if(*c == '\0')
            *freq = *curr_freq;
        else
            *freq = num;
	}
	else
    {
        *freq = DEFAULT_OP_FREQ;
    }

	return;
}

/**************************************************************************
Function:get_model_id()
This function returns the Model ID 

Arguments: model_id		 - Ptr to store the model ID, 
		   curr_model_id - ptr to current model id. in case the 
		   user enters null, this will be retained. This is used 
		   only when user=1	
		   user			 - 1 take input from user 
						   0 default value
Returns: none
***************************************************************************/
void get_model_id(unsigned char *model_id, unsigned char *curr_model_id, 
																int user, int id)
{
	unsigned char temp[CFG_MODEL_ID_LEN + 1];
	if(user)
	{
		memset(temp, '\0', CFG_MODEL_ID_LEN + 1);
		memcpy(temp, curr_model_id, CFG_MODEL_ID_LEN);
		printf("Current Model ID is %s\nEnter new Model ID: ",temp);
		gets(temp, (CFG_MODEL_ID_LEN + 1), 0);
		if(*temp == '\0')
            memcpy((char*)model_id, (const char*)curr_model_id, CFG_MODEL_ID_LEN);
		else
			memcpy((char*)model_id, (const char*)temp, CFG_MODEL_ID_LEN);
	}
	else
    {
        switch(id)
        {
        case PL_DEV_ID:
		    memcpy((char*)model_id, "PL", CFG_MODEL_ID_LEN);
		    break;
        case BW_DEV_ID:
            memcpy((char*)model_id, "BW", CFG_MODEL_ID_LEN);
		    break;
        case BD_DEV_ID:
            memcpy((char*)model_id, "BD", CFG_MODEL_ID_LEN);
		    break;
        case MI_DEV_ID:
            memcpy((char*)model_id, "MI", CFG_MODEL_ID_LEN);
            break;
        case BI_DEV_ID:
            memcpy((char*)model_id, "BI", CFG_MODEL_ID_LEN);
            break;
        case KI_DEV_ID:
            memcpy((char*)model_id, "KI", CFG_MODEL_ID_LEN);
        default:
            return;
        }
        
    }
}

/**************************************************************************
Function:void get_part_rev(char* partrev, int user)
This function returns the Part rev in string format 

Arguments: partrev		- Ptr to store the part rev,
		   curr_partrev - Ptr to current partrev. This is used in case 
		   the user enters null. This is used only when user=1	 
		   user			- 1 take input from user 
						  0  default value
Returns: none
***************************************************************************/
void get_part_rev(unsigned char *partrev, unsigned char *curr_partrev,int user)
{
	unsigned char temp[CFG_PARTREV_LEN + 1];
	
	if(user)
	{
		printf("Part Revision is %c%c\n", *(curr_partrev), *(curr_partrev + 1));
	}
	printf("Enter Part Rev.(should start with 0): ");

    gets(temp, (CFG_PARTREV_LEN + 1), 0);

	if(*temp=='\0')
		memcpy((char *)partrev, (const char*)curr_partrev, CFG_PARTREV_LEN);
	else
		memcpy((char *)partrev, (const char*)temp, CFG_PARTREV_LEN);

	return;

}


/**************************************************************************
Function:char * get_part_num(char * partnum, int user)
This function returns the Part number in string format 

Arguments: partnum		- Ptr to store the part number, 
		   curr_partnum - Ptr to current partnum. This is used in case 
		   the user enters null. This is used only when user=1	
		   user			- 1 take input from user 
						  0  default value
Returns: none
***************************************************************************/
void get_part_num(unsigned char *partnum, unsigned char *curr_partnum,int user)
{
	unsigned char temp[CFG_PARTNUM_LEN + 1];

	if(user)
	{
		memset(temp, '\0', CFG_PARTNUM_LEN + 1);
		memcpy(temp, curr_partnum, CFG_PARTNUM_LEN);
        printf("Current Part Number is %s\n", temp);
	}
	printf("Enter Part Number(should start with A): ");

	gets(temp, (CFG_PARTNUM_LEN + 1),0);
    if(*temp == '\0')
		memcpy((char *)partnum, (const char*)curr_partnum, CFG_PARTNUM_LEN);
	else
		memcpy((char *)partnum, (const char*)temp, CFG_PARTNUM_LEN);
}


/**************************************************************************
Function:void get_serial_num(char * sernum)
This function returns the Serial number in string format 

Arguments: sernum		- Ptr to store the serial number,
		   curr_sernum  - Ptr to current sernum. This is used in case 
		   the user enters null.
 		   
Returns: none
***************************************************************************/
void get_serial_num(unsigned char *sernum,unsigned char *curr_sernum, int user)
{
	unsigned char temp[CFG_SERIAL_NUM_LEN + 1];
	if(user)
	{
		memset(temp, '\0', CFG_SERIAL_NUM_LEN + 1);
		memcpy(temp, curr_sernum, CFG_SERIAL_NUM_LEN);
        printf("Current Serial Number is %s\n", temp);
	}
    printf("Enter Serial Number: ");
    
	gets(temp, (CFG_SERIAL_NUM_LEN + 1),0);
	if(*temp=='\0')
		memcpy((char *)sernum, (const char*)curr_sernum, CFG_SERIAL_NUM_LEN);
	else
		memcpy((char *)sernum, (const char*)temp, CFG_SERIAL_NUM_LEN);
	return ;
}

/**************************************************************************
Function:void get_mac_addr(unsigned long *macaddr, user, npu)
This function returns the NPU's Ethernet MAC address

Arguments: macaddr	- Ptr to store the mac addr,
           curr_mac - Ptr to current MAC addr in case the user enters null
           user		- 1 take input from user 
					  0 default value
		   npu	    - 1 for master
					  0 for slave	
		   
Returns: none
***************************************************************************/
int get_mac_addr(unsigned char *mac, unsigned char *curr_mac, int user, char *msg)
{
	/*The MAC (Media Access Control) address is a 6-byte hex number 
	 *(e.g., 00-20-AF-ED-97-A5) coded and installed by the manufacturer of 
	 *the Ethernet hardware. The first 3 bytes are a unique Vendor ID for 
	 *the manufacturer. The last 3 bytes may be the card's incremental 
	 *serial number 
	 */
		
	unsigned int mac_vid; /*The vendor ID portion of the MAC addr*/
	unsigned int mac_sno; /*The serial no. portion of the mac addr*/
	int i;
	unsigned char str_mac[13];

	if(user)
	{
		printf("Current MAC address is %02x %02x %02x %02x %02x %02x\n", curr_mac[0],
            curr_mac[1], curr_mac[2], curr_mac[3], curr_mac[4], curr_mac[5]);
	}
	printf("%s (0xXXXXXXXXXXXX): 0x", msg);
	
	gets(str_mac, 13, 0);
	
	if(*str_mac == '\0')
	{
        if(!user)
            return ERROR;

		for(i = 0; i < 6; i++)
		{
			mac[i] = curr_mac[i];
		}
	}
	else
	{
		for(i = 0; i < 12; i++)
		{
			if(!_is_hex(str_mac[i]))
			{
				printf("Mac addrs should be a hex number\n");
				return ERROR;
			}
		}
		
		mac_vid = from_hex(str_mac, 6);
		mac[2] = mac_vid;
		mac[1] = mac_vid >> 8;
		mac[0] = mac_vid >> 16;
		if(mac_vid != MAC_VENDOR_ID)
		{
			printf("Vendor ID incorrect !!!\n");
			return ERROR;
		}
		
		mac_sno = from_hex(&str_mac[6], 6);
		if(mac_sno > 0xfffffe)
		{
			printf("MAC address serial number should be less than 0xffffff !!!\n");
			return ERROR;
		}
		mac[5] = mac_sno;
		mac[4] = mac_sno >> 8;
		mac[3] = mac_sno >> 16;
	}

	return OK;
}

int get_nxt_mac_addr(unsigned char *mac, unsigned char *nxt_mac)
{
    int i;

    for(i = 0; i < 5; i++)
        nxt_mac[i] = mac[i];

    nxt_mac[5] = mac[5] + 1;
    if(nxt_mac[5] < mac[5])
    {
        nxt_mac[4] += 1;
        if(nxt_mac[4] < mac[4])
        {
            nxt_mac[3] += 1;
            if(nxt_mac[3] < mac[3])
            {
                printf("Ran out of MAC Address\n");
                return ERROR;
            }
        }
    }

    return OK;
}

void display_config_data(char *in_cfg_data, int id)
{
	unsigned int temp;
	unsigned char temp_array[14];
    struct COMMON_CFG_CONTENT *common_data = (struct COMMON_CFG_CONTENT *)in_cfg_data;
	
    printf("\nConfiguration DATA\n\n");
	printf("Device ID                   : %d\n", common_data->dev_id);

	memset(temp_array, '\0', 13);
	memcpy(temp_array, common_data->man_id, CFG_MAN_ID_LEN);
	printf("Manufacturer's ID           : %s\n", temp_array);

	memset(temp_array, '\0', 13);
	memcpy(temp_array, common_data->model_id, CFG_MODEL_ID_LEN);
	printf("Model ID                    : %s\n", temp_array);

	memset(temp_array, '\0', 13);
	memcpy(temp_array, common_data->part_num, CFG_PARTNUM_LEN);
	printf("Part Number                 : %s\n", temp_array);

	printf("Part Revision               : %c%c\n", common_data->part_rev[0], common_data->part_rev[1]);

	memset(temp_array, '\0', 13);
	memcpy(temp_array, common_data->serial_num, CFG_SERIAL_NUM_LEN);
	printf("Serial Number               : %s\n", temp_array);

	if(id == KI_DEV_ID)
	{
        struct KI_EEPROM_CONTENT *ki_data = (struct KI_EEPROM_CONTENT *)in_cfg_data;

		printf("Master NPU's MAC Address    : 0x%02x%02x%02x%02x%02x%02x\n", 
                    ki_data->M_ether_mac_addr[0], ki_data->M_ether_mac_addr[1],
                    ki_data->M_ether_mac_addr[2], ki_data->M_ether_mac_addr[3],
                    ki_data->M_ether_mac_addr[4], ki_data->M_ether_mac_addr[5]);

		printf("Slave NPU's MAC Address     : 0x%02x%02x%02x%02x%02x%02x\n", 
                    ki_data->S_ether_mac_addr[0], ki_data->S_ether_mac_addr[1],
                    ki_data->S_ether_mac_addr[2], ki_data->S_ether_mac_addr[3],
                    ki_data->S_ether_mac_addr[4], ki_data->S_ether_mac_addr[5]);
    }
    if(id == PL_DEV_ID)
	{
        struct PL_EEPROM_CONTENT *pl_data = (struct PL_EEPROM_CONTENT *)in_cfg_data;

		printf("MAC Address for Port 0      : 0x%02x%02x%02x%02x%02x%02x\n", 
                    pl_data->mac_addr0[0], pl_data->mac_addr0[1],
                    pl_data->mac_addr0[2], pl_data->mac_addr0[3],
                    pl_data->mac_addr0[4], pl_data->mac_addr0[5]);
		printf("MAC Address for Port 1      : 0x%02x%02x%02x%02x%02x%02x\n", 
                    pl_data->mac_addr1[0], pl_data->mac_addr1[1],
                    pl_data->mac_addr1[2], pl_data->mac_addr1[3],
                    pl_data->mac_addr1[4], pl_data->mac_addr1[5]);
        printf("MAC Address for Port 2      : 0x%02x%02x%02x%02x%02x%02x\n", 
                    pl_data->mac_addr2[0], pl_data->mac_addr2[1],
                    pl_data->mac_addr2[2], pl_data->mac_addr2[3],
                    pl_data->mac_addr2[4], pl_data->mac_addr2[5]);
        printf("MAC Address for Port 3      : 0x%02x%02x%02x%02x%02x%02x\n", 
                    pl_data->mac_addr3[0], pl_data->mac_addr3[1],
                    pl_data->mac_addr3[2], pl_data->mac_addr3[3],
                    pl_data->mac_addr3[4], pl_data->mac_addr3[5]);
    }
    if(id == MI_DEV_ID)
	{
        struct MI_EEPROM_CONTENT *mi_data = (struct MI_EEPROM_CONTENT *)in_cfg_data;

		printf("Operating frequency         : %d MHZ\n", mi_data->ch_freq);
    }

	if(id == KI_DEV_ID)
	{
		switch(common_data->test_status)
		{
		case SKU1_PASS:
			printf("Test Status                 : SKU1_PASS\n");
			break;
		case SKU1_FAIL:
			printf("Test Status                 : SKU1_FAIL\n");
			break;
		case SKU2_PASS:
			printf("Test Status                 : SKU2_PASS\n");
			break;
		case SKU2_FAIL:
			printf("Test Status                 : SKU2_FAIL\n");
			break;
		default:
			printf("Test Status                 : Not Tested\n");
		}
	}
	else
	{
		switch(common_data->test_status)
		{
		case PASS:
			printf("Test Status                 : PASS\n");
			break;
		case FAIL:
			printf("Test Status                 : FAIL\n");
			break;
		default:
			printf("Test Status                 : Not Tested\n");
		}
	}

	memset(temp_array, '\0', 13);
	memcpy(temp_array, common_data->date, CFG_DATE_LEN);
	printf("Date                        : %s\n", temp_array);

	temp = 0;
	temp = common_data->chksum[0];
	temp = temp | (common_data->chksum[1] << 8);
	temp = temp | (common_data->chksum[2] << 16);
	temp = temp | (common_data->chksum[3] << 24);
	printf("CheckSum                    : %x\n", temp);
}

void read_config_data(char *buf, int id)
{
	int i, chksum_addr;
	unsigned int checksum = 0;
	unsigned int read_chksum = 0, cfg_data_sz;
    unsigned char slave_addr;
	
	switch(id)
	{
    case PL_DEV_ID:
		slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct PL_EEPROM_CONTENT);
		break;
    case BW_DEV_ID:
    case BD_DEV_ID:
        slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT);
		break;
	case MI_DEV_ID:
		slave_addr = SLAVE_ADDR_4;
		cfg_data_sz = sizeof(struct MI_EEPROM_CONTENT);
		break;
	case BI_DEV_ID:
		slave_addr = SLAVE_ADDR_3;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT);
		break;
	case KI_DEV_ID:
		slave_addr = SLAVE_ADDR_1;
		cfg_data_sz = sizeof(struct KI_EEPROM_CONTENT);
		break;
    default:
        return;
	}
	
    if(i2c_seq_read(slave_addr,0x0, buf, cfg_data_sz))
    {
        printf("Encountered problem in reading EPROM\n");
        return;
    }
    	
	/* r will be used by calculate_checksum() */
	r = 55665;
	chksum_addr = (PAGESZ*4) + 9;

	/*calculate the checksum of the read data excluding the checksum field*/
    for(i = 0; i < chksum_addr; i++)
	{
		checksum += calculate_chksum(buf[i]);
	}
	
    i += 4;
	for(; i < cfg_data_sz; i++)
	{
		checksum += calculate_chksum(buf[i]);
	}

	/*covert the stored checksum in to integer*/	
	read_chksum = 0;
	read_chksum = buf[chksum_addr];
	read_chksum = read_chksum | (buf[(chksum_addr + 1)] << 8);
	read_chksum = read_chksum | (buf[(chksum_addr + 2)] << 16);
	read_chksum = read_chksum | (buf[(chksum_addr + 3)] << 24);
	
	if(read_chksum != checksum)
    {
		printf("Warning - checksum error!!!\n");
        printf("read checksum = %x, computed checksum = %x\n",
                                        read_chksum,checksum);
    }
}

int get_data(char *data, int user, int id)
{
	unsigned char passwd[9];
	char data_buf[MAX_CFG_DATA_SZ];
    struct COMMON_CFG_CONTENT *common_data = (struct COMMON_CFG_CONTENT *)data;
    struct COMMON_CFG_CONTENT *curr_content = (struct COMMON_CFG_CONTENT *)data_buf;

	memset(data_buf, 0, MAX_CFG_DATA_SZ);
	if(user)
	{
		printf("Enter Password: ");
        passwd_input = 1;
		gets(passwd, (CFG_PASSWD_LEN + 1), 0);
        passwd_input = 0;
		if(strcmp((char*)passwd, "ixdp2400")) 
		{
			printf("Invalid Password!\n");
			return ERROR;
		}
        read_config_data(data_buf, id);
	}
	common_data->prom_fmt = '0';
	get_dev_id(&common_data->dev_id, &curr_content->dev_id, user, id);
	get_man_id(common_data->man_id, curr_content->man_id, user);
	get_part_rev(common_data->part_rev, curr_content->part_rev, user);
	get_part_num(common_data->part_num, curr_content->part_num, user);
	get_serial_num(common_data->serial_num, curr_content->serial_num, user);
	get_model_id(common_data->model_id, curr_content->model_id, user, id);

	if(!user) 
		memcpy((char *)common_data->passwd, "ixdp2400", CFG_PASSWD_LEN);

    if(id == KI_DEV_ID)
    {
        struct KI_EEPROM_CONTENT *ki_cfg_data = (struct KI_EEPROM_CONTENT *)data;
        struct KI_EEPROM_CONTENT *ki_curr_content = (struct KI_EEPROM_CONTENT *)data_buf;

	    if(!get_mac_addr(ki_cfg_data->M_ether_mac_addr, ki_curr_content->M_ether_mac_addr,  user,
            "Enter Master NPU's Ethernet MAC Address"))
            return ERROR;

        if(!get_nxt_mac_addr(ki_cfg_data->M_ether_mac_addr, ki_cfg_data->S_ether_mac_addr))
            return ERROR;
        
        printf("Slave NPU Ethernet MAC Addr : 0x%02x%02x%02x%02x%02x%02x\n", 
            ki_cfg_data->S_ether_mac_addr[0], ki_cfg_data->S_ether_mac_addr[1],
            ki_cfg_data->S_ether_mac_addr[2], ki_cfg_data->S_ether_mac_addr[3],
            ki_cfg_data->S_ether_mac_addr[4], ki_cfg_data->S_ether_mac_addr[5]);
    }
    if(id == PL_DEV_ID)
    {
        struct PL_EEPROM_CONTENT *pl_cfg_data = (struct PL_EEPROM_CONTENT *)data;
        struct PL_EEPROM_CONTENT *pl_curr_content = (struct PL_EEPROM_CONTENT *)data_buf;

	    if(!get_mac_addr(pl_cfg_data->mac_addr0, pl_curr_content->mac_addr0,  user,
            "Enter MAC Address for Port 0"))
            return ERROR;

        if(!get_nxt_mac_addr(pl_cfg_data->mac_addr0, pl_cfg_data->mac_addr1))
            return ERROR;

        if(!get_nxt_mac_addr(pl_cfg_data->mac_addr1, pl_cfg_data->mac_addr2))
            return ERROR;

        if(!get_nxt_mac_addr(pl_cfg_data->mac_addr2, pl_cfg_data->mac_addr3))
            return ERROR;

        printf("MAC Address for Port 1 : 0x%02x%02x%02x%02x%02x%02x\n", 
            pl_cfg_data->mac_addr1[0], pl_cfg_data->mac_addr1[1],
            pl_cfg_data->mac_addr1[2], pl_cfg_data->mac_addr1[3],
            pl_cfg_data->mac_addr1[4], pl_cfg_data->mac_addr1[5]);
        printf("MAC Address for Port 2 : 0x%02x%02x%02x%02x%02x%02x\n", 
            pl_cfg_data->mac_addr2[0], pl_cfg_data->mac_addr2[1],
            pl_cfg_data->mac_addr2[2], pl_cfg_data->mac_addr2[3],
            pl_cfg_data->mac_addr2[4], pl_cfg_data->mac_addr2[5]);
        printf("MAC Address for Port 3 : 0x%02x%02x%02x%02x%02x%02x\n", 
            pl_cfg_data->mac_addr3[0], pl_cfg_data->mac_addr3[1],
            pl_cfg_data->mac_addr3[2], pl_cfg_data->mac_addr3[3],
            pl_cfg_data->mac_addr3[4], pl_cfg_data->mac_addr3[5]);
    }
    if(id == MI_DEV_ID)
    {
        struct MI_EEPROM_CONTENT *mi_cfg_data = (struct MI_EEPROM_CONTENT *)data;
        struct MI_EEPROM_CONTENT *mi_curr_content = (struct MI_EEPROM_CONTENT *)data_buf;

	    get_op_freq(&(mi_cfg_data->ch_freq), &(mi_curr_content->ch_freq), user);
    }
	return OK;

}

static int write_config_data(char *databuf, int user, int id)
{
	unsigned int checksum=0;
	int i;
    unsigned char slave_addr, chksum_addr;
    int cfg_data_sz;
    struct COMMON_CFG_CONTENT *data = (struct COMMON_CFG_CONTENT *)databuf;
	
	switch(id)
    {
    case PL_DEV_ID:
		slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct PL_EEPROM_CONTENT);
		break;
    case BW_DEV_ID:
    case BD_DEV_ID:
		slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT);
		break;
    case MI_DEV_ID:
		slave_addr = SLAVE_ADDR_4;
		cfg_data_sz = sizeof(struct MI_EEPROM_CONTENT);
		break;
	case BI_DEV_ID:
		slave_addr = SLAVE_ADDR_3;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT);
		break;
	case KI_DEV_ID:
		slave_addr = SLAVE_ADDR_1;
		cfg_data_sz = sizeof(struct KI_EEPROM_CONTENT);
		break;
    default:
        return ERROR;
	}
	
    //Calculate checksum
	r = 55665;
    chksum_addr = (PAGESZ * 4)+9;
    for(i = 0; i < chksum_addr; i++)
	{
		checksum += calculate_chksum(databuf[i]);
	}
	
    i += 4;
	for(; i < cfg_data_sz; i++)
	{
		checksum += calculate_chksum(databuf[i]);
	}

    data->chksum[0] = checksum;
	data->chksum[1] = checksum >> 8;
	data->chksum[2] = checksum >> 16;
	data->chksum[3] = checksum >> 24;
	 
	for(i = 0; i < cfg_data_sz; i += 16)
	{
        if(i2c_page_write(slave_addr, i, &databuf[i], 16))
        {
            printf("Encountered problem in writing EPROM\n");
            return ERROR;
        }
	}
    return OK;
}

/******************************************************************************
Function	:	calculate_chksum() caluculates the checksum 
Arguments	:   value - data whose checksum has to be calculated
Return		:   returns the checksum
******************************************************************************/
unsigned char calculate_chksum(unsigned char value)
{
	unsigned char cipher = (value ^ (r >> 8));
	r = (cipher + r) * c1 + c2;
	return cipher;	
}

/******************************************************************************
Function	: from_hex() converts a string to integer of 4 bytes
Arguments	: buf- Ptr to the str
			  size-string length
Return		: the 4 byte integer
******************************************************************************/
unsigned long from_hex(unsigned char *buf, int size)
{
    unsigned char h[256];
    unsigned long result;
    unsigned int c;

	for (c = 0; c < 256; c++)
		h[c] = 0;
	for (c = '0'; c <= '9'; c++)
		h[c] = c - '0';
	for (c = 'a'; c <= 'f'; c++)
		h[c] = c - 'a' + 10;
	for (c = 'A'; c <= 'F'; c++)
		h[c] = c - 'A' + 10;
	
	result = 0;
    while (size > 0) {
        result = (result * 16) + h[*buf++];
        size--;
    }
    return result;
}


/******************************************************************************
Function	: cmd_cfg_rd() will be invoked when the user chooses to read the 
			  cfg data from the BootMonitor prompt.This will check if the data 
			  is valid and displays the data if valid.
Arguments	: not used. 
Return		: none
*******************************************************************************/			  
void cmd_cfg_rd(int argc, char * argv[])
{
	char cfg_data[MAX_CFG_DATA_SZ];
	unsigned char datr, slave_addr;
	int id;
    struct option_info opts[1];
	bool id_set = false;
    
    init_opts(&opts[0], 'n', true, OPTION_ARG_TYPE_NUM,(void **)&id,
                            (bool *)&id_set, "Device ID");    

    if(!scan_opts(argc, argv, 2, opts, 1, (void **)0, OPTION_ARG_TYPE_NUM, "Device ID"))
    {
        cfg_usage("Invalid arguments");
        return;
    }
	if(!id_set)
	{
		printf("Device ID required\n");
		return;
	}

    switch(id)
    {
    case PL_DEV_ID:
    case BW_DEV_ID:
    case BD_DEV_ID:
        slave_addr = SLAVE_ADDR_2;
		break;
    case MI_DEV_ID:
        slave_addr = SLAVE_ADDR_4;
		break;
	case BI_DEV_ID:
        slave_addr = SLAVE_ADDR_3;
		break;
	case KI_DEV_ID:
        slave_addr = SLAVE_ADDR_1;
		break;
	default:
		printf("Invalid Device ID\n");
		return;
    }

    if(i2c_read(slave_addr, 0x0, &datr))
    {
        printf("Encountered problem in reading EPROM\n");
        return;
    }
	if(datr == '0')
	{
		read_config_data(cfg_data, id);
		display_config_data(cfg_data,id);
	}
	else
	{
		printf("EEPROM is either corrupted or not programmed!\n");	
	}
}

/******************************************************************************
Function	: cmd_cfg_wr() will be invoked when the user chooses to write/edit 
			  cfg data from the BootMonitor prompt. This will check if the Data
			  is valid.if not valid restores the default values. if valid 
			  prompts the user to edit the values. if user enters incorrect 
			  values, the operation is aborted.
Arguments	: not used. 
Return		: none
*******************************************************************************/	
void cmd_cfg_wr(int arc, char *argv[])
{
	char cfg_data[MAX_CFG_DATA_SZ];
	unsigned char datr, slave_addr;
    int id;
    struct option_info opts[1];
	bool id_set = false;
    
    init_opts(&opts[0], 'n', true, OPTION_ARG_TYPE_NUM,(void **)&id,
                            (bool *)&id_set, "cfg write device ID");    

    if(!scan_opts(argc, argv, 2, opts, 1, (void **)0, OPTION_ARG_TYPE_NUM, "Device ID"))
    {
        cfg_usage("Invalid arguments");
        return;
    }
	if(!id_set)
	{
		printf("Device ID required\n");
		return;
	}

	memset(cfg_data, 0, MAX_CFG_DATA_SZ);
    switch(id)
    {
    case PL_DEV_ID:
    case BW_DEV_ID:
    case BD_DEV_ID:
        slave_addr = SLAVE_ADDR_2;
		break;
    case MI_DEV_ID:
        slave_addr = SLAVE_ADDR_4;
		break;
	case BI_DEV_ID:
        slave_addr = SLAVE_ADDR_3;
		break;
	case KI_DEV_ID:
        slave_addr = SLAVE_ADDR_1;
		break;
	default:
		printf("Invalid Device ID\n");
		return;
    }

    if(i2c_read(slave_addr, 0x0, &datr))
    {
        printf("Encountered problem in reading EPROM\n");
        return;
    }

	if(datr != '0')
	{
		if(!get_data(cfg_data, DEFAULT, id))
        {
			printf("Aborting operation...\n");
			return;
		}
		if(!write_config_data(cfg_data, DEFAULT,id))
            return;
        read_config_data(cfg_data,id);
        display_config_data(cfg_data,id);
	}
	else
	{
		if(!get_data(cfg_data,USER,id))
		{
            printf("Aborting operation...\n");
			return;
		}
        if(!write_config_data(cfg_data, USER,id))
            return;
        read_config_data(cfg_data,id);
        display_config_data(cfg_data,id);
	}
}

/******************************************************************************
Function	: get_test_status() will be update the test status field and date 
			  field of the cfg data.
Arguments	: ts - Ptr to store test status data 
			  curr_ts - current test status 
			  user    - 1 take input from user 
					    0  default value
Return		: OK if the entered test sttaus is valid, else ERROR
*******************************************************************************/	
int get_test_status(unsigned char *curr_ts, int user, int id)
{
	char ts[2];

	*curr_ts = NOT_TESTED;
	if(user)
	{
		if(id == KI_DEV_ID)
		{
			printf("Enter Test Status(1 - SKU1_PASS, 2 - SKU1_FAIL, 3 - SKU2_PASS, 4 - SKU2_FAIL): ");

			gets((char *)ts, 2, 0);
			if((*ts < '1') ||(*ts > '4'))
			{
				printf("Invalid Test Status Code!!!\n");
				return ERROR;
			}
		}
		else
		{
			printf("Enter Test Status(1 - PASS, 2 - FAIL): ");

			gets((char *)ts, 2, 0);
			if((*ts < '1') ||(*ts > '2'))
			{
				printf("Invalid Test Status Code!!!\n");
				return ERROR;
			}
		}
		*curr_ts = *ts;
	}

	return OK;
}

/******************************************************************************
Function	:get_date() prompts the user to enter the date field. 
Arguments	: date-ptr to store the date field
			  curr_date - current date field data
			  user    - 1 take input from user 
					    0  default value
Return		:none
******************************************************************************/					    
void get_date(unsigned char *curr_date,int user)
{
	char date[CFG_DATE_LEN + 1];

	if(user)
	{
		memset(date, '\0', CFG_DATE_LEN + 1);
		memcpy(date, curr_date, CFG_DATE_LEN);
		printf("Test Date is: %s\n", date);

		printf("Enter new Test Date: ");
		gets(date, (CFG_DATE_LEN + 1),0);
		if(*date == '\0')
			return;

		memcpy((char *)curr_date, (const char*)date, CFG_DATE_LEN);
	}
}


/******************************************************************************
Function	: cmd_cfg_ts() will be invoked when the user chooses to update 
			  test sttaus and date fields of the config data after running 
			  diagnotics
Arguments	: not used. 
Return		: none
*******************************************************************************/	
void cmd_cfg_ts(int argc, char*argv[])
{
    char curr_data[MAX_CFG_DATA_SZ];
	int id, cfg_data_sz;
    struct option_info opts[1];
	bool id_set = false;
    unsigned char slave_addr;
    struct COMMON_CFG_CONTENT *cfg_data = (struct COMMON_CFG_CONTENT *)curr_data;
    
    init_opts(&opts[0], 'n', true, OPTION_ARG_TYPE_NUM,(void **)&id,
                            (bool *)&id_set, "cfg read device ID");    

    if(!scan_opts(argc, argv, 2, opts, 1, (void **)0, OPTION_ARG_TYPE_NUM, "Device ID"))
    {
        cfg_usage("invalid arguments");
        return;
    }
	if(!id_set)
	{
		printf("Device ID required\n");
		return;
	}

    switch(id)
	{
    case PL_DEV_ID:
		slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct PL_EEPROM_CONTENT);
		break;
    case BW_DEV_ID:
    case BD_DEV_ID:
        slave_addr = SLAVE_ADDR_2;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT);
		break;
	case MI_DEV_ID:
		slave_addr = SLAVE_ADDR_4;
		cfg_data_sz = sizeof(struct MI_EEPROM_CONTENT);
		break;
	case BI_DEV_ID:
		slave_addr = SLAVE_ADDR_3;
		cfg_data_sz = sizeof(struct COMMON_CFG_CONTENT); 
		break;
	case KI_DEV_ID:
		slave_addr = SLAVE_ADDR_1;
		cfg_data_sz = sizeof(struct KI_EEPROM_CONTENT);
		break;
    default:
        printf("Invalid Device ID\n");
        return;
	}
	
    if(i2c_seq_read(slave_addr,0x0, curr_data, cfg_data_sz))
    {
        printf("Encountered problem in reading EPROM\n");
        return;
    }
	get_test_status(&(cfg_data->test_status), USER, id);
	get_date(cfg_data->date, USER);
	if(!write_config_data(curr_data, USER, id))
        return;
	display_config_data(curr_data,id);
}
