#ifndef ONCE_HAL_INTR_H
#define ONCE_HAL_INTR_H

//==========================================================================
//
//      hal_intr.h
//
//      HAL Interrupt and clock support
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    nickg, gthomas
// Contributors: nickg, gthomas,
//               jlarmour
// Date:         1999-02-20
// Purpose:      Define Interrupt support
// Description:  The macros defined here provide the HAL APIs for handling
//               interrupts and the clock.
//              
// Usage:        #include </hal/hal_intr.h>
//               ...
//              
//
//####DESCRIPTIONEND####
//
//==========================================================================

//#include <pkgconf/hal.h>

#include "type.h"

// This is to allow a variant to decide that there is no platform-specific
// interrupts file; and that in turn can be overridden by a platform that
// refines the variant's ideas.
#include "hal_platform_ints.h" // default less-complex platforms

// Spurious interrupt (no interrupt source could be found)
#define HAL_INTERRUPT_NONE -1

//--------------------------------------------------------------------------
// ARM exception vectors.

// These vectors correspond to VSRs. These values are the ones to use for
// HAL_VSR_GET/SET

#define HAL_VECTOR_RESET                0
#define HAL_VECTOR_UNDEF_INSTRUCTION    1
#define HAL_VECTOR_SOFTWARE_INTERRUPT   2
#define HAL_VECTOR_ABORT_PREFETCH       3
#define HAL_VECTOR_ABORT_DATA           4
#define HAL_VECTOR_reserved             5
#define HAL_VECTOR_IRQ                  6
#define HAL_VECTOR_FIQ                  7

#define HAL_VSR_MIN                     0
#define HAL_VSR_MAX                     7
#define HAL_VSR_COUNT                   8

// Exception vectors. These are the values used when passed out to an
// external exception handler using hal_deliver_exception()

#define HAL_EXCEPTION_ILLEGAL_INSTRUCTION \
          HAL_VECTOR_UNDEF_INSTRUCTION
#define HAL_EXCEPTION_INTERRUPT \
          HAL_VECTOR_SOFTWARE_INTERRUPT
#define HAL_EXCEPTION_CODE_ACCESS    HAL_VECTOR_ABORT_PREFETCH
#define HAL_EXCEPTION_DATA_ACCESS    HAL_VECTOR_ABORT_DATA

#define HAL_EXCEPTION_MIN     HAL_EXCEPTION_ILLEGAL_INSTRUCTION
#define HAL_EXCEPTION_MAX     HAL_EXCEPTION_DATA_ACCESS
#define HAL_EXCEPTION_COUNT   (HAL_EXCEPTION_MAX - \
                                      HAL_EXCEPTION_MIN + 1)

//--------------------------------------------------------------------------
// Static data used by HAL

// ISR tables
externC UINT32    hal_interrupt_handlers[HAL_ISR_COUNT];
externC UINT32   hal_interrupt_data[HAL_ISR_COUNT];
externC UINT32    hal_interrupt_objects[HAL_ISR_COUNT];

// Platform setup memory size (0 if unknown by hardware)
externC UINT32   hal_dram_size;

#ifdef INT_HAL_ARM_MEM_REAL_REGION_TOP

externC UINT8 *hal_arm_mem_real_region_top( UINT8 *_regionend_ );
                                                
# define HAL_MEM_REAL_REGION_TOP( _regionend_ ) \
    hal_arm_mem_real_region_top( _regionend_ )
#endif

//--------------------------------------------------------------------------
// Interrupt state storage

typedef UINT32 INTERRUPT_STATE;

//--------------------------------------------------------------------------
// Interrupt control macros

#ifndef __thumb__
// Note: This disables both FIQ and IRQ interrupts!
#define HAL_DISABLE_INTERRUPTS(_old_)           \
    asm volatile (                              \
        "mrs %0,cpsr;"                          \
        "mrs r4,cpsr;"                          \
        "orr r4,r4,#0xC0;"                      \
        "msr cpsr,r4"                           \
        : "=r"(_old_)                           \
        :                                       \
        : "r4"                                  \
        );

#define HAL_ENABLE_INTERRUPTS()                 \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "bic r3,r3,#0xC0;"                      \
        "msr cpsr,r3"                           \
        :                                       \
        :                                       \
        : "r3"                                  \
        );

#define HAL_RESTORE_INTERRUPTS(_old_)           \
    asm volatile (                              \
        "mrs r3,cpsr;"                          \
        "and r4,%0,#0xC0;"                      \
        "bic r3,r3,#0xC0;"                      \
        "orr r3,r3,r4;"                         \
        "msr cpsr,r3"                           \
        :                                       \
        : "r"(_old_)                            \
        : "r3", "r4"                            \
        );

#define HAL_QUERY_INTERRUPTS(_old_)             \
    asm volatile (                              \
        "mrs r4,cpsr;"                          \
        "and r4,r4,#0xC0;"                      \
        "eor %0,r4,#0xC0;"                      \
        : "=r"(_old_)                           \
        :                                       \
        : "r4"                                  \
        );
#else // __thumb__

// Thumb mode does not have access to the PSR registers;  

#if 0 // These don't seem to always work
#define HAL_DISABLE_INTERRUPTS(_old_)                   \
    asm volatile (                                      \
        "ldr r4,=10f;"                                  \
        "bx r4;"                                        \
        ".code 32;"                                     \
        "10:;"                                          \
        "mrs %0,cpsr;"                                  \
        "mrs r4,cpsr;"                                  \
        "orr r4,r4,#0xC0;"                              \
        "msr cpsr,r4;"                                  \
        "ldr r4,=10f+1;"                                \
        "bx  r4;"                                       \
        ".code 16;"                                     \
        "10:;"                                          \
        : "=r"(_old_)                                   \
        :                                               \
        : "r4"                                          \
        );

#define HAL_ENABLE_INTERRUPTS()                         \
    asm volatile (                                      \
        "ldr r3,=10f;"                                  \
        "bx r3;"                                        \
        ".code 32;"                                     \
        "10:;"                                          \
        "mrs r3,cpsr;"                                  \
        "bic r3,r3,#0xC0;"                              \
        "msr cpsr,r3;"                                  \
        "ldr r3,=10f+1;"                                \
        "bx  r3;"                                       \
        ".code 16;"                                     \
        "10:;"                                          \
        :                                               \
        :                                               \
        : "r3"                                          \
        );

#define HAL_RESTORE_INTERRUPTS(_old_)                   \
    asm volatile (                                      \
        "ldr r3,=10f;"                                  \
        "bx r3;"                                        \
        ".code 32;"                                     \
        "10:;"                                          \
        "mrs r3,cpsr;"                                  \
        "and r4,%0,#0xC0;"                              \
        "bic r3,r3,#0xC0;"                              \
        "orr r3,r3,r4;"                                 \
        "msr cpsr,r3;"                                  \
        "ldr r3,=10f+1;"                                \
        "bx  r3;"                                       \
        ".code 16;"                                     \
        "10:;"                                          \
        :                                               \
        : "r"(_old_)                                    \
        : "r3", "r4"                                    \
        );

#define HAL_QUERY_INTERRUPTS(_old_)                     \
    asm volatile (                                      \
        "ldr r4,=10f;"                                  \
        "bx r4;"                                        \
        ".code 32;"                                     \
        "10:;"                                          \
        "mrs r4,cpsr;"                                  \
        "and r4,r4,#0xC0;"                              \
        "eor %0,r4,#0xC0;"                              \
        "ldr r4,=10f+1;"                                \
        "bx  r4;"                                       \
        ".code 16;"                                     \
        "10:;"                                          \
        : "=r"(_old_)                                   \
        :                                               \
        : "r4"                                          \
        );
#else

externC UINT32 hal_disable_interrupts(void);
externC void       hal_enable_interrupts(void);
externC void       hal_restore_interrupts(UINT32);
externC UINT32 hal_query_interrupts(void);

#define HAL_DISABLE_INTERRUPTS(_old_)                   \
    _old_ = hal_disable_interrupts();

#define HAL_ENABLE_INTERRUPTS()                         \
    hal_enable_interrupts();

#define HAL_RESTORE_INTERRUPTS(_old_)                   \
    hal_restore_interrupts(_old_);

#define HAL_QUERY_INTERRUPTS(_old_)                     \
    _old_ = hal_query_interrupts();
#endif

#endif // __thumb__

//--------------------------------------------------------------------------
// Routine to execute DSRs using separate interrupt stack

#ifdef  IMP_HAL_COMMON_INTERRUPTS_USE_INTERRUPT_STACK

// these are offered solely for stack usage testing
// if they are not defined, then there is no interrupt stack.
#define HAL_INTERRUPT_STACK_BASE interrupt_stack_base
#define HAL_INTERRUPT_STACK_TOP  interrupt_stack
// use them to declare these extern however you want:
//       extern char HAL_INTERRUPT_STACK_BASE[];
//       extern char HAL_INTERRUPT_STACK_TOP[];
// is recommended
#endif

//--------------------------------------------------------------------------
// Vector translation.

#ifndef HAL_TRANSLATE_VECTOR
#define HAL_TRANSLATE_VECTOR(_vector_,_index_) \
    (_index_) = (_vector_)
#endif

//--------------------------------------------------------------------------
// Interrupt and VSR attachment macros

#define HAL_INTERRUPT_IN_USE( _vector_, _state_)                      \
    MACRO_START                                                       \
    UINT32 _index_;                                                   \
    HAL_TRANSLATE_VECTOR ((_vector_), _index_);                       \
                                                                      \
    if( hal_interrupt_handlers[_index_] == 0 )  					  \
        (_state_) = 0;                                                \
    else                                                              \
        (_state_) = 1;                                                \
    MACRO_END

#define HAL_INTERRUPT_ATTACH( _vector_, _isr_, _data_, _object_ )     \
    MACRO_START                                                       \
    if( hal_interrupt_handlers[_vector_] == 0 ) 					  \
    {                                                                 \
        hal_interrupt_handlers[_vector_] = (UINT32)_isr_;             \
        hal_interrupt_data[_vector_] = (UINT32) _data_;               \
        hal_interrupt_objects[_vector_] = (UINT32)_object_;           \
    }                                                                 \
    MACRO_END

#define HAL_INTERRUPT_DETACH( _vector_, _isr_ )                       \
    MACRO_START                                                       \
    if( hal_interrupt_handlers[_vector_] == (UINT32)_isr_ )           \
    {                                                                 \
        hal_interrupt_handlers[_vector_] = 0;   					  \
        hal_interrupt_data[_vector_] = 0;                             \
        hal_interrupt_objects[_vector_] = 0;                          \
    }                                                                 \
    MACRO_END

//--------------------------------------------------------------------------
// Interrupt controller access

externC void hal_interrupt_mask(int);
externC void hal_interrupt_unmask(int);
externC void hal_interrupt_acknowledge(int);
externC void hal_interrupt_configure(int, int, int);
externC void hal_interrupt_set_level(int, int);

#define HAL_INTERRUPT_MASK( _vector_ )                     \
    hal_interrupt_mask( _vector_ ) 
#define HAL_INTERRUPT_UNMASK( _vector_ )                   \
    hal_interrupt_unmask( _vector_ )
#define HAL_INTERRUPT_ACKNOWLEDGE( _vector_ )              \
    hal_interrupt_acknowledge( _vector_ )
#define HAL_INTERRUPT_CONFIGURE( _vector_, _level_, _up_ ) \
    hal_interrupt_configure( _vector_, _level_, _up_ )
#define HAL_INTERRUPT_SET_LEVEL( _vector_, _level_ )       \
    hal_interrupt_set_level( _vector_, _level_ )

//--------------------------------------------------------------------------
// Clock control

externC void hal_clock_initialize(UINT32);
externC void hal_clock_read(UINT32 *);
externC void hal_clock_reset(UINT32, UINT32);

#define HAL_CLOCK_INITIALIZE( _period_ )   hal_clock_initialize( _period_ )
#define HAL_CLOCK_RESET( _vec_, _period_ ) hal_clock_reset( _vec_, _period_ )
#define HAL_CLOCK_READ( _pvalue_ )         hal_clock_read( _pvalue_ )
#ifdef VAR_KERNEL_COUNTERS_CLOCK_LATENCY
#define HAL_CLOCK_LATENCY( _pvalue_ )      HAL_CLOCK_READ( (UINT32 *)_pvalue_ )
#endif

//--------------------------------------------------------------------------
#endif // ifndef ONCE_HAL_INTR_H
// End of hal_intr.h
