//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#ifndef ONCE_HAL_IO_H
#define ONCE_HAL_IO_H

#include "type.h"

//-----------------------------------------------------------------------------
// IO Register address.
// This type is for recording the address of an IO register.

typedef volatile UINT32 HAL_IO_REGISTER;


//-----------------------------------------------------------------------------
// BYTE Register access.
// Individual and vectorized access to 8 bit registers.

// Little-endian version
#if (BYTEORDER == LSBFIRST)

#define HAL_READ_UINT8( _register_, _value_ ) \
        ((_value_) = *((volatile UINT8 *)(_register_)))

#define HAL_WRITE_UINT8( _register_, _value_ ) \
        (*((volatile UINT8 *)(_register_)) = (_value_))

#define HAL_READ_UINT8_VECTOR( _register_, _buf_, _count_, _step_ )     \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        (_buf_)[_i_] = ((volatile UINT8 *)(_register_))[_j_];        	\
    MACRO_END

#define HAL_WRITE_UINT8_VECTOR( _register_, _buf_, _count_, _step_ )    \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        ((volatile UINT8 *)(_register_))[_j_] = (_buf_)[_i_];        	\
    MACRO_END

#else // Big-endian version

#define HAL_READ_UINT8( _register_, _value_ ) \
        ((_value_) = *((volatile UINT8 *)((UINT32)(_register_)^3)))

#define HAL_WRITE_UINT8( _register_, _value_ ) \
        (*((volatile UINT8 *)((UINT32)(_register_)^3)) = (_value_))

#define HAL_READ_UINT8_VECTOR( _register_, _buf_, _count_, _step_ )     \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    volatile UINT8* _r_ = ((UINT32)(_register_)^3);            			\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        (_buf_)[_i_] = _r_[_j_];                                        \
    MACRO_END

#define HAL_WRITE_UINT8_VECTOR( _register_, _buf_, _count_, _step_ )    \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    volatile UINT8* _r_ = ((UINT32)(_register_)^3);            			\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        _r_[_j_] = (_buf_)[_i_];                                        \
    MACRO_END

#endif // Big-endian

//-----------------------------------------------------------------------------
// 16 bit access.
// Individual and vectorized access to 16 bit registers.
    
// Little-endian version
#if (BYTEORDER == LSBFIRST)

#define HAL_READ_UINT16( _register_, _value_ ) \
        ((_value_) = *((volatile UINT16 *)(_register_)))

#define HAL_WRITE_UINT16( _register_, _value_ ) \
        (*((volatile UINT16 *)(_register_)) = (_value_))

#define HAL_READ_UINT16_VECTOR( _register_, _buf_, _count_, _step_ )    \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        (_buf_)[_i_] = ((volatile UINT16 *)(_register_))[_j_];      	\
    MACRO_END

#define HAL_WRITE_UINT16_VECTOR( _register_, _buf_, _count_, _step_ )   \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        ((volatile UINT16 *)(_register_))[_j_] = (_buf_)[_i_];      	\
    MACRO_END

#else // Big-endian version

#define HAL_READ_UINT16( _register_, _value_ ) \
        ((_value_) = *((volatile UINT16 *)((UINT32)(_register_)^3)))

#define HAL_WRITE_UINT16( _register_, _value_ ) \
        (*((volatile UINT16 *)((UINT32)(_register_)^3)) = (_value_))

#define HAL_READ_UINT16_VECTOR( _register_, _buf_, _count_, _step_ )    \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    volatile UINT16* _r_ = ((UINT32)(_register_)^3);          			\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        (_buf_)[_i_] = _r_[_j_];                                        \
    MACRO_END

#define HAL_WRITE_UINT16_VECTOR( _register_, _buf_, _count_, _step_ )   \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    volatile UINT16* _r_ = ((UINT32)(_register_)^3);          			\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        _r_[_j_] = (_buf_)[_i_];                                        \
    MACRO_END

#endif // Big-endian

//-----------------------------------------------------------------------------
// 32 bit access.
// Individual and vectorized access to 32 bit registers.

// Note: same macros for little- and big-endian systems.
    
#define HAL_READ_UINT32( _register_, _value_ ) \
        ((_value_) = *((volatile UINT32 *)(_register_)))

#define HAL_WRITE_UINT32( _register_, _value_ ) \
        (*((volatile UINT32 *)(_register_)) = (_value_))

#define HAL_READ_UINT32_VECTOR( _register_, _buf_, _count_, _step_ )    \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        (_buf_)[_i_] = ((volatile UINT32 *)(_register_))[_j_];     	 	\
    MACRO_END

#define HAL_WRITE_UINT32_VECTOR( _register_, _buf_, _count_, _step_ )   \
    MACRO_START                                                     	\
    S32 _i_,_j_;                                                		\
    for( _i_ = 0, _j_ = 0; _i_ < (_count_); _i_++, _j_ += (_step_))     \
        ((volatile UINT32 *)(_register_))[_j_] = (_buf_)[_i_];      	\
    MACRO_END

// Enforce a flow "barrier" to prevent optimizing compiler from reordering 
// operations.
#define HAL_IO_BARRIER()

//-----------------------------------------------------------------------------
// Include plf_io.h for platforms that define it.
#include "hal_platform_ints.h"  // Interrupt vectors

#define XSCALE_PCI_TYPE_0_CONFIG_BASE	0xDA000000
#define XSCALE_PCI_TYPE_1_CONFIG_BASE	0xDB000000
#define XSCALE_PCI_MEM_BASE				0xE0000000
#define XSCALE_PCI_MEM_SZ_MASK			0x1FFFFFFF
#define XSCALE_PCI_IO_BASE				0xD8000000
#define XSCALE_PCI_IO_SZ_MASK			0x1FFFFFF
#define PRIMARY_BUS_NUM					0
#define SECONDARY_BUS_NUM				1
#define PRIMARY_MEM_BASE				XSCALE_PCI_MEM_BASE
#define PRIMARY_IO_BASE					XSCALE_PCI_IO_BASE


// Read a value from the PCI configuration space of the appropriate
// size at an address composed from the bus, devfn and offset.
//-----------------------------------------------------------------------------
// Resources

// Map PCI device resources starting from these addresses in PCI space.
#define HAL_PCI_ALLOC_BASE_MEMORY		0x0
#define HAL_PCI_ALLOC_BASE_IO			0x0

#define HAL_PCI_MEM_SPACE_SIZE			0x20000000
#define HAL_PCI_IO_SPACE_SIZE			0x2000000

// This is where the PCI spaces are mapped in the CPU's address space.
#define HAL_PCI_PHYSICAL_MEMORY_BASE	XSCALE_PCI_MEM_BASE
#define HAL_PCI_PHYSICAL_IO_BASE		XSCALE_PCI_IO_BASE

// Translate the PCI interrupt requested by the device (INTA#, INTB#,
// INTC# or INTD#) to the associated CPU interrupt (i.e., HAL vector).
// everything is connected to intb
#define HAL_PCI_TRANSLATE_INTERRUPT(__bus, __devfn, __vec, __valid)	\
    MACRO_START                                                 	\
	__vec = HAL_INTERRUPT_PCI_INTB;		                    		\
    __valid = true;										            \
    MACRO_END

// This is the lowest level where devfns are used.
#define PCI_DEV_MAKE_DEVFN(__dev, __fn) (((__dev)<<3)|(__fn))
#define PCI_DEV_GET_DEV(__devfn) ((__devfn>>3)&0x1f)
#define PCI_DEV_GET_FN(__devfn) (__devfn&0x7)

// Some buggy PCI chips force us to ignore certain devices so that
// they may be handled specially.
#ifdef HAL_PCI_IGNORE_DEVICE
#define PCI_IGNORE_DEVICE(__bus, __dev, __fn) \
            HAL_PCI_IGNORE_DEVICE((__bus), (__dev), (__fn))
#else
#define PCI_IGNORE_DEVICE(__bus, __dev, __fn) 0
#endif

//-----------------------------------------------------------------------------
#endif // ifndef ONCE_HAL_IO_H
// End of hal_io.h
