/* hal_ixdp2400.h
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXP2400
subsystem: DIAG
author: lagarwal, Feb 13, 02
revisions:
--------------------------------------------------------------------------
*/

#ifndef ONCE_HAL_IXDP2400_H
#define ONCE_HAL_IXDP2400_H

#define SWAP32(x)               \
  (((x) << 24) |                \
  (((x) & 0x0000FF00) << 8) |   \
  (((x) & 0x00FF0000) >> 8) |   \
  (((unsigned int)(x)) >> 24))

#define BOOT_THRU_BM				(1 << 31)
#define BOOT_DONT_INIT_PCI			(1 << 30)

#define	RAM_BASE					0x0
#define RAM_SIZE					(256 * 1024 * 1024)
#define SLAVE_POST_FLAG_LOC			0x1FFFFC
#define POST_INCOMPLETE				0x0
#define POST_COMPLETE				0x1

#define SRAM_BASE					0x80000000
#define MIN_SRAM_SIZE				(4 * 1024 * 1024) /* 4MB SRAM */
#define MAX_SRAM_SIZE				(32 * 1024 * 1024) /* 32MB SRAM */
#define MAX_SRAM_SIZE_CSR_VAL		(5 << 8)
#define SRAM_COMPARE_LOCATION		0x101000
#define SRAM_TEST_LOCATION			0x1000
#define SRAM_SIZE_LOOPS				4
#define TEST_PAT					0x12345678
#define COMPARISON_PAT				0xAA55AA55
#define MAX_QDR_CHANNEL				2

#define	PCI_MEM_PAGE_MASK			0xE0000000
#define	PCI_IO_PAGE_MASK			0xFFFF0000

#define SLOW_PORT_BASE				0xC4000000

#define SCRATCHPAD_BASE				0xCA000000

#define TIMER_CLK_FREQ				50000000	/* timer runs at 50 MHz */
#define _TICKS_PER_USEC				(TIMER_CLK_FREQ / 1000000)

#define CSR_BAR						0
#define SRAM_BAR					1
#define SDRAM_BAR					2

#define CPLD_CSR_BASE				(SLOW_PORT_BASE + 0x3000000)

#define SYSTEM_LED_REG_OFF			0x0
#define ALPHANUM_DIS_DATA_OFF		0x4
#define SYS_CLOCK_SPEED_OFF			0x8
#define CPLD_INT_OFF				0xC
#define CPLD_REV_OFF				0x10
#define SYS_CLK_M_OFF				0x14
#define SYS_CLK_N_OFF				0x18
#define CPLD_INT_MASK_OFF			0x48

#define SYSTEM_LED					(CPLD_CSR_BASE + SYSTEM_LED_REG_OFF)
#define ALPHANUM_DIS_DATA			(CPLD_CSR_BASE + ALPHANUM_DIS_DATA_OFF)
#define SYS_CLOCK_SPEED				(CPLD_CSR_BASE + SYS_CLOCK_SPEED_OFF)
#define CPLD_INT					(CPLD_CSR_BASE + CPLD_INT_OFF)
#define CPLD_REV					(CPLD_CSR_BASE + CPLD_REV_OFF)
#define SYS_CLK_M					(CPLD_CSR_BASE + SYS_CLK_M_OFF)
#define SYS_CLK_N					(CPLD_CSR_BASE + SYS_CLK_N_OFF)
#define CPLD_INT_MASK				(CPLD_CSR_BASE + CPLD_INT_MASK_OFF)

#define SYSTEM_LED_REG				((volatile UINT32 *)SYSTEM_LED)
#define ALPHANUM_DIS_DATA_REG		((volatile UINT8 *)ALPHANUM_DIS_DATA)
#define SYS_CLOCK_SPEED_REG			((volatile UINT32 *)SYS_CLOCK_SPEED)
#define CPLD_INT_REG				((volatile UINT32 *)CPLD_INT)
#define CPLD_REV_REG				((volatile UINT32 *)CPLD_REV)
#define SYS_CLK_M_REG				((volatile UINT32 *)SYS_CLK_M)
#define SYS_CLK_N_REG				((volatile UINT32 *)SYS_CLK_N)
#define CPLD_INT_MASK_REG			((volatile UINT32 *)CPLD_INT_MASK)

#define WRITE_TO_CHAR_RAM			0x48
#define DISPLAY_ASCII				0x18

#define USE_8_BIT_DATA				0x1
#define USE_32_BIT_DATA				0x0

#define WR_HD						0x1
#define WR_PW						0x2
#define WR_SU						0x0
#define SP_WTC1_VAL					(WR_SU << 6 | WR_PW << 2 | WR_HD)

#define NO_BOOT_ROM					0x0
#define CFG_PCI_BOOT_HOST			(1 << 2)
#define CFG_PROM_BOOT				(1 << 1)

#define IXP2400_VENDOR_ID			0x8086
#define IXP2400_DEVICE_ID			0x9001

#define WDOG_RST_ENABLE				(1 << 24)
#define INIT_COMP					(1 << 21)
#define RSTALL						(1 << 16)
#define RESET_DRAM0					(1 << 11)
#define RESET_SRAM1					(1 << 4)
#define RESET_SRAM0					(1 << 3)
#define PCIRST						(1 << 2)
#define RESET_PCI					(1 << 1)
#define RESET_XSCALE				(1 << 0)
#define NOT_RESET_PCI				~(PCIRST | RESET_PCI)

#define PCI_CONTROL_CFG_RST_DIR		(1 << 28) /* PCI Centrl Function bit */
#define PCI_CONTROL_BE_DEO			(1 << 22) /* PCI Big Endian Data Enable Out */
#define PCI_CONTROL_BE_DEI			(1 << 21) /* PCI Big Endian Data Enable In */
#define PCI_CONTROL_BE_BEO			(1 << 20) /* PCI Big Endian Byte Enable Out */
#define PCI_CONTROL_BE_BEI			(1 << 19) /* PCI Big Endian Byte Enable In */
#define PCI_CONTROL_PNR				(1 << 17) /* PCI Not Reset bit of SA_CONTROL */

#define SRAM_200MHZ_DIVISOR			0x3
#define SRAM_150MHZ_DIVISOR			0x4
#define SDRAM_DIVISOR				0x4

#define PIPELINE					(1 << 10)
#define PARITY_ENABLE				(1 << 3)

#define FLASH_WRITE_ENABLE			(1 << 9)
#define FLASH_ALIAS_DISABLE			(1 << 8)

#define DDR_RX_DLL_VAL				0x11
#define DDR_RX_DESKEW_VAL			0x11
#define DDR_RDDLYSEL_RECEN_VAL		0x11
#define DDR_RCOMP_IO_CONFIG_VAL		0x0

#define TCL							(1 << 2)
#define TRC							(1 << 6)
#define TWTR						(1 << 10)
#define REF_EN						(1 << 14)
#define REF_COUNT					(10 << 15)
#define NUM_ROW_COL					(3 << 23)
#define NUM_ROW_COL_32_8			(3 << 23)
#define NUM_ROW_COL_64_8			(4 << 23)
#define NUM_SIDES					(1 << 26)
#define TRRD						(1 << 28)
#define DU_CONTROL_VAL				(TRRD | NUM_SIDES | NUM_ROW_COL_32_8 | REF_COUNT | REF_EN | TWTR | TRC | TCL)
#define DU_CONTROL_VAL_1G			(TRRD | NUM_SIDES | NUM_ROW_COL_64_8 | REF_COUNT | REF_EN | TWTR | TRC | TCL)

#define RESET_DLL					0x100
#define LOAD_MODE_NORMAL			0x62
#define PRECHARGE_ALL				(1 << 10)
#define EXT_LD_MODE					(1 << 12)
#define SIDE0						(1 << 14)
#define SIDE1						(1 << 15)
#define CKE							(1 << 16)
#define PRECHARGE					(1 << 29)
#define REFRESH						(1 << 30)
#define LD_MODE_REG					(1 << 31)

#define DISABLE_CHK					(1 << 8)

#define SOFT_INT					(1 << 0)
#define ERROR_SUM					(1 << 1)
#define UART_INT					(1 << 2)
#define GPIO_INT					(1 << 3)
#define TIMER_UFLOW					(0xF << 4)
#define SP_INT						(1 << 8)
#define PCI_INT						(1 << 15)
#define PCI_DOORBELL				(1 << 13)

#define IRQ_ENABLE_SET_VAL			(PCI_DOORBELL | SP_INT | TIMER_UFLOW | GPIO_INT | UART_INT | ERROR_SUM | SOFT_INT)

#define PCIIM						(1 << 25)
#define PILM						(3 << 26)

#define XSCALE_INT_ENABLE_VAL		(PILM | PCIIM)

#define ACTIVATE_TIMER				(1 << 7)

#define T1_CTL_VAL					ACTIVATE_TIMER

#define CLEAR_INT					1

#define T1_CLR_VAL					CLEAR_INT

#define IO_SPACE_ENABLE				(1 << 0)
#define MEM_SPACE_ENABLE			(1 << 1)
#define BUS_MASTER_ENABLE			(1 << 2)
#define WR_INV_ENABLE				(1 << 4)
#define PERR_ENABLE					(1 << 6)
#define SERR_ENABLE					(1 << 8)
#define FAST_BACK_TO_BACK			(1 << 9)

#define PCR_VAL						(WR_INV_ENABLE | MEM_SPACE_ENABLE | IO_SPACE_ENABLE)
#define PCI_CMD_STAT_VAL			(WR_INV_ENABLE | BUS_MASTER_ENABLE | MEM_SPACE_ENABLE | IO_SPACE_ENABLE)
#define SLAVE_PCI_CMD_STAT_VAL		(WR_INV_ENABLE | MEM_SPACE_ENABLE | IO_SPACE_ENABLE)

#define XSIM						~(1 << 1)

#define PCI_ADDR_EXT_VAL			0xD800E000

#define PMU_BASE					0xC0050000

#define PMUCONTCFG					(PMU_BASE + 0xF00)

#define CHAPCMD0					(PMU_BASE + 0x0)
#define CHAPEVN0					(PMU_BASE + 0x4)
#define CHAPSTAT0					(PMU_BASE + 0x8)
#define CHAPDATA0					(PMU_BASE + 0xC)

#define CHAPCMD1					(PMU_BASE + 0x10)
#define CHAPEVN1					(PMU_BASE + 0x14)
#define CHAPSTAT1					(PMU_BASE + 0x18)
#define CHAPDATA1					(PMU_BASE + 0x1C)

#define CHAPCMD2					(PMU_BASE + 0x20)
#define CHAPEVN2					(PMU_BASE + 0x24)
#define CHAPSTAT2					(PMU_BASE + 0x28)
#define CHAPDATA2					(PMU_BASE + 0x2C)

#define CHAPCMD3					(PMU_BASE + 0x30)
#define CHAPEVN3					(PMU_BASE + 0x34)
#define CHAPSTAT3					(PMU_BASE + 0x38)
#define CHAPDATA3					(PMU_BASE + 0x3C)

#define CHAPCMD4					(PMU_BASE + 0x40)
#define CHAPEVN4					(PMU_BASE + 0x44)
#define CHAPSTAT4					(PMU_BASE + 0x48)
#define CHAPDATA4					(PMU_BASE + 0x4C)

/* Global control registers */
#define GLOBAL_CONTROL_BASE			0xC0004A00

#define PRODUCT_ID_OFF              0x0
#define MISC_CONTROL_OFF			0x4
#define IXP_RESET0_OFF				0xC
#define CCR_OFF						0x14
#define STRAP_OPTIONS_OFF			0x18

#define PRODUCT_ID				    (GLOBAL_CONTROL_BASE + PRODUCT_ID_OFF)
#define MISC_CONTROL				(GLOBAL_CONTROL_BASE + MISC_CONTROL_OFF)
#define IXP_RESET0					(GLOBAL_CONTROL_BASE + IXP_RESET0_OFF)
#define CCR							(GLOBAL_CONTROL_BASE + CCR_OFF)
#define STRAP_OPTIONS				(GLOBAL_CONTROL_BASE + STRAP_OPTIONS_OFF)

#define PRODUCT_ID_REG			    ((volatile UINT32 *)PRODUCT_ID)
#define MISC_CONTROL_REG			((volatile UINT32 *)MISC_CONTROL)
#define IXP_RESET0_REG				((volatile UINT32 *)IXP_RESET0)
#define CCR_REG						((volatile UINT32 *)CCR)
#define STRAP_OPTIONS_REG			((volatile UINT32 *)STRAP_OPTIONS)

/* Timer registers */
#define TIMER_CSR_BASE				0xC0020000

#define T1_CTL_OFF					0x0
#define T1_CLD_OFF					0x10
#define T1_CSR_OFF					0x20
#define T1_CLR_OFF					0x30

#define T1_CTL_REG					((volatile UINT32 *)(TIMER_CSR_BASE + T1_CTL_OFF))
#define T1_CLD_REG					((volatile UINT32 *)(TIMER_CSR_BASE + T1_CLD_OFF))
#define T1_CSR_REG					((volatile UINT32 *)(TIMER_CSR_BASE + T1_CSR_OFF))
#define T1_CLR_REG					((volatile UINT32 *)(TIMER_CSR_BASE + T1_CLR_OFF))

/* slowport registers */
#define SLOWPORT_REGISTER_BASE		0xC0080000

#define SP_CCR_OFF					0x0
#define SP_WTC1_OFF					0x4
#define SP_WTC2_OFF					0x8
#define SP_RTC1_OFF					0xC
#define SP_RTC2_OFF					0x10
#define SP_FAC_OFF					0x20
#define SP_FRM_OFF					0x24

#define SP_CCR						(SLOWPORT_REGISTER_BASE + SP_CCR_OFF)
#define SP_WTC1						(SLOWPORT_REGISTER_BASE + SP_WTC1_OFF)
#define SP_WTC2						(SLOWPORT_REGISTER_BASE + SP_WTC2_OFF)
#define SP_RTC1						(SLOWPORT_REGISTER_BASE + SP_RTC1_OFF)
#define SP_RTC2						(SLOWPORT_REGISTER_BASE + SP_RTC2_OFF)
#define SP_FAC						(SLOWPORT_REGISTER_BASE + SP_FAC_OFF)
#define SP_FRM						(SLOWPORT_REGISTER_BASE + SP_FRM_OFF)

#define SP_CCR_REG					((volatile UINT32 *)SP_CCR)
#define SP_WTC1_REG					((volatile UINT32 *)SP_WTC1)
#define SP_WTC2_REG					((volatile UINT32 *)SP_WTC2)
#define SP_RTC1_REG					((volatile UINT32 *)SP_RTC1)
#define SP_RTC2_REG					((volatile UINT32 *)SP_RTC2)
#define SP_FAC_REG					((volatile UINT32 *)SP_FAC)
#define SP_FRM_REG					((volatile UINT32 *)SP_FRM)

/* sram registers */
#define QDR_CH_CSR_BASE				0xCC010000

#define QDR_CH_CONTROL_OFF			0x0
#define QDR_RX_DLL_OFF				0x228
#define QDR_RD_PTR_OFF				0x240
#define QDR_RX_DESKEW_OFF			0x244
#define QDR_RCMP_SETUP_CONTROL_OFF	0x300 
#define QDR_RCMP_PMOS_OVER_OFF		0x30C 
#define QDR_RCMP_NMOS_OVER_OFF		0x310
#define QDR_RCMP_PMOS_NMOS_OVER_OFF	0x314

/* dram registers */
#define DRAM_CSR_BASE				0xD0009000

#define DU_CONTROL_OFF				0x0
#define DU_ERROR_STATUS_1_OFF		0x8
#define DU_ERROR_STATUS_2_OFF		0x10
#define DU_ECC_TEST_OFF				0x18
#define DU_INIT_OFF					0x20
#define DU_CONTROL2_OFF				0x28
#define DDR_RCOMP_IO_CONFIG_OFF		0x3C0
#define DDR_RDDLYSEL_RECEN_OFF		0x3C8
#define DDR_RX_DLL_OFF				0x650
#define DDR_RX_DESKEW_OFF			0x688

#define CR0_FRCSMRCOMP_OFF			0x100
#define CR0_DSTRENGTHSEL_OFF		0x130
#define CR0_DDQRCOMP_OFF			0x148
#define CR0_DCTLRCOMP_OFF			0x190
#define CR0_DRCVRCOMP_OFF			0x1D8
#define CR0_DCKERCOMP_OFF			0x228
#define CR0_DCSRCOMP_OFF			0x2B0
#define CR0_DCKRCOMP_OFF			0x338
#define CR0_DX8X16CKECSCKSEL_OFF	0x220
#define CR0_RCOMPPRD_OFF			0x108
#define CR0_DIGFIL_OFF				0x118

#define CR0_SLEWPROGRAMMED_OFF		0x128
#define CR0_OVRRIDEH_OFF			0x138
#define CR0_OVRRIDEV_OFF			0x140
#define CR0_JT_CONFIG_OFF			0x3C0

#define DU_CONTROL					(DRAM_CSR_BASE + DU_CONTROL_OFF)
#define DU_ERROR_STATUS_1			(DRAM_CSR_BASE + DU_ERROR_STATUS_1_OFF)
#define DU_ERROR_STATUS_2			(DRAM_CSR_BASE + DU_ERROR_STATUS_2_OFF)
#define DU_ECC_TEST					(DRAM_CSR_BASE + DU_ECC_TEST_OFF)
#define DU_INIT						(DRAM_CSR_BASE + DU_INIT_OFF)
#define DU_CONTROL2					(DRAM_CSR_BASE + DU_CONTROL2_OFF)
#define DDR_RCOMP_IO_CONFIG			(DRAM_CSR_BASE + DDR_RCOMP_IO_CONFIG_OFF)
#define DDR_RDDLYSEL_RECEN			(DRAM_CSR_BASE + DDR_RDDLYSEL_RECEN_OFF)
#define DDR_RX_DLL					(DRAM_CSR_BASE + DDR_RX_DLL_OFF)
#define DDR_RX_DESKEW				(DRAM_CSR_BASE + DDR_RX_DESKEW_OFF)

#define CR0_FRCSMRCOMP				(DRAM_CSR_BASE + CR0_FRCSMRCOMP_OFF)
#define CR0_DSTRENGTHSEL			(DRAM_CSR_BASE + CR0_DSTRENGTHSEL_OFF)
#define CR0_DDQRCOMP				(DRAM_CSR_BASE + CR0_DDQRCOMP_OFF)
#define CR0_DCTLRCOMP				(DRAM_CSR_BASE + CR0_DCTLRCOMP_OFF)
#define CR0_DRCVRCOMP				(DRAM_CSR_BASE + CR0_DRCVRCOMP_OFF)
#define CR0_DCKERCOMP				(DRAM_CSR_BASE + CR0_DCKERCOMP_OFF)
#define CR0_DCSRCOMP				(DRAM_CSR_BASE + CR0_DCSRCOMP_OFF)
#define CR0_DCKRCOMP				(DRAM_CSR_BASE + CR0_DCKRCOMP_OFF)
#define CR0_DX8X16CKECSCKSEL		(DRAM_CSR_BASE + CR0_DX8X16CKECSCKSEL_OFF)
#define CR0_RCOMPPRD				(DRAM_CSR_BASE + CR0_RCOMPPRD_OFF)
#define CR0_DIGFIL					(DRAM_CSR_BASE + CR0_DIGFIL_OFF)

#define CR0_SLEWPROGRAMMED			(DRAM_CSR_BASE + CR0_SLEWPROGRAMMED_OFF)
#define CR0_OVRRIDEH				(DRAM_CSR_BASE + CR0_OVRRIDEH_OFF)
#define CR0_OVRRIDEV				(DRAM_CSR_BASE + CR0_OVRRIDEV_OFF)
#define CR0_JT_CONFIG				(DRAM_CSR_BASE + CR0_JT_CONFIG_OFF)

#define DU_CONTROL_REG				((volatile UINT32 *)DU_CONTROL)
#define DU_ERROR_STATUS_1_REG		((volatile UINT32 *)DU_ERROR_STATUS_1)
#define DU_ERROR_STATUS_2_REG		((volatile UINT32 *)DU_ERROR_STATUS_2)
#define DU_ECC_TEST_REG				((volatile UINT32 *)DU_ECC_TEST)
#define DU_INIT_REG					((volatile UINT32 *)DU_INIT)
#define DU_CONTROL2_REG				((volatile UINT32 *)DU_CONTROL2)
#define DDR_RCOMP_IO_CONFIG_REG		((volatile UINT32 *)DDR_RCOMP_IO_CONFIG)
#define DDR_RDDLYSEL_RECEN_REG		((volatile UINT32 *)DDR_RDDLYSEL_RECEN)
#define DDR_RX_DLL_REG				((volatile UINT32 *)DDR_RX_DLL)
#define DDR_RX_DESKEW_REG			((volatile UINT32 *)DDR_RX_DESKEW)

#define CR0_FRCSMRCOMP_REG			((volatile UINT32 *)CR0_FRCSMRCOMP)
#define CR0_DSTRENGTHSEL_REG		((volatile UINT32 *)CR0_DSTRENGTHSEL)
#define CR0_DDQRCOMP_REG			((volatile UINT32 *)CR0_DDQRCOMP)
#define CR0_DCTLRCOMP_REG			((volatile UINT32 *)CR0_DCTLRCOMP)
#define CR0_DRCVRCOMP_REG			((volatile UINT32 *)CR0_DRCVRCOMP)
#define CR0_DCKERCOMP_REG			((volatile UINT32 *)CR0_DCKERCOMP)
#define CR0_DCSRCOMP_REG			((volatile UINT32 *)CR0_DCSRCOMP)
#define CR0_DCKRCOMP_REG			((volatile UINT32 *)CR0_DCKRCOMP)
#define CR0_DX8X16CKECSCKSEL_REG	((volatile UINT32 *)CR0_DX8X16CKECSCKSEL)
#define CR0_RCOMPPRD_REG			((volatile UINT32 *)CR0_RCOMPPRD)
#define CR0_DIGFIL_REG				((volatile UINT32 *)CR0_DIGFIL)

#define CR0_SLEWPROGRAMMED_REG		((volatile UINT32 *)CR0_SLEWPROGRAMMED)
#define CR0_OVRRIDEH_REG			((volatile UINT32 *)CR0_OVRRIDEH)
#define CR0_OVRRIDEV_REG			((volatile UINT32 *)CR0_OVRRIDEV)
#define CR0_JT_CONFIG_REG			((volatile UINT32 *)CR0_JT_CONFIG)

/* interrupt registers */
#define INTERRUPT_CONTROLLER_BASE	0xD6000000

#define FIQ_STATUS_OFF				0x4
#define IRQ_STATUS_OFF				0x8
#define FIQ_ENABLE_OFF				0xC
#define FIQ_ENABLE_SET_OFF			0xC
#define IRQ_ENABLE_OFF				0x10
#define IRQ_ENABLE_SET_OFF			0x10
#define FIQ_ENABLE_CLR_OFF			0x14
#define IRQ_ENABLE_CLR_OFF			0x18
#define FIQ_IRQ_SOFT_INT_OFF		0x54

#define IRQ_STATUS_REG				((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + IRQ_STATUS_OFF))
#define IRQ_ENABLE_REG				((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + IRQ_ENABLE_OFF))
#define IRQ_ENABLE_SET_REG			((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + IRQ_ENABLE_SET_OFF))
#define IRQ_ENABLE_CLR_REG			((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + IRQ_ENABLE_CLR_OFF))
#define FIQ_STATUS_REG				((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + FIQ_STATUS_OFF))
#define FIQ_ENABLE_REG				((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + FIQ_ENABLE_OFF))
#define FIQ_ENABLE_SET_REG			((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + FIQ_ENABLE_SET_OFF))
#define FIQ_ENABLE_CLR_REG			((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + FIQ_ENABLE_CLR_OFF))
#define FIQ_IRQ_SOFT_INT_REG		((volatile UINT32 *)(INTERRUPT_CONTROLLER_BASE + FIQ_IRQ_SOFT_INT_OFF))

/* pci config registers */
#define PCI_CFG_REG_BASE			0xDE000000

#define PCI_CMD_STAT_OFF			0x4
#define PCI_CSR_BAR_OFF				0x10
#define PCI_SRAM_BAR_OFF			0x14
#define PCI_DRAM_BAR_OFF			0x18
#define PCI_RCOMP_OVER_OFF			0x60

#define PCI_CMD_STAT				(PCI_CFG_REG_BASE + PCI_CMD_STAT_OFF)
#define PCI_CSR_BAR					(PCI_CFG_REG_BASE + PCI_CSR_BAR_OFF)
#define PCI_SRAM_BAR				(PCI_CFG_REG_BASE + PCI_SRAM_BAR_OFF)
#define PCI_DRAM_BAR				(PCI_CFG_REG_BASE + PCI_DRAM_BAR_OFF)
#define PCI_RCOMP_OVER				(PCI_CFG_REG_BASE + PCI_RCOMP_OVER_OFF)

#define PCI_CMD_STAT_REG			((volatile UINT32 *)PCI_CMD_STAT)
#define PCI_CSR_BAR_REG				((volatile UINT32 *)PCI_CSR_BAR)
#define PCI_SRAM_BAR_REG			((volatile UINT32 *)PCI_SRAM_BAR)
#define PCI_DRAM_BAR_REG			((volatile UINT32 *)PCI_DRAM_BAR)
#define PCI_RCOMP_OVER_REG			((volatile UINT32 *)PCI_RCOMP_OVER)

/* pci local csrs */
#define PCI_LOCAL_CSR_BASE			0xDF000000

#define PCI_OUT_INT_MASK_OFF		0x34
#define MAILBOX_0_OFF				0x50
#define MAILBOX_1_OFF				0x54
#define MAILBOX_2_OFF				0x58
#define MAILBOX_3_OFF				0x5c
#define XSCALE_DOORBELL_OFF			0x60
#define SRAM_BASE_ADDR_MASK_OFF		0xFC
#define DRAM_BASE_ADDR_MASK_OFF		0x100
#define PCI_CONTROL_OFF				0x13C
#define PCI_ADDR_EXT_OFF			0x140
#define XSCALE_INT_STATUS_OFF		0x158
#define XSCALE_INT_ENABLE_OFF		0x15C

#define PCI_OUT_INT_MASK_REG		((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + PCI_OUT_INT_MASK_OFF))
#define MAILBOX_0_REG				((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF))
#define MAILBOX_1_REG				((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF))
#define MAILBOX_2_REG				((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF))
#define MAILBOX_3_REG				((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF))
#define XSCALE_DOORBELL_REG			((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + XSCALE_DOORBELL_OFF))
#define SRAM_BASE_ADDR_MASK_REG		((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + SRAM_BASE_ADDR_MASK_OFF))
#define DRAM_BASE_ADDR_MASK_REG		((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + DRAM_BASE_ADDR_MASK_OFF))
#define PCI_CONTROL_REG				((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + PCI_CONTROL_OFF))
#define PCI_ADDR_EXT_REG			((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + PCI_ADDR_EXT_OFF))
#define XSCALE_INT_STATUS_REG		((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + XSCALE_INT_STATUS_OFF))
#define XSCALE_INT_ENABLE_REG		((volatile UINT32 *)(PCI_LOCAL_CSR_BASE + XSCALE_INT_ENABLE_OFF))

/* csr addresses as seen from PCI */
#define GLOBAL_CONTROL_BASE_FRM_PCI	0x4A00

#define MISC_CONTROL_FRM_PCI		(GLOBAL_CONTROL_BASE_FRM_PCI + MISC_CONTROL_OFF)
#define IXP_RESET0_FRM_PCI			(GLOBAL_CONTROL_BASE_FRM_PCI + IXP_RESET0_OFF)
#define CCR_FRM_PCI					(GLOBAL_CONTROL_BASE_FRM_PCI + CCR_OFF)
#define STRAP_OPTIONS_FRM_PCI		(GLOBAL_CONTROL_BASE_FRM_PCI + STRAP_OPTIONS_OFF)

#define SLOW_PORT_CSR_BASE_FRM_PCI	0x80000

#define SCRATCH_BASE_FRM_PCI		0xF0000

#define QDR_CH_BASE_FRM_PCI			0xF9000

#define DRAM_CH0_BASE_FRM_PCI		0xFD800

#define DU_CONTROL_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + DU_CONTROL_OFF)
#define DU_ECC_TEST_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + DU_ECC_TEST_OFF)
#define DU_INIT_FRM_PCI				(DRAM_CH0_BASE_FRM_PCI + DU_INIT_OFF)
#define DU_CONTROL2_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + DU_CONTROL2_OFF)
#define DDR_RCOMP_IO_CONFIG_FRM_PCI	(DRAM_CH0_BASE_FRM_PCI + DDR_RCOMP_IO_CONFIG_OFF)
#define DDR_RDDLYSEL_RECEN_FRM_PCI	(DRAM_CH0_BASE_FRM_PCI + DDR_RDDLYSEL_RECEN_OFF)
#define DDR_RX_DLL_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + DDR_RX_DLL_OFF)
#define DDR_RX_DESKEW_FRM_PCI		(DRAM_CH0_BASE_FRM_PCI + DDR_RX_DESKEW_OFF)

#define CR0_FRCSMRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_FRCSMRCOMP_OFF)
#define CR0_DSTRENGTHSEL_FRM_PCI		(DRAM_CH0_BASE_FRM_PCI + CR0_DSTRENGTHSEL_OFF)
#define CR0_DDQRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DDQRCOMP_OFF)
#define CR0_DCTLRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DCTLRCOMP_OFF)
#define CR0_DRCVRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DRCVRCOMP_OFF)
#define CR0_DCKERCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DCKERCOMP_OFF)
#define CR0_DCSRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DCSRCOMP_OFF)
#define CR0_DCKRCOMP_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_DCKRCOMP_OFF)
#define CR0_DX8X16CKECSCKSEL_FRM_PCI	(DRAM_CH0_BASE_FRM_PCI + CR0_DX8X16CKECSCKSEL_OFF)
#define CR0_RCOMPPRD_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_RCOMPPRD_OFF)
#define CR0_DIGFIL_FRM_PCI				(DRAM_CH0_BASE_FRM_PCI + CR0_DIGFIL_OFF)

#define CR0_SLEWPROGRAMMED_FRM_PCI		(DRAM_CH0_BASE_FRM_PCI + CR0_SLEWPROGRAMMED_OFF)
#define CR0_OVRRIDEH_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_OVRRIDEH_OFF)
#define CR0_OVRRIDEV_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_OVRRIDEV_OFF)
#define CR0_JT_CONFIG_FRM_PCI			(DRAM_CH0_BASE_FRM_PCI + CR0_JT_CONFIG_OFF)

#define PCI_CSR_BASE_FRM_PCI		0xFE000

#define PCI_OUT_INT_MASK_FRM_PCI	(PCI_CSR_BASE_FRM_PCI + PCI_OUT_INT_MASK_OFF)
#define PCI_CONTROL_FRM_PCI			(PCI_CSR_BASE_FRM_PCI + PCI_CONTROL_OFF)
#define PCI_ADDR_EXT_FRM_PCI		(PCI_CSR_BASE_FRM_PCI + PCI_ADDR_EXT_OFF)

#define TB_VENDOR_ID				0x8086
#define TB_DEVICE_ID				0xB154

/* 21154 PCI-to-PCI Bridge register offset */
#define VIDR_OFF					0x0
#define DIDR_OFF					0x2
#define PCR_OFF						0x4
#define PSR_OFF						0x6
#define RIDR_OFF					0x8
#define PIR_OFF						0x9
#define CLSR_OFF					0xC
#define PLTR_OFF					0xD
#define HTR_OFF						0xE
#define PBNR_OFF					0x18
#define SBNR_OFF					0x19
#define SUBBNR_OFF					0x1A
#define SLTR_OFF					0x1B
#define IOBR_OFF					0x1C
#define IOLR_OFF					0x1D
#define SSR_OFF						0x1E
#define MBR_OFF						0x20
#define MLR_OFF						0x22
#define PMBR_OFF					0x24
#define PMLR_OFF					0x26
#define ECPPR_OFF					0x34
#define SUBIDR_OFF					0x36
#define BCR_OFF						0x3E
#define TB_CCR_OFF					0x40
#define	ACR_OFF						0x42


// UART defines start 
#define DEFAULT_BAUD	        57600
#define DEFAULT_BAUD_DIVISOR    (50000000 / (16 * DEFAULT_BAUD)) 

#define UARTLCR_DivisorLatchAccess 0x80
#define UART_LineControl           0x0C     //reg addr
#define UART_DivisorLatchLSB       0x00	    //reg addr DLAB=0
#define UART_DivisorLatchMSB       0x04	    //reg addr DLAB=1
#define UARTLCR_CharLength8        0x03  
#define UARTLCR_StopBits1          0x00  
#define UART_InterruptEnable       0x04     //reg addr DLAB=0
#define UARTFCR_Enable             (1<<0)
#define UART_FIFOControl           0x08     //reg addr
#define UARTFCR_RXReset            (1<<1)
#define ARTFCR_TXReset             (1<<2)
#define UARTFCR_Mode0RXRDYTXRDY    (0<<3)
#define UARTFCR_RXTrigger1         (0<<6)
#define UART_LineStatus            0x14     //reg addr
#define UART_Transmit              0        //reg addr
#define UART_Receive               0        //reg addr
#define UARTLSR_TXHoldingEmpty     (1<<5)
#define IXP2400_UART_ADDR		   0xC0030000

//end of UART defines 
#endif /* ONCE_HAL_IXDP2400_H */
