#ifndef ONCE_HAL_PLATFORM_INTS_H
#define ONCE_HAL_PLATFORM_INTS_H
//==========================================================================
//
//      hal_platform_ints.h
//
//      HAL Interrupt and clock support
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
#include "common.h"

typedef void pci_ISR(UINT32 vector, UINT32 data);

#define PCI_INTERRUPT_BASE				32
#define INT_REG_EMPTY					0
#define PCI_ISR_COUNT					8 // for CPLD interrupts

#define HAL_INTERRUPT_reserved0	0
#define HAL_INTERRUPT_SOFT		1 // soft interrupt
#define HAL_INTERRUPT_ERR		2 // error
#define HAL_INTERRUPT_SERIAL	3 // UART
#define HAL_INTERRUPT_GPIO		4 // GPIO
#define HAL_INTERRUPT_TIMER_1	5 // Timer 1
#define HAL_INTERRUPT_TIMER_2	6 // timer 2
#define HAL_INTERRUPT_TIMER_3	7 // timer 3
#define HAL_INTERRUPT_TIMER_4	8 // timer 4
#define HAL_INTERRUPT_SLOW_PORT	9 // slow port
#define HAL_INTERRUPT_FRAMER	10 // framer
#define HAL_INTERRUPT_DMA_1		11 // DMA engine 1
#define HAL_INTERRUPT_DMA_2		12 // DMA engine 2
#define HAL_INTERRUPT_DMA_3		13 // DMA engine 3
#define HAL_INTERRUPT_PCI_DBELL	14 // doorbell
#define HAL_INTERRUPT_ME_ATTN	15 // microengine
#define HAL_INTERRUPT_PCI		16 // pci inta or intb

#define HAL_INTERRUPT_PCI_CONT	(PCI_INTERRUPT_BASE + 26) // PCI control
#define HAL_INTERRUPT_PCI_INTA	(PCI_INTERRUPT_BASE + 27) // INTA
#define HAL_INTERRUPT_PCI_INTB	(PCI_INTERRUPT_BASE + 28) // INTB
#define HAL_INTERRUPT_PCI_DMA_1	(PCI_INTERRUPT_BASE + 29) // DMA engine 1
#define HAL_INTERRUPT_PCI_DMA_2	(PCI_INTERRUPT_BASE + 30) // DMA engine 2
#define HAL_INTERRUPT_PCI_DMA_3	(PCI_INTERRUPT_BASE + 31) // DMA engine 3

// CPLD PCI interrupt bits
#define CPLD_INGRESS_NPU_INT			0
#define CPLD_NIC_INT					1
#define CPLD_MEDIA_PCI_INT				2
#define CPLD_MEDIA_SLOWPORT_INT			3
#define CPLD_SF_PCI_INT					4
#define CPLD_SF_SLOWPORT_INT			5
#define CPLD_PMC_INT					6
#define CPLD_ADUC_INT					7

// CPLD interrupt register defines
#define ETHERNET_INT					(1 << 1)

#define HAL_ISR_MIN              0
#define HAL_ISR_MAX              64

#define HAL_ISR_COUNT            (HAL_ISR_MAX+1)

// The vector used by the Real time clock
#define HAL_INTERRUPT_RTC        HAL_INTERRUPT_TIMER_1

extern void hal_delay_us(UINT32 usecs);
#define HAL_DELAY_US(n)          hal_delay_us(n);

//----------------------------------------------------------------------------
// Reset.
#include "hal_ixdp2400.h"        // registers
#include "hal_io.h"             // IO macros

// reset the board
extern void ixdp2400_reset(void);
#define HAL_PLATFORM_RESET()   ixdp2400_reset()

#define HAL_PLATFORM_RESET_ENTRY 0x00000000

void pciIntConnect(int vector, UINT32 routine, UINT32 data);
void pciIntDisconnect(int vector);

void hal_interrupt_mask_all(void);
void hal_interrupt_unmask_all(void);

#endif // ONCE_HAL_PLATFORM_INTS_H
