/* hal_platform_setup.h
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXP2400
subsystem: DIAG
author: lagarwal
revisions:
--------------------------------------------------------------------------
*/
#ifndef ONCE_HAL_PLATFORM_SETUP_H
#define ONCE_HAL_PLATFORM_SETUP_H

.include "../include/mlt_arm_ixdp2400_romram_arm.h"	@@ Location of the ROM
.include "../include/hal_ixdp2400_arm.h"				@@ Platform specific hardware definitions
.include "../include/hal_mmu_arm.h"					@@ MMU definitions
.include "../include/post.h"							@@ POST

.equ MMU_Control_BTB, 0x800

.macro RAMRAM_COPY

	mov r0, pc
	ldr r4, =0xFFFF0000
	and r0, r0, r4
	mov r4, r0		@store for PC jump, which is perform after copy
	ldr r1, =0
	ldr r2, =(0x200000)	@ 2MB to copy
ram_copy_loop:
	ldr r3, [r0], #4
    str r3, [r1], #4
	cmp r1, r2
	bne ram_copy_loop
.endm
	

@@ If we are doing a ROMRAM startup copy all sections up to the start of 
@@ the data section to RAM.
.macro ROMRAM_COPY
	ldr r0, =(MEM_REGION_rom)
    ldr r1, =0
    ldr r2, =(MEM_REGION_rom_SIZE)
copy_loop:
	ldr r3, [r0], #4
    str r3, [r1], #4
	cmp r1, r2
	bne copy_loop
    ldr r0, =830f
    mov pc, r0
830:                                                                
.endm

@@define PLATFORM_SETUP1 _platform_setup1

    @@ Display all the 4 chars.
	.macro HEX_DISPLAY char4, char3, char2, char1
	ldr r0, =PRODUCT_ID
	ldr r1, [r0]
	and r1, r1, #0xF0
	ldr	r0, =ALPHANUM_DIS_DATA
	cmp r1, #0
	@@ branch if A step
	beq 669f

	@@ else B step
	ldrb r1, =\char1
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char2
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char3
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char4
	strb r1, [r0]
	b 671f

669:	
	ldrb r1, =\char4
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char3
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char2
	strb r1, [r0]
	add r0, r0, #1
	ldrb r1, =\char1
	strb r1, [r0]
	b 671f

	.ltorg
671:
	.endm

	@@ wait for coprocessor write complete
	.macro CPWAIT reg
    mrc  p15,0,\reg,c2,c0,0
	mov  \reg,\reg
	sub  pc,pc,#4
	.endm

	@@ Enable the BTB
	.macro BTB_INIT reg
	mrc	p15, 0, \reg, c1, c0, 0
	orr	\reg, \reg, #MMU_Control_BTB
	mcr	p15, 0, \reg, c1, c0, 0
	CPWAIT  \reg
	.endm

	@@ form a first-level section entry
	.macro FL_SECTION_ENTRY base,x,ap,p,d,c,b
	.word (\base << 20) | (\x << 12) | (\ap << 10) | (\p << 9) | (\d << 5) | (\c << 3) | (\b << 2) | 2

	.endm

	@@ form a first-level page table entry
	.macro FL_PT_ENTRY base,p,d
	@@ I wanted to use logical operations here, but since I am using symbols later 
	@@ to fill in the parameters, I had to use addition to force the assembler to
	@@ do it right
	.word \base + (\p << 9) + (\d << 5) + 1
	.endm

	@@ form a second level small page entry
	.macro SL_SMPAGE_ENTRY base,ap3,ap2,ap1,ap0,c,b
	.word (\base << 12) | (\ap3 << 10) | (\ap2 << 8) | (\ap1 << 6) | (\ap0 << 4) | (\c << 3) | (\b << 2) | 2

	.endm

	@@ form a second level extended small page entry
	.macro SL_XSMPAGE_ENTRY base,x,ap,c,b
	.word (\base << 12) | (\x << 6) | (\ap << 4) | (\c << 3) | (\b << 2) | 3
	.endm


	@@ start of platform setup
	.macro _platform_setup1

	@@ This is where we wind up immediately after reset.
	@@ since we need to align the mmu table on a 16k boundary, we just branch around the page
	@@ table which we will locate at FLASH_BASE+0x4000.
	b _real_platform_setup

	.p2align 13
	@@ the following alignment creates the mmu table at address 0x4000.
    mmu_table:

	.set __base,0

	@@ make first Mb of SDRAM cacheable
	.rept 0x1
	FL_SECTION_ENTRY __base,0,3,0,0,1,1
	.set __base,__base+1
	.endr

	@@ make 2nd Mb of SDRAM non-cacheable
	.rept 0x1
	FL_SECTION_ENTRY __base,1,3,0,0,0,1
	.set __base,__base+1
	.endr

	@@ Make rest of SDRAM and SRAM rw, cacheable and bufferable
	.rept 0x800 - 0x2
	FL_SECTION_ENTRY __base,0,3,0,0,1,1
	.set __base,__base+1
	.endr

	@@ map sram
	.rept 0xC00 - 0x800
	FL_SECTION_ENTRY __base,0,3,0,0,1,1
	.set __base,__base+1
	.endr

	@@ Make CSR, flash and PCI Mem as rw, non-cacheable and non-bufferable
	.rept 0x1000 - 0xC00
	FL_SECTION_ENTRY __base,1,3,0,0,0,1
	.set __base,__base+1
	.endr

_real_platform_setup:
@ .ifdef __ARMEB__
	@@ switch to big endian mode first thing so that we don't have to mess with constants
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #0x80
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT   r0
@ .endif */
	ldr r0, =STRAP_OPTIONS			@@ Load the address of the strap options Register in r0
	ldr r1, [r0]					@@ get value in r1
	and r10, r1, #CFG_PROM_BOOT		@@ save single/dual boot rom in r10
	and r9, r1, #CFG_PCI_BOOT_HOST

	@@ check if coming from BM
	mov r2, pc
	cmp r2, #(0x200000)	@assumes running from any address greater than first 2Mb to be from BM
	bls 	_standalone_init

_booting_from_BM:
	RAMRAM_COPY

	HEX_DISPLAY 'J', 'U', 'M', 'P'
		
	nop
	sub r2, pc, r4
	mov pc, r2
	nop

	@indicate boot from BM
	mov r2, #1
	ldr r1, =boot_from_boot_monitor
	str	r2, [r1]
	ldr	r1, =untouched_diag_location
	str r4, [r1]

	b	warm_reset

_standalone_init:

	@@ read the value of RTC1 and put it in WTC1, RTC2, WTC2 register
	ldr r0, =SP_RTC1
	ldr r1, [r0]
	ldr	r0, =SP_WTC1		@@ Load the address of the slow port write timing control Register in r0
	str	r1, [r0]			@@ Store to slow port write timing control Register
	ldr	r0, =SP_WTC2
	str	r1, [r0]
	ldr	r0, =SP_RTC2
	str	r1, [r0]

	ldr	r0, =MISC_CONTROL				@@ Load the address of the misc control Register in r0
	ldr	r1, [r0]						@@ store old value in r1
	ldr r2, =FLASH_WRITE_ENABLE
	orr r1, r1, r2						@@ make flash write enable for flash utility
	str	r1, [r0]						@@ Store to misc control Register

	HEX_DISPLAY ' ', 'P', 'C', 'I'

	cmp r10, #0
	/* On slave if flash is not present then I am doing all the init. from master so skip everything */
	beq do_post

	/* take pci out of reset
	   I am taking PCI out of reset here and not in cyg_hal_plf_pci_init function so that in dual flash mode
	   slave can set init_comp bit as soon as possible */
	ldr	r0, =IXP_RESET0
	ldr	r1, [r0]
	bic	r1, r1, #PCIRST
	bic	r1, r1, #RESET_PCI
	str	r1, [r0]

	HEX_DISPLAY 'P', 'C', 'I', 'C'
	@@ 8 dummy writes to flush pci cmd fifo
	mov r3, #0x0
	ldr	r0, =(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF)
	ldr	r1, =0x0
pci_cmd_fifo_flush_loop:
	str	r1, [r0]
	add r3, r3, #1
	cmp r3, #8
	bne pci_cmd_fifo_flush_loop

/*@ #ifdef A0_REV
	HEX_DISPLAY 'P', 'C', 'I', 'R'
	@@ set PCI rcomp registers
	ldr	r0, =PCI_RCOMP_OVER
	ldr	r1, =0x153239
	str	r1, [r0]
@ #endif */

	cmp r9, #0
	/* On slave skip setting pci rcomp register */
	beq ddr_rcomp_set

/*@ #ifdef A0_REV
	@@ I have to write slave's pci rcomp register within 2000 cycles.
	ldr	r0, =0xDA200060
	ldr	r1, =0x153239
	str	r1, [r0]
@ #endif */

	HEX_DISPLAY 'D', 'D', 'R', 'R'

ddr_rcomp_set:
	/* now DRAM rcomp IO config registers */
	ldr	r0, =DDR_RCOMP_IO_CONFIG
	ldr	r1, =DDR_RCOMP_IO_CONFIG_VAL
	str	r1, [r0]

/*@ #ifdef A0_REV
	ldr r0, =SYS_CLK_M
	ldr r1, [r0]
	and r1, r1, #0xFF
	ldr r4, =SRAM_150MHZ_DIVISOR
	cmp r1, #0x55
	bne sram_divisor_set
@ #endif */
	ldr r4, =SRAM_200MHZ_DIVISOR

sram_divisor_set:
	/* now set the operating freq. for SRAM/SDRAM interface */
	ldr	r0, =CCR						@@ Load the address of the clock control Register in r0
	ldr	r1, [r0]						@@ store old value in r1
	ldr r2, =(~(0xF00FF))
	and r1, r1, r2						@@ clear sdram and sram divisor default value
	orr r1, r1, #(SDRAM_DIVISOR << 16)	@@ Set SDRAM clock ratio
	mov r5, r4, lsl #4
	orr r1, r1, r5						@@ Set SRAM channel 1 clock ratio
	orr r1, r1, r4						@@ Set SRAM channel 0 clock ratio
	mov r3, #0x0
	mcr p15, 0, r0, c7, c10, 4    		@@ DCU drain instruction
	b clk_stab

	/* wait for some time so that clock gets stabilized
	   make this code aligned at 32 bytes so that it is aligned on prefetch boundary */
	.p2align 5
clk_stab:
	str	r1, [r0]						@@ Store to clock control Register
	HEX_DISPLAY 'C', 'L', 'K', 'I'
clk_stab_loop:
	add r3, r3, #1
	cmp r3, #0x200
	bne clk_stab_loop
	b	clk_step_done

	.p2align 5
clk_step_done:
	HEX_DISPLAY 'C', 'L', 'K', 'D'
	/* now DRAM IO config registers */
	ldr	r0, =DDR_RDDLYSEL_RECEN
	ldr	r1, =DDR_RDDLYSEL_RECEN_VAL
	str	r1, [r0]

	ldr	r0, =DDR_RX_DLL
	ldr	r1, =DDR_RX_DLL_VAL
	str	r1, [r0]

	ldr	r0, =DDR_RX_DESKEW
	ldr	r1, =DDR_RX_DESKEW_VAL
	str	r1, [r0]




	/*Rcomp Initialisation
	  Select Stength for Pin Groups */
	ldr r0, =CR0_DSTRENGTHSEL
	ldr r1, =0x036db6db		@ rcv 1.5x rest 1.5x 
	str	r1, [r0]

	/* Program Rcomp offset registers to 0 (until characterised values are available)
	   Bit[15] = Sign for pulldown
	   Bit[14:8] = Value for Pulldown
	   Bit[7] = Sign for pullup
	   Bit[6:0] = Value for Pullup */

	ldr r1, =0x00000000 	
	
	ldr r0, =CR0_DDQRCOMP		@ DQ/DQS offset
	str	r1, [r0]

	ldr	r0, =CR0_DCTLRCOMP		@ MA/BA/RAS#/CAS#/WE# offset
	str	r1, [r0]

	ldr r0, =CR0_DRCVRCOMP		@ RCV offset
	str	r1, [r0]

	ldr r0, =CR0_DCKERCOMP		@ CKE offset
	str	r1, [r0]

	ldr r0, =CR0_DCSRCOMP		@ CS# offset
	str	r1, [r0]

	ldr r0, =CR0_DCKRCOMP		@ CK/CK# offset [31:16 for X16, 15:0 for x8]
	str	r1, [r0]

	/* Select x8 or x16 slew compensation for CKE, CS, CK */
	ldr r0, =CR0_DX8X16CKECSCKSEL
	ldr r1, =0x11111111		@ for x8 slew compensation
	str	r1, [r0]

	/* Select rcomp Period */
	ldr r0, =CR0_RCOMPPRD
	ldr r1, =0x00000000		@ for period = 2097152 clock cyles
	str	r1, [r0]

	/* set Dig Filter Clamp */
	ldr r0, =CR0_DIGFIL
	ldr r1, =0x00000000		@ for no filter
	str	r1, [r0]


	/* Program DDR DQ/DQS pull-up and pull-down Slew Lookup table registers */
	/* (cr0_ddqpslew0 - cr0_ddqpslew3) and (cr0_ddqnslew0 - cr0_ddqnslew3) */

	ldr r1, =0xcccccccc

	ldr r0, =0xd0000150
	ldr r2, =0xd0000188
DQ_DQS:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble DQ_DQS

	/* Program MA/BA/RAS#/CAS#/WE# pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dctlpslew0 - cr0_dctlpslew3) and (cr0_dctlnslew0 - cr0_dctlnslew3) */
	ldr r0, =0xd0000198
	ldr r2, =0xd00001D0
MA_BA_RAS_CAS_WE:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble MA_BA_RAS_CAS_WE


	/* Program RCV pull-up and pull-down Slew Lookup table registers */
	/* (cr0_drcvpslew0 - cr0_drcvpslew3) and (cr0_drcvnslew0 - cr0_drcvnslew3) */
	ldr r0, =0xd00001E0
	ldr r2, =0xd0000218
RCV:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble RCV


	/* Program CKE x8 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dckex8pslew0 - cr0_dckex8pslew3) and (cr0_dckex8nslew0 - cr0_dckex8nslew3) */
	ldr r0, =0xd0000230
	ldr r2, =0xd0000268
CKE_x8:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CKE_x8


	/* Program CKE x16 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dckex16pslew0 - cr0_dckex16pslew3) and (cr0_dckex16nslew0 - cr0_dckex16nslew3) */
	ldr r0, =0xd0000270
	ldr r2, =0xd00002A8
CKE_x16:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CKE_x16


	/* Program CS# x8 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dcsx8pslew0 - cr0_dcsx8pslew3) and (cr0_dcsx8nslew0 - cr0_dcsx8nslew3) */
	ldr r0, =0xd00002B8
	ldr r2, =0xd00002F0
CS_x8:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CS_x8


	/* Program CS# x16 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dcsx16pslew0 - cr0_dcsx16pslew3) and (cr0_dcsx16nslew0 - cr0_dcsx16nslew3) */
	ldr r0, =0xd00002F8
	ldr r2, =0xd0000330
CS_x16:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CS_x16


	/* Program CK CK# x8 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dckx8pslew0 - cr0_dckx8pslew3) and (cr0_dckx8nslew0 - cr0_dckx8nslew3) */
	ldr r0, =0xd0000340
	ldr r2, =0xd0000378
CK_x8:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CK_x8


	/* Program CK CK# x16 pull-up and pull-down Slew Lookup table registers */
	/* (cr0_dckx16pslew0 - cr0_dckx16pslew3) and (cr0_dckx16nslew0 - cr0_dckx16nslew3) */
	ldr r0, =0xd0000380
	ldr r2, =0xd00003B8

CK_x16:
	str	r1, [r0]
	add r0, r0, #0x8
	cmp r0, r2
	ble CK_x16


/* Flag Slew Programmed register as done */
	ldr r0, =CR0_SLEWPROGRAMMED
	ldr r1, =0x00000001
	str	r1, [r0]

/* cr0_ovrrideh: turn off h override, 0x0 */
	
	ldr r1, =0x00000000

	ldr r0, =CR0_OVRRIDEH	
	str	r1, [r0]

/* cr0_ovrridev: turn off v override, 0x0 */
	ldr r0, =CR0_OVRRIDEV
	str	r1, [r0]

/* De-Select Test Mode (cr0_jt_config) at address offset 0x3C0 
// User Overide RCOMP settings */
	ldr r0, =CR0_JT_CONFIG
	str	r1, [r0]




	@@ wait for 2000 cycles
	mov r0, #0x0
wait_ddr_io:
	add r0, r0, #1
	cmp r0, #2000
	bne wait_ddr_io

	HEX_DISPLAY 'D', 'D', 'R', 'I'

	@@ now config SDRAM interface
	ldr	r0, =DU_CONTROL		@@ Load the address of the dram controller control Register in r0
	ldr	r1, =DU_CONTROL_VAL	@@ Load value in r1
	str	r1, [r0]			@@ Store to dram controller control Register

	@@ enable clock
	ldr	r0, =DU_INIT
	ldr	r1, =CKE
	str	r1, [r0]

	@@ precharge all
	ldr	r0, =DU_INIT
	ldr	r1, =(PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL)
	str	r1, [r0]
wait_precharge1:
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #PRECHARGE
	bne wait_precharge1

	@@ issue EMRS to enable DLL
	ldr	r0, =DU_INIT
	ldr	r1, =(LD_MODE_REG | SIDE1 | SIDE0 | EXT_LD_MODE)
	str	r1, [r0]
wait_enable:
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #LD_MODE_REG
	bne wait_enable

	@@ Reset DLL
	ldr	r0, =DU_INIT
	ldr	r1, =(LD_MODE_REG | SIDE1 | SIDE0 | RESET_DLL)
	str	r1, [r0]
wait_reset:
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #LD_MODE_REG
	bne wait_reset

	@@ precharge all
	ldr	r0, =DU_INIT
	ldr	r1, =(PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL)
	str	r1, [r0]
wait_precharge2:
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #PRECHARGE
	bne wait_precharge2

	@@ issue 2 auto refresh commands
	ldr	r0, =DU_INIT
	ldr	r1, =(REFRESH | SIDE1 | SIDE0)
	str	r1, [r0]
wait_refresh1:
	@@ wait till auto refresh is done
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #REFRESH
	bne wait_refresh1

	ldr	r0, =DU_INIT
	ldr	r1, =(REFRESH | SIDE1 | SIDE0)
	str	r1, [r0]
wait_refresh2:
	@@ wait till auto refresh is done
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #REFRESH
	bne wait_refresh2

	@@ init device operation
	@@ settings - burst length = 4, burst type - sequential, CAS latency = 2.5
	ldr	r0, =DU_INIT
	ldr	r1, =(LD_MODE_REG | SIDE1 | SIDE0 | LOAD_MODE_NORMAL)
	str	r1, [r0]
wait_start:
	ldr r0, =DU_INIT
	ldr r1, [r0]
	tst	r1, #LD_MODE_REG
	bne wait_start

	@@ disable ecc so that it doesn't generate ecc errors.
	@@ even if you disable ecc ecc will be calculated and stored
	ldr r0, =DU_ECC_TEST		@@ load the register address in r0        
    ldr r1, [r0]				@@ read the reg contnets into r1
    orr r1, r1, #DISABLE_CHK	@@ set the disable bit
    str r1, [r0]				@@ write the value to the register

	HEX_DISPLAY 'P', 'O', 'S', 'T'

	@@ here is what I am doing here.
	@@ flash is aliased at 0 so I am trying to start executing from real flash
	@@ addr. so that when I unalias flash I still have the right value in pc.
	ldr	r2, =flash_real_addr
	ldr	r3, =SLOW_PORT_BASE
	add r2, r3, r2
	mov pc, r2

flash_real_addr:
	ldr	r0, =MISC_CONTROL				@@ Load the address of the misc control Register in r0
	ldr	r1, [r0]						@@ store old value in r1
	ldr r2, =FLASH_ALIAS_DISABLE
	orr r1, r1, r2						@@ disable flash aliasing
	str	r1, [r0]						@@ Store to misc control Register

do_post:
	POST	@@ POST reads SPD, calculates DRAM size and stores it in r8

	cmp r10, #0
	@@ On slave if flash is not present then jump to scrubbing
	beq scrub_set

	@@ reconfig DDR control register to reflect the right size
	ldr	r0, =DU_CONTROL
	cmp r8, #(512 * 1024 * 1024)
	beq du_control_set
	ldr	r1, =DU_CONTROL_VAL_1G
	str	r1, [r0]

du_control_set:
	HEX_DISPLAY 'C', 'O', 'P', 'Y'

	ROMRAM_COPY

	HEX_DISPLAY 'S', 'C', 'R', 'B'

	ldr	r11, =(MEM_REGION_ram + MEM_REGION_rom_SIZE)
	b scrub_mem

scrub_set:
	@@ Master scrubs first 2M of slave
	ldr	r11, =(MEM_REGION_ram + 0x200000)

scrub_mem:
	@@ scrub/init SDRAM
	mov	r0, #0				@@ scrub with 0x0000:0000
	mov	r1, #0
	mov	r2, #0	
	mov	r3, #0
	mov	r4, #0
	mov	r5, #0
	mov	r6, #0
	mov	r7, #0

10: @@ fastScrubLoop
	stmia r11!, {r0-r7}
	cmp r11, r8
	beq	15f
	b 10b

15:
	HEX_DISPLAY ' ', 'M', 'M', 'U'

	@@ Set POST flag location to complete
	ldr	r0, =SLAVE_POST_FLAG_LOC
	ldr	r1, =POST_COMPLETE
	str	r1, [r0]

	ldr r0, =DU_ECC_TEST		@@ Load the register addr into r0
    ldr r1, [r0]			    @@ load the register contents into r1
    bic r1, r1, #DISABLE_CHK	@@ clear the DISABLE_CHK bit
    str r1, [r0]				@@ write it back to the register

	@@ Enable access to all coprocessor registers
	ldr	r0, =0x2001			@@ enable access to all coprocessors
	mcr	p15, 0, r0, c15, c1, 0
	
	mcr	p15, 0, r0, c7, c10, 4		@@ drain the write & fill buffers
	CPWAIT r0		
	
	mcr	p15, 0, r0, c7, c7, 0		@@ flush Icache, Dcache and BTB
	CPWAIT r0	
	
	mcr	p15, 0, r0, c8, c7, 0		@@ flush instuction and data TLBs
	CPWAIT r0	
	
	@@ Enable the Icache
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_I
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT  r0

	@@ Set the TTB register
	ldr	r0, =0x4000
	mcr	p15, 0, r0, c2, c0, 0

	@@ Enable permission checks in all domains
	ldr	r0, =0x55555555
	mcr	p15, 0, r0, c3, c0, 0
	
	@@ Enable the MMU
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_M
	orr	r0, r0, #MMU_Control_R
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT  r0
	
	mcr	p15, 0, r0, c7, c10, 4		@@ drain the write & fill buffers
	CPWAIT r0

	@@ Enable the Dcache
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_C
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT  r0

    @@ Initialize branch target buffer
    BTB_INIT r0

	@@ Save SDRAM size
    ldr     r1, =hal_dram_size
	str	r8, [r1]
	.endm    @@ _platform_setup1

/*---------------------------------------------------------------------------*/
/* end of hal_platform_setup.h                                               */
#endif /* ONCE_HAL_PLATFORM_SETUP_H */
