//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#ifndef ONCE_INFRA_TYPE_H
#define ONCE_INFRA_TYPE_H

//#include <stddef.h>           // Definition of NULL from the compiler

// -------------------------------------------------------------------------
// Some useful macros. These are defined here by default.

// __externC is used in mixed C/C++ headers to force C linkage on an external
// definition. It avoids having to put all sorts of ifdefs in.

#define __externC extern
// Also define externC for now - but it is deprecated
#define externC __externC

// -------------------------------------------------------------------------
// The header <basetype.h> defines the base types used here. It is
// supplied either by the target architecture HAL, or by the host
// porting kit. They are all defined as macros, and only those that
// make choices other than the defaults given below need be defined.

#define LSBFIRST 1234
#define MSBFIRST 4321

#ifdef __ARMEB__
	#define BYTEORDER		MSBFIRST    // Big endian
#else
	#define BYTEORDER		LSBFIRST    // Little endian
#endif
#define DOUBLE_BYTEORDER	MSBFIRST    // Big? endian

//-----------------------------------------------------------------------------
// ARM does not usually use labels with underscores.

#define LABEL_NAME(_name_) _name_
#define LABEL_DEFN(_name_) _name_

//-----------------------------------------------------------------------------
// Override the alignment definitions from type.h. ARM only allows 4 
// byte alignment whereas the default is 8 byte.

#define ARC_ALIGNMENT 4
#define ARC_P2ALIGNMENT 2
#if (ARC_ALIGNMENT) != (1 << ARC_P2ALIGNMENT)
	#error "Inconsistent ARC_ALIGNMENT and ARC_P2ALIGNMENT values"
#endif

typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned int   UINT; 
typedef unsigned long long   ULONG;
typedef unsigned char*  PUCHAR;
typedef unsigned short* PUSHORT;
typedef unsigned int*   PUINT;
typedef unsigned long long*   PULONG;
typedef unsigned char   UINT8,  *PU8;
typedef unsigned short  UINT16, *PU16;
typedef unsigned int    UINT32, *PU32;
typedef char            S8,  *PS8;
typedef short           S16, *PS16;
typedef int             S32, *PS32;

// -------------------------------------------------------------------------
// The obvious few that compilers may define for you.
// But in case they don't:

#ifndef NULL
# define NULL 0
#endif

#ifndef __cplusplus

//typedef halbool bool;

# ifndef false
#  define false 0
# endif

# ifndef true
#  define true (!false)
# endif

#endif

// -------------------------------------------------------------------------
// Allow creation of procedure-like macros that are a single statement,
// and must be followed by a semi-colon

#define MACRO_START do {
#define MACRO_END   } while (0)

// -------------------------------------------------------------------------
// Constructor ordering macros.  These are added as annotations to all
// static objects to order the constuctors appropriately.

#if defined(__cplusplus) && defined(__GNUC__)
# define BLD_ATTRIB_INIT_PRI( _pri_ ) __attribute__((init_priority(_pri_)))
#else
// FIXME: should maybe just bomb out if this is attempted anywhere else?
// Not sure
# define BLD_ATTRIB_INIT_PRI( _pri_ )
#endif
    
// The following will be removed eventually as it doesn't allow the use of
// e.g. pri+5 format
#define INIT_PRIORITY( _pri_ ) BLD_ATTRIB_INIT_PRI( INIT_##_pri_ )

#define BLD_ATTRIB_INIT_BEFORE( _pri_ ) BLD_ATTRIB_INIT_PRI(_pri_-100)
#define BLD_ATTRIB_INIT_AFTER( _pri_ )  BLD_ATTRIB_INIT_PRI(_pri_+100)

#define INIT_HAL                    10000
#define INIT_SCHEDULER              11000
#define INIT_INTERRUPTS             12000
#define INIT_DRIVERS                13000
#define INIT_CLOCK                  14000
#define INIT_IDLE_THREAD            15000
#define INIT_THREADS                16000
#define INIT_KERNEL                 40000
#define INIT_MEMALLOC               47000
#define INIT_IO                     49000
#define INIT_LIBC                   50000
#define INIT_COMPAT                 55000
#define INIT_APPLICATION            60000
#define INIT_PREDEFAULT             65534
#define INIT_DEFAULT                65535

// -------------------------------------------------------------------------
// COMPILER-SPECIFIC STUFF

#ifdef __GNUC__
	// Force a 'C' routine to be called like a 'C++' contructor
	#define BLD_ATTRIB_CONSTRUCTOR __attribute__((constructor))
	// Define a compiler-specific rune for saying a function doesn't return
	#define BLD_ATTRIB_NORET __attribute__((noreturn))

	// How to define weak symbols - this is only relevant for ELF and a.out,
	// but that won't be a problem for eCos
	#define BLD_ATTRIB_WEAK __attribute__ ((weak))

	// How to define alias to symbols. Just pass in the symbol itself, not
	// the string name of the symbol
	#define BLD_ATTRIB_ALIAS(__symbol__) \
	        __attribute__ ((alias (#__symbol__)))

	// This effectively does the reverse of the previous macro. It defines
	// a name that the attributed variable or function will actually have
	// in assembler.
	#define __Str(x) #x
	#define __Xstr(x) __Str(x)
	#define BLD_ATTRIB_ASM_ALIAS(__symbol__) \
	            __asm__ ( __Xstr( LABEL_DEFN( __symbol__ ) ) )

	// Shows that a function returns the same value when given the same args, but
	// note this can't be used if there are pointer args
	#define BLD_ATTRIB_CONST __attribute__((const))

	// Assign a defined variable to a specific section
	#define BLD_ATTRIB_SECTION(__sect__) __attribute__((section (__sect__)))

	// Give a type or object explicit minimum alignment
	#define BLD_ATTRIB_ALIGN(__align__) __attribute__((aligned(__align__)))

#else // non-GNU

	#define BLD_ATTRIB_CONSTRUCTOR

	#define BLD_ATTRIB_NORET
	    // This intentionally gives an error only if we actually try to
	    // use it.  #error would give an error if we simply can't.
	// FIXME: Had to disarm the bomb - the BLD_ATTRIB_WEAK macro is now
	//        (indirectly) used in host tools.
	#define BLD_ATTRIB_WEAK /* !!!-- Attribute weak not defined --!!! */

	#define BLD_ATTRIB_ALIAS(__x__) !!!-- Attribute alias not defined --!!!

	#define BLD_ATTRIB_ASM_ALIAS(__symbol__) !!!-- Asm alias not defined --!!!

	#define BLD_ATTRIB_CONST

	#define BLD_ATTRIB_ALIGN(__align__) !!!-- Alignment alias not defined --!!!

#endif

// How to define weak aliases. Currently this is simply a mixture of the
// above

#define BLD_ATTRIB_WEAK_ALIAS(__symbol__) \
        BLD_ATTRIB_WEAK BLD_ATTRIB_ALIAS(__symbol__)

// -------------------------------------------------------------------------
// Various "flavours" of memory regions that can be described by the 
// Memory Layout Tool (MLT).

#define MEM_REGION_ATTR_R  0x01  // Region can be read
#define MEM_REGION_ATTR_W  0x02  // Region can be written

// -------------------------------------------------------------------------
#endif // ONCE_INFRA_TYPE_H multiple inclusion protection
// EOF type.h
