//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#define  _FLASH_PRIVATE_
#include "flash_io.h"
#include "hal_cache.h"
#include "hal_ixdp2400.h"

struct flash_info flash_info;
extern void eprintf(char *f, ...);
extern int flash_unlock_block(volatile unsigned char *block, int block_size, int blocks);
extern int flash_program_buf(volatile unsigned char *addr, unsigned char *data, int len);
extern int flash_erase_block(volatile unsigned char *block);
extern int flash_lock_block(volatile unsigned char *block);


int flash_init()
{
    int err;

	*(SP_FRM_REG) = USE_8_BIT_DATA;

	flash_info.init = 0;

    if (flash_info.init)
	{
		return FLASH_ERR_OK;
	}

    if ((err = flash_hwr_init()) != FLASH_ERR_OK)
	{
        return err;
    }

	flash_info.block_mask = ~(flash_info.block_size-1);
    flash_info.init = 1;
    return FLASH_ERR_OK;
}


int flash_erase(void *addr, int len, void **err_addr)
{
    unsigned char *block, *end_addr;
    int stat = 0;
//	int d_cache, i_cache;

    if (!flash_info.init)
	{
        return FLASH_ERR_NOT_INIT;
    }

    block = (unsigned char *)((UINT32)addr & flash_info.block_mask);
    end_addr = (unsigned char *)((UINT32)addr+len);

    eprintf("... Erase from 0x%X-0x%X: ", (void*)block, (void*)end_addr);

    HAL_FLASH_CACHES_OFF(d_cache, i_cache);
    FLASH_Enable(block, end_addr);
    while (block < end_addr)
	{
        // Supply the blocksize for a gross check for erase success
        stat = flash_erase_block((unsigned char*)block);//, flash_info.block_size);
        stat = flash_hwr_map_error(stat);
        if (stat)
		{
            *err_addr = (void *)block;
            break;
		}
        block += flash_info.block_size / sizeof(*block);
        eprintf(".");
    }
    
	FLASH_Disable(block, end_addr);
    HAL_FLASH_CACHES_ON(d_cache, i_cache);
    eprintf("\n");
    return (stat);
}


int flash_program(void *_addr, void *_data, int len, void **err_addr)
{
    int stat = 0;
    int size;
    unsigned char *addr = (unsigned char *)_addr;
    unsigned char *data = (unsigned char *)_data;
    UINT32 tmp;
//	int d_cache, i_cache;

    if (!flash_info.init)
	{
        return FLASH_ERR_NOT_INIT;
    }

    eprintf("... Program from 0x%X-0x%X at 0x%X: ", (void*)data, (void*)(((UINT32)data)+len), (void*)addr);

    HAL_FLASH_CACHES_OFF(d_cache, i_cache);
    FLASH_Enable((unsigned char*)addr, (unsigned char *)(addr + len));
    while (len > 0)
	{
        size = len;
        if (size > flash_info.block_size)
		{
			size = flash_info.block_size;
		}

        tmp = (UINT32)addr & ~flash_info.block_mask;
        if (tmp)
		{
			tmp = flash_info.block_size - tmp;
            if (size > tmp)
			{
				size = tmp;
			}
        }

		// flash can only be written in 8 bit mode.
//		*(SP_FRM_REG) = USE_8_BIT_DATA;
        stat = flash_program_buf(addr, data, size);//,flash_info.block_mask, flash_info.buffer_size);
		// restore 32 bit mode
//		*(SP_FRM_REG) = USE_32_BIT_DATA;

        stat = flash_hwr_map_error(stat);
        if (stat)
		{
            *err_addr = (void *)addr;
            break;
        }
        eprintf(".");
        len -= size;
        addr += size/sizeof(*addr);
        data += size/sizeof(*data);
    }
	
	FLASH_Disable((unsigned char*)addr, (unsigned char *)(addr+len));
	HAL_FLASH_CACHES_ON(d_cache, i_cache);
	eprintf("\n");
	return (stat);
}


int flash_lock(void *addr, int len, void **err_addr)
{
    unsigned char *block, *end_addr;
    int stat = 0;
//    int d_cache, i_cache;

    if (!flash_info.init)
	{
        return FLASH_ERR_NOT_INIT;
    }

    block = (unsigned char *)((UINT32)addr & flash_info.block_mask);
    end_addr = (unsigned char *)((UINT32)addr + len);

	eprintf("... Lock from 0x%X-0x%X: ", block, end_addr);

    HAL_FLASH_CACHES_OFF(d_cache, i_cache);
    FLASH_Enable(block, end_addr);
    while (block < end_addr)
	{
		stat = flash_lock_block((unsigned char*)block);
		stat = flash_hwr_map_error(stat);
		if(stat)
		{
			*err_addr = (void *)block;
			break;
		}
		block += flash_info.block_size / sizeof(*block);
        eprintf(".");
    }

    FLASH_Disable(block, end_addr);
    HAL_FLASH_CACHES_ON(d_cache, i_cache);
    eprintf("\n");
    return (stat);
}


int flash_unlock(void *addr, int len, void **err_addr)
{
    unsigned char *block, *end_addr;
    int stat = 0;
//    int d_cache, i_cache;

    if (!flash_info.init)
	{
        return FLASH_ERR_NOT_INIT;
    }

    block = (unsigned char *)((UINT32)addr & flash_info.block_mask);
    end_addr = (unsigned char *)((UINT32)addr+len);

    eprintf("... Unlock from 0x%X-0x%X: ", block, end_addr);

    HAL_FLASH_CACHES_OFF(d_cache, i_cache);
    FLASH_Enable(block, end_addr);
    while (block < end_addr)
	{
        stat = flash_unlock_block((unsigned char*)block, flash_info.block_size, flash_info.blocks);
        stat = flash_hwr_map_error(stat);
        if (stat)
		{
            *err_addr = (void *)block;
            break;
        }
		block += flash_info.block_size / sizeof(*block);
        eprintf(".");
    }
    
	FLASH_Disable(block, end_addr);
    HAL_FLASH_CACHES_ON(d_cache, i_cache);
    eprintf("\n");
    return (stat);
}


char *flash_errmsg(int err)
{
    switch (err) {
    case FLASH_ERR_OK:
        return "No error - operation complete\n";
    case FLASH_ERR_ERASE_SUSPEND:
        return "Device is in erase suspend state\n";
    case FLASH_ERR_PROGRAM_SUSPEND:
        return "Device is in program suspend state\n";
    case FLASH_ERR_INVALID:
        return "Invalid FLASH address\n";
    case FLASH_ERR_ERASE:
        return "Error trying to erase\n";
    case FLASH_ERR_LOCK:
        return "Error trying to lock/unlock\n";
    case FLASH_ERR_PROGRAM:
        return "Error trying to program\n";	
    case FLASH_ERR_PROTOCOL:
        return "Generic error\n";
    case FLASH_ERR_PROTECT:
        return "Device/region is write-protected\n";
    case FLASH_ERR_NOT_INIT:
        return "FLASH sub-system not initialized\n";
    case FLASH_ERR_DRV_VERIFY:
        return "Data verify failed after operation\n";
    case FLASH_ERR_DRV_TIMEOUT:
        return "Driver timed out waiting for device\n";
    case FLASH_ERR_DRV_WRONG_PART:
        return "Driver does not support device\n";
    case FLASH_ERR_LOW_VOLTAGE:
        return "Device reports low voltage\n";
    default:
        return "Unknown error\n";
    }
}


// EOF io/flash/..../flash.c
