//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#include "flash.h"
#include "hal_cache.h"
#include "common.h"

extern UINT32 silicon_rev;

//
// CAUTION!  This code must be copied to RAM before execution.  Therefore,
// it must not contain any code which might be position dependent!
//

int
flash_program_buf(volatile unsigned char *addr, unsigned char *data, int len)
{
    volatile unsigned char *ROM;
    volatile unsigned char *BA;
    unsigned char stat = 0;
    int timeout = 5000000;
    int i, wc, cache_on;

    HAL_DCACHE_IS_ENABLED(cache_on);
    if (cache_on)
	{
        HAL_DCACHE_SYNC();
        HAL_DCACHE_DISABLE();
    }

    ROM = (unsigned char *)FLASH_BASE;

	// do single byte writes.  Using the buffer never seemed to work.
    BA  = (unsigned char *)((unsigned long)addr & ~(FLASH_BLOCK_SIZE -1));

    // Clear any error conditions
    ROM[3] = FLASH_Clear_Status;

    wc = 32;
    while (len >= wc)
	{
        len -= wc;

        BA[3] = FLASH_Write_Buffer;
        timeout = 5000000;
        while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready)
		{
            if (--timeout == 0)
			{
                stat |= 0x0100;
                goto bad;
            }
            BA[3] = FLASH_Write_Buffer;
        }
        BA[3] = wc-1;  // Count is 0..N-1

		// writing to reverse byte order is not working. So, try writing in order
        if(((silicon_rev & 0xF0) >> 4))
        {
            // B0 rev. onwards silicon have slowport write endianness fix.
            for (i = 0;  i < wc;  i++)
			    addr[i] = data[(i^3)];
        }
        else
        {
	        for (i = 0;  i < wc;  i++)
			    addr[(i^3)] = data[i];
        }
		addr += 32;
		data += 32;

	    BA[3] = FLASH_Confirm;

		ROM[3] = FLASH_Read_Status;
		timeout = 5000000;
		while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready)
		{
			if (--timeout == 0)
			{
				eprintf("stat = 0x%02x\n", stat);
				stat |= 0x0200;
				goto bad;
			}
		}
    }
    
    while (len > 0)
	{
        ROM[3] = FLASH_Program;

        if(((silicon_rev & 0xF0) >> 4))
        {
            // B0 rev. onwards silicon have slowport write endianness fix.
            *addr = *((unsigned char *)((int)data^3));
        }
        else
        {
	        *((unsigned char *)((int)addr^3)) = *data;
        }
		data++;
		addr++;

        timeout = 5000000;
        while(((stat = ROM[0]) & FLASH_Status_Ready) != FLASH_Status_Ready)
		{
            if (--timeout == 0)
			{
                stat |= 0x0300;
                goto bad;
            }
        }
        --len;
    }

    // Restore ROM to "normal" mode
 bad:
    ROM[3] = FLASH_Reset;            

    if (cache_on)
	{
        HAL_DCACHE_ENABLE();
    }

    return stat;
}
