//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#include "hal_intr.h"           // interrupt macros
#include "hal_arm_ixdp2400.h"
#include "cfg_utility.h"
#include "misc_func.h"
#include "uart.h"

externC UINT32 strap_options_val;
extern struct board_config board_config_data;
extern int serial_osc;

//=============================================================================
// Serial driver
//=============================================================================

//-----------------------------------------------------------------------------
// There is one serial port
#define DEV_SERIAL_BASE_A    0xC0030000 // port A

//-----------------------------------------------------------------------------
// Define the serial registers.
#define DEV_SERIAL_RBR   0x00  // receiver buffer register, read, dlab = 0
#define DEV_SERIAL_THR   0x00 // transmitter holding register, write, dlab = 0
#define DEV_SERIAL_DLL   0x00 // divisor latch (LS), read/write, dlab = 1
#define DEV_SERIAL_IER   0x04 // interrupt enable register, read/write, dlab = 0
#define DEV_SERIAL_DLM   0x04 // divisor latch (MS), read/write, dlab = 1
#define DEV_SERIAL_IIR   0x08 // interrupt identification register, read, dlab = 0
#define DEV_SERIAL_FCR   0x08 // fifo control register, write, dlab = 0
#define DEV_SERIAL_LCR   0x0C // line control register, write
#define DEV_SERIAL_LSR   0x14 // line status register, read
#define DEV_SERIAL_SCR   0x1C // scratch pad register

// The interrupt enable register bits.
#define SIO_IER_ERDAI   0x01            // enable received data available irq
#define SIO_IER_ETHREI  0x02            // enable THR empty interrupt
#define SIO_IER_ELSI    0x04            // enable receiver line status irq
#define SIO_IER_RTOIE	0x10			// enable receive timeout int.
#define SIO_IER_UUE		0x40			// enable uart unit

// The interrupt identification register bits.
#define SIO_IIR_IP      0x01            // 0 if interrupt pending
#define SIO_IIR_ID_MASK 0x0e            // mask for interrupt ID bits
#define ISR_Tx  0x02
#define ISR_Rx  0x04

// The line status register bits.
#define SIO_LSR_DR      0x01            // data ready
#define SIO_LSR_OE      0x02            // overrun error
#define SIO_LSR_PE      0x04            // parity error
#define SIO_LSR_FE      0x08            // framing error
#define SIO_LSR_BI      0x10            // break interrupt
#define SIO_LSR_THRE    0x20            // transmitter holding register empty
#define SIO_LSR_TEMT    0x40            // transmitter register empty
#define SIO_LSR_ERR     0x80            // any error condition

// The line control register bits.
#define SIO_LCR_WLS0   0x01             // word length select bit 0
#define SIO_LCR_WLS1   0x02             // word length select bit 1
#define SIO_LCR_STB    0x04             // number of stop bits
#define SIO_LCR_PEN    0x08             // parity enable
#define SIO_LCR_EPS    0x10             // even parity select
#define SIO_LCR_SP     0x20             // stick parity
#define SIO_LCR_SB     0x40             // set break
#define SIO_LCR_DLAB   0x80             // divisor latch access bit

// The FIFO control register
#define SIO_FCR_FCR0   0x01             // enable xmit and rcvr fifos
#define SIO_FCR_FCR1   0x02             // clear RCVR FIFO
#define SIO_FCR_FCR2   0x04             // clear XMIT FIFO


//-----------------------------------------------------------------------------
typedef struct {
    UINT8* base;
    S32 msec_timeout;
    int isr_vector;
} channel_data_t;

static channel_data_t channels[1] = {
    { (UINT8*)DEV_SERIAL_BASE_A, 1000, HAL_INTERRUPT_SERIAL},
};

//-----------------------------------------------------------------------------
static void
init_serial_channel(channel_data_t* __ch_data)
{
    UINT8* base = __ch_data->base;
    UINT8 lcr;
	unsigned int baudrate, divisor;

	baudrate = DEFAULT_BAUD;

	// calculate the divisor based on 50 MHz system clock
	divisor = serial_osc / (16 * baudrate);
	
    // 8-1-no parity.
    lcr = SIO_LCR_WLS0 | SIO_LCR_WLS1;

    lcr |= SIO_LCR_DLAB;
    HAL_WRITE_UINT8(base+DEV_SERIAL_LCR, lcr);
    HAL_WRITE_UINT8(base+DEV_SERIAL_DLL, divisor);
    HAL_WRITE_UINT8(base+DEV_SERIAL_DLM, divisor >> 8);
    lcr &= ~SIO_LCR_DLAB;
    HAL_WRITE_UINT8(base+DEV_SERIAL_LCR, lcr);
	HAL_WRITE_UINT8(base+DEV_SERIAL_IER, 0x0);
    HAL_WRITE_UINT8(base+DEV_SERIAL_FCR, 0x07); // Enable & clear FIFO
	HAL_WRITE_UINT8(base+DEV_SERIAL_IER, 0x40);	// Enable UART
}

void
hal_plf_comms_init()
{
    static int initialized = 0;

    if (initialized)
        return;

    initialized = 1;

	// Disable interrupts.
    HAL_INTERRUPT_MASK(channels[0].isr_vector);

	// Init channels
    init_serial_channel(&channels[0]);//hal_plf_serial_init();
}


/*---------------------------------------------------------------------------*/
/* End of hal_serial.c */
