//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#define  _FLASH_PRIVATE_
#include "flash_io.h"
#include "hal_cache.h"
#include "flash.h"
#include "misc_func.h"

extern void eprintf(char *f, ...);
extern int flash_query(unsigned char *data);

#define _si(p) ((p[1]<<8)|p[0])

#ifndef CYGHWR_IO_FLASH_DEVICE_NOT_IN_RAM
// Use this function to make function pointers anonymous - forcing the
// compiler to use jumps instead of branches when calling driver
// services.
static void* __anonymizer(void* p)
{
  return p;
}
#endif

int flash_hwr_init(void)
{
    unsigned char data[2];
    typedef int code_fun(unsigned char *);
    code_fun *_flash_query;
    int stat, num_regions, region_size;

#ifdef CYGHWR_IO_FLASH_DEVICE_NOT_IN_RAM
	{
		extern char flash_query, flash_query_end;
		int code_len;

		// Copy 'program' code to RAM for execution
		code_len = (unsigned long)&flash_query_end - (unsigned long)&flash_query;
		_flash_query = (code_fun *)flash_info.work_space;
		memcpy(_flash_query, &flash_query, code_len);
		HAL_DCACHE_SYNC();  // Should guarantee this code will run
		HAL_ICACHE_DISABLE(); // is also required to avoid old contents
	}
#else
	{
        externC code_fun flash_query;
        _flash_query = (code_fun*) __anonymizer(&flash_query);
   }
#endif

    memset(data,0,2);
    stat = (*_flash_query)((void*)&data);
    HAL_ICACHE_ENABLE();

    if(data[0] != FLASH_Intel_code) 
	{
		eprintf("Not Intel flash = %x\n", data[0]);
		return FLASH_ERR_HWR;
	}
	
	switch (data[1])
	{
    case (unsigned char)0x18:
		eprintf("16M flash found, 128 regions\n");
        num_regions = 128;
        region_size = 0x20000;    //128K
        break;
    case (unsigned char)0x17:
		eprintf("8M flash found, 64 regions\n");
        num_regions = 64;
        region_size = 0x20000;
        break;
    case (unsigned char)0x16:
		eprintf("4M flash found, 32 regions\n");
        num_regions = 32;
        region_size = 0x20000;
        break;
    default:
		eprintf("Unknown primary device type: %x\n", data[1]);
        return FLASH_ERR_HWR;
	}
	flash_info.block_size = region_size;
	flash_info.blocks = num_regions;
	flash_info.start = (void *)FLASH_BASE;
	flash_info.end = (void *)(FLASH_BASE + (num_regions * region_size));
	return FLASH_ERR_OK;
}

// Map a hardware status to a package error
int
flash_hwr_map_error(int err)
{
    if (err & 0x7E)
	{
        eprintf("Err = %x\n", err);
        if (err & 0x10)
		{
            return FLASH_ERR_PROGRAM;
        }
		else if (err & 0x20)
		{
            return FLASH_ERR_ERASE;
        }
		else 
        	return FLASH_ERR_HWR;
    }
	else
	{
        return FLASH_ERR_OK;
    }
}
