//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#include "hal_arm_ixdp2400.h"
#include "mlt_arm_ixdp2400_romram.h"
#include "hal_intr.h"           // Interrupt names
#include "cfg_utility.h"
#include "i2c.h"	//</io/i2c.h>
#include "hal_ixdp2400.h"
#include "pci.h"
#include "common.h"
#include "led.h"
#include "diag_utils.h"

#define NPU_SPEED (600 * 1000000)	// in hz
UINT32 cpld_rev = 0, board_rev = 0, serial_osc = 0, silicon_rev = 0;

extern void eprintf(char *f, ...);
extern void install_other_isr(void);
extern void hal_plf_comms_init(void);

UINT32 pciIntList[PCI_ISR_COUNT];		/* list of int handlers */
UINT32 pciIntDataList[PCI_ISR_COUNT];	/* list of data objects */

externC UINT32 strap_options_val, boot_from_boot_monitor;
externC pci_device slave_dev_info;
UINT32 sram_size[2] = {0, 0};
UINT32 sram_chan[2] = {0, 0};
struct board_config board_config_data;

void pciInt(void)
{
	int i;
	pci_ISR *isr;
	unsigned char cpld_int_val;

	for(i = 0; i < PCI_ISR_COUNT; i++)
	{
		if(strap_options_val & CFG_PCI_BOOT_HOST)
		{// master has more than one PCI interrupt
            cpld_int_val = *(CPLD_INT_REG);
			if((pciIntList[i] != (UINT32)0) && (cpld_int_val & (1 << i)))
			{
				isr = (pci_ISR *)pciIntList[i];
				(*isr)(i, pciIntDataList[i]);
			}
		}
		else	// slave only has NIC interrupt
		{
			if((pciIntList[i] != (UINT32)0))
			{
				isr = (pci_ISR *)pciIntList[i];
				(*isr)(i, pciIntDataList[i]);
			}
		}
	}
}

void pciIntConnect(int vector, UINT32 routine, UINT32 data)
{
	if(pciIntList[vector] == 0)
	{
		pciIntList[vector]     = routine;
		pciIntDataList[vector] = data;
        if(strap_options_val & CFG_PCI_BOOT_HOST)
            PUT32(CPLD_INT_MASK_REG, (GET32(CPLD_INT_MASK_REG) & ~(1<<vector)));
        else
            hal_interrupt_unmask(HAL_INTERRUPT_PCI_INTB);;
	}
	else
	{
        if(pciIntList[vector] != routine)
        {
		    eprintf("***Warning***: PCI interrupt handler changed for vector 0x%x without disconnecting\n",
				vector);
        }
	}
}

void pciIntDisconnect(int vector)
{
	pciIntList[vector]     = 0;
	pciIntDataList[vector] = (UINT32)NULL;
    if(strap_options_val & CFG_PCI_BOOT_HOST)
        PUT32(CPLD_INT_MASK_REG, (GET32(CPLD_INT_MASK_REG) | (1<<vector)));
    else
        hal_interrupt_mask(HAL_INTERRUPT_PCI_INTB);
}

void hal_sram_init(int channel, int skip_csr, int frm_pci, int offset)
{
	int i, j, temp, dcache_status;
	int qdr_ch_size = (1024*1024), qdr_ch_csr_base, qdr_base, sram_csr_val, sram_detected = ERROR;
	volatile UINT32 temp1, temp2;

	Set_LED("SRAM");
	
	// calculate the csr base and qdr base
	if(frm_pci)
	{
		qdr_ch_csr_base = slave_dev_info.base_map[CSR_BAR] + QDR_CH_BASE_FRM_PCI + (channel * 0x400);
		qdr_base = slave_dev_info.base_map[SRAM_BAR] + (channel * 0x10000000);
	}
	else
	{
		qdr_ch_csr_base = QDR_CH_CSR_BASE + (channel * 0x400000);
		qdr_base = SRAM_BASE + (channel * 0x10000000);
	}

	// init csrs
	if(!skip_csr)
	{
		unsigned int addr;

		// rcomp registers are only 4 bytes distant. So, when this function will be called by pci init it won't
		// work properly due to pci bug. Insert a dummy read cycle in between write operations.
		for(addr = 0x340; addr <= 0x3bc; addr += 4)
		{
			*((volatile UINT32 *) (qdr_ch_csr_base + addr)) = 0xcccc;
#ifdef A0_REV
			temp = *((volatile UINT32 *) (qdr_ch_csr_base + addr));
#endif
		}
		// Invert rcomp polarity
		*((volatile UINT32 *)(qdr_ch_csr_base + 0x300)) = 0x00010060;
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + 0x300));

#ifdef QDR1
		// for 50 ohm ref resistor in real board/ tester
		*((volatile UINT32 *)(qdr_ch_csr_base + 0x318)) = 0xffff3;
#ifdef A0_REV
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + 0x318));
#endif
		// wait for 1ms
		hal_delay_us(1000);
		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_RX_DLL_OFF)) = 0x48;
#else
		// for 50 ohm ref resistor in real board/ tester
		*((volatile UINT32 *)(qdr_ch_csr_base + 0x318)) = 0x518c3;//0xffff3;
#ifdef A0_REV
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + 0x318));
#endif
		// wait for 1ms
		hal_delay_us(1000);
		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_RX_DLL_OFF)) = 0x52;
#endif
#ifdef A0_REV
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + QDR_RX_DLL_OFF));
#endif

		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_RX_DESKEW_OFF)) = 0x12;
#ifdef A0_REV
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + QDR_RX_DESKEW_OFF));
#endif

		// set sram csr to use max sram size
		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF)) = MAX_SRAM_SIZE_CSR_VAL;
#ifdef A0_REV
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF));
#endif
		hal_delay_us(3000);

		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_RD_PTR_OFF)) = 0x4;		
		// read rd_ptr_off back to ensure that write is completed
		temp = *((volatile UINT32 *)(qdr_ch_csr_base + QDR_RD_PTR_OFF));
	}

	if(!frm_pci)
	{
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}
		for(j = 1; j <= 5; j++)
		{
			*((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF)) = PIPELINE | (j << 7);
			temp = *((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF));
			hal_delay_us(10000);

			for(i = 0; i < SRAM_SIZE_LOOPS; i++)
			{
				#ifdef DEBUG_SIZING
					eprintf("qdr_ch_size: %d\n", qdr_ch_size);
				#endif

				PUT32(qdr_base + qdr_ch_size + 256, COMPARISON_PAT);
				temp1 = GET32(qdr_base + qdr_ch_size + 256);
				if(temp1 != COMPARISON_PAT)
				{
					#ifdef DEBUG_SIZING
						eprintf("pattern mis-match: %x\n", temp1);
					#endif
					break;
				}

				PUT32(qdr_base + 256, 0xA5A5A5A5);
				PUT32(qdr_base + 256 + (qdr_ch_size/2), 0x55AAAA55);

				temp1 = GET32(qdr_base + 256);
				temp2 = GET32(qdr_base + 256 + (qdr_ch_size/2));
				if(temp1 == temp2)
				{
					#ifdef DEBUG_SIZING
						eprintf("mirror-ing: 0x%x, 0x%x\n", temp1, temp2);
					#endif
					break;
				}
				sram_detected = OK;
				qdr_ch_size = qdr_ch_size * 2;
			}
		}
		if(dcache_status != 0)	// if dcache ON
			dcache_on();

		if(sram_detected == OK)
		{
			eprintf("SRAM Channel %d: 0x%08x - 0x%08x\n", channel, qdr_base, (qdr_base + qdr_ch_size));
			sram_chan[channel] = 0x1;
		}
		else
		{
			eprintf("No SRAM found on channel %d\n", channel);
			qdr_ch_size = 0;
		}

		//save sram size
		sram_size[channel] = qdr_ch_size;
		
		temp = 1024 * 1024;
		for(i = 0; i < 6; i++)
		{
			temp = temp * 2;
			if(temp == qdr_ch_size)
				break;
		}
		sram_csr_val = i << 7;
		sram_csr_val |= PIPELINE;
		
		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF)) = sram_csr_val;
	}

	if(!frm_pci && qdr_ch_size)
	{		
		// initialize whole SRAM with 0 so that it doesn't generate parity errors
		memset((char *)(qdr_base + offset), 0, (qdr_ch_size - offset));
		*((volatile UINT32 *)(qdr_ch_csr_base + QDR_CH_CONTROL_OFF)) |= PARITY_ENABLE;
	}
}

// Some initialization has already been done before we get here.
//
// Set up the interrupt environment.
// Set up the MMU so that we can use caches.
// Enable caches.
// - All done!

void Calc_DRAM_Size(void)
{
	unsigned char num_rows[5] = "", density_rows[5] = "";
	unsigned int num_of_rows, density_of_rows;
	int i;
	
	// Read the DRAM info from DRAM I2C PROM and calculate the size of total DRAM memory
	
	i2c_read(SLAVE_ADDR_0, 0x5, num_rows);		// # module rows
	i2c_read(SLAVE_ADDR_0, 0x1F, density_rows);	//density of each row on module

	num_of_rows = (int)num_rows[0];
	density_of_rows = (int)density_rows[0];

	// check the density of one row
	for(i = 0; i < 8; i++)	// 8 bits to be checked
	{
		if( (density_of_rows & (0x1<<i)) != 0)	// checking each bit
		{
			hal_dram_size = (1 << (22 + i));	// starts at 4Mb (2^22)
			break;		// do not need to check further since there's only 1 row
		}
	}
	hal_dram_size *= num_of_rows;
}

void hal_hardware_init(void)
{
	int i;

	// Let the timer run at a default rate (for delays)
    hal_clock_initialize(HAL_RTC_PERIOD);

	// If slave boot from master, then copy configurations
	if(!(strap_options_val & CFG_PCI_BOOT_HOST))
	{
		PUINT s = (PUINT)&board_config_data;

		Set_LED("CFG ");

		for(i = 0; i < sizeof(struct board_config); i += 4)
		{
			s[i/4] = *((volatile UINT32 *)(SCRATCHPAD_BASE + i));
		}
	}

	Set_LED("OSC ");
	cpld_rev = *(CPLD_REV_REG);
	board_rev = (cpld_rev & 0xF0) >> 4;
	if(board_rev < 4)
	{
		serial_osc = NPU_SPEED / 12;
	}
	else
	{
		int M_clk = 0;
		int N_clk, temp;
		int denom_array[] = {2, 4, 8, 16, 1, 2, 4, 8};

		temp = *(SYS_CLK_M_REG);
		M_clk = (temp & (0xFF)) * 2;

		temp = *(SYS_CLK_N_REG);
		N_clk = denom_array[(temp & 0x7)];

		serial_osc = ((3125000 * M_clk)/N_clk)/2;
	}

	// if this is master npu then read config data
	if(strap_options_val & CFG_PCI_BOOT_HOST)
	{
		Set_LED("CFG ");
		// read config data
		i2c_seq_read(SLAVE_ADDR_1, 0, (unsigned char *)&board_config_data.config_data, CONFIG_DATA_SIZE);
		if(board_config_data.config_data.prom_fmt == '0')
		{
			// save in scratch for later reference
			// remember scratch is not byte writable
			board_config_data.sdram_size = hal_dram_size;
			// set the valid flag
			board_config_data.config_valid = CONFIG_DATA_VALID;
		}
	}
	else
	{
		// if slave has flash then init valid flag with 0 so that we will know later
		// whether master has copied config data or not
		if(!boot_from_boot_monitor)
		{
			if(strap_options_val & CFG_PROM_BOOT)
			{
				board_config_data.config_valid = 0;
			}
		}
	}

	Set_LED("UART");
	// Set up eCos/ROM interfaces
	hal_plf_comms_init();

    silicon_rev = *(PRODUCT_ID_REG);
    {
        char *min_product_type[] = {"IXP2800", "IXP2800 with Crypto", "IXP2400"};
        char *maj_rev[] = {"A1", "B0"};

        eprintf("Found %s Rev. %s\n", min_product_type[((silicon_rev & 0xFF00) >> 8)], maj_rev[((silicon_rev & 0xF0) >> 4)]);
    }

	Set_LED("SRAM");
	// init qdr channels
	for(i = 0; i < MAX_QDR_CHANNEL; i++)
	{
		hal_sram_init(i, 0, 0, 0);
	}

    // Now take MSF out of reset
    *(IXP_RESET0_REG) &= ~(0x80);

	// Calculate DRAM size
	if(boot_from_boot_monitor)
	{
		Set_LED("DRAM");
		Calc_DRAM_Size();
	}

	Set_LED("INTa");
	// init pci int and data list
	for (i = 0; i < PCI_ISR_COUNT; i++)
	{
		pciIntList[i]     = 0;
		pciIntDataList[i] = (UINT32)NULL;
	}

	// attach and enable pci intb
    HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_PCI_INTB, &pciInt,
						HAL_INTERRUPT_PCI_INTB, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_PCI_INTB);

	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_TIMER_1, &hal_clock_reset,
						 HAL_INTERRUPT_TIMER_1, 0);

    install_other_isr();
}

static UINT32 _period;

void hal_clock_initialize(UINT32 period)
{
    _period = period;

	// I am using timer 1 at 50 MHZ.
	// load the counter
	*(T1_CLD_REG) = period;

	// start the timer
	*(T1_CTL_REG) = T1_CTL_VAL;
}

// This routine is called during a clock interrupt.

void hal_clock_reset(UINT32 vector, UINT32 period)
{
    // clear the timer interrupt
    *(T1_CLR_REG) = T1_CLR_VAL;
}

// Read the current value of the clock, returning the number of hardware
// "ticks" that have occurred (i.e. how far away the current value is from
// the start)

void hal_clock_read(UINT32 *pvalue)
{
	// read the current counter value
    *pvalue = *(T1_CSR_REG);
}

// reset the system
void ixdp2400_reset()
{
	// only master can reset the system
	if(strap_options_val & CFG_PCI_BOOT_HOST)
	{
		*(IXP_RESET0_REG) |= RSTALL;
	}
	else
	{
		eprintf("Only master can reset the system\n");
	}
}

// Delay for some usecs.
void hal_delay_us(UINT32 delay)
{
    UINT32 now, last, diff, ticks;

    hal_clock_read(&last);
    diff = ticks = 0;

    while (delay > ticks) {
	hal_clock_read(&now);

	if (now > last)
	    diff += ((_period - now) + last);
	else
	    diff += (last - now);

	last = now;

	if (diff >= _TICKS_PER_USEC) {
	    ticks += (diff / _TICKS_PER_USEC);
	    diff %= _TICKS_PER_USEC;
	}
    }
}

// -------------------------------------------------------------------------

typedef UINT32 ISR(UINT32 vector, UINT32 data);

void hal_interrupt_mask_all(void)
{
    unsigned rtmp = 0;

    // disable IRQ
    asm volatile("mrs %0,cpsr\n"
        "orr %0,%0,#0x80\n"
        "msr cpsr,%0\n"
        : "=r"(rtmp) : );
}

void hal_interrupt_unmask_all(void)
{
    unsigned rtmp = 0;

    // enable IRQ
    asm volatile("mrs %0,cpsr\n"
        "bic %0,%0,#0x80\n"
        "msr cpsr,%0\n"
        : "=r"(rtmp) : );
}

// This routine is called to respond to a hardware interrupt (IRQ).  It
// should interrogate the hardware and return the IRQ vector number.
int hal_IRQ_handler(void)
{
	UINT32 reg_val;
	UINT32 i;

	// check the irq status register
	reg_val = *(IRQ_STATUS_REG);
	if(reg_val != INT_REG_EMPTY)
	{
		for(i = 0; i < 32; i++)
		{
			// if its a external pci int. then I need to resolve xscale int status reg.
			if((reg_val & (1 << i)) && (i != (HAL_INTERRUPT_PCI - 1)))
			{
				// return the vector
				return (i + 1);
			}
		}
	}

	reg_val = *(FIQ_STATUS_REG);
	if(reg_val != INT_REG_EMPTY)
	{
		for(i = 0; i < 32; i++)
		{
			// if its a external pci int. then I need to resolve xscale int status reg.
			if((reg_val & (1 << i)) && (i != (HAL_INTERRUPT_PCI - 1)))
			{
				// return the vector
				return (i + 1);
			}
		}
	}

	// resolve xscale interrupt status register
	reg_val = *(XSCALE_INT_STATUS_REG);
	// silicon team told me that this is raw status. so AND it with enable register.
	reg_val = reg_val & *(XSCALE_INT_ENABLE_REG);
	if(reg_val != INT_REG_EMPTY)
	{
		for(i = 0; i < 32; i++)
		{
			if(reg_val & (1 << i))
			{
				// add the PCI_INTERRUPT_BASE and return the vector
				return (PCI_INTERRUPT_BASE + i + 1);
			}
		}
	}
    return HAL_INTERRUPT_NONE; // This shouldn't happen!
}

//
// Interrupt control
//

void hal_interrupt_mask(int vector)
{
	if((vector <= HAL_ISR_MIN) || (vector > HAL_ISR_MAX))
	{
		// do nothing
		return;
	}
	if(vector <= PCI_INTERRUPT_BASE)
	{
		// mask the appropriate bit in irq enable register.
		*(IRQ_ENABLE_CLR_REG) = (1 << (vector - 1));
	}
	else
	{
		// mask the appropriate bit in xscale int enable register
		*(XSCALE_INT_ENABLE_REG) &= ~(1 << (vector - (PCI_INTERRUPT_BASE + 1)));
	}
	return;
}

// for unmasking dma int. unmask it at both xscale and pci side otherwise it won't work.
void hal_interrupt_unmask(int vector)
{
	if((vector <= HAL_ISR_MIN) || (vector > HAL_ISR_MAX))
	{
		// do nothing
		return;
	}
	if(vector <= PCI_INTERRUPT_BASE)
	{
		// unmask the appropriate bit in irq enable register.
		*(IRQ_ENABLE_SET_REG) = (1 << (vector - 1));
	}
	else
	{
		// enable PCI int. in irq enable register
		*(IRQ_ENABLE_SET_REG) = (1 << (HAL_INTERRUPT_PCI - 1));

		// unmask the appropriate bit in xscale interrupt enable register.
		*(XSCALE_INT_ENABLE_REG) |= (1 << (vector - (PCI_INTERRUPT_BASE + 1)));
	}
	return;
}

/*------------------------------------------------------------------------*/
// EOF ixdp2400_misc.c
