/* ixdp2400_pci.c
---------------------------------------------------------------------------
                 Copyright (c) 2002, 2003 Intel Corporation
						 All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer. 
* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 
* Neither the name of Intel Corporation nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                      
---------------------------------------------------------------------------
system: IXP2400
subsystem: DIAG
author: lagarwal, Feb 4, 02
revisions:
--------------------------------------------------------------------------
*/

#include "hal_ixdp2400.h"
#include "hal_arm_ixdp2400.h"
#include "mlt_arm_ixdp2400_romram.h"

#include "pci.h"
#include "hal_io.h"
#include "hal_cache.h"
#include "cfg_utility.h"

extern void hal_sram_init(int channel, int skip_csr, int frm_pci, int offset);

externC UINT32	strap_options_val, board_rev, boot_from_boot_monitor, untouched_diag_location, serial_osc;
pci_device	master_dev_info, slave_dev_info;	// for master & slave npu info.
extern struct board_config board_config_data;

extern void eprintf(char *f, ...);

extern char __rom_data_start[];
extern char __rom_data_end[];
extern char __ram_data_start[];

void slave_ixp_init(void)
{
	int i, j, temp;
	int slave_sdram_passed = 1;
	UINT32 rom_data_start = (UINT32)__rom_data_start;
	UINT32 rom_data_end = (UINT32)__rom_data_end;
	UINT32 ram_data_start = (UINT32)__ram_data_start;

	/* PCI unit does a byte swapping by default. But we don't want to swap bytes when copying into slave's memory.
	So set these bits so that PCI unit doesn't do byte swapping. */
	*(PCI_CONTROL_REG) |= (PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);

	// SDRAM, SRAM and CSR bars are mapped in master's pci mem space.
	// first BAR is CSR, second is SRAM and third is SDRAM
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_CONTROL_FRM_PCI)) |= (PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);

	// check to see if slave has flash or not
	if((*(volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + STRAP_OPTIONS_FRM_PCI) & CFG_PROM_BOOT))
	{
		eprintf("Slave NPU has flash\n");
	}
	else
	{
		// slave doesn't have flash. So, start slave init
		eprintf("Single flash system detected.\n");
		
		if(!boot_from_boot_monitor)
		{
			// enable outbound pci int. from xscale
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_OUT_INT_MASK_FRM_PCI)) &= XSIM;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_OUT_INT_MASK_FRM_PCI));
#endif
			
			// set the upper bits for pci mem and io transactions
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_ADDR_EXT_FRM_PCI)) = 0x0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_ADDR_EXT_FRM_PCI));
#endif
			
			// slave pci init is complete. So start accepting pci transactions
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI)) |= INIT_COMP;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI));
#endif
			
			// now I need to config SDRAM/SRAM also. So, that I can download image in SDRAM.
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RCOMP_IO_CONFIG_FRM_PCI)) = DDR_RCOMP_IO_CONFIG_VAL;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RCOMP_IO_CONFIG_FRM_PCI));
#endif
			
			// setup operating freq. for SDRAM/SRAM interface
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI));
			temp &= ~(0xF00FF);
#ifdef A0_REV
			if(serial_osc == 50000000)
			{
				temp |= ((SDRAM_DIVISOR << 16) | (SRAM_150MHZ_DIVISOR << 4) | SRAM_150MHZ_DIVISOR);
			}
			else
			{
				temp |= ((SDRAM_DIVISOR << 16) | (SRAM_200MHZ_DIVISOR << 4) | SRAM_200MHZ_DIVISOR);
			}
#else
			temp |= ((SDRAM_DIVISOR << 16) | (SRAM_200MHZ_DIVISOR << 4) | SRAM_200MHZ_DIVISOR);
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI)) = temp;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CCR_FRM_PCI));
#endif
			hal_delay_us(1000); // 1ms
			
			// now config SDRAM interface
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RDDLYSEL_RECEN_FRM_PCI)) = DDR_RDDLYSEL_RECEN_VAL;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RDDLYSEL_RECEN_FRM_PCI));
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DLL_FRM_PCI)) = DDR_RX_DLL_VAL;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DLL_FRM_PCI));
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DESKEW_FRM_PCI)) = DDR_RX_DESKEW_VAL;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DDR_RX_DESKEW_FRM_PCI));
#endif
			
			// cr0_dstrengthsel
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DSTRENGTHSEL_FRM_PCI)) = 0x036db6db;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DSTRENGTHSEL_FRM_PCI));
#endif
			
			// cr0_ddqrcomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DDQRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DDQRCOMP_FRM_PCI));
#endif
			
			// cr0_dctlrcomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCTLRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCTLRCOMP_FRM_PCI));
#endif
			
			// cr0_drcvrcomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DRCVRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DRCVRCOMP_FRM_PCI));
#endif
			
			// cr0_dckercomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKERCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKERCOMP_FRM_PCI));
#endif
			
			// cr0_dcsrcomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCSRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCSRCOMP_FRM_PCI));
#endif
			
			// cr0_dckrcomp
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKRCOMP_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DCKRCOMP_FRM_PCI));
#endif
			
			// cr0_dx8x16ckecscksel
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DX8X16CKECSCKSEL_FRM_PCI)) = 0x11111111;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DX8X16CKECSCKSEL_FRM_PCI));
#endif
			
			// cr0_rcompprd
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_RCOMPPRD_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_RCOMPPRD_FRM_PCI));
#endif
			
			// cr0_digfil
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DIGFIL_FRM_PCI)) = 0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_DIGFIL_FRM_PCI));
#endif
			
			/* Program DDR DQ/DQS pull-up and pull-down Slew Lookup table registers */
			/* (cr0_ddqpslew0 - cr0_ddqpslew3) and (cr0_ddqnslew0 - cr0_ddqnslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x150); i <= (DRAM_CH0_BASE_FRM_PCI + 0x188); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program MA/BA/RAS#/CAS#/WE# pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dctlpslew0 - cr0_dctlpslew3) and (cr0_dctlnslew0 - cr0_dctlnslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x198); i <= (DRAM_CH0_BASE_FRM_PCI + 0x1D0); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program RCV pull-up and pull-down Slew Lookup table registers */
			/* (cr0_drcvpslew0 - cr0_drcvpslew3) and (cr0_drcvnslew0 - cr0_drcvnslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x1E0); i <= (DRAM_CH0_BASE_FRM_PCI + 0x218); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CKE x8 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dckex8pslew0 - cr0_dckex8pslew3) and (cr0_dckex8nslew0 - cr0_dckex8nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x230); i <= (DRAM_CH0_BASE_FRM_PCI + 0x268); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CKE x16 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dckex16pslew0 - cr0_dckex16pslew3) and (cr0_dckex16nslew0 - cr0_dckex16nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x270); i <= (DRAM_CH0_BASE_FRM_PCI + 0x2A8); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CS# x8 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dcsx8pslew0 - cr0_dcsx8pslew3) and (cr0_dcsx8nslew0 - cr0_dcsx8nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x2B8); i <= (DRAM_CH0_BASE_FRM_PCI + 0x2F0); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CS# x16 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dcsx16pslew0 - cr0_dcsx16pslew3) and (cr0_dcsx16nslew0 - cr0_dcsx16nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x2F8); i <= (DRAM_CH0_BASE_FRM_PCI + 0x330); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CK CK# x8 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dckx8pslew0 - cr0_dckx8pslew3) and (cr0_dckx8nslew0 - cr0_dckx8nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x340); i <= (DRAM_CH0_BASE_FRM_PCI + 0x378); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Program CK CK# x16 pull-up and pull-down Slew Lookup table registers */
			/* (cr0_dckx16pslew0 - cr0_dckx16pslew3) and (cr0_dckx16nslew0 - cr0_dckx16nslew3) */
			for(i = (DRAM_CH0_BASE_FRM_PCI + 0x380); i <= (DRAM_CH0_BASE_FRM_PCI + 0x3B8); i += 8)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i)) = 0xcccccccc;
#ifdef A0_REV
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + i));
#endif
			}
			
			/* Flag Slew Programmed register as done */
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_SLEWPROGRAMMED_FRM_PCI)) = 0x1;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_SLEWPROGRAMMED_FRM_PCI));
#endif
			
			/* Force SM Rcomp */
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI)) = 0x1;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI));
#endif
			hal_delay_us(1000);
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI)) = 0x0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_FRCSMRCOMP_FRM_PCI));
#endif
			
			/* cr0_ovrrideh: turn off h override, 0x0 */
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEH_FRM_PCI)) = 0x0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEH_FRM_PCI));
#endif
			
			/* cr0_ovrridev: turn off v override, 0x0 */
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEV_FRM_PCI)) = 0x0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_OVRRIDEV_FRM_PCI));
#endif
			
			/* De-Select Test Mode (cr0_jt_config)
			// User Overide RCOMP settings */
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_JT_CONFIG_FRM_PCI)) = 0x0;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + CR0_JT_CONFIG_FRM_PCI));
#endif
			hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_CONTROL_FRM_PCI)) = *DU_CONTROL_REG;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_CONTROL_FRM_PCI));
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = CKE;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI));
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & PRECHARGE)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | EXT_LD_MODE);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | RESET_DLL);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (PRECHARGE | SIDE1 | SIDE0 | PRECHARGE_ALL);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & PRECHARGE)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (REFRESH | SIDE1 | SIDE0);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & REFRESH)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (REFRESH | SIDE1 | SIDE0);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & REFRESH)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) = (LD_MODE_REG | SIDE1 | SIDE0 | LOAD_MODE_NORMAL);
			while(*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_INIT_FRM_PCI)) & LD_MODE_REG)
				hal_delay_us(1000); // 1ms
			
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_ECC_TEST_FRM_PCI)) |= DISABLE_CHK;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + DU_ECC_TEST_FRM_PCI));
#endif
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + MISC_CONTROL_FRM_PCI)) |= FLASH_ALIAS_DISABLE;
#ifdef A0_REV
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + MISC_CONTROL_FRM_PCI));
#endif
			
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;
			// check slave's SDRAM
			for(i = 0; i < MEM_REGION_rom_SIZE; i+=4)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i)) = 0x5A5A5A5A;
				
				// read it back
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
				if(temp != 0x5A5A5A5A)
				{
					eprintf("slave SDRAM POST FAIL! addr = 0x%08X, actual = 0x%08X, expected = 0x5A5A5A5A\n", i, temp);
					slave_sdram_passed = 0;
					break;
				}
			}
			eprintf("Slave SDRAM POST complete\n");
		}
		
		if(slave_sdram_passed || boot_from_boot_monitor)
		{
			// now download Diag to slave's SDRAM.
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;

			// copy the text section
			for(i = 0; i < MEM_REGION_rom_SIZE; i = i + 4)
			{
				if(!boot_from_boot_monitor)
				{
					*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i))
						= *((volatile UINT32 *)(SLOW_PORT_BASE + i));
				}
				else
				{
					*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i))
						= *((volatile UINT32 *)(RAM_BASE + untouched_diag_location + i));
				}
#ifdef A0_REV
				// read it back to ensure that xscale core doesn't do a burst write.
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
#endif
			}

			// copy the data section
			for(i = rom_data_start, j = ram_data_start; i < rom_data_end; i+=4, j+=4)
			{
				if(!boot_from_boot_monitor)
				{
					*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + j))
						= *((volatile UINT32 *)(SLOW_PORT_BASE + i));
				}
				else
				{
					*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + j))
						= *((volatile UINT32 *)(RAM_BASE + untouched_diag_location + i));
				}
#ifdef A0_REV
				// read it back to ensure that xscale core doesn't do a burst write.
				temp = *((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
#endif
			}
			
			if(!boot_from_boot_monitor)
			{
				// scrub only 2M
				for(i = MEM_REGION_rom_SIZE; i < 0x200000; i = i + 4)
				{
					*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i)) = 0;
#ifdef A0_REV				
					// read it back to ensure that xscale core doesn't do a burst write.
					temp = *((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + i));
#endif
				}
			}

			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
			if(boot_from_boot_monitor)
			{
				*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI)) |= (RESET_XSCALE);
				hal_delay_us(1000);
			}
			// final step - take slave out of reset and I am done with slave.
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + IXP_RESET0_FRM_PCI))	&= ~(RESET_XSCALE);


			// now wait to see if slave completes its POST
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;

			i = 0;
			eprintf("Waiting for slave to complete POST.");
			while(*((volatile UINT32 *)(slave_dev_info.base_map[SDRAM_BAR] + SLAVE_POST_FLAG_LOC)) != POST_COMPLETE)
			{
				// wait for 15s max
				if(i > 30)
				{
					eprintf("\nSlave POST FAIL!");
					break;
				}
				// wait for 500ms
				hal_delay_us(500000);
				eprintf(".");
				i++;
			}
			eprintf("\n");
		}
	}

	// copy config data to slave's scratchpad
	if(board_config_data.config_valid == CONFIG_DATA_VALID)
	{
		PUINT s = (PUINT)&board_config_data;
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		for(i = 0; i < sizeof(struct board_config); i += 4)
		{
			*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + i))
				= s[i/4];
#ifdef A0_REV			
			// read it back to prevent PCI Mem bug
			temp = *((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + i));
#endif
		}
	}
#ifdef __ARMEB__
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	*((volatile UINT32 *)(slave_dev_info.base_map[CSR_BAR] + PCI_CONTROL_FRM_PCI)) &= ~(PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);

	/* 82559 is LE. So, now I need to clear byte swapping bits so that when 82559 talks to IXP2400
	pci unit does byte swapping */
	*(PCI_CONTROL_REG) &= ~(PCI_CONTROL_BE_DEO | PCI_CONTROL_BE_DEI | PCI_CONTROL_BE_BEO | PCI_CONTROL_BE_BEI);
#endif

	return;
}

PCI_ADDRESS64 force_pci_alloc_mem(PCI_ADDRESS64 *base, int *addr_ext, UINT32 size)
{
    PCI_ADDRESS64 aligned_addr;

    if(size >= 0x8000000)
    {
        (*addr_ext)++;
        aligned_addr = (*addr_ext) << 29;
    }
    else
    {
        aligned_addr = (*base + size - 1) & ~(size - 1);
        // Update the resource pointer and return values.
        *base = aligned_addr + size;
    }

    return aligned_addr;
}

void hal_plf_pci_init(void)
{
    pci_device_id dev_id = PCI_NULL_DEVID;
	pci_device_id master_dev_id;
	UINT8 next_bus = SECONDARY_BUS_NUM;
	int slave_found = 0;
	int sram_bar_mask_array[] = {0x1FC0000, 0x3FC0000, 0x7FC0000, 0xFFC0000};
	int dram_bar_mask_array[] = {0x7F00000, 0xFF00000, 0x1FF00000, 0x3FF00000};
	int sram_bar_size, dram_bar_size, sram_bar, dram_bar;

	// if there is no flash then pci init. is done by master. so don't do it again
	if(strap_options_val & CFG_PROM_BOOT)	// flash is present
	{
		if(!boot_from_boot_monitor)
		{
			/* set the upper bits for pci mem and io transactions */
			*(PCI_ADDR_EXT_REG) = 0x0;
				
			/* Setup SRAM Base Address Mask Reg */
			sram_bar_size = (strap_options_val & 0xC0) >> 6;
			*(SRAM_BASE_ADDR_MASK_REG) = sram_bar_mask_array[sram_bar_size];
			sram_bar = 0x2000000 * (1 << sram_bar_size);

			/* Setup SDRAM Base Address Mask Reg */
			dram_bar_size = (strap_options_val & 0x30) >> 4;
			*(DRAM_BASE_ADDR_MASK_REG) = dram_bar_mask_array[dram_bar_size];
			dram_bar = 0x8000000 * (1 << dram_bar_size);
		
			if(strap_options_val & CFG_PCI_BOOT_HOST)	// If Master
			{
                PCI_ADDRESS64 pci_base = HAL_PCI_ALLOC_BASE_MEMORY;
                int addr_ext = 0;

				*(PCI_CMD_STAT_REG) = PCI_CMD_STAT_VAL;
				*(PCI_CSR_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, 0x100000);
				*(PCI_SRAM_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, sram_bar);
				*(PCI_DRAM_BAR_REG) = force_pci_alloc_mem(&pci_base, &addr_ext, dram_bar);
				pci_set_memory_base(pci_base, addr_ext);
			}

			/* enable outbound pci int. from xscale */
			*(PCI_OUT_INT_MASK_REG) &= XSIM;

			*(IXP_RESET0_REG) |= INIT_COMP;
		}

		// if this is master npu then init bus
		if(strap_options_val & CFG_PCI_BOOT_HOST)
		{
			if(!boot_from_boot_monitor)
			{
				// Initialize Primary PCI bus (bus 0)
				pci_configure_bus(0, &next_bus);
			}

			// prepare dev_id so that we skip slave
			master_dev_id = PCI_DEV_MAKE_ID(0, PCI_DEV_MAKE_DEVFN(6, 0));
			pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &master_dev_id);
			pci_get_device_info(master_dev_id, &master_dev_info);

			// first npu would be slave
			dev_id = PCI_NULL_DEVID;
			// configure slave npu
			if(pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &dev_id))
			{
				if(dev_id != master_dev_id)
				{
					pci_get_device_info(dev_id, &slave_dev_info);
					slave_found = 1;
				}
			}

			if(slave_found)
			{
				slave_ixp_init();
			}
			else
			{
				eprintf("can't find slave NPU\n");
			}
		}
		else
		{
			int i = 0;
			PUINT s = (PUINT)&board_config_data;
			struct board_config *board_config_data_temp = (struct board_config *)SCRATCHPAD_BASE;

			eprintf("Waiting for master to copy config data.");
			// wait till master copies the config data and assigns BAR
			while(board_config_data_temp->config_valid != CONFIG_DATA_VALID)
			{
				// wait for 5s max
				if(i > 10)
				{
					eprintf("\nMaster hasn't copied the config data");
					break;
				}
				// wait for 500ms
				hal_delay_us(500000);
				eprintf(".");
				i++;		
			}
			eprintf("\n");

			// Copy config data from scratch
			for(i = 0; i < sizeof(struct board_config); i += 4)
			{
				s[i/4] = *((volatile UINT32 *)(SCRATCHPAD_BASE + i));
			}
		}
	}

	// on slave I need to populate master_dev_info and slave_dev_info and validate config data
	if(!(strap_options_val & CFG_PCI_BOOT_HOST))
	{
		dev_id = PCI_NULL_DEVID;
		// this is slave npu
		pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &dev_id);
		pci_get_device_info(dev_id, &slave_dev_info);

		// now find master npu
		pci_find_device(IXP2400_VENDOR_ID, IXP2400_DEVICE_ID, &dev_id);
		pci_get_device_info(dev_id, &master_dev_info);
	}
	*(PCI_ADDR_EXT_REG) = 0;
}

// Use "naked" attribute to suppress C prologue/epilogue
static void __attribute__ ((naked)) __pci_abort_handler(void) 
{
    asm ( "subs	pc, lr, #4\n" );
}

static UINT32 orig_abort_vec;

static inline UINT32 *pci_config_setup(UINT32 bus,
					   UINT32 devfn,
					   UINT32 offset)
{
	UINT32 *paddr;
    UINT32 dev = PCI_DEV_GET_DEV(devfn);
    UINT32 fn  = PCI_DEV_GET_FN(devfn);

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
	}

    // Offsets must be dword-aligned
    offset &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{ // Type 0 config
		(UINT32)paddr |= ((1 << (dev + 16)) | (fn << 8) | offset | 0);
	}
	else
	{ // Type 1 config
		(UINT32)paddr |= ((bus << 16) | (dev << 11) | (fn << 8) | offset | 0);
	}

	orig_abort_vec = ((volatile UINT32 *)0x20)[4];
    ((volatile unsigned *)0x20)[4] = (unsigned)__pci_abort_handler;
    HAL_ICACHE_SYNC();

    return paddr;
}

static int clear_master_aborts(void)
{
	UINT32 temp;
	int err = 0;

	temp = *(PCI_CONTROL_REG);
	if(temp & 0x120)
	{
		err = 1;
		*(PCI_CONTROL_REG) = temp;
	}
	temp = *(PCI_CMD_STAT_REG);
	if(temp & 0x20000000)
	{
		err = 1;
		*(PCI_CMD_STAT_REG) = temp;
#ifdef A0_REV
		while (temp & 0x20000000)
		{
			*(PCI_CMD_STAT_REG) = temp;
			temp = *(PCI_CMD_STAT_REG);
		}
#endif
	}

	((volatile unsigned *)0x20)[4] = orig_abort_vec;
    HAL_ICACHE_SYNC();

	return err;
}

UINT32 hal_plf_pci_cfg_read_dword (UINT32 bus,
					   UINT32 devfn,
					   UINT32 offset)
{
    UINT32 *pdata, config_data;

    pdata = pci_config_setup(bus, devfn, offset);

    config_data = *pdata;

	if(clear_master_aborts())
		return 0xFFFFFFFF;
	else
		return config_data;
}


void hal_plf_pci_cfg_write_dword (UINT32 bus,
				      UINT32 devfn,
				      UINT32 offset,
				      UINT32 data)
{
    UINT32 *pdata;

    pdata = pci_config_setup(bus, devfn, offset);

    *pdata = data;

	clear_master_aborts();
}


UINT16 hal_plf_pci_cfg_read_word (UINT32 bus,
					  UINT32 devfn,
					  UINT32 offset)
{
    UINT32 *pdata;
    UINT16 config_data;

    pdata = pci_config_setup(bus, devfn, offset);

    config_data = (UINT16)(((*pdata) >> ((offset % 0x4) * 8)) & 0xffff);

    if(clear_master_aborts())
		return 0xFFFF;
	else
		return config_data;
}

void hal_plf_pci_cfg_write_word (UINT32 bus,
				     UINT32 devfn,
				     UINT32 offset,
				     UINT16 data)
{
    UINT32 *pdata, mask, temp;

    pdata = pci_config_setup(bus, devfn, offset);

    mask = ~(0x0000ffff << ((offset % 0x4) * 8));

    temp = (UINT32)(((UINT32)data) << ((offset % 0x4) * 8));
    *pdata = (*pdata & mask) | temp;

	clear_master_aborts();
}

UINT8 hal_plf_pci_cfg_read_byte (UINT32 bus,
					 UINT32 devfn,
					 UINT32 offset)
{
    UINT32 *pdata;
    UINT8 config_data;

    pdata = pci_config_setup(bus, devfn, offset);

    config_data = (UINT8)(((*pdata) >> ((offset % 0x4) * 8)) & 0xff);

	if(clear_master_aborts())
		return 0xFF;
	else
		return config_data;
}


void hal_plf_pci_cfg_write_byte (UINT32 bus,
				     UINT32 devfn,
				     UINT32 offset,
				     UINT8 data)
{
    UINT32 *pdata, mask, temp;

    pdata = pci_config_setup(bus, devfn, offset);

    mask = ~(0x000000ff << ((offset % 0x4) * 8));
    temp = (UINT32)(((UINT32)data) << ((offset % 0x4) * 8));
    *pdata = (*pdata & mask) | temp;

	clear_master_aborts();
}

