//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// <http://www.redhat.com/>                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================

#include "hal_io.h"
#include "pci.h"

//#define PKG_IO_PCI_DEBUG

extern void eprintf(char* f, ...);
extern UINT32 hal_plf_pci_cfg_read_dword (UINT32 bus, UINT32 devfn, UINT32 offset);
extern UINT16 hal_plf_pci_cfg_read_word  (UINT32 bus, UINT32 devfn, UINT32 offset);
extern UINT8 hal_plf_pci_cfg_read_byte   (UINT32 bus, UINT32 devfn, UINT32 offset);
extern void hal_plf_pci_cfg_write_dword (UINT32 bus, UINT32 devfn, UINT32 offset, UINT32 val);
extern void hal_plf_pci_cfg_write_word  (UINT32 bus, UINT32 devfn, UINT32 offset, UINT16 val);
extern void hal_plf_pci_cfg_write_byte   (UINT32 bus, UINT32 devfn, UINT32 offset, UINT8 val);

externC void hal_plf_pci_init(void);

static int pci_lib_initialized = false;

void
pci_init( void )
{
    if (!pci_lib_initialized) {
        // Initialize the PCI bus, preparing it for general access.
        hal_plf_pci_init();

        pci_lib_initialized = true;
    }
}

//---------------------------------------------------------------------------
// Common device configuration access functions

void 
pci_get_device_info ( pci_device_id devid, pci_device *dev_info )
{
    int i;
    UINT8 bus = PCI_DEV_GET_BUS(devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(devid);
    UINT8 header_type;

    dev_info->devid = devid;

    dev_info->vendor = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_VENDOR);
    
	dev_info->device = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_DEVICE);
    
	dev_info->command = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_COMMAND);
    
	dev_info->status = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_STATUS);

	dev_info->class_rev =
			hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_CLASS_REV);
    dev_info->cache_line_size =
			hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_CACHE_LINE_SIZE);
    dev_info->latency_timer =
			hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_LATENCY_TIMER);
	header_type = hal_plf_pci_cfg_read_byte (bus, devfn, PCI_CFG_HEADER_TYPE);

    dev_info->header_type = (pci_header_type) header_type;
    dev_info->bist = hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_BIST);


    if ((dev_info->header_type & PCI_CFG_HEADER_TYPE_MASK) == PCI_HEADER_BRIDGE)
		dev_info->num_bars = 2;
    else
		dev_info->num_bars = 6;

    for (i = 0; i < dev_info->num_bars; i++)
        dev_info->base_address[i] = hal_plf_pci_cfg_read_dword(bus,
									devfn, PCI_CFG_BAR_BASE + 4*i);


    // If device is disabled, probe BARs for sizes.
    if ((dev_info->command & PCI_CFG_COMMAND_ACTIVE) == 0) {

        for (i = 0; i < dev_info->num_bars; i++){
            UINT32 size;

            hal_plf_pci_cfg_write_dword(bus, devfn, 
                                          PCI_CFG_BAR_BASE + 4*i,
                                          0xffffffff);
            size = hal_plf_pci_cfg_read_dword(bus, devfn, 
                                         PCI_CFG_BAR_BASE + 4*i);

            dev_info->base_size[i] = size;
            dev_info->base_map[i] = 0xffffffff;




	    // No reason to scan beyond first inactive BAR.
	    if (size == 0) {
		dev_info->num_bars = i;
		break;
	    }

            // Check for a 64bit memory region.
            if (PCI_CFG_BAR_SPACE_MEM == 
                (size & PCI_CFG_BAR_SPACE_MASK)) {
                if (size & PRI_CFG_BAR_MEM_TYPE_64) {
                    // Clear fields for next BAR - it's the upper 32 bits.
                    i++;
                    dev_info->base_size[i] = 0;
                    dev_info->base_map[i] = 0xffffffff;
                }
            }
        }

    } else {
	// If the device is already configured. Fill in the base_map.
	PCI_ADDRESS64 tmp_addr = 0;
	UINT32 bar;





        for (i = 0; i < dev_info->num_bars; i++){

	    dev_info->base_size[i] = 0;

            bar = dev_info->base_address[i];

	    // No reason to scan beyond first inactive BAR.
//	    if (bar == 0) {
//		dev_info->num_bars = i;
//		break;
//	    }

	    if ((bar & PCI_CFG_BAR_SPACE_MASK) == PCI_CFG_BAR_SPACE_IO) {
		dev_info->base_map[i] = (bar & (PRI_CFG_BAR_IO_MASK & XSCALE_PCI_IO_SZ_MASK)) + HAL_PCI_PHYSICAL_IO_BASE;
	    } else {
		tmp_addr = bar & (PRI_CFG_BAR_MEM_MASK & XSCALE_PCI_MEM_SZ_MASK);

		if ((bar & PRI_CFG_BAR_MEM_TYPE_MASK) == PRI_CFG_BAR_MEM_TYPE_64)
		    tmp_addr |= ((PCI_ADDRESS64)(dev_info->base_address[i+1] & PRI_CFG_BAR_MEM_MASK)) << 32;
			
		tmp_addr += HAL_PCI_PHYSICAL_MEMORY_BASE;

		dev_info->base_map[i] = tmp_addr;
		if ((bar & PRI_CFG_BAR_MEM_TYPE_MASK) == PRI_CFG_BAR_MEM_TYPE_64)
		    dev_info->base_map[i++] = tmp_addr >> 32;
	    }
        }
    }


    switch (dev_info->header_type & PCI_CFG_HEADER_TYPE_MASK) {
    case PCI_HEADER_NORMAL:
        dev_info->header.normal.cardbus_cis =
				hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_CARDBUS_CIS);
        dev_info->header.normal.sub_vendor =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_SUB_VENDOR);
        dev_info->header.normal.sub_id =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_SUB_ID);
        dev_info->header.normal.rom_address =
				hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_ROM_ADDRESS);
        dev_info->header.normal.cap_list =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_CAP_LIST);
        
		dev_info->header.normal.int_line =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_INT_LINE);
        dev_info->header.normal.int_pin =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_INT_PIN);
        dev_info->header.normal.min_gnt =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_MIN_GNT);
        dev_info->header.normal.max_lat =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_MAX_LAT);
        break;
    case PCI_HEADER_BRIDGE:
        dev_info->header.bridge.pri_bus =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_PRI_BUS);
        dev_info->header.bridge.sec_bus =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_SEC_BUS);
        dev_info->header.bridge.sub_bus =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_SUB_BUS);
        dev_info->header.bridge.sec_latency_timer =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_SEC_LATENCY_TIMER);
        dev_info->header.bridge.io_base =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_IO_BASE);
        dev_info->header.bridge.io_limit =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_IO_LIMIT);
        dev_info->header.bridge.sec_status =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_SEC_STATUS);
        dev_info->header.bridge.mem_base =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_MEM_BASE);
        dev_info->header.bridge.mem_limit =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_MEM_LIMIT);
        dev_info->header.bridge.prefetch_base =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_PREFETCH_BASE);
        dev_info->header.bridge.prefetch_limit =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_PREFETCH_LIMIT);
        dev_info->header.bridge.prefetch_base_upper32 =
				hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_PREFETCH_BASE_UPPER32);
        dev_info->header.bridge.prefetch_limit_upper32 =
				hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_PREFETCH_LIMIT_UPPER32);
        dev_info->header.bridge.io_base_upper16 =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_IO_BASE_UPPER16);
        dev_info->header.bridge.io_limit_upper16 =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_IO_LIMIT_UPPER16);
		dev_info->header.bridge.rom_address = 
        		hal_plf_pci_cfg_read_dword(bus, devfn, PCI_CFG_BRIDGE_ROM_ADDRESS);
        dev_info->header.bridge.int_line =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_INT_LINE);
        dev_info->header.bridge.int_pin =
				hal_plf_pci_cfg_read_byte(bus, devfn, PCI_CFG_INT_PIN);
        dev_info->header.bridge.control =
				hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_BRIDGE_CONTROL);
        break;
    case PCI_HEADER_CARDBUS_BRIDGE:
        eprintf("PCI device header 'cardbus bridge' support not implemented");
        break;
    default:
        eprintf("Unknown PCI device header type");
        break;
    }
}

void 
pci_set_device_info ( pci_device_id devid, pci_device *dev_info )
{
    UINT8 bus = PCI_DEV_GET_BUS(devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(devid);
    int i;

    // Only writable entries are updated.
    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_COMMAND,
                                  dev_info->command);
    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_STATUS,
                                  dev_info->status);
    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_CACHE_LINE_SIZE,
                                 dev_info->cache_line_size);
    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_LATENCY_TIMER,
                                 dev_info->latency_timer);
    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_BIST,
                                 dev_info->bist);

    for (i = 0; i < dev_info->num_bars; i++) {
	hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_BAR_BASE+4*i,
				      dev_info->base_address[i]);
    }

    switch (dev_info->header_type & PCI_CFG_HEADER_TYPE_MASK) {
    case PCI_HEADER_NORMAL:
        hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_CARDBUS_CIS,
                                      dev_info->header.normal.cardbus_cis);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_SUB_VENDOR,
                                      dev_info->header.normal.sub_vendor);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_SUB_ID,
                                      dev_info->header.normal.sub_id);
        hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_ROM_ADDRESS,
                                      dev_info->header.normal.rom_address);

        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_INT_LINE,
                                     dev_info->header.normal.int_line);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_INT_PIN,
                                     dev_info->header.normal.int_pin);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_MIN_GNT,
                                     dev_info->header.normal.min_gnt);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_MAX_LAT,
                                     dev_info->header.normal.max_lat);
        break;
    case PCI_HEADER_BRIDGE:
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_PRI_BUS,
				     dev_info->header.bridge.pri_bus);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SEC_BUS,
				     dev_info->header.bridge.sec_bus);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SUB_BUS,
				     dev_info->header.bridge.sub_bus);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SEC_LATENCY_TIMER,
				     dev_info->header.bridge.sec_latency_timer);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_IO_BASE,
				     dev_info->header.bridge.io_base);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_IO_LIMIT,
				     dev_info->header.bridge.io_limit);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_SEC_STATUS,
				      dev_info->header.bridge.sec_status);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_MEM_BASE,
				      dev_info->header.bridge.mem_base);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_MEM_LIMIT,
				      dev_info->header.bridge.mem_limit);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_PREFETCH_BASE,
				      dev_info->header.bridge.prefetch_base);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_PREFETCH_LIMIT,
				      dev_info->header.bridge.prefetch_limit);
        hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_PREFETCH_BASE_UPPER32,
				      dev_info->header.bridge.prefetch_base_upper32);
        hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_PREFETCH_LIMIT_UPPER32,
				      dev_info->header.bridge.prefetch_limit_upper32);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_IO_BASE_UPPER16,
				      dev_info->header.bridge.io_base_upper16);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_IO_LIMIT_UPPER16,
				      dev_info->header.bridge.io_limit_upper16);
        hal_plf_pci_cfg_write_dword(bus, devfn, PCI_CFG_BRIDGE_ROM_ADDRESS,
				      dev_info->header.bridge.rom_address);
        hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_INT_LINE,
				     dev_info->header.bridge.int_line);
        hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_BRIDGE_CONTROL,
				      dev_info->header.bridge.control);
        break;
    case PCI_HEADER_CARDBUS_BRIDGE:
        eprintf("PCI device header 'cardbus bridge' support not implemented");
        break;
    default:
        eprintf("Unknown PCI device header type");
        break;
    }

    // Update values in dev_info.
    pci_get_device_info(devid, dev_info);
}


//---------------------------------------------------------------------------
// Specific device configuration access functions
void 
pci_read_config_uint8( pci_device_id devid,
                           UINT8 offset, UINT8 *val)
{
    *val = hal_plf_pci_cfg_read_byte(PCI_DEV_GET_BUS(devid),
                                PCI_DEV_GET_DEVFN(devid),
                                offset);
}

void 
pci_read_config_uint16( pci_device_id devid,
                            UINT8 offset, UINT16 *val)
{
    *val = hal_plf_pci_cfg_read_word(PCI_DEV_GET_BUS(devid),
                                 PCI_DEV_GET_DEVFN(devid),
                                 offset);
}

void
pci_read_config_uint32( pci_device_id devid,
                            UINT8 offset, UINT32 *val)
{
    *val = hal_plf_pci_cfg_read_dword(PCI_DEV_GET_BUS(devid),
                                 PCI_DEV_GET_DEVFN(devid),
                                 offset);
}


// Write functions
void
pci_write_config_uint8( pci_device_id devid,
                            UINT8 offset, UINT8 val)
{
    hal_plf_pci_cfg_write_byte(PCI_DEV_GET_BUS(devid),
                                 PCI_DEV_GET_DEVFN(devid),
                                 offset, val);
}

void
pci_write_config_uint16( pci_device_id devid,
                             UINT8 offset, UINT16 val)
{
    hal_plf_pci_cfg_write_word(PCI_DEV_GET_BUS(devid),
                                  PCI_DEV_GET_DEVFN(devid),
                                  offset, val);
}

void
pci_write_config_uint32( pci_device_id devid,
                             UINT8 offset, UINT32 val)
{
    hal_plf_pci_cfg_write_dword(PCI_DEV_GET_BUS(devid),
                                  PCI_DEV_GET_DEVFN(devid),
                                  offset, val);
}

//------------------------------------------------------------------------
// Device find functions

int
pci_find_next( pci_device_id cur_devid, 
                   pci_device_id *next_devid )
{
    UINT8 bus = PCI_DEV_GET_BUS(cur_devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(cur_devid);
    UINT8 dev = PCI_DEV_GET_DEV(devfn);
    UINT8 fn = PCI_DEV_GET_FN(devfn);

#ifdef PKG_IO_PCI_DEBUG
    eprintf("pci_find_next: start[%x] ...\n",(unsigned)cur_devid);
#endif

    // If this is the initializer, start with 0/0/0
    if (PCI_NULL_DEVID == cur_devid) {
        bus = dev = fn = 0;
    } else if (PCI_NULL_DEVFN == (cur_devid & PCI_NULL_DEVFN)) {
        dev = fn = 0;
    } else {
        // Otherwise, check multi-function bit of device's first function
        UINT8 header;

	devfn = PCI_DEV_MAKE_DEVFN(dev, 0);
	header = hal_plf_pci_cfg_read_byte(bus, devfn,
				    PCI_CFG_HEADER_TYPE);
	if (header & PCI_CFG_HEADER_TYPE_MF) {
	    // Multi-function device. Increase fn.
	    fn++;
	    if (fn >= PCI_MAX_FN) {
		fn = 0;
		dev++;
	    }
	} else {
	    // Single-function device. Skip to next.
	    dev++;
	}
    }

    // Note: Reset iterators in enclosing statement's "next" part.
    //       Allows resuming scan with given input values. 
    for (;bus < PCI_MAX_BUS; bus++, dev=0) {
        for (;dev < PCI_MAX_DEV; dev++, fn=0) {
            for (;fn < PCI_MAX_FN; fn++) {
                UINT16 vendor = 0xFFFF;

		if (PCI_IGNORE_DEVICE(bus, dev, fn))
		    continue;

                devfn = PCI_DEV_MAKE_DEVFN(dev, fn);
                vendor = hal_plf_pci_cfg_read_word(bus, devfn,
                                             PCI_CFG_VENDOR);
                if (PCI_VENDOR_UNDEFINED != vendor) {
#ifdef PKG_IO_PCI_DEBUG
                eprintf("   Bus: %d, Dev: %d, Fn: %d, Vendor: %x\n", bus, dev, fn, vendor);
#endif

                    *next_devid = PCI_DEV_MAKE_ID(bus, devfn);
                    return true;
                }
            }
        }
    }

#ifdef PKG_IO_PCI_DEBUG
    eprintf("nothing.\n");
#endif

    return false;
}

//
// Scan for a particular device, starting with 'devid'
// 'devid' is updated with the next device if found.
//         is not changed if no suitable device is found.
int
pci_find_device( UINT16 vendor, UINT16 device,
                     pci_device_id *devid )
{
    pci_device_id new_devid = *devid;

#ifdef PKG_IO_PCI_DEBUG
    eprintf("pci_find_device - vendor: %x, device: %x\n", vendor, device);
#endif
    // Scan entire bus, check for matches on valid devices.
    while (pci_find_next(new_devid, &new_devid)) {
        UINT8 bus = PCI_DEV_GET_BUS(new_devid);
        UINT8 devfn = PCI_DEV_GET_DEVFN(new_devid);
        UINT16 v, d;

        // Check that vendor matches.
        v = hal_plf_pci_cfg_read_word(bus, devfn,
                                     PCI_CFG_VENDOR);
        d = hal_plf_pci_cfg_read_word(bus, devfn,
                                     PCI_CFG_DEVICE);
#ifdef PKG_IO_PCI_DEBUG
        eprintf("... PCI vendor = %x, device = %x\n", v, d);
#endif
        if (v != vendor) continue;

        // Check that device matches.
        if (d == device) {
            *devid = new_devid;
            return true;
        }
    }

    return false;
}

int
pci_find_class( UINT32 dev_class, pci_device_id *devid )
{
    // Scan entire bus, check for matches on valid devices.
    while (pci_find_next(*devid, devid)) {
        UINT8 bus = PCI_DEV_GET_BUS(*devid);
        UINT8 devfn = PCI_DEV_GET_DEVFN(*devid);
        UINT32 c;

        // Check that class code matches.
        c = hal_plf_pci_cfg_read_dword(bus, devfn,
                                     PCI_CFG_CLASS_REV);
        c >>= 8;
        if (c == dev_class)
            return true;
    }

    return false;
}

int
pci_find_matching( pci_match_func *matchp, 
                       void * match_callback_data,
                       pci_device_id *devid )
{
    pci_device_id new_devid = *devid;

#ifdef PKG_IO_PCI_DEBUG
    eprintf("pci_find_matching - func is at %x\n", (unsigned int)matchp);
#endif
    // Scan entire bus, check for matches on valid devices.
    while (pci_find_next(new_devid, &new_devid)) {
        UINT8 bus = PCI_DEV_GET_BUS(new_devid);
        UINT8 devfn = PCI_DEV_GET_DEVFN(new_devid);
        UINT16 v, d;
        UINT32 c;

        // Check that vendor, device and class match.
        v = hal_plf_pci_cfg_read_word(bus, devfn,
                                     PCI_CFG_VENDOR);
        d = hal_plf_pci_cfg_read_word(bus, devfn,
                                     PCI_CFG_DEVICE);
        c = hal_plf_pci_cfg_read_dword(bus, devfn,
                                     PCI_CFG_CLASS_REV);
        c >>= 8;
#ifdef PKG_IO_PCI_DEBUG
        eprintf("... PCI vendor = %x, device = %x, class %x\n", v, d, c);
#endif
        // Check that device matches as the caller desires:
        if ( (*matchp)(v, d, c, match_callback_data) ) {
            *devid = new_devid;
            return true;
        }
    }

    return false;
}

//------------------------------------------------------------------------
// Resource Allocation

static PCI_ADDRESS64 pci_memory_base = HAL_PCI_ALLOC_BASE_MEMORY;
static PCI_ADDRESS32 pci_io_base = HAL_PCI_ALLOC_BASE_IO;
static int pci_mem_addr_ext = 0;

void
pci_set_memory_base(PCI_ADDRESS64 base, int addr_ext)
{
    pci_memory_base = base;
    pci_mem_addr_ext = addr_ext;
}

void
pci_set_io_base(PCI_ADDRESS32 base)
{
    pci_io_base = base;
}

int
pci_configure_device( pci_device *dev_info )
{
    int bar;
    UINT32 flags;
    int ret = true;

    // If device is already active, just return true as
    // pci_get_device_info has presumably filled in
    // the base_map already.
    if ((dev_info->command & PCI_CFG_COMMAND_ACTIVE) != 0)
	return true;

    if (dev_info->num_bars > 0) {
	for (bar = 0; bar < dev_info->num_bars; bar++) {
	    flags = dev_info->base_size[bar];

	    ret = false;

	    if ((flags & PCI_CFG_BAR_SPACE_MASK) == PCI_CFG_BAR_SPACE_MEM){
		ret |= pci_allocate_memory(dev_info, bar, 
					       &pci_memory_base);

		// If this is a 64bit memory region, skip the next bar
		// since it will contain the top 32 bits.
		if (flags & PRI_CFG_BAR_MEM_TYPE_64)
		    bar++;
	    } else
		ret |= pci_allocate_io(dev_info, bar, &pci_io_base);

	    if (!ret)
		return ret;
	}
    }

    pci_translate_interrupt(dev_info, &dev_info->hal_vector);

    return ret;
}

// This is the function that handles resource allocation. It doesn't
// affect the device state.
// Should not be called with top32bit-bar of a 64bit pair.
inline int
pci_allocate_memory_priv( pci_device *dev_info, UINT32 bar,
                              PCI_ADDRESS64 *base, 
                              PCI_ADDRESS64 *assigned_addr)
{
    UINT32 mem_type, flags;
    PCI_ADDRESS64 size, aligned_addr;

    // Get the probed size and flags
    flags = dev_info->base_size[bar];

    // Decode size
    size = (PCI_ADDRESS64) ((~(flags & PRI_CFG_BAR_MEM_MASK))+1);




    // Calculate address we will assign the device.
    // This can be made more clever, specifically:
    //  1) The lowest 1MB should be reserved for devices with 1M memory type.
    //     : Needs to be handled.
    //  2) The low 32bit space should be reserved for devices with 32bit type.
    //     : With the usual handful of devices it is unlikely that the
    //       low 4GB space will become full.
    //  3) A bitmap can be used to avoid fragmentation.
    //     : Again, unlikely to be necessary.
    //
    // For now, simply align to required size.

#ifdef PKG_IO_PCI_DEBUG
	eprintf("base is 0x%08x\n", *base);
#endif

    // Is the request for memory space?
    if (PCI_CFG_BAR_SPACE_MEM != (flags & PCI_CFG_BAR_SPACE_MASK))
        return false;

    // Check type of memory requested...
    mem_type = PRI_CFG_BAR_MEM_TYPE_MASK & flags;

    if(size >= 0x8000000)
    {
        pci_mem_addr_ext++;
        aligned_addr = pci_mem_addr_ext << 29;
    }
    else
    {
        aligned_addr = (*base+size-1) & ~(size-1);
        // Update the resource pointer and return values.
        *base = aligned_addr+size;
    }
    *assigned_addr = aligned_addr;
#ifdef PKG_IO_PCI_DEBUG
	eprintf("base is 0x%08x\n", *base);
#endif

    dev_info->base_map[bar] = (UINT32) 
        ((aligned_addr & XSCALE_PCI_MEM_SZ_MASK) + HAL_PCI_PHYSICAL_MEMORY_BASE) & 0xffffffff;

    // If a 64bit region, store upper 32 bits in the next bar.
    // Note: The CPU is not necessarily able to access the region
    // linearly - it may have to do it in segments. Driver must handle that.
    if (PRI_CFG_BAR_MEM_TYPE_64 == mem_type) {
        dev_info->base_map[bar+1] = (UINT32) 
            (((aligned_addr & XSCALE_PCI_MEM_SZ_MASK) + HAL_PCI_PHYSICAL_MEMORY_BASE) >> 32) & 0xffffffff;
    }
    
    return true;
}

int
pci_allocate_memory( pci_device *dev_info, UINT32 bar,
                         PCI_ADDRESS64 *base)
{
    UINT8 bus = PCI_DEV_GET_BUS(dev_info->devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(dev_info->devid);
    PCI_ADDRESS64 assigned_addr;
    int ret;

    // Check that device is inactive.
    if ((dev_info->command & PCI_CFG_COMMAND_ACTIVE) != 0)
        return false;

    // Allocate memory space for the device.
    ret = pci_allocate_memory_priv(dev_info, bar, base, &assigned_addr);

    if (ret) {
        // Map the device and update the BAR in the dev_info structure.
        hal_plf_pci_cfg_write_dword(bus, devfn,
                                      PCI_CFG_BAR_BASE+4*bar, 
                                      (UINT32) 
                                      (assigned_addr & 0xffffffff));
        dev_info->base_address[bar] = hal_plf_pci_cfg_read_dword(bus, devfn,
                                     PCI_CFG_BAR_BASE+4*bar);


        // Handle upper 32 bits if necessary.
        if (dev_info->base_size[bar] & PRI_CFG_BAR_MEM_TYPE_64) {
            hal_plf_pci_cfg_write_dword(bus, devfn,
                                          PCI_CFG_BAR_BASE+4*(bar+1), 
                                          (UINT32) 
                                          ((assigned_addr >> 32)& 0xffffffff));
            dev_info->base_address[bar+1] = hal_plf_pci_cfg_read_dword(bus, devfn,
                                         PCI_CFG_BAR_BASE+4*(bar+1));

        }
#ifdef PKG_IO_PCI_DEBUG
		eprintf("Assigned %d BAR 0x%08x Memory address\n", bar, dev_info->base_address[bar]);
#endif
    }

    return ret;
}    

int
pci_allocate_io_priv( pci_device *dev_info, UINT32 bar, 
                          PCI_ADDRESS32 *base, 
                          PCI_ADDRESS32 *assigned_addr)
{
    UINT32 flags, size;
    PCI_ADDRESS32 aligned_addr;

    // Get the probed size and flags
    flags = dev_info->base_size[bar];

    // Decode size
    size = (~(flags & PRI_CFG_BAR_IO_MASK))+1;

    // Calculate address we will assign the device.
    // This can be made more clever.
    // For now, simply align to required size.
    aligned_addr = (*base+size-1) & ~(size-1);

    // Is the request for IO space?
    if (PCI_CFG_BAR_SPACE_IO != (flags & PCI_CFG_BAR_SPACE_MASK))
        return false;

    // Update the resource pointer and return values.
    *base = aligned_addr+size;
    dev_info->base_map[bar] = aligned_addr+HAL_PCI_PHYSICAL_IO_BASE;
    *assigned_addr = aligned_addr;

    return true;
}


int
pci_allocate_io( pci_device *dev_info, UINT32 bar, 
                     PCI_ADDRESS32 *base)
{
    UINT8 bus = PCI_DEV_GET_BUS(dev_info->devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(dev_info->devid);
    PCI_ADDRESS32 assigned_addr;
    int ret;
    
    // Check that device is inactive.
    if ((dev_info->command & PCI_CFG_COMMAND_ACTIVE) != 0)
        return false;

    // Allocate IO space for the device.
    ret = pci_allocate_io_priv(dev_info, bar, base, &assigned_addr);

    if (ret) {
        // Map the device and update the BAR in the dev_info structure.
        hal_plf_pci_cfg_write_dword(bus, devfn,
                                      PCI_CFG_BAR_BASE+4*bar, 
                                      assigned_addr);
        dev_info->base_address[bar] = hal_plf_pci_cfg_read_dword(bus, devfn,
                                     PCI_CFG_BAR_BASE+4*bar);

#ifdef PKG_IO_PCI_DEBUG
		eprintf("Assigned %d BAR 0x%08x IO address\n", bar, dev_info->base_address[bar]);
#endif
    }

    return ret;
}

// translation from macro
int
pcihw_translate_interrupt(UINT8 bus, UINT8 devfn, UINT32 *vec)
{
    int valid;

    HAL_PCI_TRANSLATE_INTERRUPT(bus, devfn, *vec, valid);

    return valid;
}

int
pci_translate_interrupt( pci_device *dev_info,
                             UINT32 *vec )
{
    UINT8 bus = PCI_DEV_GET_BUS(dev_info->devid);
    UINT8 devfn = PCI_DEV_GET_DEVFN(dev_info->devid);

    return pcihw_translate_interrupt(bus, devfn, vec);
}


// Initialize devices on a given bus and all subordinate busses.
int
pci_configure_bus( UINT8 bus,
		       UINT8 *next_bus )
{
    UINT8 devfn, header_type;
    pci_device_id devid;
    pci_device dev_info;

    PCI_ADDRESS64 mem_start, mem_limit, mem_base;
    PCI_ADDRESS32 io_start, io_limit, io_base;

    // Scan only this bus for valid devices.
    devid = PCI_DEV_MAKE_ID(bus, 0) | PCI_NULL_DEVFN;

#ifdef PKG_IO_PCI_DEBUG
    eprintf("Configuring bus %d.\n", bus);
#endif

    while (pci_find_next(devid, &devid) && bus == PCI_DEV_GET_BUS(devid)) {

        devfn = PCI_DEV_GET_DEVFN(devid);
	
	// Get the device info
	pci_get_device_info(devid, &dev_info);

#ifdef PKG_IO_PCI_DEBUG
	eprintf("\n");
	eprintf("Configuring PCI Bus   : %d\n", bus);
	eprintf("            PCI Device: %d\n", PCI_DEV_GET_DEV(devfn));
	eprintf("            PCI Func  : %d\n", PCI_DEV_GET_FN(devfn));
	eprintf("            Vendor Id : 0x%08X\n", dev_info.vendor);
	eprintf("            Device Id : 0x%08X\n", dev_info.device);
#endif

	header_type = dev_info.header_type & PCI_CFG_HEADER_TYPE_MASK;

	// Check for supported header types.
	if (header_type != PCI_HEADER_NORMAL &&
	    header_type != PCI_HEADER_BRIDGE) {
	    eprintf("Unsupported PCI header type");
	    continue;
	}

	// Only PCI-to-PCI bridges
	if (header_type == PCI_HEADER_BRIDGE &&
	    (dev_info.class_rev >> 8) != PCI_CLASS_BRIDGE_PCI_PCI) {
	    eprintf("Unsupported PCI bridge class");
	    continue;
	}

	// Configure the base registers
	if (!pci_configure_device(&dev_info)) {
	    // Apparently out of resources.
	    eprintf("pci_configure_device failed");
	    break;
	}
	
	// Activate non-bridge devices.
	if (header_type != PCI_HEADER_BRIDGE) {
	    dev_info.command |= (PCI_CFG_COMMAND_IO 	// enable I/O space
		              | PCI_CFG_COMMAND_MEMORY	// enable memory space
		              | PCI_CFG_COMMAND_MASTER); 	// enable bus master
	    pci_write_config_uint16(dev_info.devid, PCI_CFG_COMMAND, dev_info.command);
	} else {
	    //  Bridge Configuration

	    // Set up the bus numbers that define the bridge
	    dev_info.header.bridge.pri_bus = bus;
	    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_PRI_BUS,
					 dev_info.header.bridge.pri_bus);

	    dev_info.header.bridge.sec_bus = *next_bus;
	    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SEC_BUS,
					 dev_info.header.bridge.sec_bus);

	    // Temporarily set to maximum so config cycles get passed along.
	    dev_info.header.bridge.sub_bus = PCI_MAX_BUS - 1;
	    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SUB_BUS,
					 dev_info.header.bridge.sub_bus);

	    // increment bus counter
	    *next_bus += 1;

	    // To figure the sizes of the memory and I/O windows, save the
	    // current base of memory and I/O before configuring the bus
	    // or busses on the secondary side of the bridge. After the
	    // secondary side is configured, the difference between the
	    // current values and saved values will tell the size.

	    // For bridges, the memory window must start and end on a 1M
	    // boundary and the I/O window must start and end on a 4K
	    // boundary. We round up the mem and I/O allocation bases
	    // to appropriate boundaries before configuring the secondary
	    // bus. Save the pre-rounded values in case no mem or I/O
	    // is needed we can recover any space lost due to rounding.

	    // round up start of PCI memory space to a 1M boundary
	    mem_base = pci_memory_base;
	    pci_memory_base += 0xfffff;
	    pci_memory_base &= ~0xfffff;
	    mem_start = pci_memory_base;

	    // round up start of PCI I/O space to a 4 Kbyte start address
	    io_base = pci_io_base;
	    pci_io_base += 0xfff;
	    pci_io_base &= ~0xfff;
	    io_start = pci_io_base;
		
	    // configure the subordinate PCI bus
	    pci_configure_bus (dev_info.header.bridge.sec_bus, next_bus);
			
	    // set subordinate bus number to last assigned bus number
	    dev_info.header.bridge.sub_bus = *next_bus - 1;
	    hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_SUB_BUS,
					 dev_info.header.bridge.sub_bus);

	    // Did sub bus configuration use any I/O?
	    if (pci_io_base > io_start) {

		// round up end of bridge's I/O space to a 4K boundary
		pci_io_base += 0xfff;
		pci_io_base &= ~0xfff;
		io_limit = pci_io_base - 0x1000;

		// Enable I/O cycles across bridge
		dev_info.command |= PCI_CFG_COMMAND_IO;

		// 32 Bit I/O?
		if ((dev_info.header.bridge.io_base & 0x0f) == 0x01) {
		    // I/O Base Upper 16 Bits Register
		    dev_info.header.bridge.io_base_upper16 = io_start >> 16;
		    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_IO_BASE_UPPER16,
						 dev_info.header.bridge.io_base_upper16);
		    // I/O Limit Upper 16 Bits Register
		    dev_info.header.bridge.io_limit_upper16 = io_limit >> 16;
		    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_IO_LIMIT_UPPER16,
						 dev_info.header.bridge.io_limit_upper16);
		}

		// I/O Base Register
		dev_info.header.bridge.io_base = (io_start & 0xf000) >> 8;
		hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_IO_BASE,
					     dev_info.header.bridge.io_base);
		// I/O Limit Register
		dev_info.header.bridge.io_limit = (io_limit & 0xf000) >> 8;
		hal_plf_pci_cfg_write_byte(bus, devfn, PCI_CFG_IO_LIMIT,
					     dev_info.header.bridge.io_limit);

	    } else {
		// No I/O space used on secondary bus.
		// Recover any space lost on unnecessary rounding
		pci_io_base = io_base;
	    }

	    // Did sub bus configuration use any memory?
	    if (pci_memory_base > mem_start) {

		// round up end of bridge's PCI memory space to a 1M boundary
		pci_memory_base += 0xfffff;
		pci_memory_base &= ~0xfffff;
		mem_limit = pci_memory_base - 0x100000;

		// Enable memory cycles across bridge
		dev_info.command |= PCI_CFG_COMMAND_MEMORY;

		// Memory Base Register
		dev_info.header.bridge.mem_base = (mem_start >> 16) & 0xfff0;
		hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_MEM_BASE,
						 dev_info.header.bridge.mem_base);

		// Memory Limit Register
		dev_info.header.bridge.mem_limit = (mem_limit >> 16) & 0xfff0;
		hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_MEM_LIMIT,
						 dev_info.header.bridge.mem_limit);

		// Prefetchable memory not yet supported across bridges.
		// Disable by making limit < base
		{
		    UINT16 tmp_word;

		    tmp_word = 0;
		    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_PREFETCH_LIMIT,
						 tmp_word);
		    tmp_word = 0xfff0;
		    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_PREFETCH_BASE,
						 tmp_word);
		}
	    } else {
		// No memory space used on secondary bus.
		// Recover any space lost on unnecessary rounding
		pci_memory_base = mem_base;
	    }

	    // Setup the bridge command register
	    dev_info.command |= PCI_CFG_COMMAND_MASTER;
	    dev_info.command |= PCI_CFG_COMMAND_SERR;
	    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_COMMAND,
					  dev_info.command);

	    /* Setup the Bridge Control Register */
	    dev_info.header.bridge.control |= PCI_CFG_BRIDGE_CTL_PARITY;
	    dev_info.header.bridge.control |= PCI_CFG_BRIDGE_CTL_SERR;
	    dev_info.header.bridge.control |= PCI_CFG_BRIDGE_CTL_MASTER;
	    hal_plf_pci_cfg_write_word(bus, devfn, PCI_CFG_BRIDGE_CONTROL,
					  dev_info.header.bridge.control);
	}
    }
#ifdef PKG_IO_PCI_DEBUG
    eprintf("Finished configuring bus %d.\n", bus);
#endif

    return true;
}

//-----------------------------------------------------------------------------
// end of pci.
