@@==========================================================================
@@####COPYRIGHTBEGIN####
@@                                                                          
@@ -------------------------------------------                              
@@ The contents of this file are subject to the Red Hat eCos Public License 
@@ Version 1.1 (the "License"); you may not use this file except in         
@@ compliance with the License.  You may obtain a copy of the License at    
@@ <http:@@www.redhat.com/>                                                   
@@                                                                          
@@ Software distributed under the License is distributed on an "AS IS"      
@@ basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
@@ License for the specific language governing rights and limitations under 
@@ the License.                                                             
@@                                                                          
@@ The Original Code is eCos - Embedded Configurable Operating System,      
@@ released September 30, 1998.                                             
@@                                                                          
@@ The Initial Developer of the Original Code is Red Hat.                   
@@ Portions created by Red Hat are                                          
@@ Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
@@ All Rights Reserved.                                                     
@@ -------------------------------------------                              
@@                                                                          
@@####COPYRIGHTEND####
@@==========================================================================

.include "../include/hal_platform_setup.h"
.include "../include/arm.inc"

@@ HWR_HAL_ROM_VADDR is used when compiling for a different location
@@ from the base of ROM.  hal_platform_setup.h might define it.  For
@@ example, if flash is from 0x50000000 upwards (as on SA11x0), and we are
@@ to execute at 0x50040000, then we want the reset vector to point to
@@ 0x0004pqrs - the unmapped ROM address of the code - rather than
@@ 0x0000pqrs, which is the offset into our flash block.
@@ 
@@ But usually it's not defined, so the behaviour is the obvious.

        .file   "vectors.S"

@@==========================================================================
@@ Hardware exception vectors.
@@   This entire section will be copied to location 0x0000 at startup time.
@@
        .code   32
        .section ".vectors","ax"

        .global __exception_handlers
__exception_handlers:
		@.byte 0x18, 0xF0, 0x9F, 0xE5
		b       reset_vector                    @@ 0x00
        @@ldr     pc,.reset_vector                @@ 0x00
        ldr     pc,.undefined_instruction       @@ 0x04
        ldr     pc,.software_interrupt          @@ 0x08 start && software int
        ldr     pc,.abort_prefetch              @@ 0x0C
        ldr     pc,.abort_data                  @@ 0x10
        .word   0                               @@ unused
        ldr     pc,.IRQ                         @@ 0x18
        ldr     pc,.FIQ                         @@ 0x1C

@@ The layout of these pointers should match the vector table above since
@@ they are copied in pairs.
        .global vectors
vectors:
.reset_vector:			.word (reset_vector - __exception_handlers)	@@ 0x20
.undefined_instruction:	.word undefined_instruction	@@ 0x24
.software_interrupt:	.word software_interrupt	@@ 0x28
.abort_prefetch:		.word abort_prefetch		@@ 0x2C
.abort_data:			.word abort_data			@@ 0x30
        				.word 0						@@ 0x34
.IRQ:					.word IRQ					@@ 0x38
.FIQ:					.word FIQ					@@ 0x3c
        @@ location 0x40 is used for storing DRAM size        
        .section ".fixed_vectors"
        @@ Interrupt/exception VSR pointers
        .globl  hal_vsr_table
hal_vsr_table:
        .rept   8               
        .long   0
        .endr

        .globl  hal_dram_size
hal_dram_size:  
        .long   0
		.globl  boot_from_boot_monitor
boot_from_boot_monitor:  
        .long   0
		.globl	untouched_diag_location
untouched_diag_location:
		.long	0
	@@ what, if anything, hal_dram_type means is up to the platform
        .globl  hal_dram_type
hal_dram_type:  
        .long   0
		.globl strap_options_val
strap_options_val:
		.long 0

        .balign 32
        
        .text   
@@ Startup code which will get the machine into supervisor mode
        .global reset_vector
        .type   reset_vector,function
reset_vector:
        _platform_setup1        @@ Early stage platform initialization
                                @@ which can set DRAM size at 0x40
								@@ see <cyg/hal/hal_platform_setup.h>

        @@ Come here to reset board
warm_reset:

        @@ We cannot access any LED registers until after PLATFORM_SETUP1
        HEX_DISPLAY 'S', 'T', 'A', 'K'

        @@ initialize interrupt/exception environments
        ldr     sp,.__startup_stack
        mov     r0,#(CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_IRQ_MODE)
        msr     cpsr,r0
        ldr     sp,.__exception_stack
        mov     r0,#(CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_UNDEF_MODE)
        msr     cpsr,r0
        ldr     sp,.__exception_stack

        @@ initialize CPSR (machine state register)
        mov     r0,#(CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_SUPERVISOR_MODE)
        msr     cpsr,r0

        @@ Note: some functions in LIBGCC1 will cause a "restore from SPSR"!!
        msr     spsr,r0

        @@ initialize stack
        ldr     sp,.__startup_stack


@@ if slave booting through PCI, then master has moved data section
@@ else, move data section from rom area to ram area
	cmp r9, #0
	bne 3f		@@ branch if master
	cmp r10, #0
	beq move_data_section_done	@@ branch if slave boot through PCI

3:
@@ if from boot monitor or standalone from flash
	ldr 	r2, =boot_from_boot_monitor
	ldr		r1, [r2]
	cmp		r1, #0x1
	bne		load_flash_addr

	ldr		r2, =untouched_diag_location
	ldr 	r1, [r2]
	b		move_data_section

load_flash_addr:
	ldr	r1, =SLOW_PORT_BASE


move_data_section:
	@@ Relocate [copy] data from ROM to RAM
	ldr     r2, .__rom_data_start
    add     r1, r1, r2
    ldr     r2, .__ram_data_start
    ldr     r3, .__ram_data_end
    cmp     r2, r3			@@ jump if no data to move
    beq     move_data_section_done
    sub     r1, r1, #4		@@ loop adjustments
    sub     r2, r2, #4
1:  ldr     r0, [r1,#4]!	@@ copy info
    str     r0, [r2,#4]!
    cmp     r2, r3
    bne     1b

move_data_section_done:



		@@ Save Strap Options
		ldr r0, =STRAP_OPTIONS		@@ Load the address of the strap options Register in r0
		ldr r1, [r0]				@@ get value in r1
		ldr r2, =strap_options_val
		str	r1, [r2]

        @@ Call platform specific hardware initialization
        bl      hal_hardware_init

        bl      c_start
_start_hang:
        b       _start_hang
        .code   32
        
        .global reset_platform
        .type   reset_platform,function
reset_platform:         
        mov     r0,#0
        mov     pc,r0           @@ Jump to reset vector        

init_done:
        .long   0xDEADB00B

@@
@@ Exception handlers
@@ Assumption: get here from a non-user context [mode]
@@
        .code   32
undefined_instruction:
		HEX_DISPLAY 'U', 'D', 'E', 'F'
        ldr     sp,.__undef_exception_stack     @@ get good stack
        stmfd   sp!,{r0-r5}                     @@ save some supervisor regs
        mrs     r1,spsr
        tst     r1,#CPSR_THUMB_ENABLE
        subeq   r0,lr,#4                @@ PC at time of interrupt (ARM)
        subne   r0,lr,#2                @@ PC at time of interrupt (thumb)
        mov     r2,#HAL_EXCEPTION_ILLEGAL_INSTRUCTION
        mov     r3,sp
        b       call_exception_handler

        .code   32
software_interrupt:
		HEX_DISPLAY 'S', 'I', 'N', 'T'
        stmfd   sp!,{r8}
        ldr     r8,.__undef_exception_stack     @@ get good stack
        stmfd   r8!,{r0-r5}                     @@ save some supervisor regs
        mov     r3,r8
        ldmfd   sp!,{r8}
        mrs     r1,spsr
        tst     r1,#CPSR_THUMB_ENABLE
        subeq   r0,lr,#4                @@ PC at time of SWI (ARM)
        subne   r0,lr,#2                @@ PC at time of SWI (thumb)
        mov     r2,#HAL_EXCEPTION_INTERRUPT
        b       call_exception_handler

        .code   32
abort_prefetch:
		HEX_DISPLAY 'P', 'R', 'E', 'F'
        ldr     sp,.__undef_exception_stack     @@ get good stack
        stmfd   sp!,{r0-r5}                     @@ save some supervisor regs
        sub     r0,lr,#4                        @@ PC at time of interrupt
        mrs     r1,spsr
        mov     r2,#HAL_EXCEPTION_CODE_ACCESS
        mov     r3,sp
        b       call_exception_handler

        .code   32
abort_data:
		HEX_DISPLAY 'D', 'A', 'T', 'A'
	    ldr     sp,.__undef_exception_stack     @@ get good stack
        stmfd   sp!,{r0-r5}                     @@ save some supervisor regs
        sub     r0,lr,#4                        @@ PC at time of interrupt
        mrs     r1,spsr
        mov     r2,#HAL_EXCEPTION_DATA_ACCESS
        mov     r3,sp
        b       call_exception_handler

@@
@@ Dispatch an exception handler.

        .code   32
call_exception_handler:
        @@
        @@ On Entry:
        @@
        @@ r4,r5 = scratch
        @@ r3 = pointer to temp save area
        @@ r2 = vector number
        @@ r1 = exception psr
        @@ r0 = exception pc
        @@ 
        @@ [r3+20]: exception r5
        @@ [r3+16]: exception r4
        @@ [r3+12]: exception r3
        @@ [r3+8] : exception r2
        @@ [r3+4] : exception r1
        @@ [r3]   : exception r0
        
        mrs     r4,cpsr                 @@ switch to Supervisor Mode
        bic     r4,r4,#CPSR_MODE_BITS
        orr     r4,r4,#CPSR_SUPERVISOR_MODE
        msr     cpsr,r4

        mov     r5,sp                   @@ save original svc sp
        @@
        @@ r5 holds original svc sp, current sp is stack to use
        @@

        stmfd   sp!,{r0-r2,r5}          @@ push svc_sp, vector, psr, pc
        
        @@ switch to pre-exception mode to get banked regs
        mov     r0,sp                   @@ r0 survives mode switch
        mrs     r2,cpsr                 @@ Save current psr for return
        orr     r1,r1,#CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE
        msr     cpsr,r1
        stmfd   r0!,{r8-r12,sp,lr}
        msr     cpsr,r2                 @@ back to svc mode
        mov     sp,r0                   @@ update stack pointer

        @@ now save pre-exception r0-r7 on current stack
        ldmfd   r3,{r0-r5}
        stmfd   sp!,{r0-r7}

        @@ sp needs fixing if exception occured in SVC mode.
        ldr     r1,[sp,#armreg_cpsr]
        and     r1,r1,#CPSR_MODE_BITS
        cmp     r1,#CPSR_SUPERVISOR_MODE
        ldreq   r1,[sp,#armreg_svcsp]
        streq   r1,[sp,#armreg_sp]

        @@
        @@ Return from exception
        @@
return_from_exception:

        ldr     r0,[sp,#armreg_cpsr]
        msr     spsr,r0

        @@ return to supervisor mode is simple
        and     r1,r0,#CPSR_MODE_BITS
        cmp     r1,#CPSR_SUPERVISOR_MODE
		ldmeqfd sp,{r0-r14,pc}^

        @@
        @@ return to other non-user modes is a little trickier
        @@

        @@ switch to pre-exception mode and restore r8-r14
        add     r2,sp,#armreg_r8
        mrs     r1,cpsr
        orr     r0,r0,#CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE
        msr     cpsr,r0
        ldmfd   r2,{r8-r14}
        msr     cpsr, r1        @@ back to svc mode

        @@ move sp and pc for final load
        ldr     r0,[sp,#armreg_svcsp]
        str     r0,[sp,#armreg_r8]
        ldr     r0,[sp,#armreg_pc]
        str     r0,[sp,#armreg_r9]

        @@ restore r0-r7,sp and return from exception
        ldmfd   sp,{r0-r7,sp,pc}^

@@ Handle device interrupts
        .code   32
FIQ:
		@@ We can get here from any non-user mode.
        @@@mrs     r8,spsr                 @@ CPSR at time of interrupt
        @@@and     r9,r8,#CPSR_MODE_BITS   @@ isolate pre-interrupt mode
        @@@cmp	r9,#CPSR_IRQ_MODE
        @@@bne	1f
        @@ If FIQ interrupted IRQ mode, just return with FIQ disabled.
        @@ The common interrupt handling takes care of the rest.
        @@@orr	r8,r8,#CPSR_FIQ_DISABLE
        @@@msr	spsr,r8
        @@@subs	pc,lr,#4
    @@@1:
        @@ If FIQ interrupted other non-user mode, switch to IRQ mode and
        @@ fall through to IRQ handler.
        @@@ldr     sp,.__exception_stack   @@ get good stack to save lr and spsr
        @@@stmfd   sp,{r8,lr}
        @@@mov     r8,#CPSR_IRQ_MODE|CPSR_FIQ_DISABLE|CPSR_IRQ_DISABLE
        @@@msr     cpsr,r8			@@ switch to IRQ mode
        @@@ldr     sp,.__exception_stack   @@ get regs saved in FIQ mode
        @@@ldmfd	sp,{sp,lr}
        @@@msr     spsr,sp

        @@ now it looks like we got an IRQ instead of an FIQ except that
        @@ FIQ is disabled so we don't recurse.
IRQ:
        @@ Note: I use this exception stack while saving the context because
        @@ the current SP does not seem to be always valid in this CPU mode.
        ldr     sp,.__exception_stack   @@ get good stack
        stmfd   sp!,{r0-r5}             @@ save some supervisor regs
        sub     r0,lr,#4                @@ PC at time of interrupt
        mrs     r1,spsr
        mov     r2,#HAL_VECTOR_IRQ
        mov     r3,sp
	
handle_IRQ_or_FIQ:

        mrs     r4,cpsr                 @@ switch to Supervisor Mode
        bic     r4,r4,#CPSR_MODE_BITS
        orr     r4,r4,#CPSR_SUPERVISOR_MODE
        msr     cpsr,r4

        mov     r5,sp                   @@ save original svc sp
        stmfd   sp!,{r0-r2,r5}          @@ push svc_sp, vector, psr, pc

        @@ switch to pre-exception mode to get banked regs
        mov     r0,sp                   @@ r0 survives mode switch
        mrs     r2,cpsr                 @@ Save current psr for return
        orr     r1,r1,#CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE
        msr     cpsr,r1
        stmfd   r0!,{r8-r12,sp,lr}
        msr     cpsr,r2                 @@ back to svc mode
        mov     sp,r0                   @@ update stack pointer

        @@ now save pre-exception r0-r7 on current stack
        ldmfd   r3,{r0-r5}
        stmfd   sp!,{r0-r7}

        @@ sp needs fixing if exception occured in SVC mode.
        @@ sp needs fixing if exception occured in SVC mode.
        ldr     r1,[sp,#armreg_cpsr]
        and     r1,r1,#CPSR_MODE_BITS
        cmp     r1,#CPSR_SUPERVISOR_MODE
        ldreq   r1,[sp,#armreg_svcsp]
        streq   r1,[sp,#armreg_sp]

        mov     v6,sp                   @@ Save pointer to register frame

        @@ Switch to interrupt stack
        ldr     r2,.irq_level           @@ current number of nested interrupts
        ldr     r0,[r2]
        add     r1,r0,#1
        str     r1,[r2]                 @@ if was zero, switch stacks
        cmp     r0,#0
        moveq   r1,sp                   @@ save old stack pointer
        ldreq   sp,.__interrupt_stack
        stmeqfd sp!,{r1}
10:


        @@ The entire CPU state is now stashed on the stack,
        @@ increment the scheduler lock and handle the interrupt

        mov     r0,v6
        bl      hal_IRQ_handler         @@ determine interrupt source
        mov     v1,r0                   @@ returned vector #

        mov     r0,v1                   @@ vector #

        cmp     r0,#HAL_INTERRUPT_NONE   @@ spurious interrupt
        bne     10f
        b       spurious_IRQ

10:     ldr     r1,.hal_interrupt_data
        ldr     r1,[r1,v1,lsl #2]       @@ handler data
        ldr     r2,.hal_interrupt_handlers
        ldr     v3,[r2,v1,lsl #2]       @@ handler (indexed by vector #)
        mov     r2,v6                   @@ register frame (this is necessary
                                        @@ for the ISR too, for ^C detection)

        mov     lr,pc                   @@ invoke handler (call indirect
        mov     pc,v3                   @@ thru v3)

spurious_IRQ:


        @@ If we are returning from the last nested interrupt, move back
        @@ to the thread stack. interrupt_end() must be called on the
        @@ thread stack since it potentially causes a context switch.
        ldr     r2,.irq_level
        ldr     r3,[r2]
        subs    r1,r3,#1
        str     r1,[r2]
        ldreq   sp,[sp]         @@ This should be the saved stack pointer

	@@ return from IRQ is same as return from exception
	b	return_from_exception

@@ Dummy/support functions

        .global _psr
        .global _sp

_psr:
        mrs     r0,cpsr
        mov     pc,lr

_sp:
        mov     r0,sp
        mov     pc,lr
@@
@@ Pointers to various objects.
@@
.__startup_stack:			.word __startup_stack
.__exception_stack:			.word __exception_stack
.__undef_exception_stack:	.word __undef_exception_stack
.__bss_start:				.word __bss_start
.__bss_end:					.word __bss_end
._end:						.word _end
.__rom_data_start:			.word __rom_data_start
.__rom_data_end:			.word __rom_data_end
.__ram_data_start:			.word __ram_data_start
.__ram_data_end:			.word __ram_data_end
.hal_interrupt_handlers:	.word hal_interrupt_handlers
.hal_interrupt_data:		.word hal_interrupt_data
.hal_interrupt_objects:		.word hal_interrupt_objects
.__exception_handlers:		.word __exception_handlers
.init_flag:					.word init_flag
.irq_level:					.word irq_level
.__pci_window: 				.word __pci_window
.__interrupt_stack:			.word __interrupt_stack

@@ -------------------------------------------------------------------------
@@ Interrupt vector tables.
@@ These tables contain the isr, data and object pointers used to deliver
@@ interrupts to user code.

@@ Despite appearances, their sizes are not #defines, but .equ symbols
@@ generated by magic without proper dependencies in arm.inc
@@ Recompiling will not DTRT without manual intervention.

        .data

init_flag:
        .balign 4
        .long   0

        @@.extern hal_default_isr

        .globl  hal_interrupt_handlers
hal_interrupt_handlers:
        .rept   HAL_ISR_COUNT
        .long   0	@@hal_default_isr
        .endr

        .globl  hal_interrupt_data
hal_interrupt_data:
        .rept   HAL_ISR_COUNT
        .long   0
        .endr

        .globl  hal_interrupt_objects
hal_interrupt_objects:
        .rept   HAL_ISR_COUNT
        .long   0
        .endr

@@ -------------------------------------------------------------------------
@@ Temporary interrupt stack
        
        .section ".bss"

@@ Small stacks, only used for saving information between CPU modes
__exception_stack_base: 
        .rept   32
        .long   0
        .endr
__exception_stack:
        .rept   32
        .long   0
        .endr
__undef_exception_stack:

@@ Runtime stack used during all interrupt processing
.equ HAL_COMMON_INTERRUPTS_STACK_SIZE, 4096
        .balign 16
        .global interrupt_stack_base
interrupt_stack_base:
__interrupt_stack_base:
        .rept HAL_COMMON_INTERRUPTS_STACK_SIZE
        .byte 0
        .endr
        .balign 16
        .global interrupt_stack
interrupt_stack:
__interrupt_stack:
irq_level:
        .long   0

        .balign 16
__startup_stack_base:
        .rept 512
        .byte 0
        .endr
        .balign 16
__startup_stack:

@@ --------------------------------------------------------------------------
@@  end of vectors.S
