/* console_monitor.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: tep, Aug 18, 00  Initial version ported to IXP1200
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "console_monitor.h"
#include "uart.h"
#include "diag_utils.h"
#include "misc_func.h"

char commandLine[NCMDLINE];
char prevCommandLine[NCMDLINE];
int cmdIndex = 0;
int numasdecimal;
int decimalok;

int getCmdLine(char* cmdLine)
{
	extern void PutChar(char c);

	char c; 
	int i, pos, j, oldestHitNext, upArrow = 0;
	static char cmdBuf[CMD_BUF_SIZE][CMD_LINE_LENGTH + 1];
	static int oldestCmdBuf = 0, nextCmdBuf = 0, currentCmdBuf = 0;
	static int enteredFirstCmd = 0;

	cmdBuf[CMD_BUF_SIZE][CMD_LINE_LENGTH] = '\0';

	// initialization each time in
	PUT32(UART_FCR, 0x1);	// Enable receiver FIFO interrupt
	PUT32(UART_IER, 0x5<<4);	// Enable UART Unit and receiver time out interrupt
	cmdLine[0] = '\0';
	pos = 0;
	upArrow = 0;
	oldestHitNext = 0;


	while(pos < CMD_LINE_LENGTH)
	{
		c = GetChar();
		
		switch(c)
		{
		case '\r':
		case '\n':	PutChar('\n');
					// don't enter blank cmds. into cmdBuf
					if( pos == 0) 
						return 1;

					// Copy current cmd line into history buffer
					i = 0;
		        	while(cmdLine[i] != '\0')
					{
						cmdLine[i] |= 0x20;		// always use lower case
						cmdBuf[nextCmdBuf][i] = cmdLine[i] ;       
						i++;
					}
					cmdBuf[nextCmdBuf][i] = '\0';

					// Sync buffer pointers
					currentCmdBuf = nextCmdBuf; 
					if( enteredFirstCmd && (nextCmdBuf == oldestCmdBuf))
					{
						nextCmdBuf++;
						if( nextCmdBuf >= CMD_BUF_SIZE)
							nextCmdBuf = 0;
						oldestCmdBuf = nextCmdBuf;
					}
					else
					{
						nextCmdBuf++;
						if( nextCmdBuf >= CMD_BUF_SIZE)
							nextCmdBuf = 0;
					}

			        enteredFirstCmd = 1;
			        return 1;
			        // save the data in previous buffer
			        break;

		case 0x1B:  // ESCAPE CHAR
					// check if cache is on, if so, time out if too long
					//TEP         i = 1000;
					
					// MODIFIED START
					i = 10000;	//TEP
					if (get_CP15()&ICACHE_BIT) i = 1000000;
					// MODIFID END
					
					while (i--);
    				if (!Charav()); // if no char is available, it is the escape key
			        else
					{
						c = GetChar();
						if (c != '[');
						else
							c = GetChar();

						switch(c)
						{
            			case 'A':  	// UP ARROW
									if(enteredFirstCmd)
									{ // else bit bucket

						                // print don't decrement currentCmdBuf
						                if( currentCmdBuf == oldestCmdBuf )
										{
											// move cursor to begining of line; don't erase prompt
											while( pos-- > 0 )
											{
												PutChar(BACKSPACE);
												PutChar(' ');
												PutChar(BACKSPACE);
											}
											// copy currentCmdBuf to cmdLine
											j = 0;
											while( cmdBuf[currentCmdBuf][j] != '\0' &&
													( j < CMD_LINE_LENGTH ))
											{
												cmdLine[j] = cmdBuf[currentCmdBuf][j]; 
				                    			j++;
											}
											cmdLine[j] = '\0';
											pos = strlen( cmdLine);
							
											eprintf( "%s", cmdLine);
											upArrow = 0;
				                		}

						                // print and decrement currentCmdBuf
						                else
										{
											// erase line
											while( pos-- > 0)
											{
												PutChar(BACKSPACE);
												PutChar(' ');
												PutChar(BACKSPACE);
											}

											// copy currentCmdBuf to cmdLine
											j = 0;
											while( cmdBuf[currentCmdBuf][j] != '\0' && 
													( j < CMD_LINE_LENGTH ))
											{
												cmdLine[j] = cmdBuf[currentCmdBuf][j];
				                    			j++;
											}
											cmdLine[j] = '\0';
											pos = strlen( cmdLine); 
											--currentCmdBuf;
											if( currentCmdBuf < 0 )
												currentCmdBuf = CMD_BUF_SIZE - 1;
											upArrow = 1;
							
											eprintf( "%s", cmdLine);
  
										}
									}
									break;

						case 'B':	// DOWN ARROW
									if( enteredFirstCmd)
									{
			                			if((( currentCmdBuf + 1 ) % CMD_BUF_SIZE ) == nextCmdBuf )
										{
											// bottom of buffer, blank line for editing
											while( pos-- > 0)
											{
												PutChar(BACKSPACE);
												PutChar(' ');
												PutChar(BACKSPACE);
											}
											cmdLine[0] = '\0';
											pos = strlen( cmdLine); 
											eprintf( "%s", cmdLine);
										}
										else
										{
											// Increment and print new cmd line
											if( upArrow )
											{
												++currentCmdBuf;
												upArrow = 0;
											}
											++currentCmdBuf;
											currentCmdBuf = currentCmdBuf % CMD_BUF_SIZE;
											while( pos-- > 0)
											{
												PutChar(BACKSPACE);
												PutChar(' ');
												PutChar(BACKSPACE);
											}
			                    			j = 0;
											while( cmdBuf[currentCmdBuf][j] != '\0' && 
			                         				( j < CMD_LINE_LENGTH ))
											{
												cmdLine[j] = cmdBuf[currentCmdBuf][j];
												j++;
											}
			                  				cmdLine[j] = '\0';
			                  				pos = strlen( cmdLine);
			                  				if( currentCmdBuf == nextCmdBuf )
			                    				if( --currentCmdBuf < 0 )
			                      					currentCmdBuf = CMD_BUF_SIZE - 1;
											eprintf( "%s", cmdLine);
			                  			}
			              			}
			            			break;

			            case 'C': 	// RIGHT ARROW									
			            case 'D': 	// LEFT ARROW									
			            case 'H': 	// HOME
			            case 'J': 	// PAGE DOWN
			            case 'K': 	// END
			            case 'M': 	// PAGE UP
			            default: 	break;
						}	// switch(c)
					}
      				break;
		case 0x7F:	eprintf( "\nDEL\n");
      				break;
		case '\b':	// BACKSPACE
        			if( pos > 0)
					{
						PutChar(BACKSPACE);
						PutChar(' ');
						PutChar(BACKSPACE);
						pos--;
						cmdLine[pos] = '\0';
					}
      				break;
		default:	
                    PutChar(c);
					cmdLine[pos++] = c;
					cmdLine[pos] = '\0';
					break;
		}	// switch(c)
	}
	return 1;
}
