/* diag_utils.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Feb 05, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "hal_ixdp2400.h"
#include "hal_io.h"
#include "uart.h"
#include "misc_func.h"
#include "syslog.h"
#include "console_monitor.h"
#include "led.h"

extern int serial_osc;
extern UINT32 t_all_verbose;
extern UINT32 silicon_rev;
int slowport_code[16];

#define ICACHE_BIT 1<<12
#define DCACHE_BIT 1<<2

// ***** NOTE *****
// From the function flush() (line 197) and below
// the codes are re-used from IXP1200

//
// Function: Set_LED
//
// Description: Display 4 chars according to the user input
//
void Set_LED(unsigned char* buffer)
{
	int i, loop;
	
	loop = strlen(buffer);

	if(((silicon_rev & 0xF0) >> 4))
	{
		if(loop >= 4)
		{
			for(i = 0; i < 4; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + i, (buffer[i] & 0x7F));
			}
		}
		else
		{
			for(i = 0; i < loop; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + i, (buffer[i] & 0x7F));
			}

			for(; i < 4; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + i, ASCII_SPACE);
			}
		}
	}
	else
	{
		if(loop >= 4)
		{
			for(i = 0; i < 4; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + 3 - i, (buffer[i] & 0x7F));
			}
		}
		else
		{
			for(i = 0; i < loop; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + 3 - i, (buffer[i] & 0x7F));
			}

			for(; i < 4; i++)
			{
				PUT8(ALPHANUM_DIS_DATA_REG + 3 - i, ASCII_SPACE);
			}
		}
	}
}

//
// Function: dump_slowport
//
// Description: use for logging puposes. Dumps cod[] onto
//				the slowport
//
void dump_slowport(int code[], int numInt, int forever)
{
	int i;
	do
	{
		for(i = 0; i < numInt ; i++)
		{
			PUT32(SLOW_PORT_BASE + 0x1800000, code[i]);	// *TODO*: remove magic number
		}
	} while(0);//forever); 
}

//
// Function: Enable_CPSR_Int
//
// Description: Enables the FIQ and IRQ in CPSR by clearing bit 6 and 7
//
void Enable_CPSR_Int(void)
{
	asm("mrs	r0, cpsr;"
        "bic	r0, r0, #0x40;" // enable FIQ
		"bic	r0, r0, #0x80;" // enable IRQ
        "msr	cpsr, r0;"
        );
}

//
// Function: Disable_CPSR_Int
//
// Description: Disables the FIQ and IRQ in CPSR by clearing bit 6 and 7
//
void Disable_CPSR_Int(void)
{
	asm("mrs	r0, cpsr;"
        "orr	r0, r0, #0x40;" // disable FIQ
		"orr	r0, r0, #0x80;" // disable IRQ
        "msr	cpsr, r0;"
        );
}

//
// Function: get_CP15
//
// Description: Gets the value for Co-Processor 15
//
unsigned int get_CP15(void)
{
    unsigned int x = 0;

	asm("mrc p15,0,%0,c1,c0,0;" : "=r"(x) :);

    return x;
}

//
// Function: icache_on
//
void icache_on(void)
{
	if( (get_CP15() & (ICACHE_BIT)) != 0)
		eprintf("Instruction cache is already enabled\n");
	else
	{
		eprintf("Enabling Instruction cache\n");
		// Enable Icache by writing a 1 to bit 12 of the CP15 - control register
		asm("mrc  p15,0,r1,c1,c0,0;"
            "orr  r1,r1,#0x1000;" // enable ICache
            "mcr  p15,0,r1,c1,c0,0;"
			"mcr    p15,0,r1,c7,c5,0;"  // invalidate instruction cache
			);
	}
}

//
// Function: icache_off
//
void icache_off(void)
{
	eprintf("Disabling Instruction cache\n");	
	asm("mrc    p15,0,r1,c1,c0,0;"
        "bic    r1,r1,#0x1000;" // disable Icache
        "mcr    p15,0,r1,c1,c0,0;"
        "mcr    p15,0,r1,c7,c5,0;"  // invalidate instruction cache
        );
}

//
// Function: dcache_on
//
void dcache_on(void)
{
	if( (get_CP15() & DCACHE_BIT) != 0)
		eprintf("Data cache is already enabled\n");
	else
	{
		if(t_all_verbose != MINIMAL)
			eprintf("Enabling Data cache\n");
		// Enable data cahce and MMU by writing 1 to bits 0 and 2
		asm("mrc  p15,0,r1,c7,c10,4;"   // drain write buffer
			// cpuwait
	        "mrc    p15,0,r1,c2,c0,0;"  // arbitrary read
	        "mov    r1,r1;"
	        "sub    pc,pc,#4;"
	        "nop;"
		    "mrc  p15,0,r1,c1,c0,0;"
        	"orr  r1,r1,#0x0007;"  // enable DCache (also ensures the 
                               	   // MMU and alignment faults are    
                               	   // enabled)                       
        	"mcr  p15,0,r1,c1,c0,0;"
			"mrc  p15,0,r1,c2,c0,0;"  // arbitrary read
        	"mov  r1,r1;"
        	"sub  pc,pc,#4;"
        	"mcr  p15,0,r1,c7,c6,0;" // invalidate data cache
			// cpuwait
	        "mrc    p15,0,r1,c2,c0,0;"  // arbitrary read
	        "mov    r1,r1;"
	        "sub    pc,pc,#4;"
			"nop"
           );
	}
}

void dcache_sync(void)
{
    /* The best way to evict a dirty line is by using the          */
    /* line allocate operation on non-existent memory.             */
    asm("mov    r0, #0x70000000;"   /* use upper 256M of SDRAM for cache flush region */
        "add    r1, r0, #0x8800;"   /* 32KB cache + 2KB mini cache */
 		"667: "
        "mcr    p15,0,r0,c7,c2,5;"  /* allocate a line    */
        "add    r0, r0, #32;"       /* 32 bytes/line      */
        "teq    r1, r0;"
        "bne    667b;"
        "mcr    p15,0,r0,c7,c6,0;"  /* invalidate data cache */
        /* cpuwait */
        "mrc    p15,0,r1,c2,c0,0;"  /* arbitrary read   */
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
        "mcr    p15,0,r0,c7,c10,4;" /* and drain the write buffer */
        /* cpuwait */
        "mrc    p15,0,r1,c2,c0,0;"  /* arbitrary read   */
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
        "nop"
        :
        :
        : "r0","r1"      /* Clobber list */
        );
}

//
// Function: dcache_off
//
void dcache_off(void)
{
	if(t_all_verbose != MINIMAL)
		eprintf("Disabling Data cache\n");	

	dcache_sync();

	asm("mrc  p15,0,r1,c1,c0,0;"
        "bic  r1,r1,#0x0007;"  // disable DCache (also ensures the
                               // MMU and alignment faults are
                               // disabled)
        "mcr  p15,0,r1,c1,c0,0;"
        // cpuwait
        "mrc    p15,0,r1,c2,c0,0;"  // arbitrary read
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
        "mcr    p15,0,r1,c7,c6,0;" // invalidate data cache
        // cpuwait
        "mrc    p15,0,r1,c2,c0,0;"  // arbitrary read
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
		"mcr    p15,0,r0,c7,c10,4;" // and drain the write buffer
        // cpuwait
        "mrc    p15,0,r1,c2,c0,0;"  // arbitrary read
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
        "nop"
		);
}

//
// Function: dcache_flush
//
void dcache_flush(void)
{
	asm("mcr	p15,0,r0,c7,c10,4;"	// drain the write buffer
	    "mcr	p15,0,r1,c7,c6,0;"	// drain the data cache
	    "mrc    p15,0,r1,c2,c0,0;"  /* arbitrary read   */
        "mov    r1,r1;"
        "sub    pc,pc,#4;"
        "nop"
		);
}

//
// Function: flush
//
void flush(void)
{                               /* flush input buffer */
  while (Charav());
}


//*******************************
//* Functions used by cmd_print *
//*******************************

static void PutDotChar(char c)
{
	PutChar ((c >= 0x20 && c < 0x7f) ? c : '.');
}

static void PrintLine(int size, int addr)
{
	int i;	
	volatile int place = addr;

	eprintf ("%08x: ", place);
	if (size == 0)
	{
		place &= 0xfffffffc;
		for (i = 0; i < 4; i += 1)
		{
			eprintf("%08x ", GET32 (place));
			place += 4;
		}
	}

	if (size == 1)
	{
		place &= 0xfffffffe;
		for (i = 0; i < 8; i += 1)
		{
			eprintf("%04x ", GET16 (place));
			place += 2;
		}
	}

	if (size == 2)
	{
		place &= 0xffffffff;
		for (i = 0; i < 16; i += 1)
		{
			eprintf("%02x ", GET8 (place));
			place += 1;
		}
	}

	for (i = 0; i < 16; i += 1) PutDotChar (GET8(addr + i) & 0xff);
	eprintf("\n");
}

static void PrintPlace(int size, int place, volatile int tplace)
{
	eprintf ("%08x ", place);
	if (size == 0)
		eprintf ("%08x: ", GET32(tplace));
	if (size == 1)
		eprintf ("%04x: ", GET16(tplace));
	if (size == 2) 
		eprintf ("%02x: ", GET8(tplace));
}

static void PrintMem(int size, int first, int last)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	first &= 0xfffffff0;
	while (first <= last)
	{
		PrintLine (size, first);
		first += 16;
	}
	acL->defAddress =(int) first;
}

//
// Function: cmd_print
//
// Description:
//
void cmd_print(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	long first, size;

	first = (acL->argc < 2) ? acL->defAddress : acL->hexArg[1];
	size  = (acL->argc < 3) ? 0x80 : acL->hexArg[2];
	PrintMem(acL->numArgs, first, (first + size));
}

/*---------------------------------------------------------------------------
  routine: 
        This routine will interactively change system memory.  
  params:
        size: byte, word, or intword
        place: physical location to change
  returns:
        void
 ----------------------------------------------------------------------------*/
static void ChangeMem(int size, int place)
{
	char    c;
	int     value = 0;
	int		string_count = 0;
	volatile int tplace;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if (size == 0) place &= 0xfffffffc;   // mask off for longword (32 bits)
	if (size == 1) place &= 0xfffffffe;   // mask off for short (16 bits)
	if (size == 2) place &= 0xffffffff;   // mask off for byte (8 bits)
	eprintf("place: %x\n");

	tplace = place;
	PrintPlace(size, place, tplace);      // show current location
	while(1)
	{                            // stay in loop 'til exit command
		c = GetChar();                  // get input from parser

		if (c == (char)0x1B)	// ESCAPE
		{
			eprintf("\n");
			return;
		}

		if (c == '\b')		// BACKSPACE
		{
			PutChar(c);
			string_count--;
			value = value>>4;
		}

		if (c == '-')	// if '-' go backwards by size
		{
			if (size == 0) place -= 4;        // go backwards by ints
			if (size == 1) place -= 2;        // go backwards by shorts
			if (size == 2) place -= 1;        // go backwards by bytes
			tplace = place;
			eprintf("\n");
			PrintPlace(size, place, tplace);
			string_count = 0;
			value = 0;
			continue;
		}
		if (c == '\r')	// if <CR> go forwards
		{
			if(string_count != 0)
			{
				if (size == 0) 
			      PUT32(tplace, value);  // update location with int data
				if (size == 1) 
				      PUT16(tplace, value);
				if (size == 2) 
				      PUT8(tplace, value); // update location with byte data
				eprintf("\r");
			}
			else
			{
				if (size == 0) place += 4;        // go forwards by ints
				if (size == 1) place += 2;        // go forwards by shorts
				if (size == 2) place += 1;        // go forwards by bytes
				tplace = place;
				eprintf("\n");
			}
			PrintPlace(size, place, tplace);
			string_count = 0;
			value = 0;
			continue;
		}
		/*
		*  check if the value is hex, if so, then convert it to something
		*  we understand... 
		*/
		c = tolower (c);
		if (isdigit (c) || ((c >= 'a') && (c <= 'f')))
		{
			if(string_count < 8)
			{
				PutChar(c);//UnGetChar (c);
				if(c <= '9')
					value = (value<<4) + (int)(c - '0');//ReadHex ();
				else
					value = (value<<4) + (int)(c - 'a' + 10);
				string_count++;
			}
			continue;
		}
	}
	acL->defAddress = (int)place;          // keep track of where we are
}

void cmd_change(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	long base;

	base = (acL->argc < 2) ? acL->defAddress : acL->hexArg[1];
	ChangeMem(acL->numArgs, base);
}

/*---------------------------------------------------------------------------
  routine: 
        This routine will fill a block of system memory with the value 
        that is passed into the routine. It also keeps track of where it 
        last filled.
  params:
        first: starting physical address of block to move
        last:  last physical address of block to move
        value: the constant value that will be used to fill the block
  returns:
        void
 ----------------------------------------------------------------------------*/
static void FillMem(int first, int last, unsigned value)
{
	register unsigned int  i;    /* local variables used in fill */
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	first &= 0xfffffffc;         /* mask off to int word */
	for (i=first;i<last;i+=4) 
	PUT32 (i, value);        /* fill the entire block */
#ifdef A0_REV
	if(i >= 0xE0000000)	// if accessing PCI, then perform a read back
		GET32(i);
#endif
	acL->defAddress = (int)last;  /* save the last fill location */
}

void cmd_fill(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	FillMem(acL->hexArg[1], acL->hexArg[2], acL->hexArg[3]); 
}


/*---------------------------------------------------------------------------
  routine:
        This routine will move a block of system memory from one physical 
        location to another physical location.
  params:
        first: starting physical address of block to move
        last:  last physical address of block to move
        destination: first physical address of where to move the block
  returns:
        void
 ----------------------------------------------------------------------------*/
static void MoveBlock(int first, int last, int dest)
{
	register int i;              /* local variables used in copy*/
	register int temp;

	first &= 0xfffffffc;         /* mask off to int word */
	for (i=first;i<last;i+=4)
	{/* copy all int words to dest */
		temp = GET32 (i); /* read the source data */
		PUT32(dest, temp); /* write the data to dest */
#ifdef A0_REV
		if(dest >= 0xE0000000)	// if accessing PCI, then perform a read back
			GET32(dest);
#endif
		dest += 4;                 /* increment pointer */
	}
}

void cmd_blockmove(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	MoveBlock(acL->hexArg[1], acL->hexArg[2], acL->hexArg[3]); 
}

void cmd_wrv(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned int *pntr = (unsigned int *)acL->hexArg[1];
	register unsigned int x,times = acL->decArg[2];
	unsigned volatile int tmp;
	unsigned volatile int v[3];

	for (x=0;x<times;x++)
		pntr[x] = x;

	for (x=0;x<times;x++)
	{
		if ((tmp=pntr[x]) != x)
		{
			v[0] = pntr[x];
			v[1] = pntr[x+1];
			v[2] = pntr[x+2];
			eprintf("ouch ! bad data addr: %08x, act: %08x, exp: %08x, xor: %08x\n",
			&pntr[x],tmp,x,x^tmp);
	  		eprintf(" more data: %08x %08x %08x\n",v[0],v[1],v[2]);
	  		return ;
		}
	}
}


//*********************************************
//* PCI Memory space read and write functions *
//*********************************************
/*static unsigned int getPCIaddrext(unsigned int memaddr)
{
	if( ((memaddr & PCI_MEM_PAGE_MASK) >> 29) <= 7)
	{
		return( (((memaddr & PCI_MEM_PAGE_MASK)>>29)<<13) );
	}
	else
	{
		return 0;
	}
}
*/

void cmd_memrb(void)
{
	UINT32 memloc, ans;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;
    
	do
	{
		if (acL->decArg[3])
		{
	  		ans = GET8((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE);
		}
		else
		{
	  		eprintf("0x%08x: 0x%02X\n", memloc, GET8((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[2] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


void cmd_memwb(void)
{
	UINT32 value, memloc;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;

	do
	{
		if (acL->decArg[4]) 
		{
			PUT8(((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE), value);
		}
		else
		{ 
			eprintf("Old: 0x%08x: 0x%02x\n", memloc, GET8((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
			PUT8(((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE), value);
			eprintf("New: 0x%08x: 0x%02x\n", memloc, GET8((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[3] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


void cmd_memrw(void)
{
	UINT32 memloc, ans;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;
    
	do
	{
		if (acL->decArg[3]) 
		{
			ans = GET16((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE);
		}
		else 
		{
			eprintf("0x%08x: 0x%04x\n", memloc, GET16((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[2] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


void cmd_memww(void)
{
	UINT32 value, memloc;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;
        
	do
	{
		if (acL->decArg[4])
		{
			PUT16((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE, value);
		}
		else
		{
			eprintf("Old: 0x%08x: 0x%04x\n", memloc, GET16((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
			PUT16(((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE), value);
			eprintf("New: 0x%08x: 0x%04x\n", memloc, GET16((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[3] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


void cmd_memrd(void)
{
	UINT32 memloc, ans;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;
        
	do
	{
		if (acL->decArg[3])
		{
			ans = GET32((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE);
		}
		else 
		{
			eprintf("0x%08x: 0x%08x\n", memloc, GET32((memloc & ~PCI_MEM_PAGE_MASK) | XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[2] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


void cmd_memwd(void)
{
	UINT32 value, memloc;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = (memloc & PCI_MEM_PAGE_MASK) >> 16;
        
	do
	{
		if(acL->decArg[4]) 
		{
			PUT32((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE, value);
		}
		else
		{ 
			eprintf("Old: 0x%08x: 0x%08x\n", memloc, GET32((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
			PUT32(((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE), value);
			eprintf("New: 0x%08x: 0x%08x\n", memloc, GET32((memloc & ~PCI_MEM_PAGE_MASK) + XSCALE_PCI_MEM_BASE));
		}
	}
	while (acL->decArg[3] && !(KEYPRESS));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}

//*****************************************
//* Functions used by PCI IO space access *
//*****************************************
/*
static void outportb(unsigned int port, unsigned int data)
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    PUT8((XSCALE_PCI_IO_BASE + port),data&0xFF);
}

static void outportw(unsigned int port, unsigned int data)
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    PUT16((XSCALE_PCI_IO_BASE + port),data&0xFFFF);
}    

static void outportl(unsigned int port, unsigned int data)
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    PUT32((XSCALE_PCI_IO_BASE + port),data);
}    

static unsigned int inportb(unsigned int port)  
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    return (GET8(XSCALE_PCI_IO_BASE + port) & 0xFF);
}

static UINT inportw(unsigned int port)  
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    return (GET16(XSCALE_PCI_IO_BASE + port) & 0xFFFF);
}

static UINT inportl(unsigned int port)  
{
	PUT32(PCI_ADDR_EXT_REG, (GET32(PCI_ADDR_EXT_REG) & 0xFF) |
							((XSCALE_PCI_IO_BASE + port)<<16));
    return (GET32(XSCALE_PCI_IO_BASE + port));
}
*/

//*********************************
//* PCI IO space access functions *
//*********************************
void cmd_iorb(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int ans, memloc;

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[3])
		{
			eprintf("0x%08X:  0x%02X\n", memloc, GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			ans = GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE);
		}
	}
	while (acL->hexArg[2]&&!(KEYPRESS));
}


void cmd_iowb(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int memloc, value;

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[4])
		{
			eprintf("Old: 0x%08X:  0x%02X\n", memloc, GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
			PUT8(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
			eprintf("New: 0x%08X:  0x%02X\n", memloc, GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			PUT8(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
		}
	}
	while (acL->hexArg[3]&&!(KEYPRESS));
}


void cmd_iorw(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int ans, memloc;

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[3])
		{
			eprintf("0x%08X:  0x%04X\n", memloc, GET16((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			ans = GET16((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE);
		}
	}
	while (acL->hexArg[2]&&!(KEYPRESS));
}

void cmd_ioww(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int memloc, value;

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[4])
		{
			eprintf("Old: 0x%08X:  0x%04X\n", memloc, GET16((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
			PUT16(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
			eprintf("New: 0x%08X:  0x%04X\n", memloc, GET16((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			PUT16(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
		}
	}
	while (acL->hexArg[3]&&!(KEYPRESS));
}

void cmd_iord(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int ans, memloc;

	memloc = acL->hexArg[1];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[3])
		{
			eprintf("0x%08X:  0x%08X\n", memloc, GET32((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			ans = GET32((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE);
		}
	}
	while (acL->hexArg[2]&&!(KEYPRESS));
}

void cmd_iowd(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int memloc, value;

	memloc = acL->hexArg[1];
	value = acL->hexArg[2];

	flush();

	*(PCI_ADDR_EXT_REG) = memloc & PCI_IO_PAGE_MASK;

	do
	{
		if (!acL->hexArg[4])
		{
			eprintf("Old: 0x%08X:  0x%08X\n", memloc, GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
			PUT32(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
			eprintf("New: 0x%08X:  0x%08X\n", memloc, GET8((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE));
		}
		else 
		{
			PUT32(((memloc & ~PCI_IO_PAGE_MASK) + XSCALE_PCI_IO_BASE), value);
		}
	}
	while (acL->hexArg[3]&&!(KEYPRESS));
}


//**********************************************
//* Functions used by PCI configuration access *
//**********************************************
static void outcfgb(UINT bus, UINT s, UINT f, UINT p, UINT d)
{
	UINT32 *paddr;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		PUT8(paddr, d);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		PUT8(paddr, d);
	}
}

static void outcfgw(UINT bus, UINT s, UINT f, UINT p, UINT d)
{
	UINT32 *paddr;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		PUT16(paddr, d);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		PUT16(paddr, d);
	}
}

static void outcfgl(UINT bus, UINT s, UINT f, UINT p, UINT d)
{
	UINT32 *paddr;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		PUT32(paddr, d);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		PUT32(paddr, d);
	}
}

static unsigned int incfgb(UINT bus, UINT s, UINT f, UINT p)
{
	UINT32 *paddr;
	register UINT tmp=0;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		tmp = GET8(paddr);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		tmp = GET8(paddr);
	}
	return tmp;
}

static unsigned int incfgw(UINT bus, UINT s, UINT f, UINT p)
{
	UINT32 *paddr;
	register UINT tmp=0;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		tmp = GET16(paddr);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		tmp = GET16(paddr);
	}
	return tmp;
}

static unsigned int incfgl(UINT bus, UINT s, UINT f, UINT p)
{
	UINT32 *paddr;
	register UINT tmp=0;

	// Offsets must be dword-aligned
    p &= ~3;

	if(bus == PRIMARY_BUS_NUM)
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_0_CONFIG_BASE;
		(UINT)paddr |= ((1 << (s + 16)) | (f << 8) | p | 0);
		tmp = GET8(paddr);
	}
	else
	{
		paddr = (UINT32 *)XSCALE_PCI_TYPE_1_CONFIG_BASE;
		(UINT32)paddr |= ((bus << 16) | (s << 11) | (f << 8) | p | 1);
		tmp = GET8(paddr);
	}
	return tmp;
}


//********************************************
//* PCI Configuration space access functions *
//********************************************
void cmd_cfgrb(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf("%02x\n",incfgb(acL->decArg[1],acL->decArg[2],0,acL->hexArg[3])&0xff);
}

void cmd_cfgwb(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	outcfgb(acL->decArg[1],acL->decArg[2],0,acL->hexArg[3],acL->hexArg[4]&0xff);
}

void cmd_cfgrw(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf("%04x\n",incfgw(acL->decArg[1],acL->decArg[2],0,acL->hexArg[3])&0xffFF);
}

void cmd_cfgww(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	outcfgw( acL->decArg[1],acL->decArg[2],0,acL->hexArg[3],acL->hexArg[4] & 0xffff);
}

void cmd_cfgrd(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf("%08x\n",incfgl(acL->decArg[1],acL->decArg[2],0,acL->hexArg[3]));
}

void cmd_cfgwd(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	outcfgl(acL->decArg[1],acL->decArg[2],0,acL->hexArg[3],acL->hexArg[4]);
}

//
// Function: cmd_setbaud
//
// Description: allows the user to set the baudrate
//
void cmd_setbaud(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned char dlrl, dlrh;
	int baudrate;	

	if(acL->argc != 2)
	{
		eprintf("Usage of command setbaud:\n");
		eprintf("\t setbaud <d:baudrate>\n");
		eprintf("\t Baudrates supported:\n");
		eprintf("\t 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200\n");
	}
	else
	{
		PUT8(UART_LCR, (GET8(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0
		PUT8(UART_IER, (GET8(UART_IER) & 0xBF));	// Disable UART
		PUT8(UART_LCR, (GET8(UART_LCR) | (1<<7)));	// Bit 7 = 1; DLAB = 1

		baudrate = acL->decArg[1];
		dlrl = ((serial_osc/baudrate)/16)&0xFF;
		dlrh = ((serial_osc/baudrate)/16) >>8;
		PUT32(UART_DLRL, dlrl);
		PUT32(UART_DLRH, dlrh);

		PUT8(UART_LCR, (GET8(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0
		PUT8(UART_IER, (GET8(UART_IER) | (1<<6)));	// Enable UART
	}
}

//
// Function: cmd_viewlog
//
//Description: Calls the syslog_show function which will display the syslog
//
void cmd_viewlog(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int no_of_entries;

	no_of_entries = acL->decArg[1];
	if(acL->Verboselevel == LOG_OFF)
		eprintf("Syslog Disabled.\n");

	syslog_show(no_of_entries);
}

//
// Function: cmd_set_verbose
//
// Decription: allows the user to set the verbose level
//
void cmd_set_verbose(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if((acL->argc < 2) || (acL->decArg[1] < 0) || (acL->decArg[1] > 5))
	{
		eprintf("setverbose <verbose_level>, where\n");
		eprintf("\t <verbose_level> are:\n");
		eprintf("\t 0x0 - DISABLE\n");
		eprintf("\t 0x1 - TYPE_ERROR\n");
		eprintf("\t 0x2 - TYPE_WARNING\n");
		eprintf("\t 0x3 - TYPE_DBGMSG1\n");
		eprintf("\t 0x4 - TYPE_DBGMSG2\n");
		eprintf("\t 0x5 - TYPE_DBGMSG3\n");
	}
	else
	{
		acL->Verboselevel = acL->decArg[1];	
		syslog_chg_verbose(acL->decArg[1]);
	}
}

//
// Function: set_hoe
//
// Description: allows the user to enable/disable the Halt On Error
//
void set_hoe(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->decArg[1] == ENABLED)
		acL->HaltOnError = ENABLED;
	else
		acL->HaltOnError = DISABLED;
}

//
// Function: cmd_reset
//
// Description: Resets the board
//
void cmd_reset(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == SLAVE)
	{
		eprintf("Please exit from MASTER.\n");
		return;
	}
	PUT32(IXP_RESET0, RSTALL);
}

//
// Function: Charav
//
// Description: Checks if there's a char on the UART
//
int Charav(void)
{
	volatile int temp;

	temp = GET8(UART_LSR) & LSR_DR;

	return(temp);
}

//
// Function: GetChar
//
// Description: gets a char from the UART
//
char GetChar(void)
{
	while (!Charav());   // wait until character is available

	return (GET8(UART_RBR));
}

//
// Function: PutChar
//
// Description: Puts a char onto the UART
//
void PutChar(char c)
{
	while ((GET8(UART_LSR) & LSR_TEMT) == 0);
	PUT8(UART_THR, c);
	if (c == '\n')
		PUT8(UART_THR, '\r');
	
}
