/* msf_init.c
 *
 * Contains functions for initialising the MSF
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, May 27, 2002
 *  revisions: 
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "msf_init.h"
#include "common.h"
#include "diagstruct.h"
#include "hal_ixdp2400.h"
#include "led.h"

void Msf_OutOfReset(void)
{
	UINT32 reset0_reg;

	// reset MSF
	reset0_reg = GET32(0xC0004A0C);
	reset0_reg = reset0_reg | (1 << 7);
	PUT32(0xC0004A0C, reset0_reg);

	// Bring MSF out of reset
	reset0_reg = GET32(0xC0004A0C);
	reset0_reg = reset0_reg & ~(1 << 7);
	PUT32(0xC0004A0C, reset0_reg);

	hal_delay_us(500000);

	eprintf("MSF out of reset...\n");

	return;
}

int Msf_PLL()
{
	volatile UINT32 temp_val;
	UINT32 freq;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if (acL->HostType == MASTER)
	{
		freq = (UINT32)MSF_CLKCFG_EG;
	}
	else
	{
		freq = (UINT32)MSF_CLKCFG_IN;
	}

	temp_val = ((UINT32)MCCR_MASK1 | (freq << 16));
	PUT32(0xC0004A08, temp_val);

	temp_val = ((UINT32)MCCR_MASK2 | (freq << 16));
	PUT32(0xC0004A08, temp_val);

	temp_val = ((UINT32)MCCR_MASK3 | (freq << 16));
	PUT32(0xC0004A08, temp_val);

	hal_delay_us(1000000);		// delay 1s

	if (((GET32(0xC0004A08) >> 28) != 0xF) && (((GET32(0xC0004A08) >> 28) & 0xC) != 0xC) && (((GET32(0xC0004A08) >> 28) & 0x3) != 0x3))
	{
		eprintf("PLL Reg val:0x%X\tExpected val:0x%X or 0x%X...\n", GET32(0xC0004A08), 0x50000000, 0xF0000000);
		eprintf("PLL setup failed...\n");
		return ERROR;
	}
	else
	{
		eprintf("PLL setup completed...\n");
	}

	return OK;
}
