/* diagstruct.h
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 16, 02
 * 
 * --------------------------------------------------------------------------
 */
#ifndef _DIAGSTRUCT_H
#define _DIAGSTRUCT_H

#include "common.h"

#define ACADDRESS 0xF0000	// store diagstruct at 960Kb

#define MASTER 	1
#define SLAVE 	0

#define SINGLE_FLASH 	1
#define DUAL_FLASH		2

/*flag to perform a test as part of T all*/
#define TFLAG_SRAM   	0
#define TFLAG_SDRAM  	1
#define TFLAG_UENG   	2
#define TFLAG_TIMERS 	3
#define TFLAG_ENET   	4
#define TFLAG_PCI  		5
#define TFLAG_XSCALE  	6
#define TFLAG_I2C   	7
#define TFLAG_GPIO 		8
#define TFLAG_LED   	9
#define TFLAG_UART   	10
#define TFLAG_SF  		11
#define TFLAG_LOOPBACK 	12
#define TFLAG_TMON 		13
#define TFLAG_FLASH		14
#define TFLAG_LINERATE	15

// flags used by "t all"
#define MAXIMUM	0x1
#define MINIMAL	0x2


#define MAX_PCI_DEVICE 32	/* for now we support up to ??? pci devices total */

typedef struct _DiagCommon {
	char *targetName; 		/* system platform name */
	char *targetCpu;  		/* system cpu */
	int version;			/* Diag version number */
	int argc;				/* parsed arguement count */
	int SDRAMSize;			/* size of system SDRAM memory */
	int SRAMSize;			/* size of system SRAM memory */
	int UStoreSize;			/* size of microstore memory */
	int CPURevision;		/* Silicon revision */
	int CPUSpeed;			/* Core cpu speed */
	int HostType;			/* NPU Host Type: Master or Slave */
	int display_verbose;	/* flag to determine if test all is selected */
	int hexArg[15];			/* hexidecimal arguments */
	int hexArgOk[15];		/* valid flag for hex args */
	int decArg[15];			/* decimal arguments */
	int decArgOk[15];		/* valid flags for dec args */
	int tFlag[20];			/* debug flags for DIAG */
	int defAddress;			/* default address of last access */
	char *argv[10];			/* convert cmdline into strings */
	int numArgs;			/* number of fixed arguments */
	char cmdLine[128];		/* command line parser buffer */
	int Verboselevel; 		/*This is used by the logging function*/
	int HaltOnError; 		/* This is a flag that certain tests should check*/
	int Err;
} DiagCommon, *PDiagCommon;

//DiagCommon *ac;		/* declare this for the world to use */


typedef struct {		/* declare the command structure */
  char *cmdw;		/* command string used in compare */
  char *syntax;		/* parser syntax for command */
  int hflag;		/* help flag to group commands */
  void (*routine)(void);	/* routine to call when match */
  int numArg;		/* number of arguments expected */
  char *helpstr;		/* help string when user requests it */
} DiagCmd, *PDiagCmd;

#endif
