/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: lagarwal $
*   DATE:           $Date: 2002/11/15 17:47:44 $
*   LAST MODIFIED:  $Modtime: 5/17/02 11:09a $
*   REV#:           $Revision: 1.1.1.1 $
*   LOGFILE:        $Logfile: /Embedded/source/linecards/batteryDW/bdwtests.c $
*   WORKFILE:       $Workfile: bdwtests.c $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose: 
*
* Notes:
*			
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log:
* --------------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include "common_types.h"
#include "common_def.h"
#include "common.h"
#include "hal.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "diagstruct.h"
#include "bdw.h"
#include "ixf_api_sonet_d.h"
#include "ixf6048d.h"
#include "ixf6048.h"
#include "btables.h"
#include "bdwtests.h"
#include "i2c.h"
#include "i2c_test.h"
#include "bdwcommon.h"
#include "hal_ixdp2400.h"
#include "led.h"
#include "syslog.h"
#include "memory.h"
#include "common_types.h"
#include "common_def.h"
#include "bkcompat.h"
#include "diag_utils.h"
#include "ixf6048f.h"
#include "uart.h"

/***********************************************************************
****                       INTERNAL ROUTINES                        ****
***********************************************************************/


/******************************************************************
* Function : TurnPRBSOn
*-----------------------------------------------------------------
* Description  : This routine places the device in PRBS mode and 
*                enable PRBS interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid pointer to the device.
*           chanNum : Specifies the channel number.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
TurnPRBSOn(bb_ChipData_t *pChipData, int chanNum)
{
    bb_ChipSegment_t section;
    ushort regVal;

    /* Assign the channel number */
    section.group.channel = chanNum;

    section.block.flow = BOTH_FLOWS;

    /* Start PRBS Test */
    IxfApiSetOpMode(pChipData, &section, TEST_MODE, NULL);

    /* Clear PRBS Interrupts */
    IxfApiGenericRead(pChipData, TWO_BYTES, PRBSINT, 1, &regVal);

    /* Enable PRBS Interrupts */
    //IxfApiGenericRead(pChipData, TWO_BYTES, PRBSINTEN, 1, &regVal);
    regVal = (1 << chanNum);
    IxfApiGenericWrite(pChipData, TWO_BYTES, PRBSINTEN, 1, &regVal);
}

/******************************************************************
* Function : TurnPRBSOff
*-----------------------------------------------------------------
* Description  : This routine disables PRBS interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid pointer to the device.
*           chanNum : Specifies the channel number.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
TurnPRBSOff(bb_ChipData_t *pChipData, int chanNum)
{
    ushort regVal;

    /* Disable PRBS Interrupts */
    IxfApiGenericRead(pChipData, TWO_BYTES, PRBSINTEN, 1, &regVal);
    regVal &= ~(1 << chanNum);
    IxfApiGenericWrite(pChipData, TWO_BYTES, PRBSINTEN, 1, &regVal);
}


/******************************************************************
* Function : DumpRegAccessResults
*-----------------------------------------------------------------
* Description  : This routine displays the results of the register
*                access tests.
*-----------------------------------------------------------------
* Inputs  : status : The results of the register access test.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
DumpRegAccessResults(BDW_Status_t status)
{
    // prepare structure for dumping
    syslog.source_comp = MASTER_BATTERY_DIAG;

    switch(status)
    {
    case BDW_FAIL:
        syslog.type = TYPE_ERROR;
        Set_LED("FAIL");
        // prepare structure for dumping
        strcpy(syslog.desc, "Register Access Test Failed");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("REGISTER ACCESS TEST FAIL!!!\n");
        break;
    case BDW_SUCCESS:
        syslog.type = TYPE_ERROR;
        Set_LED("OKAY");
        // prepare structure for dumping
        strcpy(syslog.desc, "Register Access Test Passed");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("REGISTER ACCESS TEST PASS!!!\n");
        break;
    default:
        Set_LED("????");
        eprintf("UNKNOWN RESULTS\n");
        break;
    }
}

/******************************************************************
* Function : DumpPRBSResults
*-----------------------------------------------------------------
* Description  : This routine displays the results of the PRBS test.
*-----------------------------------------------------------------
* Inputs  : status : The results of the PRBS test.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
DumpPRBSResults(BDW_Status_t status)
{
    // prepare structure for dumping
    syslog.type = TYPE_ERROR;
    syslog.source_comp = MASTER_BATTERY_DIAG;

    switch(status)
    {
    case BDW_PRBS_TEST_ALREADY_RUNNING:
        Set_LED("RUNN");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Test already running");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("PRBS TEST ALREADY RUNNING\n");
        break;
    case BDW_INVALID_CHANNEL:
        Set_LED("CHAN");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Invalid Channel");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("INVALID CHANNEL\n");
        break;
    case BDW_UNKNOWN_MEDIA_CARD:
        Set_LED("MEDI");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Unknown Media");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("UNKNOWN MEDIA CARD\n");
        break;
    case BDW_PRBS_INVALID_MODE:
        Set_LED("MODE");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Invalid Mode");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("INVALID MODE\n");
        break;
    case BDW_PRBS_TEST_NOT_STARTED:
        Set_LED("STRT");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Test not started");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("PRBS TEST NOT STARTED\n");
        eprintf("PRBS TEST FAIL!!!\n");
        break;
    case BDW_PRBS_TESTED_ENDED_IN_ERROR:
        Set_LED("FAIL");
        // prepare structure for dumping
        strcpy(syslog.desc, "PRBS Test Failed: Tested ended in error");

        // write to syslog
        syslog_dump(&syslog, sizeof(SYSLOG_DATA));
        eprintf("PRBS TEST FAIL!!!\n");
        break;
    case BDW_SUCCESS:
        Set_LED("OKAY");
        eprintf("TEST PASS\n");
        break;
    default:
        Set_LED("????");
        eprintf("UNKNOWN RESULTS\n");
        break;
    }

}


/******************************************************************
* Function : StartPRBSLoopbackTest
*-----------------------------------------------------------------
* Description  : This routine starts the PRBS loopback test.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid chip handle pointer.
*           chanNum : Specifies the channel number.
*           mode : Specifies INTERNAL or EXTERNAL loopback
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : eduCommon structure
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
BDW_Status_t
StartPRBSLoopbackTest(bb_ChipData_t *pChipData, int chanNum,
    LoopBackMode_t mode)
{

    register eduCommon* acL = (eduCommon *) ACADDRESS;
    const InitRegTable_t *pLoopIntInitTable;
    const InitRegTable_t *pLoopExtInitTable;
    ushort regVal = 0x0;
    uint cnt = 0;

    /* Check to see if we are Battey Drew */
    if (!(strcmp(acL->i2cData.model_id, BATTETY_DREW_CARD)))
    {
        eprintf("SINGLE OC-48 Line card detected!!\n");
        if (chanNum > 0)
        {
            return BDW_INVALID_CHANNEL;
        }

        /* Assign the Internal loopback initialization table */
        pLoopIntInitTable = BDIntLoopbackArray;

        /* Assign the External loopback initialization table */
        pLoopExtInitTable = BDExtLoopbackArray;
    }
    /* Check to see if we are Battey Wallace */
    else if (!(strcmp(acL->i2cData.model_id, BATTETY_WALLACE_CARD)))
    {
        eprintf("QUAD OC-12 Line card detected!!\n");

        /* Assign the Internal loopback initialization table */
        pLoopIntInitTable = BWIntLoopbackArray;

        /* Assign the External loopback initialization table */
        pLoopExtInitTable = BWExtLoopbackArray;
    }
    /* If not Drew or Wallace, then we are in trouble */
    else
    {
        return BDW_UNKNOWN_MEDIA_CARD;
    }


    /* Save Channel number in global common structure. */
    acL->prbsChanNum = chanNum;

    /* Configure IXF6048 for PRBS internal or external loop back */
    if (mode == INTERNAL)
    {
        /* Initialize Drew chip for internal loopback */
        IxfApiInitChip(pChipData, (InitRegTable_t *) pLoopIntInitTable);
    }
    else if (mode == EXTERNAL)
    {
        /* Initialize Drew chip for internal loopback */
        IxfApiInitChip(pChipData, (InitRegTable_t *) pLoopExtInitTable);
    }
    else
    {
        return BDW_PRBS_INVALID_MODE;
    }

    /* Sleep for 1 second */
    OS_TASK_SLEEP(1);

    do {
        /* Clear the PRBS Interrupts if present before test begins */
        IxfApiGenericRead(pChipData, TWO_BYTES, PRBSINT, 1, &regVal);
        cnt++;

        if (cnt > 100)
            break;

    } while (regVal & (1 << chanNum));

    if (cnt > 100)
    {
        eprintf("Could not clear the PRBS alarms before starting test.\n");
        return BDW_FAIL;
    }

    eprintf("The PRBS interrupts are cleared.\n");

    /* Start Prbs Testing */
    TurnPRBSOn(pChipData, chanNum);

    eprintf("The Test has started.\n");

    return BDW_SUCCESS;
}


/******************************************************************
* Function : StopPRBSLoopbackTest
*-----------------------------------------------------------------
* Description  : This routine stops the PRBS loopback test.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid chip handle pointer.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : eduCommon structure
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
BDW_Status_t 
StopPRBSLoopbackTest(bb_ChipData_t *pChipData)
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    unsigned short chanNum;

    /* Get channel number for the common structure */
    chanNum = acL->prbsChanNum;

    /* Stop PRBS Test */
    TurnPRBSOff(pChipData, chanNum);

    return BDW_SUCCESS;
}

/******************************************************************
* Function : CheckDefaultValues
*-----------------------------------------------------------------
* Description  : This routine compares read data from the device
*                to the expected reset defaults from the data sheet.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid chip handle pointer.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : eduCommon structure
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
BDW_Status_t 
CheckDefaultValues(bb_ChipData_t *pChipData)
{
    int i=0;
    bb_ChipSegment_t section;
    bb_RegPointer_type pReg;

    /* Assign channel number */
    section.group.channel = 0;

    /* Read and check the default values */
    while ((defaultArray[i].offset != (bb_RegPointer_type)0xFFFF) &&
        (defaultArray[i].value != IXF_INIT_DATA_DELIMITER))
    {
        /* Form the address */
        pReg = (pChipData->BaseAddress + (ulong) defaultArray[i].offset);

        if (READ_FROM_HW(pReg) != defaultArray[i].value)
        {
            eprintf("BA = %08x ", pChipData->BaseAddress);
            eprintf("MISMATCH on addr = %08x", pReg);
            eprintf(" EXPECTED=0x%04x", defaultArray[i].value);
            eprintf(" ACTUAL=0x%04x\n", READ_FROM_HW(pReg));

            return BDW_FAIL;
        }

        i++;
    }

    return BDW_SUCCESS;
}

/***********************************************************************
****                       PUBLIC ROUTINES                          ****
***********************************************************************/


/******************************************************************
* Function : BdwPRBSLoopbackTest
*-----------------------------------------------------------------
* Description  : This routine performs a media loopback test by
*                generating a PRBS test pattern and then analyzing
*                the pattern.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid chip handle pointer.
*           chanNum : Specifies the channel number.
*           mode : Specifies INTERNAL or EXTERNAL loopback
*           timeOut : Time in seconds to perform the test.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : eduCommon structure
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
BDW_Status_t 
BdwPRBSLoopbackTest(bb_ChipData_t *pChipData, int chanNum,
    LoopBackMode_t mode, int timeOut)
{
    BDW_Status_t status;
    ushort regVal;
    ushort chipId;
    bb_ChipSegment_t section;

    /* Print Banner */
    eprintf("\n\nPerforming PRBS Test on:\nChannel# : %d\nLoopback : \
%s\nTimeout : %d secs\n\n",
        chanNum, (mode) ? "EXTERNAL" : "INTERNAL", timeOut);

    /* Assign channel number */
    section.group.channel = chanNum;

    eprintf("Resetting the device\n");
    /* Reset Device */
    IxfApiReset(pChipData, &section, bb_RESET_CHIP);

    eprintf("Checking default values\n");
    if (CheckDefaultValues(pChipData) == BDW_FAIL)
    {
        eprintf("FAILED Reset\n");
        return BDW_FAIL;
    }
    eprintf("Default values are good\n");

    /* Start Test */
    status = StartPRBSLoopbackTest(pChipData, chanNum, mode);

    if (status == BDW_SUCCESS)
    {
        /* Poll the PRBS interrupt to see if PRBS Test failed */
        while (timeOut)
        {
            eprintf("%d ", timeOut);

            /* Read the PRBS Interrupts */
            IxfApiGenericRead(pChipData, TWO_BYTES, PRBSINT, 1, &regVal);

            IxfApiGenericRead(pChipData, TWO_BYTES, CHIP_ID, 1, &chipId);

            /* Make sure we can read the chip id first */
            if (chipId != 0x0201)
            {
                eprintf("\nInvalid chipId %x\n", chipId);
                break;
            }

            /* Check to see if we got an interrupt on our channel */
            if (regVal & (1 << chanNum))
            {
                break;
            }

            /* Sleep for 1 second */
            OS_TASK_SLEEP(1);

            timeOut--;
        }

        eprintf("\n");

        /* Stop Timer */
        status = StopPRBSLoopbackTest(pChipData);
    }

    /* Check to see if we got an error during the test */
    if ((timeOut) || (status != BDW_SUCCESS))
    {
        status = BDW_PRBS_TESTED_ENDED_IN_ERROR;
    }

    /* Dump Results */
    DumpPRBSResults(status);

    return status;
}


/******************************************************************
* Function : BdwRegisterAccessTest
*-----------------------------------------------------------------
* Description  : This routine performs a read and write test on
*                device to make sure the basics are working.
*-----------------------------------------------------------------
* Inputs  : pChipData : Valid chip handle pointer.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : eduCommon structure
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
BDW_Status_t 
BdwRegisterAccessTest(bb_ChipData_t *pChipData)
{
    bb_ChipSegment_t section;

    /* Assign channel number */
    section.group.channel = 0;

    eprintf("Resetting the device\n");
    /* Reset Device */
    IxfApiReset(pChipData, &section, bb_RESET_CHIP);

    eprintf("Checking default values\n");
    if (CheckDefaultValues(pChipData) == BDW_FAIL)
    {
        return BDW_FAIL;
    }
    eprintf("Default values are good\n");

    /********* Register Testing *******************/

    /* Write data */
    eprintf("Write 0xAAAA pattern\n");
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(MACNF)), 0xAAAA);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(GOCNF)), 0xAAAA);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(COCNF(0))), 0xAAAA);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(R_COCNF(0))), 0xAAAA);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(T_COCNF(0))), 0xAAAA);

    /* Read and Compare data */
    eprintf("Reading and Comparing data\n");
    if (
        ((ushort) IXF6048_REG_VAL(MACNF)         != (ushort)  0x2) ||
        ((ushort) IXF6048_REG_VAL(GOCNF)         != (ushort)  0xAAA) ||
        ((ushort) IXF6048_REG_VAL(COCNF(0))      != (ushort)  0x2A) ||
        ((ushort) IXF6048_REG_VAL(R_COCNF(0))    != (ushort)  0x2AA) ||
        ((ushort) IXF6048_REG_VAL(T_COCNF(0))    != (ushort)  0xAAA) 
        )
    {
        return BDW_FAIL;
    }

    /* Write data */
    eprintf("Write 0xFFFF pattern\n");
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(MACNF)), 0xFFFF);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(GOCNF)), 0xFFFF);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(COCNF(0))), 0xFFFF);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(R_COCNF(0))), 0xFFFF);
    WRITE_TO_HW((bb_RegPointer_type)(IXF6048_REG(T_COCNF(0))), 0xFFFF);

    /* Read and Compare data */
    eprintf("Reading and Comparing data\n");
    if (
        ((ushort) IXF6048_REG_VAL(MACNF)         != (ushort)  0x7) ||
        ((ushort) IXF6048_REG_VAL(GOCNF)         != (ushort)  0x3FFF) ||
        ((ushort) IXF6048_REG_VAL(COCNF(0))      != (ushort)  0x3F) ||
        ((ushort) IXF6048_REG_VAL(R_COCNF(0))    != (ushort)  0x3FF) ||
        ((ushort) IXF6048_REG_VAL(T_COCNF(0))    != (ushort)  0xFFF) 
        )
    {
        return BDW_FAIL;
    }

    return BDW_SUCCESS;
}

static void BdwUnit_Help(void)
{
	eprintf("> t unit <option> <channel> <iteration> <mode> <time>, where\n");
	eprintf("<option> are:\n");
	eprintf("r - Register read/write test\n");
	eprintf("l - PRBS loopback test\n");
	eprintf("h - LED help message\n");
}

/******************************************************************
* Function : BdwUnit_Test
*-----------------------------------------------------------------
* Description  : This routine calls the specific unit test given
*                the parsed command line instruction
*-----------------------------------------------------------------
* Inputs  : None.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : eduCommon structure.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
BdwUnit_Test()
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    char opt;
    int i;
    int channel, mode, time, iterations = 1;
    BDW_Status_t status;
    bb_ChipData_t *pChipData = acL->chipId;

    opt = acL->dc.argv[2][0];
    channel = acL->dc.decArg[3];
    iterations = acL->dc.decArg[3];
    mode = acL->dc.decArg[4];
    time = acL->dc.decArg[5];

    switch(opt)
    {

    case 'r':
        for (i=0; i < iterations; i++)
        {
            eprintf("***************************\n");
            eprintf(" R/W Test Iteration #: %d\n", (i+1));
            eprintf("***************************\n");
            /* RegisterAccess Test */
            status = BdwRegisterAccessTest(pChipData);
            DumpRegAccessResults(status);
            if(KEYPRESS)
		    {
			    break;
		    }
        }
        break;

    case 'l':
        Set_LED("PRBS");
        /* PRBS Loopback Test */
        BdwPRBSLoopbackTest(pChipData, channel, mode, time);
        break;

    case 'h':
	default:
        BdwUnit_Help();
    }
}

/******************************************************************
* Function : BdwUnit_All
*-----------------------------------------------------------------
* Description  : This routine calls all specific unit test
*-----------------------------------------------------------------
* Inputs  : None.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : eduCommon structure.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
void
BdwUnit_All()
{
    int forever = 0, loops = 1;
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    int channel, mode;
    BDW_Status_t status;
    bb_ChipData_t *pChipData = acL->chipId;
    int MaxNumOfChans;
    int failFlag = 0;

    if(acL->dc.argc < 3)
		forever = 0;
	else
	{
        loops = acL->dc.decArg[2];
        if(loops == 0)
            forever = 1;
        else
            forever = 0;
    }
    
    eprintf("**********************\n");
    eprintf("* Perforning RW TEST *\n");
    eprintf("**********************\n\n");
    while(loops-- || forever)
	{
        /* RegisterAccess Test */
        status = BdwRegisterAccessTest(pChipData);
        
        if (status != BDW_SUCCESS)
            failFlag = 1;
        
        DumpRegAccessResults(status);
        eprintf("\n\n");
        
        eprintf("************************\n");
        eprintf("* Perforning PRBS TEST *\n");
        eprintf("************************\n\n");
        
        /* Check to see if we are Battey Drew */
        if (!(strcmp(acL->i2cData.model_id, BATTETY_DREW_CARD)))
        {
            MaxNumOfChans  = 1;
        }
        /* Check to see if we are Battey Wallace */
        else if (!(strcmp(acL->i2cData.model_id, BATTETY_WALLACE_CARD)))
        {
            MaxNumOfChans = 4;
        }
        /* If not Drew or Wallace, then we are in trouble */
        else
        {
            eprintf("Unknown media card\n");
            return;
        }
        
        for (mode = INTERNAL; (mode <= EXTERNAL); mode++)
        {
            for (channel = 0; (channel < MaxNumOfChans); channel++)
            {
                /* PRBS Loopback Test */
                status = BdwPRBSLoopbackTest(pChipData, channel, mode, BDW_ALL_FIXED_TIME);
                
                if (status != BDW_SUCCESS)
                    failFlag = 1;
            }
            eprintf("\n\n");
        }
        
        /* Print the final results */
        if (failFlag)
        {
            eprintf("TEST FAIL!!!\n");
        }
        else
        {
            eprintf("ALL TEST PASS!!!\n");
        }
        if(KEYPRESS)
        {
            break;
        }
    }
}




















