/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: lagarwal $
*   DATE:           $Date: 2003/06/12 23:42:09 $
*   LAST MODIFIED:  $Modtime: 5/20/02 4:41p $
*   REV#:           $Revision: 1.8 $
*   LOGFILE:        $Logfile: /Embedded/source/linecards/batteryDW/testmgr.c $
*   WORKFILE:       $Workfile: testmgr.c $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose: 
*
* Notes:
*			
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log:
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "memory.h"
#include "common.h"
#include "diagstruct.h"
#include "bdw.h"
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet_d.h"
#include "ixf6048d.h"
#include "ixf6048.h"
#include "bdwtests.h"
#include "i2c.h"
#include "i2c_test.h"
#include "uart.h"
#include "hal_ixdp2400.h"
#include "console_monitor.h"
#include "diag_utils.h"


#include <stdio.h>
#include <string.h>
#include <ctype.h>

// function prototypes
extern char* Get_CPU(void);
extern int Get_CPU_Speed(void);
extern int Get_CPU_Rev(void);
extern int Get_SRAM_Size(void);
extern int Get_SDRAM_Size(void);
extern int Get_Host_Type(void);

extern void I2C_Test(void);
void BdwUnit_All();
void DiagTests();
void PrintBanner();
extern void cmd_reset(void);
void parseCmd(char *string, UINT pntr, int index);

extern void makeNumber(int i);
DiagCmd *findCmd(unsigned int pntr, int idx);
void cmd_help();
void GetI2CData(I2Cstruct_t *data);
void cmd_i2crd(void);
void cmd_i2cwr(void);

extern UINT32 sram_size[2];

/* Main Command Table */
DiagCmd bdwcmdarray[] = {
    { "help",     "s",    5,  cmd_help,       0, "print this message" },
    { "?",        "s",    5,  cmd_help,       0, "print this message" },
    { "h",        "s",    5,  cmd_help,       0, "print this message" },
    { "test",  "s",    3,  DiagTests,      0, "test, type t <cr> for help"},
    { "t",       "s",    3,  DiagTests,      0, "test, type t <cr> for help"},
    { "banner",   "",     0,  PrintBanner,    1, "Print Banner "},
    { "exit",     "",     0,  cmd_reset,      1, "Exit Diagnostics"},
    { "p", "xx", 0,cmd_print, 0, "<low> <high> print memory (32 bit)" },
    { "pb", "xx", 0,cmd_print, 2, "<low> <high> print memory (8 bit)" },
    { "pw", "xx", 0,cmd_print, 1,"<low> <high> print memory (16 bit)" },
    { "pd", "xx", 0,cmd_print, 0, "<low> <high> print memory (32 bit)" },
    { "c", "x", 0,cmd_change, 0, "change memory (32 bits)" },
    { "cb", "x", 0,cmd_change, 2, "change memory (8 bits)" },
    { "cw", "x", 0,cmd_change, 1, "change memory (16 bits)" },
    { "cd", "x", 0,cmd_change, 0, "change memory (32 bits)" },
    { "i2cread", "XXD", 0, cmd_i2crd, 1," I2C read <X:slave_addr> <X:offset_addr> <D:count>"},
    { "i2cwrite", "XXS", 0, cmd_i2cwr, 1," I2C write <X:slave_addr> <X:offset_addr> <S:data>"},
    { "viewlog", "D", 0, cmd_viewlog, 1, "View the entries in the log"},
	{ "clearlog", "", 0, syslog_clear, 0, "Clear all entries in the log"},
	{ "setverbose", "d", 0, cmd_set_verbose, 1, "Set verbose level <verbose_level>"},
    { 0,          0,      0,  0,0,0}
};


/* Diagnostic Command Table */
DiagCmd bdwtestarray   [] = {
    { "i2c",      "sxx",  0,  I2C_Test,       0, ""},
    { "unit",     "sddd", 0,  BdwUnit_Test,   0, ""},
    { "all",	  "d",	  0,  BdwUnit_All,	  0, "[d:loops]"},
    {0,0,0,0,0,0}
};

void DiagTests()
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    DiagCmd *findCmd(UINT, int), *cmd;

    if (acL->dc.argc == 1)
    {
        eprintf("\nusage: t 'name' <args>\n");
        eprintf("where 'name' <args> is one of the following:\n\n");
        eprintf("t unit r <iterations>        (performs the R/W test)\n");
        eprintf("t unit l chan<0-3> mode<0 or 1> time<in seconds> (performs PRBS loopback test)\n");
        eprintf("t i2c                        (performs test on I2C)\n");
        eprintf("t all                        (performs all tests)\n");
        return;
    }
    cmd = findCmd((UINT)bdwtestarray,1);   /* find the command in our table */
    if (cmd != 0)
    {
        acL->dc.numArgs = cmd->numArg;  /* load up the arguments */
        (*(cmd->routine))();        /* call the requested routine now... */
    }
    else
        eprintf("unknown test command\n");
}


void PrintBanner()
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    int i;
    unsigned short regVal;
    char *resultStr;

    eprintf("\n%s Board Diagnostics ver. %d.%d, %s, %s\n",
        acL->dc.targetName, ((acL->dc.version>>8) & 0xFF),
        (acL->dc.version & 0xFF), __TIME__, __DATE__);

    eprintf("DRAM:%d Mb,\n", acL->dc.SDRAMSize/1024/1024);
    eprintf("SRAM:%d Mb; Ch0: %d Mb, Ch1: %d Mb,\n",
        acL->dc.SRAMSize/1024/1024,
        sram_size[0]/1024/1024,
        sram_size[1]/1024/1024);

    if(acL->dc.HostType == MASTER)
    {
        unsigned char temp[16];

        eprintf("******************************************************************\n");

        /* Get the I2C Data */
        GetI2CData(&(acL->i2cData));
        
        /* Build a test status result string */
        switch((acL->i2cData.test_status - '1'))
        {
        case 0:
            resultStr = "PASSED";
            break;
        case 1:
            resultStr = "FAILED";
            break;
        default:
            resultStr = "NOT TESTED";
        }
        
        /* Convert string to upper case */
        i=0;
        while(acL->i2cData.model_id[i])
        {
            acL->i2cData.model_id[i] = toupper(acL->i2cData.model_id[i]);
            i++;
        }
        
        if (strcmp(acL->i2cData.model_id, "BW") &&
            strcmp(acL->i2cData.model_id, "BD"))
        {
            strcpy(acL->i2cData.model_id, "UNKNOWN");
        }

        eprintf("Media Card         : ");
        eprintf("%s", acL->i2cData.model_id);
        eprintf("\n");
        
        eprintf("Device Id          : ");
        eprintf("%d", acL->i2cData.dev_id);
        eprintf("\n");
        
        eprintf("Device Rev.        : ");
        eprintf("%c%c", acL->i2cData.part_rev[0], acL->i2cData.part_rev[1]);
        eprintf("\n");

		memset(temp, '\0', 16);
		memcpy(temp, acL->i2cData.part_num, CFG_PARTNUM_LEN);
        eprintf("Part Number        : %s\n", temp);

        memset(temp, '\0', 16);
		memcpy(temp, acL->i2cData.serial_num, CFG_SERIAL_NUM_LEN);
        eprintf("Serial Number      : %s\n", temp);
        
        memset(temp, '\0', 16);
		memcpy(temp, acL->i2cData.date, CFG_DATE_LEN);
        eprintf("Test Date          : %s\n", temp);

        eprintf("Test Status        : ");
        eprintf("%s", resultStr);
        eprintf("\n");
        
        eprintf("Target CPU         : ");
        eprintf("%s", acL->dc.targetCpu);
        eprintf("\n");
        
        IxfApiGenericRead((bb_ChipData_t *) acL->chipId, TWO_BYTES, 0x4001, 1, &regVal);
        eprintf("Media CPLD version : %02d\n", (regVal>>8));
        
        eprintf("******************************************************************\n");
    }
}


/* getCommands

  Function:
*/
void getCommands()
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;

    if(acL->dc.HostType == MASTER)
    parseCmd("MSTR_BATTERY_DIAG> ",(UINT)bdwcmdarray,(int)0);  // the prompt of course
    else
        parseCmd("SLV_BATTERY_DIAG> ",(UINT)bdwcmdarray,(int)0);  // the prompt of course
}


//
// Function: parseCmd
//
// Decription:	Displays the prompt, then gets users input.
//				Separates user arguments to their respective
//				arrays. Calls findCmd and calls the routine
//				if the command matches a routine
//
void parseCmd(char *string, UINT pntr, int index)
{
    register eduCommon *acL = (eduCommon *) ACADDRESS;
    DiagCmd *findCmd(UINT, int), *cmd;
    char* s;
    int i = 0;

    bzero(acL->dc.cmdLine, 128); /* initialize the command buffer */
    eprintf("%s",string);

    getCmdLine(acL->dc.cmdLine);	// read user input from command line
    s = acL->dc.cmdLine;	// point to the cmdLine buffer

    // Clear arg count, and all the arguments
    acL->dc.argc = 0;
    for(i = 0; i < 10; i++)
    {
        acL->dc.hexArgOk[i] = FALSE;
        acL->dc.decArgOk[i] = FALSE;
        acL->dc.hexArg[i] = 0;
        acL->dc.decArg[i] = 0;
        acL->dc.argv[i] = 0;
    }

    while(1)
    {
        while(*s == ' ')	// skip spaces
            s++;
        if (*s == 0)	// if end of string, exit scanner
            break;
        acL->dc.argv[acL->dc.argc++] = s;	// store beginning of string
        if (*s == '"')
        {
            acL->dc.argv[acL->dc.argc - 1]++;
            while ((*s != '"') && (*s != 0))
            s++;    // skip argument
            if(*s == '"')
                *s++ = 0;
        }
        else
        {
            while((*s != ' ') && (*s != 0))
            s++;	// skip argument
            if(*s == ' ')
                *s++ = 0;	// put in a terminator, if necessary
        }
    }

    if(acL->dc.argc == 0)
        return;

    for (i = 1; i < acL->dc.argc; i++) makeNumber(i);
    // Search for the command in our command table
    cmd = findCmd(pntr,index);

    if(cmd != 0)
    {
        acL->dc.numArgs = cmd->numArg; /* load up the arguments */
        (*(cmd->routine))();
    }
    else
        eprintf("***Error***: Command not found\n");
}


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* function: makeNumber()                                                   */
/*                                                                          */
/* purpose: to convert the ascii string in the argv[i] array to an actual   */
/*          binary digit.                                                   */
/*                                                                          */
/* calls: isDigit(), tolower()                                              */
/*                                                                          */
/* abstract: the ascii number will be parsed from start to finish. When     */
/*           done, the decarg array will contain valid values               */
/*                                                                          */
/* returns: void                                                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
void makeNumber(int i)
{
    register eduCommon *acL = (eduCommon *) ACADDRESS;
    char *s, c;
    int v = 0;

    s = acL->dc.argv[i];                   /* initialize all the arrays */
    acL->dc.hexArg[i] = 0;
    acL->dc.decArg[i] = 0;
    acL->dc.decArgOk[i] = TRUE;
    acL->dc.hexArgOk[i] = TRUE;
    while (*s) {
        c = *s++;
        if (isdigit(c))                   /* check if argument is a number */
        v = c - '0';                    /* if so, then make it binary */
        else {
            acL->dc.decArgOk[i] = FALSE;       /* if hex, then we may have failed IsDigit */
            c = tolower(c);                 /* lower it */
            if (((c >= 'a') && (c <= 'f'))) {
                v = c - 'a' + 10;             /* see if its hex, if so, make it binary */
            }
            else acL->dc.hexArgOk[i] = FALSE;
        }
        if (acL->dc.decArgOk[i])             /* fill in the decimal argument array */
            acL->dc.decArg[i] = (acL->dc.decArg[i] * 10) + v;
        if (acL->dc.hexArgOk[i])             /* fill in the hex argument array */
            acL->dc.hexArg[i] = (acL->dc.hexArg[i] * 16) + v;
    }
}

//
// Function: findCmd
//
// Description: Looks for a command in an array. If not found,
//				displays "unknown command"
//
DiagCmd *findCmd(unsigned int pntr, int idx)
{
    register eduCommon *acL = (eduCommon *) ACADDRESS;
    DiagCmd *p;
    int wantargs, matchOk, i;
    char syn_curr;
    p = (DiagCmd*) pntr;	/* point to the command/test array */

    while (p->cmdw)	/* search til there aren't any more commands in the table*/
    {
        if (strcmp(acL->dc.argv[idx], p->cmdw) == 0)
        {
            matchOk = TRUE;                   /* we found a match, now check syntax */
            wantargs = strlen(p->syntax);
            //if(wantargs == (acL->dc.argc - 1))	// cannot be done since "test" has variable args
            for (i = 1; i <= wantargs; i++)
            {
                syn_curr = p->syntax[i-1];
                if ((syn_curr=='X') && !acL->dc.hexArgOk[i+idx])
                    matchOk = FALSE;
                if (syn_curr=='x' && (i+idx) < acL->dc.argc && !acL->dc.hexArgOk[i+idx])
                    matchOk = FALSE;
                if ((syn_curr =='D') && !acL->dc.decArgOk[i+idx])
                    matchOk = FALSE;
                if (syn_curr=='d' && (i+idx) < acL->dc.argc && !acL->dc.decArgOk[i+idx])
                    matchOk = FALSE;
                if (syn_curr=='S' && i >= acL->dc.argc)
                    matchOk = FALSE;
            }

            if (matchOk)
            return(p);           /* checks out, return pointer */
            else
                eprintf("Syntax is incorrect, type 'h' to get help\n");

            if (idx)
                eprintf("syntax error... type 't %s %s'\n",acL->dc.argv[1],p->helpstr);
            else
                eprintf("syntax error... type 'h %s'\n",acL->dc.argv[0]);

            return(0);                        /* syntax is wrong, return fail status */
        }
        p++;                                /* bump to next command in array */
    }
    eprintf("unknown command\n");
    return(0);
}


//
// Function: cmd_help
//
// Description: Displays all commands available to the user at the highest level
//
void cmd_help()
{

    /*
    This function is invoked when the user types the help command at the Diag prompt.
    This function will display a help message on all the commands available in diagnostics. 
    */
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    DiagCmd *p;
    char *s, syncmd;

    p = (DiagCmd *) &bdwcmdarray[0];

    if (acL->dc.argc == 1)
    {
        eprintf("\n****************************************************************************\n");
        eprintf("\t\t\tBATTERY DREW/WALLACE HELP MENU\n");
        eprintf("Access Commands:\n");
        eprintf("p[bwd].......print memory contents       c[bwd].......change memory contents\n");
        eprintf("i2cread......Read an I2C device          i2cwrite.....Write an I2C device\n");
        eprintf("viewlog.......view entries in log        clearlog.....clears the system log\n");
        eprintf("setverbose.......Set verbose level\n");
        eprintf("\n");
        eprintf("Utility Commands:\n");
        eprintf("?.............display this help screen   banner.......Print banner\n");
        eprintf("exit..........Exit Diagnostics           h(elp).......display this help screen\n");
        eprintf("t(est)........Run DIAG test\n");

        eprintf("\n");
        eprintf("Note: additional help is available on the above commands,\n");
        eprintf("for help on each command type 'h command<cr>'.\n\n");
    }
    else
    {
        while (p->cmdw)
        {                   /* print expected syntax for each command */

            if (strcmp(p->cmdw, acL->dc.argv[1]) == 0)
            {
                eprintf("%s: %s\n", p->cmdw, p->helpstr);
                s = p->syntax;
                eprintf("arguments: ");
                while(*s)
                {

                    syncmd = *s++;
                    switch (syncmd)
                    {
                    case 'X': eprintf("<hex> ");
                        break;
                    case 'x': eprintf("<opt hex> ");
                        break;
                    case 'D': eprintf("<dec> ");
                        break;
                    case 'd': eprintf("<opt dec> ");
                        break;
                    case 'S': eprintf("<str> ");
                        break;
                    case 's': eprintf("<opt str> ");
                        break;
                    case ' ': eprintf("<none> ");
                        break;
                    }
                }
                eprintf("\n");
            }
            p++;
        }
    }
}



// *Note: only use for bring up
// Function: cmd_i2crd
//
// Description: allows the user to read any location of
//				any I2C PROM
//
void cmd_i2crd(void)
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    unsigned char slave_addr = acL->dc.hexArg[1];
    unsigned char offset_addr = acL->dc.hexArg[2];
    unsigned char count = acL->dc.decArg[3];
    unsigned char buffer[56] = "";
    unsigned char i = 0;

    if(acL->dc.HostType == SLAVE)
    {
        eprintf("This command can only be run on master NPU\n");
        return;
    }
    if( (offset_addr + count) > 255)
    {
        eprintf("The address and count used exceeds the address of 255\n");
    }
    else
    {
        i2c_seq_read(slave_addr, offset_addr, buffer, count);

        for(i = 0; i < count; i++)
            eprintf("%x ", buffer[i]);
        eprintf("\n");
    }
}


// *Note: only use for bring up
// Function: cmd_i2cwr
//
// Description: allows the user to write any location of
//				any I2C PROM
//
void cmd_i2cwr(void)
{
    register eduCommon* acL = (eduCommon *) ACADDRESS;
    unsigned char buffer[56] = "", count, c;
    unsigned int i;

    if(acL->dc.HostType == SLAVE)
    {
        eprintf("This command can only be run on master NPU\n");
        return;
    }

    count = strlen(acL->dc.argv[3])/2;

    for(i = 0; i < count; i++)
    {
        c = tolower (acL->dc.argv[3][i*2]);
        if (isdigit (c) || ((c >= 'a') && (c <= 'f')))
        {
            if(c <= '9')
                buffer[i] = ((unsigned char)(c - '0'))<<4;
            else
                buffer[i] = ((unsigned char)(c - 'a' + 10))<<4;
        }

        c = tolower (acL->dc.argv[3][i*2 + 1]);
        if (isdigit (c) || ((c >= 'a') && (c <= 'f')))
        {
            if(c <= '9')
                buffer[i] += (unsigned char)(c - '0');
            else
                buffer[i] += (unsigned char)(c - 'a' + 10);
        }
    }
    i2c_page_write(acL->dc.hexArg[1], acL->dc.hexArg[2], buffer, count);
}

void install_other_isr()
{
}
