/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: lagarwal $
 *   DATE:           $Date: 2003/05/01 20:42:51 $
 *   LAST MODIFIED:  $Modtime: 5/17/02 1:58p $
 *   REV#:           $Revision: 1.4 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf6048/bkcompat.c $
 *   WORKFILE:       $Workfile: bkcompat.c $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6048
 *
 * Purpose: Provides the API for Ixf6048 Specific routines.  This
 *          is meant to glue the new and old api together for 
 *          backward compatibilty.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: bkcompat.c,v $
 *   Revision 1.4  2003/05/01 20:42:51  lagarwal
 *   no message
 *
 *   Revision 1.3  2003/01/29 00:22:22  lagarwal
 *   no message
 *
 *   Revision 1.2  2002/11/18 17:41:00  lagarwal
 *   no message
 *
 *   Revision 1.1.1.1  2002/11/15 17:51:11  lagarwal
 *   rename dir
 *
 *   Revision 1.10  2002/09/09 21:49:21  cmah1
 *   no message
 *
 *   Revision 1.8  2002/07/23 16:12:58  machambe
 *   no message
 *
 *   Revision 1.7  2002/07/21 03:17:57  machambe
 *   no message
 *
 * 
 * 22    5/17/02 2:01p Machambe
 * Updating InitChip
 * 
 * 21    5/16/02 6:09p Machambe
 * Added SetOpMode
 * 
 * 20    4/18/02 8:42a Machambe
 * 
 * 19    4/15/02 2:28p Machambe
 * 
 * 18    4/15/02 2:21p Machambe
 * Adding Function Init
 * 
 * 17    4/15/02 10:23a Machambe
 * Adding changing ohBytes routine to support K1K2 (ushort)
 * 
 * 16    4/11/02 5:08p Machambe
 * Adding memory allocation for more trace strings
 * 
 * 15    4/08/02 12:12a Machambe
 * Adding Length to GetTrace
 * 
 * 14    4/05/02 12:36p Machambe
 * Adding "include string.h" for non-VXWORKS build
 * 
 * 13    3/26/02 6:22p Machambe
 * Added 6048 GetBuildVersion
 * 
 * 12    12/04/01 4:37p Machambe
 * 
 * 11    12/01/01 11:39p Machambe
 * 
 * 10    12/01/01 6:41p Machambe
 * changing hw_if to hal
 * 
 * 9     11/16/01 11:32a Machambe
 * 
 * 8     8/07/01 1:13p Machambe
 * Fixing SetLCDFilter
 * 
 * 7     6/25/01 11:56a Machambe
 * removing warning
 * 
 * 6     6/25/01 11:08a Machambe
 * Get rid of some warnings
 * 
 * 5     6/22/01 4:54p Machambe
 * 
 * 4     6/18/01 4:10p Machambe
 * 
 * 3     5/18/01 4:54p Machambe
 * 
 * 2     5/03/01 3:57p Machambe
 * 
 * --------------------------------------------------------------------------
 */ 

#ifdef VXWORKS
#include "vxWorks.h"
#include "msgQLib.h"
#else
#include "stdio.h"
#include "string.h"
#include "stdlib.h"
#endif


#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api_sonet_d.h"
#include "ixf6048d.h"
#include "hal.h"
#include "ixf6048f.h"
#include "ixf6048b.h"
#include "bld6048.h"
#include "os.h"
#include "type.h"

#define ixf6048_GET_NUMB_WINSZ_32BIT(RegAddr_M,RegAddr_L) \
		  ((ulong) ((ulong)(RegAddr_M << 16)) | \
				   ((ulong)(RegAddr_L)))

#define ixf6048_GET_CHANNEL_FROM_SECTION_PARAM(section)		((uchar) (section->group.channel & 0xf))

#define SWAPNTOHS(x)	((((x) & 0x00ff) << 8) | \
			 (((x) & 0xff00) >> 8))

extern UINT32 silicon_rev;

/******************************************************************
* Function : Ixf6048GetStatus
*-----------------------------------------------------------------
* Description  : Gets the Status.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						status from.
*           selStatus : Type of Status: 
* Outputs : pStatus	: place to put Status.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048GetStatus(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_SelStatus_e selStatus, void* pStatus)
				
{
	return ixf6048_GetStatus(pChipData, section->group.channel,
                             selStatus, pStatus);
}

/******************************************************************
* Function : Ixf6048GetOpMode
*-----------------------------------------------------------------
* Description  : Gets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						status from.
*           opMode    : The mode of operation.
* Outputs : pModeCfg  : Place to put mode.
*-----------------------------------------------------------------
* Returns : Stst
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048GetOpMode(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_OperMode_e *opMode, void* pModeCfg)

{
    return ixf6048_GetChanOpMode(pChipData, section->group.channel,
                                 pModeCfg);
}

/******************************************************************
* Function : Ixf6048GetTrace
*-----------------------------------------------------------------
* Description  : Gets the Rx J0/J1/J2 Trace
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*				section : Determines the section(block) to get the trace
*  						from.
*				TraceType : Type of Trace: Rx J0/J1/J2 for system and
*								line side.
* Outputs : pTrace	: place to return Rx Trace String.
*           pLength : The length of pTrace.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : Note: at least 65 bytes long for pTrace
******************************************************************/
bb_Error_e 
Ixf6048GetTrace(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
			   bb_TraceType_e TraceType, char* pTrace, ushort* pLength)
{
    return ixf6048_GetTrace(pChipData, section->group.channel,
                            TraceType, pTrace, pLength);
}

/******************************************************************
* Function : Ixf6048SetTrace
*-----------------------------------------------------------------
* Description  : Set Expected/Tx J0/J1 Trace
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to get the trace
*  						from.
*			TraceType : Type of Trace: Expected/Tx J0/J1 for system
*						or line side.
*			TraceFormat : Specifies the Trace Format
*			pTrace	: null-terminated Trace String (data only)
*                     Driver adds CRC7, or LF/CR, if needed
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*    Sets the new Trace String in the Associated Storage Structure 
*    Sets the new TraceFormat in the Associated Storage Structure
*    Error Condition if Unsuccessful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
*   The bb_TraceType_e value for  
*          bb_RX_J0 and bb_RX_J1
*   must be Set to "Ignore Jn Trace (No Trace)". 
*   This Condition is Forced in this Function.
*    --> see R_Jn_ESTRA Registers
******************************************************************/
bb_Error_e 
Ixf6048SetTrace(bb_ChipData_t* pChipData, bb_ChipSegment_t *section, 
			   bb_TraceType_e TraceType, bb_TraceFormat_e TraceFormat,
			   char* pTrace)
{
    return ixf6048_SetTrace(pChipData, section->group.channel,
                            TraceType, TraceFormat, pTrace);
}


/******************************************************************
* Function : Ixf6048GetCounters
*-----------------------------------------------------------------
* Description  : Retrieve a set of Counters
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section :	Determines the section(block) to get the
*						counters from.
*			SelCounters :  Selects the set of counters
* Outputs : pCounters : Place to put Counters
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048GetCounters(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				  bb_SelCounters_e SelCounters, void* pCounters)
{
    return ixf6048_GetCounters(pChipData, section->group.channel,
                               SelCounters, pCounters);
}

/******************************************************************
* Function : Ixf6048InitAlarmCallback
*-----------------------------------------------------------------
* Description  : Initialize the Driver
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				pAlarmCallback : Alarm Callback
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048InitAlarmCallback(bb_ChipData_t* pChipData, void* pAlarmCallbackArg)
{
    return ixf6048_InitAlarmCallback((ixf6048_pAlarmCallback) pAlarmCallbackArg); 
}

/******************************************************************
* Function : Ixf6048ChipIsr
*-----------------------------------------------------------------
* Description  : Handle Interupts for Chip
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments:   Interrupts are reported through a Callback function.
*
*	 Transmit and receive Interrupts act independantly of each another.
*
*   Interrupts have been prioritized to be handled according to the severity 
*   of the effecting interupt. See the MasterInterruptHandler function for 
*	 specifics
******************************************************************/
void 
Ixf6048ChipIsr(bb_ChipData_t* pChipData)
{
    ixf6048_ChipIsr(pChipData);
}

/******************************************************************
* Function : Ixf6048GetChipInfo
*-----------------------------------------------------------------
* Description  : Get the chip information.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : pChipInfo : Place to return chip info
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048GetChipInfo(bb_ChipData_t* pChipData, bb_ChipInfo_t* pChipInfo)
{
    ixf6048_ChipInfo_t tempChipInfo;
    bb_Error_e rc;
 
    /* Call Device Specific Driver */ 
    rc = ixf6048_GetChipInfo(pChipData, &tempChipInfo);

    pChipInfo->ChipVersion = tempChipInfo.ChipVersion;
    pChipInfo->ChipId = (ushort) tempChipInfo.ChipId;

    return rc;
}


/******************************************************************
* Function : Ixf6048SetChipOnline
*-----------------------------------------------------------------
* Description  : Puts some or all of the chip online.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section	 :	Specifies which portion of the chip to 
*								put online.
Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None. 
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048SetChipOnline(bb_ChipData_t* pChipData, bb_ChipSegment_t *section)
{
    ixf6048_SetChipOnline(pChipData);
    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048SetChipOffline
*-----------------------------------------------------------------
* Description  : Takes some or all of the chip offline.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section	 :	Specifies which portion of the chip to 
*								take offline.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048SetChipOffline(bb_ChipData_t* pChipData, bb_ChipSegment_t *section)
{
    ixf6048_SetChipOffline(pChipData);
    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048SetOhByte
*-----------------------------------------------------------------
* Description  : Set an Overhead Byte: K1,K2, Expected/Tx C2,
*				 HPM RDI in G1
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to set the byte
*			SelOhByte : Selects a particular Overhead byte 
*			OhByte : Value of selected Overhead byte
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048SetOhByte(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_SelOhBytes_e SelOhByte, ushort OhByte)
{
    return ixf6048_SetOhByte(pChipData, section->group.channel,
                             SelOhByte, OhByte);
}

/******************************************************************
* Function : Ixf6048GetOhBytes
*-----------------------------------------------------------------
* Description  : Gets Overhead Bytes: Rx MST, Rx HPM
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to get the bytes
*			SelOhBytes : Selects the Overhead Bytes 
* Outputs : pOhBytes : place to put Overhead Bytes, of type:
bb_RxMstBytes_t, or ixf6048_RxHpmBytes_t
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e
Ixf6048GetOhBytes(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				 bb_SelOhBytes_e SelOhBytes, void* pOhBytes)
{
    return ixf6048_GetOhBytes(pChipData, section->group.channel,
                              SelOhBytes, pOhBytes);
}

/******************************************************************
* Function : Ixf6048Reset
*-----------------------------------------------------------------
* Description  : Reset (chip or a section of the chip) and then 
*				 reconfigure it. To restart traffic, the chip or 
*				 section of chip must be set back online
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*				section : Determines the section(block) to reset
*				ResetType : Specifies type of Reset
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048Reset(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
			 void* ResetType)
{
    return ixf6048_Reset(pChipData, section->group.channel,
                        (ixf6048_ResetType_e) ResetType);
}

/******************************************************************
* Function : Ixf6048CfgTest
*-----------------------------------------------------------------
* Description  : Run Tests
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				testType : The type of test to run.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e
Ixf6048CfgTest(bb_ChipData_t* pChipData, bb_ChipSegment_t *section, 
               bb_TestType_e testType, void *ptTestCfg)
{
    return bb_INV_TEST_CNFG;
}

/******************************************************************
* Function : Ixf6048SetAlarmCfg
*-----------------------------------------------------------------
* Description  : Edits Interrupt Enabled Masks for a Specific Set
*						of Interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section : Determines the section(block) of the alarm(s)
*							 to edit.
*				AlarmType : Specifies the type of Alarm
*				pAlarmCfg : A pointer to the alarm structure.
* Outputs : 
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048SetAlarmCfg(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				  bb_AlarmType_e AlarmType, void* pAlarmCfg)
{
    return ixf6048_SetAlarmCfg(pChipData, section->group.channel,
                               section->block.flow, AlarmType, pAlarmCfg);
}

/******************************************************************
* Function : Ixf6048Read
*-----------------------------------------------------------------
* Description  :
* This routine will read starting at a particular 
*						location and read for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start reading from.
*				length : Number of words to read.
* Outputs : buffer : The buffer to contain the read data.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048Read(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
            ulong address, ushort length, void *buffer)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    uint ndx;
    uchar *oneBytePtr;
    ushort *twoBytePtr;
    ulong *fourBytePtr;
    ulong temp;    

#ifdef DBG	
    printf("IxfApiGenericRead pCD = %08x WS = %d addr =\
        %08x len = %d b[0] = %x\n",
        pChipData, wordSize, address, length, ((uchar *) buffer)[0]);
#endif
    
    switch (wordSize)
    {
        
    case ONE_BYTE:		
        oneBytePtr = (uchar*) buffer;
        for(ndx=0; ndx < length; ndx++, oneBytePtr++)
        {
            *oneBytePtr = 
                (uchar) READ_FROM_HW((bb_RegPointer_type)(BaseAddress + 
                                                    (ushort) address + ndx));
#ifdef DBG	
    		printf("Reading address %p contents = %04x\n", ((bb_RegPointer_type)(BaseAddress + 
                                                    (ushort) address + ndx)), *oneBytePtr);
#endif
         }
        break;
     
    case TWO_BYTES:
        twoBytePtr = (ushort*) buffer;
        for(ndx=0; ndx < length; ndx++, twoBytePtr++)
        {
            *twoBytePtr = READ_FROM_HW((bb_RegPointer_type)(BaseAddress +
                                                    (ushort) address + ndx));
#ifdef DBG	
    		printf("Reading address %p contents = %04x\n", ((bb_RegPointer_type)(BaseAddress + 
                                                    (ushort) address + ndx)), *twoBytePtr);
#endif
        }
        break;

case FOUR_BYTES:
        fourBytePtr = (ulong*) buffer;
        
        for(ndx=0; ndx < length; ndx++, fourBytePtr++)
        {
            temp = (ulong) (READ_FROM_HW((bb_RegPointer_type)(BaseAddress + (ushort) address + (ndx * 2))));
            *fourBytePtr = READ_FROM_HW((bb_RegPointer_type)(BaseAddress +
                           (ushort) address + ((ndx* 2) + 1))) << 16 | temp;


#ifdef DBG	
    		printf("Reading address %p contents = %08x\n", ((bb_RegPointer_type)(BaseAddress + 
                                                    (ushort) address + ndx)), *fourBytePtr);
#endif
        }
        
        break;
    case EIGHT_BYTES:
    default:
        return bb_WORD_SIZE_UNSUPPORTED;
        break;
    }
    
    return bb_NO_ERROR;
}


/******************************************************************
* Function : Ixf6048Write
*-----------------------------------------------------------------
* Description  : This routine will write starting at a particular 
*						location and write for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start writing to.
*				length : Number of words to write.
*				buffer : The buffer to contain the read data.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048Write(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
             ulong address, ushort length, void *buffer)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    uint ndx;
    uchar *oneBytePtr;
    ushort *twoBytePtr;
    ulong *fourBytePtr;
    
#ifdef DBG	
    printf("IxfApiGenericWrite pCD = %08x WS = %d addr =\
        %08x len = %d b[0] = %x",
        pChipData, wordSize, address, length, ((uchar *) buffer)[0]);
#endif
    
    switch (wordSize)
    {
    case ONE_BYTE:		
        oneBytePtr = (uchar*) buffer;
        for(ndx=0; ndx < length; ndx++, oneBytePtr++)
        {
#ifdef DBG	
    	printf("Writing to address %p\n", (BaseAddress + ((ushort) address + ndx)));
#endif
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx), *oneBytePtr);
        }
        break;
    case TWO_BYTES:
        twoBytePtr = (ushort*) buffer;
        for(ndx=0; ndx < length; ndx++, twoBytePtr++)
        {
#ifdef DBG	
    	printf("Writing to address %p\n", (BaseAddress + ((ushort) address + ndx)));
#endif
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx), *twoBytePtr);
        }
        break;
    case FOUR_BYTES:
        fourBytePtr = (ulong*) buffer;
        for(ndx=0; ndx < length; ndx++, fourBytePtr++)
        {
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx),
                                            (ushort) (*fourBytePtr >> 16));
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx), 
                                            (ushort) (*fourBytePtr & 0xFFFF));
        }
        break;
    case EIGHT_BYTES:
    default:
        return bb_WORD_SIZE_UNSUPPORTED;
        break;
    }
    
    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048GetBuildVersion
*-----------------------------------------------------------------
* Description  : This routine returns the firmware build version of 
*                the ixf6048 driver.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : drvName : Null terminated string describing the driver
*           date : The build date of the driver.
*           buildVer : The version number of the driver.
*           buildRev : The revision number of the driver.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048GetBuildVersion(bb_ChipData_t* pChipData, char* drvName,
                       char *date, ushort *buildVer, ushort *buildRev)
{
    /* Get the ixf6048 build information */
    strcpy(drvName, IXF6048_BUILD_DRIVER_NAME);
    strcpy(date, IXF6048_BUILD_DATE);
    *buildVer = IXF6048_BUILD_VERSION;
    *buildRev = IXF6048_BUILD_REV;

    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048GetLCDFilter
*-----------------------------------------------------------------
* Description  : This routine will retrieve the LCD Filter value.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section : Determines the section(channel) to get the value.
* Outputs : value : The LCD value.
*-----------------------------------------------------------------
* Returns : Status;
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
#if 0
bb_Error_e 
Ixf6048GetLCDFilter(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    void *value)
{
    ulong *lcdValue;

	lcdValue = (ulong*) value;
	
	/* read the LCD Filter */				
    *lcdValue = READ_FROM_HW(pChipData->BaseAddress + 
                  ixf6048_R_LCDFLTR_CHAN_MSB_OFFSET(section->group.channel));

    /* The the Most Significant part over */
    *lcdValue = *lcdValue << 16;

    /* Read in the Least Significant part */
	*lcdValue |= READ_FROM_HW(pChipData->BaseAddress +
                  ixf6048_R_LCDFLTR_CHAN_LSB_OFFSET(section->group.channel));

    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048SetLCDFilter
*-----------------------------------------------------------------
* Description  : This routine will retrieve the LCD Filter value.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section : Determines the section(channel) to get the value.
*           value : The LCD value.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status;
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
Ixf6048SetLCDFilter(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    void *value)
{
    ushort lcdMsbValue;
    ushort lcdLsbValue;

	lcdMsbValue = (ushort) ((((ulong)value) >> 16) & 0xFFFF);
	lcdLsbValue = (ushort) (((ulong)value) & 0xFFFF);
	
	/* Write MSB LCD Filter */				
    WRITE_TO_HW(pChipData->BaseAddress + 
                ixf6048_R_LCDFLTR_CHAN_MSB_OFFSET(section->group.channel),
                lcdMsbValue);

	/* Write LSB LCD Filter */				
    WRITE_TO_HW(pChipData->BaseAddress + 
                ixf6048_R_LCDFLTR_CHAN_LSB_OFFSET(section->group.channel),
                lcdLsbValue);

    return bb_NO_ERROR;
}
#endif
/******************************************************************
* Function : Ixf6048InitAllocMemory
*-----------------------------------------------------------------
* Description  : This routine dynamically allocates memory within
*				 the "pChipData" parameter for those members that 
*				 need dynamic allocation.
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure without 
*						pointers memory allocation.
* Outputs : pChipData : A pointer to the chip data structure with 
*						pointer memory allocation.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiInit for the completion of the initialization
******************************************************************/
bb_Error_e 
Ixf6048InitAllocMemory(bb_ChipData_t *pChipData)
{
    uint ndx;
    ixf6048_ChipCfg_t* pChipCfg;
    ixf6048_AlarmCfg_t* pAlarmCfg;
    
    /* Allocate memory for ChipCfg */
    pChipData->pChipCfg =
           (ixf6048_ChipCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_ChipCfg_t));	
    
    /* Cast the void pointer */
    pChipCfg = (ixf6048_ChipCfg_t *) pChipData->pChipCfg;
    
    /* Initialize the ChipLevel Configuration */
    
    pChipCfg->pChipLevelCfg = (ixf6048_ChipLevelCfg_t *) 
                            OS_CMD_ALLOC_MEM(sizeof(ixf6048_ChipLevelCfg_t));
    
    
    /* Initialize the Channel Configurations */
    for(ndx=0; ndx < MAX_NUM_CHANNELS; ndx++) 
    {
        pChipCfg->pChanCfg[ndx] = 
             (ixf6048_ChanCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_ChanCfg_t));
        memset(pChipCfg->pChanCfg[ndx], 0, sizeof(ixf6048_ChanCfg_t));

        pChipCfg->pChanCfg[ndx]->pChanIfCfg = 
         (ixf6048_ChanIfCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_ChanIfCfg_t));
        memset(pChipCfg->pChanCfg[ndx]->pChanIfCfg, 0, sizeof(ixf6048_ChanIfCfg_t));

        pChipCfg->pChanCfg[ndx]->pOhtChanCfg = 
         (ixf6048_OhtChanCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_OhtChanCfg_t));
        memset(pChipCfg->pChanCfg[ndx]->pOhtChanCfg, 0, sizeof(ixf6048_OhtChanCfg_t));

        pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxRstCfg.pExpectedJ0String = 
            (uchar *) OS_CMD_ALLOC_MEM(sizeof(uchar) * 64);
        memset(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxRstCfg.pExpectedJ0String, 0, sizeof(uchar) * 64);

        pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxHptCfg.pExpectedJ1String = 
            (uchar *) OS_CMD_ALLOC_MEM(sizeof(uchar) * 64);
        memset(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxHptCfg.pExpectedJ1String, 0, sizeof(uchar) * 64);

        pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxRstCfg.pJ0String = 
            (uchar *) OS_CMD_ALLOC_MEM(sizeof(uchar) * 64);
        memset(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxRstCfg.pJ0String, 0, sizeof(uchar) * 64);

	    pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxHptCfg.pTxJ1String = 
            (uchar *) OS_CMD_ALLOC_MEM(sizeof(uchar) * 64);
        memset(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxHptCfg.pTxJ1String, 0, sizeof(uchar) * 64);

        pChipCfg->pChanCfg[ndx]->pAtmChanCfg = 
         (ixf6048_AtmChanCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_AtmChanCfg_t));
        memset(pChipCfg->pChanCfg[ndx]->pAtmChanCfg, 0, sizeof(ixf6048_AtmChanCfg_t));

        pChipCfg->pChanCfg[ndx]->pPosChanCfg = 
         (ixf6048_PosChanCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_PosChanCfg_t));
        memset(pChipCfg->pChanCfg[ndx]->pPosChanCfg, 0, sizeof(ixf6048_PosChanCfg_t));
    }
    
    /* Allocate memory for AlarmCfg */
    pChipData->pAlarmCfg = 
                 (ixf6048_AlarmCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf6048_AlarmCfg_t));
    memset(pChipData->pAlarmCfg, 0, sizeof(ixf6048_AlarmCfg_t));
    
    /* Cast the void pointer */
    pAlarmCfg = (ixf6048_AlarmCfg_t *) pChipData->pAlarmCfg;
    
    /* Initialize the Channel Alarm Configurations */
    for(ndx=0; ndx < MAX_NUM_CHANNELS; ndx++) 
    {
        pAlarmCfg->pChanAlarmCfg[ndx] = (ixf6048_ChanAlarmCfg_t *) 
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_ChanAlarmCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx], 0, sizeof(ixf6048_ChanAlarmCfg_t));

        pAlarmCfg->pChanAlarmCfg[ndx]->pOhtChanAlarmCfg = (ixf6048_OhtChanAlarmCfg_t *)
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_OhtChanAlarmCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx]->pOhtChanAlarmCfg, 0, sizeof(ixf6048_OhtChanAlarmCfg_t));

        pAlarmCfg->pChanAlarmCfg[ndx]->pUtopiaAlarmsCfg = (ixf6048_UtopiaAlarmsCfg_t *) 
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_UtopiaAlarmsCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx]->pUtopiaAlarmsCfg, 0, sizeof(ixf6048_UtopiaAlarmsCfg_t));

        pAlarmCfg->pChanAlarmCfg[ndx]->pAtmChanAlarmCfg = (ixf6048_AtmChanAlarmCfg_t *) 
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_AtmChanAlarmCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx]->pAtmChanAlarmCfg, 0, sizeof(ixf6048_AtmChanAlarmCfg_t));

        pAlarmCfg->pChanAlarmCfg[ndx]->pPosChanAlarmCfg = (ixf6048_PosChanAlarmCfg_t *) 
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_PosChanAlarmCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx]->pPosChanAlarmCfg, 0, sizeof(ixf6048_PosChanAlarmCfg_t));

        pAlarmCfg->pChanAlarmCfg[ndx]->pMiscAlarmsCfg = (ixf6048_MiscAlarmsCfg_t *) 
            OS_CMD_ALLOC_MEM(sizeof(ixf6048_MiscAlarmsCfg_t));
        memset(pAlarmCfg->pChanAlarmCfg[ndx]->pMiscAlarmsCfg, 0, sizeof(ixf6048_MiscAlarmsCfg_t));

    }

    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048DeAllocMemory
*-----------------------------------------------------------------
* Description  : This routine frees the dynamically allocated memory
*				 within the "pChipData" structure
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048DeAllocMemory(bb_ChipData_t *pChipData)
{
    uint ndx;
    ixf6048_ChipCfg_t* pChipCfg;
    ixf6048_AlarmCfg_t* pAlarmCfg;
    
    /* Deallocate memory for ChipCfg */
    if (pChipData->pChipCfg) 
    {
        /* Cast the void pointer */
        pChipCfg = (ixf6048_ChipCfg_t *) pChipData->pChipCfg;
           
        /* Cycle thru the channels */
        for(ndx=0; ndx < MAX_NUM_CHANNELS; ndx++) 
        {			
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pPosChanCfg);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pAtmChanCfg);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxRstCfg.pExpectedJ0String);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->RxHptCfg.pExpectedJ1String);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxRstCfg.pJ0String);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pOhtChanCfg->TxHptCfg.pTxJ1String);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pOhtChanCfg);           
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]->pChanIfCfg);
            OS_CMD_DEALLOC_MEM(pChipCfg->pChanCfg[ndx]);
        }  
        
        OS_CMD_DEALLOC_MEM(pChipCfg->pChipLevelCfg);
        OS_CMD_DEALLOC_MEM(pChipData->pChipCfg);
                
    }
    else 
    {
        /* Exit with error.  
        The chip configuration parameter is not initialized */
        return bb_NO_CHIP_CFG;
    }
    
    /* Deallocate memory for AlarmCfg */
    if (pChipData->pAlarmCfg) 
    {
        pAlarmCfg = (ixf6048_AlarmCfg_t *) pChipData->pAlarmCfg;

        /* Cycle thru the channels */
        for(ndx=0; ndx < MAX_NUM_CHANNELS; ndx++) 
        {
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]->pMiscAlarmsCfg);
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]->pPosChanAlarmCfg);
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]->pAtmChanAlarmCfg);
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]->pUtopiaAlarmsCfg);
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]->pOhtChanAlarmCfg);
            OS_CMD_DEALLOC_MEM(pAlarmCfg->pChanAlarmCfg[ndx]);
        }
        
        OS_CMD_DEALLOC_MEM(pChipData->pAlarmCfg);

    } 
    else 
    {
        
        /* Exit with error.  
        The chip configuration parameter is not initialized */
        return bb_NO_ALARM_CFG;
    }
    
    return bb_NO_ERROR;
}

/******************************************************************
* Function : Ixf6048InitChip
*-----------------------------------------------------------------
* Description  : This routine commits the internal register table
*                down to the chip and initializes the bb_ChipData_t
*                structure.
*-----------------------------------------------------------------
* Inputs  : pChipData	: A pointer to an allocated structure.
*           pTable : Table to be committed to the device.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further initiali-
*			zation.
******************************************************************/
bb_Error_e 
Ixf6048InitChip(bb_ChipData_t* pChipData, InitRegTable_t *pTable)
{
    bb_Error_e rc = bb_NO_ERROR;
    InitRegTable_t *pRegPtr;

    /* Initialize the register pointer */
    pRegPtr = pTable;

    /* Loop through the array commiting the data down to the chip */
    while ((pRegPtr->offset != (ushort*)0xFFFF) &&
           (pRegPtr->value  != IXF_INIT_DATA_DELIMITER))
    {
        WRITE_TO_HW((pChipData->BaseAddress + 
           ((ulong) pRegPtr->offset << REG_SHIFT)), pRegPtr->value);
        pRegPtr++;
    }

    return rc;
}

/******************************************************************
* Function : Ixf6048GetWindowSize
*-----------------------------------------------------------------
* Description  : Gets the window size for clear and setting 
*                Degraded B2 error.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						window size from.
*           mode      : The clearing/setting mode.
* Outputs : value  : Window size value.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048GetWindowSize(bb_ChipData_t* pChipData,
                     bb_ChipSegment_t *section,
                     bb_WindowSizeMode_e mode, ulong* value)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    ushort offset;
    uchar channel;
   
    /*  Getting the channel number */
    channel = ixf6048_GET_CHANNEL_FROM_SECTION_PARAM(section);
    
    /* Get the proper offset */
    if (mode == SETTING)
    {
        offset = (ushort) ixf6048_WINSZ_SDEGB2_CHAN_LSB_OFFSET(channel);
    } 
    else
    {
        offset = (ushort) ixf6048_WINSZ_CDEGB2_CHAN_LSB_OFFSET(channel);
    }
    
    /* Disable preemption because of the 32-bit register
	   double read */
    OS_CMD_DISABLE_PREEMPTION();
    
    /* Get Windows Size */
    *value = ixf6048_GET_NUMB_WINSZ_32BIT(READ_FROM_HW(BaseAddress + offset + 1),
                                          READ_FROM_HW(BaseAddress + offset));

    /* Enable preemption because of the 32-bit register
	   double read */
    OS_CMD_ENABLE_PREEMPTION();

    return bb_NO_ERROR;
}


/******************************************************************
* Function : Ixf6048SetWindowSize
*-----------------------------------------------------------------
* Description  : Sets the window size for clear and setting 
*                Degraded B2 error.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to set the
*  						window size.
*           mode      : The clearing/setting mode.
*           value     : Window size value.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048SetWindowSize(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                     bb_WindowSizeMode_e mode, ulong value)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    uchar channel;
    
    /*  Getting the channel */
    channel = ixf6048_GET_CHANNEL_FROM_SECTION_PARAM(section);

    /* Disable preemption because of the 32-bit register
	   double read */
    OS_CMD_DISABLE_PREEMPTION();

    /* Get the proper offset */
    if (mode == SETTING)
    {
        /* Set Windows Size */
        ixf6048_SET_LSB_SDEGB2_WINDOW_SIZE(BaseAddress,
                                             channel, (ushort)(value & 0xFFFF));
        ixf6048_SET_MSB_SDEGB2_WINDOW_SIZE(BaseAddress,
                                     channel, (ushort) ((value >> 16) & 0xFFFF));    
    } 
    else
    {
        /* Set Windows Size */
        ixf6048_SET_CLR_LSB_DEGB2_WINDOW_SIZE(BaseAddress,
                                             channel, (ushort)(value & 0xFFFF));
        ixf6048_SET_CLR_MSB_DEGB2_WINDOW_SIZE(BaseAddress,
                                     channel, (ushort) ((value >> 16) & 0xFFFF));
    }
    
    /* Enable preemption because of the 32-bit register
	   double read */
    OS_CMD_ENABLE_PREEMPTION();
    
    return bb_NO_ERROR;
}


/******************************************************************
* Function : Ixf6048SetOpMode
*-----------------------------------------------------------------
* Description  : Sets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
*           pModeCfg  : place to put mode.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
Ixf6048SetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e opMode, void *pModeCfg)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    bb_RegPointer_type pReg;
    bb_Error_e rc = bb_NO_ERROR;
    ushort chanNum = section->group.channel;

    switch (opMode)
    {
    case ATM_MODE:
        if ((section->block.flow == TX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_ATM));
        }
        if ((section->block.flow == RX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_ATM));
        }
        break;
    case POS_MODE:
        if ((section->block.flow == TX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_POS));
        }
        if ((section->block.flow == RX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_POS));
        }
        break;
    case TRANSPARENT_MODE:
        if ((section->block.flow == TX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_TRANSPARENT));
        }
        if ((section->block.flow == RX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_TRANSPARENT));
        }
        break;
    case TEST_MODE: /* PRBS */
        if ((section->block.flow == TX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_TEST));
        }
        if ((section->block.flow == RX) || (section->block.flow == BOTH_FLOWS))
        {
            pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(chanNum);
            WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bCHAN_MODE_TEST));
        }
        break;
    default:
        rc = bb_INV_PARAMETER;
        break;
    }

    return bb_NO_ERROR;
}

ushort
SlowPortRead(bb_RegPointer_type addr)
{
	return SWAPNTOHS(*addr);
}

ushort
SlowPortWrite(bb_RegPointer_type addr, ushort val)
{
    if(((silicon_rev & 0xF0) >> 4))
    {
    }
    else
    {
        addr = (bb_RegPointer_type)((int)addr^2);
    }

    *addr = val;

	return bb_NO_ERROR;
}


