/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: lagarwal $
 *   DATE:           $Date: 2002/11/15 17:51:11 $
 *   LAST MODIFIED:  $Modtime: 5/16/02 5:31p $
 *   REV#:           $Revision: 1.1.1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/include/bkcompat.h $
 *   WORKFILE:       $Workfile: bkcompat.h $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6192
 *
 * Purpose: Provides the API for Ixf6048 Specific routines.  This
 *          is meant to glue the new and old api together for 
 *          backward compatibilty.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: bkcompat.h,v $
 *   Revision 1.1.1.1  2002/11/15 17:51:11  lagarwal
 *   rename dir
 *
 *   Revision 1.4  2002/09/09 21:49:21  cmah1
 *   no message
 *
 *   Revision 1.2  2002/07/08 14:13:28  machambe
 *   no message
 *
 *   Revision 1.1  2002/05/22 00:05:41  machambe
 *   Initial update
 *
 * 
 * 8     5/16/02 6:10p Machambe
 * Added 2nd param to InitChip
 * 
 * 7     4/05/02 12:42p Machambe
 * Adding more routines for IXF API support
 * 
 * 6     3/26/02 6:23p Machambe
 * 
 * 5     12/01/01 11:35p Machambe
 * 
 * 4     6/25/01 12:02p Machambe
 * 
 * 3     6/22/01 4:54p Machambe
 * 
 * 2     5/03/01 3:54p Machambe
 * 
 * --------------------------------------------------------------------------
 */ 

#ifndef _BKCOMPAT_H_ /* Include file contents only once per compilation */
#define _BKCOMPAT_H_

/******************************************************************
* Function : Ixf6048GetStatus
*-----------------------------------------------------------------
* Description  : Gets the Status.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						status from.
*           selStatus : Type of Status: 
* Outputs : pStatus	: place to put Status.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048GetStatus(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_SelStatus_e selStatus, void* pStatus);

/******************************************************************
* Function : Ixf6048GetOpMode
*-----------------------------------------------------------------
* Description  : Gets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						status from.
*           opMode    : The mode of operation.
* Outputs : pModeCfg  : Place to put mode.
*-----------------------------------------------------------------
* Returns : Stst
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048GetOpMode(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_OperMode_e *opMode, void* pModeCfg);

/******************************************************************
* Function : Ixf6048GetTrace
*-----------------------------------------------------------------
* Description  : Gets the Rx J0/J1/J2 Trace
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*				section : Determines the section(block) to get the trace
*  						from.
*				TraceType : Type of Trace: Rx J0/J1/J2 for system and
*								line side.
* Outputs : pTrace	: place to return Rx Trace String.
*           pLength : The length of pTrace.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : Note: at least 65 bytes long for pTrace
******************************************************************/
extern bb_Error_e 
Ixf6048GetTrace(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
			   bb_TraceType_e TraceType, char* pTrace, ushort* pLength);

/******************************************************************
* Function : Ixf6048SetTrace
*-----------------------------------------------------------------
* Description  : Set Expected/Tx J0/J1 Trace
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to get the trace
*  						from.
*			TraceType : Type of Trace: Expected/Tx J0/J1 for system
*						or line side.
*			TraceFormat : Specifies the Trace Format
*			pTrace	: null-terminated Trace String (data only)
*                     Driver adds CRC7, or LF/CR, if needed
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*    Sets the new Trace String in the Associated Storage Structure 
*    Sets the new TraceFormat in the Associated Storage Structure
*    Error Condition if Unsuccessful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
*   The bb_TraceType_e value for  
*          bb_RX_J0 and bb_RX_J1
*   must be Set to "Ignore Jn Trace (No Trace)". 
*   This Condition is Forced in this Function.
*    --> see R_Jn_ESTRA Registers
******************************************************************/
extern bb_Error_e 
Ixf6048SetTrace(bb_ChipData_t* pChipData, bb_ChipSegment_t *section, 
			   bb_TraceType_e TraceType, bb_TraceFormat_e TraceFormat,
			   char* pTrace);


/******************************************************************
* Function : Ixf6048GetCounters
*-----------------------------------------------------------------
* Description  : Retrieve a set of Counters
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section :	Determines the section(block) to get the
*						counters from.
*			SelCounters :  Selects the set of counters
* Outputs : pCounters : Place to put Counters
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048GetCounters(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				  bb_SelCounters_e SelCounters, void* pCounters);

/******************************************************************
* Function : Ixf6048InitAlarmCallback
*-----------------------------------------------------------------
* Description  : Initialize the Driver
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				pAlarmCallback : Alarm Callback
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048InitAlarmCallback(bb_ChipData_t* pChipData, AlarmCallBack pAlarmCallbackArg);

/******************************************************************
* Function : Ixf6048ChipIsr
*-----------------------------------------------------------------
* Description  : Handle Interupts for Chip
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments:   Interrupts are reported through a Callback function.
*
*	 Transmit and receive Interrupts act independantly of each another.
*
*   Interrupts have been prioritized to be handled according to the severity 
*   of the effecting interupt. See the MasterInterruptHandler function for 
*	 specifics
******************************************************************/
extern bb_Error_e 
Ixf6048ChipIsr(bb_ChipData_t* pChipData);

/******************************************************************
* Function : Ixf6048GetChipInfo
*-----------------------------------------------------------------
* Description  : Get the chip information.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : pChipInfo : Place to return chip info
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048GetChipInfo(bb_ChipData_t* pChipData, bb_ChipInfo_t* pChipInfo);


/******************************************************************
* Function : Ixf6048SetChipOnline
*-----------------------------------------------------------------
* Description  : Puts some or all of the chip online.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section	 :	Specifies which portion of the chip to 
*								put online.
Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None. 
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048SetChipOnline(bb_ChipData_t* pChipData, bb_ChipSegment_t *section);

/******************************************************************
* Function : Ixf6048SetChipOffline
*-----------------------------------------------------------------
* Description  : Takes some or all of the chip offline.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section	 :	Specifies which portion of the chip to 
*								take offline.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048SetChipOffline(bb_ChipData_t* pChipData, bb_ChipSegment_t *section);

/******************************************************************
* Function : Ixf6048SetOhByte
*-----------------------------------------------------------------
* Description  : Set an Overhead Byte: K1,K2, Expected/Tx C2,
*				 HPM RDI in G1
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to set the byte
*			SelOhByte : Selects a particular Overhead byte 
*			OhByte : Value of selected Overhead byte
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048SetOhByte(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				bb_SelOhBytes_e SelOhByte, ushort OhByte);

/******************************************************************
* Function : Ixf6048GetOhBytes
*-----------------------------------------------------------------
* Description  : Gets Overhead Bytes: Rx MST, Rx HPM
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*			section : Determines the section(block) to get the bytes
*			SelOhBytes : Selects the Overhead Bytes 
* Outputs : pOhBytes : place to put Overhead Bytes, of type:
bb_RxMstBytes_t, or ixf6048_RxHpmBytes_t
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e
Ixf6048GetOhBytes(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				 bb_SelOhBytes_e SelOhBytes, void* pOhBytes);

/******************************************************************
* Function : Ixf6048Reset
*-----------------------------------------------------------------
* Description  : Reset (chip or a section of the chip) and then 
*				 reconfigure it. To restart traffic, the chip or 
*				 section of chip must be set back online
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section : Determines the section(block) to reset
*           resetType : Specifies type of Reset
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048Reset(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
			 bb_SelResetType_e resetType);

/******************************************************************
* Function : Ixf6048CfgTest
*-----------------------------------------------------------------
* Description  : Run Tests
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				testType : The type of test to run.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e
Ixf6048CfgTest(bb_ChipData_t* pChipData, bb_ChipSegment_t *section, 
               bb_TestType_e testType, void *ptTestCfg);

/******************************************************************
* Function : Ixf6048SetAlarmCfg
*-----------------------------------------------------------------
* Description  : Edits Interrupt Enabled Masks for a Specific Set
*						of Interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				section : Determines the section(block) of the alarm(s)
*							 to edit.
*				AlarmType : Specifies the type of Alarm
*				pAlarmCfg : A pointer to the alarm structure.
* Outputs : 
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048SetAlarmCfg(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
				  bb_AlarmType_e AlarmType, void* pAlarmCfg);

/******************************************************************
* Function : Ixf6048Read
*-----------------------------------------------------------------
* Description  :
* This routine will read starting at a particular 
*						location and read for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start reading from.
*				length : Number of words to read.
* Outputs : buffer : The buffer to contain the read data.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048Read(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
				  ulong address, ushort length, void *buffer);

/******************************************************************
* Function : Ixf6048Write
*-----------------------------------------------------------------
* Description  : This routine will write starting at a particular 
*						location and write for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start writing to.
*				length : Number of words to write.
*				buffer : The buffer to contain the read data.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048Write(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
				   ulong address, ushort length, void *buffer);

/******************************************************************
* Function : Ixf6048GetBuildVersion
*-----------------------------------------------------------------
* Description  : This routine returns the firmware build version of 
*                the ixf6192 driver.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : drvName : Null terminated string describing the driver
*           date : The build date of the driver.
*           buildVer : The version number of the driver.
*           buildRev : The revision number of the driver.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048GetBuildVersion(bb_ChipData_t* pChipData, char* drvName,
                       char *date, ushort *buildVer, ushort *buildRev);

/******************************************************************
* Function : Ixf6048GetLCDFilter
*-----------------------------------------------------------------
* Description  : This routine will retrieve the LCD Filter value.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section : Determines the section(channel) to get the value.
* Outputs : value : The LCD value.
*-----------------------------------------------------------------
* Returns : Status;
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048GetLCDFilter(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    void *value);

/******************************************************************
* Function : Ixf6048SetLCDFilter
*-----------------------------------------------------------------
* Description  : This routine will retrieve the LCD Filter value.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*			section : Determines the section(channel) to get the value.
*           value : The LCD value.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status;
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
Ixf6048SetLCDFilter(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    void *value);

/******************************************************************
* Function : Ixf6048InitAllocMemory
*-----------------------------------------------------------------
* Description  : This routine dynamically allocates memory within
*				 the "pChipData" parameter for those members that 
*				 need dynamic allocation.
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure without 
*						pointers memory allocation.
* Outputs : pChipData : A pointer to the chip data structure with 
*						pointer memory allocation.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiInit for the completion of the initialization
******************************************************************/
extern bb_Error_e 
Ixf6048InitAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************
* Function : Ixf6048DeAllocMemory
*-----------------------------------------------------------------
* Description  : This routine frees the dynamically allocated memory
*				 within the "pChipData" structure
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048DeAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************
* Function : Ixf6048InitChip
*-----------------------------------------------------------------
* Description  : This routine commits the internal register table
*                down to the chip and initializes the bb_ChipData_t
*                structure.
*-----------------------------------------------------------------
* Inputs  : pChipData	: A pointer to an allocated structure.
*           pTable : Table to be committed to the device.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further initiali-
*			zation.
******************************************************************/
extern bb_Error_e 
Ixf6048InitChip(bb_ChipData_t* pChipData, InitRegTable_t *pTable);

/******************************************************************
* Function : Ixf6048GetWindowSize
*-----------------------------------------------------------------
* Description  : Gets the window size for clear and setting 
*                Degraded B2 error.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						window size from.
*           mode      : The clearing/setting mode.
* Outputs : value  : Window size value.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048GetWindowSize(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                     bb_WindowSizeMode_e mode, ulong* value);

/******************************************************************
* Function : Ixf6048SetWindowSize
*-----------------------------------------------------------------
* Description  : Sets the window size for clear and setting 
*                Degraded B2 error.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to set the
*  						window size.
*           mode      : The clearing/setting mode.
*           value     : Window size value.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048SetWindowSize(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                     bb_WindowSizeMode_e mode, ulong value);

/******************************************************************
* Function : Ixf6048SetOpMode
*-----------------------------------------------------------------
* Description  : Sets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
*           pModeCfg  : place to put mode.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
Ixf6048SetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e opMode, void *pModeCfg);

#endif   /* _BKCOMPAT_H_ */
