/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: lagarwal $
 *   DATE:           $Date: 2002/11/15 17:51:11 $
 *   LAST MODIFIED:  $Modtime: 5/17/02 1:58p $
 *   REV#:           $Revision: 1.1.1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf6048/ixf6048.c $
 *   WORKFILE:       $Workfile: bkcompat.c $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6048
 *
 * Purpose: IXF6048 bits & functions - used only by the driver
 *			
 * Notes: Only the values required by the Driver are defined.
 *    Note that addresses in the Data Sheet are referenced to 16-bit words,
 *    and in fact are word addresses. Depending on your compiler and your
 *	  uP, it may be necessary to double the Register offsets
 * 	  in order to Generate the correct 16-bit aligned addresses. 
 * 		- Some Compilers will automatically double these offset, while 
 *		  others will require that the author double the values.
 *      - Some uP's do Not support odd addresses for 16-Bit accesses.
 *    To Resolve these issues and others, the constant:
 *				REG_SHIFT
 *	  has been included in the Address Macro, and should be Defined as 
 *    a value of 0 or 1 depending on your Addressing requirements.
 *
 *    Note that only 16-bit accesses are supported by the registers.
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: ixf6048b.h,v $
 *   Revision 1.1.1.1  2002/11/15 17:51:11  lagarwal
 *   rename dir
 *
 *   Revision 1.6  2002/10/31 19:03:23  machambe
 *   no message
 *
 *   Revision 1.2  2002/10/27 18:21:36  machambe
 *   no message
 *
 * --------------------------------------------------------------------------
 */ 
#ifndef BB6048B_H_ /* Include file contents only once per compilation */
#define BB6048B_H_

/* See Comment in File header above */
#ifdef EMBEDDED_ENV
#define REG_SHIFT 1
#else								/* SFT_PLATFORM */
#define REG_SHIFT 0
#endif 

/*Get Shift Bit Macro*/
#define	ixf6048_GET_SHIFT_BIT(StartBit, Shift)   (StartBit<<Shift)

/* Bit Shifts */
#define ixf6048_BIT_SHIFT_0	0
#define ixf6048_BIT_SHIFT_1	1
#define ixf6048_BIT_SHIFT_2	2
#define ixf6048_BIT_SHIFT_3	3
#define ixf6048_BIT_SHIFT_4	4
#define ixf6048_BIT_SHIFT_5	5
#define ixf6048_BIT_SHIFT_6	6
#define ixf6048_BIT_SHIFT_7	7
#define ixf6048_BIT_SHIFT_8	8
#define ixf6048_BIT_SHIFT_9	9
#define ixf6048_BIT_SHIFT_10	10
#define ixf6048_BIT_SHIFT_11	11
#define ixf6048_BIT_SHIFT_12	12
#define ixf6048_BIT_SHIFT_13	13
#define ixf6048_BIT_SHIFT_14	14
#define ixf6048_BIT_SHIFT_15	15

/* Default register Value of 0 */
#define ixf6048_ZERO_DFLT 0

/***********************************************************************
 * Global Registers: MACNF: Microprocessor Access Config Reg
 * Page 186 - (000)01H
 **********************************************************************/

#define  ixf6048_MACNF_OFFSET (0x01 << REG_SHIFT) /* Byte Offset of register */

#define  ixf6048_bBYTE_CHG_UPD_DSBL  0x0004  /* Set to Disable receive Byte 
                                            Regster Updates when Byte change
                                            interupt is active. Default= 0 */
#define  ixf6048_bCNTR_TEST_EN    0x0002     /* Set to force counter overflow */
                                            /* on B1 bit & Bit/Blk for B2,
                                               M1 REI, B3, B1 REI Counters */
#define  ixf6048_bMAS_INT_EN      0x0001     /* Set to enable chip interrupts */

/***********************************************************************
 * Global Registers: CHIP_ID: Chip ID & Version Number
 * Page 186 - (000)02H
 **********************************************************************/

#define  ixf6048_CHIP_ID_OFFSET (0x02 << REG_SHIFT) /* Byte Offset of register */

#define  ixf6048_mCHIP_VER      0xFF00   /* MSB Mask for Chip Version */
#define  ixf6048_mCHIP_ID       0x00FF   /* LSB Mask for Chip ID */

/***********************************************************************
 * Global Registers: SRESET: Software Reset Register
 * Reg  - (000)03H
 * Set (Writing 1 bit) => held in reset
 * Clr (Writing 0 bit) => comes out of reset
 * Note: Even if bit was set (& chip/channel held in reset) it will read back 0
 * Note: To clear a reset for a bit, write 0x0000 to register.
 **********************************************************************/

#define  ixf6048_SW_RESET_OFFSET (0x03 << REG_SHIFT) /* Byte Offset of register */

#define  ixf6048_bSW_RESET_CHIP      0x0100 /* Reset entire Amazon chip */

/* Set or clear the reset transmit channel reset register. */
/* Note: SetClr Must be 0 or 1                             */
#define  ixf6048_RESET_XMT_CH(BaseAddress, Chan, SetClr)   \
             (WRITE_TO_HW(BaseAddress + ixf6048_SW_RESET_OFFSET,   \
                  ( (0x0010 << (Chan)  ) &  \
                    (SetClr << (Chan+4)) ) ) )

#define  ixf6048_bRESET_XMT_CH(Chan, SetClr) ((0x0010<<(Chan)) &  \
                                                (SetClr << (Chan+4)) )

#define  ixf6048_RESET_RCV_CH(BaseAddress, Chan, SetClr)   \
             (WRITE_TO_HW(BaseAddress + ixf6048_SW_RESET_OFFSET,   \
                  ( (0x0001<<(Chan) ) &  \
                    (SetClr << Chan ) ) ) )

#define  ixf6048_bRESET_RCV_CH(Chan, SetClr) ((0x0001<<(Chan)) &  \
                                                (SetClr << (Chan)) )

#define  ixf6048_RESET_CHANNEL(BaseAddress, Chan, SetClr)   \
             (WRITE_TO_HW(BaseAddress + ixf6048_SW_RESET_OFFSET,   \
                  ( (0x0011<<(Chan) ) &  \
                    ((SetClr << Chan )|(SetClr << (Chan+4)) ) ) ) )

/***********************************************************************
 * Global Registers: BUF_ACNTS: Buffer all counters -- Page 188 - (000)04H
 **********************************************************************/

#define  ixf6048_BUF_ACNTS_OFFSET (0x04 << REG_SHIFT)

         /* Chan = 0-3 below */
/* Mask to Buffer all OHT Rx Counters for channel. Tx is Unused */
#define  ixf6048_MASK_OHT_CHAN_CNTRS(Chan) \
            (0x1100 << Chan)     
/* Mask to Buffer all ATM/POS Rx,Tx Counters for chan */
#define  ixf6048_MASK_ATM_POS_CHAN_CNTRS(Chan)   \
         (0x0011 << Chan)     
/* Mask to Buffer all ATM/POS Rx Counters for chan */
#define  ixf6048_MASK_ATM_POS_RX_CHAN_CNTRS(Chan)   \
         (0x0001 << Chan)     
/* Mask to Buffer all ATM/POS Tx Counters for chan */
#define  ixf6048_MASK_ATM_POS_TX_CHAN_CNTRS(Chan)   \
         (0x0010 << Chan)     

/* Buffer the counters defined by "Mask" */
#define  ixf6048_BUFFER_COUNTERS(BaseAddress, ChanMask) \
            ((WRITE_TO_HW(BaseAddress + ixf6048_BUF_ACNTS_OFFSET, ChanMask)))
/***********************************************************************
 * Global Registers: SDH_GIS: SDH Global Interrupt Source
 * Page 189 - (000)05H
 *
 * Use:
 * 1) If any of 16-bits in reg not = 0, then => SDH interrupt(s) to handle
 * 2) Check each channel 0-3 for active SDH interrupt, if so:
 *    3) Check (masked bits for 1 channel) against type of interrupt:
 *       namely: Rx RST, Rx MST, Rx MSA, Rx HPT.
 **********************************************************************/

#define  ixf6048_SDH_GIS_OFFSET (0x05 << REG_SHIFT) 

#define  ixf6048_SDH_CHAN_INT(Chan) \
         (0x1111 << Chan)     /* to see if any interrupt for specific chan */

#define  ixf6048_RX_HPT_INT   0xF000   /* Test for Rx HPT int for any chan */
#define  ixf6048_RX_ADP_INT   0x0F00   /* Test for Rx MSA int for any chan */
#define  ixf6048_RX_MUX_INT   0x00F0   /* Test for Rx MST int for any chan */
#define  ixf6048_RX_REG_INT   0x000F   /* Test for Rx RG  int for any chan */

/***********************************************************************
 * Global Registers: ATMPOS_GIS: ATM & POS Global Interrupt Source
 * Page  - (000)06H
 *
 * Use:
 * 1) If any of 16-bits in reg not = 0, then => ATM or POS int(s) to handle
 * 2) Check each channel 0-3 for active ATM/POS interrupt, if so:
 *    3) Check (masked bits for 1 channel) against type of interrupt:
 *       namely: Rx ATM or Tx ATM (if ATM chan)
 *               Rx POS or Tx POS (if POS chan)
 **********************************************************************/

#define  ixf6048_ATMPOS_GIS_OFFSET (0x06 << REG_SHIFT) 

#define  ixf6048_ATM_CHAN_INT(Chan) \
         (0x1100 << Chan)     /* to see if ATM interrupt for specific chan */

#define  ixf6048_POS_CHAN_INT(Chan) \
         (0x0011 << Chan)     /* to see if POS interrupt for specific chan */

#define  ixf6048_TX_ATM_INT   0xF000   /* Test for Tx ATM int for any chan */
#define  ixf6048_RX_ATM_INT   0x0F00   /* Test for Rx ATM int for any chan */
#define  ixf6048_TX_POS_INT   0x00F0   /* Test for Tx POS int for any chan */
#define  ixf6048_RX_POS_INT   0x000F   /* Test for Rx POS int for any chan */

/***********************************************************************
 * Global Registers: S_AIS: SDH/SONET Rx AIS
 * Page 190 - (000)07H
 * This Register is primarily used for Testing purposes. It indicates the
 * internal status of the internal chip logic for AIS generation Processes.
 **********************************************************************/

#define  ixf6048_S_AIS_OFFSET (0x07 << REG_SHIFT) 

#define  ixf6048_GEN_S_AIS_ST(Chan) \
         (0x1111 << Chan)     /*Present Status of AIS generator */

#define  ixf6048_GEN_RST_AIS_ST 0xF000  /* Mask Status of RST AIS generator */
#define  ixf6048_GEN_MST_AIS_ST 0x0F00  /* Mask Status of MST AIS generator */
#define  ixf6048_GEN_MSA_AIS_ST 0x00F0  /* Mask Status of MSA AIS generator */
#define  ixf6048_GEN_HPT_AIS_ST 0x000F  /* Mask Status of HPT AIS generator */
/***********************************************************************
 * Global Registers: GOCNF: Global Operational Configuration
 * Page 191 - (000)08H
 * This Register configures AMAZON-A global configuration features.
 **********************************************************************/

#define  ixf6048_GOCNF_OFFSET (0x08 << REG_SHIFT) 

/* PSN_ADDED B1 Release Changes */
#define  ixf6048_bGEN_IO_VAL         0x2000
#define  ixf6048_bGEN_IO_MODE        0x1000
#define  ixf6048_bRCV_U_OUT_EN_CNF   0x0800
#define  ixf6048_bXMT_U_OUT_EN_CNF   0x0400
#define  ixf6048_bXMT_PECL_MSB_CNF   0x0200
#define  ixf6048_bRCV_PECL_MSB_CNF   0x0100
#define  ixf6048_UTOPIA_OUT_ENABLE	 0x0010
#define  ixf6048_AMAZON_OUT_ENABLE   0x0008
/* PSN_END B1 Release Changes */

#define  ixf6048_bU64_MODE     0x0080  /* Configure UTOPIA Interface to 64 Bit Mode */

#define  ixf6048_bQUAD_MODE    0x0040  /* Line Side = Quad Channel Mode */

#define  ixf6048_bC_MODE       0x0020  /* Set => Line Side = Concatenated Mode */
                                    /* Note: Only used if Single Chan Mode */

                                    /* UTOPIA & I/O bus Outputs En/Disabled */
#define  ixf6048_CHIP_ENABLE    0x0018/* |= to Set => Enabled */
#define  ixf6048_CHIP_DISABLE   0xFFE7/* &= to Clr => Disabled */

                                    /* Note: Masked value cannot = 0x7 */
#define  ixf6048_mUTOPIA_ADDR_BASE 0x0007   /* Mask for UTOPIA Addr Base */


/***********************************************************************
 * Channel Registers: COCNF: Channel Operational Configuration
 * Page 193 - (0cc)09H
 **********************************************************************/

#define  ixf6048_COCNF_CHAN_OFFSET(Chan)  \
         (((0x09)+(Chan<<8)) << REG_SHIFT)

#define  ixf6048_bCHAN_REPEATER_MODE  0x0020   /* Chan Mode = Repeater  */

#define  ixf6048_bCHAN_SYS_LOOPBACK   0x0010   /* Enable Local Loopback    */

  /* This requires that the Rx and Tx Line Side interfaces are the same: 
     namely RxLineSideInterface = TxLineSideInterface in MuxChanIfConfig */
#define  ixf6048_bLINE_LOOP_BACK   0x0008   /* Enable Line Loopback   */

#define  ixf6048_bCH_ENA          0x0004   /* Enable Channel traffic */

/* */
#define  ixf6048_mCHAN_RATE       0x0003   /* Mask for Chan Rate bits */
#define  ixf6048_bCHAN_RATE_OC48  0x0003   /* Chan rate = OC-48 (Single Chan - Chan 0)*/
#define  ixf6048_bCHAN_RATE_OC12  0x0002   /* Chan rate = OC-12           */
#define  ixf6048_bCHAN_RATE_OC3   0x0001   /* Chan rate = OC-3            */
#define  ixf6048_bCHAN_RATE_OC1   0x0000   /* Chan rate = OC-1            */


/***********************************************************************
 * Channel Registers: R_COCNF: Rx Channel Operational Configuration
 * Page 195-6 - (0cc)0AH
 **********************************************************************/

#define  ixf6048_R_COCNF_CHAN_OFFSET(Chan)    \
                     (((0x0A)+(Chan<<8)) << REG_SHIFT) /* Byte Offset of register */

/* PSN_ADDED Added for B1 Release Software */
#define  ixf6048_bRCV_TRANS_DESCR_EN     0x0200
/* PSN_END B1 Release Changes */
#define  ixf6048_bRCV_LOCK_CNF        0x0100 /* For RLOCK_i to be Active High */
#define  ixf6048_mCO_CNF              0x00E0 
     /* Convert ixf6048_RxTtlClockDivide_e to bits */
#define  ixf6048_SET_CO_CNF(Reg, ClkDiv)   \
                     (Reg |= ( (Reg & ~ixf6048_mCO_CNF      ) |  \
                                  ((ClkDiv << 5) & ixf6048_mCO_CNF) ) )
#define  ixf6048_RCV_CO_CNF_TTL_CLK_DIV(ClkDiv) ((ClkDiv-1)<<5)/* RX COCNF TTL Clock */
/* See ixf6048_RxLineSideInterface_e for Bit Definitions of IfMode*/
#define  ixf6048_mIF_MODE              (0x0007<<2)
#define  ixf6048_GET_IF_MODE(Reg)       \
                     ((Reg & ixf6048_mIF_MODE) >> 2)
/* Convert ixf6048_RxLineSideInterface_e to Bit Values*/
#define  ixf6048_SET_IF_MODE(Reg,IfMode)   \
                     (Reg |= ( (Reg & ~ixf6048_mIF_MODE    ) | \
                                  ((IfMode<< 2)& ixf6048_mIF_MODE) ) )

/* the bit for the corresponding IF Mode */
#define  ixf6048_bIF_MODE(IfMode) (IfMode<< 2)

#define  ixf6048_mCHAN_MODE             0x0003 /* Mask the Chanel Mode */
#define  ixf6048_bCHAN_MODE_POS         0x0003 /* Chan Mode = POS        */
#define  ixf6048_bCHAN_MODE_ATM         0x0002 /* Chan Mode = ATM          */
#define  ixf6048_bCHAN_MODE_TRANSPARENT 0x0001 /* Chan Mode = Transparent*/
#define  ixf6048_bCHAN_MODE_TEST        0x0000 /* Chan Mode = Repeater   */

/***********************************************************************
 * Channel Registers: T_COCNF: Tx Channel Operational Configuration
 * Page  - (0cc)0BH
 **********************************************************************/

#define  ixf6048_T_COCNF_CHAN_OFFSET(Chan)  \
               (((0x0B)+(Chan<<8)) << REG_SHIFT) /* Byte Offset of register */

/* PSN_ADDED Added for B1 Release Software */
#define  ixf6048_bXMT_TRANS_SCR_EN      0x0800               
/* PSN_END B1 Release Changes */
#define  ixf6048_bXMT_PCLK_OUT          0x0400 /* |= to Disable PECL out clk */

/* Convert ixf6048_TxTimingSource_e to bits */
#define  ixf6048_mXMT_TIM_REF           (0x03<<8)
#define  ixf6048_SET_XMT_TIM_REF_CLK_SRC(Reg, ClkSrc) \
                     (Reg |= ( (Reg & ~ixf6048_mXMT_TIM_REF    ) | \
                                  ((ClkSrc<< 8)& ixf6048_mXMT_TIM_REF) ) )

#define  ixf6048_XMT_TIM_REF_CLK_SRC(ClkSrc) \
         (ClkSrc<<8)      /* Convert ixf6048_TxTimingSource_e to bits */

#define  ixf6048_XMT_CO_CNF_TTL_CLK_DIV(ClkDiv)   \
         ((ClkDiv-1)<<5)      /* Convert ixf6048_TxTtlClockDivide_e to bits */
/* See ixf6048_TxLineSideInterface_e for Bit Definitions of IfMode */
/* Notes: CO_CNF Clock Source Defined in Reg R_COCNF above
          IF_MODE N-Bit TTL or PCL Defined in Reg R_COCNF above
          Chan Modes Defined in Reg R_COCNF above  */

/***********************************************************************
 * Global Registers: OHPCNF: Overhead Ports Configuration
 * Page  - (000)0CH
 **********************************************************************/
#define  ixf6048_OHPCNF_OFFSET (0x0C << REG_SHIFT) /* Byte Offset of register */

#define  ixf6048_bOW_PlS_CNF      0x0002   /* for Location of Pulse Byte */
#define  ixf6048_bOH_PORT_MODE    0x0001   /* Set for DCC & Orderwire 
                                             Insertion/extraction Ports Quad 
                                             PHY Mode                   */
/***********************************************************************
 * Global Registers: R_FPCNF: Rx Frame Pulse Configuration
 * Page  - (000)0DH
 **********************************************************************/
#define  ixf6048_R_FPCNF_OFFSET (0x0D << REG_SHIFT)

#define  ixf6048_mRCV_FPI_CNF         0x00FF   /* Receive Input frame Pulse */

/***********************************************************************
 * Global Registers: T_FPCNF: Tx Frame Pulse Configuration
 * Page  - (000)0EH
 **********************************************************************/

#define  ixf6048_T_FPCNF_OFFSET (0x0E << REG_SHIFT) 

#define  ixf6048_XMT_FPO_CNF_MASK     0xFF00   /* Mask for value */
#define  ixf6048_XMT_FPI_CNF_MASK     0x00FF   /* Mask for value */

#define  SET_XMT_FRAME_PULSE_CNF(Reg, fpo, fpi)   \
            (Reg = (((fpo << 8) & ixf6048_XMT_FPO_CNF_MASK) | \
                    (fpi &ixf6048_XMT_FPI_CNF_MASK) ) )
/***********************************************************************
 * Global Registers: OCPCNF: Output Clock Polarity Configuration
 * Page  - (000)0FH
 **********************************************************************/

#define  ixf6048_OCPCNF_OFFSET (0x0F << REG_SHIFT) 

#define  ixf6048_mOCPCNF    0xFFFD   /* Mask for Register */

/* See ixf6048_OutClockPolarity_e for bits*/

/***********************************************************************
 * Global Registers: ICPCNF1: Input Clock Polarity Configuration 1
 * Page  - (000)10H
 **********************************************************************/

#define  ixf6048_ICPCNF1_OFFSET (0x10 << REG_SHIFT)

/* Note: see ixf6048_InClockPolarity1_e for bits*/
/***********************************************************************
 * Global Registers: ICPCNF2: Input Clock Polarity Configuration 2
 * Page  - (000)11H
 **********************************************************************/

#define  ixf6048_ICPCNF2_OFFSET (0x11 << REG_SHIFT) 

#define  ixf6048_OCPCNF2_MASK     0x0003   /* Mask for value */

/* Note: See ixf6048_InClockPolarity2_e for bits */
/***********************************************************************
 * Global Registers: ICMR1: Input Clock Monitoring Reg
 * Page  - (000)12H
 **********************************************************************/
#define  ixf6048_ICMR1_OFFSET (0x12 << REG_SHIFT)

#define  ixf6048_eCHG_TCI_T3_INVERTED	0x8000
#define  ixf6048_eCHG_TCI_T2_INVERTED	0x4000
#define  ixf6048_eCHG_TCI_T1_INVERTED	0x2000
#define  ixf6048_eCHG_TCI_T0_INVERTED	0x1000

#define  ixf6048_eCHG_RCI_T3_INVERTED	0x0800
#define  ixf6048_eCHG_RCI_T2_INVERTED	0x0400
#define  ixf6048_eCHG_RCI_T1_INVERTED	0x0200
#define  ixf6048_eCHG_RCI_T0_INVERTED	0x0100

#define  ixf6048_eCHG_TSCI_P3_INVERTED	0x0080
#define  ixf6048_eCHG_TSCI_P2_INVERTED	0x0040
#define  ixf6048_eCHG_TSCI_P1_INVERTED	0x0020
#define  ixf6048_eCHG_TSCI_P0_INVERTED	0x0010

#define  ixf6048_eCHG_RSCI_P3_INVERTED	0x0008
#define  ixf6048_eCHG_RSCI_P2_INVERTED	0x0004
#define  ixf6048_eCHG_RSCI_P1_INVERTED	0x0002
#define  ixf6048_eCHG_RSCI_P0_INVERTED	0x0001


/* Note: See ixf6048_InClockMonitor1_e */
/***********************************************************************
 * Global Registers: ICMR2: Input Clock Monitoring Reg
 * Page  - (000)13H
 **********************************************************************/
#define  ixf6048_ICMR2_OFFSET (0x13 << REG_SHIFT) 

#define  ixf6048_mICMR2       0xFF03   /* Mask for value */
#define ixf6048_eCHG_TXCLK_3_INVERTED	0x8000
#define ixf6048_eCHG_TXCLK_2_INVERTED	0x4000
#define ixf6048_eCHG_TXCLK_1_INVERTED	0x2000
#define ixf6048_eCHG_TXCLK_0_INVERTED	0x1000

#define	ixf6048_eCHG_RXCLK_3_INVERTED	0x0800
#define	ixf6048_eCHG_RXCLK_2_INVERTED	0x0400
#define	ixf6048_eCHG_RXCLK_1_INVERTED	0x0200
#define	ixf6048_eCHG_RXCLK_0_INVERTED	0x0100

#define	ixf6048_eCHG_TPCI_P_INVERTED	0x0002
#define	ixf6048_eCHG_RPCI_P_INVERTED	0x0001


/* Note: See ixf6048_InClockMonitor2_e */
/***********************************************************************
 * Global Registers: NCMODECNF: Non-concatenated mode Coniguration register
 * Page  - (000)14H
 **********************************************************************/

#define  ixf6048_NCMODECNF_OFFSET (0x14 << REG_SHIFT)

#define  ixf6048_mNCMODECNF        0x001F   /* Mask for value */
#define  ixf6048_bJ0_MUX_CNF       0x0010   /* C0 Bytes are Overwritten */
#define  ixf6048_bRES_M1_MUX_CNF   0x0008   /* Reserved for Level One Communications(tm) Testing*/
#define  ixf6048_bB_W_MUX_CNF      0x0004   /* Byte Interleaved Mux      */
#define  ixf6048_bRES_M1_DMX_CNF   0x0002   /* Reserved for Level One Communications(tm) Testing */
#define  ixf6048_bB_W_DMX_CNF      0x0001   /* Byte Interleaved De-Mux   */

/***********************************************************************
 * Global Registers: TESTCNF: Test Configuration register
 * Page  - (000)15H
 **********************************************************************/

#define  ixf6048_TESTCNF_OFFSET (0x15 << REG_SHIFT)

/* NOTE: The User Must write a 0 to all of the following Bits */
#define  ixf6048_TESTCNF_MASK        0x003F   /* use ClrBits macro with Mask*/
/***********************************************************************
 * Global Registers: TESTRAM_0: Test RAM register
 * Page  - (000)16H
 **********************************************************************/
/* Note: This Register is Reserved for Level One Communication Use Only */
#define  ixf6048_TESTRAM_0_OFFSET (0x16 << REG_SHIFT)

/***********************************************************************
 * Global Registers: TESTRAM_1: Test RAM register
 * Page  - (000)17H
 **********************************************************************/
/* Note: This Register is Reserved for Level One Communication Use Only */
#define  ixf6048_TESTRAM_1_OFFSET (0x17 << REG_SHIFT)

 /***********************************************************************
 * Global Registers: TESTRAM_2: Test RAM register
 * Page  - (000)18H
 **********************************************************************/
/* Note: This Register is Reserved for Level One Communication Use Only */
#define  ixf6048_TESTRAM_2_OFFSET (0x18 << REG_SHIFT)

/***********************************************************************
 * Global Registers: TESTRAM_3: Test RAM register
 * Page  - (000)19H
 **********************************************************************/
/* Note: This Register is Reserved for Level One Communication Use Only */
#define  ixf6048_TESTRAM_3_OFFSET (0x19 << REG_SHIFT)

/***********************************************************************
 * Global Registers: PRBSINT: PRBS Analyze Interupt register
 * Page  - (000)1AH
 **********************************************************************/

#define  ixf6048_PRBSINT_OFFSET (0x1A << REG_SHIFT)

#define  ixf6048_bPRBSI(Chan) (0x0001<<Chan) /* Read Only  */
/***********************************************************************
 * Global Registers: PRBSINTEN: PRBS Analyze Interupt Enable register
 * Page  - (000)1BH
 **********************************************************************/
#define  ixf6048_PRBSINTEN_OFFSET (0x1B << REG_SHIFT)

#define  ixf6048_bPRBSINTEN(Chan) (0x0001<<Chan)  

/* See  ixf6048_PRBSINT_OFFSET for Masks */
/***********************************************************************
 * Global Registers: TALBINT: Tx Alarm Bus Interupt register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_TALBINT_OFFSET (0x1C << REG_SHIFT)

#define  ixf6048_mTALB_INT    0xFFFF   /* Mask for All Interupt Bits */

#define  ixf6048_TPAL_CRCI_CHAN3	0x8000
#define  ixf6048_TPAL_CRCI_CHAN2	0x4000
#define  ixf6048_TPAL_CRCI_CHAN1	0x2000
#define  ixf6048_TPAL_CRCI_CHAN0	0x1000

#define  ixf6048_TPAL_BUSI_CHAN3	0x0800
#define  ixf6048_TPAL_BUSI_CHAN2	0x0400
#define  ixf6048_TPAL_BUSI_CHAN1	0x0200
#define  ixf6048_TPAL_BUSI_CHAN0	0x0100


#define  ixf6048_TSAL_CRCI_CHAN3	0x0080
#define  ixf6048_TSAL_CRCI_CHAN2	0x0040
#define  ixf6048_TSAL_CRCI_CHAN1	0x0020
#define  ixf6048_TSAL_CRCI_CHAN0	0x0010

#define  ixf6048_TSAL_BUSI_CHAN3	0x0008
#define  ixf6048_TSAL_BUSI_CHAN2	0x0004
#define  ixf6048_TSAL_BUSI_CHAN1	0x0002
#define  ixf6048_TSAL_BUSI_CHAN0	0x0001

/***********************************************************************
 * Global Registers: TALBINTEN: Tx Alarm Bus Interupt Enable register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_TALBINTEN_OFFSET (0x1D << REG_SHIFT)

#define  ixf6048_GET_TALBINTEN_CHAN_INT_MASK(Chan) (0x1111 << Chan)
/***********************************************************************
 * Global Registers: LSPCNF: Line Side Parity Configuration register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_LSPCNF_OFFSET (0x1E << REG_SHIFT)

#define  ixf6048_bPRTY_MODE_CNF    0x0004   /* Protect parallel data bus plus Frame Pulse  */
#define  ixf6048_bRCV_PRTY_CNF     0x0002   /* 1 - Serves as even parity */
#define  ixf6048_bXMT_PRTY_CNF     0x0001   /* 1 serves as even parity */


/***********************************************************************
 * Global Registers: LSPINT: Line Side Parity Interupt register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_LSPINT_OFFSET (0x1F << REG_SHIFT)

#define  ixf6048_mLSP_INT          0x0001   /* LSP Interupt mask */

/***********************************************************************
 * Global Registers: LSPINTEN: Line Side Parity Interupt Enable register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_LSPINTEN_OFFSET (0x20 << REG_SHIFT)

/***********************************************************************
 * Global Registers: MISC_GIS: Miscelaneous Global Interupt Source register
 * Page  - (000)H
 **********************************************************************/
#define  ixf6048_MISC_GIS_OFFSET (0x21 << REG_SHIFT)

#define  ixf6048_mMISC_GIS_INT   0x001F   /* MISC GIS Interupt Mask */

#define	 ixf6048_MISC_PRBS_INT  0x0010
#define	 ixf6048_MISC_TALB_INT	0x0008
#define	 ixf6048_MISC_LSP_INT	0x0004
#define	 ixf6048_MISC_XMTUTOP_INT	0x0002
#define	 ixf6048_MISC_RCVUTOP_INT	0x0001


/***********************************************************************
 * UTOPIA INTERFACE REGISTERS 
 *
 **********************************************************************/
 
/***********************************************************************
 * Global Register: R_UICNF: Rx UTOPIA i/f Configuration
 * Page  - (000)70H
 **********************************************************************/
#define  ixf6048_R_UICNF_OFFSET (0x70 << REG_SHIFT)

#define  ixf6048_mR_UICNF          0x07FF  /* Register Mask Read/Write*/

/* PSN_ADDED Added for B1 Release */
#define  ixf6048_bRCV_VAL2_CNF     0x0800  /* reading blocked after EOF */
/* PSN_END B1 RElease Changes */
#define  ixf6048_bRCV_ATM_HEC      0x0400  /* 1 - 2 status bits are passed */
#define  ixf6048_bRCV_SMALL_MEM    0x0200  /* 1 = 2K FIFO; 0 = 16K FIFO */
#define  ixf6048_bRCV_TEST_O_EN    0x0100   /* Unused signals not high imp */
                                               /* Note: This is always = 0 */
#define  ixf6048_bRCV_VAL_CNF      0x0080   /* RXVal deassert on End of Pkt */
#define  ixf6048_bRCV_FIF_EMPT_EOF 0x0040   /* RXFA_i deassert, even if EOF */

     /* The Remainder of bits common to Rx/Tx i/f  -- See Register (000)50H - T_UICNF*/
#define  ixf6048_bRX_TX_DIR_STAT_CNF       0x0020    /* Rx/TxFA: Multiplexed status */
#define  ixf6048_bRX_TX_M_PHY_DEV_CNF      0x0010    /* Config Receive Interface -- See Spec */
#define  ixf6048_bRX_TX_U_QUAD             0x0008    /* UTOPIA Multi-PHY Single/Quad */

#define  ixf6048_mRX_TX_U_WIDTH            0x0006    /* Mask for following Bits */
#define  ixf6048_U_WIDTH_QUAD_SINGLE_8BIT  0x0000    /* Single/Quad Chan,8-bit UTOPIA i/f */
#define  ixf6048_U_WIDTH_QUAD_SINGLE_16BIT 0x0002    /* Single/Wuad Chan,16-bit UTOPIA i/f. Bit 3 MUST be 0*/
#define  ixf6048_U_WIDTH_SINGLE_32BIT      0x0004    /* Single Chan, 32-bit UTOPIA i/f. Bit 3 MUST be 0*/
#define  ixf6048_U_WIDTH_SINGLE_64BIT      0x0006    /* Single Chan, 64-bit UTOPIA i/f. Bit 3 MUST be 0*/

#define  ixf6048_SET_U_WIDTH(Reg, Width)    \
            (Reg = (Reg & ~ixf6048_mRX_TX_U_WIDTH) | (Width & ixf6048_mRX_TX_U_WIDTH))

#define  ixf6048_bRCV_XMT_SEL_MODE         0x0001   /* POS UTOPIA interface is mem mapped */

/* PSN_TODO -- Where does this Fit? */
#define  ixf6048_FIFO_RESTART_LEVEL(NumBytes) \
         ((((NumBytes-1)/64)<<1)&0x7E)    /* Rx/Tx FIFO Restart Level */

/***********************************************************************
 * Global Register: R_UIIML: Rx UTOPIA Interface Initiation 
 *                                  Minimum Level
 * Page  - (000)71H
 **********************************************************************/

#define  ixf6048_R_UIIML_OFFSET (0x71 << REG_SHIFT)

#define  ixf6048_mR_UIIML  0x00FF   /* Mask for Rx UTOPIA IIML */

/***********************************************************************
 * Channel Registers: R_UICHCNF: Rx UTOPIA i/f Channel Configuration 
 * Page  - (0cc)60H
 **********************************************************************/

#define  ixf6048_R_UICHCNF_CHAN_OFFSET(Chan)  \
            (((0x60)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_UICHCNF_CHAN_MASK    0x0FFF /* Register Access Mask */
 
         /* All bits common to Rx/Tx i/f for channel */

#define  ixf6048_bBURST_CNF     0x0800 /* Rx/TxFA Active low */
#define  ixf6048_bF_A_CNF       0x0400 /* Rx/TxFA Active low */

#define  ixf6048_bPRTY_CNF      0x0200 /* 1 = Even Parity */

#define  ixf6048_bD_R_CNF       0x0100 /* 1 - 2 Clk Delay in Utopia Interface */
                                              /* Device Rx from SONET/SDH */
                                              /* Device Tx to   SONET/SDH */
#define  ixf6048_bFIFO_RST      0x0080 /* Set, to Halt Chan Rx/Tx (FIFO_Rst) */
                                     /* Clr, to Restart Chan Rx/Tx */
#define  ixf6048_bCELL_STRUCT   0x0040  /* Pass NULL HEC Word */

#define  ixf6048_mC_A_DEASSERT  0x3F  /* For Rx/Tx */
#define  ixf6048_SET_C_A_DEASSERT(Reg, CadVal)   \
                     (Reg = (Reg & ~ixf6048_mC_A_DEASSERT) | (CadVal & ixf6048_mC_A_DEASSERT) )
/***********************************************************************
 * Channel Registers: R_PWM: Rx Programmable Watermark 
 * Page  - (0cc)61H
 * **********************************************************************/

#define  ixf6048_R_PWM_CHAN_OFFSET(Chan)  \
            (((0x61)+(Chan<<8))<<REG_SHIFT)

/* Note: See Reg  T_NFPWM (Below) for definition of 
   ixf6048_SET_WATERMARK and Mask Definitions */

/***********************************************************************
 * Global Register: T_UICNF: Tx UTOPIA i/f Configuration 
 * Page  - (000)50H
 **********************************************************************/
#define  ixf6048_T_UICNF_OFFSET (0x50<<REG_SHIFT)

#define  ixf6048_mT_UICNF           0x00FF  /* Register Mask Read/Write*/

#define  ixf6048_bXMT_SMALL_MEM     0x0080  /* 1 = 2K FIFO; 0 = 16K FIFO */
                                          /* Note: This is always = 0 */
#define  ixf6048_XMT_FIF_EMPT_EOF   0x0040    /* RXFA_i deassert, even if EOF */

/* Note: The Remaining Bits are common to Register R_UICNF and have
   been Defined There */

/***********************************************************************
 * Global Register: T_UIIML: Tx UTOPIA Interface Initiation 
 *                                  Minimum Level 
 * Page  - (000)51H
 **********************************************************************/
#define  ixf6048_T_UIIML_OFFSET (0x51<<REG_SHIFT)

#define  ixf6048_mT_UIIML         0x00FF /* Mask for Tx UTOPIA IIML */

/***********************************************************************
 * Channel Registers: T_UICHCNF: Tx UTOPIA i/f Channel Configuration
 * Page  - (0cc)40H
 **********************************************************************/
#define  ixf6048_T_UICHCNF_CHAN_OFFSET(Chan)  \
             (((0x40)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mT_UICHCNF_CHAN    0x07FF /* Register Access Mask */

/* Note: All bit Masks for this register are common to  
         register (0cc)60H and have been defined there. */

/***********************************************************************
 * Channel Registers: T_UIFDP: UTOPIA Interface FIFO depth
 * Page  - (0cc)41H
 **********************************************************************/

#define  ixf6048_T_UIFDP_CHAN_OFFSET(Chan)  \
            (((0x41)+(Chan<<8))<<REG_SHIFT)

/* Fifo Depth Mask */
#define  ixf6048_mXMT_F_D_CNF_CHAN_0     0xFF   /* ChanMask for Chan 0 */
#define  ixf6048_mXMT_F_D_CNF_CHANS_1_3  0x1F   /* ChanMask for Channels 1-3 */
#define  ixf6048_SET_XMT_F_D_CNF(Reg, FifoDepth, ChanMask)   \
               (Reg = (FifoDepth & ChanMask) )

/***********************************************************************
 * Channel Registers:
 *                   T_NFPWM: Tx Near Full  Programmable Watermark
 * Page  - (0cc)42H
 *                   T_NEPWM: Tx Near Empty Programmable Watermark
 * Page  - (0cc)43H
 **********************************************************************/
#define  ixf6048_T_NFPWM_CHAN_OFFSET(Chan)  \
            (((0x42)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_NEPWM_CHAN_OFFSET(Chan)  \
            (((0x43)+(Chan<<8))<<REG_SHIFT)

/* [11:0] Bits are available for Chan 0, 
   [8:0] Bits are available for Chan 1-3 */

#define  ixf6048_mWATERMARK_CHAN_0   0x0FFF
#define  ixf6048_mWATERMARK_CHAN_1_3 0x01FF

#define  ixf6048_SET_WATERMARK(Reg, NumBytes, WatermarkMask)  \
         (Reg = NumBytes & WatermarkMask)

/* Note:  ixf6048_WATERMARK is used also for R_PWM register */
/***********************************************************************
 * Global Registers:
 *                  R_UTOINT: Rx UTOPIA Interrupt
 * Page  - (000)72H (COR)
 *                  T_UTOINT: Tx UTOPIA Interrupt
 * Page  - (000)52H (COR)
 *    These alarms put together to form: ixf6048_UtopiaAlarms_t
 **********************************************************************/
#define  ixf6048_R_UTOINT_OFFSET (0x72<<REG_SHIFT)
#define  ixf6048_T_UTOINT_OFFSET (0x52<<REG_SHIFT)

#define  ixf6048_mR_UTOINT    0x000F   /* MASK RX FIFO UFI interupts */
#define  ixf6048_mT_UTOINT    0x0FFF   /* Mask ALL TX UTOPIA Interupts */

#define  ixf6048_mRCV_FIFO_OF_INT 0x00F0  /* Rx FIFO Overflow Interupt */
#define  ixf6048_SHIFT_RCV_FIFO_OF_INT(Chan) ( 0x0010 << Chan )

#define  ixf6048_mRCV_FIFO_UF_INT 0x000F  /* Rx FIFO Underflow Interupt */
#define  ixf6048_SHIFT_RCV_FIFO_UF_INT(Chan) ( 0x0001 << Chan )

#define  ixf6048_mTX_FIFO_UF_INT  0xF000  /* Tx FIFO Underflow Interupt*/
#define  ixf6048_SHIFT_TX_FIFO_UF_INT(Chan)   ( 0x1000 << Chan )

#define  ixf6048_mXMT_SOC_INT     0x0F00  /* Start Of Cell  Interupt */
#define  ixf6048_SHIFT_XMT_SOC_INT(Chan)  ( 0x0100 << Chan )

#define  ixf6048_mTX_PRTY_ERR_INT 0x00F0  /* Parity Error Interupt */
#define  ixf6048_SHIFT_TX_PRTY_ERR_INT(Chan)  ( 0x0010 << Chan )

#define  ixf6048_mTX_FIFO_OF_INT  0x000F  /* Tx FIFO Overflow Interupt*/
#define  ixf6048_SHIFT_TX_FIFO_OF_INT(Chan)   ( 0x0001 << Chan )

#define  ixf6048_GET_R_UTOPIA_INT_CHAN_MASK(Chan) (ixf6048_mR_UTOINT & (0x1111 << Chan))
#define  ixf6048_GET_T_UTOPIA_INT_CHAN_MASK(Chan) (ixf6048_mT_UTOINT & (0x1111 << Chan))

/***********************************************************************
 * Global Registers:
 *                  R_UTOINT: Rx UTOPIA Interrupt Enable
 * Page  - (000)73H
 *                  T_UTOINT: Tx UTOPIA Interrupt Enable
 * Page  - (000)53H
 *    These alarms put together to form: ixf6048_UtopiaAlarms_t
 **********************************************************************/
#define  ixf6048_R_UTOINTEN_OFFSET (0x73<<REG_SHIFT) 
#define  ixf6048_T_UTOINTEN_OFFSET (0x53<<REG_SHIFT) 

#define  ixf6048_mRX_FIFO_UF_INT_EN  0x000F  /* Tx/Rx FIFO Underflow Interupts EN*/

/* Note: See Interupt Register for Masks */

/*#define  ixf6048_mTX_SOC_INT_EN      0x0F00  *//* Start Of Cell Interupts EN */
/*#define  ixf6048_mTX_PRTY_ERR_INT_EN 0x00F0  *//* Parity Error Interupts EN */
/*#define  ixf6048_mTX_FIFO_OF_INT_EN  0x000F  *//* Tx/Rx FIFO Underflow Interupts EN*/

#define  ixf6048_SHIFT_RX_FIFO_OF_INT_EN(Chan)   (0x0010 << Chan)
#define  ixf6048_SHIFT_RX_FIFO_UF_INT_EN(Chan)   (0x0001 << Chan)

#define  ixf6048_SHIFT_TX_FIFO_UF_INT_EN(Chan)  (0x1000 << Chan)
#define  ixf6048_SHIFT_TX_SOC_INT_EN(Chan)      (0x0100 << Chan)
#define  ixf6048_SHIFT_TXPRTY_ERR_INT_EN(Chan)  (0x0010 << Chan)
#define  ixf6048_SHIFT_TX_FIFO_OF_INT_EN(Chan)  (0x0001 << Chan)

/***********************************************************************
 * Channel Registers:
 **********************************************************************/

/***********************************************************************
 * Channel Registers: R_RSTC: Receive RST Configuration
 * Page  - (1cc)80H
 **********************************************************************/

#define  ixf6048_R_RSTC_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x80)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mNO_FORCE_R_RSTC     0xFFFF


#define  ixf6048_bRCV_SCRMBL_CNFG    0x8000 /* Enable Receive Scrambler 2e7 */

#define  ixf6048_bRCV_F_BA_DSBL      0x4000   /* Use RFPI Pulse */
#define  ixf6048_bRCV_FWD_OFF_CNFG   0x2000   /* Bit 13 */
#define  ixf6048_bRCV_FWD_ON_CNFG    0x1000   /* Bit 12 */

#define  ixf6048_bCNFG_B1_CNTR       0x0800   /* 1 = Cnt B1 Blk errs
                                                0 = Cnt Bit Errors */

#define	 ixf6048_bRST_CLK_LOCKN		 0x0100
#define  ixf6048_bRST_AIS_LOFN		 0x0080
#define  ixf6048_bRST_AIS_LOSN		 0x0040
#define  ixf6048_bRST_AIS_LOCKN		 0x0020
#define	 ixf6048_bRST_AIS_TIMN		 0x0008

#define  ixf6048_mRCV_LOS_CNFG                  0x0600
#define  ixf6048_RCV_LOS_CNFG_ON_25US_LOS_EN    0x0600
#define  ixf6048_RCV_LOS_CNFG_ON_25US_ZEROS_EN  0x0400
#define  ixf6048_RCV_LOS_CNFG_ON_20US_LOS_EN    0x0200
#define  ixf6048_RCV_LOS_CNFG_ON_20US_ZEROS_EN  0x0000
#define  ixf6048_SET_RST_RCV_LOS_CNFG(Reg, LosVal)  \
                     (Reg |= (LosVal<<9) & ixf6048_mRCV_LOS_CNFG)

#define  ixf6048_bRST_CLK_LOCK_EN          0x0100 /* 1 = Auto Clk Switch because of LOCK or LOS */

#define  ixf6048_bRST_AIS_LOF_EN      0x0080   /*1 -> AIS Generation during LOF */
#define  ixf6048_bRST_AIS_LOS_EN      0x0040   /*1 -> AIS Generation during LOS */
#define  ixf6048_bRST_AIS_LOCK_EN     0x0020   /*1 -> AIS Generation during LOCK */
#define  ixf6048_bRST_AIS_FRC         0x0010   /*1 -> Force AIS gen from the RST 
                                                section to the MST section via software*/

#define  ixf6048_bRST_AIS_TIM_EN      0x0008   /*1 -> Enable AIS during active j0MsMtchSt */

#define  ixf6048_mLOCK_ITG                 0x0006  /* Configure LOCK alarm Filtering Mask */
#define  ixf6048_NO_LOCK_FILTERING_EN      0x0000  /* Bit 2 = 0 For No Filtering */
#define  ixf6048_WEAK_LOC_FILTERING_EN     0x0004  /* Weak LOCK filtering */
#define  ixf6048_STRONG_LOC_FILTERING_EN   0x0006  /* Strong LOCK Filtering */
#define  ixf6048_SET_LOCK_ITG(Reg, FilterVal)   \
               (Reg |= (FilterVal<<1) & ixf6048_mLOCK_ITG)

#define  ixf6048_bCNFG_FRM_ACQ             0x0001  /*1 -> Robust Acquisition */

/***********************************************************************
 * Channel Registers: LOF_LMN:   Loss of Frame L, M, & N Configuration
 * Page  - (1cc)81H
 *
 * To Set L,M,N:
 *    ushort   Reg;
 *    ushort*  BaseAddress;
 *
 *    Reg = 0;
 *    ixf6048_STORE_L (L, Reg);
 *    ixf6048_STORE_M (M, Reg);
 *    ixf6048_STORE_N (N, Reg);
 *    *(BaseAddress + ixf6048_LOF_LMN_CHAN_OFFSET(Chan)) |= Reg;
 **********************************************************************/

#define  ixf6048_LOF_LMN_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x81)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_bRCV_OOF_CNFG      0x8000   /* Out of Frame Config Bit */
#define	 ixf6048_bL_LOF_LMN			0x7C00	//L
#define	 ixf6048_bM_LOF_LMN			0x03E0	//M
#define	 ixf6048_bN_LOF_LMN			0x001F	//N


#define  ixf6048_STORE_L(LMN, L)   \
         LMN |= (((L-1)&0x1F)<<10)

#define  ixf6048_STORE_M(LMN, M)   \
         LMN |= (((M-1)&0x1F)<<5)

#define  ixf6048_STORE_N(LMN, N)   \
         LMN |= ((N-1)&0x1F)

/***********************************************************************
 * Channel Counter Registers: OOF_ECNT:  Out of Frame Event Counter
 * Page  - (1cc)82H
 **********************************************************************/

#define  ixf6048_OOF_ECNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x82)+(Chan<<8))<<REG_SHIFT)

#define	 ixf6048_OOF_ECNT	0x1FFF

#define  ixf6048_GET_OOF_CNT(BaseAddress,Chan)   \
         (READ_FROM_HW(BaseAddress + ixf6048_OOF_ECNT_CHAN_OFFSET(Chan)) & 0x1FFF)

/***********************************************************************
 * Channel Counter Registers: B1_ERRCNT: B1 Error Counter
 * Page  - (1cc)83H
 **********************************************************************/

#define  ixf6048_B1_ERRCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x83)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_B1_ERRCNT	0xFFFF

#define  ixf6048_GET_B1_ERRCNT(BaseAddress,Chan) \
         (READ_FROM_HW(BaseAddress + ixf6048_B1_ERRCNT_CHAN_OFFSET(Chan)))

/***********************************************************************
 * Channel Registers:
 *                  R_J0_ESTRA: J0 Rx Expected String Data Access 
 * Page  - (1cc)85H
 *                  R_J0_ASTRA: J0 Rx Accepted String Data Access
 * Page  - (1cc)86H
 *
 *                  R_J1_ESTRA: J1 Rx Expected String Data Access
 * Page  - (1cc)ADH
 *                  T_J1_TSTRA: J1 Tx          String Data Access
 * Page  - (1cc)AFH
 *
 *                  T_J0_TSTRA: J0 Tx          String Data Access 
 * Page  - (1cc)E4H
 *                  R_J1_ASTRA: J1 Rx Accepted String Data Access
 * Page  - (1cc)EEH
 *
 * Trace = EXPECTED/RX/TX_J0/J1, e.g. EXPECTED_J0
 *
 * To Write Trace Byte:
 *    while(! ixf6048_CAN_WRITE_TRACE(TX_J0, 0x1200, 0))
 *       ;  [wait until can write new trace byte]
 *    ixf6048_WRITE_TRACE(TX_J0, 0x1200, 0, ByteIdx,TraceByte);
 *
 * To Read Trace Byte:
 *     Set the Trace Read Bit 
 *     ixf6048_READ_TRACE(pReg, iByteIndex, pWorkString++);
 *     Wait for the Read Access 
 *     while(!(ixf6048_READ_TRACE_DONE(*pReg)));
 *     Mask out the Byte
 *     CharByte = ixf6048_GET_TRACE_BYTE(*pReg);
 *
 **********************************************************************/

#define  ixf6048_EXPECTED_J0_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x85)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_RX_J0_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0x86)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_RX_J1_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0xAD)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_EXPECTED_J1_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0xAF)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_TX_J0_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0xE4)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_TX_J1_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0xEE)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_CAN_WRITE_TRACE(pReg) ((READ_FROM_HW(pReg) & 0x8000) == 0)

#define  ixf6048_WRITE_TRACE(pReg, ByteIdx, TraceByte) \
          WRITE_TO_HW(pReg, (0x8000 + (ByteIdx<<8) + (*TraceByte)))

#define  ixf6048_READ_TRACE(pReg, ByteIdx) \
                       (WRITE_TO_HW(pReg, (0x4000 + (ByteIdx<<8))))

#define  ixf6048_READ_TRACE_DONE(TraceReg) \
         ((TraceReg & 0x4000) == 0)

#define  ixf6048_GET_TRACE_BYTE(TraceReg) \
         (TraceReg & 0x00FF)

#define ixf6048_mEXP_J0_STRG_ADDR 		0x3F00 /* Bits [13:8] represents the string pointer value (RAM address). Whenever Microprocessor access this register to read, this address will be automatically increased to next address. */
#define ixf6048_mEXP_J0_STRG_DATA 		0x00FF /* Bits [7:0] represents the data value. */
#define ixf6048_mEXP_J0_STRG_ADDR_SHFT  8      /* Shift Value for Slot Index */


/***********************************************************************
 * Channel Registers: J0_RSTC: J0 Received Trace Configuration
 * Page  - (1cc)87H
 **********************************************************************/

#define  ixf6048_J0_RSTC_CHAN_OFFSET(Chan)  \
         (((1<<10)+(0x87)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_bRS_TIM_ON_CRC7_ERR_DSB     0x0020 /* Config setting of RS-Tim Alm*/
                                                   /* When CRC-7 is wrong */
#define  ixf6048_bRS_TIM_ON_UNSTABLE_EN      0x0010 /* Config setting of RS-Tim Alm*/
                                                   /* When Trace is unstable */
#define  ixf6048_bRCV_J0_STABLE_CNFG         0x0008 /* 1- 5 Consectutive identical Messages*/

#define  ixf6048_mRCV_J0_CNF                  0x0007 /* Mask for Configure J0 Rx Trace Id Format */
#define  ixf6048_RX_JN_64_WITH_LF_CR_EN       0x0007
#define  ixf6048_RX_JN_64_FREE_FORM_EN        0x0006
#define  ixf6048_RX_JN_16_WITH_CRC7_EN        0x0004
#define  ixf6048_IGNORE_RX_JN_EN              0x0002
#define  ixf6048_RX_JN_1_BYTE_EN              0x0000

#define  ixf6048_GET_RCV_J0_CNF_FORMAT(Reg) (Reg & ixf6048_mRCV_J0_CNF)

#define  ixf6048_SET_RCV_J0_CNF_FORMAT(Reg, NewFormat)   \
           (Reg = ((Reg & ~ixf6048_mRCV_J0_CNF) | NewFormat) )
           
/***********************************************************************
 * Channel Registers:
 *                      IS_RG: Rx RST Interrupt Source
 * Page  - (1cc)D0H  (COR)
 *                      IE_RG: Rx RST Interrupt Enable
 * Page  - (1cc)D4H
 *                      S_RG: Rx RST Status
 * Page  - (1cc)D8H
 **********************************************************************/

#define  ixf6048_IS_RG_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD0)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_RX_RST_OOF_OVFL_ALM	0x0200
#define	 ixf6048_RX_RST_B1_OVFL_ALM		0x0100
#define  ixf6048_RX_RST_J0_UNSTABLE_ALM 0x0040
#define  ixf6048_RX_RST_J0_MISMATCH_ALM 0x0020
#define  ixf6048_RX_RST_J0_CRC7_ALM		0x0010
#define  ixf6048_RX_RST_LOS_ALM			0x0008
#define  ixf6048_RX_RST_LO_CLK_ALM		0x0004
#define  ixf6048_RX_RST_LOF_ALM			0x0002
#define  ixf6048_RX_RST_OOF_ALM			0x0001


#define  ixf6048_IE_RG_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD4)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_S_RG_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD8)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mIS_RG          0x037F  /* Interupt Reg Mask */
#define  ixf6048_mIE_RG          0x037F  /* Interupt Enable Reg Mask */   
#define  ixf6048_mS_RG           0x007F  /* Interupt Status Reg Mask */

/* See  ixf6048_RxRstAlarms_e */
/***********************************************************************
 * Channel Registers: R_MST_C: Rx MST Configuration
 * Page  - (1cc)90H
 **********************************************************************/

#define  ixf6048_R_MST_C_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x90)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mR_MST_C           0x27FF

/* See ixf6048_RxMstCfg_t for Settings  
            -- see also IS_MUX (1cc)D1H */

#define  ixf6048_bMST_APS_FILT_CNFG              0x2000

#define  ixf6048_bMST_AIS_DET_CNT                0x0400

#define  ixf6048_mMST_RDI_DET_CNT                0x0300   
#define  ixf6048_K2_RDI_DETECT_IN_3_FRAMES_EN    0x0000
#define  ixf6048_K2_RDI_DETECT_IN_5_FRAMES_EN    0x0100
#define  ixf6048_K2_RDI_DETECT_IN_10_FRAMES_EN   0x0200
#define  ixf6048_K2_RDI_DETECT_IN_16_FRAMES_EN   0x0300
      /* Set from ixf6048_K2RdiDetect_e Value  */ 
#define  ixf6048_SET_K2_RDI_DETECT(Reg, DetRdiVal)  \
               (Reg = (Reg | ((DetRdiVal << 8) & ixf6048_mMST_RDI_DET_CNT)))

#define  ixf6048_bAIS_ON_EXC_B2_EN     0x0080
#define  ixf6048_MST_AIS_FRC           0x0040  /* Force Tx AIS -- used for Testing */
#define  ixf6048_MST_AIS_EN            0x0020
#define  ixf6048_MST_GEN_REI_CNF       0x0010

#define  ixf6048_MST_RCV_REI_CNF       0x0008
#define  ixf6048_MST_RDI_ON_EXC_B2_EN  0x0004
#define  ixf6048_MST_RDI_FRC           0x0002 /* Force Tx RDI -- used for Testing */
#define  ixf6048_MST_BLK_ERR_CNFG      0x0001 /* 1 -> Multiple Blocks Per Frame */

/***********************************************************************
 * Channel Counter Registers: B2_BLKCNT: B2 Block Error Counter
 * Page  - (1cc)96-95H
 **********************************************************************/

#define  ixf6048_B2_BLKCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x95)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_B2_BLKCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x96)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_B2_BLK_CNT_MASK_M 	0x0001
#define ixf6048_B2_BLK_CNT_MASK_L 	0xFFFF

#define  ixf6048_GET_B2_BLKCNT(BaseAddress,Chan)                      \
         (READ_FROM_HW(BaseAddress + ixf6048_B2_BLKCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_B2_BLKCNT_CHAN_MSB_OFFSET(Chan))&0x1) <<16 ))

/***********************************************************************
 * Channel Counter Registers: B2_BIPCNT: B2 BIP Error Counter
 * Page  - (1cc)98-97H
 **********************************************************************/

#define  ixf6048_B2_BIPCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x97)+(Chan<<8))<<REG_SHIFT)/* Offset of LSB */

#define  ixf6048_B2_BIPCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x98)+(Chan<<8))<<REG_SHIFT) /* Offset of MSB */

#define ixf6048_B2_CNT_MASK_M 		0x003F
#define ixf6048_B2_BIP_CNT_MASK_L 	0xFFFF

#define  ixf6048_GET_B2_BIPCNT(BaseAddress,Chan)                      \
         (READ_FROM_HW(BaseAddress + ixf6048_B2_BIPCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_B2_BIPCNT_CHAN_MSB_OFFSET(Chan))&0x3F) <<16 ))


/***********************************************************************
 * Channel Counter Registers:MR_BLKCNT: MST REI Block Error Counter
 * Page  - (1cc)9A-99H
 **********************************************************************/

#define  ixf6048_MR_BLKCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x99)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_MR_BLKCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x9A)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_MST_REI_BLKCNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_MR_BLKCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_MR_BLKCNT_CHAN_MSB_OFFSET(Chan))&0x1) <<16 ))


/***********************************************************************
 * Channel Counter Registers: MR_BIPCNT: MST REI BIP Error Counter
 * Page  - (1cc)9C-9BH
 **********************************************************************/

#define  ixf6048_MR_BIPCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x9B)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_MR_BIPCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x9C)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_MR_BIPCNT_MASK_M  0x001F
#define ixf6048_MR_BIPCNT_MASK_L  0xFFFF

#define  ixf6048_GET_MST_REI_BIPCNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_MR_BIPCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_MR_BIPCNT_CHAN_MSB_OFFSET(Chan))&0x1F) <<16 ))


/***********************************************************************
 * Channel Registers: R_K2K1: Rx K1 & K2 Bytes
 * Page  - (1cc)9DH
 **********************************************************************/

#define  ixf6048_R_K2K1_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x9D)+(Chan<<8))<<REG_SHIFT)


         /* To get K1 & K2, using ixf6048_RxMstBytes_t:
               ushort   Reg;
               ixf6048_RxMstBytes_t RxMst;

               Reg = ixf6048_GET_K1K2(BaseAddress,Chan);
               RxMst.K1 = ixf6048_GET_K1(Reg);
               RxMst.K2 = ixf6048_GET_K2(Reg);
         */
#define  ixf6048_GET_K1K2(BaseAddress,Chan) \
            (READ_FROM_HW(BaseAddress + ixf6048_R_K2K1_CHAN_OFFSET(Chan)))

#define  ixf6048_GET_K1(K1K2)  (K1K2&0x00FF)

#define  ixf6048_MST_K1  0x00FF
#define  ixf6048_MST_K1_SHIFT    0

#define  ixf6048_GET_K2(K1K2)  ((K1K2&0xFF00)>>8)

#define  ixf6048_MST_K2  0xFF00
#define  ixf6048_MST_K2_SHIFT    8

/***********************************************************************
 * Channel Registers: R_S1: Rx S1 Byte
 * Page  - (1cc)9FH
 **********************************************************************/

#define  ixf6048_R_S1_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x9F)+(Chan<<8))<<REG_SHIFT)

         /* To get S1, using ixf6048_RxMstBytes_t:
               ixf6048_RxMstBytes_t RxMst;

               RxMst.S1 = ixf6048_GET_S1(BaseAddress,Chan);
         */
#define  ixf6048_GET_S1(BaseAddress,Chan) \
         (READ_FROM_HW(BaseAddress + ixf6048_R_S1_CHAN_OFFSET(Chan))&0x00FF)

#define  ixf6048_mRCV_S1 0x00FF
/***********************************************************************
 * Channel Registers: WINSZ_SB2: Window Size for setting ExcB2ErrSt
 * Page  - (1cc)B0H
 **********************************************************************/

#define  ixf6048_WINSZ_SB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB0)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mEXC_B2_SET_WIN_SZ  0x07FF

#define  ixf6048_SET_EXC_B2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_SB2_CHAN_OFFSET(Chan),  \
         (((WindowSize/8)-1) & ixf6048_mEXC_B2_SET_WIN_SZ)))
/***********************************************************************
 * Channel Registers:  
 *                  CWIN_SB2: Consecutive Windows for setting ExcB2ErrSt 
 * Page - (1cc)B1H
 *                  E_EXCWIN_SB2: Errs/Window for setting ExcB2ErrSt
 * Page - (1cc)B2H
 **********************************************************************/

#define  ixf6048_CWIN_SB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB1)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_EXCWIN_SB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB2)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mEXC_B2_SET_WIN_NUM  0x007F
#define  ixf6048_mEXC_B2_MIN          0x0FFF

#define  ixf6048_SET_EXC_B2_NUM_ERRS(BaseAddress,Chan,NumErrors)   \
          (WRITE_TO_HW(BaseAddress + ixf6048_CWIN_SB2_CHAN_OFFSET(Chan),  \
                  (NumErrors & ixf6048_mEXC_B2_SET_WIN_NUM)))

#define  ixf6048_SET_EXC_B2_NUM_WINDOWS(BaseAddress,Chan,NumWindows)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_EXCWIN_SB2_CHAN_OFFSET(Chan),  \
                  (NumWindows & ixf6048_mEXC_B2_MIN)))
/***********************************************************************
 * Channel Registers: WINSZ_CB2: Window Size for clearing ExcB2ErrSt
 * Page  - (1cc)B3H
 **********************************************************************/

#define  ixf6048_WINSZ_CB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB3)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mEXC_B2_CLR_WIN_SZ  0x07FF

#define  ixf6048_SET_CLR_EXC_B2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_CB2_CHAN_OFFSET(Chan),  \
         (((WindowSize/8)-1) & ixf6048_mEXC_B2_CLR_WIN_SZ)))

/***********************************************************************
 * Channel Registers:  
 *                  CWIN_CB2: Consecutive Windows for clearing ExcB2ErrSt
 * Page - (1cc)B4H
 *                  E_NEXCWIN_CB2: Errs/Window         for clearing ExcB2ErrSt 
 * Page - (1cc)B5H
 **********************************************************************/

#define  ixf6048_CWIN_CB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB4)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_E_NEXCWIN_CB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB5)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mEXC_B2_CLR_WIN_NUM     0x007F
#define  ixf6048_mEXC_B2_MAX             0x0FFF

#define  ixf6048_SET_CLR_EXC_B2_NUM_ERRS(BaseAddress,Chan,NumErrors)   \
          (WRITE_TO_HW(BaseAddress + ixf6048_CWIN_CB2_CHAN_OFFSET(Chan),  \
                  (NumErrors & ixf6048_mEXC_B2_CLR_WIN_NUM)))

#define  ixf6048_SET_CLR_EXC_B2_NUM_WINDOWS(BaseAddress,Chan,NumWindows)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_E_NEXCWIN_CB2_CHAN_OFFSET(Chan),  \
                  (NumWindows & ixf6048_mEXC_B2_MAX)))


/***********************************************************************
 * Channel Registers: WINSZ_SDEGB2: Window Size for setting DegB2ErrSt 
 * Page  - (1cc)B7-B6H
 **********************************************************************/

#define  ixf6048_WINSZ_SDEGB2_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0xB6)+(Chan<<8))<<REG_SHIFT) /* Offset of LSB */

#define  ixf6048_WINSZ_SDEGB2_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0xB7)+(Chan<<8))<<REG_SHIFT) /* Offset of MSB */

#define ixf6048_DEG_B2_SET_WINSZ_MASK_M   0x007F
#define ixf6048_DEG_B2_SET_WINSZ_MASK_L	  0xFFFF

/* Note: Window Size Must be a 32 Bit Value */
#define  ixf6048_SET_LSB_SDEGB2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_SDEGB2_CHAN_LSB_OFFSET(Chan), \
         (ushort) (((WindowSize/8)-1)&0x0000FFFF)))
#define  ixf6048_SET_MSB_SDEGB2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_SDEGB2_CHAN_MSB_OFFSET(Chan), \
         (ushort) ((((WindowSize/8)-1)&0x00FF0000)>>16)))

/***********************************************************************
 * Channel Registers: 
 *                  WINSZ_SDEGB2: Consecutive Windows for setting DegB2ErrSt 
 * Page - (1cc)B8H
 *                  E_DEGWIN_SB2: Errs/Window for setting DegB2ErrSt 
 * Page - (1cc)B9H
 **********************************************************************/

#define  ixf6048_CWIN_SDEGB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xB8)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_E_DEGWIN_SB2_CHAN_OFFSET(Chan) \
            (((1<<10)+(0xB9)+(Chan<<8)) <<REG_SHIFT)

#define  ixf6048_CWIN_SDEGB2_MASK         0x007F
#define  ixf6048_E_DEGWIN_SB2_MASK        0x0FFF

#define  ixf6048_SET_DEG_B2_ERRORS(BaseAddress,Chan,NumErrors)   \
          (WRITE_TO_HW(BaseAddress + ixf6048_CWIN_SDEGB2_CHAN_OFFSET(Chan), \
               (NumErrors & ixf6048_CWIN_SDEGB2_MASK)))

#define  ixf6048_SET_DEG_B2_WINDOWS(BaseAddress,Chan,NumWindows)  \
            (WRITE_TO_HW(BaseAddress + ixf6048_E_DEGWIN_SB2_CHAN_OFFSET(Chan),  \
               (NumWindows & ixf6048_E_DEGWIN_SB2_MASK)))

/***********************************************************************
 * Channel Registers: WINSZ_CDEGB2: Window Size for clearing DegB2ErrSt
 * Page  - (1cc)BB-BAH
 **********************************************************************/

#define  ixf6048_WINSZ_CDEGB2_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0xBA)+(Chan<<8))<<REG_SHIFT) /* Offset of LSB */

#define  ixf6048_WINSZ_CDEGB2_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0xBB)+(Chan<<8))<<REG_SHIFT) /* Offset of MSB */

/* Note: Window Size MUST be a 32 Bit Value */
#define  ixf6048_SET_CLR_LSB_DEGB2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_CDEGB2_CHAN_LSB_OFFSET(Chan), \
         (ushort) (((WindowSize/8)-1)&0x0000FFFF)))
#define  ixf6048_SET_CLR_MSB_DEGB2_WINDOW_SIZE(BaseAddress,Chan,WindowSize)    \
         (WRITE_TO_HW(BaseAddress + ixf6048_WINSZ_CDEGB2_CHAN_MSB_OFFSET(Chan), \
         (ushort) ((((WindowSize/8)-1)&0x00FF0000)>>16)))

/***********************************************************************
 * Channel Registers:  
 *                  CWIN_CDEGB2: Consecutive Windows for clearing DegB2ErrSt 
 * Page - (1cc)BCH
 *                  E_NDEGWIN_CB2: Errs/Window for clearing DegB2ErrSt 
 * Page - (1cc)BDH
 **********************************************************************/

#define  ixf6048_CWIN_CDEGB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xBC)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_E_NDEGWIN_CB2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xBD)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_CWIN_CDEGB2_MASK         0x007F
#define  ixf6048_E_NDEGWIN_CB2_MASK       0x0FFF

#define  ixf6048_SET_CLEAR_DEG_B2_ERRORS(BaseAddress,Chan,NumErrors)   \
          (WRITE_TO_HW(BaseAddress + ixf6048_CWIN_CDEGB2_CHAN_OFFSET(Chan),  \
                  (NumErrors & ixf6048_CWIN_CDEGB2_MASK)))

#define  ixf6048_SET_CLEAR_DEG_B2_WINDOWS(BaseAddress,Chan,NumWindows)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_E_NDEGWIN_CB2_CHAN_OFFSET(Chan),  \
                  (NumWindows & ixf6048_E_NDEGWIN_CB2_MASK)))

/***********************************************************************
 * Channel Registers:
 *                  IS_MUX: Rx MST Interrupt Source 
 * Page  - (1cc)D1H 
 *                  IE_MUX: Rx MST Interrupt Enable
 * Page  - (1cc)D5H
 *                  S_MUX: Rx MST Status
 * Page  - (1cc)D9H
 **********************************************************************/

#define  ixf6048_IS_MUX_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD1)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_IE_MUX_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD5)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_S_MUX_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD9)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mIS_MUX    0x7FFF
#define  ixf6048_mIE_MUX    0x7FFF
#define  ixf6048_mS_MUX     0x00FF

#define	   ixf6048_eRX_MST_S1_ALM           0x4000 /* Change in Rx S1 byte */
#define    ixf6048_eRX_MST_K1_ALM           0x2000 /* Change in Rx K1 */
#define    ixf6048_eRX_MST_K2_ALM           0x1000 /* Change in Rx K2 */
#define    ixf6048_eRX_MST_B2_BIT_OVFL_ALM  0x0800 /* B2 Bit err cntr overflowed */
#define    ixf6048_eRX_MST_B2_BLK_OVFL_ALM  0x0400 /* B2 Block err cntr overflowed */
#define    ixf6048_eRX_MST_REI_BIT_OVFL_ALM 0x0200 /* REI Bit err cntr overflowed */
#define    ixf6048_eRX_MST_REI_BLK_OVFL_ALM 0x0100 /* REI Block err cntr overflowed */
#define    ixf6048_eRX_MST_S8_UNSTABLE_ALM  0x0080 /* Change in Rx S1 byte unstable */
#define    ixf6048_eRX_MST_K9_UNSTABLE_ALM  0x0040 /* Change in Rx K1 byte unstable */
#define    ixf6048_eRX_MST_K2_UNSTABLE_ALM  0x0020 /* Change in Rx K2 byte unstable */
#define    ixf6048_eRX_MST_DEG_B2_ERR_ALM   0x0010 /* Change in Degraded B2 err stat */
#define    ixf6048_eRX_MST_SF_ALM           0x0008 /* Change in MST SF status */
#define    ixf6048_eRX_MST_RDI_ALM          0x0004 /* Change in MST RDI status */
#define    ixf6048_eRX_MST_EXC_B2_ERR_ALM   0x0002 /* Change in Excessive B2 err stat */
#define    ixf6048_eRX_MST_AIS_ALM          0x0001 /* Change in MST AIS */

/* NOTE: Some interupt Bits are Cleared by reading the related status bits.*/
/* Interupt Bits are defined by ixf6048_RxMstAlarms_e */

/***********************************************************************
 * Channel Registers: R_MSA_C: Rx MSA Configuration 
 *  Page  - (1cc)A0H
 **********************************************************************/

#define  ixf6048_R_MSA_C_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA0)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_NO_FORCE_R_MSA_C_MASK  0x03FF
#define  ixf6048_bDAUAISC_EN             0x0200
#define  ixf6048_bLOPC_ON_SV_DEF_EN      0x0100
#define  ixf6048_bLOPC_ON_MSA_AIS_EN     0x0080
#define  ixf6048_bRCV_CONC_PT_DET_CNFG   0x0040

#define  ixf6048_bAU_PTR_JUST_CNFG       0x0020
#define  ixf6048_mEXP_AU_PNTR_SS         0x0018
#define  ixf6048_STORE_EXPC_AU_PNTR_SS_BITS(Reg,SS)    \
         Reg = (Reg & ~ixf6048_mEXP_AU_PNTR_SS) | ((SS & 0x3)<<3)

#define  ixf6048_EXP_AU_PNTR_SS(SS) ((SS&0x3)<<3)

#define  ixf6048_bRCV_MSA_AIS_EN         0x0004
#define  ixf6048_bRCV_MSA_AIS_FORCE      0x0002
#define  ixf6048_bAU_PTR_SS_EN           0x0001

/***********************************************************************
 * Channel Counter Registers:  
 *                  R_AU_NCNT: Rx Negative AU Pointer Justification Event Counter 
 * Page - (1cc)A1H
 *                  R_AU_PCNT: Rx Positive AU Pointer Justification Event Counter 
 * Page - (1cc)A2H
 **********************************************************************/

#define  ixf6048_R_AU_NCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA1)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_AU_PCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA2)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_R_AU_NCNT(BaseAddress,Chan)   \
         (READ_FROM_HW(BaseAddress + ixf6048_R_AU_NCNT_CHAN_OFFSET(Chan)) & 0x07FF)

#define  ixf6048_GET_R_AU_PCNT(BaseAddress,Chan)   \
         (READ_FROM_HW(BaseAddress + ixf6048_R_AU_PCNT_CHAN_OFFSET(Chan)) & 0x07FF)

/***********************************************************************
 * Channel Registers:
 *                      IS_ADP: Rx MSA Interrupt Source    (COR)
 * Page  - (1cc)D2H     
 *                      IE_ADP: Rx MSA Interrupt Enable
 * Page  - (1cc)D6H
 *                      S_ADP: Rx MSA Status
 * Page  - (1cc)DAH
 * Masks are defined in ixf6048_RxMsaAlarms_e and ixf6048_RxMsaStatus_e 
 **********************************************************************/

#define  ixf6048_IS_ADP_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD2)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_R_AU_COUNTER_MASK	0x07FF

#define  ixf6048_IE_ADP_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD6)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_S_ADP_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xDA)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mIS_ADP    0x003F
#define  ixf6048_mIE_ADP    0x003F
#define  ixf6048_mS_ADP     0x0027

#define	ixf6048_eRX_MSA_LOPC_ALM         0x020 /* Change in LOPC status */
#define	ixf6048_eRX_MSA_AU_NEG_OVFL_ALM  0x010 /* AU Neg Just Cntr Overflow */
#define ixf6048_eRX_MSA_AU_POS_OVFL_ALM  0x008 /* AU Pos Just Cntr Overflow */
#define ixf6048_eRX_MSA_LOP_ALM          0x004 /* Change in Loss of Ptr status */
#define ixf6048_eRX_MSA_NDF_ALM          0x002 /* Change in New Data Flag status */
#define ixf6048_eRX_MSA_AU_AIS_ALM       0x001  /* Change in AU AIS status */


/***********************************************************************
 * Channel Registers: R_HPT_C1: Rx HPT Configuration 1
 * Page  - (1cc)A4H
 **********************************************************************/

#define  ixf6048_R_HPT_C1_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA4)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mHPT_RDI_DET_CNT                0xC000 /* Mask bits 15:14*/
#define  ixf6048_HPT_RDI_DET_ON_3_G1_BYTES_EN    0x0000
#define  ixf6048_HPT_RDI_DET_ON_5_G1_BYTES_EN    0x4000
#define  ixf6048_HPT_RDI_DET_ON_10_G1_BYTES_EN   0x8000
#define  ixf6048_HPT_RDI_DET_ON_16_G1_BYTES_EN   0xC000

#define  ixf6048_SET_HPT_RDI_DET_CNT(Reg, DetCnt)   \
            (Reg |= (DetCnt << 14) & ixf6048_mHPT_RDI_DET_CNT) 

#define  ixf6048_bHPT_RDI_DET_ENH_CNFG           0x2000
#define  ixf6048_bC2_MS_MTCH_CNT                 0x1000

#define  ixf6048_bHP_SLM_ON_C2_UNSTABLE_EN       0x0800
#define  ixf6048_bB3_CNTR_CNFG                   0x0400
#define  ixf6048_bHPT_REI_CNTR_CNFG              0x0200
#define  ixf6048_bRCV_STUFF_CFG                  0x0100

#define  ixf6048_bHPT_RDI_ON_LCD_EN              0x0080


#define  ixf6048_mHPT_RDI_ON_SLM_EN                 0x0060 /* Masks Bits 6:5 */
#define  ixf6048_HPT_SLM_SETS_HPT_RDI_PAY_DEF_EN    0x0060
#define  ixf6048_HPT_SLM_SETS_HPT_RDI_CON_DEF_EN    0x0040
#define  ixf6048_DISABLE_HPT_RDI_ON_HPT_SLM_EN      0x0000
#define  ixf6048_SET_HPT_RDI_ON_SLM_EN(Reg, HptRdiSetVal)    \
            (Reg |= (HptRdiSetVal << 5) & ixf6048_mHPT_RDI_ON_SLM_EN)

#define  ixf6048_bHPT_RDI_ON_TIM_EN                 0x0010
#define  ixf6048_bHPT_RDI_ON_UNEQP_EN               0x0008

#define  ixf6048_mHPT_RDI_GEN_ENH_CNFG              0x0006 /* Mask Bits 2:1 */
#define  ixf6048_GEN_ENHANCED_HPT_RDI_EN            0x0004
#define  ixf6048_GEN_HPT_RDI_AS_111_EN              0x0002
#define  ixf6048_GEN_HPT_RDI_AS_100_EN              0x0000
#define  ixf6048_SET_HPT_RDI_GEN_ENH_CNFG(Reg, HptRdiGenVal)    \
            (Reg |= (HptRdiGenVal << 1) & ixf6048_mHPT_RDI_GEN_ENH_CNFG)

#define  ixf6048_bG1_SP_ON_RPAL_CNFG                0x0001

/***********************************************************************
 * Channel Registers: R_HPT_C2: Rx HPT Configuration 2
 * Page  - (1cc)A5H
 **********************************************************************/
#define  ixf6048_R_HPT_C2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA5)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mR_HPT_C2               0x3F3E

#define  ixf6048_bHP_TIM_ON_CRC7_ERR_DSB 0x2000
#define  ixf6048_bHP_TIM_ON_UNSTABLE_EN  0x1000

#define  ixf6048_bRCV_J1_STABLE_CNFG     0x0800

#define  ixf6048_mRCV_J1_CNF               0x0700 /* Masks bits 10:8 */
/* -- For Below Masks see RCV_J0_CNF above for Generic values prior to Shift  
#define  ixf6048_RX_J1_64_WITH_LF_CR_EN    0x0700
#define  ixf6048_RX_J1_64_FREE_FORM_EN     0x0600
#define  ixf6048_RX_J1_16_WITH_CRC7_EN     0x0400
#define  ixf6048_IGNORE_RX_J1_EN           0x0200
#define  ixf6048_RX_J1_1_BYTE_EN           0x0000
*/
#define  ixf6048_GET_RCV_J1_CNF_FORMAT(Reg)( (Reg & ixf6048_mRCV_J1_CNF) >>8)
#define  ixf6048_SET_RCV_J1_CNF_FORMAT(Reg, NewFormat) \
            (Reg = ((Reg & ~ixf6048_mRCV_J1_CNF) | (NewFormat << 8)))
            
#define  ixf6048_SHIFT_RCV_J1_CNF_FORMAT(NewFormat) (NewFormat << 8)                                               

#define  ixf6048_bRCV_HPT_AIS_FRC          0x0020   /* Used in Testing */
#define  ixf6048_bRCV_HPT_AIS_SLM_ENBL     0x0010   

#define  ixf6048_bRCV_HPT_AIS_UNEQ_ENBL    0x0008
#define  ixf6048_bRCV_HPT_AIS_TIM_ENBL     0x0004
/***********************************************************************
 * Channel Registers: EXP_C2: Expected C2 Byte
 * Page  - (1cc)A6H
 **********************************************************************/

#define  ixf6048_EXP_C2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA6)+(Chan<<8))<<REG_SHIFT)

         /* Sets value of Expected C2 Byte */
#define  ixf6048_PUT_EXP_C2(BaseAddress,Chan,C2) \
         (WRITE_TO_HW(BaseAddress + ixf6048_EXP_C2_CHAN_OFFSET(Chan), \
         (C2 & 0x00FF)))

/***********************************************************************
 * Channel Registers: R_C2: Rx C2 Byte -- Page  - (1cc)A7H
 **********************************************************************/

#define  ixf6048_R_C2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA7)+(Chan<<8))<<REG_SHIFT)

         /* Returns value of Rx C2 Byte */
#define  ixf6048_GET_RX_C2(BaseAddress,Chan) \
         (READ_FROM_HW(BaseAddress + ixf6048_R_C2_CHAN_OFFSET(Chan))&0x00FF)

/***********************************************************************
 * Channel Registers: R_HPT_RDI: Rx HPT RDI Bits
 * Page  - (1cc)A8H
 **********************************************************************/

#define  ixf6048_R_HPT_RDI_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA8)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mHPT_RCV_RDI_BITS 0x0E
#define  ixf6048_mHPT_RCV_SP_BIT   0x01
#define  ixf6048_mHPT_RDI_G1       0x000F

         /* Returns value of Rx HPT RDI Bits[3:1] plus G1 Spare Bit[0] */
#define  ixf6048_GET_RX_HPT_RDI(BaseAddress,Chan) \
         (READ_FROM_HW(BaseAddress + ixf6048_R_HPT_RDI_CHAN_OFFSET(Chan)) & ixf6048_mHPT_RDI_G1)

/***********************************************************************
 * Channel Counter Registers:  
 *                  B3_ECNT:    B3 Error Event Counter 
 * Page - (1cc)A9H
 *                  HPTREI_CNT: HPT REI        Counter 
 * Page - (1cc)AAH
 **********************************************************************/

#define  ixf6048_B3_ECNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xA9)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_HPTREI_CNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xAA)+(Chan<<8))<<REG_SHIFT)


#define  ixf6048_GET_B3_ERRCNT(BaseAddress,Chan)   \
         (READ_FROM_HW(BaseAddress + ixf6048_B3_ECNT_CHAN_OFFSET(Chan)))

#define  ixf6048_GET_HPT_REI_CNT(BaseAddress,Chan)   \
         (READ_FROM_HW(BaseAddress + ixf6048_HPTREI_CNT_CHAN_OFFSET(Chan)))


/* NOTE: Registers (1cc)ADH and (1cc)AF have been defined above */

/***********************************************************************
 * Channel Registers:
 *    IS_HPT: Rx HPT Interrupt Source -- Page  - (1cc)D3H
 *    IE_HPT: Rx HPT Interrupt Enable -- Page  - (1cc)D7H
 *     S_HPT: Rx HPT Status           -- Page  - (1cc)DBH
 **********************************************************************/

#define  ixf6048_IS_HPT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD3)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_IE_HPT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xD7)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_S_HPT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xDB)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mIS_HPT    0xFFF
#define  ixf6048_mIE_HPT    0xFFF
#define  ixf6048_mS_HPT     0x0FF

/* Note: The Individual Interupts are defined by ixf6048_RxHptAlarms_e 
         and ixf6048_RxHptStatus_e */

#define ixf6048_eRX_HPT_RDI_ALM         0x0800 /* Change in HPT RDI status */
#define ixf6048_eRX_HPT_C2_ALM          0x0400 /* Change in Rx C2 byte */
#define ixf6048_eRX_HPT_REI_OVFL_ALM    0x0200 /* HPT REI cntr overflow */
#define ixf6048_eRX_HPT_B3_OVFL_ALM     0x0100 /* B3 cntr overflow */
#define ixf6048_eRX_HPT_C2_UNSTABLE_ALM 0x0080 /* Change in Rx C2 unstable status */
#define ixf6048_eRX_HPT_J1_ALL_ZERO_ALM 0x0040 /* Change in Rx J1 unstable status */
#define ixf6048_eRX_HPT_J1_UNSTABLE_ALM 0x0020 /* Change in Rx J1 unstable status */
#define ixf6048_eRX_HPT_J1_MISMATCH_ALM 0x0010 /* Change in Rx J1 Mismatch status */
#define ixf6048_eRX_HPT_J1_CRC7_ALM     0x0008 /* Change in Rx J1 CRC7 status */
#define ixf6048_eRX_HPT_UNEQ_ALM        0x0004 /* Change in HPT Unequipped status */
#define ixf6048_eRX_HPT_SLM_ALM         0x0002 /* Change in HPT SLM status */
#define ixf6048_eRX_HPT_VC_AIS_ALM      0x0001  /* Change in VC AIS status */



/***********************************************************************
 * Channel Registers: T_RMST_OP: Tx RST,MST Operational Reg
 * -- Page  - (1cc)E0H
 **********************************************************************/

#define  ixf6048_T_RMST_OP_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE0)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mT_RMST_OP            0xFFFF

#define  ixf6048_bXMT_SCRMBL_CNFG            0x8000
#define  ixf6048_bXMT_UN_BY_CNFG             0x4000

#define  ixf6048_mXMT_TZ0_CNFG               0x3000   /* Masks Bits 13:12 */
         /* see ixf6048_TxTzo_e */
#define  ixf6048_SET_TX_TZ0_BY_LINE_SPEED_EN(Reg, LineSpeed)  \
            (Reg |= (LineSpeed << 12) & ixf6048_mXMT_TZ0_CNFG);

#define  ixf6048_bTX_TZ0_BY_LINE_SPEED_EN(LineSpeed)  \
            ((LineSpeed << 12) & ixf6048_mXMT_TZ0_CNFG)

#define  ixf6048_mXMT_SZ0_CNFG                0x0C00   /* Masks Bits 11:10 */
         /* see ixf6048_TxSzo_e */
#define  ixf6048_SET_TX_SZ0_BY_LINE_SPEED_EN(Reg, LineSpeed)  \
            (Reg |= (LineSpeed << 10) & ixf6048_mXMT_SZ0_CNFG)

#define  ixf6048_bTX_SZ0_BY_LINE_SPEED_EN(LineSpeed)  \
            ((LineSpeed << 10) & ixf6048_mXMT_SZ0_CNFG)

#define  ixf6048_mXMT_FZ0_CNFG                0x0300   /* Masks Bits 9:8 */  
         /* see ixf6048_TxFzo_e */
#define  ixf6048_SET_TX_FZ0_BY_LINE_SPEED_EN(Reg, LineSpeed)  \
            (Reg |= (LineSpeed << 8) & ixf6048_mXMT_FZ0_CNFG)

#define  ixf6048_bTX_FZ0_BY_LINE_SPEED_EN(LineSpeed)  \
            ((LineSpeed << 8) & ixf6048_mXMT_FZ0_CNFG)

#define  ixf6048_mXMT_J0_CNFG                 0x00C0  /* Masks Bits 7:6 */ 
#define  ixf6048_TX_JN_64_EN                  0x0003
#define  ixf6048_TX_JN_16_EN                  0x0002
#define  ixf6048_TX_JN_DEFAULT_EN             0x0001
#define  ixf6048_TX_JN_1_BYTE_EN              0x0000
         /* See ixf6048_TxJ0Format_e */
#define  ixf6048_GET_TX_J0_CNFG(Reg) ((Reg & ixf6048_mXMT_J0_CNFG) >> 6)
#define  ixf6048_SET_TX_J0_CNFG(Reg, NewCnfg)   \
            (Reg |= ((Reg & ~ixf6048_mXMT_J0_CNFG) | (NewCnfg << 6)))

#define  ixf6048_SHIFT_TX_JO_CNFG(NewCnfg) NewCnfg << 6

#define  ixf6048_mINV_A1                    0x0030  /* Masks Bits 5:4 */
#define  ixf6048_bFORCE_A1_ERRS             0x0020
#define  ixf6048_bFORCE_4_A1_ERRS           0x0030
#define  ixf6048_bNO_A1_INVERSION           0x0000

#define  ixf6048_mINV_B2                    0x000C  /* Masks Bits 3:2 */
#define  ixf6048_bFORCE_B2_ERRS             0x0008
#define  ixf6048_bFORCE_ONE_B2_ERR          0x000C
#define  ixf6048_bNO_B2_INVERSION           0x0000

#define  ixf6048_mINV_B1                    0x0003  /* Masks Bits 1:0 */
#define  ixf6048_bFORCE_B1_ERRS             0x0002
#define  ixf6048_bFORCE_ONE_B1_ERR          0x0003
#define  ixf6048_bNO_B1_INVERSION           0x0000




/***********************************************************************
 * Channel Registers: T_SC_RSOH: Tx Source Configuration for RST Overhead Bytes
 * Page  - (1cc)E1H
 **********************************************************************/

#define  ixf6048_T_SC_RSOH_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE1)+(Chan<<8))<<REG_SHIFT)

/* PSN_ADDED for B1 Release */
#define  ixf6048_bTSOH_INS_ENA                0x8000
/* PSN_END B1 Release Changes */

#define  ixf6048_bMT_UN_RSOH_SRC              0x0400

#define  ixf6048_mXMT_D1D3_SRC                0x0300  /* Masks bits 9:8 */
         /* see  ixf6048_TxD1D3Source_e */
#define  ixf6048_SET_XMT_D1D3_SRC(Reg, D1D3Source)   \
            (Reg |= (D1D3Source << 8) & ixf6048_mXMT_D1D3_SRC)

#define  ixf6048_bXMT_D1D3_SRC(D1D3Source)   \
            ((D1D3Source << 8) & ixf6048_mXMT_D1D3_SRC)

#define  ixf6048_bXMT_NU_2_SRC                0x0080

#define  ixf6048_mXMT_F1_SRC                  0x0060 /* Masks Bits 6:5 */
         /* see  ixf6048_TxF1Source_e */
#define  ixf6048_SET_XMT_F1_SRC(Reg, F1Source)   \
            (Reg |= (F1Source << 5) & ixf6048_mXMT_F1_SRC)

#define  ixf6048_bXMT_F1_SRC(F1Source)   \
            ((F1Source << 5) & ixf6048_mXMT_F1_SRC)

#define  ixf6048_mXMT_E1_SRC                  0x001C /* Masks Bits 4:2 */
         /* see  ixf6048_TxE1Source_e */
#define  ixf6048_SET_XMT_E1_SRC(Reg, E1Source)   \
            (Reg |= (E1Source << 2) & ixf6048_mXMT_E1_SRC)

#define  ixf6048_bXMT_E1_SRC(E1Source)   \
            ((E1Source << 2) & ixf6048_mXMT_E1_SRC)

#define  ixf6048_bXMT_NU_1_Z0_SRC             0x0002
#define  ixf6048_bXMT_J0_SRC                  0x0001


/***********************************************************************
 * Channel Register: T_SC_MSOH: Tx Source Configuration for MSOH Bytes Register
 * Page  - (1cc)E2H
 **********************************************************************/

#define  ixf6048_T_SC_MSOH_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE2)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mMST_RDI_ON_HYST_CNFG           0xC000 /* Masks Bits 15:14 */
         /* See ixf6048_TxMstRdiHysteresis_e for NumFramesEn*/
#define  ixf6048_SET_MST_RDI_ON_HYST_CNFG(Reg, NumFramesEn)  \
            (Reg |= (NumFramesEn << 14) & ixf6048_mMST_RDI_ON_HYST_CNFG)

#define  ixf6048_bMST_RDI_ON_HYST_CNFG(NumFramesEn)  \
            ((NumFramesEn << 14) & ixf6048_mMST_RDI_ON_HYST_CNFG)
            
#define  ixf6048_mXMT_MST_REI_SRC                0x3000 /*Masks Bits 13:12*/
         /* See ixf6048_TxMstReiSource_e for Source */
#define  ixf6048_SET_MST_REI_SRC(Reg, Source)  \
            (Reg |= (Source << 12) & ixf6048_mXMT_MST_REI_SRC)

#define  ixf6048_bMST_REI_SRC(Source)  \
            ((Source << 12) & ixf6048_mXMT_MST_REI_SRC)

#define  ixf6048_mXMT_MST_RDI_SRC                0x0C00 /*Masks Bits 11:10*/
         /* see ixf6048_TxMstRdiSource_e for Source Vals */
#define  ixf6048_SET_MST_RDI_SRC(Reg, Source)  \
            (Reg |= (Source << 10) & ixf6048_mXMT_MST_RDI_SRC)

#define  ixf6048_bMST_RDI_SRC(Source)  \
            ((Source << 10) & ixf6048_mXMT_MST_RDI_SRC)

#define  ixf6048_bXMT_K2_APS_SRC                 0x0200
#define  ixf6048_bXMT_K1_SRC                     0x0100
#define  ixf6048_bTSAL_BUS_CNFG                  0x0080
#define  ixf6048_bXMT_PRC_MSOH_SRC               0x0040

#define  ixf6048_mXMT_E2_SRC                     0x0038 /*Masks Bits 5:3*/
         /* see ixf6048_TxE2Source_e for Source Vals */
#define  ixf6048_SET_E2_SRC(Reg, Source)  \
            (Reg |= (Source << 3) & ixf6048_mXMT_E2_SRC)

#define  ixf6048_bE2_SRC(Source)  \
            ((Source << 3) & ixf6048_mXMT_E2_SRC)

#define  ixf6048_mXMT_D4D1_SRC                   0x0006 /*Masks Bits 2:1*/
#define  ixf6048_D4_D12_SRC_576K_BITS_EN         0x0000 /* Clear Bits 2:1 */
#define  ixf6048_D4_D12_SRC_UNUSED_EN            0x0002
#define  ixf6048_D4_D12_SRC_FROM_DOWNSTREAM_EN   0x0004
         /* See ixf6048_TxD4D12Source_e for Source */
#define  ixf6048_SET_XMT_D4D1_SRC(Reg, Source)  \
            (Reg |= (Source <<1) & ixf6048_mXMT_D4D1_SRC)

#define  ixf6048_bXMT_D4D1_SRC(Reg, Source)  \
            ((Source <<1) & ixf6048_mXMT_D4D1_SRC)

#define  ixf6048_bXMT_UN_MSOH_SRC                0x0001

/* Note that Reg T_J0_STRA (1cc)E4H has been defined above */


/***********************************************************************
 * Channel Register: MP_TK2K1: uProcessor Provided Transmit K1 & K2 
 *                             Bytes Register
 * Page  - (1cc)E5H
 **********************************************************************/

#define  ixf6048_MP_TK2K1_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE5)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mTX_K2_BYTE 0xFF00
#define  ixf6048_mTX_K1_BYTE 0x00FF 

#define  ixf6048_GET_TX_K2_BYTE(BaseAddress, Chan)  \
          ((READ_FROM_HW(BaseAddress + ixf6048_MP_TK2K1_CHAN_OFFSET(Chan)) & ixf6048_mTX_K2_BYTE) >> 8)

#define  ixf6048_SET_TX_K2_BYTE(BaseAddress, Chan, K2Byte)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_MP_TK2K1_CHAN_OFFSET(Chan), \
                  ((K2Byte << 8) & ixf6048_mTX_K2_BYTE)))

#define  ixf6048_GET_TX_K1_BYTE(BaseAddress, Chan)  \
          (READ_FROM_HW(BaseAddress + ixf6048_MP_TK2K1_CHAN_OFFSET(Chan)) & ixf6048_mTX_K1_BYTE)

#define  ixf6048_SET_TX_K1_BYTE(BaseAddress, Chan, K1Byte)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_MP_TK2K1_CHAN_OFFSET(Chan),  \
                  (K1Byte &  ixf6048_mTX_K1_BYTE)))

/***********************************************************************
 * Channel Register: MP_TS1: uProcessor Provided Transmit S1 
 *                           Byte Register
 * Page  - (1cc)E6H
 **********************************************************************/

#define  ixf6048_MP_TS1_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE6)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_TX_S1_BYTE(Chan) \
         (ixf6048_MP_TS1_CHAN_OFFSET(Chan) & 0x00FF)

#define  ixf6048_SET_TX_S1_BYTE(BaseAddress, Chan, S1Byte)  \
          (WRITE_TO_HW(BaseAddress + ixf6048_MP_TS1_CHAN_OFFSET(Chan),  \
                  (S1Byte &  0x00FF)))


/*#####################################################################*
 *  SONET/SDH TX MUX SECTION ADAPTATION CHANNEL REGISTERS              *
 *                                                                     *
 *#####################################################################*/

/***********************************************************************
 * Channel Register: T_AU_PTS: Tx AU Pointer Operations Configuration  
 * Page  - (1cc)E9H
 **********************************************************************/

#define  ixf6048_T_AU_PTS_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE9)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_bXMT_SET_AU_PT_VAL         0x8000
#define  ixf6048_bXMT_SET_AU_POS            0x4000
#define  ixf6048_bXMT_SET_AU_NEG            0x2000
#define  ixf6048_bXMT_SET_NDF               0x1000

#define  ixf6048_mXMT_AU_PT_SS              0x0C00 /* Bits[11:10] */
#define  ixf6048_SET_XMT_AU_PT_SS(Reg, PtrSS)   \
            (Reg = (Reg & ~ixf6048_mXMT_AU_PT_SS) | \
                      ((PtrSS << 10) & ixf6048_mXMT_AU_PT_SS) )

#define  ixf6048_mXMT_AU_PT_VAL             0x03FF
#define  ixf6048_SET_XMT_AU_PT_VAL(Reg, PtrVAL)   \
            (Reg = (Reg & ~ixf6048_mXMT_AU_PT_VAL) | \
                     (PtrVAL & ixf6048_mXMT_AU_PT_VAL) )
/***********************************************************************
 * Channel Register: T_CAU_PT: Tx Current AU Pointer Value
 * Page  - (1cc)EAH
 **********************************************************************/

#define  ixf6048_T_CAU_PT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xEA)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mXMT_CUR_AU_PT             0x03FF

/*#####################################################################*
 *  SONET/SDH TX HIGH ORDER PATH TERMINATION CHANNEL REGISTERS               *
 *                                                                     *
 *#####################################################################*/

/***********************************************************************
 * Channel Register: T_HPT_C: Tx HPT Configuration
 * Page  - (1cc)E8H
 **********************************************************************/

#define  ixf6048_T_HPT_C_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xE8)+(Chan<<8))<<REG_SHIFT)

/* PSN_ADDED For B1 Release */
#define  ixf6048_bTPOH_INS_ENA           0x8000
/* PSN_END B1 Release Changes */

#define  ixf6048_bXMT_STUF_CNFG          0x4000

#define  ixf6048_bXMT_AU_PNTR_SRC        0x2000

#define  ixf6048_XMT_POH_SRC             0x1000  

#define  ixf6048_XMT_H4_CNFG             0x0400

#define  ixf6048_mXMT_F2_SRC                     0x0300 /* Masks Bits 9:8 */ 
#define  ixf6048_F2_SRC_64K_BITS_EN              0x0000 
#define  ixf6048_F2_SRC_UNUSED_EN                0x0100 
#define  ixf6048_F2_SRC_FROM_DOWNSTREAM_EN       0x0200
#define  ixf6048_SET_XMT_F2_SRC(Reg, F2Src)   \
           (Reg = (Reg & ~ixf6048_mXMT_F2_SRC) |  \
                     ((F2Src <<8) & ixf6048_mXMT_F2_SRC) )

#define  ixf6048_mXMT_F3_SRC                     0x00C0 /* Masks Bits 7:6 */ 
#define  ixf6048_F3_SRC_64K_BITS_EN              0x0000 
#define  ixf6048_F3_SRC_UNUSED_EN                0x0040
#define  ixf6048_F3_SRC_SRC_FROM_DOWNSTREAM_EN   0x0080
#define  ixf6048_SET_XMT_F3_SRC(Reg, F3Src)   \
           (Reg = (Reg & ~ixf6048_mXMT_F3_SRC) |  \
                     ((F3Src <<6) & ixf6048_mXMT_F3_SRC) )

#define  ixf6048_mXMT_HPT_REI_SRC                0x000C /* Masks Bits 3:2 */ 
#define  ixf6048_G1_REI_SRC_HW_SUPPLIED_EN       0x0000 /* Clear Bits 3:2 */
#define  ixf6048_G1_REI_SRC_TPAL_INPUT_PIN_EN    0x0004
#define  ixf6048_G1_REI_SRC_SET_TO_0_EN          0x0008
#define  ixf6048_SET_HPT_REI_SRC(Reg, ReiSrc)   \
           (Reg = (Reg & ~ixf6048_mXMT_HPT_REI_SRC) |  \
                     ((ReiSrc <<2) & ixf6048_mXMT_HPT_REI_SRC) )

#define  ixf6048_mXMT_HPT_RDI_SRC                0x0003 /* Masks Bits 1:0 */ 
#define  ixf6048_G1_RDI_SRC_HW_SUPPLIED_EN       0x0000 
#define  ixf6048_G1_RDI_SRC_TPAL_INPUT_PIN_EN    0x0001
#define  ixf6048_G1_RDI_SRC_U_PROCESSOR_SUPPLIED_EN 0x0002
#define  ixf6048_SET_HPT_RDI_SRC(Reg, RdiSrc)   \
           (Reg = (Reg & ~ixf6048_mXMT_HPT_RDI_SRC) |  \
                     (RdiSrc & ixf6048_mXMT_HPT_RDI_SRC) )

/***********************************************************************
 * Channel Register: T_HPT_OPC: Tx HPT Operational Configuration
 * Page  - (1cc)EFH
 **********************************************************************/

#define  ixf6048_T_HPT_OPC_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xEF)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_bTPAL_BUS_CNFG              0x8000
#define  ixf6048_bINV_H1_CONC_IND            0x1000

#define  ixf6048_mHPT_RDI_XMT_HYST_CNFG          0x0C00 /* Masks Bits 11:10 */
#define  ixf6048_G1_RDI_STABLE_FOR_20_FRAMES_EN  0x0C00
#define  ixf6048_G1_RDI_STABLE_FOR_10_FRAMES_EN  0x0800
#define  ixf6048_G1_RDI_STABLE_FOR_5_FRAMES_EN   0x0400
#define  ixf6048_G1_RDI_STABLE_FOR_1_FRAME_EN    0x0000 /* Clear Bits 11:10 */
#define  ixf6048_SET_HPT_RDI_XMT_HYST_CNFG(Reg, Config)   \
           (Reg = (Reg & ~ixf6048_mHPT_RDI_XMT_HYST_CNFG) |  \
                     ((Config <<6) & ixf6048_mHPT_RDI_XMT_HYST_CNFG) )

#define  ixf6048_bHPT_RDI_XMT_ENH_CNFG          0x0200
#define  ixf6048_bXMT_UNEQP                     0x0100

#define  ixf6048_bXMT_UNEQP_J1_CNFG              0x0080  
#define  ixf6048_bXMT_UNEQP_N1_CNFG              0x0040

#define  ixf6048_mXMT_J1_CNFG                    0x0030 /*Masks bits 5:4 */
/* Masks for XMT J1 CNFG Defined above in ixf6048_T_RMST_OP_CHAN_OFFSET */
#define  ixf6048_GET_TX_J1_CNFG(Reg) ((Reg & ixf6048_mXMT_J1_CNFG) >>4)
#define  ixf6048_SET_TX_J1_CNFG(Reg, NewCnfg)   \
            (Reg = ((Reg & ~ixf6048_mXMT_J1_CNFG) | \
                    ((NewCnfg << 4)& ixf6048_mXMT_J1_CNFG)) )

#define  ixf6048_SHIFT_TX_J1_CNFG(NewCnfg) NewCnfg << 4

#define  ixf6048_bXMT_MSA_AIS_FRC                0x0008
#define  ixf6048_bXMT_HPT_AIS_FRC                0x0004

#define  ixf6048_mINV_B3                         0x0003 /* Masks bits 1:0 */
#define  ixf6048_INVERT_B3_BYTE_NO_INVERSION     0x0000 /* Clear Bits 1:0 */
#define  ixf6048_bINVERT_B3_BYTE_FOREVER          0x0002
#define  ixf6048_bINVERT_B3_BYTE_FOR_1_FRAME      0x0003

/***********************************************************************
 * Channel Register: MP_TC2: uProcessor Provided Tx C2 Byte
 * Page  - (1cc)EBH
 **********************************************************************/

#define  ixf6048_MP_TC2_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xEB)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mXMT_C2              0x00FF
#define  ixf6048_SET_XMT_C2(Reg, C2Val) \
            Reg =  C2Val & ixf6048_mXMT_C2;
/***********************************************************************
 * Channel Register: MP_THPTRDI: uProcessor Provided Tx HPT-RDI Bits
 * Page  - (1cc)ECH
 **********************************************************************/

#define  ixf6048_MP_THPTRDI_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0xEC)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mXMT_HPT_RDI         0x000E
#define  ixf6048_SET_XMT_HPT_RDI(Reg, G1Val)  \
            (Reg = ((Reg & ~ixf6048_mXMT_HPT_RDI) | \
                    ((G1Val<<1) & ixf6048_mXMT_HPT_RDI)) )

#define  ixf6048_bXMT_G1_SP_BIT       0x0001


/* Note that Reg (1cc)EEH has been defined above */


/*#####################################################################*
 *  ATM RECEIVE CHANNEL REGISTERS
 *                                                                     *
 *#####################################################################*/

/***********************************************************************
 * Channel Register: R_ACPCNF: Rx ATM Cell Processor Configuration (COR)
 * Page  - (1cc)20H
 **********************************************************************/

#define  ixf6048_R_ACPCNF_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x20)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mR_ACPCNF_CHAN              0x07FF  
#define  ixf6048_bRCV_GFC_CONTROLLER         0x0400  /* Read Only */
#define  ixf6048_bRCV_GFC_CONTROLLED         0x0200  /* Read Only */

#define  ixf6048_bRCV_IU_FLTR_EN             0x0100
#define  ixf6048_bRCV_CORRECT_EN             0x0080

#define  ixf6048_mRCV_CORR_DET_CNF           0x0060  /* Masks bits 6:5 */
#define  ixf6048_RX_HEC_MODE_ON_1_CELL_EN    0x0000  /* Clear Bits 6:5 */
#define  ixf6048_RX_HEC_MODE_ON_2_CELL_EN    0x0020 
#define  ixf6048_RX_HEC_MODE_ON_4_CELL_EN    0x0040
#define  ixf6048_RX_HEC_MODE_ON_8_CELL_EN    0x0060

#define  ixf6048_SET_RCV_CORR_DET_CNF(Reg, CorVal) \
            (Reg = ( (Reg & ~ixf6048_mRCV_CORR_DET_CNF) | \
                     ( (CorVal<<5) & ixf6048_mRCV_CORR_DET_CNF) ) ) 

#define  ixf6048_bRCV_LCD_CNF                0x0010
#define  ixf6048_bRCV_SYNC_CNF               0x0008
#define  ixf6048_bRCV_PRESYNC_CNF            0x0004  
#define  ixf6048_bRCV_HEC_ADD                0x0002  /* Add "01010101" to HEC prior to comparison */
#define  ixf6048_bRCV_DESCR_EN               0x0001  /* descrambler Enable */

/***********************************************************************
 * Channel Register: R_IUCFLTR: Rx Idle/Unassigned Cell Filter 
 * Page  - (1cc)21H
 **********************************************************************/

#define  ixf6048_R_IUCFLTR_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x21)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mRCV_GFC_MASK               0xF000 /* Masks bits 15:12 */
#define  ixf6048_SET_RCV_GFC_MASK(Reg, Mask) \
            (Reg = ( (Reg & ~ixf6048_mRCV_GFC_MASK) | \
                     (Mask<<12) ) )

#define  ixf6048_mRCV_PTI_MASK               0x0E00 /* Masks bits 11:9  */
#define  ixf6048_SET_RCV_PTI_MASK(Reg, Mask) \
            (Reg = ( (Reg & ~ixf6048_mRCV_PTI_MASK) | \
                     (Mask<<9) ) )

#define  ixf6048_bRCV_CLP_MASK               0x0100 /* Masks bit 8 */ 
#define  ixf6048_SET_RCV_CLP_MASK(Reg, Mask) \
            (Reg = ( (Reg & ~ixf6048_bRCV_CLP_MASK) | \
                     (Mask<<8) ) )

#define  ixf6048_mRCV_GFC_PATT               0x00F0 /* Masks bits 7:4 */
#define  ixf6048_SET_RCV_GFC_PATT(Reg, Pattern) \
            (Reg = ( (Reg & ~ixf6048_mRCV_GFC_PATT) | \
                     (Pattern<<4) ) )

#define  ixf6048_mRCV_PTI_PATT               0x000E /* Masks bits 3:1 */
#define  ixf6048_SET_RCV_PTI_PATT(Reg, Pattern) \
            (Reg = ( (Reg & ~ixf6048_mRCV_PTI_PATT) | \
                     (Pattern<<1) ) )

#define  ixf6048_bRCV_CLP_PATT               0x0001 /* Masks bit 0 */
#define  ixf6048_SET_RCV_CLP_PATT(Reg, Pattern) \
            (Reg = ( (Reg & ~ixf6048_bRCV_CLP_PATT) | Pattern ) )
                      
/***********************************************************************
 * Channel Register: R_LCDFLTR: Rx LCD Filter
 * Page  - (1cc)23-22H
 **********************************************************************/

#define  ixf6048_R_LCDFLTR_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x22)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_LCDFLTR_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x23)+(Chan<<8))<<REG_SHIFT)

/***********************************************************************
 * Channel Register: R_ACELLCNT: Rx ATM Cell Counter 
 * Page  - (1cc)25-24H
 **********************************************************************/

#define  ixf6048_R_ACELLCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x24)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_ACELLCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x25)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_RX_ATM_CELL_CNT(BaseAddress,Chan)                 \
         ((READ_FROM_HW(BaseAddress + ixf6048_R_ACELLCNT_CHAN_LSB_OFFSET(Chan))) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_ACELLCNT_CHAN_MSB_OFFSET(Chan))&0x00FF)<<16))

/***********************************************************************
 * Channel Register: R_ICELLCNT: Rx Idle Cell Counter  
 * Page  - (1cc)27-26H
 **********************************************************************/

#define  ixf6048_R_ICELLCNT_CHAN_LSB_OFFSET(Chan)  \
         (((1<<10)+(0x26)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_ICELLCNT_CHAN_MSB_OFFSET(Chan)  \
         (((1<<10)+(0x27)+(Chan<<8)) <<REG_SHIFT)

#define  ixf6048_GET_RX_IDLE_CELL_CNT(BaseAddress,Chan)                 \
         ((READ_FROM_HW(BaseAddress + ixf6048_R_ICELLCNT_CHAN_LSB_OFFSET(Chan))) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_ICELLCNT_CHAN_MSB_OFFSET(Chan))&0x00FF)<<16))

/***********************************************************************
 * Channel Register: R_CHECNT: Rx Correctable HEC Error Counter 
 * Page  - (1cc)28H
 **********************************************************************/

#define  ixf6048_R_CHECNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x28)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_COR_HEC_CNT(BaseAddress,Chan)  \
             (READ_FROM_HW(BaseAddress + ixf6048_R_CHECNT_CHAN_OFFSET(Chan)))
/***********************************************************************
 * Channel Register: R_UHECNT: Rx Uncorrectable HEC Error Counter 
 * Page  - (1cc)29H
 **********************************************************************/

#define  ixf6048_R_UHECNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x29)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_UNCOR_HEC_CNT(BaseAddress,Chan)  \
             (READ_FROM_HW(BaseAddress + ixf6048_R_UHECNT_CHAN_OFFSET(Chan)))
/***********************************************************************
 * Channel Register: R_CFOCNT: Rx Cell FIFO Overflow Counter 
 * Page  - (1cc)2AH
 **********************************************************************/

#define  ixf6048_R_CFOCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x2A)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_FIFO_OFLOW_CNT(BaseAddress,Chan)  \
             (READ_FROM_HW(BaseAddress + ixf6048_R_CFOCNT_CHAN_OFFSET(Chan)))

/***********************************************************************
 * Channel Register: R_ATMINT: Rx ATM Interrupt (and Status) Register
 * Page  - (1cc)2BH                                            (COR)
 **********************************************************************/

#define  ixf6048_R_ATMINT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x2B)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mRCV_ATM_INT_STATUS_BITS   0x0C00

/* Note: The Status Bits have been defined in ixf6048_RxAtmStatus_e */
/* Note: The Interupt Masks are defined in ixf6048_RxAtmAlarms_e */

/***********************************************************************
 * Channel Register: R_ATMINTEN: Rx ATM Interrupt Enable  
 * Page  - (1cc)2CH
 **********************************************************************/

#define  ixf6048_R_ATMINTEN_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x2C)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mR_ATM_INTERRUPT_EN            0x03FF

/* Note: The Masks for the Interupt enables have 
         been defined in ixf6048_RxAtmAlarms_e.
*/

/*#####################################################################*
 *  ATM TRANSMIT CHANNEL REGISTERS
 *                                                                     *
 *#####################################################################*/
/***********************************************************************
 * Channel Register: T_ACPCNF: Tx ATM Cell Processor Configuration
 * Page  - (1cc)10H
 **********************************************************************/

#define  ixf6048_T_ACPCNF_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x10)+(Chan<<8))<<REG_SHIFT)

/* Bit 13 Added for B1 Release */
#define  ixf6048_bXMT_TEST_VPI			  0x2000 /* Bit 13 */

#define  ixf6048_mXMT_EXT_HEC             0x1800 /* Masks bit 12:11 */
#define  ixf6048_SET_XMT_EXT_HEC(Reg, ExtHecSrc) \
            (Reg = (Reg & ~ixf6048_mXMT_EXT_HEC) | (ExtHecSrc << 11) )

#define  ixf6048_bXMT_GFC_CONTROLLER      0x0400
#define  ixf6048_bXMT_GFC_CONTROLLED      0x0200
/* #define  ixf6048_SET_GFC_ROLE(reg, GfcRole) */

#define  ixf6048_mXMT_GFC_CNT                 0x01C0 /* Masks bits 8:6 */
#define  ixf6048_SET_HALT_RATIO(Reg, Ratio) \
            (Reg = (Reg & ~ixf6048_mXMT_GFC_CNT) | ((Ratio<< 6 )& ixf6048_mXMT_GFC_CNT))

#define  ixf6048_bXMT_ATM_READ_EN             0x0020

#define  ixf6048_mXMT_H_ERR_CNF              0x0018 /* Masks bits 4:3 */
#define  ixf6048_INSERT_HEC_COR_CELL_EN      0x0000 /* Clear Bits 4:3 */
#define  ixf6048_INSERT_1_HEC_UNCOR_CELL_EN  0x0008
#define  ixf6048_INSERT_HEC_COR_CELLS_EN     0x0010
#define  ixf6048_INSERT_HEC_UNCOR_CELLS_EN   0x0018

#define  ixf6048_bXMT_H_ERR_EN               0x0004 /* Controls the insertion of HEC Cells */
#define  ixf6048_bXMT_HEC_ADD                0x0002
#define  ixf6048_bXMT_SCR_EN                 0x0001
/***********************************************************************
 * Channel Register: T_ICELLP: Tx Idle Cell Pattern 
 * Page  - (1cc)11H
 **********************************************************************/

#define  ixf6048_T_ICELLP_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x11)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mTX_PAYLOAD_PATTERN    0xFF00
#define  ixf6048_mTX_GFC_PATTERN        0x00F0
#define  ixf6048_mTX_PTI_PATTERN        0x000E
#define  ixf6048_mTX_CLP_PATTERN        0x0001  
/***********************************************************************
 * Channel Register: T_ACELLCNT: Tx ATM Cell Counter
 * Page  - (1cc)13-12H
 **********************************************************************/

#define  ixf6048_T_ACELLCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x12)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_ACELLCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x13)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_TX_ATM_CELL_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_T_ACELLCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_T_ACELLCNT_CHAN_MSB_OFFSET(Chan))&0x00FF)<<16))

/***********************************************************************
 * Channel Register: T_ICELLCNT: Tx Idle Cell Counter
 * Page  - (1cc)15-14H
 **********************************************************************/

#define  ixf6048_T_ICELLCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x14)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_ICELLCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x15)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_GET_TX_IDLE_CELL_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_T_ICELLCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_T_ICELLCNT_CHAN_MSB_OFFSET(Chan))&0x00FF)<<16))

/***********************************************************************
 * Channel Register: T_ATMINT: Tx ATM Interupt Register    (COR)
 * Page  - (1cc)16H
 **********************************************************************/

#define  ixf6048_T_ATMINT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x16)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mT_ATM_INTERRUPT                0x0003

/* Note: The Interupt Masks have been defined in ixf6048_TxAtmAlarms_e */

/***********************************************************************
 * Channel Register: T_ATMINTEN: Tx ATM Interupt Enable
 * Page  - (1cc)17H
 **********************************************************************/

#define  ixf6048_T_ATMINTEN_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x17)+(Chan<<8))<<REG_SHIFT)

/* Note: The Interupt Enable Masks have been defined in ixf6048_TxAtmAlarms_e */
/*#####################################################################*
 *  POS RECEIVE CHANNEL REGISTERS
 *                                                                     *
 *#####################################################################*/

/***********************************************************************
 * Channel Register: R_PHCCNF: Rx POS HDLC Controller Configuration
 * Page  - (1cc)60H
 **********************************************************************/

#define  ixf6048_R_PHCCNF_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x60)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_R_PHCCNF_CHAN_DFLT		0x006F

#define	ixf6048_bRCV_CNT_WR_FR          0x1000

#define	ixf6048_bRCV_A_BYTE_CNT_EN		0x0800
#define	ixf6048_bRCV_FCS_ERR			0x0400
#define	ixf6048_bRCV_FCS_PASS			0x0200
#define	ixf6048_bRCV_TR_DESCR_EN		0x0100 

#define	ixf6048_bRCV_FIF_EN				0x0080
#define	ixf6048_bRCV_MAX_PLD_EN			0x0040 /* Packet Length Discard */
#define	ixf6048_bRCV_MIN_PLD_EN			0x0020 
#define	ixf6048_bRCV_A_C_CHK			0x0010 /* Address and Control Field */

#define	ixf6048_mRCV_FCS_CNF			0x000C /* Masks Bits 3:2 */
#define	ixf6048_RX_FCS_CRC_16_NO_CHECK_EN		0x0000 /* Clear Bits 3:2 */
#define	ixf6048_RX_FCS_CRC_32_NO_CHECK_EN		0x0004
#define	ixf6048_RX_FCS_CRC_16_CHECK_EN			0x0008
#define	ixf6048_RX_FCS_CRC_32_CHECK_EN			0x000C

#define	ixf6048_SET_RCV_FCS_CNF(Reg, FcsCnfg)   \
            (Reg = (Reg & ~ixf6048_mRCV_FCS_CNF) | (FcsCnfg << 2) )

#define	ixf6048_bRCV_A_C_PASS         0x0002 /* Adress and Control Field */
#define	ixf6048_bRCV_DESCR_EN         0x0001 /* Descrambling enable */

/***********************************************************************
 * Channel Register: R_MINPL: Rx Minimum Packet Length
 * Page  - (1cc)61H
 **********************************************************************/

#define  ixf6048_R_MINPL_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x61)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_R_MINPL_CHAN_DFLT		0x0002

#define  ixf6048_SET_MIN_PACKET_LENGTH(BaseAddress, Chan, PacketLength) \
         (WRITE_TO_HW(BaseAddress + ixf6048_R_MINPL_CHAN_OFFSET(Chan),  \
         (PacketLength & 0x00FF)))
            

/***********************************************************************
 * Channel Register: R_MAXPL: Rx Maximum Packet Length
 * Page  - (1cc)62H
 **********************************************************************/

#define  ixf6048_R_MAXPL_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x62)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_R_MAXPL_CHAN_DFLT		0x0600

#define  ixf6048_SET_MAX_PACKET_LENGTH(BaseAddress, Chan, PacketLength) \
         (WRITE_TO_HW(BaseAddress + ixf6048_R_MAXPL_CHAN_OFFSET(Chan),  \
         (PacketLength)))
/***********************************************************************
 * Channel Register: R_FRMCNT: Rx Frame Counter
 * Page  - (1cc)64-63H
 **********************************************************************/

#define  ixf6048_R_FRMCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x63)+(Chan<<8)) << REG_SHIFT)

#define  ixf6048_R_FRMCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x64)+(Chan<<8)) << REG_SHIFT)

#define ixf6048_R_FRMCNT_CNT_MASK_M 	0x07FF
#define ixf6048_R_FRMCNT_CNT_MASK_L		0xFFFF

#define  ixf6048_GET_RX_FRAME_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_R_FRMCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_FRMCNT_CHAN_MSB_OFFSET(Chan))&0x07FF)<<16))

/***********************************************************************
 * Channel Register: R_BYTECNT: Rx Byte Counter
 * Page  - (1cc)66-65H
 **********************************************************************/

#define  ixf6048_R_BYTECNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x65)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_BYTECNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x66)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_R_BYTECNT_CNT_MASK_M 	0x1FFF
#define ixf6048_R_BYTECNT_CNT_MASK_L	0xFFFF

#define  ixf6048_GET_RX_BYTE_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_R_BYTECNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_BYTECNT_CHAN_MSB_OFFSET(Chan))&0x1FFF)<<16))

/***********************************************************************
 * Channel Register: R_AFCNT: Rx Aborted Frame Counter
 * Page  - (1cc)68-67H
 **********************************************************************/

#define  ixf6048_R_AFCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x67)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_AFCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x68)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_R_AFCNT_CNT_MASK_M 	0x000F
#define ixf6048_R_AFCNT_CNT_MASK_L	0xFFFF

#define  ixf6048_GET_RX_ABORTED_FRAME_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_R_AFCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_AFCNT_CHAN_MSB_OFFSET(Chan))&0x000F)<<16))

/***********************************************************************
 * Channel Register: R_FCSECNT: Rx FCS Error Counter
 * Page  - (1cc)6A-69H
 **********************************************************************/

#define  ixf6048_R_FCSECNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x69)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_FCSECNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x6A)+(Chan<<8)) <<REG_SHIFT)

#define ixf6048_R_FCSECNT_CNT_MASK_M 	0x000F
#define ixf6048_R_FCSECNT_CNT_MASK_L	0xFFFF

#define  ixf6048_GET_RX_FCS_ERROR_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_R_FCSECNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_FCSECNT_CHAN_MSB_OFFSET(Chan))&0x000F)<<16))

/***********************************************************************
 * Channel Register: R_PFOCNT: Rx Packet FIFO Overflow Counter
 * Page  - (1cc)6BH
 **********************************************************************/

#define  ixf6048_R_PFOCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x6B)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_R_PFOCNT_CNT_MASK		0xFFFF

#define  ixf6048_GET_RX_FIFO_OVERFLOW_COUNTER(BaseAddress,Chan)    \
             (READ_FROM_HW(BaseAddress +  ixf6048_R_PFOCNT_CHAN_OFFSET(Chan)))

/***********************************************************************
 * Channel Register: R_MINPLECNT: Rx Minimum Packet Length error Counter
 * Page  - (1cc)6D-6CH
 **********************************************************************/

#define  ixf6048_R_MINPLECNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x6C)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_R_MINPLECNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x6D)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_R_MINPLECNT_CNT_MASK_M 	0x07FF
#define ixf6048_R_MINPLECNT_CNT_MASK_L	0xFFFF

#define  ixf6048_GET_RX_MIN_PACKET_LEN_ERR_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_R_MINPLECNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_R_MINPLECNT_CHAN_MSB_OFFSET(Chan))&0x07FF)<<16))

/***********************************************************************
 * Channel Register: R_MAXPLECNT: Rx Maximum Packet Length Error Counter
 * Page  - (1cc)6EH
 **********************************************************************/

#define  ixf6048_R_MAXPLECNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x6E)+(Chan<<8)) <<REG_SHIFT)

#define ixf6048_R_MAXPLECNT_CNT_MASK	0xFFFF

#define  ixf6048_GET_RX_MAX_PACKET_LEN_ERR_CNT(BaseAddress, Chan) \
            (READ_FROM_HW(BaseAddress + ixf6048_R_MAXPLECNT_CHAN_OFFSET(Chan)))
/***********************************************************************
 * Channel Register: R_POSINT: Rx POS Interupt Register       (COR)
 * Page  - (1cc)6FH
 **********************************************************************/

#define  ixf6048_R_POSINT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x6F)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_mRX_POS_INTERRUPT		0x7FFF

#define	ixf6048_R_FIFO_OVERFLOW			0x0001
#define	ixf6048_R_FCS_ERROR				0x0002
#define	ixf6048_R_ABORT_SEQ				0x0004
#define	ixf6048_R_ADDR_CONTROL			0x0008
#define	ixf6048_R_CTRL_ESC_REQ			0x0010
#define	ixf6048_R_MIN_PKT_LEN			0x0020
#define	ixf6048_R_MAX_PKT_LEN			0x0040
#define	ixf6048_R_SHORT_FRAME			0x0080
#define	ixf6048_R_FIFO_OF_CNT			0x0100
#define	ixf6048_R_FCS_ERR_CNT			0x0200
#define	ixf6048_R_ABORT_CNT				0x0400
#define	ixf6048_R_FRAME_CNT				0x0800
#define	ixf6048_R_BYTE_CNT				0x1000
#define	ixf6048_R_MIN_PKT_LEN_CNT		0x2000
#define	ixf6048_R_MAX_PKT_LEN_CNT		0x4000


/* Note: The Interupt Masks have been defined in ixf6048_RxPosAlarms_e */

/***********************************************************************
 * Channel Register: R_POSINTEN: Rx POS Interupt Enable
 * Page  - (1cc)70H
 **********************************************************************/

#define  ixf6048_R_POSINTEN_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x70)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_R_FIFO_OVERFLOW_EN			0x0001
#define	ixf6048_R_FCS_ERROR_EN				0x0002
#define	ixf6048_R_ABORT_SEQ_EN				0x0004
#define	ixf6048_R_ADDR_CONTROL_EN			0x0008
#define	ixf6048_R_CTRL_ESC_REQ_EN			0x0010
#define	ixf6048_R_MIN_PKT_LEN_EN			0x0020
#define	ixf6048_R_MAX_PKT_LEN_EN			0x0040
#define	ixf6048_R_SHORT_FRAME_EN			0x0080
#define	ixf6048_R_FIFO_OF_CNT_EN			0x0100
#define	ixf6048_R_FCS_ERR_CNT_EN			0x0200
#define	ixf6048_R_ABORT_CNT_EN				0x0400
#define	ixf6048_R_FRAME_CNT_EN				0x0800
#define	ixf6048_R_BYTE_CNT_EN				0x1000
#define	ixf6048_R_MIN_PKT_LEN_CNT_EN		0x2000
#define	ixf6048_R_MAX_PKT_LEN_CNT_EN		0x4000

/* Note: Enable Masks are defined in ixf6048_RxPosAlarms_e */

/*#####################################################################*
 *  POS TRANSMIT CHANNEL REGISTERS
 *                                                                     *
 *#####################################################################*/

/***********************************************************************
 * Channel Register: T_PHCCNF: Transmit POS HDLC Controller Configuration
 * Page  - (1cc)40H
 **********************************************************************/

#define  ixf6048_T_PHCCNF_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x40)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_T_PHCCNF_CHAN_DFLT		0x004F

#define  ixf6048_bXMT_TR_SCR_EN          0x0100

#define  ixf6048_bXMT_FIF_EN             0x0080
#define  ixf6048_bXMT_POS_READ_EN        0x0040
#define  ixf6048_bXMT_FCS_ERR_CNF        0x0020 /* Test -- FCS Insertion*/
#define  ixf6048_bXMT_FCS_ERR            0x0010 /* Test -- FCS Insertion*/

#define  ixf6048_mXMT_FCS_CNF            0X000C  /* Masks Bits 3:2 */
#define  ixf6048_TX_FCS_CONFIG_NONE_EN   0x0000  /* Clear Bits 3:2 */
#define  ixf6048_TX_FCS_CONFIG_CRC_16    0x0008  
#define  ixf6048_TX_FCS_CONFIG_CRC_32    0X000C

#define  ixf6048_bXMT_A_C_PASS           0x0002  /* Address and Control Fields */
#define  ixf6048_bXMT_SCR_EN             0x0001  /* Scrambler Enable */
                                             

/***********************************************************************
 * Channel Register: T_IPGCTRL: Tx Interpacket Gap Control (Tx Flow Control)
 * Page  - (1cc)41H
 **********************************************************************/

#define  ixf6048_T_IPGCTRL_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x41)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_T_IPGCTRL_CHAN_DFLT		0x0401

#define  ixf6048_bXMT_IPG_REL_EN         0x4000
#define  ixf6048_bXMT_IPG_REL_CFG        0x2000

/* Masks bits 12:10 -- The Dividing/Multiplying Factor*/
#define  ixf6048_mXMT_IPG_REL            0x1C00

#define  ixf6048_bXMT_IPG_ABS_EN         0x0200 
#define  ixf6048_bXMT_IPG_ABS_CNF        0x0100
/* Number of Flag Chars to Insert between Frames */
#define  ixf6048_mXMT_IPG_ABS            0x00FF 
/***********************************************************************
 * Channel Register: T_FRMCNT: Tx Frame Counter
 * Page  - (1cc)43-42H
 **********************************************************************/

#define  ixf6048_T_FRMCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x42)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_FRMCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x43)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_T_FRMCNT_CNT_MASK_M 	0x07FF
#define ixf6048_T_FRMCNT_CNT_MASK_L 	0xFFFF

#define  ixf6048_GET_TX_FRAME_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_T_FRMCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_T_FRMCNT_CHAN_MSB_OFFSET(Chan))&0x03FF)<<16))

/***********************************************************************
 * Channel Register: T_BYTECNT: Tx Byte Counter 
 * Page  - (1cc)45-44H
 **********************************************************************/

#define  ixf6048_T_BYTECNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x44)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_BYTECNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x45)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_T_BYTECNT_CNT_MASK_M 	0x1FFF
#define ixf6048_T_BYTECNT_CNT_MASK_L 	0xFFFF

#define  ixf6048_GET_TX_BYTE_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_T_BYTECNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_T_BYTECNT_CHAN_MSB_OFFSET(Chan))&0x1FFF)<<16))

/***********************************************************************
 * Channel Register: T_AFCNT: Tx Aborted Frame Counter 
 * Page  - (1cc)47-46H
 **********************************************************************/

#define  ixf6048_T_AFCNT_CHAN_LSB_OFFSET(Chan)  \
            (((1<<10)+(0x46)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_T_AFCNT_CHAN_MSB_OFFSET(Chan)  \
            (((1<<10)+(0x47)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_T_AFCNT_CNT_MASK_M 	0x000F
#define ixf6048_T_AFCNT_CNT_MASK_L 	0xFFFF

#define  ixf6048_GET_TX_ABORTED_FRAME_CNT(BaseAddress,Chan)                 \
         (READ_FROM_HW(BaseAddress + ixf6048_T_AFCNT_CHAN_LSB_OFFSET(Chan)) +   \
          ((READ_FROM_HW(BaseAddress + ixf6048_T_AFCNT_CHAN_MSB_OFFSET(Chan))&0x000F)<<16))

/***********************************************************************
 * Channel Register: T_PFUCNT: Tx Packet FIFO Underflow Counter
 * Page  - (1cc)48H
 **********************************************************************/

#define  ixf6048_T_PFUCNT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x48)+(Chan<<8))<<REG_SHIFT)

#define ixf6048_T_PFUCNT_CNT_MASK 	0xFFFF

#define  ixf6048_GET_TX_FIFO_UFLOW_COUNT(BaseAddress, Chan)  \
          (READ_FROM_HW(BaseAddress + ixf6048_T_PFUCNT_CHAN_OFFSET(Chan)))
/***********************************************************************
 * Channel Register: T_POSINT: Tx POS Interupt Register  (COR)
 * Page  - (1cc)49H
 **********************************************************************/

#define  ixf6048_T_POSINT_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x49)+(Chan<<8))<<REG_SHIFT)

#define  ixf6048_mTX_POS_INTERRUPT            0x007F  /* Masks bits 6:0 */

#define	ixf6048_T_FIFO_UNDERFLOW		0x0001
#define	ixf6048_T_FCS_ERROR				0x0002
#define	ixf6048_T_ABORT_SEQ				0x0004
#define	ixf6048_T_FIFO_UFLOW_CNT		0x0008
#define	ixf6048_T_ABORT_CNT				0x0010
#define	ixf6048_T_FRAME_CNT				0x0020
#define	ixf6048_T_BYTE_CNT				0x0040

/* Note: The Interupt Masks are defined By ixf6048_TxPosAlarms_e */

/***********************************************************************
 * Channel Register: T_POSINTEN: Tx POS Interupt Enable
 * Page  - (1cc)4AH
 **********************************************************************/

#define  ixf6048_T_POSINTEN_CHAN_OFFSET(Chan)  \
            (((1<<10)+(0x4A)+(Chan<<8))<<REG_SHIFT)

#define	ixf6048_T_FIFO_UNDERFLOW_EN			0x0001
#define	ixf6048_T_FCS_ERROR_EN				0x0002
#define	ixf6048_T_ABORT_SEQ_EN				0x0004
#define	ixf6048_T_FIFO_UFLOW_CNT_EN			0x0008
#define	ixf6048_T_ABORT_CNT_EN				0x0010
#define	ixf6048_T_FRAME_CNT_EN				0x0020
#define	ixf6048_T_BYTE_CNT_EN				0x0040

/* Note: Enable Masks Are Defined by ixf6048_TxPosAlarms_e */

#endif   /* BB6048B_H_ */

