/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: lagarwal $
 *   DATE:           $Date: 2002/11/15 17:51:12 $
 *   LAST MODIFIED:  $Modtime: 12/03/01 9:55a $
 *   REV#:           $Revision: 1.1.1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/os.h $
 *   WORKFILE:       $Workfile: os.h $
 *
 * --------------------------------------------------------------------------
 * Project: ALL
 *
 * Purpose: Provides the specfication for the eiffel alarm, status, & counters
 *          handler routines.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: os.h,v $
 *   Revision 1.1.1.1  2002/11/15 17:51:12  lagarwal
 *   rename dir
 *
 *   Revision 1.3  2002/09/09 21:49:19  cmah1
 *   no message
 *
 *   Revision 1.1  2002/05/22 00:05:42  machambe
 *   Initial update
 *
 * 
 * 11    12/03/01 9:57a Jaalliso
 * Added CR/LF to end of file to get rid of GNU C compiler warning
 * 
 * 10    12/01/01 6:34p Machambe
 * 
 * 9     11/06/01 10:41a Machambe
 * Added open, read, write, close, & pipe macros
 * 
 * 8     7/24/01 11:47a Machambe
 * 
 * 7     7/12/01 10:31a Machambe
 * 
 * 6     7/06/01 3:53p Machambe
 * 
 * 5     7/03/01 4:09p Machambe
 * 
 * 4     5/03/01 3:54p Machambe
 * 
 * --------------------------------------------------------------------------
 */ 

#ifndef __OS_H_
#define __OS_H_



#ifdef VXWORKS
/***************************************************************************
 *               Define OS abstraction layer for VxWorks OS                * 
****************************************************************************/
#define OS_CMD_ALLOC_MEM(size) (malloc(size))
#define OS_CMD_DEALLOC_MEM(ptr) (free(ptr))
#define OS_CMD_GET_TASK_ID() (taskIdSelf())
#define OS_CMD_ENABLE_PREEMPTION() (taskUnlock())
#define OS_CMD_DISABLE_PREEMPTION() (taskLock())
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) \
		(taskSpawn(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10))

#define OS_CMD_CREATE_SEMAPHORE(opt) (semMCreate(opt))
#define OS_CMD_GIVE_MUTEX_SEMAPHORE(id) (semGive(id))
#define OS_CMD_TAKE_MUTEX_SEMAPHORE(id, timeOut) (semTake(id, timeOut))

#define OS_CMD_ENABLE_INTERRUPTS(irq) (intUnlock(irq))
#define OS_CMD_DISABLE_INTERRUPTS() (intLock())
#define OS_CMD_VERIFY_TASK_ID(tId)  (taskIdVerify(tId))

/* defines */ 
#define MAX_MSG_Q_MSGS (10) 
#define MSG_Q_PRI 1
 
#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) (msgQCreate (maxMsg, maxMsgLen, priority))  
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) (msgQSend (id, msg, size, flag,priority))
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	(msgQReceive(id, msg, size, flag))

#define OS_CMD_TASK_DELETE(tId) (taskDelete(tId))

#define OS_CMD_CREATE_PIPE(name, num, size) (pipeDevCreate(name, num, size))
#define OS_CMD_OPEN(name, flag, mode)  (open(name, flag, mode))
#define OS_CMD_READ(fd, buffer, size)  (read(fd, buffer, size))
#define OS_CMD_WRITE(fd, buffer, size) (write(fd, buffer, size))
#define OS_CMD_CLOSE(fd)               (close(fd))

/* Global variable for Msg Q's */
extern MSG_Q_ID ixfMsgQId;



#else 
/***************************************************************************
 *     Define OS abstraction layer for when no embedded OS is defined      * 
****************************************************************************/

#define OS_CMD_ALLOC_MEM(size) (malloc(size))
#define OS_CMD_DEALLOC_MEM(ptr) (free(ptr))
#define OS_CMD_GET_TASK_ID()
#define OS_CMD_ENABLE_PREEMPTION() 
#define OS_CMD_DISABLE_PREEMPTION()
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) ()

#define OS_CMD_CREATE_SEMAPHORE()
#define OS_CMD_GIVE_MUTEX_SEMAPHORE()
#define OS_CMD_TAKE_MUTEX_SEMAPHORE()

#define OS_CMD_ENABLE_PREEMPTION() 
#define OS_CMD_DISABLE_PREEMPTION()

#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) ()
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) ()
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	()
#define OS_CMD_VERIFY_TASK_ID(tId)  ()
#define OS_CMD_TASK_DELETE(tId) ()

#define OS_CMD_CREATE_PIPE(name, num, size) ()
#define OS_CMD_OPEN(name, flag, mode)  ()
#define OS_CMD_READ(fd, buffer, size)  ()
#define OS_CMD_WRITE(fd, buffer, size) ()
#define OS_CMD_CLOSE(fd)               ()

#endif


#endif /* __OS_H_ */
