/*
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------------
 */

#if !defined(__ERROR_CODE_H__)
#define __ERROR_CODE_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Pre-processor symbol and macro definitions.
 * 
 * These definitions map to the various error conditions that occur in 
 * MAC diagnostics. 
 */

#define DONE				0x0

#define INIT_ERR_BASE			0x100
#define MAC_INIT_FAIL			INIT_ERR_BASE + 0x0
#define DEVICE_NOT_FOUND		INIT_ERR_BASE + 0x1
#define MICROENGINE_INIT_FAIL		INIT_ERR_BASE + 0x2
#define MICROENGINE_LOAD_FAIL		INIT_ERR_BASE + 0x3
#define MEM_WRITE_ERR			INIT_ERR_BASE + 0x4
#define ABORT				INIT_ERR_BASE + 0x5

// Rx stat register errors 
#define RXOCTETSTOTALOK_ERR		INIT_ERR_BASE + 0x6
#define RXOCTETSTOTALBAD_ERR		INIT_ERR_BASE + 0x7
#define RXUCPKTS_ERR			INIT_ERR_BASE + 0x8
#define RXMCPKTS_ERR			INIT_ERR_BASE + 0x9
#define RXBCPKTS_ERR			INIT_ERR_BASE + 0xA
#define RXPKTS64OCTETS_ERR		INIT_ERR_BASE + 0xB
#define RXPKTS65TO127OCTETS_ERR		INIT_ERR_BASE + 0xC
#define RXPKTS128TO255OCTETS_ERR	INIT_ERR_BASE + 0xD
#define RXPKTS256TO511OCTETS_ERR	INIT_ERR_BASE + 0xE
#define RXPKTS512TO1023OCTETS_ERR	INIT_ERR_BASE + 0xE
#define RXPKTS1024TO1518OCTETS_ERR	INIT_ERR_BASE + 0xF
#define RXPKTS1519TOMAXOCTETS_ERR	INIT_ERR_BASE + 0x10
#define RX_TAGGED_ERR			INIT_ERR_BASE + 0x11
#define RXDATAERROR_ERR			INIT_ERR_BASE + 0x12
#define ALIGNERRORS_ERR			INIT_ERR_BASE + 0x13
#define LONGERRORS_ERR			INIT_ERR_BASE + 0x14
#define JABBERERRORS_ERR		INIT_ERR_BASE + 0x15
#define RX_PAUSEMACCONTROL_ERR		INIT_ERR_BASE + 0x16
#define RX_BAD_PAUSE_FRAME_ERR		INIT_ERR_BASE + 0x17
#define SHORTERRORS_ERR			INIT_ERR_BASE + 0x18
#define RUNTERRORS_ERR			INIT_ERR_BASE + 0x19
#define VERYLONGERRORS_ERR		INIT_ERR_BASE + 0x1A
#define SYMBOL_ERRORS			INIT_ERR_BASE + 0x1B
#define SEQUENCE_ERR			INIT_ERR_BASE + 0x1C
#define RX_FCS_ERROR			INIT_ERR_BASE + 0x1D


//Tx stat register errors
#define TX_ERR_BASE			RX_FCS_ERROR+1
#define OCTETSTRANSMITTEDOK_ERR		TX_ERR_BASE + 0x1
#define OCTETSTRANSMITTEDBAD_ERR	TX_ERR_BASE + 0x2
#define TXUCPKTS_ERR			TX_ERR_BASE + 0x3
#define TXMCPKTS_ERR			TX_ERR_BASE + 0x4
#define TXBCPKTS_ERR			TX_ERR_BASE + 0x5
#define TXPKTS64OCTETS_ERR		TX_ERR_BASE + 0x6
#define TXPKTS65TO127OCTETS_ERR		TX_ERR_BASE + 0x7
#define TXPKTS128TO255OCTETS_ERR	TX_ERR_BASE + 0x8
#define TXPKTS256TO511OCTETS_ERR	TX_ERR_BASE + 0x9
#define TXPKTS512TO1023OCTETS_ERR	TX_ERR_BASE + 0xA
#define TXPKTS1024TO1518OCTETS_ERR	TX_ERR_BASE + 0xB
#define TXPKTS1519TOMAXOCTETS_ERR	TX_ERR_BASE + 0xC
#define TX_TAGGED_ERR			TX_ERR_BASE + 0xD
#define TX_PAUSEMACCONTROL_ERR		TX_ERR_BASE + 0xE
#define TXEXCESSIVELENGTHDROP_ERR	TX_ERR_BASE + 0xF
#define CRCERROR 			TX_ERR_BASE + 0x10
#define TXUNDERRUN_ERR			TX_ERR_BASE + 0x11
#define DEFFERED_TX_ERR			TX_ERR_BASE + 0x12
#define SINGLE_COLLISION_ERR		TX_ERR_BASE + 0x13
#define TXMUL_COLLISIONS_ERR		TX_ERR_BASE + 0x14
#define TXEXCV_COLLISIONS_ERR		TX_ERR_BASE + 0x15
#define TXLATE_COLLISIONS_ERR		TX_ERR_BASE + 0x16
#define TXEXCV_DEFERRALS_ERR		TX_ERR_BASE + 0x17
#define TXTOT_COLLISIONS_ERR		TX_ERR_BASE + 0x18


//miscellaneous register errors
#define MISC_ERR_BASE			TXEXCV_DEFERRALS_ERR + 1
#define PERPORT_ERR			MISC_ERR_BASE + 0x1
#define AUTOSCAN_ERR			MISC_ERR_BASE + 0x2
#define MDIO_ERR			MISC_ERR_BASE + 0x3
#define MDC_SPEED_ERR			MISC_ERR_BASE + 0x4
#define AUTONEG_ERR			MISC_ERR_BASE + 0x5
#define WATERMARK_ERR			MISC_ERR_BASE + 0x6
#define NO_RX_LED_ERR			MISC_ERR_BASE + 0x7
#define RX_LED_ERR			MISC_ERR_BASE + 0x8
#define AMBER_LED_ERR			MISC_ERR_BASE + 0x9
#define LED_ERR				MISC_ERR_BASE + 0xA
#define I2C_ERR				MISC_ERR_BASE + 0xB
#define SPI_BUS_4x8_ERR			MISC_ERR_BASE + 0xC
#define SPI_BUS_32_ERR			MISC_ERR_BASE + 0xD
#define SPI_INBAND_ERR			MISC_ERR_BASE + 0xE
#define SPI_TENB_ERR			MISC_ERR_BASE + 0xF
#define SPI_TERR_ERR			MISC_ERR_BASE + 0x10
#define SPI_TPA_ERR			MISC_ERR_BASE + 0x11
#define UNI_FIL_ERR			MISC_ERR_BASE + 0x12
#define MULTI_FIL_ERR			MISC_ERR_BASE + 0x13
#define BRD_FIL_ERR			MISC_ERR_BASE + 0x14
#define VLAN_FIL_ERR			MISC_ERR_BASE + 0x15
#define PAUSE_FIL_ERR			MISC_ERR_BASE + 0x16
#define CRC_STRIP_ERR			MISC_ERR_BASE + 0x17
#define CRC_ADD_ERR			MISC_ERR_BASE + 0x18
#define POWER_DN_ERR			MISC_ERR_BASE + 0x19
#define TXFIFO_PAD_ERR			MISC_ERR_BASE + 0x1A
#define RXFIFO_PAD_ERR			MISC_ERR_BASE + 0x1B
#define FRAME_SZ_ERR			MISC_ERR_BASE + 0x1C
#define RXFIFO_RESET_ERR		MISC_ERR_BASE + 0x1D
#define TXFIFO_RESET_ERR		MISC_ERR_BASE + 0x1E
#define RXTX_PATH_ERR			MISC_ERR_BASE + 0x1F
#define INTNL_LPBK_ERR			MISC_ERR_BASE + 0x20
#define EXTNL_LPBK_ERR			MISC_ERR_BASE + 0x21
#define LINERATE_ERR			MISC_ERR_BASE + 0x22
#define PLL_LOCK_ERR			MISC_ERR_BASE + 0x23

/**
 * Type definitions.
 */

/**
 * Prototypes for interface functions.
 */

/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__ERROR_CODE_H__) */



