/*
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------------
 */

/**
 * System defined include files required.0
 */

/**
 * User defined include files required.
 */
#include "mac_init.h"
#include "diag.h"
#include "mac_diag.h"
#include "packet_pool.h"
#include "error_map.h"
#include "prototypes.h"
#include "register_map.h"
#include "diagstruct.h"


extern UINT32 media_seat;
/**
 * These bit masks are used to set a bit in the mac_test_flag
 * The corresponding diagnostic routine against the set bit is invoked.
 */
#define SYSLB_TEST_MASK         0x8000  	/* bit 15 */
#define LPBK_TEST_MASK          0x10000 	/* bit 16 */
#define REG_RD					0x100000    /* bit 20 */
#define REG_WR					0x200000    /* bit 21 */
#define RD_ALL_REGS				0x400000	/* bit 22 */
#define SYSLB2_TEST_MASK        0x1000000  	/* bit 24 */
#define SYSLB3_TEST_MASK        0x2000000  	/* bit 25 */
#define PHY_READ_ALL			0xF		
#define PHY_REG_WRITE			0xD
#define INTERNAL_LPBK			0xE
#define LINE_TEST_MASK			0xC

/* Local function prototypes */

UINT32 validate_params(void);
void get_option_from_user(void);
short check_autoneg_status();
short check_autoneg_status_mixed_mode();
UINT32 do_mac_test(void);


/**
 * Extern function prototypes.
 */
extern void gets(char *str, int len);
extern void read_all_regs();
extern void gbe_mac_phy_write(UINT32, UINT32,UINT32);
extern void gbe_mac_phy_read_all(void);
extern UINT32 pl_SysLpbk_test(void);
extern int getCmdLine(char* cmdLine);
extern UINT32 line_side_lpbk(void);
extern UINT32 gbe_mac_phy_read(UINT32 arg_PhyAddress, UINT32 arg_RegAddress);
extern UINT32 config_phy();
extern void mac_reset_config(void);
extern void mac_config();
extern void config_phy_for_line_lpbk();
extern UINT32 Sys_Loopback2_PL_Master(void);
extern UINT32 Sys_Loopback3_PL_Master(void);
extern void spRestore();
extern void spConfig();
extern void Enable_CPSR_Int(void);
extern void eprintf(char *f, ...);
extern UINT32 lpbk_test(void);
extern void reg_write( volatile UINT32 offset, UINT32 val);
extern UINT32 reg_read( volatile UINT32 offset);
extern void Disable_CPSR_Int(void);
extern int ctoi(char c);
extern unsigned int htoi(char *s);
extern void phy_write();
extern void get_stats();
extern void check_error_display_rxstats(void);
extern void check_error_display_txstats(void);

/**
 * Static function prototypes.
 */
static void show_options_help();




char *aTestHelp[] =
{
	"t mac <option>",
	"where options are --",
	" ",
	"    m: Media Loopback test",
	"    syntax for media loopback test is -",
	"    t mac m <loopback mode> <mode> [speed] [duplex] [port] [loop count]",
	" ",
	"    y: System Loopback test",
	"    syntax for system loopback test is -",
	"    t mac y <loopback mode> <mode> [speed] [duplex] [port] [loop count]",
	" ",
	"    d: Lineside Loopback test",
	"    syntax for system loopback test is -",
	"    t mac d <loopback mode> <mode> [speed] [duplex] [port] [loop count]",
	" ",
	"    where",
	"    <loopback mode>: i -> internal loopback",
	"                     e -> external loopback",
	" ",
	"    <mode>: f -> fiber mode",
	"            c -> copper mode",
	"            m0 -> mixed mode with Port 0,1 Copper and Port 2,3 Fibre",
	"            m2 -> mixed mode with Port 0,1 Fibre and Port 2,3 Copper",
	" ",
	"    [speed]:",
	"           10   -> 10 Mbps (copper mode only)",
	"           100  -> 100 Mbps (copper mode only)",
	"           1000 -> 1000 Mbps (copper, fiber and mixed mode)",
	" ",
	"    [duplex]: h -> half duplex (copper mode only)",
	"              f -> full diplex (copper, fiber and mixed mode)",
	" ",
	"    [port]: ",
	"          0/1/2/3 -> Test is executed on one of these ports",
	"          4       -> Test is executed on all ports",
	" ",
	"    [loop count] -> Number of loops for the loopback test.",
    " ",
    "    NOTE: For mixed mode",
    "          <loopback mode> can only be 'e' and [port] must be 4",
	" ",
    "r: read a MAC register",
    "w: write to a MAC register",
    "g: read all MAC registers",
    "a: read all PHY registers",
    "p: write PHY registers",
    
};


#define aTestHelpSz (sizeof(aTestHelp) /sizeof(aTestHelp[0]))  

/** Global variables **/
UINT32 total_args=0,loop=1,ports=4;
UINT8 channel='0',test_opt='?' ;
UINT32 speed=1000;  
UINT8 dplx='f',loopback_mode='i';
UINT32 rxstat[4][30],txstat[4][30];
UINT32 mac_test_flag = 0;

/**
 * Function definitions.
 */

    void mac_test(void)
    {
        register PDiagCommon acL = (PDiagCommon) ACADDRESS;    
		test_opt = acL->argv[2][0];	                
   
        if(acL->HostType == MASTER)             // if master NPU
        { 
			get_option_from_user();

            if (do_mac_test() != DONE)
            {
                eprintf("Aborting Execution !\n");
				return;
            }

			return;
        } 

    }

  
    void get_option_from_user(void)
    {

        short bad_option = 1;
		switch (test_opt) 
        {

			case 'a': mac_test_flag |= PHY_READ_ALL; 	
                      bad_option = 0;
                      break;

			case 'p': mac_test_flag |= PHY_REG_WRITE; 	
                      bad_option = 0;
                      break;

			case 'g': mac_test_flag |= RD_ALL_REGS; 	
                      bad_option = 0;
                      break;
		  		             
			case 'r': mac_test_flag |= REG_RD; 	
                      bad_option = 0;
                      break;

			case 'w': mac_test_flag |= REG_WR; 	
                      bad_option = 0;
                      break;

            case 'm': //eprintf("setting mac test flag\n");
					  mac_test_flag |= LPBK_TEST_MASK;
                      bad_option = 0;
                      break;

            case 'y': //eprintf("setting mac test flag\n");
					  mac_test_flag |= SYSLB_TEST_MASK;
                      bad_option = 0;
                      break;

			case '2': mac_test_flag |= SYSLB2_TEST_MASK;
                      bad_option = 0;
                      break;

			case '3': mac_test_flag |= SYSLB3_TEST_MASK;
                      bad_option = 0;
                      break;

            case 'd': 
					  mac_test_flag |= LINE_TEST_MASK;
                      bad_option = 0;
                      break;

			default:  
    				  show_options_help();
					  break;
        }

		return ;
    }


    static void show_options_help()
    {
        short i=0;

        eprintf("Mac test options\n");
        eprintf("----------------\n");
       
        for (i=0;i<aTestHelpSz;i++)
        {
            eprintf("%s\n",aTestHelp[i]);
        }
    }

	
	UINT32 do_mac_test(void) 
    {
		char saddr[5],sval[10];
		short i=0;
		unsigned int address = 0,value=0;	
	
        switch (mac_test_flag)
        {

			case SYSLB_TEST_MASK:
	           /*
		        * invoke system loopback tests
				*/
                 pl_SysLpbk_test(); 
                 break;

			case SYSLB2_TEST_MASK:
				/*
		        * invoke system loopback test # 2
				*/
				Sys_Loopback2_PL_Master(); 
                break;

			case SYSLB3_TEST_MASK:
				/*
		        * invoke system loopback test # 3
				*/
				Sys_Loopback3_PL_Master(); 
                break;

			case LPBK_TEST_MASK:
				/*
				* invoke internal and external media loopback tests
				*/

				lpbk_test(); 
                break;

		    case INTERNAL_LPBK:
				lpbk_test();
				break;

		    case LINE_TEST_MASK:
				line_side_lpbk();
				break;

		    case PHY_REG_WRITE:
				phy_write();
				break;

			case PHY_READ_ALL:
				spConfig();
				gbe_mac_phy_read_all();
		        spRestore();
				break;

			case RD_ALL_REGS:
				read_all_regs();
				break;

			case REG_RD:
				spConfig();
				/*** read vallejo registers ***/
  				eprintf("\nEnter Vallejo Reg. offset :");
				for (i=0;i<=4;i++)
					saddr[i] = '\0';
				i=0;
				gets(saddr, 4);
	
				address=htoi(saddr);
				eprintf("\nReading offset 0x%x, value = 0x%x\n",address,reg_read(address));
				strcpy(saddr,"00000");
				spRestore();
				break;
			case REG_WR:
				spConfig();
				/*** read vallejo registers ***/
		        eprintf("\nEnter Vallejo Reg. offset :");
				for (i=0;i<=4;i++)
					saddr[i] = '\0';
				i=0;
				gets(saddr, 4);
				eprintf("\nEnter Value :");
				gets(sval, 9);
				address = htoi(saddr);
				value = htoi(sval);

				reg_write(address,value);
				eprintf("writing offset 0x%x, value = 0x%x\n",address,value);

				strcpy(saddr,"00000");
				strcpy(sval,"000000000");
				spRestore();	
				break;
		} /* end of switch */
	
		/* reset the mac test flag */
        mac_test_flag = 0;
	    return DONE;
    }

UINT32 validate_params()
{
	short fault=0;
	
	if ((loopback_mode != 'i') && (loopback_mode != 'e'))
	{
		eprintf("setting loopback mode to internal..\n");
		loopback_mode = 'i';
	}
	

	switch(channel)
	{
		case 'f':
					if (dplx == 'h')
					{
						eprintf("Half duplex mode not supported on fiber channels.\n");
						fault = 1;
					}
					else if (dplx != 'f')
					{
						eprintf("Invalid duplex mode. \n");
						fault = 1;
					}
					if (speed != 1000)
					{
						eprintf("Invalid speed parameter for fiber channels. It should be 1000.\n");
						fault = 1;
					}
					break;

		case 'c':
					if ((dplx != 'f') && (dplx != 'h'))
					{
						eprintf("Invalid duplex selected for copper channels..\n");
						fault = 1;
					}

					if ((speed != 10) && (speed != 100) && (speed != 1000))
					{
						eprintf("Invalid speed selected for copper channels\n");
						fault = 1;
					}
					if ((speed == 1000) && (dplx == 'h'))
					{
						eprintf("Half duplex operation at gigabit speeds is not supported. \n");
						fault = 1;
					}
					break;

		case 'm':	if (dplx != 'f')
					{
						eprintf("Full duplex only in mixed mode\n");
						fault = 1;
					}

					if (speed != 1000)
					{
						eprintf("Speed has to be 1000 for mixed mode\n");
						fault = 1;
					}

					if (loopback_mode != 'e')
					{
						eprintf("Loopback mode has to be set to external for mixed mode\n");
						fault = 1;
					}

					if (ports != 4)
					{
						eprintf("Four ports are required for mixed mode\n");
						fault = 1;
					}
					break;

		default:
					eprintf("Invalid Channel Mode\n");
					show_options_help();
					fault = 1;
					break;
	}

	if (fault)
		return INVALID_PARAMETERS;

	if ((ports < 0) || (ports > 4))
	{
		eprintf("Invalid port number\n");
		eprintf("Test will be executed on all 4 ports \n");
		ports = 4;
	}

	if (loop == 0)
	{
		eprintf("Loopback test executed for infinite packets..\n");
		eprintf("Please press a key to stop test ..\n");
	}
	return 0;
}
					
short check_autoneg_status()
{
	short s_PortCount = 0, ctr = 0, done = 0;
	UINT32 regval = 0, link=0,receiver_status=0, autoneg_complete=0;
	UINT32 auto_neg_complete_status = ERROR;
	
	eprintf("Waiting for auto-negotiation to complete ....\n");
	for(s_PortCount=0; s_PortCount<4; s_PortCount++)
    {		
		if ((s_PortCount == ports) || (ports == 4))
		{
			do
			{	
				switch(speed)
				{
				case 1000:
						regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x11);
						link = (link >> 12) & 0x1;
						receiver_status = (regval >> 12) & 0x3;
						//eprintf("receiver status = 0x%x\n",receiver_status);
						if (receiver_status == 0x3)
							done = 1;
						break;
				case 100:
						//regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x01);
						link = (link >>2) & 1;
						done = 1;
						break;
				case 10:
						//regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x16);
						done = gbe_mac_phy_read(s_PortCount,0x01);
						link = (link >> 6) & 1;
						done = (done >> 2) & 1;
						break;
				}
				//eprintf("Regval for port %d  = 0x%08X\n",s_PortCount,regval);	
				//receiver_status = (regval >> 12) & 0x3;

				autoneg_complete = gbe_mac_phy_read(s_PortCount,0x1);
				if ((((autoneg_complete >> 5) & 1) == 0x1) && (link == 1) && done)
				{
					auto_neg_complete_status = OK;
					break;
				}
				else
				{	
					ctr ++;
					auto_neg_complete_status = ERROR;
				}
			}while (ctr < 200);

			if (auto_neg_complete_status == ERROR)
			{
				if (loopback_mode == 'e')
				{
					eprintf("Auto-negotiation failed for port %d , link = 0x%08X, auto = 0x%08X\n",s_PortCount,link, autoneg_complete);
				}
			}
		}
	}

	return auto_neg_complete_status;
}


UINT32 cu_init(UINT32 set_int_lpbk)
{
	int port;
	
	if(!set_int_lpbk)
	{
		/** configure PHY registers **/
		if (config_phy() == ABORT)
			return ABORT;
	}
	else
	{
		config_phy_for_line_lpbk();
	}

   /* apply reset to MAC device */
    mac_reset_config();

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(INTERFACE_MODE,0xF);		/** enable copper mode **/
	switch(dplx)
	{
		case 'h' :      
				reg_write(DESIRED_DUPLEX,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x0);
				break;
		case 'f' :
				reg_write(DESIRED_DUPLEX,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x1);
				break;
		default : 
				eprintf("Illegal duplex optinon.\n");
				eprintf("The opertation will be full duplex..\n");
				break;
	}			
	
	reg_write(SERDES_CLKMODE_REG,0xF);
	reg_write(SPI3_TX_GLOBAL_CONFIG,0x20aa0f); 
	reg_write(SPI3_RX_CONFIG, 0xAA0F80);

	reg_write(DIV_CONFIG_WORD,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x114D); 

	/* Pause control not to be disabled */
	/*
	reg_write(FC_MODE,0);  
	reg_write(FC_MODE+INDEX_PORT1,0);  
	reg_write(FC_MODE+INDEX_PORT2,0);  
	reg_write(FC_MODE+INDEX_PORT3,0); 
	*/
	reg_write(0x5B3,0xF0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	if (speed == 100)
	{
	    reg_write(MAC_IF_MODE,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x5);  
	}

	if (speed == 10)
	{
	    reg_write(MAC_IF_MODE,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x4);  
	}
	
	if (speed == 1000)
	{
	    reg_write(MAC_IF_MODE,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x6);  
	}
	reg_write(SERDES_CLKMODE_REG,0xF);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(TXFIFO_PAD_ENB,0xAAAA);  
	reg_write(SERDES_TX_RX_POW_DOWN,0x0); 

	mac_config();
	reg_write(GBIC_CTRL_REG,0x1C00);    

	reg_write(LED_MODE, 0x3);

	// Activate units
	reg_write(0x505, 0x0);		// MAC Soft reset 
	reg_write(0x506, 0x0);		// MDIO Reset
	reg_write(0x620, 0x0);		// TX FIFO Reset
	reg_write(0x59E, 0x0);		// Rx FIFO Reset

	//mac_disable_reset();

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);
	reg_write(PORT_ENABLE,0xF);

	// fix for CRC error @ 1G
	for(port = 0; port < 4; port++)
    {
        gbe_mac_phy_write(port, 22, 0x2018);
		gbe_mac_phy_write(port, 24, 0x3248);
    }
	return DONE;
}
