/*
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------------
 */

/**
 * User defined include files required.
 */
#include "diag.h"
#include "mac_init.h"
#include "error_map.h" 
#include "register_map.h"
#include "reg_api.h"
#include "common.h"
#include "diagstruct.h"
/**
 * Pre-processor symbols and macros used in this file.
 */

#define PLL_LOCK_TIMER		50		/* Check PLL lock register 50 times 
                                   	   with a delay of 1ms */

#define PHY_REG_ADDRESS          0x1F
#define PHY_DEVICE_ADDRESS       0x03
#define PHY_DEVICE_SHIFT         0x08
#define PHY_READ                 0x120000
#define PHY_WRITE                0x110000
#define DEFAULT_SET              0x01

#define NUM_PHY_REGS			0x1F


/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

    static configData aMacResetConfigTbl[ ] = 
    {
        {MAC_SOFT_RESET,        0x0000000F, 1},
        {TXFIFO_PORT_RESET,     0x0000000F, 1},         
        {RXFIFO_PORT_RESET,     0x0000000F, 1},
        {SPI3_TX_GLOBAL_CONFIG, 0x00E00000, 1}         
        //{MDIO_RESET,            0x00000001, 1}
    };

#define MacResetConfigTblSz (sizeof(aMacResetConfigTbl) /sizeof(aMacResetConfigTbl[0]))

    static configData aMacDisableResetConfigTbl[] =
    {
        {MAC_SOFT_RESET,        0x00000000, 1},
        {TXFIFO_PORT_RESET,     0x00000000, 1},
        {RXFIFO_PORT_RESET,     0x00000000, 1},
        {MDIO_RESET,            0x00000000, 1},
        {SERDES_CLKMODE_REG,    0x0000000F, 1},
        {PORT_ENABLE,           0x0000000F, 1}
    };

#define MacDisableResetConfigTblSz (sizeof(aMacDisableResetConfigTbl) /sizeof(aMacDisableResetConfigTbl[0]))


/**
 * Extern function prototypes.
 */

extern UINT32 reg_read( volatile UINT32 offset);
extern void reg_write( volatile UINT32 offset, UINT32 val);
extern void hal_delay_us(unsigned int usecs);
extern void eprintf(char *f, ...);
extern void gets(char *str, int len);
extern void Disable_CPSR_Int(void);
extern void Enable_CPSR_Int(void);
extern short check_autoneg_status();
extern short check_autoneg_status_mixed_mode();
extern UINT32 cu_init(UINT32 set_int_lpbk);
extern int getCmdLine(char* cmdLine);

/** local functions **/
void mac_reset_config(void);
void mac_disable_reset(void);
UINT32 config_phy(); // phy configuraton register 0 (control register) is configured 
void gbe_mac_phy_write(UINT32 arg_PhyAddress,UINT32 arg_RegAddress,UINT32 arg_WriteValue);
void gbe_mac_phy_read_all();
UINT32 gbe_mac_phy_read(UINT32 arg_PhyAddress, UINT32 arg_RegAddress);
void fiber_init(void);
void mixed_mode_init(UINT32 set_int_lpbk);
void mixed_mode_init1(UINT32 set_int_lpbk);
void spRestore();
void spConfig();
unsigned int htoi(char *s);
int ctoi(char c);
void get_stats();
void check_error_display_rxstats(void);
void check_error_display_txstats(void);
void gets(char *str, int len);

/* Globals */
extern UINT32 loop,speed;  
extern char channel,test_opt,dplx, loopback_mode;
extern UINT32 media_seat;
extern UINT32 silicon_rev;

UINT32 orig_sp_val[8];
UINT32 cport=0;
UINT32 mdio_enabled = 0;
UINT32 sf_mdio_enabled = 0;
extern UINT32 rxstat[4][30],txstat[4][30];
extern UINT32 ports;
/**
 * Function definitions.
 */

    UINT32 init_mac_dev (UINT32 set_int_lpbk) 
    {
 
     	UINT32 pll_lock_status =0;
        UINT32 pll1_tuning_status,pll2_tuning_status,pll3_tuning_status =0;
		UINT32 total_delay=0, status=0;
	
		/* configure slow port in mode 3 */
        spConfig();

        pll_lock_status=reg_read(SERDES_PLL_LOCK_STATUS);
		pll1_tuning_status= reg_read(PLL1_TUNE_REG); 
		pll2_tuning_status= reg_read(PLL2_TUNE_REG); 
		pll3_tuning_status= reg_read(PLL3_TUNE_REG); 
        
	eprintf("Waiting for PLL Lock to set...\n");
	
	do
        {
            hal_delay_us(1000);		/* delay of 1 ms */

            pll_lock_status=reg_read(SERDES_PLL_LOCK_STATUS);
			pll1_tuning_status= reg_read(PLL1_TUNE_REG); 
			pll2_tuning_status= reg_read(PLL2_TUNE_REG); 
			pll3_tuning_status= reg_read(PLL3_TUNE_REG); 
			if ( (pll1_tuning_status != 0x534) || (pll2_tuning_status != 0x534) || (pll3_tuning_status != 0x5be) )
            {
				eprintf("pll1_tuning_status=0x%08X\n", pll1_tuning_status);
				eprintf("pll2_tuning_status=0x%08X\n", pll2_tuning_status);
				eprintf("pll3_tuning_status=0x%08X\n", pll3_tuning_status);
				eprintf("Pll Tuning register lock failed !\n");
				return ABORT;
			}

//	    MSG("PLL lock 0x%x\n",pll_lock_status,0,0);
//	    MSG("PLL lock 0x%x\n",pll1_tuning_status,0,0);
//	    MSG("PLL lock 0x%x\n",pll2_tuning_status,0,0);
//	    MSG("PLL lock 0x%x\n",pll3_tuning_status,0,0);

            total_delay ++;
            if (total_delay > PLL_LOCK_TIMER)
            {
                eprintf("PLL Lock Timer : Timed out\n");
				eprintf("pll1_tuning_status=0x%08X\n", pll1_tuning_status);
				eprintf("pll2_tuning_status=0x%08X\n", pll2_tuning_status);
				eprintf("pll3_tuning_status=0x%08X\n", pll3_tuning_status);
				eprintf("pll_lock_status=0x%08X\n", pll_lock_status);
				return PLL_LOCK_ERR;
            }
        }
		while(pll_lock_status != 0x3);


		switch (channel)
		{
			case 'f' :
			case 'F' : 
		    			eprintf("\nConfiguring Fiber Channels ..\n");	
		    			fiber_init();	/** configure the device in fiber mode **/
						break;
	
			case 'c' :
			case 'C' : 
				        eprintf("\nConfiguring Copper Channels ..\n");	       	
		    			status=cu_init(set_int_lpbk);
						break;

	         case 'm' :
             case 'M' :
                     	if (cport == 0)
						{
                    		eprintf("\nConfiguring channel 0,1 in Copper and 2, 3 in Fiber ..\n");
						}
						else if (cport == 2)
						{
                    		eprintf("\nConfiguring channel 0,1 in Fibre and 2, 3 in Copper ..\n");
						}
						mixed_mode_init(set_int_lpbk);
						break;

            case 'n' :
            case 'N' :
						eprintf("\nConfiguring chaneel 2,3 in Copper and 0, 1 in Fiber ..\n");
						mixed_mode_init1(set_int_lpbk);
                    	break;

		    	default :	//eprintf("\nIllegal Channel option.. configuring fiber as default. \n");	
		    			fiber_init();	/** configure the device in fiber mode **/
					break;
		}

		if (status == ABORT)
			return ABORT;
	
        /* apply delay of 1 second for clock stabilization */
        hal_delay_us(1000000);    

		/* configure station addresses */
		
		reg_write((STATION_ADDR_LOW), 0xA0A0A0A0);
		reg_write((STATION_ADDR_HIGH),0x00A0);
	
		reg_write((STATION_ADDR_LOW + 0x80), 0xB0B0B0B0);
		reg_write((STATION_ADDR_HIGH + 0x80),0x00B0);
	
		reg_write((STATION_ADDR_LOW + 0x100), 0xC0C0C0C0);
		reg_write((STATION_ADDR_HIGH + 0x100),0x00C0);
		
		reg_write((STATION_ADDR_LOW + 0x180), 0xD0D0D0D0);
		reg_write((STATION_ADDR_HIGH + 0x180),0x00D0);
		
		/* 48-bit globally assigned multicast pause frame destination address */
		reg_write((MUL_PORT_ADDR_LOW), 0xA0A0A0A0);
		reg_write((MUL_PORT_ADDR_HIGH), 0xA1A0);
	
		reg_write((MUL_PORT_ADDR_LOW + 0x80), 0xB0B0B0B0);
		reg_write((MUL_PORT_ADDR_HIGH + 0x80), 0xA1B0);
	
		reg_write((MUL_PORT_ADDR_LOW + 0x100), 0xC0C0C0C0);
		reg_write((MUL_PORT_ADDR_HIGH + 0x100), 0xA1C0);
	
		reg_write((MUL_PORT_ADDR_LOW + 0x180), 0xD0D0D0D0);
		reg_write((MUL_PORT_ADDR_HIGH + 0x180), 0xA1D0);

		spRestore();	
        return DONE;
    }   

    void mac_disable_reset(void)
    {
        short i=0;

        for (i=0;i<MacDisableResetConfigTblSz; i++)
        {
            reg_write((aMacDisableResetConfigTbl[i].reg_addr), aMacDisableResetConfigTbl[i].config_val);
        }
    }

    void mac_reset_config(void)
    {
        short i=0;

        for (i=0;i<MacResetConfigTblSz; i++)
        {
            reg_write(aMacResetConfigTbl[i].reg_addr, aMacResetConfigTbl[i].config_val);
        }
    }

void config_phy_for_line_lpbk()
{
	short port_ctr = 0;

	for (port_ctr=0;port_ctr<4; port_ctr++)
	{
			//gbe_mac_phy_write(port_ctr, 0x1B, 0x105);	/* Set LED Control in PHY */
            gbe_mac_phy_write(port_ctr, 0x19, 0xB400);	/* Enable all interrupts */
	        gbe_mac_phy_write(port_ctr, 0x17, 0x1300);	// Enable RGMII mode
			if (speed == 1000)
				gbe_mac_phy_write(port_ctr, 0x9, 0x300);		/* set gigabit operation */

			gbe_mac_phy_write(port_ctr, 0, 0x1240);		// set auto-neg
	}
}


UINT32 config_phy()
{
	int s_PortCount =0;
	UINT32 status=0;
 
	for(s_PortCount=0; s_PortCount<4;s_PortCount++)
    {		
			//gbe_mac_phy_write(s_PortCount, 0x1B, 0x105);	/* Set LED Control in PHY */
            gbe_mac_phy_write(s_PortCount, 0x19, 0xB400);	/* Enable all interrupts */
	        gbe_mac_phy_write(s_PortCount, 0x17, 0x1300);	// Enable RGMII mode

	    switch (speed)
	    {
		case 1000 :	//eprintf("configuring 1000 \n");
				switch(dplx)
				{
					case 'h':
						gbe_mac_phy_write(s_PortCount, 0x4, 0x0);
						gbe_mac_phy_write(s_PortCount, 0x9, 0x100);		/* set gigabit operation */
						gbe_mac_phy_write(s_PortCount, 0, 0x1240); 
						gbe_mac_phy_write(s_PortCount, 22, 0x2018); 
						gbe_mac_phy_write(s_PortCount, 24, 0x3248); 
						//gbe_mac_phy_write(s_PortCount, 0, 0x1040);
						break;
					case 'f':
						gbe_mac_phy_write(s_PortCount, 0x4, 0x0);
						gbe_mac_phy_write(s_PortCount, 0x9, 0x300);		/* set gigabit operation */
						gbe_mac_phy_write(s_PortCount, 0, 0x1240); 
						gbe_mac_phy_write(s_PortCount, 22, 0x18); 
						gbe_mac_phy_write(s_PortCount, 24, 0x3248); 
						break;
				}
						     		       		
	 			//gig_config();
				break;
		case 100:
				//eprintf("configuring 100 \n");		
			        switch(dplx)
	        		{
		    			case 'h' :  //gbe_mac_phy_write(s_PortCount, 0, 0x1240);    
									gbe_mac_phy_write(s_PortCount, 0x4, 0x80);    
									gbe_mac_phy_write(s_PortCount, 0x9, 0x400); 
									gbe_mac_phy_write(s_PortCount, 0, 0x1240); 
									//gbe_mac_phy_write(s_PortCount, 0, 0x2000);
				    				break;
		    			case 'f' :  //gbe_mac_phy_write(s_PortCount, 0, 0x1240);
									gbe_mac_phy_write(s_PortCount, 0x4, 0x180);    
									gbe_mac_phy_write(s_PortCount, 0x9, 0x400); 
									gbe_mac_phy_write(s_PortCount, 0, 0x1240);
									//gbe_mac_phy_write(s_PortCount, 0, 0x2100);
				    				break;
	        		}		
				break;

		case 10:
				//eprintf("configuring 10 \n");						
			        switch(dplx)
	        		{
		    			case 'h' : 		//gbe_mac_phy_write(s_PortCount, 0, 0x1240);
										gbe_mac_phy_write(s_PortCount, 0x4, 0x60);    
										gbe_mac_phy_write(s_PortCount, 0x9, 0x400); 
										gbe_mac_phy_write(s_PortCount, 0, 0x1240);
										//gbe_mac_phy_write(s_PortCount, 0, 0x0);
										break;
		    			case 'f' :		//gbe_mac_phy_write(s_PortCount, 0, 0x1240);
										gbe_mac_phy_write(s_PortCount, 0x4, 0x40);    
										gbe_mac_phy_write(s_PortCount, 0x9, 0x400);   
										gbe_mac_phy_write(s_PortCount, 0, 0x1240);
										//gbe_mac_phy_write(s_PortCount, 0, 0x0100);
				    					break;
	        		}		
				break;

		default : 	eprintf("Bad speed option ... configuring 100 Mbps speed.\n");
				//eprintf("configuring 100 \n");		
			        switch(dplx)
	        		{
		    			case 'h' :      gbe_mac_phy_write(s_PortCount, 0, 0x2000);
				    			break;
		    			case 'f' :      gbe_mac_phy_write(s_PortCount, 0, 0x2100);
				    			break;
	        		}		
				break;
	    	}
        }

	if (channel == 'm')
	{
		check_autoneg_status_mixed_mode();
	}
	else
	{
		status = check_autoneg_status();
	}
	if (status == ERROR)
	{
		eprintf("Auto-negotiation FAILED \n");
		return ABORT;
	}
	return DONE;
}

void gbe_mac_phy_write( UINT32 arg_PhyAddress,
                               UINT32 arg_RegAddress,
                               UINT32 arg_WriteValue
                              )
{
    UINT32 s_ReadValue;
    UINT32 s_WriteValue = 0;
    UINT32 s_RegAddress, s_PhyAddress;
    UINT32 mdio_done = 0;
    UINT32 ctr=0;

	if(((!media_seat) && (!mdio_enabled)) || ((media_seat) && (!sf_mdio_enabled)))
	{
		reg_write(MDIO_CTL,0x6); 	// enable mdi and autoscan
		
		s_ReadValue = reg_read((unsigned int)MDIO_CTL);
		s_ReadValue = s_ReadValue >> 3;
		s_ReadValue &= DEFAULT_SET;
		mdio_done = s_ReadValue;
		while(mdio_done != 0)
		{
			ctr++;
			hal_delay_us(10000);
			if (ctr > 5)
			{
				eprintf("ERROR: COULD NOT PERFORM MDIO READ\n");
				return;
			}
			s_ReadValue = reg_read((unsigned int)MDIO_CTL);
			s_ReadValue = s_ReadValue >> 3;
			s_ReadValue &= DEFAULT_SET;
			mdio_done = s_ReadValue;
		}
		ctr = 0;
		if(media_seat)
		{
			sf_mdio_enabled = 1;
		}
		else
		{
			mdio_enabled = 1;
		}
	}

    s_RegAddress = arg_RegAddress & PHY_REG_ADDRESS;
	s_PhyAddress = arg_PhyAddress & PHY_DEVICE_ADDRESS;
	s_PhyAddress = s_PhyAddress << PHY_DEVICE_SHIFT;
	
	s_WriteValue = s_PhyAddress | s_RegAddress;
	s_WriteValue = s_WriteValue | PHY_WRITE;
	
	reg_write(MDIO_SINGLE_RW_DATA,arg_WriteValue);  // write value to be programmed in phy
	
	reg_write(MDIO_SNGL_CMD, s_WriteValue);        // write command to mdio
	
	s_ReadValue = reg_read((unsigned int)MDIO_SNGL_CMD);
	
	s_ReadValue = s_ReadValue >> 20;
	s_ReadValue &= DEFAULT_SET;
	mdio_done = s_ReadValue;
	
	while (mdio_done != 0)
	{
		ctr++;
		hal_delay_us(10000);
		if (ctr > 5)
		{
			eprintf("ERROR: COULD NOT PERFORM MDIO WRITE\n");
			break;
		}
		s_ReadValue = reg_read((unsigned int)MDIO_SNGL_CMD);
		s_ReadValue = s_ReadValue >> 20;
		s_ReadValue &= DEFAULT_SET;
		mdio_done = s_ReadValue;
	}
}

void gbe_mac_phy_read_all()
{
    short i=0;
    short phy = 0;
	char inp[2],p;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->argv[3][0] == 's')
	{
		media_seat = 0xBABECAFE;
	}

	eprintf("Enter phy no (0-3) :\n");
	gets(inp,2);
	p = inp[0];

	switch (p)
	{
		case '0' :	phy = 0;
					break;
		case '1':	phy = 1;
					break;
		case '2' :	phy = 2;
					break;
		case '3' :	phy = 3;
					break;
		default :	phy = 0;
					break;
	}

	eprintf("Reading PHY %d Registers \n",phy);
	eprintf("------------------------\n");
    eprintf("\n Reg Number \t\t	Value\n");
    
    for (i=0;i<=NUM_PHY_REGS;i++)
    {
		eprintf("0x%x \t\t  0x%x\n", i, gbe_mac_phy_read(phy, i));
    }
	media_seat = 0;
}

UINT32 gbe_mac_phy_read( UINT32 arg_PhyAddress,
                         UINT32 arg_RegAddress)
		
{
    UINT32 s_ReadValue;
    UINT32 s_WriteValue = 0;
    UINT32 s_RegAddress, s_PhyAddress;
    UINT32 mdio_done = 0;
    UINT32 ctr=0;
    short phy = arg_PhyAddress;

	if(((!media_seat) && (!mdio_enabled)) || ((media_seat) && (!sf_mdio_enabled)))
	{
		reg_write(MDIO_CTL,0x6); 	// enable mdi and autoscan
		
		s_ReadValue = reg_read((unsigned int)MDIO_CTL);
		s_ReadValue = s_ReadValue >> 3;
		s_ReadValue &= DEFAULT_SET;
		mdio_done = s_ReadValue;
		while(mdio_done != 0)
		{
			ctr++;
			hal_delay_us(10000);
			if (ctr > 5)
			{
				eprintf("ERROR: COULD NOT PERFORM MDIO READ\n");
				return 0;
			}
			s_ReadValue = reg_read((unsigned int)MDIO_CTL);
			s_ReadValue = s_ReadValue >> 3;
			s_ReadValue &= DEFAULT_SET;
			mdio_done = s_ReadValue;
		}
		ctr = 0;
		if(media_seat)
		{
			sf_mdio_enabled = 1;
		}
		else
		{
			mdio_enabled = 1;
		}
	}
	s_RegAddress = arg_RegAddress;
	
	s_PhyAddress = phy << PHY_DEVICE_SHIFT;
	s_WriteValue = s_PhyAddress | s_RegAddress;
	s_WriteValue = s_WriteValue | PHY_READ;
	
	reg_write(MDIO_SNGL_CMD, s_WriteValue);     // write command to mdio
	
	s_ReadValue = reg_read((unsigned int)MDIO_SNGL_CMD);
	s_ReadValue = s_ReadValue >> 20;
	s_ReadValue &= DEFAULT_SET;
	mdio_done = s_ReadValue;
	
	while(mdio_done != 0)
	{
		ctr++;
		hal_delay_us(10000);
		if (ctr > 5)
		{
			eprintf("ERROR: COULD NOT PERFORM MDIO READ\n");
			break;
		}
		s_ReadValue = reg_read((unsigned int)MDIO_SNGL_CMD);
		s_ReadValue = s_ReadValue >> 20;
		s_ReadValue &= DEFAULT_SET;
		mdio_done = s_ReadValue;
	}
	
	if(ctr < 6)
		return(reg_read(MDIO_SINGLE_RW_DATA) >> 16);
	else
		return 0;
}

void mac_config()
{
	int port=0;

	for(port=0; port<NUM_PORTS;port++)
	{
		reg_write((PKT_FILTER_CTL+(port*PORTSZ)),0); 
	}
}

void fiber_init()
{

   /* apply reset to MAC device */
    mac_reset_config();

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(INTERFACE_MODE,0x0);	/** enable fiber mode **/
	reg_write(SERDES_CLKMODE_REG,0xF);

	reg_write(SPI3_TX_GLOBAL_CONFIG,0x20aa0f);  
	reg_write(SPI3_RX_CONFIG, 0xAA0F80);

	reg_write(LED_MODE, 0x3);	
	
	/* enable crc addition */
	reg_write(DIV_CONFIG_WORD,0x116D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x116D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x116D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x116D);  

	/* enable crc strip */
	reg_write(0x5B3,0xF0);

	/* Pause control not to be disabled */
	/*
	reg_write(FC_MODE,0);  
	reg_write(FC_MODE+INDEX_PORT1,0);  
	reg_write(FC_MODE+INDEX_PORT2,0);  
	reg_write(FC_MODE+INDEX_PORT3,0); 
	*/

	reg_write(TXFIFO_PAD_ENB,0xAAAA);  
	reg_write(SERDES_TX_RX_POW_DOWN,0x0); 

    reg_write((RXFIFO_DROP_ENABLE),0x0);
	mac_config();

	mac_disable_reset();
	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0xF);
}

#if 0
void gig_config()
{

	short s_PortCount=0;

	for(s_PortCount=0; s_PortCount<4;s_PortCount++)
	{		
		gbe_mac_phy_write(s_PortCount, 31, 0x0);
		gbe_mac_phy_write(s_PortCount, 27, 0x15);
		gbe_mac_phy_write(s_PortCount, 31, 0x0);
		gbe_mac_phy_write(s_PortCount, 9, 0x1b00);
		gbe_mac_phy_write(s_PortCount, 31, 0x2a30);
		gbe_mac_phy_write(s_PortCount, 8, 0x208);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa242);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 0, 0x8242);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa244);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x298d);
		gbe_mac_phy_write(s_PortCount, 0, 0x8244);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa244);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x289a);
		gbe_mac_phy_write(s_PortCount, 0, 0x8244);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa246);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fb1);
		gbe_mac_phy_write(s_PortCount, 0, 0x8246);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa246);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fc0);
		gbe_mac_phy_write(s_PortCount, 0, 0x8246);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xaa42);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x218d);
		gbe_mac_phy_write(s_PortCount, 0, 0x8a42);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xaa42);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 0, 0x8a42);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xaa44);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x298d);
		gbe_mac_phy_write(s_PortCount, 0, 0x8a44);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xaa44);
		gbe_mac_phy_write(s_PortCount, 1, 0x289a);
		gbe_mac_phy_write(s_PortCount, 0, 0x8a44);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xaa46);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fb1);
		gbe_mac_phy_write(s_PortCount, 0, 0x8a46);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 31, 0xaa46);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fc0);
		gbe_mac_phy_write(s_PortCount, 31, 0x8a46);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb242);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x218d);
		gbe_mac_phy_write(s_PortCount, 0, 0x9242);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb242);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x2066);
		gbe_mac_phy_write(s_PortCount, 0, 0x9242);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb242);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x2066);
		gbe_mac_phy_write(s_PortCount, 0, 0x9242);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb244);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x298d);
		gbe_mac_phy_write(s_PortCount, 0, 0x9244);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb244);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x289a);
		gbe_mac_phy_write(s_PortCount, 0, 0x9244);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb246);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fb1);
		gbe_mac_phy_write(s_PortCount, 0, 0x9246);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xb246);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fc0);
		gbe_mac_phy_write(s_PortCount, 0, 0x9246);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba42);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x218d);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a42);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba42);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x2066);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a42);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba44);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x298d);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a44);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba44);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x289a);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a44);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba46);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fb1);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a46);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xba46);
		gbe_mac_phy_write(s_PortCount, 2, 0x0);
		gbe_mac_phy_write(s_PortCount, 1, 0x3fc0);
		gbe_mac_phy_write(s_PortCount, 0, 0x9a46);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa70c);
		gbe_mac_phy_write(s_PortCount, 2, 0xf0);
		gbe_mac_phy_write(s_PortCount, 1, 0xe30d);
		gbe_mac_phy_write(s_PortCount, 0, 0x870c);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa770);
		gbe_mac_phy_write(s_PortCount, 2, 0x85);
		gbe_mac_phy_write(s_PortCount, 1, 0x1ff0);
		gbe_mac_phy_write(s_PortCount, 0, 0x8770);
		gbe_mac_phy_write(s_PortCount, 31, 0x52b5);
		gbe_mac_phy_write(s_PortCount, 0, 0xa770);
		gbe_mac_phy_write(s_PortCount, 2, 0x85);
		gbe_mac_phy_write(s_PortCount, 1, 0x1f40);
		gbe_mac_phy_write(s_PortCount, 0, 0x8770);
		gbe_mac_phy_write(s_PortCount, 31, 0x2a30);
		gbe_mac_phy_write(s_PortCount, 8, 0x8);
		gbe_mac_phy_write(s_PortCount, 31, 0x2a30);
		gbe_mac_phy_write(s_PortCount, 5, 0x8018);
		gbe_mac_phy_write(s_PortCount, 31, 0x0);
		gbe_mac_phy_write(s_PortCount, 24, 0x2247);
		gbe_mac_phy_write(s_PortCount, 31, 0x0);
		gbe_mac_phy_write(s_PortCount, 0, 0x140);
	}
}
#endif

void ShowRegisterVal()
{

spConfig();
	
	eprintf(" Vallejo Register configuration -- \n");

	eprintf("JTAG id : 0x%x\n",reg_read(0x50C));
	eprintf("Port Enable : 0x%x\n",reg_read(0x500));
	eprintf("Loopback : 0x%x\n",reg_read(0x5b2));
	eprintf("Mini frame padding : 0x%x\n",reg_read(0x63E));
	eprintf("Interface mode : 0x%x\n",reg_read(0x501));
	eprintf("Diverse Config : 0x%x\n",reg_read(0x018));
	eprintf("Desired Duplex : 0x%x\n",reg_read(0x02));
	eprintf("Rx Config word : 0x%x\n",reg_read(0x16));
	eprintf("SPI-3 Tx  : 0x%x\n",reg_read(0x700));
	eprintf("SPI-3 Rx : 0x%x\n",reg_read(0x701));
	eprintf("LED control : 0x%x\n",reg_read(0x509));
	eprintf("Station Address Low: 0x%x\n",reg_read(0x0));
	eprintf("Station Address High: 0x%x\n",reg_read(0x1));
	eprintf("PORT0_RX_HIGH_WATERMARK:0x%x\n",reg_read(PORT0_RX_HIGH_WATERMARK));
	eprintf("PORT1_RX_HIGH_WATERMARK:0x%x\n",reg_read(PORT1_RX_HIGH_WATERMARK));			
	eprintf("PORT2_RX_HIGH_WATERMARK:0x%x\n",reg_read(PORT2_RX_HIGH_WATERMARK));	
	eprintf("PORT3_RX_HIGH_WATERMARK:0x%x\n",reg_read(PORT3_RX_HIGH_WATERMARK));	

	eprintf("PORT0_RX_LOW_WATERMARK:0x%x\n",reg_read(PORT0_RX_LOW_WATERMARK));
	eprintf("PORT1_RX_LOW_WATERMARK:0x%x\n",reg_read(PORT1_RX_LOW_WATERMARK));			
	eprintf("PORT2_RX_LOW_WATERMARK:0x%x\n",reg_read(PORT2_RX_LOW_WATERMARK));	
	eprintf("PORT3_RX_LOW_WATERMARK:0x%x\n",reg_read(PORT3_RX_LOW_WATERMARK));	
	
	eprintf("RXFIFO_DROP_ENABLE: 0x%x\n",reg_read(RXFIFO_DROP_ENABLE));
	eprintf("RXFIFO_CAPT_LPBK_ENB: 0x%x\n",reg_read(RXFIFO_CAPT_LPBK_ENB));
	eprintf("RXFIFO_PAD_CRCSTRIP_ENB: 0x%x\n",reg_read(RXFIFO_PAD_CRCSTRIP_ENB));


	eprintf("PORT0_TX_HIGH_WATERMARK:0x%x\n",reg_read(PORT0_TX_HIGH_WATERMARK));
	eprintf("PORT1_TX_HIGH_WATERMARK:0x%x\n",reg_read(PORT1_TX_HIGH_WATERMARK));			
	eprintf("PORT2_TX_HIGH_WATERMARK:0x%x\n",reg_read(PORT2_TX_HIGH_WATERMARK));	
	eprintf("PORT3_TX_HIGH_WATERMARK:0x%x\n",reg_read(PORT3_TX_HIGH_WATERMARK));	

	eprintf("PORT0_TX_LOW_WATERMARK:0x%x\n",reg_read(PORT0_TX_LOW_WATERMARK));
	eprintf("PORT1_TX_LOW_WATERMARK:0x%x\n",reg_read(PORT1_TX_LOW_WATERMARK));			
	eprintf("PORT2_TX_LOW_WATERMARK:0x%x\n",reg_read(PORT2_TX_LOW_WATERMARK));	
	eprintf("PORT3_TX_LOW_WATERMARK:0x%x\n",reg_read(PORT3_TX_LOW_WATERMARK));	
	
	eprintf("PORT0_TX_MAC_THRESHOLD: 0x%x\n",reg_read(PORT0_TX_MAC_THRESHOLD));
	eprintf("PORT1_TX_MAC_THRESHOLD: 0x%x\n",reg_read(PORT1_TX_MAC_THRESHOLD));
	eprintf("PORT2_TX_MAC_THRESHOLD: 0x%x\n",reg_read(PORT2_TX_MAC_THRESHOLD));
	eprintf("PORT3_TX_MAC_THRESHOLD: 0x%x\n",reg_read(PORT3_TX_MAC_THRESHOLD));

	eprintf("TXFIFO_OVFLW: 0x%x\n",reg_read(TXFIFO_OVFLW));
	eprintf("LOOP_RXDATA_TXFIFO: 0x%x\n",reg_read(LOOP_RXDATA_TXFIFO));
		eprintf("PORT0_TX_FRAME_REMOVED: 0x%x\n",reg_read(PORT0_TX_FRAME_REMOVED));
	eprintf("PORT1_TX_FRAME_REMOVED: 0x%x\n",reg_read(PORT0_TX_FRAME_REMOVED));

	eprintf("PORT2_TX_FRAME_REMOVED: 0x%x\n",reg_read(PORT0_TX_FRAME_REMOVED));

	eprintf("PORT3_TX_FRAME_REMOVED: 0x%x\n",reg_read(PORT0_TX_FRAME_REMOVED));

spRestore();
}


void mixed_mode_init(UINT32 set_int_lpbk)
{
//	if(!set_int_lpbk)
//	{
//		config_phy();
//	}
//	else
//	{
//		config_phy_for_line_lpbk();
//	}

	// apply reset to MAC device
	config_phy();
    mac_reset_config();

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	if (cport == 0)
	{
		reg_write(INTERFACE_MODE,0x3);	// enable copper mode on port 0 and 1
	}
	else
	{
		reg_write(INTERFACE_MODE,0xC);	// enable copper mode on port 2 and 3
	}

//	switch(dplx)
//	{
//		/* chinmay: only Cu ports can be in half duplex mode */
//		case 'h' :      reg_write(DESIRED_DUPLEX,0x0);
//				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x0);
//				//reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x0);    
//				//reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x0);
//				break;
//		case 'f' :
	if (cport == 0)
	{
		reg_write(DESIRED_DUPLEX,0x1);
		reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x1);
	}
	else
	{
		reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x1);    
		reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x1);
	}
//			   break;
//		default :       eprintf("Illegal duplex optinon.\n");
//				eprintf("The opertation will be full duplex..\n");
//				break;
//	}			
	
		//For mixed mode, fiber ports must always be in full duplex
//		reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x1);   
//		reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x1);

	reg_write(SERDES_CLKMODE_REG,0xF);

//#ifdef SPHY4x8
	reg_write(SPI3_TX_GLOBAL_CONFIG,0x20aa0f);  
//#else
//	reg_write(SPI3_TX_GLOBAL_CONFIG,0x00aa0f);  
//#endif

	// Enable CRC addition
	if (cport == 0)
	{
		reg_write(DIV_CONFIG_WORD,0x114D);				// Cu
		reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x114D);	// Cu
		reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x116D);	// Fibre
		reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x116D);  // Fibre
	}
	else
	{
		reg_write(DIV_CONFIG_WORD,0x116D);				// Fibre
		reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x116D);  // Fibre
		reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x114D);	// Cu
		reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x114D);	// Cu
	}

	// Enable CRC stripping
	reg_write(0x5B3,0xF0);

	reg_write(SERDES_CLKMODE_REG,0x0);

//	if (speed == 100)
//	{
//	    reg_write(MAC_IF_MODE,0x5);  
//	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x5);  
//	    //reg_write(MAC_IF_MODE+INDEX_PORT2,0x5);  
//	    //reg_write(MAC_IF_MODE+INDEX_PORT3,0x5);  
//	}

//	if (speed == 10)
//	{
//	    reg_write(MAC_IF_MODE,0x4);  
//	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x4);  
//	    //reg_write(MAC_IF_MODE+INDEX_PORT2,0x4);  
//	    //reg_write(MAC_IF_MODE+INDEX_PORT3,0x4);  
//	}
//	
//	if (speed == 1000)
//	{

	if (cport == 0)
	{
		reg_write(MAC_IF_MODE,0x6);  
		reg_write(MAC_IF_MODE+INDEX_PORT1,0x6);  
	}
	else
	{
		reg_write(MAC_IF_MODE+INDEX_PORT2,0x6);  
		reg_write(MAC_IF_MODE+INDEX_PORT3,0x6);  
	}

//	    //reg_write(MAC_IF_MODE+INDEX_PORT2,0x6);  
//	    //reg_write(MAC_IF_MODE+INDEX_PORT3,0x6);  
//	}

	reg_write(SERDES_CLKMODE_REG,0xF);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(TXFIFO_PAD_ENB,0xAAAA);  

	reg_write(GBIC_CTRL_REG,0x1C00);    

	reg_write(LED_MODE, 0x3);

	// Activate units
	reg_write(0x505, 0x0);		// MAC Soft reset 
	reg_write(0x506, 0x0);		// MDIO Reset
	reg_write(0x620, 0x0);		// TX FIFO Reset
	reg_write(0x59E, 0x0);		// Rx FIFO Reset

	mac_disable_reset();

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);
	reg_write(PORT_ENABLE,0xF);
}


void mixed_mode_init1(UINT32 set_int_lpbk)
{
    if(!set_int_lpbk)
    {
        config_phy();
    }
    else
    {
        config_phy_for_line_lpbk();
    }
   /* apply reset to MAC device */
    mac_reset_config();

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(INTERFACE_MODE,0x3);	// enable copper mode on port 0 and 1

	switch(dplx)
	{
		/* chinmay: only Cu ports can be in half duplex mode */
		case 'h' : /*     reg_write(DESIRED_DUPLEX,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x0);*/
				reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x0);    
				reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x0);
				break;
		case 'f' :
	   	       reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x1);
			   reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x1);
			   break;
		default :       eprintf("Illegal duplex optinon.\n");
				eprintf("The opertation will be full duplex..\n");
				break;
	}			
	
	/* For mixed mode, fiber ports must always be in full duplex */
		reg_write(DESIRED_DUPLEX+INDEX_PORT0,0x1);   
		reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x1);

	reg_write(SERDES_CLKMODE_REG,0xF);

#ifdef SPHY4x8
	reg_write(SPI3_TX_GLOBAL_CONFIG,0x20aa0f);  
#else
	reg_write(SPI3_TX_GLOBAL_CONFIG,0x00aa0f);  
#endif

	reg_write(DIV_CONFIG_WORD,0x116D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x116D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x114D);	// Fibre
	reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x114D);  // Fibre

	reg_write(0x5B3,0xF0);

	reg_write(SERDES_CLKMODE_REG,0x0);

	if (speed == 100)
	{
	    //reg_write(MAC_IF_MODE,0x5);  
	    //reg_write(MAC_IF_MODE+INDEX_PORT1,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x5);  
	}

	if (speed == 10)
	{
	  //  reg_write(MAC_IF_MODE,0x4);  
	   // reg_write(MAC_IF_MODE+INDEX_PORT1,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x4);  
	}
	
	if (speed == 1000)
	{
//	    reg_write(MAC_IF_MODE,0x6);  
//	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x6);  
	}
	reg_write(SERDES_CLKMODE_REG,0xF);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(TXFIFO_PAD_ENB,0xAAAA);  

	reg_write(GBIC_CTRL_REG,0x1C00);    

	reg_write(LED_MODE, 0x3);

	// Activate units
	reg_write(0x505, 0x0);		// MAC Soft reset 
	reg_write(0x506, 0x0);		// MDIO Reset
	reg_write(0x620, 0x0);		// TX FIFO Reset
	reg_write(0x59E, 0x0);		// Rx FIFO Reset

	mac_disable_reset();

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);
	reg_write(PORT_ENABLE,0xF);
}

short check_autoneg_status_mixed_mode()
{
	short s_PortCount = 0, flag = OK, ctr = 0, done = 0;
	UINT32 regval = 0, link=0,receiver_status=0, autoneg_complete=0;
	
	eprintf("Waiting for auto-negotiation to complete ....\n");
	for(s_PortCount=cport; s_PortCount<(cport+2); s_PortCount++)
    {		
		do
		{	
			switch(speed)
			{
				case 1000:
							regval = gbe_mac_phy_read(s_PortCount,0xA);
							link = gbe_mac_phy_read(s_PortCount,0x11);
							link = (link >> 12) & 0x1;
							receiver_status = (regval >> 12) & 0x3;
							//eprintf("receiver status = 0x%x\n",receiver_status);
							if (receiver_status == 0x3)
							{
								done = 1;
							}
							break;
				case 100:
							//regval = gbe_mac_phy_read(s_PortCount,0xA);
							link = gbe_mac_phy_read(s_PortCount,0x01);
							link = (link >>2) & 1;
							done = 1;
							break;
				case 10:
							//regval = gbe_mac_phy_read(s_PortCount,0xA);
							link = gbe_mac_phy_read(s_PortCount,0x16);
							done = gbe_mac_phy_read(s_PortCount,0x01);
							link = (link >> 6) & 1;
							done = (done >> 2) & 1;
							break;
			}
			//eprintf("Regval for port %d  = 0x%08X\n",s_PortCount,regval);	
			//receiver_status = (regval >> 12) & 0x3;
			autoneg_complete = gbe_mac_phy_read(s_PortCount,0x1);
			if ((((autoneg_complete >> 5) & 1) == 0x1) && (link == 1) && done)
			{
				flag = OK;
				break;
			}
			else
			{	
				ctr ++;
				flag = ERROR;
			}
		}while (ctr < 5000);
	
		if (flag == ERROR)
		{
			eprintf("Auto-negotiation failed for port %d , link = 0x%08X, auto = 0x%08X\n",s_PortCount,link, autoneg_complete);
				
		}
	}

	return flag;
}

void read_all_regs()
{

  int c=0;
  int port_offset = 0;
  int i=0;
  register PDiagCommon acL = (PDiagCommon) ACADDRESS;

  if(acL->argv[3][0] == 's')
  {
	  media_seat = 0xBABECAFE;
  }

  spConfig();
  
  eprintf("offset \t value\n");
  eprintf("MAC control registers ----\n");
  for (i=0;i<=3;i++)
  {
      eprintf("port %d ----\n",i);
      for (c=0;c<=0x1B;c++)
      {
          eprintf("0x%x \t 0x%x\n", c, reg_read(port_offset+c));
      }
 
      port_offset = port_offset + 0x80;
  }
  port_offset = 0;

  eprintf("RX stat registers -- \n");
  for (i=0;i<=3;i++)
  {
      eprintf("port %d ----\n",i);
      for (c=0x20;c<=0x39;c++)
      {
          eprintf("0x%x \t 0x%x\n", c, reg_read(port_offset+c));
      }
 
      port_offset = port_offset + 0x80;
  }
  port_offset = 0;

  eprintf("TX stat registers -- \n");

  for (i=0;i<=3;i++)
  {
      eprintf("port %d ----\n",i);
      for (c=0x40;c<=0x58;c++)
      {
          eprintf("0x%x \t 0x%x\n", c, reg_read(port_offset+c));
      }
 
      port_offset = port_offset + 0x80;
  }
  port_offset = 0;

  eprintf("PHY autoscan registers -- \n");

  for (i=0;i<=3;i++)
  {
      eprintf("port %d ----\n",i);
      for (c=0x60;c<=0x7F;c++)
      {
          eprintf("0x%x \t 0x%x\n", c, reg_read(port_offset+c));
      }
 
      port_offset = port_offset + 0x80;
  }

  eprintf("Global status and config registers -- \n");

  for (c=0x500;c<=0x50c;c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }
 

  eprintf("RX FIFO registers -- \n");

  for (c=0x580;c<=(0x580+0x3f);c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }
  
  eprintf("TX FIFO registers -- \n");
  
  for (c=0x600;c<=0x63e;c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }
  
  eprintf("MDIO registers -- \n");
  
  for (c=0x680;c<=0x683;c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }
  
  eprintf("SPI-3 registers -- \n");
  
  for (c=0x700;c<=0x716;c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }
  
  eprintf("Serdes registers -- \n");
  
  for (c=0x780;c<=(0x780 + 0x18);c++)
  {
	  eprintf("0x%x \t 0x%x\n",c,reg_read(c));
  }

  eprintf("GBIC registers -- \n");
  
  for (c = (0x780 + 0x19); c <= (0x780 + 0x1B); c++)
  {
	  eprintf("0x%x \t 0x%x\n", c, reg_read(c));
  }

  eprintf("PLL tune registers -- \n");
  
  for (c = (0x780 + 0x1C); c <= (0x780 + 0x1E); c++)
  {
	  eprintf("0x%x \t 0x%x\n", c, reg_read(c));
  }

  eprintf(" -- Read complete -- \n");

  spRestore();
  media_seat = 0;

}


void spConfig()
{
   Disable_CPSR_Int();

	orig_sp_val[0] = GET32(0xC0080000);
	orig_sp_val[1] = GET32(0xC0080004);
	orig_sp_val[2] = GET32(0xC0080008);
	orig_sp_val[3] = GET32(0xC008000C);
	orig_sp_val[4] = GET32(0xC0080010);
	orig_sp_val[5] = GET32(0xC0080014);
	orig_sp_val[6] = GET32(0xC0080018);
	orig_sp_val[7] = GET32(0xC008001C);


	if(((silicon_rev & 0xF0) >> 4))
	{
		//PUT32(0xC0080000, 0x6);
		PUT32(0xC0080000, 0xD);
	}
	else
	{
		PUT32(0xC0080000, 0x2);
	}
	PUT32(0xC0080008, 0x62);
	PUT32(0xC0080010, 0x62);
	PUT32(0xC0080018, 0x3);
	PUT32(0xC008001C, 0x33);

 Enable_CPSR_Int();
}

void spRestore()
{
   Disable_CPSR_Int();

		PUT32(0xC0080000, orig_sp_val[0]);
		PUT32(0xC0080004, orig_sp_val[1]);
		PUT32(0xC0080008, orig_sp_val[2]);
		PUT32(0xC008000C, orig_sp_val[3]);
		PUT32(0xC0080010, orig_sp_val[4]);
		PUT32(0xC0080014, orig_sp_val[5]);
		PUT32(0xC0080018, orig_sp_val[6]);
		PUT32(0xC008001C, orig_sp_val[7]);
 Enable_CPSR_Int();

}

int ctoi(char c)
{
	int j=0;

    if ((c >= '0') && (c <= '9'))  // check if number
	{
		j = c - '0';
    }
    return j;	
}

unsigned int htoi(char *s)
{
	unsigned int number = 0,temp=1;	
	unsigned int len=0,i=1;
	char *t=s;

	while (*s++ != 0) 
	    len++;

	while(i<len) 
	{
  	    temp = temp * 16;
	    i++;
	}

	i=0;
	while (t[i] != '\0')
	{
	    number = number + (temp*ctoi(t[i]));
        i++;
	    temp = temp/16;	
	}

	return number;
}


void phy_write()
{
	char p_addr[3],val[10];
	UINT32 address=0;
	UINT32 value=0;
	UINT32 port=0;

    spConfig();

	eprintf("Enter phy reg address :\n");
	gets(p_addr,4);
	eprintf("Enter value :\n");
	gets(val,9);

	address=htoi(p_addr);	
	value=htoi(val);
	
    for(port=0; port<4;port++)
    {
        gbe_mac_phy_write(port, address, value);
    }
    spRestore();
}


void get_stats()
{
	UINT32 port_offset=0;
	short i=0,c=0;

	spConfig();

	for (i=0;i<=3;i++)
	{
		for (c=0x20;c<=0x39;c++)
		{
			rxstat[i][c-0x20]=reg_read(port_offset+c);
		}
 
		port_offset = port_offset + 0x80;
	}  

	eprintf("Reading tx stat registers -- \n");

	for (i=0;i<=3;i++)
	{
		for (c=0x40;c<=0x58;c++)
		{
			txstat[i][c-0x40]=reg_read(port_offset+c);
		}
 
		port_offset = port_offset + 0x80;
	}
	
	spRestore(); 

	check_error_display_rxstats();
	check_error_display_txstats();
}

/** This function displays the messages for 
    respective error conditions only. **/

void check_error_display_rxstats()
{
	short port = 0, rxport = 0;

	for (port=0;port<=3;port++)
	{
		if ((channel == 'f') && (ports != 4))
		{
			rxport = ports;
		}
		if ((channel == 'c') && (ports != 4))
		{
			switch(ports)
			{
				case 0: rxport = 1;
						break;
				case 1: rxport = 0;
						break;
				case 2: rxport = 3;
						break;
				case 3: rxport = 2;
						break;
			}
		}
		if (ports == 4)
			rxport = port;
			
		if (port != rxport)
			continue;

		eprintf("\n Rx Statistics for port %d\n",rxport);
		eprintf("-------------------------------\n");

		eprintf("Rx Unicast packets : %d\n",rxstat[port][2]);
		eprintf("Rx 64 byte packets : %d\n",rxstat[port][5]);
		eprintf("Rx 65-127 byte packets : %d\n",rxstat[port][6]);
		eprintf("Rx 128-255 byte packets : %d\n",rxstat[port][7]);
		eprintf("Rx FCS Errors : %d\n",rxstat[port][12]);
		eprintf("Rx Rx Data Errors : %d\n",rxstat[port][14]);
		eprintf("Rx Align Errors : %d\n",rxstat[port][15]);
		eprintf("Rx Jabber Errors : %d\n",rxstat[port][17]);
		eprintf("Rx Runt Errors : %d\n",rxstat[port][21]);
		eprintf("Rx Carrier Extend Errors : %d\n",rxstat[port][23]);
		eprintf("Rx Sequence Errors : %d\n",rxstat[port][24]);
		eprintf("Rx Symbol Errors : %d\n",rxstat[port][25]);
	}

}

void check_error_display_txstats()
{
	short port = 0, txport = 0;

	for (port=0;port<=3;port++)
	{
		if (ports != 4)
		{
			txport = ports;
		}
		else
			txport = port;
			
		if (port != txport)
			continue;

		eprintf("\n Tx Statistics for port %d\n",txport);
		eprintf("-------------------------------\n");

		eprintf("Tx Unicast packets : %d\n",txstat[port][2]);
		eprintf("Tx 64 byte packets : %d\n",txstat[port][5]);
		eprintf("Tx 65-127 byte packets : %d\n",txstat[port][6]);
		eprintf("Tx 128-255 byte packets : %d\n",txstat[port][7]);
		eprintf("Tx FCS Errors : %d\n",txstat[port][12]);
		eprintf("Tx Excessive collision Errors : %d\n",txstat[port][17]);
		eprintf("Tx Excessive Deferral Errors : %d\n",txstat[port][18]);
		eprintf("Tx Underrun : %d\n",txstat[port][20]);
		eprintf("Tx CRC Errors : %d\n",txstat[port][22]);
		
	}

}

void gets(char *str, int len)
{
    char temp[30];

	getCmdLine(temp);
    memcpy(str, temp, (len - 1));
	str[len - 1] = '\0';
}


