/*
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------------
 */

/**
 * System defined include files required.0
 */

/**
 * User defined include files required.
 */
#include "mac_util.h"
#include "mac_init.h"
#include "diag.h"
#include "mac_diag.h"
#include "packet_pool.h"
#include "error_map.h"
#include "prototypes.h"
#include "register_map.h"
#include "i2c_diag.h"
#include "i2c.h"
#include "cfg_utility.h"
#include "diagstruct.h"


/**
 * These bit masks are used to set a bit in the mac_test_flag
 * The corresponding diagnostic routine against the set bit is invoked.
 */
#define RMON_TEST_MASK          0x1     	/* bit 0 */
#define PERPORT_TEST_MASK       0x2     	/* bit 1 */
#define ATSCAN_TEST_MASK        0x4     	/* bit 2 */
#define POWDN_TEST_MASK         0x8     	/* bit 3 */
#define RESET_TEST_MASK         0x10    	/* bit 4 */
#define WMARK_TEST_MASK         0x20    	/* bit 5 */
#define VLPSE_TEST_MASK         0x40    	/* bit 6 */
#define CRC_TEST_MASK           0x80    	/* bit 7 */
#define PAD_TEST_MASK           0x100   	/* bit 8 */
#define FRSZ_TEST_MASK          0x200   	/* bit 9 */
#define ANEG_TEST_MASK          0x400   	/* bit 10 */
#define MDIO_TEST_MASK          0x800   	/* bit 11 */
#define LED_TEST_MASK           0x1000  	/* bit 12 */
#define SPI3_TEST_MASK          0x2000  	/* bit 13 */
#define I2C_TEST_MASK           0x4000  	/* bit 14 */
#define SYSLB_TEST_MASK         0x8000  	/* bit 15 */
#define LPBK_TEST_MASK          0x10000 	/* bit 16 */
#define LRATE_TEST_MASK         0x20000 	/* bit 17 */
#define ADR_FILTER_MASK			0x40000 	/* bit 18 */
#define ALL_TESTS_MASK          0x80000 	/* bit 19 */
#define REG_RD					0x100000    /* bit 20 */
#define REG_WR					0x200000    /* bit 21 */
#define RD_ALL_REGS				0x400000	/* bit 22 */
#define PHY_READ_ALL			0xF		
#define PHY_REG_WRITE			0xD
#define INTERNAL_LPBK			0xE
#define I2C_EEPROM				0xA
#define TEST_LPBK_XSCALE		0xB
#define LINE_TEST_MASK			0xC
#define MDC_SEL_MASK			0xA1
 

unsigned int mac_test_flag = 0;

UINT32 *pl_sys_lb_data64_0;
UINT32 *pl_sys_lb_data64_1;
UINT32 *pl_sys_lb_data64_2;
UINT32 *pl_sys_lb_data64_3;
UINT32 *pl_sys_lb_data96_0;
UINT32 *pl_sys_lb_data96_1;
UINT32 *pl_sys_lb_data96_2;
UINT32 *pl_sys_lb_data96_3;
UINT32 *pl_sys_lb_data138_0;
UINT32 *pl_sys_lb_data138_1;
UINT32 *pl_sys_lb_data138_2;
UINT32 *pl_sys_lb_data138_3;

UINT32 PKT_4_SZ = 1*4;
UINT32 PKT_48_SZ = 11*4;
UINT32 PKT_96_SZ = 23*4;
UINT32 PKT_64_SZ = 15*4;
UINT32 PKT_138_SZ = 132;
UINT32 PKT_256_SZ = 256;
UINT32 PKT_512_SZ = 512;
UINT32 PKT_1024_SZ = 1024;
UINT32 PKT_1520_SZ = 1520;
UINT32 PKT_5K_SZ = 5*1024;
UINT32 PKT_10K_SZ = 11*1024;

/** Global variables **/
UINT32 total_args = 0, loop = 1, ports = 4;
UINT8 channel='0',test_opt='?' ;
UINT32 speed=1000;  
UINT8 dplx='f',loopback_mode='e';
UINT8 test_param = '?';
UINT32 rxstat[4][30],txstat[4][30];

extern diag_error *p_aErrorMap;

char *aTestHelp[] =
{
	"Syntax :",
	"t mac <option> <test_param> [fiber mode] [1000] <port no.> <loop count>",
	"where -",
	"[fiber_mode] {f} -> tests run on fiber configuration.",
	"[1000] -> speed",    
	"[port]: ",
	"      0/1/2/3 -> Test is executed on one of these ports",
	"      4       -> Test is executed on all ports",
	"[loop count] -> Number of loops for the loopback test.",
	"<option> <test_param>:",           
    "1. f u: Unicast address filtering",
	"syntax: t mac f u f 1000 <port> <loop>",
    "2. f m: Multicast address filtering",
	"syntax: t mac f m f 1000 <port> <loop>",
    "3. f b: Broadcast address filtering",
	"syntax: t mac f b f 1000 <port> <loop>",
    "4. s r: SPI-3 TERR signal test",
	"syntax: t mac s r f 1000 <port> <loop>",	
    "5. s t: SPI-3 TPA/TENB signal test",
	"syntax: t mac s t f 1000 <port> <loop>",	
    "6. s m: SPI-3 MPHY-32 mode/Inband addressing test",
	"syntax: t mac s m f 1000 <port> <loop>",	
    "7. p: Per Port Register tests",
	"syntax: t mac p",
    "8. i: I2C tests",
	"syntax: t mac i",
    "9. m: MDIO test ",
	"syntax: t mac m",
    "10. m a: Autoscan test ",
	"syntax: t mac m a",
    "11. d e: Tx FIFO Padding test",
	"syntax: t mac d e f 1000 <port> <loop>",
    "12. d p: Rx FIFO Pre-pending test",
	"syntax: t mac d p f 1000 <port> <loop>",
    "13. v l: Vlan frame filtering test",
	"syntax: t mac v l f 1000 <port> <loop>",
    "14. v p: Pause frame filtering test",
	"syntax: t mac v p f 1000 <port> <loop>",
    "15. w r: Rx FIFO Power down test",
	"syntax: t mac w r f 1000 <port> <loop>",
    "16. w t: Tx FIFO Power down test",
	"syntax: t mac w r f 1000 <port> <loop>",
    "17. e r: Rx FIFO Reset test",
	"syntax: t mac e r f 1000 <port> <loop>",
    "18. e t: Tx FIFO Reset test",
	"syntax: t mac e t f 1000 <port> <loop>",
    "19. a n: Auto-negotiation test",
	"syntax: t mac a n f 1000 <port> <loop>",
    "20. l d:  LED test with Rx/Tx",
	"syntax: t mac l d f 1000 <port> <loop>",
    "21. l o:  LED test with disabled ports",
	"syntax: t mac l o f 1000 <port> <loop>",
    "22. c a:  CRC Append and Strip test",
	"syntax: t mac c a f 1000 <port> <loop>",
    "23. k t:  Tx FIFO Watermark test",
	"syntax: t mac k t f 1000 <port> <loop>",
    "24. k r:  Rx FIFO Watermark test",
	"syntax: t mac k r f 1000 <port> <loop>",
    "25. z c:  Frame Size test",
	"syntax: t mac z c f 1000 <port> <loop>",
    "26 :  RMON test: ",
	"           RxOctetsTotalOK/OctetsTransmittedOK",
	"           RxUCPckts/TxUCPkts",           
	"           RxMCPkts/TxMCPkts",
	"           RxBCPkts/TxBCPkts",
	"           RxPkts64Octets/TxPkts64Octets",
	"           RxPkts65to127Octets/TxPkts65to127Octets",
	"           RxPkts128to255Octets/TxPkts128to255Octets",
	"syntax: t mac r 1 f 1000 <port> <loop>",
    "27 :  RMON test: ",
	"           RxPkts256to511Octets/TxPkts256to511Octets",
	"           RxPkts512to1023Octets/TxPkts512to1023Octets",
	"           RxPkts1024to1518Octets/TxPkts1024to1518Octets",
	"           Rxpkts1519toMaxOctets/Txpkts1519toMaxOctets",  
	"syntax: t mac r j f 1000 <port> <loop>",

    "28 :  RMON test: ",
	"           Rx Tagged/Tx Tagged",           
	"syntax: t mac r 2 f 1000 <port> <loop>",
    "29 :  RMON test: ",
	"           PauseMacControlReceivedCounter/TxPauseFrames",           
	"syntax: t mac r 3 f 1000 <port> <loop>",
	"30: RMON test: FCSError/CRCError",           
	"syntax: t mac r 4 f 1000 <port> <loop>",
	"31: RMON test: Rx Runt Error test",           
	"syntax: t mac r 5 c 1000 <port> <loop>",
	"32: RMON test: Rx Short Error test",           
	"syntax: t mac r 6 c 1000 <port> <loop>",
	"33: RMON test: Tx Collision test",           
	"syntax: t mac r c c 1000 <port> <loop>",
	"34: RMON test: Rx DataError test",           
	"syntax: t mac r d c 1000 <port> <loop>",
	"35: RMON test: Rx AlignError test",           
	"syntax: t mac r a c 1000 <port> <loop>",
	"36: RMON test: Rx SequenceError test",           
	"syntax: t mac r s c 1000 <port> <loop>",
	"37: RMON test: Rx SymbolError test",           
	"syntax: t mac r y c 1000 <port> <loop>",
	"38: m s: MDC Speed Selection test",           
	"syntax: t mac m s",

	
};


#define aTestHelpSz (sizeof(aTestHelp) /sizeof(aTestHelp[0]))  


UINT32 tx_data_pl[] =	{ RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,		
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						};

UINT32 fs_tx_data_pl[] =	{ RING_DATA_96_0,	RING_DATA_96_0,		RING_DATA_96_0,		RING_DATA_96_0,	
							  RING_DATA_64_0,	RING_DATA_64_0,		RING_DATA_64_0,		RING_DATA_64_0,	
							  RING_DATA_138_0,	RING_DATA_138_0,	RING_DATA_138_0,	RING_DATA_138_0,	
							  RING_DATA_256_0,	RING_DATA_256_0,	RING_DATA_256_0,	RING_DATA_256_0,	
							  RING_DATA_512_0,	RING_DATA_512_0,	RING_DATA_512_0,	RING_DATA_512_0,	
							  RING_DATA_1024_0, RING_DATA_1024_0, 	RING_DATA_1024_0, 	RING_DATA_1024_0, 
							  RING_DATA_1520_0, RING_DATA_1520_0, 	RING_DATA_1520_0, 	RING_DATA_1520_0, 		
							  RING_DATA_5K_0,	RING_DATA_5K_0,		RING_DATA_5K_0,		RING_DATA_5K_0,	
							  RING_DATA_10K_0,	RING_DATA_10K_0,	RING_DATA_10K_0,	RING_DATA_10K_0	
							};

UINT32 fs_tx_data_pl_0[] =	{	RING_DATA_96_0, RING_DATA_64_0, RING_DATA_138_0,
								RING_DATA_256_0, RING_DATA_512_0, RING_DATA_1024_0,
								RING_DATA_1520_0, RING_DATA_5K_0, RING_DATA_10K_0
							};
UINT32 fs_tx_data_pl_1[] =	{	RING_DATA_96_1, RING_DATA_64_1, RING_DATA_138_1,
								RING_DATA_256_1, RING_DATA_512_1, RING_DATA_1024_1,
								RING_DATA_1520_1, RING_DATA_5K_1, RING_DATA_10K_1
							};

UINT32 fs_tx_data_pl_2[] =	{	RING_DATA_96_2, RING_DATA_64_2, RING_DATA_138_2,
								RING_DATA_256_2, RING_DATA_512_2, RING_DATA_1024_2,
								RING_DATA_1520_2, RING_DATA_5K_2, RING_DATA_10K_2
							};
UINT32 fs_tx_data_pl_3[] =	{	RING_DATA_96_3, RING_DATA_64_3, RING_DATA_138_3,
								RING_DATA_256_3, RING_DATA_512_3, RING_DATA_1024_3,
								RING_DATA_1520_3, RING_DATA_5K_3, RING_DATA_10K_3
							};


UINT32 tx_data_pl_0[] =	{	RING_DATA_96_0, RING_DATA_96_0, RING_DATA_96_0,
							RING_DATA_64_0, RING_DATA_64_0, RING_DATA_64_0,
							RING_DATA_138_0, RING_DATA_138_0, RING_DATA_138_0
						};

UINT32 tx_data_pl_1[] =	{	RING_DATA_96_1, RING_DATA_96_1, RING_DATA_96_1,
							RING_DATA_64_1, RING_DATA_64_1, RING_DATA_64_1,
							RING_DATA_138_1, RING_DATA_138_1, RING_DATA_138_1
						};
UINT32 tx_data_pl_2[] =	{	RING_DATA_96_2, RING_DATA_96_2, RING_DATA_96_2,
							RING_DATA_64_2, RING_DATA_64_2, RING_DATA_64_2,
							RING_DATA_138_2, RING_DATA_138_2, RING_DATA_138_2
						};
UINT32 tx_data_pl_3[] =	{	RING_DATA_96_3, RING_DATA_96_3, RING_DATA_96_3,
							RING_DATA_64_3, RING_DATA_64_3, RING_DATA_64_3,
							RING_DATA_138_3, RING_DATA_138_3, RING_DATA_138_3
						};

UINT32 tx_data_pl_48_0[] ={	RING_DATA_48_0, RING_DATA_48_0, RING_DATA_48_0,
							RING_DATA_64_0, RING_DATA_64_0, RING_DATA_64_0,
							RING_DATA_138_0, RING_DATA_138_0, RING_DATA_138_0
						};
UINT32 tx_data_pl_48_1[] ={	RING_DATA_48_1, RING_DATA_48_1, RING_DATA_48_1,
							RING_DATA_64_1, RING_DATA_64_1, RING_DATA_64_1,
							RING_DATA_138_1, RING_DATA_138_1, RING_DATA_138_1
						};
UINT32 tx_data_pl_48_2[] ={	RING_DATA_48_2, RING_DATA_48_2, RING_DATA_48_2,
							RING_DATA_64_2, RING_DATA_64_2, RING_DATA_64_2,
							RING_DATA_138_2, RING_DATA_138_2, RING_DATA_138_2
						};
UINT32 tx_data_pl_48_3[] ={	RING_DATA_48_3, RING_DATA_48_3, RING_DATA_48_3,
							RING_DATA_64_3, RING_DATA_64_3, RING_DATA_64_3,
							RING_DATA_138_3, RING_DATA_138_3, RING_DATA_138_3
						};
UINT32 tx_data_pl_48[] ={ RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
						  RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
						  RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,		
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						};
UINT32 tx_data_pl_4_0[] ={	RING_DATA_4_0, RING_DATA_4_0, RING_DATA_4_0,
							RING_DATA_64_0, RING_DATA_64_0, RING_DATA_64_0,
							RING_DATA_138_0, RING_DATA_138_0, RING_DATA_138_0
						};
UINT32 tx_data_pl_4_1[] ={	RING_DATA_4_1, RING_DATA_4_1, RING_DATA_4_1,
							RING_DATA_64_1, RING_DATA_64_1, RING_DATA_64_1,
							RING_DATA_138_1, RING_DATA_138_1, RING_DATA_138_1
						};
UINT32 tx_data_pl_4_2[] ={	RING_DATA_4_2, RING_DATA_4_2, RING_DATA_4_2,
							RING_DATA_64_2, RING_DATA_64_2, RING_DATA_64_2,
							RING_DATA_138_2, RING_DATA_138_2, RING_DATA_138_2
						};
UINT32 tx_data_pl_4_3[] ={	RING_DATA_4_3, RING_DATA_4_3, RING_DATA_4_3,
							RING_DATA_64_3, RING_DATA_64_3, RING_DATA_64_3,
							RING_DATA_138_3, RING_DATA_138_3, RING_DATA_138_3
						};
UINT32 tx_data_pl_4[] ={ RING_DATA_4_0, RING_DATA_4_1, RING_DATA_4_2, RING_DATA_4_3,
						  RING_DATA_4_0, RING_DATA_4_1, RING_DATA_4_2, RING_DATA_4_3,
						  RING_DATA_4_0, RING_DATA_4_1, RING_DATA_4_2, RING_DATA_4_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,		
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						};

UINT32 PKT_SEQ_SZ = sizeof(tx_data_pl_0);
UINT32 ALL_PORT_PKT_SEQ_SZ = sizeof(tx_data_pl);

/**
 * Extern function prototypes.
 */

extern void Disable_CPSR_Int(void);
extern void Enable_CPSR_Int(void);
extern void eprintf(char *f, ...);

extern UINT32 rmon_test(void);
extern UINT32 per_port_reg_test(void);
extern UINT32 autoscan_test(void);
extern UINT32 mdio_config_test(void);
extern UINT32 auto_neg_test(void);
extern UINT32 Rx_Tx_Watermark_test(void);
extern UINT32 LED_test(void);
extern UINT32 I2C_test(void);
extern UINT32 spi3_test(void);
extern UINT32 mac_address_filter_test(void);
extern UINT32 vlan_pause_filter_test (void);
extern UINT32 crc_test(void);
extern UINT32 powerDown_test(void);
extern UINT32 padding_test(void);
extern UINT32 frame_size_test(void);
extern UINT32 reset_test(void);
extern UINT32 rx_tx_path_test(void);
extern UINT32 lpbk_test(void);
extern void make_ErrorMap(void);
extern void reg_write( volatile UINT32 offset, UINT32 val);
extern UINT32 reg_read( volatile UINT32 offset);
extern void test_lpbk(void);
extern UINT32 wmark_test(void);
extern void spRestore();
extern void spConfig();
extern int ctoi(char c);
extern void read_all_regs();
extern UINT32 macScan(void);
extern UINT32 init_mac_dev (UINT32);
extern void gets(char *str, int len);
extern void gbe_mac_phy_write(UINT32, UINT32,UINT32);
extern void gbe_mac_phy_read_all(void);
extern UINT32 pl_SysLpbk_test(void);
extern int getCmdLine(char* cmdLine);
extern UINT32 gbe_mac_phy_read(UINT32 arg_PhyAddress, UINT32 arg_RegAddress);
extern UINT32 config_phy();
extern void mac_reset_config(void);
extern void mac_config();
extern unsigned int htoi(char *s);
extern void phy_write();
extern void get_stats();
extern void check_error_display_rxstats(void);
extern void check_error_display_txstats(void);


/* Local function prototypes */
void basic_config();
UINT32 validate_params(void);

void get_option_from_user(void);
UINT32 assign_pkt_pointers(UINT32 ports, UINT32 *pktptr, UINT32 cflag, UINT32 *pnum);
UINT32 cu_init(UINT32 set_int_lpbk);
UINT32 do_mac_test(void);
short check_autoneg_status();


/**
 * Static function prototypes.
 */
static void show_options_help();


/**
 * Function definitions.
 */

    void mac_test(void)
    {
        register PDiagCommon acL = (PDiagCommon) ACADDRESS;    
		test_opt = acL->argv[2][0];	                
   
        if(acL->HostType == MASTER)             // if master NPU
        { 
            /*
             ** initialize error map
             */
        
            make_ErrorMap();

			get_option_from_user();

            if (do_mac_test() != DONE)
            {
                eprintf("Aborting Execution !\n");
				return;
            }

			return;
        } 

    }



    void get_option_from_user(void)
    {
        short bad_option = 1;


        switch (test_opt) 
        {

		case 'q': mac_test_flag |= PHY_READ_ALL; 	
                  bad_option = 0;
                  break;
		case 'b': mac_test_flag |= PHY_REG_WRITE; 	
                  bad_option = 0;
                  break;
		case 'g': mac_test_flag |= RD_ALL_REGS; 	
                  bad_option = 0;
                  break;
		case 'h': mac_test_flag |= REG_RD; 	
                  bad_option = 0;
                  break;
		case 'y': mac_test_flag |= REG_WR; 	
                  bad_option = 0;
                  break;
		case 'r': mac_test_flag |= RMON_TEST_MASK; 	
                  bad_option = 0;
                  break;
        case 'f': mac_test_flag |= ADR_FILTER_MASK;
                  bad_option = 0;
                  break;
        case 'p': mac_test_flag |= PERPORT_TEST_MASK;
		          bad_option = 0;
                  break;
        case 'a': mac_test_flag |= ANEG_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'c': mac_test_flag |= CRC_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'd': mac_test_flag |= PAD_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'm': mac_test_flag |= MDIO_TEST_MASK;
	              bad_option = 0;
                  break;
        case 'i': mac_test_flag |= I2C_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'v': mac_test_flag |= VLPSE_TEST_MASK;
                  bad_option = 0;
                  break;
        case 's': mac_test_flag |= SPI3_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'w': mac_test_flag |= POWDN_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'e': mac_test_flag |= RESET_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'z': mac_test_flag |= FRSZ_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'l': mac_test_flag |= LED_TEST_MASK;
                  bad_option = 0;
                  break;
        case 'k': mac_test_flag |= WMARK_TEST_MASK;
	              bad_option = 0;
                  break;
        case '?': show_options_help();
                  break;
        default:  
				  //eprintf("bad test option: %c\n",test_opt);
    			  show_options_help();
				  //return INVALID_PARAMETERS;
                  break;
        }

    }

    static void show_options_help()
    {
        short i=0;

        eprintf("Mac test options\n");
        eprintf("----------------\n");
       
        for (i=0;i<aTestHelpSz;i++)
        {
            eprintf("%s\n",aTestHelp[i]);
        }
    }



    UINT32 do_mac_test(void)
	{
		char saddr[5],sval[10];
		short i=0;
		unsigned int address = 0,value=0;	

		switch (mac_test_flag)
		{

	    case PHY_REG_WRITE:
				phy_write();
 				break;

		case PHY_READ_ALL:
				spConfig();
				gbe_mac_phy_read_all();
		        spRestore();
				break;
		case RD_ALL_REGS:
				read_all_regs();
				break;

		case REG_RD:
				spConfig();
				/*** read vallejo registers ***/
  				eprintf("\nEnter Vallejo Reg. offset :");
				for (i=0;i<=4;i++)
					saddr[i] = '\0';
				i=0;
				gets(saddr, 4);
				address=htoi(saddr);
				eprintf("\nReading offset 0x%08X, value = 0x%08X\n",address,reg_read(address));
				strcpy(saddr,"00000");
				spRestore();
				break;
		case REG_WR:
				spConfig();

				/*** read vallejo registers ***/
		        eprintf("\nEnter Vallejo Reg. offset :");
				for (i=0;i<=4;i++)
					saddr[i] = '\0';
				i=0;
				gets(saddr, 4);
				eprintf("\nEnter Value :");
				gets(sval, 9);
				address = htoi(saddr);
				value = htoi(sval);

				reg_write(address,value);
				eprintf("writing offset 0x%x, value = 0x%x\n",address,value);

				strcpy(saddr,"00000");
				strcpy(sval,"000000000");
				spRestore();	
				break;

		case ADR_FILTER_MASK:
				/*
                * invoke MAC address filtering diagnostic tests
                */
                mac_address_filter_test();
                break;
        case SPI3_TEST_MASK:
                /*
                * invoke spi-3 diagnostic tests
                */
				spi3_test();	
                break;

		case RMON_TEST_MASK: 
                /*
                * invoke diagnostic tests for Rx statistic registers.
                */ 
      	        rmon_test(); 
				break;

        case PERPORT_TEST_MASK:
                /*
                * invoke diagnostic tests for per port registers.
                */
                per_port_reg_test(); 
                break;
		case I2C_TEST_MASK:
                /*
                * invoke I2C diagnostic tests
                */
                i2c_test (); 
                break;

        case CRC_TEST_MASK:
                /*
                * invoke CRC diagnostic tests
                */
                crc_test(); 
                break;

        case MDIO_TEST_MASK:
				/*
                * invoke mdio diagnostic tests
                */
                mdio_config_test (); /* mdio config register diagnostics */
                //mdc_speed_test ();   /* mdio clock speed test */
                break;

        case PAD_TEST_MASK:
               /*
               * invoke Rx/Tx fifo padding diagnostic tests
               */
               padding_test(); 
               break;

        case VLPSE_TEST_MASK:
               /*
               * invoke VLAN and PAUSE frame diagnostic tests
               */
               vlan_pause_filter_test(); 
               break;

        case POWDN_TEST_MASK:
               /*
               * invoke fifo powerdown diagnostic tests
               */
               powerDown_test(); 
               break;

        case RESET_TEST_MASK:
               /*
               * invoke reset diagnostic tests
               */
               reset_test(); 
               break;
        case FRSZ_TEST_MASK:
               /*
               * invoke frame size diagnostic tests
               */
               frame_size_test(); 
               break;

        case ANEG_TEST_MASK:
			   /*
               * invoke auto-negotiation diagnostic tests
               */

               auto_neg_test(); 
               break;

		case LED_TEST_MASK:
		      /*
			  * invoke LED diagnostic tests
			  */
              led_test(); 
              break;

        case WMARK_TEST_MASK:
              /*
               * invoke fifo watermark diagnostic tests
               */
              wmark_test(); 
              break;

		} /* end of switch */
	
		/* reset the mac test flag */
        mac_test_flag = 0;
        return DONE;
	}

UINT32 validate_params()
{
	short fault=0;
	
	switch(channel)
	{
		case 'f':
					if (speed != 1000)
					{
						eprintf("Invalid speed parameter for fiber channels. It should be 1000.\n");
						fault = 1;
					}
					break;

		case 'c':
					if ((speed != 10) && (speed != 100) && (speed != 1000))
					{
						eprintf("Invalid speed selected for copper channels\n");
						fault = 1;
					}
					break;

		default:
					eprintf("Invalid Channel Mode\n");
					show_options_help();
					fault = 1;
					break;
	}

	if ((ports < 0) || (ports > 4))
	{
		eprintf("Invalid port number\n");
		fault = 1;
	}

	if (fault)
		return INVALID_PARAMETERS;

	if (loop == 0)
	{
		eprintf("Loopback test executed for infinite packets..\n");
		eprintf("Please press a key to stop test ..\n");
	}
	return 0;
}
					
UINT32 assign_pkt_pointers(UINT32 ports, UINT32 *pktptr, UINT32 cflag, UINT32 *pnum)
{
	short iteration=0;

	switch(ports)
	{
	case 0: pktptr = tx_data_pl_0;
			*pnum = PKT_SEQ_SZ/4; 
			iteration = 1;
			break;
	case 1: pktptr = tx_data_pl_1;
			*pnum = PKT_SEQ_SZ/4;
			iteration = 1;
			break;
	case 2: pktptr = tx_data_pl_2;
			*pnum = PKT_SEQ_SZ/4;
			iteration = 1;
			break;
	case 3: pktptr = tx_data_pl_3;
			*pnum = PKT_SEQ_SZ/4;
			iteration = 1;
			break;
	case 4: pktptr = tx_data_pl;
			*pnum = ALL_PORT_PKT_SEQ_SZ/(4*4);
			if (cflag)
				iteration = 2;
			else
				iteration = 4;
			break;
	}

	return iteration;
}


short check_autoneg_status()
{
	short s_PortCount = 0, flag = OK, ctr = 0, done = 0;
	UINT32 regval = 0, link=0,receiver_status=0, autoneg_complete=0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	short port;
	
	port = acL->decArg[6];
	
	eprintf("Waiting for auto-negotiation to complete ....\n");
	for(s_PortCount=0; s_PortCount<4; s_PortCount++)
    {		
		if ((s_PortCount == port) || (port == 4))
		{
			do
			{	
				switch(speed)
				{
				case 1000:
						regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x11);
						link = (link >> 12) & 0x1;
						receiver_status = (regval >> 12) & 0x3;
						//eprintf("receiver status = 0x%x\n",receiver_status);
						if (receiver_status == 0x3)
							done = 1;
						break;
				case 100:
						//regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x01);
						link = (link >>2) & 1;
						done = 1;
						break;
				case 10:
						//regval = gbe_mac_phy_read(s_PortCount,0xA);
						link = gbe_mac_phy_read(s_PortCount,0x16);
						done = gbe_mac_phy_read(s_PortCount,0x01);
						link = (link >> 6) & 1;
						done = (done >> 2) & 1;
						break;
				}
				//eprintf("Regval for port %d  = 0x%08X\n",s_PortCount,regval);	
				//receiver_status = (regval >> 12) & 0x3;
				autoneg_complete = gbe_mac_phy_read(s_PortCount,0x1);
				if ((((autoneg_complete >> 5) & 1) == 0x1) && (link == 1) && done)
				{
					flag = OK;
					break;
				}
				else
				{	
					ctr ++;
					flag = ERROR;
				}
			}while (ctr < 200);
		
			if (flag == ERROR)
			{
				eprintf("Auto-negotiation failed for port %d , link = 0x%08X, auto = 0x%08X\n",s_PortCount,link, autoneg_complete);
					
			}
		}
	}

	return flag;
}

UINT32 cu_init(UINT32 set_int_lpbk)
{
	int port;

	if (config_phy() == ABORT)
			return ABORT;

   /* apply reset to MAC device */
    mac_reset_config();

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);

    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(PORT_ENABLE,0x0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(INTERFACE_MODE,0xF);		/** enable copper mode **/
	switch(dplx)
	{
		case 'h' :      
				reg_write(DESIRED_DUPLEX,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x0);
				reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x0);
				break;
		case 'f' :
				reg_write(DESIRED_DUPLEX,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT1,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT2,0x1);
				reg_write(DESIRED_DUPLEX+INDEX_PORT3,0x1);
				break;
		default : 
				eprintf("Illegal duplex optinon.\n");
				eprintf("The opertation will be full duplex..\n");
				break;
	}			
	
	reg_write(SERDES_CLKMODE_REG,0xF);
	reg_write(SPI3_TX_GLOBAL_CONFIG,0x20aa0f);  
	reg_write(SPI3_RX_CONFIG, 0xAA0F80);

	reg_write(DIV_CONFIG_WORD,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT1,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT2,0x114D);  
	reg_write(DIV_CONFIG_WORD+INDEX_PORT3,0x114D); 

	reg_write(0x5B3,0xF0);

	reg_write(SERDES_CLKMODE_REG,0x0);
	if (speed == 100)
	{
	    reg_write(MAC_IF_MODE,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x5);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x5);  
	}

	if (speed == 10)
	{
	    reg_write(MAC_IF_MODE,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x4);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x4);  
	}
	
	if (speed == 1000)
	{
	    reg_write(MAC_IF_MODE,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT1,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT2,0x6);  
	    reg_write(MAC_IF_MODE+INDEX_PORT3,0x6);  
	}
	reg_write(SERDES_CLKMODE_REG,0xF);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);

	reg_write(TXFIFO_PAD_ENB,0xAAAA);  
	reg_write(SERDES_TX_RX_POW_DOWN,0x0); 

	mac_config();
	reg_write(GBIC_CTRL_REG,0x1C00);    

	reg_write(LED_MODE, 0x3);

	// Activate units
	reg_write(0x505, 0x0);		// MAC Soft reset 
	reg_write(0x506, 0x0);		// MDIO Reset
	reg_write(0x620, 0x0);		// TX FIFO Reset
	reg_write(0x59E, 0x0);		// Rx FIFO Reset

	//mac_disable_reset();

	reg_write(SERDES_CLKMODE_REG,0x0);
	reg_write(SERDES_CLKMODE_REG,0xf);
    reg_write(0x505,0xf);
	reg_write(0x505,0x0);
	reg_write(PORT_ENABLE,0xF);

	// fix for CRC error @ 1G
	for(port = 0; port < 4; port++)
    {
        gbe_mac_phy_write(port, 22, 0x2018);
		gbe_mac_phy_write(port, 24, 0x3248);
    }

	return DONE;
}
