/* addr_filter.c
 *
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: chinmay, NOVEMBER, 2002
 * 
 * 
 * ------------------------------------------------------------------------------
 */

typedef unsigned long ulong;
typedef unsigned short ushort;

#include "common.h"
#include "memory.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "syslog.h"
#include "uart.h"
#include "msf_init.h"
#include "diag_utils.h"

#include "error_code.h"
#include "error_map.h"
#include "reg_api.h"
#include "register_map.h"
#include "diag.h"
#include "dp_proto.h"
#include "mac_util.h"

#include "pl_sys_loopback_eg_rx.h"
#include "pl_sys_loopback_eg_tx.h"
#include "pl_sys_loopback_in_rx.h"
#include "pl_sys_loopback_in_tx.h"

#include "mphy_loopback_eg_rx.h"
#include "mphy_loopback_eg_tx.h"
#include "mphy_loopback_in_rx.h"
#include "mphy_loopback_in_tx.h"

#define UNI_PKTFILCTRL	0x1
#define MUL_PKTFILCTRL	0x2
#define BRD_PKTFILCTRL	0x4
#define HI_MUL_ADDR	0xA1B0C0D0
#define LO_MUL_ADDR	0xE0F0
#define MPHY 1
#define UC_CHECK	0x00			/* This is the 1st byte of the unicast frame */
#define MC_CHECK	0xB1			/* This is the 1st byte of the multicast frame */
#define BC_CHECK	0xFF			/* This is the 1st byte of the broadcast frame */

extern const int pl_sys_loopback_eg_rx[];
extern const int pl_sys_loopback_eg_tx[];
extern const int pl_sys_loopback_in_rx[];
extern const int pl_sys_loopback_in_tx[];

extern const int mphy_loopback_eg_rx[];
extern const int mphy_loopback_eg_tx[];
extern const int mphy_loopback_in_rx[];
extern const int mphy_loopback_in_tx[];


UINT32 Sz_pl_sys_loopback_eg_rx= sizeof(pl_sys_loopback_eg_rx);
UINT32 Sz_pl_sys_loopback_eg_tx= sizeof(pl_sys_loopback_eg_tx);
UINT32 Sz_pl_sys_loopback_in_rx= sizeof(pl_sys_loopback_in_rx);
UINT32 Sz_pl_sys_loopback_in_tx= sizeof(pl_sys_loopback_in_tx);

UINT32 Sz_mphy_loopback_eg_rx= sizeof(mphy_loopback_eg_rx);
UINT32 Sz_mphy_loopback_eg_tx= sizeof(mphy_loopback_eg_tx);
UINT32 Sz_mphy_loopback_in_rx= sizeof(mphy_loopback_in_rx);
UINT32 Sz_mphy_loopback_in_tx= sizeof(mphy_loopback_in_tx);

extern UINT32 data64_0[];
extern UINT32 data64_1[];
extern UINT32 data64_2[];
extern UINT32 data64_3[];
extern UINT32 data96_0[];
extern UINT32 data96_1[];
extern UINT32 data96_2[];
extern UINT32 data96_3[];
extern UINT32 data138_0[];
extern UINT32 data138_1[];
extern UINT32 data138_2[];
extern UINT32 data138_3[];

extern pci_device slave_dev_info;		// for slave npu info.
extern UINT32 PKT_96_SZ,PKT_64_SZ,PKT_138_SZ;
extern UINT32 PKT_SEQ_SZ,ALL_PORT_PKT_SEQ_SZ;

extern  UINT32 *pl_sys_lb_data96_0;
extern  UINT32 *pl_sys_lb_data64_0;
extern  UINT32 *pl_sys_lb_data138_0;
extern  UINT32 *pl_sys_lb_data96_1;
extern  UINT32 *pl_sys_lb_data64_1;
extern  UINT32 *pl_sys_lb_data138_1;
extern  UINT32 *pl_sys_lb_data96_2;
extern  UINT32 *pl_sys_lb_data64_2;
extern  UINT32 *pl_sys_lb_data138_2;
extern  UINT32 *pl_sys_lb_data96_3;
extern  UINT32 *pl_sys_lb_data64_3;
extern  UINT32 *pl_sys_lb_data138_3;

extern UINT32 tx_data_pl[];
extern UINT32 tx_data_pl_0[];
extern UINT32 tx_data_pl_1[];
extern UINT32 tx_data_pl_2[];
extern UINT32 tx_data_pl_3[];

#define UC_PKTS			3	 /* expected unicast packets, per port */
#define MC_PKTS			3	 /* expected multicast packets, per port */
#define BC_PKTS			3	 /* expected broadcast packets, per port */
#define EXP_DROPPED_PKTS	3	 /* expected dropped packets, per port */

/***************************************************************************
** The rxport is to be indexed by the txport and the value is the corresponding rxport.
** Tx port	  Rxport
**	0			1
**  1			0
**  2			3
**  3			2
*********************************************************************************/
UINT32 rxport[4] = {1, 0, 3, 2};
short ucCount[4],mcCount[4],bcCount[4];

/** globals **/
extern UINT32 total_args,loop,ports,speed;  
extern char channel,test_opt;
extern UINT8 test_param;

extern UINT32 validate_params(void);
extern UINT32 init_mac_dev(UINT32);
extern void spConfig(void);
extern void spRestore(void);
extern void get_stats(void);


/*  local functions */
short verify_filter(short);
void get_packets_for_filter_test();
void clear_stat_regs();
void Slave_pl_SysLpbk_test(int param);

// ******************************************************************************
// Function: void mac_address_filter_test(void)
//
// This function is called by the Test Manager when the user invokes the
// point Lone address filtering test.
// ******************************************************************************
UINT32 mac_address_filter_test(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, tx_count=0, rx_count=0;
	UINT32 scratch_ring_tx_count, scratch_ring_rx;
	UINT32 dcache_status, forever = 0, combination, port_num, rx_port_num;
	UINT32 loop_count=0;
	UINT32 *pl_sys_lb_data=0;
	UINT32 scratch_read_value=0, dram_addr, orig_data_size=0;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 pass_port[4],rxflag[4];
	UINT32 expected_num_pkts = 0,iteration = 0, port_ctr=0, pnum=0, *pktptr=0;
	UINT32 exp_pkt_sz = 0;
	UINT32 bad_pkt_size = 0;
	UINT32 test_passed=0,counter=0;
	short cflag = 0;
	
    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	test_param = acL->argv[3][0];
	channel = acL->argv[4][0]; 		   	
	speed = acL->decArg[5];	
	ports = acL->decArg[6];
	loop = acL->decArg[7];

	if (validate_params() == INVALID_PARAMETERS)
	{
		eprintf("Invalid parameters. \n");
		return INVALID_PARAMETERS;
	}

	if (channel == 'c')
		cflag = 1;
	else
		cflag = 0;

	if (total_args <= 7)
	{
		loop = 1;
	}

	if (loop == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}


	/* initialize port array to 0 */
	for (ctr = 0;ctr<4;ctr++)
	{
		pass_port[ctr] = 0;
		rxflag[ctr] = ERROR;
		/* initialize packet counts to 0 */
		ucCount[ctr]=0;
		mcCount[ctr]=0;
		bcCount[ctr]=0;

	}

	/* get the packet data used specifically for the test */
	get_packets_for_filter_test();

	switch(ports)
	{
		case 0: pktptr = tx_data_pl_0;
				pnum = PKT_SEQ_SZ/4; 
				iteration = 1;
				break;
		case 1: pktptr = tx_data_pl_1;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 2: pktptr = tx_data_pl_2;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 3: pktptr = tx_data_pl_3;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 4: pktptr = tx_data_pl;
				pnum = ALL_PORT_PKT_SEQ_SZ/(4*4);
				if (cflag)
					iteration = 2;
				else
					iteration = 4;
				break;
	}


	scratch_ring_rx = SCR_RING_INSTR_ADDR + (5 << 2);
	scratch_ring_tx_count = SCR_RING_INSTR_ADDR + (4 << 2);

	if(acL->HostType == MASTER)		// if master NPU
	{
		flag = OK;

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		eprintf("Initialising media card. Please wait...\n");
		status = init_mac_dev(0);	
		if (status != DONE)
		{
			eprintf("Aborting Test\n");
			return ABORT;
		}
		
		clear_stat_regs();
		spConfig();
#if 0
		/* clear drop pkt counters */
		reg_read(0x5A2);
		reg_read(0x5A3);
		reg_read(0x5A4);
		reg_read(0x5A5);
#endif

		switch(test_param)
		{
		case 'u':
				reg_write((RXFIFO_DROP_ENABLE),0xF);  
				for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
				{
					/*forward only unicast packets */
					reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),UNI_PKTFILCTRL); 
				}
				break;
		case 'm':
			/* bad packets to be dropped */
			reg_write((RXFIFO_DROP_ENABLE),0xF);  
			for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
			{
				/* forward only multicast packets */
				reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),MUL_PKTFILCTRL); 
			}
			
			break;
		case 'b':
			/* enable the bad packets to be dropped */
			reg_write((RXFIFO_DROP_ENABLE),0xF);  
			for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
			{
				/* forward only broadcast packtes */
				reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),BRD_PKTFILCTRL);
			}
			reg_write((PORT_ENABLE),0xF);
			break;

		case '1': /* This case handles the RMON stat reg test */
				break;

		default :
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}

		spRestore();
		eprintf("Media card initialisation completed.\n");

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		PUT32(slave_mailbox0_addr, 0);

#ifdef __ARMEB__
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif

		PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), DRAM_BASE_SRC);
		// Write packet data to DRAM for microcode to use
		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_3[(ctr / 4)]);
		}

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

		// Invoke the slave's PCI doorbell
		Set_Pci_Doorbell(PTLONE_SYS_LPBK, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		ctr = 0;

		// Wait for a signal that slave NPU has completed loading of ucode and init
		do
		{
			if (ctr == 1500)	// After 10 seconds
			{
				eprintf("\nTimed out...Slave NPU was not able to load microcode and init.\n");
				mailbox_value = INIT_ERROR;
				flag = ERROR;
				break;
			}

			temp = GET32(slave_mailbox0_addr);

			hal_delay_us(10000);
			ctr++;
		}
		while (temp == 0);
		
		if (flag == OK)	// If slave completed microcode loading successfully
		{
			// Load Master's NPU with ucode and init
			DownLoadUcode(PL_SYS_LOOPBACK_EG_TX_ME, sizeof(pl_sys_loopback_eg_tx), (UINT32 *)pl_sys_loopback_eg_tx);
			DownLoadUcode(PL_SYS_LOOPBACK_EG_RX_ME, sizeof(pl_sys_loopback_eg_rx), (UINT32 *)pl_sys_loopback_eg_rx);

			hal_delay_us(1000000);

			//MSG("DEBUG: (sizeof(tx_data_pl) / sizeof(UINT32)) / 4 = %d\n", ((sizeof(tx_data_pl) / sizeof(UINT32)) / 4),0,0);

			do
			{
				//for (combination = 0; combination < ((sizeof(tx_data_pl) / sizeof(UINT32)) / 4); combination++)
				for (combination = 0; combination < pnum; combination++)
				{
					//for (port_num = 0; port_num < 4; port_num++)
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
						{
							port_num = ports;
							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), pktptr[combination]);
						}
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;

							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_pl[((combination * 4) + port_num)]);
						}
					}
				}

//				data_to_send = tx_data_pl[internal_loop_count];
//				MSG("DEBUG: internal_loop_count = %d\n", internal_loop_count,0,0);
//				MSG("DEBUG: data_to_send = 0x%08X\n", data_to_send,0,0);
//				PUT32(scratch_ring_tx, data_to_send);

				for (combination = 0; combination < pnum; combination++)
				{
					flag = OK;

					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
							port_num = ports;
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;
						}
						//MSG("iteration = %d, port = %d\n",iteration,port_num,0);

						ctr = 0;

						do			// Do while waiting for all the packets to be transmitted
						{
							if (ctr == 1000)	// After 1 second
							{
								eprintf("\nTimed out...No indication from scratch ring that all packets were transmitted\n");
								mailbox_value = NOT_SENT;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if the packets have been transmitted
							if ((scratch_read_value = GET32(scratch_ring_tx_count)) == 0)
							{
								hal_delay_us(2);
								ctr++;
							}
							else
							{
								switch(ports)
								{
									case 0 : 
									case 1 : 
									case 2 : 
									case 3 : 
											eprintf("Transmitted on port %d: %d         \r", ports,(scratch_read_value));
											
											break;
    
									case 4 : // all ports 
											if (!cflag)
											{
												eprintf("Transmitted on ports 0-3 : %d        \r", (scratch_read_value)/4);
												iteration = 4;
											}
											else
											{
												eprintf("Transmitted on ports 0 and 2 : %d\r", (scratch_read_value)/2);
												iteration = 2;
											}
											break;
								}
								MSG("Transmitted Packets (total): %d\r", scratch_read_value,0,0);
								tx_count++;
 
							}
						}
						while (scratch_read_value == 0);

						PUT32(scratch_ring_tx_count, 0);

						tx_count =  scratch_read_value;
						if (flag == ERROR)
						{
							break;
						}
					}

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}
				
				for (combination = 0; combination < pnum; combination++)
				{
	
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (!cflag)
						{
							if (ports != 4)
								port_num = ports;
							else
								port_num = port_ctr;
						}
						else
						{
							if (ports != 4)
								port_num = rxport[ports];
							else
								port_num = port_ctr*2 + 1;
						}


						ctr = 0;

						do			// Do while waiting for a signal that a packet has been received
						{
							if (ctr == 300)	// After 1 second
							{
								//eprintf("\nTimed out...No indication from scratch ring that all packets were received\n");
								//mailbox_value = NOT_RECEIVED;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if packets have been received
							if ((scratch_read_value = GET32(scratch_ring_rx)) == 0)
							{
								hal_delay_us(100);
								ctr++;
							}

							//MSG("port %d  scratch_read_value = 0x%x\n",port_num,scratch_read_value,0);
						}
						while (scratch_read_value == 0);

						if (flag == ERROR)
						{
							break;
						}

						//MSG("DEBUG: scratch_read_value = 0x%08X\n", scratch_read_value,0,0);

						rx_count++;
						pkt_size = scratch_read_value & MASK_8BIT;
						rx_port_num = (scratch_read_value >> 8) & MASK_8BIT;
						dram_addr = DRAM_BASE_DEST + ((scratch_read_value >> 16) << 8);
						// set the port on which packet was received.
						pass_port[rx_port_num]++;
						rxflag[rx_port_num] = OK;

						//MSG("port : %d   pkt size = 0x%x\n",rx_port_num, pkt_size,0);
						
						//MSG("DEBUG: pkt_size=0x%08X\tdram_addr=0x%08X\n\n", pkt_size, dram_addr,0);

						//MSG("DEBUG: pkt_size=%d\ttx_data_pl=%d\n", pkt_size, (tx_data_pl[((combination * 4) + port_num)] & 0xFFFF),0);

#if 0
					/* Vallejo WORKAROUND : Vallejo is not stripping of the CRC 
											for 96 byte packets in gig/half duplex mode */
						if ((pkt_size == 0x60) || (pkt_size == 0x40) || (pkt_size == 0x8A))
						{
							eprintf("\npacket size = 0x%x\n",pkt_size);
							pkt_size = pkt_size - 4;
						}
						/****** end of Workaround *******/
#endif

						if (ports == 4)
						{
							exp_pkt_sz = tx_data_pl[((combination * 4) + port_num)] & 0xFFFF;
						}
						else
						{
							exp_pkt_sz = pktptr[combination] & 0xFFFF;
						}
						switch(test_param)
						{
						case 'u':
								/* 64 byte packets are unicast packets.
								   These are dropped if unicast filtering is on,
							       as the destination address is different from
							       the station address for this test.
								   Next expected packet is a 138 byte frmae.*/
								if (exp_pkt_sz == 0x3C)
									exp_pkt_sz = 0x84;
								break;
						case 'm':
								break;

						case 'b':
								/* 96 byte packets are broadcast packets.
								   These are dropped if broadcast filtering is on,
								   Next expected packet is a 64 byte frmae. */
								if (exp_pkt_sz == 0x5C)
									exp_pkt_sz = 0x3C;
								else
									exp_pkt_sz = 0x84;
								break;
						}

						if (pkt_size != exp_pkt_sz)
						{
							eprintf("\nActual packet size = 0x%x\n",pkt_size);
							eprintf("Expected packet size: 0x%x\n",exp_pkt_sz);
							mailbox_value = INCORRECT_SIZE;
							flag = INCORRECT_SIZE;		// Set flag as error
							rxflag[rx_port_num] = ERROR;
							break;
						}

						if (cflag)
						{
							switch(rx_port_num)
							{
								case 0:  // rx on port 0 , check for pkt transmitted from port 1
									if (pkt_size == (RING_DATA_96_1 & MASK_16BIT) )
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
									}
									else if (pkt_size == (RING_DATA_64_1 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
									}
									else if (pkt_size == (RING_DATA_138_1 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
									}
									else
										bad_pkt_size = 1;
									break;

								case 1:  // rx on port 1 , check for pkt transmitted from port 0
									if (pkt_size == (RING_DATA_96_0 & MASK_16BIT))
									{
										//eprintf("received 96 byte on port 1\n");
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
									}
									else if (pkt_size == (RING_DATA_64_0 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
									}
									else if (pkt_size == (RING_DATA_138_0 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
									}
									else 
										bad_pkt_size = 1;
									break;
								case 2:  // rx on port 2 , check for pkt transmitted from port 3
									if (pkt_size == (RING_DATA_96_3 & MASK_16BIT))
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
									}
									else if (pkt_size == (RING_DATA_64_3 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
									}
									else if (pkt_size == (RING_DATA_138_3 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
									}
									else
										bad_pkt_size = 1;

									break;
								case 3:  // rx on port 2 , check for pkt transmitted from port 2
									if (pkt_size == (RING_DATA_96_2 & MASK_16BIT))
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
									}
									else if (pkt_size == (RING_DATA_64_2 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
									}
									else if (pkt_size == (RING_DATA_138_2 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
									}
									else 
										bad_pkt_size = 1;

									break;
							}
							if (bad_pkt_size ==1)
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
							
						}
						else  // cflag = 0 -> fiber mode
						{

							if ((pkt_size == (RING_DATA_96_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
							}
							else if ((pkt_size == (RING_DATA_64_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
							}
							else if ((pkt_size == (RING_DATA_138_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
							}
							else if ((pkt_size == (RING_DATA_96_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
							}
							else if ((pkt_size == (RING_DATA_64_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
							}
							else if ((pkt_size == (RING_DATA_138_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
							}
							else if ((pkt_size == (RING_DATA_96_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
							}
							else if ((pkt_size == (RING_DATA_64_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
							}
							else if ((pkt_size == (RING_DATA_138_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
							}
							else if ((pkt_size == (RING_DATA_96_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
							}
							else if ((pkt_size == (RING_DATA_64_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
							}
							else if ((pkt_size == (RING_DATA_138_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
							}
							else
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
						}

						for (ctr = 0; ctr < orig_data_size; ctr+=4)
						{
							// Read the DRAM for the longword that was received
							temp = GET32(dram_addr + ctr);
							
							// Compare with original value
							if (temp != pl_sys_lb_data[(ctr / 4)])
							{
								mailbox_value = ERROR_PATTERN;
								eprintf("Packet:%d Longword:%d Expected:0x%X  Received:0x%X\n", (rx_count + 1), (ctr / 4), pl_sys_lb_data[(ctr / 4)], temp);
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
							if (ctr == 0)
							{
								//eprintf("1st LW : 0x%x\n",temp);
								if (((temp >>24) & 0xFF)== UC_CHECK)
									ucCount[rx_port_num]++;  /* unicast frame received */
								if (((temp >>24) & 0xFF)== MC_CHECK)
									mcCount[rx_port_num]++;  /* multicast frame received */
								if (((temp >>24) & 0xFF) == BC_CHECK)
									bcCount[rx_port_num]++;  /* broadcast frame received */
							}

						}

						if (flag == ERROR)
						{
							break;
						}

						hal_delay_us(10);
					}
					if (flag == INCORRECT_SIZE)
						break;
					if (flag == ERROR)
					{
						if ((test_param == 'u') || (test_param == 'b') || (test_param == 'm'))
							;
						else
						{
							break;
						}
					}
				}
				if (flag == INCORRECT_SIZE)
					break;
				if (flag == ERROR)
				{
					if ((test_param == 'u') || (test_param == 'b')|| (test_param == 'm') )
							;
					else
					{
						break;
					}
				}

	
				  loop_count++;

				if (forever && KEYPRESS)
				{
					break;
				}
				//eprintf("Loop count = %d\r",loop_count);
			}
			while ((loop_count != loop) || forever);
			
			if (loop == 0)
			{
				counter = loop_count;
			}
			else
			{
				counter = loop;
			}

		    switch(test_param)
			{
			case 'u': expected_num_pkts = counter*6;
				      break;
			case 'm': expected_num_pkts = counter*6;
				      break;
			case 'b': expected_num_pkts = counter*6;
				      break;
			case '1': expected_num_pkts = counter*9;
				      break;
			}
			MSG("\nexpected_num_pkts = %d\n",expected_num_pkts,0,0);

			switch(ports)
			{
		       case 0:
		       case 1:
		       case 2:
		       case 3:  if (!cflag) 
						{ 
							if ((pass_port[ports] != expected_num_pkts) || (pass_port[ports] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", ports);
								eprintf("Packets Received = %d\n",pass_port[ports]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",ports,pass_port[ports]);
								flag = OK;
							}
						}
						else
						{
							if ((pass_port[rxport[ports]] != expected_num_pkts) || (pass_port[rxport[ports]] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", rxport[ports]);
								eprintf("Packets Received on port %d = %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = OK;
							}
						}
						break;

		       case 4:  eprintf("\n");
						if (!cflag)
						{
							for (ctr=0; ctr<4; ctr++)
							{
								if ((pass_port[ctr] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr);
									eprintf("Packets Received = %d\n",pass_port[ctr]);
									rxflag[ctr] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr],ctr);
								}
							}
						}
						else
						{
							for (ctr=0; ctr<2; ctr++)
							{
								if ((pass_port[ctr*2+1] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr*2+1);
									eprintf("Packets Received = %d\n",pass_port[ctr*2+1]);
									rxflag[ctr*2+1] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr*2+1] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr*2+1],ctr*2+1);
								}
							}
						}
						
						break;
						
			   default:   eprintf("This is bad value for port number \n");
								return INVALID_PARAMETERS;
			}

		}
		eprintf("\n");

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}

		if (loop == 0)
			loop = loop_count;
		test_passed=verify_filter(cflag);
	
		if (test_passed)
		{
			switch(test_param)
			{
			case 'u':
				eprintf("Unicast MAC filtering test...PASSED\n");
				break;
			case 'm':
				eprintf("Multicast MAC filtering test...PASSED\n");
				break;
			case 'b':
				eprintf("Broadcast MAC filtering test...PASSED\n");
				break;
			case '1':
				eprintf("RMON test for the following Registers ...PASSED\n");
				eprintf("RxOctetsTotalOK / OctetsTransmittedOK\n");
				eprintf("RxOctetsBAD / OctetsTransmittedBad\n");
				eprintf("RxUCPckts / TxUCPkts\n");
				eprintf("RxMCPckts / TxMCPkts\n");
				eprintf("RxBCPckts / TxBCPkts\n");
				eprintf("RxPkts64Octets / TxPkts64Octets\n");
				eprintf("RxPkts65to127Octets / TxPkts65to127Octets\n");
				eprintf("RxPkts128to255Octets / TxPkts128to255Octets\n");
				break;

			default:
				eprintf("Invalid parameters. \n");
				return INVALID_PARAMETERS;
			}

			Set_LED("MAFP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		} 
		else
		{
			if (mailbox_value == INIT_ERROR)
			{
				eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
			}
			else if (mailbox_value == NOT_RECEIVED)
			{
				eprintf("There was no sign that the expected packet was received\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The packet received was not the same as the packet transmitted.\n");
			}
			else if (mailbox_value == INCORRECT_SIZE)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The packet received was not the same size as the packet transmitted.\n");
			}
				
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave

			switch(test_param)
			{
			case 'u':
				eprintf("Unicast MAC filtering test...FAILED\n");
				break;
			case 'm':
				eprintf("Multicast MAC filtering test...FAILED\n");
				break;
			case 'b':
				eprintf("Broadcast MAC filtering test...FAILED\n");
				break;
			case '1':
				eprintf("RMON test for packet count Registers ...FAILED\n");
			}

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			Set_LED("LswF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		

		Clear_All_ME();
		Msf_OutOfReset();

	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}
	return DONE;
}


// ******************************************************************************
// Function: void Slave_pl_SysLpbk_test(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
void Slave_pl_SysLpbk_test(int param)
{
	UINT32 status, scratch_ring_addr, scratch_val, ctr=0, verbose;

	scratch_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
	verbose = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	// Load ME with microcode
	if (param != 1)
	{
		eprintf("Downloading SPHY code..\n");
		DownLoadUcode(PL_SYS_LOOPBACK_IN_RX_ME, sizeof(pl_sys_loopback_in_rx), (UINT32 *)pl_sys_loopback_in_rx);
		DownLoadUcode(PL_SYS_LOOPBACK_IN_TX_ME, sizeof(pl_sys_loopback_in_tx), (UINT32 *)pl_sys_loopback_in_tx);
	}
	else
	{
		eprintf("Downloading MPHY code\n");
		DownLoadUcode(PL_SYS_LOOPBACK_IN_RX_ME, sizeof(mphy_loopback_in_rx), (UINT32 *)mphy_loopback_in_rx);
		DownLoadUcode(PL_SYS_LOOPBACK_IN_TX_ME, sizeof(mphy_loopback_in_tx), (UINT32 *)mphy_loopback_in_tx);
	}

	hal_delay_us(1000000);

	do
	{
		if ((scratch_val = GET32(scratch_ring_addr)))
		{
			hal_delay_us(1);
			eprintf("Packets (total): %d\r", scratch_val);
			ctr = 0;
		}
		else
		{
			hal_delay_us(10);
			ctr++;
		}
	}
	while (ctr != 500000);

	eprintf("\n");

	Clear_All_ME();
	Msf_OutOfReset();
}

short verify_filter(short cflag)
{
	UINT32 rxucpkts[4],rxmcpkts[4],rxbcpkts[4],dropped[4];
	UINT32 rxtotalOctets[4],rxBadoctets[4],rx64[4],rx65to127[4],rx128to255[4];
	UINT32 txucpkts[4],txmcpkts[4],txbcpkts[4];
	UINT32 txtotalOctets[4],txBadoctets[4],tx64[4],tx65to127[4],tx128to255[4];

	short test_passed = 1,iteration,port,port_ctr;
	short rxok=0,txok=0,res=0;


	if (ports == 4)
		if (!cflag)
			iteration = 4;
		else
			iteration = 2;
	else
		iteration = 1;

	spConfig();
    for(port=0; port<4;port++)
    {
		/* Reading Rx statistics */
		rxucpkts[port] = reg_read(((port*PORTSZ)+RX_UC_PKTS));  
		rxmcpkts[port] = reg_read(((port*PORTSZ)+RX_MC_PKTS));  
		rxbcpkts[port] = reg_read(((port*PORTSZ)+RX_BC_PKTS));  
		dropped[port] = reg_read(port+PORT0_RXFIFO_PKT_DROP);  
		rxtotalOctets[port] = reg_read(((port*PORTSZ)+RX_OCTETS_OK));  
		rxBadoctets[port] = reg_read(((port*PORTSZ)+RX_OCTETS_BAD));  
		rx64[port] = reg_read(((port*PORTSZ)+RX_PKTS_64));  
		rx65to127[port] = reg_read(((port*PORTSZ)+RX_PKTS_65_127));  
		rx128to255[port] = reg_read(((port*PORTSZ)+RX_PKTS_128_255));  
		/* Reading Tx statistics */
		txucpkts[port] = reg_read(((port*PORTSZ)+TX_UC_PKTS));  
		txmcpkts[port] = reg_read(((port*PORTSZ)+TX_MC_PKTS));  
		txbcpkts[port] = reg_read(((port*PORTSZ)+TX_BC_PKTS));  
		txtotalOctets[port] = reg_read(((port*PORTSZ)+TX_OCTETS_OK));  
		txBadoctets[port] = reg_read(((port*PORTSZ)+TX_OCTETS_BAD));  
		tx64[port] = reg_read(((port*PORTSZ)+TX_PKTS_64));  
		tx65to127[port] = reg_read(((port*PORTSZ)+TX_PKTS_65_127));  
		tx128to255[port] = reg_read(((port*PORTSZ)+TX_PKTS_128_255));  

	}
	spRestore();

	for(port_ctr=0; port_ctr<iteration;port_ctr++)
	{
		if ((!cflag) && (ports == 4))
			port = port_ctr;
		if ((cflag) && (ports == 4))
		{
			port = port_ctr*2;
			port = rxport[port];
		}
		if ((!cflag) && (ports != 4))
			port = ports;
		if ((cflag) && (ports != 4))
			port = rxport[ports];

		eprintf("PORT = %d\n",port);

		if ( (rxucpkts[port] == 0) ||
			 (rxmcpkts[port] == 0) ||
			 (rxbcpkts[port] == 0) )
		{
			test_passed = 0;
		}

		switch (test_param)
		{
		case 'u':
		
				eprintf("RX UC pkts = %d on port %d\n",rxucpkts[port],port);
				eprintf("RX MC pkts = %d on port %d\n",rxmcpkts[port],port);
				eprintf("RX BC pkts = %d on port %d\n",rxbcpkts[port],port);
				eprintf("Dropped UC pkts = %d on port %d\n",dropped[port],port);

				if ((ucCount[port] != 0) || 
					(mcCount[port] != (loop*MC_PKTS)) ||
					(bcCount[port] != (loop*BC_PKTS)) ||
					(dropped[port] != (loop *EXP_DROPPED_PKTS)))
				{
					eprintf("Unicast packet filtering test ERROR , Port %d\n",port);
					test_passed = 0;
				}
				else 
					res|= 1<<port_ctr;
				

		break;
		case 'm':
				eprintf("RX UC pkts = %d on port %d\n",rxucpkts[port],port);
				eprintf("RX MC pkts = %d on port %d\n",rxmcpkts[port],port);
				eprintf("RX BC pkts = %d on port %d\n",rxbcpkts[port],port);
				eprintf("Dropped MC pkts = %d on port %d\n",dropped[port],port);

				if ((ucCount[port] != (loop*UC_PKTS)) || 
					(mcCount[port] != 0) ||
					(bcCount[port] != (loop*BC_PKTS)) ||
					(dropped[port] != (loop *EXP_DROPPED_PKTS)))
				{
					eprintf("Multicast packet filtering test ERROR , Port %d\n",port);
					test_passed = 0;
				}				
				else
					res|= 1<<port_ctr;
				
				break;
		case 'b':
				eprintf("RX UC pkts = %d on port %d\n",rxucpkts[port],port);
				eprintf("RX MC pkts = %d on port %d\n",rxmcpkts[port],port);
				eprintf("RX BC pkts = %d on port %d\n",rxbcpkts[port],port);
				eprintf("Dropped BC pkts = %d on port %d\n",dropped[port],port);
	
				
				if ((ucCount[port] != (loop*UC_PKTS)) || 
					(mcCount[port] != (loop*MC_PKTS)) ||
					(bcCount[port] != 0) ||
					(dropped[port] != (loop *EXP_DROPPED_PKTS)))
				{
					eprintf("Broadcast packet filtering test ERROR , Port %d\n",port);
					test_passed = 0;
				}					
				else
					res|= 1<<port_ctr;
				
				break;
		case '1':

				test_passed = 0;
				rxok=0;
				txok=0;

				eprintf("\nRx stats for port %d\n",port);

				eprintf("RX UC pkts = \t\t%d on port %d\n",rxucpkts[port],port);
				eprintf("RX MC pkts = \t\t%d on port %d\n",rxmcpkts[port],port);
				eprintf("RX BC pkts = \t\t%d on port %d\n",rxbcpkts[port],port);
				eprintf("RX Total Octets = \t%d on port %d\n",rxtotalOctets[port],port);
				eprintf("RX BAD Octets = \t%d on port %d\n",rxBadoctets[port],port);
				eprintf("RX 64 byte pkts = \t%d on port %d\n",rx64[port],port);
				eprintf("RX 65-127 byte pkts = \t%d on port %d\n",rx65to127[port],port);
				eprintf("RX 128-255 pkts = \t%d on port %d\n",rx128to255[port],port);
								
				if (cflag)
					port=rxport[port];
				eprintf("\nTx stats for port %d\n",port);

				eprintf("TX UC pkts = \t\t%d on port %d\n",txucpkts[port],port);
				eprintf("TX MC pkts = \t\t%d on port %d\n",txmcpkts[port],port);
				eprintf("TX BC pkts = \t\t%d on port %d\n",txbcpkts[port],port);
				eprintf("TX Total Octets = \t%d on port %d\n",txtotalOctets[port],port);
				eprintf("TX BAD Octets = \t%d on port %d\n",txBadoctets[port],port);
				eprintf("TX 64 byte pkts = \t%d on port %d\n",tx64[port],port);
				eprintf("TX 65-127 byte pkts = \t%d on port %d\n",tx65to127[port],port);
				eprintf("TX 128-255 pkts = \t%d on port %d\n",tx128to255[port],port);

				if (cflag)
					port=rxport[port];

				if (rxucpkts[port] != (loop*UC_PKTS))
					eprintf("%s\n", ERR_STR(RXUCPKTS_ERR));
				else if (rxmcpkts[port] != (loop*MC_PKTS)) 
					eprintf("%s\n", ERR_STR(RXMCPKTS_ERR));
				else if (rxbcpkts[port] != (loop*BC_PKTS)) 
					eprintf("%s\n", ERR_STR(RXBCPKTS_ERR));
				else if (rxtotalOctets[port] != (loop*888)) 
					eprintf("%s\n", ERR_STR(RXOCTETSTOTALOK_ERR));
				else if (rxBadoctets[port] != (loop*0x0)) 
					eprintf("%s\n", ERR_STR(RXOCTETSTOTALBAD_ERR));
				else if (rx64[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(RXPKTS64OCTETS_ERR));
				else if (rx64[port] != (loop*0x3))
					eprintf("%s\n", ERR_STR(RXPKTS65TO127OCTETS_ERR));
				else if (rx65to127[port] != (loop*0x3) )
					eprintf("%s\n", ERR_STR(RXPKTS64OCTETS_ERR));
				else if (rx128to255[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(RXPKTS128TO255OCTETS_ERR));
				else
				{
					eprintf("Rx stats. are OK ! \n");
					rxok = 1;
				}

				if (cflag)
					port=rxport[port];

				if (txucpkts[port] != (loop*UC_PKTS)) 
					eprintf("%s\n", ERR_STR(TXUCPKTS_ERR));
				else if (txmcpkts[port] != (loop*MC_PKTS)) 
					eprintf("%s\n", ERR_STR(TXMCPKTS_ERR));
				else if (txbcpkts[port] != (loop*BC_PKTS)) 
					eprintf("%s\n", ERR_STR(TXBCPKTS_ERR));
				else if (txtotalOctets[port] != (loop*888)) 
					eprintf("%s\n", ERR_STR(OCTETSTRANSMITTEDOK_ERR));
				else if (txBadoctets[port] != (loop*0x0)) 
					eprintf("%s\n", ERR_STR(OCTETSTRANSMITTEDBAD_ERR));
				else if (tx64[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(TXPKTS64OCTETS_ERR));
				else if (tx64[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(TXPKTS65TO127OCTETS_ERR));
				else if (tx65to127[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(TXPKTS64OCTETS_ERR));
				else if (tx128to255[port] != (loop*0x3)) 
					eprintf("%s\n", ERR_STR(TXPKTS128TO255OCTETS_ERR));
				else
				{
					eprintf("Tx stats. are OK ! \n");
					txok = 1;
				}

				if (rxok && txok)
					res|= 1<<port_ctr;
				break;

		default: 
				eprintf("Illegal Test parameter\n");
				test_passed = 0;
				break;
		}

	}
	//eprintf("rxok = %x, txok= %x, ports = %x, res = %x\n",rxok,txok,ports,res);



	if ((res == 0xF) && (ports == 0x4) && (!cflag))
		test_passed = 1;
	else
		if ((res == 0x3) && (ports == 0x4) && cflag)
			test_passed = 1;
		else
			if ((res == 0x1) && (ports != 0x4))
				test_passed = 1;
			else
				test_passed = 0;


	return test_passed;
}


void get_packets_for_filter_test()
{
	pl_sys_lb_data64_0 = data64_0;
	pl_sys_lb_data64_1 = data64_1;
	pl_sys_lb_data64_2 = data64_2;
	pl_sys_lb_data64_3 = data64_3;

	pl_sys_lb_data96_0 = data96_0;
	pl_sys_lb_data96_1 = data96_1;
	pl_sys_lb_data96_2 = data96_2;
	pl_sys_lb_data96_3 = data96_3;

	pl_sys_lb_data138_0 = data138_0;
	pl_sys_lb_data138_1 = data138_1;
	pl_sys_lb_data138_2 = data138_2;
	pl_sys_lb_data138_3 = data138_3;

}



void clear_stat_regs()
{
	int port=0, offset=0x20; 

	eprintf("Clearing statistics registers\n");
	spConfig();

	/* Read all Rx stat regs */
    for(port=0; port<4;port++)
    {
		offset = 0x20;
		while(offset <= 0x39)
		{
			reg_read((port*0x80)+offset);
			offset++;
		}
	}

	/* Read all Tx stat regs */
	for(port=0; port<4;port++)
    {
		offset = 0x40;
		while(offset <= 0x58)
		{
			reg_read((port*0x80)+offset);
			offset++;
		}
	}

	/* Read all Rx FIFO regs */
	offset = 0x580;
	while(offset <= (0x580 + 0x3F))
	{
		reg_read((port*0x80)+offset);
		offset++;
	}

	/* Read all Tx FIFO regs */
	offset = 0x600;
	while(offset <= (0x600 + 0x3E))
	{
		reg_read((port*0x80)+offset);
		offset++;
	}

	spRestore();
}



