/* pad_diag.c
 *
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: chinmay, NOVEMBER, 2002
 * 
 * 
 * ------------------------------------------------------------------------------
 */

typedef unsigned long ulong;
typedef unsigned short ushort;

#include "common.h"
#include "memory.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "syslog.h"
#include "uart.h"
#include "msf_init.h"
#include "diag_utils.h"

#include "error_code.h"
#include "error_map.h"
#include "reg_api.h"
#include "register_map.h"
#include "diag.h"
#include "dp_proto.h"
#include "mac_util.h"

#define UNI_PKTFILCTRL	0x1
#define MUL_PKTFILCTRL	0x2
#define BRD_PKTFILCTRL	0x4
#define HI_MUL_ADDR	0xA1B0C0D0
#define LO_MUL_ADDR	0xE0F0

extern const int pl_sys_loopback_eg_rx[];
extern const int pl_sys_loopback_eg_tx[];
extern const int pl_sys_loopback_in_rx[];
extern const int pl_sys_loopback_in_tx[];

extern pci_device slave_dev_info;		// for slave npu info.
extern UINT32 PKT_48_SZ,PKT_64_SZ,PKT_138_SZ,PKT_4_SZ;
extern UINT32 PKT_SEQ_SZ,ALL_PORT_PKT_SEQ_SZ;

extern UINT32 Sz_pl_sys_loopback_eg_rx;
extern UINT32 Sz_pl_sys_loopback_eg_tx;
extern UINT32 Sz_pl_sys_loopback_in_rx;
extern UINT32 Sz_pl_sys_loopback_in_tx;

/** pl_sys_lb_data96_0 will be pointing to 48 byte data in this test */
extern  UINT32 *pl_sys_lb_data96_0;
extern  UINT32 *pl_sys_lb_data64_0;
extern  UINT32 *pl_sys_lb_data138_0;
extern  UINT32 *pl_sys_lb_data96_1;
extern  UINT32 *pl_sys_lb_data64_1;
extern  UINT32 *pl_sys_lb_data138_1;
extern  UINT32 *pl_sys_lb_data96_2;
extern  UINT32 *pl_sys_lb_data64_2;
extern  UINT32 *pl_sys_lb_data138_2;
extern  UINT32 *pl_sys_lb_data96_3;
extern  UINT32 *pl_sys_lb_data64_3;
extern  UINT32 *pl_sys_lb_data138_3;

extern UINT32 tx_data_pl_48[];
extern UINT32 tx_data_pl_48_0[];
extern UINT32 tx_data_pl_48_1[];
extern UINT32 tx_data_pl_48_2[];
extern UINT32 tx_data_pl_48_3[];
extern UINT32 tx_data_pl_4[];
extern UINT32 tx_data_pl_4_0[];
extern UINT32 tx_data_pl_4_1[];
extern UINT32 tx_data_pl_4_2[];
extern UINT32 tx_data_pl_4_3[];

#define PKTS_64			6	 /* expected 64 byte packets, per port */
#define PKTS_138		3	 /* expected 138 packets, per port */
#define EXP_BAD_PKTS	0	 /* expected dropped packets, per port */

#define SHORT_PKT_NOT_PADDED	0xBAD0
#define PADDED_64_PKT			0xBAD1
#define PADDED_138_PKT			0xBAD2
#define PREPEND_FAILED			0xBAD3

extern UINT32 data64_0[];
extern UINT32 data64_1[];
extern UINT32 data64_2[];
extern UINT32 data64_3[];
extern UINT32 data48_0[];
extern UINT32 data4_0[];
extern UINT32 data138_0[];
extern UINT32 data138_1[];
extern UINT32 data138_2[];
extern UINT32 data138_3[];


/** globals **/
extern UINT32 total_args,loop,ports,speed;  
extern char channel,test_opt;
extern UINT8 test_param;

extern UINT32 validate_params(void);
extern UINT32 init_mac_dev(UINT32);
extern void spConfig(void);
extern void spRestore(void);
extern void get_stats(void);
extern void Set_Pci_Doorbell(int bit_num, int db_intr_addr);
extern void get_packets_for_filter_test();
extern void clear_stat_regs();

/* Local Functions */
void get_packets_for_pad_test();
int verify_padding(short cflag);
void get_exp_pkt_sz(UINT32 * exp_sz);
void get_error(UINT32 pkt_size, UINT32 exp_pkt_sz, UINT32 *mailbox_value);
void cu_init(UINT32 set_int_lpbk);

	/***************************************************************************
	** The rxport is to be indexed by the txport and the value is the corresponding rxport.
	** Tx port	  Rxport
	**	0			1
	**  1			0
	**  2			3
	**  3			2
	*********************************************************************************/
	extern UINT32 rxport[4];


// ******************************************************************************
// Function: void pad_test(void)
//
// This function is called by the Test Manager when the user invokes the
// point Lone system crc test
// ******************************************************************************
UINT32 padding_test(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, tx_count=0, rx_count=0;
	UINT32 scratch_ring_tx_count, scratch_ring_rx;
	UINT32 dcache_status, forever = 0, combination, port_num, rx_port_num;
	UINT32 loop_count=0;
	UINT32 scratch_read_value=0, dram_addr;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	UINT32 pass_port[4],rxflag[4];
	UINT32 expected_num_pkts = 0,iteration = 0, port_ctr=0, pnum=0, *pktptr=0;
	UINT32 exp_pkt_sz = 0,cflag = 0;
	UINT32 counter=0;

    channel = 'f';		

	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	test_param = acL->argv[3][0];
	channel = acL->argv[4][0]; 		   	
	speed = acL->decArg[5];	
	ports = acL->decArg[6];
	loop = acL->decArg[7];

	if (validate_params() == INVALID_PARAMETERS)
	{
		eprintf("Invalid parameters. \n");
		return INVALID_PARAMETERS;
	}
	if (channel == 'c')
	{
		cflag = 1;
	}

	if (total_args <= 7)
	{
		loop = 1;
	}

	if (loop == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}
	
	/* initialize port array to 0 */
	for (ctr = 0;ctr<4;ctr++)
	{
		pass_port[ctr] = 0;
		rxflag[ctr] = ERROR;
	}

	/* get the packet data used specifically for the test */
	get_packets_for_pad_test();

	switch(ports)
	{
		case 0: 
				if (test_param=='6')
					pktptr = tx_data_pl_4_0;
				else	
					pktptr = tx_data_pl_48_0;
				pnum = PKT_SEQ_SZ/4; 
				iteration = 1;
				break;
		case 1: 
				if (test_param=='6')
					pktptr = tx_data_pl_4_1;
				else	
					pktptr = tx_data_pl_48_1;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 2: 
				if (test_param=='6')
					pktptr = tx_data_pl_4_2;
				else	
					pktptr = tx_data_pl_48_2;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 3: 
				if (test_param=='6')
					pktptr = tx_data_pl_4_3;
				else	
					pktptr = tx_data_pl_48_3;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 4: 
				if (test_param=='6')
					pktptr = tx_data_pl_4;
				else	
					pktptr = tx_data_pl_48;
				pnum = ALL_PORT_PKT_SEQ_SZ/(4*4);
				if (cflag)
					iteration = 2;
				else
					iteration = 4;
				break;
	}


	scratch_ring_rx = SCR_RING_INSTR_ADDR + (5 << 2);
	scratch_ring_tx_count = SCR_RING_INSTR_ADDR + (4 << 2);

	if(acL->HostType == MASTER)		// if master NPU
	{
		flag = OK;

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		eprintf("Initialising media card. Please wait...\n");
		status = init_mac_dev(0);	
		if (status != DONE)
		{
			eprintf("Aborting Test\n");
			return ABORT;
		}
	
		clear_stat_regs();
		spConfig();

		switch(test_param)
		{
		case 'e':	/* Enable Padding */
			    eprintf("Testing Padding ...\n");
				//reg_write((RXFIFO_DROP_ENABLE),0xF);  
				for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
				{
					/*enable Padding */
					if (!cflag)
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x11ED); 
					else
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x11CD); 
					/* enable CRC strip */
					reg_write(RXFIFO_PAD_CRCSTRIP_ENB, 0xF0);
				}
				break;

			case 'p':  /* Enable Pre-pending */
			    eprintf("Testing Pre-pending ...\n");
				//reg_write((RXFIFO_DROP_ENABLE),0xF);  
				for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
				{
					/*enable Padding */
					if (!cflag)
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x11ED); 
					else
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x11CD); 

					/* enable CRC strip and Pre-pending*/
					reg_write(RXFIFO_PAD_CRCSTRIP_ENB, 0xFF);
				}
				break;

			case '5':  /* RMON test for Short/Runt Rx stat. regs */
			case '6':
				reg_write((RXFIFO_DROP_ENABLE),0xF);  			    
				//reg_write((RXFIFO_DROP_ENABLE),0xF);  
				for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
				{
					if (!cflag)
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x116D); 
					else
						reg_write((DIV_CONFIG_WORD+(port_ctr*PORTSZ)),0x114D); 

					/* set shortRunt Threshold to 40 bytes */
					reg_write(SHORT_RUNT_TSHD +(port_ctr*PORTSZ), 0x28);
				}
				reg_write(RXFIFO_PAD_CRCSTRIP_ENB, 0xF0);
				break;

		default :
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}

		spRestore();
		eprintf("Media card initialisation completed.\n");

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		PUT32(slave_mailbox0_addr, 0);

#ifdef __ARMEB__
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif

		PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), DRAM_BASE_SRC);
		// Write packet data to DRAM for microcode to use
		if (test_param == '6')
		{
			for (ctr = 0; ctr < PKT_4_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_4_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_0[(ctr / 4)]);
			}
		}
		else
		{
			for (ctr = 0; ctr < PKT_48_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_48_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_0[(ctr / 4)]);
			}
		}
		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_0[(ctr / 4)]);
		}
		if (test_param == '6')
		{
			for (ctr = 0; ctr < PKT_4_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_4_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_1[(ctr / 4)]);
			}
		}
		else
		{
			for (ctr = 0; ctr < PKT_48_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_48_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_1[(ctr / 4)]);
			}
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_1[(ctr / 4)]);
		}

		if (test_param == '6')
		{
			for (ctr = 0; ctr < PKT_4_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_4_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_2[(ctr / 4)]);
			}
		}
		else
		{
			for (ctr = 0; ctr < PKT_48_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_48_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_2[(ctr / 4)]);
			}
		}


		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_2[(ctr / 4)]);
		}
		if (test_param == '6')
		{
			for (ctr = 0; ctr < PKT_4_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_4_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_3[(ctr / 4)]);
			}
		}
		else
		{
			for (ctr = 0; ctr < PKT_48_SZ; ctr+=4)
			{
				PUT32((DRAM_BASE_SRC + ((RING_DATA_48_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_3[(ctr / 4)]);
			}
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_3[(ctr / 4)]);
		}

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);


		// Invoke the slave's PCI doorbell
		Set_Pci_Doorbell(PTLONE_SYS_LPBK, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		ctr = 0;

		// Wait for a signal that slave NPU has completed loading of ucode and init
		do
		{
			if (ctr == 1500)	// After 10 seconds
			{
				eprintf("\nTimed out...Slave NPU was not able to load microcode and init.\n");
				mailbox_value = INIT_ERROR;
				flag = ERROR;
				break;
			}

			temp = GET32(slave_mailbox0_addr);

			hal_delay_us(10000);
			ctr++;
		}
		while (temp == 0);
		
		if (flag == OK)	// If slave completed microcode loading successfully
		{
			// Load Master's NPU with ucode and init
			DownLoadUcode(PL_SYS_LOOPBACK_EG_TX_ME, Sz_pl_sys_loopback_eg_tx, (UINT32 *)pl_sys_loopback_eg_tx);
			DownLoadUcode(PL_SYS_LOOPBACK_EG_RX_ME, Sz_pl_sys_loopback_eg_rx, (UINT32 *)pl_sys_loopback_eg_rx);

			hal_delay_us(1000000);

			do
			{
				for (combination = 0; combination < pnum; combination++)
				{
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
						{
							port_num = ports;
							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), pktptr[combination]);
						}
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;

							if (test_param == '6')
								PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_pl_4[((combination * 4) + port_num)]);
							else
								PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_pl_48[((combination * 4) + port_num)]);
						}
					}
				}

				for (combination = 0; combination < pnum; combination++)
				{
					flag = OK;
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
							port_num = ports;
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;
						}

						ctr = 0;

						do			// Do while waiting for all the packets to be transmitted
						{
							if (ctr == 1000)	// After 1 second
							{
								eprintf("\nTimed out...No indication from scratch ring that all packets were transmitted\n");
								mailbox_value = NOT_SENT;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if the packets have been transmitted
							if ((scratch_read_value = GET32(scratch_ring_tx_count)) == 0)
							{
								hal_delay_us(2);
								ctr++;
							}
							else
							{
								switch(ports)
								{
									case 0 : 
									case 1 : 
									case 2 : 
									case 3 : 
											eprintf("Transmitted on port %d: %d\r", ports,(scratch_read_value));
											
											break;
    
									case 4 : // all ports 
											if (!cflag)
											{
												eprintf("Transmitted on ports 0-3 : %d\r", (scratch_read_value)/4);
												iteration = 4;
											}
											else
											{
												eprintf("Transmitted on ports 0 and 2 : %d\r", (scratch_read_value)/2);
												iteration = 2;
											}
											break;
								}

								tx_count++;
							}
						}
						while (scratch_read_value == 0);

						PUT32(scratch_ring_tx_count, 0);

						tx_count =  scratch_read_value;
						if (flag == ERROR)
						{
							break;
						}
					}

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}
				
				for (combination = 0; combination < pnum; combination++)
				{
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (!cflag)
						{
							if (ports != 4)
								port_num = ports;
							else
								port_num = port_ctr;
						}
						else
						{
							if (ports != 4)
								port_num = rxport[ports];
							else
								port_num = port_ctr*2 + 1;
						}

						ctr = 0;

						do			// Do while waiting for a signal that a packet has been received
						{
							if (ctr == 300)	// After 1 second
							{
								//eprintf("\nTimed out...No indication from scratch ring that all packets were received\n");
								mailbox_value = NOT_RECEIVED;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if packets have been received
							if ((scratch_read_value = GET32(scratch_ring_rx)) == 0)
							{
								hal_delay_us(100);
								ctr++;
							}
						}
						while (scratch_read_value == 0);

						if (flag == ERROR)
						{
							break;
						}

						//MSG("DEBUG: scratch_read_value = 0x%08X\n", scratch_read_value,0,0);

						rx_count++;
						pkt_size = scratch_read_value & MASK_8BIT;
						rx_port_num = (scratch_read_value >> 8) & MASK_8BIT;
						dram_addr = DRAM_BASE_DEST + ((scratch_read_value >> 16) << 8);
						// set the port on which packet was received.
						pass_port[rx_port_num]++;
						rxflag[rx_port_num] = OK;


						if (ports == 4)
						{
							if (test_param == '6')
								exp_pkt_sz = tx_data_pl_4[((combination * 4) + port_num)] & 0xFFFF;
							else
								exp_pkt_sz = tx_data_pl_48[((combination * 4) + port_num)] & 0xFFFF;
						}
						else
						{
							exp_pkt_sz = pktptr[combination] & 0xFFFF;
						}
						
						
						if (test_param != '5')
							get_exp_pkt_sz(&exp_pkt_sz);
						else
						{
							exp_pkt_sz = pkt_size;
						}
						
						//eprintf("Expected size : %d, pkt size = %d\n",exp_pkt_sz,pkt_size);
						if (pkt_size != exp_pkt_sz)
						{
							eprintf("\nActual packet size = 0x%x\n",pkt_size);
							eprintf("Expected packet size: 0x%x\n",exp_pkt_sz);
							get_error(pkt_size, exp_pkt_sz, (UINT32*)(&mailbox_value));
							flag = ERROR;		// Set flag as error
							rxflag[rx_port_num] = ERROR;
							break;
						}

						if (flag == ERROR)
						{
							if (test_param == '5')
								;
							else
								break;
						}

						//hal_delay_us(800);
					}

					if (flag == ERROR)
					{
						if (test_param == '5')
								;
							else
								break;
					}
				}

				if (flag == ERROR)
				{
					if (test_param == '5')
						;
					else
						break;
	
				}

				loop_count++;

				if (forever && KEYPRESS)
				{
					break;
				}
			}
			while ((loop_count != loop) || forever);
			
			if (loop == 0)
			{
				counter = loop_count;
				loop = loop_count;
			}
			else
			{
				counter = loop;
			}
			switch(test_param)
			{
			case '5': expected_num_pkts = counter * 6;
					  break;
			default:  expected_num_pkts = counter * 9;
					  break;
			}

			MSG("\nexpected_num_pkts = %d\n",expected_num_pkts,0,0);

			switch(ports)
			{
		       case 0:
		       case 1:
		       case 2:
		       case 3:  if (!cflag) 
						{ 
							if ((pass_port[ports] != expected_num_pkts) || (pass_port[ports] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", ports);
								eprintf("Packets Received = %d\n",pass_port[ports]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",ports,pass_port[ports]);
								flag = OK;
							}
						}
						else
						{
							if ((pass_port[rxport[ports]] != expected_num_pkts) || (pass_port[rxport[ports]] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", rxport[ports]);
								eprintf("Packets Received on port %d = %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = OK;
							}
						}
						break;

		       case 4:  eprintf("\n");
						if (!cflag)
						{
							for (ctr=0; ctr<4; ctr++)
							{
								if ((pass_port[ctr] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr);
									eprintf("Packets Received = %d\n",pass_port[ctr]);
									rxflag[ctr] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr],ctr);
									//flag = OK;
								}
							}
						}
						else
						{
							for (ctr=0; ctr<2; ctr++)
							{
								if ((pass_port[ctr*2+1] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr*2+1);
									eprintf("Packets Received = %d\n",pass_port[ctr*2+1]);
									rxflag[ctr*2+1] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr*2+1] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr*2+1],ctr*2+1);
								}
							}
						}
						
						break;
						
			   default:   eprintf("This is bad value for port number \n");
								return INVALID_PARAMETERS;
			}

		}
		eprintf("\n");

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}

		if (loop_count ==0)
			loop = loop_count;
	
			
		if (verify_padding(cflag))
		{
			if (test_param == 'e')
				eprintf("\nPadding test PASSED !\n");
			else if (test_param == 'p')
				eprintf("\nPre-pending test PASSED !\n");
			else if (test_param == '5')
				eprintf("\nRMON: Rx Runt stat. reg test PASSED !\n");
			else if (test_param == '6')
				eprintf("\nRMON: Rx Short stat. reg test PASSED !\n");


			Set_LED("PASS");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		} 
		else
		{
			if (mailbox_value == INIT_ERROR)
			{
				eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
			}
			else if (mailbox_value == NOT_RECEIVED)
			{
				eprintf("There was no sign that the expected packet was received\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The packet received was not the same as the packet transmitted.\n");
			}
			else if (mailbox_value == SHORT_PKT_NOT_PADDED)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("Short packet was not padded to 64 byte length.\n");
			}
			else if (mailbox_value == PADDED_64_PKT)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("Error. 64 byte packet was padded.\n");
			}
			else if (mailbox_value == PADDED_138_PKT)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("Error. 138 byte packet was padded.\n");
			}
			else if (mailbox_value == PREPEND_FAILED)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("Error. 2 byte prepend failed.\n");
			}


			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			switch(test_param)
			{
			case 'e':
				strcpy(syslog.desc, "\nPAD test FAILED!");
				eprintf("PAD test FAILED!\n");
				break;
			case 'p':
				strcpy(syslog.desc, "\nPrepending test FAILED!");
				eprintf("Prepending test FAILED!");
				break;
			case '5':
				strcpy(syslog.desc, "\nRMON: Rx Runt stat. reg test FAILED!");
				eprintf("\nRMON: Rx Runt stat. reg test FAILED!");
				break;
			case '6':
				strcpy(syslog.desc, "\nRMON: Rx Short stat. reg test FAILED!");
				eprintf("\nRMON: Rx Short stat. reg test FAILED!");
				break;

			}
			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			
			Set_LED("FAIL");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
		
		Clear_All_ME();
		Msf_OutOfReset();

	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}
	return DONE;
}


int verify_padding(short cflag)
{
	int port=0,ctr=0,port_ctr=0,test_passed=1;
	UINT32 rx64pkts[4],	 rx138pkts[4];
	UINT32 tx64pkts[4], tx138pkts[4];
	UINT32 Rxrunt[4],TxBad[4], RxShort[4];
	short res=0, rxok=0,txok=0;

	spConfig();
    for(port=0; port<4;port++)
    {
		/* Reading the total packets received */
		rx64pkts[port]=reg_read(((port*PORTSZ)+RX_PKTS_64));  
		rx138pkts[port]=reg_read(((port*PORTSZ)+RX_PKTS_128_255));  
		Rxrunt[port] = reg_read(((port*PORTSZ)+RUNT_ERR));  
		RxShort[port] = reg_read(((port*PORTSZ)+SHORT_ERR));  

		tx64pkts[port]=reg_read(((port*PORTSZ)+TX_PKTS_64));  
		tx138pkts[port]=reg_read(((port*PORTSZ)+TX_PKTS_128_255));  
		TxBad[port] = reg_read(((port*PORTSZ)+TX_OCTETS_BAD));  
	}
	spRestore();

	if (ports == 4)
	{
		if (!cflag)
			ctr = 4;
		else
			ctr = 2;
	}
	else
		ctr = 1;

	for(port_ctr=0; port_ctr<ctr;port_ctr++)
	{
		if ((!cflag) && (ports == 4))
			port = port_ctr;
		if ((cflag) && (ports == 4))
		{
			port = port_ctr*2;
			port = rxport[port];
		}
		if ((!cflag) && (ports != 4))
			port = ports;
		if ((cflag) && (ports != 4))
			port = rxport[ports];

		eprintf("Port %d: 64 byte packets received=0x%x\n",port,rx64pkts[port]);
		eprintf("Port %d: 138 byte packets received=0x%x\n",port,rx138pkts[port]);
		eprintf("Port %d: Runt Packets received 0x%x\n",port,Rxrunt[port]);
		eprintf("Port %d: Short Packets received 0x%x\n",port,RxShort[port]);
		if (cflag)
			port = rxport[port];

		eprintf("Port %d: 64 byte packets transmitted=0x%x\n",port,tx64pkts[port]);
		eprintf("Port %d: 138 byte packets transmitted=0x%x\n",port,tx138pkts[port]);
		eprintf("Port %d: Octets transmitted with Errors = 0x%x\n",port,TxBad[port]);


		switch(test_param)
		{
		case 'e':
		case 'p':
			if (cflag)
				port = rxport[port];
			if ((rx64pkts[port] != (loop*PKTS_64)) || 
				(rx138pkts[port] != (loop*PKTS_138)) ||
				(Rxrunt[port] != (loop *EXP_BAD_PKTS)))
			{
				eprintf("Received frames with Padding Errors on port %d\n",port);
				rxok = 0;
			}
			else
			{
				if ((rx64pkts[port] == 0) || 
					(rx138pkts[port] == 0))
				{
					rxok = 0;
				}
				else
					rxok = 1;
			}

			if (cflag)
				port = rxport[port];

			if ((tx64pkts[port] != (loop*PKTS_64)) || 
				(tx138pkts[port] != (loop*PKTS_138)) ||
				(TxBad[port] != (loop *EXP_BAD_PKTS)))
			{
				eprintf("Transmitted frames with Padding Errors on port %d\n",port);
				txok = 0;
			}
			else
			{
				if ((tx64pkts[port] == 0) || 
					(tx138pkts[port] == 0))
				{
					txok = 0;
				}
				else txok = 1;
			}

			if(rxok && txok)
				res|= 1<<port_ctr;

			break;
		case '5':
			if (cflag)
				port = rxport[port];

			if ((Rxrunt[port] == 3*loop) && (RxShort[port] == 0))
			{
				res|= 1<<port_ctr;
				eprintf("RMON Runt frame stat. counter OK, port %d\n",port);
			}
			else
			{
				eprintf("RMON Runt frame stat. counter BAD, port %d\n",port);
				res|= 0<<port_ctr;
			}
			break;			
		case '6':
			if (cflag)
				port = rxport[port];

			if ((Rxrunt[port] == 0) && (RxShort[port] == 3*loop) && (RxShort[port] != 0))
			{
				res|= 1<<port_ctr;
				eprintf("RMON Short frame stat. counter OK, port %d\n",port);
			}
			else
			{
				eprintf("RMON Short frame stat. counter BAD, port %d\n",port);
				res|= 0<<port_ctr;
			}
			break;			

		}

	}

	if ((res == 0xF) && (ports == 0x4) && (!cflag))
		test_passed = 1;
	else
		if ((res == 0x3) && (ports == 0x4) && cflag)
			test_passed = 1;
		else
			if ((res == 0x1) && (ports != 0x4))
				test_passed = 1;
			else
							test_passed = 0;
	return test_passed;
}


void get_packets_for_pad_test()
{
	pl_sys_lb_data64_0 = data64_0;
	pl_sys_lb_data64_1 = data64_1;
	pl_sys_lb_data64_2 = data64_2;
	pl_sys_lb_data64_3 = data64_3;

	if (test_param == '6') /* Rx Short reg. test */
	{
		/* use the same 48 byte frame for all ports */
		pl_sys_lb_data96_0 = data4_0;
		pl_sys_lb_data96_1 = data4_0;
		pl_sys_lb_data96_2 = data4_0;
		pl_sys_lb_data96_3 = data4_0;
	}
	else
	{
		/* use the same 48 byte frame for all ports */
		pl_sys_lb_data96_0 = data48_0;
		pl_sys_lb_data96_1 = data48_0;
		pl_sys_lb_data96_2 = data48_0;
		pl_sys_lb_data96_3 = data48_0;
	}
	
	pl_sys_lb_data138_0 = data138_0;
	pl_sys_lb_data138_1 = data138_1;
	pl_sys_lb_data138_2 = data138_2;
	pl_sys_lb_data138_3 = data138_3;

}

void get_exp_pkt_sz(UINT32 * exp_sz)
{
	UINT32 pkt_sz = *exp_sz;

	switch (test_param)
	{
	case 'e': /* padding enabled */
			if (pkt_sz == 0x2C)
				pkt_sz = 0x3C;
			break;
	case 'p': /*pre-pending and padding enabled */
			if ((pkt_sz == 0x2C) || (pkt_sz == 0x3C))
				pkt_sz = 0x3C + 2; 
			if (pkt_sz == 0x84)
				pkt_sz = 0x84 + 2;
			break;
	case '5': /* RMON runt/short stat reg */
		/* Vallejo workaround for bacCRC append */
			if (pkt_sz == 0x2C) 
				pkt_sz=0x30;
			//if (pkt_sz == 0x3C) 
			//	pkt_sz=0x84;


	case '6': /* RMON runt/short stat reg */
		/* Vallejo workaround for bacCRC append */
			if (pkt_sz == 0x8) 
				pkt_sz=4;

			
	}
	*exp_sz = pkt_sz;
}

void get_error(UINT32 pkt_size, UINT32 exp_pkt_sz, UINT32 *mailbox_value)
{
	switch (test_param)
	{
	case 'e':
			if ((pkt_size == 0x2C) && (exp_pkt_sz == 0x3C))
				*mailbox_value = SHORT_PKT_NOT_PADDED;
			if ((pkt_size > 0x3C) && (pkt_size < 0x84) && (exp_pkt_sz == 0x3C))
				*mailbox_value = PADDED_64_PKT;
			if ((pkt_size > 0x84) && (exp_pkt_sz == 0x84))
				*mailbox_value = PADDED_138_PKT;
			break;
	case 'p':
			if ( (((pkt_size == 0x2C) || (pkt_size == 0x3C)) && (exp_pkt_sz == 0x3E))
				|| ((pkt_size == 0x84) && (exp_pkt_sz == 0x86)) )
	
				*mailbox_value = PREPEND_FAILED;
			break;
	}
}




