/* vp_filter.c
 *
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: chinmay, NOVEMBER, 2002
 * 
 * 
 * ------------------------------------------------------------------------------
 */

typedef unsigned long ulong;
typedef unsigned short ushort;

#include "common.h"
#include "memory.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "syslog.h"
#include "uart.h"
#include "msf_init.h"
#include "diag_utils.h"

#include "error_code.h"
#include "error_map.h"
#include "reg_api.h"
#include "register_map.h"
#include "diag.h"
#include "dp_proto.h"
#include "mac_util.h"

#define VLAN_PKTFILCTRL		0x8
#define PAUSE_PKTFILCTRL	0x0

#define HI_MUL_ADDR			0xA1B0C0D0
#define LO_MUL_ADDR			0xE0F0
#define MPHY				1
#define VLAN_CHECK			0x8100
#define PAUSE_CHECK			0x8100

short VCount[4],PCount[4];

extern const int pl_sys_loopback_eg_rx[];
extern const int pl_sys_loopback_eg_tx[];
extern const int pl_sys_loopback_in_rx[];
extern const int pl_sys_loopback_in_tx[];

extern UINT32 Sz_pl_sys_loopback_eg_rx;
extern UINT32 Sz_pl_sys_loopback_eg_tx;
extern UINT32 Sz_pl_sys_loopback_in_rx;
extern UINT32 Sz_pl_sys_loopback_in_tx;

extern UINT32 vlan64_0[];
extern UINT32 pause64_0[];
extern UINT32 data96_0[];
extern UINT32 data96_1[];
extern UINT32 data96_2[];
extern UINT32 data96_3[];
extern UINT32 data138_0[];
extern UINT32 data138_1[];
extern UINT32 data138_2[];
extern UINT32 data138_3[];

extern pci_device slave_dev_info;		// for slave npu info.
extern UINT32 PKT_96_SZ,PKT_64_SZ,PKT_138_SZ;
extern UINT32 PKT_SEQ_SZ,ALL_PORT_PKT_SEQ_SZ;

extern  UINT32 *pl_sys_lb_data96_0;
extern  UINT32 *pl_sys_lb_data64_0;
extern  UINT32 *pl_sys_lb_data138_0;
extern  UINT32 *pl_sys_lb_data96_1;
extern  UINT32 *pl_sys_lb_data64_1;
extern  UINT32 *pl_sys_lb_data138_1;
extern  UINT32 *pl_sys_lb_data96_2;
extern  UINT32 *pl_sys_lb_data64_2;
extern  UINT32 *pl_sys_lb_data138_2;
extern  UINT32 *pl_sys_lb_data96_3;
extern  UINT32 *pl_sys_lb_data64_3;
extern  UINT32 *pl_sys_lb_data138_3;

extern UINT32 tx_data_pl[];
extern UINT32 tx_data_pl_0[];
extern UINT32 tx_data_pl_1[];
extern UINT32 tx_data_pl_2[];
extern UINT32 tx_data_pl_3[];

extern UINT32 rxport[4];

#define VLAN_PKTS				3	 /* expected unicast packets, per port */
#define PAUSE_PKTS				3	 /* expected broadcast packets, per port */
#define EXP_DROPPED_PKTS		3	 /* expected dropped packets, per port */
#define VLAN_FILTER_FAILED		0xBAD4
#define PAUSE_FILTER_FAILED		0xBAD5

/** globals **/
extern UINT32 total_args,loop,ports,speed;  
extern char channel,test_opt;
extern UINT8 test_param;

extern UINT32 validate_params(void);
extern UINT32 init_mac_dev(UINT32);
extern void spConfig(void);
extern void spRestore(void);
extern void get_stats(void);
extern void clear_stat_regs();
extern UINT32 assign_pkt_pointers(UINT32 ports, UINT32 *pktptr, UINT32 cflag, UINT32 *pnum);

/*  local functions */
void get_packets_for_vpse_filter_test();
short verify_vpse_filter();
void get_vpse_error(UINT32 pkt_size, UINT32 exp_pkt_sz, UINT32 *mailbox_value);

// ******************************************************************************
// Function: void vlan_pause_filter_test(void)
//
// This function is called by the Test Manager when the user invokes the
// point Lone VLAN/PAUSE frame filtering test.
// ******************************************************************************
UINT32 vlan_pause_filter_test(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, tx_count=0, rx_count=0;
	UINT32 scratch_ring_tx_count, scratch_ring_rx;
	UINT32 dcache_status, forever = 0, combination, port_num, rx_port_num;
	UINT32 loop_count=0;
	UINT32 *pl_sys_lb_data=0;
	UINT32 scratch_read_value=0, dram_addr, orig_data_size=0;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 pass_port[4],rxflag[4];
	UINT32 expected_num_pkts = 0,iteration = 0, port_ctr=0, pnum=0, *pktptr=0;
	UINT32 exp_pkt_sz = 0,cflag = 0;
	/***************************************************************************
	** The rxport is to be indexed by the txport and the value is the corresponding rxport.
	** Tx port	  Rxport
	**	0			1
	**  1			0
	**  2			3
	**  3			2
	*********************************************************************************/
	UINT32 rxport[4] = {1, 0, 3, 2};
	UINT32 bad_pkt_size = 0;
	UINT32 test_passed=0,counter=0;
	
    channel = 'f';		

	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	test_param = acL->argv[3][0];
	channel = acL->argv[4][0]; 		   	
	speed = acL->decArg[5];	
	ports = acL->decArg[6];
	loop = acL->decArg[7];

	if (validate_params() == INVALID_PARAMETERS)
	{
		eprintf("Invalid parameters. \n");
		return INVALID_PARAMETERS;
	}

	if (channel == 'c')
	{
		cflag = 1;
	}

	if (total_args <= 7)
	{
		loop = 1;
	}

	if (loop == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}
	
	/* initialize port array to 0 */
	for (ctr = 0;ctr<4;ctr++)
	{
		pass_port[ctr] = 0;
		rxflag[ctr] = ERROR;
		/* initialize counts to 0 */
		VCount[ctr]=0;
		PCount[ctr]=0;
	}
	/* get the packet data used specifically for the test */
	get_packets_for_vpse_filter_test();

	iteration = assign_pkt_pointers(ports, pktptr, cflag, &pnum);

	switch(ports)
	{
		case 0: pktptr = tx_data_pl_0;
				pnum = PKT_SEQ_SZ/4; 
				iteration = 1;
				break;
		case 1: pktptr = tx_data_pl_1;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 2: pktptr = tx_data_pl_2;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 3: pktptr = tx_data_pl_3;
				pnum = PKT_SEQ_SZ/4;
				iteration = 1;
				break;
		case 4: pktptr = tx_data_pl;
				pnum = ALL_PORT_PKT_SEQ_SZ/(4*4);
				if (cflag)
					iteration = 2;
				else
					iteration = 4;
				break;
	}



	scratch_ring_rx = SCR_RING_INSTR_ADDR + (5 << 2);
	scratch_ring_tx_count = SCR_RING_INSTR_ADDR + (4 << 2);

	if(acL->HostType == MASTER)		// if master NPU
	{
		flag = OK;

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		eprintf("Initialising media card. Please wait...\n");
		status = init_mac_dev(0);	
		if (status != DONE)
		{
			eprintf("Aborting Test\n");
			return ABORT;
		}
		
		clear_stat_regs();
		spConfig();
#if 0
		/* clear drop pkt counters */
		reg_read(0x5A2);
		reg_read(0x5A3);
		reg_read(0x5A4);
		reg_read(0x5A5);
#endif
		switch(test_param)
		{
		case 'l':
				reg_write((RXFIFO_DROP_ENABLE),0xF);  
				for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
				{
					/*filter vlan packets */
					reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),VLAN_PKTFILCTRL); 
				}
				break;
		case 'p':
			/* bad packets to be dropped */
			reg_write((RXFIFO_DROP_ENABLE),0xF);  
			for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
			{
				/* filter pause packets */
				reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),PAUSE_PKTFILCTRL); 
			}

		case '2': /* This case handles the RMON Pause/Vlan stat reg test */
		case '3':
			/* bad packets to be dropped */
			reg_write((RXFIFO_DROP_ENABLE),0xF);  
			for(port_ctr=0; port_ctr<NUM_PORTS;port_ctr++)
			{
				/* filter pause packets */
				reg_write((PKT_FILTER_CTL+(port_ctr*PORTSZ)),PAUSE_PKTFILCTRL); 
			}
				break;			
		


		default :
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}

		spRestore();
		eprintf("Media card initialisation completed.\n");

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		PUT32(slave_mailbox0_addr, 0);

#ifdef __ARMEB__
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
			PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif

		PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), DRAM_BASE_SRC);
		// Write packet data to DRAM for microcode to use
		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_96_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_64_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < PKT_138_SZ; ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_3[(ctr / 4)]);
		}

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

		// Invoke the slave's PCI doorbell
		Set_Pci_Doorbell(PTLONE_SYS_LPBK, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		ctr = 0;

		// Wait for a signal that slave NPU has completed loading of ucode and init
		do
		{
			if (ctr == 1500)	// After 10 seconds
			{
				eprintf("\nTimed out...Slave NPU was not able to load microcode and init.\n");
				mailbox_value = INIT_ERROR;
				flag = ERROR;
				break;
			}

			temp = GET32(slave_mailbox0_addr);

			hal_delay_us(10000);
			ctr++;
		}
		while (temp == 0);
		
		if (flag == OK)	// If slave completed microcode loading successfully
		{
			// Load Master's NPU with ucode and init
			DownLoadUcode(PL_SYS_LOOPBACK_EG_TX_ME, Sz_pl_sys_loopback_eg_tx, (UINT32 *)pl_sys_loopback_eg_tx);
			DownLoadUcode(PL_SYS_LOOPBACK_EG_RX_ME, Sz_pl_sys_loopback_eg_rx, (UINT32 *)pl_sys_loopback_eg_rx);

			hal_delay_us(1000000);

			//MSG("DEBUG: (sizeof(tx_data_pl) / sizeof(UINT32)) / 4 = %d\n", ((sizeof(tx_data_pl) / sizeof(UINT32)) / 4),0,0);

			do
			{
				//for (combination = 0; combination < ((sizeof(tx_data_pl) / sizeof(UINT32)) / 4); combination++)
				for (combination = 0; combination < pnum; combination++)
				{
					//for (port_num = 0; port_num < 4; port_num++)
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
						{
							port_num = ports;
							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), pktptr[combination]);
						}
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;

							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_pl[((combination * 4) + port_num)]);
						}
					}
				}

//				data_to_send = tx_data_pl[internal_loop_count];
//				MSG("DEBUG: internal_loop_count = %d\n", internal_loop_count,0,0);
//				MSG("DEBUG: data_to_send = 0x%08X\n", data_to_send,0,0);
//				PUT32(scratch_ring_tx, data_to_send);

				for (combination = 0; combination < pnum; combination++)
				{
					flag = OK;

					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
							port_num = ports;
						else
						{
							if (cflag)
								port_num = port_ctr*2;
							else
								port_num = port_ctr;
						}
						//MSG("iteration = %d, port = %d\n",iteration,port_num,0);

						ctr = 0;

						do			// Do while waiting for all the packets to be transmitted
						{
							if (ctr == 1000)	// After 1 second
							{
								eprintf("\nTimed out...No indication from scratch ring that all packets were transmitted\n");
								mailbox_value = NOT_SENT;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if the packets have been transmitted
							if ((scratch_read_value = GET32(scratch_ring_tx_count)) == 0)
							{
								hal_delay_us(2);
								ctr++;
							}
							else
							{
								switch(ports)
								{
									case 0 : 
									case 1 : 
									case 2 : 
									case 3 : 
											eprintf("Transmitted on port %d: %d           \r", ports,(scratch_read_value));
											
											break;
    
									case 4 : // all ports 
											if (!cflag)
											{
												eprintf("Transmitted on ports 0-3 : %d       \r", (scratch_read_value)/4);
												iteration = 4;
											}
											else
											{
												eprintf("Transmitted on ports 0 and 2 : %d          \r", (scratch_read_value)/2);
												iteration = 2;
											}
											break;
								}
								//MSG("Transmitted Packets (total): %d\r", scratch_read_value,0,0);
								tx_count++;
 
							}
						}
						while (scratch_read_value == 0);

						PUT32(scratch_ring_tx_count, 0);

						tx_count =  scratch_read_value;
						if (flag == ERROR)
						{
							break;
						}
					}

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}
				
				for (combination = 0; combination < pnum; combination++)
				{
	
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (!cflag)
						{
							if (ports != 4)
								port_num = ports;
							else
								port_num = port_ctr;
						}
						else
						{
							if (ports != 4)
								port_num = rxport[ports];
							else
								port_num = port_ctr*2 + 1;
						}


						ctr = 0;

						do			// Do while waiting for a signal that a packet has been received
						{
							if (ctr == 300)	// After 1 second
							{
								//eprintf("\nTimed out...No indication from scratch ring that all packets were received\n");
								//mailbox_value = NOT_RECEIVED;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if packets have been received
							if ((scratch_read_value = GET32(scratch_ring_rx)) == 0)
							{
								hal_delay_us(100);
								ctr++;
							}

							//MSG("port %d  scratch_read_value = 0x%x\n",port_num,scratch_read_value,0);
						}
						while (scratch_read_value == 0);

						if (flag == ERROR)
						{
							break;
						}

						//MSG("DEBUG: scratch_read_value = 0x%08X\n", scratch_read_value,0,0);

						rx_count++;
						pkt_size = scratch_read_value & MASK_8BIT;
						rx_port_num = (scratch_read_value >> 8) & MASK_8BIT;
						dram_addr = DRAM_BASE_DEST + ((scratch_read_value >> 16) << 8);
						// set the port on which packet was received.
						pass_port[rx_port_num]++;
						rxflag[rx_port_num] = OK;

						if (ports == 4)
						{
							exp_pkt_sz = tx_data_pl[((combination * 4) + port_num)] & 0xFFFF;
						}
						else
						{
							exp_pkt_sz = pktptr[combination] & 0xFFFF;
						}
						switch(test_param)
						{
						case 'l':
						case 'p':
						case '3':

								if (exp_pkt_sz == 0x3C)
									exp_pkt_sz = 0x84;
								break;
						}

						if (pkt_size != exp_pkt_sz)
						{
							eprintf("\nActual packet size = 0x%x\n",pkt_size);
							eprintf("expected pkt size: 0x%x\n",exp_pkt_sz);
							get_vpse_error(pkt_size, exp_pkt_sz, (UINT32*)(&mailbox_value));
							flag = ERROR;		// Set flag as error
							rxflag[rx_port_num] = ERROR;
							break;
						}

						if (cflag)
						{
							switch(rx_port_num)
							{
								case 0:  // rx on port 0 , check for pkt transmitted from port 1
									if (pkt_size == (RING_DATA_96_1 & MASK_16BIT) )
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
									}
									else if (pkt_size == (RING_DATA_64_1 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
									}
									else if (pkt_size == (RING_DATA_138_1 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
									}
									else
										bad_pkt_size = 1;
									break;

								case 1:  // rx on port 1 , check for pkt transmitted from port 0
									if (pkt_size == (RING_DATA_96_0 & MASK_16BIT))
									{
										//eprintf("received 96 byte on port 1\n");
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
									}
									else if (pkt_size == (RING_DATA_64_0 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
									}
									else if (pkt_size == (RING_DATA_138_0 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
									}
									else 
										bad_pkt_size = 1;
									break;
								case 2:  // rx on port 2 , check for pkt transmitted from port 3
									if (pkt_size == (RING_DATA_96_3 & MASK_16BIT))
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
									}
									else if (pkt_size == (RING_DATA_64_3 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
									}
									else if (pkt_size == (RING_DATA_138_3 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
									}
									else
										bad_pkt_size = 1;

									break;
								case 3:  // rx on port 2 , check for pkt transmitted from port 2
									if (pkt_size == (RING_DATA_96_2 & MASK_16BIT))
									{
										orig_data_size = PKT_96_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
									}
									else if (pkt_size == (RING_DATA_64_2 & MASK_16BIT))
									{
										orig_data_size = PKT_64_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
									}
									else if (pkt_size == (RING_DATA_138_2 & MASK_16BIT))
									{
										orig_data_size = PKT_138_SZ ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
									}
									else 
										bad_pkt_size = 1;

									break;
							}
							if (bad_pkt_size ==1)
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
							
						}
						else  // cflag = 0 -> fiber mode
						{

							if ((pkt_size == (RING_DATA_96_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
							}
							else if ((pkt_size == (RING_DATA_64_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
							}
							else if ((pkt_size == (RING_DATA_138_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
							}
							else if ((pkt_size == (RING_DATA_96_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
							}
							else if ((pkt_size == (RING_DATA_64_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
							}
							else if ((pkt_size == (RING_DATA_138_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
							}
							else if ((pkt_size == (RING_DATA_96_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
							}
							else if ((pkt_size == (RING_DATA_64_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
							}
							else if ((pkt_size == (RING_DATA_138_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
							}
							else if ((pkt_size == (RING_DATA_96_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_96_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
							}
							else if ((pkt_size == (RING_DATA_64_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_64_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
							}
							else if ((pkt_size == (RING_DATA_138_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = PKT_138_SZ;
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
							}
							else
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
						}

						for (ctr = 0; ctr < orig_data_size; ctr+=4)
						{
							// Read the DRAM for the longword that was received
							temp = GET32(dram_addr + ctr);
							// Compare with original value
							if (temp != pl_sys_lb_data[(ctr / 4)])
							{
								mailbox_value = ERROR_PATTERN;
								eprintf("Packet:%d Longword:%d Expected:0x%X  Received:0x%X\n", (rx_count + 1), (ctr / 4), pl_sys_lb_data[(ctr / 4)], temp);
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
							if (ctr == 12)
							{
								//eprintf("1st LW : 0x%x\n",temp);
								if (((temp >>16) & 0xFFFF)== VLAN_CHECK)
									VCount[rx_port_num]++;  /* vlan frame received */
								if (((temp >>16) & 0xFFFF)== PAUSE_CHECK)
									PCount[rx_port_num]++;  /* pause frame received */
							}
						}

						if (flag == ERROR)
						{
							break;
						}

						hal_delay_us(800);
					}

					if (flag == ERROR)
					{
						if ((test_param == 'l') || (test_param == 'p')||(test_param == '3') )
							;
						else
						{
							break;
						}
					}
				}

				if (flag == ERROR)
				{
					if ((test_param == 'l') || (test_param == 'p') || (test_param == '3'))
							;
					else
					{
						break;
					}
				}

	
				  loop_count++;

				if (forever && KEYPRESS)
				{
					break;
				}
			}
			while ((loop_count != loop) || forever);
			
			if (loop == 0)
			{
				counter = loop_count;
			}
			else
			{
				counter = loop;
			}

		    switch(test_param)
			{
			case 'l': 
			case 'p':
			case '3':
						expected_num_pkts = counter*6;
						break;
			case '2':
						expected_num_pkts = counter*9;
						break;
			}
			MSG("\nexpected_num_pkts = %d\n",expected_num_pkts,0,0);

			switch(ports)
			{
		       case 0:
		       case 1:
		       case 2:
		       case 3:  if (!cflag) 
						{ 
							if ((pass_port[ports] != expected_num_pkts) || (pass_port[ports] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", ports);
								eprintf("Packets Received = %d\n",pass_port[ports]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",ports,pass_port[ports]);
								flag = OK;
							}
						}
						else
						{
							if ((pass_port[rxport[ports]] != expected_num_pkts) || (pass_port[rxport[ports]] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", rxport[ports]);
								eprintf("Packets Received on port %d = %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = OK;
							}
						}
						break;

		       case 4:  eprintf("\n");
						if (!cflag)
						{
							for (ctr=0; ctr<4; ctr++)
							{
								if ((pass_port[ctr] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr);
									eprintf("Packets Received = %d\n",pass_port[ctr]);
									rxflag[ctr] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr],ctr);
								}
							}
						}
						else
						{
							for (ctr=0; ctr<2; ctr++)
							{
								if ((pass_port[ctr*2+1] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr*2+1);
									eprintf("Packets Received = %d\n",pass_port[ctr*2+1]);
									rxflag[ctr*2+1] = ERROR;
									flag = ERROR;
								}
								if (pass_port[ctr*2+1] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr*2+1],ctr*2+1);
								}
							}
						}
						
						break;
						
			   default:   eprintf("This is bad value for port number \n");
								break;
			}

		}
		eprintf("\n");

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}

		if (loop == 0)
			loop = loop_count;

		test_passed = verify_vpse_filter(cflag);
	
		if (test_passed)
		{
			switch(test_param)
			{
			case 'l':
				eprintf("\nVLAN filtering test...PASSED\n");
				break;
			case 'p':
				eprintf("\nPAUSE filtering test...PASSED\n");
			case '2':
				eprintf("\nRMON test for the following Registers ...PASSED\n");
				eprintf("Rx Tagged / Tx Tagged\n");
				break;
			case '3':
				eprintf("\nRMON test for the following Registers ...PASSED\n");
				eprintf("Rx PauseMacControlReceivedCounter / TxPauseFrames\n");
				break;

			default:
				eprintf("Invalid parameters. \n");
				return INVALID_PARAMETERS;
			}

			Set_LED("MAFP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		} 
		else
		{
			if (mailbox_value == INIT_ERROR)
			{
				eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
			}
			else if (mailbox_value == NOT_RECEIVED)
			{
				eprintf("There was no sign that the expected packet was received\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The packet received was not the same as the packet transmitted.\n");
			}
			else if (mailbox_value == INCORRECT_SIZE)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The packet received was not the same size as the packet transmitted.\n");
			}
			else if (mailbox_value == VLAN_FILTER_FAILED)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The Vlan packet was not filtered.\n");
			}
			else if (mailbox_value == PAUSE_FILTER_FAILED)
			{
				eprintf("mailbox = 0x%x\n",mailbox_value);
				eprintf("The Pause packet was not filtered.\n");
			}

			switch(test_param)
			{
			case 'l':
				strcpy(syslog.desc, "VLAN Filtering test FAILED!\n");
				eprintf("VLAN Filtering test FAILED!");
				break;
			case 'p':
				strcpy(syslog.desc, "PAUSE Filtering test FAILED!\n");
				eprintf("PAUSE Filtering test FAILED!");
			case '2':
				strcpy(syslog.desc, "Rx Tagged/Tx Tagged test FAILED!\n");
				eprintf("Rx Tagged/Tx Tagged statistics test FAILED!");
				break;
			case '3':
				strcpy(syslog.desc, "Pause statistics test FAILED!\n");
				eprintf("RMON: PAUSE statistics test FAILED!");
				break;
			}

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			Set_LED("LswF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		

		Clear_All_ME();
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}
	return DONE;
}

short verify_vpse_filter(UINT32 cflag)
{
	UINT32 rxVlanpkts[4],rxPausepkts[4],dropped[4];
	UINT32 txVlanpkts[4],txPausepkts[4];
	short test_passed = 1,iteration,port,port_ctr,res=0;
	
	if (ports == 4)
		if (!cflag)
			iteration = 4;
		else
			iteration = 2;
	else
		iteration = 1;

	spConfig();
    for(port=0; port<4;port++)
    {
		/* Reading the total packets received */
		rxVlanpkts[port] = reg_read(((port*PORTSZ)+VLAN_TAG));  
		rxPausepkts[port] = reg_read(((port*PORTSZ)+RX_PAUSE_MAC_CTL));  
		txVlanpkts[port] = reg_read(((port*PORTSZ)+TX_VLAN_TAG));  
		txPausepkts[port] = reg_read(((port*PORTSZ)+TX_PAUSE_FRAME));  
		dropped[port] = reg_read(port+PORT0_RXFIFO_PKT_DROP);  
	}
	spRestore();

	for(port_ctr=0; port_ctr<iteration;port_ctr++)
	{
		if ((!cflag) && (ports == 4))
			port = port_ctr;
		if ((cflag) && (ports == 4))
		{
			port = port_ctr*2;
			port = rxport[port];
		}
		if ((!cflag) && (ports != 4))
			port = ports;
		if ((cflag) && (ports != 4))
			port = rxport[ports];

		switch (test_param)
		{
		case 'l':
		
				eprintf("RX VLAN Tagged pkts = %d on port %d\n",rxVlanpkts[port],port);
				eprintf("Dropped VLAN pkts = %d on port %d\n",dropped[port],port);

				if ((VCount[port] != 0) || 
					(dropped[port] != (loop*EXP_DROPPED_PKTS)))
				{
					eprintf("VLAN filtering test ERROR , Port %d\n",port);
					test_passed = 0;
				}
				else 
					res|= 1<<port_ctr;

		break;

		case 'p':
				eprintf("\nRX Pause pkts = %d on port %d\n",rxPausepkts[port],port);
				eprintf("Dropped Pause pkts = %d on port %d\n",dropped[port],port);
	
				if ((PCount[port] != 0) || 
					(dropped[port] != (loop*EXP_DROPPED_PKTS)))
				{
					eprintf("Pause packet filtering test ERROR , Port %d\n",port);
					test_passed = 0;
				}
				else 
					res|= 1<<port_ctr;


				break;

		case '2':
				eprintf("\nRX VLAN Tagged pkts = %d on port %d\n",rxVlanpkts[port],port);
	
				if (rxVlanpkts[port] != (loop*VLAN_PKTS)) 
				{
					eprintf("%s    port=%d\n ", ERR_STR(RX_TAGGED_ERR),port);
					test_passed = 0;
				}
				if (cflag)
					port=rxport[port];

				eprintf("TX VLAN Tagged pkts = %d on port %d\n",txVlanpkts[port],port);

				if (txVlanpkts[port] != (loop*VLAN_PKTS)) 
				{
					eprintf("%s    port=%d\n ", ERR_STR(TX_TAGGED_ERR),port);
					test_passed = 0;
				}
				else 
					res|= 1<<port_ctr;


				break;
		case '3':
				eprintf("\nRX PAUSE pkts = %d on port %d\n",rxPausepkts[port],port);
	
				if (rxPausepkts[port] != (loop*PAUSE_PKTS)) 
				{
					eprintf("Bad value in PauseMacControlReceivedCounter register,  port=%d\n ",port);
					test_passed = 0;
				}
				if (cflag)
					port=rxport[port];

				eprintf("TX PAUSE pkts = %d on port %d\n",txPausepkts[port],port);

				if (txPausepkts[port] != (loop*PAUSE_PKTS)) 
				{
					eprintf("%s    port=%d\n ", ERR_STR(TX_PAUSEMACCONTROL_ERR),port);
					test_passed = 0;
				}
				else 
					res|= 1<<port_ctr;


				break;


		default: 
				eprintf("Illegal Test parameter\n");
				test_passed = 0;
				break;
		}

	}

	if ((res == 0xF) && (ports == 0x4) && (!cflag))
		test_passed = 1;
	else
		if ((res == 0x3) && (ports == 0x4) && cflag)
			test_passed = 1;
		else
			if ((res == 0x1) && (ports != 0x4))
				test_passed = 1;
			else
				test_passed = 0;

	return test_passed;
}


void get_packets_for_vpse_filter_test()
{
	if ((test_param == 'l') || (test_param == '2'))//vlan
	{
		pl_sys_lb_data64_0 = vlan64_0;
	}
	else if ((test_param == 'p') || (test_param == '3')) //pause
	{
		pl_sys_lb_data64_0 = pause64_0;
	}

	pl_sys_lb_data64_1 = pl_sys_lb_data64_0;
	pl_sys_lb_data64_2 = pl_sys_lb_data64_0;
	pl_sys_lb_data64_3 = pl_sys_lb_data64_0;

	pl_sys_lb_data96_0 = data96_0;
	pl_sys_lb_data96_1 = data96_1;
	pl_sys_lb_data96_2 = data96_2;
	pl_sys_lb_data96_3 = data96_3;

	pl_sys_lb_data138_0 = data138_0;
	pl_sys_lb_data138_1 = data138_1;
	pl_sys_lb_data138_2 = data138_2;
	pl_sys_lb_data138_3 = data138_3;

}


void get_vpse_error(UINT32 pkt_size, UINT32 exp_pkt_sz, UINT32 *mailbox_value)
{
	switch (test_param)
	{
	case 'l':
			if ((pkt_size == 0x3C) && (exp_pkt_sz == 0x84))
				*mailbox_value = VLAN_FILTER_FAILED;
			break;
	case 'p':
			if ((pkt_size == 0x3C) && (exp_pkt_sz == 0x84))
				*mailbox_value = PAUSE_FILTER_FAILED;
			break;
	}
}





