/* commands.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 16, 02
 * 
 * --------------------------------------------------------------------------
 */

#include "misc_func.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "uart.h"
#include "i2c.h"
#include "test_main.h"
#include "test_mngr.h"
#include "console_monitor.h"
#include "hal_ixdp2400.h"
#include "commands.h"
#include "hal_intr.h"

// static function prototypes declared here to remove compilation warnings
static void parseCmd(char *string, UINT pntr, int index);
static void cmd_help(void);
static void cmd_i2crd(void);
static void cmd_i2cwr(void);
static void cmd_test(void);
static void cmd_tflag(void);
static void cmd_print_banner(void);
static void ADUC_Interrupt_Enable(void);
static void cmd_getenet(void);
#ifdef INTERNAL_TEST
static void internal_test1(void);
static void internal_test2(void);
#endif

static DiagCmd cmdarray[] = {
	{ "t",			"s",	3, cmd_test,		0, "test, type t <cr> for help"},
	{ "test",		"s",	3, cmd_test,		0, "test, type t <cr> for help"},
	{ "h", 			"s", 	5, cmd_help, 		0, "print this message" },
	{ "help", 		"s", 	5, cmd_help, 		0, "print this message" },
	{ "?", 			"s", 	5, cmd_help, 		0, "print this message" },
	{ "p", 			"xx", 	0, cmd_print, 		0, "<addr> <size> print memory (32 bit)" },
	{ "pb", 		"xx", 	0, cmd_print, 		2, "<addr> <size> print memory (8 bit)" },
	{ "pw", 		"xx", 	0, cmd_print, 		1, "<addr> <size> print memory (16 bit)" },
	{ "pd", 		"xx", 	0, cmd_print, 		0, "<addr> <size> print memory (32 bit)" },
	{ "c", 			"x", 	0, cmd_change, 		0, "change memory (32 bits)" },
	{ "cb", 		"x", 	0, cmd_change, 		2, "change memory (8 bits)" },
	{ "cw", 		"x", 	0, cmd_change, 		1, "change memory (16 bits)" },
	{ "cd", 		"x", 	0, cmd_change, 		0, "change memory (32 bits)" },
	{ "fill", 		"XXX", 	0, cmd_fill, 		0, "<low> <high> <val> (fill mem)" },
	{ "blockcopy", 	"XXX", 	0, cmd_blockmove,	0, "copy <low> <high> <dest>" },
	{ "ion", 		"", 	0, icache_on, 		0, "enable Icache" },
	{ "ioff", 		"", 	0, icache_off, 		0, "disable Icache" },
	{ "wrv", 		"XD", 	0, cmd_wrv, 		0, "write read verify to <location> <size>" },
	{ "setbaud", 	"d", 	4, cmd_setbaud, 	0, "setbaud <baud> (...,9600,19200,38400, 57600, 115200, ...)" },
	{ "sethoe", 	"D", 	4, set_hoe, 		0, "set Halt On Error"},
	{ "don", 		"", 	4, dcache_on, 		0, "create pagetable, enable Dcache, enable wb " },
	{ "doff", 		"", 	4, dcache_off, 		0, "disable Dcache" },
	{ "tf",			"xx",	6, cmd_tflag,		0, "enable DIAG T ALL test flags <device> <dbg level>"},
	{ "memrb",  	"Xdd", 	2, cmd_memrb, 		0, "read byte from PCI memory <addr> <rept> <silent>" },
	{ "memwb", 		"XXdd", 2, cmd_memwb, 		0, "write byte to PCI memory <addr> <data> <rept> <silent>" },
	{ "memrw",  	"Xdd", 	2, cmd_memrw, 		0, "read word from PCI memory <addr> <rept> <silent>" },
	{ "memww", 		"XXdd", 2, cmd_memww, 		0, "write word to PCI memory <addr> <data> <rept> <silent>" },
	{ "memrd",  	"Xdd", 	2, cmd_memrd, 		0, "read double word from PCI memory <addr> <rept> <silent>" },
	{ "memwd", 		"XXdd", 2, cmd_memwd, 		0, "write double word to PCI memory <addr> <data> <rept> <silent>" },
	{ "iorb", 		"Xxd", 	2, cmd_iorb, 		0, "PCI byte read port <addr> <rept> <silent>" },
	{ "iowb", 		"XXdd", 2, cmd_iowb, 		0, "PCI byte write port <addr> <data> <rept>" },
	{ "iorw", 		"Xxd", 	2, cmd_iorw, 		0, "PCI word read port <addr> <rept> <silent>" },
	{ "ioww", 		"XXdd", 2, cmd_ioww, 		0, "PCI word write port <addr> <data> <rept>" },
	{ "iord", 		"Xxd", 	2, cmd_iord, 		0, "PCI long read port <addr> <rept> <silent>" },
	{ "iowd", 		"XXdd", 2, cmd_iowd, 		0, "PCI long write port <addr> <data> <rept>" },
	{ "cfgrb",		"DDX", 	2, cmd_cfgrb,		0, "PCI cfg <bus> <slot> read byte <addr>"},
	{ "cfgwb",		"DDXX", 2, cmd_cfgwb,		0, "PCI cfg <bus> <slot> write byte <addr> <data>"},
	{ "cfgrw",		"DDX", 	2, cmd_cfgrw,		0, "PCI cfg <bus> <slot> read word <addr>"},
	{ "cfgww",		"DDXX", 2, cmd_cfgww,		0, "PCI cfg <bus> <slot> write word <addr> <data>"},
	{ "cfgrd",		"DDX", 	2, cmd_cfgrd,		0, "PCI cfg <bus> <slot> read long <addr>"},
	{ "cfgwd",		"DDXX", 2, cmd_cfgwd,		0, "PCI cfg <bus> <slot> write long <addr> <data>"},
	{ "i2cread",	"XXD", 	2, cmd_i2crd,		0, "I2C read <X:slave_addr> <X:offset_addr> <D:count>"},
	{ "i2cwrite",	"XXS", 	2, cmd_i2cwr,		0, "I2C write <X:slave_addr> <X:offset_addr> <S:data>"},
	{ "banner", 	"", 	0, cmd_print_banner,1, "Print Banner "},
	{ "tempread", 	"d", 	0, TMon_Read,		0, "Read Temperature Limits"},
	{ "tempset", 	"DDDD", 3, TMon_Set_Limits,	0, "Set Temperature Limits <I_low> <I_high> <E_low> <E_high>"},
	{ "voltread", 	"", 	0, VMon_Read,		0, "Read Voltage Limits"},
	{ "voltset",	"sdd", 	3, VMon_Set_Limits,	0, "Set Voltage Limits <s:type> <d:low> <d:high>"},
	{ "getenet",	"", 	0, cmd_getenet, 	1, "get NIC ethernet address"},
	{ "aducset",	"", 	0, ADUC_Interrupt_Enable, 0, "Enables ADUC interrupt"},
	{ "exit",		"", 	0, cmd_reset, 		1, "Exit Diagnostics"},
	{ "viewlog",	"D", 	0, cmd_viewlog, 	1, "View the entries in the log"},
	{ "clearlog",	"", 	0, syslog_clear, 	0, "Clear all entries in the log"},
	{ "setverbose",	"d", 	0, cmd_set_verbose, 1, "Set verbose level <verbose_level>"},
	{ "linerate",	"", 	0, Line_Rate_Test, 	0, "Run linerate test"},
#ifdef INTERNAL_TEST
	{ "itest1",		"xxdd",	3, internal_test1,	0, "test, type t <cr> for help"},
	{ "itest2",		"xxxd", 3, internal_test2,		0, "test, type t <cr> for help"},
#endif
	{ 0, 0, 0, 0, 0}
};

#ifdef INTERNAL_TEST
static void internal_test1()
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned int start, size, num_of_patterns, loops;
	unsigned int pattern_array[64], i;
	PUINT p;

	if(acL->argc < 5)
	{
		eprintf("**ERROR**: missing arguments.\n");
		eprintf(">itest1 [x:start_addr] [x:size] [d:number_of_patterns] [d:loops]\n");
	}
	else
	{
		start = acL->hexArg[1];
		size = acL->hexArg[2];
		num_of_patterns = acL->decArg[3];
		loops = acL->decArg[4];

		if(num_of_patterns > 64)
		{
			eprintf("number of patterns is limited to 64 only.\n");
		}
		else
		{
			p = (PUINT)SCRATCHPAD_BASE;
			for(i = 0; i < num_of_patterns; i++)
				pattern_array[i] = p[i];

			do
			{
				for(i = 0; i < size; i+=4)
				{
					PUT32(start + i, pattern_array[(i/4)%num_of_patterns]);
				}
				PutChar('.');
				if(KEYPRESS)
					return;
			} while(loops--);
		}
	}
	return;
}

static void internal_test2()
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned int read_addr, write_addr, size, verbose, loops;
	unsigned int i;
	volatile unsigned int read_1, read_2, read_3, read_4;

	if(acL->argc < 6)
	{
		eprintf("**ERROR**: missing arguments.\n");
		eprintf(">itest2 [x:read_addr] [x:write_addr] [x:size] [d:loops] [d:verbose]\n");
	}
	else
	{
		read_addr = acL->hexArg[1];
		write_addr = acL->hexArg[2];
		size = acL->hexArg[3];
		loops = acL->decArg[4];
		verbose = acL->decArg[5];

		
		do
		{
			for(i = 0; i < size; i+=16)
			{
				read_1 = GET32(read_addr + i);
				read_2 = GET32(read_addr + i + 4);

				PUT32(write_addr + i,     read_1);
				PUT32(write_addr + i + 4, read_2);

				read_3 = GET32(read_addr + i + 8);
				read_4 = GET32(read_addr + i + 12);

				PUT32(write_addr + i + 8,  read_3);
				PUT32(write_addr + i + 12, read_4);

				if(verbose)
				{
					eprintf("0x%08x: %08x %08x %08x %08x\n",
							read_addr + i, read_1, read_2, read_3, read_4);
				}
				if(KEYPRESS)
					return;
			}
		} while(loops--);
	}
	return;
}
#endif

//
// Function: getCommands
//
// Description: Interface from the Test Manager and sets
//				the display for prompt and the array name
//				to be search for routines
//
void getCommands()
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	if(acL->HostType == MASTER)
		parseCmd("MASTER_DIAG> ",(UINT)cmdarray,(int)0);  // the prompt of course
	else
		parseCmd("SLAVE_DIAG> ",(UINT)cmdarray,(int)0);  // the prompt of course

}

//
// Function: parseCmd
//
// Decription:	Displays the prompt, then gets users input.
//				Separates user arguments to their respective
//				arrays. Calls findCmd and calls the routine
//				if the command matches a routine
//
static void parseCmd(char *string, UINT pntr, int index)
{	
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	DiagCmd *findCmd(UINT, int), *cmd;
	char* s;
	int i = 0;
	
	bzero(acL->cmdLine, 128); /* initialize the command buffer */
	eprintf("%s",string);

	getCmdLine(acL->cmdLine);	// read user input from command line
	s = acL->cmdLine;	// point to the cmdLine buffer

	// Clear arg count, and all the arguments
	acL->argc = 0;
	for(i = 0; i < 10; i++)
	{
		acL->hexArgOk[i] = FALSE; acL->decArgOk[i] = FALSE;
		acL->hexArg[i] = 0; acL->decArg[i] = 0; acL->argv[i] = 0;
	}

	while(1)
	{
	    while(*s == ' ')	// skip spaces
			s++;
	    if (*s == 0)	// if end of string, exit scanner
			break;
	    acL->argv[acL->argc++] = s;	// store beginning of string
	    if (*s == '"')
		{
	      acL->argv[acL->argc - 1]++;
	      while ((*s != '"') && (*s != 0))
		  	s++;    // skip argument
	      if(*s == '"')
		  	*s++ = 0;
	    }
		else
		{
	      while((*s != ' ') && (*s != 0))
		  	s++;	// skip argument
	      if(*s == ' ')
		  	*s++ = 0;	// put in a terminator, if necessary
	    }
	}

	if(acL->argc == 0)
		return;

	for (i = 1; i < acL->argc; i++) makeNumber(i);

	// Search for the command in our command table
	cmd = findCmd(pntr,index);

	if(cmd != 0)
	{
		acL->numArgs = cmd->numArg; /* load up the arguments */
		(*(cmd->routine))();
	}
	else
		eprintf("***Error***: Command not found\n");
}


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* function: makeNumber()                                                   */
/*                                                                          */
/* purpose: to convert the ascii string in the argv[i] array to an actual   */
/*          binary digit.                                                   */
/*                                                                          */
/* calls: isDigit(), tolower()                                              */
/*                                                                          */
/* abstract: the ascii number will be parsed from start to finish. When     */
/*           done, the decarg array will contain valid values               */
/*                                                                          */
/* returns: void                                                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
void makeNumber(int i)
{
  register PDiagCommon acL = (PDiagCommon) ACADDRESS;
  char *s, c;
  int v = 0;

  s = acL->argv[i];                   /* initialize all the arrays */
  acL->hexArg[i] = 0; acL->decArg[i] = 0;
  acL->decArgOk[i] = TRUE; acL->hexArgOk[i] = TRUE;
  while (*s) {
    c = *s++;
    if (isdigit(c))                   /* check if argument is a number */
      v = c - '0';                    /* if so, then make it binary */
    else {
      acL->decArgOk[i] = FALSE;       /* if hex, then we may have failed IsDigit */
      c = tolower(c);                 /* lower it */
      if (((c >= 'a') && (c <= 'f'))) {
        v = c - 'a' + 10;             /* see if its hex, if so, make it binary */
      }
      else acL->hexArgOk[i] = FALSE;
    }
    if (acL->decArgOk[i])             /* fill in the decimal argument array */
      acL->decArg[i] = (acL->decArg[i] * 10) + v;
    if (acL->hexArgOk[i])             /* fill in the hex argument array */
      acL->hexArg[i] = (acL->hexArg[i] * 16) + v;
  }
}

//
// Function: findCmd
//
// Description: Looks for a command in an array. If not found,
//				displays "unknown command"
//
DiagCmd *findCmd(unsigned int pntr, int idx)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	DiagCmd *p;
	int wantargs, matchOk, i;
	char syn_curr;
	p = (DiagCmd*) pntr;	/* point to the command/test array */

	while (p->cmdw)	/* search til there aren't any more commands in the table*/
	{
    	if (strcmp(acL->argv[idx], p->cmdw) == 0)
		{
    		matchOk = TRUE;                   /* we found a match, now check syntax */
    		wantargs = strlen(p->syntax);
			//if(wantargs == (acL->argc - 1))	// cannot be done since "test" has variable args
			for (i = 1; i <= wantargs; i++)
			{
	        	syn_curr = p->syntax[i-1];
	        	if ((syn_curr=='X') && !acL->hexArgOk[i+idx])
					matchOk = FALSE;
	        	if (syn_curr=='x' && (i+idx) < acL->argc && !acL->hexArgOk[i+idx])
					matchOk = FALSE;
	    	    if ((syn_curr =='D') && !acL->decArgOk[i+idx])
					matchOk = FALSE;
	        	if (syn_curr=='d' && (i+idx) < acL->argc && !acL->decArgOk[i+idx])
					matchOk = FALSE;
	        	if (syn_curr=='S' && i >= acL->argc)
					matchOk = FALSE;
	      	}			

	      	if (matchOk)
	  			return(p);           /* checks out, return pointer */
			else
				eprintf("Syntax is incorrect, type 'h' to get help\n");

      		if (idx)
	  			eprintf("syntax error... type 't %s %s'\n",acL->argv[1],p->helpstr);      
    		else
	  			eprintf("syntax error... type 'h %s'\n",acL->argv[0]);
		      
	    	return(0);                        /* syntax is wrong, return fail status */
		}
    	p++;                                /* bump to next command in array */
	}
	eprintf("unknown command\n");
	return(0);
}

//
// Function: cmd_help
//
// Description: Displays all commands available to the user at the highest level
//
static void cmd_help()
{
	/*
	This function is invoked when the user types the help command at the Diag prompt.
	This function will display a help message on all the commands available in diagnostics. 
	*/
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	DiagCmd *p;
	char *s, syncmd;

	p = (DiagCmd *)&cmdarray[0];
	if (acL->argc == 1)
	{
		eprintf("\n");
		eprintf("Access Commands:\n");
		eprintf("blockcopy.....copy memory block          c[bwd].......change memory contents\n");
		eprintf("cfg[rw][bwd]..PCI config access          fill.........fill memory with pattern\n");
		eprintf("getenet.......get NIC Ethernet address   io[rw][bwd]..PCI IO access\n");
		eprintf("i2cread.......Read an I2C device         i2cwrite.....Write an I2C device\n");
		eprintf("mem[rw][bwd]..PCI memory access          p[bwd].......print memory contents\n");
		eprintf("tempread......Read temperature limits    tempset......Set temperature limits\n");
		eprintf("voltread......Read voltage limits        voltset......Set voltage limits\n");
		eprintf("viewlog.......view entries in log        wrv..........Write-Read-Verify memory\n");
		
		eprintf("\n");
		eprintf("Kernel Commands:\n");                         
		eprintf("don...........dcache enable              doff.........dcache off\n");
		eprintf("ion...........icache enable              ioff.........icache off\n");
		eprintf("setbaud.......set serial baud rate       tf...........set DIAG test flags\n");
		
		eprintf("\n");
		eprintf("Utility Commands:\n");
		eprintf("?.............display this help screen   banner.......Print banner\n");
		eprintf("exit..........Exit Diagnostics           h(elp).......display this help screen\n");
		eprintf("sethoe........Set Halt On Error          setverbose...Set verbose level\n");
		eprintf("t(est)........Run DIAG test\n");
		
		eprintf("\n");
		eprintf("Note: additional help is available on the above commands,\n");
		eprintf("for help on each command type 'h command<cr>'.\n\n");
	}
	else
	{
		while (p->cmdw)
		{                   /* print expected syntax for each command */
	  		if (strcmp(p->cmdw, acL->argv[1]) == 0)
			{
		    	eprintf("%s: %s\n", p->cmdw, p->helpstr);
			    s = p->syntax;
			    eprintf("arguments: ");
			    while(*s)
				{
					syncmd = *s++;
					switch (syncmd)
					{
						case 'X': eprintf("<hex> ");break;
						case 'x': eprintf("<opt hex> ");break;
						case 'D': eprintf("<dec> ");break;
						case 'd': eprintf("<opt dec> ");break;
						case 'S': eprintf("<str> ");break;
						case 's': eprintf("<opt str> ");break;
						case ' ': eprintf("<none> ");break;
					}
		    	}
		    	eprintf("\n");
				return;
			}
			p++;
		}
		eprintf("***ERROR***: Help not found for command. Please ensure command exist\n");
	}
}

// *Note: only use for bring up
// Function: cmd_i2crd
//
// Description: allows the user to read any location of
//				any I2C PROM
//
static void cmd_i2crd(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned char slave_addr = acL->hexArg[1];
	unsigned char offset_addr = acL->hexArg[2];
	unsigned char count = acL->decArg[3];
	unsigned char buffer[56] = "";
	unsigned char i = 0;

    if(acL->HostType != MASTER)
	{
		eprintf("This command can only be run on master NPU\n");
		return;
	}

	if( (offset_addr + count) > 255)
	{
		eprintf("The address and count used exceeds the address of 255\n");		
	}
	else
	{
		i2c_seq_read(slave_addr, offset_addr, buffer, count);

		for(i = 0; i < count; i++)
			eprintf("%x ", buffer[i]);
		eprintf("\n");
	}
}


// *Note: only use for bring up
// Function: cmd_i2cwr
//
// Description: allows the user to write any location of
//				any I2C PROM
//
static void cmd_i2cwr(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned char buffer[56] = "", s[30] = "", count, c;
	unsigned int i;
	
    if(acL->HostType != MASTER)
	{
		eprintf("This command can only be run on master NPU\n");
		return;
	}

	count = strlen(acL->argv[3])/2;

	for(i = 0; i < count; i++)
	{
		c = tolower (acL->argv[3][i*2]);
		if (isdigit (c) || ((c >= 'a') && (c <= 'f')))
		{
			if(c <= '9')
				buffer[i] = ((unsigned char)(c - '0'))<<4;
			else
				buffer[i] = ((unsigned char)(c - 'a' + 10))<<4;
		}

		c = tolower (acL->argv[3][i*2 + 1]);
		if (isdigit (c) || ((c >= 'a') && (c <= 'f')))
		{
			if(c <= '9')
				buffer[i] += (unsigned char)(c - '0');
			else
				buffer[i] += (unsigned char)(c - 'a' + 10);
		}
	}
	eprintf("Please enter password to write:");
	getCmdLine(s);	// read user input from command line
	if(strcmp(s, "ixdp2400") == 0)
		i2c_page_write(acL->hexArg[1], acL->hexArg[2], buffer, count);
	else
		eprintf("Incorrect password!\n");
}

//
// Function: cmd_tflag
//
// Description: allows the user to set the flags for which test
//				to run when the "alltest" command is invoked
//
static void cmd_tflag(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->argc != 3)
	{
		eprintf("Usage of the command tf:\n");
		eprintf("> tf <h:ID> <h:status (0/1)>\n");
		eprintf("Test Type  ID\t status\n");
		eprintf("=========  ==\t ======\n");
		eprintf("SRAM       0 \t %s\n", (acL->tFlag[TFLAG_SRAM])?	 "ENABLED" : "DISABLED");
		eprintf("SDRAM      1 \t %s\n", (acL->tFlag[TFLAG_SDRAM])?	 "ENABLED" : "DISABLED");
		eprintf("UENG       2 \t %s\n", (acL->tFlag[TFLAG_UENG])?	 "ENABLED" : "DISABLED");
		eprintf("TIMERS     3 \t %s\n", (acL->tFlag[TFLAG_TIMERS])?	 "ENABLED" : "DISABLED");
		eprintf("NIC        4 \t %s\n", (acL->tFlag[TFLAG_ENET])?	 "ENABLED" : "DISABLED");
		eprintf("PCI        5 \t %s\n", (acL->tFlag[TFLAG_PCI])? 	 "ENABLED" : "DISABLED");
		eprintf("XSCALE     6 \t %s\n", (acL->tFlag[TFLAG_XSCALE])?	 "ENABLED" : "DISABLED");
		eprintf("I2C        7 \t %s\n", (acL->tFlag[TFLAG_I2C])?	 "ENABLED" : "DISABLED");
		eprintf("GPIO       8 \t %s\n", (acL->tFlag[TFLAG_GPIO])?	 "ENABLED" : "DISABLED");
		eprintf("LED        9 \t %s\n", (acL->tFlag[TFLAG_LED])?	 "ENABLED" : "DISABLED");
		eprintf("UART       A \t %s\n", (acL->tFlag[TFLAG_UART])?	 "ENABLED" : "DISABLED");
		eprintf("SF         B \t %s\n", (acL->tFlag[TFLAG_SF])?		 "ENABLED" : "DISABLED");
		eprintf("LOOPBACK   C \t %s\n", (acL->tFlag[TFLAG_LOOPBACK])?"ENABLED" : "DISABLED");
		eprintf("TMON       D \t %s\n", (acL->tFlag[TFLAG_TMON])?	 "ENABLED" : "DISABLED");
		eprintf("FLASH      E \t %s\n", (acL->tFlag[TFLAG_FLASH])?	 "ENABLED" : "DISABLED");
		eprintf("LINERATE   F \t %s\n", (acL->tFlag[TFLAG_LINERATE])?"ENABLED" : "DISABLED");
	}
	else
	{
		//tFlag[device] = debug_level
		acL->tFlag[acL->hexArg[1]] = acL->hexArg[2];
	}
}


// Function: cmd_test
//
static void cmd_test(void)
{
	Diag_Tests();
}

// Function: cmd_print_banner
//
static void cmd_print_banner(void)
{
	Print_Banner();
}

//
// Function: ADUC_Interrupt_Enable
//
// Description: Enables ADUC interrupt
//
static void ADUC_Interrupt_Enable(void)
{
	pciIntConnect(CPLD_ADUC_INT, (UINT32)Usr_Temp_Ctrl_ISR, 0);	// for temperature & voltage
}

//
// Function: cmd_getenet
//
// Description: Displays the Ethernet MAC address
//
static void cmd_getenet(void)
{
	extern int get_ether_addr(int display);
	get_ether_addr(ENABLED);
}
