/* test_main.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 21, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */
#include "diagstruct.h"
#include "test_main.h"
#include "misc_func.h"
#include "commands.h"
#include "uart.h"

extern UINT32 sram_chan[], sram_size[];
extern void mac_test(void);
extern void Dual_Media_Test(void);
extern void Media_On_SF_IF_Test(void);

volatile UINT32 t_all_verbose = MAXIMUM;

DiagCmd testarray[] = {
/*	cmd str,   syntax, hflag, (*routine),numArg, helpstr */
    { "mac",	"sssdsdd",	 0,	mac_test,	0,"<test_opt><media loopback mode><channel_opt><speed><duplex><port>[loop_count]"},
	{ "mem",	"ssxxxx",3,	Memory_Test, 	0,"<mem_opt><test opt><bgn><size>[pattern][loops]"},
	{ "led",	"ss",	 0,	LED_Test,		0,"<test opt>[4 char]"},
	{ "xscale",	"ss",	 0,	Xscale_Test,	0,"<test opt>"},	
	{ "uart", 	"s",	 0,	UART_Test,		0,"<test opt>"},
	{ "all",	"dd",	 0,	all_test,		0,"[d:loops][d:verbose]"},
	{ "flash",	"s", 	 0,	Flash_Test, 	0,"<test opt>"},
	{ "gpio",	"s",	 0,	GPIO_Test,		0,"<test opt>"},
	{ "i2c",	"sxx",	 0,	I2C_Test,		0,"<test opt><addr><data>"},
	{ "lb",		"s",	 0,	Loopback_Test, 	0,"<test opt>[d:loops][media loopback mode][slave verbose]"},
	{ "linerate","s", 	 0,	Line_Rate_Test, 0,"<test opt>"},
	{ "nic", 	"sddd", 0,	Ethernet_test, 	0,"<test opt>[d:link_speed][d:loops][d:verbose]"},
	{ "pci",	"s",	 0,	Pci_Test,		0,"<test opt>"},
	{ "sf",		"s",	 0,	Msf_Test,		0,"<test opt>"},
	{ "dual",	"sd",	 0, Dual_Media_Test,0,"<media_opt>[d:loops]"},
	{ "mediasf","sd",	 0, Media_On_SF_IF_Test,0,"<media_opt>[d:loops]"},
#ifdef NPU_SCREEN
	{ "screen",	"",	 0,	screen_test,		0,""},
#endif
	{ "timers",	"s",	 1,	Timer_Test,		0,"<test opt>"},
	{ "tmon", 	"s", 	 0,	TMon_Test, 		0,"<test opt>"},
	{ "ueng",	"sd",	 0,	ME_Test,		0,"<test opt> [d:microengine #]"},
#ifdef EMI
	{ "emi",	"ssds",		 0, EMI_Test,		0, ""},
#endif

#ifdef INTERNAL_TEST
	{ "analog",	"s",		 0, Analog_Measurement,		0, ""},
#endif
	{0,0,0,0,0,0}
};

//
// Function: Diag_Tests
//
// Description: Interface to all the Test Suites from the Test Manager.
//				Displays all the tests available and the arguments required
//				for each test if no test is specified. If a test is specified,
//				search for the routine. If the routine cannot be found (no such
//				test exisits) then it displays "unknown test command"
//
void Diag_Tests(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	DiagCmd *findCmd(UINT, int), *cmd;
	
	if (acL->argc == 1)
	{
		eprintf("\nusage: t 'name' <args> [passes] 0=forever \n");
	    eprintf("where 'name' <args> is one of the following:\n\n");
	    eprintf(" all             (tests the system with tests that are enabled by tflags)\n");
		eprintf(" flash           (tests the flash lock capability)\n");
	    eprintf(" gpio            (tests the GPIO registers and some selected pins)\n");
		eprintf(" i2c             (tests the capability to access the I2C)\n");
		eprintf(" led             (tests the LED in displaying alpha-numeric characters)\n");
//		eprintf(" linerate        (tests the system in handling traffic at the media)\n");
		eprintf(" lb              (tests the media, switch fabric and system in loopback mode)\n");
		eprintf(" mem             (tests QDR and DDR memory through multiple tests)\n");
		eprintf(" nic             (tests the NIC through self, internal, and external tests)\n");
		eprintf(" pci             (tests some of the PCI functionalities)\n");
		eprintf(" sf              (tests some MSF registers and switch fabric functionalities)\n");
	    eprintf(" timers          (tests the capability of the timers (interrupt driven))\n");
		eprintf(" tmon            (tests the ADUC's ID and WRV of its registers)\n");
	    eprintf(" uart            (tests UART's read/write functionality)\n");
		eprintf(" ueng            (tests microengine's generic functionality)\n");
		eprintf(" xscale          (tests XScale's Interrupts and Scratchpad)\n");
#ifdef INTERNAL_TEST
		eprintf(" analog          (runs analog measurement functions)\n");
#endif
		return;
	}

	cmd = findCmd((UINT)testarray,1);   /* find the command in our table */
	if (cmd != 0)
	{
	    acL->numArgs = cmd->numArg;  /* load up the arguments */
		(*(cmd->routine))();        /* call the requested routine now... */
	}
	else
		eprintf("*** unknown test command ***\n");
}

//
// Function: Seperate_Arguments
//
// Description: This function is a subset of the parseCmd function in
//				commands.c. Intead of getting a string from getCmdLine,
//				it takes the string arg and does the same thing
//
void Seperate_Arguments(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int i;
	char* s;

	s = acL->cmdLine;

	// Clear arg count, and all the arguments
	acL->argc = 0;
	for(i = 0; i < 10; i++)
	{
		acL->hexArgOk[i] = FALSE; acL->decArgOk[i] = FALSE;
		acL->hexArg[i] = 0; acL->decArg[i] = 0; acL->argv[i] = 0;
	}

	while(1)
	{
	    while(*s == ' ')	// skip spaces
			s++;
	    if (*s == 0)	// if end of string, exit scanner
			break;
	    acL->argv[acL->argc++] = s;	// store beginning of string
	    if (*s == '"')
		{
	      acL->argv[acL->argc - 1]++;
	      while ((*s != '"') && (*s != 0))
		  	s++;    // skip argument
	      if(*s == '"')
		  	*s++ = 0;
	    }
		else
		{
	      while((*s != ' ') && (*s != 0))
		  	s++;	// skip argument
	      if(*s == ' ')
		  	*s++ = 0;	// put in a terminator, if necessary
	    }
	}

	for (i = 1; i < acL->argc; i++) makeNumber(i);
}

//
// Function: all_test
//
// Description: Runs the test which its flag is set to be run
//				when "t all" is entered by the user at the
//				command prompt
//
void all_test(void)
{
	int forever = 0, loops = 1, count = 1;
	int all_test_status = OK;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	t_all_verbose = MINIMAL;

	if(acL->argc < 3)
		forever = 0;
	else
	{
		loops = acL->decArg[2];
		if(loops == 0)
			forever = 1;
		else
			forever = 0;

		if(acL->argc >= 4)
		{
			if(acL->decArg[3] == 0x1)
				t_all_verbose = MAXIMUM;
			else
				t_all_verbose = MINIMAL;
		}
	}

	while(loops-- || forever)
	{
		eprintf("Loop number: %d\n", count++);
		all_test_status = OK;

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_SRAM])
		{
			eprintf("SRAM Test:\n");
			eprintf("==========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t mem s a 0 400000 12341234 1 0");
			Seperate_Arguments();
            acL->hexArg[5] = (sram_size[1] >= sram_size[0]) ? sram_size[1] : sram_size[0];
			Memory_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("SRAM TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_SDRAM])
		{
			eprintf("SDRAM Test:\n");
			eprintf("===========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t mem d a 0 1FE00000 12341234 1 0");
			Seperate_Arguments();
			Memory_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("SDRAM TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_UART])
		{
			eprintf("UART Test:\n");
			eprintf("==========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t uart a");
			Seperate_Arguments();
			UART_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("UART TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_XSCALE])
		{
			eprintf("XSCALE Test:\n");
			eprintf("============\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t xscale a");
			Seperate_Arguments();
			Xscale_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("XSCALE TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_GPIO])
		{
			eprintf("GPIO Test:\n");
			eprintf("==========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t gpio a");
			Seperate_Arguments();
			GPIO_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("GPIO TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_TIMERS])
		{
			eprintf("Timers Test:\n");
			eprintf("============\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t timers g");
			Seperate_Arguments();
			Timer_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("TIMER TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_I2C])
		{
			eprintf("I2C Test:\n");
			eprintf("=========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t i2c a 80 ab");
			Seperate_Arguments();
			I2C_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("I2C TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_LED])
		{
			eprintf("LED Test:\n");
			eprintf("=========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t led n");
			Seperate_Arguments();
			LED_Test();
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_UENG])
		{
			eprintf("Microengine Test:\n");
			eprintf("=================\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t ueng a");
			Seperate_Arguments();
			ME_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("MICROENGINE TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_FLASH])
		{
			eprintf("Flash Test:\n");
			eprintf("===========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t flash l");
			Seperate_Arguments();
			Flash_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("FLASH TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_PCI])
		{
			eprintf("PCI Test:\n");
			eprintf("=========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t pci a");
			Seperate_Arguments();
			Pci_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("PCI TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_ENET])
		{
			eprintf("NIC Test:\n");
			eprintf("=========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t nic a");
			Seperate_Arguments();
			Ethernet_test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("NIC TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_TMON])
		{
			eprintf("TMon Test:\n");
			eprintf("==========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t tmon a");
			Seperate_Arguments();
			TMon_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("TMON TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_SF])
		{
			eprintf("MSF Test:\n");
			eprintf("=========\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t sf a");
			Seperate_Arguments();
			Msf_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("MSF TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;

		if(acL->tFlag[TFLAG_LOOPBACK])
		{
			eprintf("Loopback Test:\n");
			eprintf("==============\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t lb a");
			Seperate_Arguments();
			Loopback_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
			}
			else
			{
				eprintf("LOOPBACK TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}

		if(KEYPRESS)
			break;
/*
		if(acL->tFlag[TFLAG_LINERATE])
		{
			eprintf("Linerate Test:\n");
			eprintf("==============\n");
			bzero(acL->cmdLine, 128); // initialize the command buffer
			strcpy(acL->cmdLine, "t lr");
			Seperate_Arguments();
			// *TODO* Line_Rate_Test();
			if(acL->Err != OK)
			{
				all_test_status = ERROR;
				eprintf("LINERATE TEST FAIL\n");
			}
			else
			{
				eprintf("LINERATE TEST PASS\n");
			}
			if(acL->Err && acL->HaltOnError)
				break;
		}*/

		if(all_test_status != ERROR)
			eprintf("All Diagnostics tests PASS\n");
	}

	t_all_verbose = MAXIMUM;
}

#ifdef NPU_SCREEN
//
// Function: screen_test
//
// Description: Runs the test determined for screening purposes
//		when "t screen" is entered by the user
//
void screen_test(void)
{
	int all_test_status = OK;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	t_all_verbose = MAXIMUM;

	eprintf("SRAM Test:\n");
	eprintf("==========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, " t mem s a 0 400000 12341234");
	Seperate_Arguments();
    acL->hexArg[5] = (sram_size[1] >= sram_size[0]) ? sram_size[1] : sram_size[0];
	Memory_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("SRAM TEST PASS\n");
	}

	eprintf("SDRAM Test:\n");
	eprintf("===========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, " t mem d a 0 800000 12341234");
	Seperate_Arguments();
	Memory_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("SDRAM TEST PASS\n");
	}

	eprintf("PCI Test:\n");
	eprintf("=========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t pci a");
	Seperate_Arguments();
	Pci_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("PCI TEST PASS\n");
	}

	eprintf("NIC Test:\n");
	eprintf("=========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t nic a");
	Seperate_Arguments();
	Ethernet_test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("NIC TEST PASS\n");
	}

	eprintf("Microengine Test:\n");
	eprintf("=================\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t ueng a");
	Seperate_Arguments();
	ME_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("MICROENGINE TEST PASS\n");
	}
	
	eprintf("Timers Test:\n");
	eprintf("============\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t timers g");
	Seperate_Arguments();
	Timer_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("TIMER TEST PASS\n");
	}
	
	eprintf("XSCALE Test:\n");
	eprintf("============\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t xscale a");
	Seperate_Arguments();
	Xscale_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("XSCALE TEST PASS\n");
	}

	eprintf("GPIO Test:\n");
	eprintf("==========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t gpio a");
	Seperate_Arguments();
	GPIO_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("GPIO TEST PASS\n");
	}

	eprintf("MSF Test:\n");
	eprintf("=========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t sf a");
	Seperate_Arguments();
	Msf_Test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("MSF TEST PASS\n");
	}

	eprintf("MAC media loopback Test:\n");
	eprintf("=========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t mac m i f 1000 f 4 10");
	Seperate_Arguments();
	mac_test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("MAC MEDIA LOOPBACK TEST PASS\n");
	}

	eprintf("MAC system loopback Test:\n");
	eprintf("=========\n");
	bzero(acL->cmdLine, 128); // initialize the command buffer
	strcpy(acL->cmdLine, "t mac y i f 1000 f 0 10");
	Seperate_Arguments();
	mac_test();
	if(acL->Err != OK)
	{
		all_test_status = ERROR;
		if(acL->HaltOnError)
		{
			t_all_verbose = MINIMAL;
			return;
		}
	}
	else
	{
		eprintf("MAC SYSTEM LOOPBACK TEST PASS\n");
	}

	if(all_test_status != ERROR)
		eprintf("All Diagnostics tests PASS\n");


	t_all_verbose = MINIMAL;

}
#endif
