/* test_mngr.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 21, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "test_mngr.h"
#include "memory.h"
#include "i2c_test.h"
#include "i2c.h"
#include "led.h"
#include "commands.h"
#include "syslog.h"
#include "diag_utils.h"
#include "hal_ixdp2400.h"
#include "ethernet.h"
#include "hal_intr.h"

// static function prototypes
static char* Get_CPU(void);
static int Get_CPU_Speed(void);
static int Get_CPU_Rev(void);
static int Get_SRAM_Size(void);
static int Get_SDRAM_Size(void);
static int Get_Host_Type(void);

extern void UART_Interrupt_Handler(void);

extern void Timer_Interrupt_Handler(void);
#ifdef EMI
extern void EMI_Timer2_Interrupt_Handler(void);
#endif

extern void XScale_Soft_Interrupt_Handler(void);
extern void Doorbell_Intr_Received(void);
extern void Usr_Temp_Ctrl_ISR(void);

extern UINT32 sram_size[2];
extern UINT32 strap_options_val;

void install_other_isr()
{
    HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_SERIAL, &UART_Interrupt_Handler,
						HAL_INTERRUPT_SERIAL, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_SERIAL);

#ifdef EMI
	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_TIMER_2, &EMI_Timer2_Interrupt_Handler,
						HAL_INTERRUPT_TIMER_2, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_TIMER_2);
#else
	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_TIMER_2, &Timer_Interrupt_Handler,
						HAL_INTERRUPT_TIMER_2, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_TIMER_2);
#endif

	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_TIMER_3, &Timer_Interrupt_Handler,
						HAL_INTERRUPT_TIMER_3, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_TIMER_3);

	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_TIMER_4, &Timer_Interrupt_Handler,
						HAL_INTERRUPT_TIMER_4, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_TIMER_4);

	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_PCI_DBELL, &Doorbell_Intr_Received,
						HAL_INTERRUPT_PCI_DBELL, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_PCI_DBELL);

	HAL_INTERRUPT_ATTACH(HAL_INTERRUPT_SOFT, &XScale_Soft_Interrupt_Handler,
						HAL_INTERRUPT_SOFT, 0);
	HAL_INTERRUPT_UNMASK(HAL_INTERRUPT_SOFT);
}

int main(void)
{    
	Test_Mngr();
	return 0;
}

//
// Function: Test_Mngr
//
// Decription: The entry of the Dianostics Suites.
//
void Test_Mngr(void)
{
	Enable_CPSR_Int();
	if((get_CP15() & DCACHE_BIT) == 0)	// if dcache OFF
		dcache_on();

	Diag_Struct_Init();

	Ethernet_init();
	Ethernet_rx_init();

	Get_Eeprom_Mac_Addr();

	Set_LED("DIAG");
	Print_Banner();
	while (1) getCommands();
}

//
// Function: Diag_Struct_Init
//
// Description: Initializes the struct with values that will be used
//				by other functions
//
void Diag_Struct_Init(void)
{
	//This function initializes the main structure of diagnostics.
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	//Setup the Diag common structure with default values.
	acL->tFlag[TFLAG_SRAM]		= 1;	
	acL->tFlag[TFLAG_SDRAM]		= 1;	/* do the T ALL SDRAM test */
	acL->tFlag[TFLAG_UENG]		= 1;	/* do the T ALL UENG test */
	acL->tFlag[TFLAG_TIMERS]	= 1;	/* do the T ALL TIMERS test */
	acL->tFlag[TFLAG_ENET] 		= 1;	/* do the T ALL ENET test */
	acL->tFlag[TFLAG_PCI] 		= 1;	/* do the T ALL PCI test */
	acL->tFlag[TFLAG_XSCALE]	= 1;	/* do the T ALL XSCALE test */
	acL->tFlag[TFLAG_I2C]		= 0;	
	acL->tFlag[TFLAG_GPIO]		= 0;	
	acL->tFlag[TFLAG_LED]		= 1;	/* do the T ALL LED test */
	acL->tFlag[TFLAG_UART]		= 0;	/* do the T ALL UART test */
	acL->tFlag[TFLAG_SF]		= 0;	/* do the T ALL SF test */
	acL->tFlag[TFLAG_LOOPBACK]	= 0;	/* do the T ALL LOOPBACK test */
	acL->tFlag[TFLAG_TMON]		= 0;	
	acL->tFlag[TFLAG_FLASH]		= 0;	
	acL->tFlag[TFLAG_LINERATE]	= 0;
	acL->display_verbose		= MINIMAL;
	acL->version 				= 0x7;	/* eg. 0x0405 = ver 4.5; 0x0310 = ver 3.16 */
	acL->targetName 			= (char*) "IXDP2400";
	acL->targetCpu 				= Get_CPU();
	acL->SDRAMSize 				= Get_SDRAM_Size();
	acL->SRAMSize 				= Get_SRAM_Size();
	acL->CPURevision 			= Get_CPU_Rev();/* set revision before speed! */
	acL->CPUSpeed 				= Get_CPU_Speed();
	acL->HostType				= Get_Host_Type();	// Master/Slave
	acL->UStoreSize 			= 4096;
	acL->defAddress 			= 0;	/* no default address yet */
	acL->Verboselevel			= LOG_OFF;
	acL->HaltOnError			= ENABLED;
	acL->Err					= OK;
}

//
// Function: Print_Banner
//
// Description: Displays board info
//
void Print_Banner(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	
	eprintf("\n%s Board Diagnostics ver. %d.%d, %s, %s\n",
			acL->targetName, ((acL->version>>8) & 0xFF), (acL->version & 0xFF), __TIME__, __DATE__);

	eprintf("NPU Speed:%d Mhz\n", acL->CPUSpeed/1000000);

	eprintf("CPLD Rev: %d.%d\n", ( (GET32(CPLD_REV_REG)>>4) & 0xF), ( GET32(CPLD_REV_REG) & 0xF));
	
	eprintf("DRAM:%d Mb,\n", acL->SDRAMSize/1024/1024);
	eprintf("SRAM:%d Mb; Ch0: %d Mb, Ch1: %d Mb,\n",
			acL->SRAMSize/1024/1024,
			sram_size[0]/1024/1024,
			sram_size[1]/1024/1024);
}

//
// Function: Get_CPU
//
// Description: Reads PRODUCT_ID register to obtain CPU Product Type
//
static char* Get_CPU(void)
{
	switch( (GET32(GLOBAL_CONTROL_BASE) >> 8) & 0xFF)
	{
		case 0:	return("IXP2800");
		case 1: return("IXP2800 with Crypto");
		case 2: return("IXP2400");
		default: return("Unknown NPU");
	}
}

//
// Function: Get_CPU_Speed
//
// Description: Reads M_CLK and N_CLK and calculate CPU Speed
//
static int Get_CPU_Speed(void)
{
	// This function will return the speed of NPU in Hz.
	int M_clk = 0;
	int N_clk, temp;
	int denom_array[] = {2, 4, 8, 16, 1, 2, 4, 8};

	if(((GET32(CPLD_REV_REG)>>4) & 0xF) >= 4)	// if board rev is 4 and up
	{
		temp = *(SYS_CLK_M_REG);
		M_clk = (temp & (0xFF)) * 2;

		temp = *(SYS_CLK_N_REG);
		N_clk = denom_array[(temp & 0x7)];

		temp = 6 * ((3125000 * M_clk)/N_clk);	// in Hz
	}
	else
		temp = (600 * 1000000);	// in hz
	return (temp);
}

//
// Function: Get_CPU_Rev
//
// Description: Reads PRODUCT_ID register to obtain the CPU Rev
//
static int Get_CPU_Rev(void)
{
	// This function will return the NPU revision.
	// It will read the PRODUCT_ID register and return the value to caller.

	// PRODUCT_ID's offset is zero from GLOBAL_CONTROL_BASE
	int cpu_rev = 0xFF & GET32(GLOBAL_CONTROL_BASE);

	return (cpu_rev);
}

//
// Function: Get_SRAM_size
//
// Description: grabs the sram size from BIC
//
static int Get_SRAM_Size(void)
{
	return (sram_size[0] + sram_size[1]);
}
//
// Function: Get_SDRAM_size
//
// Description: Reads number of module rows and density of each row on module
//				from the DRAM I2C PROM. Then, calculates the size of DRAM.
//
static int Get_SDRAM_Size(void)
{
	extern UINT32 hal_dram_size;	// calculated in BIC and stored
	return (hal_dram_size);
}

//
// Function: Get_Host_Type
//
// Description: Determines if current NPU is master or slave
//
static int Get_Host_Type(void)
{
	if(GET32(STRAP_OPTIONS) & CFG_PCI_BOOT_HOST)	// if master NPU
		return MASTER;
	else
		return SLAVE;
}
