/* analog.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, 13 November 2002
 * 
 * --------------------------------------------------------------------------
 */

typedef struct initRegTable
{
	unsigned short* offset;
	unsigned short value;
} InitRegTable_t;

typedef struct  // Complete Data for a Chip
{
   unsigned short*   BaseAddress;	   // Base Address of chip
   unsigned long        ChipType;      // Type of Chip
   void*                pChipCfg;      // Pointer to Chip Specific Configuration
   void*                pAlarmCfg;     // Pointer to Chip Specific Alarm Config
   void*                funcPtr;       // Pointer to the chip's api func's
} bb_ChipData_t;

#include "common.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "hal_io.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "led.h"
#include "msf_common.h"
#include "uart.h"
#include "msf_init.h"
#include "uc_load.h"
#include "pci.h"
#include "analog_ucode.h"

#define DRAM_BASE_SRC			0x1000000
#define DRAM_BASE_DEST			0x2000000
#define RING_DATA_64_0			0x0002003A

#define ANALOG_IN_RX_ME	0
#define ANALOG_IN_TX_ME	1
#define ANALOG_EG_RX_ME	0
#define ANALOG_EG_TX_ME	1

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;
extern unsigned int *orig_sp_val;

typedef unsigned long ulong;
typedef unsigned short ushort;

extern const InitRegTable_t BD_IntLoopbackArray[];
extern const InitRegTable_t BD_ExtLoopbackArray[];

extern ulong IxfApiInit(void *baseAddress, ulong chipType, bb_ChipData_t *pChipData);
extern ulong Ixf6048InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);
extern void Save_N_Set_Sp_Vals(void);
extern void Restore_Sp_Vals(void);

extern pci_device slave_dev_info;		// for slave npu info.

void Analog_BD_Master(void);
void Analog_BD_Slave(void);

UINT32 analog_data[] =	{ 0x55555555, 0x55555555, 0x55555555, 0x55555555,
								  0x55555555, 0x55555555, 0x55555555, 0x55555555,
								  0x55555555, 0x55555555, 0x55555555, 0x55555555,
								  0x55555555, 0x55555555, 0x55550000				};


extern void Write_21555(void);
extern void Read_21555(void);
extern void Write_21154(void);
extern void Read_21154(void);
extern void Write_NPU(void);
extern void Read_NPU(void);

extern void Analog_BD_Master(void);
extern void Analog_BD_Slave(void);

static void Analog_Help(void);
void Read_Pci(void);
void Write_Pci(void);
void Analog_Msf(void);

void Analog_Measurement(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	opt = acL->argv[2][0];
	
	switch(opt)
	{
		case 'r': 	Read_Pci();
					break;

		case 'w': 	Write_Pci();
					break;

		case 'm': 	Analog_Msf();
					break;

		case 'h':
		default	:	Analog_Help();
	}
}


static void Analog_Help(void)
{
	eprintf("Diag> t analog <option>,\n");
	eprintf("where <option> are\n");
	eprintf("\tr5 - Read 21555\n");
	eprintf("\tr4 - Read 21554\n");
	eprintf("\trn - Read NPU\n");
	eprintf("\tw5 - Write 21555\n");
	eprintf("\tw4 - Write 21554\n");
	eprintf("\twn - Write NPU\n");
	eprintf("\tm  - MSF\n");
	eprintf("\th  - Help menu\n");
}


void Read_Pci(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if (acL->argv[2][1] == '5')
	{
		Set_LED("ANp5");
		eprintf("Press any key to stop...\n");
		Read_21555();
	}
	else if (acL->argv[2][1] == '4')
	{
		Set_LED("ANp4");
		eprintf("Press any key to stop...\n");
		Read_21154();
	}
	else if (acL->argv[2][1] == 'n')
	{
		Set_LED("ANpn");
		eprintf("Press any key to stop...\n");
		Read_NPU();
	}
	else
	{
		Analog_Help();
	}
}

void Write_Pci(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if (acL->argv[2][1] == '5')
	{
		Set_LED("ANp5");
		eprintf("Press any key to stop...\n");
		Read_21555();
	}
	else if (acL->argv[2][1] == '4')
	{
		Set_LED("ANp4");
		eprintf("Press any key to stop...\n");
		Read_21154();
	}
	else if (acL->argv[2][1] == 'n')
	{
		Set_LED("ANpn");
		eprintf("Press any key to stop...\n");
		Read_NPU();
	}
	else
	{
		Analog_Help();
	}
}

void Analog_Msf(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
		Analog_BD_Master();
	}
	else							// if slave NPU
	{
		Analog_BD_Slave();
	}
}

void Analog_BD_Master(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, count=0, internal_loop_count;
	UINT32 scratch_ring_tx, scratch_ring_rx;
	UINT32 dcache_status, forever = 1;
	UINT32 *sys_lb_data;
	UINT32 data_to_send, scratch_read_value=0, dram_addr, orig_data_size;
	bb_ChipData_t pc;
	volatile int mailbox_value, temp;

	scratch_ring_rx = SCR_RING_INSTR_ADDR + (1 << 2);
	scratch_ring_tx = SCR_RING_INSTR_ADDR + (0 << 2);

	Set_LED("ANmd");

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((unsigned short*) 0xC6000000, (ulong)3, &pc);

	eprintf("Setting to internal loopback mode...\n");
	Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);

	eprintf("Media card initialisation completed.\n");

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	for (ctr = 0; ctr < sizeof(analog_data); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), analog_data[(ctr / 4)]);
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	PUT32(slave_mailbox0_addr, 0);

#ifdef __ARMEB__
	PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
	PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif

	ctr = 25;		// For 25 second wait

	// Wait for a signal that slave NPU has completed loading of ucode and init
	do
	{
		if (KEYPRESS)
		{
			eprintf("\nUser interrupt...Ending Analog Measurment program for MSF\n");
			break;
		}

		eprintf("Please type \"t analog m\" on the slave console...%02d\r", ctr);

		if (ctr == 0)	// After 25 seconds
		{
			eprintf("\nTimed out...Slave NPU was not able to load microcode and init.\n");
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox0_addr);

		if (temp == 0)
		{
			hal_delay_us(1000000);
			ctr--;
		}
		else
		{
			eprintf("\n");
		}
	}
	while (temp == 0);
		
	if ((flag == OK) && !KEYPRESS)	// If slave completed microcode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(ANALOG_EG_TX_ME, sizeof(analog_bd_eg_tx), (UINT32 *)analog_bd_eg_tx);
		DownLoadUcode(ANALOG_EG_RX_ME, sizeof(analog_bd_eg_rx), (UINT32 *)analog_bd_eg_rx);

		hal_delay_us(1000000);

		do
		{
			count++;
			eprintf("Loop: %d\r", count);

			internal_loop_count = 0;

			do
			{
				data_to_send = RING_DATA_64_0;
				PUT32(scratch_ring_tx, data_to_send);

				ctr = 0;
				do			// Do while waiting for a signal that a packet has been received
				{
					if (ctr == 10000)	// After 5 seconds
					{
						eprintf("Timed out...The mailbox0 register did not indicate that a packet was received\n");
						mailbox_value = NOT_RECEIVED;
						flag = ERROR;
						break;
					}

					// Poll on MAILBOX0 to check if the test has been completed
					scratch_read_value = GET32(scratch_ring_rx);
	
					hal_delay_us(500);

					ctr++;
				}
				while (scratch_read_value == 0);

				if (flag == ERROR)
				{
					break;
				}

				pkt_size = scratch_read_value & 0xFFFF;
				dram_addr = DRAM_BASE_DEST + (scratch_read_value >> 8);

				if (pkt_size != (data_to_send & 0xFFFF))
				{
					mailbox_value = INCORRECT_SIZE;
					flag = ERROR;		// Set flag as error
					break;
				}

				orig_data_size = sizeof(analog_data);
				sys_lb_data = (UINT32 *)analog_data;

				for (ctr = 0; ctr < orig_data_size; ctr+=4)
				{
					// Read the DRAM for the longword that was received
					temp = GET32(dram_addr + ctr);

					// Compare with original value
					if (temp != sys_lb_data[(ctr / 4)])
					{
						mailbox_value = ERROR_PATTERN;
						eprintf("Longword:%d Expected:0x%X  Received:0x%X\n", (ctr / 4), sys_lb_data[(ctr / 4)], temp);
						flag = ERROR;		// Set flag as error
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}

				internal_loop_count++;
			}
			while (internal_loop_count < 12);

			if (flag == ERROR)
			{
				break;
			}

			if (forever && KEYPRESS)
			{
				PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF), 0);
				break;
			}
		}
		while (forever);

		eprintf("\n");
	}

	Restore_Sp_Vals();

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (flag == ERROR)
	{
		if (mailbox_value == INIT_ERROR)
		{
			eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("The was no sign that the expected packet was received\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted.\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted.\n");
		}

		eprintf("Analog measurement...FAIL!\n");
	}		

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 0);

	Clear_All_ME();
}


void Analog_BD_Slave(void)
{
	UINT32 status, scratch_ring_addr, scratch_val, ctr=0, dcache_status;

	scratch_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);

	Set_LED("ANmd");

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	DownLoadUcode(ANALOG_IN_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	PUT32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF, 0xDAD0DAD1);

	// Load ME with microcode
	DownLoadUcode(ANALOG_IN_RX_ME, sizeof(analog_bd_in_rx), (UINT32 *)analog_bd_in_rx);
	DownLoadUcode(ANALOG_IN_TX_ME, sizeof(analog_bd_in_tx), (UINT32 *)analog_bd_in_tx);

	if (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF) == 1)
	{
		do
		{
			if ((scratch_val = GET32(scratch_ring_addr)))
			{
				if (scratch_val == RECEIVE_ERROR)
				{
					eprintf("The packet received had an error in it.\n");
					break;
				}
				eprintf("Loop: %d\r", scratch_val);
				ctr = 0;
			}
			hal_delay_us(5);
			ctr++;
		}
		while (ctr != 500000);
	}
	else
	{
		do
		{
			hal_delay_us(5);
		}
		while (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF));
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}
