 /* dual_media.c
 * 
 * This file contains the functions that are required to run the media loopback
 * test. One function is to load the microcode and run the test on the master
 * NPU and chack the status of the test. The second function is called by the
 * PCI doorbell interrupt handler and will load the microcode for the slave NPU
 *
 *-----------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-----------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, April, 2002
 *  revisions: cmah1, March 14, 2003
 * 
 * 
 * ----------------------------------------------------------------------------
 */

#include "common.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "uart.h"
#include "syslog.h"
#include "diag_utils.h"
#include "msf_init.h"
#include "memory.h"
#include "init_bdw.h"
#include "dual_media.h"

#include "mac_init.h"
#include "error_code.h"
#include "reg_api.h"


// Function Declarations
//=======================
static void Dual_Media_BD(void);
static void Dual_Media_BW(void);
static void Dual_Media_PL(void);
static void Dual_Media_BW_PL(void);
static void Media_BD_On_SF_IF(void);
static void Media_BW_On_SF_IF(void);
static void Media_PL_On_SF_IF(void);
static void Init_Media_Card_BDW_M(UINT8 card_type, UINT8 mode);
static void Init_Media_Card_BDW_S(UINT8 card_type, UINT8 mode);
static int Init_Media_Card_PL_M(void);
static int Init_Media_Card_PL_S(void);
void Dual_Media_Test_Help(void);
void Media_On_SF_IF_Test_Help(void);

extern void Set_Fibre_LB_1port(UINT32 port_num);
extern void Loopback_Help(void);
extern void ShowRegisterVal();
extern void Save_N_Set_Sp_Vals(void);
extern void Restore_Sp_Vals(void);

extern UINT32 loop, ports, speed, media_seat;  
extern char test_opt, channel, dplx, loopback_mode;
extern void spConfig(void);
extern void spRestore(void);
extern UINT32 validate_params(void);
extern unsigned int *orig_sp_val;

// External Arrays
//=================
extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;

extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern const InitRegTable_t BD_IntLoopbackArray[];
extern const InitRegTable_t BD_ExtLoopbackArray[];
extern const InitRegTable_t BW_IntLoopbackArray[];
extern const InitRegTable_t BW_ExtLoopbackArray[];

#define MEDIATYPE_ADDR_M		0x2000
#define MEDIATYPE_ADDR_S		0x2004
#define CHANNEL_MODE_OFFSET		0x2010
#define SLAVE_TX_START			0xBEEB

extern pci_device slave_dev_info;		// for slave npu info.

// Packet/Cell Array
//===================

UINT32 dual_lb_data[4][16] = {{ 0x0A010102, 0x01030104, 0x01050106, 0x01070108,
									  0x0109010A, 0x010B010C, 0x010D010E, 0x010F0110,
									  0x01110112, 0x01130114, 0x01150116, 0x01170118,
									  0x0119011A, 0x011B011C, 0x011D0000				},
									{ 0x1B011102, 0x11031104, 0x11051106, 0x11071108,
									  0x1109110A, 0x110B110C, 0x110D110E, 0x110F1110,
									  0x11111112, 0x11131114, 0x11151116, 0x11171118,
									  0x1119111A, 0x111B111C, 0x111D0000 				},
									{ 0x2C012102, 0x21032104, 0x21052106, 0x21072108,
									  0x2109210A, 0x210B210C, 0x210D210E, 0x210F2110,
									  0x21112112, 0x21132114, 0x21152116, 0x21172118,
									  0x2119211A, 0x211B211C, 0x211D0000				},
									{ 0x3D013102, 0x31033104, 0x31053106, 0x31073108,
									  0x3109310A, 0x310B310C, 0x310D310E, 0x310F3110,
									  0x31113112, 0x31133114, 0x31153116, 0x31173118,
									  0x3119311A, 0x311B311C, 0x311D0000				}};

UINT32 dual_lb_atm[4][13] = {	{0xBABE0000, 0x01020304, 0x05060708, 0x090A0B0C,
								     0x0D0E0F10, 0x11121314, 0x15161718, 0x191A1B1C,
									 0x1D1E1F20, 0x21222324, 0x25262728, 0x292A2B2C,
									 0x2D2E2F30},
									{0xBABE1111, 0x31323334, 0x35363738, 0x393A3B3C,
									 0x3D3E3F40, 0x41424344, 0x45464748, 0x494A4B4C,
									 0x4D4E4F50, 0x51525354, 0x55565758, 0x595A5B5C,
									 0x5D5E5F60},
									{0xBABE2222, 0x61626364, 0x65666768, 0x696A6B6C,
									 0x6D6E6F70, 0x71727374, 0x75767778, 0x797A7B7C,
									 0x7D7E7F80, 0x81828384, 0x85868788, 0x898A8B8C,
									 0x8D8E8F90},
									{0xBABE3333, 0x91929394, 0x95969798, 0x999A9B9C,
									 0x9D9E9FA0, 0xA1A2A3A4, 0xA5A6A7A8, 0xA9AAABAC,
									 0xADAEAFB0, 0xB1B2B3B4, 0xB5B6B7B8, 0xB9BABBBC,
									 0xBDBEBFC0}};

// ******************************************************************************
// Function: void Dual_Media_Test(void)
//
// This function is called by the Test Manager when the user invokes the
// media loopback test
// ******************************************************************************
void Dual_Media_Test(void)
{
	char media_type_m, media_type_s;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 media_type_addr_m, media_type_addr_s;

	test_opt = '?';

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		media_type_m = acL->argv[2][0];
		media_type_s = acL->argv[2][1];

		if ((media_type_m != 'd') && (media_type_m != 'w') && (media_type_m != 'p') && (media_type_s != 'd') && (media_type_s != 'w') && (media_type_s != 'p'))
		{
			eprintf("Please enter valid media card types for both media card seats.\n");
			Dual_Media_Test_Help();
			return;
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		media_type_addr_m = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MEDIATYPE_ADDR_M;
		media_type_addr_s = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MEDIATYPE_ADDR_S;

		PUT32((SCRATCHPAD_BASE + MEDIATYPE_ADDR_M), (UINT32)media_type_m);
		PUT32((SCRATCHPAD_BASE + MEDIATYPE_ADDR_S), (UINT32)media_type_s);

#ifdef __ARMEB__
		PUT32(media_type_addr_m, SWAP32((UINT32)media_type_m));
		PUT32(media_type_addr_s, SWAP32((UINT32)media_type_s));
#else
		PUT32(media_type_addr_m, (UINT32)media_type_m);
		PUT32(media_type_addr_s, (UINT32)media_type_s);
#endif
	}
	else
	{
		media_type_m = (UINT8)GET32(SCRATCHPAD_BASE + MEDIATYPE_ADDR_M);
		media_type_s = (UINT8)GET32(SCRATCHPAD_BASE + MEDIATYPE_ADDR_S);
	}

	switch(media_type_m)
	{
		case 'd':	switch (media_type_s)
					{
						case 'd':	Dual_Media_BD();
									break;
//						case 'w':	Dual_Media_BD_BW();
//									break;
//						case 'p':	Dual_Media_BD_PL();
//									break;
					}
					break;
		
		case 'w':	switch (media_type_s)
					{
//						case 'd':	Dual_Media_BD_BW();
//									break;
						case 'w':	Dual_Media_BW();
									break;
						case 'p':	Dual_Media_BW_PL();
									break;
					}
					break;

		case 'p':	switch (media_type_s)
					{
//						case 'd':	Dual_Media_BD_PL();
//									break;
						case 'w':	Dual_Media_BW_PL();
									break;
						case 'p':	Dual_Media_PL();
									break;
					}
					break;

		default:	eprintf("You must specify the media card type (d/w/p).\n");
					Dual_Media_Test_Help();
	}
}


void Dual_Media_Test_Help(void)
{
	eprintf("Diag> t dual [media type] [loops] [media loopback mode] [channel *] [speed *]\n");
	eprintf("where <option> are\n");
	eprintf("\n    [media type] is the type of media card that is on the system\n");
	eprintf("\tdd - Battery Drew (Single OC-48)\n");
	eprintf("\tww - Battery Wallace (Quad OC-12)\n");
	eprintf("\tpp - Point Lone (Quad GigE)\n");
	eprintf("\n    [loops] is number of loops for system loopback test\n");
	eprintf("\ta decimal value more than 0\n");
	eprintf("\n    [media loopback mode] are\n");
	eprintf("\ti - internal loopback (default)\n");
	eprintf("\te - external loopback\n");
	eprintf("\n    [channel *] are\n");
	eprintf("\tc - copper mode\n");
	eprintf("\tf - fibre mode\n");
	eprintf("\n    [speed *] are\n");
	eprintf("\t10   - 10Mbps (for copper mode only)\n");
	eprintf("\t100  - 100Mbps (for copper mode only)\n");
	eprintf("\t1000 - 1000Mbps (for copper and fibre modes)\n");
	eprintf("\n* For Point Lone only\n");
}


// ******************************************************************************
// Function: void Media_On_SF_IF_Test(void)
//
// This function is called by the Test Manager when the user invokes the
// media loopback test for media cards on the switch fabric interface
// ******************************************************************************
void Media_On_SF_IF_Test(void)
{
	char media_type;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 media_type_addr;

	test_opt = '?';

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		media_type = acL->argv[2][0];

		if ((media_type != 'd') && (media_type != 'w') && (media_type != 'p'))
		{
			eprintf("Please enter valid media card types for both media card seats.\n");
			Media_On_SF_IF_Test_Help();
			return;
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		media_type_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MEDIATYPE_ADDR_S;

		PUT32((SCRATCHPAD_BASE + MEDIATYPE_ADDR_S), (UINT32)media_type);

#ifdef __ARMEB__
		PUT32(media_type_addr, SWAP32((UINT32)media_type));
#else
		PUT32(media_type_addr, (UINT32)media_type);
#endif
	}
	else
	{
		media_type = (UINT8)GET32(SCRATCHPAD_BASE + MEDIATYPE_ADDR_S);
	}

	switch(media_type)
	{
		case 'd':	Media_BD_On_SF_IF();
					break;
		
		case 'w':	Media_BW_On_SF_IF();
					break;

		case 'p':	Media_PL_On_SF_IF();
					break;

		default:	eprintf("You must specify the media card type (d/w/p).\n");
					Media_On_SF_IF_Test_Help();
	}
}


void Media_On_SF_IF_Test_Help(void)
{
	eprintf("Diag> t mediasf [media type] [loops] [media loopback mode] [channel *] [speed *]\n");
	eprintf("where <option> are\n");
	eprintf("\n    [media type] is the type of media card that is on the system\n");
	eprintf("\td - Battery Drew (Single OC-48)\n");
	eprintf("\tw - Battery Wallace (Quad OC-12)\n");
	eprintf("\tp - Point Lone (Quad GigE)\n");
	eprintf("\n    [loops] is number of loops for system loopback test\n");
	eprintf("\ta decimal value more than 0\n");
	eprintf("\n    [media loopback mode] are\n");
	eprintf("\ti - internal loopback (default)\n");
	eprintf("\te - external loopback\n");
	eprintf("\n    [channel *] are\n");
	eprintf("\tc - copper mode\n");
	eprintf("\tf - fibre mode\n");
	eprintf("\n    [speed *] are\n");
	eprintf("\t10   - 10Mbps (for copper mode only)\n");
	eprintf("\t100  - 100Mbps (for copper mode only)\n");
	eprintf("\t1000 - 1000Mbps (for copper and fibre modes)\n");
	eprintf("\n* For Point Lone only\n");
}


// *******************************************************************************
// Function: void Init_Media_Card_BDW_M(UINT8 card_type, UINT8 mode)
//
// This function is called by the Dual_Media_BD/Dual_Media_BW to initialize
// the media card
// *******************************************************************************
static void Init_Media_Card_BDW_M(UINT8 card_type, UINT8 mode)
{
	bb_ChipData_t pc;

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS_M, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if (card_type == MEDIA_BD)
	{
		if (mode == EXT_LPBK)
		{
			eprintf("Setting Battery Drew to fiber loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
		}
		else
		{
			eprintf("Setting Battery Drew to internal loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
		}
	}
	else
	{
		if (mode == EXT_LPBK)
		{
			eprintf("Setting Battery Wallace to fiber loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
		else
		{
			eprintf("Setting Battery Wallace to internal loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
		}
	}		

	Restore_Sp_Vals();

	return;
}

// *******************************************************************************
// Function: void Init_Media_Card_BDW_S(UINT8 card_type, UINT8 mode)
//
// This function is called by the Dual_Media_BD/Dual_Media_BW to initialize
// the media card
// *******************************************************************************
static void Init_Media_Card_BDW_S(UINT8 card_type, UINT8 mode)
{
	bb_ChipData_t pc;

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS_S, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if (card_type == MEDIA_BD)
	{
		if (mode == EXT_LPBK)
		{
			eprintf("Setting Battery Drew to fiber loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
		}
		else
		{
			eprintf("Setting Battery Drew to internal loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
		}
	}
	else
	{
		if (mode == EXT_LPBK)
		{
			eprintf("Setting Battery Wallace to fiber loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
		else
		{
			eprintf("Setting Battery Wallace to internal loopback mode...\n");
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
		}
	}		

	Restore_Sp_Vals();

	return;
}


// *******************************************************************************
// Function: void Init_Media_Card_PL_M(void)
//
// This function is called by the Dual_Media_PL to initialize the Point Lone card
// which is seated on the Media connector
// *******************************************************************************
static int Init_Media_Card_PL_M(void)
{
	UINT32 status;

	media_seat = 0;

	status = init_mac_dev(0);

	if (status != DONE)
	{
		eprintf("Aborting Test\n");
		return 1;
	}

    spConfig();

	if (loopback_mode == 'i')
	{
	    /* configure internal loopback */
	    reg_write(0x5b2,0xF00);
	    reg_write(0x5b3,0x0);	
	}
	else
	{
		reg_write(0x5b2,0x0);
	}

//	ShowRegisterVal();

    spRestore();
	return 0;
}

// *******************************************************************************
// Function: void Init_Media_Card_PL_S(void)
//
// This function is called by the Dual_Media_PL to initialize the Point Lone card
// which is seated on the Switch Fabric connector
// *******************************************************************************
static int Init_Media_Card_PL_S(void)
{
	UINT32 status;

	media_seat = 0xBABECAFE;

	status = init_mac_dev(0);

	if (status != DONE)
	{
		eprintf("Aborting Test\n");
		PUT32((SCRATCHPAD_BASE + 0x3FFC), 0);
		media_seat = 0;
		return 1;
	}

    spConfig();

	if (loopback_mode == 'i')
	{
	    /* configure internal loopback */
	    reg_write(0x5b2,0xF00);
	    reg_write(0x5b3,0x0);	
	}
	else
	{
		reg_write(0x5b2,0x0);
	}

    spRestore();
	
//	ShowRegisterVal();

	media_seat = 0;
	return 0;
}


// *******************************************************************************
// Function: void Dual_Media_BD(void)
//
// This function is to test two BD media cards
// *******************************************************************************
static void Dual_Media_BD(void)
{
	UINT32 ctr, flag, status, port_ctr;
	UINT32 data_rcv = NOT_RCVD;
	UINT32 data_rcv_bytes = 0;
	UINT32 port_0_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
//	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT32 temp0 = 0, temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr;
	UINT32 loop_count, loop_counter;
	UINT32 ring_value;
	UINT32 end_test_flag;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_counter = 0;
	
	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (acL->argc > 3)
		{
			loop_count = acL->decArg[3];
		}
		else
		{
			loop_count = 1;
		}

		if (acL->argc > 4)
		{
			loopback_mode = acL->argv[4][0];
		}
		else
		{
			loopback_mode = 'i';
		}

		eprintf("Initialising media card. Please wait...\n");
		port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		if (loopback_mode == 'e')
		{
			Init_Media_Card_BDW_M(MEDIA_BD, EXT_LPBK);
			Init_Media_Card_BDW_S(MEDIA_BD, EXT_LPBK);
		}
		else
		{
			Init_Media_Card_BDW_M(MEDIA_BD, INT_LPBK);
			Init_Media_Card_BDW_S(MEDIA_BD, INT_LPBK);
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Write packet data to DRAM for microcode to use
		for(port_ctr = 0; port_ctr < 1; port_ctr++)
		{
			for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
			}
		}
		
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(DUAL_MEDIA_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		// Load Master's NPU with ucode and init
		Clear_All_ME();
		DownLoadUcode(0, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(dual_bd_eg_rx), (UINT32 *)dual_bd_eg_rx);
		DownLoadUcode(1, sizeof(dual_bd_eg_tx), (UINT32 *)dual_bd_eg_tx);
		hal_delay_us(1000);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 1; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + EGRESS_FOR_RX_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
				}
			}
		
			
//			eprintf("Start TX and waiting to RX packet...\n");
			PUT32(port_0_put_ring_addr, START_TX);

			data_rcv = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 1; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv= RCVD;
						data_rcv_bytes = ring_value & MASK_16BIT;
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			for (port_ctr = 0; port_ctr < 1; port_ctr++)
			{
				if (data_rcv_bytes != START_TX)
				{
					eprintf("Packet size is incorrect...\n");
					flag = ERROR;
					break;
				}

				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					temp1 = GET32((DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET) + EGRESS_FOR_RX_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								ctr, temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", loop_counter);
		}
		while (loop_counter < loop_count);

		if (flag == ERROR)
		{
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
			PUT32(slave_mailbox2_addr, 1);
		}

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Dual Battery Drew test...PASSED\n");
			Set_LED("DddP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Dual Battery Drew test...FAIL!\n");
			Set_LED("DddF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);

		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Clear DRAM for microcode to use for receive
		for(port_ctr = 0; port_ctr < 1; port_ctr++)
		{
			for (ctr = 0; ctr < (sizeof(dual_lb_data[port_ctr])*4); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
			}
		}

		// Load ME with microcode
		Clear_All_ME();
		DownLoadUcode(0, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(dual_bd_in_rx), (UINT32 *)dual_bd_in_rx);
		DownLoadUcode(1, sizeof(dual_bd_in_tx), (UINT32 *)dual_bd_in_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF), 0);
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			if (ctr == 750000)
			{
				eprintf("No packets received...Timed out...\n");
				break;
			}

			ring_value = GET32(get_ring_addr);
			
			end_test_flag = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
			if (end_test_flag > 0)
			{
				break;
			}

			if (ring_value > 0)
			{
				eprintf("Packets received: %d\r", ring_value);
				ctr = 0;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}
		}
		while (ring_value != loop_count);
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}


// *******************************************************************************
// Function: void Dual_Media_BW(void)
//
// This function is to test two BW media cards
// *******************************************************************************
static void Dual_Media_BW(void)
{
	UINT32 ctr, flag, status, port_ctr;
	UINT32 data_rcv[4] = {NOT_RCVD, NOT_RCVD, NOT_RCVD, NOT_RCVD};
	UINT32 data_rcv_bytes[4] = {0, 0, 0, 0};
	UINT32 port_0_put_ring_addr, port_1_put_ring_addr, port_2_put_ring_addr, port_3_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
//	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT32 temp0 = 0, temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr;
	UINT32 loop_count, loop_counter;
	UINT32 ring_value;
	UINT32 end_test_flag;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_counter = 0;
	
	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (acL->argc > 3)
		{
			loop_count = acL->decArg[3];
		}
		else
		{
			loop_count = 1;
		}

		if (acL->argc > 4)
		{
			loopback_mode = acL->argv[4][0];
		}
		else
		{
			loopback_mode = 'i';
		}

		eprintf("Initialising media card. Please wait...\n");
		port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
		port_1_put_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
		port_2_put_ring_addr = SCR_RING_INSTR_ADDR + (2 << 2);
		port_3_put_ring_addr = SCR_RING_INSTR_ADDR + (3 << 2);
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		if (loopback_mode == 'e')
		{
			Init_Media_Card_BDW_M(MEDIA_BW, EXT_LPBK);
			Init_Media_Card_BDW_S(MEDIA_BW, EXT_LPBK);
		}
		else
		{
			Init_Media_Card_BDW_M(MEDIA_BW, INT_LPBK);
			Init_Media_Card_BDW_S(MEDIA_BW, INT_LPBK);
		}


		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Write packet data to DRAM for microcode to use
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
			}
		}
		
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(DUAL_MEDIA_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		// Load Master's NPU with ucode and init
		Clear_All_ME();
		DownLoadUcode(0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(dual_bw_eg_rx), (UINT32 *)dual_bw_eg_rx);
		DownLoadUcode(1, sizeof(dual_bw_eg_tx), (UINT32 *)dual_bw_eg_tx);
		hal_delay_us(1000);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + EGRESS_FOR_RX_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
				}
			}
		
			
//			eprintf("Start TX and waiting to RX packet...\n");
			PUT32(port_0_put_ring_addr, START_TX);
			PUT32(port_1_put_ring_addr, START_TX);
			PUT32(port_2_put_ring_addr, START_TX);
			PUT32(port_3_put_ring_addr, START_TX);

			data_rcv[0] = NOT_RCVD;
			data_rcv[1] = NOT_RCVD;
			data_rcv[2] = NOT_RCVD;
			data_rcv[3] = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv[((ring_value >> 16) & MASK_16BIT)] = RCVD;
						data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)] = ring_value & MASK_16BIT;
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (data_rcv_bytes[port_ctr] != START_TX)
				{
					eprintf("Packet size is incorrect...\n");
					flag = ERROR;
					break;
				}

				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					temp1 = GET32((DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET) + EGRESS_FOR_RX_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								ctr, temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", (loop_counter * 4));
		}
		while (loop_counter < loop_count);

		if (flag == ERROR)
		{
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
			PUT32(slave_mailbox2_addr, 1);
		}

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Dual Battery Wallace test...PASSED\n");
			Set_LED("DppP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Dual Battery Wallace test...FAIL!\n");
			Set_LED("DppF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);

		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Clear DRAM for microcode to use for receive
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < (sizeof(dual_lb_data[port_ctr])*4); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
			}
		}

		Clear_All_ME();
		DownLoadUcode(0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		// Load ME with microcode
		DownLoadUcode(0, sizeof(dual_bw_in_rx), (UINT32 *)dual_bw_in_rx);
		DownLoadUcode(1, sizeof(dual_bw_in_tx), (UINT32 *)dual_bw_in_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF), 0);
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			if (ctr == 750000)
			{
				eprintf("No packets received...Timed out...\n");
				break;
			}

			ring_value = GET32(get_ring_addr);

			end_test_flag = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
			if (end_test_flag > 0)
			{
				break;
			}

			if (ring_value > 0)
			{
				eprintf("Packets received: %d\r", ring_value);
				ctr = 0;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}
		}
		while (ring_value != (loop_count * 4));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}


// *******************************************************************************
// Function: void Dual_Media_PL(void)
//
// This function is to test two PL media cards
// *******************************************************************************
static void Dual_Media_PL(void)
{
	UINT32 ctr, flag, status, port_ctr;
	UINT32 data_rcv[4] = {NOT_RCVD, NOT_RCVD, NOT_RCVD, NOT_RCVD};
	UINT32 data_rcv_bytes[4] = {0, 0, 0, 0};
	UINT32 port_0_put_ring_addr, port_1_put_ring_addr, port_2_put_ring_addr, port_3_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
	UINT32 temp0 = 0, temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr;
	UINT32 loop_count, loop_counter;
	UINT32 ring_value;
	UINT32 end_test_flag;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];
	channel = acL->argv[5][0];
	speed = acL->decArg[6];
	dplx = 'f';
	ports = 4;
	
	loop_counter = 0;
	
	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return;
		}
	
		eprintf("Initialising media card. Please wait...\n");
		port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
		port_1_put_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
		port_2_put_ring_addr = SCR_RING_INSTR_ADDR + (2 << 2);
		port_3_put_ring_addr = SCR_RING_INSTR_ADDR + (3 << 2);
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		if(Init_Media_Card_PL_M())
		{
			eprintf("media card init. failed\n");
			return;
		}
		
		if(Init_Media_Card_PL_S())
		{
			eprintf("media card init. failed\n");
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Write packet data to DRAM for microcode to use
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
			}
		}
		
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(DUAL_MEDIA_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		// Load Master's NPU with ucode and init
		DownLoadUcode(0, sizeof(dual_pl_eg_rx), (UINT32 *)dual_pl_eg_rx);
		DownLoadUcode(1, sizeof(dual_pl_eg_tx), (UINT32 *)dual_pl_eg_tx);
		hal_delay_us(1000);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + EGRESS_FOR_RX_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
				}
			}
		
			
//			eprintf("Start TX and waiting to RX packet...\n");
			PUT32(port_0_put_ring_addr, START_TX);
			PUT32(port_1_put_ring_addr, START_TX);
			PUT32(port_2_put_ring_addr, START_TX);
			PUT32(port_3_put_ring_addr, START_TX);

			data_rcv[0] = NOT_RCVD;
			data_rcv[1] = NOT_RCVD;
			data_rcv[2] = NOT_RCVD;
			data_rcv[3] = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv[((ring_value >> 16) & MASK_16BIT)] = RCVD;
						data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)] = ring_value & MASK_16BIT;
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (data_rcv_bytes[port_ctr] != START_TX)
				{
					eprintf("Packet size is incorrect...\n");
					flag = ERROR;
					break;
				}

				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					temp1 = GET32((DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET) + EGRESS_FOR_RX_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								ctr, temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", (loop_counter * 4));
		}
		while (loop_counter < loop_count);

		eprintf("\n");

		if (flag == ERROR)
		{
			*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
			PUT32(slave_mailbox2_addr, 1);
		}

		if (flag == OK)
		{
			eprintf("Dual Point Lone test...PASSED\n");
			Set_LED("DppP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Dual Point Lone test...FAIL!\n");
			Set_LED("DppF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);

		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Clear DRAM for microcode to use for receive
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < (sizeof(dual_lb_data[port_ctr])*4); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
			}
		}

		// Load ME with microcode
		DownLoadUcode(0, sizeof(dual_pl_in_rx), (UINT32 *)dual_pl_in_rx);
		DownLoadUcode(1, sizeof(dual_pl_in_tx), (UINT32 *)dual_pl_in_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF), 0);
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			if (ctr == 750000)
			{
				eprintf("No packets received...Timed out...\n");
				break;
			}

			ring_value = GET32(get_ring_addr);

			end_test_flag = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
			if (end_test_flag > 0)
			{
				break;
			}

			if (ring_value > 0)
			{
				eprintf("Packets received: %d\r", ring_value);
				ctr = 0;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}
		}
		while (ring_value != (loop_count * 4));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}

// *******************************************************************************
// Function: void Dual_Media_BW_PL(void)
//
// This function is to test a system with a BW and a PL media card on the same
// system
// *******************************************************************************
static void Dual_Media_BW_PL(void)
{
	UINT32 ctr, flag, status, port_ctr, temp_port_ctr;
	UINT32 data_rcv[4] = {NOT_RCVD, NOT_RCVD, NOT_RCVD, NOT_RCVD};
	UINT32 data_rcv_bytes[4] = {0, 0, 0, 0};
	UINT32 port_0_put_ring_addr, port_1_put_ring_addr, port_2_put_ring_addr, port_3_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
//	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT32 temp0 = 0, temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr;
	UINT32 loop_count, loop_counter;
	UINT32 ring_value;
	UINT8 media_type_m, media_type_s;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	//t dual p  loops i/e f/c 10/100/1000 f/h loops
	//acL->argv
	//acL->decArg

	loopback_mode = acL->argv[4][0];
	channel = acL->argv[5][0];
	speed = acL->decArg[6];
	dplx = 'f';
	ports = 4;
	
	loop_counter = 0;
	
	flag = OK;

	media_type_m = (UINT8)GET32(SCRATCHPAD_BASE + MEDIATYPE_ADDR_M);
	media_type_s = (UINT8)GET32(SCRATCHPAD_BASE + MEDIATYPE_ADDR_S);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return;
		}

		if (acL->argc > 3)
		{
			loop_count = acL->decArg[3];
		}
		else
		{
			loop_count = 1;
		}

		eprintf("Initialising media card. Please wait...\n");
		port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
		port_1_put_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
		port_2_put_ring_addr = SCR_RING_INSTR_ADDR + (2 << 2);
		port_3_put_ring_addr = SCR_RING_INSTR_ADDR + (3 << 2);
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		if ((media_type_m == MEDIA_BW) && (media_type_s == MEDIA_PL))
		{
			Init_Media_Card_BDW_M(MEDIA_BW, INT_LPBK);
			if(Init_Media_Card_PL_S())
			{
				eprintf("media card init. failed\n");
				return;
			}
		}
		else if ((media_type_m == MEDIA_PL) && (media_type_s == MEDIA_BW))
		{
			if(Init_Media_Card_PL_M())
			{
				eprintf("media card init. failed\n");
				return;
			}
			Init_Media_Card_BDW_S(MEDIA_BW, INT_LPBK);
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Write packet data to DRAM for microcode to use
		if (channel == 'f')		// if fibre
		{
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
				}
			}
		}
		else	// if (channel == 'c') if copper
		{
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (port_ctr == 0)
				{
					temp_port_ctr = 1;
				}
				else if (port_ctr == 1)
				{
					temp_port_ctr = 0;
				}
				else if (port_ctr == 2)
				{
					temp_port_ctr = 3;
				}
				else //if (port_ctr == 3)
				{
					temp_port_ctr = 2;
				}

				for (ctr = 0; ctr < sizeof(dual_lb_data[temp_port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (temp_port_ctr*PORT_OFFSET) + ctr), dual_lb_data[temp_port_ctr][(ctr / 4)]);
				}
			}
		}
		
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(DUAL_MEDIA_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		// Load Master's NPU with ucode and init
		if ((media_type_m == MEDIA_BW) && (media_type_s == MEDIA_PL))
		{
			DownLoadUcode(0, sizeof(dual_pl_eg_rx), (UINT32 *)dual_pl_eg_rx);
			DownLoadUcode(1, sizeof(dual_bw_eg_tx), (UINT32 *)dual_bw_eg_tx);
		}
		else if ((media_type_m == MEDIA_PL) && (media_type_s == MEDIA_BW))
		{
			DownLoadUcode(0, sizeof(dual_bw_eg_rx), (UINT32 *)dual_bw_eg_rx);
			DownLoadUcode(1, sizeof(dual_pl_eg_tx), (UINT32 *)dual_pl_eg_tx);
		}
		hal_delay_us(1000);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + EGRESS_FOR_RX_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
				}
			}
		
//			eprintf("Start TX and waiting to RX packet...\n");
			PUT32(port_0_put_ring_addr, START_TX);
			PUT32(port_1_put_ring_addr, START_TX);
			PUT32(port_2_put_ring_addr, START_TX);
			PUT32(port_3_put_ring_addr, START_TX);

			data_rcv[0] = NOT_RCVD;
			data_rcv[1] = NOT_RCVD;
			data_rcv[2] = NOT_RCVD;
			data_rcv[3] = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv[((ring_value >> 16) & MASK_16BIT)] = RCVD;
						data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)] = ring_value & MASK_16BIT;
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (data_rcv_bytes[port_ctr] != START_TX)
				{
					eprintf("Packet size is incorrect...\n");
					flag = ERROR;
					break;
				}

				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					temp1 = GET32((DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET) + EGRESS_FOR_RX_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								ctr, temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", (loop_counter * 4));
		}
		while (loop_counter < loop_count);

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Dual Media test with Battery Wallace and Point Lone...PASSED\n");
			Set_LED("DppP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Dual Media test with Battery Wallace and Point Lone...FAIL!\n");
			Set_LED("DppF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);

		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Clear DRAM for microcode to use for receive
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < (sizeof(dual_lb_data[port_ctr])*4); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), 0x0);
			}
		}

		// Load ME with microcode
		if ((media_type_m == MEDIA_BW) && (media_type_s == MEDIA_PL))
		{
			DownLoadUcode(0, sizeof(dual_bw_in_rx), (UINT32 *)dual_bw_in_rx);
			DownLoadUcode(1, sizeof(dual_pl_in_tx), (UINT32 *)dual_pl_in_tx);
		}
		else if ((media_type_m == MEDIA_PL) && (media_type_s == MEDIA_BW))
		{
			DownLoadUcode(0, sizeof(dual_pl_in_rx), (UINT32 *)dual_pl_in_rx);
			DownLoadUcode(1, sizeof(dual_bw_in_tx), (UINT32 *)dual_bw_in_tx);
		}
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			if (ctr == 750000)
			{
				eprintf("No packets received...Timed out...\n");
				break;
			}

			ring_value = GET32(get_ring_addr);

			if (ring_value > 0)
			{
				eprintf("Packets received: %d\r", ring_value);
				ctr = 0;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}
		}
		while (ring_value != (loop_count * 4));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}


// *******************************************************************************
// Function: void Media_BD_On_SF_IF(void)
//
// This function is to test BD media card on the switch fabric interface
// *******************************************************************************
static void Media_BD_On_SF_IF(void)
{
	UINT32 ctr, flag, status, port_ctr;
	UINT32 data_rcv = NOT_RCVD;
	UINT32 data_rcv_bytes = 0;
	UINT32 port_0_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
//	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT32 temp0 = 0;
	volatile UINT32 temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, signal_from_master;
	UINT32 loop_count, loop_counter, tx_counter;
	UINT32 ring_value;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_counter = 0;
	
	flag = OK;

	port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (acL->argc > 3)
		{
			loop_count = acL->decArg[3];
		}
		else
		{
			loop_count = 1;
		}

		loopback_mode = acL->argv[4][0];

		eprintf("Initialising media card. Please wait...\n");

		if (loopback_mode == 'e')
		{
			Init_Media_Card_BDW_S(MEDIA_BD, EXT_LPBK);
		}
		else
		{
			Init_Media_Card_BDW_S(MEDIA_BD, INT_LPBK);
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Load Master's NPU with ucode and init
		DownLoadUcode(0, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(media_lb_bd_on_sf_iface_rx), (UINT32 *)media_lb_bd_on_sf_iface_rx);
		hal_delay_us(1000);

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(MEDIA_SF_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr * PORT_OFFSET) + ctr), 0);
				}
			}

#ifdef __ARMEB__
			PUT32(slave_mailbox0_addr, SWAP32(SLAVE_TX_START));
#else
			PUT32(slave_mailbox0_addr, SLAVE_TX_START);
#endif					
			
			data_rcv = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 1; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv = RCVD;
						data_rcv_bytes = ring_value & MASK_16BIT;
//						eprintf("data_rcv_bytes[%d]=%d\n", ((ring_value >> 16) & MASK_16BIT), data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)]);
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}


			for (port_ctr = 0; port_ctr < 1; port_ctr++)
			{
				if (data_rcv_bytes != START_TX)
				{
					if (data_rcv_bytes != (START_TX + 4))
					{
						eprintf("Packet size is incorrect...\n");
						flag = ERROR;
						break;
					}
				}


				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					hal_delay_us(100);
					temp1 = GET32(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET)), temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", loop_counter);

			if ((loop_count == 0) && KEYPRESS)
			{
#ifdef __ARMEB__
				PUT32(slave_mailbox0_addr, SWAP32(TEST_COMPLETE));
#else
				PUT32(slave_mailbox0_addr, TEST_COMPLETE);
#endif					
				break;
			}
		}
		while (loop_counter != loop_count);

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Battery Drew on Switch Fabric I/F Loopback test...PASSED\n");
			Set_LED("MSdP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Battery Drew on Switch Fabric I/F Loopback test...FAIL!\n");
			Set_LED("MSdF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);
		loop_counter = 0;

//		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		
		for(port_ctr = 0; port_ctr < 1; port_ctr++)
		{
			for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
			}
		}

		// Load ME with microcode
		DownLoadUcode(0, sizeof(media_lb_bd_on_sf_iface_tx), (UINT32 *)media_lb_bd_on_sf_iface_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			signal_from_master = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);

			if (signal_from_master == TEST_COMPLETE)
			{
				break;
			}
			else if (signal_from_master == SLAVE_TX_START)
			{
				PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);
				PUT32(port_0_put_ring_addr, START_TX);

				ctr = 0;
				do
				{
					if (ctr == 100000)
					{
						eprintf("No indication of any packets transmitted\n");
						flag = ERROR;
						break;
					}

					tx_counter = GET32(get_ring_addr);

					if (tx_counter > 0)
					{
						eprintf("Packets transmitted: %d\r", tx_counter);
						break;
					}

					ctr++;
					hal_delay_us(10);
				}
				while (tx_counter == 0);

				if (flag == ERROR)
				{
					break;
				}

				loop_counter++;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}

			if (ctr == 750000)
			{
				eprintf("No indication from Master to transmit packets\n");
				break;
			}
		}
		while ((loop_counter != loop_count) || (loop_count == 0));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}


// *******************************************************************************
// Function: void Media_BW_On_SF_IF(void)
//
// This function is to test BW media card on the switch fabric interface
// *******************************************************************************
static void Media_BW_On_SF_IF(void)
{
	UINT32 ctr, flag, status, port_ctr;
	UINT32 data_rcv[4] = {NOT_RCVD, NOT_RCVD, NOT_RCVD, NOT_RCVD};
	UINT32 data_rcv_bytes[4] = {0, 0, 0, 0};
	UINT32 port_0_put_ring_addr, port_1_put_ring_addr, port_2_put_ring_addr, port_3_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
//	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT32 temp0 = 0;
	volatile UINT32 temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, signal_from_master;
	UINT32 loop_count, loop_counter, tx_counter;
	UINT32 ring_value;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_counter = 0;
	
	flag = OK;

	port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
	port_1_put_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
	port_2_put_ring_addr = SCR_RING_INSTR_ADDR + (2 << 2);
	port_3_put_ring_addr = SCR_RING_INSTR_ADDR + (3 << 2);
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		if (acL->argc > 3)
		{
			loop_count = acL->decArg[3];
		}
		else
		{
			loop_count = 1;
		}

		loopback_mode = acL->argv[4][0];

		eprintf("Initialising media card. Please wait...\n");

		if (loopback_mode == 'e')
		{
			Init_Media_Card_BDW_S(MEDIA_BW, EXT_LPBK);
		}
		else
		{
			Init_Media_Card_BDW_S(MEDIA_BW, INT_LPBK);
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
#endif

		// Load Master's NPU with ucode and init
		DownLoadUcode(0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(media_lb_bw_on_sf_iface_rx), (UINT32 *)media_lb_bw_on_sf_iface_rx);
		hal_delay_us(1000);

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(MEDIA_SF_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr * PORT_OFFSET) + ctr), 0);
				}
			}

#ifdef __ARMEB__
			PUT32(slave_mailbox0_addr, SWAP32(SLAVE_TX_START));
#else
			PUT32(slave_mailbox0_addr, SLAVE_TX_START);
#endif					
			
			data_rcv[0] = NOT_RCVD;
			data_rcv[1] = NOT_RCVD;
			data_rcv[2] = NOT_RCVD;
			data_rcv[3] = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv[((ring_value >> 16) & MASK_16BIT)] = RCVD;
						data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)] = ring_value & MASK_16BIT;
//						eprintf("data_rcv_bytes[%d]=%d\n", ((ring_value >> 16) & MASK_16BIT), data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)]);
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}


			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (data_rcv_bytes[port_ctr] != START_TX)
				{
					if (data_rcv_bytes[port_ctr] != (START_TX + 4))
					{
						eprintf("Packet size is incorrect...\n");
						flag = ERROR;
						break;
					}
				}


				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					hal_delay_us(100);
					temp1 = GET32(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET)), temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", (loop_counter * 4));

			if ((loop_count == 0) && KEYPRESS)
			{
#ifdef __ARMEB__
				PUT32(slave_mailbox0_addr, SWAP32(TEST_COMPLETE));
#else
				PUT32(slave_mailbox0_addr, TEST_COMPLETE);
#endif					
				break;
			}
		}
		while (loop_counter != loop_count);

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Battery Wallace on Switch Fabric I/F Loopback test...PASSED\n");
			Set_LED("MSwP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Battery Wallace on Switch Fabric I/F Loopback test...FAIL!\n");
			Set_LED("MSwF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);
		loop_counter = 0;

//		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
			}
		}

		// Load ME with microcode
		DownLoadUcode(0, sizeof(media_lb_bw_on_sf_iface_tx), (UINT32 *)media_lb_bw_on_sf_iface_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			signal_from_master = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);

			if (signal_from_master == TEST_COMPLETE)
			{
				break;
			}
			else if (signal_from_master == SLAVE_TX_START)
			{
				PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);
				PUT32(port_0_put_ring_addr, START_TX);
				PUT32(port_1_put_ring_addr, START_TX);
				PUT32(port_2_put_ring_addr, START_TX);
				PUT32(port_3_put_ring_addr, START_TX);

				ctr = 0;
				do
				{
					if (ctr == 100000)
					{
						eprintf("No indication of any packets transmitted\n");
						flag = ERROR;
						break;
					}

					tx_counter = GET32(get_ring_addr);

					if (tx_counter > 0)
					{
						eprintf("Packets transmitted: %d\r", tx_counter);
						break;
					}

					ctr++;
					hal_delay_us(10);
				}
				while (tx_counter == 0);

				if (flag == ERROR)
				{
					break;
				}

				loop_counter++;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}

			if (ctr == 750000)
			{
				eprintf("No indication from Master to transmit packets\n");
				break;
			}
		}
		while ((loop_counter != loop_count) || (loop_count == 0));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}


// *******************************************************************************
// Function: void Media_PL_On_SF_IF(void)
//
// This function is to test PL media card on the switch fabric interface
// *******************************************************************************
static void Media_PL_On_SF_IF(void)
{
	UINT32 ctr, flag, status, port_ctr, temp_port_ctr;
	UINT32 data_rcv[4] = {NOT_RCVD, NOT_RCVD, NOT_RCVD, NOT_RCVD};
	UINT32 data_rcv_bytes[4] = {0, 0, 0, 0};
	UINT32 port_0_put_ring_addr, port_1_put_ring_addr, port_2_put_ring_addr, port_3_put_ring_addr, get_ring_addr;
	UINT32 dcache_status;	
	UINT32 temp0 = 0;
	volatile UINT32 temp1 = 0;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, signal_from_master;
	UINT32 loop_count, loop_counter, tx_counter;
	UINT32 ring_value;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	loop_counter = 0;
	
	flag = OK;

	port_0_put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
	port_1_put_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
	port_2_put_ring_addr = SCR_RING_INSTR_ADDR + (2 << 2);
	port_3_put_ring_addr = SCR_RING_INSTR_ADDR + (3 << 2);
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	if(acL->HostType == MASTER) // ##### MASTER #####
	{
		loop_count = acL->decArg[3];
		loopback_mode = acL->argv[4][0];
		channel = acL->argv[5][0];
		speed = acL->decArg[6];
		dplx = 'f';
		ports = 4;
	
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return;
		}
	
		eprintf("Initialising media card. Please wait...\n");
		if(Init_Media_Card_PL_S())
		{
			eprintf("media card init. failed\n");
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0
		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;

#ifdef __ARMEB__
		PUT32(slave_mailbox0_addr, SWAP32(MASTER_COMPLETE_INIT));
		PUT32(slave_mailbox1_addr, SWAP32(loop_count));
		PUT32(slave_mailbox2_addr, SWAP32((UINT32)channel));
#else
		PUT32(slave_mailbox0_addr, MASTER_COMPLETE_INIT);
		PUT32(slave_mailbox1_addr, loop_count);
		PUT32(slave_mailbox2_addr, (UINT32)channel);
#endif

		// Load Master's NPU with ucode and init
		DownLoadUcode(0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
		hal_delay_us(1000000);

		Clear_All_ME();
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		DownLoadUcode(0, sizeof(media_lb_pl_on_sf_iface_rx), (UINT32 *)media_lb_pl_on_sf_iface_rx);
		hal_delay_us(1000);

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(MEDIA_SF_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		eprintf("Waiting for slave to finish ucode download...\n");
		ctr = 0;
		do
		{
			if (ctr == 10000)
			{
				eprintf("Slave could not completed microcode download!!! ENDING TEST...\n");
				Clear_All_ME();
				return;
			}

#ifdef __ARMEB__
			temp0 = SWAP32(GET32(slave_mailbox0_addr));
#else
			temp0 = GET32(slave_mailbox0_addr);
#endif

			ctr++;
			hal_delay_us(1000);
		}
		while (temp0 != SLAVE_READY_TO_RX_TX);

		do
		{
			// Clear DRAM for microcode to use for receive
			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr * PORT_OFFSET) + ctr), 0);
				}
			}

#ifdef __ARMEB__
			PUT32(slave_mailbox0_addr, SWAP32(SLAVE_TX_START));
#else
			PUT32(slave_mailbox0_addr, SLAVE_TX_START);
#endif					
			
			data_rcv[0] = NOT_RCVD;
			data_rcv[1] = NOT_RCVD;
			data_rcv[2] = NOT_RCVD;
			data_rcv[3] = NOT_RCVD;

			ctr = 0;

			for(port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				do
				{
					if (ctr == 50000)
					{
						eprintf("No packets received...Timed out...\n");
						flag = ERROR;
						break;
					}

					ring_value = GET32(get_ring_addr);

					if (ring_value > 0)
					{
						data_rcv[((ring_value >> 16) & MASK_16BIT)] = RCVD;
						data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)] = ring_value & MASK_16BIT;
//						eprintf("data_rcv_bytes[%d]=%d\n", ((ring_value >> 16) & MASK_16BIT), data_rcv_bytes[((ring_value >> 16) & MASK_16BIT)]);
						ctr = 0;
					}
					else
					{
						ctr++;
						hal_delay_us(100);
					}
				}
				while (ring_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}


			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				if (data_rcv_bytes[port_ctr] != START_TX)
				{
					if (data_rcv_bytes[port_ctr] != (START_TX + 4))
					{
						eprintf("Packet size is incorrect...\n");
						flag = ERROR;
						break;
					}
				}


				for (ctr = 0; ctr < START_TX; ctr+=4)
				{
					temp0 = dual_lb_data[port_ctr][ctr/4];
					hal_delay_us(100);
					temp1 = GET32(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET));
					
					if(temp0 != temp1)
					{
						if (flag == OK)
						{
							flag = ERROR;
						}

						eprintf("Packet data integrity error!!! Loc: 0x%x, Expected: 0x%x, Actual: 0x%x\n",
								(DRAM_BASE_OFFSET + ctr + (port_ctr*PORT_OFFSET)), temp0, temp1);
					}
				}

				if (flag == ERROR);
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_counter++;
			eprintf("Packets received: %d\r", (loop_counter * 4));

			if ((loop_count == 0) && KEYPRESS)
			{
#ifdef __ARMEB__
				PUT32(slave_mailbox0_addr, SWAP32(TEST_COMPLETE));
#else
				PUT32(slave_mailbox0_addr, TEST_COMPLETE);
#endif					
				break;
			}
		}
		while (loop_counter != loop_count);

		eprintf("\n");

		if (flag == OK)
		{
			eprintf("Point Lone on Switch Fabric I/F Loopback test...PASSED\n");
			Set_LED("MSpP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Point Lone on Switch Fabric I/F Loopback test...FAIL!\n");
			Set_LED("MSpF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		
	}
	else // ##### SLAVE #####
	{
		loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);
		channel = (UINT8)GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
		loop_counter = 0;

//		eprintf("Loop_count: %d\n", loop_count);

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		
		for(port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			if (channel == 'c')
			{
				temp_port_ctr = (port_ctr & 0x2) & (!(port_ctr & 0x1));
				for (ctr = 0; ctr < sizeof(dual_lb_data[temp_port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[temp_port_ctr][(ctr / 4)]);
				}
			}
			else //if (channel == 'f')
			{
				for (ctr = 0; ctr < sizeof(dual_lb_data[port_ctr]); ctr+=4)
				{
					PUT32((DRAM_BASE_OFFSET + (port_ctr*PORT_OFFSET) + ctr), dual_lb_data[port_ctr][(ctr / 4)]);
				}
			}
		}

		// Load ME with microcode
		DownLoadUcode(0, sizeof(media_lb_pl_on_sf_iface_tx), (UINT32 *)media_lb_pl_on_sf_iface_tx);
		hal_delay_us(1000);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), SLAVE_READY_TO_RX_TX);

		ctr = 0;

		do
		{
			signal_from_master = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);
			
			if (signal_from_master == TEST_COMPLETE)
			{
				break;
			}
			else if (signal_from_master == SLAVE_TX_START)
			{
				PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);
				PUT32(port_0_put_ring_addr, START_TX);
				PUT32(port_1_put_ring_addr, START_TX);
				PUT32(port_2_put_ring_addr, START_TX);
				PUT32(port_3_put_ring_addr, START_TX);

				ctr = 0;
				do
				{
					if (ctr == 100000)
					{
						eprintf("No indication of any packets transmitted\n");
						flag = ERROR;
						break;
					}

					tx_counter = GET32(get_ring_addr);

					if (tx_counter > 0)
					{
						eprintf("Packets transmitted: %d\r", tx_counter);
						break;
					}

					ctr++;
					hal_delay_us(10);
				}
				while (tx_counter == 0);

				if (flag == ERROR)
				{
					break;
				}

				loop_counter++;
			}
			else
			{
				ctr++;
				hal_delay_us(10);
			}

			if (ctr == 750000)
			{
				eprintf("No indication from Master to transmit packets\n");
				break;
			}
		}
		while ((loop_counter != loop_count) || (loop_count == 0));
	} // ##### SLAVE END #####

	eprintf("\n");

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}

