/* emi_test.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 14, 02
 *  revisions: dalsraja
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "timer.h"
#include "led.h"
#include "uart.h"
#include "diag_utils.h"
#include "syslog.h"
#include "hal_ixdp2400.h"
#include "hal_platform_ints.h"
#include "msf_init.h"
#include "pci.h"
#include "uc_load.h"
#include "msf_common.h"
#include "reg_api.h"
#include "error_code.h"
#include "mac_init.h"

#ifdef EMI
#include "emi_test_ucode.h"

typedef struct initRegTable
{
	unsigned short* offset;
	unsigned short value;
} InitRegTable_t;


typedef struct  // Complete Data for a Chip
{
   unsigned short*   BaseAddress;	   // Base Address of chip
   unsigned long        ChipType;      // Type of Chip
   void*                pChipCfg;      // Pointer to Chip Specific Configuration
   void*                pAlarmCfg;     // Pointer to Chip Specific Alarm Config
   void*                funcPtr;       // Pointer to the chip's api func's
} bb_ChipData_t;

typedef unsigned long ulong;
typedef unsigned short ushort;

UINT32 run_nic_tx_rx = false;

extern pci_device slave_dev_info, nic_dev_info;		// for slave npu and nic info
extern unsigned int *orig_sp_val;

#define EMI_TEST_BD_TX_ME	0
#define EMI_TEST_BD_RX_ME	0
#define EMI_TEST_BW_TX_ME0	0
#define EMI_TEST_BW_TX_ME1	1
#define EMI_TEST_BW_TX_ME2	2
#define EMI_TEST_BW_TX_ME3	3
#define EMI_TEST_BW_RX_ME0	0
#define EMI_TEST_BW_RX_ME1	1
#define EMI_TEST_BW_RX_ME2	2
#define EMI_TEST_BW_RX_ME3	3

#define EMI_TEST_PL_TX_ME0	0
#define EMI_TEST_PL_TX_ME1	1
#define EMI_TEST_PL_TX_ME2	2
#define EMI_TEST_PL_TX_ME3	3
#define EMI_TEST_PL_RX_ME0	0
#define EMI_TEST_PL_RX_ME1	1
#define EMI_TEST_PL_RX_ME2	2
#define EMI_TEST_PL_RX_ME3	3

#define TBUF_ELEM_SIZE		0x80
#define TOT_TBUF_ELEM		(0x2000 / TBUF_ELEM_SIZE)
#define PACKET_SIZE			64
#define ELEMENTS_IN_ARRAY	(PACKET_SIZE / 4)
#define TBUF_BASE			0xC8002000
#define END_TEST_FLAG		0xDEAD
#define ETHERNET_ERROR		0xEBAD
#define EMI_DRAM_BASE		0x1000000

#define SEQ_UPDATE_BASE		0x1300
#define SPEED_ADDR			0x1100

/* timer (0x3000000) = 1 pkt/s;
 * 1 pkt = (1500 bytes)* (8 bits/byte) = 12000 bits
 * 1Mb = (1000000 bits)/(12000 bits/pkt) = 83.33 pkts = 250/3 pkts
 * 1Mb/s  = timer(0x3000000/(250/3))
 * 20Mb/s = timer(0x3000000/(250/3)/20)
 * but since both NPUs are transmitting, multiply the number by 2
 */
#define ETHERNET_1MB_PER_S	(0x03000000/(250/3))
#define ETHERNET_40MB_PER_S	(ETHERNET_1MB_PER_S/40)	// currently it is not accurate but it is ~35Mb/s
#define ETHERNET_THROUGHPUT	ETHERNET_40MB_PER_S*2

#define INIT_ADDR		0x200
#define MESSAGE_ADDR	0x1000

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;

extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern const InitRegTable_t BD_IntLoopbackArray[];
extern const InitRegTable_t BD_ExtLoopbackArray[];
extern const InitRegTable_t BW_IntLoopbackArray[];
extern const InitRegTable_t BW_ExtLoopbackArray[];

extern UINT32 ports, speed, cport;  
extern char channel, test_opt, dplx, loopback_mode;

extern ulong IxfApiInit(void *baseAddress, ulong chipType, bb_ChipData_t *pChipData);
extern ulong Ixf6048InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);
extern void spConfig(void);
extern void spRestore(void);
extern void Save_N_Set_Sp_Vals(void);
extern void Restore_Sp_Vals(void);

//extern void reg_write( volatile uint32 offset, uint32 val);

UINT32 emi_packet0[] = {	0x00000000, 0x00010002, 0x00030004, 0x00050006,
								0x00070008, 0x0009000A, 0x000B000C, 0x000D000E,
								0x000F0010, 0x00110012, 0x00130014, 0x00150016,
								0x00170018, 0x0019001A, 0x001B001C, 0x001D001E	};

UINT32 emi_packet1[] = {	0x00010000, 0x10011002, 0x10031004, 0x10051006,
								0x10071008, 0x1009100A, 0x100B100C, 0x100D100E,
								0x100F1010, 0x10111012, 0x10131014, 0x10151016,
								0x10171018, 0x1019101A, 0x101B101C, 0x101D101E  };

UINT32 emi_packet2[] = {	0x00020000, 0x20012002, 0x20032004, 0x20052006,
								0x20072008, 0x2009200A, 0x200B200C, 0x200D200E,
								0x200F2010, 0x20112012, 0x20132014, 0x20152016,
								0x20172018, 0x2019201A, 0x201B201C, 0x201D201E  };

UINT32 emi_packet3[] = {	0x00030000, 0x30013002, 0x30033004, 0x30053006,
								0x30073008, 0x3009300A, 0x300B300C, 0x300D300E,
								0x300F3010, 0x30113012, 0x30133014, 0x30153016,
								0x30173018, 0x3019301A, 0x301B301C, 0x301D301E  };

void EMI_Test(void);
void EMI_Timer2_Interrupt_Handler(void);
void EMI_BD_Master(void);
void EMI_BD_Slave(void);
void EMI_BW_Master(void);
void EMI_BW_Slave(void);
void EMI_Help(void);
void EMI_Deactivate_Timer(void);
void EMI_Timer_Init(void);
void EMI_Deactivate_Timer(void);
extern void emi_init(void);
extern int emi_txpacket(void);
void EMI_PL_Master(void);
void EMI_PL_Slave(void);
extern int get_niclink_status(void);
void EMI_Help(void)
{
	eprintf("Diag> t emi <option>\n");
	eprintf("where <option> are\n");
	eprintf("\td - Battery Drew (Single OC-48)\n");
	eprintf("\tw - Battery Wallace (Quad OC-12)\n");
    eprintf("\tp  <mode> <speed> <duplex> - Point Lone (Quad Gigabit Ethernet)\n");
    eprintf("\twhere <mode> are\n");
    eprintf("\tf - fiber\n");
    eprintf("\tc - copper\n");
    eprintf("\tm0 - mixed mode with Ports 0,1 Copper and Ports 2,3 Fibre\n");
    eprintf("\tm2 - mixed mode with Ports 0,1 Fibre and Ports 2,3 Copper\n");
    eprintf("\t<speed> - 1000\n");
    eprintf("\t<duplex> f - full\n");
	eprintf("\th - Help\n");
}

void EMI_Test(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	if(acL->HostType == MASTER)
	{
		if (acL->argv[2][0] == 'd')
		{
			EMI_BD_Master();
		}
		else if (acL->argv[2][0] == 'w')
		{
			EMI_BW_Master();
		}
         else if (acL->argv[2][0] == 'p')
		{
			EMI_PL_Master();
		}
       	else
		{
			EMI_Help();
		}
	}
	else
	{
		if (acL->argv[2][0] == 'd')
		{
			EMI_BD_Slave();
		}
		else if (acL->argv[2][0] == 'w')
		{
			EMI_BW_Slave();
		}
        else if (acL->argv[2][0] == 'p')
		{
			EMI_PL_Slave();
		}
		else
		{
			EMI_Help();
		}
	}
}

void EMI_Timer_Init(void)
{
	emi_init();
	// Timer 2
	run_nic_tx_rx = false;
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER2_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLR, CLEAR_INTERRUPT);		// Clear Timer2 Interrupt
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER2_BIT); 		// Set Timer2 IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLD, ETHERNET_THROUGHPUT);	// Load Timer2 Counter
	PUT32(TIMERS_TIMER2_CTL, ACTIVATE_TIMER);		// Activate Timer2
}


void EMI_Timer2_Interrupt_Handler(void)
{
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER2_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLR, CLEAR_INTERRUPT);		// Clear Timer2 Interrupt
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER2_BIT); 		// Set Timer2 IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLD, ETHERNET_THROUGHPUT);	// Load Timer2 Counter
	PUT32(TIMERS_TIMER2_CTL, ACTIVATE_TIMER);		// Activate Timer2
	//	PutChar('.');
	run_nic_tx_rx = true;
}

void EMI_Deactivate_Timer(void)
{
	PUT32(TIMERS_TIMER2_CTL, 0);	// Deactivate Timer2
	PUT32(TIMERS_TIMER2_CLR, 0x1);
}

void EMI_BD_Master(void)
{
	UINT32 status;
	UINT32 init_complete_addr, init_complete_value, ctr;
	UINT32 slave_message_addr, slave_message_value;
	UINT32 dcache_status;
	UINT32 element, array;
	UINT32 ethernet_status = 0;
	bb_ChipData_t pc;
    int prev_link_status =0, curr_link_stat=1;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);

	eprintf("Setting to fiber loopback mode...\n");
	Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");
#if 0	 //This is needed only for the EMI testing
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	// Put the data into the TBUF for transmitting
	for (element = 0; element < TOT_TBUF_ELEM; element++)
	{
		for (array = 0; array < ELEMENTS_IN_ARRAY; array++)
		{
			if (array == 0)
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE)), (emi_packet0[array] | (element << 24)));
			}
			else
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE) + (array * 4)), emi_packet0[array]);
			}
		}
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	init_complete_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + INIT_ADDR;
	slave_message_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MESSAGE_ADDR;

	PUT32(init_complete_addr, 1);

	ctr = 30;
	do	// Wait for a signal from the slave for indication of code loading complete
	{
		eprintf("Please type \"t emi d\" on the slave console...%02d\r", ctr);
		if (ctr == 0)	// After 30 seconds
		{
			eprintf("\nNo response from Slave...Ending EMI test!!!\n");
			Clear_All_ME();
			return;
		}

		// Poll on slave's scratch to check if init completed by Slave
		init_complete_value = GET32(init_complete_addr);

		if (init_complete_value != 0)
		{
			hal_delay_us(1000000);
			ctr--;
		}
	}
	while (init_complete_value != 0);
	eprintf("\n");

	DownLoadUcode(EMI_TEST_BD_TX_ME, sizeof(emi_bd_tx), (UINT32 *)emi_bd_tx);
	
	hal_delay_us(10000);

	EMI_Timer_Init();

	ctr = 0;
	do
	{
#ifdef __ARMEB__
		slave_message_value = SWAP32(GET32(slave_message_addr));
#else
		slave_message_value = GET32(slave_message_addr);
#endif

		if (KEYPRESS)
		{
#ifdef __ARMEB__
			PUT32(slave_message_addr, SWAP32(END_TEST_FLAG));
#else
			PUT32(slave_message_addr, END_TEST_FLAG);
#endif
			break;
		}

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
#endif
				if(acL->HaltOnError)
					break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}
	if((curr_link_stat)  && (ethernet_status != ERROR))
		eprintf("%d\r", ctr++);

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
#endif
                if(acL->HaltOnError)
                    break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}

		if (slave_message_value == SEQUENCE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA sequence error was seen by the slave\n");
		}
		else if (slave_message_value == RECEIVE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA receive error was seen by the slave (please check the packet)\n");
		}

		if ((slave_message_value == SEQUENCE_ERROR) || (slave_message_value == RECEIVE_ERROR))
		{
			if(acL->HaltOnError)
				eprintf("EMI Test Failed!!!\n");
			Set_LED("EmiF");
		}
		else if(slave_message_value == ETHERNET_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
			Set_LED("EmiF");
		}

		if (slave_message_value == 0)
		{
			hal_delay_us(1);
		}
	}
	while((slave_message_value == 0) || !acL->HaltOnError);

	EMI_Deactivate_Timer();

	Clear_All_ME();

#if 0	 //This is needed only for the EMI testing
	//Restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif

}


void EMI_BD_Slave(void)
{
	UINT32 status, scratch_val, dcache_status, ctr;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_BD_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
#if 0	 //This is needed only for the EMI testing
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR), 0);
	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR + 4), 0);

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_BD_RX_ME, sizeof(emi_bd_rx), (UINT32 *)emi_bd_rx);

	hal_delay_us(1000000);

	PUT32((SCRATCHPAD_BASE + INIT_ADDR), 0);
	
	ctr = 0;
	do
	{
		scratch_val = GET32(SCRATCHPAD_BASE + MESSAGE_ADDR);
		eprintf("%d\r", ctr++);

        if(scratch_val == END_TEST_FLAG)
            break;

        if ((scratch_val == SEQUENCE_ERROR) || (scratch_val == RECEIVE_ERROR))
	    {
		    if(acL->HaltOnError)
			{
				eprintf("\nThe Receive Status Word was 0x%08X\n", GET32(SCRATCHPAD_BASE + MESSAGE_ADDR + 4));
				eprintf("The received packet is stored in 0x%X (DRAM)\n", EMI_DRAM_BASE);
			}
	    }
	    else if(scratch_val == ETHERNET_ERROR)
	    {
		    if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
	    }
	}
	while((scratch_val == 0) || !acL->HaltOnError);

	Clear_All_ME();
#if 0	 //This is needed only for the EMI testing
	//Restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif

}

// SAMUEL: Add in interrupt handler, writing to slave's scratchpad location (scratch base + MESSAGE_ADDR) 
// if ethernet error occurs


void EMI_BW_Master(void)
{
	UINT32 status;
	UINT32 init_complete_addr, init_complete_value, ctr, port_ctr;
	UINT32 slave_message_addr, slave_message_value;
	UINT32 dcache_status;
	UINT32 element, array;
	UINT32 ethernet_status = 0;
	UINT32 *emi_data;
	bb_ChipData_t pc;
	int prev_link_status =0, curr_link_stat=1;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	
	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	
	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);

	eprintf("Setting to fiber loopback mode...\n");
	Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
//	Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

#if 0	 //This is needed only for the EMI testing
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			emi_data = (UINT32 *)emi_packet0;
		}
		else if (port_ctr == 1)
		{
			emi_data = (UINT32 *)emi_packet1;
		}
		else if (port_ctr == 2)
		{
			emi_data = (UINT32 *)emi_packet2;
		}
		else
		{
			emi_data = (UINT32 *)emi_packet3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEMENTS_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (emi_data[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), emi_data[array]);
				}
			}
		}
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	init_complete_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + INIT_ADDR;
	slave_message_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MESSAGE_ADDR;

	PUT32(init_complete_addr, 1);

	ctr = 30;
	do	// Wait for a signal from the slave for indication of code loading complete
	{
		eprintf("Please type \"t emi w\" on the slave console...%02d\r", ctr);
		if (ctr == 0)	// After 30 seconds
		{
			eprintf("\nNo response from Slave...Ending EMI test!!!\n");
			Clear_All_ME();
			return;
		}

		// Poll on slave's scratch to check if init completed by Slave
		init_complete_value = GET32(init_complete_addr);

		if (init_complete_value != 0)
		{
			hal_delay_us(1000000);
			ctr--;
		}
	}
	while (init_complete_value != 0);
	eprintf("\n");

	DownLoadUcode(EMI_TEST_BW_TX_ME1, sizeof(emi_bw_tx), (UINT32 *)emi_bw_tx);
	DownLoadUcode(EMI_TEST_BW_TX_ME2, sizeof(emi_bw_tx), (UINT32 *)emi_bw_tx);
	DownLoadUcode(EMI_TEST_BW_TX_ME3, sizeof(emi_bw_tx), (UINT32 *)emi_bw_tx);
	DownLoadUcode(EMI_TEST_BW_TX_ME0, sizeof(emi_bw_tx), (UINT32 *)emi_bw_tx);
	
	hal_delay_us(10000);

	EMI_Timer_Init();

	ctr = 0;
	do
	{
#ifdef __ARMEB__
		slave_message_value = SWAP32(GET32(slave_message_addr));
#else
		slave_message_value = GET32(slave_message_addr);
#endif

//		eprintf("slave_message_value=0x%08X\n", slave_message_value);
		if (KEYPRESS)
		{
#ifdef __ARMEB__
			PUT32(slave_message_addr, SWAP32(END_TEST_FLAG));
#else
			PUT32(slave_message_addr, END_TEST_FLAG);
#endif
			break;
		}

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
#endif
				if(acL->HaltOnError)
					break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}
if((curr_link_stat)  && (ethernet_status != ERROR))
		eprintf("%d\r", ctr++);

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
		#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
		#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
		#endif
				if(acL->HaltOnError)
					break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}

		if (slave_message_value == SEQUENCE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA sequence error was seen by the slave\n");
		}
		else if (slave_message_value == RECEIVE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA receive error was seen by the slave (please check the packet)\n");
		}
    
		if ((slave_message_value == SEQUENCE_ERROR) || (slave_message_value == RECEIVE_ERROR))
		{
			if(acL->HaltOnError)
				eprintf("EMI Test Failed!!!\n");
			Set_LED("EmiF");
		}
		else if(slave_message_value == ETHERNET_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
			Set_LED("EmiF");
		}
    
		if (slave_message_value == 0)
		{
			hal_delay_us(1);
		}
	}
	while((slave_message_value == 0) || !acL->HaltOnError);

	EMI_Deactivate_Timer();

	Clear_All_ME();
#if 0	 //This is needed only for the EMI testing
	//restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif

}


void EMI_BW_Slave(void)
{
	UINT32 status, scratch_val, dcache_status, ctr;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_BW_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
#if 0	 //This is needed only for the EMI testing
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR), 0);
	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR + 4), 0);

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_BW_RX_ME1, sizeof(emi_bw_rx), (UINT32 *)emi_bw_rx);
	DownLoadUcode(EMI_TEST_BW_RX_ME2, sizeof(emi_bw_rx), (UINT32 *)emi_bw_rx);
	DownLoadUcode(EMI_TEST_BW_RX_ME3, sizeof(emi_bw_rx), (UINT32 *)emi_bw_rx);
	DownLoadUcode(EMI_TEST_BW_RX_ME0, sizeof(emi_bw_rx), (UINT32 *)emi_bw_rx);

	hal_delay_us(1000000);

	PUT32((SCRATCHPAD_BASE + INIT_ADDR), 0);
	
	ctr = 0;
	do
	{
		scratch_val = GET32(SCRATCHPAD_BASE + MESSAGE_ADDR);
		eprintf("%d\r", ctr++);

        if(scratch_val == END_TEST_FLAG)
            break;

        if ((scratch_val == SEQUENCE_ERROR) || (scratch_val == RECEIVE_ERROR))
	    {
		    if(acL->HaltOnError)
			{
				eprintf("\nThe Receive Status Word was 0x%08X\n", GET32(SCRATCHPAD_BASE + MESSAGE_ADDR + 4));
				eprintf("The received packet is stored in 0x%X (DRAM)\n", EMI_DRAM_BASE);
			}
	    }
	    else if(scratch_val == ETHERNET_ERROR)
	    {
		    if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
	    }
	}
	while((scratch_val == 0) || !acL->HaltOnError);

	Clear_All_ME();

#if 0	 //This is needed only for the EMI testing	
	//restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif

}


void EMI_PL_Master(void)
{
	UINT32 status;
	UINT32 init_complete_addr, init_complete_value, ctr, port_ctr;
	UINT32 slave_message_addr, slave_message_value;
	UINT32 dcache_status;
	UINT32 element, array;
	UINT32 ethernet_status = 0;
	UINT32 *emi_data;
	int prev_link_status =0, curr_link_stat=1;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

    channel ='f'; //set the channel to fiber by default
    channel = acL->argv[3][0]; 
    speed = acL->decArg[4];
	dplx = acL->argv[5][0];
	eprintf("Initialising PL media card.  Please wait...\n");
    
	ports = 4;
	loopback_mode = 'e';

	if (channel  == 'f')
	{
		if (speed != 1000)
		{
			eprintf("For fibre mode, only speed of 1000 can be selected.\n");
			return;
		}

		if (dplx != 'f')
		{
			eprintf("Fibre mode can only be run in full duplex.\n");
			return;
		}

		eprintf("Initialising media card in fiber mode. Please wait...\n");
	}
    else if (channel =='c')
	{
		if ((speed != 1000) && (speed != 100) && (speed != 10))
		{
			eprintf("Please specify a speed of 10, 100 or 1000.\n");
			return;
		}

		if ((dplx != 'f') && (dplx != 'h'))
		{
			eprintf("Please specify a proper duplex mode.\n");
			return;
		}

		eprintf("Initialising media card in copper mode. Please wait...\n");
	}
    else if (channel == 'm')
	{
		if ((acL->argv[3][1] != '0') && (acL->argv[3][1] != '2'))
		{
			eprintf("Please specify which port is supposed to be copper (0 or 2)\n");
			return;
		}

		if (acL->argv[3][1] == '0')
		{
			cport = 0;
		}
		else if (acL->argv[3][1] == '2')
		{
			cport = 2;
		}

		if (speed != 1000)
		{
			eprintf("For mixed mode, only speed of 1000 can be selected.\n");
			return;
		}

		if (dplx != 'f')
		{
			eprintf("Mixed mode can only be run in full duplex.\n");
			return;
		}

		eprintf("Initialising media card in mixed mode. Please wait...\n");
	}
	else
	{
		eprintf("Please specify a valid media type!!! ABORTING EMI TEST...\n");
		return;
	}
    
	status = init_mac_dev(0);	
    if (status != DONE)
	{
		eprintf("Media Init FAIL!!!. Aborting Test\n");
		return;
	}

    spConfig();
	reg_write(0x5b2,0x0);
	spRestore();
   	
	eprintf("Media card initialisation completed.\n");
#if 0	 //This is needed only for the EMI testing	
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			emi_data = (UINT32 *)emi_packet0;
		}
		else if (port_ctr == 1)
		{
			emi_data = (UINT32 *)emi_packet1;
		}
		else if (port_ctr == 2)
		{
			emi_data = (UINT32 *)emi_packet2;
		}
		else
		{
			emi_data = (UINT32 *)emi_packet3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEMENTS_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (emi_data[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), emi_data[array]);
				}
			}
		}
	}

	if (speed == 1000)
	{
		PUT32((SCRATCHPAD_BASE + SPEED_ADDR), 48);
	}
	else if (speed == 100)
	{
		PUT32((SCRATCHPAD_BASE + SPEED_ADDR), 480);
	}
	else if (speed == 10)
	{
		PUT32((SCRATCHPAD_BASE + SPEED_ADDR), 4800);
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	init_complete_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + INIT_ADDR;
	slave_message_addr = slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + MESSAGE_ADDR;

	PUT32(init_complete_addr, 1);
	
	ctr = 30;
	do	// Wait for a signal from the slave for indication of code loading complete
	{
		eprintf("Please type \"t emi p\" on the slave console...%02d\r", ctr);
		if (ctr == 0)	// After 30 seconds
		{
			eprintf("\nNo response from Slave...Ending EMI test!!!\n");
			Clear_All_ME();
			return;
		}

		// Poll on slave's scratch to check if init completed by Slave
		init_complete_value = GET32(init_complete_addr);

		if (init_complete_value != 0)
		{
			hal_delay_us(1000000);
			ctr--;
		}
	}
	while (init_complete_value != 0);
	eprintf("\n");

	if(dplx == 'f')
	{
		DownLoadUcode(EMI_TEST_PL_TX_ME1, sizeof(emi_pl_tx), (UINT32 *)emi_pl_tx);
	}

	DownLoadUcode(EMI_TEST_PL_TX_ME2, sizeof(emi_pl_tx), (UINT32 *)emi_pl_tx);

	if(dplx == 'f')
	{
		DownLoadUcode(EMI_TEST_PL_TX_ME3, sizeof(emi_pl_tx), (UINT32 *)emi_pl_tx);
	}
	
	DownLoadUcode(EMI_TEST_PL_TX_ME0, sizeof(emi_pl_tx), (UINT32 *)emi_pl_tx);
	
	hal_delay_us(10000);

	EMI_Timer_Init();

	ctr = 0;
	do
	{
#ifdef __ARMEB__
		slave_message_value = SWAP32(GET32(slave_message_addr));
#else
		slave_message_value = GET32(slave_message_addr);
#endif

//		eprintf("slave_message_value=0x%08X\n", slave_message_value);

		if (KEYPRESS)
		{
#ifdef __ARMEB__
			PUT32(slave_message_addr, SWAP32(END_TEST_FLAG));
#else
			PUT32(slave_message_addr, END_TEST_FLAG);
#endif
			break;
		}

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
#endif
				if(acL->HaltOnError)
					break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}
if((curr_link_stat)  && (ethernet_status != ERROR))
		eprintf("%d\r", ctr++);

		if(run_nic_tx_rx == true)
		{
			curr_link_stat =get_niclink_status();
            if(prev_link_status != curr_link_stat)
			{
				prev_link_status = curr_link_stat;
				if(acL->Verboselevel !=0)
					eprintf ("\nNIC Link %s\n", (curr_link_stat == 1) ? "UP" : "DOWN");
			}
			ethernet_status = emi_txpacket();
			run_nic_tx_rx = false;
			if(ethernet_status == ERROR)
			{
				slave_message_value = ETHERNET_ERROR;
		#ifdef __ARMEB__
				PUT32(slave_message_addr, SWAP32(ETHERNET_ERROR));
		#else
				PUT32(slave_message_addr, ETHERNET_ERROR);
		#endif
				if(acL->HaltOnError)
					break;
			}
/*			else
			{
				slave_message_value = 0;
				PUT32(slave_message_addr, 0);
			}
*/		}

		if (slave_message_value == SEQUENCE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA sequence error was seen by the slave\n");
		}
		else if (slave_message_value == RECEIVE_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nA receive error was seen by the slave (please check the packet)\n");
		}
    
		if ((slave_message_value == SEQUENCE_ERROR) || (slave_message_value == RECEIVE_ERROR))
		{
			if(acL->HaltOnError)
				eprintf("EMI Test Failed!!!\n");
			Set_LED("EmiF");
		}
		else if(slave_message_value == ETHERNET_ERROR)
		{
			if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
			Set_LED("EmiF");
		}
    
		if (slave_message_value == 0)
		{
			hal_delay_us(1);
		}
	}
	while((slave_message_value == 0) || !acL->HaltOnError);

	EMI_Deactivate_Timer();

	Clear_All_ME();

#if 0	 //This is needed only for the EMI testing
	//restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif

}


void EMI_PL_Slave(void)
{
	UINT32 status, scratch_val, dcache_status, ctr;
	UINT32 ctr1=0, ctr3=0, seq_port1_new=0, seq_port3_new=0, seq_port1_old=0, seq_port3_old=0;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_BW_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
#if 0	 //This is needed only for the EMI testing
	//For emi disable HOE and verbose vgd
	acL->HaltOnError = 0;
	acL->Verboselevel =0;
#endif

	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR), 0);
	PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR + 4), 0);

	Clear_All_ME();
	DownLoadUcode(EMI_TEST_PL_RX_ME1, sizeof(emi_pl_rx), (UINT32 *)emi_pl_rx);
	DownLoadUcode(EMI_TEST_PL_RX_ME2, sizeof(emi_pl_rx), (UINT32 *)emi_pl_rx);
	DownLoadUcode(EMI_TEST_PL_RX_ME3, sizeof(emi_pl_rx), (UINT32 *)emi_pl_rx);
	DownLoadUcode(EMI_TEST_PL_RX_ME0, sizeof(emi_pl_rx), (UINT32 *)emi_pl_rx);

	hal_delay_us(1000000);

	PUT32((SCRATCHPAD_BASE + INIT_ADDR), 0);
	
	ctr = 0;
	do
	{
		seq_port1_new = GET32(SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0x4);
		seq_port3_new = GET32(SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0xC);

		if (seq_port1_old == seq_port1_new)
		{
			ctr1++;
		}
		else
		{
			ctr1 = 0;
			seq_port1_old = seq_port1_new;
		}

		if (seq_port3_old == seq_port3_new)
		{
			ctr3++;
		}
		else
		{
			ctr3 = 0;
			seq_port3_old = seq_port3_new;
		}

		hal_delay_us(100);

		scratch_val = GET32(SCRATCHPAD_BASE + MESSAGE_ADDR);
		if ((scratch_val == RECEIVE_ERROR))
			eprintf("%d %d %d\r", ctr, ctr1, ctr3);
		else
			 eprintf("%d %d %d\r", ctr++, ctr1, ctr3);


        if(scratch_val == END_TEST_FLAG)
		{
			break;
		}

        if ((scratch_val == SEQUENCE_ERROR) || (scratch_val == RECEIVE_ERROR))
	    {
		   if(acL->HaltOnError)
			{
		     eprintf("\nThe Receive Status Word was 0x%08X\n", GET32(SCRATCHPAD_BASE + MESSAGE_ADDR + 4));
		     eprintf("The received packet is stored in 0x%X (DRAM)\n", EMI_DRAM_BASE);
			}
	    }
	    else if(scratch_val == ETHERNET_ERROR)
	    {
		    if(acL->HaltOnError)
				eprintf("\nEthernet FAIL!!!\n");
	    }
	}
	while(((scratch_val == 0) || !acL->HaltOnError) && ((ctr1 != 1000)|| !acL->HaltOnError) && ((ctr3 != 1000)|| !acL->HaltOnError));

	if ((ctr1 == 1000) || (ctr3 == 1000))
	{
		PUT32((SCRATCHPAD_BASE + MESSAGE_ADDR), RECEIVE_ERROR);
	}

	Clear_All_ME();
#if 0	 //This is needed only for the EMI testing
	//restore HOE and verbose vgd
	acL->HaltOnError = 1;
	acL->Verboselevel =1;
#endif
}

#endif
