//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                       
// Some portions of the code were developed by Intel Corporation. 
// Portions created by Intel Corporation are:
// Copyright  2002-2004 Intel Corporation
//####COPYRIGHTEND####
//=============================================================================

#include "misc_func.h"
#include "cfg_utility.h"
#include "diagstruct.h"
#include "memory.h"
#include "ethernet.h"
#include "uart.h"
#include "diag_utils.h"
#include "console_monitor.h"
#include "led.h"
#include "syslog.h"
#include "hal_ixdp2400.h"
#include "hal_io.h"
#include "pci.h"
#include "pci_cfg.h"
#include "hal_platform_ints.h"
#include "hal_intr.h"
#include "pci_diag.h"

// Static Function prototypes
//============================
static void Ether_Help(void);
static void ether_test(void);

static void display_loopback_init_error(void);
static void display_internal_loopback_error(void);
static void display_external_error(void);

static UINT16 readMDI(UINT8 phyAdd, UINT8 regAdd);

static char *malloc (int size);
static int i559SelfTest(void);
static void sendCommand (UINT8 cuc, UINT8 ruc, UINT32 scb_general_ptr);
static void makePacket (UINT8 *pPacket, int length);
static int i559Init(void);

static int waitForInt(void);
static int i559AddrSet(void);
static int initPHY(UINT32 device_type, int loop_mode);
static int i559Config(UINT8 loopBackMode);
static int waitForRxInt(void);
static int checkPacket (UINT8 *pTxBuffer, UINT8 *pRxBuffer, int	length);
static int txPacket(char *pBuf);
static void pci_ether_test();
static int Device_Seek(UINT32 vendor_id, UINT32 dev_id,
						UINT32 dev_ind);
static void setUpPacket(char *pBuf);
static void setup_tx_cmd_blk(void);
static void signal_next_npu_to_rx(void);
void get_mac_from_eeprom(void);

/* Globals needed by both main program and irq handler */
static volatile struct SCBtype *pSCB = NULL;	/* Pointer to SCB in use */
static volatile UINT32 waitSem = 0;	/* Used to block test until interrupt */
static volatile UINT32 rxSem = 0;	/* Used to block test until rx sinterrupt */
static volatile UINT16 i559Status = 0;	/* Status code from SCB */
static UINT32 i82559_heap_size = 0;
static UINT8 *i82559_heap_base = NULL;
static UINT8 *i82559_heap_free = NULL;
static UINT32 adapter;		/* Ptr to PCI Ethernet adapter */
static volatile UINT8 S_MAC_address[6];	/* MAC address 6bytes */
static volatile UINT8 D_MAC_address[6];	/* MAC address 6bytes */
static volatile UINT8 EEPROM_MAC_address[6];	/* MAC address 6bytes */
static UINT32 phy_id = 0;

/* 82559 required data structures which must be allocated */
static struct rfd *pRfd = NULL;
static union cmdBlock *pCmdBlock = NULL;
static char	*pPacketBuf = "";
static char	*pPacketBuf_base = "";

pci_device nic_dev_info;
extern pci_device slave_dev_info, master_dev_info;

extern UINT32 t_all_verbose;
extern char __pci_window[];
extern struct board_config board_config_data;

extern int get_niclink_status(void);
// test flags
//============
static volatile int link_test_flag = 0;
static volatile int internal_loopback_test_flag = 0;
static volatile int external_test_flag = 0;
static volatile int self_test_flag = 0;

static volatile int waiting_for_packet = DISABLED;
static int verbose = DISABLED;
static volatile int transceiver = SELF;
static volatile int link_speed = SPEED_100M;
volatile int external_loops = 0;

/*******************
	Ether_Help
********************/
void Ether_Help(void)
{
	eprintf("> t nic <option> <d:link_speed> <d:loops> <d:verbose>,\n");
	eprintf("where <option> are\n");
	eprintf("l - Ethernet NIC Link Test\n");
	eprintf("i - Ethernet NIC Internal Loopback Test\n");
	eprintf("e[s/n] - Ethernet NIC External Test [Self/Next NPU]\n");
	eprintf("s - Ethernet NIC Self Test\n");
	eprintf("a - All Ethernet NIC Tests\n");
	eprintf("h - help\n");
}

/*******************
	Ethernet_Test
********************/
void Ethernet_test(void)
{
	char opt;
	int forever = 0, i, loops = 1;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
		
	opt = acL->argv[2][0];
	link_test_flag = 0;
	internal_loopback_test_flag = 0;
	external_test_flag = 0;
	self_test_flag = 0;
	verbose = 0;
	waiting_for_packet = DISABLED;

	acL->Err = OK;

	if(acL->argc < 4)
	{
		link_speed = AUTO_SPEED;
		loops = 1;
	}
	else
	{
		link_speed = acL->decArg[3];
		if(!((link_speed == SPEED_10M) || (link_speed == SPEED_100M)))
			link_speed = AUTO_SPEED;

		if(acL->argc < 5)
			loops = 1;
		else
		{
			loops = acL->decArg[4];
			if(loops == 0)
				forever = 1;

			if(acL->argc >= 5)
				verbose = acL->decArg[5];
		}
	}

	switch(opt)
	{
		case 'l': 	link_test_flag = 1;
					break;

		case 'i': 	internal_loopback_test_flag = 1;
					break;

		case 'e': 	external_test_flag = 1;
					external_loops = loops;
					loops = 1;
					forever = 0;
					if(acL->argv[2][1] == 's')
						transceiver = SELF;
					else
						transceiver = NEXT_NPU;
					break;

		case 's':	self_test_flag = 1;
					break;

		case 'a':	self_test_flag = 1;
					link_test_flag = 1;
					internal_loopback_test_flag = 1;
					break;
		case 'h':
		default	:	Ether_Help();
					return;
	}

	// Check MAC addr from I2C against MAC addr from EEPROM
	for(i = 0; i < 6; i++)
	{
		if(EEPROM_MAC_address[i] != S_MAC_address[i])
		{
			eprintf("MAC Addresses from I2C and EEPROM does not match!\n");
			acL->Err = ERROR;
			break;
		}
	}
	// Check first 3 bytes of MAC to be 0x0090D7
	if(	(EEPROM_MAC_address[0] != 0x00) ||
		(EEPROM_MAC_address[1] != 0x90) ||
		(EEPROM_MAC_address[2] != 0xD7))
	{
		eprintf("First three bytes of MAC address does not match 0x0090D7\n");
		acL->Err = ERROR;
	}

	
	while(loops-- || forever)
	{
		if(loops != 0)
			eprintf("loop count: %d\n", loops);
		if(KEYPRESS)
			break;
		ether_test();
	}
	
	if(acL->Err == ERROR)
		eprintf("NIC TEST FAIL!!!\n");

	Ethernet_rx_init();
	
	return;
}

/********************
	ether_test
*********************/
void ether_test(void)
{
	*PORT_REG(adapter) = PORT_RESET;
	// Get the UUT's ethernet address
	if (get_ether_addr(DISABLED) == ERROR)
	{
		eprintf("Error Reading Adapter Ethernet Address\n");
		return;
	}
	pci_ether_test();
}

void emi_init(void)
{
	*PORT_REG(adapter) = PORT_RESET;
	i559Init();
	i559AddrSet();
	internal_loopback_test_flag = 0;
	get_ether_addr(DISABLED);
	i559Config (NO_LOOP_BACK);
}

/********************
	Ethernet_init
*********************/
void Ethernet_init(void)
{
	if (Device_Seek(VENDOR_INTEL, I82559ER, 0) != OK)
	{
		eprintf("No supported Ethernet devices found\n");
		return;
	}

	get_ether_addr(DISABLED);

	/* read the PCI BAR for the Ethernet controller */
	adapter = nic_dev_info.base_map[0];

	*(PCI_ADDR_EXT_REG) = (nic_dev_info.base_address[0] & PCI_MEM_PAGE_MASK) >> 16;

	/* strip off BAR indicator bits */
	adapter &= 0xfffffff0;

	/* pointer to on-chip SCB */
	pSCB = (struct SCBtype *)(adapter + SCB_OFFSET);

	if (verbose)
	{
		eprintf("pSCB: 0x%x\n", pSCB);	
		eprintf ("NIC Base Address  = 0x%X\n", adapter);
	}

	/* Initialize malloc's memory pool pointer */
	i82559_heap_base = (char *) __pci_window;
	i82559_heap_size = 32*1024;  // match actual size in if_shmem.S
	i82559_heap_free = i82559_heap_base;
	
	if (verbose)
		eprintf("Resetting chip... ");

	// reset the 82559 to start with a clean slate
	*PORT_REG(adapter) = PORT_RESET;

	if (verbose)
		eprintf("Done.\n");
}

/**********************
	Ethernet_rx_init
************************/
void Ethernet_rx_init(void)
{
	rxSem = 0;	/* no Receive interrupt */
	waiting_for_packet = ENABLED;
	transceiver = NEXT_NPU;
	internal_loopback_test_flag = 0;

	*PORT_REG(adapter) = PORT_RESET;	
	get_ether_addr(DISABLED);

	/* Initialize malloc's memory pool pointer */
	i82559_heap_base = (char *) __pci_window;
	i82559_heap_size = 32*1024;  // match actual size in if_shmem.S
	i82559_heap_free = i82559_heap_base;
	
	// Initialize data structures
	i559Init();
	
	// Set hardware address
	i559AddrSet();
	
	// Configuring for external
	i559Config (NO_LOOP_BACK);

	// Setup a packet
	setUpPacket (pPacketBuf);
	setup_tx_cmd_blk();

	return;
}

/*******************
	Device_Seek
*********************/
int Device_Seek(UINT32 vendor_id, UINT32 dev_id, UINT32 dev_ind)
{
	UINT32 nic_dev_id = PCI_NULL_DEVID;
	UINT32 bus_number, device_number, function_number;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	bus_number = PCI_DEV_GET_BUS(dev_id);
	device_number = PCI_DEV_GET_DEV(PCI_DEV_GET_DEVFN(dev_id));
	function_number = PCI_DEV_GET_FN(PCI_DEV_GET_DEVFN(dev_id));

	if (verbose)
	{
		eprintf("Looking for Adapter on PCI Bus with:\n");
		eprintf("Vendor Id = 0x%04x\n",vendor_id);
		eprintf("Device Id = 0x%04x\n",dev_id);
		eprintf("Index     = %x\n",dev_ind);
	}
    if(pci_find_device(vendor_id, dev_id, &nic_dev_id) != true)
    {
		eprintf("Scan for NIC 1 FAIL!\n");
		return (ERROR);
    }
    else
    {
		if(acL->HostType == MASTER)	// Slave performs test on the second NIC
		{
			if(pci_find_device(vendor_id, dev_id, &nic_dev_id) != true)
		    {
				eprintf("Scan for NIC 2 FAIL!\n");
				return (ERROR);
		    }
		}

		pci_get_device_info(nic_dev_id, &nic_dev_info);
		if (verbose)
		{
			eprintf("Adapter found at :\n");
			eprintf("  PCI Bus Number : %d\n", bus_number);
			eprintf("  Device Number  : %d\n", device_number);
			eprintf("  Function Number: %d\n", function_number);
			eprintf("nic_dev_info.base_map[0]: 0x%x\n", nic_dev_info.base_map[0]);
			eprintf("nic_dev_info.base_map[1]: 0x%x\n", nic_dev_info.base_map[1]);
			eprintf("nic_dev_info.base_map[2]: 0x%x\n", nic_dev_info.base_map[2]);
		}
		return (OK);
    }
}


/*******************
	get_ether_addr
********************/
int get_ether_addr(int display)
{
	int i;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)
	{
		for (i=0; i<6; i++)
		{
			S_MAC_address[i] = board_config_data.config_data.M_ether_mac_addr[i];

			if((transceiver == SELF) || internal_loopback_test_flag)
				D_MAC_address[i] = board_config_data.config_data.M_ether_mac_addr[i];
			else
				D_MAC_address[i] = board_config_data.config_data.S_ether_mac_addr[i];
		}
	}
	else	// SLAVE
	{
		for (i=0; i<6; i++)
		{
			S_MAC_address[i] = board_config_data.config_data.S_ether_mac_addr[i];

			if((transceiver == SELF) || internal_loopback_test_flag)
				D_MAC_address[i] = board_config_data.config_data.S_ether_mac_addr[i];
			else
				D_MAC_address[i] = board_config_data.config_data.M_ether_mac_addr[i];
		}
	}
	
	if(display)
	{
		eprintf("Ethernet Address = 0x");
		for (i=0; i<6; i++)
	    {
	        eprintf("%02X", S_MAC_address[i]);
	    }
	    eprintf("\n");
	}

	return(OK);
}

/*********************
	pci_ether_test
*********************/
void pci_ether_test(void)
{
	UINT16 phy_addr_reg, temp1, temp2;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	
	// Self Test
	//===========
	if(self_test_flag)
	{
		// Run the built-in self test through the port register
		if (i559SelfTest() == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			eprintf ("NIC Self Test... FAIL!\n");
			Set_LED("Ns F");
			acL->Err = ERROR;
			slowport_code[0] = SP_NIC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "NIC Self Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));		
			return;
		}
		if(t_all_verbose != MINIMAL)
			eprintf ("NIC Self Test... PASS\n");
		Set_LED("Ns P");
		slowport_code[0] = SP_NIC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		// Reset clears the interrupt mask
		pciIntDisconnect(CPLD_NIC_INT);
	}

	// Link Test
	//===========
	if(link_test_flag || external_test_flag)
	{
		temp1 = readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_ID_1);
		temp2 = readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_ID_2);
		phy_id = ((temp1 << 16) | temp2);
		
		if ((phy_id & 0xfffffff0) == I82555_PHY_ID)
		{
			if(verbose)
				eprintf ("Intel 82555/558 PHY detected...\n");

			/* dummy read for reliable status */
			(void)readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT);

			temp1 = readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT);
			if(verbose)
				eprintf ("Status Register Link Status is %s\n", (temp1 & MDI_STAT_LINK) ? "UP" : "DOWN");

			phy_addr_reg = readMDI (MDI_DEFAULT_PHY_ADDR, I82555_STATCTRL_REG);

			if (temp1 & MDI_STAT_LINK)	// speed only valid with good LNK
  			{
				if(verbose)
		   			eprintf ("Connect Speed is %s\n", (phy_addr_reg & I82555_100_MBPS) ? "100Mbps" : "10Mbps");
				if(link_speed == AUTO_SPEED)
					link_speed = (phy_addr_reg & I82555_100_MBPS) ? SPEED_100M : SPEED_10M;
				if(t_all_verbose != MINIMAL)
					eprintf ("NIC Link Test... PASS\n");
				Set_LED("Nl P");
				slowport_code[0] = SP_NIC;
				slowport_code[1] = SP_PASS;
				dump_slowport(slowport_code, 2, SP_NON_FATAL);
			}
			else
			{
				eprintf ("Connect Speed is NOT VALID\n");
				eprintf ("NIC Link Test... FAIL!\n");
				Set_LED("Nl F");
				acL->Err = ERROR;
				slowport_code[0] = SP_NIC;
				slowport_code[1] = SP_FAIL;
				dump_slowport(slowport_code, 2, SP_NON_FATAL);
				// prepare structure for dumping
				syslog.type = TYPE_ERROR;
				syslog.source_comp = MASTER_DIAG;	// Determine master/slave
				strcpy(syslog.desc, "NIC Link Test FAIL!");

				// write to syslog
				syslog_dump(&syslog, sizeof(SYSLOG_DATA));		
				return;
			}
		}
		else
		{
			eprintf("***ERROR***: I82555 PHY not detected\n");
			Set_LED("Nl F");
			acL->Err = ERROR;
			slowport_code[0] = SP_NIC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "NIC Link Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			return;
		}
	} // if(link_test_flag)

	// Initialization for Loopback/External Test
	//==================================
	if(internal_loopback_test_flag || external_test_flag)
	{
		// Initialize data structures
		if(verbose)
			eprintf ("Initializing ethernet controller.\n");
		if (i559Init() == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_loopback_init_error();
			return;
		}
		// Set hardware address
		if(verbose)
			eprintf ("Setting hardware address.\n");
		if (i559AddrSet() == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_loopback_init_error();
			return;
		}
	}

	// Internal Loopback Test
	//========================
	if(internal_loopback_test_flag)
	{
		// Configure for Internal loopback
		if (i559Config (INT_LOOP_BACK) == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_internal_loopback_error();
			return;
		}

		// Initialize receive buffer and enable receiver
		if(verbose)
			eprintf ("Initializing receive buffer and enabling receiver.\n");
		if (i559RUStart () == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_internal_loopback_error();
			return;
		}	

		// Create a packet
		if(verbose)
			eprintf ("Sending packet.\n");
		setUpPacket (pPacketBuf);
		if (txPacket (pPacketBuf) == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_internal_loopback_error();
			return;
		}
		if(t_all_verbose != MINIMAL)
			eprintf ("NIC Internal Loopback Test... PASS\n");
		Set_LED("Ni P");
		slowport_code[0] = SP_NIC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}

	// External Test
	//===============
	if(external_test_flag)
	{
		if(transceiver == SELF)
		{
			link_speed = SPEED_10M;
		}

		if(internal_loopback_test_flag == 1)
		{
			// internal and external to other NPUs have different MAC addr
			internal_loopback_test_flag = 0;
			get_ether_addr(DISABLED);
			internal_loopback_test_flag = 1;
		}

		// Configuring for external
		if (i559Config (NO_LOOP_BACK) == ERROR)
		{
			pciIntDisconnect(CPLD_NIC_INT);
			display_external_error();
			return;
		}

		for(temp1 = 0; temp1 < external_loops; temp1++)
		{
			// Create a packet
			if(verbose)
				eprintf ("Sending packet.\n");
			setUpPacket (pPacketBuf);

			// Initialize receive buffer and enable receiver
			if(verbose)
				eprintf ("Initializing receive buffer and enabling receiver.\n");

			signal_next_npu_to_rx();

			if ((i559RUStart() == ERROR) || (txPacket(pPacketBuf) == ERROR))
			{
				pciIntDisconnect(CPLD_NIC_INT);
				display_external_error();
				return;
			}
			PutChar('.');
		}

		if(t_all_verbose != MINIMAL)
			eprintf ("\nNIC External Test... PASS\n");
		Set_LED("Ne P");
		slowport_code[0] = SP_NIC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		
	}	// if(external_test_flag)
}

/******************************************************************************
*
* sendCommand - send a command to the 82559 via the on-chip SCB
*
* Send a command to the 82559.  On the 82559, the Channel Attention signal
* has been replaced by an on-chip SCB.  Accesses to the Command Word portion
* of the SCB automatically forces the '559 to look at the various data
* structures which make up its interface.
*/
static void sendCommand (UINT8 cuc, UINT8 ruc, UINT32 scb_general_ptr)
{
	register CMD_STAT_U temp_cmdStat;
	volatile union cmdBlock *pBlock = (union cmdBlock *)BUS_TO_VIRT(scb_general_ptr);
	volatile int loop_ctr;
#ifdef A0_REV
	UINT32 temp;
#endif

	/* Mask adapter interrupts to prevent the interrupt handler from
	   playing with the SCB */
	pciIntDisconnect(CPLD_NIC_INT);

	/* must wait for the Command Unit to become idle to prevent
       us from issueing a CU_START to an active Command Unit */
    for (loop_ctr = BUSY_WAIT_LIMIT; loop_ctr > 0; loop_ctr--)
    {
        if ((pSCB->cmdStat.words.status & SCB_S_CUMASK) == SCB_S_CUIDLE)
            break;
    }
    if (loop_ctr == 0)
    {
        eprintf("sendCommand: CU won't go idle, command ignored\n");
		pciIntConnect(CPLD_NIC_INT, (UINT32)i559IntHandler, 0);
        return;
    }
	/* when setting the command word, read the current word from
	   the SCB and preserve the upper byte which contains the interrupt
	   mask bit */
#ifdef __ARMEB__
	temp_cmdStat.words.command = (pSCB->cmdStat.words.command & 0xff);
#else
	temp_cmdStat.words.command = (pSCB->cmdStat.words.command & 0xff00);
#endif
	temp_cmdStat.words.status = 0;

	/* set up the Command and Receive unit commands */
	temp_cmdStat.bits.cuc = cuc & 0xF;
	temp_cmdStat.bits.ruc = ruc & 0x7;

	/* Clear flag */
	waitSem = 0;

	/* write the General Pointer portion of the SCB first */
#ifdef __ARMEB__
		pSCB->scb_general_ptr = SWAP32(scb_general_ptr);
#else
		pSCB->scb_general_ptr = scb_general_ptr;
#endif

#ifdef A0_REV
	temp = pSCB->scb_general_ptr;
#endif

	/* write the Command Word of the SCB */
	pSCB->cmdStat.words.command = temp_cmdStat.words.command;
#ifdef A0_REV
	temp = pSCB->cmdStat.words.command;
#endif

	/* only wait for command which will complete immediately */
	if ((scb_general_ptr != 0/* NULL*/) && (ruc != RU_START))
	{
		/* wait for command acceptance and completion */
		for (loop_ctr = BUSY_WAIT_LIMIT; loop_ctr > 0; loop_ctr--)
		{
			if ((pSCB->cmdStat.bits.cuc == 0) && (pBlock->nop.c == 1))
				break;
		}
		if (loop_ctr == 0)
		{
			eprintf("\nsendCommand: Timeout on command complete\n");
			eprintf("pBlock: %x\n", pBlock);
			eprintf("pSCB: 0x%x\n", pSCB);
			eprintf("Cmd Complete bit = %02X\n", pBlock->nop.c);
			eprintf("pSCB->cmdStat.bits.cuc: %x\n", pSCB->cmdStat.bits.cuc);
			eprintf("pBlock->nop.c: %x\n", pBlock->nop.c);
			eprintf("CU command       = 0x%02X\n", cuc);
			eprintf("RU command       = 0x%02X\n", ruc);
			eprintf("SCB Gen Ptr      = 0x%X\n", scb_general_ptr);
			eprintf("scb status       = 0x%04X\n", pSCB->cmdStat.words.status);
			eprintf("scb command      = 0x%04X\n", pSCB->cmdStat.words.command);
		}
	}

#ifdef ETHERNET_DEBUG
	/* DEBUG */
	eprintf("scb command = 0x%04X\n", pSCB->cmdStat.words.command);
	eprintf("scb status  = 0x%04X\n", pSCB->cmdStat.words.status);
#endif

	pciIntConnect(CPLD_NIC_INT, (UINT32)i559IntHandler, 0);
	return;
}


/*******************
	i559IntHandler
********************/
int i559IntHandler(int arg)
{
	register CMD_STAT_U temp_cmdStat;
	volatile int rxFlag = FALSE;

//	Set_LED("NICi");

    temp_cmdStat.words.status = pSCB->cmdStat.words.status;

    /* check to see if it was the PRO/100 */
    if (temp_cmdStat.words.status & I559_INT)
    {
		/* Wait for command word to clear - indicates no pending commands */
		while (pSCB->cmdStat.words.command)
			;

		/* Update global status variable */
		i559Status = temp_cmdStat.words.status;

#ifdef ETHERNET_DEBUG
		eprintf ("i559IntHandler: i559Status = 0x%04x\n", i557Status);
#endif

		/* If the interrupt was due to a received frame... */
		if (temp_cmdStat.bits.fr)
		    rxFlag = TRUE;

		temp_cmdStat.words.status = temp_cmdStat.words.status & I559_INT;

		/* Acknowledge interrupt by setting ack bits */
		pSCB->cmdStat.words.status = temp_cmdStat.words.status;

		/* Wait for command word to clear - indicates IACK accepted */
		while (pSCB->cmdStat.words.command)
			;

#ifdef ETHERNET_DEBUG
		/* DEBUG */
		eprintf ("give waitSem\n");
#endif

		/* Update global status variable and unblock task */
		waitSem = 1;

		if (rxFlag == TRUE)
		{
			rxSem = 1;
			if(waiting_for_packet)
			{
				if (checkPacket (pCmdBlock->transmit.txData, pRfd->rxData, ETHERMTU) == ERROR)
				{
					eprintf("\nIntegrity of data received FAIL!!!\n");
				}
				else
				{
					hal_delay_us(1);
				}

				sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));
				if (pCmdBlock->transmit.ok != 1)
				{	
					eprintf ("tx FAIL!  Status: 0x%04X.\n",	pSCB->cmdStat.words.status);
				}
				sendCommand (CU_NOP, RU_ABORT, 0);
			}
		}
	}

	return(0);			/* nothing serviced - return 0 */
}

/*******************
	i559SelfTest
********************/
static int i559SelfTest(void)
{
	volatile struct selfTest *pSelfTestMem;
	UINT32 oldWord2;
	long delay;
	UINT32 temp;
	int	rtnVal;

	/* Allocate some memory for the self test */
	pSelfTestMem = (struct selfTest *) malloc (sizeof(struct selfTest));

	if (pSelfTestMem == NULL)
	{
		eprintf ("Couldn't get memory for self test.\n");
		return (ERROR);
	}

	if (verbose)
	{
		eprintf ("Sending PORT* self-test command...\n");
		eprintf ("Local Dump address = 0x%X\n", pSelfTestMem);
	}

	/* Set all bits in  second word, wait until it changes or a timeout */
	pSelfTestMem->u.word2 = ~0;
	oldWord2 = pSelfTestMem->u.word2;

	temp = (VIRT_TO_BUS(pSelfTestMem)) | PORT_SELF_TEST;

	#ifdef __ARMEB__
		*PORT_REG(adapter) = SWAP32(temp);
	#else
		*PORT_REG(adapter) = temp;
	#endif

	/* Wait for test completion or for timeout */
	for (delay = 0; (delay < MAX_DELAY) && (pSelfTestMem->u.word2 == oldWord2); delay++);	/* Wait... */

	/* Print results */
	if (verbose || (pSelfTestMem->u.word2 == oldWord2))
	{
		eprintf ("Self test result: %s\n", (pSelfTestMem->u.bits.selfTest) ? "Fail" : "Pass");
		eprintf ("ROM content test: %s\n", (pSelfTestMem->u.bits.romTest)  ? "Fail" : "Pass");
		eprintf ("Register test:    %s\n", (pSelfTestMem->u.bits.regTest)  ? "Fail" : "Pass");
		eprintf ("Diagnose test:    %s\n", (pSelfTestMem->u.bits.diagnTest)? "Fail" : "Pass");
		#ifdef __ARMEB__
			eprintf ("ROM signature:    0x%x\n", SWAP32(pSelfTestMem->romSig));
		#else
			eprintf ("ROM signature:    0x%x\n", pSelfTestMem->romSig);
		#endif
	}

	rtnVal = pSelfTestMem->u.bits.selfTest ? ERROR : OK;

	return (rtnVal);
}

/*******************
	i559Init
********************/
static int i559Init(void)
{
	/* Get memory for system data structures */
	if( ((pCmdBlock = (union cmdBlock *) malloc (sizeof(union cmdBlock))) == NULL) ||
		((pRfd = (struct rfd *) malloc (sizeof(struct rfd))) == NULL) || 
		((pPacketBuf = (char *) malloc (ETHERMTU + sizeof(UINT16) + sizeof(D_MAC_address))) == NULL))
	{
		eprintf ("Memory allocation FAIL!\n");
		return (ERROR);
	}
	pPacketBuf_base = pPacketBuf;
	if(verbose)
	{
		eprintf("pCmdBlock: 0x%x, size: 0x%x\n", pCmdBlock, sizeof(union cmdBlock));
		eprintf("pRfd: 0x%x, size: 0x%x\n", pRfd, sizeof(struct rfd));
		eprintf("pPacketBuf: 0x%x, size: 0x%x\n", pPacketBuf,
				(ETHERMTU + sizeof(UINT16) + sizeof(D_MAC_address)));
	}

	/* Set EL bits in command block and rfd so we don't fall of the end */
	pCmdBlock->nop.el = END_OF_LIST;
	pRfd->el = END_OF_LIST;

	/* Initialize */
	if(verbose)
		eprintf ("Initializing... ");

	/* set up the CU and RU base values to 0x0 */
	sendCommand (LOAD_CU_BASE, RU_NOP, 0);
	sendCommand (CU_NOP, LOAD_RU_BASE, 0);
	
	/* Initialize interrupts */
	pciIntConnect(CPLD_NIC_INT, (UINT32)i559IntHandler, 0);

	if(verbose)
		eprintf ("Done\n");

	return (OK);
}

/*******************
	i559AddrSet
********************/
static int i559AddrSet(void)
{
	if(verbose)
	{
		eprintf ("Setting hardware ethernet address to ");
	    eprintf ("%02X:%02X:%02X:", S_MAC_address[0], S_MAC_address[1], S_MAC_address[2]);
	    eprintf ("%02X:%02X:%02X... ", S_MAC_address[3], S_MAC_address[4], S_MAC_address[5]);
	}

	/* Set up iaSetup command block and execute */
	bzero ((char *) pCmdBlock, sizeof(union cmdBlock));
	pCmdBlock->iaSetup.code = IA_SETUP;
	pCmdBlock->iaSetup.el = END_OF_LIST;
	bcopy ((UINT32*)S_MAC_address, (UINT32*)pCmdBlock->iaSetup.enetAddr, sizeof(S_MAC_address));

	sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

	if ((waitForInt() == ERROR) || (pCmdBlock->iaSetup.ok != 1))
	{
		if(!waiting_for_packet)
		{
			eprintf ("FAIL! Status: 0x%04X.\n", pSCB->cmdStat.words.status);
			eprintf ("C bit = %d\n",pCmdBlock->iaSetup.c);
			eprintf ("OK bit = %d\n",pCmdBlock->iaSetup.ok);
		}
		return (ERROR);
	}
    if(verbose)
		eprintf ("done.\n");

	return (OK);
}

/*******************
	i559Config
********************/
static int i559Config(UINT8 loopBackMode)
{
	if(verbose)
		eprintf ("\nConfiguring for ");
	switch (loopBackMode)
	{
		case INT_LOOP_BACK:
			if(verbose)
				eprintf ("internal loopback... ");
			break;
		case EXT_LOOP_BACK:
			if(verbose)
				eprintf ("external loopback, LPBK* active... ");
			break;
		case NO_LOOP_BACK:
			if(verbose)
				eprintf ("no loopback... ");
			break;
		default:
			eprintf ("Unknown loopback mode, exiting...\n");
			return (ERROR);
	}

	/* Set up configure command block and execute */
	bzero ((char *) pCmdBlock, sizeof(union cmdBlock));
	pCmdBlock->configure.code = CONFIGURE;
	pCmdBlock->configure.el = END_OF_LIST;
	pCmdBlock->configure.configData[ 0] = CONFIG_BYTE_00;
	pCmdBlock->configure.configData[ 1] = CONFIG_BYTE_01;
	pCmdBlock->configure.configData[ 2] = CONFIG_BYTE_02;
	pCmdBlock->configure.configData[ 3] = CONFIG_BYTE_03;
	pCmdBlock->configure.configData[ 4] = CONFIG_BYTE_04;
	pCmdBlock->configure.configData[ 5] = CONFIG_BYTE_05;
	pCmdBlock->configure.configData[ 6] = CONFIG_BYTE_06;
	pCmdBlock->configure.configData[ 7] = CONFIG_BYTE_07;
	pCmdBlock->configure.configData[ 8] = CONFIG_BYTE_08;
	pCmdBlock->configure.configData[ 9] = CONFIG_BYTE_09;
	pCmdBlock->configure.configData[10] = CONFIG_BYTE_10 | loopBackMode;
	pCmdBlock->configure.configData[11] = CONFIG_BYTE_11;
	pCmdBlock->configure.configData[12] = CONFIG_BYTE_12;
	pCmdBlock->configure.configData[13] = CONFIG_BYTE_13;
	pCmdBlock->configure.configData[14] = CONFIG_BYTE_14;
	pCmdBlock->configure.configData[15] = CONFIG_BYTE_15;
	pCmdBlock->configure.configData[16] = CONFIG_BYTE_16;
	pCmdBlock->configure.configData[17] = CONFIG_BYTE_17;
	pCmdBlock->configure.configData[18] = CONFIG_BYTE_18;

	if (link_speed == SPEED_100M)
		pCmdBlock->configure.configData[19] = CONFIG_BYTE_19_100T;
	else
	{
		pCmdBlock->configure.configData[19] = CONFIG_BYTE_19_10T;
		pCmdBlock->configure.configData[20] = CONFIG_BYTE_20;
		pCmdBlock->configure.configData[21] = CONFIG_BYTE_21;
	}

	sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

	if ((waitForInt() == ERROR) || (pCmdBlock->configure.ok != 1))
	{
		if(!waiting_for_packet)
			eprintf ("FAIL! Status: 0x%04X.\n", pSCB->cmdStat.words.status);
		return (ERROR);
	}

	initPHY (phy_id, loopBackMode);  /* set up the PHY interface appropriately */

	if(verbose)
		eprintf ("done.\n");

	return (OK);
}

/*******************
	i559RUStart
********************/
int i559RUStart(void)
{
	volatile long delay;

#ifdef ETHERNET_DEBUG
	eprintf ("Enabling receiver... ");
#endif

//scrub receiver buffer
//bzero ((char *) pRfd, sizeof(struct rfd));

	/* Set end-of-list bit in the rfd so we don't fall off the end */
	pRfd->el = END_OF_LIST;
	pRfd->s = 1;
	pRfd->sf = 0;				/* Simplified mode */
	pRfd->rbdAddr = (UINT8 *) 0xffffffff; 	/* No RBD */
	/* buffer size: */
	pRfd->size = sizeof (pRfd->rxData) + sizeof (pRfd->destAddr) + sizeof (pRfd->sourceAddr) + sizeof (pRfd->length); 

	sendCommand (CU_NOP, RU_START, VIRT_TO_BUS(pRfd));
	/*
	 * Poll, can't use waitForInt (), as this step doesn't generate interrupts.
	 */

	i559Status = 0;

	/* Wait for timeout (i557Status changes) or RU_STAT is RU_READY */
	for (delay = 0; (delay < MAX_DELAY) && (pSCB->cmdStat.bits.rus != RU_READY); delay++)
		;	/* Wait... */
	if (pSCB->cmdStat.bits.rus != RU_READY)
	{
		eprintf ("FAIL! in RUStart. Status: 0x%04X.\n",
		pSCB->cmdStat.words.status);
		return (ERROR);
	}
#ifdef ETHERNET_DEBUG
	eprintf ("done.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);
#endif

	return (OK);
}

/***********************
	setup_tx_cmd_blk
************************/
void setup_tx_cmd_blk(void)
{
	UINT16 tcbCount;

	/* Set up transmit command block and execute */
	bzero ((char *) pCmdBlock, sizeof(union cmdBlock));
	pCmdBlock->transmit.code = TRANSMIT;
	pCmdBlock->transmit.el = END_OF_LIST;
	pCmdBlock->transmit.sf = 0;				/* Simplified mode */
	pCmdBlock->transmit.tbdAddr = (UINT8 *) 0xffffffff; /* No TBD */
	
	#ifdef __ARMEB__
		pCmdBlock->transmit.eof = 1;			/* Entire frame here */
		/* # bytes to tx: */
		tcbCount =	sizeof (pCmdBlock->transmit.destAddr) +
										sizeof (pCmdBlock->transmit.length) +
										sizeof (pCmdBlock->transmit.txData);
		pCmdBlock->transmit.tcbCount1 = tcbCount & 0xFF;
		pCmdBlock->transmit.tcbCount2 = (tcbCount >> 8) & 0x3F;
	#else
		pCmdBlock->transmit.eof = 1;			/* Entire frame here */
		/* # bytes to tx: */
		pCmdBlock->transmit.tcbCount =	sizeof (pCmdBlock->transmit.destAddr) +
										sizeof (pCmdBlock->transmit.length) +
										sizeof (pCmdBlock->transmit.txData);
	#endif

	bcopy ((UINT32*)pPacketBuf, (UINT32*)pCmdBlock->transmit.destAddr, sizeof(D_MAC_address) + sizeof(UINT16) + ETHERMTU);
}

/*******************
	txPacket
********************/
static int txPacket(char *pBuf)
{
	int status = OK;
	int link_status =1;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	setup_tx_cmd_blk();

	rxSem = 0;	/* no Receive interrupt */

	sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

	link_status = get_niclink_status();
	if (waitForInt() == ERROR)
	{
		if(!link_status)//vgd
		      status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("No Transmit Interrupt\n");
			status = ERROR;
		}
	}

	if (pCmdBlock->transmit.ok != 1)
	{	
		//vgd
		if(!link_status)
		      status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("tx FAIL!  Status: 0x%04X.\n",
			pSCB->cmdStat.words.status);
			status = ERROR;
		}
	}

	if (status == ERROR)
	{
		//vgd
		if(!link_status)
		      status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("Transmit OK = %d\n", pCmdBlock->transmit.ok);
			return (ERROR);
		}
	}

	if (waitForRxInt() == ERROR)
	{
		//vgd
		if(!link_status)
		      status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("No Receive Interrupt\n");
			status = ERROR;
		}
	}

	if (pRfd->ok != 1)
	{
		//vgd
		if(!link_status)
		      status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("rx FAIL! Status: 0x%04X.\n", pSCB->cmdStat.words.status);
			status = ERROR;
		}
	}

	/* If RU still ready, hang for receive interrupt */
	if (pSCB->cmdStat.bits.rus == RU_READY)
	{
		
		if (waitForRxInt() == ERROR)
		{
			//vgd
			if(!link_status)
		      status = LINK_DN;
			else
			{
				if(acL->Verboselevel !=0)
					eprintf ("No Receive Interrupt\n");
				status = ERROR;
			}
		}

		if (pRfd->ok != 1)
		{
			//vgd
			if(!link_status)
		      status = LINK_DN;
			else
			{
				if(acL->Verboselevel !=0)
					eprintf ("rx FAIL! Status: 0x%04X.\n", pSCB->cmdStat.words.status);
				status = ERROR;
			}
		}
	}

	if (status == ERROR) 
	{
#ifdef ETHERNET_DEBUG
		if(acL->Verboselevel !=0)
		{
			eprintf ("\nTransmit Stats:\n");
			eprintf ("---------------\n");
			eprintf ("Transmit OK = %d\n", pCmdBlock->transmit.ok);

			eprintf ("\nReceive Stats:\n");
			eprintf ("---------------\n\n");
			eprintf ("Receive OK = %d\n", pRfd->ok);
			eprintf ("CRC Error = %d\n", pRfd->crcErr);
			eprintf ("Alignment Error = %d\n", pRfd->alignErr);
			eprintf ("Resource Error = %d\n", pRfd->noRsrc);
			eprintf ("DMA Overrun Error = %d\n", pRfd->dmaOverrun);
			eprintf ("Frame Too Short Error = %d\n", pRfd->frameTooshort);
			eprintf ("Receive Collision Error = %d\n", pRfd->rxColl);
		}
#endif
		return (ERROR);
	}

#ifdef ETHERNET_DEBUG
	eprintf ("Packet Actual Size = %d\n", pRfd->actCount);
#endif

	if (checkPacket (pCmdBlock->transmit.txData, pRfd->rxData, ETHERMTU) == ERROR)
	{
		if(!link_status) /*vgd*/
		   status = LINK_DN;
		else
		{
			if(acL->Verboselevel !=0)
				eprintf ("data verify error.\n");
			return (ERROR);
		}
	}
	else
	{
		if (verbose)
			eprintf ("data OK.\n");
	}

	return (OK);
}

/*******************
	setUpPacket
********************/
static void setUpPacket(char *pBuf)
{
	bcopy ((UINT32*)D_MAC_address, (UINT32*)pBuf, sizeof(D_MAC_address));
	pBuf += sizeof(D_MAC_address); /* skip dest. address */

	*((UINT16 *) pBuf) = ETHERMTU;
	pBuf += sizeof(UINT16);      /* skip length field */
	makePacket ((UINT8*)pBuf, ETHERMTU);
}

/****************
	makePacket
*****************/
static void makePacket (UINT8 *pPacket, int length)
{
	int byteNum;	/* Current byte number */

	for (byteNum = 0; byteNum < length; byteNum++)
		*pPacket++ = byteNum + ' ';
}

/*
 * Verify contents of a received packet to what was transmitted.
 * Returns OK if they match, ERROR if not.
 */
static int checkPacket (UINT8 *pTxBuffer,		/* Pointer data that was transmitted */
						UINT8 *pRxBuffer,		/* Pointer data that was received */
						int	length)				/* How many bytes to check */
{
	int byteNum;	/* Current byte number */

#ifdef ETHERNET_DEBUG
     eprintf ("\n");
     eprintf ("Transmit Buffer at 0x%08X\n", pTxBuffer);
     eprintf ("Receive  Buffer at 0x%08X\n", pRxBuffer);
#endif
     for (byteNum = 0; byteNum < length; byteNum++)
     {
		if (*pTxBuffer++ != *pRxBuffer++)
		{
			#ifdef ETHERNET_DEBUG
			eprintf("Error at byte 0x%x\n", byteNum);
			eprintf("Expected 0x%02X, got 0x%02X\n", *(pTxBuffer - 1), *(pRxBuffer - 1));
			#endif
            return (ERROR);
		}
     }
     return (OK);
}

void display_loopback_init_error(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf ("NIC Loopback Initialization... FAIL!\n");
	Set_LED("NliF");
	acL->Err = ERROR;
	slowport_code[0] = SP_NIC;
	slowport_code[1] = SP_FAIL;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
	// prepare structure for dumping
	syslog.type = TYPE_ERROR;
	syslog.source_comp = MASTER_DIAG;	// Determine master/slave
	strcpy(syslog.desc, "NIC Loopback Initialization FAIL!");

	// write to syslog
	syslog_dump(&syslog, sizeof(SYSLOG_DATA));		
}

void display_internal_loopback_error(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf ("NIC Internal Loopback Test... FAIL!\n");
	Set_LED("Ni F");
	acL->Err = ERROR;
	slowport_code[0] = SP_NIC;
	slowport_code[1] = SP_FAIL;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
	// prepare structure for dumping
	syslog.type = TYPE_ERROR;
	syslog.source_comp = MASTER_DIAG;	// Determine master/slave
	strcpy(syslog.desc, "NIC Internal Loopback Test FAIL!");

	// write to syslog
	syslog_dump(&syslog, sizeof(SYSLOG_DATA));		
}

void display_external_error(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	eprintf ("NIC External Loopback Test... FAIL!\n");
	Set_LED("Ne F");
	acL->Err = ERROR;
	slowport_code[0] = SP_NIC;
	slowport_code[1] = SP_FAIL;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
	// prepare structure for dumping
	syslog.type = TYPE_ERROR;
	syslog.source_comp = MASTER_DIAG;	// Determine master/slave
	strcpy(syslog.desc, "NIC External Loopback Test FAIL!");

	// write to syslog
	syslog_dump(&syslog, sizeof(SYSLOG_DATA));
}

/*
 * Take the semaphore and block until i557 interrupt or timeout.
 * Returns OK if an interrupt occured, ERROR if a timeout.
 */
static int waitForInt(void)
{
	int num_us = 0; 

	while ((waitSem == 0) && (num_us != 2000000)) /* wait max 2secs for the interrupt */
	{	
		hal_delay_us(1);
		num_us++;
	}

    if (!waitSem)
    {
		if(!waiting_for_packet)
			eprintf("Wait error!\n"); 
		return (ERROR);
    }
    else
		return (OK);
}

/*******************
	waitForRxInt
********************/
static int waitForRxInt(void)
{
	int num_us = 2000; /*vgd 2000000;*/
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	while ((rxSem == 0) && (num_us--))	/* wait max 2secs for the interrupt */
	{	
		hal_delay_us(1);
	}

    if (!rxSem)
    {
		if(!( readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT) & MDI_STAT_LINK)) /*vgd*/
			return LINK_DN;
		else
		if(acL->Verboselevel !=0)
			eprintf("Rx Wait error!\n"); 
		return (ERROR);
    }
    else
		return (OK);
}


/*******************
	initPHY
********************/
static int initPHY(UINT32 device_type, int loop_mode)
{
//	UINT16 temp_reg;
	UINT8 revision;

	/* strip off revision and phy. id information */ 
	revision = (UINT8)(device_type & REVISION_MASK);
	device_type &= ~REVISION_MASK;

    switch (device_type)
	{
		case I82553_PHY_ID:
		case I82553_REVAB_PHY_ID:
		case I82555_PHY_ID:
			break;

		default:
			return (ERROR);
			break;
  }

  return (OK);
}

/*
 * "Poor Man's Malloc" - return a pointer to a block of memory at least
 * The block returned will have been zeroed.
 */
static char *malloc (int size) /* number of bytes needed */
{
	char *p_memory;
    UINT32 _size = 0;

    p_memory = NULL;
    size = (size + 0x10) & ~0xF;	// round to 16 bytes
    if ( (i82559_heap_free+size) < (i82559_heap_base+i82559_heap_size) )
	{
        UINT32 *p;
        p_memory = (char *)i82559_heap_free;
        i82559_heap_free += size;
        for(p = (UINT32 *)p_memory; _size < size; _size += 4 )
            *p++ = 0;
    }
    return p_memory;
}

/*******************
	readMDI
********************/
static UINT16 readMDI(UINT8 phyAdd, UINT8 regAdd)//, UINT16 data)
{
	register MDI_CONTROL_U mdiCtrl;
	int num_ms = 0; 

	/* prepare for the MDI operation */
	mdiCtrl.bits.ready = MDI_NOT_READY;
	mdiCtrl.bits.intEnab = MDI_POLLED;	/* no interrupts */
	mdiCtrl.bits.op = MDI_READ_OP;
	mdiCtrl.bits.phyAdd = phyAdd & 0x1f;
	mdiCtrl.bits.regAdd = regAdd & 0x1f;

	/* start the operation */
	#ifdef __ARMEB__
		*MDI_CTL_REG(adapter) = SWAP32(mdiCtrl.word);
	#else
		*MDI_CTL_REG(adapter) = mdiCtrl.word;
	#endif

	/* delay a bit */
	hal_delay_us(1000);

	/* poll for completion */
	#ifdef __ARMEB__
		mdiCtrl.word = SWAP32(*MDI_CTL_REG(adapter));
	#else
		mdiCtrl.word = *MDI_CTL_REG(adapter);
	#endif

	while ((mdiCtrl.bits.ready == MDI_NOT_READY) && (num_ms != 2000)) /* wait max 2secs */
	{	
		#ifdef __ARMEB__
			mdiCtrl.word = SWAP32(*MDI_CTL_REG(adapter));
		#else
			mdiCtrl.word = *MDI_CTL_REG(adapter);
		#endif
		hal_delay_us(1000);
		num_ms++;
	}	
	
	if (num_ms >= 2000)
	{
		eprintf ("readMDI Timeout!\n");
		return (0);
	}
	else 
		return ((UINT16)mdiCtrl.bits.data);
}



static int GetEepromSize(long ioaddr)
{
    unsigned short retval = 0;
    int ee_addr = ioaddr + SCBeeprom;
    int i, addrbits;

    // Should already be not-selected, but anyway:
    PUT16(ee_addr^3, EEPROM_ENB & ~EEPROM_CS);
    hal_delay_us(10);
    PUT16(ee_addr^3, EEPROM_ENB);
    hal_delay_us(10);
    
    // Shift the read command bits out.
    for (i = 2; i >= 0; i--)
	{
        short dataval = (6 & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT16(ee_addr^3, EEPROM_ENB | dataval);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB | dataval | EEPROM_SHIFT_CLK);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB | dataval);
        hal_delay_us(2);
    }
    // Now clock out address zero, looking for the dummy 0 data bit
    for ( i = 1; i <= 12; i++ )
	{
        PUT16(ee_addr^3, EEPROM_ENB);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB | EEPROM_SHIFT_CLK);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB);
        hal_delay_us(2);
        retval = GET16(ee_addr^3) & EEPROM_DATA_READ;
        if ( 0 == retval )
        {
            break; // The dummy zero est arrive'
        }
    }

    addrbits = i;
    if ( 6 != i && 8 != i )
	{
        i = 6;
        addrbits = 0;  // So caller can know if the EEPROM is missing
    }

    // clear the dataval, leave the clock low to read in the data regardless
    PUT16(ee_addr^3, EEPROM_ENB);
    hal_delay_us(1);
    
    retval = GET16(ee_addr^3);
    if ( (EEPROM_DATA_READ & retval) != 0 )
	{
    }
    hal_delay_us(1);
    
    for (i = 15; i >= 0; i--)
	{
        PUT16(ee_addr^3, EEPROM_ENB | EEPROM_SHIFT_CLK);
        hal_delay_us(2);
        retval = GET16(ee_addr^3);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB);
        hal_delay_us(2);
    }
    
    // Terminate the EEPROM access.
    PUT16(ee_addr^3, EEPROM_ENB & ~EEPROM_CS);
    hal_delay_us(100);
    
    return addrbits;
}

static int Eeprom_Read(long ioaddr, int location, int addr_len)
{
    unsigned short retval = 0;
    int ee_addr = ioaddr + SCBeeprom;
    int read_cmd = location | (6 << addr_len);
    int i, tries = 10;

 	while(1)
	{
		// Should already be not-selected, but anyway:
		PUT16(ee_addr^3, EEPROM_ENB & ~EEPROM_CS);
		hal_delay_us(10);
		PUT16(ee_addr^3, EEPROM_ENB);
		hal_delay_us(10);

		// Shift the read command bits out, changing only one bit per time.
		for (i = addr_len + 2; i >= 0; i--)
		{
		    short dataval = (read_cmd & (1 << i)) ? EEPROM_DATA_WRITE : 0;
		    PUT16(ee_addr^3, EEPROM_ENB | dataval);
		    hal_delay_us(2);
		    PUT16(ee_addr^3, EEPROM_ENB | dataval | EEPROM_SHIFT_CLK);
		    hal_delay_us(2);
		    PUT16(ee_addr^3, EEPROM_ENB | dataval);
		    hal_delay_us(2);
		}

		// clear the dataval, leave the clock low
		PUT16(ee_addr^3, EEPROM_ENB);
		hal_delay_us(1);

		retval = GET16(ee_addr^3);
		// This should show a zero in the data read bit to confirm that the
		// address transfer is compelete.  If not, go to the start and try
		// again!
		if ( (0 != (retval & EEPROM_DATA_READ)) && (tries-- > 0) )
		{
		// Terminate the EEPROM access.
		    PUT16(ee_addr^3, EEPROM_ENB & ~EEPROM_CS);
		    hal_delay_us(100);
		}
		else
			break;
	}

    if ( (EEPROM_DATA_READ & retval) != 0 )
	{
    }
    hal_delay_us(1);
    retval = 0;

    for (i = 15; i >= 0; i--)
	{
        PUT16(ee_addr^3, EEPROM_ENB | EEPROM_SHIFT_CLK);
        hal_delay_us(2);
        retval = (retval << 1) | ((GET16(ee_addr^3) & EEPROM_DATA_READ) ? 1 : 0);
        hal_delay_us(2);
        PUT16(ee_addr^3, EEPROM_ENB);
        hal_delay_us(2);
    }
    
    // Terminate the EEPROM access.
    PUT16(ee_addr^3, EEPROM_ENB & ~EEPROM_CS);
    hal_delay_us(100);
    
    return retval;
}


void Get_Eeprom_Mac_Addr(void)
{
	int addr_length = 8, i, count;
	UINT32 ioaddr;
	UINT16 checksum, value;

	ioaddr = nic_dev_info.base_map[1];

	*(PCI_ADDR_EXT_REG) = (nic_dev_info.base_address[1] & PCI_IO_PAGE_MASK);

	// read eeprom and get 82559's mac address
    addr_length = GetEepromSize(ioaddr);

	// (this is the length of the *EEPROM*s address, not MAC address)
    for (checksum = 0, i = 0, count = 0; count < 64; count++)
	{
        
        // read word from eeprom
        value = Eeprom_Read(ioaddr, count, addr_length);

        checksum += value;
        if (count < 3)
		{
            EEPROM_MAC_address[i++] = value & 0xFF;
            EEPROM_MAC_address[i++] = (value >> 8) & 0xFF;
        }
	}
	eprintf("EEPROM MAC Address: 0x");
	for (i=0; i<6; i++)
    {
        eprintf("%02X", EEPROM_MAC_address[i]);
    }
    eprintf("\n");
	*(PCI_ADDR_EXT_REG) = (nic_dev_info.base_address[0] & PCI_MEM_PAGE_MASK) >> 16;
}

static void signal_next_npu_to_rx(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)
	{
		*(PCI_ADDR_EXT_REG) =
			(slave_dev_info.base_address[CSR_BAR] & PCI_MEM_PAGE_MASK) >> 16;
		Set_Pci_Doorbell(NIC_RECV_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
	}
	else	// slave
	{
		*(PCI_ADDR_EXT_REG) =
			(master_dev_info.base_address[CSR_BAR] & PCI_MEM_PAGE_MASK) >> 16;
		Set_Pci_Doorbell(NIC_RECV_BIT, (master_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
	}

	*(PCI_ADDR_EXT_REG) = (nic_dev_info.base_address[0] & PCI_MEM_PAGE_MASK) >> 16;
	hal_delay_us(100);
}


int emi_txpacket(void)
{
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// handler: hal_delay_us(100)
	setUpPacket (pPacketBuf);	// estimated at most 10us
	signal_next_npu_to_rx();	// hal_delay_us(100)
    if ((i559RUStart() == ERROR) || (txPacket(pPacketBuf) == ERROR))
    {
        if(acL->HaltOnError)
		    pciIntDisconnect(CPLD_NIC_INT);
		return ERROR;
	}

	return OK;
}

int get_niclink_status()
{
	UINT16 temp1;
	
	temp1 = readMDI (MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT);
	if(temp1 & MDI_STAT_LINK)
		return 1;
	else
		return 0;
}
