/* flash_test.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Apr 24, 02
 *  revisions: dalsraja, July 26, 2002
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "misc_func.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "i2c.h"
#include "i2c_test.h"
#include "led.h"
#include "syslog.h"
#include "flash_io.h"


// static function prototypes
static void Flash_Test_Help(void);
void Flash_Lock_Test(void);

//
// Function: Flash_Test_Help
//
// Description: Displays all the options available for the Flash test
//				on the console.
//
static void Flash_Test_Help()
{
	eprintf("Diag> t flash <option>, where,\n");
	eprintf("<option> are\n");
	eprintf("l - Flash Lock Test\n");
	eprintf("h - help\n");
}

//
// Function: Flash_Test
//
// Description: Calls the appropriate function(s) of the Flash test
//				depending on user input.
//
void Flash_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	opt = acL->argv[2][0];

	switch(opt)
	{
		case 'l': 	Flash_Lock_Test();
					break;
		case 'h':
		default: 	Flash_Test_Help();
	}

	if(acL->Err == ERROR)
		eprintf("FLASH TEST FAIL!!!\n");
}

//
// Function: Flash_Lock_Test
//
// Description: Locks the Entry point of syslog and then tries to write to it.
//				Unlocks if lock is successful. Returns error if unable to lock
//				or able to program after lock.
//
void Flash_Lock_Test()
{
	void *err_addr;
	char buf[16] = "FLASH_LOCK_TEST";
	SYSLOG_INFO x;
	int stat;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	compute_last_logentry_ptr(&x);

	stat = flash_lock((void*)x.last_entry, 64, (void*)&err_addr);
	if(stat != FLASH_ERR_OK)
	{
		eprintf(flash_errmsg(stat));
	}
	else
	{
		stat = flash_program((void*)x.last_entry, (char *)buf,  64, (void*)&err_addr);
		if(stat != FLASH_ERR_OK)
		{
			eprintf("Flash Lock Test... PASS!\n");
			eprintf(flash_errmsg(stat));
			Set_LED("Fl P");
			slowport_code[0] = SP_FLSH;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			flash_unlock((void*)x.last_entry, 64, (void*)&err_addr);
		}
		else
		{
			eprintf("Flash Lock Test... FAIL!\n");
			Set_LED("Fl F");
			slowport_code[0] = SP_FLSH;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Flash Lock Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));		
			acL->Err = ERROR;
		}
	}
}

