/* led.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 15, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "misc_func.h"
#include "diagstruct.h"
#include "led.h"
#include "uart.h"
#include "diag_utils.h"
#include "hal_ixdp2400.h"
#include "syslog.h"

//static function prototypes
static void LED_Help(void);
#ifdef BLINK_CAPABLE
static void Blinking_Cursor_Test(void);
static void _LED_Blink(int ms);
#endif
static void Small_Char_LED_Test(void);
static void Big_Char_LED_Test(void);
static void Numerical_Char_LED_Test(void);

//
// Function: LED_Help
//
// Description: Displays all the options available for the LED test
//				on the console
//
static void LED_Help(void)
{
	eprintf("> t LED <option>, where\n");
	eprintf("<option> are:\n");
	eprintf("s - LED small caps char test\n");
	eprintf("b - LED big caps char test\n");
	eprintf("n - LED numerical char test\n");
//	eprintf("c - LED blinking cursor test\n");
	eprintf("r - Set LED test <4-alpha-numerical chars>\n");
//	eprintf("i - Init LED\n");
	eprintf("a - All LED tests\n");
	eprintf("h - LED help message\n");
}

//
// Function: LED_Test
//
// Description: Calls the appropriate function(s) of the LED test
//				depending on user input.
//
void LED_Test(void)
{
	char opt;
	char* pattern;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	
	opt = acL->argv[2][0];
	pattern = acL->argv[3];

	eprintf("LED Test started...\n");
	
	switch(opt)
	{
		//case 'c':	Blinking_Cursor_Test();
		//			break;

		case 's': 	Small_Char_LED_Test();
					break;

		case 'b': 	Big_Char_LED_Test();
					break;		

		case 'n': 	Numerical_Char_LED_Test();
					break;

		case 'r': 	if(acL->argc < 4)
					{
						eprintf("***WARNING***: Pattern argument missing. Pattern \"test\" used as default\n");
						strcpy(pattern, "test");
					}
					eprintf("Setting LED with %s...\n", pattern);
					Set_LED(pattern);
					slowport_code[0] = SP_LED;
					slowport_code[1] = SP_DONE;
					dump_slowport(slowport_code, 2, SP_NON_FATAL);
					eprintf("Setting LED done!\n");
					break;

//		case 'i':	LED_Init();
//					break;

		case 'a': 	//Blinking_Cursor_Test();
					Small_Char_LED_Test();
					Big_Char_LED_Test();
					Numerical_Char_LED_Test();
					if(acL->argc < 4)
					{
						eprintf("***WARNING***: Pattern argument missing. Pattern \"test\" used as default\n");
						strcpy(pattern, "test");
					}
					eprintf("Setting LED with %s...\n", pattern);
					Set_LED(pattern);
					slowport_code[0] = SP_LED;
					slowport_code[1] = SP_DONE;
					dump_slowport(slowport_code, 2, SP_NON_FATAL);
					eprintf("Setting LED done!\n");
					break;
		
		case 'h':
		default:	LED_Help();
	}
}

//
// Function: Blinking_Cursor_Test
//
// Description: Blinking a fully lighted LED 5 times.
//
#ifdef BLINK_CAPABLE
static void Blinking_Cursor_Test(void)
{
	eprintf("Blinking Cursor Test started...\n");

	// Blink cursor display 5 times
	LED_CR.bits.CUE = 1;	// Set IC to display Cursor
	PUT8(LED_DISPLAY_CTL_REG, LED_CR.all_bits);
	hal_delay_us(500000);
	_LED_Blink(500000);	// 1st Blink
	_LED_Blink(500000);	// 2nd Blink
	_LED_Blink(500000);	// 3rd Blink
	_LED_Blink(500000);	// 4th Blink
	_LED_Blink(500000);	// 5th Blink
	LED_CR.bits.CUE = 0;	// Set IC to display ASCII

	eprintf("Blinking Cursor Test Completed!\n");
	slowport_code[0] = SP_LED;
	slowport_code[1] = SP_DONE;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
}
#endif
//
// Function: Small_Char_LED_Test
//
// Description: Displays the chars a-z on the LED
//
static void Small_Char_LED_Test(void)
{
	int data;

	eprintf("Small Characters LED Test started...\n");

	
	for(data = ASCII_a; data <= ASCII_z; data++)
	{
		if(KEYPRESS) return;

		PUT8(ALPHANUM_DIS_DATA_REG + 0, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 1, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 2, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 3, data);
		
		hal_delay_us(500000);
	}

	eprintf("Small Characters LED Test completed!\n");
	slowport_code[0] = SP_LED;
	slowport_code[1] = SP_DONE;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
}

//
// Function: Big_Char_LED_Test
//
// Description: Displays the chars A-Z on the LED
//
static void Big_Char_LED_Test(void)
{
	int data;

	eprintf("Big Characters LED Test started...\n");

	for(data = ASCII_A; data <= ASCII_Z; data++)
	{
		if(KEYPRESS) return;

		PUT8(ALPHANUM_DIS_DATA_REG + 0, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 1, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 2, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 3, data);
		
		hal_delay_us(500000);
	}

	eprintf("Big Characters LED Test completed!\n");
	slowport_code[0] = SP_LED;
	slowport_code[1] = SP_DONE;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
}

//
// Function: Numerical_Char_LED_Test
//
// Description: Displays the numbers 0-9 on the LED
//
static void Numerical_Char_LED_Test(void)
{
	int data;

	eprintf("Numerical Characters LED Test started...\n");

	for(data = ASCII_0; data <= ASCII_9; data++)
	{
		if(KEYPRESS) return;

		PUT8(ALPHANUM_DIS_DATA_REG + 0, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 1, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 2, data);
		PUT8(ALPHANUM_DIS_DATA_REG + 3, data);
		
		hal_delay_us(500000);
	}

	eprintf("Numerical Characters LED Test completed!\n");
	slowport_code[0] = SP_LED;
	slowport_code[1] = SP_DONE;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
}

//
// Function: Set_Blinking_LED
//
// Description: Calls Set_LED but adds blinking capability
//
#ifdef BLINK_CAPABLE
void Set_Blinking_LED(char* buffer, int blink_times, int ms)
{
	int i;

	Set_LED(buffer);
	for(i = 0; i < blink_times; i++)
		_LED_Blink(ms);
}
#endif

//
// Function: LED_Blink
//
// Description: Enabling and disabling a blank display on the LED with
//				a delay in between the two operations.
//
#ifdef BLINK_CAPABLE
static void _LED_Blink(int us)
{
	LED_CR.bits.BL_ = 0;		// Blank Display
	PUT8(LED_DISPLAY_CTL_REG, LED_CR.all_bits);
	hal_delay_us(us);
	LED_CR.bits.BL_ = 1;		// Disable Blank Display
	PUT8(LED_DISPLAY_CTL_REG, LED_CR.all_bits);
	hal_delay_us(us);
}
#endif
