/* line_rate.c
 *
 * This line rate test is not a performance benchmark. It will be used to
 * verify whether the system is able to handle the data traffic that is
 * pumped to the media port at whatever line rate that is pumped into the
 * system. A packet generator will be required for this test.
 *
 * NOTE: THIS IS NOT A PERFORMANCE BENCHMARK!!!! IT IS JUST TO CHECK IF THE
 *       HARDWARE IS ABLE TO HANDLE DATA AT THE LINERATE THAT IS BEING
 *       PUMPED INTO THE SYSTEM
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, April, 2002
 *  revisions: dalsraja, November 11, 2002
 * 
 * 
 * --------------------------------------------------------------------------
 */

// define this to gather line rate stats.
//#define LINERATE_STAT

typedef struct  // Complete Data for a Chip
{
   unsigned short*   BaseAddress;	   // Base Address of chip
   unsigned long        ChipType;      // Type of Chip
   void*                pChipCfg;      // Pointer to Chip Specific Configuration
   void*                pAlarmCfg;     // Pointer to Chip Specific Alarm Config
   void*                funcPtr;       // Pointer to the chip's api func's
} bb_ChipData_t;

typedef unsigned long ulong;
typedef unsigned short ushort;

#include "line_rate.h"
#include "line_rate_ucode.h"
#include "common.h"
#include "diagstruct.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "led.h"
#include "uart.h"
#include "syslog.h"
#include "diag_utils.h"
#include "msf_init.h"
#include "mac_init.h"
#include "error_code.h"
#include "common_def.h"

static void Line_Rate_BD_Master(void);
static void Line_Rate_BD_Slave(void);
static void Line_Rate_BW_Master(void);
static void Line_Rate_BW_Slave(void);
static void Line_Rate_PL_Master(void);
static void Line_Rate_PL_Slave(void);
#ifdef LINERATE_STAT
void setup_pmu_for_msf(void);
static void Print_PL_Stat(void);
#include "hal.h"
#include "bdw.h"
static void Print_BW_Stat(void);
#endif

static void Line_Rate_Help(void);

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;
extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern const InitRegTable_t BD_ExtLoopbackArray[];
extern const InitRegTable_t BW_ExtLoopbackArray[];

extern ulong IxfApiInit(void *baseAddress, ulong chipType, bb_ChipData_t *pChipData);

extern ulong Ixf6048InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);
extern void spConfig(void);
extern void reg_write(volatile UINT32 offset, UINT32 val);
extern void spRestore(void);
extern void Save_N_Set_Sp_Vals(void);
extern void Restore_Sp_Vals(void);

extern pci_device slave_dev_info;		// for slave npu info.

#define LINE_RATE_BD_EG_RX_ME		0
#define LINE_RATE_BD_EG_TX_ME		16
#define LINE_RATE_BD_IN_RX_ME		0
#define LINE_RATE_BD_IN_TX_ME		16

#define LINE_RATE_BW_IN_RX_0_ME		0
#define LINE_RATE_BW_IN_RX_1_ME		1
#define LINE_RATE_BW_IN_RX_2_ME		2
#define LINE_RATE_BW_IN_RX_3_ME		3
#define LINE_RATE_BW_IN_TX_ME		16
#define LINE_RATE_BW_EG_RX_ME		16
#define LINE_RATE_BW_EG_TX_0_ME		0
#define LINE_RATE_BW_EG_TX_1_ME		1
#define LINE_RATE_BW_EG_TX_2_ME		2
#define LINE_RATE_BW_EG_TX_3_ME		3

#define DUPLEX_ADDR					0x2000

static void Line_Rate_Help(void)
{
	eprintf("Diag> linerate <media_type>\n");
	eprintf("where <media_type> are\n");
	eprintf("\td - Battery Drew (1 x OC-48)\n");
	eprintf("\tw - Battery Wallace (4 x OC-12)\n");
	eprintf("\tp <channel_opt> <speed> <duplex> - Point Lone (4 x OC-12)\n");
	eprintf("\th - Linerate Help\n");
}

void Line_Rate_Test(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT8 media_type;

	if(acL->HostType == MASTER)		// if master NPU
	{
		if (acL->argc < 3)
		{
			Line_Rate_Help();
			return;
		}

		media_type = acL->argv[2][0];

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;

#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF), SWAP32((UINT32)media_type));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF), (UINT32)media_type);
#endif
		switch(media_type)
		{
			case 'd':	Line_Rate_BD_Master();
						break;
			
			case 'w':	Line_Rate_BW_Master();
						break;
			
			case 'p':	Line_Rate_PL_Master();
						break;
#ifdef LINERATE_STAT
			case 's':	Print_PL_Stat();
						break;
			case 't':	Print_BW_Stat();
						break;
#endif

			default:	Line_Rate_Help();
		}
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
	}		
}


void Line_Rate_Test_Slave(void)
{
	UINT8 media_type;

//	media_type = (UINT8)(GET32(SCRATCHPAD_BASE + CARD_OPTION_ADDR) & 0xFF);
	media_type = (UINT8)GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);

	switch(media_type)
	{
		case 'd':	Line_Rate_BD_Slave();
					break;
		
		case 'w':	Line_Rate_BW_Slave();
					break;

		case 'p':	Line_Rate_PL_Slave();
					break;
		
		default:	eprintf("Unknown card\n");
	}
}

// This function is called by test manager when the user call for the line rate test
static void Line_Rate_BD_Master(void)
{
	int flag, status;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	bb_ChipData_t pc;

	flag = OK;

	if(acL->HostType == MASTER)		// if master NPU
	{
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		eprintf("Initialising media card. Please wait...\n");

		Save_N_Set_Sp_Vals();

		IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
		
		Restore_Sp_Vals();
		eprintf("Media card initialisation completed.\n");

		status = DownLoadUcode(LINE_RATE_BD_EG_RX_ME, sizeof(line_rate_bd_eg_rx), (unsigned int *)line_rate_bd_eg_rx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME0 FAIL!\n");
			flag = ERROR;
		}

		status = DownLoadUcode(LINE_RATE_BD_EG_TX_ME, sizeof(line_rate_bd_eg_tx), (unsigned int *)line_rate_bd_eg_tx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME16 FAIL!\n");
			flag = ERROR;
		}

		if (flag == ERROR)
		{
			eprintf("Line rate test could not be run\n");
			Set_LED("LR F");
			return;
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(LINERATE_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}
}


// This function will be called by the PCI Doorbell function, after the master initiates
// a PCI Doorbell interrupt on the slave NPU
static void Line_Rate_BD_Slave(void)
{
	int flag = OK, status;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	DownLoadUcode(0, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	status = DownLoadUcode(LINE_RATE_BD_IN_RX_ME, sizeof(line_rate_bd_in_rx), (UINT32 *)line_rate_bd_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME0 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BD_IN_TX_ME, sizeof(line_rate_bd_in_tx), (UINT32 *)line_rate_bd_in_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME16 FAIL!\n");
		flag = ERROR;
	}

	if (flag == OK)
	{
		eprintf("All microcode loading completed\n");
	}
}


static void Line_Rate_BW_Master(void)
{
	int flag, status;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	bb_ChipData_t pc;

	flag = OK;

	if(acL->HostType == MASTER)		// if master NPU
	{
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}

		eprintf("Initialising media card. Please wait...\n");

		Save_N_Set_Sp_Vals();

		IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		
		Restore_Sp_Vals();
		eprintf("Media card initialisation completed.\n");

		Clear_All_ME();

		status = DownLoadUcode(LINE_RATE_BW_EG_TX_0_ME, sizeof(line_rate_bw_eg_tx), (unsigned int *)line_rate_bw_eg_tx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME0 FAIL!\n");
			flag = ERROR;
		}

		status = DownLoadUcode(LINE_RATE_BW_EG_TX_1_ME, sizeof(line_rate_bw_eg_tx), (unsigned int *)line_rate_bw_eg_tx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME1 FAIL!\n");
			flag = ERROR;
		}

		status = DownLoadUcode(LINE_RATE_BW_EG_TX_2_ME, sizeof(line_rate_bw_eg_tx), (unsigned int *)line_rate_bw_eg_tx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME2 FAIL!\n");
			flag = ERROR;
		}

		status = DownLoadUcode(LINE_RATE_BW_EG_TX_3_ME, sizeof(line_rate_bw_eg_tx), (unsigned int *)line_rate_bw_eg_tx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME3 FAIL!\n");
			flag = ERROR;
		}

		status = DownLoadUcode(LINE_RATE_BW_EG_RX_ME, sizeof(line_rate_bw_eg_rx), (unsigned int *)line_rate_bw_eg_rx);
		if (status != UENG_SUCCESS)
		{
			eprintf("Microstore loading to ME16 FAIL!\n");
			flag = ERROR;
		}

		if (flag == ERROR)
		{
			eprintf("Line rate test could not be run\n");
			Set_LED("LR F");
			return;
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		Set_Pci_Doorbell(LINERATE_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
#ifdef LINERATE_STAT
		setup_pmu_for_msf();
#endif
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}
}


// This function will be called by the PCI Doorbell function, after the master initiates
// a PCI Doorbell interrupt on the slave NPU
static void Line_Rate_BW_Slave(void)
{
	int flag = OK, status;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	status = DownLoadUcode(LINE_RATE_BW_IN_RX_0_ME, sizeof(line_rate_bw_in_rx), (UINT32 *)line_rate_bw_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME0 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_1_ME, sizeof(line_rate_bw_in_rx), (UINT32 *)line_rate_bw_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME1 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_2_ME, sizeof(line_rate_bw_in_rx), (UINT32 *)line_rate_bw_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME3 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_3_ME, sizeof(line_rate_bw_in_rx), (UINT32 *)line_rate_bw_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME3 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_TX_ME, sizeof(line_rate_bw_in_tx), (UINT32 *)line_rate_bw_in_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME16 FAIL!\n");
		flag = ERROR;
	}

	if (flag == OK)
	{
		eprintf("All microcode loading completed\n");
	}
}

#ifdef LINERATE_STAT
void setup_pmu_for_msf(void)
{
	// restart counter 0 - 4
	*((UINT32 *)CHAPCMD0) = ((0x5 << 16) | 1);
	*((UINT32 *)CHAPCMD1) = ((0x5 << 16) | 1);
	*((UINT32 *)CHAPCMD2) = ((0x5 << 16) | 1);
	*((UINT32 *)CHAPCMD3) = ((0x5 << 16) | 1);
	*((UINT32 *)CHAPCMD4) = ((0x5 << 16) | 1);

	// setup all the events
	*((UINT32 *)PMUCONTCFG) = ((1 << 18) | (0 << 13) | (3 << 7) | 72);
	hal_delay_us(1000);
	*((UINT32 *)PMUCONTCFG) = ((1 << 18) | (1 << 13) | (3 << 7) | 73);
	hal_delay_us(1000);
	*((UINT32 *)PMUCONTCFG) = ((1 << 18) | (2 << 13) | (3 << 7) | 74);
	hal_delay_us(1000);
	*((UINT32 *)PMUCONTCFG) = ((1 << 18) | (3 << 13) | (3 << 7) | 75);
	hal_delay_us(1000);
	*((UINT32 *)PMUCONTCFG) = ((1 << 18) | (4 << 13) | (3 << 7) | 98);
	hal_delay_us(1000);

	// associate event with counters
	*((UINT32 *)CHAPEVN0) = ((6 << 28) | (6 << 23) | (1 << 15) | (6 << 12) | (0 << 7) | 6);
	*((UINT32 *)CHAPEVN1) = ((6 << 28) | (6 << 23) | (1 << 15) | (6 << 12) | (1 << 7) | 6);
	*((UINT32 *)CHAPEVN2) = ((6 << 28) | (6 << 23) | (1 << 15) | (6 << 12) | (2 << 7) | 6);
	*((UINT32 *)CHAPEVN3) = ((6 << 28) | (6 << 23) | (1 << 15) | (6 << 12) | (3 << 7) | 6);
	*((UINT32 *)CHAPEVN4) = ((6 << 28) | (6 << 23) | (1 << 15) | (6 << 12) | (4 << 7) | 6);
}

void dump_pmu_counters(void)
{
	// sample counter 0 - 4
	*((UINT32 *)CHAPCMD0) = ((0x2 << 16) | 1);
	*((UINT32 *)CHAPCMD1) = ((0x2 << 16) | 1);
	*((UINT32 *)CHAPCMD2) = ((0x2 << 16) | 1);
	*((UINT32 *)CHAPCMD3) = ((0x2 << 16) | 1);
	*((UINT32 *)CHAPCMD4) = ((0x2 << 16) | 1);

	// print the counter value
	eprintf("Counter 0 = %d\n", *((UINT32 *)CHAPDATA0));
	eprintf("Counter 1 = %d\n", *((UINT32 *)CHAPDATA1));
	eprintf("Counter 2 = %d\n", *((UINT32 *)CHAPDATA2));
	eprintf("Counter 3 = %d\n", *((UINT32 *)CHAPDATA3));
	eprintf("Counter 4 = %d\n", *((UINT32 *)CHAPDATA4));
}

static void Print_PL_Stat(void)
{
	dump_pmu_counters();
}
static void Print_BW_Stat(void)
{
	int regvalue;
	int i;
	int portoffset;

	Save_N_Set_Sp_Vals();
	for(i = 0; i < 4; i++)
	{
		portoffset = i * 0x100;
		eprintf("Port = %d\n", i);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0460));
		eprintf("Receive POS HDLC Controller Configuration = 0x%08x\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x04D8));
		eprintf("Receive Regenerator Section Status = 0x%08x\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0463));
		eprintf("Receive Frame Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0465));
		eprintf("Receive Byte Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0467));
		eprintf("Receive Abort Frame Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0469));
		eprintf("Receive FCS Error Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x046C));
		eprintf("Receive Minimum Packet Length Error Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0440));
		eprintf("Transmit POS HDLC Controller Configuration = 0x%08x\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0442));
		eprintf("Transmit Frame Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0444));
		eprintf("Transmit Byte Counter = %d\n", regvalue);

		regvalue = IXF6048_REG_VAL((portoffset + 0x0446));
		eprintf("Transmit Abort Frame Counter = %d\n", regvalue);
	}
	Restore_Sp_Vals();
}
#endif

static void Line_Rate_PL_Master(void)
{
	int flag, status;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	extern char channel, dplx;
	extern int speed, ports;

	flag = OK;

	channel = acL->argv[3][0];
	speed = acL->decArg[4];
	dplx = acL->argv[5][0];
	ports = 4;

	switch(channel)
	{
		case 'f':	if(dplx == 'h')
					{
						eprintf("Half duplex mode not supported on fiber channels.\n");
						return;
					}
					else if(dplx != 'f')
					{
						eprintf("Invalid duplex mode. \n");
						return;
					}
					if(speed != 1000)
					{
						eprintf("Invalid spees parameter for fiber channels. It should be 1000.\n");
						return;
					}
					break;

		case 'c':	if((dplx != 'f') && (dplx != 'h'))
					{
						eprintf("Invalid duplex selected for copper channels..\n");
						return;
					}
					if((speed != 10) && (speed != 100) && (speed != 1000))
					{
						eprintf("Invalid speed selected for copper channels\n");
						return;
					}
					if((speed == 1000) && (dplx == 'h'))
					{
						eprintf("Half duplex operation at gigabit speeds is not supported. \n");
						return;
					}
					break;

		default:	eprintf("Invalid Channel Mode\n");
					Line_Rate_Help();
					return;
	}

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
	
	eprintf("Initialising media card. Please wait...\n");
	
	status = init_mac_dev(1);
	if (status != DONE)
	{
		eprintf("Aborting Test\n");
		return;
	}

	spConfig();
	reg_write(0x59f, 0xF);
	reg_write(0x5b2,0x0);
	spRestore();

	eprintf("Media card initialisation completed.\n");

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	if(dplx == 'f')
	{
#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + DUPLEX_ADDR), SWAP32(0));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + DUPLEX_ADDR), 0);
#endif
	}
	else
	{
#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + DUPLEX_ADDR), SWAP32(1));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + DUPLEX_ADDR), 1);
#endif
	}

	status = DownLoadUcode(LINE_RATE_BW_EG_TX_0_ME, sizeof(line_rate_pl_eg_tx), (unsigned int *)line_rate_pl_eg_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME0 FAIL!\n");
		flag = ERROR;
	}
	
	status = DownLoadUcode(LINE_RATE_BW_EG_TX_1_ME, sizeof(line_rate_pl_eg_tx), (unsigned int *)line_rate_pl_eg_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME1 FAIL!\n");
		flag = ERROR;
	}
	
	status = DownLoadUcode(LINE_RATE_BW_EG_TX_2_ME, sizeof(line_rate_pl_eg_tx), (unsigned int *)line_rate_pl_eg_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME2 FAIL!\n");
		flag = ERROR;
	}
	
	status = DownLoadUcode(LINE_RATE_BW_EG_TX_3_ME, sizeof(line_rate_pl_eg_tx), (unsigned int *)line_rate_pl_eg_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME3 FAIL!\n");
		flag = ERROR;
	}
	
	status = DownLoadUcode(LINE_RATE_BW_EG_RX_ME, sizeof(line_rate_pl_eg_rx), (unsigned int *)line_rate_pl_eg_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME16 FAIL!\n");
		flag = ERROR;
	}
	
	if (flag == ERROR)
	{
		eprintf("Line rate test could not be run\n");
		Set_LED("LR F");
		return;
	}
	
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	Set_Pci_Doorbell(LINERATE_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

#ifdef LINERATE_STAT
	setup_pmu_for_msf();
#endif
}


// This function will be called by the PCI Doorbell function, after the master initiates
// a PCI Doorbell interrupt on the slave NPU
static void Line_Rate_PL_Slave(void)
{
	int flag = OK, status;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_0_ME, sizeof(line_rate_pl_in_rx), (UINT32 *)line_rate_pl_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME0 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_1_ME, sizeof(line_rate_pl_in_rx), (UINT32 *)line_rate_pl_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME1 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_2_ME, sizeof(line_rate_pl_in_rx), (UINT32 *)line_rate_pl_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME2 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_RX_3_ME, sizeof(line_rate_pl_in_rx), (UINT32 *)line_rate_pl_in_rx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME3 FAIL!\n");
		flag = ERROR;
	}

	status = DownLoadUcode(LINE_RATE_BW_IN_TX_ME, sizeof(line_rate_pl_in_tx), (UINT32 *)line_rate_pl_in_tx);
	if (status != UENG_SUCCESS)
	{
		eprintf("Microstore loading to ME16 FAIL!\n");
		flag = ERROR;
	}

	if (flag == OK)
	{
		eprintf("All microcode loading completed\n");
	}
}
