/* loopback.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, May 13, 2002
 *  revisions:
 * 
 * --------------------------------------------------------------------------
 */

#include "common.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "hal_ixdp2400.h"
#include "rbuf_flush_ucode.h"
#include "sf_loopback.h"
#include "media_loopback.h"
#include "sys_loopback.h"
#include "init_bdw.h"
#include "init_bd_pos.h"
#include "init_bw_pos.h"
#include "init_bw_atm.h"

void Set_Fibre_LB_1port(UINT32 port_num);
extern ushort SlowPortWrite(bb_RegPointer_type addr, ushort val);


//********************************************************************************************
// Function: void Loopback_Help(void)
//
// Displays the list of instructions for the loopback tests that can be called
// 
// Called by Loopback_Test()
//********************************************************************************************
void Loopback_Help(void)
{
	eprintf("Diag> t lb <option>[media type][ATM mode*] [burst***] [loops] [media loopback mode] [media port]\n");
	eprintf("where <option> are\n");
	eprintf("\ts - Switch fabric loopback test\n");
	eprintf("\tm - Media loopback test\n");
	eprintf("\ty - System loopback test\n");
	eprintf("\t2 - System loopback 2 test\n");
	eprintf("\th - Help\n");
	eprintf("\n    [media type] is the type of media card that is on the system\n");
	eprintf("\td - Battery Drew (Single OC-48)\n");
	eprintf("\tw - Battery Wallace (Quad OC-12)\n");
	eprintf("\n    [ATM mode] is the rx/tx mode to be set to ATM\n");
	eprintf("\ta - Set the test to ATM mode\n");
	eprintf("\n    *** [burst] used only in system loopback 3 test-ignore for others\n");
	eprintf("\ta decimal number between 1 and 1000\n");
	eprintf("\n    [loops] is number of loops for system loopback test\n");
	eprintf("\n    *** [loops] Not applicable for system loopback 2 test\n");
	eprintf("\ta decimal number between 0 and 100000\n");
	eprintf("\n    [media loopback mode] are\n");
	eprintf("\ti - internal loopback (default)\n");
	eprintf("\te - fiber loopback\n");
	eprintf("\n    [media port] are\n");
	eprintf("\t0 - Port 0\n");
	eprintf("\t1 - Port 1\n");
	eprintf("\t2 - Port 2\n");
	eprintf("\t3 - Port 3\n");
	eprintf("\t4 - All ports (default)\n");
	eprintf("\n* OPTIONAL\n");
	eprintf("\n*** Please note that there should be no space typed between\n");
	eprintf("\n<option>, [media type] and [ATM mode]\n");
}


//********************************************************************************************
// Function: void Loopback_Test(void)
//
// Determine which loopback test is called. The available tests are:-
//		Switch fabric loopback test
//		Media loopback test
// 		System loopback test
// 
// Called by Test Manager
//********************************************************************************************
void Loopback_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == SLAVE)
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}

	acL->Err = OK;
	opt = acL->argv[2][0];
	
	switch(opt)
	{
		case 's':	Sf_Loopback_Test();
					break;
		
		case 'm':	Media_Loopback_Test();
					break;

		case 'y': 	Sys_Loopback_Test();
					break;

		case '2':	Sys_Loopback2_Test();
					break;

		case '3':	Sys_Loopback3_Test();
					break;

		case 'a':	Sf_Loopback_Test();
					Media_Loopback_Test();
					Sys_Loopback_Test();
					break;

		case 'h':
		default	:	Loopback_Help();
	}

	if(acL->Err == ERROR)
		eprintf("LOOPBACK TEST FAIL!!!\n");

	return;	
}

void Set_Fibre_LB_1port(UINT32 port_num)
{
	UINT32 addr;

	addr = IXF6048_BASE_ADDRESS + (((port_num << 8) + 0x0009) << REG_SHIFT);
	SlowPortWrite((bb_RegPointer_type)addr, 0x0006);

//	addr = IXF6048_BASE_ADDRESS + (((port_num << 8) + 0x000B) << REG_SHIFT);
//	SlowPortWrite((bb_RegPointer_type)addr, 0x060B);
}
