/*---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * --------------------------------------------------------------------------
 */

#include "diag_utils.h"
#include "syslog.h"
#include "led.h"
#include "diagstruct.h"
#include "msf_common.h"
#include "misc_func.h"
#include "memory.h"
#include "uc_load.h"
#include "media_loopback.h"
#include "uart.h"

#include "error_code.h"
#include "error_map.h"
#include "reg_api.h"
#include "register_map.h"
#include "diag.h"
#include "dp_proto.h"
#include "hal_platform_ints.h"
#include "pci_diag.h"
#include "pci.h"
#include "pl_media_loopback_tx.h"
#include "pl_media_loopback_rx.h"

extern pci_device slave_dev_info;		// for slave npu info.
extern const int pl_media_loopback_tx[];
extern const int pl_media_loopback_rx[];

extern void Msf_OutOfReset(void);
extern int Msf_PLL(void);
extern void spConfig(void);
extern void spRestore(void);
extern void Set_Rcomp(void);
extern int DownLoadUcode(unsigned short Uengine, unsigned short SizeUcode, unsigned int *Ucode);
extern UINT32 init_mac_dev(UINT32);
extern UINT32 validate_params(void);
extern unsigned int htoi(char *s);
extern void get_stats(void);

#define DRAM_BASE_OFFSET	0x1000000
#define PKT_SIZE		60
#define CARD_OPTION_ADDR	0
#define SCRATCH_MSG_BASE	0x1000

UINT32 pl_media_lb_data0[] =	{ 0x00214500, 0x003e0000, 0x00000406, 0xb6bb0000,
					  0x00000000, 0x00000102, 0x03040506, 0x0708090a,
					  0x0b0c0d0e, 0x0f101112, 0x13141516, 0x1718191a,
					  0x1b1c1d1e, 0x1f202122, 0x23242526 };

UINT32 pl_media_lb_data1[] =	{ 0x00214500, 0x003e0001, 0x00000406, 0xb6ba0000,
					  0x00000000, 0x00003132, 0x33343536, 0x3738393a,
					  0x3b3c3d3e, 0x3f404142, 0x43444546, 0x4748494a,
					  0x4b4c4d4e, 0x4f505152, 0x53545556 };

UINT32 pl_media_lb_data2[] =	{ 0x00214500, 0x003e0002, 0x00000406, 0xb6b90000,
					  0x00000000, 0x00006162, 0x63646566, 0x6768696a,
					  0x6b6c6d6e, 0x6f707172, 0x73747576, 0x7778797a,
					  0x7b7c7d7e, 0x7f808182, 0x83848586 };

UINT32 pl_media_lb_data3[] =	{ 0x00214500, 0x003e0003, 0x00000406, 0xb6b80000,
					  0x00000000, 0x00009192, 0x93949596, 0x9798999a,
					  0x9b9c9d9e, 0x9fa0a1a2, 0xa3a4a5a6, 0xa7a8a9aa,
					  0xabacadae, 0xafb0b1b2, 0xb3b4b5b6 };



/** globals **/
extern UINT32 total_args,loop,speed,ports;  
extern char channel,test_opt,dplx,loopback_mode;


// ******************************************************************************
// Function: void lpbk_test(void)
//
// This function is called by the Test Manager when the user invokes the
// media loopback test
// ******************************************************************************
UINT32 lpbk_test(void)
{
	UINT32 ctr, flag, status,port;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, slave_mailbox3_addr;
	volatile int mailbox_value, temp, stop = 0;
	UINT32 dcache_status;
	UINT32 loop_count, forever = 0, packet_num = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 put_ring_base, get_ring_addr;
	UINT32 *pkt=0, fail_port=0xFF;
	UINT32 port_ctr = 0,msg=0, txok = 0;
	short iteration = 1, port_num=0,txpkts = 0,per_port_pkts = 0;
	short cflag = 0;
	//char *l=0;
	

    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	loopback_mode = acL->argv[3][0];
	channel = acL->argv[4][0]; 
	dplx = acL->argv[6][0]; 
	loop = acL->decArg[8];
	speed = acL->decArg[5];	
	ports = acL->decArg[7];
	
	if((channel != 'c') && (channel != 'f'))
	{
		eprintf("Media loopback supports only fiber and copper channels\n");
		return INVALID_PARAMETERS;
	}

	if (total_args <= 7)
	{
		loop = 1;
	}
		
	if (channel == 'c')
	{
		cflag = 1;
	}

	if (loopback_mode != 'i')
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}
	}

	if (ports != 4)
		iteration = 1;
	else
	{
		if (cflag)
			iteration = 2;
		else
			iteration = 4;
	}


	put_ring_base = SCR_RING_INSTR_ADDR;
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	if(acL->HostType == MASTER)		// if master NPU
	{
		flag = OK;

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		// initializing media card
		status=init_mac_dev(0);	
		if (status != DONE)
		{
			eprintf("Aborting Test\n");
			return ABORT;
		}
		
		if (loopback_mode == 'i')
		{
		    /* configure internal loopback */
		    spConfig();
		    reg_write(0x5b2,0xF00);
		    reg_write(0x5b3,0x0);	
		    spRestore();

			cflag = 0;
			channel = 'f';
			if (ports != 4)
				iteration = 1;
			else
				iteration = 4;

		}
		else
		{
		    spConfig();
		    reg_write(0x5b2,0x0);
		    spRestore();
		}
			
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// write packet to be transmitted on each port, into DRAM.
		for (port = 0; port < 4; port++)
		{
		    switch(port)
		    {
			case 0: pkt = (UINT32 *)pl_media_lb_data0;
							break;		

			case 1: pkt = (UINT32 *)pl_media_lb_data1;
							break;

			case 2: pkt = (UINT32 *)pl_media_lb_data2;
							break;

			case 3: pkt = (UINT32 *)pl_media_lb_data3;
							break;
		    }
		    for (ctr = 0; ctr < sizeof(pl_media_lb_data0); ctr+=4)
		    {
			PUT32((DRAM_BASE_OFFSET + (port << 8) + ctr), pkt[(ctr / 4)]);
			//eprintf("port %d -- data : 0x%x\n",port,pkt[(ctr/4)]);
		    }
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
		slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;
		slave_mailbox3_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF;

		// Init master's mailbox0 to 0
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

		// Init slave's mailbox0 and mailbox2 to 0
		PUT32(slave_mailbox0_addr, 0);
		PUT32(slave_mailbox3_addr, 0);
		
		loop_count = loop;
		msg = (cflag << 31) | ((loop_count & 0x7FFF)<<16) | ports;

#ifdef __ARMEB__
		PUT32(slave_mailbox2_addr, SWAP32(msg));
#else
		PUT32(slave_mailbox2_addr, msg);
		
#endif

		Set_Pci_Doorbell(PTLONE_MEDIA_LPBK, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		ctr = 0;
		do		// Wait for a signal that slave NPU has completed loading of ucode and init
		{
			if (ctr == 1500)	// After 15 seconds
			{
				mailbox_value = INIT_ERROR;
				flag = ERROR;
				break;
			}

			temp = swap32(GET32(slave_mailbox3_addr));

			hal_delay_us(10000);
			ctr++;
		}
		while (temp == 0);

		if (temp != INIT_COMPLETE)
		{
			eprintf("MSF init failure on Ingress ...\n");
			return ABORT;
		}
			
        status = DownLoadUcode(PTLONE_MEDIA_LOOPBACK_TX_ME, sizeof(pl_media_loopback_tx), (UINT32 *)(pl_media_loopback_tx));
		if (status != OK)
		{
			eprintf("Error downloading microcode on Master NPU ...\n");
			return ABORT;
		}

		hal_delay_us(1000);

		if (loop_count == 0)
		{
		    forever = 1;
		}
		while ((loop_count != per_port_pkts) || forever)
		{

		    if (forever && KEYPRESS)
		    {
				stop = 1;
				PUT32(slave_mailbox2_addr, stop);
		    }

			// put byte count in the scratch ring locations for all the 4 ports.	
			for (port = 0; port < iteration; port++)
			{
				if (ports != 4)
					port_num = ports;
				else
					if (cflag)
						port_num = port*2;
					else
						port_num = port;

				PUT32((put_ring_base + (port_num << 2)), PKT_SIZE);
		    }


           for (port_ctr = 0; port_ctr < iteration; port_ctr++)
		   {
		        ctr = 0;

				if (ports != 4)
					port_num = ports;
				else
					if (cflag)
						port_num = port_ctr*2;
					else
						port_num = port_ctr;
		        do	// check if the packet was transmitted by master NPU
		        {
		   	        if (ctr == 300000)	// After 3 seconds
					{
						mailbox_value = NOT_SENT;
						flag = ERROR;
						break;
					}

					// Poll on scratch ring to check if packet was transmitted
					mailbox_value = GET32(get_ring_addr);

					hal_delay_us(1);
					ctr++;
		        }
		        while (mailbox_value != 0);
				txpkts = mailbox_value;
		        if (flag == ERROR)
		        {

		            fail_port = port_ctr;		
					break;
		        }
				packet_num++;
		   }  // end of port for loop


	        if (flag == OK)		// If master has successfully transmitted packets
  		    {
				txok = 1;	
				eprintf("Transmitted : %d\r",packet_num);
				ctr = 0;
				mailbox_value = 0;
				do	// Wait for a signal from the slave for indication of completion of test
				{
 		            if (ctr == 50000)	// After 3 seconds
					{
			            mailbox_value = SLAVE_MAILBOX1_NO_SIG;
			            flag = ERROR;
			            break;
					}

		            // Poll on slave's MAILBOX1 to check if packet was received by slave
			        mailbox_value = GET32(slave_mailbox1_addr);

 			        hal_delay_us(10);
			        ctr++;
				}
				while (mailbox_value == 0);

				PUT32(slave_mailbox1_addr, 0);
				//eprintf("slave mailbox 1 : 0x%x\n",mailbox_value);
			}
			if (ports == 4)
			{
				if (cflag)
					per_port_pkts = packet_num/2;
				else
					per_port_pkts = packet_num/4;
			}
			else
				per_port_pkts = packet_num;

			
			if (flag == ERROR)
			{
			    break;
			}
			
		
  		    if (stop == 1)
		    {
				break;
		    }
				
		  	
	    }  // end of while loop_count 
	    eprintf("\n");
		if ((flag == OK) || (txok))
		{

			switch(ports)
			{
				case 0 : 
				case 1 : 
				case 2 : 
				case 3 : 
						eprintf("\nTransmitted on port %d: %d\n", ports,packet_num);
						break;
    
				case 4 : // all ports 
						if (!cflag)
							eprintf("\nTransmitted on ports 0-3 : %d\n", packet_num/4);
						else
							eprintf("\nTransmitted on ports 0 and 2 : %d\n", packet_num/2);
						break;
			}
		}
		if (flag == OK)
		{
			ctr = 0;
			do	// Wait for a signal from the slave for indication of completion of test
			{
				if (ctr == 500000)	// After 3 seconds
				{
					mailbox_value = SLAVE_MAILBOX0_NO_SIG;
					flag = ERROR;
					break;
				}

				// Poll on slave's MAILBOX1 to check if packet was received by slave
#ifdef __ARMEB__
				mailbox_value = SWAP32(GET32(slave_mailbox0_addr));
#else
				mailbox_value = GET32(slave_mailbox0_addr);
#endif
				hal_delay_us(100);
				ctr++;
			}
			while (mailbox_value == 0);
		}

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}


		if (mailbox_value == TEST_COMPLETE)
		{
			eprintf("Point Lone Media loopback test...PASSED\n");
			Set_LED("LbkP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			if (mailbox_value == SLAVE_MAILBOX0_NO_SIG)
			{
				eprintf("No signal was received from slave NPU to indicate completion\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("The packet received was not the same as the packet transmitted\n");
			}
			else if (mailbox_value == INCORRECT_SIZE)
			{
				eprintf("The packet received was not the same size as the packet transmitted\n");
			}
			else if (mailbox_value == SLAVE_MAILBOX1_NO_SIG)
			{
				eprintf("Slave NPU did not indicate that it received a packet\n");
			}
			else if (mailbox_value == NOT_SENT)
			{
				eprintf("Master NPU did not manage to transmit a packet\n");
			}
			else if (mailbox_value == NOT_RECEIVED)
			{
				eprintf("Slave NPU did not receive a packet\n");
			}
			else if (mailbox_value == RECEIVE_ERROR)
			{
				eprintf("Slave NPU receive a packet with an error\n");
			}
			else if (mailbox_value == INIT_ERROR)
			{
				eprintf("Slave NPU could not load its microcode and init\n");
			}



			/* get statistics from Vallejo */
			get_stats();

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Point Lone Media loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("Media loopback test...FAILED!\n");
			Set_LED("LbkF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		

	if (test_opt == 'i')  /* reset loopback reg to 0. after test is finished. */
	{
	    /* configure internal loopback */
	    spConfig();
	    reg_write(RXFIFO_CAPT_LPBK_ENB,0x0);
	    spRestore();
	}


		Clear_All_ME();

	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}	
	return DONE;
}


// ******************************************************************************
// Function: void PL_Media_Loopback_Slave(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
void PL_Media_Loopback_Slave(void)
{
	UINT32 ctr, port_ctr, flag, status, loop_count, forever, packet_num = 0, error, rx_port_num=0;
	UINT32 dram_addr, lw1;
	UINT32 packet_len, mailbox_value=0;
	UINT32 get_ring_addr;
	UINT dcache_status;
	UINT32 *media_lb_data=0;
	UINT32 pktlen_compare = PKT_SIZE;
	UINT32 pass_port[4],ports=0;
	UINT32 mailbox2_val = 0;
	UINT32 rxflag[4], temp =0;
	UINT32 iteration = 0,port_num=0, per_port_pkts = 0;
	/***************************************************************************
	** The rxport is to be indexed by the txport and the value is the corresponding rxport.
	** Tx port	  Rxport
	**	0			1
	**  1			0
	**  2			3
	**  3			2
	*********************************************************************************/
	UINT32 rxport[4] = {1, 0, 3, 2};
	UINT32 msg;
	short cflag = 0;

	flag = OK;
	get_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}
	

	/* initialize port array to 0 */
	for (ctr = 0;ctr<4;ctr++)
	{
		pass_port[ctr] = 0;
		rxflag[ctr] = ERROR;
	}

	msg = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
	ports = msg & 0xFF;
	cflag = msg >> 31;
	loop_count = (msg >> 16) & 0x7FFF;

	if ((ports == 4) && (cflag) )
		iteration = 2;
	else if((ports == 4) && (!cflag))
		iteration = 4;
	else
		iteration = 1;

	if (loop_count == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}
	
	flag = DownLoadUcode(PTLONE_MEDIA_LOOPBACK_RX_ME, sizeof(pl_media_loopback_rx), (UINT32 *)(pl_media_loopback_rx));
	if (flag != OK)
		eprintf("Error downloading microcode on slave !\n");

	hal_delay_us(1000);

	while ((loop_count != per_port_pkts) || forever)
	{
		for (port_ctr = 0; port_ctr < iteration; port_ctr++)
		{
			if (!cflag)
			{
				if (ports != 4)
					port_num = ports;
				else
					port_num = port_ctr;
			}
			else
			{
				if (ports != 4)
					port_num = rxport[ports];
				else
					port_num = port_ctr*2 + 1;
			}
		
			ctr = 0;
			mailbox_value = 0;
			do		// Poll for completion of receive by ME
			{
				if (ctr == 50000)	// After 3 seconds
				{
				    	eprintf("\nTimed out...packet not received on port %d\n",port_num);
					    mailbox_value = NOT_RECEIVED;
					    flag = ERROR;
						rxflag[port_num] = ERROR;
					    break;
				}

				mailbox_value = GET32(get_ring_addr);

				hal_delay_us(100);
				ctr++;
			}
			while (mailbox_value == 0);

			if (flag == ERROR)
			{
				//eprintf("C. flag == ERROR ..\n");
				break;
			}

			rx_port_num = mailbox_value >> 24;

            pass_port[rx_port_num] = 1;
			rxflag[rx_port_num] = OK;
			packet_num++;
			error = (mailbox_value >> 16) & 0xFF;
			packet_len = mailbox_value & MASK_16BIT;
			/** workaround for Vallejo bug :
			    inconsistent CRC stripping by vallejo */
			if (packet_len == 0x40)
				packet_len = packet_len - 4;

			if (error > 0)
			{
				if (error & 1)
				{
					eprintf("\nBit 10 (ABORT ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 1) & 1)
				{
					eprintf("\nBit 11 (LENGTH ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 2) & 1)
				{
					eprintf("\nBit 12 (PARITY ERROR) of Receive Status Word set\n");
				}

				mailbox_value = RECEIVE_ERROR;

				flag = ERROR;
				rxflag[rx_port_num] = ERROR;
				break;
			 }

			if (packet_len == pktlen_compare)
			{
				dram_addr = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + (rx_port_num << 2));
				if (!cflag)
				{
					if (rx_port_num == 0)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data0;
					}
					else if (rx_port_num == 1)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data1;
					}
					else if (rx_port_num == 2)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data2;
					}
					else if (rx_port_num == 3)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data3;
					}
					else
					{
						mailbox_value = INCORRECT_SIZE;
						flag = ERROR;
						rxflag[rx_port_num] = ERROR;
						break;
					}
				}
				else
				{
					if (rx_port_num == 0)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data1;
					}
					else if (rx_port_num == 1)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data0;
					}
					else if (rx_port_num == 2)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data3;
					}
					else if (rx_port_num == 3)
					{
						media_lb_data = (UINT32 *)pl_media_lb_data2;
					}
					else
					{
						mailbox_value = INCORRECT_SIZE;
						flag = ERROR;
						rxflag[rx_port_num] = ERROR;
						break;
					}
				}

				for (ctr = 0; ctr < packet_len-4; ctr+=4)
				{
					lw1 = GET32(dram_addr + ctr);

					if (lw1 != media_lb_data[(ctr / 4)])
					{
						eprintf("ERROR: ctr=%d Orig=0x%08X Received=0x%08X\n", ctr, media_lb_data[(ctr / 4)], lw1);
						mailbox_value = ERROR_PATTERN;
						flag = ERROR;
						rxflag[rx_port_num] = ERROR;
					}
				}

				if(mailbox_value == ERROR_PATTERN)
				{
					eprintf("Port %d received a packet with pattern errors\n", rx_port_num);
					break;
				}
			}
			else
			{
				    mailbox_value = INCORRECT_SIZE;
					eprintf("incorrect size ..\n");
					flag = ERROR;
					rxflag[rx_port_num] = ERROR;
				    break;
			}
		}	// end of per port processing

		   switch(ports)
		   {
		       case 0:
		       case 1:
		       case 2:
		       case 3:   
						if (!cflag)
						{
							if (pass_port[ports] == 0)
							{
								eprintf("Did not Receive packet on port %d\n", ports);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
						}
						else
						{
							if (pass_port[rx_port_num] == 0)
							{
								eprintf("Did not Receive packet on port %d\n", rx_port_num);
								flag = ERROR;
								rxflag[rxport[ports]] = ERROR;
							}
						}


							break;

		       case 4:  
						if (!cflag)
						{
							for (ctr=0; ctr<4; ctr++)
							{
								if (pass_port[ctr] == 0)
								{
									eprintf("Did not Receive packet on port %d\n", ctr);
									rxflag[ctr] = ERROR;
								}
							}
						}
						else
						{
							for (ctr=0; ctr<2; ctr++)
							{
								if (pass_port[ctr*2+1] == 0)
								{
									eprintf("Did not Receive packet on port %d\n", ctr*2+1);
									rxflag[ctr*2+1] = ERROR;
								}
							}
						}
						
						break;
						
			   default:   eprintf("This is bad value for port number \n");
								break;

			}

		
		for (ctr =0;ctr <4; ctr ++)
		{
			if (rxflag[ctr] == OK)
				temp ++;
		}

		if ((ports >=0) && (ports <=3))
		{
			if (!cflag)
			{
				if (rxflag[ports] == OK)
				{
					flag = OK;
				}
				else 
				{
					flag = ERROR;
				}
			}
			else
			{
				if (rxflag[rxport[ports]] == OK)
				{
					flag = OK;
				}
				else 
				{
					flag = ERROR;
				}
			}
		}
		if ((ports == 4) && (temp == 2))
		{
			flag = OK;		
		}

		if((ports == 4) && (cflag))
			per_port_pkts = packet_num/2;
		else if((ports == 4) && (!cflag))
			per_port_pkts = packet_num/4;
		else 
			per_port_pkts = packet_num;
		
		eprintf("Received : %d\r",packet_num);

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), PACKET_RECEIVED);
		// mailbox 2 has  : loop_count (31:16) and port (15:0)
		mailbox2_val = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
		mailbox2_val = (mailbox2_val >> 16) & 0x7FFF;
		
		if ((forever && mailbox2_val) || (flag == ERROR))
		{
			break;
		}

	}	// end of while loop

	eprintf("\n");
	
	switch(ports)
	{
		case 0:
		case 1:
		case 2:
		case 3:
				if (!cflag)
				{
					if (pass_port[ports] == 1)
					{	
						eprintf("Received %d packets on Port %d\n",packet_num,ports);
						flag = OK;
					}
				}
				else
				{
					if (pass_port[rxport[ports]] == 1)
					{	
						eprintf("Received %d packets on Port %d\n",packet_num,rxport[ports]);
						flag = OK;
					}
				}
				break;
		case 4:
				if (!cflag)
				{
					for (ctr=0; ctr<4; ctr++)
					{
						if (pass_port[ctr] == 1)
						{
							eprintf("Received: %d on port %d\n", packet_num/4, ctr);
						}
						
					}
				}
				else
				{
					for (ctr=0; ctr<2; ctr++)
					{
						if (pass_port[ctr*2+1] == 1)
						{
							eprintf("Received: %d on port %d\n", packet_num/2, ctr*2+1);
						}
						
					}
				}

				break;
	}
	
 
	if (flag == OK)
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), TEST_COMPLETE);
	}
	else
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}



/******************************************************
* line test 
*
* This test places the Vallejo in line side loopback. 
* The packets coming in on the RX buffer are moved into the TX buffer and 
* are not forwarded to the SPI-3 interface.
* This test will require an external packet generator.
********************************************************/

UINT32 line_side_lpbk()
{
	short ports = 0,status=0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	loopback_mode = acL->argv[3][0];
	channel = acL->argv[4][0]; 
	dplx = acL->argv[6][0]; 
	speed = acL->decArg[5];	

	loop = 1;
	ports = 4;
	loopback_mode = 'e';

	if (validate_params() == INVALID_PARAMETERS)
	{
		eprintf("Invalid parameters. \n");
		return INVALID_PARAMETERS;
	}

	if(acL->HostType == MASTER)		// if master NPU
	{

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		// initializing media card
		init_mac_dev(1);		    	

	    /* configure line side loopback */
		spConfig();
		reg_write(0x61f,0xF);
		spRestore();

		eprintf("Media card configured ... Ready for packets \n");
		eprintf("This test requires a packet generator !\n");
		eprintf("Please press any key for end of test \n");

		while(1)
		{
			if (KEYPRESS)
			{
				/* un-configure line side loopback */
				spConfig();
				reg_write(0x61f,0x0);
				spRestore();

				eprintf("End of TEST \n");
				break;
			}
		}
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}	
	return DONE;
}

