/* media_loopback.c
 * 
 * This file contains the functions that are required to run the media loopback
 * test. One function is to load the microcode and run the test on the master
 * NPU and chack the status of the test. The second function is called by the
 * PCI doorbell interrupt handler and will load the microcode for the slave NPU
 *
 *-----------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-----------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, April, 2002
 *  revisions:
 * 
 * 
 * ----------------------------------------------------------------------------
 */

#include "media_loopback.h"
#include "media_loopback_ucode.h"
#include "common.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "uart.h"
#include "syslog.h"
#include "diag_utils.h"
#include "msf_init.h"
#include "memory.h"
#include "init_bdw.h"

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;
extern unsigned int *orig_sp_val;
extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern void Set_Fibre_LB_1port(UINT32 port_num);
extern void Loopback_Help(void);
extern void Save_N_Set_Sp_Vals(void);
extern void Restore_Sp_Vals(void);

extern const InitRegTable_t BD_IntLoopbackArray[];
extern const InitRegTable_t BD_ExtLoopbackArray[];
extern const InitRegTable_t BW_IntLoopbackArray[];
extern const InitRegTable_t BW_ExtLoopbackArray[];
extern const InitRegTable_t BW_IntLoopbackArray_ATM[];
extern const InitRegTable_t BW_ExtLoopbackArray_ATM[];

#define DRAM_BASE_OFFSET	0x1000000
#define PACKET_SIZE			64
#define CELL_SIZE			52
#define CARD_OPTION_ADDR	0x0
#define SCRATCH_MSG_BASE	0x1000

extern pci_device slave_dev_info;		// for slave npu info.

UINT32 media_lb_data0[] =	{ 0x00214500, 0x003e0000, 0x00000406, 0xb6bb0000,
								  0x00000000, 0x00000102, 0x03040506, 0x0708090a,
								  0x0b0c0d0e, 0x0f101112, 0x13141516, 0x1718191a,
								  0x1b1c1d1e, 0x1f202122, 0x23242526, 0x2728292a };

UINT32 media_lb_data1[] =	{ 0x00214500, 0x003e0001, 0x00000406, 0xb6ba0000,
								  0x00000000, 0x00003132, 0x33343536, 0x3738393a,
								  0x3b3c3d3e, 0x3f404142, 0x43444546, 0x4748494a,
								  0x4b4c4d4e, 0x4f505152, 0x53545556, 0x5758595a };

UINT32 media_lb_data2[] =	{ 0x00214500, 0x003e0002, 0x00000406, 0xb6b90000,
								  0x00000000, 0x00006162, 0x63646566, 0x6768696a,
								  0x6b6c6d6e, 0x6f707172, 0x73747576, 0x7778797a,
								  0x7b7c7d7e, 0x7f808182, 0x83848586, 0x8788898a };

UINT32 media_lb_data3[] =	{ 0x00214500, 0x003e0003, 0x00000406, 0xb6b80000,
								  0x00000000, 0x00009192, 0x93949596, 0x9798999a,
								  0x9b9c9d9e, 0x9fa0a1a2, 0xa3a4a5a6, 0xa7a8a9aa,
								  0xabacadae, 0xafb0b1b2, 0xb3b4b5b6, 0xb7b8b9ba };

UINT32 media_lb_atm0[] =	{ 0xBABE0000, 0x01020304, 0x05060708, 0x090A0B0C,
								  0x0D0E0F10, 0x11121314, 0x15161718, 0x191A1B1C,
								  0x1D1E1F20, 0x21222324, 0x25262728, 0x292A2B2C,
								  0x2D2E2F30									 };

UINT32 media_lb_atm1[] =	{ 0xBABE1111, 0x31323334, 0x35363738, 0x393A3B3C,
								  0x3D3E3F40, 0x41424344, 0x45464748, 0x494A4B4C,
								  0x4D4E4F50, 0x51525354, 0x55565758, 0x595A5B5C,
								  0x5D5E5F60									 };

UINT32 media_lb_atm2[] =	{ 0xBABE2222, 0x61626364, 0x65666768, 0x696A6B6C,
								  0x6D6E6F70, 0x71727374, 0x75767778, 0x797A7B7C,
								  0x7D7E7F80, 0x81828384, 0x85868788, 0x898A8B8C,
								  0x8D8E8F90									 };

UINT32 media_lb_atm3[] =	{ 0xBABE3333, 0x91929394, 0x95969798, 0x999A9B9C,
								  0x9D9E9FA0, 0xA1A2A3A4, 0xA5A6A7A8, 0xA9AAABAC,
								  0xADAEAFB0, 0xB1B2B3B4, 0xB5B6B7B8, 0xB9BABBBC,
								  0xBDBEBFC0									 };

// ******************************************************************************
// Function: void Media_Loopback_Test(void)
//
// This function is called by the Test Manager when the user invokes the
// media loopback test
// ******************************************************************************
void Media_Loopback_Test(void)
{
	UINT32 opt;
	UINT8 media_type, mode;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	media_type = acL->argv[2][1];
	mode = acL->argv[2][2];

	if (mode == 'a')
	{
		opt = (mode << 8) | media_type;
	}
	else
	{
		opt = media_type;
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;

#ifdef __ARMEB__
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + CARD_OPTION_ADDR), SWAP32(opt));
#else
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + CARD_OPTION_ADDR), opt);
#endif
	
	switch(media_type)
	{
		case 'd':	Media_Loopback_BD();
					break;
		
		case 'w':	if (acL->argv[2][2] == 'a')
					{
						Media_Loopback_BW_ATM();
					}
					else
					{
						Media_Loopback_BW();
					}
					break;

		default:	eprintf("You must specify the media card type.\n");
					Loopback_Help();
	}
}


void Media_Loopback_Test_Slave(void)
{
	UINT32 opt;
	char media_type, mode;

	opt = GET32(SCRATCHPAD_BASE + CARD_OPTION_ADDR);
	media_type = (char)(opt	& 0xFF);
	mode = (char)((opt >> 8) & 0xFF);

	switch(media_type)
	{
		case 'd':	Media_Loopback_BD_Slave();
					break;
		
		case 'w':	if (mode == 'a')
					{
						Media_Loopback_BW_ATM_Slave();
					}
					else
					{
						Media_Loopback_BW_Slave();
					}
					break;

		default:	eprintf("Master did not specify which media card is on the system.\n");
	}
}


void Media_Loopback_BD(void)
{
	UINT32 ctr, flag, status;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, slave_mailbox3_addr;
	volatile int mailbox_value, temp, stop = 0;
	UINT32 put_ring_addr, get_ring_addr;
	UINT32 dcache_status;
	UINT32 total_arg, loop_count, forever = 0, packet_num = 0;
	UINT8 loopback_mode;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	bb_ChipData_t pc;

	total_arg = acL->argc;
	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];

	put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
	get_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
	}

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

	if (total_arg > 3)
	{
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
	slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;
	slave_mailbox3_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF;

	// Init master's mailbox0 to 0
	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

	// Init slave's mailbox0 and mailbox2 to 0
	PUT32(slave_mailbox0_addr, 0);
	PUT32(slave_mailbox3_addr, 0);

#ifdef __ARMEB__
	PUT32(slave_mailbox2_addr, SWAP32(loop_count));
#else
	PUT32(slave_mailbox2_addr, loop_count);
#endif

	// Write packet data to DRAM for microcode to use
	for (ctr = 0; ctr < sizeof(media_lb_data0); ctr+=4)
	{
		PUT32((DRAM_BASE_OFFSET + ctr), media_lb_data0[(ctr / 4)]);
	}

	Set_Pci_Doorbell(MEDIA_LB_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do		// Wait for a signal that slave NPU has completed loading of ucode and init
	{
		if (ctr == 1500)	// After 15 seconds
		{
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox3_addr);

		hal_delay_us(10000);
		ctr++;
	}
	while (temp == 0);

	if (flag == OK)		// If Slave NPUs has completed its ucode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_loopback_bd_tx), (UINT32 *)media_loopback_bd_tx);

		hal_delay_us(1000);

		while (((loop_count != packet_num) || forever) && (flag == OK))
		{
			PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), DRAM_BASE_OFFSET);
			PUT32(put_ring_addr, PACKET_SIZE);

			if (forever && KEYPRESS)
			{
				stop = 1;
				PUT32(slave_mailbox2_addr, stop);
			}

			ctr = 0;
			do	// Do this and while checking if the packet was transmitted by master NPU
			{
				if (ctr == 3000000)	// After 3 seconds
				{
					mailbox_value = NOT_SENT;
					flag = ERROR;
					break;
				}

				// Poll on scratch ring to check if packet was transmitted
				mailbox_value = GET32(get_ring_addr);
		
				hal_delay_us(1);
				ctr++;
			}
			while (mailbox_value == 0);

			if (flag == OK)		// If master has successfully transmitted a packet
			{
				packet_num++;
				eprintf("Transmitted: %d\r", packet_num);

				ctr = 0;
				do	// Wait for a signal from the slave for indication of completion of test
				{
					if (ctr == 3000000)	// After 3 seconds
					{
						mailbox_value = SLAVE_MAILBOX1_NO_SIG;
						flag = ERROR;
						break;
					}

					// Poll on slave's MAILBOX1 to check if packet was received by slave
#ifdef __ARMEB__
					mailbox_value = SWAP32(GET32(slave_mailbox1_addr));
#else
					mailbox_value = GET32(slave_mailbox1_addr);
#endif

					hal_delay_us(1);
					ctr++;
				}
				while (mailbox_value == 0);

				if ((mailbox_value != PACKET_RECEIVED) && (flag != ERROR))
				{
					flag = ERROR;
					break;
				}

				PUT32(slave_mailbox1_addr, 0);
			}

			if (stop == 1)
			{
				break;
			}
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		ctr = 0;
		do	// Wait for a signal from the slave for indication of completion of test
		{
			if (ctr == 300000)	// After 3 seconds
			{
				mailbox_value = SLAVE_MAILBOX0_NO_SIG;
				flag = ERROR;
				break;
			}

			// Poll on slave's MAILBOX1 to check if packet was received by slave
#ifdef __ARMEB__
			mailbox_value = SWAP32(GET32(slave_mailbox0_addr));
#else
			mailbox_value = GET32(slave_mailbox0_addr);
#endif
			
			hal_delay_us(10);
			ctr++;
		}
		while (mailbox_value == 0);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (mailbox_value == TEST_COMPLETE)
	{
		eprintf("Media loopback test...PASSED\n");
		Set_LED("LmdP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		if (mailbox_value == SLAVE_MAILBOX0_NO_SIG)
		{
			eprintf("No signal was received from slave NPU to indicate completion\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted\n");
		}
		else if (mailbox_value == SLAVE_MAILBOX1_NO_SIG)
		{
			eprintf("Slave NPU did not indicate that it received a packet\n");
		}
		else if (mailbox_value == NOT_SENT)
		{
			eprintf("Master NPU did not manage to transmit a packet\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("Slave NPU did not receive a packet\n");
		}
		else if (mailbox_value == RECEIVE_ERROR)
		{
			eprintf("Slave NPU receive a packet with an error\n");
		}
		else if (mailbox_value == INIT_ERROR)
		{
			eprintf("Slave NPU could not load its microcode and init\n");
		}
			
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Media loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("Media loopback test...FAIL!\n");
		Set_LED("LmdF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Media_Loopback_Test_Slave(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
void Media_Loopback_BD_Slave(void)
{
	UINT32 ctr, flag, status, loop_count, forever, packet_num = 0, error;
	UINT32 lw1;
	UINT32 packet_len, mailbox_value=0;
	UINT32 get_ring_addr;
	UINT dcache_status;

	flag = OK;
	get_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), 0);

	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
	if (loop_count == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}

	// Load ME with microcode
	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_loopback_bd_rx), (UINT32 *)media_loopback_bd_rx);

	hal_delay_us(1000);

	while ((loop_count != packet_num) || forever)
	{
		ctr = 0;
		do		// Poll for completion of receive by ME
		{
			if (ctr == 3000000)	// After 3 seconds
			{
				eprintf("\nTimed out...No signal from scratch ring indicating any packet received\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}

			mailbox_value = GET32(get_ring_addr);

			hal_delay_us(1);
			ctr++;
		}
		while (mailbox_value == 0);

		if (flag == ERROR)
		{
			break;
		}

		packet_num++;
		eprintf("Received: %d\r", packet_num);

//		eprintf("DEBUG: mailbox_value = %08X\n", mailbox_value);

		error = mailbox_value >> 16;
		packet_len = mailbox_value & MASK_16BIT;

//		eprintf("DEBUG: error = %08X\tpacket_len = %08X\n", error, packet_len);

		if (error > 0)
		{
			if (error & 1)
			{
				eprintf("\nBit 10 (ABORT ERROR) of Receive Status Word set\n");
			}
			else if ((error >> 1) & 1)
			{
				eprintf("\nBit 11 (LENGTH ERROR) of Receive Status Word set\n");
			}
			else if ((error >> 2) & 1)
			{
				eprintf("\nBit 12 (PARITY ERROR) of Receive Status Word set\n");
			}

			mailbox_value = RECEIVE_ERROR;
			flag = ERROR;
		}
	
		if (flag != ERROR)
		{
			if (packet_len == PACKET_SIZE)
			{
				for (ctr = 0; ctr < packet_len; ctr+=4)
				{
					lw1 = GET32(DRAM_BASE_OFFSET + ctr);
	//				eprintf("DEBUG: ctr=%d addr=%d lw1=0x%X\n", ctr, (DRAM_BASE_OFFSET + ctr), lw1);
					if (lw1 != media_lb_data0[(ctr / 4)])
					{
	//					eprintf("DEBUG: ctr=%d Orig=0x%X Received=0x%X\n", ctr, media_lb_data0[(ctr / 4)], lw1);
						mailbox_value = ERROR_PATTERN;
						flag = ERROR;
						break;
					}
				}
			}
			else
			{
				mailbox_value = INCORRECT_SIZE;
				flag = ERROR;
			}
		}

		if (flag != ERROR)
		{
			for (ctr = 0; ctr < (PACKET_SIZE * 2); ctr+=4)
			{
				PUT32((DRAM_BASE_OFFSET + ctr), 0xAAAAAAAA);
			}
		}


		if (flag != ERROR)
		{
			PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), PACKET_RECEIVED);
		}
		else
		{
			PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), mailbox_value);
		}

		if ((forever && (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF))) || (flag == ERROR))
		{
			break;
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), TEST_COMPLETE);
	}
	else
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Media_Loopback_BW(void)
//
// This function is called by the Test Manager when the user invokes the
// media loopback test
// ******************************************************************************
void Media_Loopback_BW(void)
{
	UINT32 ctr, port_ctr, flag, status;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, slave_mailbox3_addr;
	volatile int mailbox_value, temp, stop = 0;
	UINT32 put_ring_base, get_ring_addr;
	UINT32 dcache_status;
	UINT32 total_arg, loop_count, forever = 0, packet_num = 0, ext_port_num;
	UINT8 loopback_mode;
	UINT32 *media_lb_data;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	bb_ChipData_t pc;

	total_arg = acL->argc;
	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];
	ext_port_num = acL->decArg[5];

	if ((total_arg == 6) && (ext_port_num > 4))
	{
		Loopback_Help();
		return;
	}

	put_ring_base = SCR_RING_INSTR_ADDR;
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");

		if ((total_arg > 5) && (ext_port_num >= 0) && (ext_port_num < 4))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
			Set_Fibre_LB_1port(ext_port_num);
		}
		else			// if ((total_arg < 6) || ((total_arg > 5) && (ext_port_num == 4)))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
	}

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

	if (total_arg > 3)
	{
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
	slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;
	slave_mailbox3_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF;

	// Init master's mailbox0 to 0
	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

	// Init slave's mailbox0 and mailbox2 to 0
	PUT32(slave_mailbox0_addr, 0);
	PUT32(slave_mailbox3_addr, 0);
#ifdef __ARMEB__
	PUT32(slave_mailbox2_addr, SWAP32(loop_count));
#else
	PUT32(slave_mailbox2_addr, loop_count);
#endif

	// Write packet data to DRAM for microcode to use
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			media_lb_data = (UINT32 *)media_lb_data0;
		}
		else if (port_ctr == 1)
		{
			media_lb_data = (UINT32 *)media_lb_data1;
		}
		else if (port_ctr == 2)
		{
			media_lb_data = (UINT32 *)media_lb_data2;
		}
		else
		{
			media_lb_data = (UINT32 *)media_lb_data3;
		}

		for (ctr = 0; ctr < sizeof(media_lb_data0); ctr+=4)
		{
			PUT32((DRAM_BASE_OFFSET + (port_ctr << 8) + ctr), media_lb_data[(ctr / 4)]);
		}
	}

	Set_Pci_Doorbell(MEDIA_LB_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do		// Wait for a signal that slave NPU has completed loading of ucode and init
	{
		if (ctr == 1500)	// After 15 seconds
		{
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox3_addr);

		hal_delay_us(10000);
		ctr++;
	}
	while (temp == 0);

	if (flag == OK)		// If Slave NPUs has completed its ucode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_loopback_bw_tx), (UINT32 *)media_loopback_bw_tx);

		hal_delay_us(1000);

		while (((loop_count != packet_num) || forever) && (flag == OK))
		{
			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				PUT32((put_ring_base + (port_ctr << 2)), PACKET_SIZE);
				PUT32((put_ring_base + (port_ctr << 2)), DRAM_BASE_OFFSET);
				hal_delay_us(100);
			}

			if (forever && KEYPRESS)
			{
				stop = 1;
				PUT32(slave_mailbox2_addr, stop);
			}

			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				ctr = 0;
				mailbox_value = 0;
			
				do	// Do this and while checking if the packet was transmitted by master NPU
				{
					if (ctr == 300000)	// After 3 seconds
					{
						mailbox_value = NOT_SENT;
						flag = ERROR;
						break;
					}

					// Poll on scratch ring to check if packet was transmitted
					mailbox_value = GET32(get_ring_addr);
		
					hal_delay_us(10);
					ctr++;
				}
				while (mailbox_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}
			
			if (flag == OK)		// If master has successfully transmitted packets
			{
				packet_num++;
				eprintf("Transmitted: %d\r", (packet_num * 4));

				ctr = 0;
				do	// Wait for a signal from the slave for indication of completion of test
				{
					if (ctr == 300000)	// After 3 seconds
					{
						mailbox_value = SLAVE_MAILBOX1_NO_SIG;
						flag = ERROR;
						break;
					}

					// Poll on slave's MAILBOX1 to check if packet was received by slave
					mailbox_value = GET32(slave_mailbox1_addr);

					hal_delay_us(10);
					ctr++;
				}
				while (mailbox_value == 0);

				PUT32(slave_mailbox1_addr, 0);
			}

			if (stop == 1)
			{
				break;
			}
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		ctr = 0;
		do	// Wait for a signal from the slave for indication of completion of test
		{
			if (ctr == 300000)	// After 3 seconds
			{
				mailbox_value = SLAVE_MAILBOX0_NO_SIG;
				flag = ERROR;
				break;
			}

			// Poll on slave's MAILBOX1 to check if packet was received by slave
#ifdef __ARMEB__
			mailbox_value = SWAP32(GET32(slave_mailbox0_addr));
#else
			mailbox_value = GET32(slave_mailbox0_addr);
#endif
			
			hal_delay_us(10);
			ctr++;
		}
		while (mailbox_value == 0);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (mailbox_value == TEST_COMPLETE)
	{
		eprintf("Media loopback test...PASSED\n");
		Set_LED("LmwP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		if (mailbox_value == SLAVE_MAILBOX0_NO_SIG)
		{
			eprintf("No signal was received from slave NPU to indicate completion\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted\n");
		}
		else if (mailbox_value == SLAVE_MAILBOX1_NO_SIG)
		{
			eprintf("Slave NPU did not indicate that it received a packet\n");
		}
		else if (mailbox_value == NOT_SENT)
		{
			eprintf("Master NPU did not manage to transmit a packet\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("Slave NPU did not receive a packet\n");
		}
		else if (mailbox_value == RECEIVE_ERROR)
		{
			eprintf("Slave NPU receive a packet with an error\n");
		}
		else if (mailbox_value == INIT_ERROR)
		{
			eprintf("Slave NPU could not load its microcode and init\n");
		}
			
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Media loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("Media loopback test...FAIL!\n");
		Set_LED("LmwF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Media_Loopback_BW_Slave(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
void Media_Loopback_BW_Slave(void)
{
	UINT32 ctr, port_ctr, flag, status, loop_count, forever, packet_num = 0, error, rx_port_num;
	UINT32 dram_addr, lw1;
	UINT32 packet_len, mailbox_value=0;
	UINT32 get_ring_addr;
	UINT dcache_status;
	UINT32 *media_lb_data;

	flag = OK;
	get_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), 0);

	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
	if (loop_count == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}

	// Load ME with microcode
	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_loopback_bw_rx), (UINT32 *)media_loopback_bw_rx);

	hal_delay_us(1000);

	while ((loop_count != packet_num) || forever)
	{
		for (port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			ctr = 0;
			do		// Poll for completion of receive by ME
			{
				if (ctr == 3000000)	// After 3 seconds
				{
					eprintf("\nTimed out...No signal from scratch ring indicating any packet received\n");
					mailbox_value = NOT_RECEIVED;
					flag = ERROR;
					break;
				}

				mailbox_value = GET32(get_ring_addr);

				hal_delay_us(1);
				ctr++;
			}
			while (mailbox_value == 0);

			if (flag == ERROR)
			{
				break;
			}

			rx_port_num = mailbox_value >> 24;
			error = (mailbox_value >> 16) & 0xFF;
			packet_len = mailbox_value & MASK_16BIT;

			if (error > 0)
			{
				if (error & 1)
				{
					eprintf("\nBit 10 (ABORT ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 1) & 1)
				{
					eprintf("\nBit 11 (LENGTH ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 2) & 1)
				{
					eprintf("\nBit 12 (PARITY ERROR) of Receive Status Word set\n");
				}

				mailbox_value = RECEIVE_ERROR;
				flag = ERROR;
				eprintf("Port %d received a packet with errors\n", rx_port_num);
				break;
			}
	
			if (packet_len == PACKET_SIZE)
			{
				dram_addr = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + (rx_port_num << 2));

				if (rx_port_num == 0)
				{
					media_lb_data = (UINT32 *)media_lb_data0;
				}
				else if (rx_port_num == 1)
				{
					media_lb_data = (UINT32 *)media_lb_data1;
				}
				else if (rx_port_num == 2)
				{
					media_lb_data = (UINT32 *)media_lb_data2;
				}
				else if (rx_port_num == 3)
				{
					media_lb_data = (UINT32 *)media_lb_data3;
				}
				else
				{
					mailbox_value = INCORRECT_SIZE;
					flag = ERROR;
					eprintf("Port %d received an incorrect sized packet\n", rx_port_num);
					break;
				}

				for (ctr = 0; ctr < packet_len; ctr+=4)
				{
					lw1 = GET32(dram_addr + ctr);
//					eprintf("GOOD: ctr=%02d addr=%08X lw=0x%08X\n", ctr, (DRAM_BASE_OFFSET + ctr), lw1);
					if (lw1 != media_lb_data[(ctr / 4)])
					{
//						eprintf("ERROR: ctr=%d Orig=0x%08X Received=0x%08X\n", ctr, media_lb_data[(ctr / 4)], lw1);
						mailbox_value = ERROR_PATTERN;
						flag = ERROR;
					}
				}

				if(mailbox_value == ERROR_PATTERN)
				{
					eprintf("Port %d received a packet with pattern errors\n", rx_port_num);
					break;
				}
			}
			else
			{
				mailbox_value = INCORRECT_SIZE;
				flag = ERROR;
				break;
			}

		}

		packet_num++;
		eprintf("Received: %d\r", (packet_num * 4));

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), PACKET_RECEIVED);

		if ((forever && (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF))) || (flag == ERROR))
		{
			break;
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), TEST_COMPLETE);
	}
	else
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}


void Media_Loopback_BW_ATM(void)
{
	UINT32 ctr, port_ctr, flag, status;
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, slave_mailbox3_addr;
	volatile int mailbox_value, temp, stop = 0;
	UINT32 put_ring_base, get_ring_addr;
	UINT32 dcache_status;
	UINT32 total_arg, loop_count, forever = 0, packet_num = 0, ext_port_num;
	UINT8 loopback_mode;
	UINT32 *media_lb_data;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	bb_ChipData_t pc;

	total_arg = acL->argc;
	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];
	ext_port_num = acL->decArg[5];

	if ((total_arg == 6) && (ext_port_num > 4))
	{
		Loopback_Help();
		return;
	}

	put_ring_base = SCR_RING_INSTR_ADDR;
	get_ring_addr = SCR_RING_INSTR_ADDR + (4 << 2);

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");

		if ((total_arg == 6) && (ext_port_num >= 0) && (ext_port_num < 4))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray_ATM);
			Set_Fibre_LB_1port(ext_port_num);
		}
		else		// if ((total_arg < 6) || ((total_arg > 5) && (ext_port_num == 4)))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray_ATM);
		}
	}
	else
	{
		eprintf("Setting to internal loopback ATM mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray_ATM);
	}

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

	if (total_arg > 3)
	{
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
	slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;
	slave_mailbox3_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF;

	// Init master's mailbox0 to 0
	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

	// Init slave's mailbox0 and mailbox2 to 0
	PUT32(slave_mailbox0_addr, 0);
	PUT32(slave_mailbox3_addr, 0);
#ifdef __ARMEB__
	PUT32(slave_mailbox2_addr, SWAP32(loop_count));
#else
	PUT32(slave_mailbox2_addr, loop_count);
#endif

	// Write packet data to DRAM for microcode to use
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			media_lb_data = (UINT32 *)media_lb_atm0;
		}
		else if (port_ctr == 1)
		{
			media_lb_data = (UINT32 *)media_lb_atm1;
		}
		else if (port_ctr == 2)
		{
			media_lb_data = (UINT32 *)media_lb_atm2;
		}
		else
		{
			media_lb_data = (UINT32 *)media_lb_atm3;
		}

		for (ctr = 0; ctr < sizeof(media_lb_data0); ctr+=4)
		{
			PUT32((DRAM_BASE_OFFSET + (port_ctr << 8) + ctr), media_lb_data[(ctr / 4)]);
		}
	}

	Set_Pci_Doorbell(MEDIA_LB_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do		// Wait for a signal that slave NPU has completed loading of ucode and init
	{
		if (ctr == 1500)	// After 15 seconds
		{
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox3_addr);

		hal_delay_us(10000);
		ctr++;
	}
	while (temp == 0);

	if (flag == OK)		// If Slave NPUs has completed its ucode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_loopback_bw_atm_tx), (UINT32 *)media_loopback_bw_atm_tx);

		hal_delay_us(1000);

		while (((loop_count != packet_num) || forever) && (flag == OK))
		{
			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				PUT32((put_ring_base + (port_ctr << 2)), CELL_SIZE);
				PUT32((put_ring_base + (port_ctr << 2)), DRAM_BASE_OFFSET);
				hal_delay_us(100);
			}

			if (forever && KEYPRESS)
			{
				stop = 1;
				PUT32(slave_mailbox2_addr, stop);
			}

			for (port_ctr = 0; port_ctr < 4; port_ctr++)
			{
				ctr = 0;
				mailbox_value = 0;
			
				do	// Do this and while checking if the packet was transmitted by master NPU
				{
					if (ctr == 300000)	// After 3 seconds
					{
						mailbox_value = NOT_SENT;
						flag = ERROR;
						break;
					}

					// Poll on scratch ring to check if packet was transmitted
					mailbox_value = GET32(get_ring_addr);
		
					hal_delay_us(10);
					ctr++;
				}
				while (mailbox_value == 0);

				if (flag == ERROR)
				{
					break;
				}
			}
			
			if (flag == OK)		// If master has successfully transmitted packets
			{
				packet_num++;
				eprintf("Transmitted: %d\r", (packet_num * 4));

				ctr = 0;
				do	// Wait for a signal from the slave for indication of completion of test
				{
					if (ctr == 300000)	// After 3 seconds
					{
						mailbox_value = SLAVE_MAILBOX1_NO_SIG;
						flag = ERROR;
						break;
					}

					// Poll on slave's MAILBOX1 to check if packet was received by slave
					mailbox_value = GET32(slave_mailbox1_addr);

					hal_delay_us(10);
					ctr++;
				}
				while (mailbox_value == 0);

				PUT32(slave_mailbox1_addr, 0);
			}

			if (stop == 1)
			{
				break;
			}
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		ctr = 0;
		do	// Wait for a signal from the slave for indication of completion of test
		{
			if (ctr == 300000)	// After 3 seconds
			{
				mailbox_value = SLAVE_MAILBOX0_NO_SIG;
				flag = ERROR;
				break;
			}

			// Poll on slave's MAILBOX1 to check if packet was received by slave
#ifdef __ARMEB__
			mailbox_value = SWAP32(GET32(slave_mailbox0_addr));
#else
			mailbox_value = GET32(slave_mailbox0_addr);
#endif
			
			hal_delay_us(10);
			ctr++;
		}
		while (mailbox_value == 0);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (mailbox_value == TEST_COMPLETE)
	{
		eprintf("Media loopback test...PASSED\n");
		Set_LED("LmwP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		if (mailbox_value == SLAVE_MAILBOX0_NO_SIG)
		{
			eprintf("No signal was received from slave NPU to indicate completion\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted\n");
		}
		else if (mailbox_value == SLAVE_MAILBOX1_NO_SIG)
		{
			eprintf("Slave NPU did not indicate that it received a packet\n");
		}
		else if (mailbox_value == NOT_SENT)
		{
			eprintf("Master NPU did not manage to transmit a packet\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("Slave NPU did not receive a packet\n");
		}
		else if (mailbox_value == RECEIVE_ERROR)
		{
			eprintf("Slave NPU receive a packet with an error\n");
		}
		else if (mailbox_value == INIT_ERROR)
		{
			eprintf("Slave NPU could not load its microcode and init\n");
		}
			
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Media loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("Media loopback test...FAIL!\n");
		Set_LED("LmwF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	Clear_All_ME();
}


void Media_Loopback_BW_ATM_Slave(void)
{
	UINT32 ctr, port_ctr, flag, status, loop_count, forever, packet_num = 0, error, rx_port_num;
	UINT32 dram_addr, lw1;
	UINT32 packet_len, mailbox_value=0;
	UINT32 get_ring_addr;
	UINT dcache_status;
	UINT32 *media_lb_data;

	flag = OK;
	get_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), 0);

	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
	if (loop_count == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}

	// Load ME with microcode
	DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_loopback_bw_atm_rx), (UINT32 *)media_loopback_bw_atm_rx);

	hal_delay_us(1000);

	while ((loop_count != packet_num) || forever)
	{
		for (port_ctr = 0; port_ctr < 4; port_ctr++)
		{
			ctr = 0;
			do		// Poll for completion of receive by ME
			{
				if (ctr == 3000000)	// After 3 seconds
				{
					eprintf("\nTimed out...No signal from scratch ring indicating any packet received\n");
					mailbox_value = NOT_RECEIVED;
					flag = ERROR;
					break;
				}

				mailbox_value = GET32(get_ring_addr);

				hal_delay_us(1);
				ctr++;
			}
			while (mailbox_value == 0);

			if (flag == ERROR)
			{
				eprintf("DEBUG: Not packet received\n");
				break;
			}

			rx_port_num = mailbox_value >> 24;
			error = (mailbox_value >> 16) & 0xFF;
			packet_len = mailbox_value & MASK_16BIT;

			if (error > 0)
			{
				if (error & 1)
				{
					eprintf("\nBit 10 (ABORT ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 1) & 1)
				{
					eprintf("\nBit 11 (LENGTH ERROR) of Receive Status Word set\n");
				}
				else if ((error >> 2) & 1)
				{
					eprintf("\nBit 12 (PARITY ERROR) of Receive Status Word set\n");
				}

				mailbox_value = RECEIVE_ERROR;
				flag = ERROR;
				eprintf("Port %d received a packet with errors\n", rx_port_num);
				break;
			}
	
			if (packet_len == CELL_SIZE)
			{
				dram_addr = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + (rx_port_num << 2));

				if (rx_port_num == 0)
				{
					media_lb_data = (UINT32 *)media_lb_atm0;
				}
				else if (rx_port_num == 1)
				{
					media_lb_data = (UINT32 *)media_lb_atm1;
				}
				else if (rx_port_num == 2)
				{
					media_lb_data = (UINT32 *)media_lb_atm2;
				}
				else if (rx_port_num == 3)
				{
					media_lb_data = (UINT32 *)media_lb_atm3;
				}
				else
				{
					mailbox_value = INCORRECT_SIZE;
					flag = ERROR;
					eprintf("Port %d received an incorrect sized packet\n", rx_port_num);
					break;
				}

				for (ctr = 0; ctr < packet_len; ctr+=4)
				{
					lw1 = GET32(dram_addr + ctr);
//					eprintf("GOOD: ctr=%02d addr=%08X lw=0x%08X\n", ctr, (DRAM_BASE_OFFSET + ctr), lw1);
					if (lw1 != media_lb_data[(ctr / 4)])
					{
//						eprintf("ERROR: ctr=%d Orig=0x%08X Received=0x%08X\n", ctr, media_lb_data[(ctr / 4)], lw1);
						mailbox_value = ERROR_PATTERN;
						flag = ERROR;
					}
				}

				if(mailbox_value == ERROR_PATTERN)
				{
					eprintf("Port %d received a packet with pattern errors\n", rx_port_num);
					break;
				}
			}
			else
			{
				mailbox_value = INCORRECT_SIZE;
				flag = ERROR;
				break;
			}

		}

		packet_num++;
		eprintf("Received: %d\r", (packet_num * 4));

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), PACKET_RECEIVED);

		if ((forever && (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF))) || (flag == ERROR))
		{
			break;
		}
	}

	eprintf("\n");

	if (flag == OK)
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), TEST_COMPLETE);
	}
	else
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}
