/* memory.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 15, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "memory.h"
#include "diag_utils.h"
#include "uart.h"
#include "led.h"
#include "hal_ixdp2400.h"
#include "syslog.h"

//static function prototypes
static void Memory_Help(void);
static void SRAM_Test(UINT mem_size, UINT first, UINT last, UINT reps);
static void SDRAM_Test(UINT mem_size, UINT first, UINT last, UINT reps);
static int SDRAM_ECC_Test(PUINT first);
static int Test_Mem(UINT base, UINT mem_size, UINT* first, UINT* last, UINT flags);
static int Walking_AddrBus_Test(UINT base, UINT size);
static int Walking_Test(UINT base, UINT mem_size, PUINT first, PUINT last, UINT pattern);
static int Checker_Test(UINT base, UINT mem_size, PUINT first, PUINT last, UINT seed);
static int Unique_Test(UINT base, UINT mem_size, PUINT first, PUINT last);
static int Incremental_Test(UINT base, UINT mem_size, PUINT first, PUINT last);

// max SDRAM supported is 1GB but space allocated is 2GB
// max SRAM supported is 64MB but space allocated is 1GB

static unsigned int SIGNAL = 0x1000;
static unsigned int forever = 0;

#define SRAM	1
#define SDRAM	2
static unsigned int mem_type;
#define ON	1
#define OFF	0
static unsigned int verbose = OFF;
//static unsigned int hoe = 0;

static UINT counter = 0;

extern UINT32 sram_chan[], sram_size[];

#define MAXIMUM	0x1
#define MINIMAL	0x2
extern UINT32 t_all_verbose;

//
// Function: Memory_Help
//
// Description: 
//
static void Memory_Help(void)
{
	eprintf("> t mem <option> <S:test option> <X:start addr> <X:size> <x:data> <x:loops> <x:verbose>\n");
	eprintf("where <option> are:\n");
	eprintf("  s  - SRAM memory test\n");
	eprintf("  d  - DRAM memory test\n");
	eprintf("  a  - all tests\n");
	eprintf("  h  - help\n\n");

	eprintf("where <test option> are:\n");
	eprintf("  w1 - Walking '1's Test\n");
	eprintf("  w0 - Walking '0's Test\n");
	eprintf("  w  - Both Walking Test\n");
	eprintf("  c5 - Checker 0x5A5A5A5A Test\n");
	eprintf("  ca - Checker 0xA5A5A5A5 Test\n");
	eprintf("  c  - Both Checker Test\n");
	eprintf("  u  - Unique Test\n");
	eprintf("  b  - Address Bus Test\n");
	eprintf("  p  - Pattern Test (<x:data> is required)\n");
	eprintf("  i  - Incremental Test\n");
	eprintf("  e  - ECC Test (ONLY FOR DRAM)\n");
	eprintf("  a  - All Test\n");
}

//
// Function: Memory_Test
//
// Description: 
//
void Memory_Test(void)
{
	char opt[16];	
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	UINT dcache_status;
	//hoe = acL->HaltOnError;

	UINT SDRAM_Size = acL->SDRAMSize;
  	UINT SRAM_Size = acL->SRAMSize;

	UINT first = acL->hexArg[4];
	UINT size = acL->hexArg[5];
	UINT loops = 1;	// defaults to 1 if not specified
	verbose = OFF;

	strcpy(opt, acL->argv[2]);
	forever = 0;

	if(opt[0] == 'h')
		Memory_Help();
	else
	{

	if(acL->argc < 6)
	{
		eprintf("***ERROR***: Insufficient arguments. Type \"t mem h\" for help\n");
		return;
	}
	else if(acL->argc > 7)
	{
		loops = acL->hexArg[7];
		if(loops == 0)
			forever = 1;

		if(acL->argc > 8)
		{
			if(acL->hexArg[8] != OFF)
			verbose = ON;
		}
	}

	if (first&3)
	{
	    eprintf("memory location MUST be on longword boundary\n");
	    eprintf("aligning to longword boundary\n");
	    first &=0xFFFFFFFC;
	}

	if(!(get_CP15() & ICACHE_BIT))
	{
    	eprintf("Note: Enable Icache to speedup memory diagnostics \n");
		SIGNAL = 0x10000;//SIGNAL = 0x1000;
	}
	else
		SIGNAL = 0x100000;

	switch(opt[0])
	{
		case 's': 	SRAM_Test(SRAM_Size, first, size, loops);	// QDR
					break;

		case 'd': 	SDRAM_Test(SDRAM_Size, first, size, loops);	// DDR
					break;

		case 'a':	dcache_status = get_CP15() & DCACHE_BIT;
					if(dcache_status != 0)	// if dcache ON
						dcache_off();
					SRAM_Test(SRAM_Size, first, SRAM_Size/2, loops);	// QDR non-burst
					SDRAM_Test(SDRAM_Size, first, SDRAM_Size, loops);	// DDR non-burst

					dcache_on();
					SRAM_Test(SRAM_Size, first, SRAM_Size/2, loops);	// QDR Burst
					SDRAM_Test(SDRAM_Size, first, SDRAM_Size, loops);	// DDR Burst

					if(dcache_status == 0)	// if dcache OFF
						dcache_off();
					break;

		case 'h':
		default:	Memory_Help();
	}
	}// if(opt[0] == 'h')
}
//
// Function: SRAM_Test
//
// Description: 
//
static void SRAM_Test(UINT mem_size, UINT ch0_first, UINT size, UINT loops)
{
	unsigned int ch0_last, ch1_last, ch1_first = ch0_first;
	unsigned int flags = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	char opt[16];

	acL->Err = OK;
	strcpy(opt, acL->argv[3]);
	mem_type = SRAM;

	switch(opt[0])
	{
		case 'w':	if(opt[1] == '1')		// Walking '1's
						flags = WALK_1;
					else if(opt[1] == '0')	// Walking '0's
						flags = WALK_0;
					else					// Both Walks
						flags = WALK;
					break;

		case 'c':	if(opt[1] == '5')		// Checker 0x5A5A5A5A
						flags = CHECKER_5A;
					else if(opt[1] == 'a')	// Checker 0xA5A5A5A5
						flags = CHECKER_A5;
					else					// Both Checkers
						flags = CHECKER;
					break;

		case 'u':	flags = UNIQUE;			// Unique Test
					break;

		case 'i':	flags = INCREMENTAL;	// Incremental Test
					break;

		case 'b':	flags = ADDR_BUS;		// Addr Bus Test
					break;

		case 'p':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Pattern input is missing.\n");						
					else
						flags = USER_DEFINE;// User defined pattern test;
					break;

		case 'm':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Number of pattern input is missing.\n");						
					else
						flags = MULTI_PATRN;
					break;

		case 'a':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Pattern input is missing.\n");						
					else
						flags = ALL_TEST & ~DRAM_ECC;	// All tests except DRAM's ECC
					break;

		case 'h':
		default:	Memory_Help();
	}

	strcpy(opt, acL->argv[2]);

	ch0_first += QDR_CHAN0_BEGIN;
	ch0_last = ch0_first + size;

	ch1_first += QDR_CHAN1_BEGIN;
	ch1_last  = ch1_first + size;


	// *** check boundaries ***
	if(opt[1] == '0')
	{
		if(ch0_last > (QDR_CHAN0_BEGIN + sram_size[0]))
		{
			eprintf("***WARNING***: Testing beyond memory area!\n");
			eprintf("Testing channel 0: 0x%08x - 0x%08x\n", ch0_first, ch0_last);
		}
	}
	else if(opt[1] == '1')
	{
		if(ch1_last > (QDR_CHAN1_BEGIN + sram_size[1]))
		{
			eprintf("***WARNING***: Testing beyond memory area!\n");
			eprintf("Testing channel 1: 0x%08x - 0x%08x\n", ch1_first, ch1_last);
		}
	}
	else
	{
		if(ch0_last > (QDR_CHAN0_BEGIN + sram_size[0]))
		{
			ch0_last = QDR_CHAN0_BEGIN + sram_size[0];
			eprintf("Testing channel 0: 0x%08x - 0x%08x\n", ch0_first, ch0_last);
		}

		if(ch1_last > (QDR_CHAN1_BEGIN + sram_size[1]))
		{
			ch1_last = QDR_CHAN1_BEGIN + sram_size[1];
			eprintf("Testing channel 1: 0x%08x - 0x%08x\n", ch1_first, ch1_last);
		}
	}

	// Check which channel to run test
	if(flags != 0)
	{
		while(loops-- || forever)
		{
			if(t_all_verbose != MINIMAL)
				eprintf("Loop countdown:%d\n", loops);
			if(opt[1] == '0')
			{
				eprintf("SRAM Channel 0:\n");
				Test_Mem(QDR_CHAN0_BEGIN, sram_size[0],
						(PUINT)ch0_first, (PUINT)ch0_last, flags);
			}
			else if(opt[1] == '1')
			{
				eprintf("SRAM Channel 1:\n");
				Test_Mem(QDR_CHAN1_BEGIN, sram_size[1],
						(PUINT)ch1_first, (PUINT)ch1_last, flags);
			}
			else
			{
				if(sram_chan[0] == 0)
				{
					if(acL->HostType == MASTER)
					{
						eprintf("***ERROR***: SRAM Channel 0 not detected. SRAM FAIL!\n");
						acL->Err = ERROR;
					}
					else
					{
						eprintf("***WARNING***: SRAM Channel 0 not detected\n");
					}
				}
				else
				{
					eprintf("SRAM Channel 0:\n");
					Test_Mem(QDR_CHAN0_BEGIN, sram_size[0],
							(PUINT)ch0_first, (PUINT)ch0_last, flags);
					if(t_all_verbose == MINIMAL)
					{
						if(acL->Err != ERROR)
							eprintf("PASS\n");
					}
				}
				if(KEYPRESS)
				{
					return;
				}
				
				if(sram_chan[1] == 0)
				{
					eprintf("***ERROR***: SRAM Channel 1 not detected. SRAM FAIL!\n");
					acL->Err = ERROR;
				}
				else
				{
					eprintf("SRAM Channel 1:\n");
					Test_Mem(QDR_CHAN1_BEGIN, sram_size[1],
							(PUINT)ch1_first, (PUINT)ch1_last, flags);
					if(t_all_verbose == MINIMAL)
					{
						if(acL->Err != ERROR)
							eprintf("PASS\n");
					}
				}

				if(acL->Err == ERROR)
				{
					eprintf("SRAM TEST FAIL!!!\n");
					return;
				}
			}
			
			if(KEYPRESS)
			{
				return;
			}
		}
	}

	if(acL->Err == ERROR)
		eprintf("SRAM TEST FAIL!!!\n");

	return;
}

//
// Function: SDRAM_Test
//
// Description: 
//
static void SDRAM_Test(UINT mem_size, UINT first, UINT size, UINT loops)
{
	unsigned int last, flags = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	char opt[16];

	acL->Err = OK;
	strcpy(opt, acL->argv[3]);
	mem_type = SDRAM;

	switch(opt[0])
	{
		case 'w':	if(opt[1] == '1')		// Walking '1's
						flags = WALK_1;
					else if(opt[1] == '0')	// Walking '0's
						flags = WALK_0;
					else					// Both Walks
						flags = WALK;
					break;

		case 'c':	if(opt[1] == '5')		// Checker 0x5A5A5A5A
						flags = CHECKER_5A;
					else if(opt[1] == 'A')	// Checker 0xA5A5A5A5
						flags = CHECKER_A5;
					else					// Both Checkers
						flags = CHECKER;
					break;

		case 'u':	flags = UNIQUE;			// Unique Test
					break;

		case 'i':	flags = INCREMENTAL;	// Incremental Test
					break;

		case 'b':	flags = ADDR_BUS;		// Addr Bus Test
					break;

		case 'p':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Pattern input is missing.\n");						
					else
						flags = USER_DEFINE;// User defined pattern test;
					break;

		case 'e':	flags = DRAM_ECC;		// DRAM ECC test
					break;

		case 'm':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Number of pattern input is missing.\n");						
					else
						flags = MULTI_PATRN;
					break;

		case 'a':	if(acL->argc < 7)
						eprintf("Insufficient arguments. Pattern input is missing.\n");						
					else
						flags = ALL_TEST  & ~USER_DEFINE;	// All tests except User define;
					break;

		case 'h':
		default:	Memory_Help();
	}

	first += SDRAM_USER_SPACE_BEGIN;
	last = first + size;

	// check boundaries
	if(last > (SDRAM_BASE + mem_size))
	{
		eprintf("***ERROR***: SDRAM test address beyond memory size.");
		eprintf("Memory size: %d\n", mem_size);
		return;
	}

	if(flags != 0)
	{
		while(loops-- || forever)
		{
			Test_Mem(SDRAM_USER_SPACE_BEGIN, mem_size, (PUINT)first, (PUINT)last, flags);
			if(KEYPRESS)
				return;
		}
		if(acL->Err == ERROR)
			eprintf("SDRAM TEST FAIL!!!\n");
	}
	// burst: Sausalito has been set for 4 burst
}

//
// Function: Test_Mem
//
// Description: 
//
static int Test_Mem(UINT base, UINT mem_size, UINT* first, UINT* last, UINT flags)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int pattern, dcache_status;

	flush();	/* flush the input buffer */

	if(flags & WALK_1)
	{
		// Walking '1' Test
		Set_LED("WLK1");
		if(t_all_verbose != MINIMAL)
			eprintf("Walking '1' Test started...\n");
		switch(Walking_Test(base, mem_size, first, last, 1))
		{
			case ERROR:	eprintf("Walking '1' Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qw1F");
						else
							Set_LED("Dw1F");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Walking '1' Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qw1P");
						else
							Set_LED("Dw1P");
						break;

			case USER_ABORT:
						eprintf("Walking '1' Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}
	
	if(flags & WALK_0)
	{
		// Walking '0' Test
		Set_LED("WLK0");
		if(t_all_verbose != MINIMAL)
			eprintf("Walking '0' Test started...\n");
		switch(Walking_Test(base, mem_size, first, last, 0))
		{
			case ERROR:	eprintf("Walking '0' Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qw0F");
						else
							Set_LED("Dw0F");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Walking '0' Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qw0P");
						else
							Set_LED("Dw0P");
						break;

			case USER_ABORT:
						eprintf("Walking '0' Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}
	

	if(flags & CHECKER_5A)
	{
		// Checker 0x5A Test
		Set_LED("CK5A");
		if(t_all_verbose != MINIMAL)
			eprintf("Checker 0x5A5A5A5A Test started...\n");
		switch(Checker_Test(base, mem_size, first, last, 0x5A5A5A5A))
		{
			case ERROR:	eprintf("Checker 0x5A5A5A5A Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qc5F");
						else
							Set_LED("Dc5F");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Checker 0x5A5A5A5A Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qc5P");
						else
							Set_LED("Dc5P");
						break;

			case USER_ABORT:
						eprintf("Checker 0x5A5A5A5A Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}

	if(flags & CHECKER_A5)
	{
		// Checker 0xA5 Test
		Set_LED("CKA5");
		if(t_all_verbose != MINIMAL)
			eprintf("Checker 0xA5A5A5A5 Test started...\n");
		switch(Checker_Test(base, mem_size, first, last, 0xA5A5A5A5))
		{
			case ERROR:	eprintf("Checker 0xA5A5A5A5 Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("QcaF");
						else
							Set_LED("DcaF");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Checker 0xA5A5A5A5 Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("QcaP");
						else
							Set_LED("DcaP");
						break;

			case USER_ABORT:
						eprintf("Checker 0xA5A5A5A5 Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}
	
	if(flags & UNIQUE)
	{
		// Unique Test
		Set_LED("UNIQ");
		if(t_all_verbose != MINIMAL)
			eprintf("Unique Test started...\n");
		switch(Unique_Test(base, mem_size, first, last))
		{
			case ERROR:	eprintf("Unique Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qu F");
						else
							Set_LED("Du F");
						acL->Err = ERROR;
						return ERROR;
			
			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Unique Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qu P");
						else
							Set_LED("Du P");
						break;

			case USER_ABORT:
						eprintf("Unique Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}

	if(flags & INCREMENTAL)
	{
		// Incremental Test
		Set_LED("INCR");
		if(t_all_verbose != MINIMAL)
			eprintf("Incremental Test started...\n");
		switch(Incremental_Test(base, mem_size, first, last))
		{
			case ERROR:	eprintf("Incremental Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qi F");
						else
							Set_LED("Di F");
						acL->Err = ERROR;
						return ERROR;
			
			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Incremental Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qi P");
						else
							Set_LED("Di P");
						break;

			case USER_ABORT:
						eprintf("Incremental Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}
	
	if(flags & ADDR_BUS)
	{
		// Walking Address Bus Test
		Set_LED("WLKA");
		if(t_all_verbose != MINIMAL)
			eprintf("Walking Address Bus Test stared...\n");
		switch(Walking_AddrBus_Test(base, mem_size))
		{
			case ERROR:	eprintf("Walking Address Bus Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("Qb F");
						else
							Set_LED("Db P");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("Walking Address Bus Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("Qb P");
						else
							Set_LED("Db P");
						break;

			case USER_ABORT:
						eprintf("Walking Address Bus Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}

	if(flags & USER_DEFINE)
	{
		// User Defined Pattern Test
		pattern = acL->hexArg[6];
		Set_LED("USER");
		if(t_all_verbose != MINIMAL)
			eprintf("User Defined Pattern Test started...\n");
		switch(Checker_Test(base, mem_size, first, last, pattern))
		{
			case ERROR:	eprintf("User Defined Pattern Test... FAIL!\n");
						if(mem_type == SRAM)
							Set_LED("QudF");
						else
							Set_LED("DudF");
						acL->Err = ERROR;
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("User Defined Pattern Test... PASS\n");
						if(mem_type == SRAM)
							Set_LED("QudP");
						else
							Set_LED("DudP");
						break;

			case USER_ABORT:
						eprintf("User Defined Pattern Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
	}

	if(flags & DRAM_ECC)
	{
		// SDRAM ECC Test
		Set_LED("ECC ");
		if(t_all_verbose != MINIMAL)
			eprintf("SDRAM ECC Test started...\n");
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)
			dcache_off();

		switch(SDRAM_ECC_Test(first))
		{
			case ERROR:	eprintf("SDRAM ECC Test... FAIL!\n");	
						Set_LED("De F");
						acL->Err = ERROR;
						if(dcache_status == 1)
							dcache_on();
						return ERROR;

			case OK:	if(t_all_verbose != MINIMAL)
							eprintf("SDRAM ECC Test... PASS\n");
						Set_LED("De P");
						break;

			case USER_ABORT:
						eprintf("SDRAM ECC Test... ABORTED\n");
						Set_LED("ABRT");
						break;
		}
		if(dcache_status != 0)
			dcache_on();
		while( (GET32(DU_ERROR_STATUS_1_REG) & 0x3) != 0)
		{
			PUT32(DU_ERROR_STATUS_1_REG, 0x3);	// Clears ECC errors
			PUT32(DU_ECC_TEST, 0x0);
		}
	}

	return OK;
}

//
// Function: Walking_Test
//
// Description: 
//
static int Walking_Test(UINT base, UINT mem_size, PUINT first, PUINT last, UINT pattern)
{
	// Note: This test does not test every location but every 64 LW.
	PUINT p;
	UINT  v,d,longword,z,t, status = OK;
  
	d = 1; p = first; longword = 0;z = 1;

	for (v = 0; v < 32; v++)
	{
		longword = 0;
		if(verbose == ON)
		{
			eprintf("Walking %d's Pattern (d-stream), pattern %08x  %c",
	       			pattern, pattern?d:~d&0xFFFFFFFF,(verbose > 1)?'\n':'\r');
		}
		if (KEYPRESS) return USER_ABORT;
		while (&p[longword]  < last)
		{
	  		if(verbose == ON)
			{
	    		if(!(longword%0x4000))
	      			eprintf("writing address 0x%08x with 0x%08x\r",
	            			(UINT)&p[longword], pattern?d:(UINT)(~d&0xFFFFFFFF));
			}
	  		p[longword] = pattern?d:((UINT)(~d&0xffffffff));
	  		longword+=64;
		}
		if (KEYPRESS) return USER_ABORT;

		longword = 0;
		while (&p[longword] < last)
		{
	  		if(verbose == ON)
			{
	    		if(!(longword%0x4000))
					eprintf("reading address 0x%08x                \r", (UINT)&p[longword]);
	  		}
	  		if((t=p[longword]) != (pattern?d:((UINT)(~d&0xffffffff))))
			{
				status = ERROR;
	    		eprintf("****ERROR***: addr: %08x: actual: %08x, exp: %08x\n",
	           			(UINT)&p[longword], t, pattern?d:(UINT)(~d&0xFFFFFFFF));

				// prepare structure for dumping
				syslog.type = TYPE_ERROR;
				syslog.source_comp = MASTER_DIAG;
				if(mem_type == SRAM)
				{
					if( (base + mem_size/2) > (UINT)&p[longword])
					{
						// bank 1
						if( (t & 0xFF) != (d & 0xFF) )
							eprintf("Failure in Chip 1\n");
						if( ((t>>8) & 0xFF) != ((d>>8) & 0xFF) )
							eprintf("Failure in Chip 2\n");
						if( ((t>>16) & 0xFF) != ((d>>16) & 0xFF) )
							eprintf("Failure in Chip 1\n");
						if( ((t>>24) & 0xFF) != ((d>>24) & 0xFF) )
							eprintf("Failure in Chip 2\n");
					}
					else
					{
						// bank 2
						if( (t & 0xFF) != (d & 0xFF) )
							eprintf("Failure in Chip 3\n");
						if( ((t>>8) & 0xFF) != ((d>>8) & 0xFF) )
							eprintf("Failure in Chip 4\n");
						if( ((t>>16) & 0xFF) != ((d>>16) & 0xFF) )
							eprintf("Failure in Chip 3\n");
						if( ((t>>24) & 0xFF) != ((d>>24) & 0xFF) )
							eprintf("Failure in Chip 4\n");
					}
					if( ((PUINT)(base + mem_size/2) < last) && ((UINT)&p[longword] < (base + mem_size/2)) )
					{
						p = (PUINT)(base + mem_size/2);
						longword = 0;
					}
					else
					{
						strcpy(syslog.desc, "SRAM Checker Test FAIL!");
						// write to syslog
						syslog_dump(&syslog, sizeof(SYSLOG_DATA));
						return status;
					}
				}
				else	//SDRAM
					strcpy(syslog.desc, "SDRAM Walking Test FAIL!");

				// write to syslog
				syslog_dump(&syslog, sizeof(SYSLOG_DATA));

				return status;
			}
	  		longword+=64;
		}
		d = d << 1;
	}
	eprintf("                                                   \r");
	return status;
}

//
// Function: Checker_Test
//
// Description: 
//
static int Checker_Test(UINT base, UINT mem_size, PUINT first, PUINT last, UINT seed)
{
	PUINT p;
	UINT  d,t;
	int n, x, status = OK;

	if(verbose == ON)
		eprintf("Pattern: 0x%x\n");
	x = n = 0;d = seed;p = first;

	while (&p[x] < last)
	{
		if (verbose == ON)
			eprintf("writing address 0x%08x\r", (UINT)&p[x], d);
		else
		{
			if(!(x%SIGNAL))
			{
				eprintf(".");
			}
		}
		if (KEYPRESS) return USER_ABORT;
      	
		p[x] = d;
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
		if ((n&3) != 3)
			d = ~d;		
    
    	++n;
	}

	eprintf("\r");

	x = n = 0;
	d = seed;
	while (&p[x] < last)
	{
		if (verbose == ON)
			eprintf("reading address 0x%08x\r", (UINT)&p[x]);
		else
		{
			if(!(x%SIGNAL))
			{
				eprintf(".");
			}
		}
		if (KEYPRESS) return USER_ABORT;
		
		if ((t=p[x]) != d)
		{
			eprintf("\n***ERROR***: addr: %08x actual: %08x exp: %08x\n", (UINT)&p[x], t ,d);
			status = ERROR;

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			if(mem_type == SRAM)
			{
				if( (base + mem_size/2) > (UINT)&p[x])
				{
					// bank 1
					if( (t & 0xFF) != (d & 0xFF) )
						eprintf("Failure in Chip 1\n");
					if( ((t>>8) & 0xFF) != ((d>>8) & 0xFF) )
						eprintf("Failure in Chip 2\n");
					if( ((t>>16) & 0xFF) != ((d>>16) & 0xFF) )
						eprintf("Failure in Chip 1\n");
					if( ((t>>24) & 0xFF) != ((d>>24) & 0xFF) )
						eprintf("Failure in Chip 2\n");
				}
				else
				{
					// bank 2
					if( (t & 0xFF) != (d & 0xFF) )
						eprintf("Failure in Chip 3\n");
					if( ((t>>8) & 0xFF) != ((d>>8) & 0xFF) )
						eprintf("Failure in Chip 4\n");
					if( ((t>>16) & 0xFF) != ((d>>16) & 0xFF) )
						eprintf("Failure in Chip 3\n");
					if( ((t>>24) & 0xFF) != ((d>>24) & 0xFF) )
						eprintf("Failure in Chip 4\n");
				}
				if( ((PUINT)(base + mem_size/2) < last) && ((UINT)&p[x] < (base + mem_size/2)) )
				{
					p = (PUINT)(base + mem_size/2);
					x = 0;
				}
				else
				{
					strcpy(syslog.desc, "SRAM Checker Test FAIL!");
					// write to syslog
					syslog_dump(&syslog, sizeof(SYSLOG_DATA));
					return status;
				}
			}
			else	//SDRAM
			{
				strcpy(syslog.desc, "SDRAM Checker Test FAIL!");
				// write to syslog
				syslog_dump(&syslog, sizeof(SYSLOG_DATA));
				return status;
			}
	    }
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
	    if ((n&3) != 3)
			d = ~d;    

    	++n;
	}

	eprintf("                                                 \r");
	return status;
}

//
// Function: Unique_Test
//
// Description: 
//
static int Unique_Test(UINT base, UINT mem_size, PUINT first, PUINT last)
{
	register PUINT p;
	UINT t;
	int x;

	if(verbose == ON)
		eprintf("Address as Data Pattern\n");
	p = first;
	x = 0;
	while (&p[x] < last)
	{
		if (!(x%SIGNAL))
		{
			if (verbose == ON)
				eprintf("writing address 0x%08x\r",(UINT)&p[x], (UINT)&p[x]);
			else
			{
				eprintf(".");
			}

			if (KEYPRESS) return USER_ABORT;
		}
		p[x] = (UINT) &p[x];
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
	}

	eprintf("\r");

	x = 0;
	while (&p[x] < last)
	{
		if (!(x%SIGNAL))
		{
			if (verbose == ON)
				eprintf("reading address 0x%08x\r", (UINT)&p[x]);
			else
			{
				eprintf(".");
			}

			if (KEYPRESS) return USER_ABORT;
		}
		if ((t=p[x]) != (UINT) &p[x])
		{
			eprintf("\n***ERROR***: addr: %08x actual: %08x exp: %08x\n", (UINT)&p[x], t, (UINT)&p[x]);

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			if(mem_type == SRAM)
				strcpy(syslog.desc, "SRAM Unique Test FAIL!");
			else	//SDRAM
				strcpy(syslog.desc, "SDRAM Unique Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			
			return ERROR;
		}
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
	}
	eprintf("                                                 \r");
	return OK;
}

//
// Function: Walking_AddrBus_Test
//
// Description: Checks that none of the Address pins
//				are tied together
//
static int Walking_AddrBus_Test(UINT user_base, UINT size)
{

	PUINT p = (PUINT) user_base;
	int x = 0, test;
	UINT base;
	if(mem_type == SRAM)
		base = user_base;
	else
		base = SDRAM_BASE;

	while ((UINT)&p[1<<(x+2)] < (base + size))
	{
		if(KEYPRESS) return USER_ABORT;
	
		if((1<<x) >= (user_base/4))
			p = (PUINT) 0;
		p[1<<(x+1)] = 0x1;
		p[1<<x] = 0xA5A5A5A5;		
		p[1<<(x+2)] = ~(0x0);

		if(verbose == ON)
			eprintf("Writing to address %08x, %08x, %08x\n",
					&p[1<<x], &p[1<<(x+1)], &p[1<<(x+2)]);

		test = p[1<<(x+1)];
		if(test == 0xA5A5A5A5)	// Addr line A[x] and A[x+1] are linked
		{
			eprintf("***Error***: at address lines %d and %d\n", x, x+1);
			
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			if(mem_type == SRAM)
				strcpy(syslog.desc, "SRAM Walking AddrBus Test FAIL!");
			else	//SDRAM
				strcpy(syslog.desc, "SDRAM Walking AddrBus Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			return ERROR;
		}
		else if(test == ~(0x0))	// Addr line A[x+1] and A[x+2] are linked
		{
			eprintf("***Error***: at address lines %d and %d\n", x+1, x+2);

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			if(mem_type == SRAM)
				strcpy(syslog.desc, "SRAM Walking AddrBus Test FAIL!");
			else	//SDRAM
				strcpy(syslog.desc, "SDRAM Walking AddrBus Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			return ERROR;
		}
		x++;
	}
	return OK;
}


//
// Function: SDRAM_ECC_TEST
//
// Description: Checks that no ECC error is generated when there are no
//				ECC errors forced.
//				Ensures only a correctable ECC error is generated when a
//				forced correctable ECC error is generated.
//				Ensures only an uncorrectable ECC error is generated when a
//				forced uncorrectable ECC error is generated.
//
static int SDRAM_ECC_Test(PUINT first)
{
	int status = OK;
	register int ecc_counter;
	int temp = 0;
	char answer = 'n';
	UINT x = 0;
	volatile UINT du_err_stat_1;
	PULONG p;
	int rd_back_csr;

	if(KEYPRESS) return USER_ABORT;

	p = (PULONG)first;

	// Accesses the DU_ECC_TEST Register to force an ECC error.
	// The ECC error can only be generated when written into memory
	// Assumes ECC is enabled

	PUT32(DU_ERROR_STATUS_1_REG, 0x3);	// Clears any previous ECC errors

	// Write and read back data from SDRAM memory
	p[x] = 0xDEADBEEFBABECAFE;
	hal_delay_us(25000);
	
	du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);

	// Ensure that no ECC error has been generated since the
	// error injection has not been enabled yet 
	if( (du_err_stat_1 & 0x3) != 0)
	{
		eprintf("***Error***: ECC Error detected when ECC error is not forced\n");

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;
		strcpy(syslog.desc, "SDRAM S0 ECC Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		status = ERROR;
	}

	
	// Force correctable error and check for it
	//==========================================
	PUT32(DU_ECC_TEST, 0x1);
	rd_back_csr = GET32(DU_ECC_TEST);
	if(rd_back_csr != 1)
		eprintf("DRAM CSR is not updated\n");
	
	// Write and read back data from SDRAM memory
	// The write should generate a correctable ECC error
	p[x] = 0xBABECAFEDEADBEEF;
	ecc_counter = 0xFFFF;
	while(ecc_counter--)
		temp++;

	PUT32(DU_ECC_TEST, 0x0);

	du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);

	// Check for a correctable ECC error to be generated.
	if( (du_err_stat_1 & 0x1) == 0x1)	// Check for correctable error
	{
		if(verbose == ON)
			eprintf("Address with forced ECC correctable error %08x\n",
					(du_err_stat_1)&0xFFFFFFF8);
	}
	else
	{
		eprintf("***Error***: Forced ECC Correctable Error not detected\n");

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;
		strcpy(syslog.desc, "SDRAM S1 ECC Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		status = ERROR;
	}

	// Ensure that an uncorrectable error is not generated
	if( (du_err_stat_1 & 0x2) != 0x0)	// Check for uncorrectable error
	{
		eprintf("***Error***: Forced ECC Correctable Error but Uncorrectable detected\n");

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;
		strcpy(syslog.desc, "SDRAM S2 ECC Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		status = ERROR;
	}

	// Check and clears all ECC erros collected
	du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);
	while((du_err_stat_1 & 0x3) != 0)
	{
		PUT32(DU_ECC_TEST, 0x0);
		hal_delay_us(1000);
		PUT32(DU_ERROR_STATUS_1_REG, 0x3);	// Clears ECC errors
		hal_delay_us(1000);
		du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);
	}

	p[x] = 0xBABECAFEDEADBEEF;
	hal_delay_us(25000);


	// Force uncorrectable error and check for it
	//============================================
	if(t_all_verbose != MINIMAL)
	{
		eprintf("***Warning: Forcing an Uncorrectable ECC error causes errors on specified address.\n");
		eprintf("Do you want to continue? ('y' to continue)\n");
		hal_delay_us(5000000);
		if(!KEYPRESS)
			return status;
		answer = GetChar();
	}

	if(answer == 'y')
	{
		PUT32(DU_ECC_TEST, 0x3);
		rd_back_csr = GET32(DU_ECC_TEST);
		if(rd_back_csr != 3)
			eprintf("DRAM CSR is not updated\n");

		// Write and read back data from SDRAM memory
		// The write should generate an uncorrectable ECC error

		p[x+1] = 0xDEADBEEFBABECAFE;
		ecc_counter = 0xFFFF;
		while(ecc_counter--)
			temp++;
		PUT32(DU_ECC_TEST, 0x0);
		
		du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);
		// Check for an uncorrectable ECC error to be generated.
		if( (du_err_stat_1 & 0x2) == 0x2)
		{
			eprintf("Address with forced ECC uncorrectable error %08x\n",
					(du_err_stat_1)&0xFFFFFFF8);
		}
		else
		{
			eprintf("***Error***: Forced ECC Uncorrectable Error not detected\n");

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			strcpy(syslog.desc, "SDRAM S3 ECC Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			status = ERROR;
		}

		// Ensure that a correctable error is not generated
		if( (du_err_stat_1 & 0x1) != 0x0)	// Check for correctable error
		{
			eprintf("***Error***: Forced ECC Uncorrectable Error but Correctable detected\n");

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			strcpy(syslog.desc, "SDRAM S4 ECC Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			status = ERROR;
		}
		PUT32(DU_ECC_TEST, 0x0);	// Disable forcing of ECC errors
		PUT32(DU_ERROR_STATUS_1_REG, 0x3);	// Clears ECC errors

		// Check and clears all ECC erros collected
		du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);
		while((du_err_stat_1 & 0x3) != 0)
		{
			PUT32(DU_ECC_TEST, 0x0);
			hal_delay_us(1000);
			PUT32(DU_ERROR_STATUS_1_REG, 0x3);	// Clears ECC errors
			hal_delay_us(1000);
			du_err_stat_1 = GET32(DU_ERROR_STATUS_1_REG);
		}
	}
	
	return status;
}

//
// Function: Incremental_Test
//
// Description: 
//
static int Incremental_Test(UINT base, UINT mem_size, PUINT first, PUINT last)
{
	register PUINT p;
	UINT t;
	int x;
	UINT test_counter = 0;

	if(verbose == ON)
		eprintf("Incremented Data Pattern  (d-stream)\n");

	p = first;
	x = 0;
	test_counter = counter;
	while (&p[x] < last)
	{
		if (!(x%SIGNAL))
		{
			if (verbose == ON)
				eprintf("writing address %08x with %08x \r",(UINT)&p[x], counter);
			else
			{
				eprintf(".");
			}

			if (KEYPRESS) return USER_ABORT;
		}
		p[x] = counter++;
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
	}

	eprintf("\r");

	x = 0;
	while (&p[x] < last)
	{
		if (!(x%SIGNAL))
		{
			if (verbose == ON)
				eprintf("reading address %08x \r", (UINT)&p[x]);
			else
			{
				eprintf(".");
			}

			if (KEYPRESS) return USER_ABORT;
		}
		if ((t=p[x]) != test_counter)
		{
			eprintf("\n***ERROR***: addr: %08x actual: %08x exp: %08x\n", (UINT)&p[x], t, test_counter);

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;
			if(mem_type == SRAM)
				strcpy(syslog.desc, "SRAM Unique Test FAIL!");
			else	//SDRAM
				strcpy(syslog.desc, "SDRAM Unique Test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			
			return ERROR;
		}
		test_counter++;
		if(mem_type == SRAM)
			x++;
		else
			x+=16;
	}
	eprintf("                                                 \r");
	return OK;
}

