/* microengine.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Mar 07, 02
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "microengine.h"
#include "diag_utils.h"
#include "led.h"
#include "hal_ixdp2400.h"
#include "uc_load.h"
#include "syslog.h"
#include "uart.h"
#include "me_xscale_sync.h"
// microcode
#include "./microcode/ME_4k_Control_Store.h"
#include "./microcode/ME_Register_Test.h"
#include "./microcode/ME_Context_Test.h"
#include "./microcode/ME_Timers_Counters_Test.h"



// static function prototypes
static void ME_Help(void);
static void ME_ControlStore_Test(void);
static void ME_Register_Test(void);
static void ME_Context_Test(void);
static void ME_Timers_Cntrs_Test(void);

static unsigned int ueng_number;

//
// Function: ME_Help
//
// Description: Displays the options for the Microengine Test
//
static void ME_Help(void)
{
 	eprintf("> t ueng <option> <d:microengine #> <d:loops>,\n");
	eprintf("where <option> are\n");
	eprintf("c - Control Store test\n");
	eprintf("r - Register test\n");
	eprintf("t - Timers and Counters test\n");
	eprintf("s - Context test\n");
	eprintf("h - help\n");
	eprintf("a - all tests\n");
	eprintf("NOTE: arg \"microengine #\" is 0-3, 16-19 and default are all microengine\n");
}

//
// Function: ME_Test
//
// Description: Calls the appropriate function(s) from user input
//
void ME_Test(void)
{
	char opt;
	unsigned int loops = 1; //number of times the test(s) is to be repeated
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned int forever;

	acL->Err = OK;
	forever = 0;

	opt = acL->argv[2][0];
	if(acL->argc < 4)
	{
		ueng_number = ALL_UENG;
	}
	else
	{
		ueng_number = acL->decArg[3];
		if( ((ueng_number < 0) || (ueng_number > 19)) ||
			((ueng_number > 3) && (ueng_number < 16)) )
		{
			ueng_number = ALL_UENG;
		}
		
		if(acL->argc < 5)
			loops = 1;
		else
		{
			loops = acL->decArg[4];
			if(loops == 0)
				forever = 1;
		}
	}

	switch(opt)
	{
		case 'c':	while(loops-- || forever)
                    {
						ME_ControlStore_Test();
                        if(KEYPRESS)
			                break;
                    }
					break;

		case 'r':	while(loops-- || forever)
                    {
						ME_Register_Test();
                        if(KEYPRESS)
			                break;
                    }
					break;

		case 't':	while(loops-- || forever)
                    {
						ME_Timers_Cntrs_Test();
                        if(KEYPRESS)
			                break;
                    }
					break;

		case 's':	while(loops-- || forever)
                    {
						ME_Context_Test();
                        if(KEYPRESS)
			                break;
                    }
					break;

		case 'a':	while(loops-- || forever)
					{
						ME_ControlStore_Test();
						ME_Register_Test();
						ME_Timers_Cntrs_Test();
						ME_Context_Test();
                        if(KEYPRESS)
			                break;
					}
					break;

		case 'h':
		default:	ME_Help();
	}

	if(acL->Err == ERROR)
		eprintf("MICROENGINE TEST FAIL!!!\n");
}

//
// Function: ME_ControlStore_Test
//
// Desciption: This function will test the micro-engine control store. The ME
//			  control store is similar to SRAM which contains ME code of
//			  instruction and data for execution.  It will load 4K byte of
//			  instruction and verify if the loading is successful
//
static void ME_ControlStore_Test(void)
{
	int ME_num, status = OK;
	volatile int test;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// Initialize space in scratch used for ME to pass status to XScale
	for(ME_num = ME0; ME_num <= ME7; ME_num++)
		PUT32(SCRATCHPAD_BASE + ((1 + ME_num)<<B_TO_LW_SHF), 0);		// 1 is just an offset

	if(ueng_number == ALL_UENG)
	{
		DownLoadUcode(0,  sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(1,  sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(2,  sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(3,  sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(16, sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(17, sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(18, sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);
		DownLoadUcode(19, sizeof(ME_4k_Control_Store_ME), (unsigned int *)ME_4k_Control_Store_ME);

		hal_delay_us(10000);

		for(ME_num = ME0; ME_num <= ME7; ME_num++)
		{
			test = GET32(SCRATCHPAD_BASE + ((1 + ME_num)<<B_TO_LW_SHF));	// 1 is just an offset
			if(test != CONTROL_STORE_EXP_VAL)	// value expected from microengine from its calculation
			{
				eprintf("ME %d: Control Store Test FAIL! Value Read = %x\n", ME_num, test);
				status = ERROR;
			}
		}
	}
	else
	{
		DownLoadUcode(ueng_number, sizeof(ME_4k_Control_Store_ME),
					 (unsigned int *)ME_4k_Control_Store_ME);
		hal_delay_us(10000);

		ME_num = (ueng_number & 0x3) | ((ueng_number>>2) & 0x4);

		test = GET32(SCRATCHPAD_BASE + ((1 + ME_num)<<B_TO_LW_SHF));	// 1 is just an offset
		if(test != CONTROL_STORE_EXP_VAL)	// value expected from microengine from its calculation
		{
			eprintf("ME %d: Control Store Test FAIL! Value Read = %x\n", ME_num, test);
			status = ERROR;
		}
	}

	eprintf("ME Control Store Test... ");
	if(status == OK)
	{
		eprintf("PASS.\n");
		Set_LED("MEcP");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n\n");
		Set_LED("MEcF");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "ME Control Store Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}
	Clear_All_ME();
}

//
// Function: ME_Register_Test
//
// Description: To test the following registers:
//				- GPR
//				- Transfer Registers
//				- Next Neighbour Registers
//				- Local Memory Registers
//
static void ME_Register_Test(void)
{
	int location, thread_num, ME_num = 0, test, status = OK, thd_start = 0, thd_end = 64;
	int thread_fail = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 320; location++)
		PUT32(SCRATCHPAD_BASE + (location<<B_TO_LW_SHF), 0);

	if(ueng_number == ALL_UENG)
	{
		DownLoadUcode(19, sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(18, sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(17, sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(16, sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(3,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(2,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(1,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
		DownLoadUcode(0,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	}
	else
	{
		DownLoadUcode(ueng_number, sizeof(ME_Register_Test_ME),
					 (unsigned int *)ME_Register_Test_ME);
		ME_num = (ueng_number & 0x3) | ((ueng_number>>2) & 0x4);
		thd_start = ME_num * 8;
		thd_end = thd_start + 8;
	}

	hal_delay_us(100000);

	// Verify Xfer Reg Test (scratch location 0 - 127 (Lword))
	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
	{
		test = GET32(SCRATCHPAD_BASE + (((thread_num)*2)<<B_TO_LW_SHF));
		if(test != COMPLETE) // if test did not complete
		{
			thread_fail |= 1;
			status = ERROR;
		}
		else // test completed
		{
			test = GET32(SCRATCHPAD_BASE + (((thread_num)*2 + 1)<<B_TO_LW_SHF));
			if(test != OK)
			{
				thread_fail |= 2;
				status = ERROR;
			}
		}
		if( ((thread_num + 1)%8) == 0)
		{
			if(thread_fail != 0)
				eprintf("Xfer Reg Test for ME%d FAIL!\n", thread_num/8);
			thread_fail = 0;
		}
	}
	
/* If required to identify by which thread instead of by microengine
	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
	{
		test = GET32(SCRATCHPAD_BASE + (((thread_num)*2)<<B_TO_LW_SHF));
		if(test != COMPLETE) // if test did not complete
		{
			eprintf("Xfer Reg Test for thread %d did not complete.\n", thread_num);
			status = ERROR;
		}
		else // test completed
		{
			test = GET32(SCRATCHPAD_BASE + (((thread_num)*2 + 1)<<B_TO_LW_SHF));
			if(test != OK)
			{
				eprintf("Xfer Reg Test for thread %d FAIL with %x\n", thread_num, test);
				status = ERROR;
			}
		}
	}
*/
	// Verify GPR Test (scratch location 128 - 255 (Lword))
	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
	{
		// GPR test location offset = 128
		test = GET32(SCRATCHPAD_BASE + ((128 + (thread_num)*2)<<B_TO_LW_SHF));
		if(test != COMPLETE) // if test completed
		{
			thread_fail |= 1;
			status = ERROR;
		}
		else // test completed
		{
			test = GET32(SCRATCHPAD_BASE + ((128 + (thread_num)*2 + 1)<<B_TO_LW_SHF));
			if(test != OK)
			{
				thread_fail |= 2;
				status = ERROR;
			}
		}
		if( ((thread_num + 1)%8) == 0)
		{
			if(thread_fail != 0)
				eprintf("GPR Test for ME%d FAIL!\n", thread_num/8);
			thread_fail = 0;
		}
	}

/* If required to identify by which thread instead of by microengine
	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
	{
		// GPR test location offset = 128
		test = GET32(SCRATCHPAD_BASE + ((128 + (thread_num)*2)<<B_TO_LW_SHF));
		if(test != COMPLETE) // if test completed
		{
			eprintf("GPR Test for thread %d did not complete.\n", thread_num);
			status = ERROR;
		}
		else // test completed
		{
			test = GET32(SCRATCHPAD_BASE + ((128 + (thread_num)*2 + 1)<<B_TO_LW_SHF));
			if(test != OK)
			{
				eprintf("GPR Test for thread %d FAIL with %x\n", thread_num, test);
				status = ERROR;
			}
		}
	}
*/

	if(status != ERROR)
	{
		if(ueng_number == ALL_UENG)
		{
			// Verify Local Mem Addr(LMA) Test (scratch location 256 - 271 (Lword))
			for(ME_num = ME0; ME_num <= ME7; ME_num++)
			{
				// LMA test location offset = 256
				test = GET32(SCRATCHPAD_BASE + ((256 + ME_num*2)<<B_TO_LW_SHF));
				if(test != COMPLETE)
				{
					eprintf("Local Memory Test for ME %d did not complete.\n", ME_num);
					status = ERROR;
				}
				else
				{
					test = GET32(SCRATCHPAD_BASE + ((256 + ME_num*2 + 1)<<B_TO_LW_SHF));
					if(test != OK)
					{
						eprintf("Local Memory Test for ME %d FAIL! Failing value = 0x%x\n", ME_num, test);
						status = ERROR;
					}
				}
			}


			// Verify Next Neighbor(NN) Int Reg Test (scratch location 288 - 303 (Lword))
			for(ME_num = ME0; ME_num <= ME7; ME_num++)
			{
				// NN Reg Int test location offset = 288
				test = GET32(SCRATCHPAD_BASE + ((288 + (ME_num)*2)<<B_TO_LW_SHF));
				if(test != COMPLETE)
				{
					eprintf("Next Neighbor Int Test for ME%d did not complete.\n", ME_num);
					status = ERROR;
				}
				else
				{
					test = GET32(SCRATCHPAD_BASE + ((288 + (ME_num)*2 + 1)<<B_TO_LW_SHF));
					if(test != OK)
					{
						eprintf("Next Neighbor Int Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
						status = ERROR;
					}
				}
			}

			// Verify Next Neighbor(NN) Ext Reg Test (scratch location 304 - 319 (Lword))
			for(ME_num = ME0; ME_num <= ME7; ME_num++)
			{
				// NN Reg Ext test location offset = 304
				test = GET32(SCRATCHPAD_BASE + ((304 + (ME_num)*2)<<B_TO_LW_SHF));
				if(test != COMPLETE)
				{
					eprintf("Next Neighbor Ext Test for ME%d did not complete.\n", ME_num);
					status = ERROR;
				}
				else
				{
					test = GET32(SCRATCHPAD_BASE + ((304 + (ME_num)*2 + 1)<<B_TO_LW_SHF));
					if(test != OK)
					{
						eprintf("Next Neighbor Ext Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
						status = ERROR;
					}
				}
			}
		}

		else
		{
			// Verify Local Mem Addr(LMA) Test (scratch location 256 - 271 (Lword))
			// LMA test location offset = 256
			test = GET32(SCRATCHPAD_BASE + ((256 + ME_num*2)<<B_TO_LW_SHF));
			if(test != COMPLETE)
			{
				eprintf("Local Memory Test for ME %d did not complete.\n", ME_num);
				status = ERROR;
			}
			else
			{
				test = GET32(SCRATCHPAD_BASE + ((256 + ME_num*2 + 1)<<B_TO_LW_SHF));
				if(test != OK)
				{
					eprintf("Local Memory Test for ME %d FAIL! Failing value = 0x%x\n", ME_num, test);
					status = ERROR;
				}
			}


			// Verify Next Neighbor(NN) Int Reg Test (scratch location 288 - 303 (Lword))
			// NN Reg Int test location offset = 288
			test = GET32(SCRATCHPAD_BASE + ((288 + (ME_num)*2)<<B_TO_LW_SHF));
			if(test != COMPLETE)
			{
				eprintf("Next Neighbor Int Test for ME%d did not complete.\n", ME_num);
				status = ERROR;
			}
			else
			{
				test = GET32(SCRATCHPAD_BASE + ((288 + (ME_num)*2 + 1)<<B_TO_LW_SHF));
				if(test != OK)
				{
					eprintf("Next Neighbor Int Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
					status = ERROR;
				}
			}

			// Verify Next Neighbor(NN) Ext Reg Test not performed
			eprintf("***NOTE***: Next Neighbor Ext Test not tested in single ME test\n");
		}
	}

	eprintf("ME Register Test... ");
	if(status == OK)
	{
		eprintf("PASS.\n");
		Set_LED("MErP");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n\n");
		Set_LED("MErF");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "ME Register Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
		acL->Err = ERROR;
	}

	Clear_All_ME();
}

//
// Function: ME_Context_Test
//
// Description: This function will test the micro-engine hardware contexts. 
//			  	There are 8 hardware contexts in the micro-engine.  All 4 states
//			  	of each context -- inactive, executing,	ready and sleep will
//				be verified though tests have been written for inactive and
//				sleep only but the other two are always tested.
//
static void ME_Context_Test(void)
{
	int location, test, ME_num, status = OK;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 32; location++)
		PUT32(SCRATCHPAD_BASE + (location<<B_TO_LW_SHF), 0);

	if(ueng_number == ALL_UENG)
	{
		DownLoadUcode(19, sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(18, sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(17, sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(16, sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(3,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(2,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(1,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
		DownLoadUcode(0,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);

		hal_delay_us(10000);

		// Verify Inactive State Test (scratch location 0 - 15 (Lword))
		for(ME_num = ME0; ME_num <= ME7; ME_num++)
		{
			test = GET32(SCRATCHPAD_BASE + ((ME_num*2)<<B_TO_LW_SHF));
			if(test == COMPLETE)	// indication that Inactive State Test completed
			{
				test = GET32(SCRATCHPAD_BASE + ((ME_num*2 + 1)<<B_TO_LW_SHF));
				if(test != PASS)
				{
					eprintf("ME %d FAIL Inactive State Test.\n", ME_num);
					status = ERROR;
				}
			}
			else
			{
				eprintf("ME %d FAIL to complete Inactive State Test.\n", ME_num);
				status = ERROR;
			}
		}

		// Verify Sleep State Test (scratch location 16 - 31 (Lword))
		for(ME_num = ME0; ME_num <= ME7; ME_num++)
		{ 
			test = GET32(SCRATCHPAD_BASE + ((16 + ME_num*2)<<B_TO_LW_SHF));
			if(test == COMPLETE)	// indication that Sleep State Test completed
			{
				test = GET32(SCRATCHPAD_BASE + ((16 + ME_num*2 + 1)<<B_TO_LW_SHF));
				if(test != PASS)
				{
					eprintf("ME %d FAIL Sleep State Test.", ME_num);
					eprintf("Stopped at thread %d\n", (test&0x7));
					status = ERROR;
				}
			}
			// Check if one of the thd did not sleep
			else if(test == THD_DID_NOT_SLEEP)
			{
				eprintf("ME %d thds did not go to sleep.\n", ME_num);
				status = ERROR;
			}
			else
			{
				eprintf("ME %d FAIL to complete Sleep State Test.\n", ME_num);
				status = ERROR;
			}
		}
	}
	else
	{
		DownLoadUcode(ueng_number, sizeof(ME_Context_Test_ME),
					 (unsigned int *)ME_Context_Test_ME);
		
		hal_delay_us(10000);

		ME_num = (ueng_number & 0x3) | ((ueng_number>>2) & 0x4);

		// Verify Inactive State Test (scratch location 0 - 15 (Lword))
		test = GET32(SCRATCHPAD_BASE + ((ME_num*2)<<B_TO_LW_SHF));
		if(test == COMPLETE)	// indication that Inactive State Test completed
		{
			test = GET32(SCRATCHPAD_BASE + ((ME_num*2 + 1)<<B_TO_LW_SHF));
			if(test != PASS)
			{
				eprintf("ME %d FAIL Inactive State Test.\n", ME_num);
				status = ERROR;
			}
		}
		else
		{
			eprintf("ME %d FAIL to complete Inactive State Test.\n", ME_num);
			status = ERROR;
		}

		// Verify Sleep State Test (scratch location 16 - 31 (Lword))
		test = GET32(SCRATCHPAD_BASE + ((16 + ME_num*2)<<B_TO_LW_SHF));
		if(test == COMPLETE)	// indication that Sleep State Test completed
		{
			test = GET32(SCRATCHPAD_BASE + ((16 + ME_num*2 + 1)<<B_TO_LW_SHF));
			if(test != PASS)
			{
				eprintf("ME %d FAIL Sleep State Test.", ME_num);
				eprintf("Stopped at thread %d\n", (test&0x7));
				status = ERROR;
			}
		}
		// Check if one of the thd did not sleep
		else if(test == THD_DID_NOT_SLEEP)
		{
			eprintf("ME %d thds did not go to sleep.\n", ME_num);
			status = ERROR;
		}
		else
		{
			eprintf("ME %d FAIL to complete Sleep State Test.\n", ME_num);
			status = ERROR;
		}

	}

	eprintf("ME Context Test... ");
	if(status == OK)
	{
		eprintf("PASS.\n");
		Set_LED("MEsP");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n\n");
		Set_LED("MEsF");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "ME Context Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}

	Clear_All_ME();
}

// 
// Function: ME_Timers_Cntrs_Test
//
// Description: Tests the Timers and Counters in the microengines
//
static void ME_Timers_Cntrs_Test(void)
{
	int ME_num, test, status = OK;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// Initialize space in scratch used for ME to pass status to XScale
	for(ME_num = ME0; ME_num <= ME7; ME_num++)
		PUT32(SCRATCHPAD_BASE + (ME_num<<B_TO_LW_SHF), 0);

	if(ueng_number == ALL_UENG)
	{
		DownLoadUcode(19, sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(18, sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(17, sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(16, sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(3,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(2,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(1,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
		DownLoadUcode(0,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);

		hal_delay_us(1000);

		// Check status for each ME
		for(ME_num = ME0; ME_num <= ME7; ME_num++)
		{
			test = GET32(SCRATCHPAD_BASE + (ME_num<<B_TO_LW_SHF));
			if(test == 0)	// test did not complete to write status
			{
				eprintf("ME %d did not complete\n", ME_num);
				status = ERROR;
			}
			else if(test != ME_TIMER_COUNTER_VAL_EXPECTED)
			{
				eprintf("ME %d has wrong timer value. Expected 0x%x, read 0x%x\n",
						ME_num, ME_TIMER_COUNTER_VAL_EXPECTED, test);
				status = ERROR;
			}
		}
	}
	else
	{	
		eprintf("***NOTE***:ME0 is always loaded as it inter-threads the other MEs.\n");
		// requires ME0 else it will fail (ME0 sends signal to the other MEs)
		DownLoadUcode(ueng_number, sizeof(ME_Timers_Counters_Test_ME),
					 (unsigned int *)ME_Timers_Counters_Test_ME);
		if(ueng_number != 0)
		{
			DownLoadUcode(0, sizeof(ME_Timers_Counters_Test_ME),
						 (unsigned int *)ME_Timers_Counters_Test_ME);
		}

		hal_delay_us(1000);

		ME_num = (ueng_number & 0x3) | ((ueng_number>>2) & 0x4);
		
		// Check status for each ME
		test = GET32(SCRATCHPAD_BASE + (ME_num<<B_TO_LW_SHF));
		if(test == 0)	// test did not complete to write status
		{
			eprintf("ME %d did not complete\n", ME_num);
			status = ERROR;
		}
		else if(test != ME_TIMER_COUNTER_VAL_EXPECTED)
		{
			eprintf("ME %d has wrong timer value. Expected 0x%x, read 0x%x\n",
					ME_num, ME_TIMER_COUNTER_VAL_EXPECTED, test);
			status = ERROR;
		}
	}
	
	// Perform logging of test results
	eprintf("Microengine Timers and Counters Test... ");
	if(status == OK)
	{
		eprintf("PASS.\n");
		Set_LED("MEtP");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n\n");
		Set_LED("MEtF");
		slowport_code[0] = SP_ME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "ME Timers and Counters Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}

	Clear_All_ME();
}

