/* msf.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, March, 2002
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "msf.h"
#include "common.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "uart.h"
#include "syslog.h"
#include "diag_utils.h"
#include "uc_load.h"
#include "msf_unicast_ucode.h"
#include "msf_cbus_ucode.h"
#include "memory.h"
#include "msf_init.h"

#define DRAM_BASE						0x1000000
#define SRAM_BASE_ADDR					0x100000

static void Msf_Help(void);

extern pci_device slave_dev_info;		// for slave npu info.
extern pci_device master_dev_info;		// for master npu info.


UINT32 MsfRegister[] = {
	MSF_RX_CONTROL,
	MSF_TX_CONTROL,
	CSIX_TYPE_MAP,
	HWM_CONTROL,
	RX_MPHY_POLL_LIMIT,
	RX_THREAD_FREELIST_TIMEOUT_0,
	RX_THREAD_FREELIST_TIMEOUT_1,
	RX_THREAD_FREELIST_TIMEOUT_2,
	RX_THREAD_FREELIST_TIMEOUT_3,
	TX_MPHY_P0LL_LIMIT,
	RX_UP_CONTROL_0,
	RX_UP_CONTROL_1,
	RX_UP_CONTROL_2,
	RX_UP_CONTROL_3,
	TX_UP_CONTROL_0,
	TX_UP_CONTROL_1,
	TX_UP_CONTROL_2,
	TX_UP_CONTROL_3
};

UINT32 MsfRegisterMask[] = {
	MSF_RX_CONTROL_MASK,
	MSF_TX_CONTROL_MASK,
	CSIX_TYPE_MAP_MASK,
	HWM_CONTROL_MASK,
	RX_MPHY_POLL_LIMIT_MASK,
	RX_THREAD_FREELIST_TIMEOUT_0_MASK,
	RX_THREAD_FREELIST_TIMEOUT_1_MASK,
	RX_THREAD_FREELIST_TIMEOUT_2_MASK,
	RX_THREAD_FREELIST_TIMEOUT_3_MASK,
	TX_MPHY_P0LL_LIMIT_MASK,
	RX_UP_CONTROL_0_MASK,
	RX_UP_CONTROL_1_MASK,
	RX_UP_CONTROL_2_MASK,
	RX_UP_CONTROL_3_MASK,
	TX_UP_CONTROL_0_MASK,
	TX_UP_CONTROL_1_MASK,
	TX_UP_CONTROL_2_MASK,
	TX_UP_CONTROL_3_MASK
};


// msf_unicast_data[] will be stored to DRAM and used to generate a Unicast CFrame
// for the Unicast CFrame test
UINT32 msf_unicast_data[] =	{ 0x00010203, 0x04050607, 0x08090A0B, 0x0C0D0E0F,
									  0x10111213, 0x14151617, 0x18191A1B, 0x1C1D1E1F,
									  0x20212223, 0x24252627, 0x28292A2B, 0x2C2D2E2F,
									  0x30313233, 0x34353637, 0x38393A3B, 0x3C3D3E3F };

// msf_cbus_data[] will be stored to DRAM and used to generate a Flow Control CFrame
// for the CBUS test
UINT32 msf_cbus_data[] = { 0x3F3E3D3C, 0x3B3A3938, 0x37363534, 0x33323130,
								 0x2F2E2D2C, 0x2B2A2928, 0x27262524, 0x23222120,
								 0x1F1E1D1C, 0x1B1A1918, 0x17161514, 0x13121110,
								 0x0F0E0D0C, 0x0B0A0908, 0x07060504, 0x03020100 };


//********************************************************************************************
// Function: void Msf_Help(void)
//
// Displays the list of instructions for the MSF Tests that can be called
// 
// Called by Msf_Test()
//********************************************************************************************
static void Msf_Help(void)
{
	eprintf("Diag> t sf <option>,\n");
	eprintf("where <option> are\n");
	eprintf("\tr - MSF register write-read test\n");
	eprintf("\tu - Unicast CFrame test\n");
	eprintf("\tc - CBUS test\n");
	eprintf("\ta - All MSF tests\n");
}


//********************************************************************************************
// Function: void Msf_Test(void)
//
// Determine which MSF test is called - MSF register test, unicast CFrame test, CBUS
// 		Register test - writes and reads a mask to a selected few registers
// 		Unicast CFrame test - Execute the Unicast CFrame test
// 		CBUS test - tests the CBUS functionality
// 
// Called by Test Manager
//********************************************************************************************
void Msf_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	opt = acL->argv[2][0];
	
	switch(opt)
	{
		case 'r': 	Msf_Register_Test();
					break;

		case 'u': 	Msf_Unicast_Test();
					break;

		case 'c': 	Msf_Cbus_Test();
					break;

		case 'a': 	Msf_Register_Test();
					Msf_Unicast_Test();
					Msf_Cbus_Test();
					break;

		case 'h':
		default	:	Msf_Help();
	}

	if(acL->Err == ERROR)
		eprintf("MSF TEST FAIL!!!\n");
}



// ********************************************************************************************
// Function: void Msf_Register_Test(void)
//
// Writes and reads a mask to a selected few MSF registers and checks whether the data
// written is the same as the data read from the registers.
// ********************************************************************************************
void Msf_Register_Test(void)
{
	int ctr, original_val, read_val, flag;
//	int status;

	Msf_OutOfReset();
//	status = Msf_PLL();
//	if (status == ERROR)
//	{
//		return;
//	}
//	Set_Rcomp();

	flag = OK;
	
	for (ctr = 0; ctr < (sizeof(MsfRegister)/sizeof(UINT32)); ctr++)
	{
		// Read the value of the register before performing the test
		original_val = GET32(MSF_BASE + MsfRegister[ctr]);

		// Write 1s to the Readable/Writeable bits using the predefined register mask
		PUT32((MSF_BASE + MsfRegister[ctr]), MsfRegisterMask[ctr]);

		// Read the value from the register
		read_val = GET32(MSF_BASE + MsfRegister[ctr]) & MsfRegisterMask[ctr];

		// Compare the read value and written value
		// If not the same, display a message
		if (read_val != MsfRegisterMask[ctr])
		{
			eprintf("MSF Register:0x%X  Write value:0x%X  Read value:0x%X\n", (MSF_BASE + MsfRegister[ctr]), MsfRegisterMask[ctr], read_val);
			flag = ERROR;
		}

		// Write 0s to the Readable/Weiteable bits
		PUT32((MSF_BASE + MsfRegister[ctr]), 0);

		// Read the value from the register
		read_val = GET32(MSF_BASE + MsfRegister[ctr]) & MsfRegisterMask[ctr];

		// Compare the read value and written value
		// If not the same, display a message
		if (read_val != 0)
		{
			eprintf("MSF Register:0x%X  Write value:0x0  Read value:0x%X\n", (MSF_BASE + MsfRegister[ctr]), read_val);
			flag = ERROR;
		}

		// Restore the original value of the register before the test was run
		PUT32((MSF_BASE + MsfRegister[ctr]), original_val);
	}


	if (flag == OK)
	{
		eprintf("\nMSF Register test...PASSED\n");
		Set_LED("MSrP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Scratch ring test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("\nMSF Register test...FAIL!\n");
		Set_LED("MSrF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}		
}


// ********************************************************************************************
// Function: void Msf_Unicast_Test(void)
//
// Startes the MSF Unicast CFrame test. It can only be initiated by the Master NPU.
// Once the Master NPU has completed its init, it initiaies a PCI Doorbell interrupt on the
// Slave NPU to get the slave NPU to initialise itself and start the test.
//
// Called by Msf_Test()
// ********************************************************************************************
void Msf_Unicast_Test(void)
{
	int ctr, temp=0, mailbox_value, flag, doorbell_intr_addr, slave_mailbox0_addr, status;
	UINT dcache_status;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	doorbell_intr_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	if(acL->HostType == MASTER)		// if master NPU
	{
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		flag = OK;

		// Init MAILBOX0 to 0
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

		// Init slave's mailbox0 to 0
		PUT32(slave_mailbox0_addr, 0);
		
		DownLoadUcode(UNICAST_RX_ME, sizeof(msf_unicast_rx), (unsigned int *)msf_unicast_rx);

		hal_delay_us(500000);

		Set_Pci_Doorbell(MSF_U_BIT, doorbell_intr_addr);

		ctr = 0;
		do
		{
			// poll on master's mailbox register
			mailbox_value = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);

			hal_delay_us(1000000);
		
			ctr++;
			if (ctr == 20)	// After 20 seconds
			{
				eprintf("Timed out...Packet not received...\n");
				break;
			}
		}
		while (mailbox_value == 0);

		// If MAILBOX0 register indicates that test is complete
		if (((mailbox_value >> 16) & MASK_16BIT) == TEST_COMPLETE)
		{
			// Read the received value from DRAM and compare with original data
			for (ctr = 0; ctr < sizeof(msf_unicast_data); ctr+=4)
			{
				// Read value from DRAM
				temp = GET32(DRAM_BASE + ctr);
//				eprintf("ctr=%d\ttemp=0x%X\n", ctr, temp);
				// Compare data in DRAM and original data
				if (temp != msf_unicast_data[(ctr / 4)])
				{
					if (flag != ERROR)
					{
						flag = ERROR;
					}

					eprintf("Longword %d of packet: Expected:0x%X  Received:0x%X\n", (ctr / 4), msf_unicast_data[(ctr / 4)], temp);
				}
			}
		}
		else	// If MAILBOX0 register value does not indicate that test is complete
		{
			flag = ERROR;
		}

		if (flag == OK)
		{
			eprintf("\nMSF Unicast CFrame test...PASSED\n");
			Set_LED("MSuP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Scratch ring test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("\nMSF Unicast CFrame test...FAIL!\n");
			Set_LED("MSuF");
			acL->Err = ERROR;
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}
}


// ***************************************************************************************************
// Function: void Msf_Unicast_Test_Slave(void)
//
// This test is run on the slave and is called by the PCI Doorbell Interrupt handler function after
// the Master NPU initiates a PCI Doorbell interrupt
// It initialises the Slave NPU by loading the microcode and starts the Unicast CFrame test.
//
// Called by doorbell_intr_received() (found in pci_diag.c)
// ***************************************************************************************************
void Msf_Unicast_Test_Slave(void)
{
	int ctr, mailbox_value, status;
	UINT dcache_status;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	// Write packet data to DRAM for microcode to use in transmit
	for (ctr = 0; ctr < sizeof(msf_unicast_data); ctr+=4)
	{
		PUT32((DRAM_BASE + ctr), msf_unicast_data[(ctr / 4)]);
	}

	// Download the microcode to the ME
	DownLoadUcode(UNICAST_TX_ME, sizeof(msf_unicast_tx), (unsigned int *)msf_unicast_tx);

	ctr = 0;
	
	// poll on MAILBOX0 for signal that unicast frame was sent
	do
	{
		mailbox_value = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF);
		hal_delay_us(1000000);
		
		ctr++;
		if (ctr == 20)	// After 30 milliseconds
		{
			eprintf("Timed out...");
			break;
		}
	}
	while (mailbox_value == 0);

	// Check MAILBOX0 value and compare result
	if ((mailbox_value >> 16) == PACKET_SENT)
	{
		eprintf("\nUnicast CFrame has been sent...\n");

		eprintf("\nLook at Master's console for test result...\n");
	}
	else
	{
		eprintf("Packet was not sent...\n");

		eprintf("\nLook at Master's console for test result...\n");
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


// ********************************************************************************************
// Function: void Msf_Cbus_Test(void)
//
// Startes the MSF CBUS test. It can only be initiated by the Master NPU.
// Once the Master NPU has completed its init, it initiaies a PCI Doorbell interrupt on the
// Slave NPU to get the slave NPU to initialise itself and start the test.
//
// Called by Msf_Test()
// ********************************************************************************************
void Msf_Cbus_Test(void)
{
	int ctr, mailbox_value, slave_mailbox0_addr, doorbell_intr_addr;
	register PDiagCommon acL = (PDiagCommon)ACADDRESS;
	UINT32 status;
	UINT dcache_status;

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	doorbell_intr_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF;

	Msf_OutOfReset();

	status = Msf_PLL();

	if (status == ERROR)
	{
		return;
	}

//	Set_Rcomp();

	// Init slave's mailbox0 to 0
	PUT32(slave_mailbox0_addr, 0);

	if(acL->HostType == MASTER)		// if master NPU
	{
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}
		
		DownLoadUcode(CBUS_RX_ME, sizeof(msf_cbus_rx), (unsigned int *)msf_cbus_rx);

		hal_delay_us(500000);

		Set_Pci_Doorbell(MSF_C_BIT, doorbell_intr_addr);

		ctr = 0;
		do
		{
			// Poll on slave's MAILBOX0 to check if pass or fail
			mailbox_value = GET32(slave_mailbox0_addr);
			
			hal_delay_us(1000000);		// Delay 1 second
		
			ctr++;
			if (ctr == 20)	// After 20 seconds
			{
				eprintf("Timed out...\n");
				break;
			}
		}
		while (mailbox_value == 0);


		#ifdef __ARMEB__
			mailbox_value = SWAP32(mailbox_value);
		#endif

//		eprintf("DEBUG: mailbox_value = 0x%X\n", mailbox_value);

		if (mailbox_value == (TEST_COMPLETE & 0xFFFF))
		{
			eprintf("\nMSF CBUS test...PASSED\n");
			Set_LED("MScP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			if (mailbox_value == ((INCORRECT_SIZE << 16) & 0xFFFF0000))
			{
				eprintf("Packet received by Slave NPU was not the expected size\n");
			}
			else if  (mailbox_value == ((INCORRECT_TYPE << 16) & 0xFFFF0000))
			{
				eprintf("Incorrect packet type was received\n");
			}
			else if (mailbox_value == ((NOT_RECEIVED << 16) & 0xFFFF0000))
			{
				eprintf("Packet was not received\n");
			}

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Scratch ring test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			
			eprintf("\nMSF CBUS test...FAIL!\n");
			Set_LED("MScF");
			acL->Err = ERROR;
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}		

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}
}


// ***************************************************************************************************
// Function: void Msf_Cbus_Test_Slave(void)
//
// This test is run on the slave and is called by the PCI Doorbell Interrupt handler function after
// the Master NPU initiates a PCI Doorbell interrupt
// It initialises the Slave NPU by loading the microcode and starts the CBUS test.
//
// Called by doorbell_intr_received() (found in pci_diag.c)
// ***************************************************************************************************
void Msf_Cbus_Test_Slave(void)
{
	int ctr, temp, temp1, temp2, flag, mailbox_value;
	UINT32 status;
	UINT dcache_status;

	flag = OK;

	Msf_OutOfReset();

	status = Msf_PLL();

	if (status == ERROR)
	{
		return;
	}

//	Set_Rcomp();

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	// Init MAILBOX1 to 0
	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF), 0);

	// Write packet data to DRAM for microcode to use
	for (ctr = 0; ctr < sizeof(msf_cbus_data); ctr+=4)
	{
		PUT32((DRAM_BASE + ctr), msf_cbus_data[(ctr / 4)]);
	}

	// Load ME with microcode
	DownLoadUcode(CBUS_TX_ME, sizeof(msf_cbus_tx), (unsigned int *)msf_cbus_tx);

	ctr = 0;

	// Wait for ME to signal through MAILBOX1 
	do
	{
		mailbox_value = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF);

		hal_delay_us(1000000);
		
		ctr++;
		if (ctr == 20)	// After 20 seconds
		{
			eprintf("Timed out...Packet not received\n");
			mailbox_value = ((NOT_RECEIVED << 16) & 0xFFFF0000);
		}
	}
	while (mailbox_value == 0);
		
	if (((mailbox_value >> 16) & 0xFFFF) == (TEST_COMPLETE & 0xFFFF))
	{
		// Compare received and transmitted data
		for (ctr = 0; ctr < sizeof(msf_cbus_data); ctr+=4)
		{
//			temp1 = GET32(SRAM_BASE + SRAM_BASE_ADDR + ctr);
//			temp2 = GET32(SRAM_BASE + SRAM_BASE_ADDR + ctr + 1);
			temp1 = GET32(SCRATCHPAD_BASE + 0x200 + ctr);
			temp2 = GET32(SCRATCHPAD_BASE + 0x200 + ctr + 4);

			// Concatenate LSB 16 bit of first long word and MSB 16 bit of second long word
			temp = ((temp1 << 16) & 0xFFFF0000) | ((temp2 >> 16) & 0xFFFF);

			// Compare with original value
			if (temp != msf_cbus_data[(ctr / 4)])
			{
				if (flag != ERROR)		// If the flag is not already marked as error
				{
					flag = ERROR;		// Set flag as error
				}

				eprintf("Longword %d of packet: Expected:0x%X  Received:0x%X\n", (ctr / 4), msf_cbus_data[(ctr / 4)], temp);
			}
/*			else
			{
//				eprintf("ctr=%d\ttemp=0x%08X\n", ctr, temp);
			}
*/		}
	}
	else
	{
		eprintf("\nThere was an error in the process of receiving the packet.\n");
		flag = ERROR;
	}

	if (flag == OK)
	{
		mailbox_value = (TEST_COMPLETE & 0xFFFF);
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), /*TEST_COMPLETE*/mailbox_value);
	}
	else
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}
