/* pci_diag.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, Feb 2002
 * 
 * --------------------------------------------------------------------------
 */

#include "common.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "hal_io.h"
#include "pci.h"
#include "pci_diag.h"
#include "pci_dma.h"
#include "syslog.h"
#include "hal_ixdp2400.h"
#include "21555.h"
#include "led.h"
#include "commands.h"
#include "msf.h"
#include "sf_loopback.h"
#include "media_loopback.h"
#include "sys_loopback.h"
#include "line_rate.h"
#include "ethernet.h"
#include "diag.h"
#include "uart.h"

#ifdef INTERNAL_TEST
void Write_21555(void);
void Read_21555(void);
void Write_21154(void);
void Read_21154(void);
void Write_NPU(void);
void Read_NPU(void);
#endif

static void i21555SromWrite(int reg, unsigned char val);
static void i21555SromWriteEnable(void);
static UINT8 i21555SromRead(int reg);
static int i21555SromErase(int reg);

extern pci_device master_dev_info;		// for master npu info.
extern pci_device slave_dev_info;		// for slave npu info.
extern pci_device nic_dev_info;
extern void pci_read_config_uint32( pci_device_id devid, UINT8 offset, UINT32 *val);
extern void pci_write_config_uint32( pci_device_id devid, UINT8 offset, UINT32 val);
extern void pci_read_config_uint16( pci_device_id devid, UINT8 offset, UINT16 *val);
extern void pci_write_config_uint16( pci_device_id devid, UINT8 offset, UINT16 val);
extern void pci_read_config_uint8( pci_device_id devid, UINT8 offset, UINT8 *val);
extern void pci_write_config_uint8( pci_device_id devid, UINT8 offset, UINT8 val);
extern UINT16 hal_plf_pci_cfg_read_word  (UINT32 bus, UINT32 devfn, UINT32 offset);
extern UINT32 hal_plf_pci_cfg_read_dword (UINT32 bus, UINT32 devfn, UINT32 offset);
extern void PL_Media_Loopback_Slave(void);
extern void Slave_pl_SysLpbk_test(void);
extern void Sys_Loopback2_PL_Slave(void);
extern void Sys_Loopback3_PL_Slave(void);
extern void Dual_Media_Test(void);
extern void Media_On_SF_IF_Test(void);
extern UINT32 t_all_verbose;
extern void Clear_All_ME(void);

extern UINT32 t_all_verbose;

#define ANALOG_WRITE_21555	0xFFFFFFFF
#define ANALOG_WRITE_21154	0xFF
#define ANALOG_WRITE_NPU	0xFFFFFFFF

#define ANALOG_21555_OFFSET	0x8
#define ANALOG_21154_OFFSET	0xC

static unsigned char sromPreloadMask[] = {
	0xc0, /* [7:6] = 10b to enable serial preload, [5:0] reserved */
	0x00, 0x00, 0x00,
	0xff, /* primary programming interface */
	0xff, /* primary sub-class code */
	0xff, /* primary base clase code */
	0xff, /* subsystem vendor id [7:0] */
	0xff, /* subsystem vendor id [15:8] */
	0xff, /* subsystem id [7:0] */
	0xff, /* subsystem id [15:8] */
	0xff, /* primary minimum grant */
	0xff, /* primary maximum latency */
	0xff, /* secondary programming interface */
	0xff, /* secondary sub-class code */
	0xff, /* secondary base class code */
	0xff, /* secondary minimum grant */
	0xff, /* secondary maximum latency */
	0x06, /* downstream memory 0 setup [7:0] */
	0xf0, /* downstream memory 0 setup [15:8] */
	0xff, /* downstream memory 0 setup [23:16] */
	0xff, /* downstream memory 0 setup [31:24] */
	0xcf, /* downstream i/o or memory 1 setup [7:0] */
	0xff, /* downstream i/o or memory 1 setup [15:8] */
	0xff, /* downstream i/o or memory 1 setup [23:16] */
	0xff, /* downstream i/o or memory 1 setup [31:24] */
	0x06, /* downstream memory 2 setup [7:0] */
	0xf0, /* downstream memory 2 setup [15:8] */
	0xff, /* downstream memory 2 setup [23:16] */
	0xff, /* downstream memory 2 setup [31:24] */
	0x06, /* downstream memory 3 setup [7:0] */
	0xf0, /* downstream memory 3 setup [15:8] */
	0xff, /* downstream memory 3 setup [23:16] */
	0xff, /* downstream memory 3 setup [31:24] */
	0xff, /* downstream memory 3 setup upper 32 bits [7:0] */
	0xff, /* downstream memory 3 setup upper 32 bits [15:8] */
	0xff, /* downstream memory 3 setup upper 32 bits [23:16] */
	0xff, /* downstream memory 3 setup upper 32 bits [31:24] */
	0xf1, /* bit [0] primary expansion ROM setup (enable), bits [7:4] primary expansion ROM setup [15:11] */
	0xff, /* primary expansion ROM setup [23:16] */
	0xcf, /* upstream i/o or memory 0 setup [7:0] */
	0xff, /* upstream i/o or memory 0 setup [15:8] */
	0xff, /* upstream i/o or memory 0 setup [23:16] */
	0xff, /* upstream i/o or memory 0 setup [31:24] */
	0x06, /* upstream memory 1 setup [7:0] */
	0xf0, /* upstream memory 1 setup [15:8] */
	0xff, /* upstream memory 1 setup [23:16] */
	0xff, /* upstream memory 1 setup [31:24] */
	0xff, /* chip control 0 [7:0] */
	0xcf, /* chip control 0 [15:8] */
	0xff, /* chip control 1 [7:0] */
	0xff, /* chip control 1 [15:8] */
	0xff, /* arbiter control [7:0] */
	0x03, /* arbiter control [15:8] */
	0x7f, /* primary SERR# disable [7:0] */
	0x7f, /* secondary SERR# disable [7:0] */
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff, /* power management data 0-7 */
	0x00, /* reserved */
	0xfc, /* [2] BiST supported, [3] power mgmt. data enable, [5:4] power mgmt. csr [14:13],
		  [7:6] power mgmt. capabilities [1:0] */
	0xff /* [0] power mgmt. cap. [2], [1] power mgmt. cap. [5], [7:2] power mgmt. cap [14:9] */
};

//********************************************************************************************
// Function: void Pci_Help(void)
//
// Displays the list of instructions for the PCI Tests that can be called
// 
// Called by Pci_Test()
//********************************************************************************************
static void Pci_Help(void)
{
	eprintf("Diag> t pci <option>,\n");
	eprintf("where <option> are\n");
	eprintf("\tf - Search for PCI devices\n");
	eprintf("\ti - PCI doorbell interrupt test\n");
	eprintf("\td - PCI DMA test\n");
	eprintf("\ta - All PCI tests\n");
}


//********************************************************************************************
// Function: void Pci_Test(void)
//
// Determine which PCI test is called. The available tests are:-
//		Find the non-transparent bridge
//		Find the transparent bridge
// 		PCI doorbell interrupt test
// 		PCI DMA test
// 
// Called by Test Manager
//********************************************************************************************
void Pci_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	opt = acL->argv[2][0];
	
	switch(opt)
	{
		case 'f': 	Find_All_Pci();
					break;

		case 'i': 	Pci_Doorbell_Test();
					break;

		case 'd': 	Pci_Dma_Test();
					break;
		
		case 'a':	Find_All_Pci();
					//Pci_Doorbell_Test();
					Pci_Dma_Test();
					break;

		case 'h':
		default	:	Pci_Help();
	}

	*(PCI_ADDR_EXT_REG) = (nic_dev_info.base_address[0] & PCI_MEM_PAGE_MASK) >> 16;

	if(acL->Err == ERROR)
		eprintf("PCI TEST FAIL!!!\n");
	return;	
}


//********************************************************************************************
// Function: void find_i21555(void)
//
// This routine looks for the i21555 device
//
// Inputs:	NONE
//
// Outputs:	NONE
//********************************************************************************************
void Find_i21555(void)
{
	UINT32 handle;
	int instance;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	instance = 0;
	
	handle = pci_find_device(VENDOR_INTEL, DEVICE_21555, &instance);

	if (handle == 0)
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "i21555 device not found");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("i21555 Device not found!\n");
		Set_LED("PCnF");
		acL->Err = ERROR;
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		return;
	}
	else
	{
		eprintf("i21555 found...\n");
		Set_LED("PCnP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}

	pci_read_config_uint32(handle, PCI_SECONDARY_CSR_IO_BAR, &ntbInfo.ioBase);

	if (ntbInfo.ioBase == 0)
	{
		eprintf("i21555 No resources assigned!\n");
	}
}



//********************************************************************************************
// Function: void Find_All_Pci(void)
//
// This routine looks for all the available PCI devices
//
// Inputs:	NONE
//
// Outputs:	NONE
//********************************************************************************************
void Find_All_Pci(void)
{
	UINT8 handle = 0x3F;
	pci_device_id new_devid = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	while (pci_find_next(new_devid, &new_devid))
	{
		UINT8 bus = PCI_DEV_GET_BUS(new_devid);
		UINT8 devfn = PCI_DEV_GET_DEVFN(new_devid);
		UINT8 devid = PCI_DEV_GET_DEV(devfn);
		UINT16 v, d;

		v = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_VENDOR);
        d = hal_plf_pci_cfg_read_word(bus, devfn, PCI_CFG_DEVICE);

		if ((v == VENDOR_INTEL) && (d == DEVICE_21555))
		{
			if(t_all_verbose != MINIMAL)
			{
				eprintf("PCI non-transparent bridge found(DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
			}
			handle = handle & ~((UINT8)(1 << 0));
		}
		else if ((v == VENDOR_INTEL) && (d == DEVICE_21154))
		{
			if(t_all_verbose != MINIMAL)
				eprintf("PCI transparent bridge found(DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
			handle = handle & ~((UINT8)(1 << 1));
		}
		else if ((v == VENDOR_INTEL) && (d == DEVICE_NIC))
		{
			if (devid == 3)
			{
				if(t_all_verbose != MINIMAL)
					eprintf("Master Ethernet NIC found (DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
				handle = handle & ~((UINT8)(1 << 2));
			}
			else if (devid == 4)
			{
				if(t_all_verbose != MINIMAL)
					eprintf("Slave Ethernet NIC found (DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
				handle = handle & ~((UINT8)(1 << 3));
			}
		}
		else if ((v == VENDOR_INTEL) && (d == DEVICE_NPU))
		{
			if (devid == 5)
			{
				if(t_all_verbose != MINIMAL)
					eprintf("IXP2400 Slave NPU found (DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
				handle = handle & ~((UINT8)(1 << 4));
			}
			else if (devid == 7)
			{
				if(t_all_verbose != MINIMAL)
					eprintf("IXP2400 Master NPU found (DeviceID:0x%X, VendorID:0x%X, devid=%d)\n", d, v, devid);
				handle = handle & ~((UINT8)(1 << 5));
			}
		}
	}

	if (handle > 0)
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Some PCI devices not found");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("Some PCI devices not found!\n");
		eprintf("PCI device scan test...FAIL!\n");
		Set_LED("PCfF");
		acL->Err = ERROR;
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		return;
	}
	else
	{
		if(t_all_verbose != MINIMAL)
		{
			eprintf("All PCI devices found\n");
			eprintf("PCI device scan test...PASS\n");
		}
		Set_LED("PCfP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
}



// i21555SromWriteEnable
//
// This routine sets the 21555 SROM to the write enable state.  
//
// RETURNS: Nothing
//
static void i21555SromWriteEnable(void)
{
	PUT32((ntbInfo.ioBase+I21555_ROM_ADDR), I21555_ROM_ADDR_WriteEnable);
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		hal_delay_us(1000);	// 1 ms delay
	}

	return;
}


// i21555SromWrite
//
// This routine writes a byte to a location in the 21555 SROM
//
// RETURNS: STATUS
//
static void i21555SromWrite(int reg, unsigned char val)
{
	i21555SromWriteEnable();
	PUT32((ntbInfo.ioBase+I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Write));
	PUT8((ntbInfo.ioBase+I21555_ROM_DATA), (val & sromPreloadMask[reg]));
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		hal_delay_us(1000);	// 1 ms delay
	}
	
	// This needs to be done again. Without it, apparently PCI does not work properly
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		hal_delay_us(1000);	// 1 ms delay
	}

	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromPoll)
	{
		PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
		while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
		{
			hal_delay_us(1000);	// 1 ms delay
		}
	}
}


// i21555SromRead
// 
// This routine reads a byte from a location in the 21555's SROM
//
// RETURNS: SROM Data which was read from the address specified
//
static UINT8 i21555SromRead(int reg)
{
	PUT32((ntbInfo.ioBase + I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Read));

	while (GET8(ntbInfo.ioBase + I21555_ROM_CTRL) & (I21555_ROM_CTRL_SromStartBusy | I21555_ROM_CTRL_PromStartBusy))
	{
		hal_delay_us(1000);	// 1 ms delay
	}

	PUT8((ntbInfo.ioBase + I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);

	while (GET8(ntbInfo.ioBase + I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		hal_delay_us(1000);	// 1 ms delay
	}

	return GET8(ntbInfo.ioBase + I21555_ROM_DATA);
}


// i21555SromErase
//
// This routine erases a byte in the 21555 SROM
//
// RETURNS: STATUS
//
static int i21555SromErase(int reg)
{
	i21555SromWriteEnable();

	PUT32((ntbInfo.ioBase + I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Erase));

	PUT8((ntbInfo.ioBase + I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);

	while (GET8((ntbInfo.ioBase + I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy))
	{
		hal_delay_us(1000);	// 1 ms delay
	}

	return 0;
}


// i21555SromReprogram
//
// This routine reprograms the 21555 SROM from the supplied address
//
// RETURNS: STATUS
//
int i21555SromReprogramAll(UINT8 *newData)
{
	int ctr;

	for (ctr=0;ctr<SROM_NUM_BYTES;ctr++)
	{
		if (*newData == i21555SromRead(ctr))	// if newData == oldData
		{
			eprintf("file %s, line %d: %d matches\n", __FILE__, __LINE__, ctr);
			newData++;							// if newData == oldData, skip
			continue;
		}
		else
		{
			i21555SromErase(ctr);					// else erase and write new value
			eprintf("file %s, line %d: writing 0x%x to %d\n", __FILE__, __LINE__, *newData, ctr);
			i21555SromWrite(ctr, *newData++);
		}
	}

	return 0;
}


// i21555SromDump
//
// This routine dumps the contents of the 21555 SROM to memory.
//
// RETURNS: STATUS
//
int i21555SromDump(UINT8 *data)
{
	int ctr;

	for (ctr=0;ctr<SROM_NUM_BYTES;ctr++)
	{
		*data++ = i21555SromRead(ctr);
	}

	return 0;
}


// Function: void find_i21154(void)
//
// This routine looks for the i21154 device
//
// Inputs:	NONE
//
// Outputs:	NONE
//
void Find_i21154(void)
{
	int handle;
	pci_device_id dev_id = PCI_NULL_DEVID;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	handle = pci_find_device(TB_VENDOR_ID, TB_DEVICE_ID, &dev_id);

	if (handle == 0)
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "i21154 device not found");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
		
		eprintf("i21554 was not found...\n");
		Set_LED("PCtF");
		acL->Err = ERROR;
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("i21154 was found...\n");
		Set_LED("PCtP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
}


// void Pci_Doorbell_Test(void)
//
// This routine initiates the PCI doorbell test
//
// Inputs:	NONE
//
// Outputs:	NONE
//
void Pci_Doorbell_Test(void)
{
	int ctr;
	pci_device dev_info;
	int doorbell_intr_addr;
	int delay_time = 30;
	char master_text[] = "master";
	char slave_text[] = "slave";
	char *next_npu_text;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
//		eprintf("DEBUG:Pci_Doorbell_Test - Master\n");
		dev_info = slave_dev_info;		// Get slave's mapped info
		next_npu_text = slave_text;
	}
	else											// if slave NPU
	{
//		eprintf("DEBUG:Pci_Doorbell_Test - Slave\n");
		dev_info = master_dev_info;		// Get master's mapped info
		next_npu_text = master_text;
	}

	// Calculate the other NPU's doorbell register address
//	eprintf("DEBUG:dev_info.base_map[CSR_BAR] = 0x%X\n", dev_info.base_map[CSR_BAR]);
	*(PCI_ADDR_EXT_REG) = (dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	doorbell_intr_addr = dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF;

	// Write a pattern to the other NPU's Mailbox0 register to init
	// the location. This register will be polled later on by the NPU
	// running this function. Once the other NPU has received the doorbell
	// interrupt, will write a value of 0 to this register.
	PUT32((dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF), PATTERN1);
	
	// Write a '1' to bit 0 of the other NPU's PCI_DOORBELL register to initiate
	// the doorbell interrupt test
//	eprintf("DEBUG:GET32(doorbell_intr_addr)=0x%X\n", GET32(doorbell_intr_addr));
	Set_Pci_Doorbell(0, doorbell_intr_addr);
	
//	eprintf("DEBUG:GET32(doorbell_intr_addr)=0x%X\n", GET32(doorbell_intr_addr));
	if(t_all_verbose != MINIMAL)
	{
		eprintf("\nBit 0 of %s's XScale doorbell interrupt was set to 1...\n", next_npu_text);
		eprintf("\nPolling on Mailbox0 of %s for signal that it received the interrupt...\n", next_npu_text);
	}

	// Poll on the other NPU's Mailbox0 register and check if it is 0.
	// The other NPU's Mailbox0 register is set to 0 by the the other NPU's XScale as part of
	// the diag code. If the other NPU does not indicate that it has received the doorbell interrupt
	// after 30ms, it is assumed that the test fails
	for (ctr = 0; ctr < delay_time; ctr++)
	{
		hal_delay_us(1000);		// 1ms delay

		*(PCI_ADDR_EXT_REG) = (dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		if (GET32(dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF) == 0)
		{
			break;
		}

		if (ctr == (delay_time - 1))
		{
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "PCI doorbell test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("\nThe %s did not signal that doorbell interrupt was received\n", next_npu_text);
			eprintf("\nPCI doorbell interrupt test...FAIL!\n");
			Set_LED("PCiF");
			acL->Err = ERROR;
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);

			return;
		}
	}

	if(t_all_verbose != MINIMAL)
	{
		eprintf("\nMailbox0 of %s indicates that the interrupt was received...\n", next_npu_text);
		eprintf("\nPCI doorbell interrupt test...PASS\n");
	}
	Set_LED("PCiP");
	slowport_code[0] = SP_XSC;
	slowport_code[1] = SP_PASS;
	dump_slowport(slowport_code, 2, SP_NON_FATAL);
}


// Function: void Doorbell_Intr_Received(void)
//
// This routine is initiated when a doorbell interrupt is received
//
// Inputs:	NONE
//
// Outputs:	NONE
//
void Doorbell_Intr_Received(void)
{
	int temp, ctr, doorbell_reg_value, intr_no;

//	PUT32(IRQ_ENABLE_CLR_REG, PCI_DOORBELL);	// temporarily disable IRQ pci doorbell interrupt
	intr_no = -1;
//eprintf("MAC: TAKE ME OUT\n");
	doorbell_reg_value = GET32(PCI_LOCAL_CSR_BASE + XSCALE_DOORBELL_OFF);	// Read the XSCALE_DOORBELL register
	//eprintf("DEBUG:doorbell_reg_value = 0x%X\n", doorbell_reg_value);

	// This for assumes that there is only one interrupt, as diagnostics is a controlled environment
	// So after finding one interrupt, it exits the 'for' loop
	for (ctr = 0; ctr < 32; ctr++)		// Check each bit of the register
	{
		// Check if the bit is a '0'. If '0', that is the interrupt
		if (((doorbell_reg_value >> ctr) & 1) == 0)
		{
			intr_no = ctr;
			// Write a '1' to reset the bit which caused the interrupt
			temp = (1 << intr_no);
			PUT32((PCI_LOCAL_CSR_BASE + XSCALE_DOORBELL_OFF), temp);
			break;
		}
	}

	if (intr_no != -1)
	{
		switch (intr_no)
		{
			case DB_BIT:		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);
								break;
			case MSF_U_BIT:		Msf_Unicast_Test_Slave();
								break;
			case MSF_C_BIT:		Msf_Cbus_Test_Slave();
								break;
			case SF_LB_BIT:		Sf_Loopback_Test_Slave();
								break;
			case MEDIA_LB_BIT:	Media_Loopback_Test_Slave();
								break;
			case SYS_LB_BIT:	Sys_Loopback_Test_Slave();
								break;
			case LINERATE_BIT:	Line_Rate_Test_Slave();
								break;
			case SYS_LB2_BIT:	Sys_Loopback2_Test_Slave();
								break;
			case SYS_LB3_BIT:	Sys_Loopback3_Test_Slave();
								break;
			case DUAL_MEDIA_BIT:Dual_Media_Test();
								break;
			case MEDIA_SF_BIT:	Media_On_SF_IF_Test();
								break;
			case NIC_RECV_BIT:	i559RUStart ();
								break;
            case PTLONE_MEDIA_LPBK :    eprintf("slave: doorbell handler\n");
										PL_Media_Loopback_Slave();
										break;
			case PTLONE_SYS_LPBK :		eprintf("slave: doorbell handler\n");
										Slave_pl_SysLpbk_test();
										break;
			case PTLONE_SYS_LPBK2:		Sys_Loopback2_PL_Slave();
										break;
			case PTLONE_SYS_LPBK3:		Sys_Loopback3_PL_Slave();
										break;
		}
	}

//	PUT32(IRQ_ENABLE_SET_REG, PCI_DOORBELL);	// enable FIQ pci doorbell interrupt
}

// Function: void Set_Pci_Doorbell(int bit_num, int db_intr_addr)
//
// This function when called will set the specified bit of the doorbell interrupt register
// of the other NPU
//
// Inputs:		 bit_num = the bit number on the doorbell register to set
//			db_intr_addr = the address of the other NPU's XScale Doorbell
//						    register (as seen over PCI)
//
// Outputs:	NONE
//
void Set_Pci_Doorbell(int bit_num, int db_intr_addr)
{
	int temp;

	// Write a '1' to the selected bit of the PCI_DOORBELL register
	// to initiate a doorbell interrupt
	temp = SET_BIT(bit_num);
	#ifdef __ARMEB__
	temp = SWAP32(temp);
	#endif
	PUT32(db_intr_addr, temp);
}


// Function: void Pci_Dma_Test(void)
//
// This function will call for the PCI DMA test to be run and display
// the result of the test
//
// Inputs:	NONE
//
// Outputs:	NONE
//
void Pci_Dma_Test(void)
{
	int status, ctr;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT dcache_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	for (ctr = 0; ctr < 4096; ctr ++)
	{
		PUT32((DRAM_ADDR + (ctr * 4)), (0xDADA0000 | ctr));
	}

	status = dmaTest();

	if (status == OK)
	{
		if(t_all_verbose != MINIMAL)
		{
			eprintf("PCI DMA test...PASS\n");
		}
		Set_LED("PCdP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "PCI DMA test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("PCI DMA test...FAIL!\n");
		Set_LED("PCdF");
		acL->Err = ERROR;
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}

#ifdef INTERNAL_TEST
void Write_21555(void)
{
	UINT32 handle;
	UINT32 instance = 0;
	
	handle = pci_find_device(VENDOR_INTEL, DEVICE_21555, &instance);

	pci_read_config_uint32(instance, PCI_SECONDARY_CSR_IO_BAR, &ntbInfo.ioBase);

//	eprintf("instance=0x%08X\tntbInfo.ioBase=0x%08X\n", handle, ntbInfo.ioBase);

	*(PCI_ADDR_EXT_REG) = ntbInfo.ioBase & PCI_IO_PAGE_MASK;

	eprintf("Writing value of 0x%X to 21555...\n", ANALOG_WRITE_21555);

	while (!KEYPRESS)
	{
		PUT32(((ntbInfo.ioBase & (PRI_CFG_BAR_IO_MASK & XSCALE_PCI_IO_SZ_MASK))
						+ XSCALE_PCI_IO_BASE + ANALOG_21555_OFFSET), ANALOG_WRITE_21555);
	}
}

void Read_21555(void)
{
	UINT32 handle, temp;
	int instance = 0;
	
	handle = pci_find_device(VENDOR_INTEL, DEVICE_21555, &instance);

	pci_read_config_uint32(instance, PCI_SECONDARY_CSR_IO_BAR, &ntbInfo.ioBase);

//	eprintf("instance=0x%08X\tntbInfo.ioBase=0x%08X\n", handle, ntbInfo.ioBase);

	*(PCI_ADDR_EXT_REG) = ntbInfo.ioBase & PCI_IO_PAGE_MASK;

	eprintf("Reading from 21555...\n");

	while (!KEYPRESS)
	{
		temp = GET32((ntbInfo.ioBase & (PRI_CFG_BAR_IO_MASK & XSCALE_PCI_IO_SZ_MASK)) + XSCALE_PCI_IO_BASE + ANALOG_21555_OFFSET);
//		eprintf("Read value = 0x%08X\n", temp);
	}
}

void Write_21154(void)
{
	UINT32 handle;
	UINT32 instance = 0;
	
	handle = pci_find_device(VENDOR_INTEL, DEVICE_21154, &instance);

//	eprintf("handle=0x%08X\tinstance=0x%08X\n", handle, instance);

	eprintf("Writing value of 0x%X to 21154...\n", ANALOG_WRITE_21154);

	while (!KEYPRESS)
	{
		pci_write_config_uint8(instance, ANALOG_21154_OFFSET, ANALOG_WRITE_21154);
	}
}

void Read_21154(void)
{
	UINT32 handle;
	UINT32 instance = 0;
	UINT8 value;

	handle = pci_find_device(VENDOR_INTEL, DEVICE_21154, &instance);

//	eprintf("handle=0x%08X\tinstance=0x%08X\n", handle, instance);

	eprintf("Reading from 21154...\n");

	while (!KEYPRESS)
	{
		pci_read_config_uint8(instance, ANALOG_21154_OFFSET, &value);
//		eprintf("Read value = 0x%02X\n", value);
	}
}

void Write_NPU(void)
{
	pci_device dev_info;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
		dev_info = slave_dev_info;		// Get slave's mapped info
	}
	else											// if slave NPU
	{
		dev_info = master_dev_info;		// Get master's mapped info
	}
	
	*(PCI_ADDR_EXT_REG) = (dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	
	eprintf("Writing value of 0x%X to NPU...\n", ANALOG_WRITE_NPU);

	while (!KEYPRESS)
	{
		PUT32((dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF), ANALOG_WRITE_NPU);
	}
}

void Read_NPU(void)
{
	UINT32 temp;
	pci_device dev_info;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
		dev_info = slave_dev_info;		// Get slave's mapped info
	}
	else											// if slave NPU
	{
		dev_info = master_dev_info;		// Get master's mapped info
	}

	*(PCI_ADDR_EXT_REG) = (dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	
	eprintf("Reading from NPU...\n");

	while (!KEYPRESS)
	{
		temp = GET32(dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF);
//		eprintf("Read value = 0x%08X\n", temp);
	}
}
#endif
