/* pci_dma.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, Feb 2002
 * 
 * --------------------------------------------------------------------------
 */

#include "common.h"
#include "diagstruct.h"
#include "pci.h"
#include "pci_dma.h"
#include "hal_ixdp2400.h"
#include "uart.h"

extern pci_device slave_dev_info;		// for slave npu info
extern pci_device master_dev_info;		// for master npu info


/*=========================================================================*/
/* DMA engine test*/
/*=========================================================================*/
int dmaTest(void) 
{
	UINT32 x, /*passes=0, */loops, forever=0;

	// Deiva: The diTable[] was modified by removing the PCI_ADDR field
	static dmaInfoType diTable[] = {

  /* |- channel to test*/
  /* | |- put initial descriptor in DMA registers flag*/
  /* | | |- first descriptor start address*/
  /* | | |          |- number of descriptors to create, based on this descriptor*/
  /* | | |          | |- descriptor span*/
  /* | | |          | |  |- buffer span*/
  /* | | |          | |  |        |- first DRAM address*/
  /* | | |          | |  |        |                   |- size of transfer*/
  /* | | |          | |  |        |                   |        |- prescale*/
  /* | | |          | |  |        |                   |        | |- iBdelay*/
  /* | | |          | |  |        |                   |        | | |- direction of DMA*/
  /* | | |          | |  |        |                   |        | | |         |- PCI read type*/
  /* | | |          | |  |        |                   |        | | |         |        |- PCI read length*/
  /* | | |          | |  |        |                   |        | | |         |        | |- DRAM read length*/
  /* | | |          | |  |        |                   |        | | |         |        | | |- padd*/
	{1,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF1),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	#if 0
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,2,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,3,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,4,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_LINE,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_LINE,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,3,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,2,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,1,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,0,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,0,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},

	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,2,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,3,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},

	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},

	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,1,3,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,0,2,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,0,1,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,1,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,1,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,1,4,0}},

	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	#endif
	{0,0,0,0,0,0,0,{0,0,0,0,0,0,0,0}}};


	/*loops = (ac->argc == 2) ? 1 : ac->decArg[2];*/
	loops = 0;
	if (loops == 0)
	{
		forever = 1;           /* if params = 0, then do forever */
	}

//	eprintf("IXP2400 DMA engine test\n");

//	while (forever)
//	{
		for (x = 0; diTable[x].descriptorStart != 0; x++)
		{
//			eprintf(" DMA channel iteration index: %02d\n",x);

			// initialize the descriptors (if needed)
			initDescriptors((dmaInfoType *)&diTable[x]);

//			eprintf(" Starting DMA engines\n");

			//start the DMA engine(s)
			if (startDMA((dmaInfoType *)&diTable[x]) == ERROR)
			{
				return ERROR;
			}

//			eprintf(" Waiting for completion\n");

			// poll waiting for completion
			if (waitForCompletion((dmaInfoType *)&diTable[x]) == ERROR)
			{
				return ERROR;
			}

//			eprintf(" Verifying results\n");

			// verify the results are correct
			if (verifyResults((dmaInfoType *)&diTable[x]) == ERROR)
			{
				return ERROR;
			}
		}    
		
//		passes++;
		
//		if (KEYPRESS)
//		{
//			eprintf("A key was pressed...DMA test stopped\n");
//			return OK;
//		} // check for keypressed now
//	}  
	return OK;
}


/*=========================================================================*/
/* initialize the memory descriptors*/
/*=========================================================================*/
void initDescriptors(dmaInfoType *dmaInfo)
{
	byteCountRegType bcReg;
	PU32 /*dscPtr = (PU32) dmaInfo->descriptorStart, */oldPtr = 0;
	UINT32 /*x, tmpDsc, */offset=0;
	unsigned int pci_addr;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
		pci_addr = (slave_dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	}
	else
	{
		pci_addr = (master_dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	}

	// setup the bytecount register/descriptor bits
	bcReg.bits.byteCount = dmaInfo->control.sizeBytes;
	bcReg.bits.reserved = 0;	//bcReg.bits.iBDelay = dmaInfo->control.iBDelay;
	bcReg.bits.direction = dmaInfo->control.direction;
	bcReg.bits.eoChain = 0;

//	eprintf("DEBUG: In initDescriptors bcReg %08x\n",bcReg.dword);

	// if registerStart = 1, setup DMA transfer in DMA registers.

	// Channel = 3, is to do simultaneous DMA on channel 1 and 2.  If you want
	// to do a channel 3 registerStart must = 1 and you can not do a linked list
	// of the two channels at this time, so descriptorNum must = 0.

	if (dmaInfo->registerStart == 1)
	{
		/**/
		/* setup DMA registers to start the process*/
		/**/
		if (dmaInfo->channel&1)
		{
//			eprintf("\nDEBUG: pci_addr=0x%08X\n", pci_addr);
//			eprintf("DEBUG: dmaInfo->dramAddress=0x%08X\n", dmaInfo->dramAddress);
//			eprintf("DEBUG: dmaInfo->descriptorStart=0x%08X\n", dmaInfo->descriptorStart);
//			eprintf("DEBUG: bcReg.dword=0x%08X\n", bcReg.dword);

			PUT32(CHAN_1_PCI_ADDR, pci_addr);		// Deiva: This line was modified
			PUT32(CHAN_1_DRAM_ADDR, dmaInfo->dramAddress);
			PUT32(CHAN_1_DESC_PTR, dmaInfo->descriptorStart);
			PUT32(CHAN_1_BYTE_COUNT, bcReg.dword);

//			eprintf("\nDEBUG: CHAN_1_PCI_ADDR=0x%X\n", GET32(CHAN_1_PCI_ADDR));
//			eprintf("DEBUG: CHAN_1_DRAM_ADDR=0x%X\n", GET32(CHAN_1_DRAM_ADDR));
//			eprintf("DEBUG: CHAN_1_DESC_PTR=0x%X\n", GET32(CHAN_1_DESC_PTR));
//			eprintf("DEBUG: CHAN_1_BYTE_COUNT=0x%X\n\n", GET32(CHAN_1_BYTE_COUNT));

			// initialize the buffers with unique values in source, and
			// 0's in the destination
//D			initDmaBuffers(dmaInfo,0,dmaInfo->dramAddress, pci_addr);	// Deiva: This line was modified

			// update the offset for the next descriptor
//D			offset += dmaInfo->bufferSpan;
		}

		if (dmaInfo->channel&2)
		{
			PUT32(CHAN_2_PCI_ADDR, pci_addr + offset);		// Deiva: This line was modified
			PUT32(CHAN_2_DRAM_ADDR, dmaInfo->dramAddress+offset);
			PUT32(CHAN_2_DESC_PTR, dmaInfo->descriptorStart);
			PUT32(CHAN_2_BYTE_COUNT, bcReg.dword);

//			eprintf("\n\nDEBUG: CHAN_2_PCI_ADDR=0x%X\n", GET32(CHAN_2_PCI_ADDR));
//			eprintf("DEBUG: CHAN_2_DRAM_ADDR=0x%X\n", GET32(CHAN_2_DRAM_ADDR));
//			eprintf("DEBUG: CHAN_2_DESC_PTR=0x%X\n", GET32(CHAN_2_DESC_PTR));
//			eprintf("DEBUG: CHAN_2_BYTE_COUNT=0x%X\n\n", GET32(CHAN_2_BYTE_COUNT));

			// initialize the buffers with unique values in source, and
			// 0's in the destination
//D			initDmaBuffers(dmaInfo, 0, dmaInfo->dramAddress + offset, pci_addr + offset);	// Deiva: This line was modified

			// update the offset for the next descriptor
//D			offset += dmaInfo->bufferSpan;
		}
	}
	else
	{
		// setup DMA descriptor pointer
		if (dmaInfo->channel&1)
		{
			PUT32(CHAN_1_DESC_PTR, dmaInfo->descriptorStart);
		}
		
		if(dmaInfo->channel&2)
		{
			PUT32(CHAN_2_DESC_PTR, dmaInfo->descriptorStart+offset);
		}
	}


	// now setup the rest of the descriptors
//	for (x = 0; x < dmaInfo->descriptorNum; x++)
//	{
//		dscPtr[0] = bcReg.dword;
//		dscPtr[1] = pci_addr + offset;				// Deiva: This line was modified
//		dscPtr[2] = dmaInfo->dramAddress + offset;
//		initDmaBuffers(dmaInfo, x + 1, dmaInfo->dramAddress + offset, pci_addr + offset);	// Deiva: This line was modified
//
//		/ create the descriptor pointer
//		tmpDsc = (UINT32)&dscPtr[0] + dmaInfo->descriptorSpan;
//		dscPtr[3] = tmpDsc;
//		// eprintf("descriptor %d (BC:%08x,PCI:%08x,DRAM:%08x,NEXT:%08x)\n",
//		//  x,dscPtr[0],dscPtr[1],dscPtr[2],dscPtr[3]);
//
//		// save the old pointer so we can set the eoc bit in the last desriptor
//		oldPtr = dscPtr;
//		dscPtr = (PU32)tmpDsc;
//
//		// update the offset for the next descriptor
//		offset += dmaInfo->bufferSpan;
//	}

	// set the end of chain bit here, if number descriptors equals 0
	// we need to set it in the register
	if (dmaInfo->descriptorNum == 0)
	{
		if (dmaInfo->channel&1)
		{
			PUT32(CHAN_1_BYTE_COUNT, GET32(CHAN_1_BYTE_COUNT) | 0x80000000);
		}
		
		if (dmaInfo->channel&2)
		{
			PUT32(CHAN_2_BYTE_COUNT, GET32(CHAN_2_BYTE_COUNT) | 0x80000000);
		}
	}
	else
	{
		// eprintf("writing 80000000 -> %08x\n",&oldPtr[0]);
		oldPtr[0] |= 0x80000000;
	}
}


/*=========================================================================*/
/* initialize the memory descriptors*/
/*=========================================================================*/
void initDmaBuffers(dmaInfoType *dmaInfo, UINT32 descNum, UINT32 dramAddress, UINT32 pciAddress)
{
	int i;
	PU32 srcBufPntr, dstBufPntr;

	// put the PCI buffer into footbridge space
	pciAddress |= 0x80000000;

	// check which direction the transfer is going
	// eprintf("In initDmaBuffers, ");
	if (dmaInfo->control.direction == DRAMtoPCI)
	{
		srcBufPntr = (PU32)dramAddress;
		dstBufPntr = (PU32)pciAddress;
	}
	else
	{
		srcBufPntr = (PU32)pciAddress;
		dstBufPntr = (PU32)dramAddress;
	}

	// eprintf("src: %08x dst: %08x\n",srcBufPntr,dstBufPntr);

	// initialize the buffer, descNum will make sure that each descriptor
	// data block will contain unique data
	for (i = 0; i < (dmaInfo->control.sizeBytes / 4); i++)
	{
		dstBufPntr[i] = 0; 
		srcBufPntr[i] = i ^ 0xA5A4A3A2 ^ descNum;
//		eprintf("DEBUG: srcBufPntr[i]=0x%X\r", srcBufPntr[i]);
	}
//	eprintf("\n");
}


/*=========================================================================*/
/* start the DMA engine by writing the control register*/
/*=========================================================================*/
int startDMA(dmaInfoType *dmaInfo)
{
	dmaControlRegType controlReg;

	/**/
	/* setup the channel control bits. */
	/**/
	controlReg.dword = 0;
	controlReg.bits.enable = 1;
	controlReg.bits.firstInReg = dmaInfo->registerStart;
	// Deiva: Some code was removed at this point

	/**/
	/* start the DMA by writing control register of each channel*/
	/**/
	if (dmaInfo->channel & 1)
	{
		PUT32(CHAN_1_CONTROL, 0x8C);
		if (GET32(CHAN_1_CONTROL) != 0)
		{
			eprintf("ouch ! unable to reset DMA 1 control register %08x\n",GET32(CHAN_1_CONTROL));
			return ERROR;
		}
		/*eprintf("starting chan 1 DMA now... writing %08x -> control reg\n",controlReg.dword);*/
		PUT32(CHAN_1_CONTROL, controlReg.dword);
	}

	if (dmaInfo->channel&2)
	{
		PUT32(CHAN_2_CONTROL,0x8C);
		if (GET32(CHAN_2_CONTROL) != 0)
		{
			eprintf("ouch ! unable to reset DMA 2 control register %08x\n",GET32(CHAN_2_CONTROL));
			return ERROR;
		}
		/*eprintf("starting chan 2 DMA now... writing %08x -> control reg\n",controlReg.dword);*/
		PUT32(CHAN_2_CONTROL, controlReg.dword);
	}
	return OK;
}


/*=========================================================================*/
/* wait for some completion status from FootBridge*/
/*=========================================================================*/
UINT32 waitForCompletion(dmaInfoType *dmaInfo)
{
	UINT32 tmp,times = 0x10000,value;
	int mailbox0_addr;
	S8 done_1 = 0, done_2 = 0, done = 0, chk_1, chk_2;

	mailbox0_addr = PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF;

	chk_1 = dmaInfo->channel&1;
	chk_2 = dmaInfo->channel&2;

	/*eprintf("waiting for completion...\n");*/

	while (--times);

	times = 0x100000;

	/**/
	/* this will wait for Channel Transfer Done or Channel Error*/
	/* or Channel Chain Done*/
	/* */
	while ((!done) && times--)
	{
		if (chk_1)
		{
			if ((tmp = GET32(CHAN_1_CONTROL)) & 0x88)
			{
				done_1 = 1;            
				/*eprintf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",
				  GET32(CHAN_1_BYTE_COUNT),GET32(CHAN_1_PCI_ADDR),
				GET32(CHAN_1_DRAM_ADDR),GET32(CHAN_1_CONTROL));*/
				if (tmp & 0x8)
				{
					eprintf("ouch ! channel 1 error bit set\n");
					eprintf("BC:%08x, PCI:%08x DRAM:%08x CNTRL:%08x\n",
					GET32(CHAN_1_BYTE_COUNT),GET32(CHAN_1_PCI_ADDR),
					GET32(CHAN_1_DRAM_ADDR),GET32(CHAN_1_CONTROL));
				}
			}
		}
		else
		{
			done_1 = 1;
		}

		/**/
		/* write a unique pattern out*/
		/**/
		value = 0xa55a1234^times;
		PUT32(mailbox0_addr,value);

		/**/
		/* verify that it's getting there*/
		/**/
		if ((tmp=GET32(mailbox0_addr)) != value)
		{
			eprintf(" ouch ! unable to correctly access mailbox reg \n");
			eprintf("      addr: %08x actual: %08x exp: %08x xor: %08x   \n", mailbox0_addr, tmp, value, tmp^value);
			/**/
			/* check if we are to continue on error*/
			/**/
			if (checkErrorAndReturn())
			{
				return ERROR;
			}
		}

		if (chk_2)
		{
			if ((tmp=GET32(CHAN_2_CONTROL))&0x88)
			{
				done_2 = 1;            
				/*eprintf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",*/
				/*  GET32(CHAN_2_BYTE_COUNT),GET32(CHAN_2_PCI_ADDR),*/
				/*GET32(CHAN_2_DRAM_ADDR),GET32(CHAN_2_CONTROL));*/
				if (tmp&0x8)
				{
					eprintf("ouch ! channel 2 error bit set\n");
					eprintf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",
					GET32(CHAN_2_BYTE_COUNT),GET32(CHAN_2_PCI_ADDR),
					GET32(CHAN_2_DRAM_ADDR),GET32(CHAN_2_CONTROL));
					/**/
					/* check if we are to continue on error*/
					/**/
					if (checkErrorAndReturn())
					{
						return ERROR;
					}
				}
			}
		}
		else
		{
			done_2 = 1;
		}
	
		if (done_1 && done_2)
		{
			done = 1;
		}
	}
	
	if (times == 0)
	{
		eprintf("ouch ! timed out waiting for DMA completion\n");
		return ERROR;
	}

	return OK;      
}


/*=========================================================================*/
/* verify that counters and all transferred data are correct.*/
/*=========================================================================*/

UINT32 verifyResults(dmaInfoType *dmaInfo)
{
	UINT32 tmp, value, x, pval, dval, d;
	PU32 pciAddress, dramAddress;
	PU32 dscPtr = (PU32)dmaInfo->descriptorStart;
	UINT32 pci_addr, xscale_pci_addr;
	pci_device dev_info;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == MASTER)		// if master NPU
	{
		dev_info = slave_dev_info;
	}
	else
	{
		dev_info = master_dev_info;
	}

	pci_addr = (dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	*(PCI_ADDR_EXT_REG) = (dev_info.base_address[SDRAM_BAR] & 0xE0000000) >> 16;
	xscale_pci_addr = dev_info.base_map[SDRAM_BAR] + PCI_DRAM_OFFSET;

	// eprintf("verifying DMA transfer buffers...numdesc: %d\n",dmaInfo->descriptorNum);

	// check if single transfer, if so verify counters et al
	if (dmaInfo->descriptorNum == 0)
	{
		// check channel 1
		if (dmaInfo->channel & 1)
		{
			// check that the counter went to zero
			if ((tmp=GET32(CHAN_1_BYTE_COUNT) & 0xFFFFFF) != 0)
			{
				eprintf("ouch ! channel 1 byte count not 0\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_BYTE_COUNT, tmp, 0, tmp^0);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the PCI address incremented correctly
			value = pci_addr + dmaInfo->control.sizeBytes;	// Deiva: This line was modified

//			eprintf("DEBUG: value=0x%X CHAN_1_PCI_ADDR=0x%X\n", value, GET32(CHAN_1_PCI_ADDR));

			if ((tmp=GET32(CHAN_1_PCI_ADDR)) != value)
			{
				eprintf("ouch ! channel 1 PCI address incorrect\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_PCI_ADDR, tmp, value, tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the DRAM address incremented correctly*/
			value = dmaInfo->dramAddress + dmaInfo->control.sizeBytes;

//			eprintf("DEBUG: value=0x%X CHAN_1_DRAM_ADDR=0x%X\n", value, GET32(CHAN_1_DRAM_ADDR));

			if ((tmp=GET32(CHAN_1_DRAM_ADDR)) != value)
			{
				eprintf("ouch ! channel 1 DRAM address incorrect\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_DRAM_ADDR, tmp, value, tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}  

		// check channel 2
		if (dmaInfo->channel & 2)
		{
			if ((tmp=GET32(CHAN_2_BYTE_COUNT)&0xFFFFFF) != 0)
			{
				eprintf("ouch ! channel 2 byte count not 0\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n",
				CHAN_2_BYTE_COUNT,tmp,0,tmp^0);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the PCI address incremented correctly
			if( dmaInfo->channel == 3)
			{
				value = pci_addr + dmaInfo->control.sizeBytes + dmaInfo->bufferSpan;	// Deiva: This line was modified
			}
			else
			{
				value = pci_addr + dmaInfo->control.sizeBytes;	// Deiva: This line was modified
			}

			if ((tmp = GET32(CHAN_2_PCI_ADDR)) != value)
			{
				eprintf("ouch ! channel 2 PCI address incorrect\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_2_PCI_ADDR,tmp,value,tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the DRAM address incremented correctly
			if( dmaInfo->channel == 3)
			{
				value = dmaInfo->dramAddress+dmaInfo->control.sizeBytes+dmaInfo->bufferSpan;
			}
			else
			{
				value = dmaInfo->dramAddress+dmaInfo->control.sizeBytes;
			}

			if ((tmp=GET32(CHAN_2_DRAM_ADDR)) != value)
			{
				eprintf("ouch ! channel 2 DRAM address incorrect\n");
				eprintf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n",
				CHAN_2_DRAM_ADDR,tmp,value,tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}


		// verify that the source and destination buffers are the same
		pciAddress = (PU32)(xscale_pci_addr);    // Deiva: This line was modified
		dramAddress = (PU32)dmaInfo->dramAddress;    

		for (x = 0; x < (dmaInfo->control.sizeBytes / 4); x++)
		{
			if ((pval = pciAddress[x]) != (dval = dramAddress[x]))
			{
				eprintf("ouch ! DMA buffer compare error\n");
				if (dmaInfo->control.direction == DRAMtoPCI)
				{
					eprintf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
											&dramAddress[x], &pciAddress[x], dval, pval);
				}
				else
				{
					eprintf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
											&pciAddress[x], &dramAddress[x], pval, dval);
				}
				
				eprintf(", xor:%08x\n", pval^dval);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}
	}
	else
	{  
		// now go verify all the data transferred correctly, we need to traverse
		// the descriptors to find out what was supposed to happen.
		for (d=0;d<dmaInfo->descriptorNum;d++)
		{
			//eprintf("checking descriptor %d (BC:%08x,PCI:%08x,DRAM:%08x,NEXT:%08x)\n",
			//d,dscPtr[0],dscPtr[1],dscPtr[2],dscPtr[3]);
			pciAddress = (PU32)(dscPtr[1]|0x80000000);
			dramAddress = (PU32)dscPtr[2];
			for (x=0;x<(dscPtr[0]&0xFFFFFF);x++)
			{
				if ((pval=pciAddress[x]) != (dval=dramAddress[x]))
				{
					eprintf("ouch ! buffer compare error, descriptor %d\n",x);
					if (dmaInfo->control.direction == DRAMtoPCI)
					{
						eprintf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
						&pciAddress[x],&dramAddress[x],pval,dval);
					}
					else
					{
						eprintf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
						&dramAddress[x],&pciAddress[x],dval,pval);
					}
				
					eprintf(", xor:%08x\n",pval^dval);

					// check if we are to continue on error
					if (checkErrorAndReturn())
					{
						return ERROR;
					}
				}
			}

			// follow the list for more buffers
			dscPtr = (PU32)dscPtr[3];
		}
	}
	return OK;
}


int checkErrorAndReturn()
{
   return 1;
}
