/* pl_sys_loopback.c
 *
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: chinmay, NOVEMBER, 2002
 * 
 * 
 * ------------------------------------------------------------------------------
 */

typedef unsigned long ulong;
typedef unsigned short ushort;

#include "common.h"
#include "memory.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "syslog.h"
#include "uart.h"
#include "msf_init.h"
#include "diag_utils.h"


#include "error_code.h"
#include "error_map.h"
#include "reg_api.h"
#include "register_map.h"
#include "diag.h"
#include "dp_proto.h"
#include "mac_init.h"
#include "pl_sys_loopback_eg_rx.h"
#include "pl_sys_loopback_eg_tx.h"
#include "pl_sys_loopback_in_rx.h"
#include "pl_sys_loopback_in_tx.h"
#include "sys_loopback_pl_2_in_rx.h"
#include "sys_loopback_pl_2_in_tx.h"
#include "sys_loopback_pl_2_eg_rx.h"
#include "sys_loopback_pl_2_eg_tx.h"
#include "sys_loopback_pl_3_in_rx.h"
#include "sys_loopback_pl_3_in_tx.h"
#include "sys_loopback_pl_3_eg_rx.h"
#include "sys_loopback_pl_3_eg_tx.h"


extern const int pl_sys_loopback_eg_rx[];
extern const int pl_sys_loopback_eg_tx[];
extern const int pl_sys_loopback_in_rx[];
extern const int pl_sys_loopback_in_tx[];


#define DRAM_BASE_SRC			0x1000000
#define DRAM_BASE_DEST			0x2000000
#define SCRATCH_ADDR_OFF		0x200
#define RX_SRAM_BASE			0x400000
#define CARD_OPTION_ADDR		0x0
#define SCRATCH_MSG_BASE		0x1000
#define PACKET_ACTIVITY_ADDR	0x1200
#define SEQ_UPDATE_BASE			0x1300

#define SYS_LB2_MSG_ADDR		0x1500
#define SYS_LB3_MSG_ADDR		0x1500

#define FUTURE_COUNT_STEP_ADDR	0x1600
#define LOOP_COUNT_ADDR			0x1604
#define BURST_COUNT_ADDR		0x1608
#define ME_TO_XSCALE_MSG_ADDR	0x160C
#define XSCALE_MSG_ADDR			0x1610
#define END_TEST_ADDR			0x1620

#define PL_SYS_LOOPBACK_IN_RX_ME	0
#define PL_SYS_LOOPBACK_IN_TX_ME	1
#define PL_SYS_LOOPBACK_EG_RX_ME	0
#define PL_SYS_LOOPBACK_EG_TX_ME	1

#define SYS_LB2_BW_IN_RX_ME0	0
#define SYS_LB2_BW_IN_RX_ME1	1
#define SYS_LB2_BW_IN_RX_ME2	2
#define SYS_LB2_BW_IN_RX_ME3	3
#define SYS_LB2_BW_IN_TX_ME		16

#define SYS_LB2_BW_EG_TX_ME0	0
#define SYS_LB2_BW_EG_TX_ME1	1
#define SYS_LB2_BW_EG_TX_ME2	2
#define SYS_LB2_BW_EG_TX_ME3	3
#define SYS_LB2_BW_EG_RX_ME		16

#define SYS_LB3_BW_IN_RX_ME0	0
#define SYS_LB3_BW_IN_RX_ME1	1
#define SYS_LB3_BW_IN_RX_ME2	2
#define SYS_LB3_BW_IN_RX_ME3	3
#define SYS_LB3_BW_IN_TX_ME		16

#define SYS_LB3_BW_EG_TX_ME0	0
#define SYS_LB3_BW_EG_TX_ME1	1
#define SYS_LB3_BW_EG_TX_ME2	2
#define SYS_LB3_BW_EG_TX_ME3	3
#define SYS_LB3_BW_EG_RX_ME		16

#define RING_DATA_64_0			0x0002003C
#define RING_DATA_96_0			0x0000005C
#define RING_DATA_138_0			0x00040086
#define RING_DATA_64_1			0x0012003C
#define RING_DATA_96_1			0x0010005C
#define RING_DATA_138_1			0x00140086
#define RING_DATA_64_2			0x0022003C
#define RING_DATA_96_2			0x0020005C
#define RING_DATA_138_2			0x00240086
#define RING_DATA_64_3			0x0032003C
#define RING_DATA_96_3			0x0030005C
#define RING_DATA_138_3			0x00340086

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;

extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern pci_device slave_dev_info;		// for slave npu info.
extern pci_device master_dev_info;		// for master npu info.

#define TBUF_BASE				0xC8002000
#define TBUF_ELEM_SIZE			0x80
#define TOT_TBUF_ELEM			(0x2000 / TBUF_ELEM_SIZE)
#define SYS_LB2_DATA_SIZE		48
#define ELEM_IN_ARRAY			(SYS_LB2_DATA_SIZE / 4)

UINT32 pl_sys_lb_data96_0[] =	{ 0x00010002, 0x00030004, 0x00050006, 0x00070008,
									  0x0009000A, 0x000B000C, 0x000D000E, 0x000F0010,
									  0x00110012, 0x00130014, 0x00150016, 0x00170018, 
									  0x0019001A, 0x001B001C, 0x001D001E, 0x001F0020,
									  0x00210022, 0x00230024, 0x00250026, 0x00270028,
									  0x00110029, 0x002A002B, 0x002C0000					};


UINT32 pl_sys_lb_data64_0[] =	{ 0x01010102, 0x01030104, 0x01050106, 0x01070108,
									  0x0109010A, 0x010B010C, 0x010D010E, 0x010F0110,
									  0x01110112, 0x01130114, 0x01150116, 0x01170118,
									  0x0119011A, 0x011B011C, 0x011D0000					};


UINT32 pl_sys_lb_data138_0[] =	{ 0x02010202, 0x02030204, 0x02050206, 0x02070208,
										  0x0209020A, 0x020B020C, 0x020D020E, 0x020F0210,
										  0x02110212, 0x02130214, 0x02150216, 0x02170218,
										  0x0219021A, 0x021B021C, 0x021D021E, 0x021F0220,
										  0x02210222, 0x02230224, 0x02250226, 0x02270228,
										  0x0229022A, 0x022B022C, 0x022D022E, 0x022F0230,
										  0x02310232, 0x02330234, 0x02350236, 0x02370238,
										  0x0239023A, 0x023B023C, 0x023D023E, 0x023F0240,
										  0x02410242										};


UINT32 pl_sys_lb_data96_1[] =	{ 0x11010002, 0x11030004, 0x11050006, 0x11070008,
									  0x1109000A, 0x110B000C, 0x110D000E, 0x110F0010,
									  0x11110012, 0x11130014, 0x11150016, 0x11170018, 
									  0x1119001A, 0x111B001C, 0x111D001E, 0x111F0020,
									  0x11210022, 0x11230024, 0x11250026, 0x11270028,
									  0x11110029, 0x112A002B, 0x112C0000					};	


UINT32 pl_sys_lb_data64_1[] =	{ 0x11011102, 0x11031104, 0x11051106, 0x11071108,
									  0x1109110A, 0x110B110C, 0x110D110E, 0x110F1110,
									  0x11111112, 0x11131114, 0x11151116, 0x11171118,
									  0x1119111A, 0x111B111C, 0x111D0000					};


UINT32 pl_sys_lb_data138_1[] =	{ 0x12011202, 0x12031204, 0x12051206, 0x12071208,
										  0x1209120A, 0x120B120C, 0x120D120E, 0x120F1210,
										  0x12111212, 0x12131214, 0x12151216, 0x12171218,
										  0x1219121A, 0x121B121C, 0x121D121E, 0x121F1220,
										  0x12211222, 0x12231224, 0x12251226, 0x12271228,
										  0x1229122A, 0x122B122C, 0x122D122E, 0x122F1230,
										  0x12311232, 0x12331234, 0x12351236, 0x12371238,
										  0x1239123A, 0x123B123C, 0x123D123E, 0x123F1240,
										  0x12411242										};

UINT32 pl_sys_lb_data96_2[] =	{ 0x21010002, 0x21030004, 0x21050006, 0x21070008,
									  0x2109000A, 0x210B000C, 0x210D000E, 0x210F0010,
									  0x21110012, 0x21130014, 0x21150016, 0x21170018, 
									  0x2119001A, 0x211B001C, 0x211D001E, 0x211F0020,
									  0x21210022, 0x21230024, 0x21250026, 0x21270028,
									  0x21110029, 0x212A002B, 0x212C0000					};	


UINT32 pl_sys_lb_data64_2[] =	{ 0x21012102, 0x21032104, 0x21052106, 0x21072108,
									  0x2109210A, 0x210B210C, 0x210D210E, 0x210F2110,
									  0x21112112, 0x21132114, 0x21152116, 0x21172118,
									  0x2119211A, 0x211B211C, 0x211D0000					};


UINT32 pl_sys_lb_data138_2[] =	{ 0x22012202, 0x22032204, 0x22052206, 0x22072208,
										  0x2209220A, 0x220B220C, 0x220D220E, 0x220F2210,
										  0x22112212, 0x22132214, 0x22152216, 0x22172218,
										  0x2219221A, 0x221B221C, 0x221D221E, 0x221F2220,
										  0x22212222, 0x22232224, 0x22252226, 0x22272228,
										  0x2229222A, 0x222B222C, 0x222D222E, 0x222F2230,
										  0x22312232, 0x22332234, 0x22352236, 0x22372238,
										  0x2239223A, 0x223B223C, 0x223D223E, 0x223F2240,
										  0x22412242										};

UINT32 pl_sys_lb_data96_3[] =	{ 0x31010002, 0x31030004, 0x31050006, 0x31070008,
									  0x3109000A, 0x310B000C, 0x310D000E, 0x310F0010,
									  0x31110012, 0x31130014, 0x31150016, 0x31170018, 
									  0x3119001A, 0x311B001C, 0x311D001E, 0x311F0020,
									  0x31210022, 0x31230024, 0x31250026, 0x31270028,
									  0x31110029, 0x312A002B, 0x312C0000					};	


UINT32 pl_sys_lb_data64_3[] =	{ 0x31013102, 0x31033104, 0x31053106, 0x31073108,
									  0x3109310A, 0x310B310C, 0x310D310E, 0x310F3110,
									  0x31113112, 0x31133114, 0x31153116, 0x31173118,
									  0x3119311A, 0x311B311C, 0x311D0000					};


UINT32 pl_sys_lb_data138_3[] =	{ 0x32013202, 0x32033204, 0x32053206, 0x32073208,
										  0x3209320A, 0x320B320C, 0x320D320E, 0x320F3210,
										  0x32113212, 0x32133214, 0x32153216, 0x32173218,
										  0x3219321A, 0x321B321C, 0x321D321E, 0x321F3220,
										  0x32213222, 0x32233224, 0x32253226, 0x32273228,
										  0x3229322A, 0x322B322C, 0x322D322E, 0x322F3230,
										  0x32313232, 0x32333234, 0x32353236, 0x32373238,
										  0x3239323A, 0x323B323C, 0x323D323E, 0x323F3240,
										  0x32413242										};


UINT32 tx_data_pl[] =	{ RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_2, RING_DATA_96_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,		
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
						  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3	};

UINT32 tx_data_pl_023[] =	{ RING_DATA_96_1, RING_DATA_96_0, RING_DATA_96_2, RING_DATA_96_3,
							  RING_DATA_96_1, RING_DATA_96_0, RING_DATA_96_2, RING_DATA_96_3,
							  RING_DATA_96_1, RING_DATA_96_0, RING_DATA_96_2, RING_DATA_96_3,
							  RING_DATA_64_1, RING_DATA_64_0, RING_DATA_64_2, RING_DATA_64_3,
							  RING_DATA_64_1, RING_DATA_64_0, RING_DATA_64_2, RING_DATA_64_3,
							  RING_DATA_64_1, RING_DATA_64_0, RING_DATA_64_2, RING_DATA_64_3,
							  RING_DATA_138_1, RING_DATA_138_0, RING_DATA_138_2, RING_DATA_138_3,		
							  RING_DATA_138_1, RING_DATA_138_0, RING_DATA_138_2, RING_DATA_138_3,
							  RING_DATA_138_1, RING_DATA_138_0, RING_DATA_138_2, RING_DATA_138_3	};

UINT32 tx_data_pl_012[] =	{ RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_3, RING_DATA_96_2,
							  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_3, RING_DATA_96_2,
							  RING_DATA_96_0, RING_DATA_96_1, RING_DATA_96_3, RING_DATA_96_2,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_3, RING_DATA_64_2,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_3, RING_DATA_64_2,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_3, RING_DATA_64_2,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_3, RING_DATA_138_2,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_3, RING_DATA_138_2,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_3, RING_DATA_138_2	};

UINT32 tx_data_pl_0[] =	{	RING_DATA_96_0, RING_DATA_96_0, RING_DATA_96_0,
							RING_DATA_64_0, RING_DATA_64_0, RING_DATA_64_0,
							RING_DATA_138_0, RING_DATA_138_0, RING_DATA_138_0	};

UINT32 tx_data_pl_1[] =	{	RING_DATA_96_1, RING_DATA_96_1, RING_DATA_96_1,
							RING_DATA_64_1, RING_DATA_64_1, RING_DATA_64_1,
							RING_DATA_138_1, RING_DATA_138_1, RING_DATA_138_1	};

UINT32 tx_data_pl_2[] =	{	RING_DATA_96_2, RING_DATA_96_2, RING_DATA_96_2,
							RING_DATA_64_2, RING_DATA_64_2, RING_DATA_64_2,
							RING_DATA_138_2, RING_DATA_138_2, RING_DATA_138_2	};

UINT32 tx_data_pl_3[] =	{	RING_DATA_96_3, RING_DATA_96_3, RING_DATA_96_3,
							RING_DATA_64_3, RING_DATA_64_3, RING_DATA_64_3,
							RING_DATA_138_3, RING_DATA_138_3, RING_DATA_138_3	};

extern UINT32 sys_lb_data2_0[];
extern UINT32 sys_lb_data2_1[];
extern UINT32 sys_lb_data2_2[];
extern UINT32 sys_lb_data2_3[];

/** globals **/

extern UINT32 total_args,loop,ports,speed, cport;  
extern char channel,test_opt,dplx,loopback_mode;
extern UINT32 validate_params(void);
extern void spConfig(void);
extern void spRestore(void);
extern void get_stats(void);


// ******************************************************************************
// Function: void pl_SysLpbk_test(void)
//
// This function is called by the Test Manager when the user invokes the
// point Lone system loopback test
// ******************************************************************************
UINT32 pl_SysLpbk_test(void)
{

	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, tx_count=0, rx_count=0;
	UINT32 scratch_ring_tx_count, scratch_ring_rx;
	UINT32 dcache_status, forever = 0, combination, port_num, rx_port_num;
	UINT32 loop_count=0;
	UINT32 *pl_sys_lb_data=0;
	UINT32 scratch_read_value=0, dram_addr, orig_data_size=0;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 pass_port[4],rxflag[4];
	UINT32 expected_num_pkts = 0,iteration = 0, port_ctr=0, pnum=0, *pktptr=0;
	UINT32 exp_pkt_sz = 0,cflag = 0;

	/***************************************************************************
	** The rxport is to be indexed by the txport and the value is the corresponding rxport.
	** Tx port	  Rxport
	**	0			1
	**  1			0
	**  2			3
	**  3			2
	*********************************************************************************/
	UINT32 rxport[4] = {1, 0, 3, 2};
	UINT32 bad_pkt_size = 0;

    // set channel to fiber by default.

    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	loopback_mode = acL->argv[3][0];
	channel = acL->argv[4][0];
	speed = acL->decArg[5];	
	dplx = acL->argv[6][0]; 
	ports = acL->decArg[7];
	loop = acL->decArg[8];

	if (channel == 'c')
	{
		cflag = 1;
	}

	if (total_args <= 7)
	{
		loop = 1;
	}

	if (loop == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}

    if (loopback_mode != 'i')
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}
	}

	if (channel == 'm')
	{
		if (acL->argv[4][1] == '2')		// For indication of which port is copper
		{
			cport = 2;
		}
		else
		{
			cport = 0;
		}

		loopback_mode = 'e';
		cflag = 0;
		ports = 4;
	}

	/* initialize port array to 0 */
	for (ctr = 0; ctr < 4; ctr++)
	{
		pass_port[ctr] = 0;
		rxflag[ctr] = ERROR;
	}
	
	if (channel != 'm')
	{
		switch(ports)
		{
			case 0: pktptr = tx_data_pl_0;
					pnum = sizeof(tx_data_pl_0)/4; 
					iteration = 1;
					break;
			case 1: pktptr = tx_data_pl_1;
					pnum = sizeof(tx_data_pl_1)/4;
					iteration = 1;
					break;
			case 2: pktptr = tx_data_pl_2;
					pnum = sizeof(tx_data_pl_2)/4;
					iteration = 1;
					break;
			case 3: pktptr = tx_data_pl_3;
					pnum = sizeof(tx_data_pl_3)/4;
					iteration = 1;
					break;
			case 4: pktptr = tx_data_pl;
					pnum = sizeof(tx_data_pl)/(4*4);
					if (cflag)
					{
						iteration = 2;
					}
					else
					{
						iteration = 4;
					}
					break;
		}
	}
	else		// if (channel == 'm')
	{
		if (cport == 0)
		{	
			pktptr = tx_data_pl_023;
			pnum = sizeof(tx_data_pl_023) / (4 * 4);
		}
		else
		{	
			pktptr = tx_data_pl_012;
			pnum = sizeof(tx_data_pl_012) / (4 * 4);
		}

		iteration = 4;
	}

	scratch_ring_rx = SCR_RING_INSTR_ADDR + (5 << 2);
	scratch_ring_tx_count = SCR_RING_INSTR_ADDR + (4 << 2);

	if(acL->HostType == MASTER)		// if master NPU
	{
		flag = OK;

		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return ABORT;
		}

		eprintf("Initialising media card. Please wait...\n");

		status = init_mac_dev(0);
		if (status != DONE)
		{
			eprintf("Aborting Test\n");
			return ABORT;
		}

		if (loopback_mode == 'i')
		{
		    /* configure internal loopback */
		    spConfig();
		    reg_write(0x5b2,0xF00);
		    reg_write(0x5b3,0x0);	
		    spRestore();

			/** For Copper ports, the loopback scheme is A->B and C->D,
			    internal loopback always occurs on the same port. 
				hence the channel is forced to fiber. **/

			cflag = 0;
			channel = 'f';
			if (ports != 4)
			{
				iteration = 1;
			}
			else
			{
				iteration = 4;
			}
		}
		else
		{
			spConfig();
			reg_write(0x5b2,0x0);
			spRestore();
		}

		eprintf("Media card initialisation completed.\n");

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
		slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
		PUT32(slave_mailbox0_addr, 0);

#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif


		PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), DRAM_BASE_SRC);
		// Write packet data to DRAM for microcode to use
		for (ctr = 0; ctr < sizeof(pl_sys_lb_data96_0); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data64_0); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data138_0); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_0[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data96_1); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data64_1); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data138_1); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_1 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_1[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data96_2); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data64_2); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data138_2); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_2 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_2[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data96_3); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_96_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data96_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data64_3); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_64_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data64_3[(ctr / 4)]);
		}

		for (ctr = 0; ctr < sizeof(pl_sys_lb_data138_3); ctr+=4)
		{
			PUT32((DRAM_BASE_SRC + ((RING_DATA_138_3 >> 8) & (MASK_16BIT << 8)) + ctr), pl_sys_lb_data138_3[(ctr / 4)]);
		}

		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

		// Invoke the slave's PCI doorbell
		Set_Pci_Doorbell(PTLONE_SYS_LPBK, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		ctr = 0;

		// Wait for a signal that slave NPU has completed loading of ucode and init
		do
		{
			if (ctr == 1500)	// After 10 seconds
			{
				eprintf("\nTimed out...Slave NPU was not able to load microcode and init.\n");
				mailbox_value = INIT_ERROR;
				flag = ERROR;
				break;
			}

			temp = GET32(slave_mailbox0_addr);

			hal_delay_us(10000);
			ctr++;
		}
		while (temp == 0);
		
		if (flag == OK)	// If slave completed microcode loading successfully
		{
			// Load Master's NPU with ucode and init
			DownLoadUcode(PL_SYS_LOOPBACK_EG_TX_ME, sizeof(pl_sys_loopback_eg_tx), (UINT32 *)pl_sys_loopback_eg_tx);
			DownLoadUcode(PL_SYS_LOOPBACK_EG_RX_ME, sizeof(pl_sys_loopback_eg_rx), (UINT32 *)pl_sys_loopback_eg_rx);

			hal_delay_us(1000000);

			//eprintf("DEBUG: (sizeof(tx_data_pl) / sizeof(UINT32)) / 4 = %d\n", ((sizeof(tx_data_pl) / sizeof(UINT32)) / 4));

			do
			{
				for (combination = 0; combination < pnum; combination++)
				{
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
						{
							port_num = ports;
							PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), pktptr[combination]);
						}
						else
						{
							if (channel == 'm')
							{
								//if(port_ctr > cport)
								//{
								//	port_num = port_ctr + 1;
								//}
								//else
								//{
									port_num = port_ctr;
								//}

								PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), pktptr[((combination * 4) + port_ctr)]);
							}
							else
							{
								if (cflag)
								{
									port_num = port_ctr * 2;
								}
								else
								{
									port_num = port_ctr;
								}

								PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_pl[((combination * 4) + port_num)]);
							}
						}
					}
				}

				//data_to_send = tx_data_pl[internal_loop_count];
				//eprintf("DEBUG: internal_loop_count = %d\n", internal_loop_count);
				//eprintf("DEBUG: data_to_send = 0x%08X\n", data_to_send);
				//PUT32(scratch_ring_tx, data_to_send);

				for (combination = 0; combination < pnum; combination++)
				{
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (ports != 4)
						{
							port_num = ports;
						}
						else
						{
							if (channel == 'm')
							{
								//if (port_ctr > cport)
								//{
								//	port_num = port_ctr + 1;
								//}
								//else
								//{
									port_num = port_ctr;
								//}
							}
							else
							{
								if (cflag)
								{
									port_num = port_ctr * 2;
								}
								else		// if (cflag == 0)
								{
									port_num = port_ctr;
								}
							}
						}

						//eprintf("iteration = %d, port = %d\n",iteration,port_num);
						ctr = 0;
						do			// Do while waiting for all the packets to be transmitted
						{
							if (ctr == 100000)	// After 1 second
							{
								eprintf("\nTimed out...No indication from scratch ring that all packets were transmitted\n");
								mailbox_value = NOT_SENT;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if the packets have been transmitted
							if ((scratch_read_value = GET32(scratch_ring_tx_count)) == 0)
							{
								hal_delay_us(2);
								ctr++;
							}
							else
							{
								if (channel != 'm')
								{
									switch(ports)
									{
										case 0 : 
										case 1 : 
										case 2 : 
										case 3 : 
												eprintf("Transmitted on port %d: %d\r", ports,(scratch_read_value));
												break;
										case 4 : // all ports 
												if (!cflag)
												{
													eprintf("Transmitted on ports 0-3 : %d\r", (scratch_read_value)/4);
													iteration = 4;
												}
												else
												{
													eprintf("Transmitted on ports 0 and 2 : %d\r", (scratch_read_value)/2);
													iteration = 2;
												}
												break;
									}
								}
								else		// if (channel == 'm')
								{
									if (cport == 0)
									{
										eprintf("Transmitted on ports 0, 2 and 3 : %d\r", (scratch_read_value) / 4);
									}
									else
									{
										eprintf("Transmitted on ports 0, 1 and 2 : %d\r", (scratch_read_value) / 4);
									}
								}

								//eprintf("Transmitted Packets (total): %d\r", scratch_read_value);
								tx_count++;							  
							}
						}
						while (scratch_read_value == 0);

						PUT32(scratch_ring_tx_count, 0);

						tx_count =  scratch_read_value;
						if (flag == ERROR)
						{
							break;
						}
					}

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}
				
				for (combination = 0; combination < pnum; combination++)
				{
					for (port_ctr = 0; port_ctr < iteration; port_ctr++)
					{
						if (!cflag)		// if (cflag == 0)
						{
							if (ports != 4)
							{
								port_num = ports;
							}
							else
							{
								//if ((channel == 'm') && (port_ctr > cport))
								//{
								//	port_num = port_ctr + 1;
								//}
								//else
								//{
									port_num = port_ctr;
								//}
							}
						}
						else
						{
							if (ports != 4)
							{
								port_num = rxport[ports];
							}
							else
							{
								port_num = (port_ctr * 2) + 1;
							}
						}

						ctr = 0;

						do			// Do while waiting for a signal that a packet has been received
						{
							if (ctr == 30000)	// After 1 second
							{
								eprintf("\nTimed out...No indication from scratch ring that all packets were received\n");
								mailbox_value = NOT_RECEIVED;
								flag = ERROR;
								break;
							}

							// Poll on scratch ring to check if packets have been received
							if ((scratch_read_value = GET32(scratch_ring_rx)) == 0)
							{
								hal_delay_us(100);
								ctr++;
							}

							//eprintf("port %d  scratch_read_value = 0x%x\n",port_num,scratch_read_value);
						}
						while (scratch_read_value == 0);

						if (flag == ERROR)
						{
							break;
						}

						//eprintf("DEBUG: scratch_read_value = 0x%08X\n", scratch_read_value);

						rx_count++;
						pkt_size = scratch_read_value & MASK_8BIT;
						rx_port_num = (scratch_read_value >> 8) & MASK_8BIT;
						dram_addr = DRAM_BASE_DEST + ((scratch_read_value >> 16) << 8);

						// set the port on which packet was received.

						pass_port[rx_port_num]++;
						rxflag[rx_port_num] = OK;

						//eprintf("DEBUG: pkt_size=0x%08X\tdram_addr=0x%08X\n\n", pkt_size, dram_addr);

						/* Vallejo WORKAROUND : Vallejo is not stripping of the CRC 
							for 96 byte packets in gig/half duplex mode */
						if (pkt_size == 0x60) 
						{
							pkt_size = pkt_size - 4;
						}

						/****** end of Workaround *******/

						if (ports == 4)
						{
							if (channel != 'm')
							{
								exp_pkt_sz = tx_data_pl[((combination * 4) + port_num)] & 0xFFFF;
							}
							else
							{
								exp_pkt_sz = pktptr[((combination * 4) + port_ctr)] & 0xFFFF;
							}
						}
						else
						{
							exp_pkt_sz = pktptr[combination] & 0xFFFF;
						}

						if (pkt_size != exp_pkt_sz)
						{
							mailbox_value = INCORRECT_SIZE;
							flag = ERROR;		// Set flag as error
							rxflag[rx_port_num] = ERROR;
							break;
						}

						if (cflag)
						{
							switch(rx_port_num)
							{
								case 0:  // rx on port 0 , check for pkt transmitted from port 1
									if (pkt_size == (RING_DATA_96_1 & MASK_16BIT) )
									{
										orig_data_size = sizeof(pl_sys_lb_data96_1) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
									}
									else if (pkt_size == (RING_DATA_64_1 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data64_1) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
									}
									else if (pkt_size == (RING_DATA_138_1 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data138_1) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
									}
									else
										bad_pkt_size = 1;
									break;

								case 1:  // rx on port 1 , check for pkt transmitted from port 0
									if (pkt_size == (RING_DATA_96_0 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data96_0) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
									}
									else if (pkt_size == (RING_DATA_64_0 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data64_0) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
									}
									else if (pkt_size == (RING_DATA_138_0 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data138_0) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
									}
									else
									{
										bad_pkt_size = 1;
									}

									break;

								case 2:  // rx on port 2 , check for pkt transmitted from port 3
									if (pkt_size == (RING_DATA_96_3 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data96_3) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
									}
									else if (pkt_size == (RING_DATA_64_3 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data64_3) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
									}
									else if (pkt_size == (RING_DATA_138_3 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data138_3) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
									}
									else
									{
										bad_pkt_size = 1;
									}

									break;

								case 3:  // rx on port 2 , check for pkt transmitted from port 2
									if (pkt_size == (RING_DATA_96_2 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data96_2) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
									}
									else if (pkt_size == (RING_DATA_64_2 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data64_2) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
									}
									else if (pkt_size == (RING_DATA_138_2 & MASK_16BIT))
									{
										orig_data_size = sizeof(pl_sys_lb_data138_2) ;
										pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
									}
									else
									{
										bad_pkt_size = 1;
									}

									break;
							}

							if (bad_pkt_size ==1)
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
						}
						else  // cflag = 0 -> fiber mode or mixed mode
						{
							if ((pkt_size == (RING_DATA_96_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data96_0);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_0;
							}
							else if ((pkt_size == (RING_DATA_64_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data64_0);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_0;
							}
							else if ((pkt_size == (RING_DATA_138_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_0 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data138_0);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_0;
							}
							else if ((pkt_size == (RING_DATA_96_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data96_1);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_1;
							}
							else if ((pkt_size == (RING_DATA_64_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data64_1);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_1;
							}
							else if ((pkt_size == (RING_DATA_138_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_1 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data138_1);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_1;
							}
							else if ((pkt_size == (RING_DATA_96_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data96_2);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_2;
							}
							else if ((pkt_size == (RING_DATA_64_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data64_2);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_2;
							}
							else if ((pkt_size == (RING_DATA_138_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_2 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data138_2);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_2;
							}
							else if ((pkt_size == (RING_DATA_96_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_96_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data96_3);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data96_3;
							}
							else if ((pkt_size == (RING_DATA_64_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data64_3);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data64_3;
							}
							else if ((pkt_size == (RING_DATA_138_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_3 >> 20) & MASK_8BIT)))
							{
								orig_data_size = sizeof(pl_sys_lb_data138_3);
								pl_sys_lb_data = (UINT32 *)pl_sys_lb_data138_3;
							}
							else
							{
								eprintf("pkt size mismatch !! port = %d\n",rx_port_num);
								mailbox_value = NOT_COMPLETE;
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
						}

						for (ctr = 0; ctr < orig_data_size; ctr+=4)
						{
							// Read the DRAM for the longword that was received
							temp = GET32(dram_addr + ctr);

							// Compare with original value
							if (temp != pl_sys_lb_data[(ctr / 4)])
							{
								mailbox_value = ERROR_PATTERN;
								eprintf("Packet:%d Longword:%d Expected:0x%X  Received:0x%X\n", (rx_count + 1), (ctr / 4), pl_sys_lb_data[(ctr / 4)], temp);
								flag = ERROR;		// Set flag as error
								rxflag[rx_port_num] = ERROR;
								break;
							}
						}

						if (flag == ERROR)
						{
							break;
						}

						hal_delay_us(800);
					}

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}

				loop_count++;

				if (forever && KEYPRESS)
				{
					break;
				}

			}
			while ((loop_count != loop) || forever);

			

			if (loop == 0)
			{
				expected_num_pkts = loop_count * 9;
			}
			else
			{
				expected_num_pkts = loop * 9;
			}

			switch(ports)
			{
		       case 0:
		       case 1:
		       case 2:
		       case 3:  if (!cflag) 
						{ 
							if ((pass_port[ports] != expected_num_pkts) || (pass_port[ports] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", ports);
								eprintf("Packets Received = %d\n",pass_port[ports]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",ports,pass_port[ports]);
								flag = OK;
							}
						}
						else
						{
							if ((pass_port[rxport[ports]] != expected_num_pkts) || (pass_port[rxport[ports]] == 0))
							{
								eprintf("\nDid not Receive expected number of packets on port %d\n", rxport[ports]);
								eprintf("Packets Received on port %d = %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = ERROR;
								rxflag[ports] = ERROR;
							}
							else
							{
								eprintf("\nReceived on Port %d : %d\n",rxport[ports],pass_port[rxport[ports]]);
								flag = OK;
							}
						}

						break;

		       case 4:  
						eprintf("\n");

						if (!cflag)
						{
							for (ctr=0; ctr<4; ctr++)
							{
								if ((pass_port[ctr] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr);
									eprintf("Packets Received = %d\n",pass_port[ctr]);
									rxflag[ctr] = ERROR;
									flag = ERROR;
								}

								if (pass_port[ctr] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr],ctr);
								}
							}
						}
						else
						{
							for (ctr=0; ctr<2; ctr++)
							{
								if ((pass_port[ctr*2+1] != expected_num_pkts) || (pass_port[ports] == 0))
								{
									eprintf("\nDid not Receive expected number of packets on port %d\n", ctr*2+1);
									eprintf("Packets Received = %d\n",pass_port[ctr*2+1]);
									rxflag[ctr*2+1] = ERROR;
									flag = ERROR;
								}

								if (pass_port[ctr*2+1] == expected_num_pkts)
								{
									eprintf("Received %d packets on Port %d\n",pass_port[ctr*2+1],ctr*2+1);
								}
							}
						}

						break;

			   default:	eprintf("This is bad value for port number \n");
						break;
			}
		}
		eprintf("\n");

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}
		
		if (flag == OK)
		{
			eprintf("System loopback test...PASSED\n");
			Set_LED("LswP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			if (mailbox_value == INIT_ERROR)
			{
				eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
			}
			else if (mailbox_value == NOT_RECEIVED)
			{
				eprintf("The was no sign that the expected packet was received\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("The packet received was not the same as the packet transmitted.\n");
			}
			else if (mailbox_value == INCORRECT_SIZE)
			{
				eprintf("The packet received was not the same size as the packet transmitted.\n");
			}

			/* get statistics from Vallejo */
			get_stats();

			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "System loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));
			eprintf("System loopback test...FAIL!\n");
			Set_LED("LswF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		

		Clear_All_ME();
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}
	return DONE;
}


// ******************************************************************************
// Function: void Slave_pl_SysLpbk_test(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
void Slave_pl_SysLpbk_test(void)
{
	UINT32 status, scratch_ring_addr, scratch_val, ctr=0, verbose;

	scratch_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
	verbose = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
	// Load ME with microcode
	DownLoadUcode(PL_SYS_LOOPBACK_IN_RX_ME, sizeof(pl_sys_loopback_in_rx), (UINT32 *)pl_sys_loopback_in_rx);
	DownLoadUcode(PL_SYS_LOOPBACK_IN_TX_ME, sizeof(pl_sys_loopback_in_tx), (UINT32 *)pl_sys_loopback_in_tx);

	hal_delay_us(1000000);

	do
	{
		if ((scratch_val = GET32(scratch_ring_addr)))
		{
			hal_delay_us(1);
			eprintf("Packets (total): %d\r", scratch_val);
			ctr = 0;
		}
		else
		{
			hal_delay_us(10);
			ctr++;
		}
	}
	while (ctr != 500000);

	eprintf("\n");

	Clear_All_ME();
}

UINT32 Sys_Loopback2_PL_Master(void)
{
	UINT32 ctr, element, array, status;
	UINT32 dcache_status;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 port_ctr=0;
	UINT32 cflag = 0;
	UINT32 flag = OK;
	UINT32 *sys_lb_data2;
//	UINT32 activity_value1, activity_value3;

    // set channel to fiber by default.
    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	loopback_mode = acL->argv[3][0];
	channel = acL->argv[4][0]; 		   	
	dplx = acL->argv[6][0]; 
	speed = acL->decArg[5];	
	ports = 4;

	if (channel == 'c')
	{
		cflag = 1;
	}

    if (loopback_mode != 'i')
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}
	}

	if(acL->HostType != MASTER)		// if slave NPU
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}
	
	if (channel == 'm')
	{
		if (acL->argv[4][1] == '2')		// For indication of which port is copper
		{
			cport = 2;
		}
		else
		{
			cport = 0;
		}

		loopback_mode = 'e';
		cflag = 0;
		ports = 4;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return ABORT;
	}

	eprintf("Initialising media card. Please wait...\n");
	status = init_mac_dev(0);
	if (status != DONE)
	{
		eprintf("Aborting Test\n");
		return ABORT;
	}

	if (loopback_mode == 'i')
	{
	    /* configure internal loopback */
	    spConfig();
	    reg_write(0x5b2,0xF00);
	    reg_write(0x5b3,0x0);	
	    spRestore();

		/** For Copper ports, the loopback scheme is A->B and C->D,
		internal loopback always occurs on the same port. 
		hence the channel is forced to fiber. **/

		cflag = 0;
		channel = 'f';
	}
	else
	{
		spConfig();
		reg_write(0x5b2,0x0);
		spRestore();
	}

	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
		}
		else if (port_ctr == 1)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_1;
		}
		else if (port_ctr == 2)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_2;
		}
		else
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEM_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
				}
			}
		}
	}

	if(speed == 10)
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x1000);
	if(speed == 100)
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x200);
	if(speed == 1000)
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x140);

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Set_Pci_Doorbell(PTLONE_SYS_LPBK2, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

	mailbox_value = 0;

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB2_BW_EG_RX_ME, sizeof(sys_loopback_pl_2_eg_rx), (UINT32 *)sys_loopback_pl_2_eg_rx);
	
		if (dplx =='f')
		{
			DownLoadUcode(SYS_LB2_BW_EG_TX_ME1, sizeof(sys_loopback_pl_2_eg_tx), (UINT32 *)sys_loopback_pl_2_eg_tx);
		}

		DownLoadUcode(SYS_LB2_BW_EG_TX_ME2, sizeof(sys_loopback_pl_2_eg_tx), (UINT32 *)sys_loopback_pl_2_eg_tx);

		if (dplx =='f')
		{
			DownLoadUcode(SYS_LB2_BW_EG_TX_ME3, sizeof(sys_loopback_pl_2_eg_tx), (UINT32 *)sys_loopback_pl_2_eg_tx);
		}

		DownLoadUcode(SYS_LB2_BW_EG_TX_ME0, sizeof(sys_loopback_pl_2_eg_tx), (UINT32 *)sys_loopback_pl_2_eg_tx);
	
		eprintf("Press any key to stop the test...\n");

		do
		{
			mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
			hal_delay_us(10000);
		}
		while ((mailbox_value == 0) && (!KEYPRESS));
//		eprintf("ctr = %d\n", ctr);
//		eprintf("mailbox_value = %d\n", mailbox_value);
	}

	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (mailbox_value == INCORRECT_SIZE)
	{
		eprintf("An incorrect sized packet was received\n");
		flag = ERROR;
	}
	else if (mailbox_value == RECEIVE_ERROR)
	{
		eprintf("There was an error in the process of receiving the packet\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	if (flag == OK)
	{
		eprintf("System loopback test 2...PASSED\n");
		Set_LED("L2pP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 2...FAIL!\n");
		Set_LED("L2pF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return DONE;
}

void Sys_Loopback2_PL_Slave(void)
{
	UINT32 status, scratch_value, dcache_status, seq_port1_old=0, seq_port3_old=0, seq_port1_new, seq_port3_new;
	UINT32 ctr1, ctr3;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();

	PUT32((SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0x0), 0);
	PUT32((SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0x4), 0);
	PUT32((SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0x8), 0);
	PUT32((SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0xC), 0);

	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof(sys_loopback_pl_2_in_rx), (UINT32 *)sys_loopback_pl_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME1, sizeof(sys_loopback_pl_2_in_rx), (UINT32 *)sys_loopback_pl_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME2, sizeof(sys_loopback_pl_2_in_rx), (UINT32 *)sys_loopback_pl_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME3, sizeof(sys_loopback_pl_2_in_rx), (UINT32 *)sys_loopback_pl_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_TX_ME, sizeof(sys_loopback_pl_2_in_tx), (UINT32 *)sys_loopback_pl_2_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR), 0);

	ctr1 = 0;
	ctr3 = 0;

	do
	{
		seq_port1_new = GET32(SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0x4);
		seq_port3_new = GET32(SCRATCHPAD_BASE + SEQ_UPDATE_BASE + 0xC);

		if (seq_port1_old == seq_port1_new)
		{
			ctr1++;
		}
		else
		{
			ctr1 = 0;
			seq_port1_old = seq_port1_new;
		}

		if (seq_port3_old == seq_port3_new)
		{
			ctr3++;
		}
		else
		{
			ctr3 = 0;
			seq_port3_old = seq_port3_new;
		}

		hal_delay_us(10000);

		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR);
	}
	while ((scratch_value == 0) && (ctr1 != 300) && (ctr3 != 300));

	if ((ctr1 == 300) || (ctr3 == 300))
	{
		eprintf("ctr1=%d ctr3=%d\n", ctr1, ctr3);

		*(PCI_ADDR_EXT_REG) = (master_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;

#ifdef __ARMEB__
		PUT32((master_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SCRATCH_MSG_BASE), SWAP32(RECEIVE_ERROR));
#else
		PUT32((master_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SCRATCH_MSG_BASE), RECEIVE_ERROR);
#endif
		eprintf("SCRATCH_MSG_BASE=0x%08X\n", GET32(master_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SCRATCH_MSG_BASE));
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}

UINT32 Sys_Loopback3_PL_Master(void)
{
	UINT32 status, temp, flag = OK;
	UINT32 ctr, port_ctr, scratch_ring;
	UINT32 burst = 0, loops = 0;
	UINT32 dcache_status, mailbox_value, ring_value;
	UINT32 element, array;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 *sys_lb_data2;
	UINT32 cflag = 0;

	// set channel to fiber by default.
    channel = 'f';		
	total_args = acL->argc;
	test_opt = acL->argv[2][0];
	loopback_mode = acL->argv[3][0];
	channel = acL->argv[4][0]; 		   	
	speed = acL->decArg[5];
	dplx = acL->argv[6][0]; 
	burst = acL->decArg[7];
	loops = acL->decArg[8];
	ports = 4;

	if (channel == 'c')
	{
		cflag = 1;
	}

    if (loopback_mode != 'i')
	{
		if (validate_params() == INVALID_PARAMETERS)
		{
			eprintf("Invalid parameters. \n");
			return INVALID_PARAMETERS;
		}
	}

	if(acL->HostType != MASTER)		// if slave NPU
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return DONE;	// To indicate that the slave does not do this.
	}

	if (channel == 'm')
	{
		if (acL->argv[4][1] == '2')		// For indication of which port is copper
		{
			cport = 2;
		}
		else
		{
			cport = 0;
		}

		loopback_mode = 'e';
		cflag = 0;
		ports = 4;
	}

	if ((burst > 1024) || (burst == 0) || (loops == 0))
	{
		return INVALID_PARAMETERS;
	}

	scratch_ring = SCR_RING_INSTR_ADDR + (0 << 2);

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return ABORT;
	}

	eprintf("Initialising media card. Please wait...\n");

	status = init_mac_dev(0);
	if (status != DONE)
	{
		eprintf("Aborting Test\n");
		return ABORT;
	}

	if (loopback_mode == 'i')
	{
	    /* configure internal loopback */
	    spConfig();
	    reg_write(0x5b2,0xF00);
	    reg_write(0x5b3,0x0);	
	    spRestore();

		/** For Copper ports, the loopback scheme is A->B and C->D,
		internal loopback always occurs on the same port. 
		hence the channel is forced to fiber. **/

		cflag = 0;
		channel = 'f';
	}
	else
	{
		spConfig();
		reg_write(0x5b2,0x0);
		spRestore();
	}

	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
		}
		else if (port_ctr == 1)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_1;
		}
		else if (port_ctr == 2)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_2;
		}
		else
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEM_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
				}
			}
		}
	}

	if(speed == 10)
	{
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x1000);
	}
	else if(speed == 100)
	{
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x200);
	}
	else 	//if(speed == 1000)
	{
		PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), 0x140);
	}
		

	PUT32((SCRATCHPAD_BASE + LOOP_COUNT_ADDR), loops);
	PUT32((SCRATCHPAD_BASE + BURST_COUNT_ADDR), burst);

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Set_Pci_Doorbell(PTLONE_SYS_LPBK3, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

	mailbox_value = 0;

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB3_BW_EG_RX_ME, sizeof(sys_loopback_pl_3_eg_rx), (UINT32 *)sys_loopback_pl_3_eg_rx);
		if (dplx =='f')
		{
			DownLoadUcode(SYS_LB3_BW_EG_TX_ME1, sizeof(sys_loopback_pl_3_eg_tx), (UINT32 *)sys_loopback_pl_3_eg_tx);
		}
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME2, sizeof(sys_loopback_pl_3_eg_tx), (UINT32 *)sys_loopback_pl_3_eg_tx);
		if (dplx =='f')
		{
			DownLoadUcode(SYS_LB3_BW_EG_TX_ME1, sizeof(sys_loopback_pl_3_eg_tx), (UINT32 *)sys_loopback_pl_3_eg_tx);
		}
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME0, sizeof(sys_loopback_pl_3_eg_tx), (UINT32 *)sys_loopback_pl_3_eg_tx);
	
		if (loops == 0)
		{
			eprintf("Press any key to stop the test...\n");
		}

		ctr = 0;
		do
		{
			if (ctr == 50000)	// After 5 seconds
			{
				eprintf("Timed out...No signal that any packet was received.\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}

			ring_value = GET32(scratch_ring);

			hal_delay_us(100);

			if ((loops == 0) && KEYPRESS)
			{
				break;
			}

			ctr++;

			if (ring_value > 0)
			{
				eprintf("%d\r", ring_value);

				if ((ring_value == loops) && (loops != 0))
				{
					break;
				}

				ring_value = 0;
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x0), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x4), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x8), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0xC), 1);
				ctr = 0;
			}
		}
		while ((ring_value == 0) && !KEYPRESS);

		eprintf("\n");
	}

	mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (mailbox_value == INCORRECT_SIZE)
	{
		eprintf("An incorrect sized packet was received\n");
		flag = ERROR;
	}
	else if (mailbox_value == RECEIVE_ERROR)
	{
		eprintf("There was an error in the process of receiving the packet\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	if (flag == OK)
	{
		eprintf("System loopback test 3...PASSED\n");
		Set_LED("L3pP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 3...FAIL!\n");
		Set_LED("L3pF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return DONE;
}


void Sys_Loopback3_PL_Slave(void)
{
	UINT32 status, scratch_value, dcache_status;
//	UINT32 ctr1, ctr3, seq_port1_new, seq_port3_new, seq_port1_old, seq_port3_old;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME0, sizeof(sys_loopback_pl_3_in_rx), (UINT32 *)sys_loopback_pl_3_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME1, sizeof(sys_loopback_pl_3_in_rx), (UINT32 *)sys_loopback_pl_3_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME2, sizeof(sys_loopback_pl_3_in_rx), (UINT32 *)sys_loopback_pl_3_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME3, sizeof(sys_loopback_pl_3_in_rx), (UINT32 *)sys_loopback_pl_3_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_TX_ME, sizeof(sys_loopback_pl_3_in_tx), (UINT32 *)sys_loopback_pl_3_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR), 0);

	do
	{
		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR);
	}
	while (scratch_value == 0);

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}
