/* sf_loopback.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, April, 2002
 *  revisions: dalsraja, May 8, 2002
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "sf_loopback.h"
#include "sf_loopback_ucode.h"
#include "common.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "diag_utils.h"
#include "led.h"
#include "syslog.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "msf_init.h"
#include "memory.h"
#include "uart.h"
#include "uc_load.h"

#define LOOP					5

#define SF_LOOPBACK_RX_ME		0
#define SF_LOOPBACK_TX_ME		0

#define DRAM_BASE_SRC			0x1000000
#define SCRATCH_BASE_DEST		0x400

#define TOTAL_NUM_OF_BYTES		64

extern pci_device slave_dev_info;		// for slave npu info.

UINT32 sf_lb_data[] =	{ 0x00010203, 0x04050607, 0x08090A0B, 0x0C0D0E0F,
							  0x10111213, 0x14151617, 0x18191A1B, 0x1C1D1E1F,
							  0x20212223, 0x24252627, 0x28292A2B, 0x2C2D2E2F,
							  0x30313233, 0x34353637, 0x38393A3B, 0x3C3D3E3F };


//****************************************************************************
// Function: void Sf_Loopback_Test(void)
//
// This function is called by the Test Manager to run the switch fabric
// loopback test
//****************************************************************************
void Sf_Loopback_Test(void)
{
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, doorbell_intr_addr;
	UINT32 mailbox_value, prev_mailbox_val, flag;
	UINT32 ctr, loop_count, status, forever = 0;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 dcache_status;

	flag = OK;

	if (acL->argc > 3)
	{
		loop_count = acL->decArg[3];
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	slave_mailbox1_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_1_OFF;
	slave_mailbox2_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF;
	doorbell_intr_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF;
	
	if(acL->HostType == MASTER)		// if master NPU
	{
		Msf_OutOfReset();
		status = Msf_PLL();
		if (status == ERROR)
		{
			return;
		}
//		Set_Rcomp();
	
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		// Init slave's mailbox0 to 0
		PUT32(slave_mailbox0_addr, 0);
		PUT32(slave_mailbox1_addr, 0);

#ifdef __ARMEB__
		PUT32(slave_mailbox2_addr, SWAP32(loop_count));
#else
		PUT32(slave_mailbox2_addr, loop_count);
#endif

		DownLoadUcode(SF_LOOPBACK_TX_ME, sizeof(sf_loopback_rx), (UINT32 *)sf_loopback_rx);

		hal_delay_us(1000);

		// Initiate a doorbell interrupt on the slave it perform the loading
		// of the microcode on the slave's NPU.
		Set_Pci_Doorbell(SF_LB_BIT, doorbell_intr_addr);

		mailbox_value = 0;
		ctr = 0;
		do
		{
			prev_mailbox_val = mailbox_value;
			if (ctr == 30)	// After 15 seconds
			{
				eprintf("Timed out...No indication of packet received my slave\n");
				mailbox_value = SLAVE_MAILBOX1_NO_SIG;
				flag = ERROR;
				break;
			}

			// Poll on slave's MAILBOX0 to check if pass or fail
			mailbox_value = GET32(slave_mailbox1_addr);

#ifdef __ARMEB__
			mailbox_value = SWAP32(mailbox_value);
#endif
		
			hal_delay_us(500000);
	
			if (prev_mailbox_val != mailbox_value)
			{
				ctr = 0;
			}
			else
			{
				ctr++;
			}

			if (forever && KEYPRESS)
			{
				*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
				PUT32(slave_mailbox2_addr, 0);
				mailbox_value = 0;
				break;
			}
		}
		while ((mailbox_value != loop_count) || forever);


		ctr = 0;
		if (flag == OK)
		{
			do
			{
				if (ctr == 30)	// After 15 seconds
				{
					eprintf("Timed out...No signal from slave mailbox\n");
					mailbox_value = SLAVE_MAILBOX0_NO_SIG;
					flag = ERROR;
					break;
				}

				// Poll on slave's MAILBOX0 to check if pass or fail
				mailbox_value = GET32(slave_mailbox0_addr);

#ifdef __ARMEB__
				mailbox_value = SWAP32(mailbox_value);
#endif
			
				hal_delay_us(500000);
		
				ctr++;
			}
			while (mailbox_value == 0);
		}

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}

		if (mailbox_value == TEST_COMPLETE)
		{
			eprintf("\nSwitch fabric loopback test...PASSED\n");
			Set_LED("LsfP");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_PASS;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
		}
		else
		{
			if (mailbox_value == INCORRECT_SIZE)
			{
				eprintf("Packet received by Slave NPU was not the expected size\n");
			}
			else if (mailbox_value == SLAVE_MAILBOX0_NO_SIG)
			{
				eprintf("Packet was not received by slave NPU\n");
			}
			else if (mailbox_value == SLAVE_MAILBOX1_NO_SIG)
			{
				eprintf("Packet count not incremented by slave NPU\n");
			}
			else if (mailbox_value == ERROR_PATTERN)
			{
				eprintf("The packet received was not the same as the packet transmitted.\n");
			}
				
			// prepare structure for dumping
			syslog.type = TYPE_ERROR;
			syslog.source_comp = MASTER_DIAG;	// Determine master/slave
			strcpy(syslog.desc, "Switch fabric loopback test FAIL!");

			// write to syslog
			syslog_dump(&syslog, sizeof(SYSLOG_DATA));

			eprintf("\nSwitch fabric loopback test...FAIL!\n");
			Set_LED("LsfF");
			slowport_code[0] = SP_XSC;
			slowport_code[1] = SP_FAIL;
			dump_slowport(slowport_code, 2, SP_NON_FATAL);
			acL->Err = ERROR;
		}		

		// Invoke the slave's PCI doorbell
//		Set_Pci_Doorbell(CLEAR_ME_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
		Clear_All_ME();
	}
	else
	{
		eprintf("\nThis test cannot be run directly on the Slave console\n");
		return;	// To indicate that the slave does not do this.
	}
}


// ****************************************************************************
// Function: void Sf_Loopback_Test_Slave(void)
//
// This function is called by the PCI Doorbell interrupt handler when the
// master initiates a PCI doorbell interrupt by setting a particular bit on
// the slave's doorbell register
// ****************************************************************************
void Sf_Loopback_Test_Slave(void)
{
	UINT32 mask=0, mailbox_value, status, total_len, packet_num, rx_num_of_bytes=0;
	UINT32 get_ring_addr, put_ring_addr, loop_count, forever;
	UINT32 ctr, temp=0, flag, num_lw, lw1, lw2;
	UINT dcache_status;
	int payload_len;

	flag = OK;
	packet_num = 0;
	mailbox_value = 0;
	loop_count = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF);
	PUT32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF, 0xFFFFFFFF);

	if (loop_count == 0)
	{
		forever = 1;
	}
	else
	{
		forever = 0;
	}

	put_ring_addr = SCR_RING_INSTR_ADDR + (0 << 2);
	get_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();


	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	// Write packet data to DRAM for microcode to use
	for (ctr = 0; ctr < sizeof(sf_lb_data); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ctr), sf_lb_data[(ctr / 4)]);
	}

	// Load ME with microcode
	DownLoadUcode(SF_LOOPBACK_TX_ME, sizeof(sf_loopback_tx), (UINT32 *)sf_loopback_tx);

	hal_delay_us(100000);

	while ((loop_count != packet_num) || forever)	// Perform this loop for specified number of times
	{
		if (forever && (!(GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF))))
		{
			break;
		}

		for (ctr = 0; ctr < 128; ctr++)
		{
			PUT32((SCRATCHPAD_BASE + SCRATCH_BASE_DEST + (ctr << 2)), 0xAAAAAAAA);
		}

		// Before waiting for the next packet, write 0 to the scratchpad location
		// that will be polled on by the XScale
		PUT32(put_ring_addr, TOTAL_NUM_OF_BYTES);

		ctr = 0;
		do
		{
			if (ctr == 300000)	// After 3 seconds
			{
				eprintf("Timed out...Scratch ring 1 not written by ME\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}

			rx_num_of_bytes = GET32(get_ring_addr);

			hal_delay_us(10);
		
			ctr++;
		}
		while (rx_num_of_bytes == 0);

		packet_num++;
		eprintf("Received=%d\r", packet_num);
		PUT32(PCI_LOCAL_CSR_BASE + MAILBOX_1_OFF, packet_num);

		// Get payload size from header of the packet
		payload_len = ((GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST)) >> 16) & 0xFF;

		total_len = payload_len + 4;

		// calculate the number of long words(32-bits) of data in SRAM
		num_lw = (total_len >> 2) + ((total_len & 1) | ((total_len >> 1) & 1));

		if (num_lw != (rx_num_of_bytes / 4))
		{
			eprintf("There was an error in the size of the received packet...\n");
			mailbox_value = INCORRECT_SIZE;
			flag = ERROR;
			break;
		}

		for (ctr = 0; ctr < num_lw; ctr++)
		{
			if (payload_len >= 4)
			{
				lw1 = GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST + (ctr << 2));
				lw2 = GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST + ((ctr + 1) << 2));
				temp = ((lw1 & MASK_16BIT) << 16) | ((lw2 >> 16) & MASK_16BIT);
				mask = 0xFFFFFFFF;
			}
			else if (payload_len > 2)
			{
				lw1 = GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST + (ctr << 2));
				lw2 = GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST + ((ctr + 1) << 2));
				temp = ((lw1 & MASK_16BIT) << 16) | ((lw2 >> 16) & 0xFF00);
				mask = 0xFFFFFF00;
			}
			else if (payload_len > 0)
			{
				lw1 = GET32(SCRATCHPAD_BASE + SCRATCH_BASE_DEST + (ctr << 2));
				if (payload_len == 1)
				{
					temp = ((lw1 & 0xFF00) << 16);
					mask = 0xFF000000;
				}
				else
				{
					temp = ((lw1 & MASK_16BIT) << 16);
					mask = 0xFFFF0000;
				}
			}


			if (payload_len > 0)
			{
				if (temp != (sf_lb_data[ctr] & mask))
				{
					eprintf("Packet num=%d ctr=%d Expected:0x%08X Received:0x%08X\n", packet_num, ctr, sf_lb_data[ctr], temp);
					mailbox_value = ERROR_PATTERN;
					flag = ERROR;
				}
//				else
//				{
//					eprintf("Packet num=%d ctr=%d Expected:0x%08X Received:0x%08X\n", packet_num, ctr, sf_lb_data[ctr], temp);
//				}
			}

			payload_len-=4;
		}

		if (flag == ERROR)
		{
			break;
		}
	}
	eprintf("\n");

	if (flag == OK)
	{
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), TEST_COMPLETE);
	}
	else
	{
		eprintf("\nThere was an error in the process of receiving the packet.\n");
		PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), mailbox_value);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
	return;
}
