 /* sys_loopback.c
 *
 *-------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *-------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2400
 *  subsystem: DIAG
 *  author: dalsraja, April, 2002
 *  revisions: dalsraja, May 7, 2002
 * 
 * 
 * ------------------------------------------------------------------------------
 */

#include "sys_loopback.h"
#include "sys_loopback_ucode.h"
#include "common.h"
#include "memory.h"
#include "msf_common.h"
#include "diagstruct.h"
#include "led.h"
#include "uc_load.h"
#include "pci_diag.h"
#include "pci.h"
#include "hal_ixdp2400.h"
#include "memory.h"
#include "syslog.h"
#include "uart.h"
#include "msf_init.h"
#include "diag_utils.h"
#include "init_bdw.h"

UINT32 orig_sp_val[8];

extern const int rbuf_flush_bd[];
extern const int sizeof_rbuf_flush_bd;

extern const int rbuf_flush_bw[];
extern const int sizeof_rbuf_flush_bw;

extern void Set_Fibre_LB_1port(UINT32 port_num);
extern void Loopback_Help(void);

extern const InitRegTable_t BD_IntLoopbackArray[];
extern const InitRegTable_t BD_ExtLoopbackArray[];
extern const InitRegTable_t BW_IntLoopbackArray[];
extern const InitRegTable_t BW_ExtLoopbackArray[];

extern pci_device slave_dev_info;		// for slave npu info.

// Static Function Declarations
//==============================
static void Sys_Loopback_BD_Master(void);
static void Sys_Loopback_BD_Slave(void);

static void Sys_Loopback2_BD_Master(void);
static void Sys_Loopback2_BD_Slave(void);

static void Sys_Loopback3_BD_Master(void);
static void Sys_Loopback3_BD_Slave(void);

static void Sys_Loopback_BW_Master(void);
static void Sys_Loopback_BW_Slave(void);

static void Sys_Loopback2_BW_Master(void);
static void Sys_Loopback2_BW_Slave(void);

static void Sys_Loopback3_BW_Master(void);
static void Sys_Loopback3_BW_Slave(void);


UINT32 sys_lb_data48_0[] =	{ 0x00010002, 0x00030004, 0x00050006, 0x00070008,
									  0x0009000A, 0x000B000C, 0x000D000E, 0x000F0010,
									  0x00110012, 0x00130014, 0x00150000			  };

UINT32 sys_lb_data64_0[] =	{ 0x01010102, 0x01030104, 0x01050106, 0x01070108,
									  0x0109010A, 0x010B010C, 0x010D010E, 0x010F0110,
									  0x01110112, 0x01130114, 0x01150116, 0x01170118,
									  0x0119011A, 0x011B011C, 0x011D0000			  };


UINT32 sys_lb_data138_0[] =	{ 0x02010202, 0x02030204, 0x02050206, 0x02070208,
									  0x0209020A, 0x020B020C, 0x020D020E, 0x020F0210,
									  0x02110212, 0x02130214, 0x02150216, 0x02170218,
									  0x0219021A, 0x021B021C, 0x021D021E, 0x021F0220,
									  0x02210222, 0x02230224, 0x02250226, 0x02270228,
									  0x0229022A, 0x022B022C, 0x022D022E, 0x022F0230,
									  0x02310232, 0x02330234, 0x02350236, 0x02370238,
									  0x0239023A, 0x023B023C, 0x023D023E, 0x023F0240,
									  0x02410242									  };


UINT32 sys_lb_data48_1[] =	{ 0x10011002, 0x10031004, 0x10051006, 0x10071008,
									  0x1009100A, 0x100B100C, 0x100D100E, 0x100F1010,
									  0x10111012, 0x10131014, 0x10150000			  };

UINT32 sys_lb_data64_1[] =	{ 0x11011102, 0x11031104, 0x11051106, 0x11071108,
									  0x1109110A, 0x110B110C, 0x110D110E, 0x110F1110,
									  0x11111112, 0x11131114, 0x11151116, 0x11171118,
									  0x1119111A, 0x111B111C, 0x111D0000			  };


UINT32 sys_lb_data138_1[] =	{ 0x12011202, 0x12031204, 0x12051206, 0x12071208,
									  0x1209120A, 0x120B120C, 0x120D120E, 0x120F1210,
									  0x12111212, 0x12131214, 0x12151216, 0x12171218,
									  0x1219121A, 0x121B121C, 0x121D121E, 0x121F1220,
									  0x12211222, 0x12231224, 0x12251226, 0x12271228,
									  0x1229122A, 0x122B122C, 0x122D122E, 0x122F1230,
									  0x12311232, 0x12331234, 0x12351236, 0x12371238,
									  0x1239123A, 0x123B123C, 0x123D123E, 0x123F1240,
									  0x12411242									  };

UINT32 sys_lb_data48_2[] =	{ 0x20012002, 0x20032004, 0x20052006, 0x20072008,
									  0x2009200A, 0x200B200C, 0x200D200E, 0x200F2010,
									  0x20112012, 0x20132014, 0x20150000			  };

UINT32 sys_lb_data64_2[] =	{ 0x21012102, 0x21032104, 0x21052106, 0x21072108,
									  0x2109210A, 0x210B210C, 0x210D210E, 0x210F2110,
									  0x21112112, 0x21132114, 0x21152116, 0x21172118,
									  0x2119211A, 0x211B211C, 0x211D0000			  };


UINT32 sys_lb_data138_2[] =	{ 0x22012202, 0x22032204, 0x22052206, 0x22072208,
									  0x2209220A, 0x220B220C, 0x220D220E, 0x220F2210,
									  0x22112212, 0x22132214, 0x22152216, 0x22172218,
									  0x2219221A, 0x221B221C, 0x221D221E, 0x221F2220,
									  0x22212222, 0x22232224, 0x22252226, 0x22272228,
									  0x2229222A, 0x222B222C, 0x222D222E, 0x222F2230,
									  0x22312232, 0x22332234, 0x22352236, 0x22372238,
									  0x2239223A, 0x223B223C, 0x223D223E, 0x223F2240,
									  0x22412242									  };

UINT32 sys_lb_data48_3[] =	{ 0x30013002, 0x30033004, 0x30053006, 0x30073008,
									  0x3009300A, 0x300B300C, 0x300D300E, 0x300F3010,
									  0x30113012, 0x30133014, 0x30150000			  };

UINT32 sys_lb_data64_3[] =	{ 0x31013102, 0x31033104, 0x31053106, 0x31073108,
									  0x3109310A, 0x310B310C, 0x310D310E, 0x310F3110,
									  0x31113112, 0x31133114, 0x31153116, 0x31173118,
									  0x3119311A, 0x311B311C, 0x311D0000			  };


UINT32 sys_lb_data138_3[] =	{ 0x32013202, 0x32033204, 0x32053206, 0x32073208,
									  0x3209320A, 0x320B320C, 0x320D320E, 0x320F3210,
									  0x32113212, 0x32133214, 0x32153216, 0x32173218,
									  0x3219321A, 0x321B321C, 0x321D321E, 0x321F3220,
									  0x32213222, 0x32233224, 0x32253226, 0x32273228,
									  0x3229322A, 0x322B322C, 0x322D322E, 0x322F3230,
									  0x32313232, 0x32333234, 0x32353236, 0x32373238,
									  0x3239323A, 0x323B323C, 0x323D323E, 0x323F3240,
									  0x32413242									  };

UINT32 tx_data_bd[] =	{ RING_DATA_48_0, RING_DATA_64_0, RING_DATA_138_0,
							  RING_DATA_64_0, RING_DATA_138_0, RING_DATA_48_0,
							  RING_DATA_138_0, RING_DATA_48_0, RING_DATA_64_0,
							  RING_DATA_138_0, RING_DATA_64_0, RING_DATA_48_0	};

UINT32 tx_data_bw[] =	{ RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
							  RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
							  RING_DATA_138_0, RING_DATA_138_1, RING_DATA_138_2, RING_DATA_138_3,
							  RING_DATA_48_0, RING_DATA_48_1, RING_DATA_48_2, RING_DATA_48_3,
							  RING_DATA_64_0, RING_DATA_64_1, RING_DATA_64_2, RING_DATA_64_3		};

UINT32 sys_lb_data2_0[] = {	0x00000000, 0x00010002, 0x00030004, 0x00050006,
									0x00070008, 0x0009000A, 0x000B000C, 0x000D000E,
									0x000F0010, 0x00110012, 0x00130014, 0x00150016	};
//									0x00170018, 0x0019001A, 0x001B001C, 0x001D001E	};

UINT32 sys_lb_data2_1[] = {	0x00010000, 0x10011002, 0x10031004, 0x10051006,
									0x10071008, 0x1009100A, 0x100B100C, 0x100D100E,
									0x100F1010, 0x10111012, 0x10131014, 0x10151016	};
//									0x10171018, 0x1019101A, 0x101B101C, 0x101D101E  };

UINT32 sys_lb_data2_2[] = {	0x00020000, 0x20012002, 0x20032004, 0x20052006,
									0x20072008, 0x2009200A, 0x200B200C, 0x200D200E,
									0x200F2010, 0x20112012, 0x20132014, 0x20152016	};
//									0x20172018, 0x2019201A, 0x201B201C, 0x201D201E  };

UINT32 sys_lb_data2_3[] = {	0x00030000, 0x30013002, 0x30033004, 0x30053006,
									0x30073008, 0x3009300A, 0x300B300C, 0x300D300E,
									0x300F3010, 0x30113012, 0x30133014, 0x30153016	};
//									0x30173018, 0x3019301A, 0x301B301C, 0x301D301E  };



// ******************************************************************************
// Function: void Sys_Loopback_Test(void)
//
// This function is called by the Test Manager when the user invokes the
// system loopback test
// ******************************************************************************
void Sys_Loopback_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	opt = acL->argv[2][1];

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + CARD_OPTION_ADDR), SWAP32((UINT32)opt));	
	
	switch(opt)
	{
		case 'd':	Sys_Loopback_BD_Master();
					break;
		
		case 'w':	Sys_Loopback_BW_Master();
					break;

		default:	eprintf("You must specify the media card type.\n");
					Loopback_Help();
	}
}


void Sys_Loopback_Test_Slave(void)
{
	char opt;

	opt = (char)((GET32(SCRATCHPAD_BASE + CARD_OPTION_ADDR)) & 0xFF);
	

	switch(opt)
	{
		case 'd':	Sys_Loopback_BD_Slave();
					break;
		
		case 'w':	Sys_Loopback_BW_Slave();
					break;

		default:	eprintf("Master did not specify which media card is on the system.\n");
	}
}


void Sys_Loopback2_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	opt = acL->argv[2][1];

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + CARD_OPTION_ADDR), SWAP32((UINT32)opt));	
	
	switch(opt)
	{
		case 'd':	Sys_Loopback2_BD_Master();
					break;
		
		case 'w':	Sys_Loopback2_BW_Master();
					break;

		default:	eprintf("You must specify the media card type.\n");
					Loopback_Help();
	}
}


void Sys_Loopback2_Test_Slave(void)
{
	char opt;

	opt = (char)((GET32(SCRATCHPAD_BASE + CARD_OPTION_ADDR)) & 0xFF);

	switch(opt)
	{
		case 'd':	Sys_Loopback2_BD_Slave();
					break;
		
		case 'w':	Sys_Loopback2_BW_Slave();
					break;

		default:	eprintf("Master did not specify which media card is on the system.\n");
	}
}

void Sys_Loopback3_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	opt = acL->argv[2][1];

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + CARD_OPTION_ADDR), SWAP32((UINT32)opt));	
	
	switch(opt)
	{
		case 'd':	Sys_Loopback3_BD_Master();
					break;
		
		case 'w':	Sys_Loopback3_BW_Master();
					break;

		default:	eprintf("You must specify the media card type.\n");
					Loopback_Help();
	}
}


void Sys_Loopback3_Test_Slave(void)
{
	char opt;

	opt = (char)((GET32(SCRATCHPAD_BASE + CARD_OPTION_ADDR)) & 0xFF);

	switch(opt)
	{
		case 'd':	Sys_Loopback3_BD_Slave();
					break;
		
		case 'w':	Sys_Loopback3_BW_Slave();
					break;

		default:	eprintf("Master did not specify which media card is on the system.\n");
	}
}

void Save_N_Set_Sp_Vals(void)
{
	orig_sp_val[0] = GET32(0xC0080000);
	orig_sp_val[1] = GET32(0xC0080004);
	orig_sp_val[2] = GET32(0xC0080008);
	orig_sp_val[3] = GET32(0xC008000C);
	orig_sp_val[4] = GET32(0xC0080010);
	orig_sp_val[5] = GET32(0xC0080014);
	orig_sp_val[6] = GET32(0xC0080018);
	orig_sp_val[7] = GET32(0xC008001C);

	PUT32(0xC0080000, 0x3);
	PUT32(0xC0080008, 0x91);
	PUT32(0xC0080010, 0x91);
	PUT32(0xC0080014, 0);
	PUT32(0xC0080018, 0x3);
	PUT32(0xC008001C, 0x13);
}

void Restore_Sp_Vals(void)
{
	PUT32(0xC0080000, orig_sp_val[0]);
	PUT32(0xC0080004, orig_sp_val[1]);
	PUT32(0xC0080008, orig_sp_val[2]);
	PUT32(0xC008000C, orig_sp_val[3]);
	PUT32(0xC0080010, orig_sp_val[4]);
	PUT32(0xC0080014, orig_sp_val[5]);
	PUT32(0xC0080018, orig_sp_val[6]);
	PUT32(0xC008001C, orig_sp_val[7]);
}


// ******************************************************************************
// Function: void Sys_Loopback_BD_Master(void)
//
// This function is called by Sys_Loopback_Test
// ******************************************************************************
static void Sys_Loopback_BD_Master(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, count=0;
	UINT32 scratch_ring_tx, scratch_ring_rx;
	UINT32 dcache_status, forever = 0;	
	UINT8 loopback_mode;
	UINT32 loop_count, verbose=0, total_arg;
	UINT32 *sys_lb_data;
	UINT32 internal_loop_count, data_to_send, scratch_read_value=0, dram_addr, orig_data_size;
	bb_ChipData_t pc;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	total_arg = acL->argc;
	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];

	scratch_ring_rx = SCR_RING_INSTR_ADDR + (1 << 2);
	scratch_ring_tx = SCR_RING_INSTR_ADDR + (0 << 2);

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//		Set_Rcomp();

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
	}

	eprintf("Media card initialisation completed.\n");

	if (total_arg > 3)
	{
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	// Write packet data to DRAM for microcode to use
	for (ctr = 0; ctr < sizeof(sys_lb_data48_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_48_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data48_0[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data64_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data64_0[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data138_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data138_0[(ctr / 4)]);
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

//		pkt_size = PACKET_SIZE;

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	PUT32(slave_mailbox0_addr, 0);

	if ((total_arg == 6) && (verbose == 1))
	{
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 0);
	}
	else
	{
#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif
	}

	// Invoke the slave's PCI doorbell
	Set_Pci_Doorbell(SYS_LB_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;

	// Wait for a signal that slave NPU has completed loading of ucode and init
	do
	{
		if (ctr == 1500)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox0_addr);

		hal_delay_us(10000);
		ctr++;
	}
	while (temp == 0);
	
	if (flag == OK)	// If slave completed microcode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(SYS_LOOPBACK_EG_TX_ME, sizeof(sys_loopback_bd_eg_tx), (UINT32 *)sys_loopback_bd_eg_tx);
		DownLoadUcode(SYS_LOOPBACK_EG_RX_ME, sizeof(sys_loopback_bd_eg_rx), (UINT32 *)sys_loopback_bd_eg_rx);

		hal_delay_us(1000000);

		do
		{
			count++;
			eprintf("Loop: %d\r", count);

			internal_loop_count = 0;

			do
			{
				data_to_send = tx_data_bd[internal_loop_count];
//				eprintf("DEBUG: internal_loop_count = %d\n", internal_loop_count);
//				eprintf("DEBUG: data_to_send = 0x%08X\n", data_to_send);
				PUT32(scratch_ring_tx, data_to_send);

				ctr = 0;
				do			// Do while waiting for a signal that a packet has been received
				{
					if (ctr == 10000)	// After 5 seconds
					{
						eprintf("Timed out...The mailbox0 register did not indicate that a packet was received\n");
						mailbox_value = NOT_RECEIVED;
						flag = ERROR;
						break;
					}

					// Poll on MAILBOX0 to check if the test has been completed
					scratch_read_value = GET32(scratch_ring_rx);
		
					hal_delay_us(500);
	
					ctr++;
				}
				while (scratch_read_value == 0);

				if (flag == ERROR)
				{
					break;
				}

				pkt_size = scratch_read_value & 0xFFFF;
				dram_addr = DRAM_BASE_DEST + (scratch_read_value >> 8);
//				eprintf("DEBUG: pkt_size=0x%08X\tdram_addr=0x%08X\n\n", pkt_size, dram_addr);

				if (pkt_size != (data_to_send & 0xFFFF))
				{
					mailbox_value = INCORRECT_SIZE;
					flag = ERROR;		// Set flag as error
					break;
				}

//				eprintf("DEBUG: scratch_read_value & 0xFFFF=%d\n", (scratch_read_value & 0xFFFF));
//				eprintf("DEBUG: RING_DATA_48_0 & 0xFFFF=%d\n", (RING_DATA_48_0 & 0xFFFF));
//				eprintf("DEBUG: RING_DATA_64_0 & 0xFFFF=%d\n", (RING_DATA_64_0 & 0xFFFF));
//				eprintf("DEBUG: RING_DATA_138_0 & 0xFFFF=%d\n", (RING_DATA_138_0 & 0xFFFF));

				if ((scratch_read_value & 0xFFFF) == (RING_DATA_48_0 & 0xFFFF))
				{
					orig_data_size = sizeof(sys_lb_data48_0);
					sys_lb_data = (UINT32 *)sys_lb_data48_0;
				}
				else if ((scratch_read_value & 0xFFFF) == (RING_DATA_64_0 & 0xFFFF))
				{
					orig_data_size = sizeof(sys_lb_data64_0);
					sys_lb_data = (UINT32 *)sys_lb_data64_0;
				}
				else if ((scratch_read_value & 0xFFFF) == (RING_DATA_138_0 & 0xFFFF))
				{
					orig_data_size = sizeof(sys_lb_data138_0);
					sys_lb_data = (UINT32 *)sys_lb_data138_0;
				}
				else
				{
					mailbox_value = NOT_COMPLETE;
					flag = ERROR;		// Set flag as error
					break;
				}

				for (ctr = 0; ctr < orig_data_size; ctr+=4)
				{
					// Read the DRAM for the longword that was received
					temp = GET32(dram_addr + ctr);

					// Compare with original value
					if (temp != sys_lb_data[(ctr / 4)])
					{
						mailbox_value = ERROR_PATTERN;
						eprintf("Packet:%d Longword:%d Expected:0x%X  Received:0x%X\n", (internal_loop_count + 1), (ctr / 4), sys_lb_data[(ctr / 4)], temp);
						flag = ERROR;		// Set flag as error
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}

				internal_loop_count++;
			}
			while (internal_loop_count < (sizeof(tx_data_bd) / sizeof(UINT32)));

			if (flag == ERROR)
			{
				break;
			}

			if (forever && KEYPRESS)
			{
				PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_2_OFF), 0);
				break;
			}

			loop_count--;
		}
		while (loop_count || forever);
	}
	eprintf("\n");

	Restore_Sp_Vals();

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (flag == OK)
	{
		eprintf("System loopback test...PASSED\n");
		Set_LED("LsdP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		if (mailbox_value == INIT_ERROR)
		{
			eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("The was no sign that the expected packet was received\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted.\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted.\n");
		}

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test...FAIL!\n");
		Set_LED("LsdF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
	}		

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 0);

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Sys_Loopback_BD_Slave(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
static void Sys_Loopback_BD_Slave(void)
{
	UINT32 status, scratch_ring_addr, scratch_val, ctr=0;

	scratch_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	DownLoadUcode(SYS_LOOPBACK_IN_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	PUT32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF, 0xDAD0DAD1);

	// Load ME with microcode
	DownLoadUcode(SYS_LOOPBACK_IN_RX_ME, sizeof(sys_loopback_bd_in_rx), (UINT32 *)sys_loopback_bd_in_rx);
	DownLoadUcode(SYS_LOOPBACK_IN_TX_ME, sizeof(sys_loopback_bd_in_tx), (UINT32 *)sys_loopback_bd_in_tx);

	if (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF) == 1)
	{
		do
		{
			if ((scratch_val = GET32(scratch_ring_addr)))
			{
				if (scratch_val == RECEIVE_ERROR)
				{
					eprintf("The packet received had an error in it.\n");
					break;
				}
				eprintf("Loop: %d\r", scratch_val);
				ctr = 0;
			}
			hal_delay_us(5);
			ctr++;
		}
		while (ctr != 500000);
	}
	else
	{
		do
		{
			hal_delay_us(5);
		}
		while (GET32(PCI_LOCAL_CSR_BASE + MAILBOX_2_OFF));
	}

	Clear_All_ME();
}









// ******************************************************************************
// Function: void Sys_Loopback2_BD_Master(void)
//
// This function is implements a BD System Loopback Test in continuous mode
// ******************************************************************************
static void Sys_Loopback2_BD_Master(void)
{
	UINT32 status, temp, flag = OK;
	UINT32 ctr;
	UINT32 dcache_status, mailbox_value;
	UINT32 element, array;
	bb_ChipData_t pc;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 total_arg;
	UINT8 loopback_mode;
	UINT32 *sys_lb_data2;

	total_arg = acL->argc;
	loopback_mode = acL->argv[5][0];

	if (total_arg < 3)
	{
		Loopback_Help();
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();
	
	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
	}

	Restore_Sp_Vals();
	
	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
	
	for (element = 0; element < (TOT_TBUF_ELEM); element++)
	{
		for (array = 0; array < ELEM_IN_ARRAY; array++)
		{
			if (array == 0)
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
			}
			else
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
			}
		}
	}

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Set_Pci_Doorbell(SYS_LB2_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	// Check if slave has completed its init and download of microcode
	ctr = 0;	
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

//	PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), acL->decArg[4]);
	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET), 0);
	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET), 0);
	mailbox_value = 0;

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB2_BD_EG_RX_ME, sizeof(sys_loopback_bd_2_eg_rx), (UINT32 *)sys_loopback_bd_2_eg_rx);
		DownLoadUcode(SYS_LB2_BD_EG_TX_ME, sizeof(sys_loopback_bd_2_eg_tx), (UINT32 *)sys_loopback_bd_2_eg_tx);

		eprintf("Press any key to stop the test...\n");

		do
		{
			mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
			hal_delay_us(100);
		}
		while ((mailbox_value == 0) && !KEYPRESS);
	}

	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	eprintf("Packets received    = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET));
//	eprintf("Packets transmitted = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET));

	if (flag == OK)
	{
		eprintf("System loopback test 2...PASSED\n");
		Set_LED("L2dP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test 2 FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 2...FAIL!\n");
		Set_LED("L2dF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
		hal_delay_us(1000000);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}


// ******************************************************************************
// Function: void Sys_Loopback2_BD_Slave(void)
//
// This function is implements a BD System Loopback Test in continuous mode
// ******************************************************************************
void Sys_Loopback2_BD_Slave(void)
{
	UINT32 status, scratch_value, dcache_status;
//    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BD_IN_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(3000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET), 0);
	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET), 0);

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BD_IN_RX_ME, sizeof(sys_loopback_bd_2_in_rx), (UINT32 *)sys_loopback_bd_2_in_rx);
	DownLoadUcode(SYS_LB2_BD_IN_TX_ME, sizeof(sys_loopback_bd_2_in_tx), (UINT32 *)sys_loopback_bd_2_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR), 0);

	// Wait for Master's signal to stop test.
	do
	{
		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR);
	}
	while (scratch_value == 0);
/*
	eprintf("Packets received    = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET));
	eprintf("Packets transmitted = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET));
*/
	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}











// ******************************************************************************
// Function: void Sys_Loopback3_BD_Master(void)
//
// This function is similar to Sys_Loopback_BD_Master with additional burst option
// ******************************************************************************
static void Sys_Loopback3_BD_Master(void)
{
	UINT32 status, temp, flag = OK;
	UINT32 ctr, scratch_ring;
	UINT32 burst, loops;
	UINT32 dcache_status, mailbox_value, ring_value;
	UINT32 element, array;
	bb_ChipData_t pc;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 total_arg;
	UINT8 loopback_mode;
	UINT32 *sys_lb_data2;

	total_arg = acL->argc;
	burst = acL->decArg[3];
	loops = acL->decArg[4];
	loopback_mode = acL->argv[5][0];
//				  0  1	   2a		2b			2c			3		4				5				6
// eprintf("Diag> t lb <option>[media type][ATM mode*] [burst***] [loops] [media loopback mode] [media port]\n");
	if (total_arg < 5)
	{
		Loopback_Help();
		return;
	}

	if ((burst > 1024) || (burst == 0) || (loops == 0))
	{
		Loopback_Help();
		return;
	}

	scratch_ring = SCR_RING_INSTR_ADDR + (0 << 2);


	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_ExtLoopbackArray);
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BD_IntLoopbackArray);
	}

	Restore_Sp_Vals();
	
	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
	
	for (element = 0; element < (TOT_TBUF_ELEM); element++)
	{
		for (array = 0; array < ELEM_IN_ARRAY; array++)
		{
			if (array == 0)
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
			}
			else
			{
				PUT32((TBUF_BASE + (element * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
			}
		}
	}

	PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), /*ME_FUTURE_COUNT_VALUE*/5);
	PUT32((SCRATCHPAD_BASE + LOOP_COUNT_ADDR), loops);
	PUT32((SCRATCHPAD_BASE + BURST_COUNT_ADDR), burst);

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Set_Pci_Doorbell(SYS_LB3_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

	mailbox_value = 0;

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET), 0);
	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET), 0);

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB3_BD_EG_RX_ME, sizeof(sys_loopback_bd_3_eg_rx), (UINT32 *)sys_loopback_bd_3_eg_rx);
		DownLoadUcode(SYS_LB3_BD_EG_TX_ME, sizeof(sys_loopback_bd_3_eg_tx), (UINT32 *)sys_loopback_bd_3_eg_tx);
	
		if (loops == 0)
		{
			eprintf("Press any key to stop the test...\n");
		}

		ctr = 0;
		do
		{
			if (ctr == 5000000)	// After 5 seconds
			{
				eprintf("Timed out...No signal that any packet was received.\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}

			ring_value = GET32(scratch_ring);

//			hal_delay_us(1);

			if ((loops == 0) && KEYPRESS)
			{
				break;
			}

			ctr++;

			if (ring_value > 0)
			{
				eprintf("%d\r", ring_value);

				if ((ring_value == loops) && (loops != 0))
				{
					break;
				}

				ring_value = 0;
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x0), 1);
				ctr = 0;
			}
		}
		while ((ring_value == 0) && !KEYPRESS);

		eprintf("\n");
	}

	eprintf("Packets received    = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET));
	eprintf("Packets transmitted = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET));

	mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	if (flag == OK)
	{
		eprintf("System loopback test 3...PASSED\n");
		Set_LED("L3dP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test 3 FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 3...FAIL!\n");
		Set_LED("L3dF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
		hal_delay_us(1000000);
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
}

// ******************************************************************************
// Function: void Sys_Loopback3_BD_Slave(void)
//
// This function is similar to Sys_Loopback2_BD_Slave with additional burst option
// ******************************************************************************
void Sys_Loopback3_BD_Slave(void)
{
	UINT32 status, scratch_value, dcache_status;
//    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	
	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB3_BD_IN_RX_ME, sizeof_rbuf_flush_bd, (UINT32 *)rbuf_flush_bd);
	hal_delay_us(1000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET), 0);
	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET), 0);

	Clear_All_ME();
	DownLoadUcode(SYS_LB3_BD_IN_RX_ME, sizeof(sys_loopback_bd_3_in_rx), (UINT32 *)sys_loopback_bd_3_in_rx);
	DownLoadUcode(SYS_LB3_BD_IN_TX_ME, sizeof(sys_loopback_bd_3_in_tx), (UINT32 *)sys_loopback_bd_3_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR), 0);

	do
	{
		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR);
	}
	while (scratch_value == 0);

	eprintf("Packets received    = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_RX_OFFSET));
	eprintf("Packets transmitted = 0x%x\n", GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE + PKTS_TX_OFFSET));

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}













// ******************************************************************************
// Function: void Sys_Loopback_BW_Master(void)
//
// This function is called by Sys_Loopback_Test
// ******************************************************************************
static void Sys_Loopback_BW_Master(void)
{
	UINT32 ctr, flag, slave_mailbox0_addr, pkt_size, status, tx_count=0, rx_count=0;
	UINT32 scratch_ring_tx_count, scratch_ring_rx;
	UINT32 dcache_status, forever = 0, combination, port_num, rx_port_num;
	UINT8 loopback_mode;
	UINT32 loop_count, verbose=0, total_arg, ext_port_num;
	UINT32 *sys_lb_data;
	UINT32 scratch_read_value=0, dram_addr, orig_data_size;
	bb_ChipData_t pc;
	volatile int mailbox_value, temp;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	total_arg = acL->argc;
	loop_count = acL->decArg[3];
	loopback_mode = acL->argv[4][0];
	ext_port_num = acL->decArg[5];

	if ((total_arg == 6) && (ext_port_num > 4))
	{
		Loopback_Help();
		return;
	}

	scratch_ring_rx = SCR_RING_INSTR_ADDR + (5 << 2);
	scratch_ring_tx_count = SCR_RING_INSTR_ADDR + (4 << 2);

	flag = OK;

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((bb_RegPointer_type)IXF6048_BASE_ADDRESS, bb_6048_CHIP, &pc);

	IxfApiReset(&pc, 0, ixf6048_eRESET_CHIP);
	IxfApiReset(&pc, 0, ixf6048_eRESET_RX_FIFO);
	IxfApiReset(&pc, 0, ixf6048_eRESET_TX_FIFO);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");

		if ((total_arg > 5) && (ext_port_num >= 0) && (ext_port_num < 4))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
			Set_Fibre_LB_1port(ext_port_num);
		}
		else		// if ((total_arg < 6) || ((total_arg > 5) && (ext_port_num == 4)))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
	}

	eprintf("Media card initialisation completed.\n");

	if (total_arg > 3)
	{
		if (loop_count == 0)
		{
			forever = 1;
		}
	}
	else
	{
		loop_count = 1;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	slave_mailbox0_addr = slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_0_OFF;
	PUT32(slave_mailbox0_addr, 0);

	if ((total_arg == 6) && (verbose == 1))
	{
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 0);
	}
	else
	{
#ifdef __ARMEB__
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), SWAP32(1));
#else
		PUT32((slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + MAILBOX_3_OFF), 1);
#endif
	}

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), DRAM_BASE_SRC);
	// Write packet data to DRAM for microcode to use
	for (ctr = 0; ctr < sizeof(sys_lb_data48_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_48_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data48_0[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data64_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data64_0[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data138_0); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_138_0 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data138_0[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data48_1); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_48_1 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data48_1[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data64_1); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_1 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data64_1[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data138_1); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_138_1 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data138_1[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data48_2); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_48_2 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data48_2[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data64_2); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_2 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data64_2[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data138_2); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_138_2 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data138_2[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data48_3); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_48_3 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data48_3[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data64_3); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_64_3 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data64_3[(ctr / 4)]);
	}

	for (ctr = 0; ctr < sizeof(sys_lb_data138_3); ctr+=4)
	{
		PUT32((DRAM_BASE_SRC + ((RING_DATA_138_3 >> 8) & (MASK_16BIT << 8)) + ctr), sys_lb_data138_3[(ctr / 4)]);
	}

	PUT32((PCI_LOCAL_CSR_BASE + MAILBOX_0_OFF), 0);

//		pkt_size = PACKET_SIZE;

	// Invoke the slave's PCI doorbell
	Set_Pci_Doorbell(SYS_LB_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;

	// Wait for a signal that slave NPU has completed loading of ucode and init
	do
	{
		if (ctr == 1500)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			mailbox_value = INIT_ERROR;
			flag = ERROR;
			break;
		}

		temp = GET32(slave_mailbox0_addr);

		hal_delay_us(10000);
		ctr++;
	}
	while (temp == 0);
	
	if (flag == OK)	// If slave completed microcode loading successfully
	{
		// Load Master's NPU with ucode and init
		DownLoadUcode(SYS_LOOPBACK_EG_TX_ME, sizeof(sys_loopback_bw_eg_tx), (UINT32 *)sys_loopback_bw_eg_tx);
		DownLoadUcode(SYS_LOOPBACK_EG_RX_ME, sizeof(sys_loopback_bw_eg_rx), (UINT32 *)sys_loopback_bw_eg_rx);

		hal_delay_us(1000000);

//			eprintf("DEBUG: (sizeof(tx_data_bw) / sizeof(UINT32)) / 4 = %d\n", ((sizeof(tx_data_bw) / sizeof(UINT32)) / 4));

		do
		{
			for (combination = 0; combination < ((sizeof(tx_data_bw) / sizeof(UINT32)) / 4); combination++)
			{
				for (port_num = 0; port_num < 4; port_num++)
				{
					PUT32((SCR_RING_INSTR_ADDR + (port_num << 2)), tx_data_bw[((combination * 4) + port_num)]);
//						eprintf("DEBUG: data_to_send = 0x%08X\n", tx_data_bw[((combination * 4) + port_num)]);
					hal_delay_us(1);
				}
			}

//					data_to_send = tx_data_bw[internal_loop_count];
//					eprintf("DEBUG: internal_loop_count = %d\n", internal_loop_count);
//					eprintf("DEBUG: data_to_send = 0x%08X\n", data_to_send);
//					PUT32(scratch_ring_tx, data_to_send);

			for (combination = 0; combination < ((sizeof(tx_data_bw) / sizeof(UINT32)) / 4); combination++)
			{
				for (port_num = 0; port_num < 4; port_num++)
				{
					ctr = 0;

					do			// Do while waiting for all the packets to be transmitted
					{
						if (ctr == 500000)	// After 1 second
						{
							eprintf("Timed out...No indication from scratch ring that all packets were transmitted\n");
							mailbox_value = NOT_SENT;
							flag = ERROR;
							break;
						}

						// Poll on scratch ring to check if the packets have been transmitted
						if ((scratch_read_value = GET32(scratch_ring_tx_count)) == 0)
						{
							hal_delay_us(2);
							ctr++;
						}
						else
						{
							eprintf("Packets (total): %d\r", scratch_read_value);
							tx_count++;
						}
					}
					while (scratch_read_value == 0);

					if (flag == ERROR)
					{
						break;
					}
				}

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			for (combination = 0; combination < ((sizeof(tx_data_bw) / sizeof(UINT32)) / 4); combination++)
			{
				for (port_num = 0; port_num < 4; port_num++)
				{
					ctr = 0;

					do			// Do while waiting for a signal that a packet has been received
					{
						if (ctr == 500000)	// After 1 second
						{
							eprintf("Timed out...No indication from scratch ring that all packets were received\n");
							mailbox_value = NOT_RECEIVED;
							flag = ERROR;
							break;
						}

						// Poll on scratch ring to check if packets have been received
						if ((scratch_read_value = GET32(scratch_ring_rx)) == 0)
						{
							hal_delay_us(2);
							ctr++;
						}
					}
					while (scratch_read_value == 0);

					if (flag == ERROR)
					{
						break;
					}

//						eprintf("DEBUG: scratch_read_value = 0x%08X\n", scratch_read_value);

					rx_count++;
					pkt_size = scratch_read_value & MASK_8BIT;
					rx_port_num = (scratch_read_value >> 8) & MASK_8BIT;
					dram_addr = DRAM_BASE_DEST + ((scratch_read_value >> 16) << 8);
//							eprintf("DEBUG: pkt_size=0x%08X\tdram_addr=0x%08X\n\n", pkt_size, dram_addr);

//						eprintf("DEBUG: pkt_size=%d\ttx_data_bw=%d\n", pkt_size, (tx_data_bw[((combination * 4) + port_num)] & 0xFFFF));
					if (pkt_size != (tx_data_bw[((combination * 4) + port_num)] & 0xFFFF))
					{
						mailbox_value = INCORRECT_SIZE;
						flag = ERROR;		// Set flag as error
						break;
					}

					if ((pkt_size == (RING_DATA_48_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_48_0 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data48_0);
						sys_lb_data = (UINT32 *)sys_lb_data48_0;
					}
					else if ((pkt_size == (RING_DATA_64_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_0 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data64_0);
						sys_lb_data = (UINT32 *)sys_lb_data64_0;
					}
					else if ((pkt_size == (RING_DATA_138_0 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_0 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data138_0);
						sys_lb_data = (UINT32 *)sys_lb_data138_0;
					}
					else if ((pkt_size == (RING_DATA_48_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_48_1 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data48_1);
						sys_lb_data = (UINT32 *)sys_lb_data48_1;
					}
					else if ((pkt_size == (RING_DATA_64_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_1 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data64_1);
						sys_lb_data = (UINT32 *)sys_lb_data64_1;
					}
					else if ((pkt_size == (RING_DATA_138_1 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_1 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data138_1);
						sys_lb_data = (UINT32 *)sys_lb_data138_1;
					}
					else if ((pkt_size == (RING_DATA_48_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_48_2 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data48_2);
						sys_lb_data = (UINT32 *)sys_lb_data48_2;
					}
					else if ((pkt_size == (RING_DATA_64_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_2 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data64_2);
						sys_lb_data = (UINT32 *)sys_lb_data64_2;
					}
					else if ((pkt_size == (RING_DATA_138_2 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_2 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data138_2);
						sys_lb_data = (UINT32 *)sys_lb_data138_2;
					}
					else if ((pkt_size == (RING_DATA_48_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_48_3 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data48_3);
						sys_lb_data = (UINT32 *)sys_lb_data48_3;
					}
					else if ((pkt_size == (RING_DATA_64_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_64_3 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data64_3);
						sys_lb_data = (UINT32 *)sys_lb_data64_3;
					}
					else if ((pkt_size == (RING_DATA_138_3 & MASK_16BIT)) && (rx_port_num == ((RING_DATA_138_3 >> 20) & MASK_8BIT)))
					{
						orig_data_size = sizeof(sys_lb_data138_3);
						sys_lb_data = (UINT32 *)sys_lb_data138_3;
					}
					else
					{
						mailbox_value = NOT_COMPLETE;
						flag = ERROR;		// Set flag as error
						break;
					}

					for (ctr = 0; ctr < orig_data_size; ctr+=4)
					{
						// Read the DRAM for the longword that was received
						temp = GET32(dram_addr + ctr);

						// Compare with original value
						if (temp != sys_lb_data[(ctr / 4)])
						{
							mailbox_value = ERROR_PATTERN;
							eprintf("Packet:%d Longword:%d Expected:0x%X  Received:0x%X\n", (rx_count + 1), (ctr / 4), sys_lb_data[(ctr / 4)], temp);
							flag = ERROR;		// Set flag as error
							break;
						}
					}

					if (flag == ERROR)
					{
						break;
					}

					hal_delay_us(800);
				}

				if (flag == ERROR)
				{
					break;
				}
			}

			if (flag == ERROR)
			{
				break;
			}

			loop_count--;

			if (forever && KEYPRESS)
			{
				break;
			}
		}
		while (loop_count || forever);
	}
	eprintf("\n");

	Restore_Sp_Vals();

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (flag == OK)
	{
		eprintf("System loopback test...PASSED\n");
		Set_LED("LswP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		if (mailbox_value == INIT_ERROR)
		{
			eprintf("The slave NPU did not succeed in loading the microcode to its microengines\n");
		}
		else if (mailbox_value == NOT_RECEIVED)
		{
			eprintf("The was no sign that the expected packet was received\n");
		}
		else if (mailbox_value == ERROR_PATTERN)
		{
			eprintf("The packet received was not the same as the packet transmitted.\n");
		}
		else if (mailbox_value == INCORRECT_SIZE)
		{
			eprintf("The packet received was not the same size as the packet transmitted.\n");
		}
			
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test...FAIL!\n");
		Set_LED("LswF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
		hal_delay_us(1000000);
	}		

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Sys_Loopback_BW_Slave(void)
//
// This function will be called by the PCI Doorbell Interrupt handler after the
// Master NPU initiates a PCI Doorbell Interrupt on the Slave NPU
// ******************************************************************************
static void Sys_Loopback_BW_Slave(void)
{
	UINT32 status, scratch_ring_addr, scratch_val, ctr=0, verbose;

	scratch_ring_addr = SCR_RING_INSTR_ADDR + (1 << 2);
	verbose = GET32(PCI_LOCAL_CSR_BASE + MAILBOX_3_OFF);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	DownLoadUcode(SYS_LOOPBACK_IN_RX_ME, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}
//	Set_Rcomp();

	// Load ME with microcode
	DownLoadUcode(SYS_LOOPBACK_IN_RX_ME, sizeof(sys_loopback_bw_in_rx), (UINT32 *)sys_loopback_bw_in_rx);
	DownLoadUcode(SYS_LOOPBACK_IN_TX_ME, sizeof(sys_loopback_bw_in_tx), (UINT32 *)sys_loopback_bw_in_tx);

	hal_delay_us(1000000);

	do
	{
		if ((scratch_val = GET32(scratch_ring_addr)))
		{
			if (verbose == 1)
			{
				eprintf("Packets (total): %d\r", scratch_val);
			}

			ctr = 0;
		}
		else
		{
			hal_delay_us(1);
			ctr++;
		}
	}
	while (ctr != 500000);

	eprintf("\n");

	Clear_All_ME();
}

// ******************************************************************************
// Function: void Sys_Loopback2_BW_Master(void)
//
// This function is similar to Sys_Loopback_BW_Master but in continuous mode
// ******************************************************************************
static void Sys_Loopback2_BW_Master(void)
{
	UINT32 status, temp, flag = OK;
	UINT32 ctr, port_ctr;
	UINT32 dcache_status, mailbox_value;
	UINT32 element, array;
	bb_ChipData_t pc;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 total_arg, ext_port_num;
	UINT8 loopback_mode;
	UINT32 *sys_lb_data2;

	total_arg = acL->argc;
	loopback_mode = acL->argv[4][0];
	ext_port_num = acL->decArg[5];

	if ((total_arg == 6) && (ext_port_num > 4))
	{
		Loopback_Help();
		return;
	}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);

	if ((total_arg > 4) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");

		if ((total_arg > 5) && (ext_port_num >= 0) && (ext_port_num < 4))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
			Set_Fibre_LB_1port(ext_port_num);
		}
		else		// if ((total_arg < 6) || ((total_arg > 5) && (ext_port_num == 4)))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
	}

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
		}
		else if (port_ctr == 1)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_1;
		}
		else if (port_ctr == 2)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_2;
		}
		else
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEM_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
				}
			}
		}
	}

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Set_Pci_Doorbell(SYS_LB2_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

	mailbox_value = 0;

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB2_BW_EG_RX_ME, sizeof(sys_loopback_bw_2_eg_rx), (UINT32 *)sys_loopback_bw_2_eg_rx);
		DownLoadUcode(SYS_LB2_BW_EG_TX_ME1, sizeof(sys_loopback_bw_2_eg_tx), (UINT32 *)sys_loopback_bw_2_eg_tx);
		DownLoadUcode(SYS_LB2_BW_EG_TX_ME2, sizeof(sys_loopback_bw_2_eg_tx), (UINT32 *)sys_loopback_bw_2_eg_tx);
		DownLoadUcode(SYS_LB2_BW_EG_TX_ME3, sizeof(sys_loopback_bw_2_eg_tx), (UINT32 *)sys_loopback_bw_2_eg_tx);
		DownLoadUcode(SYS_LB2_BW_EG_TX_ME0, sizeof(sys_loopback_bw_2_eg_tx), (UINT32 *)sys_loopback_bw_2_eg_tx);
	
		eprintf("Press any key to stop the test...\n");

		do
		{
			mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
			hal_delay_us(100);
		}
		while ((mailbox_value == 0) && !KEYPRESS);
	}

	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB2_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (mailbox_value == INCORRECT_SIZE)
	{
		eprintf("An incorrect sized packet was received\n");
		flag = ERROR;
	}
	else if (mailbox_value == RECEIVE_ERROR)
	{
		eprintf("There was an error in the process of receiving the packet\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (flag == OK)
	{
		eprintf("System loopback test 2...PASSED\n");
		Set_LED("L2wP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 2...FAIL!\n");
		Set_LED("L2wF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
		hal_delay_us(1000000);
	}		

}


void Sys_Loopback2_BW_Slave(void)
{
	UINT32 status, scratch_value, dcache_status;
//    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME1, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME2, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME3, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB2_BW_IN_TX_ME, sizeof(sys_loopback_bw_2_in_tx), (UINT32 *)sys_loopback_bw_2_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR), 0);

	do
	{
		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB2_MSG_ADDR);
	}
	while (scratch_value == 0);

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}


// ******************************************************************************
// Function: void Sys_Loopback3_BW_Master(void)
//
// This function is similar to Sys_Loopback_BW_Master with additional burst option
// ******************************************************************************
static void Sys_Loopback3_BW_Master(void)
{
	UINT32 status, temp, flag = OK;
	UINT32 ctr, port_ctr, scratch_ring;
	UINT32 burst, loops;
	UINT32 dcache_status, mailbox_value, ring_value;
	UINT32 element, array;
	bb_ChipData_t pc;
    register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	UINT32 total_arg, ext_port_num;
	UINT8 loopback_mode;
	UINT32 *sys_lb_data2;

	total_arg = acL->argc;
	burst = acL->decArg[3];
	loops = acL->decArg[4];
	loopback_mode = acL->argv[5][0];
	ext_port_num = acL->decArg[6];

	if ((total_arg == 7) && (ext_port_num > 4))
	{
		Loopback_Help();
		return;
	}

	if (total_arg < 5)
	{
		Loopback_Help();
		return;
	}

	if ((burst > 1024) || (burst == 0) || (loops == 0))
	{
		Loopback_Help();
		return;
	}

	scratch_ring = SCR_RING_INSTR_ADDR + (0 << 2);


	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	eprintf("Initialising media card. Please wait...\n");

	Save_N_Set_Sp_Vals();

	IxfApiInit((unsigned short*) 0xc6000000, (ulong)3, &pc);

	if ((total_arg > 5) && (loopback_mode == 'e'))
	{
		eprintf("Setting to fiber loopback mode...\n");

		if ((total_arg > 6) && (ext_port_num >= 0) && (ext_port_num < 4))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
			Set_Fibre_LB_1port(ext_port_num);
		}
		else		// if ((total_arg < 6) || ((total_arg > 5) && (ext_port_num == 4)))
		{
			Ixf6048InitChip(&pc, (InitRegTable_t *)BW_ExtLoopbackArray);
		}
	}
	else
	{
		eprintf("Setting to internal loopback mode...\n");
		Ixf6048InitChip(&pc, (InitRegTable_t *)BW_IntLoopbackArray);
	}

	Restore_Sp_Vals();

	eprintf("Media card initialisation completed.\n");

	// Put the data into the TBUF for transmitting
	for (port_ctr = 0; port_ctr < 4; port_ctr++)
	{
		if (port_ctr == 0)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_0;
		}
		else if (port_ctr == 1)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_1;
		}
		else if (port_ctr == 2)
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_2;
		}
		else
		{
			sys_lb_data2 = (UINT32 *)sys_lb_data2_3;
		}

		for (element = 0; element < (TOT_TBUF_ELEM/4); element++)
		{
			for (array = 0; array < ELEM_IN_ARRAY; array++)
			{
				if (array == 0)
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE)), (sys_lb_data2[array] | (element << 24)));
				}
				else
				{
					PUT32((TBUF_BASE + (((port_ctr * (TOT_TBUF_ELEM/4)) + element) * TBUF_ELEM_SIZE) + (array * 4)), sys_lb_data2[array]);
				}
			}
		}
	}

	PUT32((SCRATCHPAD_BASE + FUTURE_COUNT_STEP_ADDR), /*ME_FUTURE_COUNT_VALUE*/0x400);

	PUT32((SCRATCHPAD_BASE + LOOP_COUNT_ADDR), loops);
	PUT32((SCRATCHPAD_BASE + BURST_COUNT_ADDR), burst);

	PUT32((SCRATCHPAD_BASE + SCRATCH_MSG_BASE), 0);
	*(PCI_ADDR_EXT_REG) = (slave_dev_info.base_address[CSR_BAR] & 0xE0000000) >> 16;
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Set_Pci_Doorbell(SYS_LB3_BIT, (slave_dev_info.base_map[CSR_BAR] + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	ctr = 0;
	do
	{
		if (ctr == 15000)	// After 10 seconds
		{
			eprintf("Timed out...Slave NPU was not able to load microcode and init.\n");
			flag = ERROR;
			break;
		}

		temp = GET32(slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR);

		hal_delay_us(1000);
		ctr++;
	}
	while (temp != 0);

	mailbox_value = 0;

	if (flag == OK)
	{
		DownLoadUcode(SYS_LB3_BW_EG_RX_ME, sizeof(sys_loopback_bw_3_eg_rx), (UINT32 *)sys_loopback_bw_3_eg_rx);
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME1, sizeof(sys_loopback_bw_3_eg_tx), (UINT32 *)sys_loopback_bw_3_eg_tx);
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME2, sizeof(sys_loopback_bw_3_eg_tx), (UINT32 *)sys_loopback_bw_3_eg_tx);
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME3, sizeof(sys_loopback_bw_3_eg_tx), (UINT32 *)sys_loopback_bw_3_eg_tx);
		DownLoadUcode(SYS_LB3_BW_EG_TX_ME0, sizeof(sys_loopback_bw_3_eg_tx), (UINT32 *)sys_loopback_bw_3_eg_tx);
	
		if (loops == 0)
		{
			eprintf("Press any key to stop the test...\n");
		}

		do
		{
			if (ctr == 50000)	// After 5 seconds
			{
				eprintf("Timed out...No signal that any packet was received.\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}

			ring_value = GET32(scratch_ring);

			hal_delay_us(100);

			if ((loops == 0) && KEYPRESS)
			{
				break;
			}

			if (ring_value > 0)
			{
				eprintf("%d\r", ring_value);

				if ((ring_value == loops) && (loops != 0))
				{
					break;
				}

				ring_value = 0;
//				hal_delay_us(500);
//				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x0), 0);
//				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x4), 0);
//				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x8), 0);
//				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0xC), 0);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x0), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x4), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0x8), 1);
				PUT32((SCRATCHPAD_BASE + XSCALE_MSG_ADDR + 0xC), 1);
				ctr = 0;
			}
		}
		while ((ring_value == 0) && !KEYPRESS);

		eprintf("\n");
/*		do
		{
			if (ctr == 50000)	// After 10 seconds
			{
				eprintf("Timed out...No signal of test ending.\n");
				mailbox_value = NOT_RECEIVED;
				flag = ERROR;
				break;
			}
scratch_ring
			mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);

			hal_delay_us(100);

			if ((loops == 0) && KEYPRESS)
			{
				eprintf("User intervetion...\n");
				break;
			}
		}
		while (mailbox_value == 0);
*/	}


	mailbox_value = GET32(SCRATCHPAD_BASE + SCRATCH_MSG_BASE);
	PUT32((slave_dev_info.base_map[CSR_BAR] + SCRATCH_BASE_FRM_PCI + SYS_LB3_MSG_ADDR), 1);
	Clear_All_ME();

	if (mailbox_value == SEQUENCE_ERROR)
	{
		eprintf("A sequence error was seen\n");
		flag = ERROR;
	}
	else if (mailbox_value == INCORRECT_SIZE)
	{
		eprintf("An incorrect sized packet was received\n");
		flag = ERROR;
	}
	else if (mailbox_value == RECEIVE_ERROR)
	{
		eprintf("There was an error in the process of receiving the packet\n");
		flag = ERROR;
	}
	else if (KEYPRESS)
	{
		eprintf("User intervetion...\n");
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	if (flag == OK)
	{
		eprintf("System loopback test 3...PASSED\n");
		Set_LED("L3wP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "System loopback test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		eprintf("System loopback test 3...FAIL!\n");
		Set_LED("L3wF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		acL->Err = ERROR;
		hal_delay_us(1000000);
	}		

}


void Sys_Loopback3_BW_Slave(void)
{
	UINT32 status, scratch_value, dcache_status;
//    register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB2_BW_IN_RX_ME0, sizeof_rbuf_flush_bw, (UINT32 *)rbuf_flush_bw);
	hal_delay_us(1000000);

	Clear_All_ME();
	Msf_OutOfReset();
	status = Msf_PLL();
	if (status == ERROR)
	{
		return;
	}

	Clear_All_ME();
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME0, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME1, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME2, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_RX_ME3, sizeof(sys_loopback_bw_2_in_rx), (UINT32 *)sys_loopback_bw_2_in_rx);
	DownLoadUcode(SYS_LB3_BW_IN_TX_ME, sizeof(sys_loopback_bw_2_in_tx), (UINT32 *)sys_loopback_bw_2_in_tx);

	PUT32((SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR), 0);

	do
	{
		scratch_value = GET32(SCRATCHPAD_BASE + SYS_LB3_MSG_ADDR);
	}
	while (scratch_value == 0);

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	Clear_All_ME();
}
