/* temperature.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Apr 18, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "temperature.h"
#include "led.h"
#include "i2c.h"
#include "diag_utils.h"
#include "misc_func.h"
#include "hal_ixdp2400.h"
#include "uart.h"
#include "syslog.h"
#include "hal_intr.h"

// static function prototypes
static void TMon_ID_Test(void);
static void TMon_Register_Test(void);
static void TMon_Help(void);

//
// Function: Tmon_Help
//
// Description: Displays all the options available for the Tmon test
//				on the console.
//
static void TMon_Help(void)
{
	eprintf("> t tmon <option>,\n");
	eprintf("where <option> are\n");
	eprintf("r - Temp. ID test\n");
	eprintf("v - Temp. Registers test\n");
	eprintf("a - all tests\n");
	eprintf("h - help\n");	
}

//
// Function: TMon_Test
//
// Description: Calls the appropriate function(s) of the Tmon test
//				depending on user input.
//
void TMon_Test(void)
{
	char opt;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	if(acL -> HostType != MASTER)
	{
		eprintf("This Test is only available on the Master!\n");
	}
	else
	{

		opt = acL->argv[2][0];

		switch(opt)
		{
			case 'r': 	TMon_ID_Test();
						break;

			case 'v': 	TMon_Register_Test();
						break;

			case 'a': 	TMon_ID_Test();
						TMon_Register_Test();
						break;

			case 'h':
			default:	TMon_Help();
		}

		if(acL->Err == ERROR)
			eprintf("TMON TEST FAIL!!!\n");
	}
}

//
// Function: TMon_ID_Test
//
// Description: Checks ADUC812_ID and TEMPERATURE_ID for expected values
//
static void TMon_ID_Test(void)
{
	int status = OK;
	unsigned char id[4] = "";

	i2c_read(ADUC812_SLAVE_ADDR, ADUC812_ID_OFF, id);

	if(id[0] != ADUC812_ID_EXPECTED)
	{
		eprintf("ADUC812_ID_EXPECTED does not match, expected = %x, read = %x\n",
				ADUC812_ID_EXPECTED, id);
		status = ERROR;
	}

	i2c_read(ADUC812_SLAVE_ADDR, TEMPERATURE_ID_OFF, id);

	if(id[0] != TEMPERATURE_ID_EXPECTED)
	{
		eprintf("TEMPERATURE_ID_EXPECTED does not match, expected = %x, read = %x\n",
				TEMPERATURE_ID_EXPECTED, id);
		status = ERROR;
	}
	
	eprintf("TMon ID Test... ");

	if(status == ERROR)
	{
		eprintf("FAIL!\n");
		Set_LED("TEiF");
		slowport_code[0] = SP_TEMP;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Tmon Read Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
	}
	else
	{
		eprintf("PASS\n");
		Set_LED("TEiP");
		slowport_code[0] = SP_TEMP;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}		

}

//
// Function: TMon_Write_Read_Verify_Test
//
// Description: Verifies the capability of the Egress and Ingress
//				temperature limits registers to be accessed by
//				writing, reading and verifying. The original values
//				are read and stored and finally restored after the test
//
static void TMon_Register_Test(void)
{
	unsigned char E_temp_low[4] = "", E_temp_high[4] = "";
	unsigned char I_temp_low[4] = "", I_temp_high[4] = "";
	unsigned char test_low[4] = "", test_high[4] = "";
	int status = OK;

	test_high[0] = HIGH_TEMP_LIMIT;
	test_low[0] = LOW_TEMP_LIMIT;

	// Store values of temp. for high and low limits for both Egress and Ingress
	i2c_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, E_temp_low);
	i2c_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, E_temp_high);
	i2c_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, I_temp_low);
	i2c_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, I_temp_high);

	// Write the low and high limits for both Egress & Ingress
	i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, test_high[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, test_low[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, test_high[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, test_low[0]);
	hal_delay_us(1000);
	
	// Read and verify the limits for Egress
	i2c_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, test_high);
	i2c_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, test_low);

	if(test_high[0] != HIGH_TEMP_LIMIT)
	{
		eprintf("Egress High Limit FAIL! exp = %d, read = %d\n",
				HIGH_TEMP_LIMIT, test_high[0]);
		status = ERROR;
	}
	if(test_low[0] != LOW_TEMP_LIMIT)
	{
		eprintf("Egress Low Limit FAIL! exp = %d, read = %d\n",
				LOW_TEMP_LIMIT, test_low[0]);
		status = ERROR;
	}


	// Read and verify the limits for Ingress
	i2c_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, test_high);
	i2c_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, test_low);

	if(test_high[0] != HIGH_TEMP_LIMIT)
	{
		eprintf("Ingress High Limit FAIL! exp = %d, read = %d\n",
				HIGH_TEMP_LIMIT, test_high[0]);
		status = ERROR;
	}
	if(test_low[0] != LOW_TEMP_LIMIT)
	{
		eprintf("Ingress Low Limit FAIL! exp = %d, read = %d\n",
				LOW_TEMP_LIMIT, test_low[0]);
		status = ERROR;
	}

	// Restore values of temp. for high and low limits for both Egress and Ingress
	i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, E_temp_high[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, E_temp_low[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, I_temp_high[0]);
	hal_delay_us(1000);
	i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, I_temp_low[0]);

	// Log results
	eprintf("Tmon WRV Test... ");
	if(status == ERROR)
	{
		eprintf("FAIL!\n");
		Set_LED("TEvF");
		slowport_code[0] = SP_TEMP;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Tmon WRV Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
	}
	else
	{
		eprintf("PASS\n");
		Set_LED("TEvP");
		slowport_code[0] = SP_TEMP;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}		
}

//
// Function: TMon_Read
//
// Description: This function is invoked when the user enters the
//				"tempread" command from the Diag prompt. This
//				function displays the monitored, the high limit,
//				and the low limit for both the Egress and Ingress
//				temperatures.
//
void TMon_Read(void)
{
	unsigned char temp[4] = "";
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int forever = 0;

	if(acL->argc > 1)
		forever = 1;

	if(acL -> HostType != MASTER)
	{
		eprintf("This function is only available on the Master!\n");
	}
	else
	{
		do
		{
			// read and displays temperature values for Egress
			i2c_seq_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_MON_OFF, temp, 1);
			eprintf("Egress temp : mon = %d C, ", temp[0]);

			i2c_seq_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, temp, 1);
			eprintf("high = %d C, ", temp[0]);

			i2c_seq_read(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, temp, 1);
			eprintf("low = %d C\n", temp[0]);


			// read and displays temperature values for Ingress
			i2c_seq_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_MON_OFF, temp, 1);
			eprintf("Ingress temp: mon = %d C, ", temp[0]);

			i2c_seq_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, temp, 1);
			eprintf("high = %d C, ", temp[0]);

			i2c_seq_read(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, temp, 1);
			eprintf("low = %d C\n", temp[0]);

			if(KEYPRESS)
				return;
		} while (forever);
	}
}

//
// Function: TMon_Set_Limits
//
// Description: This function is invoked when the user enters the 
//				"tempset" command from the Diag prompt. This function
//				allows the user to set the low and high temperature limits
//				for both the Egress and Ingress.
//
void TMon_Set_Limits(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned char I_temp_low = 0, I_temp_high = 0;
	unsigned char E_temp_low = 0, E_temp_high = 0;

	if(acL -> HostType != MASTER)
	{
		eprintf("This function is only available on the Master!\n");
	}
	else
	{
		I_temp_low = acL->decArg[1];
		i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_LOW_OFF, I_temp_low);
		hal_delay_us(1000);

		E_temp_low = acL->decArg[3];
		i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_LOW_OFF, E_temp_low);
		hal_delay_us(1000);

		if(acL->decArg[2] > 255)
		{
			eprintf("***WARNING***: Ingress max temperature high limit is 255. 255 is set\n");
			I_temp_high = 255;
		}
		else
		{
			I_temp_high = acL->decArg[2];
		}
		i2c_write(ADUC812_SLAVE_ADDR, INGRESS_TEMP_HIGH_OFF, I_temp_high);
		hal_delay_us(1000);

		if(acL->decArg[4] > 255)
		{
			eprintf("***WARNING***: Egress max temperature high limit is 255. 255 is set\n");
			E_temp_high = 255;
		}
		else
		{
			E_temp_high = acL->decArg[4];
		}
		i2c_write(ADUC812_SLAVE_ADDR, EGRESS_TEMP_HIGH_OFF, E_temp_high);
	}
}

//
// Function: Usr_Temp_Ctrl_ISR
//
// Description: Displays the interrupts from the ADUC812
//
void Usr_Temp_Ctrl_ISR(void)
{
	unsigned char interrupts[4] = "";

	i2c_seq_read(ADUC812_SLAVE_ADDR, ADUC812_INTERRUPT_OFF, interrupts, 4);

	// Check for interrupts from Byte one
	if( (interrupts[2] & 0x80) != 0)
		eprintf("3.3V Main High Interrupted\n");
	if( (interrupts[2] & 0x40) != 0)
		eprintf("3.3V Main Low Interrupted\n");

	if( (interrupts[2] & 0x20) != 0)
		eprintf("3.3V High Interrupted\n");
	if( (interrupts[2] & 0x10) != 0)
		eprintf("3.3V Low Interrupted\n");

	if( (interrupts[2] & 0x8) != 0)
		eprintf("2.5V Main High Interrupted\n");
	if( (interrupts[2] & 0x4) != 0)
		eprintf("2.5V Main Low Interrupted\n");

	if( (interrupts[2] & 0x2) != 0)
		eprintf("2.5V High Interrupted\n");
	if( (interrupts[2] & 0x1) != 0)
		eprintf("2.5V Low Interrupted\n");



	// Check for interrupts from Byte two
	if( (interrupts[1] & 0x80) != 0)
		eprintf("1.8V High Interrupted\n");
	if( (interrupts[1] & 0x40) != 0)
		eprintf("1.8V Low Interrupted\n");

	if( (interrupts[1] & 0x20) != 0)
		eprintf("1.5V High Interrupted\n");
	if( (interrupts[1] & 0x10) != 0)
		eprintf("1.5V Low Interrupted\n");

	if( (interrupts[1] & 0x8) != 0)
		eprintf("1.3V High Interrupted\n");
	if( (interrupts[1] & 0x4) != 0)
		eprintf("1.3V Low Interrupted\n");

	if( (interrupts[1] & 0x2) != 0)
		eprintf("1.25V High Interrupted\n");
	if( (interrupts[1] & 0x1) != 0)
		eprintf("1.25V Low Interrupted\n");



	// Check for interrupts from Byte three
	if( (interrupts[0] & 0x80) != 0)
		eprintf("E-NPU Temperature High Interrupted\n");
	if( (interrupts[0] & 0x40) != 0)
		eprintf("E-NPU Temperature Low Interrupted\n");

	if( (interrupts[0] & 0x20) != 0)
		eprintf("I-NPU Temperature High Interrupted\n");
	if( (interrupts[0] & 0x10) != 0)
		eprintf("I-NPU Temperature Low Interrupted\n");

	// User will have to  enable it back from Diag prompt
	pciIntDisconnect(CPLD_ADUC_INT);

	eprintf("*NOTE*:ADUC interrupt disabled.\n");
	eprintf("To enable it call \"aducset\" from Diag prompt\n");
}


