/* timer.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 14, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "timer.h"
#include "led.h"
#include "uart.h"
#include "diag_utils.h"
#include "syslog.h"
#include "hal_ixdp2400.h"
#include "hal_platform_ints.h"

// static function prototypes
static void Timer_Help(void);
static void Generic_Timer_Test(void);
static void Watchdog_Timer_Test(void);

volatile int diag_timer_int_flag = 0;
extern UINT32 t_all_verbose;

//
// Function: Timer_Help
//
// Description: Displays all the options available for the Timer test
//				on the console.
//
static void Timer_Help(void)
{
	eprintf("> t timers <option>,\n");
	eprintf("where <option> are\n");
	eprintf("g - Generic Timer Test\n");
	eprintf("w - Watchdog Timer Test\n");
	eprintf("a - all tests\n");
	eprintf("h - help\n");	
}

//
// Function: Timer_Test
//
// Description: Calls the appropriate function(s) of the Timer test
//				depending on user input.
//
void Timer_Test(void)
{
	//determine which timer tests is called - generic, watchdog, or all
	// Generic - covers all four timers
	// Watchdog - covers only the fourth timer
	// All - covers all timer tests

	char opt;
	unsigned int loops = 1; //number of times the test(s) is to be repeated
	unsigned int a;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	opt = acL->argv[2][0];
	if(acL->argc > 3)
		loops = acL->decArg[3];
	
	switch(opt)
	{
		case 'g': 	for(a = 0; a < loops; a++)
						Generic_Timer_Test();
					break;

		case 'w': 	Watchdog_Timer_Test();
					break;

		case 'a': 	Generic_Timer_Test();
					Watchdog_Timer_Test();
					break;

		case 'h':
		default:	Timer_Help();
	}

	if(acL->Err == ERROR)
		eprintf("TIMER TEST FAIL!!!\n");
}

//
// Function: Generic_Timer_Test
//
// Description: Test 3 of the timers (does not test the watchdog timer)
//				Tests each of the timers separately by enabling each timer
//				and setting its timer counter. The interrupt handler for the
//				timer will set the diag_timer_int_flag, which will be checked
//				after a certain delay. If it is set, then the test for that
//				particular timer will PASS, else it FAILS.
//
static void Generic_Timer_Test(void)
{
	unsigned int Delay;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

#ifdef TEST_TIMER1	// requires timer1 to be re-initialized because it is used
	// Timer 1
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER1_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER1_CLR, CLEAR_INTERRUPT);		// Clear Timer1 Interrupt
	PUT32(TIMERS_TIMER1_CLD, 0x00000FFF);			// Load Timer1 Counter
	PUT32(TIMERS_TIMER1_CTL, ACTIVATE_TIMER);		// Activate Timer1
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER1_BIT); 		// Set Timer1 IRQ ENABLE
	

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	hal_delay_us(100);
	if(t_all_verbose != MINIMAL)
		eprintf("Timer 1... ");
	if(diag_timer_int_flag != 0)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS!\n");
		Set_LED("Tg1P");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("Tg1F");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Timer1 Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}
#endif

#ifndef EMI
	// Timer 2
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER2_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLR, CLEAR_INTERRUPT);		// Clear Timer2 Interrupt
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER2_BIT); 		// Set Timer2 IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLD, 0x0000020);			// Load Timer2 Counter
	PUT32(TIMERS_TIMER2_CTL, ACTIVATE_TIMER);		// Activate Timer2

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	hal_delay_us(100);
	if(t_all_verbose != MINIMAL)
		eprintf("Timer 2... ");
	if(diag_timer_int_flag != 0)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS!\n");
		Set_LED("Tg2P");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("Tg2F");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Timer2 Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}
#endif

	// Timer 3
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER3_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER3_CLR, CLEAR_INTERRUPT);		// Clear Timer3 Interrupt
	PUT32(TIMERS_TIMER3_CLD, 0x00000FFF);			// Load Timer3 Counter
	PUT32(TIMERS_TIMER3_CTL, ACTIVATE_TIMER);		// Activate Timer3
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER3_BIT); 		// Set Timer3 IRQ ENABLE
	
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	hal_delay_us(100);
	if(t_all_verbose != MINIMAL)
		eprintf("Timer 3... ");
	if(diag_timer_int_flag != 0)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS!\n");
		Set_LED("Tg3P");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("Tg3F");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Timer3 Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}

	// Timer 4
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER4_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER4_CLR, CLEAR_INTERRUPT);		// Clear Timer3 Interrupt
	PUT32(TIMERS_TIMER4_CLD, 0x00000FFF);			// Load Timer3 Counter
	PUT32(TIMERS_TIMER4_CTL, ACTIVATE_TIMER);		// Activate Timer3
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER4_BIT); 		// Set Timer3 IRQ ENABLE
	
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	hal_delay_us(100);
	if(t_all_verbose != MINIMAL)
		eprintf("Timer 4... ");
	if(diag_timer_int_flag != 0)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS!\n");
		Set_LED("Tg4P");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("Tg4F");
		slowport_code[0] = SP_TIME;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "Timer4 Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));

		acL->Err = ERROR;
	}
}

//
// Function: Watchdog_Timer_Test
//
// Description: Enables the Watchdog timer and sets its timer counter.
//
static void Watchdog_Timer_Test(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	if(acL->HostType == SLAVE)
	{
		eprintf("*WARNING*: It is recommended to reset MASTER as well.\n");
		eprintf("Press any key to continue watchdog timer test.\n");
		while(!KEYPRESS);
	}

	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER4_BIT);	// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER4_CLR, CLEAR_INTERRUPT);	// Clear Timer Interrupt
	PUT32(TIMERS_TIMER4_CLD, 0x00000FFF);		// Load Timer4 Counter
	PUT32(TIMERS_TWDE, 0x1);					// Enable Watchdog Timer
	PUT32(TIMERS_TIMER4_CTL, ACTIVATE_TIMER);	// Activate Timer4
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER4_BIT); 	// Set Timer4 IRQ ENABLE
	PUT32(IXP_RESET0, WDOG_RST_ENABLE);

	Set_LED("Wdog");
	eprintf("Waiting for Watchdog reset! Press Any Key to stop wait.\n");

	while(1)
	{
		if(KEYPRESS)
		{
			eprintf("Exiting Watchdog Reset wait.\n");
			break;
		}
	}
}

void Timer_Interrupt_Handler(void)
{
	int temp;

	diag_timer_int_flag = 1;

#ifdef TEST_TIMER1	// requires timer1 to be re-initialized because it is used by i2c
	PUT32(TIMERS_TIMER1_CTL, 0);	// Deactivate Timer1
	PUT32(TIMERS_TIMER1_CLR, 0x1);
#endif
	Set_LED("TIME");

#ifndef EMI
	PUT32(TIMERS_TIMER2_CTL, 0);	// Deactivate Timer2
	PUT32(TIMERS_TIMER2_CLR, 0x1);
#endif

	PUT32(TIMERS_TIMER3_CTL, 0);	// Deactivate Timer3
	PUT32(TIMERS_TIMER4_CTL, 0);	// Deactivate Timer4
	
	PUT32(TIMERS_TIMER3_CLR, 0x1);
	PUT32(TIMERS_TIMER4_CLR, 0x1);
	temp = GET32(TIMERS_TIMER4_CLR);
}
