/* voltage.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Apr 18, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "voltage.h"
#include "led.h"
#include "i2c.h"
#include "misc_func.h"

//
// Function: VMon_Read
//
// Description: This function gets invoked when the user enters the
//				command voltread from the Diag prompt. It displays
//				the monitored, and high, and low voltages of 3.3V,
//				2.5V, 1.8V, 1.5V, 1.3V, and 1.25V
//
void VMon_Read(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	unsigned char volt[4] = "";
	int actual_volt;

	if(acL -> HostType != MASTER)
	{
		eprintf("This function is only available on the Master!\n");
	}
	else
	{
		// read and display 3.3V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_3_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("3.3V : mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_3_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_3_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);


		// read and display 2.5V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_2_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("2.5V : mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_2_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_2_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);


		// read and display 1.8V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_1_8V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("1.8V : mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_1_8V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_1_8V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);


		// read and display 1.5V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_1_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("1.5V : mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_1_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_1_5V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);


		// read and display 1.3V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_1_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("1.3V : mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_1_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_1_3V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);


		// read and display 1.25V values (mon, high, low)
		i2c_seq_read(ADUC812_SLAVE_ADDR, MON_1_25V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("1.25V: mon = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, HIGH_1_25V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("high = %d uV, ", actual_volt);

		i2c_seq_read(ADUC812_SLAVE_ADDR, LOW_1_25V_OFF, volt, 2);
		actual_volt = ((int)volt[0]<<8) + (int)volt[1];
		actual_volt *= ADUC_2_5V_REF_RES;
		eprintf("low = %d uV\n", actual_volt);
	}
}

//
// Function: VMon_Set_Limits
//
// Description: This function is invoked when the user enters the 
//				"vset" command from the Diag prompt. This function
//				allows the user to set the low and high temperature limits
//				for both the Egress and Ingress in micro-volts (uV)
//
void VMon_Set_Limits(void)
{
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;
	int actual_volt_low = 0, actual_volt_high = 0;
	unsigned char volt_low[4] = "", volt_high[4] = "";

	if(acL->HostType != MASTER)
	{
		eprintf("This function is only available on the Master!\n");
	}
	else
	{
		if(acL->argc < 4)	// there should be 4 arguments;
		{
			eprintf("Insufficient arguments\n");
			return;
		}
		else
		{
			actual_volt_low = acL->decArg[2];
			actual_volt_high = acL->decArg[3];
			// Check if user did not specify voltage in uV
			if((actual_volt_low < 1000000) || (actual_volt_high < 1000000))// if less than 1000000 uV
			{
				eprintf("Voltage is in uV!\n");
				return;
			}

			if(acL->argv[1][0] == '3')	// 3.3V
			{
				actual_volt_low /=ADUC_5V_REF_RES;
				actual_volt_high /=ADUC_5V_REF_RES;
				volt_low[0] = (unsigned char)((actual_volt_low >> 8) & 0xFF);
				volt_low[1] = (unsigned char)actual_volt_low & 0xFF;
				volt_high[0] = (unsigned char)((actual_volt_high >> 8) & 0xFF);
				volt_high[1] = (unsigned char)actual_volt_high & 0xFF;
				i2c_page_write(ADUC812_SLAVE_ADDR, LOW_3_3V_OFF, volt_low, 2);
				hal_delay_us(1000);
				i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_3_3V_OFF, volt_high, 2);
				hal_delay_us(1000);
			}
			else if(acL->argv[1][0] == '2')	// 2.5V
			{
				actual_volt_low /=ADUC_5V_REF_RES;
				actual_volt_high /=ADUC_5V_REF_RES;
				volt_low[0] = (unsigned char)((actual_volt_low >> 8) & 0xFF);
				volt_low[1] = (unsigned char)actual_volt_low & 0xFF;
				volt_high[0] = (unsigned char)((actual_volt_high >> 8) & 0xFF);
				volt_high[1] = (unsigned char)actual_volt_high & 0xFF;
				i2c_page_write(ADUC812_SLAVE_ADDR, LOW_2_5V_OFF, volt_low, 2);
				hal_delay_us(1000);
				i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_2_5V_OFF, volt_high, 2);
				hal_delay_us(1000);
			}
			else if(acL->argv[1][0] == '1')	// 1.8V, 1.5V, 1.3V, and 1.25V
			{
				actual_volt_low /=ADUC_2_5V_REF_RES;
				actual_volt_high /=ADUC_2_5V_REF_RES;
				volt_low[0] = (unsigned char)((actual_volt_low >> 8) & 0xFF);
				volt_low[1] = (unsigned char)actual_volt_low & 0xFF;
				volt_high[0] = (unsigned char)((actual_volt_high >> 8) & 0xFF);
				volt_high[1] = (unsigned char)actual_volt_high & 0xFF;
				
				if(acL->argv[1][2] == '8')	// 1.8V
				{
					i2c_page_write(ADUC812_SLAVE_ADDR, LOW_1_8V_OFF, volt_low, 2);
					hal_delay_us(1000);
					i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_1_8V_OFF, volt_high, 2);
					hal_delay_us(1000);
				}
				else if(acL->argv[1][2] == '5')	// 1.5V
				{
					i2c_page_write(ADUC812_SLAVE_ADDR, LOW_1_5V_OFF, volt_low, 2);
					hal_delay_us(1000);
					i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_1_5V_OFF, volt_high, 2);
					hal_delay_us(1000);
				}
				else if(acL->argv[1][2] == '3')	// 1.3V
				{
					i2c_page_write(ADUC812_SLAVE_ADDR, LOW_1_3V_OFF, volt_low, 2);
					hal_delay_us(1000);
					i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_1_3V_OFF, volt_high, 2);
					hal_delay_us(1000);
				}
				else if(acL->argv[1][2] == '2')	// 1.25V
				{
					i2c_page_write(ADUC812_SLAVE_ADDR, LOW_1_25V_OFF, volt_low, 2);
					hal_delay_us(1000);
					i2c_page_write(ADUC812_SLAVE_ADDR, HIGH_1_25V_OFF, volt_high, 2);
					hal_delay_us(1000);
				}
				else
				{
					eprintf("****ERROR****: Undefined voltage type\n");
				}
			}
			else
			{
				eprintf("****ERROR****: Undefined voltage type\n");
			}				
		}
	}
}
