/* xscale.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2400
 *  subsystem: DIAG
 *  author: ccm, Jan 17, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "diagstruct.h"
#include "xscale.h"
#include "scratch.h"
#include "led.h"
#include "diag_utils.h"
#include "hal_ixdp2400.h"
#include "syslog.h"
#include "hal_platform_ints.h"

static void Xscale_help(void);
static void Xscale_Interrupt_Test(void);

static volatile int diag_soft_int_flag = 0;
extern UINT32 t_all_verbose;

//
// Function: Xscale_help
//
// Description: Displays all the options available for the XScale test
//				on the console.
//
static void Xscale_help(void)
{
	eprintf("> t xscale <option>, where\n");
	eprintf("<option> are:\n");
	eprintf("s - Xscale scratchpad test\n");
	eprintf("i - Xscale interrupt test\n");	
	eprintf("a - all tests\n");
	eprintf("h - help\n");
}

//
// Function: Xscale_Test
//
// Description: Calls the appropriate function(s) of the XScale test
//				depending on user input.
//
void Xscale_Test(void)
{	
	// Determine which Xscale tests is called - scratchpad, interrupt, or all	

	char opt;
	unsigned int loops = 1;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	acL->Err = OK;
	opt = acL->argv[2][0];
	if(acL->argc > 3)
		loops = 1;//acL->decArg[3];
	
	switch(opt)
	{
		case 's': 	while(loops--)
						Xscale_Scratchpad_Test();
					break;

		case 'i': 	while(loops--)
						Xscale_Interrupt_Test();
					break;

		case 'a': 	while(loops--)
					{
						Scratch_Pattern_Test();
						ScratchRing_Test();
						Xscale_Interrupt_Test();
					}
					break;

		case 'h':
		default:	Xscale_help();
	}

	if(acL->Err == ERROR)
		eprintf("XSCALE TEST FAIL!!!\n");
}

//
// Function: Xscale_Interrupt_Test
//
// Description: Test for both IRQ and FIQ soft interrupts
//
static void Xscale_Interrupt_Test(void)
{
	unsigned int delay;
	register PDiagCommon acL = (PDiagCommon) ACADDRESS;

	// Testing for IRQ Soft Interrupt

	delay = 0xFF;
	diag_soft_int_flag = 0;					// clear soft interrupt flag
	PUT32(FIQ_ENABLE_CLR_REG, SOFT_INT);	// disable FIQ soft interrupt
	PUT32(IRQ_ENABLE_SET_REG, SOFT_INT);	// enable IRQ soft interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, SOFT_INT);	// trigger soft interrupt

	while(delay--);
	if(t_all_verbose != MINIMAL)
		eprintf("XScale IRQ Test... ");
	delay = 0xFF;
	while((diag_soft_int_flag == 0) && --delay);
	if(diag_soft_int_flag == IRQ_SOFT)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS.\n");
		Set_LED("XSiP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("XSiF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
		
		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "XScale IRQ Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
		acL->Err = ERROR;
	}

	// Testing for FIQ Soft Interrupt

	delay = 0xFF;
	diag_soft_int_flag = 0;					// clear soft interrupt flag
	PUT32(IRQ_ENABLE_CLR_REG, SOFT_INT);	// disable IRQ soft interrupt
	PUT32(FIQ_ENABLE_SET_REG, SOFT_INT);	// enable FIQ soft interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, SOFT_INT);	// trigger soft interrupt

	while(delay--);
	if(t_all_verbose != MINIMAL)
		eprintf("XScale FIQ Test... ");
	delay = 0xFF;
	while((diag_soft_int_flag == 0) && --delay);
	if(diag_soft_int_flag == FIQ_SOFT)
	{
		if(t_all_verbose != MINIMAL)
			eprintf("PASS.\n");
		Set_LED("XSfP");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_PASS;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);
	}
	else
	{
		eprintf("FAIL!\n");
		Set_LED("XSfF");
		slowport_code[0] = SP_XSC;
		slowport_code[1] = SP_FAIL;
		dump_slowport(slowport_code, 2, SP_NON_FATAL);

		// prepare structure for dumping
		syslog.type = TYPE_ERROR;
		syslog.source_comp = MASTER_DIAG;	// Determine master/slave
		strcpy(syslog.desc, "XScale FIQ Test FAIL!");

		// write to syslog
		syslog_dump(&syslog, sizeof(SYSLOG_DATA));
		acL->Err = ERROR;
	}
}

//
// Function: XScale_Soft_Interrupt_Handler
//
// Description: Interrupt handler for soft interrupt
//
void XScale_Soft_Interrupt_Handler(void)
{
	Set_LED("XSCL");
	
	if( (GET32(IRQ_STATUS_REG) & SOFT_INT) != 0)
	{
		diag_soft_int_flag |= IRQ_SOFT;
		PUT32(IRQ_ENABLE_CLR_REG, SOFT_INT);	// disable IRQ soft interrupt
	}

	if( (GET32(FIQ_STATUS_REG) & SOFT_INT) != 0)
	{
		diag_soft_int_flag |= FIQ_SOFT;
		PUT32(FIQ_ENABLE_CLR_REG, SOFT_INT);	// disable FIQ soft interrupt
	}	

	// Clear interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, 0x0);	// clear soft interrupt
}
