/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_internal.h
 *
 * = DESCRIPTION
 *      This is the header file for the defining the data structures
 *      used in driver code
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *       6/20/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 * $Id: ixf1104ce_internal.h,v 1.2 2003/05/15 22:38:22 nmafzal Exp $
 */

#if !defined(__IXF1104CE_INTERNAL_H__) 
#define __IXF1104CE_INTERNAL_H__
#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Pre-processor symbol and macro definitions.
 */
#if defined(__DEBUG__)

#if defined __LINUX__
#if !defined(__LINUX_USER__)
#define MESSAGE(A,B,C,D,E,F,G) printk(A,B,C,D,E,F,G)     
#endif
#else /* if defined __LINUX_USER___ */
#define MESSAGE(A,B,C,D,E,F,G) printf(A,B,C,D,E,F,G)   
#endif
#else
#define MESSAGE(A,B,C,D,E,F,G)
#endif

typedef unsigned int uint32;

#define REG_READ(reg,result)           (*(result) = *(volatile uint32*)(reg))
#define REG_WRITE(reg,data)    (*((volatile uint32*)(reg)) = (data)) 

/**
 * Type definitions.
 */

#define PTLONE_ON_MEDIA_IF         0
#define PTLONE_ON_SF_IF            1


/**
 * Pt Lone Media Card Port related 
 */
#define PT_LONE_GBE_PORT_MAX    0x04                   /* No of GbE MAC Ports */
#define PT_LONE_GBE_PORT_0      0x00
#define PT_LONE_GBE_PORT_1      0x01
#define PT_LONE_GBE_PORT_2      0x02
#define PT_LONE_GBE_PORT_3      0x03


/**
 * Pt Lone Media card Device Related information 
 */
#define PT_LONE_DEVICE_ID       0x6                    /* Pt Lone Media Card device ID */
#define MEDIA_IF_EEPROM_ADDR    0xa4                   /* media i/f cfg PROM address */
#define SF_IF_EEPROM_ADDR       0xa6                   /* sw i/f cfg prom address */
#define EEPROM_DEV_ID_OFF       0x01
#define EEPROM_MAC_PORT_OFF_0   0x50   
#define EEPROM_MAC_PORT_OFF_1   0x56   
#define EEPROM_MAC_PORT_OFF_2   0x60   
#define EEPROM_MAC_PORT_OFF_3   0x66   


#define INITIALIZED             0x01
#define UNINITIALIZED           0x00

/** 
 * Time definitition used for introducing the delays 
 */
#define WAIT_FOR_SEMAPHORE      0x00000001    /* Specify the amount of time in ticks to wait on the semaphore.*/
#define WAIT_PLL_LOCK           0x00000001    /* The minimum delay is of 1/60 Sec, using the sys clock rate in ticks,
                                                specy according to the system clock rate to introduce 10mSec delay.*/
#define WAIT_CLK_STABLE         0x00000001    /* The minimum delay is of 1/60 Sec, using the sys clock rate in ticks,
                                                specy according to the system clock rate to introduce 1000uSec delay.*/

#define MDI_COMMAND_DELAY       0x00000001    /* Exact value is not known */


#define PT_LONE_SERDES_BASE     0x780

#define MAC_CTL_REG_PORT0       0x000
#define MAC_CTL_REG_PORT1       0x080
#define MAC_CTL_REG_PORT2       0x100
#define MAC_CTL_REG_PORT3       0x180


#define RX_STAT_REG_PORT0       0x20
#define RX_STAT_REG_PORT1       0xA0
#define RX_STAT_REG_PORT2       0x120
#define RX_STAT_REG_PORT3       0x1A0

#define TX_STAT_REG_PORT0       0x40
#define TX_STAT_REG_PORT1       0xC0
#define TX_STAT_REG_PORT2       0x140
#define TX_STAT_REG_PORT3       0x1C0

#define STAT_CONFIG_REG         0x500
#define RX_FIFO_BASE            0x580
#define TX_FIFO_BASE            0x600
#define MDIO_BASE               0x680
#define SPI3_BASE               0x700
#define SERDES_BASE             PT_LONE_SERDES_BASE
#define GBIC_BASE               0x19


/**
 * #define the registers of the Vallejo 
 */

/**
 *  per port control and status register offsets
 */
#define STATION_ADDR_LOW                0x00
#define STATION_ADDR_HIGH               0x01
#define DESIRED_DUPLEX                  0x02
#define FDFC_TYPE                       0x03
#define VLAN_TYPE                       0x04
#define COLL_DIST                       0x05
#define COLL_TSHD                       0x06
#define FCTX_TIMER                      0x07
#define FDFC_ADDR_LOW                   0x08
#define FDFC_ADDR_HIGH                  0x09
#define IPG_RECEIVE_TIME1               0x0A
#define IPG_RECEIVE_TIME2               0x0B
#define IPG_TRANSMIT_TIME               0x0C
#define PAUSE_THRESHOLD                 0x0E
#define MAX_FRAME_SIZE                  0x0F
#define MAC_IF_MODE                     0x10
#define FLUSH_TX                        0x11
#define FC_MODE                         0x12
#define FC_BACK_PRESSURE_LEN            0x13
#define SHORT_RUNT_TSHD                 0x14
#define UNKNOWN_FRAME_STT               0x15
#define RX_CONFIG_WORD                  0x16
#define TX_CONFIG_WORD                  0x17
#define DIV_CONFIG_WORD                 0x18
#define PKT_FILTER_CTL                  0x19
#define MUL_PORT_ADDR_LOW               0x1A
#define MUL_PORT_ADDR_HIGH              0x1B


/**
 *  This defines per port MAC receiver statistic 
 *  counters. These registers are read only, and cleared when read.  These
 *  registers are basically counters, which store the statistics regarding
 *  the respective frame received.
 *
 */
#define RX_OCTETS_OK                0x20
#define RX_OCTETS_BAD               0x21
#define RX_UC_PKTS                  0x22
#define RX_MC_PKTS                  0x23
#define RX_BC_PKTS                  0x24
#define RX_PKTS_64                  0x25
#define RX_PKTS_65_127              0x26
#define RX_PKTS_128_255             0x27
#define RX_PKTS_256_511             0x28
#define RX_PKTS_512_1023            0x29
#define RX_PKTS_1024_1518           0x2A
#define RX_PKTS_1519_MAX            0x2B
#define RX_FCS_ERR                  0x2C
#define VLAN_TAG                    0x2D
#define RX_DATA_ERR                 0x2E
#define RX_ALIGN_ERR                0x2F
#define RX_LONG_ERR                 0x30
#define RX_JABBER_ERR               0x31
#define RX_PAUSE_MAC_CTL            0x32
#define RX_UNKNOWN_CTL_FRAME        0x33
#define VLONG_ERR                   0x34
#define RUNT_ERR                    0x35
#define SHORT_ERR                   0x36
#define CARRIER_EXTEND_ERROR        0x37
#define SEQ_ERR                     0x38
#define SYMBOL_ERR                  0x39


/**
 *
 *   This defines the per port register details on the MAC transmit statistic
 *   counters. These registers are read only, and cleared when read.  These
 *   registers are basically counters, which store the statistics regarding
 *   the respective frame received.
 *
 */
#define TX_OCTETS_OK                0x40
#define TX_OCTETS_BAD               0x41
#define TX_UC_PKTS                  0x42
#define TX_MC_PKTS                  0x43
#define TX_BC_PKTS                  0x44
#define TX_PKTS_64                  0x45
#define TX_PKTS_65_127              0x46
#define TX_PKTS_128_255             0x47
#define TX_PKTS_256_511             0x48
#define TX_PKTS_512_1023            0x49
#define TX_PKTS_1024_1518           0x4A
#define TX_PKTS_1519_MAX            0x4B
#define TX_DEFERRED_ERR             0x4C
#define TX_TOTAL_COLLISION          0x4D
#define TX_SINGLE_COLLISION         0x4E
#define TX_MUL_COLLISION            0x4F
#define LATE_COLLISION              0x50
#define TX_EXCV_COLLISION           0x51
#define TX_EXCV_DEFERRED_ERR        0x52
#define TX_EXCV_LEN_DROP            0x53
#define TX_UNDERRUN                 0x54
#define TX_VLAN_TAG                 0x55
#define TX_CRC_ERR                  0x56
#define TX_PAUSE_FRAME              0x57
#define FC_COLLISION_SEND           0x58

/**
 * PHY autoscan register offsets 
 */
#define AS_CTL_REG                  0x60
#define AS_STAT                     0x61
#define AS_PHY_ID_REG1              0x62
#define AS_PHY_ID_REG2              0x63
#define AN_ADVT_REG                 0x64
#define AN_LP_BSPG_ABL              0x65
#define AN_EXP_REG                  0x66
#define AN_NXTPG_TX                 0x67



/**
 *  This defines the global status and configuration register offsets
 */
#define PORT_ENABLE                 STAT_CONFIG_REG + 0x00
#define INTERFACE_MODE              STAT_CONFIG_REG + 0x01
#define LINK_UP_STATUS              STAT_CONFIG_REG + 0x02
#define PHY_REF_CLK_SEL             STAT_CONFIG_REG + 0x03
#define RESET_CORE_CLOCK            STAT_CONFIG_REG + 0x04
#define MAC_SOFT_RESET              STAT_CONFIG_REG + 0x05
#define MDIO_RESET                  STAT_CONFIG_REG + 0x06
#define PAUSE_BEHAVIOR              STAT_CONFIG_REG + 0x07
#define UI_ENDIAN_MODE              STAT_CONFIG_REG + 0x08
#define LED_MODE                    STAT_CONFIG_REG + 0x09
#define LED_FLASH_RATE              STAT_CONFIG_REG + 0x0A
#define LED_FAULT_ACTION            STAT_CONFIG_REG + 0x0B
#define JTAG_ID                     STAT_CONFIG_REG + 0x0C


/**
 *  This defines the registers which  provides the information regarding the 
 *  receive mode configuration of the Vallejo MAC. 
 *  Rx FIFO Register offset,  the receive FIFO characteristics.
 */
#define PORT0_RX_HIGH_WATERMARK     RX_FIFO_BASE + 0x00
#define PORT1_RX_HIGH_WATERMARK     RX_FIFO_BASE + 0x01
#define PORT2_RX_HIGH_WATERMARK     RX_FIFO_BASE + 0x02
#define PORT3_RX_HIGH_WATERMARK     RX_FIFO_BASE + 0x03

#define PORT0_RX_LOW_WATERMARK      RX_FIFO_BASE + 0x0A
#define PORT1_RX_LOW_WATERMARK      RX_FIFO_BASE + 0x0B
#define PORT2_RX_LOW_WATERMARK      RX_FIFO_BASE + 0x0C
#define PORT3_RX_LOW_WATERMARK      RX_FIFO_BASE + 0x0D

#define PORT0_RX_FRAME_REMOVED      RX_FIFO_BASE + 0x14
#define PORT1_RX_FRAME_REMOVED      RX_FIFO_BASE + 0x15
#define PORT2_RX_FRAME_REMOVED      RX_FIFO_BASE + 0x16
#define PORT3_RX_FRAME_REMOVED      RX_FIFO_BASE + 0x17

#define RXFIFO_PORT_RESET           RX_FIFO_BASE + 0x1E
#define RXFIFO_DROP_ENABLE          RX_FIFO_BASE + 0x1F
#define RXFIFO_OVRFLW_EVENT         RX_FIFO_BASE + 0x20
#define RXFIFO_OUT_OF_SEQ           RX_FIFO_BASE + 0x21

#define PORT0_RXFIFO_PKT_DROP       RX_FIFO_BASE + 0x22
#define PORT1_RXFIFO_PKT_DROP       RX_FIFO_BASE + 0x23
#define PORT2_RXFIFO_PKT_DROP       RX_FIFO_BASE + 0x24
#define PORT3_RXFIFO_PKT_DROP       RX_FIFO_BASE + 0x25

#define PORT0_RXFIFO_RW_PTR         RX_FIFO_BASE + 0x26
#define PORT1_RXFIFO_RW_PTR         RX_FIFO_BASE + 0x27
#define PORT2_RXFIFO_RW_PTR         RX_FIFO_BASE + 0x28
#define PORT3_RXFIFO_RW_PTR         RX_FIFO_BASE + 0x29

#define RXPORT0_OCCUPNCY_CTR        RX_FIFO_BASE + 0x2A
#define RXPORT1_OCCUPNCY_CTR        RX_FIFO_BASE + 0x2B
#define RXPORT2_OCCUPNCY_CTR        RX_FIFO_BASE + 0x2C
#define RXPORT3_OCCUPNCY_CTR        RX_FIFO_BASE + 0x2D

#define CAPTURED_PKT_LEN            RX_FIFO_BASE + 0x2E
#define INDIRECT_ACC_CTL            RX_FIFO_BASE + 0x2F
#define INDIRECT_MEM_DATA_REG1      RX_FIFO_BASE + 0x30
#define INDIRECT_MEM_DATA_REG2      RX_FIFO_BASE + 0x31
#define RXFIFO_CAPT_LPBK_ENB        RX_FIFO_BASE + 0x32
#define RXFIFO_PAD_CRCSTRIP_ENB     RX_FIFO_BASE + 0x33

#define PORT0_MATCH_PATTERN         RX_FIFO_BASE + 0x34
#define PORT1_MATCH_PATTERN         RX_FIFO_BASE + 0x35
#define PORT2_MATCH_PATTERN         RX_FIFO_BASE + 0x36
#define PORT3_MATCH_PATTERN         RX_FIFO_BASE + 0x37

#define PORT0_JUMBO_PKT_SIZE        RX_FIFO_BASE + 0x38
#define PORT1_JUMBO_PKT_SIZE        RX_FIFO_BASE + 0x39
#define PORT2_JUMBO_PKT_SIZE        RX_FIFO_BASE + 0x3A
#define PORT3_JUMBO_PKT_SIZE        RX_FIFO_BASE + 0x3B

#define PKT_DROP_CAP_FIFO_0         RX_FIFO_BASE + 0x3C
#define PKT_DROP_CAP_FIFO_1         RX_FIFO_BASE + 0x3D
#define PKT_DROP_CAP_FIFO_2         RX_FIFO_BASE + 0x3E
#define PKT_DROP_CAP_FIFO_3         RX_FIFO_BASE + 0x3F



/**
 *  This defines the registers that provides the information regarding the 
 *  transmit mode configuration of the Vallejo MAC. 
 */
#define PORT0_TX_HIGH_WATERMARK     TX_FIFO_BASE + 0x00
#define PORT1_TX_HIGH_WATERMARK     TX_FIFO_BASE + 0x01
#define PORT2_TX_HIGH_WATERMARK     TX_FIFO_BASE + 0x02
#define PORT3_TX_HIGH_WATERMARK     TX_FIFO_BASE + 0x03

#define PORT0_TX_LOW_WATERMARK      TX_FIFO_BASE + 0x0A
#define PORT1_TX_LOW_WATERMARK      TX_FIFO_BASE + 0x0B
#define PORT2_TX_LOW_WATERMARK      TX_FIFO_BASE + 0x0C
#define PORT3_TX_LOW_WATERMARK      TX_FIFO_BASE + 0x0D

#define PORT0_TX_MAC_THRESHOLD      TX_FIFO_BASE + 0x14
#define PORT1_TX_MAC_THRESHOLD      TX_FIFO_BASE + 0x15
#define PORT2_TX_MAC_THRESHOLD      TX_FIFO_BASE + 0x16
#define PORT3_TX_MAC_THRESHOLD      TX_FIFO_BASE + 0x17

#define TXFIFO_OVFLW                TX_FIFO_BASE + 0x1E
#define LOOP_RXDATA_TXFIFO          TX_FIFO_BASE + 0x1F
#define TXFIFO_PORT_RESET           TX_FIFO_BASE + 0x20

#define PORT0_TX_FRAME_REMOVED      TX_FIFO_BASE + 0x21
#define PORT1_TX_FRAME_REMOVED      TX_FIFO_BASE + 0x22
#define PORT2_TX_FRAME_REMOVED      TX_FIFO_BASE + 0x23
#define PORT3_TX_FRAME_REMOVED      TX_FIFO_BASE + 0x24

#define PORT0_TXFIFO_PKT_DROP       TX_FIFO_BASE + 0x25
#define PORT1_TXFIFO_PKT_DROP       TX_FIFO_BASE + 0x26
#define PORT2_TXFIFO_PKT_DROP       TX_FIFO_BASE + 0x27
#define PORT3_TXFIFO_PKT_DROP       TX_FIFO_BASE + 0x28

#define PORT0_TXFIFO_RW_PTR         TX_FIFO_BASE + 0x29
#define PORT1_TXFIFO_RW_PTR         TX_FIFO_BASE + 0x2A
#define PORT2_TXFIFO_RW_PTR         TX_FIFO_BASE + 0x2B
#define PORT3_TXFIFO_RW_PTR         TX_FIFO_BASE + 0x2C

#define TXPORT0_OCCUPNCY_CTR        TX_FIFO_BASE + 0x2D
#define TXPORT1_OCCUPNCY_CTR        TX_FIFO_BASE + 0x2E
#define TXPORT2_OCCUPNCY_CTR        TX_FIFO_BASE + 0x2F
#define TXPORT3_OCCUPNCY_CTR        TX_FIFO_BASE + 0x30

#define PORT0_INS_FIFO_DATA_REG0    TX_FIFO_BASE + 0x31
#define PORT0_INS_FIFO_DATA_REG1    TX_FIFO_BASE + 0x32
#define PORT0_INS_FIFO_CTL_REG      TX_FIFO_BASE + 0x33

#define PORT1_INS_FIFO_DATA_REG0    TX_FIFO_BASE + 0x34
#define PORT1_INS_FIFO_DATA_REG1    TX_FIFO_BASE + 0x35
#define PORT1_INS_FIFO_CTL_REG      TX_FIFO_BASE + 0x36

#define PORT2_INS_FIFO_DATA_REG0    TX_FIFO_BASE + 0x37
#define PORT2_INS_FIFO_DATA_REG1    TX_FIFO_BASE + 0x38
#define PORT2_INS_FIFO_CTL_REG      TX_FIFO_BASE + 0x39

#define PORT3_INS_FIFO_DATA_REG0    TX_FIFO_BASE + 0x3A
#define PORT3_INS_FIFO_DATA_REG1    TX_FIFO_BASE + 0x3B
#define PORT3_INS_FIFO_CTL_REG      TX_FIFO_BASE + 0x3C

#define TXFIFO_PORTDROP_INS_ENB     TX_FIFO_BASE + 0x3D
#define TXFIFO_PAD_ENB              TX_FIFO_BASE + 0x3E



/**
 *   This defines the register with their offset, that provides the configuration registers 
 *   related to the MDIO Block.
 *   MDIO Regster Offsets
 */
#define MDIO_SNGL_CMD               MDIO_BASE + 0x00
#define MDIO_SINGLE_RW_DATA         MDIO_BASE + 0x01
#define AS_PHY_ADDR                 MDIO_BASE + 0x02
#define MDIO_CTL                    MDIO_BASE + 0x03


/**
 *  SPI-3 register offsets
 */
#define SPI3_TX_GLOBAL_CONFIG           SPI3_BASE + 0x00
#define SPI3_RX_CONFIG                  SPI3_BASE + 0x01

#define SPI3_TX_INT_STATUS_PORT0        SPI3_BASE + 0x02 
#define SPI3_TX_INT_STATUS_PORT1        SPI3_BASE + 0x03
#define SPI3_TX_INT_STATUS_PORT2        SPI3_BASE + 0x04
#define SPI3_TX_INT_STATUS_PORT3        SPI3_BASE + 0x05

#define SPI3_ADR_PARITY_ERROR_PORT      SPI3_BASE + 0x0A 

#define SPI3_PKT_DROP_CTR_PORT0         SPI3_BASE + 0x0B
#define SPI3_SYNCERR_DROP_CTR_PORT0     SPI3_BASE + 0x0C
#define SPI3_PKT_SHORT_DROP_CTR_PORT0   SPI3_BASE + 0x0D
 
#define SPI3_PKT_DROP_CTR_PORT1         SPI3_BASE + 0x0E
#define SPI3_SYNCERR_DROP_CTR_PORT1     SPI3_BASE + 0x0F
#define SPI3_PKT_SHORT_DROP_CTR_PORT1   SPI3_BASE + 0x10

#define SPI3_PKT_DROP_CTR_PORT2         SPI3_BASE + 0x11
#define SPI3_SYNCERR_DROP_CTR_PORT2     SPI3_BASE + 0x12
#define SPI3_PKT_SHORT_DROP_CTR_PORT2   SPI3_BASE + 0x13

#define SPI3_PKT_DROP_CTR_PORT3         SPI3_BASE + 0x14
#define SPI3_SYNCERR_DROP_CTR_PORT3     SPI3_BASE + 0x15
#define SPI3_PKT_SHORT_DROP_CTR_PORT3   SPI3_BASE + 0x16


/**
 * serdes register offset
 */
#define SERDES_ACDC_COUPLING            SERDES_BASE + 0x00
#define SERDES_TX_DRV_COEFF             SERDES_BASE + 0x01
#define SERDES_TX_DER_POWER_LEVEL       SERDES_BASE + 0x04
#define SERDES_TX_LINK_VALIDATION       SERDES_BASE + 0x06
#define SERDES_TX_RX_POW_DOWN           SERDES_BASE + 0x07
#define SERDES_RX_DATA_SYNC             SERDES_BASE + 0x08
#define SERDES_RX_LINK_VALIDATION       SERDES_BASE + 0x09
#define SERDES_RX_LINK_STT              SERDES_BASE + 0x0A
#define SERDES_RX_PHASE_ROT             SERDES_BASE + 0x0B
#define SERDES_RX_PHASE_ROT_BUS         SERDES_BASE + 0x0C
#define SERDES_RX_LATCH_OBSRV_01        SERDES_BASE + 0x0E
#define SERDES_RX_LATCH_OBSRV_23        SERDES_BASE + 0x0F
#define SERDES_RX_SIGNAL_LEVEL          SERDES_BASE + 0x13
#define SERDES_CLKMODE_REG              SERDES_BASE + 0x14
#define SERDES_RX_TEST_CTRL_REG         SERDES_BASE + 0x15
#define SERDES_SERDES_TX_CONFIG         SERDES_BASE + 0x16
#define SERDES_SERDES_RX_CONFIG         SERDES_BASE + 0x17
#define SERDES_PLL_LOCK_STATUS          SERDES_BASE + 0x18



/**
 *  GBIC block register offset
 */
#define GBIC_STATUS_REG                 SERDES_BASE + 0x19
#define GBIC_CTRL_REG                   SERDES_BASE + 0x1A
#define I2C_CTRL_REG                    SERDES_BASE + 0x1B
#define PLL1_TUNE_REG                   SERDES_BASE + 0x1C
#define PLL2_TUNE_REG                   SERDES_BASE + 0x1D
#define PLL3_TUNE_REG                   SERDES_BASE + 0x1E

/**
 * TYPENAME: gbe_mac_port_states
 * 
 * DESCRIPTION: Possible states of a Port
 *
 */
typedef enum gbe_mac_e_port_state {
    CLOSED = 0,
    OPENED
} gbe_mac_port_state;


/**
 * TYPENAME: gbe_mac_ioctl_ptr
 *
 * DESCRIPTION: IOCTL structures, the pointer to this structure, typecast with
 *              the void *, is passed in the IOCTL API.
 */
typedef struct gbe_mac_s_ioctl_ptr {
      uint32  portIndex;             /* Port Number */
      uint32  value;                 /* Buffer which hold the value to be set/get */
} gbe_mac_ioctl_ptr;


/**
 * TYPENAME: gbe_mac_ioctl_ptr_64
 *
 * DESCRIPTION:
 *
 */
typedef struct gbe_mac_s_ioctl_ptr_64 {
    uint32  portIndex;             /* Port Number */
    uint32  valueHigh;             /* High 47:32 bits of the address */
    uint32  valueLow;              /* Low 31:0 btis of the assress */
} gbe_mac_ioctl_ptr_64;


/**
 * TYPENAME: gbe_mac_rmon_stat[102]
 *
 * DESCRIPTION:
 *  This array maintains the statistics information in 2-32 bits register value.
 *  Each register comprises two 32-bit register.
 *
 */
 /**
  * MAC RX Statistics Registers
  *                                  Index in the Array
  *  uint32  rxOk[2];                     0
  *  uint32  rxBad[2];                    2
  *  uint32  rxUcPkts[2];                 4
  *  uint32  rxMcPkts[2];                 6
  *  uint32  rxBcPkts[2];                 8
  *  uint32  rxPkt64[2];                 10
  *  uint32  rxPkt65[2];                 12
  *  uint32  rxPkt128[2];                14
  *  uint32  rxPkt256[2];                16
  *  uint32  rxPkt512[2];                18
  *  uint32  rxPkt1024[2];               20
  *  uint32  rxPkt1519[2];               22
  *  uint32  rxFcsError[2];              24
  *  uint32  rxVlanTag[2];               26
  *  uint32  rxDataError[2];             28
  *  uint32  rxAlignError[2];            30
  *  uint32  rxLongError[2];             32
  *  uint32  rxJabberError[2];           34
  *  uint32  rxPauseMacCtl[2];           36
  *  uint32  rxUnknownCtlFrame[2];       38
  *  uint32  rxVlongError[2];            40
  *  uint32  rxRuntError[2];             42
  *  uint32  rxShortError[2];            44
  *  uint32  rxCarrierExtendError[2];    46  
  *  uint32  rxSeqError[2];              48
  *  uint32  rxSymbolError[2];           50
  *  
  *
  * MAC TX Statistics Registers
  *
  *  uint32  txOk[2];                    52
  *  uint32  txBad[2];                   54
  *  uint32  txUcPkts[2];                56
  *  uint32  txMcPkts[2];                58
  *  uint32  txBcPkts[2];                60
  *  uint32  txPkt64[2];                 62
  *  uint32  txPkt65[2];                 64
  *  uint32  txPkt128[2];                66
  *  uint32  txPkt256[2];                68
  *  uint32  txPkt512[2];                70
  *  uint32  txPkt1024[2];	         72
  *  uint32  txPkt1519[2];               74
  *  uint32  txDeferred[2];              76
  *  uint32  txCollision[2];	         78
  *  uint32  txSingleCollision[2];       80
  *  uint32  txMulCollision[2];          82
  *  uint32  txLateCollision[2];         84
  *  uint32  txExcvCollisionErr[2];      86
  *  uint32  txExcvDeferredErr[2];       88
  *  uint32  txExcvLenDrop[2];           90
  *  uint32  txUnderrun[2];              92
  *  uint32  txVlanTag[2];               94
  *  uint32  txCrcError[2];              96
  *  uint32  txPauseFrame[2];            98
  *  uint32  txFcCollisionSend[2];      100
  */
//uint32 gbeMacRmonStat[4][102];

/**
 * TYPENAME: gbe_mac_add_result[2]
 *
 * DESCRIPTION:
 *  This structure maintains the result of the two 32-bit value addition.
 *
 *   uint32 resultLow;                    0
 *   uint32 resultHigh;                   1
 */
//uint32 gbeMacAddResult[2];


/**
 * Prototypes for interface functions.
 */
/**
 * NAME: gbe_mac_read_32_bits
 *
 * DESCRIPTION:
 *      This function read one register at the value of arg_Offset, and holds the read value 
 *      in the arg_pValue.  this intern calls the READ_REG macro.
 *
 * @Param: -IN    : arg_DeviceNumber
                  : arg_Offset
 * @Param: -INOUT : arg_pValue
 * @Param: -OUT   :
 *
 * @Return: NULL
 */
void gbe_mac_read_32_bits (
                           uint32 arg_DeviceNumber,
                           uint32 arg_Offset,
                           uint32 *arg_pValue
                          );


/**
 * NAME: gbe_mac_read_64_bits
 *
 * DESCRIPTION:
 *      This function read two consecutive registers at the value of arg_Offset, 
 *      and at arg_Offset+1, and holds the read values  
 *      in the arg_pValue1, and arg_pValue2.  This intern calls the READ_REG macro.
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : arg_Offset 
 * @Param: -INOUT : arg_pValue1
 *                : arg_pValue2
 * @Param: -OUT   :
 *
 * @Return: 
 */
void gbe_mac_read_64_bits (
                             uint32 arg_DeviceNumber,
                             uint32 arg_Offset,
                             uint32 *arg_pValue1,
                             uint32 *arg_pValue2
                            );


/**
 * NAME: gbe_mac_write_32_bits
 *
 * DESCRIPTION:
 *      This function write the given value to the specified location
 *      of the register map.  It interns calls the write macros 
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : arg_Offset 
 *                : arg_Data
 * @Param: -INOUT : 
 * @Param: -OUT   :
 *
 * @Return: NULL
 */
void gbe_mac_write_32_bits (
                            uint32 arg_DeviceNumber,
                            uint32 arg_Offset,
                            uint32 arg_Data
                          );

/**
 * NAME: gbe_mac_write_64_bits
 *
 * DESCRIPTION:
 *      This function writes the given values to the specified two consecutive locations
 *      of the register map.  It interns calls the write macros
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : arg_Offset
 *                : arg_Data1
 *                : arg_Data2
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: NULL.
 */
void gbe_mac_write_64_bits (
                                  uint32 arg_DeviceNumber,
                                  uint32 arg_Offset,
                                  uint32 arg_Data1,
                                  uint32 arg_Data2 
                                 );

/**
 * NAME: read_32_bits_from_isr
 *
 *
 * DESCRIPTION:
 *      This function read a register at the value of arg_Offset, and holds the read value
 *      in the arg_pValue.  this inturn calls the READ_REG macro.
 *      This routine is called only called from the interrupt service routine.
 *
 * @Param: -IN    : arg_DeviceNumber
                  : arg_Offset
 * @Param: -INOUT : arg_pValue
 * @Param: -OUT   :
 *
 * @Return: NULL
 */
void read_32_bits_from_isr(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 *arg_pValue
                         );


/** NAME: gbe_mac_phy_read
 *
 * DESCRIPTION:
 *      This function read the PHY register value through MDIO i/f.
 *
 * @Param: -IN    : arg_DeviceAddress
 *                : arg_phy_address
 *                : arg_reg_address
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: Value read or the error
 */
uint32 gbe_mac_phy_read(
                        uint32 arg_DeviceNumber,
                        uint32 arg_phy_address,
                        uint32 arg_reg_address,
                        uint32 *arg_ReadValue
                       );

/** NAME: gbe_mac_phy_write
 *
 * DESCRIPTION:
 *      This function writes to the PHY register value through MDIO i/f.
 *
 * @Param: -IN    : arg_DeviceAddress
 *                : arg_phy_address
 *                : arg_reg_address
 *                : arg_data
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: error code 
 */
gbe_mac_error gbe_mac_phy_write(
                               uint32 arg_DeviceNumber,
                               uint32 arg_phy_address,
                               uint32 arg_reg_address,
                               uint32 arg_data
                              );



/** NAME: gbe_mac_config
 *
 * DESCRIPTION:
 *      This function configures the specific registers to their
 *      configured value.
 *
 * @Param: -IN    : arg_DeviceAddress
 *                : arg_PortMask
 *                : arg_mode
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: gbe_mac_error token.
 */
/*gbe_mac_error gbe_mac_config (*/
gbe_mac_error gbe_mac_config (
                              uint32 arg_DeviceNumber,
                              gbe_mac_IOCTL_CMD arg_PortMask,
                              short arg_mode
                             );

/**
 * NAME: gbe_mac_config_handler
 *
 * DESCRIPTION:
 *      This function parses the GET/SET type ioctl commands.
 *      and calls the get/set config handler accordingly.
 *
 * @Param:  - IN    : arg_DeviceNumber
 *                  : arg_cmd :IOCTL Command number,
 * @Param:  - INOUT : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param:  - OUT
 *
 * @Return: gbe_mac_error token.
 */
 gbe_mac_error gbe_mac_config_handler (
		                             uint32 arg_DeviceNumber,
                                             gbe_mac_IOCTL_CMD arg_cmd,
                                             void * arg_pIoctlPtr
                                            );
/**
 * NAME: gbe_mac_get_config_handler
 *
 * DESCRIPTION:
 *      This function implements the GET type ioctl commands.  It
 *      reads the value of the specified register and store that
 *      value in the value parameter of the IOCTL_PTR passed to it
 *      as a void pointer.  And based upon the read value it generate
 *      the ERROR CODE.
 *
 * @Param:  -IN    : arg_DeviceNumber
 *                 : arg_cmd :IOCTL Command number,
 * @Param:  -INOUT : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param:  -OUT   :
 *
 * @Return: gbe_mac_error token.
 */
gbe_mac_error gbe_mac_get_config_handler (
		                          uint32 arg_deviceNumber,
                                          gbe_mac_IOCTL_CMD arg_cmd,
                                          void * arg_pIoctlPtr
                                         );

/**
 * NAME: gbe_mac_set_config_handler
 *
 * DESCRIPTION:
 *      This function implements the SET type ioctl commands.  It write
 *      the given value, passed through the IOCTL_PTR, to the specified
 *      register.
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : arg_cmd :IOCTL Command number,
 *                : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param: -INOUT :
 * @Param: -OUT
 *
 * @Return: gbe_mac_error token.
 */
gbe_mac_error gbe_mac_set_config_handler (
		                          uint32 arg_DeviceNumber,
                                          gbe_mac_IOCTL_CMD arg_cmd,
                                          void * arg_pIoctlPtr
                                         );

/**
 * NAME: gbe_mac_reconfig_handler
 *
 * DESCRIPTION:
 *      This function implements the SET type ioctl commands.  It sets
 *      the configuration registers whenever the configuration of the
 *      MAC port is changed.
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : arg_cmd :IOCTL Command number,
 *                : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param: -INOUT :
 * @Param: -OUT
 *
 * @Return: gbe_mac_error token.
 */
gbe_mac_error gbe_mac_reconfig_handler(
                                       uint32 arg_DeviceNumber,
		                       gbe_mac_IOCTL_CMD arg_cmd,
                                       void * arg_pIoctlPtr
                                      );

/**
 * NAME: gbe_mac_isr
 *
 * DESCRIPTION:
 *      This ISR is called when the interrupt comes.
 *
 * @Param: -IN    : arg_DeviceNumber
 * @Param: -INOUT
 * @Param: -OUT
 *
 * @Return: NULL.
 */
void gbe_mac_isr(
                 uint32 arg_DeviceNumber
                );



/**
 * Exported variables.
 */
#if !defined(__LINUX_USER__)                    /* Common global data for Linux kernel mdoe, and VxWorks */
typedef struct gbe_mac_s_global_data {
        uint32 g_PortState[4];                  /* Maitianing the Port Status saperately for each port */
        uint32 g_PortUsageCount[4];             /* Maintains per port usage count, it helps in determing
						   the other users, while one is trying to close the port. */
}gbe_mac_global_data;

#else                                           /* Global Data specific to the Linux User Mode driver */
typedef struct gbe_mac_s_global_data {
        uint32 g_PortState[4];                  /* Maitianing the Port Status saperately for each port */
        uint32 g_PortUsageCount[4];             /* Maintains per port usage count, it helps in determing
                                                   the other users, while one is trying to close the port. */
        uint32 g_UsageCount;                    /* Maintains the Total number of application using the individually
                                                   Pt Lone media card saperately. */
        uint32 g_MacStatus;
        int  g_MutexSemaphore;                /* Mutex Semaphore */

}gbe_mac_global_data;
#endif /* end defined __LINUX_USER__ */


typedef struct gbe_mac_s_global_struct {
       uint32 gbeMacRmonStat[4][102];        /* RMON Structure */
       uint32 gbeMacAddResult[2];            /* Structure holding the sum of the two 32-bit value addition */
} gbe_mac_global_struct;


/**
 * This srtucture holds the MAC address of the MAC ports.  This value is updated from
 * the value read from the I2C EEPROM
 */
typedef struct gbe_mac_s_mac_address {
       uint32   macAddressPort_0[2];
       uint32   macAddressPort_1[2];
       uint32   macAddressPort_2[2];
       uint32   macAddressPort_3[2];
} gbe_mac_mac_address;

#if defined(__LINUX_USER__)
/**
 * structure for storing the callback function, and relevant data
 */


typedef struct gbe_mac_s_calback {
        void      (*p_CallbackFn)();
        uint32    appHandle;
        void *    p_UserContext;
}gbe_mac_callback;

#endif  /* __LINUX_USER__ */



/**
 * EEPROM Structure used in the Pt Lone Media Card.
 *
 * struct _s_media_card_eeprom_content {
 *        unsigned char   prom_fmt;
 *        unsigned char   dev_id;
 *        unsigned char   man_id[12];
 *        unsigned char   part_rev [2];
 *
 *        unsigned char   part_num[10];
 *        unsigned char   serial_num[10];
 *        unsigned char   model_id[12];
 *
 *        unsigned char   rsvd1[8];
 *        unsigned char   passwd[8];
 *
 *        unsigned char   ether_mac_addr_port_0[6];
 *        unsigned char   ether_mac_addr_port_1[6];
 *        unsigned char   rsvd2[4];
 *
 *        unsigned char   ether_mac_addr_port_2[6];
 *        unsigned char   ether_mac_addr_port_3[6];
 *        unsigned char   rsvd3[12];
 * };
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /*end defined __IXF1104CE_INTERNAL_H__*/
