/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_ioctl.h
 *
 * = DESCRIPTION
 *      This header file contains the IOCTL commands and their value
 *      and the Error Code numbers, along with the API which are
 *      exposed to the user to use the corresponding driver. 
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *      6/20/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 * $Id: ixf1104ce_ioctl.h,v 1.1.1.1 2003/04/11 23:17:25 nmafzal Exp $
 */

#if !defined(__IXF1104CE_IOCTL_H__)
#define __IXF1104CE_IOCTL_H__

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Pre-processor symbol and macro definitions.
 */

#define GET_IOCTL       0xE300  /* GET type IOCTLs starts from 0xE300 */
#define SET_IOCTL       0xE200  /* SET type IOCTLs starts from 0xE200 */

#define TRUE            1
#define FALSE           0 

/**
 * Type definitions.
 */

/**
 * TYPENAME: gbe_mac_error
 *
 * DESCRIPTION: Error code enumerator
 *
 */
typedef enum gbe_mac_e_error {
    SUCCESS = 0x0000,
    REGISTRATION_FAILED = 0x0400,
    ERROR_DEVICE_ALREADY_INITIALIZED,			/* 0x0401 */
    PLL_NOT_LOCKED,					/* 0x0402 */
    INTERRUPT_INSTALLATION_FAILED,			/* 0x0403 */
    INTERRUPT_FREEING_FAILED,				/* 0x0404 */
    IOCLT_ERROR_INPUT_REG_VALUE_OUT_OF_RANGE,		/* 0x0405 */
    IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID,		/* 0x0406 */
    IOCTL_ERROR_UNKNOWN_IOCTL_CODE, 			/* 0x0407 */
    IOCTL_BUFFER_POINTER_NULL,				/* 0x0408 */
    IOCTL_BUFFER_POINTER_INVALID,			/* 0x0409 */
    IOCTL_PORT_NOT_OPEN,				/* 0x040A */
    MODE_ALREADY_SET_IN_SPECIFIED_DUPLEX_MODE,		/* 0x040B */
    ERROR_INVALID_DUPLEX_MODE,				/* 0x040C */
    FAIL_RX_FIFO_ERRORED_FRAME_DROP_IS_DISABLE,		/* 0x040D */
    ERROR_CONFLICT_WITH_HIGH_WATERMARK,			/* 0x040E */
    ERROR_VALID_ONLY_FOR_COPPER_MODE,			/* 0x040F */
    ERROR_VALID_ONLY_FOR_FIBER_MODE,			/* 0x0410 */
    ERROR_MDI_ENABLE_BIT_IS_RESET_IN_MDI_CONTROL_REG,	/* 0x0411 */
    ERROR_CONFLICT_WITH_LOW_WATERMARK,			/* 0x0412 */
    ERROR_NOT_ALLOWED_IN_LOOPBACK_MODE,			/* 0x0413 */
    ERROR_CONFLICT_WITH_JUMBO_FRAME_SIZE,		/* 0x0414 */
    PLL_TRANSMIT_LOCK_STATUS_FAILED,			/* 0x0415 */
    IOCTL_READ_VALUE_OUT_OF_RANGE,			/* 0x0416 */
    ERROR_COULD_NOT_COPY_RMON_STAT_FROM_KERNEL_MODE,	/* 0x0417 */
    ERROR_MAC_NOT_INITIALIZED,				/* 0x0418 */
    ERROR_GET_RMON_STAT_TASK_COULD_NOT_SPAWNED,		/* 0x0419 */
    ERROR_COULD_NOT_OPEN_DEVICE,			/* 0x041A */
    SEMAPHORE_COULD_NOT_CREATED,			/* 0x041B */
    ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE,			/* 0x041C */
    ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE,		/* 0x041D */
    EEPROM_OF_PT_LONE_MEDIA_CARD_ON_MEDIA_INTERFACE_IS_NOT_PROGRAMMED,		/* 0x041E */
    EEPROM_OF_PT_LONE_MEDIA_CARD_ON_SWITCH_FABRIC_INTERFACE_IS_NOT_PROGRAMMED,	/* 0x041F */
    ERROR_INVALID_HANDLE,				/* 0x0420 */
    ERROR_PREVIOUS_MDI_COMMAND_STILL_NOT_COMPLETED,	/* 0x0421 */
    ERROR_COULD_NOT_PERFORM_MDIO_READ,			/* 0x0422 */
    ERROR_MDI_RD_WR_DISABLE,				/* 0x0423 */
    ERROR_COULD_NOT_PERFORM_MDIO_WRITE,			/* 0x0424 */
    SPI3_TX_LOCK_FAILED,				/* 0x0425 */
    ERROR_SHARED_MEMORY_NOT_CREATED,			/* 0x0426 */
    ERROR_SHARED_MEMORY_NOT_LOCKED,			/* 0x0427 */
    ERROR_SHARED_MEMORY_NOT_UNLOCKED,			/* 0x0428 */
    ERROR_SHARED_MEMORY_NOT_DESTROYED,			/* 0x0429 */
    ERROR_SHARED_MEMORY_NOT_DETATCHED,			/* 0x042A */
    ERROR_SHARED_MEMORY_NOT_ATTACHED,			/* 0x042B */
    ERROR_SEMAPHORE_NOT_GRABED,				/* 0x042C */
    INVALID_PORT_MASK,					/* 0x042D */
    MEMORY_ALLOCATION_FAILS,				/* 0x042E */
    ERROR_SP_CSR_BASE_COULD_NOT_MAPPED,			/* 0x042F */
    WARNING_ONLY_LOWER_FOUR_BYTES_ARE_WRITABLE,		/* 0x0430 */
    WARNING_ONLY_LOWER_THREE_BITS_ARE_WRITABLE,		/* 0x0431 */
    NO_MORE_CALLBACK_FOR_INTERRUPT, 			/* 0x0432 */
    ERROR_PORT_ALREADY_OPENED = 			0x0440,
    ERROR_INPUT_PORT_NUMBER_INVALID,    		/* 0x0441 */
    ERROR_DEVICE_NOT_OPENED,	          	        /* 0x0442 */
    ERROR_PORT_NOT_OPEN,                     		/* 0x0443 */
    ERROR_CALLED_IOCTL_FAILED 			        /* 0x0444 */

} gbe_mac_error;

/**
 * TYPENAME: gbe_mac_e_IOCTL_CMD
 * 
 * DESCRIPTION: 
 *
 */
typedef enum gbe_mac_e_IOCTL_CMD {
/**
 * MAC Control SET IOCTL Commands
 */
    SET_STN_ADDR = 0xE200,
    SET_DUPLEX_MODE,		/* 0xE201 */
    SET_FDFC_TYPE,		/* 0xE202 */
    SET_COLLISION_DIST,		/* 0xE203 */
    SET_COLLISION_THLD,		/* 0xE204 */
    SET_FCTX_TIMER,		/* 0xE205 */
    SET_FDFC_ADDR,		/* 0xE206 */
    SET_IPG_RECEIVE_TIME1,	/* 0xE207 */
    SET_IPG_RECEIVE_TIME2,	/* 0xE208 */
    SET_IPG_TRANSMIT_TIME,	/* 0xE209 */
    SET_PAUSE_THRESHOLD,     	/* 0xE20A */
    SET_MAX_FRAME_SIZE,		/* 0xE20B */
    SET_MAC_IF_MODE,		/* 0xE20C */
    SET_FLUSH_TX,		/* 0xE20D */
    SET_FC_MODE,		/* 0xE20E */
    SET_FC_BACK_PRESSURE_LEN,	/* 0xE20F */
    SET_SHORT_RUNT_TH,          /* 0xE210 */
    SET_UNKNOWN_FRAME_STT,	/* 0xE211 */
    SET_TX_CONFIG_WORD,		/* 0xE212 */
    SET_DIV_CONFIG_WORD,	/* 0xE213 */
    SET_PKT_FILTER_CTL,		/* 0xE214 */
    SET_MUL_PORT_ADD,         	/* 0xE215 */

/**
 * SET IOCTL Commands for configuring Global registers
 */
    SET_PORT_STATUS,  		/* 0xE216 */
    SET_INTERFACE_MODE,		/* 0xE217 */
    SET_MAC_SOFT_RESET,		/* 0xE218 */
    SET_MDIO_RESET,		/* 0xE219 */
    SET_UI_ENDIAN_MODE,  	/* 0xE21A */
    SET_LED_MODE,		/* 0xE21B */
    SET_LED_FLASH_RATE,		/* 0xE21C */
    SET_LED_FAULT_ACTION, 	/* 0xE21D */

/**
 * SET IOCTLs used to configure the RX FIFO
 */
    SET_RFIFO_HIGH_WATERMARK,	/* 0xE21E */ 
    SET_RFIFO_LOW_WATERMARK,	/* 0xE21F */
    SET_RX_FIFO_PORT_RESET,    	/* 0xE220 */  
    SET_RX_FIFO_FRAME_DROP,	/* 0xE221 */
    SET_INDIRECT_ADR_CTL,	/* 0xE222 */
    SET_CAPTURE_ENABLE_RX_FIFO,	/* 0xE223 */
    SET_PRE_PENDING_CRC_ENABLE,	/* 0xE224 */
    SET_MATCHING_PATTERN,	/* 0xE225 */
    SET_JUMBO_PKT_SIZE,    	/* 0xE226 */ 

/**
 * SET IOCTL list to configure the TX FIFO
 */
    SET_TX_FIFO_HIGH_WATERMARK,      /*e227*/
    SET_TX_FIFO_LOW_WATERMARK,       /*e228*/ 
    SET_MAC_THRESHOLD,               /*e229*/
    SET_LOOP_RX_TX,                  /*e22a*/
    SET_TX_FIFO_PORT_RESET,          /*e22b*/
    SET_TX_INSERT_DATA,              /*e22c*/
    SET_TX_FIFO_INFO_ADR,            /*e22d*/
    SET_TX_FIFO_DROP_INSERT,         /*e22e*/
    SET_TX_MINI_FRAME_SIZE,          /*e22f*/

/**
 * SET IOCTLs to configure MDIO interface
 */
    SET_MDIO_CMD_ADDR,              /*e230*/
    SET_MDIO_SINGLE_RW_DATA,        /*e231*/
    SET_AS_PHY_ADDR,                /*e232*/
    SET_MDIO_CTL,                   /*e233*/

/**
 * SET IOCTLS to configure SPI-3 Interface
 */
    SET_SPI3_TX_CONFIG,             /*e234*/
    SET_SPI3_RX_CONFIG,             /*e235*/
    SET_CLOCK_IF_MODE,              /*e236*/

/**
 * SET IOCTLs used to configure SERDES Interface
 */
    SET_ACDC_COUPLING,              /*e237*/
    SET_SERDES_TX_DRV_COEFF,        /*e238*/
    SET_TX_DRV_POW_LEVEL,           /* e239*/
    SET_TX_LINK_VALIDATION,         /*e23a*/
    SET_TX_RX_POW_DOWN,             /*e23b*/
    SET_RX_DATA_SYNC,               /*e23c*/

/**
 * SET IOCTLs to control GBIC module 
 */
    SET_GBIC_CTL,                   /*e23d*/
    SET_I2C_CTL_DATA,               /*e23e*/
    SET_PLL_TUNE_1,                 /*e23f*/
    SET_PLL_TUNE_2,                 /*e240*/
    SET_PLL_TUNE_3,                 /*e241*/

/**
 * Used in Driver API 
 */
    SET_PORT_INIT_ENABLE,           /*e242*/
    SET_PORT_STATE_CLOSE,           /*e243*/

/**
 * Configuring the Copper PHY register
 */
    SET_PHY_REGISTER,               /*e244*/

/**
 * Configuring the Desired mode and speed
 */
    SET_CONFIG_MODE,                /*e245*/


/**
 * MAC Control GET IOCTL Commands
 */
    GET_STN_ADDR = 0xE300,
    GET_DUPLEX_MODE,
    GET_FDFC_TYPE,
    GET_COLLISION_DIST,
    GET_COLLISION_THLD,
    GET_FCTX_TIMER,
    GET_FDFC_ADDR,
    GET_IPG_RECEIVE_TIME_1,
    GET_IPG_RECEIVE_TIME_2,
    GET_IPG_TRANSMIT_TIME,
    GET_PAUSE_THRESHOLD,
    GET_MAX_FRAME_SIZE,
    GET_MAC_IF_MODE,
    GET_FLUSH_TX,
    GET_FC_MODE,
    GET_FC_BACK_PRESSURE_LEN,
    GET_SHORT_RUNT_TH,
    GET_UNKNOWN_FRAME_STT,
    GET_RX_CONFIG_WORD,
    GET_TX_CONFIG_WORD,
    GET_DIV_CONFIG_WORD,
    GET_PKT_FILTER_CTL,
    GET_MUL_PORT_ADD,           /* 0xe316 */

/**
 * GET IOCTLS for MAC Receive Statistics Counters.
 */
    GET_RX_OCTETS_OK,           /* 0xe317 */
    GET_RX_OCTETS_BAD,
    GET_RX_UC_PKTS,
    GET_RX_MC_PKTS,
    GET_RX_BC_PKTS,
    GET_RX_PKTS_64,
    GET_RX_PKTS_65_127,
    GET_RX_PKTS_128_255,
    GET_RX_PKTS_256_511,
    GET_RX_PKTS_512_1023,
    GET_RX_PKTS_1024_1518,
    GET_RX_PKTS_1519_MAX,
    GET_RX_FCS_ERR,
    GET_VLAN_TAG,
    GET_RX_DATA_ERR,
    GET_RX_ALLIGN_ERR,
    GET_RX_LONG_ERR,
    GET_RX_JABBER_ERR,
    GET_RX_PAUSE_MAC_CTL,
    GET_RX_UNKNOWN_CTL_FRAME,
    GET_VLONG_ERR,
    GET_RUNT_ERR,
    GET_SHORT_ERR,
    GET_SEQ_ERR,
    GET_SYMBOL_ERR,           /* e32f */

/**
 * GET IOCTLS for MAC Transmit Statistics Counters.
 */
    GET_TX_OCTETS_OK,         /* e330 */
    GET_TX_OCTETS_BAD,
    GET_TX_UC_PKTS,
    GET_TX_MC_PKTS,
    GET_TX_BC_PKTS,
    GET_TX_PKTS_64,
    GET_TX_PKTS_65_127,
    GET_TX_PKTS_128_255,
    GET_TX_PKTS_256_511,
    GET_TX_PKTS_512_1023,
    GET_TX_PKTS_1024_1518,
    GET_TX_PKTS_1519_MAX,
    GET_TX_DEFERRED_ERR,
    GET_TX_TOTAL_COLLISION,
    GET_TX_SINGLE_COLLISION,
    GET_TX_MUL_COLLISION,
    GET_LATE_COLLISION,
    GET_TX_EXCV_COLLISION,
    GET_TX_EXCV_DEFERRED_ERR,
    GET_TX_EXCV_LEN_DROP,
    GET_TX_UNDERRUN,
    GET_TX_VLAN_TAG,
    GET_TX_CRC_ERR,
    GET_TX_PAUSE_FRAME,
    GET_FC_COLLISION_SEND,        /* e348 */

/**
 * GET IOCTL Commands for accessing Global Status and Configuration registers
 */
    GET_PORT_STATUS,              /* e349 */
    GET_INTERFACE_MODE,
    GET_LINK_UP_STATUS,
    GET_RESET_CORE_CLOCK,
    GET_PAUSE_BEHAVIOR,
    GET_MAC_SOFT_RESET,
    GET_MDIO_RESET,
    GET_UI_ENDIAN_MODE,
    GET_LED_MODE,
    GET_LED_FLASH_RATE,
    GET_LED_FAULT_ACTION,
    GET_JTAG_ID,                  /* e354 */

/**
 * GET IOCTLs commands to monitor the configuration of the RX FIFO
 */
    GET_RX_FIFO_HIGH_WATERMARK,   /* e355 */
    GET_RX_FIFO_LOW_WATERMARK,
    GET_RX_FRAME_REMOVED,
    GET_RX_FIFO_PORT_RESET,
    GET_RX_FIFO_FRAME_DROP,
    GET_RX_FIFO_OVERFLOW_STT,
    GET_OUT_SEQUENCE_INFO,
    GET_DROPPED_PKTS,
    GET_RW_PTR_RX_FIFO,
    GET_OCCUPANCY_RX_FIFO,
    GET_CAPTURED_PKT_LEN,
    GET_INDIRECT_ADR_CTL,
    GET_READ_DATA,
    GET_CAPTURE_ENABLE_RX_FIFO,
    GET_PRE_PENDING_CRC_ENABLE,
    GET_MATCHING_PATTERN,
    GET_JUMBO_PKT_SIZE,
    GET_PKT_DROP_CAP_FIFO,        /* e366 */

/**
 * GET IOCTL commands to monitor the TX FIFO
 */
    GET_TX_FIFO_HIGH_WATERMARK,   /* e367 */
    GET_TX_FIFO_LOW_WATERMARK,
    GET_MAC_THRESHOLD,
    GET_TX_FIFO_OVERFLOW_STT,
    GET_LOOP_RX_TX_STT,
    GET_TX_FIFO_PORT_RESET,
    GET_TX_DROP_FRAME,
    GET_TX_DROP_PKTS,
    GET_TX_RW_PTR,
    GET_TX_OCCUPANCY,
    GET_TX_INSERT_DATA,
    GET_TX_FIFO_INFO_ADR,
    GET_TX_FIFO_DROP_INSERT,
    GET_TX_MINI_FRAME_SIZE,       /* e374 */

/**
 * GET IOCTLs to monitor MDIO interface
 */
    GET_MDIO_CMD_ADDR,            /* e375 */
    GET_MDIO_SINGLE_RW_DATA,
    GET_AS_PHY_ADDR,
    GET_MDIO_CTL,                 /* e378 */

/**
 * GET IOCTLS to monitor SPI-3 Interface
 */
    GET_SPI3_TX_CONFIG,            /* e379 */
    GET_SPI3_RX_CONFIG,
    GET_SPI3_TX_INT_STATUS,
    GET_SPI3_ADR_PARITY_ERROR,
    GET_SPI3_PKT_DISABLE_PORT,
    GET_SPI3_PKT_SYNC_ERR,
    GET_SPI3_PKT_SHORT_DROP,        /* e37f */

/**
 * GET IOCTLs to monitor SERDES Interface
 */
    GET_ACDC_COUPLING,             /* e380 */
    GET_SERDES_TX_DRV_COEFF,
    GET_TX_DRV_POW_LEVEL,
    GET_TX_LINK_VALIDATION,
    GET_TX_RX_POW_DOWN_STT,
    GET_RX_DATA_SYNC,
    GET_RX_LINK_VALIDATION,
    GET_RX_LINK_STT,
    GET_RX_PHASE_ROT,
    GET_RX_PHASE_ROT_BUS,
    GET_RX_LATCH_OBSRV_01,
    GET_RX_LATCH_OBSRV_23,
    GET_RX_SIGNAL_LEVEL,
    GET_CLOCK_INTERFACE_MODE,
    GET_SERDES_TX_CONFIG,
    GET_SERDES_RX_CONFIG,
    GET_PLL_LOCK,                  /* e390 */

/**
 * GET IOCTLs to monitor GBIC module 
 */
    GET_GBIC_STAUS,       /* e391 */
    GET_GBIC_CTL,
    GET_I2C_CTL_DATA,
    GET_PLL_TUNE_1,
    GET_PLL_TUNE_2,
    GET_PLL_TUNE_3,

/**
 * Get RMON statistics.
 */
    GET_RMON_STAT,

/**
 * Get the PHY register
 */
    GET_PHY_REGISTER

} gbe_mac_IOCTL_CMD;


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* #ifndef __IXF1104CE_IOCTL_H__ */
