/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_config.c
 *
 * = DESCRIPTION
 *      This file contains the functions used in configuring the Vallejo MAC 
 *      to the specified mode, given by the application.
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *       6/20/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf1104ce_config.c,v 1.2 2003/05/15 11:44:51 dheeraj Exp $";


/**
 * System defined include files required.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#if !defined(__LINUX_USER__)
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/delay.h>
#endif /* end __LINUX__ */

/**
 * User defined include files required.
 */
#include "ixf1104ce_ioctl.h"
#include "ixf1104ce_internal.h"
                                                  

#if defined(__LINUX_USER__)  /* Make sure for the corresponding file path */
#include "ixf1104ce_shm.h"
#endif /* end __LINUX_USER__ */


/**
 * Types definitions whose scope is limited to this file.
 */
#define PORT_MASK               0xF
#define HIGH_BIT                0x1
#define LOW_BIT                 0x0

#define DEFAULT_SET             0x1
 
#define INT_MASK_SPI3           0x000001FF
#define INT_MASK_GBIC           0x00F03C0F

#define INT_PART_SYNC           0x00000100 
#define INT_FULL_SYNC           0x00000080 
#define INT_DATA_PARITY         0x00000040 
#define INT_ADDR_PARITY         0x00000020 
#define INT_DISABLE_PORT        0x00000010 
#define INT_TX_ERR              0x00000008 
#define INT_SSOP_ERR            0x00000004 
#define INT_SMALL_PKT_0_8       0x00000002 
#define INT_SMALL_PKT_9_14      0x00000001 

#define INT_RX_LOST_3           0x00800000 
#define INT_RX_LOST_2           0x00400000 
#define INT_RX_LOST_1           0x00200000 
#define INT_RX_LOST_0           0x00100000 

#define INT_TX_FAULT_3          0x00002000
#define INT_TX_FAULT_2          0x00001000
#define INT_TX_FAULT_1          0x00000800
#define INT_TX_FAULT_0          0x00000400

#define INT_MOD_DEF_3           0x00000008
#define INT_MOD_DEF_2           0x00000004
#define INT_MOD_DEF_1           0x00000002
#define INT_MOD_DEF_0           0x00000001

#define RXLOS_MASK              0x00F00000
#define TXLOS_MASK              0x00003C00
#define MODDEF_MASK             0x0000000F

#define MAKE_AND                0x7FFFFFFF
#define SHIFT_RIGHT             31
#define SHIFT_NIBBLE            0x04

#define SPI3_BLOCK_MODE         0x10
#define SPI3_PARITY_MODE        0x08
#define SPI3_4x8_MODE           0x0020AA0F
#define SPI3_MPHY_MODE          0x0000AA0F
#define ODD_PARITY              0x0
#define EVEN_PARITY             0x1


#define MII_MODE_STATUS          0x4
#define MDI_PROGRESS_STATUS      0x8
#define MDI_CONTROL_VAL          0x6
#define MDI_AUTOSCAN_ENABLE      0xF

#define MDI_ENABLE               0x04
#define PHY_REG_ADDRESS          0x1F
#define PHY_DEVICE_ADDRESS       0x03
#define PHY_DEVICE_SHIFT         0x08
#define PHY_READ                 0x120000
#define PHY_WRITE                0x110000
#define MDI_STATUS_SHIFT         0x03
#define MDIO_CMD_STATUS          0x14
#define SHIFT_WORD               0x10

#define MDI_INT_STATUS_OFFSET    0x1A
#define MDI_LINK_STATUS          0x01

#define CARD_ON_MEDIA_IF         0x0

#define SHIFT_BYTE               0x08

#define PLL1_AFTER_RESET        0x534
#define PLL2_AFTER_RESET        0x534
#define PLL3_AFTER_RESET        0x5BE

#define SPI3_TX_INT_STATUS      0x702
#define PORTSZ                  0x80

#define SPI3_RX_CORE_ENABLE 		0x00000080
#define SPI3_RX_PORT_ENABLE   		0x00AA0f80
#define SPI3_RX_EVEN_PARITY_MASK 	0x0000f000

/**
 * defined while using the taskDelay, supposed to be removed,
 * once the timer APIs are available
 */
#define SLEEP_TIME             2
#define PLL_LOCK_TIMER         20

#define OFFSET_MULT 1

/** 
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
#if defined (__LINUX_USER__)
extern gbe_mac_shared_data *g_pGbeMacSharedData;
#else
extern gbe_mac_global_data gbeMacGlobalData[2];

/**
 * structure used to signal a user process
 */ 
extern struct fasync_struct *i2s_async_queue;

#endif

extern gbe_mac_global_struct gbeMacGlobalStruct[2]; /* Each one structure for each Pt Lone Media Card */
extern gbe_mac_mac_address gbeMacAddress[2];             /* For the two Pt Lone cards */

#if defined __LINUX__ || __LINUX_USER__
#define SP_CCR_OFF              0x0
#define SP_WTC1_OFF             0x4
#define SP_WTC2_OFF             0x8
#define SP_RTC1_OFF             0xC
#define SP_RTC2_OFF             0x10
#define SP_PCR_OFF              0x18
#define SP_ADC_OFF              0x1c
#define SP_FAC_OFF              0x20
#define SP_FRM_OFF              0x24


#define IXP2400_REG_READ(reg,result) \
                (*(result) = *(volatile unsigned int *)(reg))
#define IXP2400_REG_WRITE(reg,data)  \
                (*((volatile unsigned int *)(reg)) = (data))
#endif //__LINUX__

#if defined __LINUX__
typedef struct _SPCfg_ {
  
   uint32 spCCR;
   uint32 spWTC2;
   uint32 spRTC2;
   uint32 spPCR;
   uint32 spADC;

} SPCfg;

#endif // __LINUX__

#if !defined(__LINUX_USER__)
#if defined(__LINUX__)
    struct timer_list s_RmonStatTimer_MI;
    struct timer_list s_RmonStatTimer_SF;

#endif
#endif

/** Global Variable to store the PHY interrupt status register Value */
uint32 s_PhyIntReadValue[2][4];



/* offset per port */
extern uint32 g_PerPortOffset[4];

#if defined __LINUX_USER__ || __LINUX__

extern uint32 pSPCfgBase;
extern uint32 *PT_LONE_REG_BASE_0;
extern uint32 *PT_LONE_REG_BASE_1;
#endif  /*  __LINUX_USER__ || __LINUX__ */

#if defined __LINUX_USER__ || __LINUX__
/* structure corresponds to the slowport registers */

int  spCfgOld[5];
int  spCfgPtLone[5] = {  
                           0x0d,
                           0x62,
                           0x62,
                           0x03,
                           0x33
                          };
#endif /* if defined __LINUX__ || __LINUX_USER__ */

#if !defined(____LINUX_USER__)
unsigned int gbic_status = 0;
#endif /* __LINUX_USER__ */


uint32 spi3_status_port0 = 0;
uint32 spi3_status_port1 = 0;
uint32 spi3_status_port2 = 0;
uint32 spi3_status_port3 = 0;


/**
 * Static function prototypes.
 */
/**
 * NAME: gbe_mac_evaluate_rmon_stat
 *
 * DESCRIPTION
 *       This function reads the RMON Statistics register values and accumulates
 *       in a saperate structure, which maintains the 64-bit counter for holding 
 *       the RMON Statistics values.
 *
 * @Param: -IN    :arg_DeviceNumber
 *                :Port Number
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: gbe_mac_error token.
 */
static gbe_mac_error gbe_mac_evaluate_rmon_stat(
                                                uint32 arg_DeviceNumber,
                                                uint32 arg_PortNumber
                                               );


/**
 * NAME: gbe_mac_add_two_lword
 *
 * DESCRIPTION:
 *       This function reads the specified RMON statistics register, and add it to
 *       its corresponding 64-bit counter, and updates the add_structure fields.
 *
 * @Param: -IN    : arg_DeviceNumber
 *                : uint32 arg_Low     Low 32-bit value of the 64-bit counter register
 *                : uint32 arg_High    High 16-bit value of the 64-bit counter register
 *                : uint32 arg_Value   RMON register value
 * @Param: -INOUT
 * @Param: -OUT
 *
 * @Return: gbe_mac_error.
 */
gbe_mac_error gbe_mac_add_two_lword(
                                           uint32 arg_DeviceNumber,
                                           uint32 arg_Low,
                                           uint32 arg_High,
                                           uint32 arg_Value
                                          );

#if !defined(__LINUX_USER__)   /* for Linux kernel Mode only */
/**
 * TYPENAME: gbe_mac_update_rmon_MI (unsigned long arg_ptr)
 *
 * DESCRIPTION:  
 *           This function updates the RMON statistics for the card
 *           present on the Media interface.
 *
 * Parameters
 * IN    :  Unsigned long *
 * INOUT :
 * OUT   :
 *
 * Returns: NULL
 */
void gbe_mac_update_rmon_MI (
                             unsigned long arg_ptr
                            );

/**
 * TYPENAME: gbe_mac_update_rmon_SF (unsigned long arg_ptr)
 *
 * DESCRIPTION:
 *           This function updates the RMON statistics for the card
 *           present on the Switch fabric interface.
 *
 * Parameters
 * IN    : unsigned long *
 * INOUT :
 * OUT   :
 *
 * Returns: NULL
 */
void gbe_mac_update_rmon_SF (
                             unsigned long arg_ptr
                            );


/**
 * TYPENAME: gbe_mac_copper_int_stat_MI (unsigned long arg_ptr)
 *
 * DESCRIPTION:
 *           This function is called in the ISR if the copper interrupt
 *           is received from the card on the Media interface.
 *
 * Parameters:
 * IN     :unsigned long *
 * INOUT  :
 * OUT    :
 *
 * Returns:NULL
 */
void gbe_mac_copper_int_stat_MI (
                             unsigned long arg_ptr
                            );

/**
 * TYPENAME: gbe_mac_copper_int_stat_SF (unsigned long arg_ptr)
 *
 * DESCRIPTION:
 *           This function is called in the ISR if the copper interrupt
 *           is received from the card on the Switch fabric interface.
 *
 * Parameters:
 * IN     :unsigned long *
 * INOUT  : 
 * OUT    :
 *
 * Returns: NULL
 */
void gbe_mac_copper_int_stat_SF (
                             unsigned long arg_ptr
                            );
#else
/* Needs to define Mechanism to get RMON statistics in User Mode */
#endif /* __LINUX_USER__ */

#if defined  __LINUX_USER__
void ixdp2400SPCfgSave(unsigned int  *SP_old_config);
void ixdp2400SPCfgRestore(unsigned int  *SP_new_config);
#else /*if defined __LINUX_USER__ */

void ixdp2400SPCfgSave(unsigned int *SPREG);
void ixdp2400SPCfgRestore(unsigned int *SPREG);
#endif

/**
 * TYPENAME: void config_phy
 *
 * DESCRIPTION:
 *        This function performa the basic PHY configuration, based on 
 *        the information, in which mode and speed	 
 *
 * Parameters:
 * IN        : uint32 arg_DeviceNumber,
 *           : uint32 speed,   
 *           : uint32 duplex ,
 *           : uint32 arg_PortMask
 * INOUT
 * OUT
 *
 * Returns:  NULL
 */

void config_phy(
                uint32 arg_DeviceNumber,
                uint32 speed,
                uint32 duplex ,
                uint32 arg_PortMask
           );

/**
 * TYPENAME: mac_reset
 *
 * DESCRIPTION:
 *            This function puts the MAC in reset state.
 *
 * Parameters:
 * IN        : uint32 arg_DeviceNumber
 * INOUT
 * OUT
 *
 * Returns:NULL
 */
void mac_reset(
               uint32 arg_DeviceNumber
              );

/**
 * TYPENAME: mac_disable_reset
 *
 * DESCRIPTION:
 *        This function disables the MAC reset state.
 *
 * Parameters:
 * IN      : uint32 arg_DeviceNumber
 * INOUT
 * OUT
 *
 * Returns:NULL
 */
void mac_disable_reset(
                       uint32 arg_DeviceNumber
                      );

/**
 * TYPENAME: gbe_mac_basic_configuration
 *
 * DESCRIPTION:
 *      This fuction performa the basic configuration of the Vallejo MAC
 *      in the Fiber mode.
 *
 * Parameters:
 * IN        : uint32 arg_DeviceNumber,
 *           : uint32 arg_portMask,
 *           : uint32 arg_mode
 * INOUT
 * OUT
 *
 * Returns:NULL
 */
void gbe_mac_basic_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_portMask,
                                 uint32 arg_mode
                                );

/**
 * TYPENAME: basic_diverse_config
 *
 * DESCRIPTION:
 *      This fuction changes the basic configuration of the Vallejo MAC
 *      in the specified mode, speed and duplex.
 *
 * Parameters:
 * IN        : uint32 arg_DeviceNumber,
 *           : uint32 arg_portMask,
 *           : uint32 arg_mode
 * INOUT
 * OUT
 *
 * Returns:NULL
 */
void basic_diverse_config(
                           uint32 arg_DeviceNumber,
                           uint32 arg_PortMask,
                           uint32 s_DiverseConfigValue
                         );

/**
 * TYPENAME: basic_duplex_configuration
 *
 * DESCRIPTION:
 *     This function writes the duplex mode value to the respective
 *     register based on the configuration.
 *
 * Parameters:
 * IN      : uint32 arg_DeviceNumber,
 *         : uint32 arg_portMask,
 *         : uint32 s_DuplexModeValue
 * INOUT
 * OUT
 *
 * Returns: void
 */
void basic_duplex_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_portMask,
                                 uint32 s_DuplexModeValue
                               );

/**
 * TYPENAME: mac_change_reset
 *
 * DESCRIPTION:
 *     This function enables the mac reset for specific specified port.
 *     This function is called from the change configuraiton fn".
 *
 * Parameters:
 * IN      : uint32 arg_DeviceNumber,
 *         : uint32 arg_portMask
 * INOUT
 * OUT
 *
 * Returns: void
 */
void mac_change_reset(
                      uint32 arg_DeviceNumber,
                      uint32 arg_portMask
                     );


/**
 * TYPENAME: basic_interface_mode
 *
 * DESCRIPTION:
 *     This function writes the interface mode register, and is called from
 *     the configuration function.
 *
 * Parameters:
 * IN     : uint32 arg_DeviceNumber,
 *        : uint32 arg_PortMask,
 *        : uint32 arg_IFMode
 * INOUT
 * OUT
 *
 * Returns:void
 */
void basic_interface_mode(
                          uint32 arg_DeviceNumber,
                          uint32 arg_PortMask,
                          uint32 arg_IFMode
                         );


/**
 * TYPENAME: do_port_enable
 *
 * DESCRIPTION:
 *     This function writes the Port enable register, and is called from
 *     the basic and change configuration function, when the respective 
 *     ports needs to be open..
 *
 * Parameters:
 * IN     : uint32 arg_DeviceNumber,
 *        : uint32 arg_PortMask,
 * INOUT
 * OUT
 *
 * Returns:void
 */
void do_port_enable(
                          uint32 arg_DeviceNumber,
                          uint32 arg_PortMask
                   );


/**
 * TYPENAME: basic_channel_mode
 *
 * DESCRIPTION:
 *     This function updates the Interface mode register.
 *
 * Parameters:
 * IN     : uint32 arg_DeviceNumber,
 *        : uint32 arg_PortMask,
 *        : uint32 arg_mode
 * INOUT
 * OUT
 *
 * Returns:
 */
void basic_channel_mode(
                        uint32 arg_DeviceNumber,
                        uint32 arg_PortMask,
                        uint32 arg_mode
                       );


/**
 * TYPENAME: gbe_mac_change_configuration
 *
 * DESCRIPTION:
 *    This function configures the Vallejo in the specified mode.  It does
 *    only as much configuration, that is required to work the Vallejo 
 *    properly.
 *
 * Parameters:
 * IN    : uint32 arg_DeviceNumber,
 *       : uint32 arg_PortMask,
 *       : uint32 arg_mode
 * INOUT
 * OUT
 *
 * Returns:void
 */
void gbe_mac_change_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_PortMask,
                                 uint32 arg_mode
                                  );


/**
 * Function definitions.
 */

/**
 * This function reads the 64bits value, two 32-bit registers, from the
 * register map at one shot.
 */
void gbe_mac_read_64_bits(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 *arg_pValue1,
                          uint32 *arg_pValue2
                         )
{
    uint32 s_RegOffset=0;

    if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
    {
        s_RegOffset =(uint32) (PT_LONE_REG_BASE_0 + (OFFSET_MULT*arg_Offset));       
    }
    else
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_1 + (OFFSET_MULT*arg_Offset));
    }

        /**
         * Save the current slow port structure configuration.
         */
        ixdp2400SPCfgSave(spCfgOld); 
        /**
         * Change the slow port configuration for the Pt Lone regmap access
         */
        ixdp2400SPCfgRestore(spCfgPtLone);
        REG_READ(s_RegOffset, arg_pValue1);
        REG_READ((s_RegOffset+4), arg_pValue2);

        ixdp2400SPCfgRestore(spCfgOld);
    
    return;
}

/**
 * This function reads the 32bits register from the register map at one shot.
 */
void gbe_mac_read_32_bits(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 *arg_pValue
                         )
{
    uint32 s_RegOffset=0;
    
    if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_0 + (OFFSET_MULT*arg_Offset));
    }
    else
    {
        s_RegOffset =(uint32)( PT_LONE_REG_BASE_1 + (OFFSET_MULT*arg_Offset));
    }

        /**
         * Save the current slow port structure configuration.
         */
        ixdp2400SPCfgSave(spCfgOld);
        /**
         * Change the slow port configuration for the Pt Lone regmap access 
         */
        ixdp2400SPCfgRestore(spCfgPtLone);
        REG_READ(s_RegOffset, arg_pValue);

        ixdp2400SPCfgRestore(spCfgOld);
        

#ifdef __READ_REG__
#if !defined(__LINUX_USER__)
    printk("\n R... [0x%03x]  [0x%08x]\n", arg_Offset,*arg_pValue);
#else
    printf("\n R... [0x%03x]  [0x%08x]\n", arg_Offset,*arg_pValue);
#endif //__LINUX_USER__
#endif //__READ_REG__
    return;
}

/**
 * This function writes the 64bits value, two 32-bit registers, to the
 * register map at one shot.
 */
void gbe_mac_write_64_bits(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 arg_pValue1,
                          uint32 arg_pValue2
                         )
{
    uint32 s_RegOffset=0;
    
    if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_0 + (OFFSET_MULT*arg_Offset));
    }
    else
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_1 + (OFFSET_MULT*arg_Offset));
    }

        /**
         * Save the current slow port structure configuration.
         */
        ixdp2400SPCfgSave(spCfgOld);
        /**
         * Change the slow port configuration for the Pt Lone regmap access 
         */
        ixdp2400SPCfgRestore(spCfgPtLone);
        REG_WRITE(s_RegOffset, arg_pValue1);
        REG_WRITE((s_RegOffset+4), arg_pValue2);

        ixdp2400SPCfgRestore(spCfgOld);
    return;
}

/**
 * This function writes the 32bits registers, to the register map at one shot.
 */
void gbe_mac_write_32_bits(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 arg_pValue
                         )
{
    uint32 s_RegOffset=0;

#ifdef __WRITE_REG__
#if !defined(__LINUX_USER__)
    printk("\n .W.. [0x%03x]  [0x%08x]\n", arg_Offset,arg_pValue);
#else
    printf("\n .W.. [0x%03x]  [0x%08x]\n", arg_Offset,arg_pValue);
#endif //__LINUX_USER__
#endif //__WRITE_REG__

    if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_0 + (OFFSET_MULT*arg_Offset));
    }
    else
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_1 + (OFFSET_MULT*arg_Offset));
    }

        /**
         * Save the current slow port structure configuration.
         */
        ixdp2400SPCfgSave(spCfgOld);
        /**
         * Change the slow port configuration for the Pt Lone regmap access
         */
        ixdp2400SPCfgRestore(spCfgPtLone);
        REG_WRITE(s_RegOffset, arg_pValue);

        ixdp2400SPCfgRestore(spCfgOld);

    return;
}


/**
 * This function reads the 32bits register, from the register map at one shot.
 * This function is basically called from the ISR.
 */
#if !defined(__LINUX_USER__)
void read_32_bits_from_isr(
                          uint32 arg_DeviceNumber,
                          uint32 arg_Offset,
                          uint32 *arg_pValue
                         )
{
    uint32 s_RegOffset=0;

    if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
    {
        s_RegOffset =(uint32)( PT_LONE_REG_BASE_0 + (OFFSET_MULT*arg_Offset));
    }
    else
    {
        s_RegOffset = (uint32)(PT_LONE_REG_BASE_1 + (OFFSET_MULT*arg_Offset));
    }

    /**
     * Save the current slow port structure configuration.
     */
    ixdp2400SPCfgSave(spCfgOld);
    /**
     * Change the slow port configuration for the Pt Lone regmap access
     */
    ixdp2400SPCfgRestore(spCfgPtLone);
    REG_READ(s_RegOffset, arg_pValue);

    ixdp2400SPCfgRestore(spCfgOld);


#ifdef __READ_REG_ISR__
#if !defined(__LINUX_USER__)
    printk("\n R.ISR.. [0x%03x]  [0x%08x]\n", arg_Offset,*arg_pValue);
#else
    printf("\n R.ISR.. [0x%03x]  [0x%08x]\n", arg_Offset,*arg_pValue);
#endif //__LINUX_USER__
#endif //__READ_REG_ISR__
    return;
}
#endif  /* __LINUX_USER__ */

/**
 * This function reads one 16-bit PHY register, through MDIO.
 */
gbe_mac_error gbe_mac_phy_read(
                               uint32 arg_DeviceNumber,
                               uint32 arg_PhyAddress,
                               uint32 arg_RegAddress,
                               uint32 *arg_ReadValue
                              )
{
    uint32 s_ReadValue;
    uint32 s_WriteValue = 0;
    uint32 s_Count = 0;
    uint32 s_RegAddress, s_PhyAddress;
    uint32 s_ReturnValue = SUCCESS;
    uint32 s_MdioDone = 1;

    /* Enable autoscan */
    gbe_mac_write_32_bits(arg_DeviceNumber,\
                         AS_PHY_ADDR, 0xF);

    /* Enable MDI to perform single rd/wr operation */
    gbe_mac_read_32_bits(arg_DeviceNumber,\
                         MDIO_CTL, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("R... MDIO_CTL 0x%08x\n",s_ReadValue);    
#else
    printf("R... MDIO_CTL 0x%08x\n",s_ReadValue);    
#endif //__LINUX_USER__
#endif // __READ_PHY__

    s_WriteValue = s_ReadValue | MDI_ENABLE;
    gbe_mac_write_32_bits(arg_DeviceNumber,MDIO_CTL, s_WriteValue);

    /* Read the MDIO Command and Address register */
    /* Check from the MDIO command register */
    gbe_mac_read_32_bits(arg_DeviceNumber,MDIO_CTL, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("R... MDIO_CTL 0x%08x\n",s_ReadValue);    
#else
    printf("R... MDIO_CTL 0x%08x\n",s_ReadValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__

	s_MdioDone = s_ReadValue >> 3;
	s_MdioDone &= 1;

	while(s_MdioDone != 0) {
		s_Count++;
#if defined(__LINUX_USER__)
        usleep(MDI_COMMAND_DELAY);        
#else
		udelay(10000);
#endif
		if (s_Count > 5) {
#if defined(__LINUX_USER__)
			printf("ERROR: COULD NOT PERFORM MDIO READ\n");
#else
			printk("ERROR: COULD NOT PERFORM MDIO READ\n");
#endif
			return ERROR_PREVIOUS_MDI_COMMAND_STILL_NOT_COMPLETED;
		}
		gbe_mac_read_32_bits(arg_DeviceNumber,MDIO_CTL, &s_ReadValue);
    		s_MdioDone = s_ReadValue >> 3;
    		s_MdioDone &= 1;
	}

        s_RegAddress = arg_RegAddress & PHY_REG_ADDRESS;
        s_PhyAddress = arg_PhyAddress & PHY_DEVICE_ADDRESS;
        s_PhyAddress = s_PhyAddress << PHY_DEVICE_SHIFT;

        s_WriteValue = s_PhyAddress | s_RegAddress;
        s_WriteValue = s_WriteValue | PHY_READ;

    /* Write to MDIO Command */
	gbe_mac_write_32_bits(arg_DeviceNumber,MDIO_SNGL_CMD, s_WriteValue);

	/* Read the signal command register to check for the command completion */
	gbe_mac_read_32_bits(arg_DeviceNumber,MDIO_SNGL_CMD, &s_ReadValue);

#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk("R... MDIO_SGNL_CMD 0x%08x\n",s_ReadValue);
#else
        printf("R... MDIO_SGNL_CMD 0x%08x\n",s_ReadValue);    
#endif //__LINUX_USER__
#endif // __READ_PHY__

        s_ReadValue = s_ReadValue >> MDIO_CMD_STATUS;
        s_MdioDone = s_ReadValue & DEFAULT_SET;

		s_Count=0;
        while(s_MdioDone == DEFAULT_SET) {
            s_Count++;
#if defined(__LINUX_USER__)
            usleep(MDI_COMMAND_DELAY);    
#else
           udelay(1000);
#endif
            if(s_Count > 5) {
                MESSAGE("ERROR_COULD_NOT_PERFORM_MDIO_READ\n",0,0,0,0,0,0);
                s_ReturnValue = ERROR_COULD_NOT_PERFORM_MDIO_READ;
                break;
            }
            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                 MDIO_SNGL_CMD, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
            printk("R... MDIO_SGNL_CMD 0x%08x\n",s_ReadValue);    
#else
            printf("R... MDIO_SGNL_CMD 0x%08x\n",s_ReadValue);    
#endif //__LINUX_USER__
#endif // __READ_PHY__
            s_ReadValue = s_ReadValue >> MDIO_CMD_STATUS;
            s_MdioDone = s_ReadValue & DEFAULT_SET;
	} /* while */
	if(s_Count < 6) {/* MDIO Read operation completes */
            gbe_mac_read_32_bits(arg_DeviceNumber, MDIO_SINGLE_RW_DATA,\
                                 &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("R... MDIO_SINGLE_RW_DATA 0x%08x\n",s_ReadValue);    
#else
    printf("R... MDIO_SINGLE_RW_DATA 0x%08x\n",s_ReadValue);    
#endif //__LINUX_USER__
#endif // __READ_PHY__
            s_ReadValue = s_ReadValue >> SHIFT_WORD;
            *arg_ReadValue = s_ReadValue;
	}/* end-if MDIO Completes */

    return s_ReturnValue;
}


/**
 * This function writes one 16-bit PHY register, through MDIO.
 */
gbe_mac_error gbe_mac_phy_write(
                               uint32 arg_DeviceNumber,
                               uint32 arg_PhyAddress,
                               uint32 arg_RegAddress,
                               uint32 arg_WriteValue
                              )
{
    uint32 s_ReadValue;
    uint32 s_WriteValue = 0;
    uint32 s_Count = 0;
    uint32 s_RegAddress, s_PhyAddress;
    uint32 s_ReturnValue = SUCCESS;
    uint32 s_MdioDone = 1;

#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("PHY_REG_WRITE PORT[%d] Add[0x%02x] 0x%04x\n",arg_PhyAddress,arg_RegAddress,arg_WriteValue);
#else
    printf("PHY_REG_WRITE PORT[%d] Add[0x%02x] 0x%04x\n",arg_PhyAddress,arg_RegAddress,arg_WriteValue);
#endif //__LINUX_USER__
#endif

    /* Enable autoscan */
    gbe_mac_write_32_bits(arg_DeviceNumber,\
                         AS_PHY_ADDR, 0xF);

    /* Enable MDI to perform single rd/wr operation */
    gbe_mac_read_32_bits(arg_DeviceNumber,\
                         MDIO_CTL, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("R... MDIO_CTL 0x%08x\n",s_ReadValue);
#else
    printf("R... MDIO_CTL 0x%08x\n",s_ReadValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__

    s_WriteValue = s_ReadValue | MDI_ENABLE;
    gbe_mac_write_32_bits(arg_DeviceNumber,\
                          MDIO_CTL, s_WriteValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk(".W.. MDIO_CTL 0x%08x\n",s_WriteValue);
#else
    printf(".W.. MDIO_CTL 0x%08x\n",s_WriteValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__

    /* Read the MDIO Command and Address register */
    /* Check from the MDIO command register */
    gbe_mac_read_32_bits(arg_DeviceNumber,\
                         MDIO_CTL, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
    printk("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#else
    printf("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__

    s_MdioDone = s_ReadValue >> 3;
    s_MdioDone &= 1;

	while(s_MdioDone != 0) {
		s_Count++;
#if defined(__LINUX_USER__)
        usleep(MDI_COMMAND_DELAY);        
#else
		udelay(10000);
#endif
		if (s_Count > 5) {
#if defined(__LINUX_USER__)
			printf("ERROR: COULD NOT PERFORM MDIO WRITE\n");
#else
			printk("ERROR: COULD NOT PERFORM MDIO WRITE\n");
#endif
			return ERROR_PREVIOUS_MDI_COMMAND_STILL_NOT_COMPLETED;
		}
		gbe_mac_read_32_bits(arg_DeviceNumber,MDIO_CTL, &s_ReadValue);
    		s_MdioDone = s_ReadValue >> 3;
    		s_MdioDone &= 1;
	}

#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk("MDIO READY\n");
#else
        printf("MDIO READY\n");
#endif //__LINUX_USER__
#endif // __READ_PHY__
		s_Count = 0;
        s_RegAddress = arg_RegAddress & PHY_REG_ADDRESS;
        s_PhyAddress = arg_PhyAddress & PHY_DEVICE_ADDRESS;
        s_PhyAddress = s_PhyAddress << PHY_DEVICE_SHIFT;

        s_WriteValue = s_PhyAddress | s_RegAddress;
        s_WriteValue = s_WriteValue | PHY_WRITE;

#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk("WRITE VALUE :0x%x\n",arg_WriteValue);
#else
        printf("WRITE VALUE :0x%x\n",arg_WriteValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__
        /* write value to be programmed in PHY */
        gbe_mac_write_32_bits(arg_DeviceNumber,\
                              MDIO_SINGLE_RW_DATA,\
                              arg_WriteValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk(".W.. MDIO_SINGLE_RW_DATA 0x%08x\n",arg_WriteValue);
#else
        printf(".W.. MDIO_SINGLE_RW_DATA 0x%08x\n",arg_WriteValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__
        /* write command to MDIO */
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk("WRITE VALUE :0x%x\n",s_WriteValue);
#else
        printf("WRITE VALUE :0x%x\n",s_WriteValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__
        gbe_mac_write_32_bits(arg_DeviceNumber,\
                              MDIO_SNGL_CMD, s_WriteValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk(".W.. MDIO_SNGLCMD 0x%08x\n",s_WriteValue);
#else
        printf(".W.. MDIO_SNGLCMD 0x%08x\n",s_WriteValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__

        /*Read the signal comand register to check for the command completion */
        gbe_mac_read_32_bits(arg_DeviceNumber,MDIO_SNGL_CMD, &s_ReadValue);

#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
        printk("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#else
        printf("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__
        s_ReadValue = s_ReadValue >> MDIO_CMD_STATUS;
        s_MdioDone = s_ReadValue & DEFAULT_SET;
         
        while(s_MdioDone == DEFAULT_SET) {
            s_Count++;
#if defined(__LINUX_USER__)
            usleep(MDI_COMMAND_DELAY);         
#else
           udelay(1000);
#endif
            if(s_Count > 5) {
                MESSAGE("ERROR_COULD_NOT_PERFORM_MDIO_READ\n",0,0,0,0,0,0);
#if !defined(__LINUX_USER__)
                printk("ERROR_COULD_NOT_PERFORM_MDIO_READ\n");
#else
                printf("ERROR_COULD_NOT_PERFORM_MDIO_READ\n");
#endif //__LINUX_USER__
                s_ReturnValue = ERROR_COULD_NOT_PERFORM_MDIO_READ;
                break;
            }
            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                 MDIO_SNGL_CMD, &s_ReadValue);
#ifdef __READ_PHY__
#if !defined(__LINUX_USER__)
            printk("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#else
            printf("R... MDIO_SNGL_CMD 0x%08x\n",s_ReadValue);
#endif //__LINUX_USER__
#endif // __READ_PHY__
            s_ReadValue = s_ReadValue >> MDIO_CMD_STATUS;
            s_MdioDone = s_ReadValue & DEFAULT_SET;

	}/* end-if MDIO Completes */

    return s_ReturnValue;
}

/**
 * This function is called by the GbEMAC_Ioctl API, to parse the ioctl command,
 * and call the appropriate IOCTL handler.
 */
gbe_mac_error gbe_mac_config_handler(
                                     uint32 arg_DeviceNumber,
                                     gbe_mac_IOCTL_CMD arg_Cmd, 
                                     void *arg_pIoctlStruct
                                    )
{
    uint32 s_IoctlType=0;
    uint32 s_RetValue=0;

    MESSAGE("Config Handler function gbe_mac_config_handler is invoked\n",0,0,0,0,0,0);
    MESSAGE("Device number is %d\n",arg_DeviceNumber,0,0,0,0,0);

    /**
     * AND the arg_Cmd with the GET_IOCTL, ie 0xE300, which results same in case of GET 
     * type ioctl, ans changes to 0xE200 in case of SET type IOCTLs.
     */
    s_IoctlType = arg_Cmd & GET_IOCTL;
    MESSAGE("IOCTL Command value is %x\n",arg_Cmd,0,0,0,0,0);

    /* Initializes the return value to SUCCESS */
    s_RetValue = SUCCESS;

    switch(s_IoctlType)
    {
        case GET_IOCTL:
            /* ioctl is of GET type */
            MESSAGE("Command is GET ioctl, and GET IOCTL handler function is called\n",0,0,0,0,0,0);
            MESSAGE("called ioctl command value is %x\n",arg_Cmd,0,0,0,0,0);
            s_RetValue = gbe_mac_get_config_handler(arg_DeviceNumber,arg_Cmd,arg_pIoctlStruct);
            break;

        case SET_IOCTL:
            /* ioctl is of SET type */
            MESSAGE("Command is SET ioctl, and SET IOCTL handler function is called\n",0,0,0,0,0,0);
            MESSAGE("called ioctl command value is %x\n",arg_Cmd,0,0,0,0,0);
            s_RetValue = gbe_mac_set_config_handler(arg_DeviceNumber,arg_Cmd,arg_pIoctlStruct);
            break;

        default:
            /* Invalid ioctl command */
            MESSAGE("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
            s_RetValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
            break;
    };/* end-switch(s_IoctlType) */
    MESSAGE("Returning from the Config Handler Function gbe_mac_config_handler.\n",0,0,0,0,0,0);
    return s_RetValue;
}


/**
 * This function is called by the GbEMAC_DeviceStart() to perform the basic
 * Vallejo configurstion, while opening the device.
 */
gbe_mac_error gbe_mac_config(
                             uint32 arg_DeviceNumber,
                             uint32 arg_PortMask,
                             short arg_mode
                            )
{

   uint32 s_Count;
   uint32 s_ConfigFlag = TRUE,s_ReturnValue;
   uint32 s_ResourceCount = 0, s_ReadValue;
   uint32 s_PllTune1,s_PllTune2,s_PllTune3,s_PllLock;

#ifdef __LINUX__
 #ifdef __SIT_TEST__
   int SPREG[5];
 #endif //__SIT_TEST__
#endif //__LINUX__

   MESSAGE("\nGbEMAC_DeviceStart ->gbe_mac_config Function is invoked for configuration\n",0,0,0,0,0,0);
   MESSAGE("              Device %d\n",arg_DeviceNumber,0,0,0,0,0);
 
   /* Initialize the return value to SUCCESS */
   s_ReturnValue = SUCCESS; 

    g_PerPortOffset[0] = MAC_CTL_REG_PORT0;
    g_PerPortOffset[1] = MAC_CTL_REG_PORT1;
    g_PerPortOffset[2] = MAC_CTL_REG_PORT2;
    g_PerPortOffset[3] = MAC_CTL_REG_PORT3;

   /**
    * Take the Mutex Semaphore for doing Inititial Configuration.
    */
#if defined(__LINUX_USER__)
    /**
     * Introduce the 10mSec delay to ensure that all the PLL has been locked.
     */

    MESSAGE(" Trying to grab the Mutex Semaphore g_MutexSemaphore\n",0,0,0,0,0,0);
/*    MESSAGE("SHARED MEMORY SEM ID= %d ",g_pGbeMacSharedData->g_ShmMutexSemaphore,0,0,0,0,0);*/
    MESSAGE("Semaphore ID =%d\n",(g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_MutexSemaphore)\
             ,0,0,0,0,0);
    if(gbe_mac_device_lock(g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_MutexSemaphore) ==1 )
    {
        /* Grab the Mutex Semaphore */
        s_ResourceCount++;

        MESSAGE("\tTake the Mutex Semaphore and initial requisite configuration starts\n",0,0,0,0,0,0);
        if(g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_MacStatus != INITIALIZED)
        /* Device is not initialized earlier, so initialization starts */
#else
    if(s_ReturnValue == SUCCESS)                   /* For Linux kernel mode,consider succes here */
    MESSAGE("NO need of semaphore in Kernel Mode driver\n",0,0,0,0,0,0);
#endif
        {
            /**
             * Check for the SPI-3 Transmit PLL Lock Status registers. 
             */


            MESSAGE("\tCheck for the SPI-3 Transmit PLL Lock Status registers\n",0,0,0,0,0,0);
            gbe_mac_read_32_bits(arg_DeviceNumber,PLL1_TUNE_REG,&s_PllTune1);
            gbe_mac_read_32_bits(arg_DeviceNumber,PLL2_TUNE_REG,&s_PllTune2);
            gbe_mac_read_32_bits(arg_DeviceNumber,PLL3_TUNE_REG,&s_PllTune3);

            if((s_PllTune1 == PLL1_AFTER_RESET) &&
               (s_PllTune2 == PLL2_AFTER_RESET) &&
               (s_PllTune3 == PLL3_AFTER_RESET))
            { /* SPI-3 Transmit lock has been achieved. */
                /**
                 * Check whether PLL lock has been achieved?
                 */
                MESSAGE("\tCheck whether PLL lock has been achieved\n",0,0,0,0,0,0);
                gbe_mac_read_32_bits(arg_DeviceNumber,SERDES_PLL_LOCK_STATUS,&s_PllLock);
                if(s_PllLock == 0x3)
                { /* PLL Lock has been achieved. */
                    /** 
                     * Perform Configuration sequence, if no error occurs earlier.
                     */
                    MESSAGE("Configuring the MAC Registers ...\n ",0,0,0,0,0,0);

                    gbe_mac_write_32_bits(arg_DeviceNumber,0x500,0x0);

                    gbe_mac_basic_configuration(arg_DeviceNumber,arg_PortMask,arg_mode);

                    MESSAGE(" -Configured\n",0,0,0,0,0,0);
                }
                else
                { /* PLL lock has failed */
                    s_ConfigFlag = FALSE;
                    MESSAGE("PLL_TRANSMIT_LOCK_STATUS_FAILED\n",0,0,0,0,0,0);
                    s_ReturnValue = PLL_TRANSMIT_LOCK_STATUS_FAILED;
                } /* end-if PLL lock */
            }
            else
            { /* SPI-3 Transmit Lock failed */
                s_ConfigFlag = FALSE;
                MESSAGE("SPI3_TX_LOCK_FAILED\n",0,0,0,0,0,0);
                s_ReturnValue = SPI3_TX_LOCK_FAILED;
            } /* end-if SPI-3 Transmit lock */ 
       }/* end-if device is not initialized earlier */
#if defined(__LINUX_USER__)
       else
        { /* Device is already initialized */
            s_ConfigFlag = FALSE;
            MESSAGE("ERROR_DEVICE_ALREADY_INITIALIZED\n",0,0,0,0,0,0);
            s_ReturnValue = ERROR_DEVICE_ALREADY_INITIALIZED;
        } /* end-if gbeMacGlobalData[arg_DeviceNumber].g_MacStatus */
#endif
       /**
         * Wait for 1uSec for the clock to stabalize
        */
       MESSAGE("Introduce delay in Linux\n",0,0,0,0,0,0);
#if defined(__LINUX_USER__)
       usleep(WAIT_CLK_STABLE);
#else
       udelay(WAIT_CLK_STABLE);                
#endif   

#if !defined(__LINUX_USER__)
       MESSAGE("Create the Process which updates the RMON Statistics\n",0,0,0,0,0,0);
       MESSAGE(" timer Initialized in init_module - \n",0,0,0,0,0,0);

       if(s_ConfigFlag)
       { /* if Configuration has been completed successfully */
           if(arg_DeviceNumber == PTLONE_ON_MEDIA_IF)
           {
               /* Set the Timer for Pt Lone card on the MEDIA INTERFACE */
               MESSAGE("Setting the Timer for Pt Lone card on the MEDIA INTERFACE\n",0,0,0,0,0,0);
               init_timer(&s_RmonStatTimer_MI);
               s_RmonStatTimer_MI.function = gbe_mac_update_rmon_MI;
               s_RmonStatTimer_MI.data = 0;
               s_RmonStatTimer_MI.expires = jiffies + (28*HZ); /* 28 second */
               add_timer(&s_RmonStatTimer_MI);
           }
           else if(arg_DeviceNumber == PTLONE_ON_SF_IF)
                {
                    /* SAet the Timer for the PtLone Card on the Switch Fabric Interface */
                    MESSAGE("Setting the Timer for PtLone Card on the Switch Fabric Interface \n",0,0,0,0,0,0);
                    init_timer(&s_RmonStatTimer_SF);
                    s_RmonStatTimer_SF.function = gbe_mac_update_rmon_SF;
                    s_RmonStatTimer_SF.data = 0;
                    s_RmonStatTimer_SF.expires = jiffies + (28*HZ); /* 28 second */
                    add_timer(&s_RmonStatTimer_SF);
                }                    
       }
#else
       /* Timer Mechanism for the User Mode Linux Driver */
#endif
   
        /**
         * Enable all the PHY interrupts 
         */
        gbe_mac_phy_write(arg_DeviceNumber,0x0, 0x19, 0xB400);
        gbe_mac_phy_write(arg_DeviceNumber,0x1, 0x19, 0xB400);
        gbe_mac_phy_write(arg_DeviceNumber,0x2, 0x19, 0xB400);
        gbe_mac_phy_write(arg_DeviceNumber,0x3, 0x19, 0xB400);
 
        gbe_mac_phy_read(arg_DeviceNumber,0x0, 0x1A,&s_ReadValue);
            s_PhyIntReadValue[arg_DeviceNumber][0] = s_ReadValue;
        gbe_mac_phy_read(arg_DeviceNumber,0x1, 0x1A,&s_ReadValue);
            s_PhyIntReadValue[arg_DeviceNumber][1] = s_ReadValue;
        gbe_mac_phy_read(arg_DeviceNumber,0x2, 0x1A,&s_ReadValue);
            s_PhyIntReadValue[arg_DeviceNumber][2] = s_ReadValue;
        gbe_mac_phy_read(arg_DeviceNumber,0x3, 0x1A,&s_ReadValue);
            s_PhyIntReadValue[arg_DeviceNumber][3] = s_ReadValue;
        


       if(s_ConfigFlag)
       { /* The complete configuration has done successfully */
           gbe_mac_write_32_bits(arg_DeviceNumber,PORT_ENABLE,arg_PortMask);
           for(s_Count=0; s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
           { /* Set the status of each port to OPENED */
               if((arg_PortMask & HIGH_BIT) == HIGH_BIT)
               {
#if defined(__LINUX_USER__)
                   g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = OPENED;
                   g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_PortUsageCount[s_Count]++;
#else
                   gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = OPENED;
#endif
                   MESSAGE("Port %d is OPENED\n",s_Count,0,0,0,0,0);
               }
               arg_PortMask = arg_PortMask >> HIGH_BIT;
           }       
           /* Set the Mac Status as Initialized */ 
#if defined(__LINUX_USER__)  /* for Linux User Mode driver */
           g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_MacStatus = INITIALIZED;
#endif
       }
#if defined(__LINUX_USER__)
    }
    else
    { /* Could not grab the Mutex Semaphore in estimated time due to time out */
        s_ConfigFlag = FALSE;
        MESSAGE("ERROR_SEMAPHORE_NOT_GRABED\n",0,0,0,0,0,0);
        s_ReturnValue = ERROR_SEMAPHORE_NOT_GRABED; /* SEMAPHORE CUOLD NOT BE GRABED */
    } /* end-if sem grab */
#endif

    if(s_ResourceCount == 1)
    {
        /**
         * Release the Mutex Semaphore.
         */
#if defined(__LINUX_USER__)
           gbe_mac_device_unlock(g_pGbeMacSharedData->gbeMacGlobalData[arg_DeviceNumber].g_MutexSemaphore);
#else
        MESSAGE("No need of semaphore in Linux Kernel Mode driver\n",0,0,0,0,0,0);
#endif                                                                          /* Releasing Semaphore */
        MESSAGE("Config Handler has released the semaphore\n",0,0,0,0,0,0);
    }

    if(s_ConfigFlag)
    {
        MESSAGE("Basic Configuration is performed successfully\n",0,0,0,0,0,0);
        MESSAGE("GbEMAC_DeviceStart ->gbe_mac_config Function completes the requisite configuration\n",0,0,0,0,0,0);
    }
    else
    {
        MESSAGE("Basic Configuration is FAILED\n",0,0,0,0,0,0);
        MESSAGE("GbEMAC_DeviceStart ->gbe_mac_config Function could not complete the requisite configuration\n",0,0,0,0,0,0);
    }
    return s_ReturnValue;
}


/**
 * This function is used to change the basic configuration from the IOCTL command.
 * it is called from the gbe_mac_set_config_handler() routine.
 */
gbe_mac_error gbe_mac_reconfig_handler(
                                       uint32 arg_DeviceNumber,
                                       gbe_mac_IOCTL_CMD arg_Cmd,
                                       void *arg_pIoctlStruct
                                      )
{
    uint32 s_GetStatus; 
    uint32 s_PortNumber;
    uint32 s_PortMaskHigh, s_PortMaskLow;
    gbe_mac_ioctl_ptr *p_IoctlPtr;
    uint32 s_MacSoftReadValue,s_TxFifoResetReadValue,s_PortEnableReadValue,s_ClkIfModeReadValue;
    uint32 s_WriteValue, s_ReturnValue;
    uint32 s_ReconfigFlag = TRUE;
    
    /* Initialize the s_returnValue to SUCCESS */
    s_ReturnValue = SUCCESS;
    
    p_IoctlPtr = (gbe_mac_ioctl_ptr *)arg_pIoctlStruct;
    s_PortNumber = p_IoctlPtr->portIndex;
    s_WriteValue = p_IoctlPtr->value;

    MESSAGE("For changing the present configuration gbe_mac_reconfig_handler Function is invoked\n",0,0,0,0,0,0);

        /**
         * When the chip is changed between Copper and Fiber mode, it should be done with MAC and 
         * Transmit FIFO under soft Reset and port enable de-asserted. Also it is required that 
         * there is no packet transfer in progress, as they will be dropped then. Follow these steps
         * for the same
         *    -Apply soft reset to MAC and Transmit FIFO blocks
         *    -De-assert the port enable
         *    -Reset the clock mode change enable bit
         *    -Change the mode
         *    -Set clock mode change enable bit
         *    -Wait for about 1us
         *    -Release Soft reset
         *    -Assert the port enable
         *    -Wait for Auto-negotiation completion in case of SERDES mode selection
        */
        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0xf);

        s_GetStatus = HIGH_BIT << s_PortNumber;
        s_PortMaskHigh = s_GetStatus;
        s_PortMaskLow = (PORT_MASK ^ s_GetStatus) & PORT_MASK;
        MESSAGE("RECONFIG_HANDLER: s_PortMaskHigh 0x%x s_PortMaskLow 0x%x\n",\
                 s_PortMaskHigh, s_PortMaskLow,0,0,0,0);

        gbe_mac_read_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,&s_MacSoftReadValue);
        s_MacSoftReadValue &= PORT_MASK;
        s_MacSoftReadValue |= s_PortMaskHigh;
        gbe_mac_write_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,s_MacSoftReadValue);

        gbe_mac_read_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,&s_TxFifoResetReadValue);
        s_TxFifoResetReadValue &= PORT_MASK;
        s_TxFifoResetReadValue |= s_PortMaskHigh;
        gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,s_TxFifoResetReadValue);        
        MESSAGE("RECONFIG_HANDLER: TXFIFO_PORT_RESET\n",0,0,0,0,0,0);

        gbe_mac_read_32_bits(arg_DeviceNumber,PORT_ENABLE,&s_PortEnableReadValue);
        s_PortEnableReadValue &= PORT_MASK;
        s_PortEnableReadValue &= s_PortMaskLow;
        gbe_mac_write_32_bits(arg_DeviceNumber,PORT_ENABLE,s_PortEnableReadValue);
        MESSAGE("RECONFIG_HANDLER: PORT_ENABLE\n",0,0,0,0,0,0);

        gbe_mac_read_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,&s_ClkIfModeReadValue);  
        s_ClkIfModeReadValue &= PORT_MASK;
        s_ClkIfModeReadValue &= s_PortMaskLow;
        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,s_ClkIfModeReadValue);
        MESSAGE("RECONFIG_HANDLER: SERDES_CLKMODE_REG\n",0,0,0,0,0,0);

        switch(arg_Cmd)
        {
            case SET_DUPLEX_MODE:
                MESSAGE("RECONFIG-DUPLEX-[0x%x]\n",s_WriteValue,0,0,0,0,0);
#if defined __RECONFIG_TEST__
#if !defined(__LINUX_USER__)
                printk("RECONFIG-DUPLEX-[0x%x]\n",s_WriteValue);
#else
                printf("RECONFIG-DUPLEX-[0x%x]\n",s_WriteValue);
#endif //__LINUX_USER__
#endif /* __RECONFIG_TEST__ */
                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                      (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),s_WriteValue);
                break;             

            case SET_INTERFACE_MODE:
                MESSAGE("RECONFIG-INTERFACE-[0x%x]\n",s_WriteValue,0,0,0,0,0);
#if defined __RECONFIG_TEST__
#if !defined(__LINUX_USER__)
                printk("RECONFIG-INTERFACE-[0x%x]\n",s_WriteValue);
#else
                printf("RECONFIG-INTERFACE-[0x%x]\n",s_WriteValue);
#endif //__LINUX_USER__
#endif /* __RECONFIG_TEST__ */
                gbe_mac_write_32_bits(arg_DeviceNumber,INTERFACE_MODE,s_WriteValue);
                break;

            case SET_MAC_IF_MODE:
                MESSAGE("RECONFIG-IF_MODE-[0x%x]\n",s_WriteValue,0,0,0,0,0);
#if defined __RECONFIG_TEST__
#if !defined(__LINUX_USER__)
                printk("RECONFIG-IF_MODE-[0x%x]\n",s_WriteValue);
#else
                printf("RECONFIG-IF_MODE-[0x%x]\n",s_WriteValue);
#endif //__LINUX_USER__
#endif /* __RECONFIG_TEST__ */
                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                      (g_PerPortOffset[s_PortNumber]+MAC_IF_MODE),s_WriteValue);
                break;

            case SET_SPI3_TX_CONFIG:
                MESSAGE("RECONFIG-SPI3-TXCONFIG-[0x%x]\n",s_WriteValue,0,0,0,0,0);
#if defined __RECONFIG_TEST__
#if !defined(__LINUX_USER__)
                printk("RECONFIG-SPI3-TXCONFIG-[0x%x]\n",s_WriteValue);
#else
                printf("RECONFIG-SPI3-TXCONFIG-[0x%x]\n",s_WriteValue);
#endif //__LINUX_USER__
#endif /* __RECONFIG_TEST__ */
                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                      SPI3_TX_GLOBAL_CONFIG,s_WriteValue);
                break;
    
            default:
                MESSAGE("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
                s_ReturnValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
                break;
        }; /* end-switch(arg_Cmd) */
 
        s_ClkIfModeReadValue |= s_PortMaskHigh;
        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,s_ClkIfModeReadValue);   

#if defined(__LINUX_USER__)
        usleep(WAIT_CLK_STABLE);             
#else
        udelay(WAIT_CLK_STABLE);            
#endif /* Introduce delay to clock to be stable */

        s_MacSoftReadValue &= s_PortMaskLow;
        gbe_mac_write_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,s_MacSoftReadValue);

        s_TxFifoResetReadValue &= s_PortMaskLow;
        gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,s_TxFifoResetReadValue);

        s_PortEnableReadValue |= s_PortMaskHigh;
        gbe_mac_write_32_bits(arg_DeviceNumber,PORT_ENABLE,s_PortEnableReadValue);  

        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0xf);

 
    if(s_ReconfigFlag)
    { /* Re-Configuration successfully completed. */
        /**
         * Release the Mutex Semaphore.
         */
        MESSAGE("Present configurationis changed to desired one in gbe_mac_reconfig_handler Function.\n",0,0,0,0,0,0);
    }
    else
    { /* Re-Configuration fails */
        MESSAGE("Could not reconfigure\n",0,0,0,0,0,0);
    } 
    return s_ReturnValue;
}

/**
 * The ISR will simply release the respective semaphore, on which the
 * callback dispatcher task is waiting.
 */

#if !defined(__LINUX_USER__)
void gbe_mac_isr(
                 uint32 arg_DeviceNumber
                )
{
    uint32 readVal,s_PortMode;
    uint32 s_TempStatus;
    uint32 s_PortCount;
    readVal = 0;


/*    MESSAGE("ISR: Checking for GBIC interrupt status ..\n",0,0,0,0,0,0);*/

    /**
     *  Reading the GBIC Interrupt Status 
     */
    read_32_bits_from_isr(arg_DeviceNumber,GBIC_STATUS_REG,&readVal);
    if(gbic_status != readVal)
    {
        MESSAGE("ISR: GBIC : 0x%x READ_VAL 0x%x\n",gbic_status,readVal,0,0,0,0);
        gbic_status = readVal;
        if(s_PortMode < 15 )
        { 
            s_TempStatus = gbic_status >>20;
            s_TempStatus = s_TempStatus ^ 0x0F;
            printk("FB_INT: PORT STATUS [0x%x]\n",s_TempStatus);
        } 
    }
    
    /** 
     * Reading PHY interrupt status register to enable the Interrupt,
     * this value is stored in a global array for future use.
     */
    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
    {
        gbe_mac_phy_read(arg_DeviceNumber,s_PortCount,\
                         MDI_INT_STATUS_OFFSET,&s_PhyIntReadValue[arg_DeviceNumber][s_PortCount]);
    }

    kill_fasync (&i2s_async_queue, SIGIO, POLL_IN);

    return;
}    
#endif


#if !defined(__LINUX_USER__)
/**
 * This function repeatedly calls itself upfdate the RMON stats for the Media card 
 * on the Media interface.
 */
void gbe_mac_update_rmon_SF (
                             unsigned long arg_ptr
                            )
{
    uint32 s_PortNumber;
    uint32 s_ReturnValue;

    MESSAGE("Function gbe_mac_update_rmon_SF to UPDATE Switch Fabric RMON statistics \n",0,0,0,0,0,0);
    for(s_PortNumber = 0;s_PortNumber<PT_LONE_GBE_PORT_MAX;s_PortNumber++)
    { /* update the RMON statistics registers for each port */
        s_ReturnValue = gbe_mac_evaluate_rmon_stat(PTLONE_ON_SF_IF, s_PortNumber);
    } /* end-for loop */

    init_timer(&s_RmonStatTimer_SF);
    s_RmonStatTimer_SF.function = gbe_mac_update_rmon_SF;
    s_RmonStatTimer_SF.data = 0;
    s_RmonStatTimer_SF.expires = jiffies + (28 * HZ); /* 28 second */
    add_timer(&s_RmonStatTimer_SF);

    return;
}

/**
 * This function repeatedly calls itself upfdate the RMON stats for the Media card 
 * on the Switch fabric.
 */
void gbe_mac_update_rmon_MI (
                             unsigned long arg_ptr
                            )
{
    uint32 s_PortNumber;
    uint32 s_ReturnValue;

    MESSAGE("Function gbe_mac_update_rmon_MI to UPDATE Media Interface RMON statistics \n",0,0,0,0,0,0);
    for(s_PortNumber = 0;s_PortNumber<PT_LONE_GBE_PORT_MAX;s_PortNumber++)
    { /* update the RMON statistics registers for each port */
        s_ReturnValue = gbe_mac_evaluate_rmon_stat(PTLONE_ON_MEDIA_IF, s_PortNumber);
    } /* end-for loop */

    init_timer(&s_RmonStatTimer_MI);
    s_RmonStatTimer_MI.function = gbe_mac_update_rmon_MI;
    s_RmonStatTimer_MI.data = 0;
    s_RmonStatTimer_MI.expires = jiffies + (28 * HZ); /* 28 second */
    add_timer(&s_RmonStatTimer_MI);

    return;
}


/**
 * This function is called by the ISR when the CU interrupt occurs on the 
 * PtLone card, on the media interface.
 */
void gbe_mac_copper_int_stat_MI(
                                unsigned long arg_ptr
                               )
{
    uint32 s_DeviceNumber = 0;
    uint32 s_LinkStatus;
    uint32 s_PortCount;
    uint32 s_ReadValue=0, s_Temp=0;

    MESSAGE("INT MEDIA INTERFACE COPPER.....",0,0,0,0,0,0);

    s_LinkStatus = 0;
    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
    {
        s_ReadValue = s_PhyIntReadValue[s_DeviceNumber][s_PortCount];
        s_Temp = s_ReadValue & 0x3400;
        if(s_Temp != 0)
        {
            gbe_mac_phy_read(s_DeviceNumber,s_PortCount,\
                             MDI_LINK_STATUS,&s_ReadValue);
            MESSAGE("MDI_LINK_STATUS [0x%x] \n",s_ReadValue,0,0,0,0,0);
            if((s_ReadValue & 0x4) == 0x4)
            {
                printk("LINK UP .....PORT %d\n",s_PortCount);
            }
            else
            {
                printk("LINK DOWN .....PORT %d\n",s_PortCount);
            }
        }
    }
    s_LinkStatus = 0;
    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
    {
        s_Temp = 0;
        gbe_mac_phy_read(s_DeviceNumber,s_PortCount,\
                         MDI_LINK_STATUS,&s_ReadValue);
        
        if((s_ReadValue & 0x4) == 0x4)
        {
            s_Temp = 0x1 << s_PortCount;
        }
        s_LinkStatus = s_LinkStatus | s_Temp;
    }

    MESSAGE("s_LinkStatus [0x%x]\n\n",s_LinkStatus,0,0,0,0,0);
    s_LinkStatus = s_LinkStatus & PORT_MASK;

    printk("INT: CU: MI: LINK STATUS 0x%x\n",s_LinkStatus);

    return;
}

/**
 * This function is called by the ISR when the CU interrupt occurs on the 
 * PtLone card, on the Switch fabriv interface.
 */
void gbe_mac_copper_int_stat_SF(
                                unsigned long arg_ptr
                               )
{
    uint32 s_DeviceNumber = 1;
    uint32 s_LinkStatus;
    uint32 s_PortCount;
    uint32 s_ReadValue=0, s_Temp=0;

    MESSAGE("INT SWITCH FABRIC COPPER.....",0,0,0,0,0,0);

    s_LinkStatus = 0;
    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
    {
        s_ReadValue = s_PhyIntReadValue[s_DeviceNumber][s_PortCount];
        s_Temp = s_ReadValue & 0x3400;
        if(s_Temp != 0)
        {
            gbe_mac_phy_read(s_DeviceNumber,s_PortCount,\
                             MDI_LINK_STATUS,&s_ReadValue);
            MESSAGE("MDI_LINK_STATUS [0x%x] \n",s_ReadValue,0,0,0,0,0);
            if((s_ReadValue & 0x4) == 0x4)
            {
                printk("LINK UP .....PORT %d\n",s_PortCount);
            }
            else
            {
                printk("LINK DOWN .....PORT %d\n",s_PortCount);
            }
        }
    }
    s_LinkStatus = 0;
    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
    {
        s_Temp = 0;
        gbe_mac_phy_read(s_DeviceNumber,s_PortCount,\
                         MDI_LINK_STATUS,&s_ReadValue);
        
        if((s_ReadValue & 0x4) == 0x4)
        {
            s_Temp = 0x1 << s_PortCount;
        }
        s_LinkStatus = s_LinkStatus | s_Temp;
    }

    MESSAGE("s_LinkStatus [0x%x]\n\n",s_LinkStatus,0,0,0,0,0);
    s_LinkStatus = s_LinkStatus & PORT_MASK;

    printk("INT: CU: SF: LINK STATUS 0x%x\n",s_LinkStatus);

    return;
}

#endif /* !__LINUX_USER__ */


/**
 * This function reads the RMON registers and updates the locally maintained
 * counter.
 */
gbe_mac_error gbe_mac_evaluate_rmon_stat(
                                         uint32 arg_DeviceNumber,
                                         uint32 arg_PortNumber
                                        )
{
    uint32 s_ReturnValue; 
    uint32 s_PortNumber;
    uint32 s_ReadValue;
   
    s_ReturnValue = SUCCESS;
    s_PortNumber = arg_PortNumber;
    
    /**
     * MAC RX Statistics Registers
     */
    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_OCTETS_OK),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][0],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][1],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][0] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][1] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_OCTETS_BAD),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][2],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][3],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][2] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][3] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_UC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][4],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][5],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][4] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][5] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_MC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][6],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][7],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][6] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][7] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_BC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][8],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][9],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][8] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][9] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_64),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][10],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][11],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][10] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][11] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_65_127),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][12],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][13],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][12] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][13] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_128_255),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][14],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][15],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][14] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][15] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_256_511),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][16],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][17],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][16] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][17] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_512_1023),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][18],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][19],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][18] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][19] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_1024_1518),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][20],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][21],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][20] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][21] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PKTS_1519_MAX),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][22],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][23],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][22] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][23] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_FCS_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][24],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][25],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][24] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][25] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + VLAN_TAG),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][26],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][27],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][26] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][27] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_DATA_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][28],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][29],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][28] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][29] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_ALIGN_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][30],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][31],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][30] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][31] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_LONG_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][32],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][33],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][32] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][33] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_JABBER_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][34],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][35],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][34] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][35] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_PAUSE_MAC_CTL),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][36],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][37],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][36] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][37] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RX_UNKNOWN_CTL_FRAME),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][38],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][39],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][38] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][39] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + VLONG_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][40],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][41],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][40] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][41] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + RUNT_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][42],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][43],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][42] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][43] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + SHORT_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][44],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][45],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][44] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][45] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + CARRIER_EXTEND_ERROR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][46],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][47],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][46] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][47] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + SEQ_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][48],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][49],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][48] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][49] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + SYMBOL_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][50],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][51],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][50] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][51] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    /**
     * MAC TX Statistics Registers
     */
    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_OCTETS_OK),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][52],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][53],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][52] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][53] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_OCTETS_BAD),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][54],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][55],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][54] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][55] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_UC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][56],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][57],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][56] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][57] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_MC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][58],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][59],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][58] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][59] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_BC_PKTS),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][60],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][61],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][60] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][61] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_64),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][62],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][63],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][62] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][63] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_65_127),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][64],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][65],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][64] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][65] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_128_255),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][66],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][67],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][66] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][67] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_256_511),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][68],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][69],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][68] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][69] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_512_1023),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][70],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][71],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][70] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][71] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_1024_1518),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][72],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][73],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][72] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][73] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PKTS_1519_MAX),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][74],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][75],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][74] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][75] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_DEFERRED_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][76],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][77],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][76] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][77] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_TOTAL_COLLISION),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][78],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][79],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][78] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][79] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_SINGLE_COLLISION),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][80],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][81],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][80] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][81] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_MUL_COLLISION),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][82],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][83],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][82] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][83] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + LATE_COLLISION),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][84],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][85],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][84] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][85] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_EXCV_COLLISION),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][86],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][87],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][86] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][87] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_EXCV_DEFERRED_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][88],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][89],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][88] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][89] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_EXCV_LEN_DROP),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][90],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][91],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][90] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][91] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_UNDERRUN),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][92],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][93],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][92] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][93] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_VLAN_TAG),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][94],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][95],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][94] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][95] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_CRC_ERR),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][96],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][97],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][96] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][97] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + TX_PAUSE_FRAME),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][98],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][99],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][98] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][99] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    gbe_mac_read_32_bits(arg_DeviceNumber,(( 0x80 * s_PortNumber) + FC_COLLISION_SEND),&s_ReadValue);
    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][100],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][101],\
                                          s_ReadValue);
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][100] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][101] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

    return s_ReturnValue;
}

/**
 * his function adds the two 32-bit register value, and updates the 64-bit
 * counter
 */
gbe_mac_error gbe_mac_add_two_lword(
                                    uint32 arg_DeviceNumber,
                                    uint32 arg_Low,
                                    uint32 arg_High,
                                    uint32 arg_Value
                                   )
{
    uint32 s_ReturnValue=0;
    uint32 s_ResultLow = 0, s_ResultHigh = arg_High;
    uint32 s_LowValue=0, s_LowCarry=0;
    uint32 s_Value=0, s_ValueCarry=0;
    uint32 s_Add=0,s_AddCarry=0;
    uint32 temp=0,carry=0;

    s_ReturnValue = SUCCESS;

    s_LowValue = arg_Low & MAKE_AND;
    s_LowCarry = arg_Low >> SHIFT_RIGHT;

    s_Value = arg_Value & MAKE_AND;
    s_ValueCarry = arg_Value >> SHIFT_RIGHT;

    s_Add = s_LowValue + s_Value;

    s_AddCarry = s_Add >> SHIFT_RIGHT;
    s_Add = s_Add & MAKE_AND;

    temp = s_LowCarry + s_ValueCarry + s_AddCarry;

    s_ResultLow = temp & 0x1;

    s_ResultLow = s_ResultLow << SHIFT_RIGHT;
    s_ResultLow = s_ResultLow | s_Add;

    carry = temp >> 0x1;

    s_ResultHigh = s_ResultHigh + carry;

    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0] = s_ResultLow;
    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1] = s_ResultHigh;

    return s_ReturnValue;
}

#if defined __LINUX_USER__  
void ixdp2400SPCfgSave(unsigned int *SPREG)
{
//     MESSAGE("\nixdp2400SPCfgSave",0,0,0,0,0,0);
     REG_READ(( pSPCfgBase+  SP_CCR_OFF),&SPREG[0]); 
     REG_READ(( pSPCfgBase+  SP_WTC2_OFF),&SPREG[1]);
     REG_READ(( pSPCfgBase+  SP_RTC2_OFF),&SPREG[2]);
     REG_READ(( pSPCfgBase+  SP_PCR_OFF),&SPREG[3]);
     REG_READ(( pSPCfgBase+  SP_ADC_OFF),&SPREG[4]);
}
void ixdp2400SPCfgRestore(unsigned int *SPREG)
{
//    MESSAGE("\nixdp2400SPCfgRestore called",0,0,0,0,0,0);
    REG_WRITE(( pSPCfgBase+  SP_CCR_OFF), SPREG[0]);
    REG_WRITE(( pSPCfgBase+  SP_WTC2_OFF), SPREG[1]);
    REG_WRITE(( pSPCfgBase+  SP_RTC2_OFF), SPREG[2]);
    REG_WRITE(( pSPCfgBase+  SP_PCR_OFF), SPREG[3]);
    REG_WRITE(( pSPCfgBase+  SP_ADC_OFF), SPREG[4]);
}
#endif /*  #if defined __LINUX_USER__ */

#ifdef __LINUX__
void ixdp2400SPCfgSave(unsigned int *SPREG)
{
    /* MESSAGE("\nixdp2400SPCfgSave",0,0,0,0,0,0);*/
     REG_READ(( pSPCfgBase +  SP_CCR_OFF),&SPREG[0]); 
     REG_READ(( pSPCfgBase+  SP_WTC2_OFF),&SPREG[1]);
     REG_READ(( pSPCfgBase+  SP_RTC2_OFF),&SPREG[2]);
     REG_READ(( pSPCfgBase+  SP_PCR_OFF),&SPREG[3]);
     REG_READ(( pSPCfgBase+  SP_ADC_OFF),&SPREG[4]);
}
void ixdp2400SPCfgRestore(unsigned int *SPREG)
{
   /* MESSAGE("\nixdp2400SPCfgRestore called",0,0,0,0,0,0);*/
    REG_WRITE(( pSPCfgBase+  SP_CCR_OFF), SPREG[0]);
    REG_WRITE(( pSPCfgBase+  SP_WTC2_OFF), SPREG[1]);
    REG_WRITE(( pSPCfgBase+  SP_RTC2_OFF), SPREG[2]);
    REG_WRITE(( pSPCfgBase+  SP_PCR_OFF), SPREG[3]);
    REG_WRITE(( pSPCfgBase+  SP_ADC_OFF), SPREG[4]);
}
#endif //__LINUX__


/**
 * This function configures the PHY registers as per the specified
 * mode, passed by the application.
 */
void config_phy(
                uint32 arg_DeviceNumber, 
	 	uint32 speed, 
		uint32 duplex , 	
		uint32 arg_PortMask
	   )
{
    int s_PortCount =0;
    int s_Port;

    s_Port = arg_PortMask & 0x0F;
    for(s_PortCount=0; s_PortCount<4;s_PortCount++)
    {		
        if(((s_Port >> s_PortCount) & 0x01) == 0x00)
        {
            MESSAGE("SKIPing PORT [%d]\n",s_PortCount,0,0,0,0,0);
 	    continue;
        }

        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x17, 0x1300);	/* Enable RGMII mode*/

#ifdef __FORCE_PHY_CONFIG__ /* For forced PHY configuration. Forced PHY config is disabled for now. */
        switch (speed)
        {
   	    case 1000 :	
		MESSAGE("configuring 1000 \n",0,0,0,0,0,0);
		switch(duplex)
		{

		    case 1:
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x300);		/* set gigabit operation */
			/* CRC error fix  ----------
			
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 24, 0x3248); 
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 22, 0x18);  
			**********************/
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x16, 0x18);
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x18, 0x3240);
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x1240); 

#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x04]  [0x0000]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x09]  [0x0300]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x04]  [0x0000]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x09]  [0x0300]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
		}
		break;

	    case 100:
		MESSAGE("configuring 100 \n",0,0,0,0,0,0);		
	        switch(duplex)
       		{
   		    case 0 :
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x4, 0x80);    
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x400); 
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x1240); 
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x04]  [0x0080]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x04]  [0x0080]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
		    case 1 :
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x4, 0x180);    
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x400); 
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x1240);
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x04]  [0x0180]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x04]  [0x0180]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
       		}		
		break;
	    case 10:
		MESSAGE("configuring 10 \n",0,0,0,0,0,0);						
	        switch(duplex)
       		{
   		    case 0 :
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x4, 0x60);    
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x400); 
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x1240);
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x04]  [0x0060]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x04]  [0x0060]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
		    case 1 :
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x4, 0x40);    
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x400);   
			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x1240);
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x04]  [0x0040]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printk(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x04]  [0x0040]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x09]  [0x0400]\n", s_PortCount);
    printf(" PHY...[0x%x]  [0x00]  [0x1240]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
       		}		
		break;

	    default :
	 	MESSAGE("Bad speed option ... configuring 100 Mbps speed.\n",0,0,0,0,0,0);
		MESSAGE("configuring 100 \n",0,0,0,0,0,0);		
	        switch(duplex)
       		{
   		    case 0 :
		        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x2000);
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x00]  [0x2000]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x00]  [0x2000]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
			break;
   		    case 1 :
 			gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0, 0x2100);
#ifdef __CONFIG_PHY__
#if !defined(__LINUX_USER__)
    printk(" PHY...[0x%x]  [0x00]  [0x2100]\n", s_PortCount);
#else
    printf(" PHY...[0x%x]  [0x00]  [0x2100]\n", s_PortCount);
#endif //__LINUX_USER__
#endif //__CONFIG_PHY__
    			break;
       		}		
		break;
        }
#endif /* __FORCE_PHY_CONFIG__  forced configuration */

        if(speed == 1000)
                {
                        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x9, 0x0300);
                        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x16, 0x18);
                        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x18, 0x3240);

                }
        gbe_mac_phy_write(arg_DeviceNumber,s_PortCount, 0x0, 0x1240);
    }

    return;
}


/**
 * This function perfoerms the basic configuration, and configures the 
 * Vallejo in the specified mode, given by the user in the open API.
 */
void gbe_mac_basic_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_PortMask,
                                 uint32 arg_mode
                                )
{
    uint32 s_StationAddressLow=0, s_StationAddressHigh=0;
    short duplex = 0,speed = 0;
    uint32 s_WriteSpi3Value=0;
    uint32 s_rgmiiConf=0, s_DiverseConfig=0,mode = arg_mode;
    uint32 spi3_block_mode=0,spi3_parity_mode=0,spi3_parity_mask=0;
    uint32 pmask, ifmode;

    uint32 s_WriteSpi3RxValue=0,spi3_rx_parity_mask=0, s_Temp;

#if defined(__LINUX_USER__)    

    spi3_block_mode = mode & SPI3_BLOCK_MODE;
    if(spi3_block_mode == 0)
    { /* When 0, indicates that SPI-3 block operates in 32-bit MPHY mode */
        s_WriteSpi3Value = SPI3_MPHY_MODE;
        s_WriteSpi3RxValue = SPI3_RX_PORT_ENABLE; 
        MESSAGE("MPHY-",0,0,0,0,0,0);
    }
    else
    { /* When 1, indicates that the SPI-3 block operates in 4x8 mode. */
        s_WriteSpi3Value = SPI3_4x8_MODE;
        s_WriteSpi3RxValue = SPI3_RX_PORT_ENABLE; 
        MESSAGE("SPHY-",0,0,0,0,0,0);
    }
 
    spi3_parity_mode = mode & SPI3_PARITY_MODE;
    if(spi3_parity_mode == SPI3_PARITY_MODE) /* Even parity*/
    {
        if(arg_DeviceNumber == CARD_ON_MEDIA_IF)
        {
            spi3_parity_mask = arg_PortMask & 0x0F;
            spi3_parity_mask = spi3_parity_mask << SHIFT_NIBBLE;

            s_Temp = arg_PortMask & 0x0F;
            spi3_rx_parity_mask = (s_Temp << 12) & SPI3_RX_EVEN_PARITY_MASK;
        }
        else
        {
            spi3_parity_mask = arg_PortMask & 0xF0;
        }
        MESSAGE("EVEN-",0,0,0,0,0,0); 
    }
    else
    {
        MESSAGE("ODD-",0,0,0,0,0,0);
    }
    s_WriteSpi3Value = s_WriteSpi3Value | spi3_parity_mask;
    MESSAGE("SPI-3 write value = 0x%x",s_WriteSpi3Value,0,0,0,0,0);

    s_WriteSpi3RxValue = s_WriteSpi3RxValue | spi3_rx_parity_mask;
    MESSAGE("SPI-3 RX Config write value = 0x%x",s_WriteSpi3RxValue,0,0,0,0,0);

    MESSAGE("MODE ===== [0x%x]\n",mode,0,0,0,0,0);

    s_DiverseConfig = 0x114D;
    pmask = arg_PortMask;
    mode = mode & 0x07;
    switch(mode)
    {
	case 0 :   duplex = 1;
		   ifmode = 0;
		   speed = 1000;
                   s_DiverseConfig = 0x116D;
		   s_rgmiiConf = 0x3;
                   MESSAGE("FB-FD-1000\n",0,0,0,0,0,0);
		   break;
	
	case 1 :   duplex = 0;
		   ifmode = 1;
		   speed = 1000;
		   s_rgmiiConf = 0x6;
                   MESSAGE("CU-HD-1000\n",0,0,0,0,0,0);
		   break; 

	case 2 :   duplex = 1;
		   ifmode = 1;
		   speed = 1000;
		   s_rgmiiConf = 0x6;
                   MESSAGE("CU-FD-1000\n",0,0,0,0,0,0);
		   break; 

	case 3 :   duplex = 0;
		   ifmode = 1;
		   speed = 100;
		   s_rgmiiConf = 0x5;
                   MESSAGE("CU-HD-100\n",0,0,0,0,0,0);
		   break; 

	case 4 :   duplex = 1;
		   ifmode = 1;
		   speed = 100;
		   s_rgmiiConf = 0x5;
                   MESSAGE("CU-FD-100\n",0,0,0,0,0,0);
		   break; 

	case 5 :   duplex = 0;
		   ifmode = 1;
		   speed = 10;
		   s_rgmiiConf = 0x4;
                   MESSAGE("CU-HD-10\n",0,0,0,0,0,0);
		   break; 

	case 6 :   duplex = 1;
		   ifmode = 1;
		   speed = 10;
		   s_rgmiiConf = 0x4;
                   MESSAGE("CU-FD-10\n",0,0,0,0,0,0);
		   break; 

	default:   MESSAGE("Illegal configuration attempt ..\n",0,0,0,0,0,0);
		   MESSAGE("Default configuration ..\n",0,0,0,0,0,0);
	           return;
    }
#else /* __LINUX__ */
    /** 
     * By default, configure all the 4 ports of the Vallejo in Fiber mode,
     *  with SPHY4x8 Mode, and ODD parity.
     */
    mode = 0;
    arg_PortMask = 0xF;
    duplex = 1;
    speed = 1000;
    s_rgmiiConf = 3;
    s_DiverseConfig = 0x116D;
    s_WriteSpi3Value = 0x020AA0F;
#ifdef __READ_REG__
#if !defined(__LINUX_USER__)
    printk(" SPI3 WriteVal : [0x%8x]\n",s_WriteSpi3Value);
#else
    printf(" SPI3 WriteVal : [0x%8x]\n",s_WriteSpi3Value);
#endif //__LINUX_USER__
#endif

#endif /* __LINUX_USER__) */
    
	
#if defined(__LINUX_USER__)
    mac_change_reset(arg_DeviceNumber, arg_PortMask);
#else
    mac_reset(arg_DeviceNumber);
#endif

    if(mode != 0)
    {
        MESSAGE("Configuring the PHY ....",0,0,0,0,0,0);
        config_phy(arg_DeviceNumber, speed, duplex ,arg_PortMask);
        MESSAGE("Configured \n",0,0,0,0,0,0);
    }

    gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0x0);
    gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0x0);

    /**
     * Set Station Address for Port 0.
     */
    MESSAGE("\nConfiguring MAC Address ...",0,0,0,0,0,0);
    s_StationAddressLow = gbeMacAddress[arg_DeviceNumber].macAddressPort_0[0];
    s_StationAddressHigh = gbeMacAddress[arg_DeviceNumber].macAddressPort_0[1];
    gbe_mac_write_64_bits(arg_DeviceNumber,(MAC_CTL_REG_PORT0+STATION_ADDR_LOW),\
                                           s_StationAddressLow, s_StationAddressHigh);

    /**
     * Set station address for port 1.
     */
    s_StationAddressLow = gbeMacAddress[arg_DeviceNumber].macAddressPort_1[0];
    s_StationAddressHigh = gbeMacAddress[arg_DeviceNumber].macAddressPort_1[1];
    gbe_mac_write_64_bits(arg_DeviceNumber,(MAC_CTL_REG_PORT1+STATION_ADDR_LOW),\
    s_StationAddressLow, s_StationAddressHigh);

    /**
     * Set station address for port 2.
     */
    s_StationAddressLow = gbeMacAddress[arg_DeviceNumber].macAddressPort_2[0];
    s_StationAddressHigh = gbeMacAddress[arg_DeviceNumber].macAddressPort_2[1];
    gbe_mac_write_64_bits(arg_DeviceNumber,(MAC_CTL_REG_PORT2+STATION_ADDR_LOW),\
                          s_StationAddressLow, s_StationAddressHigh);

    /**
     * Set station address for port 3.
     */
    s_StationAddressLow = gbeMacAddress[arg_DeviceNumber].macAddressPort_3[0];
    s_StationAddressHigh = gbeMacAddress[arg_DeviceNumber].macAddressPort_3[1];
    gbe_mac_write_64_bits(arg_DeviceNumber,(MAC_CTL_REG_PORT3+STATION_ADDR_LOW),\
    			  s_StationAddressLow, s_StationAddressHigh);
    MESSAGE(" -Configured",0,0,0,0,0,0);

    /**
     * Set the various configurations likes CRC enable, AN enable, padding enable etc.
     */
    MESSAGE("\nConfiguring Diverse Config Registers...",0,0,0,0,0,0);
    basic_diverse_config(arg_DeviceNumber,arg_PortMask,s_DiverseConfig);
    MESSAGE(" -Configured",0,0,0,0,0,0);
			
    MESSAGE("\nConfiguring Other requisite Vallejo Registers, PLS WAIT...",0,0,0,0,0,0);
    /**
     * Enable packets for short packets, and set minimal number of bytes in each packets 
     * equals to 0x50 (for all the ports)
     */
    gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PAD_ENB,0x0000AAAA);

    /**
     *  Enable padding and CRC stripping
     */
    gbe_mac_write_32_bits(arg_DeviceNumber,RXFIFO_PAD_CRCSTRIP_ENB,0xF0);

    /**
     *  Enable Dropping of packets on error for all the 4 ports.
     *  Jumbo pkts are not dropped in any case.
     */
     gbe_mac_write_32_bits(arg_DeviceNumber,RXFIFO_DROP_ENABLE,0xF);
    			
    /**
     *  Set SPI-3 Transmit and global configuration register
     *   1). 4x8 Mode configuration
     *   2). TX Burst Size - 128 Bytes maximum burst size
     *   3). TX Parity Sense - ODD
     */
    /* set SPI-3 TX config Register */
    gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_TX_GLOBAL_CONFIG,s_WriteSpi3Value);
    /* set SPI-3 RX config Register */
    gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_RX_CONFIG,s_WriteSpi3RxValue);

    /** enable GBIC interrupts **/
    gbe_mac_write_32_bits(arg_DeviceNumber,GBIC_CTRL_REG,0x1FC00);		      
    gbe_mac_read_32_bits(arg_DeviceNumber,GBIC_STATUS_REG,&gbic_status);		      

    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0x0);
    MESSAGE("Configuring the Interface Mode ....",0,0,0,0,0,0);
    basic_interface_mode(arg_DeviceNumber,arg_PortMask, s_rgmiiConf);
    MESSAGE("Configured\n",0,0,0,0,0,0);

    MESSAGE("Configuring the Duplex Mode ....",0,0,0,0,0,0);
    basic_duplex_configuration(arg_DeviceNumber,arg_PortMask,duplex);
    MESSAGE("Configured\n",0,0,0,0,0,0);

#if defined(__LINUX_USER__)
    basic_channel_mode(arg_DeviceNumber,arg_PortMask,arg_mode);
#else /* Configuring all the four ports in Fiber*/
    gbe_mac_write_32_bits(arg_DeviceNumber,INTERFACE_MODE,0x0);
#endif

    /* Enable the port, by writing the Port Enable register */
    do_port_enable(arg_DeviceNumber,arg_PortMask);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0xf);

    mac_disable_reset(arg_DeviceNumber);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0x0);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0xf);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0x0);

#if defined(__PHY_TEST__)
    s_ReturnValue = gbe_mac_phy_write(arg_DeviceNumber,\
                                      0, 0x19, 0xA000);
    gbe_mac_phy_read(arg_DeviceNumber,\
                     0, 0x19, &s_ReadValue);
#if !defined(__LINUX_USER__)
    printk("\nGBE_MAC_PHY_READ [0x%x] [0x19] [0x%04x]\n",s_Count,s_ReadValue);
#else
    printf("\nGBE_MAC_PHY_READ [0x%x] [0x19] [0x%04x]\n",s_Count,s_ReadValue);
#endif //__LINUX_USER__
#endif /*__PHY_TEST__ */

    return;
}


/**
 * This function perfoerms the change configuration, and configures the 
 * Vallejo in the specified mode, given by the user in the change ioctl command.
 */
void gbe_mac_change_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_PortMask,
                                 uint32 arg_mode
                                  )
{
    short pmask = arg_PortMask;
    short ifmode = 0, duplex = 0,speed = 0;
    short spi3_block_mode=0, spi3_parity_mode=0, spi3_parity_mask = 0x00;
    uint32 s_WriteSpi3Value=0;
    uint32 readVal=0,s_WriteSpi3RxValue=0,spi3_rx_parity_mask=0, s_Temp;
    uint32 s_rgmiiConf=0, s_DiverseConfig=0,mode = arg_mode;


        spi3_block_mode = mode & SPI3_BLOCK_MODE;
        if(spi3_block_mode == 0)
        { /* When 0, indicates that SPI-3 block operates in 32-bit MPHY mode */
            s_WriteSpi3Value = SPI3_MPHY_MODE;
            s_WriteSpi3RxValue = SPI3_RX_PORT_ENABLE; 
            MESSAGE("MPHY-",0,0,0,0,0,0);
        }
        else
        { /* When 1, indicates that the SPI-3 block operates in 4x8 mode. */
            s_WriteSpi3Value = SPI3_4x8_MODE;
            s_WriteSpi3RxValue = SPI3_RX_PORT_ENABLE; 
            MESSAGE("SPHY-",0,0,0,0,0,0);
        }
 
        spi3_parity_mode = mode & SPI3_PARITY_MODE;
        gbe_mac_read_32_bits(arg_DeviceNumber,0x700,&readVal);
        if(spi3_parity_mode == SPI3_PARITY_MODE) /* Even parity*/
        {
            if(arg_DeviceNumber == CARD_ON_MEDIA_IF)
            {
                readVal = readVal & 0xF0;
                spi3_parity_mask = arg_PortMask & 0x0F;
                spi3_parity_mask = spi3_parity_mask << SHIFT_NIBBLE;
                spi3_parity_mask |= readVal;                

                /* Configuring the SPI3 RX config register */
                gbe_mac_read_32_bits(arg_DeviceNumber,0x701,&readVal);
                readVal = readVal & 0xFF80;
                s_Temp = arg_PortMask & 0x0F;
                spi3_rx_parity_mask = s_Temp << 12;
                spi3_rx_parity_mask |= readVal;    
            }
            else
            {
                spi3_parity_mask = arg_PortMask & 0xF0;
                readVal &= 0xF0;
                spi3_parity_mask |= readVal;

                /* Configuring the SPI3 RX config register - EVEN Parity */
                gbe_mac_read_32_bits(arg_DeviceNumber,0x701,&readVal);
                readVal = readVal & 0xFF80;
                s_Temp = arg_PortMask & 0x0F0;
                spi3_rx_parity_mask = s_Temp << 8;
                spi3_rx_parity_mask |= readVal;  
            }
            MESSAGE("EVEN-",0,0,0,0,0,0); 
        }
        else
        {
            MESSAGE("ODD-",0,0,0,0,0,0);
        }
        s_WriteSpi3Value = s_WriteSpi3Value | spi3_parity_mask;
	MESSAGE("SPI-3 write value = 0x%x",s_WriteSpi3Value,0,0,0,0,0);

        s_WriteSpi3RxValue = s_WriteSpi3RxValue | spi3_rx_parity_mask;
	MESSAGE("SPI-3 RX Config write value = 0x%x",s_WriteSpi3RxValue,0,0,0,0,0);

        MESSAGE("MODE ===== [0x%x]\n",mode,0,0,0,0,0);

	s_DiverseConfig = 0x114D;
        pmask = arg_PortMask;
        mode = mode & 0x07;
	switch(mode)
	{
		case 0 :   duplex = 1;
			   ifmode = 0;
			   speed = 1000;
                           s_DiverseConfig = 0x116D;
 			   s_rgmiiConf = 0x3;
                           MESSAGE("FB-FD-1000\n",0,0,0,0,0,0);
			   break;
	
		case 1 :   duplex = 0;
			   ifmode = 1;
			   speed = 1000;
 			   s_rgmiiConf = 0x6;
                           MESSAGE("CU-HD-1000\n",0,0,0,0,0,0);
			   break; 

		case 2 :   duplex = 1;
			   ifmode = 1;
			   speed = 1000;
 			   s_rgmiiConf = 0x6;
                           MESSAGE("CU-FD-1000\n",0,0,0,0,0,0);
			   break; 

		case 3 :   duplex = 0;
			   ifmode = 1;
			   speed = 100;
 			   s_rgmiiConf = 0x5;
                           MESSAGE("CU-HD-100\n",0,0,0,0,0,0);
			   break; 

		case 4 :   duplex = 1;
			   ifmode = 1;
			   speed = 100;
 			   s_rgmiiConf = 0x5;
                           MESSAGE("CU-FD-100\n",0,0,0,0,0,0);
			   break; 
		case 5 :   duplex = 0;
			   ifmode = 1;
			   speed = 10;
 			   s_rgmiiConf = 0x4;
                           MESSAGE("CU-HD-10\n",0,0,0,0,0,0);
			   break; 

		case 6 :   duplex = 1;
			   ifmode = 1;
			   speed = 10;
 			   s_rgmiiConf = 0x4;
                           MESSAGE("CU-FD-10\n",0,0,0,0,0,0);
			   break; 

		
		default:   MESSAGE("Illegal configuration attempt ..\n",0,0,0,0,0,0);
			   MESSAGE("Default configuration ..\n",0,0,0,0,0,0);
		           return;
	}

    mac_change_reset(arg_DeviceNumber, arg_PortMask);

    if(mode != 0)
    {
        MESSAGE("Configuring the PHY ....",0,0,0,0,0,0);
        config_phy(arg_DeviceNumber, speed, duplex ,arg_PortMask);
#if defined(__PHY_TEST__)
        gbe_mac_phy_write(arg_DeviceNumber,\
                                      0, 0x19, 0xA000);
        gbe_mac_phy_read(arg_DeviceNumber,\
                         0, 0x19, &readVal);
#if !defined(__LINUX_USER__)
        printk("\nGBE_MAC_PHY_READ [0x19] [0x%04x]\n",readVal);
#else
        printf("\nGBE_MAC_PHY_READ [0x19] [0x%04x]\n",readVal);
#endif //__LINUX_USER__
#endif /* __PHY_TEST__ */
        MESSAGE("Configured \n",0,0,0,0,0,0);
    }

    gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0x0);
    gbe_mac_write_32_bits(arg_DeviceNumber,SERDES_CLKMODE_REG,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0x0);

    /**
     * Set the various configurations likes CRC enable, AN enable, padding enable etc.
     */
    MESSAGE("\nConfiguring Diverse Config Registers...",0,0,0,0,0,0);
    basic_diverse_config(arg_DeviceNumber,arg_PortMask,s_DiverseConfig);
    MESSAGE(" -Configured",0,0,0,0,0,0);
			
    MESSAGE("\nConfiguring Other requisite Vallejo Registers, PLS WAIT...",0,0,0,0,0,0);

    /**
     *  Set SPI-3 Transmit and global configuration register
     *   1). 4x8 Mode configuration
     *   2). TX Burst Size - 128 Bytes maximum burst size
     *   3). TX Parity Sense - ODD
     */
    /* set SPI-3 TX config Register */
    gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_TX_GLOBAL_CONFIG,s_WriteSpi3Value);
    /* set SPI-3 RX config Register */
    gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_RX_CONFIG,s_WriteSpi3RxValue);


    /**
     * Assert the Port Enable.
     */
    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0x0);
    MESSAGE("Configuring the Interface Mode ....",0,0,0,0,0,0);
    basic_interface_mode(arg_DeviceNumber,arg_PortMask, s_rgmiiConf);
    MESSAGE("Configured\n",0,0,0,0,0,0);

    MESSAGE("Configuring the Duplex Mode ....",0,0,0,0,0,0);
    basic_duplex_configuration(arg_DeviceNumber,arg_PortMask,duplex);
    MESSAGE("Configured\n",0,0,0,0,0,0);

    basic_channel_mode(arg_DeviceNumber,arg_PortMask,arg_mode);

    /* Enable the port, by writing the Port Enable register */
    do_port_enable(arg_DeviceNumber,arg_PortMask);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0xf);

    mac_disable_reset(arg_DeviceNumber);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0x0);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0xf);

    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0xf);
    gbe_mac_write_32_bits(arg_DeviceNumber,0x505,0x0);

    return;
}


/**
 * This function performs the MAC reset.
 */
void mac_reset(uint32 arg_DeviceNumber)
{
        gbe_mac_write_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,0xF);
        gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,0xF);
        gbe_mac_write_32_bits(arg_DeviceNumber,RXFIFO_PORT_RESET,0xF);
        gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_TX_GLOBAL_CONFIG,0x00E00000);
        gbe_mac_write_32_bits(arg_DeviceNumber,MDIO_RESET,0x1);

    return;
}


/**
 * This function disables the MAC reset.
 */
void mac_disable_reset(uint32 arg_DeviceNumber)
{
        gbe_mac_write_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,RXFIFO_PORT_RESET,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,MDIO_RESET,0x0);
        gbe_mac_write_32_bits(arg_DeviceNumber,0x794,0xF);

    return;
}

/**
 * This function writes the Diverse configuration value to the respective port,
 * whose mask is passed in arg_PortMask.
 */
void basic_diverse_config(
                           uint32 arg_DeviceNumber,
                           uint32 arg_PortMask,
                           uint32 s_DiverseConfigValue
                         )
{
    uint32 s_Count;

    MESSAGE("Configuring the Diverse Configuration ..0x%x.\n",arg_PortMask,0,0,0,0,0);
    for(s_Count=0;s_Count<4;s_Count++)
    {
        if(((arg_PortMask >> s_Count) & 0x1) == 0x1)
        {
            MESSAGE(" Diverse Config [0x%x] for PORT [0x%x] \n",s_DiverseConfigValue,s_Count,0,0,0,0);
            gbe_mac_write_32_bits(arg_DeviceNumber,((0x80 * s_Count) + 0x18),s_DiverseConfigValue);
        }
    }

    MESSAGE(" Configured\n",0,0,0,0,0,0);
    return;
}

/**
 * This function writes the duplex configuration value to the respective port,
 * whose mask is passed in arg_PortMask.
 */
void basic_duplex_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_PortMask,
                                 uint32 s_DuplexModeValue
                               )
{
    uint32 s_Count;
    uint32 s_AddArray[] = {0,0x80,0x100,0x180};
    MESSAGE("Configuring the DUPLEX Mode ...0x%x\n",arg_PortMask,0,0,0,0,0);

    for(s_Count=0;s_Count<4;s_Count++)
    {
        if(((arg_PortMask >> s_Count) & 0x1) == 0x1)
        {
            MESSAGE(" DESIRED_DUPLEX[0x%x] Port[0x%x] \n",s_DuplexModeValue,s_Count,0,0,0,0);
            gbe_mac_write_32_bits(arg_DeviceNumber,(s_AddArray[s_Count]+DESIRED_DUPLEX),s_DuplexModeValue);
        }
    }
    MESSAGE("..Configured\n",0,0,0,0,0,0);
    return ;
}

/**
 * This function enables the MAC reset for individual ports.
 */
void mac_change_reset(uint32 arg_DeviceNumber, uint32 arg_PortMask)
{
        gbe_mac_write_32_bits(arg_DeviceNumber,MAC_SOFT_RESET,arg_PortMask);
        gbe_mac_write_32_bits(arg_DeviceNumber,TXFIFO_PORT_RESET,arg_PortMask);
        gbe_mac_write_32_bits(arg_DeviceNumber,RXFIFO_PORT_RESET,arg_PortMask);
        gbe_mac_write_32_bits(arg_DeviceNumber,SPI3_TX_GLOBAL_CONFIG,0x00E00000);
}

/**
 * This function writes the interface mode value to the respective port,
 * whose mask is passed in arg_PortMask.
 */
void basic_interface_mode(uint32 arg_DeviceNumber, uint32 arg_PortMask, uint32 arg_IFMode)
{
    uint32 s_Count;

    MESSAGE("Configuring the IF Mode ...0x%x\n",arg_PortMask,0,0,0,0,0);
    for(s_Count=0;s_Count<4;s_Count++)
    {
        if(((arg_PortMask >> s_Count) & 0x1) == 0x1)
        {
            MESSAGE("**** MAC_IF_MODE[0x%x] Port[0x%x] \n",arg_IFMode,s_Count,0,0,0,0);
            gbe_mac_write_32_bits(arg_DeviceNumber,((0x80 * s_Count) + MAC_IF_MODE),arg_IFMode);
        }
    }

    MESSAGE(" Configured\n",0,0,0,0,0,0);
    return;
}


/**
 * This function updates the Interface mode register.
 */
void basic_channel_mode(uint32 arg_DeviceNumber, uint32 arg_PortMask, uint32 arg_mode)
{

    uint32 readVal,s_WriteVal;

    MESSAGE("Configuring the FIBER/COPPER Mode ...PORT [0x%x]\n",arg_PortMask,0,0,0,0,0);

    gbe_mac_read_32_bits(arg_DeviceNumber,INTERFACE_MODE, &readVal);

    arg_mode = arg_mode & 0x7;

    if(arg_mode != 0)
    {
        s_WriteVal = arg_PortMask | readVal;
        MESSAGE("------[0x%x] ...\n",s_WriteVal,0,0,0,0,0);
    }
    else
    {
        s_WriteVal = arg_PortMask ^ 0xF;
        s_WriteVal &= readVal;
        MESSAGE("------[0x%x] ...\n",s_WriteVal,0,0,0,0,0);
    }

#if defined(__SIT_TEST__)
#if !defined(__LINUX_USER__)
    printk("FIBER/COPPER MODE-[0x%x] ...\n",s_WriteVal);
#else
    printf("FIBER/COPPER MODE-[0x%x] ...\n",s_WriteVal);
#endif //__LINUX_USER__
#endif /*__SIT_TEST__ */
    gbe_mac_write_32_bits(arg_DeviceNumber,INTERFACE_MODE,s_WriteVal);

    MESSAGE(" Configured\n",0,0,0,0,0,0);
    return;
}

/**
 * This function updates the Port Enable register register.
 */
void do_port_enable(uint32 arg_DeviceNumber, uint32 arg_PortMask)
{

    uint32 readVal,s_WriteVal;

    MESSAGE("Configuring the FIBER/COPPER Mode ...PORT [0x%x]\n",arg_PortMask,0,0,0,0,0);

    gbe_mac_read_32_bits(arg_DeviceNumber,PORT_ENABLE, &readVal);

    s_WriteVal = arg_PortMask | readVal;

#if defined(__SIT_TEST__)
#if !defined(__LINUX_USER__)
    printk("PORTs ENABLED-[0x%x] ...\n",s_WriteVal);
#else
    printf("PORTs ENABLED-[0x%x] ...\n",s_WriteVal);
#endif //__LINUX_USER__
#endif /*__SIT_TEST__ */

    gbe_mac_write_32_bits(arg_DeviceNumber,PORT_ENABLE,s_WriteVal);

    MESSAGE(" Configured\n",0,0,0,0,0,0);
    return;
}



