/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_linux_driver.h
 *
 * = DESCRIPTION
 *      
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *       1/8/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 * $Id: ixf1104ce_linux_driver.h,v 1.1.1.1 2003/04/11 23:17:26 nmafzal Exp $
 */

#if !defined(__IXF1104CE_LINUX_DRIVER_H__)
#define __IXF1104CE_LINUX_DRIVER_H__

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>


/**
 * Pre-processor symbol and macro definitions.
 */
#define PRESENT               1
#define TOTAL_PT_LONE_CARDS   2

/**
 * Type definitions.
 */

/**
 * Prototypes for interface functions.
 */
/**
 * NAME:  init_module
 *
 * DESCRIPTION:
 *  init_module function is called by the system call insmod command call.
 *  This function registers the specified device for the specified major
 *   number driver 
 * @Param:  - IN    : NULL
 * @Param:  - INOUT : NULL
 * @Param:  - OUT   : void
 *
 * @Return: int value
 */
int init_module (
                  void
                 );


/**
 * NAME:  cleanup_module
 *
 * DESCRIPTION: 
 *  Called by rmmod at the end to remove the Pt. Lone GbE
 *  Media Card device driver from kernel
 *
 * @Param:  - IN    : NULL
 * @Param:  - INOUT : NULL
 * @Param:  - OUT   : NULL
 *
 * @Return: NULL
 */
void cleanup_module (
                     void
                    );


/**
 * NAME: GbEMAC_open
 *
 * DESCRIPTION:Initialize the Device
 * 
 * @Param:  - IN    : struct inode *my_inode
 *                  : struct file *my_file
 * @Param:  - INOUT : 
 * @Param:  - OUT   :
 *
 * @Return:  
 */
int GbEMAC_open(
                struct inode *my_inode,
                struct file *my_file
               );

/**
 * NAME:
 *
 * DESCRIPTION: 
 *  Called to close the device, opened by the open call.
 *  Deallocate anything that open allocated in GbEMAC_file->private_data.
 *  Shut down the Device on the last close.
 *  Devrement the Usage count.
 *
 * @Param:  - IN    : struct inode *my_inode
 *                  : struct file * my_file
 * @Param:  - INOUT :
 * @Param:  - OUT   :
 *
 * @Return:
 */
int GbEMAC_close(
                 struct inode *my_inode,
                 struct file * my_file
                );

/**
 * NAME:
 *
 * DESCRIPTION: 
 *  Called by the ioctl system call, issued by the user application, that 
 *   wants to change the configuration values of the MAC device.
 *
 * @Param:  - IN    : struct inode *
 *                  : struct file *
 *                  : uint32
 * @Param:  - INOUT : void *
 * @Param:  - OUT   :
 *
 * @Return:
 */
int GbEMAC_ioctl(
                 struct inode *,
                 struct file *,
                 unsigned int ,
                 unsigned long
                );

/**
 * NAME: GbEMAC_i2s_fasync
 *
 * DESCRIPTION: 
 *       This functions gets registered in the kernel, to give the asynchronous
 *       notification to the user space process, when the interrupt occurred.
 *
 * @Param:  - IN    : int fd
 *                  : struct file *
 *                  : int mode
 * @Param:  - INOUT : 
 * @Param:  - OUT   :
 *
 * @Return:
 */
int GbEMAC_i2s_fasync(
                      int fd_inode,
                      struct file *filp,
                      int mode
                     );


/**
 * NAME: check_card_presence
 *
 * DESCRIPTION: 
 *       This function check the presence of the PtLone media card on the base-board.
 *
 * @Param:  - IN    : struct inode *
 *                  : struct file *
 *                  : uint32
 * @Param:  - INOUT : void *
 * @Param:  - OUT   :
 *
 * @Return:
 */
int check_card_presence(
                        int arg_DeviceNumber
                       );



/**
 * Exported variables.
 */
#define PHY_MSF_BASE	0xc8000000
#define PHY_MEDIA_SP	0xc6000000


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* __IXF1104CE_LINUX_DRIVER_H__ */
