/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_linux_driver.c
 *
 * = DESCRIPTION
 *      
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *       8/2/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf1104ce_linux_driver.c,v 1.1.1.1 2003/04/11 23:17:26 nmafzal Exp $";

/**
 * System defined include files required.
 */
#ifndef MODULE
#define MODULE
#endif

#define __NO_VERSION__
#include <linux/config.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/tqueue.h>

#include <../fs/hpfs/hpfs_fn.h>

#include <asm/arch/ixp2000.h>
#include <asm/arch/irqs.h>

#include <asm/uaccess.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
//#include <asm/arch/ixmb2400.h>
#include <linux/init.h>
#include <asm/poll.h>

/**
 * User defined include files required.
 */
#include "ixf1104ce_linux_driver.h"
#include "ixf1104ce_ioctl.h"
#include "ixf1104ce_internal.h"


/**
 * Pre-processor symbols and macros used in this file.
 */
#define MAJOR_PTLONE                10       /* The Pt Lone Media Card driver
                                                is registerd as a misc device
                                                under the character device. */
#define HIGH_BIT                     1
#define LOW_BIT                      0

#if defined __LINUX__
uint32 *pSPCfgBase;
uint32 *PT_LONE_REG_BASE_0;
uint32 *PT_LONE_REG_BASE_1;

#define SP_CSR_OFF      0x80000
#define PHYS_CAP_CSR_BASE     0xc0000000
#define SP_CSR_BASE     PHYS_CAP_CSR_BASE + SP_CSR_OFF
#endif


#define BYTE_SIZE_PTLONE_REGMAP        0x7FF*4 
/**
 * Types definitions whose scope is limited to this file.
 */
uint32 g_aCardPresence[2] = {0,0};

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
gbe_mac_global_data gbeMacGlobalData[2];


/**
 * TYPENAME: gbe_mac_fops
 * 
 * DESCRIPTION: 
 *      The Pt. Lone GbE Media Card device driver implements the specified
 *      device methods, and uses the tagged format to declare its
 *      file_operations structure as
 *
 */
static struct file_operations gbe_mac_fops = {
          ioctl: GbEMAC_ioctl,
          open: GbEMAC_open,
          release: GbEMAC_close,
          fasync: GbEMAC_i2s_fasync,         
};      

/**
 * dynamic allocation of the Major number from the kernel 
 */
int g_MajorNumber = 253;

/**
 * Extern function prototypes.
 */
extern int ptlone_i2c_init(void);
extern void ptlone_i2c_exit(void);

/**
 * structure used to signal a user process
 */
static struct fasync_struct *i2s_async_queue;

extern int check_card_presence(
                               int arg_DeviceNumber
                              );

#if defined(__LINUX__)
    extern struct timer_list s_RmonStatTimer_MI;
    extern struct timer_list s_RmonStatTimer_SF;
#endif /* __LINUX__ */


/**
 * Static function prototypes.
 */
static void gbe_mac_MEDIA(int,void *,struct pt_regs *);

static void gbe_mac_SF(int,void *,struct pt_regs *);


/**
 * Function definitions.
 */
/**
 * This function gets called on the insmod system call.
 */
int init_module(
                void
               )
{
    uint32 s_Result, s_Count;
    uint32 s_DeviceFlag = FALSE;
#ifdef __SIT_TEST__
    int i;
#endif //__SIT_TEST__

   
   MESSAGE("\n            INIT_MODULE is getting called...\n",0,0,0,0,0,0);
    /** 
     * Make sure that all the Port are closed.
     */
    for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
    {
        /* Make the Each MAC Port state as Closed */
        gbeMacGlobalData[0].g_PortState[s_Count] = CLOSED;
        gbeMacGlobalData[1].g_PortState[s_Count] = CLOSED;
        /* Initialize the Per Port USAGE count to '0' */
        gbeMacGlobalData[0].g_PortUsageCount[s_Count] = 0;
        gbeMacGlobalData[1].g_PortUsageCount[s_Count] = 0;
    }

    /**
     * Check for the Pt Lone MEdia card presence, by reading the EEPROM of Card 
     */
    /** 
     * Insert the I2C driver module, by calling the init function  for B0 Board. 
     */
    s_Result = ptlone_i2c_init();
    if(s_Result == 0)
    {
        printk("I2C Module loaded successfully....\n");
        s_DeviceFlag = TRUE;
    }


    /**
     * Check the presence of the Pt Lone card on Media Interface.
     */
    if(s_DeviceFlag == TRUE)
    {
        s_Result = check_card_presence(0);
    }
    else
    {
        s_Result = 0;
    }

    if(s_Result == PRESENT)
    {
        printk("PtLone GbE Media card PRESENT on the Media Interface...\n");
        printk("Devices are ptlone_media0,ptlone_media1,ptlone_media2,ptlone_media3...\n");
        g_aCardPresence[0] = 1;
        s_DeviceFlag = TRUE;
    }
    else
    {
        printk("MEDIA CARD on the Media Interface is not present!!!\n");
    }

    /**
     * Check the presence of the Pt Lone card on Switch Fabric Interface.
     */
    if(s_DeviceFlag == TRUE)
    {
        s_Result = check_card_presence(1);
    }
    else
    {
        s_Result = 0;
    }

    if(s_Result == PRESENT)
    {
        printk("PtLone Media card PRESENT on the Switch Fabric Interface...\n");
        printk("Devices are ptlone_sw0,ptlone_sw1,ptlone_sw2,ptlone_sw3...\n");
        g_aCardPresence[1] = 1;
        s_DeviceFlag = TRUE;
    }
    else
    {
        printk("MEDIA CARD on the Switch Fabric Interface is not present!!!\n");
    }

    if(s_DeviceFlag == TRUE)
    {/* If any of the Card is present */
        /**
         * Register the driver for the character device under misc catagory,
         * which has the major number 10.
         */
        s_Result = register_chrdev(g_MajorNumber, "PtLone",&gbe_mac_fops );
        if(s_Result == 0)
        {
            printk("PtLone Module for Card Driver is registered with major:%d\n",g_MajorNumber);
            MESSAGE ("To configure and Initialize the Vallejo MAC of\n",0,0,0,0,0,0);
            MESSAGE ("Pt. Lone Gigabit Ethernet Media Card,\n",0,0,0,0,0,0);
            MESSAGE ("you'll have to create a device file. \n",0,0,0,0,0,0);
            MESSAGE ("USAGE:\n",0,0,0,0,0,0);
            MESSAGE ("mknod PtLone<port number> c %d <minor>\n", g_MajorNumber,0,0,0,0,0);
            MESSAGE ("with different minor numbers.\n\n",0,0,0,0,0,0);
        }
        else
        {
            printk( "PtLone module cannot be registered!!!!.\n");
            MESSAGE("Registered to number %d\n",g_MajorNumber,0,0,0,0,0);
            return -1;
        }
    }

    PT_LONE_REG_BASE_0 =(unsigned int *) __ioremap((unsigned long)PHY_MEDIA_SP,0x100000, 0);
    if(PT_LONE_REG_BASE_0 ==(void*)-1)
    {
        MESSAGE("\nioremap failed for PT_LONE_REG_BASE_0",0,0,0,0,0,0);
        return -1;
    }

    PT_LONE_REG_BASE_1 = (uint32*)__ioremap((unsigned long)PHY_MSF_BASE,0x100000 ,0 );
    if(PT_LONE_REG_BASE_1 ==(void*)-1)
    {
        MESSAGE("\nioremap failed for PT_LONE_REG_BASE_1",0,0,0,0,0,0);
        return -1;
    }


    pSPCfgBase = (unsigned int *)__ioremap((unsigned long)SP_CSR_BASE,0x100000 ,0 );

//    pSPCfgBase = (VIRT_CAP_CSR + SP_CSR_OFF);
    MESSAGE("\n The pSPCfgBase  is %x ",*pSPCfgBase,0,0,0,0,0);

    if((PT_LONE_REG_BASE_0 == NULL))  
    {
	 MESSAGE("\n Fatal: __ioremapc call  failed... exiting",0,0,0,0,0,0);
         return -1;

    }   
    MESSAGE("__ioremap",0,0,0,0,0,0);

    /**
     * Register the Interrupt 
     */
    if(g_aCardPresence[0] == 1)
    {

        MESSAGE("Registering the ISR for Media IF Card\n",0,0,0,0,0,0);
        s_Result = request_irq(IRQ_IXDP2400_MEDIA_PCI, 
                               &gbe_mac_MEDIA,
                               SA_INTERRUPT,
                               "PtLone0",
                               NULL
                              );
        if(s_Result != 0)
        {
            printk("Error in Registering the ISR for Media IF Card!!!\n");
        }
    } 

    if(g_aCardPresence[1] == 1)
    {
        s_Result = request_irq(IRQ_IXDP2400_SF_PCI,
                               &gbe_mac_SF,
                               SA_INTERRUPT,
                               "PtLone4",
                               NULL
                              );
        if(s_Result != 0)
        {
            printk("Error in Registering the ISR for Switch Fabric IF Card!!!\n");
        }
    }
    
    /**
     * call the basic initialization routines for both the cards
     * after checking their presence
     */
    for(s_Count=0;s_Count<2;s_Count++)
    {
        if(g_aCardPresence[s_Count] == 1)
        {
            MESSAGE("gbe mac config\n",0,0,0,0,0,0);
            /** open the ports 0-3  in Fiber Mode, 4x8 SPI3 Mode with Odd Parity
             *   4    SPI-3 Block Mode
             *                              0 - MPHY Mode
             *                              1 - 4x8 Mode
             *   3    SPI-3 Parity
             *                              0 - Odd Parity
             *                              1 - Even Parity
             *   2-0  Channel, duplex and speed selection mode
             *                              000 :  Fiber operation
             *                              001 :  Gigabit copper, half duplex
             *                              010 :  Gigabit copper, full duplex
             *                              011 :  10/100 copper, half duplex
             *                              100 :  10/100 copper, full duplex
             */
            s_Result = gbe_mac_config(s_Count,0,0);
            if(s_Result == SUCCESS)
            {
                printk("Configuration Successful......\n");
            }
            else
            {
                printk("Basic Configuration Fails!!!!!\n");
            }
        }
    }

    return 0;
}

/**
 * This function is getting called ont he rmmod system call, the the driver
 * is to be unloaded from the kernel.
 */
void cleanup_module(
                    void
                   )
{
    uint32 s_Result;
    uint32 s_Count;

    MESSAGE("\n            CLEAN_UP MODULE is getting called...\n",0,0,0,0,0,0);

    for(s_Count=0;s_Count<2;s_Count++)
    {/* Reset both the cards */
        if(g_aCardPresence[s_Count] == 1)
        {
            gbe_mac_write_32_bits(s_Count,(MAC_SOFT_RESET),0xF);
            gbe_mac_write_32_bits(s_Count,(TXFIFO_PORT_RESET),0xF);
            gbe_mac_write_32_bits(s_Count,(RXFIFO_PORT_RESET),0x0);
            gbe_mac_write_32_bits(s_Count,(SPI3_TX_GLOBAL_CONFIG),0x00C00000);

            /* Disable all the PHY interrupts */
            gbe_mac_phy_write(s_Count,0x0, 0x19, 0x0);
            gbe_mac_phy_write(s_Count,0x1, 0x19, 0x0);
            gbe_mac_phy_write(s_Count,0x2, 0x19, 0x0);
            gbe_mac_phy_write(s_Count,0x3, 0x19, 0x0);

            /** Disable GBIC interrupts **/
            gbe_mac_write_32_bits(s_Count,GBIC_CTRL_REG,0x0);

            gbe_mac_write_32_bits(s_Count,(MDIO_RESET),0x1);
        }
    }

    if(g_aCardPresence[0] ==1)
    {
        free_irq(IRQ_IXDP2400_MEDIA_PCI, NULL);   
    }
    if(g_aCardPresence[1] == 1)
    {
        free_irq(IRQ_IXDP2400_SF_PCI, NULL); 
    }

    /**
     * Delete both the timers used for updating RMION stats. 
     */
    del_timer_sync(&s_RmonStatTimer_MI);         
    MESSAGE("Deleting the Timer s_RmonStatTimer_MI...\n",0,0,0,0,0,0);
    del_timer_sync(&s_RmonStatTimer_SF);
    MESSAGE("Deleting the Timer s_RmonStatTimer_SF...\n",0,0,0,0,0,0);

    /** 
     * Make sure that all the Port are closed.
     */
    for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
    {
        /* Make the Each MAC Port state as Closed */
        gbeMacGlobalData[0].g_PortState[s_Count] = CLOSED;
        gbeMacGlobalData[1].g_PortState[s_Count] = CLOSED;
        /* Initialize the Per Port USAGE count to '0' */
        gbeMacGlobalData[0].g_PortUsageCount[s_Count] = 0;
        gbeMacGlobalData[1].g_PortUsageCount[s_Count] = 0;
    }

    /**
     * If board version is B0, remove the I2C driver 
     */
    ptlone_i2c_exit();

    s_Result = unregister_chrdev(g_MajorNumber,"PtLone");
    if(s_Result == 0)
    {
        printk("<1>PtLone module for Media Interface card cleaned successfully...\n");
    }
    else
    {
        printk("<1>PtLone Module for Media Interface COULD NOT BE CLEANED UP !!!\n");
    }

    return;
}


/**
 * This function is called internally, when the user application calls the open
 * for the respective ports.
 */
int GbEMAC_open(
              struct inode *GbEMAC_inode,
              struct file *GbEMAC_file
             )
{

    uint32 s_PortNumber, s_Result;
    uint32 s_Major,s_Minor, s_DeviceNumber = 0, s_Value;
    gbe_mac_ioctl_ptr g_GbeMacIoctlPtr;

    MESSAGE("OPEN:GbEMAC_open is invoked...\n",0,0,0,0,0,0);
    /**
     * increment the usage count before doing anything else
     */
    MOD_INC_USE_COUNT;

    /* Retrieve the MINOR number from the devicxe file */
    s_PortNumber = MINOR(GbEMAC_inode->i_rdev);

    s_Major = MAJOR(GbEMAC_inode->i_rdev);
    s_Minor = MINOR(GbEMAC_inode->i_rdev);
    MESSAGE("MAJOR %d          MINOR %d\n",s_Major,s_Minor,0,0,0,0);

    fasync_helper((uint32)GbEMAC_inode, GbEMAC_file,( F_SETOWN | F_SETFL | FASYNC) , &i2s_async_queue);

    MESSAGE("Module is in use = %d \n",MOD_IN_USE,0,0,0,0,0);
    MESSAGE("Open device major=%d minor=%d i_rdev=0x%x\n", \
            MAJOR(GbEMAC_inode->i_rdev), MINOR(GbEMAC_inode->i_rdev),GbEMAC_inode->i_rdev,0,0,0);

    if(s_PortNumber >3)
    {/* intended for second card, sitting on the switch fabric interface */
        s_DeviceNumber = 1;
        s_PortNumber -= 4;
    }
   
    /* Calculate the MASK value for the respective port, that is bo opened */ 
    s_Value = HIGH_BIT << s_PortNumber;
    MESSAGE("PORT MASK : 0x%x\n",s_Value,0,0,0,0,0);

    /* Fill the Port number and the value in the IOCTL structure */
    g_GbeMacIoctlPtr.portIndex = s_PortNumber;
    g_GbeMacIoctlPtr.value = s_Value;

    if(gbeMacGlobalData[s_DeviceNumber].g_PortState[s_PortNumber] == OPENED)
    {/* The respective port has already being opened */
        MESSAGE("Port is Already Opened\n",0,0,0,0,0,0);
        gbeMacGlobalData[s_DeviceNumber].g_PortUsageCount[s_PortNumber]++;
    }
    else
    { /* The respective port needs to be enabled, and the corresponding values 
         are to be set accordingly.  For this an IOCTL is called, which enables the port,
         and set the port state as OPENED in the global data */
        s_Result = gbe_mac_config_handler(s_DeviceNumber,SET_PORT_INIT_ENABLE,(void *)&g_GbeMacIoctlPtr);
        if(s_Result == SUCCESS)
        {
            gbeMacGlobalData[s_DeviceNumber].g_PortUsageCount[s_PortNumber]++;
            MESSAGE("Port %d ENABLED SUCCESSFULLY\n",s_PortNumber,0,0,0,0,0);
        }
        else
        {
            MESSAGE("Port %d COULD NOT ENABLED\n",s_PortNumber,0,0,0,0,0);
        }
    }
    MESSAGE("EXITING from the GbEMAC_open() routine\n",0,0,0,0,0,0);
    return s_Result;
}

/**
 * This function is invoked, when the user application calls the close 
 * for the port, it has opened earlier.
 */
int GbEMAC_close(
               struct inode *GbEMAC_inode,
               struct file * GbEMAC_file
              )
{
    uint32 s_PortNumber, s_Result;
    uint32 s_DeviceNumber=0, s_Value = 0;
    gbe_mac_ioctl_ptr g_GbeMacIoctlPtr;    

    MESSAGE("\nCLOSE:GbEMAC_close is invoked \n",0,0,0,0,0,0);

    MESSAGE("Close device major=%d minor=%d \n", MAJOR(GbEMAC_inode->i_rdev),\
            MINOR(GbEMAC_inode->i_rdev),0,0,0,0);
    MESSAGE("Module is in use = %d \n",MOD_IN_USE,0,0,0,0,0);
    
    s_PortNumber = MINOR(GbEMAC_inode->i_rdev);

    GbEMAC_i2s_fasync (GbEMAC_inode->i_uid, GbEMAC_file, 0);

    if(s_PortNumber >3)
    {/* intended for second card, sitting on the switch fabric interface */
        s_DeviceNumber = 1;
        s_PortNumber -= 4;
    }

    MESSAGE("*******PORT NUMBER and VALUE .............\n",0,0,0,0,0,0);
    s_Value = HIGH_BIT << s_PortNumber;
    g_GbeMacIoctlPtr.portIndex = s_PortNumber;
    g_GbeMacIoctlPtr.value = s_Value;

    MESSAGE("*******PORT NUMBER and VALUE has been assigned\n",0,0,0,0,0,0);

    if(gbeMacGlobalData[s_DeviceNumber].g_PortUsageCount[s_PortNumber] == 0)
    {
        MESSAGE("PORT_ALREADY_CLOSED\n",0,0,0,0,0,0);
        s_Result = IOCTL_PORT_NOT_OPEN;
    }
    else
    {
        gbeMacGlobalData[s_DeviceNumber].g_PortUsageCount[s_PortNumber]--;
        if(gbeMacGlobalData[s_DeviceNumber].g_PortUsageCount[s_PortNumber] == 0)
        {/* if this application is the last application using this particular port */
            s_Result = gbe_mac_config_handler(s_DeviceNumber,SET_PORT_STATE_CLOSE,\
                                              (void *)&g_GbeMacIoctlPtr);        

            if(s_Result == SUCCESS)
            {
                MESSAGE("Port %d DISABLED SUCCESSFULLY\n",s_PortNumber,0,0,0,0,0);
            }
            else
            {
                MESSAGE("Port %d COULD NOT DISABLED\n",s_PortNumber,0,0,0,0,0);
            }
        }   
    }    

    /**
     * Decrement the usage count before doing anything else
     */
    MOD_DEC_USE_COUNT;

    MESSAGE("EXITING fromt the GbEMAC_close() routine\n",0,0,0,0,0,0);
    return s_Result;
}


int GbEMAC_i2s_fasync (
                       int fd_inode,
                       struct file *GbEMAC_file, 
                       int mode
                      ) 
{   
    return 1;
}


/**
 * This function is called by the ioctl command.
 */
int GbEMAC_ioctl(
               struct inode *GbEMAC_inode,
               struct file * GbEMAC_file,
               uint32 cmd,
               unsigned long arg_IoctlPtr
              )
{
    uint32 port_index;
    uint32 s_Result, s_deviceNumber=0, s_Result_memcpy;
    gbe_mac_ioctl_ptr_64 g_GbeMacIoctlPtr;

    MESSAGE("\nIOCTL:API giga_ioctl is invoked \n",0,0,0,0,0,0);

    if(GbEMAC_inode == NULL)
    {
        MESSAGE("GbEMAC_inode NULL\n",0,0,0,0,0,0);
        return -1;
    }
  
    if(GbEMAC_file == NULL)
    {
        MESSAGE("GbEMAC__file NULL\n",0,0,0,0,0,0);
        return -1;
    }        

    port_index = MINOR(GbEMAC_inode->i_rdev);       // retrieve the minor number => to the port number
                                                        // if not opened, return with the error message

    /**
     * Address translation of the buffer from the user space to the 
     * kernel space.
     */
    MESSAGE("MINOR NUMBER is %d\n",port_index,0,0,0,0,0);

    /**
     * Copy the user space memory to the kernel space 
     */
    if((void *)arg_IoctlPtr == NULL)
    {
        MESSAGE("IOCTL_BUFFER_POINTER_NULL\n",0,0,0,0,0,0);
        return IOCTL_BUFFER_POINTER_NULL;
    }

    s_Result_memcpy = (uint32)memcpy(&g_GbeMacIoctlPtr,(void *)arg_IoctlPtr, sizeof(gbe_mac_ioctl_ptr_64));  
    if(s_Result_memcpy <= 0)
    {
        MESSAGE("Error in Address Translation\n",0,0,0,0,0,0);
        return -1;
    }
    else
    {
        MESSAGE("Number of Bytes Transferred %d\n",s_Result_memcpy,0,0,0,0,0);
        MESSAGE("PORT NUMBER %d\nValue High %d\nValue Low %d\n",g_GbeMacIoctlPtr.portIndex, g_GbeMacIoctlPtr.valueHigh, g_GbeMacIoctlPtr.valueLow,0,0,0);
    }

    if(port_index >3)
    {/* intended for second card, sitting on the switch fabric interface */
        s_deviceNumber = 1;
        port_index -= 4;
    }

    g_GbeMacIoctlPtr.portIndex = port_index;
    s_Result = gbe_mac_config_handler(s_deviceNumber,cmd,(void *)&g_GbeMacIoctlPtr);
    if(s_Result == SUCCESS)
    {
        MESSAGE("GbEMAC_ioctl SUCCESS\n",0,0,0,0,0,0);
    }
    else
    {
        MESSAGE("GbEMAC_ioctl FAILS\n",0,0,0,0,0,0);
    }

    s_Result_memcpy = (uint32)memcpy((void *)arg_IoctlPtr, &g_GbeMacIoctlPtr,sizeof(gbe_mac_ioctl_ptr_64));   
    if(s_Result_memcpy <= 0)
    {
        MESSAGE("Error in Address Translation\n",0,0,0,0,0,0);
        return -1;
    }

    if((cmd & 0xE300) == 0xE300)
    {
        MESSAGE("->READ  VALUES are..",0,0,0,0,0,0);
        MESSAGE("[HIGH : 0x%8X] [LOW : 0x%8X]\n",\
                 g_GbeMacIoctlPtr.valueHigh,g_GbeMacIoctlPtr.valueLow,0,0,0,0);
    }

    MESSAGE("Exiting from the GbEMAC_iotcl() routine\n",0,0,0,0,0,0); 
    return s_Result;
}

  						  

static void gbe_mac_MEDIA(
                          int arg_Irg, 
                          void *arg_DevStruct,
                          struct pt_regs *arg_PtRegs
                         )
{
    /**
     * Call the ISR function for the PtLone card, which sits 
     * on the Media Interface
     */
    MESSAGE("ISR called\n",0,0,0,0,0,0);
    gbe_mac_isr(0);

    return;

}


static void gbe_mac_SF(
                       int arg_Irg,
                       void *arg_DevStruct,
                       struct pt_regs *arg_PtRegs
                      )
{
    /**
     * Call the ISR function for the PtLone card, which sits
     * on the Media Interface
     */
    MESSAGE("ISR called\n",0,0,0,0,0,0);
    gbe_mac_isr(1);

    return;
}


MODULE_LICENSE("Intel Corp.");

 
