/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf1104ce_linux_interface.c
 *
 * = DESCRIPTION
 *      This file implements the function, which reads the EEPROM contents
 *      from the Pt Lone card through I2C.
 *
 * = AUTHOR
 *      Dheeraj Singh
 *      dheeraj.singh@wipro.com
 *
 * = CHANGE HISTORY
 *       8/2/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf1104ce_linux_interface.c,v 1.1.1.1 2003/04/11 23:17:26 nmafzal Exp $";


/**
 * System defined include files required.
 */

/**
 * User defined include files required.
 */
#include "ixf1104ce_linux_driver.h"
#include "ixf1104ce_ioctl.h"
#include "ixf1104ce_internal.h"
#include <linux/slab.h>
#include <asm/io.h>
#include <linux/i2c.h>

//#include <linux/i2c-algo-ixmb.h>
//#include <asm/arch/tvm.h>

#include <linux/config.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <asm/io.h>
#include <linux/i2c.h>
#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>

#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/i2c-id.h>
#include <linux/string.h>
#include <linux/rtc.h>

#include <asm/system.h>


/**
 * Pre-processor symbols and macros used in this file.
 */
#define DEVICE_NOT_PRESENT        0x00
#define EEPROM_NOT_PROGRAMMED     0xFF
#define SHIFT_BYTE                0x08
#define SHIFT_NIBBLE              0x04
#define PTLONE_DEV_ID_OFF         0x06

#define BUF_SIZE         256

/**
 * Types definitions whose scope is limited to this file.
 */

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

/* Two structure of the global data for two PtLone MEdia Cards */
extern gbe_mac_global_data gbeMacGlobalData[2];


#define DAT(x) ((unsigned int)(x->data))

unsigned char s_i2c_contents_buf[BUF_SIZE]; /* contents from i2c device */
unsigned char s_i2c_contents_sf[BUF_SIZE]; /* contents on SF side slot */
unsigned char s_i2c_contents_mf[BUF_SIZE]; /* contents on media side slot */
unsigned int g_DeviceNumber;

static struct i2c_driver ptlone_driver;

struct i2c_adapter *adap_mi, *adap_sf;

static int
ptlone_detach(struct i2c_client *client);

static int
ptlone_probe(struct i2c_adapter *adap);

static struct i2c_driver ptlone_driver = {
        name:           "PTLone",
        id:             I2C_HW_B_IXDP2400,
        flags:          I2C_DF_NOTIFY,
        attach_adapter: ptlone_probe,
        detach_client:  ptlone_detach,
};

static struct i2c_client *this_client;
static int i2c_ptlone_initialized;

static unsigned short ignore[] = { I2C_CLIENT_END };

/* device 0x52=media slot, device 0x53=switch fabric slot */
static unsigned short normal_addr[] = { 0x52,0x53, I2C_CLIENT_END };

static struct i2c_client_address_data addr_data_mi = {
        normal_i2c:             normal_addr,
        normal_i2c_range:       ignore,
        probe:                  ignore,
        probe_range:            ignore,
        ignore:                 ignore,
        ignore_range:           ignore,
        force:                  ignore,
};

/* Each one structure for each Pt Lone Media Card */
gbe_mac_global_struct gbeMacGlobalStruct[2]; 

/* For the two Pt Lone cards */
gbe_mac_mac_address gbeMacAddress[2];
 
/* offset per port */
uint32 g_PerPortOffset[4] = {
                                MAC_CTL_REG_PORT0,
                                MAC_CTL_REG_PORT1,
                                MAC_CTL_REG_PORT2,
                                MAC_CTL_REG_PORT3
                            };

/**
 * Extern function prototypes.
 */
int check_card_presence(
                        int arg_DeviceNumber
                       );

/**
 * Static function prototypes.
 */
/**
 * NAME: get_device_EEPROM_offset
 *
 * DESCRIPTION: This function returns the address of the EEPROM on the device
 *
 * @Param: -IN    : uint32 arg_deviceNumber
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: EEPROM offset for the device on the I2C.
 */
unsigned char get_device_EEPROM_offset(
                                       uint32 arg_DeviceNumber
                                      );


static inline void outlong(unsigned int dat, int addr)
{
        *((unsigned int *) (addr)) = dat;
}

static inline unsigned int readlong(int addr)
{
        return *((unsigned int *) (addr));
}


static __init int ptlone_i2c_init(void)
{
         int err;

         i2c_ptlone_initialized = 0;

    	MESSAGE("Loading I2C IXD2410 drivere.\n",0,0,0,0,0,0);
         err = i2c_add_driver(&ptlone_driver);
         if (err) {
                 printk("ptlone.o: Register I2C driver failed, errno is %d\n"
                         ,err);
                 return err;
         }
         i2c_ptlone_initialized ++;
         return 0;
}

int __exit ptlone_i2c_exit(void)
{
        int res;

    	MESSAGE("I2C module exited for IXD2410 device.\n",0,0,0,0,0,0);
        if (i2c_ptlone_initialized) {
                ptlone_detach(this_client);
                if ((res = i2c_del_driver(&ptlone_driver))) {
                        printk(KERN_ERR "i2c-ptlone.o: Driver deregistration failed, "
                               "module not removed.\n");
                        return res;
                }
                i2c_ptlone_initialized --;
        }
	kfree(this_client);
        return 0;
}

static int
ptlone_detach(struct i2c_client *client)
{
    	MESSAGE("I2C calling attach for IXD2410 device.\n",0,0,0,0,0,0);
        i2c_detach_client(client);

        return 0;
}

static int
ptlone_attach(struct i2c_adapter *adap, int addr, unsigned short flags,
        int kind)
{
        struct i2c_msg msgs[2] = { 
		{ addr, 0, 1, s_i2c_contents_buf },
		{ addr, I2C_M_RD, BUF_SIZE, s_i2c_contents_buf }
	};
        int i;

    	MESSAGE("I2C calling attach for IXD2410 device.\n",0,0,0,0,0,0);
        this_client = kmalloc(sizeof(*this_client), GFP_KERNEL);

        if (!this_client) {
                return -ENOMEM;
        }
    	MESSAGE("I2C address for IXD2410 device:%x.\n",addr,0,0,0,0,0);

        strcpy(this_client->name, "PTLONE");
        this_client->id         = ptlone_driver.id;
        this_client->flags      = 0;
        this_client->addr       = addr;
        this_client->adapter    = adap;
        this_client->driver     = &ptlone_driver;
        this_client->data       = NULL;

        for (i=0; i<BUF_SIZE;i++)
                s_i2c_contents_buf[i] = 0;

        if (i2c_transfer(this_client->adapter, msgs, 2) == 2) {
                DAT(this_client) = s_i2c_contents_buf[0];
		if (addr == 0x52) { /* if media slot */
        		for (i=0; i<BUF_SIZE;i++)
                		s_i2c_contents_mf[i] = s_i2c_contents_buf[i];

		} else {
        		for (i=0; i<BUF_SIZE;i++)
                		s_i2c_contents_sf[i] = s_i2c_contents_buf[i];
		}

	}


        i2c_attach_client(this_client);


#if defined(__DEBUG__)
        for (i=0; i<BUF_SIZE; i=i+8)
               printk("%02d: %02x %02x %02x %02x %02x %02x %02x %02x\n"
                , i
                , s_i2c_contents_buf[i], s_i2c_contents_buf[i+1], s_i2c_contents_buf[i+2],\
                  s_i2c_contents_buf[i+3],s_i2c_contents_buf[i+4],s_i2c_contents_buf[i+5],\
                  s_i2c_contents_buf[i+6],s_i2c_contents_buf[i+7]);
#endif


        return 0;
}

static int
ptlone_probe(struct i2c_adapter *adap)
{
    	MESSAGE("I2C Probing IXD2410 device.\n",0,0,0,0,0,0);
        return i2c_probe(adap, &addr_data_mi, ptlone_attach);
}


/**
 * Function definitions.
 */
int check_card_presence(
                        int arg_DeviceNumber
                       )
{
    /**
     * This actually reads the EEPROM of the Pt Lone Media card through 
     * I2C bus, and returns PRESENT if the card is present.
     */
    unsigned char s_DeviceOffset, s_MacOffset[4], s_ByteCount, s_Temp;
    unsigned char s_DevID;
    uint32 s_ReturnValue;
    uint32 s_PortCount,s_MacAddressHigh,s_MacAddressLow;
    unsigned char s_MacAddr_r[6];
    uint32 s_DeviceStartFlag;
    unsigned char *s_i2c_contents;

    s_ReturnValue = PRESENT;
    MESSAGE("CARD PRESENCE is checked for card %d\n",arg_DeviceNumber,0,0,0,0,0);

    /**
     * set the device EEPROM address according to the device 
     */
    s_DeviceOffset = get_device_EEPROM_offset(arg_DeviceNumber);

    /**
     *  Set the Offset of the MAC Address in the EEPROM
     */
    s_MacOffset[0] = EEPROM_MAC_PORT_OFF_0;
    s_MacOffset[1] = EEPROM_MAC_PORT_OFF_1;
    s_MacOffset[2] = EEPROM_MAC_PORT_OFF_2;
    s_MacOffset[3] = EEPROM_MAC_PORT_OFF_3;

    /**
     * Read the device ID from the EEPROM
     */
    /**
     * call the function to attach the i2c driver 
     */
    g_DeviceNumber = arg_DeviceNumber;
    if(arg_DeviceNumber == 0)
    {
	s_DevID = s_i2c_contents_mf[0x1]; 
	s_i2c_contents = &s_i2c_contents_mf[0];
    }
    else if(arg_DeviceNumber == 1)
    {
	s_DevID = s_i2c_contents_sf[0x1]; 
	s_i2c_contents = &s_i2c_contents_sf[0];
    }
    else
    {
        printk("Wrong Device Number !!!\n");
    }


    MESSAGE("Device ID Read %d\n",s_DevID,0,0,0,0,0);

    if(s_DevID == PT_LONE_DEVICE_ID)
    {/* the read valuer for the device id matches with the defined value of the device ID */
        for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
        {/* Countinue for all the 4 MAC ports */
            for(s_ByteCount=0; s_ByteCount<6; s_ByteCount++)
            {/* read all the 6-bytes of the MAC address */
                s_MacAddr_r[s_ByteCount] = s_i2c_contents[s_MacOffset[s_PortCount] + s_ByteCount];
            }/* end-for reading the MAC Address from the EEPROM */
            MESSAGE("PORT MAC ADDRESS 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n",s_MacAddr_r[0],\
                     s_MacAddr_r[1],s_MacAddr_r[2],s_MacAddr_r[3],s_MacAddr_r[4],s_MacAddr_r[5]); 
            /* Calculate the MAC address, low and high part */
            s_MacAddressHigh = 0;
            s_MacAddressHigh = (s_MacAddr_r[0] << SHIFT_BYTE | s_MacAddressHigh);
            s_MacAddressHigh = s_MacAddressHigh | s_MacAddr_r[1];

            s_MacAddressLow = 0;
            s_MacAddressLow = (s_MacAddressLow | (s_MacAddr_r[2] << (3*SHIFT_BYTE ))\
                                               | (s_MacAddr_r[3] << (2*SHIFT_BYTE )) \
                                               | (s_MacAddr_r[4] << (SHIFT_BYTE )) \
                                               | s_MacAddr_r[5] );

            switch(s_PortCount)
            { /* Assign the MAC address to the corresponding MAC port */
                case 0: /* MAC Port 0 */
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_0[0] = s_MacAddressLow;
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_0[1] = s_MacAddressHigh;
                        break;

                case 1: /* MAC Port 1 */
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_1[0] = s_MacAddressLow;
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_1[1] = s_MacAddressHigh;
                        break;

                case 2: /* MAC Port 2 */
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_2[0] = s_MacAddressLow;
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_2[1] = s_MacAddressHigh;
                        break;

                case 3: /* MAC Port 3 */
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_3[0] = s_MacAddressLow;
                        gbeMacAddress[arg_DeviceNumber].macAddressPort_3[1] = s_MacAddressHigh;
                        break;

                default:
                        break;
            }; /* end-switch case statements */
        } /* end-for reading the EEPROM for the MAC address */
    } /* end-if Valid device ID found */
    else if(s_DevID == DEVICE_NOT_PRESENT)
        { /* Device is not PRESENT */
            MESSAGE("Read Value is %d -> DEVICE_NOT_PRESENT\n",s_DevID,0,0,0,0,0);
            switch(arg_DeviceNumber)
            {
                case 0:
                        MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE;
                        s_DeviceStartFlag = FALSE;
                        break;

                case 1:
                        MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE;
                        s_DeviceStartFlag = FALSE;
                        break;

                default:
                        break;
            };/* end-switch */
        } /* end-if Device is not present */
        else if(s_DevID == EEPROM_NOT_PROGRAMMED)
            { /* EEPROM is not programmed */
                MESSAGE("Read Value is %d -> EEPROM_NOT_PROGRAMMED\n",s_DevID,0,0,0,0,0);
                switch(arg_DeviceNumber)
                {
                   case 0:
                       MESSAGE("EEPROM_OF_PT_LONE_MEDIA_CARD_ON_MEDIA_INTERFACE_IS_NOT_PROGRAMMED\n",0,0,0,0,0,0);
                       s_ReturnValue = EEPROM_OF_PT_LONE_MEDIA_CARD_ON_MEDIA_INTERFACE_IS_NOT_PROGRAMMED;
                       s_DeviceStartFlag = FALSE;
                       break;

                   case 1:
                       MESSAGE("EEPROM_OF_PT_LONE_MEDIA_CARD_ON_SWITCH_FABRIC_INTERFACE_IS_NOT_PROGRAMMED\n",0,0,0,0,0,0);
                       s_ReturnValue = EEPROM_OF_PT_LONE_MEDIA_CARD_ON_SWITCH_FABRIC_INTERFACE_IS_NOT_PROGRAMMED;
                       s_DeviceStartFlag = FALSE;
                       break;

                   default:
                       break;
               }; /* end-switch */
            }/* end-if EEPROM is not programmed */
            else
            {/* Some other card is present on the Interface. */
               switch(arg_DeviceNumber)
               {
                   case 0:
                        MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE;
                        s_DeviceStartFlag = FALSE;
                        break;

                   case 1:
                        MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE;
                        s_DeviceStartFlag = FALSE;
                        break;

                   default:
                        break;
                }
            } /* end-if other card is present */

    return s_ReturnValue;
}     


unsigned char get_device_EEPROM_offset(uint32 arg_DeviceNumber)
{
    if(arg_DeviceNumber == 0)
    {
        return MEDIA_IF_EEPROM_ADDR;
    }
    else
    {
        return SF_IF_EEPROM_ADDR;
    }
}

