/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_internal.h
 *
 * = DESCRIPTION
 *      This is the header file for the defining the data structures
 *      used in driver code
 *
 * = CHANGE HISTORY
 *       *      1/30/2003 14:23:24 PM - creation time
 *
 * ============================================================================
 * $Id: ixf6048_internal.h,v 1.1 2003/03/07 05:21:28 cmah1 Exp $
 */

#if !defined(__IXF6048_INTERNAL_H__)
#define __IXF6048_INTERNAL_H__
#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * Pre-processor symbol and macro definitions.
 */
#define MESSAGE(A,B,C,D,E,F,G)

typedef unsigned int uint32;

/**
 * TYPENAME: atm_pos_port_states
 *
 * DESCRIPTION: Possible states of a Port
 *
 */
typedef enum atm_pos_e_port_state {
    CLOSED = 0,
    OPENED
} atm_pos_port_state;


/**
 * TYPENAME: atm_pos_ioctl_ptr
 *
 * DESCRIPTION: IOCTL structures, the pointer to this structure, typecast with
 *              the void *, is passed in the IOCTL API.
 */
typedef struct atm_pos_s_ioctl_ptr {
      uint32  portIndex;             /* Port Number */
      uint32  value;                 /* Buffer which hold the value to be set/get */
} atm_pos_ioctl_ptr;


/**
 * TYPENAME: atm_pos_ioctl_ptr_64
 *
 * DESCRIPTION:
 *
 */
typedef struct atm_pos_s_ioctl_ptr_64 {
    uint32  portIndex;             /* Port Number */
    uint32  valueHigh;             /* High 47:32 bits of the address */
    uint32  valueLow;              /* Low 31:0 btis of the assress */
} atm_pos_ioctl_ptr_64;


/**
 * Prototypes for interface functions.
 */

/**
 * NAME: atm_pos_config_handler
 *
 * DESCRIPTION:
 *      This function parses the GET/SET type ioctl commands.
 *      and calls the get/set config handler accordingly.
 *
 * @Param:  - IN    : arg_cmd :IOCTL Command number,
 * @Param:  - INOUT : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param:  - OUT
 *
 * @Return: atm_pos_error token.
 */
atm_pos_error atm_pos_config_handler (
                                             atm_pos_IOCTL_CMD arg_cmd,
                                             void * arg_pIoctlPtr64
                                            );
/**
 * NAME: atm_pos_get_config_handler
 *
 * DESCRIPTION:
 *      This function implements the GET type ioctl commands.  It
 *      reads the value of the specified register and store that
 *      value in the value parameter of the IOCTL_PTR passed to it
 *      as a void pointer.  And based upon the read value it generate
 *      the ERROR CODE.
 *
 * @Param:  -IN    : arg_cmd :IOCTL Command number,
 * @Param:  -INOUT : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param:  -OUT   :
 *
 * @Return: atm_pos_error token.
 */
atm_pos_error atm_pos_get_config_handler (
                                          atm_pos_IOCTL_CMD arg_cmd,
                                          void * arg_pIoctlPtr64
                                         );

/**
 * NAME: atm_pos_set_config_handler
 *
 * DESCRIPTION:
 *      This function implements the SET type ioctl commands.  It write
 *      the given value, passed through the IOCTL_PTR, to the specified
 *      register.
 *
 * @Param: -IN    : arg_cmd :IOCTL Command number,
 *                : arg_pIoctlPtr: IOCTL struct pointer typecasted with the void *.
 * @Param: -INOUT :
 * @Param: -OUT
 *
 * @Return: atm_pos_error token.
 */
atm_pos_error atm_pos_set_config_handler (
                                          atm_pos_IOCTL_CMD arg_cmd,
                                          void * arg_pIoctlPtr64
                                         );


/**
 * Exported variables.
 */
#if !defined(__LINUX_USER__)                    /* Common global data for Linux kernel mdoe, and VxWorks */
typedef struct atm_pos_s_global_data {
        uint32 g_PortState[4];                  /* Maitianing the Port Status saperately for each port */
        uint32 g_PortUsageCount[4];             /* Maintains per port usage count, it helps in determing
						   the other users, while one is trying to close the port. */
}atm_pos_global_data;

#else                                           /* Global Data specific to the Linux User Mode driver */
typedef struct atm_pos_s_global_data {
        uint32 g_PortState[4];                  /* Maitianing the Port Status saperately for each port */
        uint32 g_PortUsageCount[4];             /* Maintains per port usage count, it helps in determing
                                                   the other users, while one is trying to close the port. */
        uint32 g_UsageCount;                    /* Maintains the Total number of application using the individually
                                                   Battery Drew/Wallace media card saperately. */
        int  g_MutexSemaphore;                /* Mutex Semaphore */

}atm_pos_global_data;
#endif /* end defined __LINUX_USER__ */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /*end defined __IXF6048_INTERNAL_H__*/
