/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_ioctl.h
 *
 * = DESCRIPTION
 *      This header file contains the IOCTL commands and their value
 *      and the Error Code numbers, along with the API which are
 *      exposed to the user to use the corresponding driver. 
 *
 * = AUTHOR
 *
 * = CHANGE HISTORY
 *      1/30/2003 14:23:24 PM - creation time
 *
 * ============================================================================
 * $Id: ixf6048_ioctl.h,v 1.7 2003/05/30 03:57:23 cmah1 Exp $
 */

#if !defined(__IXF6048_IOCTL_H__)
#define __IXF6048_IOCTL_H__

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Pre-processor symbol and macro definitions.
 */

#define GET_IOCTL       0xE300  /* GET type IOCTLs starts from 0xE300 */
#define SET_IOCTL       0xE200  /* SET type IOCTLs starts from 0xE200 */

#define TRUE            1
#define FALSE           0

/**
 * Type definitions.
 */

/**
 * TYPENAME: atm_pos_error
 *
 * DESCRIPTION: Error code enumerator
 *
 */
typedef enum atm_pos_e_error {
    SUCCESS = 0x0000,
    REGISTRATION_FAILED = 0x0400,
    ERROR_DEVICE_ALREADY_INITIALIZED,
    PLL_NOT_LOCKED,
    INTERRUPT_INSTALLATION_FAILED,
    INTERRUPT_FREEING_FAILED,
    IOCLT_ERROR_INPUT_REG_VALUE_OUT_OF_RANGE,
    IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID,
    IOCTL_ERROR_UNKNOWN_IOCTL_CODE,
    IOCTL_BUFFER_POINTER_NULL,
    IOCTL_BUFFER_POINTER_INVALID,
    IOCTL_PORT_NOT_OPEN,
    MODE_ALREADY_SET_IN_SPECIFIED_DUPLEX_MODE,
    ERROR_INVALID_DUPLEX_MODE,
    FAIL_RX_FIFO_ERRORED_FRAME_DROP_IS_DISABLE,
    ERROR_CONFLICT_WITH_HIGH_WATERMARK,
    ERROR_VALID_ONLY_FOR_COPPER_MODE,
    ERROR_VALID_ONLY_FOR_FIBER_MODE,
    ERROR_MDI_ENABLE_BIT_IS_RESET_IN_MDI_CONTROL_REG,
    ERROR_CONFLICT_WITH_LOW_WATERMARK,
    ERROR_NOT_ALLOWED_IN_LOOPBACK_MODE,
    ERROR_CONFLICT_WITH_JUMBO_FRAME_SIZE,
    PLL_TRANSMIT_LOCK_STATUS_FAILED,
    IOCTL_READ_VALUE_OUT_OF_RANGE,
    ERROR_COULD_NOT_COPY_RMON_STAT_FROM_KERNEL_MODE,
    PCI_ERROR_COULD_NOT_GRAB_SEMAPHORE,
    ERROR_GET_RMON_STAT_TASK_COULD_NOT_SPAWNED,
    ERROR_COULE_NOT_OPEN_DEVICE,
    ERROR_INT_LOCK_FAILED,
    ERROR_TASK_LOCK_FAILED,
    SEMAPHORE_COULD_NOT_CREATED,
    ERROR_INVALID_HANDLE,
    ERROR_PREVIOUS_MDI_COMMAND_STILL_NOT_COMPLETED,
    ERROR_COULD_NOT_PERFORM_MDIO_READ,
    ERROR_MDI_RD_WR_DISABLE,
    ERROR_COULD_NOT_PERFORM_MDIO_WRITE,
    SPI3_TX_LOCK_FAILED,
    ERROR_SHARED_MEMORY_NOT_CREATED,
    ERROR_SHARED_MEMORY_NOT_LOCKED,
    ERROR_SHARED_MEMORY_NOT_UNLOCKED,
    ERROR_SHARED_MEMORY_NOT_DESTROYED,
    ERROR_SHARED_MEMORY_NOT_DETATCHED,
    ERROR_SHARED_MEMORY_NOT_ATTACHED,
    ERROR_SEMAPHORE_NOT_GRABED,
    INVALID_PORT_MASK,
    MEMORY_ALLOCATION_FAILS,
    WARNING_ONLY_LOWER_FOUR_BYTES_ARE_WRITABLE = 0x430,
    WARNING_ONLY_LOWER_THREE_BITS_ARE_WRITABLE
} atm_pos_error;

/**
 * TYPENAME: atm_pos_e_IOCTL_CMD
 *
 * DESCRIPTION:
 *
 */
typedef enum atm_pos_e_IOCTL_CMD {
/**
 * ATM POS Control SET IOCTL Commands
 */
    SET_OPMODE = 0xE201,
    SET_CHIP_ONLINE = 0xE202,
    SET_CHIP_OFFLINE = 0xE203,
    SET_ALARM_CFG = 0xE204,
    SET_WINDOW_SIZE = 0xE205,
    SET_TRACE = 0xE206,
    SET_OH_BYTES = 0xE207,
    GENERIC_WRITE = 0xE210,
    GENERIC_READ = 0xE211,
    SET_CARD_MODE = 0xE212,
    SET_ME_TX = 0xE230,

    SET_CFG = 0xE220,
    SET_RMST_SOURCE_BYTES = 0xE221,

/**
 * ATM POS Control GET IOCTL Commands
 */
    GET_STATUS = 0xE300,
    GET_OPMODE = 0xE301,
    GET_COUNTERS = 0xE302,
    GET_CHIP_INFO = 0xE303,
    GET_BUILD_VERSION = 0xE304,
    GET_WINDOW_SIZE = 0xE305,
    GET_TRACE = 0xE306,
    GET_OH_BYTES = 0xE307,

    GET_CFG = 0xE320,
    GET_RMST_SOURCE_BYTES = 0xE321,
    GET_MAPPED_FRAME = 0xE322,
    GET_AU_POINTER = 0xE323

} atm_pos_IOCTL_CMD;


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* #ifndef __IXF6048_IOCTL_H__ */
