/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_config.c
 *
 * = DESCRIPTION
 *      This file initializes the Mutex semaphore and performs the media
 *      configuration as per the requirement.
 *
 * = CHANGE HISTORY
 *       2/25/2003 14:14:24 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf6048_config.c,v 1.1 2003/03/07 05:21:39 cmah1 Exp $";


/**
 * System defined include files required.
 */

#include <linux/module.h>
#include <linux/kernel.h>

/**
 * User defined include files required.
 */
#include "ixf6048_ioctl.h"
#include "ixf6048_internal.h"

/** Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
extern atm_pos_global_data atmPosGlobalData;

atm_pos_error atm_pos_config_handler(
                                     atm_pos_IOCTL_CMD arg_Cmd,
                                     void *arg_pIoctlStruct64
                                    )
{
    uint32 s_IoctlType;
    uint32 s_RetValue;

    MESSAGE("Config Handler function atm_pos_config_handler is invoked\n",0,0,0,0,0,0);

    /**
     * AND the arg_Cmd with the GET_IOCTL, ie 0xE300, which results same in case of GET
     * type ioctl, ans changes to 0xE200 in case of SET type IOCTLs.
     */
    s_IoctlType = arg_Cmd & GET_IOCTL;
    MESSAGE("IOCTL Command value is %x\n",arg_Cmd,0,0,0,0,0);

    /* Initializes the return value to SUCCESS */
    s_RetValue = SUCCESS;

    switch(s_IoctlType)
    {
        case GET_IOCTL:
            /* ioctl is of GET type */
            MESSAGE("Command is GET ioctl, and GET IOCTL handler function is called\n",0,0,0,0,0,0);
            MESSAGE("called ioctl command value is %x\n",arg_Cmd,0,0,0,0,0);
            s_RetValue = atm_pos_get_config_handler(arg_Cmd, arg_pIoctlStruct64);
            break;

        case SET_IOCTL:
            /* ioctl is of SET type */
            MESSAGE("Command is SET ioctl, and SET IOCTL handler function is called\n",0,0,0,0,0,0);
            MESSAGE("called ioctl command value is %x\n",arg_Cmd,0,0,0,0,0);
            s_RetValue = atm_pos_set_config_handler(arg_Cmd, arg_pIoctlStruct64);
            break;

        default:
            /* Invalid ioctl command */
            MESSAGE("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
            s_RetValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
            break;
    };/* end-switch(s_IoctlType) */
    MESSAGE("Returning from the Config Handler Function atm_pos_config_handler.\n",0,0,0,0,0,0);
    return s_RetValue;
}

