/* ===========================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_get_ioctl.c
 *
 * = DESCRIPTION
 *     This file implements the function used for the GET type ioctl commands.
 *     It reads the value of the specified register and store that
 *     value in the value parameter of the atm_pos_ioctl_ptr passed to it
 *     as a void pointer.  And based upon the read value it generate
 *     the ERROR CODE.
 *
 * = CHANGE HISTORY
 *      2/18/2003 13:33:24 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf6048_get_ioctl.c,v 1.8 2003/06/03 05:38:57 cmah1 Exp $";


/**
 * System defined include files required.
 */
#include <linux/module.h>
#include <linux/kernel.h>

typedef int sint;
typedef short sshort;
typedef char schar;
typedef unsigned char uchar;
typedef unsigned char boolean;  /* used in function calls */
/**
 * User defined include files required.
 */
#include "common_def.h"
#include "ixf_api_d.h"          /* BroadBand Utility Definitions and Function headers */
#include "ixf_api.h"
#include "ixf_api_sonet_d.h"
#include "ixf_api_sonet.h"
#include "ixf6048d.h"
#include "ixf6048_ioctl.h"
#include "ixf6048_internal.h"
#include "ixf6048_linux_driver.h"

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
extern atm_pos_global_data atmPosGlobalData;
extern bb_ChipData_t pChipData;
extern int Media_Card;
extern int Card_Mode;

/**
 * Function definitions.
 */

atm_pos_error
atm_pos_get_config_handler( atm_pos_IOCTL_CMD arg_cmd,
                            void *arg_pIoctlPtr64)
{

    static uint32 s_ReturnValue;
    atm_pos_ioctl_ptr *p_IoctlPointer = NULL;
    atm_pos_ioctl_ptr_64 *p_IoctlPointer64 = NULL;
    uint32 s_ReadValue1, s_ReadValue2;

#if defined(__LINUX_USER__)
    uint32 s_Temp;
    uint32 s_Command;
    int s_Result;
#endif
    uint32 s_GetIoctlFlag = TRUE;
    uint32 s_RegAddress, s_PhyAddress;

    bb_Error_e ixfStatus;
    bb_ChipInfo_t ChipInfo;
    ulong sz = 0;
    bb_ChipSegment_t section;
    ushort version, revision;
    char dataBuf[64];
    uchar J0String[64];
    ushort len;
    char myArray[8];
    ixf6048_AtmCounters_t pAtm;  /* ATM Counters */
    ixf6048_PosCounters_t pPos;  /* POS Counters */
    ixf6048_ChanMode_e pChanMode;
    ixf6048_InClkStatus_t pClkStatus;

    section.block.block = 0;
    section.block.side = 0;
    section.block.flow = 0;
    section.group.channel = 0;
    section.indirect.slot = 0;
    section.value = 0;

    s_ReturnValue = SUCCESS;
    /**
     *  Check whether the void pointer is not NULL
     */
    MESSAGE("atm_pos_get_config_handle function is invoked\n",0,0,0,0,0,0);
    MESSAGE("ADDRESS OF IOCTL_PTR is %u\n",(uint32)arg_pIoctlPtr64,0,0,0,0,0);

    if(arg_pIoctlPtr64 == NULL)
    {
        MESSAGE("IOCTL_BUFFER_POINTER_NULL\n",0,0,0,0,0,0);
        s_GetIoctlFlag = FALSE;
        s_ReturnValue = IOCTL_BUFFER_POINTER_NULL;
    }
    else
    {
        MESSAGE("IOCTL_BUFFER Pointer is not NULL\n",0,0,0,0,0,0);
    }

    p_IoctlPointer64 = (atm_pos_ioctl_ptr_64 *)arg_pIoctlPtr64;
    MESSAGE("IOCTL_COMMAND is for getting 32-bit value only\n",0,0,0,0,0,0);

    /**
     *  Check for port state.
     */
    if(s_GetIoctlFlag)
    {
        if(atmPosGlobalData.g_PortState[p_IoctlPointer64->portIndex] == CLOSED)
        {
            MESSAGE("IOCTL_PORT_NOT_OPEN\n",0,0,0,0,0,0);
            s_GetIoctlFlag = FALSE;
            s_ReturnValue = IOCTL_PORT_NOT_OPEN;
        }
    }

    if(s_GetIoctlFlag)
    {
        MESSAGE("GET IOCTL_COMMAND is getting executed\n",0,0,0,0,0,0);
        switch(arg_cmd)
        {
		case GET_STATUS:
			printk("GET_STATUS\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiGetStatus(&pChipData, &section, ixf6048_eIN_CLK_STATUS, &pClkStatus);
			/* (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_SelStatus_e selStatus, void *pStatus); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetStatus\n");
			else
			{
				p_IoctlPointer64->valueLow = pClkStatus.Icmr1Reg;
				p_IoctlPointer64->valueHigh = pClkStatus.Icmr2Reg;
			}
			break;

		case GET_OPMODE:
			printk("GET_OPMODE\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiGetOpMode(&pChipData, &section, CONFIG_MODE, &pChanMode);
			/* IxfApiGetOpMode (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_OperMode_e * opMode, void *pModeCfg); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetOpMode\n");
			else
			{
				p_IoctlPointer64->valueLow = pChanMode;
			}
			break;

		case GET_COUNTERS:
			printk("GET_COUNTERS\n");
			if(OPT_POS_MODE == Card_Mode)
			{
				ixfStatus = IxfApiGetCounters (&pChipData, &section, bb_POS_COUNTERS, &pPos);
			}
			else // (OPT_ATM_MODE == Card_Mode)
			{
				ixfStatus = IxfApiGetCounters (&pChipData, &section, bb_ATM_COUNTERS, &pAtm);
			}
			/* IxfApiGetCounters (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_SelCounters_e SelCounters, void *pCounters); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetCounters\n");
			else
			{
			/* printk("bb_ATM_COUNTERS: %x, %x, %x, %x, %x, %x, %x\n", pAtm.RxCellCnt,
				pAtm.RxIdleCellCnt, pAtm.RxCorHecErrCnt, pAtm.RxUncorHecErrCnt,
				pAtm.RxOvflCellCnt, pAtm.TxCellCnt, pAtm.TxIdleCellCnt);
			*/
				if(OPT_POS_MODE == Card_Mode)
				{
					p_IoctlPointer64->valueLow = pPos.RxFrameCnt;
					p_IoctlPointer64->valueHigh = pPos.TxFrameCnt;
				}
				else // (OPT_ATM_MODE == Card_Mode)
				{
					p_IoctlPointer64->valueLow = pAtm.RxCellCnt;
					p_IoctlPointer64->valueHigh = pAtm.TxCellCnt;
				}
			}
			break;

		case GET_CHIP_INFO:
			printk("GET_CHIP_INFO\n");
			ixfStatus = IxfApiGetChipInfo(&pChipData, &ChipInfo);
			/* IxfApiGetChipInfo (bb_ChipData_t * pChipData, bb_ChipInfo_t * pChipInfo) */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetChipInfo\n");
			else
			{
				p_IoctlPointer64->valueLow = ChipInfo.ChipId & 0xF;
				p_IoctlPointer64->valueHigh = ChipInfo.ChipVersion;
			}
			break;

		case GET_BUILD_VERSION:
			printk("GET_BUILD_VERSION\n");
			ixfStatus = IxfApiGetBuildVersion (&pChipData, bb_SW_VERSION_CHIP_TYPE,
							   dataBuf, myArray, &version, &revision);
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetBuildVersion\n");
			else
			{
				p_IoctlPointer64->valueLow = version;
				p_IoctlPointer64->valueHigh = revision;
				printk("%s\n", dataBuf);
			}
			/* IxfApiGetBuildVersion (bb_ChipData_t * pChipData, bb_SoftwareVersion_e swType,
			char *drvName, char *date, ushort * buildVer, ushort * buildRev); */
			break;

		case GET_WINDOW_SIZE:
			printk("GET_WINDOW_SIZE\n");
			/* Get WindowSize */
			ixfStatus = IxfApiGetWindowSize (&pChipData, &section, CLEARING, &sz);
			/* bb_Error_e IxfApiGetWindowSize (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_WindowSizeMode_e mode, ulong * value); */

			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetWindowSize\n");
			else
			{
				p_IoctlPointer64->valueLow = sz;
			}
			break;

		case GET_TRACE:
			printk("GET_TRACE\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiGetTrace(&pChipData, &section, bb_EXPECTED_J0,
						   (char *) J0String, &len);
			printk("%s\n", J0String);
			/* IxfApiGetTrace (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_TraceType_e TraceType, char *pTrace, ushort * pLength); */
			break;

		case GET_OH_BYTES:
			printk("GET_OH_BYTES\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiGetOhBytes (&pChipData, &section, bb_EXPECTED_C2, myArray);
			/* IxfApiGetOhBytes (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_SelOhBytes_e SelOhBytes, void *pOhBytes); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetOhBytes\n");
			else
			{
				p_IoctlPointer64->valueLow = myArray[0];
			}
			break;

		case GET_CFG:
			printk("GET_CFG is not supported\n");
			IxfApiGetCfg (&pChipData, &section, e_GLOBAL_CONFIG);
			/* IxfApiGetCfg (bb_ChipData_t * pChipData,
					 bb_ChipSegment_t * ptSegment, bb_SelConfig_e SelCfg); */
			break;

		case GET_RMST_SOURCE_BYTES:
			printk("GET_RMST_SOURCE_BYTES is not supported\n");
			/* IxfGetRmstSourceBytes_t (bb_ChipData_t * pChipData, bb_ChipSegment_t * section); */
			break;

		case GET_MAPPED_FRAME:
			printk("GET_MAPPED_FRAME is not supported\n");
			/* IxfApiGetMappedFrames (bb_ChipData_t * pChipData, bb_SignalType_e sigType,
			ulong src, ulong destFrameNum, ulong numOfFrames, ushort * address); */
			break;

		case GET_AU_POINTER:
			printk("GET_AU_POINTER is not supported\n");
			/* IxfApiGetFramesAuPointer (bb_ChipData_t * pChipData, ulong src,
				bb_SignalType_e sigType, ulong srcFrameNum,
				ulong numOfFrames, ushort * AuPtrValue); */
			break;

		default:
			printk("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
			s_GetIoctlFlag = FALSE;
			s_ReturnValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
			break;

	};
    }

    if(s_GetIoctlFlag)
    {
        MESSAGE("atm_pos_get_config_handle is executed successfully\n",0,0,0,0,0,0);
    }
    else
    {
        MESSAGE("atm_pos_get_config_handle coult NOT executed\n",0,0,0,0,0,0);
    }

    return s_ReturnValue;
}
