/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_set_ioctl.c
 *
 * = DESCRIPTION
 *      This file implements the SET ioctl commands, which are used to
 *      configure the various registers apart from the regular config
 *      routine.
 *
 * = CHANGE HISTORY
 *       2/25/2003 14:13:24 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf6048_set_ioctl.c,v 1.15 2003/06/10 00:31:45 cmah1 Exp $";


/**
 * System defined include files required.
 */
#include <linux/kernel.h>
#include <stdio.h>
#include <asm/io.h>

typedef int sint;
typedef short sshort;
typedef char schar;
typedef unsigned char uchar;
typedef unsigned char boolean;  /* used in function calls */
/**
 * User defined include files required.
 */
#include "common_def.h"
#include "ixf_api_d.h"          /* BroadBand Utility Definitions and Function headers */
#include "ixf_api.h"
#include "ixf_api_sonet_d.h"
#include "ixf_api_sonet.h"
#include "ixf6048b.h"
#include "ixf6048d.h"
#include "ixf6048_ioctl.h"
#include "ixf6048_internal.h"
#include "ixf6048_linux_driver.h"

#ifdef TEST_WITH_UE
#include "uc_load.h"
#include "bd_atm_tx.h"
#include "bw_atm_tx.h"
#include "eg_bd_pos_tx.h"
#include "eg_bw_pos_tx.h"
#endif

#include "ix_cc_atm_pos_tx_ixf6048_regs.h"
#include "ixf6048_ext_lpbk.h"

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
extern atm_pos_global_data atmPosGlobalData;
extern bb_ChipData_t pChipData;
extern int Media_Card;
extern int Card_Mode;

extern void Set_SP(uint32, uint32, uint32, uint32, uint32, uint32, uint32, uint32);
extern uint32 sp_reg[];

uint32* UE_CSR_Base;
/**
 * Function definitions.
 */
atm_pos_error atm_pos_set_config_handler(
                                         atm_pos_IOCTL_CMD arg_cmd,
                                         void *arg_pIoctlPtr64
                                        )
{
    atm_pos_ioctl_ptr *p_IoctlPointer = NULL;
    atm_pos_ioctl_ptr_64 *p_IoctlPointer64 = NULL;
    uint32 s_SetIoctlFlag = TRUE;              /* Flag reset to 0, when an error occurs. */
    uint32 s_Count;
    uint32 mask=0;

    static uint32 s_ReturnValue, s_WriteValue, s_ReadValue;
    static uint32 s_ReadValue1, s_ReadValue2;
    static uint32 s_WriteHigh, s_WriteLow;
    static uint32 s_GetPortNumber, s_GetStatus, s_PhyMode, s_PortCount;
    static uint32 s_Temp;
    uint32 s_PhyAddress,s_ConfigValue, s_RegAddress;

    uint32 s_ResourceCount=0;

    bb_Error_e ixfStatus;
    bb_ChipInfo_t ChipInfo;
    ulong set_sz = 0;
    bb_ChipSegment_t section;
    char myArray[512];
    ushort temp1;
    ushort temp2;
    char dataBuf[64];
    uchar J0String[64];
    ushort len;
    char string64[] = "Expected J0 String that is longer than 64 bytes in length and should be truncated ";
    ixf6048_ChanMode_e pChanMode;
    ixf6048_AtmChanAlarmCfg_t pAtmChanAlms;
    char ReadBuf[8];
    char WriteBuf[8];

    section.block.block = 0;
    section.block.side = 0;
    section.block.flow = 0;
    section.group.channel = 0;
    section.indirect.slot = 0;
    section.value = 0;

    s_ReturnValue = SUCCESS;

    MESSAGE("IOCTL API calls atm_pos_set_config_handler for SET type IOCTL Commands\n",0,0,0,0,0,0);

    /**
     *  Check whether the void pointer is not NULL
     */
    if(arg_pIoctlPtr64 == NULL)
    { /* passed Ioctl Pointer is NULL */
        MESSAGE("IOCTL buffer pointer NULL\n",0,0,0,0,0,0);
        s_SetIoctlFlag = FALSE;
        s_ReturnValue = IOCTL_BUFFER_POINTER_NULL;
    } /* end-if arg_pIoctlPtr64 */

    p_IoctlPointer64 = (atm_pos_ioctl_ptr_64 *)arg_pIoctlPtr64;

    /**
     *  Check for the Validity of the void pointer.
     */
    if(s_SetIoctlFlag)
    { /* if Ioctl Pointer is not NULL */
        p_IoctlPointer = (atm_pos_ioctl_ptr *)arg_pIoctlPtr64;
        MESSAGE("IOCTL_BUFFER_POINTER_VALID\n",0,0,0,0,0,0);      /* corresponding checking for the digit */
    } /* Ioctl Pointer is valid */


    /**
     *  Check for port state.
     */
    if(s_SetIoctlFlag)
    {
        /**
         * Copy the value to be written in the register in the local register, s_WriteValue.
         */
        s_WriteValue = p_IoctlPointer->value;

	s_ResourceCount++;
	switch(arg_cmd)
	{
		/**
		* Configurs register associated with each port.
		*/
		case SET_OPMODE:
			printk("SET_OPMODE\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			section.block.flow == BOTH_FLOWS;
			pChanMode = p_IoctlPointer64->valueLow;
			if(p_IoctlPointer64->valueLow == ixf6048_bCHAN_MODE_TEST)
			{
				ixfStatus = IxfApiSetOpMode (&pChipData, &section, TEST_MODE, &pChanMode);
			}
			else if(p_IoctlPointer64->valueLow == ixf6048_bCHAN_MODE_POS)
			{
				ixfStatus = IxfApiSetOpMode (&pChipData, &section, POS_MODE, &pChanMode);
			}
			else if(p_IoctlPointer64->valueLow == ixf6048_bCHAN_MODE_ATM)
			{
				ixfStatus = IxfApiSetOpMode (&pChipData, &section, ATM_MODE, &pChanMode);
			}
			else if(p_IoctlPointer64->valueLow == ixf6048_bCHAN_MODE_TRANSPARENT)
			{
				ixfStatus = IxfApiSetOpMode (&pChipData, &section, TRANSPARENT_MODE, &pChanMode);
			}
			else
			{
				printk("ERROR: Unsupported Mode\n");
			}
			/* IxfApiSetOpMode (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_OperMode_e opMode, void *pModeCfg); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiSetOpMode\n");
			break;

		case SET_CHIP_ONLINE:
			printk("SET_CHIP_ONLINE\n");
			ixfStatus = IxfApiSetChipOnline(&pChipData, &section);
			/* IxfApiSetChipOnline (bb_ChipData_t * pChipData, bb_ChipSegment_t * section); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiSetChipOnline\n");
			break;

		case SET_CHIP_OFFLINE:
			printk("SET_CHIP_OFFLINE\n");
			ixfStatus = IxfApiSetChipOffline (&pChipData, &section);
			/* IxfApiSetChipOffline (bb_ChipData_t * pChipData, bb_ChipSegment_t * section); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiSetChipOffline\n");
			break;

		case SET_ALARM_CFG:
			printk("SET_ALARM_CFG\n");
			pAtmChanAlms.RxAlarms = p_IoctlPointer64->valueLow;
			pAtmChanAlms.TxAlarms = p_IoctlPointer64->valueHigh;
			ixfStatus = IxfApiSetAlarmCfg (&pChipData, &section, bb_6048_ATM_ALARM, &pAtmChanAlms);
			/* IxfApiSetAlarmCfg (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
			bb_AlarmType_e AlarmType, void *pAlarmCfg); */
			break;

		case SET_WINDOW_SIZE:
			printk("SET_WINDOW_SIZE\n");
			ixfStatus = IxfApiSetWindowSize (&pChipData, &section, CLEARING, p_IoctlPointer64->valueLow);
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiGetWindowSize\n");
			break;

		case SET_TRACE:
			printk("SET_TRACE\n");
			ixfStatus = IxfApiAllocDataStructureMem (&pChipData);
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiAllocDataStructureMem\n");
			ixfStatus = IxfApiDeAllocMemory (&pChipData);
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiDeAllocMemory\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiSetTrace (&pChipData, &section, bb_EXPECTED_J0, bb_64_BYTE_WITH_LF_CR, string64);
			/* IxfApiSetTrace (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
				bb_TraceType_e TraceType, bb_TraceFormat_e TraceFormat, char *pTrace); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiSetTrace\n");
			break;

		case SET_OH_BYTES:
			printk("SET_OH_BYTES\n");
			section.group.channel = p_IoctlPointer64->portIndex;
			ixfStatus = IxfApiSetOhByte (&pChipData, &section, bb_EXPECTED_C2, p_IoctlPointer64->valueLow);
			/* IxfApiSetOhByte (bb_ChipData_t * pChipData, bb_ChipSegment_t * section,
				bb_SelOhBytes_e SelOhByte, ushort OhByte); */
			if (ixfStatus != bb_NO_ERROR)
				printk("ERROR: At IxfApiSetOhByte\n");
			break;

		case GENERIC_READ:
			memset (ReadBuf, '0', 8);
			ixfStatus = IxfApiGenericRead (&pChipData, 1, p_IoctlPointer64->valueHigh,
							1, ReadBuf);
			/* IxfApiGenericRead (bb_ChipData_t * pChipData, bb_Word_Size_t wordSize,
					ulong address, ushort length, void *buffer); */
			if (ixfStatus != bb_NO_ERROR)
			{
				printk ("IxfApiGenericRead FAILED!!!\n");
			}
			else
			{
				p_IoctlPointer64->valueLow  = ReadBuf[0];
			}

			break;

		case GENERIC_WRITE:
			WriteBuf[0] = p_IoctlPointer64->valueLow;

			ixfStatus = IxfApiGenericWrite (&pChipData, 1, p_IoctlPointer64->valueHigh,
							1, WriteBuf);
			/* IxfApiGenericWrite (bb_ChipData_t * pChipData, bb_Word_Size_t wordSize,
					ulong address, ushort length, void *buffer); */
			if (ixfStatus != bb_NO_ERROR)
			{
				printk ("IxfApiGenericWrite FAILED!!!\n");
			}
			break;

		case SET_CARD_MODE:
			printk("SET_CARD_MODE\n");

			if(BDREW_CARD == p_IoctlPointer64->valueHigh)	// Media_Card
			{
				Media_Card=BDREW_CARD;
				if(OPT_POS_MODE == p_IoctlPointer64->valueLow)
				{
					Card_Mode=OPT_POS_MODE;
					IxfApiInitChip (&pChipData, (InitRegTable_t *)RegInitIXF6048TablePosMode0);
					printk("Media_Card: Battery Drew; Card_Mode: POS\n");
				}
				else if(OPT_ATM_MODE == p_IoctlPointer64->valueLow)
				{
					Card_Mode=OPT_ATM_MODE;
					IxfApiInitChip (&pChipData, (InitRegTable_t *)BD_ExtLoopbackArray_ATM);
					printk("Media_Card: Battery Drew; Card_Mode: ATM\n");
				}
				else
				{
					printk("***ERROR***: Invalid mode selected\n");
					printk("    Card_Mode=0: POS\n");
					printk("    Card_Mode=1: ATM\n");
					return 1;
				}
			}
			else if(BWALLACE_CARD == p_IoctlPointer64->valueHigh)	// Media_Card
			{
				Media_Card=BWALLACE_CARD;
				if(OPT_POS_MODE == p_IoctlPointer64->valueLow)
				{
					Card_Mode=OPT_POS_MODE;
					IxfApiInitChip (&pChipData, (InitRegTable_t *)BW_ExtLoopbackArray);
					printk("Media_Card: Battery Wallace; Card_Mode: POS\n");
				}
				else if(OPT_ATM_MODE == p_IoctlPointer64->valueLow)
				{
					Card_Mode=OPT_ATM_MODE;
					IxfApiInitChip (&pChipData, (InitRegTable_t *)BW_ExtLoopbackArray_ATM);
					printk("Media_Card: Battery Wallace; Card_Mode: ATM\n");
				}
				else
				{
					printk("***ERROR***: Invalid mode selected\n");
					printk("    Card_Mode=0: POS\n");
					printk("    Card_Mode=1: ATM\n");
					return 1;
				}
			}

			break;

#ifdef TEST_WITH_UE
		case SET_ME_TX:
			printk("SET_ME_TX\n");
			UE_CSR_Base = (unsigned int *) __ioremap((unsigned long)0xc0018000, 0x5000, 0);
			if(BWALLACE_CARD == Media_Card)
			{
				if(OPT_POS_MODE == Card_Mode)
				{
					DownLoadUcode(1, sizeof(eg_bw_pos_tx), (uint32 *)eg_bw_pos_tx);
				}
				else // (OPT_ATM_MODE == Card_Mode)
				{
					DownLoadUcode(1, sizeof(bw_atm_tx), (uint32 *)bw_atm_tx);
				}
			}
			else // (BDREW_CARD == Media_Card)
			{
				if(OPT_POS_MODE == Card_Mode)
				{
					DownLoadUcode(1, sizeof(eg_bd_pos_tx), (uint32 *)eg_bd_pos_tx);
				}
				else // (OPT_ATM_MODE == Card_Mode)
				{
					DownLoadUcode(1, sizeof(bd_atm_tx), (uint32 *)bd_atm_tx);
				}
			}
			udelay(10000);
			Clear_All_ME();
			iounmap (UE_CSR_Base);
			break;
#endif
		case SET_CFG:
			printk("SET_CFG is not supported\n");
			IxfApiSetCfg (&pChipData, &section, e_GLOBAL_CONFIG);
			/* IxfApiSetCfg (bb_ChipData_t * pChipData, bb_ChipSegment_t * ptSegment, bb_SelConfig_e SelCfg); */
			break;

		case SET_RMST_SOURCE_BYTES:
			printk("SET_RMST_SOURCE_BYTES is not supported\n");
			/* IxfSetRmstSourceBytes_t (bb_ChipData_t * pChipData, bb_ChipSegment_t * section); */
			break;

		default:
			printk("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
			s_ReturnValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
			break;
	}; /* end-switch case */
    } /* end-if s_SetIoctlFlag Flag */

    MESSAGE("atm_pos_set_config_handler() routines completes execution.\n",0,0,0,0,0,0);
    return s_ReturnValue;
}

