/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: nmafzal $
 *   DATE:           $Date: 2002/11/20 18:21:52 $
 *   LAST MODIFIED:  $Modtime: 1/23/02 2:21p $
 *   REV#:           $Revision: 1.2 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/hal.h $
 *   WORKFILE:       $Workfile: hal.h $
 *
 * --------------------------------------------------------------------------
 * Project: ALL
 *
 * Purpose: Contains the macros for the hardware abstraction layer.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: hal.h,v $
 *   Revision 1.2  2002/11/20 18:21:52  nmafzal
 *   Made B0 changes to work on A1 and B0 board. The, basically swapping is not needed in B0 NPU
 *
 *   Revision 1.1.1.1  2002/10/31 00:56:05  nmafzal
 *   putting it in proper place
 *
 *   Revision 1.1  2002/10/30 13:00:01  machambe
 *   Initial CVS checkin
 *
 * 
 * 10    1/23/02 5:44p Rsood
 * Added macro for Simulator
 * 
 * 9     1/14/02 7:02p Machambe
 * Removing Processor directives and FORCED_ERR_JF
 * 
 * 8     1/09/02 4:47p Jjfreema
 * Added code for Froced Error Program by Joshua Freeman.
 * 
 * 7     12/14/01 6:51p Psnahlou
 * 
 * 5     11/16/01 11:16p Machambe
 * 
 * 4     11/06/01 10:42a Machambe
 * Added support for SA1100 nonaligned .
 * 
 * 3     7/03/01 4:09p Machambe
 * 
 * 2     5/03/01 3:54p Machambe
 * 
 * --------------------------------------------------------------------------
 */

#ifndef __HW_IF_H_              /* Include file contents only once per compilation */
#define __HW_IF_H_

#ifdef SIMULATOR                /* SIMULATOR */

#define READ_FROM_HW(addr) (SimulatorRead(addr))
#define WRITE_TO_HW(addr, value) (SimulatorWrite(addr, value))

#elif SLOW_PORT             /* Fixes Slow Port A0/A1 silicon issue */

#define READ_FROM_HW(addr) (SlowPortRead(addr))
#define WRITE_TO_HW(addr, value) (SlowPortWrite(addr,value))

#elif CODE_GEN                  /* Code Generation */

extern ushort CodeGenRead (bb_RegPointer_type addr);
extern ushort CodeGenWrite (bb_RegPointer_type addr, ushort val);
#define READ_FROM_HW(addr) (CodeGenRead(addr))
#define WRITE_TO_HW(addr, value) (CodeGenWrite(addr,value))

#elif SOCKET_COMM

extern ushort SocketCommRead (bb_RegPointer_type addr);
extern ushort SocketCommWrite (bb_RegPointer_type addr, ushort value);
#define READ_FROM_HW(addr) (SocketCommRead(addr))
#define WRITE_TO_HW(addr, value) (SocketCommWrite(addr, value))

#else /* Default - Memory mapped device */

#define READ_FROM_HW(addr) ((*(addr)))
#define WRITE_TO_HW(addr, value) ((*(addr)) = (value))

#endif




#endif /* __HW_IF_H_ */
