/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: nmafzal $
 *   DATE:           $Date: 2002/10/31 00:56:05 $
 *   LAST MODIFIED:  $Modtime: 5/17/02 1:58p $
 *   REV#:           $Revision: 1.1.1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf6048/ixf6048d.h $
 *   WORKFILE:       $Workfile: ixf6048d.h $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6048
 *
 * Purpose: Provides the data structures for Ixf6048
 *			
 * Notes:
 *    This File is used to Generate Source Code and therefore the 
 *    following Rules MUST be followed when defining enums and Structures:
 *
 *    Enums:
 *   
 *    typedef enum                  Must be Line 1 
 *    {                             Open Bracket On a Line by itself -- Comments NOT allowed
 *          TheFirstEnum = 0,       = 0,1... is optional, and comments are OK
 *          ThenNextEnum,           Comma delineates the values
 *             ...
 *          TheLastEnumValue        The LastLine Comma is optional
 *    } TheEnumName;                TheName MUST be on the Same Line as Close Parenthesis
 *	  -- Comments MUST NOT On the Open Parenthesis line!
 *    -- Comments may traverse multiple lines!
 *    -- Blank Lines Are Allowed
 *    -- The Open and Close Parentheses MUST be in the First Position on the Line.
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: ixf6048d.h,v $
 *   Revision 1.1.1.1  2002/10/31 00:56:05  nmafzal
 *   putting it in proper place
 *
 *   Revision 1.1  2002/10/30 13:02:23  machambe
 *   Initial CVS checkin
 *
 *   Revision 1.2  2002/10/27 18:21:36  machambe
 *   no message
 *
 *   Revision 1.2  2002/10/14 09:41:20  machambe
 *   no message
 */

#ifndef BB6048D_H_              /* Include file contents only once per compilation */
#define BB6048D_H_

/***********************************************************************/
#ifdef AMAZON_BORLAND_UI
#include "bb_util.h"            //LC -1/3/02/ -needed to include this file
#include "oht_data.h"
#endif
/************************************************************************/

/* Global Constants */
#define MAX_NUM_CHANNELS 4

/***********************************************************************
 * Chip Configuration structures
 **********************************************************************/
/* PSN_  -- Comment Added Here
	MACNF Bit[2]
*/
typedef enum
{
  ixf6048_eBYTE_CHG_UPDATE_ENABLE = 0,
  ixf6048_eBYTE_CHG_UPDATE_DISABLE
}
ixf6048_ByteChgUpdDsbl_e;

/* PSN_ADDED B1 Release Updates
----------- GOCNF Config Bits For B2 Release ----------------*/
/* GOCNF Bit[13] */
typedef enum
{
  ixf6048_eGEN_IO_VAL_LOW = 0,
  ixf6048_eGEN_IO_VAL_HIGH
}
ixf6048_GenIOVal_e;

/* GOCNF Bit[12] */
typedef enum
{
  ixf6048_eGEN_IO_MODE_INPUT = 0,
  ixf6048_eGEN_IO_MODE_OUTPUT
}
ixf6048_GenIOMode_e;

/* GOCNF Bit[9] GOCNF Bit[8]
	 XMT and RCV PECL MSB Config */
typedef enum
{
  ixf6048_ePECL_MSB_CNF_BIT_PN_15 = 0,
  ixf6048_ePECL_MSB_CNF_BIT_PN_0
}
ixf6048_PeclMsbCnf_e;

/* PSN_END Added Bits */
/*------------------ UTOPIA Config -----------------------------------*/
/* PSN_ADDED B1 Release Updates
----------- GOCNF Config Bits For B2 Release ----------------*/
/* GOCNF Bit[11] GOCNF Bit[10]
	XmtUOutEnCnf and RcvUOutEnCnf */
typedef enum
{
  ixf6048_eUTOPIA_OUT_EN_CNF_TRISTATE_IF_UOEN_EQ_0 = 0,
  ixf6048_eUTOPIA_OUT_EN_CNF_NOT_DISABLED_INDEPENDANTLY
}
ixf6048_UOutEnCnf_e;
/* PSN_END Added Bits */

/* UTOPIA interface - See compatability table Pg 10
         Note: Used for Rx and Tx UTOPIA interface
         See reg  GOCNF Bit 7 for U64Mode
         See regs R_COCNF Bits[4:2], 
                  T_COCNF Bits[4:2]
         and R_UICNF, T_UICNF      */
typedef enum
{
  ixf6048_eUTOPIA_L3_64_BIT_USING_TTL_PINS = 1, /* U64Mode = 0 */
  ixf6048_eUTOPIA_L3_64_BIT_USING_SOH_PINS,     /* U64Mode = 1 */
  /* Level 3 with 64-bit data bus, using either
     SOH or TTL pins, for extra UTOPIA pins */
  ixf6048_eUTOPIA_L3_32_BIT,    /* Level 3 with 32-bit data bus */
  ixf6048_eUTOPIA_L3_4_X_8_BIT, /* Level 3 with 4 independent 8-bit buses */
  ixf6048_eUTOPIA_L2_16_BIT,    /* Level 2 with 16-bit data bus */
  ixf6048_eUTOPIA_L2_4_X_16_BIT,        /* Level 2 with 4 independant 16-bit data buses */
  ixf6048_eUTOPIA_L2_4_X_8_BIT, /* Level 2 with 4 independant 8-bit data buses */
  ixf6048_eUTOPIA_L2_8_BIT,     /* Level 2 with 8-bit data bus */
  ixf6048_eUTOPIA_L1_4_X_8_BIT  /* Level 1 with 4 independent 8-bit buses */
}
ixf6048_UtopiaInterface_e;

/* Single/Multiple Devices on PHY - Regs R_UICNF, T_UICNF bit 3
      Note:
         o  For Single   UTOPIA config - RXFA, TXFA always driven.
         o  For Multiple UTOPIA config - RXFA, TXFA only driven
            when RXADDR, TXADDR  matches the device address.
         o  For Multi-PHY  h/w config - must use MULTIPLE spec
         o  For Single-PHY h/w config - can use either spec. */
typedef enum
{
  ixf6048_eSINGLE_UTOPIA_DEV = 0,       /* Single UTOPIA device on PHY */
  ixf6048_eMULTIPLE_UTOPIA_DEVS /* Multiple UTOPIA devices on PHY */
}
ixf6048_UtopiaMultiPhy_e;

/* Configuration of the Receive interface as follows 
      -- When Amazon-A Shares the Rx interface, The Rx outputs MUST only be 
         driven when Selected. 
      -- When Amazon-a is the only device, then the output CAN be always be driven.
      -- R_UICNF, T_UICNF Bit 4 */

typedef enum
{
  ixf6048_eOUTPUT_ALWAYS_DRIVEN = 0,    /* CAN be used when Single device */
  ixf6048_eOUTPUT_ONLY_DRIVEN_WHEN_SELECTED     /* MUST be used when Interface is Shared */
}
ixf6048_MPhyDevCnf_e;


/* Config for the Extra HEC byte when in 8-Bit mode -- Reg R_UICNF Bit10 */
typedef enum
{
  ixf6048_eEXTRA_BYTE_IS_RX_BYTE = 0,   /* The Received HEC is passed as the Extra Byte */
  ixf6048_eEXTRA_BYTE_IS_STATUS,        /* 2 Status Bits are passed -- Bit0 and Bit 1 */
  ixf6048_eNOT_IN_8_BIT_MODE    /* This bit is ignored -- Place holder only */
}
ixf6048_RcvAtmHecConfig_e;

/* Config to reduce the size of the Rx/Tx UTOPIA FIFO memory used by Channel 0
         -- R_UICNF Bit 9, T_UICNF Bit 7 */
typedef enum
{
  ixf6048_eSMALL_MEM_CHAN_0_FIFO_16K = 0,       /* Chan 0 Fifo is 16k Bytes */
  ixf6048_eSMALL_MEM_CHAN_0_FIFO_2K     /* Chan 0 Fifo is 2k Bytes */
}
ixf6048_UtopiaSmallMemCfg_e;
/* Configure whether impedance for unused UTOPIA signals
                  is held high, or not.; e.g. POS signals in ATM mode,
                  or Quad channel signals for Single Channel mode
                  - Reg R_UICNF bit 8 */
typedef enum
{
  ixf6048_eUNUSED_UTOPIA_PINS_HIGH_IMP = 0,
  ixf6048_eUNUSED_UTOPIA_PINS_NOT_HIGH_IMP
}
ixf6048_UnusedUtopiaPinsImp_e;

/* OrderWire Pulse byte configuration - OHPCNF bit 1 */
typedef enum
{
  ixf6048_eMSB_SAME_CLK_CYCLE_AS_PULSE = 0,
  ixf6048_eLSB_SAME_CLK_CYCLE_AS_PULSE
}
ixf6048_OrderWirePulseByteLocn_e;

/* Overhead Port mode configuration - OHPCNF bit 0. 
   This option is only used in Quad Mode */
typedef enum
{
  ixf6048_eOH_PORT_LOGICAL_INTERFACE_NUM2 = 0,
  ixf6048_eOH_PORT_LOGICAL_INTERFACE_NUM3
}
ixf6048_OverheadPortMode_e;


/* Input Clock Polarity Config - Set 1 - ICPCNF1 */
typedef ushort ixf6048_InClockPolarity1_e;

#define	ixf6048_eTCI_T3_INVERTED   0x8000
#define ixf6048_eTCI_T2_INVERTED   0x4000
#define ixf6048_eTCI_T1_INVERTED   0x2000
#define ixf6048_eTCI_T0_INVERTED   0x1000

#define ixf6048_eRCI_T3_INVERTED   0x0800
#define ixf6048_eRCI_T2_INVERTED   0x0400
#define ixf6048_eRCI_T1_INVERTED   0x0200
#define ixf6048_eRCI_T0_INVERTED   0x0100

#define ixf6048_eTSCI_P3_INVERTED  0x0080
#define ixf6048_eTSCI_P2_INVERTED  0x0040
#define ixf6048_eTSCI_P1_INVERTED  0x0020
#define ixf6048_eTSCI_P0_INVERTED  0x0010

#define ixf6048_eRSCI_P3_INVERTED  0x0008
#define ixf6048_eRSCI_P2_INVERTED  0x0004
#define ixf6048_eRSCI_P1_INVERTED  0x0002
#define ixf6048_eRSCI_P0_INVERTED  0x0001

/* Input Clock Polarity Config - Set 2 - ICPCNF2 */
typedef ushort ixf6048_InClockPolarity2_e;
#define ixf6048_eTPCI_P_INVERTED 0x0002
#define ixf6048_eRPCI_P_INVERTED 0x0001

/* Output Clock Polarity Config - OCPCNF */
typedef ushort ixf6048_OutClockPolarity_e;
#define ixf6048_eTCO_T3_INVERTED  0x8000
#define ixf6048_eTCO_T2_INVERTED  0x4000
#define ixf6048_eTCO_T1_INVERTED  0x2000
#define ixf6048_eTCO_T0_INVERTED  0x1000

#define ixf6048_eRCO_T3_INVERTED  0x0800
#define ixf6048_eRCO_T2_INVERTED  0x0400
#define ixf6048_eRCO_T1_INVERTED  0x0200
#define ixf6048_eRCO_T0_INVERTED  0x0100

#define ixf6048_eTSCO_P3_INVERTED 0x0080
#define ixf6048_eTSCO_P2_INVERTED 0x0040
#define ixf6048_eTSCO_P1_INVERTED 0x0020
#define ixf6048_eTSCO_P0_INVERTED 0x0010

#define ixf6048_eROWC_INVERTED	   0x0008
#define ixf6048_eTOWC_INVERTED     0x0004
/* ixf6048_eReserved               0x0002,   */
#define ixf6048_eTPCO_P_INVERTED   0x0001

/* Input Clock Monitoring Reg 1 - ICMR1 */
typedef ushort ixf6048_InClockMonitor1_e;
#define ixf6048_eCHG_TCI_T3_INVERTED 0x8000
#define ixf6048_eCHG_TCI_T2_INVERTED 0x4000
#define ixf6048_eCHG_TCI_T1_INVERTED 0x2000
#define ixf6048_eCHG_TCI_T0_INVERTED 0x1000

#define ixf6048_eCHG_RCI_T3_INVERTED 0x0800
#define ixf6048_eCHG_RCI_T2_INVERTED 0x0400
#define ixf6048_eCHG_RCI_T1_INVERTED 0x0200
#define ixf6048_eCHG_RCI_T0_INVERTED 0x0100

#define ixf6048_eCHG_TSCI_P3_INVERTED 0x0080
#define ixf6048_eCHG_TSCI_P2_INVERTED 0x0040
#define ixf6048_eCHG_TSCI_P1_INVERTED 0x0020
#define ixf6048_eCHG_TSCI_P0_INVERTED 0x0010

#define ixf6048_eCHG_RSCI_P3_INVERTED 0x0008
#define ixf6048_eCHG_RSCI_P2_INVERTED 0x0004
#define ixf6048_eCHG_RSCI_P1_INVERTED 0x0002
#define ixf6048_eCHG_RSCI_P0_INVERTED 0x0001

typedef ushort ixf6048_InClockMonitor2_e;       /* Input Clock Monitoring Reg 2 - ICMR2 */
#define ixf6048_eCHG_TXCLK_3_INVERTED 0x8000
#define ixf6048_eCHG_TXCLK_2_INVERTED 0x4000
#define ixf6048_eCHG_TXCLK_1_INVERTED 0x2000
#define ixf6048_eCHG_TXCLK_0_INVERTED 0x1000

#define ixf6048_eCHG_RXCLK_3_INVERTED 0x0800
#define ixf6048_eCHG_RXCLK_2_INVERTED 0x0400
#define ixf6048_eCHG_RXCLK_1_INVERTED 0x0200
#define ixf6048_eCHG_RXCLK_0_INVERTED 0x0100

#define ixf6048_eCHG_TPCI_P_INVERTED  0x0002
#define ixf6048_eCHG_RPCI_P_INVERTED  0x0001

/*  ------- Non-Concatenation Mode Register Configuration --------- */
/* Configuration for the C0 byte generation from the SONET/SDH Mux
   See Reg NCMODECNF Bit 4 */
typedef enum
{
  ixf6048_eC0_BYTES_NOT_OVER_WRITTEN = 0,
  ixf6048_eC0_BYTES_OVER_WRITTEN
}
ixf6048_J0MuxConfig_e;

/* Configuration for the way that the SONET/SDH Mux/Demux  
   interleaves/DeInterleaves the SONET/SDH Frames generated/received
   by each Tx Channel.
   See Reg NCMODECNF Bit 2, Bit 0 */
typedef enum
{
  ixf6048_eWORD_32_BIT_INTERLED = 0,
  ixf6048_eBYTE_INTERLEAVED
}
ixf6048_MuxDemuxConfig_e;

/* Parity Mode Configuration 
      -- LPSCNF Bit 2 */
typedef enum
{
  ixf6048_ePARITY_PROTECTS_PARALLEL_DATA_BUS_ONLY = 0,
  ixf6048_ePARITY_PROTECTS_PARALLEL_DATA_BUS_AND_FRAME_PULSE
}
ixf6048_ParityModeConfig_e;
/* Rx/Tx Mode Configuration 
      -- LPSCNF Bit 1 and Bit 0 */
typedef enum
{
  ixf6048_eODD_PARITY = 0,
  ixf6048_eEVEN_PARITY
}
ixf6048_ParityConfig_e;

/* Line side Parity Configuration 
      - Reg LSPCNF   */
typedef struct
{
  ixf6048_ParityModeConfig_e PrtyModeConfig;
  ixf6048_ParityConfig_e RxParityConfig;
  ixf6048_ParityConfig_e TxParityConfig;
}
ixf6048_LineSideParityConfig_t;

/* Non Concatenated Mode config Register 
   See Reg NCMODECNF Bits 0, 2, 4*/
typedef struct
{
  ixf6048_J0MuxConfig_e J0MuxConfig;
  ixf6048_MuxDemuxConfig_e BWMuxConfig;
  ixf6048_MuxDemuxConfig_e BWDeMuxConfig;
}
ixf6048_NonConcatModeCnfg_t;

/*------------------ POS-specific UTOPIA Config ----------------------------*/

/* PSN_ADDED for B1 Release */
/* R_UICNF Bit[11] Pos Mode Only 
	-- Block Reads on EOF, or if FIFO is Empty
	-- Receive side only  */
typedef enum
{
  ixf6048_eRCV_VAL2_CNF_READ_BLOCKED_IF_EOF_OR_FIFO_EMPTY = 0,
  ixf6048_eRCV_VAL2_CNF_READ_BLOCKED_IF_EOF_ONLY
}
ixf6048_RcvVal2Cnf_e;
/* PSN_END B1 RElease Changes */

/* POS-UTOPIA Config  - Regs R_UICNF, T_UICNF bit 0.
         (used for Rx & Tx interface).
         If like ATM interface, has data transfer and port selection
         processes running in parallel.
         If memory mapped, there is no selection cycle or FIFO status
         polling, just port addressing. */

typedef enum
{
  ixf6048_ePOS_UTOPIA_LIKE_ATM = 0,
  ixf6048_ePOS_UTOPIA_MEMORY_MAPPED
}
ixf6048_PosUtopiaSelMode_e;

/* RXFA,TXFA Status config  - Regs R_UICNF, T_UICNF bit 5
                  namely: Direct Status Indication mode, or
                          Multiplexed Status Polling */
typedef enum
{
  ixf6048_eFA_MULTIPLEXED_STATUS_POLLING = 0,
  ixf6048_eFA_DIRECT_STATUS_INDICATION
}
ixf6048_FaStatus_e;

/* RXVAL config for POS mode - Reg R_UICNF bit 7
         Note: Configures whether RXVAL is deasserted only when
         reading an empty FIFO, or also at end of each packet.
         Note: This would be useful to provide time for an attached
         packet switch to switch buffers, in between packets. */
typedef enum
{
  ixf6048_eRXVAL_DEASSERT_ON_FIFO_EMPTY = 0,
  ixf6048_eRXVAL_DEASSERT_ON_EOP
}
ixf6048_RxValDeassertAtEop_e;

/* RXFA config for POS mode - Reg R_UICNF bit 6
         Note: Configures how the RXFA (Frame Available) signal is
         deasserted when there are fewer words in the FIFO than
         specified by the Rx Programmable Watermark
         (BytesAvailForDataAvail in UtopiaPosChanCfg).
         Namely, if Frame Available is deasserted even if EOF present,
         or only when EOF not present. */
typedef enum
{
  ixf6048_eRXFA_DEASSERT_ON_LOW_IF_NO_EOF = 0,
  ixf6048_eRXFA_DEASSERT_ON_LOW_ALWAYS
}
ixf6048_RxFaDeassertOnLow_e;

/* Config for POS mode - Reg T_UICNF bit 6
         Note: When 0, The POS xmitter will start the Transmission 
         of a new packet if the FIFO contains an EOF OR the FIFO contains 
         a number of words equal or greater than the Transmit initiation 
         Min level (Reg T_UIIML)*/
typedef enum
{
  ixf6048_eTX_AT_EOF_OR_GT_MIN_LEVEL = 1,
  ixf6048_eTX_AT_GT_MIN_LEVEL_ONLY
}
ixf6048_TxEofEmpty_e;

/* ----------------------------------------------------------------------
   Notes for Utopia-specific Chip Configuration:
   This Structure contains the Chip-Level Utopia Configuration that are
   common to all 4 channels. This is divided into the Transmit and receive
   Sides.
   ---------------------------------------------------------------------- */
typedef struct                  /* Utopia-specific Chip Configuration */
{
  /* Rx Side Config */
/* PSN_ADDED B1 Release Updates */
  ixf6048_UOutEnCnf_e RcvUOutEnCnf;     /* GOCNF Bit[11] RcvUOutEnCnf B1 Release */
/* PSN_END Added Bits */
  ixf6048_UtopiaInterface_e RxUtopiaInterface;  /* R_UICNF Bit 3, GOCNF Bit[7] */
  ixf6048_RcvAtmHecConfig_e RcvAtmHecConfig;    /* R_UICNF Bit 10 - Only Valid in 
                                                   8 Bit Mode (RcvUWidth ='00') */
  ixf6048_UtopiaSmallMemCfg_e RxUtopiaSmallMemCfg;      /* R_UICNF Bit 9 */

  ixf6048_UtopiaMultiPhy_e RxUtopiaMultiPhy;    /* R_UICNF Bit 5 */
  ixf6048_MPhyDevCnf_e RcvMPhyDevCnf;   /* R_UICNF Bit 4 */
  ixf6048_UnusedUtopiaPinsImp_e UnusedUtopiaPinsImp;

  /* Tx Side Config */
/* PSN_ADDED B1 Release Updates */
  ixf6048_UOutEnCnf_e XmtUOutEnCnf;     /* GOCNF Bit[10] XmtUOutEnCnf B1 Release */
/* PSN_END Added Bits */
  ixf6048_UtopiaInterface_e TxUtopiaInterface;  /* T_UICNF Bit 3 */
  ixf6048_UtopiaSmallMemCfg_e TxUtopiaSmallMemCfg;      /* T_UICNF Bit 7 */

  ixf6048_UtopiaMultiPhy_e TxUtopiaMultiPhy;    /* T_UICNF Bit 5 */
  ixf6048_MPhyDevCnf_e XmtMPhyDevCnf;   /* T_UICNF Bit 4 */

  /* Common Setup */
  uchar UtopiaDevBaseAddress;   /* GOCNF Bits[2:0] */

}
ixf6048_UtopiaChipCfg_t;

/* ----------------------------------------------------------------------
   Notes for POS-specific Chip configuration below:
   The Pos specific Config describes the behaviour of the Utopia interface
   when in POS mode. 
   RxFifoRestartLevel: defines number of words in the Rx FIFO to restart
   accepting packets after an Rx FIFO overflow. See also reg R_UIIML bits 7:1.

   TxFifoRestartLevel: defines number of words in the Tx FIFO to restart
   transmitting packets, after a Tx FIFO underflow. See also Reg T_UIIML bits 7:1.

   For Rx/TxFifoRestartLevel:
      For Quad mode: for channels 0-3, user's value is quantized to (16N +1)
                     where N = 0..15; i.e. quantized value is 1,17,33, ... 241
      For Single Channel mode: For channel 0, user's value is quantized to
                     (16N + 1), where N = 0..63, i.e. quantized value
                     is 1,17,33, ... 1009
   ---------------------------------------------------------------------- */

typedef struct                  /* POS-specific Chip configuration */
{
  /* Rx side Cfg */
  ixf6048_PosUtopiaSelMode_e RxPosUtopiaSelMode;        /* For Rx */
  ixf6048_FaStatus_e RxFaStatus;        /* For Rx */
  ixf6048_RxValDeassertAtEop_e RxValDeassertAtEop;
  ixf6048_RxFaDeassertOnLow_e RxFaDeassertOnLow;
  ushort RxFifoRestartLevel;    /* See above */
/* PSN_ADDED for B1 Release */
  ixf6048_RcvVal2Cnf_e RxVal2Cnf;       /* R_UICNF Bit[11] Rx Mode Only */
/* PSN_END B1 RElease Changes */

  /* Transmit Cfg */
  ixf6048_PosUtopiaSelMode_e TxPosUtopiaSelMode;        /* For Tx */
  ixf6048_FaStatus_e TxFaStatus;        /* For Tx */
  ixf6048_TxEofEmpty_e TxEofEmpty;
  ushort TxFifoRestartLevel;    /* See above */
}
ixf6048_PosChipCfg_t;

/* ----------------------------------------------------------------------
   Notes for Chip configuration below:

   UtopiaDevBaseAddress: 3-bit base address of UTOPIA device, in the UTOPIA
   interface. Compared to RXADDR[4:2] and TXADDR[4:2]. See Reg GOCNF 2:0
   ---------------------------------------------------------------------- */
typedef struct                  /* Amazon Chip configuration */
{
  ixf6048_ByteChgUpdDsbl_e ByteChgUpdDsbl;      /* Enables/Disables Receive byte Updates
                                                   when a Register's byte Change interupt
                                                   is Active -- See Reg MACNF - Bit2 */

  /* PSN_ADDED B1 Release Following Config Bits */
  ixf6048_GenIOVal_e GenIOVal;  /* GOCNF Bit[13] */
  ixf6048_GenIOMode_e GenIOMode;        /* GOCNF Bit[12] */

  ixf6048_PeclMsbCnf_e XmtPeclMsbCnf;   /* XMT PECL MSB Config GOCNF Bit[9] */
  ixf6048_PeclMsbCnf_e RcvPeclMsbCnf;   /* Rcv PECL MSB Config GOCNF Bit[8] */
  /* PSN_END End Of Added Bits for B2 Release */

  ixf6048_UtopiaChipCfg_t UtopiaChipCfg;        /* Utopia Interface Configuration */
  ixf6048_OrderWirePulseByteLocn_e OrderWirePulseByteLocn;      /* see Above -- OHPCNF */
  ixf6048_OverheadPortMode_e OverheadPortMode;
  uchar RxFramePulseByteNum;    /* R_FPCNF 7:0 */
  uchar TxFramePulseOutByteNum; /* T_FPCNF 15:8 */
  uchar TxFramePulseInByteNum;  /* T_FPCNF 7:0 */
  ixf6048_OutClockPolarity_e OutClockPolarity;  /* OCPCNF */
  ixf6048_InClockPolarity1_e InClockPolarity1;  /* ICPCNF1 */
  ixf6048_InClockPolarity2_e InClockPolarity2;  /* ICPCNF2 */

  ixf6048_NonConcatModeCnfg_t NonconcatModeCnfg;        /* NCMODECNF */
  ixf6048_LineSideParityConfig_t LineSideParityConfig;  /* LSPCNF    */
  ixf6048_PosChipCfg_t PosChipCfg;      /* {0} if no POS chans */

}
ixf6048_ChipLevelCfg_t;

/***********************************************************************
 * Channel Interface Configuration structures - for Channels 0-3
 *
 * Note: For Single channel interface, only Channel 0 may be configured.
 *
 * Note: To support the interfacing of dissimilar Line Side interfaces
 *       (including between SONET and SDH) in a Repeater or Transparent
 *       channel configuration, the Rx and Tx Line Side interfaces are
 *       specified indepently.
 *       When configured as an ATM or POS channel, warnings are issued if
 *       the Rx and Tx Line Side interfaces are different, but the
 *       configuration is accepted as specified.
 *
 *       The Rx and Tx sides of the UTOPIA interface are specified to the
 *       Driver as a unit, and the Driver configures both sides of the
 *       UTOPIA hardware interface in the same way.
 **********************************************************************/

/*------------------ SONET/SDH Line Side Channel Config -----------------*/

/* SONET/SDH Line Side Speed - See compatability table Pg 10
         See also:
         Reg GOCNF bits 6:5; 
         Reg COCNF bits 1:0, */
typedef enum
{
  ixf6048_eSINGLE_STS_48C = 1,  /* Single STS-48c */
  ixf6048_eSINGLE_STM_16C,      /* Single STM-16c */
  /* = ixf6048_eSINGLE_STS_48C, */

  ixf6048_eSINGLE_STS_48,       /* Single STS-48 */
  ixf6048_eSINGLE_STM_16,       /* Single STM-16 */
  /* = ixf6048_eSINGLE_STS_48, */

  ixf6048_eSINGLE_STS_12,       /* Single STS-12 */
  ixf6048_eSINGLE_STM_4,        /* Single STM-4 */
  /* = ixf6048_eSINGLE_STS_12, */

  ixf6048_eSINGLE_STS_12C,      /* Single STS-12c */
  ixf6048_eSINGLE_STS_3C,       /* Single STS-3c */
  ixf6048_eSINGLE_STS_3,        /* Single STS-3 */

  ixf6048_eQUAD_STS_12C,        /* Quad STS-12c */
  ixf6048_eQUAD_STM_4C,         /* Quad STM-4c */
  /* = ixf6048_eQUAD_STS_12C, */

  ixf6048_eQUAD_STS_3C,         /* Quad STS-3c */
  ixf6048_eQUAD_STM_1,          /* Quad STM-1 */
  /* = ixf6048_eQUAD_STS_3C, */

  ixf6048_eQUAD_STS_1           /* Quad STS-1 */
}
ixf6048_LineSideSpeed_e;

/* Rx Line Side Interface - See compatability table Pg 10
         See R_COCNF Bits 4:2
         Note: 32-bit TTL and 16-bit or 8-bit PECL only valid for
               Single Channel line interface. */
typedef enum
{
  ixf6048_eRX_32_BIT_TTL = 0x0, /* 32-bit TTL  -- Single Mode Only */
  ixf6048_eRX_8_BIT_TTL = 0x2,  /* 8-bit TTL */
  ixf6048_eRX_1_BIT_TTL = 0x3,  /* 1-bit TTL */

  ixf6048_eRX_16_BIT_PECL = 0x4,        /* 16-bit PECL -- Single Mode Only */
  ixf6048_eRX_8_BIT_PECL = 0x5, /* 8-bit PECL  -- Single Mode Only */
  ixf6048_eRX_1_BIT_PECL = 0x7  /* 1-bit PECL */
}
ixf6048_RxLineSideInterface_e;

/* Rx TTL Clock config to get TTL Rx output clock - 
        -- Rx Input clock divided by 1,2,..32
        -- See: R_COCNF, T_COCNF Bits 7:5   */
typedef enum
{
  ixf6048_eRX_TTL_CLK_DIV_1 = 1,
  ixf6048_eRX_TTL_CLK_DIV_2,
  ixf6048_eRX_TTL_CLK_DIV_4,
  ixf6048_eRX_TTL_CLK_DIV_8,
  ixf6048_eRX_TTL_CLK_DIV_16,
  ixf6048_eRX_TTL_CLK_DIV_32,
  ixf6048_eRX_TTL_CLK_IS_8_KHZ, /* Rx output clock is 8 KHz */
  ixf6048_eRX_TTL_CLK_IS_TRISTATED      /* Rx output clock is tristated */
}
ixf6048_RxTtlClockDivide_e;

/* Receive LOCK Config: 
   -- Configures how the input pins RLOCK_i (i=0,1,2,3) are used to 
      switch the internal Receive Cloick Reference to the Blue Clock. 
   -- R_COCNF Bit 8*/
typedef enum
{
  ixf6048_eSWITCH_TO_BLUE_CLOCK_ON_ACTIVE_HIGH = 0,     /* Switch when RLOCK_i=1 */
  ixf6048_eSWITCH_TO_BLUE_CLOCK_ON_ACTIVE_LOW   /* Switch when RLOCK_i=0 */
}
ixf6048_RcvLockConfig_e;

/* Tx Line Side Interface - See compatability table Pg 10
      See:Reg T_COCNF 4:2
      Note: 32-bit TTL and 16-bit or 8-bit PECL only valid for
            Single Channel line interface.*/
typedef enum
{
  ixf6048_eTX_32_BIT_TTL = 0x0, /* 32-bit TTL -- Single Mode Only */
  ixf6048_eTX_8_BIT_TTL = 0x2,  /* 8-bit TTL */
  ixf6048_eTX_1_BIT_TTL = 0x3,  /* 1-bit TTL */

  ixf6048_eTX_16_BIT_PECL = 0x4,        /* 16-bit PECL -- Single Mode Only */
  ixf6048_eTX_8_BIT_PECL = 0x5, /* 8-bit PECL  -- Single Mode Only */
  ixf6048_eTX_1_BIT_PECL = 0x7  /* 1-bit PECL */
}
ixf6048_TxLineSideInterface_e;

/* Operation of the PECL  Output Clock
      -- T_COCNF Bit 10 */
typedef enum
{
  ixf6048_eXMT_PECL_CLK_ENABLED = 0,
  ixf6048_eXMT_PECL_CLK_DISABLED        /* PECL Output Clock in High Impedence */
}
ixf6048_XmtPClkOutConfig_e;

/* Tx Timing Source config -- Transmitter is clock 
         - T_COCNF 9:8 */
typedef enum
{
  ixf6048_eTX_CLK_IS_STOPPED = 0,       /* Internally Tied Low */
  ixf6048_eTX_CLK_IS_RX_SERIAL_CLK,     /* corresponding Rx serial clock input */
  ixf6048_eTX_CLK_IS_TX_CHAN_INPUT_CLK, /* Its own (per channel) input clock */
  ixf6048_eTX_CLK_IS_TX_COMMON_CLK_INPUT        /* The transmit common clock input */
}
ixf6048_TxTimingSource_e;

/* Tx TTL Clock config to get TTL Tx output clock - 
         -- Tx Source clock divided by 1,2,..32
         -- See: reg T_COCNF Bits 7:5  */
typedef enum
{                               /* PSN_DEBUG -- Set to 0 */
  ixf6048_eTX_TTL_CLK_DIV_1 = 0,
  ixf6048_eTX_TTL_CLK_DIV_2,
  ixf6048_eTX_TTL_CLK_DIV_4,
  ixf6048_eTX_TTL_CLK_DIV_8,
  ixf6048_eTX_TTL_CLK_DIV_16,
  ixf6048_eTX_TTL_CLK_DIV_32,
  ixf6048_eTX_TTL_CLK_IS_8_KHZ, /* Tx output clock is 8 KHz */
  ixf6048_eTX_TTL_CLK_IS_TRISTATED      /* Tx output clock is tristated */
}
ixf6048_TxTtlClockDivide_e;

/*------------------ UTOPIA Interface Channel Config --------------------*/

/* Polarity of Rx,Tx parity (even/odd) 
        See Regs: R_UICHCNF, T_UICHNF bit 9 */
typedef enum
{
  ixf6048_eUTOPIA_ODD_PARITY = 0,
  ixf6048_eUTOPIA_EVEN_PARITY
}
ixf6048_UtopiaParity_e;

/* Rx,Tx Decode-response delay config - 
        See Regs: R_UICHCNF, T_UICHNF bit 8 */
typedef enum
{
  ixf6048_eONE_CYCLE_DECODE_RESPONSE_DELAY = 0,
  ixf6048_eTWO_CYCLE_DECODE_RESPONSE_DELAY
}
ixf6048_UtopiaResponseDelay_e;

/* For RXFA & RXPFA polarity config  
    See Reg : R_UICHCNF bit 10 */
typedef enum
{
  ixf6048_eRXFA_ACTIVE_HIGH = 0,        /* High means frame available */
  ixf6048_eRXFA_ACTIVE_LOW      /* Low  means frame available */
}
ixf6048_RxFaPolarity_e;

/* For TXPFA, TXSFA, TXFA polarity config  
    See Reg : T_UICHCNF bit 10 */
typedef enum
{
  ixf6048_eTXFA_ACTIVE_HIGH = 0,        /* High means free space avail */
  ixf6048_eTXFA_ACTIVE_LOW      /* Low  means free space avail */
}
ixf6048_TxFaPolarity_e;

/*------------------ ATM-specific UTOPIA Channel Configuration -----------*/

/* Note: See cell structures on page .
   Note: HEC is always generated by Amazon chip.
         UTOPIA interface can Rx/Tx 52 bytes (omiting space for HEC byte)
         or pass another 8/16/32-bit (dummy) word where HEC byte would be. */

/* ATM Cell Data Cfg -- RXENB is only taken into account at the beginning 
         of the cell */
typedef enum
{
  ixf6048_eCELL_BURST_NORMAL_MODE = 0,
  ixf6048_eCELL_BURST_PASS_THROUGH_MODE
}
ixf6048_CellBurstCnf_e;
/* ATM Cell data structure - pass null HEC word or not
        See Regs: R_UICHCNF, T_UICHCNF bit 6 */
typedef enum
{
  ixf6048_ePASS_NULL_HEC_WORD = 0,      /* Rx/Tx dummy word for HEC byte */
  ixf6048_eDO_NOT_PASS_HEC      /* No Rx/Tx dummy word for HEC byte */
}
ixf6048_DummyHecWord_e;

/* Note:

   RxCellAvailDeassertByteNum configures when the "Cell Available in FIFO to
   send to UTOPIA" signals are deasserted, so the ATM Layer device can know
   that the current ATM cell is the last ATM cell in the FIFO. Namely, the
   value specifies the ATM payload byte number (1-48), after which the
   "Cell Available" signals will be deasserted, on the next RXCLK rising edge
   after the word containing the specified ATM payload byte.
   See also Reg R_UICHCNF bits 5:0

   TxCellAvailDeassertByteNum configures when the "Space Available in FIFO to
   hold cell from UTOPIA" signals are deasserted, so the ATM Layer device can
   know that the current ATM cell is the last ATM cell to be sent to the FIFO,
   for now. Namely, the value specifies the ATM payload byte number (9-48),
   after which the "Space Available" signals will be deasserted, on the next
   TXCLK rising edge after the word containing the specified ATM payload byte.
   See also  Reg T_UICHCNF bits 5:0 XmtCADeassert

   TxFifoDepth configures the depth of the Tx FIFO, in cells. Namely, for
   Channel 0 (1-64 cells) and for Channels 1-3 (1-16 cells).
   After the Tx FIFO contains TxFifoDepth number of cells, the "Space
   Available" signal (TXFA) is deasserted. This is available to be able to
   reduce the cell delay, by limiting the Tx FIFO depth.
   See also Reg T_UIFDP bits 6:0 XmtFDCnf
*/
typedef struct                  /* ATM-specific Channel config */
{
  ixf6048_CellBurstCnf_e RxCellBurstCnf;
  ixf6048_CellBurstCnf_e TxCellBurstCnf;

  ixf6048_DummyHecWord_e RxDummyHecWord;
  ixf6048_DummyHecWord_e TxDummyHecWord;

  uchar RxCellAvailDeassertByteNum;
  uchar TxCellAvailDeassertByteNum;

  uchar TxFifoDepth;

}
ixf6048_UtopiaAtmChanCfg_t;

/*------------------ POS-specific UTOPIA Channel Configuration -----------*/
/***********************************************************************
 * Data to configure programmable POS watermarks
 *
 * Notes:
 *    RcvPWM -- configures when the "POS Data Available" signal
 *    (RXFA) is asserted and deasserted.
 *    Data Available when Rx FIFO contains an End of Packet, or at least
 *    BytesAvailForDataAvail number of bytes.
 *    Data Not Available when Rx FIFO does not contain an End of Packet and
 *    fewer bytes available than that specified by RcvPWM.
 *    See also R_PWM 11:0.
 *
 *    RxFaDeassertOnLow -- specifies whether Data Available signal asserted
 *    only when FIFO contains BytesAvailForDataAvail number of bytes,
 *    ignoring presence of End of Packet.
 *    See also R_UICNF bit 6.
 *
 *    XmtNFPWM -- configures when the "Free Space Available in the Tx FIFO"
 *    signals (TXFA etc.) are asserted and deasserted, as follows.
 *    See also T_NFPWM & T_NEPWM 9:0 for XmtNFPWM & XmtNEPWM
 *
 *    "Space Available" asserted -- when the Tx FIFO has free at least
 *    XmtNFPWM number of bytes.
 *
 *    "Space Available" deasserted -- when the Tx FIFO has fewer than
 *    XmtNEPWM  number of bytes free.
 *
 *    Num Bytes quantized by Driver to 4,8,12,...Max
 *    BytesAvailForDataAvail Range: Chan 1-3: 4-1024, Chan 0: 4-4096
 *    xxxSpaceAvail Range: Chan 1-3: 4-256,  Chan 0: 4-1024
 *    Note: Num Bytes converted to num 4-byte words for hardware.
 **********************************************************************/
/* To configure values of POS watermarks */
typedef struct
{
  ushort RcvPWM;                /* Rx Pos Watermark */
  ushort XmtNFPWM;              /* Tx Near Full WM  */
  ushort XmtNEPWM;              /* Tx Near Empty WM */
}
ixf6048_PosWatermarks_t;

/* Channel Interface Config */
typedef struct
{

  ixf6048_LineSideSpeed_e LineSideSpeed;
  ixf6048_RcvLockConfig_e RcvLockConfig;

  ixf6048_RxLineSideInterface_e RxLineSideInterface;
  ixf6048_RxTtlClockDivide_e RxTtlClockDivide;

  ixf6048_XmtPClkOutConfig_e XmtPClkOutConfig;
  ixf6048_TxLineSideInterface_e TxLineSideInterface;
  ixf6048_TxTimingSource_e TxTimingSource;
  ixf6048_TxTtlClockDivide_e TxTtlClockDivide;

  ixf6048_UtopiaParity_e RxUtopiaParity;
  ixf6048_UtopiaParity_e TxUtopiaParity;

  ixf6048_UtopiaResponseDelay_e RxUtopiaResponseDelay;
  ixf6048_UtopiaResponseDelay_e TxUtopiaResponseDelay;

  ixf6048_RxFaPolarity_e RxFaPolarity;
  ixf6048_TxFaPolarity_e TxFaPolarity;

  /* Config(s) below can be 0'ed out if not needed */
  ixf6048_UtopiaAtmChanCfg_t UtopiaAtmChanCfg;  /* Ignored if not ATM */
  ixf6048_PosWatermarks_t PosWatermarks;        /* Ignored if not POS */

}
ixf6048_ChanIfCfg_t;

/***********************************************************************
 * Rx Regenerator Section Termination Configuration - Pg 198 on
 **********************************************************************/

/* Rx RST Descrambling config - 
        See: Reg R_RSTC bit 15 */
typedef enum
{
  ixf6048_eDISABLE_RX_RST_DESCRAMBLING = 0,     /* Disable Rx RST Descrambling */
  ixf6048_eENABLE_RX_RST_2E7_DESCRAMBLING       /* Enable Rx RST Descrambling 2e7 */
}
ixf6048_RxRstScrmblCnfg_e;

/* Use external frame pulse reference input (RFPI), or not  
        See: Reg R_RSTC bit 14 */
typedef enum
{
  ixf6048_eRX_RST_IGNORE_RFPI = 0,      /* Ignore RFPI - enable internal alignment */
  ixf6048_eRX_RST_USE_RFPI      /* Use    RFPI - disable internal alignment */
}
ixf6048_RxRstFBaDsbl_e;

/* Config of Frame Work Checking during Frame desynchronization
        -- Only Valid for OC12 and OC48.
        See: Reg R_RSTC bit 13, 12 */
typedef enum
{
  ixf6048_eFWD_OFF_CNFG_LONG_FORMAT = 0,        /* 24 Bytes for OC-48, Entire Frame for OC-12 */
  ixf6048_eFWD_OFF_CNFG_SHORT_FORMAT    /* 6 bytes for OC-48 and OC-12 */
}
ixf6048_RstRcvFwdCnfg_e;


/* B1 error counter counts Bit or Block errors
        See: Reg R_RSTC bit 11 */
typedef enum
{
  ixf6048_eCOUNT_B1_BIT_ERRORS = 0,     /* Count B1 bit errors */
  ixf6048_eCOUNT_B1_BLOCK_ERRORS        /* Count B1 block errors */
}
ixf6048_B1CntrCfg_e;

/* Config Loss of Signal Detection -
        See: Reg R_RSTC bit 10:9 */
typedef enum
{
  ixf6048_eRX_RST_LOS_ON_20US_ZEROS = 0,        /* LOS = 20 Microsecs of 0's */
  ixf6048_eRX_RST_LOS_ON_20US_LOS,      /* LOS = 20 Microsecs of no change */
  ixf6048_eRX_RST_LOS_ON_25US_ZEROS,    /* LOS = 25 Microsecs of 0's */
  ixf6048_eRX_RST_LOS_ON_25US_LOS       /* LOS = 25 Microsecs of no change */
}
ixf6048_RxRstLos_e;


/* Enable/Disable Blue Clock on Loss of Signal or Clock  
        See: Reg R_RSTC bit 8 */
typedef enum
{
  ixf6048_eDISABLE_BLUE_CLOCK_ON_LOS_LOC = 0,
  ixf6048_eENABLE_BLUE_CLOCK_ON_LOS_LOC
}
ixf6048_BlueClkOnLosLoc_e;

/* Enable/Disable AIS Gen from RST to MST on Loss of Frame
        See: Reg R_RSTC bit 7 */
typedef enum
{
  ixf6048_eDISABLE_RX_RST_AIS_ON_LOF = 0,
  ixf6048_eENABLE_RX_RST_AIS_ON_LOF
}
ixf6048_RxRstAisOnLof_e;

/* Enable/Disable AIS Gen from RST to MST on Loss of Signal
        See: Reg R_RSTC bit 6 */
typedef enum
{
  ixf6048_eDISABLE_RX_RST_AIS_ON_LOS = 0,
  ixf6048_eENABLE_RX_RST_AIS_ON_LOS
}
ixf6048_RxRstAisOnLos_e;

/* Enable/Disable AIS Gen from RST to MST on Loss of Clock
        See: Reg R_RSTC bit 5 */
typedef enum
{
  ixf6048_eDISABLE_RX_RST_AIS_ON_LOC = 0,
  ixf6048_eENABLE_RX_RST_AIS_ON_LOC
}
ixf6048_RxRstAisOnLoc_e;

/* Enable/Disable AIS Gen from RST to MST on J0 mismatch
        See: Reg R_RSTC bit 3 */
typedef enum
{
  ixf6048_eDISABLE_RX_RST_AIS_ON_J0 = 0,
  ixf6048_eENABLE_RX_RST_AIS_ON_J0
}
ixf6048_RxRstAisOnJ0_e;


/* Configure Loss of Clock Alarm Filtering
        See: Reg R_RSTC bits 2:1 */
typedef enum
{
  ixf6048_eNO_LOC_FILTERING = 0,
  ixf6048_eWEAK_LOC_FILTERING = 2,
  ixf6048_eSTRONG_LOC_FILTERING = 3
}
ixf6048_RxRstLocFiltering_e;

/* STM-0/STS-1 NDF Frame Acquisition config (else Normal)
        See: Reg R_RSTC bit 0 */
typedef enum
{
  ixf6048_eSTM0_NDF_NORMAL_FRAME_ACQUISITION = 1,
  ixf6048_eSTM0_NDF_ROBUST_FRAME_ACQUISITION
}
ixf6048_NdfFrameAcqCfg_e;

/* Out of Frame Configuration    LOF_LMN Bit 15
      -- Num consectutive eroneous frame words to detect 
         before going to an Out of Frame Condition */
typedef enum
{
  ixf6048_eOOF_CNFG_4_FRAMES = 0,       /* Declare OOF within 500 uS */
  ixf6048_eOOF_CNFG_5_FRAMES    /* Declare OOF within 625 uS */
}
ixf6048_RcvOofCnfg_e;

/* Loss of Frame State Machine Parameters 
    See Reg LOF_LMN         */
typedef struct
{
  uchar L;                      /* 1-32 = num non-consecutive frames with OOF to enter LOF state */
  uchar M;                      /* 1-32 = num consecutive frames with no OOF to reenter Normal */
  uchar N;                      /* 1-32 = num consecutive frames with no OOF to reenter Normal
                                   state, from LOF state */
}
ixf6048_LofStateMachineParms_t;

/* Configure whether J0 CRC7 error masks J0 Mismatch
        See: reg J0_RSTC bit 5 */
typedef enum
{
  ixf6048_eJ0_CRC7_MASKS_MISMATCH = 0,  /*CRC7 error masks Mismatch error */
  ixf6048_eJ0_CRC7_AND_MISMATCH_INDEPENDANT     /* CRC7 and Mismatch error are separate */
}
ixf6048_J0Crc7Mismatch_e;

/* Configure whether J0 Unstable sets J0 Mismatch
        See: reg J0_RSTC bit 4 */
typedef enum
{
  ixf6048_eJ0_UNSTABLE_SETS_MISMATCH = 0,       /* J0 Unstable sets J0 Mismatch */
  ixf6048_eJ0_UNSTABLE_AND_MISMATCH_INDEPENDANT /* J0 Unstable & Mismatch separate */
}
ixf6048_J0UnstableMismatch_e;

/* Configure num of consecutive, identical J0 strings to be
        See: reg J0_RSTC bit 3 */
typedef enum
{
  ixf6048_eJ0_STABLE_AFTER_3 = 0,       /* J0 stable after 3 consecutive strings */
  ixf6048_eJ0_STABLE_AFTER_5    /* J0 stable after 5 consecutive strings */
}
ixf6048_J0StableNum_e;

/* Configure Expected J0 format 
            -- Maps to Register Bits 
            -- Also Used for Expected J1 Format in HPT section
        See: Reg J0_RSTC bits 2:0*/
typedef enum
{
  ixf6048_eRX_Jn_64_WITH_LF_CR = 0x7,   /* 64 bytes with LF,CR at end */
  ixf6048_eRX_Jn_64_FREE_FORM = 0x6,    /* 64 bytes, free form */
  ixf6048_eRX_Jn_16_WITH_CRC7 = 0x4,    /* 16 bytes, starting with CRC7 (SDH) */
  ixf6048_eIGNORE_RX_Jn = 0x2,  /* Ignore J0 */
  ixf6048_eRX_Jn_1_BYTE = 0x0   /* 1 byte */
}
ixf6048_RxJnFormat_e;


typedef struct                  /* Rx Regenerator Section Termination Configuration */
{
  ixf6048_RxRstScrmblCnfg_e RxRstScrmblCnfg;
  ixf6048_RxRstFBaDsbl_e RxRstFBaDsbl;
  ixf6048_RstRcvFwdCnfg_e RxRstRcvFwdOffCnfg;
  ixf6048_RstRcvFwdCnfg_e RxRstRcvFwdOnCnfg;
  ixf6048_B1CntrCfg_e B1CntrCfg;
  ixf6048_RxRstLos_e RxRstLos;
  ixf6048_BlueClkOnLosLoc_e BlueClkOnLosLoc;
  ixf6048_RxRstAisOnLof_e RxRstAisOnLof;
  ixf6048_RxRstAisOnLos_e RxRstAisOnLos;
  ixf6048_RxRstAisOnLoc_e RxRstAisOnLoc;
  ixf6048_RxRstAisOnJ0_e RxRstAisOnJ0;
  ixf6048_RxRstLocFiltering_e RxRstLocFiltering;
  ixf6048_NdfFrameAcqCfg_e NdfFrameAcqCfg;
  ixf6048_RcvOofCnfg_e RcvOofCnfg;      /* Reg LOF_LMN */
  ixf6048_LofStateMachineParms_t LofStateMachineParms;
  ixf6048_J0Crc7Mismatch_e J0Crc7Mismatch;
  ixf6048_J0UnstableMismatch_e J0UnstableMismatch;
  ixf6048_J0StableNum_e J0StableNum;
  ixf6048_RxJnFormat_e ExpectedJ0Format;
  uchar *pExpectedJ0String;
  /* -> Expected J0 string, if used.
     Driver appends LF,CR or prepends dummy
     byte for CRC7, if appropriate */
}
ixf6048_RxRstCfg_t;


/***********************************************************************
 * Rx Multiplexer Section Termination Configuration - Pg 208 on
 **********************************************************************/
/* B2 Excessive Error Configuration: Regs (1cc)B0H - (1cc)0xB5 
      Registers: WINSZ_SB2, CWIN_SB2, E_EXCWIN_SB2, WINSZ_C2, CWIN_CB2, E_NEXCWIN_CB2 */
typedef struct
{
  struct ixf6048_SetExcB2_t     /* Parms for Setting Excessive B2 Error Condition */
  {
    ushort WindowSize;          /* (8 to 2**14) = (8-16384)
                                   Number of frames per window;
                                   Driver quantizes to (8,16,24 .. 2**14) */

    uchar NumExcessiveErrs;
    /* (1 to 2**12 - 1) = (1-4095)
       Number of errors/window to have window considered
       as excessively errored */

    uchar NumExcessiveWindows;
    /* (0, or 1 to 2**7 - 1) = (0, or 1-127)
       Number consecutive excessively errored windows
       to set Excessively Errored Condition */
    /* NOTE: if Excessively Errored Condition not set
       and NumExcessiveWindows = 0, then
       Excessively Errored Condition never set */
  }
  Set;

  struct ixf6048_ClrExcB2_t     /* Parms for Clearing Excessive B2 Error Condition */
  {
    ushort WindowSize;          /* (8 to 2**14) = (8-16384)
                                   Number of frames per window;
                                   Driver quantizes to (8,16,24 .. 2**14) */

    uchar NumNonexcessiveErrs;
    /* (1 to 2**12 - 1) = (1-4095)
       Number of errors/window to have window considered
       as non-excessively errored */

    uchar NumNonexcessiveWindows;
    /* (1 to 2**7 - 1) = (1-127)
       Number consecutive non-excessively errored
       windows to clear Excessively Errored Condition */

  }
  Clear;

}
ixf6048_B2ExcErrCfg_t;

/* B2 (Degraded Signal Defect) Error Configuration (1cc)B7H - (1cc)0xBD  
         Registers: WINSZ_SDEGB2, CWIN_SDEGB2, E_DEGWIN_SB2, 
                     WINSZ_CDEGB2, CWIN_CDEGB2, E_NDEGWIN_CB2 */
typedef struct
{                               /* Parms for Setting Degraded B2 Error Condition */
  struct ixf6048_SetDegB2_t
  {
    ulong WindowSize;           /* (8 to 2**14) = (8-16384)
                                   Number of frames per window;
                                   Driver quantizes to (8,16,24 .. 2**14) */
    uchar NumDegradedErrs;
    /* (1 to 2**12 - 1) = (1-4095)
       Number of errors/window to have window considered
       as degraded */
    uchar NumDegradedWindows;
    /* (0, or 1 to 2**7 - 1) = (0, or 1-127)
       Number consecutive degraded windows
       to set Degraded Errored Condition */
    /* NOTE: if Degraded Errored Condition not set
       and NumDegradedWindows = 0, then
       Degraded Error Condition never set */
  }
  Set;
  /* Parms for Clearing Degraded B2 Error Condition */
  struct ixf6048_ClrDegB2_t
  {
    ulong WindowSize;           /* (8 to 2**14) = (8-16384)
                                   Number of frames per window;
                                   Driver quantizes to (8,16,24 .. 2**14) */
    uchar NumNondegradedErrs;
    /* (1 to 2**12 - 1) = (1-4095)
       Number of errors/window to have window considered
       as not degraded */
    uchar NumNondegradedWindows;
    /* (1 to 2**7 - 1) = (1-127)
       Number consecutive non-degraded windows to clear
       Degraded Errored Condition */
  }
  Clear;
}
ixf6048_B2DegErrCfg_t;

/* K1,K2 Filtering: Combined or separately - R_MST_C bit 13 */
typedef enum
{
  ixf6048_eK1K2_SEPARATE = 0,   /* Filter K1 & K2 bytes separately */
  ixf6048_eK1K2_COMBINED        /* Filter K1 & K2 bytes as unit */
}
ixf6048_K1K2Filtering_e;

typedef enum                    /* K2 RDI Filtering for MST AIS: 3 or 5 consecutive frames,
                                   with value of '111', to set (or different, to clear)
                                   R_MST_C bit 10 */
{
  ixf6048_eK2_AIS_DETECT_IN_3_FRAMES = 0,
  ixf6048_eK2_AIS_DETECT_IN_5_FRAMES
}
ixf6048_K2AisDetect_e;

typedef enum                    /* K2 RDI Filtering for MST RDI: 3,5,10,16 consecutive frames,
                                   with value of '110', to set (or different, to clear)
                                   R_MST_C 9:8 */
{
  ixf6048_eK2_RDI_DETECT_IN_3_FRAMES = 0,
  ixf6048_eK2_RDI_DETECT_IN_5_FRAMES,
  ixf6048_eK2_RDI_DETECT_IN_10_FRAMES,
  ixf6048_eK2_RDI_DETECT_IN_16_FRAMES
}
ixf6048_K2RdiDetect_e;


typedef enum                    /* Enable/Disable AIS generation from MST to MSA
                                   and generation of Signal Fail (SF) on Excessive B2 Error
                                   R_MST_C bit 7 */
{
  ixf6048_eENABLE_AIS_TO_MSA_ON_EXC_B2_ERR = 1, /* Enable AIS and Signal Fail
                                                   on Exc B2 Err */
  ixf6048_eDISABLE_AIS_TO_MSA_ON_EXC_B2_ERR     /* Disable AIS and Signal Fail
                                                   on Exc B2 Err */
}
ixf6048_AisToMsaOnExcB2Err_e;

typedef enum                    /* Enable/Disable automatic AIS generation from MST to MSA,
                                   R_MST_C bit 5 */
{
  ixf6048_eDISABLE_AIS_TO_MSA = 0,      /* Disable AIS from MST to MSA */
  ixf6048_eENABLE_AIS_TO_MSA    /* Enable AIS from MST to MSA */
}
ixf6048_AisToMsa_e;


typedef enum                    /* Config generated MST REI (based on B2 BIP errs) to be
                                   inserted into Tx M1 byte as BIP or Block errs
                                   (for STM-4c/STS-12c or STM-16c/STS-48c formats)
                                   R_MST_C bit 4 */
{
  ixf6048_eTX_REI_BIP_ERRS = 0, /* MST REI as Bit err */
  ixf6048_eTX_REI_BLK_ERRS      /* MST REI as Block err */
}
ixf6048_TxReiBipBlk_e;

typedef enum                    /* Config Rx MST REI as tracking BIP or Block errs
                                   (for STM-4c/STS-12c or STM-16c/STS-48c formats)
                                   R_MST_C bit 3 */
{
  ixf6048_eRX_REI_BIP_ERRS = 0, /* MST REI as Bit err */
  ixf6048_eRX_REI_BLK_ERRS      /* MST REI as Block err */
}
ixf6048_RxReiBipBlk_e;


typedef enum                    /* Enable/Disable MST RDI generation (i.e. Tx K2[2:0] = '110')
                                   on Excessive B2 Error. R_MST_C bit 2 */
{
  ixf6048_eDISABLE_MST_RDI_ON_EXC_B2_ERR = 0,   /* Disable MST RDI on Exc B2 Err */
  ixf6048_eENABLE_MST_RDI_ON_EXC_B2_ERR /* Enable MST RDI on Exc B2 Err */
}
ixf6048_MstRdiOnExcB2Err_e;

typedef enum                    /* Config block size of a concatenated frame,
                                   for Rx B2 detected errored blocks, and
                                   for Tx REI Generated errored block - R_MST_C bit 0 */
{
  ixf6048_eSINGLE_BLK_PER_CONCAT_FRAME = 0,
  ixf6048_eMULT_2430_BYTE_BLKS_PER_FRAME
}
ixf6048_BlksPerConcatFrame_e;

typedef struct                  /* Rx MST Configuration structure */
{
  ixf6048_B2ExcErrCfg_t B2ExcErrCfg;
  ixf6048_B2DegErrCfg_t B2DegErrCfg;

  ixf6048_K1K2Filtering_e K1K2Filtering;        /* Bit13 */
  ixf6048_K2AisDetect_e K2AisDetect;    /* Bit10 */
  ixf6048_K2RdiDetect_e K2RdiDetect;    /* Bits[9:8] */
  ixf6048_AisToMsaOnExcB2Err_e AisToMsaOnExcB2Err;      /* Bit7 */
  ixf6048_AisToMsa_e AisToMsa;  /* Bit5 */
  ixf6048_TxReiBipBlk_e TxReiBipBlk;    /* Bit4 */
  ixf6048_RxReiBipBlk_e RxReiBipBlk;    /* Bit3 */
  ixf6048_MstRdiOnExcB2Err_e MstRdiOnExcB2Err;  /* Bit2 */
  ixf6048_BlksPerConcatFrame_e BlksPerConcatFrame;      /* Bit0 */
}
ixf6048_RxMstCfg_t;


/***********************************************************************
 * Rx Multiplexer Section Adaptation Configuration - Reg R_MSA_C
 **********************************************************************/
/* Enable/Disable use of AuAisc detection
                  R_MSA_C bit 9 */
typedef enum
{
  ixf6048_eAU_AISC_DETECTION_DISABLE = 0,
  ixf6048_eAU_AISC_DETECTION_ENABLE
}
ixf6048_UseAuAiscDetection_e;

/* Enable/Disable HP-RDI generation during LOPC
                  R_MSA_C bit 8 */
typedef enum
{
  ixf6048_eHP_RDI_GEN_DURING_LOPC_DISABLE = 0,
  ixf6048_eHP_RDI_GEN_DURING_LOPC_ENABLE
}
ixf6048_LopcOnServerDefect_e;

/* Enable/Disable AIS generation during LOPC
                  R_MSA_C bit 7 */
typedef enum
{
  ixf6048_eAIS_GEN_DURING_LOPC_DISABLE = 0,
  ixf6048_eAIS_GEN_DURING_LOPC_ENABLE
}
ixf6048_LopcOnMsaAis_e;

/* Enable/Disable Config Det of concatenation indication for H1/H2 Pairs
                  R_MSA_C bit 6 */
typedef enum
{
  ixf6048_eVERIFY_CONCAT_PTR_ON_ALL_H1H2_PAIRS = 0,
  ixf6048_eVERIFY_CONCAT_PTR_ON_SOME_H1H2_PAIRS
}
ixf6048_ConcPtDetCnfg_e;

/* Use SONET or SDH AU Pointer justification rules
                  R_MSA_C bit 5 */
typedef enum
{
  ixf6048_eUSE_SONET_AU_PTRS = 0,
  ixf6048_eUSE_SDH_AU_PTRS
}
ixf6048_UseSonetSdhAuPtrs_e;

/* Normal Operation / Force AIS generation from MSA to HPT, when
      See R_MSA_C bit 5 */
typedef enum
{
  ixf6048_eNORMAL_AIS_TO_HPT = 0,
  ixf6048_eFORCE_AIS_TO_HPT
}
ixf6048_RcvHptAisFrc_e;

/* Enable/Disable AIS generation from MSA to HPT, when
         AU AIS (H1:H2 = All 1's) or
         LOP (Invalid Pointer Value, Or
              SS bits != 'ss' And AU Ptr SS bits enabled)
              (where ss = ExpectedAuPtrSsBits, below)
      See R_MSA_C bit 2 */
typedef enum
{
  ixf6048_eDISABLE_AIS_TO_HPT = 0,
  ixf6048_eENABLE_AIS_TO_HPT
}
ixf6048_AisToHpt_e;

/* Use/Ignore AU pointer SS bits - R_MSA_C bit 0 */
typedef enum
{
  ixf6048_eIGNORE_AU_PTR_SS_BITS = 0,
  ixf6048_eUSE_AU_PTR_SS_BITS
}
ixf6048_UseAuPtrSsBits_e;

/* Configuration for the R_MSA_C Register */
typedef struct                  /* Rx Multiplexer Section Adaptation Config */
{
  ixf6048_UseAuAiscDetection_e UseAuAiscDetection;      /* Bit9 */
  ixf6048_LopcOnServerDefect_e LopcOnServerDefect;      /* Bit8 */
  ixf6048_LopcOnMsaAis_e LopcOnMsaAis;  /* Bit7 */
  ixf6048_ConcPtDetCnfg_e ConcPtDetCnfg_e;      /* Bit6 */
  ixf6048_UseSonetSdhAuPtrs_e UseSonetSdhAuPtrs;        /* Bit5 */
  ixf6048_AisToHpt_e AisToHpt;  /* Bit2 */
  ixf6048_UseAuPtrSsBits_e UseAuPtrSsBits;      /* Bit1 */

  uchar ExpectedAuPtrSsBits;    /* if SS bits used
                                   See R_MSA_C 4:3 */
}
ixf6048_RxMsaCfg_t;


/***********************************************************************
 *
 * Rx High Order Path Termination Configuration - 
 *    Regs: R_HPT_C1, R_HPT_C2
 **********************************************************************/

/* Enable/Disable AIS Generation from HPT to HPA on alarm
                  R_HPT_C2 bits 4,3,2 */
typedef enum
{
  ixf6048_eDISABLE_AIS_TO_HPA = 0,
  ixf6048_eENABLE_AIS_TO_HPA
}
ixf6048_AisToHpa_e;

/* High Order Path G1 RDI Detection Count:
                  3,5,10,16 consecutive frames - R_HPT_C1 15:14 */
typedef enum
{
  ixf6048_eHPT_RDI_DETECT_ON_3_G1_BYTES = 0,
  ixf6048_eHPT_RDI_DETECT_ON_5_G1_BYTES,
  ixf6048_eHPT_RDI_DETECT_ON_10_G1_BYTES,
  ixf6048_eHPT_RDI_DETECT_ON_16_G1_BYTES
}
ixf6048_HptRdiDetect_e;

/* Config detection of Rx HPT RDI Enhanced, or not
                  R_HPT_C1 bit 13, and Table 1 on Pg 99 */
typedef enum
{
  ixf6048_eDETECT_NON_ENHANCED_HPT_RDI = 0,
  ixf6048_eDETECT_ENHANCED_HPT_RDI
}
ixf6048_DetectEnhancedHptRdi_e;

/* HPT Signal Label Mismatch Count - R_HPT_C1 bit 12 */
typedef enum
{
  ixf6048_eHPT_SL_MISMATCH_ON_3_C2 = 0,
  ixf6048_eHPT_SL_MISMATCH_ON_5_C2
}
ixf6048_HptSlMismatchCnt_e;

/* Setting of the HP-SLM alarm when the receive C2 signal label is unstable
      -- R_HPT_C1 bit 11 */
typedef enum
{
  ixf6048_eSLM_ALARM_INDEPENDANT_OF_C2_UNSTABLE = 0,
  ixf6048_eFORCE_HPT_SLM_ALARM_ON_C2_UNSTABLE
}
ixf6048_HpSlmOnC2Config_e;


/* B3 error counter counts Bit or Block errors
                  R_HPT_C1 bit 10 */
typedef enum
{
  ixf6048_eCOUNT_B3_BIT_ERRORS = 0,
  ixf6048_eCOUNT_B3_BLOCK_ERRORS
}
ixf6048_B3CntrCfg_e;

/* HPT REI error counter counts Bit or Block errors
                  R_HPT_C1 bit 9 */
typedef enum
{
  ixf6048_eCOUNT_HPT_REI_BIT_ERRORS = 0,
  ixf6048_eCOUNT_HPT_REI_BLOCK_ERRORS
}
ixf6048_HptReiCntrCfg_e;

/* Config 2 fixed stuff columns (30 & 59) as part
                  of payload, or not (for STS-1/STM-0 mode only)
                  R_HPT_C1 bit 8, T_HPT_C bit 14 */
typedef enum
{
  ixf6048_eCOLS_30_59_PAYLOAD = 0,      /* Part of the payload */
  ixf6048_eCOLS_30_59_STUFF     /* Fixed stuff columns */
}
ixf6048_Cols30_59_e;

/* Enable/Disable HPT RDI on ATM Loss of Cell Delineation (LCD)
                  R_HPT_C1 bit 7 */
typedef enum
{
  ixf6048_eDISABLE_HPT_RDI_ON_ATM_LCD = 0,
  ixf6048_eENABLE_HPT_RDI_ON_ATM_LCD
}
ixf6048_HptRdiOnAtmLcd_e;

/* Config HPT RDI on HPT Signal Label mismatch, as
                  Connectivity Defect, or Payload Defect, or ignore.
                  R_HPT_C1 6:5 */
typedef enum
{
  ixf6048_eDISABLE_HPT_RDI_ON_HPT_SLM = 0,      /*  SLM does not set HPT RDI */
  ixf6048_eHPT_SLM_SETS_HPT_RDI_CON_DEF = 2,    /* SLM is RDI Connectivity Defect */
  ixf6048_eHPT_SLM_SETS_HPT_RDI_PAY_DEF = 3     /*  SLM is RDI Payload Defect */
}
ixf6048_HptRdiOnHptSlm_e;

/* Enable/Disable HPT RDI on J1 string mismatch
                  R_HPT_C1 bit 4 */
typedef enum
{
  ixf6048_eDISABLE_HPT_RDI_ON_J1_MISMATCH = 0,
  ixf6048_eENABLE_HPT_RDI_ON_J1_MISMATCH
}
ixf6048_HptRdiOnJ1Mismatch_e;


/* Enable/Disable HPT RDI on HPT Unequipped alarm,
                  as Connectivity Defect - R_HPT_C1 bit 3 */
typedef enum
{
  ixf6048_eDISABLE_HPT_RDI_ON_HPT_UNEQ = 0,
  ixf6048_eENABLE_HPT_RDI_ON_HPT_UNEQ
}
ixf6048_HptRdiOnHptUneq_e;

/* Config generation of (enhanced or not) HPT RDI - R_HPT_C1 2:1
                  Values are:
                     Non-enhanced G1 RDI bits: RDI = '100'; no RDI = '000'
                     Non-enhanced G1 RDI bits: RDI = '111'; no RDI = '011'
                     Enhanced G1 RDI bits: See Table 1 on Pg 99 */
typedef enum
{
  ixf6048_eGEN_HPT_RDI_AS_100 = 0,
  ixf6048_eGEN_HPT_RDI_AS_111 = 1,
  ixf6048_eGEN_ENHANCED_HPT_RDI = 2
}
ixf6048_GenHptRdiCfg_e;

/* Config value of G1 spare bit at RPAL output,
                  as a 0 or a 1 - Pg R_HPT_C1 bit 0 */
typedef enum
{
  ixf6048_eG1_SPARE_BIT_IS_0 = 0,
  ixf6048_eG1_SPARE_BIT_IS_1
}
ixf6048_G1SpareBit_e;

/* Configure whether J1 CRC7 error masks J1 Mismatch
                  R_HPT_C2 bit 13 */
typedef enum
{
  ixf6048_eJ1_CRC7_AND_MISMATCH_SEP = 0,        /* CRC7 and Mismatch error are separate */
  ixf6048_eJ1_CRC7_MASKS_MISMATCH       /*CRC7 error masks Mismatch error */
}
ixf6048_J1Crc7Mismatch_e;

/* Configure whether J1 Unstable sets J1 Mismatch
                  R_HPT_C2 bit 12 */
typedef enum
{
  ixf6048_eJ1_UNSTABLE_AND_MISMATCH_SEP = 0,    /* J1 Unstable & Mismatch separate */
  ixf6048_eJ1_UNSTABLE_SETS_MISMATCH    /* J1 Unstable sets J1 Mismatch */
}
ixf6048_J1UnstableMismatch_e;

/* Configure num of consecutive, identical J1 strings to be
                  considered stable and accepted - R_HPT_C2 bit 11 */
typedef enum
{
  ixf6048_eJ1_STABLE_AFTER_3 = 0,       /* J1 stable after 3 consecutive strings */
  ixf6048_eJ1_STABLE_AFTER_5    /* J1 stable after 5 consecutive strings */
}
ixf6048_J1StableCnt_e;


/* Configure Expected J1 format - R_HPT_C2 10:8 */
/* Note that ixf6048_RxJnFormat_e is Defined in R_RST_C Reg  above*/

/* The following is for Documentation Only
typedef enum   
{
   ixf6048_eRX_Jn_1_BYTE       = 0,   1 byte 
   ixf6048_eIGNORE_RX_Jn       = 2    Ignore J1 
   ixf6048_eRX_Jn_16_WITH_CRC7 = 4,   16 bytes, starting with CRC7 (SDH) 
   ixf6048_eRX_Jn_64_FREE_FORM = 6,   64 bytes, free form 
   ixf6048_eRX_Jn_64_WITH_LF_CR = 7,  64 bytes with LF,CR at end 
} ixf6048_RxJnFormat_e;  */

/* Rx High Order Path Termination Config */
typedef struct
{
  ixf6048_HptRdiDetect_e HptRdiDetect;  /* R_HPT_C1 Bits[15:14] */
  ixf6048_DetectEnhancedHptRdi_e DetectEnhancedHptRdi;  /* R_HPT_C1 Bit 13 */
  ixf6048_HptSlMismatchCnt_e HptSlMismatchCnt;  /* R_HPT_C1 Bit 12 */
  ixf6048_HpSlmOnC2Config_e HpSlmOnC2Config;    /* R_HPT_C1 Bit 11 */
  ixf6048_B3CntrCfg_e B3CntrCfg;        /* R_HPT_C1 Bit 10 */
  ixf6048_HptReiCntrCfg_e HptReiCntrCfg;        /* R_HPT_C1 Bit 9 */
  ixf6048_Cols30_59_e Cols30_59;        /* R_HPT_C1 Bit 8 */
  ixf6048_HptRdiOnAtmLcd_e HptRdiOnAtmLcd;      /* R_HPT_C1 Bit 7 */
  ixf6048_HptRdiOnHptSlm_e HptRdiOnHptSlm;      /* R_HPT_C1 Bits[6:5] */

  ixf6048_HptRdiOnJ1Mismatch_e HptRdiOnJ1Mismatch;      /* R_HPT_C1 Bit 4 */
  ixf6048_HptRdiOnHptUneq_e HptRdiOnHptUneq;    /* R_HPT_C1 Bit 3 */
  ixf6048_GenHptRdiCfg_e GenHptRdiCfg;  /* R_HPT_C1 Bits[2:1] */
  ixf6048_G1SpareBit_e G1SpareBit;      /* R_HPT_C1 Bit 0 */

  ixf6048_J1Crc7Mismatch_e J1Crc7Mismatch;      /* R_HPT_C2 Bit 13 */
  ixf6048_J1UnstableMismatch_e J1UnstableMismatch;      /* R_HPT_C2 Bit 12 */
  ixf6048_J1StableCnt_e J1StableCnt;    /* R_HPT_C2 Bit 11 */
  ixf6048_RxJnFormat_e ExpectedJ1Format;        /* R_HPT_C2 Bits[10:8] */
  /* R_HPT_C2 Bits[7:6] Unused */
  ixf6048_RcvHptAisFrc_e RcvHptAisFrc;  /* R_HPT_C2 Bit 5 */

  ixf6048_AisToHpa_e RcvHptAisSlmEnbl;  /* R_HPT_C2 Bit 4 */
  ixf6048_AisToHpa_e RcvHptAisUneqEnbl; /* R_HPT_C2 Bit 3 */
  ixf6048_AisToHpa_e RcvHptAisTimEnbl;  /* R_HPT_C2 Bit 2 */
  /* R_HPT_C2 Bits[1:0] Unused */

  uchar ExpectedC2;             /* Expected HPT Signal Label */
  uchar *pExpectedJ1String;     /* -> J1 string, if used */
  /* Driver appends LF,CR or prepends dummy
     byte for CRC7, if appropriate */
}
ixf6048_RxHptCfg_t;

/***********************************************************************
 *
 * Tx Regenerator Section Termination Configuration - 
 *
 **********************************************************************/

/* Tx RST Scrambling config - T_RMST_OP bit 15 */
typedef enum
{
  ixf6048_eDISABLE_TX_RST_SCRAMBLING = 0,       /* Disable Tx RST Scrambling */
  ixf6048_eENABLE_TX_RST_2E7_SCRAMBLING /* Enable Tx RST Scrambler 2e7 */
}
ixf6048_TxRstScrambling_e;

/* Config unused Tx overhead byte values: 0x00, 0xFF
   See: T_RMST_OP bit 14  */
typedef enum
{
  ixf6048_eUNUSED_TX_OH_ZEROS = 0,      /* Tx 0x00 in unused Overhead bytes */
  ixf6048_eUNUSED_TX_OH_ONES    /* Tx 0xFF in unused Overhead bytes */
}
ixf6048_UnusedTxOh_e;

/* Config Tx TZ0 byte value: See T_RMST_OP bits 13:12 */
typedef enum
{
  ixf6048_eTX_TZ0_USE_ABOVE_DEFAULT = 0,        /* use UnusedTxOh default value */
  ixf6048_eTX_TZ0_IS_AA,        /* value is 0xAA */
  ixf6048_eTX_TZ0_BY_LINE_SPEED /* value varies, by Line Speed */
}
ixf6048_TxTzo_e;

/* Config Tx SZ0 byte value: See T_RMST_OP bits 11:10 */
typedef enum
{
  ixf6048_eTX_SZ0_USE_ABOVE_DEFAULT = 0,        /* use UnusedTxOh default value */
  ixf6048_eTX_SZ0_IS_AA,        /* value is 0xAA */
  ixf6048_eTX_SZ0_BY_LINE_SPEED /* value varies, by Line Speed */
}
ixf6048_TxSzo_e;

/* Config Tx FZ0 byte value: See T_RMST_OP bits 9:8 */
typedef enum
{
  ixf6048_eTX_FZ0_USE_ABOVE_DEFAULT = 0,        /* use UnusedTxOh default value */
  ixf6048_eTX_FZ0_IS_AA,        /* value is 0xAA */
  ixf6048_eTX_FZ0_BY_LINE_SPEED /* value varies, by Line Speed */
}
ixf6048_TxFzo_e;

/* PSN_ADDED for B1 Release */
/* T_SC_RSOH Bit[15], T_HPT_C Bit[15] 
	-- ENable the SOH insertion from the TSOH serial input bus channel (cc)
	-- ENable the POH insertion from the TPOH serial input bus channel (cc)*/
typedef enum
{
  ixf6048_eOH_INS_DISABLED = 0,
  ixf6048_eOH_INS_ENABLED
}
ixf6048_OHInsEna_e;
/* PSN_END B1 Release change */

/* Tx Undefined RSOH Source, except for Row 1,2 National Use
   bytes (ignored if STM-0/STS-1) - 
   See T_SC_RSOH bit 10 */
typedef enum
{
  ixf6048_eTX_UN_USE_ABOVE_DEFAULT = 0, /* use UnusedTxOh default value */
  ixf6048_eTX_UN_RX_BYTE        /* Passed through from Rx Byte */
}
ixf6048_TxUnSource_e;

/* Tx Row 1 National Use / Z0 Source (ignored if STM-0/STS-1)
                  - See T_SC_RSOH bit 1 */
typedef enum
{
  ixf6048_eTX_NU1Z0_FROM_HW = 0,        /* use value from internal h/w */
  ixf6048_eTX_NU1Z0_RX_BYTE     /* Passed through from Rx Byte */
}
ixf6048_TxNu1Z0Source_e;

/* Tx D1-D3 Source - See T_SC_RSOH bits 9:8 */
typedef enum
{
  ixf6048_eTX_D1D3_TRD = 0,     /* From TRD */
  ixf6048_eTX_D1D3_USE_DEFAULT, /* use UnusedTxOh default value */
  ixf6048_eTX_D1D3_RX_BYTE      /* From Rx Byte */
}
ixf6048_TxD1D3Source_e;


/* Tx Row 2 National Use Source (ignored if STM-0/STS-1)
   - See T_SC_RSOH bit 7 */
typedef enum
{
  ixf6048_eTX_NU2_USE_ABOVE_DEFAULT = 0,        /* use UnusedTxOh default value */
  ixf6048_eTX_NU2_RX_BYTE       /* Passed through from Rx Byte */
}
ixf6048_TxNu2Source_e;

/* Tx F1 source - See T_SC_RSOH bits 6:5 */
typedef enum
{
  ixf6048_eTX_F1_TDOW = 0,      /* From TDOW */
  ixf6048_eTX_F1_USE_DEFAULT = 1,       /* use UnusedTxOh default value */
  ixf6048_eTX_F1_RX_BYTE = 2    /* From Rx Byte */
}
ixf6048_TxF1Source_e;

/* Tx E1 source - See T_SC_RSOH bits 4:2 */
typedef enum
{
  ixf6048_eTX_E1_TROW = 0,      /* From TROW */
  ixf6048_eTX_E1_QUIET_PCM = 1, /* Quiet PCM code: 0x7f */
  ixf6048_eTX_E1_USE_DEFAULT = 2,       /* use UnusedTxOh default value */
  ixf6048_eTX_E1_RX_BYTE = 4    /* From Rx Byte */
}
ixf6048_TxE1Source_e;

/* Tx J0 Source - See T_SC_RSOH bit 0 */
typedef enum
{
  ixf6048_eTX_J0_CPU = 0,       /* From CPU */
  ixf6048_eTX_J0_RX_BYTE        /* From Rx Byte */
}
ixf6048_TxJ0Source_e;

/* Configure Tx J0 format - See T_RMST_OP Bits 7:6 */
typedef enum
{
  ixf6048_eTX_Jn_1_BYTE = 0,    /* 1 byte */
  ixf6048_eTX_Jn_DEFAULT,       /* Default value = 0x01 (former C1 byte) */
  ixf6048_eTX_Jn_16_WITH_CRC7,  /* 16 bytes, starting with CRC7 (SDH) */
  ixf6048_eTX_Jn_64_FREE_FORM   /* 64 bytes, free form */
}
ixf6048_TxJnFormat_e;


typedef struct                  /* Tx Regenerator Section Termination Configuration */
{
  ixf6048_TxRstScrambling_e TxRstScrambling;    /* T_RMST_OP bit 15 */
  ixf6048_UnusedTxOh_e UnusedTxOh;      /* T_RMST_OP bit 14 */
  ixf6048_TxTzo_e TxTzo;        /* T_RMST_OP bits[13:12] */
  ixf6048_TxSzo_e TxSzo;        /* T_RMST_OP bits[11:10] */
  ixf6048_TxFzo_e TxFzo;        /* T_RMST_OP bits[9:8] */
  ixf6048_TxJnFormat_e TxJ0Format;      /* T_RMST_OP bits[7:6] */
  /* T_RMST_OP bits[5:1] Used for Testing */

/* PSN_ADDED for B1 Release */
  ixf6048_OHInsEna_e TSOHInsEna;        /* T_SC_RSOH Bit[15] TSOHINS[cc] control input pin enable */
/* PSN_END B1 Release change */
  ixf6048_TxUnSource_e TxUnSource;      /* (ignored if STM-0/STS-1) T_SC_RSOH bit 10 */
  ixf6048_TxD1D3Source_e TxD1D3Source;  /* T_SC_RSOH bits 9:8 */
  ixf6048_TxNu2Source_e TxNu2Source;    /* (ignored if STM-0/STS-1) T_SC_RSOH bit 7 */
  ixf6048_TxF1Source_e TxF1Source;      /* T_SC_RSOH bits 6:5 */
  ixf6048_TxE1Source_e TxE1Source;      /* T_SC_RSOH bits 4:2 */
  ixf6048_TxNu1Z0Source_e TxNu1Z0Source;        /* (ignored if STM-0/STS-1) T_SC_RSOH bit 1 */
  ixf6048_TxJ0Source_e TxJ0Source;      /* T_SC_RSOH bit 0 */

  uchar *pJ0String;             /* -> J0 string, if from CPU */
  /* Driver appends LF,CR or prepends CRC7,
     if appropriate */
}
ixf6048_TxRstCfg_t;

/***********************************************************************
 *
 * Tx Multiplexer Section Termination Configuration - 
 *
 **********************************************************************/

/* Tx input Section Alarm Bus config - See T_SC_MSOH bit 7 */
typedef enum
{
  ixf6048_eTX_IN_SECTION_ALM_BUS_CONTRADIR = 0, /* Contradirectional bus */
  ixf6048_eTX_IN_SECTION_ALM_BUS_CODIR  /* Codirectional bus */
}
ixf6048_TxInSectionAlarmBus_e;

/* Tx processed MSOH bytes (B2, K1, K2, S1, M1) source
                  (Ignored for Repeater; passed through) - See T_SC_MSOH bit 6 */
typedef enum
{
  ixf6048_eTX_MSOH_NORMAL_SOURCES = 0,  /* From normal sources (H/W, CPU,
                                           Serial Access) */
  ixf6048_eTX_MSOH_RX_BYTE      /* From Rx byte (passed through) */
}
ixf6048_TxMsoh_Source_e;

/* Tx unused MSOH bytes source (all except B2, K1, K2,
                  D4-D12, S1, M1, E2) - See T_SC_MSOH bit 0
                  (Ignored for Repeater; passed through) */
typedef enum
{
  ixf6048_eTX_UN_MSOH_USE_ABOVE_DEFAULT = 0,    /* use UnusedTxOh default value */
  ixf6048_eTX_UN_MSOH_RX_BYTE   /* From Rx byte (passed through) */
}
ixf6048_TxUnMsoh_Source_e;


/* Tx K2 RDI Bits Source (Ignored for Repeater) - See T_SC_MSOH bits 11:10
                  (See also ixf6048_TxK2ApsSource_e) */
typedef enum
{
  ixf6048_eTX_MST_RDI_FROM_HW = 0,      /* RDI from internal hardware */
  ixf6048_eTX_MST_RDI_FROM_TSAL = 1,    /* RDI from TSAL input pin */
  ixf6048_eTX_MST_RDI_FROM_CPU = 2      /* RDI from CPU */
}
ixf6048_TxMstRdiSource_e;

/* MST RDI Hysteresis config: 1,5,10,20 frames - 
   See T_SC_MSOH bits 15:14
                  (Ignored for Repeater)       */
typedef enum
{
  ixf6048_eTX_MST_RDI_FOR_1_FRAME = 0,
  ixf6048_eTX_MST_RDI_FOR_5_FRAMES = 1,
  ixf6048_eTX_MST_RDI_FOR_10_FRAMES = 2,
  ixf6048_eTX_MST_RDI_FOR_20_FRAMES = 3
}
ixf6048_TxMstRdiHysteresis_e;

/* Tx M1 REI Bits Source (Ignored for Repeater) - 
    See T_SC_MSOH bits 13:12*/
typedef enum
{
  ixf6048_eTX_MST_REI_FROM_RX_B2_ERRS = 0,      /* REI from Received B2 errors */
  ixf6048_eTX_MST_REI_FROM_TSAL = 1,    /* REI from TSAL input */
  ixf6048_eTX_MST_REI_EQ_ZEROS = 2      /* Send 0's for REI bits in M1 */
}
ixf6048_TxMstReiSource_e;

/* Tx K1 byte Source (Ignored for Repeater) - See T_SC_MSOH bit 8 */
typedef enum
{
  ixf6048_eTX_K1_FROM_TSAL = 1, /* K1 from TSAL input pin */
  ixf6048_eTX_K1_FROM_CPU       /* K1 from CPU */
}
ixf6048_TxK1Source_e;

/* Tx K2 APS Bits Source (Ignored for Repeater) 
    - See T_SC_MSOH bit 9
         (See also ixf6048_TxMstRdiSource_e for K2[2:0]) */
typedef enum
{
  ixf6048_eTX_K2_APS_FROM_CPU = 0,      /* K2 APS from CPU */
  ixf6048_eTX_K2_APS_FROM_TSAL  /* K2 APS bits from TSAL input pin */
}
ixf6048_TxK2ApsSource_e;

/* Tx E2 source - See T_SC_MSOH bits 5:3
          (Ignored for Repeater; passed through) */
typedef enum
{
  ixf6048_eTX_E2_TMOW = 0,      /* From TMOW */
  ixf6048_eTX_E2_QUIET_PCM = 1, /* Quiet PCM code: 0x7f */
  ixf6048_eTX_E2_USE_DEFAULT = 2,       /* use UnusedTxOh default value */
  ixf6048_eTX_E2_RX_BYTE = 4    /* From Rx Byte */
}
ixf6048_TxE2Source_e;

/* Tx D4-D12 Source - See T_SC_MSOH bits 2:1
         (Ignored for Repeater; passed through) */
typedef enum
{
  ixf6048_eTX_D4D12_TMD = 0,    /* From TMD */
  ixf6048_eTX_D4D12_USE_ABOVE_DEFAULT = 1,      /* use UnusedTxOh default value */
  ixf6048_eTX_D4D12_RX_BYTE = 2 /* From Rx Byte */
}
ixf6048_TxD4D12Source_e;


/* Tx Multiplexer Section Termination Configuration
                  (Not needed for a Repeater) */
typedef struct
{
  ixf6048_TxMstRdiHysteresis_e TxMstRdiHysteresis;      /* T_SC_MSOH bits[15:14] */
  ixf6048_TxMstReiSource_e TxMstReiSource;      /* T_SC_MSOH bits[13:12] */
  ixf6048_TxMstRdiSource_e TxMstRdiSource;      /* T_SC_MSOH bits[11:10] */
  ixf6048_TxK2ApsSource_e TxK2ApsSource;        /* T_SC_MSOH bit 9 */
  ixf6048_TxK1Source_e TxK1Source;      /* T_SC_MSOH bit 8 */
  ixf6048_TxInSectionAlarmBus_e TxInSectionAlarmBus;    /* T_SC_MSOH bit 7 */
  ixf6048_TxMsoh_Source_e TxMsoh_Source;        /* T_SC_MSOH bit 6 */
  ixf6048_TxE2Source_e TxE2Source;      /* T_SC_MSOH bits 5:3 */
  ixf6048_TxD4D12Source_e TxD4D12Source;        /* T_SC_MSOH bits 2:1 */
  ixf6048_TxUnMsoh_Source_e TxUnMsoh_Source;    /* T_SC_MSOH bit 0 */
  /* See Regs MP_TK2K1 and MP_TS1 for K1,K2,S1 */
  uchar K1;                     /* if from CPU */
  uchar K2;                     /* If MST RDI or K2 APS from CPU */
  uchar S1;
}
ixf6048_TxMstCfg_t;

/***********************************************************************
 *
 * Tx Multiplexer Section Adaptation Configuration
 *
 **********************************************************************/

typedef struct                  /* Tx Multiplexer Section Adaptation Configuration */
{
  ushort InitialTxAuPtr;        /* Initial Tx AU Pointer (0-782) - reg T_AU_PTS bits 9:0 */
  uchar TxSsBits;               /* Tx SS bits, in Tx AU pointer - reg T_AU_PTS bits 11:10 */
}
ixf6048_TxMsaCfg_t;

/***********************************************************************
 *
 * Tx High Order Path Termination Configuration
 *
 **********************************************************************/
/* PSN_ADDED COMMENT */
/* Note: Bit 15 defined above in T_SC_RSOH register 
		 -- See ixf6048_OHInsEna_e */

typedef enum                    /* Path Overhead Byte Source - Reg T_HPT_C bit 12
                                   (Ignored for Repeater; Rx POH bytes passed through) */
{
  ixf6048_eTX_POH_SPEC_BELOW = 0,       /* Each POH Byte source specified below */
  ixf6048_eTX_POH_RX_BYTES      /* All POH bytes passed through */
}
ixf6048_TxPohSource_e;

typedef enum                    /* Tx AU Ptr source (Ignored for Repeater) - Reg T_HPT_C bit 13 */
{
  ixf6048_eTX_AU_PTR_FROM_CPU = 0,      /* Tx AU Ptrs from CPU & internal h/w */
  ixf6048_eTX_AU_PTR_FROM_RX_BYTE       /* passed through */
}
ixf6048_TxAuPtrSource_e;

typedef enum                    /* Tx H4 config (Ignored for Repeater) - Reg T_HPT_C bit 10 */
{
  ixf6048_eTX_H4_IS_ZEROS = 0,  /* Tx H4 = 0x00 */
  ixf6048_eTX_H4_IS_ONES        /* Tx H4 = 0xFF */
}
ixf6048_TxH4Value_e;

typedef enum                    /* F2 Byte Source (Ignored for Repeater) - Reg T_HPT_C bit 9:8 */
{
  ixf6048_eTX_F2_TPOW1 = 0,     /* From TPOW1 */
  ixf6048_eTX_F2_USE_ABOVE_DEFAULT = 1, /* use UnusedTxOh default value */
  ixf6048_eTX_F2_RX_BYTE = 2    /* From Rx Byte */
}
ixf6048_TxF2Source_e;

typedef enum                    /* F3 Byte Source (Ignored for Repeater) - Reg T_HPT_C bit 7:6 */
{
  ixf6048_eTX_F3_TPOW2 = 0,     /* From TPOW2 */
  ixf6048_eTX_F3_USE_ABOVE_DEFAULT = 1, /* use UnusedTxOh default value */
  ixf6048_eTX_F3_RX_BYTE = 2    /* From Rx Byte */
}
ixf6048_TxF3Source_e;

typedef enum                    /* G1 REI Source  (Ignored for Repeater) - Reg T_HPT_C bits 3:2 */
{
  ixf6048_eFEEDBACK_B3_ERRS = 0,        /* Feedback B3 Errors in REI of G1 byte */
  ixf6048_eG1_REI_FROM_TPAL = 1,        /* G1 REI from TPAL input pin */
  ixf6048_eG1_REI_ZEROS = 2     /* G1 REI = 0 */
}
ixf6048_HptReiSource_e;

typedef enum                    /* G1 RDI Source  (Ignored for Repeater) - Reg T_HPT_C bit 1:0 */
{
  ixf6048_eG1_RDI_FROM_HW = 0,  /* G1 RDI from Hardware */
  ixf6048_eG1_RDI_FROM_TPAL = 1,        /* G1 RDI from TPAL input pin */
  ixf6048_eG1_RDI_FROM_CPU = 2  /* G1 RDI from CPU */
}
ixf6048_HptRdiSource_e;

typedef enum                    /* Tx Input Path Alarm Bus config - Reg T_HPT_OPC bit 15 */
{
  ixf6048_eTX_IN_PATH_ALM_BUS_CONTRADIR = 0,    /* Contradirectional bus */
  ixf6048_eTX_IN_PATH_ALM_BUS_CODIR     /* Codirectional bus */
}
ixf6048_TxInPathAlarmBus_e;

typedef enum                    /* Invert concatenation indicator value - T_HPT_OPC bit 12 */
{
  ixf6048_eINV_H1_CONC_IND_NO_INVERSION = 0,
  ixf6048_eINV_H1_CONC_IND_INVERT_FOREVER
}
ixf6048_InvH1ConcInd_e;



typedef enum                    /* HPT RDI Hysteresis config: 1,5,10,20 frames - Reg T_HPT_OPC bit 11:10 */
{
  ixf6048_eTX_HPT_RDI_FOR_1_FRAME = 0,
  ixf6048_eTX_HPT_RDI_FOR_5_FRAMES,
  ixf6048_eTX_HPT_RDI_FOR_10_FRAMES,
  ixf6048_eTX_HPT_RDI_FOR_20_FRAMES
}
ixf6048_TxHptRdiHysteresis_e;

 /* Config Tx of (enhanced or not) HPT RDI - Reg T_HPT_OPC bit 9
    For enhanced G1 RDI bits: See Table 1 on Pg 99 */
typedef enum
{
  ixf6048_eTx_NON_ENHANCED_HPT_RDI = 0, /* Non-Enhanced G1 RDI bit  (1 bit)  */
  ixf6048_eTX_ENHANCED_HPT_RDI  /* Enhanced     G1 RDI bits (3 bits) */
}
ixf6048_TxEnhancedHptRdi_e;

/* ISSUE: See also ixf6048_GenHptRdiCfg_e - Pg 224 2:1
          They seem to conflict. */

/* Enable/Disable Transmit of Unequipped,
      on unequipped VC-3 (for STM-0) or VC-4 (for STM-1).
      Reg T_HPT_OPC bit 8
      Note: Enabling transmission of VC Unequipped means sending:
            C2 = 0
            N1 = 0
            valid B3
            J1 & N1 as specified, below */
typedef enum
{
  ixf6048_eENABLE_VC_UNEQ = 0,
  ixf6048_eDISABLE_VC_UNEQ      /* VC unequipped is normal */
}
ixf6048_TxVcUneq_e;

/* J1 String value on HPT Unequipped - Reg T_HPT_OPC bit 7 */
typedef enum
{
  ixf6048_eTX_J1_SEND_STRING_ON_HPT_UNEQ = 0,   /* Keep sending J1 string, from source as
                                                   specified by TxJ1Source, on HPT Unequipped */
  ixf6048_eTX_J1_SEND_ZEROS_ON_HPT_UNEQ /* Send 0's in J1 string, on HPT Unequipped */
}
ixf6048_TxJ1UnequippedValue_e;

/* N1 String value on HPT Unequipped - Reg T_HPT_OPC bit 6 */
typedef enum
{
  ixf6048_eTX_N1_SEND_TBD_ON_HPT_UNEQ = 0,      /* ISSUE: TBD to be replaced */
  /* Keep sending N1, from source as
     specified by TBD, on HPT Unequipped */
  ixf6048_eTX_N1_SEND_ZEROS_ON_HPT_UNEQ /* Send 0's in N1, on HPT Unequipped */
}
ixf6048_TxN1UnequippedValue_e;

/* Configure Tx J1 format 
      -- Reg T_HPT_OPC 5:4 
      -- See ixf6048_TxJnFormat_e */

typedef struct                  /* Tx High Order Path Configuration */
{
/* PSN_ADDED for B1 Release */
  ixf6048_OHInsEna_e TPOHInsEna;        /* T_HPT_C Bit[15]  */
/* PSN_END B1 Release change */
  ixf6048_Cols30_59_e Cols30_59;        /*  T_HPT_C bit 14 */
  ixf6048_TxAuPtrSource_e TxAuPtrSource;        /*  T_HPT_C bit 13 */
  ixf6048_TxPohSource_e TxPohSource;    /*  T_HPT_C bit 12 */
  /* T_HPT_C bit 11 Unused */
  ixf6048_TxH4Value_e TxH4Value;        /*  T_HPT_C bit 10 */
  ixf6048_TxF2Source_e TxF2Source;      /*  T_HPT_C bits[9:8] */
  ixf6048_TxF3Source_e TxF3Source;      /*  T_HPT_C bits[7:6] */
  /* T_HPT_C bits[5:4] Unused */
  ixf6048_HptReiSource_e HptReiSource;  /*  T_HPT_C bits[3:2] */
  ixf6048_HptRdiSource_e HptRdiSource;  /*  T_HPT_C bits[1:0] */

  ixf6048_TxInPathAlarmBus_e TxInPathAlarmBus;  /* T_HPT_OPC bit 15 */
  /*T_HPT_OPC bits[14:13] unused */
  ixf6048_InvH1ConcInd_e InvH1ConcInd;  /* T_HPT_OPC bit 12 */
  ixf6048_TxHptRdiHysteresis_e TxHptRdiHysteresis;      /* T_HPT_OPC bits[11:10] */
  ixf6048_TxEnhancedHptRdi_e TxEnhancedHptRdi;  /* T_HPT_OPC bit 9 */
  ixf6048_TxVcUneq_e TxVcUneq;  /* T_HPT_OPC bit 8 */
  ixf6048_TxJ1UnequippedValue_e TxJ1UnequippedValue;    /* T_HPT_OPC bit 7 */
  ixf6048_TxN1UnequippedValue_e TxN1UnequippedValue;    /* T_HPT_OPC bit 6 */
  ixf6048_TxJnFormat_e TxJ1Format;      /* T_HPT_OPC bits[5:4] */
  /* T_HPT_OPC Bits[3:0] used for Testing */

  uchar C2;                     /* reg MP_TC2     */
  uchar HptRdi;                 /* reg MP_THPTRDI Bits[3:1] */
  uchar XmtG1SpBit;             /* reg MP_THPTRDI Bit 0 */
  uchar *pTxJ1String;           /* -> J1 string, if used */
  /* Driver appends LF,CR or prepends CRC7,
     if appropriate */
}
ixf6048_TxHptCfg_t;

/***********************************************************************
 * SONET/SDH Channel Config
 **********************************************************************/

typedef struct                  /* SONET / SDH Channel Configuration */
{
  ixf6048_RxRstCfg_t RxRstCfg;
  ixf6048_RxMstCfg_t RxMstCfg;
  ixf6048_RxMsaCfg_t RxMsaCfg;
  ixf6048_RxHptCfg_t RxHptCfg;

  ixf6048_TxRstCfg_t TxRstCfg;
  ixf6048_TxMstCfg_t TxMstCfg;
  ixf6048_TxMsaCfg_t TxMsaCfg;
  ixf6048_TxHptCfg_t TxHptCfg;

}
ixf6048_OhtChanCfg_t;

/***********************************************************************
 * ATM Rx Channel Config 
 **********************************************************************/
/* Note: R_ACPCNF bits 10 and 9 are Read only and configured at Run Time */

/* Drop/pass idle cells - See R_ACPCNF bit 8 */
typedef enum
{
  ixf6048_ePASS_IDLE_CELLS = 0,
  ixf6048_eDROP_IDLE_CELLS
}
ixf6048_DropPassIdleCell_e;

/* Drop/Correct cell with single-bit HEC error - See R_ACPCNF bit 7
                  Note: If cells with a single-bit HEC error are dropped,
                        they are also treated as an uncorrectable HEC err */
typedef enum
{
  ixf6048_eDROP_SINGLE_BIT_HEC_ERRS = 0,
  ixf6048_eCORRECT_SINGLE_BIT_HEC_ERRS
}
ixf6048_DropCorSingleBitHecErrs_e;

/* Num consecutive cells with error-free HEC required, while in
      Detection mode, to return to Correction mode - See R_ACPCNF bits 6:5 */
typedef enum
{
  ixf6048_eRX_COR_DETECT_AFTER_1 = 0,
  ixf6048_eRX_COR_DETECT_AFTER_2 = 1,
  ixf6048_eRX_COR_DETECT_AFTER_4 = 2,
  ixf6048_eRX_COR_DETECT_AFTER_8 = 3
}
ixf6048_RxCorDetect_e;

/* Drop/pass cells in Loss of Cell Delineation (LCD) state
      - See R_ACPCNF bit 4 */

typedef enum
{
  ixf6048_ePASS_LCD_CELLS = 0,
  ixf6048_eDROP_LCD_CELLS
}
ixf6048_DropPassLcdCells_e;

/* Drop/pass cells with uncorrectable HEC error in SYNC state
      - See R_ACPCNF bit 3 */
typedef enum
{
  ixf6048_eDROP_UNCOR_CELLS_IN_SYNC = 0,
  ixf6048_ePASS_UNCOR_CELLS_IN_SYNC
}
ixf6048_DropPassUncorCellsInSync_e;

/* Drop/pass cells with correct HEC in PRESYNC state
      See R_ACPCNF bit 2 */
typedef enum
{
  ixf6048_eDROP_OK_CELLS_IN_PRESYNC = 0,
  ixf6048_ePASS_OK_CELLS_IN_PRESYNC
}
ixf6048_DropPassOkCellsInPresync_e;

/* XOR '01010101' to HEC before comparison to expected value,
       or before mapping ATM cell into SPE
      - See R_ACPCNF bit 1, T_ACPCNF Bit1 */
typedef enum
{
  ixf6048_eDISABLE_HEC_XOR = 0,
  ixf6048_eENABLE_HEC_XOR
}
ixf6048_HecXor_e;

/* Enable/Disable SPE-level cell payload descrambling
      with polynomial x**43 +  1   - See R_ACPCNF bit 0 */
typedef enum
{
  ixf6048_eDISABLE_SPE_CELL_DESCRAMBLING = 0,
  ixf6048_eENABLE_SPE_CELL_DESCRAMBLING
}
ixf6048_SpeCellDescrambling_e;

/* Number of Milliseconds Loss of Cell Delineation (LCD) defect
      has to persist, before declaring Out of Cell Delineation
      (OCD); same time for SYNC to clear OCD.
      Note: Driver converts Num Milliseconds to the number of
      corresponding clock periods (at the previously specified
      Line Side speed) for the hardware. 
      See also R_LCDFLTR bits 18:0
      See also Table 24                     */
typedef enum
{
  ixf6048_eLCD_FILTER_1_MS = 0, /* Set or cleared after 1 Millisecond */
  ixf6048_eLCD_FILTER_2_MS,
  ixf6048_eLCD_FILTER_3_MS,
  ixf6048_eLCD_FILTER_4_MS      /* Set or cleared after 4 Milliseconds */
}
ixf6048_LcdFilter_e;

/* Rx IDLE/Unassigned Cell Filter - reg R_IUCFLTR; bit numbers below.
      An Idle/Unassigned cell has VPI/VCI = 0's, with the
      specified masked values of the GFC, PTI and CLP bits */
typedef struct
{
  uchar GfcMask;                /* bits [3:0] are mask for GFC bits          15:12 */
  uchar GfcPattern;             /* bits [3:0] are masked value of GFC bits    7:4  */

  uchar PtiMask;                /* bits [2:0] are mask for PTI bits          11:9  */
  uchar PtiPattern;             /* bits [2:0] are masked value of PTI bits    3:1  */

  uchar ClpMask;                /* bit 0 is the mask for CLP bit             bit 8 */
  uchar ClpPattern;             /* bit 0 is the masked value of CLP bit      bit 0 */
}
ixf6048_IdleCellFilter_t;

/* Rx ATM Channel Configuration */
typedef struct
{
  ixf6048_DropPassIdleCell_e DropPassIdleCell;
  ixf6048_DropCorSingleBitHecErrs_e DropCorSingleBitHecErrs;
  ixf6048_RxCorDetect_e RxCorDetect;
  ixf6048_DropPassLcdCells_e DropPassLcdCells;
  ixf6048_DropPassUncorCellsInSync_e DropPassUncorCellsInSync;
  ixf6048_DropPassOkCellsInPresync_e DropPassOkCellsInPresync;
  ixf6048_HecXor_e HecXor;
  ixf6048_SpeCellDescrambling_e SpeCellDescrambling;

  ixf6048_LcdFilter_e LcdFilter;

  ixf6048_IdleCellFilter_t IdleCellFilter;

}
ixf6048_RxAtmChanCfg_t;


/***********************************************************************
 *
 * ATM Tx Channel Config 
 *
 **********************************************************************/
/* PSN_ADDED For B1 Release */
/* T_ACPCNF Bit [13]  
	-- Note: This is a Test Configuration Register
		     When Set, Idle CElls will be sent with VPI = 32(20h) */
typedef enum
{
  ixf6048_eXMT_TEST_VPI_NORMAL_MODE = 0,
  ixf6048_eXMT_TEST_VPI_DEBUG_MODE
}
ixf6048_XmtTestVPI_e;
/* PSN_END End of B1 Release Change */

/* Configuration for the HEC insertion.  Reg T_ACPCNF bits[12:11] */
typedef enum
{
  ixf6048_eHEC_INSERTED_BY_ATM_PROCESSOR = 0,
  ixf6048_eHEC_INSERTED_AFTER_XOR = 1,
  ixf6048_eHEC_INSERTED_IS_EXTRA_UTOPIA_BYTE = 2
}
ixf6048_HecInsertion_e;

/* Specification of this chip's GFC role - Reg T_ACPCNF bits 10,9 */
typedef enum
{
  ixf6048_eGFC_CONTROLLED = 0,  /* This chip is a GFC Controlled Device */
  ixf6048_eGFC_CONTROLLER,      /* This chip is a GFC Controller Device */
  ixf6048_eGFC_NOT_ACTIVE       /* GFC inactive on this channel */
}
ixf6048_GfcRole_e;

/* GFC Halt bit ratio (if GFC Controller) - Reg T_ACPCNF bits 8:6 */
typedef enum
{
  ixf6048_eNEVER_SEND_HALT = 0,
  ixf6048_eSEND_HALT_HALF_TIME,
  ixf6048_eSEND_HALT_TWO_THIRDS_TIME,
  ixf6048_eSEND_HALT_THREE_FOURTHS_TIME,
  ixf6048_eSEND_HALT_FOUR_FIFTHS_TIME,
  ixf6048_eSEND_HALT_FIVE_SIXTHS_TIME,
  ixf6048_eSEND_HALT_SIX_SEVENTHS_TIME,
  ixf6048_eSEND_HALT_SEVEN_EIGHTS_TIME
}
ixf6048_GfcHaltRatio_e;

/* Enable The Reading of the ATM Layer Cells from Tx Atm FIFO 
            Reg T_ACPCNF bit 5 */
typedef enum
{
  ixf6048_eREAD_EN_DISABLED = 0,
  ixf6048_eREAD_EN_NORMAL_MODE
}
ixf6048_XmtReadEn_e;

/* Header Error Config: Configs the insertion of errors into the transmitted ATM Cells
            Reg T_ACPCNF bits[4:3] */
typedef enum
{
  ixf6048_eCOR_ERR_INSERTED_AFTER_1_CELL = 0,
  ixf6048_eUNCOR_ERR_INSERTED_AFTER_1 = 1,
  ixf6048_eCOR_ERRS_INSERTED_ON_H_ERR_EN = 2,
  ixf6048_eUNCOR_ERRS_INSERTED_AFTER_8 = 3
}
ixf6048_XmtHErrCnf_e;

/* Header Error Enable: Enables ths insertion of HEC errors into the transmitted ATM Cells
                        according to XmtHErrCnf bits: 
            Reg T_ACPCNF bit 2 */
typedef enum
{
  ixf6048_eHEC_ERR_EN_INSERT_DEFAULT_STATE = 0,
  ixf6048_eHEC_ERR_EN_INSERT = 1
}
ixf6048_XmtHErrEn_e;


#if(0)                          /* repeated here for documentation */
/* XOR '01010101' to HEC before comparison to expected value,
                  or not - reg R_ACPCNF bit 1, or before mapping ATM cell into SPE
                  - Reg T_ACPCNF bit 1 */
typedef enum
{
  ixf6048_eDISABLE_HEC_XOR = 0,
  ixf6048_eENABLE_HEC_XOR
}
ixf6048_HecXor_e;

#endif

/* Enable/Disable SPE-level cell payload scrambling
                  with polynomial x**43 +  1   - Reg T_ACPCNF bit 0 */
typedef enum
{
  ixf6048_eDISABLE_SPE_CELL_SCRAMBLING = 0,
  ixf6048_eENABLE_SPE_CELL_SCRAMBLING
}
ixf6048_SpeCellScrambling_e;

typedef struct                  /* Tx Idle cell definition - Reg T_ICELLP [15:0] */
{
  uchar GfcValue;               /* Value in [3:0] sent in GFC bits of ATM header */
  uchar PtiValue;               /* Value in [2:0] sent in PTI bits of ATM header */
  uchar ClpValue;               /* Value in bit 0 sent in CLP bit  of ATM header */
  uchar PayloadValue;           /* Value for 48-byte ATM payload */
}
ixf6048_TxIdleCellDef_t;

typedef struct                  /* Tx ATM Channel Configuration */
{                               /* T_ACPCNF Reg */
  ixf6048_HecInsertion_e HecInsertion;  /* Bits 12:11 */
  ixf6048_GfcRole_e GfcRole;    /* Bits 10:9  */
  ixf6048_GfcHaltRatio_e GfcHaltRatio;  /* Bits 8:6   */

  ixf6048_XmtReadEn_e XmtReadEn;        /* Bit 5 */
  ixf6048_XmtHErrCnf_e XmtHErrCn;       /* Bits 4:3 */
  ixf6048_XmtHErrEn_e XmtHErrEn;        /* Bit 2 */

  ixf6048_HecXor_e HecXor;      /* Bit 1      */
  ixf6048_SpeCellScrambling_e SpeCellScrambling;        /* Bit 0      */
/* PSN_ADDED For B1 Release */
  ixf6048_XmtTestVPI_e XmtTestVPI;      /* T_ACPCNF Bit [13]  -- Note: This is a Test Configuration Register */
/* PSN_END End of B1 Release Change */

  ixf6048_TxIdleCellDef_t TxIdleCellDef;
}
ixf6048_TxAtmChanCfg_t;

/***********************************************************************
 * ATM Channel Config (Rx & Tx sides)
 **********************************************************************/

typedef struct                  /* ATM Channel Configuration */
{
  ixf6048_RxAtmChanCfg_t RxAtmChanCfg;
  ixf6048_TxAtmChanCfg_t TxAtmChanCfg;

}
ixf6048_AtmChanCfg_t;

/***********************************************************************
 *
 * POS Rx Channel Config
 *
 **********************************************************************/

/* Configure Line or Traffic Monitoring - reg R_PHCCNF bit 12
                  Line Monitoring implies performance monitoring is not
                  affected by FIFO overflows.
                  Traffic Monitoring implies only frames written into the
                  FIFO are considered for performance monitoring purposes. */
typedef enum
{
  ixf6048_eRX_POS_LINE_MONITORING = 0,
  ixf6048_eRX_POS_TRAFFIC_MONITORING
}
ixf6048_RxPosLineOrTrafficMon_e;

/* Count all bytes, or only bytes in OK frames - reg R_PHCCNF bit 11*/
typedef enum
{
  ixf6048_eRX_POS_COUNT_ALL_BYTES = 0,
  ixf6048_eRX_POS_COUNT_OK_BYTES
}
ixf6048_RxPosByteCountCfg_e;

/* Have FCS errors mark packet as errored in FIFO, or not
        - reg R_PHCCNF bit 10 */
typedef enum
{
  ixf6048_eFCS_ERR_NOT_MARKED_IN_FIFO = 0,
  ixf6048_eFCS_ERR_MARKED_IN_FIFO
}
ixf6048_FcsErrMarkedInFifo_e;

/* Configure format of Rx FIFO for POS
      reg R_PHCCNF bit 9, reg R_PHCCNF bit 1 */
typedef enum
{
  ixf6048_eRX_POS_FIFO_IS_PPP = 1,      /* Just PPP frame to FIFO: Bit1=0, Bit9=0 */
  ixf6048_eRX_POS_FIFO_IS_PPP_FCS,      /* PPP frame plus FCS:Bit1=0, Bit9=1 */
  ixf6048_eRX_POS_FIFO_IS_PPP_AC        /* PPP frame plus Address & Control Bit1=1, Bit9 Ignored */
}
ixf6048_RxPosFifoFormat_e;

/* Enable/Disable HDLC Frame descrambling, after byte destuffing
      with polynomial 1 + x**29 + x**30 + x**49  - reg R_PHCCNF bit 8 */
typedef enum
{
  ixf6048_eDISABLE_FRAME_DESCRAMBLING = 0,
  ixf6048_eENABLE_FRAME_DESCRAMBLING
}
ixf6048_FrameDescrambling_e;

/* Enable/Disable incoming SPE bytes descrambling, before
      HDLC Frame processing, with polynomial 1 + x**43
      reg R_PHCCNF bit 0 */
typedef enum
{
  ixf6048_eDISABLE_SPE_FRAME_DESCRAMBLING = 0,
  ixf6048_eENABLE_SPE_FRAME_DESCRAMBLING
}
ixf6048_SpeFrameDescrambling_e;

/* Enable/Disable Frame Intrafilling.
      Configures whether frame is aborted on FIFO underflow,
      or continued with Control Escape character pairs.
      reg R_PHCCNF bit 7, reg T_PHCCNF bit 7 */
typedef enum
{
  ixf6048_eDISABLE_FRAME_INTRAFILLING = 0,
  ixf6048_eENABLE_FRAME_INTRAFILLING
}
ixf6048_FrameIntrafilling_e;

/* Overlength packets are Accepted, or Marked as Errored in FIFO
      (where user must discard) - reg R_PHCCNF bit 6 */
typedef enum
{
  ixf6048_eACCEPT_LONG_FRAMES = 0,
  ixf6048_eLONG_FRAMES_MARKED_ERR
}
ixf6048_LongFrameHandling_e;

/* Underlength packets are Accepted, or Marked as Errored in FIFO
      (where user must discard) - reg R_PHCCNF bit 5 */
typedef enum
{
  ixf6048_eACCEPT_SHORT_FRAMES = 0,
  ixf6048_eSHORT_FRAMES_MARKED_ERR
}
ixf6048_ShortFrameHandling_e;


/* HDLC Address & Control field checking:
      Accept all, or only accept All-Stations address (0xFF)
      with Control field = 0x03 (Unnumbered Information command
      with Poll/Final bit = 0) - reg R_PHCCNF bit 4 */
typedef enum
{
  ixf6048_eRX_POS_NO_AC_CHK = 0,        /* Address * Control fields not checked */
  ixf6048_eRX_POS_AC_CHK        /* Only Accept Address 0xFF & Control 0x03 */
}
ixf6048_RxPosAcCheck_e;

/* Configure Rx FCS - reg R_PHCCNF 3:2 */
typedef enum
{
  ixf6048_eIGNORE_FCS_16 = 0,   /* 16-bit FCS - Ignored */
  ixf6048_eIGNORE_FCS_32,       /* 32-bit FCS - Ignored */
  ixf6048_eUSE_FCS_16,          /* Use CRC-16 (CRC-CCITT) */
  ixf6048_eUSE_FCS_32           /* Use CRC-32 */
}
ixf6048_RxFcs_e;

typedef struct                  /* Rx POS Channel Configuration */
{
  ixf6048_RxPosLineOrTrafficMon_e RxPosLineOrTrafficMon;
  ixf6048_RxPosByteCountCfg_e RxPosByteCountCfg;
  ixf6048_FcsErrMarkedInFifo_e FcsErrMarkedInFifo;
  ixf6048_RxPosFifoFormat_e RxPosFifoFormat;
  ixf6048_FrameDescrambling_e FrameDescrambling;
  ixf6048_SpeFrameDescrambling_e SpeFrameDescrambling;
  ixf6048_FrameIntrafilling_e FrameIntrafilling;
  ixf6048_LongFrameHandling_e LongFrameHandling;
  ixf6048_ShortFrameHandling_e ShortFrameHandling;
  ixf6048_RxPosAcCheck_e RxPosAcCheck;
  ixf6048_RxFcs_e RxFcs;
  uchar RxShortFrameLen;        /* Min Packet Len R_MINPL */
  ushort RxLongFrameLen;        /* Max Packet Len R_MAXPL */

}
ixf6048_RxPosChanCfg_t;

/***********************************************************************
 *
 * POS Tx Channel Config
 *
 **********************************************************************/

/* Configure whether Address & Control fields are generated
      by the Amazon chip (as All-Stations address (0xFF)
      with Control field = 0x03 (Unnumbered Information command
      with Poll/Final bit = 0)) or whether Address & Control fields
      are accepted from the Link Layer device.
      reg T_PHCCNF bit 1 */
typedef enum
{
  ixf6048_eTX_POS_GENERATE_AC = 0,      /* Amazon generates Address & Control */
  ixf6048_eTX_POS_ACCEPT_AC     /* Amazon accepts Address & Control */
}
ixf6048_TxPosGenAcceptAc_e;

/* Enable/Disable outgoing SPE bytes scrambling, after
                  HDLC Frame processing, with polynomial 1 + x**43
                  reg T_PHCCNF bit 0 */
typedef enum
{
  ixf6048_eDISABLE_SPE_FRAME_SCRAMBLING = 0,
  ixf6048_eENABLE_SPE_FRAME_SCRAMBLING
}
ixf6048_SpeFrameScrambling_e;

/* Enable/Disable HDLC Frame scrambling, before byte stuffing,
                  with polynomial 1 + x**29 + x**30 + x**49 
                  reg T_PHCCNF bit 8 */
typedef enum
{
  ixf6048_eDISABLE_FRAME_SCRAMBLING = 0,
  ixf6048_eENABLE_FRAME_SCRAMBLING
}
ixf6048_FrameScrambling_e;

#if(0)                          /* repeated here for documentation */

/* Enable/Disable Frame Intrafilling.
                  Configures whether frame is aborted on FIFO underflow,
                  or continued with Control Escape character pairs.
                  reg R_PHCCNF bit 7, reg T_PHCCNF bit 7 */
typedef enum
{
  ixf6048_eDISABLE_FRAME_INTRAFILLING = 0,
  ixf6048_eENABLE_FRAME_INTRAFILLING
}
ixf6048_FrameIntrafilling_e;

#endif

/* Enable/Disable Reading of POS-Packets from the Tx FIFO.
      See reg T_PHCCNF bit 6 */
typedef enum
{
  ixf6048_ePOS_MAP_FLAG_CHARS_TO_SPE = 0,
  ixf6048_ePOS_READ_PACKETS_FROM_TX_FIFO        /* Default - Normal Mode. */
}
ixf6048_ReadTxPosFifoEnable_e;

typedef enum                    /* Configure Tx FCS - reg T_PHCCNF bits 3:2 */
{
  ixf6048_eSEND_NO_FCS = 0,     /* Do not send an FCS */
  ixf6048_eSEND_FCS_16 = 2,     /* Send CRC-16 (CRC-CCITT) */
  ixf6048_eSEND_FCS_32 = 3      /* Send CRC-32 */
}
ixf6048_TxFcs_e;


/* Configures proportional filling.
                  reg T_IPGCTRL bits 14,13; Table 25,26 */
typedef enum
{
  ixf6048_eTX_POS_NO_PROPORTIONAL_FILL = 1,     /* No proportional filling */
  ixf6048_eTX_POS_FILL_LEN_DIV_2_EXP_N, /* Fill = max (1, Len / 2**N) */
  ixf6048_eTX_POS_FILL_LEN_MULT_2_EXP_N /* Fill = max (1, Len * 2**N) */
}
ixf6048_TxPosProportionalFill_e;

/* Configures constant filling.
                  reg T_IPGCTRL bits 9,8 ; Table 27,28 */
typedef enum
{
  ixf6048_eTX_POS_NO_CONSTANT_FILL = 1, /* No Constant filling */
  ixf6048_eTX_POS_FILL_MULT_1,  /* Fill = max(1, 1*IPGAbs[7:0]) */
  ixf6048_eTX_POS_FILL_MULT_256 /* Fill = max(1, 256*IPGAbs[7:0]) */
}
ixf6048_TxPosConstantFill_e;

/* Tx POS Interpacket Gap Control (Tx Flow Control) - reg T_IPGCTRL
      Note: You may use none, either, or both methods.
            Number inter-frame Flags
            = max (1, NumInterframeFlags, ProportionalFillNum) */
typedef struct
{
  ixf6048_TxPosProportionalFill_e TxPosProportionalFill;        /* Configures proportional filling
                                                                   See Bits 14, 13 */
  uchar N;                      /* N, for proportional fill, if used.
                                   See also T_IPGCTRL 12:10 */
    ixf6048_TxPosConstantFill_e TxPosConstantFill;      /* Configures Constant IFG filling: 
                                                           See also T_IPGCTRL bit 9 and 8 */
  ushort NumInterframeFlags;    /* 0..65,536 (if > 256, quantized to N*256)
                                   See also T_IPGCTRL 7:0;  table 27
                                   XmtIPGAbs[7:0] */


}
ixf6048_TxPosFlowControl_t;

typedef struct                  /* Tx POS Channel Configuration */
{
  ixf6048_TxPosGenAcceptAc_e TxPosGenAcceptAc;  /* Reg T_PHCCNF Bit 1 */
  ixf6048_SpeFrameScrambling_e SpeFrameScrambling;      /* Reg T_PHCCNF Bit 0 */
  ixf6048_FrameScrambling_e FrameScrambling;    /* Reg T_PHCCNF Bit 8 */
  ixf6048_FrameIntrafilling_e FrameIntrafilling;        /* Reg T_PHCCNF Bit 7 */
  ixf6048_ReadTxPosFifoEnable_e ReadTxPosFifoEnable;    /* Reg T_PHCCNF Bit 6 */
  ixf6048_TxFcs_e TxFcs;        /* Reg T_PHCCNF bits[3:2] */

  ixf6048_TxPosFlowControl_t TxPosFlowControl;  /* Reg T_IPGCTRL */
}
ixf6048_TxPosChanCfg_t;

/***********************************************************************
 *
 * POS Channel Config (Rx & Tx sides)
 *
 **********************************************************************/

typedef struct                  /* POS Channel Configuration */
{
  ixf6048_RxPosChanCfg_t RxPosChanCfg;
  ixf6048_TxPosChanCfg_t TxPosChanCfg;

}
ixf6048_PosChanCfg_t;


/***********************************************************************
 *
 * Complete Channel Config
 *
 **********************************************************************/
typedef enum                    /* Splits chip into Receive and Transmit sides -- (Reset) */
{
  ixf6048_eSIDE_RECEIVE = 1,
  ixf6048_eSIDE_TRANSMIT,
  ixf6048_eSIDE_BOTH
}
ixf6048_Side_e;

/* PSN_ADDED for B1 Release */
/* T_COCNF Bit [11] and R_COCNF Bit[9] 
		Scramble/descramble payload (1+X^43) in Transparent Mode */
typedef enum
{
  ixf6048_eTR_SCR_EN_SCRAMBLER_DISABLED = 0,
  ixf6048_eTR_SCR_EN_SCRAMBLER_ENABLED
}
ixf6048_TrScrEn_e;
/* PSN_END B1 Release Changes*/

/* When In Repeater mode, All MSOH, HPOH, and VC data is passed
   though internally, and no off chip connection is required.
      -- Reg COCNF Bit 5 */
typedef enum
{
  ixf6048_eCHANNEL_IN_NORMAL_MODE = 0,
  ixf6048_eCHANNEL_IN_REPEATER_MODE
}
ixf6048_RepeaterMode_e;

typedef enum                    /* Channel Mode - Reg R_COCNF T_COCNF 1:0 */
{
  ixf6048_eTEST_MODE_CHAN = 0x0,        /* Channel is repeater */
  ixf6048_eTRANSPARENT_MODE_CHAN = 0x1, /* Channel is transparent */
  ixf6048_eATM_MODE_CHAN = 0x2, /* Channel is ATM interface */
  ixf6048_ePOS_MODE_CHAN = 0x3  /* Channel is POS interface */
}
ixf6048_ChanMode_e;

typedef struct                  /* Complete Channel Configuration */
{
/* PSN_ADDED for B1 Release */
  ixf6048_TrScrEn_e TrScrEn;    /* Regs T_COCNF[11], R_COCNF Bit[9] */
/* PSN_END B1 Release Changes*/
  ixf6048_RepeaterMode_e RepeaterMode;  /* Reg COCNF Bit 5 */
  ixf6048_ChanMode_e ChanMode;  /* Reg T_COCNF R_COCNF 1:0 */
  ixf6048_ChanIfCfg_t *pChanIfCfg;      /* Channel Interface config */
  ixf6048_OhtChanCfg_t *pOhtChanCfg;    /* Channel OHT config */
  ixf6048_AtmChanCfg_t *pAtmChanCfg;    /* NULL if not ATM */
  ixf6048_PosChanCfg_t *pPosChanCfg;    /* NULL if not POS */
}
ixf6048_ChanCfg_t;

/***********************************************************************
 *
 * Combined Chip Configuration Structure - for IXF6048 (Amazon-A)
 *
 **********************************************************************/

typedef enum                    /* Specifies Single-Chan or Quad-Chan initialization;
                                   For Quan Chan: Overhead Port Config - reg OHPCNF bit 0 */
{
  ixf6048_eSINGLE_CHAN = 1,     /* Single Chan config */
  ixf6048_eQUAD_OH_ALARM_PORTS_IF,      /* Quad Channel - OH Ports I/F # 2 */
  ixf6048_eQUAD_DCC_ORDERWIRE_IF        /* Quad Channel - OH Ports I/F # 3 */
}
ixf6048_NumChanCfg_e;


typedef struct                  /* Amazon-A Chip Config struct;
                                   -> Chan 1-3 cfg = NULL, if Single Channel Config */
{
  ixf6048_NumChanCfg_e NumChanCfg;      /* Single or Quad Chan cfg */
  ixf6048_ChipLevelCfg_t *pChipLevelCfg;        /* -> Chip config */

  ixf6048_ChanCfg_t *pChanCfg[4];       /* -> Channel 0-3 config */
/*   ixf6048_ChanCfg_t*       pChan1Cfg; *//* -> Channel 1 config */
/*   ixf6048_ChanCfg_t*       pChan2Cfg; *//* -> Channel 2 config */
/*   ixf6048_ChanCfg_t*       pChan3Cfg; *//* -> Channel 3 config */
}
ixf6048_ChipCfg_t;

/***********************************************************************
 *
 * Amazon Alarm & Status Bits - Global
 *
 **********************************************************************/

/*---------------Master/Global Interupt Control Bits------------------*/

/* Register MACNF Bit 0 -- Master Interupt Enable Bit */
typedef enum
{
  ixf6048_eALARMS_ENABLED = 1,
  ixf6048_eALARMS_DISABLED
}
ixf6048_MasterIntEn_e;

/*---------------Global Interupt Source Registers Config--------------*/
    /* Generic Global Chanel Alarm Masks */

#define ixf6048_mCHAN0_ALARM 0x1111     /* Mask for channel 0 alarms */
#define ixf6048_mCHAN1_ALARM 0x2222     /* Mask for channel 1 alarms */
#define ixf6048_mCHAN2_ALARM 0x4444     /* Mask for channel 2 alarms */
#define ixf6048_mCHAN3_ALARM 0x8888     /* Mask for channel 3 alarms */

/* Register SDH_GIS  -- SDH Global Interupt Source Register   */

    /* All SONET/SDH Global Interupt Enables All Channels
       See SDH_GIS */

typedef ushort ixf6048_OhtGlobalAlarms; /* Reg SDH_GIS */

#define ixf6048_mRCV_HPT_INT       0xF000       /* Rx HPT Alarms Mask */
#define ixf6048_mRCV_ADP_INT       0x0F00       /* Rx ADP Alarms Mask */
#define ixf6048_mRCV_MUX_INT       0x00F0       /* Rx MUX Alarms Mask */
#define ixf6048_mRCV_RG_INT        0x000F       /* Rx RG Alarms Mask */

#define ixf6048_ALL_GLOBAL_SDH_ALARMS 0xFFFF    /* All SDH Alarms (all chans) */
                                          /* Note: This may be used to config
                                             GlobalSdhAlarms in your static config
                                             data. The Driver will mask off
                                             the appropriate bits for the
                                             channel being configured */

/* Register ATMPOS_GIS -- ATM and POS Global Interupt Source Register  */

    /* All ATM and POS Ints Here */
typedef ushort ixf6048_AtmPosGlobalAlarms;      /* Reg ATMPOS_GIS              */

#define ixf6048_mXMT_ATM_INT       0xF000       /* Tx ATM Alarms Mask */
#define ixf6048_mRCV_ATM_INT       0x0F00       /* Rx ATM Alarms Mask */
#define ixf6048_mXMT_POS_INT       0x00F0       /* Tx POS Alarms Mask */
#define ixf6048_mRCV_POS_INT       0x000F       /* Rx POS Alarms Mask */

#define ixf6048_mATM_ALARMS   (ixf6048_mXMT_ATM_INT | ixf6048_mRCV_ATM_INT)
#define ixf6048_mPOS_ALARMS   (ixf6048_mXMT_POS_INT | ixf6048_mRCV_POS_INT)


/* Register MISC_GIS -- Miscelaneous Global Interupt Source Register */

typedef ushort ixf6048_MiscGlobalAlarms;        /* Reg MISC_GIS */

    /* PRBSI, TALBINT, LSPINT, XMTUtoInt, RCVUtoInt here */
#define ixf6048_mGLOBAL_PRBS_ANLYZER_INT        0x0010
#define ixf6048_mGLOBAL_XMT_ALARM_BUS_INT       0x0008
#define ixf6048_mGLOBAL_LINE_SIDE_PARITY_INT    0x0004
#define ixf6048_mGLOBAL_XMT_UTOPIA_INT          0x0002
#define ixf6048_mGLOBAL_RCV_UTOPIA_ALARM        0x0001

#define ixf6048_mMISC_ALARMS   (ixf6048_mGLOBAL_PRBS_ANLYZER_INT | \
                                ixf6048_mGLOBAL_XMT_ALARM_BUS_INT | \
                                 ixf6048_mGLOBAL_LINE_SIDE_PARITY_INT)

#define ixf6048_mUTOPIA_ALARMS (ixf6048_mGLOBAL_XMT_UTOPIA_INT | \
                                ixf6048_mGLOBAL_RCV_UTOPIA_ALARM)

/*  Collected Global Alarms --   */

typedef struct
{
  ixf6048_OhtGlobalAlarms OhtGlobalAlarms;
  ixf6048_AtmPosGlobalAlarms AtmPosGlobalAlarms;
  ixf6048_MiscGlobalAlarms MiscGlobalAlarms;
}
ixf6048_GlobalAlarmCfg_t;


/***********************************************************************
 *
 * Amazon Alarm & Status Bits 
 *
 **********************************************************************/

/*--------------- SONET/SDH Alarms --------------------------------------*/

 /* Rx Regenerator Section Termination Alarm bits - Reg IS_RG */
typedef ushort ixf6048_RxRstAlarms_e;

#define ixf6048_eRX_RST_OOF_OVFL_ALM     0x0200 /* Out of Frame counter overflowed */
#define ixf6048_eRX_RST_B1_OVFL_ALM      0x0100 /* B1 Error counter overflowed */
#define ixf6048_eRX_RST_J0_UNSTABLE_ALM  0x0040 /* Change in J0 Unstable bit */
#define ixf6048_eRX_RST_J0_MISMATCH_ALM  0x0020 /* Change in J0 Mismatch bit */
#define ixf6048_eRX_RST_J0_CRC7_ALM      0x0010 /* Change in J0 CRC7 err bit */
#define ixf6048_eRX_RST_LOS_ALM          0x0008 /* Change in Loss of Signal bit */
#define ixf6048_eRX_RST_LO_CLK_ALM       0x0004 /* Change in Loss of Clock bit */
#define ixf6048_eRX_RST_LOF_ALM          0x0002 /* Change in Loss of Frame bit */
#define ixf6048_eRX_RST_OOF_ALM          0x0001 /* Change in Out of Frame bit */

/* Rx Regenerator Section Termination Status bits - Reg S_RG */
typedef ushort ixf6048_RxRstStatus_e;

#define ixf6048_eRX_RST_J0_UNSTABLE_ST   0x0040 /* J0 Unstable active */
#define ixf6048_eRX_RST_J0_MISMATCH_ST   0x0020 /* J0 Mismatch active */
#define ixf6048_eRX_RST_J0_CRC7_ST       0x0010 /* J0 CRC7 err active */
#define ixf6048_eRX_RST_LOS_ST           0x0008 /* Loss of Signal active */
#define ixf6048_eRX_RST_LO_CLK_ST        0x0004 /* Loss of Clock active */
#define ixf6048_eRX_RST_LOF_ST           0x0002 /* Loss of Frame active */
#define ixf6048_eRX_RST_OOF_ST           0x0001 /* Out of Frame active */

#define ixf6048_ALL_RX_RST_ALARMS  0x007F

/* Rx Multiplexer Section Termination Alarm bits - IS_MUX */
typedef ushort ixf6048_RxMstAlarms_e;

#define ixf6048_eRX_MST_S1_ALM           0x4000 /* Change in Rx S1 byte */
#define ixf6048_eRX_MST_K1_ALM           0x2000 /* Change in Rx K1 */
#define ixf6048_eRX_MST_K2_ALM           0x1000 /* Change in Rx K2 */
#define ixf6048_eRX_MST_B2_BIT_OVFL_ALM  0x0800 /* B2 Bit err cntr overflowed */
#define ixf6048_eRX_MST_B2_BLK_OVFL_ALM  0x0400 /* B2 Block err cntr overflowed */
#define ixf6048_eRX_MST_REI_BIT_OVFL_ALM 0x0200 /* REI Bit err cntr overflowed */
#define ixf6048_eRX_MST_REI_BLK_OVFL_ALM 0x0100 /* REI Block err cntr overflowed */
#define ixf6048_eRX_MST_S1_UNSTABLE_ALM  0x0080 /* Change in Rx S1 byte unstable */
#define ixf6048_eRX_MST_K1_UNSTABLE_ALM  0x0040 /* Change in Rx K1 byte unstable */
#define ixf6048_eRX_MST_K2_UNSTABLE_ALM  0x0020 /* Change in Rx K2 byte unstable */
#define ixf6048_eRX_MST_DEG_B2_ERR_ALM   0x0010 /* Change in Degraded B2 err stat */
#define ixf6048_eRX_MST_SF_ALM           0x0008 /* Change in MST SF status */
#define ixf6048_eRX_MST_RDI_ALM          0x0004 /* Change in MST RDI status */
#define ixf6048_eRX_MST_EXC_B2_ERR_ALM   0x0002 /* Change in Excessive B2 err stat */
#define ixf6048_eRX_MST_AIS_ALM          0x0001 /* Change in MST AIS */

#define ixf6048_ALL_RX_MST_ALARMS  0x7FFF
#define ixf6048_MOST_RX_MST_ALARMS  0x7F1F      /* All but S1,K1,K2 unstable */

/* Rx Multiplexer Section Termination Alarm Status - reg S_MUX */
typedef ushort ixf6048_RxMstStatus_e;

#define ixf6048_eRX_MST_S1_UNSTABLE_ST   0x080  /* Rx S1 byte unstable */
#define ixf6048_eRX_MST_K1_UNSTABLE_ST   0x040  /* Rx K1 byte unstable */
#define ixf6048_eRX_MST_K2_UNSTABLE_ST   0x020  /* Rx K2 byte unstable */
#define ixf6048_eRX_MST_DEG_B2_ERR_ST    0x010  /* Degraded Signal Defect active */
#define ixf6048_eRX_MST_SF_ST            0x008  /* MST Signal Fail active */
#define ixf6048_eRX_MST_RDI_ST           0x004  /* '110' detected in Rx K2[2:0] */
#define ixf6048_eRX_MST_EXC_B2_ERR_ST    0x002  /* Excessive B2 err active */
#define ixf6048_eRX_MST_AIS_ST           0x001  /* '111' detected in Rx K2[2:0] */

/* Rx Multiplexer Section Adaptation Alarm bits - reg I_ADP */
typedef ushort ixf6048_RxMsaAlarms_e;

#define ixf6048_eRX_MSA_LOPC_ALM         0x020  /* Change in LOPC status */
#define ixf6048_eRX_MSA_AU_NEG_OVFL_ALM  0x010  /* AU Neg Just Cntr Overflow */
#define ixf6048_eRX_MSA_AU_POS_OVFL_ALM  0x008  /* AU Pos Just Cntr Overflow */
#define ixf6048_eRX_MSA_LOP_ALM          0x004  /* Change in Loss of Ptr status */
#define ixf6048_eRX_MSA_NDF_ALM          0x002  /* Change in New Data Flag status */
#define ixf6048_eRX_MSA_AU_AIS_ALM       0x001  /* Change in AU AIS status */

#define ixf6048_ALL_RX_MSA_ALARMS 0x003F

/* Rx Multiplexer Section Adaptation Status bits - Reg S_ADP */
typedef ushort ixf6048_RxMsaStatus_e;

#define ixf6048_eRX_MSA_LOPC_ST          0x020  /* LossOfPointerConcatenation status */
#define ixf6048_eRX_MSA_LOP_ST           0x004  /* Loss of Ptr active */
#define ixf6048_eRX_MSA_NDF_ST           0x002  /* New Data Flag active */
#define ixf6048_eRX_MSA_AU_AIS_ST        0x001  /* AU AIS active (H1:H2 = all 1's) */


/* Rx High Order Path Termination Alarm bits - reg IS_HPT */
typedef ushort ixf6048_RxHptAlarms_e;

#define ixf6048_eRX_HPT_RDI_ALM          0x0800 /* Change in HPT RDI status */
#define ixf6048_eRX_HPT_C2_ALM           0x0400 /* Change in Rx C2 byte */
#define ixf6048_eRX_HPT_REI_OVFL_ALM     0x0200 /* HPT REI cntr overflow */
#define ixf6048_eRX_HPT_B3_OVFL_ALM      0x0100 /* B3 cntr overflow */
#define ixf6048_eRX_HPT_C2_UNSTABLE_ALM  0x0080 /* Change in Rx C2 unstable status */
#define ixf6048_eRX_HPT_J1_ALL_ZERO_ALM  0x0040 /* Change in Rx J1 unstable status */
#define ixf6048_eRX_HPT_J1_UNSTABLE_ALM  0x0020 /* Change in Rx J1 unstable status */
#define ixf6048_eRX_HPT_J1_MISMATCH_ALM  0x0010 /* Change in Rx J1 Mismatch status */
#define ixf6048_eRX_HPT_J1_CRC7_ALM      0x0008 /* Change in Rx J1 CRC7 status */
#define ixf6048_eRX_HPT_UNEQ_ALM         0x0004 /* Change in HPT Unequipped status */
#define ixf6048_eRX_HPT_SLM_ALM          0x0002 /* Change in HPT SLM status */
#define ixf6048_eRX_HPT_VC_AIS_ALM       0x0001 /* Change in VC AIS status */


#define ixf6048_ALL_RX_HPT_ALARMS       0xFFF
#define ixf6048_MOST_RX_HPT_ALARMS       0xF9F  /* All but Rx C2,J1 unstable */

/* Rx High Order Path Termination Status bits - reg S_HPT */
typedef ushort ixf6048_RxHptStatus_e;

#define ixf6048_eRX_HPT_C2_UNSTABLE_ST      0x0080      /* Rx C2 unstable active */
#define ixf6048_eRX_HPT_J1_ALL_ZERO_ALM_ST  0x0040      /* Change in Rx J1 unstable status */
#define ixf6048_eRX_HPT_J1_UNSTABLE_ST      0x0020      /* Rx J1 unstable active */
#define ixf6048_eRX_HPT_J1_MISMATCH_ST      0x0010      /* Rx J1 Mismatch active */
#define ixf6048_eRX_HPT_J1_CRC7_ST          0x0008      /* Rx J1 CRC7 error */
#define ixf6048_eRX_HPT_UNEQ_ST             0x0004      /* HPT Unequipped active */
#define ixf6048_eRX_HPT_SLM_ST              0x0002      /* HPT SLM active */
#define ixf6048_eRX_HPT_VC_AIS_ST           0x0001      /* VC AIS active */


/*--------------- ATM Alarms -----------------------------------------*/
/* Rx ATM Alarm bits - reg R_ATMINT */
typedef ushort ixf6048_RxAtmAlarms_e;

#define ixf6048_eRX_ATM_IDLE_CELL_CNTR_OVFL_ALM   0x200 /* Rx Idle cell cntr ovfl */
#define ixf6048_eRX_ATM_CELL_CNTR_OVFL_ALM        0x100 /* Rx ATM cell cntr ovfl */
#define ixf6048_eRX_ATM_UNCOR_HEC_CNTR_OVFL_ALM   0x080 /* Uncorrectable HEC err cntr ovfl */
#define ixf6048_eRX_ATM_COR_HEC_CNTR_OVFL_ALM     0x040 /* Correctable HEC err cntr ovfl */
#define ixf6048_eRX_ATM_FIFO_OVFL_CNTR_OVFL_ALM   0x020 /* Rx FIFO Overflow cntr ovfl */
#define ixf6048_eRX_ATM_LCD_ALM                   0x010 /* Change in LCD status */
#define ixf6048_eRX_ATM_OCD_ALM                   0x008 /* Change in OCD (SYNC) status */
#define ixf6048_eRX_ATM_UNCOR_HEC_ALM             0x004 /* Uncorrectable HEC err */
#define ixf6048_eRX_ATM_COR_HEC_ALM               0x002 /* Correctable HEC err */
#define ixf6048_eRX_ATM_FIFO_OVFL_ALM             0x001 /* Rx FIFO Overflow */

#define ixf6048_ALL_RX_ATM_ALARMS  0x03FF       /* All Rx ATM Alarms */

/* Rx ATM Status bits - Reg R_ATMINT */
typedef ushort ixf6048_RxAtmStatus_e;
#define ixf6048_eRX_ATM_LCD_ST           0x0800 /* LCD active (Loss of Cell Delin) */
#define ixf6048_eRX_ATM_OCD_ST           0x0400 /* OCD active (in Hunt or Presync) */


/* Tx ATM Alarm bits - T_ATMINT */
typedef ushort ixf6048_TxAtmAlarms_e;
#define ixf6048_eTX_ATM_IDLE_CELL_CNTR_OVFL_ALM  0x002  /* Tx Idle cell cntr ovfl */
#define ixf6048_eTX_ATM_CELL_CNTR_OVFL_ALM       0x001  /* Tx ATM cell cntr ovfl */


#define ixf6048_ALL_TX_ATM_ALARMS 0x0003        /* All Tx ATM Alarms */

/*--------------- POS Alarms -----------------------------------------*/
/* Rx POS Alarm bits - R_POSINT, R_POSINTEN */
typedef ushort ixf6048_RxPosAlarms_e;

#define ixf6048_eRX_POS_MAX_PKT_LEN_CNTR_OVFL_ALM  0x4000       /* Max Pkt Len Err Cntr Overflow */
#define ixf6048_eRX_POS_MIN_PKT_LEN_CNTR_OVFL_ALM  0x2000       /* Min Pkt Len Err Cntr Overflow */
#define ixf6048_eRX_POS_BYTE_CNTR_OVFL_ALM         0x1000       /* Rx Byte Cntr Overflow */
#define ixf6048_eRX_POS_FRAME_CNTR_OVFL_ALM        0x0800       /* Rx Frame Cntr Overflow */
#define ixf6048_eRX_POS_ABORT_FRAME_CNTR_OVFL_ALM  0x0400       /* Rx Aborted Frame Cntr Overflow */
#define ixf6048_eRX_POS_FCS_ERR_CNTR_OVFL_ALM      0x0200       /* Rx FCS Error Cntr Overflow */
#define ixf6048_eRX_POS_FIFO_OVFL_CNTR_OVFL_ALM    0x0100       /* Rx FIFO Ovfl Cntr Overflow */
#define ixf6048_eRX_POS_SHORT_FRAME_DISCARDED_ALM  0x0080       /* Rx Short Frame Discard Cntr Ovfl */
#define ixf6048_eRX_POS_LONG_FRAME_ALM             0x0040       /* Rx Long Frame Discard Cntr Ovfl */
#define ixf6048_eRX_POS_SHORT_FRAME_ALM            0x0020       /* Short Frame received */
#define ixf6048_eRX_POS_2_ESCAPES_ALM              0x0010       /* Control Escape pair received */
#define ixf6048_eRX_POS_ADDRESS_CONTROL_ALM        0x0008       /* Addr/Control not = 0xFF/0x3 */
#define ixf6048_eRX_POS_ABORT_FRAME_ALM            0x0004       /* Aborted frame received */
#define ixf6048_eRX_POS_FCS_ERR_ALM                0x0002       /* Frame with FCS err received */
#define ixf6048_eRX_POS_FIFO_OVFL_ALM              0x0001       /* Rx FIFO overflowed */

#define ixf6048_ALL_RX_POS_ALARMS 	0x7FFF  /* All Rx POS Alarms */

/* Tx POS Alarm bits - T_POSINT */
typedef ushort ixf6048_TxPosAlarms_e;

#define ixf6048_eTX_POS_BYTE_CNTR_OVFL_ALM         0x0040       /* Tx Byte Cntr Overflow */
#define ixf6048_eTX_POS_FRAME_CNTR_OVFL_ALM        0x0020       /* Tx Frame Cntr Overflow */
#define ixf6048_eTX_POS_ABORT_FRAME_CNTR_OVFL_ALM  0x0010       /* Tx Aborted Frame Cntr Overflow */
#define ixf6048_eTX_POS_FIFO_UNFL_CNTR_OVFL_ALM    0x0008       /* Tx FIFO Underflow Cntr Overflow */
#define ixf6048_eTX_POS_ABORT_FRAME_ALM            0x0004       /* Tx Frame aborted by Link Layer */
#define ixf6048_eTX_POS_FCS_ERR_ALM                0x0002       /* FCS Err inserted into Tx Frame */
#define ixf6048_eTX_POS_FIFO_UNFL_ALM              0x0001       /* Tx FIFO Underflow */



#define ixf6048_ALL_TX_POS_ALARMS 	0x007F  /* All Tx POS Alarms */
/*--------------- UTOPIA Alarms --------------------------------------*/
     /* UTOPIA Rx & Tx Alarms for All Channels
        See also R_UTOINT,T_UTOINT */
typedef struct
{
  ushort RxAlarms;              /* R_UTOINT */
  ushort TxAlarms;              /* T_UIOINT */
}
ixf6048_UtopiaAlarmsCfg_t;

#define ixf6048_ALL_RX_UTOPIA_ALARMS 0x000F     /* All Rx UTOPIA Alarms (all chans) */
#define ixf6048_ALL_TX_UTOPIA_ALARMS 0x0FFF     /* All Tx UTOPIA Alarms (all chans) */
#define ixf6048_POS_TX_UTOPIA_ALARMS 0x0F0F     /* All Tx POS UTOPIA Alarms (All Chans) */

#define ixf6048_mRX_FIFO_UNFL 0x000F    /* Rx FIFO Underflow alarms */
#define ixf6048_mTX_SOCI      0x0F00    /* Tx Start of Cell Incorrect alms */
#define ixf6048_mTX_PARITY    0x00F0    /* TX Parity Error alarms */
#define ixf6048_mTX_FIFO_OVFL 0x000F    /* TX FIFO Overflow alarms */

/*--------------- MISC Alarms -----------------------------------------*/
/* PRBSINT -- PRBS Analyzer Interupt Register ((000)1AH) Bits 3:0*/
typedef ushort ixf6048_PRBSAlarms;      /* PRBSINT */

#define ixf6048_mPRBS_ALARMS  0x000F    /* Reg PRBSINT 3:0 */

/* TALBINT -- Transmit Alarm Bus Interupt Reg ((000)1CH) Bits 15:0 */
typedef ushort ixf6048_TransmitAlarmBusAlarms;

#define ixf6048_mTPAL_CRC_ALARMS  0xF000        /* Reg TALBINT 15:12 */
#define ixf6048_mTPAL_BUS_ALARMS  0x0F00        /* Reg TALBINT 11:8 */
#define ixf6048_mTSAL_CRC_ALARMS  0x00F0        /* Reg TALBINT 7:4 */
#define ixf6048_mTSAL_BUS_ALARMS  0x000F        /* Reg TALBINT 3:0 */

/* LSPINT  -- Line Side Parity Interupt Register ((000)1FH) Bit 0 */
typedef ushort ixf6048_LineSideParityAlarm;     /* Reg LSPINT Bit 0 */

#define ixf6048_mLSP_ALARM  0x0001      /* Reg LSPINT Bit 0 */

/***********************************************************************
 *
 * Collected - Interesting Alarm Condition Specifications
 *
 **********************************************************************/
typedef struct                  /* SONET/SDH Channel Alarm Config */
{
  ixf6048_RxRstAlarms_e RxRstAlarms;
  ixf6048_RxMstAlarms_e RxMstAlarms;
  ixf6048_RxMsaAlarms_e RxMsaAlarms;
  ixf6048_RxHptAlarms_e RxHptAlarms;
}
ixf6048_OhtChanAlarmCfg_t;

typedef struct                  /* ATM Channel Alarm Config */
{
  ixf6048_RxAtmAlarms_e RxAlarms;
  ixf6048_TxAtmAlarms_e TxAlarms;
}
ixf6048_AtmChanAlarmCfg_t;

typedef struct                  /* POS Channel Alarm Config */
{
  ixf6048_RxPosAlarms_e RxAlarms;
  ixf6048_TxPosAlarms_e TxAlarms;
}
ixf6048_PosChanAlarmCfg_t;

/* Non Channel based MISC Alarms Config*/
typedef struct
{
  ixf6048_PRBSAlarms PRBSAlarms;        /* Reg PRBSI   3:0  */
  ixf6048_TransmitAlarmBusAlarms TransmitAlarmBusAlarms;        /* Reg TALBINT 15:0 */
  ixf6048_LineSideParityAlarm LineSideParityAlarm;      /* LSPINT Bit 0 */
}
ixf6048_MiscAlarmsCfg_t;

/* UtopiaAlarmscfg is defined above! */

typedef struct                  /* Channel Alarm Configuration */
{
  ixf6048_OhtChanAlarmCfg_t *pOhtChanAlarmCfg;
  ixf6048_UtopiaAlarmsCfg_t *pUtopiaAlarmsCfg;  /* UTOPIA Cfg Per Channel */
  ixf6048_AtmChanAlarmCfg_t *pAtmChanAlarmCfg;  /* Ignored, if not ATM Chan */
  ixf6048_PosChanAlarmCfg_t *pPosChanAlarmCfg;  /* Ignored, if not POS Chan */
  ixf6048_MiscAlarmsCfg_t *pMiscAlarmsCfg;      /* Combination of all Channels */
}
ixf6048_ChanAlarmCfg_t;


/* Collected Alarm Config - for chip */
typedef struct
{
  ixf6048_ChanAlarmCfg_t *pChanAlarmCfg[4];     /* One Per channel */
}
ixf6048_AlarmCfg_t;

/***********************************************************************
 * Status - for Alarm Callback Function
 *
 * Note: Alarms - have a bit set for each currently active interesting alarm
 *                condition, which caused an interrupt. (i.e. this represents
 *                the Interrupt Source regs, masked by interesting alarms).
 *
 * Note: The following pieces of information are undefined, unless there
 *       is an active alarm bit, in the corresponding alarm section.
 *       For example, if no Rx HPT alarm is active, then the status,
 *       counters and overhead bytes contained in the Rx HPT alarm status
 *       structure (namely ixf6048_RxHptAlarmStatus_t) will all be undefined.
 *
 *       Status - is the unmasked readout of the current Interrupt Status regs.
 *                This is not read, unless an alarm is active for this section.
 *       Counters - are read only if an interesting counter-overflow occured.
 *                  Otherwise, their content is undefined.
 *       Overhead Bytes - are read before each alarm callback,
 *                but only for those alarm sections which are active.
 **********************************************************************/

/*-------------- OHT Alarm Callback Function Status ---*/

typedef struct                  /* Rx RST Alarm Callback Status */
{
  ixf6048_RxRstAlarms_e Alarms; /* Current Interesting Alarms */
  ixf6048_RxRstStatus_e Status; /* Current Status */
  ushort OofCntr;               /* OOF Cntr, if overflowed */
  ushort B1Cntr;                /* B1 Cntr,  if overflowed */
}
ixf6048_RxRstAlarmStatus_t;


/*  Rx Multiplexer Section Termination Bytes - 
           See Regs R_K1K2, R_S1         */
typedef struct
{
  uchar K1;
  uchar K2;
  uchar S1;
}
ixf6048_RxMstBytes_t;

typedef struct                  /* Rx MST Alarm Callback Status */
{
  ixf6048_RxMstAlarms_e Alarms; /* Current Interesting Alarms */
  ixf6048_RxMstStatus_e Status; /* Current Status */
  ixf6048_RxMstBytes_t MstBytes;        /* Current overhead bytes */
  ulong B2BitCntr;              /* B2 Bit Err Cntr, if Overflow */
  ulong ReiBitCntr;             /* REI Bit Err Cntr, if Overflow */
  ushort B2BlkCntr;             /* B2 Blk Err Cntr, if Overflow */
  ushort ReiBlkCntr;            /* REI Bit Err Cntr, if Overflow */
}
ixf6048_RxMstAlarmStatus_t;

typedef struct                  /* Rx MSA Alarm Callback Status */
{
  ixf6048_RxMsaAlarms_e Alarms; /* Current Interesting Alarms */
  ixf6048_RxMsaAlarms_e Status; /* Current Status */
  ushort AuNegJustCntr;         /* AU Ptr Neg Just. Cntr, if ovfl */
  ushort AuPosJustCntr;         /* AU Ptr Pos Just. Cntr, if ovfl */
}
ixf6048_RxMsaAlarmStatus_t;



/* High Order Path Overhead Bytes - See regs R_C2, R_HPT_RDI */
typedef struct
{
  uchar C2;                     /* Rx C2 byte (HPT Signal Label) */
  uchar Rdi;                    /* Rx HPT RDI G1[3:1] and spare bit G1[0:0] */
}
ixf6048_RxHptBytes_t;

typedef struct                  /* Rx HPT Alarm Callback Status */
{
  ixf6048_RxHptAlarms_e Alarms; /* Current Interesting Alarms */
  ixf6048_RxHptStatus_e Status; /* Current Status */
  ixf6048_RxHptBytes_t HptBytes;        /* Current overhead bytes */
  ushort B3Cntr;                /* B3 Error counter, if overflowed */
  ushort ReiCntr;               /* HPT REI counter, if overflowed */
}
ixf6048_RxHptAlarmStatus_t;


typedef struct                  /* Combined OHT Alarm Status */
{
  ushort Channel;               /* Interrupting channel = 0..3 */
  ushort AisStatus;             /* Reg S_AIS masked for each channel */
  ixf6048_RxRstAlarmStatus_t RxRst;
  ixf6048_RxMstAlarmStatus_t RxMst;
  ixf6048_RxMsaAlarmStatus_t RxMsa;
  ixf6048_RxHptAlarmStatus_t RxHpt;
}
ixf6048_OhtAlarmStatus_t;


/*--------------- ATM Alarm Callback Function Status ------------*/

typedef struct                  /* Combined ATM Alarm Status */
{
  uint Channel;                 /* Interrupting channel = 0..3 */
  ixf6048_RxAtmAlarms_e RxAlarms;
  ixf6048_RxAtmStatus_e RxStatus;
  ixf6048_TxAtmAlarms_e TxAlarms;
  /* Following counters read, only on overflow */
  ulong RxAtmCellCntr;          /* Rx Accepted ATM Cell counter */
  ulong RxIdleCellCntr;         /* Rx Idle Cell counter */
  ushort RxCorHecCntr;          /* Rx Correctable HEC error counter */
  ushort RxUncorHecCntr;        /* Rx Uncorrectable HEC error counter */
  ushort RxFifoOvflCntr;        /* Cnt of OK Rx Cells lost due to Rx FIFO Ovfl */
  ulong TxAtmCellCntr;          /* Tx ATM Cell counter */
  ulong TxIdleCellCntr;         /* Tx Idle Cell counter */
}
ixf6048_AtmAlarmStatus_t;


/*-------------- POS Alarm Callback Function Status ------------*/

typedef struct                  /* Combined POS Alarm Status */
{
  uint Channel;                 /* Interrupting channel = 0..3 */
  ixf6048_RxPosAlarms_e RxAlarms;
  ixf6048_TxPosAlarms_e TxAlarms;
  /* Following counters read, only on overflow */
  ulong RxFrameCntr;            /* Rx Accepted Frame counter */
  ulong RxByteCntr;             /* Rx Accepted Byte counter */
  ulong RxAbortFrameCntr;       /* Rx Aborted Frame counter */
  ulong RxFcsErrCntr;           /* Rx FCS Error counter */
  ushort RxFifoOvflCntr;        /* Cnt of Rx Frames lost to Rx FIFO Ovfl */
  ulong RxShortFrameCntr;       /* Cnt of Rx Frames shorter than Min Len */
  ushort RxLongFrameCntr;       /* Cnt of Rx Frames longer than Max len */
  ulong TxFrameCntr;            /* Tx Frame counter */
  ulong TxByteCntr;             /* Tx Byte counter */
  ulong TxAbortFrameCntr;       /* Tx Aborted (by Link Layer) Frame counter */
  ushort TxFifoUnflCntr;        /* Cnt of Tx Frames Aborted on Tx FIFO Underfl */
}
ixf6048_PosAlarmStatus_t;

/*-------------- UTOPIA Alarm Callback Function Status ------------------*/

typedef struct                  /* UTOPIA Alarm Status */
{
  uint Channel;                 /* Interrupting channel = 0..3 */
  ushort RxAlarms;              /* UTOPIA RX Statuses for all channels */
  ushort TxAlarms;              /* UTOPIA TX Statuses for all channels */
}
ixf6048_UtopiaAlarmStatus_t;


/*-------------- MISC Alarm Callback Function Status ------------*/

typedef struct                  /* Combined MISC Alarm Status */
{
  uint Channel;                 /* Interrupting channel = 0..3 */
  ixf6048_PRBSAlarms PRBSAlarms;        /* Reg PRBSI   3:0  */
  ixf6048_TransmitAlarmBusAlarms TxAlarmBusAlarms;      /* Reg TALBINT 15:0 */
  ixf6048_LineSideParityAlarm LineSideParityAlarm;      /* LSPINT Bit 0 */
}
ixf6048_MiscAlarmStatus_t;

/***********************************************************************
 * Chip Info for Amazon Chips -- See also Pg 162
 **********************************************************************/

typedef struct                  /* Information set by the hardware */
{
  uchar ChipVersion;            /* Chip Version */
  uchar ChipId;                 /* Chip ID */
}
ixf6048_ChipInfo_t;

/***********************************************************************
 * Run Amazon Channel Tests
 *
 * Channel must be in the test mode, before calling bb_Test.
 *
 * pTestCfg -> ixf6048_ChanTestCfg_t
 *
 * Tests include:
 *    o  SDH Test  -- To introduce SONET/SDH errors
 *                   -->Introduce Rx Sdh Errors
 *                   -->Introduce Tx Sdh Errors
 *    o  Ptr Test  -- To test AU pointers and/or SS bits
 *    o  ATM Test  -- To introduce ATM (HEC) errors
 *    o  POS Test  -- To introduce POS (FCS) errors
 *    o  Loopback  -- To do local or line loopback
 *
 * When the Channel Test mode is first entered, no errors are generated.
 *
 * Note that going from Channel Test mode to a non-Test Channel mode will
 * stop the introduction of errors, and restore the previous values of the
 * AU pointers and SS bits. Any in-progress loopbacks will be terminated.
 **********************************************************************/

/*----------------- Force one or more Error Conditions ---------------*/
/* Error Conditions for ixf6048_ChanSdhTest function */
typedef ushort ixf6048_ChanRxSdhTest_e;

#define ixf6048_eFORCE_CNTR_OVFL 	0x0001  /* Force overflow on B1, B2 Bit/Blk, */
#define ixf6048_eCLEAR_CNTR_OVFL	0x0002  /* M1 REI Bit/Blk, B3 Bit/Blk, G1 REI */
#define ixf6048_eFORCE_RX_RST_AIS	0x0004  /* Force AIS from RST to MST, to simulate */
#define ixf6048_eCLEAR_RX_RST_AIS	0x0008  /* loss of Rx Frame or Rx Signal Fail */

#define ixf6048_eFORCE_RX_MST_AIS	0x0010  /* Force AIS from MST to MSA, to simulate */
#define ixf6048_eCLEAR_RX_MST_AIS	0x0020  /* Excessive B2 Errors Condition. */

#define ixf6048_eFORCE_RX_MST_RDI	0x0040  /* Force Rx MST RDI: K2[2:0] = '110' */
#define ixf6048_eCLEAR_RX_MST_RDI	0x0080  /* simulating Excessive B2 Errors */
                                   /* Condition */
#define ixf6048_eFORCE_RX_MSA_AIS	0x0100  /* Force AIS from MSA to HPT, to simulate */
#define ixf6048_eCLEAR_RX_MSA_AIS	0x0200  /* AIS detected in received AU Pointer 
                                                   (H1:H2 bytes), or Loss of Pointer
                                                   (i.e. invalid pointer value, or
                                                   (SS bits not equal configured value
                                                   with AU Pointer SS bits enabled)). */
#define ixf6048_eFORCE_RX_HPT_AIS	0x0400  /* Force AIS from HPT to HPA, to simulate */
#define ixf6048_eCLEAR_RX_HPT_AIS	0x0800  /* HPT Signal Label Mismatch, or
                                                   HPT Unequipped, or
                                                   J1 String Mismatch */


/* Error Conditions for ixf6048_ChanSdhTest function */
typedef ushort ixf6048_ChanTxSdhTest_e;

#define ixf6048_eFORCE_TX_MST_RDI	0x0001  /* /Clear/Force Tx MST RDI: K2[2:0] = '110' */
#define ixf6048_eCLEAR_TX_MST_RDI	0x0002  /* simulating Excessive B2 Errors
                                                   Condition -- This is Achieved by Setting 
                                                   MST_RDI Source to R_MST_C and then 
                                                   Forcing R_MST_RDI */
#define ixf6048_eFORCE_TX_MSA_AIS	0x0004  /* Force AIS at the pointer processing */
#define ixf6048_eCLEAR_TX_MSA_AIS	0x0008  /* block level (MSA) towards the SDH
                                                   network */
#define ixf6048_eFORCE_TX_HPT_AIS	0x0010  /* Force AIS at the HPT level towards */
#define ixf6048_eCLEAR_TX_HPT_AIS	0x0020  /* the SDH network */

#define ixf6048_eFORCE_4_A1_ERRS 	0x0040  /* Invert A1 byte for 4 frames */
#define ixf6048_eFORCE_A1_ERRS   	0x0080  /* Invert A1 byte in each frame */

#define ixf6048_eFORCE_ONE_B1_ERR	0x0100  /* Invert B1 byte for one frame */
#define ixf6048_eFORCE_B1_ERRS   	0x0200  /* Invert B1 byte in each frame */

#define ixf6048_eFORCE_ONE_B2_ERR	0x0400  /* Invert B2 byte for one frame */
#define ixf6048_eFORCE_B2_ERRS   	0x0800  /* Invert B2 byte in each frame */

#define ixf6048_eFORCE_ONE_B3_ERR	0x1000  /* Invert B3 byte for one frame */
#define ixf6048_eFORCE_B3_ERRS   	0x2000  /* Invert B3 byte in each frame */



typedef struct
{
  ixf6048_ChanRxSdhTest_e RxSdhTest;
  ixf6048_ChanTxSdhTest_e TxSdhTest;
}
ixf6048_ChanSdhTest_t;
/*----------------- Change Tx AU Pointers or SS bits --------------*/

/* Note:
   The channel must be in the test mode to call this function.

   The initial value of the AU Ptr and the SS bits are saved, the first
   time the ixf6048_ChanPtrTest function is called.

   The initial pointer value may be set to a valid value (0-782) or an
   invalid value (783-1023).

   When the channel leaves the test mode, if the AU Ptr or SS bits have
   been changed by a call to the ixf6048_ChanPtrTest function, then the
   original values of the AU Ptr and SS bits are automatically restored,
   by the Driver.

   See also Pg 241,242.
*/

/* Tx AU Pointer actions for ixf6048_ChanPtrTest */
typedef ushort ixf6048_ChanPtrTest_e;

#define ixf6048_eSET_AU_PTR_TEST      0x01      /* Set initial AU Ptr value */
#define ixf6048_eSET_AU_PTR_SS_TEST   0x02      /* Set SS bits value */
#define ixf6048_eINC_AU_PTR_TEST      0x04      /* Increment AU Ptr */
#define ixf6048_eDEC_AU_PTR_TEST      0x08      /* Decrement AU Ptr */
#define ixf6048_eSET_NDF_PTR_TEST     0x10      /* Send An NDF */


/*----------------- Cause ATM Errors in a Channel --------------------*/
/* Note:
   The channel must be in the test mode to call this function.

   This test involves forcing correctable or uncorrectable HEC errors
   into the ATM cell.
*/

/* ATM errors */
typedef ushort ixf6048_ChanAtmTest_e;

#define ixf6048_eSTOP_ATM_ERRS        0x0       /* Stop ATM errors */
#define ixf6048_eFORCE_1_COR_ERR      0x1       /* Force 1 correctable error */
#define ixf6048_eFORCE_COR_ERRS       0x2       /* Force correctable errors */
#define ixf6048_eFORCE_1_UNCOR_ERR    0x4       /* Force 1 uncorrectable error */
#define ixf6048_eFORCE_UNCOR_ERRS     0x8       /* Force uncorrectable errors */


/*----------------- Cause POS HDLC Errors in a Channel --------------------*/

/* Note:
   The channel must be in the test mode to call this function.

   This test involves forcing FCS errors into the Tx HDLC frames.
*/

/* POS errors */
typedef ushort ixf6048_ChanPosTest_e;

#define ixf6048_eSTOP_POS_ERRS        0x0       /* Stop POS HDLC errors */
#define ixf6048_eFORCE_1_FCS_ERR      0x1       /* Force 1 FCS error */
#define ixf6048_eFORCE_FCS_ERRS       0x2       /* Force FCS errors */


/*----------------- Do local or line loopbacks for a Channel ---------------*/

/* Note:
   The channel must be in the test mode to call this function.

   The Local Loopback -- loops back data, inside the Amazon chip, from the
   Tx side to the Rx side.

   The Line Loopback -- loops back received SONET/SDH data and resends it back
   to the SONET/SDH network. This requires that the Rx and Tx Line Side
   interfaces are the same: namely RxLineSideInterface = TxLineSideInterface
   in MuxChanIfConfig.

   If the Application and Chip Operational Modes are Online, then loopback
   data will be sent.

   In the Single Channel mode, only channel 0 is available.

   See also reg COCNF bit 6,5
*/

typedef enum                    /* Set type of channel loopback */
{
  ixf6048_eCHAN_LOOPBACK_OFF,   /* Stop channel loopback(s) */
  ixf6048_eCHAN_LOCAL_LOOPBACK, /* Do local channel loopback, inside chip */
  ixf6048_eCHAN_LINE_LOOPBACK,  /* Loop Rx SDH data back to SDH network */
  ixf6048_eCHAN_BOTH_LOOPBACKS  /* Do both local and line loopbacks */
}
ixf6048_ChanLoopback_e;

/*----------------- Test or Loopback Configuration ---------------*/

typedef enum                    /* Define set of tests to run */
{
  ixf6048_eCHAN_SDH_TESTS,      /* To introduce SONET/SDH errors */
  ixf6048_eCHAN_PTR_TESTS,      /* To test AU pointers and/or SS bits */
  ixf6048_eCHAN_ATM_TESTS,      /* To introduce ATM (HEC) errors */
  ixf6048_eCHAN_POS_TESTS,      /* To introduce POS (FCS) errors */
  ixf6048_eCHAN_LOOPBACKS       /* To do local or line loopback */
}
ixf6048_ChanTests_e;

typedef struct                  /* AU Pointer Test Config */
{
  ixf6048_ChanPtrTest_e ChanPtrTest;    /* AU Ptr tests */
  ushort InitialPtr;            /* Initial Ptr val, if needed */
  uchar SsBits;                 /* SS bits, if needed */
}
ixf6048_ChanPtrTestCfg_t;

typedef struct                  /* Amazon Test config */
{
  ixf6048_ChanTests_e ChanTests;        /* Defines set of tests to run */
  int ChanNum;                  /* Channel Num = 0-3 */
  union
  {
    ixf6048_ChanSdhTest_t ChanSdhTest;  /* SDH tests to run */
    ixf6048_ChanPtrTestCfg_t ChanPtrTestCfg;    /* AU Ptr tests to run */
    ixf6048_ChanAtmTest_e ChanAtmTest;  /* ATM tests to run */
    ixf6048_ChanPosTest_e ChanPosTest;  /* POS tests to run */
    ixf6048_ChanLoopback_e ChanLoopback;        /* Loopback specification */
  }
  u;
}
ixf6048_ChanTestCfg_t;


/***********************************************************************
 * Data for functions to Get various Overhead Bytes:
 **********************************************************************/

#if(0)                          /* Repeated from above - for documentation */

typedef struct                  /* High Order Path Overhead Bytes */
{
  uchar C2;                     /* Rx C2 byte (HPT Signal Label) */
  uchar Rdi;                    /* Rx HPT RDI G1[3:1] and spare bit G1[0:0] */
}
ixf6048_RxHptBytes_t;

#endif

/***********************************************************************
 * For function to Get Counters: OHT, ATM, POS
 **********************************************************************/

typedef struct                  /* All (Amazon) OHT Counters */
{
  bb_RxRstCounters_t RxRst;
  bb_RxMstCounters_t RxMst;
  bb_RxHptCounters_t RxHpt;
  bb_AuPtrCounters_t RxMsa;
}
ixf6048_OhtCounters_t;


typedef struct                  /* ATM Counters; Reading clears counters */
{
                           /*-------- Rx ATM Counters --------*/
  ulong RxCellCnt;              /* 24-bit count of accepted Rx ATM cells */
  ulong RxIdleCellCnt;          /* 24-bit count of idle/unassigned cells */
  ushort RxCorHecErrCnt;        /* 16-bit count of correctable HEC errors */
  ushort RxUncorHecErrCnt;      /* 16-bit count of uncorrectable HEC errors */
  ushort RxOvflCellCnt;         /* 16-bit count of cells lost to FIFO overrun */

                           /*-------- Tx ATM Counters --------*/
  ulong TxCellCnt;              /* 24-bit count of Tx ATM cells */
  ulong TxIdleCellCnt;          /* 24-bit count of idle/unassigned cells sent */

}
ixf6048_AtmCounters_t;


typedef struct                  /* POS Counters; Reading clears counters */
{
                           /*-------- Rx POS Counters  --------*/
  ulong RxFrameCnt;             /* 27-bit count of accepted Rx frames */
  ulong RxByteCnt;              /* 29-bit count of Rx Bytes (in OK frames,
                                   or in all frames, as configured by
                                   RxPosByteCountConfig in RxPosChanConfig) */
  ulong RxAbtFrameCnt;          /* 20-bit count of Aborted frames */
  ulong RxFcsErrCnt;            /* 20-bit count of frames with FCS errors */
  ushort RxOvflFrameCnt;        /* 16-bit count of frames lost to FIFO overflow */
  ulong RxShortFrameCnt;        /* 27-bit count of short frames received */
  ushort RxLongFrameCnt;        /* 16-bit count of long frames received */

                           /*-------- Tx POS Counters  --------*/
  ulong TxFrameCnt;             /* 27-bit count of (non-aborted) frames sent */
  ulong TxByteCnt;              /* 29-bit count of all Tx bytes put into SPE */
  ulong TxAbtFrameCnt;          /* 20-bit count of frames aborted by link layer */
  ushort TxUnflFrameCnt;        /* 16-bit count of frames aborted by underflow */
}
ixf6048_PosCounters_t;

                           /*------------- All Counters --------*/

typedef struct                  /* All Amazon counter sets */
{
  ixf6048_OhtCounters_t OhtCounters;    /* OHT counters for channel */
  ixf6048_ChanMode_e ChanMode;  /* ATM or POS Channel */

  union
  {
    ixf6048_AtmCounters_t Atm;  /* ATM Counters, or */
    ixf6048_PosCounters_t Pos;  /* POS Counters */
  }
  u;
}
ixf6048_AllCounters_t;

/***********************************************************************
 * For Reset Function
 **********************************************************************/

typedef enum                    /* Specifies Chip or Channel Reset */
{
  ixf6048_eRESET_CHIP,          /* Reset entire chip; then reconfig chip */
  ixf6048_eRESET_RX_CHAN,       /* Reset Rx side of Channel; then recfg Rx Side */
  ixf6048_eRESET_TX_CHAN,       /* Reset Tx side of Channel; then recfg Tx Side */
  ixf6048_eRESET_CHAN,          /* Reset both sides of Channel; then recfg chan */
  ixf6048_eRESET_RX_FIFO,       /* Reset the receive FIFO */
  ixf6048_eRESET_TX_FIFO        /* Reset the transmit FIFO */
}
ixf6048_ResetType_e;

/***********************************************************************
 * For function to Get Status: Rx AIS, Input Clock Activity, Far-end GFC Role
 **********************************************************************/

typedef enum                    /* To select type of status to get */
{
  ixf6048_eRX_AIS_STATUS,       /* pStatus -> ushort: Reg 0x7) */
  ixf6048_eIN_CLK_STATUS,       /* pStatus -> ixf6048_InClkStatus_t */
  ixf6048_eFAR_END_GFC_ROLE     /* pStatus -> ixf6048_FarEndGfcRole_e */
}
ixf6048_SelStatus_e;

typedef struct                  /* To hold status on Input Clock Activity */
{
  ushort Icmr1Reg;              /* ICMR1 register (0x12) */
  ushort Icmr2Reg;              /* ICMR2 register (0x13) */
}
ixf6048_InClkStatus_t;


typedef enum                    /* Identification of far-end's GFC role - R_ACPCNF bits 10,9 */
{
  ixf6048_eFAR_END_GFC_CONTROLLER = 0x400,
  ixf6048_eFAR_END_GFC_CONTROLLED = 0x200
}
ixf6048_FarEndGfcRole_e;

/***********************************************************************
 * For function to Set Config: POS Watermarks, POS Flow Control, Chip's GFC Role
 **********************************************************************/

typedef enum                    /* To select type of config to set */
{
  ixf6048_eCFG_POS_WATERMARKS,  /* pCfg -> ixf6048_PosWatermarks_t */
  ixf6048_eCFG_POS_FLOW_CONTROL,        /* pCfg -> ixf6048_TxPosFlowControl_t */
  ixf6048_eCFG_GFC_ROLE         /* pCfg -> ixf6048_GfcRole_t */
}
bb_6048_SelCfg_e;


/***********************************************************************
 * Data to configure Chip's GFC Role
 **********************************************************************/

typedef struct                  /* Configure Chip's GFC Role */
{
  ixf6048_GfcRole_e GfcRole;    /* this chip's GFC role */
  ixf6048_GfcHaltRatio_e GfcHaltRatio;  /* GFC Halt Ratio (if controller) */
}
ixf6048_GfcRole_t;


#endif /* BB6048D_H_ */
