/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: nmafzal $
*   DATE:           $Date: 2002/10/31 00:56:05 $
*   LAST MODIFIED:  $Modtime: 4/16/02 12:04p $
*   REV#:           $Revision: 1.1.1.1 $
*   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf_api_ether.c $
*   WORKFILE:       $Workfile: ixf_api_ether.c $
*
* --------------------------------------------------------------------------
* Project: IXF API
*
* Purpose: XGMAC portion of the IXF API
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: ixf_api_ether.c,v $
*   Revision 1.1.1.1  2002/10/31 00:56:05  nmafzal
*   putting it in proper place
*
*   Revision 1.1  2002/10/30 13:00:01  machambe
*   Initial CVS checkin
*
 * 
 * 8     4/16/02 2:47p Jaalliso
 * Renaming file to ixf_api_ether.c
 * 
 * 7     4/05/02 12:34p Machambe
 * Adding checking of pointers tables
* 
* 6     2/14/02 5:06p Srodrig1
* 
* 5     2/14/02 2:51p Jaalliso
* Changing return code bb_INV_CHIP_TYPE to bb_FN_NOT_SUPPORTED
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#else
#ifndef LINUX                   /* rammj: Changed this */
#include "memory.h"
#endif
#endif

#include "devlib.h"             /* rammj: Changed order of inclusion */
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "builds.h"
#include "ixf_api_ether.h"


/****************************************************************************
* Function : IxfApiGetMacAddress
*----------------------------------------------------------------------------
* Description : This function returns the MAC (Media Access Control)address 
*               for the selected chip. This is a 64 bits address 
*----------------------------------------------------------------------------
* Inputs : pChipData : pointer to chip data structure.
*          section : pointer to chip segment structure. Not used, only for 
*                    possible modifications. 
*          
* Outputs : address : Data structure that will hold the MAC address.
*----------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE : Invalid Chip it doesn't have the MAC Functionality.
*           bb_NO_ERROR: Successful completion
*----------------------------------------------------------------------------
* Access Globals : None
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/

bb_Error_e
IxfApiGetMacAddress (bb_ChipData_t * pChipData,
                     bb_ChipSegment_t * section,
                     IxfApi_MacAddress_t * address)
{
  XgmacFunctionPointers_t *pChipFuncPtrs;
  bb_Error_e rc = bb_NO_ERROR;

  /* Check to see if the specific chip's function pointer table
     has be initialized */
  if (((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
      if (((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr ==
          NULL)
        {
          /* Set Return Code */
          rc = bb_FN_NOT_SUPPORTED;
        }
    }
  else
    {
      /* Set Return Code */
      rc = bb_FN_NOT_SUPPORTED;
    }

  /* Assign the chip's COMMON function pointers */
  if (rc != bb_FN_NOT_SUPPORTED)
    {
      pChipFuncPtrs =
        ((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr;

      /* If function pointer is valid */
      if (pChipFuncPtrs->GetMacAddressPtr)
        {
          /* Call Function Pointer */
          rc =
            ((pChipFuncPtrs->GetMacAddressPtr) (pChipData, section, address));
        }
      else
        {
          rc = bb_INV_CHIP_TYPE;
        }
    }

  return rc;
}


/****************************************************************************
* Function : IxfApiSetMacAddress
*----------------------------------------------------------------------------
* Description : This function returns the MAC (Media Access Control)address 
*               for the selected chip. This is a 64 bits address 
*----------------------------------------------------------------------------
* Inputs : pChipData : pointer to chip data structure.
*          section : pointer to chip segment structure. Not used, only for 
*                    possible modifications. 
*          address : Data structure that will hold the MAC address to be set
*                    in the chip.
*          
* Outputs : 
*----------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE : Invalid Chip it doesn't have the MAC Functionality.
*           bb_NO_ERROR: Successful completion
*----------------------------------------------------------------------------
* Access Globals : None
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
bb_Error_e
IxfApiSetMacAddress (bb_ChipData_t * pChipData,
                     bb_ChipSegment_t * section,
                     IxfApi_MacAddress_t * address)
{
  XgmacFunctionPointers_t *pChipFuncPtrs;
  bb_Error_e rc = bb_NO_ERROR;

  /* Check to see if the specific chip's function pointer table
     has be initialized */
  if (((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
      if (((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr ==
          NULL)
        {
          /* Set Return Code */
          rc = bb_FN_NOT_SUPPORTED;
        }
    }
  else
    {
      /* Set Return Code */
      rc = bb_FN_NOT_SUPPORTED;
    }

  /* Assign the chip's COMMON function pointers */
  if (rc != bb_FN_NOT_SUPPORTED)
    {
      pChipFuncPtrs =
        ((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr;

      /* If function pointer is valid */
      if (pChipFuncPtrs->SetMacAddressPtr)
        {
          /* Call Function Pointer */
          rc =
            ((pChipFuncPtrs->SetMacAddressPtr) (pChipData, section, address));
        }
      else
        {
          rc = bb_INV_CHIP_TYPE;
        }
    }

  return rc;
}
