/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: nmafzal $
*   DATE:           $Date: 2002/10/31 00:56:05 $
*   LAST MODIFIED:  $Modtime: 5/14/02 4:06p $
*   REV#:           $Revision: 1.1.1.1 $
*   LOGFILE:        $Logfile:  $
*   WORKFILE:       $Workfile: linuxossupport.c $
*
* --------------------------------------------------------------------------
* Project: Angel Island
*
* Purpose: Provides the implementation to the linux support routines.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: linuxossupport.c,v $
*   Revision 1.1.1.1  2002/10/31 00:56:05  nmafzal
*   putting it in proper place
*
*   Revision 1.1  2002/10/30 13:00:01  machambe
*   Initial CVS checkin
*
*   Revision 1.3  2002/10/25 20:24:49  machambe
*   no message
*
* --------------------------------------------------------------------------
*/

#include<stdlib.h>
#include<stdio.h>
#include<unistd.h>
#include<semaphore.h>
#include"linuxossupport.h"

/**************************************************************/
/* Example Linux support that requires special processing to  */
/* perform some function found in the ICG OS Abstraction      */
/* Layer.                                                     */
/**************************************************************/

sem_t *
LinuxSemCreate ()
{
  sem_t *s;

  s = malloc (sizeof (sem_t));
  sem_init ((sem_t *) s, 0, 0);
  return (sem_t *) s;
}                               /* sem_t *LinuxSemCreate() */

int
LinuxSemTake (sem_t * id, int timeout)
{
  int interval, counter;

  if (sem_trywait (id) == 0)
    {
      /* Semaphore was ready to be taken */
      printf ("Semaphore acquired immediately!\n");
      return 0;
    }                           /* if */

  /* At this point, the semaphore was blocked last time we tried. */
  /* So we will try again 4 times, each after waiting one fourth  */
  /* of the time indicated in the timeout. Other schemes are      */
  /* possible such as an exponential wait (1,2,4, etc. ms until   */
  /* the indicated time) and may be worth trying.                 */
  interval = 250 * timeout;

  for (counter = 0; counter < 4; counter++)
    {
      printf ("Semaphore not acquired - waiting %dms...", interval / 1000);
      usleep (interval);
      if (sem_trywait (id) == 0)
        {
          /* Semaphore was ready to be taken */
          printf ("got it!\n");
          return 0;
        }                       /* if */
      printf ("failed.\n");
    }                           /* for */

  printf ("Timeout expired - giving up!\n", interval);
  return -1;
}

void
LinuxSemDestroy (sem_t * id)
{
  sem_destroy (id);
  free (id);
}                               /* LinuxSemDestroy() */
