/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: nmafzal $
*   DATE:           $Date: 2003/02/13 23:11:41 $
*   LAST MODIFIED:  $Modtime: 5/14/02 4:06p $
*   REV#:           $Revision: 1.3 $
*   LOGFILE:        $Logfile:  $
*   WORKFILE:       $Workfile: slowport.c $
*
* --------------------------------------------------------------------------
* Project: Angel Island
*
* Purpose: Provides the implementation to the slow port read and write routines.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: slowport.c,v $
*   Revision 1.3  2003/02/13 23:11:41  nmafzal
*   added B0 support, fixed some issues with slowport read function. Only write is effected not the read.
*
*   Revision 1.2  2002/11/20 18:21:52  nmafzal
*   Made B0 changes to work on A1 and B0 board. The, basically swapping is not needed in B0 NPU
*
*   Revision 1.1.1.1  2002/10/31 00:56:05  nmafzal
*   putting it in proper place
*
*   Revision 1.1  2002/10/30 13:00:02  machambe
*   Initial CVS checkin
*
*   Revision 1.3  2002/10/25 20:24:49  machambe
*   no message
*
* --------------------------------------------------------------------------
*/
#include <asm/arch/hardware.h>
#include "common_types.h"
#include "common_def.h"
#include "slowport.h"

#define SWAPNTOHS(x)    ((((x) & 0x00ff) << 8) |  (((x) & 0xff00) >> 8))

#ifdef SLOW_PORT
ushort
SlowPortRead (bb_RegPointer_type addr)
{


 	if (get_npu_product_id(MAJ_REV) == 0) /*A1 silicon*/ {
        //printf("A1: reading %08p (0x%04x)\n", addr, *addr); 
  		return SWAPNTOHS (*addr);
	}
    	else {
        //printf("B0: reading %08p (0x%04x)\n", addr, *addr); 
  		return SWAPNTOHS (*addr);
	}
}

ushort
SlowPortWrite (bb_RegPointer_type addr, ushort val)
{

 /* the following piece of code which swaps the addr bytes fixes the IXP2400
   slow port write bug.This routine should not be called in the B0 silicon */

    if(get_npu_product_id(MAJ_REV) ==0) /*A1 silicon*/
    {
        if ((ulong)addr & 0x2)
            (ulong) addr &= 0xFFFFFFFD;
        else
            (ulong) addr |= 0x00000002;
    }

//printf("writing %08p \'0x04\' (0x%04x)\n", addr, (((ulong) addr >> 1) & 0xFFFF), val);
  	*addr = val;
  	return bb_NO_ERROR;
}
#endif /* SLOW_PORT */
