/*
 * ============================================================================
 *  COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_linux_driver.h
 *
 * = DESCRIPTION
 *
 *
 * = CHANGE HISTORY
 *       2/18/2003 21:54:24 PM - creation time
 *
 * ============================================================================
 * $Id: ixf6048_linux_driver.h,v 1.2 2003/06/03 05:38:57 cmah1 Exp $
 */

#if !defined(__IXF6048_LINUX_DRIVER_H__)
#define __IXF6048_LINUX_DRIVER_H__

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>


#define IXF6048_BASE_ADDRESS	0xc6000000
#define DEVICE_SIZE		0x20000
#define BDREW_CARD	0
#define BWALLACE_CARD	1
#define OPT_POS_MODE	0
#define OPT_ATM_MODE	1

#define GET32(address)		(*(volatile unsigned int *)(address))

/**
 * dynamic allocation of the Major number from the kernel
 */
#define g_MajorNumber 250

/* Definitions used for MSF PLL */
#define PLL_104MHZ	0xA
#define PLL_125MHZ	0xF

#define MSF_CLKCFG_MEDIA	PLL_104MHZ
#define MSF_CLKCFG_SF		PLL_125MHZ

#define MSF_CLKCFG_IN	(MSF_CLKCFG_MEDIA | (MSF_CLKCFG_SF << 4))
#define MSF_CLKCFG_EG	((MSF_CLKCFG_MEDIA << 4) | MSF_CLKCFG_SF)

#define MCCR_MASK1	0xF0F0
#define MCCR_MASK2	0xF0
#define MCCR_MASK3	0x0


/**
 * Prototypes for interface functions.
 */
/**
 * NAME:  init_module
 *
 * DESCRIPTION:
 *  init_module function is called by the system call insmod command call.
 *  This function registers the specified device for the specified major
 *   number driver
 * @Param:  - IN    : NULL
 * @Param:  - INOUT : NULL
 * @Param:  - OUT   : void
 *
 * @Return: int value
 */
int init_module (
                  void
                 );


/**
 * NAME:  cleanup_module
 *
 * DESCRIPTION:
 *  Called by rmmod at the end to remove the BDW AtmPos
 *  Media Card device driver from kernel
 *
 * @Param:  - IN    : NULL
 * @Param:  - INOUT : NULL
 * @Param:  - OUT   : NULL
 *
 * @Return: NULL
 */
void cleanup_module (
                     void
                    );


/**
 * NAME: AtmPos_open
 *
 * DESCRIPTION:Initialize the Device
 *
 * @Param:  - IN    : struct inode *my_inode
 *                  : struct file *my_file
 * @Param:  - INOUT :
 * @Param:  - OUT   :
 *
 * @Return:
 */
int AtmPos_open(
                struct inode *my_inode,
                struct file *my_file
               );

/**
 * NAME:
 *
 * DESCRIPTION:
 *  Called to close the device, opened by the open call.
 *  Deallocate anything that open allocated in AtmPOS_file->private_data.
 *  Shut down the Device on the last close.
 *
 * @Param:  - IN    : struct inode *my_inode
 *                  : struct file * my_file
 * @Param:  - INOUT :
 * @Param:  - OUT   :
 *
 * @Return:
 */
int AtmPos_close(
                 struct inode *my_inode,
                 struct file * my_file
                );

/**
 * NAME:
 *
 * DESCRIPTION: This function will
 *
 * @Param:  - IN    : struct inode *
 *                  : struct file *
 *                  : uint32
 * @Param:  - INOUT : void *
 * @Param:  - OUT   :
 *
 * @Return:
 */
int AtmPos_ioctl(
                 struct inode *,
                 struct file *,
                 unsigned int ,
                 unsigned long
                );
/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* __IXF6048_LINUX_DRIVER_H__ */
