/*
 * ============================================================================
 * Author:      Mark Chambers <mark.a.chambers@intel.com>
 * Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * This software is supplied under the terms of a licence agreement or
 * nondisclosure agreement with Intel Corporation and may not be copied
 * or disclosed except in accordance with the terms in that agreement.
 *
 * Ixf6048 Interrupt handler
 * ixf6048_irq.c
 *
 * ============================================================================
*/


#include <linux/config.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>
#include <linux/init.h>
#include <asm/poll.h>


typedef int sint;
typedef short sshort;
typedef char schar;
typedef unsigned char uchar;
typedef unsigned char boolean;  /* used in function calls */

#include "common_def.h"
#include "ixf_api_d.h"          /* BroadBand Utility Definitions and Function headers */
#include "ixf6048_irq.h"
#include "slowport.h"

extern bb_ChipData_t pChipData;
int ixf6048irq_isr_indicator = 0;

static struct media_slowport_cfg
{
  int spCCR;
  int spWTC2;
  int spRTC2;
  int spPCR;
  int spADC;

}
sp_cfg, sp_cfg_old;

#define ERROR	-1
#define OK 	0

void
slowport_cfg_save (struct media_slowport_cfg *sp_cpld_cfg)
{
	IXP2400_REG_READ (IXP2000_SLOWPORT_CCR, sp_cpld_cfg->spCCR);
	IXP2400_REG_READ (IXP2000_SLOWPORT_WTC2, sp_cpld_cfg->spWTC2);
	IXP2400_REG_READ (IXP2000_SLOWPORT_RTC2, sp_cpld_cfg->spRTC2);
	IXP2400_REG_READ (IXP2000_SLOWPORT_PCR, sp_cpld_cfg->spPCR);
	IXP2400_REG_READ (IXP2000_SLOWPORT_ADC, sp_cpld_cfg->spADC);
}

void
slowport_cfg_restore (struct media_slowport_cfg *sp_cpld_cfg)
{
	IXP2400_REG_WRITE (IXP2000_SLOWPORT_CCR, sp_cpld_cfg->spCCR);
	IXP2400_REG_WRITE (IXP2000_SLOWPORT_WTC2, sp_cpld_cfg->spWTC2);
	IXP2400_REG_WRITE (IXP2000_SLOWPORT_RTC2, sp_cpld_cfg->spRTC2);
	IXP2400_REG_WRITE (IXP2000_SLOWPORT_PCR, sp_cpld_cfg->spPCR);
	IXP2400_REG_WRITE (IXP2000_SLOWPORT_ADC, sp_cpld_cfg->spADC);
}

void
slowport_cfg_dump (void)
{
	unsigned int val;
	IXP2400_REG_READ (IXP2000_SLOWPORT_CCR, val);
	printk ("<1> CCR = %x\n", val);
	IXP2400_REG_READ (IXP2000_SLOWPORT_WTC2, val);
	printk ("<1> WTC2= %x\n", val);
	IXP2400_REG_READ (IXP2000_SLOWPORT_RTC2, val);
	printk ("<1> RTC2= %x\n", val);
	IXP2400_REG_READ (IXP2000_SLOWPORT_PCR, val);
	printk ("<1> PCR= %x\n", val);
	IXP2400_REG_READ (IXP2000_SLOWPORT_ADC, val);
	printk ("<1> ADC= %x\n", val);
}

void
ixf6048irq_isr (int irq, void *dev_id, struct pt_regs *regs)
{
	unsigned short globalEnable;

	slowport_cfg_save (&sp_cfg_old);

	slowport_cfg_restore (&sp_cfg);

	/* Read Master Enable bit */
#ifdef SLOW_PORT
	globalEnable = *((unsigned short *) pChipData.BaseAddress + 2);
	if (get_npu_product_id(MAJ_REV) == 0) /*A1 silicon*/
	{
		globalEnable = SWAPNTOHS (globalEnable);

		/* Disable Master Interrupt Enable */
		*((unsigned short *) pChipData.BaseAddress) = (globalEnable & 0x6);
	}
	else
	{
		globalEnable = *((unsigned short *) pChipData.BaseAddress + 2);
		*((unsigned short *) pChipData.BaseAddress) = (globalEnable & 0x6);
	}
#else
	globalEnable = *((unsigned short *) pChipData.BaseAddress + 2);

	/* Disable Master Interrupt Enable */
	*((unsigned short *) (pChipData.BaseAddress + 2)) = (globalEnable & 0x6);
#endif

	slowport_cfg_restore (&sp_cfg_old);

	ixf6048irq_isr_indicator = 1;

	ixf6048_ChipIsr (&pChipData);
}

int
ixf6048irq_init (void)
{
	int error = 0;

	sp_cfg.spCCR = 3;
	sp_cfg.spWTC2 = 0x91;
	sp_cfg.spRTC2 = 0x91;
	sp_cfg.spPCR = 3;
	sp_cfg.spADC = 0x13;

	/* Should be running only from Master
	* because only Master is hooked up to ADuC and has the
	* current values for both Master and Slave limits
	* so does not make sense to run it from slave side
	*/
	error = request_irq (IRQ_IXDP2400_MEDIA_SP, &ixf6048irq_isr,
			     SA_INTERRUPT, "bdw_ixp2000", NULL);
	if (error)
	{
		printk ("cannot get assigned IRQ %i\n", IRQ_IXDP2400_MEDIA_SP);
	}
	else
	{
		printk ("Interrupt Service Routine for Baterry Drew/Wallace loaded(IRQ %i).\n",
			IRQ_IXDP2400_MEDIA_SP);
	}
	return error;
}

void
ixf6048irq_exit (void)
{
	free_irq (IRQ_IXDP2400_MEDIA_SP, NULL);
}
