/**
 * ============================================================================
 *   COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ixf6048_linux_driver.c
 *
 * = DESCRIPTION
 *
 *
 * = CHANGE HISTORY
 *       2/18/2003 10:39:24 PM - creation time
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: ixf6048_linux_driver.c,v 1.11 2003/06/03 05:38:58 cmah1 Exp $";

/**
 * System defined include files required.
 */
#ifndef MODULE
#define MODULE
#endif

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <asm/hardware.h>
#include <asm-arm/arch-ixp2000/memory.h>
#include <asm/arch/irqs.h>
#include <asm/io.h>
#include <fs/hpfs/hpfs_fn.h>

/**
 * User defined include files required.
 */

typedef int sint;
typedef short sshort;
typedef char schar;
typedef unsigned char uchar;
typedef unsigned char boolean;  /* used in function calls */
#include "common_def.h"
#include "ixf_api_d.h"          /* BroadBand Utility Definitions and Function headers */
#include "ixf_api.h"
#include "ixf6048_linux_driver.h"
#include "ixf6048_ioctl.h"
#include "ixf6048_internal.h"

#include "ixf6048_irq.h"
#include "ixf6048s.h"
#include "ixf_api_sonet_d.h"
#include "ixf_api_sonet.h"
#include "ixf6048d.h"


/**
 * TYPENAME: AtmPos_fops
 *
 * DESCRIPTION:
 *      The Battery Drew/Wallace Atm Pos Media Card device driver implements the specified
 *      device methods, and uses the tagged format to declare its
 *      file_operations structure as
 *
 */
static struct file_operations AtmPos_fops = {
          ioctl: AtmPos_ioctl,
          open: AtmPos_open,
          release: AtmPos_close,
};

/**
 * function prototypes.
 */
static void Get_SP(uint32[]);
void Set_SP(uint32, uint32, uint32, uint32, uint32, uint32, uint32, uint32);

/**
 * Global declarations
 */
atm_pos_global_data atmPosGlobalData;
bb_ChipData_t pChipData;
uint32 sp_reg[8] = {0,0,0,0,0,0,0,0};
int Media_Card = BDREW_CARD;	// 0 = Battery Drew(default); 1 = Battery Wallace
int Card_Mode = OPT_POS_MODE;	// 0 = POS(default); 1 = ATM
unsigned char	*VIRT_IXF6048_BASE_ADDR;

/*
 Function: Ixf6048ClearAlarmCallback
*/
void
Ixf6048ClearAlarmCallback (ushort * BaseAddress, AlarmCallbackType_e AlarmType, /* Type of Alarm */
                           void *pAlarmStatus)  /* -> Alarm status struct */
{
	switch (AlarmType)
	{/* pAlarmStatus -> ixf6048_GlobalAlarmCfg_t */
		case bb_6048_MISC_ALARM:
			printk ("Received Transient  MISC ALARM\n");
			break;
		case bb_6048_OHT_ALARM:
			printk ("Received Transient  a UTOPIA ALARM\n");
			break;
		case bb_6048_UTOPIA_ALARM:
			printk ("Received Transient  a UTOPIA ALARM\n");
			break;
		case bb_6048_POS_ALARM:
			printk ("Received Transient  a POS ALARM\n");
			break;
		case bb_6048_ATM_ALARM:
			printk ("Received Transient  a ATM ALARM\n");
			break;
		default:
			printk ("Received Transient  alarm number %d\n", AlarmType);
			break;
	}
}


/*
 Function: init_module
*/
int
init_module(void)
{
	bb_Error_e ixfStatus;

#ifdef DEBUG
	printk("\nINIT_MODULE is getting called\n");
#endif

	/// Registration of device
	if(register_chrdev(g_MajorNumber, "BDW", &AtmPos_fops ) == 0)
	{
#ifdef DEBUG
		printk("Battery Drew/Wallace char device registered successfully\n");
#endif
	}
	else
	{
		printk("Battery Drew/Wallace char device COULD NOT BE REGISTERED!!!!\n");
	}
	/* ########################## Initialization of Registers ########################## */

	// Store original SlowPort settings
	Get_SP(sp_reg);
	// Set SlowPort to access IXF6048
	Set_SP(0x3, 0x68, 0x91, 0x68, 0x91, 0x0, 0x3, 0x13);
	VIRT_IXF6048_BASE_ADDR = (unsigned char*)  __ioremap((unsigned long)IXF6048_BASE_ADDRESS, DEVICE_SIZE, 0);
	ixfStatus = IxfApiInit ((bb_RegPointer_type) VIRT_IXF6048_BASE_ADDR, bb_6048_CHIP, &pChipData);
	if (ixfStatus != bb_NO_ERROR)
	{
		printk ("IxfApiInit FAILED!!!\n");
		return 1;
	}
	if(npu_is_master())	// master
	{
	/* ########################## Registering the Interrupt ########################## */
		IxfApiInitAlarmCallback (&pChipData, (AlarmCallBack) Ixf6048ClearAlarmCallback);
		if(ixf6048irq_init() != 0)
		{
			printk("Registering IRQ failed!!!\n");
			return 1;
		}
	}
	// Re-store original SlowPort settings
	Set_SP(sp_reg[0], sp_reg[1], sp_reg[2], sp_reg[3], sp_reg[4], sp_reg[5], sp_reg[6], sp_reg[7]);

	return 0;
}


/*
 Function: cleanup_module
*/
void
cleanup_module(void)
{
	bb_ChipSegment_t section;
	bb_Error_e ixfStatus;

	section.block.block = 0;
	section.block.side = 0;
	section.block.flow = 0;
	section.group.channel = 0;
	section.indirect.slot = 0;
	section.value = 0;

#ifdef DEBUG
	printk("\nCLEAN_UP MODULE is getting called\n");
	printk("Cleaning Battery Drew/Wallace module\n");
#endif

	/* Reset Chip, Sections & Fifos */
	ixfStatus = IxfApiReset (&pChipData, &section, bb_RESET_CHIP); /* Reset chip */
	if (ixfStatus != bb_NO_ERROR)
	{
		printk ("IxfApiReset FAILED!!!\n");
	}

	if(npu_is_master())	// master
	{
		// Disconnect IRQ
		ixf6048irq_exit();
	}

	iounmap ((void *) pChipData.BaseAddress);

	if(unregister_chrdev(g_MajorNumber, "BDW") == 0)
	{
#ifdef DEBUG
		printk("Battery Drew/Wallace char device unregistered successfully\n");
#endif
	}
	else
	{
		printk("Battery Drew/Wallace char device COULD NOT BE UNREGISTERED!!!!\n");
	}
	return;
}


/*
 Function: AtmPos_open
*/
int
AtmPos_open( struct inode *AtmPos_inode,
             struct file *AtmPos_file)
{
	uint32 s_PortNumber;

	/**
	* increment the usage count before doing anything else
	*/
	MOD_INC_USE_COUNT;      /*increment the usage count */

	MESSAGE("\nAtmPos_open is invoked\n",0,0,0,0,0,0);

	/* Retrieve the MINOR number from the device file */
	s_PortNumber = MINOR(AtmPos_inode->i_rdev);

	MESSAGE("Module is in use = %d \n",MOD_IN_USE,0,0,0,0,0);
	MESSAGE("Open device major=%d minor=%d i_rdev=0x%x\n", \
		MAJOR(AtmPos_inode->i_rdev), MINOR(AtmPos_inode->i_rdev),AtmPos_inode->i_rdev,0,0,0);

	if(atmPosGlobalData.g_PortState[s_PortNumber] == OPENED)
	{/* The respective port has already being opened */
		MESSAGE("Port is Already Opened\n",0,0,0,0,0,0);
	}
	else
	{
		atmPosGlobalData.g_PortState[s_PortNumber] = OPENED;
	}
	MESSAGE("EXITING from the AtmPos_open() routine\n",0,0,0,0,0,0);
	return 0;
}


/*
 Function: AtmPos_close
*/
int
AtmPos_close( struct inode *AtmPos_inode,
              struct file * AmtPos_file)
{
	uint32 s_PortNumber, s_Result = 0;

	/**
	* Decrement the usage count before doing anything else
	*/
	MOD_DEC_USE_COUNT;

	MESSAGE("\nAtmPos_close is invoked \n",0,0,0,0,0,0);

#ifdef DEBUG
	printk("Close device major=%d minor=%d \n",
		MAJOR(AtmPos_inode->i_rdev), MINOR(AtmPos_inode->i_rdev));
#endif
	MESSAGE("Module is in use = %d \n",MOD_IN_USE,0,0,0,0,0);

	s_PortNumber = MINOR(AtmPos_inode->i_rdev);

	if(atmPosGlobalData.g_PortState[s_PortNumber] == CLOSED)
	{
#ifdef DEBUG
		printk("PORT_ALREADY_CLOSED\n",0,0,0,0,0,0);
#endif
		s_Result = IOCTL_PORT_NOT_OPEN;
	}
	else
	{
		atmPosGlobalData.g_PortState[s_PortNumber] = CLOSED;
	}
#ifdef DEBUG
	MESSAGE("EXITING from the AtmPos_close() routine\n",0,0,0,0,0,0);
#endif
	return s_Result;
}


/*
 Function: AtmPos_ioctl
*/
int
AtmPos_ioctl( struct inode *AtmPos_inode,
              struct file * AtmPos_file,
              uint32 cmd,
              unsigned long arg_IoctlPtr)
{
	uint32 port_index;
	uint32 s_Result, s_Result_memcpy;
	atm_pos_ioctl_ptr_64 g_AtmPosIoctlPtr;

#ifdef DEBUG
	MESSAGE("\nAPI AtmPos_ioctl is invoked \n",0,0,0,0,0,0);
#endif
	if(AtmPos_inode == NULL)
	{
		MESSAGE("AtmPos_inode NULL\n",0,0,0,0,0,0);
		return -1;
	}

	if(AtmPos_file == NULL)
	{
		MESSAGE("AtmPos__file NULL\n",0,0,0,0,0,0);
		return -1;
	}

	if((void *)arg_IoctlPtr == NULL)
	{
		MESSAGE("IOCTL_BUFFER_POINTER_NULL\n",0,0,0,0,0,0);
		return IOCTL_BUFFER_POINTER_NULL;
	}

	port_index = MINOR(AtmPos_inode->i_rdev);       // retrieve the minor number => to the port number
							// if not opened, return with the error message

	MESSAGE("MINOR NUMBER is %d\n",port_index,0,0,0,0,0);


	if( (cmd != SET_CHIP_ONLINE) && (cmd != SET_CHIP_OFFLINE))
	{
		/**
		* Copy the user space memory to the kernel space */
		s_Result_memcpy = (uint32)memcpy_fromfs(&g_AtmPosIoctlPtr, (void *)arg_IoctlPtr,
							sizeof(atm_pos_ioctl_ptr_64));
		if(s_Result_memcpy <= 0)
		{
			MESSAGE("Error in Address Translation\n",0,0,0,0,0,0);
			return -1;
		}
		else
		{
			MESSAGE("Number of Bytes Transferred %d\n",s_Result_memcpy,0,0,0,0,0);
#ifdef DEBUG
			printk("Port number = %d, Low value = %d, High value = %d\n",
			g_AtmPosIoctlPtr.portIndex, g_AtmPosIoctlPtr.valueHigh, g_AtmPosIoctlPtr.valueLow);
#endif
		}
	}

	g_AtmPosIoctlPtr.portIndex = port_index;

	// Set SlowPort to access IXF6048
	Set_SP(0x3, 0x68, 0x91, 0x68, 0x91, 0x0, 0x3, 0x13);

	s_Result = atm_pos_config_handler(cmd,(void *)&g_AtmPosIoctlPtr);
	if(s_Result == SUCCESS)
	{
		MESSAGE("AtmPos_ioctl SUCCESS!!!\n",0,0,0,0,0,0);
	}
	else
	{
		printk("AtmPos_ioctl FAILED!!!\n",0,0,0,0,0,0);
	}

	// Re-store original SlowPort settings
	Set_SP(sp_reg[0], sp_reg[1], sp_reg[2], sp_reg[3], sp_reg[4], sp_reg[5], sp_reg[6], sp_reg[7]);

	if(((cmd & GET_IOCTL) == GET_IOCTL) || (cmd == GENERIC_READ))
	{
		/* Copy the kernel space to user space memory */
		s_Result_memcpy = (uint32)memcpy_tofs((void *)arg_IoctlPtr, &g_AtmPosIoctlPtr,
							sizeof(atm_pos_ioctl_ptr_64));
		if(s_Result_memcpy <= 0)
		{
			MESSAGE("Error in Address Translation\n",0,0,0,0,0,0);
			return -1;
		}

		MESSAGE("READ VALUES are..",0,0,0,0,0,0);
		MESSAGE("[HIGH : 0x%8X] [LOW : 0x%8X]\n",
			g_AtmPosIoctlPtr.valueHigh,g_AtmPosIoctlPtr.valueLow,0,0,0,0);
	}

	MESSAGE("Exiting from the AtmPos_iotcl() routine\n",0,0,0,0,0,0);
	return s_Result;
}


// Get SlowPort Settings
static void
Get_SP(uint32 sp_reg[8])
{
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x00, sp_reg[0]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x04, sp_reg[1]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x08, sp_reg[2]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x0C, sp_reg[3]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x10, sp_reg[4]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x14, sp_reg[5]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x18, sp_reg[6]);
	IXP2400_REG_READ(SLOWPORT_CSR_VIRT_BASE + 0x1C, sp_reg[7]);
}

// Set SlowPort Settings
void
Set_SP(uint32 val0, uint32 val1, uint32 val2, uint32 val3,
       uint32 val4, uint32 val5, uint32 val6, uint32 val7)
{
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x00, val0);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x04, val1);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x08, val2);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x0C, val3);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x10, val4);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x14, val5);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x18, val6);
	IXP2400_REG_WRITE(SLOWPORT_CSR_VIRT_BASE + 0x1C, val7);
}
