/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
* --------------------------------------------------------------------------
*/

#if !defined(__IXF1104CE_DRIVER_API_H__)
#define __IXF1104CE_DRIVER_API_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/** The following global variables are used by the application 
    to get the interrupt status. 
**/
extern unsigned int gbic_status ;
extern unsigned int spi3_status_port0;
extern unsigned int spi3_status_port1;
extern unsigned int spi3_status_port2;
extern unsigned int spi3_status_port3;
/**
 * Prototypes for interface functions.
 */
/**
 * NAME: GbEMAC_DeviceStart
 *
 * DESCRIPTION: 
 *       This function is called by the application to open the device.
 * 
 * @Param:  - IN    : arg_PortMask - The Port Mask, indication the ports to 
 *                                   be opened.
 *
 * @Param:  - IN    : arg_mode -  indicates fiber/Cu mode, duplex and speed of operation.
 *                    mode value interpretation:
 *
 *   Bit Position    Associated to         Values
 *      4            SPI-3 Block Mode         0 : MPHY Mode
 *                                            1 : 4x8 Mode
 *      3            SPI-3 Tx Parity          0 : Odd Parity
 *                                            1 : Even Party
 *     2-0           channel, mode
 *                   and speed              000	: Fiber operation   	
 *                                          001	: gigabit copper, half duplex  
 *                                          010	: gigabit copper, full duplex 
 *                                          011	: 10/100 copper, half duplex   
 *                                          100	: 10/100 copper, full duplex   
 *
 * @Param:  - INOUT : NULL
 *
 * @Param:  - OUT   : arg_Handle   - The Pointer to the handle is passed,
 *                                   which is updated when the device is 
 *                                   opened successfully.
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
unsigned int GbEMAC_DeviceStart(unsigned int arg_PortMask,
                                unsigned int *arg_Handle,
				unsigned int arg_mode);


/**
 * NAME: GbEMAC_DeviceStop
 *
 * DESCRIPTION: 
 *      This function is called by the application to close the device.
 * 
 * @Param:  - IN    : arg_PortMask - The Port Mask, indication the ports to 
 *                                   be opened.
 *                    arg_Handle   - The Pointer to the handle is passed,
 *                                   which is updated when the device is 
 *                                   opened successfully.
 * @Param:  - INOUT : NULL 
 * @Param:  - OUT   : NULL
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
unsigned int GbEMAC_DeviceStop(
                               unsigned int arg_PortMask,
                               unsigned int *arg_Handle			       
                              );

/**
 * NAME: GbEMAC_Ioctl
 *
 * DESCRIPTION:
 *     This function is calles by the application to call the ioctl command. 
 * 
 * @Param:  - IN     : arg_IoctlCommand - the IOCTL command to be implemented.
 * 
 * @Param:  - INOUT  : arg_Handle       - Passes the Handle, that it gets the 
 *                                        device open call.
 *                     arg_pIoctlStruct - The pointer to the structure is passed
 *                                        which contains the value for GET ioctls,
 *                                        and which holds the value in SET ioctls.
 * @Param:  - OUT  
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
unsigned int GbEMAC_Ioctl(
                          unsigned int *arg_Handle,
                          unsigned int arg_IoctlCommand, 
                          void *arg_pIoctlStruct
                         );
/**
 * NAME: GbEMAC_Callback
 *
 * DESCRIPTION:
 *     This function is registers the user callback functions to an array,
 *     and when the interrupt occurs, all the respective functions are called.
 *
 * @Param:  - IN     : arg_Handle       - Passes the Handle, that it gets the
 *                                       device open call.
 *                   : arg_pCallback -  The user application function, whose
 *                                      pointer is passed to this API to register
 *                                      with the driver as a callback function.
 *                                      Whenever an interrupt has been generated,
 *                                      this function is called by the driver library.
 * @Param:  - OUT
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
unsigned int GbEMAC_Callback(
                             unsigned int *arg_Handle,
                             void * arg_pCallback,
                             void * arg_pUserContext
                            );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /*end defined(__IXF1104CE_DRIVER_API_H__)*/
