/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
* --------------------------------------------------------------------------
*/

static const char s_FileIdentifier[] = "$Id: ixf1104ce_driver_api.c,v 1.13 2003/04/24 20:35:58 gdvaddad Exp $";


/**
 * System defined include files required.
 */
#include <vxworks.h>
#include <logLib.h>
#include <stdlib.h>
#include <taskLib.h>
#include <intLib.h>
#include <semLib.h>
#include <objLib.h>
/* #include <intArchLib.h>*/


/**
 * User defined include files required.
 */
#include "../include/ixf1104ce_driver_api.h"
#include "../../common/include/ixf1104ce_ioctl.h"
#include "../../common/include/ixf1104ce_internal.h"

/* For pciIntConnect related functionality */

/** 
 * For the Master and Slave NPU Identification Support, include this BSP file,
 */
#include "ixdp2400.h"
#include "ixdp2400misc.h"     
#include "ixdp2400i2c.h"
/*include "pciIomapLib.h"*/


/**
 * Types definitions whose scope is limited to this file.
 */
#define EGRESS                    0x01
#define INGRESS                   0x00

#define DEVICE_MASK_0             0x0F
#define DEVICE_MASK_1             0xF0
#define DEVICE_0                  0x01
#define DEVICE_1                  0x02
#define NUMBER_OF_DEVICES         0x02              /* Max Number of Pt Lone cards on the KI Board */

#define DEFAULT_SET               0x01

#define SHIFT_BYTE                0x08
#define SHIFT_NIBBLE              0x04 

#define DEVICE_NOT_PRESENT        0x00
#define EEPROM_NOT_PROGRAMMED     0xFF

#define MEDIA_IF_INT_NUM          0x03
#define SF_IF_INT_NUM             0x05

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
/**
 * Array of structure, that holds the pointer for the callback functions
 */
gbe_mac_callback gbeMacCallback[32];

uint32 gbeMacCallbackTaskFiber[2];
uint32 gbeMacCallbackTaskCopper[2];

SEM_ID g_SemBCreateFiber[2];
SEM_ID g_SemBCreateCopper[2];

/* Two structure of the global data for two PtLone MEdia Cards */
gbe_mac_global_data gbeMacGlobalData[2] = { { {CLOSED,CLOSED,CLOSED,CLOSED}, 
                                              0,
                                              0,0,0,0,
                                              0,
                                              0,
                                              NULL
                                             },
                                             { {CLOSED,CLOSED,CLOSED,CLOSED},
                                              0,
                                              0,0,0,0,
                                              0,
                                              0,
                                              NULL
                                             }
                                           };

gbe_mac_global_struct gbeMacGlobalStruct[2]; /* Each one structure for each Pt Lone Media Card */

gbe_mac_mac_address gbeMacAddress[2];             /* For the two Pt Lone cards */

/* offset per port */
uint32 g_PerPortOffset[4] = {
                             MAC_CTL_REG_PORT0,
                             MAC_CTL_REG_PORT1,
                             MAC_CTL_REG_PORT2,
                             MAC_CTL_REG_PORT3
                            };

uint32 g_NpuStatus;

uint32 s_IntNumber = 0;         /* Interrupt Vector number, to which the ISR is to be connected.*/


/**
 * Extern function prototypes.
 */
/**
 * NAME: gbe_mac_config
 *
 * DESCRIPTION: This function performs the initial configuration.
 * 
 * @Param: -IN    :   uint32 arg_DeviceNumber
 *                :   uint32 arg_PortMask
 * @Param: -INOUT :
 * @Param: -OUT   :
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
extern gbe_mac_error gbe_mac_config(
                                    uint32 arg_DeviceNumber,
                                    uint32 arg_PortMask,
				    short arg_mode
                                   );

/**
 * NAME: gbe_mac_config_handler
 *
 * DESCRIPTION: This function parse the GET/SET type of te ioctl and 
 *              calls the corresponding function accordingly. 
 *
 * @Param: -IN    : uint32 arg_deviceNumber
 *                  uint32 ioctl_command - IOCTL Command name
 * @Param: -INOUT : void *ioctl_struct 
 * @Param: -OUT   :
 *
 * @Return: 0 if successful or a valid gbe_mac_error token for failure.
 */
extern gbe_mac_error gbe_mac_config_handler(
                                            uint32 arg_DeviceNumber,
                                            uint32 ioctl_command,
                                            void *ioctl_struct
                                           );

/**
 * NAME: get_intConnect_number
 *
 * DESCRIPTION: This function returns the number, to which the ISR to 
 *              be connected into the interrupt vector table
 *
 * @Param: -IN    : uint32 arg_deviceNumber
 * @Param: -INOUT :  
 * @Param: -OUT   : 
 *
 * @Return: interrupt vector number
 */
uint32 get_intConnect_number(
                             uint32 arg_DeviceNumber
                            );

/**
 * NAME: get_device_EEPROM_offset
 *
 * DESCRIPTION: This function returns the address of the EEPROM on the device
 *
 * @Param: -IN    : uint32 arg_deviceNumber
 * @Param: -INOUT :  
 * @Param: -OUT   : 
 *
 * @Return: EEPROM offset for the device on the I2C.
 */
unsigned char get_device_EEPROM_offset(
                                       uint32 arg_DeviceNumber
                                      );


extern void gbe_mac_change_configuration(
                                          uint32 arg_DeviceNumber,
                                          uint32 arg_PortMask,
                                          uint32 arg_mode
                                         );

/**
 * Function definitions.
 */
gbe_mac_error GbEMAC_DeviceStart(
		                 uint32 arg_PortMask,
                                 uint32 *arg_Handle,
				 uint32 arg_mode
                                )				
{
    uint32 s_IntStatus = 0;
    uint32 s_DeviceStartFlag = TRUE;
    gbe_mac_error s_ReturnValue = -1,s_PortCount = -1,s_MacAddressHigh = -1,s_MacAddressLow = -1;
    uint32 s_ResourceCount = 0;
    uint32 s_Handle = 0;        /* a local variable used for the Handle */
    uint32 s_Temp = 0, s_DeviceCount = 0;
    short mode = arg_mode;

    uint32 s_DeviceMask = 0;        /* Used in generating the Handle, which is returned by thei API to 
                                       the calling application. */
    uint32 s_Temp1 = -1,s_Count;
    uint32 g_DeviceStatus[2] = {0,0};

    gbe_mac_ioctl_ptr *s_TempIoctl;
    unsigned char s_MacAddr_r[6];
    unsigned char s_DeviceOffset, s_MacOffset[4], s_ByteCount;
    unsigned char s_DevID;
    
    s_ReturnValue = SUCCESS;
    MESSAGE("Driver API GbEMAC_DeviceStart is Invoked\n",0,0,0,0,0,0);
  
    /**
     * The code for NPU Identification.
     */
    g_NpuStatus = isNPUMaster();
    if(g_NpuStatus == EGRESS)
    {/* Running on the EGRESS NPU */
        MESSAGE("\tRunning on the EGRESS NPU\n",0,0,0,0,0,0);
    }/* end-if running on EGRESS NPU */
    else
    { /* Running on the INGRESS NPU */
        MESSAGE("\tERROR_NPU_IS_INGRESS\n",0,0,0,0,0,0);
        return ERROR_NPU_IS_INGRESS;
    }/* end-if INGRESS */
 
    s_TempIoctl = (gbe_mac_ioctl_ptr *)malloc(sizeof(gbe_mac_ioctl_ptr));
    
    /** 
     * check whether any port in Card 0 is to be opened
     */
    s_Temp = arg_PortMask & DEVICE_MASK_0;
    if(s_Temp)
    {
        s_DeviceMask = s_DeviceMask | DEVICE_0;	
        g_DeviceStatus[0] = s_Temp;	
        MESSAGE("Point Lone Card on the Media interface is to be opened\n",0,0,0,0,0,0);
    }

    /**
     * Check whether any port in the Card 1 is to be opened
     */
    s_Temp = arg_PortMask & DEVICE_MASK_1;
    if(s_Temp)
    {
        s_DeviceMask = s_DeviceMask | DEVICE_1;
        g_DeviceStatus[1] = (s_Temp >> SHIFT_NIBBLE);  
        MESSAGE("Point Lone Card on the Switch Fabric interface is to be opened\n",0,0,0,0,0,0);
    }		      

    /**
     * Generate the Handle, that includes the Device MAsk and the Port Mask
     *
     *     9                2    0
     *     |                |    |  
     *     |    8-bits      | 2- |
     *     |                |bits|
     *     <---Port Mask--->
     *                      <---->   
     *                       Device
     *                        Mask
     */
    s_Handle |= s_DeviceMask;
    s_Temp = arg_PortMask << DEVICE_1;
    s_Handle |= s_Temp;
   
    /* Set the Offset of the MAC Address in the EEPROM */
    s_MacOffset[0] = EEPROM_MAC_PORT_OFF_0;
    s_MacOffset[1] = EEPROM_MAC_PORT_OFF_1;
    s_MacOffset[2] = EEPROM_MAC_PORT_OFF_2;
    s_MacOffset[3] = EEPROM_MAC_PORT_OFF_3;

   
    for(s_DeviceCount = 0; s_DeviceCount<NUMBER_OF_DEVICES; s_DeviceCount++)
    { /* s_DeviceCount = 0 => Pt Lone Media card 0, 
                              having the ports numbered 0-3, on the Media Interface
	 s_DeviceCount = 1 => Pt Lone Media Card 1, 
                              having the ports numbered 4-7, on the Switch Fabric Media Interface.
	 */
      MESSAGE("** Pt Lone Media Card %d ** ",s_DeviceCount,0,0,0,0,0);
      if(g_DeviceStatus[s_DeviceCount])
      {	/* Card needs to be opened */      
        MESSAGE("-OPENING ",0,0,0,0,0,0);
        s_ResourceCount = 0;	    
        if((gbeMacGlobalData[s_DeviceCount].g_MacStatus == INITIALIZED))
        { /* Pt Lone media card is already initialized. */
            MESSAGE("-ALREADY_INITIALIZED \n",0,0,0,0,0,0);

	    /* This is a patch for setting the per port mode, other than default */	

            MESSAGE("arg_PortMask [0x%x]\n",arg_PortMask,0,0,0,0,0);
            gbe_mac_change_configuration(s_DeviceCount,arg_PortMask,arg_mode);
            MESSAGE("arg_PortMask [0x%x]\n",arg_PortMask,0,0,0,0,0);

            s_Temp1 = g_DeviceStatus[s_DeviceCount];
            if(taskLock() == OK)
            {/* taskLock() Successfully achieved */
               s_ResourceCount++;
               if((*arg_Handle ) == 0)
               { 
                   for(s_Temp=0; s_Temp<PT_LONE_GBE_PORT_MAX; s_Temp++)
                   {/* for each port individually, increase the per port usage count */
                       if((s_Temp1 & DEFAULT_SET) == DEFAULT_SET)
                       {                        
                           gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Temp]++;
                       }
                       s_Temp1 = s_Temp1 >> DEFAULT_SET;
                   }
                   gbeMacGlobalData[s_DeviceCount].g_UsageCount ++; /* Increment the Pt Lone media card usage count */
                   MESSAGE("Increment the Usage Count, g_UsageCount = %d\n",\
                            gbeMacGlobalData[s_DeviceCount].g_UsageCount,0,0,0,0,0);
               }
               else
               {
                   if(s_DeviceCount == 0)
                   {
                       s_Temp = (*arg_Handle & 0x0F);
                   }
                   else
                   {
                       s_Temp = (*arg_Handle & 0xF0) >> 0x04;
                   }
                   for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
                   {/* for each port individually, increase/decrease the per port usage count */
                       if(((s_Temp1 & DEFAULT_SET) == DEFAULT_SET) &&
                          ((s_Temp & DEFAULT_SET) != DEFAULT_SET))
                       {/* Port is opened */
                           gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_PortCount]++;
                       }

                       if(((s_Temp1 & DEFAULT_SET) != DEFAULT_SET) &&
                          ((s_Temp & DEFAULT_SET) == DEFAULT_SET))
                       {/* Port is closed */
                           gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_PortCount]--;
                       }

                       s_Temp1 = s_Temp1 >> DEFAULT_SET;
                       s_Temp = s_Temp >> DEFAULT_SET;
                   }
               }
               s_ReturnValue = SUCCESS;
           }
           else
           {/* taskLock Failed */
               MESSAGE("ERROR_TASK_LOCK_FAILED\n",0,0,0,0,0,0);
               s_DeviceStartFlag = FALSE;                
               s_ReturnValue = ERROR_TASK_LOCK_FAILED;
           }
           if(s_ResourceCount == 1)
           {
               taskUnlock();
               s_ResourceCount--;		    
           }
        }/* end-if card has been initialized earlier */
        else
        {/* It is the first application calling device start */
            MESSAGE("-INITIALIZING \n",0,0,0,0,0,0);
            if(taskLock() == OK)
            {/* task lock Success */
                MESSAGE("taskLock() achieved successfully\n",0,0,0,0,0,0);
                s_ResourceCount++; /* Resource Allocated = 1 */

                /* Evaluete the interrupt vector number  */
                s_IntNumber = get_intConnect_number(s_DeviceCount);

                /* set the device EEPROM address according to the device */
                s_DeviceOffset = get_device_EEPROM_offset(s_DeviceCount);
                MESSAGE("Device offset = 0x%x\n",s_DeviceOffset,0,0,0,0,0);
                /**
                 * Read the device ID from the EEPROM 
                 */
                i2c_read(s_DeviceOffset,EEPROM_DEV_ID_OFF,&s_DevID);
                MESSAGE("Device id = 0x%d\n",s_DevID,0,0,0,0,0);
 
                if(s_DevID == PT_LONE_DEVICE_ID)
                {/* the read value for the device id matches with the defined value of the device ID */
                    for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
                    {/* Countinue for all the 4 MAC ports */
                        for(s_ByteCount=0; s_ByteCount<6; s_ByteCount++)
                        {/* read all the 6-bytes of the MAC address */
                            s_Temp = s_MacOffset[s_PortCount] + s_ByteCount;
                            i2c_read(s_DeviceOffset,s_Temp,&s_MacAddr_r[s_ByteCount]);
                        }/* end-for reading the MAC Address from the EEPROM */
                        /* Calculate the MAC address, low and high part */
                        s_MacAddressHigh = 0;
                        s_MacAddressHigh = (s_MacAddr_r[0] << SHIFT_BYTE | s_MacAddressHigh);
                        s_MacAddressHigh = s_MacAddressHigh | s_MacAddr_r[1];

                        s_MacAddressLow = 0;
                        s_MacAddressLow = (s_MacAddressLow | (s_MacAddr_r[2] << (3*SHIFT_BYTE ))\
                                                           | (s_MacAddr_r[3] << (2*SHIFT_BYTE )) \
                                                           | (s_MacAddr_r[4] << (SHIFT_BYTE )) \
                                                           | s_MacAddr_r[5] );
                        MESSAGE("PORT [%d] MAC [0x%x 0x%x]\n",s_PortCount,\
                                           s_MacAddressHigh, s_MacAddressLow,0,0,0);
                        switch(s_PortCount)
                        { /* Assign the MAC address to the corresponding MAC port */
                            case 0: /* MAC Port 0 */
                                    gbeMacAddress[s_DeviceCount].macAddressPort_0[0] = s_MacAddressLow;
                                    gbeMacAddress[s_DeviceCount].macAddressPort_0[1] = s_MacAddressHigh;
                                    break;

                            case 1:/* MAC Port 1 */
                                    gbeMacAddress[s_DeviceCount].macAddressPort_1[0] = s_MacAddressLow;
                                    gbeMacAddress[s_DeviceCount].macAddressPort_1[1] = s_MacAddressHigh;
                                    break;

                            case 2:/* MAC Port 2 */
                                    gbeMacAddress[s_DeviceCount].macAddressPort_2[0] = s_MacAddressLow;
                                    gbeMacAddress[s_DeviceCount].macAddressPort_2[1] = s_MacAddressHigh;
                                    break;

                            case 3:/* MAC Port 3 */
                                    gbeMacAddress[s_DeviceCount].macAddressPort_3[0] = s_MacAddressLow;
                                    gbeMacAddress[s_DeviceCount].macAddressPort_3[1] = s_MacAddressHigh;
                                    break;

                            default:
                                    break;
                        }; /* end-switch case statements */
                    } /* end-for reading the EEPROM for the MAC address */
                } /* end-if Valid device ID found */
                else if(s_DevID == DEVICE_NOT_PRESENT)
                { /* Device is not PRESENT */
                    switch(s_DeviceCount)
                    {
                        case 0:
                            MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE\n",0,0,0,0,0,0);
                            s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_MEDIA_INTERFACE;
                            s_DeviceStartFlag = FALSE;
                            break;
   
                        case 1:
                            MESSAGE("ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE\n",0,0,0,0,0,0);
                            s_ReturnValue = ERROR_PT_LONE_MEDIA_CARD_IS_NOT_PRESENT_ON_SWITCH_FABRIC_INTERFACE;
                            s_DeviceStartFlag = FALSE;
                            break;
                       
                        default:
                            break;
                    };/* end-switch */
                } /* end-if Device is not present */
                else if(s_DevID == EEPROM_NOT_PROGRAMMED)
                { /* EEPROM is not programmed */
                    switch(s_DeviceCount)
                    {
                        case 0:
                            MESSAGE("EEPROM_OF_PT_LONE_MEDIA_CARD_ON_MEDIA_INTERFACE_IS_NOT_PROGRAMMED\n",0,0,0,0,0,0);
                            s_ReturnValue = EEPROM_OF_PT_LONE_MEDIA_CARD_ON_MEDIA_INTERFACE_IS_NOT_PROGRAMMED;
                            s_DeviceStartFlag = FALSE;
                            break;

                        case 1:
                            MESSAGE("EEPROM_OF_PT_LONE_MEDIA_CARD_ON_SWITCH_FABRIC_INTERFACE_IS_NOT_PROGRAMMED\n",0,0,0,0,0,0);
                            s_ReturnValue = EEPROM_OF_PT_LONE_MEDIA_CARD_ON_SWITCH_FABRIC_INTERFACE_IS_NOT_PROGRAMMED;
                            s_DeviceStartFlag = FALSE;
                            break;

                        default:
                            break;
                    }; /* end-switch */
                } /* end-if EEPROM is not programmed */

                if(s_DeviceStartFlag)
                {/* Pt Lone media card is present */
                    gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore = semMCreate(SEM_Q_FIFO | SEM_DELETE_SAFE);
                    if(gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore != NULL)
                    {/* Mutex Semaphore created successfully */
                        MESSAGE("\tMutex Semaphore created, ID %u\n",\
                                 (uint32)gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore,0,0,0,0,0);
                        s_ResourceCount++;                       /* Resource Allocated = 2 */
#if !defined(__VXSIM__)
                        /* s_IntNumber -> interrupt vector to attach to */	
		        /* gbe_mac_isr -> Handler routine to be called  */
		        /* s_DeviceCount -> parameter passed to routine */
                        s_IntStatus = pciIntConnect((VOIDFUNCPTR *)s_IntNumber, 
                                                    (VOIDFUNCPTR)gbe_mac_isr,   
                                                    s_DeviceCount               
                                                   );
                        /**
                         * Connect the ISR to the interrupt vector.
                         */
                        if(s_IntStatus == OK)            
                        {/* ISR connected successfully */
                            MESSAGE("\tgbe_mac_isr ISR is successfully connected\n",0,0,0,0,0,0);
                            /**
                             * Enable the ISR
                             */
                            pciIntEnable(s_IntNumber);
                        }/* end-if ISR connected successfully */
                        else   
                        {/* ISR could Not connected successfully */
                            MESSAGE("ISR_REGISTERTATION_FAILED\n",0,0,0,0,0,0);
                            s_DeviceStartFlag = FALSE;
                            s_ReturnValue = ISR_REGISTERTATION_FAILED;
                        } 
#endif/* end of __VXSIM__ */
                    }/* end-if semCreate */
                    else                  
                    { /* ERROR in Mutex Semaphore Creation */
                        s_DeviceStartFlag = FALSE;
                        MESSAGE("SEMAPHORE_COULD_NOT_CREATED\n",0,0,0,0,0,0);
                        s_ReturnValue = SEMAPHORE_COULD_NOT_CREATED;
                    }
                    if((s_ResourceCount == 2) && (s_DeviceStartFlag == FALSE))
                    {
                        MESSAGE("\tDelete the Mutex Semaphore, as some error has occured\n",0,0,0,0,0,0);
                        semDelete(gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore);
                        s_ResourceCount--;
                    }

                    /* This function can pass the Base Address of the Register Map in the Slow-port */
                    s_ReturnValue = gbe_mac_config(s_DeviceCount,g_DeviceStatus[s_DeviceCount],mode);
                    if(s_ReturnValue != SUCCESS)
                    {
                        s_DeviceStartFlag = FALSE;
                        MESSAGE("Driver API GbEMAC_DeviceStart could NOT complete the execution\n",0,0,0,0,0,0);
         
#if !defined(__VXSIM__)
			/* s_IntNumber -> interrupt vector to attach to */
			/* gbe_mac_isr -> Handler routine to be called  */
                        pciIntDisable(get_intConnect_number(s_DeviceCount));
                        s_IntStatus = pciIntDisconnect((VOIDFUNCPTR *)s_IntNumber, (VOIDFUNCPTR)gbe_mac_isr);
                        if(s_IntStatus == OK)
                        {
                            MESSAGE("Interrupt has been disconnected\n",0,0,0,0,0,0);
                        }
                        MESSAGE("Disable the ISR\n",0,0,0,0,0,0);
#endif /* __VXSIM__ */
                        semDelete(gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore);
                        MESSAGE("\tDELETE the g_MutexSemaphore Semaphore\n",0,0,0,0,0,0);
                    }
		    /* This is a patch for setting the per port mode, other than default */	

                } /* end-if s_DeviceStartFlag flag is true */
                taskUnlock();
                MESSAGE("\ttackUnlock had been done\n",0,0,0,0,0,0);
              }/* end-if taskLock() */
              else
              { /* taskLock() Failed */
                  MESSAGE("ERROR_TASK_LOCK_FAILED\n",0,0,0,0,0,0);
                  s_DeviceStartFlag = FALSE;
                  s_ReturnValue = ERROR_TASK_LOCK_FAILED;
              }
#if 0
              if(s_ResourceCount != FALSE)            
              { /* UnLock the task, locked earlier. */
                  taskUnlock();
                  MESSAGE("\ttackUnlock had been done\n",0,0,0,0,0,0);
                  s_ResourceCount--;
              }
#endif

            if(taskLock() == OK)
            { /* task lock Success */
                s_ResourceCount++; /* Resource Allocated = 1 */
                gbeMacGlobalData[s_DeviceCount].g_UsageCount++;     /* Increase the Usage count anyway */
                MESSAGE("\tIncrease the Usage count, UsageCount = %u\n",\
                          gbeMacGlobalData[s_DeviceCount].g_UsageCount,0,0,0,0,0);
                taskUnlock();
                MESSAGE("\ttackUnlock had been done\n",0,0,0,0,0,0);
            }/* end-if taskLock() */
            else                     
            {/* taskLock() Failed */
                MESSAGE("ERROR_TASK_LOCK_FAILED\n",0,0,0,0,0,0);
                s_DeviceStartFlag = FALSE;
                s_ReturnValue = ERROR_TASK_LOCK_FAILED;
            }/* end-if taskLock failed */
#if 0
            if(s_ResourceCount != 0)         
            {/* UnLock the task, lockec earlier. */
                taskUnlock();
                MESSAGE("\ttackUnlock had been done\n",0,0,0,0,0,0);
                s_ResourceCount--;
            }
#endif
            /**
             * Initialize the callback function array to NULL
             */
            for(s_Count=0; s_Count<MAX_APP_SUPPORTED; s_Count++)
            {
                if(gbeMacCallback[s_Count].appHandle == *(arg_Handle))
                {
                    gbeMacCallback[s_Count].p_CallbackFn = NULL;
                    gbeMacCallback[s_Count].appHandle = 0;

                }
            }
        } /* end-if the device is opened for the first time.*/

        if(s_ReturnValue == SUCCESS)
        {
            MESSAGE("Driver API GbEMAC_DeviceStart completes the execution\n",0,0,0,0,0,0);
        }
        else
        {
            MESSAGE("Driver API GbEMAC_DeviceStart could NOT complete the execution\n",0,0,0,0,0,0);
        }
      } /* end-if s_DeviceStatus, at the starting of the for loop */
      MESSAGE("-- %d --\n",s_DeviceCount,0,0,0,0,0);
    }/* end-for device number */

    if(s_DeviceStartFlag == TRUE)
    {
        *arg_Handle = s_Handle;
        MESSAGE("HANDLE 0x%x\n",s_Handle,0,0,0,0,0);
    }
    else
    {
        *arg_Handle = -1;
    }
    return s_ReturnValue;
}






gbe_mac_error GbEMAC_DeviceStop(
                                 uint32 arg_PortMask,
                                 uint32 *arg_Handle
                                )				
{
	
    uint32 s_Count=0, s_DeviceCount=0;
    uint32 s_DeviceStopFlag = TRUE;
    uint32 s_ReturnValue=-1;
    uint32 s_ResourceCount = 0;
    uint32 s_DeviceStatus[2] = {0,0};
    uint32 s_Temp=0, s_Handle=0; 
    uint32 s_IntStatus = 0;
	
    s_ReturnValue = SUCCESS;

    
    /**
     * check for the proper handle
     */
    if(((*arg_Handle) <= 0) || ((*arg_Handle) > 0x3FF))
    {
        s_DeviceStopFlag = FALSE;
        MESSAGE("INVALID HANDLE\n",0,0,0,0,0,0);
        s_ReturnValue = ERROR_INVALID_HANDLE;
    }
    
    MESSAGE("CLOSE : handle[0x%04x]\n",*arg_Handle,0,0,0,0,0);
    if(s_DeviceStopFlag)
    {/* Valid Handle */
        /**
         * check whether any port in Card 0 is to be opened
         */
        s_Temp = arg_PortMask;
        if(s_Temp & DEVICE_MASK_0)
        {
            s_DeviceStatus[0] = s_Temp & DEVICE_MASK_0 ;
        }
        /**
         * Check whether any port in the Card 1 is to be opened
         */
        s_Temp = arg_PortMask;
        if(s_Temp & DEVICE_MASK_1)
        {
            s_DeviceStatus[1] = (s_Temp >> SHIFT_NIBBLE);
        }
    
        for(s_DeviceCount=0;s_DeviceCount<NUMBER_OF_DEVICES; s_DeviceCount++)
        {
           MESSAGE("** Pt Lone Media Card %d ** ",s_DeviceCount,0,0,0,0,0);
           if(s_DeviceStatus[s_DeviceCount])
           {/* Any port of this device is to be closed. */
               if((gbeMacGlobalData[s_DeviceCount].g_MacStatus == UNINITIALIZED) ||
                  (gbeMacGlobalData[s_DeviceCount].g_UsageCount == 0))
               {/* Device is not still initialized */
                   s_DeviceStopFlag = FALSE;
                   MESSAGE("-MAC_NOT_INITIALIZED \n",0,0,0,0,0,0);
                   s_ReturnValue = ERROR_MAC_NOT_INITIALIZED;
               }/* end-if Device is not initialized */
               else
               {/* Device is initialized earlier */
                   MESSAGE("-CLOSING\n",0,0,0,0,0,0);
                   if(taskLock() == OK)
                   {/* Tasklock achieved */
                       s_ResourceCount++;
                       if(gbeMacGlobalData[s_DeviceCount].g_UsageCount == 1)
                       {/* If it the last application */
                           MESSAGE("The calling application is the last Application using the driver\n",0,0,0,0,0,0);
                           g_NpuStatus = isNPUMaster();
                           if(g_NpuStatus == EGRESS)
                           {/* Running on the EGRESS NPU */
                               MESSAGE("\nRunning on the EGRESS NPU\n",0,0,0,0,0,0);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (MAC_SOFT_RESET),\
                                                     0xF);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (TXFIFO_PORT_RESET),\
                                                     0xF);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (RXFIFO_PORT_RESET),\
                                                     0x0);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (SPI3_TX_GLOBAL_CONFIG),\
                                                     0x00C00000);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (GBIC_CTRL_REG),\
                                                     0x0);
		    	       /* Disable all interrupts */
	                       gbe_mac_phy_write(s_DeviceCount,0x0, 0x19, 0x0);
                               gbe_mac_phy_write(s_DeviceCount,0x1, 0x19, 0x0);
                               gbe_mac_phy_write(s_DeviceCount,0x2, 0x19, 0x0);
                               gbe_mac_phy_write(s_DeviceCount,0x3, 0x19, 0x0);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (MDIO_RESET),\
                                                     0x1);
                               gbe_mac_write_32_bits(s_DeviceCount,\
                                                     (PORT_ENABLE),\
                                                     0x0);
                               MESSAGE("Config resgisters are reseted\n",0,0,0,0,0,0);
#if !defined(__VXSIM__)
                               pciIntDisable(get_intConnect_number(s_DeviceCount));
                               s_IntStatus = pciIntDisconnect((VOIDFUNCPTR *)s_IntNumber,\
                                                              (VOIDFUNCPTR)gbe_mac_isr);
                               if(s_IntStatus == OK)
                               {
                                    MESSAGE("Interrupt has been disconnected\n",0,0,0,0,0,0);
                               }
                               MESSAGE("Disable the ISR\n",0,0,0,0,0,0);
#endif /* __VXSIM__ */
                               /* Delete the corresponding task */
                               taskDelete(gbeMacGlobalData[s_DeviceCount].g_TaskID);
                               /* Delete the task spawned for the ISR */
                               taskDelete(gbeMacCallbackTaskFiber[s_DeviceCount]);
                               taskDelete(gbeMacCallbackTaskCopper[s_DeviceCount]);
                               /* Delete the Binary semaphore created for ISR */
                               semDelete(g_SemBCreateFiber[s_DeviceCount]);
                               semDelete(g_SemBCreateCopper[s_DeviceCount]);

                               for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
                               {/* Make the state of each port as closed */
                                   gbeMacGlobalData[s_DeviceCount].g_PortState[s_Count] = CLOSED;
                                   gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Count] = 0;
                               }
                               MESSAGE("\tMAC Ports are CLOSED\n",0,0,0,0,0,0);
                               gbeMacGlobalData[s_DeviceCount].g_MacStatus = UNINITIALIZED;
                               gbeMacGlobalData[s_DeviceCount].g_UsageCount = 0;
                               gbeMacGlobalData[s_Count].g_TaskID = 0;
                               gbeMacGlobalData[s_Count].g_MutexSemaphore = NULL;
                               MESSAGE("\tMAC Status UNINITIALIZED\n",0,0,0,0,0,0);
                               semDelete(gbeMacGlobalData[s_DeviceCount].g_MutexSemaphore);
                               MESSAGE("\tDELETE the g_MutexSemaphore Semaphore\n",0,0,0,0,0,0);
                           }/* end-if running on EGRESS */
			   for(s_Count=0; s_Count<MAX_APP_SUPPORTED; s_Count++)
   			   {
			            gbeMacCallback[s_Count].p_CallbackFn = NULL;
			            gbeMacCallback[s_Count].appHandle = 0;
			            MESSAGE("CLEAN_UP:Associated callback functions had been deleted as per,\
			                     Index %d, Handle %d\n",s_Count,*(arg_Handle),0,0,0,0); 
			   }
                       }/* end-if last application */
                       else
                       {
                           /**
                            * Now check whether the calling application is the last user for any particular port
                            * or tries to close the port which it has not opened
                            */
                           s_Temp = s_DeviceStatus[s_DeviceCount];
                           s_Handle = *arg_Handle;
                           for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
                           {
                               if((s_Temp & DEFAULT_SET) == DEFAULT_SET)
                               {
                                   if((s_Handle & DEFAULT_SET) == DEFAULT_SET)
                                   {/* this particular port this application has already opened */
                                       if(gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Count] != 0)
                                       {
                                           if(gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Count] == 1)
                                           {
                                               gbeMacGlobalData[s_DeviceCount].g_PortState[s_Count] = CLOSED;        
                                               gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Count] = 0;
                                           }			       
                                           else
                                           {
                                               gbeMacGlobalData[s_DeviceCount].g_PortUsageCount[s_Count]--;
                                           }
                                       }
                                   }
                               }
                               s_Temp = s_Temp >> DEFAULT_SET;
                               s_Handle = s_Handle >> DEFAULT_SET;
                           }
                           gbeMacGlobalData[s_DeviceCount].g_UsageCount--;
                       }
                       MESSAGE("g_UsageCount = %d\n",gbeMacGlobalData[s_DeviceCount].g_UsageCount,0,0,0,0,0);
                       taskUnlock();
                       s_ResourceCount--;
                       MESSAGE("taskUnlock() has been done\n",0,0,0,0,0,0);
                   }/* end-if taskLock achieved */
                   else
                   {/* taskLock failed */
                       MESSAGE("ERROR_TASK_LOCK_FAILED\n",0,0,0,0,0,0);
                       s_DeviceStopFlag = FALSE;
                       s_ReturnValue = ERROR_TASK_LOCK_FAILED;
                   }
#if 0
                   if(s_ResourceCount != 0)
                   {
                       taskUnlock();
                       s_ResourceCount--;
                       MESSAGE("taskUnlock() has been done\n",0,0,0,0,0,0);
                   }
#endif
               }/* end-if device is initialized earlier */
           }/* end-if the respective device is called in the close call */
        }/* end-for loop */
    }/* end-if valid handle */
    for(s_Count=0; s_Count<MAX_APP_SUPPORTED; s_Count++)
    {
        if(gbeMacCallback[s_Count].appHandle == *(arg_Handle))
        {
            MESSAGE("gbeMacCallback[%d] 0x%x  Handle 0x%x\n",s_Count,gbeMacCallback[s_Count].appHandle,*(arg_Handle),0,0,0);
            gbeMacCallback[s_Count].p_CallbackFn = NULL;
            gbeMacCallback[s_Count].appHandle = 0;
            MESSAGE("Associated callback functions had been deleted as per,\
                     Index %d, Handle %d\n",s_Count,*(arg_Handle),0,0,0,0); 
        }
    }
    *arg_Handle = 0;
    MESSAGE("-- %d --\n",s_DeviceCount,0,0,0,0,0);

    /**
     * Remove the associated callback function
     */
    return s_ReturnValue;
}


gbe_mac_error GbEMAC_Ioctl(uint32 *arg_Handle, uint32 ioctl_command, void * ioctl_struct)
{

    uint32 s_ReturnValue;
    uint32 s_IoctlFlag = TRUE;
    uint32 s_PortIndex;
    gbe_mac_ioctl_ptr *s_IoctlPtr;

    s_ReturnValue = SUCCESS;

    /**
     * check for the proper handle
     */
    if(((*arg_Handle) <= 0) || ((*arg_Handle) > 0x3FF))
    {
        s_IoctlFlag = FALSE;
        MESSAGE("INVALID HANDLE\n",0,0,0,0,0,0);
        s_ReturnValue = ERROR_INVALID_HANDLE;
    }

    s_IoctlPtr = (gbe_mac_ioctl_ptr *)ioctl_struct;
    s_PortIndex = s_IoctlPtr->portIndex;

    g_NpuStatus = isNPUMaster();

    if((s_PortIndex < 0) || (s_PortIndex >7))
    {
        s_IoctlFlag = FALSE;
        MESSAGE("IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID\n",0,0,0,0,0,0);
        s_ReturnValue = IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID;      
    }        

    if(s_IoctlFlag)
    {
        if(s_PortIndex < 4)
        {
            MESSAGE("** Pt Lone Media Card 0 PORT[0x%x]\n",s_PortIndex,0,0,0,0,0);
            if(gbeMacGlobalData[0].g_MacStatus != UNINITIALIZED)
            {
                if(g_NpuStatus == EGRESS)
	        {
	            s_ReturnValue = gbe_mac_config_handler(0,ioctl_command,ioctl_struct);
	        }
                else
                {
                   s_IoctlFlag = FALSE;
                   s_ReturnValue = ERROR_NPU_IS_INGRESS;
                }
            }
            else
            {
                s_IoctlFlag = FALSE;
                MESSAGE("ERROR_MAC_NOT_INITIALIZED\n",0,0,0,0,0,0);
                s_ReturnValue = ERROR_MAC_NOT_INITIALIZED;
            }
            MESSAGE("-- 0 --\n",0,0,0,0,0,0);
        }
        else
        {
            MESSAGE("** Pt Lone Media Card 1 **\n",0,0,0,0,0,0);
            if(gbeMacGlobalData[1].g_MacStatus != UNINITIALIZED)
            {
                s_IoctlPtr->portIndex = s_PortIndex - 4;   
                if(g_NpuStatus == EGRESS)
	        {
                    MESSAGE("** Pt Lone Media Card 0 PORT[0x%x]\n",s_PortIndex,0,0,0,0,0);
	            s_ReturnValue = gbe_mac_config_handler(1,ioctl_command,ioctl_struct);
   	        }
                else
                {
                    s_IoctlFlag = FALSE;
                    s_ReturnValue = ERROR_NPU_IS_INGRESS;
                }
            }
            MESSAGE("-- 1 --\n",0,0,0,0,0,0);
        }
    }	
		
    if(s_IoctlFlag)
    {
        MESSAGE("IOCTL API called Successfully\n",0,0,0,0,0,0);
    }
    else
    {
        MESSAGE("IOCTL API could not called\n",0,0,0,0,0,0);
    }

    return s_ReturnValue;
}


uint32 get_intConnect_number(uint32 arg_DeviceNumber)
{
    if(arg_DeviceNumber == 0)
    {
        
	  return CPLD_MEDIA_PCI_INT;
    }
    else if(arg_DeviceNumber == 1)
    {
        return CPLD_SF_PCI_INT;
    }
    return CPLD_MEDIA_PCI_INT;
}

unsigned char get_device_EEPROM_offset(uint32 arg_DeviceNumber)
{
    if(arg_DeviceNumber == 0)
    {
        return MEDIA_IF_EEPROM_ADDR;
    }
    else if(arg_DeviceNumber == 1)
    {
        return SF_IF_EEPROM_ADDR;
    }
    return MEDIA_IF_EEPROM_ADDR;
}

unsigned int GbEMAC_Callback(
                             uint32 *argHandle,
                             void * arg_pCallback,
                             void * arg_pUserContext
                   )
{
    uint32 s_ReturnValue = SUCCESS;
    uint32 s_IoctlFlag = TRUE;
    uint32 s_Count;

    if(((*argHandle) <= 0) || ((*argHandle) > 0x3FF))
    {
        s_IoctlFlag = FALSE;
        MESSAGE("INVALID HANDLE\n",0,0,0,0,0,0);
        s_ReturnValue = ERROR_INVALID_HANDLE;
    }

    MESSAGE("GbEMAC_Callback Called .....Handle[0x%x]\n",*argHandle,0,0,0,0,0);
    if(s_IoctlFlag)
    {
        for(s_Count=0;s_Count<MAX_APP_SUPPORTED; s_Count++)
        { /* Check whether this application has any registered callback fn" */
            if(gbeMacCallback[s_Count].appHandle == *(argHandle))
            { /* Over write the callback function pointer */
                gbeMacCallback[s_Count].p_CallbackFn = arg_pCallback;
                gbeMacCallback[s_Count].p_UserContext = arg_pUserContext;
                MESSAGE("There is a callback already for this interrupt, so OVER_WRITTEN\n",
                              0,0,0,0,0,0);
                return SUCCESS;
            }
        }

        for(s_Count=0;s_Count<MAX_APP_SUPPORTED; s_Count++)
        { /* Register the callback function */
            if(gbeMacCallback[s_Count].appHandle == 0)
            { /* Empty handler in the array */
                gbeMacCallback[s_Count].p_CallbackFn = arg_pCallback;
                gbeMacCallback[s_Count].appHandle = *(argHandle);
                gbeMacCallback[s_Count].p_UserContext = arg_pUserContext;
                MESSAGE("callback is registered at index for Interrupt %d\n",s_Count,\
                         0,0,0,0,0);
                return SUCCESS;
            }
        }

        if(s_Count == MAX_APP_SUPPORTED)
        {/* The specified array is full */
            MESSAGE("NO_MORE_CALLBACK_FOR_INTERRUPT\n",0,0,0,0,0,0);
            s_ReturnValue = NO_MORE_CALLBACK_FOR_INTERRUPT;
        }
    }
    return s_ReturnValue;
}


