/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
* --------------------------------------------------------------------------
*/

/**
 * System defined include files required.
 */
#if defined(__VXWORKS__)
#include <vxworks.h>
#include <stdlib.h>
#include <taskLib.h>
#include <intLib.h>
#include <logLib.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <ctype.h>
#endif


/**
 * User defined include files required.
 */
#if defined(__VXWORKS__)
#include "../include/ixf1104ce_ioctl.h"
#include "../include/ixf1104ce_internal.h"
#else
#include "../include/ixf1104ce_ioctl.h"
#include "../include/ixf1104ce_internal.h"
#endif


/**
 * Types definitions whose scope is limited to this file.
 */
#define VALUE_IN_RANGE               0x1
#define VALUE_OUT_OF_RANGE           0x0
#define GET_RMON_TOCTL_RANGE_LOW     0x0E317
#define GET_RMON_IOCTL_RANGE_HIGH    0x0E349


/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
extern gbe_mac_global_data gbeMacGlobalData[2];
extern gbe_mac_global_struct gbeMacGlobalStruct[2]; /* Each one structure for each Pt Lone Media Card */
extern gbe_mac_mac_address gbeMacAddress[2];             /* For the two Pt Lone cards */

/* offset per port */
extern uint32 g_PerPortOffset[4];
extern unsigned int gbic_status;
extern unsigned int g_CuLinkStatusMI,g_CuLinkStatusSF;

extern gbe_mac_error gbe_mac_add_two_lword(
                                           uint32 arg_DeviceNumber,
                                           uint32 arg_Low,
                                           uint32 arg_High,
                                           uint32 arg_Value
                                          );

/*
 * gbe_mac_giga_port_struct *gbe_mac_giga_port;
 * gbe_mac_rmon_stat *gbeMacRmonStat[4];
 */

/**
 * Function definitions.
 */

gbe_mac_error gbe_mac_get_config_handler(
                                         uint32 arg_DeviceNumber,
                                         gbe_mac_IOCTL_CMD arg_cmd,
                                         void *arg_pIoctlPtr
                                        )
{
	
    static uint32 s_ReturnValue=0; 
    static uint32 s_PortNumber=0,s_ReadValue=0;
    gbe_mac_ioctl_ptr *p_IoctlPointer = NULL;
    gbe_mac_ioctl_ptr_64 *p_IoctlPointer64 = NULL;    
    uint32 s_ReadValue1, s_ReadValue2;

#if defined(__VALLEJO_LINK_UP__)
#define HIGH_BIT                 0x01
#define MDI_LINK_STATUS          0x01
    uint32 s_ReadValueCU=0,s_Count=0;
    uint32 s_Temp=0,s_ReadValuePortEnable=0,s_ReadValuePortIfMode=0;
    uint32 s_TempC=0, s_TempF=0,s_LinkStatus=0,s_PortCount=0;
#endif

    uint32 s_GetIoctlFlag = TRUE;
    uint32 s_RegAddress=0, s_PhyAddress=0;

    s_ReturnValue = SUCCESS;

    g_PerPortOffset[0] = MAC_CTL_REG_PORT0;
    g_PerPortOffset[1] = MAC_CTL_REG_PORT1;
    g_PerPortOffset[2] = MAC_CTL_REG_PORT2;
    g_PerPortOffset[3] = MAC_CTL_REG_PORT3;

    /**
     *  Check whether the void pointer is not NULL
     */
    MESSAGE("---->  gbe_mac_get_config_handle function is invoked\n",0,0,0,0,0,0);
    MESSAGE("ADDRESS OF IOCTL_PTR is %u\n",(uint32)arg_pIoctlPtr,0,0,0,0,0);
    MESSAGE("Device Number %d\n",arg_DeviceNumber,0,0,0,0,0);

    if(arg_pIoctlPtr == NULL)
    {
        MESSAGE("IOCTL_BUFFER_POINTER_NULL\n",0,0,0,0,0,0);
        s_GetIoctlFlag = FALSE;
        s_ReturnValue = IOCTL_BUFFER_POINTER_NULL;
    }
    else
    {
        MESSAGE("IOCTL_BUFFER Pointer is not NULL\n",0,0,0,0,0,0);
    }

    if((arg_cmd >= GET_RMON_TOCTL_RANGE_LOW) &&
       (arg_cmd <  GET_RMON_IOCTL_RANGE_HIGH))
    {
        p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *)arg_pIoctlPtr;
        s_PortNumber = p_IoctlPointer64->portIndex;
        MESSAGE("IOCTL_COMMAND is for getting RMON statistics\n",0,0,0,0,0,0);
    }
    else
    {
        p_IoctlPointer = (gbe_mac_ioctl_ptr *)arg_pIoctlPtr;
        s_PortNumber = p_IoctlPointer->portIndex;
        MESSAGE("IOCTL_COMMAND is for getting 32-bit value only\n",0,0,0,0,0,0);
    }
    
            
    /**
     * Check for the Valid Port Number for port individual command.
     */
    if(s_GetIoctlFlag)
    {
        if((s_PortNumber < PT_LONE_GBE_PORT_0 ) || 
           (s_PortNumber > PT_LONE_GBE_PORT_3))
        {
            MESSAGE("IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID\n",0,0,0,0,0,0);
            MESSAGE("Entered Port Number value is %d\n",s_PortNumber,0,0,0,0,0);
            s_GetIoctlFlag = FALSE;
            s_ReturnValue = IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID;
        }
    }

    /**
     *  Check for port state.
     */
    if(s_GetIoctlFlag)
    {
      if(arg_cmd != 0xe34B)
      {
        if(gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_PortNumber] == CLOSED)
        {
            MESSAGE("IOCTL_PORT_NOT_OPEN\n",0,0,0,0,0,0);
            s_GetIoctlFlag = FALSE;
            s_ReturnValue = IOCTL_PORT_NOT_OPEN;
        }
      }
    }
    

    if(s_GetIoctlFlag)
    { 
        MESSAGE("GET IOCTL_COMMAND is getting executed\n",0,0,0,0,0,0);
        switch(arg_cmd)
        {
            case GET_STN_ADDR: 
                p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                s_PortNumber = p_IoctlPointer64->portIndex;
                gbe_mac_read_64_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+STATION_ADDR_LOW),\
                                     &s_ReadValue1, &s_ReadValue2);
                p_IoctlPointer64->valueLow = s_ReadValue1;
                p_IoctlPointer64->valueHigh = s_ReadValue2;
                break;

            case GET_DUPLEX_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FDFC_TYPE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+FDFC_TYPE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_COLLISION_DIST:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+COLL_DIST),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_COLLISION_THLD:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+COLL_TSHD ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FCTX_TIMER:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+FCTX_TIMER ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FDFC_ADDR:
                p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                s_PortNumber = p_IoctlPointer64->portIndex;
                gbe_mac_read_64_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+FDFC_ADDR_LOW),\
                                     &s_ReadValue1, &s_ReadValue2);
                p_IoctlPointer64->valueLow = s_ReadValue1;
                p_IoctlPointer64->valueHigh = s_ReadValue2;
                break;
	
            case GET_IPG_RECEIVE_TIME_1:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ IPG_RECEIVE_TIME1),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_IPG_RECEIVE_TIME_2:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+IPG_RECEIVE_TIME2 ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_IPG_TRANSMIT_TIME:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ IPG_TRANSMIT_TIME),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PAUSE_THRESHOLD:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+PAUSE_THRESHOLD ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MAX_FRAME_SIZE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+MAX_FRAME_SIZE ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MAC_IF_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ MAC_IF_MODE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FLUSH_TX:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ FLUSH_TX),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FC_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+FC_MODE ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_FC_BACK_PRESSURE_LEN:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ FC_BACK_PRESSURE_LEN),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SHORT_RUNT_TH:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ SHORT_RUNT_TSHD),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_UNKNOWN_FRAME_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ UNKNOWN_FRAME_STT),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_CONFIG_WORD:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ RX_CONFIG_WORD),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_CONFIG_WORD:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ TX_CONFIG_WORD),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_DIV_CONFIG_WORD:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ DIV_CONFIG_WORD),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PKT_FILTER_CTL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ PKT_FILTER_CTL),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MUL_PORT_ADD:
                p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                s_PortNumber = p_IoctlPointer64->portIndex;
                gbe_mac_read_64_bits(arg_DeviceNumber,\
                                     (g_PerPortOffset[s_PortNumber]+ MUL_PORT_ADDR_LOW),\
                                     &s_ReadValue1, &s_ReadValue2);
                p_IoctlPointer64->valueLow = s_ReadValue1;
                p_IoctlPointer64->valueHigh = s_ReadValue2;
                break;

            /**
             * MAC RX Statistics Register IOCTL Commands.
             */        
            case GET_RX_OCTETS_OK:
  	        if (taskLock() == OK)
		{
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_OCTETS_OK),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][0],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][1],\
                                          s_ReadValue);
  		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][0] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][1] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
            
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][0];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][1];
                    taskUnlock();
                }
                break;

            case GET_RX_OCTETS_BAD:
  	        if (taskLock() == OK)
		{
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_OCTETS_BAD),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][2],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][3],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][2] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
 		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][3] = \
	                                  gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

	            p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][2];
 		    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][3];
                    taskUnlock();
                }
                break;

            case GET_RX_UC_PKTS:
  	        if (taskLock() == OK)
		{
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_UC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][4],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][5],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][4] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][5] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

	            p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][4];
 	            p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][5];
                    taskUnlock();
		}
                break;

            case GET_RX_MC_PKTS:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_MC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][6],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][7],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][6] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][7] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
             
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][6];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][7];
 		    taskUnlock();
		}
                break;

            case GET_RX_BC_PKTS:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_BC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][8],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][9],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][8] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][9] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][8];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][9];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_64:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_64),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][10],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][11],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][10] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][11] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];

                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][10];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][11];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_65_127:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_65_127),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][12],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][13],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][12] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][13] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][12];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][13];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_128_255:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_128_255),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][14],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][15],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][14] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][15] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][14];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][15];
                    taskUnlock();
		}
                break;
 
            case GET_RX_PKTS_256_511:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_256_511),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][16],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][17],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][16] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][17] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][16];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][17];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_512_1023:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_512_1023),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][18],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][19],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][18] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][19] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][18];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][19];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_1024_1518:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_1024_1518),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][20],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][21],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][20] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][21] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][20];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][21];
                    taskUnlock();
		}
                break;

            case GET_RX_PKTS_1519_MAX:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PKTS_1519_MAX),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][22],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][23],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][22] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][23] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][22];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][23];
                    taskUnlock();
		}
                break;

            case GET_RX_FCS_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_FCS_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][24],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][25],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][24] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
  		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][25] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][24];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][25];
                    taskUnlock();
		}
                break;

            case GET_VLAN_TAG:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + VLAN_TAG),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][26],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][27],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][26] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][27] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][26];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][27];
                    taskUnlock();
		}
                break;

            case GET_RX_DATA_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_DATA_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][28],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][29],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][28] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][29] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][28];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][29];
                    taskUnlock();
		}
                break;

            case GET_RX_ALLIGN_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_ALIGN_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][30],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][31],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][30] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][31] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][30];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][31];
                    taskUnlock();
		}
                break;

            case GET_RX_LONG_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_LONG_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][32],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][33],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][32] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][33] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][32];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][33];
                    taskUnlock();
		}
                break;

            case GET_RX_JABBER_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_JABBER_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][34],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][35],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][34] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][35] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][34];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][35];
                    taskUnlock();
		}
                break;

            case GET_RX_PAUSE_MAC_CTL:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_PAUSE_MAC_CTL),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][36],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][37],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][36] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][37] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][36];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][37];
                    taskUnlock();
		}
                break;

            case GET_RX_UNKNOWN_CTL_FRAME:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RX_UNKNOWN_CTL_FRAME),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][38],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][39],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][38] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][39] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][38];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][39];
                    taskUnlock();
		}
                break;

            case GET_VLONG_ERR:
		if(taskLock() == OK)
                {
    		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + VLONG_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][40],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][41],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][40] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][41] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][40];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][41];
                    taskUnlock();
		}
                break;

            case GET_RUNT_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + RUNT_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][42],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][43],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][42] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][43] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][42];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][43];
                    taskUnlock();
		}
                break;

            case GET_SHORT_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + SHORT_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][44],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][45],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][44] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][45] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][44];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][45];
                    taskUnlock();
		}
                break;

            case GET_SEQ_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + SEQ_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][48],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][49],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][48] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][49] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][48];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][49];
                    taskUnlock();
		}
                break;

            case GET_SYMBOL_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + SYMBOL_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][50],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][51],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][50] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][51] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][50];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][51];
                    taskUnlock();
		}
                break;


            /**
             * TX Statistics Register IOCTL Commands.
             */
            case GET_TX_OCTETS_OK:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_OCTETS_OK),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][52],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][53],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][52] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][53] = \
	                                   gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][52];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][53];
                    taskUnlock();
		}
                break;

            case GET_TX_OCTETS_BAD:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_OCTETS_BAD),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][54],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][55],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][54] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][55] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][54];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][55];
                    taskUnlock();
		}
                break;

            case GET_TX_UC_PKTS:
 		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_UC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][56],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][57],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][56] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][57] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][56];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][57];
                    taskUnlock();
		}
                break;

            case GET_TX_MC_PKTS:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_MC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][58],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][59],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][58] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][59] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][58];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][59];
                    taskUnlock();
		}
                break;

            case GET_TX_BC_PKTS:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_BC_PKTS),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][60],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][61],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][60] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][61] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][60];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][61];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_64:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_64),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][62],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][63],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][62] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][63] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][62];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][63];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_65_127:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_65_127),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][64],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][65],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][64] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][65] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][64];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][65];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_128_255:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_128_255),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][66],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][67],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][66] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][67] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][66];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][67];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_256_511:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_256_511),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][68],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][69],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][68] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][69] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][68];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][69];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_512_1023:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_512_1023),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][70],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][71],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][70] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][71] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][70];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][71];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_1024_1518:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_1024_1518),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][72],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][73],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][72] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][73] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][72];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][73];
                    taskUnlock();
		}
                break;

            case GET_TX_PKTS_1519_MAX:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PKTS_1519_MAX),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][74],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][75],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][74] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][75] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][74];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][75];
                    taskUnlock();
		}
                break;

            case GET_TX_DEFERRED_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_DEFERRED_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][76],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][77],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][76] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][77] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][76];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][77];
                    taskUnlock();
		}
                break;

            case GET_TX_TOTAL_COLLISION:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_TOTAL_COLLISION),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][78],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][79],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][78] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][79] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][78];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][79];
                    taskUnlock();
		}
                break;

            case GET_TX_SINGLE_COLLISION:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_SINGLE_COLLISION),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][80],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][81],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][80] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][81] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][80];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][81];
                    taskUnlock();
		}
                break;

            case GET_TX_MUL_COLLISION:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_MUL_COLLISION),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][82],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][83],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][82] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][83] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][82];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][83];
                    taskUnlock();
		}
                break;

            case GET_LATE_COLLISION:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + LATE_COLLISION),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][84],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][85],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][84] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][85] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][84];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][85];
                    taskUnlock();
		}
                break;

            case GET_TX_EXCV_COLLISION:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_EXCV_COLLISION),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][86],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][87],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][86] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][87] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][86];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][87];
                    taskUnlock();
		}
                break;

            case GET_TX_EXCV_DEFERRED_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_EXCV_DEFERRED_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][88],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][89],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][88] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][89] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][88];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][89];
                    taskUnlock();
		}
                break;

            case GET_TX_EXCV_LEN_DROP:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_EXCV_LEN_DROP),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][90],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][91],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][90] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][91] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][90];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][91];
                    taskUnlock();
		}
                break;

            case GET_TX_UNDERRUN:
		if(taskLock() == OK)
                {
  		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_UNDERRUN),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][92],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][93],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][92] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][93] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][92];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][93];
                    taskUnlock();
		}
                break;

            case GET_TX_VLAN_TAG:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_VLAN_TAG),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][94],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][95],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][94] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][95] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][94];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][95];
                    taskUnlock();
		}
                break;

            case GET_TX_CRC_ERR:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_CRC_ERR),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][96],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][97],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][96] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][97] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][96];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][97];
                    taskUnlock();
		}
                break;

            case GET_TX_PAUSE_FRAME:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + TX_PAUSE_FRAME),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][98],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][99],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][98] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][99] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][98];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][99];
                    taskUnlock();
		}
                break;

            case GET_FC_COLLISION_SEND:
		if(taskLock() == OK)
                {
		    read_32_bits_from_isr(arg_DeviceNumber,((0x80 *s_PortNumber) + FC_COLLISION_SEND),&s_ReadValue);
		    s_ReturnValue = gbe_mac_add_two_lword(arg_DeviceNumber,\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][100],\
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][101],\
                                          s_ReadValue);
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][100] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[0];
		    gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][101] = \
                                          gbeMacGlobalStruct[arg_DeviceNumber].gbeMacAddResult[1];
                    p_IoctlPointer64->valueLow = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][100];
                    p_IoctlPointer64->valueHigh = gbeMacGlobalStruct[arg_DeviceNumber].gbeMacRmonStat[s_PortNumber][101];
                    taskUnlock();
		}
                break;


            /**
             * Global Status and Configuration Registers IOCTL Commands.
             */
            case GET_PORT_STATUS:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (PORT_ENABLE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_INTERFACE_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (INTERFACE_MODE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_LINK_UP_STATUS:
#if defined(__VALLEJO_LINK_UP__)
                /**
                 * This is work arround for the bug in Vallejo, as the Link up status registers
                 * does't gets updated as the link goes up.  The link information is retrived
                 * from the PHY register in for the Copper Link up, and from the GBIC control
                 * register for the Vallejo Link up.
                 */
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (INTERFACE_MODE),\
                                     &s_ReadValue1);
                s_ReadValue2 = 0;
                /**
                 * If Port is configured in Copper
                 */
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (PORT_ENABLE),\
                                     &s_ReadValuePortEnable);

                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (INTERFACE_MODE),\
                                     &s_ReadValuePortIfMode);

                s_TempC = s_ReadValuePortEnable & s_ReadValuePortIfMode;

                s_LinkStatus=0;
                s_Temp = 0;
	        for(s_PortCount=0; s_PortCount<PT_LONE_GBE_PORT_MAX; s_PortCount++)
  	        {
/*
                     gbe_mac_read_32_bits(arg_DeviceNumber,((0x80 * s_PortCount) + 0x61),\
                                          &s_ReadValue1);
*/
                      gbe_mac_phy_read(arg_DeviceNumber,s_PortCount,\
                                       0x01,&s_ReadValue1);

  	              if((s_ReadValue1 & 0x4) == 0x4)
   	              {
                          s_Temp = (0x1) << s_PortCount;
                      }

                      s_LinkStatus = s_LinkStatus | s_Temp;
                }

                s_TempC = s_TempC & s_LinkStatus;

                /**
                 * The port is configured in Fiber, so GBIC status register needs to be read.
                 * Value is updated in s_Temp variable.
                 */
                s_TempF = 0;

                s_TempF = s_ReadValuePortEnable ^ s_ReadValuePortIfMode;

                if(s_TempF != 0)
                {
                    s_ReadValue1 = gbic_status;
                    s_ReadValue1 = (s_ReadValue1 >> 20) & 0xF;
                    s_TempF = s_TempF ^ s_ReadValue1;
                }

                s_ReadValue2 = s_TempC | s_TempF;
                p_IoctlPointer->value = s_ReadValue2;
#else
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (LINK_UP_STATUS),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
#endif
                break;

            case GET_MAC_SOFT_RESET:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (MAC_SOFT_RESET),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MDIO_RESET:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (MDIO_RESET),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_UI_ENDIAN_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (UI_ENDIAN_MODE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_LED_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (LED_MODE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_LED_FLASH_RATE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (LED_FLASH_RATE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_LED_FAULT_ACTION:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (LED_FAULT_ACTION),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_JTAG_ID:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (JTAG_ID),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;


            /**
             * RX FIFO Register IOCTL Commands
             */
            case GET_RX_FIFO_HIGH_WATERMARK:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_RX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_RX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_RX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_RX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_RX_FIFO_LOW_WATERMARK:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_RX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_RX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_RX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_RX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_RX_FRAME_REMOVED:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_RX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_RX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_RX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_RX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_RX_FIFO_PORT_RESET:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_PORT_RESET),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_FIFO_FRAME_DROP:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_DROP_ENABLE),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_FIFO_OVERFLOW_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_OVRFLW_EVENT),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_OUT_SEQUENCE_INFO:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_OUT_OF_SEQ),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_DROPPED_PKTS:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_RXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_RXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_RXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_RXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_RW_PTR_RX_FIFO:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_RXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_RXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_RXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_RXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_OCCUPANCY_RX_FIFO:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (RXPORT0_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (RXPORT0_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (RXPORT0_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (RXPORT0_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_CAPTURED_PKT_LEN:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (CAPTURED_PKT_LEN),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_INDIRECT_ADR_CTL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (INDIRECT_ACC_CTL),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_READ_DATA:
                p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                s_PortNumber = p_IoctlPointer64->portIndex;
                gbe_mac_read_64_bits(arg_DeviceNumber,\
                                     (INDIRECT_MEM_DATA_REG1),\
                                     &s_ReadValue1, &s_ReadValue2);
                p_IoctlPointer64->valueLow = s_ReadValue1;
                p_IoctlPointer64->valueHigh = s_ReadValue2;
                break;

            case GET_CAPTURE_ENABLE_RX_FIFO:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_CAPT_LPBK_ENB),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PRE_PENDING_CRC_ENABLE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (RXFIFO_PAD_CRCSTRIP_ENB),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MATCHING_PATTERN:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_MATCH_PATTERN),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_MATCH_PATTERN),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_MATCH_PATTERN),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_MATCH_PATTERN),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_JUMBO_PKT_SIZE:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_JUMBO_PKT_SIZE),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_JUMBO_PKT_SIZE),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_JUMBO_PKT_SIZE),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_JUMBO_PKT_SIZE),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_PKT_DROP_CAP_FIFO:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PKT_DROP_CAP_FIFO_0),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PKT_DROP_CAP_FIFO_1),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PKT_DROP_CAP_FIFO_2),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PKT_DROP_CAP_FIFO_3),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;


            /**
             * TX FIFO Registers IOCTL commands
             */
            case GET_TX_FIFO_HIGH_WATERMARK:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TX_HIGH_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_FIFO_LOW_WATERMARK:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TX_LOW_WATERMARK),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_MAC_THRESHOLD:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TX_MAC_THRESHOLD),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TX_MAC_THRESHOLD),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TX_MAC_THRESHOLD),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TX_MAC_THRESHOLD),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_FIFO_OVERFLOW_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (TXFIFO_OVFLW),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_LOOP_RX_TX_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (LOOP_RXDATA_TXFIFO),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_FIFO_PORT_RESET:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (TXFIFO_PORT_RESET),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_DROP_FRAME:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TX_FRAME_REMOVED),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_DROP_PKTS:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TXFIFO_PKT_DROP),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_RW_PTR:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT0_TXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT1_TXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT2_TXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (PORT3_TXFIFO_RW_PTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_OCCUPANCY:
                switch(s_PortNumber)
                {
                    case 0:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (TXPORT0_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    case 1:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (TXPORT1_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 2:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (TXPORT2_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;
                    case 3:
                          gbe_mac_read_32_bits(arg_DeviceNumber,\
                                               (TXPORT3_OCCUPNCY_CTR),\
                                               &s_ReadValue1);
                          p_IoctlPointer->value = s_ReadValue1;
                          break;

                    default:
                          break;
                };
                break;

            case GET_TX_INSERT_DATA:
                p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                s_PortNumber = p_IoctlPointer64->portIndex;
                switch(s_PortNumber)
                {
                    case 0:
                        gbe_mac_read_64_bits(arg_DeviceNumber,\
                                             (PORT0_INS_FIFO_DATA_REG0),\
                                             &s_ReadValue1, &s_ReadValue2);
                        p_IoctlPointer64->valueLow = s_ReadValue1;
                        p_IoctlPointer64->valueHigh = s_ReadValue2;
                        break;
  
                    case 1:
                        gbe_mac_read_64_bits(arg_DeviceNumber,\
                                             (PORT1_INS_FIFO_DATA_REG0),\
                                             &s_ReadValue1, &s_ReadValue2);
                        p_IoctlPointer64->valueLow = s_ReadValue1;
                        p_IoctlPointer64->valueHigh = s_ReadValue2;
                        break;

                    case 2:
                        gbe_mac_read_64_bits(arg_DeviceNumber,\
                                             (PORT2_INS_FIFO_DATA_REG0),\
                                             &s_ReadValue1, &s_ReadValue2);
                        p_IoctlPointer64->valueLow = s_ReadValue1;
                        p_IoctlPointer64->valueHigh = s_ReadValue2;
                        break;

                    case 3:
                        gbe_mac_read_64_bits(arg_DeviceNumber,\
                                             (PORT3_INS_FIFO_DATA_REG0),\
                                             &s_ReadValue1, &s_ReadValue2);
                        p_IoctlPointer64->valueLow = s_ReadValue1;
                        p_IoctlPointer64->valueHigh = s_ReadValue2;
                        break;

                    default:
                        break;
                };    
                break;

 
            case GET_TX_FIFO_INFO_ADR:
                switch(s_PortNumber)
                {
                    case 0:
                        gbe_mac_read_32_bits(arg_DeviceNumber,\
                                             (PORT0_INS_FIFO_CTL_REG),\
                                             &s_ReadValue1);
                        p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 1:
                        gbe_mac_read_32_bits(arg_DeviceNumber,\
                                             (PORT1_INS_FIFO_CTL_REG),\
                                             &s_ReadValue1);
                        p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 2:
                        gbe_mac_read_32_bits(arg_DeviceNumber,\
                                             (PORT2_INS_FIFO_CTL_REG),\
                                             &s_ReadValue1);
                        p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 3:
                        gbe_mac_read_32_bits(arg_DeviceNumber,\
                                             (PORT3_INS_FIFO_CTL_REG),\
                                             &s_ReadValue1);
                        p_IoctlPointer->value = s_ReadValue1;
                        break;

                    default:
                        break;
                };
                break;

            case GET_TX_FIFO_DROP_INSERT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (TXFIFO_PORTDROP_INS_ENB),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_MINI_FRAME_SIZE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (TXFIFO_PAD_ENB),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;


            /**
             * MDIO Command and Register IOCTL Commands
             */
            case GET_MDIO_CMD_ADDR:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (MDIO_SNGL_CMD),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MDIO_SINGLE_RW_DATA:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (MDIO_SINGLE_RW_DATA),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_AS_PHY_ADDR:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (AS_PHY_ADDR),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_MDIO_CTL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (MDIO_CTL),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            /**
             * SPI-3 Transmit and Global Configuration IOCTL Commands
             */
            case GET_SPI3_TX_CONFIG:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SPI3_TX_GLOBAL_CONFIG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SPI3_RX_CONFIG:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SPI3_RX_CONFIG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SPI3_TX_INT_STATUS:
                switch(s_PortNumber)
                {
                    case 0:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_TX_INT_STATUS_PORT0),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 1:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_TX_INT_STATUS_PORT1),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 2:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_TX_INT_STATUS_PORT2),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 3:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_TX_INT_STATUS_PORT3),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    default:
                        break;
                };
                break;

            case GET_SPI3_ADR_PARITY_ERROR:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SPI3_ADR_PARITY_ERROR_PORT),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SPI3_PKT_DISABLE_PORT:
                switch(s_PortNumber)
                {
                    case 0:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_DROP_CTR_PORT0),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 1:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_DROP_CTR_PORT1),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 2:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_DROP_CTR_PORT2),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 3:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_DROP_CTR_PORT3),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    default:
                        break;
                };
                break;

            case GET_SPI3_PKT_SYNC_ERR:
                switch(s_PortNumber)
                {
                    case 0:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_SYNCERR_DROP_CTR_PORT0),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 1:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_SYNCERR_DROP_CTR_PORT1),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 2:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_SYNCERR_DROP_CTR_PORT2),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 3:
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_SYNCERR_DROP_CTR_PORT3),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    default:
                        break;
                 };
                 break;

            case GET_SPI3_PKT_SHORT_DROP:
                switch(s_PortNumber)
                {
                    case 0:   
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_SHORT_DROP_CTR_PORT0),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 1:   
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_SHORT_DROP_CTR_PORT1),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 2:   
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_SHORT_DROP_CTR_PORT2),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    case 3:   
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (SPI3_PKT_SHORT_DROP_CTR_PORT3),\
                                                 &s_ReadValue1);
                            p_IoctlPointer->value = s_ReadValue1;
                        break;

                    default:
                        break;
                };
                break;

            /**
             * SERDES Block Registers IOCTL Commands
             */
            case GET_ACDC_COUPLING:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_ACDC_COUPLING),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SERDES_TX_DRV_COEFF:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_TX_DRV_COEFF),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_DRV_POW_LEVEL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_TX_DER_POWER_LEVEL),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_LINK_VALIDATION:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_TX_LINK_VALIDATION),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_TX_RX_POW_DOWN_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_TX_RX_POW_DOWN),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;


            case GET_RX_DATA_SYNC:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_DATA_SYNC),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_LINK_VALIDATION:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_LINK_VALIDATION),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_LINK_STT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_LINK_STT),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_PHASE_ROT:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_PHASE_ROT),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_PHASE_ROT_BUS:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_PHASE_ROT_BUS),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_LATCH_OBSRV_01:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_LATCH_OBSRV_01),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_LATCH_OBSRV_23:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_LATCH_OBSRV_23),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_RX_SIGNAL_LEVEL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_RX_SIGNAL_LEVEL),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_CLOCK_INTERFACE_MODE:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_CLKMODE_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SERDES_TX_CONFIG:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_SERDES_TX_CONFIG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_SERDES_RX_CONFIG:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_SERDES_RX_CONFIG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PLL_LOCK:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (SERDES_PLL_LOCK_STATUS),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;


            /**
             * GBIC Register IOCTL Commands
             */
            case GET_GBIC_STAUS:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (GBIC_STATUS_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_GBIC_CTL:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (GBIC_CTRL_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_I2C_CTL_DATA:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (I2C_CTRL_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PLL_TUNE_1:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (PLL1_TUNE_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;
	
            case GET_PLL_TUNE_2:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (PLL2_TUNE_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;
	
            case GET_PLL_TUNE_3:
                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                     (PLL3_TUNE_REG),\
                                     &s_ReadValue1);
                p_IoctlPointer->value = s_ReadValue1;
                break;

            case GET_PHY_REGISTER:
                 s_PhyAddress = s_PortNumber;
                 s_RegAddress = p_IoctlPointer->value;
                 s_ReturnValue = gbe_mac_phy_read(arg_DeviceNumber,\
                                                  s_PhyAddress,\
                                                  s_RegAddress,\
                                                  &s_ReadValue1);
                  p_IoctlPointer->value = s_ReadValue1;
                  break;

            default:
            	MESSAGE("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
                s_GetIoctlFlag = FALSE;
                s_ReturnValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
            	break;

	};
    }

    if(s_GetIoctlFlag)
    {
        MESSAGE("gbe_mac_get_config_handle is executed successfully\n",0,0,0,0,0,0);
    }
    else
    {
        MESSAGE("gbe_mac_get_config_handle coult NOT executed\n",0,0,0,0,0,0);
    }

    return s_ReturnValue;
}
