/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
* --------------------------------------------------------------------------
*/


/**
 * System defined include files required.
 */
#if defined(__VXWORKS__)
#include <vxworks.h>
#include <logLib.h>
#include <semLib.h>
#include <taskLib.h>
#include <fioLib.h>
#include <objLib.h>

#else /* defined (__LINUX__) */
#include <stdlib.h>
#include <linux/kernel.h> 
#include <stdio.h>
#endif

#include <stdio.h>

/**
 * User defined include files required.
 */
#if defined(__VXWORKS__)
#include "../include/ixf1104ce_ioctl.h"
#include "../include/ixf1104ce_internal.h"
#else /* defined (__LINUX__) */
#include "../include/ixf1104ce_ioctl.h"
#include "../include/ixf1104ce_internal.h"
#endif

/**
 * Types definitions whose scope is limited to this file.
 */
#define FIBER_MODE               0x0
#define COPPER_MODE              0x1

#define HALF_DUPLEX              0x0
#define DUPLEX_MODE              0x1
#define FULL_DUPLEX              0x1

#define FORCE_DUPLEX             0x2
#define PHY_DUPLEX               0x3

#define DEFAULT_RESET            0x0
#define DEFAULT_SET              0x1

#define LED_FLASH_RATE_SET       0x3
#define LED_FAULT_DISABLE        0xF

#define PORT_GLOBAL_MASK         0xF

#define LOOP_BACK                0x8
#define DEFAULT_MSB              0x10
#define MDI_ENABLE               0x2
#define MDI_CONTROL              0x3
#define I2C_CONTROL              0x13FFE00
#define TX_LINK_VALIDATION       0xF03C00
#define RX_TX_PWR_DWN            0x3C0F
#define ENDIAN_MASK              0x80000001

#define FDFC                     0x3
#define HDFC                     0x4
#define DIV_CONFIG_CONSTRAINT    0x1FFFF
#define GBIC_CTL_MASK            0x1FC0F

#define LOW_16_BITS              0xffff
#define HIGH_16_BITS             0xffff0000
#define FDFC_TYPE_SET            0xffff
#define COLLISION_DISTANCE       0x3ff
#define COLLISION_THRESHOLD      0xff
#define FCTX_TIMER_MASK          0xffff
#define IPG_TIME                 0x3ff
#define MAX_FRAME_SIZE_MASK      0x3fff
#define FC_BACK_PRESSURE_LEN_MASK 0x3f
#define SHORT_RUNT_THRESHOLD     0x1f

#define MDI_CONTROL_MASK         0x7
#define PLL_TUNE                 0x3FF
#define SPI3_RECEIVE_RESET       0x400000
#define LOOP_BACK_ENABLE         8
#define CRC_STRIPPING_ENABLE     0xF0

#define MII_MODE_STATUS          0x4
#define MDI_PROGRESS_STATUS      0x8
#define MDI_CONTROL_VAL          0x6
#define MDI_AUTOSCAN_ENABLE      0xF

#define PLL_TUNE_MASK            0x2FF
#define TX_CONFIG_MASK           0xB1E0
#define MAC_SPEED                0x03
#define PHY_DUPLEX_MODE          0x0100

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
extern gbe_mac_global_data gbeMacGlobalData[2];
extern gbe_mac_global_struct gbeMacGlobalStruct[2]; /* Each one structure for each Pt Lone Media Card */
extern gbe_mac_mac_address gbeMacAddress[2];             /* For the two Pt Lone cards */

/* offset per port */
extern uint32 g_PerPortOffset[4];


/**
 * NAME: gbe_mac_change_configuration
 *
 * DESCRIPTION: This function reconfigures the Vallejo in the 
 *              specified mode
 *
 * @Param: -IN    : uint32 arg_deviceNumber
 *                : uint32 arg_PortMask,
 *                : uint32 arg_mode
 * @Param: -INOUT :  
 * @Param: -OUT   : 
 *
 * @Return: 
 */
extern void gbe_mac_change_configuration(
                                 uint32 arg_DeviceNumber,
                                 uint32 arg_PortMask,
                                 uint32 arg_mode
                                  );

#if 0
extern void set_mode_and_duplex(uint32 arg_DeviceNumber, uint32 arg_PortMask, short mode);
#endif

/**
 * Function definitions.
 */
gbe_mac_error gbe_mac_set_config_handler(
                                         uint32 arg_DeviceNumber,
                                         gbe_mac_IOCTL_CMD arg_cmd,
                                         void *arg_pIoctlPtr
                                        )
{
    gbe_mac_ioctl_ptr *p_IoctlPointer = NULL;
    gbe_mac_ioctl_ptr_64 *p_IoctlPointer64 = NULL;
    uint32 s_SetIoctlFlag = TRUE;              /* Flag reset to 0, when an error occurs. */
    uint32 s_Count=0;
    uint32 mask=0;
	
    static uint32 s_ReturnValue, s_WriteValue, s_ReadValue;
    static uint32 s_ReadValue1, s_ReadValue2;
    static uint32 s_WriteHigh, s_WriteLow;
    static uint32 s_GetPortNumber, s_GetStatus, s_PhyMode, s_PortCount;
    static uint32 s_PortNumber,s_Temp,s_Mode,s_PortMask;
  
    uint32 s_ResourceCount=0;
    uint32 s_PhyAddress=0,s_ConfigValue=0, s_RegAddress=0;

    s_ReturnValue = SUCCESS;

    MESSAGE("IOCTL API calls gbe_mac_set_config_handler for SET type IOCTL Commands\n",0,0,0,0,0,0);
    MESSAGE("Device Number is %d\n",arg_DeviceNumber,0,0,0,0,0);

    g_PerPortOffset[0] = MAC_CTL_REG_PORT0;
    g_PerPortOffset[1] = MAC_CTL_REG_PORT1;
    g_PerPortOffset[2] = MAC_CTL_REG_PORT2;
    g_PerPortOffset[3] = MAC_CTL_REG_PORT3;
    
    /**
     *  Check whether the void pointer is not NULL
     */
    if(arg_pIoctlPtr == NULL)
    { /* passed Ioctl Pointer is NULL */
        MESSAGE("IOCTL_BUFFER_POINTER_NULL\n",0,0,0,0,0,0);
        s_SetIoctlFlag = FALSE;
        s_ReturnValue = IOCTL_BUFFER_POINTER_NULL;
    } /* end-if arg_pIoctlPtr */


    /**
     * Check for the Valid Port Number, if ioctl cmd in intended for one port.
     */
    p_IoctlPointer = (gbe_mac_ioctl_ptr *)arg_pIoctlPtr;
    if(s_SetIoctlFlag)
    { /* Ioctl Pointer is valid */
        s_PortNumber = p_IoctlPointer->portIndex;
        if((s_PortNumber < PT_LONE_GBE_PORT_0 ) ||
           (s_PortNumber > PT_LONE_GBE_PORT_3))
        { /* port Number is InValid */
            MESSAGE("IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID [0x%x]\n",s_PortNumber,0,0,0,0,0);
            s_SetIoctlFlag = FALSE;
            s_ReturnValue = IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID;
        }
        else
        { /* valid Port Number */
            MESSAGE("Valid Port Number [0x%x]\n",s_PortNumber,0,0,0,0,0);
        } /*end-if-else Port Number check */
    }

    /**
     *  Check for port state.
     */
    if(s_SetIoctlFlag)
    {
        if((arg_cmd != SET_PORT_STATUS) ||
           (arg_cmd != SET_PORT_INIT_ENABLE))
        {
            if(arg_cmd < 0xE216)                       /* The SET IOCTL command is for per port register. */
            { /* for Port Specific Ioctl */
                 MESSAGE("Opening the Ports\n",0,0,0,0,0,0);
                 if(gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_PortNumber] == CLOSED)
                 { /* Referenced port number is CLOSED */
                     MESSAGE("IOCTL_PORT_NOT_OPEN\n",0,0,0,0,0,0);
                     s_SetIoctlFlag = FALSE;
                     s_ReturnValue = IOCTL_PORT_NOT_OPEN;
                 } /* end-if port status */
            } /* end-if ioctl command */
        }/* end-if SET_PORT_STATUS IOCTL Command */
    }

    if(s_SetIoctlFlag)
    {
        /**
         * Copy the value to be written in the register in the local register, s_WriteValue.
         */
        s_WriteValue = p_IoctlPointer->value;
        MESSAGE("Trying to grab the Mutex Semaphore\n",0,0,0,0,0,0);
#if defined(__VXWORKS__)
        if(semTake(gbeMacGlobalData[arg_DeviceNumber].g_MutexSemaphore,\
                   WAIT_FOR_SEMAPHORE) == OK)
        { /* Take the semaphore */
#else                                                          /* if defined __LINUX__ */
                                                               /* If grab the Mutex Semaphore successfully */
#endif                                                         /* end __LINUX__ */
            MESSAGE("The set_config_handler functions grabs Mutex Semaphore and executes in switch-case\n",0,0,0,0,0,0);
            s_ResourceCount++;
            switch(arg_cmd)
            {
                /**
                 * Configurs register associated with each MAC port.
                 */
                case SET_STN_ADDR:                             /* Set Source MAC Address */
                    p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                    s_PortNumber = p_IoctlPointer64->portIndex;
                    s_WriteLow = p_IoctlPointer64->valueLow;
                    s_WriteHigh = p_IoctlPointer64->valueHigh;
                    gbe_mac_write_64_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+STATION_ADDR_LOW),\
                                          s_WriteLow, s_WriteHigh);
                    break;

                case SET_DUPLEX_MODE:                          /* Set half/full-duplex operation Mode. */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                         &s_ReadValue);
                    if((s_ReadValue & DUPLEX_MODE) != 
                       (s_WriteValue & DUPLEX_MODE))
                    {
                        s_ReturnValue = gbe_mac_reconfig_handler(arg_DeviceNumber,arg_cmd, arg_pIoctlPtr);
                    }
                    else
                    {
                        MESSAGE("MODE_ALREADY_SET_IN_SPECIFIED_DUPLEX_MODE\n",0,0,0,0,0,0);
                        s_ReturnValue = MODE_ALREADY_SET_IN_SPECIFIED_DUPLEX_MODE;
                    }
                    break;

                case SET_FDFC_TYPE:                            /* Set Type field of the Transmitted Pause frames. */
                    s_WriteValue &= FDFC_TYPE_SET;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+FDFC_TYPE),\
                                          s_WriteValue);
                    break;

                case SET_COLLISION_DIST:			/* Only valid in Half-duplex */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                         &s_ReadValue);
                    if((s_ReadValue & DUPLEX_MODE) == HALF_DUPLEX)
                    {
                        s_WriteValue &= COLLISION_DISTANCE;
                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                              (g_PerPortOffset[s_PortNumber]+COLL_DIST),\
                                              s_WriteValue);
                    }
                    else
                    {
                        MESSAGE("ERROR_INVALID_DUPLEX_MODE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_INVALID_DUPLEX_MODE;
                    }
                    break;

                case SET_COLLISION_THLD:                       /* Only valid in Half-duplex */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                         &s_ReadValue);
                    if((s_ReadValue & DUPLEX_MODE) == HALF_DUPLEX)
                    {
                        s_WriteValue &= COLLISION_THRESHOLD;
                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                              (g_PerPortOffset[s_PortNumber]+COLL_TSHD),\
                                              s_WriteValue);
                    }
                    else
                    {
                        MESSAGE("ERROR_INVALID_DUPLEX_MODE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_INVALID_DUPLEX_MODE;
                    }
                    break;

                case SET_FCTX_TIMER:                           /* Set Pause length sent to the receiving station. */
                    s_WriteValue &= FCTX_TIMER_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ FCTX_TIMER),\
                                          s_WriteValue);
                    break;

                case SET_FDFC_ADDR:                            /* Set Multicast Pause frame destination address. */
                    p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                    s_PortNumber = p_IoctlPointer64->portIndex;
                    s_WriteLow = p_IoctlPointer64->valueLow;
                    s_WriteHigh = p_IoctlPointer64->valueHigh;
                    gbe_mac_write_64_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+FDFC_ADDR_LOW),\
                                          s_WriteLow, s_WriteHigh);
                    break;

                case SET_IPG_RECEIVE_TIME1:                 /*Set first part of IPG time for non back-to-back transmisson*/
                    s_WriteValue &= IPG_TIME;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ IPG_RECEIVE_TIME1),\
                                          s_WriteValue);
                    break;

                case SET_IPG_RECEIVE_TIME2:               /*Set second part of IPG time for non back-to-back transmisson*/
                    s_WriteValue &= IPG_TIME;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ IPG_RECEIVE_TIME2),\
                                          s_WriteValue);
                    break;

                case SET_IPG_TRANSMIT_TIME:                /* Set IPG time for back-to-back transmissions. */
                    s_WriteValue &= IPG_TIME;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ IPG_TRANSMIT_TIME),\
                                          s_WriteValue);
                    break;

                case SET_PAUSE_THRESHOLD:
                    s_WriteValue &= LOW_16_BITS;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ PAUSE_THRESHOLD),\
                                          s_WriteValue);
                    break;

                case SET_MAX_FRAME_SIZE:                  /* Set Max frame size MAC can receive and transmit */
                    s_WriteValue &= MAX_FRAME_SIZE_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ MAX_FRAME_SIZE),\
                                          s_WriteValue);
                    break;

                case SET_MAC_IF_MODE:			
                    s_ReturnValue = gbe_mac_reconfig_handler(arg_DeviceNumber,arg_cmd, arg_pIoctlPtr);

                    if(s_ReturnValue == SUCCESS)
                    {/* configuration has changed successfully */
                        s_ReturnValue = gbe_mac_phy_read(arg_DeviceNumber,\
                                                         s_PortNumber,23,&s_ReadValue);
                        if(s_ReturnValue == SUCCESS)
                        {/* if PHY read operation success */
                            if((s_WriteValue & MII_MODE_STATUS) == 0)
                            {
                                s_Temp = s_ReadValue & 0xFFF0;
                            }
                            else
                            {
                                s_Temp = s_ReadValue | 0x1FF0;
                            }
                            s_ReturnValue = gbe_mac_phy_write(arg_DeviceNumber,\
                                                              s_PortCount,23,s_Temp);
                            s_Temp = s_WriteValue & MAC_SPEED;
                            switch(s_Temp)
                            {/* set MAC spped to be set in the PHY configuration register */
                                case 0:
                                        s_WriteValue = 0xDFBF;
                                        break;
    
                                case 1:
                                        s_WriteValue = 0xFFBF;
                                        break;
 
                                default:
                                        s_WriteValue = 0xDFBF;
                                        break;
                            }; 
                            
                            s_ReturnValue = gbe_mac_phy_read(arg_DeviceNumber,\
                                                             s_PortNumber,0,&s_ReadValue);
                            
                            s_ReadValue = s_ReadValue | 0x2040;
                            s_WriteValue = s_WriteValue & s_ReadValue;
			    /** patched : phy reg 0 should be 0x1200 */	
			    s_WriteValue = 0x1200;
                            gbe_mac_phy_write(arg_DeviceNumber,\
                                              s_PortCount,0,s_WriteValue);
                        }/*end-if PHY read operation success */
                    }/* end-if change configuration success */
                    break;

                case SET_FLUSH_TX:                      /* Set to flush all Transmit data */
                    s_WriteValue &= DEFAULT_SET;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ FLUSH_TX),\
                                          s_WriteValue);
                    break;

                case SET_FC_MODE:                      /* Indicate flow control mode for RX TX MAC */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (g_PerPortOffset[s_PortNumber]+ DESIRED_DUPLEX),\
                                         &s_ReadValue);

                    if((s_ReadValue & DUPLEX_MODE) == FULL_DUPLEX)
                    {
                        s_Temp = s_WriteValue & FDFC;
                        if(s_Temp != 0)
                        {
                            gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                  (g_PerPortOffset[s_PortNumber]+ FC_MODE),\
                                                  s_WriteValue);
                        }
                        else
                        {
                            MESSAGE("ERROR_INVALID_DUPLEX_MODE\n",0,0,0,0,0,0);
                            s_ReturnValue = ERROR_INVALID_DUPLEX_MODE;
                        }
                    }
                    else
                    {
                        s_Temp = s_WriteValue & HDFC;
                        if(s_Temp != 0)
                        {
                            gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                  (g_PerPortOffset[s_PortNumber]+ FC_MODE),\
                                                  s_WriteValue);
                        }
                        else
                        {
                            MESSAGE("ERROR_INVALID_DUPLEX_MODE\n",0,0,0,0,0,0);
                            s_ReturnValue = ERROR_INVALID_DUPLEX_MODE;
                        } 
                    }
                    break;

                case SET_FC_BACK_PRESSURE_LEN:
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                         &s_ReadValue);

                    if((s_ReadValue & DUPLEX_MODE) == HALF_DUPLEX)
                    {
                        s_WriteValue &= FC_BACK_PRESSURE_LEN_MASK;
                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                  (g_PerPortOffset[s_PortNumber]+ FC_BACK_PRESSURE_LEN),\
                                                  s_WriteValue);
                    }
                    else
                    {
                        MESSAGE("ERROR_INVALID_DUPLEX_MODE\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_INVALID_DUPLEX_MODE;
                    }
                    break;

                case SET_SHORT_RUNT_TH:               
                    s_WriteValue &= SHORT_RUNT_THRESHOLD;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ SHORT_RUNT_TSHD ),\
                                          s_WriteValue);
                    break;

                case SET_UNKNOWN_FRAME_STT:                    /* Forward/Discard Unknown control frames */
                    s_WriteValue &= DEFAULT_SET;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ UNKNOWN_FRAME_STT),\
                                          s_WriteValue);
                    break;

                case SET_TX_CONFIG_WORD:  
                    s_WriteValue &= TX_CONFIG_MASK;   
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ TX_CONFIG_WORD ),\
                                          s_WriteValue);
                    break;

                case SET_DIV_CONFIG_WORD:
                    s_WriteValue &= DIV_CONFIG_CONSTRAINT;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+ DIV_CONFIG_WORD ),\
                                          s_WriteValue);

                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (INTERFACE_MODE),\
                                         &s_GetPortNumber);
                    s_GetPortNumber = s_GetPortNumber >> s_PortNumber;
                    if((s_GetStatus & DEFAULT_SET) == COPPER_MODE)
                    {
                        if(((s_WriteValue >> FORCE_DUPLEX) & DEFAULT_SET) == DEFAULT_RESET)
                        {
                            s_PhyMode = (s_WriteValue >> PHY_DUPLEX) & DEFAULT_SET;
                            gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                 (g_PerPortOffset[s_PortNumber]+DESIRED_DUPLEX),\
                                                 &s_ReadValue);
                            if((s_PhyMode & DUPLEX_MODE) !=
                               (s_WriteValue & DUPLEX_MODE))
                            {
                                p_IoctlPointer->value = s_PhyMode;
                                s_ReturnValue = gbe_mac_reconfig_handler(arg_DeviceNumber,SET_DUPLEX_MODE, arg_pIoctlPtr);
                                if(s_ReturnValue == SUCCESS)
                                {
                                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                          (AS_PHY_ADDR),\
                                                          &s_ReadValue);
                                    if((s_ReadValue >> s_PortNumber) == DEFAULT_SET)
                                    {
                                        gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                             (g_PerPortOffset[s_PortNumber] + AS_CTL_REG),\
                                                              &s_ReadValue);
                                    }
                                    else
                                    {
                                        s_ReturnValue = gbe_mac_phy_read(arg_DeviceNumber,\
                                                                          s_PortNumber,0,&s_ReadValue);
                                    }
                                    s_ReadValue = s_ReadValue | PHY_DUPLEX_MODE;
                                    switch(s_PhyMode)
                                    {
                                        case HALF_DUPLEX:
                                                s_WriteValue = 0xFEFF;
                                                break;

                                        case FULL_DUPLEX:
                                                s_WriteValue = 0xFFFF;
                                                break;
   
                                        default:
                                                break;
                                    };
                                    s_WriteValue = s_WriteValue & s_ReadValue; 
				    /* Patch: Force auto-neg when changing duplex */
				    s_WriteValue = 0x1200;	 
                                    s_ReturnValue = gbe_mac_phy_write(arg_DeviceNumber,\
                                                                  s_PortNumber,0,s_WriteValue);
                                }
                            }
                        }
                    }
                    break;

                case SET_PKT_FILTER_CTL:                /* Filtering of specific packets types */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (RXFIFO_DROP_ENABLE),\
                                         &s_ReadValue);
                    s_GetPortNumber = s_ReadValue >> s_PortNumber;
                    s_GetStatus = s_GetPortNumber & DEFAULT_SET;
                    if(s_GetStatus == DEFAULT_SET)
                    {
                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                              (PKT_FILTER_CTL),\
                                              s_WriteValue);
                    }
                    else
                    {
                        MESSAGE("FAIL_RX_FIFO_ERRORED_FRAME_DROP_IS_DISABLE\n",0,0,0,0,0,0);
                        s_ReturnValue = FAIL_RX_FIFO_ERRORED_FRAME_DROP_IS_DISABLE;
                    }
                    break;

                case SET_MUL_PORT_ADD:
                    p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                    s_PortNumber = p_IoctlPointer64->portIndex;
                    s_WriteLow = p_IoctlPointer64->valueLow;
                    s_WriteHigh = p_IoctlPointer64->valueHigh;
                    gbe_mac_write_64_bits(arg_DeviceNumber,\
                                          (g_PerPortOffset[s_PortNumber]+MUL_PORT_ADDR_LOW),\
                                          s_WriteLow, s_WriteHigh);
                    break;

                /**
                 * Global Status and Configuration Register IOCTL Commands
                 */
                case SET_PORT_STATUS:
                    /**
                     * Port Enable de-assertion should be done when data transfer is not in progress in Receive 
                     * and Transmit paths.
                     */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (PORT_ENABLE),\
                                         &s_ReadValue);

		    s_Temp = s_WriteValue;
		    
		    mask = 1;
                    for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
                    {
                       /* if((s_ReadValue & DEFAULT_SET) == DEFAULT_RESET) */
			if(((s_ReadValue & mask)>>s_Count) == DEFAULT_RESET)
                        {
                            /*if((s_Temp & DEFAULT_SET) == DEFAULT_SET) */
			    if(((s_Temp & mask)>>s_Count) == DEFAULT_SET)	
                            {
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (TXFIFO_PORT_RESET),\
                                                      s_WriteValue);
                                s_WriteValue = s_WriteValue ^ PORT_GLOBAL_MASK;
				s_WriteValue &= PORT_GLOBAL_MASK;
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      TXFIFO_PORT_RESET,\
                                                      s_WriteValue);
                                gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = OPENED;
                            }
                       }
                       if(((s_ReadValue & mask)>>s_Count) == DEFAULT_SET)
                       {
                           if(((s_Temp & mask)>>s_Count) == DEFAULT_RESET)
                           {
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      TXFIFO_PORT_RESET,\
                                                      s_WriteValue);
                               gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = CLOSED;
                           }
                       }
			#if 0
                        s_ReadValue = s_ReadValue >> DEFAULT_SET;
			s_Temp = s_Temp >> DEFAULT_SET;			
			#endif 
			mask = mask << 1;
                    }

	
                    /*s_WriteValue = s_Temp;*/
		    s_WriteValue = s_Temp;

	            MESSAGE("PORT STATUS IOCTL- write val = 0x%x\n",s_WriteValue,0,0,0,0,0);	
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          PORT_ENABLE,\
                                          s_WriteValue);
                    break;

                case SET_INTERFACE_MODE:
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    p_IoctlPointer->value = s_WriteValue;
                    s_ReturnValue = gbe_mac_reconfig_handler(arg_DeviceNumber,arg_cmd, arg_pIoctlPtr);
                    break;

                case SET_MAC_SOFT_RESET:                        /* Per port s/w activated reset of MAC core */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          MAC_SOFT_RESET,\
                                          s_WriteValue);

                    s_Temp = s_WriteValue ^ PORT_GLOBAL_MASK;
                    s_Temp = s_Temp & PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          TXFIFO_PORT_RESET,\
                                          s_Temp);
                    for(s_Count=0;s_Count < PT_LONE_GBE_PORT_MAX; s_Count++)
                    {
                        s_Temp = s_WriteValue;
                        if((s_Temp & DEFAULT_SET) == TRUE)
                        {
                            gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = CLOSED;
                        }
                        s_WriteValue = s_WriteValue >> DEFAULT_SET;
                    }
                    break;

                case SET_MDIO_RESET:
                    s_WriteValue &= DEFAULT_SET;                /* Software activated reset of the MDIO module */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (MDIO_RESET),\
                                          s_WriteValue);
                    break;

                case SET_UI_ENDIAN_MODE:                      
                    s_WriteValue &= ENDIAN_MASK;               
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (UI_ENDIAN_MODE),\
                                          s_WriteValue);
                    break;

                case SET_LED_MODE:                             /* Global selection of LED mode */
                    s_WriteValue &= DEFAULT_SET;               
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (LED_MODE),\
                                          s_WriteValue);
                    break;

                case SET_LED_FLASH_RATE:                       /* Global selection of LED flash rate */
                    s_WriteValue &= LED_FLASH_RATE_SET;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (LED_FLASH_RATE),\
                                          s_WriteValue);
                    break;

                case SET_LED_FAULT_ACTION:                     /* Per-port Fault disable */
                    s_WriteValue &= LED_FAULT_DISABLE;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (LED_FAULT_ACTION),\
                                          s_WriteValue);
                    break;


                /**
                 * RX FIFO Register IOCTL Commands
                 */
                case SET_RFIFO_HIGH_WATERMARK:                 /* Set High Watermark level */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT0_RX_LOW_WATERMARK,\
                                                     &s_ReadValue1);
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT0_JUMBO_PKT_SIZE,\
                                                     &s_ReadValue2);
                                break;
                        case 1:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT1_RX_LOW_WATERMARK,\
                                                     &s_ReadValue1);
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT1_JUMBO_PKT_SIZE,\
                                                     &s_ReadValue2);
                                break;
                        case 2:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT2_RX_LOW_WATERMARK,\
                                                     &s_ReadValue1);
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT2_JUMBO_PKT_SIZE,\
                                                     &s_ReadValue2);
                                break;
                        case 3:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT3_RX_LOW_WATERMARK,\
                                                     &s_ReadValue1);
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT3_JUMBO_PKT_SIZE,\
                                                     &s_ReadValue2);
                                break;
                        default:
                                break;
                    };

                    if(s_ReadValue1 < s_WriteValue)
                    {
                        if(s_ReadValue2 < s_WriteValue)
                        {
                            switch(s_PortNumber)
                            {
                                case 0:
                                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                              PORT0_RX_HIGH_WATERMARK,\
                                                              s_WriteValue);
                                        break;
                                case 1:
                                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                              PORT1_RX_HIGH_WATERMARK,\
                                                              s_WriteValue);
                                        break;
                                case 2:
                                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                              PORT2_RX_HIGH_WATERMARK,\
                                                              s_WriteValue);
                                        break;
                                case 3:
                                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                              PORT3_RX_HIGH_WATERMARK,\
                                                              s_WriteValue);
                                        break;
                                default:
                                        break;
                            };
                        }
                        else
                        {
                            MESSAGE("ERROR_CONFLICT_WITH_JUMBO_FRAME_SIZE\n",0,0,0,0,0,0);
                            s_ReturnValue = ERROR_CONFLICT_WITH_JUMBO_FRAME_SIZE;
                        }
                    }
                    else
                    {
                        MESSAGE("ERROR_CONFLICT_WITH_LOW_WATERMARK\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_CONFLICT_WITH_LOW_WATERMARK;
                    }
                    break;

                case SET_RFIFO_LOW_WATERMARK:               /* Set Low Watermark level */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT0_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 1:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT1_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 2:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT2_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 3:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT3_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        default:
                                break;
                    };

                    if(s_ReadValue > s_WriteValue)
                    {
                        switch(s_PortNumber)
                        {
                            case 0:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          PORT0_RX_LOW_WATERMARK,\
                                                          s_WriteValue);
                                    break;
                            case 1:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          PORT1_RX_LOW_WATERMARK,\
                                                          s_WriteValue);
                                    break;
                            case 2:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          PORT2_RX_LOW_WATERMARK,\
                                                          s_WriteValue);
                                    break;
                            case 3:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          PORT3_RX_LOW_WATERMARK,\
                                                          s_WriteValue);
                                    break;
                            default:
                                    break;
                        };
                    }
                    else
                    {
                        MESSAGE("ERROR_CONFLICT_WITH_HIGH_WATERMARK\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_CONFLICT_WITH_HIGH_WATERMARK;
                    }
                    break;

                case SET_RX_FIFO_PORT_RESET:              /* soft reset register for each port in the RX block */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          RXFIFO_PORT_RESET,\
                                          s_WriteValue);

                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         SPI3_TX_GLOBAL_CONFIG,\
                                         &s_ReadValue);

                    s_ReadValue = s_ReadValue | SPI3_RECEIVE_RESET;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          SPI3_TX_GLOBAL_CONFIG,\
                                          s_ReadValue);
                    break;

                case SET_RX_FIFO_FRAME_DROP:              /* configure drooping of error packets */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (RXFIFO_DROP_ENABLE),\
                                          s_WriteValue);
                    break;

                case SET_INDIRECT_ADR_CTL:                
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (INDIRECT_ACC_CTL),\
                                          s_WriteValue);
                    break;

                case SET_CAPTURE_ENABLE_RX_FIFO:          /* enables the capture feature at different ports */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (RXFIFO_CAPT_LPBK_ENB),\
                                          s_WriteValue);
                    break;

                case SET_PRE_PENDING_CRC_ENABLE:          /* enables to pre-pend, and enables the CRC stripping */
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         RXFIFO_CAPT_LPBK_ENB,\
                                         &s_ReadValue);
                    s_Temp = s_ReadValue >> LOOP_BACK_ENABLE;
                    s_Temp = s_Temp & PORT_GLOBAL_MASK;
                    s_Temp = s_Temp ^ s_WriteValue;
                    s_Temp = s_Temp | CRC_STRIPPING_ENABLE;		    
		    s_WriteValue = s_WriteValue & s_Temp;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          RXFIFO_PAD_CRCSTRIP_ENB,\
                                          s_WriteValue);
                    break;

                case SET_MATCHING_PATTERN:                /* Set matching pattern to capturre 
                                                             specific packets from data traffic */
                    s_WriteValue &= LOW_16_BITS;
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT0_MATCH_PATTERN),\
                                                      s_WriteValue);
                                break;
                        case 1:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT1_MATCH_PATTERN),\
                                                      s_WriteValue);
                                break;
                        case 2:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT2_MATCH_PATTERN),\
                                                      s_WriteValue);
                                break;
                        case 3:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT3_MATCH_PATTERN),\
                                                      s_WriteValue);
                                break;
                        default:
                                break;
                    };
                    break;

                case SET_JUMBO_PKT_SIZE:                 /* Set Jumbo Packet size */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT0_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 1:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT1_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 2:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT2_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        case 3:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     PORT3_RX_HIGH_WATERMARK,\
                                                     &s_ReadValue);
                                break;
                        default:
                                break;
                    };
                    if(s_ReadValue > s_WriteValue)
                    {
                        switch(s_PortNumber)
                        {
                            case 0:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT0_JUMBO_PKT_SIZE),\
                                                          s_WriteValue);
                                    break;
                            case 1:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT1_JUMBO_PKT_SIZE),\
                                                          s_WriteValue);
                                    break;
                            case 2:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT2_JUMBO_PKT_SIZE),\
                                                          s_WriteValue);
                                    break;
                            case 3:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT3_JUMBO_PKT_SIZE),\
                                                          s_WriteValue);
                                    break;
                            default:
                                    break;
                        }; 
                    }
                    else
                    {
                        MESSAGE("ERROR_CONFLICT_WITH_HIGH_WATERMARK\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_CONFLICT_WITH_HIGH_WATERMARK;
                    }
                    break;

                /**
                 * TX FIFO Register IOCTL Commands
                 */
                case SET_TX_FIFO_HIGH_WATERMARK:              /* Set TX high watermark level */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT0_TX_LOW_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 1:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT1_TX_LOW_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 2:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT2_TX_LOW_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 3:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT3_TX_LOW_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        default:
                                break;
                    };
                    if(s_ReadValue < s_WriteValue)
                    {
                        switch(s_PortNumber)
                        {
                            case 0:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT0_TX_HIGH_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 1:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT1_TX_HIGH_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 2:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT2_TX_HIGH_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 3:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT3_TX_HIGH_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            default:
                                    break;
                        };
                    }
                    else
                    {
                        MESSAGE("ERROR_CONFLICT_WITH_LOW_WATERMARK\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_CONFLICT_WITH_LOW_WATERMARK;
                    }
                    break;

                case SET_TX_FIFO_LOW_WATERMARK:               /* Set TX Low watermark level */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT0_TX_HIGH_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 1:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT1_TX_HIGH_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 2:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT2_TX_HIGH_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        case 3:
                                gbe_mac_read_32_bits(arg_DeviceNumber,\
                                                     (PORT3_TX_HIGH_WATERMARK),\
                                                     &s_ReadValue);
                                break;
                        default:
                                break;
                    };
                    if(s_ReadValue > s_WriteValue)
                    {
                        switch(s_PortNumber)
                        {
                            case 0:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT0_TX_LOW_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 1:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT1_TX_LOW_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 2:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT2_TX_LOW_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            case 3:
                                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                          (PORT3_TX_LOW_WATERMARK),\
                                                          s_WriteValue);
                                    break;
                            default:
                                    break;
                        };
                    }
                    else
                    {
                        MESSAGE("ERROR_CONFLICT_WITH_HIGH_WATERMARK\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_CONFLICT_WITH_HIGH_WATERMARK;
                    }
                    break;

                case SET_MAC_THRESHOLD:                   /* MAC threshold for TX FIFO  */
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT0_TX_MAC_THRESHOLD),\
                                                      s_WriteValue);
                                break;
                        case 1:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT1_TX_MAC_THRESHOLD),\
                                                      s_WriteValue);
                                break;
                        case 2:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT2_TX_MAC_THRESHOLD),\
                                                      s_WriteValue);
                                break;
                        case 3:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT3_TX_MAC_THRESHOLD),\
                                                      s_WriteValue);
                                break;
                        default:
                                break;
                    };
                    break;

                case SET_LOOP_RX_TX:                      /* enables to perform the external loop back */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (LOOP_RXDATA_TXFIFO),\
                                          s_WriteValue);
                    break;

                case SET_TX_FIFO_PORT_RESET:             /* port reset register for each port in the TX block */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,
                                          (TXFIFO_PORT_RESET),\
                                          s_WriteValue);
                    break;

                case SET_TX_INSERT_DATA:                 
                    p_IoctlPointer64 = (gbe_mac_ioctl_ptr_64 *) arg_pIoctlPtr;
                    s_PortNumber = p_IoctlPointer64->portIndex;
                    s_WriteLow = p_IoctlPointer64->valueLow;
                    s_WriteHigh = p_IoctlPointer64->valueHigh;
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_write_64_bits(arg_DeviceNumber,\
                                                      (PORT0_INS_FIFO_DATA_REG0),\
                                                      s_WriteLow, s_WriteHigh);
                                break;

                        case 1:
                                gbe_mac_write_64_bits(arg_DeviceNumber,\
                                                      (PORT1_INS_FIFO_DATA_REG0),\
                                                      s_WriteLow, s_WriteHigh);
                                break;
  
                        case 2:
                                gbe_mac_write_64_bits(arg_DeviceNumber,\
                                                      (PORT2_INS_FIFO_DATA_REG0),\
                                                      s_WriteLow, s_WriteHigh);
                                break;

                        case 3:
                                gbe_mac_write_64_bits(arg_DeviceNumber,\
                                                      (PORT3_INS_FIFO_DATA_REG0),\
                                                      s_WriteLow, s_WriteHigh);
                                break;

                        default:
                                MESSAGE("IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID\n",0,0,0,0,0,0);
                                s_ReturnValue = IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID;
                                break;
                    };
                    break;

                case SET_TX_FIFO_INFO_ADR:
                    switch(s_PortNumber)
                    {
                        case 0:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT0_INS_FIFO_CTL_REG),\
                                                      s_WriteValue);
                                break;
                        case 1:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT1_INS_FIFO_CTL_REG),\
                                                      s_WriteValue);
                                break;
                        case 2:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT2_INS_FIFO_CTL_REG),\
                                                      s_WriteValue);
                                break;
                        case 3:
                                gbe_mac_write_32_bits(arg_DeviceNumber,\
                                                      (PORT3_INS_FIFO_CTL_REG),\
                                                      s_WriteValue);
                                break;
                        default:
                                MESSAGE("IOCLT_ERROR_INPUT_PORT_NUMBER_INVALID\n",0,0,0,0,0,0);
                                break;
                    };
                    break;

                case SET_TX_FIFO_DROP_INSERT:                 /* to independently enable the individual TX FIFO */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (TXFIFO_PORTDROP_INS_ENB),\
                                          s_WriteValue);
                    break;

                case SET_TX_MINI_FRAME_SIZE:                   /* independently enable different minimum length of the 
                                                                  packet to be transmitted to MAC */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (TXFIFO_PAD_ENB),\
                                          s_WriteValue);
                    break;

                /**
                 * MDIO Register IOCTL Commands.
                 */
                case SET_MDIO_CMD_ADDR:
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (MDIO_SNGL_CMD),\
                                          s_WriteValue);
                    break;

                case SET_MDIO_SINGLE_RW_DATA:
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (MDIO_CTL),\
                                         &s_ReadValue);
                    s_GetStatus = (s_ReadValue >> MDI_ENABLE) & DEFAULT_SET;
                    if(s_GetStatus == DEFAULT_SET)
                    {
                        s_GetStatus = s_WriteValue >> DEFAULT_MSB;
                        if(s_GetStatus > 0)
                        {
                            MESSAGE("WARNING_ONLY_LOWER_FOUR_BYTES_ARE_WRITABLE\n",0,0,0,0,0,0);
                        }
                        s_WriteValue &= LOW_16_BITS;
                        gbe_mac_write_32_bits(arg_DeviceNumber,\
                                              MDIO_SINGLE_RW_DATA,\
                                              s_WriteValue);
                    }
                    else
                    {
                        MESSAGE("ERROR_MDI_ENABLE_BIT_IS_RESET_IN_MDI_CONTROL_REG\n",0,0,0,0,0,0);
                        s_ReturnValue = ERROR_MDI_ENABLE_BIT_IS_RESET_IN_MDI_CONTROL_REG;
                    }
                    break;

                case SET_AS_PHY_ADDR:
                    s_WriteValue &= MDI_AUTOSCAN_ENABLE;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (AS_PHY_ADDR),\
                                          s_WriteValue);
                    break;

                case SET_MDIO_CTL:
                    s_Temp = s_WriteValue >> MDI_CONTROL;
                    if(s_Temp > 0)
                    {
                        MESSAGE("WARNING_ONLY_LOWER_THREE_BITS_ARE_WRITABLE\n",0,0,0,0,0,0);
                    }
                    s_WriteValue &= MDI_CONTROL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (MDIO_CTL),\
                                          s_WriteValue);
                    break;


                /**
                 * SPI-3 Register IOCTL Command
                 */
                case SET_SPI3_TX_CONFIG:             
                    /**
                     * Value
                     * SPI3 Block Mode     0 - MPHY
                     *                     1 - 4x8 Mode     -- Default Value
                     * SPI3 Parity         0 - Odd Parity   -- Default Value
                     *                     1 - Even Parity
                     */
                    s_ReturnValue = gbe_mac_reconfig_handler(arg_DeviceNumber,arg_cmd, arg_pIoctlPtr);
                    break;

                case SET_SPI3_RX_CONFIG:               
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SPI3_RX_CONFIG),\
                                          s_WriteValue);
                    break;

                /** 
                 * SERDES Config IOCTL Command
                 */
                case SET_ACDC_COUPLING:                    /* selection of AC/DC coupling on output of each SerDes port */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_ACDC_COUPLING),\
                                          s_WriteValue);
                    break;

                case SET_SERDES_TX_DRV_COEFF:              /* select programmable drive strengths on each SerDes ports */
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_TX_DRV_COEFF),\
                                          s_WriteValue);
                    break;

                case SET_TX_DRV_POW_LEVEL:                
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_TX_DER_POWER_LEVEL),\
                                          s_WriteValue);
                    break;

                case SET_TX_LINK_VALIDATION:               /* configuration of Links and stores the status of the links */
                    s_WriteValue &= TX_LINK_VALIDATION;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_TX_LINK_VALIDATION),\
                                          s_WriteValue);
                    break;

                case SET_TX_RX_POW_DOWN:   
                    s_WriteValue &= RX_TX_PWR_DWN;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_TX_RX_POW_DOWN),\
                                          s_WriteValue);
                    break;

                case SET_RX_DATA_SYNC:                      /* Allows the incoming data stream to be slipped by one 
                                                               bit each time the signal is activated */
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_RX_DATA_SYNC),\
                                          s_WriteValue);
                    break;
                  
                case SET_CLOCK_IF_MODE:
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (SERDES_CLKMODE_REG),\
                                          s_WriteValue);
                    break;

                /**
                 * GBIC Register IOCTL Commands
                 */
                case SET_GBIC_CTL:
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (GBIC_CTRL_REG),\
                                          s_WriteValue);
                    break;

                case SET_I2C_CTL_DATA:
                    s_WriteValue &= I2C_CONTROL;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (I2C_CTRL_REG),\
                                          s_WriteValue);
                    break;

                case SET_PLL_TUNE_1:
                    s_WriteValue &= PLL_TUNE_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (PLL1_TUNE_REG),\
                                          s_WriteValue);
                    break;

                case SET_PLL_TUNE_2:
                    s_WriteValue &= PLL_TUNE_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (PLL2_TUNE_REG),\
                                          s_WriteValue);
                    break;


                case SET_PLL_TUNE_3:
                    s_WriteValue &= PLL_TUNE_MASK;
                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          (PLL3_TUNE_REG),\
                                          s_WriteValue);
                    break;

                case SET_PORT_INIT_ENABLE:
                    s_WriteValue &= PORT_GLOBAL_MASK;
                    gbe_mac_read_32_bits(arg_DeviceNumber,\
                                         (PORT_ENABLE),\
                                         &s_ReadValue);
                    s_Temp = s_WriteValue;
                    for(s_Count=0;s_Count<PT_LONE_GBE_PORT_MAX;s_Count++)
                    {
                        if((s_Temp & DEFAULT_SET) == DEFAULT_SET)
                        {
                            gbeMacGlobalData[arg_DeviceNumber].g_PortState[s_Count] = OPENED;
                        }
                        s_Temp = s_Temp >> DEFAULT_SET;
                    }
                    s_WriteValue = s_WriteValue | s_ReadValue;

                    gbe_mac_write_32_bits(arg_DeviceNumber,\
                                          PORT_ENABLE,\
                                          s_WriteValue);
                    break;

                case SET_PHY_REGISTER:
                   s_PhyAddress = s_PortNumber;
                   s_RegAddress = (s_WriteValue >> 16) & 0x1F;
                   s_ConfigValue = (s_WriteValue & 0xFFFF);
                   MESSAGE("REG ADD 0x%x  VALUE 0x%x\n",s_RegAddress,s_ConfigValue,0,0,0,0);
                   s_ReturnValue = gbe_mac_phy_write( arg_DeviceNumber,\
                                                     s_PhyAddress,\
                                                     s_RegAddress,\
                                                     s_ConfigValue);
                   break;

                case SET_CHANGE_CONFIG:
                    s_PortMask = (0x1) << s_PortNumber;
                    s_Mode = s_WriteValue & 0x1F; 
#if 0
                    set_mode_and_duplex(arg_DeviceNumber,\
                                                 s_PortMask,\
                                                 s_Mode);
#endif
                    gbe_mac_change_configuration(arg_DeviceNumber,\
                                                 s_PortMask,\
                                                 s_Mode);
                    break;


                default:
                    MESSAGE("IOCTL_ERROR_UNKNOWN_IOCTL_CODE\n",0,0,0,0,0,0);
                    s_ReturnValue = IOCTL_ERROR_UNKNOWN_IOCTL_CODE;
                    break;
            }; /* end-switch case */
#if defined(__VXWORKS__)
	} /* end -if semTake successful. */
        else
        { /* Could not grab the Semaphore */
            MESSAGE("S_objLib_OBJ_TIMEOUT\n",0,0,0,0,0,0);
            s_SetIoctlFlag = FALSE;
            s_ReturnValue = S_objLib_OBJ_TIMEOUT;
        }
#else
#endif
    } /* end-if s_SetIoctlFlag Flag */

    if(s_ResourceCount == 1)
    { /* Semaphore has been taken, */
#if defined(__VXWORKS__)
        /* Release the Semaphore */
        semGive(gbeMacGlobalData[arg_DeviceNumber].g_MutexSemaphore);
        s_ResourceCount--;
#else                                          /* For Linux Environment. */
#endif                                         /* Releasing Semaphore */
        MESSAGE("set_config_handler function releases the Mutex Semaphore\n",0,0,0,0,0,0);
    }

    return s_ReturnValue;
}

