/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: gdvaddad $
 *   DATE:           $Date: 2002/12/08 06:38:39 $
 *   LAST MODIFIED:  $Modtime: 5/03/02 1:37p $
 *   REV#:           $Revision: 1.3 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/generic/genericdriver.c $
 *   WORKFILE:       $Workfile: genericdriver.c $
 *
 * --------------------------------------------------------------------------
 * Project: Generic Driver
 *
 * Purpose: Provides the specfication to the configuration routines.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: genericdriver.c,v $
 *   Revision 1.3  2002/12/08 06:38:39  gdvaddad
 *   phase 5c/6
 *
 *   Revision 1.2  2002/10/25 20:24:49  machambe
 *   no message
 *
 *   Revision 1.1  2002/10/05 00:08:38  machambe
 *   no message
 *
 *   Revision 1.1  2002/08/14 23:04:18  machambe
 *   no message
 *
 * 
 * 4     5/03/02 1:39p Srodrig1
 * 
 * 3     5/02/02 2:56p Srodrig1
 * Fixing 8 bit alignmenet on the read
 * 
 * 1     4/17/02 2:04p Srodrig1
 * Generic chip driver functions
 * 
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#include "msgQLib.h"
#else 
#include <memory.h>
#endif

#include <string.h>
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "hal.h"
#include "bldGeneric.h"
#include "genericdriver.h"
#include "sim.h"

#ifdef SLOW_PORT
extern ushort SlowPortRead(bb_RegPointer_type addr);
extern ushort SlowPortWrite(bb_RegPointer_type addr, ushort val);
#endif

/******************************************************************
* Function : GenericGetBuildVersion
*-----------------------------------------------------------------
* Description  : This routine returns the firmware build version of 
*                the Generic driver.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : drvName : Null terminated string describing the driver
*           date : The build date of the driver.
*           buildVer : The version number of the driver.
*           buildRev : The revision number of the driver.
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
GenericDriverGetBuildVersion(bb_ChipData_t* pChipData, char* drvName,
    char *date, ushort *buildVer, ushort *buildRev)
{
    /* Get the Generic Driver build information */
    strcpy(drvName, GENERIC_BUILD_DRIVER_NAME);
    strcpy(date, GENERIC_BUILD_DATE);
    *buildVer = GENERIC_BUILD_VERSION;
    *buildRev = GENERIC_BUILD_REV;

    return bb_NO_ERROR;
}

/****************************************************************************
* Function : GenericRead
*----------------------------------------------------------------------------
* Description  : This routine will read starting at a particular 
*				 location and read for N number of words.
*----------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start reading from.
*				length : Number of words to read.
* Outputs : buffer : The buffer to contain the read data.
*           
*----------------------------------------------------------------------------
* Returns : bb_WORD_SIZE_UNSUPPORTED
*           bb_NO_ERROR
*----------------------------------------------------------------------------
* Access Globals : none
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
bb_Error_e 
GenericDriverRead(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
    ulong address, ushort length, void *buffer)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    uint ndx;
    uchar *oneBytePtr;
    ushort *twoBytePtr;
    ulong *fourBytePtr;
    ulong temp;

    switch (wordSize)
    {

    case ONE_BYTE:
        oneBytePtr = (uchar*) buffer;
        for(ndx=0; ndx < length; ndx++, oneBytePtr++)
        {

            *oneBytePtr =
                (uchar) READ_FROM_HW(((bb_RegPointer_type)BaseAddress) +
                (address + (ulong)ndx));
        }

        break;

    case TWO_BYTES:
        twoBytePtr = (ushort*) buffer;
        for(ndx=0; ndx < length; ndx++, twoBytePtr++)
        {
            *twoBytePtr = READ_FROM_HW((bb_RegPointer_type)(BaseAddress +
                ((ushort) address + ndx)));

        }
        break;

    case FOUR_BYTES:
        fourBytePtr = (ulong*) buffer;

        for(ndx=0; ndx < length; ndx++, fourBytePtr++)
        {

            temp = (ulong) (READ_FROM_HW(BaseAddress + (ushort) address +
                (ndx * 2)));
            *fourBytePtr = READ_FROM_HW((bb_RegPointer_type)(BaseAddress +
                (ushort) address + ((ndx* 2) + 1))) << 16 | temp;

        }

        break;
    case EIGHT_BYTES:
    default:
        return bb_WORD_SIZE_UNSUPPORTED;
        break;
    }

    return bb_NO_ERROR;
}
/****************************************************************************
* Function : GenericWrite
*----------------------------------------------------------------------------
* Description  : This routine will write starting at a particular 
*						location and write for N number of words.
*----------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start writing to.
*				length : Number of words to write.
*				buffer : The buffer to contain the read data.
* Outputs : None.
*----------------------------------------------------------------------------
* Returns : bb_WORD_SIZE_UNSUPPORTED
*           bb_NO_ERROR
*----------------------------------------------------------------------------
* Access Globals : none
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
bb_Error_e 
GenericDriverWrite(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
    ulong address, ushort length, void *buffer)
{
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress;
    uint ndx;
    uchar *oneBytePtr;
    ushort *twoBytePtr;
    ulong *fourBytePtr;


    switch (wordSize)
    {
    case ONE_BYTE:
        oneBytePtr = (uchar*) buffer;
        for(ndx=0; ndx < length; ndx++, oneBytePtr++)
        {
            WRITE_TO_HW( (bb_RegPointer_type)BaseAddress + (address + ndx), *oneBytePtr);
        }
        break;
    case TWO_BYTES:
        twoBytePtr = (ushort*) buffer;

        for(ndx=0; ndx < length; ndx++, twoBytePtr++)
        {
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx), *twoBytePtr);
        }
        break;
    case FOUR_BYTES:
        fourBytePtr = (ulong*) buffer;
        for(ndx=0; ndx < length; ndx++, fourBytePtr++)
        {
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx),
                (ushort) (*fourBytePtr >> 16));
            WRITE_TO_HW(BaseAddress + ((ushort) address + ndx), 
                (ushort) (*fourBytePtr & 0xFFFF));
        }
        break;
    case EIGHT_BYTES:
    default:
        return bb_WORD_SIZE_UNSUPPORTED;
        break;
    }

    return bb_NO_ERROR;
}


