/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: machambe $
 *   DATE:           $Date: 2002/10/05 00:08:38 $
 *   LAST MODIFIED:  $Modtime: 5/03/02 1:38p $
 *   REV#:           $Revision: 1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/generic/genericdriver.h $
 *   WORKFILE:       $Workfile: genericdriver.h $
 *
 * --------------------------------------------------------------------------
 * Project: Generic Driver
 *
 * Purpose: Provides the specfication to the configuration routines.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: genericdriver.h,v $
 *   Revision 1.1  2002/10/05 00:08:38  machambe
 *   no message
 *
 *   Revision 1.1  2002/08/14 23:04:18  machambe
 *   no message
 *
 * 
 * 3     5/03/02 1:39p Srodrig1
 * 
 * 2     4/24/02 10:39a Srodrig1
 * 
 * 1     4/17/02 2:04p Srodrig1
 * Generic chip driver functions
 * 
 * 
 * --------------------------------------------------------------------------
 */ 


#ifndef __GENERIC_DRIVER_H_
#define __GENERIC_DRIVER_H_

/******************************************************************
* Function : GenericGetBuildVersion
*-----------------------------------------------------------------
* Description  : This routine returns the firmware build version of 
*                the Generic driver.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : drvName : Null terminated string describing the driver
*           date : The build date of the driver.
*           buildVer : The version number of the driver.
*           buildRev : The revision number of the driver.
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
GenericDriverGetBuildVersion(bb_ChipData_t* pChipData, char* drvName,
                       char *date, ushort *buildVer, ushort *buildRev);


/****************************************************************************
* Function : GenericRead
*----------------------------------------------------------------------------
* Description  : This routine will read starting at a particular 
*				 location and read for N number of words.
*----------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start reading from.
*				length : Number of words to read.
* Outputs : buffer : The buffer to contain the read data.
*           
*----------------------------------------------------------------------------
* Returns : bb_WORD_SIZE_UNSUPPORTED
*           bb_NO_ERROR
*----------------------------------------------------------------------------
* Access Globals : none
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
extern bb_Error_e 
GenericDriverRead(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
            ulong address, ushort length, void *buffer);

/****************************************************************************
* Function : GenericWrite
*----------------------------------------------------------------------------
* Description  : This routine will write starting at a particular 
*						location and write for N number of words.
*----------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*				wordSize : Identifies the size of a word.
*				address : Location where to start writing to.
*				length : Number of words to write.
*				buffer : The buffer to contain the read data.
* Outputs : None.
*----------------------------------------------------------------------------
* Returns : bb_WORD_SIZE_UNSUPPORTED
*           bb_NO_ERROR
*----------------------------------------------------------------------------
* Access Globals : none
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
extern bb_Error_e 
GenericDriverWrite(bb_ChipData_t* pChipData, bb_Word_Size_t wordSize,
             ulong address, ushort length, void *buffer);


#endif
