/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: machambe $
 *   DATE:           $Date: 2002/10/27 18:21:36 $
 *   LAST MODIFIED:  $Modtime: 5/17/02 1:58p $
 *   REV#:           $Revision: 1.2 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf6048/ixf6048f.h $
 *   WORKFILE:       $Workfile: ixf6048f.h $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6048
 *
 * Purpose:  Define additional functions for OHT (IXF6048) Driver
 * 	         This File also contains Specific Build Flags for the 
 *	         ixf6048 device driver.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: ixf6048f.h,v $
 *   Revision 1.2  2002/10/27 18:21:36  machambe
 *   no message
 *
 *   Revision 1.2  2002/10/14 09:41:20  machambe
 *   no message
 */ 

#ifndef BB6048F_H_ /* Include file contents only once per compilation */
#define BB6048F_H_


/* --------------------------	--------------------------	
	Build Flags
   --------------------------	--------------------------	*/

	/* Define strings for Error Messages */
#if(0)
	#define INC_ERROR_STRINGS
#endif

#if(1)   /* Change this to (1) to output debug messages from the driver */
   #define UNIT_TEST    /* Define this for Unit Test output */
   #include <stdio.h>   /* Get printf and putchar support */

   #if(1)
      #define IGNORE_JN_READBACK_ERRS
                  /* Define this to ignore J0,J1,J2 String readback errors
                     (as part of Unit Testing) */
   #endif
#endif

/* --------------------------	--------------------------	
   Public Function
   --------------------------	--------------------------	*/
typedef void (*ixf6048_pAlarmCallback)   /* Alarm Callback Pointer to Function*/
   (ushort*          BaseAddress,   /* BaseAddress of the Chip */
    bb_AlarmType_e   AlarmType,     /* Type of Alarm */
    void*            pAlarmStatus); /* ixf6048_Oht/Utopia/Atm/PosAlarmStatus_t*/

/***********************************************************************
 * Function to get the Rx J0/J1 Trace
 **********************************************************************/

extern bb_Error_e ixf6048_GetTrace /* Get Rx J0/J1 Trace */
   (bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
    uint             ChanNum,    /* Chan 0-3 */
    bb_TraceType_e   TraceType,  /* Type of Trace: Rx J0/J1 */
    char*            pTrace,     /* -> place to return null-terminated
                                       Rx Trace String.
                                       Note: at least 65 bytes long */
    ushort* pLength);

/***********************************************************************
 * Function to Set an Overhead Byte: K1,K2, Expected/Tx C2, HPT RDI in G1
 **********************************************************************/

extern bb_Error_e ixf6048_SetOhByte /* Set Overhead Byte */
   (bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
    uint             ChanNum,    /* Channel Num = 0-3 */
    bb_SelOhByte_e   SelOhByte,  /* to select a particular Overhead byte */
    ushort            OhByte);    /* value of selected Overhead byte */

/***********************************************************************
 * Function to Get Overhead Bytes: Rx MST, Rx HPT
 **********************************************************************/

extern bb_Error_e ixf6048_GetOhBytes /* Get Overhead Bytes */
   (bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
    uint             ChanNum,    /* Channel Num = 0-3 */
    bb_SelOhBytes_e  SelOhBytes, /* to select the Overhead Bytes */
    void*            pOhBytes);  /* -> place to put Overhead Bytes, of type:
                                       bb_RxMstBytes_t, or
                                       ixf6048_RxHptBytes_t */

/***********************************************************************
 * Function to Reset (chip or channel) and then reconfigure it.
 * To restart traffic, the chip or channel must be set back online,
 * using bb_SetOpMode.
 **********************************************************************/

extern bb_Error_e ixf6048_Reset
   (bb_ChipData_t*       pChipData,  /* -> Initialized Chip Data */
    uint                 ChanNum,   /* Channel Num = 0-3, if applicable */
    ixf6048_ResetType_e  ResetType);  /* Specifies type of Chip/Chan Reset */

/***********************************************************************
 * Function to Get Status: Rx AIS, Input Clock Activity, Far-end GFC Role
 *
 * Note:
 * The test for the far-end GFC Role depends upon having active ATM traffic.
 *
 * The far end is identified as a GFC controller if an ATM cell has been
 * received with a GFC Halt, SET-A, or SET-B command, since this function
 * was last called.
 *
 * The far end is identified as a GFC controlled device if an ATM cell has
 * been received with the GFC Controlled bit set, since this function
 * was last called.
 **********************************************************************/

extern bb_Error_e ixf6048_GetStatus
   (bb_ChipData_t*      pChipData,  /* -> Initialized Chip Data */
    uint                ChanNum,    /* Channel Num = 0-3 */
    ixf6048_SelStatus_e SelStatus,  /* To select type of status to get */
    void*               pStatus);   /* -> place to put Status, of type
                                       ushort (for Rx AIS Status: Reg 0x7), or
                                       ixf6048_InClkStatus_t, or
                                       ixf6048_FarEndGfcRole_e */

/***********************************************************************
 * Function to Set Config: POS Watermarks, POS Flow Control, Chip's GFC Role
 **********************************************************************/

extern bb_Error_e ixf6048_SetCfg
   (bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
    uint             ChanNum,    /* Channel Num = 0-3 */
    bb_6048_SelCfg_e SelCfg,     /* To select type of config to set */
    void*            pCfg);      /* -> configuration structure, of type
                                    ixf6048_PosWatermarks_t, or
                                    ixf6048_TxPosFlowControl_t, or
                                    ixf6048_GfcRole_t */


/***********************************************************************
 * Function to Initialize the Driver
 **********************************************************************/
bb_Error_e ixf6048_InitAlarmCallback /* Init the Amazon-A driver */
   (ixf6048_pAlarmCallback   pAlarmCallback);  /* -> Alarm Callback */

/***********************************************************************
 * Function to Initialize the Chip
 **********************************************************************/
bb_Error_e ixf6048_InitChip   /* Init a Chip */
   (bb_ChipData_t*   pChipData);    /* -> validated Chip Data */

/***********************************************************************
 * Function to Handle Interupts for Specified Amazon Chip
 **********************************************************************/
void ixf6048_ChipIsr        /* Handle interrupts, for specified Amazon chip */
   (bb_ChipData_t*   pChipData);    /* -> validated Chip Data */

/***********************************************************************
 * Function to Get the information for the chip
 **********************************************************************/
bb_Error_e ixf6048_GetChipInfo   /* get Chip info for chip */
   (bb_ChipData_t*   pChipData,    /* -> validated Chip Data */
    ixf6048_ChipInfo_t*  pChipInfo);  /* -> place to return chip info */

/***********************************************************************
 * Function to Retrieve the Channel Mode
 **********************************************************************/
bb_Error_e ixf6048_GetChanOpMode /* Get Chan Op mode */
   		(bb_ChipData_t*   pChipData,    /* -> validated Chip Data */
    	 uint          ChanNum,       /* Channel 0-3 */
    	 ixf6048_ChanMode_e*  pOpMode);      /* -> place to return Chan Mode for chan */
/***********************************************************************
 * Function to run a Test
 **********************************************************************/
bb_Error_e ixf6048_Test  /* Run Test */
   (bb_ChipData_t*   		pChipData,    /* -> validated Chip Data */
    ixf6048_ChanTestCfg_t*  pTestCfg);  /* -> Test cfg -- for Channel */

/***********************************************************************
 * Function to Set Expected/Tx J0/J1 Trace
 **********************************************************************/
bb_Error_e ixf6048_SetTrace  /* Set Expected/Tx J0/J1 Trace */
   (bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
    uint             ChanNum,    /* Channel 0-3 */
    bb_TraceType_e   TraceType,  /* Type of Trace: Expected/Tx J0/J1 */
    bb_TraceFormat_e TraceFormat,/* Specifies the Trace Format */
    char*            pTrace);    /* -> null-terminated Trace String (data only)
                                    Driver adds CRC7, or LF/CR, if needed */

/***********************************************************************
 * Function to Retrieve a set of Counters
 **********************************************************************/
bb_Error_e ixf6048_GetCounters /* Get a set of Rx Counters */
   (bb_ChipData_t*   pChipData,    /* -> validated Chip Data */
    uint             ChanNum,    /* Channel 0-3 */
    bb_SelCounters_e SelCounters,/* to select the set of counters */
    void*            pCounters); /* -> place to put Counters */

/* ------------------ Chip - Global Mode Setting Functions  ------------------ */
/***********************************************************************
 * Put the Chip On-line  -- Enable interupts and Data Bus
 **********************************************************************/
void ixf6048_SetChipOnline (bb_ChipData_t*   pChipData);

/***********************************************************************
 * Put the Chip Off-line -- Disable interupts and Data Bus                                                
 **********************************************************************/
void ixf6048_SetChipOffline(bb_ChipData_t*   pChipData);

/* ------------------ Channel - Mode Setting Functions  ------------------ */
/***********************************************************************
 * Set A Channel On-Line
 **********************************************************************/
bb_Error_e ixf6048_SetChanOnline(bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
								 uint 			  ChanNum);   /* Channel Number */

/***********************************************************************
 * Set a Channel Off-Line
 **********************************************************************/
bb_Error_e ixf6048_SetChanOffline(bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
								 uint 			  ChanNum);   /* Channel Number */

/***********************************************************************
 * Put a channel into Test Mode
 **********************************************************************/
bb_Error_e ixf6048_SetChanTestMode(bb_ChipData_t*   pChipData,  /* -> Initialized Chip Data */
								   uint 			ChanNum);   /* Channel Number */

bb_Error_e ixf6048_SetAlarmCfg(bb_ChipData_t*   pChipData,
                               uint             ChanNum,
                               ixf6048_Side_e   Side,
                               bb_AlarmType_e   AlarmType,
                               void*            pAlarmCfg);

#endif   /* BB6048F_H_ */

