/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: machambe $
 *   DATE:           $Date: 2002/10/05 00:08:39 $
 *   LAST MODIFIED:  $Modtime: 12/03/01 9:55a $
 *   REV#:           $Revision: 1.1 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/os.h $
 *   WORKFILE:       $Workfile: os.h $
 *
 * --------------------------------------------------------------------------
 * Project: ALL
 *
 * Purpose: Provides the specfication for the eiffel alarm, status, & counters
 *          handler routines.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 */ 

#ifndef __OS_H_
#define __OS_H_

/* Task states */
#define TASK_RUNNING 1
#define TASK_SUSPEND 1
#define TASK_READY 1
#define TASK_PENDING 1


#ifdef VXWORKS
/***************************************************************************
 *               Define OS abstraction layer for VxWorks OS                * 
****************************************************************************/
#define OS_CMD_ALLOC_MEM(size) (malloc(size))
#define OS_CMD_DEALLOC_MEM(ptr) (free(ptr))
#define OS_CMD_GET_TASK_ID() (taskIdSelf())
#define OS_CMD_ENABLE_PREEMPTION() (taskUnlock())
#define OS_CMD_DISABLE_PREEMPTION() (taskLock())
#define OS_CMD_CREATE_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) \
		(taskInit(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10))
#define OS_CMD_START_TASK(id) (taskActivate(id))
#define OS_CMD_STOP_TASK(id) (taskSuspend(id))
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) \
		(taskSpawn(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10))

#define OS_CMD_CREATE_SEMAPHORE(opt) (semMCreate(opt))
#define OS_CMD_GIVE_MUTEX_SEMAPHORE(id) (semGive(id))
#define OS_CMD_TAKE_MUTEX_SEMAPHORE(id, timeOut) (semTake(id, timeOut))

#define OS_CMD_ENABLE_INTERRUPTS(irq) (intUnlock(irq))
#define OS_CMD_DISABLE_INTERRUPTS() (intLock())
#define OS_CMD_VERIFY_TASK_ID(tId)  (taskIdVerify(tId))

/* defines */ 
#define MAX_MSG_Q_MSGS (10) 
#define MSG_Q_PRI 1
 
#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) (msgQCreate (maxMsg, maxMsgLen, priority))  
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) (msgQSend (id, msg, size, flag,priority))
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	(msgQReceive(id, msg, size, flag))

#define OS_CMD_TASK_DELETE(tId) (taskDelete(tId))

#define OS_CMD_CREATE_PIPE(name, num, size) (pipeDevCreate(name, num, size))
#define OS_CMD_OPEN(name, flag, mode)  (open(name, flag, mode))
#define OS_CMD_READ(fd, buffer, size)  (read(fd, buffer, size))
#define OS_CMD_WRITE(fd, buffer, size) (write(fd, buffer, size))
#define OS_CMD_CLOSE(fd)               (close(fd))

#define OS_CMD_GET_TASK_STATE(tId, state) \
{ \
   TASK_DESC tDesc; \
   taskInfoGet(tId, &tDesc); \
   (tDesc.state) ? state = 1 : state = 0; \
}

/* Global variable for Msg Q's */
extern MSG_Q_ID ixfMsgQId;



#else 
/***************************************************************************
 *     Define OS abstraction layer for when no embedded OS is defined      * 
****************************************************************************/

#define OS_CMD_ALLOC_MEM(size) (malloc(size))
#define OS_CMD_DEALLOC_MEM(ptr) (free((void*)ptr))
#define OS_CMD_GET_TASK_ID()
#define OS_CMD_ENABLE_PREEMPTION() 
#define OS_CMD_DISABLE_PREEMPTION()
#define OS_CMD_CREATE_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) \
                         (arg2 * arg3)
#define OS_CMD_START_TASK(id) (id = id)
#define OS_CMD_STOP_TASK(id) (id=id)
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) ()

#define OS_CMD_CREATE_SEMAPHORE()
#define OS_CMD_GIVE_MUTEX_SEMAPHORE()
#define OS_CMD_TAKE_MUTEX_SEMAPHORE()

#define OS_CMD_ENABLE_PREEMPTION() 
#define OS_CMD_DISABLE_PREEMPTION()

#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) ()
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) ()
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	()
#define OS_CMD_VERIFY_TASK_ID(tId)  ()
#define OS_CMD_TASK_DELETE(tId) ()

#define OS_CMD_CREATE_PIPE(name, num, size) ()
#define OS_CMD_OPEN(name, flag, mode)  ()
#define OS_CMD_READ(fd, buffer, size)  ()
#define OS_CMD_WRITE(fd, buffer, size) ()
#define OS_CMD_CLOSE(fd)               ()
#define OS_CMD_GET_TASK_STATE(tId, state) (state = 1)
#endif


#endif /* __OS_H_ */
