/*********************************************************************************
    Programmer : M.Todd, R.Slimm
	Filename: ediag_error.c
 
	Description : TBD

	History

	Synopsis											Date		Issue	Init

	Routine released									12/30/99	R001	MACT


		 
**********************************************************************************/

#include "vm.h"
#include "esttype.h"
#include "cmd.h"
#include "lib.h"
#include "vderror.h"
#ifdef FFS_ON
#include "ffs.h"
#endif

LOCAL ERROR_CODE TestError = ERR_NONE;	

/* Prototype */
UINT32 LogToFFSFile(char *filename, char *msg, UINT32 operation);

#ifndef MINAGENT
/***************************************************************************
* Function:	LogError()
*
* Purpose:	log an error
*
* Notes:	
****************************************************************************/
void LogError (ERROR_CODE error, ERROR_ACTION err_action, char *msg, ...)
{
#ifndef MINAGENT
	va_list	ap;							        /* varible argument pointers */
	char msgstring[MAX_ERR_MSG_LENGTH];

	if ( strlen(msg) > MAX_ERR_MSG_LENGTH )     /* Check msg length is not illegal */
	  return;

    if( (err_action | LOG_TO_CONSOLE)   || 
        (err_action | LOG_TO_FILE)      ||
        (err_action | LOG_TO_NVRAM)      )
    {
        va_start( ap, msg );			        /* setup variable argument pointer */	

        vsprintf(msgstring, msg, ap);           /* print msg string to buffer */ 

        if( err_action | LOG_TO_CONSOLE )
        {
            if ( TestError == error )			/* remind user if this a 'Forced Error' */ 
	            Info("\n! FORCED ERROR !\n");

            Info("\nErrorCode: %d\n", error);	/* Info error code */
            Info("%s", msgstring);	        /* Info remaining parameters  */
        }
	    
        if( err_action | LOG_TO_FILE )
	        LogErrorFFS(error, msgstring);
    
        if( err_action | LOG_TO_NVRAM )
            LogErrorNV(error, msgstring);
 
	    va_end(ap);			
    }
#endif
	return;
}

/***************************************************************************
* Function:	LogErrorFFS()
*
* Purpose:	Log the error to the FFS Device
*           
* Notes:	
****************************************************************************/

STATUS LogErrorFFS(ERROR_CODE error, char *msg)
{
    STATUS status = ERROR_BASE;

    LogToFFSFile(ERROR_FILENAME, msg, ADD_MSG_TO_FILE);
    
    /* TBD printing the error code */

    return status;
}

/***************************************************************************
* Function:	LogErrorNV()
*
* Purpose:	Log the error to the NVRAM Device
*           
* Notes:	
****************************************************************************/

STATUS LogErrorNV(ERROR_CODE error, char *msg)
{
    STATUS status = ERROR_BASE;

    /* TBD */

    return status;
}

/***************************************************************************
* Function:	ForcedError()
*
* Purpose:	See if an error has been forced. Typically used to test error
*           messages and handling.
* Notes:	
****************************************************************************/
BOOLEAN ForcedError (ERROR_CODE error)
{
  return (TestError == error) ? TRUE : FALSE;
}


/***************************************************************************
* Function:	ForceError()
*
* Purpose:	Force global 'TestError' variable to a specific error code.
*
* Notes:
****************************************************************************/
void ForceError (ERROR_CODE error)
{
  TestError = error;
}

/***************************************************************************
* Function:	ForceError()
*
* Purpose:	Force global 'TestError' variable to a specific error code.
*
* Notes:
****************************************************************************/
UINT32 LogToFFSFile(char *filename, char *msg, UINT32 operation)
{
	UINT32  BytesWritten = 0;
    char    time[20];

#ifdef FFS_ON
    FFSHANDLE *pHandle;

    if( operation | ADD_MSG_TO_FILE)
    {
	    if (!(pHandle = OpenFile(filename)))
        {
            pHandle = CreateFile(filename, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF);
        }
        
        sprintf(time,"0x%08X  ",GetLocalTimestamp());
        WriteFile(pHandle, time, strlen(time), &BytesWritten);
        WriteFile(pHandle, msg, strlen(msg), &BytesWritten);
        
	    CloseFile(pHandle);
    }
#endif

	return (BytesWritten);
}
#else

void LogError (ERROR_CODE error, ERROR_ACTION err_action, char *msg, ...)
{
    // Do something for MINAGENT??
	return;
}
#endif // MINAGENT