/***************************************************************************
*
* Module: vdError.h
*
* Description: Error Handling Defines
*
* Project:     EST VisionWare -- vWare
*
* Functions: 
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/
#ifndef __ERRORH__
#define __ERRORH__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _ASMCODE_			/* Nothing above this point */

#define ERROR_FILENAME      "error.txt"
#define ADD_MSG_TO_FILE     0x0001              /* FFS operations */

#define  CONT_ON_ERROR      0x00000001			/* ERROR_ACTIONs */
#define  HALT_ON_ERROR      0x00000002
#define  IGNOR_ERROR  	    0x00000004
#define  STATUS_NERROR 	    0x00000008

#define  LOG_TO_FILE	    0x00000010			/* FILE_ACTIONS */
#define  LOG_TO_NVRAM       0x00000020
#define  LOG_TO_CONSOLE     0x00000040

typedef  STATUS  ERROR_CODE;
typedef  STATUS  STATUS_CODE;
typedef  UINT32  ERROR_ACTION;
typedef  UINT32  NVRAM_ACTION;


/* Error Code Bases */

#define ERROR_BASE  	0

/* Diagnostic Error Codes */

#define ERR_NONE			0
#define ERR_FAKE_ERROR 		(ERROR_BASE + 0) 
#define ERR_MEMORY_TEST_1   (ERROR_BASE + 1)
#define ERR_MEMORY_TEST_2   (ERROR_BASE + 2)
#define ERR_MEMORY_TEST_3   (ERROR_BASE + 3)
#define ERR_MEMORY_TEST_4	(ERROR_BASE + 4)
#define ERR_CRC_TEST		(ERROR_BASE + 5)

#define ERR_PCMCIA_TEST		(ERROR_BASE + 6)
#define ERR_MEMORY_TEST_5   (ERROR_BASE + 7)
#define ERR_PCI				(ERROR_BASE + 8)
#define ERR_OVERLAY_MEM		(ERROR_BASE + 9)
#define ERR_860_AUTO		(ERROR_BASE + 10)
#define ERR_I2C				(ERROR_BASE + 11)
#define ERR_EEPROM			(ERROR_BASE + 12)
#define ERR_FLASH			(ERROR_BASE + 13)
#define ERR_MEMORY_TEST_7   (ERROR_BASE + 14)
#define ERR_MEMCOPY			(ERROR_BASE + 15)
#define ERR_MEMSCAN1		(ERROR_BASE + 16)
#define ERR_MEMSCAN2		(ERROR_BASE + 17)
#define ERR_MEMFILL			(ERROR_BASE + 18)
#define ERR_MEMBLKCPY		(ERROR_BASE + 19)
#define ERR_MEMBLKCMP		(ERROR_BASE + 20)

/*  Diagnostic Error Messages  */

#define MSG_NONE			NULL

#define MAX_ERR_MSG_LENGTH	256	/* Limit size of message */
#define MAX_ERR_MSG_PARMS    5	/* Limit number of parameters */

#ifdef  BUILD_WITH_ERR_MSGS		/* Define/Undefined  */

#define MSG_FAKETEST_FAIL   "Error: FakeTest has failed"

#else	/* BUILD_WITH_ERR_MSGS */

#define MSG_FAKETEST_FAIL   MSG_NONE

#endif	/* BUILD_WITH_ERR_MSGS */

extern void    LogError  ( ERROR_CODE  error, ERROR_ACTION err_action, char *msg, ... );
extern void    ForceError (ERROR_CODE error);
extern BOOLEAN ForcedError (ERROR_CODE error);
extern STATUS  LogErrorFFS(ERROR_CODE error, char *msg);
extern STATUS  LogErrorNV(ERROR_CODE error, char *msg);

#endif	/* _ASMCODE_ */		/* Nothing below this point */
							
#ifdef __cplusplus
}
#endif

#endif /* __ERRORH__ */

