/***************************************************************************
*
* Module:      vdgDiagExec.c       
*
* Description: vWare Diagnostics Executive Entry  
*
* Project:     vWare
*
* Functions:   vdiag()
*
* Notes:        
*
* History:		See Visual SourceSafe History
*
* Disclaimer:    Copyright (c) 1995, 2000, EST Corporation
*    This program is confidential and proprietary to EST Corporation
*  and, without the express prior written permission of an officer of
*  EST Corporation, may not be copied, reproduced, disclosed to others
*  published  or  used, in  whole or in part, for any purpose other than
*  that for which it is being  made available. This program is protected
*  pursuant to applicable copyright laws, including the U.S. Copyright Act.
*  This program is to be returned to EST Corporation upon the earlier of
*  demand and the accomplishment of the purpose for which it was provided.
****************************************************************************/

// Includes
#include "vm.h"
#include "cmd.h"
#include "shell.h"
#include "vdexec.h"
#ifdef FFS_ON
#include "ffs.h"
#endif

// Local Defines
#define DEBUG_VDIAG             

// External References
IMPORT void HelpDiags(char *pIn, void *pOut);

IMPORT UINT8    CoreState;

// Forward References
void vdgHelpCmdScripts(void);
void vdgConfigErrorLogging(void);
void vdgConfigHaltOnError(void);
void vdgViewVdiagConfOptions(void);
void vdgHelpTestScripts(void);
void PrintPrompt(void);
void vdgPrintMenu(void);
void QueryCommand(char *lpszCommand,char *lpszMessage);
void EditScriptFile(void);
void SetPrompt(char *lpszPrompt);
void vdgConfigVerboseMode(void);
void vdgRunVdiagCmdScript(void);
void vdgViewVdiagCmdScript(void);



// Global Declarations
char m_szPrompt[64];

// Local Declarations (statics)
static STATUS  vwConsoleStatus( UINT8 );
static STATUS  vwFileSystemStatus( UINT8);

// Error Logging Flags -- Global Flags manipulated through access functions 
static BOOL bErrorLogging = ENABLE;  
static BOOL bConsoleErrorLogging = ENABLE;
static BOOL bFileErrorLogging = ENABLE;
static BOOL bVerboseMode = ENABLE;
static BOOL bHaltOnError = ENABLE;


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void vdiag(void *pIn, void *pOut)
{
   char        *szHelpStr;
   char        szBuffer[256];
   char          str[64];
   int           nCurCmdNum;
   int            nDiagID;
   int            nCurCommand = -1;
   tStreamHeader  *pCommandStream;

   // Set Core running vWARE permission
   CoreState = 0;

   // Prompt
   SetPrompt("VDIAG");
   PrintPrompt();

   // Printout menu
   vdgPrintMenu();

   // Loop until done configuring
   while ( nCurCommand != ENDLOOP )
   {

      // get selection through generic prompt
      QueryCommand(szBuffer,NULL);

      if ( !strncmp(szBuffer,"help",4) )
      {
         szHelpStr = szBuffer + 5;
         HelpDiags(szHelpStr, NULL);
      }
      else if ( !strcmp(szBuffer,"q") )
      {
         // Quit
         nCurCommand = ENDLOOP;
      }
      else
      {
         nCurCmdNum = atoi(szBuffer);

         switch ( nCurCmdNum )
         {
            /***********************************
             * Display Menu for unknown commands
             ***********************************/
            default:
               // Printout menu
               vdgPrintMenu();
               break;

               /*************************
               * Display Diags Test List
               **************************/
            case 1:
               // General help for all diag functions
               HelpDiags(NULL, NULL);

               // Help on Test Scripts
               vdgHelpTestScripts();
               break;

               /*************************
               * Execute Diagnostic Test
               **************************/
            case 2:
               // Change prompt to show new state
               SetPrompt("ENTER CMD");

               // Get next command from user
               QueryCommand(szBuffer,NULL);

               // Pass user command to vWARE to execute
               ParseCommand(szBuffer);

               // Change prompt back to orig state
               SetPrompt("VDIAG");
               break;

               /**************************
               * Configure Logging Options
               ***************************/
            case 3:
               vdgConfigErrorLogging();

               // Change prompt back to orig state
               SetPrompt("VDIAG");
               break;

               /**************************
               * Configure Verbose Mode
               ***************************/
            case 4:
               vdgConfigVerboseMode();

               // Change prompt back to orig state
               SetPrompt("VDIAG");
               break;


               /**************************
               * Configure On Error Action
               ***************************/
            case 5:
               vdgConfigHaltOnError();
               
               // Change prompt back to orig state
               SetPrompt("VDIAG");
               break;

               /**************************
               * View Configuration Options
               ***************************/
            case 6:
               vdgViewVdiagConfOptions();
               
               // Change prompt back to orig state
               SetPrompt("VDIAG");
               break;

               /*****************************
                * Run Diagnostic Command File
                *****************************/
            case 7:
// mt               vdgRunVdiagCmdScript();
               
               // Change prompt back to orig state
 // mt              SetPrompt("VDIAG");
               break;               
 
               /******************************
               * View Diagnostic Command File
               *******************************/
            case 8:
// mt               vdgViewVdiagCmdScript();
               
               // Change prompt back to orig state
 // mt              SetPrompt("VDIAG");
               break;  

               /************************************
               * Edit/Create Diagnostic Command File
               *************************************/
            case 9:
 // mt              EditScriptFile();
 // mt              Info("\nFinished\n");
               break;   

#if 0
               Info("\nSaving vDIAG Script Routine........\n");

               // Write script out to file
               WriteScriptFile();

               // Show dir so user can see file
               ParseCommand("dir");

               Info("\nFinished ... vdiag.reg\n");
               break;
#endif


               /*************************
               * View Error Log File
               **************************/
            case 10:
               Info("******** Error Log **********\n\n");

               sprintf(szBuffer,"type %s",ERROR_FILE);

               // Issue the type command to the file
               ParseCommand(szBuffer);
               break;

               /*************************
               * Clear Error File
               **************************/
            case 11:
               Info("\nClearing Error Log........\n");
#ifdef FFS_ON
               // Always delete exisiting
               DeleteFile(ERROR_FILE);
#endif
               Info("\nFinished\n");
               break;
         }
      } // end else
   }
}


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void QueryCommand(char *lpszCommand,char *lpszMessage)
{
   int         nBufSize = 0;
   char     ch;
   char     szBuffer[256];
   int         bDone = 0;

   if ( !lpszCommand )
      return;

   // clear the buffer
   memset(szBuffer,0,256);

   if ( !lpszMessage )
      PrintPrompt();
   else
      ResetCommandLine(lpszMessage);

   while ( !bDone )
   {
      char c = getchar();

      switch ( c )
      {
         // ESC
         case 0x1B:
            sprintf(szBuffer," ");
            bDone = 1;
            break;

            // linefeed
         case 0x0A:
            c = 0x0D;   // change linefeed to carriage return
            // falls through to case 0x0D

            // carriage return
         case 0x0D:
            Info("\n");
            bDone =  1;       
            break;

            // backspace
         case 0x08:
            nBufSize = strlen(szBuffer);
            szBuffer[nBufSize-1] = 0;
            Info("%c",'\b'); //,BackspaceString);
            break;

         default:
            putchar(c);
            sprintf(szBuffer,"%s%c",szBuffer,c);
            break;
      }
   }

   sprintf(lpszCommand,"%s",szBuffer);
}


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void SetPrompt(char *lpszPrompt)
{
   if ( !lpszPrompt )
      return;

   sprintf(m_szPrompt,"%s",lpszPrompt);
}


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void EditScriptFile(void)
{
#if 0
   UINT32  BytesRead = 0;
   UINT8   *pData;

#ifdef FFS_ON
   FFSHANDLE *pHandle;

   // Open file if not there Ram struct is empty
   if ( (pHandle = OpenFile(DEFAULT_CMD_FILE)) )
   {
      // Read file into Ram struct to be edited
      // ReadFile(HANDLE,FILEPTR,UINT8*DATA,DATALEN,*BYTESREAD)
      ReadFile(pHandle, 0,(INT8 *)pData, sizeof(pData), &BytesRead);
      if ( BytesRead )
      {
      }
   }
   // Close file   
   CloseFile(pHandle);

   // Edit Ram Stuff


#endif
#endif
}


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void WriteScriptFile()
{
   UINT32  BytesWritten = 0;
   char    TempCommand[20];

#ifdef FFS_ON
   FFSHANDLE *pHandle;

   // Always delete exisiting
   DeleteFile(DEFAULT_CMD_FILE);

   // Open file or Create if not already there
   if ( !(pHandle = OpenFile(DEFAULT_CMD_FILE)) )
   {
      pHandle = CreateFile(DEFAULT_CMD_FILE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF);
   }

   // Put a couple of example command in the file   
   strcpy(TempCommand,"mt1 a00000 a00100 n\r\n");
   WriteFile(pHandle, TempCommand, strlen(TempCommand), &BytesWritten);

   // Do one that causes an error when run
   strcpy(TempCommand,"mt1 a00100 a00000 n\r\n");
   WriteFile(pHandle, TempCommand, strlen(TempCommand), &BytesWritten);

   // Close file   
   CloseFile(pHandle);
#endif
}
/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void vdgHelpCmdScripts(void)
{
   // Display a list of Diagnostic Command Scripts     

   // TOBEDONE

}

/***************************************************************************
* Function: vdConfigErrorLogging()
*
* Purpose:	Prompts user for vDiags Error Logging Configuration 
*
* Returns:	None
****************************************************************************/
void vdgConfigErrorLogging(void)
{
   char  szBuffer[25];

   // Change prompt to show new state
   SetPrompt("Error Logging ? ([Y]/N): ");

   // Get Y/N response
   QueryCommand(szBuffer,NULL);

   switch (toupper(*szBuffer))
   {
      default:
         Info("\nInvalid Response (defaulting to 'Y'): \n");
      case 'Y':
      case CR:
      case LF:
      case LTERM:
         vdgErrorLogging(ENABLE);
         break;
      case 'N':
         vdgErrorLogging(DISABLE);
         return;
   }

   // Log Errors to the console ?
   if ( vwConsoleStatus(VW_CONSOLE_AVAIL_FOR_LOGGING) )
   {
      SetPrompt("Log Errors To Console ? ([Y]/N): ");
      QueryCommand(szBuffer,NULL);

      switch (toupper(*szBuffer))
      {
         default:
            Info("\nInvalid Response (defaulting to 'Y'): \n");
         case 'Y':
         case CR:
         case LF:
         case LTERM:
            vdgConsoleErrorLogging(ENABLE);
            break;
         case 'N':
            vdgConsoleErrorLogging(DISABLE);
            return;
      }
   }

   // Log Errors to a file ?
   if ( vwFileSystemStatus(VW_AVAIL_FOR_LOGGING) )
   {
      SetPrompt("Log Errors To a File ? ([Y]/N): ");
      QueryCommand(szBuffer,NULL);

      switch (toupper(*szBuffer))
      {
         default:
            Info("\nInvalid Response (defaulting to 'Y'): \n");
         case 'Y':
         case CR:
         case LF:
         case LTERM:
            vdgFileErrorLogging(ENABLE);

            SetPrompt("Error File [ErrorLog.txt]: ");
            QueryCommand(szBuffer,NULL);

#ifdef DEBUG_VDIAG
            Info("\nToBeDone\n");
#endif
            // Check filename validity and create file (overwrite warning)

            // TOBEDONE

            break;
         case 'N':
            vdgFileErrorLogging(DISABLE);
            return;
      }
   }
   return;
}

/***************************************************************************
* Function: vdgConfigVerboseMode()
*
* Purpose:   Configure the verbose mode    
*
* Returns:
****************************************************************************/
void vdgConfigVerboseMode(void)
{
   char  szBuffer[25];

   // Change prompt to show new state
   SetPrompt("Verbose On ? ([Y]/N): ");

   // Get Y/N response
   QueryCommand(szBuffer,NULL);

   switch (toupper(*szBuffer))
   {
      default:
         Info("\nInvalid Response (defaulting to 'Y'): \n");
      case 'Y':
      case CR:
      case LF:
      case LTERM:
         vdgVerboseMode(ENABLE);
         break;
      case 'N':
         vdgVerboseMode(DISABLE);
         return;
   }

}

/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void vdgConfigHaltOnError(void)
{
   char  szBuffer[25];

   // Configure the Error Response Action: HaltOnError, !HaltOnError     

   // Change prompt to show new state
   SetPrompt("Halt On Error ? ([Y]/N): ");

   // Get Y/N response
   QueryCommand(szBuffer,NULL);

   switch (toupper(*szBuffer))
   {
      default:
         Info("\nInvalid Response (defaulting to 'Y'): \n");
      case 'Y':
      case CR:
      case LF:
      case LTERM:
         vdgHaltOnError(ENABLE);
         break;
      case 'N':
         vdgHaltOnError(DISABLE);
         return;
   }
}


/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
void vdgViewVdiagConfOptions(void)
{
   char enableMsg[8];
   char disableMsg[9];
   char *pMsg;

   strcpy( enableMsg, "ENABLED" );
   strcpy( disableMsg, "DISABLED" );

   // View Configuration Options     

   
   // Error Logging

   if (vdgIsErrorLogging(ENABLE))
   {
      Info("\nError Logging: %s", enableMsg);

      // Console Logging ?
      pMsg =  (vdgIsConsoleErrorLogging(ENABLE)) ? enableMsg : disableMsg;
      Info("\n\tTo Console: %s", pMsg);

      // File Logging ?
      pMsg =  (vdgIsFileErrorLogging(ENABLE)) ? enableMsg : disableMsg;
      Info("\n\tTo File: %s", pMsg);

   }
   else
      Info("\nError Logging: %s", disableMsg);

   // Verbose Enable/Disable ?

   pMsg =  (vdgIsVerboseMode(ENABLE)) ? enableMsg : disableMsg;
   Info("\nVerbose Mode: %s", pMsg);

   // HaltOnError Enable/Disable ?

   pMsg =  (vdgIsHaltOnError(ENABLE)) ? enableMsg : disableMsg;
   Info("\nHalt On Error: %s", pMsg);
}

/***************************************************************************
* Function: vdgHelpTestScripts()
*
* Purpose:  Displays help menu for 'test scripts'
*
* Returns:  None
****************************************************************************/
void vdgHelpTestScripts( void )
{
   Info("\n========= TEST COMMAND SCRIPTS ============\n\n");

   // querry file system for tcs files


   return;
}

/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
static STATUS vwConsoleStatus( UINT8 request )
{
   return(TRUE);
}

/***************************************************************************
* Function:
*
* Purpose:
*
* Returns:
****************************************************************************/
static STATUS vwFileSystemStatus( UINT8 request )
{
   return(TRUE);
}


/***************************************************************************
* Function: vdgIsErrorLogging( ENABLE/DISABLE )
*
* Purpose:  Check global Error Logging variable bErrorLogging.
*
* Returns:  TRUE/FALSE
****************************************************************************/
BOOL vdgIsErrorLogging(BOOL state)
{
   return(state == bErrorLogging) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgErrorLogging(ENABLE/DISABLE)
*
* Purpose:  Enable (TRUE) or Disable (FALSE) global error logging flag.
*
* Returns:  TRUE/FALSE
****************************************************************************/
void vdgErrorLogging(BOOL enableErrorLogging)
{
   if (enableErrorLogging == DISABLE)
   {
      bErrorLogging = FALSE;
      vdgConsoleErrorLogging(DISABLE);
      vdgFileErrorLogging(DISABLE);
   }
   else
      bErrorLogging = TRUE;
}


/***************************************************************************
* Function: vdgIsConsoleErrorLogging( ENABLE/DISABLE )
*
* Purpose:  Check global Console Error Logging variable bConsoleErrorLogging.
*
* Returns:  TRUE/FALSE
****************************************************************************/
BOOL vdgIsConsoleErrorLogging(BOOL state)
{
#ifdef DEBUG_VDIAG
   if (bConsoleErrorLogging && vdgIsErrorLogging(DISABLE))
   {
      Info("\nDEBUG WARNING: Inconsistant bConsoleErrorLogging & bErrorLogging flags\n");
   }
#endif

   return(state == bConsoleErrorLogging) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgConsoleErrorLogging()
*
* Purpose:  Enable (TRUE) or Disable (FALSE) global console error loggging flag.
*
* Returns:  TRUE/FALSE
****************************************************************************/
void vdgConsoleErrorLogging(BOOL enableErrorLogging)
{
#ifdef DEBUG_VDIAG
   if (enableErrorLogging && vdgIsErrorLogging(DISABLE))
   {
      Info("\nDEBUG WARNING: Inconsistant bConsoleErrorLogging & bErrorLogging flags\n");
   }
#endif

   bConsoleErrorLogging = (enableErrorLogging == ENABLE) ? TRUE : FALSE;
}


/***************************************************************************
* Function: vdgIsFileErrorLogging( ENABLE/DISABLE )
*
* Purpose:  Check global Console Error Logging variable bFileErrorLogging.
*
* Returns:  TRUE/FALSE
****************************************************************************/
BOOL vdgIsFileErrorLogging(BOOL state)
{
#ifdef DEBUG_VDIAG
   if (bFileErrorLogging && vdgIsErrorLogging(DISABLE))
   {
      Info("\nDEBUG WARNING: Inconsistant bConsoleErrorLogging & bErrorLogging flags\n");
   }
#endif

   return(state == bFileErrorLogging) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgFileErrorLogging()
*
* Purpose:  Enable (TRUE) or Disable (FALSE) global file error loggging flag.
*
* Returns:  TRUE/FALSE
****************************************************************************/
void vdgFileErrorLogging(BOOL enableErrorLogging)
{
#ifdef DEBUG_VDIAG
   if (enableErrorLogging && vdgIsErrorLogging(DISABLE))
   {
      Info("\nDEBUG WARNING: Inconsistant bFileErrorLogging & bErrorLogging flags\n");
   }
#endif

   bFileErrorLogging = (enableErrorLogging == ENABLE) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgVerboseMode() 
*
* Purpose:  Enable (TRUE) or Disable (FALSE) global verbose flag.
*
* Returns:  TRUE/FALSE
****************************************************************************/
void vdgVerboseMode(BOOL enableVerboseMode)
{
   bVerboseMode = (enableVerboseMode == ENABLE) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgIsVerboseMode( ENABLE/DISABLE )
*
* Purpose:  Check global Verbose Flag for ENABLE/DISABLE
*
* Returns:  TRUE/FALSE
****************************************************************************/
BOOL vdgIsVerboseMode(BOOL state)
{
   return(state == bVerboseMode) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgHaltOnError() 
*
* Purpose:  Enable (TRUE) or Disable (FALSE) global HaltOnError flag.
*
* Returns:  TRUE/FALSE
****************************************************************************/
void vdgHaltOnError(BOOL enableHaltOnError)
{
   bHaltOnError = (enableHaltOnError == ENABLE) ? TRUE : FALSE;
}

/***************************************************************************
* Function: vdgIsHaltOnError( ENABLE/DISABLE )
*
* Purpose:  Check global HaltOnError Flag for ENABLE/DISABLE
*
* Returns:  TRUE/FALSE
****************************************************************************/
BOOL vdgIsHaltOnError(BOOL state)
{
   return(state == bHaltOnError) ? TRUE : FALSE;
}


/***************************************************************************
* Function: PrintPrompt()
*
* Purpose:  print out prompt
*
* Returns:
****************************************************************************/
void PrintPrompt()
{
   ResetCommandLine(m_szPrompt);
}


/***************************************************************************
* Function: vdgPrintMemu()
*
* Purpose:  Print Diagnostics Menu
*
* Returns:  None
****************************************************************************/
void vdgPrintMenu()
{
   printf("\nSelect Option:\n\n");
   printf("\t(q) Quit\n");
   printf("\t(help <test>) help on test\n");

   printf("\t(1)  List Tests\n");
   printf("\t(2)  Execute Test\n");

   printf("\t(3)  Configure Logging Options\n");
   printf("\t(4)  Configure Verbose Mode\n");
   printf("\t(5)  Configure On-Error Actions\n");  
   printf("\t(6)  View diagnostic configuration\n");

   printf("\t(7)  Run Diagnostic Command File\n");
   printf("\t(8)  View Diagnostic Command File\n");
   printf("\t(9)  Edit Diagnostic Command File\n");

   printf("\t(10) View Error Log\n");
   printf("\t(11) Clear Error Log\n");
}


/***************************************************************************
* Function: vdgRunVdiagCmdScript()
*
* Purpose:  Executes the prompted command script
*
* Returns:  None
****************************************************************************/
void vdgRunVdiagCmdScript(void)
{
#ifdef FFS_ON
   char  szBuffer1[25];
   FFSHANDLE *pHandle;
   
   // Change prompt to show new state
   SetPrompt("vDiag Command Script to run ? \n");
   QueryCommand(szBuffer1,NULL);

   // Check for command file

   if ((pHandle = OpenFile(szBuffer1)) )
   {
      char  szBuffer2[25];

      Info("\nStarting vDIAG Script Routine........\n");

      sprintf(szBuffer2,"load %s",szBuffer1);
      ParseCommand(szBuffer2);
      Info("\nFinished\n");
   }
   else
      Info("\nCommand File not found\n");
#endif
   return;
}

               
/***************************************************************************
* Function: vdgViewVdiagCmdScript()
*
* Purpose:  Prints the prompted command script
*
* Returns:  None
****************************************************************************/
void vdgViewVdiagCmdScript(void)
{
#ifdef FFS_ON
   char  szBuffer[25];
   FFSHANDLE *pHandle;
   
   // Change prompt to show new state
   SetPrompt("vDiag Command Script to display ? \n");
   QueryCommand(szBuffer,NULL);

   // Check for command file
   
   if ((pHandle = OpenFile(szBuffer)) )
   {
      Info("********** Script Log **********\n\n");
      sprintf(szBuffer,"type %s",szBuffer);
      ParseCommand(szBuffer);
   }
   else
      Info("\nCommand File not found\n");
#endif
}


/***************************************************************************
*                       Install Command into vWare
****************************************************************************/
//Command(CMD_VDIAG, "vdiag", vdiag, 0, "Diagnostic Executive", "");
//CommandEnd();