/* vdlog.c - Logging */

/* Copyright 1984-1999 Wind River Systems, Inc. */

/*
modification history
--------------------
01a,25jan02, pgp  created
01b,06feb02, pgp  Removed Function name from log
*/

/*
DESCRIPTION

VisionWARE Logging

INCLUDE FILES:  
*/

/* includes */

#include "vm.h"
#include "cmd.h"
#include "cmds.h"
#include "vdlog.h"
#ifdef FFS_ON
#include "ffs.h"
#endif

#if (LOG_ENBL==ON)

/* defines */

/* typedefs */

/* globals */

/* locals */

INT32   	logIdx = 0;				/* Log Index */
INT32   	logCnt = 0;				/* Log Entry Counter */
LOG	        pLog[LOG_ENTRIES];	/* Log */

/* forward declartions */


typedef struct
	{
	INT32 Start;
	INT32 End;
	} vdDisplayLogLong_in;

/******************************************************************************
*
* vdDisplayLogLong - Display Log
*
* Displays the log in Long format
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdDisplayLogLong 
	(
	vdDisplayLogLong_in	*pIn    /* Start and End location for log */
	)
	{

	INT32	idx;            /* index */
	INT32	cnt;            /* counter */
	INT32	displayCnt;     /* number to display */

	printf ("\n");

	/* Set the starting location */
	if (logIdx+1 >= LOG_ENTRIES)
		idx = 0;
	else
		idx = logIdx + 1;

	/* Loop through all entries in log */
	for (cnt=0, displayCnt=1; 
		 cnt < LOG_ENTRIES; 
		 cnt++)
		{

		/* Make sure the entry is not NULL */
		if (pLog[idx].idx != NULL)
			{

			/* Check to see if we need to display this entry */
 			if (displayCnt >= pIn->Start && displayCnt <= pIn->End)
				{
				/* Print the log entry */
				printf ("\nIndex : %d", pLog[idx].idx);
				printf ("\nFile Name : %s:%d", pLog[idx].fileName, pLog[idx].line);
				printf ("\nParam A : 0x%x", pLog[idx].param_a);
				printf ("\nParam B : 0x%x", pLog[idx].param_b);
				printf ("\nLog Entry : %s", pLog[idx].log);
				printf ("\n");
				printf ("\n--------------------------------------------------");
				}

			displayCnt++;

			}

			/* update the index */
			if (++idx >= LOG_ENTRIES)
				idx = 0;
		}

	printf ("\n");

	return;

	}
Command (CMD_READ_LOG_LONG,"dl",vdDisplayLogLong,0,"Display Log","Display the log to console in long format");
OptArg (UINT32ARG, "Start location to display log from", "Start location to display log from",(-1));
OptArg (UINT32ARG, "End location to display log to", "End location to display log to",(0x3FFFFFFF));
CommandEnd ();


typedef struct
	{
	INT32 Start;
	INT32 End;
	} vdDisplayLogShort_in;

/******************************************************************************
*
* vdDisplayLogShort - Display Log
*
* Displays the log in Short Format
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdDisplayLogShort 
	(
	vdDisplayLogShort_in	*pIn    /* Start and End location for log */
	)
	{

	INT32	idx;            /* index */
	INT32	cnt;            /* counter */
	INT32	displayCnt;     /* number to display */

	printf ("\n");

	/* Set the starting location */
	if (logIdx+1 >= LOG_ENTRIES)
		idx = 0;
	else
		idx = logIdx + 1;

	/* Loop through all entries in log */
	for (cnt=0, displayCnt=1; 
		 cnt < LOG_ENTRIES; 
		 cnt++)
		{

		/* Make sure the entry is not NULL */
		if (pLog[idx].idx != NULL)
			{

			/* Check to see if we need to display this entry */
 			if (displayCnt >= pIn->Start && displayCnt <= pIn->End)
				{
				/* Print the log entry - short format */
				printf ("\n%d : %s", pLog[idx].idx, pLog[idx].log);
				}

			displayCnt++;

			}

			/* update the index */
			if (++idx >= LOG_ENTRIES)
				idx = 0;
		}

	printf ("\n");

	return;

	}
Command (CMD_READ_LOG_SHORT,"dls",vdDisplayLogShort,0,"Display Log","Display the log to console in short format");
OptArg (UINT32ARG, "Start location to display log from", "Start location to display log from",(-1));
OptArg (UINT32ARG, "End location to display log to", "End location to display log to",(0x3FFFFFFF));
CommandEnd ();
				 

#ifdef FFS_ON

/******************************************************************************
*
* vdLogToFFSFile - Save the Log
*
* Save the Log to the Flash File System
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdLogToFFSFile (void)
	{

	UINT32  BytesWritten = 0;   /* number of bytes written */
    char    buffer[100];        /* line buffer */
	char	newLine[4];         /* new line buffer */
	INT32	idx;                /* index */
	INT32	cnt;                /* counter */


    FFSHANDLE *pHandle;

	/* Open the file if it is there, otherwise create one */
	if (!(pHandle = OpenFile(LOG_FILENAME)))
        {
        pHandle = CreateFile(LOG_FILENAME, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF);
        }
        
	/* Write a new line to the file */
	sprintf (newLine, "\n");
    WriteFile(pHandle, newLine, strlen(newLine), &BytesWritten);
	sprintf (buffer, "\n--------------------------------------------------");
	WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);

	/* Set the starting location */
	if (logIdx+1 >= LOG_ENTRIES)
		idx = 0;
	else
		idx = logIdx + 1;

	/* Loop through all entries in log */
	for (cnt=0; cnt < LOG_ENTRIES; cnt++)
		{

		if (pLog[idx].idx != NULL)
			{

			/* Print the log entry */
			sprintf (buffer, "\nIndex : %d", pLog[idx].idx);
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);
			sprintf (buffer, "\nFile Name : %s:%d", pLog[idx].fileName, pLog[idx].line);
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);
			sprintf (buffer, "\nParam A : 0x%x", pLog[idx].param_a);
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);
			sprintf (buffer, "\nParam B : 0x%x", pLog[idx].param_b);
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);
			sprintf (buffer, "\nLog Entry : %s", pLog[idx].log);
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);
			WriteFile(pHandle, newLine, strlen(newLine), &BytesWritten);
			sprintf (buffer, "\n--------------------------------------------------");
			WriteFile(pHandle, buffer, strlen(buffer), &BytesWritten);

			}

			/* update the index */
			if (++idx >= LOG_ENTRIES)
				idx = 0;
		}

	/* Write a new line */	
    WriteFile(pHandle, newLine, strlen(newLine), &BytesWritten);

	/* Close the file */
	CloseFile(pHandle);

	return;

	}
Command (CMD_LOG_TO_FFS,"lffs",vdLogToFFSFile,0,"Write Log to FFS","Write Log to FFS");
CommandEnd ();

#endif /* FFS_ON */


/******************************************************************************
*
* vdClearLog - Clear Log
*
* Clears the log 
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdClearLog (void) 
	{

    logCnt = 0;
    logIdx = 0;

    /* Erase the log */
    memset (pLog, NULL, (sizeof(LOG) * LOG_ENTRIES));

	return;

	}
Command (CMD_CLEAR_LOG,"cl",vdClearLog,0,"Clear the Log","Clear the log");
CommandEnd ();
				 

/******************************************************************************
*
* vdLogCnt - Log Count
*
* Diplays the number of log entries in the log the log 
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdLogCnt (void) 
	{

    /* Check to see if we have filled the log */
    if (logCnt >= LOG_ENTRIES) 
        {
        /* Log filled it, so just display the max entries */
        printf ("\nLog Contains %d entries", LOG_ENTRIES);
        }
    else
        {
        /* Log not filled just just display current count */
        printf ("\nLog Contains %d entries", logCnt);
        }

    printf ("\n");

	return;

	}
Command (CMD_LOG_CNT,"lc",vdLogCnt,0,"Display Log Count","Displays the number of entries in the log");
CommandEnd ();


/******************************************************************************
*
* vdParseFileName - Parse the file name out
*
* Takes the full path for the filename from __FILE__ and returns only the 
* filename part of it.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdParseFileName 
	(
	UINT8 *		filepath,		/* filename with full path to parse */
	UINT8 *		filename        /* filename to return to */
	)
	{

	UINT32		idx;
	UINT32		nmIdx;

	/* Parse through the name */
	idx = nmIdx = 0;
	while (nmIdx < MAX_FILE_NAME)
		{
		/* Save this character to the filename string */
		filename[nmIdx] = filepath[idx];

		/* Increment the counter and see if done */
		if (++nmIdx >= MAX_FILE_NAME)
			{
			/* 
			 * The name was longer than MAX so, erase last and 
			 * assign NULL
			 */
			filename[nmIdx-1] = NULL;
			break;
			}

		/* Check to see if the char is a / or a \ */
		idx++;
		if (filepath[idx] == '\\' || filepath[idx] == '/')
			{
			/* 
			 * Don't want the full path, so reset the name idx and increment
			 * past this char 
			 */
			nmIdx = 0;
			idx++;
			}
		}
	
	return;

	}


/******************************************************************************
*
* vdCopyLog - Copy the log String
*
* Copies the log string
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdCopyLog
	(
	UINT8 *		src,	/* source */
	UINT8 *		dest,	/* destination */
	UINT32		maxLength	/* Max Length to copy */
	)
	{

	UINT8	idx;		/* index */

	/* Save the string */
	for( idx=0; idx < maxLength; idx++ )
		{
		/* Save this character to the string */
		dest[idx] = src[idx];

		/* check to see if there is any more to add */
		if( idx+1 >= maxLength || 
			src[idx] == NULL )
			{
			/* Either at the end or the string has exceeded the max length */
			dest[idx] = NULL;
			break;
			}
		}

	return;

	}


/******************************************************************************
*
* vdLog - Add Log
*
* Adds a log entry to the log if the log has been initialized.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void vdLog 
	(
	char *		log,			/* Log String */
	char *		filename,		/* File name log occured */
	UINT32		lineNo,			/* File line number */
	UINT32		param_a,		/* Log Param A */
	UINT32		param_b			/* Log Param B */
	)
	{

    INT32       tmpIdx;         /* Temp index */

	/* Save the file name - Can't rely on the string library */
	vdParseFileName (filename, pLog[logIdx].fileName);

	/* Copy the string info name */
	strncpy (pLog[logIdx].log,log, MAX_LOG_LGTH);

	/* Save info to the current location in the log */
	pLog[logIdx].idx = ++logCnt;
	pLog[logIdx].line = lineNo;
	pLog[logIdx].param_a = param_a;
	pLog[logIdx].param_b = param_b;

    /* Get the index to the previous entry in the log */
    tmpIdx = ((logIdx - 1) < 0)? LOG_ENTRIES : logIdx - 1;
        
    /* Check to see if this log entry is already there */
    if (strncmp (pLog[logIdx].log, pLog[tmpIdx].log, MAX_LOG_LGTH) ||
        strncmp (pLog[logIdx].fileName, pLog[tmpIdx].fileName, MAX_FILE_NAME) ||
        (lineNo != pLog[tmpIdx].line) ||
        (param_a != pLog[tmpIdx].param_a) ||
        (param_b != pLog[tmpIdx].param_b))
        {
        /* Not matching log entry - Increment the log pointer */
        logIdx++;
        logIdx = (logIdx % LOG_ENTRIES);
        }
    else
        {
        /* Matching Log entry - Reset the log counter */
        logCnt--;

        /* Clear the entry we just filled */
        pLog[logIdx].idx = 0;
        pLog[logIdx].line = 0;
        pLog[logIdx].param_a = 0;
        pLog[logIdx].param_b = 0;
        }

	return;

	}

#endif /* LOG_ENBL */

