/***************************************************************************
*
* Module: vdLoop.c
*
* Description: Routine allows the user to loop at a specified address
			   with a specified data pattern.
*
* Project:     EST VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/

//  Includes
#include "vm.h"
#include "esttype.h"
#include "cmd.h"
#include "lib.h"
#include "vdloop.h"  

//  Local Defines
void LoopDelay(UINT32 DelayLoops);
UINT32 GetUserString(int MenuNumber); 
//int Loop_atoiHex(const char *InputStream);

void HelpText(void);
loopinput scopein;

void scopeloop(loopinput *in) 
{
#ifndef MINAGENT
	char StartStopLoop,KeyboardInput;
	int NumberOfLoops = 0;
	int MaxNumberOfLoops;
	int KeyPressValid,SubKeyPressValid;
	UINT32 LoopStartAddress;
	UINT32 TestPattern;
	UINT32 ReadBackValue;
	UINT32 *MemoryLocationPtr;
	UINT32 Delay;
	UINT32 ReturnedHexValue;
	
	/*********************************
	 * Get user specified values and *
	 * set variables accordingly	 *
	 *********************************/
		
	
	LoopStartAddress = in->LoopStart;
	TestPattern = in-> Pattern;
	MaxNumberOfLoops = in -> LoopEnd;
	Delay = in->TimeDelay;
	
	
	while(NumberOfLoops<MaxNumberOfLoops)
	{	
		MemoryLocationPtr = (UINT32 *)LoopStartAddress;

		LoopDelay(Delay);
				
		*MemoryLocationPtr = TestPattern;
		
		LoopDelay(Delay);

		ReadBackValue = *MemoryLocationPtr;
		
		NumberOfLoops++;
		 
		/*****************************************************************
		 * Check to see if the maximum number of loops has been reached, *
		 * if 'YES' prompt display the main menu and prompt the user to  *
		 * enter a selection.											 *
		 *****************************************************************/
		if(NumberOfLoops==(MaxNumberOfLoops))
		{
			KeyPressValid = SubKeyPressValid = 1;

			while(KeyPressValid)
			{	
				printf(" \n");
				printf("\t\tMain Menu\n");
				printf(" \n");
				printf("\t\t 1\t Re-start the program with the current settings.\n");
				printf("\t\t 2\t Display the current settings.\n");
				printf("\t\t 3\t Change the current settings via the Configuration\n");
				printf("\t\t  \t Menu.\n");
				printf("\t\t 4\t Display the program's \"help\" text.\n");
				printf("\t\t 5\t Exit the program.\n");
				printf(" \n");
				printf("Please select an option :  "); 
				KeyboardInput= GetUserString(5);
			
				switch(KeyboardInput)
				{
					case '1' :	KeyPressValid = 0;
								NumberOfLoops = 0;
								break;

					case '2' :	printf(" \n");
								printf("Current Settings\n");
								printf("\t Address \t = \t0x%08X\n", LoopStartAddress);
								printf("\t Test Pattern \t = \t0x%08X\n", TestPattern);
								printf("\t Number of loops = \t0x%08X\n", MaxNumberOfLoops);
								printf("\t Value of delay  = \t0x%08X\n", Delay);
								break;
							   

					case '3' :	// now display the sub menu and prompt for a selection
								while(SubKeyPressValid)
								{
								    printf(" \n");
									printf("\t\tConfiguration Menu.\n");
									printf(" \n");
									printf("\t\t 1\t Change the address of where the program loops.\n");
									printf("\t\t 2\t Change the data being written to the address.\n");
									printf("\t\t 3\t Change the number of loops the program executes.\n");
									printf("\t\t 4\t Change the time delay between loops.\n");
									printf("\t\t 5\t Display the current settings.\n");
									printf("\t\t 6\t Exit menu and re-start the program with the current\n");
									printf("\t\t  \t settings.\n");
								
									printf(" \n");
									printf("Please select an option:  ");
									KeyboardInput = GetUserString(5);
			
									switch(KeyboardInput)
									{
										case '1' :	ReturnedHexValue = GetUserString(1);
													LoopStartAddress = ReturnedHexValue;
													SubKeyPressValid = 1;											 
													break;

										case '2' :	ReturnedHexValue = GetUserString(2);
													TestPattern = ReturnedHexValue;
													SubKeyPressValid = 1;											 
													break;

										case '3' :	ReturnedHexValue = GetUserString(3);
													MaxNumberOfLoops = ReturnedHexValue;
													SubKeyPressValid = 1;											 
													break;

										case '4' :	ReturnedHexValue = GetUserString(4);
													Delay = ReturnedHexValue;
													SubKeyPressValid = 1;											 
													break;
																				
										case '5' :  printf("Current Settings\n");
													printf("\t Address \t = \t0x%08X\n", LoopStartAddress);
													printf("\t Test Pattern \t = \t0x%08X\n", TestPattern);
													printf("\t Number of loops = \t0x%08X\n", MaxNumberOfLoops);
													printf("\t Value of delay  = \t0x%08X\n", Delay);
													SubKeyPressValid = 1;
													break;

										case '6' :	SubKeyPressValid = 0;
													Info("Running routine with current values\n");
													break;

									}// end of sub menu switch
								}// end of while sub menu
								KeyPressValid= 0; // need to exit the main loop
							    NumberOfLoops = 0;
								break;

					case '4' :  HelpText();
								break;

					case '5' :	Info("terminating scopeloop program\n");
								KeyPressValid = 0;
								break;

					default  :	Info("not a valid key press, please re-enter selection\n");
								break;

				}//end of main menu switch

			}//end of while main menu
		}// end of if loop
	}// end of while loop
#endif
}// end of scopeloop


// Start of Functions

 /************************************************************************
  *	Function to generate a delay between each of the write / read loops. *
  *	The reason for this is to allow enough set-up and hold time for the  *
  *	devices under test. 											     *
  ************************************************************************/

void LoopDelay(UINT32 DelayLoops)
{
	UINT32 DelayCounter;
	for (DelayCounter = 0; DelayCounter<=DelayLoops;DelayCounter++)
	{
		// doesnt need to do anything as it is just a delay loop
	}

	return;

}// end of LoopDelay

 /************************************************************************************
  * The purpose of this function is to get the new values from the user. As there is *
  * no Gets function within the vWARE command structures, this routine effectively   *
  * creates one. The routine takes in the value of the menu option passed to it and  *
  *	then prints the matching command line. The user then enters the new value for    *
  *	the menu item.The routine checks for the carriage return value in the buffer to  * 
  *	indicate that the value is correct.The routine then checks to see if the delete  *
  *	key had been pressed at any time and if so deletes the required character. The   * 
  *	routine then converts the ascii value to a hex number before passing it back to  *
  * the calling program.															 *
  ************************************************************************************/

UINT32 GetUserString(int MenuNumber)
{
	char UserString[32]; 
	char KeyboardInput; 
	// Generate message strings
	char MessageAddress[50];
	char MessageData[50]; 
	char MesageLoop[50];  
	char MessageTime[50];
	char InputChar[3];
	char buffer[32];
	char TempChar; 
	int Keyboardindex = 0;
	int index = 0;
	int indexb=0;
	int MaxArrayIndex;   
	char MenuReturn;
	char KeyReturn[2];
	
	// Variable for special characters  
	UINT16 EOF = 0x0d; // Assign EOF with the value a Carriage Return keypress would generate
	UINT16 DEL = 0x08; // Assign DEL with the value a Delete keypress would generate.
	
	// Variable for return value 
	UINT32 HexValue = 0;
    
    KeyReturn[1] = '\0';                     
    strcpy(MessageAddress,"Enter new value for Address 0x"); 
    strcpy(MessageData,"Enter new value for Data 0x");
    strcpy(MesageLoop,"Enter new value for Loop Number (in hex) 0x");
    strcpy(MessageTime,"Enter new value for Time delay (in hex) 0x"); 
#ifndef MINAGENT
	// Check MenuNumber and print corresponding message
	Info(" \n"); // ensures message is on a new line 
	if (MenuNumber==1)
	{
		Info("%s",MessageAddress);
	}
	else if (MenuNumber==2)
	{
		Info("%s",MessageData);
	}
	else if (MenuNumber==3)
	{
		Info("%s",MesageLoop);
	}
	else if (MenuNumber==4)
	{
		Info("%s",MessageTime);
	}     
	
	KeyboardInput = 0;
	while(KeyboardInput!=EOF)
	{	KeyboardInput= getchar();  
		if(KeyboardInput!=EOF)
		{
		  if (MenuNumber==5) // this function allows the menu case statements to work
		  { 
			if (KeyboardInput==0x31)
			{
			  MenuReturn = '1'; 
			}
			else if (KeyboardInput==0x32)
			{
			  MenuReturn = '2'; 
			}
			else if (KeyboardInput==0x33)
			{
			  MenuReturn = '3'; 
			}
			else if (KeyboardInput==0x34)
			{
			  MenuReturn = '4'; 
			}
			else if (KeyboardInput==0x35)
			{
			  MenuReturn = '5'; 
			}	
			else if (KeyboardInput==0x36)
			{
			  MenuReturn = '6'; 
			}		             
			else
			{
			  MenuReturn = '9'; 
			}
							
			return(MenuReturn);
		}

		  UserString[index] = KeyboardInput;   
		  KeyReturn[0] = KeyboardInput;
		  printf("%s",KeyReturn); 
		  index++;  
		}
		else if(KeyboardInput==DEL)
		{
			index=index-2;
			UserString[index]=KeyboardInput; //new value will be in buffer
		}
	
	
		//now add the end of file character for conversion and printing
		else
		{
			UserString[index] = '\0';
			MaxArrayIndex=index;
		}
	}// end of while           
	
	                                 
	// Get ready to start removing unwanted characters by copying the array 
	for (index=0;UserString[index]!='\0';index++)
	{
		buffer[index]=UserString[index];
		buffer[MaxArrayIndex]='\0';
	}

    // Routine to remove unwanted characters 
	for (index=0,indexb=0;UserString[index]!='\0';index++)
	{
		if (UserString[index]==0x08)
		{
			indexb=(index+2);//go to value to use as replacement
			index= (index-2);// set pointer at value to delete

			while(UserString[index+2]==0x08)
			{
			  for(;indexb<=MaxArrayIndex;indexb++,index++)
			  {
				UserString[index]= buffer[indexb]; // get correct value
				UserString[indexb] = 0x00; // remove new value so not to use it again
				UserString[indexb-2] = 0x00; //remove del character 
			  }

			}// end of while loop 

			//now copy old array again so that it is same as original 
			for (index=0;UserString[index]!='\0';index++)
			{
				buffer[index]=UserString[index];
				buffer[MaxArrayIndex]='\0';
			}
			index=0;
			indexb=0;

		}// end of if statement 
		
			
	}// end of for loop 

	HexValue = htoi(UserString);
#endif
	return(HexValue);

}// end of GetUserString 

 /*******************************************************************
  * The purpose of this function is to hold all of the text for the	*
  * programs help command. Any new features , bugs etc should be	*
  * listed here.													*
  *******************************************************************/ 


void HelpText(void)
{
#ifndef MINAGENT
	printf(" \n");
	printf("\t\t Help Message \n");
	printf(" \n");
	printf("\tThis program is designed to enable the user to monitor \"write\"\n");
	printf("\tand \"read\" operations  at  a single  memory location  with  an\n");
	printf("\toscilloscope.\n");
	printf("\tThe  program first \"writes\" a pre-defined  test pattern to the\n"); 
	printf("\tlocation  and  then after a pre-set  time  period has elapsed,\n"); 
	printf("\tcarries out a \"read\" from the same location. The  procedure is\n");
	printf("\trepeated   until  the  number  of  required \"write\" and \"read\"\n");
	printf("\tcycles have been executed.\n");	
	printf("\tOnce the  program  completes, it automatically calls the  Main\n"); 
    printf("\tMenu.\n"); 
	printf("\tThe  program  does NOT perform any analysis on the data \"read\"\n");
	printf("\tfrom the location to see if it is correct.\n");
	printf("\tAll  of the  values  used  in  the  program  can be changed by\n");
	printf("\tselecting  the  required  menu option  from the  configuration\n");
	printf("\tmenu.\n");
#endif
	return;

}// end of Helptext
         
 /*********************************************************************
  * The purpose of this function is to convert the ascii input values *
  * from the keyboard that are stored in the users string to their    *
  * true value. The function is included directly in the routine so   *
  * that it has minimal dependencies.								  *
  ********************************************************************/ 
#if 0
int Loop_atoiHex(const char *InputStream)
{
	int				i, j;
	unsigned char	c,cVal;
	int				nVal = 0;
	int				nExp;

	j = strlen(InputStream);

	for (i = 0; i < j; i++)
	{
		c = tolower(*InputStream++);

		switch (c)
		{			
			case 'a':
				cVal = 10;
				break;

			case 'b':
				cVal = 11;
				break;

			case 'c':
				cVal = 12;
				break;

			case 'd':
				cVal = 13;
				break;

			case 'e':
				cVal = 14;
				break;

			case 'f':
				cVal = 15;
				break;

			default:
				if (c <= '9' && c >='0')
					cVal = c - '0';
				else
					cVal = 0;
				break;
		}

		nExp = exp(16,j-(i+1));
		nVal += cVal * nExp;
	}
	return nVal;
}


#endif

 /******************************************************************************
  * The purpose of this routine is to allow external memory test procedures to *
  * call the function without having to include an initialization procedure in *
  * each of the calling functions. With this routine included, all that is now *
  * needed to run this routine externally is the call to this command line.	   *
  ******************************************************************************/

void ExtScopeLoopSetup(UINT32 address, UINT32 pattern)
{
	// set default values
	int loopcount = 20;
	UINT16 delay = 20;
	
	scopein.LoopStart = address;
	scopein.Pattern = pattern;
	scopein.LoopEnd = loopcount;
	scopein.TimeDelay = delay;

	scopeloop(&scopein);

	return;
}
    
 /*****************
  * User commands *
  *****************/
/*
Command(CMD_LOOP, "tdfloop", scopeloop, 0, "scopeloop", "Address looping routine");
Arg(UINT32ARG, "start address", "Enter the address for the program to loop on");
Arg(UINT32ARG, "test pattern", "Enter the test pattern to be written to the address");
Arg(UINT32ARG, "number of loops", "Enter number of loops (write & read cycles)");
Arg(UINT32ARG, "loop time delay", "Enter the time delay between each write and read cycle");
CommandEnd();*/