
/* vdPCISETSHOW.c - Individual target generic pci routines  */

/* Copyright 1999-2000 Wind River Systems, Inc. */

/*
modification history
--------------------
01	25June02, rcs,		Added new pciread function 
  */


/*DESCRIPTION This file contains generic pci function calls that can be accessed from the
command line

  INCLUDE FILES :  vm.h, cmd.h, cmds.h, pci.h */

/* includes */
#include "vm.h"
#include "cmd.h"
#include "cmds.h"
#include "pci.h"

/* defines */

/* typedefs */

/* globals */
CommandGroup(CMDGROUP_PCI, "vPCI COMMANDS");

/* locals */
enum
{
	CMD_PCISHOW = MAKECMD(CMDGROUP_PCI, 1),
	CMD_PCISET,
	CMD_PCIHEADER,
	CMD_PCIREAD

};



/****************************************************************************
*
* vdPCISETSHOW.c List all of the devices on a specific pci bus
*
*
* RETURNS : N/A
*
* ERRNO:  N/A
*/

typedef struct _pcishowIn
	{
	UINT32	busno; /*Arg Description*/
	} pcishowIn;



void pcishow(
			 pcishowIn* pIn,
			 void* pUnused
			 )
			{
			pciDeviceShow(pIn->busno);
			}

/* Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");*/
Command(CMD_PCISHOW,"pcishow",pcishow,REPEATABLE,"pcishow","pcishow");
Arg(UINT32ARG,"bus number","enter PCI bus number");
CommandEnd();




/****************************************************************************
*
* pciset	Set a specified pci register
*
*
* RETURNS : N/A
*
* ERRNO:  N/A
*/

typedef struct _pcisetIn
	{
	UINT32	Width;
	UINT32	busNo; /*Arg Description*/
	UINT32	deviceNo; /*Arg Description*/
	UINT32	funcNo; /*Arg Description*/
	UINT32	offset; /*Arg Description*/
	UINT32	value; /*Arg Description*/
	} pcisetIn;


void pciset(
			pcisetIn* p,
			void* pUnused2
			)
			{
			switch(p->Width)
			{
			case 1:
				pciConfigOutByte(p->busNo,p->deviceNo,p->funcNo,p->offset,p->value);
				break;
			case 2:
				pciConfigOutWord(p->busNo,p->deviceNo,p->funcNo,p->offset,p->value);
				break;
			case 4:
				pciConfigOutLong(p->busNo,p->deviceNo,p->funcNo,p->offset,p->value);
				break;
			default:
				break;
			}
			}

/* Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");*/
Command(CMD_PCISET,"pciset",pciset,SIZESUFFIX,"pciset","pciset");
Arg(UINT32ARG,"bus","PCI bus number");
Arg(UINT32ARG,"device","PCI device number");
Arg(UINT32ARG,"function","PCI function number");
Arg(UINT32ARG,"offset","PCI header offset");
Arg(UINT32ARG,"value","value to write into header");
CommandEnd();


/****************************************************************************
*
* pciheader Display the header information for a specified device
*
*
* RETURNS : N/A
*
* ERRNO:  N/A
*/

typedef struct _pciheaderIn
	{
	UINT32	busNo;  /* Arg Description */
	UINT32	deviceNo; /*Arg Description */
	UINT32	funcNo;  /*Arg Description */
	} pciheaderIn;



void pciheader(
			   pciheaderIn* pIn,
			   void* pUnused
			   )
				{
				pciHeaderShow(pIn->busNo,pIn->deviceNo,pIn->funcNo);
				}

/* Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");*/
Command(CMD_PCIHEADER,"pciheader",pciheader,0,"pciheader","pciheader");
Arg(UINT32ARG,"bus","PCI bus number");
Arg(UINT32ARG,"device","PCI device number");
Arg(UINT32ARG,"function","PCI function number");
CommandEnd();

/*************************************************************************
* pciread - Definition for the pciread Diagnostic
*
* RETURNS: N/A
* ERRNO: N/A
*/

typedef struct
	{
	int size;
	int bus;
	int device;
	int function;
	int Offset;
	}pciread_in;


void pciread
    (
	pciread_in *pin, 
	void *pout
    )
    {

	ULONG ConfigAddress;
	UINT32 Data;

	ConfigAddress = PciEncode(pin->bus,pin->device,pin->function); 

	switch (pin->size)
	{
	case 4: Data = PciCfgRead(ConfigAddress,pin->Offset,REG_32BIT); 
			printf("Register at Offset %x = %08X \n", pin->Offset, Data);
			break;
	case 2: Data = PciCfgRead(ConfigAddress,pin->Offset,REG_16BIT); 
			printf("Register at Offset %x = %04X \n", pin->Offset, Data);	
			break;
	case 1: Data = PciCfgRead(ConfigAddress,pin->Offset,REG_8BIT); 
			printf("Register at Offset %x = %02X \n", pin->Offset, Data);
			break;
	default : break;
	}
    }

Command(CMD_PCIREAD,"pciread",pciread,SIZESUFFIX,"pciread","read a specified pci register");
Arg(UINT32ARG,"bus","PCI bus number");
Arg(UINT32ARG,"device","PCI device number");
Arg(UINT32ARG,"function","PCI function number");
Arg(UINT32ARG,"offset","PCI header offset");
CommandEnd();

