/******************************************************************************
*
* Module: vdtdf1.c
*
* Description: Simple memory test
*
* Project:     HSI VisionWare -- vWare
*
* Functions: fnActualTest,write_data_TDF1,read_data_TDF1
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

//  Includes
#include "vm.h"
#include "esttype.h" 
#include "cmd.h"
#include "lib.h"
#include "vware.h"
#include "vdtdf1.h"
#include "vderror.h"
#include "vdloop.h"
#include "board.h" 
#include "boot.h"

//  Local Defines
#define Pass 0
#define Fail 1
#define ExecutionOk 0
#define ExecutionFail 0xffffffff
#define UserAbort 2
#ifndef RAM2START
    #define RAM2START 0
	#define RAM2END 0
#endif
#ifndef RAM3START
	#define RAM3START 0
	#define RAM3END 0
#endif
//  External References
extern PROTECTEDAREA ProtectedArea[];
extern UINT32 ProtectedAreas;
extern unsigned int CharWaiting(void);
//  Forward References

//  Global Declarations
//char cEscape;
UINT8 QuitEnabled = 0x0;

//  Local Declarations (statics)
TDF1_out01 ErrorCodes1;
UINT32 fnActualTest(UINT32 Start, UINT32 End, UINT32 Width, UINT32 NumberOfPatterns, char Scope);
void write_data_TDF1( UINT32 Width, UINT32 Addr, UINT32 Data);
UINT32 read_data_TDF1( UINT32 Width, UINT32 Addr);


//Start of main routine
int TDF1(TDF1_in *pIn, TDF1_out01 *pOut,
		unsigned long *pOutputStructureNumber)
{
  
  volatile UINT32 lStartAddr 	= pIn->Start; 
	volatile UINT32 lEndAddr	= pIn->End;
	volatile UINT32 lWidth 		= pIn->Width;
	volatile UINT32	lCount;  
	volatile UINT32 lCount2;
	volatile UINT32 lTemp = ProtectedAreas+1;
	volatile UINT32 lStartRegion = 0xffffffff;
	volatile UINT32 lEndRegion = 0xffffffff;
	volatile UINT32 lNumberOfTests;
	volatile UINT32 lNumberOfPatterns	= pIn->lNumberOfPatterns;
			 UINT32 lStart[10];
			 UINT32 lEnd[10];
			 char	cScope;
			 UINT32	TDF1_pass_fail =Pass;

   ((TDF1_out02 *)pOut)->ReturnCode = Pass; // reset ReturnCode variable


   //Get user defined input variables
	//cEscape				= pIn->Escape[0];
	if((tolower(pIn->Escape[0]) == 'y'))
	{
		QuitEnabled = 0x1;
	}
	cScope				= pIn->Scopeloop[0];
   

	//Ensure that the width has been captured correctly
	if(lWidth != 0x4)
	{
		if(lWidth != 0x2)
		{
			if(lWidth != 0x1)
			{
				((TDF1_out02 *)pOut)->PassFail = Fail;
				((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail; 
				LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, BAD WIDTH\n");
				TDF1_pass_fail = Fail;
				return;
			}
		}
	}

	if(0)
	{
	//Ensure start address is within speicified memory region for RAM1
	if((lStartAddr < RAM1START)|| (lStartAddr > (RAM1END)))
	{
		if((lStartAddr < RAM2START)|| (lStartAddr > (RAM2END)))
		{
			if((lStartAddr < RAM3START)|| (lStartAddr > (RAM3END)))
			{
				((TDF1_out02 *)pOut)->PassFail = Fail;
				((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
				LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, START ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, START ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				TDF1_pass_fail =Fail;
				return(Fail);
			}
		}
	}

	//Ensure end address is within speicified memory region for RAM1
	if((lEndAddr < RAM1START) || (lEndAddr >(RAM1END)))
	{
		if((lEndAddr < RAM2START) || (lEndAddr >(RAM2END)))
		{
			if((lStartAddr < RAM3START)|| (lStartAddr > (RAM3END)))
			{
				((TDF1_out02 *)pOut)->PassFail = Fail;
				((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
				LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, END ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, END ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				TDF1_pass_fail =Fail;
				return(Fail);
			}
		}
	}

	} //endif (0)

	//Ensure start address is properly aligned for access size
	if (lStartAddr % lWidth != 0)
	{
		((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, START ADDRESS ALLIGNMENT ERROR\n");
		sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, START ADDRESS ALLIGNMENT ERROR\n");
		TDF1_pass_fail =Fail;
		return(Fail);
	} 


	//Ensure end address is properly aligned for access size						      
	if (lEndAddr % lWidth != 0)
	{
		((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, END ADDRESS ALLIGNMENT ERROR\n");
		sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, END ADDRESS ALLIGNMENT ERROR\n");
		TDF1_pass_fail =Fail;
		return(Fail);
	} 

		
	//Ensure Start address < End Address 					
	if ( lEndAddr < lStartAddr ) 
	{
		((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, START ADDRESS > END ADDRESS\n");
		sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, START ADDRESS > END ADDRESS\n");
		TDF1_pass_fail =Fail;
		return(Fail);
	}

	//Ensure Start address != End Address
	if(lStartAddr ==lEndAddr)
	{
		((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, START ADDRESS = END ADDRESS\n");
		sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, START ADDRESS = END ADDRESS\n");
		TDF1_pass_fail =Fail;
		return(Fail);
	}

                        
    //Ensure that the number of patterns has been entered correctly
    if(lNumberOfPatterns >0x2)
    {                
    	((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1,  NUMBER OF PATTERNS OUT OF RANGE\n");
		sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, NUMBER OF PATTERNS OUT OF RANGE\n");
		TDF1_pass_fail =Fail;
		return(Fail);
	}
                           
    
	SetOutputNumber(1);
	strcpy(pOut->Result, "fail");

	
	if(Protected(pIn->Start,(pIn->End - pIn->Start))) 
	{
		/**********************************************************************
		 * Sort the visionWare protected areas based on their start addresses *
		 **********************************************************************/
		for(lCount2 = 0; lCount2 < ProtectedAreas-1; lCount2++)
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++)
			{
				if(ProtectedArea[lCount+1].pFirst < ProtectedArea[lCount].pFirst)
				{
					ProtectedArea[lTemp].pFirst		= ProtectedArea[lCount].pFirst; 
					ProtectedArea[lTemp].pLast		= ProtectedArea[lCount].pLast;
					ProtectedArea[lCount].pFirst	= ProtectedArea[lCount+1].pFirst;
					ProtectedArea[lCount].pLast		= ProtectedArea[lCount+1].pLast;
					ProtectedArea[lCount+1].pFirst	= ProtectedArea[lTemp].pFirst; 
					ProtectedArea[lCount+1].pLast	= ProtectedArea[lTemp].pLast;
				}
			}
		}

		//assign defaults
		lStart[0]	= lStartAddr;
		lEnd[0]		= lEndAddr;

		
		/**************************************************************************************
		 * Check to see if the start address of the range to be tested resides in a Protected *
		 * region. If it is, find the region that it resides in, record the region and  reset *	
		 * the start address to the first free memory location after the region.              *
		 **************************************************************************************/
		if((Protected(lStartAddr,lWidth))!=0x0)
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++) 
			{
				if((lStartAddr >= (UINT32)ProtectedArea[lCount].pFirst)&(lStartAddr <= (UINT32)ProtectedArea[lCount].pLast))
				{
					lStart[0] = (UINT32)ProtectedArea[lCount].pLast+1;
					lStartRegion = lCount;
					if(lStart[0] >= (RAM1END))
					{
						if(lStart[0] >= (RAM2END))
						{
							return(1);
						}
					}
				}
			}
		}

		/**************************************************************************************
		 * Check to see if the end address  of the range to be tested  resides in a Protected *
		 * region. If it is, find the region that it resides in, record the region and  reset *	
		 * the end address to the first free memory location before the region.               *
		 **************************************************************************************/
		if((Protected(pIn->End,lWidth))!=0x0) 
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++)
			{
				if((pIn->End >= (UINT32)ProtectedArea[lCount].pFirst)&(pIn->End <= (UINT32)ProtectedArea[lCount].pLast))
				{
					lEnd[0] = (UINT32)ProtectedArea[lCount].pFirst-1;
					lEndRegion = lCount;
					if((lEnd[0]>RAM1START)&&(lEnd[0]<RAM2START))
					{
						lEnd[0] = RAM1START;
					}
					//Range could be in either RAM2 or RAM3
					else if((lEnd[0]>RAM2START)&&(lEnd[0]<RAM3START))
					{
						lEnd[0] = RAM2START;
					}
					//Range must reside in RAM3START region
					else if(lEnd[0]>RAM3START)
					{
						lEnd[0] = RAM3START;
					}
				}
			}
		}

		/**********************************************************************************
		 * If start and stop are in the same protected block, print message and exit test *
		 **********************************************************************************/
		if(((lStartRegion != 0xffffffff) & (lEndRegion !=0xffffffff)))
		{

			if(lStartRegion == lEndRegion)
			{
				LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, ENTIRE MEMORY RANGE IS PROTECTED\n");
				sprintf(((TDF1_out02 *)pOut)->ResultString,"ERROR: TDF1, ENTIRE MEMORY RANGE IS PROTECTED\n");
				((TDF1_out02 *)pOut)->PassFail = Fail;
				((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
				return(Fail);
			}
		}
		else
		{
			lStartRegion = 0;
		}
	

		/****************************************************
		 * Assign the regions of memory that can be tested. *
		 ****************************************************/
		if(ProtectedAreas !=0x0)
		{
			lStartAddr = lStart[0];

			for(lCount = lStartRegion; lCount <= ProtectedAreas-1; lCount++)
			{
				if((lEndAddr > (UINT32)ProtectedArea[lCount+1].pFirst)&(lStart[lCount] < (UINT32)ProtectedArea[ProtectedAreas-1].pLast)) 
				{
			   		lEnd[lCount] = (UINT32)ProtectedArea[lCount+1].pFirst;
			   		lStart[lCount+1] = (UINT32)ProtectedArea[lCount+1].pLast+1;
				}
				else 
				{
					lEnd[lCount] = lEndAddr;
					lNumberOfTests = lCount;
					lCount = ProtectedAreas-1;
				}
			}
		}

		/**********************************************************************************
		 * Keep calling the memory test function until all of the assigned memory regions *
		 * have been tested.															  *
		 **********************************************************************************/
		Info("Requested range includes protected areas that will not be tested.\ntype mmap <cr> for details.\n\n");
		if(QuitEnabled)
		{
			Info("Press 'q' at any time to abort test (test may take a few seconds to halt)\n");
		}
		for(lCount = 0x0 ;lCount <= lNumberOfTests; lCount++)
		{
			if(lStart[lCount] !=lEnd[lCount])
			{
				TDF1_pass_fail = TDF1_pass_fail | fnActualTest(lStart[lCount], lEnd[lCount], lWidth, lNumberOfPatterns, cScope);
			}
			if(TDF1_pass_fail ==UserAbort)
			{
				lCount = lNumberOfTests;
			}
	   	}
	}  			
	else
	{
		if(QuitEnabled)
		{
			Info("Press 'q' at any time to abort test (test may take a few seconds to halt)\n");
		}
		if(lStartAddr !=lEndAddr)
			TDF1_pass_fail = TDF1_pass_fail | fnActualTest(lStartAddr, lEndAddr, lWidth, lNumberOfPatterns, cScope);
	}		   		

   	if(TDF1_pass_fail ==0x0)
	{
		Info("Success: TDF1, passed address range 0x%X .... 0x%X\n", pIn->Start,pIn->End);
		((TDF1_out02 *)pOut)->PassFail = Pass;
		((TDF1_out02 *)pOut)->ReturnCode = ExecutionFail;
		sprintf(((TDF1_out02 *)pOut)->ResultString,"Success: TDF1, passed address range 0x%X .... 0x%X\n", pIn->Start,pIn->End);
	}
	else
	{
		((TDF1_out02 *)pOut)->PassFail = Fail;
		((TDF1_out02 *)pOut)->ReturnCode = Pass;
		strcpy(((TDF1_out02 *)pOut)->ResultString, ErrorCodes1.Result);
	}
	return(((TDF1_out02 *)pOut)->PassFail);
}


/****************************************************************************************
* Function: fnActualTest(UINT32, UINT32, UINT32, UINT32, char)							*
*																						*
* Purpose:  The purpose of this routine is, as the name suggests to perform the	 actual	*
*			tests on the specified memory ranges passed to it. The test is broken  down	*
			into  two  sections.  If  a  Longword  width  argument  is passed, to  save	* 
			processing time, the routine handles the checking differently  to that of a *
			Word or Byte Argument. The reason for this is that as the Longword does not *
			require any masking, a simple  pointer is used to both write  and  read the	*
			test data. Both the Word and Byte arguments require calls to the functions	*
			Write_data_TDF1 and read_data_TDF1.											*
*																						*
* Returns:  UINT32 (pass / fail result)													*
*****************************************************************************************/
UINT32 fnActualTest(UINT32 Start, UINT32 End, UINT32 Width, UINT32 NumberOfPatterns, char Scope)
{

   UINT32 lAddr 				= Start; 
   UINT32 lStartAddr			= Start;
   UINT32 lEndAddr				= End;
   UINT32 lWidth				= Width;
   UINT32 lNumberOfPatterns		= NumberOfPatterns;
   UINT32 lData[4]				= {0xaaaa5555, 0x5555aaaa, 0xa5a5a5a5, 0xaa55aa55};
   UINT32 lRxData;
   UINT32 lDataMask;
   UINT32 lMemUnderTest;
   UINT32 lTestNumber			= 1;
   UINT32 *pIndex;
   UINT32 lCount; 
   UINT32 lAddress;
   UINT32 PassFail  			= Pass;
   UINT32 LastAddress			= (End - Width);
   char	cScope					= Scope;
  
    SetOutputNumber(2);
	switch (lWidth) 
	{
		case 4:
			lDataMask = 0xffffffff;
			break;
		case 2:
			lDataMask = 0xffff;
			break;
		case 1:		
			lDataMask = 0xff;
			break; 
	}

	/**************************************************************
	 * Add sanity check to see that start address is within RAM   *
	 **************************************************************/
	 if(lStartAddr > RAM1END)
	{
		if(lStartAddr > RAM2END)
		{
			if(lStartAddr > RAM3END)
			{
				return(0);
			}
		}
	}


	/**************************************************************
	 * Mask Data Values for selected address width using DataMask *
     **************************************************************/
	for( lCount = 0; lCount < 3; lCount ++)
		lData[lCount] = lData[lCount] & lDataMask;
  	
	/**************************************************************************************************
	 * In order to speed up the processing, the  default  Longword  condition has been isolated from  *
	 * the Word and Byte conditions. The number of process steps has also been reduced and therefore, *  
	 * unlike the Words and Byte tests, the routine does not write  the inverse of the first pattern, *
	 * but simply writes the full Longword version of the next test pattern in the Data array.		  *
     **************************************************************************************************/
	if(lWidth ==0x4)
	{
		for(lCount = 0; lCount < lNumberOfPatterns; lCount++)
		{
			Info("Processing 0x%X ... 0x%X with test pattern %X\n", lStartAddr, lEndAddr, lCount+1 );	

			//Write the test pattern in Longword format
			pIndex = (UINT32 *)lStartAddr;	
			do
			{
				*pIndex = lData[lCount];
				pIndex++;
			}while(pIndex<=(UINT32 *)LastAddress);
	
			if(QuitEnabled)
			{
				if(CharWaiting())
				{
					if(tolower(getchar()=='q'))
					{
						LogError(ERR_MEMORY_TEST_1,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF1, TEST ABORTED BY USER\n");
						sprintf(ErrorCodes1.Result, "ERROR: TDF1, TEST ABORTED BY USER\n");
						lCount = lNumberOfPatterns;
						return(UserAbort);
					}
				}
			}
				
			
			//Verify that the pattern was written correctly
			for(pIndex=(UINT32 *)lStartAddr; pIndex <= (UINT32 *)LastAddress; pIndex++)
			{
				if(*pIndex != lData[lCount]) 
    			{
    				LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",lData[lCount],pIndex,*pIndex);
					sprintf(ErrorCodes1.Result, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",lData[lCount],pIndex,*pIndex);				
					PassFail = Fail;
					pIndex = (UINT32 *)lEndAddr;
					if(cScope == 'y')
					{
						ExtScopeLoopSetup((UINT32)pIndex, lData[lCount]);
					}
					return(PassFail);
    			}
			}
		}
	}
	else
	{	//write the test pattern in either Word or Byte format
		for(lCount = 0; lCount < lNumberOfPatterns; lCount++)
		{

			Info("Processing 0x%X ... 0x%X with test pattern %X\n", lStartAddr, lEndAddr, lCount+1 );
			lAddr = lStartAddr;
			do
			{
				if(lCount ==0x0)
				{
					write_data_TDF1( lWidth, lAddr, lData[lWidth+1]);
				}
				else
				{
					write_data_TDF1( lWidth, lAddr, ~(lData[lWidth+1]));
				}
				lAddr = lAddr+lWidth;
			}while (lAddr < lEndAddr);

			if(QuitEnabled)
			{
				if(CharWaiting())
				{
					if(tolower(getchar()=='q'))
					{
						LogError(ERR_MEMORY_TEST_1,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF1, TEST ABORTED BY USER\n");
						sprintf(ErrorCodes1.Result, "ERROR: TDF1, TEST ABORTED BY USER\n");
						lAddr = lEndAddr;
						lCount = lNumberOfPatterns;
						return(UserAbort);
					}
				}
			}
		


			for(lAddr= lStartAddr; lAddr < LastAddress; lAddr+=lWidth)
			{
				lRxData = read_data_TDF1( lWidth, lAddr);
				if(lCount ==0x0)
				{
					if (lRxData != (lData[lWidth+1]&lDataMask))
					{
						LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",lData[lWidth+1],lAddr,lRxData);
//						sprintf(ErrorCodes1.Result, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",(lData[lWidth+1]&lDataMask),lAddr,lRxData);				
						PassFail = Fail;
						if(cScope == 'y')
						{
							ExtScopeLoopSetup(lAddr, lData[lWidth+1]);
						}
					}
				}
				else
				{
					if (lRxData != (~(lData[lWidth+1])&lDataMask))
					{
						LogError(ERR_MEMORY_TEST_1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",lData[lWidth+1],lAddr,lRxData);
//						sprintf(ErrorCodes1.Result, "ERROR: TDF1, Expected 0x%X at 0x%X, Received 0x%X\n",(lData[lWidth+1]&lDataMask),lAddr,lRxData);				
						PassFail = Fail;
						if(cScope == 'y')
						{
							ExtScopeLoopSetup(lAddr, lData[lWidth+1]);
						}
					}
				}
				
				if(PassFail)
					lAddr = lEndAddr;
			}
		}
	}
  return(PassFail);
}



/************************************************************************************
* Function: write_data_TDF1(UINT32, UINT32, UINT32)									*
*																					*
* Purpose:  Writes specified  data to specified memory location based on the memory *
*			access  size  (also specified). Width must be 1, 2,4 or 8 (byte, short, *
*			word, longword respectively) for this implementation. This code assumes *  
*			that all values sent to it have already been verified as valid.			*
*																					* 
* Returns: void																		*
*************************************************************************************/
void write_data_TDF1( UINT32 Width, UINT32 Addr, UINT32 Data)
{
	// write data has already been masked properly and width 
	// has already been verified to be valid

	universal p;			   

	p.uvalue = Addr;

	switch (Width)
	{
	case 1:
		*p.pByte = (UINT8)Data;
		break;

	case 2:
		*p.pHalf = (UINT16)Data;
		break;

	case 4:
		*p.pWord = (UINT32)Data;
		break;
	}
}




/*************************************************************************************
* Function: read_data_TDF1(UINT32, UINT32)											 *
*																					 *
* Purpose:  Reads  data  from a specified memory location based on the memory access * 
*			size ( also specified ). Width  must  be 2,4 or 8 (short, word, longword *
*			respectively) for this implementation. This code assumes that all values *
*			sent to it have already been verified as valid.							 *
*																					 *
* Returns:  void																	 *
**************************************************************************************/
UINT32 read_data_TDF1( UINT32 Width, UINT32 Addr)
{
	union
	{
		UINT8 *sAddr;
		UINT16 *wAddr;
		UINT32 *lAddr;
		UINT32 Addr;
	} p;			   

	p.Addr = Addr;

	switch (Width)
	{
	case 1:
		return (UINT32)*p.sAddr;
	
	case 2:
		return (UINT32)*p.wAddr;
	
	case 4:
		return (UINT32)*p.lAddr;
	}
}

Command(CMD_TDF1, "tdf1", TDF1, SIZESUFFIX|SILENTTYPE|DIAGNOSTIC, "TDF1", "simple ram test between two addresses");
Arg(UINT32ARG,"start address", "enter the start address of the memory to be tested");
Arg(UINT32ARG, "end address", "enter the end address of the memory to be tested");
OptArg(UINT32ARG, "number of patterns to perform (1 or 2)", "perform two tests on each address (1 or 2)",1);
OptArg(CHAR2ARG,"escape", "select ability to exit diagnostic whilst running (y or n)","n");
OptArg(CHAR4ARG,"scopeloop", "select ability to loop on error (y or n)","n"); 
Output(UINT32ARG, "Return Code");
Output(CHAR8ARG, "Result");
NewOutput(UINT32ARG, "Return Code");
Output(CHAR8ARG, "Result");
Output(UINT32ARG, "Address");
Output(UINT32ARG, "Expected");
Output(UINT32ARG, "Read");
NewOutput(UINT32ARG, "Return Code");
Output(CHAR64ARG, "Error");
CommandEnd();





