/******************************************************************************
*
* Module: vdtdf2.c
*
* Description: omplex memory test
*
* Project:     HSI VisionWare -- vWare
*
* Functions: fnFloatOnesAndZeros
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"
#include "esttype.h"
#include "cmd.h"
#include "lib.h"
#include "vdtdf2.h"
#include "vderror.h"
#include "vdloop.h"
#include "board.h"
#include "boot.h"

//  Local Defines
#define Fail		1
#define Pass		0
#define RUN			3
#define ExecutionOk 0
#define ExecutionFail 0xffffffff
#define UserAbort 2
#ifndef RAM2START
    #define RAM2START 0
	#define RAM2END 0
#endif
#ifndef RAM3START
	#define RAM3START 0
	#define RAM3END 0
#endif

//Externs
extern PROTECTEDAREA ProtectedArea[];
extern UINT32 ProtectedAreas;
extern unsigned int CharWaiting(void);

TDF2OutParams ErrorCodes2;

UINT32 fnFloatOnesAndZeros(UINT32 StartAddress, UINT32 EndAddress, UINT32 NumberOfPatterns, char Scope);


//Globals
UINT8 QuitEnable = 0x0;

int TDF2(TDF2InParams *In, TDF2OutParams *Out)
{
  volatile UINT32 mt2_pass_fail =Pass;
  volatile UINT32 lWidth;
  volatile UINT32 lStartAddr;
  volatile UINT32 lEndAddr;
  volatile UINT32 lTestNumber = 1;
  volatile UINT32 lTestValue;
  volatile UINT32 lCount;
  volatile UINT32 lCount2;
  volatile UINT32 lTemp = ProtectedAreas+1;
  volatile UINT32 lStartRegion = 0xffffffff;
  volatile UINT32 lEndRegion = 0xffffffff;
  volatile UINT32 lNumberOfTests;
  volatile UINT32 lNumberOfPatterns;
   		   UINT32 lStart[0x10];
		   UINT32 lEnd[0x10];
		   char cScope;

	
 
  Out->Status = RUN;

  lWidth = 0x4;//Use LongWord setting to increase speed of test
  lStartAddr = In->StartAddress;
  lEndAddr   = In->EndAddress;
  lNumberOfPatterns = In->NumberOfPatterns;
  cScope = In->Scopeloop[0];
 
	if((tolower(In->Escape[0]))=='y')
	{
		QuitEnable = 0x1;
	}	

   /*********************************************************
    * Ensure start and end addresses are properly aligned   *
    * for access size									    *
    *********************************************************/
	if (lStartAddr % lWidth != 0)
	{
		Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, MISALIGNED START ADDRESS\n");
		sprintf(Out->ResultString,"ERROR: TDF2, MISALIGNED START ADDRESS\n");
		mt2_pass_fail =Fail;
		return(Fail);
	} 

	if (lEndAddr % lWidth != 0)
	{
		Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, MISALIGNED END ADDRESS\n");
		sprintf(Out->ResultString,"ERROR: TDF2, MISALIGNED END ADDRESS\n");
		mt2_pass_fail = Fail;
		return(Fail);
	}  

	/********************************************************
	 * Ensure End address > Start Address 					*
	 ********************************************************/
	if ( lEndAddr < lStartAddr ) 
	{
		Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, START ADDRESS > END ADDRESS\n");
		sprintf(Out->ResultString,"ERROR; TDF2, START ADDRESS > END ADDRESS\n");
		mt2_pass_fail =Fail;
		return(Fail);
	}

	/********************************************************
	 * Ensure Start address != End Address 					*
	 ********************************************************/
	if ( lStartAddr == lEndAddr ) 
	{
		Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, START ADDRESS = END ADDRESS\n");
		sprintf(Out->ResultString,"ERROR; TDF2, START ADDRESS = END ADDRESS\n");
		mt2_pass_fail =Fail;
		return(Fail);
	}       

if(0)
{
	//Ensure start address is within speicified memory region for RAM1
	if((lStartAddr < RAM1START)|| (lStartAddr > (RAM1END)))
	{
		if((lStartAddr < RAM2START)|| (lStartAddr > (RAM2END)))
		{
			if((lStartAddr < RAM2START)|| (lStartAddr > (RAM2END)))
			{
				Out->PassFail = Fail;
				Out->ReturnCode = ExecutionFail;
				LogError(ERR_MEMORY_TEST_2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF2, START ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				sprintf(Out->ResultString,"ERROR: TDF2, START ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				mt2_pass_fail =Fail;
				return(Fail);
			}
		}
	}

	//Ensure end address is within speicified memory region for RAM1
	if((lEndAddr < RAM1START) || (lEndAddr >(RAM1END)))
	{
		if((lEndAddr < RAM2START) || (lEndAddr >(RAM2END)))
		{
			if((lEndAddr < RAM3START) || (lEndAddr >(RAM3END)))
			{
				Out->PassFail = Fail;
				Out->ReturnCode = ExecutionFail;
				LogError(ERR_MEMORY_TEST_2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF2, END ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				sprintf(Out->ResultString,"ERROR: TDF2, END ADDRESS OUTSIDE VALID MEMORY RANGE\n");
				mt2_pass_fail =Fail;
				return(Fail);
			}
		}
	}
 }
	/*****************************************************************
	 * Ensure that the number of patterns has been entered correctly *
	 *****************************************************************/  
    if(lNumberOfPatterns >0x2)
    {                
    	Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2,  NUMBER OF PATTERNS OUT OF RANGE\n");
		sprintf(Out->ResultString,"ERROR; TDF2, NUMBER OF PATTERNS OUT OF RANGE\n");
		mt2_pass_fail =Fail;
		return(Fail);
	}
	
	if(Protected(In->StartAddress,(In->EndAddress - In->StartAddress))) 
	{
		/**********************************************************************
		 * Sort the visionWare protected areas based on their start addresses *
		 **********************************************************************/
		for(lCount2 = 0; lCount2 < ProtectedAreas-1; lCount2++)
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++)
			{
				if(ProtectedArea[lCount+1].pFirst < ProtectedArea[lCount].pFirst)
				{
					ProtectedArea[lTemp].pFirst		= ProtectedArea[lCount].pFirst; 
					ProtectedArea[lTemp].pLast		= ProtectedArea[lCount].pLast;
					ProtectedArea[lCount].pFirst	= ProtectedArea[lCount+1].pFirst;
					ProtectedArea[lCount].pLast		= ProtectedArea[lCount+1].pLast;
					ProtectedArea[lCount+1].pFirst	= ProtectedArea[lTemp].pFirst; 
					ProtectedArea[lCount+1].pLast	= ProtectedArea[lTemp].pLast;
				}
			}
		}

		//assign defaults
		lStart[0]	= lStartAddr;
		lEnd[0]		= lEndAddr;


		/**************************************************************************************
		 * Check to see if the start address of the range to be tested resides in a Protected *
		 * region. If it is, find the region that it resides in, record the region and  reset *	
		 * the start address to the first free memory location after the region.              *
		 **************************************************************************************/
		if((Protected(lStartAddr,lWidth))!=0x0)
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++) 
			{
				if((lStartAddr >= (UINT32)ProtectedArea[lCount].pFirst)&(lStartAddr <= (UINT32)ProtectedArea[lCount].pLast))
				{
					lStart[0] = (UINT32)ProtectedArea[lCount].pLast+1;
					lStartRegion = lCount;
					if(lStart[0] >= (RAM1END))
					{
						if(lStart[0] >= (RAM2END))
						{
							return(Fail);
						}
					}
				}
			}
		}

		/**************************************************************************************
		 * Check to see if the end address  of the range to be tested  resides in a Protected *
		 * region. If it is, find the region that it resides in, record the region and  reset *	
		 * the end address to the first free memory location before the region.               *
		 **************************************************************************************/
		if((Protected(In->EndAddress,lWidth))!=0x0) 
		{
			for(lCount = 0; lCount < ProtectedAreas-1; lCount++)
			{
				if((In->EndAddress >= (UINT32)ProtectedArea[lCount].pFirst)&(In->EndAddress <= (UINT32)ProtectedArea[lCount].pLast))
				{
					lEnd[0] = (UINT32)ProtectedArea[lCount].pFirst-1;
					lEndRegion = lCount;
					//Check to see if we need to use RAM1START as the base address
					if((lEnd[0]>RAM1START)&&(lEnd[0]<RAM2START))
					{
						lEnd[0] = RAM1START;
					}
					//Range could be in either RAM2 or RAM3
					else if((lEnd[0]>RAM2START)&&(lEnd[0]<RAM3START))
					{
						lEnd[0] = RAM2START;
					}
					//Range must reside in RAM3START region
					else if(lEnd[0]>RAM3START)
					{
						lEnd[0] = RAM3START;
					}
				}
			}
		}

		/**********************************************************************************
		 * If start and stop are in the same protected block, print message and exit test *
		 **********************************************************************************/
		if(((lStartRegion != 0xffffffff) & (lEndRegion !=0xffffffff)))
		{

			if(lStartRegion == lEndRegion)
			{
				Out->PassFail = Fail;
				Out->ReturnCode = ExecutionFail;
				LogError(ERR_MEMORY_TEST_2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF2, ENTIRE MEMORY RANGE IS PROTECTED\n");
				sprintf(Out->ResultString,"ERROR: TDF2, ENTIRE MEMORY RANGE IS PROTECTED\n");
				return(Fail);
			}
		}
		else
		{
			lStartRegion = 0;
		}
	

		/****************************************************
		 * Assign the regions of memory that can be tested. *
		 ****************************************************/
		if(ProtectedAreas !=0x0)
		{
			lStartAddr = lStart[0];

			for(lCount = lStartRegion; lCount <= ProtectedAreas-1; lCount++)
			{
				if((lEndAddr > (UINT32)ProtectedArea[lCount+1].pFirst)&(lStart[lCount] < (UINT32)ProtectedArea[ProtectedAreas-1].pLast))  
				{
			   		lEnd[lCount] = (UINT32)ProtectedArea[lCount+1].pFirst;
			   		lStart[lCount+1] = (UINT32)ProtectedArea[lCount+1].pLast+1;
			 	}
				else
				{
					lEnd[lCount] = lEndAddr;
					lNumberOfTests = lCount;
					lCount = ProtectedAreas-1;
				}
			}
		}

		/**********************************************************************************
		 * Keep calling the memory test function until all of the assigned memory regions *
		 * have been tested.															  *
		 **********************************************************************************/
		Info("Requested range includes protected areas that will not be tested.\ntype mmap <cr> for details.\n\n");
		if(QuitEnable)
		{
			Info("Press 'q' at any time to abort test (test may take a few seconds to halt)\n");
		}
		for(lCount = 0x0 ;lCount <= lNumberOfTests; lCount++)
		{
			if(lStart[lCount] !=lEnd[lCount])
			{
				mt2_pass_fail = mt2_pass_fail | fnFloatOnesAndZeros(lStart[lCount], lEnd[lCount], lNumberOfPatterns, cScope);
				if(mt2_pass_fail == UserAbort)
				{
					lCount = lNumberOfTests;
				}
			}

	   	}
	}  			
	else
	{
		if(QuitEnable)
		{
			Info("Press 'q' at any time to abort test (test may take a few seconds to halt)\n");
		}
		if(lStartAddr !=lEndAddr)
			mt2_pass_fail = mt2_pass_fail | fnFloatOnesAndZeros(lStartAddr,lEndAddr,lNumberOfPatterns, cScope);
	}

   	if(mt2_pass_fail ==0x0)
	{
		Info("Success: TDF2, passed address range 0x%X .... 0x%X\n", In->StartAddress,In->EndAddress);
	}
	else
	{
		Out->PassFail = Fail;
		Out->ReturnCode = ExecutionFail;
		strcpy(Out->ResultString, ErrorCodes2.ResultString);
	}
	return(Out->PassFail);
}

/*************************************************************************************
* Function: fnFloatOnesAndZeros(UINT32, UINT32, UINT32, char)						 *
*																					 *
* Purpose:  The purpose of this function is to float either a '1' or a '1' and a '0' *
*           in turn through all of  the specified  memory locations depending on the *
*           variable "lNumberOfPatterns".											 *
*																					 *
* Returns:  UINT32 Pass / Fail value												 *
**************************************************************************************/
UINT32 fnFloatOnesAndZeros(UINT32 lStartAddress, UINT32 lEndAddress, UINT32 lNumberOfPatterns, char cScope)
{
	UINT32 lEndValue = 0xffffffff;
	UINT32 lStartValue = 0x0;
	UINT32 lData;
	UINT32 *pIndex;
	UINT32 lCount;
	UINT32 lPassFail = Pass;
	UINT32 lAddress;


	/**************************************************************
	 * Add sanity check to see that start address is within RAM   *
	 **************************************************************/
	 if(lStartAddress > RAM1END)
	{
		if(lStartAddress > RAM2END)
		{
			if(lStartAddress > RAM3END)
			{
				return(0);
			}
		}
	}
	for(lCount = 0x0; lCount < lNumberOfPatterns; lCount++)
	{
		Info("processing range 0x%X ... 0x%X with test pattern %X\n", lStartAddress, lEndAddress, lCount+1);
		//set up parameters depending on which test is to be performed
		if(lCount ==0x0)
		{
			lData = lStartValue;
		}
		else 
		{
			lData = 0xfffffff;
			lEndValue = 0;
		}
    
		//now test the memory
		do
		{
			if(lCount ==0x0)
			{
				lData = ((lData << 1) | ( 0x01010101));
			}
			else
			{
				lData = ((lData & 0x7F7F7F7F) << 1);
			}			
			//copy test pattern to all memory locations
			pIndex = (UINT32 *)lStartAddress;
			do
			{
			  *pIndex = lData;
			  pIndex++;
			}while(pIndex < (UINT32 *)lEndAddress);

			if(QuitEnable)
			{
		    //now check to see if key pressed
			    if(CharWaiting())
				{
					if(tolower(getchar()=='q'))
					{
						LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, TEST ABORTED BY USER\n");
						sprintf(ErrorCodes2.ResultString, "ERROR: TDF2, TEST ABORTED BY USER\n");
						lData = lEndValue;
						return(UserAbort);
					}
				}
			}
			
			//now check that it was written to all locations
			for(pIndex = (UINT32 *)lStartAddress; pIndex < (UINT32 *)lEndAddress; pIndex++)
			{
				if(*pIndex != lData)
				{
					lPassFail = Fail;
					LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",pIndex,*pIndex,lData);
					sprintf(ErrorCodes2.ResultString, "ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",pIndex,*pIndex,lData);
					if(cScope == 'y')
					{
						ExtScopeLoopSetup((UINT32)pIndex, lData);
					}
					pIndex = (UINT32 *)lEndAddress;
					return(Fail);
				}
			}
				
		}while (lData != lEndValue);

		//verify start address hasnt changed
		pIndex = (UINT32 *)lStartAddress;
		if(*pIndex != lData)
		{
			lPassFail = Fail;
			LogError(ERR_MEMORY_TEST_2,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA RE-VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%x\n",pIndex,*pIndex,lData);
			sprintf(ErrorCodes2.ResultString, "ERROR: TDF2, DATA RE-VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",pIndex,*pIndex,lData);
		}    
		
	}
	return(lPassFail);
}

Command(CMD_TDF2,"tdf2",TDF2, SIZESUFFIX|SILENTTYPE, "TDF2","complex ram test between two addresses");
Arg(UINT32ARG, "start address", "enter the start address of memory to be tested");
Arg(UINT32ARG, "end address", "enter the end address of memory to be tested");
OptArg(UINT32ARG, "number of patterns to perform (1 or 2)", "perform two tests on each address (1 or 2)",1);
OptArg(CHAR2ARG,"escape", "select ability to exit diagnostic whilst running (y or n)","n");
OptArg(CHAR4ARG,"scopeloop","select ability to loop on error (y or n)", "n");
Output(UINT32ARG, "PassFail");
Output(CHAR64ARG, "ResultString");
CommandEnd();



