/***************************************************************************
*
* Module: vdtdf3.c
*
* Description: Simple Continuous Memory Test
*
* Project:     EST VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/

//  Includes
#include "vm.h"
#include "esttype.h"
#include "cmd.h"
#include "lib.h"
#include "vware.h"
#include "vdtdf3.h"
#include "vderror.h"

//  Local Defines
#define Pass 0
#define Fail 1
#define ExecutionOk 0
#define ExecutionFail 0xffffffff
#define RunAgain 0x0f0f0f0f

//  External References

//  Forward References

//  Global Declarations

//  Local Declarations (statics)


////////////////////////////////////////////////////////////
// write_data_TDF3
//
// writes specified data to specified memory location based 
// on the memory access size (also specified).
// Width must be 1, 2,4 or 8 (byte, short, word, longword respectively)
// for this implementation.
// This code assumes that all values sent to it have already 
// been verified as valid.

void write_data_TDF3( UINT32 Width, UINT32 Addr, UINT32 Data)
{
	// write data has already been masked properly and width 
	// has already been verified to be valid

	universal p;			   

	p.uvalue = Addr;

	switch (Width)
	{
	case 1:
		*p.pByte = (UINT8)Data;
		break;

	case 2:
		*p.pHalf = (UINT16)Data;
		break;

	case 4:
		*p.pWord = (UINT32)Data;
		break;
	}
}

////////////////////////////////////////////////////////////
// read_data_TDF3
//
// reads data from specified memory location based 
// on the memory access size (also specified).
// Width must be 2,4 or 8 (short, word, longword respectively)
// for this implementation.
// This code assumes that all values sent to it have already 
// been verified as valid.

UINT32 read_data_TDF3( UINT32 Width, UINT32 Addr)
{
	union
	{
		UINT8 *sAddr;
		UINT16 *wAddr;
		UINT32 *lAddr;
		UINT32 Addr;
	} p;			   

	p.Addr = Addr;

	switch (Width)
	{
	case 1:
		return (UINT32)*p.sAddr;
	
	case 2:
		return (UINT32)*p.wAddr;
	
	case 4:
		return (UINT32)*p.lAddr;
	}
}

/***************************************************************************
*	DF_01_SRTest - Simple Memory Test
*
*		LOOP (thru all memory locations)
*			write AA, 55 or 66 to a memory location (repeat pattern every 3 bytes)
*			verify location just written
*			get next location
*		ENDLOOP
*		Verify memory at start location still holds original value: AA, (55 second pass)
*		Repeat above LOOP using these patterns in this order: 55, AA, 99
*		
*	INPUT:
*		All input parameters already setup in global TestParams struct
*
*	PARAMETERS used by this test:
*		Width
*		StartAddr
*		EndAddr
*
****************************************************************************/


int TDF3(TDF3_in *pIn, TDF3_out01 *pOut, unsigned long *pOutputStructureNumber)
{
  
   int ok_to_test=1;
   int TDF3_pass_fail =Pass;
   UINT32	PassInc;
   UINT32   EndAddr;
   UINT32   StartAddr; 
   UINT32	Data[6] = 	/* This definition of Data does not create relocatable code */
				{0xaaaaaaaa, 0x55555555, 0x99999999,
				 0x55555555, 0xaaaaaaaa, 0x66666666};
   UINT32   Width;
   UINT32	Addr;
   UINT32	RxData;
   UINT32	Ind = 0;
   UINT32   DataMask;
   UINT32   MemUnderTest;
   UINT32   TestValue;
   UINT32   Increment;
   UINT32   TestNumber = 1;

   ((TDF3_out02 *)pOut)->ReturnCode = Pass; // reset ReturnCode variable


  	SetOutputNumber(2);

	switch (Width = pIn->Width) 
	{
		case 4:
			DataMask = 0xffffffff;
			break;
		case 2:
			DataMask = 0xffff;
			break;
		case 1:		
			DataMask = 0xff;
			break; 
		default:
			TDF3_pass_fail =1;
			((TDF3_out02 *)pOut)->PassFail = Fail;
			((TDF3_out02 *)pOut)->ReturnCode = RunAgain; 
			LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, BAD WIDTH");
			sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, BAD WIDTH");
			return;
	}

	/**************************************************************/
	/* Mask Data Values for selected address width using DataMask */
	for( Ind = 0; Ind < 6; Ind ++)
		Data[Ind] = Data[Ind] & DataMask;


	StartAddr = pIn->Start;
	EndAddr   = pIn->End;

    
   
  
	/**********************************************/
	/* Insure start address is properly aligned   */
	/* for access size						      */
	if (StartAddr % Width != 0)
	{
		((TDF3_out02 *)pOut)->PassFail = Fail;
		((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, START ADDRESS ALLIGNMENT ERROR\n");
		sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, START ADDRESS ALLIGNMENT ERROR\n");
		TDF3_pass_fail =1;
		return;
	} 


	/**********************************************/
	/* Insure end address is properly aligned   */
	/* for access size						      */
	if (EndAddr % Width != 0)
	{
		((TDF3_out02 *)pOut)->PassFail = Fail;
		((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, END ADDRESS ALLIGNMENT ERROR\n");
		sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, END ADDRESS ALLIGNMENT ERROR\n");
		TDF3_pass_fail =1;
		return;
	} 

	
	
	
	/********************************************************/
	/* Insure Start address < End Address 					*/
	if ( EndAddr < StartAddr ) 
	{
		((TDF3_out02 *)pOut)->PassFail = Fail;
		((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, START ADDRESS > END ADDRESS\n");
		sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, START ADDRESS > END ADDRESS\n");
		TDF3_pass_fail =1;
		return;
	}

	SetOutputNumber(1);
	strcpy(pOut->Result, "fail");

	/*******************************************************/
	/* Set values for user messages						   */
	if ((pIn->End - pIn->Start)>0x100000)
	{
		TestValue = 0x100000;
		Increment = 0xFFFFF;
	}
	else if((pIn->End - pIn->Start)< 0xFFFF)
	{
		TestValue = pIn->End;
		Increment = (pIn->End - pIn->Start);
	}
	else
	{
		TestValue = 0x10000;
		Increment = 0xFFFF;
	}

	

	Ind     = 0;
	PassInc = 0;
	do  /* There will be two loops at this level, the first will write the */
	{	/* first three data patterns and the second writes the last three  */

		// First zero out the memory under test.
		Addr = StartAddr;
		MemUnderTest = Addr;
		while(MemUnderTest < EndAddr)
		{
			ok_to_test = Protected(MemUnderTest,Width);
			if(ok_to_test!=1)
			{

				write_data_TDF3(Width, MemUnderTest, 0);
			}
		  MemUnderTest += Width;
		}
		
		Addr = StartAddr;
		MemUnderTest = Addr;
#ifndef MINAGENT
		Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",Addr,Addr+Increment,TestNumber);
#endif
		
						
		do	/* This loop will loop thru all of the locations in the memory */
		{	/* area under test.											   */
			ok_to_test = Protected(MemUnderTest,Width);
			if(ok_to_test!=1)
			{
				if (pOut->Read = read_data_TDF3(Width, MemUnderTest))
				{
					pOut->Address = MemUnderTest;
					pOut->Expected = 0;
					((TDF3_out02 *)pOut)->PassFail = Fail;
					((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
					LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, READ ERROR AT ADDRESS %08X  EXPECTED 0\n", MemUnderTest);
					sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, READ ERROR AT ADDRESS %08X  EXPECTED 0\n", MemUnderTest);
					TDF3_pass_fail =1;
					return;
				}
			
				write_data_TDF3( Width, MemUnderTest, Data[Ind]);
				RxData = read_data_TDF3( Width, MemUnderTest);

				if (RxData != Data[Ind])
				{
					pOut->Address = MemUnderTest;
					pOut->Expected = Data[Ind];
					pOut->Read = RxData;
					((TDF3_out02 *)pOut)->PassFail = Fail;
					((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
					LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, RXDATA ERROR AT ADDRESS %08X  EXPECTED %08X  READ %08X\n", MemUnderTest, Data[Ind],RxData);
					sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, RXDATA ERROR AT ADDRESS %08X  EXPECTED %08X  READ %08X\n", MemUnderTest, Data[Ind],RxData);
					TDF3_pass_fail =1;
					return;
				}

			}
			/* Process Next memory Location	*/
			MemUnderTest = MemUnderTest + Width; 
			Ind = ((Ind +1) % 3) + PassInc;
#ifndef MINAGENT
		if(!(MemUnderTest % TestValue) & (MemUnderTest != EndAddr))
			Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",MemUnderTest,MemUnderTest+Increment,TestNumber);
#endif

		} while (MemUnderTest < EndAddr);
		
		/****************************************/
		/* Recover Start Address           		*/
		Addr = StartAddr;
		MemUnderTest = Addr;
		
		/****************************************/
		/* re-verify value at start address     */
		ok_to_test = Protected(MemUnderTest,Width);
		if(ok_to_test!=1)
		{

			RxData = read_data_TDF3( Width, Addr);

			if (RxData != Data[PassInc])
			{
				((TDF3_out02 *)pOut)->PassFail = Fail;
				((TDF3_out02 *)pOut)->ReturnCode = RunAgain;
				pOut->Address = Addr;
				pOut->Expected = Data[PassInc];
				pOut->Read = RxData;
				LogError(ERR_MEMORY_TEST_3,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF3, RXDATA ERROR AT ADDRESS %08X  EXPECTED %08X  READ %08X\n", MemUnderTest, Data[PassInc],RxData);
				sprintf(((TDF3_out02 *)pOut)->ResultString,"ERROR: TDF3, RXDATA ERROR AT ADDRESS %08X  EXPECTED %08X  READ %08X\n", Addr, Data[PassInc],RxData);
				TDF3_pass_fail =1;
				return;
			}
			
		}
				
		/****************************************************/
		/* Increment Pass offset and reset Ind Offset		*/
		PassInc = PassInc + 3;
		Ind = 3;

		TestNumber++;
		
	} while ( PassInc < 6 );
	SetOutputNumber(0);
    strcpy(pOut->Result, "pass"); // if it got to here it must have passed.
	TDF3_pass_fail = TDF3_pass_fail | 0;
   
	((TDF3_out02 *)pOut)->PassFail = Pass;                            
    ((TDF3_out02 *)pOut)->ReturnCode = RunAgain; 
	
#ifndef MINAGENT
	Info("Success: TDF3, passed address range 0x%08X .... 0x%08X\n", pIn->Start,pIn->End);
#endif				

   return (TDF3_pass_fail);
}

Command(CMD_TDF3, "tdf3", TDF3, SIZESUFFIX|HIDDEN, "TDF3", "simple ram test between two addresses (continuous)");
Arg(UINT32ARG,"start address", "enter the start address of the memory to be tested");
Arg(UINT32ARG, "end address", "enter the end address of the memory to be tested");
Output(UINT32ARG, "Return Code");
Output(CHAR8ARG, "Result");
NewOutput(UINT32ARG, "Return Code");
Output(CHAR8ARG, "Result");
Output(UINT32ARG, "Address");
Output(UINT32ARG, "Expected");
Output(UINT32ARG, "Read");
NewOutput(UINT32ARG, "Return Code");
Output(CHAR64ARG, "Error");
CommandEnd();