/***************************************************************************
*
* Module: vdtdf4.c
*
* Description:/
*
* Project:     EST VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/

//  Includes
#include "vm.h"
#include "esttype.h"
#include "cmd.h"
#include "lib.h"
#include "vdtdf4.h"
#include "vderror.h"

//  Local Defines
#define Fail		1
#define Pass		0
#define RUN			3

#define ExecutionOk 0
//#define RunAgain 0xffffffff // mtodd
#define RunAgain 0x0f0f0f0f

int TDF4(TDF4InParams *In, TDF4OutParams *Out)
{
  int ok_to_test = Fail;
  int mt2_pass_fail =Pass;
  UINT32	Data;
  UINT32	Addr;
  UINT32	RxData;
  UINT32	EndValue, StartValue;
  UINT32	Width;
  UINT32	StartAddr, EndAddr;
  UINT32	TestNumber = 1;
  UINT32    TestValue;
  UINT32    Increment;
 
 
  Out->Status = RUN;

  

  Width = 4;// always use LongWord regardless of user setting
  StartAddr = In->StartAddress;
  EndAddr   = In->EndAddress;
 	

   /*********************************************************/
	/* Insure start and end addresses are properly aligned   */
	/* for access size									     */

	if (StartAddr % Width != 0)
	{
		/*record_error (ADDR_NOT_ALIGNED, StartAddr, 
					  0, 0, Width, "Misaligned Start Address",
					  pTestParams, NO_RECOVER);*/
		Out->PassFail = Fail;
		Out->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF4, MISALIGNED START ADDRESS\n");
		sprintf(Out->ResultString,"ERROR: TDF4, MISALIGNED START ADDRESS\n");
		mt2_pass_fail =Fail;
		return;
	} 
	else if (EndAddr % Width != 0)
	{
		/*record_error (ADDR_NOT_ALIGNED, EndAddr, 
					  0, 0, Width, "Misaligned End Address",
					  pTestParams, NO_RECOVER);*/
		Out->PassFail = Fail;
		Out->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF4, MISALIGNED END ADDRESS\n");
		sprintf(Out->ResultString,"ERROR: TDF4, MISALIGNED END ADDRESS\n");
		mt2_pass_fail = Fail;
		return;
	}  


	/********************************************************/
	/* Insure End address > Start Address 					*/
	if ( EndAddr <= StartAddr ) 
	{
		/*record_error (START_END_ADDR_CONFLICT, StartAddr, 
					  EndAddr, 0, 0, 
					  "Start Addr greater than End Addr",
					  pTestParams, NO_RECOVER);*/
		Out->PassFail = Fail;
		Out->ReturnCode = RunAgain;
		LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR; TDF4, START ADDRESS > END ADDRESS\n");
		sprintf(Out->ResultString,"ERROR; TDF4, START ADDRESS > END ADDRESS\n");
		mt2_pass_fail =Fail;
		return;
	}


	/*******************************************************/
	/* Set values for user messages						   */
	if ((In->EndAddress - In->StartAddress)>0x100000)
	{
		TestValue = 0x100000;
		Increment = 0xFFFFF;
	}
	else if ((In->EndAddress -In->StartAddress)< 0xFFFF)
	{
		TestValue = In->EndAddress;
		Increment = (In->EndAddress - In->StartAddress);
	}
	else
	{
		TestValue = 0x10000;
		Increment = 0xFFFF;
	}

	/********************************************/
	/* First Clear Entire Area To Be Tested		*/
	
	Addr = StartAddr;
	do 
	{
		ok_to_test=Protected(Addr,Width);
		if (ok_to_test !=1)
		{
			*(UINT32 *)Addr = 0;		  
		}
		Addr+=Width;
	} while ( Addr < EndAddr);


	/********************************************/
	/* Float one through each memory location	*/
	
	EndValue = 0xFFFFFFFF;
	StartValue = 0; 

	Addr = StartAddr;

#ifndef MINAGENT
		Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",Addr,Addr+Increment, TestNumber);
#endif

	do	// Loop through each memory location in memory
	{	// region under test
		
		Data = StartValue; 
		do
		{
			ok_to_test=Protected(Addr,Width);
			if (ok_to_test !=1)
			{
				Data = ((Data << 1) | 0x01010101);
			
				*(UINT32 *)Addr = Data;				
				RxData = *(UINT8 *)Addr;
				
				RxData = *(UINT32 *)Addr;
				if (RxData != Data)
				{
					/*record_error (DATA_VERIFY_ERR, RxData, 
						Data, Addr, 0, "Data verify Error",
						pTestParams, RECOVER_OK);*/
					Out->PassFail = Fail;
					Out->ReturnCode = RunAgain;
					LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%x\n",Addr,RxData,Data);
					sprintf(Out->ResultString, "ERROR: TDF4, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",Addr,RxData,Data);
					mt2_pass_fail =Fail;
					return;
				}
			}
			else
			{
				Data = EndValue; // get out command to prevent endless loop if data is protected
		
			}
		}
		while (Data != EndValue); 

		/* Get next memory address	*/
		Addr = Addr + Width;
#ifndef MINAGENT
		if(!(Addr % TestValue) & (Addr != EndAddr))
			Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",Addr,Addr+Increment, TestNumber);
#endif

	} while ( Addr < EndAddr );

	TestNumber++;

	/* Verify data in StartAddr hasn't changed. */
	
	Addr = StartAddr;
	RxData = *(UINT32 *)Addr;
	
	ok_to_test=Protected(Addr,Width);
	if (ok_to_test !=1)
	{
		if (RxData != EndValue)
		{
			Out->PassFail = Fail;
			Out->ReturnCode = RunAgain;
			LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%x\n",Addr,RxData,Data);
			sprintf(Out->ResultString, "ERROR: TDF4, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",Addr,RxData,Data);
			mt2_pass_fail =1;
			return;
		}
	}


	/*** Float ZERO ***/

	EndValue   = 0;
	StartValue = 0xffffffff;

	Addr = StartAddr;

#ifndef MINAGENT
		Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",Addr,Addr+Increment, TestNumber);
#endif

	do	// Loop through each memory location in
	{	// region under test
		
		Data = StartValue; 
		do
		{
			ok_to_test=Protected(Addr,Width);
			if (ok_to_test !=1)
			{
				Data = ((Data & 0x7F7F7F7F) << 1);
				*(UINT32 *)Addr = (UINT32)Data;			
				RxData = *(UINT32 *)Addr;
			
				if (RxData != Data)
				{
					/*record_error (DATA_VERIFY_ERR, RxData, 
					Data, Addr, 0, "Data verify Error",
					pTestParams, RECOVER_OK);*/
					//((memory_test_1_out02 *)pOut)->PassFail = 1; //set as a fail
					Out->PassFail = Fail;
					Out->ReturnCode = RunAgain;
					LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%x\n",Addr,RxData,Data);
					sprintf(Out->ResultString, "ERROR: TDF4, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",Addr,RxData,Data);
				    mt2_pass_fail =1;
					return;
				}
			}
			else 
			{
				Data = EndValue; // get out command to prevent endless loop if data is protected
		
			}
		}
		while (Data != EndValue); 
		
		/* Get next memory address	*/
		Addr = Addr + Width;
		
#ifndef MINAGENT
		if(!(Addr % TestValue) & (Addr != EndAddr))
			Info("Processing address range 0x%08X ... 0x%08X with test pattern %d\n",Addr,Addr+Increment, TestNumber);
#endif

	} while ( Addr < EndAddr );
	
	/* Verify data in StartAddr hasn't changed. */
	Addr = StartAddr;
	RxData = *(UINT32 *)Addr;	
	
	ok_to_test=Protected(Addr,Width);
	if (ok_to_test !=1)
	{
		if (RxData != EndValue)
		{
			/*record_error (DATA_REVERIFY_ERR, RxData, 
			EndValue, StartAddr, 0, "Data re-verify Error",
			pTestParams, RECOVER_OK);*/
			Out->PassFail = Fail;
			Out->ReturnCode = RunAgain;
			LogError(ERR_MEMORY_TEST_4,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: TDF2, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%x\n",Addr,RxData,Data);
			sprintf(Out->ResultString, "ERROR: TDF4, DATA VERIFY ERROR AT 0x%08X, READ 0x%X, EXPECTED 0x%X\n",Addr,RxData,Data);
			Out->Status = Fail;
			mt2_pass_fail =1;
			return;
		}
	}
                   
    if (mt2_pass_fail ==0)
	{
		Out->Status = Pass;
#ifndef MINAGENT
		Info("Success: TDF4, passed address range 0x%08X .... 0x%08X\n", In->StartAddress,In->EndAddress);
#endif				

	}
	
  return (mt2_pass_fail);
}

Command(CMD_TDF4,"tdf4",TDF4, SIZESUFFIX|HIDDEN, "TDF4","complex ram test between two addresses (continuous)");
Arg(UINT32ARG, "start address", "enter the start address of memory to be tested");
Arg(UINT32ARG, "end address","enter the end address of memory to be tested"); 
Output(UINT32ARG, "PassFail");
Output(CHAR64ARG, "ResultString");
CommandEnd();
