/***************************************************************************
*
* Module: vdtdf5.c
*
* Description: Perform a CRC-16 test on memory
*
* Project:     EST VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/

//  Includes
#include "vm.h"
#include "cmd.h"
#include "vderror.h"

//  Local Defines
#define CRCCalc(a) ((a<<16)+(a<<15)+(a<<2)+1)  //TDF5 Macro definition
#define Pass 0
#define Fail 1
#define ExecutionOk 0
#define ExecutionFail 0xffffffff

/////////////////////////////////////////
// Input Structure for Diagnostic
/////////////////////////////////////////

typedef struct _TDF5In
{
	UINT32  Width; 
	UINT32	StartAddress;	// Enter the start address
	UINT32	EndAddress;	// Enter the end address
} TDF5In;

/////////////////////////////////////////
// Output Structure for Diagnostic
/////////////////////////////////////////

typedef struct _TDF5Out
{
	UINT32	PassFail;	// Holds the tests Pass Fail value, pass = 0
	UINT32	ReturnCode;	// Holds the tests return value
	char	ResultString[128];	// Holds the last message written by the routine
} TDF5Out;


/////////////////////////////////////////
// Definition for TDF5 Diagnostic
/////////////////////////////////////////

void TDF5(TDF5In* pIn, TDF5Out* pOut)
{
	UINT32		StartAddress;
	UINT32		CRCTestAddress;
	UINT32		EndAddress;
	UINT16		CRCValue =0;
	UINT8		*NewMemValue;
	UINT8		RValue = 0;
	
	StartAddress=pIn->StartAddress;
	EndAddress=pIn->EndAddress;
	
	/********************************************************/
	/*End Address < Start Address ?						*/
	if( EndAddress < StartAddress )
	{
		pOut->PassFail = Fail;
		pOut->ReturnCode = ExecutionFail;
		LogError(ERR_CRC_TEST,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: CRC TEST START ADDRESS > END ADDRESS\n");
		sprintf(pOut->ResultString,"ERROR: TDF5 TEST, START ADDRESS > END ADDRESS\n");
		return;
	}

	/********************************************************/
	/*End Address == Start Address ?						*/
	if( EndAddress == StartAddress )
	{
		pOut->PassFail = Fail;
		pOut->ReturnCode = ExecutionFail;
		LogError(ERR_CRC_TEST,CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE,"ERROR: CRC TEST START ADDRESS = END ADDRESS\n");
		sprintf(pOut->ResultString,"ERROR: TDF5 TEST, START ADDRESS = END ADDRESS\n");
		return;
	}

#ifndef MINAGENT		
	Info("CRC-16 test running\n");
#endif
	pOut->ReturnCode = ExecutionOk;

	CRCTestAddress = StartAddress;
	do
	{
		NewMemValue = (UINT8 *)CRCTestAddress;
		RValue = *NewMemValue;
		CRCValue = CRCValue + CRCCalc(RValue);
		CRCTestAddress++;
	} while ( CRCTestAddress <= EndAddress );

	sprintf(pOut->ResultString,"\nCompleted... CRC-16 Value = %4X\n",CRCValue);
	pOut->PassFail = Pass;
	pOut->ReturnCode = CRCValue;
#ifndef MINAGENT	
	  Info("%s",pOut->ResultString);
#endif

	return;
}

Command(CMD_TDF5,"tdf5",TDF5,SIZESUFFIX|REPEATABLE | SILENTTYPE,"TDF5","performs a crc-16 check on memory");
Arg(UINT32ARG,"startaddress","enter the start address");
Arg(UINT32ARG,"endaddress","enter the end address");
Output(UINT32ARG,"holds the tests pass fail value, pass = 0");
Output(UINT32ARG,"holds the tests return code value");
Output(CHAR64ARG,"ResultString");
CommandEnd();