/***************************************************************************
*
* Module: vdtdf6.c
*
* Description: Complex memory test that checks for wrap around on
*			   8 and 16 Megabyte boundaries.
*
* Project:     EST VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright (c) 1995, 2000, EST Corporation
*    This  program  is confidential and proprietary to EST Corporation
*	 and, without the  express  prior written permission of an officer
*	 of EST  Corporation, may not be  copied, reproduced, disclosed to
*	 others published  or used, in  whole  or in part, for any purpose
*	 other than that for which it is being made available.This program
*	 is protected pursuant to applicable copyright laws, including the
*	 U.S.  Copyright  Act. This   program  is  to  be  returned to EST
*	 Corporation  upon the earlier of demand and the accomplishment of
*	 the purpose for which it was provided.
****************************************************************************/
//  Includes
#include "vm.h"
#include "cmd.h" 
#include "vderror.h"    


//  Local Defines
/////////////////////////////////////////
// Input Structure for Diagnostic
/////////////////////////////////////////

typedef struct _TDF6In
{
	UINT32  Width;
	UINT32	MemoryStartAddress;	// Please enter the start address
	UINT32	MemoryEndAddress;	// Please enter the end address
	UINT32  MemBaseAddress;     // Please enter the lower memory boundary 
	UINT32  MemUpperBoundary;	// Please enter the upper memory boundary
	char Scope[4]; // Select ability to loop on error 'y' or 'n'
} TDF6In;

/////////////////////////////////////////
// Output Structure for Diagnostic
/////////////////////////////////////////

typedef struct _TDF6Out
{
	UINT32 PassFail; // Numerical value indicating Pass / Fail status
	UINT32 ReturnCode;
	char ResultString[128];   
	
} TDF6Out;

/////////////////////////////////////////
// Definition for TDF6 Diagnostic
/////////////////////////////////////////
#define Pass 0
#define Fail 1
#define ExecutionOk 0
#define ExecutionFail 0xffffffff

void TDF6(TDF6In* pIn, TDF6Out* pOut)
{
	UINT32 Width = 0x4; // set to long as a default
	UINT32 Start;
	UINT32 End;
	UINT32 MemMapStart;  
	UINT32 MemMapEnd;
	UINT32 BoundarySize	= 0x800000; // test on 8M boundaries
	UINT32 *MemoryPointer;
	UINT32 AddressPointer; 
	UINT32 AddressPointerCpy;
	UINT32 MemoryDataCpy;
	UINT32 GonePastZero = 0xF0000000; // take into account 8 and 16 meg
	UINT32 TestPattern;
	UINT32 TestPattern1 = 0xAAAAAAAA;
	UINT32 TestPattern2 = 0x55555555;
	UINT32 TestPattern3 = 0x99999999;
	UINT32 TestPattern4 = 0x66666666;
	UINT32 DataMask;
	
	int OkToWrite;
	int BoundaryProt;
	int TestCycle;
	char Scope;

	Start = pIn->MemoryStartAddress;
	End = pIn->MemoryEndAddress;
//	MemMapStart = pIn->MemBaseAddress; 
//	MemMapEnd = pIn->MemUpperBoundary; 
	MemMapStart = pIn->MemoryStartAddress; 
	MemMapEnd = pIn->MemoryEndAddress; 
	Scope = pIn->Scope[0];

	End-=0x4;
                
	pOut->PassFail = Pass; // set as a default value
	pOut->ReturnCode = ExecutionOk;


	// check that start address is not greater than end address
	if (Start > End)
	{
		LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, START ADDRESS > END ADDRESS\n");
	    pOut->PassFail = Fail;
		pOut->ReturnCode = ExecutionFail;
		sprintf(pOut->ResultString, "ERROR: TDF6, START ADDRESS > END ADDRESS\n");
		return;
	}

	// check that start address is evenly alligned
	if (Start % Width !=0)
	{
		LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, START ADDRESS NOT EVENLY ALIGNED\n");
		pOut->PassFail = Fail;
		pOut->ReturnCode = ExecutionFail;
		sprintf(pOut->ResultString, "ERROR: TDF6, START ADDRESS NOT EVENLY ALIGNED\n");
		return;
	}


	// check that end address is evenly alligned
	if (End % Width !=0)
	{
		LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, END ADDRESS NOT EVENLY ALIGNED\n");
		pOut->PassFail = Fail;
		pOut->ReturnCode = ExecutionFail;
		sprintf(pOut->ResultString, "ERROR: TDF6, END ADDRESS NOT EVENLY ALIGNED\n");
		return;
	}
		


	// zero all locations in memory to be tested (leave protected alone!)
	for(AddressPointer = Start; AddressPointer <= End; AddressPointer+=Width)
	{
		OkToWrite = Protected(AddressPointer,Width); //(one location at a time)
		if(!OkToWrite)
		{
		  MemoryPointer = (UINT32 *)AddressPointer;
		  *MemoryPointer = 0x0;
		}
	}// End of set memory to zero

     
// for loop here to repeat test for all combinations
	for (TestCycle = 0; TestCycle <=3; TestCycle++)
	{
		if (TestCycle ==0)
		{
			TestPattern = TestPattern1;
		}
		else if (TestCycle ==1)
		{
			TestPattern = TestPattern2;
		}
		else if (TestCycle ==2)
		{
			TestPattern = TestPattern3;
		}
		else if (TestCycle ==3)
		{
			TestPattern = TestPattern4;
		}
		// end of if loop to set test pattern values
		
		
   		for(AddressPointer = Start; AddressPointer <= End; AddressPointer+=Width)
   		{
   	 		AddressPointerCpy = AddressPointer;                
   	 	
   	 		// check if address is protected, if not write pattern, if it is leave alone.
   	 		OkToWrite = Protected(AddressPointerCpy,Width);
			if(!OkToWrite)
	 		{
				MemoryPointer = (UINT32 *)AddressPointerCpy;
				*MemoryPointer = TestPattern;
				MemoryDataCpy = TestPattern;
			}
			else
			{
				MemoryPointer = (UINT32 *)AddressPointerCpy;
				MemoryDataCpy = *MemoryPointer;
			}

		
			// now check to see test pattern was written correctly
			if (*MemoryPointer == MemoryDataCpy)
			{
		 		pOut->PassFail = Pass;
			}
			else
			{
				pOut->PassFail = Fail;
				pOut->ReturnCode = ExecutionFail;
				sprintf(pOut->ResultString, "ERROR: TDF6, DATA VERIFY ERROR AT ADDRESS 0x%X  READ 0x%X  EXPECTED 0x%X\n", MemoryPointer, MemoryDataCpy,TestPattern);
				LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, DATA VERIFY ERROR AT ADDRESS 0x%X  READ 0x%X  EXPECTED 0x%X\n", MemoryPointer, MemoryDataCpy,TestPattern);
				return;
			} // end of test if pattern written correctly
		



			// now check first lower 8 Meg Boundary   
			// add check to see if boundary value is protected.
			
			if(((AddressPointer-BoundarySize)>=MemMapStart)&((AddressPointer-BoundarySize)<GonePastZero))
			{
				AddressPointerCpy = (AddressPointerCpy-BoundarySize);
				BoundaryProt = Protected(AddressPointerCpy,AddressPointerCpy);
				MemoryPointer = (UINT32 *)AddressPointerCpy;
				if ((*MemoryPointer == MemoryDataCpy)&(MemoryDataCpy!=0x0)&(MemoryDataCpy!=0xFFFFFFFF)&(!BoundaryProt))
				{    
					LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6,TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					sprintf(pOut->ResultString, "ERROR: TDF6,TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					pOut->PassFail = Fail;
					pOut->ReturnCode = ExecutionFail;
					return;
				}
				AddressPointerCpy = AddressPointer; // restore pointer value for next test
				
			}	

			// now check lower 16 Meg Boundary
			if(((AddressPointer-(2*BoundarySize))>=MemMapStart)&((AddressPointer-(2*BoundarySize))<GonePastZero))
			{                                                      
				AddressPointerCpy = (AddressPointerCpy- (2*BoundarySize));    
				BoundaryProt = Protected(AddressPointerCpy,AddressPointerCpy);
				MemoryPointer = (UINT32 *)AddressPointerCpy;
			    if ((*MemoryPointer == MemoryDataCpy)&(MemoryDataCpy!=0x0)&(MemoryDataCpy!=0xFFFFFFFF)&(!BoundaryProt))
				{     
					LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					sprintf(pOut->ResultString, "ERROR: TDF6,TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					pOut->PassFail = Fail;
					pOut->ReturnCode = ExecutionFail;
					return;
				}
				AddressPointerCpy = AddressPointer; // restore pointer value for next test
				
			}

			// now check upper 8 Meg Boundary
			if((AddressPointer+BoundarySize)<=MemMapEnd)
			{     
				AddressPointerCpy = (AddressPointerCpy+BoundarySize);         
				BoundaryProt = Protected(AddressPointerCpy,AddressPointerCpy);
				MemoryPointer = (UINT32 *)AddressPointerCpy;
				if ((*MemoryPointer == MemoryDataCpy)&(MemoryDataCpy!=0x0)&(MemoryDataCpy!=0xFFFFFFFF)&(!BoundaryProt))
				{
					LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					sprintf(pOut->ResultString, "ERROR: TDF6,TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					pOut ->PassFail = Fail;
					pOut->ReturnCode = ExecutionFail;
					return;
				}
				AddressPointerCpy = AddressPointer; // restore pointer value for next test
				
			}


			// now check upper 16 Meg Boundary
			if((AddressPointer+(2*BoundarySize))<=MemMapEnd)
			{                                         
				AddressPointerCpy = (AddressPointerCpy+(2*BoundarySize));     
				BoundaryProt = Protected(AddressPointerCpy,AddressPointerCpy);
				MemoryPointer = (UINT32 *)AddressPointerCpy;				
				if ((*MemoryPointer == MemoryDataCpy)&(MemoryDataCpy!=0x0)&(MemoryDataCpy!=0xFFFFFFFF)&(!BoundaryProt))
				{
					LogError(ERR_MEMORY_TEST_5, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: TDF6, TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					sprintf(pOut->ResultString, "ERROR: TDF6,TEST PATTERN 0x%X AT ADDRESS 0x%X IS REPEATED AT ADDRESS 0x%X\n",MemoryDataCpy,AddressPointer,MemoryPointer);
					pOut ->PassFail = Fail;
					pOut->ReturnCode = ExecutionFail;
					return;
				}
				AddressPointerCpy = AddressPointer; // restore pointer value for next test
				
			}

			// now clear location to allow future boundary testing (if protected leave alone)
			OkToWrite = Protected(AddressPointerCpy,Width); //(one location at a time)
	 		if(!OkToWrite)
	 		{
				MemoryPointer = (UINT32 *)AddressPointer;
				*MemoryPointer = 0x0;
			}       
		
			
		}// end of current test cycle
	}// end of all test cycles  
#ifndef MINAGENT
	Info("Success: TDF6 Passed\n");
#endif
	
}// end of MT5

// User commands

Command(CMD_TDF6,"tdf6",TDF6,SIZESUFFIX |REPEATABLE | SILENTTYPE,"TDF6","memory diagnostic that also tests for wrap around on 8 and 16 megabyte boundarys");
Arg(UINT32ARG,"memorystartaddress","please enter the start address for test");
Arg(UINT32ARG,"memoryendaddress","please enter the end address for test");
Arg(UINT32ARG,"membaseaddress","please enter the lower memory boundary"); 
Arg(UINT32ARG,"memupperboundary","please enter the upper memory boundary");
OptArg(CHAR4ARG,"loop on error","select ability to loop on error 'y' or 'n'", "n");
Output(UINT32ARG,"PassFail");       
Output(CHAR64ARG,"ResultString");
CommandEnd();