/******************************************************************************
*
* Module:		pcihdr.h
*
* Description: GENERIC PCI header definitions (architecture non-specific)
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#ifndef __PCIHDR_H__
#define __PCIHDR_H__

/*-------------------------------------*/
/* Base Address Register (BAR) defines */
/*-------------------------------------*/
#define PCIHDR_BAR_IO				0x00000001
#define PCIHDR_BAR_MEM				0x00000000
#define PCIHDR_BAR_ADDR_TYPE_MASK	0x00000006
#define PCIHDR_BAR_ADDR_TYPE_32BIT	0x00000000
#define PCIHDR_BAR_ADDR_TYPE_1MEG	0x00000002
#define PCIHDR_BAR_ADDR_TYPE_64BIT	0x00000004
#define PCIHDR_BAR_PREFETCHABLE		0x00000008

/*-------------------------------------*/
/* Command Register defines            */
/*-------------------------------------*/
#define PCIHDR_CMDREG_IOSP		0x0001	/* Enable IO space accesses */
#define PCIHDR_CMDREG_MEMSP		0x0002	/* Enable MEM space accesses */
#define PCIHDR_CMDREG_MASTR		0x0004	/* Enable PCI Mastership */
#define PCIHDR_CMDREG_SPCYC		0x0008	/* Monitor special cycles */
#define PCIHDR_CMDREG_MEMWINV	0x0010	/* Enable memory write and invalidate */
#define PCIHDR_CMDREG_VGASNP	0x0020	/* Enable VGA palette snooping */
#define PCIHDR_CMDREG_PERR		0x0040	/* Enable Parity error response */
#define PCIHDR_CMDREG_STEP		0x0080	/* Enable address/data stepping */
#define PCIHDR_CMDREG_SERR		0x0100	/* Enable SERR driver */
#define PCIHDR_CMDREG_FSTB2B	0x0200	/* Enable back to back transactions */

#define PCIHDR_CLASS_NONE		0x00	/* Device has no class code */
#define PCIHDR_CLASS_MASS		0x01	/* Mass storage controller */
#define PCIHDR_CLASS_NETWORK	0x02	/* Network controller */
#define PCIHDR_CLASS_DISPLAY	0x03	/* Display controller */
#define PCIHDR_CLASS_MMEDIA		0x04	/* Multi-media controller */
#define PCIHDR_CLASS_MEMCTL		0x05	/* Memory controller */
#define PCIHDR_CLASS_BRIDGE		0x06	/* Bridge controller */
#define PCIHDR_CLASS_SCC		0x07	/* Simple communication controller */
#define PCIHDR_CLASS_PER		0x08	/* Base System Peripherals */
#define PCIHDR_CLASS_IN			0x09	/* Input Devices */
#define PCIHDR_CLASS_DOCK		0x0A	/* Docking Stations */
#define PCIHDR_CLASS_PROC		0x0B	/* Processors */
#define PCIHDR_CLASS_SBUS		0x0C	/* Serial Bus Controllers */
#define PCIHDR_CLASS_NOFIT		0xFF	/* Device doesn't fit any classcodes*/

#define PCIHDR_SUBCLASS_OTHER	0x80    /* Other subclass */

/* for class 0  (pre rev 2.0 of PCI)*/
#define PCIHDR_SUBCLASS_NONVGA	0x00	/* Non VGA device */
#define PCIHDR_SUBCLASS_VGADEV	0x01	/* VGA compatible device */

/* for class 1 (Mass Storage) */
#define PCIHDR_SUBCLASS_SCSI	0x00	/* SCSI */
#define PCIHDR_SUBCLASS_IDE		0x01	/* IDE  */
#define PCIHDR_SUBCLASS_FLOP	0x02	/* Floppy  */
#define PCIHDR_SUBCLASS_IPI		0x03	/* IPI */
#define PCIHDR_SUBCLASS_RAID	0x04	/* Raid Controller */

/* for class 2 (Network) */
#define PCIHDR_SUBCLASS_ETHER	0x00	/* Ethernet */
#define PCIHDR_SUBCLASS_TOKEN	0x01	/* Token */
#define PCIHDR_SUBCLASS_FDDI	0x02	/* FDDI */
#define PCIHDR_SUBCLASS_ATM		0x03	/* ATM Controller */

/* for class 3 (Display) */
#define PCIHDR_SUBCLASS_VGA		0x00	/* VGA */
#define PCIHDR_SUBCLASS_XGA		0x01	/* XGA */

/* for class 4 (Multimedia) */
#define PCIHDR_SUBCLASS_VIDEO	0x00	/* Video */
#define PCIHDR_SUBCLASS_AUDIO	0x01	/* Audio */

/* for class 5 (Memory Controller) */
#define PCIHDR_SUBCLASS_RAM		0x00	/* RAM */
#define PCIHDR_SUBCLASS_FLASH	0x01	/* FLASH */

/* for class 6 (Bridge Devices) */
#define PCIHDR_SUBCLASS_HOST	0x00	/* Host bridge */
#define PCIHDR_SUBCLASS_ISA		0x01	/* ISA */
#define PCIHDR_SUBCLASS_EISA	0x02	/* EISA */
#define PCIHDR_SUBCLASS_MC		0x03	/* MC */
#define PCIHDR_SUBCLASS_PCI		0x04	/* PCI */
#define PCIHDR_SUBCLASS_PCMCIA	0x05	/* PCMCIA */
#define PCIHDR_SUBCLASS_NBUS	0x06	/* PCI NuBus */
#define PCIHDR_SUBCLASS_CBUS	0x07	/* PCI CardBus */

/* for class 7 (Simple Communications Controllers) */
#define PCIHDR_SUBCLASS_SER		0x00	/* Serial Controller */
#define PCIHDR_SUBCLASS_PAR		0x01	/* Parallel Controller */

/* for class 8 (Base System Peripherals) */
#define PCIHDR_SUBCLASS_PIC		0x00	/* Interrupt Controller */
#define PCIHDR_SUBCLASS_DMA		0x01	/* DMA Controller */
#define PCIHDR_SUBCLASS_TIMER	0x02	/* Timer */
#define PCIHDR_SUBCLASS_RTC		0x03	/* RTC Controller */

/* for class 9 (Input Devices) */
#define PCIHDR_SUBCLASS_KEY		0x00	/* Keyboard Controller */
#define PCIHDR_SUBCLASS_PEN		0x01	/* Digitizer Pen */
#define PCIHDR_SUBCLASS_MOUSE	0x02	/* Mouse Controller */

/* for class 0xA (Docking Stations) */
#define PCIHDR_SUBCLASS_STN		0x00	/* Docking Station */

/* for class 0xB (Processors) */
#define PCIHDR_SUBCLASS_386		0x00	/* 386 processor */
#define PCIHDR_SUBCLASS_486		0x01	/* 486 processor */
#define PCIHDR_SUBCLASS_PENTIUM	0x02	/* Pentium processor */
#define PCIHDR_SUBCLASS_ALPHA	0x10	/* Alpha processor */
#define PCIHDR_SUBCLASS_POWERPC	0x20	/* PowerPC */
#define PCIHDR_SUBCLASS_COPROC	0x40	/* Co-processor */

/* for class 0xC (Serial Bus Controllers) */
#define PCIHDR_SUBCLASS_FIRE	0x00	/* Firewire */
#define PCIHDR_SUBCLASS_ACCESS	0x01	/* Access.bus */
#define PCIHDR_SUBCLASS_SSA		0x02	/* Serial Storage Architecture */
#define PCIHDR_SUBCLASS_USB		0x03	/* USB -- Universal Serial Bus */
#define PCIHDR_SUBCLASS_FIBRE	0x04	/* Fibre Channel */

typedef struct pci_classcode_data {
    UCHAR classcode;
    UCHAR *classtext;
}PCI_CLASSCODE_DATA;

static PCI_CLASSCODE_DATA class_initdata[]=
{
   {PCIHDR_CLASS_NONE, "Built before Class definitions"},
   {PCIHDR_CLASS_MASS, "Mass Storage Controller"},
   {PCIHDR_CLASS_NETWORK, "Network Controller"},
   {PCIHDR_CLASS_DISPLAY, "Display Controller"},
   {PCIHDR_CLASS_MMEDIA, "Multimedia Device"},
   {PCIHDR_CLASS_MEMCTL, "Memory Controller"},
   {PCIHDR_CLASS_BRIDGE, "Bridge Device"},
   {PCIHDR_CLASS_SCC, "Simple Communications Controller"},
   {PCIHDR_CLASS_PER, "Base System Peripheral"},
   {PCIHDR_CLASS_IN, "Input Device"},
   {PCIHDR_CLASS_DOCK, "Docking Station"},
   {PCIHDR_CLASS_PROC, "Processor"},
   {PCIHDR_CLASS_SBUS, "Serial Bus Controller"},
   {PCIHDR_CLASS_NOFIT, "Device not in defined Class Codes"}
};

#if 1
typedef volatile struct PCIHEADER
{
	UINT16	VendorID;
	UINT16	DeviceID;
	UINT16	CommandRegister;
	UINT16	StatusRegister;
	UINT8	RevisionID;
	UINT8	Interface;
	UINT8	SubClass;
	UINT8	BaseClass;
	UINT8	CacheLineSize;
	UINT8	LatencyTimer;
	UINT8	HeaderType;
	UINT8	BIST;
	UINT32	BAR0;
	UINT32	BAR1;
	UINT32	BAR2;
	UINT32	BAR3;
	UINT32	BAR4;
	UINT32	BAR5;
	UINT32	CardbusCISPointer;
	UINT16	SubsystemVendorID;
	UINT16	SubsystemID;
	UINT32	ExpansionROMBaseAddress;
	UINT32	Reserved[2];
	UINT8	InterruptLine;
	UINT8	InterruptPin;
	UINT8	Min_Gnt;
	UINT8	Max_Lat;
} PCIHEADER;
#else
typedef volatile struct {
    /* 0x00 */ USHORT  pci_vendor_id;     /* Vendor ID                 */
    /* 0x02 */ USHORT  pci_dev_id;        /* Device ID                 */

    /* 0x04 */ USHORT  pci_cmd_reg;       /* Command Register          */
    /* 0x06 */ USHORT  pci_dev_status;    /* Device Status             */

    /* 0x08 */ UCHAR   pci_rev_id;        /* Revision ID               */
    /* 0x09 */ UCHAR   pci_reg_prog_intf; /* Register programming intf */
    /* 0x0A */ UCHAR   pci_sub_class;     /* Sub-class of device       */
    /* 0x0B */ UCHAR   pci_base_class;    /* Base class of device      */

    /* 0x0C */ UCHAR   pci_cache_lnsize;  /* Cache line size           */
    /* 0x0D */ UCHAR   pci_lat_timer;     /* Latency timer             */
    /* 0x0E */ UCHAR   pci_hdr_type;      /* Header type               */
    /* 0x0F */ UCHAR   pci_bist;          /* Built in self test        */

    /* 0x10 */ ULONG   pci_bar00;         /* base address register 00  */
    /* 0x14 */ ULONG   pci_bar01;         /* base address register 01  */
    /* 0x18 */ ULONG   pci_bar02;         /* base address register 02  */
    /* 0x1C */ ULONG   pci_bar03;         /* base address register 03  */
    /* 0x20 */ ULONG   pci_bar04;         /* base address register 04  */
    /* 0x24 */ ULONG   pci_bar05;         /* base address register 05  */
    /* 0x28 */ ULONG   pci_cis_pointer;   /* Cardbus CIS Pointer       */

    /* 0x2C */ USHORT  pci_sub_vendor_id; /* Subsystem Vendor ID      */
    /* 0x2E */ USHORT  pci_subsystem_id;  /* Subsystem ID             */

    /* 0x30 */ ULONG   pci_eromba;        /* Expansion ROM base address*/
    /* 0x34 */ ULONG   rsvd01;            /* reserved                  */
    /* 0x38 */ ULONG   rsvd02;            /* reserved                  */

    /* 0x3C */ UCHAR   pci_intln;         /* Interrupt line            */
    /* 0x3D */ UCHAR   pci_intpin;        /* Interrupt pin             */
    /* 0x3E */ UCHAR   pci_min_gnt;       /* Burst Period len - 33 MHZ */
    /* 0x3F */ UCHAR   pci_max_lat;       /* Len of access to PCI bus  */
} PCI_CONFIG;
#endif

#define PCI_HEADERTYPE0		(0 << 16)
#define PCI_HEADERTYPE1		(1 << 16)

#define PCIMULTIFUNC		(1 << 23)
#define CLASS_PCI2PCI_BRIDGE	((PCIHDR_CLASS_BRIDGE << 8) | PCIHDR_SUBCLASS_PCI)

#define PCI_DEV_VEND		0	/* Device ID, Vendor ID */
#define PCI_STAT_COM		1	/* Status register, Command register */
#define PCI_CLASS_REV		2	/* Class code, Revision ID */
#define PCI_HEADER			3	/* Header type, etc. */
#define PCI_BASE_ADDR0		4	/* Base Address 0 */
#define PCI_BASE_ADDR1		5	/* Base Address 1 */
#define PCI_BASE_ADDR2		6	/* Base Address 2 */
#define PCI_BASE_ADDR3		7	/* Base Address 3 */
#define PCI_BASE_ADDR4		8	/* Base Address 4 */
#define PCI_BASE_ADDR5		9	/* Base Address 5 */
#define PCI_CISPTR			10	/* CardBus CIS Pointer */
#define PCI_SUBSYSIDS		11	/* Subsystem IDs */
#define PCI_EROMBA			12	/* Expansion ROM Base Address */
#define PCI_INT_BMSTR		15	/* Max_Lat, Min_Gnt, Interrupt Pin, Int Line */

#define PCI_BUS_RANGE		6	/* Sec. Latency, Sub. Bus, Sec. Bus, Pri. Bus */
#define PCI_STAT_IO_LO		7	/* Sec. status, I/O limit low, I/O base low */
#define PCI_MEM_HI			8	/* Memory limit and base */
#define PCI_PREMEM_LO		9	/* Prefetchable low base and low limit */
#define PCI_PREMEM_B64		10	/* Prefetchable base upper 32 bits */
#define PCI_PREMEM_L64		11	/* Prefetchable limit upper 32 bits */
#define PCI_IO_HI			12	/* I/O limit high, I/O base high */
#define PCI_BR_EROMBA		14	/* Expansion ROM Base Address */

#endif	/* #ifndef __PCIHDR_H__ */

