/* vSerialLib.c - boot serial driver */

/* Copyright 2001 Wind River Systems, Inc. */

/*
 * DESCRIPTION
 * This library is the driver entry point to connect from an external application.
 */

#define YES	1
#define NO	0
/* VWARE_PARAMS_BEGIN */
#define	vSERIAL_EXT_INTERFACE		NO		/* Expose API	CHOICE	NO|YES */
/* VWARE_PARAMS_END */

#if vSERIAL_EXT_INTERFACE
/* includes */

#define __VSERIALLIB_CPP__

#include "vm.h"
#include "esd.h"
#include "esdriver.h"
#include "vSerialLib.h"

/* defines */

#define	CONS_BUF_SIZE	32	/* characters buffer size */

/* externals */

extern tESDRIVER	Esdriver;
extern ESD			Esd;
extern void			(*esdCallback) (UINT8 *, UINT32);	/* read call back */
extern "C" void 	EnableInterrupts ();

/* locals */

/*
 * Only one device can be used at a time at now. Things have to be declared dynamically
 * to change this.
 */

static HANDLE			serialHdl;					/* serial device handle */
static UINT8			charBuf [CONS_BUF_SIZE];	/* local characters buffer */
static volatile int		next2Write = 0;		/* index to the next free character in the buffer */
static volatile int		next2Read = 0;		/* index to the next character to read */

/******************************************************************************
*
*  readHandler - process characters
*
* This routine is th high level ISR. It is called when characters are received.
*/

static void readHandler
    (
    UINT8 *	pBuffer,	/* read characters buffer */
    UINT32	chars		/* number of characters to read */
    )
    {
    int		ix;

	/* write characters to local buffer, check for overflow and increment next2Write */

    for (ix = 0; (ix < chars) && (next2Read != (next2Write + 1) % CONS_BUF_SIZE);
		 ix++, next2Write = (next2Write + 1) % CONS_BUF_SIZE)
		charBuf [next2Write] = *(pBuffer++);
    }
    
/******************************************************************************
*
* vSerialOpen - open a serial console channel
*
* This routine opens and initializes a console channel.
*/

extern "C" UINT32 vSerialOpen (void)
    {
    HANDLE serHandle;
    DSTATUS rc;
    
	esdCallback = readHandler;				/* initialize the call back */
	EnableInterrupts ();					/* manage interrupts if not done yet */
	rc = Esd.Open(&serHandle);
    return ((UINT32) serHandle);
    }

/******************************************************************************
*
* vSerialClose - close a serial console channel
*
* This routine closes a console channel.
*/

extern "C" int vSerialClose (void)
    {
    esdCallback = NULL;						/* reset the read call back */
    return (Esd.Close (Esd.hDevice));	/* reset the driver */
    }

/******************************************************************************
*
* vSerialRead - read a character on a serial console channel
*
* This routine reads a character on a serial console channel.
*
* RETURNS: the read character.
*/

extern "C" UINT8 vSerialRead (void)
    {
    UINT8	readChar;

	/* loop until there is something to read */

    while (next2Read == next2Write);

	readChar = charBuf [next2Read];					/* read the character */
	next2Read = (next2Read + 1) % CONS_BUF_SIZE;	/* increment the read index */
    return (readChar);
    }

/******************************************************************************
*
* vSerialWrite - write a character on a serial console channel
*
* This routine writes a character on a serial console channel.
*/

extern "C" int vSerialWrite
    (
    INT8	c							/* character to write */
    )
    {
    return (Esd.Write (Esd.hDevice, &c, 1));	/* write the character on the device */
    }

/******************************************************************************
*
* vSerialIoctl - IO control of the serial console channel
*
* This routine manages the serial console channel.
*/

extern "C" int vSerialIoctl
    (
    UINT32	funcCode,	/* function code */
    void *	pArgs		/* function arguments */
    )
    {
    return (Esd.Ioctl (Esd.hDevice, funcCode, pArgs));
    }

/******************************************************************************
*
* vSerialRxRdy - data to read
*
* This routine returns TRUE if there are data to read.
*
* RETURNS: TRUE if there is something to read, FALSE otherwise.
*/

extern "C" BOOL vSerialRxRdy (void)
    {
    return (next2Read != next2Write);	/* Buffer is not empty */
    }

/******************************************************************************
*
* vSerialTxRdy - data to transmit
*
* This routine returns TRUE if data can be sent.
*
* RETURNS: always TRUE.
*/

extern "C" BOOL vSerialTxRdy (void)
    {
    return TRUE;
    }

VWARELIB(SERIALLIB_OPEN,		vSerialOpen,	"UINT32 vSerialOpen(void)");
VWARELIB(SERIALLIB_CLOSE,		vSerialClose,	"int vSerialClose(void)");
VWARELIB(SERIALLIB_READCHAR,	vSerialRead,	"UINT8 vSerialRead(void)");
VWARELIB(SERIALLIB_WRITECHAR,	vSerialWrite,	"int vSerialWrite(INT8)");
VWARELIB(SERIALLIB_IOCTL,		vSerialIoctl,	"int vSerialIoctl(UINT32, void *)");
VWARELIB(SERIALLIB_RXRDY,		vSerialRxRdy,	"BOOL vSerialRxRdy(void)");
VWARELIB(SERIALLIB_TXRDY,		vSerialTxRdy,	"BOOL vSerialTxRdy(void)");

#endif
