/******************************************************************************
*
* Module: estack.h
*
* Description: Stack Module Definitions
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#ifndef __ESTACK_H__
#define __ESTACK_H__

enum
{
	SHELLVAR_MAC = MAKESHELLVAR(SHELLGROUP_ESTACK, 1),
	SHELLVAR_IP = MAKESHELLVAR(SHELLGROUP_ESTACK, 10),
	SHELLVAR_SUBNET,
	SHELLVAR_GATEWAY,
	SHELLVAR_REMOTENAME = MAKESHELLVAR(SHELLGROUP_ESTACK, 32),
	SHELLVAR_REMOTEIP
};

#define REMOTENAMELENGTH	32
#define REMOTEHOSTS			4

global IPADDR MyIPADDR;

typedef __packed__ union MAC
{
	UINT8  Bytes[6];
	UINT16 HalfWords[3];
} _PackedType MAC;

global MAC MyMAC;
global IPADDR SourceIPADDR;
global MAC SourceMAC;
global UINT16 SourcePort;

typedef __packed__ struct
{
	MAC  TargetMAC;
	MAC  SourceMAC;
	UINT16 Protocol;
	UINT8 Data;
} _PackedType ETHERNETPACKET;

typedef __packed__ struct
{
	UINT8  Size; 			// version in upper nybble
							// lower nybble is size of header in 32-bit words
	UINT8  TypeOfService; 	// type of service
	UINT16 PacketLength;	// packet length in bytes (includes header)
	UINT16 DatagramNumber;
	UINT16 Fragmentation;
	UINT8  TimeToLive;
	UINT8  Protocol;
	UINT16 HeaderChecksum;
	IPADDR SourceIPADDR;
	IPADDR TargetIPADDR;
	UINT8  Data;			// also start of options if necessary
} _PackedType IPPACKET;

typedef __packed__ struct
{
	UINT16 HardwareType;	// 0001 = 10Mb Ethernet
	UINT16 Protocol;		// must be 0x0800 to indicate IP
	UINT8 AddressLengthMAC;	// must be 6
	UINT8 AddressLengthIP;	// must be 4 for IP v4
	UINT16 Opcode;			// 0001 = request, 0002 = reply
	MAC SourceMAC;
	IPADDR SourceIPADDR;
	MAC TargetMAC;
	IPADDR TargetIPADDR;
} _PackedType ARPPACKET;

typedef __packed__ struct
{
	UINT8 Type;				// 0x00 = echo reply
							// 0x08 = echo request
							// 0x0D = timestamp request
							// 0x0E = timestamp response
	UINT8 Code;
	UINT16 Checksum;
	UINT8 Data;
} _PackedType ICMPPACKET;

typedef __packed__ struct
{
	UINT16 SourcePort;
	UINT16 TargetPort;
	UINT16 Length;
	UINT16 Checksum;
	UINT8 Data;
} _PackedType UDPPACKET;

// TCP Header Flags
#define TCP_URGENTDATA		0x20
#define TCP_VALIDACK		0x10
#define TCP_PUSHREQUEST		0x08
#define TCP_RESETSESSION	0x04
#define TCP_SYNCHSEQUENCE	0x02
#define TCP_FINALDATASENT	0x01

typedef __packed__ struct
{
	UINT16 SourcePort;
	UINT16 TargetPort;
	UINT32 SeqNumber;
	UINT32 AckNumber;
	UINT8 HeaderLength;			// in upper nybble, measured in 32-bit words
	UINT8 Flags;				// see TCP_xxxxx flags above
	UINT16 WindowSize;
	UINT16 Checksum;
	UINT16 UrgentPointer;
} _PackedType TCPPACKET;

//typedef __packed__ struct ESTACKEVENT
typedef struct ESTACKEVENT
{
	struct ESTACKEVENT *pNext;
	UINT16 TargetPort;
	UINT8 Protocol;
	UINT8 Flags;
	void (*Vector)(UINT32, void *, UINT32);
	UINT32 self;
//} _PackedType ESTACKEVENT;
} ESTACKEVENT;

global ESTACKEVENT *pEstackEventHead;

//#define ESTACK_FLAGS_ONESHOT	0x01

#endif		// #ifndef __ESTACK_H__