/******************************************************************************
*
* Module: ip.h
*
* Description: IP Module Definitions
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#ifndef __IP_H__
#define __IP_H__

#include "estack.h"
#include "eed.h"
#include "daemon.h"
#include "timer.h"

extern "C" UINT32 ComputeIPChecksum(void *p, UINT32 Length, UINT32 Seed, UINT32 OddFlag);
extern "C" UINT32 ComputeIntermediateChecksum(void *p, UINT32 Length, UINT32 Seed, UINT32 OddFlag);
extern "C" unsigned int ComputePseudoChecksum(IPADDR, IPADDR, UINT8, UINT16);

typedef struct _DEFRAG
{
	UINT16 DatagramNumber;
	UINT16 offset;
	UINT16 DatagramSize;
	UINT8  Protocol;
	IPADDR SourceIPADDR;
	IPADDR TargetIPADDR;	
	struct _DEFRAG *next;
	UINT8 *Buffer;
} DEFRAG;

//  Fragmentation macros
#define IP_DF		0x4000			//  Don't fragment bit
#define IP_MF		0x2000			//  More fragments bit
#define IP_OFFSET	0x1FFF			//  Fragmentation offset
#define MAX_DATAGRAM_SIZE 65514

#define LocalMAC	MyMAC
#define LocalIP		MyIPADDR

class IPSocket : public daemon
{
protected:
	UINT16 LocalPort;
	UINT16 RemotePort;
//	IPADDR LocalIP;				// global variable - make local for multi-homing
	IPADDR RemoteIP;
//	MAC LocalMAC;				// global variable
	MAC RemoteMAC;
	UINT8 TOS;
	UINT8 TTL;
	static UINT16 DatagramNumber;
	IPPACKET *pIP;
	ETHERNETPACKET *pEP;
	UINT32 TotalFragmentLength;
	DEFRAG *DefragPtr;
	DEFRAG *DefragBase;

    UINT8 ip_reass(UDPPACKET **pUDP); 
	void FreeFragments(void);
	void UpdateFragmentNode(UINT32 PacketLength, UINT8 HeaderLength);

public:
	IPSocket(UINT8 Protocol, UINT16 TargetPort, UINT8 Flags);

	void *GetIpBuffer(UINT32 Length = 0, void *pOptions = 0, UINT32 OptionsLength = 0);
	void Vector(void *v, UINT32 PacketLength);
	void Send(UINT8 *pData1 = 0, UINT32 Length1 = 0, UINT8 *pData2 = 0, UINT32 Length2 = 0);
	virtual void Receive(void *v, UINT32 Length) = 0;
	static void ArpVector(IPSocket *self, void *v, UINT32 Length);
	static void ArpTimeout(IPSocket *self, TIMEOUT *p);
	UINT32 ARP(void);
	static TIMEOUT ArpTimer;
	static ESTACKEVENT ArpEvent;
	static volatile UINT32 ArpState;
	void Unreachable(int Code);
    
};

#endif			// __IP_H__
