/* vNetLib.c - boot ethernet driver */

/* Copyright 2001 Wind River Systems, Inc. */

/*
 * DESCRIPTION
 * This library is the ethernet driver entry point to connect from an external application.
 */

#define YES	1
#define NO	0
/* VWARE_PARAMS_BEGIN */
#define	vNET_EXT_INTERFACE		NO		/* Expose API	CHOICE	NO|YES */
/* VWARE_PARAMS_END */

#if vNET_EXT_INTERFACE

#define __VNETLIB_CPP__

/* includes */

#include "vm.h"
#include "eedriver.h"
#include "eed.h"
#include "vNetLib.h"

/* defines */

#define	NET_BUF_SIZE	1536	/* packet buffer size */
#define	NET_BUFS		25		/* numbers of packets */

/* externals */

extern EED			Eed;
extern void			(*eedCallback) (UINT8 *, UINT32);	/* read call back */
extern "C" void		EnableInterrupts ();
extern "C" int		memcmp (void * s1, void * s2, int n);

/* typedefs */

typedef struct	netbuf
	{
	UINT32		length;					/* number of bytes written in the buffer */
	UINT8		netBuf [NET_BUF_SIZE];	/* local net buffer */
	}	NETBUF;

/*
 * At now, only one ethernet device is managed since everything is declared statically.
 */

static HANDLE		ethHdl;					/* ethernet device handle */
static NETBUF		bufArray [NET_BUFS];	/* local net buffer array */
static int			next2Write = 0;			/* index to the next free buffer in the array */
static int			next2Read = 0;			/* index to the next buffer to read */

/******************************************************************************
*
*  readHandler - process packets
*
* This routine is the high level ISR. It is called when packets are received.
*
* RETURNS: NA
*/

static void readHandler
    (
    UINT8 *	pBuffer,	/* data buffer to read */
    UINT32	length		/* data length */
    )
    {
	/* check for packet length */

	if (length > NET_BUF_SIZE)
		return;

    /* check if this packet is for us */

	if (memcmp (&(((ETHERNETPACKET *) pBuffer)->TargetMAC), &MyMAC, 6) != 0)
		return;

	/* check for a free buffer */

	if (bufArray [next2Write].length == 0)
		{
		memcpy (bufArray [next2Write].netBuf, pBuffer, length);
		bufArray [next2Write].length = length;
		next2Write = (next2Write + 1) % NET_BUFS;
		}
    return;
    }
    
/******************************************************************************
*
* vNetOpen - open an ethernet handle
*
* This routine opens and initializes an ethernet handle.
*
* RETURNS: Device descriptor
*/

extern "C" UINT32 vNetOpen (void)
    {
    HANDLE ethHandle;
    DSTATUS rc;
    
	eedCallback = readHandler;				/* initialize the call back */
	EnableInterrupts ();					/* manage interrupts if not done yet */
	rc = Eed.Open(&ethHandle);
    return ((UINT32) ethHandle);
    }

/******************************************************************************
*
* vNetClose - close an ethernet connection
*
* This routine closes an ethernet connection.
*
* RETURNS: OK always
*/

extern "C" int vNetClose (void)
    {
    eedCallback = NULL;						/* reset the read call back */
    return (Eed.Close (Eed.hDevice));	/* reset the driver */
    }

/******************************************************************************
*
* vNetRead - read a network packet
*
* This routine reads a network packet.
*
* RETURNS: number of read characters, 0 if nothing to read.
*/

extern "C" int vNetRead
	(
	UINT8 *	pBuf,	/* storage buffer */
	UINT32	length	/* number of characters to read */
	)
    {
	UINT32		lenMin;

	/* check for full buffer */

	if (bufArray [next2Read].length == 0)
		return 0;	/* nothing to read */

	/* store buffer and free it */

	lenMin = min (length, bufArray [next2Read].length);
	memcpy (pBuf, bufArray [next2Read].netBuf, lenMin);

	bufArray [next2Read].length = 0;		/* indicate the buffer is free */
	next2Read = (next2Read + 1) % NET_BUFS;	/* increment the read index */
	return lenMin;
    }

/******************************************************************************
*
* vNetWrite - write a packet on network
*
* This routine writes a packet on network.
*
* RETURNS: OK always
*/

extern "C" int vNetWrite
    (
    UINT8 *	pBuf,	/* buffer to write */
	INT32	length	/* number of bytes to write */
    )
    {
	Eed.Send (pBuf, length);	/* write the packet */
	return (STATUS_OK);
	}

/******************************************************************************
*
* vNetIoctl - IO control of the network device
*
* This routine manages the network device.
*
* RETURNS: OK or an ERROR if IOCL command is not available
*/

extern "C" int vNetIoctl
    (
    UINT32		funcCode,	/* function code */
    void *		pArgs		/* function arguments */
    )
    {
    return Eed.Ioctl (Eed.hDevice, funcCode, pArgs);
    }

VWARELIB(NETLIB_OPEN,	vNetOpen,	"UINT32 vNetOpen(void)");
VWARELIB(NETLIB_CLOSE,	vNetClose,	"int vNetClose(void)");
VWARELIB(NETLIB_READ,	vNetRead,	"int vNetRead(UINT8 *, UINT32)");
VWARELIB(NETLIB_WRITE,	vNetWrite,	"int vNetWrite(UINT8 *, INT32)");
VWARELIB(NETLIB_IOCTL,	vNetIoctl,	"int vNetIoctl(UINT32, void *)");
#endif
