/******************************************************************************
*
* Module:		ffs.h
*
* Description:	FFS Module Definitions
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#ifndef __FFS_H__
#define __FFS_H__

//  This file is used by both vxWorks and visionWare
#ifndef INCLUDE_VWARE_LAUNCH

#include "flash.h"

enum
{
	SHELLVAR_SECTORS = MAKESHELLVAR(SHELLGROUP_FFS, 1)
};
#endif

typedef struct
{
	char Signature[6];
	UINT16 ID;				// ID of -1 (0xFFFF) indicates the erase block
	UINT16 Sectors;
} FFSBLOCK;

#if 0
typedef struct
{
	UINT32 TypeLength;		// 8 bits type, 24 bits length
	UINT32 Next;			// block/offset encoded.  -1 = end of chain
} FFSFRAGMENT;
#endif

#define MINFRAGSIZE			0x20
#define Roundup(x)	(((x) + (MINFRAGSIZE - 1)) & ~(MINFRAGSIZE - 1))
#define FFSMAXBLOCKS		128
#define FFSMINBLOCKS		2
#define LENGTHMASK			0xFFFFFF
#define TYPEMASK			0xFF
#define LENGTHBITS			24

#define FFSTYPE_FREE		0xFF
#define FFSTYPE_FRAGMENT	0x0F
#define FFSTYPE_FILEHEADER	0xF0
#define FFSTYPE_DELETED		0x00
#define FFSTYPE_INVALID		0xAA

typedef struct				// FFS::FindFreeSpace takes a pointer to an FFSFFS
{							// structure and completes it.
	UINT32 Length;
	UINT8 *Address;
	UINT32 BlockOffset;
} FFSFFS;

#include "vTypes.h"

typedef struct FFSHANDLE
{
	struct FFSHANDLE *pNext;
	UINT32 Header;			// block/offset encoded position of file header
	UINT32 Fragment;		// block/offset encoded position of current fragment
	UINT32 FragmentOffset;	// offset into current fragment (for reading)
	UINT32 FileOffset;		// offset into file (for reading)
} FFSHANDLE;

#define FFSERASEBLOCKID	0xFFFF

#ifndef INCLUDE_VWARE_LAUNCH
#ifdef __cplusplus
class FFS
{
private:
	FFSBLOCK *pFirstBlock;
	UINT16 Status;
	UINT8 OffsetBits;
	UINT8 LastError;
	UINT32 OffsetMask;
	UINT32 BlockMask;
	UINT8 *Blocks[FFSMAXBLOCKS];
	HLIST FFSHANDLES;
	FLASHOBJECT *pFO;
	UINT32 (*WriteVector)(volatile UINT8 *Dest, UINT8 *Source, UINT32 Length);
	UINT32 (*EraseVector)(volatile UINT8 *SectorAddress);
	UINT32 ValidateFileSystem(void);
	UINT32 FindFreeSpace(FFSFFS *pFFS, UINT32 SizeNeeded, UINT32 SizeWanted);
	UINT32 FlashWrite(UINT8 *pDest, UINT8 *pSource, UINT32 Length);
	UINT32 FlashEraseSector(UINT8 *pSector);
	UINT32 _DeleteFile(FFSFRAGMENT *pFragment);

public:
	UINT32 BlockSize;
	UINT16 NumberOfBlocks;
	FFS(FLASHOBJECT *pFO);
	UINT32 Format(void);
	HANDLE CreateFile(char *Name, UINT32 Attributes, UINT32 InitialIP, UINT32 LoadAddress);
	HANDLE OpenFile(char *Name);
	UINT32 FindFirst(FFSFF *pFF, char *Name);
	UINT32 FindNext(FFSFF *pFF, char *Name, UINT32 SkipFirst=1);
	UINT32 CloseFile(HANDLE pHandle);
	UINT32 AbortFile(HANDLE pHandle);
	UINT32 ReadFile(HANDLE pHandle, UINT32 FileOffset, UINT8 *pBuffer, UINT32 Length, UINT32 *BytesRead);
	UINT32 WriteFile(HANDLE pHandle, UINT8 *pBuffer, UINT32 Length, UINT32 *BytesWritten);
	UINT32 DeleteFile(char *Name);
	UINT32 GetLastError(void);
	UINT32 DecodeBlock(UINT32);
	UINT32 DecodeOffset(UINT32);
	UINT32 Encode(UINT32 block, UINT32 offset);
	UINT32 Filesize(HANDLE pHandle);
	FFSFRAGMENT *Decode(UINT32);
	UINT32 GarbageCollection(void);
	void FfsResult(char *pFilename);
};
#endif		// #ifdef __cplusplus
#endif

#ifdef INCLUDE_VWARE_LAUNCH
#define global
#endif

// APIGUI
global UINT32 Format(void);
global HANDLE CreateFile(char *Name, UINT32 Attributes, UINT32 InitialIP, UINT32 LoadAddress);
global HANDLE OpenFile(char *Name);
global UINT32 FindFirst(FFSFF *pFF, char *Name);
global UINT32 FindNext(FFSFF *pFF, char *Name);
global UINT32 CloseFile(HANDLE pHandle);
global UINT32 AbortFile(HANDLE pHandle);
global UINT32 ReadFile(HANDLE pHandle, UINT32 FileOffset, UINT8 *pBuffer, UINT32 Length, UINT32 *BytesRead);
global UINT32 WriteFile(HANDLE pHandle, UINT8 *pBuffer, UINT32 Length, UINT32 *BytesWritten);
global UINT32 DeleteFile(char *Name);
global UINT32 GetLastError(void);
global FFSFRAGMENT *Decode(UINT32 x);
global UINT32 BlockSize(void);
global UINT32 NumberOfBlocks(void);
global void FfsResult(char *pFilename);
global UINT32 Filesize(HANDLE pHandle);
// APIGUIEND

#endif
