/******************************************************************************
*
* Module:		volume1.cpp
*
* Description:	Functions for a specific instance (volume1) of an FFS object
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"
#include "ffs.h"
#include "shell.h"

//  External References
extern FLASHOBJECT FFSFLASHOBJECT;

//  Global Declarations
FFS Volume1(&FFSFLASHOBJECT);

/***************************************************************************
* Function:	FileSize(HANDLE pHandle);
*
* Purpose:	Determine the size of an open file.
*
* Returns:	UINT32 FileSize
****************************************************************************/
extern "C" UINT32 Filesize(HANDLE pHandle)
{
	Volume1.Filesize(pHandle);
}

/***************************************************************************
* Function:	FfsResult(char *pFilename);
*
* Purpose:	Display the formatted results of the last FFS operation.
*
* Returns:	void
****************************************************************************/
extern "C" void FfsResult(char *pFilename)
{
	Volume1.FfsResult(pFilename);
}

/***************************************************************************
* Function:	Format(void);
*
* Purpose:	format the filesystem on volume 1
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 Format(void)
{
	return Volume1.Format();
}

/***************************************************************************
* Function:	CreateFile(char *Name, UINT32 Attributes, UINT32 InitialIP,
*					UINT32 LoadAddress);
*
* Purpose:	Create a file on volume 1
*
* Returns:	HANDLE to the created file
****************************************************************************/
extern "C" HANDLE CreateFile(char *Name, UINT32 Attributes,
		UINT32 InitialIP, UINT32 LoadAddress)
{
	return Volume1.CreateFile(Name, Attributes, InitialIP, LoadAddress);
}

/***************************************************************************
* Function:	OpenFile(char *Name);
*
* Purpose:	Opan a file on volume 1
*
* Returns:	HANDLE to the open file
****************************************************************************/
extern "C" HANDLE OpenFile(char *Name)
{
	return Volume1.OpenFile(Name);
}

/***************************************************************************
* Function:	FindFirst(FFSFF *pFF, char *Name);
*
* Purpose:	Find the first instance of a file with the specified name
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 FindFirst(FFSFF *pFF, char *Name)
{
	return Volume1.FindFirst(pFF, Name);
}

/***************************************************************************
* Function:	FindNext(FFSFF *pFF, char *Name);
*
* Purpose:	Find the next instance of a file with the specified name
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 FindNext(FFSFF *pFF, char *Name)
{
	return Volume1.FindNext(pFF, Name, 1);
}

/***************************************************************************
* Function:	CloseFile(HANDLE pHandle);
*
* Purpose:	Close an open file
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 CloseFile(HANDLE pHandle)
{
	return Volume1.CloseFile(pHandle);
}

/***************************************************************************
* Function:	AbortFile(HANDLE pHandle);
*
* Purpose:	Close and delete an open file
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 AbortFile(HANDLE pHandle)
{
	return Volume1.AbortFile(pHandle);
}

/***************************************************************************
* Function:	ReadFile(HANDLE pHandle, UINT32 FileOffset, UINT8 *pBuffer,
*					UINT32 Length, UINT32 *BytesRead);
*
* Purpose:	Read from an open file
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 ReadFile(HANDLE pHandle, UINT32 FileOffset,
		UINT8 *pBuffer, UINT32 Length, UINT32 *BytesRead)
{
	return Volume1.ReadFile(pHandle, FileOffset, pBuffer, Length, BytesRead);
}

/***************************************************************************
* Function:	WriteFile(HANDLE pHandle, UINT32 FileOffset, UINT8 *pBuffer,
*					UINT32 Length, UINT32 *BytesWritten);
*
* Purpose:	Append to an open file
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 WriteFile(HANDLE pHandle, UINT8 *pBuffer,
		UINT32 Length, UINT32 *BytesWritten)
{
	return Volume1.WriteFile(pHandle, pBuffer, Length, BytesWritten);
}

/***************************************************************************
* Function:	DeleteFile(char *Name);
*
* Purpose:	Delete a file
*
* Returns:	UINT32 status
****************************************************************************/
extern "C" UINT32 DeleteFile(char *Name)
{
	return Volume1.DeleteFile(Name);
}

/***************************************************************************
* Function:	GetLastError(void);
*
* Purpose:	Returns the error code of the last error that occurred on volume 1
*
* Returns:	UINT32 ErrorCode
****************************************************************************/
extern "C" UINT32 GetLastError(void)
{
	return Volume1.GetLastError();
}

/***************************************************************************
* Function:	Decode(UJINT32 x);
*
* Purpose:	Convert an encoded FFS address into a physical address
*
* Returns:	FFSFRAGMENT *
****************************************************************************/
extern "C" FFSFRAGMENT *Decode(UINT32 x)
{
	return Volume1.Decode(x);
}

/***************************************************************************
* Function:	BlockSize(void);
*
* Purpose:	Determine the block size of the filesytem on volume 1
*
* Returns:	UINT32 BlockSize
****************************************************************************/
extern "C" UINT32 BlockSize(void)
{
	return Volume1.BlockSize;
}

/***************************************************************************
* Function:	NumberOfBlocks(void);
*
* Purpose:	Determine the number of blocks allocated to the filesystem on
*			volume 1
*
* Returns:	UINT32 Blocks
****************************************************************************/
extern "C" UINT32 NumberOfBlocks(void)
{
	return Volume1.NumberOfBlocks;
}

ShellAction(SHELLFLAG_FORMAT, "reformat", SHELLACTION_BEFORESAVING, Format);

Component("FFS", COMPONENT_FFS, Volume1);
Region("FFS", REGION_FFSFLASH, FFSFLASHSTART, FFSFLASHEND);
