// Intel 28F128J3 BITS:8 BUS:8 ACCESS:INTEGER

/* 
 * Copyright 2002 Wind River Systems, Inc.
 * Copyright 2003 Intel Corp.
 */
/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Baseline/Flash/Intel_28F128J3_N_8_8_I.c 1     3/05/04 2:54p Ajp $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Baseline/Flash/Intel_28F128J3_N_8_8_I.c $
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 5     10/29/03 1:40p Tepulkow
 * Increased the flash write timeout by 4X due to flash chip spec update
 * that indicates the programming time to be TBD.  4X the oringinal
 * timeout value worked in the lab so the value was doubled again!
 * 
 * 2     8/21/03 8:00a Daly
 * added vss tags, modification history
 *
 * Date		By  Comment
 * -------	--- -------
 * 4/30/03 	JJD Fixed timeout issue with large block writes
 * 5/20/03  JJD Fixed miscalculation in sector size when programming
 */
#include "vm.h"
#include "flash.h"

typedef float	FLOAT;
typedef double	DOUBLE;

#if __PACKUNALIGNED__
typedef __packed__ struct tSOURCE
{
	UINT8 Integer;
} _PackedType tSOURCE;
#define ReadUnaligned(x)    ((tSOURCE *)(x))->Integer
#else
#define ReadUnaligned(x)	*(UINT8 *)(x)
#endif

#ifdef __LITTLE_ENDIAN__
#define	MASKSHIFTOP1	<<
#define MASKSHIFTOP2	>>
#else
#define MASKSHIFTOP1	>>
#define MASKSHIFTOP2	<<
#endif

typedef union tFLASH
{
	UINT8 Integer;
	UINT8 Float;
} tFLASH;

static const tFLASH Erase1 = 		{ 0x20 };
static const tFLASH Erase2 = 		{ 0xD0 };
static const tFLASH Program =		{ 0x40 };
static const tFLASH ReadArray =		{ 0xFF };
static const tFLASH ClearStatus =	{ 0x50 };
static const tFLASH Erased =		{ (UINT8)-1 };
static const tFLASH ReadyBits =		{ 0x80 };
static const tFLASH EraseBits =		{ 0x20 };
static const tFLASH ProgramBits =	{ 0x10 };
static const tFLASH BufferWrite =	{ 0xE8 };
static const tFLASH ConfirmWrite =	{ 0xD0 };
static const tFLASH BufferCount =	{ 0x1F };

#ifdef UPDATE
static UINT32 FlashWrite(volatile UINT8 *pDest, UINT8 *pSource,
		UINT32 Length)
{
	unsigned int OriginalLength = Length;
	UINT8 Mask;
	tFLASH Data;
	unsigned int n;

	UINT8 TempBuffer[0x20];

	while(Length)
	{
	volatile tFLASH *pBlock = (tFLASH *)((UINT32)pDest & ~0x1FFFF);
		*(volatile UINT8 *)pDest = ReadArray.Float;

		if ((((UINT32)pDest & 0x1F) == 0) && (Length >= 0x20))
		{
			memcpy((char *)&TempBuffer, pSource, 0x20);
			pBlock->Float = BufferWrite.Float;
			IOSync();
			if (pBlock->Integer == ReadyBits.Integer)
			{
				pBlock->Float = BufferCount.Float;

				for (n = 0; n < 0x20; ++n)
#if __LITTLE_ENDIAN__
					((volatile UINT8 *)pDest)[n] = TempBuffer[n];
#else /* __BIG_ENDIAN__ */
					((volatile UINT8 *)pDest)[n^3] = TempBuffer[n^3];
#endif

				pBlock->Float = ConfirmWrite.Float;

				while (*(volatile UINT8 *)pDest != ReadyBits.Integer)
					;

				pDest += 0x20;
				pSource += 0x20;
				Length -= 0x20;

				continue;
			}

			*(volatile UINT8 *)pDest = ReadArray.Float;
		}

		Data.Integer = *(UINT8 *)pSource++;

		*(volatile UINT8 *)pDest = ClearStatus.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Program.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Data.Float;
		IOSync();

		while (*(volatile UINT8 *)pDest != ReadyBits.Integer)
			;

		++pDest;
		--Length;
	}

	*(volatile UINT8 *)(pDest - 1) = ReadArray.Float;
	return OriginalLength - Length;
}

static UINT32 FlashEraseSector(volatile UINT8 *pSector)
{
	*(volatile UINT8 *)pSector = ClearStatus.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase1.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase2.Float;
	IOSync();

	while (*(volatile UINT8 *)pSector != ReadyBits.Integer)
		;

	*(volatile UINT8 *)pSector = ReadArray.Float;
	return 0;
}


#else					// #ifdef UPDATE
#ifdef BUILD_TYPE_XIP
#include "timer.h"

static TIMEOUT FlashTimer;
static volatile UINT32 TimedOut;
static UINT32 (*pFlashWrite)(volatile UINT8 *, UINT8 *, UINT32, volatile UINT32 *) = 0;
static UINT32 (*pFlashErase)(volatile UINT8 *, volatile UINT32 *) = 0;

static void FlashTimeout(UINT32 Parameter, TIMEOUT *pTimeout)
{
	TimedOut = 1;
}

static void SetFlashTimer(UINT32 Milliseconds)
{
	if (!FlashTimer.Parameter)
	{
		GetVmmContext(&FlashTimer.vc);
		FlashTimer.Vector = FlashTimeout;
		FlashTimer.Parameter = 1;
	}
	else
		KillTimer(&FlashTimer);

	TimedOut = 0;
	FlashTimer.Timer = Milliseconds;
	InstallTimer(&FlashTimer);
}

static UINT32 __FlashWrite(volatile UINT8 *pDest, UINT8 *pSource,
		UINT32 Length, volatile UINT32 *pTimedOut)
{
	unsigned int OriginalLength = Length;
	UINT8 Mask;
	tFLASH Data;
	unsigned int n;

	UINT8 TempBuffer[0x20];

	while(Length)
	{
	volatile tFLASH *pBlock = (tFLASH *)((UINT32)pDest & ~0x1FFFF);
		*(volatile UINT8 *)pDest = ReadArray.Float;

		if ((((UINT32)pDest & 0x1F) == 0) && (Length >= 0x20))
		{
			memcpy((char *)&TempBuffer, pSource, 0x20);
			pBlock->Float = BufferWrite.Float;
			IOSync();
			if (pBlock->Integer == ReadyBits.Integer)
			{
				pBlock->Float = BufferCount.Float;

				for (n = 0; n < 0x20; ++n)
#if __LITTLE_ENDIAN__
					((volatile UINT8 *)pDest)[n] = TempBuffer[n];
#else /* __BIG_ENDIAN__ */
					((volatile UINT8 *)pDest)[n^3] = TempBuffer[n^3];
#endif

				pBlock->Float = ConfirmWrite.Float;

				while ((*(volatile UINT8 *)pDest != ReadyBits.Integer) && !(*pTimedOut))
					;

				if (*pTimedOut)
					break;

				pDest += 0x20;
				pSource += 0x20;
				Length -= 0x20;

				continue;
			}

			*(volatile UINT8 *)pDest = ReadArray.Float;
		}

		Data.Integer = *(UINT8 *)pSource++;

		*(volatile UINT8 *)pDest = ClearStatus.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Program.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Data.Float;
		IOSync();

		while ((*(volatile UINT8 *)pDest != ReadyBits.Integer) && !(*pTimedOut))
			;

		if (*pTimedOut)
			break;

		++pDest;
		--Length;
	}

	*(volatile UINT8 *)(pDest - 1) = ReadArray.Float;
	return OriginalLength - Length;
}

static UINT32 FlashWrite(volatile UINT8 *pDest, UINT8 *pSource,
		UINT32 Length)
{
	unsigned int ActualLength;
	unsigned int CodeLength;

	if (!pFlashWrite)
	{
		CodeLength = (UINT32)FlashWrite - (UINT32)__FlashWrite;
		pFlashWrite = (UINT32 (*)(volatile UINT8 *, UINT8 *, UINT32,
				volatile UINT32 *))malloc(CodeLength);
		memcpy((void *)pFlashWrite, __FlashWrite, CodeLength);
	}
	SetFlashTimer(5000);		// 5 second timeout
	ActualLength = (*pFlashWrite)(pDest, pSource, Length, &TimedOut);
	KillTimer(&FlashTimer);
	return ActualLength;
}

static UINT32 __FlashErase(volatile UINT8 *pChip, volatile UINT8 *pSector,
		volatile UINT32 *pTimedOut)
{
	*(volatile UINT8 *)pSector = ClearStatus.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase1.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase2.Float;
	IOSync();

	while ((*(volatile UINT8 *)pSector != ReadyBits.Integer) && !(*pTimedOut))
		;

	*(volatile UINT8 *)pSector = ReadArray.Float;
	return TimedOut;
}

static UINT32 FlashEraseSector(volatile UINT8 *pSector)
{
	UINT32 CodeLength;
	volatile UINT8 *pChip = (volatile UINT8 *)((unsigned int)pSector
			& ~(0x1000000 - 1));

	if (!pFlashErase)
	{
		CodeLength = (UINT32)FlashEraseSector - (UINT32)__FlashErase;
		pFlashErase = (UINT32 (*)(volatile UINT8 *, volatile UINT8 *,
				volatile UINT32 *))malloc(CodeLength);
		memcpy((void *)pFlashErase, __FlashErase, CodeLength);
	}

	SetFlashTimer(5000);
	(*pFlashErase)(pChip, pSector, &TimedOut);
	KillTimer(&FlashTimer);

	return 0;
}

#else			// #ifdef BUILD_TYPE_XIP
#include "timer.h"

static TIMEOUT FlashTimer;
static volatile UINT32 TimedOut;

static void FlashTimeout(UINT32 Parameter, TIMEOUT *pTimeout)
{
	TimedOut = 1;
}

static void SetFlashTimer(UINT32 Milliseconds)
{
	if (!FlashTimer.Parameter)
	{
		GetVmmContext(&FlashTimer.vc);
		FlashTimer.Vector = FlashTimeout;
		FlashTimer.Parameter = 1;
	}
	else
		KillTimer(&FlashTimer);

	TimedOut = 0;
	FlashTimer.Timer = Milliseconds;
	InstallTimer(&FlashTimer);
}

static UINT32 FlashWrite(volatile UINT8 *pDest, UINT8 *pSource,
		UINT32 Length)
{
	unsigned int OriginalLength = Length;
	UINT8 Mask;
	tFLASH Data;
	unsigned int n;

	UINT8 TempBuffer[0x20];

	SetFlashTimer(5000+(4*5000*(Length/0x20000)));
	while(Length)
	{
	volatile tFLASH *pBlock = (tFLASH *)((UINT32)pDest & ~0x1FFFF);
		*(volatile UINT8 *)pDest = ReadArray.Float;

		if ((((UINT32)pDest & 0x1F) == 0) && (Length >= 0x20))
		{
			memcpy((char *)&TempBuffer, pSource, 0x20);
			pBlock->Float = BufferWrite.Float;
			IOSync();
			if (pBlock->Integer == ReadyBits.Integer)
			{
				pBlock->Float = BufferCount.Float;

				for (n = 0; n < 0x20; ++n)
#if __LITTLE_ENDIAN__
					((volatile UINT8 *)pDest)[n] = TempBuffer[n];
#else /* __BIG_ENDIAN__ */
					((volatile UINT8 *)pDest)[n^3] = TempBuffer[n^3];
#endif

				pBlock->Float = ConfirmWrite.Float;

				while ((*(volatile UINT8 *)pDest != ReadyBits.Integer) && !TimedOut)
					;

				if (TimedOut)
					break;

				pDest += 0x20;
				pSource += 0x20;
				Length -= 0x20;

				continue;
			}

			*(volatile UINT8 *)pDest = ReadArray.Float;
		}

		Data.Integer = *(UINT8 *)pSource++;

		*(volatile UINT8 *)pDest = ClearStatus.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Program.Float;
		IOSync();
		*(volatile UINT8 *)pDest = Data.Float;
		IOSync();

		while ((*(volatile UINT8 *)pDest != ReadyBits.Integer) && !TimedOut)
			;

		if (TimedOut)
			break;

		++pDest;
		--Length;
	}

	KillTimer(&FlashTimer);
	*(volatile UINT8 *)(pDest - 1) = ReadArray.Float;
	return OriginalLength - Length;
}

static UINT32 FlashEraseSector(volatile UINT8 *pSector)
{
	SetFlashTimer(5000);

	*(volatile UINT8 *)pSector = ClearStatus.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase1.Float;
	IOSync();
	*(volatile UINT8 *)pSector = Erase2.Float;
	IOSync();

	while ((*(volatile UINT8 *)pSector != ReadyBits.Integer) && !TimedOut)
		;

	KillTimer(&FlashTimer);
	*(volatile UINT8 *)pSector = ReadArray.Float;
	return TimedOut;
}

#endif
#endif

FLASHOBJECT FO_28F128J3x1 =
{
	FlashWrite,
	FlashEraseSector,
	0x20000,
	"Intel 28F128J3 ("
	"16384 x 8"
	") 1 Device"
};

