/***************************************************************************
*
* Module:		AssignedNumbers.h
*
* Description:	Primary number assignment for vWare.
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc.  All Rights Reserved
*
****************************************************************************/
#ifndef __ASSIGNEDNUMBERS_H__
#define __ASSIGNEDNUMBERS_H__

#ifndef __ASSEMBLER__
/**********************************************************************
 * GROUP Macro - extracts the group number from a library, command    *
 * or shellvar number.                                                *
 *********************************************************************/
#define GROUP(x)	((x) >> 8)

/**********************************************************************
 * MAKECMD Macro                                                      *
 *********************************************************************/
#define MAKECMD(group, n)	((group << 8) + n)

/**********************************************************************
 * MAKELIB Macro                                                      *
 *********************************************************************/
#define MAKELIB(group, n)	((group << 8) + n)

/**********************************************************************
 * MAKESHELLVAR Macro                                                 *
 *********************************************************************/
#define MAKESHELLVAR(group, n)	((group << 8) + n)

/**********************************************************************
 * Libary and Command Group Number Assignments                        *
 *********************************************************************/
enum
{
	LIBGROUP_BASELINE = 9,	// 0x0900 - 0x09FF: Baseline functions
	LIBGROUP_FFS = 10,		// 0x0A00 - 0x0AFF: File System functions
	LIBGROUP_NET = 8,		// 0x0800 - 0x08FF: TCP/IP functions
							// 0x0300 - 0x03FF: Reserved
	LIBGROUP_CPU = 4,		// 0x0400 - 0x04FF: CPU/Core functions
	LIBGROUP_BOARD,			// 0x0500 - 0x05FF: Board-specific functions
	LIBGROUP_SERIAL,		// 0x0600 - 0x06FF: Serial functions
	                    	// 0x0700 - 0x0FFF: Reserved for additional
							//					library groups
	CMDGROUP_VWARE = 0x10,	// 0x1000 - 0x10FF: vWARE commands
	CMDGROUP_VCONSOLE,		// 0x1100 - 0x11FF: vCONSOLE commands
	CMDGROUP_VFILE,			// 0x1200 - 0x12FF: vFILE commands
	CMDGROUP_VMON,			// 0x1300 - 0x13FF: vMON commands
	CMDGROUP_VFLASH,		// 0x1400 - 0x14FF:	vFLASH commands
	CMDGROUP_VSTACK,		// 0x1500 - 0x15FF: vSTACK commands
	CMDGROUP_VDIAG,			// 0x1600 - 0x16FF: vDIAG commands
	CMDGROUP_BOARD,			// 0x1700 - 0x17FF: Board-specific commands
	CMDGROUP_SCRIPT,		// 0x1800 - 0x18FF: Command-script commands
	CMDGROUP_VFPGA,			// 0x1900 - 0x19FF: FPGA commands
	CMDGROUP_PCI,			// 0x1A00 - 0x1AFF: PCI commands
							// 0x1B00 - 0x2EFF: Reserved for additional
							//					command groups
	CMDGROUP_TEMP = 0x2F,	// 0x2F00 - 0x2FFF: Temporary/test commands
	CMDGROUP_HSIVMM0 = 0x30,// 0x3000 - 0x30FF: First HSI VMM
							// 0x3100 - 0x5FFF: Reserved for additional
							//					HSI VMMs
							// 0x6000 - 0x9FFF: Reserved for HSI
	CMDGROUP_USER0 = 0xA0,	// 0xA000 - 0xA0FF: First User VMM
							// 0xA100 - 0xEFFF: Reserved for additional
							//					User VMMs
	CMDGROUP_RSRVD = 0xF0	// 0xF000 - 0xFFFF: Reserved for HSI
};


/**********************************************************************
 * SHELLVAR Number Assigments.                                        *
**********************************************************************/
enum
{
							// 0x0000 - 0x00FF: Reserved for HSI
	SHELLGROUP_VM = 1,		// 0x0100 - 0x01FF: VisionMON Shell variables
							// 0x0200 - 0x02FF: Reserved for HSI
	SHELLGROUP_FFS = 3,		// 0x0300 - 0x03FF: File System Shell variables
							// 0x0400 - 0x04FF: Reserved for HSI
	SHELLGROUP_ESTACK = 5,	// 0x0500 - 0x05FF: TCP/IP Shell variables
							// 0x0600 - 0x06FF: Reserved for HSI
	SHELLGROUP_SERIAL = 7,	// 0x0700 - 0x07FF: Serial Shell variables
							// 0x0800 - 0x0FFF: Reserved for HSI
	SHELLGROUP_BOARD = 0x10,// 0x1000 - 0x10FF: Board-Specific
							// 0x1100 - 0x11FF: Reserved for HSI
	SHELLGROUP_CORE = 0x12,	// 0x1200 - 0x12FF: Core-Specific
							// 0x1300 - 0x13FF: Reserved for HSI
	SHELLGROUP_CPU = 0x14,	// 0x1400 - 0x14FF: CPU-Specific
							// 0x1500 - 0x7FFF: Reserved for HSI
	SHELLGROUP_USER0 = 0x80,// 0x8000 - 0x80FF: User group 0
							// 0x8100 - 0x9FFF: Reserved for user
	SHELLGROUP_RSRVD = 0xF0	// 0xF000 - 0xFFFF: Reserved for HSI
};

/**********************************************************************
 * VM SHELLVAR Number Assigments.                                     *
**********************************************************************/
enum
{
	SHELLVAR_BOOTDELAY = MAKESHELLVAR(SHELLGROUP_VM, 1),
	SHELLVAR_BOOTSCRIPT,
	SHELLVAR_DEFAULTCHANNEL,
	SHELLVAR_LINES,
	SHELLVAR_PROMPT,
	SHELLVAR_OS_BOOTLINE,
	SHELLVAR_vxBOOT_DEV_NAME,
	SHELLVAR_vxELF_FILE_LOC,
	SHELLVAR_vxUSERNAME,
	SHELLVAR_vxPASSWORD,
	SHELLVAR_vxFLAGS,
	SHELLVAR_vxTARGET_NAME
};

/**********************************************************************
 * Component Number Assignments.                                      *
**********************************************************************/
enum
{
							// 0x0000 - 0x00FF: Reserved for HSI
	COMPONENT_ETHERNET = 0x0100,
	COMPONENT_SERIAL,
	COMPONENT_TMD,
	COMPONENT_FFS,
	COMPONENT_FREEFLASH,
	COMPONENT_SHELL,
	COMPONENT_MON,
	COMPONENT_TFTP,
	COMPONENT_HTTP,
	COMPONENT_USER0 = 0x8000
};

/**********************************************************************
 * Region Number Assignments.                                         *
**********************************************************************/
enum
{
	REGION_FFSFLASH = 1,
	REGION_FREEFLASH,
	REGION_BOOTFLASH,
	REGION_USER0 = 0x100,				// new style
	FIRST_USER_REGION = REGION_USER0	// old style
};

#include "vTypes.h"

/**********************************************************************
 * Vector/IRQ Number Assignments                                      *
 *********************************************************************/
#define CPUIRQBASE		0x00000000
#define BOARDIRQBASE	0x00008000
#define PIC1IRQBASE		0x00010000
#define PIC2IRQBASE		0x00020000
#define PIC3IRQBASE		0x00030000
#define PIC4IRQBASE		0x00040000
#define FIRSTVECTORID	0x00070000

#endif	// #ifndef __ASSEMBLER__

/**********************************************************************
 * StopReason Number Assignments.                                     *
**********************************************************************/
								// 0x0000 - 0x7FFF: Board/Process-specific
								// 0x8000 - 0xFFFC: Reserved for HSI
#define	STOPREASON_SYSCALL	0xFFFD
#define	STOPREASON_GOTCHAR	0xFFFE
#define	STOPREASON_NOREASON	0xFFFF

#endif	// __ASSIGNEDNUMBERS_H__
