/***************************************************************************
*
* Module:		cmd.h
*
* Description:	Command definition macros
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc.  All Rights Reserved
*
****************************************************************************/
#ifndef	__CMD_H__
#define __CMD_H__

#include "cmds.h"

/****************************************************************************
 * Stream Identifiers                                                       *
 ***************************************************************************/
#define STREAMTYPE(x)	((x) & 0xF0)

#define ARGSTREAM		0x10
#define SWITCHSTREAM	0x20
#define OUTPUTSTREAM	0x30
#define NEWOUTPUTSTREAM	0x40
#define ENDSTREAM		0xF0

#define ARRAYSIZE(x)	(((x) & 0x0F) + 1)

/****************************************************************************
 * Command Flags                                                            *
 ***************************************************************************/
#define SIZESUFFIX		0x01
#define HIDDEN			0x02
#define SILENTTYPE		0x00			// obsolete
#define SCRIPTONLY		0x04
#define WHILERUNNING	0x08
#define REPEATABLE		0x10
#define DIAGNOSTIC		0x20
#define COMMANDGROUP	0x40
#define COMMANDPREFIX	0x80

/****************************************************************************
 * Argument/Switch Flags                                                    *
 *                                                                          *
 * Bits 0-5:  (size of variable)-1                                          *
 * Bits 6-7:  reserved                                                      *
 * Bits 8-10: variable type                                                 *
 *            000 = integer                                                 *
 *            001 = character/string                                        *
 *            010 = boolean (only applicable for switches)                  *
 *            011 = raw                                                     *
 *            100 = IP address                                              *
 * Bit 11:    argument is optional                                          *
 * Bit 12:    argument/switch is signed                                     *
 * Bit 13:    argument/switch is in decimal                                 *
 * Bit 14:    array                                                         *
 * Bit 15:    array; first element is required                              *
 ***************************************************************************/
#define INTARG			0x000
#define CHARARG			0x100
#define BOOLEANARG		0x200
#define RAWARG			0x300
#define IPARGTYPE		0x400
#define OPTARG			0x800
#define SIGNEDARG		0x1000
#define DECIMALARG		0x2000
#define ARRAYARG		0x4000
#define FIRSTREQUIRED	0x8000

/****************************************************************************
 * Argument/Switch Types                                                    *
 ***************************************************************************/
#define IPARG		(IPARGTYPE | 3)
#define UINT8ARG	(INTARG)
#define UINT16ARG	(INTARG | 1)
#define UINT32ARG	(INTARG | 3)
#define UINT64ARG	(INTARG | 7)			// not fully supported
#define INT8ARG		(UINT8ARG | SIGNEDARG)	// not fully supported
#define INT16ARG	(UINT16ARG | SIGNEDARG)	// not fully supported
#define INT32ARG	(UINT32ARG | SIGNEDARG)	// not fully supported
#define INT64ARG	(UINT64ARG | SIGNEDARG)	// not fully supported
#define CHAR1ARG	(CHARARG)
#define CHAR2ARG	(CHARARG | 1)
#define CHAR4ARG	(CHARARG | 3)
#define CHAR8ARG	(CHARARG | 7)
#define CHAR10ARG	(CHARARG | 9)
#define CHAR12ARG	(CHARARG | 11)
#define CHAR16ARG	(CHARARG | 15)
#define CHAR20ARG	(CHARARG | 19)
#define CHAR24ARG	(CHARARG | 23)
#define CHAR30ARG	(CHARARG | 29)
#define CHAR32ARG	(CHARARG | 31)
#define CHAR40ARG	(CHARARG | 39)
#define CHAR50ARG	(CHARARG | 49)
#define CHAR60ARG	(CHARARG | 59)
#define CHAR64ARG	(CHARARG | 63)
#define BOOLEAN8ARG		(BOOLEANARG)		// only valid for switches
#define BOOLEAN16ARG	(BOOLEANARG | 1)	// only valid for switches
#define BOOLEAN32ARG	(BOOLEANARG | 3)	// only valid for switches
#define RAW64ARG	(RAWARG | 63)

/****************************************************************************
 * Command Group Structure and Macro                                        *
 ***************************************************************************/
typedef struct tGroup
{
	UINT32 *pNumber;
	char *Name;
} tGroup;

#define	CommandGroup(Number, Name)		\
global const int __ ## Number = Number;	\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #Name);				\
asm ("	.section	.cmdgroups,4,r");	\
asm ("	.word	__" #Number);			\
asm	("	.word	cmdnames");

/****************************************************************************
 * Command Prefix Structure and Macro                                       *
 ***************************************************************************/
typedef struct tPrefix
{
	char *Name;
} tPrefix;

#define CommandPrefix(Name)				\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #Name);				\
asm ("	.section	.prefixes,4,r");	\
asm ("	.word	cmdnames");

/****************************************************************************
 * tStreamHeader Structure and the Command macro.  Each instance of the     *
 * Command macro creates a tStreamHeader structure which describes the      *
 * command.                                                                 *
 ***************************************************************************/
typedef struct
{
	UINT32 Vector;						// execution address for commands
	UINT32 *pNumber;					// command number
	UINT16 Flags;
	UINT16 Reserved;
	char *pStrings;
} tStreamHeader;

#define Command(Number, Name, Vector, Flags, FriendlyName, HelpString)\
global const int __ ##Number = Number;	\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #Name);				\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.word	" #Vector);				\
asm ("	.word	__" #Number);			\
asm	("	.half	" EVAL(Flags));			\
asm ("	.half	0");					\
asm	("	.word	cmdnames");

/****************************************************************************
 * tStreamEnd structure and the commandEnd macro.   Every command def-      *
 * inition must terminate with a CommandEnd macro which will create a       *
 * tStreamEnd structure.                                                    *
 ***************************************************************************/
typedef struct
{
	UINT16 EndMarker;
	UINT16 Reserved;
} tStreamEnd;

#define CommandEnd()					\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(ENDSTREAM));		\
asm ("	.half	0");

/****************************************************************************
 * tStreamComponent structure.  Each argument and switch of a command is    *
 * encoded as a tStreamComponent.                                           *
 ***************************************************************************/
typedef struct
{
	UINT16 Type;						// declares what type of component this is.
	UINT16 Flags;
	char *pStrings;
	UINT32 DefaultValue;
} tStreamComponent;

#define Arg(Flags, FriendlyName, HelpString)\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(ARGSTREAM));		\
asm ("	.half	" EVAL(Flags));			\
asm ("	.word	cmdnames");				\
asm ("	.word	0");

#define OptArg(Flags, FriendlyName, HelpString, DefaultValue)\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(ARGSTREAM));		\
asm ("	.half	" EVAL(Flags | OPTARG));\
asm ("	.word	cmdnames");				\
asm (".if (("EVAL(Flags)") & "EVAL(CHARARG)") == "EVAL(CHARARG));\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdname:	.set	.");			\
asm	("	.asciz	" #DefaultValue);		\
asm	("	.section	.commands,4,r");	\
asm	("	.word	cmdname");				\
asm	(".else");							\
asm	("	.word	" #DefaultValue);		\
asm	(".endif");

#define ArrayArg(Length, Flags, FriendlyName, HelpString)\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(ARGSTREAM | (Length - 1)));\
asm ("	.half	" EVAL(Flags | ARRAYARG));\
asm ("	.word	cmdnames");				\
asm	("	.word	0");

#define OptArrayArg(Length, Flags, FriendlyName, HelpString)\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(ARGSTREAM | (Length - 1)));\
asm ("	.half	" EVAL(Flags | OPTARG | ARRAYARG));\
asm ("	.word	cmdnames");				\
asm	("	.word	0");

#define Switch(Name, Flags, FriendlyName, HelpString)\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #Name);				\
asm	("	.asciz	" #FriendlyName);		\
asm	("	.asciz	" #HelpString);			\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(SWITCHSTREAM));	\
asm ("	.half	" EVAL(Flags));			\
asm ("	.word	cmdnames");				\
asm ("	.word	0");

#define Output(Flags, FriendlyName)		\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(OUTPUTSTREAM));	\
asm ("	.half	" EVAL(Flags));			\
asm ("	.word	cmdnames");				\
asm ("	.word	0");

#define NewOutput(Flags, FriendlyName)	\
asm	("	.section	.vstrings,1,r");	\
asm	("cmdnames:	.set	.");			\
asm	("	.asciz	" #FriendlyName);		\
asm ("	.section	.commands,4,r");	\
asm ("	.half	" EVAL(NEWOUTPUTSTREAM));\
asm ("	.half	" EVAL(Flags));			\
asm ("	.word	cmdnames");				\
asm ("	.word	0");

typedef struct Cmd
{
	UINT16 Number;
	UINT32 Vector;
	VMMCONTEXT vc;
	tStreamHeader *ByteStream;
} Cmd;

typedef __packed__ struct tVMM
{
	VMMCONTEXT vc;
	tStreamHeader *pCommands;
	UINT16 nCommands;
	tGroup *pGroups;
	UINT16 nGroups;
	char **pPrefixes;
	UINT16 nPrefixes;
} _PackedType tVMM;

#endif		// #ifndef __CMD_H__
