/***************************************************************************
*
* Module:		lib.h
*
* Description:	Visionware library prototypes
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc.  All Rights Reserved
*
****************************************************************************/
#ifndef __LIB_H__
#define __LIB_H__

#include <stdarg.h>

typedef struct tPROTECTEDAREA
{
	void *pFirst;
	void *pLast;
} PROTECTEDAREA;

typedef struct tSORTSTRUCT
{
	void *pStart;
	void *pEnd;
	UINT32 LowBound;
	UINT32 StructSize;
	UINT32 FieldOffset;
	void *pLastMatch;
} tSORTSTRUCT;

#define min(a, b)  (((a) < (b)) ? (a) : (b))
#define max(a, b)  (((a) > (b)) ? (a) : (b))

typedef int (*HANDLER)(void *);

// APIGUI
global UINT32 Result(UINT32 ResultCode, const char *pFormat, ...);
global void Info(const char *pFormat, ...);

global void Noop(void);

global void *malloc(size_t size);
global void *calloc(size_t num, size_t size);
global void free(void *memblock);

global void *memcpy(void *pDest, const void *pSrc, unsigned int count);
global void *memset(void *pDest, int c, unsigned int count);
global char *strcpy(char *pDest, const char *pSrc);
global char *strcat(char *pDest,const char *pSrc);
global unsigned int strlen(const char *pString);
global char *strncpy(char *pDest, const char *pSrc, unsigned int count);
global int strncmp(const char *pString1, const char *pString2, unsigned int count);
global int strcmp(const char *pString1, const char *pString2);
global int stricmp(const char *pString1,const char *pString2);
global int strnicmp(const char *pString1, const char *pString2, unsigned int count);
global int tolower(int c);
global int toupper(int c);
global char *strtoupper(char *s);
global int isdigit(int);
global int isxdigit(int);
global int isalpha(int);
global int __iscsym(int);
global char *strchr(const char *pString, int c);
global char *strnchr(const char *pString, int c, unsigned int length);

global int vsprintf(char *pOutput, const char *pFormat, va_list args);
global int vprintf( const char *format, va_list argptr );
global int sprintf(char *buffer, const char *format, ...);
global int printf(const char *format, ...);
global int more(const char *format, ...);

global int putchar(int c);
global int getchar(void);
global int puts(const char *s);
global int _kbhit(void);

global int atoi(const char*);
global int htoi(const char *);
global UINT64 __atol(char **);
global UINT64 __htol(char **);

global const void *FindComponent(UINT32);
global tRegion *FindRegion(UINT32);
global void *FindNextLowest(tSORTSTRUCT *);
global void Protect(void *, UINT32);

global HANDLE InstallHandler(UINT32, HANDLER, void *);
global void RemoveHandler(HANDLE);
global void DisableIRQ(HANDLE);
global void EnableIRQ(HANDLE);

// APIGUIEND

#ifdef __LITTLE_ENDIAN__
global UINT16 ntohs(UINT16);
global UINT16 htons(UINT16);
global UINT32 ntohl(UINT32);
global UINT32 htonl(UINT32);
#else
#define ntohs(x)	(x)
#define htons(x)	(x)
#define ntohl(x)	(x)
#define htonl(x)	(x)
#endif

#endif					// #ifndef __LIB__H
