/******************************************************************************
*
* Module:		tf.c
*
* Description:
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"
#include "flash.h"
#include "cmd.h"
#include "tf.h"

//  External References
extern const FLASHOBJECT FREEFLASHOBJECT;
extern UINT32 Protected(void *, UINT32);
extern UINT32 ErrorAddress;
extern FLASHOBJECT *IsFlash(UINT32);
extern tRegion *IsRegion(char **);

/****************************************************************************
* Function:	tferase(charf *pRegionName, void *pOut);
*
* Purpose:	erase a flash region
*
* Returns:	void
****************************************************************************/
void tferase(char *pRegionName, void *pOut)
{
	UINT32 i;
	tRegion *pRegion;
	char **pName = &pRegionName;
	FLASHOBJECT *pFO;

	if (pRegion = IsRegion(pName))
	{
		if (pFO = IsFlash(pRegion->Start))
		{
			for (i = pRegion->Start; i && (i < pRegion->End); i += pFO->SectorSize)
				(*pFO->EraseVector)((UINT8 *)i);
		   Result(0, "%s erased", pRegion->pName);
		}
		else
			Result(1, "cannot find FLASHOBJECT for \"%s\"", pRegion->pName);
	}
	else
		Result(2, "unknown region \"%s\"", pRegionName);
}

CommandPrefix("tf");

Command(CMD_TFERASE, "tf erase", tferase, HIDDEN, "", "");
CommandEnd();

Command(CMD_ERASE, "erase", tferase, 0, "", "flash erase by region: ffs,free,boot,etc.");
OptArg(CHAR16ARG, "region", "", "free");
CommandEnd();

Component("Free Flash", COMPONENT_FREEFLASH, FREEFLASHOBJECT);
Region("Free Flash", REGION_FREEFLASH, FREEFLASHSTART, FREEFLASHEND);

/****************************************************************************
* Function:	SetFlash(in_FlashTest *pIn, void *pOut);
*
* Purpose:	Write data to flash.  Analogous to the "sm" command (SetMemory).
*
* Returns:	void
****************************************************************************/
void SetFlash(in_FlashTest *pIn, void *pOut)
{
	universal u;
	universal b;
	int i;
	int Length = pIn->Width * pIn->Count;
	tRegion *pRegion;
	UINT8 Buffer[64];
	UINT32 n;

	u.uvalue = pIn->Address;

	pRegion = FindRegion(REGION_FREEFLASH);

	if (!((u.uvalue >= pRegion->Start) && ((u.uvalue + Length) <= (pRegion->End + 1))))
	{
		Result(2, "specified address is not part of free flash 0x%08X",u.uvalue);
		return;
	}

	if (Protected(u.pVoid, Length))
	{
		Result(1, "address 0x%08X is protected", ErrorAddress);
		return;
	}

	for (i = 0; i < Length; ++i)
	{
		if (u.pByte[i] != 0xFF)
			break;
	}

	if (i < Length)
	{
		UINT32 EraseStart = u.uvalue & ~(FREEFLASHOBJECT.SectorSize - 1);
		UINT32 Sectors = (EraseStart + FREEFLASHOBJECT.SectorSize) >= (u.uvalue + Length)
				? 1 : 2;

		if (!pIn->OkToErase)
		{
			printf("Erasing %08X to %08X\nPress 'y' to continue, any other key to abort\n",
					EraseStart, EraseStart + (Sectors * FREEFLASHOBJECT.SectorSize));
			if (tolower(getchar()) != 'y')
			{
				Result(3, "aborted by user");
				return;
			}
		}

		while (Sectors--)
		{
			if ((*FREEFLASHOBJECT.EraseVector)((UINT8 *)EraseStart))
			{
				Result(4, "flash erase error at %08X", EraseStart);
				return;
			}
			EraseStart += FREEFLASHOBJECT.SectorSize;
		}
	}

	b.pByte = Buffer;

	for (i = 0; i < pIn->Count; ++i)
	{
		n = pIn->Values[i];

		switch(pIn->Width)
		{
		case 4:
			*b.pWord++ = n;
			break;

		case 2:
			*b.pHalf++ = (UINT16)n;
			break;

		default:
			*b.pByte++ = (UINT8)n;
			break;
		}
	}

	if ((*FREEFLASHOBJECT.WriteVector)(u.pByte, Buffer, Length) != Length)
		Result(5, "flash write error");
	else
		Result(0,"");
}

Command(CMD_SF, "sf", SetFlash, SIZESUFFIX, "write pattern to free flash", "write pattern to free flash");
Arg(UINT32ARG, "Start Address", "");
OptArrayArg(16, UINT32ARG | FIRSTREQUIRED, "Values", "");
Switch("y", BOOLEAN32ARG, "Suppress erase prompt", "");
CommandEnd();
