/***************************************************************************
*
* Module:		timestamp.c
*
* Description:	Timestamp functions
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc.  All Rights Reserved
*
****************************************************************************/
//  Includes
#include "vm.h"
#include "timer.h"
#include "cmd.h"

//  Static Declarations
static UINT32	TIME;
static HANDLE hTimer;

/****************************************************************************
* Function:	IncTime(UINT32 Unused, TIMEOUT *pT);
*
* Purpose:	Routine that gets called when TimeTimer runs out
*
* Returns:	void
****************************************************************************/
static void IncTime(void *pUnused)
{
	++TIME;
}

/****************************************************************************
* Function:	InitTimer(void);
*
* Purpose:	Inititiate a 50 ms. tick counter to be used as a global timestamp
*
* Returns:	void
****************************************************************************/
void InitTimer()
{
	vLOG("InitTimer: Enter Init Timer", NULL, NULL);

	TIME = 0;
	InstallPeriodicTimer(50, IncTime, 0);

	vLOG("InitTimer: Exit Init Timer", NULL, NULL);
}

Init(608, InitTimer);

/****************************************************************************
* Function:	GetLocalTimestamp(void);
*
* Purpose:	Access method for the global timestamp
*
* Returns:	void
****************************************************************************/
UINT32 GetLocalTimestamp()
{
    return TIME;
}

/****************************************************************************
* Function:	GetTimeStamp(void *pIn, void *pOut);
*
* Purpose:	Command to display the current global timestamp
*
* Returns:	void
****************************************************************************/
void GetTimestamp(void *pIn, void *pOut)
{
	UINT32 FiftyMsTicks = TIME;
	UINT32 Hours, Minutes, Seconds;

	Seconds = FiftyMsTicks / 20;
	Minutes = Seconds / 60;
	Hours = Minutes / 60;

	Result(0, "%02d:%02d:%02d.%02d", Hours, Minutes % 60, Seconds % 60,
			(FiftyMsTicks % 20) * 5);
}

Command(CMD_TIME, "timestamp", GetTimestamp, REPEATABLE, "get timestamp", "time - hours:minutes:seconds:hundreds");
CommandEnd();
