/*
 *  Do NOT put a guard on this file - it will be included multiple times during one
 *  compile, pulling different pieces based on need.  If a guard is used, it will only
 *  allow it to be included the first time.
 */

#include "esttype.h"

/*  Use multiple guards to avoid multiple files  */

#ifndef __VTYPES_FFS_H__
#define __VTYPES_FFS_H__

typedef struct
{
	UINT32 TypeLength;		/* 8 bits type, 24 bits length  */
	UINT32 Next;			/* block/offset encoded.  -1 = end of chain  */
} FFSFRAGMENT;

typedef struct
{
	FFSFRAGMENT f;
	UINT32 Attributes;
	UINT32 LoadAddress;
	UINT32 InitialIP;
	UINT32 Reserved[3];
	UINT8 Name[32];		  	/* name is stored in ASCIIZ form  */
} FFSFILEHEADER;

typedef struct				/* FFS:FindFile takes a pointer to an FFSFF */
{							/* structure and completes it               */
	FFSFILEHEADER *Address;
	UINT32 BlockOffset;
} FFSFF;

// File Attributes
#define ATTRIBTYPE				0x0F
#define ATTRIBTYPE_ESTBIN		0x00
#define ATTRIBTYPE_BDX			0x01
#define ATTRIBTYPE_ESTSCRIPT	0x02
//#define ATTRIBTYPE_VMM			0x02

#endif

#ifndef __VTYPES_HANDLES_H__
#define __VTYPES_HANDLES_H__

typedef void *HANDLE;

#endif

#ifndef __VTYPES_ESDRIVER_H__
#define __VTYPES_ESDRIVER_H__

#define		IOCTL_SETBAUD		1
#define		IOCTL_GETBAUD		2

#endif

#ifndef __VTYPES_ASSIGNEDNUMBERS_H__
#define __VTYPES_ASSIGNEDNUMBERS_H__

/**********************************************************************
 * Error Number Assignments.                                          *
**********************************************************************/
enum
{
	PARSECMD_SUCCESS = 0,		// 0 - success
	FFSOK = 0,
								// 0x01 - 0x7F: Command-Specific
	FIRST_PARSE_ERROR = 0x80,	// 0x80 - 0x9F: Parse errors
	PARSECMD_USAGE = 0x80,
	PARSECMD_UNKNOWN,
	PARSECMD_INVALIDSTATE,
	PARSECMD_STREAMERROR,
	PARSECMD_BADOUTPUT,
	EXPRESS_SYNTAX,
	EXPRESS_PARENTHESES,
	PARSECMD_SCRIPTONLY,

	FIRST_STACK_ERROR = 0xA0,	// 0xA0 - 0xBF: TCP/IP Stack errors
	STACKERROR_ARPTIMEOUT = 0xA0,
	STACKERROR_UNKNOWNHOST,
	STACKERROR_NOHOSTSPECIFIED,
	STACKERROR_TFTPSERVERABORT,
	STACKERROR_TFTPTIMEOUT,
	STACKERROR_WRITEPROTECT,
	STACKERROR_FILEERROR,
	STACKERROR_NOTFTP,
	STACKERROR_UNKNOWNERROR,

	FIRST_FFS_ERROR = 0xC0,		// 0xC0 - 0xDF: File System errors
	FFSUNFORMATTED = 0xC0,
	FFSCORRUPT,
	FFSERROR,
	FFSFULL,
	FFSNOTFOUND,
	FFSFILEEXISTS,
	FFSNOHANDLES,
	FFSFILEOPEN,
	FFSBADHANDLE,
	FFSREADPASTEOF,

	ERR_RSRVD = 0xE0			// 0xE0 - 0xFF: Reserved for HSI
};

#endif
