/***************************************************************************
*
* Module:		vware.h
*
* Description:	Miscellaneous vware stuff.
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc.  All Rights Reserved
*
****************************************************************************/
#ifndef __vWARE_H__
#define __vWARE_H__

global UINT8 CoreState;
global UINT16 StopReason;

global void GetVmmContext(VMMCONTEXT *pVC);

global CONTEXT UserContext, vWAREContext, TempContext;

global UINT8 Running;

typedef struct tFini
{
	UINT32 nOrder;
	void (*Vector)(tOsParams *);
} tFini;

typedef struct tInit
{
	UINT32 nOrder;
	void (*Vector)(void);
} tInit;

typedef union
{
	void *pVoid;
	UINT8 *pByte;
	UINT16 *pHalf;
	UINT32 *pWord;
	UINT32 uvalue;
	UINT64 *pDouble;
} universal;

typedef struct
{
	UINT8 *Address;
	UINT16 Length;
	UINT8 PacketNumber;
	UINT8 Checksum;
} _PackedType BDXHEADER;

#define EI_NIDENT   16
typedef struct
{
	UINT8   e_ident[EI_NIDENT];
	UINT16  e_type;
	UINT16  e_machine;
	UINT32  e_version;
	UINT32  e_entry;
	UINT32  e_phoff;
	UINT32  e_shoff;
	UINT32  e_flags;
	UINT16  e_ehsize;
	UINT16  e_phentsize;
	UINT16  e_phnum;
	UINT16  e_shentsize;
	UINT16  e_shnum;
	UINT16  e_shstrndx;
} _PackedType ELFHEADER;

typedef struct
{
	char Signature[8];		// "ESTFBINR" (no terminating null)
	UINT32 FirstByte;
	UINT32 PaddedLastByte;
	UINT8 FillChar;
	UINT8 Compressed;		// 0 = not compressed
	UINT8 Type;				// 0 = BIN, 1 = BDX
	UINT8 Reserved[5];
	UINT32 FirstByteRepeat;
	UINT32 UnpaddedLastByte;
} _PackedType BINHEADER;

typedef  struct tLibFunc
{
	UINT32 Number;
	VOIDFUNCPTR Vector;
} tLibFunc;

typedef struct tLibName
{
	UINT32 Number;
	char Name[256];
} tLibName;

typedef struct tComponent
{
	UINT32 *pNumber;
	void *pTable;
	char *pName;
} tComponent;

typedef struct tRegion
{
	UINT32* pNumber;
	UINT32 Start;
	UINT32 End;
	char *pName;
} tRegion;

#define ALIGN(a,x) ( \
		((unsigned)(a) + (unsigned)(x) - 1) & ~((unsigned)(x) - 1))

#define SHORT_SWAP(x) 	( (((unsigned short)(x) << 8) & 0xFF00) | \
			(((unsigned short)(x) >>  8) & 0x00FF) )


#ifdef __LITTLE_ENDIAN__

  #define LSWAP(x)	(x)
  #define SSWAP(x)	(x)

  #define POS_LSWAP(x) ( \
		(((ULONG)(x) << 24) & 0xFF000000) | \
		(((ULONG)(x) <<  8) & 0x00FF0000) | \
		(((ULONG)(x) >>  8) & 0x0000FF00) | \
		(((ULONG)(x) >> 24) & 0x000000FF) )

#else

  #define LSWAP(x) ( \
		(((ULONG)(x) << 24) & 0xFF000000) | \
		(((ULONG)(x) <<  8) & 0x00FF0000) | \
		(((ULONG)(x) >>  8) & 0x0000FF00) | \
		(((ULONG)(x) >> 24) & 0x000000FF) )

  #define SSWAP(x) ( (((unsigned short)(x) << 8) & 0xFF00) | \
		(((unsigned short)(x) >>  8) & 0x00FF) )

  #define POS_LSWAP(x) (x)

#endif

#ifdef	__GNUC__
#define	LibFuncSection	__attribute__((section(".libfunc")))
#define	LibNameSection	__attribute__((section(".libname")))
#define	ComponentSection	__attribute__((section(".components")))
#define	RegionSection	__attribute__((section(".regions")))
#define LibFuncStorageSpec
#define LibNameStorageSpec
#define ComponentStorageSpec
#define RegionStorageSpec

#define LibFunc(number, vector, name)	\
	static tLibFunc LibFunc##number LibFuncSection =	\
	{ number, (VOIDFUNCPTR)vector };	\
	static tLibName LibName##number LibNameSection =	\
	{ number, name }

#define Component(name, number, address)	\
	static tComponent Component##number ComponentSection =	\
	{ number, (void *)&address, name };

#define Region(name, number, start, end)	\
	static tRegion Region##number RegionSection =	\
	{ number, start, end, name };

#else	// #ifdef __GNUC__

#define Component(name, number, address)	\
global const int __ ## number = number;		\
asm	("	.section	.vstrings,1,r");		\
asm	("cmdnames:	.set	.");				\
asm	("	.asciz	" #name);					\
asm	("	.section	.components,4,r");		\
asm	("	.word	__" #number);				\
asm	("	.word	" EVAL(address));			\
asm	("	.word	cmdnames");

#define Region(name, number, start, end)	\
global const int __ ## number = number;		\
asm	("	.section	.vstrings,1,r");		\
asm	("cmdnames:	.set	.");				\
asm	("	.asciz	" #name);					\
asm	("	.section	.regions,4,rw");		\
asm	("	.word	__" #number);				\
asm	("	.word	" EVAL(start));				\
asm	("	.word	" EVAL(end));				\
asm	("	.word	cmdnames");

#define	Fini(Number, Vector)				\
asm	("	.section	.vfini,4,r");			\
asm ("	.word	" #Number);					\
asm	("	.word	" #Vector);

#define Init(Number, Vector)				\
asm ("	.section	.vinit,4,r");			\
asm	("	.word	" #Number);					\
asm	("	.word	" #Vector);

#if 0
#define LibFunc(number, vector, name)		\
asm	("	.section	.libfunc,4,r");			\
asm	("	.word	" EVAL(number));			\
asm	("	.word	" #vector);					\
asm	("	.section	.libname,4,r");			\
asm	("	.word	" EVAL(number));			\
asm	("	.asciz	" EVAL(name));
#else
#define LibFunc(number, vector, name)
#endif

#define VWARELIB(number, vector, name)		\
global const int __LibFunc ## number = number;	\
asm	("	.section	.libfunc,4,r");			\
asm	("	.word	__LibFunc" #number);		\
asm	("	.word	" #vector);					\
asm	("	.section	.libname,4,r");			\
asm	("	.word	__LibFunc" #number);		\
asm	("	.asciz	" EVAL(name));				\
asm	("	.alignn	256");

#endif	// #ifndef __GNUC__

typedef struct tVmmEntry
{
	void (*EntryPoint)(void);
	VMMCONTEXT vc;
} tVmmEntry;

global tVmmEntry Ve;

#endif					// #ifndef __vWARE_H__
