/******************************************************************************
*
* Module: shell.h
*
* Description: Shell Module Definitions
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:      See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#ifndef __SHELL_H__
#define __SHELL_H__


typedef struct tShellVar
{
    UINT32 *pNumber;
    char *Name;
    UINT8 Type;
    UINT8 Flags;
    UINT8 Size;
    
    /*
     * If the number of elements exceeds 32, the elements above 32 will always
     * be displayed.  I.E. - there will be no way to hide these values through
     * the isHidden field.
     */
     
    UINT8 Elements;
    
    /*
     * The IsHidden flags will prevent the variable from being displayed by the
     * .shell or shell commands.  Each bit corresponds to an element, with LSB 
     * corresponding to the first element.
     */
     
    UINT32 isHidden;
    void (*Vector)(struct tShellVar *pSS, int Element, void *v);
    char **Selections;
} tShellVar;

typedef struct tShellAction
{
    UINT16 Number;
    UINT16 Flags;
    char *Name;
    void (*Vector)(void);
} tShellAction;

#ifdef  __GNUC__
#define ShellVar(number, name, type, flags, size, elements, hidden, vector, selections) \
    static tShellVar ShellVar##number __attribute((section(".shellvar"))) = \
    { number, type, flags, name, size, elements, hidden, vector, selections }

#define ShellAction(number, name, flags, vector)    \
    static tShellAction ShellAction##number __attribute((section(".shellaction"))) = \
    { number, flags, name, vector }

#else   // #ifdef __GNUC__

#define ShellVar(number, name, type, flags, size, elements, hidden, vector, selections)\
global const __ ## number = number;     \
asm ("  .section    .vstrings,1,r");    \
asm ("cmdnames: .set    .");            \
asm ("  .asciz  " #name);               \
asm ("  .section    .shellvar,4,r");    \
asm ("  .word   __" #number);           \
asm ("  .word   cmdnames");             \
asm ("  .byte   " EVAL(type));      \
asm ("  .byte   " EVAL(flags));     \
asm ("  .byte   " EVAL(size));      \
asm ("  .byte   " EVAL(elements));  \
asm ("  .word   " EVAL(hidden));    \
asm ("  .word   " #vector);             \
asm ("  .word   " #selections);

#define ShellAction(number, name, flags, vector)\
asm ("  .section    .vstrings,1,r");    \
asm ("cmdnames: .set    .");            \
asm ("  .asciz  " #name);               \
asm ("  .section    .shellaction,4,r"); \
asm ("  .half   " EVAL(number));        \
asm ("  .half   " EVAL(flags));     \
asm ("  .word   cmdnames");             \
asm ("  .word   " #vector); 

#endif  // #ifndef __GNUC__

global void *GetShellVarPointer(UINT32 number);
global UINT32 ReadShellVars(void *pShellVars, UINT32 Size);
global UINT32 WriteShellVars(void *pShellVars, UINT32 Size);

global UINT32 ShellVarValueMin;
global UINT32 ShellVarValueMax;
global UINT32 ShellVarVariable1;
global UINT32 ShellVarVariable2;

global const char __SHELLVAR_START[];
global const char __SHELLVAR_END[];
global const char __SHELLACTION_START[];
global const char __SHELLACTION_END[];

#define SHELLVARS ((UINT32)(__SHELLVAR_END - __SHELLVAR_START) / sizeof(tShellVar))
#define SHELLACTIONS ((UINT32)(__SHELLACTION_END - __SHELLACTION_START) / sizeof(tShellAction))


// Shell Object Types
// These cannot be enums because they are needed by the assembler in the
// ShellVar macro.
#define SHELLTYPE_IP        0
#define SHELLTYPE_MAC       1
#define SHELLTYPE_UINT8     2
#define SHELLTYPE_UINT16    3
#define SHELLTYPE_UINT32    4
#define SHELLTYPE_UINT64    5
#define SHELLTYPE_STRING    6
// add new types here
#define SHELLTYPE_SELECTION2    200
#define SHELLTYPE_SELECTION3    201
#define SHELLTYPE_SELECTION4    202
#define SHELLTYPE_SELECTION5    203
#define SHELLTYPE_SELECTION6    204
#define SHELLTYPE_SELECTION7    205

//  Values for isHidden field
#define SHELLVAR_HIDDEN         1
#define SHELLVAR_DISPLAY        0

#define HIDE_ELE_MASK(element)      (1<<(element))


// Shell Flags and Actions
// Every flag/action must be a power of two.
// Add actions contiguously from 1 to 0x80.
#define SHELLFLAG_FORMAT    1
#define SHELLFLAG_REBOOT    2

#define SHELLACTION_BEFORESAVING    1
#define SHELLACTION_AFTERSAVING     2

// Link works backwards.  That is, setting the SHELLFLAG_LINKED in a SHELLSETTING
// structure means that is is linked with the previous SHELLSETTING structure.
#define SHELLFLAG_LINKED    0x80

#endif          // #ifndef __SHELL_H__
