/* XScale_int.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: 
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */
/******************************************************************************
*
* Module: XScale_int.c
*
* Description: Tests XScale FIQ and IRQ interrupts
*
* Project:     Deer Island Diagnostics
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:       
*					      I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*******************************************************************************/

//  Includes
#include "vm.h"
#include "cmd.h"

extern void Set_LED(char* buffer);

//  Local Defines
#define XSCALE_INT_START 1
#define XSCALE_INT_END 1
#define FIQ_ENABLE 0x1
#define FIQ_ENABLE_CLEAR 1
#define FIQ_ENABLE_SET 1
#define FIQ_STATUS 1
#define FIQ_RAW_STATUS 1
#define FIQ_SOFT 1

#define IRQ_ENABLE		  1
#define IRQ_ENABLE_CLEAR 1
#define IRQ_ENABLE_SET 1
#define IRQ_STATUS 1
#define IRQ_RAW_STATUS 1
#define IRQ_SOFT 1

////////////////////////////////////////////////////
// Definition for XScale_int Diagnostic
////////////////////////////////////////////////////
	   
int XScale_int(void* pUnused1,void* pUnused2)
{
	printf("This is the XScale_int.c diagnostic\n");

	printf("Start FIQ int test\n");

 // Check for pending interrupts in FIQ_STATUS and FIQ_RAW_STATUS
    if (GET32(FIQ_STATUS && 0x01) != 0){
		printf("Pending FIQ Interrupts\n");
		return 1;
	   }

 //	Write a 1 to bit 0 of the FIQ_ENABLE_CLEAR register
 	PUT32(FIQ_ENABLE_CLEAR, 0x01);


 //	Check that bit 0 of the FIQ_ENABLE register is clear
    if (GET32(FIQ_ENABLE && 0x01) != 0){
		printf("Error FIQ_ENABLE not cleared\n");
		return 1;
	   }


 //	Write a 0 to bit 0 of the FIQ_SOFT register
 	PUT32(FIQ_SOFT, 0x0);

 //	Check bit 0 clear in FIQ_RAW_STATUS
 //	Check bit 0 clear in FIQ_STATUS
    if ((GET32(FIQ_STATUS && 0x01)) || (GET32(FIQ_RAW_STATUS && 0x01)) != 0){
		printf("Pending FIQ Interrupts\n");
		return 1;
	   }

 //	Write a 1 to bit 0 of the FIQ_SOFT register
 	PUT32(FIQ_SOFT, 0x01);

 //	Check bit 0 set in FIQ_RAW_STATUS
    if (GET32(FIQ_RAW_STATUS && 0x01) != 1){
		printf("Error FIQ_RAW_STATUS not set\n");
		return 1;
	   }

 //	Check bit 0 clear in FIQ_STATUS
    if (GET32(FIQ_STATUS && 0x01) != 0){
		printf("Error FIQ_STATUS not set\n");
		return 1;
	   }

 //	Write a 1 to bit 0 of the FIQ_ENABLE_SET register
 	PUT32(FIQ_ENABLE_SET, 0x01);

 //	Check bit 0 set of FIQ_STATUS register
    if (GET32(FIQ_STATUS && 0x01) != 1){
		printf("Error FIQ_STATUS not set\n");
		return 1;
	   }

 //	Clear the FIQ interrupt 
 	PUT32(FIQ_SOFT, 0x0);
 	PUT32(FIQ_ENABLE_CLEAR, 0x01);

 //	FIQ test complete





	return 0;
}




// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_XSCALE_INT,"XScale_int",XScale_int,DIAGNOSTIC,"XScale_int","XScale IRQ and FIQ interrupt test");
CommandEnd();

