/* board.c - Board File */

/* Copyright 1984-1999 Wind River Systems, Inc. 
 * Parts Copyright 2002-2004 Intel Corporation 
 */

/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/board.c 2     3/12/04 8:22a Ajpasqui $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/board.c $
 * 
 * 2     3/12/04 8:22a Ajpasqui
 * Changed rev to 3.50.310D
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 3     11/19/03 12:16p Daly
 * updated to 3.50.310C
 * 
 * 1     11/19/03 12:08p Daly
 * 
 * 13    10/29/03 1:38p Tepulkow
 * Changed version to 310B for flash timeout fix
 * 
 * 14    10/27/03 2:14p Tepulkow
 * Changed version tpo 310A from 310
 * 
 * 11    10/23/03 3:19p Ajpasqui
 * Changed the rev to 3.50.310 for PR2 release
 * 
 * 10    10/22/03 4:12p Daly
 * Merging from $/dev_2000/BoardSupport/VisionWARE2.1/PROJECTS/IXP
 * 
 * 10     10/22/03 2:46p Ajpasqui
 * Changed the version from 4.00.NoBldNum to 3.50.NoBldNum
 * 
 * 9     10/22/03 3:59p Daly
 * fixed 2 typos
 * 
 * 8     10/22/03 2:09p Daly
 * updated with review comments
 * 
 * 7     10/02/03 4:01p Ajpasqui
 * Roll back to 4.00.NoBldNum for this Boot Manager only release
 * 
 * 6     10/02/03 3:21p Ajpasqui
 * Changed version field to 5.00.NoBldNum
 * 
 * 5     9/11/03 10:09a Daly
 * improved SRAM channel initialization, bumped buildnum.
 * 
 * 4     8/20/03 3:02p Daly
 * updated old revision history
 * 
 * 3     8/20/03 2:29p Daly
 * added $Header and $Log
 *
modification history
--------------------
01a,21may02, jjd  created from 01a BRH
01b,28aug02, jjd  added SRAM initialization
01c,20nov02, jjd  fixed brd_pcimem_pci2cpu()
01d,29jan03, ajp  print VERSION string and Ingress/Egress prompt
01e,29jan03, ajp  added call to InitDoorbellIRQ()
01f,11feb03, jjd  enable PCIB
01g,13feb03, jjd  added SRAM init code, PCI interrupt enable at CPLD
01h,25feb03, jjd  more SRAM init
01i,28feb03, jjd  SRAM/RDRAM size display, TCAM discovery.  SRAM CLK_CONTROL
                  reset for odd divisors
01j,04mar03, jjd  fixed rom_outstring(), delay timer, MAC address handling
01k,06mar03, jjd  added support for reading temperature sensor
01l,12mar03, jjd  changed isEgressNpu() to look at PCI_CFG_BOOT_HOST strap
01m,17mar03, ajp  allow disabling LED display during diags
01n,25mar03, ajp  doorbell interrupt enable
01o,27mar03, jjd  fixed SRAM size printout
01p,27mar03, jjd  cont'd: memsize array should be 4 elements
01q,28mar03, jjd  use TIMER4 for delay ticks
01r,02apr03, jjd  removed SRAM odd divisor init, moved to boardinit.s
01s,23apr03, jjd  added SRAM slew rate init for completeness
01t,28apr03, jjd  display NPU frequency
02a,23mar03, jjd  reset command added
02b,03jun03, ajp  changed VERSION to 2.xx.xx
02c,09jul03, jjd  generalize frequency calculation
03a,25jul03, ajp  changed VERSION to 3.xx.xx
03b,20aug03, jjd  manual update modification history, will now be maintained by
                  vss log
*/

/*
DESCRIPTION

Board File - User Functions

INCLUDE FILES:  vm.h
				timestamp.h
				revision.h
				pci.h
				estack.h
				cmd.h
				flash.h
*/

/* includes */
#include "vm.h"
#include "timestamp.h"
#include "revision.h"
#ifdef PCI_ON
#include "pci.h"
#endif
#ifdef ESTACK_ON
#include "estack.h"
#endif
#include "timer.h"
#include "cmd.h"
#include "flash.h"
#include "ADM1031.h"
#include "ixp2x00.h"

/* defines */
#ifndef SZ_1M
#define SZ_1M 0x100000
#endif

#define IDTTCAM_GET_DEVID 0x1800001
#define IDTTCAM 0x0021e067 /* device id of IDT TCAM device */
#define IDTTCAM_ONE_EARLY (IDTTCAM >> 16)
#define IDTTCAM_HALF_EARLY ((IDTTCAM >> 16) | (((IDTTCAM >> 16) & 0xFFFF) << 16))
#define IDTTCAM_ONE_LATE (IDTTCAM << 16)
#define IDTTCAM_HALF_LATE ((IDTTCAM << 16) | (((IDTTCAM << 16) & 0xFFFF0000) >> 16))
#define SRAMVAL 0x11224488 /* memory pattern in SRAM */
#define SRAMVAL_ONE_EARLY (SRAMVAL >> 16)
#define SRAMVAL_HALF_EARLY ((SRAMVAL >> 16) | ( ((SRAMVAL >> 16) & 0xFFFF) << 16))
#define SRAMVAL_ONE_LATE (SRAMVAL << 16)
#define SRAMVAL_HALF_LATE ((SRAMVAL << 16) | ( ((SRAMVAL << 16) & 0xFFFF0000) >> 16))

/* globals */
#ifdef PCI_ON
UINT32 PciBaseAddr;
#endif
int EgressTemp;
int IngressTemp;
int UpdateTemp;
int Led_Test;

Region ("RDRAM", REGION_RAM, RAM1START, (RAM1START + RAM1SIZE - 1));
Region ("SSRAM0", REGION_SRAM0, 0x80000000, 0x807FFFFF);
Region ("SSRAM1", REGION_SRAM1, 0x90000000, 0x907FFFFF);
Region ("SSRAM2", REGION_SRAM2, 0xa0000000, 0xa07FFFFF);
Region ("SSRAM3", REGION_SRAM3, 0xb0000000, 0xb07FFFFF);

/*Region ("80312 Registers", REGION_MMR, 0x400, 0x1FFF);*/
Region ("Boot Flash", REGION_BOOTFLASH, BOOTFLASHSTART, BOOTFLASHEND);
Region ("MSF", REGION_MSF, 0xc8000000, 0xc9FFFFFF);
Region ("Scratch", REGION_SCRATCH, 0xca000000, 0xca003FFF);
Region ("Gasket", REGION_GASKET, 0xd6000000, 0xd600FFFF);
Region ("PCI Window", REGION_PCIWINDOW, 0xE0000000, 0xFFFFFFFF);
/*Region ("EPLD", REGION_EPLD, 0xFE800000, 0xFE8FFFFF); */


CommandGroup (CMDGROUP_BOARD, "USER DIAGNOSTICS");
CommandGroup (CMDGROUP_VFPGA, "vFPGA COMMANDS");

/* Largest command file that can be loaded */
UINT32  MaxCmdFileSize=0x8000;

/* locals */
static int nDevsSram[4];
static int nDevsTcam[4];
static HANDLER hDataAbort;

/* forward prototypes */
void EnableExternalIrq (void);
extern void InitMMU (void);
void DataAbortHandler (void *);
void InitSRAMChannels(void);

extern char *tempstring;

#ifdef FLASH_ON
extern FLASHOBJECT FREEFLASHOBJECT;
#endif
#ifdef FFS_ON
extern FLASHOBJECT  FFSFLASHOBJECT;
#endif
#if 1
extern FLASHOBJECT BOOTFLASHOBJECT;
#endif

/* forward prototypes */
#if ( defined(PCI_ON) && defined(PCI_AUTO_ON) )
static int sysPciAutoConfigInclude (PCI_SYSTEM *pSys,PCI_LOC *pLoc,UINT devVend);
static UINT8 sysPciAutoConfigIntAsgn (PCI_SYSTEM *pSys,PCI_LOC *pLoc,UCHAR pin);
#endif

/******************************************************************************
*
* rom_outstring - print debug messages
*
* This function prints out strings.
*
* RETURNS: N/A
*/
#include "serial16550.h"
void rom_outstring
        (
        char *string
        )
        {
        int timeout;
        U550 *uart = (U550 *)SERIALADR;

        while (*string)
            {
            timeout = 10000;
            while(--timeout > 0)
                {
                if (uart->LSTAT & LSR_TEMT) break;
                }
            if (*string == '\n')
                {
                uart->TXDATA = '\r';
                timeout = 10000;
                while(--timeout > 0)
                    {
                    if (uart->LSTAT & LSR_TEMT) break;
                    }
                }
            uart->TXDATA = *string++;
            }
        }

#if defined(PCI_ON)
UINT32 brd_pcimem_cpu2pci(addr,bridge);
UINT32 brd_pcimem_pci2cpu(addr,bridge);        
/******************************************************************************
*
* brd_pcimem_cpu2pci - change CPU view address to PCI view
*
* This function works around an apparent bug in the Diab compiler
*
* RETURNS: UINT32
*
* ERRNO: N/A
*/
UINT32 brd_pcimem_cpu2pci
       (
       UINT32 addr,
       UINT32 bridge
       )
       {
       return (addr + PciBaseAddr);
       }       

/******************************************************************************
*
* brd_pcimem_pci2cpu - change PCI view address to CPU view
*
* This function works around an apparent bug in the Diab compiler
*
* RETURNS: UINT32
*
* ERRNO: N/A
*/
UINT32 brd_pcimem_pci2cpu
       (
       UINT32 addr,
       UINT32 bridge
       )
       {
		if (PciBaseAddr & addr)
       		return (addr - PciBaseAddr);
		else
			return (addr);
       }       
#endif /* PCI_ON */

/******************************************************************************
*
* isEgressNpu - decide whether this is egress/ingress NPU
*
* Determine using the STRAP_OPTIONS register
*
* RETURNS: UINT32
*
* ERRNO: N/A
*/
UINT32 isEgressNpu()
{
        return(*(volatile UINT *)STRAP_OPTIONS & STRAP_OPTIONS_CfgBootHost);
}

/******************************************************************************
*
* EarlyBoardInit - Early Board Initialization
*
* Early board initialization.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void EarlyBoardInit (void)
	{
        UINT16 tmp16;
        UINT32 tmp32;

        rom_outstring("\nEarlyBoardInit\n");
#ifdef ESTACK_ON
	/* 
	 * Install temporary data-abort handler.
	 * GEV made dependent on ESTACK -- this handler supports PCI autoconfig
	 */
	hDataAbort = InstallHandler (VECTOR_DATA_ABORT, (HANDLER)DataAbortHandler,
								 (void *)&TempContext);
#endif
	/* enable delayticks timer */

        rom_outstring("Delay timer calibration: 3");
	DelayNMsec(1000);
	rom_outstring("\b2");
	DelayNMsec(1000);
	rom_outstring("\b1");
	DelayNMsec(1000);
	rom_outstring("\b0\n");

        InitSRAMChannels();

	InitMMU ();

	/* Protect MMU Mapped Exceptions */
	Protect ((void *)RAM1START, 0x2000);
	Protect ((void *)(RAM1END-0x10000),0x10001);

	/* Host Bridge Configuration */
/* #ifdef ESTACK_ON */
  #ifdef PCI_ON
{
volatile int *led_display = (int *)(0xc700010c);
*led_display-- = (int)'P';
*led_display-- = (int)'C';
*led_display-- = (int)'I';
*led_display = (int)' ';
}
	 PciPrimaryPCIInit ();
{
volatile int *led_display = (int *)(0xc700010c);
*led_display-- = (int)'P';
*led_display-- = (int)'C';
*led_display-- = (int)'I';
*led_display = (int)'1';
}
	/* PCI Configuration LIbrary Initialization */
	pciConfigLibInit (PCI_MECHANISM_0,(int)PciCfgRead,(int)PciCfgWrite,0);

#define BusNo	0
#define DevNo	(PCI_MAX_DEV + 1)
#define FuncNo	0

        if (IS_EGRESS_NPU)
            {
	#ifdef PCI_AUTO_ON
	{
                /* XXX put if (MASTER) around this stuff */
		static PCI_SYSTEM sysParams;

		/* 32-bit Non-prefetchable Memory Space */

		sysParams.pciMemIo32 =          PCI_PP_MEM_START;
		sysParams.pciMemIo32Size =      PCI_PP_MEM_SIZE;

		/* 32-bit Prefetchable Memory Space */

		sysParams.pciMem32 =            0; /* not supported on Deer Island */
		sysParams.pciMem32Size =        0;

		/* 16-bit ISA I/O Space - First block of memory in the I/O space */

		sysParams.pciIo16 =         0; /* not supported on Deer Island */
		sysParams.pciIo16Size =     0;

		/* 32-bit PCI I/O Space - Remaining memory in the I/O space */

		sysParams.pciIo32 =         PCI_PP_IO_START;
		sysParams.pciIo32Size =     PCI_PP_IO_SIZE;

		/* Configuration space parameters */

		sysParams.cacheSize =           32; /* XXX _CACHE_ALIGN_SIZE/4 */
		sysParams.maxLatency =          0x40;
		sysParams.autoIntRouting =      FALSE;
		sysParams.includeRtn =          sysPciAutoConfigInclude;
		sysParams.intAssignRtn =        sysPciAutoConfigIntAsgn; /* was NULL */
		sysParams.bridgePreConfigInit =     NULL;
		sysParams.bridgePostConfigInit =    NULL;

		/* Perform AutoConfig */

		pciAutoConfig (&sysParams);
	}

	#endif /* PCI_AUTO_ON */
            }
        else /* !IS_EGRESS_NPU */
            {
            /* wait until PCI bus is configured before continuing */
            do {
               pciConfigInWord(BusNo,DevNo,FuncNo,PCI_CFG_COMMAND,&tmp16);
            } while (!(tmp16 & PCI_CMD_MASTER_ENABLE));
            }
        /* set pci base address for pci memory mapping macros */
        pciConfigInLong(BusNo,DevNo,FuncNo,PCI_CFG_BASE_ADDRESS_2,&PciBaseAddr);
        PciBaseAddr &= ~0xF;
  #endif /* PCI_ON */
/* #endif /* ESTACK_ON */

	/* Initialize the PICs */
	PicInitIxp2000IRQ ();
	PicInitIxp2000FIRQ ();
	}

/* force this function to be called before any C++ constructors */
Init (200, EarlyBoardInit);


/******************************************************************************
*
* PrintHeader - Print Header
*
* Print a signon message.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
#define VERSION "3.50.310D"

void PrintHeader (void)
	{
	universal ipAddr;
	universal macAddr;

	int frequency;
        int i,memsize[4];

	printf ("\n \nWind River visionWARE v%s Deer Island Board", VISMON_REV);
	printf ("\nWind River +1(781) 828 5588\n");
	printf ("Build #%u, %s %s\n \n", BUILDNUMBER, BUILDDATE, BUILDTIME);

	frequency = (*(UINT *)STRAP_OPTIONS & STRAP_OPTIONS_CfgPllMask) >> 8;
	frequency *= 50;
	if (frequency > 1000)
	    printf("uEngine: 1.%d GHz, XScale: %d MHz\n",
		(frequency % 1000)/100, frequency/2);
	else
	    printf("uEngine: %d Mhz, XScale: %d Mhz\n", frequency,frequency/2);
	for (i=0;i<3;i++)
	    {
	    if (i && !(*(UINT *)(RDRAM_0_CONTROL + (0x400 * i)) & 
                       RDRAM_CONTROL_NumChannelsMask))
	        continue;
	    memsize[i] = (8 << ((*(UINT *)(RDRAM_0_CONTROL + (0x400 * i)) & 
                                 RDRAM_CONTROL_SizeMask) >> 8));
            }
	printf ("RDRAM Total : %dMB\n",(memsize[0]+memsize[1]+memsize[2]));
	for (i=0;i<3;i++)
            {
            if (memsize[i])
	        printf ("\tChannel %d : %dMB\n",i,memsize[i]);
	    else
                break;
	    }
	printf("Sram Channel ");
        for (i=0;i<4;i++)
            {
            memsize[i] = (((*(UINT *)(SRAM_0_CONTROL + (0x100000*i)) & 
                            SRAM_CONTROL_SizeMask) * nDevsSram[i] * 2) >> 7);
	    printf ("%d: %dMB%s",i,memsize[i],(nDevsTcam[i] ? ",TCAM" : ""));
	    if (i != 3) printf(", "); 
            }
	printf("\nIntel Version %s\n\n", VERSION);   

#ifdef ESTACK_ON
#if (TGTBOARD != IXMB2800)
	macAddr.pVoid = (void *)GetShellVarPointer (SHELLVAR_MAC);
#else
	macAddr.pVoid = (void *)&MyMAC;      /* Deer Island has serial eeprom */
#endif
	ipAddr.pVoid = (void *)GetShellVarPointer (SHELLVAR_IP);
	printf ("MAC : %02X-%02X-%02X-%02X-%02X-%02X IP : %d.%d.%d.%d",
			macAddr.pByte[0],macAddr.pByte[1],macAddr.pByte[2],
			macAddr.pByte[3],macAddr.pByte[4],macAddr.pByte[5],
			ipAddr.pByte[0],ipAddr.pByte[1],ipAddr.pByte[2],
			ipAddr.pByte[3]);
	printf ("\nType \"shell\" to set IP and/or MAC addresses\n");
#endif

	printf ("Type \"help\" to see available commands\n \n");
	}

Init (990, PrintHeader);

/******************************************************************************
*
* ReadTemp - Read Board Temperature
*
* RETURNS: N/A
*
* ERRNO: N/A
*/
static void ReadTemp()
{
	unsigned char buf;

	i2c_read(I2CADM1031,ADM1031_Remote_Temp_1_Value,&buf);
	EgressTemp = (int)buf;
	i2c_read(I2CADM1031,ADM1031_Remote_Temp_2_Value,&buf);
	IngressTemp = (int)buf;
	UpdateTemp = 1;
}

static HANDLE TempHandle;

void DisableTemp()
{
	if (!IS_EGRESS_NPU)
	    RemoveTimer(TempHandle);
}

void EnableTemp()
{
	if (!IS_EGRESS_NPU)
	    InstallPeriodicTimer(10000,ReadTemp,0);
}
/******************************************************************************
*
* BoardInit - Board Init
*
* Final Board Initialization
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void BoardInit (void)
	{
	*(volatile int *)(0xc7000140) = 1;

	/* Enable Interrupts */
	EnableInterrupts ();

#ifdef ESTACK_ON
	if (*(UINT32 *)GetShellVarPointer (SHELLVAR_DEFAULTCHANNEL) == 1)
		Ethernet (0, 0);
#endif

	if (!IS_EGRESS_NPU)
	    {
	    unsigned char cfgreg;
	    i2c_write(I2CADM1031,ADM1031_Remote_1_Therm_Limit,0x7f);	/* max limit */
	    i2c_write(I2CADM1031,ADM1031_Remote_2_Therm_Limit,0x7f);	/* max limit */
	    i2c_read(I2CADM1031,ADM1031_Config_1,&cfgreg);
	    cfgreg |= 1;
	    i2c_write(I2CADM1031,ADM1031_Config_1,cfgreg);
	    TempHandle = InstallPeriodicTimer(10000,ReadTemp,0);
	    }
	InitDoorbellIRQ(); 

	}

Init (599, BoardInit);


/******************************************************************************
*
* BoardLaunch - Board Launch
*
* Called as part of the Launch command. Populates structure for OS
* being bootloaded to understand how the board has been setup
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void BoardLaunch 
	(
	tOsParams *pOS
	)
	{
	/* Memory blocks */
	pOS->RamStart = RAM1START;
	pOS->RamSize = RAM1SIZE;
	pOS->FlashStart = FLASHSTART;
	pOS->FlashSize = FLASHSIZE;
	pOS->MmrBase = 0x1000;

	/* MMU stuff */
	pOS->MmuTableBase = pOS->MmuBaseTable = (RAM1START + RAM1SIZE)
						- (pOS->MmuTableSize = 65536);

	/* System stuff */
	pOS->XtalSpeed = CLOCKSPEED;

	}

Fini (0, BoardLaunch);


/******************************************************************************
*
* IsFlash - Is Address Flash Memory
*
* Called throughout the code, this function should return TRUE or 1 if the 
*  Addr passed in is inside a flash region. 
* 
* This is very specific to user board. The algorithm may be more complex 
* depending board design
*
* RETURNS: pointer to a FLASHOBJECT structure
* 
* ERRNO: N/A
*/

FLASHOBJECT *IsFlash
	(
	UINT32 Addr
	)
	{
#ifdef FLASH_ON
	if ((Addr >= FREEFLASHSTART) && (Addr <= FREEFLASHEND))
		return(&FREEFLASHOBJECT);
#endif

#if 1
	if ((Addr >= BOOTFLASHSTART) && (Addr <= BOOTFLASHEND))
		return(&BOOTFLASHOBJECT);
#endif

#ifdef FFS_ON
	if ((Addr >= FFSFLASHSTART) && (Addr <= FFSFLASHEND))
		return(&FFSFLASHOBJECT);
#endif

	return(0);
	}


/******************************************************************************
*
* BoardReset - Board Reset
*
* Cause a hardware reset.  Some boards have no way to do this from software.  
* In those cases we just display a message asking the user to power-cycle the 
* board.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/

void BoardReset (void)
	{
	*(volatile UINT *)RESET_0 = RESET_0_RstAll;
	while (1);
	}

Command (CMD_RESET, "reset", BoardReset, 0, "Reset visionWARE", "");
CommandEnd ();


/******************************************************************************
*
* BoardIOCtrl - Board I/O Control
*
* Provides a board specific call for generic code to ask for board or system 
* resources
*
* RETURNS: STATUS
* 
* ERRNO: N/A
*/

UINT32 BoardIOCtrl
	(
	UINT32 nFuncCode, 
	void *pArgs
	)
	{
	UINT32          nStatus;
	UINT32          begin;
	int             delta;
	volatile UINT32 *yavgtsr;
	volatile int i,j,k;

	switch (nFuncCode)
		{
		case DELAYTICKS:
			{
#if 1
			/* this will bust if (currenttime < 0 && (currenttime - ticks) > 0) */
			unsigned int end;
			end  =  0xffffffff - *(UINT *)pArgs;
			*(volatile UINT *)TIMER_4_CLD = 0xffffffff;
			*(volatile UINT *)TIMER_4_CTL = TIMER_CTL_Act;
			while (*(volatile UINT *)TIMER_4_CSR > end)
				;
			*(volatile UINT *)TIMER_4_CTL = 0;
#else
			for (i=0;i<1000;++i)
				for (j=0;j<(150);++j)
					k=0;
#endif
			/* 
			 * Delay for number of ticks
			 * pArg is number of ticks 
			 */
			}
			break;

#ifdef YAVAPAI_CHIPSET
		case STARTCOUNTERS:
			YAVAPAI.PCIPICU.GPOD |= 0x01;
			break;
		case ENABLECOUNTERS:
			/* setup gates 0,1,2 of timer to stop counting */
			YAVAPAI.PCIPICU.GPOD &= 0xFFFFFFFC;
			/* allow GPIO for gates 0,1,2 to be enabled */
			YAVAPAI.PCIPICU.GPOE &= 0xFFFFFFFC;
			break;
#endif

		default:
			nStatus = BIOBADFUNCCODE;
			break;
		}
	}

/******************************************************************************
*
* InitSRAMChannels - Initialize SRAM
*
* Initialize QDR SRAM interface.  This code works if no external decoding is
* used.
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
typedef struct sram_channel {
    UINT control;
    UINT parity_stat1;
    UINT parity_stat2;
    int dummy0[63];
    UINT int_pipeline;
    int dummy1[61];
	UINT dft_ctl;
	int dummy1a[9];
    UINT rx_dll;
    int dummy2[5];
    UINT rd_ptr_offset;
    UINT rx_deskew;
    int dummy3[46];
    UINT rcmp_setup_ctl;
    int dummy4[4];
	UINT rcmp_scomp_override;
    UINT rcmp_slew_index_ctl;
    int dummy5[9];
    UINT rcmp_addr_pmos_pu_slew[4];
    UINT rcmp_addr_nmos_pd_slew[4];
    UINT rcmp_data_pmos_pu_slew[4];
    UINT rcmp_data_nmos_pd_slew[4];
    UINT rcmp_kclk_pmos_pu_slew[4];
    UINT rcmp_kclk_nmos_pd_slew[4];
    UINT rcmp_dq_pmos_pu_slew[4];
    UINT rcmp_dq_nmos_pd_slew[4];
} SRAM_CHANNEL;

void InitSRAMChannels(void)
    {
    volatile SRAM_CHANNEL *sram_ctrl;
    volatile UINT *sram;
    UINT temp,sizebits;
    UINT sram_ctrl_addr[] = {0xcc010000,0xcc410000,0xcc810000,0xccc10000};    
    int i,j,k,nchannel;
    int dllLow,dllHigh;
    int dllFound,dllLowFound;
    int tcamFound;
    int currentDll;
	int notAstep;
	int rd_ptr_offset;

	if ( (*(volatile UINT *)PRODUCT_ID & PRODUCT_ID_MajRevMask) > PRODUCT_ID_Astep) {
		rom_outstring("Detected B0\n");
		notAstep = 1;
	}
	else
		notAstep = 0;

    for (nchannel=0,sram_ctrl=(SRAM_CHANNEL *)SRAM_0_CONTROL,
                 sram=(UINT *)SRAM_BASE;nchannel<4;nchannel++,sram += (SZ_1M * 256)/4)
        {
        rom_outstring("Initializing SRAM channel ");
        switch (nchannel)
            {
			case 0: rom_outstring("0: Centering Data Eye: ");
					if (notAstep)
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(2);
					else
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(0);
					break;
			case 1: rom_outstring("1: Centering Data Eye: ");
					if (notAstep)
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(2);
					else
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(0);
					break;
			case 2: rom_outstring("2: Centering Data Eye: ");
					if (notAstep)
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(2);
					else
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(0);
					break;
			case 3: rom_outstring("3: Centering Data Eye: ");
					if (notAstep)
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(2);
					else
						rd_ptr_offset = SRAM_RD_PTR_OFFSET_PtrOffset(0);
					break;
            }
        sram_ctrl = (SRAM_CHANNEL *)(sram_ctrl_addr[nchannel]);

		if (notAstep)
			{
			sram_ctrl->rcmp_scomp_override |= SRAM_RCMP_SCOMP_OVERRIDE_TxDllDisable;
			sram_ctrl->rcmp_scomp_override |= SRAM_RCMP_SCOMP_OVERRIDE_RxDllDisable;
			sram_ctrl->rcmp_scomp_override &= ~SRAM_RCMP_SCOMP_OVERRIDE_KClockEnable;
			DelayNMsec(10);
			sram_ctrl->rcmp_scomp_override &= ~SRAM_RCMP_SCOMP_OVERRIDE_TxDllDisable;
			DelayNMsec(10);
			sram_ctrl->rcmp_scomp_override &= ~SRAM_RCMP_SCOMP_OVERRIDE_RxDllDisable;
			DelayNMsec(10);
			}

		sram_ctrl->control = SRAM_CONTROL_Size32M;
	for (k=0;k<4;k++)
	    {
	    sram_ctrl->rcmp_addr_pmos_pu_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_addr_nmos_pd_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_data_pmos_pu_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_data_nmos_pd_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_kclk_pmos_pu_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_kclk_nmos_pd_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_dq_pmos_pu_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    sram_ctrl->rcmp_dq_nmos_pd_slew[k] = SRAM_SLEW_POWERUP_DEFAULT;
	    }

		if (notAstep)
			sram_ctrl->int_pipeline = SRAM_INT_PIPELINE_Nominal;
		else
        	sram_ctrl->int_pipeline = SRAM_INT_PIPELINE_Add1;

	sram_ctrl->rx_dll = SRAM_RX_DLL_SlaveSel(0);

	(void)sram_ctrl->rx_dll;
	sram_ctrl->rx_deskew = SRAM_RX_DESKEW_MasterSel(0x8);
	(void)sram_ctrl->rx_deskew;

	sram_ctrl->rcmp_setup_ctl = (SRAM_RCMP_SETUP_CTL_FilterClamp1|
				SRAM_RCMP_SETUP_CTL_RcmpPeriod2ms);
	DelayNMsec(10);
	sram_ctrl->rcmp_slew_index_ctl = (SRAM_SLEW_INDEX_CTL_ASlewClamp|
					SRAM_SLEW_INDEX_CTL_AStrength(6)|
					SRAM_SLEW_INDEX_CTL_DSlewClamp|
					SRAM_SLEW_INDEX_CTL_DStrength(6)|
					SRAM_SLEW_INDEX_CTL_KSlewClamp|
					SRAM_SLEW_INDEX_CTL_KStrength(6)|
					SRAM_SLEW_INDEX_CTL_DQSlewClamp|
					SRAM_SLEW_INDEX_CTL_DQStrength(6));
	DelayNMsec(10);
	sram_ctrl->rcmp_setup_ctl |= (SRAM_RCMP_SETUP_CTL_RcmpLock|SRAM_RCMP_SETUP_CTL_TablesProgrammed);
	if (notAstep)
		sram_ctrl->dft_ctl = 0x2000;
	DelayNMsec(10);
	if (notAstep)
		{
		sram_ctrl->rcmp_setup_ctl &= ~SRAM_RCMP_SETUP_CTL_RcmpLock;
		sram_ctrl->dft_ctl = 0;
		DelayNMsec(1);
		sram_ctrl->rcmp_scomp_override |= SRAM_RCMP_SCOMP_OVERRIDE_KClockEnable;
		sram_ctrl->rcmp_setup_ctl |= SRAM_RCMP_SETUP_CTL_RcmpLock;
		DelayNMsec(10);
		}

	sram_ctrl->rd_ptr_offset = rd_ptr_offset;

#if (TGTBOARD == IXMB2800)
	*(volatile UINT *)RESET_0 |= RESET_0_ExtRst;
	DelayNMsec(1);
	/* see if there's a TCAM on this port */
	sram[24576] = IDTTCAM_GET_DEVID;
	DelayNMsec(1);
#endif 
        sram[1] = SRAMVAL;
		dllLowFound=dllLow=dllHigh=tcamFound=0;

		for (currentDll=0;currentDll<0x18;currentDll++) 
			{
            sram_ctrl->rx_dll = currentDll;
            (void)sram_ctrl->rx_dll;
            DelayNMsec(1);
            sram_ctrl->rd_ptr_offset = rd_ptr_offset;
			(void)sram_ctrl->rd_ptr_offset;			

			if (sram[1] == SRAMVAL)
				{
				rom_outstring("*");
				if (!dllLowFound) 
					{
					dllLowFound = 1;
					dllLow = currentDll;
					}	
				}
			else if (sram[1] == IDTTCAM)
				{
				rom_outstring("T");
				tcamFound = 1;
				if (!dllLowFound)
					{
					dllLowFound = 1;
					dllLow = currentDll;
					}
				}
			else if (dllLowFound) 
				{
				rom_outstring("-");
				dllHigh = currentDll;
				break;
				}
			else
				rom_outstring("-");
			}
					
        dllFound = (dllLow + dllHigh)/2;
        for (i=0;i<=dllFound;i++)
            {
            sram_ctrl->rx_dll = i;
            (void)sram_ctrl->rx_dll;
            }
        sram_ctrl->rd_ptr_offset = rd_ptr_offset;
		(void)sram_ctrl->rd_ptr_offset;
        rom_outstring(" done\n");

	if (tcamFound)
		rom_outstring("TCAM Found\n");

        rom_outstring("Sizing SRAM channel ");
        switch (nchannel)
            {
            case 0: rom_outstring("0.........");break;
            case 1: rom_outstring("1.........");break;
            case 2: rom_outstring("2.........");break;
            case 3: rom_outstring("3.........");break;
            }
        sram_ctrl->control = SRAM_CONTROL_Size32M;

        /* see if *any* sram exists */
	if (!tcamFound)
                {
                sram[0] = 0xaaaaaaaa;
                sram[1] = 0x00000000;

                if (sram[0] != 0xaaaaaaaa)
                        {
                        nDevsSram[nchannel] = 0;
                        goto found;
                        }

                sram[0] = 0x55555555;
                sram[1] = 0x00000000;
    
                if (sram[0] != 0x55555555)
                        {
                        nDevsSram[nchannel] = 0;
                        goto found;
                        }

                /* 
                 * some sram exists, size this channel
                 * first, find depth of device, then find # of devices
                 */
                nDevsSram[nchannel] = 1;

                for (i = (SZ_1M * 32)/4,j=5; i >= SZ_1M/4; i>>=1,j--)
                sram[i] = j << 7;

                temp = sram[0];	/* reading gives us the sram_size field */
                } // if (!tcamFound)
	else
                {
                temp = SRAM_CONTROL_Size16M;
                nDevsTcam[nchannel]++;
                }
        sizebits = temp >> 7;
        sram_ctrl->control = temp;

        /* probe for device 1 */
        if (temp != SRAM_CONTROL_Size32M)
                {
                sram[(SZ_1M<<sizebits)/4] = 0x12345678;
                sram[0] = 0xffffffff;
                if (sram[(SZ_1M<<sizebits)/4] == 0x12345678)
                        nDevsSram[nchannel]++;
                else
                        goto found; /* don't support non-contiguous portsel */
        	}

        /* probe for device 2 */
        if (temp < SRAM_CONTROL_Size16M)
                {
                sram_ctrl->control |= SRAM_CONTROL_PortCtl1;
                sram[((SZ_1M * 2)<<sizebits)/4] = 0x12345678;
                sram[0] = 0xffffffff;
                if (sram[((SZ_1M * 2)<<sizebits)/4] == 0x12345678)
                        nDevsSram[nchannel]++;
                else
                        {
                        sram_ctrl->control &= ~SRAM_CONTROL_PortCtl1;
                        /* don't support non-contiguous portsel */
                        goto found;
                        }
                }

        /* probe for device 3 */
        if (temp < SRAM_CONTROL_Size4M)
                {
                sram_ctrl->control |= SRAM_CONTROL_PortCtl0;
                sram[((SZ_1M * 3)<<sizebits)/4] = 0x12345678;
                sram[0] = 0xffffffff;
                if (sram[((SZ_1M * 3)<<sizebits)/4] == 0x12345678)
                        nDevsSram[nchannel]++;
                else
                        sram_ctrl->control &= ~SRAM_CONTROL_PortCtl0;
                }
found:
        rom_outstring("done\n");
giveup:		;
        } // for ()
    }

#if ( defined(PCI_ON) && defined(PCI_AUTO_ON) )

/******************************************************************************
* sysPciAutoConfigIntAsgn - Determine the IRQ used for this device
*
* This user-specified routine takes as input both the bus-device-function
* tuple, and an 8-bit quantity containing the contents of the interrupt Pin
* register from the PCI configuration header of the device under consideration.
* The interrupt pin register specifies which of the four PCI Interrupt request
* lines available are connected.
*
*/

static UINT8 sysPciAutoConfigIntAsgn
    (
    PCI_SYSTEM *pSys,
    PCI_LOC *pLoc,
    UCHAR pin
    )
    {

    if (pLoc->bus == 1)
        {
	    return ((UINT8)BRD_MAP_PCI_IRQ(pin,pLoc->device));
        }

    return (-1);

    }

/******************************************************************************
*
* sysPciAutoConfigInclude - Determine if function is to be autoConfigured
*
* This function is called with PCI bus, device, function, and vendor
* information.  It returns an indication of whether or not the particular
* function should be included in the automatic configuration process.
* This capability is useful if it is desired that a particular function
* NOT be automatically configured.  Of course, if the device is not
* included in automatic configuration, it will be unusable unless the
* user's code made provisions to configure the function outside of the
* the automatic process.
*
* RETURNS: TRUE if function is to be included in automatic configuration,
* FALSE otherwise.
*/

static int sysPciAutoConfigInclude
    (
    PCI_SYSTEM *pSys,		/* AutoConfig system information */
    PCI_LOC *pciLoc,		/* PCI address of this function */
    UINT     devVend		/* Device/vendor ID number      */
    )
    {

    BOOL retVal = ERROR;  /* default to excluded */

    switch (devVend)
	{
	/* example code for an excluded device */
	default:
	    retVal = OK;
	    break;
	}

    return retVal;
    }

#endif /* PCI_AUTO_ON  PCI_ON */

