/* board.h - Board File */

/* 
 * Copyright 1984-1999 Wind River Systems, Inc. 
 * Copyright 2002-2003 Intel Corporation
 */

/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/board.h 1     3/05/04 2:54p Ajpasqui $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/board.h $
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 6     10/13/03 8:04a Ajpasqui
 * Removed crypto tests
 * 
 * 5     10/01/03 8:50a Ajpasqui
 * Removed waits for KEYPRESS and added slowport diagnostic(hidden)
 * 
 * 4     8/28/03 3:13p Ajpasqui
 * added factoryqdr test
 * 
 * 3     8/20/03 3:21p Daly
 *
modification history
--------------------
01a,28aug02, jjd  created
01b,23sep02, ajp  added setenet and pcibsrom r/w commands
01c,27sep02, ajp  removed syslog, eprintf
01d,30sep02, ajp  added temp and TCAM tests
01e,11oct02, ajp  added TcAM stubs
01f,21oct02, ajp  added temperature
01g,22nov02, jjd  use 28F128J3A flash
01h,03jan03, jjd  updated CPU2XXX macros
01i,15jan03, jjd  added BOARD_SIM param to switch between sim and real board
01j,29jan03, ajp  added TCAM files
01k,29jan03, ajp  added VERSION string
01l,11feb03, jjd  use PMU timer, fix SIM_ON/SIM_OFF defines
01m,14feb03, ajp  merge
01n,27feb03, ajp  added loop tests
01m,27feb03, ajp  removed comma from list end
01n,03mar03, jjd  fix i2c definitions
01o,12mar03, jjd  allow IS_EGRESS_NPU to be used in .c and .cpp
01p,19mar03, jjd  removed crypto command
01q,27mar03, jjd  fixed for TIMER1
01r,03apr03, ajp  added scroll on/off for diags
01s,16apr03, ajp  added debug memory write
01t,23apr03, ajp  FFS size changed
02a,23may03, jjd  fixed SERIALCLK speed
02b,09jun03, ajp  removed old demo test functions
02c,17jun03, ajp  added factoryw and factoryr commands
03a,20aug03, jjd  updated old modification history, future will be managed
                  by vss log
*/

/*
DESCRIPTION

Board File

INCLUDE FILES:  boot.h
*/

#ifndef __INCboardh
#define __INCboardh

/*  Includes */
#include "boot.h"

#ifndef IXP2800
#error Must define IXP2800
#else
#define IXP2x00
#endif
// VWARE_PARAMS_BEGIN
#define	FLASHSTART		0xC4000000		/* Runtime Flash Address	CORE|UINT32 */
#define	FLASHSIZE		0x01000000		/* Runtime Flash Size	CORE|UINT32 */
#define	BOOTFLASHSTART		(FLASHSTART + 0)		/* Boot Block Flash Address	CORE|UINT32 */
#define	BOOTFLASHSIZE		0x00100000		/* Boot Block Flash size	CORE|UINT32 */
#define	RAM1START		0x00000000		/* RAM Start Address	CORE|UINT32 */
#define	RAM1SIZE		0x04000000		/* RAM Size	CORE|UINT32 */
#define	CLOCKSPEED		66000000		/* External Bus Speed	CORE|FREQUENCY	MHz */
#define	I2C_SCL		7		/* I2C SCL GPIO pin	CORE|CHOICE	0|1|2|3|4|5|6|7 */
#define	I2C_SDA		6		/* I2C SDA GPIO pin	CORE|CHOICE	0|1|2|3|4|5|6|7 */
#define	I2C_ARB		5		/* GPIO to release config inputs	CHOICE	0|1|2|3|4|5|6|7 */
#define	BAUDRATE		9600		/* Baud Rate	UINT32 */
#define	MEM_ERROR_CHECKING		MEM_NONE		/* Memory Error Checking Mode	CORE|CHOICE	MEM_NONE|MEM_PARITY|MEM_ECC */
#define	MAC_ADDR_A		0x1122		/* MAC Address	MAC	1122-3344-5566 */
#define	MAC_ADDR_B		0x3344
#define	MAC_ADDR_C		0x5566
#define	DEFAULTPROMPT1		"Egress"		/* Default Prompt	STRING */
#define	DEFAULTPROMPT2		"Ingress"		/* Default Prompt	STRING */
#define	MMU_TRANSLATION_BASE		0x003FC000		/* MMU Table Address	UINT32 */
#define	LED_BASE		0xc7000100		/* LED Display Base	 */
#define	BOARD_SIM		SIM_OFF		/* Board Simulation Mode	CORE|CHOICE	SIM_OFF|SIM_ON */
// VWARE_PARAMS_END

#if (I2C_SCL == I2C_SDA)
#error Must define I2C_SCL different from I2C_SDA
#endif
#if defined(I2C_ARB) && ( (I2C_ARB == I2C_SCL) || (I2C_ARB == I2C_SDA))
#error Must define I2C_ARB different from I2C_SCL or I2C_SDA
#endif

// VFILE_PARAMS_BEGIN
#define	FFSFLASHSTART		(BOOTFLASHSTART + BOOTFLASHSIZE)		/* Flash File System Start	UINT32 */
#define	FFSFLASHSIZE		0x00700000		/* Flash File System Size	UINT32 */
// VFILE_PARAMS_END

// VFLASH_PARAMS_BEGIN
#define FREEFLASHSTART	(FFSFLASHSTART + FFSFLASHSIZE)		// Free Flash Start	UINT32
#define FREEFLASHSIZE	(FLASHSIZE - (BOOTFLASHSIZE + FFSFLASHSIZE))	// Free Flash Size	UINT32
// VFLASH_PARAMS_END

/* Board define used in drivers */
#define BOOTFLASHOBJECT	FO_28F128J3x1

#define IMAGESTART		(FLASHIMAGE)
#define BOOTFLASHEND	(BOOTFLASHSTART + BOOTFLASHSIZE - 1)
#define FFSFLASHEND		(FFSFLASHSTART + FFSFLASHSIZE - 1)
#define FREEFLASHEND	(FREEFLASHSTART + FREEFLASHSIZE - 1)
#define FLASHEND		(FLASHSTART + FLASHSIZE - 1)
#define RAM1END			(RAM1START + RAM1SIZE -1)

#undef YAVAPAI_CHIPSET

/* Set to 0x01 for Big Endian Mode and to 0x00 for Little Endian Mode */
#define BIG_ENDIAN	0x01

/****************************************************************************
* BoardIOCtrl Prototype
****************************************************************************/
#ifndef __ASSEMBLER__
UINT32 BoardIOCtrl(UINT32 nFuncCode, void *pArgs);
#endif

/****************************************************************************
* BoardIOCtrl Defines
****************************************************************************/
#ifndef __ASSEMBLER__
enum
{
	DELAYTICKS = 0,
	STARTCOUNTERS,
	ENABLECOUNTERS
};
#endif
/****************************************************************************
* BoardIOCtrl Return Codes
****************************************************************************/
#ifndef __ASSEMBLER__
enum
{
	BIOERROR = 0,
	BIOBADFUNCCODE,
	BIOSUCCESS
};
#endif

#define NOTUSED	0

/****************************************************************************
* PIC Devices
****************************************************************************/
/* VWARE_PARAMS_BEGIN */
#define	PICADRS		0xD6000008		/* PIC Address	UINT32 */
#define	PICIRQADRS		0xD6000008		/* PIC Address	UINT32 */
#define	PICIRQ		CPUIRQBASE		/* PIC IRQ	UINT32 */
#define	PICFIRQ		CPUFIRQBASE		/* PIC FIRQ	UINT32 */
#define	PIC_NUM_IRQ		PIC1IRQBASE		/* External PIC Num IRQ	CHOICE	PIC1IRQBASE|PIC2IRQBASE|PIC3IRQBASE|PIC4IRQBASE */
#define	PIC_NUM_FIRQ		PIC2IRQBASE		/* External PIC Num FIRQ	CHOICE	PIC1IRQBASE|PIC2IRQBASE|PIC3IRQBASE|PIC4IRQBASE */
#define	PICFIQADRS		(PICIRQADRS-4)		/* PICFIRQ Address	UINT32 */
/* VWARE_PARAMS_END */

/****************************************************************************
* Serial Devices
****************************************************************************/
// VWARE_PARAMS_BEGIN
#define	SERIALCLK		50000000		/* Serial Clock Speed	UINT32 */
#define	SERIALADR		0xc0030000		/* Serial Address	UINT32 */
#define	SERIALIRQ		(PIC_NUM_IRQ + 2)		/* Serial IRQ	UINT32 */
#define	SERIALSPACING		3		/* Spacing between 16550 regs	UINT32 */
// VWARE_PARAMS_END

/****************************************************************************
* Ethernet Devices
****************************************************************************/
// VWARE_PARAMS_BEGIN
#define	ETHERNETIRQ		(PIC_NUM_IRQ + 15)		/* Ethernet IRQ	UINT32 */
// VWARE_PARAMS_END

/****************************************************************************
* Timer Devices
****************************************************************************/
// VWARE_PARAMS_BEGIN
#define	TIMERCLK		50000000		/* Timer Clock Speed	UINT32 */
#define	TIMERADR		0xC0020000		/* Timer Address	UINT32 */
#define	TIMERIRQ		(PIC_NUM_IRQ + 4)		/* Timer IRQ	UINT32 */
// VWARE_PARAMS_END

/****************************************************************************
* Delay Devices
****************************************************************************/
// VWARE_PARAMS_BEGIN
#define	DELAYCLK		CLOCKSPEED		/* Delay Counter Clock	UINT32 */
// VWARE_PARAMS_END

/****************************************************************************
* Bus Devices
****************************************************************************/

/****************************************************************************
* PCI Configuration Defines
****************************************************************************/
#define HOST_BRIDGE_NUM		0

// VPCI_PARAMS_BEGIN
#define	PCI_TRANS_MAP_0		RAM1START		/* PCI Translation SDRAM Base 0	CORE|UINT32 */
#define	PCI_TRANS_MAP_1		(RAM1START + 0x02000000)		/* PCI Translation SDRAM Base 0	CORE|UINT32 */
#define	PCI_IO_BASE		0xD8000000		/* PCI IO Start Address	CORE|UINT32 */
#define	PCI_IO_SIZE		0x00010000		/* PCI IO Size	CORE|UINT32 */
#define	PCI_MEM_BASE		0xE0000000		/* PCI MEM Start Address	CORE|UINT32 */
#define PCI_MEM_SIZE			0x20000000	// PCI MEM Size				CORE|UINT32
#define	EGRESS_PCI_SRAM_SIZE		0x40000		/* IXP PCI Egress SRAM Size	CORE|UINT32 */
#define	INGRESS_PCI_SRAM_SIZE		0x40000		/* IXP PCI Ingress SRAM Size	CORE|UINT32 */
#define	EGRESS_PCI_DRAM_BASE		0x0		/* IXP PCI Egress DRAM Base	UINT32 */
#define	EGRESS_PCI_DRAM_SIZE		0x4000000		/* IXP PCI Egress DRAM Size	CORE|UINT32 */
#define	INGRESS_PCI_DRAM_SIZE		0x4000000		/* IXP PCI_Egress DRAM Size	CORE|UINT32 */
// VPCI_PARAMS_END

#define HOST_BRIDGE_IDSEL	1
#define PCI_IO_END				(PCI_IO_BASE + PCI_IO_SIZE - 1)
#define PCI_MEM_END				(PCI_MEM_BASE + PCI_MEM_SIZE - 1)

/****************************************************************************
* BRD_PCIMEM_CPU2PCI(addr, bridge)
*     mapping the PCI memory address from CPU view to PCI view
*
* BRD_PCIIO_CPU2PCI(addr, bridge)
*     mapping the PCI IO address from CPU view to PCI view
*
* BRD_PCIIO_PCI2CPU(addr, bridge)
*     mapping the PCI IO address from PCI view to CPU view
*
* BRD_PCIMEM_PCI2CPU(addr, bridge)
*     mapping the PCI memory address from PCI view to CPU view
*****************************************************************************/
//#define BRD_PCIIO_CPU2PCI(addr, bridge)  ((ULONG)((UINT32)(addr) & 0x01FFFFFF) | PCI_IO_BASE)
//#define BRD_PCIIO_PCI2CPU(addr, bridge)  ((ULONG)((UINT32)(addr) & 0x01FFFFFF) | 0x0A000000)
//#define BRD_PCIMEM_CPU2PCI(addr, bridge) ((ULONG)(addr?(((ULONG)(addr) & 0x01FFFFFF) | (PCI_MEM_BASE + 0x02000000)):(0)))
//#define BRD_PCIMEM_PCI2CPU(addr, bridge) (addr?(((ULONG)((ULONG)(LSWAP(addr)) & 0x01FFFFFF) | RAM1START)):(0))
#ifndef __ASSEMBLER__
#define BRD_PCIIO_CPU2PCI(addr, bridge)  ((ULONG)(addr))
#define BRD_PCIIO_PCI2CPU(addr, bridge)  ((ULONG)(addr))
#if 1
#define BRD_PCIMEM_CPU2PCI(addr, bridge) (brd_pcimem_cpu2pci(addr,bridge))
#define BRD_PCIMEM_PCI2CPU(addr, bridge) (brd_pcimem_pci2cpu(LSWAP(addr),bridge))
#else
#define BRD_PCIMEM_PCI2CPU(addr, bridge) (addr ? (LSWAP(addr) + (*(volatile ULONG *)0xde000018 & ~0xf)) : 0)
#define BRD_PCIMEM_CPU2PCI(addr, bridge) (LSWAP(addr - (*(volatile ULONG *)0xde000018 & ~0xf)))
#endif

#define BRD_MAP_PCI_IRQ(pin,dev)  (PIC_NUM_IRQ + 27)

#define DEV2CPU_ADDR(x)		(BRD_PCIMEM_PCI2CPU((x),0))
#define CPU2DEV_ADDR(x)		(LSWAP(BRD_PCIMEM_CPU2PCI((x),0)))
#endif

/****************************************************************************
* PCI Autoconfig Configuration Defines - Mandatory stuff that must be defined in
* order to use vWare's PCI Autoconfig.
****************************************************************************/
#define BOARD_USE_PCI_MEMORY_SPACE
#define PCI_MAX_DEV		8
#define DRAM_BASE			RAM1START
#define DRAM_END			((RAM1START-RAM1SIZE)-1)

// VPCI_PARAMS_BEGIN
#define	PCI_PP_MEM_START		PCI_MEM_BASE		/* PCI Auto Config Memory Start Address	UINT32 */
#define	PCI_PP_MEM_SIZE		PCI_MEM_SIZE-1	/* PCI Auto Config Memory Size	UINT32 */
#define	PCI_PP_IO_START		PCI_IO_BASE	/* PCI Auto Config IO Start Address	UINT32 */
#define	PCI_PP_IO_SIZE		PCI_IO_SIZE-1		/* PCI Auto Config IO Size	UINT32 */
#define PCI_PP_IO16_SIZE	PCI_IO_SIZE-1		// PCI Auto Config16 IO Size	UINT32
// VPCI_PARAMS_END
#ifndef __ASSEMBLER__
#ifdef __cplusplus
extern "C" {
#endif
extern UINT32 isEgressNpu();
#define IS_EGRESS_NPU (isEgressNpu())
#ifdef __cplusplus
}
#endif
#endif

#define PCI_DRAM_START		DRAM_BASE
#define PCI_DRAM_END		DRAM_END

/****************************************************************************
* I2C Slave Addresses
****************************************************************************/
#define	I2CEEPROM	0x51
#define I2CSPD0		0x54
#define I2CSPD1		0x55
#define I2CSPD2		0x56
#define I2CADM1031	0x2d

/****************************************************************************
*	Memory Region Numbering
****************************************************************************/
#ifndef __ASSEMBLER__
enum
{
	REGION_RAM = REGION_USER0,
	REGION_SRAM0 = REGION_USER0,
	REGION_SRAM1 = REGION_USER0,
	REGION_SRAM2 = REGION_USER0,
	REGION_SRAM3 = REGION_USER0,
    REGION_MSF,
    REGION_SCRATCH,
    REGION_GASKET,
	REGION_PCIWINDOW,
	REGION_EPLD
};
#endif
//	REGION_MMR,


/************************************************************
*   Command Group Numbering ENUM
*
*   This enum assigns user diags unique numbers based on 
*   CMDGROUP_BOARD unique number area. All diag must
*   be assigned a system unique ID
*   New diags can add ID here
************************************************************/
#ifndef __ASSEMBLER__
enum
    {
     // DIAGS_BEGIN
	CMD_USER_DIAGS_BEGIN = MAKECMD(CMDGROUP_BOARD, 1),
	CMDUSER_I8255XMDI,
	CMDUSER_I8255XMDIW,
	CMD_LED,
	CMD_SETLED,
	CMD_GPIO,
	CMD_XSCALE,
	CMDUSER_I2CREAD,
	CMDUSER_I2CWRITE,
	CMD_I2C_PATTERN,
	CMD_I2C_WRITE,
	CMD_I2C_PAGE,
	CMD_I2C_ALL,
	CMD_TIMER,
	CMD_UART,
	CMD_MICRO,
	CMD_SF_TEST,
	CMD_MSF,
	CMD_ION,
	CMD_IOFF,
	CMD_DON,
	CMD_DOFF,
	CMD_ETHER,
	CMD_MEDIA_TEST,
	CMD_MACSCAN,
	CMD_PCIDIAG,
    CMD_MEM,
    CMD_GETENET,
	CMD_SETENET,
	CMD_PCIBSROMR,
	CMD_PCIBSROMW,
	CMD_TEMPD,
	CMD_TEMPW,
	CMD_TEMPR,
	CMD_TCAM,
	CMD_SCROLL_OFF,
	CMD_SCROLL_ON,
	CMD_FACTORYR,
	CMD_FACTORYW,
	CMD_FACTORYQDR,
	CMD_FACTORYSP,
	CMDUSER_I2C_TEST
	// DIAGS_END
    };
#endif	

#endif  /* __INCboardh */
