/* boardinit.s - Deer Island low level board initialization */

/* Copyright 1984-1999 Wind River Systems, Inc.
 * 2002-2003 Intel Corporation
 */

/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/boardinit.s 1     3/05/04 2:54p Ajpasqui $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/boardinit.s $
; 
; 1     3/05/04 2:54p Ajpasqui
; 
; 3     8/20/03 4:48p Daly
 *
modification history
--------------------
01a,16may02, jjd  created
01b,08aug02, jjd  bug fixes
01c,15oct02, jjd  debug levelization code
01d,30oct02, jjd  code executes from aliased flash then jumps to unaliased
01e,31oct02, jjd  more bug fixes in rdram init
01f,04nov02, jjd  "
01g,04nov02, jjd  can't write sub-words to scratch, use 32 bit accesses
01h,07nov02, jjd  bug fixes, address table generation code added
01i,08nov02, jjd  bug fixes in outer loop of InitChannel(), fixed bug in
                  address table generation, added lookup tables for each
                  field in CLOCK_CONTROL register
01j,22nov02, jjd  .align directives to account for odd byte lookup tables
01k,07jan03, jjd  resolve CLOCK_CONTROL register settings, wait for SIO !busy
                  before doing SETR/CLRR/SETF/PDNE ops
01l,13jan03, jjd  fixed CONTROL 3 channel
01m,14jan03, jjd  cleanup
01n,15jan03, jjd  fixed bug, RDRAM_CONTROL_X registers not all written in
                  step 4.1
01o,21jan03, jjd  fixed NAPX register setting so sim won't complain. fixed
                  tRP setting in CONFIG_1 register for odd (-35) parts
01q,05feb03, jjd  fixed version working for 1GHz input, workaround for TCDLY1
                  bug applied
01r,07feb03, jjd  added pipeline clearing code, fixed to init ArchnChannelsMax
01s,13feb03, jjd  fixed init for multiple PLL settings and pulse generator at
                  1GHz.  fixed undefined opcodes for 'mul' instructions
01t,20feb03, jjd  fix QDR divisor tables
01u,28feb03, jjd  added more memory error reporting code, align loops for speed
01v,06mar03, jjd  unreset microengines by default
01w,01apr03, jjd  fixed 1066 divisors
01x,02apr03, jjd  write 0 to CLOCK_CONTROL divisor fields before final value
02a,23may03, jjd  fixed TCDLY1 RDRAM workaround, writing TPARM after TCDLY1
                  make slowport clock 25MHz, increase SERIALWAIT as a result.
                  TCDLY1 bug is an error in *all* RDRAM devices, we're just
                  the first ones to discover it.
02b,09jul03, jjd  added 600MHz support for test/debug
02c,04aug03, jjd fixed slowport timing
03a,05aug03, jjd fixed slowport timing in new project.
03b,20aug03, jjd updated old modification history, history will now be
                 managed by vss log
*/

/*
DESCRIPTION

Deer Island low level board initialization

INCLUDE FILES:  cpu.h
                board.h
                ixp2x00.h
                hdlx2416.h
*/
*=== Includes ====================
#include "cpu.h"
#include "board.h"
#include "ixp2x00.h"
#include "hdlx2416.h"
        
*=== External References =========
	.extern TT

*=== Global Declarations =========
	.globl	InitializeBoard

*=== Macros ======================
CPWAIT:	.macro	reg
	.set	noreorder
	mrc	p15, 0, reg, c2, c0, 0	# arbitrary read of CP15
	mov	reg, reg		# wait for it
	sub	pc,pc,#4		# branch to next instruction
	.set	reorder
	.endm

#if 1	/* XXX need this in for real H/W */
DELAY:	.macro	cycles,reg
	.set	noreorder
	ldr	reg,=cycles
.Lm\@:	subs	reg,reg,#1
	bne	.Lm\@
	.set	reorder
	.endm
#else
DELAY:	.macro	cycles,reg
	nop
	.endm
#endif /* BOARD_SIM */

MAKEDEVADDR: .macro destreg,serialdevadr
             ldr destreg,=serialdevadr
*	mov	destreg,serialadr LSL 11
*	orr	destreg,destreg,serialdev LSL 4
	.endm

#define DEBUG_RDRAM


DBGSTR: .macro string
	ldr	r0,=.Lm\@
        b       1f
.Lm\@:  .asciz  string
        .align  2
1:      bl      _rom_outstring_
        .endm

#define ArchnChannelsMax              3
#define ArchnDRAMsMax                 8
#define DeviceType                    1
#define SPEED_1066 1

#define nDrams		r5
#define DepBankDevs	r6
#define Protocol	r7
#define SplitCoreDev	r8
#define Corg		r9
#define CurrentChan	r10

/* size will be determined in step 4.1.0 */
#define RDRAM_CONTROL_Val1Chan   (RDRAM_CONTROL_ReadBkPresDis | \
                                  RDRAM_CONTROL_NumChannels(1) | \
                                  RDRAM_CONTROL_Size8MB)
#define RDRAM_CONTROL_Val2Chan  (RDRAM_CONTROL_ReadBkPresDis | \
                                RDRAM_CONTROL_NumChannels(2) | \
                                RDRAM_CONTROL_Size8MB)

#define RDRAM_CONTROL_Val3Chan (RDRAM_CONTROL_NumChannels(3) | \
                                RDRAM_CONTROL_Size8MB)
#if 0
#define RDRAM_CONTROL_Val3Chan  (RDRAM_CONTROL_ReadBkPresDis | \

                                RDRAM_CONTROL_Size8MB)
#endif
#define RDRAM_RAC_INIT_Step2Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_RacReset | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(4) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) )

#define RDRAM_RAC_INIT_Step5Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_DoStep5 | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(6) | \
                                   RDRAM_RAC_INIT_RacCctlIn(64) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) | \
                                   RDRAM_RAC_INIT_ClockLock | \
                                   RDRAM_RAC_INIT_DoRacInit )

#define RDRAM_RAC_INIT_Step6Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_DoStep5 | \
                                   RDRAM_RAC_INIT_DoStep6 | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(13) | \
                                   RDRAM_RAC_INIT_RacCctlIn(64) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) | \
                                   RDRAM_RAC_INIT_ClockLock | \
                                   RDRAM_RAC_INIT_DoRacInit )

/* SCK == DCLK/SCLK_DIV <= 1MHz*/
#ifdef SPEED_1066		
#define SCLK_DIV 133	/* a nominal DCLK == 66 needs a bigger divisor */
#else
#define SCLK_DIV 100	/* DCLK == 50 */
#endif /* SPEED_1066 */

#define RDRAM_SERIAL_CONFIG_Val  ( RDRAM_SERIAL_CONFIG_SclkFreqDiv(SCLK_DIV) |\
                                   RDRAM_SERIAL_CONFIG_NapXCnt(4) |\
                                   RDRAM_SERIAL_CONFIG_Pdnx(4) |\
                                   RDRAM_SERIAL_CONFIG_Napx(1) |\
                                   RDRAM_SERIAL_CONFIG_RintStart(0) |\
                                   RDRAM_SERIAL_CONFIG_RintWidth(0) )

#define SP_WTC2_CONFIG_Val	0 /* XXX some value needs to be here */       
*=== Section =====================
	.section	".boot2","ax"
        
InitTable:
	.long	0			# table terminator

       
#define SP_CCR_INITVAL  0x0
#define SP_WTC1_INITVAL 0x0
#define SP_RTC1_INITVAL 0x0
#define SP_FAC_INITVAL  0x0
#define SP_FRM_INITVAL  0x0

/*
 * Clock settings explained:
 * Input clock of 100MHz is multiplied by the value in PLL field of
 * STRAP_OPTIONS register (ignoring least significant bit).  This is
 * divided by 2 to get the ME frequency, AKA TCLK.  The PLL frequency 
 * is divided by 4 to get the XScale frequency, AKA PCLK.  The 
 * CLOCK_CONTROL register has several fields that control the clocks to 
 * other parts of the IXP2800.  The APB clock is derived from TCLK.  TCLK
 * is divided by the APB field x4.  (0x2 = divide by 8, 0x3 = divide by 12),etc
 * The APB clock should be <= 50MHz.  The DRAM clock is derived from TCLK/2.
 * TCLK/2 is divided by the DRAM field.  (0x2 = divide TCLK/2 by 2, 0x3 = 
 * divide TCLK/2 by 3, etc).  The DRAM clock for 1066 should be <= 533/Drcgclk)
 * Drcgclk for Deer Island is 8.  The SRAM clocks are derived from TCLK.
 * TCLK is divided by the SRAM field.  (0x2 = divide TCLK by 2, 0x3 = divide
 * TCLK by 3, etc).
 */

APB_Tbl:	# to get 50MHz
	.byte	0x4,0x5,0x5,0x6,0x6,0x7
	.byte	0x7,0x8,0x8,0x9,0x9,0xa
	.byte	0xa,0xb,0xb,0xc,0xc

RDRAM_1066_Tbl: # to get <= 66.625
	.byte	0x6,0x7,0x8,0x9,0x9,0xa		# 800Mhz, 900Mhz, 1GHz, 1.1GHz, 1.2GHz, 1.3GHz
	.byte	0xb,0xc,0xc,0xe,0xe,0xf		# 1.4GHz, 1.5GHz, 1.6GHz, 1.7GHz, 1.8GHz, 1.9GHz
	.byte	0xf,0xf,0xf,0xf,0xf
RDRAM_800_Tbl: # to get <= 50 
	.byte	0x8,0x9,0xa,0xb,0xc,0xd
	.byte	0xe,0xf,0xf,0xf,0xf,0xf
	.byte	0xf,0xf,0xf,0xf,0xf

QDR_250_Tbl: # to get <= 250MHz
	.byte	0x4,0x4,0x5,0x5,0x6,0x6
	.byte	0x7,0x7,0x8,0x8,0x9,0x9
	.byte	0xa,0xa,0xb,0xb,0xc
QDR_200_Tbl: # to get <= 200MHz
	.byte	0x4,0x5,0x5,0x6,0x6,0x7
	.byte	0x7,0x8,0x8,0x9,0x9,0xa
	.byte	0xa,0xb,0xb,0xc,0xc
QDR_233_Tbl: # to get <= 233MHz
	.byte	0x4,0x4,0x5,0x5,0x6,0x6
	.byte	0x7,0x7,0x8,0x8,0x9,0x9
	.byte	0xa,0xa,0xb,0xb,0xc

		.align 2
#include "serial16550.h"


#define SERIALWAIT 10000

/*****************************************************************************
* 
* _rom_outb_ - print out a byte to serial console
*
* Write a byte to the serial console, if a timeout occurs for txrdy, send
* anyways.  This code is only used in this file running from rom.
* This code can use r11-r12 as scratch, they are not used anywhere else.
*
* INPUT: byte to write in r0
*
* RETURNS: N/A
*/
_rom_outb_: 
        stmfd   sp!,{r1-r3}
        sub     sp,sp,#0x10     /* ABIFIX XXX */
        mov     r11,#'\n'
        cmp     r0,r11
        bne     2f
        mov     r0,#'\r'        /* if it's a 'newline' output a CR first */
2:
        mov     r1,#SERIALWAIT  /* load the timeout */
        ldr     r11,=SERIALADR  /* base address of UART */
        add     r11,r11,#LSTAT    /* offset of the line status */
1:      ldrb    r2,[r11]
        mov     r3,#LSR_THRE    /* check to see if ready to TX */
        cmp     r2,r3
        bne     1f              /* if so, go to transmit */
        subs    r1,r1,#1        /* else, decrement timeout */ 
        bne     1b              /* and loop */
1:      ldr     r11,=SERIALADR  
        add     r11,r11,#TXDATA /* offset of transmit register */
        strb    r0,[r11]        
        mov     r2,#'\r'        /* if transmitted a CR, need to tx an LF */
        cmp     r0,r2           /* if its not a CR, we're done */
        bne     1f
        mov     r0,#'\n'        /* if transmitted a CR, now send a LF */
        b       2b
1:
        add     sp,sp,#0x10
        ldmfd   sp!,{r1-r3}     /* restore */
        mov     pc,lr

/*****************************************************************************
* 
* _rom_outstring_ - print a string out to the serial console
*
* INPUT: char *string in r0
*
* RETURNS: N/A
*/
_rom_outstring_: 
        stmfd   sp!,{r1-r3,lr}
        sub     sp,sp,#0x10             /* ABIFIX, XXX */
        mov     r1,r0
1:      ldrb    r0,[r1],#1
        cmp     r0,#0
        beq     2f
        bl      _rom_outb_
        b       1b
2:      add     sp,sp,#0x10
        ldmfd   sp!,{r1-r3,pc}          /* restore and return */
  
/*****************************************************************************
* 
* _rom_outhex_ - print register contents in hex
*
* INPUT: number to print in r0
*
* RETURNS: N/A
*/
_rom_outhex_:
        stmfd sp!,{r1-r3,lr}
	sub	sp,sp,#0x10
        mov     r1,r0
        mov     r3,#32
1:      mov     r0,r1, LSR #28        # get top 4 bits
        add     r0,r0,#'0'
        cmp     r0,#'9'
        ble     2f
        add     r0,r0,#('A' - '0' -10)
2:      bl      _rom_outb_
        mov     r1,r1,LSL #4
        subs    r3,r3,#4
        bne     1b
	add	sp,sp,#0x10
        ldmfd   sp!,{r1-r3,pc}
        
/*****************************************************************************
* 
* _rom_consinit - initialize serial console
*
* INPUT: N/A
*
* RETURNS: N/A
*/
#define CONSOLEBAUD_41 (41666666/BAUDRATE/16)
#define CONSOLEBAUD_50 (50000000/BAUDRATE/16)
_rom_consinit_:
        ldr     r0,=SERIALADR
	add	r1,r0,#IER
	ldr	r2,=UUEN
	strb	r2,[r1]
        add     r1,r0,#FCR
        ldr     r2,=(ENABLE_FIFO|CLEAR_RCV_FIFO|CLEAR_XMIT_FIFO)
        strb    r2,[r1]
        /* set baud rate */
        add     r1,r0,#LCR
        ldr     r2,=DLA_BIT
        strb    r2,[r1]
	ldr	r2,=STRAP_OPTIONS
	ldr	r2,[r2]
	ldr	r1,=STRAP_OPTIONS_CfgPllMask
	and	r2,r2,r1
	cmp	r1,r2
        ldrne   r1,=CONSOLEBAUD_50		/* if we have a valid mulitplier, use 50 */
	ldreq	r1,=CONSOLEBAUD_41		/* if we have an invalid one, we're on the pulsegen */
        mov     r2,#0xff
        and     r2,r1,r2
        add     r3,r0,#DLLO
        strb    r2,[r3]
        mov     r1,r1,LSR #8
        add     r2,r0,#DLHI
        strb    r1,[r2]
        add     r1,r0,#LCR
        ldr     r2,=CHAR_SIZE_8
        strb    r2,[r1]
        mov     pc,lr
 
/*****************************************************************************
* 
* _hdlx2416_init_ - initialize hdlx2416 LED display
*
* initialize the low-level display for debugging info in this module
* this is necessary here because we dont have a stack yet        
*
* INPUT: N/A
*
* RETURNS: N/A
*/
_hdlx2416_init_:
        ldr     r0,=(LED_BASE + HDLX2416_CR)
        ldr     r1,=(HDLX2416_CR_ENABLE | HDLX2416_CR_DON | HDLX2416_CR_B100)
        str     r1,[r0]
        mov     pc,lr

/*****************************************************************************
* _hdlx2416_display_reg_ (r0)
*
* display the 4 chars in r0, bits 31-24 first, then 23-16
* and 15-8,7-0.  this is done by shifting the desired byte
* right down to bits 7-0 of the register.  the shift lefts
* are to push the already displayed bytes off the end of
* register, so when we shift back left, those bits are 
* shifted back as zeros.  this saves having to mask bits off
* with a logical "and" instruction.
*
* INPUT: r0
* uses: r0,r1,r2,r3
*
* RETURNS: N/A
*/
_hdlx2416_display_reg_:
        ldr     r1,=(LED_BASE+HDLX2416_CRAM+12)
        mov     r2,r0,LSR #24   
        str     r2,[r1],#-4
        mov     r2,r0,LSL #8
        mov     r2,r2,LSR #24
        str     r2,[r1],#-4
        mov     r2,r0,LSL #16
        mov     r2,r2,LSR #24
        str     r2,[r1],#-4
        mov     r2,r0,LSL #24
        mov     r2,r2,LSR #24
        str     r2,[r1]
        mov     pc,lr

/**********************************
* RDRAM helper functions
**********************************/

/*****************************************************************************
*        
* SioWrite(devaddr,data) - RDRAM Serial I/O write command
*
* write data on RDRAM SIO bus through IXP2800 SERIAL_CMD registers
* depends on r10 holding the current channel number (0-2)
* 
* INPUT: r0 = devaddr, r1 = data
*        
* uses: r2,r3
*
* RETURNS: N/A
*/
SioWrite:
        cmp     CurrentChan,#1
        ldrlt   r2,=RDRAM_0_SERIAL_CMD
        ldreq   r2,=RDRAM_1_SERIAL_CMD
        ldrgt   r2,=RDRAM_2_SERIAL_CMD
1:      ldr     r3,[r2]
        ands    r3,r3,#RDRAM_SERIAL_CMD_Busy
        bne     1b
	nop
	nop
        str     r1,[r2,#8]
        orr     r0,r0,#SIOWRITEOP
        str     r0,[r2]
        mov     pc,lr

/*****************************************************************************
*
* SioRead(devaddr) - RDRAM Serial I/O read
*
* read data on RDRAM SIO bus through IXP2800 SERIAL_CMD registers
* depends on r10 holding the current channel number (0-2)
*
* INPUT: r0 = devaddr
*
* uses: r1,r2
*
* RETURNS: data in r0
*/
SioRead:
        cmp     CurrentChan,#1
        ldrlt   r1,=RDRAM_0_SERIAL_CMD
        ldreq   r1,=RDRAM_1_SERIAL_CMD
        ldrgt   r1,=RDRAM_2_SERIAL_CMD
1:      ldr     r2,[r1]
        ands    r2,r2,#RDRAM_SERIAL_CMD_Busy
        bne     1b
        orr     r0,r0,#SIOREADOP
        str     r0,[r1]
2:      ldr     r0,[r1]
        ands    r0,r0,#RDRAM_SERIAL_CMD_Busy
        bne     2b
        ldr     r0,[r1,#8]
        mov     r0,r0,LSR #16
        mov     pc,lr
/*****************************************************************************
*
* SioBcast(devaddr,data) - RDRAM Serial I/O broadcast command
*
* broadcast data on RDRAM SIO bus through IXP2800 SERIAL_CMD registers
* depends on r10 holding the current channel number (0-2)
*
* INPUT: r0 = devaddr, r1= data
*
* uses: r2,r3,r13
*
* RETURNS: N/A
*/
SioBcast:
        cmp     CurrentChan,#1
        ldrlt   r2,=RDRAM_0_SERIAL_CMD
        ldreq   r2,=RDRAM_1_SERIAL_CMD
        ldrgt   r2,=RDRAM_2_SERIAL_CMD
1:	ldr	r3,[r2]
        ands    r3,r3,#RDRAM_SERIAL_CMD_Busy
        bne     1b
        str     r1,[r2,#8]
        ldr     r1,=(SIOWRITEOP|SIOSBC)
        orr     r3,r0,r1
        str     r3,[r2]
        mov     pc,lr

InitializeBoard:
/*****************************************************************************
* 
* InitializeBoard() - Do all the low level initialization
*
* The code is still running at 0x0 (Flash aliased to 0x0) it jumps to 0xc4xxxxxx
* later during the RDRAM init code.
*
* configure clocks, this consists of:
* RDRAM (1066 assumed for now)
* SRAM (? all the same speed)
* 50MHz APB (recommended,controls S/P, UART, Timers)
* Slowport Flash access
* The way this works:   there is an input clock to IXP2800 (100MHz typical)
* that gets multiplied up to get the 2x ME frequency.  This multiplier is chosen via
* the PLL strap options.  This internal frequency is selectable from 1.6GHz to 
* 4.8GHz in 100MHz increments. The XScale gets 1/2 ME frequency as does the 
* Command/Push/Pull buses which interconnect everything.  The CLOCK CONTROL 
* register controls the divisors to the other functional units except the PCI I/F
* The SRAMs can have different clocks, but we set them all the same.  The APB uni
* which consists of the GPIO,Slowport,UART, and Timers get a 50MHz clock.  The 
* Slowport has an additional Clock control register which presumably divides down
* the APB clock.  The RDRAM 
* clock is a reference clock to the DRCG to generate the proper speed clock for 
* 1066 RDRAM.
*/
/* enable slowport */
	mov	r0,#0xc0000000
	orr	r0,r0,#0x80000
	orr	r0,r0,#0x1c
	mov	r1,#0x33	/* setup SP_ADC 32/32 */
	str	r1,[r0]
	bic	r0,r0,#4
	mov	r1,#3
	str	r1,[r0]		/* write to SP_PCR, mode 3 */
	bic	r0,r0,#0x10
	mov	r1,#0x4a
	str	r1,[r0]		/* write to SP_WTC2 */
	mov	r1,#0x51
	str	r1,[r0,#8]	/* write to SP_RTC2 */
	mov	r1,#0x1
	bic	r0,r0,#8
	str	r1,[r0]

/* Enable Big Endian, XXX is there a way to check to see if this is needed? */
        ldr     r0, =0x000000F8
        mcr     p15, 0, r0, c1, c0, 0
        CPWAIT  r0

/* bring stuff out of reset */
	ldr	r0,=RESET_0
	mov	r1,#0x00000000
	str	r1,[r0]
	ldr	r0,=RESET_1
	mov	r1,#0
	str	r1,[r0]

        ldr     r0,=STRAP_OPTIONS
        ldr     r1,[r0]
        ldr     r0,=STRAP_OPTIONS_CfgPllMask
        and     r1,r0,r1
	cmp	r0,r1
	beq	3f
/* 600MHz ME frequency not supported, only for test/debug */
	ldr	r0,=0x0c00
	cmp	r0,r1
	bne	2f
	ldr	r2,=0x03000000
	ldr	r1,=CLOCK_CONTROL
	str	r2,[r1]
	DELAY	500, r0
	ldr	r2,=0x03053333
	b	1f
3:
	ldr	r2,=0x03000000
	ldr	r1,=CLOCK_CONTROL
	str	r2,[r1]
	DELAY	500,r0
	ldr	r2,=0x03055555
	b	1f
2:
	mov	r0,r1
	mov	r0,r0,LSR #9
	sub	r0,r0,#8

	ldr	r1,=APB_Tbl
	ldrb	r1,[r1,r0]
	mov	r2,r1,LSL #24
	ldr	r1,=CLOCK_CONTROL
	str	r2,[r1] 
	DELAY	500,r1

#ifdef SPEED_1066
	ldr	r1,=RDRAM_1066_Tbl
#else
	ldr	r1,=RDRAM_800_Tbl
#endif
	ldrb	r1,[r1,r0]
	orr	r2,r2,r1,LSL #16
	ldr	r1,=QDR_200_Tbl
	ldrb	r1,[r1,r0]
	orr	r2,r2,r1,LSL #12
	orr	r2,r2,r1,LSL #8
	orr	r2,r2,r1,LSL #4
	orr	r2,r2,r1
1:
        ldr     r1,=CLOCK_CONTROL
        str     r2,[r1]
	DELAY	500,r0
/* Drain write and Fill buffer */
	mcr	p15,0,r0,c7,c10,4
	CPWAIT	r0
	CPWAIT	r0
	
	DELAY	0x20000,r0

/* for this module, set the stack ptr so we can jump around easier */
        ldr     sp,=SCRATCH_BASE+SCRATCH_SIZE
        sub     sp,sp,#0x10
 
#if (BOARD_SIM == SIM_OFF)        
	bl      _hdlx2416_init_         /* initialize low-level display */
	bl      _rom_consinit_          /* initialize the serial console */
#endif

#define	INITMSG 0x696e6974
	ldr	r0,=INITMSG
	bl	_hdlx2416_display_reg_
/*****************************************************************************
* POST part 1, testing SCRATCH memory because it gets used for serial I/O and
* for temporary storage of variable data during RDRAM initialization.
*****************************************************************************/
#if (BOARD_SIM == SIM_OFF)
        ldr     r0,=SCRATCH_BASE
        mov     r1,#0
        ldr     r2,=SCRATCH_SIZE
        add     r2,r0,r2
        /* first, initialize scratch to 0 */
1:      
        str     r1,[r0],#4
        cmp     r0,r2
        bne     1b

        /* pass 1, from low to high, r[i] = 0, 0xAAAAAAAA -> w[i] */
        ldr     r0,=SCRATCH_BASE
        ldr     r1,=0xAAAAAAAA
        ldr     r2,=SCRATCH_SIZE
        add     r2,r0,r2
1:      ldr     r3,[r0]
        movs    r3,r3           /* this should set the zero bit */
        bne     wantzeros_error
        str     r1,[r0],#4
        cmp     r0,r2
        bne     1b

        /* pass 2, from high to low, r[i] = 0xAAAAAAAA, 0x55555555 -> w[i] */
        ldr     r2,=SCRATCH_BASE
        ldr     r1,=0x55555555
        ldr     r4,=0xAAAAAAAA
        ldr     r0,=SCRATCH_SIZE
        add     r0,r0,r2
1:      sub     r0,r0,#4
        ldr     r3,[r0]
        cmp     r3,r4
        bne     wantAs_error
        str     r1,[r0]
        cmp     r0,r2
        bne     1b

        /* pass 3, from low to high, r[i] = 0x55555555 */
        ldr     r0,=SCRATCH_BASE
        ldr     r1,=0x55555555
        ldr     r2,=SCRATCH_SIZE
        add     r2,r0,r2
1:      ldr     r3,[r0],#4
        cmp     r3,r1
        bne     want5s_error
        cmp     r0,r2
        bne     1b

        b       scratch_passed

        /* in these error subroutines, r0 = address of error, r3 = data read */
wantzeros_error:
#define ERR0 0x45525230 /* "ERR0" */
        /* for now, just display "ERR0" on alpha display */
        ldr     r0,=ERR0        
        bl      _hdlx2416_display_reg_
1:      b       1b      /* loop forever */

wantAs_error:
#define ERR1 0x45525231 /* "ERR1" */
        /* for now, just display "ERR1" on alpha display */
        ldr     r0,=ERR1
        bl      _hdlx2416_display_reg_
1:      b       1b      /* loop forever */

want5s_error:
#define ERR2 0x45525232 /* "ERR2" */
        /* just display "ERR2" on alpha display */
        ldr     r0,=ERR2
        bl      _hdlx2416_display_reg_
1:      b       1b

scratch_passed:
#endif /* BOARD_SIM */
/*****************************************************************************
* End POST part 1, testing Scratch memory
*****************************************************************************/
        
/****************************
*  Turn off MMU Cache
****************************/
#if (BOARD_SIM == SIM_OFF)
	mrc	p15,0,r0,c1,c0,0	/* disable the MMU, the I & D caches 
                                         and the BTB */
	ldr	r1,=0x23FA
	and	r0,r0,r1
	mcr	p15,0,r0,c1,c0,0
	CPWAIT	r0

	mcr	p15,0,r0,c8,c7,0	/* invalidate TLB */
	CPWAIT r0
	mcr	p15,0,r0,c7,c7,0	/* invalidate I & B caches and BTB */
	CPWAIT	r0
#endif
/************************************
* RDRAM RMC/RAC Init goes here
************************************/
/* straight port of John Perez code */
#ifdef DEBUG_RDRAM
        DBGSTR  "Starting\040RDRAM\040init...\n"
#endif
        /* initialize RDRAM_X_CONTROL registers */
        mov     r2,#ArchnChannelsMax
        subs    r2,r2,#2

        /* set RDRAM channel 2 control register */
        ldrgt   r1,=RDRAM_CONTROL_Val3Chan      /* either 3 chan or 1 chan */
        ldrle   r1,=RDRAM_CONTROL_Val1Chan
        ldr     r0,=RDRAM_2_CONTROL
        str     r1,[r0]

        /* set RDRAM channel 1&0 control register */
        ldrgt   r1,=RDRAM_CONTROL_Val3Chan      /* either 3|2|1 */
        ldreq   r1,=RDRAM_CONTROL_Val2Chan
        ldrlt   r1,=RDRAM_CONTROL_Val1Chan
        ldr     r0,=RDRAM_1_CONTROL             /* channel 1&0 always equal */
        str     r1,[r0]
        ldr     r0,=RDRAM_0_CONTROL
        str     r1,[r0]

        /* initialize RDRAM_X_SERIAL_CONFIG registers */
        ldr     r0,=RDRAM_0_SERIAL_CONFIG
        ldr     r1,=RDRAM_SERIAL_CONFIG_Val
        str     r1,[r0]
        mov     r2,#ArchnChannelsMax
        subs    r2,r2,#1
        ldrgt   r0,=RDRAM_1_SERIAL_CONFIG
        strgt   r1,[r0]
        subs    r2,r2,#1
        ldrgt   r0,=RDRAM_2_SERIAL_CONFIG
        strgt   r1,[r0]

        /*
         * since i wrote this as a single channel init function,
         * we'll wrap it with a loop that uses r10 as the channel to 
         * initialize.  ArchnChannelsMax says how many channels to init.
         * in order to choose the correct channel addresses, we'll do
         * a compare of current channel (r10) to 1.  if less than,
         * load channel 0 stuff, == means channel 1, greater than is 2.
         */
        mov     CurrentChan,#0        
InitChannel:    
        /* InitChannel(channel) { */
#ifdef DEBUG_RDRAM
        DBGSTR "Starting\040step\0402.0\040..."
#endif
        /* 
         * Rambus Initialization Guide Step 2.0
         * This section initializes the RAC (Rambus ASIC Controller)
         * which is the Rambus physical protocol controller.
         */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_1    /* choose channel */
        ldreq   r0,=RDRAM_1_CONFIG_1
        ldrgt   r0,=RDRAM_2_CONFIG_1
        ldr     r1,=RDRAM_CONFIG_1_RacStep2Enable
        str     r1,[r0]

        cmp     CurrentChan,#1
        ldrlt   r2,=RDRAM_0_RAC_INIT    /* choose channel */
        ldreq   r2,=RDRAM_1_RAC_INIT
        ldrgt   r2,=RDRAM_2_RAC_INIT
        ldr     r1,=RDRAM_RAC_INIT_Step2Val
        str     r1,[r2]
        /* wait here */
        orr     r1,r1,#0x2
        str     r1,[r2]                 /* start step 2 */

        /* wait for step 2 to be done */
1:      ldr     r1,[r2]
        ands    r1,r1,#RDRAM_RAC_INIT_InitInProgress
        bne     1b

	/* clear RAC_RESET */
	ldr	r1,[r2]
	bic	r1,r1,#RDRAM_RAC_INIT_RacReset
	str	r1,[r2]
        
        mov     r1,#0
        str     r1,[r0]                 /* turn off Rac2StepEnable bit */

#ifdef DEBUG_RDRAM
        DBGSTR  "done\n"
#endif        
        /* delay here */

        /*
         * Rambus Initialization Guide Step 3.0, RDRAM initialization
         */
#ifdef DEBUG_RDRAM
        DBGSTR  "Starting\040Step\0403.0\040..."
#endif        
        /* 
         * Step 3.1/3.2
         * This section resets the RDRAMs on this channel by 
         * asserting SIO reset.  SIO reset is the RESET command
         * delivered via the serial (SIO) interface of the
         * RDRAMs.  This is done via a SIO broadcast.
         */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_SERIAL_CMD  /* choose channel */
        ldreq   r0,=RDRAM_1_SERIAL_CMD
        ldrgt   r0,=RDRAM_2_SERIAL_CMD
        ldr     r1,=( RDRAM_SERIAL_CMD_SerBcast | SIORESETOP )

        str     r1,[r0]
1:      ldr     r1,[r0]
        ands    r1,r1,#RDRAM_SERIAL_CMD_Busy
        bne     1b                      /* wait for !busy */

        /* 
         * Step 3.3.0 
         * The RDRAM TEST77 register must be written with 0
         * before any other operations to RDRAM.  This is done
         * via an SIO broadcast.
         */
        ldr     r0,=0x26800             /* dev 0, TEST77 */
        mov     r1,#0
        bl     SioBcast

        /* 
         * Step 3.4.0 
         * Write the TCYCLE parameter to the RDRAMs.  This is
         * done via an SIO broadcast.  According to the Init Guide, this value
         * is calculated as: 1000000/(Fastclock MHz * 64)  where fastclock
         * is CTM,CFM.  Ex. for a Castine PLL multiplier of 28 (CFG_PLL_MULT = 0x1c)
         * the CTM clock would be ~509 MHz.  Going by the guide, the code should
         * write 30 or 31 (can round either way).  This can be accomplished simply
         * by a LUT based off of CFG_PLL_MULT in STRAP_OPTIONS register.
         */
        ldr     r0,=0x26000
        mov     r1,#0x27                /* recommended by Rambus */
        bl     SioBcast

        /* 
         * Step 3.5.0 
         * Assign individual SDEVID addresses to the RDRAMs.
         * First write an address of 0x3F to the INIT register
         * of the RDRAMs.  This is done via SIO broadcast.
         */

        ldr     r1,=0x405f              /* XXX don't write TEN for 1066, write SRP */
        ldr     r0,=0x10800
        bl     SioBcast

        /* 
         * Step 3.5.7-3.5.13
         * The serial repeater bit in all the RDRAMs was set to 0 by
         * the broadcast above, and all the SDEVIDs set to 0x3F.
         * The code here writes to a single device with SDEVID 0x3F,
         * assigning the loop counter (starting at 0 and incrementing)
         * to SDEVID field of the INIT register.  If the data read
         * back from that SDEVID RDRAM is what was just written, 
         * then the device exists.  Subsequent writes to SDEVID = 0x3F
         * will be serially repeated through the earlier devices whose
         * SDEVID is now assigned uniquely.
         */
        ldr     r4,=ArchnDRAMsMax
        mov     nDrams,#0
1:      ldr     r1,=0x00C0              /* enable serial repeater, PSX */
        orr     r1,r1,nDrams

        ldr     r0,=0x10bf0             /* dev 0x3f, INIT */
        bl     SioWrite

        ldr     r0,=(INIT << 11)        /* INIT */
        orr     r0,r0,nDrams, LSL #4	/* dev */
        bl     SioRead
        ldr     r1,=0x00C0
        orr     r1,r1,nDrams
	and	r0,r0,r1		/* TEN bit is undefined for 1066 */
        cmp     r0,r1
        addeq   nDrams,nDrams,#1        
        bne     1f
        cmp     r5,r4
        bne     1b

        /* r5 now contains nDRAMs */
1:      cmp     nDrams,#0
        beq     1b                      /* oops, if no DRAMs found, loop */

        /* 
         * Step 3.6.0 
         * This code reads the core organization of the RDRAMs, then
         * writes DEVID in all RDRAMs.  Since the IXP2800 RMC doesn't
         * support different organizations on the same channel, it's
         * safe to just assign the DEVIDs sequentially. DEVID is the
         * address that the RDRAMs match for normal read/write accesses.
         */

        ldr     r0,=(CNFGA << 11)
        bl     SioRead
        mov     DepBankDevs,r0,LSR #3
        and     DepBankDevs,DepBankDevs,#1	/* r6 = DepBanksDev */
        mov     Protocol,r0,LSR #10		/* r7 = Protocol version */

        ldr     r0,=(CNFGB << 11)
        bl     SioRead
        mov     SplitCoreDev,r0,LSR #4
        and     SplitCoreDev,SplitCoreDev,#1	/* r8 = SplitCoreDev */
        mov     Corg,r0,LSR #5
        and     Corg,Corg,#0x1f			/* r9 = organization lookup */

        mov     r4,#0
1:      ldr     r0,=(DEVID << 11)
        orr     r0,r0,r4,LSL #4
        mov     r1,r4
        bl     SioWrite
        add     r4,r4,#1
        cmp     r4,nDrams
        bne     1b

        /* 
         * Step 3.7.0 
         * This section writes the PDNX & PDNXA registers of the RDRAMs.  The
         * PDN function is not supported by Castine, but these register need
         * to be written to get their DLLs to lock.  These values should be
         * '1' for SCK <= 1MHz.  The RMC changes SCK during PDNE operations as
         * specified in the NapXCnt field of SERIAL_CONFIG, these settings (2,3)
         * will work for 800-1066 RDRAMs with the NapXCnt field of 4 above.
         * N.B. 64M RDRAMs are limited to SCK of 50MHz due to current size of
         * PDNX field. (3 bits)
         */
        ldr     r0,=(PDNXA << 11)
        mov     r1,#0x2                 /* sim code says 0x2 */
        bl     SioBcast
        ldr     r0,=(PDNX << 11)
        ldr     r1,=0x3                 /* sim code says 0x3 */
        bl     SioBcast

        /* 
         * Step 3.7.6
         * This section writes the SERIAL_CONFIG register of the IXP2800
         * RDRAM controller to match the value of PNDX set above
         */
#if 0	/* check that original SERIAL_CONFIG_Val is ok, it is see above comment */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_SERIAL_CONFIG
        ldreq   r0,=RDRAM_1_SERIAL_CONFIG
        ldrgt   r0,=RDRAM_2_SERIAL_CONFIG
        ldr     r1,[r0]
        and     r1,r1,#RDRAM_SERIAL_CONFIG_PdnxMask
        mov     r2,#1
        orr     r1,r1,r2, LSL #12
        str     r1,[r0]
#endif
        /* 
         * Step 3.8.0
         * This section writes the NAPX RDRAM registers.  IXP2800 doesn't
         * support NAP mode.  Write these params so simulation doesn't complain.
         * for SCK of 31.8MHz, NAPXA field = ceil[tNAPXA,MAX (50ns) / tSCYCLE (31.4)]
         * = 2.  NAPX field = ceil[(NAPXA * tSYCLE + tNAPXB,MAX (40ns))/tSYCLE]
         * = 3.  DQS = 1 for -35 1066 part. so NAPX = 0x462
         */
	ldr	r0,=(NAPX << 11)
	ldr	r1,=0x462			/* sim complains if NAPX not set */
	bl	SioBcast

        /* 
         * Step 3.9.0 
         * This section writes the TPARM RDRAM registers.  This register 
         * contains several fields used by the RDRAM to time the interval
         * between read command and read data.
         */
        mov     r0,#DeviceType
        ldr     r1,=tCACTbl
        ldrb    r1,[r1,r0]
        sub     r1,r1,#8
        ldr     r0,=TPARMTbl
        ldrb    r1,[r0,r1]
        mov     r1,r1,LSL #4
        orr     r1,r1,#0xa			/* tCLS = tCAS = 2 */
        ldr     r0,=(TPARM << 11)
        bl     SioBcast

        /* 
         * Step 3.10.0 
         * This section writes TCDLY1 RDRAM registers.  
         */
        mov     r0,#DeviceType
        ldr     r1,=tCACTbl
        ldrb    r1,[r1,r0]
        sub     r1,r1,#8
        ldr     r0,=TCDLY1Tbl
        ldrb    r1,[r1,r0]
        ldr     r0,=(TCDLY1 << 11)
        bl	SioBcast

        /* 
         * Step 3.11.0 
         * This section writes TFRM RDRAM register with tRCD for devtype.  The
         * TFRM register determines the minimum number of clocks between the
         * ROW-ATTN packet and the first COL packet.
         */
        mov     r0,#DeviceType
        ldr     r1,=tRCDTbl
        ldrb    r1,[r1,r0]
        ldr     r0,=(TFRM << 11)
        bl	SioBcast

        /* 
         * Step 3.12.0 
         * This section writes SETR/CLRR RDRAM registers, performing a general
         * reset on the RDRAMs.
         * The TEST34/TEST78 writes fix a Rambus DLL false start issue and are
         * not covered in the Initialization Guide.  The issue was unkown at the
         * time the Guide was published.  The 2 TEST registers *must* be written
         * in the order below.
         */
        mov     r1,#4
        ldr     r0,=(TEST78 << 11)
        bl	SioBcast                /* fixes a DLL false start issue */

        mov     r1,#0x40
        ldr     r0,=(TEST34 << 11)
        bl	SioBcast                /* this is part of fix */

        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_SERIAL_CMD  /* select channel */
        ldreq   r0,=RDRAM_1_SERIAL_CMD
        ldrgt   r0,=RDRAM_2_SERIAL_CMD
        mov     r4,#0
	mov	r1,#0		
4:	ldr	r2,[r0]
	ands	r1,r2,#RDRAM_SERIAL_CMD_Busy
	bne	4b
1:      orr     r2,r1,r4,LSL #4               /* put in SDEVID */
        orr     r2,r2,#SIOSETROP
        str     r2,[r0]
2:      ldr     r2,[r0]
        ands    r2,r2,#RDRAM_SERIAL_CMD_Busy
        bne     2b
        orr     r2,r1,r4,LSL #4     
        orr     r2,r2,#SIOCLRROP
        str     r2,[r0]
3:      ldr     r2,[r0]
        ands    r2,r2,#RDRAM_SERIAL_CMD_Busy
        bne     3b
        add     r4,r4,#1
        cmp     r4,nDrams
        bne     1b

        /* DLL fix */
        mov     r1,#0
        ldr     r0,=(TEST34 << 11)
        bl	SioBcast

        mov     r1,#0
        ldr     r0,=(TEST78 << 11)
        bl	SioBcast

        /* 
         * Step 3.13.0 
         * This section writes CCA/CCB RDRAM current control registers.
         */
        mov	r1,#0x40                /* XXX set CCA midrange, clear ASYMA */
        ldr	r0,=(CCA << 11)
        bl	SioBcast
        mov	r1,#0x40
	ldr	r0,=(CCB << 11)
        bl	SioBcast

        /* 
         * Step 3.14.0 
         * This section causes an RDRAM powerdown exit.  The RDRAMs are left
         * in the powerdown state after SETR/CLRR, step 3.12.0.
         */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_SERIAL_CMD  /* select channel */
        ldreq   r0,=RDRAM_1_SERIAL_CMD
        ldrgt   r0,=RDRAM_2_SERIAL_CMD
2:	ldr	r1,[r0]
	ands	r1,r1,#RDRAM_SERIAL_CMD_Busy
	bne	2b
        ldr     r1,=(RDRAM_SERIAL_CMD_SerBcast | SIOPDNEOP)
        str     r1,[r0]
1:      ldr     r1,[r0]
        ands    r1,r1,#RDRAM_SERIAL_CMD_Busy
        bne     1b

        /* 
         * Step 3.15.0 
         * This section write SETF RDRAM regiseter, locking the RDRAM to
         * a particular read domain
         */
        ldr     r1,=( RDRAM_SERIAL_CMD_SerBcast | SIOSETFOP )
        str     r1,[r0]
1:      ldr     r1,[r0]
        ands    r1,r1,#RDRAM_SERIAL_CMD_Busy
        bne     1b

#ifdef DEBUG_RDRAM
        DBGSTR  "done\n"
#endif                
        /*
         * Step 4.0
         * RMC configuration 
         */
        /* 
         * Step 4.1.0 
         * This section RMC configuration to set the delay between the COL
         * packet with a read command and the DQ packet with read data
         * (measured at the RDRAM pins)
         */
#ifdef DEBUG_RDRAM
        DBGSTR  "Starting\040Step\0404.0\040..."
#endif
        /*
         * rewrite the channel_size in the CONTROL register(s)
         */
        mov     r2,#ArchnChannelsMax
        subs    r2,r2,#2
        ldrgt   r1,=RDRAM_CONTROL_Val3Chan
        ldreq   r1,=RDRAM_CONTROL_Val2Chan
        ldrlt   r1,=RDRAM_CONTROL_Val1Chan
                
        ldr     r0,=DevSizeTbl
        ldrb    r0,[r0,Corg]
        mul     r2,r0,nDrams
	mov	r0,r2

        ldr     r2,=ChannelSizeTbl
        mov     r0,r0,LSR #4
1:      ldr     r3,[r2],#4
        cmp     r3,r0
        addne   r2,r2,#4
        bne     1b
        ldr     r2,[r2]
        orr     r1,r1,r2, LSL #8

        /*
         * check to see if this is the first pass.  if it is, then write
         * the CONTROL registers depending on ArchnChannels
         * if ( (ArchnChannels - 2) > 0 ) write channel 2
         * if ( (ArchnChannels - 2) >= 0) write channel 1
         * always write channel 0 in any case.
         */
        cmp     CurrentChan,#0
        bne     skipsetsize
        mov     r2,#ArchnChannelsMax
        subs    r2,r2,#2
        ldr	r0,=RDRAM_2_CONTROL
        str	r1,[r0]
        ldr	r0,=RDRAM_1_CONTROL
        str	r1,[r0]
        ldr     r0,=RDRAM_0_CONTROL
        str     r1,[r0]

skipsetsize:    
        /* wait */

        /*
         * Step 4.2
         * This section configures row/col timing for the RDRAM mememory
         * controller channel.  LUTs are indexed by DeviceType.
         */
        ldr     r0,=tRCDTbl
        ldrb    r1,[r0,#DeviceType]	/* tRCD[DeviceType] */
        ldr     r0,=tCACTbl
        ldrb    r3,[r0,#DeviceType]              /* tCAC[DeviceType] << 4 */
        orr     r1,r1,r3,LSL #4
        ldr     r0,=tRASTbl
        ldrb    r3,[r0,#DeviceType]     /* tRAS[DeviceType][4:2] << 8 */
	ands	r0,r3,#0x3		/* check if it's 'odd' (1066 -35) */
        mov     r3,r3,LSR #2
	addne	r3,r3,#1		/* small adjustment for -35 parts */
        and     r3,r3,#0x7
        orr     r1,r1,r3,LSL #8
        ldr     r0,=tRPTbl
        ldrb    r3,[r0,#DeviceType]     /* tRP[DeviceType] << 12 */
        addne   r3,r3,#2                /* for -35 parts, tRAS+tRP must = 32 */
        orr     r1,r1,r3,LSL #12
        ldr     r0,=tOFFPTbl
        ldrb    r3,[r0,#DeviceType]     /* tOFFP[DeviceType] << 16 */
        orr     r1,r1,r3,LSL #16
        ldr     r0,=tRASTbl
        ldrb    r3,[r0,#DeviceType]     /* tRAS[DeviceType][5:2] << 20 */
        mov     r3,r3,LSR #2
        and     r3,r3,#0xF
        orr     r1,r1,r3,LSL #20
        ldr     r0,=tRPTbl
        ldrb    r3,[r0,#DeviceType]     /* tRP[DeviceType][4:2] << 24 */
        mov     r3,r3,LSR #2
        and     r3,r3,#0x7
        orr     r1,r1,r3,LSL #24
	// XXX JJD, BtoB normally 0, except for 3 RIMM systems?
	orr	r1,r1,#RDRAM_CONFIG_1_BtoBRead

        cmp     CurrentChan,#1        
        ldrlt   r0,=RDRAM_0_CONFIG_1    /* select channel */
        ldreq   r0,=RDRAM_1_CONFIG_1
        ldrgt   r0,=RDRAM_2_CONFIG_1
        str     r1,[r0]

        /* wait */

        /* 
         * Step 4.3
         * This section sets the refresh interval by using LUTs.  r9 
         * holds the organization determined in step 3.6.
         */
        sub     r1,nDrams,#1
        mov     r0,#DeviceType
        cmp     r0,#3
        ldrlt   r0,=Refresh1066MHzTbl
        ldrge   r0,=Refresh800MHzTbl
        ldr     r3,=BankTbl
        ldrb    r2,[r3,Corg]
        orr     r1,r1,r2,LSL #16
        ldr     r3,=RowTbl
        ldrb    r4,[r3,Corg]
        add     r2,r2,r4
        sub     r2,r2,#12
        mov     r2,r2,LSL #1
        ldrh    r2,[r0,r2]
        mov     r2,r2,LSL #5
        orr     r1,r1,r2
        mov     r0,#DeviceType
        cmp     r0,#3
        ldrlt   r0,=CCCCnt1066MHzTbl
        ldrge   r0,=CCCCnt800MHzTbl
	sub	r3,nDrams,#1
        ldrb    r2,[r0,r3]

        orr     r1,r1,r2,LSL #19
        ldr     r0,=tRPTbl
        ldrb    r0,[r0,#DeviceType]
        orr     r1,r1,r0,LSL #27
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_2    /* select channel */
        ldreq   r0,=RDRAM_1_CONFIG_2
        ldrgt   r0,=RDRAM_2_CONFIG_2
        str     r1,[r0]
        /* wait */

        /*
         * Step 4.4
         * This section sets the current control interval using LUTs. r9
         * contains the organization determined in step 3.6
         */
        mov     r1,nDrams               	/* nDRAMs */
        orr     r1,r1,DepBankDevs,LSL #6	/* DepBanksDev */
        orr     r1,r1,SplitCoreDev,LSL #7      	/* SplitCoreDev */
        ldr     r3,=ColTbl
        ldrb    r2,[r3,Corg]                
        and     r2,r2,#1
        orr     r1,r1,r2,LSL #8
        ldr     r3,=RowTbl
        ldrb    r2,[r3,Corg]
        and     r2,r2,#0xf
        orr     r1,r1,r2,LSL #9
        ldr     r3,=BankTbl
        ldrb    r2,[r3,Corg]
        and     r2,r2,#0x7
        orr     r1,r1,r2,LSL #13
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_3    /* select channel */
        ldreq   r0,=RDRAM_1_CONFIG_3
        ldrgt   r0,=RDRAM_2_CONFIG_3
        str     r1,[r0]
        /* wait */
#ifdef DEBUG_RDRAM
        DBGSTR   "done\n"
#endif 
        /*
         * Step 5.0
         * This section performs RDRAM current control.
         */
#ifdef DEBUG_RDRAM
        DBGSTR  "Starting\040Step\0405.0\040..."
#endif
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_RAC_INIT    /* select channel */
        ldreq   r0,=RDRAM_1_RAC_INIT
        ldrgt   r0,=RDRAM_2_RAC_INIT
        ldr     r1,=RDRAM_RAC_INIT_Step5Val
        str     r1,[r0]
        /* wait */

1:      ldr     r1,[r0]
        ands    r1,r1,#RDRAM_RAC_INIT_Step5Busy
        bne     1b
        /* wait */

#ifdef DEBUG_RDRAM
        DBGSTR "done\n"
#endif        
        /*
         * Step 6.0
         * This section performs RDRAM core Init and read levelization.
         */
#ifdef DEBUG_RDRAM
        DBGSTR "Starting\040Step\0406.0\040..."
#endif
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_RAC_INIT    /* select channel */
        ldreq   r0,=RDRAM_1_RAC_INIT
        ldrgt   r0,=RDRAM_2_RAC_INIT
        ldr     r1,=RDRAM_RAC_INIT_Step6Val
        str     r1,[r0]
        /* wait */

1:      ldr     r1,[r0]
        ands    r1,r1,#RDRAM_RAC_INIT_Step6Busy
        bne     1b
        /* wait */

        /* 
         * Step 6.2.0 
         * This section performs read equalization.  This is where the 
         * round trip propagation delay is made equal for all RDRAMs on
         * the channel.
         */
        /* up until this point, flash is aliased to 0x0, unalias now */
        cmp     CurrentChan,#0
        bne     1f

        /* jump to 0xc4xxxxxx */
        ldr     pc,=1f
1:              
        ldr     r0,=MISC_CONTROL
        mov     r1,#(MISC_CONTROL_FlashAliasDisable|MISC_CONTROL_FlashWriteEnable)
        ldr     r2,[r0]
        orr     r1,r1,r2
        str     r1,[r0]

#define USE_ADDR_TBL 1
#ifdef USE_ADDR_TBL
        mov     r4,#0
        mov     r2,#ArchnChannelsMax
        subs    r2,r2,#2
	blt	one_channel
	beq	two_channel
	b	three_channel
Mod3Tbl:.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.byte 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2
	.align 2
	/* 3 channel, r4=i, r5=nDrams */
three_channel:
1:	mov	r0,nDrams,LSL #2
	cmp	r4,r0
	beq	table_done
	and	r1,r4,#3
	cmp	r1,#3
	addeq	r4,r4,#1
	beq	1b		/* if (i & 3) == 3, continue */
	ldr	r0,=Mod3Tbl
	ldrb	r0,[r0,r4]
	mov	r0,r0,LSL #5	/* r0 = channel_offset */
	mov	r1,r4,LSL #7	/* r1 = addr */
	cmp	nDrams,#2	/* if (parts_per_chan == (1 || 2) */
	bgt	1f
	mov	r2,r1,LSR #9
	and	r2,r2,#1	/* r2 = devid_ptr */
	b	2f
1:	cmp	nDrams,#4	/* if (parts_per_chan == 4 */
	bne	1f
	mov	r2,r1,LSR #10
	and	r2,r2,#1
	mov	r3,r1,LSR #8
	and	r3,r3,#2
	orr	r2,r2,r3	/* r2 = devid_ptr */
	b	2f
1:	cmp	nDrams,#8	/* if (parts_per_chan == 8) */
	bne	1f
	mov	r2,r1,LSR #11
	and	r2,r2,#1
	mov	r3,r1,LSR #9
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #7
	and	r3,r3,#4
	orr	r2,r2,r3	/* r2 = devid_ptr */
	b	2f
1:	cmp	nDrams,#16	/* if (parts_per_chan == 16) */
	bne	1f
	mov	r2,r1,LSR #12
	and	r2,r2,#1
	mov	r3,r1,LSR #10
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #8
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #6
	and	r3,r3,#8
	orr	r2,r2,r3	/* r2 = devid_ptr */
	b	2f
1:	cmp	nDrams,#32	/* if (parts_per_chan == 32 */
err:	bne	err
	mov	r2,r1,LSR #13
	and	r2,r2,#1
	mov	r3,r1,LSR #11
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #9
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #7
	and	r3,r3,#8
	orr	r2,r2,r3
	mov	r3,r1,LSR #5
	and	r3,r3,#0x10
	orr	r2,r2,r3	/* r2 = devid_ptr */
2:	ldr	r3,=SCRATCH_BASE
	add	r3,r3,#0x100
	add	r0,r0,r2
	mov	r0,r0,LSL #2	/* r0=(chan_offset+devid_ptr)*4 */
	str	r1,[r0,r3]	/* addr_array[chan_offset+devid_ptr] = addr */
	add	r4,r4,#1
	b	three_channel

two_channel:
	and	r0,r4,#1
	mov	r0,r0,LSL #5	/* r0 = chan_offset = (i%2)*32 */
	mov	r1,r4,LSL #7	/* r1 = addr = (i << 7) */
	cmp	nDrams,#2	/* if (parts_per_chan == (1 || 2) */
	bgt	1f
	mov	r2,r1,LSR #8
	and	r2,r2,#1	/* r2 = devid_ptr */
	b	2f
1:	cmp	nDrams,#4	/* if (parts_per_chan == 4) */
	bne	1f
	mov	r2,r1,LSR #9
	and	r2,r2,#1
	mov	r3,r1,LSR #7
	and	r3,r3,#2
	orr	r2,r2,r3	/* r2 */
	b	2f
1:	cmp	nDrams,#8	/* if (parts_per_chan == 8) */
	bne	1f
	mov	r2,r1,LSR #10
	and	r2,r2,#1
	mov	r3,r1,LSR #8
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #6
	and	r3,r3,#4
	orr	r2,r2,r3
	b	2f
1:	cmp	nDrams,#16	/* if (parts_per_chan == 16) */
	bne	1f
	mov	r2,r1,LSR #11
	and	r2,r2,#1
	mov	r3,r1,LSR #9
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #7
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #5
	and	r3,r3,#8
	orr	r2,r2,r3
	b	2f
1:	cmp	nDrams,#32
err1:	bne	err1
	mov	r2,r1,LSR #12
	and	r2,r2,#1
	mov	r3,r1,LSR #10
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #8
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #6
	and	r3,r3,#8
	orr	r2,r2,r3
	mov	r3,r1,LSR #4
	and	r3,r3,#0x10
	orr	r2,r2,r3

2:	ldr	r3,=SCRATCH_BASE
	add	r3,r3,#0x100
	add	r0,r0,r2
	mov	r0,r0,LSL #2	/* r0=(chan_offset+devid_ptr)*4 */
	str	r1,[r0,r3]	/* addr_array[chan_offset+devid_ptr] = addr */
	add	r4,r4,#1
	mov	r0,nDrams,LSL #1
	cmp	r4,r0
	bne	two_channel
	b	table_done

one_channel:
	mov	r1,r4,LSL #7	/*r1 = addr */
	cmp	nDrams,#2
	bgt	1f
	mov	r2,r1,LSR #7
	and	r2,r2,#1
	b	2f
1:	cmp	nDrams,#4
	bne	1f
	mov	r2,r1,LSR #8
	and	r2,r2,#1
	mov	r3,r1,LSR #6
	and	r3,r3,#2
	orr	r2,r2,r3
	b	2f
1:	cmp	nDrams,#8
	bne	1f
	mov	r2,r1,LSR #9
	and	r2,r2,#1
	mov	r3,r1,LSR #7
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #5
	and	r3,r3,#4
	orr	r2,r2,r3
	b	2f
1:	cmp	nDrams,#16
	bne	1f
	mov	r2,r1,LSR #10
	and	r2,r2,#1
	mov	r3,r1,LSR #8
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #6
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #4
	and	r3,r3,#8
	orr	r2,r2,r3
	b	2f
1:	cmp	nDrams,#32
err2:	bne	err2
	mov	r2,r1,LSR #11
	and	r2,r2,#1
	mov	r3,r1,LSR #9
	and	r3,r3,#2
	orr	r2,r2,r3
	mov	r3,r1,LSR #7
	and	r3,r3,#4
	orr	r2,r2,r3
	mov	r3,r1,LSR #5
	and	r3,r3,#8
	orr	r2,r2,r3
	mov	r3,r1,LSR #3
	and	r3,r3,#0x10
	orr	r2,r2,r3

2:	ldr	r3,=SCRATCH_BASE
	add	r3,r3,#0x100
	mov	r2,r2,LSL #2	
	str	r1,[r2,r3]	/* addr_array[devid_ptr] = addr */
	add	r4,r4,#1
	cmp	r4,nDrams
	bne	one_channel

table_done:
#endif
#ifndef USE_ADDR_TBL
        /* set Dev 0 to current DUT's DEVID */
1:      ldr     r0,=(DEVID << 11)
        mov     r1,r4
        bl     SioWrite
        /* set DUT's DEVID to 0 */
        ldr     r0,=(DEVID << 11)
        orr     r0,r0,r4,LSL #4
        mov     r1,#0
        bl     SioWrite
#endif
        /* disable error checking bits */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONTROL     /* select channel */
        ldreq   r0,=RDRAM_1_CONTROL
        ldrgt   r0,=RDRAM_2_CONTROL
        ldr     r1,=(RDRAM_CONTROL_ParityEnable|RDRAM_CONTROL_EccEnable)
        ldr     r2,[r0]
        bic     r2,r2,r1
        str     r2,[r0]

#ifndef USE_ADDR_TBL        
        /* write 0xffffffff (x8) to selected channel */
        mov     r2,#128
        mul     r0,r2,CurrentChan
	mov	r2,r0
        mvn     r0,#0
        mov     r1,r0
        stmia   r2!,{r0-r1}
        stmia   r2!,{r0-r1}
        stmia   r2!,{r0-r1}
        stmia   r2!,{r0-r1}
#else
	mov	r4,#0
1:	mov	r0,CurrentChan,LSL #5	/* r0 = channel*32 */
	add	r0,r0,r4		/* r0 = r0 + dev_to_levelize (r4)*/
	ldr	r1,=SCRATCH_BASE
	add	r1,r1,#0x100
	ldr	r0,[r1,r0,LSL #2]	/* r0 = address to device */
	mvn	r1,#0
	mvn	r2,#0
	stmia	r0!,{r1-r2}
	stmia	r0!,{r1-r2}
	stmia	r0!,{r1-r2}
	stmia	r0!,{r1-r2}
	sub	r0,r0,#32
#endif
        /*
         * here's a tricky part, r0 is address, r1 goes from 0-7
         * r2 is read data, r3 is pattern, r4 = DUT, r6 = device_delay,
         * try to discover proper device_delay
         */
#ifndef USE_ADDR_TBL
        mov     r0,#128
        mul     r1,r0,CurrentChan       /* multiply by channel # */
	mov	r0,r1
#endif
        mov     r6,#0
        mov     r1,#0
2:      ldr     r2,[r0],#4      /* read RDRAM, hoping for 0xFFFFFFFF */
        ldr     r3,=0xFFFF0000  /* if delay is off by 1, only get half data */
        cmp     r2,r3
        addeq   r6,r6,#1        /* if half data, device_delay += 1, done */
        beq     foundit
        mov     r3,#0           /* if delay is off by 2 or more, only read 0s */
        cmp     r2,r3
        addeq   r6,r6,#2        /* if no data, device_delay += 2, try again */
        beq     notfoundyet
	mvn	r3,#0
        cmp     r2,r3           /* error check should be here */
        beq     foundit
notfoundyet:       
        add     r1,r1,#1
        cmp     r1,#8
        bne     2b              /* hope we've found it when r1 = 7 */
foundit:
        /*
         * should probably offset this memory for different channel
         * so we can possibly inspect what each channel wrote
         */
        ldr     r1,=SCRATCH_BASE        /* easy for now */
        str     r6,[r1,r4, LSL #2]
#ifndef USE_ADDR_TBL
        /* set DUT's DEVID back to proper value */
        ldr     r0,=(DEVID << 11)
        orr     r0,r0,r4,LSL #4
        mov     r1,r4
        bl     SioWrite
#endif
        add     r4,r4,#1                
        cmp     r4,nDrams
        bne     1b
#ifndef USE_ADDR_TBL
        /* cleanup, put Dev 0 DEVID back to 0 */
        ldr     r0,=(DEVID << 11)        
        mov     r1,#0
        bl     SioWrite
#endif                
        /* find max delay */
        mov     r6,#0           /* r6 = max_device_delay */
        mov     r3,#0           /* i */
        mov     r8,#0           /* TEMP */
        mov     r4,#4
1:      ldr     r0,=SCRATCH_BASE
        ldr     r1,[r0,r3,LSL #2]
        cmp     r1,r4
        movgt   r0,#1
        orrgt   r8,r8,r0,LSL r3
        cmp     r1,r6
        movge   r6,r1           /* r6 = (r1 >= r6) ? r1 : r6 */
        add     r3,r3,#1
        cmp     r3,nDrams
        bne     1b

        /* write computed val to config_1066_group */
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_1066_CONFIG_GRP     /* select channel */
        ldreq   r0,=RDRAM_1_1066_CONFIG_GRP
        ldrgt   r0,=RDRAM_2_1066_CONFIG_GRP
        str     r8,[r0]

#ifdef SPEED_1066
        cmp     r8,#0
        beq     skipthispart

        /* r3 = i, r8 = TEMP (1066_CONFIG_GRP), SCRATCH_BASE = delay_array */
        mov     r3,#0   
1:      mov     r0,#1
        mov     r2,r0,LSL r3
        ands    r0,r8,r2        
        ldr     r0,=SCRATCH_BASE
        ldr     r0,[r0,r3,LSL #2]	/* get delay_array[i] */
        ldr     r1,=tCACTbl
        ldrb    r1,[r1,#DeviceType]	/* tCAC[Device_type] */
        moveq   r2,#4           	/* if TEMP[i:i] == 0 */
        movne   r2,#8           	/* if TEMP[i:i] != 0 */
        sub     r2,r2,r0        	/* delay_array[i] = tCAC[device_type] + */
        add     r0,r2,r1        	/* (4,8) - delay_array[i] */
        ldr     r1,=SCRATCH_BASE
        str     r0,[r1,r3,LSL #2]
        add     r3,r3,#1
        cmp     r3,nDrams
        bne     1b
	b	storetCAC

skipthispart:
        mov     r3,#0
1:      ldr     r0,=SCRATCH_BASE
        ldr     r1,[r0,r3,LSL #2]
        ldr     r2,=tCACTbl
        ldrb    r2,[r2,#DeviceType]
	add	r2,r2,r6
	sub	r2,r2,r1
	str	r2,[r0,r3,LSL #2]
        add     r3,r3,#1
        cmp     r3,nDrams
        bne     1b

storetCAC:
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_1    /* select channel */
        ldreq   r0,=RDRAM_1_CONFIG_1
        ldrgt   r0,=RDRAM_2_CONFIG_1
        ldr     r1,[r0]
	nop
	nop
        ldr     r0,=tCACTbl
        ldrb    r0,[r0,#DeviceType]
	cmp	r8,#0			/* if TEMP != 0 */
        addne   r0,r0,#4		/* then tCAC[device_type] + 4 */
	addeq	r0,r0,r6		/* else tCAC[device_type] + max_delay */
        ldr     r3,=RDRAM_CONFIG_1_tCACMask
	mvn	r3,r3
        and     r3,r1,r3
        orr     r1,r3,r0,LSL #4
	cmp	CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_1
        ldreq   r0,=RDRAM_1_CONFIG_1
        ldrgt   r0,=RDRAM_2_CONFIG_1
        str     r1,[r0]                 /* store for now */
#endif

        /* delay_array now holds the desired levelized tCAC */
#ifdef SPEED_1066
        cmp     Protocol,#3           /* r7 = protocol_version,from above */
        bne     skipTCPSUpdate
        ldr     r0,=(TCPS << 11)
        mov     r1,#2
        bl     SioBcast

        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_CONFIG_1    /* select channel */
        ldreq   r0,=RDRAM_1_CONFIG_1
        ldrgt   r0,=RDRAM_2_CONFIG_1
        ldr     r1,[r0]
        mvn     r2,#RDRAM_CONFIG_1_tOffpMask
        and     r1,r1,r2       
        orr     r1,r1,#0x50000
        str     r1,[r0]
#endif
skipTCPSUpdate:
        mov     r4,#0

1:      ldr     r0,=SCRATCH_BASE        /* delay_array */

        ldr     r0,[r0,r4,LSL #2]
        sub     r0,r0,#8

		ldr		r1,=LevelizedTCDLY1Tbl
		ldrb	r1,[r1,r0]
		ldr		r6,=LevelizedTPARMTbl
		ldrb	r6,[r6,r0]
		ldr		r0,=(TCDLY1 << 11)
		orr		r0,r0,r4,LSL #4
		bl		SioWrite
		ldr		r0,=(TPARM << 11)
		mov		r1,r6,LSL #4
		orr		r1,r1,#0xA
		orr		r0,r0,r4,LSL #4
		bl		SioWrite

        add     r4,r4,#1
        cmp     r4,nDrams
        bne     1b

        /* 6.3.0 config for Relax (N/A for IXP2800) */

        /* 
         * Step 6.4.0
         * This section performs pipeline clearing.
         */
#if 1					/* not in Specman code, but in RDRAM spec */
	mov	r4,#0
1:	mov	r0,CurrentChan,LSL #5	/* r0 = channel*32 */
	add	r0,r0,r4		/* r0 = r0 + dev_to_levelize (r4)*/
	ldr	r1,=SCRATCH_BASE
	add	r1,r1,#0x100
	ldr	r0,[r1,r0,LSL #2]	/* r0 = address to device */
	ldr	r0,[r0]			/* read device */
	add	r4,r4,#1
	cmp	r4,nDrams
	bne	1b
#endif
        cmp     CurrentChan,#1
        ldrlt   r0,=RDRAM_0_RAC_INIT    /* select channel */
        ldreq   r0,=RDRAM_1_RAC_INIT
        ldrgt   r0,=RDRAM_2_RAC_INIT
        ldr     r1,[r0]
        orr     r1,r1,#RDRAM_RAC_INIT_AllInitDone
        str     r1,[r0]
        /* wait */
	DELAY	50000,r1
        ldr     r1,[r0]
        /* wait */

        /*
         * at this point, channel initialization is COMPLETE
         */
#ifdef DEBUG_RDRAM
        DBGSTR "done\n"
#endif
        add     CurrentChan,CurrentChan,#1
        cmp     CurrentChan,#ArchnChannelsMax
        bne     InitChannel             /* do next channel */

/*************************************
* RDRAM INIT COMPLETE
*************************************/
#undef CurrentChan
#undef Corg
#undef SplitCoreDev
#undef Protocol
#undef DepBankDevs
#undef nDrams                        
/************************************
* RDRAM POST goes here
* test WORKSPACE->WORKSPACE+WORKSPACESIZE
************************************/
        ldr     r0,=RDRAM_0_CONTROL
        ldr     r1,[r0]
        ldr     r2,=(RDRAM_CONTROL_EccEnable|RDRAM_CONTROL_ParityEnable)
        bic     r1,r1,r2                /* clear ECC/Parity Enable bits */

        str     r1,[r0]
        ldr     r0,=RDRAM_1_CONTROL
        ldr     r1,[r0]
        bic     r1,r1,r2
        str     r1,[r0]
        ldr     r0,=RDRAM_2_CONTROL
        ldr     r1,[r0]
        bic     r1,r1,r2
        str     r1,[r0]

	/* turn on the I-cache, BTB */
#if 0
        mrc     p15,0,r0,c1,c0,0
        orr     r0,r0,#0x1800
        mcr     p15,0,r0,c1,c0,0
        CPWAIT  r0
#endif
continue:
#define MSG0 0x6d656d30
        ldr     r0,=MSG0
        bl      _hdlx2416_display_reg_

        ldr     r0,=WORKSPACE
        mov     r1,#0
        ldr     r2,=WORKSPACESIZE

        add     r2,r0,r2
	b	1f
	.literals		/* dump literals */
        /* first, initialize mem to 0 */
	.align	4
1:      
        str     r1,[r0],#4
        cmp     r0,r2
        bne     1b

#define MSG1 0x6d656d31
        ldr     r0,=MSG1
        bl      _hdlx2416_display_reg_

        /* pass 1, from low to high, r[i] = 0, 0xAAAAAAAA -> w[i] */
        ldr     r0,=WORKSPACE
        ldr     r1,=0xAAAAAAAA
        ldr     r2,=WORKSPACESIZE	/* same as XXX above */

        add     r2,r0,r2
	.align	5
1:      ldr     r3,[r0]
        movs    r3,r3			/* this should set the zero bit */
        bne     wantzeros_error_rdram
        str     r1,[r0],#4
        cmp     r0,r2
        bne     1b
	
        /* pass 2, from high to low, r[i] = 0xAAAAAAAA, 0x55555555 -> w[i] */
#define MSG2 0x6d656d32
        ldr     r0,=MSG2
        bl      _hdlx2416_display_reg_

        ldr     r2,=WORKSPACE
        ldr     r1,=0x55555555
        ldr     r4,=0xAAAAAAAA
        ldr     r0,=WORKSPACESIZE /* XXX */

        add     r0,r0,r2
	.align	5
1:      sub     r0,r0,#4
        ldr     r3,[r0]
        cmp     r3,r4
        bne     wantAs_error_rdram
        str     r1,[r0]
        cmp     r0,r2
        bne     1b

        /* pass 3, from low to high, r[i] = 0x55555555 */
#define MSG3 0x6d656d33
        ldr     r0,=MSG3
        bl      _hdlx2416_display_reg_

        ldr     r0,=WORKSPACE
        ldr     r1,=0x55555555
        ldr     r2,=WORKSPACESIZE

        add     r2,r0,r2
	.align	5
1:      ldr     r3,[r0],#4
        cmp     r3,r1
        bne     want5s_error_rdram
        cmp     r0,r2
        bne     1b

        b       rdram_passed

        /* in these error subroutines, r0 = address of error, r3 = data read */
wantzeros_error_rdram:
#define ERR3 0x45525233 /* "ERR3" */
        mov     r4,r0
        ldr     r0,=ERR3        
        bl      _hdlx2416_display_reg_

        DBGSTR  "RDRAM\040Memory\040Error\040@\040"
	mov	r0,r4
	bl	_rom_outhex_
	DBGSTR	"\040want\04000000000\040got\040"
	mov	r0,r3
	bl	_rom_outhex_

1:      b       1b      /* loop forever */


wantAs_error_rdram:
#define ERR4 0x45525234 /* "ERR4" */
        mov     r4,r0
        ldr     r0,=ERR4
        bl      _hdlx2416_display_reg_

        DBGSTR  "RDRAM\040Memory\040Error\040@\040"
	mov	r0,r4
	bl	_rom_outhex_
	DBGSTR	"\040want\040AAAAAAAA\040got\040"
	mov	r0,r3
	bl	_rom_outhex_
1:	b	1b

want5s_error_rdram:
#define ERR5 0x45525235 /* "ERR5" */
	mov	r4,r0
        ldr     r0,=ERR5
        bl      _hdlx2416_display_reg_

        DBGSTR  "RDRAM\040Memory\040Error\040@\040"
	mov	r0,r4
	bl	_rom_outhex_
	DBGSTR	"\040want\04055555555\040got\040"
	mov	r0,r3
	bl	_rom_outhex_
1:      b       1b

rdram_passed:   
	DBGSTR 	"RDRAM\040Memory\040Test\040Passed"
#if 0               
#if ((MEM_ERR_CHECKING == MEM_ECC) || (MEM_ERR_CHECKING == MEM_PARITY))
/************************************
* RDRAM ECC/Parity init, done with dcache enabled 
************************************/
        ldr     r0,=TT
        mcr     p15,0,r0,c2,c0,0	# set up the TTB pointer

        CPWAIT  r0
        CPWAIT  r0
        .long   0                       # XXX ?
        .long   0                       # XXX ?

        mrc     p15,0,r0,c1,c0,0
        orr     r0,r0,#0x5              # enable the MMU,Cache
        mcr     p15,0,r0,c1,c0,0
        CPWAIT  r0

        mcr     p15,0,r0,c7,c10,4       # drain the w/b
        CPWAIT  r0

/* enable ECC generation */
        ldr     r0,=RDRAM_0_CONTROL
        ldr     r1,[r0]
#if (MEM_ERR_CHECKING == MEM_ECC)
        ldr     r2,=RDRAM_CONTROL_EccEnable
#else
        ldr     r2,=RDRAM_CONTROL_ParityEnable
#endif
        orr     r1,r1,r2
        str     r1,[r0]
        ldr     r0,=RDRAM_1_CONTROL
        ldr     r1,[r0]
        orr     r1,r1,r2
        str     r0[r1]
        ldr     r0,=RDRAM_2_CONTROL
        ldr     r1,[r0]
        orr     r1,r1,r2
        str     r1,[r0]
/* disable ECC test writes */
        ldr     r0,=RDRAM_0_ECC_TEST
        mov     r1,#0
        str     r1,[r0]
        ldr     r0,=RDRAM_1_ECC_TEST
        str     r1,[r0]
        ldr     r0,=RDRAM_2_ECC_TEST
        str     r1,[r0]
/* write to all of RDRAM (cached writes) XXX only if write-allocate mode */
        ldr     r0,=RAM1START
        ldr     r9,=RAM1START+RAM1SIZE
        mov     r1,#0
        mov     r2,#0
        mov     r3,#0
        mov     r4,#0
        mov     r5,#0
        mov     r6,#0
        mov     r7,#0
        mov     r8,#0
1:      stm     r0!,{r1-r8}
        cmp     r0,r9
        bne     1b        

/* flush(clean) the cache to get the last set of writes out */
        mov     r0,#1024
        ldr     r1,=RAM1START
1:      mcr     p15,0,r1,c7,c2,5
        add     r1,r1,#32
        subs    r0,r0,#1
        bne     1b
/* invalidate the dcache */
        mcr     p15,0,r0,c7,c6,0
/* flush the w/b */
        mcr     p15,0,r0,c7,c10,4       # drain the w/b
        CPWAIT  r0

/* disable MMU, etc */
	mrc	p15,0,r0,c1,c0,0	/* disable the MMU, the I & D caches, */
                        		/* and the BTB */
        ldr     r0,=0x23FA
	and	r0,r0,r1
	mcr	p15,0,r0,c1,c0,0
	CPWAIT	r0

	mcr	p15,0,r0,c8,c7,0	# invalidate TLB
	CPWAIT r0
	mcr	p15,0,r0,c7,c7,0	# invalidate I & B caches and BTB
	CPWAIT	r0
#endif /* ECC_ENABLED */
#endif
/****************************
*  RAM Init Done
****************************/
/* initialize the stack */
	ldr	sp,=ENDWORKSPACE
	sub	sp,sp,#0x10

	b	_BoardInit

/* Bank/Row/Col/devsize/devaddr boundary tables */
BankTbl:        
        .byte   0x4,0x3,0xF,0xF
        .byte   0x5,0x4,0x2,0x2
        .byte   0x5,0x2,0x4,0x5
        .byte   0x4,0x2
        
RowTbl: 
        .byte   0x9,0x9,0xF,0xF
        .byte   0x9,0xa,0xb,0xc
        .byte   0x9,0xc,0xa,0xa
        .byte   0xb,0xd
        
ColTbl: 
        .byte   0x6,0x6,0xF,0xF
        .byte   0x6,0x6,0x7,0x6
        .byte   0x7,0x7,0x7,0x7 /* XXX some question about these last ones */
        .byte   0x7,0x7        

DevSizeTbl:
        .byte    8, 4,0xff,0xff
        .byte   16,16,  16, 16
        .byte   32, 32, 32, 64
        .byte   64, 64

ChannelSizeTbl: 
        .long   0,0x0
        .long   1,0x1
        .long   2,0x2
        .long   4,0x3
        .long   8,0x4
        .long  16,0x5
        .long  32,0x6
        .long  64,0x7     
/*
 *  speed           1066   | 800
 * speed grade  30  35  32 |40  45
 * dev type      0   1   2   3   4
 *              ------------------
 */
tRCTbl: 
        .byte   28, 32, 28, 28, 28        
tRASTbl:
        .byte   20, 22, 20, 20, 20
tRPTbl:
        .byte    8, 10,  8,  8,  8
tPPTbl:
        .byte    8,  8,  8,  8,  8
tRRTbl:
        .byte    8,  8,  8,  8,  8
tRCDTbl:
        .byte    7,  9,  9,  7,  9
tCACTbl:
        .byte    8,  9,  8,  8,  8
tCWDTbl:
        .byte    6,  6,  6,  6,  6
tCCTbl:
        .byte    4,  4,  4,  4,  4
tPACKETTbl:
        .byte    4,  4,  4,  4,  4
tRTRTbl:
        .byte    8,  8,  8,  8,  8
tOFFPTbl:
        .byte    4,  4,  4,  4,  4
tRDPTbl:
        .byte    5,  5,  4,  4,  4
tRTPTbl:
        .byte    5,  5,  4,  4,  4

TCDLY1Tbl:
        .byte   0, 1, 2, 2, 2

TPARMTbl:
        .byte   3, 3, 3, 4, 5

LevelizedTPARMTbl:
        .byte   3, 4, 4, 4, 5
LevelizedTCDLY1Tbl:
        .byte   0, 0, 1, 2, 2

	.align 1
Refresh800MHzTbl:
        .half   0x030D,0x0186,0x00C3,0x0061,0x0030

Refresh1066MHzTbl:
        .half   0x040F,0x0207,0x0103,0x0081,0x0040


CCCCnt800MHzTbl:
        .byte 0x98,0x4c,0x32,0x26,0x1E,0x19,0x15,0x13
        .byte 0x10,0x0F,0x0D,0x0C,0x0B,0x0A,0x0A,0x09
        .byte 0x08,0x08,0x08,0x07,0x07,0x06,0x06,0x06
        .byte 0x06,0x05,0x05,0x05,0x05,0x05,0x04,0x04

CCCCnt1066MHzTbl:
        .byte 0xCA,0x65,0x43,0x32,0x28,0x21,0x1c,0x19
        .byte 0x16,0x14,0x12,0x10,0x0F,0x0E,0x0D,0x0C
        .byte 0x0B,0x0B,0x0A,0x0A,0x09,0x09,0x08,0x08
        .byte 0x08,0x07,0x07,0x07,0x06,0x06,0x06,0x06


	.align 3
	nop
	nop
	nop
	nop
        .end