/******************************************************************************
*
* Module:		Boot.c
*
* Description:
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:		All functions in this module must be assigned to the .boot2
*				section.  Also, no global or static variables can be used
*				in this module.
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "boot.h"

//  Section Definitions
#pragma section BOOT2 ".boot2" ".boot2" near-absolute RX

/****************************************************************************
* Declare all functions defined in this module as part of the .boot2 section.
****************************************************************************/
#pragma use_section BOOT2 _BoardInit,_memcpy

/****************************************************************************
* Function:	_memcpy(UINT32 *pDest, UINT32 *pSource, unsigned int nBytes)
*
* Purpose:	The memcpy function intrinsic to VisionWARE is in the .text
*			section and so is unavailable for use.  Here we define a simple
*			memcpy derivative which expects pDest and pSource to be 32-bit
*			aligned and nBytes to be a multiple of 4.
*
* Returns:	nothing
****************************************************************************/
void _memcpy(UINT32 *pDest, UINT32 *pSource, unsigned int nBytes)
{
	nBytes >>= 2;
	--pDest;
	--pSource;
	do
	{
		*++pDest = *++pSource;
	} while (--nBytes);
}

/****************************************************************************
* Function:	_BoardInit(void)
*
* Purpose:	copy the vWARE image from flash to RAM and then jump to the
*			image in RAM.
*
* Returns:	does not return
****************************************************************************/
void _BoardInit()
{
	vWAREImageStruct*	pImageStruct;
	UINT32				nRAMStart;
	void (* _WarmStart)(UINT32, UINT32, UINT32, UINT32) =
			(void (*)(UINT32, UINT32, UINT32, UINT32))(STARTWORKSPACE + sizeof(vWAREImageStruct));

	// Initialize pImageStruct to point to the start of the vWARE image.
	pImageStruct = (vWAREImageStruct *)FLASHIMAGE;

#ifndef BUILD_TYPE_XIP
	// Find start of vWARE RAM
	nRAMStart = STARTWORKSPACE + pImageStruct->m_nCodeSize;

	// Copy vWARE to RAM
	_memcpy((void *)STARTWORKSPACE, (void *)FLASHIMAGE, pImageStruct->m_nCopySize);

#else
	// Find start of vWARE RAM
    nRAMStart = STARTWORKSPACE;

    // Copy vWARE Data to RAM
    _memcpy((void *)STARTWORKSPACE, (void *)(FLASHIMAGE + pImageStruct->m_nCodeSize),
			pImageStruct->m_nCopySize - pImageStruct->m_nCodeSize);
#endif

	_WarmStart(nRAMStart, ENDWORKSPACE - nRAMStart, 1, 1);
}
