*****************************************************************************
*
* Module:	Bootasm.s
*
* Description:	Reset vector processing
*
* Project:	HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:	See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*****************************************************************************
*=== Includes ====================
#include "boot.h"
        
/* Macros */
/* Make MMU Translation Table - Note: macro too deep when nested more than 64 */

MakeTransTable: .MACRO addr,addrSize,options,total
    .if total
       .if total <= 0x40
           .long ((addr << addrSize) | options)
           MakeTransTable (addr+1), addrSize, options, (total-1)
       .else
           .if total <= 0x256
               MakeTransTable addr, addrSize, options, 0x40
               MakeTransTable (addr+0x40), addrSize, options, (total-0x40)
           .else
               MakeTransTable addr, addrSize, options, 0x256
               MakeTransTable (addr+0x256), addrSize, options, (total-0x256)
           .endif
       .endif
    .endif
    .ENDM

/* Create First Level Section Descriptor */

#define SectionDesc(addr, options, total) \
	MakeTransTable addr, 20, options, total

/* 
 *Coarse page descriptor contains 14 bits, since it is easy to manipulate
 * the hex values, here the addr is shifted only by 12 bits, instead of 14bits.
 * So make sure that bit 11 and 10 are set to correctly in  options.
 * Bit 11 and 10 in options should contain the 2 LSBs from the coarse page address
 */

#define CoarsePageDesc(addr, options) \
	MakeTransTable addr, 12, options, 1

#define SmallPage(addr, options, total) \
	MakeTransTable addr, 12, options, total
        
*=== Defines =====================
#if 0
#define LEVEL1NC(page)	 .long	(((page) << 20) | 0x1C06)    
#define LEVEL1NCNB(page)	 .long	(((page) << 20) | 0x1C04)
#define LEVEL1(page)     .long	(((page) << 20) | 0x1C0E)
#endif
*=== Global Declarations =========
	.globl	_ColdStart, TT, TTEnd

*=== External References =========
	.extern InitializeBoard
	.extern _BoardInit

*=== Section =====================
	.section	".boot1","ax"

_ColdStart:
	bl	InitializeBoard         # this is address FLASH_START+0 */
* this is where the new dummy vector table is
	movs pc,r14	/*Undefined Instruction 0x4*/		
	movs pc,r14	/*Software Interrupt 0x8*/
	subs pc,r14,#4 /*Prefetch Abort 0xc */
	subs pc,r14,#4 /*Data Abort 0x10 */
	subs pc,r14,#4 /*Reserved vector 0x14 may also be .long */
	subs pc,r14,#4 /*IRQ 0x18 */
	subs pc,r14,#4 /*FIRQ 0x1c */


/* initialize the stack */
	ldr	sp,=ENDWORKSPACE        # this is address FLASH_START+4
	sub	sp,sp,#0x10

	b	_BoardInit

*=== Section =====================
        .section        ".ttb","ax"

TT:                     # Translation Table
SectionDesc (0x000, 0x00000c02, 0x300)	/* 768 : RDRAM */
SectionDesc (0x300, 0x00000000, 0x500) /* 1280 : Reserved */
SectionDesc (0x800, 0x00000c02, 0x004)  /* 4 : SRAM channel 0 */
SectionDesc (0x900, 0x00000c02, 0x004)  /* 4 : SRAM channel 1 */
SectionDesc (0xa00, 0x00000c02, 0x004)  /* 4 : SRAM channel 2 */
SectionDesc (0xb00, 0x00000c02, 0x004)  /* 4 : SRAM channel 3 */
SectionDesc (0x810, 0x00000000, 0x3F0) /* 1008 : Reserved */
SectionDesc (0xc00, 0x00000c02, 0x020)  /* 32 : CAP CSRs */
SectionDesc (0xc20, 0x00000000, 0x020)  /* 32 : Reserved */
SectionDesc (0xc40, 0x00000FF2, 0x010)  /* 16 : Flash */
SectionDesc (0xc50, 0x00000000, 0x010)  /* 16 : Reserved */
SectionDesc (0xc60, 0x00000c02, 0x020)  /* 32 : Slowport */
SectionDesc (0xc80, 0x00000c02, 0x020)  /* 32 : MSF */
SectionDesc (0xca0, 0x00000c02, 0x020)  /* 32 : Scratch (there's only 16K) */
SectionDesc (0xcc0, 0x00000c02, 0x020)  /* 32 : SRAM CSR & Queue */
SectionDesc (0xce0, 0x00000c02, 0x020)  /* 32 : SRAM Ring */
SectionDesc (0xd00, 0x00000c02, 0x020)  /* 32 : DRAM CSR */
SectionDesc (0xd20, 0x00000000, 0x040)  /* 64 : Reserved */
SectionDesc (0xd60, 0x00000c02, 0x020)  /* 32 : Gasket CSR */
SectionDesc (0xd80, 0x00000c02, 0x020)  /* 32 : PCI I/O (only 64K) */
SectionDesc (0xda0, 0x00000c02, 0x020)  /* 32 : PCI Cfg */
SectionDesc (0xdc0, 0x00000c02, 0x020)  /* 32 : PCI Spec/Ack */
SectionDesc (0xde0, 0x00000c02, 0x010)  /* 16 : PCI config hdr */
SectionDesc (0xdf0, 0x00000c02, 0x010)  /* 16 : PCI local CSRs */
SectionDesc (0xe00, 0x00000c02, 0x200)  /* 512: PCI memory space */
TTEnd:  
        .end        

        /* the following isn't going to be used because it's too confusing
         * and if (most) everything is a 1:1 mapping and we only use section
         * descriptors, then it takes the same space as the below
         * implementation.
         */
#if 0        
LEVEL1NCNB(0xC40)		# Pages 0 - 7 are the boot flash @ 0x800000
LEVEL1NCNB(0xC41)
LEVEL1NCNB(0xC42)
LEVEL1NCNB(0xC43)
LEVEL1NCNB(0xC44)
LEVEL1NCNB(0xC45)
LEVEL1NCNB(0xC46)
LEVEL1NCNB(0xC47)
LEVEL1NCNB(0xC48)
LEVEL1(0x0)           # Pages 9 - 72 are the SDRAM @ 0x00000000 ..
LEVEL1(0x1)           # addressed at 0x00800000 in order to have ..
LEVEL1(0x2)           # a shorter translation table
LEVEL1(0x3)
LEVEL1(0x4)
LEVEL1(0x5)
LEVEL1(0x6)
LEVEL1(0x7)
LEVEL1(0x8)
LEVEL1(0x9)
LEVEL1(0xA)
LEVEL1(0xB)
LEVEL1(0xC)
LEVEL1(0xD)
LEVEL1(0xE)
LEVEL1(0xF)
LEVEL1(0x10)
LEVEL1(0x11)
LEVEL1(0x12)
LEVEL1(0x13)
LEVEL1(0x14)
LEVEL1(0x15)
LEVEL1(0x16)
LEVEL1(0x17)
LEVEL1(0x18)
LEVEL1(0x19)
LEVEL1(0x1A)
LEVEL1(0x1B)
LEVEL1(0x1C)
LEVEL1(0x1D)
LEVEL1(0x1E)
LEVEL1(0x1F)
LEVEL1(0x20)
LEVEL1(0x21)
LEVEL1(0x22)
LEVEL1(0x23)
LEVEL1(0x24)
LEVEL1(0x25)
LEVEL1(0x26)
LEVEL1(0x27)
LEVEL1(0x28)
LEVEL1(0x29)
LEVEL1(0x2A)
LEVEL1(0x2B)
LEVEL1(0x2C)
LEVEL1(0x2D)
LEVEL1(0x2E)
LEVEL1(0x2F)
LEVEL1(0x30)
LEVEL1(0x31)
LEVEL1(0x32)
LEVEL1(0x33)
LEVEL1(0x34)
LEVEL1(0x35)
LEVEL1(0x36)
LEVEL1(0x37)
LEVEL1(0x38)
LEVEL1(0x39)
LEVEL1(0x3A)
LEVEL1(0x3B)
LEVEL1(0x3C)
LEVEL1(0x3D)
LEVEL1(0x3E)
LEVEL1(0x3F)
LEVEL1(0x40)
LEVEL1(0x41)
LEVEL1(0x42)
LEVEL1(0x43)
LEVEL1(0x44)
LEVEL1(0x45)
LEVEL1(0x46)
LEVEL1(0x47)
LEVEL1(0x48)
LEVEL1(0x49)
LEVEL1(0x4A)
LEVEL1(0x4B)
LEVEL1(0x4C)
LEVEL1(0x4D)
LEVEL1(0x4E)
LEVEL1(0x4F)
LEVEL1(0x51)
LEVEL1(0x52)
LEVEL1(0x53)
LEVEL1(0x54)
LEVEL1(0x55)
LEVEL1(0x56)
LEVEL1(0x57)
LEVEL1(0x58)
LEVEL1(0x59)
LEVEL1(0x5A)
LEVEL1(0x5B)
LEVEL1(0x5C)
LEVEL1(0x5D)
LEVEL1(0x5E)
LEVEL1(0x5F)
LEVEL1(0x61)
LEVEL1(0x62)
LEVEL1(0x63)
LEVEL1(0x64)
LEVEL1(0x65)
LEVEL1(0x66)
LEVEL1(0x67)
LEVEL1(0x68)
LEVEL1(0x69)
LEVEL1(0x6A)
LEVEL1(0x6B)
LEVEL1(0x6C)
LEVEL1(0x6D)
LEVEL1(0x6E)
LEVEL1(0x6F)
LEVEL1(0x71)
LEVEL1(0x72)
LEVEL1(0x73)
LEVEL1(0x74)
LEVEL1(0x75)
LEVEL1(0x76)
LEVEL1(0x77)
LEVEL1(0x78)
LEVEL1(0x79)
LEVEL1(0x7A)
LEVEL1(0x7B)
LEVEL1(0x7C)
LEVEL1(0x7D)
LEVEL1(0x7E)
LEVEL1(0x7F)
LEVEL1(0x81)
LEVEL1(0x82)
LEVEL1(0x83)
LEVEL1(0x84)
LEVEL1(0x85)
LEVEL1(0x86)
LEVEL1(0x87)
LEVEL1(0x88)
LEVEL1(0x89)
LEVEL1(0x8A)
LEVEL1(0x8B)
LEVEL1(0x8C)
LEVEL1(0x8D)
LEVEL1(0x8E)
LEVEL1(0x8F)
LEVEL1(0x91)
LEVEL1(0x92)
LEVEL1(0x93)
LEVEL1(0x94)
LEVEL1(0x95)
LEVEL1(0x96)
LEVEL1(0x97)
LEVEL1(0x98)
LEVEL1(0x99)
LEVEL1(0x9A)
LEVEL1(0x9B)
LEVEL1(0x9C)
LEVEL1(0x9D)
LEVEL1(0x9E)
LEVEL1(0x9F)
LEVEL1(0xa1)
LEVEL1(0xa2)
LEVEL1(0xa3)
LEVEL1(0xa4)
LEVEL1(0xa5)
LEVEL1(0xa6)
LEVEL1(0xa7)
LEVEL1(0xa8)
LEVEL1(0xa9)
LEVEL1(0xaA)
LEVEL1(0xaB)
LEVEL1(0xaC)
LEVEL1(0xaD)
LEVEL1(0xaE)
LEVEL1(0xaF)
LEVEL1(0xb1)
LEVEL1(0xb2)
LEVEL1(0xb3)
LEVEL1(0xb4)
LEVEL1(0xb5)
LEVEL1(0xb6)
LEVEL1(0xb7)
LEVEL1(0xb8)
LEVEL1(0xb9)
LEVEL1(0xbA)
LEVEL1(0xbB)
LEVEL1(0xbC)
LEVEL1(0xbD)
LEVEL1(0xbE)
LEVEL1(0xbF)
#endif

