/* cfg_utility.h
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  Author(s):    gvaddadi
 *  Date:         3/9/2002
*/

#ifndef I2C_UTILITY_H
#define I2C_UTILITY_H

#ifndef IXP2800
#include "hal_ixdp2400.h"
#endif //IXP2800

/*Default data in the serial EPROM*/

#define MASTER  1
#define SLAVE   0
#define USER    1
#define DEFAULT 0
#ifdef IXP2800
#define PROM_ERROR   0
#define PROM_OK	    1
#else
#define ERROR   0
#define OK	    1
#endif //IXP2800
#define PAGESZ  16

#define CFG_MAN_ID_LEN        12    /*bytes*/
#define CFG_MODEL_ID_LEN      12    /*bytes*/
#define CFG_PARTNUM_LEN       10    /*bytes*/
#define CFG_PARTREV_LEN        2    /*bytes*/
#define CFG_SERIAL_NUM_LEN    10    /*bytes*/
#define CFG_RSVD1_LEN          8    /*bytes*/
#define CFG_RSVD2_LEN          3    /*bytes*/
#define CFG_DATE_LEN           8    /*bytes*/
#define CFG_PASSWD_LEN         8    /*bytes*/
#define CFG_UART_BAUD_LEN      4    /*bytes*/
#define CFG_PROC_SPEED_LEN     4    /*bytes*/
#define CFG_MAC_ADDR_LEN       6    /*bytes*/
#define CFG_CHKSUM_LEN         4    /*bytes*/
#define CFG_STAT_IP_ADDR_LEN   4
#define CFG_RSVD3_LEN          8


#define CHKSUM_ADDR			0x49


#define DEFAULT_DEVID	        0x1     /*kent Island*/
#define DEFAULT_MANID	        "Intel"
#define DEFAULT_MODELID	        "KI"
#define DEFAULT_PROC_SPEED	    600
#define DEFAULT_BAUD	        38400

#define CONFIG_DATA_SIZE	    128
#define DEV_ID_ADDR			    0x1
#define CONFIG_DATA_VALID	    0x12345678

#define MAC_VENDOR_ID		    0x0090D7


/*test status defines*/
#define NOT_TESTED      '0'
#define KI_PASS	        '1'
#define KI_FAIL         '2'
#define SKU1_PASS       '3'
#define SKU1_FAIL       '4'

struct EEPROM_CONTENT {
	unsigned char	prom_fmt; //1
    unsigned char	dev_id;  //1
	unsigned char	man_id[12]; //[CFG_MAN_ID_LEN]; //12
	unsigned char	part_rev [2];//[CFG_PARTREV_LEN]; //2
	
	unsigned char	part_num[10]; //[CFG_PARTNUM_LEN]; //10
	unsigned char	serial_num[10]; //[CFG_SERIAL_NUM_LEN]; //10
	unsigned char	model_id[12]; //[CFG_MODEL_ID_LEN]; //12

	unsigned char	rsvd1[8]; //[CFG_RSVD1_LEN]; //8
	unsigned char	passwd[8]; //[CFG_PASSWD_LEN]; //8
	
	unsigned char	test_status;	//1
	unsigned char	date[8]; //[CFG_DATE_LEN]; //8
	unsigned char	chksum[4]; //[CFG_CHKSUM_LEN]; //4
	unsigned char	rsvd2[3]; //[CFG_RSVD2_LEN]; //3
	
	unsigned int	M_proc_speed; //4
	unsigned int	M_uart_baud; //4
	unsigned char	M_ether_mac_addr[6]; //6
    unsigned char   rsvda[2];

    unsigned char	M_stat_ip_addr[4]; //4
	unsigned int	S_proc_speed ; //4
	unsigned int	S_uart_baud; //4
	unsigned char	S_stat_ip_addr[4]; //4	
    
    unsigned char	S_ether_mac_addr[6]; //6
    unsigned char	rsvd3[10];
	
};

#ifdef IXP2800
#define MAX_QDR_CHANNEL 2
#endif //IXP2800

struct board_config
{
	struct EEPROM_CONTENT config_data;
	int sdram_size;
	int qdr_channel_size[MAX_QDR_CHANNEL];
	int flash_size;
	int config_valid;
};

struct PAGE_DATA{
	unsigned char page1[16];
	int page1_flag;
	unsigned char page2_3[32];
	int page2_3_flag;
	unsigned char page4[16];
	int page4_flag;
	unsigned char page5[16];
	int page5_flag;
	unsigned char page6_7[32];
	int page6_7_flag;
	unsigned char page8[16];
	int page8_flag;
};

#endif
