/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/copy2flash.c 1     3/05/04 2:54p Ajpasqui $
 * copy2flash.c	
 * Copyright 2003 Intel Corp.
 *
 * Used to manage free flash area.  This code enables copying blocks of memory
 * to flash.
 *
 * Date		By  Comment
 * -------	--- -------
 * 4/28/03	JJD Created from 'tf.c'.
 * 4/30/03	JJD Fixed bug to handle large block writes.
 * 5/20/03  JJD Fixed bug in number of sectors to erase.
 *
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/copy2flash.c $
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 2     8/20/03 4:54p Daly
 * added VSS tags
 */
//  Includes
#include "vm.h"
#include "flash.h"
#include "cmd.h"
#include "copy2flash.h"

//  External References
extern const FLASHOBJECT FREEFLASHOBJECT;
extern UINT32 Protected(void *, UINT32);
extern UINT32 ErrorAddress;
extern FLASHOBJECT *IsFlash(UINT32);
extern tRegion *IsRegion(char **);

//Component("Free Flash", COMPONENT_FREEFLASH, FREEFLASHOBJECT);
//Region("Free Flash", REGION_FREEFLASH, FREEFLASHSTART, FREEFLASHEND);

/****************************************************************************
* Function:	Copy2Flash(in_Copy2Flash *pIn, void *pOut);
*
* Purpose:	Copy memory data to flash.
*
* Returns:	void
****************************************************************************/
void Copy2Flash(in_Copy2Flash *pIn, void *pOut)
{
	universal u;
	universal b;
	int i;
	int Length = pIn->Length;
	tRegion *pRegion;
	UINT8 Buffer[64];
	UINT32 n;

	u.uvalue = pIn->FlashStart;
	b.uvalue = pIn->MemStart;

	pRegion = FindRegion(REGION_FREEFLASH);

	if (!((u.uvalue >= pRegion->Start) && ((u.uvalue + Length) <= (pRegion->End + 1))))
	{
		Result(2, "specified address is not part of free flash 0x%08X",u.uvalue);
		return;
	}

	if (Protected(u.pVoid, Length))
	{
		Result(1, "address 0x%08X is protected", ErrorAddress);
		return;
	}

	for (i = 0; i < Length; ++i)
	{
		if (u.pByte[i] != 0xFF)
			break;
	}

	if (i < Length)
	{
		UINT32 EraseStart = u.uvalue & ~(FREEFLASHOBJECT.SectorSize - 1);
		UINT32 Sectors = (Length % FREEFLASHOBJECT.SectorSize) ? 
			((Length / FREEFLASHOBJECT.SectorSize) + 1) : (Length / FREEFLASHOBJECT.SectorSize);

		if (!pIn->OkToErase)
		{
			printf("Erasing %08X to %08X\nPress 'y' to continue, any other key to abort\n",
					EraseStart, EraseStart + (Sectors * FREEFLASHOBJECT.SectorSize));
			if (tolower(getchar()) != 'y')
			{
				Result(3, "aborted by user");
				return;
			}
		}

		while (Sectors--)
		{
			if ((*FREEFLASHOBJECT.EraseVector)((UINT8 *)EraseStart))
			{
				Result(4, "flash erase error at %08X", EraseStart);
				return;
			}
			EraseStart += FREEFLASHOBJECT.SectorSize;
		}
	}

	if ((*FREEFLASHOBJECT.WriteVector)(u.pByte, b.pByte, Length) != Length)
		Result(5, "flash write error");
	else
		Result(0,"");
}

Command(CMD_COPYTOFLASH, "copy2flash", Copy2Flash, 0, "copy memory contents to free flash", "copy memory contents to free flash");
Arg(UINT32ARG, "Memory Start Address", "");
Arg(UINT32ARG, "Flash Start Address", "");
Arg(UINT32ARG, "Length", "");
Switch("y", BOOLEAN32ARG, "Suppress erase prompt", "");
CommandEnd();
