/* diag_utils.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Feb 05, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2800SlowPort.h"

extern int serial_osc;
void dcache_sync(void);
asm void icache_on_asm (void);
asm void icache_off_asm (void);
asm void dcache_on_asm (void);
asm void dcache_off_asm (void);
asm void Enable_CPSR_Int_asm(void);
asm void Disable_CPSR_Int_asm(void);
asm unsigned int get_CP15_asm (void);

// ***** NOTE *****
// From the function flush() (line 197) and below
// the codes are re-used from IXP1200


//
// Function: Enable_CPSR_Int
//
// Description: Enables the FIQ and IRQ in CPSR by clearing bit 6 and 7
//
void Enable_CPSR_Int(void)
{
	printf("Enabling FIQ and IRQ in CPSR\n");
	Enable_CPSR_Int_asm();

}

asm void Enable_CPSR_Int_asm(void)
{
	mrs	r0, cpsr
	bic	r0, r0, #0x40 // enable FIQ
	bic	r0, r0, #0x80 // enable IRQ
	msr	cpsr, r0
}

//
// Function: Disable_CPSR_Int
//
// Description: Disables the FIQ and IRQ in CPSR by clearing bit 6 and 7
//
void Disable_CPSR_Int(void)
{
	printf("Disabling FIQ and IRQ in CPSR\n");
	Disable_CPSR_Int_asm();

}

asm void Disable_CPSR_Int_asm(void)
{
 	mrs	r0, cpsr
    orr	r0, r0, #0x40 // disable FIQ
	orr	r0, r0, #0x80 // disable IRQ
    msr	cpsr, r0
}

//
// Function: get_CP15
//
// Description: Gets the value for Co-Processor 15
//

unsigned int get_CP15(void)
{
	unsigned int x = 0;

	x = get_CP15_asm();

	return x;
}

asm unsigned int get_CP15_asm (void)
{
! "r0"
	mrc p15,0,r0,c1,c0,0
}

//
// Function: icache_on
//
void icache_on(void)
{
	if( (get_CP15() & (ICACHE_BIT)) != 0)
		printf("Instruction cache is already enabled\n");
	else
	{
		printf("Enabling Instruction cache\n");
		icache_on_asm();
	}

}

asm void icache_on_asm (void)
{
	// Enable Icache by writing a 1 to bit 12 of the CP15 - control register
	mrc  p15,0,r1,c1,c0,0
	orr  r1,r1,#0x1000 // enable ICache
	mcr  p15,0,r1,c1,c0,0
	mcr   p15,0,r1,c7,c5,0  // invalidate instruction cache
}

//
// Function: icache_off
//
void icache_off(void)
{
	printf("Disabling Instruction cache\n");

	icache_off_asm();
}

asm void icache_off_asm(void)
{
	mrc    p15,0,r1,c1,c0,0
	bic    r1,r1,#0x1000 		// disable Icache
	mcr    p15,0,r1,c1,c0,0
	mcr    p15,0,r1,c7,c5,0  	// invalidate instruction cache
}

//
// Function: dcache_on
//
void dcache_on(void)
{
	if( (get_CP15() & DCACHE_BIT) != 0)
		printf("Data cache is already enabled\n");
	else
	{
		printf("Enabling Data cache\n");
		dcache_on_asm();
	}

}

asm void dcache_on_asm (void)
{
	// Enable data cahce and MMU by writing 1 to bits 0 and 2
	mrc  p15,0,r1,c7,c10,4   // drain write buffer
	// cpuwait
    mrc    p15,0,r1,c2,c0,0  // arbitrary read
    mov    r1,r1
    sub    pc,pc,#4
    nop
	mrc  p15,0,r1,c1,c0,0

	// enable DCache (also ensures the MMU and alignment faults are   
	// enabled)                       
	orr  r1,r1,#0x0007   
	mcr  p15,0,r1,c1,c0,0
	mrc  p15,0,r1,c2,c0,0  	// arbitrary read
	mov  r1,r1
	sub  pc,pc,#4
	mcr  p15,0,r1,c7,c6,0 	// invalidate data cache

	// cpuwait
	mrc    p15,0,r1,c2,c0,0	 // arbitrary read
	mov    r1,r1
	sub    pc,pc,#4
	nop
}

//
// Function: dcache_off
//
void dcache_off (void)
{
	printf("Disabling Data cache\n");

	dcache_sync();

	dcache_off_asm();
}

asm void dcache_off_asm()
{
	mrc  p15,0,r1,c1,c0,0
	bic  r1,r1,#0x0007  // disable DCache (also ensures the
                        // MMU and alignment faults are
                        // disabled)
	mcr  p15,0,r1,c1,c0,0
	// cpuwait
	mrc    p15,0,r1,c2,c0,0  // arbitrary read
	mov    r1,r1
	sub    pc,pc,#4
	mcr    p15,0,r1,c7,c6,0 // invalidate data cache
	// cpuwait
	mrc    p15,0,r1,c2,c0,0  // arbitrary read
	mov    r1,r1
	sub    pc,pc,#4
	mcr    p15,0,r0,c7,c10,4 // and drain the write buffer
	// cpuwait
	mrc    p15,0,r1,c2,c0,0  // arbitrary read
	mov    r1,r1
	sub    pc,pc,#4
	nop
}


asm void dcache_sync ()
{
%lab 667
! "r0", "r1"
    /* The best way to evict a dirty line is by using the          */
    /* line allocate operation on non-existent memory.             */
	mov    r0, #0x70000000    /* use upper 256M of SDRAM for cache flush region */
	add    r1, r0, #0x8800   /* 32KB cache + 2KB mini cache */
	667: 
	mcr    p15,0,r0,c7,c2,5  /* allocate a line    */
	add    r0, r0, #32       /* 32 bytes/line      */
	teq    r1, r0
	bne    667
	mcr    p15,0,r0,c7,c6,0  /* invalidate data cache */
	/* cpuwait */
	mrc    p15,0,r1,c2,c0,0  /* arbitrary read   */
	mov    r1,r1
	sub    pc,pc,#4
	mcr    p15,0,r0,c7,c10,4 /* and drain the write buffer */
	/* cpuwait */
	mrc    p15,0,r1,c2,c0,0  /* arbitrary read   */
	mov    r1,r1
	sub    pc,pc,#4
	nop

}


//
// Function: dcache_flush
//
asm void dcache_flush ()
{
	mcr	   p15,0,r0,c7,c10,4	// drain the write buffer
	mcr	   p15,0,r1,c7,c6,0	// drain the data cache
	mrc    p15,0,r1,c2,c0,0  /* arbitrary read   */
	mov    r1,r1
	sub    pc,pc,#4
	nop
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_ION,"ion",icache_on,DIAGNOSTIC,"ICacheOn","enable instruction cache");
CommandEnd();

Command(CMD_IOFF,"ioff",icache_off,DIAGNOSTIC,"ICacheOff","disable instruction cache");
CommandEnd();

Command(CMD_DON,"don",dcache_on,DIAGNOSTIC,"DCacheOn","enable data cache");
CommandEnd();

Command(CMD_DOFF,"doff",dcache_off,DIAGNOSTIC,"DCacheOff","disable data cache");
CommandEnd();

