/******************************************************************************
*
* Module: esd.cpp
*
* Description: Serial Module Definitions
*
* Project:     HSI VisionWare -- vWare
*
* Functions:   This module calls generic serial function down to the driver
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
#include "vm.h"
#include "esd.h"
#include "cmd.h"
#include "cmds.h"
#include "vconsole.h"

extern tESDRIVER Esdriver;

extern "C" void ProcessCharacters(UINT8 *pBuffer, UINT32 length);
extern UINT32 ProcessMode;
ESD Esd;
ESD *pEsd;

/* serial callback to use within an application */
extern "C" void (*esdCallback) (UINT8 *, UINT32) = NULL;

void ESD::SetBaudRate(UINT32 Speed)
{
	Esdriver.Ioctl(hDevice, IOCTL_SETBAUD, &Speed);
}

ESD::ESD(void)
{
	ESD::Open(&hDevice);
		
	pEsd = this;
}

DSTATUS ESD::Open(HANDLE *pHandle, void *pArgs)
{
	if (!hDevice)
		hDevice = *pHandle = Esdriver.Open(&Esdriver);

#if (COMMANDCHANNELOUT == DEFAULTSERIAL)
	ConsoleInit(this);
#endif

	return STATUS_OK;	
}

/*  
 *  The Close() method does NOT use the dev parameter.  The dev parameter
 *  is a relic from previous (non-C++) versions of this driver.  It
 *  is included here to match the Read/Write/Ioctl functions.
 */
DSTATUS ESD::Close(HANDLE dev)
{
	if (hDevice)
		Esdriver.Close(hDevice);
	hDevice = 0;	
	
	/* If we are using the serial port console, close it too  */
	if (&Esd == pCommandChannel)
		pCommandChannel = NULL;
		
	return STATUS_OK;
}

DSTATUS ESD::Write(HANDLE dev, char *buffer, unsigned int nBytes)
{
	return Esdriver.Write(hDevice, (UINT8 *)buffer, nBytes);
}

DSTATUS ESD::Ioctl(HANDLE dev, unsigned int funcCode, void *pArgs)
{
	return Esdriver.Ioctl(hDevice, funcCode, pArgs);
}

extern "C" void SetBaudRate(UINT32 Speed)
{
	Esd.SetBaudRate(Speed);
}

extern "C" void EsdriverProcessCharacters(UINT8 *pBuffer, UINT32 chars)
{
	if (esdCallback != NULL)	/* use the application call back if set */
		esdCallback (pBuffer, chars);
	else
    	::ProcessCharacters(pBuffer, chars);
}

extern "C" void Serial(void *pIn, void *pOut)
{
	pCommandChannel = &Esd;
}

Command(CMD_SERIAL, "serial", Serial, WHILERUNNING, "Serial", "switch to the serial channel");
CommandEnd();

Component("Serial", COMPONENT_SERIAL, Esd);

/***************************************************************************
 * Function: void SerialLaunch(tOsParams *pOS)
 * 
 * Purpose:	 Called prior to launching an OS.  Completes the Ethernet and
 * 	         TCP/IP fields of the tOsParams structure that gets passed to
 *           the operating system.  Also calls the driver's Close function.
 *
 *			 The purpose of the 8260 stub is for BSD OS. Problems were 
 *			 encountered when opening the 8260 serial drivers after a close
 *			 had been issued.
 * 
 * 
 * Returns:  nothing
 ***************************************************************************/	  
#ifndef TGT_SBC8260
extern "C" void SerialLaunch(tOsParams *pOS)
{
	Esd.Close(Esd.hDevice);
}
#else
extern "C" void SerialLaunch(tOsParams *pOS)
{
}
#endif

Fini(0, SerialLaunch);
