/* ether_net.h
//=============================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//=============================================================================
*/
/* Portions of this file modified by Intel Corporation. 
Portions created by Intel Corporation are:
Copyright  2002-2004 Intel Corporation  */

#ifndef _ETHERNET_H
#define _ETHERNET_H

#ifndef NULL
#define NULL 0
#endif

#define SWAP16(x)         \
  (((x) & 0xFF) << 8) |   \
  (((x) >> 8))

#define RX	1
#define TX	2

#define	PCI_MEM_PAGE_MASK			0xE0000000
#define	PCI_IO_PAGE_MASK			0xFFFF0000

#ifndef IXP2800
void Ethernet_test(void);
#endif //IXP2800

void Get_Eeprom_Mac_Addr(void);
int get_ether_addr(void);
int i559IntHandler(int arg);
void unlockEeprom(UINT32, int, int);
void eraseEeprom(UINT32, int, int);
void writeEeprom (UINT32 , int, UINT16, int);
void lockEeprom (UINT32 , int, int);


#define VENDOR_INTEL    0x8086
#define INTEL_NAME		"Intel Corporation Inc."
#define I82559ER		0x1209
#define I82559ER_NAME	"82559ER 10/100 LAN Controller"

#define ETHERMTU	1500

#define MAX_DELAY	6000000

#define SPEED_NOLINK	0
#define SPEED_10M		10
#define SPEED_100M		100

#define ETHERNET_DATA   0xDEADBEEF

#define BUSY_WAIT_LIMIT		0xf000			/* the upper limit on a busy wait
											   for command completion, etc. */

#define PCI_DRAM_BAR_REG ((volatile UINT32 *) (IXP2000_PCI_BASE + IXP2000_PCI_CFG_BASE_ADDRESS_2))

// Bus masters can get to all of SDRAM using direct mapping.
#define VIRT_TO_BUS(_x_) ((UINT32)((UINT32)(_x_) | (*(PCI_DRAM_BAR_REG) & ~0xF)))
#define BUS_TO_VIRT(_x_) ((UINT32)((UINT32)(_x_) & ~(*(PCI_DRAM_BAR_REG) & ~0xF)))

//  EEPROM Control bits.
#define EEPROM_SHIFT_CLK	0x01            // EEPROM shift clock
#define EEPROM_CS			0x02            // EEPROM chip select
#define EEPROM_DATA_WRITE	0x04            // EEPROM chip data in
#define EEPROM_DATA_READ	0x08            // EEPROM chip data out
#define EEPROM_ENB			(0x4800 | EEPROM_CS)

#define EE_CMD_WRITE	0x05		/* WRITE opcode, 101 */
#define EE_CMD_READ	    0x06		/* READ  opcode, 110 */
#define EE_CMD_ERASE	0x07		/* ERASE opcode, 111 */
#define EE_CMD_EWEN	    0x13		/* ERASE/WRITE enable opcode, 10011 */
#define EE_CMD_EWDS 	0x10		/* ERASE/WRITE disable opcode, 10110 */

/* EEPROM misc. defines */

#define EE_CMD_BITS	    3		/* number of opcode bits */
#define EE_ADDR_BITS	8		/* number of address bits for a 256 reg eeprom */
#define EE_DATA_BITS	16		/* number of data bits */
#define EE_SIZE		0x100		/* 0x100 words */
#define EE_CHECKSUM	0xbaba		/* checksum */

#ifndef __ARMEB__
#define SCBStatus       0x02	// Rx/Command Unit command and status.
#define SCBCmd          0x00	// Rx/Command Unit command and status.
#define SCBPointer      0x04	// General purpose pointer.
#define SCBPort         0x08	// Misc. commands and operands.
#define SCBflash        0x0E	// Flash memory control.
#define SCBeeprom       0x0C	// EEPROM memory control.
#define SCBCtrlMDI      0x10	// MDI interface control.
#define SCBEarlyRx      0x14	// Early receive byte count.
#else /*__ARMEB__ */
#define SCBStatus       0x00
#define SCBCmd          0x02
#define SCBPointer      0x04
#define SCBPort         0x08
#define SCBflash        0x0C
#define SCBeeprom       0x0E
#define SCBCtrlMDI      0x10
#define SCBEarlyRx      0x14
#endif


/* PCI Runtime Register offsets */
#define SCB_OFFSET			0
#define SCB_STAT_REG(n)		((UINT16 *)(n + 0x00))
#define SCB_CMD_REG(n) 		((UINT16 *)(n + 0x02))
#define SCB_GENPTR_REG(n)	((UINT32 *)(n + 0x04))
#define PORT_REG(n) 		((UINT32 *)(n + 0x08))
#define FLASH_CTL_REG(n)	((UINT16 *)(n + 0x0c))
#define EEPROM_CTL_REG(n)	((UINT16 *)(n + 0x0e))
#define MDI_CTL_REG(n) 		((UINT32 *)(n + 0x10))
#define RXBC_REG(n) 		((UINT32 *)(n + 0x14))

/* PORT* commands (lower 4 bits) */
#define PORT_RESET		((UINT32) 0x0)
#define PORT_SELF_TEST	((UINT32) 0x1)
#define PORT_DUMP		((UINT32) 0x3)

/* Command codes for the command fields of command descriptor blocks */
#define NOP			0
#define IA_SETUP	1
#define CONFIGURE	2
#define MC_SETUP	3
#define TRANSMIT	4
#define TDR			5
#define DUMP		6
#define DIAGNOSE	7

/* Commands for CUC in command word of SCB */
#define CU_NOP				0
#define CU_START			1
#define CU_RESUME			2
#define LOAD_DUMPCTR_ADDR	4	/* Load Dump Counters Address */
#define DUMP_STAT_COUNTERS	5	/* Dump Statistical Counters  */
#define LOAD_CU_BASE		6	/* Load CU Base Register      */
#define DUMP_RESET_COUNTERS	7   /* Dump and Reset Statistical Counters */

#ifdef __ARMEB__	// 100% correct
	#define SCB_S_CUMASK	0xc000		/* state mask */
#else
	#define SCB_S_CUMASK	0x00c0		/* state mask */
#endif
#define SCB_S_CUIDLE	(0x00 << 6)	/* CU is idle */
#define SCB_S_CUSUSP	(0x01 << 6)	/* CU is suspended */
#define SCB_S_CUACTIVE	(0x02 << 6)	/* CU is active */
#define SCB_S_CURSV1	(0x03 << 6)	/* reserved */

/* Commands for RUC in command word of SCB */
#define RU_NOP			0
#define RU_START		1
#define RU_RESUME		2
#define RU_ABORT		4
#define LOAD_HDS		5		/* Load Header Data Size      */
#define LOAD_RU_BASE	6		/* Load RU Base Register      */
#define RBD_RESUME		7		/* Resume frame reception     */

/* Misc. defines */
#define END_OF_LIST	1
#define BUSY		1

/* RU Status field */
#define RU_IDLE			0x0
#define RU_SUSPENDED	0x1
#define RU_NORESOURCE	0x2
#define RU_READY		0x4
#define RU_SUSP_NORBD	0x5
#define RU_NORSRC_NORBD	0x6
#define RU_READY_NORBD	0xc

/* Mask for interrupt status bits in SCB - six possible sources */
#ifdef __ARMEB__		// 100% correct
	#define I559_INT		0x00fc
#else
	#define I559_INT		0xfc00
#endif

/* MDI definitions */
#define MDI_WRITE_OP	0x01
#define MDI_READ_OP		0x02
#define MDI_NOT_READY	0
#define MDI_POLLED		0
#define MDI_DEFAULT_PHY_ADDR 1	/* when only one PHY */

/* PHY device register addresses */

/* generic register addresses */
#define MDI_PHY_CTRL		0
#define MDI_PHY_STAT		1
#define MDI_PHY_ID_1		2
#define MDI_PHY_ID_2		3
#define MDI_PHY_AUTO_AD		4
#define MDI_PHY_AUTO_LNK	5
#define MDI_PHY_AUTO_EXP	6

#ifdef IXP2800
#define I82559_PHY_ID		0x02a80154
#endif //IXP2800
#define I82555_PHY_ID		0x02a80150
#define ICS1890_PHY_ID		0x0015f420
#define DP83840_PHY_ID		0x20005c00
#define I82553_PHY_ID		0x02a80350
#define I82553_REVAB_PHY_ID	0x03e00000

/* I82555/558 Status and Control register */
#define I82555_STATCTRL_REG	0x10
#define I82555_100_MBPS		(1 << 1)
#define I82555_10_MBPS		(0 << 1)

#define REVISION_MASK		0xf

/* MDI Control Register bits */
#define MDI_CTRL_COLL_TEST		(1 << 7)
#define MDI_CTRL_FULL_DUPLEX	(1 << 8)
#define MDI_CTRL_RESTART_AUTO	(1 << 9)
#define MDI_CTRL_ISOLATE		(1 << 10)
#define MDI_CTRL_POWER_DOWN		(1 << 11)
#define MDI_CTRL_AUTO_ENAB		(1 << 12)
#define MDI_CTRL_AUTO_DISAB		(0 << 12)
#define MDI_CTRL_100_MBPS		(1 << 13)
#define MDI_CTRL_10_MBPS		(0 << 13)
#define MDI_CTRL_LOOPBACK		(1 << 14)
#define MDI_CTRL_RESET			(1 << 15)

/* MDI Status Register bits */
#define MDI_STAT_EXTENDED		(1 << 0)
#define MDI_STAT_JABBER			(1 << 1)
#define MDI_STAT_LINK			(1 << 2)
#define MDI_STAT_AUTO_CAPABLE	(1 << 3)
#define MDI_STAT_REMOTE_FLT		(1 << 4)
#define MDI_STAT_AUTO_COMPLETE	(1 << 5)
#define MDI_STAT_10BASET_HALF	(1 << 11)
#define MDI_STAT_10BASET_FULL	(1 << 12)
#define MDI_STAT_TX_HALF		(1 << 13)
#define MDI_STAT_TX_FULL		(1 << 14)
#define MDI_STAT_T4_CAPABLE		(1 << 15)

/*
 * Bit definitions for the configure command.  NOTE:  Byte offsets are
 * offsets from the start of the structure (8 and up) to correspond
 * with the offsets in the PRO/100 PCI Adapter manual.
 */

/* Byte 0 */
#define BYTE_COUNT		0x16		/* use all 22 configure bytes */
#define CONFIG_BYTE_00	(BYTE_COUNT)

/* Byte 1 */
#define RX_FIFO_LIMIT	0x08
#define CONFIG_BYTE_01	(RX_FIFO_LIMIT)

/* Byte 2 */
#define ADAPT_IFS		0x00
#define CONFIG_BYTE_02	(ADAPT_IFS)

/* Byte 3 - must be 0x00 */
#define CONFIG_BYTE_03	(0x00)

/* Byte 4 */
#define RX_DMA_BCOUNT	0x00
#define CONFIG_BYTE_04	(RX_DMA_BCOUNT)

/* Byte 5 */
#define TX_DMA_BCOUNT	0x00
#define DMA_BCOUNT_ENAB	0x80
#define CONFIG_BYTE_05	(DMA_BCOUNT_ENAB | TX_DMA_BCOUNT)

/* Byte 6 */
#define NO_LATE_SCB		0x00
#define NO_TNO_INT		0x00	/* no interrupt on xmit failure */
#define INT_CU_IDLE		0x08	/* interrupt when CU goes idle */
#define NO_SV_BAD_FRAME	0x00	/* don't save bad frames */
#define DISCARD_RX_OVER	0x00	/* discard overrun frames */
#define BYTE6_REQUD		0x32	/* required "1" bits */

#define CONFIG_BYTE_06	(NO_LATE_SCB | NO_TNO_INT | INT_CU_IDLE |\
			 NO_SV_BAD_FRAME | DISCARD_RX_OVER | BYTE6_REQUD)

/* Byte 7 */
#define DISCARD_SHORT_RX	0x00	/* discard short rx frames */
#define ONE_URUN_RETRY		0x02	/* one underrun retry */
#define CONFIG_BYTE_07	(DISCARD_SHORT_RX | ONE_URUN_RETRY)

/* Byte 8 */
#define USE_503_MODE	0x00
#define USE_MII_MODE	0x01
#define CONFIG_BYTE_08	(USE_MII_MODE)

/* Byte 9 */
#define CONFIG_BYTE_09	(0x00)

/* Byte 10 */
#define INSERT_SRC_ADDR	0		/* Source address comes from IA of '559 */
#define PREAMBLE_LEN	0x20	/* 7 bytes */
#define NO_LOOP_BACK	0x00
#define INT_LOOP_BACK	0x40
#define EXT_LOOP_BACK	0xc0
#define BYTE10_REQUD	0x06	/* required "1" bits */
#define CONFIG_BYTE_10	(NO_LOOP_BACK | PREAMBLE_LEN | INSERT_SRC_ADDR | BYTE10_REQUD)

/* Byte 11 */
#define LIN_PRIORITY	0	/* normal CSMA/CD */
#define CONFIG_BYTE_11	(LIN_PRIORITY)

/* Byte 12 */
#define LIN_PRIORITY_MODE	0
#define IF_SPACING			96	/* inter-frame spacing */
#define CONFIG_BYTE_12		(IF_SPACING | LIN_PRIORITY_MODE)

/* Byte 13 */
#define CONFIG_BYTE_13	(0x00)

/* Byte 14 */
#define CONFIG_BYTE_14	(0xf2)

/* Byte 15 */
#define PROM_MODE		0		/* not promiscuous */
#define BROADCAST		0		/* disabled */
#define CRS				0x80	/* CDT = carrier */
#define BYTE15_REQUD	0x48	/* required "1" bits */
#define CONFIG_BYTE_15	(PROM_MODE | BROADCAST | CRS | BYTE15_REQUD)

/* Byte 16 */
#define CONFIG_BYTE_16	(0x00)

/* Byte 17 */
#define CONFIG_BYTE_17	(0x40)

/* Byte 18 */
#define STRIPPING_DISABLE	0x00
#define STRIPPING_ENABLE	0x01
#define PADDING_ENABLE  	0x02
#define XFER_CRC			0x04	/* store CRC */
#define NO_XFER_CRC			0x00
#define BYTE18_REQUD		0xf0	/* required "1" bits */
#define CONFIG_BYTE_18		(NO_XFER_CRC | PADDING_ENABLE | STRIPPING_ENABLE | BYTE18_REQUD)

/* Byte 19 */
#define NO_FORCE_FDX		0x00
#define FORCE_FDX			0x40
#define FDX_PIN_ENAB		0x80
#define CONFIG_BYTE_19_10T	FORCE_FDX
#define CONFIG_BYTE_19_100T	NO_FORCE_FDX

/* Byte 20 */
#define NO_MULTI_IA		0x00
#define CONFIG_BYTE_20	(NO_MULTI_IA)

/* Byte 21 */
#define NO_MULTI_ALL	0x00
#define CONFIG_BYTE_21	(NO_MULTI_ALL)


/*
 * 82559 structures.  NOTE: the 559 is used in 32-bit linear addressing
 * mode.  See alignment restrictions above.
 */

/* Result of PORT* self-test command - MUST be 16 byte aligned! */
#ifdef __ARMEB__
struct selfTest {
	UINT32	romSig;			/* signature of rom */
	union 
	{					/* Flag bits - as UINT32 or field */
		struct
		{
			UINT32	rsrv1 		: 2;
			UINT32	diagnTest 	: 1;
			UINT32	rsrv2 		: 1;
			UINT32	regTest 	: 1;
			UINT32	romTest 	: 1;
			UINT32	rsrv3 		: 2;
			UINT32	rsrv4 		: 3;
			UINT32	selfTest 	: 1;
			UINT32	rsrv5 		: 4;
			UINT32	rsrv6 		: 16;
		} bits;
		UINT32	word2;
	} u;
};
#else
struct selfTest {
	UINT32	romSig;			/* signature of rom */
	union {					/* Flag bits - as UINT32 or field */
		struct {
			UINT32	rsrv1 		: 2;
			UINT32	romTest 	: 1;
			UINT32	regTest 	: 1;
			UINT32	rsrv2 		: 1;
			UINT32	diagnTest 	: 1;
			UINT32	rsrv3 		: 6;
			UINT32	selfTest 	: 1;
			UINT32	rsrv4 		: 19;
		} bits;
		UINT32	word2;
	} u;
};
#endif

/* MDI Control Register */
#ifdef __ARMEB__
typedef union
{
    struct
    {
		UINT32 rsrv		: 2;	/* reserved */   
    	UINT32 intEnab	: 1;	/* 1 = interrupt at end of cycle */
    	UINT32 ready	: 1;	/* 1 = operation complete */
    	UINT32 op		: 2;	/* opcode, 1 for MDI write, 2 for MDI read */
    	UINT32 phyAdd	: 5;	/* PHY address */
    	UINT32 regAdd	: 5;	/* PHY register address */
    	UINT32 data		: 16;	/* data to write or data read */
    } bits;
    UINT32 word;
} MDI_CONTROL_U;

#else
typedef union
{
    struct
    {
    	UINT32 data		: 16;	/* data to write or data read */
    	UINT32 regAdd	: 5;	/* PHY register address */
    	UINT32 phyAdd	: 5;	/* PHY address */
    	UINT32 op		: 2;	/* opcode, 1 for MDI write, 2 for MDI read */
    	UINT32 ready	: 1;	/* 1 = operation complete */
    	UINT32 intEnab	: 1;	/* 1 = interrupt at end of cycle */
    	UINT32 rsrv		: 2;	/* reserved */   
    } bits;
    UINT32 word;
} MDI_CONTROL_U;
#endif

/* Command/Status Word of SCB */
#ifdef __ARMEB__
typedef union
{
    struct
    {
	/* status */
		UINT32	cus : 2;	/* Command unit status */
		UINT32	rus : 4;	/* Receive unit status */
		UINT32	rsrv1: 2;		/* Reserved */
		UINT32	cx	: 1;	/* Cmd exec completed */
		UINT32	fr	: 1;	/* Frame reception done */
		UINT32	cna : 1;	/* CU not active */
		UINT32	rnr : 1;	/* RU not ready */
		UINT32	mdi : 1;	/* MDI read/write complete */
		UINT32	swi : 1;	/* Software generated int. */
		UINT32  er	: 1;	/* Early Receive */
		UINT32	fcp : 1;	/* Flow Control Pause */
		
	/* command */
		UINT32	cuc : 4;	/* Command unit command */
		UINT32	rsrv2 : 1;		/* Reserved */
		UINT32	ruc : 3;	/* Receive unit command */
		UINT32	rsrv3 : 6;		/* Reserved */
		UINT32	si	: 1;	/* Software generated int. */
		UINT32	m	: 1;	/* Interrupt mask bit */
		
    } bits;
    struct
    {
		UINT16	status;
		UINT16	command;
    } words;
} CMD_STAT_U;

#else
typedef union
{
    struct
    {
	/* status */
		UINT32	rsrv1: 2;		/* Reserved */
		UINT32	rus : 4;	/* Receive unit status */
		UINT32	cus : 2;	/* Command unit status */
		UINT32	fcp : 1;	/* Flow Control Pause */
		UINT32  er	: 1;	/* Early Receive */
		UINT32	swi : 1;	/* Software generated int. */
		UINT32	mdi : 1;	/* MDI read/write complete */
		UINT32	rnr : 1;	/* RU not ready */
		UINT32	cna : 1;	/* CU not active */
		UINT32	fr	: 1;	/* Frame reception done */
		UINT32	cx	: 1;	/* Cmd exec completed */
	/* command */
		UINT32	ruc : 3;	/* Receive unit command */
		UINT32	rsrv2 : 1;		/* Reserved */
		UINT32	cuc : 4;	/* Command unit command */
		UINT32	m	: 1;	/* Interrupt mask bit */
		UINT32	si	: 1;	/* Software generated int. */
		UINT32	rsrv3 : 6;		/* Reserved */
    } bits;
    struct
    {
		UINT16	status;
		UINT16	command;
    } words;
} CMD_STAT_U;
#endif

/* System command block - on chip for the 82559 */
struct SCBtype 
{
	CMD_STAT_U	cmdStat; 
	UINT32		scb_general_ptr;	/* SCB General Pointer */
};


/* Command blocks - declared as a union; some commands have different fields */
union cmdBlock {

#ifdef __ARMEB__
	/* No operation */
	struct
	{
		UINT32	rsrv1	: 8;	/* reserved bits (set to 0) */
		UINT32	c		: 1;	/* 1 = command completed */
		UINT32	rsrv2	: 1;	/* reserved bits (set to 0) */
		UINT32	ok		: 1;	/* 1 = command completed, no error */
		UINT32	rsrv3	: 5;	/* reserved bits (set to 0) */
		UINT32	rsrv4	: 5;	/* reserved bits (set to 0) */
		UINT32	code	: 3;	/* command code (0 = NOP) */
		UINT32	el		: 1;	/* 1 = last cmdBlock in list */
		UINT32	s		: 1;	/* 1 = suspend CU upon completion */
		UINT32	i		: 1;	/* 1 = interrupt upon completion */
		UINT32	rsrv5	: 5;	/* reserved bits (set to 0) */
		union cmdBlock *link;	/* next block in list */
	} nop;
	/* Individual address setup */
	struct
	{
		UINT32	rsrv1	: 8;	/* reserved bits (set to 0) */
		UINT32	c		: 1;	/* 1 = command completed */
		UINT32	rsrv2	: 1;	/* reserved bits (set to 0) */
		UINT32	ok		: 1;	/* 1 = command completed, no error */
		UINT32	rsrv3	: 5;	/* reserved bits (set to 0) */
		UINT32	rsrv4	: 5;	/* reserved bits (set to 0) */
		UINT32	code	: 3;	/* command code (0 = NOP) */
		UINT32	el		: 1;	/* 1 = last cmdBlock in list */
		UINT32	s		: 1;	/* 1 = suspend CU upon completion */
		UINT32	i		: 1;	/* 1 = interrupt upon completion */
		UINT32	rsrv5	: 5;	/* reserved bits (set to 0) */
		union cmdBlock *link;	/* next block in list */
		UINT8	enetAddr[6];	/* hardware ethernet address */
		UINT16	rsrv6;		/* padding */
	} iaSetup;
	/* Configure */
	struct
	{
		UINT32	rsrv1	: 8;	/* reserved bits (set to 0) */
		UINT32	c		: 1;	/* 1 = command completed */
		UINT32	rsrv2	: 1;	/* reserved bits (set to 0) */
		UINT32	ok		: 1;	/* 1 = command completed, no error */
		UINT32	rsrv3	: 5;	/* reserved bits (set to 0) */
		UINT32	rsrv4	: 5;	/* reserved bits (set to 0) */
		UINT32	code	: 3;	/* command code (0 = NOP) */
		UINT32	el		: 1;	/* 1 = last cmdBlock in list */
		UINT32	s		: 1;	/* 1 = suspend CU upon completion */
		UINT32	i		: 1;	/* 1 = interrupt upon completion */
		UINT32	rsrv5	: 5;	/* reserved bits (set to 0) */
		union cmdBlock *link;	/* next block in list */
		UINT8	configData[22];	/* configuration data */
	} configure;
	/* Multicast address setup */
	struct
	{
		UINT32	rsrv1	: 8;	/* reserved bits (set to 0) */
		UINT32	c		: 1;	/* 1 = command completed */
		UINT32	rsrv2	: 1;	/* reserved bits (set to 0) */
		UINT32	ok		: 1;	/* 1 = command completed, no error */
		UINT32	rsrv3	: 5;	/* reserved bits (set to 0) */
		UINT32	rsrv4	: 5;	/* reserved bits (set to 0) */
		UINT32	code	: 3;	/* command code (0 = NOP) */
		UINT32	el		: 1;	/* 1 = last cmdBlock in list */
		UINT32	s		: 1;	/* 1 = suspend CU upon completion */
		UINT32	i		: 1;	/* 1 = interrupt upon completion */
		UINT32	rsrv5	: 5;	/* reserved bits (set to 0) */
		union cmdBlock *link;	/* next block in list */
		UINT16	mcCount;		/* # of bytes in mcAddrList[] */
		UINT8	mcAddrList[6];	/* list of multicast addresses */
	} mcSetup;
	/* Transmit */
	struct
	{
		UINT32	rsrv1	: 8;	/* reserved bits (set to 0) */
		UINT32	c		: 1;	/* 1 = command completed */
		UINT32	rsrv2	: 1;	/* reserved bits (set to 0) */
		UINT32	ok		: 1;	/* 1 = command completed, no error */
		UINT32	u		: 1;	/* 1 = underrun was encountered */
		UINT32	rsrv3	: 4;	/* reserved bits (set to 0) */
		UINT32	rsrv4	: 4;	/* reserved bits (set to 0) */
		UINT32	sf		: 1;	/* 1 = flexible mode */
		UINT32	code	: 3;	/* command code (4 = transmit) */
		UINT32	el		: 1;	/* 1 = last cmdBlock in list */
		UINT32	s		: 1;	/* 1 = suspend CU upon completion */
		UINT32	i		: 1;	/* 1 = interrupt upon completion */
		UINT32	rsrv5	: 5;	/* reserved bits (set to 0) */
		union cmdBlock *link;	/* next block in list */
		UINT8	*tbdAddr;		/* tx buf addr; all 1s for simp mode */
		UINT32	tcbCount1 : 8;	/* # bytes to be tx from cmd block */
		UINT32	eof : 1;		/* 1 = entire frame in cmd block */
		UINT32	rsrv6 : 1;		/* reserved (set to 0) */
		UINT32	tcbCount2 : 6;	/* # bytes to be tx from cmd block */
		UINT8	tx_threshold;	/* # of bytes in FIFO before xmission */
		UINT8	tbd_number;		/* # of tx. buffers in TBD array */
		UINT8	destAddr[6];	/* destination hardware address */
		UINT16	length;			/* 802.3 packet length (from packet) */
		UINT8	txData[ETHERMTU];	/* optional data to tx */
	} transmit;
	/* Dump 82559 registers */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (6 = dump) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT8	*bufAddr;	/* where to dump registers */
	} dump;
	/* Diagnose - perform self test */
	struct {
		UINT32	rsrv1 : 11;	/* reserved bits (set to 0) */
		UINT32	f : 1;		/* 1 = self test failed */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv3 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (7 = diagnose) */
		UINT32	rsrv4 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
	} diagnose;
#else
	/* No operation */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (0 = NOP) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
	} nop;
	/* Individual address setup */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (1 = ia setup) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT8	enetAddr[6];	/* hardware ethernet address */
		UINT16	rsrv4;		/* padding */
	} iaSetup;
	/* Configure */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (2 = configure) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT8	configData[22];	/* configuration data */
	} configure;
	/* Multicast address setup */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (3 = mc setup) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT16	mcCount;	/* # of bytes in mcAddrList[] */
		UINT8	mcAddrList[6];	/* list of multicast addresses */
	} mcSetup;
	/* Transmit */
	struct {
		UINT32	rsrv1 : 12;		/* reserved bits (set to 0) */
		UINT32	u : 1;			/* 1 = underrun was encountered */
		UINT32	ok : 1;			/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;		/* reserved bits (set to 0) */
		UINT32	c : 1;			/* 1 = command completed */
		UINT32	code : 3;		/* command code (4 = transmit) */
		UINT32	sf : 1;			/* 1 = flexible mode */
		UINT32	rsrv3 : 9;		/* reserved bits (set to 0) */
		UINT32	i : 1;			/* 1 = interrupt upon completion */
		UINT32	s : 1;			/* 1 = suspend CU upon completion */
		UINT32	el : 1;			/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT8	*tbdAddr;		/* tx buf addr; all 1s for simp mode */
		UINT32	tcbCount : 14;	/* # bytes to be tx from cmd block */
		UINT32	rsrv4 : 1;		/* reserved (set to 0) */
		UINT32	eof : 1;		/* 1 = entire frame in cmd block */
		UINT8	tx_threshold;	/* # of bytes in FIFO before xmission */
		UINT8	tbd_number;		/* # of tx. buffers in TBD array */
		UINT8	destAddr[6];	/* destination hardware address */
		UINT16	length;			/* 802.3 packet length (from packet) */
		UINT8	txData[ETHERMTU];	/* optional data to tx */
	} transmit;
	/* Dump 82559 registers */
	struct {
		UINT32	rsrv1 : 13;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (6 = dump) */
		UINT32	rsrv3 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
		UINT8	*bufAddr;	/* where to dump registers */
	} dump;
	/* Diagnose - perform self test */
	struct {
		UINT32	rsrv1 : 11;	/* reserved bits (set to 0) */
		UINT32	f : 1;		/* 1 = self test failed */
		UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
		UINT32	ok : 1;		/* 1 = command completed, no error */
		UINT32	rsrv3 : 1;	/* reserved bits (set to 0) */
		UINT32	c : 1;		/* 1 = command completed */
		UINT32	code : 3;	/* command code (7 = diagnose) */
		UINT32	rsrv4 : 10;	/* reserved bits (set to 0) */
		UINT32	i : 1;		/* 1 = interrupt upon completion */
		UINT32	s : 1;		/* 1 = suspend CU upon completion */
		UINT32	el : 1;		/* 1 = last cmdBlock in list */
		union cmdBlock *link;	/* next block in list */
	} diagnose;
#endif
};


/* Receive frame descriptors (uses simplified memory structure) */
struct rfd {
#ifdef __ARMEB__
	UINT32	frameTooshort : 1;
	UINT32	rsrv1 : 1;	/* reserved bits (set to 0) */
	UINT32	typeFrame : 1;	/* Type field of pkt. indicates a TYPE frame */
	UINT32	rxErr : 1;	/* RX_ER pin asserted during frame reception */
	UINT32	rsrv2 : 2;	/* reserved bits (set to 0) */
	UINT32	iaMatch : 1;	/* Dest addr matched chip's hardware addr */
	UINT32	rxColl : 1;	/* 1 = collision on reception */
	UINT32	c : 1;		/* 1 = command completed */
	UINT32	rsrv3 : 1;	/* reserved bits (set to 0) */
	UINT32	ok : 1;		/* 1 = command completed, no error */
	UINT32	rsrv4 : 1;	/* reserved bits (set to 0) */
	UINT32	crcErr : 1;	/* CRC error on aligned frame */
	UINT32	alignErr : 1;	/* CRC error on misaligned frame */
	UINT32	noRsrc : 1;	/* No resources (out of buffer space) */
	UINT32	dmaOverrun : 1;	/* DMA overrun (couldn't get local bus) */
	UINT32	rsrv5 : 3;	/* reserved bits (set to 0) */
	UINT32	h : 1;		/* 1 = Header RFD */
	UINT32	sf : 1;		/* 1 = Flexible mode */
	UINT32	rsrv6 : 3;	/* reserved bits (set to 0) */
	UINT32	el : 1;		/* 1 = last cmdBlock in list */
	UINT32	s : 1;		/* 1 = suspend CU upon completion */
	UINT32	rsrv7 : 6;	/* reserved bits (set to 0) */
	union cmdBlock *link;	/* next block in list */
	UINT8	*rbdAddr;	/* rx buf desc addr; all 1s for simple mode */

	UINT32	actCount : 14;	/* # bytes in this buffer (set by 82559) */
	UINT32	f : 1;		/* 1 = buffer used */
	UINT32	eof : 1;	/* 1 = last buffer for this frame */

	UINT32	size : 14;	/* # bytes avail in this buffer (set by CPU) */
	UINT32	rsrv8 : 2;	/* reserved bits (set to 0) */

	UINT8	destAddr[6];	/* destination address */
	UINT8	sourceAddr[6];	/* source address */
	UINT16	length;		/* 802.3 packet length (from packet) */
	UINT8	rxData[ETHERMTU];	/* optional data (simplified mode) */
#else
	UINT32	rxColl : 1;	/* 1 = collision on reception */
	UINT32	iaMatch : 1;	/* Dest addr matched chip's hardware addr */
	UINT32	rsrv1 : 2;	/* reserved bits (set to 0) */
	UINT32	rxErr : 1;	/* RX_ER pin asserted during frame reception */
	UINT32	typeFrame : 1;	/* Type field of pkt. indicates a TYPE frame */
	UINT32	rsrv2 : 1;	/* reserved bits (set to 0) */
	UINT32	frameTooshort : 1;
	UINT32	dmaOverrun : 1;	/* DMA overrun (couldn't get local bus) */
	UINT32	noRsrc : 1;	/* No resources (out of buffer space) */
	UINT32	alignErr : 1;	/* CRC error on misaligned frame */
	UINT32	crcErr : 1;	/* CRC error on aligned frame */
	UINT32	rsrv3 : 1;	/* reserved bits (set to 0) */
	UINT32	ok : 1;		/* 1 = command completed, no error */
	UINT32	rsrv4 : 1;	/* reserved bits (set to 0) */
	UINT32	c : 1;		/* 1 = command completed */
	UINT32	rsrv5 : 3;	/* reserved bits (set to 0) */
	UINT32	sf : 1;		/* 1 = Flexible mode */
	UINT32	h : 1;		/* 1 = Header RFD */
	UINT32	rsrv6 : 9;	/* reserved bits (set to 0) */
	UINT32	s : 1;		/* 1 = suspend CU upon completion */
	UINT32	el : 1;		/* 1 = last cmdBlock in list */
	union cmdBlock *link;	/* next block in list */
	UINT8	*rbdAddr;	/* rx buf desc addr; all 1s for simple mode */
	UINT32	actCount : 14;	/* # bytes in this buffer (set by 82559) */
	UINT32	f : 1;		/* 1 = buffer used */
	UINT32	eof : 1;	/* 1 = last buffer for this frame */
	UINT32	size : 14;	/* # bytes avail in this buffer (set by CPU) */
	UINT32	rsrv7 : 2;	/* reserved bits (set to 0) */
	UINT8	destAddr[6];	/* destination address */
	UINT8	sourceAddr[6];	/* source address */
	UINT16	length;		/* 802.3 packet length (from packet) */
	UINT8	rxData[ETHERMTU];	/* optional data (simplified mode) */
#endif
};

#endif
