/* ethernet.c
//=============================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//=============================================================================
*/
/* Portions of this file modified by Intel Corporation. 
Portions created by Intel Corporation are:
Copyright  2002-2004 Intel Corporation  */



#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2x00.h"

#include "led.h"
#include "cfg_utility.h"
#include "ether_net.h"
#include "diag_utils.h"


/* Globals needed by both main program and irq handler */
static volatile struct SCBtype *pSCB = NULL;	/* Pointer to SCB in use */
static volatile UINT32 waitSem = 0;	/* Used to block test until interrupt */
static volatile UINT32 rxSem = 0;	/* Used to block test until rx sinterrupt */
static volatile UINT16 i559Status = 0;	/* Status code from SCB */
static UINT32 i82559_heap_size = 0;
static UINT8 *i82559_heap_base = NULL;
static UINT8 *i82559_heap_free = NULL;
static UINT32 adapter;		/* Ptr to PCI Ethernet adapter */
static volatile UINT8 S_MAC_address[6];	/* MAC address 6bytes */
static volatile UINT8 D_MAC_address[6];	/* MAC address 6bytes */
static volatile UINT8 EEPROM_MAC_address[6];	/* MAC address 6bytes */
/*static long timer0_ticks = 0;*/
static UINT32 phy_id = 0;

/* 82559 required data structures which must be allocated */
static struct rfd *pRfd = NULL;
static union cmdBlock *pCmdBlock = NULL;
static char	*pPacketBuf = "";

static volatile int link_speed = SPEED_NOLINK;

#define CPLD_INTERRUPT_ETHERNET		1 // ethernet
#define PCI_ISR_COUNT				8 // for CPLD interrupts
UINT32 pciIntList[PCI_ISR_COUNT];		/* list of int handlers */
UINT32 pciIntDataList[PCI_ISR_COUNT];	/* list of data objects */
UINT32 nic_basemap[6];

// Static Function prototypes
//============================
static void Ether_Help(void);
static int ether_test(UINT32);
static void display_loopback_init_error(void);
static void display_internal_loopback_error(void);
static void display_external_error(void);

static UINT16 readMDI(int unit, UINT8 phyAdd, UINT8 regAdd);
#ifdef SUPPORT_OTHER_PHY
static void writeMDI(int unit, UINT8 phyAdd, UINT8 regAdd, UINT16 data);
#endif
static int i559SelfTest(void);
static void sendCommand (UINT8 cuc, UINT8 ruc, UINT32 scb_general_ptr);
static void makePacket (UINT8 *pPacket, int length);
static int i559Init(void);
static int waitForInt(void);
static int i559AddrSet(void);
static int initPHY(UINT32 device_type, int loop_mode);
static int i559Config(UINT8 loopBackMode);
static int i559RUStart(void);
static void setUpPacket(char *pBuf);
static int waitForRxInt(void);
static int checkPacket (UINT8 *pTxBuffer, UINT8 *pRxBuffer, int	length);
static int txPacket(char *pBuf);
static int pci_ether_test();
static int Device_Seek(UINT32 vendor_id, UINT32 dev_id,
						UINT32 dev_ind);

// test flags
//============
static volatile int link_test_flag = 0;
static volatile int internal_loopback_test_flag = 0;
static volatile int external_test_flag = 0;
static volatile int self_test_flag = 0;


int loopback_test_flag = 0;
int verbose = 0;
int data_offset = 0;
int transceiver;
int already_reset = 0;

	typedef struct _NIC_testIn
	{
	UINT32 NIC_sel;
	UINT32 link_sel;
	UINT32 internal_sel;
	UINT32 external_sel;
	UINT32 self_sel;
	UINT32 all_sel;
  	} NIC_testIn;


	typedef struct _SETENET_testIn
	{
	UINT32 NIC_sel;
	UINT8 ENET_DATA_0;
	UINT8 ENET_DATA_1;
	UINT8 ENET_DATA_2;
	UINT8 ENET_DATA_3;
	UINT8 ENET_DATA_4;
	UINT8 ENET_DATA_5;
  	} SETENET_testIn;

	typedef struct _GETENET_testIn
	{
	UINT32 NIC_sel;
  	} GETENET_testIn;


HANDLE ETHERNETISR;
/*******************
Ethernet_Test
********************/
int Ethernet_test(NIC_testIn* pIn,PassFail_testOut* pOut)
	{
	char opt;
	unsigned int loops;
	int forever = 0;
	int status;
	link_test_flag = 0;
	internal_loopback_test_flag = 0;
	external_test_flag = 0;
	self_test_flag = 0;
    verbose = 1;	
	loops = 1;
	pOut->ReturnCode = PASS;         
	pOut->PassFail = PASS;

	if ((!IS_EGRESS_NPU) && (!pIn->NIC_sel))
	{
	printf("NIC 0 only allowed on Egress NPU\n");

	pOut->PassFail = FAIL;
	return(pOut->PassFail);
	}

	if ((IS_EGRESS_NPU) && (pIn->NIC_sel))
	{
	printf("NIC 1 only allowed on Ingress NPU\n");
	pOut->PassFail = FAIL;
	return(pOut->PassFail);
	}

	//decide what test the user has picked	
	if ((pIn->link_sel) || (pIn->all_sel))
	link_test_flag = 1;		 // user picked link test

    if ((pIn->internal_sel) || (pIn->all_sel)) 	 
    internal_loopback_test_flag = 1;	//user picked internal loopback

    if ((pIn->external_sel) || (pIn->all_sel))
 	external_test_flag = 1;	 //user picked external 

    if ((pIn->self_sel) || (pIn->all_sel))
    self_test_flag = 1;	//user picked self test
 

	// Read the configuration strapping to decide if PCI master
	if(IS_EGRESS_NPU)
		transceiver = TX;
	else
		transceiver = RX;

	//pciIntConnect(CPLD_INTERRUPT_ETHERNET, (UINT32)i559IntHandler, 0xdeadbeef);
	/* Install the Ethernet Handler */
	ETHERNETISR = InstallHandler (ETHERNETIRQ, (HANDLER)i559IntHandler, (void *)ETHERNET_DATA);

	/* Enable the IRQ for the doorbell ISR */
	EnableIRQ (ETHERNETISR);

	while(loops--)
	{
		if(KEYPRESS)
		{
			//pciIntDisconnect(CPLD_INTERRUPT_ETHERNET);
			return;
		}
	  	status = ether_test(pIn->NIC_sel);
	}

	//pciIntDisconnect(CPLD_INTERRUPT_ETHERNET);
	DisableIRQ (ETHERNETISR);
	RemoveHandler (ETHERNETISR);

	pOut->PassFail = status;
	return(pOut->PassFail);
}


/********************
	ether_test
*********************/
int ether_test(UINT32 NIC_SEL)
{
	int status;

	if (Device_Seek(VENDOR_INTEL, I82559ER, NIC_SEL ) != OK)
	{
		printf("No supported Ethernet devices found\n");
		return(FAIL);
	}
	status = pci_ether_test();
	pSCB = NULL;
	i82559_heap_base = NULL;
	i82559_heap_free = NULL;
	pRfd = NULL;
	pCmdBlock = NULL;
	pPacketBuf = "";

	return(status);
}

/*******************
	Device_Seek
*********************/
int Device_Seek(UINT32 vendor_id, UINT32 dev_id, UINT32 dev_ind)
{
	UINT32 nic_dev_id = -1;
	UINT32 bus_number, device_number, function_number;

	verbose = 1;

	if (verbose)
	{
		printf("Looking for Adapter on PCI Bus with:\n");
		printf("Vendor Id = 0x%04x\n",vendor_id);
		printf("Device Id = 0x%04x\n",dev_id);
		printf("Index     = %x\n",dev_ind);
	}

	if (pciFindDevice (vendor_id, dev_id, dev_ind, &bus_number, &device_number, &function_number) != OK)
	{
		printf("Failed to Find NIC %d\n", device_number);
		return (ERROR);
	}

	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_0, 
		 &nic_basemap[0]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_1, 
		&nic_basemap[1]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_2, 
		 &nic_basemap[2]);

	//nic_basemap[0] = BRD_PCIMEM_PCI2CPU(nic_basemap[0], 0);
		nic_basemap[1] = BRD_PCIIO_PCI2CPU(nic_basemap[1], 0); 
		nic_basemap[1] &= 0xfffffffc;



	if (verbose)
	{
		printf("Adapter found at :\n");
		printf("  PCI Bus Number : %d\n", bus_number);
		printf("  Device Number  : %d\n", device_number);
		printf("  Function Number: %d\n", function_number);
		printf("  nic_basemap[0]: 0x%x\n", nic_basemap[0]);
		printf("  nic_basemap[1]: 0x%x\n", nic_basemap[1]);
		printf("  nic_basemap[2]: 0x%x\n\n", nic_basemap[2]);
	}

return (OK);
    
}


#define ASCII_TO_DEC 48
void atod(char a, int* b)
{
	*b = (int)(a - ASCII_TO_DEC);
}

static void resetChip(void)
{
	*PORT_REG(adapter) = PORT_RESET;
	DelayNUsec(5000);
}

/*********************
	pci_ether_test
*********************/
int pci_ether_test(void)
{
	char buf[4] = "";
	UINT16 phy_addr_reg, temp1, temp2;
	int status = PASS;

	/* read the PCI BAR for the Ethernet controller */
	adapter = nic_basemap[0]; 

	/* strip off BAR indicator bits */
	adapter &= 0xfffffff0;

	/* pointer to on-chip SCB */
	pSCB = (struct SCBtype *)(adapter + SCB_OFFSET);


	if (verbose)
	{
		printf("pSCB: 0x%x\n", pSCB);	
		printf ("NIC Base Address  = 0x%X\n", adapter);
	}
	if (verbose)
		printf("Resetting chip... ");

	// reset the 82559 to start with a clean slate
	resetChip();
	if (verbose)
		printf("Done.\n");

	// Get the UUT's ethernet address
	if (get_ether_addr () == ERROR)
	{
		printf("Error Reading Adapter Ethernet Address\n");
		return(FAIL);
	}

	// Self Test
	//===========
	if(self_test_flag)
	{
		// Run the built-in self test through the port register
		if (i559SelfTest() == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
 	  		DelayNUsec(1000000);
			printf ("NIC Self Test... FAIL\n");
 			DelayNUsec(1000000);
			
			return(FAIL);
		}
		printf ("NIC Self Test... PASS\n");
		// Reset clears the interrupt mask
	}

	// Link Test
	//===========
	if(link_test_flag || external_test_flag)
	{
		temp1 = readMDI (0 ,MDI_DEFAULT_PHY_ADDR, MDI_PHY_ID_1);
		temp2 = readMDI (0 ,MDI_DEFAULT_PHY_ADDR, MDI_PHY_ID_2);
		phy_id = ((temp1 << 16) | temp2);
		printf("phy_id = %x\n", phy_id);
		if ((phy_id ) == I82559_PHY_ID)

		{
			if(verbose)
				printf ("Intel 82559 PHY detected\n");

			/* dummy read for reliable status */
			(void)readMDI (0, MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT);

			temp1 = readMDI (0, MDI_DEFAULT_PHY_ADDR, MDI_PHY_STAT);
			if(verbose)
				printf ("Status Register Link Status is %s\n", (temp1 & MDI_STAT_LINK) ? "UP" : "DOWN");

			phy_addr_reg = readMDI (0, MDI_DEFAULT_PHY_ADDR, I82555_STATCTRL_REG);

			if (temp1 & MDI_STAT_LINK)	// speed only valid with good LNK
  			{
				if(verbose)
		   			printf ("Connect Speed is %s\n", (phy_addr_reg & I82555_100_MBPS) ? "100Mbps" : "10Mbps");
				link_speed = (phy_addr_reg & I82555_100_MBPS) ? SPEED_100M : SPEED_10M;
				printf ("NIC Link Test... PASS\n");
			}
			else
			{
				printf ("Connect Speed is NOT VALID\n");
				printf ("NIC Link Test... FAIL\n");
				return(FAIL);
			}
		}
		else
		{
			printf("***ERROR***: I82555 PHY not detected\n");
			return(FAIL);
		}
	} // if(link_test_flag)

	// Initialization for Loopback/External Test
	//==================================
	if(internal_loopback_test_flag || external_test_flag)
	{
		// Initialize data structures
		if(verbose)
			printf ("Initializing ethernet controller.\n");
		if (i559Init() == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_loopback_init_error();
			return(FAIL);
		}

		// Set hardware address
		if(verbose)
			printf ("Setting hardware address.\n");
		if (i559AddrSet() == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_loopback_init_error();
			return(FAIL);
		}
	}

	// Internal Loopback Test
	//========================
	if(internal_loopback_test_flag)
	{
		// Configure for Internal loopback
		if (i559Config (INT_LOOP_BACK) == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_internal_loopback_error();
			return(FAIL);
		}

		DelayNUsec(100000);

		// Initialize receive buffer and enable receiver
		if(verbose)
			printf ("Initializing receive buffer and enabling receiver.\n");
		if (i559RUStart () == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_internal_loopback_error();
			return(FAIL);
		}	

		// Send a packet
		if(verbose)
			printf ("Sending packet.\n");
		setUpPacket (pPacketBuf);
		loopback_test_flag = 1;
		if (txPacket (pPacketBuf) == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_internal_loopback_error();
			return(FAIL);
		}
		loopback_test_flag = 0;
		printf ("NIC Internal Loopback Test... PASS\n");
	}

	// External Test
	//===============
	if(external_test_flag)
	{
		// Configuring for external
		if (i559Config (NO_LOOP_BACK) == ERROR)
		//if (i559Config (EXT_LOOP_BACK) == ERROR)
		{
			//To Do DisableIRQ(p->esd8xxISR);
			display_external_error();
			return(FAIL);
		}

		DelayNUsec(100000);

		if(transceiver == RX)
			printf ("Press return to start receiving external packets\n");
		else
			printf ("Press return to start transmitting external packets\n");
		while(!KEYPRESS);

		setUpPacket (pPacketBuf);

		if ((i559RUStart() == ERROR)||(txPacket (pPacketBuf) == ERROR))
		{
			printf ("Double-check TP cable and 10/100 Base T Hub\n");
			printf ("Try testing them with another system\n");
			printf ("(such as a workstation) that is working correctly.\n");

			//To Do DisableIRQ(p->esd8xxISR);

			display_external_error();
			return(FAIL);
		}

			//To Do DisableIRQ(p->esd8xxISR);

		if(transceiver == RX)
		{
			printf ("NIC External Receive Test... PASS\n");
		}
		else
		{
			printf ("NIC External Transmit Test... PASS\n");
		}
	}	// if(external_test_flag)


	return(PASS);
}

void display_loopback_init_error(void)
{

	printf ("NIC Loopback Initialization... FAIL\n");
	DelayNUsec(1000000);
}

void display_internal_loopback_error(void)
{
	printf ("NIC Internal Loopback Test... FAIL\n");
	DelayNUsec(1000000);
}

void display_external_error(void)
{
	printf ("NIC External Loopback Test... FAIL\n");
	DelayNUsec(1000000);
}

void find_ethernet_address(GETENET_testIn* pIn,void* pUnused)
{
	if (Device_Seek(VENDOR_INTEL, I82559ER, pIn->NIC_sel ) != OK)
	{
		printf("No supported Ethernet devices found\n");
		return;
	}


	if (get_ether_addr () == ERROR)
	{
		printf("Error Reading Adapter Ethernet Address\n");
		return;
	}
}

/*******************
	get_ether_addr
********************/
int get_ether_addr(void)
{
	int i;
	struct board_config board_config_data;

	Get_Eeprom_Mac_Addr();  //Get the data into our struct

	return(OK);
}

/*******************
	set_ether_addr
********************/

int set_ether_addr(SETENET_testIn *pIn, PassFail_testOut* pOut)
{
	int i;
	int prom_size;
	UINT16 eepromsum;
	UINT32 ioaddr;
	struct board_config board_config_data;
	int addr_length;

	pOut->ReturnCode = PASS;         
	pOut->PassFail = PASS;



	if (Device_Seek(VENDOR_INTEL, I82559ER, pIn->NIC_sel ) != OK)
	{
		printf("No supported Ethernet devices found\n");
		pOut->PassFail = FAIL;
		return(pOut->PassFail);
	}


	//ioaddr = nic_basemap[1];
	ioaddr = nic_basemap[0];

	// read eeprom and get 82559's address length
  	addr_length = GetEepromSize(ioaddr);
    switch (addr_length)
    {
	case 6: prom_size = 64; break;
	case 7: prom_size = 128; break;
	case 8: prom_size = 256; break;
    }

	printf("MAC EEPROM addr_length = %x\n", addr_length);
	printf("MAC EEPROM prom_size = %d\n", prom_size);

	EEPROM_MAC_address[0] = pIn->ENET_DATA_0;
	EEPROM_MAC_address[1] = pIn->ENET_DATA_1;
	EEPROM_MAC_address[2] = pIn->ENET_DATA_2;
	EEPROM_MAC_address[3] = pIn->ENET_DATA_3;
	EEPROM_MAC_address[4] = pIn->ENET_DATA_4;
	EEPROM_MAC_address[5] = pIn->ENET_DATA_5;

    writeEeprom(ioaddr,0,(EEPROM_MAC_address[1]<<8|EEPROM_MAC_address[0]), addr_length);
    writeEeprom(ioaddr,1,(EEPROM_MAC_address[3]<<8|EEPROM_MAC_address[2]), addr_length);
    writeEeprom(ioaddr,2,(EEPROM_MAC_address[5]<<8|EEPROM_MAC_address[4]), addr_length);
	writeEeprom(ioaddr,3, 0x0303,addr_length);
	writeEeprom(ioaddr,5, 0x0201,addr_length);
	writeEeprom(ioaddr,6, 0x4701,addr_length);
	writeEeprom(ioaddr,10,0x4080,addr_length);
	writeEeprom(ioaddr,11,0x000C,addr_length);
	writeEeprom(ioaddr,12,0x8086,addr_length);
	writeEeprom(ioaddr,13,0x0000,addr_length);

	eepromsum = 0;
    for (i=0;i<prom_size-1;i++)
       	eepromsum += Eeprom_Read(ioaddr, i, addr_length);

    printf("write checksum 1= 0x%04x\n",eepromsum);

    eepromsum = (UINT16)EE_CHECKSUM - eepromsum;
    printf("write checksum 2= 0x%04x\n",eepromsum);

	writeEeprom(ioaddr, prom_size-1, eepromsum, addr_length);

	return(pOut->PassFail);
}

/*******************
	i559SelfTest
********************/
static int i559SelfTest(void)
{
	volatile struct selfTest *pSelfTestMem;
	UINT32 oldWord2;
	long delay;
	UINT32 temp;
	int	rtnVal;


	/* Allocate some memory for the self test */
 	pSelfTestMem = (struct selfTest *) malloc (sizeof(struct selfTest));

	pSelfTestMem = (struct selfTest *)ALIGN(pSelfTestMem,32);


	if (pSelfTestMem == NULL)
	{
		printf ("Couldn't get memory for self test.\n");
		return (ERROR);												
	}

	if (verbose)
	{
		printf ("Sending PORT self-test command...\n");

		printf ("Local Dump address = 0x%X\n", pSelfTestMem);

	}

	/* Set all bits in  second word, wait until it changes or a timeout */
	pSelfTestMem->u.word2 = ~0;
	oldWord2 = pSelfTestMem->u.word2;

	temp = (VIRT_TO_BUS(pSelfTestMem)) | PORT_SELF_TEST;

	#ifdef __ARMEB__
		*PORT_REG(adapter) = SWAP32(temp);
	#else
		*PORT_REG(adapter) = temp;
	#endif

	/* Wait for test completion or for timeout */
	for (delay = 0; (delay < MAX_DELAY) && (pSelfTestMem->u.word2 == oldWord2); delay++);	/* Wait... */

	/* Print results */
	if (verbose || (pSelfTestMem->u.word2 == oldWord2))
	{
		printf ("Self test result: %s\n", (pSelfTestMem->u.bits.selfTest) ? "FAIL" : "PASS");
		printf ("ROM content test: %s\n", (pSelfTestMem->u.bits.romTest) ? "FAIL" : "PASS");
		printf ("Register test:    %s\n", (pSelfTestMem->u.bits.regTest) ? "FAIL" : "PASS");
		printf ("Diagnose test:    %s\n", (pSelfTestMem->u.bits.diagnTest) ? "FAIL" : "PASS");
		#ifdef __ARMEB__
			printf ("ROM signature:    0x%x\n", SWAP32(pSelfTestMem->romSig));
		#else
			printf ("ROM signature:    0x%x\n", pSelfTestMem->romSig);
		#endif
	}

	rtnVal = pSelfTestMem->u.bits.selfTest ? PROM_OK : PROM_ERROR;

	free((void *)pSelfTestMem);

	return (rtnVal);
}

/******************************************************************************
*
* sendCommand - send a command to the 82559 via the on-chip SCB
*
* Send a command to the 82559.  On the 82559, the Channel Attention signal
* has been replaced by an on-chip SCB.  Accesses to the Command Word portion
* of the SCB automatically forces the '559 to look at the various data
* structures which make up its interface.
*/
static void sendCommand (UINT8 cuc, UINT8 ruc, UINT32 scb_general_ptr)
{
	register CMD_STAT_U temp_cmdStat;
	volatile union cmdBlock *pBlock = (union cmdBlock *)BUS_TO_VIRT(scb_general_ptr);
	volatile int loop_ctr;
	UINT32 temp;

	/* Mask adapter interrupts to prevent the interrupt handler from
	   playing with the SCB */

	//To Do DisableIRQ(p->esd8xxISR);

	DelayNUsec(100000);

	/* must wait for the Command Unit to become idle to prevent
       us from issueing a CU_START to an active Command Unit */
    for (loop_ctr = BUSY_WAIT_LIMIT; loop_ctr > 0; loop_ctr--)
    {
        if ((pSCB->cmdStat.words.status & SCB_S_CUMASK) == SCB_S_CUIDLE)
            break;
    }
    if (loop_ctr == 0)
    {
        printf("sendCommand: CU won't go idle, command ignored\n");

		//To Do EnableIRQ(p->esd8xxISR);

        return;
    }

	/* when setting the command word, read the current word from
	   the SCB and preserve the upper byte which contains the interrupt
	   mask bit */
	#ifdef __ARMEB__
	temp_cmdStat.words.command = (pSCB->cmdStat.words.command & 0xff);
	#else
	temp_cmdStat.words.command = (pSCB->cmdStat.words.command & 0xff00);
	#endif
	temp_cmdStat.words.status = 0;

	/* set up the Command and Receive unit commands */
	temp_cmdStat.bits.cuc = cuc & 0xF;
	temp_cmdStat.bits.ruc = ruc & 0x7;

	/* Clear flag */
	waitSem = 0;

	/* write the General Pointer portion of the SCB first */
	#ifdef __ARMEB__
		pSCB->scb_general_ptr = SWAP32(scb_general_ptr);
	#else
		pSCB->scb_general_ptr = scb_general_ptr;
	#endif
	temp = pSCB->scb_general_ptr;

	/* write the Command Word of the SCB */
	pSCB->cmdStat.words.command = temp_cmdStat.words.command;
	temp = pSCB->cmdStat.words.command;

	/* only wait for command which will complete immediately */
	if ((scb_general_ptr != 0/* NULL*/) && (ruc != RU_START))
	{
		/* wait for command acceptance and completion */
		for (loop_ctr = BUSY_WAIT_LIMIT; loop_ctr > 0; loop_ctr--)
		{
			if ((pSCB->cmdStat.bits.cuc == 0) && (pBlock->nop.c == 1))
				break;
		}
		if (loop_ctr == 0)
		{
			printf("\nsendCommand: Timeout on command complete\n");
			printf("pBlock: %x\n", pBlock);
			printf("pSCB: 0x%x\n", pSCB);
			printf("Cmd Complete bit = %02X\n", pBlock->nop.c);
			printf("pSCB->cmdStat.bits.cuc: %x\n", pSCB->cmdStat.bits.cuc);
			printf("pBlock->nop.c: %x\n", pBlock->nop.c);
			printf("CU command       = 0x%02X\n", cuc);
			printf("RU command       = 0x%02X\n", ruc);
			printf("SCB Gen Ptr      = 0x%X\n", scb_general_ptr);
			printf("scb status       = 0x%04X\n", pSCB->cmdStat.words.status);
			printf("scb command      = 0x%04X\n", pSCB->cmdStat.words.command);
		}
	}

		//To Do EnableIRQ(p->esd8xxISR);

	return;
}


/*
 * Setup contents of a packet.
 */
static void makePacket (UINT8 *pPacket, int length)
{
	int byteNum;	/* Current byte number */

	for (byteNum = 0; byteNum < length; byteNum++)
		*pPacket++ = data_offset + byteNum + ' ';
}

/*******************
	i559IntHandler
********************/
int i559IntHandler(int arg)
{
	register CMD_STAT_U temp_cmdStat;
	register int rxFlag = FALSE;

    temp_cmdStat.words.status = pSCB->cmdStat.words.status;

    /* check to see if it was the PRO/100 */
    if (temp_cmdStat.words.status & I559_INT)
    {
		/* Wait for command word to clear - indicates no pending commands */
		while (pSCB->cmdStat.words.command)
			;

		/* Update global status variable */
		i559Status = temp_cmdStat.words.status;

		/* If the interrupt was due to a received frame... */
		if (temp_cmdStat.bits.fr)
		    rxFlag = TRUE;

		temp_cmdStat.words.status = temp_cmdStat.words.status & I559_INT;

		/* Acknowledge interrupt by setting ack bits */
		pSCB->cmdStat.words.status = temp_cmdStat.words.status;

		/* Wait for command word to clear - indicates IACK accepted */
		while (pSCB->cmdStat.words.command)
			;

		//printf ("DEBUG: waitSem\n");

		/* Update global status variable and unblock task */
		waitSem = 1;

		if (rxFlag == TRUE)
			rxSem = 1;

		return(1);		/* serviced - return 1 */
	}
    
	return(0);			/* nothing serviced - return 0 */
}

/*******************
	i559Init
********************/
static int i559Init(void)
{
	/* Get memory for system data structures */
	if( ((pCmdBlock = (union cmdBlock *) malloc (sizeof(union cmdBlock))) == NULL) ||
		((pRfd = (struct rfd *) malloc (sizeof(struct rfd))) == NULL) || 
		((pPacketBuf = (char *) malloc (ETHERMTU + sizeof(UINT16) + 18)) == NULL))
	{
		printf ("Memory allocation failed.\n");
		return (ERROR);
	}
	if(verbose)
	{
		printf("pCmdBlock: 0x%x, size: 0x%x\n", pCmdBlock, sizeof(union cmdBlock));
		printf("pRfd: 0x%x, size: 0x%x\n", pRfd, sizeof(struct rfd));
		printf("pPacketBuf: 0x%x, size: 0x%x\n", pPacketBuf, (ETHERMTU + sizeof(UINT16) + 18));
	}

	/* Set EL bits in command block and rfd so we don't fall off the end */
	pCmdBlock->nop.el = END_OF_LIST;
	pRfd->el = END_OF_LIST;

	/* Initialize */
	if(verbose)
		printf ("Initializing... ");

	/* set up the CU and RU base values to 0x0 */
	sendCommand (LOAD_CU_BASE, RU_NOP, 0);
	sendCommand (CU_NOP, LOAD_RU_BASE, 0);
	
	/* Initialize interrupts */
	//To Do EnableIRQ(p->esd8xxISR);

	if(verbose)
		printf ("Done\n");

	return (OK);
}

/*
 * Take the semaphore and block until i557 interrupt or timeout.
 * Returns OK if an interrupt occured, ERROR if a timeout.
 */
static int waitForInt(void)
{
	int num_ms = 0; 

	while ((waitSem == 0) && (num_ms != 2000)) /* wait max 2secs for the interrupt */
	{	
		DelayNUsec(1000);
		num_ms++;
	}

    if (!waitSem)
    {
		printf("Wait error!\n"); 
		return (ERROR);
    }
    else
		return (OK);
}

/*******************
	i559AddrSet
********************/
static int i559AddrSet(void)
{
	printf ("Setting hardware ethernet address to ");
    printf ("%02X:%02X:%02X:", EEPROM_MAC_address[0], EEPROM_MAC_address[1], EEPROM_MAC_address[2]);
    printf ("%02X:%02X:%02X... ", EEPROM_MAC_address[3], EEPROM_MAC_address[4], EEPROM_MAC_address[5]);

	/* Set up iaSetup command block and execute */
    memset((char *) pCmdBlock,0, sizeof(union cmdBlock));

	pCmdBlock->iaSetup.code = IA_SETUP;
	pCmdBlock->iaSetup.el = END_OF_LIST;

    memcpy ((UINT32*)EEPROM_MAC_address, (UINT32*)pCmdBlock->iaSetup.enetAddr, sizeof(EEPROM_MAC_address));

	sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

	if ((waitForInt() == ERROR) || (pCmdBlock->iaSetup.ok != 1))
	{printf("test4\n");
		printf ("failed.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);
		printf ("C bit = %d\n",pCmdBlock->iaSetup.c);
		printf ("OK bit = %d\n",pCmdBlock->iaSetup.ok);
		return (ERROR);
	}
    if(verbose)
		printf ("done.\n");

	return (OK);
}

/*******************
	initPHY
********************/
static int initPHY(UINT32 device_type, int loop_mode)
{
//	UINT16 temp_reg;
	UINT8 revision;

	/* strip off revision and phy. id information */ 
	revision = (UINT8)(device_type & REVISION_MASK);
	device_type &= ~REVISION_MASK;

    switch (device_type)
	{
#ifdef SUPPORT_OTHER_PHY
		case ICS1890_PHY_ID:
			temp_reg = readMDI (0, MDI_DEFAULT_PHY_ADDR, MDI_PHY_CTRL);  // get ready for loopback setting
	    
			switch (loop_mode)
			{
				case EXT_LOOP_BACK:  // loopback on the MII interface
    			temp_reg |= MDI_CTRL_LOOPBACK;	// MII loopback
					break;

				case INT_LOOP_BACK:
				default:
					break;
			}
    	    
			writeMDI(0, MDI_DEFAULT_PHY_ADDR, MDI_PHY_CTRL, temp_reg);
			break;
		
		case DP83840_PHY_ID:  /* set the Intel-specified "must set" bits */
    		temp_reg = readMDI (0,MDI_DEFAULT_PHY_ADDR, DP83840_PCR_REG);
    		temp_reg |= (PCR_TXREADY_SEL | PCR_FCONNECT);
    		writeMDI (0,MDI_DEFAULT_PHY_ADDR, DP83840_PCR_REG, temp_reg);

			// get ready for loopback setting
    		temp_reg = readMDI (0,MDI_DEFAULT_PHY_ADDR, DP83840_LOOPBACK_REG);
    		temp_reg &= CLEAR_LOOP_BITS;

			switch (loop_mode)
			{
				case EXT_LOOP_BACK:
    			temp_reg |= TWISTER_LOOPBACK;
				break;

				case INT_LOOP_BACK:
				default:
				break;
			}

    		writeMDI (0,MDI_DEFAULT_PHY_ADDR, DP83840_LOOPBACK_REG, temp_reg);
			break;
#endif
		case I82553_PHY_ID:
		case I82553_REVAB_PHY_ID:
		case I82555_PHY_ID:
			break;

		default:
			return (ERROR);
			break;
  }

  return (OK);
}

/*******************
	i559Config
********************/
static int i559Config(UINT8 loopBackMode)
{
	printf ("\nConfiguring for ");
	switch (loopBackMode)
	{
		case INT_LOOP_BACK:
			printf ("internal loopback... ");
			break;
		case EXT_LOOP_BACK:
			printf ("external loopback, LPBK* active... ");
			break;
		case NO_LOOP_BACK:
			printf ("no loopback... ");
			break;
		default:
			printf ("Unknown loopback mode, exiting...\n");
			return (ERROR);
	}

	/* Set up configure command block and execute */
    memset((char *) pCmdBlock,0, sizeof(union cmdBlock));

	pCmdBlock->configure.code = CONFIGURE;
	pCmdBlock->configure.el = END_OF_LIST;
	pCmdBlock->configure.configData[ 0] = CONFIG_BYTE_00;
	pCmdBlock->configure.configData[ 1] = CONFIG_BYTE_01;
	pCmdBlock->configure.configData[ 2] = CONFIG_BYTE_02;
	pCmdBlock->configure.configData[ 3] = CONFIG_BYTE_03;
	pCmdBlock->configure.configData[ 4] = CONFIG_BYTE_04;
	pCmdBlock->configure.configData[ 5] = CONFIG_BYTE_05;
	pCmdBlock->configure.configData[ 6] = CONFIG_BYTE_06;
	pCmdBlock->configure.configData[ 7] = CONFIG_BYTE_07;
	pCmdBlock->configure.configData[ 8] = CONFIG_BYTE_08;
	pCmdBlock->configure.configData[ 9] = CONFIG_BYTE_09;
	pCmdBlock->configure.configData[10] = CONFIG_BYTE_10 | loopBackMode;
	pCmdBlock->configure.configData[11] = CONFIG_BYTE_11;
	pCmdBlock->configure.configData[12] = CONFIG_BYTE_12;
	pCmdBlock->configure.configData[13] = CONFIG_BYTE_13;
	pCmdBlock->configure.configData[14] = CONFIG_BYTE_14;
	pCmdBlock->configure.configData[15] = CONFIG_BYTE_15;
	pCmdBlock->configure.configData[16] = CONFIG_BYTE_16;
	pCmdBlock->configure.configData[17] = CONFIG_BYTE_17;
	pCmdBlock->configure.configData[18] = CONFIG_BYTE_18;

	if (link_speed == SPEED_100M) 
		pCmdBlock->configure.configData[19] = CONFIG_BYTE_19_100T;
	else
	{
		pCmdBlock->configure.configData[19] = CONFIG_BYTE_19_10T;
		pCmdBlock->configure.configData[20] = CONFIG_BYTE_20;
		pCmdBlock->configure.configData[21] = CONFIG_BYTE_21;
	}

	sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

	if ((waitForInt() == ERROR) || (pCmdBlock->configure.ok != 1))
	{
		printf ("failed.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);
		return (ERROR);
	}

	initPHY (phy_id, loopBackMode);  /* set up the PHY interface appropriately */

	printf ("done.\n");

	return (OK);
}

/*******************
	i559RUStart
********************/
static int i559RUStart(void)
{
	volatile long delay;

	printf ("DEBUG: Enabling receiver... ");

    memset((char *) pRfd,0, sizeof(struct rfd));

	/* Set end-of-list bit in the rfd so we don't fall off the end */
	pRfd->el = END_OF_LIST;
	pRfd->s = 1;
	pRfd->sf = 0;				/* Simplified mode */
	pRfd->rbdAddr = (UINT8 *) 0xffffffff; 	/* No RBD */
	/* buffer size: */
	pRfd->size = sizeof (pRfd->rxData) + sizeof (pRfd->destAddr) + sizeof (pRfd->sourceAddr) + sizeof (pRfd->length); 

	sendCommand (CU_NOP, RU_START, VIRT_TO_BUS(pRfd));

	/*
	 * Poll, can't use waitForInt (), as this step doesn't generate interrupts.
	 */

	i559Status = 0;

	/* Wait for timeout (i557Status changes) or RU_STAT is RU_READY */
	for (delay = 0; (delay < MAX_DELAY) && (pSCB->cmdStat.bits.rus != RU_READY); delay++)
		;	/* Wait... */

	if (pSCB->cmdStat.bits.rus != RU_READY)
	{
		printf ("failed.  Status: 0x%04X.\n",
		pSCB->cmdStat.words.status);
		return (ERROR);
	}

	printf ("done.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);

	return (OK);
}

/*******************
	setUpPacket
********************/
static void setUpPacket(char *pBuf)
{
	memcpy ((UINT32*)EEPROM_MAC_address, (UINT32*)pBuf, sizeof(EEPROM_MAC_address));

	pBuf += sizeof(EEPROM_MAC_address); /* skip dest. address */

	*((UINT16 *) pBuf) = 0;
	pBuf += sizeof(UINT16);      /* skip length field */
	makePacket ((UINT8*)pBuf, ETHERMTU);
}

/*******************
	waitForRxInt
********************/
static int waitForRxInt(void)
{
	int num_ms = 0; 

	while ((rxSem == 0) && (num_ms != 2000)) /* wait max 2secs for the interrupt */
	{	
		DelayNUsec(1000);
		num_ms++;
	}

    if (!rxSem)
    {
		printf("Rx Wait error!\n"); 
		return (ERROR);
    }
    else
		return (OK);
}

/*
 * Verify contents of a received packet to what was transmitted.
 * Returns OK if they match, ERROR if not.
 */
static int checkPacket (UINT8 *pTxBuffer,		/* Pointer data that was transmitted */
						UINT8 *pRxBuffer,		/* Pointer data that was received */
						int	length)				/* How many bytes to check */
{
int byteNum;	/* Current byte number */

     printf ("\n");
     printf ("Transmit Buffer at 0x%08X\n", pTxBuffer);
     printf ("Receive  Buffer at 0x%08X\n", pRxBuffer);

     for (byteNum = 0; byteNum < length; byteNum++)
     {
		if (*pTxBuffer++ != *pRxBuffer++)
		{
			printf("Error at byte 0x%x\n", byteNum);
			printf("Expected 0x%02X, got 0x%02X\n", *(pTxBuffer - 1), *(pRxBuffer - 1));
            return (ERROR);
		}
     }
     return (OK);
}

/*******************
	txPacket
********************/
static int txPacket(char *pBuf)
{
	int status = OK;
	UINT16 tcbCount;

	/* Set up transmit command block and execute */
    memset((char *) pCmdBlock,0, sizeof(union cmdBlock));

	pCmdBlock->transmit.code = TRANSMIT;
	pCmdBlock->transmit.el = END_OF_LIST;
	pCmdBlock->transmit.sf = 0;				/* Simplified mode */
	pCmdBlock->transmit.tbdAddr = (UINT8 *) 0xffffffff; /* No TBD */
	
	#ifdef __ARMEB__
		pCmdBlock->transmit.eof = 1;			/* Entire frame here */
		/* # bytes to tx: */
		tcbCount =	sizeof (pCmdBlock->transmit.destAddr) +
										sizeof (pCmdBlock->transmit.length) +
										sizeof (pCmdBlock->transmit.txData);
		pCmdBlock->transmit.tcbCount1 = tcbCount & 0xFF;
		pCmdBlock->transmit.tcbCount2 = (tcbCount >> 8) & 0x3F;
	#else
		pCmdBlock->transmit.eof = 1;			/* Entire frame here */
		/* # bytes to tx: */
		pCmdBlock->transmit.tcbCount =	sizeof (pCmdBlock->transmit.destAddr) +
										sizeof (pCmdBlock->transmit.length) +
										sizeof (pCmdBlock->transmit.txData);
	#endif

	//printf ("DEBUG: destAddr size = %d\n", sizeof (pCmdBlock->transmit.destAddr));
	//printf ("DEBUG: length size = %d\n", sizeof (pCmdBlock->transmit.length));
	//printf ("DEBUG: Transmitting %d bytes\n", pCmdBlock->transmit.tcbCount1);
	//printf ("DEBUG: Transmitting %d bytes\n", pCmdBlock->transmit.tcbCount2);

	memcpy ((UINT32*)pBuf, (UINT32*)pCmdBlock->transmit.destAddr, sizeof(EEPROM_MAC_address) + sizeof(UINT16) + ETHERMTU);

	rxSem = 0;	/* no Receive interrupt */
	if((transceiver == TX) || (loopback_test_flag))
	{
		sendCommand (CU_START, RU_NOP, VIRT_TO_BUS(pCmdBlock));

		if (waitForInt() == ERROR)
		{
			printf ("No Transmit Interrupt\n");
			status = ERROR;
		}

		if (pCmdBlock->transmit.ok != 1)
		{	
			printf ("tx failed.  Status: 0x%04X.\n",
			pSCB->cmdStat.words.status);
			status = ERROR;
		}

		if (status == ERROR)
		{
			printf ("Transmit OK = %d\n", pCmdBlock->transmit.ok);
			return (ERROR);
		}
	}

	if((transceiver == RX) || (loopback_test_flag))
	{
		if (waitForRxInt() == ERROR)
		{
			printf ("No Receive Interrupt\n");
			status = ERROR;
		}

		if (pRfd->ok != 1)
		{
			printf ("rx failed.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);
			status = ERROR;
		}

		/* If RU still ready, hang for receive interrupt */
		if (pSCB->cmdStat.bits.rus == RU_READY)
		{
			if (waitForRxInt() == ERROR)
			{
			    printf ("No Receive Interrupt\n");
				status = ERROR;
			}

			if (pRfd->ok != 1)
			{
				printf ("rx failed.  Status: 0x%04X.\n", pSCB->cmdStat.words.status);
				status = ERROR;
			}
		}
	}

	if (status == ERROR)
	{
		printf ("\nTransmit Stats:\n");
		printf ("---------------\n");
		printf ("Transmit OK = %d\n", pCmdBlock->transmit.ok);

		printf ("\nReceive Stats:\n");
		printf ("---------------\n\n");
		printf ("Receive OK = %d\n", pRfd->ok);
		printf ("CRC Error = %d\n", pRfd->crcErr);
		printf ("Alignment Error = %d\n", pRfd->alignErr);
		printf ("Resource Error = %d\n", pRfd->noRsrc);
		printf ("DMA Overrun Error = %d\n", pRfd->dmaOverrun);
		printf ("Frame Too Short Error = %d\n", pRfd->frameTooshort);
		printf ("Receive Collision Error = %d\n", pRfd->rxColl);
		return (ERROR);
	}

	printf ("Packet Actual Size = %d\n", pRfd->actCount);

	if((transceiver == RX) || (loopback_test_flag))
	{
		if (checkPacket (pCmdBlock->transmit.txData, pRfd->rxData, ETHERMTU) == ERROR)
		{
			printf ("data verify error.\n");
			return (ERROR);
		}
    
		if (verbose)
			printf ("data OK.\n");
	}
	return (OK);
}

/*******************
	readMDI
********************/
static UINT16 readMDI(int unit, UINT8 phyAdd, UINT8 regAdd)//, UINT16 data)
{
	register MDI_CONTROL_U mdiCtrl;
	int num_ms = 0; 

	/* prepare for the MDI operation */
	mdiCtrl.bits.ready = MDI_NOT_READY;
	mdiCtrl.bits.intEnab = MDI_POLLED;	/* no interrupts */
	mdiCtrl.bits.op = MDI_READ_OP;
	mdiCtrl.bits.phyAdd = phyAdd & 0x1f;
	mdiCtrl.bits.regAdd = regAdd & 0x1f;

	/* start the operation */
	#ifdef __ARMEB__
		*MDI_CTL_REG(adapter) = SWAP32(mdiCtrl.word);
		
	#else
		*MDI_CTL_REG(adapter) = mdiCtrl.word;
	#endif
	/* delay a bit */
	DelayNUsec(1000);
	/* poll for completion */
	#ifdef __ARMEB__
		mdiCtrl.word = SWAP32(*MDI_CTL_REG(adapter));
	#else
		mdiCtrl.word = *MDI_CTL_REG(adapter);
	#endif
	while ((mdiCtrl.bits.ready == MDI_NOT_READY) && (num_ms != 2000)) /* wait max 2secs */
 	{	
		#ifdef __ARMEB__
			mdiCtrl.word = SWAP32(*MDI_CTL_REG(adapter));
		#else
			mdiCtrl.word = *MDI_CTL_REG(adapter);
		#endif
		DelayNUsec(1000);
		num_ms++;
	}	

	if (num_ms >= 2000)
	{
		printf ("readMDI Timeout!\n");
		return (0);
	}
	else 
	{
		return ((UINT16)mdiCtrl.bits.data);
	}
}

#ifdef SUPPORT_OTHER_PHY
/*******************
	writeMDI
********************/
void writeMDI (int unit, UINT8 phyAdd, UINT8 regAdd, UINT16 data)
{
	register MDI_CONTROL_U mdiCtrl;
	int num_ms = 0;


	/* prepare for the MDI operation */
	mdiCtrl.bits.ready = MDI_NOT_READY;
	mdiCtrl.bits.intEnab = MDI_POLLED;  /* no interrupts */
	mdiCtrl.bits.op = MDI_WRITE_OP;
	mdiCtrl.bits.phyAdd = phyAdd & 0x1f;
	mdiCtrl.bits.regAdd = regAdd & 0x1f;
	mdiCtrl.bits.data = data & 0xffff;

	/* start the operation */
	*MDI_CTL_REG(adapter) = mdiCtrl.word;

	/* delay a bit */
	DelayNUsec(1000);

	/* poll for completion */
	mdiCtrl.word = *MDI_CTL_REG(adapter);

	while ((mdiCtrl.bits.ready == MDI_NOT_READY) && (num_ms != 2000)) 
	{
		mdiCtrl.word = *MDI_CTL_REG(adapter);
		DelayNUsec(1000);
		num_ms++;
	}	
	if (num_ms >= 2000)
		printf ("writeMDI Timeout!\n");
 
	return;
}
#endif				

int GetEepromSize(long ioaddr)
{
    unsigned short retval = 0;
    int ee_addr = ioaddr + SCBeeprom;
    int i, addrbits;


	printf("DEBUG: ioaddr = %x\n",ioaddr);

    // Should already be not-selected, but do anyway:
    PUT8(ee_addr, (char) (EEPROM_ENB & ~EEPROM_CS));
    DelayNUsec(10);
    PUT8(ee_addr, (char) EEPROM_ENB);
    DelayNUsec(10);
    // Shift the read command bits out.
    for (i = 2; i >= 0; i--)
	{
        short dataval = (6 & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT8(ee_addr, EEPROM_ENB | dataval);
        DelayNUsec(2);
        PUT8(ee_addr, EEPROM_ENB | dataval | EEPROM_SHIFT_CLK);
        DelayNUsec(2);
        PUT8(ee_addr, EEPROM_ENB | dataval);
        DelayNUsec(2);
    }
    // Now clock out address zero, looking for the dummy 0 data bit
    for ( i = 1; i <= 12; i++ )
	{
        PUT8(ee_addr, (char) EEPROM_ENB);
        DelayNUsec(2);
        PUT8(ee_addr, (char) (EEPROM_ENB | EEPROM_SHIFT_CLK));
        DelayNUsec(2);
        PUT8(ee_addr, (char) EEPROM_ENB);
        DelayNUsec(2);
        retval = GET8(ee_addr) & EEPROM_DATA_READ;
        if ( 0 == retval )
        {
            break; // The dummy zero est arrive'
        }
    }

    addrbits = i;
    if ( 6 != i && 8 != i )
	{
		printf("eeprom not found addrbits =%x\n", addrbits);

        i = 6;
        addrbits = 0;  // So caller can know if the EEPROM is missing
    }

    // clear the dataval, leave the clock low to read in the data regardless
    PUT8(ee_addr, (char) EEPROM_ENB);
    DelayNUsec(1);
    
    retval = GET8(ee_addr);
    if ( (EEPROM_DATA_READ & retval) != 0 )
	{
    }
    DelayNUsec(1);
    
    for (i = 15; i >= 0; i--)
	{
        PUT8(ee_addr, (char) (EEPROM_ENB | EEPROM_SHIFT_CLK));
        DelayNUsec(2);
        retval = GET8(ee_addr);
        DelayNUsec(2);
        PUT8(ee_addr, (char) EEPROM_ENB);
        DelayNUsec(2);
    }
    
    // Terminate the EEPROM access.
    PUT8(ee_addr, (char) (EEPROM_ENB & ~EEPROM_CS));
    DelayNUsec(100);
    
    return addrbits;
}

int Eeprom_Read(long ioaddr, int location, int addr_len)
{
    unsigned short retval = 0;
    int ee_addr = ioaddr + SCBeeprom;
    int read_cmd = location | (6 << addr_len);
    int i, tries = 10;

 	while(1)
	{
		// Should already be not-selected, but anyway:
		PUT8(ee_addr, (char) (EEPROM_ENB & ~EEPROM_CS));
		DelayNUsec(10);
		PUT8(ee_addr, (char) EEPROM_ENB);
		DelayNUsec(10);

		// Shift the read command bits out, changing only one bit per time.
		for (i = addr_len + 2; i >= 0; i--)
		{
		    short dataval = (read_cmd & (1 << i)) ? EEPROM_DATA_WRITE : 0;
		    PUT8(ee_addr, (char) (EEPROM_ENB | dataval));
		    DelayNUsec(2);
		    PUT8(ee_addr, (char) (EEPROM_ENB | dataval | EEPROM_SHIFT_CLK));
		    DelayNUsec(2);
		    PUT8(ee_addr, (char) (EEPROM_ENB | dataval));
		    DelayNUsec(2);
		}

		// clear the dataval, leave the clock low
		PUT8(ee_addr, (char) (EEPROM_ENB));
		DelayNUsec(1);

		retval = GET8(ee_addr);
		// This should show a zero in the data read bit to confirm that the
		// address transfer is compelete.  If not, go to the start and try
		// again!
		if ( (0 != (retval & EEPROM_DATA_READ)) && (tries-- > 0) )
		{
		// Terminate the EEPROM access.
		    PUT8(ee_addr, (char) EEPROM_ENB & ~EEPROM_CS);
		    DelayNUsec(100);
		}
		else
			break;
	}

    if ( (EEPROM_DATA_READ & retval) != 0 )
	{
    }
    DelayNUsec(1);
    retval = 0;

    for (i = 15; i >= 0; i--)
	{
        PUT8(ee_addr, (char) (EEPROM_ENB | EEPROM_SHIFT_CLK));
        DelayNUsec(2);
        retval = (retval << 1) | ((GET8(ee_addr) & EEPROM_DATA_READ) ? 1 : 0);
        DelayNUsec(2);
        PUT8(ee_addr, (char) EEPROM_ENB);
        DelayNUsec(2);
    }
    
    // Terminate the EEPROM access.
    PUT8(ee_addr, (char) (EEPROM_ENB & ~EEPROM_CS));
    DelayNUsec(100);
    
    return retval;
}


void Get_Eeprom_Mac_Addr(void)
{
	int addr_length = 8, i, count;
	UINT32 ioaddr;
	UINT16 checksum, value;								  
	int prom_size;

	//ioaddr = nic_basemap[1];
	ioaddr = nic_basemap[0];

	// read eeprom and get 82559's mac address

	// (this is the length of the *EEPROM*s address, not MAC address)
    addr_length = GetEepromSize(ioaddr);

    switch (addr_length)
    {
	case 6: prom_size = 64; break;
	case 7: prom_size = 128; break;
	case 8: prom_size = 256; break;
    }


	printf("Debug only addr_length = %x\n", addr_length);
	printf("Debug only prom_size = %d\n", prom_size);

    for (checksum = 0, i = 0, count = 0; count < prom_size; count++)
	{
        
        // read word from eeprom
        value = Eeprom_Read(ioaddr, count, addr_length);

        checksum += value;
        if (count < 3)
		{
            EEPROM_MAC_address[i++] = value & 0xFF;
            EEPROM_MAC_address[i++] = (value >> 8) & 0xFF;
        }
	}
	printf("EEPROM MAC Address = ");
	printf("%02X:%02X:%02X:%02X:%02X:%02X\n", EEPROM_MAC_address[0],\
	 	EEPROM_MAC_address[1], EEPROM_MAC_address[2], EEPROM_MAC_address[3],
	  	EEPROM_MAC_address[4],EEPROM_MAC_address[5]);

	printf("\nRead checksum = %04x\n",checksum);



}

void eraseEeprom (UINT32 ioaddr, int reg, int addr_len)
{
    int i,j;
    UINT16 data;

    int ee_addr = ioaddr + SCBeeprom;

    PUT8(ee_addr, EEPROM_CS);

    for (i=EE_CMD_BITS -1; i >=0; i--) {
        data = (EE_CMD_ERASE & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }
    for (i=addr_len - 1; i >=0; i--) {
	data = (reg & (1 << i)) ? EEPROM_DATA_WRITE : 0;
  	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }

    data = GET8(ee_addr);
    data &= ~(EEPROM_CS|EEPROM_SHIFT_CLK);
    PUT8(ee_addr, data);
    data |= EEPROM_CS;
    PUT8(ee_addr, data);
    while (!(data = GET8(ee_addr)) & EEPROM_DATA_READ)
	DelayNUsec(1000);

    PUT8(ee_addr, 0);
	DelayNUsec(1000);
}

void writeEeprom (UINT32 ioaddr, int reg, UINT16 val, int addr_len)
{
    int i,j;
    UINT16 data;

    int ee_addr = ioaddr + SCBeeprom;
	
	unlockEeprom(ioaddr,reg,addr_len);
	eraseEeprom(ioaddr,reg,addr_len);

    PUT8(ee_addr, EEPROM_CS);

    for (i=EE_CMD_BITS -1; i >=0; i--) {
        data = (EE_CMD_WRITE & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT8(ee_addr, EEPROM_CS | data);
 		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data);
		DelayNUsec(1000);
    }

    for (i=addr_len - 1; i >=0; i--) {
	data = (reg & (1 << i)) ? EEPROM_DATA_WRITE : 0;
  	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }

    for (i=EE_DATA_BITS - 1;i >=0; i--) {
 	data = (val & (1 << i)) ? EEPROM_DATA_WRITE : 0;
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }
    data = GET8(ee_addr);
    data &= ~(EEPROM_CS|EEPROM_SHIFT_CLK);
    PUT8(ee_addr, data);
    data |= EEPROM_CS;
    PUT8(ee_addr, data);
    while (!(data = GET8(ee_addr)) & EEPROM_DATA_WRITE)
	DelayNUsec(1000);

    PUT8(ee_addr, 0);

	lockEeprom(ioaddr,reg,addr_len);
}

void lockEeprom (UINT32 ioaddr, int reg, int addr_len)
{
    int i,j;
    UINT16 data;

    int ee_addr = ioaddr + SCBeeprom;
    
    for (i=4; i >=0; i--) {
        data = (EE_CMD_EWDS & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT8(ee_addr, EEPROM_CS | data);
		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data);
		DelayNUsec(1000);
    }
    for (i=addr_len - 3; i >=0; i--) {
	data = (reg & (1 << i)) ? EEPROM_DATA_READ : 0;
  	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }
    PUT8(ee_addr, 0);
	DelayNUsec(1000);

}

void unlockEeprom(UINT32 ioaddr, int reg, int addr_len)
{
    int i,j;
    UINT16 data;
	
    int ee_addr = ioaddr + SCBeeprom;

    PUT8(ee_addr, EEPROM_CS);

    for (i=4; i >=0; i--)
     {
        data = (EE_CMD_EWEN & (1 << i)) ? EEPROM_DATA_WRITE : 0;
        PUT8(ee_addr, EEPROM_CS | data);
		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
		DelayNUsec(1000);
        PUT8(ee_addr, EEPROM_CS | data);
		DelayNUsec(1000);
    }
    for (i=addr_len - 3; i >=0; i--) {
	data = (reg & (1 << i)) ? EEPROM_DATA_WRITE : 0;
  	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data | EEPROM_SHIFT_CLK);
	DelayNUsec(1000);
	PUT8(ee_addr, EEPROM_CS | data);
	DelayNUsec(1000);
    }
    PUT8(ee_addr, 0);
	DelayNUsec(1000);

}




// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_ETHER,"nic",Ethernet_test,DIAGNOSTIC,"EthernetTest","runs network controller link, loopback, or self tests");
Arg(UINT32ARG, "select NIC device (0 or 1)", "select which NIC to test (0 or 1)");
Switch("l", BOOLEAN32ARG, "Ethernet NIC Link Test", "");
Switch("i", BOOLEAN32ARG, "Ethernet NIC Internal Loopback Test", "");
Switch("e", BOOLEAN32ARG, "Ethernet NIC External Loopback Test", "");
Switch("s", BOOLEAN32ARG, "Ethernet NIC Self Test", "");
Switch("a", BOOLEAN32ARG, "All Ethernet NIC Tests", "");
CommandEnd();


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_GETENET,"getenet",find_ethernet_address,DIAGNOSTIC,"get ethernet address","display ethernet address");
Arg(UINT32ARG, "select NIC device (0 or 1)", "select which NIC to read (0 or 1)");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_SETENET,"setenet",set_ether_addr,DIAGNOSTIC,"set ethernet address","set ethernet addr(Format: setenet NIC#(0 or 1) AB CD EF GH IJ KL)");
Arg(UINT32ARG, "select NIC device (0 or 1)", "select which NIC to write (0 or 1)");
Arg(UINT8ARG,"enet address bits AB", "");
Arg(UINT8ARG,"enet address bits CD", "");
Arg(UINT8ARG,"enet address bits EF", "");
Arg(UINT8ARG,"enet address bits GH", "");
Arg(UINT8ARG,"enet address bits IJ", "");
Arg(UINT8ARG,"enet address bits KL", "");
CommandEnd();


