/* factory_wtest.uc
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *																					 
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  ported: ajp, August '03
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

// qdr0_mxsx.uc
//
// Code to characterize the QDR interface on channel 0.
// Includes burst writes and burst reads of
//   - checkerboard pattern
//   - walking one
//   - walking zero
//   - pseudo-random data sequences
//
// The data returned to uEngine via QDR read is written
// out on PCI for verification.
//
// Modifications for qdr0123_m8s8_auto.uc:
// - all 4 SRAM channels remain in active (non-low-power) mode
// - all channels do RCOMP/SCOMP activities
// - only channel 0 proceeds with work

// start reg-map
// km_special -> reg_b0
// five_a -> reg_b1
// a_five -> reg_b2
// zero_f -> reg_b3
// f_zero -> reg_b4
// walk_one -> reg_b5
// walk_zero -> reg_b6
// offset -> reg_b7
// base0 -> reg_b10
// base1 -> reg_b11
// base2 -> reg_b12
// base3 -> reg_b13
// rcomp_base -> reg_b14
// scomp_base -> reg_b15
// counter1 -> reg_a0
// counter2 -> reg_a1
// csr_val -> reg_a2
// temp_adr -> reg_a3
// this_base -> reg_a8
// end reg-map

.num_contexts 4

// All transfer registers are in order defined here:
.sig SIG2 SIG3
.reg offset this_base a_five five_a zero_f f_zer0 km_special base0 base1 base2 base3 rcomp_base scomp_base
.reg counter1 counter2 csr_val temp_adr f_zero  
.reg $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 $xfer8 $xfer9 $xfer10 $xfer11 $xfer12 $xfer13 $xfer14 $xfer15 $xfer16 $xfer17 $xfer18 $xfer19 $xfer20 $xfer21 $xfer22 $xfer23 $xfer24 $xfer25 $xfer26 $xfer27 $xfer28 $xfer29 $xfer30 $xfer31
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 $xfer8 $xfer9 $xfer10 $xfer11 $xfer12 $xfer13 $xfer14 $xfer15 $xfer16 $xfer17 $xfer18 $xfer19 $xfer20 $xfer21 $xfer22 $xfer23 $xfer24 $xfer25 $xfer26 $xfer27 $xfer28 $xfer29 $xfer30 $xfer31
; <NUM_CTX_USED=4>
; <CODE_START>

; <NO_MEISA>
ctx_CTX_A_start#:

;;*************************************************************************
;;      do_rcomp_scomp_init
;;      this does rcomp and scomp csr inits
;;      argument is the per-channel most-significant bits (msbs)
;;*************************************************************************
#macro do_rcomp_scomp_init(msbs)

   immed[rcomp_base,0x0300]
   immed_w1[rcomp_base, msbs]
   immed[scomp_base,0x0340]
   immed_w1[scomp_base, msbs]

   // Write the RCMP_STRENGTH_SLEW_INDEX_SEL to 1.0
   //
   immed[csr_val,0x5ad6]
   immed_w1[csr_val,0x000b]
   alu[$xfer0, --, B, csr_val]
   sram[csr_wr,$xfer0,rcomp_base,0x18],sig_done[SIG3]

rcomp_strength#:
   br_!signal[SIG3,rcomp_strength#]

   // Write the slew tables.
   //
   immed[csr_val,0x3333]
   immed_w1[csr_val,0x3333]
   alu[$xfer0, --, B, csr_val]
   immed[counter1,0x007c]
   immed_w1[counter1,0x0000]

scomp_loop#:
   sram[csr_wr,$xfer0,scomp_base,counter1]
   alu[counter1, counter1, -, 0x0004]
   bne [scomp_loop#]

   sram[csr_wr,$xfer0,scomp_base,counter1],sig_done[SIG3]
#endm

;;*************************************************************************

   // Seed each of the pseudo random number generator and datapath
   // signature CSRs.
   // 
   local_csr_wr[Pseudo_Random_Number, 0x69]
   local_csr_wr[Datapath_Signature, 0xff]

   // Place holders for base addresses of the SRAM channel 0-3
   // CSR address space.  Add 0x200 to the true base addresses
   // since the src_op2 field of the sram command can't handle
   // more than 7 bits.
   //
   immed[base0,0x0200]
   immed_w1[base0,0x0000]
   immed[base1,0x0200]
   immed_w1[base1,0x4000]
   immed[base2,0x0200]
   immed_w1[base2,0x8000]
   immed[base3,0x0200]
   immed_w1[base3,0xc000]

   // Write the RX_DESKEW register for channel 0.
   //
   immed[counter1,0x0000]
   immed_w1[counter1,0x0000]
   
deskew_write#:
   alu[counter1, counter1, +, 0x0001]
   alu[$xfer0, --, B, counter1]  
   sram[csr_wr,$xfer0,base0,0x44],sig_done[SIG3]

rx_deskew#:
   br_!signal[SIG3,rx_deskew#]

   // Delay for 1000 SCLK (= 4000 TCLK @AG4) between
   // increments of the value.
   // Each loop takes 5 TCLK so need >800 iterations -- use 1024.
   //
   immed[counter2,0x0400]
   immed_w1[counter2,0x0000]
deskew_delay#:
   alu[counter2, counter2, -, 0x0001]
   bne [deskew_delay#]
   alu[--, counter1, -, 0x0008]
   bne [deskew_write#]

   // Write the RX_DLL register for channel 0.
   //
   immed[csr_val,0x0008]
   immed_w1[csr_val,0x0000]
   alu[$xfer0, --, B, csr_val]
   sram[csr_wr,$xfer0,base0,0x28],sig_done[SIG3]

rx_dll#:
   br_!signal[SIG3,rx_dll#]

//noRComp don't need to do this upstairs
//noRComp   do_rcomp_scomp_init(0x0000)
//noRComp   do_rcomp_scomp_init(0x4000)
//noRComp   do_rcomp_scomp_init(0x8000)
//noRComp   do_rcomp_scomp_init(0xc000)

//REMOVED SETUP CONTROL CSR WRITE LOOP
//
//

//noRComp   // Delay for 1000 SCLK (= 4000 TCLK @AG4) after the last
//noRComp   // forced RCOMP update.
//noRComp   //
//noRComp   immed[counter2,0x1000]
//noRComp   immed_w1[counter2,0x0000]
//noRCompoffset_delay#:
//noRComp   alu[counter2, counter2, -, 0x0001]
//noRComp   bne [offset_delay#]

   // Write the RD_PTR_OFFSET register for each channel.
   immed[csr_val,0x0000]
   immed_w1[csr_val,0x0000]
   alu[$xfer0, --, B, csr_val]
   sram[csr_wr,$xfer0,base0,0x40]
   sram[csr_wr,$xfer0,base1,0x40]
   sram[csr_wr,$xfer0,base2,0x40]
   sram[csr_wr,$xfer0,base3,0x40],sig_done[SIG3]

rd_ptr_offset#:
   br_!signal[SIG3,rd_ptr_offset#]

   // Add 1 uS of latency to allow side-effect
   // reset of rd_ptr_offset writing to propagate.
   // THIS IS REQUIRED! Otherwise, Xs appear
   // on the PCI bus
   immed[counter1, 0x00C8]
   immed_w1[counter1,0x0000]
latency_target#:
   alu[counter1, counter1, -, 0x0001]
   bne [latency_target#]


//REMOVED MEASURED PMOS/NMOS to eliminate mis-compare on tester
//read the PMOS and NMOS_MEASURED csr's and write the results to PCI
//	immed[temp_adr, 0x304]
//	sram[csr_rd,$xfer2,temp_adr,0x00],sig_done[SIG3]
//first_read#:
//	br_!signal[SIG3,first_read#]

//	immed[temp_adr, 0x308]
//	sram[csr_rd,$xfer3,temp_adr,0x00],sig_done[SIG3]
//second_read#:
//	br_!signal[SIG3,second_read#]

// copy results from read transfer to write transfer registers:
//	alu[$xfer2, --, b, $xfer2]
//	alu[$xfer3, --, b, $xfer3]
// write to pci (copied from your code with transfer count changed to 2 to write two registers.
//	pci[write,$xfer2,0x0,offset,2],ctx_swap[SIG2]


        immed[offset,0x0]
        immed[this_base,0x0000]
        immed_w1[this_base,0x0000]
        br [start_test#]
;<END_CONTEXT>

; <NO_MEISA>
ctx_CTX_B_start#:
        immed[offset,0x0080]
        immed[this_base,0x0000]
        immed_w1[this_base,0x4000]
        br [start_test#]
;<END_CONTEXT>

; <NO_MEISA>
ctx_CTX_C_start#:
        immed[offset,0x0100]
        immed[this_base,0x0000]
        immed_w1[this_base,0x8000]
        br [start_test#]
;<END_CONTEXT>

; <NO_MEISA>
ctx_CTX_D_start#:
        immed[offset,0x0180]
        immed[this_base,0x0000]
        immed_w1[this_base,0xc000]
        br [start_test#]

start_test#:
// Need to initialize these regs every ctx due to ctx relative approach.
// ckbd,ichkb is GPRB 1,2; 17,18, etc.
   immed[a_five,0x5555]
   immed_w1[a_five,0xaaaa]

   immed[five_a,0xaaaa]
   immed_w1[five_a,0x5555]

   immed[zero_f,0xffff]
   immed_w1[zero_f,0x0000]

   immed[f_zero,0x0000]
   immed_w1[f_zero,0xffff]

   immed[km_special,0x8463]
   immed_w1[km_special,0xf00d]

//   immed[walk_zero,0xffff]
//   immed_w1[walk_zero,0xfffe]

//   immed[walk_one,0x0000]
//   immed_w1[walk_one,0x0001]


   // do the following 200 times
   immed[counter1,0x00c8]
   immed_w1[counter1,0x0000]
work_loop#:

// Write to SRAM Xfer out Registers
   alu[$xfer0, --, B, a_five]
   alu[$xfer1, --, B, a_five]
   alu[$xfer2, --, B, five_a]
   alu[$xfer3, --, B, five_a]
   alu[$xfer4, --, B, zero_f]
   alu[$xfer5, --, B, zero_f]
   alu[$xfer6, --, B, f_zero]
   alu[$xfer7, --, B, f_zero]
   alu[$xfer8, --, B, five_a]
   alu[$xfer9, --, B, zero_f]
   alu[$xfer10, --, B, a_five]
   alu[$xfer11, --, B, km_special]
   alu[$xfer12, --, B, f_zero]
   alu[$xfer13, --, B, km_special]
   alu[$xfer14, --, B, five_a]
   alu[$xfer15, --, B, km_special]

// Get a value of ALU output allowing indirect
   alu_shf[--,--,B,0x1F,<<21]
   
// Write SRAM using indirect. Just specify the first reg.
// above shift statement gets you a length of 16
   sram[write,$xfer0,this_base,offset,7],ctx_swap[SIG2],indirect_ref
   nop
   
// Get it back into sram transfer in RF
   alu_shf[--,--,B,0x1F,<<21]
   sram[read,$xfer0,this_base,offset,7],ctx_swap[SIG2],indirect_ref
   nop

   alu[$xfer0,--,B,$xfer0]
   alu[$xfer1,--,B,$xfer1]
   alu[$xfer2,--,B,$xfer2]
   alu[$xfer3,--,B,$xfer3]
   alu[$xfer4,--,B,$xfer4]
   alu[$xfer5,--,B,$xfer5]
   alu[$xfer6,--,B,$xfer6]
   alu[$xfer7,--,B,$xfer7]																				 
   alu[$xfer8,--,B,$xfer8]
   alu[$xfer9,--,B,$xfer9]
   alu[$xfer10,--,B,$xfer10]
   alu[$xfer11,--,B,$xfer11]
   alu[$xfer12,--,B,$xfer12]
   alu[$xfer13,--,B,$xfer13]
   alu[$xfer14,--,B,$xfer14]
   alu[$xfer15,--,B,$xfer15]
    
//noPCI   // PCI writes validates SRAM XFER ins and SRAM XFER outs
//noPCI   pci[write,$xfer0,0x0,offset,8]
//noPCI   pci[write,$xfer8,0x0,offset,8],ctx_swap[SIG2]               

   // remainder of test skipped

   // go around the work loop again
   alu[counter1, counter1, -, 0x0001]
//   bne [work_loop#]
   br [work_loop#]

br[test_passed#]
;<END_CONTEXT>

test_failed#:
ctx_arb[kill]
nop
nop
nop

test_passed#:
ctx_arb[kill]
