/* gpio.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Jan 21, 02
 *  revisions:
 * 
 01b, 08/20/03 ajp Force to all tests if none are selected
 01a, 08/20/03 ajp Added pass/fail returns for script file break on error
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2000Gpio.h"
#include "ixp2x00.h"
#include "i2c_test.h"
#include "diag_utils.h"
#include "i2c.h"
#include "led.h"

// static function prototypes
static void GPIO_Help(void);
static void GPIO_Detection_Test(void);
static int GPIO_Level_Detect_Test(void);
static int GPIO_Edge_Detect_Test(void);
static int GPIO_Reg_Test(void);


	typedef struct _GPIO_testIn
	{
	UINT32 detec_sel;
	UINT32 register_sel;
	UINT32 edge_sel;
	UINT32 level_sel;
	UINT32 all_sel;
 	} GPIO_testIn;

#define EDGE_LEVEL_GPIO_TO_TEST	(1 << 6)

//
// Function: GPIO_Test
//
// Description: Calls the appropriate function(s) of the GPIO test
//				depending on user input.
//

//
// Function: GPIO_Test
//
// Description: Calls the appropriate function(s) of the GPIO test
//				depending on user input.
//
int GPIO_Test(GPIO_testIn * pIn, PassFail_testOut* pOut)
{
	char opt;
	int passreg = 0, passedge = 0, passlevel = 0;

	if ((!pIn->detec_sel) && (!pIn->all_sel) && (!pIn->register_sel) && (!pIn->edge_sel))
		pIn->all_sel = 1;

	if ((pIn->detec_sel) || (pIn->all_sel))
	GPIO_Detection_Test();

	if ((pIn->register_sel) || (pIn->all_sel))
	passreg = GPIO_Reg_Test();
		
	if ((pIn->edge_sel) || (pIn->all_sel))	
 		if(!IS_EGRESS_NPU)
		  	{
		  	printf("GPIO Edge Detect Test is available");
		  	printf(" on the Egress NPU only\n");
			return;
			}
		else
		passedge = GPIO_Edge_Detect_Test();

	if ((pIn->level_sel) || (pIn->all_sel))	
 		if(!IS_EGRESS_NPU)
		  	{
		  	printf("GPIO Level Detect Test is available");
		  	printf(" on the Egress NPU only\n");
			return;
			}
		else
		passlevel = GPIO_Level_Detect_Test();

	pOut->ReturnCode = PASS;         
        
	pOut->PassFail = (passreg | passedge | passlevel);

	return(pOut->PassFail);

}

//
// Function: GPIO_Detection_Test
//
// Description: Reads and displays the current pin direction, level,
//				edge detection, level sensitivity, and interrupt of the
//				GPIO pins
//
static void GPIO_Detection_Test(void)
{
	int temp;

	temp = GET32(GPIO_PDPR);
	printf("GPIO Registers:\n");
	printf("  Pin Direction: 0x%02x\n", temp);

	temp = GET32(GPIO_PLR);
	printf("  Pin Level: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as low and '1' as high

	temp = GET32(GPIO_REDR);
	printf("  Rising Edge Detect Enable: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as disable and '1' as enable

	temp = GET32(GPIO_FEDR);
	printf("  Falling Edge Detect Enable: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as disable and '1' as enable

	temp = GET32(GPIO_LSHR);
	printf("  Level Sensitive High Detect Enable: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as disable and '1' as enable

	temp = GET32(GPIO_LSLR);
	printf("  Level Sensitive Low Detect Enable: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as disable and '1' as enable

	temp = GET32(GPIO_INER);
	printf("  Interrupt Enable: 0x%02x\n", temp);
	// If the above does not work then use similar alt method as PDPR
	// but with '0' as disable and '1' as enable
}

//
// Function: GPIO_Reg_Test
//
// Description: Display and stores the existing GPIO register
//				values. Then, check the capability to pull all
//				the register values to a HIGH by setting and
//				reading back the register values. Finally, 
//				restore the registers back to their original values.
//
static int GPIO_Reg_Test(void)
{
	int PDPR, POPR, PLR, temp, status = OK;

	printf("GPIO Register Test started... \n");

	// Save existing register values of PDPR and POPR
	PDPR = GET32(GPIO_PDPR);
	POPR = GET32(GPIO_POPR);
	PLR = GET32(GPIO_PLR);

	// Display the current values of PDPR, POPR, and PLR
	printf("PDPR: 0x%02x\n", PDPR);
	printf("POPR: 0x%02x\n", POPR);
	printf("PLR: 0x%02x\n", PLR);

	// Set pins to output by setting bits[7:0] in GPIO_PDSR
	PUT32(GPIO_PDSR, 0xFF);
	// Read GPIO_PDPR to ensure pin directions are correct
	// are configured to output
	temp = GET32(GPIO_PDPR);
	if((temp & 0xFF) != 0xFF)
	{
		printf("Setting of GPIO_PDSR FAILED. GPIO_PDPR = %02x\n", temp);
		status = ERROR;
	}

	// Clear pins to input by setting bits[7:0] in GPIO_PDCR
	PUT32(GPIO_PDCR, 0xFF);
	// Read GPIO_PDPR and GPIO_PLR to ensure pins
	// are configured to input
	temp = GET32(GPIO_PDPR);
	if((temp & 0xFF) != 0x0)
	{
		printf("Clearing of GPIO_PDCR FAILED. GPIO_PDPR = %02x\n", temp);
		status = ERROR;
	}

	// Set pins to output by setting bits[7:0] in GPIO_PDPR
	PUT32(GPIO_PDPR, 0xFF);
	// Read GPIO_PDPR and GPIO_PLR to ensure pins
	// are configured to output
	temp = GET32(GPIO_PDPR);
	if((temp & 0xFF) != 0xFF)
	{
		printf("Setting of GPIO_PDPR FAILED. GPIO_PDPR = %02x\n", temp);
		status = ERROR;
	}

	// Set pins to level high by setting bits[7:0] in GPIO_POSR
	PUT32(GPIO_POSR, 0xFF);
	// Read GPIO_POPR to ensure pins are configured to level high
	temp = GET32(GPIO_POPR);
	if((temp & 0xFF) != 0xFF)
	{
		printf("Setting of GPIO_POSR FAILED\n. GPIO_POPR = %02x\n", temp);
		status = ERROR;
	}

	// Clear pins to level low by setting bits[7:0] in GPIO_POCR
	PUT32(GPIO_POCR, 0xFF);
	// Read GPIO_POPR to ensure pins are configured to level low
	temp = GET32(GPIO_POPR);
	if((temp & 0xFF) != 0x0)
	{
		printf("Clearing of GPIO_POCR FAILED\n. GPIO_POPR = %02x\n", temp);
		status = ERROR;
	}

	// Set pins to level high by setting bits[7:0] in GPIO_POPR
	PUT32(GPIO_POPR, 0xFF);
	temp = GET32(GPIO_POPR);
	if((temp & 0xFF) != 0xFF)
	{
		printf("Setting of GPIO_POPR FAILED\n. GPIO_POPR = %02x\n", temp);
		status = ERROR;
	}

	// Restore POPR and PDPR registers
	PUT32(GPIO_PDPR, PDPR);
	PUT32(GPIO_POPR, POPR);

	
	printf("GPIO Register Test... ");
	if(status == OK)
	{
		printf("PASS\n");
		return(PASS);
	}
	else
	{
		printf("FAIL\n");
		return(FAIL);
	}
}

//
// Function: GPIO_Edge_Detect_Test
//
// Description: Set GPIO for rising edge detection and calls
//				i2c_read to trigger a rising edge. Checks for
//				rising edge detection.
//				Set GPIO for falling edge detection and calls
//				i2c_read to trigger a falling edge. Checks for
//				falling edge detection.
//
static int GPIO_Edge_Detect_Test(void)
{
	int temp;
	unsigned char buffer[2];

	// Set Pin 6(EDGE_LEVEL_GPIO_TO_TEST) to Detect Rising Edge
	PUT32(GPIO_REDR, EDGE_LEVEL_GPIO_TO_TEST);
	// Clear Pin 6(EDGE_LEVEL_GPIO_TO_TEST) of previous edge detect status
	PUT32(GPIO_EDSR, EDGE_LEVEL_GPIO_TO_TEST);

	// Make a I2C read operation to trigger rising edge
	i2c_read(SLAVE_ADDR_0, 0x1, buffer);

	temp = GET32(GPIO_EDSR);
	printf("GPIO Rising Edge Detect... ");
	if( (temp & EDGE_LEVEL_GPIO_TO_TEST) != 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL\n");
		DelayNUsec(1000000);
	}

	// Clear all pins to disable Detect Rising Edge
	PUT32(GPIO_REDR, 0x0);

	// Set Pin 6(EDGE_LEVEL_GPIO_TO_TEST) to Detect Falling Edge
	PUT32(GPIO_FEDR, EDGE_LEVEL_GPIO_TO_TEST);

	// Clear Pin 6(EDGE_LEVEL_GPIO_TO_TEST) of previous edge detect status
	PUT32(GPIO_EDSR, EDGE_LEVEL_GPIO_TO_TEST);

	// Make a I2C write operation to ensure a falling edge is triggered on a read
	i2c_write(SLAVE_ADDR_1, (RESERVED_HIGH_ADDR + 1), 0xE);

	// Make a I2C read operation to trigger falling edge
	i2c_read(SLAVE_ADDR_0, 0x1, buffer);

	temp = GET32(GPIO_EDSR);
	printf("Falling Edge Detect... ");
	if( (temp & EDGE_LEVEL_GPIO_TO_TEST) != 0)
	{
		printf("PASS\n");
		return(PASS);
	}
	else
	{
		printf("FAIL\n");
		return(FAIL);
	}
}

//
// Function: GPIO_Level_Detect_Test
//
// Description: Sets GPIO for level HIGH sensitivity and calls
//				i2c_read to trigger a level HIGH. Checks for
//				level HIGH detection.
//				Sets GPIO for level LOW sensitivity and calls
//				i2c_read to trigger a level LOW. Checks for
//				level LOW detection.
//
static int GPIO_Level_Detect_Test(void)
{
	int temp;
	int passfail = 0;
	unsigned char buffer[2];

	// Set Pin 6(EDGE_LEVEL_GPIO_TO_TEST) to Enable Level Sensitive High
	PUT32(GPIO_LSHR, EDGE_LEVEL_GPIO_TO_TEST);

	// Clear Pin 6(EDGE_LEVEL_GPIO_TO_TEST) of previous Level Sensitive status
	PUT32(GPIO_LDSR, EDGE_LEVEL_GPIO_TO_TEST);

	// Make a I2C write operation to ensure a level HIGH is triggered on a read
	i2c_write(SLAVE_ADDR_1, (RESERVED_HIGH_ADDR + 1), 0xE);

	// Make a I2C read operation to trigger a level change
	i2c_read(SLAVE_ADDR_0, 0x1, buffer);

	// Read GPIO_LDSR to determine if a Level Sensitive High was detected
	temp = GET32(GPIO_LDSR);
	printf("GPIO Level Sensitive High... ");
	if( (temp & EDGE_LEVEL_GPIO_TO_TEST) != 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL\n");
		passfail = 1;
	}

	// Clear Pin 6(EDGE_LEVEL_GPIO_TO_TEST) to Disable Level Sensitive High
	PUT32(GPIO_LSHR, 0);

	// Clear Pin 6(EDGE_LEVEL_GPIO_TO_TEST) of previous Level Sensitive status
	PUT32(GPIO_LDSR, EDGE_LEVEL_GPIO_TO_TEST);

	// Set Pin 6(EDGE_LEVEL_GPIO_TO_TEST) to Enable Level Sensitive Low
	PUT32(GPIO_LSLR, EDGE_LEVEL_GPIO_TO_TEST);

	// Make a I2C read operation to trigger a level change
	i2c_read(SLAVE_ADDR_0, 0x1, buffer);

	// Read GPIO_LDSR to determine if a Level Sensitive Low was detected
	temp = GET32(GPIO_LDSR);
	printf("Level Sensitive Low... ");
	if( (temp & EDGE_LEVEL_GPIO_TO_TEST) != 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL\n");
		passfail = 1;
	}
	if (passfail)
		return(FAIL);
	else
		return(PASS);
}

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_GPIO,"gpio",GPIO_Test,DIAGNOSTIC,"GPIOTest","runs gpio detection, register, edge, and level tests");
Switch("d", BOOLEAN32ARG, "GPIO detection test", "");
Switch("r", BOOLEAN32ARG, "GPIO register test", "");
Switch("e", BOOLEAN32ARG, "GPIO edge detection test", "");
Switch("l", BOOLEAN32ARG, "GPIO level detection test", "");
Switch("a", BOOLEAN32ARG, "Run all GPIO tests", "");
CommandEnd();
