/* hdlx2416.c - HDLX2416 LED Display Driver */

/* Copyright 1984-1999 Wind River Systems, Inc., 2002 Intel Corporation */

/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/hdlx2416.c 1     3/05/04 2:54p Ajpasqui $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/hdlx2416.c $
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 2     8/20/03 4:59p Daly
 * added VSS tags
 *
modification history
--------------------
01a,16may02, jjd  created
01a,22nov02, jjd  added #ifndef LED_BASE

*/

/*
DESCRIPTION

HDLX-2416 LED Display Driver

INCLUDE FILES: vm.h
               hdlx2416.h
*/
#include "vm.h"
#include "hdlx2416.h"

#ifndef LED_BASE
#define LED_BASE 0	/* XXX this will be defined in board.h */
#endif
static void msdelay(int delay) {} /* XXX this will be defined somewhere */

/*****************************************************************************
*
* hdlx2416_init() - initialize LED display
*
* initialize hdlx2416 display with 100% brightness
*
* RETURNS: N/A
*/
static void hdlx2416_init()
    {
    volatile unsigned int *display_cr;
    
    display_cr = (volatile unsigned int *)(LED_BASE + HDLX2416_CR);

    *display_cr = HDLX2416_CR_DON | HDLX2416_CR_B100;
    }

/*****************************************************************************
*
* hdlx2416_blank(enable) - turn LED display blank on/off
*
* turn hdlx2416 display blanking on or off
*
* RETURNS: N/A
*/
static void hdlx2416_blank
    (
     int enable
    )
    {
    volatile unsigned int *display_cr;

    display_cr = (volatile unsigned int *)(LED_BASE + HDLX2416_CR);

    if (enable)
        *display_cr = (*display_cr & HDLX2416_CR_BRMASK);
    else
        *display_cr = ((*display_cr & HDLX2416_CR_BRMASK) | HDLX2416_CR_DBLK);
    }

/*****************************************************************************
*
* hdlx2416_display(str) - display 4 characters to LED display
*
* display 4 chars on hdlx2416 pointed to by str
*
* RETURNs: N/A
*/
static void hdlx2416_display
    (
     const char *str
    )
    {
    volatile unsigned int *display;
    int i;

    display = (volatile unsigned int *)(LED_BASE + HDLX2416_CRAM);

    for (i = 0;i < 4; i++)
        display[i] = *str++;
    }

/*****************************************************************************
*
* hdlx2416_clear() - clear LED display
*
* clear hdlx2416 display by writing 4 space characters (0x20)
*
* RETURNS: N/A
*/
static void hdlx2416_clear()
    {
    hdlx2416_display("    ");
    }

/*****************************************************************************
*
* board_displayinit() - general board level display initialization
*
* initialize board display, calls the board specific display init function
* 
* RETURNS: N/A
*/
void board_displayinit()
    {
    hdlx2416_init();
    }

/*****************************************************************************
*
* boardblank(enable) - general board level display blanking
*
* blank board display, calls board specific display blank function
*
* RETURNS: N/A
*/
void boardblank
    (
     int blankenable
    )
    {
    hdlx2416_blank(!blankenable);
    }

/***************************************************************************** 
*
* boarddisplay(msg) - general board level display message
*
* display 4 chars packed in a register
*
* RETURNS: N/A
*/
void boarddisplay
    (
     unsigned int msg
    )
    {
    char buf[5];
    char *bp = buf;
    int i;

    boardblank(0);	/* disable blank */

    for (i=0;i<4;i++,msg>>=8)
        bp[i] = msg & 0xff;	
    
    bp[4] = '\0';

    hdlx2416_display(buf);
    }

/*****************************************************************************
*
* boardmessage (msg) - general board display string
* 
* display a message, if more than 4 chars then scroll across display
*
* RETURNS: N/A
*/
void boardmessage
    (
     const char *msg
    )
    {
    int len = strlen(msg);

    hdlx2416_clear();

    if (len <= 4)
        boarddisplay((unsigned int)msg);
    else
        {
        char buf[5];
        int i,j;

        for (i = 0;i < (len + 3);i++)
            {
            for (j = i;j < (i + 4); j++)
                buf[(j-i)^3] = ((j < 3) || (i > len-1)) ? ' ' : msg[j-3];
            buf[4] = '\0';
            hdlx2416_display(buf);
            msdelay(100);
            }
        }
    }
 
