/* i2c.h
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: vgd
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */
/*Revision history:
3/27/02: modified by vgd to change the #defines fo CLOCK and DATA 
as the GPIO pins used for I2C have been changed by HW design. 
Also updated the GPIO base register as per current PRM.
**************************************************************/
/*************************************************************
Filename	:	i2c.h
Desc: Header file for the i2c driver
***************************************************************/
#ifndef I2C_H
#define I2C_H

#ifdef IXP2800
#include "ixp2000Gpio.h"
#else
#include "gpio.h"
#endif //IXP2800

#define LOW 0
#define HIGH 1

/*write this POSR to drive the clock pin (GPIO pin 7)High and POCR for LOW*/
#define CLOCK	        0x80 

/*write this POSR to drive the data pin (gpio pin 6)High and POCR for LOW*/
#define DATA	        0x40 

/*This GPIO bit 2 has to be configured as output and drive a high to cause 
 *a pull up on the SDA line.
 */
#define I2C_INIT_BIT    0x04 

/*hal_delay_us() is defined in ixdp2400_misc.c*/
#define CLOCK_LOW_TIME	            5 /*us*/ 
#define CLOCK_HIGH_TIME	            5 /*us*/
#define START_CONDITION_HOLD_TIME   5 /*us*/
#define STOP_CONDITION_HOLD_TIME    5 /*us*/

#define I2C_ERROR  	1
#define I2C_NOERR	0
#define I2C_READ    0x01
#define I2C_WRITE   0x0

#define SLAVE_ADDR_0 0xa0 /*ie., a2,a1,a0 bits are 000*/ //DRAM
#define SLAVE_ADDR_1 0xa2 /*ie., a2,a1,a0 bits are 001*/ //KI
#define SLAVE_ADDR_2 0xa4 /*ie., a2,a1,a0 bits are 010*/
#define SLAVE_ADDR_3 0xa6 /*ie., a2,a1,a0 bits are 011*/
#define SLAVE_ADDR_4 0xa8 /*ie., a2,a1,a0 bits are 100*/
#define SLAVE_ADDR_5 0xaa /*ie., a2,a1,a0 bits are 101*/ // ADUC

#define KI_DEV_ID		0x1
#define BI_DEV_ID		0x2
#define MI_DEV_ID		0x3
#define BD_DEV_ID		0x4
#define BW_DEV_ID		0x5
#define PL_DEV_ID		0x6
#define TC_DEV_ID		0x7

/*
 *   slave address = DEV_TYPE | DEV_ADDR | MODE .  
 *   Device type is fixed to 1010 i.e., 0xa. 
 *   ___________________________________
 *   1 | 0 | 1 | 0 | A2 | A1 | A0| R/W|
 *   -----------------------------------
 * The LSB is r/W bit. if it is 1 the slave is in read mode and if it is 0 
 * the slave is in write mode.
 */

#define I2C_BUFSIZE	256

                
#define REG_WRITE(reg,data) \
        (*((volatile unsigned long *)(reg)) = (unsigned long int)(data))    


#endif
