/* i2c_test.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: Intel
 * 
 * Revisions:
 * 01a, 09/15/03 ajp Added pass/fail returns for script file break on error
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2x00.h"
#include "i2c_test.h"
#include "i2c.h"
#include "diag_utils.h"
#include "led.h"

// static function prototypes
	int I2C_Pattern_Test(void * pIn, PassFail_testOut* pOut);
	int I2C_Page_Write_Test(I2C_testIn);

	unsigned int Test_Pattern[4] = {0xA5,0x5A,0x00,0xFF};




// User input data
typedef struct _I2C_testIn
{
	unsigned char   i2c_addr;
	unsigned char   i2c_data;
} I2C_testIn;


//
// Function: I2C_Pattern_Test
//

int I2C_Pattern_Test(void * pIn, PassFail_testOut* pOut)
{
	unsigned int addr;
	int j, failure = 0;
	unsigned char pattern_w, pattern_r[16];
	pOut->PassFail = PASS;

	Scroll_Off();

	printf("**** WARNING ****: The I2C Pattern Test reduces ");
	printf("the write life of the I2C by writing to the I2C.\n");
	for(addr = Test_Start_Addr; addr<(Test_End_Addr+1); addr++)
		for(j = 0, pattern_w = Test_Pattern[j]; j<4; j++)
		{
			i2c_write((unsigned char)SLAVE_ADDR, (unsigned char)addr, pattern_w);
			DelayNMsec(10);
			i2c_read(SLAVE_ADDR, addr, pattern_r);
			DelayNMsec(10);
			if(pattern_w != pattern_r[0])
			{
				printf("Writing pattern failed at addr: %d\n", addr);
				failure++;
			}
		}
	printf("I2C Pattern Test... ");
	if(failure == 0)
	{
		printf("PASS\n");

	}
	else
	{
		printf("FAIL\n");
		pOut->PassFail = FAIL;
	}

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);
}

//
// Function: I2C_Write_Test
//
int I2C_Write_Test(I2C_testIn* pIn, PassFail_testOut* pOut)
{
	unsigned int addr;
	unsigned char data;
	unsigned char data_r;

	addr  = pIn->i2c_addr;
	data  = pIn->i2c_data;
	pOut->PassFail = PASS;

	Scroll_Off();

	printf("**** WARNING ****: The I2C Pattern Test reduces ");
	printf("the write life of the I2C by writing to the I2C.\n");
	// If addr falls within config data area, do not allow write
	if( (addr >= RESERVED_HIGH_ADDR) || (addr >= EEPROM_SIZE) )
	{
		printf("The address used falls in reserved space.\n");
//		printf("Please use addresses from %08x to %08x\n",
//				(RESERVED_HIGH_ADDR+1), (EEPROM_SIZE-1));
		printf("Please use addresses from %08x to %08x\n",
				(0x0), (0x7F));
		pOut->PassFail = FAIL;
	}
	else
	{
		i2c_write(SLAVE_ADDR, addr, data);
		DelayNMsec(100);

		i2c_read(SLAVE_ADDR, addr, &data_r);
		DelayNMsec(100);

		printf("I2C Write Test... ");
		if(data == data_r)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
			pOut->PassFail = FAIL;
		}



	}
	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);

}

//
// Function: I2C_Page_Write_Test
//
int I2C_Page_Write_Test(I2C_testIn* pIn, PassFail_testOut* pOut)
{
	unsigned char start_addr;
	int i;
	unsigned char first_byte[16];
	char data[16], data1[16];

	start_addr  = pIn->i2c_addr;

	pOut->PassFail = PASS;

	Scroll_Off();

	printf("**** WARNING ****: The I2C Page Write Test reduces ");
	printf("the write life of the I2C by writing to the I2C.\n");

	for(i = 0; i < 16; i++)
			data[i] = start_addr + i;
	i2c_page_write(SLAVE_ADDR, start_addr, (unsigned char)data, 16);
	DelayNMsec(100);
	i2c_seq_read(SLAVE_ADDR, start_addr, &data1, 16);
	DelayNMsec(100);
	printf("I2C Page Write Test... ");
	if(strcmp(data, data1) == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL\n");
		pOut->PassFail = FAIL;
	}

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_I2C_PATTERN,"i2c_patt",I2C_Pattern_Test,0,"i2c pattern test","i2c pattern test");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_I2C_WRITE,"i2c_wrt",I2C_Write_Test,0,"i2c write","i2c write test");
Arg(UINT8ARG,"h:addr","h:addr");
Arg(UINT8ARG,"h:data","h:data");
CommandEnd();


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_I2C_PAGE,"i2c_pg",I2C_Page_Write_Test,0,"i2c page","i2c page write test");
Arg(UINT8ARG,"h:addr","h:addr");
Arg(UINT8ARG,"h:data","h:data");
CommandEnd();
 