/******************************************************************************
*
* Module: i8255x.h
*
* Description: i8255x Module Definitions
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

#if __cplusplus
extern "C" {
#endif

#ifndef I8255XH
#define I8255XH

#include "ethernet.h"
#include "vm.h"

#ifdef PCI_AUTO_ON
#include "pciautoconfig.h"
#endif

// #include <types.h>

typedef volatile unsigned char VUCHAR;
typedef volatile unsigned short VUSHORT;
typedef volatile unsigned long VULONG;

#define DELAY_CONSTANT		0x1000

#define MIN_RFDS	8	/* Minimum RFDs in RFA needed */
#define LAN_TBD		4	/* number of TX buffers that can be associated with 1 tx cmd */

#define MAX_BYTES	1520

#define SIMPLE      0x0000
#define FLEXIBLE    0x0008


/* WARNING!! The following pragma puts the Diab compiler into a mode where
   all enclosed structure declarations are byte swapped. This makes building
   structures with PCI endian-ness easy. The "pragma pack(0)" ends this mode.
   Be sure you understand what you're doing if you add to or change anything
   between the 2 pragmas.
*/

// #pragma pack (0,0,1)

typedef volatile struct {
    USHORT	status;
    USHORT	cmd;
    void	*lnk;
}CMD_BLK;

typedef CMD_BLK DIAG;

typedef struct {
    VUCHAR	cr_us;
    VUCHAR	stat_ack;
    VUCHAR	cr_uc;
    VUCHAR	int_mask;
    ULONG	gen_ptr;
    ULONG	port;
    USHORT	flash_ctl;
    USHORT	eeprom_ctrl;
    ULONG	mdi_ctrl;
    ULONG	rx_dma_count;
    ULONG	the_last_word;
}SCB_CRS;
/*--------------------------------------------------------------------*/
/* The first (hardware dictated) part of a Tramsmit Command Block is  */
/* most conveniently defined as a byte swapped structure.             */
/*--------------------------------------------------------------------*/
typedef volatile struct {
    USHORT	status;
    USHORT	cmd;
    struct tcb	*next;
    void	*bd_addr;
    USHORT	count;
    UCHAR	tx_thresh;
    UCHAR	tbd_num;
} TCB;
typedef struct {	/* Transmit buffer Descriptor */
    void	*baddr;
    USHORT	size;
    USHORT	unused;
} TBD;
typedef volatile struct rfd {	/* Receive Frame Descriptor */
    USHORT	status;
    USHORT	cmd;
    void	*next;
    void	*rbdptr;
    USHORT	count;
    USHORT	size;
}RFD;

// #pragma pack (0)

typedef struct tcb {
    TCB		tcb;			/* 0x10 bytes */
    ULONG	pad[8];
    TBD		tbd_array[LAN_TBD];    /* make the TBD array part of the TCB */
} TXB;

typedef struct pkb {
    RFD		rfd;
    UCHAR	da[6];
    UCHAR	sa[6];
    USHORT	type;
    UCHAR	data[MAX_BYTES];
    ULONG	spare[2];
} PKB;

#define PKB_SIZE (ALIGN(sizeof(PKB),32))

// #pragma pack(1)

typedef struct {
    CMD_BLK	cmd_blk;
    UCHAR	byte_count, b1, b2, b3;
    UCHAR	b4, b5, b6, b7;
    UCHAR	b8, b9, b10, b11;
    UCHAR	b12, b13, b14, b15;
    UCHAR	b16, b17, b18, b19;
    UCHAR	b20, b21, b22, b23;
}CFG;
typedef struct {
    CMD_BLK	cmd_blk;
    UCHAR	byte_1;		/* IAddr byte 1 */
    UCHAR	byte_2;		/* IAddr byte 2 */
    UCHAR	byte_3;		/* IAddr byte 3 */
    UCHAR	byte_4;		/* IAddr byte 4 */
    UCHAR	byte_5;		/* IAddr byte 5 */
    UCHAR	byte_6;		/* IAddr byte 6 */
    UCHAR	fill[6];	/* padded for the hell of it */
} IAS;

// #pragma pack()

// #pragma pack(1)
typedef struct  {
    UCHAR	byte[6];
} e_addr;
// #pragma pack()

/*----------------------------------------------*/
/* Ethernet header structure                    */
/*----------------------------------------------*/
// #pragma pack(1)
struct etheader {
    e_addr	daddr;	/* destination address     */
    e_addr	saddr;	/* source address          */
    USHORT	type;	/* packet type             */
};
// #pragma pack()

#define ETH_HEADER_SIZE   14

/*----------------------------------------------------------------------*/
/* Lan Interface Common Information                                     */
/*----------------------------------------------------------------------*/
typedef struct lan_intf {
    tEEDRIVER	func_ptrs;
    ULONG	demarc;
	
#ifdef PCI_AUTO_ON
    PCI_LOC	p_dev;
#endif

    UINT32	PciHandle;
    SCB_CRS	*IOaddr;
    UINT32	good_open;

    ULONG	irq;
    ULONG	num_lan_tcb;
    ULONG	num_lan_pkb;

    e_addr	if_enet_addr;
    e_addr	if_broad_addr;	

    TXB		*tcb_free;
    TXB		*ni_tcb;
    int		tcb_count;

    PKB		*pkb_free;
    PKB		*PacketsPtr;
    int		pkb_count;

    PKB		*rfa_head, *rfa_tail, *last_unk_rfd;
    ULONG	rfa_count, ru_stopped, restarted_ru;

    TXB		*begin_cbl, *end_cbl, *last_unk_cb;
    TXB		*beg_saved_blks, *end_saved_blks;

    ULONG	inoctets;
    ULONG	innucastpkts;
    ULONG	inucastpkts;
    ULONG	unknown_cmd_completed;
    ULONG	num_fr_events;
    ULONG	num_rnr_events;
    ULONG	num_cna_events;
    ULONG	num_saved_blks;
    ULONG	num_cu_idles;
    ULONG	total_cmds_completed;
    ULONG	num_cus_active_during_cu_idle;
    ULONG	num_rx_passed_up;
    ULONG	impossible_happened;
    CFG		config;
    IAS		ia_struct;
    DIAG	diag;
	HANDLE	esd8xxISR;

} LAN_INTF;

/* SCB command register definitions */

#define RUC_NOP			0
#define RUC_START		1
#define RUC_RESUME		2
#define RUC_ABORT		4
#define RUC_LOAD_HDS		5	/* Load Header Data  */
#define RUC_LOAD_RU_BASE	6	/* Load RU Base  */
#define RUC_RBD_RESUME		7

#define CUC_NOP			0
#define CUC_START		0x10
#define CUC_RESUME		0x20
#define CUC_LOAD_DUMPCTR	0x40	/* Load Dump Counters Address */
#define CUC_DUMP_STAT		0x50	/* Dump Statistical Counters */
#define CUC_LOAD_CU_BASE	0x60	/* Load CU Base Register       */
#define CUC_DUMP_RESET_STAT	0x70	/* Dump and Reset Statistical Counters */

enum{ I557_CSR_INT = 0x1, I557_CSR_SI = 0x2};

/* SCB status register definitions */

/* RUS status */
#define RUS_SUPENDED		0x0004
#define RUS_NO_RESOURCES	0x0008
#define RUS_READY		0x0010
#define RUS_SUPENDED_NO_RBD	0x0024
#define RUS_NO_RESOURCES_NO_RBD	0x0028
#define RUS_READY_NORBD		0x0030

/* CUS status */
#define CUS_SUSPEDED		0x0040
#define CUS_ACTIVE		0x0080

/* interrupts */
#define INT_CX			0x80
#define INT_FR			0x40
#define INT_CNA			0x20
#define INT_RNR			0x10
#define INT_MDI			0x08
#define INT_SWI			0x04
#define INT_FCP			0x01

/* Mask for interrupt status bits in SCB   */
#define STATUS_INT_MASK		0xfc00
 
#define CMD_EL			0x8000
#define CMD_S			0x4000
#define CMD_I			0x2000

#define STATUS_C		0x8000
#define STATUS_OKAY		0x2000
#define STATUS_F		0x0800

#define CMD_ACT_NOP		0x0000
#define CMD_ACT_IA_SETUP	0x0001
#define CMD_ACT_CONFIGURE	0x0002
#define CMD_ACT_MCA_SETUP	0x0003
#define CMD_ACT_TRANSMIT	0x0004
#define CMD_ACT_DUMP		0x0006
#define CMD_ACT_DIAGNOSE	0x0007

/*---------------------------------------------------------------------*/
/* Configuration Structure.                                            */
/*---------------------------------------------------------------------*/

//#pragma pack(1)
/*---------------------------------------------------------------------*/
/* Individual Address Structure.                                       */
/*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
/* Statistics area                                                     */
/*---------------------------------------------------------------------*/
typedef volatile struct stats {
    ULONG       xmit_good;
    ULONG       xmit_maxcol;
    ULONG       xmit_latecol;
    ULONG       xmit_undrrun;
    ULONG       xmit_lostcarrier;
    ULONG       xmit_deferred;
    ULONG       xmit_singlecol;
    ULONG       xmit_multiplecol;
    ULONG       xmit_totalcol;
    ULONG       rcv_good;
    ULONG       rcv_crcerr;
    ULONG       rcv_alignerr;
    ULONG       rcv_resourceerr;
    ULONG       rcv_ovrrun;
    ULONG       rcv_cdt;
    ULONG       rcv_short;
    ULONG       fc_xmtpause;
    ULONG       fc_rcvpause;
    ULONG       fc_tcvnotsup;
    ULONG       end;
} STP;

/*---------------------------------------------------------------------*/
/* Packet types recognized by the NI driver.                           */
/*---------------------------------------------------------------------*/
#define IP_TYPE     0x0800
#define ARP_TYPE    0x0806
#define RARP_TYPE   0x8035

#define P_FAIL     -1

/* EEPROM defines */

#define EESK_BIT 1
#define EECS_BIT 2
#define EEDI_BIT 4
#define EEDO_BIT 8
#define READ_INSTR 6

#endif /* I8255XH */

#if __cplusplus
}
#endif


