/* i8255xmdi.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: jjd
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

/* includes */

#include "vm.h"
#include "cmd.h"
#include "pci.h"

/* typedefs */

/*
* Input Structure for Diagnostic
*/
typedef struct _i8255xmdiIn
{
	UINT32	device; /* Device number to find (0-1) */
} i8255xmdiIn;

typedef struct {
    UCHAR	cr_us;
    UCHAR	stat_ack;
    UCHAR	cr_uc;
    UCHAR	int_mask;
    ULONG	gen_ptr;
    ULONG	port;
    USHORT	eeprom_ctrl;
    USHORT	flash_ctl;
    ULONG	mdi_ctrl;
    ULONG	rx_dma_count;
    ULONG	the_last_word;
}SCB_CRS;

#define DEERISLAND_PCI_DEVICE_ID 0x1209
#define DEERISLAND_PCI_VENDOR_ID 0x8086

static UINT32 readMdio (SCB_CRS *scb, int phy, int reg)
{
    int result,j;
    int timeout = 100;
    volatile UINT32 *pmdiCR = &scb->mdi_ctrl;

    *pmdiCR = LSWAP(0x08000000 | (reg << 16) | (phy<<21));
    do {
        for (j=0;j<50;j++);
            result = LSWAP(*pmdiCR);
        if (--timeout < 0)
            printf("readMdio timed out: 0x%x\n",result);
    } while (! (result & 0x10000000));

    return (result & 0xffff);

}

static UINT32 writeMdio (SCB_CRS *scb, int phy, int reg, int val)
{
    int result,j;
    int timeout = 100;
    volatile UINT32 *pmdiCR = &scb->mdi_ctrl;

    *pmdiCR = LSWAP((0x04000000 | (reg<<16) | (phy<<21) | val));

    do {
        for (j=0;j<50;j++);
        result = LSWAP(*pmdiCR);
        if (--timeout < 0)
            printf ("writeMdio timed out: 0x%x\n",result);
    } while (! (result & 0x10000000));

    return (result & 0xffff);

}

/*************************************************************************
* i8255xmdi - Definition for the i8255xmdi Diagnostic
*
* RETURNS: '0' for success, '1' for failure
* ERRNO: N/A
*/

int i8255xmdi
    (
    i8255xmdiIn *    pIn    /* Input Structure for Diagnostic */
    )
    {
            int bus, device, function,ba;
            SCB_CRS *scb;
            int i;

            if (pIn->device > 1 || pIn->device < 0) Result(-1,"Device number out of range\n");

            device = pIn->device;
            if (pciFindDevice(DEERISLAND_PCI_VENDOR_ID,
                              DEERISLAND_PCI_DEVICE_ID,
                              device, &bus, &device, &function) != OK)
                    Result(-1, "Device not found\n");

            pciConfigInLong(bus,device,function,PCI_CFG_BASE_ADDRESS_0,(UINT32 *)&ba);
            scb = (SCB_CRS *)(ba & ~0xF);

            for (i=0;i<8;i+=2)
                    printf("MII[%d]: 0x%04x MII[%d]: 0x%04x\n", i, readMdio(scb,1,i),
                           i+1,readMdio(scb,1,i+1));
            for (i=16;i<31;i+=2)
                    printf("MII[%d]: 0x%04x MII[%d]: 0x%04x\n", i, readMdio(scb,1,i),
                           i+1,readMdio(scb,1,i+1));

    Result(0, "This is the i8255xmdi diagnostic\n");

    /* Add return code: '0' for success, '1' for failure */
    return 0;
    }

/* Macro Format: Command(CMDUSER_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String"); */
Command(CMDUSER_I8255XMDI,"i8255xmdi",i8255xmdi,DIAGNOSTIC,"i8255xmdi","Read/Write MDI I/F on 8255x NICs");
Arg(UINT32ARG,"device","Device number to find (0-1)");
CommandEnd();

