/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 BY Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/* ixd2810.h - IXD2810 MAC device header file

modification history
--------------------
01a, 08oct02,eme	created
*/

#ifndef __INCixd2810h
#define __INCixd2810h

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _ASMLANGUAGE

#define uint32_t UINT32
#include <ixdp2000.h>
#include <ixd2810Port.h>
#include <ixd2810Mmap.h>
#include <ixd2810ModNum.h>

/* Board IDs */
#define BOARD_ID	0x2810

/* Number of device supported */
#define N_DEVICES	2

/* Port number found on IXD2810 */
#define N_PORTS	10

/* Converts a port number to the corresponding port mask bit definition */
#define	PORT_MASK(p)	(1 << p)

/* Memory size allocated for each device */
#define IXD2810_MAC_SIZE SZ_8M

/* Board Id location offset */
#define IXD2810_BOARDID_OFFSET	SZ_1M
#define IXD2810_BOARDID_MASK 	0xfffff  /* masks */

/*
 * Register access Macros
 */
#define IXD2810_REG_GET(reg, result) \
	((result) = *(volatile uint32_t *)(reg))
#define IXD2810_REG_SET(reg, data)  \
	(*((volatile uint32_t *)(reg)) = (data))

/* Provides the ioctl commands */
#define IXD2810_IOCTL_GET	0
#define IXD2810_IOCTL_SET   1

/* maximum name strings length */
#define	NAME_SIZE		30

/* externs */
IMPORT uint32_t intLevelList[];
IMPORT char*  taskNames[];
/* 
 * Defines a data type with all possible memory bases 
 * allocated for their corresponding registers block.
 */
typedef enum 
	{
	PORT0_BLOCK,
	PORT1_BLOCK,
	PORT2_BLOCK,
	PORT3_BLOCK,
	PORT4_BLOCK,
	PORT5_BLOCK,
	PORT6_BLOCK,
	PORT7_BLOCK,
	PORT8_BLOCK,
	PORT9_BLOCK,
	GLOBAL_BLOCK,
	RX_BLOCK,
	TX_BLOCK,
	RESERVED_BLOCK,
	SPI42_BLOCK,
	SERDES_BLOCK,
	GBIC_BLOCK,
	BOARDID_BLOCK
	} BLOCK_ID;

LOCAL char *ixd2810BlockStr[] = 
	{
	"PORT0_BLOCK",
	"PORT1_BLOCK",
	"PORT2_BLOCK",
	"PORT3_BLOCK",
	"PORT4_BLOCK",
	"PORT5_BLOCK",
	"PORT6_BLOCK",
	"PORT7_BLOCK",
	"PORT8_BLOCK",
	"PORT9_BLOCK",
	"GLOBAL_BLOCK",
	"RX_BLOCK",
	"TX_BLOCK",
	"RESERVED_BLOCK",
	"SPI42_BLOCK",
	"SERDES_BLOCK",
	"GBIC_BLOCK",
	"BOARDID_BLOCK"
	};

/* device board ID base locations */
LOCAL uint32_t ixd2810BoardIdBase[N_DEVICES] =
	{
	(UINT32) MEDIA_BOARD_BASE,
	(UINT32) SF_BOARD_BASE
	};


/* 
 * Device header parameters to commnicate between the application 
 * and the ixd2810 using the memory block access method.
 */
typedef struct ixd2810_block
	{
	BLOCK_ID 	blockId;		/* section of the memory mapped IO to access */
	uint32_t	startingReg;	/* register to access */
	uint32_t   	*data;	 		/* buffer for read/write information */
	uint32_t   	regNum;  		/* length of data buffer the appl expects */
	} block_t;

/*
 * Data maintained by the driver during the execution of the code 
 */
typedef struct _ixd2810_dev
	{
	uint32_t	devId;			/* device ID */
	uint32_t 	vbase;			/* virtual address base */
	uint32_t	gbicStatus;		/* GBIC Status */
    BOOL		created;		/* TRUE if this device has been created */
	BOOL		isRunning;		/* device current status/mode */
	int 		intCnt;			/* interrupt count */
	int			taskId;			/* task ID spawn */
	DEV_PORTS 	ports[N_PORTS];	/* ports on device */
	} IXD2810_DEV;

/* Function prototypes */
uint32_t ixd2810_Create(uint32_t devId);
STATUS ixd2810_Start(uint32_t devId);
STATUS ixd2810_Stop(uint32_t devId);
STATUS ixd2810_Ioctl(uint32_t devId, uint32_t cmd, block_t *stBlock);
	
#endif	/* _ASMLANGUAGE */


#ifdef __cplusplus
}
#endif

#endif /* __INCixd2810h */


