/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 BY Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/* ixd2810Mmap.h - Memory map addresses

modification history
--------------------
01a, 20oct02,eme	created
*/
#ifndef __INCixd2810Mmaph
#define __INCixd2810Mmaph

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Retrieve the Port Index/Address
 * Assumption: variable port is a valid port number
 */
#define BLOCK_BASE_OFFSET	0x0
#define BLOCK_BASE_ADRS(b) (((b & 0xf) << 7) + BLOCK_BASE_OFFSET)

/* 
 * Assumption: variable reg is a valid register specifying the
 * address value of the register to access in the port. The
 * register address has to come from any of the following 
 * register groups:
 *	MAC Control Register
 *	MAC RX Statistics Register
 *	MAC TX Statistics Register
 */
#define BLOCK_REG_ADRS(block, regOffset) (BLOCK_BASE_ADRS(block) + regOffset)

/* 
 * MAC Ctrl Registers (Port Index + Offset) 
 */
#define STATION_ADRS_LOW 	0x00	/* Source MAC address bit 31-0: R/W */ 
#define STATION_ADRS_HIGH	0x01	/* Source MAC address bit 47-0: R/W */ 
#define FDFC_TYPE			0x03	/* TX Puase FRame FDFC Type field */
#define FC_TX_TIMER			0x07	/* Pause length */
#define FDFC_ADRS_LOW		0x08	/* lowest 32 bits of the FDFC */
#define FDFC_ADRS_HIGH		0x09	/* highest 16 32 bits of the FDFC */
#define IPG_RX_TIME1		0x0A	/* First part of the IPG time for */
#define IPG_RX_TIME2		0x0B	/* Second part of the IPG time for */
#define IPG_TX_TIME			0x0C	/* IPG time */
#define PAUSE_THLD			0x0E	/* Pause threshold time :R/W */ 
#define MAX_FRAME_SIZE		0x0F	/* Maximum frame size :R/W */ 
#define FC_ENABLE			0x12	/* Indicates flow control settings */
#define SHORT_RUNTS_THLD	0x14	/* Threshold for short and runt*/
#define DISCARD_UNK_CFRAME	0x15	/* Unknown control frames. */ 
#define RX_CONFIG_WORD		0x16	/* received config_word. */ 
#define TX_CONFIG_WORD		0x17	/* sent config_word */
#define DIV_CONFIG_WORD		0x18	/* configuration bits */
#define RX_PKT_FILTER_CTRL	0x19	/* marked filtered packets */
#define PORT_MC_ADRS_LOW	0x1A	/* port multicast frames lowest address */
#define PORT_MC_ADRS_HIGH	0x1B	/* port multicast frames highest address */

/* bit definition for the RX_CONFIG_WORD register */
#define RC_An_Complete			(1 << 21)	/* auto-negotation complete. */
#define RC_Half_Duplex			(1 << 6)	
#define RC_Full_Duplex			(1 << 5)

/* bit definitions for the DIV_CONFIG_WORD register */
#define DC_Autoneg_Enable		(1 << 5)	/* enable auto-negotiation */

/*
 * MAC RX Statistics Register Map (Port Index + Offset)
 */
#define RX_OCTETS_OK	  0x20 /* received bytes in all legal frames*/
#define RX_OCTETS_BAD	  0x21 /* received bytes in all bad frames */
#define RX_UC_PKTS		  0x22 /* unicast packets received */
#define RX_MC_PKTS	 	  0x23 /* multicast packets received */
#define RX_BC_PKTS		  0x24 /* broadcast packets received */
#define RX_PKTS_64		  0x25 /* packets received less than 64 octets */
#define RX_PKTS_65_127	  0x26 /* packets received [65-127] octets length */
#define RX_PKTS_128_255	  0x27 /* packets received [128-255] octets length */
#define RX_PKTS_256_511	  0x28 /* packets received [256-511] octets length */ 
#define RX_PKTS_512_1023  0x29 /* packets received [512-1023] octets length */ 
#define RX_PKTS_1024_1518 0x2A /* packets received [1024-1518] octets length */
#define RX_PKTS_1519_MAX  0x2B /* packets received greater than 1519 octets */
#define RX_FCS_ERR		  0x2C /* frames, received with legal size, BAD CRC */
#define RX_TAGGED		  0x2D /* frames with VLAN tag */
#define RX_DATA_ERR		  0x2E /* frames, received with code violation.  */
#define RX_ALIGN_ERR	  0x2F /* frames containing < 8 additional bits */
#define RX_LONG_ERR		  0x30 /* OK CRC and integral number of octets */
#define RX_JABBER_ERR	  0x31 /* bad CRC and a non-integral number of octets */
#define RX_MAC_CFRAME	  0x32 /* Pause MAC control frames */
#define RX_UNK_CFRAME	  0x33 /* received with an op code not 0001 */
#define RX_VLONG_ERR	  0x34 /* very long frames */
#define RX_RUNT_ERR		  0x35 /* runt error */
#define RX_SHORT_ERR	  0x36 /* short error */
#define RX_XCARRIER_ERR	  0x37 /* Gigabit half-duplex event only  */
#define RX_SEQ_ERR		  0x38 /* sequencing errors that occur */
#define RX_SYMBOL_ERR	  0x39 /* symbol errors, encountered by the PHY */

/*
 * MAC TX Statistics Register Map (Port Index + Offset)
 */
#define TX_OCTETS_OK		0x40 /* bytes transmitted in all legal frames */
#define TX_OCTETS_BAD		0x41 /* bytes transmitted in all bad frames */
#define TX_UC_PKTS	 		0x42 /* unicast packets transmitted */
#define TX_MC_PKTS	 		0x43 /* multicast packets transmitted*/
#define TX_BC_PKTS	 		0x44 /* broadcast packets transmitted*/
#define TX_PKTS_64	 		0x45 /* packets that are 64 octets */
#define TX_PKTS_65_127		0x46 /* packets that are 65-127 octets */
#define TX_PKTS_128_255		0x47 /* packets that are 128-255 octets */
#define TX_PKTS_256_511		0x48 /* packets that are 256-511 octets */
#define TX_PKTS_512_1023	0x49 /* packets that are 512-1023 octets */
#define TX_PKTS_1024_1518	0x4A /* packets that are 1024-1518 octets */
#define TX_PKTS_1519_MAX 	0x4B /* packets that are 1519-MAX octets */
#define TX_DEFERRED		 	0x4C /* times, of postponed frames */ 
#define TX_TOTAL_COLL	 	0x4D /* sum of all collision events */
#define TX_SINGLE_COLL 		0x4E /* transmitted frames with one colllision */
#define TX_MULT_COLL		0x4F /* transmitted frames with colllisions */
#define TX_LATE_COLL	 	0x50 /* times, a collision is detected */
#define TX_XCV_COLL_ERR	  	0x51 /* frames, which collides 16 times */
#define TX_XCV_DEF_ERR		0x52 /* frames postponed more than the limit */
#define TX_XCV_LEN_DROP		0x53 /* long frame dropped */
#define TX_UNDERRUN			0x54 /* internal TX error */
#define TX_TAGGED			0x55 /* OK frames with VLAN tags. */
#define TX_CRC_ERR			0x56 /* frames with bad CRC */
#define TX_PAUSE_FRAMES		0x57 /* Pause frames transmitted. :RR  */
#define TX_FC_COLL_SEND		0x58 /* generated collision for flow control */

/*
 * Global Status and Configuration Register Map
 */
#define GLOBAL_BASE		  0x500	/* Global Configuration block base adrs */
#define PORT_ENABLE		  0x00 /* Control register for each port.   */ 
#define LINK_LED_ENABLE	  0x02 /* set upon detection of link to enable */
#define MAC_SOFT_RESET	  0x05 /* software activated reset of the MAC core */ 
#define CPU_INTERFACE	  0x08 /* CPU interface Endian select.*/ 
#define LED_CONTROL		  0x09	/* Handles the LED mode.  */ 
#define LED_FLASH_RATE	  0x0A	/* Handles the flash rate  */ 
#define LED_FAULT_DISABLE 0x0B	/* Disables the LED flashing */

/*
 * RX Block Register Map
 */
#define RX_FIFO_BASE			0x580
/* assumption: port variable is a valid port number [0-9] */
#define RX_FIFO_HIGH_WM(p)	 (p + 0x00) /* High watermark for RX FIFO */
#define RX_FIFO_LOW_WM(p)	 (p + 0x0A) /* Low watermark for RX FIFO. */
#define RX_FIFO_REMOVED(p)   (p + 0x14) /* frames lost/removed on port p */
#define RX_FIFO_EFRAMES_DROP (0x1F)	 /* which frames to drop */
#define RX_FIFO_OFLOW_EVENT	 (0x20)	 /* FIFO-full status */
/*
 * TX Block Register Map
 */
#define TX_FIFO_BASE	0x600
/* assumption: port variable is a valid port number */
#define TX_FIFO_HIGH_WM(p) 	 (p + 0x00) /* High watermark for TX FIFO */
#define TX_FIFO_LOW_WM(p) 	 (p + 0x0A) /* Low watermark for TX FIFO */
#define TX_FIFO_XFER_THLD(p) (p + 0x14) /* Transfer FIFO threshold */	
#define TX_FIFO_REMOVED(p)   (p + 0x22)  /* frames lost/removed on port p */
#define TX_FIFO_OFLOW_EVENT	 (0x1E)	 /* FIFO-full status */
#define TX_FIFO_INFO_SEQ	 (0x21)	 /* out-of-sequence data is detected */

/*
 * SPI4-2 Block Register Map
 */
#define SPI42_BASE			0x700 /* SPRI4-2 block base address */
#define SPI42_RX_BURST_SIZE	0x00 /* RX start-up parameters for burst size.  */ 
#define SPI42_RX_TRAINING  	0x01 /* RX start-up parameters for training seq. */ 
#define SPI42_CALENDAR	   	0x02 /* for FIFO status calendar operation.  */ 
#define SPI42_SYNC		   	0x03 /* synchronization DIP-4 counters.  */ 

/*
 * SerDes Block Register Map
 */
#define SERDES_BASE	 		0x780   /* SerDes block base address */
#define ACDC_COUPLING		0x00	/* selection of AC or DC coupling */
#define POWER_DOWN			0x07	/* Tx and Rx power-down bits */
#define RX_SIGNAL_LEVEL		0x13	/* status of the Rx input */

/*
 * GBIC Block Register Map
 * Caveat: Due to the base address (0x799) the BLOCK_REGS_ADRS cannot be used
 * to determine the actual mem address. The GBIC_BASE has to be used instead,
 * along with the following defined offesets. These offset values may appear
 * different in the IXF1110 Data Sheet.
 */
#define GBIC_BASE			0x799	/* GBIC Block base address */
#define GBIC_STATUS			0x00	/* The interface status to GBIC :RO  */ 
#define GBIC_CONTROL		0x01	/* The GBIC module configuration  */ 
#define I2C_CTRL_DATA		0x02	/* The I2C control and data register. */ 

/* bit definitions for GBIC_CONTROL register */
#define GC_Rx_Loss_Enable	(1 << 12)
#define GC_Tx_Fault_Enable	(1 << 11)
#define GC_Mod_Def_Enable	(1 << 10)

/* bit definitions for GBIC_STATUS register */
#define GS_Rx_Loss			0x3FF00000		/* bits 29:20 */
#define GS_Tx_Fault			0x000FFC00 		/* bits 19:10 */
#define GS_Mod_Def			0x000003FF		/* bits 9:0 */

/* starting bits for the three ranges of bit definitions in GBIC_STATUS */
#define GS_TX_STARTING_BIT			(1 << 20)
#define GS_RX_STARTING_BIT			(1 << 10)
#define GS_MOD_STARTING_BIT			(1 << 0)

#ifdef __cplusplus
}
#endif

#endif /* __INCixd2810Mmaph */
