/* ixdp2800.h - header for ixdp2800 board */

/* Copyright 2001 Wind River Systems, Inc. */

/*
modification history
--------------------

01a, 16apr02, eme created from 01a of ixdp2400
*/

/*
DESCRIPTION
This file contains I/O address and related constants for the
ixdp2800 board.
*/

#ifndef INCixdp2800h
#define INCixdp2800h

/* includes */

/*
 *IXDP2800 RDRAM controller
 */
#define RDRAM_CHANNEL_SIZE	SZ_256M			/* single channel max size */
#define RDRAM_CHANNELS		3				/* number of RDRAM channels */
#define RDRAM_BASE_ADRS  0x00000000		/* RDRAM Base Address */
#define RDRAM_SIZE_TOTAL  (RDRAM_CHANNELS * RDRAM_CHANNEL_SIZE)/* max size */
#define RDRAM_PHYS_BASE	    0x00000000	/* Physical starting address of RDRAM */
#define RDRAM_VIRT_BASE	    RDRAM_PHYS_BASE	/* similar to phys base address */
/*
* RDRAM_VIRT_OFFSET is the physical address of RDRAM corresponding
* to virtual address 0
*/
#define RDRAM_VIRT_OFFSET   (RDRAM_VIRT_BASE + LOCAL_MEM_RESERVED)
#define RDRAM_PHYS_OFFSET   (RDRAM_SIZE_TOTAL - RDRAM_VIRT_OFFSET)
#define RDRAM_LOW_VIRT_SIZE RDRAM_VIRT_OFFSET

#define RDRAM_CSR_BASE_ADRS 0xD0000000
#define RDRAM_0_BASE   0xD0000000
#define RDRAM_1_BASE   0xD0000040
#define RDRAM_2_BASE   0xD0000080

#define RDRAM_0_CONTROL  	 		(RDRAM_0_BASE + 0x00)
#define RDRAM_0_ERR_STATUS_1 		(RDRAM_0_BASE + 0x08)
#define RDRAM_0_ERR_STATUS_2 		(RDRAM_0_BASE + 0x10)
#define RDRAM_0_ECC_TEST   	 		(RDRAM_0_BASE + 0x18)
#define RDRAM_0_SERIAL_CMD   		(RDRAM_0_BASE + 0x20)
#define RDRAM_0_SERIAL_DATA  		(RDRAM_0_BASE + 0x28)
#define RDRAM_0_CONFIG_1   			(RDRAM_0_BASE + 0x30)
#define RDRAM_0_CONFIG_2   			(RDRAM_0_BASE + 0x38)
#define RDRAM_0_CONFIG_3   			(RDRAM_0_BASE + 0x40)
#define RDRAM_0_RAC_INIT   			(RDRAM_0_BASE + 0x48)
#define RDRAM_0_MISC_RAC_CONTROL 	(RDRAM_0_BASE + 0x50)
#define RDRAM_0_RAC_CONFIG   		(RDRAM_0_BASE + 0x58)
#define RDRAM_0_1066_CONFIG_GRP 	(RDRAM_0_BASE + 0x60)
#define RDRAM_0_SERIAL_CONFIG   	(RDRAM_0_BASE + 0x68)

#define RDRAM_1_CONTROL  			(RDRAM_1_BASE + 0x00)
#define RDRAM_1_ERR_STATUS_1 		(RDRAM_1_BASE + 0x08)
#define RDRAM_1_ERR_STATUS_2 		(RDRAM_1_BASE + 0x10)
#define RDRAM_1_ECC_TEST   			(RDRAM_1_BASE + 0x18)
#define RDRAM_1_SERIAL_CMD   		(RDRAM_1_BASE + 0x20)
#define RDRAM_1_SERIAL_DATA  		(RDRAM_1_BASE + 0x28)
#define RDRAM_1_CONFIG_1   			(RDRAM_1_BASE + 0x30)
#define RDRAM_1_CONFIG_2   			(RDRAM_1_BASE + 0x38)
#define RDRAM_1_CONFIG_3   			(RDRAM_1_BASE + 0x40)
#define RDRAM_1_RAC_INIT   			(RDRAM_1_BASE + 0x48)
#define RDRAM_1_MISC_RAC_CONTROL 	(RDRAM_1_BASE + 0x50)
#define RDRAM_1_RAC_CONFIG   		(RDRAM_1_BASE + 0x58)
#define RDRAM_1_1066_CONFIG_GRP 	(RDRAM_1_BASE + 0x60)
#define RDRAM_1_SERIAL_CONFIG   	(RDRAM_1_BASE + 0x68)

#define RDRAM_2_CONTROL  			(RDRAM_2_BASE + 0x00)
#define RDRAM_2_ERR_STATUS_1 		(RDRAM_2_BASE + 0x08)
#define RDRAM_2_ERR_STATUS_2 		(RDRAM_2_BASE + 0x10)
#define RDRAM_2_ECC_TEST   			(RDRAM_2_BASE + 0x18)
#define RDRAM_2_SERIAL_CMD   		(RDRAM_2_BASE + 0x20)
#define RDRAM_2_SERIAL_DATA  		(RDRAM_2_BASE + 0x28)
#define RDRAM_2_CONFIG_1   			(RDRAM_2_BASE + 0x30)
#define RDRAM_2_CONFIG_2   			(RDRAM_2_BASE + 0x38)
#define RDRAM_2_CONFIG_3   			(RDRAM_2_BASE + 0x40)
#define RDRAM_2_RAC_INIT   			(RDRAM_2_BASE + 0x48)
#define RDRAM_2_MISC_RAC_CTL 		(RDRAM_2_BASE + 0x50)
#define RDRAM_2_RAC_CONFIG  	 	(RDRAM_2_BASE + 0x58)
#define RDRAM_2_1066_CONFIG_GRP 	(RDRAM_2_BASE + 0x60)
#define RDRAM_2_SERIAL_CONFIG   	(RDRAM_2_BASE + 0x68)


/* unfortunately, these definitions seem to need to be here */
/* RDRAM Control Register Addresses */
#define INIT     0x021
#define TEST34   0x022
#define CNFGA    0x023
#define CNFGB    0x024
#define DEVID    0x040
#define REFB     0x041
#define REFR     0x042
#define CCA      0x043
#define CCB      0x044
#define NAPX     0x045
#define PDNXA    0x046
#define PDNX     0x047
#define TPARM    0x048
#define TFRM     0x049
#define TCDLY1   0x04a
#define TCYCLE   0x04c
#define SKIP     0x04b
#define TEST77   0x04d
#define TEST78   0x04e
#define TEST79   0x04f
#define TCPS     0x055   

/* RDRAM SIO Operations */ 
#define SIORESETOP	0xe
#define SIOSETROP	0x2
#define SIOCLRROP	0xb
#define SIOWRITEOP	0x0
#define SIOREADOP	0x1
#define SIOPDNEOP	0x2
#define SIOSETFOP	0x4

/* RDRAM SIO Broadcast */
#define SIOSBC	0x400

#define ArchnChannelsMax              1
#define ArchnDRAMsMax                 1
#define DeviceType                    4

#define RDRAM_CONTROL_Size8MB         (0 << 8)
#define RDRAM_CONTROL_Size16MB        (1 << 8)
#define RDRAM_CONTROL_Size32MB        (2 << 8)
#define RDRAM_CONTROL_Size64MB        (3 << 8)
#define RDRAM_CONTROL_Size128MB       (4 << 8)
#define RDRAM_CONTROL_Size256MB       (5 << 8)
#define RDRAM_CONTROL_Size512MB       (6 << 8)
#define RDRAM_CONTROL_Size1GB         (7 << 8)
#define RDRAM_CONTROL_SizeMask        0x700
#define RDRAM_CONTROL_NumChannels(x)  ( ((x) - 1) << 11)  /* only if x={1,2,3} */
#define RDRAM_CONTROL_ParityEnable    (1 << 14)
#define RDRAM_CONTROL_EccEnable       (2 << 14)
#define RDRAM_CONTROL_ReadBkPresDis   (1 << 16)         /* Read BackPressure Disable */
#define RDRAM_CONTROL_BankRemap(x)    ( ((x) & 0x3) << 18)

#define RDRAM_SERIAL_CMD_SerOpMask    0xf
#define RDRAM_SERIAL_CMD_SerDev(x)    ( ((x) & 0x3f) << 4)
#define RDRAM_SERIAL_CMD_SerBcast     (1 << 10)
#define RDRAM_SERIAL_CMD_SerAddr(x)   ( ((x) & 0xfff) << 11)
#define RDRAM_SERIAL_CMD_SclkFreqDiv(x)  ( ((x) & 0xff) << 23)
#define RDRAM_SERIAL_CMD_Busy         (1 << 31)

#define RDRAM_CONFIG_1_tRCD(x)        ( ((x) & 0xf) << 0) /* {5,7,9,11} */
#define RDRAM_CONFIG_1_tCAC(x)        ( ((x) & 0xf) << 4) /* {8,9,10,11,12} */
#define RDRAM_CONFIG_1_tCACMask       0xf0
#define RDRAM_CONFIG_1_tRASSyn(x)     ( (((x) & 0x1f) >> 2) << 8) /* {16,20,24,28} */
#define RDRAM_CONFIG_1_tRP(x)         ( ((x) & 0xf) << 12)  /* {6,8,10,12} */
#define RDRAM_CONFIG_1_tOffp(x)       ( ((x) & 0x7) << 16)  /* {4,5,6,7} */
#define RDRAM_CONFIG_1_tOffpMask      0x70000
#define RDRAM_CONFIG_1_RacStep2Enable (1 << 19)
#define RDRAM_CONFIG_1_tRASRefSyn(x)  ( (((x) & 0x3f) >> 2) << 20) /* {16,20,24, etc} */
#define RDRAM_CONFIG_1_tRPrefSyn(x)   ( (((x) & 0xf) >> 2) << 24)  /* {8,12,16, etc} */
#define RDRAM_CONFIG_1_CCTestMode(x)  ( ((x) & 0x7) << 28)
#define RDRAM_CONFIG_1_BtoBRead       (1 << 31)

#define RDRAM_RAC_INIT_InitInProgress (1 << 0)
#define RDRAM_RAC_INIT_DoRacInit      (1 << 1)
#define RDRAM_RAC_INIT_ClockLock      (1 << 2)
#define RDRAM_RAC_INIT_RacT44Timer(x) ( ((x) & 0x3ff) << 3)
#define RDRAM_RAC_INIT_RacCctlIn(x)   ( ((x) & 0x7f) << 13)
#define RDRAM_RAC_INIT_MaskCurrentCtl (1 << 20)
#define RDRAM_RAC_INIT_Step6Busy      (1 << 21)
#define RDRAM_RAC_INIT_Step5Busy      (1 << 22)
#define RDRAM_RAC_INIT_AllInitDone    (1 << 23)
#define RDRAM_RAC_INIT_InitRefCctlCnt(x) ( ((x) & 0xf) << 24)
#define RDRAM_RAC_INIT_DoStep6        (1 << 28)
#define RDRAM_RAC_INIT_DoStep5        (1 << 29)
#define RDRAM_RAC_INIT_RacReset       (1 << 30)
#define RDRAM_RAC_INIT_RacPowerup     (1 << 31)

#define RDRAM_SERIAL_CONFIG_SclkFreqDiv(x) ( ((x) & 0xff) << 0)
#define RDRAM_SERIAL_CONFIG_NapXCnt(x)     ( ((x) & 0xf) << 8)
#define RDRAM_SERIAL_CONFIG_Pdnx(x)        ( ((x) & 0xf) << 12)
#define RDRAM_SERIAL_CONFIG_PdnxMask       0xf000
#define RDRAM_SERIAL_CONFIG_Napx(x)        ( ((x) & 0xf) << 16)
#define RDRAM_SERIAL_CONFIG_NapxMask       0x0f00
#define RDRAM_SERIAL_CONFIG_RintStart(x)   ( ((x) & 0xff) << 20)
#define RDRAM_SERIAL_CONFIG_RintWidth(x)   ( ((x) & 0xf) << 28)
        
#define RDRAM_CONTROL_Val        (RDRAM_CONTROL_ReadBkPresDis | \
                                  RDRAM_CONTROL_NumChannels(1) | \
                                  RDRAM_CONTROL_Size32MB)

#define RDRAM_RAC_INIT_Step2Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(4) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) )

#define RDRAM_RAC_INIT_Step5Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_DoStep5 | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(4) | \
                                   RDRAM_RAC_INIT_RacCctlIn(64) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) | \
                                   RDRAM_RAC_INIT_ClockLock | \
                                   RDRAM_RAC_INIT_DoRacInit )

#define RDRAM_RAC_INIT_Step6Val  ( RDRAM_RAC_INIT_RacPowerup | \
                                   RDRAM_RAC_INIT_DoStep5 | \
                                   RDRAM_RAC_INIT_DoStep6 | \
                                   RDRAM_RAC_INIT_InitRefCctlCnt(13) | \
                                   RDRAM_RAC_INIT_RacCctlIn(64) | \
                                   RDRAM_RAC_INIT_RacT44Timer(0x217) | \
                                   RDRAM_RAC_INIT_ClockLock | \
                                   RDRAM_RAC_INIT_DoRacInit )

#define RDRAM_SERIAL_CONFIG_Val  ( RDRAM_SERIAL_CONFIG_SclkFreqDiv(6) |\
                                   RDRAM_SERIAL_CONFIG_NapXCnt(4) |\
                                   RDRAM_SERIAL_CONFIG_Pdnx(1) |\
                                   RDRAM_SERIAL_CONFIG_Napx(1) |\
                                   RDRAM_SERIAL_CONFIG_RintStart(0) |\
                                   RDRAM_SERIAL_CONFIG_RintWidth(0) )

/* Assembly specific defs */
#ifndef _ASMLANGUAGE
#endif
#endif  /* INCixdp2800h */
