/* ixp2000PMUTimer.c - Timer driver for the Intel Ixp2000 (PMU) */

/* Copyright 1984-1999 Wind River Systems, Inc., 2003 Intel Corporation */

/*
modification history
--------------------
01a,10jan03, jjd  created from 01 ixp2000Timer.c
*/

/*
DESCRIPTION

Timer driver for the Intel IXP2000 PMU unit

INCLUDE FILES:  vm.h
				timer.h
*/

/* includes */
#include "vm.h"
#include "timer.h"
#include "ixp2x00.h"
#include "ixp2000PMU.h"

/* defines */

/* typedefs */
typedef struct ixp2000PMUTimer		/* IXP2000_PMU_TIMER */
	{
	UINT32  PMUChapCmd;	/* CHAP Cmd  */
	UINT32  PMUChapEvn;	/* CHAP Event */
        UINT32  PMUChapStat;	/* CHAP Status */
        UINT32  PMUChapData;	/* CHAP Data */
	}IXP2000_PMU_TIMER;

/* global */
extern int ClockCounterIsr(void *);
HANDLE TIMERISR;

/* locals */
IXP2000_PMU_TIMER   *timerRegs = (IXP2000_PMU_TIMER *)TIMERADR;

volatile UINT32 *PMUIntEnable = (volatile UINT32 *)(PMU_BASE + 0xc00);
volatile UINT32 *PMUIntMask   = (volatile UINT32 *)(PMU_BASE + 0xd00);
volatile UINT32 *PMUIntStat   = (volatile UINT32 *)(PMU_BASE + 0xe00);
volatile UINT32 *PMUContCfg   = (volatile UINT32 *)(PMU_BASE + 0xf00);

/* forward prototypes */
void StopClockCounter(void);
void InitClockCounter(void);


/******************************************************************************
*
* timerISR - Timer Isr
*
* Timer Interupt Service Request
*
* RETURNS: 1
* 
* ERRNO: N/A
*/
static const char runstring[]="Deer Island!!!   ";
char tempstring[256] = "Egress:    C Ingress:    C   ";
static int *display = (int *)0xc7000100;

static int timerISR (void)
	{
	static int charptr=0;
	static int ticks=0;
	const char *displaystring;
	int i,len;
	extern int EgressTemp,IngressTemp,UpdateTemp;
	extern int Led_Test;

if(!Led_Test)
{
	if (IS_EGRESS_NPU)
		displaystring = runstring;
	else
	    {
		displaystring = tempstring;
	        if (UpdateTemp)
		    {
		    sprintf(tempstring,"Egress: %d C Ingress: %d C   ",EgressTemp,IngressTemp);
		    UpdateTemp = 0;
		    }
	    }

	len = strlen(displaystring);

#if (TGTBOARD == IXMB2800)	
	if (!ticks) {
		for (i=0;i<4;i++)
			display[i^3] = displaystring[(i+charptr)%len];

		charptr = (charptr+1)%len;
	}
	ticks = (ticks+1)%20;
#endif
}
	timerRegs->PMUChapCmd = (CC_TRUE|OP_PRELOAD|OUIE|ACTIVATE);
	*PMUIntStat = ~*PMUIntStat;

	ProcessTimers (TIMERPERIOD);

	return (1);

	}


/******************************************************************************
*
* InitTimerIRQ - Init Timer Isr
*
* Initialize the Timer Interupt Service Request
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
void InitTimerIRQ (void)
	{
		
	/* Install the Handler */
	TIMERISR = InstallHandler (TIMERIRQ, (HANDLER)timerISR, (void *)TIMERPERIOD);

	/* Initalize the Clock Counter */
	InitClockCounter ();

	/* Enable the IRQ for the timer ISR */
	EnableIRQ (TIMERISR);

	return;

	}


/******************************************************************************
*
* InitClockCounter - Init the Clock Counter
*
* Initialize the Clock the Counter
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
void InitClockCounter (void)
	{
#define PULSEGEN_VAL (250000000 / 100)
#define PLL_VAL	(25000000 / 100)

	int pllMult;
	int timerval;

	pllMult = (*(volatile int *)(STRAP_OPTIONS) & STRAP_OPTIONS_CfgPllMask) >> 8;

	if (pllMult == 0x3f)
		timerval = PULSEGEN_VAL;	/* if pulse generator */
	else
		timerval = PLL_VAL * pllMult;	/* else PLL */

	timerRegs->PMUChapEvn = (DDBS_PMU | DEC0_MUX_NULL);
	timerRegs->PMUChapData = timerval;
	*PMUIntEnable = 1;		/* fix this for different PMU #s */
	timerRegs->PMUChapCmd = (CC_TRUE|OP_PRELOAD|OUIE|ACTIVATE);
	timerRegs->PMUChapCmd = (CC_TRUE|OP_START|OUIE|ACTIVATE);
	
	return;

	}

Init (605, InitTimerIRQ);
