/* ixp2000timer.c - Timer driver for the Intel Ixp2000 */

/* Copyright 1984-1999 Wind River Systems, Inc., 2002 Intel Corporation */

/*
modification history
--------------------
01a,26apr02, jjd  created from 01 BRHTimer.c
*/

/*
DESCRIPTION

Timer driver for the Intel IXP2000

INCLUDE FILES:  vm.h
				timer.h
*/

/* includes */
#include "vm.h"
#include "timer.h"

/* defines */

/* Timer Control Register */
#define IXP2000_ACT     0x00000080

/* 
 * Count Value
 * (0.01 * clkfreq) = count for 10 ms timer,
 */
#define IXP2000_CNT         (TIMERCLK * 0.01)

/* typedefs */
typedef struct ixp2000Timer		/* IXP2000_TIMER */
	{
	UINT32  TimerCtrl;	/* Timer Control */
                UINT32  :32;
                UINT32  :32;
                UINT32  :32;
	UINT32  TimerLoad;	/* Timer load Value */
                UINT32  :32;
                UINT32  :32;
                UINT32  :32;
        UINT32  TimerValue;	/* Timer Current Value */
                UINT32  :32;
                UINT32  :32;
                UINT32  :32;
        UINT32  TimerIntClr; /* Timer interrupt clear */
	}IXP2000_TIMER;

/* global */
extern int ClockCounterIsr(void *);
HANDLE TIMERISR;

/* locals */
IXP2000_TIMER   *timerRegs = (IXP2000_TIMER *)TIMERADR;

/* forward prototypes */
void StopClockCounter(void);
void InitClockCounter(void);



/******************************************************************************
*
* timerISR - Timer Isr
*
* Timer Interupt Service Request
*
* RETURNS: 1
* 
* ERRNO: N/A
*/
static const char runstring[]="IXDP2800!!!   ";
char tempstring[256] = "Egress:    C Ingress:    C   ";
static int *display = (int *)0xc7000100;


static int timerISR (void)
	{
	static int charptr=0;
	static int ticks=0;
	const char *displaystring;
	int i,len;
	extern int EgressTemp,IngressTemp,UpdateTemp;
	extern int Led_Test;

if(!Led_Test)
{
	if (IS_EGRESS_NPU)
		displaystring = runstring;
	else
	    {
		displaystring = tempstring;
	        if (UpdateTemp)
		    {
		    sprintf(tempstring,"Egress: %d C Ingress: %d C   ",EgressTemp,IngressTemp);
		    UpdateTemp = 0;
		    }
	    }

	len = strlen(displaystring);

	if (!ticks) {
		for (i=0;i<4;i++)
			display[i^3] = displaystring[(i+charptr)%len];

		charptr = (charptr+1)%len;
	}
	ticks = (ticks+1)%20;
}

    timerRegs->TimerIntClr = 1;

	ProcessTimers (TIMERPERIOD);

	return (1);

	}


/******************************************************************************
*
* InitTimerIRQ - Init Timer Isr
*
* Initialize the Timer Interupt Service Request
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
void InitTimerIRQ (void)
	{
		
	/* Install the Handler */
	TIMERISR = InstallHandler (TIMERIRQ, (HANDLER)timerISR, (void *)TIMERPERIOD);

	/* Initalize the Clock Counter */
	InitClockCounter ();

	/* Enable the IRQ for the timer ISR */
	EnableIRQ (TIMERISR);

	return;

	}


/******************************************************************************
*
* InitClockCounter - Init the Clock Counter
*
* Initialize the Clock the Counter
*
* RETURNS: N/A
* 
* ERRNO: N/A
*/
void InitClockCounter (void)
	{

	/* Set the Load value and enable the timer */
	timerRegs->TimerLoad = IXP2000_CNT;
        timerRegs->TimerCtrl = IXP2000_ACT;

	return;

	}

Init (605, InitTimerIRQ);
