/* ixp2000pci.h - IXP2000 PCI Bridge Header */

/* Copyright 1984-1999 Wind River Systems, Inc., 2002 Intel Corporation */

/*
 * $Header: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/ixp2000pci.h 1     3/05/04 2:54p Ajpasqui $
 * $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VisionWARE2.1/PROJECTS/IXP28X0/Board/IXMB2800/ixp2000pci.h $
 * 
 * 1     3/05/04 2:54p Ajpasqui
 * 
 * 3     9/04/03 1:57p Ajpasqui
 * Added handshaking using mailbox3 in the ucode files so that we don't
 * overrun the RBUF register while saing data to scratch memory
 * 
 * 2     8/20/03 5:12p Daly
 * added vss tags
 *
modification history
--------------------
01a,29apr02, jjd  created from 01a brhpci.h
01b,10oct02, ajp  change CSR accessing functions throughout diags
01c,24feb03, jjd  add PCI_XSCALE_INT_ENABLE
01d,25feb03, jjd  added PCI_CONTROL bits definitons
01e,20aug03, jjd  added modification history, now managed by vss log

*/

/*
DESCRIPTION

IXP2000 PCI Configuration Header File

INCLUDE FILES: N/A
*/

#ifndef __INCixp2000pcih
#define __INCixp2000pcih

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* defines */

#define IXP2000_PCI_BASE	0xde000000 /* IXP PCI Base Address	CORE|UINT32 */

/* Bit definitions */
#define BIT0                            0x00000001
#define BIT1                            0x00000002
#define BIT2                            0x00000004
#define BIT3                            0x00000008
#define BIT4                            0x00000010
#define BIT5                            0x00000020
#define BIT6                            0x00000040
#define BIT7                            0x00000080
#define BIT8                            0x00000100
#define BIT9                            0x00000200
#define BIT10                           0x00000400
#define BIT11                           0x00000800
#define BIT12                           0x00001000
#define BIT13                           0x00002000
#define BIT14                           0x00004000
#define BIT15                           0x00008000
#define BIT16                           0x00010000
#define BIT17                           0x00020000
#define BIT18                           0x00040000
#define BIT19                           0x00080000
#define BIT20                           0x00100000
#define BIT21                           0x00200000
#define BIT22                           0x00400000
#define BIT23                           0x00800000
#define BIT24                           0x01000000
#define BIT25                           0x02000000
#define BIT26                           0x04000000
#define BIT27                           0x08000000
#define BIT28                           0x10000000
#define BIT29                           0x20000000
#define BIT30                           0x40000000
#define BIT31                           0x80000000

/* Sizes */
#define SZ_1K                           0x00000400
#define SZ_4K                           0x00001000
#define SZ_8K                           0x00002000
#define SZ_16K                          0x00004000
#define SZ_64K                          0x00010000
#define SZ_128K                         0x00020000
#define SZ_256K                         0x00040000
#define SZ_512K                         0x00080000
#define SZ_1M                           0x00100000
#define SZ_2M                           0x00200000
#define SZ_4M                           0x00400000
#define SZ_8M                           0x00800000
#define SZ_16M                          0x01000000
#define SZ_32M                          0x02000000
#define SZ_64M                          0x04000000
#define SZ_128M                         0x08000000
#define SZ_256M                         0x10000000
#define SZ_512M                         0x20000000
#define SZ_1G                           0x40000000
#define SZ_2G                           0x80000000

/* Register Offsets */
#ifndef __LITTLE_ENDIAN__
#define IXP2000_PCI_CFG_VENDOR_ID       0x02
#define IXP2000_PCI_CFG_DEVICE_ID       0x00

#define IXP2000_PCI_CFG_COMMAND         0x06
#define IXP2000_PCI_CFG_STATUS          0x04

#define IXP2000_PCI_CFG_REVISION        0x0b
#define IXP2000_PCI_CFG_PROGRAMMING_IF  0x0a
#define IXP2000_PCI_CFG_SUBCLASS        0x09
#define IXP2000_PCI_CFG_CLASS           0x08

#define IXP2000_PCI_CFG_CACHE_LINE_SIZE 0x0f
#define IXP2000_PCI_CFG_LATENCY_TIMER   0x0e
#define IXP2000_PCI_CFG_HEADER_TYPE     0x0d
#define IXP2000_PCI_CFG_BIST            0x0c

#define IXP2000_PCI_CFG_BASE_ADDRESS_0  0x10
#define IXP2000_PCI_CFG_BASE_ADDRESS_1  0x14
#define IXP2000_PCI_CFG_BASE_ADDRESS_2  0x18
#define IXP2000_PCI_CFG_BASE_ADDRESS_3  0x1c
#define IXP2000_PCI_CFG_BASE_ADDRESS_4  0x20
#define IXP2000_PCI_CFG_BASE_ADDRESS_5  0x24
#define IXP2000_PCI_CFG_CIS             0x28

#define IXP2000_PCI_CFG_SUB_VENDER_ID   0x2e
#define IXP2000_PCI_CFG_SUB_SYSTEM_ID   0x2c

#define IXP2000_PCI_CFG_EXPANSION_ROM   0x30
#define IXP2000_PCI_CFG_RESERVED_0      0x34
#define IXP2000_PCI_CFG_RESERVED_1      0x38

#define IXP2000_PCI_CFG_DEV_INT_LINE    0x3f
#define IXP2000_PCI_CFG_DEV_INT_PIN     0x3e
#define IXP2000_PCI_CFG_MIN_GRANT       0x3d
#define IXP2000_PCI_CFG_MAX_LATENCY     0x3c
#define IXP2000_PCI_RCOMP_OVERRIDE	0x60
#define IXP2000_PCI_RCOMP_STATUS        0x64
#else

#define IXP2000_PCI_CFG_VENDOR_ID       0x00
#define IXP2000_PCI_CFG_DEVICE_ID       0x02

#define IXP2000_PCI_CFG_COMMAND         0x04
#define IXP2000_PCI_CFG_STATUS          0x06

#define IXP2000_PCI_CFG_REVISION        0x08
#define IXP2000_PCI_CFG_PROGRAMMING_IF  0x09
#define IXP2000_PCI_CFG_SUBCLASS        0x0a
#define IXP2000_PCI_CFG_CLASS           0x0b

#define IXP2000_PCI_CFG_CACHE_LINE_SIZE 0x0c
#define IXP2000_PCI_CFG_LATENCY_TIMER   0x0d
#define IXP2000_PCI_CFG_HEADER_TYPE     0x0e
#define IXP2000_PCI_CFG_BIST            0x0f

#define IXP2000_PCI_CFG_BASE_ADDRESS_0  0x10
#define IXP2000_PCI_CFG_BASE_ADDRESS_1  0x14
#define IXP2000_PCI_CFG_BASE_ADDRESS_2  0x18
#define IXP2000_PCI_CFG_BASE_ADDRESS_3  0x1c
#define IXP2000_PCI_CFG_BASE_ADDRESS_4  0x20
#define IXP2000_PCI_CFG_BASE_ADDRESS_5  0x24
#define IXP2000_PCI_CFG_CIS             0x28

#define IXP2000_PCI_CFG_SUB_VENDER_ID   0x2c
#define IXP2000_PCI_CFG_SUB_SYSTEM_ID   0x2e

#define IXP2000_PCI_CFG_EXPANSION_ROM   0x30
#define IXP2000_PCI_CFG_RESERVED_0      0x34
#define IXP2000_PCI_CFG_RESERVED_1      0x38

#define IXP2000_PCI_CFG_DEV_INT_LINE    0x3c
#define IXP2000_PCI_CFG_DEV_INT_PIN     0x3d
#define IXP2000_PCI_CFG_MIN_GRANT       0x3e
#define IXP2000_PCI_CFG_MAX_LATENCY     0x3f
#define IXP2000_PCI_RCOMP_OVERRIDE	0x60
#define IXP2000_PCI_RCOMP_STATUS        0x64
#endif

/* PCI Configuration Registers Offsets */
#define IXP2000_PCI_CSR_BASE	0xDF000000
#define PCI_OUT_INT_STATUS     0x30
#define PCI_OUT_INT_MASK       0x34
#define MAILBOX_0		       0x50
#define MAILBOX_1		       0x54
#define MAILBOX_2		       0x58
#define MAILBOX_3		       0x5C
#define SRAM_BASE_ADDR_MASK    0xFC
#define DRAM_BASE_ADDR_MASK    0x100
#define PCI_CONTROL            0x13C
#define PCI_ADDR_EXT           0x140
#define PCI_XSCALE_INT_EN	   0x15c

#define EGRESS_INGRESS_MAILBOX  0xE01FE000

#define SRAM_BASE_ADDR_MASK_Disable                  0x8FFC0000
#define SRAM_BASE_ADDR_MASK_Npr                      (1 << 30)  /* !prefetch */
#define SRAM_BASE_ADDR_MASK_MinSize                  0x40000    /* 256K */

#define DRAM_BASE_ADDR_MASK_Disable                  0xBFF00000
#define DRAM_BASE_ADDR_MASK_Npr                      (1 << 30) /* ! prefetch */
#define DRAM_BASE_ADDR_MASK_MinSize                  0x100000  /* 1MB */

#define PCI_CONTROL_CfgPciBoot         (1 << 31) /* if 1, IXP configures PCI */
#define PCI_CONTROL_CfgPromBoot        (1 << 30) /* if 1, boot from PROM */
#define PCI_CONTROL_CfgPciArb          (1 << 29) /* if 1, IXP is arbiter */
#define PCI_CONTROL_CfgRstDir          (1 << 28) /* if 1, #PCI_RST output */
#define PCI_CONTROL_DmaIdle            (1 << 27)
#define PCI_CONTROL_PciInRst           (1 << 26) /* PCI bus in reset */
#define PCI_CONTROL_TgtAbtDisable      (1 << 25)
#define PCI_CONTROL_XSInt              (1 << 24) /* generates soft int to PCI */
#define PCI_CONTROL_BEDeo              (1 << 22) /* BE Data swap Enable out */
#define PCI_CONTROL_BEDei              (1 << 21) /* BE Data swap Enable in */
#define PCI_CONTROL_BEBeo              (1 << 20) /* BE Byte Enable swap out */  
#define PCI_CONTROL_BEBei              (1 << 19) /* BE Byte Enable swap in */
#define PCI_CONTROL_Dtx                (1 << 16) /* Discard timer expired */
#define PCI_CONTROL_Trb                (1 << 15) /* */
#define PCI_CONTROL_Twr                (1 << 14) /* */
#define PCI_CONTROL_Tae                (1 << 13) /* */
#define PCI_CONTROL_Tde                (1 << 12) /* */
#define PCI_CONTROL_Tse                (1 << 11) /* */
#define PCI_CONTROL_Dde                (1 << 10) /* */
#define PCI_CONTROL_Dse                (1 << 9)  /* */
#define PCI_CONTROL_Dct                (1 << 8)  /* */
#define PCI_CONTROL_Dpe                (1 << 7)  /* */
#define PCI_CONTROL_Rta                (1 << 6)  /* Received Target Abort */
#define PCI_CONTROL_Rma                (1 << 5)  /* Received Master Abort */
#define PCI_CONTROL_Dped               (1 << 4)  /* Data parity error detected (master) */
#define PCI_CONTROL_Rs                 (1 << 3)  /* Received SERR */
#define PCI_CONTROL_PCII               (1 << 2)  /* */
#define PCI_CONTROL_AS                 (1 << 1)  /* */
#define PCI_CONTROL_CA                 (1 << 0)  /* PCI clock active */

#define PCI_ADDR_EXT_PioAddrMask       0xFFFF0000
#define PCI_ADDR_EXT_PioAddr(x)        ( (x) & PCI_ADDR_EXT_PioAddrMask)
#define PCI_ADDR_EXT_PmemAddrMask      0xE0000000
#define PCI_ADDR_EXT_PmemAddr(x)       ( ((x) & PCI_ADDR_EXT_PmemAddrMask) >> 16)

#define PCI_XSCALE_INT_EN_SoftEnable	(1 << 25)
#define PCI_XSCALE_INT_EN_PCIAEnable	(1 << 26)
#define PCI_XSCALE_INT_EN_PCIBEnable	(1 << 27)
#define PCI_XSCALE_INT_EN_Dma1Enable	(1 << 28)
#define PCI_XSCALE_INT_EN_Dma2Enable	(1 << 29)
#define PCI_XSCALE_INT_EN_Dma3Enable	(1 << 30)
#define PCI_XSCALE_INT_EN_BISTEnable	(1 << 31)

/* PCI Register defines */
#define PCI_MST_ENBL		   0x04
#define PCI_MEM_ACCESS_ENBL	   0x02
#define PCI_LAT_TIMER          0x1F



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCixp2000pcih */
