/* ixp2000pic.c - IXP2000 PIC Driver */

/* Copyright 1984-1999 Wind River Systems Inc., 2002 Intel Corporation. */

/*
modification history
--------------------
01a,26mar02, jjd  created from 01a brhpic.c
*/

/*
DESCRIPTION

IXP2000 Programmable Interrupt Controller

INCLUDE FILES:  vm.h
                ixp2000pic.h
*/

/* includes */
#include "vm.h"
#include "ixp2000pic.h"

/* defines */

/* IRQ / FIRQ Starting and Ending */
#define PIC_IRQ_BASE    PIC_NUM_IRQ
#define PIC_IRQ_END     (PIC_NUM_IRQ + 31)
#define PIC_FIRQ_BASE   PIC_NUM_FIRQ
#define PIC_FIRQ_END    (PIC_NUM_FIRQ + 31)

/*  locals */
static HLIST IrqHandles = { 0, 0, sizeof(tHANDLER) };
static HLIST FirqHandles = { 0, 0, sizeof(tHANDLER) };
static volatile IXP2000_PIC *pPICIRQ = (IXP2000_PIC *)PICIRQADRS;
static volatile IXP2000_PIC *pPICFIQ = (IXP2000_PIC *)PICFIQADRS;
/*  forward prototypes */
void ISRExit(void);


/***************************************************************************
*
* IRQ - Interrupt Service Request
*
* Service an interrupt request
*
* RETURNS: 1 if interrupt was recognized, 0 otherwise
*
* ERRNO: N/A
*/   

static int IRQ
    (
    tHANDLER *pHandler
    )
    {

    UINT32 ID;
    UINT32 bitMsk;

    /* Read the interrupt number. XXX other interrupt sources? */
    ID = pPICIRQ->PicStatus;
#if 0
    /* how do we get the UART? */
    for (bitMsk = 0x8000000; bitMsk; bitMsk >>= 1) {
	if (bitMsk & ID) {
	   while (pHandler = pHandler->pNext) {
		if ((1 << pHandler->UnbiasedID) == bitMsk) {
		    CallHandler(pHandler);
		}
		if ((bitMsk == 0x100) && (pHandler->UnbiasedID == 2)) { CallHandler(pHandler); return 1; }
	   }
	}
    }

    pPICIRQ->PicEnableClr = ID;
    return 0;
#else
    /* Check all bits in interrupt register */
    for (bitMsk = 0x40000000; bitMsk; bitMsk >>= 1) 
        {
        if (bitMsk & ID)
            {
            /* Find the registered handler and call it. */
            while (pHandler = pHandler->pNext)
                {
                if ((1 << pHandler->UnbiasedID) == bitMsk)
                    {
                    CallHandler(pHandler);
                    return 1;
                    }
                }
            }
        }

	/* Mask off the interrupt if no registered handler is found. */
    pPICIRQ->PicEnableClr = ID;

    return 0;
#endif

    }


/***************************************************************************
*
* FIRQ - Fast Interrupt Service Request
*
* Service an fast interrupt request
*
* RETURNS: 1 if interrupt was recognized, 0 otherwise
*
* ERRNO: N/A
*/   

static int FIRQ
    (
    tHANDLER *pHandler
    )
    {

    UINT32 ID;
    UINT32 bitMsk;

    /* Read the interrupt number. */
    ID = pPICFIQ->PicStatus;

    /* Check all bits in interrupt register */
    for (bitMsk = 0x40000000; bitMsk; bitMsk >>= 1) 
        {
        if (bitMsk & ID)
            {
            /* Find the registered handler and call it. */
            while (pHandler = pHandler->pNext)
                {
                if ((1 << pHandler->UnbiasedID) == bitMsk)
                    {
                    CallHandler(pHandler);
                    return 1;
                    }
                }
            }
        }

	/* Mask off the interrupt if no registered handler is found. */
    pPICFIQ->PicEnableClr = ID;
    
    return 0;

    }

/***************************************************************************
*
* PicDisableIRQ - Disable IRQ
*
* Disable the IRQ associated with a registered interrupt handler
*
* RETURNS: N/A
*
* ERRNO: N/A
*/   

static void PicDisableIRQ
    (
    tHANDLER *pHandler
    )
    {
    /* Mask off the bit to be disabled */
    pPICIRQ->PicEnableClr = (1 << pHandler->UnbiasedID);
    }

/***************************************************************************
*
* PicDisableFIRQ - Disable FIRQ
*
* Disable the FIRQ associated with a registered interrupt handler
*
* RETURNS: N/A
*
* ERRNO: N/A
*/   

static void PicDisableFIRQ
    (
    tHANDLER *pHandler
    )
    {
    /* Mask off the bit to be disabled */
    pPICFIQ->PicEnableClr = (1 << pHandler->UnbiasedID);
    }

/***************************************************************************
*
* PicEnableIRQ - Enable IRQ
*
* Enable the IRQ associated with a registered interrupt handler
*
* RETURNS: N/A
*
* ERRNO: N/A
*/   

static void PicEnableIRQ
    (
    tHANDLER *pHandler
    )
    {
    pPICIRQ->PicEnableSet = (1 << pHandler->UnbiasedID);
    }


/***************************************************************************
*
* PicEnableFIRQ - Enable FIRQ
*
* Enable the FIRQ associated with a registered interrupt handler
*
* RETURNS: N/A
*
* ERRNO: N/A
*/   

static void PicEnableFIRQ
    (
    tHANDLER *pHandler
    )
    {

    pPICFIQ->PicEnableSet = (1 << pHandler->UnbiasedID);

    }


/***************************************************************************
*
* Install - Register an interrupt or exception handler
*
* This function is not needed but must be defined for interface
*
* RETURNS: N/A
*
* ERRNO: N/A
*/   

static void Install
    (
    tHANDLER *pHandler, 
    UINT32 UnbiasedID
    )
    {
    return;
    }


/***************************************************************************
*
* PicInitIxp2000IRQ - IXP2000 PIC Initialization IRQ
*
* Set up the IXP2000 PIC IRQ to initial state
*
* RETURNS: N/A
*
* ERRNO:
*/   

void PicInitIxp2000IRQ (void)
    {
    InstallHandler (PICIRQ, (HANDLER)IRQ, (void *)&IrqHandles);
    }

/* Declare the PIC structure.  This must be a global declaration. */
tPIC PicIxp2000IRQ =
{
    PIC_IRQ_BASE,
    PIC_IRQ_END,
    &IrqHandles,
    Install,
    PicDisableIRQ,
    PicEnableIRQ,
    PicDisableIRQ
};
PIC(PicIxp2000IRQ);


/***************************************************************************
*
* PicInitIxp2000FIRQ - IXP2000 PIC Initialization FIRQ
*
* Set up the Ixp2000 PIC FIRQ to initial state
*
* RETURNS: N/A
*
* ERRNO:
*/   

void PicInitIxp2000FIRQ (void)
    {
    InstallHandler (PICFIRQ, (HANDLER)FIRQ, (void *)&FirqHandles);
    }


/* Declare the PIC structure.  This must be a global declaration. */
tPIC PicIxp2000FIRQ =
{
    PIC_FIRQ_BASE,
    PIC_FIRQ_END,
    &FirqHandles,
    Install,
    PicDisableFIRQ,
    PicEnableFIRQ,
    PicDisableFIRQ
};
PIC(PicIxp2000FIRQ);

