/* led.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Jan 15, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "led.h"
#include "diag_utils.h"

//static function prototypes
static void LED_Help(void);
#ifdef BLINK_CAPABLE
static void Blinking_Cursor_Test(void);
static void _LED_Blink(int ms);
#endif
static void Small_Char_LED_Test(void);
static void Big_Char_LED_Test(void);
static void Numerical_Char_LED_Test(void);
static void LED_Init(void);
static void _latch_LED_Data(void);
void Scroll_Off(void);
void Scroll_On(void);

extern int Led_Test;

// User input data
typedef struct _Led_testIn
{
 UINT32 small_sel;
 UINT32 big_sel;
 UINT32 num_sel;
 UINT32 all_sel;
} Led_testIn;

LED_CtlReg LED_CR;

typedef struct _Led_settestIn
{
	UINT32 led_test_data;
} Led_settestIn;

//
// Function: LED_Test
//
// Description: Calls the appropriate function(s) of the LED test
//				depending on user input.
//
void LED_Test(Led_testIn* pIn,void* pUnused)
{

	char opt;

	Scroll_Off();

	if ((pIn->small_sel) || (pIn->all_sel))
 	Small_Char_LED_Test();

	if ((pIn->big_sel) || (pIn->all_sel))
	Big_Char_LED_Test();

	if ((pIn->num_sel) || (pIn->all_sel))
	Numerical_Char_LED_Test();


}


//
// Function: LED_SET_Test
//
// Description: Calls the appropriate function(s) of the LED test
//				depending on user input.
//

void LED_Set_Test(Led_settestIn* pIn,void* pUnused)
{
	unsigned int all_data;
	{
	Set_LED(pIn->led_test_data);
	printf("Setting LED done!\n");
	} 

}

void Scroll_Off(void)
{
	Led_Test = 1;
	DisableTemp();
}

void Scroll_On(void)
{
	Led_Test = 0;
	EnableTemp();
}




//
// Function: Blinking_Cursor_Test
//
// Description: Blinking a fully lighted LED 5 times.
//
#ifdef BLINK_CAPABLE
static void Blinking_Cursor_Test(void)
{
	printf("Blinking Cursor Test started...\n");

	// Blink cursor display 5 times
	LED_CR.bits.CUE = 1;	// Set IC to display Cursor
	PUT8(LED_DISPLAY_CTL_REG, LED_CR.all_bits);
	DelayNUsec(500000);
	_LED_Blink(500000);	// 1st Blink
	_LED_Blink(500000);	// 2nd Blink
	_LED_Blink(500000);	// 3rd Blink
	_LED_Blink(500000);	// 4th Blink
	_LED_Blink(500000);	// 5th Blink
	LED_CR.bits.CUE = 0;	// Set IC to display ASCII

	printf("Blinking Cursor Test Completed!\n");
}
#endif
//
// Function: Small_Char_LED_Test
//
// Description: Displays the chars a-z on the LED
//
static void Small_Char_LED_Test(void)
{
	unsigned int data, all_data;

	printf("Lower Case LED Test started...\n");

	
	for(data = ASCII_a; data <= ASCII_z; data++)
	{
		if(KEYPRESS) return;

		all_data = (data | data << 8 | data << 16 | data << 24);

		Set_LED(all_data);
		DelayNUsec(LED_DELAY);
	}

	printf("Lower Case LED Test completed!\n");
}

//
// Function: Big_Char_LED_Test
//
// Description: Displays the chars A-Z on the LED
//
static void Big_Char_LED_Test(void)
{
	unsigned int data, all_data;

	printf("Upper Case LED Test started...\n");

	for(data = ASCII_A; data <= ASCII_Z; data++)
	{

		if(KEYPRESS) return;

		all_data = (data | data << 8 | data << 16 | data << 24);

		Set_LED(all_data);
 		
		DelayNUsec(LED_DELAY);
	}

	printf("Upper Case LED Test completed!\n");
}

//
// Function: Numerical_Char_LED_Test
//
// Description: Displays the numbers 0-9 on the LED
//
static void Numerical_Char_LED_Test(void)
{
	unsigned int data;
	unsigned int all_data;

	printf("Numeric LED Test started...\n");

	for(data = ASCII_0; data <= ASCII_9; data++)
	{
		if(KEYPRESS) return;

		all_data = (data | data << 8 | data << 16 | data << 24);

		Set_LED(all_data);
  		
		DelayNUsec(LED_DELAY);
	}

	printf("Numeric LED Test completed!\n");
}


//
// Function: Set_Blinking_LED
//
// Description: Calls Set_LED but adds blinking capability
//
void Set_Blinking_LED(char* buffer, int blink_times, int ms)
{
	int i;

   //	Set_LED(buffer);
	for(i = 0; i < blink_times; i++)
	{
		// blank the display
		boardblank(1);
		DelayNUsec(1000);
		// turn on the display
		boardblank(0);
		DelayNUsec(1000);

	}
}

//
// Function: LED_Blink
//
// Description: Enabling and disabling a blank display on the LED with
//				a delay in between the two operations.
//
static void _LED_Blink(int us)
{
 		// blank the display
		boardblank(1);
		DelayNUsec(1000);
		// turn on the display
		boardblank(0);
		DelayNUsec(1000);
		
}

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_LED,"led",LED_Test,DIAGNOSTIC,"LEDTest","tests 4-digit led display");
Switch("s", BOOLEAN32ARG, "led lower case char test", "");
Switch("b", BOOLEAN32ARG, "led upper case char test", "");
Switch("n", BOOLEAN32ARG, "led numerical char test", "");
Switch("a", BOOLEAN32ARG, "all led tests", "");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_SETLED,"led_set",LED_Set_Test,DIAGNOSTIC,"LED Set Test","sends user data to 4-alpha-numerical LEDs");
Arg(CHAR32ARG,"led data","led data");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_SCROLL_OFF,"scroll_off",Scroll_Off,DIAGNOSTIC,"Scroll Off","Scrolling off");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_SCROLL_ON,"scroll_on",Scroll_On,DIAGNOSTIC,"Scroll On","Scrolling on");
CommandEnd();


  