/* mac.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ajp
 *  revision:
 * 
 * --------------------------------------------------------------------------
 */

//  Includes
#include "vm.h"		   
#include "cmd.h"
#include "mac.h"
#include "ixf1110.h"
#include "ixf1810.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"

//  Local Defines
#define IXF1810POSInitPortTblsz \
(sizeof IXF1810POSInitPortTbl/sizeof IXF1810POSInitPortTbl[0])

#define IXF1810LANInitPortTblsz \
(sizeof IXF1810LANInitPortTbl/sizeof IXF1810LANInitPortTbl[0])

struct MacDat MacDevTab[MAXMACDEV];		  

static struct MacDat MacDatTab[]=	
{
	{MACNone, 	    "No MAC Found", 0, 0, InitNullMac},
	{MACIXF1110, 	"IXF1110 10 Port 100/1000 Ethernet MAC", 10, 0xc6000000, InitIXF1110Mac},
  	{MACIXF1810, 	"IXF18101 10G MAC", 1, 0xc6000000, InitIXF1810Mac}
};

//*****************************************************************
// Function:    int MacScan(void* pUnused)
// Description: Performs MAC detection
// Input:		None
// Outputs:		Pass/Fail
//*****************************************************************

//Scan for available MAC devices
int MacScan(void)
{
	int i;
	

	//Clear Board types and set MAC locations to "No Mac found"
	for (i=0;i<MAXMACDEV;i++)
	{		
		MacDevTab[i] = MacDatTab[0];
	}

	printf("\nScanning for SF/Media boards...\n");

	for (i=0;i<MAXMACDEV;i++)
//	for (i=0;i<1;i++)
	{
		// Write zero's to possible board ID register areas prior ID search
		PUT32(MEDIA_BOARD_BASE+(i*MacIOSz), 0x0);

		// Read PLD's to determine the type of MAC device
//		if ((GET32(MEDIA_BOARD_BASE+SZ_1M) ) == BOARD_IXD02810)
		if ((GET32(MEDIA_BOARD_BASE+(i*MacIOSz)+SZ_1M) & 0xFFFFF) == BOARD_IXD02810)
		{
			MacDevTab[i]=MacDatTab[MACIXF1110];
			MacDevTab[i].IOBase += (i*MacIOSz); 
		}
		else if ((GET32(MEDIA_BOARD_BASE+(i*MacIOSz)+SZ_1M) & 0xFFFFF) == BOARD_IXD28192)  
//		if (GET32(MEDIA_BOARD_BASE+SZ_1M) == BOARD_IXD28192)  
	   	{
			MacDevTab[i]=MacDatTab[MACIXF1810];
			MacDevTab[i].IOBase += (i*MacIOSz); 													
		}
		else //Default to None
	   	{
			MacDevTab[i] = MacDatTab[MACNone];
		}
	}
	
	//printf("Found the following Media/SF devices:\n");
	for (i=0;i<MAXMACDEV;i++)
	{
		if((MacDevTab[i]).MacType!=MACNone)
		{
			if(!i)
				printf("Detected %s on media connector\n", (MacDevTab[i]).Descstr);
			else
				printf("Detected %s on fabric connector\n", (MacDevTab[i]).Descstr);
		}
		else
			if(!i)
				printf("No MSF adapter boards found on media connector\n");
			else
				printf("No MSF adapter boards found on fabric connector\n");
	}
}


//*****************************************************************
// Function:    void InitIXF1110Mac()
// Description: Initializes IXF1110 MAC device
// Input:		int MacNum, int internal_loopback
// Outputs:		None
//*****************************************************************
void InitIXF1110Mac(int MacNum, int internal_loopback, int POS_LAN_mode)
{
	int port, count;
	UINT32 diverse, link_bits;

	printf("Initializing the IXF1110 MAC\n");

	Georges_Egress_Train();
	DelayNUsec(100000);    


			// Enable training
			Egress_Train_On();
  			DelayNUsec(100000);    
			// Disable training 
			Egress_Train_Off();



	return;







	if(internal_loopback)
		{
		PUT32((MacDevTab[MacNum].IOBase + IXF1110_SPI4_DESKEW_REG),\
         (GET32(MacDevTab[MacNum].IOBase + IXF1110_SPI4_DESKEW_REG) | DESKEW_LPBK));
		}
	else
		{
		PUT32((MacDevTab[MacNum].IOBase + IXF1110_SPI4_DESKEW_REG),\
         (GET32(MacDevTab[MacNum].IOBase + IXF1110_SPI4_DESKEW_REG) & DESKEW_LPBK_MASK));
		}


	// clear the link leds			
	PUT32((MacDevTab[MacNum].IOBase + IXF1110_LINK_LED_ENABLE_REG), 0x0);

   	// Set CPU Interface Register to Big Endian Mode
	PUT32((MacDevTab[MacNum].IOBase + IXF1110_CPU_INTERFACE_REG), \
												IXF1110_BIG_ENDIAN);

	// Enable MAC LEDS in mode 0
	PUT32((MacDevTab[MacNum].IOBase + IXF1110_LED_CONTROL_REG), \
						(IXF1110_ENABLE_LED || IXF1110_LED_MODE_0) );

	// for each mac link status loop sets link led per port
	for(port=0; port<(MacDevTab[MacNum]).PortCnt;port++)	
	{
		// Read and save diverse config reg
		diverse = GET32(MacDevTab[MacNum].IOBase + \
			IXF1110_DIVERSE_CONFIG_WRITE_REG(port));
		// Enable Auto Negotiation
		PUT32((MacDevTab[MacNum].IOBase + IXF1110_DIVERSE_CONFIG_WRITE_REG(port)), \
										 (diverse || IXF1110_ENABLE_AUTO_NEG));

	count = 0;

	while (count < AUTO_NEG_MAX_COUNT)
		{
		DelayNUsec(10000);
		// Check if auto negotiate is complete
		if (GET32(MacDevTab[MacNum].IOBase + IXF1110_RX_CONFIG_WORD_REG(port))\
													 == IXF1110_AN_COMPLETE)
			break;
		else
			count++;
		}

	if (count < AUTO_NEG_MAX_COUNT)
	{
		//set link up bit for this port
		link_bits= GET32(MacDevTab[MacNum].IOBase + \
						IXF1110_LINK_LED_ENABLE_REG);

		PUT32((MacDevTab[MacNum].IOBase + IXF1110_LINK_LED_ENABLE_REG), \
						 (link_bits || IXF1110_LINK_LED_ENABLE(port)));
	}

	// Disable Auto Negotiation
	PUT32((MacDevTab[MacNum].IOBase + IXF1110_DIVERSE_CONFIG_WRITE_REG(port)),\
									(diverse & (!IXF1110_ENABLE_AUTO_NEG)));
	}

	// Enable the 2810 int handler
	ixd2810IntDevInit();   
}

// POS Mode Programming sequence
struct CsrAddDat IXF1810POSInitPortTbl[] = 
	{
	IXF1810_GLB_RST,		 0x000F, //Reset SPI, LINE, APS, All Int Reg	
	IXF1810_GLB_CFG1,		 0x0002,
	IXF1810_GLB_CLKCFG1,	 0xA000,  
	IXF1810_GLB_CLKCFG2,	 0x0003,  
	IXF1810_GLB_LINE,		 0x0004, //Invert clk polarity of Receive Line Clk 	
	IXF1810_GLB_SPI4,		 0x0000, //SPI-4 Rx Clk use SPI4REFCLK RDCLK src
	IXF1810_GLB_RST,		 0x0000, //Disable Reset SPI, LINE, APS, All Int Reg
	0x3300,                  0x047D,
	0x3330,                  0x004D,
	0x11C1,                  0x0001,
	0x1821,					 0x0000,
	0x1841,                  0x0003,
	0x1842,                  0x0002,
	0x1843,                  0x0002,
	0x1832,                  0x0001,
	0x1833,                  0x0000,
	0x1882,                  0x0000
	}; 
//	0x11C1,                  0x0001, //internal
//	0x11C1,                  0x0000, //external

// LAN Mode Programming sequence
struct CsrAddDat IXF1810LANInitPortTbl[] = 
	{
	IXF1810_GLB_RST,		 0x000F, //Reset SPI, LINE, APS, All Int Reg	
	IXF1810_GLB_CFG1,		 0x0005,
	IXF1810_GLB_CLKCFG1,	 0xA000,  
	IXF1810_GLB_CLKCFG2,	 0x0003,  
	IXF1810_GLB_LINE,		 0x0004, //Invert clk polarity of Receive Line Clk 	
	IXF1810_GLB_SPI4,		 0x0000, //SPI-4 Rx Clk use SPI4REFCLK RDCLK src
	IXF1810_GLB_RST,		 0x0000, //Disable Reset SPI, LINE, APS, All Int Reg
	0x300e,                  0x000a,
	0x300f,                  0x0474,
	0x3200,                  0x2840,
	0x3200,                  0x2040,
	0x3200,                  0xa040,
	0x3116,					 0x0000,
	0x3117,                  0x0084,
	0x3100,                  0x7fff,
	0x3101,                  0x3804,
	0x3114,                  0x0000,
	0x3115,                  0x2600 
	}; 
//	IXF1810_RX_POL_REG,		 0xffff //Inverts RX SPI-4 polarity




//*****************************************************************
// Function:    void InitIXF1810Mac()
// Description: Initializes IXF1810 MAC device
// Input:		int MacNum, int internal_loopback
// Outputs:		None
//*****************************************************************

void InitIXF1810Mac(int MacNum, int internal_loopback, int POS_LAN_mode)
{
	int	i, element_cnt;

	struct CsrAddDat *pcsrtbl;

	// The PLD sets the Mode Balls on the IXF1810 so we need to talk to 
	// the PLD and program this to the proper mode BEFORE we program the 
	// IXF1810_GLB_CFG1 register here


	if(POS_LAN_mode)
	{
		if (internal_loopback)
			{
			printf("Initializing the IXF18101 MAC for internal loopback LAN mode\n");
			*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c688;
			}
		else
			{
			printf("Initializing the IXF18101 MAC for external loopback LAN mode\n");
			*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c698;
			}

//debugit		//Set LAN mode strap options
//debugit//		*(volatile unsigned int *) (0xc6200000) =  0x0000c688;
//debugit		*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c688;

 
		element_cnt=IXF1810LANInitPortTblsz;
		pcsrtbl=IXF1810LANInitPortTbl;
	}
	else
	{
		if (internal_loopback)
			{
			printf("Initializing the IXF18101 MAC for internal loopback POS mode\n");
			*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c0e8;
			}
		else
			{
			printf("Initializing the IXF18101 MAC for external loopback POS mode\n");
			*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c0f8;
			}

//debugit		//Set POS mode strap options
//debugit//		*(volatile unsigned int *) (0xc6200000) =  0x0000c0e8;
//debugit		*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0x200000) =  0x0000c0e8;
		element_cnt=IXF1810POSInitPortTblsz;
		pcsrtbl=IXF1810POSInitPortTbl;
	}

 	//csr table loop
 	for(i=0;i<element_cnt;i++)						
 	{
 		//printf("ADDR = 0x%X, DATA = 0x%X\n",(MacDevTab[MacNum].IOBase + (pcsrtbl[i].CsrAddr <<2)), pcsrtbl[i].CsrData);
 		*(volatile unsigned int *)(MacDevTab[MacNum].IOBase + (pcsrtbl[i].CsrAddr <<2)) = pcsrtbl[i].CsrData;
	}

	if (internal_loopback)
	{
	//Set Calypso SPI-4 FIFO internal Loopback bit
	//	*(volatile unsigned int *) (0xc600c034) =  0x00000001;
	*(volatile unsigned int *) (MacDevTab[MacNum].IOBase + 0xc034) =  0x00000001;
	}
	
	// Program the WIS(WAN Interface Sub-layer)
//	if (internal_loopback)
//		PUT16((MacDevTab[MacNum].IOBase + IXF1810_WISLOOP_CFG), \
//							IXF1810_mapper_test_loop_enable);
//	else
//		PUT16((MacDevTab[MacNum].IOBase + IXF1810_WISLOOP_CFG), \
//							IXF1810_mapper_test_loop_disable);


			// Enable training
			Egress_Train_On();
  			DelayNUsec(100000);    
			// Disable training 										 
			Egress_Train_Off();


}
				


//*****************************************************************
// Function:    void InitNullMac()
// Description: No MAC to initialize
// Input:		int MacNum
// Outputs:		None
//*****************************************************************
void InitNullMac(int MacNum, int internal_loopback, int POS_LAN_mode)
{
	printf("No MAC %d to initialize\n", MacNum);
}





