/* media_loopback_rx.uc	   
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *																					 
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ajp, April 02
 *  revisions:
 * 
 * 01a, 09/04/03 ajp Added handshake code between tx and rx ucode files
 * 
 * --------------------------------------------------------------------------
 */

#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "stdmac.uc"
#include "pci_dma.h"

.reg $SW0 $SW1 $SW2   
.reg $SR0 $SR1 $SR2 $SR3 $SR4 $SR5 $SR6 $SR7 $SR8 $SR9 $SR10 $SR11 $SR12 $SR13 $SR14 $SR15
.xfer_order $SW0 $SW1 $SW2  
.xfer_order $SR0 $SR1 $SR2 $SR3 $SR4 $SR5 $SR6 $SR7 $SR8 $SR9 $SR10 $SR11 $SR12 $SR13 $SR14 $SR15 


.reg shft_Mac rxpacketdata BoardType
.reg testparams MacNum NumMacPorts PktCount  
.reg MacPort AddrOfData save_element element temp testpkts_addr
.reg MacPortCounter MsfAddress
.reg pkt_data0 pkt_data1 pkt_data2 pkt_data3
.reg pkt_data4 pkt_data5 pkt_data6 pkt_data7
.reg pkt_data8 pkt_data9 pkt_data10 pkt_data11
.reg pkt_data12 pkt_data13 pkt_data14 pkt_data15
.reg freelist_data freelist_addr mask rx_addr
.reg address data_reg pattern
.reg mailbox0 mailbox3 pci_base
.reg bad_packets

.sig SIG_1 	SIG_7					
.addr SIG_1 1
.addr SIG_7 7
.sig scratch_write 

//	Start of SPI-4 receive ucode

	// Only use context 0
	br=ctx[0, thread_0#]	; if thread 0 continue
	ctx_arb[kill]			; not thread 0,just kill

thread_0#:
	immed[bad_packets, 0]		
	// HWM_CONTROL
//	immed32[temp_reg, HIGHWATER]
//  	alu[$SR0,	--, B, temp_reg]
//	alu[address,	--, B, HWM_CNTL_OFF]
//	msf[write, $SR0, address, 0, 1], ctx_swap[SIG_1]
		
	// Read test parameters from memory
	immed32[testparams, TESTPARAMETERS]	
	scratch[read, $SR0, testparams, 0, 4], sig_done[SIG_1]
	ctx_arb[SIG_1]

	// get test data from memory	
	alu[MacNum,		 --, B, $SR0]
	alu[NumMacPorts, --, B, $SR1]
	alu[PktCount,	 --, B, $SR2]
	alu[BoardType,	 --, B, $SR3]

	// get the 1st 32 bytes of tx/rx data
	immed32[testpkts_addr, TXRXPACKETDATA]
	scratch[read, $SR0, testpkts_addr, 0, 8], sig_done[SIG_1]
	ctx_arb[SIG_1]

	// now save this data in pkt_data0-7
	alu[pkt_data0,   --, B, $SR0]
	alu[pkt_data1,   --, B, $SR1]
	alu[pkt_data2,   --, B, $SR2]
	alu[pkt_data3,   --, B, $SR3]
	alu[pkt_data4,   --, B, $SR4]
	alu[pkt_data5,   --, B, $SR5]
	alu[pkt_data6,   --, B, $SR6]
	alu[pkt_data7,   --, B, $SR7]

	//get the 2nd 32 bytes of tx/rx data 
	immed32[testpkts_addr, TXRXPACKETDATA]
	scratch[read, $SR0, testpkts_addr, 32, 8], sig_done[SIG_1]
	ctx_arb[SIG_1]

    // now save this data in pkt_data8-15
	alu[pkt_data8,   --, B, $SR0]
	alu[pkt_data9,   --, B, $SR1]
	alu[pkt_data10,  --, B, $SR2]
	alu[pkt_data11,  --, B, $SR3]						 
	alu[pkt_data12,  --, B, $SR4]
	alu[pkt_data13,  --, B, $SR5]
	alu[pkt_data14,  --, B, $SR6]
	alu[pkt_data15,  --, B, $SR7]

get_next_packet#:
	// start the port counter at zero so that we do NumMacPorts small loops
	// before returning here. Allows code to know when it is finished
	immed[MacPortCounter, 0]

	// Allow the tx ucode to tx next packet
	immed32[$SR0, 0x88888888]
	immed[mailbox3, MAILBOX_3_OFF]										  
	immed32[pci_base, IXP2000_PCI_CSR_BASE]
	pci[write, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

small_loop#:

	// write Rx_Thread_Freelist_# with signal7, me0, thread0, $SR0
	immed[temp, FREELIST_INFO]		
	alu[freelist_data, --, B, temp]
	alu[freelist_addr, --, B, RX_THREAD_FREELIST_0_OFF]
	alu[$SW2, --, B,	freelist_data]
	msf[write, $SW2, freelist_addr, 0, 1], ctx_swap[SIG_1]	


	//Wait for signal7 defined in rx_thread_freelist
	ctx_arb[SIG_7] 

//
//		 31					RBUF Element Status					       0
//		+---------------+---------------+---------------+---------------+
//		|P| element #   |   byte count  |S|E|E|L|P|A|N|T| Port Address  |
//		+---------------+---------------+---------------+---------------+
//		31		RPROT (Protocol SPI-4 vs CSIX.  For SPI-4 will be 0)
//		30#:24	RBUF Element Number
//		23#:16	Byte Count
//		15		SOP
//		14		EOP
//		13		Error Summary
//		12		Length Error
//		11		Parity Error
//		10		Abort Error
//		9		Null
//		8		Type
//		7#:0		Port Address
//

	//  waiting for RBUF to fill
	// $SR0 should receive the status word
	// bits [30#:24] of status is the element number

	// get element #
	immed[mask, 0x7F]
	alu_shf[temp, -- , B, $SR0, >>24]
	alu[element, temp, AND, mask]
	alu[save_element, temp, AND, mask]

	// get MAC port #
	immed32[mask, 0x0F]
	alu[MacPort, $SR0, AND, mask]								

	// Parse into RBUF[element times 64]

	// Start element incremented by 1
	alu[element, element, +, 1]
    // start addrOfData pointing at top of RBUF
	immed[addrOfData, RBUF_TBUF_OFF]
find_addr#:
	// decrement the element number
	alu[element, element, -, 1] 															 
	// if element is zero we have the correct addrofData
	BEQ[get_rbuf_data#]
		
	// each element is 64 bytes into RBUF so bump address
	alu[addrOfData, addrOfData, +, 64] 
	br [find_addr#]

get_rbuf_data#:

	//	Read 1st 32 bytes of RBUF data 
	msf[read, $SR0, addrOfData, 0, 8], ctx_swap[SIG_1]
	//	Read 2nd 32 bytes of RBUF data 
	msf[read, $SR8, addrOfData, 32, 8], ctx_swap[SIG_1]

	alu[pattern, --, B, $SR0]
	alu[$SR0, --, B, pattern]
	alu[pattern, --, B, $SR1]
	alu[$SR1, --, B, pattern]
	alu[pattern, --, B, $SR2]
	alu[$SR2, --, B, pattern]
	alu[pattern, --, B, $SR3]
	alu[$SR3, --, B, pattern]
	alu[pattern, --, B, $SR4]
	alu[$SR4, --, B, pattern]
	alu[pattern, --, B, $SR5]
	alu[$SR5, --, B, pattern]
	alu[pattern, --, B, $SR6]
	alu[$SR6, --, B, pattern]
	alu[pattern, --, B, $SR7]
	alu[$SR7, --, B, pattern]
	alu[pattern, --, B, $SR8]
	alu[$SR8, --, B, pattern]
	alu[pattern, --, B, $SR9]
	alu[$SR9, --, B, pattern]
	alu[pattern, --, B, $SR10]
	alu[$SR10, --, B, pattern]
	alu[pattern, --, B, $SR11]
	alu[$SR11, --, B, pattern]
	alu[pattern, --, B, $SR12]
	alu[$SR12, --, B, pattern]
	alu[pattern, --, B, $SR13]
	alu[$SR13, --, B, pattern]
	alu[pattern, --, B, $SR14]
	alu[$SR14, --, B, pattern]
	alu[pattern, --, B, $SR15]
	alu[$SR15, --, B, pattern]

	//	All 64 bytes of RBUF data are compared to expected
	alu[--, $SR0, -, pkt_data0]
	bne[bad_packet#]											 
	alu[--, $SR1, -, pkt_data1]
	bne[bad_packet#]
	alu[--, $SR2, -, pkt_data2]
	bne[bad_packet#]
	alu[--, $SR3, -, pkt_data3]
	bne[bad_packet#]
	alu[--, $SR4, -, pkt_data4]
	bne[bad_packet#]
	alu[--, $SR5, -, pkt_data5]
	bne[bad_packet#]
	alu[--, $SR6, -, pkt_data6]
	bne[bad_packet#]
	alu[--, $SR7, -, pkt_data7]
	bne[bad_packet#]
	alu[--, $SR8, -, pkt_data8]
	bne[bad_packet#]
	alu[--, $SR9, -, pkt_data9]
	bne[bad_packet#]
	alu[--, $SR10, -, pkt_data10]
	bne[bad_packet#]
	alu[--, $SR11, -, pkt_data11]
	bne[bad_packet#]
	alu[--, $SR12, -, pkt_data12]
	bne[bad_packet#]
	alu[--, $SR13, -, pkt_data13]
	bne[bad_packet#]

good_packet#:
	// Point to good rx packets in memory
	immed32[rx_addr, RXPACKETCOUNT]
	br [write_rx_cnt#]

bad_packet#:
	immed[bad_packets, 1]

	// Point to scratch save area for debug	use
	immed32[rxpacketdata, RXPORTDATA]	// uses offset 0x200 in scratch

	// shift to appropriate port's scratch area(8 words per port are saved)
	alu[shft_Mac, --, B, MacPort, <<6]
	alu[rxpacketdata, rxpacketdata,+, shft_Mac]

	// Now save 1st 32 bytes to scratch so that xscale can display it
	scratch[write, $SR0, rxpacketdata, 0, 8], sig_done[SIG_1]
	ctx_arb[SIG_1]
	// Now save 2nd 32 bytes to scratch so that xscale can display it
	scratch[write, $SR8, rxpacketdata, 32, 8], sig_done[SIG_1]
	ctx_arb[SIG_1]															   

	// Point to bad rx packets in memory
	immed32[rx_addr, RXPACKETERR]


write_rx_cnt#:
	// shift to appropriate port's scratch area
	alu[shft_Mac, --, B, MacPort, <<2]
	alu[rx_addr,rx_addr,+, shft_Mac]

 	// read data from scratch and increment it
	scratch[incr, --, rx_addr, 0]

	// Initialize RBUF Freelist to add elements to the list
  	immed[MsfAddress, RBUF_ELEMENT_DONE]
	alu[$SR0, --, B, save_element]
	msf[write, $SR0, MsfAddress, 0, 1], ctx_swap[SIG_1]

//	// Allow the tx ucode to tx next packet
//	immed32[$SR0, 0x88888888]
//	immed[mailbox3, MAILBOX_3_OFF]										  
//	immed32[pci_base, IXP2000_PCI_CSR_BASE]
//	pci[write, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

	// Received data from another Mac port; is it the last one
	// Bump the port counter that started at 0
	alu[MacPortCounter, MacPortCounter, +, 1]
	// compare port counter to number of MAC ports
	alu[--, NumMacPorts, -, MacPortCounter]
	// if not done don't reset the MacPortCounter
	BNE[small_loop#]

	// Initialize RBUF Freelist to add elements to the list
	//  	immed[MsfAddress, RBUF_ELEMENT_DONE]
	//	immed[temp_reg, 0]
	//init_RBUF#:
	//	alu[$SR0, --, B, temp_reg]
	//	msf[write, $SR0, MsfAddress, 0, 1], ctx_swap[SIG_1]
	//	alu[temp_reg, temp_reg, +, 1]
	//	alu[--, MAX_ELEM_COUNT, -, temp_reg]
	//	bne[init_RBUF#]

	// check if we had any errors and stop if so
	.if (bad_packets == 1)
		br[EndOfTest#]
//		br[StopTest#]
	.endif


	// If here have received 64 bytes of data from all Mac ports						 
	// so decrement the packet counter and see if we are done
	alu[PktCount, PktCount, -, 1]
	BNE[get_next_packet#]



StopTest#:
	// Inform XScale that all expected packets were received
	// Write packets rx'd message to slave mailbox0
	immed[$SR5, DM_PACKETSRCVD]
	immed[mailbox0, MAILBOX_0_OFF]										  
	immed32[pci_base, IXP2000_PCI_CSR_BASE]
	pci[write, $SR5, pci_base, mailbox0, 1], ctx_swap[SIG_1]	
	
EndOfTest#:
	nop
	nop
	nop
	nop
	nop
	ctx_arb[kill]
