/* media_loopback_tx.uc	   
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ajp, April 02
 *  revisions:
 * 
 * 01a, 09/04/03 ajp Added handshake code between tx and rx ucode files
 * 
 * --------------------------------------------------------------------------
 */

#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h" 
#include "stdmac.uc"
#include "pci_dma.h"
.reg mailbox3 allowtx pci_base
.reg $SW0 $SW1 $SW2 $SW3 $SW4 $SW5 $SW6 $SW7 $SW8 
.reg $SR0 $SR1 $SR2 $SR3 $SR4 $SR5 $SR6 $SR7 $SR8 
.xfer_order $SW0 $SW1 $SW2 $SW3 $SW4 $SW5 $SW6 $SW7 
.xfer_order $SR0 $SR1 $SR2 $SR3 $SR4 $SR5 $SR6 $SR7 
.reg testparams MacNum NumMacPorts PktCount testpkts_addr debug_addr
.reg MacPort AddrOfData addrOfControl 
.reg MacAddr tbuf_control temp exheader
.reg MacPortCounter
.reg tx_pkt_cnt_addr ele_cntr shft_Mac
.reg pkt_data0 pkt_data1 pkt_data2 pkt_data3
.reg pkt_data4 pkt_data5 pkt_data6 pkt_data7
.reg pkt_data8 pkt_data9 pkt_data10 pkt_data11
.reg pkt_data12 pkt_data13 pkt_data14 pkt_data15
.reg sequence temp_reg address

.sig SIG_1 
.sig scratch_write 

//	Start of SPI-4 transmit ucode

	// Only use thread 0 
	br=ctx[0, thread_0#]
	ctx_arb[kill]			

thread_0#:

	// Read test parameters from memory
	immed32[testparams, TESTPARAMETERS]
	scratch[read, $SR0, testparams, 0, 3], sig_done[SIG_1]
	ctx_arb[SIG_1]

	// Save the test data from memory	
	alu[MacNum,		 --, B, $SR0]
	alu[NumMacPorts, --, B, $SR1]
	alu[PktCount,	 --, B, $SR2]

	//get the 1st 32 bytes of tx/rx data 
	immed32[testpkts_addr, TXRXPACKETDATA]
	scratch[read, $SR0, testpkts_addr, 0, 8], sig_done[SIG_1]	 
	ctx_arb[SIG_1]

	 // now save this data in pkt_data0-7
	alu[pkt_data0,   --, B, $SR0]
	alu[pkt_data1,   --, B, $SR1]
	alu[pkt_data2,   --, B, $SR2]
	alu[pkt_data3,   --, B, $SR3]
	alu[pkt_data4,   --, B, $SR4]
	alu[pkt_data5,   --, B, $SR5]
	alu[pkt_data6,   --, B, $SR6]
	alu[pkt_data7,   --, B, $SR7] 

	//get the 2nd 32 bytes of tx/rx data 
	immed32[testpkts_addr, TXRXPACKETDATA]
	scratch[read, $SR0, testpkts_addr, 32, 8], sig_done[SIG_1]
	ctx_arb[SIG_1]

 	// now save this data in pkt_data8-15
	alu[pkt_data8,   --, B, $SR0]
	alu[pkt_data9,   --, B, $SR1]
	alu[pkt_data10,  --, B, $SR2]
	alu[pkt_data11,  --, B, $SR3]
	alu[pkt_data12,  --, B, $SR4]
	alu[pkt_data13,  --, B, $SR5]
	alu[pkt_data14,  --, B, $SR6]
	alu[pkt_data15,  --, B, $SR7]

	// start with ele_cntr set to 128
	immed[ele_cntr, MAX_ELEM_COUNT]

	// start pointing to first element in the TBUF
	immed[addrOfData, RBUF_TBUF_OFF]	

	// start pointing to start of TBUF_ELEMENT_CONTROL register
	immed[addrOfControl, TBUF_ELEMENT_CONTROL_OFF]

	// Allow tx ucode to send 1st packet by write to mailbox 3
	immed32[$SR0, 0x88888888]
	immed[mailbox3, MAILBOX_3_OFF]										  
	immed32[pci_base, EGRESS_INGRESS_MAILBOX]
	pci[write, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

large_loop#:
													  
	// Start with Mac port 0
	immed[MacPort, 0]
   .reg train_data_addr
	immed[train_data_addr, 0xa0]

	.reg $train_data
	.sig train_data_sig

	immed[$train_data, 0x200]
	msf[write, $train_data, train_data_addr, 0, 1], ctx_swap[train_data_sig]


small_loop#:

wait_for_rx#:
	// Check scratch location before allowing tx ucode to send packets
	immed[mailbox3, MAILBOX_3_OFF]										  
	immed32[pci_base, EGRESS_INGRESS_MAILBOX]
	pci[read, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

	alu[allowtx, --, B, $SR0]
	alu[$SR0, --, B, allowtx]

	immed32[allowtx, 0x88888888]
	alu[$SR0, $SR0, -, allowtx]
	bne[wait_for_rx#]


	// Now don't allow more tx packets until rx code resets scratch location
//	immed32[$SR0, 0x77777777]
//	immed[mailbox3, MAILBOX_3_OFF]										  
//	immed32[pci_base, EGRESS_INGRESS_MAILBOX]
//	pci[write, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

	// Send data to all MAC ports incrementing element # 
	
    //write 1st 32 bytes to TBUF
	alu[$SW0, --, B,	pkt_data0]
	alu[$SW1, --, B,	pkt_data1]
	alu[$SW2, --, B,	pkt_data2]
	alu[$SW3, --, B,	pkt_data3]
	alu[$SW4, --, B,	pkt_data4]
	alu[$SW5, --, B,	pkt_data5]
	alu[$SW6, --, B,	pkt_data6]
	alu[$SW7, --, B,	pkt_data7]												   
	msf[write, $SW0, addrOfData, 0, 8], ctx_swap[SIG_1]	
															
    //write 2nd 32 bytes to TBUF
	alu[$SW0, --, B,	pkt_data8]
	alu[$SW1, --, B,	pkt_data9]
	alu[$SW2, --, B,	pkt_data10]			
	alu[$SW3, --, B,	pkt_data11]
	alu[$SW4, --, B,	pkt_data12]
	alu[$SW5, --, B,	pkt_data13]
	alu[$SW6, --, B,	pkt_data14]
	alu[$SW7, --, B,	pkt_data15]
	msf[write, $SW0, addrOfData, 32, 8], ctx_swap[SIG_1]				   

	immed[MacNum, 0]
	// Build Tx Element Control Word & Extension Header 
 	alu_shf[MacAddr, --, B, MacNum, <<3]
    alu[MacAddr, MacAddr, +, MacPort]

	immed32[tbuf_control, TX_ELM_WD]		
  	alu[tbuf_control, tbuf_control, +, MacAddr]
	immed[temp, SOP]	
	alu[tbuf_control, tbuf_control, +, temp] 
	immed[temp, EOP]	
	alu[tbuf_control, tbuf_control, +, temp] 

	//element extension header
	immed32[exheader, TX_ELM_EX_WD]
  		
	//add element number as offset into TBUF_ELEMENT_CONTROL register
//	immed[addrOfControl, TBUF_ELEMENT_CONTROL]
//	alu[addrOfControl, addrOfControl, +, element]	

	//element control word	
	alu[$SW2, --, B, tbuf_control]	

	//element extension header
	alu[$SW3, --, B, exheader]		

	// Validate tx buffer data by writing the corresponding element control word
	msf[write, $SW2, addrOfControl, 0, 2], ctx_swap[SIG_1]


	//have transmited a packet, write this info to scratch
	immed32[tx_pkt_cnt_addr, TXPACKETCOUNT]
	alu[shft_Mac, --, B, MacPort, <<2]
	alu[tx_pkt_cnt_addr,tx_pkt_cnt_addr,+, shft_Mac]

	// read data from scratch and increment it
	scratch[incr, --, tx_pkt_cnt_addr, 0]
	
	// dummy read to sync workbench data
//	scratch[read, $SR7, tx_pkt_cnt_addr, 0, 1], sig_done[SIG_1]
//	ctx_arb[SIG_1]

	// manually increment element number
	// alu[element, element, +, 1] 

	// 127 64 byte locations
	alu[ele_cntr, ele_cntr, -, 1]
	bne[keep_inc#]

	// have written all 128 elements so element count goes to MAX_ELEM_COUNT
	immed[ele_cntr, MAX_ELEM_COUNT]

	// point to start of TBUF
	immed[addrOfData, RBUF_TBUF_OFF] 

	// point to start of TBUF_ELEMENT_CONTROL register
	immed[addrOfControl, TBUF_ELEMENT_CONTROL_OFF]

	br[next_port#]


keep_inc#:
	// increment the TBUF address pointer
	alu[addrOfData, addrOfData, +, 64] 

	// increment the element control word pointer (each control is 2 LW long)
	alu[addrOfControl, addrOfControl, +, 8] 

next_port#:
	// increment MacPorts then check if equal to the NumMacPorts
	alu[MacPort, MacPort, +, 1]
	alu[$SW0, NumMacPorts, -, MacPort]
	bne[small_loop#]

	// Now don't allow more tx packets until rx code resets scratch location
	immed32[$SR0, 0x77777777]
	immed[mailbox3, MAILBOX_3_OFF]										  
	immed32[pci_base, EGRESS_INGRESS_MAILBOX]
	pci[write, $SR0, pci_base, mailbox3, 1], ctx_swap[SIG_1]	

	// If here have transmitted 64 bytes of data to all Mac ports
	// so decrement the packet counter and see if we are done
	alu[PktCount, PktCount, -, 1]
	BNE[large_loop#]
	
  
 EndOfTest#:
	nop
	ctx_arb[kill]
	nop
	nop
	nop
	nop
	br[EndOfTest#]
	nop
	nop
	nop
	nop
