/* media_test.c					 		   	  					 
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                     
 *                                                                      		   	
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          		   	 
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      		   				 
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    										  
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    				   
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            	 
 *                                                                     				  
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                 
 *                                                                      											  
 *---------------------------------------------------------------------------
 *
 *																   						  
 *  system: IXDP2800
 *  subsystem: DIAG						   
 *  author: ajp, oct 2002
 *  
 * 
 *
 * Revisions:
 * 01f, 09/25/03 ajp Print proper port number during a failure
 * 01e, 09/18/03 ajp Modified the mailbox message timeout so that extremely long pkt cnts can be sent
 * 01d, 09/11/03 ajp Changed the msf_clock_speed routines so that tx is slower than rx clk
 * 01c, 09/08/03 ajp Saves rx packet error data to scratch
 * 01b, 09/04/03 ajp Report expected/received data on failure, added new tx/rx ucode files
 * 01a, 08/18/03 ajp Added pass/fail returns for script file break on error
 * 
 * --------------------------------------------------------------------------
 */

															  
#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2x00.h"
#include "uc_load.h"
#include "mac.h"
#include "media_loopback_tx.uh"
#include "media_loopback_rx.uh"				 
#include "media_loopback_tx2.uh"
#include "media_loopback_rx2.uh"				 
#include "media_loopback_tx3.uh"
#include "media_loopback_rx3.uh"				 
#include "pci_diag.h"
#include "pci_dma.h"						   			
#include "diag_utils.h"


////////////////////////////////////////////////////					   
// Definitions for Media_test Diagnostic
////////////////////////////////////////////////////
extern Sf_Loopback_Test(void);

extern struct MacDat MacDevTab[MAXMACDEV];				

UINT32 GET_OTHER_NPUs_PCI_CSR_BASE(UINT32 npu_sel);
UINT32 GET_OTHER_NPUs_PCI_SRAM_BASE(UINT32 npu_sel);
////////////////////////////////////////////////////			 
// Input Structure for Diagnostic
////////////////////////////////////////////////////
void Msf_Clock_Speed_Tx(void);
void Msf_Clock_Speed_Rx(void);

																		
// User input data											   				 
typedef struct _Media_testIn									 				
{
	UINT8	MacNum;      //User entered MAC #
	UINT32	PktCnt;      //Packet size
	UINT32  fabric_sel;  //Look for fabric card
	UINT32  Int_Ext_sel; // Internal or external loopback mode
	UINT32  POS_LAN_sel; // Selects POS or LAN mode with POS being the default

} Media_testIn;

typedef struct _Tx_data_testIn		   
{
	UINT32	TxDataIn; 

} Tx_data_testIn;	



// 64 bytes of data to be transmitted to all mac ports		
UINT32 macportdata[] = 
	{
	0x00000001, 0x40000002, 0x80000004, 0x00000008,
	0x00000010, 0x00000020, 0x00000040, 0x00000080,
	0x00000100, 0x00000200, 0x00000400, 0x00000800,
	0x00001000, 0x00002000, 0x00004000, 0x00008000,
	0x00010000, 0x00020000, 0x00040000, 0x00080000,
	0x00100000, 0x00200000, 0x00400000, 0x00800000,
	0x01000000, 0x02000000, 0x04000000, 0x08000000,
	0x10000000, 0x20000000, 0x40000000, 0x80000000,
	};



//*****************************************************************
// Function:    int Media_test(Media_testIn* pIn,void* pUnused)
// Description: Performs Media Loopback test on Master
// Input:		Mac Number and Packet count
// Outputs:		Pass/Fail
//*****************************************************************
int Media_Loopback_Test(Media_testIn* pIn, PassFail_testOut* pOut)
{
	UINT32 x, i, status, final_status = PASS;			 
	UINT32 Tx_Pkts;
	UINT32 Rx_Pkts;
	UINT32 Rx_Errs;									 
	UINT32 OTHER_NPUs_PCI_CSR_BASE;				   
	UINT32 Egress_msf_Scratch_addr;
	UINT32 Ingress_msf_Scratch_addr;
	UINT dcache_status;						   
	pOut->PassFail = FAIL;
	pOut->ReturnCode = PASS;         
	status = PASS;

	Scroll_Off();

	//*****************************************************************
	// If here we are data packet transmitter (Egress)
	//*****************************************************************
	if(IS_EGRESS_NPU)
	{
		// If this is egress and it's a fabric loopback(SPI-4)
		if (pIn->MacNum == 1)
			{																		   
			pOut->PassFail = Media_Loopback_Test2(pIn);
			pOut->ReturnCode = PASS;         
        
			return(pOut->PassFail);
			}

		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}
		
		OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

#ifndef NO_DOORBELL
		// Tell Ingress to jump to media_test_slave function
		Set_Pci_Doorbell(MEDIA_LB_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
#endif
		Egress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

		//Get the Ingress' Scratch start addr 
	    Ingress_msf_Scratch_addr = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);
		Ingress_msf_Scratch_addr += SCRATCH_CSR_BASE_FRM_PCI;
																				 
		// Get user requested MAC # and do range check										  
		if((pIn->MacNum) >= MAXMACDEV)														
		{					
			printf("Mac number specified is invalid\n");
			return(pOut->PassFail);
		}

		//Zero out the Egress mailbox register data
		ZeroEgressMailboxes();

		//Zero out the Ingress mailbox register data
		ZeroIngressMailboxes();

		// Initialize the Egress to allow SPI-4 tx Egress-Ingress
		Egress_Init();
									   
		if (!pIn->fabric_sel) 
		{ 	
			// PCI_Master does MAC Scan 
			MacScan();

			if(MacDevTab[pIn->MacNum].MacType==MACNone)
			{ 	
				printf("No Mac %d detected, Media test aborted\n", pIn->MacNum);	  
				Cleanup_before_return();
				return(pOut->PassFail);
			}
		}
		else
		{
		// Force to 1 port for fabric test 
		MacDevTab[0].PortCnt = 1;
		MacDevTab[1].PortCnt = 1;													 
 		}
	  
		// Test start message to 
		printf("Starting MAC %d Media loopback test\n", pIn->MacNum);

		if (!pIn->fabric_sel) 
		{
		    // Calls function to init the MAC CSR's for SPI-4 input/output 
		    MacDevTab[pIn->MacNum].MacInitFunc(pIn->MacNum, pIn->Int_Ext_sel, pIn->POS_LAN_sel);
		}


		// Stuff the fabric or media selection into mailbox 1 before starting ingress
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_1) = pIn->fabric_sel;


		// Stuff the media board type into mailbox 2 before starting ingress
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_2) = MacDevTab[pIn->MacNum].MacType;


		// Send mailbox msg (DM_Ingress_Start) to Ingress								
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_INGRESS_START;

		if (pIn->fabric_sel) 
		{ 	
		// Enable training 
		Egress_Train_On();
		}
		// Wait for a signal that slave NPU has training
		if((egress_wait_mailbox_msg(3000, SWAP32(DM_INGRESS_TRAIN_COMPLETE))) == FAIL)
		{						
			printf("Timed out...Master NPU did not rx DM_INGRESS_TRAIN_COMPLETE\n");
			Cleanup_before_return();
			return(pOut->PassFail);
		}								  
																				  
		if (pIn->fabric_sel) 
		{ 	
		// Disable training 
		Egress_Train_Off();
		}

		// Clear Egress and Ingress scratch memory areas for media test storage
		for(i=0; i < XS_MEDIA_STORAGE_SIZE; i+=4)		 
		{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + i) = 0;
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + i) = 0;
		}
					   
		// Requested MAC port # to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACNUMBER) = pIn->MacNum;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACNUMBER) = SWAP32(pIn->MacNum);
														  																  
		// Port count goes to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = MacDevTab[pIn->MacNum].PortCnt;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = SWAP32(MacDevTab[pIn->MacNum].PortCnt);
	
		// MAC type goes to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) = MacDevTab[pIn->MacNum].MacType;
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) |= ((pIn->Int_Ext_sel & 0x1)<<4);
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACTYPE) = SWAP32(MacDevTab[pIn->MacNum].MacType);
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACTYPE) |= ((pIn->Int_Ext_sel & 0x1)<<4);


		// User entered packet count
		if (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) == 2)
			{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = (pIn->PktCnt + 1);
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(pIn->PktCnt);
			}
		else
			{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = pIn->PktCnt;
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(pIn->PktCnt);
			}
	
		// write 64 byte transmit packet data to Egress memory for ucode to use
		for (i=0; i < UENG_CNT; i++)													
		{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = macportdata[i];
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = SWAP32(macportdata[i]);
		}
																										

		// Send mailbox msg (DM_Init_Ingress_Scratch) (Scratch init complete) to Ingress								
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_INIT_INGRESS_SCRATCH;
																																	
		// Delay for Ingress (DM RxRdy) message; warning message if not rx'd
		if((egress_wait_mailbox_msg(0x300, SWAP32(DM_INGRESS_RXRDY))) == FAIL)
		{
			Cleanup_before_return();
			printf("Timed out...Master NPU did not rx DM_INGRESS_RXRDY\n");
			return(pOut->PassFail);																		   						   				 
		}

		// Reset the ueng's
		Clear_All_ME();
																										 
		// Load transmit ucode module	
		if (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) == 2)
			DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_tx3), media_tx3);
		else
			DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_tx), media_tx);
		
																	   
#ifndef MEDIA_LOOP
																			  
		// Wait for Ingress to send (DM packets_RX'd) message
		if((egress_wait_mailbox_msg(0xF000, SWAP32(DM_INGRESS_PACKETS_RCVD))) == FAIL)					   
		{																						  
			Cleanup_before_return();															  		   
			printf("Timed out...Master NPU did not rx DM_INGRESS_PACKETS_RCVD\n");
			return(pOut->PassFail);
		}																		  

		// Message (DM test_complete) message to Ingress	
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_TEST_COMPLETE;

#endif //MEDIA_LOOP


#ifdef MEDIA_LOOP
while(!KEYPRESS)
{																		
#endif //MEDIA_LOOP

		
		// Read the Ingress' rx'd data
		for(i=0; i < MacDevTab[pIn->MacNum].PortCnt; i++)
		{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT + (i*4)) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_RXPACKETCOUNT + (i*4)));
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR + (i*4)) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_RXPACKETERR + (i*4)));
		// read the actual rx'd data for eack port
		for(x=0; x<16; x++)
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6)) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6)));
		}

		// Test results displayed to Egress user terminal 										  
		printf("\n\tMAC %d test results:\n", pIn->MacNum);

		for(i=0; i<(MacDevTab[pIn->MacNum]).PortCnt;i++)	
		{
			status = PASS;

			Tx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT + (i*4));
			if (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) == 2)
				Tx_Pkts -= 1;
			Rx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT + (i*4));
			Rx_Errs = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR + (i*4));
			printf("Port %d Tx Packets 0x%X Rx Packets 0x%X Rx Errors 0x%X\n", \
											i, Tx_Pkts, Rx_Pkts, Rx_Errs);


			if(Rx_Errs)
				{
				status = FAIL;
				printf("      Port %d status: Failed!!!!\n", i);
				for(x=0; x<14; x++)
					{
					printf("        Packet word 0x%X (Expected):0x%08x (Received):0x%08x", x, macportdata[x],*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6))); 					
					if ((macportdata[x]) != (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6)))) 
						printf("*ERROR*"); 					
					printf("\n"); 					
					}
				}

			else if(Tx_Pkts != Rx_Pkts) 
				{
				if (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) == 2)
					{
					if (Rx_Pkts == 0)
						{
						status = FAIL;
						printf("      Port %d status: Failed!!!!\n", i);
						}
					}
				else
				{
				status = FAIL;
				printf("      Port %d status: Failed!!!!\n", i);
				}
				}
			if(status != FAIL)
				printf("      Port %d status: Passed\n", i);


			final_status |= status;
		}													 


#ifdef MEDIA_LOOP										
		DelayNUsec(1000000);    										   
}
#endif //MEDIA_LOOP

																				   
	
		// Test complete message to Egress user terminal
		if(final_status == FAIL)
		printf("\nMAC %d Media Loopback test: Failed\n", pIn->MacNum);
		else
		printf("\nMAC %d Media Loopback test: Passed\n", pIn->MacNum);
	
		// Cleanup and exit test
		Cleanup_before_return();
	
		Msf_OutOfReset();

		DelayNUsec(500000);								   

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}

		pOut->PassFail = final_status;

		return(pOut->PassFail);


	}																					
	//*****************************************************************
	// If here we are data packet receiver (Ingress)
	//*****************************************************************

	//For debug without doorbell interrupts turned on
#ifdef NO_DOORBELL
	else 
		if (!pIn->MacNum)
			{
			// Call the media connector test
			Media_Loopback_Test_Slave();						 
    		return(PASS);	  
			}
		else
			{
			// Call the fabric connector test
			Media_Loopback_Test_Slave2();						 
    		return(PASS);	  
			}

#else
	else 
	{
		printf("\nThis test cannot be started from the Slave console\n");
	    return(PASS);
	}
#endif

}

//*****************************************************************
// Function:    int Media_test(Media_testIn* pIn,void* pUnused)
// Description: Performs Media Loopback test on Slave
// Input:		Mac Number and Packet count
// Outputs:		Pass/Fail
//*****************************************************************

int Media_Loopback_Test_Slave(void)
{												   
	UINT32 i;
	UINT32 Ingress_msf_Scratch_addr;
	UINT32 read_ext_reg;
	UINT32 fabric;
	UINT dcache_status;
	UINT32 media_type;

	Scroll_Off();

	// Start Ingress
	if ((wait_mailbox_msg(7000, SWAP32(DM_INGRESS_START))) == FAIL)
	{						
		printf("Timed out...Ingress didn't get ok to start\n");
		Cleanup_before_return();
		return(FAIL);
	}
					
  	Msf_OutOfReset();
																 
  	dcache_status = get_CP15() & DCACHE_BIT;
  	if(dcache_status != 0)	// if dcache ON
  	{																					  
  		dcache_off();
  	}
 
	// Egress sends fabric selection(-p) to mailbox 1 
	fabric = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_1));
	if(fabric)
		printf("This is fabric test on media connector\n");

	// Egress sends media selection to mailbox 2
	media_type = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2));

	// Test start message to Ingress user terminal
	printf("Ingress starting Media Loopback test\n");

	if(media_type == MACIXF1110)
	{																				   
		// MAC type goes to media test storage area
		if(Georges_Ingress_Init() !=PASS)																	   
			{
			printf("Ingress did not train correctly\n");							 
			Cleanup_before_return();
			return(FAIL);
			}
	}

//	if(media_type == MACIXF1810)
	else
	{
		// Egress has been initialized, media configured, now init Ingress
		if(Ingress_Init(fabric) != PASS)
			{
			printf("Ingress did not train correctly\n");							 
			Cleanup_before_return();
			return(FAIL);
			}
	}
	// send (DM_INGRESS_TRAIN_COMPLETE) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_INGRESS_TRAIN_COMPLETE;

	// await (DM_INIT_INGRESS_SCRATCH) message from Egress else warning msg
	if ((wait_mailbox_msg(6000, SWAP32(DM_INIT_INGRESS_SCRATCH))) == FAIL)
		{
		Cleanup_before_return();
		printf("Timed out...Slave NPU did not rx DM_INIT_INGRESS_Scratch\n");
		return(FAIL);																	
		}						 									  

	// Reset and clear all ME
	Clear_All_ME();
	
	// Load receive ucode module

	// is it Cross external
	if (*(volatile unsigned int *)(0xCA000000 + XS_MACTYPE) == 2)
		DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_rx3), media_rx3);
	else
		DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_rx), media_rx);
																				   			   
	// send (DM RxRdy) to Egress				   			  
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_INGRESS_RXRDY;

	// wait for packets to be rx'd by receive ucode; error if too long
 	if ((wait_mailbox_msg(4000, DM_PACKETSRCVD)) == FAIL)
	{
		printf("Timed out...Slave NPU did not receive all data packets\n");
		//Cleanup_before_return();   												 
		//return(FAIL);																	 
	}										 
											  
	// send (DM packets_Rcvd) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_INGRESS_PACKETS_RCVD;
	
	// wait for  (DM test_complete) message from Egress 
	if((wait_mailbox_msg(0xF000, SWAP32(DM_TEST_COMPLETE))) == FAIL)
	{
		// if slave restore the address extension register
		Cleanup_before_return();
		printf("Timed out...Slave NPU did not rx DM_TEST_COMPLETE\n");
		return(FAIL);
	}

	// Test complete message to Ingress terminal
	printf("Ingress Media Loopback diagnostic complete\n");
	
	// send (DM cleanup_complete) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CLEANUP_COMPLETE;
	
	// Cleanup and exit test
	Cleanup_before_return();

  	if(dcache_status != 0)	// if dcache was originally ON									
  	{
  		dcache_on();
  	}


	return(PASS);													 
}
										  

//*****************************************************************
// Function:    int wait_mailbox_msg(delay_time, mailbox_msg)
// Description: Delays while waiting for mailbox register data
// Input:		MAC Number
// Outputs:		None
//*****************************************************************
int egress_wait_mailbox_msg(UINT32 timeout, UINT32 mailbox_msg)
{
	UINT32 time_cnt = 0x0;
	int i;

	//printf("DEBUG: mailbox_msg = 0x%x\n", mailbox_msg);

	//while(1)
	for(i=0; i<10000; i++)
	{
		
		if((*(volatile unsigned int *)(0xe0100000 + PCI_CSR_BASE_FRM_PCI + MAILBOX_0)) != mailbox_msg)
			time_cnt++;
		else
		  	break;

//		if(KEYPRESS)							 
//			return(FAIL);

		if(time_cnt >= timeout)
			break;

  		DelayNUsec(1000);
	}

	if(time_cnt >= timeout)
		{
		printf("Exceeded timeout value\n");
		return(FAIL);
		}
	else if (i >= 10000)
		{
		printf("Exceeded max delay time\n");
		return(FAIL);
		}
	else
		return(PASS);
}

//*****************************************************************
// Function:    int wait_mailbox_msg(delay_time, mailbox_msg)
// Description: Delays while waiting for mailbox register data
// Input:		MAC Number
// Outputs:		None
//*****************************************************************
int wait_mailbox_msg(UINT32 timeout, UINT32 mailbox_msg)
{
	UINT32 time_cnt = 0x0;
	UINT32 i;

	//printf("DEBUG: mailbox_msg = 0x%x\n", mailbox_msg);

	//while(1)
	for(i=0; i<10000; i++)
	{
		if((*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0)) != mailbox_msg)
			time_cnt++;
		else
		  	break;

//		if(KEYPRESS)							 
//			return(FAIL);

		if(time_cnt >= timeout)
			break;

  		DelayNUsec(1000);
	}

	if(time_cnt >= timeout)
		{
		printf("Exceeded timeout value\n");
		return(FAIL);
		}
	else if (i >= 10000)
		{
		printf("Exceeded max delay time\n");
		return(FAIL);
		}
	else
		return(PASS);
}





//*****************************************************************
// Function:    void ZeroEgressMailboxes()
// Description: Egress Mailbox values are cleared
// Input:		None
// Outputs:		None
//*****************************************************************
void ZeroEgressMailboxes()
{
	// Zero out the mailbox register data
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = 0x0;
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_1) = 0x0;
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2) = 0x0;
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_3) = 0x0;
}

//*****************************************************************
// Function:    void ZeroIngressMailboxes()
// Description: Ingress Mailbox values are cleared
// Input:		None
// Outputs:		None
//*****************************************************************
void ZeroIngressMailboxes()
{
	UINT32 OTHER_NPUs_PCI_CSR_BASE;

	OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = 0x0;
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_1) = 0x0;
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_2) = 0x0;
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_3) = 0x0;
	}

//*****************************************************************
// Function:    void Cleanup_before_return()
// Description: Clear data settup in test						
// Input:		None
// Outputs:		None
//*****************************************************************

void Cleanup_before_return()	   				   					 
{											 					   

	// Put microengines in reset through RESET_1 register write
	ClearReset_Uengine(UENG_MASK);			
	
	// Reset ueng PC's	
	ResetAllPCs();
	Clear_All_ME();

	// Reset ueng wakeup events
	ClrAllUengWakeEvt();

}	

//*****************************************************************
// Function:    UINT32 GET_OTHER_NPUs_PCI_CSR_BASE(UINT32 npu_sel)
// Description: Get NPU's CSR Base address
// Input:		NPU #
// Outputs:		CSR Base address
//*****************************************************************

UINT32 GET_OTHER_NPUs_PCI_CSR_BASE(UINT32 npu_sel)
{
	UINT32 bus_number, device_number, function_number;
	UINT32 npu_basemap[6];		  

	if (pciFindDevice (VENDOR_INTEL, DEVICE_NPU, npu_sel, &bus_number, &device_number, &function_number) != OK)
	{
		printf("Failed to find 2nd NPU %d\n", device_number);
		return (ERROR);
	}

	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_0, 
		&npu_basemap[0]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_1, 
		&npu_basemap[1]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_2, 
		&npu_basemap[2]);

	//npu_basemap[0] = BRD_PCIMEM_PCI2CPU(npu_basemap[0], 0);
	//npu_basemap[1] = BRD_PCIIO_PCI2CPU(npu_basemap[1], 0); 
	//npu_basemap[1] &= 0xfffffffc;
	
	return(npu_basemap[0]);
}

//*****************************************************************
// Function:    UINT32 GET_OTHER_NPUs_PCI_SRAM_BASE(UINT32 npu_sel)
// Description: Get NPU's DRAM Base address
// Input:		NPU #
// Outputs:		SRAM Base address
//*****************************************************************

UINT32 GET_OTHER_NPUs_PCI_SRAM_BASE(UINT32 npu_sel)
{
	UINT32 bus_number, device_number, function_number;
	UINT32 npu_basemap[6];

	if (pciFindDevice (VENDOR_INTEL, DEVICE_NPU, npu_sel, &bus_number, &device_number, &function_number) != OK)
	{
		printf("Failed to find 2nd NPU %d\n", device_number);
		return (ERROR);
	}
						  
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_0, 
		&npu_basemap[0]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_1, 
		&npu_basemap[1]);
	pciConfigInLong (bus_number, device_number, function_number, IXP2000_PCI_CFG_BASE_ADDRESS_2, 
		&npu_basemap[2]);

	//npu_basemap[0] = BRD_PCIMEM_PCI2CPU(npu_basemap[0], 0);
	//npu_basemap[1] = BRD_PCIIO_PCI2CPU(npu_basemap[1], 0); 
	//npu_basemap[1] &= 0xfffffffc;
																			
	return(npu_basemap[1]);
}


int Ingress_Init(UINT32 fabric)
{
	int read_it;
	int i;

	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;

	// Clock Control CSR (CCR)									 
	Msf_Clock_Speed_Rx();
	
	DelayNUsec(1000);
	
	// IXP_RESET0  bits: 15 (ext_rst),  22 (ext_rst_enable)
	read_it = *(volatile unsigned int *) (0xC0004A0C);
	read_it |= 0x00408000;
	*(volatile unsigned int *) (0xC0004A0C) =  read_it;
			DelayNUsec(1000);					  

	*(volatile unsigned int *) (0xC0004A10) =  0x0;
			DelayNUsec(1000);					  
	
	// msf_rx_control
	*(volatile unsigned int *) (0xc8000000) =  0x00000000;
	DelayNUsec(1000);

	// MSF_io_buf_ctl
//	*(volatile unsigned int *) (0xc8008008) =  0x00001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
	DelayNUsec(1000);
	
	// MSF_TX_Control  
	*(volatile unsigned int *) (0xc8000004) =  0xE0000000;
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc8000004) =  0xE0000C00;
	DelayNUsec(1000);
	
	// MSF_Clock_Control
	*(volatile unsigned int *) (0xc800002c) =  0x000030D0;
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x000031D5;					
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DD5;
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DF5;
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003FFF;
	DelayNUsec(1000);
	
	// csix_type_map
	*(volatile unsigned int *) (0xc8000010) =  0xAAAAAAAA;
	DelayNUsec(1000);
	
	if (!fabric) 
	{ 	
		// MSF_RX_Control  
		*(volatile unsigned int *) (0xc8000000) =  0x40000000;
		DelayNUsec(1000);

		// Set Ingress calender length to match Calypso
		*(volatile unsigned int *) (0xc8000048) =  0x00000010;
		DelayNUsec(1000);
	}

//	while(1)
	for(i=0; i<10000; i++)
	{
		read_it = *(volatile unsigned int *) (0xc8000008) ;  
		if(read_it & 0x80)
			break; 

//		if(KEYPRESS)
//		return(FAIL);

  		DelayNUsec(100);
	}

	if (i >= 10000)
		{
		printf("Exceeded max delay time\n");
		return(FAIL);
		}
	
	// Enable RX control
	*(volatile unsigned int *) (0xc8000000) =  0x40000000;
	DelayNUsec(1000);

	if (!fabric) 
	{ 	
		// Send FIFO status to Calypso
		*(volatile unsigned int *) (0xc80000A0) =  0x00000002;
		DelayNUsec(1000);
	 }

	return(PASS);	
}

Egress_Init()
{
 	unsigned int read_it;
	
	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;

	// Clock Control CSR (CCR)
	Msf_Clock_Speed_Tx();
	
	DelayNUsec(1000);
	
	// IXP_RESET0  bits: 15 (ext_rst),  22 (ext_rst_enable)
	read_it = *(volatile unsigned int *) (0xC0004A0C);
	read_it |= 0x00408000;
	*(volatile unsigned int *) (0xC0004A0C) =  read_it;
			DelayNUsec(1000);

	// IXP_RESET1
	*(volatile unsigned int *) (0xC0004A10) =  0x0;
			DelayNUsec(1000);
	
	// msf_rx_control
	*(volatile unsigned int *) (0xc8000000) =  0x00000000;
			DelayNUsec(1000);

	// MSF_io_buf_ctl
//	*(volatile unsigned int *) (0xc8008008) =  0x00001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
			DelayNUsec(1000);
	
	// MSF_TX_Control  
	*(volatile unsigned int *) (0xc8000004) =  0xE0000000;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc8000004) =  0xE0000C00;
			DelayNUsec(1000);
	
	// MSF_Clock_Control
	*(volatile unsigned int *) (0xc800002c) =  0x000030D0;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x000031D5;
			DelayNUsec(1000);

	// Continuation of slowport registers
	*(volatile unsigned int *) (0xc800002c) =  0x00003DD5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DF5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003FFF;
			DelayNUsec(1000);
	
	// csix_type_map
	*(volatile unsigned int *) (0xc8000010) =  0xAAAAAAAA;
			DelayNUsec(1000);

	// MSF_RX_Control  
	*(volatile unsigned int *) (0xc8000000) =  0x40000000;
			DelayNUsec(1000);
}



int Egress_Init2(UINT32 fabric)
{
int read_it;
int i;

	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;

	// Clock Control CSR (CCR)									 
	Msf_Clock_Speed_Rx();
	
	DelayNUsec(1000);
	
	// IXP_RESET0  bits: 15 (ext_rst),  22 (ext_rst_enable)
	read_it = *(volatile unsigned int *) (0xC0004A0C);
	read_it |= 0x00408000;
	*(volatile unsigned int *) (0xC0004A0C) =  read_it;
			DelayNUsec(1000);					  

	*(volatile unsigned int *) (0xC0004A10) =  0x0;
			DelayNUsec(1000);					  
	
	// msf_rx_control
	*(volatile unsigned int *) (0xc8000000) =  0x00000000;
	DelayNUsec(1000);

	// MSF_io_buf_ctl
//	*(volatile unsigned int *) (0xc8008008) =  0x00001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
	DelayNUsec(1000);
	
	// MSF_TX_Control  
	*(volatile unsigned int *) (0xc8000004) =  0xE0000000;
	DelayNUsec(1000);
	*(volatile unsigned int *) (0xc8000004) =  0xE0000C00;
	DelayNUsec(1000);
	
	// MSF_Clock_Control
	*(volatile unsigned int *) (0xc800002c) =  0x000030D0;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x000031D5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DD5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DF5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003FFF;
			DelayNUsec(1000);
	
	// csix_type_map
	*(volatile unsigned int *) (0xc8000010) =  0xAAAAAAAA;
			DelayNUsec(1000);
	
if (!fabric) 
	{ 	
		// MSF_RX_Control  
		*(volatile unsigned int *) (0xc8000000) =  0x40000000;
		DelayNUsec(1000);

		// Set Ingress calender length to match Calypso
		*(volatile unsigned int *) (0xc8000048) =  0x00000010;
		DelayNUsec(1000);
	}

//	while(1)
	for(i=0; i<10000; i++)
	{
		read_it = *(volatile unsigned int *) (0xc8000008) ;  
		if(read_it & 0x80)
			break; 

//		if(KEYPRESS)
//		return(FAIL);

  		DelayNUsec(100);
	}
	
	if (i >= 10000)
		{
		printf("Exceeded max delay time\n");
		return(FAIL);
		}
	
	// Enable RX control
	*(volatile unsigned int *) (0xc8000000) =  0x40000000;
	DelayNUsec(1000);

	if (!fabric) 
	{ 	
		// Send FIFO status to Calypso
		*(volatile unsigned int *) (0xc80000A0) =  0x00000002;
		DelayNUsec(1000);
	 }

	return(PASS);	
}

Ingress_Init2()
{
 	unsigned int read_it;
	
	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;

	// Clock Control CSR (CCR)
	Msf_Clock_Speed_Tx();
	
	DelayNUsec(1000);
	
	// IXP_RESET0  bits: 15 (ext_rst),  22 (ext_rst_enable)
	read_it = *(volatile unsigned int *) (0xC0004A0C);
	read_it |= 0x00408000;
	*(volatile unsigned int *) (0xC0004A0C) =  read_it;
			DelayNUsec(1000);

	// IXP_RESET1
	*(volatile unsigned int *) (0xC0004A10) =  0x0;
			DelayNUsec(1000);
	
	// msf_rx_control
	*(volatile unsigned int *) (0xc8000000) =  0x00000000;
			DelayNUsec(1000);

	// MSF_io_buf_ctl
//	*(volatile unsigned int *) (0xc8008008) =  0x00001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
			DelayNUsec(1000);
	
	// MSF_TX_Control  
	*(volatile unsigned int *) (0xc8000004) =  0xE0000000;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc8000004) =  0xE0000C00;
			DelayNUsec(1000);
	
	// MSF_Clock_Control
	*(volatile unsigned int *) (0xc800002c) =  0x000030D0;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x000031D5;
			DelayNUsec(1000);

	// Continuation of slowport registers
	*(volatile unsigned int *) (0xc800002c) =  0x00003DD5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003DF5;
			DelayNUsec(1000);
	*(volatile unsigned int *) (0xc800002c) =  0x00003FFF;
			DelayNUsec(1000);
	
	// csix_type_map
	*(volatile unsigned int *) (0xc8000010) =  0xAAAAAAAA;
			DelayNUsec(1000);

	// MSF_RX_Control  
	*(volatile unsigned int *) (0xc8000000) =  0x40000000;
			DelayNUsec(1000);
}


void Msf_Clock_Speed_Tx(void)
{
int read_switches;
int read_it;
UINT32 msf_or_value;
	read_switches = *(volatile unsigned int *) (0xC0004A18);
	read_switches &= 0x1C00;

	switch (read_switches)
	{
    	case 0x0C00: msf_or_value = 0x3 << 20;break;     // 600Mhz
    	case 0x1000: msf_or_value = 0x3 << 20;break;  	 // 800Mhz
    	case 0x1400: msf_or_value = 0x4 << 20;break;	 // 1.0Mhz
    	case 0x1800: msf_or_value = 0x5 << 20;break;	 // 1.2Mhz
    	case 0x1C00: msf_or_value = 0x3 << 20;break;	 // 1.4Mhz
	}
	
	// Clock Control CSR (CCR)									 
	read_it = *(volatile unsigned int *) (0xC0004A14);
	read_it &= 0xff0fffff;
	*(volatile unsigned int *) (0xc0004a14) =  read_it;   
	read_it |= msf_or_value;
	*(volatile unsigned int *) (0xc0004a14) =  read_it;   
}

void Msf_Clock_Speed_Rx(void)
{
int read_switches;
int read_it;
UINT32 msf_or_value;
	read_switches = *(volatile unsigned int *) (0xC0004A18);
	read_switches &= 0x1C00;

	switch (read_switches)
	{
    	case 0x0C00: msf_or_value = 0x2 << 20;break;     // 600Mhz
    	case 0x1000: msf_or_value = 0x2 << 20;break;  	 // 800Mhz
    	case 0x1400: msf_or_value = 0x3 << 20;break;	 // 1.0Mhz
    	case 0x1800: msf_or_value = 0x4 << 20;break;	 // 1.2Mhz
    	case 0x1C00: msf_or_value = 0x3 << 20;break;	 // 1.4Mhz
	}
	
	// Clock Control CSR (CCR)									 
	read_it = *(volatile unsigned int *) (0xC0004A14);
	read_it &= 0xff0fffff;
	*(volatile unsigned int *) (0xc0004a14) =  read_it;   
	read_it |= msf_or_value;
	*(volatile unsigned int *) (0xc0004a14) =  read_it;   
}




Ingress_Train_On()
{
int lookat;

	// Train Data On
	*(volatile unsigned int *) (0xc80000a0) =  0x00000100;
}

Ingress_Train_Off()
{
int lookat;

	// Train Data Off
	*(volatile unsigned int *) (0xc80000a0) =  0;
}

Egress_Train_On()
{
int lookat;

	// Train Data On
	*(volatile unsigned int *) (0xc80000a0) =  0x00000100;
}

Egress_Train_Off()
{
int lookat;

	// Train Data Off
	*(volatile unsigned int *) (0xc80000a0) =  0;
}


//*****************************************************************
// Function:    int Media_test(Media_testIn* pIn,void* pUnused)
// Description: Performs Media Loopback test on Master
// Input:		Mac Number and Packet count
// Outputs:		Pass/Fail
//*****************************************************************
int Media_Loopback_Test2(Media_testIn* pIn )
{
	UINT32 i, x, status, final_status = PASS;				 
	UINT32 Tx_Pkts;
	UINT32 Rx_Pkts;
	UINT32 Rx_Errs;									 
	UINT32 OTHER_NPUs_PCI_CSR_BASE;				   
	UINT32 Egress_msf_Scratch_addr;
	UINT32 Ingress_msf_Scratch_addr;
	UINT dcache_status;
	status = PASS;

	Egress_Init();    //allows writes to the media card

	//*****************************************************************
	// If here we are data packet receiver (Egress)
	//*****************************************************************
	if(IS_EGRESS_NPU)
	{
		dcache_status = get_CP15() & DCACHE_BIT;
		if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

		OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

		//Zero out the Egress mailbox register data
		ZeroEgressMailboxes();

		//Zero out the Ingress mailbox register data
		ZeroIngressMailboxes();

		Egress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

		//Get the Ingress' Scratch start addr 
	    Ingress_msf_Scratch_addr = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);
		Ingress_msf_Scratch_addr += SCRATCH_CSR_BASE_FRM_PCI;
																				 
		// Get user requested MAC # and do range check
		if((pIn->MacNum) >= MAXMACDEV)								   	   
		{					
			printf("Mac number specified is invalid\n");	  
			return(FAIL);													  
		}															   

		// At this point the Ingress has completed it's init
								   									  
		if (!pIn->fabric_sel) 
		{ 													  
			// PCI_Master does MAC Scan 
			MacScan();

			if(MacDevTab[pIn->MacNum].MacType==MACNone)
			{ 	
				printf("No Mac %d detected, Media test aborted\n", pIn->MacNum);	  

				Cleanup_before_return();
				return(FAIL);
			}

		}
		else
		{												  
		// Force to 1 port for fabric test 
		MacDevTab[0].PortCnt = 1;
		MacDevTab[1].PortCnt = 1;
 		}
	  
		if (!pIn->fabric_sel) 
		{
		    // Calls function to init the MAC CSR's for SPI-4 input/output 
		    MacDevTab[pIn->MacNum].MacInitFunc(pIn->MacNum, pIn->Int_Ext_sel, pIn->POS_LAN_sel);
		}
																			
#ifndef NO_DOORBELL
		// Tell Ingress to jump to media_test_slave function
		Set_Pci_Doorbell(SF_SPI4_LB_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
#endif

		// Wait for Ingress to send (DM_EGRESS_INIT) message
		if((egress_wait_mailbox_msg(4000, SWAP32(DM_EGRESS_INIT))) == FAIL)
		{
			Cleanup_before_return();
			printf("Timed out...Egress did not rx DM_EGRESS_INIT\n");
			return(FAIL);
		}

		// Test start message to Egress user terminal 
		printf("Starting MAC %d Media loopback test\n", pIn->MacNum);

		// Clear Egress and Ingress scratch memory areas for media test storage
		for(i=0; i < XS_MEDIA_STORAGE_SIZE; i+=4)		 
		{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + i) = 0;
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + i) = 0;
		}
					   
		// Requested MAC port # to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACNUMBER) = pIn->MacNum;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACNUMBER) = SWAP32(pIn->MacNum);

		// User entered packet count
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = pIn->PktCnt;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(pIn->PktCnt);
	
		// Port count goes to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = MacDevTab[pIn->MacNum].PortCnt;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = SWAP32(MacDevTab[pIn->MacNum].PortCnt);
	
		// MAC type goes to media test storage area
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) = MacDevTab[pIn->MacNum].MacType;
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACTYPE) |= (pIn->Int_Ext_sel<<4);
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACTYPE) = SWAP32(MacDevTab[pIn->MacNum].MacType);
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACTYPE) = 0x11111111;
	
		// write 64 byte transmit packet data to Egress memory for ucode to use
		for (i=0; i < UENG_CNT; i++)
		{
			*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = macportdata[i];
			*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = SWAP32(macportdata[i]);
		}


		// Stuff the fabric or media selection into mailbox 1 before starting ingress
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_1) = pIn->fabric_sel;

		// Send mailbox msg (DM_Ingress_Start) to Ingress								
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_INGRESS_START;

	// Wait for Ingress to complete training
	if ((egress_wait_mailbox_msg(7000, SWAP32(DM_INGRESS_TRAIN_COMPLETE))) == FAIL)
	{						
		printf("Timed out...Ingress did not complete training\n");
		Cleanup_before_return();
		return(FAIL);
	}

		// Ingress has been initialized now init Egress
		if(Egress_Init2(pIn->fabric_sel) != PASS)
			{
			Cleanup_before_return();
			printf("Egress did not train correctly\n");
			return(FAIL);
			}

		// send (DM_EGRESS_TRAIN_COMPLETE) message to Ingress
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_EGRESS_TRAIN_COMPLETE;

		// Wait for Ingress to send (DM Load_RX_UCODE) message
		if((egress_wait_mailbox_msg(4000, SWAP32(DM_LOAD_RX_UCODE))) == FAIL)
		{
			Cleanup_before_return();
			printf("Timed out...Egress NPU did not rx DM_LOAD_RX_UCODE\n");
			return(FAIL);
		}

		// Reset the ueng's
		Clear_All_ME();

		// Load receive ucode module	
		DownLoadUcode(MEDIA_LOOPBACK_RX_ME, sizeof(media_rx2), media_rx2);

		// send (DM_EGRESS_RxRdy) to Ingress				   			  
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_EGRESS_RXRDY;

		// wait for packets to be rx'd by receive ucode; error if too long
 		if ((wait_mailbox_msg(0xF000, DM_PACKETSRCVD)) == FAIL)
		{
			printf("Timed out...Egress NPU did not receive all data packets\n");
			//Cleanup_before_return();   
			//return(FAIL);
		}										 
										
		DelayNUsec(50000);
											  
		// Read the Ingress' tx'd data													 
		for(i=0; i < MacDevTab[pIn->MacNum].PortCnt; i++)
		{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT + (i*4)) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXPACKETCOUNT + (i*4)));
		}


		// Message (DM test_complete) message to Ingress	
		*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_TEST_COMPLETE;

				 
		// Test results displayed to Egress user terminal 					  					  
		printf("\n\tMAC %d test results:\n", pIn->MacNum);
																		   							 
		for(i=0; i<(MacDevTab[pIn->MacNum]).PortCnt; i++)	
		{
			status = PASS;

			Tx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT + (i*4));
			Rx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT + (i*4));
			Rx_Errs = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR + (i*4));
			printf("Port %d Tx Packets 0x%X Rx Packets 0x%X Rx Errors 0x%X\n", \
											i, Tx_Pkts, Rx_Pkts, Rx_Errs);

			if(Rx_Errs)
				{
				status = FAIL;
				printf("      Port %d status: Failed!!!!\n", i);
				for(x=0; x<14; x++)
					{
					printf("        Packet word 0x%X (Expected):0x%08x (Received):0x%08x", x, macportdata[x],*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6))); 					
					if ((macportdata[x]) != (*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPORTDATA + (x*4) + (i<<6)))) 
						printf("*ERROR*"); 					
					printf("\n"); 					
					}
				}

			else if(Tx_Pkts != Rx_Pkts) 
				{
				status = FAIL;
				printf("      Port %d status: Failed!!!!\n", i);
				}

			if(status != FAIL)
				printf("      Port %d status: Passed\n", i);


			final_status |= status;

		}
																	
		// Test complete message to Egress user terminal
		if(final_status == FAIL)
		printf("\nMAC %d Media Loopback test: Failed\n", pIn->MacNum);
		else
		printf("\nMAC %d Media Loopback test: Passed\n", pIn->MacNum);
	
		// Cleanup and exit test
		Cleanup_before_return();
	
		Msf_OutOfReset();

		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}
	
	    return(status);
	}																					
	//*****************************************************************
	// If here we are data packet receiver (Ingress)
	//*****************************************************************

	//For debug without doorbell interrupts turned on
#ifdef NO_DOORBELL
	else 
	{
	Media_Loopback_Test_Slave2();						 
    return(PASS);	  
	}
#else
	else 
	{
		printf("\nThis test cannot be started from the Slave console\n");
	    return(PASS);
	}
#endif

}

//*****************************************************************
// Function:    int Media_test(Media_testIn* pIn,void* pUnused)
// Description: Performs Media Loopback test on Slave
// Input:		Mac Number and Packet count
// Outputs:		Pass/Fail
//*****************************************************************

int Media_Loopback_Test_Slave2(void)
{												   
	UINT32 i;
	UINT32 Ingress_msf_Scratch_addr;
	UINT32 read_ext_reg;
	UINT32 fabric;
	UINT dcache_status;


	Scroll_Off();

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}

	// Ingress_msf_Scratch_addr is Ingress' start address for the media test data
	Ingress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

	// Initialize the Ingress to allow SPI-4 tx Ingress-Egress
	Ingress_Init2();

	// send (DM_EGRESS_INIT) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_EGRESS_INIT;

	// Wait for Egress to complete macscan and macinit
	if ((wait_mailbox_msg(7000, SWAP32(DM_INGRESS_START))) == FAIL)
	{						
		printf("Timed out...Ingress didn't get ok to start\n");
		Cleanup_before_return();
		return(FAIL);
	}

	// Egress puts media or fabric choice in mailbox 1
	fabric = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_1));

	// Test start message to Ingress user terminal
	if(fabric)
		printf("Ingress starting Fabric Loopback test on fabric connector\n");
	else
		printf("Ingress starting Media Loopback test on fabric connector\n");


	if(!fabric)
		{			   
		// Enable training 
		Ingress_Train_On();
		DelayNUsec(100000);    
		// Disable training 
		Ingress_Train_Off();
		DelayNUsec(100000);    
		}


	// Send mailbox msg (DM_INGRESS_TRAIN_COMPLETE) to Egress								
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_INGRESS_TRAIN_COMPLETE;

	if(fabric)
		{
		// Enable training 
		Ingress_Train_On();
		}

	// Wait for a signal that Egress training is complete
	if((wait_mailbox_msg(3000, SWAP32(DM_EGRESS_TRAIN_COMPLETE))) == FAIL)
	{						
		printf("Timed out...Ingress NPU did not rx DM_EGRESS_TRAIN_COMPLETE\n");
		Cleanup_before_return();
		return(FAIL);
	}								  

	if(fabric)
		{
		// Disable training 														   
		Ingress_Train_Off();
		DelayNUsec(100000);    
		}

	// send (DM_LOAD_RX_UCODE) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_LOAD_RX_UCODE;

	// Delay for Ingress (DM RxRdy) message; warning message if not rx'd
	if((wait_mailbox_msg(0x5000, SWAP32(DM_EGRESS_RXRDY))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_EGRESS_RXRDY\n");
		return(FAIL);
	}

	// Reset and clear all ME
	Clear_All_ME();

	DelayNUsec(100000);    
	
	// Load transmit ucode module			 
	DownLoadUcode(MEDIA_LOOPBACK_TX_ME, sizeof(media_tx2), media_tx2);

																				   
	// wait for  (DM test_complete) message from Egress 
	if((wait_mailbox_msg(0xf000, SWAP32(DM_TEST_COMPLETE))) == FAIL)
	{																			
		// if slave restore the address extension register					  
		Cleanup_before_return();
		printf("Timed out...Slave NPU did not rx DM_TEST_COMPLETE\n");
		return(FAIL);																		 
	}

	// Test complete message to Ingress terminal
	printf("Ingress Media Loopback diagnostic complete\n");		
	
	// send (DM cleanup_complete) message to Egress
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CLEANUP_COMPLETE;
	
	// Cleanup and exit test
	Cleanup_before_return();													   
																		 
	Msf_OutOfReset();

	if(dcache_status != 0)	// if dcache was originally ON
		{
		dcache_on();
		}

	return(PASS);

}


Georges_Egress_Train()
{
 	unsigned int read_it;
	
	// Set slowport to 12.5Mhz
 	*(volatile unsigned int *) (0xc0080000) = 0x00000002;

	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;			 
																	  
	// Clock Control CSR (CCR)
	Msf_Clock_Speed_Tx();																 
													
	DelayNUsec(1000);												 	   		 
																										   
	*(volatile unsigned int *) (0xc8000000) = 0x00000000;
//	*(volatile unsigned int *) (0xc8008008) = 0x0001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
	*(volatile unsigned int *) (0xc8000004) = 0xe0000000;
	*(volatile unsigned int *) (0xc8000004) = 0xe0000c00;
	*(volatile unsigned int *) (0xC800002c) = 0x000030D0;
	*(volatile unsigned int *) (0xC800002c) = 0x000031D5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003DD5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003DF5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003FFF;
	*(volatile unsigned int *) (0xc8000010) = 0xaaaaaaaa;
	*(volatile unsigned int *) (0xc8000000) = 0x40000000;
	*(volatile unsigned int *) (0xc8000a00) = 0x100;
	*(volatile unsigned int *) (0xc8000a00) = 0x000;
	*(volatile unsigned int *) (0xc6001424) = 0x2;
	*(volatile unsigned int *) (0xc6001408) = 0x3ff;
	*(volatile unsigned int *) (0xc6000060) = 0x112d;
	*(volatile unsigned int *) (0xc6000260) = 0x112d;
	*(volatile unsigned int *) (0xc6000460) = 0x112d;
	*(volatile unsigned int *) (0xc6000660) = 0x112d;
	*(volatile unsigned int *) (0xc6000860) = 0x112d;
	*(volatile unsigned int *) (0xc6000a60) = 0x112d;		//Don't append CRC
	*(volatile unsigned int *) (0xc6000c60) = 0x112d;
	*(volatile unsigned int *) (0xc6000e60) = 0x112d;
	*(volatile unsigned int *) (0xc6001060) = 0x112d;
	*(volatile unsigned int *) (0xc6001260) = 0x112d;
}	

int Georges_Ingress_Init()
{
int read_it;
int i;

	// Set slowport to 12.5Mhz
 	*(volatile unsigned int *) (0xc0080000) = 0x00000002;

	//Slowport fix
	*(volatile unsigned int *) (0xc0080008) = 0x00000048;
																	   
	// Clock Control CSR (CCR)									 
	Msf_Clock_Speed_Rx();																 
																					 
	DelayNUsec(1000);

	*(volatile unsigned int *) (0xc8000000) = 0x00000000;
//	*(volatile unsigned int *) (0xc8008008) = 0x00001620;
	*(volatile unsigned int *) (0xc8008008) =  0x000016A3;
	*(volatile unsigned int *) (0xc8000004) = 0xe0000000;
	*(volatile unsigned int *) (0xc8000004) = 0xe0000c00;
	*(volatile unsigned int *) (0xC800002c) = 0x000030D0;
	*(volatile unsigned int *) (0xC800002c) = 0x000031D5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003DD5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003DF5;
	*(volatile unsigned int *) (0xC800002c) = 0x00003FFF;
	*(volatile unsigned int *) (0xc8000010) = 0xaaaaaaaa;
	*(volatile unsigned int *) (0xc8000000) = 0x40000000;
	*(volatile unsigned int *) (0xc8000048) = 0xa;
	*(volatile unsigned int *) (0xc80000a0) = 0x2;
	*(volatile unsigned int *) (0xc8000048) = 0xa;
	// *(volatile unsigned int *) (0xc80000a0) = 0x2;
																								 
	*(volatile unsigned int *) (0xc80000a4) = 0xC00;

 
	//while(1)
	for(i=0; i<10000; i++)
 	{
 		read_it = *(volatile unsigned int *) (0xc8000008) ;  
 		if(read_it & 0x80)
 			break; 
 
// 		if(KEYPRESS)
// 		return(FAIL);
 
   		DelayNUsec(100);
 	}

	if (i >= 10000)
		{
		printf("Exceeded max delay time\n");
		return(FAIL);
		}
	
	//printf("DEBUG:Training = 0x%X\n", read_it);	
	 *(volatile unsigned int *) (0xc80000a0) = 0x2;
	 *(volatile unsigned int *) (0xc8000008) = 0x1084;

	return(PASS);	
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,
// "Friendly Name","Help String");
Command(CMD_MEDIA_TEST,"mac",Media_Loopback_Test,DIAGNOSTIC,"Media_test",\
								"media internal/external loopback test");
Arg(UINT8ARG,"MacNum","User entered MAC #");
Arg(UINT32ARG,"PktCnt","Packet size");
Switch("p", BOOLEAN32ARG, "Check for fabric cards", "");
Switch("i", BOOLEAN32ARG, "Internal loopback mode", "i");
Switch("l", BOOLEAN32ARG, "LAN Mode(default is POS)", "");
CommandEnd();																			    

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,
// "Friendly Name","Help String");
Command(CMD_MACSCAN,"macscan",MacScan,DIAGNOSTIC,"MacScan","scans for mac devices");
CommandEnd();




