/* microengine.c		   		  
 *						   			  
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    							 
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800											   
 *  subsystem: DIAG
 *  author: ccm, Mar 07, 02			 
 * 
 * Revisions:
 01a, 08/18/03 ajp Added pass/fail returns for script file break on error
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2x00.h"	
#include "microengine.h"
#include "diag_utils.h"   
#include "led.h"
#include "uc_load.h"
#include "me_xscale_sync.h"
#include "ME_4k_Control_Store_Test.uh"		
#include "ME_Register_Test.uh"					 
#include "ME_Context_Test.uh"
#include "ME_Timers_Counters_Test.uh"
#include "factory_Test.uh"
#include "factory_wtest.uh"
#include "factory_qdr.uh"

extern int Led_Test;

// static function prototypes
static void ME_Help(void);
static int ME_ControlStore_Test(void);
static int ME_Register_Test(void);
static int ME_Context_Test(void);
static int ME_Timers_Cntrs_Test(void);


static unsigned int forever;
static unsigned int ueng_number;

// User input data
typedef struct _Micro_testIn
{								
	UINT32 control_sel;
	UINT32 reg_sel;
	UINT32 timer_sel;
	UINT32 context_sel;
	UINT32 all_sel;
} Micro_testIn;



//*****************************************************************
// Function:    void ResetAllPCs(void)
// Description: Reset The program counters
// Input:		None
// Outputs:		None
//*****************************************************************
void ResetAllPCs(void)
{
	int Uengine, ctx;
	int ueng_base;

	for(Uengine=0; Uengine<=15; Uengine++)
	{
		ueng_base = GetUengBase(Uengine);

		for(ctx=0; ctx<=7; ctx++)
		{
			PUT32((ueng_base+CSR_CTX_POINTER), ctx);

			//these are indirect access by CSR_CTX_POINTER
			PUT32((ueng_base+CTX_STS_IND), 0x0);
			PUT32((ueng_base+ACTIVE_CTX_STS),  0x0);
		}

	}
}


//
// Function: ME_Test
//
// Description: Calls the appropriate function(s) from user input
//
int ME_Test(Micro_testIn* pIn, PassFail_testOut* pOut)
	{
	int passfail1 = 0, passfail2 = 0, passfail3 = 0, passfail4 = 0;

	Scroll_Off();

   	if ((!pIn->control_sel) & (!pIn->context_sel) & (!pIn->reg_sel) & (!pIn->timer_sel))
		(pIn->all_sel) = 1;

   	if ((pIn->control_sel) || (pIn->all_sel))
		passfail1 = ME_ControlStore_Test();

	if ((pIn->reg_sel) || (pIn->all_sel))
		{
		ueng_number = ALL_UENG;
		passfail2 = ME_Register_Test();
		}
	if ((pIn->timer_sel) || (pIn->all_sel))
		passfail3 = ME_Timers_Cntrs_Test();

	if ((pIn->context_sel) || (pIn->all_sel))
		passfail4 = ME_Context_Test();

	pOut->PassFail = (passfail1 | passfail2 | passfail3 | passfail4);

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);
  }
//
// Function: ME_ControlStore_Test
//
// Desciption: This function will test the micro-engine control store. The ME
//			  control store is similar to SRAM which contains ME code of
//			  instruction and data for execution.  It will load 4K byte of
//			  instruction and verify if the loading is successful
//
static int ME_ControlStore_Test(void)
{
	int test, ME_num, status = OK, location;
	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 0x1000; location++)			
		PUT32(SCRATCH_BASE + (location<<B_TO_LW_SHF), 0);  
 							  
	DownLoadUcode(0,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(1,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(2,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(3,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(4,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(5,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(6,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(7,   sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(16,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(17,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(18,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(19,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(20,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(21,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(22,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);
	DownLoadUcode(23,  sizeof(ME_4k_Control_Store_ME),  (unsigned int *)ME_4k_Control_Store_ME);

	DelayNUsec(10000);

	for(ME_num = ME0; ME_num <= ME15; ME_num++)
	{
		test = GET32(SCRATCH_BASE + ((1 + ME_num)<<B_TO_LW_SHF));	// 1 is just an offset
		if(test != CONTROL_STORE_EXP_VAL)	// value expected from microengine from its calculation
		{
			printf("ME %02d: Control Store Test FAIL. Value Read = %x \n", ME_num, test);
			status = ERROR;
		}
	}

	Clear_All_ME();

	printf("ME Control Store Test... ");
	if(status == OK)
	{
		printf("PASS.\n\n");
		return(PASS);
	}
	else
	{
		printf("FAIL.\n\n");
		return(FAIL);
	}
}

//
// Function: ME_Register_Test			   
//
// Description: To test the following registers:
//				- GPR
//				- Transfer Registers
//				- Next Neighbour Registers
//				- Local Memory Registers	
//										   
static int ME_Register_Test(void)		   
{
	int location, thread_num, ME_num = 0, test, status = OK, thd_start = 0, thd_end = 128;
	int thread_fail = 0;		   

	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 0x1000; location++)			
		PUT32(SCRATCH_BASE + (location<<B_TO_LW_SHF), 0);  
 							  
	DownLoadUcode(23,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);	
	DownLoadUcode(22,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(21,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(20,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(19,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);	 
	DownLoadUcode(18,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(17,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(16,  sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
 	DownLoadUcode(7,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(6,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(5,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(4,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(3,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(2,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(1,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);
	DownLoadUcode(0,   sizeof(ME_Register_Test_ME), (unsigned int *)ME_Register_Test_ME);

	DelayNUsec(1000000);
	// Verify Xfer Reg Test (scratch location 0 - 127 (Lword))
	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
	{
		test = GET32(SCRATCH_BASE + (((thread_num)*2)<<B_TO_LW_SHF));
		if(test != COMPLETE) // if test did not complete
		{
			thread_fail |= 1;
			status = ERROR;
		}									 
		else // test completed
		{
			test = GET32(SCRATCH_BASE + (((thread_num)*2 + 1)<<B_TO_LW_SHF));
			if(test != OK)
			{
				thread_fail |= 2;
				status = ERROR;
			}
		}
		if( ((thread_num + 1)%8) == 0)
		{
			if(thread_fail != 0)
				printf("Xfer Reg Test for ME%d FAIL!\n", thread_num/8);
			thread_fail = 0;
		}
	}
	
 /* If required to identify by which thread instead of by microengine
 	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
 	{
 		test = GET32(SCRATCHPAD_BASE + (((thread_num)*2)<<B_TO_LW_SHF));
 		if(test != COMPLETE) // if test did not complete
 		{
 			printf("Xfer Reg Test for thread %d did not complete.\n", thread_num);
 			status = ERROR;
 		}
 		else // test completed
 		{
 			test = GET32(SCRATCHPAD_BASE + (((thread_num)*2 + 1)<<B_TO_LW_SHF));
 			if(test != OK)
 			{
 				printf("Xfer Reg Test for thread %d FAIL with %x\n", thread_num, test);
 				status = ERROR;
 			}
 		}
 	}
 */
 	// Verify GPR Test (scratch location 256 - 511 (Lword))
 	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
 	{
 		// GPR test location offset = 256
 		test = GET32(SCRATCH_BASE + ((256 + (thread_num)*2)<<B_TO_LW_SHF));		 
 		if(test != COMPLETE) // if test completed
 		{
 			thread_fail |= 1;
 			status = ERROR;
 		}
 		else // test completed
 		{
 			test = GET32(SCRATCH_BASE + ((256 + (thread_num)*2 + 1)<<B_TO_LW_SHF));
 			if(test != OK)
 			{
 				thread_fail |= 2;
 				status = ERROR;
 			}													
 		}
 		if( ((thread_num + 1)%8) == 0) 
 		{
 			if(thread_fail != 0)
 				printf("GPR Test for ME%d FAIL!\n", thread_num/8);
 			thread_fail = 0;
 		}
 	}
 
 /* If required to identify by which thread instead of by microengine
 	for(thread_num = thd_start; thread_num < thd_end; thread_num++)
 	{
 		// GPR test location offset = 128
 		test = GET32(SCRATCH_BASE + ((128 + (thread_num)*2)<<B_TO_LW_SHF));
 		if(test != COMPLETE) // if test completed
 		{
 			printf("GPR Test for thread %d did not complete.\n", thread_num);
 			status = ERROR;
 		}
 		else // test completed
 		{
 			test = GET32(SCRATCH_BASE + ((128 + (thread_num)*2 + 1)<<B_TO_LW_SHF));
 			if(test != OK)
 			{
 				printf("GPR Test for thread %d FAIL with %x\n", thread_num, test);
 				status = ERROR;	 												 
 			}
 		}
 	}
 */							  
	if(status != ERROR)
 	{													
 	   	if(ueng_number == ALL_UENG)		
 		{														  
 			// Verify Local Mem Addr(LMA) Test (scratch location 512 - 543 (Lword))
 			for(ME_num = ME0; ME_num <= ME15; ME_num++)
 			{
 				// LMA test location offset = 512
 				test = GET32(SCRATCH_BASE + 0x800 + (ME_num<<3));
 				//printf("DEBUG: test address  = 0x%X\n",(SCRATCH_BASE + 0x900 + (ME_num<<3)));
 				//test = GET32(SCRATCH_BASE + ((0x800 + ME_num*2)<<B_TO_LW_SHF));
 				if(test != COMPLETE)
 				{
 					printf("Local Memory Test for ME %d did not complete.\n", ME_num);
 					status = ERROR;
 				}
 				else
 				{
 					test = GET32(SCRATCH_BASE + 0x800 + (ME_num<<3) + 4);
 					//test = GET32(SCRATCH_BASE + ((0x800 + ME_num*2 + 1)<<B_TO_LW_SHF));
 					if(test != OK)
 					{
 						printf("Local Memory Test for ME %d FAIL! Failing value = 0x%x\n", ME_num, test);
 						status = ERROR;
 					}							  
 				}
 			}					 
 
 
 			// Verify Next Neighbor(NN) Int Reg Test (scratch location 544 - 575 (Lword))
 			for(ME_num = ME0; ME_num <= ME15; ME_num++)
 			{
 				// NN Reg Int test location offset = 544
 				test = GET32(SCRATCH_BASE + 0x880 + (ME_num<<3));
 				//printf("DEBUG: test address  = 0x%X\n",(SCRATCH_BASE + 0x880 + (ME_num<<3)));
 				if(test != COMPLETE)
	 				{
 					printf("Next Neighbor Int Test for ME%d did not complete.\n", ME_num);
 					status = ERROR;
 					}
 				else
 				{
 				test = GET32(SCRATCH_BASE + 0x880 + (ME_num<<3) + 4);
 				//printf("DEBUG: test address  = 0x%X\n",(SCRATCH_BASE + 0x880 + (ME_num<<3) + 4));
 					if(test != OK)
 					{
 						printf("Next Neighbor Int Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
 						status = ERROR;
 					}								   
				}								 
 			}
 
 			// Verify Next Neighbor(NN) Ext Reg Test (scratch location 576 - 607 (Lword))
 			for(ME_num = ME0; ME_num <= ME15; ME_num++)
 			{
 				// NN Reg Ext test location offset = 576	 
 				test = GET32(SCRATCH_BASE + 0x900 + (ME_num<<3));
 				//printf("DEBUG: test address  = 0x%X\n",(SCRATCH_BASE + 0x900 + (ME_num<<3)));
 				if(test != COMPLETE)
 				{
 					printf("Next Neighbor Ext Test for ME%d did not complete.\n", ME_num);
 					status = ERROR;
 				}
 				else
 				{
	 				test = GET32(SCRATCH_BASE + 0x900 + (ME_num<<3) + 4);
 				//printf("DEBUG: test address  = 0x%X\n",(SCRATCH_BASE + 0x900 + (ME_num<<3) + 4));
 					if(test != OK)
 					{
 						printf("Next Neighbor Ext Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
 						status = ERROR;
 					}
 				}
 			}
 		}
 													   
 		else
 		{
 			// Verify Local Mem Addr(LMA) Test (scratch location 256 - 271 (Lword))
 			// LMA test location offset = 256
 			test = GET32(SCRATCH_BASE + ((256 + ME_num*2)<<B_TO_LW_SHF));
 			if(test != COMPLETE)
 			{
 				printf("Local Memory Test for ME %d did not complete.\n", ME_num);
 				status = ERROR;
 			}
 			else
 			{
 				test = GET32(SCRATCH_BASE + ((256 + ME_num*2 + 1)<<B_TO_LW_SHF));
 				if(test != OK)
 				{
 					printf("Local Memory Test for ME %d FAIL! Failing value = 0x%x\n", ME_num, test);
 					status = ERROR;
 				}
 			}
 
 
 			// Verify Next Neighbor(NN) Int Reg Test (scratch location 288 - 303 (Lword))
 			// NN Reg Int test location offset = 288
 			test = GET32(SCRATCH_BASE + ((288 + (ME_num)*2)<<B_TO_LW_SHF));
 			if(test != COMPLETE)
 			{
 				printf("Next Neighbor Int Test for ME%d did not complete.\n", ME_num);
 				status = ERROR;
 			}
 			else
 			{
 				test = GET32(SCRATCH_BASE + ((288 + (ME_num)*2 + 1)<<B_TO_LW_SHF));
 				if(test != OK)
 				{
 					printf("Next Neighbor Int Test for ME%d FAIL! Failing value = 0x%x\n", ME_num, test);
 					status = ERROR;
 				}
 			}
 
 			// Verify Next Neighbor(NN) Ext Reg Test not performed
 			printf("***NOTE***: Next Neighbor Ext Test not tested in single ME test\n");
 		}
 	}

	Clear_All_ME();

	printf("ME Register Test... ");
	if(status == OK)
	{
		printf("PASS.\n");
		return(PASS);
	}
	else
	{
		printf("FAIL!\n\n");
		return(FAIL);
	}

}



//
// Function: ME_Context_Test
//
// Description: This function will test the micro-engine hardware contexts. 
//			  	There are 8 hardware contexts in the micro-engine.  All 4 states
//			  	of each context -- inactive, executing,	ready and sleep will
//				be verified though tests have been written for inactive and
//				sleep only but the other two are always tested.
//
static int ME_Context_Test(void)
{
	int location, test, ME_num, status = OK;

	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 0x1000; location++)
		PUT32(SCRATCH_BASE + (location<<B_TO_LW_SHF), 0);

	DownLoadUcode(23,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);	 
	DownLoadUcode(22,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(21,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(20,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(19,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(18,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(17,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(16,  sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(7,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(6,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(5,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(4,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(3,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(2,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(1,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);
	DownLoadUcode(0,   sizeof(ME_Context_Test_ME), (unsigned int *)ME_Context_Test_ME);

									
	DelayNUsec(10000);

	// Verify Inactive State Test (scratch location 0 - 15 (Lword))
	for(ME_num = ME0; ME_num <= ME15; ME_num++)
	{
		test = GET32(SCRATCH_BASE + ((ME_num*2)<<B_TO_LW_SHF));
		if(test == COMPLETE)	// indication that Inactive State Test completed
		{
			test = GET32(SCRATCH_BASE + ((ME_num*2 + 1)<<B_TO_LW_SHF));
			if(test != PASSED)
			{
				printf("ME %d FAIL Inactive State Test.\n", ME_num);
				status = ERROR;
			}
		}
		else
		{
			printf("ME %d FAIL to complete Inactive State Test.\n", ME_num);
			status = ERROR;
		}
	}

	// Verify Sleep State Test (scratch location 16 - 31 (Lword))
	for(ME_num = ME0; ME_num <= ME15; ME_num++)
	{ 
		test = GET32(SCRATCH_BASE + ((32 + ME_num*2)<<B_TO_LW_SHF));
		if(test == COMPLETE)	// indication that Sleep State Test completed
		{
			test = GET32(SCRATCH_BASE + ((16 + ME_num*2 + 1)<<B_TO_LW_SHF));
			if(test != PASS)
			{
				printf("ME %d FAIL Sleep State Test.", ME_num);
				printf("Stopped at thread %d\n", (test&0x7));
				status = ERROR;
			}
		}
		// Check if one of the thd did not sleep
		else if(test == THD_DID_NOT_SLEEP)
		{
			printf("ME %d thds did not go to sleep.\n", ME_num);
			status = ERROR;
		}
		else
		{
			printf("ME %d FAIL to complete Sleep State Test.\n", ME_num);
			status = ERROR;
		}
	}
	Clear_All_ME();
	
	printf("ME Context Test... ");
	if(status == OK)
	{
		printf("PASS.\n\n");
		return(PASS);
	}
	else
	{
		printf("FAIL.\n\n");
		return(FAIL);
	}
}

// 
// Function: ME_Timers_Cntrs_Test
//
// Description: Tests the Timers and Counters in the microengines
//
static int ME_Timers_Cntrs_Test(void)
{
	int ME_num, test, status = OK, location;

	// Initialize space in scratch used for ME to pass status to XScale
	for(location = 0; location < 0x1000; location++)			
		PUT32(SCRATCH_BASE + (location<<B_TO_LW_SHF), 0);  
 							  
	DownLoadUcode(23,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(22,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(21,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(20,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(19,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(18,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(17,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(16,  sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(7,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(6,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(5,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(4,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(3,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(2,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(1,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);
	DownLoadUcode(0,   sizeof(ME_Timers_Counters_Test_ME), (unsigned int *)ME_Timers_Counters_Test_ME);

	DelayNUsec(1000);			 
													  		  
	// Check status for each ME			 
	for(ME_num = ME0; ME_num <= ME15; ME_num++)
	{																				
		test = GET32(SCRATCH_BASE + (ME_num<<B_TO_LW_SHF));
		if(test == 0)	// test did not complete to write status
		{
			printf("ME %d did not complete\n", ME_num);
			status = ERROR;
		}
		else if(test != ME_TIMER_COUNTER_VAL_EXPECTED)
		{
			printf("ME %d has wrong timer value. Expected 0x%x, read 0x%x\n",
					ME_num, ME_TIMER_COUNTER_VAL_EXPECTED, test);
			status = ERROR;
		}
	}																							   
																 
	Clear_All_ME();						   						
	// Perform logging of test results
	printf("Microengine Timers and Counters Test... ");
	if(status == OK)
	{
		printf("PASS.\n\n");
		return(PASS);
	}																	 
	else
	{
		printf("FAIL.\n\n");
		return(FAIL);
	}									   
}												
											
// Hidden Memory debug code 									 
void factory_mem_read(void);
void factory_mem_read(void)										  
{
	int ME_num, test, status = OK;
	printf("DEBUG code that reads SRAM channel 0 location 0x80000000\n");

	DownLoadUcode(0, sizeof(factory_MEM_Test), (unsigned int *)factory_MEM_Test);	 
}

void factory_mem_write(void);
void factory_mem_write(void)	  									  
{
	int ME_num, test, status = OK;
	printf("DEBUG code that writes SRAM channel 0 location 0x80000000\n");

	DownLoadUcode(0, sizeof(factory_MEM_WTest), (unsigned int *)factory_MEM_WTest);	 
}

// Characterize QDR ucode 									 
void factory_qdr(void);
void factory_qdr(void)										  
{
	int ME_num, test, status = OK;
	printf("DEBUG code that characterizes the QDR interface on channel 0\n");

	DownLoadUcode(0, sizeof(factory_QDR), (unsigned int *)factory_QDR);	 
}

void sp_write_sram(UINT32 location, UINT32 data);

// Characterize Slowport 									 
int factory_sp(void *pIn, PassFail_testOut* pOut);
int factory_sp(void *pIn, PassFail_testOut* pOut)										  
{
UINT32 i = 1, flash_data, stored_data;
UINT32 flash_start_addr;
int status = PASS;
pOut->PassFail = PASS;
pOut->ReturnCode = PASS;         



while((!KEYPRESS) && (status == PASS))
	{
	printf("Slowport test via multiple reads of flash (keypress stops test)\n");

	for (flash_start_addr = 0xc4000000; flash_start_addr <0xc6000000; flash_start_addr += 0x800000)
		{
		printf("\tTesting Flash address 0x%x - 0x%x\n", flash_start_addr, (flash_start_addr + 0x7ffffc));

		status = factory_sp_func(flash_start_addr);

		if(status == FAIL)
			{
			printf("\t\tSlowport test section failure!!!!\n");
			break;
			}
		else
			printf("\t\tSlowport test section passed\n");
		}

	printf("\nSlowport test pass %d complete\n\n", i);
	i++;
	}

		pOut->PassFail = status;

		return(pOut->PassFail);


}


int factory_sp_func(UINT32);
int factory_sp_func(UINT32 start_addr)										  
{
UINT32 i, flash_data, stored_data;
UINT32 max_addr;
int status = PASS;

//	max_addr = 0x800000;
	max_addr = 0x100;

	// read data from Flash and store into SRAM
	for(i=0;i<max_addr;i +=4)
		{
		// read flash data
		flash_data = sp_read_flash(start_addr, i);
//		printf("Original data 0x%x = 0x%X\n", i, flash_data);

		// Store flash data in SRAM
		sp_write_sram(i, flash_data);
		}

	// now read Flash again and compare to SRAM
	for(i=0;i<max_addr;i +=4)
		{
		flash_data = sp_read_flash(start_addr, i);
		stored_data = sp_read_sram(i);

		if(stored_data != flash_data)
			{
			printf("\t\tSlowport memory failure!!!!\n");
			printf("Flash address 0x%x, flash data = 0x%X, expected data = 0x%X\n", (start_addr + i), flash_data, stored_data);
			status = FAIL;
			break;
			}
		}


	return(status);
}


int sp_read_flash(UINT32 start_addr, UINT32 location);
int sp_read_flash(UINT32 start_addr, UINT32 location)
{
UINT32 data;

		data = *(volatile unsigned int *)(start_addr + location);

		return(data);
}

void sp_write_sram(UINT32 location, UINT32 data)
{

		*(volatile unsigned int *)(0x80000000 + location) = data;

}

int sp_read_sram(UINT32 location);
int sp_read_sram(UINT32 location)
{
UINT32 data;

		data = *(volatile unsigned int *)(0x80000000 + location);

		return(data);
}

int sp_checksum()
{
UINT32 i, data, checksum = 0;

	for(i=0x0;i<0x2000000;i +=4)
		{
		data = *(volatile unsigned int *)(0xc4000000 + i);
		checksum += data;
//		printf("Address = 0x%X Data = 0x%X\n", (0xc4000000 + i),data);
//		printf("Address = 0x%X checksum = 0x%X\n", (0xc4000000 + i),checksum);
		}
return(checksum);
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_MICRO,"ueng",ME_Test,DIAGNOSTIC,"MICROTest","microengines control store, context, register, and timer/counter test");
Switch("c", BOOLEAN32ARG, "control store test", "");
Switch("r", BOOLEAN32ARG, "register test", "");
Switch("t", BOOLEAN32ARG, "timer and counters test", "");
Switch("s", BOOLEAN32ARG, "context test", "");
Output(UINT32ARG, "PassFail");
Output(CHAR64ARG, "ResultString");
CommandEnd();

Command(CMD_FACTORYR,"factoryr",factory_mem_read,HIDDEN|DIAGNOSTIC,"Debug ueng SRAM mem read of Channel 0 locations 0x0 - 0xC","Debug ueng SRAM mem read of Channel 0 locations 0x0 - 0xC");
CommandEnd();

Command(CMD_FACTORYW,"factoryw",factory_mem_write,HIDDEN|DIAGNOSTIC,"Debug ueng SRAM mem write of Channel 0 locations 0x0 - 0xC","Debug ueng SRAM mem write of Channel 0 locations 0x0 - 0xC");
CommandEnd();

Command(CMD_FACTORYQDR,"factoryqdr",factory_qdr,HIDDEN|DIAGNOSTIC,"Characterize the QDR interface on channel 0","Characterize the QDR interface on channel 0");
CommandEnd();

Command(CMD_FACTORYSP,"factorysp",factory_sp,HIDDEN|DIAGNOSTIC,"Characterize the SP interface","Test SlowPort via flash checksum");
Output(UINT32ARG, "PassFail");
Output(CHAR64ARG, "ResultString");
CommandEnd();
