*****************************************************************************
*
* Module:	mmuasm.s
*
* Description:
*
* Project:	HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:	See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*****************************************************************************
*=== Includes ======================
#include "cpu.h"
#include "boot.h"
#include "board.h"

*=== Macros ======================
CPWAIT:	.macro	reg
	mrc	p15, 0, reg, c2, c0, 0	# arbitrary read of CP15
	mov	reg, reg		# wait for it
	sub	pc,pc,#4		# branch to next instruction
	.endm

MakeTransTable: .MACRO addr,addrSize,options,total
    .if total
       .if total <= 0x40
           .long ((addr << addrSize) | options)
           MakeTransTable (addr+1), addrSize, options, (total-1)
       .else
           .if total <= 0x256
               MakeTransTable addr, addrSize, options, 0x40
               MakeTransTable (addr+0x40), addrSize, options, (total-0x40)
           .else
               MakeTransTable addr, addrSize, options, 0x256
               MakeTransTable (addr+0x256), addrSize, options, (total-0x256)
           .endif
       .endif
    .endif
    .ENDM

/* Create First Level Section Descriptor */

#define SectionDesc(addr, options, total) \
	MakeTransTable addr, 20, options, total
        
*=== Section =====================
	.text

	.globl	InitMMU
        .globl  RomL1Ptable
************************************************************************
* InitMMU - Create a translation table in RAM.
************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void InitMMU(void);
************************************************************************
InitMMU:

************************************************************************
* IXP2800 Init
************************************************************************

        mrs     r1, cpsr
        orr     r1, r1, #0xc0
        msr     cpsr, r1

************************************************************************
* Relocate the translation table to the end of working ram
************************************************************************

	ldr	r0,=MMU_TRANSLATION_BASE
	mcr	p15,0,r0,c2,c0,0	# set up the TTB pointer
	ldr	r1,=RomL1Ptable
	ldr   r3,=RomL1PtableEnd
mmuinit11:
	ldr   r2,[r1],#4
	str	r2,[r0],#4
	cmp   r1,r3
	bne   mmuinit11

************************************************************************
* Turn on MMU 
************************************************************************
/* Enable Coprocessors CP0 */
        ldr     r0, =0x0001
        mcr     p15, 0, r0, c15, c1, 0

        mcr     p15, 0, r0, c7, c10, 4  /* Drain write/fill buffers */
        CPWAIT  r0                      /* wait for the write to happen */

/* Invalidate I-Cache, D-Cache, and BTB */
        mcr     p15, 0, r0, c7, c7, 0
        CPWAIT  r0                      /* Wait */

/* Enable Instruction Cache */
        mrc     p15, 0, r0, c1, c0, 0   /* Read Control Register*/
        orr     r0, r0, #0x1000         /* Set I-Cache bit */
        mcr     p15, 0, r0, c1, c0, 0   /* Write Back Control Register */
        CPWAIT  r0                      /* Wait */

/* Set Translation Table Base */
        ldr     r0, =MMU_TRANSLATION_BASE
        mcr     p15, 0, r0, c2, c0, 0   /* Set Translation Table Base Register */

/* Invalidate Instruction, Data TLBs */
        mcr     p15, 0, r0, c8, c7, 0   /* Flush I & D TLBs*/
        CPWAIT  r0                      /* Wait */

/* Set Domain Access Control Register */
        ldr     r0, =0xffffffff         /* Set All 16 domains to manager access */
        mcr     p15, 0, r0, c3, c0, 0   /* Set Domain Permissions */

/* Enable MMU */
        mrc     p15, 0, r0, c1, c0, 0   /* Read Control Register */
        orr     r0, r0, #0x00000001     /* Enable MMU */
        mcr     p15, 0, r0, c1, c0, 0   /* Write Back the Control Register */
        CPWAIT  r0                      /* Wait */

/* Drain Write/Fill Buffers */
        mcr     p15, 0, r0, c7, c10, 4  /* Drain */
        CPWAIT  r0                      /* Wait */

/* Enable Data Cache */
        mrc     p15, 0, r0, c1, c0, 0   /* Read Control Reg */
        orr     r0, r0, #0x00000004     /* Enable Data Cache */
        mcr     p15, 0, r0, c1, c0, 0   /* Write Back */
        CPWAIT  r0                      /* Wait */

/* Enable Branch Target Buffer */
        mrc     p15, 0, r0, c1, c0, 0   /* Read Control Reg */
        orr     r0, r0, #0x00000800     /* Enable BTB */
        mcr     p15, 0, r0, c1, c0, 0   /* Write Back the Control Reg */
        CPWAIT  r0                      /* Wait */

        mov	pc,lr			  /* return */
        
        .align  12      /* this prevents code from above polluting the table */
RomL1Ptable:                     # Translation Table
#if ((WORKSPACE & 0xb0000000) == 0x80000000)
SectionDesc (0x800, 0x00000c02, 0x001)
SectionDesc (0x001, 0x00000c0e, 0x2FF)
SectionDesc (0x300, 0x00000000, 0x501)
SectionDesc (0x801, 0x00000c02, 0x003)
SectionDesc (0x804, 0x00000000, 0x0FC)
SectionDesc (0x900, 0x00000c02, 0x004)  /* 4 : SRAM channel 1 */
SectionDesc (0x904, 0x00000000, 0x0FC)
SectionDesc (0xa00, 0x00000c02, 0x004)  /* 4 : SRAM channel 2 */
SectionDesc (0xa04, 0x00000000, 0x0FC)
SectionDesc (0xb00, 0x00000c02, 0x004)  /* 4 : SRAM channel 3 */
SectionDesc (0xb04, 0x00000000, 0x0FC)
#elif ((WORKSPACE & 0xb0000000) == 0x90000000)
SectionDesc (0x900, 0x00000c02, 0x001)
SectionDesc (0x001, 0x00000c0e, 0x2FF)
SectionDesc (0x300, 0x00000000, 0x500)
SectionDesc (0x800, 0x00000c02, 0x004)
SectionDesc (0x804, 0x00000000, 0x0FD)
SectionDesc (0x901, 0x00000c02, 0x003)
SectionDesc (0x904, 0x00000000, 0x0FC)
SectionDesc (0xa00, 0x00000c02, 0x004)
SectionDesc (0xa04, 0x00000000, 0x0FC)
SectionDesc (0xb00, 0x00000c02, 0x004)
SectionDesc (0xb04, 0x00000000, 0x0FC)
#elif ((WORKSPACE & 0xb0000000) == 0xa0000000)
SectionDesc (0xa00, 0x00000c02, 0x001)
SectionDesc (0x001, 0x00000c0e, 0x2FF)
SectionDesc (0x300, 0x00000000, 0x500)
SectionDesc (0x800, 0x00000c02, 0x004)
SectionDesc (0x804, 0x00000000, 0x0FC)
SectionDesc (0x900, 0x00000c02, 0x004)  /* 4 : SRAM channel 1 */
SectionDesc (0x904, 0x00000000, 0x0FD)
SectionDesc (0xa01, 0x00000c02, 0x003)
SectionDesc (0xa04, 0x00000000, 0x0FC)
SectionDesc (0xb00, 0x00000c02, 0x004)
SectionDesc (0xb04, 0x00000000, 0x0FC)
#elif ((WORKSPACE & 0xb0000000) == 0xb0000000)
SectionDesc (0xb00, 0x00000c02, 0x001)
SectionDesc (0x001, 0x00000c0e, 0x2FF)
SectionDesc (0x300, 0x00000000, 0x500)
SectionDesc (0x800, 0x00000c02, 0x004)
SectionDesc (0x804, 0x00000000, 0x0FC)
SectionDesc (0x900, 0x00000c02, 0x004)  /* 4 : SRAM channel 1 */
SectionDesc (0x904, 0x00000000, 0x0FC)
SectionDesc (0xa00, 0x00000c02, 0x004)
SectionDesc (0xa04, 0x00000000, 0x0FD)
SectionDesc (0xb01, 0x00000c02, 0x003)
SectionDesc (0xb04, 0x00000000, 0x0FC)
#else
SectionDesc (0x000, 0x00000c02, 0x300)	/* 768 : RDRAM */
SectionDesc (0x300, 0x00000000, 0x500) /* 1280 : Reserved */
SectionDesc (0x800, 0x00000c02, 0x040)  /* 4 : SRAM channel 0 */
SectionDesc (0x840, 0x00000000, 0x0c0)
SectionDesc (0x900, 0x00000c02, 0x040)  /* 4 : SRAM channel 1 */
SectionDesc (0x940, 0x00000000, 0x0c0)
SectionDesc (0xa00, 0x00000c02, 0x040)  /* 4 : SRAM channel 2 */
SectionDesc (0xa40, 0x00000000, 0x0c0)
SectionDesc (0xb00, 0x00000c02, 0x040)  /* 4 : SRAM channel 3 */
SectionDesc (0xb40, 0x00000000, 0x0c0)
#endif
SectionDesc (0xc00, 0x00000c02, 0x020)  /* 32 : CAP CSRs */
SectionDesc (0xc20, 0x00000000, 0x020)  /* 32 : Reserved */
SectionDesc (0xc40, 0x00000c02, 0x010)  /* 16 : Flash */
SectionDesc (0xc50, 0x00000000, 0x010)  /* 16 : Reserved */
SectionDesc (0xc60, 0x00000c02, 0x020)  /* 32 : Slowport */
SectionDesc (0xc80, 0x00000c02, 0x020)  /* 32 : MSF */
SectionDesc (0xca0, 0x00000c02, 0x020)  /* 32 : Scratch (there's only 16K) */
SectionDesc (0xcc0, 0x00000c02, 0x020)  /* 32 : SRAM CSR & Queue */
SectionDesc (0xce0, 0x00000c02, 0x020)  /* 32 : SRAM Ring */
SectionDesc (0xd00, 0x00000c02, 0x020)  /* 32 : DRAM CSR */
SectionDesc (0xd20, 0x00000000, 0x040)  /* 64 : Reserved */
SectionDesc (0xd60, 0x00000c02, 0x020)  /* 32 : Gasket CSR */
SectionDesc (0xd80, 0x00000c02, 0x020)  /* 32 : PCI I/O (only 64K) */
SectionDesc (0xda0, 0x00000c02, 0x020)  /* 32 : PCI Cfg */
SectionDesc (0xdc0, 0x00000c02, 0x020)  /* 32 : PCI Spec/Ack */
SectionDesc (0xde0, 0x00000c02, 0x010)  /* 16 : PCI config hdr */
SectionDesc (0xdf0, 0x00000c02, 0x010)  /* 16 : PCI local CSRs */
SectionDesc (0xe00, 0x00000c02, 0x200)  /* 512: PCI memory space */
RomL1PtableEnd:                  
#if 0                                
************************************************************************
* Turn on MMU DCache
************************************************************************
	ldr	r0,=(RAM1START + RAM1SIZE - 65536)
	mcr	p15,0,r0,c2,c0,0	# set up the TTB pointer
	CPWAIT	r0
	CPWAIT	r0
	.long 0
	.long 0

	mrc	p15,0,r0,c1,c0,0
	orr	r0,r0,#0x1		# enable the MMU
	mcr	p15,0,r0,c1,c0,0
	CPWAIT	r0
	
	mcr	p15,0,r0,c7,c10,4	# drain the write & fill buffers
	CPWAIT	r0
	
	ldmfd	sp!,{pc}
#endif
	.end

