/* msf.c							  
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: dalsraja, March, 2002
 *  revisions:
 * 
 * 01b, 09/24/03 ajp Added flow control training sequence
 * 01a, 09/24/03 ajp Download proper cbus ucode in msf -c test
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"  
#include "ixp2x00.h"
#include "msf.h"
#include "msf_common.h"
#include "diag_utils.h"
#include "pci_diag.h"
#include "led.h"
#include "msf_unicast_tx.uh"
#include "msf_unicast_rx.uh"
#include "msf_cbus_tx.uh"
#include "msf_cbus_rx.uh"

#define SRAM_BASE_ADDR					0x100000

static void Msf_Help(void);

const UINT32 MsfRegister[] = {
	MSF_RX_CONTROL,
	MSF_TX_CONTROL,
	CSIX_TYPE_MAP,
	HWM_CONTROL,
	RX_MPHY_POLL_LIMIT,
	RX_THREAD_FREELIST_TIMEOUT_0,
	RX_THREAD_FREELIST_TIMEOUT_1,
	RX_THREAD_FREELIST_TIMEOUT_2
};

const UINT32 MsfRegisterMask[] = {
	MSF_RX_CONTROL_MASK,
	MSF_TX_CONTROL_MASK,
	CSIX_TYPE_MAP_MASK,
	HWM_CONTROL_MASK,
	RX_MPHY_POLL_LIMIT_MASK,
	RX_THREAD_FREELIST_TIMEOUT_0_MASK,
	RX_THREAD_FREELIST_TIMEOUT_1_MASK,
	RX_THREAD_FREELIST_TIMEOUT_2_MASK,
	RX_THREAD_FREELIST_TIMEOUT_3_MASK
};


// msf_unicast_data[] will be stored to DRAM and used to generate a Unicast CFrame
// for the Unicast CFrame test
const UINT32 msf_unicast_data[] =	{ 0x00010203, 0x04050607, 0x08090A0B, 0x0C0D0E0F,
									  0x10111213, 0x14151617, 0x18191A1B, 0x1C1D1E1F,
									  0x20212223, 0x24252627, 0x28292A2B, 0x2C2D2E2F,
									  0x30313233, 0x34353637, 0x38393A3B, 0x3C3D3E3F };

// msf_cbus_data[] will be stored to DRAM and used to generate a Flow Control CFrame
// for the CBUS test
const UINT32 msf_cbus_data[] = { 0x3F3E3D3C, 0x3B3A3938, 0x37363534, 0x33323130,
								 0x2F2E2D2C, 0x2B2A2928, 0x27262524, 0x23222120,
								 0x1F1E1D1C, 0x1B1A1918, 0x17161514, 0x13121110,
								 0x0F0E0D0C, 0x0B0A0908, 0x07060504, 0x03020100 };

// User input data
typedef struct _MSF_testIn
{
	UINT32 register_sel;
	UINT32 unicast_sel;
	UINT32 cbus_sel;
	UINT32 all_sel;
} MSF_testIn;


//********************************************************************************************
// Function: void Msf_Test(void)
//
// Determine which MSF test is called - MSF register test, unicast CFrame test, CBUS
// 		Register test - writes and reads a mask to a selected few registers
// 		Unicast CFrame test - Execute the Unicast CFrame test
// 		CBUS test - tests the CBUS functionality
// 
// Called by Test Manager
//********************************************************************************************

int Msf_Test(MSF_testIn* pIn, PassFail_testOut* pOut)
	{
	int passfail1 = 0, passfail2 = 0, passfail3 = 0;

	Scroll_Off();

   	if ((pIn->register_sel) || (pIn->all_sel))
		Msf_Register_Test();												  

 	if ((pIn->unicast_sel) || (pIn->all_sel))
		passfail2 = Msf_Unicast_Test();

   	if ((pIn->cbus_sel) || (pIn->all_sel))
		passfail3 = Msf_Cbus_Test();

	pOut->PassFail = (passfail1 | passfail2 | passfail3);

	pOut->ReturnCode = PASS;         
        
	return(pOut->PassFail);

	}

// ********************************************************************************************
// Function: void Msf_Register_Test(void)
//
// Writes and reads a mask to a selected few MSF registers and checks whether the data
// written is the same as the data read from the registers.
// ********************************************************************************************
int Msf_Register_Test(void)
{
	int ctr, original_val, read_val, flag;

	flag = OK;
	
	for (ctr = 0; ctr < (sizeof(MsfRegister)/sizeof(UINT32)); ctr++)
	{
		// Read the value of the register before performing the test
		original_val = GET32(MSF_BASE + MsfRegister[ctr]);

		// Write 1s to the Readable/Writeable bits using the predefined register mask
		PUT32((MSF_BASE + MsfRegister[ctr]), MsfRegisterMask[ctr]);

		// Read the value from the register
		read_val = GET32(MSF_BASE + MsfRegister[ctr]) & MsfRegisterMask[ctr];

		// Compare the read value and written value
		// If not the same, display a message
		if (read_val != MsfRegisterMask[ctr])
		{
			printf("MSF Register:0x%X  Write value:0x%X  Read value:0x%X\n", (MSF_BASE + MsfRegister[ctr]), MsfRegisterMask[ctr], read_val);
			flag = ERROR;
		}

		// Write 0s to the Readable/Weiteable bits
		PUT32((MSF_BASE + MsfRegister[ctr]), 0);

		// Read the value from the register
		read_val = GET32(MSF_BASE + MsfRegister[ctr]) & MsfRegisterMask[ctr];

		// Compare the read value and written value
		// If not the same, display a message
		if (read_val != 0)
		{
			printf("MSF Register:0x%X  Write value:0x0  Read value:0x%X\n", (MSF_BASE + MsfRegister[ctr]), read_val);
			flag = ERROR;
		}

		// Restore the original value of the register before the test was run
		PUT32((MSF_BASE + MsfRegister[ctr]), original_val);
	}


	if (flag == OK)
	{
		printf("\nMSF Register test...PASSED\n");
		return(PASS);
	}
	else
	{
		printf("\nMSF Register test...FAILED\n");
		return(FAIL);
	}		
}


// ********************************************************************************************
// Function: void Msf_Unicast_Test(void)
//
// Startes the MSF Unicast CFrame test. It can only be initiated by the Master NPU.
// Once the Master NPU has completed its init, it initiaies a PCI Doorbell interrupt on the
// Slave NPU to get the slave NPU to initialise itself and start the test.
//
// Called by Msf_Test()
// ********************************************************************************************
int Msf_Unicast_Test(void)
{
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, doorbell_intr_addr;
	UINT32 mailbox_value, prev_mailbox_val, flag;
	UINT32 ctr, loop_count, status, forever = 0;
	UINT32 dcache_status;
	UINT32 OTHER_NPUs_PCI_CSR_BASE;
	UINT32 Egress_msf_Scratch_addr;
	UINT32 Ingress_msf_Scratch_addr;
	UINT32 Tx_Pkts, Rx_Pkts, Rx_Errs;									 

	int i;
	UINT32 rx_data, test_status;

	if(!IS_EGRESS_NPU)
		{
		printf("fabric unicast test should be run from Egress only\n");
		return(FAIL);
		}

	Scroll_Off();

	// Test start message to Ingress user terminal
	printf("Starting Fabric unicast test:\n");

	OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

	//Zero out the Ingress mailbox register data
	ZeroEgressMailboxes();
	ZeroIngressMailboxes();

	Egress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

	//Get the Ingress' Scratch start addr 
    Ingress_msf_Scratch_addr = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);
	Ingress_msf_Scratch_addr += SCRATCH_CSR_BASE_FRM_PCI;

	// Clear Egress and Ingress scratch memory areas for media test storage
	for(i=0; i < XS_MEDIA_STORAGE_SIZE; i+=4)		 
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + i) = 0;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + i) = 0;
	}
					   
	// Requested MAC port # to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACNUMBER) = 0;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACNUMBER) = SWAP32(0);

	// User entered packet count
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = 1;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(1);
	
	// Port count goes to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = 1;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = SWAP32(1);
	
	// write 64 byte transmit packet data to Egress memory for ucode to use
	for (i=0; i < UENG_CNT; i++)
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = msf_unicast_data[i];
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = SWAP32(msf_unicast_data[i]);
	}

	// Initiate a doorbell interrupt on the slave it perform the loading
	// of the microcode on the slave's NPU.
	Set_Pci_Doorbell(MSF_U_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	// Wait for Ingress to send (DM_CSIX_IMI1) message
	if((egress_wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_IMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI1\n");
		return(FAIL);
	}

    egress_media_init_1();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI1) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI1;


	// Wait for Ingress to send (DM_CSIX_IMI2) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI2\n");
		return(FAIL);
	}

    test_status = egress_media_init_2();
	if(test_status == FAIL)
		{
		printf("Egress failed to rx data training\n");
		Cleanup_before_return();
		return(FAIL);
		}
		
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI2) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI2;


	// Wait for Ingress to send (DM_CSIX_IMI3) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI3\n");
		return(FAIL);
	}

	test_status = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2));

	if(test_status == FAIL)
		{
		printf("Ingress failed to rx flow control\n");
		Cleanup_before_return();
		return(FAIL);
		}

    egress_media_init_3();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI3) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI3;


	// Wait for Ingress to send (DM_CSIX_IMI4) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI4\n");
		return(FAIL);
	}

	// Load receive ucode module
	DownLoadUcode(UNICAST_RX_ME, sizeof(msf_unicast_rx), (UINT32 *)msf_unicast_rx);
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI4) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI4;


	// wait for packets to be rx'd by receive ucode; error if too long
 	if ((wait_mailbox_msg(4000, DM_PACKETSRCVD)) == FAIL)
	{
		printf("Timed out...Slave NPU did not receive all data packets\n");
	}										 

	// Message (DM test_complete) message to Ingress	
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_TEST_COMPLETE;
				 


	// Test results displayed to Egress user terminal 										  
	printf("Fabric CSIX test results\n");

	// Read the Ingress' tx'd data
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXPACKETCOUNT));
																		   
	Tx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT);
	Rx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT);
	Rx_Errs = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR);
	printf("Port 1 Tx Packets 0x%X Rx Packets 0x%X Rx Errors 0x%X\n", Tx_Pkts, Rx_Pkts, Rx_Errs);
	

	// Test complete message to Egress user terminal
	printf("Fabric CSIX unicast test complete\n");
				 
	// Clear the mailbox registers
	ZeroEgressMailboxes();
	ZeroIngressMailboxes();
																						  
	// Cleanup and exit test
	Cleanup_before_return();
	
	Msf_OutOfReset();

	if(Rx_Errs) 
		return(FAIL);
	if(Tx_Pkts != Rx_Pkts) 
		return(FAIL);
	else
		return(PASS);

}


// ***************************************************************************************************
// Function: void Msf_Unicast_Test_Slave(void)
//
// This test is run on the slave and is called by the PCI Doorbell Interrupt handler function after
// the Master NPU initiates a PCI Doorbell interrupt
// It initialises the Slave NPU by loading the microcode and starts the Unicast CFrame test.
//
// Called by doorbell_intr_received() (found in pci_diag.c)
// ***************************************************************************************************
void Msf_Unicast_Test_Slave(void)
{
	UINT32 mask=0, mailbox_value, status, total_len, packet_num, rx_num_of_bytes=0;
	UINT32 get_ring_addr, put_ring_addr, loop_count, forever;
	UINT32 ctr, temp=0, flag, num_lw, lw1, lw2;
	UINT dcache_status;
	int payload_len;
	int flow_status;

	Scroll_Off();

	printf("Starting CSIX fabric unicast test:\n");
	
	ingress_media_init_1();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI1;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI1\n");
		return;
	}

	ingress_media_init_2();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI2;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI2\n");
		return;
	}

	flow_status = ingress_media_init_3();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI3;
	// Let egress know if we failed to get flow control
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2) = flow_status;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI3\n");
		return;
	}

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI4;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI4\n");
		return;
	}

	// Load transmit ucode module	
	DownLoadUcode(UNICAST_TX_ME, sizeof(msf_unicast_tx), (UINT32 *)msf_unicast_tx);

	DelayNMsec(10);

	if((wait_mailbox_msg(0x3000, SWAP32(DM_TEST_COMPLETE))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_TEST_COMPLETE\n");
		return;
	}

	printf("Ingress unicast test complete\n");

	// Cleanup and exit test
	Cleanup_before_return();
	
	Msf_OutOfReset();
}



// ********************************************************************************************
// Function: void Msf_Cbus_Test(void)
//
// Startes the MSF CBUS test. It can only be initiated by the Master NPU.
// Once the Master NPU has completed its init, it initiaies a PCI Doorbell interrupt on the
// Slave NPU to get the slave NPU to initialise itself and start the test.
//
// Called by Msf_Test()
// ********************************************************************************************
int Msf_Cbus_Test(void)
{
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, doorbell_intr_addr;
	UINT32 mailbox_value, prev_mailbox_val, flag;
	UINT32 ctr, loop_count, status, forever = 0;
	UINT32 dcache_status;
	UINT32 OTHER_NPUs_PCI_CSR_BASE;
	UINT32 Egress_msf_Scratch_addr;
	UINT32 Ingress_msf_Scratch_addr;
	UINT32 Tx_Pkts, Rx_Pkts, Rx_Errs;									 

	int i;
	UINT32 rx_data, test_status;

	if(!IS_EGRESS_NPU)
		{
		printf("Fabric cbus test should be run from Egress only:\n");
		return(FAIL);
		}

	Scroll_Off();

	// Test start message to Ingress user terminal
	printf("Egress starting Fabric cbus test:\n");

	OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

	//Zero out the Ingress mailbox register data
	ZeroEgressMailboxes();
	ZeroIngressMailboxes();

	Egress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

	//Get the Ingress' Scratch start addr 
    Ingress_msf_Scratch_addr = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);
	Ingress_msf_Scratch_addr += SCRATCH_CSR_BASE_FRM_PCI;

	// Clear Egress and Ingress scratch memory areas for media test storage
	for(i=0; i < XS_MEDIA_STORAGE_SIZE; i+=4)		 
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + i) = 0;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + i) = 0;
	}
					   
	// Requested MAC port # to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACNUMBER) = 0;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACNUMBER) = SWAP32(0);

	// User entered packet count
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = 1;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(1);
	
	// Port count goes to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = 1;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = SWAP32(1);
	
	// write 64 byte transmit packet data to Egress memory for ucode to use
	for (i=0; i < UENG_CNT; i++)
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = msf_unicast_data[i];
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = SWAP32(msf_unicast_data[i]);
	}

	// Initiate a doorbell interrupt on the slave it perform the loading
	// of the microcode on the slave's NPU.
	Set_Pci_Doorbell(MSF_C_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	// Wait for Ingress to send (DM_CSIX_IMI1) message
	if((egress_wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_IMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI1\n");
		return(FAIL);
	}

    egress_media_init_1();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI1) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI1;


	// Wait for Ingress to send (DM_CSIX_IMI2) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI2\n");
		return(FAIL);
	}

    test_status = egress_media_init_2();
	if(test_status == FAIL)
		{
		printf("Egress failed to rx data training\n");
		Cleanup_before_return();
		return(FAIL);
		}
		
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI2) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI2;


	// Wait for Ingress to send (DM_CSIX_IMI3) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI3\n");
		return(FAIL);
	}

	test_status = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2));

	if(test_status == FAIL)
		{
		printf("Ingress failed to rx flow control\n");
		Cleanup_before_return();
		return(FAIL);
		}

    egress_media_init_3();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI3) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI3;


	// Wait for Ingress to send (DM_CSIX_IMI4) message
	if((egress_wait_mailbox_msg(4000, SWAP32(DM_CSIX_IMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI4\n");
		return(FAIL);
	}

	// Load receive ucode module
	DownLoadUcode(CBUS_RX_ME, sizeof(msf_cbus_rx), (UINT32 *)msf_cbus_rx);
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI4) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI4;


	// wait for packets to be rx'd by receive ucode; error if too long
 	if ((wait_mailbox_msg(4000, DM_PACKETSRCVD)) == FAIL)
	{
		printf("Timed out...Slave NPU did not receive all data packets\n");
	}										 

	// Message (DM test_complete) message to Ingress	
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_TEST_COMPLETE;


	// Test results displayed to Egress user terminal 										  
	printf("Fabric CSIX test results\n");

	// Read the Ingress' tx'd data
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXPACKETCOUNT));
																		   
	Tx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT);
	Rx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT);
	Rx_Errs = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR);
	printf("Port 1 Tx Packets 0x%X Rx Packets 0x%X Rx Errors 0x%X\n", Tx_Pkts, Rx_Pkts, Rx_Errs);

	
	// Test complete message to Egress user terminal
	printf("Fabric CSIX cbus test complete\n");
				 
	// Cleanup and exit test
	Cleanup_before_return();
	
	Msf_OutOfReset();

	if(Rx_Errs) 
		return(FAIL);
	if(Tx_Pkts != Rx_Pkts) 
		return(FAIL);
	else
		return(PASS);

}


// ***************************************************************************************************
// Function: void Msf_Cbus_Test_Slave(void)
//
// This test is run on the slave and is called by the PCI Doorbell Interrupt handler function after
// the Master NPU initiates a PCI Doorbell interrupt
// It initialises the Slave NPU by loading the microcode and starts the CBUS test.
//
// Called by doorbell_intr_received() (found in pci_diag.c)
// ***************************************************************************************************
void Msf_Cbus_Test_Slave(void)
{
	UINT32 mask=0, mailbox_value, status, total_len, packet_num, rx_num_of_bytes=0;
	UINT32 get_ring_addr, put_ring_addr, loop_count, forever;
	UINT32 ctr, temp=0, flag, num_lw, lw1, lw2;
	UINT dcache_status;
	int payload_len;
	int flow_status;

	Scroll_Off();

	printf("Ingress starting CSIX fabric cbus test\n");
	
	ingress_media_init_1();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI1;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI1\n");
		return;
	}

	ingress_media_init_2();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI2;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI2\n");
		return;
	}

	flow_status = ingress_media_init_3();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI3;
	// Let egress know if we failed to get flow control
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2) = flow_status;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI3\n");
		return;
	}

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI4;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI4\n");
		return;
	}

	// Load transmit ucode module	
	DownLoadUcode(CBUS_TX_ME, sizeof(msf_cbus_tx), (UINT32 *)msf_cbus_tx);

	DelayNMsec(10);

	if((wait_mailbox_msg(0x3000, SWAP32(DM_TEST_COMPLETE))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_TEST_COMPLETE\n");
		return;
	}

	printf("Ingress cbus test complete\n");

	// Cleanup and exit test
	Cleanup_before_return();
	
	Msf_OutOfReset();

}

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_MSF,"msf",Msf_Test,DIAGNOSTIC,"MSFTest","media/switch fabric register, unicast, and Cbus tests");
Switch("r", BOOLEAN32ARG, "MSF register test", "");
Switch("u", BOOLEAN32ARG, "MSF unicast", "");
Switch("c", BOOLEAN32ARG, "MSF Cbus test", "");
CommandEnd();


   