/* pci_diag.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: dalsraja, Feb 2002
 * 
 * Revisions:
 * 01a, 09/15/03 ajp Added pass/fail returns for script file break on error
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2000pci.h"
#include "ixp2x00.h"
#include "pci_dma.h"
#include "21555.h"
#include "led.h"
#include "diag_utils.h"
#include "pci_diag.h"

static void i21555SromWriteEnable(void);
static void i21555SromWriteDisable(void);
static void Doorbell_Intr_Msg(int bit_no);
static UINT8 i21555SromRead(int reg);
static int i21555SromErase(int reg);
HANDLE DOORBELLISR;

#define DELAY_TIME 30
#define REG_SIZE 32
#define CTR_SIZE 1024
#define DRAM_ADDR_MASK 0xDADA0000

static unsigned char sromPreloadMask[] = {
	0xc0, /* [7:6] = 10b to enable serial preload, [5:0] reserved */
	0x00, 0x00, 0x00,
	0xff, /* primary programming interface */
	0xff, /* primary sub-class code */
	0xff, /* primary base clase code */
	0xff, /* subsystem vendor id [7:0] */
	0xff, /* subsystem vendor id [15:8] */
	0xff, /* subsystem id [7:0] */
	0xff, /* subsystem id [15:8] */
	0xff, /* primary minimum grant */
	0xff, /* primary maximum latency */
	0xff, /* secondary programming interface */
	0xff, /* secondary sub-class code */
	0xff, /* secondary base class code */
	0xff, /* secondary minimum grant */
	0xff, /* secondary maximum latency */
	0x06, /* downstream memory 0 setup [7:0] */
	0xf0, /* downstream memory 0 setup [15:8] */
	0xff, /* downstream memory 0 setup [23:16] */
	0xff, /* downstream memory 0 setup [31:24] */
	0xcf, /* downstream i/o or memory 1 setup [7:0] */
	0xff, /* downstream i/o or memory 1 setup [15:8] */
	0xff, /* downstream i/o or memory 1 setup [23:16] */
	0xff, /* downstream i/o or memory 1 setup [31:24] */
	0x06, /* downstream memory 2 setup [7:0] */
	0xf0, /* downstream memory 2 setup [15:8] */
	0xff, /* downstream memory 2 setup [23:16] */
	0xff, /* downstream memory 2 setup [31:24] */
	0x06, /* downstream memory 3 setup [7:0] */
	0xf0, /* downstream memory 3 setup [15:8] */
	0xff, /* downstream memory 3 setup [23:16] */
	0xff, /* downstream memory 3 setup [31:24] */
	0xff, /* downstream memory 3 setup upper 32 bits [7:0] */
	0xff, /* downstream memory 3 setup upper 32 bits [15:8] */
	0xff, /* downstream memory 3 setup upper 32 bits [23:16] */
	0xff, /* downstream memory 3 setup upper 32 bits [31:24] */
	0xf1, /* bit [0] primary expansion ROM setup (enable), bits [7:4] primary expansion ROM setup [15:11] */
	0xff, /* primary expansion ROM setup [23:16] */
	0xcf, /* upstream i/o or memory 0 setup [7:0] */
	0xff, /* upstream i/o or memory 0 setup [15:8] */
	0xff, /* upstream i/o or memory 0 setup [23:16] */
	0xff, /* upstream i/o or memory 0 setup [31:24] */
	0x06, /* upstream memory 1 setup [7:0] */
	0xf0, /* upstream memory 1 setup [15:8] */
	0xff, /* upstream memory 1 setup [23:16] */
	0xff, /* upstream memory 1 setup [31:24] */
	0xff, /* chip control 0 [7:0] */
	0xcf, /* chip control 0 [15:8] */
	0xff, /* chip control 1 [7:0] */
	0xff, /* chip control 1 [15:8] */
	0xff, /* arbiter control [7:0] */
	0x03, /* arbiter control [15:8] */
	0x7f, /* primary SERR# disable [7:0] */
	0x7f, /* secondary SERR# disable [7:0] */
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff, /* power management data 0-7 */
	0x00, /* reserved */
	0xfc, /* [2] BiST supported, [3] power mgmt. data enable, [5:4] power mgmt. csr [14:13],
		  [7:6] power mgmt. capabilities [1:0] */
	0xff /* [0] power mgmt. cap. [2], [1] power mgmt. cap. [5], [7:2] power mgmt. cap [14:9] */
};

	typedef struct _PCI_testIn
	{
//		UINT32 search_sel;
		UINT32 doorbell_sel;
		UINT32 dma_sel;
		UINT32 all_sel;
		UINT32 i21154_sel;
		UINT32 i21555_sel;
	} PCI_testIn;

	typedef struct _i21555SROM_testIn
	{
		UINT8 *addr;
	} i21555SROM_testIn;







//********************************************************************************************
// Function: void Pci_Test(void)
//
// Determine which PCI test is called. The available tests are:-
//		Find the non-transparent bridge
//		Find the transparent bridge
// 		PCI doorbell interrupt test
// 		PCI DMA test
// 
// Called by Test Manager
//********************************************************************************************
int Pci_Test(PCI_testIn * pIn, PassFail_testOut* pOut)
{

	char opt;
	int status1 = PASS;
	int status2 = PASS;
	int status3 = PASS;
	int status4 = PASS;

	//decide what test the user has picked	
	//if ((!pIn->doorbell_sel) || (!pIn->all_sel) || (!pIn->dma_sel) ||  (!pIn->i21154_sel) || (!pIn->i21555_sel))
	//	pIn->all_sel = 1;

	if ((pIn->doorbell_sel) || (pIn->all_sel))
		status1 = Pci_Doorbell_Test();

	if ((pIn->dma_sel) || (pIn->all_sel))
		status2 = Pci_Dma_Test();

	if (pIn->i21154_sel) 
		status3 = Find_i21154();

	if (pIn->i21555_sel) 
		status4 = Find_i21555();

	pOut->PassFail = (status1 | status2 | status3 | status4);

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);
}

//********************************************************************************************
// Function: void find_i21555(void)
//
// This routine looks for the i21555 device
//
// Inputs:	NONE
//
// Outputs:	NONE
//********************************************************************************************

int Find_i21555(void)
{
	UINT32 bus_number, device_number, function_number;

	if (pciFindDevice (VENDOR_INTEL, DEVICE_21555, 0, &bus_number, &device_number, &function_number) != OK)
	{
		printf("i21555 Device not found!\n");
		DelayNUsec(1000000);

		return(FAIL);
	}

	else
	{
		printf("i21555 found...\n");
	}


	pciConfigInLong (bus_number, device_number, function_number, PCI_SECONDARY_CSR_IO_BAR, 
			 &ntbInfo.ioBase);

	ntbInfo.ioBase = (BRD_PCIIO_PCI2CPU(ntbInfo.ioBase, 0) & 0xfffffffc); 

	if (ntbInfo.ioBase == 0)
	{
		printf("i21555 No resources assigned!\n");
	}

	printf("21555 found at :\n");
	printf("  PCI Bus Number : %d\n", bus_number);
	printf("  Device Number  : %d\n", device_number);
	printf("  Function Number: %d\n", function_number);
//	printf("  IOBase: 0x%x\n", ntbInfo.ioBase);
 
	return(PASS);
}




// i21555SromWriteEnable
//
// This routine sets the 21555 SROM to the write enable state.  
//
// RETURNS: Nothing
//															
static void i21555SromWriteEnable(void)
{
	PUT32((ntbInfo.ioBase+I21555_ROM_ADDR), I21555_ROM_ADDR_WriteEnable);
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		DelayNUsec(10000);	// 1 ms delay
	}

	return;
}

// i21555SromWriteDisable
//
// This routine sets the 21555 SROM to the write disabled state.  
//
// RETURNS: Nothing
//
static void i21555SromWriteDisable(void)
{
	PUT32((ntbInfo.ioBase+I21555_ROM_ADDR), I21555_ROM_ADDR_WriteDisable);
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		DelayNUsec(10000);	// 1 ms delay
	}

	return;
}



// i21555SromWrite
//
// This routine writes a byte to a location in the 21555 SROM
//
// RETURNS: STATUS
//
static void i21555SromWrite(int reg, unsigned char val)
{
	i21555SromWriteEnable();
	PUT32((ntbInfo.ioBase+I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Write));
	PUT8((ntbInfo.ioBase+I21555_ROM_DATA), (val & sromPreloadMask[reg]));
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		DelayNUsec(10000);	// 1 ms delay
	}
	
	// This needs to be done again. Without it, apparently PCI does not work properly
	PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		DelayNUsec(10000);	// 1 ms delay
	}

	while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromPoll)
	{
		PUT8((ntbInfo.ioBase+I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);
	
		while (GET8(ntbInfo.ioBase+I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
		{
			DelayNUsec(10000);	// 1 ms delay
		}
	}
	i21555SromWriteDisable();
}


// i21555SromRead
// 
// This routine reads a byte from a location in the 21555's SROM
//
// RETURNS: SROM Data which was read from the address specified
//
static UINT8 i21555SromRead(int reg)
{
	PUT32((ntbInfo.ioBase + I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Read));

	while (GET8(ntbInfo.ioBase + I21555_ROM_CTRL) & (I21555_ROM_CTRL_SromStartBusy | I21555_ROM_CTRL_PromStartBusy))
	{
		DelayNUsec(1000);	// 1 ms delay
	}

	PUT8((ntbInfo.ioBase + I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);

	while (GET8(ntbInfo.ioBase + I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy)
	{
		DelayNUsec(1000);	// 1 ms delay
	}

	return GET8(ntbInfo.ioBase + I21555_ROM_DATA);
}


// i21555SromErase
//
// This routine erases a byte in the 21555 SROM
//
// RETURNS: STATUS
//
static int i21555SromErase(int reg)
{
	i21555SromWriteEnable();

	PUT32((ntbInfo.ioBase + I21555_ROM_ADDR), (reg | I21555_ROM_ADDR_Erase));

	PUT8((ntbInfo.ioBase + I21555_ROM_CTRL), I21555_ROM_CTRL_SromStartBusy);

	while (GET8((ntbInfo.ioBase + I21555_ROM_CTRL) & I21555_ROM_CTRL_SromStartBusy))
	{
		DelayNUsec(1000);	// 1 ms delay
	}

	i21555SromWriteDisable();

	return 0;
}


// i21555SromReprogram
//
// This routine reprograms the 21555 SROM from the supplied address
//
// RETURNS: STATUS
//

int i21555SromReprogramAll(i21555SROM_testIn * pIn)
{
	int ctr;

	Find_i21555();

	for (ctr=0;ctr<SROM_NUM_BYTES;ctr++)
	{
		if (*pIn->addr == i21555SromRead(ctr))	// if newData == oldData
		{
			pIn->addr++;							// if newData == oldData, skip
			continue;
		}
		else
		{
//			i21555SromErase(ctr);					// else erase and write new value
			i21555SromWrite(ctr, *pIn->addr++);
		}
	}

	return 0;
}


// i21555SromDump
//
// This routine dumps the contents of the 21555 SROM to memory.
//
// RETURNS: STATUS
//
int i21555SromDump(i21555SROM_testIn * pIn)
{
	int ctr;

	Find_i21555();

	for (ctr=0;ctr<SROM_NUM_BYTES;ctr++)
	{
		*pIn->addr++ = i21555SromRead(ctr);
	}

	return 0;
}




// Function: void find_i21154(void)
//
// This routine looks for the i21154 device
//
// Inputs:	NONE
//
// Outputs:	NONE
//

int Find_i21154(void)
{
	UINT32 bus_number, device_number, function_number;

	if (pciFindDevice (TB_VENDOR_ID, TB_DEVICE_ID, 0, &bus_number, &device_number, &function_number) != OK)
	{
	 	printf("i21554 was not found...\n");
	 	DelayNUsec(1000000);

	 	return(FAIL);
	}

	else
	{
	 	printf("i21154 was found...\n");
 	}

	printf("21154 found at :\n");
	printf("  PCI Bus Number : %d\n", bus_number);
	printf("  Device Number  : %d\n", device_number);
	printf("  Function Number: %d\n", function_number);
//	printf("  IOBase: 0x%x\n", ntbInfo.ioBase);

	return(PASS);
 
}

// void Pci_Doorbell_Test(void)
//
// This routine initiates the PCI doorbell test
//
// Inputs:	NONE
//
// Outputs:	NONE
//
int Pci_Doorbell_Test(void)
{
	int ctr;
	int doorbell_intr_addr;
	
	UINT32 OTHER_NPUs_PCI_CSR_BASE;
	UINT32 ADDR_EXT_REG;

	if(IS_EGRESS_NPU)		// if master NPU
	{
		printf("Pci_Doorbell_Test - Master\n");
		OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

	}
	else											// if slave NPU
	{
		printf("Pci_Doorbell_Test - Slave\n");
		OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(MASTER_NPU);
//		OTHER_NPUs_PCI_CSR_BASE += PCI_MEM_BASE;
		OTHER_NPUs_PCI_CSR_BASE |= 0xE0000000;

		// if slave read and save address extension register
		ADDR_EXT_REG = *(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + PCI_ADDR_EXT); 

		// then clear the low word so that slave can see master's memory space
		*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + PCI_ADDR_EXT) = (ADDR_EXT_REG & 0xFFFF0000);

	}

	// Show the other NPU's base address
	//printf("DEBUG:OTHER_NPUs_PCI_BASE = 0x%X\n", OTHER_NPUs_PCI_CSR_BASE);

	// Write a pattern to the other NPU's Mailbox0 register to init
	// the location. This register will be polled later on by the NPU
	// running this function. Once the other NPU has received the doorbell
	// interrupt, will write a value of 0 to this register.

   	//printf("DEBUG: 	OTHER_NPUs_PCI_CSR_BASE = 0x%X\n", OTHER_NPUs_PCI_CSR_BASE);
	//printf("DEBUG:  PCI_CSR_BASE_FRM_PCI = 0x%X\n", PCI_CSR_BASE_FRM_PCI);
	//printf("DEBUG:  MAILBOX_0 =	0x%X\n", MAILBOX_0);

	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DBELL_PATT1;
	
	// Write a '1' to bit 0 of the other NPU's PCI_DOORBELL register to initiate
	// the doorbell interrupt test

	Set_Pci_Doorbell(DBELL_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));
	
	// Poll on the other NPU's Mailbox0 register and check if it is 0. The 
	// other NPU's Mailbox0 register is set to 0 by the the other NPU's XScale
	// as part of the diag code. If the other NPU does not indicate that it has
	// received the doorbell interrupt after 30ms, it is assumed that the test failed


	//printf("DEBUG: 1 slave_mailbox_0 = 0x%x\n", *(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0)); 

	//	DelayNUsec(20000000);		// 1ms delay

	//printf("DEBUG: 2 slave_mailbox_0 = 0x%x\n", *(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0)); 



	for (ctr = 0; ctr < DELAY_TIME; ctr++)
	{
		DelayNUsec(2000);		// 1ms delay


		if (*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
							PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) == DBELL_PATT3)
	   	{
			break;
		}

		if (ctr == (DELAY_TIME - 1))
		{
			printf("\nThe %s did not signal that doorbell interrupt was received\n", (IS_EGRESS_NPU ? "slave" : "master"));
			printf("\nPCI doorbell interrupt test...FAIL\n");

			if(!IS_EGRESS_NPU)		// if slave write address extension register
			{
				// if slave restore the address extension register
				*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + PCI_ADDR_EXT) = ADDR_EXT_REG;
			}

			DelayNUsec(2000);
			return(FAIL);
		}					 
	}

	printf("\nMailbox 0 of %s indicates that the interrupt was received...\n", (IS_EGRESS_NPU ? "slave" : "master"));
	printf("\nPCI doorbell interrupt test...PASS\n");

	if(!IS_EGRESS_NPU)		// if slave write address extension register
	{
		// if slave restore the address extension register
		*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + PCI_ADDR_EXT) = ADDR_EXT_REG;
	}

	return(PASS);
}


// Function: void Doorbell_Intr_Received(void)
//
// This routine is initiated when a doorbell interrupt is received
//
// Inputs:	NONE
//
// Outputs:	NONE
//
// This assumes that there is only one doorbell interrupt, as the diags are a controlled environment
// So after finding one interrupt, it exits the loop

void Doorbell_Intr_Received(void)
{
	int doorbell_reg_value;

	// Read the XSCALE_DOORBELL register
	doorbell_reg_value = *(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + XSCALE_DOORBELL_OFF) ;

	if (doorbell_reg_value == -1)
	{
	 	printf("Error: Entered doorbell interrupt but no int bits set\n");
	}

	else 
	{
		// Clear the interrupt
		*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + XSCALE_DOORBELL_OFF) = -1;

		//printf("DEBUG: doorbell1 int = 0x%x\n", doorbell_reg_value);
		//printf("DEBUG: doorbell2 int = 0x%x\n", ~doorbell_reg_value);
		//printf("DEBUG: doorbell3 int = 0x%x\n", (~doorbell_reg_value & DBELL_BIT));


		if (~doorbell_reg_value & (1<<DBELL_BIT))
			{
			printf("Got Doorbell DBELL_BIT int\n");
			//Doorbell_Intr_Msg(DBELL_BIT);
			//Write to this NPU's mailbox 0	to signify dbell rx'd
			*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DBELL_PATT2;
			}
	
		else if (~doorbell_reg_value & (1<<MSF_U_BIT))
			{												   
			 printf("Got Doorbell MSF_U_BIT int\n");
			//Doorbell_Intr_Msg(MSF_U_BIT);
			Msf_Unicast_Test_Slave();
			}

		else if (~doorbell_reg_value & (1<<MSF_C_BIT))
			{
			 printf("Got Doorbell MSF_C_BIT int\n");
			//Doorbell_Intr_Msg(MSF_C_BIT);
			Msf_Cbus_Test_Slave();
			}
	
		else if (~doorbell_reg_value & (1<<SF_LB_BIT))
			{
			 printf("Got Doorbell SF_LB_BIT int\n");
			//Doorbell_Intr_Msg(SF_LB_BIT);
			Sf_Loopback_Test_Slave();
			}
		else if (~doorbell_reg_value & (1<<MEDIA_LB_BIT))
			{
			 printf("Got Doorbell MEDIA_LB_BIT int\n");
			//Doorbell_Intr_Msg(MEDIA_LB_BIT);
			Media_Loopback_Test_Slave();
			}
		else if (~doorbell_reg_value & (1<<SF_SPI4_LB_BIT))
			{
			printf("Got Doorbell SF_SPI4_LB_BIT int\n");
			Media_Loopback_Test_Slave2();
			}
		else if (~doorbell_reg_value & (1<<READ_TEMP_BIT))
			{
			printf("Got Doorbell READ_TEMP_BIT int\n");
			ADM1031_Egress_temp_display();
			}
		else
			printf("Unknown doorbell int = 0x%x\n", doorbell_reg_value);

	}
}


// Function: static void Doorbell_Intr_Msg(int bit_no)
//
// This function will display which NPU is responding to a doorbell interrupt
// and also which bit the interrupt was sent to
//
// Inputs:	bit_no = bit number for which the doorbell interrupt was set
//
// Outputs:	NONE
//
static void Doorbell_Intr_Msg(int bit_no)
{
	// Read the configuration strapping to decide if PCI master
	if(IS_EGRESS_NPU)
	{
		printf("\nDoorbell interrupt bit %d received by master...\n", bit_no);
	}
	else		// If slave
	{
		printf("\nDoorbell interrupt bit %d received by slave...\n\n", bit_no);
	}
}


// Function: void Set_Pci_Doorbell(int bit_num, int db_intr_addr)
//
// This function when called will set the specified bit of the doorbell interrupt register
// of the other NPU
//
// Inputs:		 bit_num = the bit number on the doorbell register to set
//			db_intr_addr = the address of the other NPU's XScale Doorbell
//						    register (as seen over PCI)
//
// Outputs:	NONE
//
void Set_Pci_Doorbell(int bit_num, int db_intr_addr)
{
	int temp;

	// Write a '1' to the selected bit of the PCI_DOORBELL register
	// to initiate a doorbell interrupt
	temp = SET_BIT(bit_num);
	#ifdef __ARMEB__
	temp = SWAP32(temp);
	#endif
	*(volatile unsigned int *)(db_intr_addr) = temp;	

}


// Function: void Pci_Dma_Test(void)
//
// This function will call for the PCI DMA test to be run and display
// the result of the test
//
// Inputs:	NONE
//
// Outputs:	NONE
//
int Pci_Dma_Test(void)
{
	int ctr;
	UINT dcache_status;
	int status = PASS;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}

	for (ctr = 0; ctr < CTR_SIZE; ctr ++)
	{
		PUT32((DRAM_ADDR + (ctr * 4)), (DRAM_ADDR_MASK | ctr));
	}

	status = dmaTest();

	if (status == OK)
	{
		printf("PCI DMA test...PASS\n");
}
	else
	{
		printf("PCI DMA test...FAIL\n");
		status = FAIL;
	}

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}

	return(status);
}

#define DOORBELLIRQ (PIC_NUM_IRQ + 13)
#define DOORBELLTEST	0x55AA
//HANDLE DOORBELLISR;

void InitDoorbellIRQ (void)
	{
		
	/* Install the Handler */
	DOORBELLISR = InstallHandler (DOORBELLIRQ, (HANDLER)Doorbell_Intr_Received, (void *)DOORBELLTEST);

	/* Enable the IRQ for the doorbell ISR */
	EnableIRQ (DOORBELLISR);

	return;

	}



// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_PCIDIAG,"pci",Pci_Test,DIAGNOSTIC,"PCITest","test pci doorbell/dma and search for 21154 and 21555 bridges");
//Switch("f", BOOLEAN32ARG, "Search for PCI devices", "");
Switch("i", BOOLEAN32ARG, "pci doorbell interrupt test", "");
Switch("d", BOOLEAN32ARG, "pci DMA test", "");
Switch("a", BOOLEAN32ARG, "all pci tests", "");	
Switch("t", BOOLEAN32ARG, "find the 21154 transparent bridge", "");
Switch("n", BOOLEAN32ARG, "find the 21555 non-transparent bridge", "");
CommandEnd();

  

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_PCIBSROMR,"pcibsromr",i21555SromDump,DIAGNOSTIC,"i21555 SROM read","i21555 SROM read function");
Arg(UINT32ARG,"data dump to address", "enter the address in memory to dump data to");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_PCIBSROMW,"pcibsromw",i21555SromReprogramAll,DIAGNOSTIC,"i21555 SROM write","i21555 SROM write function");
Arg(UINT32ARG,"data dump from address", "enter the address in memory to dump data from");
CommandEnd();


						  