/* pci_dma.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: dalsraja, Feb 2002
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2000pci.h"
#include "ixp2x00.h"
#include "pci_dma.h"
#include "led.h"
#include "diag_utils.h"
#include "pci_diag.h"

/*=========================================================================*/
/* DMA engine test*/
/*=========================================================================*/
int dmaTest(void) 
{
	UINT32 x, /*passes=0, */loops, forever=0;

	// Deiva: The diTable[] was modified by removing the PCI_ADDR field
	static const dmaInfoType diTable[] = {

  /* |- channel to test*/
  /* | |- put initial descriptor in DMA registers flag*/
  /* | | |- first descriptor start address*/
  /* | | |          |- number of descriptors to create, based on this descriptor*/
  /* | | |          | |- descriptor span*/
  /* | | |          | |  |- buffer span*/
  /* | | |          | |  |        |- first DRAM address*/
  /* | | |          | |  |        |                   |- size of transfer*/
  /* | | |          | |  |        |                   |        |- prescale*/
  /* | | |          | |  |        |                   |        | |- iBdelay*/
  /* | | |          | |  |        |                   |        | | |- direction of DMA*/
  /* | | |          | |  |        |                   |        | | |         |- PCI read type*/
  /* | | |          | |  |        |                   |        | | |         |        |- PCI read length*/
  /* | | |          | |  |        |                   |        | | |         |        | |- DRAM read length*/
  /* | | |          | |  |        |                   |        | | |         |        | | |- padd*/
	{1,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	#if 0
	{2,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF1),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,2,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,3,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{3,1,DESC_START,0,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,4,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_LINE,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_LINE,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,PCItoDRAM,PCI_READ_MULTIPLE,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,3,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,2,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,1,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,0,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,0,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
//	#if 0
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,2,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,3,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},

	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,1,0,DRAMtoPCI,PCI_READ,0,4,0}},

	{1,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,1,3,0}},
	{2,1,DESC_START,2,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,0,2,0}},
	{1,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,0,1,0}},
	{2,1,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_MULTIPLE,1,4,0}},
	{1,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,1,4,0}},
	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ_LINE,1,4,0}},

	{2,0,DESC_START,3,16,XFERSIZE,(DRAM_ADDR+D_OFF0),{XFERSIZE,0,0,DRAMtoPCI,PCI_READ,0,4,0}},
	#endif
	{0,0,0,0,0,0,0,{0,0,0,0,0,0,0,0}}};


	/*loops = (ac->argc == 2) ? 1 : ac->decArg[2];*/
	loops = 0;
	if (loops == 0)
	{
		forever = 1;           /* if params = 0, then do forever */
	}

	printf("IXP2800 DMA engine test\n");

	for (x = 0; diTable[x].descriptorStart != 0; x++)
	{
		//printf("DEBUG: DMA channel iteration index: %02d\n",x);

		// initialize the descriptors (if needed)
		initDescriptors((dmaInfoType *)&diTable[x]);

		//printf("DEBUG: Starting DMA engines\n");

		//start the DMA engine(s)
		if (startDMA((dmaInfoType *)&diTable[x]) == ERROR)
		{
			return ERROR;
		}

		// poll waiting for completion
		if (waitForCompletion((dmaInfoType *)&diTable[x]) == ERROR)
		{
			return ERROR;
		}

		//printf("DEBUG: Verifying results\n");

		// verify the results are correct
		if (verifyResults((dmaInfoType *)&diTable[x]) == ERROR)
		{
			return ERROR;
		}
	}    
		
	return OK;
}


/*=========================================================================*/
/* initialize the memory descriptors*/
/*=========================================================================*/
void initDescriptors(dmaInfoType *dmaInfo)
{
	byteCountRegType bcReg;
	UINT32 *oldPtr = 0;
	UINT32 /*x, tmpDsc, */offset=0;
	UINT32 pci_addr;

	// Read the configuration strapping to decide if PCI master
//	if(IS_EGRESS_NPU)
  	//	{
	//		pci_addr = GET_OTHER_NPUs_PCI_DRAM_BASE(SLAVE_NPU);
	//	//pci_addr = (slave_dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	//	}
	//	else
	//	{
	//		pci_addr = GET_OTHER_NPUs_PCI_DRAM_BASE(MASTER_NPU);
	//
	//	//pci_addr = (master_dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	//	}

	pci_addr= *(volatile unsigned int *)(IXP2000_PCI_BASE + IXP2000_PCI_CFG_BASE_ADDRESS_2);

	pci_addr &= 0xFFFFFFF0;
	pci_addr += PCI_DRAM_OFFSET;

	//printf("DEBUG:pci_addr = 0x%X\n", pci_addr);

	// setup the bytecount register/descriptor bits
	bcReg.bits.byteCount = dmaInfo->control.sizeBytes;
	bcReg.bits.reserved = 0;	//bcReg.bits.iBDelay = dmaInfo->control.iBDelay;
	bcReg.bits.direction = dmaInfo->control.direction;
	bcReg.bits.eoChain = 0;

	//printf("DEBUG: In initDescriptors bcReg %08x\n",bcReg.dword);

	// if registerStart = 1, setup DMA transfer in DMA registers.

	// Channel = 3, is to do simultaneous DMA on channel 1 and 2.  If you want
	// to do a channel 3 registerStart must = 1 and you can not do a linked list
	// of the two channels at this time, so descriptorNum must = 0.

	if (dmaInfo->registerStart == 1)
	{
		/**/
		/* setup DMA registers to start the process*/
		/**/
		if (dmaInfo->channel&1)
		{
			//printf("\nDEBUG: pci_addr=0x%08X\n", pci_addr);
			//printf("DEBUG: dmaInfo->dramAddress=0x%08X\n", dmaInfo->dramAddress);
			//printf("DEBUG: dmaInfo->descriptorStart=0x%08X\n", dmaInfo->descriptorStart);
			//printf("DEBUG: bcReg.dword=0x%08X\n", bcReg.dword);

			PUT32(CHAN_1_PCI_ADDR, pci_addr);		// Deiva: This line was modified
			PUT32(CHAN_1_DRAM_ADDR, dmaInfo->dramAddress);
			PUT32(CHAN_1_DESC_PTR, dmaInfo->descriptorStart);
			PUT32(CHAN_1_BYTE_COUNT, bcReg.dword);

			//printf("\nDEBUG: CHAN_1_PCI_ADDR=0x%X\n", GET32(CHAN_1_PCI_ADDR));
			//printf("DEBUG: CHAN_1_DRAM_ADDR=0x%X\n", GET32(CHAN_1_DRAM_ADDR));
			//printf("DEBUG: CHAN_1_DESC_PTR=0x%X\n", GET32(CHAN_1_DESC_PTR));
			//printf("DEBUG: CHAN_1_BYTE_COUNT=0x%X\n\n", GET32(CHAN_1_BYTE_COUNT));

		}

		if (dmaInfo->channel&2)
		{
			PUT32(CHAN_2_PCI_ADDR, pci_addr + offset);		// Deiva: This line was modified
			PUT32(CHAN_2_DRAM_ADDR, dmaInfo->dramAddress+offset);
			PUT32(CHAN_2_DESC_PTR, dmaInfo->descriptorStart);
			PUT32(CHAN_2_BYTE_COUNT, bcReg.dword);

			//printf("\n\nDEBUG: CHAN_2_PCI_ADDR=0x%X\n", GET32(CHAN_2_PCI_ADDR));
			//printf("DEBUG: CHAN_2_DRAM_ADDR=0x%X\n", GET32(CHAN_2_DRAM_ADDR));
			//printf("DEBUG: CHAN_2_DESC_PTR=0x%X\n", GET32(CHAN_2_DESC_PTR));
			//printf("DEBUG: CHAN_2_BYTE_COUNT=0x%X\n\n", GET32(CHAN_2_BYTE_COUNT));

		}
	}
	else
	{
		// setup DMA descriptor pointer
		if (dmaInfo->channel&1)
		{
			PUT32(CHAN_1_DESC_PTR, dmaInfo->descriptorStart);
		}
		
		if(dmaInfo->channel&2)
		{
			PUT32(CHAN_2_DESC_PTR, dmaInfo->descriptorStart+offset);
		}
	}


	// now setup the rest of the descriptors
//	for (x = 0; x < dmaInfo->descriptorNum; x++)
//	{
//		dscPtr[0] = bcReg.dword;
//		dscPtr[1] = pci_addr + offset;				// Deiva: This line was modified
//		dscPtr[2] = dmaInfo->dramAddress + offset;
//		initDmaBuffers(dmaInfo, x + 1, dmaInfo->dramAddress + offset, pci_addr + offset);	// Deiva: This line was modified
//
//		/ create the descriptor pointer
//		tmpDsc = (UINT32)&dscPtr[0] + dmaInfo->descriptorSpan;
//		dscPtr[3] = tmpDsc;
//		// printf("descriptor %d (BC:%08x,PCI:%08x,DRAM:%08x,NEXT:%08x)\n",
//		//  x,dscPtr[0],dscPtr[1],dscPtr[2],dscPtr[3]);
//
//		// save the old pointer so we can set the eoc bit in the last desriptor
//		oldPtr = dscPtr;
//		dscPtr = (PU32)tmpDsc;
//
//		// update the offset for the next descriptor
//		offset += dmaInfo->bufferSpan;
//	}

	// set the end of chain bit here, if number descriptors equals 0
	// we need to set it in the register
	if (dmaInfo->descriptorNum == 0)
	{
		if (dmaInfo->channel&1)
		{
			PUT32(CHAN_1_BYTE_COUNT, GET32(CHAN_1_BYTE_COUNT) | 0x80000000);
		}
		
		if (dmaInfo->channel&2)
		{
			PUT32(CHAN_2_BYTE_COUNT, GET32(CHAN_2_BYTE_COUNT) | 0x80000000);
		}
	}
	else
	{
		// printf("writing 80000000 -> %08x\n",&oldPtr[0]);
		oldPtr[0] |= 0x80000000;
	}
}


/*=========================================================================*/
/* initialize the memory descriptors*/
/*=========================================================================*/
void initDmaBuffers(dmaInfoType *dmaInfo, UINT32 descNum, UINT32 dramAddress, UINT32 pciAddress)
{
	int i;
	UINT32 *srcBufPntr, *dstBufPntr;

	// put the PCI buffer into footbridge space
	pciAddress |= 0x80000000;

	// check which direction the transfer is going
	// printf("In initDmaBuffers, ");
	if (dmaInfo->control.direction == DRAMtoPCI)
	{
		srcBufPntr = (UINT32 *)dramAddress;
		dstBufPntr = (UINT32 *)pciAddress;
	}
	else
	{
		srcBufPntr = (UINT32 *)pciAddress;
		dstBufPntr = (UINT32 *)dramAddress;
	}

	 //printf("src: %08x dst: %08x\n",srcBufPntr,dstBufPntr);

	// initialize the buffer, descNum will make sure that each descriptor
	// data block will contain unique data
	for (i = 0; i < (dmaInfo->control.sizeBytes / 4); i++)
	{
		dstBufPntr[i] = 0; 
		srcBufPntr[i] = i ^ 0xA5A4A3A2 ^ descNum;
		//printf("DEBUG: srcBufPntr[i]=0x%X\r", srcBufPntr[i]);
	}
}


/*=========================================================================*/
/* start the DMA engine by writing the control register*/
/*=========================================================================*/
int startDMA(dmaInfoType *dmaInfo)
{
	dmaControlRegType controlReg;

	/**/
	/* setup the channel control bits. */
	/**/
	controlReg.dword = 0;
	controlReg.bits.enable = 1;
	controlReg.bits.firstInReg = dmaInfo->registerStart;
	// Deiva: Some code was removed at this point

	/**/
	/* start the DMA by writing control register of each channel*/
	/**/
	if (dmaInfo->channel & 1)
	{
		PUT32(CHAN_1_CONTROL, 0x8C);
		if (GET32(CHAN_1_CONTROL) != 0)
		{
			printf("Failure: unable to reset DMA 1 control register %08x\n",GET32(CHAN_1_CONTROL));
			return ERROR;
		}
		//printf("DEBUG:starting chan 1 DMA now... writing %08x -> control reg\n",controlReg.dword);
		PUT32(CHAN_1_CONTROL, controlReg.dword);
	}

	if (dmaInfo->channel&2)
	{
		PUT32(CHAN_2_CONTROL,0x8C);
		if (GET32(CHAN_2_CONTROL) != 0)
		{
			printf("Failure: unable to reset DMA 2 control register %08x\n",GET32(CHAN_2_CONTROL));
			return ERROR;
		}
		//printf("DEBUG:starting chan 2 DMA now... writing %08x -> control reg\n",controlReg.dword);
		PUT32(CHAN_2_CONTROL, controlReg.dword);
	}
	return OK;
}


/*=========================================================================*/
/* wait for some completion status from FootBridge*/
/*=========================================================================*/
UINT32 waitForCompletion(dmaInfoType *dmaInfo)
{
	UINT32 tmp,times = 0x100000,value;
	char done_1 = 0, done_2 = 0, done = 0, chk_1, chk_2;

	chk_1 = dmaInfo->channel&1;
	chk_2 = dmaInfo->channel&2;

	//printf("DEBUG: Waiting for completion...\n");

	while (--times);

	times = 0x100000;

	/**/
	/* this will wait for Channel Transfer Done or Channel Error*/
	/* or Channel Chain Done*/
	/* */
	while ((!done) && times--)
	{
		if (chk_1)
		{
			if ((tmp = GET32(CHAN_1_CONTROL)) & 0x88)
			{
				done_1 = 1;            
				/*printf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",
				  GET32(CHAN_1_BYTE_COUNT),GET32(CHAN_1_PCI_ADDR),
				GET32(CHAN_1_DRAM_ADDR),GET32(CHAN_1_CONTROL));*/
				if (tmp & 0x8)
				{
					printf("Failure: channel 1 error bit set\n");
					printf("BC:%08x, PCI:%08x DRAM:%08x CNTRL:%08x\n",
					GET32(CHAN_1_BYTE_COUNT),GET32(CHAN_1_PCI_ADDR),
					GET32(CHAN_1_DRAM_ADDR),GET32(CHAN_1_CONTROL));
				}
			}
		}
		else
		{
			done_1 = 1;
		}

		/**/
		/* write a unique pattern out*/
		/**/
		value = 0xa55a1234^times;
		*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = value;

		/**/
		/* verify that it's getting there*/
		/**/
		if ((*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0)) != value)
		{
			printf(" Failure: unable to correctly access mailbox reg \n");
			//printf("      addr: %08x actual: %08x exp: %08x xor: %08x   \n", mailbox0_addr, tmp, value, tmp^value);
			/**/
			/* check if we are to continue on error*/
			/**/
			if (checkErrorAndReturn())
			{
				return ERROR;
			}
		}

		if (chk_2)
		{
			if ((tmp=GET32(CHAN_2_CONTROL))&0x88)
			{
				done_2 = 1;            
				/*printf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",*/
				/*  GET32(CHAN_2_BYTE_COUNT),GET32(CHAN_2_PCI_ADDR),*/
				/*GET32(CHAN_2_DRAM_ADDR),GET32(CHAN_2_CONTROL));*/
				if (tmp&0x8)
				{
					printf("Failure: channel 2 error bit set\n");
					printf("BC:%08x, PCI:%08x, DRAM:%08x, CNTRL:%08x\n",
					GET32(CHAN_2_BYTE_COUNT),GET32(CHAN_2_PCI_ADDR),
					GET32(CHAN_2_DRAM_ADDR),GET32(CHAN_2_CONTROL));
					/**/
					/* check if we are to continue on error*/
					/**/
					if (checkErrorAndReturn())
					{
						return ERROR;
					}
				}
			}
		}
		else
		{
			done_2 = 1;
		}
	
		if (done_1 && done_2)
		{
			done = 1;
		}
	}
	
	if (times == 0)
	{
		printf("Error: Timed out waiting for DMA completion\n");
		return ERROR;
	}

	return OK;      
}


/*=========================================================================*/
/* verify that counters and all transferred data are correct.*/
/*=========================================================================*/

UINT32 verifyResults(dmaInfoType *dmaInfo)
{
	UINT32 tmp, value, x, pval, dval, d;
	UINT32 *pciAddress, *dramAddress;
	UINT32 *dscPtr = (UINT32 *)dmaInfo->descriptorStart;
	UINT32 pci_addr, xscale_pci_addr;

	// Read the configuration strapping to decide if PCI master
	//if(IS_EGRESS_NPU)
	//	{
	//		pci_addr = GET_OTHER_NPUs_PCI_DRAM_BASE(SLAVE_NPU);
	//	}
	//	else
	//	{
	//		pci_addr = GET_OTHER_NPUs_PCI_DRAM_BASE(MASTER_NPU);
	//	}

	pci_addr= *(volatile unsigned int *)(IXP2000_PCI_BASE + IXP2000_PCI_CFG_BASE_ADDRESS_2);

	//pci_addr = (dev_info.base_address[SDRAM_BAR] & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	pci_addr = (pci_addr & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	//xscale_pci_addr = dev_info.base_map[SDRAM_BAR] + PCI_DRAM_OFFSET;
	//	xscale_pci_addr = (pci_addr & 0xFFFFFFF0) + PCI_DRAM_OFFSET;
	xscale_pci_addr = 0x00000000 + PCI_DRAM_OFFSET;

	//printf("DEBUG: verify pci_addr = 0x%x\n", pci_addr);
	//printf("DEBUG: verify xscale_pci_addr = 0x%x\n", xscale_pci_addr);


	//printf("verifying DMA transfer buffers...numdesc: %d\n",dmaInfo->descriptorNum);

	// check if single transfer, if so verify counters et al
	if (dmaInfo->descriptorNum == 0)
	{
		// check channel 1
		if (dmaInfo->channel & 1)
		{
			// check that the counter went to zero
			if ((tmp=GET32(CHAN_1_BYTE_COUNT) & 0xFFFFFF) != 0)
			{
				printf("Failure: channel 1 byte count not 0\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_BYTE_COUNT, tmp, 0, tmp^0);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}							   
			}

			// check that the PCI address incremented correctly
			value = pci_addr + dmaInfo->control.sizeBytes;	// Deiva: This line was modified

			//printf("DEBUG: value=0x%X CHAN_1_PCI_ADDR=0x%X\n", value, GET32(CHAN_1_PCI_ADDR));

			if ((tmp=GET32(CHAN_1_PCI_ADDR)) != value)
			{
				printf("Failure: channel 1 PCI address incorrect\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_PCI_ADDR, tmp, value, tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the DRAM address incremented correctly*/
			value = dmaInfo->dramAddress + dmaInfo->control.sizeBytes;

			//printf("DEBUG: value=0x%X CHAN_1_DRAM_ADDR=0x%X\n", value, GET32(CHAN_1_DRAM_ADDR));

			if ((tmp=GET32(CHAN_1_DRAM_ADDR)) != value)
			{
				printf("Failure: channel 1 DRAM address incorrect\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_1_DRAM_ADDR, tmp, value, tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}  

		// check channel 2
		if (dmaInfo->channel == 2)
		{
			if ((tmp=GET32(CHAN_2_BYTE_COUNT)&0xFFFFFF) != 0)
			{
				printf("Failure: channel 2 byte count not 0\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n",
				CHAN_2_BYTE_COUNT,tmp,0,tmp^0);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the PCI address incremented correctly
			if( dmaInfo->channel == 3)
			{
				value = pci_addr + dmaInfo->control.sizeBytes + dmaInfo->bufferSpan;	// Deiva: This line was modified
			}
			else
			{
				value = pci_addr + dmaInfo->control.sizeBytes;	// Deiva: This line was modified
			}

			if ((tmp = GET32(CHAN_2_PCI_ADDR)) != value)
			{
				printf("Failure: channel 2 PCI address incorrect\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n", CHAN_2_PCI_ADDR,tmp,value,tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}

			// check that the DRAM address incremented correctly
			if( dmaInfo->channel == 3)
			{
				value = dmaInfo->dramAddress+dmaInfo->control.sizeBytes+dmaInfo->bufferSpan;
			}
			else
			{
				value = dmaInfo->dramAddress+dmaInfo->control.sizeBytes;
			}

			if ((tmp=GET32(CHAN_2_DRAM_ADDR)) != value)
			{
				printf("Failure: channel 2 DRAM address incorrect\n");
				printf("addr:%08x, act:%08x, exp:%08x, xor:%08x\n",
				CHAN_2_DRAM_ADDR,tmp,value,tmp^value);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}


		// verify that the source and destination buffers are the same
		pciAddress = (UINT32 *)(xscale_pci_addr);    // Deiva: This line was modified
		dramAddress = (UINT32 *)dmaInfo->dramAddress;    

		for (x = 0; x < (dmaInfo->control.sizeBytes / 4); x++)
		{
			if ((pval = pciAddress[x]) != (dval = dramAddress[x]))
			{
				printf("Failure: DMA buffer compare error\n");
				if (dmaInfo->control.direction == DRAMtoPCI)
				{
					//printf("DEBUG:DRAMtoPCI = 0x%x\n",	dmaInfo->control.direction);
					printf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
											&dramAddress[x], &pciAddress[x], dval, pval);
				}
				else
				{
					//printf("DEBUG:DRAMtoPCI = 0x%x\n",	dmaInfo->control.direction);
					printf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
											&pciAddress[x], &dramAddress[x], pval, dval);
				}
				
				printf(", xor:%08x\n", pval^dval);

				// check if we are to continue on error
				if (checkErrorAndReturn())
				{
					return ERROR;
				}
			}
		}
	}
	else
	{  
		// now go verify all the data transferred correctly, we need to traverse
		// the descriptors to find out what was supposed to happen.
		for (d=0;d<dmaInfo->descriptorNum;d++)
		{
			//printf("checking descriptor %d (BC:%08x,PCI:%08x,DRAM:%08x,NEXT:%08x)\n",
			//d,dscPtr[0],dscPtr[1],dscPtr[2],dscPtr[3]);
			pciAddress = (UINT32 *)(dscPtr[1]|0x80000000);
			dramAddress = (UINT32 *)dscPtr[2];
			for (x=0;x<(dscPtr[0]&0xFFFFFF);x++)
			{
				if ((pval=pciAddress[x]) != (dval=dramAddress[x]))
				{
					printf("Failure: buffer compare error, descriptor %d\n",x);
					if (dmaInfo->control.direction == DRAMtoPCI)
					{
						printf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
						&pciAddress[x],&dramAddress[x],pval,dval);
					}
					else
					{
						printf("srcAddr:%08x, destAddr:%08x, act: %08x, exp: %08x",
						&dramAddress[x],&pciAddress[x],dval,pval);
					}
				
					printf(", xor:%08x\n",pval^dval);

					// check if we are to continue on error
					if (checkErrorAndReturn())
					{
						return ERROR;
					}
				}
			}

			// follow the list for more buffers
			dscPtr = (UINT32 *)dscPtr[3];
		}
	}
	return OK;
}


int checkErrorAndReturn()
{
   return 1;
}
