/* pci_dma.h
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: dalsraja, Feb 2002
 * 
 * 
 * --------------------------------------------------------------------------
 */

#ifndef PCI_DMA_H

#define PCI_DMA_H

#define XFERSIZE			0x1000
#define DRAM_ADDR			0x180000		// DRAM_ADDR = 16M

#define PCI_DRAM_OFFSET		DRAM_ADDR

#define DESC_START			0x80000000		// 
#define DRAMtoPCI			1
#define PCItoDRAM			0
#define PCI_READ            0
#define PCI_READ_LINE       1
#define PCI_READ_MULTIPLE   2

#define D_OFF0				0
#define D_OFF1				0x1000
#define D_OFF2				0x2000
#define D_OFF3				0x3000

#define CHAN_1_BYTE_COUNT       (unsigned int)(IXP2000_PCI_CSR_BASE + 0x080)
#define CHAN_1_PCI_ADDR         (unsigned int)(IXP2000_PCI_CSR_BASE + 0x084)
#define CHAN_1_DRAM_ADDR        (unsigned int)(IXP2000_PCI_CSR_BASE + 0x088)
#define CHAN_1_DESC_PTR         (unsigned int)(IXP2000_PCI_CSR_BASE + 0x08C)
#define CHAN_1_CONTROL          (unsigned int)(IXP2000_PCI_CSR_BASE + 0x090)
#define CHAN_2_BYTE_COUNT       (unsigned int)(IXP2000_PCI_CSR_BASE + 0x0A0)
#define CHAN_2_PCI_ADDR         (unsigned int)(IXP2000_PCI_CSR_BASE + 0x0A4)
#define CHAN_2_DRAM_ADDR        (unsigned int)(IXP2000_PCI_CSR_BASE + 0x0A8)
#define CHAN_2_DESC_PTR         (unsigned int)(IXP2000_PCI_CSR_BASE + 0x0AC)
#define CHAN_2_CONTROL          (unsigned int)(IXP2000_PCI_CSR_BASE + 0x0B0)

#ifndef UCA_BUILD

typedef struct
{
	UINT32 sizeBytes;
	UINT8  prescale;
	UINT8  iBDelay;
	UINT8  direction;
	UINT8  readType;
	UINT8  pciReadLength;
	UINT8  dramReadLength;
	UINT16 reserved;
} dmaControlType;


typedef struct
{
	UINT32 channel;
	UINT32 registerStart;
	UINT32 descriptorStart;
	UINT32 descriptorNum;
	UINT32 descriptorSpan;
	UINT32 bufferSpan;
	UINT32 dramAddress;
	dmaControlType control;
} dmaInfoType;


typedef union 
{
	UINT32 dword;
#ifdef __ARMEB__
	struct
	{
		UINT32 eoChain:1;
		UINT32 direction:1;
		UINT32 reserved:6;
		UINT32 byteCount:24;
	} bits;
#else
	struct
	{
		UINT32 byteCount:24;
		UINT32 reserved:6;
		UINT32 direction:1;
		UINT32 eoChain:1;
	} bits;
#endif
} byteCountRegType;


typedef union
{
	UINT32 dword;
#ifdef __ARMEB__
	struct
	{
		UINT32 cxdc:16;
		UINT32 udr:1;
		UINT32 paused:1;
		UINT32 da:1;
		UINT32 reserved2:5;
		UINT32 chainDone:1;
		UINT32 reserved1:2;
		UINT32 firstInReg:1;
		UINT32 error:1;
		UINT32 xferDone:1;
		UINT32 reserved:1;	//UINT32 ttype:1;
		UINT32 enable:1;		
	} bits;

#else
	struct
	{
		UINT32 enable:1;
		UINT32 reserved:1;	//UINT32 ttype:1;
		UINT32 xferDone:1;
		UINT32 error:1;
		UINT32 firstInReg:1;
		UINT32 reserved1:2;
		UINT32 chainDone:1;
		UINT32 reserved2:5;
		UINT32 da:1;
		UINT32 paused:1;
		UINT32 udr:1;
		UINT32 cxdc:16;
	} bits;
#endif
} dmaControlRegType;


int dmaTest(void);
void initDescriptors(dmaInfoType *);
void initDmaBuffers(dmaInfoType *,UINT32,UINT32, UINT32);
int startDMA(dmaInfoType *);
UINT32 waitForCompletion(dmaInfoType *);
UINT32 verifyResults(dmaInfoType *);
int checkErrorAndReturn(void);


#endif   // __IXP2800MEDIA_H



#endif
