/* pciworkaround.uc	  
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800							 
 *  subsystem: PCI
 *  author: lfc,jjd, May 27 03
 * 
 * --------------------------------------------------------------------------
 */


#define SCRATCH_BASE_E         0x240F0500
#define SCRATCH_BASE_I			0x340F0500
#define EGRESS_VAL				0xA5A5A5A0
#define INGRESS_VAL				0xA5A5A5A1
#define START_VAL				0xA5A5A5A5
#define DATA_PATTERN1          0x0
#define DATA_PATTERN2          0xAAAAAAAA
#define LEN_S                                       0x8

// immed32
//		Description:load out_result with 32 bit constant
//					if constant fits in 16 bits, expand to a single immed 
//					if constant doesn't, expand into a immed_w0 and immed_w1
//					if it is a name (not a number), assume that it is being IMPORTED
//		Outputs:
//			out_result	GPR. destination register. 
//		Inputs:
//			VAL		constant
//		Size: 1-2 instructions (3 if output is xfer register)
//		Example Usage: immed32(output, 0x12345678);
//
#macro immed32(out_result, VAL)
#if (!isnum(VAL))
	#if (strstr(out_result, $))	; check for output xfer reg
		.begin
		.reg immed32_temp
		immed[immed32_temp, VAL]
		immed_w1[immed32_temp, VAL>>16]
		alu[out_result, --, b, immed32_temp]
	#else 	; !strstr(out_result, $)
		immed[out_result, VAL]
		immed_w1[out_result, VAL>>16]
	#endif	; strstr(out_result, $)
#elif (((VAL & 0xFFFF0000) == 0) || ((VAL & 0xFFFF0000) == 0xFFFF0000))
    immed[out_result,VAL]
#elif (((VAL & 0xFF0000FF) == 0) || ((VAL & 0xFF0000FF) == 0xFF0000FF))
    immed[out_result, (VAL >> 8), <<8]
#elif ((VAL & 0x0000FFFF) == 0)
    /* make sure that number is not seen as negative */
    immed[out_result, (0x0000FFFF & (VAL >> 16)), <<16]
#elif ((VAL & 0x0000FFFF) == 0x0000FFFF)
    /* make sure that number is seen as negative */
    immed[out_result, (0xFFFF0000 | (VAL >> 16)), <<16]
#else	; isnum(VAL)
	#if (strstr(out_result, $))	; check for output xfer reg
		.begin
		.reg immed32_temp
		immed[immed32_temp, (VAL & 0xFFFF)]
		immed_w1[immed32_temp, ((VAL >> 16) & 0xFFFF)]
		alu[out_result, --, b, immed32_temp]
		.end
	#else	; !(strstr(out_result, $))
		immed[out_result, (VAL & 0xFFFF)]
		immed_w1[out_result, ((VAL >> 16) & 0xFFFF)]
	#endif	; (strstr(out_result, $))
#endif	; (!isnum(VAL))
#endm

#macro DELAY_ME (Dtime)
.begin
.reg temp
delay_c#:
	nop
	nop
	nop
	alu [Dtime, Dtime, -, 1]
	bne [delay_c#]
.end      
#endm

#define ME_PCI_MEM          0x20000000
#define ME_PCI_CSR          0x07000000
#define ME_PCI_CONF         0x06000000
#define ME_PCI_SPEC         0x04000000
#define ME_PCI_TYPE0_SLOT1  0x02200000
#define ME_PCI_TYPE0_SLOT2  0x02800000
#define ME_PCI_TYPE1        0x03000000
#define ME_PCI_IO           0x00000000

/*************************************************************
 * Macro Name  : PCI_WRITE
 * Purpose                     : This will be used to write to PCI IO,CSR,SPEC,ACK,TYPE0
 * Design                       :  
 *                                              
 *
 * Arguments     : 
 *                      a)TYPE                         - Type of transaction such as IO/CSR/SPEC/ACK/TYPE0/TYPE1 
 *                      b)_pattern          - Data needs to be written in to specified location 
 *                      c)csr_offset- PCI CSR register offset
 *
 **************************************************************/
#macro PCI_WRITE(TYPE,_pattern,offset)
    .begin
    .reg $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
            .reg data0 data1 data2 data3 data4 data5 data6 data7       
            .xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
            .reg _mem_base 
            .sig wr_sig rd_sig
 
                        #if streq(TYPE, CSR)
                                    immed32[_mem_base, ME_PCI_CSR]     
                        #elif streq(TYPE, CONF)
                                    immed32[_mem_base, ME_PCI_CONF]
                        #elif streq(TYPE, IO)
                                    immed32[_mem_base, ME_PCI_IO]
                        #elif streq(TYPE, SPEC)
                                    immed32[_mem_base, ME_PCI_SPEC]
                        #elif streq(TYPE, TYPE0_SLOT1)
                                    immed32[_mem_base, ME_PCI_TYPE0_SLOT1]
                        #elif streq(TYPE, TYPE0_SLOT2)
                                    immed32[_mem_base, ME_PCI_TYPE0_SLOT2]
                        #elif streq(TYPE, TYPE1)
                                    immed32[_mem_base, ME_PCI_TYPE1]
                        #else
                                    #error 4 "Wrong value for PCI access Type"
                        #endif
 
                        ;set-up SRAM xfer registers with counting pattern
                        ;repeat loop to write all data by using count 1
                        alu[$xfer0,--,B,_pattern]
                        pci[write,$xfer0,_mem_base,offset,1], sig_done[wr_sig]
                        ctx_arb[wr_sig]
            .end
#endm
/*************************************************************
 * Macro Name  : PCI_READ
 * Purpose                     : This will be used to READ IO,CSR,..
 * Design                       :  
 *                                              
 *
 *
 * Arguments     : 
 *                      a)TYPE                         - Type of transaction such as IO/CSR/SPEC/ACK/TYPE0/TYPE1 
 *                      b)_ret_value- Data read from  
 *                      c)csr_offset- PCI offset
 *
 **************************************************************/
#macro PCI_READ(TYPE,result,offset)
            .begin
    .reg $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
            .reg data0 data1 data2 data3 data4 data5 data6 data7       
            .xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
            .reg _mem_base _offset
            .sig wr_sig rd_sig
                        #if streq(TYPE, CSR)
                                    immed32[_mem_base, ME_PCI_CSR]
                        #elif streq(TYPE, CONF)
                                    immed32[_mem_base, ME_PCI_CONF]
                        #elif streq(TYPE, IO)
                                    immed32[_mem_base, ME_PCI_IO]
                        #elif streq(TYPE, SPEC)
                                    immed32[_mem_base, ME_PCI_SPEC]
                        #elif streq(TYPE, TYPE0_SLOT1)
                                    immed32[_mem_base, ME_PCI_TYPE0_SLOT1]
                        #elif streq(TYPE, TYPE0_SLOT2)
                                    immed32[_mem_base, ME_PCI_TYPE0_SLOT2]
                        #elif streq(TYPE, TYPE1)
                                    immed32[_mem_base, ME_PCI_TYPE1]
                        #else
                                    #error 4 "Wrong value for PCI access Type"
                        #endif
                        
                        pci[read,$xfer0,_mem_base,offset,1], sig_done[rd_sig]
                        ctx_arb[rd_sig]

                        alu[result,--,B,$xfer0]
            .end
#endm

.begin
.reg $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 
.reg pattern _offset _pattern __offset ixp st delay_reset result mem_base length
.sig init 

/*
 * PCI workaround microcode.  This code does a small sanity check on the PCI bus
 * by reading/writing to it's own scratch memory via the PCI bus.  This code
 * is hardcoded to work with the Deer Island dual NPU type setup.
 */
	br=ctx[0, go#]
	ctx_arb[kill]
go#:
	immed32[_offset,0]
	immed32[pattern, 0x39]
	immed32[mem_base,0x0700010c]
	alu[$xfer0,--,B,pattern]
	cap[write,$xfer0, _offset, mem_base, 1], sig_done[init]
	ctx_arb[init]

	cap[read, $xfer0, STRAP_OPTIONS], sig_done[init]
	ctx_arb[init]

	immed32[mem_base,SCRATCH_BASE_E]
	br_bset[$xfer0,2,continue#]
	immed32[mem_base,SCRATCH_BASE_I ] 

continue#:
	immed32[_offset, 0x50]
	immed32[_pattern,START_VAL]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0]

	immed32[pattern, DATA_PATTERN1]
	alu[$xfer0,--,B,pattern]
	alu[$xfer1,--,B,pattern]
	alu[$xfer2,--,B,pattern]
	alu[$xfer3,--,B,pattern]
	alu[$xfer4,--,B,pattern]
	alu[$xfer5,--,B,pattern]
	alu[$xfer6,--,B,pattern]
	alu[$xfer7,--,B,pattern]
	pci[write, $xfer0,mem_base,_offset,8],sig_done[init]
	ctx_arb[init]

	immed32[delay_reset,0x10]
	DELAY_ME(delay_reset)

	pci[read, $xfer0,mem_base,_offset,8],sig_done[init]
	ctx_arb[init]

	immed32[pattern, DATA_PATTERN1]
	alu[--,$xfer0,-,pattern]
	bne[zeros_fail_0#]
	alu[--,$xfer1,-,pattern]
	bne[zeros_fail_1#]
	alu[--,$xfer2,-,pattern]
	bne[zeros_fail_2#]
	alu[--,$xfer3,-,pattern]
	bne[zeros_fail_3#]
	alu[--,$xfer4,-,pattern]
	bne[zeros_fail_4#]
	alu[--,$xfer5,-,pattern]
	bne[zeros_fail_5#]
	alu[--,$xfer6,-,pattern]
	bne[zeros_fail_6#]
	alu[--,$xfer7,-,pattern]
	bne[zeros_fail_7#]

//          Perform 8 DWORD transfer to scratch with Castine, all 0xAAAAAAAA, read verify
//          The above 2 data will avoid the case where stale data from a previous iteration was left in the scratch
	immed32[pattern, DATA_PATTERN2]
	alu[$xfer0,--,B,pattern]
	alu[$xfer1,--,B,pattern]
	alu[$xfer2,--,B,pattern]
	alu[$xfer3,--,B,pattern]
	alu[$xfer4,--,B,pattern]
	alu[$xfer5,--,B,pattern]
	alu[$xfer6,--,B,pattern]
	alu[$xfer7,--,B,pattern]

	pci[write, $xfer0,mem_base,_offset,8],sig_done[init]
	ctx_arb[init]

	immed32[delay_reset,0x10]
	DELAY_ME(delay_reset)

	pci[read, $xfer0,mem_base,_offset,8],sig_done[init]
	ctx_arb[init]

	immed32[delay_reset,0x10]
	DELAY_ME(delay_reset)

	immed32[pattern, DATA_PATTERN2]
	alu[--,$xfer0,-,pattern]
	bne[as_fail_0#]
	alu[--,$xfer1,-,pattern]
	bne[as_fail_1#]
	alu[--,$xfer2,-,pattern]
	bne[as_fail_2#]
	alu[--,$xfer3,-,pattern]
	bne[as_fail_3#]
	alu[--,$xfer4,-,pattern]
	bne[as_fail_4#]
	alu[--,$xfer5,-,pattern]
	bne[as_fail_5#]
	alu[--,$xfer6,-,pattern]
	bne[as_fail_6#]
	alu[--,$xfer7,-,pattern]
	bne[as_fail_7#]

//          Set Mailbox 0 to the address where ME will put the success flag
//          Set Mailbox 1 to the data where ME will put if success

	immed32(_pattern,0xDEADBEEF)
	immed32(_offset,0x54)
	PCI_WRITE(CSR,_pattern,_offset)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	ctx_arb[kill]


zeros_fail_0#:
	alu[pattern, --, B, $xfer0]
	alu[$xfer0, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer0,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0000]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_1#:
	alu[pattern, --, B, $xfer1]
	alu[$xfer1, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer1,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0001]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_2#:
	alu[pattern, --, B, $xfer2]
	alu[$xfer2, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer2,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0002]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_3#:
	alu[pattern, --, B, $xfer3]
	alu[$xfer3, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer3,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0003]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_4#:
	alu[pattern, --, B, $xfer4]
	alu[$xfer4, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer4,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0004]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_5#:
	alu[pattern, --, B, $xfer5]
	alu[$xfer5, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer5,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0005]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_6#:
	alu[pattern, --, B, $xfer6]
	alu[$xfer6, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer6,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0006]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
zeros_fail_7#:
	alu[pattern, --, B, $xfer7]
	alu[$xfer7, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer7,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0007]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_0#:
	alu[pattern, --, B, $xfer0]
	alu[$xfer0, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer0,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA0]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_1#:
	alu[pattern, --, B, $xfer1]
	alu[$xfer1, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer1,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA1]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_2#:
	alu[pattern, --, B, $xfer2]
	alu[$xfer2, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer2,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA2]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_3#:
	alu[pattern, --, B, $xfer3]
	alu[$xfer3, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer3,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA3]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_4#:
	alu[pattern, --, B, $xfer4]
	alu[$xfer4, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer4,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA4]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_5#:
	alu[pattern, --, B, $xfer5]
	alu[$xfer5, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer5,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA5]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_6#:
	alu[pattern, --, B, $xfer6]
	alu[$xfer6, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer6,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA6]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
as_fail_7#:
	alu[pattern, --, B, $xfer7]
	alu[$xfer7, --, B, pattern]
	immed32[mem_base,0]
	immed32[_offset,0]
	scratch[write,$xfer7,mem_base,_offset,1],sig_done[init]
	ctx_arb[init]
	immed32[_offset, 0x58]
	immed32[_pattern,0xFEED0AA7]
	PCI_WRITE[CSR,_pattern,_offset]
	immed32[_offset, 0x54]
	immed32[_pattern,0xFEEDFACE]
	PCI_WRITE[CSR,_pattern,_offset]
//	ctx_arb[kill]
//	alu[pattern,--,B,$xfer0]
//	alu[$xfer0,--,B,pattern]
//	alu[pattern,--,B,$xfer1]
//	alu[$xfer1,--,B,pattern]
//	alu[pattern,--,B,$xfer2]
//	alu[$xfer2,--,B,pattern]
//	alu[pattern,--,B,$xfer3]
//	alu[$xfer3,--,B,pattern]
//	alu[pattern,--,B,$xfer4]
//	alu[$xfer4,--,B,pattern]
//	alu[pattern,--,B,$xfer5]
//	alu[$xfer5,--,B,pattern]
//	alu[pattern,--,B,$xfer6]
//	alu[$xfer6,--,B,pattern]
//	alu[pattern,--,B,$xfer7]
//	alu[$xfer7,--,B,pattern]
//	immed32[mem_base,0]
//	immed32[_offset,0]
//	scratch[write,$xfer0,mem_base,_offset,8],sig_done[init]
//	ctx_arb[init]
//	immed32[_offset, 0x54]
//	immed32[_pattern,0xFEEDFACE]
//	PCI_WRITE[CSR,_pattern,_offset]
	ctx_arb[kill]
.end