/******************************************************************************
*
* Module:	Serial16550.h
*
* Description:
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

#ifndef __ASSEMBLER__ /* XXX why doesn't this work right now? */
#ifdef __cplusplus
extern "C" {
#endif
#endif /* __ASSEMBLER__ */

#ifndef _SERIAL16550_H
#define _SERIAL16550_H

//  Local Defines
// VWARE_PARAMS_BEGIN
#define	BAUDRATE		9600		// RS232 Baud Rate	UINT32	9600
// VWARE_PARAMS_END

#ifndef __ASSEMBLER__
#if __LITTLE_ENDIAN__
#error Should be big endian
typedef volatile struct {
    union{
	unsigned char	rxdata;
	unsigned char	txdata;
	unsigned char	dllo;
	unsigned char	pad[SERIALSPACING];
    }reg0;
    union{
	unsigned char	ier;
	unsigned char	dlhi;
	unsigned char	pad[SERIALSPACING];
    }reg1;
    union{
	unsigned char	iir;
	unsigned char	fcr;
	unsigned char	pad[SERIALSPACING];
    }reg2;
    union{
	unsigned char	lcr;
	unsigned char	pad[SERIALSPACING];
    }reg3;
    union{
	unsigned char	mcr;
	unsigned char	pad[SERIALSPACING];
    }reg4;
    union{
	unsigned char	lstat;
	unsigned char	pad[SERIALSPACING];
    }reg5;
    union{
	unsigned char	modstat;
	unsigned char	pad[SERIALSPACING];
    }reg6;
}U550;
#else
typedef volatile struct {
    struct{
        unsigned char	pad[SERIALSPACING];
	union {
	    unsigned char	rxdata;
	    unsigned char	txdata;
       	    unsigned char	dllo;
        }reg0union;
    }reg0;
    struct{
        unsigned char	pad[SERIALSPACING];
        union {
	    unsigned char	ier;
	    unsigned char	dlhi;
        }reg1union;
    }reg1;
    struct{
        unsigned char	pad[SERIALSPACING];
        union {
	    unsigned char	iir;
	    unsigned char	fcr;
        }reg2union;
    }reg2;
    struct{
        unsigned char	pad[SERIALSPACING];
	unsigned char	lcr;
    }reg3;
    struct{
        unsigned char	pad[SERIALSPACING];
	unsigned char	mcr;
    }reg4;
    struct{
        unsigned char	pad[SERIALSPACING];
	unsigned char	lstat;
    }reg5;
    struct{
        unsigned char	pad[SERIALSPACING];
	unsigned char	modstat;
    }reg6;
}U550;
#endif /* __LITTLE_ENDIAN__ */

#define TXDATA	reg0.reg0union.txdata
#define RXDATA	reg0.reg0union.rxdata
#define DLLO	reg0.reg0union.dllo
#define DLHI	reg1.reg1union.dlhi
#define IER	reg1.reg1union.ier
#define IIR	reg2.reg2union.iir
#define FCR	reg2.reg2union.fcr
#define LCR	reg3.lcr
#define MCR	reg4.mcr
#define LSTAT	reg5.lstat
#define MODSTAT	reg6.modstat

#else

#if #endian(little)
#define reg16550(x) ((x)*4+0)
#else
#define reg16550(x) ((x)*4+3)
#endif

#define TXDATA  reg16550(0)
#define RXDATA  TXDATA
#define DLLO    TXDATA
#define DLHI    reg16550(1)
#define IER     DLHI
#define IIR     reg16550(2)
#define FCR     IIR
#define LCR     reg16550(3)
#define MCR     reg16550(4)
#define LSTAT   reg16550(5)
#define MODSTAT reg16550(6)

#endif /* __ASSEMBLER__ */
/*---------------------------------------------------------------------*/
/* Define bits in the Interrupt Enable register                        */
/*---------------------------------------------------------------------*/
#define    RXINT    0x01      /* Enable receive interrupt              */
#define    TXINT    0x02      /* Enable transmit interrupt             */
#define    LSINT    0x04      /* Enable receiver line status interrupt */
#define    RTOINT   0x10      /* Enable receiver data timeout interrupt*/
#define    NRZEN    0x20      /* Enable NRZ encoding                   */
#define    UUEN     0x40      /* Enable UART unit                      */
#define    MSINT    0x08      /* Enable modem status interrupt         */

/*---------------------------------------------------------------------*/
/* Define bits in the Modem Control register (MODCTL)                  */
/*---------------------------------------------------------------------*/
#define    DTR      0x01      /* Data Terminal Ready                   */
#define    RTS      0x02      /* Request To Send                       */
#define    OUT1     0x04      /* Auxiliary out                         */
#define    OUT2     0x08      /* Enables interrupts from async card    */

/*---------------------------------------------------------------------*/
/* Define bits in the Modem Status Register (MODSTAT)                  */
/*---------------------------------------------------------------------*/
#define MOD_DCTS    0x01      /* delta CTS indicator                   */
#define MOD_DDSR    0x02      /* delta DSR indicator                   */
#define MOD_TERI    0x04      /* Trailing Edge of RI detector          */
#define MOD_DDCD    0x08      /* delta DCD indicator                   */
#define MOD_CTS     0x10      /* CTS signal                            */
#define MOD_DSR     0x20      /* DSR  signal                           */
#define MOD_RI      0x40      /* RI   signal                           */
#define MOD_DCD     0x80      /* DCD  signal                           */


/*---------------------------------------------------------------------*/
/* Define the bits in Line Status Register (LSR)                       */
/*---------------------------------------------------------------------*/
#define    LSR_DR     0x01    /* Data ready                            */
#define    LSR_OE     0x02    /* Overrun error                         */
#define    LSR_PE     0x04    /* Parity error                          */
#define    LSR_FE     0x08    /* Framing error                         */
#define    LSR_BI     0x10    /* Break interrupt                       */
#define    LSR_THRE   0x20    /* Transmitter holding register empty    */
#define    LSR_TEMT   0x40    /* Transmitter empty                     */
#define    LSR_FFE    0x80    /* Receiver FIFO error                   */


/*---------------------------------------------------------------------*/
/* Define the bits in Line Control Register (LCR)                      */
/*---------------------------------------------------------------------*/
#define  CHAR_SIZE_MASK   0x03  /* size from 5 to 8                    */
#define  CHAR_SIZE_5      0x00
#define  CHAR_SIZE_6      0x01
#define  CHAR_SIZE_7      0x02
#define  CHAR_SIZE_8      0x03

#define  STOP_BITS_MASK   0x04
#define  PARITY_BIT_MASK  0x38
#define  STOP_BITS_2      0x04  /* two stop bits if char size is 6,7,8 */
#define  STOP_BITS_1_5    0x04  /* 1.5 stop bits if char size is 5 bit */
#define  STOP_BITS_1      0x00  /* no bits needs to be set             */
#define  PARITY_ODD       0x08  /* enable odd parity bit               */
#define  PARITY_EVEN      0x18  /* enable even parity bit              */
#define  PARITY_STICK_0   0x38  /* parity bit is always 0              */
#define  PARITY_STICK_1   0x28  /* parity bit is always 1              */
#define  TR_BREAK_SIGNAL  0x40  /* send the break condtion             */
#define  DLA_BIT          0x80  /* divisor latch access                */


/*---------------------------------------------------------------------*/
/* Define the bits in FIFO Control Register (FCR) (Write-Only)         */
/*---------------------------------------------------------------------*/
#define  ENABLE_FIFO      0x01  /* 1: enable, 0 disable                */
#define  CLEAR_RCV_FIFO   0x02  /* Clear RCV FIFO (self-clearing bit)  */
#define  CLEAR_XMIT_FIFO  0x04  /* Clear XMIT FIFO (self-clearing bit) */
#define  SET_MODE_1       0x08  /* 1: mode 1; 0 mode 0                 */
#define  FIFO_THRES_MASK  0xC0
#define  FIFO_THRES_1     0x00  /* Trigger Level 1                     */
#define  FIFO_THRES_4     0x40  /* Trigger Level 4                     */
#define  FIFO_THRES_8     0x80  /* Trigger Level 8                     */
#define  FIFO_THRES_14    0xC0  /* Trigger Level 16                    */


/*---------------------------------------------------------------------*/
/* Define the bits in Interrupt Identification Register (read-only)    */
/*---------------------------------------------------------------------*/
#define  FIFO_EXIST       0xC0  /* set when in FIFO mode               */
#define  INTR_ID_MASK     0x07
#define  INTR_NOTHING     0x01  /* no interrupt                        */
#define  INTR_RCV_ERR     0x06  /* receive error interrupt             */
#define  INTR_RCV_AVAIL   0x04  /* receive data available              */
#define  INTR_XMT_EMPT    0x02  /* Xmit Holding Register empty         */
#define  INTR_MOD_STAT    0x00  /* Modem status changed                */
#define  INTR_EVENT_MASK  0x06
#define  INTR_TOD         0x08  /* FIFO timeout                        */


#ifndef __ASSEMBLER__
struct stats_struct 
	{
	UINT32 rxError;
	UINT32 txEvent;
	UINT32 rxEvent;
	UINT32 lineEvent;
	UINT32 otherError;
	UINT32 overrunError;
	UINT32 framingError;
	UINT32 fifoError;
	UINT32 fifoTod;
	};
#endif /* __ASSEMBLER__ */

#endif /* Inc16550_H */

#ifndef __ASSEMBLER__
#ifdef __cplusplus
}
#endif
#endif /* XXX why doesn't this work now? */
