/* sf_loopback.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * Revisions:
 * 01d, 09/24/03 ajp Added flow control training sequence
 * 01c, 09/18/03 ajp Clock divides for MSF tx and rx call msf_clk_speed
 * 01b, 09/18/03 ajp Modified the mailbox message timeout so that extremely long pkt cnts can be sent
 * 01a, 09/18/03 ajp Changed from 128 byte to 64 byte CSIX packets
 *
 *---------------------------------------------------------------------------
 */


#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2000Pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"  
#include "ixp2x00.h"
#include "msf_common.h"
#include "diag_utils.h"
#include "pci_diag.h"
#include "led.h"
#include "sf_loopback_tx.uh"
#include "sf_loopback_rx.uh"

void Sf_Loopback_Test_Slave(void);
void ingress_media_init_1(void);
void ingress_media_init_2(void);
int ingress_media_init_3(void);
void egress_media_init_1(void);
int egress_media_init_2(void);
void egress_media_init_3(void);

#define SF_LOOPBACK_RX_ME		0
#define SF_LOOPBACK_TX_ME		0


const UINT32 sf_lb_data[] =	{ 0x00010203, 0x04050607, 0x08090A0B, 0x0C0D0E0F,
							  0x10111213, 0x14151617, 0x18191A1B, 0x1C1D1E1F,
							  0x20212223, 0x24252627, 0x28292A2B, 0x2C2D2E2F,
							  0x30313233, 0x34353637, 0x38393A3B, 0x3C3D3E3F };


//****************************************************************************
// Function: void Sf_Loopback_Test(void)
//
// This function is called by the Test Manager to run the switch fabric
// loopback test
//****************************************************************************
int Sf_Loopback_Test(void *pIn, PassFail_testOut *pOut)
{
	UINT32 slave_mailbox0_addr, slave_mailbox1_addr, slave_mailbox2_addr, doorbell_intr_addr;
	UINT32 mailbox_value, prev_mailbox_val, flag;
	UINT32 ctr, loop_count, status, forever = 0;
	UINT32 dcache_status;
	UINT32 OTHER_NPUs_PCI_CSR_BASE;
	UINT32 Egress_msf_Scratch_addr;
	UINT32 Ingress_msf_Scratch_addr;
	UINT32 Tx_Pkts, Rx_Pkts, Rx_Errs;									 
	int i;
	UINT32 rx_data, test_status;

	pOut->ReturnCode = PASS;         
	pOut->PassFail = FAIL;

	if(!IS_EGRESS_NPU)
		{
		printf("fabric test should be run from Egress only\n");
		return(pOut->PassFail);
		}

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
		{
			dcache_off();
		}


	Scroll_Off();

	// Test start message to Ingress user terminal
	printf("Ingress starting Fabric Loopback test\n");

	OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);

	//Zero out the Ingress mailbox register data
	ZeroEgressMailboxes();
	ZeroIngressMailboxes();

	Egress_msf_Scratch_addr = UENG_MEDIA_STORAGE_START;

	//Get the Ingress' Scratch start addr 
    Ingress_msf_Scratch_addr = GET_OTHER_NPUs_PCI_CSR_BASE(SLAVE_NPU);
	Ingress_msf_Scratch_addr += SCRATCH_CSR_BASE_FRM_PCI;

	// Clear Egress and Ingress scratch memory areas for media test storage
	for(i=0; i < XS_MEDIA_STORAGE_SIZE; i+=4)		 
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + i) = 0;
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + i) = 0;
	}
					   
	// Requested MAC port # to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_MACNUMBER) = 0;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_MACNUMBER) = SWAP32(0);

	// User entered packet count
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = 0x40;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TOTALPACKETCOUNT) = SWAP32(0x40);
	
	// Port count goes to media test storage area
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = 1;
	*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_NUMBEROFMACPORTS) = SWAP32(1);
	
	// write 64 byte transmit packet data to Egress memory for ucode to use
	for (i=0; i < UENG_CNT; i++)
	{
		*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = sf_lb_data[i];
		*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXRXPACKETDATA + (i*4)) = SWAP32(sf_lb_data[i]);
	}

	// Initiate a doorbell interrupt on the slave it perform the loading
	// of the microcode on the slave's NPU.
	Set_Pci_Doorbell(SF_LB_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

	// Wait for Ingress to send (DM_CSIX_IMI1) message
	if((egress_wait_mailbox_msg(0x4000, SWAP32(DM_CSIX_IMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI1\n");
		return(pOut->PassFail);
	}

    egress_media_init_1();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI1) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI1;


	// Wait for Ingress to send (DM_CSIX_IMI2) message
	if((egress_wait_mailbox_msg(0x4000, SWAP32(DM_CSIX_IMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI2\n");
		return(pOut->PassFail);
	}

    test_status = egress_media_init_2();
	if(test_status == FAIL)
		{
		printf("Egress failed to rx data training\n");
		Cleanup_before_return();
		return(pOut->PassFail);
    	}
		
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI2) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI2;


	// Wait for Ingress to send (DM_CSIX_IMI3) message
	if((egress_wait_mailbox_msg(0x4000, SWAP32(DM_CSIX_IMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI3\n");
		return(pOut->PassFail);
	}

	test_status = SWAP32(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2));

	if(test_status == FAIL)
		{
		printf("Ingress failed to rx flow control\n");
		Cleanup_before_return();
		return(pOut->PassFail);
		}

    egress_media_init_3();
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI3) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI3;


	// Wait for Ingress to send (DM_CSIX_IMI4) message
	if((egress_wait_mailbox_msg(0x4000, SWAP32(DM_CSIX_IMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Master NPU did not rx DM_CSIX_IMI4\n");
		return(pOut->PassFail);
	}

	// Load receive ucode module
	DownLoadUcode(SF_LOOPBACK_RX_ME, sizeof(sf_loopback_rx), (UINT32 *)sf_loopback_rx);
	DelayNMsec(10);

	// Send mailbox msg (DM_CSIX_EMI4) to Ingress								
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_CSIX_EMI4;


	// wait for packets to be rx'd by receive ucode; error if too long
 	if ((wait_mailbox_msg(0x8000, DM_PACKETSRCVD)) == FAIL)
	{
		printf("Timed out...Slave NPU did not receive all data packets\n");
		//Cleanup_before_return();   
		//return(FAIL);
	}										 

	// Message (DM test_complete) message to Ingress	
	*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + \
						PCI_CSR_BASE_FRM_PCI+ MAILBOX_0) = DM_TEST_COMPLETE;
				 


	// Test results displayed to Egress user terminal 										  
	printf("Fabric CSIX test results\n");

	// Read the Ingress' tx'd data
	*(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT) = SWAP32(*(volatile unsigned int *)(Ingress_msf_Scratch_addr + XS_TXPACKETCOUNT));
																		   
	Tx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_TXPACKETCOUNT);
	Rx_Pkts = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETCOUNT);
	Rx_Errs = *(volatile unsigned int *)(Egress_msf_Scratch_addr + XS_RXPACKETERR);
	printf("Port 1 Tx Packets 0x%X Rx Packets 0x%X Rx Errors 0x%X\n", Tx_Pkts, Rx_Pkts, Rx_Errs);
	
	// Test complete message to Egress user terminal
	printf("Fabric CSIX Loopback test complete\n");
				 
	// Clear the mailbox registers
	ZeroEgressMailboxes();
	ZeroIngressMailboxes();
																						  
	// Cleanup and exit test
	Cleanup_before_return();
	
//	Msf_OutOfReset();

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}


	if(Rx_Errs) 
		return(pOut->PassFail);
	if(Tx_Pkts != Rx_Pkts) 
		return(pOut->PassFail);
	else
		{
		pOut->PassFail = PASS;
		return(pOut->PassFail);
		}

}


// ****************************************************************************
// Function: void Sf_Loopback_Test_Slave(void)
//
// This function is called by the PCI Doorbell interrupt handler when the
// master initiates a PCI doorbell interrupt by setting a particular bit on
// the slave's doorbell register
// ****************************************************************************
void Sf_Loopback_Test_Slave(void)
{
	UINT32 mask=0, mailbox_value, status, total_len, packet_num, rx_num_of_bytes=0;
	UINT32 get_ring_addr, put_ring_addr, loop_count, forever;
	UINT32 ctr, temp=0, flag, num_lw, lw1, lw2;
	UINT dcache_status;
	int payload_len;
	int flow_status;

	dcache_status = get_CP15() & DCACHE_BIT;
	if(dcache_status != 0)	// if dcache ON
	{
		dcache_off();
	}
		

	Scroll_Off();

	printf("Ingress starting CSIX fabric test\n");
	
	ingress_media_init_1();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI1;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI1))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI1\n");
		return;
	}

	ingress_media_init_2();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI2;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI2))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI2\n");
		return;
	}

	flow_status = ingress_media_init_3();
	DelayNMsec(10);

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI3;
	// Let egress know if we failed to get flow control
	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2) = flow_status;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI3))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI3\n");
		return;
	}

	*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_0) = DM_CSIX_IMI4;

	if((wait_mailbox_msg(0x3000, SWAP32(DM_CSIX_EMI4))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_CSIX_EMI4\n");
		return;
	}

	// Load transmit ucode module	
	DownLoadUcode(SF_LOOPBACK_TX_ME, sizeof(sf_loopback_tx), (UINT32 *)sf_loopback_tx);

	//ingress_test_packet();

	//DelayNMsec(10);

	if((wait_mailbox_msg(0x9000, SWAP32(DM_TEST_COMPLETE))) == FAIL)
	{
		Cleanup_before_return();
		printf("Timed out...Ingress NPU did not rx DM_TEST_COMPLETE\n");
		if(dcache_status != 0)	// if dcache was originally ON
		{
			dcache_on();
		}
		return;
	}

	printf("Ingress fabric test complete\n");

	// Cleanup and exit test
	Cleanup_before_return();

	if(dcache_status != 0)	// if dcache was originally ON
	{
		dcache_on();
	}
	
}

#define GET32(address)			(*(volatile unsigned int *)(address))
#define PUT32(address,val)		(*(volatile unsigned int *)(address) = (val))

#define get_msf(address)		GET32(0xC8000000+address)
#define set_msf(address,val)	PUT32((0xC8000000+address),val)


void ingress_media_init_1()
{
	int clock_control;
	int ixp_reset0;

	/* Setting CSR CLOCK_CONTROL */
	Msf_Clock_Speed_Tx();

	/* Setting CSR IXP_RESET0 */
	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00400080;
	ixp_reset0 &= 0xFFFF7FFF;
	PUT32(0xC0004A0C, ixp_reset0);
	DelayNMsec(10);

	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00408000;
	ixp_reset0 &= 0xFFFFFF7F;
	PUT32(0xC0004A0C, ixp_reset0);
	DelayNMsec(10);

	/* Setting CSR IXP_RESET1 */
	PUT32(0xC0004A10, 0x00000000);
	DelayNMsec(10);

	/* Setting CSR MSF_IO_BUF_CNTL */
	set_msf(0x8008, 0x000016A3);
	DelayNMsec(10);

	/* Setting CSR FC_IO_BUF_CNTL */
	set_msf(0x800C, 0x000014AF);
	DelayNMsec(10);

	/* Setting CSR MSF_Tx_Control (select transmit clock) */
	set_msf(0x04, 0x00100805);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable transmit clocks) */
	set_msf(0x2C, 0x00000050);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable transmit sections) */
/*	set_msf(0x2C, 0x00000154);
	DelayNMsec(10);
*/

}


void egress_media_init_1()
{
	int clock_control;
	int ixp_reset0;

	/* Setting CSR CLOCK_CONTROL */
	Msf_Clock_Speed_Rx();

	/* Setting CSR IXP_RESET0 */
	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00400080;
	ixp_reset0 &= 0xFFFF7FFF;
	PUT32(0xC0004A0C, ixp_reset0);
	DelayNMsec(10);

	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00408000;
	ixp_reset0 &= 0xFFFFFF7F;
	PUT32(0xC0004A0C, ixp_reset0);
	DelayNMsec(10);

	/* Setting CSR IXP_RESET1 */
	PUT32(0xC0004A10, 0x00000000);
	DelayNMsec(10);

	/* Setting CSR MSF_IO_BUF_CNTL */
	set_msf(0x8008, 0x000014AF);
	DelayNMsec(10);

	/* Setting CSR FC_IO_BUF_CNTL */
	set_msf(0x800C, 0x000014AF);
	DelayNMsec(10);

	/* Setting CSR MSF_Rx_Control (configure receive section for 2 partiton mode/duplex mode) */
	set_msf(0x0, 0x00000001);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive clock select) */
	set_msf(0x2C, 0x00003000);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive clock dll) */
	set_msf(0x2C, 0x00003C00);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive sections) */
	set_msf(0x2C, 0x00003C0A);
	DelayNMsec(10);

	/* Setting CSR MSF_Tx_Control (select transmit clock) */
	set_msf(0x04, 0x00000904);
	DelayNMsec(10);

	/* Enabling CSR MSF_Tx_Control */
	set_msf(0x04, 0x20000D04);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable transmit clocks) */
	set_msf(0x2C, 0x00003C5A);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable transmit sections) */
	set_msf(0x2C, 0x00003D5F);
	DelayNMsec(10);

#if 0
	/* Setting CSR MSF_Clock_Control (enable receive clock) */
	set_msf(0x2C, 0x00003D7F);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive calender section) */
	set_msf(0x2C, 0x00003F7F);
	DelayNMsec(10);
#endif

}


void ingress_media_init_2()
{
	/* Setting CSR MSF_Rx_Control (configure receive section for 2 partiton mode/duplex mode) */
	set_msf(0x0, 0x00000004);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive clock select) */
	set_msf(0x2C, 0x00003150);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive clock dll) */
	set_msf(0x2C, 0x00003D50);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive sections) */
	set_msf(0x2C, 0x00003D50);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive clock) */
	set_msf(0x2C, 0x00003D7F);
	DelayNMsec(10);

	/* Setting CSR MSF_Clock_Control (enable receive calender section) */
	set_msf(0x2C, 0x00003F7D);
	DelayNMsec(10);

	/* Enabling CSR MSF_Tx_Control */
	set_msf(0x04, 0x80100C01);
	DelayNMsec(10);

	/* Send data training to Egress */
//	printf("\nSending continuous data training to Egress.\n");
	/*set_msf(0xa0, 0x00000100); *****************************************************/	
	set_msf(0xa0, 0x00008100);  
	DelayNMsec(10);

//	printf("\n");

}


int egress_media_init_2()
{
	int msf_interrupt_status, dip4_err, rec_data_train;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Wait for received data training MSF_INTERRUPT_STATUS (bit 7 should be set) */

	while(attempt <= MAX_ATTEMPT)
	{

//	printf("\nAttempt %d to receive data training from Ingress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	DelayNMsec(10);

//	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	dip4_err = (msf_interrupt_status >> 2) & 0x1;
	rec_data_train = (msf_interrupt_status >> 7) & 0x1;

	//if (rec_data_train==0)
	//{
	//	printf("\tNo data training received.\n");
	//}

	//if (dip4_err==1)
	//{
	//	printf("\tDPI4 parity failed.\n");
	//	set_msf(0x08, 0x00000004);
	//	DelayNMsec(10);
	//}

	//if (rec_data_train==1 && dip4_err==0)
	//{
	//	printf("\tData training received successfully!\n");
	//	break;
	//}
	
//	if (rec_data_train==0)
//	{
//		printf("\tNo data training received.\n");
//	}

	if (dip4_err==1)
	{
//		printf("\tDPI4 parity failed.\n");
		set_msf(0x08, 0x00000004);
		DelayNMsec(10);
	}

	if (rec_data_train==1 && dip4_err==0)
	{
//		printf("\tData training received successfully!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_data_train==0) 
	{
		printf("\nFailed to receive data training from Ingress after %d attempts.\n", MAX_ATTEMPT);
		return(FAIL);
	}		
	/* Enabling CSR MSF_Rx_Control */
	set_msf(0x00, 0x80000005);
	DelayNMsec(10);

	/* Setting CSR CSIX_TYPE_MAP */
	set_msf(0x0010, 0x08);

	/* Send flow control training to Ingress */
	printf("\nSending flow control training to Ingress\n");
	set_msf(0xa8, 0x00000004);
	DelayNMsec(10);

	printf("\n");

	return(PASS);

}

int ingress_media_init_3()
{
	int train_flow_control;
	int msf_interrupt_status, rec_flw_ctl_train, parity_err;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Stop training Egress */
//	printf("\nStopped sending continuous data training to Egress.\n");
        /*set_msf(0xa0, 0x00000000);******************************************/
	set_msf(0xa0, 0x00008000);
	DelayNMsec(10);

	/* Start receiving flow control training from Egress */
	train_flow_control = get_msf(0xa8);
	train_flow_control &= 0xFFFFFFFD;
	set_msf(0xa8, train_flow_control);
	DelayNMsec(10);

	/* Wait for flow control training MSF_INTERRUPT_STATUS (bit 9 should be set) */

	while(attempt <= MAX_ATTEMPT)
	{

//	printf("\nAttempt %d to receive flow control training from Egress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	DelayNMsec(10);

//	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	parity_err = ((msf_interrupt_status >> 6) & 0x1) | ((msf_interrupt_status >> 15) & 0x1);
	rec_flw_ctl_train = (msf_interrupt_status >> 9) & 0x1;

	if (rec_flw_ctl_train==0)
	{
//		printf("\tNo flow control training received.\n");
	}

	if (parity_err==1)
	{
//		printf("\tParity failed.\n");
		set_msf(0x08, 0x00008040);
		DelayNMsec(10);
	}

	if (rec_flw_ctl_train==1 && parity_err==0)
	{
		printf("\nFlow control training received successfully!\n");
		break;
	}
	
	attempt++;

	}

	if (!rec_flw_ctl_train)
		{
		printf("\nFailed to receive flow control training from Egress after %d attempts.\n", MAX_ATTEMPT);
		return(FAIL);
		}

	/* Stop receiving flow control training */
	set_msf(0xa8, 0x00000002);
	DelayNMsec(10);

	/* Setting CSR FC_STATUS_OVERRIDE */
	set_msf(0x28, 0x00000000);
	DelayNMsec(10);

	printf("\n");

	return(PASS);
}


void egress_media_init_3()
{
	int msf_interrupt_status, rec_data_train, data_train_stopped;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Clear REC_DATA_TRAIN */
	set_msf(0x08, 0x00000080);
	DelayNMsec(10);

	/* Wait for end of data training MSF_INTERRUPT_STATUS (bit 7 should be reset) */

	while(attempt <= MAX_ATTEMPT)
	{

//	printf("\nAttempt %d to stop receiving data training from Ingress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	DelayNMsec(10);

//	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	rec_data_train = (msf_interrupt_status >> 7) & 0x1;
	data_train_stopped = (msf_interrupt_status >> 12) & 0x1;

	if (rec_data_train==1 || data_train_stopped==0)
	{
//		printf("\tStill receiving data training.\n");
		set_msf(0x08, 0x00000080);
		DelayNMsec(10);
	}

	if (rec_data_train==0 && data_train_stopped==1)
	{
		printf("Successfully stopped receiving data training!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_data_train==1 || data_train_stopped==0)
		printf("\nFailed to stop receiving data training from Ingress after %d attempts.\n", MAX_ATTEMPT);

	/* Setting CSR FC_STATUS_OVERRIDE */
	set_msf(0x28, 0x00000000);
	DelayNMsec(10);

}

void ingress_test_packet()
{
	set_msf(0x2000, 0xc43a0000);
	set_msf(0x2004, 0x00050200);
	set_msf(0x2008, 0x00ff0024);
	set_msf(0x200C, 0x00000000);
	set_msf(0x2010, 0x00050000);
	set_msf(0x2014, 0x00000000);
	set_msf(0x2018, 0x45000028);
	set_msf(0x201C, 0x00000000);
	set_msf(0x2020, 0x0306a90f);
	set_msf(0x2024, 0x18877638);
	set_msf(0x2028, 0xc001c000);
	set_msf(0x202C, 0x00010203);
	set_msf(0x2030, 0x04050607);
	set_msf(0x2034, 0x08090a0b);
	set_msf(0x2038, 0x0c0d0e0f);
	set_msf(0x203C, 0x10111213);		 


	set_msf(0x1800, 0x40000001);
	set_msf(0x1804, 0x87654321);
//	set_msf(0x1808, 0x40000001);
//	set_msf(0x180c, 0x87654321);

}






// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_SF_TEST,"fabric",Sf_Loopback_Test,DIAGNOSTIC,"SFTest","CSIX switch fabric loopback test on fabric connector");
CommandEnd();






