/******************************************************************************
*
* Module:       shellvars.cpp
*
* Description:  Console Module Definitions
*
* Project:      HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:      See Visual SourceSafe History
*   BL - Added gLocalEcho global variable
*
* Disclaimer:   Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"
#include "shell.h"
#include "vconsole.h"
#ifdef FFS_ON
#include "ffs.h"
#endif
#ifdef ESTACK_ON
#include "estack.h"
#include "vstack.h"
#include "vxBootline.h"
#endif
#include "cmd.h"

//  External References
extern UINT16 pMACAddr[];

struct _VarHeader
{
    UINT32 isHidden;
    UINT16 Number;
    UINT16 Size;
};


// Initialize Local Echo variable
extern UINT8 gLocalEcho;

void InitShellVars(void)
{
    universal u;
	int  PCI_Master;

    gLocalEcho = LOCAL_ECHO;
    u.pVoid = GetShellVarPointer(SHELLVAR_PROMPT);
	//strcpy((char *)u.pByte, DEFAULTPROMPT);

	if(IS_EGRESS_NPU)
	{
	strcpy((char *)u.pByte, DEFAULTPROMPT1);	
	}
	else
	{
	strcpy((char *)u.pByte, DEFAULTPROMPT2);
	}

#ifdef ESTACK_ON
    u.pVoid = GetShellVarPointer(SHELLVAR_MAC);
    u.pHalf[0] = pMACAddr[0];
    u.pHalf[1] = pMACAddr[1];
    u.pHalf[2] = pMACAddr[2];

    u.pVoid = GetShellVarPointer(SHELLVAR_IP);
    *u.pWord = IP_ADDR;

    u.pVoid = GetShellVarPointer(SHELLVAR_SUBNET);
    *u.pWord = SUBNET_MASK;

    u.pVoid = GetShellVarPointer(SHELLVAR_GATEWAY);
    *u.pWord = GATEWAY_IP;

    u.pVoid = GetShellVarPointer(SHELLVAR_REMOTENAME);
    strcpy((char *)u.pByte, RHOST_NAME);

    u.pVoid = GetShellVarPointer(SHELLVAR_REMOTEIP);
    *u.pWord = RHOST_IP;

    u.pVoid = GetShellVarPointer(SHELLVAR_DEFAULTCHANNEL);
    *u.pByte = COMMANDCHANNELOUT;

	u.pVoid = GetShellVarPointer(SHELLVAR_OS_BOOTLINE);
	strcpy((char *)u.pByte, vxBOOTLINE_GEN);

	u.pVoid = GetShellVarPointer(SHELLVAR_vxBOOT_DEV_NAME);
	strcpy((char *)u.pByte, vxBOOT_DEV_NAME);
	
	u.pVoid = GetShellVarPointer(SHELLVAR_vxELF_FILE_LOC);
	strcpy((char *)u.pByte, vxELF_FILE_LOC);
	
	u.pVoid = GetShellVarPointer(SHELLVAR_vxUSERNAME);
	strcpy((char *)u.pByte, vxUSERNAME);
	
	u.pVoid = GetShellVarPointer(SHELLVAR_vxPASSWORD);
	strcpy((char *)u.pByte, vxPASSWORD);
	
	u.pVoid = GetShellVarPointer(SHELLVAR_vxFLAGS);
	strcpy((char *)u.pByte, vxFLAGS);
	
	u.pVoid = GetShellVarPointer(SHELLVAR_vxTARGET_NAME);
	strcpy((char *)u.pByte, vxTARGET_NAME);
	
#endif
}

#ifdef FFS_ON
char *BootFileName = "boot.ini";

extern "C" UINT32 ReadShellVars(void *pShellVars, UINT32 Size)
{
    HANDLE Handle;
    UINT32 BytesRead = 0;
    struct _VarHeader VarHeader;
    UINT32 Offset = 0;
    universal u;
    int n, i;
    tShellVar *pSV = (tShellVar *)__SHELLVAR_START;

    InitShellVars();

    if ((Handle = OpenFile(BootFileName)) != 0)
    {
        
        /*
         * 500 is an arbitrary number - if we reach it, something went wrong, 
         * so end the loop  
         */
         
        for(i = 0; i < 500; i++)
        {
            ReadFile(Handle, Offset, (UINT8 *)&VarHeader, sizeof(struct _VarHeader), &BytesRead);
            if (BytesRead != sizeof(struct _VarHeader))
                break;
            pSV[i].isHidden = VarHeader.isHidden;
            n = (VarHeader.Size + 7) & ~7;
            if ((u.pVoid = GetShellVarPointer(VarHeader.Number)) != 0)
                ReadFile(Handle, Offset + sizeof(struct _VarHeader), u.pByte, n, &BytesRead);
            Offset += sizeof(struct _VarHeader) + n;
        }
        CloseFile(Handle);
    }
    return (Handle != 0);
}

extern "C" UINT32 WriteShellVars(void *pShellVars, UINT32 Size)
{
    HANDLE Handle;
    UINT32 BytesWritten = 0;
    UINT32 ShellVars = SHELLVARS;
    tShellVar *pSV = (tShellVar *)__SHELLVAR_START;
    int i;
    struct _VarHeader VarHeader;
    universal u;
    int Errors = 0;
    int n;

    DeleteFile(BootFileName);
    Handle = CreateFile(BootFileName, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF);

    for (i = 0; (i < ShellVars) && (!Errors); i++)
    {
        VarHeader.isHidden = pSV[i].isHidden;
        VarHeader.Number = *pSV[i].pNumber;
        VarHeader.Size = pSV[i].Size * pSV[i].Elements;
        WriteFile(Handle, (UINT8 *)&VarHeader, sizeof(struct _VarHeader), &BytesWritten);
        Errors += (BytesWritten != sizeof(struct _VarHeader));
        u.pVoid = GetShellVarPointer(*pSV[i].pNumber);
        n = (VarHeader.Size + 7) & ~7;
        WriteFile(Handle, u.pByte, n, &BytesWritten);
        Errors += (BytesWritten != n);
    }

    CloseFile(Handle);
    return(Errors == 0);
}
#else
extern "C" UINT32 ReadShellVars(void *pShellVars, UINT32 Size)
{
    InitShellVars();
    return 0;
}

extern "C" UINT32 WriteShellVars(void *pShellVars, UINT32 Size)
{
    return 0;
}
#endif

char *CommandChannels[] =
{
    "serial",
    "Ethernet",
    "Transparent"
};

ShellVar(SHELLVAR_DEFAULTCHANNEL, "Command Channel", SHELLTYPE_SELECTION2,0,1,1, SHELLVAR_DISPLAY, 0,CommandChannels);
