/* ME_Timers_Counters_Test.uc
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP2800
 *  subsystem: DIAG
 *  author: 
 *  revision:
 * 
 * --------------------------------------------------------------------------
 */
					     

#ifndef STDMAC_UC
#define STDMAC_UC

#include "constants.uc"

// API:
//		add(out_result, in_a, in_b)
//		add_c(out_result, in_a, in_b)
//		add_shf_left(out_result, in_a, in_b, in_shift_amt)
//		add_shf_right(out_result, in_a, in_b, in_shift_amt)
//		alu_op(out_result, in_a, op_spec, in_b)
//		alu_rot_left(out_result, in_a, op_spec, in_b, in_shift_amt)
//		alu_rot_right(out_result, in_a, op_spec, in_b, in_shift_amt)
//		alu_shf_left(out_result, in_a, op_spec, in_b, in_shift_amt)
//		alu_shf_right(out_result, in_a, op_spec, in_b, in_shift_amt)
//		and_shf_left(out_result, in_a, in_b, in_shift_amt)
//		and_shf_right(out_result, in_a, in_b, in_shift_amt)
//		array_index_from_elem_addr(out_index, in_addr, ELEMENT_SIZE, BASE_ADDRESS)
//			alias: index_from_addr
// 		balr(in_link_pc, target_label)
//		bitfield_extract(out_result, in_src, MSB, LSB)
//		bitfield_insert(out_result, in_a. in_b, MSB, LSB)
//		bits_clr(io_data, in_start_bit_pos, in_mask)
//		bits_set(io_data, in_start_bit_pos, in_mask)
//		divide(out_result, in_divisor, DIVIDEND)
//		elem_addr_from_array_index(out_addr, in_index, ELEMENT_SIZE, BASE_ADDRESS)
//			alias: addr_from_index
//		immed32(out_result, VAL)
//		move(out_result, in_src)
//		multiply(out_result, in_a, in_b)
//		or_shf_left(out_result, in_a, in_b, in_shift_amt)
//		or_shf_right(out_result, in_a, in_b, in_shift_amt)
//		rot_left(out_result, in_src, in_shift_amt)		
//		rot_right(out_result, in_src, in_shift_amt)
//		shf_left(out_result, in_src, in_shift_amt)		
//		shf_right(out_result, in_src, in_shift_amt)
//		sub(out_result, in_a, in_b)
//		sub_shf_left(out_result, in_a, in_b, in_shift_amt)
//		sub_shf_right(out_result, in_a, in_b, in_shift_amt)
// 		**** IXP2000 ****
//		arith_shf_right(out_result, in_src, shift_amt, SIGN_BIT_POS)
//		multiply32(out_result, in_a, in_b, OPERAND_SIZE)
//		multiply64(out_result, in_a, in_b, OPERAND_SIZE)
//		rand(out_result)
//		srand(in_src)


// internal macros to support stdmac

// _inv_shift_amt
//	Description: 	subtract in_shift_amt from 32
//
#ifdef EARLY_MEV2

#macro _inv_shift_amt(out_inv_shift_amt, in_shift_amt)
#if (MAX_IMMEDIATE > 0x1F)
	alu[out_inv_shift_amt, 32, -, in_shift_amt]
#else
	alu[out_inv_shift_amt, in_shift_amt, B-A, 1, <<5]
#endif
#endm

#endif


// _ixp_alu
//	Description: 	IXP1200 vs IXP2000 alu instructions
//					If not IXP1200 and op_spec +4, use immed to mask off the B operand
//
#macro _ixp_alu(out_result, in_a, op_spec, in_b)
#if (CHIP_VERSION == IXP1200)
	alu[out_result, in_a, op_spec, in_b]
#elif (streq ('op_spec', '+4'))
.begin
.reg ixp_temp
	alu_shf[ixp_temp, 0xF, AND, in_b]
	alu[out_result, in_a, +, ixp_temp]
.end
#else
	alu[out_result, in_a, op_spec, in_b]
#endif
#endm

// _ixp_alu_shf
//	Description: 	detect if ALU/ SHIFT operation can be done in one cycle
//					if so, use one instruction
//					else, take 2 instructions (shift and add)
//
#macro _ixp_alu_shf(out_result, in_a, op_spec, in_b, in_shift_spec)
#if (CHIP_VERSION == IXP1200)
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
// do AND, OR, ~AND, AND~, B with one instruction
#elif (streq ('op_spec', 'B'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', '~B'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', 'OR'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', 'ASR'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', 'AND'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', '~AND'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#elif (streq ('op_spec', 'AND~'))
	alu[out_result, in_a, op_spec, in_b, in_shift_spec]
#else
.begin
.reg ixp_temp
#if (streq ('op_spec', '+4'))
	alu_shf[ixp_temp, 0xF, AND, in_b, in_shift_spec]
	alu[out_result, in_a, +, ixp_temp]
#elif (streq ('op_spec', '+carry'))
	alu[out_result, in_a, +carry, 0]	
	alu_shf[ixp_temp, --, B, in_b, in_shift_spec]
	alu[out_result, out_result, +, ixp_temp]
#elif (streq ('op_spec', '-carry'))
	alu[out_result, in_a, -carry, 0]	
	alu_shf[ixp_temp, --, B, in_b, in_shift_spec]
	alu[out_result, out_result, -, ixp_temp]
#else
	alu_shf[ixp_temp, --, B, in_b, in_shift_spec]
	alu[out_result, in_a, op_spec, ixp_temp]
#endif
.end
#endif
#endm




// immed32
//		Description:load out_result with 32 bit constant
//					if constant fits in 16 bits, expand to a single immed 
//					if constant doesn't, expand into a immed_w0 and immed_w1
//					if it is a name (not a number), assume that it is being IMPORTED
//		Outputs:
//			out_result	GPR. destination register. 
//		Inputs:
//			VAL		constant
//		Size: 1-2 instructions (3 if output is xfer register)
//		Example Usage: immed32(output, 0x12345678);
//
#macro immed32(out_result, VAL)
#if (!isnum(VAL))
	#if (strstr(out_result, $))	; check for output xfer reg
		.begin
		.reg immed32_temp
		immed[immed32_temp, VAL]
		immed_w1[immed32_temp, VAL>>16]
		alu[out_result, --, b, immed32_temp]
	#else 	; !strstr(out_result, $)
		immed[out_result, VAL]
		immed_w1[out_result, VAL>>16]
	#endif	; strstr(out_result, $)
#elif (((VAL & 0xFFFF0000) == 0) || ((VAL & 0xFFFF0000) == 0xFFFF0000))
    immed[out_result,VAL]
#elif (((VAL & 0xFF0000FF) == 0) || ((VAL & 0xFF0000FF) == 0xFF0000FF))
    immed[out_result, (VAL >> 8), <<8]
#elif ((VAL & 0x0000FFFF) == 0)
    /* make sure that number is not seen as negative */
    immed[out_result, (0x0000FFFF & (VAL >> 16)), <<16]
#elif ((VAL & 0x0000FFFF) == 0x0000FFFF)
    /* make sure that number is seen as negative */
    immed[out_result, (0xFFFF0000 | (VAL >> 16)), <<16]
#else	; isnum(VAL)
	#if (strstr(out_result, $))	; check for output xfer reg
		.begin
		.reg immed32_temp
		immed[immed32_temp, (VAL & 0xFFFF)]
		immed_w1[immed32_temp, ((VAL >> 16) & 0xFFFF)]
		alu[out_result, --, b, immed32_temp]
		.end
	#else	; !(strstr(out_result, $))
		immed[out_result, (VAL & 0xFFFF)]
		immed_w1[out_result, ((VAL >> 16) & 0xFFFF)]
	#endif	; (strstr(out_result, $))
#endif	; (!isnum(VAL))
#endm


// balr
//	 	Description:  Save the current program counter in gpr in_link_pc and branch to
//					  target_label.  Program control can be returned to
//					  the current PC the current location using the rtn(in_link_pc) 
//					  instruction.
//		Inputs:
//			in_link_pc		GPR. to hold next pc for use by rtn instruction
//			target_label	subroutine start
//		Size: 2 instructions
//		Example Usage: balr(next, target);
//
#macro balr(in_link_pc, target_label)
    br [target_label], defer[1]
    load_addr[in_link_pc, ret_lab#]
ret_lab#:
#endm


// move
//		Description: move the source value into the destination register
//		Outputs:
//			out_result	destination GPR or write xfer
//		Inputs:
//			in_src	source GPR, read xfer or constant
//		Size: 1-3 instructions (2 if output is GPR, 3 if output is xfer)
//		Example Usage: move(output, input);
//
#macro move(out_result, in_src)
	#if (isnum(in_src)) 
		immed32[out_result, in_src]
	#else	;  ! isnum(in_src) 
	    alu[out_result, --, b, in_src]	
	#endif	; (isnum(in_src) 
#endm

// alu_op
//	 	Description: perform alu operation in_a op_spec in_b
//	 	Outputs:
//			out_result		GPR
//		Inputs:
//			in_a	register or constant	
//			op_spec		ALU operator
//						Legal operators are:
//							B 
//							~B 
//							+ 
//							+carry 
//							+4 
//							+8 
//							+16
//							- 
//							B-A 
//							AND 
//							~AND 
//							AND~ 
//							OR 
//							XOR 
//							+IFsign
//			in_b	register or constant	
//		Size: 1-5 instructions (6 if op_spec = +4)
//		Example Usage: alu_op(output, 0x12345678, +,  10);
//
#macro alu_op(out_result, in_a, op_spec, in_b)
#if ((isnum(in_b))&&(isnum(in_a))) //in_b and in_a are constant
	#if ((in_b > MAX_IMMEDIATE)||(in_b<0))
		.begin
		.reg _tempb
		immed32[_tempb, in_b]
		#if ((in_a > MAX_IMMEDIATE)||(in_a<0))
			.begin
			.reg _tempa
			immed32[_tempa,in_a]
			_ixp_alu[out_result, _tempa, op_spec, _tempb]
			.end
		#else 				// in_a < 32	
			_ixp_alu[out_result, in_a, op_spec, _tempb]
		#endif
		.end				
	#else 				// in_b < 32
		.begin
		.reg _tempa
		immed32[_tempa,in_a]
		_ixp_alu[out_result, _tempa, op_spec, in_b]
		.end
	#endif					
#elif (isnum(in_b))		//in_b is constant
	#if ((in_b > MAX_IMMEDIATE)||(in_b<0))
		.begin
		.reg _tempb
		immed32[_tempb, in_b]
		_ixp_alu[out_result, in_a, op_spec, _tempb]
		.end
	#else
		_ixp_alu[out_result, in_a, op_spec, in_b]
	#endif
#elif (isnum(in_a)) //in_a is constant
	#if((in_a > MAX_IMMEDIATE)||(in_a<0))
		.begin
		.reg _tempa
		immed32[_tempa, in_a]
		_ixp_alu[out_result, _tempa, op_spec, in_b]
		.end
	#else
		_ixp_alu[out_result, in_a, op_spec, in_b]
	#endif
#else
	_ixp_alu[out_result, in_a, op_spec, in_b]
#endif
#endm

// add
//		Description: 32 bit add in_a + in_b 
//	 	Outputs:
//			out_result		GPR
//		Inputs:
//			in_a	register or constant	
//			in_b	register or constant	
//		Size: 1-5 instructions
//		Example Usage: add(output, 0x1234, 0x12345678);
//
#macro add(out_result, in_a, in_b)
alu_op[out_result, in_a, +, in_b]
#endm


// add_c
// 		Description: 32 bit add in_a + in_b + previous carry
//	 	Outputs:
//			out_result		GPR
//		Inputs:
//			in_a	register or constant	
//			in_b	register or constant	
//		Size: 1-5 instructions
//		Example Usage: add_c(output, source_a, 0x12345678);
//
#macro add_c(out_result, in_a, in_b)
alu_op[out_result, in_a, +carry, in_b]
#endm


// sub
//	 	Description: subtract in_b from in_a
//	 	Outputs:
//			out_result		GPR
//		Inputs:
//			in_a	register or constant	
//			in_b	register or constant	
//		Size: 1-5 instructions
//		Example Usage: sub(output, 0x12345678, source);
//
#macro sub(out_result, in_a, in_b)
alu_op[out_result, in_a, -, in_b]
#endm


// shf_right
//		Description: shift right in_src by in_shift_amt bit positions
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_src			GPR	
//			in_shift_amt	register or constant	(0-31)
//		Size: 1-2 instructions
//		Example Usage: shf_right(output, input, 3);
//
#macro shf_right(out_result, in_src, in_shift_amt)
#if ( (strstr(out_result, 'n$')) && (!strstr(out_result, '*n$index')) ); if context relative next neighbor
	.begin
	.reg tmp_result
#else
	#ifdef tmp_result
		#warning "tmp_result is being redefined."
	#endif
	#define_eval tmp_result out_result
#endif
#if (isnum(in_src))
	#if ((in_src > MAX_IMMEDIATE)||(in_src<0))
		#define _use_temp_src
		.begin
		.reg shf_right_temp_src
		immed32[shf_right_temp_src, in_src]
	#endif
#endif
#if (isnum(in_shift_amt))
	#ifdef SHF_AMT
		#warning "SHF_AMT is being redefined."
	#endif
	#define_eval SHF_AMT in_shift_amt
	#ifdef _use_temp_src
		alu[tmp_result, --, B, shf_right_temp_src, >>/**/SHF_AMT/**/]
		.end 	//shf_right_temp_src
		#undef _use_temp_src
	#else
		alu[tmp_result, --, B, in_src, >>/**/SHF_AMT/**/]
	#endif
	#undef SHF_AMT
#else
	alu[--, in_shift_amt, OR, 0]
	#ifdef _use_temp_src
		alu[tmp_result, --, B, shf_right_temp_src, >>indirect]
		.end 	//shf_right_temp_src
		#undef _use_temp_src
	#else
		alu[tmp_result, --, B, in_src, >>indirect]
	#endif
#endif
#ifdef tmp_result;		used to support next neighbor registers
	#undef tmp_result
#else
	alu[out_result, --, b, tmp_result]
	.end;	tmp_result
#endif; def tmp_result
#endm


// shf_left
//		Description: shift left in_src by in_shift_amt bit positions
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_src			GPR	
//			in_shift_amt	register or constant	(0-31)
//		Size: 1-4 instructions
//		Example Usage: shf_left(output, input, reg);
//
#macro shf_left(out_result, in_src, in_shift_amt)
#if ( (strstr(out_result, 'n$')) && (!strstr(out_result, '*n$index')) ); if context relative next neighbor
	.begin
	.reg tmp_result
#else
	#ifdef tmp_result
		#warning "tmp_result is being redefined."
	#endif
	#define_eval tmp_result out_result
#endif
#if (isnum(in_src))
	#if ((in_src > MAX_IMMEDIATE)||(in_src<0))
		#define _use_temp_src
		.begin
		.reg shf_left_temp_src
		immed32[shf_left_temp_src, in_src]
	#endif
#endif
#if (isnum(in_shift_amt))
	#ifdef SHF_AMT
		#warning "SHF_AMT is being redefined."
	#endif
	#define_eval SHF_AMT in_shift_amt
	#ifdef _use_temp_src
		alu[tmp_result, --, B, shf_left_temp_src, <</**/SHF_AMT/**/]
		.end 	//shf_left_temp_src
		#undef _use_temp_src
	#else
		alu[tmp_result, --, B, in_src, <</**/SHF_AMT/**/]
	#endif
	#undef SHF_AMT
#else
	#ifdef EARLY_MEV2
	// HW prior to 11/8 needed to subtract shift amt from 32
	 	.begin
	 	.reg shf_left_temp_shf	
		_inv_shift_amt[shf_left_temp_shf, in_shift_amt]
		alu[--, shf_left_temp_shf, OR, 0]
		.end
	#else
		alu[--, in_shift_amt, OR, 0]
	#endif //EARLY_MEV2

	#ifdef _use_temp_src
		alu[tmp_result, --, B, shf_left_temp_src, <<indirect]
		.end 	//shf_left_temp_src
		#undef _use_temp_src
	#else
		alu[tmp_result, --, B, in_src, <<indirect]
	#endif
#endif
#ifdef tmp_result;		used to support next neighbor registers
	#undef tmp_result
#else
	alu[out_result, --, b, tmp_result]
	.end;	tmp_result
#endif; def tmp_result
#endm


// rot_right
//		Description: shift right in_src by in_shift_amt bit positions
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_src			GPR	
//			in_shift_amt	constant	(0-31)
//		Size: 1 instructions
//		Example Usage: rot_right(output, input, 3);
//
#macro rot_right(out_result, in_src, in_shift_amt)
#ifdef SHF_AMT
	#warning "SHF_AMT is being redefined."
#endif
#define_eval SHF_AMT in_shift_amt
	alu[out_result, --, B, in_src, >>rot /**/SHF_AMT/**/] 
#undef SHF_AMT
#endm


// rot_left
//		Description: shift left in_src by in_shift_amt bit positions
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_src			GPR	
//			in_shift_amt	constant	(0-31)
//		Size: 1 instruction
//		Example Usage: rot_left(output, input, 3);
//
#macro rot_left(out_result, in_src, in_shift_amt)
#ifdef SHF_AMT
	#warning "SHF_AMT is being redefined."
#endif
#define_eval SHF_AMT in_shift_amt
	alu[out_result, --, B, in_src, <<rot /**/SHF_AMT/**/] 
#undef SHF_AMT
#endm


// alu_shf_right
//		Description: shift right in_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_a		register or constant	
//			op_spec			alu operation
//			in_b		GPR	
//			in_shift_amt	register or constant	(0-31)
//		Size: 1-4 instructions
//		Example Usage: alu_shf_right(output, 0xABCD1234, +, input, 3);
//

#macro alu_shf_right(out_result, in_a, op_spec, in_b, in_shift_amt)
	#if (isnum(in_shift_amt))
		#ifdef SHF_AMT
			#warning "SHF_AMT is being redefined."
		#endif
		#define_eval SHF_AMT in_shift_amt
		#if (isnum(in_a))
			#if ((in_a>MAX_IMMEDIATE)||(in_a<0)||streq ('op_spec','+carry')||streq ('op_spec','-carry'))
				.begin
				.reg _tempa
				immed32[_tempa, in_a]
				_ixp_alu_shf[out_result, _tempa, op_spec, in_b, >>/**/SHF_AMT/**/]
				.end
			#else
				_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>/**/SHF_AMT/**/]
			#endif
		#else
			_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>/**/SHF_AMT/**/]
		#endif
		#undef SHF_AMT
	#else		// not isnum in_shift_amt
		#if ( streq ('op_spec', '+carry') || streq ('op_spec', '-carry') )
		//can't call _ixp_alu_shf for +carry or -carry because instructions have
		//to happen in a specific order to preserve the CARRY bit				
			.begin
			.reg _tempa
			#if (isnum(in_a))
				immed32[_tempa, in_a]
				alu[_tempa, _tempa, op_spec, 0]
			#else
				alu[_tempa, in_a, op_spec, 0]
			#endif
			alu[--, in_shift_amt, OR, 0]			//this instruction clears the carry bit
			#if ( streq ('op_spec', '+carry')
				_ixp_alu_shf[out_result, _tempa, + , in_b, >>indirect]
			#else
				_ixp_alu_shf[out_result, _tempa, - , in_b, >>indirect]
			#endif
			.end
		#else
			alu[--, in_shift_amt, OR, 0]
			#if (isnum(in_a)) 
				#if ((in_a > MAX_IMMEDIATE)||(in_a<0))
				.begin
				.reg _tempa
				immed32[_tempa, in_a]
				alu[--, in_shift_amt, OR, 0]
				_ixp_alu_shf[out_result, _tempa, op_spec, in_b, >>indirect]
				.end
				#else
					_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>indirect]
				#endif
			#else
				_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>indirect]
			#endif
		#endif
	#endif	
#endm


// commonly used alu ops
// add_shf_right
//		Description: shift right in_b by in_shift_amt bit positions, then add
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro add_shf_right(out_result, in_a, in_b, in_shift_amt)
	alu_shf_right[out_result, in_a, +, in_b, in_shift_amt]
#endm
// sub_shf_right
//		Description: shift right in_b by in_shift_amt bit positions, then subtract
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro sub_shf_right(out_result, in_a, in_b, in_shift_amt)
	alu_shf_right[out_result, in_a, -, in_b, in_shift_amt]
#endm
// and_shf_right
//		Description: shift right in_b by in_shift_amt bit positions, then and
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro and_shf_right(out_result, in_a, in_b, in_shift_amt)
	alu_shf_right[out_result, in_a, AND, in_b, in_shift_amt]
#endm
// or_shf_right
//		Description: shift right in_b by in_shift_amt bit positions, then OR
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro or_shf_right(out_result, in_a, in_b, in_shift_amt)
	alu_shf_right[out_result, in_a, OR, in_b, in_shift_amt]
#endm




// alu_shf_left
//		Description: shift left in_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_a		register or constant	
//			op_spec			alu operation
//			in_b		GPR	
//			in_shift_amt	register or constant	(0-31)
//		Size: 1-4 instructions
//		Example Usage: alu_shf_left(output, 0xABCD1234, +, input, 3);
//
#macro alu_shf_left(out_result, in_a, op_spec, in_b, in_shift_amt)
	#if (isnum(in_shift_amt))
		#ifdef SHF_AMT
			#warning "SHF_AMT is being redefined."
		#endif
		#define_eval SHF_AMT in_shift_amt
		#if (isnum(in_a))
			#if ((in_a>MAX_IMMEDIATE)||(in_a<0)||streq ('op_spec','+carry')||streq ('op_spec','-carry'))
				.begin
				.reg _tempa
				immed32[_tempa, in_a]
				_ixp_alu_shf[out_result, _tempa, op_spec, in_b, <</**/SHF_AMT/**/]
				.end
			#else
				// not (in_a > 31)
				_ixp_alu_shf[out_result, in_a, op_spec, in_b, <</**/SHF_AMT/**/]
				
			#endif		// not (in_a > 31)
		#else		// not (isnum(in_a))
			_ixp_alu_shf[out_result, in_a, op_spec, in_b, <</**/SHF_AMT/**/]
		#endif 		// not (isnum(in_a))
		#undef SHF_AMT
	#else		// not (isnum(in_shift_amt))

		#if ( streq ('op_spec', '+carry') || streq ('op_spec', '-carry') )
		//can't call _ixp_alu_shf for +carry or -carry because instructions have
		//to happen in a specific order to preserve the CARRY bit				
			.begin
			.reg _tempa
			#if (isnum(in_a))
				immed32[_tempa, in_a]
				alu[_tempa, _tempa, op_spec, 0]
			#else
				alu[_tempa, in_a, op_spec, 0]
			#endif
			alu[--, in_shift_amt, OR, 0]			//this instruction clears the carry bit
			#if ( streq ('op_spec', '+carry')
				_ixp_alu_shf[out_result, _tempa, + , in_b, <<indirect]
			#else
				_ixp_alu_shf[out_result, _tempa, - , in_b, <<indirect]
			#endif
			.end
		#else
			#if (isnum(in_a))
				#if ((in_a > MAX_IMMEDIATE)||(in_a<0))
					.begin
					.reg _tempa
					immed32[_tempa, in_a]

					alu[--, in_shift_amt, OR, 0]
					_ixp_alu_shf[out_result, _tempa, op_spec, in_b, <<indirect]
					.end
				#else
					alu[--, in_shift_amt, OR, 0]
					_ixp_alu_shf[out_result, in_a, op_spec, in_b, <<indirect]
				#endif
			
			#else
				alu[--, in_shift_amt, OR, 0]
				_ixp_alu_shf[out_result, in_a, op_spec, in_b, <<indirect]
			#endif
		#endif
	#endif
#endm


// commonly used alu ops
// add_shf_left
//		Description: shift left in_b by in_shift_amt bit positions, then add
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro add_shf_left(out_result, in_a, in_b, in_shift_amt)
	alu_shf_left[out_result, in_a, +, in_b, in_shift_amt]
#endm
// sub_shf_left
//		Description: shift left in_b by in_shift_amt bit positions, then subtract
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro sub_shf_left(out_result, in_a, in_b, in_shift_amt)
	alu_shf_left[out_result, in_a, -, in_b, in_shift_amt]
#endm
// and_shf_left
//		Description: shift left in_b by in_shift_amt bit positions, then AND
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro and_shf_left(out_result, in_a, in_b, in_shift_amt)
	alu_shf_left[out_result, in_a, AND, in_b, in_shift_amt]
#endm
// OR_shf_left
//		Description: shift left in_b by in_shift_amt bit positions, then or
//					in_a and shift result
//		Size: 1-4 instructions
//		Example Usage: add_shf_right(output, 0xABCD1234, +, input, 3);
//
#macro or_shf_left(out_result, in_a, in_b, in_shift_amt)
	alu_shf_left[out_result, in_a, OR, in_b, in_shift_amt]
#endm


// alu_rot_right
//		Description: rotate right in_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_a		register or constant	
//			op_spec			alu operation
//			in_b		GPR	
//			in_shift_amt	constant	(0-31)
//		Size: 1-3 instructions (4 if op_spec = +4)
//		Example Usage: alu_rot_right(output, input2, +, input, 4);
//						when: input2=5, input=0x12345678
//						results in output=0x8123456c
//
#macro alu_rot_right(out_result, in_a, op_spec, in_b, in_shift_amt)
#ifdef SHF_AMT
	#warning "SHF_AMT is being redefined."
#endif
#define_eval SHF_AMT in_shift_amt
	#if (isnum(in_a))
		#if ((in_a > MAX_IMMEDIATE)||(in_a<0))
			.begin
			.reg _tempa
			immed32[_tempa, in_a]
			_ixp_alu_shf[out_result, _tempa, op_spec, in_b, >>rot /**/SHF_AMT/**/]
			.end
		#else
			_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>rot /**/SHF_AMT/**/]
		#endif
	#else
		_ixp_alu_shf[out_result, in_a, op_spec, in_b, >>rot /**/SHF_AMT/**/]
	#endif 
#undef SHF_AMT
#endm


// alu_rot_left
//		Description: rotate left in_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	Outputs:
//			out_result			GPR
//		Inputs:
//			in_a		register or constant	
//			op_spec			alu operation
//			in_b		GPR	
//			in_shift_amt	constant	(0-31)
//		Size: 1-3 instructions (4 if op_spec = +4)
//		Example Usage: alu_rot_left(output, input2, +, input, 3);
//
#macro alu_rot_left(out_result, in_a, op_spec, in_b, in_shift_amt)
#ifdef SHF_AMT
	#warning "SHF_AMT is being redefined."
#endif
#define_eval SHF_AMT in_shift_amt
	#if (isnum(in_a))
		#if ((in_a > MAX_IMMEDIATE)||(in_a<0))
			.begin
			.reg _tempa
			immed32[_tempa, in_a]
			_ixp_alu_shf[out_result, _tempa, op_spec, in_b, <<rot /**/SHF_AMT/**/]
			.end
		#else
			_ixp_alu_shf[out_result, in_a, op_spec, in_b, <<rot /**/SHF_AMT/**/]
		#endif
	#else
		_ixp_alu_shf[out_result, in_a, op_spec, in_b, <<rot /**/SHF_AMT/**/]
	#endif 
#undef SHF_AMT
#endm


// bitfield_extract
// 		Description: extract a bit field from a register
// 					 bits are numbered 31-0, left to right
//
// 		Outputs:
//			out_result	extracted field
//		Inputs:
//			in_src		register with multiple fields
//			MSB			most significant, left bit defining field
//			LSB			least significant, right bit defining field
//
//		Size: 1-6 instructions
//
#macro 	bitfield_extract(out_result, in_src, MSB, LSB)

#ifdef _SHF_LEFT
	#warning "_SHF_LEFT is being redefined."
#endif
#ifdef _SHF_RIGHT
	#warning "_SHF_RIGHT is being redefined."
#endif

#define_eval _SHF_LEFT (31 - MSB)
#define_eval _SHF_RIGHT (_SHF_LEFT + LSB)

#if (MSB == 31)
	shf_right(out_result, in_src, LSB);
#elif ((MSB - LSB) == 7)
	alu_shf_right(out_result, 0, +8, in_src, LSB);
#elif ((MSB - LSB) == 15)
	alu_shf_right(out_result, 0, +16, in_src, LSB);
#elif ((MSB - LSB) == 4)
	alu_shf_right(out_result, 0x1F, AND, in_src, LSB);
#elif ((MSB - LSB) == 3)
	alu_shf_right(out_result, 0xF, AND, in_src, LSB);
#elif ((MSB - LSB) == 2)
	alu_shf_right(out_result, 0x7, AND, in_src, LSB);
#elif ((MSB - LSB) == 1)
	alu_shf_right(out_result, 0x3, AND, in_src, LSB);
#elif (MSB == LSB)
	alu_shf_right(out_result, 0x1, AND, in_src, LSB);
#else
	shf_left(out_result, in_src, _SHF_LEFT);
	shf_right(out_result, out_result, _SHF_RIGHT);
#endif

#undef _SHF_LEFT
#undef _SHF_RIGHT

#endm


// bitfield_insert
// 		Description: insert bitfield in_b into copy of in_a
//
// 		Outputs:
//			out_result	merged result
//		Inputs:
//			in_a		register with multiple fields
//			in_b		bits to be inserted (this field always start at bit 0. Do not 
//						shift to the position to be inserted)
//			MSB			most significant, left bit (bits left to right are 31-0)
//			LSB			least significant, right bit
//		Size:
//			2-4 instr, depending on size of bitfield (2 if mask fits in immediate)
//

// helper macro to get a mask
// because shift right 1s shifts in a sign, we are doing it the hard way.
// This macro defines 2 values to be used in bitfield_insert
#macro _calc_bitfield_mask(MSB, LSB)

#ifdef _NUM_BITS
	#warning "_NUM_BITS is being redefined."
#endif
#ifdef _BITFIELD_MASK
	#warning "_BITFIELD_MASK is being redefined."
#endif

#define_eval _NUM_BITS (MSB - LSB + 1)

#if (_NUM_BITS == 1)
#define_eval _BITFIELD_MASK 0x1
#elif (_NUM_BITS == 2)
#define_eval _BITFIELD_MASK 0x3
#elif (_NUM_BITS == 3)
#define_eval _BITFIELD_MASK 0x7
#elif (_NUM_BITS == 4)
#define_eval _BITFIELD_MASK 0xF
#elif (_NUM_BITS == 5)
#define_eval _BITFIELD_MASK 0x1F
#elif (_NUM_BITS == 6)
#define_eval _BITFIELD_MASK 0x3F
#elif (_NUM_BITS == 7)
#define_eval _BITFIELD_MASK 0x7F
#elif (_NUM_BITS == 8)
#define_eval _BITFIELD_MASK 0xFF
#elif (_NUM_BITS == 9)
#define_eval _BITFIELD_MASK 0x1FF
#elif (_NUM_BITS == 10)
#define_eval _BITFIELD_MASK 0x3FF
#elif (_NUM_BITS == 11)
#define_eval _BITFIELD_MASK 0x7FF
#elif (_NUM_BITS == 12)
#define_eval _BITFIELD_MASK 0xFFF
#elif (_NUM_BITS == 13)
#define_eval _BITFIELD_MASK 0x1FFF
#elif (_NUM_BITS == 14)
#define_eval _BITFIELD_MASK 0x3FFF
#elif (_NUM_BITS == 15)
#define_eval _BITFIELD_MASK 0x7FFF
#elif (_NUM_BITS == 16)
#define_eval _BITFIELD_MASK 0xFFFF
#elif (_NUM_BITS == 17)
#define_eval _BITFIELD_MASK 0x1FFFF
#elif (_NUM_BITS == 18)
#define_eval _BITFIELD_MASK 0x3FFFF
#elif (_NUM_BITS == 19)
#define_eval _BITFIELD_MASK 0x7FFFF
#elif (_NUM_BITS == 20)
#define_eval _BITFIELD_MASK 0xFFFFF
#elif (_NUM_BITS == 21)
#define_eval _BITFIELD_MASK 0x1FFFFF
#elif (_NUM_BITS == 22)
#define_eval _BITFIELD_MASK 0x3FFFFF
#elif (_NUM_BITS == 23)
#define_eval _BITFIELD_MASK 0x7FFFFF
#elif (_NUM_BITS == 24)
#define_eval _BITFIELD_MASK 0xFFFFFF
#elif (_NUM_BITS == 25)
#define_eval _BITFIELD_MASK 0x1FFFFFF
#elif (_NUM_BITS == 26)
#define_eval _BITFIELD_MASK 0x3FFFFFF
#elif (_NUM_BITS == 27)
#define_eval _BITFIELD_MASK 0x7FFFFFF
#elif (_NUM_BITS == 28)
#define_eval _BITFIELD_MASK 0xFFFFFFF
#elif (_NUM_BITS == 29)
#define_eval _BITFIELD_MASK 0x1FFFFFFF
#elif (_NUM_BITS == 30)
#define_eval _BITFIELD_MASK 0x3FFFFFFF
#elif (_NUM_BITS == 31)
#define_eval _BITFIELD_MASK 0x7FFFFFFF
#else (_NUM_BITS == 32)
#define_eval _BITFIELD_MASK 0xFFFFFFFF
#endif

#undef _NUM_BITS
#endm


#macro bitfield_insert(out_result, in_a, in_b, MSB, LSB)
.begin
.reg temp_merged

// define _BITFIELD_MASK
_calc_bitfield_mask(MSB, LSB);


#if (_BITFIELD_MASK & ~MAX_IMMEDIATE)
		.begin
		.reg mask
		immed32(mask, _BITFIELD_MASK);
		alu_shf_left(temp_merged, in_a, AND~, mask, LSB);
		.end
#else
		alu_shf_left(temp_merged, in_a, AND~, _BITFIELD_MASK, LSB);

#undef _BITFIELD_MASK

#endif

alu_shf_left(out_result, temp_merged, OR, in_b, LSB);
.end
#endm


// bits_clr
//		Description:		clear bits indicated by mask at starting position in_start_bit_pos
//		output/input:
//			io_data				register to modify
//		Inputs:
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_mask is shifted left by this amount.
//			in_mask			register or constant, mask of bits to clear							
//		Size: 3-5 instructions
//		Example Usage: bits_clr(reg2, bitpos, 0x3);
//
#macro bits_clr(io_data, in_start_bit_pos, in_mask)
	alu_shf_left(io_data, io_data, AND~, in_mask, in_start_bit_pos);
#endm


// bits_set
//		Description:		set bits indicated by mask at starting position in_start_bit_pos
//		output/input:
//			io_data				register to modify
//		Inputs:
//			in_start_bit_pos	register or constant<32, starting bit position. 
//								in_mask is shifted left by this amount.
//			in_mask			register or constant, mask of bits to clear							
//	 	Size: 3-5 instructions
//		Example Usage: bits_set(reg2,5,0xff);
//
#macro bits_set(io_data, in_start_bit_pos, in_mask)
	alu_shf_left(io_data, io_data, OR, in_mask, in_start_bit_pos);
#endm


//		Description:
//					out_result = mpy_in * MPY_CONST
//
//					MPY_CONST is known at compile time, so the
//					pre-processor uses its bit value to construct
//					an optimal multiply via alu/shift operations.
//
//					A simple implementation would take every
//					1-bit position n in MPY_CONST, and shift&add mpy_in
//					by n to accumulate the out_result.  So if there were
//					B 1-bits in MPY_CONST, this would take B instructions --
//					up to 32.
//
//					This implementation also uses subtraction, reducing the
//					maximum number of instructions to 16.
//
//					Eg. given MPY_CONST = 123 = 1111011b
//					123x = 64x + 32x + 16x + 8x + 2x + 1x
//					but also:
//					123x = 128x - 4x - 1x
//
//					
//		Outputs:
//					out_result		out_result = mpy_in * MPY_CONST
//		Inputs:
//					mpy_in		input GPR
//					MPY_CONST	any 32-bit input constant 
//
//		Performance:	n cycles: 1 to 16
//
//					Worst case performance is the number of 1 bits
//					in MPY_CONST.  0xxAAAAAAAA and 0x55555555 take
//					the maximum 16 cycles.
//
//					Actual performance depends on MPY_CONST like so:
//					1) Spans of two or more 1's cost 2 cycles.
//					1a) The exception is if there is only 1 span of 1's
//						and it is adjacent to bit 31 -- this costs 1 cycle.
//					2) isolated 0's or 1's cost 1 cycle each.
//
//	Resources:
//					memory -- no accesses, none consumed
//					registers -- no additional registers, just
//								out_result & input parameters.
//					instructions -- n: (n <= 16), as described above
//
//	Condition codes: data dependent -- assume they are clobbered.
//
//	Limitations:
//					mpy_in and out_result typically can't be the same GPR.
//					This is because add_shf_left and sub_shf_left
//					will operate on both out_result and mpy_in in the same cycle
//					and would result in a compile-time error.
//
//					However, mpy_in and out_result can be the same GPR in cases
//					where the macro expands to just a single instruction.
//					Eg. If MPY_CONST is zero or a positive or negative
//					even power of two.
//
//	Algorithm:
//			Start scanning the MPY_CONST from one end looking for the first
//			1-bit. If it is a single 1-bit, emit an add_shift and
//			continue. Otherwise, start counting 1-bits until you find two
//			or more 0-bits in a row. Then add the first of the two zero bits,
//			subtract the first 1-bit, and subtract any single 0-bits along
//			the way.
//
//	Size:    for IXP2000:	3-4 instructions
//
#macro multiply(out_result, in_a, in_b)
#if (CHIP_VERSION == IXP1200)

	#if (isnum(in_a) && isnum(in_b))
		// Both in_a and in_b are constant
		immed32(out_result, ((in_a) * (in_b)))	; pre-processor multiply

	#elif (!(isnum(in_a) || isnum(in_b)))
		// Neither in_a nor in_b are constant
		#error 4 "multiply() requires 1 or 2 constant parameters."

	#else
		// One of A or B is constant
		// arrange parameters for rest of macro
		#ifdef MPY_CONST
			#warning "MPY_CONST is being redefined."
		#endif
		#ifdef mpy_in
			#warning "mpy_in is being redefined."
		#endif
		#if isnum(in_a)
			// in_a is constant
			#define_eval MPY_CONST in_a
			#define_eval mpy_in in_b
		#elif isnum(in_b)
			// in_b is constant
			#define_eval MPY_CONST in_b
			#define_eval mpy_in in_a
		#endif

		#if (MPY_CONST == 0)	// multiply by 0
			immed32(out_result, 0) ; out_result = 0
		#endif

		#define MPY_FIRST_OUTPUT 1				// track first write to out_result word
		#define MPY_BASE_BIT 0
		#while (MPY_BASE_BIT < 32)

			// not currently inside a "span" of 1's

			// If this bit set...
			#if (MPY_CONST & (1 << MPY_BASE_BIT))

				// If this 1-bit is in position 31, or...
				// If next bit is clear, then "lone-1" -- not a span
				#ifdef MPY_NEXT_BIT
					#warning "MPY_NEXT_BIT is being redefined."
				#endif
				#define_eval MPY_NEXT_BIT (MPY_BASE_BIT + 1)
				#if ((MPY_BASE_BIT == 31) || (!(MPY_CONST & (1 << MPY_NEXT_BIT))))
					#if (MPY_FIRST_OUTPUT)
						shf_left(out_result, mpy_in, MPY_BASE_BIT) ; out_result = (mpy_in * 2^MPY_BASE_BIT)
						#define_eval MPY_FIRST_OUTPUT 0
					#else
						add_shf_left(out_result, out_result, mpy_in, MPY_BASE_BIT) ; out_result = out_result + (mpy_in * 2^MPY_BASE_BIT)
					#endif

				#else
					// MPY_BASE_BIT has a 1-bit
					// MPY_NEXT_BIT has a 1-bit
					// MPY_BASE_BIT < 31, so MPY_NEXT_BIT < 32

					// handle a span of 1's
					// span can be punctuated by single 0's
					// span is terminated by two 0's or end of word

					// handle bottom of span
					#if (MPY_FIRST_OUTPUT)
						sub_shf_left(out_result, 0, mpy_in, MPY_BASE_BIT); out_result = 0 - (mpy_in * 2^MPY_BASE_BIT)
						#define_eval MPY_FIRST_OUTPUT 0
					#else
						sub_shf_left(out_result, out_result, mpy_in, MPY_BASE_BIT); out_result = out_result - (mpy_in * 2^MPY_BASE_BIT)
					#endif

					// bits at BASE and NEXT are 1's, look ahead till end of span
					#define_eval MPY_NEXT_BIT (MPY_NEXT_BIT + 1)

					#while (((MPY_CONST & (1 << MPY_NEXT_BIT)) && (MPY_NEXT_BIT < 32)) || ((MPY_CONST & (1 << MPY_NEXT_BIT + 1)) && ((MPY_NEXT_BIT + 1) < 32)))

						// if skipping a lone 0, subtract it here
						#if (!(MPY_CONST & (1 << MPY_NEXT_BIT)))
							sub_shf_left(out_result, out_result, mpy_in, MPY_NEXT_BIT); out_result = out_result - (mpy_in * 2^MPY_BASE_BIT)
						#endif
						#define_eval MPY_NEXT_BIT (MPY_NEXT_BIT + 1)
					#endloop // #while in span

					// now MPY_NEXT_BIT indicates the next 0 past the span

					#if (MPY_NEXT_BIT == 32) // special case
						// if span extends through bit 31
						// we don't need to add in (mpy_in * 2^32),
						// because the previous subtraction did a borrow.
					#else
						add_shf_left(out_result, out_result, mpy_in, MPY_NEXT_BIT); out_result = out_result + (mpy_in * 2^SPAN_TOP_BIT)
					#endif
					
					#define_eval MPY_BASE_BIT (MPY_NEXT_BIT)	// skip past span
				#endif // we had a span of at least 2 1's in a row
				#undef MPY_NEXT_BIT						// clean up name space

			#endif // MPY_CONST has MPY_BASE_BIT set

			#define_eval MPY_BASE_BIT (MPY_BASE_BIT + 1)
		#endloop // #while (MPY_BASE_BIT < 32)

		#undef MPY_BASE_BIT					// clean up name space
		#undef MPY_FIRST_OUTPUT				// clean up name space
		#undef mpy_in						// clean up name space
		#undef MPY_CONST					// clean up name space

	#endif // A or B is constant
#else
	// for ixp2000 multiply maps to multiply32 with 16x16 operand size
	multiply32(out_result, in_a, in_b, OP_SIZE_16X16)
#endif
#endm // multiply



// divide
//
//		Description:
//					out_result = divisor / DIVIDEND
//		Outputs:
//					out_result		out_result = mpy_in * MPY_CONST
//		Inputs:
//					divisor		GPR variable or CONSTANT
//					DIVIDEND	for variable divisor
//									DIVIDEND must be power of 2
//								for CONSTANT divisor
//									DIVIDEND can be any CONSTANT
//
//		Performance:	variable/CONSTANT: 1 cycle
//						CONSTANT/CONSTANT: 1 - 2 cycles
//
//
//		Resources:
//					memory -- no accesses, none consumed
//					registers -- no additional registers, just
//								out_result & input parameters.
//					instructions -- 1 or 2
//
//		Condition codes: data dependent -- assume they are clobbered.
//
//		Limitations:
//					power of 2 constant DIVIDEND
//
//		Algorithm:
//					Determine what DIV_SHIFT yeilds (2^DIV_SHIFT == DIVIDEND)
//					Right shift divisor by DIV_SHIFT.
//
//		Size:		1-2 instructions
//
#macro divide(out_result, divisor, DIVIDEND)

	#if (!isnum(DIVIDEND))
		#error 4 "divide(out, divisor, DIVIDEND) requires CONSTANT power of 2 DIVIDEND."
	#elif (DIVIDEND == 0)
		#error 4 "divide(out, divisor, DIVIDEND): divide by 0"
	#elif (isnum(divisor))
		// Both divisor and DIVIDEND are constant -- go for it
		immed32(out_result, ((divisor) / (DIVIDEND)))	; pre-processor divide
	#else
		// divisor is variable and DIVIDEND is constant

		// Find what DIV_SHIFT gives (2^DIV_SHIFT == DIVIDEND)
		#ifdef DIV_SHIFT
			#warning "DIV_SHIFT is being redefined."
		#endif
		#ifdef DIV_TWO2SHIFT
			#warning "DIV_TWO2SHIFT is being redefined."
		#endif
		#define_eval DIV_SHIFT 0
		#define_eval DIV_TWO2SHIFT (1 << DIV_SHIFT)
		#while((DIV_SHIFT < 32) && (DIVIDEND != DIV_TWO2SHIFT))
			#define_eval DIV_SHIFT (DIV_SHIFT + 1)
			#define_eval DIV_TWO2SHIFT (1 << DIV_SHIFT)
		#endloop

		#if (DIV_TWO2SHIFT == DIVIDEND)
			shf_right(out_result, divisor, DIV_SHIFT)
		#else
			#error 1 "divide(out, divisor, DIVIDEND): DIVIDEND must be power of 2."
		#endif

		#undef DIV_SHIFT			// clean up name space
		#undef DIV_TWO2SHIFT		// clean up name space
	#endif // divisor is variable and DIVIDEND is constant

#endm // divide

//	array_index_from_elem_addr
//
//	Description:
//		Convert an address into an array index. Ie.
//		out_index = (in_address - BASE_ADDRESS) / ELEMENT_SIZE
//
//	Outputs:
//		out_index: index of the array element
//
//	Inputs: 
//		in_address		Array element's address
//		ELEMENT_SIZE	Size of Array Element in address units
// 		BASE_ADDRESS	Memory Address of 1st element in the array.
//
//	Implementation:
//		out_index = (in_address / ELEMENT_SIZE) - (BASE_ADDRESS / ELEMENT_SIZE)
//
//	Size: Depends on ELEMENT_SIZE and BASE_ADDRESS
//			typical: 3 = 2 immeds + 1 ALU
//
//	Limitations:
//		Attempts to compile for non power of 2 ELEMENT_SIZE will fail
//		because divide doesn't currently support non power of 2 divisors.

#macro array_index_from_elem_addr(out_index, in_address, BASE_ADDRESS, ELEMENT_SIZE)
.begin
	// Find what IFA_SHIFT gives (2^IFA_SHIFT == ELEMENT_SIZE)
	#ifdef IFA_SHIFT
		#warning "IFA_SHIFT is being redefined."
	#endif
	#ifdef IFA_TWO2SHIFT
		#warning "IFA_TWO2SHIFT is being redefined."
	#endif
	#define_eval IFA_SHIFT 0
	#define_eval IFA_TWO2SHIFT (1 << IFA_SHIFT)
	#while((IFA_SHIFT < 32) && (ELEMENT_SIZE != IFA_TWO2SHIFT))
		#define_eval IFA_SHIFT (IFA_SHIFT + 1)
		#define_eval IFA_TWO2SHIFT (1 << IFA_SHIFT)
	#endloop

	// if ELEMENT_SIZE is a power of two
	#if (IFA_TWO2SHIFT == ELEMENT_SIZE)
		#ifdef _TMP_IN_ADDRESS
			#warning "_TMP_IN_ADDRESS is being redefined"
		#endif

		#if (isnum(in_address))
			.reg tmp_in_address
			move(tmp_in_address, in_address);
			#define_eval _TMP_IN_ADDRESS	tmp_in_address
		#else
			#define_eval _TMP_IN_ADDRESS	in_address
		#endif

		alu_shf_right(out_index, ((BASE_ADDRESS) / (ELEMENT_SIZE)), b-a, _TMP_IN_ADDRESS, IFA_SHIFT)

		#ifdef _TMP_IN_ADDRESS
			#undef _TMP_IN_ADDRESS	// cleanup name space
		#endif
	#else	// ELEMENT_SIZE is NOT power of two

		// This is what we want to do, but
		// divide will not compile for non power of 2.
		sub[out_index, in_address, BASE_ADDRESS]
		divide[out_index, out_index, ELEMENT_SIZE]
	#endif

	#undef IFA_SHIFT			// clean up name space
	#undef IFA_TWO2SHIFT		// clean up name space
.end
#endm
// alias for legacy code
#define index_from_addr array_index_from_elem_addr


//	elem_addr_from_array_index
//
//	Description:
//		Convert an array element address to an array index. Ie.
//		out_address = (in_index * ELEMENT_SIZE) + BASE_ADDRESS
//
//	Outputs:
//		out_address		Address of the array element
//
//	Inputs: 
//		in_index		Index of the array element
//		ELEMENT_SIZE	Size of Array Element in address units
// 		BASE_ADDRESS	Memory Address of 1st element in the array.
//
//	Implementation:
//		if (ELEMENT_SIZE is power of 2)
//			out_address = (in_index << BUF_DAFI_SHIFT) + BASE_ADDRESS
//		else
//			out_address = (in_index * ELEMENT_SIZE) + BASE_ADDRESS
//
//	Size: Depends on ELEMENT_SIZE and BASE_ADDRESS
//			best: 1 ALU
//			typical: 3 = 2 immeds + 1 ALU
//			worst: 21 = 4 immeds + 17 ALU
//
#macro elem_addr_from_array_index(out_address, in_index, BASE_ADDRESS, ELEMENT_SIZE)

	// Find what AFI_SHIFT gives (2^AFI_SHIFT == ELEMENT_SIZE)
	#ifdef AFI_SHIFT
		#warning "AFI_SHIFT is being redefined."
	#endif
	#ifdef AFI_TWO2SHIFT
		#warning "AFI_TWO2SHIFT is being redefined."
	#endif
	#define_eval AFI_SHIFT 0
	#define_eval AFI_TWO2SHIFT (1 << AFI_SHIFT)
	#while((AFI_SHIFT < 32) && (ELEMENT_SIZE != AFI_TWO2SHIFT))
		#define_eval AFI_SHIFT (AFI_SHIFT + 1)
		#define_eval AFI_TWO2SHIFT (1 << AFI_SHIFT)
	#endloop

	// if ELEMENT_SIZE is a power of two
	#if (AFI_TWO2SHIFT == ELEMENT_SIZE)

		add_shf_left(out_address, BASE_ADDRESS, in_index, AFI_SHIFT)

	#else	// else ELEMENT_SIZE NOT a power of two
		multiply(out_address, in_index, ELEMENT_SIZE)
		add[out_address, out_address, BASE_ADDRESS]
	#endif

	#undef AFI_SHIFT			// clean up name space
	#undef AFI_TWO2SHIFT		// clean up name space
#endm
// alias for legacy code
#define addr_from_index elem_addr_from_array_index



// ******************** IXP2000 Extended Operations **************************


// arith_shf_right
//
//	Description:
//		Three cases
//		1. if SIGN_BIT_POS is set to 31, no sign extend will occur, just arith shift right.
//		2. if in_shift_amt is a constant 0, just sign extend will occur
//		3. If both in_shift_amt is non-zero and SIGN_BIT_POS is not 31, Sign extend from 
//			SIGN_BIT_POS left to bit 31. Then shift right, filling with sign b
//		
//	Outputs:
//		out_result		sign-extended shifted-right result
//
//	Inputs:
//		in_src			source operand.
//		in_shift_amt	register of constant right shift amount, values from 0-31 but cannot
//						be greater than SIGN_BIT_POS value.
//		SIGN_BIT_POS	constant position of sign bit. 31 if in_src is already sign-extended.
//
//	Size: 2-8 instr.
//
#macro arith_shf_right(out_result, in_src, in_shift_amt, SIGN_BIT_POS)
#if (CHIP_VERSION == IXP2000)
	#ifdef _SHF_AMT
		#warning "_SHF_AMT is being redefined."
	#endif
	#define_eval _SHF_AMT (31 - SIGN_BIT_POS)

	alu_shf_left(out_result, 0, OR, in_src, _SHF_AMT);
	#if (isnum(in_shift_amt))
		#ifdef _TOTAL_SHF_AMT
			#warning "_TOTAL_SHF_AMT is being redefined."
		#endif
		#define_eval _TOTAL_SHF_AMT (_SHF_AMT + in_shift_amt)
		asr[out_result, out_result, >>/**/_TOTAL_SHF_AMT/**/]
		#undef _TOTAL_SHF_AMT
	#else
		#if (_SHF_AMT > 0)
			asr[out_result, out_result, >>/**/_SHF_AMT/**/]
		#endif

		// Preserve MSB for asr instruction by testing the SIGN_BIT_POS
		//	of in_src
		br_bclr[in_src, SIGN_BIT_POS, _arith_shf_right_continue_#]
		nop
		alu_shf[in_shift_amt, in_shift_amt, OR, 1, <<31]

_arith_shf_right_continue_#:

		alu[--, in_shift_amt, OR, 0]	
		asr[out_result, out_result, >>indirect]
	#endif
	#undef _SHF_AMT
#else
	#error 4 "arith_shf_right(). wrong chip version. IXP2000 operation."
#endif
#endm


// multiply32
//
//	Description:
//		32 bit out_result = in_a * in_b, optimizing for OPERAND_SIZE specifications OP_SIZE_8X24, OP_SIZE_16X16.
//
//	Outputs:
//		out_result	32 bit multiply result
//
//	Inputs:
//		in_a		multiplicand
//		in_b		multiplier
//		OPERAND_SIZE	8x24	multiplier is 8 bits,  multiplicand is 24 bits
//						16x16	multiplier and multiplicand are 16 bits
//	Size:	3 for OP_SIZE_8x24
//			4 for OP_SIZE_16X16
//
#macro multiply32(out_result, in_a, in_b, OPERAND_SIZE)
#if (CHIP_VERSION == IXP2000)
	#if (OPERAND_SIZE == OP_SIZE_8X24)
		mul_step[in_a, in_b], 24x8_start
		mul_step[in_a, in_b], 24x8_step1
		mul_step[out_result, --], 24x8_last
	#elif (OPERAND_SIZE == OP_SIZE_16X16)
		mul_step[in_a, in_b], 16x16_start
		mul_step[in_a, in_b], 16x16_step1
		mul_step[in_a, in_b], 16x16_step2
		mul_step[out_result, --], 16x16_last
	#else
		#error 4 "multiply32(). illegal OPERAND_SIZE. valid sizes are OP_SIZE_8X24, OP_SIZE_16X16"
	#endif
#else
	#error 4 "multiply32(). wrong chip version. IXP2000 operation."
#endif
#endm


// multiply64(out_result_hi, out_result_lo, in_a, in_b, OPERAND_SIZE);
//
//	Description: 
//		64 bit output = in_a * in_b, optimizing for OPERAND_SIZE, specifications OP_SIZE_16x32, OP_SIZE_32x32.
//		Issue 8/14/01: HW doesn't yet support 16x32. A change is being investigated.
//
//	Outputs:
//		out_result_hi	most significant 32 bits of multiply result
//		out_result_lo	least significant 32 bits of multiply result
//
//	Inputs:
//		in_a		multiplicand
//		in_b		multiplier
//		OPERAND_SIZE	
//						OP_SIZE_16X32	multiplier is 16 bits,  multiplicand is 32 bits
//						OP_SIZE_32X32	multiplier and multiplicand are 32 bits
//	Size:
//		
//			5 instr for OP_SIZE_16X32
//			7 instr for OP_SIZE_32X32
//
#macro multiply64(out_result_hi, out_result_lo, in_a, in_b, OPERAND_SIZE)
#if (CHIP_VERSION == IXP2000)
	#if (OPERAND_SIZE == OP_SIZE_16X32)
		#error 4 "multiply64(). OPERAND_SIZE 16X32 not yet supported"
		mul_step[in_a, in_b], 32x16_start
		mul_step[in_a, in_b], 32x16_step1
		mul_step[in_a, in_b], 32x16_step2
		mul_step[out_result_lo, --], 32x16_last
		mul_step[out_result_hi, --], 32x16_last2

	#elif (OPERAND_SIZE == OP_SIZE_32X32)
		mul_step[in_a, in_b], 32x32_start
		mul_step[in_a, in_b], 32x32_step1
		mul_step[in_a, in_b], 32x32_step2
		mul_step[in_a, in_b], 32x32_step3
		mul_step[in_a, in_b], 32x32_step4
		mul_step[out_result_lo, --], 32x32_last
		mul_step[out_result_hi, --], 32x32_last2
	#else
		#error 4 "multiply64(). illegal OPERAND_SIZE. valid sizes are OP_SIZE_16X32, OP_SIZE_32X32"
	#endif
#else
	#error 4 "multiply64(). wrong chip version. IXP2000 operation."
#endif
#endm


// rand
//	Description:
// 		Generate a pseudo-random number
//	Output:
//		out_result		random number
//
//	Size: 2 instr.
//
#macro rand(out_result)
#if (CHIP_VERSION == IXP2000)
	local_csr_rd[PSEUDO_RANDOM_NUMBER]
	immed[out_result, 0]
#else
	#error 4 "rand(). wrong chip version. IXP2000 operation."
#endif
#endm


// srand
//	Description:
// 		Set pseudo-random number seed
//	Input:
//		in_src		random number seed, constant or register
//
//	Size: 3-5 instr.
//
#macro srand(in_src)
#if (CHIP_VERSION == IXP2000)
.begin
.reg srand_temp
	#if (isnum (in_src))
		immed32(srand_temp, in_src);
		local_csr_wr[PSEUDO_RANDOM_NUMBER, srand_temp]
	#else
		local_csr_wr[PSEUDO_RANDOM_NUMBER, in_src]
	#endif
	nop
	nop
	nop
	nop
	nop
	// Update the CSR so that subsequent reads will get
	//	random numbers
	local_csr_rd[PSEUDO_RANDOM_NUMBER]
	immed[srand_temp, 0]
.end
#else
	#error 4 "rand(). wrong chip version. IXP2000 operation."
#endif
#endm

#endif // STDMAC_UC