/* temperature.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ajp, Oct 2002
 * 
 * 01a, 08/19/03 ajp Allow tempd command to work from Egress for chip validation
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "adm1031.h"
#include "diag_utils.h"
#include "ixp2000Pci.h"
#include "ixp2800diag.h"
#include "ixp2x00.h"
#include "ixp2800Global.h"
#include "pci_diag.h"
#include "ixp2800Msf.h"


// User input data
typedef struct _Temp_displayIn
{
	UINT32	Sensor_Temp_Sel;    
	UINT32	Egress_Temp_Sel;    
	UINT32	Ingress_Temp_Sel;    
	UINT32  All_Sel;
} Temp_displayIn;

// User input data
typedef struct _Temp_writeIn				  
{
	UINT8	Temp_Addr_Sel;    
	UINT8	Temp_Data_Sel;    
} Temp_writeIn;

typedef struct _Temp_setIn
{
	UINT8	Local_temp_Tmin;     
	UINT8	Local_temp_Trange;   
	UINT8	Remote_1_temp_Tmin;     
	UINT8	Remote_1_temp_Trange;   
	UINT8	Remote_2_temp_Tmin;     
	UINT8	Remote_2_temp_Trange;   
} Temp_setIn;

int get_ADM1031_rev(void);
int ADM1031_check_diode_fault(void);
int ADM1031_check_reg_program(void);
void ADM1031_register_display(void);
//void ADM1031_init_settings(void);
UINT8 convert_if_negative(int);



//*****************************************************************
// Function:    int get_ADM1031_rev
// Description: Display the chipid and revision 
// Input:		none
// Outputs:		PASS/FAIL
//*****************************************************************

int get_ADM1031_rev(void)
{
	int sensor_id = 0;

	// find the sensor chips id and rev and display
 	if (i2c_read(ADM1031_ADDR, ADM1031_DeviceID, sensor_id) == ADM1031_CHIPID)
		printf("ADM1031 Chipid = %d\n", sensor_id);

	else
		return(ERROR);

	return(OK);
}


//*****************************************************************
// Function:    int ADM1031_check_reg_program
// Description: See if we can program the ADM1031 registers
// Input:		none
// Outputs:		PASS/FAIL
//*****************************************************************

int ADM1031_check_reg_program(void)
{
	int check_reg = 0;

	// Read the configuration register 1 
 	i2c_read(ADM1031_ADDR, ADM1031_Config_1, check_reg);

	// Reset the fan fault enable bit
 	i2c_write(ADM1031_ADDR, ADM1031_Config_1, (check_reg \
 								& (!ADM1031_Fan_Fault_En)));

	// Now read the register and see that bit is cleared
 	i2c_read(ADM1031_ADDR, ADM1031_Config_1, check_reg);
	if((check_reg & ADM1031_Fan_Fault_En) == ADM1031_Fan_Fault_En)
		{
		printf("Unable to write ADM1031 register low\n");
		return(ERROR);
		}

	// Set the fan fault enable bit
	i2c_write(ADM1031_ADDR, ADM1031_Config_1, (check_reg \
 									| ADM1031_Fan_Fault_En));
	// Check that fan fault is set
 	i2c_read(ADM1031_ADDR, ADM1031_Config_1, check_reg);
	if((check_reg & ADM1031_Fan_Fault_En) != ADM1031_Fan_Fault_En)
		{
		printf("Unable to write ADM1031 register high\n");
		return(ERROR);
		}


	return(OK);
}

//*****************************************************************
// Function:    ADM1031_register_display
// Description: Display all the ADM1031 registers
// Input:		none
// Outputs:		none
//*****************************************************************
void ADM1031_register_display(void)
{
	int i;
	int temp_disp = 0;
	unsigned char buf;

	if(IS_EGRESS_NPU)
	{
	printf("Thermal sensor can only be read from Ingress NPU\n");
	return;
	}

	//dump all ADM1031 info to screen
	for(i=ADM1031_Config_1;i<=ADM1031_Remote_Temp_2;i++)
	{
	 	i2c_read(ADM1031_ADDR, i, &buf);
		temp_disp = (int)buf;

		printf("ADM1031 register 0x%02x = 0x%08x\n", i, temp_disp);
	}

	for(i=ADM1031_DeviceID;i<=ADM1031_Therm_Behave_Rev;i++)
	{
	 	i2c_read(ADM1031_ADDR, i, &buf);
		temp_disp = (int)buf;

		printf("ADM1031 register 0x%02x = 0x%08x\n", i, temp_disp);
	}

}

//*****************************************************************
// Function:    ADM1031_register_write
// Description: Writes to ADM1031 register
// Input:		address and data to write
// Outputs:		none
//*****************************************************************
int ADM1031_register_write(Temp_writeIn* pIn, PassFail_testOut* pOut)
{
	unsigned char buf;

	if(IS_EGRESS_NPU)
	{
	printf("Thermal sensor can only be read from Ingress NPU\n");
	return;
	}

	// get the users register and data info and write to ADM1031
 	i2c_write(ADM1031_ADDR, pIn->Temp_Addr_Sel, pIn->Temp_Data_Sel);

	// display the data
	ADM1031_register_display();

	// now read the data back and verify it
 	i2c_read(ADM1031_ADDR, pIn->Temp_Addr_Sel, &buf);

	if (buf != pIn->Temp_Data_Sel)
		pOut->PassFail = FAIL;

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);

}




//*****************************************************************
// Function:    ADM1031_temp_display()
// Description: Display board temperature
// Input:		none
// Outputs:		none
//*****************************************************************

void ADM1031_temp_display(Temp_displayIn* pIn, void* pUnused)
{
	UINT n;
	UINT temp_data = 0;
	unsigned char buf;
	UINT32 OTHER_NPUs_PCI_CSR_BASE;				   
	int temp1, temp2, temp3;

	Scroll_Off();
	
	if(IS_EGRESS_NPU)
	{
		OTHER_NPUs_PCI_CSR_BASE = GET_OTHER_NPUs_PCI_CSR_BASE(0);

		Set_Pci_Doorbell(READ_TEMP_BIT, (OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI + XSCALE_DOORBELL_OFF));

		DelayNMsec(600);    										   

		temp1 = SWAP32(*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_1));
		temp2 = SWAP32(*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_2));
		temp3 = SWAP32(*(volatile unsigned int *)(OTHER_NPUs_PCI_CSR_BASE + PCI_CSR_BASE_FRM_PCI+ MAILBOX_3));
		
	printf("Board Sensor temperature = %d degrees C\n", temp1);
	printf("Egress temperature = %d degrees C\n", temp2);
	printf("Ingress temperature = %d degrees C\n", temp3);

	return;
	}


	if ((!pIn->Sensor_Temp_Sel) && (!pIn->All_Sel) && (!pIn->Egress_Temp_Sel) && (!pIn->Ingress_Temp_Sel))
		pIn->All_Sel = 1;

	if ((pIn->Sensor_Temp_Sel) || (pIn->All_Sel))
	{


		// registers contain the 8 MSBs of temperature measurement
	 	i2c_read(ADM1031_ADDR, ADM1031_Local_Temp_Value, &buf);
		printf("Board Sensor temperature = ");
		temp_data = (int)buf;
//		temp_data = convert_if_negative(temp_data);

		// Read the local extended temp register bits <7:6>
//	 	i2c_read(ADM1031_ADDR, ADM1031_Extended_Temp, temp_data);
//		n = ((temp_data >> LocalTempShft) & LocalTempMask);
		
		printf("%d degrees C\n", temp_data);
	}

	if ((pIn->Egress_Temp_Sel) || (pIn->All_Sel))
	{
		// registers contain the 8 MSBs of temperature measurement
	 	i2c_read(ADM1031_ADDR, ADM1031_Remote_Temp_1_Value, &buf);
		printf("Egress temperature = ");
//		temp_data = convert_if_negative(temp_data);
		temp_data = (int)buf;
	
		// Read the remote 1 extended temp register bits <2:0>
//	 	i2c_read(ADM1031_ADDR, ADM1031_Extended_Temp, temp_data);
//		n = (temp_data & RemoteTemp1Mask);
		
		printf("%d degrees C\n", temp_data);
	}
		
	if ((pIn->Ingress_Temp_Sel) || (pIn->All_Sel))
	{
		// registers contain the 8 MSBs of temperature measurement
	 	i2c_read(ADM1031_ADDR, ADM1031_Remote_Temp_2_Value, &buf);
		printf("Ingress temperature = ");
//		temp_data = convert_if_negative(temp_data);
		temp_data = (int)buf;
	
		// Read the remote 1 extended temp register bits <5:3>
//	 	i2c_read(ADM1031_ADDR, ADM1031_Extended_Temp, temp_data);
//		n = ((temp_data >> RemoteTemp2Shft) & RemoteTemp2Mask);
		
		printf("%d degrees C\n", temp_data);
	}

}

//*****************************************************************
// Function:    ADM1031_Egress_temp_display()
// Description: Display board temperature
// Input:		none
// Outputs:		none
//*****************************************************************

void ADM1031_Egress_temp_display(void* pUnused);
void ADM1031_Egress_temp_display(void* pUnused)
{
	UINT n;
	UINT temp_data = 0;
	unsigned char buf;
	UINT32 OTHER_NPUs_PCI_CSR_BASE;				   

	Scroll_Off();
	
	// registers contain the 8 MSBs of temperature measurement
 	i2c_read(ADM1031_ADDR, ADM1031_Local_Temp_Value, &buf);
	printf("Board Sensor temperature = ");
	temp_data = (int)buf;
	(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_1)) = temp_data;


	printf("%d degrees C\n", temp_data);
	// registers contain the 8 MSBs of temperature measurement
 	i2c_read(ADM1031_ADDR, ADM1031_Remote_Temp_1_Value, &buf);
	printf("Egress temperature = ");
	temp_data = (int)buf;
	(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_2)) = temp_data;
	
	printf("%d degrees C\n", temp_data);
		
	// registers contain the 8 MSBs of temperature measurement
 	i2c_read(ADM1031_ADDR, ADM1031_Remote_Temp_2_Value, &buf);
	printf("Ingress temperature = ");
	temp_data = (int)buf;
	(*(volatile unsigned int *)(IXP2000_PCI_CSR_BASE + MAILBOX_3)) = temp_data;
	
	printf("%d degrees C\n", temp_data);

}




//*****************************************************************
// Function:    ADM1031_convert_if_negative
// Description: Check if passed value is negative number
// Input:		Number to check
// Outputs:	    Converted value
//*****************************************************************

UINT8 convert_if_negative(int convert)
{
	// if MSB bit is set we've got a negative number
	if((convert & NEG_MASK) == NEG_MASK)
	{
	// invert bits and add one as it's negative
	convert = (!convert);
	convert = (convert++);
	printf("-");
	}

	return(convert);
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,
// "Friendly Name","Help String");
Command(CMD_TEMPD,"tempd",ADM1031_temp_display,DIAGNOSTIC,"Temperature_display",\
												"board sensor and NPU temperature display in degrees C");
Switch("s", BOOLEAN32ARG, "Display board sensor temperature", "");
Switch("e", BOOLEAN32ARG, "Display Egress  NPU  temperature", "");
Switch("i", BOOLEAN32ARG, "Display Ingress NPU  temperature", "");
Switch("a", BOOLEAN32ARG, "Display all temperatures", "");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,
// "Friendly Name","Help String");
Command(CMD_TEMPW,"tempw",ADM1031_register_write,DIAGNOSTIC,"Register_write",\
												"board sensor register write");
Arg(UINT8ARG, "register addr to write", "register addr to set");
Arg(UINT8ARG, "register data to write", "register data to set");
CommandEnd();

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,
// "Friendly Name","Help String");
Command(CMD_TEMPR,"tempr",ADM1031_register_display,DIAGNOSTIC,"Register_display",\
												"board sensor register display");
CommandEnd();


