/* timer.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Jan 14, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"
#include "ixp2800IrqCtl.h"
#include "ixp2800Global.h"
#include "ixp2x00.h"
#include "timer_test.h"
#include "led.h"
#include "diag_utils.h"

// static function prototypes
static int Timer_Help(void);
static int Generic_Timer_Test(void);
static void Watchdog_Timer_Test(void);

static volatile int diag_timer_int_flag = 0;

// User input data
typedef struct _Timer_testIn
{
	UINT32 generic_sel;
	UINT32 watchdog_sel;
	UINT32 both_sel;
} Timer_testIn;


//
// Function: Timer_Test
//
// Description: Calls the appropriate function(s) of the Timer test
//				depending on user input.
//
int Timer_Test(Timer_testIn* pIn, PassFail_testOut* pOut)
{
	char opt;
	unsigned int loops = 1;
	unsigned int a;
	UINT32 status1 = PASS;
	UINT32 status2 = PASS;

	//if ((!pIn->generic_sel) || (!pIn->watchdog_sel) || (!pIn->both_sel))	
	//{
	//	pIn->both_sel = 1;	
	//}
			 	
	if ((pIn->generic_sel) || (pIn->both_sel))	
		status1 = Generic_Timer_Test();

	if ((pIn->watchdog_sel) || (pIn->both_sel))	
		Watchdog_Timer_Test();

	pOut->PassFail = (status1 | status2);

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);

}

//
// Function: Generic_Timer_Test
//
// Description: Test 3 of the timers (does not test the watchdog timer)
//				Tests each of the timers separately by enabling each timer
//				and setting its timer counter. The interrupt handler for the
//				timer will set the diag_timer_int_flag, which will be checked
//				after a certain delay. If it is set, then the test for that
//				particular timer will PASS, else it FAILS.
//

static int Generic_Timer_Test(void)
{
	unsigned int Delay;
	UINT32 Major_Rev;
	UINT32 status = PASS;

	// check for IXP28x0 rev B0 and above
	Major_Rev = *(volatile unsigned int *)(PRODUCT_ID); 
	Major_Rev = (Major_Rev & 0xF0);
	Major_Rev = (Major_Rev >>8);

	if(Major_Rev <=2)
		{
		printf("Timer test requires IXP28x0 rev B0 and above\n\n");
		return(PASS);
		}

#ifdef TEST_TIMER1	// requires timer1 to be re-initialized because it is used by i2c
	// Timer 1
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER1_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER1_CLR, CLEAR_INTERRUPT);		// Clear Timer1 Interrupt
	PUT32(TIMERS_TIMER1_CLD, 0x00000FFF);			// Load Timer1 Counter
	PUT32(TIMERS_TIMER1_CTL, ACTIVATE_TIMER);		// Activate Timer1
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER1_BIT); 		// Set Timer1 IRQ ENABLE
	

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	printf("Timer 1... ");
	if(diag_timer_int_flag != 0)
	{
		printf("PASS!\n");
	}
	else
	{
		printf("FAIL!\n");
		status = FAIL;
	}
#endif




	// Timer 2
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER2_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLR, CLEAR_INTERRUPT);		// Clear Timer2 Interrupt
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER2_BIT); 		// Set Timer2 IRQ ENABLE
	PUT32(TIMERS_TIMER2_CLD, 0x0000020);			// Load Timer2 Counter
	PUT32(TIMERS_TIMER2_CTL, ACTIVATE_TIMER);		// Activate Timer2

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	printf("Timer 2... ");
	if(diag_timer_int_flag != 0)
	{
		printf("PASS!\n");
	}
	else
	{
		printf("FAIL!\n");
		status = FAIL;
	}


	// Timer 3
	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER3_BIT);		// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER3_CLR, CLEAR_INTERRUPT);		// Clear Timer3 Interrupt
	PUT32(TIMERS_TIMER3_CLD, 0x00000FFF);			// Load Timer3 Counter
	PUT32(TIMERS_TIMER3_CTL, ACTIVATE_TIMER);		// Activate Timer3
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER3_BIT); 		// Set Timer3 IRQ ENABLE
	
	diag_timer_int_flag = 0;	// Clear Timer Interrupt Flag

	Delay = 0xFFF;
	while((diag_timer_int_flag == 0) && --Delay);
	printf("Timer 3... ");
	if(diag_timer_int_flag != 0)
	{
		printf("PASS!\n");
	}
	else
	{
		printf("FAIL!\n");
		status = FAIL;
	}

		return(status);

}

//
// Function: Watchdog_Timer_Test
//
// Description: Enables the Watchdog timer and sets its timer counter.
//
static void Watchdog_Timer_Test(void)
{
	// Read the configuration strapping to decide if PCI master
	if(!IS_EGRESS_NPU)
	{
		printf("*WARNING*: It is recommended to reset Egress as well.\n");
		printf("Press any key to continue watchdog timer test\n");
		while(!KEYPRESS);
	}
	else
	{
		printf("*WARNING*: It is recommended to reset Ingress as well.\n");
		printf("Press any key to continue watchdog timer test\n");
		while(!KEYPRESS);
	}


	PUT32(IRQ_ENABLE_CLR_REG, IRQ_TIMER4_BIT);	// Clear IRQ ENABLE
	PUT32(TIMERS_TIMER4_CLR, CLEAR_INTERRUPT);	// Clear Timer Interrupt
	PUT32(TIMERS_TIMER4_CLD, 0x00000FFF);		// Load Timer4 Counter
	PUT32(TIMERS_TWDE, 0x1);					// Enable Watchdog Timer
	PUT32(TIMERS_TIMER4_CTL, ACTIVATE_TIMER);	// Activate Timer4
	PUT32(IRQ_ENABLE_SET_REG, IRQ_TIMER4_BIT); 	// Set Timer4 IRQ ENABLE
	PUT32(IXP_RESET0, WDOG_RST_ENABLE);

	printf("Waiting for Watchdog reset! Press Any Key to stop wait.\n");

	while(1)
	{
		if(KEYPRESS)
		{
			printf("Exiting Watchdog Reset wait.\n");
			break;
		}
	}

}

void Timer_Interrupt_Handler(void)
{
	diag_timer_int_flag = 1;

#ifdef TEST_TIMER1	// requires timer1 to be re-initialized because it is used by i2c
	PUT32(TIMERS_TIMER1_CTL, 0);	// Deactivate Timer1
	PUT32(TIMERS_TIMER1_CLR, 0x1);
#endif

	PUT32(TIMERS_TIMER2_CTL, 0);	// Deactivate Timer2
	PUT32(TIMERS_TIMER3_CTL, 0);	// Deactivate Timer3
	PUT32(TIMERS_TIMER4_CTL, 0);	// Deactivate Timer4
	
	PUT32(TIMERS_TIMER2_CLR, 0x1);
	PUT32(TIMERS_TIMER3_CLR, 0x1);
}

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_TIMER,"timer",Timer_Test,DIAGNOSTIC,"TIMERTest","generic & watchdog timer tests");
Switch("g", BOOLEAN32ARG, "generic timer test", "");
Switch("w", BOOLEAN32ARG, "watchdog timer test", "");
Switch("a", BOOLEAN32ARG, "both generic and watchdog timer tests", "");
CommandEnd();
