/* uart.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Jan 21, 02
 *  revisions:
 * 
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "ixp2800Diag.h"
#include "ixp2800IrqCtl.h"
#include "ixp2x00.h"
#include "diag_utils.h"
#include "uart.h"
#include "led.h"


// static function prototypes
static void UART_Help(void);
static int UART_Dump_Reg(void);
static int UART_Reg_Test(void);
static void UART_Polling_Test(void);
static int UART_Int_Test(void);
static void UART_FIFO_Polling_Test(void);
static int UART_FIFO_Int_Test(void);
static int _Test_Register(unsigned int reg_addr, unsigned int var);

char read_buffer[128];
char write_buffer[128];
volatile int write_counter = 0, read_counter = 0;
volatile int int_flag = 0;

int serial_osc;

// User input data
typedef struct _Uart_testIn
{
	UINT32 dump_sel;
	UINT32 reg_sel;
	UINT32 nonpoll_sel;
	UINT32 nonint_sel;
	UINT32 poll_sel;
	UINT32 int_sel;
	UINT32 all_sel;
} Uart_testIn;


//
// Function: UART_Test
//
// Description: Calls the appropriate function(s) of the UART test
//				depending on user input.
//

int UART_Test(Uart_testIn* pIn, PassFail_testOut* pOut)
{
	char opt;

	int HaltOnError = 0;
	int loops = 1;
 	int status1 = PASS;
 	int status2 = PASS;
 	int status3 = PASS;
 	int status4 = PASS;

	if ((pIn->dump_sel) || (pIn->all_sel))
		status1 = UART_Dump_Reg();

	if ((pIn->reg_sel) || (pIn->all_sel))
		status2 = UART_Reg_Test();

//	if ((pIn->nonpoll_sel) || (pIn->all_sel))
//	UART_Polling_Test();

	if ((pIn->nonint_sel) || (pIn->all_sel))
		status3 = UART_Int_Test();

//	if ((pIn->poll_sel) || (pIn->all_sel))
//	UART_FIFO_Polling_Test();

	if ((pIn->int_sel) || (pIn->all_sel))
		status4 = UART_FIFO_Int_Test();

	pOut->PassFail = (status1 | status2 | status3 | status4);

	pOut->ReturnCode = PASS;         

	return(pOut->PassFail);
}


//
// Function: UART_Dump_Reg
//
// Description: Displays the register values of:
//				DLRL, DLRH, IER, IIR, LCR, LSR, and SPR
//
static int UART_Dump_Reg(void)
{
	UINT32 temp, dlrl, dlrh;

	PUT32(UART_LCR, (GET32(UART_LCR) | (1<<7)));	// Bit 7 = 1; DLAB = 1

	// Read DLRL and DLRH
	dlrl = GET32(UART_DLRL);
	dlrh = GET32(UART_DLRH);

	PUT32(UART_LCR, (GET32(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0

	// Requires DLAB = 0 to access THR for serial output

	// Display DLRL and DLRH
	printf("DLRL - 0x%02x\n", dlrl);
	printf("DLRH - 0x%02x\n", dlrh);

	// Display IER
	temp = GET32(UART_IER);
	printf("IER  - 0x%02x\n", temp);

	// Display IIR
	temp = GET32(UART_IIR);
	printf("IIR  - 0x%02x\n", temp);

	// Display LCR
	temp = GET32(UART_LCR);
	printf("LCR  - 0x%02x\n", temp);

	// Display LSR
	temp = GET32(UART_LSR);
	printf("LSR  - 0x%02x\n", temp);

	// Display SPR
	temp = GET32(UART_SPR);
	printf("SPR  - 0x%02x\n", temp);

	return(PASS);
}

//
// Function: UART_Reg_Test
//
// Description: Checks for expected settings in the UART:
//
static int UART_Reg_Test(void)
{
	int status = OK, temp = 0;
	UINT32 lcr, iir, ier, dlrl, dlrh;

	DelayNUsec(1000);
	lcr = GET32(UART_LCR);
	if((lcr & 0x3) != 0x3)
	{
		printf("LCR Reg Read FAIL! Read 0x%x\n", lcr);
		return(FAIL);
	}

	iir = GET32(UART_IIR);
	if((iir & 0xc0) != 0xc0)	// check bits[7:6] are set for "FIFO mode is selected"
	{
		printf("IIR Reg Read FAIL! Read 0x%x\n", iir);
		return(FAIL);
	}

	ier = GET32(UART_IER);
	if((ier & 0x40) != 0x40)	//check if UART Unit is enabled
	{
		printf("IER Reg Read FAIL! Read 0x%x\n", ier);
		return(FAIL);
	}

	PUT32(UART_LCR, (GET32(UART_LCR) | (1<<7)));	// Bit 7 = 1; DLAB = 1

	dlrl = GET32(UART_DLRL);
	dlrh = GET32(UART_DLRH);

	PUT32(UART_LCR, (GET32(UART_LCR) & 0x7F));		// Bit 7 = 0; DLAB = 0
	PUT32(UART_IER, (GET32(UART_IER) & ~ENABLE_UART));// Disable UART
	PUT32(UART_LCR, (GET32(UART_LCR) | (1<<7)));		// Bit 7 = 1; DLAB = 1

	// *** Test UART_DLRL Register ***
	if(_Test_Register(UART_DLRL, dlrl) == ERROR)
	{
		PUT32(UART_LCR, (GET32(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0
		PUT32(UART_IER, (GET32(UART_IER) | ENABLE_UART));	// Enable UART
		DelayNUsec(10000);
		printf("UART_DLRL WRV FAIL!\n");
		return(FAIL);
	}


	// *** Test UART_DLRH Register ***
	if(_Test_Register(UART_DLRH, dlrh) == ERROR)
	{
		PUT32(UART_LCR, (GET32(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0
		PUT32(UART_IER, (GET32(UART_IER) | ENABLE_UART));	// Enable UART
		DelayNUsec(10000);
		printf("UART_DLRH WRV FAIL!\n");
		return(FAIL);
	}

	
	// *** Test UART_IER Register ***
	PUT32(UART_LCR, (GET32(UART_LCR) & 0x7F));	// Bit 7 = 0; DLAB = 0
	if(_Test_Register(UART_IER, ier) == ERROR)
	{
		PUT32(UART_IER, (GET32(UART_IER) | ENABLE_UART));	// Enable UART
		DelayNUsec(10000);
		printf("UART_IER WRV FAIL!\n");
		return(FAIL);
	}


	// *** Test UART_LCR Register ***
	if(_Test_Register(UART_LCR, lcr) == ERROR)
	{
		PUT32(UART_IER, (GET32(UART_IER) | ENABLE_UART));	// Enable UART
		DelayNUsec(10000);
		printf("UART_LCR WRV FAIL!\n");
		return(FAIL);
	}


	PUT32(UART_IER, (GET32(UART_IER) | ENABLE_UART));	// Enable UART
	DelayNUsec(10000);

	printf("UART Reg Test... ");

	if(status == OK)
	{
		printf("PASS\n");
		return(PASS);
	}
	else	// FAIL
	{
		printf("FAILED\n");
		return(FAIL);
	}
}

//
// Function: UART_Polling_Test
//
// Description: Tests UART in Non-FIFO Polling Mode
//
static void UART_Polling_Test(void)
{
	char string_test[16], stored_ier;
	int c, k;

	stored_ier = GET32(UART_IER);	// store original IER value
	PUT32(UART_FCR, 0);	// Disable Transmitter and Receiver FIFOs

	// Receiver Data Available Interrupt Disable
	// Transmit Data Request Interrupt Disable
	// Receiver Line Status Interrupt Disable
	// Receiver Time Out Interrupt Disable
	PUT32(UART_IER, ENABLE_UART);

	//printf("\n\rPlease enter \"test*uart\" as an input: ");
	printf("\n\rPlease enter \"t\" as an input: ");
	
	DelayNUsec(1000);

	//PUT32(UART_IER, 0x5<<4);	// Enable UART Unit and receiver time out interrupt
	//*(string_test) = getchar();
	//c = getchar();
	c = GetChar();

	printf("UART Non-FIFO Polling Test... ");
	//if(strcmp(string_test, "test*uart") == 0)
	//if(strcmp(c, "t") == 0))
	if (c == 't')
	{
		k = OK;
	}
	else
	{
		k = ERROR;
	}

	// Restore FCR and IER values
	PUT32(UART_FCR, 0x1);	// Re-enable FIFO mode
	PUT32(UART_IER, stored_ier);

	if (k == OK) {
        printf("uart diag pass\n");
    }
    else
    {
        printf("uart diag failed\n");
    }
   
	return;
}


//
// Function: UART_FIFO_Polling_Test
//
// Description: Tests UART in FIFO Polling Mode
//
static void UART_FIFO_Polling_Test(void)
{
	int stored_ier;
	char string_test[16];

	stored_ier = GET32(UART_IER);	// store original IER value
	// Enable Transmit and Receive FIFOs (bit 0)
	// Reset Receiver and Transmitter FIFOs (bits [2:1])
	PUT32(UART_FCR, 0x7);

	// Receiver Data Available Interrupt Disable
	// Transmit Data Request Interrupt Disable
	// Receiver Line Status Interrupt Disable
	// Receiver Time Out Interrupt Disable
	PUT32(UART_IER, ENABLE_UART);

	printf("\nPlease enter \"test*uart\" as an input: ");

	DelayNUsec(1000);
	*(string_test) = getchar();
 	printf("UART FIFO Polling Test... ");
	if(strcmp(string_test, "test*uart") == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL!\n");
		DelayNUsec(1000000);
	}

	// Restore FCR and IER
	PUT32(UART_FCR, 0x1);	// Re-enable FIFO mode
	PUT32(UART_IER, stored_ier);
}

//
// Function: UART_Int_Test
//
// Description: Test UART in Non-FIFO Interrupt Mode
//
static int UART_Int_Test(void)
{
	int counter, stored_ier, c, k;
	//int_flag = 0;
	//read_counter = 0;
	//write_counter = 0;

    //memset(&read_buffer,0, 128);
	//strcpy(write_buffer, "\n\rPlease enter \"test*uart\" as an input: ");

	stored_ier = GET32(UART_IER);	// store original IER value

	// Receiver Data Available Interrupt Enable
	// Transmit Data Request Interrupt Enable
	// Receiver Line Status Interrupt Enable
	// Receiver Time Out Interrupt Disable
	PUT32(UART_FCR, 0x0);	// Disable Transmitter and Receiver FIFOs
	//PUT32(UART_IER, 0x7 | ENABLE_UART);

	printf("\n\rPlease enter \"t\" as an input: ");
	DelayNUsec(1000);
	//printf("Please enter \"t\" as an input:");
	c = getchar();
	if (c == 't')
    {
		k = OK;
       //printf("\nNon-FIFO Interrupt mode test PASSED\n");
    }
    else
    {
		k = ERROR;
		//printf("\nNon-FIFO Interrupt mode test FAILED\n");
    }
	
	// Restore FCR and IER values
	PUT32(UART_IER, stored_ier);
	PUT32(UART_FCR, 0x7);	// Re-enable FIFO mode
	if (k == OK)
	{
    	printf("\nNon-FIFO Interrupt mode test PASSED\n");
		return(PASS);
	}
	else
	{
		printf("\nNon-FIFO Interrupt mode test FAILED\n");
		return(FAIL);
	}

}

//
// Function: UART_FIFO_Int_Test
//
// Description: Tests UART in FIFO Interrupt Mode
//
static int UART_FIFO_Int_Test(void)
{
	int stored_ier, counter, c;
	int_flag = 0;
	read_counter = 0;
	write_counter = 0;

    //memset(&read_buffer,0, 128);
	//strcpy(write_buffer, "\n\rPlease enter \"test*uart\" as an input: ");

	stored_ier = GET32(UART_IER);	// store original IER value
	// Enable Transmit and Receive FIFOs (bit 0)
	// Reset Receiver and Transmitter FIFOs (bits [2:1])
	// 00 = 1 byte in FIFO causes interrupt (bits [7:6])
	//PUT32(UART_FCR, 0x7);
	DelayNUsec(1000);

	//	PUT32(IRQ_ENABLE_SET_REG, UART_INT);	// Enable UART Interrupt

	// Receiver Data Available interrupt enabled (bit 0)
	// Transmit Data Request interrupt enabled (bit 1)
	// Receiver Line Status interrupt enabled (bit 2)
	// Receiver Time Out interrupt disabled (bit 4)
	//PUT32(UART_IER, 0x7 | ENABLE_UART);
	// As soon as the Transmit Data Request is enabled it will trigger
	// an interrupt to display "write buffer"
	printf("\n\rPlease enter \"t\" as an input: ");
	//printf("Please enter \"test*uart\" as an input: ");

	DelayNUsec(1000);
	c = getchar();

	// Restore FCR and IER
	PUT32(UART_FCR, 0x1);	// Re-enable FIFO mode
	PUT32(UART_IER, stored_ier);
	DelayNUsec(1000);

	if (c == 't')
    {
        printf("\nFIFO Interrupt Mode test PASSED\n");
		return(PASS);
    }
	else
	{
 		printf("\nFIFO Interrupt Mode test FAILED\n");
		return(FAIL);
	}

}

//
// Function: UART_Interrupt_Handler
//
// Description: Handles Interrupts generated by serial
//
void UART_Interrupt_Handler(void)
{
	int iir;
	volatile int temp;
	unsigned char temp_read;

	iir = GET32(UART_IIR);

	if((iir & IIR_FIFOES) == FIFO_MODE)
	{
		if((iir & 0xF) == 0x2)	// Transmit Interrupt
		{
			printf("%s", write_buffer);
			PUT32(UART_IER, (GET32(UART_IER) & ~(0x2)));
			int_flag = WRITE_INT;
		}

		else if((iir & 0xF) == 0x4)	// Receive Interrupt
		{
			do
			{
				temp_read = GET32(UART_RBR);
				PUT32(UART_THR, temp_read);
				
				// for debug: PUT32(ALPHANUM_DIS_DATA_REG + (read_counter%4), temp_read);

				if(temp_read == '\r')
				{
					PUT32(UART_IER, ENABLE_UART);	// Disable interrupt mode
					int_flag = READ_INT;
					PUT32(UART_THR, '\n');
					break;
				}
				read_buffer[read_counter++] = temp_read;
				
				temp = GET32(UART_LSR) & LSR_DR;
			} while(temp);
		}
	}
	else	// NON_FIFO_MODE
	{
		if((iir & 0xF) == 0x2)	// Transmit Interrupt
		{
			write_counter = 0;
			do
			{
				if( (GET32(UART_LSR) & (1<<6)) != 0)
					PUT32(UART_THR, write_buffer[write_counter++]);//printf("%c", write_buffer[write_counter++]);
			}while(write_buffer[write_counter] != '\0');
			PUT32(UART_IER, (GET32(UART_IER) & ~(0x2)));
			int_flag = WRITE_INT;
		}

		else if((iir & 0xF) == 0x4)	// Receive Interrupt
		{
			temp_read = GET32(UART_RBR);
		
			do
			{
				DelayNUsec(1000);
				read_buffer[read_counter] = temp_read;
				PUT32(UART_THR, read_buffer[read_counter]);
				read_counter++;
			
				do
				{
					temp = GET32(UART_LSR) & LSR_DR;
				} while(!temp);

				temp_read = GET32(UART_RBR);
			} while(temp_read != '\r');
			PUT32(UART_IER, ENABLE_UART);	// Disable Interrupt mode
			int_flag = READ_INT;
		}
	}
}


//
// Function: _Test_Register
//
// Description: Stores original register value, write-read-verify and then retore
//
static int _Test_Register(unsigned int reg_addr, unsigned int var)
{
	PUT32(UART_SPR, var);	// Store IER into Scratchpad register for temp

	PUT32(reg_addr, 1);		// Write a one to UART_IER
	var = GET32(reg_addr);	// Read back UART_IER
	if(var != 1)			// Check if IER is one
	{
		return ERROR;
	}

	PUT32(reg_addr, 0);		// Write a zero to UART_IER
	var = GET32(reg_addr);	// Read back UART_IER
	if(var != 0)			// Check if IER is zero
	{
		return ERROR;
	}

	var = GET32(UART_SPR);	// Read original IER value from Scratchpad Register
	PUT32(reg_addr, var);	// Write back the original IER

	return OK;
}

//
// Function: Charav
//
// Description: Checks if there's a char on the UART
//
int Charav(void)
{
	volatile int temp;

	temp = GET32(UART_LSR) & LSR_DR;

	return(temp);
}

//
// Function: GetChar
//
// Description: gets a char from the UART
//
char GetChar(void)
{
	while (!Charav());   // wait until character is available
	return (GET32(UART_RBR));
}


//
// Function: PutChar
//
// Description: Puts a char onto the UART
//
void PutChar(char c)
{
	while ((GET32(UART_LSR) & LSR_TEMT) == 0);
	PUT32(UART_THR, c);
	if (c == '\n')
		PUT32(UART_THR, '\r');
	
}


// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_UART,"uart",UART_Test,DIAGNOSTIC,"UARTTest","test UART display");
Switch("d", BOOLEAN32ARG, "Dump all UART readable registers", "");
Switch("r", BOOLEAN32ARG, "UART Register test", "");
//Switch("q", BOOLEAN32ARG, "UART Non-FIFO polling test", "");
Switch("j", BOOLEAN32ARG, "UART Non-FIFO interrupt test", "");
//Switch("p", BOOLEAN32ARG, "UART FIFO polling test", "");
Switch("i", BOOLEAN32ARG, "UART FIFO interrupt test", "");
Switch("a", BOOLEAN32ARG, "All UART tests", "");
CommandEnd();
