/* uc_load.c
 *
 *------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: dalsraja, March, 2002
 *  revisions: dalsraja, 1 May 2002 - incorporated code review comment changes
 * 
 * 
 * -----------------------------------------------------------------------------
 */
  
#include "uc_load.h"
#include "ixp2800Diag.h"
#include "ixp2800Msf.h"
#include "ixp2000pci.h"
#include "ixp2800Global.h"
#include "ixp2800Media.h"

extern int Led_Test;

//*****************************************************************************
// Function:	int GetUengBase(unsigned short Uengine)
// Description:	Gets the CSR base address of a specific microengine
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800, i.e. 0 - 7 and 16 - 23
// Outputs:		Returns the CSR base address of the microengine
//*****************************************************************************
int GetUengBase(unsigned short Uengine)
{
	if ((Uengine <= 7) || ((Uengine >= 16) && (Uengine <= 23)))
	{
		return UENG_CSR_BASE_ADDR(Uengine);
	}
	else
	{
		return UENG_INVALID;
	}
}


//************************************************************************
// Function:	int IsUengActive(unsigned short Uengine)
// Description:	Check the state of the microengine
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800 0 - 7 and 16 - 23
// Outputs:		Returns the state of the microengine
//************************************************************************
static int IsUengActive(unsigned short Uengine)
{
	if(GetUengBase(Uengine) == UENG_INVALID)
	{
		return(UENG_INVALID);
	}

	if((GET32(RESET_1_REG) & (1 << Uengine)) == 0)
	{
		return(UENG_NOT_RESET);
	}
	
	return(UENG_RESET);
}


//*************************************************************************
// Function:	int IsUengCtxEnabled(unsigned short Uengine)
// Description:	Check if the contects of the threads are enabled
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800 0 - 7 and 16 - 23
// Outputs:		Returns the state of the microengine
//*************************************************************************
static int IsUengCtxEnabled(unsigned short Uengine)	
{
	int ctx_enables_val, ctr, ueng_base;

	ueng_base = GetUengBase(Uengine);
	
	ctx_enables_val = ((GET32(ueng_base + CTX_ENABLES) & CTX_ENABLE_MASK) >> 8);
	
	if(ctx_enables_val != 0)			// if any of the ctx_enables bits are set
	{									// display a message stating which ctx is still active
		printf("Context ");
		
		for (ctr = 0; ctr < 8; ctr++)
		{
			if (((ctx_enables_val >> ctr) & 1) == 1)	// if the context is enabled, display the context number
			{
				printf("%d, ", ctr);
			}
		}

		printf("are active\n");

		return(UENG_ACTIVE);
	}

	return(UENG_INACTIVE);
}


//*************************************************************************
// Function:	void Reset_Uengine(unsigned short Uengine)
// Description:	Calling this function will assert a reset on the 
//				microengine specified
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800 0 - 7 and 16 - 23
// Outputs:		None
//*************************************************************************
static void Reset_Uengine(unsigned short Uengine)
{
	int reset_reg_val;

	reset_reg_val = (GET32(RESET_1_REG) & RESET_1_MASK) | (1 << Uengine);

	PUT32(RESET_1_REG, reset_reg_val);		// Assert the reset for the uengine
}


//*************************************************************************
// Function:	void ClearReset_Uengine(unsigned short Uengine)
// Description:	Calling this function will de-assert the reset on
//				the specified microengine
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800 0 - 7 and 16 - 23
// Outputs:		None
//*************************************************************************
void ClearReset_Uengine(unsigned short Uengine)
{
	int reset_reg_val;

	reset_reg_val = GET32(RESET_1_REG) & ((~(1 << Uengine)) & RESET_1_MASK);

	PUT32(RESET_1_REG, reset_reg_val);		// De-assert the reset for the unegine
}


// IXDP2400:Consider the whole function below to be new as the loading
//			of microcode to the microstore is different compared to IXP1200
//***********************************************************************************************************
// Function:	int DownLoadUcode(unsigned short Uengine, unsigned short SizeUcode, unsigned int *Ucode)
// Description:	Calling this function will load the microcode into the microstore of a specified microengine
// Inputs:		Uengine -> Microengine numbers IXP2400 0 - 4 and 16 - 19
// 				Uengine -> Microengine numbers IXP2800 0 - 7 and 16 - 23
//				SizeUcode	->	Bytesize of the microcode to be loaded
//				*Ucode		->	Pointer to the array holding the microcode data to be loaded to the ustore
// Outputs:		Return the status of the operation
//***********************************************************************************************************
int DownLoadUcode(unsigned short Uengine, unsigned short SizeUcode, unsigned int *Ucode)
{
	int status, Uaddr, ueng_base, i, Ucode_ele;

	Ucode_ele = SizeUcode / 8;	// Tot num of elements(rows) to write into the ustore
//	printf("Size of ucode: %d for Uengine %d\n", Ucode_ele, Uengine);

	ueng_base = GetUengBase(Uengine);
	
	//  Check if the specified microengine value is a valid microengine number
	if(ueng_base == UENG_INVALID)
	{
		printf("Microengine %d is not a valid microengine number\n", Uengine);

		printf("Valid microengine numbers for IXP2800 are 0-7 and 16-23\n");

		return(UENG_INVALID);
	}

	// Check the size of the microcode to be loaded into the microstore with the actual
	// size of the mircostore

	// Set the specified microengine to the Reset state by setting the respective
	// bit on the Reset_1 register
	Reset_Uengine(Uengine);

	DelayNUsec(25000);				// delay for 25 ms

	// Check if the microengine specified is in the reset mode
	status = IsUengActive(Uengine);
	if(status != UENG_RESET)
	{
		if (status == UENG_NOT_RESET)
		{
			printf("Microengine %d is not in reset state\n", Uengine);
		}

		return(status);
	}

	// Clear the reset bit of the respective microengine in the RESET_1 register
	ClearReset_Uengine(Uengine);

	DelayNUsec(25000);				// delay for 25 ms

	// Check if the contexts of the microengine specified have all been disabled
	status = IsUengCtxEnabled(Uengine);
	if(status != UENG_INACTIVE)
	{
		if (status == UENG_ACTIVE)
		{
			printf("Microengine %d contexts not disabled\n", Uengine);
		}

		return(status);
	}

	Uaddr =  ADDR_EN_BITPOS;				
	PUT32((ueng_base + USTORE_ADDR), Uaddr);	// Set bit 31 (ECS bit) of the USTORE_ADDRESS register
											// to indicate that microstore access will be performed

	*(volatile unsigned int *)(ueng_base + 0x154) = 0x400;  // Write 1 to Local CSR 0x154
	*(volatile unsigned int *)(ueng_base + 0x154) = 0;  // Write 0 to Local CSR 0x154
			
	for(i = 0; i < (Ucode_ele * 2); i+=2)		// for each microstore element to be written
	{
		PUT32((ueng_base + USTORE_DATA_L), Ucode[i]);		// Write the LSB 32 bit in first
		PUT32((ueng_base + USTORE_DATA_U), Ucode[i + 1]);	// Then write the MSB 8 bits
		// NOTE: After the USTORE_DATA_U register is written to, the pointer auto-increments and
		// points to the next location in the microstore.
		// NOTE: Each microstore instruction is 40 bits long, thus the LSB 32 bits and the MSB 8 bits
	}

	// For each microstore element, read the content of the microstore and verify against the original data
	for(i = 0; i < (Ucode_ele * 2); i+=2)
	{
		PUT32((ueng_base + USTORE_ADDR), (Uaddr | i>>1));
		if( (GET32(ueng_base + USTORE_DATA_L) != Ucode[i])
		||	(GET32(ueng_base + USTORE_DATA_U) != Ucode[i + 1]) )
		{
			printf("***Ustore error at space%d for Uengine %d\n", i>>1, Uengine);
			return(UENG_USTORE_ERROR);
		}
	}

	PUT32((ueng_base + USTORE_ADDR), 0);	// unset bit 31 of ustore address reg after completing writing to ustore

	for (i = 0; i < 8; i++)							// For each of the 8 threads/contexts
	{
		PUT32((ueng_base + CSR_CTX_POINTER), i);			// Selects which ctx is going to be indirecly referenced
		PUT32((ueng_base + CTX_STS_IND), 0);				// Set the ctx PC to 0
		PUT32((ueng_base + CTX_SIG_EVNT_IND), 0);			// Set the ctx signal event -- in CTX_SIG_EVENTS_INDIRECT
		PUT32((ueng_base + CTX_WAKEUP_EVNT_IND), 1);	// Set the ctx wakeup event to voluntary
	}

	PUT32((ueng_base + CC_ENABLE), CUR_COND_CODE_EN);		// Enable condition code in CC_ENABLE

	// Enable ctx 0 to 7 in CTX_ENABLES, clear CTL_STR_PAR_ER bit and BREAKPOINT bit
	PUT32((ueng_base + CTX_ENABLES), (CTX_ENABLE_MASK | (1 << 29) | (1 << 27)));

	return(UENG_SUCCESS);
}


void Clear_ME(unsigned short Uengine)
{
	Reset_Uengine(Uengine);
	DelayNUsec(25000);				// delay for 25 ms
	ClearReset_Uengine(Uengine);
}


void Clear_All_ME(void)
{
	Clear_ME(0);
	Clear_ME(1);
	Clear_ME(2);
	Clear_ME(3);
	Clear_ME(4);
	Clear_ME(5);
	Clear_ME(6);
	Clear_ME(7);
	Clear_ME(16);
	Clear_ME(17);
	Clear_ME(18);
	Clear_ME(19);
	Clear_ME(20);
	Clear_ME(21);
	Clear_ME(22);
	Clear_ME(23);
}
// Function:    void ClrAllUengWakeEvt(void)
// Description: Clears all wake events
// Input:		int ThdMask
// Outputs:		None
//*****************************************************************
void ClrAllUengWakeEvt(void)
{
	unsigned int Uengine, ctx;
	int ueng_base;

	for(Uengine=0; Uengine<=15; Uengine++)
	{
		ueng_base = GetUengBase(Uengine);

		//clear all events and status
		PUT32((ueng_base+USTORE_ADDR),  0x0);
		PUT32((ueng_base+CTX_ARB_CNTL), 0x0);


		for(ctx=0; ctx<=7; ctx++)
		{
			PUT32((ueng_base+CSR_CTX_POINTER), ctx);

			//these are indirect access by CSR_CTX_POINTER
			PUT32((ueng_base+CTX_STS_IND), 0x0);
			PUT32((ueng_base+ACTIVE_CTX_STS), 0x0);
			PUT32((ueng_base+CTX_WAKEUP_EVNT_IND), 0x0);
			PUT32((ueng_base+ACTIVE_CTX_WAKEUP_EVENTS), 0x0);
		}
	}
}

//*****************************************************************
// Function:    void SetAllUengWakeEvt(void)
// Description: Set wake events
// Input:		int ThdMask
// Outputs:		None
//*****************************************************************
void SetAllUengWakeEvt(void)
{
	unsigned int Uengine, ctx;
	int ueng_base;

	for(Uengine=0; Uengine<=15; Uengine++)
	{
		ueng_base = GetUengBase(Uengine);

		//clear all events and status
		PUT32((ueng_base+USTORE_ADDR),  0x0);
		PUT32((ueng_base+CTX_ARB_CNTL), 0x0);

		for(ctx=0; ctx<=7; ctx++)
		{
			PUT32((ueng_base+CSR_CTX_POINTER), ctx);

			//these are indirect access by CSR_CTX_POINTER
			PUT32((ueng_base+CTX_STS_IND), 0x0);
			PUT32((ueng_base+ACTIVE_CTX_STS), 0x0);
			PUT32((ueng_base+CTX_WAKEUP_EVNT_IND), 0x0);
			//set voluntary wake event
			PUT32((ueng_base+ACTIVE_CTX_WAKEUP_EVENTS), 0x1);
		}
	}
}



int UcodeControlStoreTest(void);
int UcodeControlStoreTest(void)
{
int error_cnt = 0;
	if (CtrlStoreTest(0) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(1) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(2) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(3) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(4) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(5) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(6) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(7) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(16) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(17) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(18) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(19) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(20) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(21) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(22) == FAIL)
		error_cnt = 1;
	if (CtrlStoreTest(23) == FAIL)
		error_cnt = 1;

	if (error_cnt)
		return(FAIL);
	else
		return(PASS);
}



int CtrlStoreTest(int);
int CtrlStoreTest(int Uengine)
{
    unsigned int data_low, data_high;

	int status, Uaddr, ueng_base, i, Ucode_ele;

	Ucode_ele = 0x1000;  // Total size of control store

	printf("\nTesting microengine 0x%x control store(size 0x%x) \n", Uengine, Ucode_ele);

	ueng_base = GetUengBase(Uengine);
	
	//printf("Microengine base = 0x%X\n", ueng_base);

	// Set the specified microengine to the Reset state by setting the respective
	// bit on the Reset_1 register
	Reset_Uengine(Uengine);

	DelayNUsec(25000);				// delay for 25 ms

	// Check if the microengine specified is in the reset mode
	status = IsUengActive(Uengine);
	if(status != UENG_RESET)
	{
		if (status == UENG_NOT_RESET)
		{
			printf("Microengine %d is not in reset state\n", Uengine);
		}


		return(status);
	}

	// Clear the reset bit of the respective microengine in the RESET_1 register
	ClearReset_Uengine(Uengine);

	DelayNUsec(25000);				// delay for 25 ms

	// Check if the contexts of the microengine specified have all been disabled
	status = IsUengCtxEnabled(Uengine);
	if(status != UENG_INACTIVE)
	{
		if (status == UENG_ACTIVE)
		{
			printf("Microengine %d contexts not disabled\n", Uengine);
		}

		return(status);
	}

	Uaddr =  ADDR_EN_BITPOS;				

	//printf("1st pass\n");

	*(volatile unsigned int *)(ueng_base + 0x154) = 0x400;  // Write 1 to Local CSR 0x154
	*(volatile unsigned int *)(ueng_base + 0x154) = 0;  // Write 0 to Local CSR 0x154
			
	*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr);	

	for(i = 0; i < Ucode_ele; i++)		// Zero all control store locations
	{

		*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0;	// Write the LSB 32 bit in first
		*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0;			// Then write the MSB 8 bits
    }

	*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr);	

	for(i = 0; i < Ucode_ele; i++)		// for each microstore element to be written
	{
		if(i%2)
		{
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0x55555555;	// Write the LSB 32 bit in first
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0x55;			// Then write the MSB 8 bits
		}
		else
		{
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0xAAAAAAAA;	// Write the LSB 32 bit in first
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0xAA;			// Then write the MSB 8 bits
		}

		// NOTE: After the USTORE_DATA_U register is written to, the pointer auto-increments and
		// points to the next location in the microstore.
		// NOTE: Each microstore instruction is 40 bits long, thus the LSB 32 bits and the MSB 8 bits
	}

	// For each microstore element, read the content of the microstore and verify against the original data
	for(i = 0; i < Ucode_ele; i++)
	{
		*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr + i);	

		if(i%2)
		{
		  data_low  = *(volatile unsigned int *)(ueng_base + USTORE_DATA_L) ;
		  data_high = *(volatile unsigned int *)(ueng_base + USTORE_DATA_U) ;


		  if ((data_low != 0x55555555) || (data_high != 0x55))		
			  {
			  printf("Failure at address 0x%X\n", i);
			  printf("Expected Data Low : 0x55555555 Read: 0x%8X at address 0x%X\n", data_low);
			  printf("Expected Data High: 0x55 Read: 0x%02X at address 0x%X\n", data_high);

			  return(FAIL);
			  }
		}
		else
		{
		  data_low  = *(volatile unsigned int *)(ueng_base + USTORE_DATA_L) ;
		  data_high = *(volatile unsigned int *)(ueng_base + USTORE_DATA_U) ;

		  if ((data_low != 0xAAAAAAAA) || (data_high != 0xAA))		
			  {
			  printf("Failure at address 0x%X\n", i);
			  printf("Expected Data Low : 0xAAAAAAAA Read: 0x%8X at address 0x%X\n", data_low);
			  printf("Expected Data High: 0xAA Read: 0x%02X at address 0x%X\n", data_high);

			  return(FAIL);
			  }
		}
	}
	//printf("2nd pass invert the data\n");

	*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr);
		
	for(i = 0; i < Ucode_ele; i++)		// Zero all control store locations
	{
		*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0;	// Write the LSB 32 bit in first
		*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0;			// Then write the MSB 8 bits
    }

	*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr);
		
	for(i = 0; i < Ucode_ele; i++)		// for each microstore element to be written
	{
		if(i%2)
		{
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0xAAAAAAAA;	// Write the LSB 32 bit in first
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0xAA;			// Then write the MSB 8 bits
		}
		else
		{
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_L) = 0x55555555;	// Write the LSB 32 bit in first
			*(volatile unsigned int *)(ueng_base + USTORE_DATA_U) = 0x55;			// Then write the MSB 8 bits
		}

		// NOTE: After the USTORE_DATA_U register is written to, the pointer auto-increments and
		// points to the next location in the microstore.
		// NOTE: Each microstore instruction is 40 bits long, thus the LSB 32 bits and the MSB 8 bits
	}

	// For each microstore element, read the content of the microstore and verify against the original data
	for(i = 0; i < Ucode_ele; i++)
	{
		*(volatile unsigned int *)(ueng_base + USTORE_ADDR) = (Uaddr + i);	

		if(i%2)
		{
		  data_low  = *(volatile unsigned int *)(ueng_base + USTORE_DATA_L) ;
		  data_high = *(volatile unsigned int *)(ueng_base + USTORE_DATA_U) ;

		  if ((data_low != 0xAAAAAAAA) || (data_high != 0xAA))		
			  {
			  printf("Failure at address 0x%04X\n", i);
			  printf("Expected Data Low : 0xAAAAAAAA Read: 0x%08X\n", data_low);
			  printf("Expected Data High: 0xAA Read: 0x%02X\n", data_high);

			  return(FAIL);
			  }
		}
		else
		{
		  data_low  = *(volatile unsigned int *)(ueng_base + USTORE_DATA_L) ;
		  data_high = *(volatile unsigned int *)(ueng_base + USTORE_DATA_U) ;

		  if ((data_low != 0x55555555) || (data_high != 0x55))		
			  {
			  printf("Failure at address 0x%04X\n", i);
			  printf("Expected Data Low : 0x55555555 Read: 0x%08X", data_low);
			  printf("Expected Data High: 0x55 Read: 0x%02X \n", data_high);

			  return(FAIL);
			  }
		}


	}

	printf("Passed Xscale control store test\n\n");
	return(PASS);
}

 









