/* xscale.c
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXDP2800
 *  subsystem: DIAG
 *  author: ccm, Jan 17, 02
 *
 *
 * Revisions:
 01b, 08/20/03 ajp Made xscale -a case be a total of any failures
 01a, 08/18/03 ajp Added pass/fail returns for script file break on error
 * 
 * --------------------------------------------------------------------------
 */

#include "vm.h"		   
#include "cmd.h"
#include "xscale.h"
#include "led.h"
#include "diag_utils.h"
#include "ixp2800IrqCtl.h"
#include "ixp2800Global.h"
#include "ixp2800Diag.h"
#include "ixp2x00.h"

HANDLE SOFTWAREISR;
HANDLE SOFTWAREFISR;

void XScale_Soft_Interrupt_Handler(void);


// User input data
typedef struct _XScale_testIn
{
	UINT32 controlstore_sel;
	UINT32 interrupt_sel;
	UINT32 all_sel;
} XScale_testIn;


extern int UcodeControlStoreTest(void);

static volatile int diag_soft_int_flag = 0;


//
// Function: Xscale_Test
//
// Description: Calls the appropriate function(s) of the XScale test
//				depending on user input.
//
int Xscale_Test(XScale_testIn* pIn, PassFail_testOut* pOut)
{
int pass1 = 0;
	if ((pIn->interrupt_sel) || (pIn->all_sel))
		pass1 = Xscale_Interrupt_Test();

	if ((pIn->controlstore_sel) || (pIn->all_sel))
		pOut->PassFail = UcodeControlStoreTest();

	pOut->ReturnCode = PASS;         
	pOut->PassFail = (pOut->PassFail | pass1);

	return(pOut->PassFail);


}

#define SOFTWAREIRQ (PIC_NUM_IRQ + 0)
#define SOFTWAREFIRQ (PIC_NUM_FIRQ + 0)

#define SOFTTEST	0x55AA
//HANDLE DOORBELLISR;

//
// Function: Xscale_Interrupt_Test
//
// Description: Test for both IRQ and FIQ soft interrupts
//
static int Xscale_Interrupt_Test(void)
{
	unsigned int delay;
	unsigned int error_cnt = 0;

	/* Install the Handler */
	SOFTWAREISR = InstallHandler (SOFTWAREIRQ, (HANDLER)XScale_Soft_Interrupt_Handler, (void *)SOFTTEST);

	/* Enable the IRQ for the doorbell ISR */
	EnableIRQ (SOFTWAREISR);

	SOFTWAREFISR = InstallHandler (SOFTWAREFIRQ, (HANDLER)XScale_Soft_Interrupt_Handler, (void *)SOFTTEST);

	/* Enable the IRQ for the doorbell ISR */
	EnableIRQ (SOFTWAREFISR);


	// Testing for IRQ Soft Interrupt

	delay = 0xFF;
	diag_soft_int_flag = 0;					// clear soft interrupt flag
	PUT32(FIQ_ENABLE_CLR_REG, SOFT_INT);	// disable FIQ soft interrupt
	PUT32(IRQ_ENABLE_SET_REG, SOFT_INT);	// enable IRQ soft interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, SOFT_INT);	// trigger soft interrupt

	while(delay--);
	printf("XSCALE IRQ TEST...");
	delay = 0xFF;
	while((diag_soft_int_flag == 0) && --delay);
	if(diag_soft_int_flag == IRQ_SOFT)
	{
		printf("PASSED\n");
	}
	else
	{
		printf("FAILED\n");
		error_cnt = 1;
	}


	// Testing for FIQ Soft Interrupt

	delay = 0xFF;
	diag_soft_int_flag = 0;					// clear soft interrupt flag
	PUT32(IRQ_ENABLE_CLR_REG, SOFT_INT);	// disable IRQ soft interrupt
	PUT32(FIQ_ENABLE_SET_REG, SOFT_INT);	// enable FIQ soft interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, SOFT_INT);	// trigger soft interrupt

	while(delay--);
	printf("XScale FIQ Test... ");
	delay = 0xFF;
	while((diag_soft_int_flag == 0) && --delay);
	if(diag_soft_int_flag == FIQ_SOFT)
	{
		printf("PASSED\n");
	}
	else
	{
		printf("FAILED\n");
		error_cnt = 1;
	}

	if (error_cnt)
		return(FAIL);
	else
		return(PASS);

}

//
// Function: XScale_Soft_Interrupt_Handler
//
// Description: Interrupt handler for soft interrupt
//
void XScale_Soft_Interrupt_Handler(void)
{
	if( (GET32(IRQ_STATUS_REG) & SOFT_INT) != 0)
	{
		diag_soft_int_flag |= IRQ_SOFT;
		PUT32(IRQ_ENABLE_CLR_REG, SOFT_INT);	// disable IRQ soft interrupt
	}

	if( (GET32(FIQ_STATUS_REG) & SOFT_INT) != 0)
	{
		diag_soft_int_flag |= FIQ_SOFT;
		PUT32(FIQ_ENABLE_CLR_REG, SOFT_INT);	// disable FIQ soft interrupt
	}	

	// Clear interrupt
	PUT32(FIQ_IRQ_SOFT_INT_REG, 0x0);	// clear soft interrupt
}

// Macro Format: Command(CMD_CommandName,"Command Name",Func_Name,Options,"Friendly Name","Help String");
Command(CMD_XSCALE,"xscale",Xscale_Test,DIAGNOSTIC,"XScaleTest","xscale functionality test");
Switch("c", BOOLEAN32ARG, "Xscale control store test", "");
Switch("i", BOOLEAN32ARG, "Xscale interrupt test", "");
Switch("a", BOOLEAN32ARG, "all tests", "");
CommandEnd();





