*****************************************************************************
*
* Module:	coreasm.s
*
* Description:
*
* Project:	HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:	See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*****************************************************************************
*=== Includes ====================
	#include "cpu.h"
	#include "cpucore.h"
	#include "pic.h"

*=== External References =========

*=== Global Declarations =========
.ifendian little
	.globl	ntohs, htons, ntohl, htonl
.endif
	.globl	memcpy, memset, memcmp
	.globl	strchr, strnchr
	.globl	CallOut, CallHandler, CallTimer
	.globl	GetVmmContext
	.globl	VleStackPointer, VleStackEnd
	.globl	MbrFnPtr, IsrFnPtr, EvtFnPtr, TmrFnPtr
	.weak	EnableInterrupts, DisableInterrupts
	.weak	EnterCriticalSection, ExitCriticalSection
	.globl	Noop

*=== Section =====================
	.section	".sbss"
VleStack:		.skip (SIZEOFVMMCONTEXT * 8)
VleStackEnd		= .
VleStackPointer:	.skip	4
CriticalStuff:		.skip	8

*=== Section =====================
	.text

.ifendian little
************************************************************************
* ntohs, htons
************************************************************************
* Called with:	a1 = value
* Returns:	a1 = byte-swapped value
* Prototype:	UINT16 htons(UINT16);
*		UINT16 ntohs(UINT16);
************************************************************************
ntohs:
htons:
	and	a2,a1,#0x0FF
	and	a1,a1,#0xFF00
	mov	a1,a1,LSR # 8
	orr	a1,a1,a2,LSL # 8
	mov	pc,lr

************************************************************************
* ntohl, htonl
************************************************************************
* Called with:	a1 = value
* Returns:	a1 = byte-swapped value
* Prototype:	UINT32 ntohl(UINT32);
*		UINT32 htonl(UINT32);
************************************************************************
ntohl:
htonl:
	eor	a2,a1,a1, ror #16
	bic	a2,a2,#0xFF0000
	mov	a1,a1,ror #8
	eor	a1,a1,a2,lsr #8
	mov	pc,lr
.endif

************************************************************************
* memcpy - copy data fom one place to another.
************************************************************************
* Called with:	a1 = destination pointer
*		a2 = source pointer
*		a3 = length (in bytes)
* Returns:	a1 = destination pointer
* Prototype:	void *memcpy(void *pDest, void *pSource, unsigned int length);
************************************************************************
memcpy:

	* Check to see if length is 0 bytes
	cmp	a3,#0
	beq	memcpyDone

	* If writing less than 16 bytes, just do bytes
	cmp	a3,#0x10
	blt	bytecopy

	* Are the addresses the same offset in a word?
	eor	a4,a1,a2
	ands	a4,a4,#3
	beq	alignword

	* Are the addresses half-word apart?
	ands	a4,a4,#1
	beq	alignhword

bytecopy:
	* Otherwise we can only do bytes
	ldrb	a4,[a2],#1
	strb	a4,[a1],#1
	subs	a3,a3,#1
	bne	bytecopy
	b	memcpyDone

alignhword:
	* Do single byte write to half-word align if necessary
	* (if LSB of address is 1)
	ands	a4,a2,#1
	beq	hwordcopy

	ldrb	a4,[a2],#1
	strb	a4,[a1],#1
	sub	a3,a3,#1

hwordcopy:
	ldrh	a4,[a2],#2
	strh	a4,[a1],#2
	sub	a3,a3,#2

	* Make sure we dont overwrite 1 too many
	cmp	a3,#1
	bgt	hwordcopy
	b	terminalbytes

alignword:
	ands	a4,a2,#3
	beq	wordcopy
	ldrb	a4,[a2],#1
	strb	a4,[a1],#1
	sub	a3,a3,#1
	b	alignword

wordcopy:
	ldr	a4,[a2],#4
	str	a4,[a1],#4
	sub	a3,a3,#4

	* Check to see if have last partial write
	cmp	a3,#3
	bgt	wordcopy

terminalbytes:
	* Copy the last partial write bytes in
	* (could be 0 passed in)
	cmp	a3,#0
	beq	memcpyDone
	ldrb	a4,[a2],#1
	strb	a4,[a1],#1
	sub	a3,a3,#1
	b	terminalbytes

memcpyDone:
	* Final address is already in a1, so just return
	mov	pc,lr

************************************************************************
* memset - fill a memory block with a particular byte.
************************************************************************
* Called with:	a1 = destination pointer
*		a2 = data byte
*		a3 = length (in bytes)
* Returns:	a1 = destination pointer
* Prototype:	void *memset(void *pDest, int c, unsigned int length);
************************************************************************
memset:
	* Check to see if length is 0 bytes
	cmp	a3,#0
	beq	memsetDone

	* If writing less than 8 bytes, just do bytes
	cmp	a3,#0x8
	blt	terminalbyteset

	* Doing more than 8 bytes, so setup a word value
	mov	a4,a2
	add	a2,a2,a4,LSL #8
	add	a2,a2,a4,LSL #16
	add	a2,a2,a4,LSL #24

	* Are we word aligned?
	ands	a4,a1,#3
	beq	memsetword

	* Do byte writes until we are aligned
	mov	r12,#4
	sub	a4,r12,a4	# a4 = how many writes to align

prealigned:
	strb	a2,[a1],#1
	sub	a3,a3,#1
	subs	a4,a4,#1
	bne	prealigned

	* Now we are word aligned
memsetword:
	str	a2,[a1],#4
	sub	a3,a3,#4

	* Check to see if have last partial write
	cmp	a3,#3
	bgt	memsetword

	* We have 0-3 final bytes to write
	cmp	a3,#0
	beq	memsetDone	# No more writes to do

	* Copy the last partial write bytes in
terminalbyteset:
	strb	a2,[a1],#1
	subs	a3,a3,#1
	bne	terminalbyteset

memsetDone:
	* Final address is already in a1, so just return
	mov	pc,lr

************************************************************************
* memcmp - compare two areas of memory
************************************************************************
* Called with:	a1 = pointer to first memory block
*		a2 = pointer to second memory block
*		a3 = number of bytes to compare
* Returns:	a1 = 0 (if same)
*	  	a1 = 1 (if block1 > block2)
*	  	a1 = -1 (if block1 < block2)
* Prototype:	int memcmp(const void *, const void *, unsigned int);
************************************************************************
memcmp:
	* We are comparing just bytes
	ldrb	a4, [a1],#1
	ldrb	r12,[a2],#1

	cmp	a4,r12
	bgt	larger
	blt	smaller

	* They are still equal so far
	subs	a3,a3,#1
	bgt	memcmp

	* They are identical, return
	mov	a1, #0
	b	memcmpdone

larger:
	* mem area 1 is greater than mem area 2, return
	mov	a1,#1
	b	memcmpdone

smaller:
	* mem area 1 is smaller than mem area 2, return
	mov	a1,#-1

memcmpdone:
	* Return
	mov 	pc,lr

************************************************************************
* strchr - scan a string for the first occurence of a character.
************************************************************************
* Called with:	a1 points to the string
*		a2 = search character
* Returns:	a1 = pointer to first occurence of chararacter in string
*		a1 = 0 if character is not in string
* Prototype:	char *strchr(const char *pString, int c);
************************************************************************
strchr:
	* always assuming that the string is null (zero) terminated
	ldrb	a3,[a1],#1
	cmp	a3,#0
	beq	nomatch
	cmp	a3,a2
	bne	strchr

ismatch:
	* Since a1 is post incremented, we need to decrement it by 1
	sub	a1,a1,#1
	b	strchrDone

nomatch:
	mov	a1,#0

strchrDone:
	mov	pc,lr

************************************************************************
* strnchr - scan a string for the first occurence of a character.
************************************************************************
* Called with:	a1 points to the string
*		a2 = search character
*		a3 = number of characters to search
* Returns:	a1 = pointer to first occurence of chararacter in string
*		a1 = 0 if character is not in string
* Prototype:	char *strnchr(const char *pString, int c, unsigned int length);
************************************************************************
strnchr:
	* always assuming that the string is null (zero) terminated

	* if length = 0 just exit
	cmp	a3,#0
	beq	nomatchn

nextcharn:
	ldrb	a4,[a1],#1
	cmp	a4,#0		# a null terminator is found
	beq	nomatchn
	cmp	a4,a2
	beq	ismatchn
	subs	a3,a3,#1
	bgt	nextcharn

nomatchn:
	mov	a1,#0
	b	strnchrDone

ismatchn:
	* since a1 was post incremented need to decrement 1
	sub	a1,a1,#1

strnchrDone:
	* Return
	mov	pc,lr

*****************************************************************************
* CallTimer - call a registered timer handler.  Install the context of the
* VMM which contains the handler before calling it.  Restore the VisionWARE
* context before returning.
*****************************************************************************
* Called with:	a1 = tTIMER *
* Returns:	void
* Prototype:	void CallTimer(tTIMER *pTimer);
*****************************************************************************
CallTimer:
	add	a4,a1,#0x28			# a4 points to a VMMCONTEXT structure
	ldr	a3,[a1,#0x08]			# a3 points to the timer handler routine
	ldr	a1,[a1,#0x0C]			# a1 points to the timer handler parameter
	b	CallOut

*****************************************************************************
* CallHandler - call a registered interrupt/exception handler.  Install the
* context of the VMM which contains the handler before calling it.  Restore
* the VisionWARE context before returning.
*****************************************************************************
* Called with:	a1 = tHANDLER *
* Returns:	void
* Prototype:	void CallHandler(tHANDLER *pHandler);
*****************************************************************************
CallHandler:
	add	a4,a1,#HandlerVmmContext	# a4 points to a VMMCONTEXT structure
	ldr	a3,[a1,#HandlerVector]		# a3 points to the handler routine
	ldr	a1,[a1,#HandlerParameter]	# a1 points to the handler parameter
* Falls through to CallOut

************************************************************************
* CallOut - call a registered function.  Install the context of the
* VMM which contains the function before calling the function.  Restore
* the VisionWARE context before returning.
************************************************************************
* Called with:	a1 = input structure pointer
*		a2 = output structure pointer
*		a3 = function pointer
*		a4 = VMMCONTEXT pointer
* Returns:	nothing
* Prototype:	void CallOut(void *pInput, void *pOutput,
*			     void (*pFunction)(void *, void *),
*			     VMMCONTEXT *pContext);
************************************************************************
CallOut:
	stmfd	sp!,{r9,lr}		# save ABI context on stack
	sub	sp,sp,#0x10		# ABIFIX
	ldr	r9,[a4,#VmmR9]		# install functions ABI registers
	mov	lr,pc			# save return address in LR
	mov	pc,a3			# call function
	add	sp,sp,#0x10		# ABIFIX
	ldmfd	sp!,{r9,pc}		# restore ABI and return

************************************************************************
* GetVmmContext - fill a VMMCONTEXT structure.  This routine is VMM-aware.
* That is, if this routine is called by from inside a VMM then the
* context is pulled from the VleStack, otherwise the current register
* values are used.
************************************************************************
* Called with:	A1 points to a VMMCONTEXT structure
* Returns:	nothing
* Prototype:	void GetVmmContext(VMMCONTEXT *);
************************************************************************
GetVmmContext:
	ldr	a2,=VleStackPointer@data
	add	a2,a2,r9		# a2 = address of VleStackPointer
	ldr	a3,[a2]			# a3 = VleStackPointer
	cmp	a3,a2
	bne	gvc0			# branch if VleStack not empty
	str	r9,[a1,#VmmR9]		# store current ABI registers
	mov	pc,lr

gvc0:	ldr	a2,[a3,#VmmR9]		# Use values pointed to by VleStackPointer
	str	a2,[a1,#VmmR9]
	mov	pc,lr

*************************************************************************
* EnterCriticalSection - if the count (at CriticalStuff) is zero then
* save the current interrupt flags (at CriticalStuff + 4) then disable
* interrupts.  Increment the count.
*************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void ExitCriticalSection(void);
*************************************************************************
EnterCriticalSection:
	ldr	a1,=CriticalStuff@data
	add	a1,a1,sb		# a1 points to CriticalStuff
	ldr	a2,[a1]
	cmp	a2,#0			# is count already non-zero?
	bne	AlreadyBlocked		# branch if yes
	add	a2,a2,#1		# increment count
	str	a2,[a1]
	mrs	a3,cpsr
	str	a3,[a1,#4]		# save current interrupt flags
	orr	a3,a3,#0xC0
	msr	cpsr_c,a3		# disable interrupts
	mov	pc,lr
AlreadyBlocked:
	add	a2,a2,#1		# increment count
	str	a2,[a1]
	mov	pc,lr

*************************************************************************
* ExitCriticalSection - if the count (at CriticalStuff) is one then
* restore the interrupt flags to their original value (saved at
* CriticalStuff + 4).  Decrement the count.
*************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void ExitCriticalSection(void);
*************************************************************************
ExitCriticalSection:
	ldr	a1,=CriticalStuff@data
	add	a1,a1,sb		# a1 points to CriticalStuff
	ldr	a2,[a1]
	subs	a2,a2,#1		# decrement the count
	bne	RemainBlocked		# branch if count is non-zero
	ldr	a3,[a1,#4]		# retrieve original flag settings
	msr	cpsr_c,a3		# restore original flag settings
RemainBlocked:
	str	a2,[a1]
	mov	pc,lr

*************************************************************************
* EnableInterrupts
*************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void EnableInterrupts(void);
*************************************************************************
EnableInterrupts:
	mrs	a1,cpsr
	and	a1,a1,#0x3F
	msr	cpsr_c,a1		# enable interrupts
	mov	pc,lr

*************************************************************************
* DisableInterrupts
*************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void DisableInterrupts(void);
*************************************************************************
DisableInterrupts:
	mrs	a1,cpsr
	orr	a1,a1,#0xC0
	msr	cpsr_c,a1		# disable interrupts
	mov	pc,lr

*************************************************************************
* MbrFnPtr, IsrFnPtr, EvtFnPtr, TmrFnPtr
*************************************************************************
* Called with:	r0 points to an object
*		r1 = voffset
*		r2 = index
*		r3 = vtbl-offset or function pointer
*************************************************************************
MbrFnPtr:
IsrFnPtr:
EvtFnPtr:
TmrFnPtr:
	cmp	r1,#0
	beq	ptr1			# branch if voffset == 0
	add	r1,r1,r0
	ldr	r1,[r1,#-1]
ptr1:	cmp	r2,#0
	bls	ptr2			# branch if index <= 0
	ldr	r3,[r3,r0]
	mov	r2,r2,lsl #2
	ldr	r0,[r3,r2]
	mov	pc,lr
ptr2:	mov	r0,r3
	mov	pc,lr

************************************************************************
* Noop
************************************************************************
* Called with:	nothing
* Returns:	nothing
* Prototype:	void Noop(void);
************************************************************************
Noop:
	mov	pc,lr

	.end
