*****************************************************************************
* Module:	estack.s
*
* Description:
*
* Project:	HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:	See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*****************************************************************************
*=== Includes ====================
	#include "cpu.h"
#ifdef TGTCPU_A940TLE
        #include "integratorMmr.h"
#endif
        
*=== External References =========
	.extern memcpy

*=== Global Declarations =========
	.globl _BufferCpy, _BufferByte, _BufferHalfword, _BufferWord
	.globl ComputeIPChecksum, ComputePseudoChecksum
	.globl ComputeIntermediateChecksum
	.globl SetBaseTime, SetElapsedTime, InitialSeqNumber
	.globl Milliseconds

*=== Section =====================
	.text

************************************************************************
* _BufferByte - copy one byte to a buffer and increment the buffer
* pointer.
************************************************************************
* Called with:	r0 = byte to buffer
*		r1 is a pointer to pointer to byte
* Returns:	nothing
* Prototype:	void _BufferByte(UINT8 byte, UINT **buffer);
************************************************************************
_BufferByte:
	ldr	r2,[r1]
	strb	r0,[r2]
	add	r2,r2,#1
	str	r2,[r1]
	mov	pc,lr

************************************************************************
* _BufferHalfword - copy two bytes to a buffer and increment the buffer
* pointer.
************************************************************************
* Called with:	r0 = halfword to buffer
*		r1 is a pointer to pointer to halfword
* Returns:	nothing
* Prototype:	void _BufferHalfword(UINT16 halfword, UINT16 **buffer);
************************************************************************
_BufferHalfword:
	ldr	r2,[r1]
	mov	r3,r0,lsr # 8
	strb	r3,[r2]
	strb	r0,[r2,#1]
	add	r2,r2,#2
	str	r2,[r1]
	mov	pc,lr

************************************************************************
* _BufferWord - copy four bytes to a buffer and increment the buffer
* pointer.
************************************************************************
* Called with:	r0 = word to buffer
*		r1 is a pointer to pointer to word
* Returns:	nothing
* Prototype:	void _BufferWord(UINT32 word, UINT32 **buffer);
************************************************************************
_BufferWord:
	ldr	r2,[r1]
	mov	r3,r0,lsr # 24
	strb	r3,[r2]
	mov	r3,r0,lsr # 16
	strb	r3,[r2,#1]
	mov	r3,r0,lsr # 8
	strb	r3,[r2,#2]
	strb	r0,[r2,#3]
	add	r2,r2,#4
	str	r2,[r1]
	mov	pc,lr

************************************************************************
* _BufferCpy - copy N bytes from one buffer to another and update the
* destination buffer pointer.
************************************************************************
* Called with:	r0 = source pointer
*		r1 = length (in bytes)
*		r2 is a pointer to pointer to void
* Returns:	nothing
* Prototype:	void _BufferCpy(void *pSource, unsigned int length,\
*			void **buffer);
************************************************************************
_BufferCpy:
	mov	r3,r2
	mov	r2,r1
	mov	r1,r0
	ldr	r0,[r3]
	add	r12,r0,r2
	str	r12,[r3]
	b	memcpy

************************************************************************
* ComputeIntermediateChecksum - compute an IP checksum but do not perform
* the final check for carries out of the lower 16 bits.  This function
* produces a number which can be used as a seed value for a further IP
* checksum computation.
************************************************************************
* Called with:	r0 = buffer pointer
*		r1 = number of bytes
*		r2 = seed value
*		r3 = non-zero if starting from an odd byte
* Returns:	r0 = IP checksum
* Prototype:	UINT32 ComputeIntermediateChecksum(void *p, UINT32 Length,
*			UINT32 Seed, UINT32 OddAddressFlag);
************************************************************************
ComputeIntermediateChecksum:
	cmp	r3,#0
	beq	cic1			# branch if not starting from odd byte
	ldrb	r3,[r0],#1		# load a byte and update buffer pointer
	add	r2,r2,r3		# add byte to seed value
	sub	r1,r1,#1		# subtract one from number of bytes
	b	cic1			# branch if no bytes left
cic0:	ldrb	r3,[r0],#1		# R3 = high byte
	ldrb	r12,[r0],#1		# R12 = low byte
	mov	r3,r3,lsl #8
	add	r3,r3,r12		# R3 = halfword
	add	r2,r2,r3		# add halfword to seed value
	sub	r1,r1,#2		# subtract two from number of bytes
cic1:	cmp	r1,#1
	bhi	cic0			# loop while two or more bytes left
	bne	cic2
	ldrb	r3,[r0],#1		# one byte left, read it, update pointer
	mov	r3,r3,lsl #8		# shift byte left 8 and add to seed value
	add	r2,r2,r3
cic2:	mov	r0,r2
	mov	pc,lr

************************************************************************
* ComputeIPChecksum - compute a final IP checksum.
************************************************************************
* Called with:	r0 = buffer pointer
*		r1 = number of bytes
*		r2 = seed value
*		r3 = non-zero if starting from an odd byte
* Returns:	r0 = IP checksum
* Prototype:	UINT32 ComputeIPChecksum(void *p, UINT32 Length,
*			UINT32 Seed, UINT32 OddAddressFlag);
************************************************************************
ComputeIPChecksum:
	stmfd	sp!,{lr}
	bl	ComputeIntermediateChecksum
	ldr	r2,=0xFFFF
	movs	r1,r0,lsr #16
	beq	cipc0
	and	r0,r0,r2
	add	r0,r0,r1
	movs	r1,r0,lsr #16
	beq	cipc0
	and	r0,r0,r2
	add	r0,r0,r1
cipc0:	eor	r0,r0,r2
	ldmfd	sp!,{pc}

************************************************************************
* ComputePseudoChecksum - compute the pseudo checksum common to many
* IP protocols.
************************************************************************
* Called with:	r0 = source IP address
*		r1 = destination IP address
*		r2 = protocol
*		r3 = length
* Returns:	r0 = pseudo checksum
* Prototype:	UINT32 ComputePseudoChecksum(IPADDR, IPADDR, UINT8, UINT16);
************************************************************************
ComputePseudoChecksum:
	add	r2,r2,r3
	ldr	r3,=0xFFFF
	mov	r12,r0,lsr #16
	and	r0,r0,r3
	add	r0,r0,r12
	add	r0,r0,r2
	mov	r12,r1,lsr #16
	and	r1,r1,r3
	add	r0,r0,r12
	add	r0,r0,r1
	mov	pc,lr

************************************************************************
* InitialSeqNumber - returns a "random" 32-bit value by reading the
* low-order word of the timebase register.
************************************************************************
* Called with:	nothing
* Returns:	r0 = 32-bit initial sequence number
* Prototype:	UINT32 InitialSeqNumber(void);
************************************************************************
InitialSeqNumber:
#undef INITSEQNUMBER

#ifdef TGTCPU_ARM7TDMI
	ldr	r0,=TCNT0
	ldr	r0,[r0]
#define INITSEQNUMBER	1
#endif

#ifdef TGTCPU_A940TLE
	ldr	r0,=RTC_DR
	ldr	r0,[r0]
#define INITSEQNUMBER	1
#endif

#ifdef TGTCPU_80200LE
	mrc	p14,0,r0,c1,c0,0	# read CCNT
	mvn	r0,r0
#define INITSEQNUMBER	1
#endif

#ifdef TGTCPU_80200BE
	mrc	p14,0,r0,c1,c0,0	# read CCNT
	mvn	r0,r0
#define INITSEQNUMBER	1
#endif

#ifdef TGTCPU_IXP2800
	mrc	p14,0,r0,c1,c0,0	# read CCNT
	mvn	r0,r0
#define INITSEQNUMBER	1
#endif

#ifndef INITSEQNUMBER
#error InitialSeqNumber not defined for this CPU
#endif
	mov	pc,lr

************************************************************************
* SetBaseTime
************************************************************************
* Called with:	a0 points to a 64-bit unsigned variable
* Returns:	nothing
* Prototype:	void SetBaseTime(UINT64 *pBaseTime);
************************************************************************
SetBaseTime:
	mov	pc,lr

************************************************************************
* SetElapsedTime
************************************************************************
* Called with:	a0 points to a 64-bit unsigned basetime variable
*		a1 points to a 64-bit unsigned elapsed time variable
* Returns:	nothing
* Prototype:	void SetElapsedTime(UINT64 *pBaseTime, UINT64 *pElapsedTime);
************************************************************************
SetElapsedTime:
	mov	pc,lr

************************************************************************
* Milliseconds - convert elapsed time to milliseconds.  This is based on
* a clock tick being equal to 2^-23.  a0 points to a 64-bit number but
* only the low-order 32 bits are used.  With a clock tick of 2^-23 this
* allows for a maximum of 512 seconds.
************************************************************************
* Called with:	a0 points to 64-bit time variable.
* Returns:	a1 = 32-bit millisecond count
* Prototype:	UINT32 Milliseconds(UINT64 *pElapsedTime);
************************************************************************
Milliseconds:
	mov	pc,lr

	.end