/******************************************************************************
*
* Module:		cpu.c
*
* Description:
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"

//  External References
extern void InitVectors(void);

#ifdef YAVAPAI_CHIPSET
//  Global Declarations
tYAVAPAI YAVAPAI;
#endif

const char *UnhandledExceptions[] =
{
	"Reset",							// 0x00
	"Undefined opcode",					// 0x01
	"Software interrupt",				// 0x02
	"Prefetch abort",					// 0x03
	"Data abort",						// 0x04
	"Reserved (0x14)",					// 0x05
	"IRQ",								// 0x06
	"FIQ",								// 0x07
};

/***************************************************************************
* Function: CpuInit(void)
*
* Purpose: 	cpu-specific initializatioon
*
* Returns:  nothing
****************************************************************************/
void CpuInit(void)
{
	InitVectors();
}

Init(12, CpuInit);

/***************************************************************************
* Function: CpuLaunch(tOsParams *pOS)
*
* Purpose: 	cpu-specific functionality performed prior to booting an OS
*
* Returns:  nothing
****************************************************************************/
void CpuLaunch(tOsParams *pOS)
{
	// Save structure pointer in R0 so that it will be passed to the OS as
	// the first parameter.
	UserContext.R0 = (UINT32)pOS;

	// Put a key value in R1 in order to validate the value in R0.
	UserContext.R1 = 0x1BADC0DE;

	UserContext.R0 = 0;
	UserContext.R1 = 243;
	// Disable interrupt's in the user context so that the OS can initialize
	// without being interrupted.
	UserContext.CPSR = 0xD3;
}

Fini(0x8000, CpuLaunch);

/***************************************************************************
* Function:	UnhandledException(UINT32 ID);
*
* Purpose:	report an unhandled exception
*
* Returns:	nothing
****************************************************************************/
void UnhandledException(UINT32 ID)
{
	StopReason = ID;
	ProcessStopReason();
}

/***************************************************************************
* Function: _InstallVector(UINT32 ID, CONTEXT *pContext, HANDLER Vector,
*						   void *pPrivateData);
*
* Purpose:  Install an exception or interrupt vector handler.
*
* Notes:	The computation of the vector address is based on the way the
*			vectors are initially setup by InitVectors in cpuasm.s.  Each
*			vector is given 24 bytes of code and data beginning at address
*			0x20.  The layout of these 24 bytes looks like this:
*				offset 0x00: code
*				offset 0x04: code
*				offset 0x08: address of structure where context will be saved
*				offset 0x0C: pointer to vector which will handle the exception or IRQ
*				offset 0x10: temporary storage for r0
*				offset 0x14: storage for parameter which will be passed to vector
*
* Returns:	nothing
****************************************************************************/
void _InstallVector(UINT32 ID, CONTEXT *pContext, HANDLER Vector,
		void *pPrivateData)
{
	universal u;

	u.uvalue = (ID * 24) + RAM1START + 0x20;

	EnterCriticalSection();
	u.pWord[2] = (UINT32)pContext;
	u.pWord[3] = (UINT32)Vector;
	u.pWord[5] = (UINT32)pPrivateData;
	ExitCriticalSection();
}

/***************************************************************************
* Function: InstallVector(UINT32 ID, HANDLER Vector, void *pPrivateData);
*
* Purpose:  Install an exception or interrupt vector handler.
*
* Returns:	nothing
****************************************************************************/
void InstallVector(UINT32 ID, HANDLER Vector, void *pPrivateData)
{
	_InstallVector(ID - FIRSTVECTORID, &TempContext, Vector, pPrivateData);
}

/***************************************************************************
* Function: RemoveVector(UINT32 ID);
*
* Purpose:  Remove a registered exception handler.  Replace it with the
*			UnhandledException function.
*
* Returns:	nothing
****************************************************************************/
void RemoveVector(UINT32 ID)
{
	_InstallVector(ID - FIRSTVECTORID, &TempContext,
			(HANDLER)UnhandledException, (void *)(ID - FIRSTVECTORID));
}

