/****************************************************************************
*
* Module:		cpucore.h
*
* Description:
*
* Project:		HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
****************************************************************************/
#ifndef __CPUCORE_H__
#define __CPUCORE_H__

#include "core.h"

/****************************************************************************
 * CONTEXT structure definition.  Contains all registers needed to
 * define an entire process context.
 ***************************************************************************/
#ifndef __ASSEMBLER__
typedef struct CONTEXT
{
	UINT32 R0;
	UINT32 R1;
	UINT32 R2;
	UINT32 R3;
	UINT32 R4;
	UINT32 R5;
	UINT32 R6;
	UINT32 R7;
	UINT32 R8;
	UINT32 R9;
	UINT32 R10;
	UINT32 R11;
	UINT32 R12;
	UINT32 R13;
	UINT32 R14;
	UINT32 PC;
	UINT32 CPSR;
} CONTEXT;
#else
#define	ContextR0	0x00
#define	ContextA1	0x00
#define ContextR1	0x04
#define ContextA2	0x04
#define ContextR2	0x08
#define ContextA3	0x08
#define ContextR3	0x0C
#define ContextA4	0x0C
#define ContextR4	0x10
#define ContextV1	0x10
#define ContextR5	0x14
#define ContextV2	0x14
#define ContextR6	0x18
#define ContextV3	0x18
#define ContextR7	0x1C
#define ContextV4	0x1C
#define ContextWR	0x1C
#define ContextR8	0x20
#define ContextV5	0x20
#define ContextR9	0x24
#define ContextV6	0x24
#define ContextSB	0x24
#define ContextR10	0x28
#define ContextV7	0x28
#define ContextSL	0x28
#define ContextR11	0x2C
#define ContextV8	0x2C
#define ContextFP	0x2C
#define ContextR12	0x30
#define ContextIP	0x30
#define ContextR13	0x34
#define ContextSP	0x34
#define ContextR14	0x38
#define ContextLR	0x38
#define ContextR15	0x3C
#define ContextPC	0x3C
#define ContextCPSR	0x40
#endif

/****************************************************************************
 * jmp_buf structure definition.  Contains all non-volatile registers as
 * defined by the ABI (or the compiler).
 ***************************************************************************/
#ifndef __ASSEMBLER__
typedef struct jmp_buf
{
	UINT32 R4;
	UINT32 R5;
	UINT32 R6;
	UINT32 R7;
	UINT32 R8;
	UINT32 R9;
	UINT32 R10;
	UINT32 R11;
	UINT32 R13;
	UINT32 R14;
	UINT32 CPSR;
} jmp_buf;
#else
#define JumpR4		0x00
#define JumpV1		0x00
#define JumpR5		0x04
#define JumpV2		0x04
#define JumpR6		0x08
#define JumpV3		0x08
#define JumpR7		0x0C
#define JumpV4		0x0C
#define JumpR8		0x10
#define JumpV5		0x10
#define JumpR9		0x14
#define JumpV6		0x14
#define JumpR10		0x18
#define JumpV7		0x18
#define JumpR11		0x1C
#define JumpV8		0x1C
#define JumpR13		0x20
#define JumpSP		0x20
#define JumpR14		0x24
#define JumpLR		0x24
#define JumpCPSR	0x28
#endif

#endif									// #ifndef __CPUCORE_H__
