*****************************************************************************
*
* Module:	entry.s
*
* Description:	warm entry point
*
* Project:	HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:	See Visual SourceSafe History
*
* Disclaimer:	Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*****************************************************************************
*=== Includes ====================
	#include "cpucore.h"
	#include "board.h"

*=== External References =========
	.extern	main
	.extern	SystemInit
	.extern	DataInit

*=== Global Declarations =========
	.globl	_WarmStart
	.globl	__init, __fini
	.globl	gDataFixup, gCodeFixup, gHeapStart, gHeapSize
	.globl	gDataStart, gCodeStart
	.globl	ClickStruct
	.globl	pMACAddr, _StartOfText

*=== Data Allocations ============
* These variables must be linked so that they appear first in the data segment.
* Do not change the order of these variables or insert other variables between
* them.
	.section	".data"
ClickStruct:	.skip	0x20
* Structure of ClickStruct is:
* 0x00 - address of BdmEntry function
* 0x04 - address of InputStructure
* 0x08 - address of OutputStructure
* 0x0C - address of OutputStructureNumber
* 0x10 - address of Ve

oDataStart	=	0x20
gDataStart:	.skip	4
oDataFixup	=	0x24
gDataFixup:	.skip	4
oCodeStart	=	0x28
gCodeStart:	.skip	4
oCodeFixup	=	0x2C
gCodeFixup:	.skip	4
oHeapStart	=	0x30
gHeapStart:	.skip	4
oHeapSize	=	0x34
gHeapSize:	.skip	4

oDataSize	=	0x1C		# relative to the start of code

*=== Section =====================
	.text

_StartOfText:

*************************************************************************
* _vWAREImageStruct - this structure must follow the structure of
* vWAREImageStruct as defined in boot.c.
*************************************************************************
_vWAREImageStruct:
	.ascii	"MAC HERE"
pMACAddr:
	.uhalf	MAC_ADDR_A, MAC_ADDR_B, MAC_ADDR_C
	.ascii	"OK"
	.long	__COPY_SIZE		# defined by the linker
	.long	__CODE_SIZE		# defined by the linker
	.long	__MINDATA_SIZE		# defined by the linker
	.long	__DATA_SIZE		# defined by the linker

*************************************************************************
* _WarmStart - enter here from boot.c or begin here for a RAM build.  This
* code must follow immediately after the vWAREImageStruct defined in
* header.s
*************************************************************************
* Called with:	a1 = address of data section
*		a2 = size of data and heap
* Returns:	does not return
* Destroys:	N/A
**************************************************************************
_WarmStart:
#ifdef TGTBUILD_RAM
	ldr	a1,=__LAZY_DATA_START
	ldr	a2,=WORKSPACESIZE
	ldr	a3,=__CODE_SIZE
	sub	a2,a2,a3
	add	sp,a1,a2
	b	LazyStart
	.literals
LazyStart:
#endif

* Save user registers on the stack.  Only need to save those registers
* defined as non-volatile by the CPU ABI.
	mrs	r12,cpsr
	stmfd	sp!,{r0-r12,r14}	# save r0-r12 and r14 on stack

* Disable interrupts.
	and	r12,r12,#0x3F
	msr	cpsr_c,r12

* Save address of gDataStart.
	str	a1,[a1,#oDataStart]

* Compute gCodeStart as runtime address (possibly relocated) of _StartOfText.
	bl	CodeStart
CodeStart:
	sub	v1,lr,#(CodeStart - _StartOfText)
	str	v1,[a1,#oCodeStart]	# save gCodeStart

* Compute gCodeFixup as gCodeStart - __ROM_BUILD_ADDR.
 	ldr	v2,=__ROM_BUILD_ADDR
	sub	v2,v1,v2		# v2 = gCodeFixup
	str	v2,[a1,#oCodeFixup]	# save gCodeFixup

* Compute gDataFixup as runtime address of data section - compiled address of
* data section.
	ldr	v3,=__RAM_BUILD_ADDR
	sub	v3,a1,v3		# v3 = gDataFixup
	str	v3,[a1,#oDataFixup]	# save gDataFixup

* Compute gHeapStart as start of data section + size of data segments.
	ldr	v3,[v1,#oDataSize]	# v3 = size of data segments
	add	a3,a1,v3		# a3 = start address of heap
	str	a3,[a1,#oHeapStart]	# save gHeapStart

* Compute gHeapSize as size of data and heap - size of data segments.
	sub	a3,a2,v3		# a3 = size of heap/stack
	str	a3,[a1,#oHeapSize]	# save gHeapSize

* Switch to the VisionWARE stack, saving the user stack pointer for later
* retrieval.  VisionWARE stack pointer is computed as address of data section
* + size of data and heap.
	mov	v4,sp
	add	sp,a1,a2

* Setup ABI-defined base registers.
	mov	sb,a1

* Call DataInit to perform pre-constructor initialization.
	bl	DataInit

* Setup all necessary vWAREContext registers.
	ldr	a1,=vWAREContext@data
	add	a1,a1,sb

	str	sp,[a1,#ContextSP]	# set up vWAREContext stack pointer

	mov	a2,#0x13		# set up vWAREContext status register
	ldr	a4,[v4,#0x0C]		# get fourth argument passed to us
	cmp	a4,#0			# if argument is zero then disable ..
	orreq	a2,a2,#0xC0		# interrupts in the status register
	str	a2,[a1,#ContextCPSR]

	str	r9,[a1,#ContextR9]	# setup vWAREContext ABI registers

	msr	cpsr_all,a2

* Call __init to invoke all object constructors.
	bl	__init

* Call SystemInit to perform post-constructor initialization.
	bl	SystemInit

* Enter VisionWARE.
	b	main

*=== Section =====================
* This section will be linked first in the .init* group.
	.section	.init$00,,rx

__init:
	stmfd	sp!,{lr}

*=== Section =====================
* This section will be linked last in the .init* group.
	.section	.init$99,,rx

	ldmfd	sp!,{pc}

*=== Section =====================
* This section will be linked first in the .fini* group.
	.section	.fini$00,,rx

__fini:
	stmfd	sp!,{lr}

*=== Section =====================
* This section will be linked last in the .fini* group.
	.section	.fini$99,,rx

	ldmfd	sp!,{pc}

	.end
