
/* irq.c - Interrupt Request */

/* Copyright 1984-1999 Wind River Systems, Inc. */

/*
modification history
--------------------
01a,14jan02, pgp  updated standards
*/

/*
DESCRIPTION

Virtual PIC for ARM4 interrupts

INCLUDE FILES:  vm.h
*/

/* includes */
#include "vm.h"

/* defines */

/* locals */
static HLIST IrqHandles		= { 0, 0, sizeof(tHANDLER) };
static HLIST FirqHandles	= { 0, 0, sizeof(tHANDLER) };

/* forward prototypes */


/***************************************************************************
*
* IRQ - Interrupt Service Request
*
* Service an interrupt request
*
* RETURNS: 1 if interrupt was recognized, 0 otherwise
*
* ERRNO: N/A
*/   

static int IRQ
    (
    tHANDLER *pHandler
    )
    {
	while (pHandler = pHandler->pNext)
	    {
		if (CallHandler (pHandler))
			return 1;
	    }

	return 0;
    }


/***************************************************************************
*
* PicInitIrq - Init IRQ
*
* Initialize the IRQ de-multiplexer
*
* RETURNS: HANDLE
*
* ERRNO: N/A
*/   

void PicInitIrq (void)
    {
	InstallVector (VECTOR_IRQ, (HANDLER)IRQ, (void *)&IrqHandles);
    }

Init (13, PicInitIrq);


/* Declare the PIC structure.  This must be a global declaration. */
tPIC PicIRQ =
    {
	CPUIRQBASE,
	CPUIRQBASE + 63,
	&IrqHandles,
	(void (*)(tHANDLER *, UINT32))Noop,
	(void (*)(tHANDLER *))Noop,
	(void (*)(tHANDLER *))Noop,
	(void (*)(tHANDLER *))Noop
    };

PIC (PicIRQ);


/***************************************************************************
*
* PicInitFirq - Init FIRQ
*
* Initialize the FIRQ de-multiplexer
*
* RETURNS: HANDLE
*
* ERRNO: N/A
*/   

void PicInitFirq (void)
    {
	InstallVector (VECTOR_FIRQ, (HANDLER)IRQ, (void *)&FirqHandles);
    }

Init(13, PicInitFirq);


/* Declare the PIC structure.  This must be a global declaration. */
tPIC PicFIRQ =
    {
	CPUIRQBASE + 64,
	CPUIRQBASE + 127,
	&FirqHandles,
	(void (*)(tHANDLER *, UINT32))Noop,
	(void (*)(tHANDLER *))Noop,
	(void (*)(tHANDLER *))Noop,
	(void (*)(tHANDLER *))Noop
    };

PIC (PicFIRQ);
