/******************************************************************************
*
* Module: vdmemblkcmp.c
*
* Description: Compare two memory blocks for the same data
*
* Project:     HSI VisionWare -- vWare
*
* Functions:	None
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

//  Includes
#include "vm.h"
#include "cmd.h"
#include "vderror.h"

//  Local Defines
#define Pass 0
#define Fail 1
#define ExecutionFail 0xffffffff

typedef struct _memblkcmpIn
{
	UINT32  lWidth;				// Size Suffix
	UINT32	lStartAddressOne;	// Enter Start address of block
	UINT32	lStartAddressTwo;	// Enter Start of block to compare to
	UINT32	lNumberOfElements;	// Enter the number of elements to comare
} memblkcmpIn;

typedef struct _memblkcmpOut
{
	UINT32 lPassFail;
	UINT32 lReturnCode;
	char   cResultString[128];
} memblkcmpOut;


UINT32 memblkcmp(memblkcmpIn* pIn,memblkcmpOut* pOut)
{

	UINT32 lCount;
	universal p,q;

	p.uvalue = pIn->lStartAddressOne;
	q.uvalue = pIn->lStartAddressTwo;


	pOut->lPassFail = Pass;

	//see if the size was entered, if not, default to 4
	if(pIn->lWidth !=0x4)
	{
		if(pIn->lWidth !=0x2)
		{
			if(pIn->lWidth !=0x1)
			{
				pIn->lWidth = 0x4;
			}
		}
	}


	//Check allignment of End Address
    if(pIn->lWidth!=0x1)
	{
		if((pIn->lStartAddressOne % pIn->lWidth!=0x0)||(pIn->lStartAddressTwo % pIn->lWidth!=0x0))
		{
			pOut->lPassFail = Fail;
			pOut->lReturnCode = Fail;
			LogError(ERR_MEMFILL, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
				"ERROR: MEMFILL, END ADDRESS ALLIGNMENT ERROR\n");
			strcpy(pOut->cResultString, "ERROR: MEMFILL, END ADDRESS ALLIGNMENT ERROR\n");
			return(pOut->lPassFail);
		}
	}


	//Perform validate function based on width
	switch (pIn->lWidth)
	{
		case 1: for(lCount = 1; lCount <= pIn->lNumberOfElements; lCount++, p.pByte++, q.pByte++)
				{
					if((UINT8)*p.pByte != (UINT8)*q.pByte)
					{
						pOut->lPassFail = Fail;
						pOut->lReturnCode = Fail;
						LogError(ERR_MEMFILL, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
							"ERROR: BLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pByte,*p.pByte, *q.pByte);
						sprintf(pOut->cResultString, "ERROR: MEMBLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pByte,*p.pByte, *q.pByte);
						lCount = pIn->lNumberOfElements;
					
					}
				}
				break;

		case 2: for(lCount = 1; lCount <= pIn->lNumberOfElements; lCount++, p.pHalf++, q.pHalf++)
				{
					if((UINT16)*p.pHalf != (UINT16)*q.pHalf)
					{
						pOut->lPassFail = Fail;
						pOut->lReturnCode = Fail;
						LogError(ERR_MEMBLKCMP, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
							"ERROR: MEMBLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pHalf,*p.pHalf, *q.pHalf);
					    sprintf(pOut->cResultString, "ERROR: MEMBLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pHalf,*p.pHalf, *q.pHalf);
						lCount = pIn->lNumberOfElements;
					
					}
				}
				break;

		case 4: for(lCount = 1; lCount <= pIn->lNumberOfElements; lCount++, p.pWord++, q.pWord++)
				{
					if((UINT32)*p.pWord != (UINT32)*q.pWord)
					{
						pOut->lPassFail = Fail;
						pOut->lReturnCode = Fail;
						LogError(ERR_MEMBLKCMP, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
							"ERROR: MEMBLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pWord,*p.pWord, *q.pWord);
						sprintf(pOut->cResultString, "ERROR: MEMBLKCMP, DATA VERIFY ERROR AT 0x%X, EXPECTED 0x%X, READ 0x%X\n", q.pWord,*p.pWord, *q.pWord);
						lCount = pIn->lNumberOfElements;
					
					}
				}
				break;
		}

	if(pOut->lPassFail == Pass)
	{
		pOut->lPassFail = Pass;
		pOut->lReturnCode = Pass;
		Info("Success, memblkcmp of 0x%X ... 0x%X and 0x%X ... 0x%X Passed\n", 
			pIn->lStartAddressOne,(pIn->lStartAddressOne+(pIn->lWidth * pIn->lNumberOfElements)),
			pIn->lStartAddressTwo,(pIn->lStartAddressTwo+(pIn->lWidth * pIn->lNumberOfElements)));
	}
						
	return(pOut->lPassFail);

	printf("This is the blkcmp diagnostic\n");
}

Command(CMD_MEMBLKCMP,"memblkcmp",memblkcmp,SIZESUFFIX,"memblkcmp","compare two blocks of memory");
Arg(UINT32ARG,"enter start address of first block","enter start address of first block");
Arg(UINT32ARG,"enter start address of block to compare to","enter Start of block to compare");
Arg(UINT32ARG,"enter the number of elements to compare","enter the number of elements to compare");
CommandEnd();

