/******************************************************************************
*
* Module: vdmemcopy.c
*
* Description: Copy memory contents from one location to another and
*			   verify.
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

//  Includes
#include "vm.h"
#include "esttype.h" 
#include "cmd.h"
#include "lib.h"
#include "vware.h"
#include "vderror.h"

//  Local Defines
#define Pass 0
#define Fail 1

//  Local Declarations (statics)
typedef struct _memcopyIn
{
	UINT32  lSizeSuffix;
	UINT32	lStartAddr1;
	UINT32	lStartAddr2;	
	UINT32	lSize;			
} memcopyIn;


typedef struct _memcopyOut
{
	UINT32 lPassFail;
	UINT32 lReturnCode;
	char   cResultString[128];
} memcopyOut;


UINT32 memcopy(memcopyIn* pIn,memcopyOut* pOut)
{
	UINT32  lCount;
	universal p, r;
	
	p.uvalue = pIn->lStartAddr1;
	r.uvalue = pIn->lStartAddr2;

	pOut->lPassFail = Pass;
	
	//see if the size was entered, if not, default to 4
	if(pIn->lSizeSuffix !=0x4)
	{
		if(pIn->lSizeSuffix !=0x2)
		{
			if(pIn->lSizeSuffix !=0x1)
			{
				pIn->lSizeSuffix = 0x4;
			}
		}
	}

	//check to see if any of the range to copy to is protected 
	if(Protected(r.uvalue,pIn->lSize))
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = Fail;
		LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
			"ERROR: MEMCOPY, EITHER SOME OR ALL OF THE AREA TO BE WRITTEN TO IS PROTECTED\n");
		strcpy(pOut->cResultString, "ERROR: MEMCOPY, EITHER ALL OR SOME OF THE AREA TO BE WRITTEN TO IS PROTECTED\n");
		return(pOut->lPassFail);
	}

   /*******************************************************************************
    * If the size of the data was not selected as byte, check the start addresses *
    * to ensure correct allignment												  *
    *******************************************************************************/
	if(pIn->lSizeSuffix !=0x1)
	{

		if(pIn->lStartAddr1 % pIn->lSizeSuffix!=0x0)
		{
			pOut->lPassFail = Fail;
			pOut->lReturnCode = Fail;
			LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
				"ERROR: MEMCOPY, START ADDRESS 1 ALLIGNMENT ERROR\n");
			strcpy(pOut->cResultString, "ERROR: MEMCOPY, START ADDRESS 1 ALLIGNMENT ERROR\n");
			return(pOut->lPassFail);
		}

		if(pIn->lStartAddr2 % pIn->lSizeSuffix !=0x0)
		{
			pOut->lPassFail = Fail;
			pOut->lReturnCode = Fail;
			LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
				"ERROR: MEMCOPY, START ADDRESS 2 ALLIGNMENT ERROR\n");
			strcpy(pOut->cResultString, "ERROR: MEMCOPY, START ADDRESS 2 ALLIGNMENT ERROR\n");
			return(pOut->lPassFail);
		}
	}

   /****************************************************************************** 
	* Now use the lSizeSuffix variable to copy the memory from  one  location to *
	* location to another. In order to reduce time, the routines are written out *
	* for each value rather than have another function call.					 *
	******************************************************************************/
	switch(pIn->lSizeSuffix)
	{
		case 1 :	for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						*r.pByte = *p.pByte;
						p.uvalue +=pIn->lSizeSuffix;
						r.uvalue +=pIn->lSizeSuffix;
					}
					p.uvalue = pIn->lStartAddr1;
					r.uvalue = pIn->lStartAddr2;
					for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						if(*r.pByte != *p.pByte)
						{
							pOut->lPassFail = Fail;
							pOut->lReturnCode = Fail;
							LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
								"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pByte, *r.pByte);
							sprintf(pOut->cResultString,"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pByte, *r.pByte);
							return(pOut->lPassFail);
						}
						p.uvalue++;
						r.uvalue++;
					}	
					break;

		case 2 :	for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						*r.pHalf = *p.pHalf;
						p.uvalue +=pIn->lSizeSuffix;
						r.uvalue +=pIn->lSizeSuffix;
					}
					p.uvalue = pIn->lStartAddr1;
					r.uvalue = pIn->lStartAddr2;
					for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						if(*r.pHalf != *p.pHalf)
						{
							pOut->lPassFail = Fail;
							pOut->lReturnCode = Fail;
							LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
								"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pHalf, *r.pHalf);
							sprintf(pOut->cResultString,"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pHalf, *r.pHalf);
							return(pOut->lPassFail);
						}
						p.uvalue++;
						r.uvalue++;
					}

					break;

		case 4 :	for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						*r.pWord = *p.pWord;
						p.uvalue +=pIn->lSizeSuffix;
						r.uvalue +=pIn->lSizeSuffix;
					}
					p.uvalue = pIn->lStartAddr1;
					r.uvalue = pIn->lStartAddr2;
					for(lCount = 0; lCount < pIn->lSize; lCount++)
					{	
						if(*r.pWord != *p.pWord)
						{
							pOut->lPassFail = Fail;
							pOut->lReturnCode = Fail;
							LogError(ERR_MEMCOPY, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, 
								"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pWord, *r.pWord);
							sprintf(pOut->cResultString,"ERROR: MEMCOPY, VERIFY ERROR AT ADDRESS %X, EXPECTED %X, READ %X\n", r.uvalue, *p.pWord, *r.pWord);
							return(pOut->lPassFail);
						}
						p.uvalue++;
						r.uvalue++;
					}
					break;

	}
	if(pOut->lPassFail ==0x0)
	{
		Info("Success: Memory Contents copied\n");
	}
	return(pOut->lPassFail);
}

Command(CMD_MEMCOPY,"memcopy",memcopy,SIZESUFFIX|DIAGNOSTIC,"memcopy","copy memory from one location to another and verify");
Arg(UINT32ARG,"first address to copy from","enter first address to copy from");
Arg(UINT32ARG,"first address to copy to","enter first address to copy to");
Arg(UINT32ARG,"number of elements to copy","enter the number of elements to copy");
CommandEnd();



