/******************************************************************************
*
* Module:	vdmemscan1.c
*
* Description: Scans memory for all occurrences of the user specified
*			   text string. 
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:	This routine does not currently support spaces being entered
*			in the user string.
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/

//  Includes
#include "vm.h"
#include "cmd.h"
#include "esttype.h" 
#include "lib.h"
#include "vware.h"
#include "vderror.h"

//  Local Defines
#define RUNNING 0x0
#define FINISHED 0xff
#define Pass 0
#define Fail 1
#define ExecutionFail 0xffffffff


typedef struct _memscan1In
{
	UINT32	lStartAddress;
	UINT32	lEndAddress;
	char	cCase[4];
	char	cString[10];	
} memscan1In;

typedef struct _memscan1Out
{
	UINT32	lPassFail;
	UINT32	lReturnCode;	
	char	cResultString[128];	
} memscan1Out;


UINT32 memscan1(memscan1In* pIn,memscan1Out* pOut)
{
	
	UINT32 lCount;
	UINT32 lCount2 = 0;
	UINT32 lStatus = RUNNING;
	UINT32 lHitCounter = 0;
	UINT32 lAddrCpy;
	char   cStringCpy[10];
	volatile UINT32 pArray;

	universal p;

	p.uvalue = pIn->lStartAddress;

	strcpy(cStringCpy, pIn->cString);

	strtoupper(cStringCpy);

	//Ensure Start address < End Address 					
	if ( pIn->lEndAddress < pIn->lStartAddress ) 
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN1, START ADDRESS > END ADDRESS\n");
		strcpy(pOut->cResultString,"ERROR: MEMSCAN1, START ADDRESS > END ADDRESS\n");
		return(Fail);
	}

	//Ensure Start address != End Address
	if(pIn->lEndAddress == pIn->lStartAddress)
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN1, START ADDRESS = END ADDRESS\n");
		strcpy(pOut->cResultString,"ERROR: MEMSCAN1, START ADDRESS = END ADDRESS\n");
		return(Fail);
	}
 
	//The code has been duplicated in order to reduce processing time switching on the case variable
    if(tolower(pIn->cCase[0])=='n')
	{
		for(lCount = pIn->lStartAddress; lCount < pIn->lEndAddress; lCount++,p.uvalue++)
		{      
			lCount2 = 0;
			pArray = 0;
			lAddrCpy = lCount;
			if((*p.pByte == pIn->cString[pArray])||(*p.pByte == cStringCpy[pArray]))
			{
				lCount2++;
				do
				{
					p.uvalue++;
					pArray++;
					if((*p.pByte == pIn->cString[pArray])||(*p.pByte == cStringCpy[pArray]))
					{
						lCount2++;
						if(lCount2 == strlen(pIn->cString))
						{
							lStatus = FINISHED;
							lHitCounter++;
						}
					}
					else
					{
						lStatus = FINISHED;
					}
				}while (lStatus != FINISHED);
				lStatus = RUNNING;
			}
			if(lCount2 == strlen(pIn->cString))
			{
				Info("Match found at address 0x%x\n",(p.pByte-(lCount2-1)));
			}
		 }
	}
	else
	{
		for(lCount = pIn->lStartAddress; lCount < pIn->lEndAddress; lCount++,p.uvalue++)
		{      
			lCount2 = 0;
			pArray = 0;
			//lAddrCpy = (UINT32 *)p.pByte;
			if(*p.pByte == pIn->cString[pArray])
			{
				lCount2++;
				do
				{
					p.uvalue++;
					pArray++;
					if(*p.pByte == pIn->cString[pArray])
					{
						lCount2++;
						if(lCount2 == strlen(pIn->cString))
						{
							lStatus = FINISHED;
							lHitCounter++;
						}
					}
					else
					{
						lStatus = FINISHED;
					}
				}while (lStatus != FINISHED);
				lStatus = RUNNING;
			}
			if(lCount2 == strlen(pIn->cString))
			{
				Info("Match found at address 0x%x\n", (p.pByte-(lCount2-1)));
			}
		 }
		
	}

	if(lHitCounter == 0x0)
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN1, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN1, UNABLE TO FIND MATCH FOR %s\n",pIn->cString);
		sprintf(pOut->cResultString,"ERROR: MEMSCAN1, UNABLE TO FIND MATCH FOR %s\n",pIn->cString);
	}
	else
	{
		pOut->lPassFail = Pass;
		if(tolower(pIn->cCase[0])=='y')
		{
			Info("No further occurances of '%s' found\n", pIn->cString);
		}
		else
		{
			Info("No further occurrences of '%s' or any upper and lower case variant of it found\n", pIn->cString);
		}

	}
	 
	return(pOut->lPassFail);
}

Command(CMD_MEMSCAN1,"memscan1",memscan1,DIAGNOSTIC,"memscan1","search memory for all occurrences of specified text (no spaces allowed)");
Arg(UINT32ARG,"start address","start address");
Arg(UINT32ARG,"end address","end address");
Arg(CHAR4ARG,"case sensitive (y or n)", "do you want the search to be case sensitive (y) or (n)");
Arg(CHARARG|15,"string","string to search for");
CommandEnd();

