/******************************************************************************
*
* Module: vdPpcMemscan2.c
*
* Description: Scan memory for all occurrences of a specified value
*
* Project:     HSI VisionWare -- vWare
*
* Functions:
*
* Notes:
*
* History:		See Visual SourceSafe History
*
* Disclaimer: Copyright 1999-2000 Wind River Systems, Inc. All Rights Reserved
*
*******************************************************************************/
//  Includes
#include "vm.h"
#include "cmd.h"
#include "vderror.h"

//  Local Defines
#define Pass 0
#define Fail 1
#define ExecutionFail 0xffffffff

typedef struct _memscan2In
{
	UINT32  lWidth;
	UINT32	lStartAddr;	
	UINT32	lEndAddr;	
	UINT32	lValue;	
} memscan2In;

typedef struct _memscan2Out
{
	UINT32	lPassFail;
	UINT32	lReturnCode;	
	char	cResultString[128];	
} memscan2Out;


UINT32 memscan2(memscan2In* pIn,memscan2Out* pOut)
{
	UINT32 lCount;
	UINT32 lHitCounter = 0;
	universal p;

	p.uvalue = pIn->lStartAddr;

	//Ensure Start address < End Address 					
	if ( pIn->lEndAddr < pIn->lStartAddr ) 
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN2, START ADDRESS > END ADDRESS\n");
		strcpy(pOut->cResultString,"ERROR: MEMSCAN2, START ADDRESS > END ADDRESS\n");
		return(Fail);
	}

	//Ensure Start address != End Address
	if(pIn->lEndAddr == pIn->lStartAddr)
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN2, START ADDRESS = END ADDRESS\n");
		strcpy(pOut->cResultString,"ERROR: MEMSCAN2, START ADDRESS = END ADDRESS\n");
		return(Fail);
	}


	//Ensure that the width has been captured correctly
	if(pIn->lWidth != 0x4)
	{
		if(pIn->lWidth != 0x2)
		{
			if(pIn->lWidth != 0x1)
			{
				pOut->lPassFail = Fail;
				pOut->lReturnCode = ExecutionFail; 
				LogError(ERR_MEMSCAN2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN2, BAD WIDTH\n");
				return(Fail);
			}
		}
	}


	switch(pIn->lWidth)
	{
		case 4:	for(lCount = pIn->lStartAddr; lCount < pIn->lEndAddr; lCount++,p.uvalue+=pIn->lWidth)
				{
					if(*p.pWord == pIn->lValue)
					{
						Info("Match found at 0x%X\n", lCount);
						lHitCounter++;
					}
				}
				break;

		case 2:	for(lCount = pIn->lStartAddr; lCount < pIn->lEndAddr; lCount++,p.uvalue+=pIn->lWidth)
				{
					if(*p.pHalf == pIn->lValue)
					{
						Info("Match found at 0x%X\n", lCount);
						lHitCounter++;
					}
				}
				break;

		case 1:	for(lCount = pIn->lStartAddr; lCount < pIn->lEndAddr; lCount++,p.uvalue++)
				{
					if(*p.pByte == pIn->lValue)
					{
						Info("Match found at 0x%X\n", lCount);
						lHitCounter++;
					}
				}
				break;
	}

	if(lHitCounter == 0x0)
	{
		pOut->lPassFail = Fail;
		pOut->lReturnCode = ExecutionFail;
		LogError(ERR_MEMSCAN2, CONT_ON_ERROR|LOG_TO_FILE|LOG_TO_CONSOLE, "ERROR: MEMSCAN2, UNABLE TO FIND MATCH FOR 0x%X\n",pIn->lValue);
		sprintf(pOut->cResultString,"ERROR: MEMSCAN2, UNABLE TO FIND MATCH FOR 0x%X\n",pIn->lValue);
	}
	else
	{
		pOut->lPassFail = Pass;
		Info("No further occurrences of 0x%X found\n", pIn->lValue);
	}
	 
	return(pOut->lPassFail);
}
Command(CMD_MEMSCAN2,"memscan2",memscan2,SIZESUFFIX|DIAGNOSTIC,"memscan2","search memory for all occurrences of specified numerical value");
Arg(UINT32ARG,"start address","start address");
Arg(UINT32ARG,"end address","end address");
Arg(UINT32ARG,"value","value to search for");
CommandEnd();


