/***
*
****/

#ifndef MMUDIAGS
#define MMUDIAGS

#ifdef __cplusplus
extern "C" {
#endif

// 2 flash, 2 sram, 1 registers for 80312
#define MAX_PHYSICAL_MEMORY_SEGMENTS 5
// !!! MAX_VIRTUAL_MEMORY_SEGMENTS is just a number, I don't know how to determine the max.
#define MAX_VIRTUAL_MEMORY_SEGMENTS 0x20
#define MEMORY_PROTECTION_BIT_IN_FIRST_DESCRIPTOR 0x00000200
#define SDRAM_SIZE_MASK 0x000000F8
// Memory management code
#define FAULT_IN_FIRST_LEVEL_DESCRIPTOR 0x01
#define FAULT_IN_SECOND_LEVEL_DESCRIPTOR 0x02
#define FAULT_IN_SECTION_DESCRIPTOR 0x03
#define FAULT_IN_COARSE_PAGE_TABLE_DESCRIPTOR 0x04
#define FAULT_IN_FINE_PAGE_TABLE_DESCRIPTOR 0x05
#define FAULT_IN_PARSING_FIRST_LEVEL_DESCRIPTOR 0x00010000
#define FAULT_IN_PARSING_SECOND_LEVEL_DESCRIPTOR 0x00020000

typedef struct
{
	UINT32 Fault;
	UINT32 ECCEnabled;
	UINT32 VirtualAddress;
	UINT32 PhysicalAddress;
	UINT32 FirstLevelDescriptor;
	UINT32 SecondLevelDescriptor;
	UINT32 StartOfPage;
	UINT32 EndOfPage;
	UINT32 Attributes;
	UINT32 xcbbits;
} tMMUAtributesForMemoryPage;

typedef struct
{
	UINT32 Start;
	UINT32 End;
} tUINT32Range;

typedef struct
{
	UINT32 NumberOfSegments;
	UINT32 StartAddress[MAX_VIRTUAL_MEMORY_SEGMENTS];
	UINT32 EndAddress[MAX_VIRTUAL_MEMORY_SEGMENTS];
} tVirtualMemoryList;

typedef struct
{
	UINT32 Fault;
	UINT32 MMUEnabled;
	UINT32 DataCacheEnabled;
	UINT32 InstructionCacheEnabled;
	UINT32 TTBase;
	UINT32 AuxControlRegValue;
} tMMUGlobalAtributes;

typedef struct
{
	UINT32 NumberOfSegments;
	UINT32 StartAddress[MAX_PHYSICAL_MEMORY_SEGMENTS];
	UINT32 EndAddress[MAX_PHYSICAL_MEMORY_SEGMENTS];
} tPhysicalMemoryList;

#ifdef __cplusplus
}
#endif
#endif // MMUDIAGS




