/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:01p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/hal.h $
*   WORKFILE:       $Workfile: hal.h $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose: Contains the macros for the hardware abstraction layer.
*			
* Notes:
*			
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/hal.h $
 * 
 * 1     3/11/04 3:01p Tepulkow
 * 
 * 1     5/16/03 1:41p Tepulkow
* 
* 10    1/23/02 5:44p Rsood
* Added macro for Simulator
* 
* 9     1/14/02 7:02p Machambe
* Removing Processor directives and FORCED_ERR_JF
* 
* 8     1/09/02 4:47p Jjfreema
* Added code for Froced Error Program by Joshua Freeman.
* 
* 7     12/14/01 6:51p Psnahlou
* 
* 5     11/16/01 11:16p Machambe
* 
* 4     11/06/01 10:42a Machambe
* Added support for SA1100 nonaligned .
* 
* 3     7/03/01 4:09p Machambe
* 
* 2     5/03/01 3:54p Machambe
* 
* --------------------------------------------------------------------------
*/

#ifndef __HW_IF_H_              /* Include file contents only once per compilation */
#define __HW_IF_H_

#include "sim.h"

#ifdef SIMULATOR                /* SIMULATOR */

#define READ_FROM_HW(addr) (SimulatorRead(addr))
#define WRITE_TO_HW(addr, value) (SimulatorWrite(addr, value))

#elif SLOW_PORT_FIX             /* Fixes Slow Port A0/A1 silicon issue */

#define READ_FROM_HW(addr) (SlowPortRead(addr))
#define WRITE_TO_HW(addr, value) (SlowPortWrite(addr,value))

#elif CODE_GEN                  /* Code Generation */

extern ushort CodeGenRead (bb_RegPointer_type addr);
extern ushort CodeGenWrite (bb_RegPointer_type addr, ushort val);
#define READ_FROM_HW(addr) (CodeGenRead(addr))
#define WRITE_TO_HW(addr, value) (CodeGenWrite(addr,value))

#elif SOCKET_COMM

extern ushort SocketCommRead (bb_RegPointer_type addr);
extern ushort SocketCommWrite (bb_RegPointer_type addr, ushort value);
#define READ_FROM_HW(addr) (SocketCommRead(addr))
#define WRITE_TO_HW(addr, value) (SocketCommWrite(addr, value))

#else /* Default - Memory mapped device */

#define READ_FROM_HW(addr) ((*(addr)))
#define WRITE_TO_HW(addr, value) ((*(addr)) = (value))

#endif




#endif /* __HW_IF_H_ */
