/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:02p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/include/ixf18100_pcs.h $
*   WORKFILE:       $Workfile: ixf18100_pcs.h $
*
* --------------------------------------------------------------------------
* Project: ixf18100 family device API
*
* Purpose: This file contains function prototypes for PCS block related
*          routines
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/include/ixf18100_pcs.h $
 * 
 * 1     3/11/04 3:02p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 13    1/10/03 2:30p Pkjain2
* 
* 10    7/11/02 4:22p Pkjain2
* Corrected some headers. Added Legal notice to some files. 
* 
* 9     7/08/02 1:16p Pkjain2
* Moved Calypso specific data stuctures into Calypso block specific data
* structure file.
* 
* 2     8/23/01 4:12p Wrbrackn
* Made changes suggested after review of the code with Mark.
* 
*
* 
* --------------------------------------------------------------------------
*/
#ifndef __IXF18100_PCS_H__
#define __IXF18100_PCS_H__

/******************************************************************************
* Function : Ixf18100PcsSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the PCS register block by writing 
*               to every writable bit and bit-field in the PCS registers. 
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SelCfg: Selects the part of the block to be configured.
*          *ptPcsCfg: A pointer to a structure containing settings for the 
*                     PCS register block.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                  ixf18100_PcsCfg_t *ptPcsCfg);

                  /******************************************************************************
                  * Function : Ixf18100PcsGetCfg
                  *------------------------------------------------------------------------------
                  * Description : This function gets the configuration settings of the PCS block
                  *------------------------------------------------------------------------------
                  * Inputs : BaseAddress: BaseAddress of the device
                  *          SelCfg: Selects the part of the block to be configured.
                  *          *ptPcsCfg: A pointer to a structure that will be filled with the
                  *           PCS register block configuration settings.
                  * Outputs : None
                  *------------------------------------------------------------------------------
                  * Returns : bb_NO_ERROR, bb_INV_PARAMETER
                  *------------------------------------------------------------------------------
                  * Access Globals : None.
                  *------------------------------------------------------------------------------
                  * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                  ixf18100_PcsCfg_t *ptPcsCfg);

                  /******************************************************************************
                  * Function : Ixf18100PcsSetTestCfg
                  *------------------------------------------------------------------------------
                  * Description : This function sets the bits and bit fields within the PCS 
                  *               register block that are associated with testing functionality
                  *               of the device.
                  *------------------------------------------------------------------------------
                  * Inputs : *ptChipData: A pointer to information about the device
                  *          *ptPcsTestCfg: A pointer to a structure that contains test 
                  *                         configuration settings for the device.
                  * Outputs : None
                  *------------------------------------------------------------------------------
                  * Returns : bb_NO_ERROR
                  *------------------------------------------------------------------------------
                  * Access Globals : None.
                  *------------------------------------------------------------------------------
                  * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsSetTestCfg(bb_ChipData_t *ptChipData,
                      ixf18100_PcsTestCfg_t *ptPcsTestCfg);

                      /******************************************************************************
                      * Function : Ixf18100PcsGetTestCfg
                      *------------------------------------------------------------------------------
                      * Description  : This function gets the test configuration settings of the 
                      *                PCS register block and stores them in a structure.
                      *------------------------------------------------------------------------------
                      * Inputs  : *ptChipData: A pointer to information about the device
                      *           *ptPcsTestCfg: A pointer to a structure that will be filled with 
                      *                          the settings used to configure the testing options 
                      *                          in the PCS register block. 
                      * Outputs : None
                      *------------------------------------------------------------------------------
                      * Returns : bb_NO_ERROR
                      *------------------------------------------------------------------------------
                      * Access Globals : None.
                      *------------------------------------------------------------------------------
                      * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsGetTestCfg(bb_ChipData_t *ptChipData,
                      ixf18100_PcsTestCfg_t *ptPcsTestCfg);

                      /******************************************************************************
                      * Function : Ixf18100PcsGetStatus
                      *------------------------------------------------------------------------------
                      * Description : This function gets the status configurations of the PCS 
                      *               register block and stores them in a structure.
                      *------------------------------------------------------------------------------
                      * Inputs : BaseAddress: BaseAddress of the device
                      *          SelStat: Selects the part of the block to get the status
                      *          *ptPcsStatus: A pointer to a structure used to store the status 
                      *                        information for the PCS register block.
                      * Outputs : None
                      *------------------------------------------------------------------------------
                      * Returns : bb_NO_ERROR
                      *------------------------------------------------------------------------------
                      * Access Globals : None.
                      *------------------------------------------------------------------------------
                      * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                     ixf18100_PcsStatus_t *ptPcsStatus);

                     /******************************************************************************
                     * Function : Ixf18100PcsGetCounters
                     *------------------------------------------------------------------------------
                     * Description : This function gets the PCS counter values and stores them in 
                     *               a structure.
                     *------------------------------------------------------------------------------
                     * Inputs : BaseAddress: BaseAddress of the device
                     *          SelCntr: Selctes the counter(s) to be read.
                     *          *ptCounters: A pointer to a structure filled with the PCS 
                     *                          counter values.
                     * Outputs : None
                     *------------------------------------------------------------------------------
                     * Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_SEL_COUNTERS
                     *------------------------------------------------------------------------------
                     * Access Globals : None.
                     *------------------------------------------------------------------------------
                     * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsGetCounters(bb_RegPointer_type BaseAddress,
                       bb_SelCounters_e SelCntr, void *ptCounters);

                       /******************************************************************************
                       * Function : Ixf18100PcsReset
                       *------------------------------------------------------------------------------
                       * Description : This function resets the PCS block
                       *------------------------------------------------------------------------------
                       * Inputs : ptChipData : Pointer to information about the device
                       * Outputs : None
                       *------------------------------------------------------------------------------
                       * Returns : bb_NO_ERROR
                       *------------------------------------------------------------------------------
                       * Access Globals : None.
                       *------------------------------------------------------------------------------
                       * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PcsReset(bb_ChipData_t *ptChipData);

#endif /*__IXF18100_PCS_H__*/
