/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************64
*     Name:    OSAL C header file
*  
*    Type:     C header file
* 
*    Desc:     This file contains the #define and common data 
*              structure used for OSAL primitives.
*
*
*    File:     osal.h 
*
*    Sid:      osal.h@@/main/1 - Tue Dec 10 10:32:45 2002
*
*    Prg:      shabbir ali 
*
*****************************************************************/

#ifndef __IX_OSAL_H__
#define __IX_OSAL_H__

#ifdef __cplusplus
extern "C" {
#endif
    
    /* Common data structures used by OSAL */
    typedef enum IX_Err
    {
        IX_NO_ERR			= 0, /* Success */
            IX_FAILED			= 1, /* "Expected" Failures */
            IX_UNKNOWN_ERROR	= 2, /* Failed for unknown reason */
            IX_OUT_OF_MEMORY	= 3, /* Failed - out of memory */
            IX_BAD_PARAMS		= 4, /* Failed - Invalid Parameter passed in */
            IX_UNKNOWN_HANDLE	= 5  /* Failed - Invalid value of parameter */
    }IX_Err_t;
    
    typedef enum IX_Boolean
    {
        IX_FALSE          = 0, /* FALSE */
            IX_TRUE           = 1  /* TRUE */
    }IX_Boolean_t;
    
    typedef struct IX_Thread_Info
    {
        char thread_name[64];      /* Thread Name */
        int stackSizeInBytes;      /* Stack Size In bytes */
        int priority;              /* Thread Priority */
        void *arg;                 /* Thread Argument */
    }IX_Thread_Info_t;
    
    typedef void *(* IX_Function_Ptr_t)(void *);
    
#define IX_NO_ERROR 0
#define IX_TRUE 1
#define IX_FALSE 0
#define IX_FAILED 1
#define IX_UNKNOWN_ERROR 2
#define IX_OUT_OF_MEMORY 3
#define IX_BAD_PARAM 4
#define IX_UNKNOWN_HANDLE 5
    
    /* Hardware specific defines */
    /* Note :- Change the define name */
    /* #ifdef IX_XSCALE */
    typedef unsigned int IX_U32;       /* 32 bit unsigned integer */
    typedef int IX_S32;                /* 32 bit signed integer */
    typedef unsigned short int IX_U16; /* 16 bit unsigned integer */
    typedef short int IX_S16;          /* 16 bit signed integer */
    typedef unsigned char IX_U8;       /* 8 bit unsigned integer */
    typedef char IX_S8;                /* 8 bit signed integer */
    /* #endif */                       /* IX_SACLE */
    
    /********* Linux User Space section ******************************/
#ifdef IX_LINUX_USR_SPACE
    
    /* Include files for Linux */
#include "pthread.h"  
#include "sys/timeb.h" /* Tick Counters */
#include "stdlib.h"    /* Allocate/Deallocate Memory */
#include "errno.h"     /* Error Code Header File */
    
    /* Enable the following lines only when the -ansi and -pedantic features of
    * the xscale_be-gcc compiler can be used fully. When enabling the following
    * include files remove the #defines in /inc/ix_osal_Linux.h */
#if 0
#include "asm-arm/system.h" /* Interrupt Enable/disable Macros */
#include "asm-arm/proc-armv/system.h" /* Interrupt Enable/disable Macros */
#endif /* if 0 */
    
    /* Data Types Specific to Linux User Space */
    typedef pthread_mutex_t *IX_Sem_t;          /* Semaphore Id Type */
    typedef pthread_t IX_Thread_id;             /* Thread Id Type */
    
#include "ix_osal_Linux.h"            /* Linux Specific Include file */
    
    /* Defines specific to Linux User Space */
#define IX_GiveBinarySem(s) IX_GiveBinarySem_LnxUsrSpc(s)
#define IX_DMA_Malloc(sizeInBytes) IX_DMA_Malloc_LnxUsrSpc(sizeInBytes)
#define IX_DMA_Free(ptr) IX_DMA_Free_LnxUsrSpc(ptr)
#define IX_Malloc(sizeInBytes,heapNumber) \
    IX_Malloc_LnxUsrSpc(sizeInBytes, heapNumber)
#define IX_Free(ptr) IX_Free_LnxUsrSpc(ptr)
#define IX_Thread_Exit() IX_Thread_Exit_LnxUsrSpc()
#define IX_GetLowResClockFrequency(frequency) \
    IX_GetLowResClockFrequency_LnxUsrSpc(frequency)
#define IX_GetHighResClockFrequency(frequency) \
    IX_GetHighResClockFrequency_LnxUsrSpc(frequency)
    
#endif /* IX_LINUX_USR_SPACE */
    /******** End of Linux User SPace Section ***************/
    
    
    /********* VxWorks section ******************************/
#ifdef IX_VXWORKS
    
    /* Include files for VxWorks */
#include "VxWorks.h"   /* Main VxWorks defines */
#include "semLib.h"    /* Binary Semaphore */
#include "sysLib.h"    /* Tick Counters */
#include "cacheLib.h"  /* Cache Safe Memory */
#include "stdlib.h"    /* Allocate/Deallocate Memory */
#include "memLib.h"    /* Alocate/Deallocate Memory */
#include "taskLib.h"   /* Task Create/Delete/Exit */
#include "intLib.h"    /* Common Error defines */
#include "objLib.h"    /* common Obj Lib define */
#include "smObjLib.h"  /* Common defines */ 
    
    /* Data Types specific to VxWorks */
    typedef SEM_ID IX_Sem_t;          /* Semaphore Id Type */
    typedef IX_U32 IX_Thread_id;      /* Thread Id Type */
    
#include "ix_osal_VxWorks.h"         /* VxWorks specific include file */
    
    /* Defines Specific to VxWorks */
#define IX_CreateBinarySem(s) IX_CreateBinarySem_VxWrks(s)
#define IX_DMA_Malloc(sizeInBytes) IX_DMA_Malloc_VxWrks(sizeInBytes)
#define IX_DMA_Free(ptr) IX_DMA_Free_VxWrks(ptr)
#define IX_Malloc(sizeInBytes,heapNumber) \
    IX_Malloc_VxWrks(sizeInBytes,heapNumber)
#define IX_Free(ptr) IX_Free_VxWrks(ptr)
#define IX_Thread_Exit() IX_Thread_Exit_VxWrks()
#define IX_GetLowResClockFrequency(frequency) \
    IX_GetLowResClockFrequency_VxWrks(frequency)
#define IX_GetHighResClockFrequency(frequency) \
    IX_GetHighResClockFrequency_VxWrks(frequency)
#define IX_GetLowResClockCount(currentTicks) \
    IX_GetLowResClockCount_VxWrks(currentTicks)
#define IX_GetHighResClockCount(currentTicks) \
    IX_GetHighResClockCount_VxWrks(currentTicks)
#endif /* IX_VXWORKS */
    /********* End of VxWorks Section *****************************/
    
    /********* Undefined OS section ******************************/
#ifdef IX_UNKNOWN
    
    typedef int IX_Sem_t;          /* Semaphore Id Type */
    typedef long IX_Thread_id;      /* Thread Id Type */
    
#define IX_CreateBinarySem(s) 
#define IX_DMA_Malloc(sizeInBytes) 
#define IX_DMA_Free(ptr) 
#define IX_Malloc(sizeInBytes,heapNumber) 
#define IX_Free(ptr) 
#define IX_Thread_Exit() 
#define IX_GetLowResClockFrequency(frequency) 
#define IX_GetHighResClockFrequency(frequency) 
#define IX_GetLowResClockCount(currentTicks) 
#define IX_GetHighResClockCount(currentTicks) 
#endif
    /********* End of Undefined Section *****************************/
    
    
    /* extern function defines */
    
    /* Binary semaphore support */
    
#ifndef IX_VXWORKS
    extern IX_Err_t IX_CreateBinarySem(IX_Sem_t *s);
#endif
    
    /* ndef IX_VXWORKS */
    extern IX_Err_t IX_DestroyBinarySem(IX_Sem_t s);
    
#ifndef IX_LINUX_USR_SPACE
    extern IX_Err_t IX_GiveBinarySem(IX_Sem_t s);
#endif /* ndef LINUX_USR_SPACE */
    
    extern IX_Err_t IX_TakeBinarySem(IX_Sem_t s,IX_Boolean_t block);
    
    /* Interrup Lock/Unlock */
    extern IX_Err_t IX_Int_Lock(IX_U32 *lockId);
    extern IX_Err_t IX_Int_Unlock(IX_U32 lockId);
    
    /* Task/Thread Creation/Deletion/Exit */
    extern IX_Err_t IX_Thread_Spawn(IX_Function_Ptr_t f,
        IX_Thread_Info_t *t, IX_Thread_id *i);
    extern IX_Err_t IX_Thread_Stop(IX_Thread_id i);
    
    /* Tick Counters */
    
#ifndef IX_VXWORKS
    extern IX_Err_t IX_GetLowResClockCount(IX_U32 *currentTicks);
    extern IX_Err_t IX_GetHighResClockCount(IX_U32 *currentTicks);
#endif /* ndef IX_VXWORKS */
    
#ifdef __cplusplus
}
#endif


#endif /* __IX_OSAL_H__ */

/**********************************************************************

  ver       pat    init                  description
  ------------ -------- ---- ----------------------------------------------
  $SID$          ---    sali 1. initial release.
  
    **********************************************************************/
