/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************64
*     Name:    OSAL C header file for VxWorks 
*  
*    Type:     C header file
* 
*    Desc:     This file contains the #define and data structures 
*              used by OSAL primitives for VxWorks.
*
*
*    File:     ix_osal_VxWorks.h 
*
*    Sid:      ix_osal_VxWorks.h@@/main/1 - Tue Dec 10 10:32:51 2002
*
*    Prg:      shabbir ali 
*
*****************************************************************/

#ifndef __IX_OSAL_VXWORKS_H__
#define __IX_OSAL_VXWORKS_H__

#ifdef __cplusplus
extern "C" {
#endif
    
#ifdef IX_VXWORKS 
    
    /* VxWorks specific defines */
    
    /* Create a Binary Semaphore */
#define IX_CreateBinarySem_VxWrks(s) \
    ( (*s) = semBCreate(SEM_Q_FIFO,SEM_EMPTY), \
    (((*s) == NULL) ? IX_OUT_OF_MEMORY : IX_NO_ERROR) )
    
    /* Allocate Cache-Safe Memory */
#define IX_DMA_Malloc_VxWrks(sizeInBytes) \
    cacheDmaMalloc((size_t)sizeInBytes)
    
    /* De-allocate Cache-Safe Memory */
#define IX_DMA_Free_VxWrks(ptr) \
    cacheDmaFree(ptr) 
    
    /* Allocate regular Memory */
#define IX_Malloc_VxWrks(sizeInBytes,heapNumber) \
    malloc(sizeInBytes)
    
    /* De-allocate Regular Memory */
#define IX_Free_VxWrks(ptr) \
    free(ptr)
    
    /* Exit a Thread */
#define IX_Thread_Exit_VxWrks() \
    exit(0)
    
    /* Get Low Resolution Clock's Frequency */
#define IX_GetLowResClockFrequency_VxWrks(frequency) \
    ((*frequency) = sysClkRateGet()) ? IX_NO_ERROR : IX_NO_ERROR
    
    /* Get High Resolution Clock's Frequency */
#define IX_GetHighResClockFrequency_VxWrks(frequency) \
    ((*frequency) = sysClkRateGet()) ? IX_NO_ERROR : IX_NO_ERROR
    
    /* Get Low Resolution Clock's Tick Count */
#define IX_GetLowResClockCount_VxWrks(currentTicks) \
    ((*currentTicks) = tickGet()) ? IX_NO_ERROR : IX_NO_ERROR
    
    /* Get High Resolution Clock's Tick Count */
#define IX_GetHighResClockCount_VxWrks(currentTicks) \
    ((*currentTicks) = tickGet()) ? IX_NO_ERROR : IX_NO_ERROR
    
    
    
    /* VxWorks specific extern function defines */
    
    /* Binary semaphore support */
    extern IX_Err_t IX_DestroyBinarySem_VxWrks(IX_Sem_t s);
    extern IX_Err_t IX_GiveBinarySem_VxWrks(IX_Sem_t s);
    extern IX_Err_t IX_TakeBinarySem_VxWrks(IX_Sem_t s,IX_Boolean_t block);
    
    
    /* Interrup Lock/Unlock */
    extern IX_Err_t IX_Int_Lock_VxWrks(IX_U32 *lockId);
    extern IX_Err_t IX_Int_Unlock_VxWrks(IX_U32 lockId);
    
    
    /* Task/Thread Creation/Deletion/Exit */
    extern IX_Err_t IX_Thread_Spawn_VxWrks(IX_Function_Ptr_t f,
        IX_Thread_Info_t *t, IX_Thread_id *i);
    extern IX_Err_t IX_Thread_Stop_VxWrks(IX_Thread_id i);
    
    extern FUNCPTR ixConvFuncVxWrks(int lsbFuncPtr, int msbFuncPtr, 
        int lsbArgPtr, int msbArgPtr, int arg4, int arg5, int arg6,
        int arg7, int arg8,int arg9);
    
    
    
#endif /* IX_VXWORKS */
    
#ifdef __cplusplus
}
#endif

#endif /* __IX_OSAL_VXWORKS_H__ */

/**********************************************************************

  ver       pat    init                  description
  ------------ -------- ---- ----------------------------------------------
  $SID$          ---    sali 1. initial release.
  
**********************************************************************/
