/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:03p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_alarms_d.h $
*   WORKFILE:       $Workfile: ixf18100_alarms_d.h $
*
* --------------------------------------------------------------------------
* Project: ixf18100 family device API
*
* Purpose: 
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_alarms_d.h $
 * 
 * 1     3/11/04 3:03p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 13    3/06/03 1:58p Pkjain2
* 
* 12    1/30/03 4:40p Pkjain2
* Made changes for PTQ release. Added Getstatus calls for SONET and
* GLOBAL block.
* 
* 11    10/17/02 11:33a Jaalliso
* Replacing tab characters with spaces
* 
* 10    8/28/02 11:23a Kssaepha
* Added Counter Callback types
* 
* 9     3/14/02 3:05p Srodrig1
* 
* 8     3/04/02 9:51a Srodrig1
* 
* 7     3/01/02 2:09p Srodrig1
* 
* 6     3/01/02 10:48a Srodrig1
* 
* 5     1/25/02 11:06a Srodrig1
* 
* 4     1/23/02 2:28a Srodrig1
* 
* 3     1/21/02 9:41p Srodrig1
* 
* 
* 1     12/15/01 9:58p Srodrig1
* Initial check in - file headers only
* 
* --------------------------------------------------------------------------
*/
#ifndef IXF18100_ALARMS_D_H
#define IXF18100_ALARMS_D_H

#include "ixf_bitfield_macros.h"

/* Interrupt Hierarchy:
The Alarm are set in the following hierarchy 

  Chip Interrupts ___ Global Clock
  |
  |___ Watch Dog
  |
  |___ Global Interrupt
  |
  |___ APS/WIS ___ Rx LOH    
  |           |
  |           |___ Framer Ohp
  |
  |
  |____ SONET/APS ___ Rx POH 
  |              |
  |              |___ Rx J1
  |              |
  |              |___ Rx Pointer
  |              |
  |              |___ Framer SOH 
  |              |
  |              |___ Framer Alarm
  |              |
  |              |___ Tx Tim (Loss of system and/or Pulse Clock)
  |              |
  |              |___ Serial Alarm
  |              |
  |              |___ Transmit Transport Serial Alarm
  | 
  |___ GFP
  |
  |___ PCS
  |
  |___ MAC 1
  |
  |___ MAC 2
  |
  |___ SIF/SPI4
  |
  |___ POS
*/



/******************************************************************************
* Alarm configuration structures:
*     These alarm configuration structures will be used to enable\disable the 
* desired alarms.
*    The structures are divided in a bitfield structure and a register, so every 
* member of structure is really a direct mapping of one of the bits of the 
* register, so we can manipulate individual bits without the use of shifts 
* and/or masks.
*     We are using a union that make us possible read/write the contents as a whole 
* register, or read/write individual bits. Example:
*
*   typedef union
*    {
*        uchar uiRegbyte;  --This is our 8 bits register
*        struct {
*           uchar bit0     :1; -- Least significant bit mapped
*           uchar bit1     :1;
*           uchar bit2     :1;
*           uchar bit3     :1;
*           uchar bit4to7  :4; -- The four most significant bits
*         } tBitField;
*     } regByte_t;
*
*   .
*   .
*   .
*   regByte_t register;
*
*   register.uiRegByte = 0x00;
*   register.tBitField.bit0 = 1; -- Now uiRegByte is 0x01
*   register.tBitField.bit4to7 = 0xF; -- Now uiRegByte is 0xF1;
*   register.tBitFiled.bit3 = 1; -- uiRegByte is 0xF9... etc.
*
******************************************************************************/ 


/*Possible states of the  Interrupt Status*/
enum 
{
    ixf18100_eINTERRUPT_INACTIVE = 0,
        ixf18100_eINTERRUPT_ACTIVE
} ixf18100_eIntrptStatus_e;


/*Possible states of the  Interrupt configuration*/
enum { 
    ixf18100_eINTERRUPT_DISABLE = 0,
        ixf18100_eINTERRUPT_ENABLE
}ixf18100_eIntrEnabling;


/* Global Chip Configuration, This is the upper level on the Interrup Hierarchy 
* from here all the remaining interrupts, have to be configured 
* Every bitfield has to have enable/disable estatus
*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_12(uint uiPosIntEn   : 1,  /* Pos interrupt enable/disable */
            uint uiSifIntEn   : 1,  /* SIF/SPI4 interrupt enabgle/disable */
            uint uiMacIntEn   : 1,  /* First part of the MAC interrupt configuration */
uint :1, 
      uint uiPcsIntEn   : 1,  /* PCS Interrupt Configuration Enable/Disable */
      uint uiGfpIntEn   : 1,  /* GFP Interrupt Configuration Enable/Disable */
      uint uiWisIntEn   : 1,  /* SONET/WIS Interrupt Configuration */
      uint uiWisApsIntEn: 1,/* APS/WIS Interrupt Configuration */
      uint uiGlbIntEn   : 1, /* Global Interrupt configuration */
      uint uiWatchIntEn : 1, /* WatchDog Interrupt Configuration */ 
      uint uiClkIntEn   : 1, /* Clock Interrupt Configuration */ 
uint : 5)
    } tCnfg;
} ixf18100_GlbChpIntCfg_t;


/* POS Receive Side Interrupt  Configuration 
*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_16(
            uint uiRcvFcsiIntEn         : 1,
            uint uiRcvAbortIntEn        : 1,
            uint uiRcvAciIntEn          : 1,
            uint uiRcvDblEsclIntEn      : 1,
            uint uiRcvMinPliIntEn       : 1,
            uint uiRcvMaxPliIntEn       : 1,
            uint uiRcvSfiIntEn          : 1,
            uint uiRcvFcsCntlIntEn      : 1,
            uint uiRcvAfcCntlIntEn      : 1,
            uint uiRcvGfcCntlIntEn      : 1,
            uint uiRcvGfbByteIntEn      : 1,
            uint uiRcvMinPlcCntlIntEn   : 1,
            uint uiRcvMaxPlcCntlIntEn   : 1,
            uint uiRcvAfbByteCntlIntEn  : 1,
            uint uiRcvFcseByteCntlIntEn : 1,
uint :1)
      
    } tCnfg;
    
}ixf18100_PosRcvIntCfg_t;



/* POS Transmit Side Interrupt Configuration
*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4( uint uiXmtFcsiIntEn      : 1,
            uint uiXmtFrmCntlIntEn   : 1,
            uint uiXmtByteCntlIntEn  : 1,
uint :13)
    } cnfg;
    
}ixf18100_PosXmtIntCfg_t;


/* SIF/SPI4 Interrupt Configuration */
typedef union
{
    ushort uiRreg;
    struct{
        BITFIELD_9(uint uiTxBusErrIntEn     : 1,
            uint uiTxParErrIntEn     : 1,
            uint uiTxFullErrIntEn    : 1,
            uint uiTxNoEopErrIntEn   : 1,
            uint uiRxBusErrIntEn     : 1,
            uint uiRxFullErrIntEn    : 1,
            uint uiRxNoEopErrIntEn   : 1,
            uint uiRxDip2FailErrIntEn: 1,
uint : 8)        
    } tCnfg;
    
}ixf18100_SifIntCfg_t;



/* PCS Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4(uint uiPcsTxFifoOvIntEn   : 1,
            uint uiPcsTxFifoUvIntEn   : 1,
            uint uiPcsRxFifoOvIntEn   : 1,
uint :13)
    } tCnfg;
    
}ixf18100_PcsIntCfg_t;


/* GFP Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_11(uint uiRxConFrmIntEn    : 1,
            uint uiRxThecErrIntEn   : 1,
            uint uiRxHecErrIntEn    : 1,
            uint uiRxFscErrIntEn    : 1,        
            uint uiRxLodSynErrIntEn : 1,
            uint uiRxLodPreIntEn    : 1,
            uint uiTxOamComIntEn    : 1,
            uint uiRxOamComIntEn    : 1,
            uint uiMaxFrmExIntEn    : 1,        
            uint uiCntlOvFlowIntEn  : 1,
uint :6)
    } tCnfg;
    
}ixf18100_GfpIntCfg_t;



/* Global Clk Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_10( uint uiClkRxFailIntEn      : 1,
            uint uiClkRxApsFailIntEn   : 1,
            uint uiClkTxFailIntEn      : 1,
            uint uiClkTxApsFailIntEn   : 1,        
            uint uiClkRefFailIntEn     : 1,
            uint uiClkEnetFailIntEn    : 1,
            uint uiClkRefSpiFailIntEn  : 1,
            uint uiClkTxSpiFailIntEn   : 1,
            uint uiClkTxFifoStatIntEn  : 1,
uint :7)
    } tCnfg;
    
}ixf18100_GlbClkIntCfg_t;


/* Global Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_3(uint uiCnfgErrIntEn    : 1,
            uint uiSpChngErrIntEn  : 1,
uint : 14)
    } tCnfg;
    
}ixf18100_GlbIntCfg_t;



/* SONET/WIS Alarm configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_10( uint uiRxPohIntEn     : 1, 
            uint uiRxJ1IntEn      : 1,
            uint uiRxPtIntEn      : 1,
            uint uiRxLohIntEn     : 1,     
            uint uiFrmSohpIntEn   : 1,
            uint uiFrmAlIntEn     : 1,
            uint uiTxTimAlIntEn   : 1,
            uint uiSerAlIntEn     : 1,
            uint uiTtSerPrtIntEn  : 1,     
uint :7)
    } tCnfg;
    
}ixf18100_WisIntCfg_t;



/* APS/WIS Interrupt configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4(uint       : 3,
            uint uiRxLohIntEn     : 1,       
            uint uiFrmOhpIntEn    : 1,
uint :11)
    } tCnfg;
    
}ixf18100_ApsWisIntCfg_t;

/* Receive line Path OverHead Interrupt Cnfg */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_16(uint uiVcAisIntEn       : 1,
            uint uiPlmpIntEn        : 1,
            uint uiUnEqpIntEn       : 1,
            uint uiC2UnstableIntEn  : 1,
            uint uiPDipChngIntEn    : 1,
            uint uiC2ChngIntEn      : 1,
            uint uiB3BlkOvFlwIntEn  : 1,
            uint uiB3BipOvFlwIntEn  : 1,
            uint uiReiBlkOvFlwIntEn : 1,
            uint uiReiBipOvFlwIntEn : 1,
            uint uiRDipChngIntEn    : 1,
            uint uiZ4K3ChngIntEn    : 1,
            uint uiF2ChngIntEn      : 1,
            uint uiH4ChngIntEn      : 1,
            uint uiZ3F3ChngIntEn    : 1,
            uint uiZ5N1ChngIntEn    : 1)
    } tCnfg;
    
}ixf18100_RlPohIntCfg_t;


/*Receive Line J1 Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_5(uint uiJ1Crc7ErrIntEn   : 1,
            uint uiJ1MstMtchIntEn   : 1,
            uint uiJ1UnstableIntEn  : 1,     
            uint uiJ1AllZeroIntEn   : 1,     
uint :12)
      
    } tCnfg;
    
}ixf18100_RlJ1IntCfg_t;



/* Receive Line Pointer Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_10(uint uiAispIntEn       : 1,
            uint uiLopIntEn        : 1,
uint : 1,
       uint uiLnNdfIntEn      : 1,
       uint uiLnNjoIntEn      : 1,
       uint uiLnPjoIntEn      : 1,
uint : 4,
       uint uiAispcIntEn      : 1,
       uint uiLopcIntEn       : 1,
uint :4)
    } tCnfg;
    
}ixf18100_RlPtrIntCfg_t;


/* Receive Line OverHead Interrupt */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_16(
            uint uiAisLIntEn          : 1,
            uint uiExcB2ErrIntEn      : 1,
            uint uiRDilIntEn          : 1,
            uint uiSfIntEn            : 1,
            uint uiK2UnstableIntEn    : 1,
            uint uiK1UnstableIntEn    : 1,
            uint uiS1UnstableIntEn    : 1,
            uint uiReiLBlkOvFlwIntEn  : 1,
            uint uiReiLBipOvFlwIntEn  : 1,
            uint uiB2BlkOvFlwIntEn    : 1,
            uint uiB2BipOvFlwIntEn    : 1,
            uint uiZ4K3ChngIntEn      : 1,
            uint uiK2ChngIntEn        : 1,
            uint uiK1ChngIntEn        : 1,
            uint uiS1ChngIntEn        : 1,
uint :1)
    } tCnfg;
    
}ixf18100_RlLohIntCfg_t;



/* Receive Line Section OverHead*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_11(
            uint uiOofIntEn       : 1,
            uint uiLofIntEn       : 1,
            uint uiExtLosIntEn    : 1,      
            uint uiIntLosIntEn    : 1,      
            uint uiJ0CrcErrIntEn  : 1,
            uint uiJ0MsMtchIntEn  : 1,
            uint uiJ0UnstableIntEn: 1,
            uint uiB1BlkOvfIntEn  : 1,
            uint uiB1BipOvfIntEn  : 1,
            uint uiOofOvfIntEn    : 1,
uint :6)
    } tCnfg;
    
}ixf18100_RlSohIntCfg_t;

/* System Clock and Frame Pulse */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4( uint uiSysLofpIntEn      : 1,
            uint uiSysFpErrIntEn     : 1,
            uint uiSysFbAlignIntEn   : 1,
uint :13)
    } tCnfg;
    
}ixf18100_TxTimAlIntCfg_t;

/*Serial alarm configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_9 ( uint uiTalDataChIntEn: 1,
            uint uiTalLofpIntEn  : 1,
            uint uiTalLocIntEn   : 1,      
            uint uiTalCrcIntEn   : 1,      
            uint uiPalDataChIntEn: 1,
            uint uiPalLofpIntEn  : 1,
            uint uiPalLocIntEn   : 1,
            uint uiPalCrcIntEn   : 1,
uint :8)
    } tCnfg;
    
}ixf18100_SerAlIntCfg_t;




/*Transmit Transport Serial Interrupt Configuration */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_3(
            uint uiTtSerPrt1IntEn: 1,
            uint uiTtSerPrt2IntEn: 1,
uint :14)
    } tCnfg;
    
}ixf18100_TtSerPrtIntCfg_t;


/*Transmit Transport Serial Interrupt Configuration Part 1 */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_15(
            uint uiTalGenRdilIntEn    : 1,
            uint uiTalLosIntEn        : 1,
            uint uiTalOofIntEn        : 1,
            uint uiTalLofIntEn        : 1,
            uint uiTalJ0mmIntEn       : 1,
            uint uiTalJ0UnstableIntEn : 1,
            uint uiTalJ0Crc7ErrIntEn  : 1,
            uint uiTalDetAislIntEn    : 1,
            uint uiTalGenAislIntEn    : 1,
            uint uiTalDrstAisIntEn    : 1,
            uint uiTalDmstAisIntEn    : 1,
            uint uiTalDteRdilIntEn    : 1,
            uint uiTalS1UnstableIntEn : 1,
            uint uiTalS1ChngIntEn     : 1,
uint :2)
    } tCnfg;
    
}ixf18100_TTSerPrt1IntCfg_t;


/*Transmit Transport Serial Interrupt Configuration Part 2 */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_15(
            uint uiTalApsCntlIntEn    : 1,
            uint uiTalApsLinkFailIntEn: 1,
            uint uiTalUpcClkFailIntEn : 1,
            uint uiTalSelTxClkFailIntEn:1,
            uint uiTalGp0IntEn        : 1,
            uint uiTalGp1IntEn        : 1,
            uint uiTalGp2IntEn        : 1,
            uint uiTalSfIntEn         : 1,
            uint uiTalEEdenIntEn      : 1,
            uint uiTalDsdIntEn        : 1,
            uint uiTalK1UnstableIntEn : 1,
            uint uiTalK2UnstableIntEn : 1,
            uint uiTalK1ChngIntEn     : 1,
            uint uiTalK2ChngIntEn     : 1,
uint :2)
    } tCnfg;
    
}ixf18100_TTSerPrt2IntCfg_t;

/* XGMAC Interrupt  Configuration 
*/
typedef union
{
    ushort uiReg;
    struct
    {       
        BITFIELD_16(
            uint uiOvflwOctTxOkEn           : 1,
            uint uiOvflwMultCastFrmTxOkEn   : 1,
            uint uiOvflwBrdCastFrmTxOkEn    : 1,
            uint uiOvflwFrmTxOkEn           : 1,
            uint uiOvflw64OctTxOkEn         : 1,
            uint uiOvflw65to127FrmTxOkEn    : 1,
            uint uiOvflw128to255FrmTxOkEn   : 1,
            uint uiOvflw256to511FrmTxOkEn   : 1,
            uint uiOvflw512to1023FrmTxOkEn  : 1,
            uint uiOvflw1024to15xxFrmTxOkEn : 1,
            uint uiOvflw15xxtoMaxFrmTxOkEn  : 1,
            uint uiOvflwVLanFrmTxOkEn       : 1,
            uint uiOvflwPauseCtrlFrmTxOkEn  : 1,
            uint uiOvflwUniCastFrmTxOkEn    : 1,
            uint uiOvflwMAcCtrlFrmTxOkEn    : 1,
            uint uiOvflwOctRxOkEn           : 1)
    } tCnfg;
    
}ixf18100_XgmacPart1IntCfg_t;

typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_16(
            uint uiOvflwMultCastFrmRxOkEn   : 1,
            uint uiOvflwBrdCastFrmRxOkEn    : 1,
            uint uiOvflwFrmRxOkEn           : 1,
            uint uiOvflw64OctRxOkEn         : 1,
            uint uiOvflw65to127FrmRxOkEn    : 1,
            uint uiOvflw128to255FrmRxOkEn   : 1,
            uint uiOvflw256to511FrmRxOkEn   : 1,
            uint uiOvflw512to1023FrmRxOkEn  : 1,
            uint uiOvflw1024to15xxFrmRxOkEn : 1,
            uint uiOvflw15xxtoMaxFrmRxOkEn  : 1,
            uint uiOvflwVLanFrmRxOkEn       : 1,
            uint uiOvflwPauseCtrlFrmRxOkEn  : 1,
            uint uiOvflwUniCastFrmRxOkEn    : 1,
            uint uiOvflwMAcCtrlFrmRxOkEn    : 1,
            uint uiOvflwEtherStsUnSizePktsEn: 1,
            uint uiOvflwEtherStsOvSizePktsEn: 1)
    } tCnfg;
    
}ixf18100_XgmacPart2IntCfg_t;

typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_6(
            uint uiOvflwEtherStsOctsEn  : 1,
            uint uiOvflwEtherStsPktsEn  : 1,
            uint uiOvflwEtherStsFrgsEn  : 1,
            uint uiOvflwEtherStsJabbEn  : 1,
            uint uiOvflwEtherFrmChkErrEn: 1,
uint :11)
      
    } tCnfg;
    
}ixf18100_XgmacPart3IntCfg_t;



typedef struct
{
    ixf18100_XgmacPart1IntCfg_t tReg1;
    ixf18100_XgmacPart2IntCfg_t tReg2;
    ixf18100_XgmacPart3IntCfg_t tReg3;
    
}ixf18100_XgmacIntCfg_t;



typedef  struct /* Alarm Configuration */
{   
    /* Global Level alarms */
    ixf18100_GlbChpIntCfg_t         GlobalChipAlarms;
    ixf18100_GlbClkIntCfg_t         GlobalClkAlarms;
    ixf18100_GlbIntCfg_t            GlobalIntAlarms;
    ixf18100_PosRcvIntCfg_t         PosRcvIntAlarms;
    ixf18100_PosXmtIntCfg_t         PosXmtIntAlarms;
    ixf18100_SifIntCfg_t            SifIntAlarms;
    ixf18100_PcsIntCfg_t            PcsIntAlarms;
    ixf18100_GfpIntCfg_t            GfpIntAlarms;
    ixf18100_WisIntCfg_t            WisIntAlarms;
    ixf18100_RlPohIntCfg_t          RlPohIntAlarms;
    ixf18100_RlJ1IntCfg_t           RlJ1IntAlarms;
    ixf18100_RlPtrIntCfg_t          RlPtrIntAlarms;
    ixf18100_RlLohIntCfg_t          RlLohIntAlarms;
    ixf18100_RlSohIntCfg_t          RlSohIntAlarms;
    ixf18100_TxTimAlIntCfg_t        TxTimAlIntAlarms;
    ixf18100_SerAlIntCfg_t          SerAlIntAlarms;
    ixf18100_TtSerPrtIntCfg_t       TTSerPrtIntAlarms;
    ixf18100_TTSerPrt1IntCfg_t      TTSerPrt1IntAlarms;
    ixf18100_TTSerPrt2IntCfg_t      TTSerPrt2IntAlarms;
    ixf18100_ApsWisIntCfg_t         ApsWisIntCfg;
    ixf18100_RlLohIntCfg_t          ApsRlLohIntCfg;
    ixf18100_RlSohIntCfg_t          ApsRlSohIntCfg;
    ixf18100_XgmacIntCfg_t          XgmacIntCfg;    
    
} ixf18100_AlarmCfgSet_t;


/* Collected Alarm Config - for chip */
typedef struct 
{
    boolean                  bAlarmsUsed;
    boolean                  bAlarmCallbackAssigned;
    AlarmCallBack            pAlarmCallback;
    ixf18100_AlarmCfgSet_t*  pAlarmCfg; 
}  ixf18100_AlarmCfg_t;





/** End of Alarm Configuration Structures **/





/****************************************************************************** 
* These enums are used to identify the individual alarm and status
* for the callback functions, 
*****************************************************************************/

typedef enum
{  
    /* ALARMS CALLBACK */
    ixf18100_eGLOBAL_CHIP_ALARM = e1810x_ALARM_BASE,               
        ixf18100_eGLOBAL_CLOCK_ALARM,
        ixf18100_eGLOBAL_ALARM,      
        ixf18100_eSIF_ALARM,         
        ixf18100_eMAC1_ALARM,        
        ixf18100_eMAC2_ALARM,        
        ixf18100_ePCS_ALARM,         
        ixf18100_eGFP_ALARM,         
        ixf18100_ePOS_GLB_ALARM,
        ixf18100_ePOS_RCV_ALARM,     
        ixf18100_ePOS_XMT_ALARM,     
        ixf18100_eWATCHDOG_ALARM,    
        ixf18100_eWIS_ALARM,         
        ixf18100_eWIS_APS_ALARM,     
        ixf18100_eRL_POH_ALARM,
        ixf18100_eRL_J1_ALARM,              
        ixf18100_eRL_PTR_ALARM,
        ixf18100_eRL_LOH_ALARM,
        ixf18100_eRL_SOH_ALARM,
        ixf18100_eFRM_AL_ALARM,
        ixf18100_eTX_TIM_AL_ALARM,
        ixf18100_eSER_AL_ALARM,
        ixf18100_eTT_SER_PRT_ALARM,
        ixf18100_eTT_SER_PRT1_ALARM,
        ixf18100_eTT_SER_PRT2_ALARM,
        ixf18100_eAPS_RL_LOH_ALARM,
        ixf18100_eAPS_RL_SOH_ALARM,
        ixf18100_eSONET_SOH_COUNTERS,
        ixf18100_eSONET_LOH_COUNTERS,
        ixf18100_eSONET_PT_COUNTERS,
        ixf18100_eSONET_POH_COUNTERS,
        ixf18100_eSONET_APS_COUNTERS,
        ixf18100_eSPI4_RX_COUNTERS,
        ixf18100_eSPI4_TX_COUNTERS,
        ixf18100_ePOS_RX_COUNTERS,
        ixf18100_ePOS_TX_COUNTERS,
        ixf18100_eGFP_RX_COUNTERS,
        ixf18100_eGFP_TX_COUNTERS,
        ixf18100_eXGMAC_RX_COUNTERS,
        ixf18100_eXGMAC_TX_COUNTERS,
        ixf18100_ePCS_COUNTERS,
        
} ixf18100_AlarmCallbackType_e;


/** End of Alarm Type callback **/



/*****************************************************************************
* Alarm status structures:
*    These structures will serve as holders of the status of the alarms.
*    The structures are divided in a bitfield structure and a register, so every 
* member of structure is really a direct mapping of one of the bits of the 
* register, so we can manipulate individual bits without the use of shifts 
* and/or masks.
*     We are using a union that make us possible read/write the contents as a whole 
* register, or read/write individual bits. Example:
*
*   typedef union
*    {
*        uchar uiRegbyte;  --This is our 8 bits register
*        struct {
*           uchar bit0     :1; -- Least significant bit mapped
*           uchar bit1     :1;
*           uchar bit2     :1;
*           uchar bit3     :1;
*           uchar bit4to7  :4; -- The four most significant bits
*         } tBitField;
*     } regByte_t;
*
*   .
*   .
*   .
*   regByte_t register;
*
*   register.uiRegByte = 0x00;
*   register.tBitField.bit0 = 1; -- Now uiRegByte is 0x01
*   register.tBitField.bit4to7 = 0xF; -- Now uiRegByte is 0xF1;
*   register.tBitFiled.bit3 = 1; -- uiRegByte is 0xF9... etc.
*****************************************************************************/

/* Chip Alarm Status Structure, this structure will hold the status value of the 
* present alarms at the chip level
*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_12(uint uiPosIntSt   : 1,  /* Pos interrupt enable/disable */
            uint uiSifIntSt   : 1,  /* SIF/SPI4 interrupt enabgle/disable */
            uint uiMacIntSt  : 1,  /* First part of the MAC interrupt configuration */
uint : 1,  /* Second part of the MAC interrupt configuration */
       uint uiPcsIntSt   : 1,  /* PCS Interrupt Configuration Enable/Disable */
       uint uiGfpIntSt   : 1,  /* GFP Interrupt Configuration Enable/Disable */
       uint uiWisIntSt   : 1,  /* SONET/WIS Interrupt Configuration */
       uint uiWisApsIntSt: 1,/* APS/WIS Interrupt Configuration */
       uint uiGlbIntSt   : 1, /* Global Interrupt configuration */
       uint uiWatchIntSt : 1, /* WatchDog Interrupt Configuration */ 
       uint uiClkIntSt   : 1, /* Clock Interrupt Configuration */ 
uint : 5 )
    } tCnfg;
} ixf18100_GlbChpIntStat_t;


/* POS Status structure this structure will hold the value of which
* Side (tx/rx) is originating the interrupt
*/
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_3(uint uiXmtIntAct   : 1,
            uint uiRcvIntAct   : 1,
uint : 14)
    } tCnfg;
    
}ixf18100_PosIntSrc_t;

/* POS Receive side interrupt status */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_16( 
            uint uiRcvFcsiIntSt         : 1,
            uint uiRcvAbortIntSt        : 1,
            uint uiRcvAciIntSt          : 1,
            uint uiRcvDblEsclIntSt      : 1,
            uint uiRcvMinPliIntSt       : 1,
            uint uiRcvMaxPliIntSt       : 1,
            uint uiRcvSfiIntSt          : 1,
            uint uiRcvFcsCntlIntSt      : 1,
            uint uiRcvAfcCntlIntSt      : 1,
            uint uiRcvGfcCntlIntSt      : 1,
            uint uiRcvGfbByteIntSt      : 1,
            uint uiRcvMinPlcCntlIntSt   : 1,
            uint uiRcvMaxPlcCntlIntSt   : 1,
            uint uiRcvAfbByteCntlIntSt  : 1,
            uint uiRcvFcseByteCntlIntSt : 1,
uint :1)
    } tCnfg;
    
}ixf18100_PosRcvIntStat_t;


/* POS Transmit Side Interrupt status */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4( uint uiXmtFcsiIntSt      : 1,
            uint uiXmtFrmCntlIntSt   : 1,
            uint uiXmtByteCntlIntSt  : 1,
uint :13)   
    } tCnfg;
    
}ixf18100_PosXmtIntStat_t;



/* SIF/SPI4 Interrupt Status */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_9(uint uiTxBusErrIntSt     : 1,
            uint uiTxParErrIntSt     : 1,
            uint uiTxFullErrIntSt    : 1,
            uint uiTxNoEopErrIntSt   : 1,
            uint uiRxBusErrIntSt     : 1,
            uint uiRxFullErrIntSt    : 1,
            uint uiRxNoEopErrIntSt   : 1,
            uint uiRxDip2FailErrIntSt: 1,
uint : 8)    
    } tCnfg;
    
}ixf18100_SifIntStat_t;


/* PCS Interrupt status */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_4(uint uiPcsTxFifoOvIntSt   : 1,
            uint uiPcsTxFifoUvIntSt   : 1,
            uint uiPcsRxFifoOvIntSt   : 1,
uint :13)        
    } tCnfg;
    
}ixf18100_PcsIntStat_t;


/* SONET/WIS interrupt status */
typedef union
{
    ushort uiReg;
    struct{
        BITFIELD_10( uint uiRxPohIntSt     : 1, 
            uint uiRxJ1IntSt      : 1,
            uint uiRxPtIntSt      : 1,
            uint uiRxLohIntSt     : 1,     
            uint uiFrmOhpIntSt    : 1,
            uint uiFrmAlIntSt     : 1,
            uint uiTxTimAlIntSt   : 1,
            uint uiSerAlIntSt     : 1,
            uint uiTtSerPrtIntSt  : 1,     
uint :7)       
    } tCnfg;
    
}ixf18100_WisIntStat_t;


/* APS/WIS Interrupt status */
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_4(uint       : 3,
            uint uiRxLohIntSt     : 1,       
            uint uiFrmOhpIntSt    : 1,
uint :11)        
    } tCnfg;
    
}ixf18100_ApsWisIntStat_t;

/* Receive Line Path Overhead Interrupt Status Structure */
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_16(uint uiVcAisIntSt       : 1,
            uint uiPlmpIntSt        : 1,
            uint uiUnEqpIntSt       : 1,
            uint uiC2UnstableIntSt  : 1,
            uint uiPDipChngIntSt    : 1,
            uint uiC2ChngIntSt      : 1,
            uint uiB3BlkOvFlwIntSt  : 1,
            uint uiB3BipOvFlwIntSt  : 1,
            uint uiReiBlkOvFlwIntSt : 1,
            uint uiReiBipOvFlwIntSt : 1,
            uint uiRDipChngIntSt    : 1,
            uint uiZ4K3ChngIntSt    : 1,
            uint uiF2ChngIntSt      : 1,
            uint uiH4ChngIntSt      : 1,
            uint uiZ3F3ChngIntSt    : 1,
            uint uiZ5N1ChngIntSt    : 1)
    } tCnfg;
    
}ixf18100_RlPohIntStat_t;


/* Receive Line Pointer Interrupt Status structure*/
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_10(uint uiAispIntSt       : 1,
            uint uiLopIntSt        : 1,
uint : 1,
       uint uiLnNdfIntSt      : 1,
       uint uiLnNjoIntSt      : 1,
       uint uiLnPjoIntSt      : 1,
uint : 4,
       uint uiAispcIntSt      : 1,
       uint uiLopcIntSt       : 1,
uint :4)
    } tCnfg;
    
}ixf18100_RlPtrIntStat_t;

/* Receive Line Line OverHead*/
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_16( 
            uint uiAisLIntSt          : 1,
            uint uiExcB2ErrIntSt      : 1,
            uint uiRDilIntSt          : 1,
            uint uiSfIntSt            : 1,
            uint uiK2UnstableIntSt    : 1,
            uint uiK1UnstableIntSt    : 1,
            uint uiS1UnstableIntSt    : 1,
            uint uiReiLBlkOvFlwIntSt  : 1,
            uint uiReiLBipOvFlwIntSt  : 1,
            uint uiB2BlkOvFlwIntSt    : 1,
            uint uiB2BipOvFlwIntSt    : 1,
            uint uiZ4K3ChngIntSt      : 1,
            uint uiK2ChngIntSt        : 1,
            uint uiK1ChngIntSt        : 1,
            uint uiS1ChngIntSt        : 1,
uint :1)
    } tCnfg;
    
}ixf18100_RlLohIntStat_t;


/* Receive Line Section OverHead */
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_11(
            uint uiOofIntSt       : 1,
            uint uiLofIntSt       : 1,
            uint uiExtLosIntSt    : 1,      
            uint uiIntLosIntSt    : 1,      
            uint uiJ0CrcErrIntSt  : 1,
            uint uiJ0MsMtchIntSt  : 1,
            uint uiJ0UnstableIntSt: 1,
            uint uiB1BlkOvfIntSt  : 1,
            uint uiB1BipOvfIntSt  : 1,
            uint uiOofOvfIntSt    : 1,
uint :6)
    } tCnfg;
    
}ixf18100_RlSohIntStat_t;


/* Transmit transport Serial Alarm Interrupt Status*/
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_3(
            uint uiTtSerPrt1IntSt: 1,
            uint uiTtSerPrt2IntSt: 1,
uint :14)
    } tCnfg;
    
}ixf18100_TtSerPrtIntStat_t;


/* Transmit transport Serial Alarm Interrupt Status part 1*/
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_15(
            uint uiTalGenRdilIntSt    : 1,
            uint uiTalLosIntSt        : 1,
            uint uiTalOofIntSt        : 1,
            uint uiTalLofIntSt        : 1,
            uint uiTalJ0mmIntSt       : 1,
            uint uiTalJ0UnstableIntSt : 1,
            uint uiTalJ0Crc7ErrIntSt  : 1,
            uint uiTalDetAislIntSt    : 1,
            uint uiTalGenAislIntSt    : 1,
            uint uiTalDrstAisIntSt    : 1,
            uint uiTalDmstAisIntSt    : 1,
            uint uiTalDteRdilIntSt    : 1,
            uint uiTalS1UnstableIntSt : 1,
            uint uiTalS1ChngIntSt     : 1,
uint :2)
    } tCnfg;
    
}ixf18100_TTSerPrt1IntStat_t;


/* Transmit transport Serial Alarm Interrupt Status part 2*/
typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_15(
            uint uiTalApsCntlIntSt    : 1,
            uint uiTalApsLinkFailIntSt: 1,
            uint uiTalUpcClkFailIntSt : 1,
            uint uiTalSelTxClkFailIntSt:1,
            uint uiTalGp0IntSt        : 1,
            uint uiTalGp1IntSt        : 1,
            uint uiTalGp2IntSt        : 1,
            uint uiTalSfIntSt         : 1,
            uint uiTalEEdenIntSt      : 1,
            uint uiTalDsdIntSt        : 1,
            uint uiTalK1UnstableIntSt : 1,
            uint uiTalK2UnstableIntSt : 1,
            uint uiTalK1ChngIntSt     : 1,
            uint uiTalK2ChngIntSt     : 1,
uint :2)
    } tCnfg;
    
}ixf18100_TTSerPrt2IntStat_t;


/** End of alarm status structures  **/ 



/**************************************************************************
* Alarm and counter values structures
*     This are the structure that holds the value of the interrupts, 
* including counters. These structure will be passed at the callback 
* functions as void pointers, is responsability of the user to cast
* to the proper structure data type. 
****************************************************************************/



/*
* SPI4/SIF alarm values
*/

/* SIF/SPI4 Counters */
typedef struct
{
    ulong txBusErrCnt;
    ulong txParErrCnt;
    ulong txFullErrCnt;
    ulong txNoEopErrCnt;
    ulong rxBusErrCnt;
    ulong rxFullErrCnt;
    ulong rxNoEopErrCnt;
    ulong rxDip2FailErrCnt;     
} ixf18100Spi4Counters;

/* Structure passed to the Protocol to handle the SIF/SPI4 alarms */
typedef struct
{
    ixf18100_SifIntStat_t    alarmsPresent;
    ixf18100Spi4Counters   counters;
}ixf18100Spi4IntStatus;




/*
*  RL POH Alarm status structure
*/
typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_4(
            uint G1SpBit: 1,
            uint RdiBits: 3,
            uint ReiBits: 4,
uint : 8)
    } cnfg;
    
}ixf18100_RlRcvG1_t;

typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_6( uint VcAisSt   : 1,
            uint PlmpSt    : 1,
            uint UnEqpSt   : 1,
            uint C2UnStblSt: 1,
            uint PdipSt    : 1,
uint : 11)
       
    } cnfg;
    
}ixf18100_RlPohS_t;


typedef struct
{
    ixf18100_RlPohIntStat_t alarms;
    ushort Z5N1Byte;
    ushort Z3F3Byte;
    ushort H4Byte;
    ushort F2Byte;
    ushort Z4K3Byte;
    ushort C2Byte;
    ushort ReipBipCnt;
    ushort ReipBlkCnt;
    ushort B3BipCnt;
    ushort B3BlkCnt;
    ixf18100_RlRcvG1_t RlRcvG1;
    ixf18100_RlPohS_t RlPohStat;
    
} ixf18100_RxPohStatus_t;



/*
*   Alarm General purpose status structure
*/


typedef struct
{
    ushort alarms;
    ushort status;
} ixf18100_GenPurposeIntStatus_t;



/*
*  Pointer alarm status data structures
*/

typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_6(
            uint AispSt     : 1,
            uint LopSt      : 1,
uint : 8,
       uint AispcSt    : 1,
       uint LopcSt     : 1,
uint : 4)
    } cnfg;
    
}ixf18100_RlPtrSlotS_t;



typedef struct
{
    ixf18100_RlPtrIntStat_t alarms;
    ushort LnPjCntOvflw;
    ushort LnNjCntOvflw;
    ixf18100_RlPtrSlotS_t RlPtrStat;
    
} ixf18100_RlPtrStatus_t;

/*  Rx Line OverHead Holding alarm values structures */

typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_9(
            uint AislSt      : 1,
            uint ExcB2ErrSt  : 1,
            uint RdilSt      : 1,
            uint SfSt        : 1,
            uint DegB2ErrSt  : 1,
            uint K2UnstableSt: 1,
            uint K1UnstableSt: 1,
            uint S1UnstableSt: 1,
uint : 8)
    } cnfg;
    
}ixf18100_RlLohS_t;

typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_2(
            uint K2Byte  : 8,
            uint K1Byte  : 8)
            
    } cnfg;
    
}ixf18100_RlK1K2_t;

typedef struct
{
    ixf18100_RlLohIntStat_t alarms;
    ulong ReiBlkOvflw;
    ulong ReilBipOvflw;
    ulong B2BlkOvflw;
    ulong B2BipOvflw;
    ushort S1Byte;
    ixf18100_RlK1K2_t K1K2Bytes;
    ixf18100_RlLohS_t RlLohStat;
    
} ixf18100_RlLohStatus_t;



/* Rx Section OverHead alarm values structures */
typedef union
{
    ushort reg;
    struct
    {
        BITFIELD_7(
            uint OofSt        : 1,
            uint LofSt        : 1,
            uint ExtLosSt     : 1,
            uint J0Crc7ErrSt  : 1,
            uint J0MstMtchSt  : 1,
            uint J0UnstableSt : 1,
uint :10)       
    } cnfg;
    
}ixf18100_RlSohS_t;



typedef struct
{
    ixf18100_RlSohIntStat_t alarms;
    ushort OofOvflw;
    ushort B1BipOvflw;
    ushort B1BlkOvflw;
    ixf18100_RlSohS_t RlSohStat;
    
} ixf18100_RlSohStatus_t;


/* Serial Port Alarm status daata structures */

typedef struct
{
    ixf18100_TTSerPrt1IntStat_t alarms;
    ushort                      TTSerPrt1IntStat;
} ixf18100_TTSerPrt1Status_t;

typedef struct
{
    ixf18100_TTSerPrt2IntStat_t alarms;
    ushort                      TTSerPrt2IntStat;  
} ixf18100_TTSerPrt2Status_t;


/* POS Alarm Values and counters */
typedef struct{ /* Pos Tx Counters */
    
    ushort rxMaxPleCnt;
    ulong  rxFcsByteErrCnt;
    ulong  rxAfByteErrCnt;
    ulong  rxMinPleCnt;
    ulong  rxGfByteCnt;
    ulong  rxGfCnt;
    ulong  rxAfCnt;
    ulong  rxFcsCnt;
    
}ixf18100_PosRxCounters_t;

/* Pos Receive Interrupt Value Register */
typedef struct
{
    ixf18100_PosRcvIntStat_t alarms;
    ixf18100_PosRxCounters_t counters;  
} ixf18100_PosRxAlarmStatus_t;

typedef struct{ /* Pos Tx Counters */
    ulong  txByteCnt;
    ulong  txFrmCnt;
}ixf18100_PosTxCounters_t;


typedef struct
{
    ixf18100_PosXmtIntStat_t alarms;
    ixf18100_PosTxCounters_t counters;  
} ixf18100_PosTxAlarmStatus_t;

typedef struct{
    uchar rxCtrlFrmCnt;
    uchar rxThecErrCnt;
    uchar rxEhecErrCnt;
    uchar rxFcsErrCnt;
    ulong rxOverSizeFrmCnt;
} ixf18100_GfpCounters_t;

typedef struct{
    ushort                   glbStatus;
    ushort                   status;
    ixf18100_GfpCounters_t   counters;
}ixf18100_GfpAlarmStatus_t;


/** End of value structures **/
#endif

