/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:03p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_global.c $
*   WORKFILE:       $Workfile: ixf18100_global.c $
*
* --------------------------------------------------------------------------
* Project: ixf18100 family device API
*
* Purpose: This file contains Global block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_global.c $
 * 
 * 1     3/11/04 3:03p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 27    2/20/03 3:19p Kssaepha
* Changed ifndef logic to ifdef to allow a combination of ixf1810x's to
* be defined
* 
* 26    2/07/03 3:05p Pkjain2
* Fixed a bug in SW Workaround when changing the Mode to Ethernet LAN/WAN
* 
* 25    1/30/03 4:40p Pkjain2
* Made changes for PTQ release. Added Getstatus calls for SONET and
* GLOBAL block.
* 
* 24    1/10/03 2:30p Pkjain2
* 
* 23    11/12/02 2:25p Jaalliso
* ixf18102 conditional compile debug
* 
* 22    11/11/02 11:58a Jaalliso
* Add check for ixf18102 build for SW_WORKAROUND
* 
* 21    8/12/02 5:30p Machambe
* Adding IXF1810x Family support
* 
* 20    7/11/02 4:21p Pkjain2
* Corrected some headers. Added Legal notice to some files. 
* 
* 19    4/16/02 4:05p Jaalliso
* Fixing #defines for Opmodes
* 
* 18    4/15/02 5:00p Jaalliso
* #define for ixf18100_ETHERNET_MODE changing
* 
* 17    4/10/02 7:00p Jaalliso
* SW Workaround for wrong default value in XGMAC test register
* 
* 16    4/10/02 5:42p Pkjain2
* 
* 15    4/09/02 11:02a Jaalliso
* Fixed comments, removed tabs, general code formatting fixes
* 
* 14    3/12/02 11:58a Srodrig1
* 
* 13    2/27/02 3:24p Pkjain2
* 
* 12    2/27/02 3:15p Srodrig1
* 
* 11    2/12/02 11:10a Jaalliso
* Renaming loopback modes
* 
* 10    2/11/02 4:07p Srodrig1
* 
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf18100_global_d.h"
#include "ixf18100_global.h"
#include "ixf18100_global_b.h"
/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
defined(INCLUDE_18103_LIB)
#include "ixf18100_sonet_b.h"
#endif
#include "ixf18100_util.h"
#include "ixf18100.h"
#include "ixf18100_d.h"
#include "hal.h"


/******************************************************************************
* Function : Ixf18100EditGlbCnfg
*------------------------------------------------------------------------------
* Description : Global configuration functions, the mode combinations are 
*               mutually exclusive.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : Pointer to values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf18100EditGlbCnfg(bb_RegPointer_type BaseAddress, ixf18100_GlbCnfg_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_CFG1_OFFSET,
        (ushort)pCnfg->glbCnfgReg);
    
#if defined(IXF1810X_SW_WORKAROUND) && !defined(INCLUDE_18102_LIB)
        /* This is a workaround for the correcting the default value in XGMAC registers
    when IXF1810X is set in Ethernet mode. */
    mode = ((pCnfg->glbCnfgReg & ixf18100_mMODE_SEL) >> ixf18100_sMODE_SEL);
    
    if (mode == ixf18100_vMODE_SEL_ETHERNET)
    {
        HAL_IXF18100_WRITE(BaseAddress + 0x3116, 0x0000);
        HAL_IXF18100_WRITE(BaseAddress + 0x3117, 0x0084);
    }
#endif /* defined(IXF1810X_SW_WORKAROUND) && !defined(INCLUDE_18102_LIB) */
    
    Ixf18100HoldSwReset(BaseAddress); /*the device has to be reset */
    
    Ixf18100ReleaseSwReset(BaseAddress);/*to acceopt the changes*/
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetGlbCnfg
*------------------------------------------------------------------------------
* Description : This function returns the mode configuration.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : The pointer of the structure to return values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetGlbCnfg(bb_RegPointer_type BaseAddress, ixf18100_GlbCnfg_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->glbCnfgReg = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_CFG1_OFFSET);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100EditGlbReset
*------------------------------------------------------------------------------
* Description : This function is used to reset One/All blocks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : Values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100EditGlbReset(bb_RegPointer_type BaseAddress,
                     ixf18100_GlbReset_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->glbResetReg &= ixf18100_mGLB_RST;
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_RST_OFFSET,
        (ushort)pCnfg->glbResetReg);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetGlbReset
*------------------------------------------------------------------------------
* Description : Gets reset configuration for all the blocks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer of the structure to store the value
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetGlbReset(bb_RegPointer_type BaseAddress,
                    ixf18100_GlbReset_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->glbResetReg = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_RST_OFFSET);
    
    pCnfg->glbResetReg &= ixf18100_mGLB_RST;
    
    return bb_NO_ERROR;
    
}

/******************************************************************************
* Function : Ixf18100HoldSwReset
*------------------------------------------------------------------------------
* Description : This function will set the Sw reset bit to 1 to reset all the
*               blocks, to release the reset use the counterpart function
*               Ixf18100ReleaseSwReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100HoldSwReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress,&resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.sftRst = ixf18100_eRESET;
    
    return Ixf18100EditGlbReset(BaseAddress,&resetCnfg);
}

/******************************************************************************
* Function : Ixf18100ReleaseSwReset
*------------------------------------------------------------------------------
* Description : This function will set the SW reset bit to 0 to release the
*               reset on all the blocks.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100ReleaseSwReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress,&resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.sftRst = ixf18100_eNORMAL;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
}

/******************************************************************************
* Function : Ixf18100HoldApsReset
*------------------------------------------------------------------------------
* Description : This function will set the APS reset bit to 1 to reset the APS
*               block, to release the reset use  the counterpart function
*               Ixf18100ReleaseApsReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100HoldApsReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.apsIntfRst = ixf18100_eRESET;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
}

/******************************************************************************
* Function : Ixf18100ReleaseApsReset
*------------------------------------------------------------------------------
* Description : This function will set the APSreset bit to 0 to release the
*               reset on the APS block.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100ReleaseApsReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.apsIntfRst = ixf18100_eNORMAL;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
    
}

/******************************************************************************
* Function : Ixf18100HoldLnIntfReset
*------------------------------------------------------------------------------
* Description : This function will set the Line Intf reset bit to 1 to reset
*               the Line Interface. To release the reset use the counterpart
*               function Ixf18100ReleaseLnIntfReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100HoldLnIntfReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.lineIntfRst = ixf18100_eRESET;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
    
}

/******************************************************************************
* Function : Ixf18100ReleaseLnIntfReset
*------------------------------------------------------------------------------
* Description : This function will set the Line Intf reset bit to 0 to release
*               the reset on the Line Interface.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100ReleaseLnIntfReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.lineIntfRst = ixf18100_eNORMAL;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
}

/******************************************************************************
* Function : Ixf18100HoldSpiReset
*------------------------------------------------------------------------------
* Description : This function will set the SPI reset bit to 1 to reset the SPI
*               block. To release the reset use  the counterpart function
*               Ixf18100ReleaseSpiReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100HoldSpiReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.spiIntfRst = ixf18100_eRESET;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
}

/******************************************************************************
* Function : Ixf18100ReleaseSpiReset
*------------------------------------------------------------------------------
* Description : This function will set the SPI reset bit to release the reset
*               on the SPI block. 
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100ReleaseSpiReset(bb_RegPointer_type BaseAddress)
{
    bb_Error_e rc;
    ixf18100_GlbReset_t resetCnfg;
    
    rc = Ixf18100GetGlbReset(BaseAddress, &resetCnfg);
    
    if(rc != bb_NO_ERROR)
        return rc;
    
    resetCnfg.Cnfg.spiIntfRst = ixf18100_eNORMAL;
    
    return Ixf18100EditGlbReset(BaseAddress, &resetCnfg);
}

/******************************************************************************
* Function : Ixf18100EditWatchDogCnfg
*------------------------------------------------------------------------------
* Description : This function will enable/disable the watchdog and will set
*               the watchdog timer
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the values to be set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100EditWatchDogCnfg(bb_RegPointer_type BaseAddress,
                         ixf18100_GlbWatchDogInfo_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->watchDogCnfgRegister.reg &= ixf18100_mWATCHDG_TMR; 
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_WATCH_OFFSET,
        (ushort)pCnfg->watchDogCnfgRegister.reg);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetWatchDogCnfg
*------------------------------------------------------------------------------
* Description : This function will get the watchdog configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetWatchDogCnfg(bb_RegPointer_type BaseAddress,
                        ixf18100_GlbWatchDogInfo_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->watchDogCnfgRegister.reg
        = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_WATCH_OFFSET);
    
    pCnfg->watchDogCnfgRegister.reg &= ixf18100_mWATCHDG_TMR; 
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetWatchDogFailAdress
*------------------------------------------------------------------------------
* Description : This function will get the address for which the watchdog 
*               timer expired
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the structure to store failure address
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetWatchDogFailAdress(bb_RegPointer_type BaseAddress,
                              ixf18100_GlbWatchDogInfo_t *pCnfg)
{
    if(pCnfg == NULL)
        return bb_NULL_ARG;
    
    pCnfg->watchDogFailAddress
        = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_FAILADDR_OFFSET);
    
    pCnfg->watchDogFailAddress &= ixf18100_mWATCHDG_ADDR; 
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100OifSfiGetGlblLnIntf
*------------------------------------------------------------------------------
* Description : This function gets the Line Intf configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100OifSfiGetGlblLnIntf(bb_RegPointer_type BaseAddress,
                            ixf18100_GlobalLineInterfaceCnfg_t *cnfg)
{
    cnfg->lineCfg.reg = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_LINE_OFFSET);
    
    cnfg->rxDataInvMask = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_RXLNDATINV_OFFSET);
    
    cnfg->txDataInvMask = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_TXLNDATINV_OFFSET);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100OifSfiSetGlblLnIntf
*------------------------------------------------------------------------------
* Description : This function sets the Line Interface configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100OifSfiSetGlblLnIntf(bb_RegPointer_type BaseAddress,
                            ixf18100_GlobalLineInterfaceCnfg_t *pCnfg)
{
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_LINE_OFFSET,
        pCnfg->lineCfg.reg);
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_RXLNDATINV_OFFSET,
        pCnfg->rxDataInvMask);
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_TXLNDATINV_OFFSET,
        pCnfg->txDataInvMask);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetClkCnfg
*------------------------------------------------------------------------------
* Description : This function gets the clock configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e 
Ixf18100GetClkCnfg(bb_RegPointer_type BaseAddress,
                   ixf18100_GlbClkCnfg_t *cnfg)
{
    /* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
    uint RegValue;
    
    RegValue = HAL_IXF18100_READ(BaseAddress + ixf18100_WISDPCFG_OFFSET );
    
    if ((RegValue >> ixf18100_sRX_MUX_CFG) & 0x0001)
    {
        cnfg->reg1.uiReg = HAL_IXF18100_READ(BaseAddress
            + ixf18100_GLB_CLKCFG1_SHAD_OFFSET);
        
        cnfg->reg2.uiReg = HAL_IXF18100_READ(BaseAddress
            + ixf18100_GLB_CLKCFG2_SHAD_OFFSET);
    }
    else
#endif
    {
        cnfg->reg1.uiReg = HAL_IXF18100_READ(BaseAddress
            + ixf18100_GLB_CLKCFG1_OFFSET);
        
        cnfg->reg2.uiReg = HAL_IXF18100_READ(BaseAddress
            + ixf18100_GLB_CLKCFG2_OFFSET);
    }
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SetClkCfg
*------------------------------------------------------------------------------
* Description : This function sets the clock configuration for the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e 
Ixf18100SetClkCfg(bb_RegPointer_type BaseAddress, ixf18100_GlbClkCnfg_t *cnfg)
{
    /* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
    uint RegValue;
    
    RegValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_WISDPCFG_OFFSET);
    
    if ((RegValue >> ixf18100_sRX_MUX_CFG) & 0x0001)
    {
        HAL_IXF18100_WRITE(BaseAddress
            + ixf18100_GLB_CLKCFG1_SHAD_OFFSET, cnfg->reg1.uiReg);
        
        HAL_IXF18100_WRITE(BaseAddress
            + ixf18100_GLB_CLKCFG2_SHAD_OFFSET, cnfg->reg2.uiReg);
    }
    else
#endif
    {
        HAL_IXF18100_WRITE(BaseAddress
            + ixf18100_GLB_CLKCFG1_OFFSET, cnfg->reg1.uiReg);
        
        HAL_IXF18100_WRITE(BaseAddress
            + ixf18100_GLB_CLKCFG2_OFFSET, cnfg->reg2.uiReg);
    }
    
    return bb_NO_ERROR;
}

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
defined(INCLUDE_18103_LIB)
/******************************************************************************
* Function : Ixf18100SetMuxCfg
*------------------------------------------------------------------------------
* Description : This function sets configuration for various Muxes
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : MuxCfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100SetMuxCfg(bb_RegPointer_type BaseAddress, bb_ChipSegment_t *ptSegment,
                  bb_SelConfig_e  SelCfg, ixf18100_ApiMuxCfg_t *MuxCfg)
{
    uint RegValue;
    
    /* If SW Config then continue with configuration else set the TAL port
    and ignore rest of config */
    RegValue = HAL_IXF18100_READ(BaseAddress + ixf18100_WISDPCFG_OFFSET);
    
    /************************************************************************/
    /* Rx Mux Configuration*/
    if (MuxCfg->RxMuxCfg.PortCtrlCfg == ixf18100_eSW_CTRL)
    {
        if(MuxCfg->RxMuxCfg.ShadowSelCfg == ixf18100_eNORMAL_CFG)
        {
            if (MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_DATA)
            {
                /* Clr bit 0, 3, 6 */
                ClrBit(RegValue, (1 << ixf18100_sRX_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sRX_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sRX_MUX_CFG));
                
            }
            else if (MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_APS_DATA)
            {
                /* Clr bit 0, 3 and set bit 6 */
                ClrBit(RegValue, (1 << ixf18100_sRX_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sRX_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sRX_MUX_CFG));
            }
        }
        else if(MuxCfg->RxMuxCfg.ShadowSelCfg == ixf18100_eSHADOW_CFG)
        {
            if (MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_DATA)
            {
                /* Clr bit 0, 6 Set bit 3*/
                ClrBit(RegValue, (1 << ixf18100_sRX_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sRX_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sRX_MUX_CFG_SHAD));
                
            }
            else if (MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_APS_DATA)
            {
                /* Set bit 3, 6 and Clr bit 0 */
                ClrBit(RegValue, (1 << ixf18100_sRX_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sRX_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sRX_MUX_CFG_SHAD));
            }
        }
    }
    else if (MuxCfg->RxMuxCfg.PortCtrlCfg == ixf18100_eAPS_TAL_PORT_CTRL)
    {
        /* Ignore Config ? */
        SetBit(RegValue, (1 << ixf18100_sRX_SELMUX_CTRL));
    }
    
    /* Tx Mux Configuration*/
    if (MuxCfg->TxMuxCfg.PortCtrlCfg == ixf18100_eSW_CTRL)
    {
        if(MuxCfg->TxMuxCfg.ShadowSelCfg == ixf18100_eNORMAL_CFG)
        {
            if (MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_APS_DATA)
            {
                /* Clr bit 1, 4, 7 */
                ClrBit(RegValue, (1 << ixf18100_sTX_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sTX_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sTX_MUX_CFG));
                
            }
            else if (MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eTX_POH_OUTPUT_DATA)
            {
                /* Clr bit 1, 4 and set bit 7 */
                ClrBit(RegValue, (1 << ixf18100_sTX_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sTX_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sTX_MUX_CFG));
            }
        }
        else if(MuxCfg->TxMuxCfg.ShadowSelCfg == ixf18100_eSHADOW_CFG)
        {
            if (MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_APS_DATA)
            {
                /* Clr bit 1, 4 Set bit 7*/
                ClrBit(RegValue, (1 << ixf18100_sTX_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sTX_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sTX_MUX_CFG_SHAD));
                
            }
            else if (MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eTX_POH_OUTPUT_DATA)
            {
                /* Set bit 4, 7 and Clr bit 1 */
                ClrBit(RegValue, (1 << ixf18100_sTX_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sTX_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sTX_MUX_CFG_SHAD));
            }
        }
    }
    else if (MuxCfg->TxMuxCfg.PortCtrlCfg == ixf18100_eAPS_TAL_PORT_CTRL)
    {
        /* Ignore Config ? */
        SetBit(RegValue, (1 << ixf18100_sTX_SELMUX_CTRL));
    }
    
    /* Tx Aps Mux Configuration*/
    if (MuxCfg->TxApsMuxCfg.PortCtrlCfg == ixf18100_eSW_CTRL)
    {
        if(MuxCfg->TxApsMuxCfg.ShadowSelCfg == ixf18100_eNORMAL_CFG)
        {
            if (MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eTX_TOH_OUTPUT_DATA)
            {
                /* Clr bit 1, 4, 7 */
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_MUX_CFG));
                
            }
            else if (MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_DATA)
            {
                /* Clr bit 1, 4 and set bit 7 */
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_SELMUX_CTRL));
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sTXAPS_MUX_CFG));
            }
        }
        else if(MuxCfg->TxApsMuxCfg.ShadowSelCfg == ixf18100_eSHADOW_CFG)
        {
            if (MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eTX_TOH_OUTPUT_DATA)
            {
                /* Clr bit 1, 4 Set bit 7*/
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sTXAPS_CFGMUX_SWCTRL));
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_MUX_CFG_SHAD));
            }
            else if (MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal
                == ixf18100_eRX_DATA)
            {
                /* Set bit 4, 7 and Clr bit 1 */
                ClrBit(RegValue, (1 << ixf18100_sTXAPS_SELMUX_CTRL));
                SetBit(RegValue, (1 << ixf18100_sTXAPS_CFGMUX_SWCTRL));
                SetBit(RegValue, (1 << ixf18100_sTXAPS_MUX_CFG_SHAD));
            }
        }
    }
    else if (MuxCfg->TxApsMuxCfg.PortCtrlCfg == ixf18100_eAPS_TAL_PORT_CTRL)
    {
        /* Ignore Config ? */
        SetBit(RegValue, (1 << ixf18100_sTXAPS_SELMUX_CTRL));
    }
    
    if (MuxCfg->ClkMuxCfg.ClkSelMuxCtrl == ixf18100_eSW_CTRL)
    {
        ClrBit(RegValue, (1 << ixf18100_sCLK_SELMUX_CTRL));
    }
    else if (MuxCfg->ClkMuxCfg.ClkSelMuxCtrl == ixf18100_eAPS_TAL_PORT_CTRL)
    {
        SetBit(RegValue, (1 << ixf18100_sCLK_SELMUX_CTRL));
    }
    
    if (MuxCfg->ClkMuxCfg.ClkCfgSWCtrl == ixf18100_eNORMAL_CFG)
    {
        ClrBit(RegValue, (1 << ixf18100_sCLK_CFGMUX_SWCTRL));
    }
    else if (MuxCfg->ClkMuxCfg.ClkCfgSWCtrl == ixf18100_eSHADOW_CFG)
    {
        SetBit(RegValue, (1 << ixf18100_sCLK_CFGMUX_SWCTRL));
    }
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_WISDPCFG_OFFSET, RegValue);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetMuxCfg
*------------------------------------------------------------------------------
* Description : This function gets configuration for various Muxes
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : MuxCfg : The pointer to the structure to store the values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetMuxCfg(bb_RegPointer_type BaseAddress, bb_ChipSegment_t *ptSegment,
                  bb_SelConfig_e SelCfg, ixf18100_ApiMuxCfg_t *MuxCfg)
{
    uint RegValue;
    
    RegValue = HAL_IXF18100_READ(BaseAddress + ixf18100_WISDPCFG_OFFSET);
    
    /* Get the Rx Mux Cfg */
    MuxCfg->RxMuxCfg.PortCtrlCfg = ((RegValue >> ixf18100_sRX_SELMUX_CTRL)
        & 0x0001);
    
    MuxCfg->RxMuxCfg.ShadowSelCfg = ((RegValue >> ixf18100_sRX_CFGMUX_SWCTRL)
        & 0x0001);
    
    if ((RegValue >> ixf18100_sRX_MUX_CFG) & 0x0001)
    {
        MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal = ixf18100_eRX_APS_DATA;
    }
    else
    {
        MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgNormal = ixf18100_eRX_DATA;
    }
    
    if ((RegValue >> ixf18100_sRX_MUX_CFG_SHAD) & 0x0001)
    {
        MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgShadow = ixf18100_eRX_APS_DATA;
    }
    else
    {
        MuxCfg->RxMuxCfg.DataPath_Cfg_u.MuxCfgShadow = ixf18100_eRX_DATA;
    }
    
    /* Get the Tx Mux Cfg */
    MuxCfg->TxMuxCfg.PortCtrlCfg = ((RegValue >> ixf18100_sTX_SELMUX_CTRL)
        & 0x0001);
    
    MuxCfg->TxMuxCfg.ShadowSelCfg = ((RegValue >> ixf18100_sTX_CFGMUX_SWCTRL)
        & 0x0001);
    
    if ((RegValue >> ixf18100_sTX_MUX_CFG) & 0x0001)
    {
        MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal = ixf18100_eRX_APS_DATA;
    }
    else
    {
        MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgNormal
            = ixf18100_eTX_POH_OUTPUT_DATA;
    }
    
    if ((RegValue >> ixf18100_sTX_MUX_CFG_SHAD) & 0x0001)
    {
        MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgShadow = ixf18100_eRX_APS_DATA;
    }
    else
    {
        MuxCfg->TxMuxCfg.DataPath_Cfg_u.MuxCfgShadow
            = ixf18100_eTX_POH_OUTPUT_DATA;
    }
    
    /* Get the Tx Aps Mux Cfg */
    MuxCfg->TxApsMuxCfg.PortCtrlCfg
        = ((RegValue >> ixf18100_sTXAPS_SELMUX_CTRL) & 0x0001);
    
    MuxCfg->TxApsMuxCfg.ShadowSelCfg
        = ((RegValue >> ixf18100_sTXAPS_CFGMUX_SWCTRL) & 0x0001);
    
    if ((RegValue >> ixf18100_sTXAPS_MUX_CFG) & 0x0001)
    {
        MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal = ixf18100_eRX_DATA;
    }
    else
    {
        MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgNormal
            = ixf18100_eTX_TOH_OUTPUT_DATA;
    }
    
    if ((RegValue >> ixf18100_sTXAPS_MUX_CFG_SHAD) & 0x0001)
    {
        MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgShadow = ixf18100_eRX_DATA;
    }
    else
    {
        MuxCfg->TxApsMuxCfg.DataPath_Cfg_u.MuxCfgShadow
            = ixf18100_eTX_TOH_OUTPUT_DATA;
    }
    
    /* Clock Mux Cfg*/
    MuxCfg->ClkMuxCfg.ClkSelMuxCtrl
        = ((RegValue >> ixf18100_sCLK_SELMUX_CTRL) & 0x0001);
    
    MuxCfg->ClkMuxCfg.ClkCfgSWCtrl
        = ((RegValue >> ixf18100_sCLK_CFGMUX_SWCTRL) & 0x0001);
    
    return bb_NO_ERROR;
}
#endif

/******************************************************************************
* Function : Ixf18100GlobalSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the global block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the values to set int he device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GlobalSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf18100_ChipLevelCfg_t *ptGlobalCfg)
{
    ixf18100_GlbClkCnfg_t              clkCfg;
    ixf18100_GlbCnfg_t                 glbCfg;
    ixf18100_GlobalLineInterfaceCnfg_t lnIntfCfg;
    ixf18100_GlobalApsCfng_t           apsCfg;
    
    /* Initialize all bits to 0's */
    clkCfg.reg1.uiReg = 0;
    clkCfg.reg2.uiReg = 0;
    glbCfg.glbCnfgReg = 0;
    lnIntfCfg.lineCfg.reg = 0;
    apsCfg.glbAps.reg = 0;
    
    /* Sanity Check */
    if(ptGlobalCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    if(SelCfg != e_GLOBAL_CONFIG)
    {
        return bb_INV_CHIP_SEGMENT;
    }
    
    /* Set Global  config fill out the register mapping and write
    the proper register */
    
    glbCfg.Cnfg.lineSpeedSel = ptGlobalCfg->GeneralCfg.GlbCfg.LineSpSel;
    
    glbCfg.Cnfg.modeSelection = ptGlobalCfg->GeneralCfg.GlbCfg.ModeSel;
    
    glbCfg.Cnfg.sonetEnable = ptGlobalCfg->GeneralCfg.GlbCfg.SonetEn;
    
    /*Write configuration to proper registers */
    Ixf18100EditGlbCnfg(BaseAddress, &glbCfg);
    
    /* Set the Line Interface Configuration */
    lnIntfCfg.lineCfg.Cfg.localLoopEn
        = ptGlobalCfg->GeneralCfg.LineCfg.LineLocalLoopEn;
    
    lnIntfCfg.lineCfg.Cfg.loopPattern
        = ptGlobalCfg->GeneralCfg.LineCfg.LineLoopPattern;
    
    lnIntfCfg.lineCfg.Cfg.refClkInInv
        = ptGlobalCfg->GeneralCfg.LineCfg.RefClkInInv;
    
    lnIntfCfg.lineCfg.Cfg.remoteLoopEn
        = ptGlobalCfg->GeneralCfg.LineCfg.LineRemoteLoopEn;
    
    lnIntfCfg.lineCfg.Cfg.rlciInv
        = ptGlobalCfg->GeneralCfg.LineCfg.RlciInv;
    
    lnIntfCfg.lineCfg.Cfg.rxLnBitOrdFlip
        = ptGlobalCfg->GeneralCfg.LineCfg.RxLnBitOrdFlip;
    
    lnIntfCfg.lineCfg.Cfg.tlciInv = ptGlobalCfg->GeneralCfg.LineCfg.TlciInv;
    
    lnIntfCfg.lineCfg.Cfg.tlcoInv = ptGlobalCfg->GeneralCfg.LineCfg.TlcoInv;
    
    lnIntfCfg.lineCfg.Cfg.tlfpoInv = ptGlobalCfg->GeneralCfg.LineCfg.TlFpoInv;
    
    lnIntfCfg.lineCfg.Cfg.txLnBitOrdFlip
        = ptGlobalCfg->GeneralCfg.LineCfg.TxLnBitOrdFlip;
    
    lnIntfCfg.rxDataInvMask = ptGlobalCfg->GeneralCfg.LineCfg.RxDataInvMask;
    
    lnIntfCfg.txDataInvMask = ptGlobalCfg->GeneralCfg.LineCfg.TxDataInvMask;
    
    /* Write to the proper registers */
    Ixf18100OifSfiSetGlblLnIntf(BaseAddress, &lnIntfCfg);
    
    /* Set the bits for the APS Global Registers */
    apsCfg.glbAps.Cfg.remoteApsLoopEn
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.ApsRemoteLoopEn;
    
    apsCfg.glbAps.Cfg.localApsLoopEn
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.ApsLocalLoopEn;
    
    apsCfg.glbAps.Cfg.rlApsCiInv
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.RlApsciInv;
    
    apsCfg.glbAps.Cfg.rxApsBitOrdFlip
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.RxALnBitOrdFlip;
    
    apsCfg.glbAps.Cfg.txApsBitOrdFlip
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.TxALnBitOrdFlip;
    
    apsCfg.glbAps.Cfg.tlApsCoInv
        = ptGlobalCfg->GeneralCfg.GlbApsCfg.TlApscoInv;
    
    apsCfg.rxApsDataInvMask = ptGlobalCfg->GeneralCfg.GlbApsCfg.RxDataInvMask;
    
    apsCfg.txApsDataInvMask = ptGlobalCfg->GeneralCfg.GlbApsCfg.TxDataInvMask;
    
    /* Write to the proper registers */
    Ixf18100SetGlblAps(BaseAddress, &apsCfg); 
    
    /* Clock Configuration bit settings */
    clkCfg.reg1.tCnfg.refOutClk = ptGlobalCfg->ClockCfg.RefOutClk;
    
    clkCfg.reg1.tCnfg.rxPathClk = ptGlobalCfg->ClockCfg.RxPathClk;
    
    clkCfg.reg1.tCnfg.txAOifClk = ptGlobalCfg->ClockCfg.TxAoifClk;
    
    clkCfg.reg1.tCnfg.txApsClk  = ptGlobalCfg->ClockCfg.TxApsClk;
    
    clkCfg.reg1.tCnfg.txFifoClk = ptGlobalCfg->ClockCfg.TxFifoClk;
    
    clkCfg.reg1.tCnfg.txOifClk  = ptGlobalCfg->ClockCfg.TxOifClk;
    
    clkCfg.reg1.tCnfg.txPathClk = ptGlobalCfg->ClockCfg.TxPathClk;
    
    clkCfg.reg1.tCnfg.txTmsprtClk = ptGlobalCfg->ClockCfg.TxTrnsprtClk;
    
    clkCfg.reg2.tCnfg.refClkInDiv = ptGlobalCfg->ClockCfg.RxRefClkLnDivSh;
    
    clkCfg.reg2.tCnfg.rxClkDiv = ptGlobalCfg->ClockCfg.RxClkDivSh;
    
    /* Write to the proper registers */
    Ixf18100SetClkCfg(BaseAddress, &clkCfg);
    
    return bb_NO_ERROR;
}


/******************************************************************************
* Function : Ixf18100GlobalGetCfg
*------------------------------------------------------------------------------
* Description : This function gets global block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GlobalGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf18100_ChipLevelCfg_t *ptGlobalCfg)
{
    ixf18100_GlbClkCnfg_t              clkCfg;
    ixf18100_GlbCnfg_t                 glbCfg;
    ixf18100_GlobalLineInterfaceCnfg_t lnIntfCfg;
    ixf18100_GlobalApsCfng_t           apsCfg;
    
    /* Initialize all bits to 0's */
    clkCfg.reg1.uiReg = 0;
    clkCfg.reg2.uiReg = 0;
    glbCfg.glbCnfgReg = 0;
    lnIntfCfg.lineCfg.reg = 0;
    apsCfg.glbAps.reg = 0;
    
    /* Sanity Check */
    if(ptGlobalCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    if(SelCfg != e_GLOBAL_CONFIG)
    {
        return bb_INV_CHIP_SEGMENT;
    }
    
    /* Read the registers containing the Global configuration */
    Ixf18100GetGlbCnfg(BaseAddress, &glbCfg);
    
    /* Fill out the structure with the proper values from the register */
    ptGlobalCfg->GeneralCfg.GlbCfg.LineSpSel = glbCfg.Cnfg.lineSpeedSel;
    ptGlobalCfg->GeneralCfg.GlbCfg.ModeSel  =  glbCfg.Cnfg.modeSelection;
    ptGlobalCfg->GeneralCfg.GlbCfg.SonetEn = glbCfg.Cnfg.sonetEnable;
    
    /******* Line Interface Configuration ********************/
    /* Read the register with the Line Interface configuration */
    Ixf18100OifSfiGetGlblLnIntf(BaseAddress, &lnIntfCfg);
    
    /* Fill out the structure with the proper values from the register */
    ptGlobalCfg->GeneralCfg.LineCfg.LineLocalLoopEn
        = lnIntfCfg.lineCfg.Cfg.localLoopEn;
    
    ptGlobalCfg->GeneralCfg.LineCfg.LineLoopPattern
        = lnIntfCfg.lineCfg.Cfg.loopPattern;
    
    ptGlobalCfg->GeneralCfg.LineCfg.RefClkInInv
        = lnIntfCfg.lineCfg.Cfg.refClkInInv;
    
    ptGlobalCfg->GeneralCfg.LineCfg.LineRemoteLoopEn
        = lnIntfCfg.lineCfg.Cfg.remoteLoopEn;
    
    ptGlobalCfg->GeneralCfg.LineCfg.RlciInv = lnIntfCfg.lineCfg.Cfg.rlciInv;
    
    ptGlobalCfg->GeneralCfg.LineCfg.RxLnBitOrdFlip
        = lnIntfCfg.lineCfg.Cfg.rxLnBitOrdFlip;
    
    ptGlobalCfg->GeneralCfg.LineCfg.TlciInv = lnIntfCfg.lineCfg.Cfg.tlciInv;
    
    ptGlobalCfg->GeneralCfg.LineCfg.TlcoInv = lnIntfCfg.lineCfg.Cfg.tlcoInv;
    
    ptGlobalCfg->GeneralCfg.LineCfg.TlFpoInv = lnIntfCfg.lineCfg.Cfg.tlfpoInv;
    
    ptGlobalCfg->GeneralCfg.LineCfg.TxLnBitOrdFlip
        = lnIntfCfg.lineCfg.Cfg.txLnBitOrdFlip;
    
    ptGlobalCfg->GeneralCfg.LineCfg.RxDataInvMask = lnIntfCfg.rxDataInvMask;
    
    ptGlobalCfg->GeneralCfg.LineCfg.TxDataInvMask = lnIntfCfg.txDataInvMask;
    
    /********* Aps Global Configuration ***************************/
    /* Read The register of the APS global Configuartion */
    Ixf18100GetGlblAps(BaseAddress, &apsCfg);
    
    /* Fill out the structures with the proper data */
    ptGlobalCfg->GeneralCfg.GlbApsCfg.ApsRemoteLoopEn
        = apsCfg.glbAps.Cfg.remoteApsLoopEn;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.ApsLocalLoopEn
        = apsCfg.glbAps.Cfg.localApsLoopEn;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.RlApsciInv
        = apsCfg.glbAps.Cfg.rlApsCiInv;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.RxALnBitOrdFlip
        = apsCfg.glbAps.Cfg.rxApsBitOrdFlip;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.TxALnBitOrdFlip
        = apsCfg.glbAps.Cfg.txApsBitOrdFlip;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.TlApscoInv
        = apsCfg.glbAps.Cfg.tlApsCoInv;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.RxDataInvMask = apsCfg.rxApsDataInvMask;
    
    ptGlobalCfg->GeneralCfg.GlbApsCfg.TxDataInvMask = apsCfg.txApsDataInvMask;
    
    /********* Clock Configuration ************************/
    /* Read The Register containing the clock configuration */
    Ixf18100GetClkCnfg(BaseAddress, &clkCfg);
    
    /* Fill out the structures with the proper data */
    ptGlobalCfg->ClockCfg.RefOutClk = clkCfg.reg1.tCnfg.refOutClk;
    
    ptGlobalCfg->ClockCfg.RxPathClk = clkCfg.reg1.tCnfg.rxPathClk;
    
    ptGlobalCfg->ClockCfg.TxAoifClk = clkCfg.reg1.tCnfg.txAOifClk;
    
    ptGlobalCfg->ClockCfg.TxApsClk = clkCfg.reg1.tCnfg.txApsClk;
    
    ptGlobalCfg->ClockCfg.TxFifoClk = clkCfg.reg1.tCnfg.txFifoClk;
    
    ptGlobalCfg->ClockCfg.TxOifClk = clkCfg.reg1.tCnfg.txOifClk;
    
    ptGlobalCfg->ClockCfg.TxPathClk = clkCfg.reg1.tCnfg.txPathClk;
    
    ptGlobalCfg->ClockCfg.TxTrnsprtClk = clkCfg.reg1.tCnfg.txTmsprtClk;
    
    ptGlobalCfg->ClockCfg.RxRefClkLnDivSh = clkCfg.reg2.tCnfg.refClkInDiv;
    
    ptGlobalCfg->ClockCfg.RxClkDivSh = clkCfg.reg2.tCnfg.rxClkDiv;
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetGlblAps
*------------------------------------------------------------------------------
* Description : This function gets the global APS configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100GetGlblAps(bb_RegPointer_type BaseAddress,
                   ixf18100_GlobalApsCfng_t *pCnfg)
{
    pCnfg->glbAps.reg = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_APS_OFFSET);
    
    pCnfg->rxApsDataInvMask = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_RXAPSDATINV_OFFSET);
    
    pCnfg->txApsDataInvMask = HAL_IXF18100_READ(BaseAddress
        + ixf18100_GLB_TXAPSDATINV_OFFSET);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SetGlblAps
*------------------------------------------------------------------------------
* Description : This functions sets global APS configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf18100SetGlblAps(bb_RegPointer_type BaseAddress,
                   ixf18100_GlobalApsCfng_t *pCnfg)
{
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_APS_OFFSET,
        pCnfg->glbAps.reg);
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_RXAPSDATINV_OFFSET,
        pCnfg->rxApsDataInvMask);
    
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_TXAPSDATINV_OFFSET,
        pCnfg->txApsDataInvMask);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GlobalSetTestCfg
*------------------------------------------------------------------------------
* Description : The function sets the test configuration
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device
*           bb_TestType_e testType: selects the test type
*           void *ptTestCfg: pointer to the test configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100GlobalSetTestCfg(bb_RegPointer_type BaseAddress,
                         bb_TestType_e testType, void *ptTestCfg)
{
    ushort  usValue;
    ulong   ulTempVal = 0;
    uchar   offset;
    
    ulTempVal = *((ulong*)(ptTestCfg));
    
    switch (testType)
    {
    case bb_LINE_LOCAL_LOOPBACK:
        offset = ixf18100_GLB_LINE_OFFSET;
        usValue = HAL_IXF18100_READ(BaseAddress + offset);
        usValue &= ~(IXF_BIT(ixf18100_sLLOOP_EN));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sLLOOP_EN);
        break;
        
    case bb_LINE_REMOTE_LOOPBACK:
        offset = ixf18100_GLB_LINE_OFFSET;
        usValue = HAL_IXF18100_READ(BaseAddress + offset);
        usValue &= ~(IXF_BIT(ixf18100_sRLOOP_EN));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sRLOOP_EN);
        break;
        
    case bb_APS_INTF_LOCAL:
        offset = ixf18100_GLB_APS_OFFSET;
        usValue = HAL_IXF18100_READ(BaseAddress + offset);
        usValue &= ~(IXF_BIT(ixf18100_sLLOOP_APS_EN));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sLLOOP_APS_EN);
        break;
        
    case bb_APS_INTF_REMOTE:
        offset = ixf18100_GLB_APS_OFFSET;
        usValue = HAL_IXF18100_READ(BaseAddress + offset);
        usValue &= ~(IXF_BIT(ixf18100_sRLOOP_APS_EN));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sRLOOP_APS_EN);
        break;
        
    default:
        return bb_INV_TEST_CNFG;
        break;
    }
    
    HAL_IXF18100_WRITE(BaseAddress + offset, usValue);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GlobalGetTestCfg
*------------------------------------------------------------------------------
* Description : The function gets the test configuration set in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           testType : The type of test to run.
*           void *pTestCfg: pointer to the test configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100GlobalGetTestCfg(bb_RegPointer_type BaseAddress,
                         bb_TestType_e testType, void *pTestCfg)
{
    ushort usValue;
    ulong  *ulRetVal = (ulong *)pTestCfg;
    
    switch (testType)
    {
    case bb_LINE_LOCAL_LOOPBACK:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_LINE_OFFSET);
        *ulRetVal = ((usValue >> ixf18100_sLLOOP_EN) & 0x01);
        break;
        
    case bb_LINE_REMOTE_LOOPBACK:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_LINE_OFFSET);
        *ulRetVal = (ulong)((usValue >> ixf18100_sRLOOP_EN) & 0x01);
        break;
        
    case bb_APS_INTF_LOCAL:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_APS_OFFSET);
        *ulRetVal = (ulong)((usValue >> ixf18100_sLLOOP_APS_EN) & 0x01);
        break;
        
    case bb_APS_INTF_REMOTE:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_APS_OFFSET);
        *ulRetVal = (ulong)((usValue >> ixf18100_sRLOOP_APS_EN) & 0x01);
        break;
        
    default:
        return bb_INV_TEST_CNFG;
    }
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GlobalGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100GlobalGetStatus(bb_RegPointer_type BaseAddress,
                        bb_SelStatus_e selStatus, void* pStatus)
{
    ushort      usValue = 0;
    
    ixf18100_GlobalStatus_t *pGlbStat = (ixf18100_GlobalStatus_t *)pStatus;
    
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_STAT_OFFSET);
    
    pGlbStat->ucIpSignalOK = (uchar)((usValue >> ixf18100_sSIG_DET) & 0x01);
    
    pGlbStat->ucRxLOSDetected = (uchar)((usValue >> ixf18100_sRLXLOS) & 0x01);
    
    pGlbStat->ucLineRate = (uchar)((usValue >> ixf18100_sLINE_RATE) & 0x01);
    
    pGlbStat->ucConfigError = (uchar)((usValue >> ixf18100_sCONFIG_ER) & 0x01);
    
    return bb_NO_ERROR;
}
