/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:03p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_global_d.h $
*   WORKFILE:       $Workfile: ixf18100_global_d.h $
*
* --------------------------------------------------------------------------
* Project: ixf1810x API
*
* Purpose: 
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf1810x/ixf18100_global_d.h $
 * 
 * 1     3/11/04 3:03p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 13    1/30/03 4:40p Pkjain2
* Made changes for PTQ release. Added Getstatus calls for SONET and
* GLOBAL block.
* 
* 12    12/02/02 2:24p Kssaepha
* Added support for Setting GFP OpModes 
* 
* 11    9/11/02 1:46p Jaalliso
* Adding copyright notice header comment
* 
* --------------------------------------------------------------------------
*/

#ifndef IXF18100_GLOBAL_D_H
#define IXF18100_GLOBAL_D_H

#include "ixf_bitfield_macros.h"

/***********************************************************************
* Chip Info for Calypso Chips 
**********************************************************************/

typedef struct             /* Information set by the hardware */
{
    ushort ChipVersion;     /* Chip Version */
    ushort ChipId;          /* Chip ID */
} ixf18100_ChipInfo_t;


/**************************************************************************
* Global Configuration Register
**************************************************************************/

typedef union 
{
    ushort glbCnfgReg;
    struct 
    {
        BITFIELD_4(
            uint  lineSpeedSel : 1,
            uint  sonetEnable  : 1,
            uint  modeSelection: 2,
uint : 12)
       
    } Cnfg;
    
} ixf18100_GlbCnfg_t;


/* Selection for the line speed (clock rate for the line interface)*/
enum
{
    ixf18100_eWAN_OC192 = 0,
        ixf18100_eLAN_10GBPS
} ixf18100_elineSpeedSelections_e;

/* SONET Framer Selection */
typedef enum
{
    ixf18100_eSONET_DISABLED = 0,
        ixf18100_eSONET_ENABLED
} ixf18100_eSonetEnable_e;

/* Modes Selection, for selecting the datapath route*/
enum 
{
    ixf18100_ePOS_MODE = 0x0,
        ixf18100_eETHERNET_MODE,
        ixf18100_eGFP_MODE
} ixf18100_eglobalModeOptions_e;


/**************************************************************************
* Global Reset Register
**************************************************************************/

typedef union 
{
    ushort glbResetReg;
    struct 
    {
        BITFIELD_5(
            uint sftRst        : 1, /*Software Reset field*/
            uint apsIntfRst    : 1, /*APS Reset*/
            uint lineIntfRst   : 1, /*Line Interface Reset */
            uint spiIntfRst    : 1, /*SPI Interaface Reset */ 
uint : 12)
    } Cnfg;
    
} ixf18100_GlbReset_t;


/* Values for the reset operations valid for all the types of reset*/
typedef enum
{
    ixf18100_eNORMAL = 0,
        ixf18100_eRESET
} ixf18100_eResetValues_e;


/**************************************************************************
* Global Wachtdog  Registers
**************************************************************************/

typedef struct
{  
    union
    {
        ushort reg;
        struct
        {
            BITFIELD_3(
                uint watchDogTmr   : 8,
                uint watchDogEnSel : 1,
uint : 7)
        } Cnfg;
    } watchDogCnfgRegister;
    
    ushort watchDogFailAddress;
} ixf18100_GlbWatchDogInfo_t;


/* Values to disable/enable the watchdog sonet*/
enum
{
    ixf18100_eWATCHDOG_DISABLE = 0,
        ixf18100_eWATCHDOG_ENABLE
} ixf18100_eWatchDogEnableSel_e;


/**************************************************************************
* Global Clock Configuration Register
**************************************************************************/

typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_8(
            uint txTmsprtClk : 2,
            uint txPathClk   : 2,
            uint rxPathClk   : 2,
            uint refOutClk   : 2,
            uint txFifoClk   : 2,
            uint txApsClk    : 2,
            uint txOifClk    : 2,
            uint txAOifClk   : 2)
    } tCnfg;
} ixf18100_GlbClkCnfg1_t;


typedef union
{
    ushort uiReg;
    struct
    {
        BITFIELD_3(
            uint refClkInDiv : 2,
            uint rxClkDiv    : 1,
uint : 13)
    } tCnfg;
} ixf18100_GlbClkCnfg2_t;

typedef struct
{
    ixf18100_GlbClkCnfg1_t reg1;
    ixf18100_GlbClkCnfg2_t reg2;    
} ixf18100_GlbClkCnfg_t;


/**************************************************************************
* Global APS and Line Configuration Register
**************************************************************************/

typedef union 
{
    ushort reg;
    struct 
    {
        BITFIELD_13(
            uint  rxLnBitOrdFlip : 1,
            uint  txLnBitOrdFlip : 1,
            uint  rlciInv        : 1,
            uint  tlcoInv        : 1,
            uint  tlciInv        : 1,
uint  : 1,
        uint  tlfpoInv       : 1,
        uint  refClkInInv    : 1,
uint  : 2,
        uint  localLoopEn    : 1,
        uint  remoteLoopEn   : 1,
        uint  loopPattern    : 1,
uint : 3)
    } Cfg;
} ixf18100_GlbLineCfg_t;

typedef struct
{ 
    ixf18100_GlbLineCfg_t lineCfg; 
    ushort rxDataInvMask;
    ushort txDataInvMask;
} ixf18100_GlobalLineInterfaceCnfg_t;


typedef struct 
{
    union 
    {
        ushort reg;
        struct 
        {
            BITFIELD_7(
                uint  rxApsBitOrdFlip   : 1,
                uint  txApsBitOrdFlip   : 1,
                uint  rlApsCiInv        : 1,
                uint  tlApsCoInv        : 1,
                uint  localApsLoopEn    : 1,
                uint  remoteApsLoopEn   : 1,
uint  : 10)
        } Cfg;
    } glbAps;
    ushort txApsDataInvMask;
    ushort rxApsDataInvMask;
} ixf18100_GlobalApsCfng_t;

/* Invert options */
enum 
{
    ixf18100_eINVERT = 0,
        ixf18100_eNOINVERT
} ixf18100_InvertSelection_e;

/**Enable disable loops*/
enum 
{
    ixf18100_eLOOPDIS = 0,
        ixf18100_eLOOPEN
} ixf18100_eLoopActivation_e;

/** Loop Pattern*/
enum 
{
    ixf18100_eLOCALDATA = 0,
        ixf18100_ePATTERN
} ixf18100_eLoopPattern_e;

/** Bit ordering*/
enum 
{
    ixf18100_eBIGENDIAN = 0,
        ixf18100_eLITTLEENDIAN
} ixf18100_eByteOrdering_e;


typedef enum 
{
    ixf18100_eTX_APS_MUX = 0,
        ixf18100_eTX_MUX,
        ixf18100_eRX_MUX,
        ixf18100_eCLOCK_MUX
} ixf18100_eWIS_Mux_e;

typedef enum 
{
    ixf18100_eTX_TOH_OUTPUT_DATA = 0,
        ixf18100_eTX_POH_OUTPUT_DATA,
        ixf18100_eRX_DATA,
        ixf18100_eRX_APS_DATA
        
} ixf18100_eDataPath_e;

typedef enum 
{    
    ixf18100_eSW_CTRL = 0,
        ixf18100_eAPS_TAL_PORT_CTRL
        
} ixf18100_ePortCtrlCfg_e;


typedef enum 
{
    ixf18100_eNORMAL_CFG = 0,
        ixf18100_eSHADOW_CFG
        
} ixf18100_eShadowCfgSel_e;

typedef enum 
{
    ixf18100_eGLB_CLKCFG = 0,
        ixf18100_eGLB_CLKCFG_SHAD 
        
} ixf18100_eClkCfgSWCtrl_e;

/* This structure controls the Rx, Tx or Tx Aps Data Mux */
typedef struct 
{
    ixf18100_ePortCtrlCfg_e     PortCtrlCfg;        /* SW or TAL Control Mux */
    ixf18100_eShadowCfgSel_e    ShadowSelCfg;       /* Shadow or Normal Config*/
    
    union datapath_cfg
    {
        ixf18100_eDataPath_e    MuxCfgNormal; /*Normal Cfg*/
        ixf18100_eDataPath_e    MuxCfgShadow; /*Shadow  Cfg*/
    } DataPath_Cfg_u;
    
} ixf18100_Mux_Cfg_t;

typedef struct
{
    ixf18100_eShadowCfgSel_e   ClkSelMuxCtrl; /* See Bit 12 */
    ixf18100_eClkCfgSWCtrl_e   ClkCfgSWCtrl;  /* See Bit 13 */
} ixf18100_Clk_Mux_Cfg_t;

/* WISDPCFG Regsiter Configures various muxes in the WIS Datapath */
typedef struct
{
    ixf18100_Mux_Cfg_t      TxApsMuxCfg; 
    ixf18100_Mux_Cfg_t      TxMuxCfg;
    ixf18100_Mux_Cfg_t      RxMuxCfg;
    /* Clock Configuration */
    ixf18100_Clk_Mux_Cfg_t  ClkMuxCfg;
    
} ixf18100_ApiMuxCfg_t;

/* Data structures for Set and Get Global Configuration */

typedef enum
{
    ixf18100_eBIT_15_IS_MSB = 0,
        ixf18100_eBIT_0_IS_MSB
        
} ixf18100_eLineBitOrderSel_e;

typedef enum
{
    ixf18100_eLOCAL_DATA = 0,
        ixf18100_e00FF_PATTERN
        
} ixf18100_eLineLoopPatternSel_e;


typedef enum
{
    ixf18100_eNORMAL_OPERATION = 0,
        ixf18100_eLOOP_ENABLED
        
} ixf18100_eLineLoopOperSel_e;

typedef enum
{
    ixf18100_eDO_NOT_INVERT = 0,
        ixf18100_eINVERT_POLARITY
        
} ixf18100_eLineInvPolaritySel_e;

typedef struct
{
    
    ixf18100_eLineLoopOperSel_e      ApsRemoteLoopEn;
    ixf18100_eLineLoopOperSel_e      ApsLocalLoopEn;
    ixf18100_eLineInvPolaritySel_e   TlApscoInv;
    ixf18100_eLineInvPolaritySel_e   RlApsciInv;
    ixf18100_eLineBitOrderSel_e      TxALnBitOrdFlip;
    ixf18100_eLineBitOrderSel_e      RxALnBitOrdFlip;
    ushort                           RxDataInvMask;
    ushort                           TxDataInvMask;
    
} ixf18100_ApsGlbIntfCfg_t;


typedef struct
{
    ixf18100_eLineLoopPatternSel_e  LineLoopPattern;
    ixf18100_eLineLoopOperSel_e     LineRemoteLoopEn;
    ixf18100_eLineLoopOperSel_e     LineLocalLoopEn;
    ixf18100_eLineInvPolaritySel_e  RefFpoInv;
    ixf18100_eLineInvPolaritySel_e  RefFpiInv;
    ixf18100_eLineInvPolaritySel_e  RefClkInInv;
    ixf18100_eLineInvPolaritySel_e  TlFpoInv;
    ixf18100_eLineInvPolaritySel_e  RlFpiInv;
    ixf18100_eLineInvPolaritySel_e  TlciInv;
    ixf18100_eLineInvPolaritySel_e  TlcoInv;
    ixf18100_eLineInvPolaritySel_e  RlciInv;
    ixf18100_eLineBitOrderSel_e     TxLnBitOrdFlip;
    ixf18100_eLineBitOrderSel_e     RxLnBitOrdFlip;
    ushort                          RxDataInvMask;
    ushort                          TxDataInvMask;
    
} ixf18100_LineIntfCfg_t;

typedef enum
{
    ixf18100_eOC192_SEL = 0,
        ixf18100_eWAN_SEL = ixf18100_eOC192_SEL,
        ixf18100_e10GBPS_SEL,
        ixf18100_eLAN_SEL = ixf18100_e10GBPS_SEL
        
} ixf18100_eLineSpeedSel_e;

typedef enum
{
    ixf18100_eSONET_DISABLE = 0,
        ixf18100_eSONET_ENABLE
} ixf18100_eSonetEn_e;

typedef enum
{
    ixf18100_ePOS_SEL = 0,
        ixf18100_eETHERNET_SEL, 
        ixf18100_eGFP_PROCESSOR_SEL
        
}ixf18100_eModeSel_e;

typedef struct
{
    ixf18100_eModeSel_e         ModeSel;    /* Invert Ref Sys 622 Input */
    ixf18100_eSonetEn_e         SonetEn;    /* Invert Ref Line 622 Output*/
    ixf18100_eLineSpeedSel_e    LineSpSel;  /* Invert Ref Line 622 Input */
    
} ixf18100_GlbCfg_t;

typedef struct /* Calypso chip level configuration */
{
    ixf18100_GlbCfg_t           GlbCfg;     /* General Chip Config. */
    ixf18100_LineIntfCfg_t      LineCfg; /* Clock  Cfg */
    ixf18100_ApsGlbIntfCfg_t    GlbApsCfg;
} ixf18100_GeneralCfg_t;


/* Values to the Mux selection for  the TX OIF and AOIF*/
typedef enum 
{
    ixf18100_eTLCI = 0,
        ixf18100_eREF_CLK_IN,
        ixf18100_eRLCI
        
} ixf18100_eGlbClkMuxSel_e;


/* Values to the Mux selection for section clocks*/
typedef enum
{
    ixf18100_eCLK_RXOIF_77 = 0,
        ixf18100_eCLK_RXAPS_77,
        ixf18100_eCLK_TXREF_77,
        ixf18100_eCLK_TXOIF_77,
        ixf18100_eCLK_RXOIF_155 = 0,
        ixf18100_eCLK_RXAPS_155,
        ixf18100_eCLK_TXREF_155,
        ixf18100_eCLK_TXOIF_155
        
} ixf18100_eGlbClkSectionMuxSel_e;

typedef enum
{
    ixf18100_eDIVIDE_RLCI_BY_8 = 0,
        ixf18100_eDIVIDE_RLCI_BY_4  
} ixf18100_eRlciIapsciDivSel_e;

typedef enum
{
    ixf18100_eDIVIDE_REF_BY_1 = 0,
        ixf18100_eDIVIDE_REF_BY_2, 
        ixf18100_eDIVIDE_REF_BY_4,
        ixf18100_eDIVIDE_REF_BY_8
} ixf18100_eRefClkInDivSel_e;

typedef struct /* Clock and timing configuration */
{
    ixf18100_eGlbClkMuxSel_e        TxAoifClk;  
    ixf18100_eGlbClkMuxSel_e        TxOifClk; 
    ixf18100_eGlbClkSectionMuxSel_e TxApsClk; 
    ixf18100_eGlbClkSectionMuxSel_e TxFifoClk;
    ixf18100_eGlbClkSectionMuxSel_e RefOutClk;
    ixf18100_eGlbClkSectionMuxSel_e RxPathClk;
    ixf18100_eGlbClkSectionMuxSel_e TxPathClk;
    ixf18100_eGlbClkSectionMuxSel_e TxTrnsprtClk;
    ixf18100_eRlciIapsciDivSel_e    RxClkDivSh;
    ixf18100_eRefClkInDivSel_e      RxRefClkLnDivSh;
} ixf18100_Clock_t;

typedef struct /* Calypso chip level configuration */
{
    ixf18100_GeneralCfg_t           GeneralCfg; /* General Chip Config. */
    ixf18100_Clock_t                ClockCfg;   /* Clock  Cfg */
    
} ixf18100_ChipLevelCfg_t;

typedef struct
{
    uchar   ucIpSignalOK;
    uchar   ucRxLOSDetected;
    uchar   ucLineRate;
    uchar   ucConfigError;
} ixf18100_GlobalStatus_t;

#endif    
