/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/11/04 3:01p $
*   LAST MODIFIED:  $Modtime: 3/05/04 12:05p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf_api_sonet.h $
*   WORKFILE:       $Workfile: ixf_api_sonet.h $
*
* --------------------------------------------------------------------------
* Project: 
*
* Purpose: 
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/VxWorks/MediaCardDrivers/IXD28192/ixf/ixf_api_sonet.h $
 * 
 * 1     3/11/04 3:01p Tepulkow
 * 
 * 1     5/16/03 1:41p Tepulkow
* 
* 6     2/13/02 1:59p Machambe
* 
* 5     2/07/02 5:03p Skadrix
* Added a new API function  - IxfApiSetBERThreshold
* 
* 4     12/12/01 4:03p Machambe
* Added IxfApiMapframe related functions.
* 
* 3     11/15/01 6:40p Machambe
* 
* 2     8/08/01 3:53p Rfranz
* Moved sonet related API functions/prototypes
* 
* 1     7/18/01 10:54a Rfranz
* Initial check in - file headers only
* 
* --------------------------------------------------------------------------
*/

#ifndef __IXF_API_SONET_H__
#define __IXF_API_SONET_H__

/******************************************************************
* Function : IxfApiGetWindowSize
*-----------------------------------------------------------------
* Description  : Gets the window size for clear and setting 
*                Degraded B2 error.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*  						window size from.
*           mode      : The clearing/setting mode.
* Outputs : value  : Window size value.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGetWindowSize(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    bb_WindowSizeMode_e mode, ulong* value);

                    /******************************************************************
                    * Function : IxfApiSetWindowSize
                    *-----------------------------------------------------------------
                    * Description  : Sets the window size for clear and setting 
                    *                Degraded B2 error.
                    *-----------------------------------------------------------------
                    * Inputs  : pChipData : Initialized Chip Data
                    *           section   : Determines the section(block) to set the
                    *  						window size.
                    *           mode      : The clearing/setting mode.
                    *           value     : Window size value.
                    * Outputs : None.
                    *-----------------------------------------------------------------
                    * Returns : Status
                    *-----------------------------------------------------------------
                    * Access Globals : None.
                    *-----------------------------------------------------------------
                    * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetWindowSize(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                    bb_WindowSizeMode_e mode, ulong value);

                    /******************************************************************
                    * Function : IxfApiSetBERThreshold
                    *-----------------------------------------------------------------
                    * Description  : Sets the Bit Error Rate Threshold for the 
                    *                Excessive or Degraded B2 Error
                    *-----------------------------------------------------------------
                    * Inputs  : *pChipData : Pointer to the data structure holding the Chip
                    *						 Data
                    *           *section :   Pointer to the data structure to determine the 
                    *                        section(block) to set the  BER Threshold for.
                    *           berThresholdSel : Specficies the Register for which to
                    *                             set the BER Threshold value.
                    *           mode :	The STS Level for which to set the BER Threshold
                    *                   value.
                    *           expNum : The exponent for which to set the BER Threshold
                    *                    value.
                    * Outputs : None
                    *----------------------------------------------------------------------------
                    * Returns : bb_NO_ERROR if successful
                    *           bb_NULL_ARG if the pointer to the counter buffer was NULL
                    *           bb_INV_PARAMETER if invalid parameter
                    *----------------------------------------------------------------------------
                    * Access Globals : None
                    *----------------------------------------------------------------------------
                    * Comments : 
*****************************************************************************/
extern bb_Error_e 
IxfApiSetBERThreshold(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                      bb_BerThresholdSel_e berThresholdSel, 
                      bb_STSMode_e mode, short expNum);

                      /******************************************************************
                      * Function : IxfApiGetTrace
                      *-----------------------------------------------------------------
                      * Description  : Gets the J0/J1/J2 Trace
                      *-----------------------------------------------------------------
                      * Inputs  : pChipData : Initialized Chip Data
                      *			section   :	Determines the section(block) to get the
                      *						trace from.
                      *			TraceType : Type of Trace: J0/J1/J2 for system and
                      *						line side.
                      * Outputs : pTrace	: place to return Rx Trace String.
                      *           pLength : The length of pTrace.
                      *-----------------------------------------------------------------
                      * Returns : bb_Error_e  Returns Error Condition if Not successful
                      *-----------------------------------------------------------------
                      * Access Globals : None.
                      *-----------------------------------------------------------------
                      * Comments : Note: at least 65 bytes long for pTrace
******************************************************************/
extern bb_Error_e 
IxfApiGetTrace(	bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
               bb_TraceType_e TraceType, char* pTrace, ushort* pLength);

               /******************************************************************
               * Function : IxfApiSetTrace
               *-----------------------------------------------------------------
               * Description  : Set Expected/Tx J0/J1 Trace
               *-----------------------------------------------------------------
               * Inputs  : pChipData : Initialized Chip Data
               *			section	  : Determines the section(block) to get the
               *						trace from.
               *			TraceType : Type of Trace: Expected/Tx J0/J1 for system
               *						or line side.
               *			TraceFormat : Specifies the Trace Format
               *			pTrace	  : null-terminated Trace String (data only)
               *						Driver adds CRC7, or LF/CR, if needed
               * Outputs : None.
               *-----------------------------------------------------------------
               * Returns : bb_Error_e  Returns Error Condition if Not successful
               *-----------------------------------------------------------------
               * Access Globals : None.
               *-----------------------------------------------------------------
               * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetTrace(	bb_ChipData_t* pChipData, bb_ChipSegment_t *section, 
               bb_TraceType_e TraceType,
               bb_TraceFormat_e TraceFormat, char* pTrace);

               /******************************************************************
               * Function : IxfApiGetOhBytes
               *-----------------------------------------------------------------
               * Description  : Gets Overhead Bytes for the Multiplex and High 
               *				Order Path sections.
               *-----------------------------------------------------------------
               * Inputs  : pChipData	: Initialized Chip Data
               *			section		: Determines the section(block) to get the 
               *						  bytes
               *			SelOhBytes	: Selects the Overhead Bytes 
               * Outputs : pOhBytes	: place to put Overhead Bytes
               *-----------------------------------------------------------------
               * Returns : bb_Error_e  Returns Error Condition if Not successful
               *-----------------------------------------------------------------
               * Access Globals : None.
               *-----------------------------------------------------------------
               * Comments : None.
******************************************************************/
extern bb_Error_e
IxfApiGetOhBytes(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                 bb_SelOhBytes_e  SelOhBytes, void* pOhBytes);

                 /******************************************************************
                 * Function : IxfApiSetOhByte
                 *-----------------------------------------------------------------
                 * Description  : Sets Overhead Bytes for the Multiplex and High 
                 *				Order Path sections.
                 *-----------------------------------------------------------------
                 * Inputs  : pChipData : Initialized Chip Data
                 *			section : Determines the section(block) to set the byte
                 *			SelOhByte : Selects a particular Overhead byte 
                 *			OhByte : Value of selected Overhead byte
                 * Outputs : None.
                 *-----------------------------------------------------------------
                 * Returns : bb_Error_e  Returns Error Condition if Not successful
                 *-----------------------------------------------------------------
                 * Access Globals : None.
                 *-----------------------------------------------------------------
                 * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetOhByte(bb_ChipData_t* pChipData, bb_ChipSegment_t *section,
                bb_SelOhBytes_e SelOhByte, ushort OhByte);

                /******************************************************************
                * Function : IxfApiMapFrames
                *-----------------------------------------------------------------
                * Description  : The routine sets up a map between the input source
                *				slot(s) to the destination slot(s).
                *-----------------------------------------------------------------
                * Inputs  : pChipData : Initialized Chip Data
                *			type : The type of frame(s) (NORMAL, MASTER, SLAVE, or AIS)
                *			sigType		: Specifies either STS_TYPE or STM_TYPE.
                *			src : Identifies where the frame(s) are coming from.
                *			dest : Identifies where the frame(s) are going to.
                *			srcFrameNum : Identifies the starting source STS slot.
                *			destFrameNum : Identifies the starting destination STS slot.
                *			numOfFrames : Specifies the number of frame(s)
                * Outputs : None.
                *-----------------------------------------------------------------
                * Returns : 
                *-----------------------------------------------------------------
                * Access Globals : 
                *-----------------------------------------------------------------
                * Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiMapFrames(bb_ChipData_t* pChipData, MapFrame_t type,
                bb_SignalType_e sigType, ulong src, ulong dest,
                ulong srcFrameNum, ulong destFrameNum, ulong numOfFrames);

                /******************************************************************
                * Function : IxfApiGetMappedFrames
                *-----------------------------------------------------------------
                * Description  :	This routine retrieves an address of the Transmit 
                *					or Receive connection memory.
                *-----------------------------------------------------------------
                * Inputs  : pChipData : Initialized Chip Data
                *			sigType		: Specifies either STS_TYPE or STM_TYPE.
                *			src : Identifies where the frame(s) are coming from.
                *			destFrameNum : Identifies from which destination frame
                *						   to read from.
                *			numOfFrames : Specifies the number of frames
                * Outputs : address : Contains the source STS slots which maps to 
                *					 the dest slots.
                *-----------------------------------------------------------------
                * Returns : 
                *-----------------------------------------------------------------
                * Access Globals : 
                *-----------------------------------------------------------------
                * Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiGetMappedFrames( bb_ChipData_t* pChipData, bb_SignalType_e sigType,
                      ulong src, ulong destFrameNum, ulong numOfFrames,
                      ushort* address);

                      /******************************************************************
                      * Function : IxfApiGetAuPointerFrames
                      *-----------------------------------------------------------------
                      * Description  : This routine retrieves N number of Administrative
                      *					  Unit pointers.  
                      *-----------------------------------------------------------------
                      * Inputs  : pChipData : Initialized Chip Data
                      *			sigType		: Specifies either STS_TYPE or STM_TYPE.
                      *			src : Idenitifies which section of the chip the STS
                      *				 Au Pointer is coming from.
                      *			srcFrameNum : Identifies the starting source STS slot.
                      *			numOfFrames : Specifies the number of STS's
                      * Outputs : AuPtrValue : Will contain the AU pointer(s) value
                      *-----------------------------------------------------------------
                      * Returns : 
                      *-----------------------------------------------------------------
                      * Access Globals : None.
                      *-----------------------------------------------------------------
                      * Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiGetFramesAuPointer(bb_ChipData_t* pChipData, ulong src,
                         bb_SignalType_e sigType, ulong srcFrameNum,
                         ulong numOfFrames, ushort *AuPtrValue);

#endif /* __IXF_API_SONET_H__*/
