/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/* ixd2810.h - IXD2810 MAC device header file

modification history
--------------------
01a, 08oct02,eme	created
01b, 29apr03, tep changed msf_slowport_base to FABRIC_SLOWPORT_BASE to coincide with 
the BSP .H file
01c, 18jun03,tep added macros from the IXDP2800 bsp .h files to allow indiependent 
build of this driver.
01d,16jun03,tep Changed BOARD_ID to IXD2810_BOARD_ID, changed N_DEVICES to 1 
from 2, changed 2810_REG macros to use slowportread and write functions, 
changed boardbase array to be sized by compiler.
01e,25jun03,tep Added gbic_status0 and 1 to 2810_DEV struct, added ALL_PORTS 
macro define
01f,26jun03,tep added ixd_version macro define.  Changes ioctl commands to 
enum.
*/

#ifndef __INCixd2810h
#define __INCixd2810h

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _ASMLANGUAGE

/* includes */
#include "vxworks.h"
#include "iosLib.h"
#include "intLib.h"
#include "iv.h"
#include "semLib.h"
#include "string.h"
#include <ixd2810Port.h>
#include <ixd2810Mmap.h>
#include <ixd2810ModNum.h>

/*IXP VERSION for IXDP2000*/
#define IXD_VERSION "3.50.310"

/* debugging utility */

/*#define DEBUG*/

#ifdef DEBUG
#define dprintf(fmt, args...)	printf(fmt, ##args)
#else
#define dprintf(fmt, args...)	
#endif

/*
 * Constants
 */
#define    SZ_0                           0x00000000
#define    SZ_1                           0x00000001
#define    SZ_2                           0x00000002
#define    SZ_4                           0x00000004
#define    SZ_8                           0x00000008
#define    SZ_16                          0x00000010
#define    SZ_32                          0x00000020
#define    SZ_48                          0x00000030
#define    SZ_64                          0x00000040
#define    SZ_128                         0x00000080
#define    SZ_256                         0x00000100
#define    SZ_512                         0x00000200
#define    SZ_1K                          0x00000400
#define    SZ_2K                          0x00000800
#define    SZ_4K                          0x00001000
#define    SZ_8K                          0x00002000
#define    SZ_16K                         0x00004000
#define    SZ_32K                         0x00008000
#define    SZ_64K                         0x00010000
#define    SZ_128K                        0x00020000
#define    SZ_256K                        0x00040000
#define    SZ_512K                        0x00080000
#define    SZ_1M                          0x00100000
#define    SZ_2M                          0x00200000
#define    SZ_3M                          0x00300000
#define    SZ_4M                          0x00400000
#define    SZ_8M                          0x00800000
#define    SZ_16M                         0x01000000
#define    SZ_32M                         0x02000000
#define    SZ_48M                         0x03000000
#define    SZ_64M                         0x04000000
#define    SZ_128M                        0x08000000
#define	   SZ_224M			              0x0e000000 	
#define    SZ_256M                        0x10000000
#define    SZ_480M                        0x1e000000
#define    SZ_512M                        0x20000000
#define	   SZ_768M						  0x30000000
#define    SZ_1G                          0x40000000
#define    SZ_2G                          0x80000000

/* Board IDs */
#define IXD2810_BOARD_ID	0x2810

/* Number of device supported */
#define N_DEVICES	1

/* Port number found on IXD2810 */
#define N_PORTS		10

/* macro define for all Port enable */
#define ALL_PORTS	0x3ff

/* Converts a port number to the corresponding port mask bit definition */
#define	PORT_MASK(p)	(1 << p)

/* Memory size allocated for each device */
#define IXD2810_MAC_SIZE SZ_8M

/* Board Id location offset */
#define IXD2810_BOARDID_OFFSET	SZ_1M
#define IXD2810_BOARDID_MASK 	0xfffff  /* masks */

/* From IXDP2800 BSP file ixdp2000.h */
#define SLOWPORT_BASE			0xC4000000
#define MSF_SLOWPORT_BASE		(SLOWPORT_BASE + 0x02000000)	
#define MEDIA_SLOWPORT_BASE		MSF_SLOWPORT_BASE	
#define FABRIC_SLOWPORT_BASE	(SLOWPORT_BASE + 0x02800000)	


/*
 * Register access Macros
 */
#define IXD2810_REG_GET(reg, result) result=SlowPortRead(reg)
#define IXD2810_REG_SET(reg, data)	 SlowPortWrite(reg, data)

/* Provides the ioctl commands */
typedef enum 
	{
	IXD2810_IOCTL_GET,
	IXD2810_IOCTL_SET
	} IOCTL_CMD;

/* externs */
IMPORT uint32_t intLevelList[];
IMPORT char*  taskNames[];
/* 
 * Defines a data type with all possible memory bases 
 * allocated for their corresponding registers block.
 */
typedef enum 
	{
	PORT0_BLOCK,
	PORT1_BLOCK,
	PORT2_BLOCK,
	PORT3_BLOCK,
	PORT4_BLOCK,
	PORT5_BLOCK,
	PORT6_BLOCK,
	PORT7_BLOCK,
	PORT8_BLOCK,
	PORT9_BLOCK,
	GLOBAL_BLOCK,
	RX_BLOCK,
	TX_BLOCK,
	RESERVED_BLOCK,
	SPI42_BLOCK,
	SERDES_BLOCK,
	GBIC_BLOCK,
	BOARDID_BLOCK
	} BLOCK_ID;

LOCAL char *ixd2810BlockStr[] = 
	{
	"PORT0_BLOCK",
	"PORT1_BLOCK",
	"PORT2_BLOCK",
	"PORT3_BLOCK",
	"PORT4_BLOCK",
	"PORT5_BLOCK",
	"PORT6_BLOCK",
	"PORT7_BLOCK",
	"PORT8_BLOCK",
	"PORT9_BLOCK",
	"GLOBAL_BLOCK",
	"RX_BLOCK",
	"TX_BLOCK",
	"RESERVED_BLOCK",
	"SPI42_BLOCK",
	"SERDES_BLOCK",
	"GBIC_BLOCK",
	"BOARDID_BLOCK"
	};

/* device board ID base locations */
LOCAL uint32_t ixd2810BoardIdBase[] =
	{
	(UINT32) (MEDIA_SLOWPORT_BASE),
	(UINT32) (FABRIC_SLOWPORT_BASE)
	};


/* 
 * Device header parameters to commnicate between the application 
 * and the ixd2810 using the memory block access method.
 */
typedef struct ixd2810_block
	{
	BLOCK_ID 	blockId;		/* section of the memory mapped IO to access */
	uint32_t	startingReg;	/* register to access */
	uint32_t   	*data;	 		/* buffer for read/write information */
	uint32_t   	regNum;  		/* length of data buffer the appl expects */
	} block_t;

/*
 * Data maintained by the driver during the execution of the code 
 */
typedef struct _ixd2810_dev
	{
	DEV_HDR	  	devHdr;
	uint32_t	devId;			/* device ID */
	uint32_t 	vbase;			/* virtual address base */
	uint32_t	gbicStatus0;	/* GBIC Status from most recent int*/
	uint32_t	gbicStatus1;	/* GBIC Status history from previous int*/
    BOOL		created;		/* TRUE if this device has been created */
	BOOL		isRunning;		/* device current status/mode */
	int 		intCnt;			/* interrupt count */
    int 		intLevel;		/* interrupt level */
	int			taskId;			/* task ID spawn */
	SEM_ID	  	mutexSem;		/* mutex semaphore */
	SEM_ID	  	syncSem;		/* sync semaphore */
	DEV_PORTS 	ports[N_PORTS];	/* ports on device */
	} IXD2810_DEV;


/* Function prototypes */
uint32_t ixd2810_Create(uint32_t devId);
STATUS ixd2810_Start(uint32_t devId);
STATUS ixd2810_Stop(uint32_t devId);
STATUS ixd2810_Ioctl(uint32_t devId, uint32_t cmd, block_t *stBlock);
	
#endif	/* _ASMLANGUAGE */


#ifdef __cplusplus
}
#endif

#endif /* __INCixd2810h */


