/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/* ixd2810Int.h - IXD2810 MAC device Interrupt header file 

modification history
--------------------
01a, 08oct02,eme	created
01b, 18jun03,tep removed the include of ixdp2000.h, removed in level list and 
task names, increased the TASK_STACK_SIZE to 4000 from 2000.
01c,24jun03,tep added GPIO macro defines.
*/
#ifndef __INCixd2810Inth__
#define __INCixd2810Inth__

/* includes */
#include <vxWorks.h>
#include <logLib.h>
#include <ixd2810.h>

/* defines */

#define INT_LVL_GPIO    		3 	/* GPIO unit IRQ */
#define	CPLD_INT_FAB			4
#define	CPLD_INT_MED			5

/*
 * GPIO Register Definitions
 */
#define GPIO_BASE_ADRS     	0xC0010000			/* GPIO Base Address */
#define GPIO_PLR   			(0xC0010000+0x00)  	/* pin level reg */
#define GPIO_PDPR   		(0xC0010000+0x04)  	/* pin level reg */
#define GPIO_PDSR  			(0xC0010000+0x08)  	/* pin direction set reg */
#define GPIO_PDCR  			(0xC0010000+0x0C)  	/* pin direction set reg */
#define GPIO_POPR  			(0xC0010000+0x10)
#define GPIO_POSR  			(0xC0010000+0x14)  	/* output data set reg */
#define GPIO_POCR  			(0xC0010000+0x18)  	/* output data clear reg */
#define GPIO_REDR  			(0xC0010000+0x1C)  	/* output data clear reg */
#define GPIO_FEDR  			(0xC0010000+0x20)  	/* output data clear reg */
#define GPIO_EDSR  			(0xC0010000+0x24)  	/* output data clear reg */
#define GPIO_LSHR  			(0xC0010000+0x28)  	/* output data clear reg */
#define GPIO_LSLR  			(0xC0010000+0x2C)  	/* output data clear reg */
#define GPIO_LDSR  			(0xC0010000+0x30)  	/* output data clear reg */
#define GPIO_INER  			(0xC0010000+0x34)  	/* output data clear reg */
#define GPIO_INSR  			(0xC0010000+0x38)  	/* output data clear reg */
#define GPIO_INCR  			(0xC0010000+0x3C)  	/* output data clear reg */
#define GPIO_INST  			(0xC0010000+0x40)  	/* Int status reg */

/* initialize GBIC Control interrupt value */
#define GC_INT_ENABLE (GC_Rx_Loss_Enable  | \
					   GC_Tx_Fault_Enable | \
					   GC_Mod_Def_Enable)
#define GC_INT_DISABLE 0
	
/* variables */

/* task spawn parameters */
#define TASK_PRIORITY		100
#define TASK_OPTS			0
#define TASK_STACK_SIZE 	4000

/* Link Up Register */
#define LINK_UP(port)			(0x1 << port)	/* bits 9:0 */

/* Port Enable Register */
#define PORT_EN(port)			(0x1 << port)	/* bits 9:0 */

#define ENABLE_BOUNDARY	0xFFFF

/* function prototypes */
STATUS ixd2810IntDevInit(IXD2810_DEV *pDev);
STATUS ixd2810IntDevRelease(IXD2810_DEV *pDev);
void ixd2810EnablePort(IXD2810_DEV *pDev, uint32_t port); 
void ixd2810DisablePort(IXD2810_DEV *pDev, uint32_t port);

#endif /* INCixd2810Inth__ */

