/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/* ixd2810sp.c - IXD2810 MAC device driver 

modification history
--------------------
01b,16jun03,tep created.
01c,25jun03,tep Added volatile directive to all reg accesses.
*/

#include <intLib.h>

/* Local defines */
#define CCR 	0xc0080000

LOCAL uint32_t 	temp_ccr;

/*******************************************************************************
 * sp_config - 
 *
 * RETURNS: void
 */
void 
sp_config(void)
{
	uint32_t data;

    volatile uint32_t *ptr = (uint32_t *) CCR;
    temp_ccr = *ptr;
	data = ((temp_ccr & ~0xF)|2);	/*force divide by 4 for slow port clk*/
    *ptr = data;
}

/*******************************************************************************
 * restore_sp_config - 
 *
 * RETURNS: void
 */
void 
restore_sp_config(void)
{

    volatile uint32_t *ptr = (uint32_t *) CCR;
    *ptr = temp_ccr;
}

/*******************************************************************************
 * SlowPortRead - 
 *
 * RETURNS: data from CSR read
 */
uint32_t                                                                          
SlowPortRead(volatile uint32_t *addr)
{
    int key;
    uint32_t val;

    key = intLock();

    sp_config();

    val =  *addr;

    restore_sp_config();

    intUnlock(key);

    return val;
}

/*******************************************************************************
 * SlowPortWrite - 
 *
 * RETURNS: void
 */
void
SlowPortWrite(volatile uint32_t *addr, uint32_t val)
{
    int key;

    key = intLock();

    sp_config();

    *addr = val;

    restore_sp_config();

    intUnlock(key);
}
