/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:58p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810Autoneg.c $
*   WORKFILE:       $Workfile: ixd2810Autoneg.c $
*
 *-------------------------------------------------------------------------------
 *	Description:
 *-------------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810Autoneg.c $
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 3     10/13/03 4:55p Tepulkow
 * added include if ixd2810sp.h
 * 
 * 2     10/07/03 2:01p Tepulkow
 * General cleanup for code review
 * 
* 
* --------------------------------------------------------------------------
*/

#include <linux/module.h>       /* module */
#include <linux/slab.h>
#include <linux/time.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/arch/irqs.h>

#include "portstat.h"
#include "ixd2810sp.h"
#include "ixd2810Board.h"
#include "ixd2810Board_s.h"
#include "ixd2810Autoneg.h"
#include "version.h"
#include "ixd2810Mmap.h"


char *szSpeed[] = {"10M", "100M", "1000M"};
char *szDuplex[] = {"Half", "Full"};
char *szLink[] = {"Down", "Up"};


/*******************************************************************************
 * performAutoneg - Starting point of auto-negotiation.
 * 
 * RETURNS: N/A
 */
void performAutoneg
    (
    IXD2810_DEV *pDev,   /* device structure pointer */
    int port             /* port number to work on */
    ) 
    {

  /* Initialize link/speed/duplex status */
  pDev->portStatus[port].link = DOWN;
  pDev->portStatus[port].duplex = HALF;
  pDev->portStatus[port].speed = SPEED_10;
  
    doFiberAutoneg(pDev, port);	/* fiber auto-negotiation */
    /* Fiber mode only supports giga speed and full duplex */
    if (pDev->portStatus[port].link == UP ) {	/* link up */
      if ((pDev->portStatus[port].speed == SPEED_1000) && 
	  (pDev->portStatus[port].duplex == FULL)) {
        configMac(pDev, port);  /* Configure MAC */
        printMsg(pDev, port, 1);   /* Print message */
      } else {
        disablePort(pDev, port);	     /* Disable port */
        printMsg(pDev, port, 2);   /* Print message */
      }
    } else {				/* link down */
      disablePort(pDev, port);
      printMsg(pDev, port, 3);
    }
}

/*******************************************************************************
 * configMac - Configure specified MAC port.  (link/speed/duplex)
 * 
 * RETURNS: N/A
 */
void configMac
    (
    IXD2810_DEV *pDev,   /* device structure pointer */
    int port             /* port number to work on */
    ) 
    {
	u32 data;
	
	/* Duplex */
	data = PORT_RD((u32 *)pDev->vbase + DESIRED_DUPLEX(port)); 
	data &= 0xFFFFFFFE;		/* Clear bit 0 */

	if (pDev->portStatus[port].duplex == FULL) {
		data |= REG_DUPLEX_FULL;
	}
    PORT_WR(data, (u32 *)pDev->vbase + DESIRED_DUPLEX(port));     

	/* Set link up*/
	data = PORT_RD((u32 *)pDev->vbase + GLOBAL_ADRS(LINK_LED_ENABLE));
	data |= LINK_UP(port);
    PORT_WR(data, (u32 *)pDev->vbase + GLOBAL_ADRS(LINK_LED_ENABLE));

	/* Enable port */
	data = PORT_RD((u32 *)pDev->vbase + GLOBAL_ADRS(PORT_ENABLE));
	data |= PORT_EN(port);
    PORT_WR(data, (u32 *)pDev->vbase + GLOBAL_ADRS(PORT_ENABLE));
}

/*******************************************************************************
 * doFiberAutoneg - Fiber auto-negotiation routine.
 * 
 * RETURNS: N/A
 */
void doFiberAutoneg
    (	
    IXD2810_DEV *pDev, /* device with corresponding port to service */
    int port	       /* port number. From 0 to 9. */
    ) 
    {

	u32 data;
	unsigned long ulDelay = jiffies + AN_TIMEOUT * HZ;

	/* Start auto-negotiation */
	data = PORT_RD((u32 *)pDev->vbase + BLOCK_REG_ADRS(port, DIV_CONFIG_WORD));
	data |= AUTONEG_ENABLE;
    PORT_WR(data, (u32 *)pDev->vbase + BLOCK_REG_ADRS(port, DIV_CONFIG_WORD));

	/* Poll Rx Config Word to determine when AN is complete or timeout */
	while (jiffies < ulDelay) {
		data = PORT_RD((u32 *)pDev->vbase + BLOCK_REG_ADRS(port, RX_CONFIG_WORD));
	
		if (data & FB_AN_COMPLETE)
			break;
	}


	if (data & FB_AN_COMPLETE) { /* successsful polling */
		/* Speed is fixed at giga */
		pDev->portStatus[port].speed = SPEED_1000;

		/* Get port status from Rx Config Word */
		if (data & HALF_DUPLEX) 
			pDev->portStatus[port].duplex = HALF;	
		if (data & FULL_DUPLEX)
			pDev->portStatus[port].duplex = FULL;

		/* Link is up when autoneg is enabled and finished */
		pDev->portStatus[port].link = UP;
	} else {  /* timeout occured, return failure */
		pDev->portStatus[port].link = DOWN;
	}

}

/*******************************************************************************
 * printMsg - Print message for current mode.
 * 
 *	mode -  1 for supported link mode.
 *	        2 for non-supported link mode.
 *		    3 for no link mode.
 * RETURNS: N/A
 */
void printMsg
    (
	IXD2810_DEV *pDev, /* device with corresponding port to service */
    int port,	       /* port number. From 0 to 9. */
    int mode           /* link mode */
    ) 
    {
	int 	speed, duplex;

	/* Get string index */
	speed = pDev->portStatus[port].speed;
	duplex = pDev->portStatus[port].duplex;

	if (mode == 1) {	/* supported link mode */
	   printk("  MAC%d port%d is configured to %s speed and %s duplex.\n",
	        	pDev->devId, port, szSpeed[speed], szDuplex[duplex]);
	}
	else {			
	   printk("  MAC%d port%d is disabled", pDev->devId, port);
	
	   if (mode == 2) {	/* parallel detection for forced 100M LP */
	      printk(" because half duplex is not supported.\n");
	   }
	   else {		/* no link mode */
	     printk(" because no link is established.\n");
	   }
	}
}

