/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:58p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810Int.c $
*   WORKFILE:       $Workfile: ixd2810Int.c $
*
 *-------------------------------------------------------------------------------
 *	Description:
 *-------------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810Int.c $
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 9     10/21/03 10:53a Tepulkow
 * Removed commented out code from the module
 * 
 * 8     10/13/03 4:55p Tepulkow
 * added include if ixd2810sp.h
 * 
 * 6     10/07/03 2:01p Tepulkow
 * General cleanup for code review
 * 
* 
* --------------------------------------------------------------------------
*/
#include <linux/module.h>	/* module */
#include <linux/sched.h>
#include <linux/tqueue.h>	/* task queue */
#include <linux/interrupt.h>	/* mark_bh */
#include <asm/uaccess.h>
#include <asm/irq.h>		/* interrupts */	
#include <asm/io.h>
#include <asm/hardware.h>

#include "portstat.h"
#include "ixd2810sp.h"
#include "ixd2810Board.h"
#include "ixd2810Board_s.h"
#include "ixd2810Mmap.h"
#include "version.h"

#include "devlib.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "ixf1110_alarms_d.h"
#include "ixd2810Int.h"

#define GPIO_INST	0x40
#define	GPIO1		0x1

/* Global pointer to chipdata structure */
static bb_ChipData_t *pChipDataIsr;


/*******************************************************************************
 * init_module - Initialize the interrupt handler for the MAC device.
 *
 * RETURNS:  OK.
 */
int init_module (void) 
{

/* We do nothing here as all init will be handled in ixd2810IsrInit() below... */

	return 0;
}

/*******************************************************************************
 * cleanup_module - Removes the interrupt handler.
 *
 * Uninitialize any configuration previous done by the ixd2810IntDevInit(),
 * releases any resources acquired during the intialization process.
 *
 * RETURNS:  ERROR in case an error occurs, otherwise OK.
 */
void cleanup_module (void) 
{
	IXD2810_DEV *pDev;
	pDev = &ixd2810Dev[0];
                                  
	/* Disable interrupt in GBIC control register */	
	PORT_WR(GC_INT_DISABLE, (u32 *)pDev->vbase + GBIC_ADRS(GBIC_CONTROL));

	/* Free IRQs for devices on board */
	free_irq(IRQ_IXP2000_GPIO1, NULL);
   
}

/*******************************************************************************
 *	ixd2810IsrInit - 	 
 *
 * RETURNS: N/A
 */
int ixd2810IsrInit
    (
    bb_ChipData_t *pChipData	/* */
    ) 
{
	int port,ret;	
	IXD2810_DEV *pDev;

	/* get pointer to device structure */
	pDev = &ixd2810Dev[0];
	pDev->devId = 0;
	
	/* Copy virtual address from caller to local structure */
	(bb_RegPointer_type) pDev->vbase = pChipData->BaseAddress;
	pChipDataIsr = pChipData;

	
	/* Read GBIC status value */
	pDev->gbicStatus = PORT_RD((u32 *)pDev->vbase + GBIC_ADRS(GBIC_STATUS));
    
	/* Assign function and argument for task queue element */
	pDev->stModDefTask.routine = getModState; 
	pDev->stModDefTask.data = (void *) pDev; 
	
	/* Initialize autoneg flag to zero */
	for (port=0; port< N_PORTS; port++) {
		pDev->doAutoNeg[port] = FALSE;
	}

	/* Install interrupt handlers for 2810 */
 	set_GPIO_IRQ_level(GPIO1, GPIO_LEVEL_LOW);
	ret = request_irq(IRQ_IXP2000_GPIO1, ixd2810Isr, 0, taskNames[pDev->devId], NULL);
	if (ret != 0) {
		printk("Can't get assigned irq %d\n", pDev->irq);		
		return ret;
	}	

	/* Enable interrupt in GBIC control register */
	PORT_WR(GC_INT_ENABLE, (u32 *)pDev->vbase + GBIC_ADRS(GBIC_CONTROL));

	return 0;
}
/*******************************************************************************
 *	ixd2810Isr - 	Call individual ISRs based on GBIC value. 
 *
 * RETURNS: N/A
 */
void ixd2810Isr
    (
    int irq,                /* interrupt number */
    void *dev_id,           /* pointer to client data */
    struct pt_regs *regs    /* processor's context  */
    ) 
    {

	u32 gstatus;
	IXD2810_DEV *pDev;

	/* get pointer to corresponding device structure */
	pDev = &ixd2810Dev[0];

	/* GBIC status register value */
	gstatus = PORT_RD((u32 *)pDev->vbase + GBIC_ADRS(GBIC_STATUS));

	/* Clear the GPIO1 interrupt */
	*(volatile unsigned int*) (GPIO_VIRT_BASE+GPIO_INST) = 0x2;

	/* Find which bit(s) changed */
	irq = gstatus ^ pDev->gbicStatus;
    
	/* Save current GBIC status value */
	pDev->gbicStatus = gstatus;


	((ixf1110_AlarmCfg_t *)pChipDataIsr->pAlarmCfg)->pAlarmCfg->Gbic_Status.ulReg = gstatus;
	((ixf1110_AlarmCfg_t *)pChipDataIsr->pAlarmCfg)->pAlarmCfg->Gbic_Change.ulReg = irq;

	if (irq & GS_Rx_Loss) {		/* Rx_Loss_Int */
		rxLossIsr(pDev, irq, gstatus);
	}
	if (irq & GS_Tx_Fault) {		/* Tx_Fault_Int */
		txFaultIsr(pDev, irq, gstatus);
	}
	if (irq & GS_Mod_Def) {		/* Mod_Def_Int */
		modDefIsr(pDev, irq, gstatus);
	}
	
	/* IXF API Callback here */
	IxfApiChipIsr(pChipDataIsr);

}

/*******************************************************************************
 * rxLossIsr - Display Rx Loss on appropriate device and port.
 *
 * RETURNS: N/A
 */
void rxLossIsr
    (
    IXD2810_DEV *pDev, /* device structure pointer */
    u32 irq,           /* changed bits in GBIC status */
    u32 status         /* current GBIC status value */
    ) 
    {
	
	u32    rxBitIndex = GS_RX_STARTING_BIT;  /* GBIC_STATUS bits 29:20 */
	int 	port;
	
	for (port=0; port< N_PORTS; port++, rxBitIndex <<= 1) {
		if (irq & rxBitIndex) {  /* bit changed for this port */
			if (status & rxBitIndex) {
				disablePort(pDev, port);	
				printk("Rx loss appeared on MAC%d port%d.\n", pDev->devId, port);
			} else {
				enablePort(pDev, port);
				printk("Rx loss disappeared on MAC%d port%d.\n ", pDev->devId, port);
			}
		}
	}

}

/*******************************************************************************
 * txFaultIsr - Display Tx Fault on appropriate device and port.
 *
 * RETURNS: N/A
 */
void txFaultIsr
    (
    IXD2810_DEV *pDev, /* device structure pointer */
    u32 irq,           /* changed bits in GBIC status */
    u32 status         /* current GBIC status value */
    ) 
    {

	u32	txBitIndex = GS_TX_STARTING_BIT; /* GBIC_STATUS bits 19:10 */
	int 	port;
	
	for (port=0; port< N_PORTS; port++, txBitIndex <<= 1) {
		if (irq & txBitIndex) {  /* bit changed for this port */
			if (status & txBitIndex)
				printk("Tx fault appeared on MAC%d port%d.\n", 
                    pDev->devId, port);
			else
				printk("Tx fault disappeared on MAC%d port%d.\n ", 
                    pDev->devId, port);
		}
	}

}

/*******************************************************************************
 * modDefIsr - Interrupt Service Routine for Mod_Def_Int. 
 *	If module was present before, display module removed on device and port.
 *	If module was not present before, start auto-negotiation on device and port.
 *
 *	In order to make ISR return shortly, put auto-negotiation into 
 * immediate task queue to execute outside ISR.     
 *
 * RETURNS: N/A
 */
void modDefIsr
    (
    IXD2810_DEV *pDev, /* device structure pointer */
    u32 irq,           /* changed bits in GBIC status */
    u32 status         /* current GBIC status value */
    ) 
    {
	    
	u32 	modBitIndex = GS_MOD_STARTING_BIT;		
	int 	port;
	
	for (port=0; port< N_PORTS; port++, modBitIndex <<= 1) {
		if (irq & modBitIndex) {  /* bit changed for this port */

			/* if bit=1, module is not present */
			if (status & modBitIndex) {
				disablePort(pDev, port);	
				printk("Module removed from MAC%d port%d.\n", 
                    pDev->devId, port);
				pDev->doAutoNeg[port] = FALSE;
			} else {  /* if bit=0, module is present */
				printk("Module inserted into MAC%d port%d.\n", 
                    pDev->devId, port);
				pDev->doAutoNeg[port] = TRUE;		
			}
		} /* end if */
	} /* end for */
	
	/* Queue the first auto-negotiation task */
	for (port=0; port<N_PORTS; port++) {
		if (pDev->doAutoNeg[port] == TRUE) {
			printk("First task to be queued for port %d\n", port);
			pDev->port = port;
			
			/* Put in immediate task queue */
			queue_task(&pDev->stModDefTask, &tq_immediate);  	
			/* Muse mask as bottom half */
			mark_bh(IMMEDIATE_BH);				
			/* Clear auto-negotiation flag */
			pDev->doAutoNeg[port] = FALSE;
			break;
		} else
			printk("First task cannot be queued for port %d\n", port);
	}
}

/*******************************************************************************
 *	getModState - function to call for auto-negotiation task.
 *
 *	RETURNS: N/A
 */
void getModState
    (
    void *ptr /* device structure pointer */
    ) 
    {

	/* Retrieve task data */
	IXD2810_DEV *pDev = (IXD2810_DEV *)&ptr;

	pDev = &ixd2810Dev[0];  
		
	/* Start auto-negotiation */
#ifdef DEBUG
	printk("Start autoneg!\n");
#else
	performAutoneg(pDev, pDev->port);	
#endif
	
	/* Queue the next auto-negotiation task */
	while (++pDev->port < N_PORTS) {
		if (pDev->doAutoNeg[pDev->port] == TRUE) {
			printk("Task about to be queued for port %d\n",pDev->port);
			queue_task(&pDev->stModDefTask, &tq_immediate); 	
			mark_bh(IMMEDIATE_BH);
			pDev->doAutoNeg[pDev->port] = FALSE;
			break;
		} else
			printk("Unable to queue task for port %d\n", pDev->port);
	}

}


/*******************************************************************************
 * disablePort - Disable MAC port and set link down.
 *
 * RETURNS:  N/A
 */
void disablePort
	(
	IXD2810_DEV *pDev, /* device with corresponding port to service */
	int port /* port number. From 0 to 9. */
	) 
	{
	
	u32 data;

	/* bits 9:0 for port enable. 0=disable */
	data = PORT_RD((u32 *) pDev->vbase + GLOBAL_ADRS(PORT_ENABLE));
	data &= (ENABLE_BOUNDARY - PORT_EN(port));
	PORT_WR(data, (u32 *)pDev->vbase  + GLOBAL_ADRS(PORT_ENABLE));

	/* bits 9:0 for link up. 0=down */
	data = PORT_RD((u32 *)pDev->vbase + GLOBAL_ADRS(LINK_LED_ENABLE));
	data &= (ENABLE_BOUNDARY - LINK_UP(port));
	PORT_WR(data, (u32 *)pDev->vbase  + GLOBAL_ADRS(LINK_LED_ENABLE));
}

/*******************************************************************************
 *	enablePort - Enable MAC port and set link up.
 *
 *	RETURNS: N/A
 */
void enablePort
	(
	IXD2810_DEV *pDev, /* device with corresponding port to service */
	int port /* port number. From 0 to 9. */
	) 
	{
	
	u32 data;
    
	/* bits 9:0 for port enable. 1=enable */
	data = PORT_RD((u32 *)pDev->vbase + GLOBAL_ADRS(PORT_ENABLE));
	data |= PORT_EN(port);
	PORT_WR(data, (u32 *)pDev->vbase + GLOBAL_ADRS(PORT_ENABLE));

	/* bits 9:0 for link up. 1=up */
	data = PORT_RD((u32 *)pDev->vbase + GLOBAL_ADRS(LINK_LED_ENABLE));	
	data |= LINK_UP(port);  
	PORT_WR(data, (u32 *)pDev->vbase + GLOBAL_ADRS(LINK_LED_ENABLE));
}

