/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:58p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810sp.c $
*   WORKFILE:       $Workfile: ixd2810sp.c $
*
 *-------------------------------------------------------------------------------
* Purpose:  The purpose of this module is to provide any slow port configuation 
* 			required to allow reliable read/write access to the IXD2810 media
*			card.  If the slow prto configuration needs to be changed, the 
*			original configuration must be saved and restored around any 
*			modofications.
*          
 *-------------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXD2810INT/autoneg/ixd2810sp.c $
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 2     10/21/03 10:29a Tepulkow
 * Added a description to the "Purpose" field in the module header.
 * 
 * 1     10/13/03 4:59p Tepulkow
 * created for slowport workaround
* 
* --------------------------------------------------------------------------
*/

#include <asm/hardware.h>
#include <asm/io.h>


/* Local defines */
static ulong ixd2810temp_ccr;

/*******************************************************************************
 * ixd2810sp_config - 
 *
 * RETURNS: void
 */
void 
ixd2810sp_config(void)
{
    ulong data;
    volatile ulong *ptr = (ulong *) IXP2000_SLOWPORT_CCR;
    ixd2810temp_ccr = *ptr;
    data = ((ixd2810temp_ccr & ~0xF)|2);	/*force divide by 4 for slow port clk*/
    *ptr = data;
}

/*******************************************************************************
 * ixd2810restore_sp_config - 
 *
 * RETURNS: void
 */
void 
ixd2810restore_sp_config(void)
{
    volatile ulong *ptr = (ulong *) IXP2000_SLOWPORT_CCR;
    *ptr = ixd2810temp_ccr;
}

/*******************************************************************************
 * ixd2810SpRead - 
 *
 * RETURNS: data from CSR read
 */
ulong                                                                          
ixd2810SpRead(volatile u32 *addr)
{
    ulong val;

    ixd2810sp_config();
    val = le32_to_cpu(readl(addr));
    ixd2810restore_sp_config();
    return val;
}



