/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/common_def.h $
*   WORKFILE:       $Workfile: common_def.h $
*
* --------------------------------------------------------------------------
* Project: All
*
* Purpose: This file defines types that are common to all IXF API and driver 
*          modules
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/common_def.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 3     10/07/03 2:02p Tepulkow
 * General cleanup for code review
 * 
* --------------------------------------------------------------------------
*/
#ifndef __COMMON_DEF_H__
#define __COMMON_DEF_H__

/******************************************************************************/
/* Including ixf_api_error_d.h here and defining bb_Error_e as a ushort       */
/* because error codes are being moved from common_def.h. and being defined   */
/* via #defines vs. enumerated type.                                          */  
/******************************************************************************/
#include "ixf_api_error_d.h"

typedef ushort bb_Error_e;      /* legacy support */


/******************************************************************************/
/* Common enum definintions                                                   */
/******************************************************************************/
/* IMPORTANT!!!!!! READ THIS BEFORE MODIFYING CODE BELOW      */
/*      Changing the order, or Adding Chip Types MAY affect   */
/*      The Serialize method in CSftPlatformDoc. Proceed with */
/*      Caution.                                              */
typedef enum   /* Identifies type of chip */
{
    bb_FPGA_6048_CHIP = 0,
        bb_6051_CHIP,         
        bb_IXF30007_CHIP,           /* IXF30007 Chip */
        bb_6048_CHIP,
        bb_FPGA_GEOS,               /* GEOS FPGA */
        bb_6192_CHIP,
        bb_FPGA_INS_6192,
        bb_FPGA_EXT_6192,
        bb_18101_CHIP,              /* IXF18101 Chip */
        bb_LXT16708_CHIP,           /* LXT16708 Chip */
        bb_IXF30009_CHIP,           /* IXF30009 Chip */
        bb_IXF30025_CHIP,           /* IXF30025 Chip */ 
        bb_IXF30005_CHIP,           /* IXF30005 Chip */  
        bb_16121_CHIP,              /* IXF16121 Chip */
        bb_FPGA_INS_18100,          /* STB insertion FPGA for 1810x */
        bb_FPGA_EXT_18100,          /* STB extraction FPGA for 1810x */
        bb_FPGA_SIF_18100,          /* SIF FPGA for ixf1810x */
        bb_FPGA_OIFMISC_18100,      /* OIF/Misc. board control logical FPGA for 1810x */
        bb_FPGA_INS_SOH_16121,      /* STB SOH Insertion FPGA for IXF 16121 */
        bb_FPGA_EXT_SOH_16121,      /* STB SOH Extraction FPGA for IXF 16121 */
        bb_FPGA_INS_SALPAL_16121,   /* STB SAL/PAL Insertion FPGA for IXF 16121 */
        bb_FPGA_EXT_SALPAL_16121,   /* STB SAL/PAL Extraction FPGA for IXF 16121 */
        bb_DVBOARD_16121,           /* DV Board for IXF 16121 */
        bb_GENERIC_CHIP,            /* Generic Chip */
        bb_IXF3502_CHIP,            /* IXF3502 */
        bb_IXF3503_CHIP,            /* IXF3503 */
        bb_18102_CHIP,              /* IXF18102 Chip */
        bb_18103_CHIP,              /* IXF18103 Chip */
        bb_18104_CHIP,              /* IXF18104 Chip */
        bb_18101_8023WIS_CHIP,      /* IXF18101 (with 802.3 WIS block) Chip */
        bb_18103_8023WIS_CHIP,      /* IXF18103 (with 802.3 WIS block) Chip */
        bb_IXF19300_CHIP,           /* IXF19300 */
        bb_IXF19301_CHIP,           /* IXF19301 */
        bb_IXF19302_CHIP,           /* IXF19302 */
        bb_IXF19303_CHIP,           /* IXF19303 */
        bb_LXT16708_DV_CHIP,        /* LXT16708 Dv Chip */ 
        bb_LXT16708_PI_CHIP,        /* LXT16708 Pi Chip */ 
        bb_LXT16708_DV_PI_CHIP,     /* LXT16708 Dv Pi Chip */
        bb_IXF35032_CHIP,           /* IXF35032 */
        bb_IXF30009_EVB_PLD,        /* IXF30009 EVB PLD */
        /*Rossy Sif*/
        bb_FPGA_SIF_1930X,        /* SIF FPGA for Ixf1930x Chip */
        /* WARNING: Add your new chip here ONLY */
		bb_1110_CHIP,
        bb_MAX_NUM_CHIPS
} bb_ChipType_e;

/* Define a Register pointer type that can be casted 
based upon the Chip type (16 bit or 8 bit) */

#ifdef IXF_REG_POINTER_32
typedef ulong *bb_RegPointer_type;
#elif IXF_REG_POINTER_8
typedef uchar *bb_RegPointer_type;
#else
typedef ushort *bb_RegPointer_type;
#endif

#ifdef IXF_REG_DATA_32
typedef ulong bb_RegDataValue_type;
#elif IXF_REG_DATA_8
typedef uchar bb_RegDataValue_type;
#else
typedef ushort bb_RegDataValue_type;
#endif


/******************************************************************************/
/* Common #defines                                                            */
/******************************************************************************/
#ifndef TRUE
#define  TRUE  1
#endif
#ifndef FALSE
#define  FALSE 0
#endif

#ifndef  NULL
#define  NULL  ((void*)0)  /* For use as NULL pointer value */
#endif

/******************************************************************************/
/* Common #defines for dimensioning arrays & loop counters                    */
/******************************************************************************/
#define CHANNEL_12  12
#define CHANNEL_16  16
#define CHANNEL_24  24
#define CHANNEL_32  32
#define CHANNEL_48  48
#define CHANNEL_64  64
#define CHANNEL_192 192
#define CHANNEL_256 256

#define GROUP_4     4
#define GROUP_12    12
#define GROUP_16    16
#define GROUP_64    64

#define SLOT_12     12
#define SLOT_192    192

/* Defined word sizes */
typedef enum 
{
    ONE_BYTE    =   1,
        TWO_BYTES   =   2,
        FOUR_BYTES  =   4,
        EIGHT_BYTES =   8,
        MAX_BYTES
} bb_Word_Size_t;

typedef struct
{
    ulong    ulLowValue;
    ulong    ulHighValue;
} uint64Struct_t;

/* Defines the structure to be used by the InitChip routine */
typedef struct initRegTable
{
    bb_RegPointer_type    offset;
    bb_RegDataValue_type  value;
} InitRegTable_t;

/* Function pointer for Alarm Callbacks */  
typedef void (*AlarmCallBack)(
                              void *pChipData, 
                              ulong AlarmType,                /* Type of Alarm */
                              void *pAlarmStatusCounters);    /* -> Alarm status counter ptr*/


#ifdef INCLUDE_TRACE

#ifdef SIMULATOR
extern void PrintToView(char *sString);

#define IXF_FUNC_ENTRY_TRC(fmtStr)      \
    do {\
    char Buffer[129] = "Entering: ";\
    strcat(Buffer, fmtStr);\
    strcat(Buffer, "\r\n");\
    PrintToView(Buffer);\
    } while(0);

#define IXF_FUNC_EXIT_TRC(fmtStr, retVal)   \
    do {\
    char Buffer[129] = "Exiting: ";\
    strcat(Buffer, fmtStr);\
    strcat(Buffer, " with Return Code: %d\r\n");\
    sprintf(Buffer, Buffer, retVal);\
    PrintToView(Buffer);\
    } while(0);

#else /* SIMULATOR */

#define IXF_FUNC_ENTRY_TRC(fmtStr)      printf(fmtStr)

#define IXF_FUNC_EXIT_TRC(fmtStr, retVal)   printf(fmtStr, retVal)

#endif /* SIMULATOR */

#else /* INCLUDE_TRACE */

#define IXF_FUNC_ENTRY_TRC(fmtStr)

#define IXF_FUNC_EXIT_TRC(fmtStr, retVal)

#endif /* INCLUDE_TRACE */

#endif /* __COMMON_DEF_H__ */
