/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/devlib.c $
*   WORKFILE:       $Workfile: devlib.c $
*
* --------------------------------------------------------------------------
* Project: IXF6192
*
* Purpose: Provides the definition of the chip array function pointer table.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/devlib.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 7     10/10/03 5:00p Tepulkow
 * added setchiponline and setchipoffline to ixf1110 functions
 * 
 * 6     10/07/03 2:02p Tepulkow
 * General cleanup for code review
 * 
* --------------------------------------------------------------------------
*/ 

#ifdef VXWORKS
#include "vxWorks.h" 
#endif

#include "common_types.h"
#include "common_def.h"
#include "devlib.h" 

#ifdef INCLUDE_GENERIC_DRIVER
#include "genericdriver.h"
#endif

#ifdef SW_DOWNLOAD
#include <symLib.h>
#include "msgQLib.h"
#include "semLib.h"
#include "os.h"
extern SYMTAB_ID  sysSymTbl;
#endif


/*****************************************************************************
****                             IXF6192                                  ****
*****************************************************************************/
#ifdef INCLUDE_6192_LIB
/* Function Pointers for the Ixf6192 Chip */
CommonFunctionPointers_t ixf6192CommonFuncPtrs = 
{
    Ixf6192InitChip,                /* InitChip */
        Ixf6192InitAllocMemory,         /* InitAllocMemory */
        Ixf6192DeAllocMemory,           /* DeAllocMemory */
        NULL,                           /* GetStatus */
        NULL,                           /* GetOpMode */
        NULL,                           /* SetOpMode */
        Ixf6192Reset,                   /* Reset */
        Ixf6192InitAlarmCallback,       /* InitAlarmCallback */
        Ixf6192ChipIsr,                 /* ChipIsr */
        Ixf6192CfgTest,                 /* CfgTest */
        Ixf6192GetCounters,             /* GetCounters */
        Ixf6192GetChipInfo,             /* GetChipInfo */
        Ixf6192SetChipOnline,           /* SetChipOnline */
        Ixf6192SetChipOffline,          /* SetChipOffline */
        Ixf6192SetAlarmCfg,             /* SetAlarmCfg */
        Ixf6192GetBuildVersion,         /* GetBuildVersion */
        Ixf6192Read,                    /* Read */
        Ixf6192Write,                   /* Write */
        Ixf6192GetCfg,                  /* GetCfg */
        Ixf6192SetCfg                   /* SetCfg */
};


SonetFunctionPointers_t ixf6192SonetFuncPtrs = 
{
    
    Ixf6192GetWindowSize,            /* GetWindow */
        Ixf6192SetWindowSize,           /* SetWindow */
        NULL,                           /* SetBERThreshold */
        Ixf6192GetTrace,                /* GetTrace */
        Ixf6192SetTrace,                /* SetTrace */
        Ixf6192GetOhBytes,              /* GetOhBytes */
        Ixf6192SetOhByte,               /* SetOhBytes */
        Ixf6192SetRmstSourceBytes,      /* SetRmstSourceBytes */
        Ixf6192GetRmstSourceBytes,      /* GetRmstSourceBytes */
        Ixf6192MapFrames,               /* MapFrame */
        Ixf6192GetMappedFrames,         /* GetMappedFrame */
        Ixf6192GetFramesAuPointer       /* GetAuPointer */
};


ChipFunctionPointers_t ixf6192FuncPtrs = 
{
    &ixf6192CommonFuncPtrs,             /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf6192SonetFuncPtrs,               /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_6192_LIB */


/*****************************************************************************
****                             IXF6048                                  ****
*****************************************************************************/
#ifdef INCLUDE_6048_LIB
/* Function Pointers for the Ixf6048 Chip */
CommonFunctionPointers_t ixf6048CommonFuncPtrs = 
{
    Ixf6048InitChip,                /* InitChip */
        Ixf6048InitAllocMemory,         /* InitAllocMemory */
        Ixf6048DeAllocMemory,           /* DeAllocMemory */
        Ixf6048GetStatus,               /* GetStatus */
        Ixf6048GetOpMode,               /* GetOpMode */
        Ixf6048SetOpMode,               /* SetOpMode */
        Ixf6048Reset,                   /* Reset */
        Ixf6048InitAlarmCallback,       /* InitAlarmCallback */
        Ixf6048ChipIsr,                 /* ChipIsr */
        Ixf6048CfgTest,                 /* CfgTest */
        Ixf6048GetCounters,             /* GetCounters */
        Ixf6048GetChipInfo,             /* GetChipInfo */
        Ixf6048SetChipOnline,           /* SetChipOnline */
        Ixf6048SetChipOffline,          /* SetChipOffline */
        Ixf6048SetAlarmCfg,             /* SetAlarmCfg */
        Ixf6048GetBuildVersion,         /* GetBuildVersion */
        Ixf6048Read,                    /* Read */
        Ixf6048Write,                   /* Write */
        NULL,                           /* GetCfg */
        NULL                            /* SetCfg */
};


SonetFunctionPointers_t ixf6048SonetFuncPtrs = 
{
    Ixf6048GetWindowSize,           /* GetWindow */
        Ixf6048SetWindowSize,           /* SetWindow */
        NULL,                           /* Set BER Threshold */
        Ixf6048GetTrace,                /* GetTrace */
        Ixf6048SetTrace,                /* SetTrace */
        Ixf6048GetOhBytes,              /* GetOhBytes */
        Ixf6048SetOhByte,               /* SetOhBytes */
        NULL,                           /* SetRmstSourceBytes */
        NULL,                           /* GetRmstSourceBytes */
        NULL,                           /* MapFrame */
        NULL,                           /* GetMappedFrame */
        NULL                            /* GetAuPointer */
};


ChipFunctionPointers_t ixf6048FuncPtrs = 
{
    &ixf6048CommonFuncPtrs,         /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf6048SonetFuncPtrs,          /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};

#endif /* INCLUDE_6048_LIB */


/*****************************************************************************
****                             IXF18100                                 ****
*****************************************************************************/
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
defined(INCLUDE_18103_LIB) || defined(INCLUDE_18104_LIB)

/* Function Pointers for the Ixf18100 Chip */
CommonFunctionPointers_t ixf1810xCommonFuncPtrs = 
{
    Ixf18100InitChip,               /* InitChip */
        Ixf18100InitAllocMemory,        /* InitAllocMemory */
        Ixf18100DeAllocMemory,          /* DeAllocMemory */
        Ixf18100GetStatus,              /* GetStatus */
        Ixf18100GetOpMode,              /* GetOpMode */
        Ixf18100SetOpMode,              /* SetOpMode */
        Ixf18100Reset,                  /* Reset */
        Ixf18100InitAlarmCallback,      /* InitAlarmCallback */
        Ixf18100ChipIsr,                /* ChipIsr */
        Ixf18100CfgTest,                /* CfgTest */
        Ixf18100GetCounters,            /* GetCounters */
        Ixf18100GetChipInfo,            /* GetChipInfo */
        NULL,                           /* SetChipOnline */
        NULL,                           /* SetChipOffline */
        Ixf18100SetAlarmCfg,            /* SetAlarmCfg */
        Ixf18100GetBuildVersion,        /* GetBuildVersion */
        Ixf18100Read,                   /* Read */
        Ixf18100Write,                  /* Write */
        Ixf18100GetCfg,                 /* GetCfg */
        Ixf18100SetCfg                  /* SetCfg */
};

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
defined(INCLUDE_18103_LIB)
SonetFunctionPointers_t ixf1810xSonetFuncPtrs = 
{
    
    Ixf18100SonetGetWindowSize,     /* GetWindow */
        Ixf18100SonetSetWindowSize,     /* SetWindow */
        NULL,                           /* Set BER Threshold */
        Ixf18100SonetGetTrace,          /* GetTrace */
        Ixf18100SonetSetTrace,          /* SetTrace */
        Ixf18100SonetGetOhBytes,        /* GetOhBytes */
        Ixf18100SonetSetOhBytes,        /* SetOhBytes */
        NULL,                           /* SetRmstSourceBytes */
        NULL,                           /* GetRmstSourceBytes */
        NULL,                           /* MapFrame */
        NULL,                           /* GetMappedFrame */
        NULL                            /* GetAuPointer */
};
#endif

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
defined(INCLUDE_18104_LIB)
XgmacFunctionPointers_t ixf1810xXgmacFuncPtrs = 
{
    Ixf18100XgmacGetAddress,        /* GetMACAddress */
        Ixf18100XgmacSetAddress         /* SetMACAddress */    
};
#endif

#ifdef INCLUDE_18101_LIB
ChipFunctionPointers_t ixf18101FuncPtrs = 
{
    &ixf1810xCommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf1810xSonetFuncPtrs,         /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        &ixf1810xXgmacFuncPtrs,         /* ETHERNET routines*/
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif

#ifdef INCLUDE_18102_LIB
ChipFunctionPointers_t ixf18102FuncPtrs = 
{
    &ixf1810xCommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf1810xSonetFuncPtrs,         /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,                           /* ETHERNET routines*/
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif

#ifdef INCLUDE_18103_LIB
ChipFunctionPointers_t ixf18103FuncPtrs = 
{
    &ixf1810xCommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf1810xSonetFuncPtrs,          /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        &ixf1810xXgmacFuncPtrs,          /* ETHERNET routines*/
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif

#ifdef INCLUDE_18104_LIB
ChipFunctionPointers_t ixf18104FuncPtrs = 
{
    &ixf1810xCommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        NULL,                           /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        &ixf1810xXgmacFuncPtrs,          /* ETHERNET routines*/
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif

#endif /* INCLUDE_18101/2/3/4_LIB */

/*****************************************************************************
****                            IXF16121                                  ****
*****************************************************************************/
#ifdef INCLUDE_16121_LIB
/* Function Pointers for the Ixf16121 Chip */
CommonFunctionPointers_t ixf16121CommonFuncPtrs = 
{
    Ixf16121InitChip,               /* InitChip */
        Ixf16121InitAllocMemory,        /* InitAllocMemory */
        Ixf16121DeAllocMemory,          /* DeAllocMemory */
        NULL,                           /* GetStatus */
        NULL,                           /* GetOpMode */
        NULL,                           /* SetOpMode */
        Ixf16121Reset,                  /* Reset */
        Ixf16121InitAlarmCallback,      /* InitAlarmCallback */
        Ixf16121ChipIsr,                /* ChipIsr */
        Ixf16121CfgTest,                /* CfgTest */
        Ixf16121GetCounters,            /* GetCounters */
        Ixf16121GetChipInfo,            /* GetChipInfo */
        Ixf16121SetChipOnline,          /* SetChipOnline */
        Ixf16121SetChipOffline,         /* SetChipOffline */
        Ixf16121SetAlarmCfg,            /* SetAlarmCfg */
        Ixf16121GetBuildVersion,        /* GetBuildVersion */
        Ixf16121Read,                   /* Read */
        Ixf16121Write,                  /* Write */
        Ixf16121GetCfg,                 /* GetCfg */
        Ixf16121SetCfg                  /* SetCfg */
};


SonetFunctionPointers_t ixf16121SonetFuncPtrs = 
{
    
    Ixf16121GetWindowSize,          /* GetWindow */
        Ixf16121SetWindowSize,          /* SetWindow */
        Ixf16121SetBERThreshold,        /* Set BER Threshold */
        Ixf16121GetTrace,               /* GetTrace */
        Ixf16121SetTrace,               /* SetTrace */
        Ixf16121GetOhBytes,             /* GetOhBytes */
        Ixf16121SetOhByte,              /* SetOhBytes */
        Ixf16121SetRmstSourceBytes,     /* SetRmstSourceBytes */
        Ixf16121GetRmstSourceBytes,     /* GetRmstSourceBytes */
        Ixf16121MapFrames,              /* MapFrame */
        Ixf16121GetMappedFrames,        /* GetMappedFrame */
        Ixf16121GetFramesAuPointer      /* GetAuPointer */
};


ChipFunctionPointers_t ixf16121FuncPtrs = 
{
    &ixf16121CommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf16121SonetFuncPtrs,         /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_16121_LIB */


/*****************************************************************************
****                             IXF30007                                 ****
*****************************************************************************/
#ifdef INCLUDE_30007_LIB
/* Function Pointers for the Ixf30007 Chip */
CommonFunctionPointers_t ixf30007CommonFuncPtrs = 
{
    Ixf30007InitChip,               /* InitChip */
        Ixf30007InitAllocMemory,        /* InitAllocMemory */
        Ixf30007DeAllocMemory,          /* DeAllocMemory */
        Ixf30007GetStatus,              /* GetStatus */
        Ixf30007GetOpMode,              /* GetOpMode */
        Ixf30007SetOpMode,              /* SetOpMode */
        Ixf30007Reset,                  /* Reset */
        Ixf30007InitAlarmCallback,      /* InitAlarmCallback */
        Ixf30007ChipIsr,                /* ChipIsr */
        Ixf30007CfgTest,                /* CfgTest */
        Ixf30007GetCounters,            /* GetCounters */
        Ixf30007GetChipInfo,            /* GetChipInfo */
        NULL,                           /* SetChipOnline */
        NULL,                           /* SetChipOffline */
        Ixf30007SetAlarmCfg,            /* SetAlarmCfg */
        Ixf30007GetBuildVersion,        /* GetBuildVersion */
        Ixf30007Read,                   /* Read */
        Ixf30007Write,                  /* Write */
        Ixf30007GetCfg,                 /* GetCfg */
        Ixf30007SetCfg                  /* SetCfg */
};

SonetFunctionPointers_t ixf30007SonetFuncPtrs = 
{
    
    NULL,                           /* GetWindow */
        NULL,                           /* SetWindow */
        NULL,                           /* Set BER Threshold */
        Ixf30007GetTrace,               /* GetTrace */
        Ixf30007SetTrace,               /* SetTrace */
        Ixf30007GetOhByte,              /* GetOhBytes */
        NULL,                           /* SetOhBytes */
        NULL,                           /* SetRmstSourceBytes */
        NULL,                           /* GetRmstSourceBytes */
        NULL,                           /* MapFrame */
        NULL,                           /* GetMappedFrame */
        NULL                            /* GetAuPointer */
};

OTNFunctionPointers_t ixf30007OTNFuncPtrs =
{
    Ixf30007GetOTNTrace,
        Ixf30007SetOTNTrace,
        Ixf30007GetOTNOhByte,
        Ixf30007SetOTNOhByte
};

ChipFunctionPointers_t ixf30007FuncPtrs = 
{
    &ixf30007CommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf30007SonetFuncPtrs,         /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        &ixf30007OTNFuncPtrs,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_30007_LIB */


/*****************************************************************************
****                             IXF30009                                 ****
*****************************************************************************/
#ifdef INCLUDE_30009_LIB
/* COMMON Function Pointers for the Ixf30009 Chip */
CommonFunctionPointers_t ixf30009CommonFuncPtrs = 
{
    Ixf30009InitChip,               /* InitChip */
        Ixf30009InitAllocMemory,        /* InitAllocMemory */
        Ixf30009DeAllocMemory,          /* DeAllocMemory */
        Ixf30009GetStatus,              /* GetStatus */
        Ixf30009GetOpMode,              /* GetOpMode */
        Ixf30009SetOpMode,              /* SetOpMode */
        Ixf30009Reset,                  /* Reset */
        Ixf30009InitAlarmCallback,      /* InitAlarmCallback */
        Ixf30009ChipIsr,                /* ChipIsr */
        Ixf30009CfgTest,                /* CfgTest */
        Ixf30009GetCounters,            /* GetCounters */
        Ixf30009GetChipInfo,            /* GetChipInfo */
        NULL,                           /* SetChipOnline */
        NULL,                           /* SetChipOffline */
        Ixf30009SetAlarmCfg,            /* SetAlarmCfg */
        Ixf30009GetBuildVersion,        /* GetBuildVersion */
        Ixf30009Read,                   /* Read */
        Ixf30009Write,                  /* Write */
        Ixf30009GetCfg,                 /* GetCfg */
        Ixf30009SetCfg                  /* SetCfg */
};

#ifdef SONET_INCLUDED
/* SONET Function Pointers for the Ixf30009 Chip */
SonetFunctionPointers_t ixf30009SonetFuncPtrs = 
{
    
    Ixf30009SonetGetWindowSize,     /* GetWindow */
        Ixf30009SonetSetWindowSize,     /* SetWindow */
        Ixf30009SetBERThreshold,        /* Set BER Threshold */
        Ixf30009SonetGetTrace,          /* GetTrace */
        Ixf30009SonetSetTrace,          /* SetTrace */
        Ixf30009SonetGetOhBytes,        /* GetOhBytes */
        Ixf30009SonetSetOhBytes,        /* SetOhBytes */
        NULL,                           /* SetRmstSourceBytes */
        NULL,                           /* GetRmstSourceBytes */
        NULL,                           /* MapFrame */
        NULL,                           /* GetMappedFrame */
        NULL                            /* GetAuPointer */
};
#endif /* SONET_INCLUDED */ 

#ifdef ETHERNET_INCLUDED
/* Ethernet Function Pointers for the Ixf30009 Chip */
XgmacFunctionPointers_t ixf30009XgmacFuncPtrs = 
{
    NULL,                           /* Ixf30009XgmacGetAddress, GetMACAddress */
        NULL                            /* Ixf30009XgmacSetAddress  SetMACAddress */    
};
#endif /* ETHERNET_INCLUDED */ 

#ifdef OTN_INCLUDED
/* OTN Function Pointers for the Ixf30009 Chip */
OTNFunctionPointers_t ixf30009OTNFuncPtrs =
{
    Ixf30009GetOTNTrace,            /* GetOTNTrace */
        Ixf30009SetOTNTrace,            /* SetOTNTrace */
        Ixf30009GetOTNOhByte,           /* GetOTNOhBytes */
        Ixf30009SetOTNOhByte            /* SetOTNOhBytes */
};
#endif /* OTN_INCLUDED */ 


ChipFunctionPointers_t ixf30009FuncPtrs = 
{
    &ixf30009CommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf30009SonetFuncPtrs,         /* SONET routines */
#endif
#ifdef ETHERNET_INCLUDED
        &ixf30009XgmacFuncPtrs,          /* ETHERNET Routines */
#endif
#ifdef OTN_INCLUDED
        &ixf30009OTNFuncPtrs,            /* OTN Routines */
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_30009_LIB */


/*****************************************************************************
****                             IXF300x5                                 ****
*****************************************************************************/
#if defined(INCLUDE_30025_LIB) || defined(INCLUDE_30005_LIB)
/* Function Pointers for the Ixf300x5 Chip */
CommonFunctionPointers_t ixf300x5CommonFuncPtrs = 
{
    Ixf300x5InitChip,               /* InitChip */
        Ixf300x5InitAllocMemory,        /* InitAllocMemory */
        Ixf300x5DeAllocMemory,          /* DeAllocMemory */
        Ixf300x5GetStatus,              /* GetStatus */
        Ixf300x5GetOpMode,              /* GetOpMode */
        Ixf300x5SetOpMode,              /* SetOpMode */
        Ixf300x5Reset,                  /* Reset */
        Ixf300x5InitAlarmCallback,      /* InitAlarmCallback */
        Ixf300x5ChipIsr,                /* ChipIsr */
        Ixf300x5CfgTest,                /* CfgTest */
        Ixf300x5GetCounters,            /* GetCounters */
        Ixf300x5GetChipInfo,            /* GetChipInfo */
        NULL,                           /* SetChipOnline */
        NULL,                           /* SetChipOffline */
        Ixf300x5SetAlarmCfg,            /* SetAlarmCfg */
        Ixf300x5GetBuildVersion,        /* GetBuildVersion */
        Ixf300x5Read,                   /* Read */
        Ixf300x5Write,                  /* Write */
        Ixf300x5GetCfg,                 /* GetCfg */
        Ixf300x5SetCfg                  /* SetCfg */
};

SonetFunctionPointers_t ixf300x5SonetFuncPtrs = 
{
    
    NULL,                           /* GetWindow */
        NULL,                           /* SetWindow */
        NULL,                           /* Set BER Threshold */
        Ixf300x5GetTrace,               /* GetTrace */
        Ixf300x5SetTrace,               /* SetTrace */
        Ixf300x5GetOhByte,              /* GetOhBytes */
        NULL,                           /* SetOhBytes */
        NULL,                           /* SetRmstSourceBytes */
        NULL,                           /* GetRmstSourceBytes */
        NULL,                           /* MapFrame */
        NULL,                           /* GetMappedFrame */
        NULL                            /* GetAuPointer */
};

OTNFunctionPointers_t ixf300x5OTNFuncPtrs =
{
    Ixf300x5GetOTNTrace,
        Ixf300x5SetOTNTrace,
        Ixf300x5GetOTNOhByte,
        Ixf300x5SetOTNOhByte
};

ChipFunctionPointers_t ixf300x5FuncPtrs = 
{
    &ixf300x5CommonFuncPtrs,        /* COMMON routines */
#ifdef SONET_INCLUDED
        &ixf300x5SonetFuncPtrs,         /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        &ixf300x5OTNFuncPtrs,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_30025_LIB || INCLUDE_30005_LIB) */



#ifdef INCLUDE_GENERIC_LIB
/*Function Pointers for the generic driver*/
CommonFunctionPointers_t genericCommonFuncPtrs = 
{
    NULL,                           /* InitChip */
        NULL,                           /* InitAllocMemory */
        NULL,                           /* DeAllocMemory */
        NULL,                           /* GetStatus */
        NULL,                           /* GetOpMode */
        NULL,                           /* SetOpMode */
        NULL,                           /* Reset */
        NULL,                           /* InitAlarmCallback */
        NULL,                           /* ChipIsr */
        NULL,                           /* CfgTest */
        NULL,                           /* GetCounters */
        NULL,                           /* GetChipInfo */
        NULL,                           /* SetChipOnline */
        NULL,                           /* SetChipOffline */
        NULL,                           /* SetAlarmCfg */
        GenericDriverGetBuildVersion,   /* GetBuildVersion */
        GenericDriverRead,              /* Read */
        GenericDriverWrite,             /* Write */
        NULL,                           /* GetCfg */
        NULL                            /* SetCfg */
};


ChipFunctionPointers_t genericFuncPtrs = 
{
    &genericCommonFuncPtrs,         /* COMMON routines */
#ifdef SONET_INCLUDED
        NULL,                           /* SONET routines*/
#endif
#ifdef ETHERNET_INCLUDED
        NULL,
#endif
#ifdef OTN_INCLUDED
        NULL,
#endif
#ifdef I2C_INCLUDED
        NULL
#endif
};
#endif /* INCLUDE_GENERIC_LIB */

/*****************************************************************************
****                             IXF3503                                  ****
*****************************************************************************/
#ifdef INCLUDE_3503_LIB
/* Function Pointers for the Ixf3503 Chip */
CommonFunctionPointers_t ixf3503CommonFuncPtrs = 
{
    Ixf3503InitChip,               /* InitChip */
        Ixf3503InitAllocMemory,        /* InitAllocMemory */
        Ixf3503DeAllocMemory,          /* DeAllocMemory */
        NULL,                          /* GetStatus */
        NULL,                          /* GetOpMode */
        Ixf3503SetOpMode,              /* SetOpMode */
        Ixf3503Reset,                  /* Reset */
        NULL,                          /* InitAlarmCallback */
        NULL,                          /* ChipIsr */
        NULL,                          /* CfgTest */
        Ixf3503GetCounters,            /* GetCounters */
        Ixf3503GetChipInfo,            /* GetChipInfo */
        NULL,                          /* SetChipOnline */
        NULL,                          /* SetChipOffline */
        Ixf3503SetAlarmCfg,            /* SetAlarmCfg */
        Ixf3503GetBuildVersion,        /* GetBuildVersion */
        Ixf3503Read,                   /* Read */
        Ixf3503Write,                  /* Write */
        Ixf3503GetCfg,                 /* GetCfg */
        Ixf3503SetCfg                  /* SetCfg */
};

#ifdef SONET_INCLUDED
SonetFunctionPointers_t ixf3503SonetFuncPtrs = 
{
    
    NULL,         /* GetWindow */
        NULL,         /* SetWindow */
        NULL,         /* Set BER Threshold */
        Ixf3503GetTrace,         /* GetTrace */
        Ixf3503SetTrace,         /* SetTrace */
        Ixf3503GetOhBytes,              /* GetOhBytes */
        Ixf3503SetOhByte,               /* SetOhBytes */
        NULL,         /* SetRmstSourceBytes */
        NULL,         /* GetRmstSourceBytes */
        NULL,         /* MapFrame */
        NULL,         /* GetMappedFrame */
        NULL,         /* GetAuPointer */
};
#endif

#ifdef DS1_INCLUDED
DS1FunctionPointers_t ixf3503DS1FuncPtrs =
{
    IxfApi3503DS1_SetCfg,     
        IxfApi3503DS1_GetCfg,    
        IxfApi3503DS1_GetFramerCfg,
        IxfApi3503DS1_SetFramerCfg,
        IxfApi3503DS1_GetFramerMode,
        IxfApi3503DS1_SetFramerMode,
        IxfApi3503DS1_GetPerfMonCfg,
        IxfApi3503DS1_SetPerfMonCfg
};
#endif

#ifdef DS3_INCLUDED
DS3FunctionPointers_t ixf3503DS3FuncPtrs =
{
    IxfApi3503DS3_SetCfg,     
        IxfApi3503DS3_GetCfg,    
        IxfApi3503DS3_GetFramerCfg,
        IxfApi3503DS3_SetFramerCfg,
        IxfApi3503DS3_GetPerfMonCfg,
        IxfApi3503DS3_SetPerfMonCfg,
        IxfApi3503DS3_GetDiagnosticCfg,
        IxfApi3503DS3_SetDiagnosticCfg
};
#endif

#ifdef E1_INCLUDED
E1FunctionPointers_t ixf3503E1FuncPtrs =
{
    IxfApi3503E1_SetCfg,     
        IxfApi3503E1_GetCfg,    
        IxfApi3503E1_GetFramerCfg,
        IxfApi3503E1_SetFramerCfg,
        IxfApi3503E1_GetFramerMode,
        IxfApi3503E1_SetFramerMode
};
#endif

#ifdef E3_INCLUDED
E3FunctionPointers_t ixf3503E3FuncPtrs =
{
    IxfApi3503E3_SetCfg,     
        IxfApi3503E3_GetCfg,    
        IxfApi3503E3_GetFramerCfg,
        IxfApi3503E3_SetFramerCfg
};
#endif

ChipFunctionPointers_t ixf3503FuncPtrs = 
{
    &ixf3503CommonFuncPtrs,         /* COMMON routines */
        
#ifdef SONET_INCLUDED
        &ixf3503SonetFuncPtrs,          /* SONET routines*/
#endif
        
#ifdef DS1_INCLUDED
        &ixf3503DS1FuncPtrs,            /* DS1 routines */
#endif
        
#ifdef DS3_INCLUDED
        &ixf3503DS3FuncPtrs,            /* DS3 routines */
#endif
        
#ifdef E1_INCLUDED
        &ixf3503E1FuncPtrs,             /* E1 routines */
#endif
        
#ifdef E3_INCLUDED
        &ixf3503E3FuncPtrs              /* E3 routines */
#endif
};
#endif /* INCLUDE_3503_LIB */

/*****************************************************************************
****                             IXF3502                                  ****
*****************************************************************************/
#ifdef INCLUDE_3502_LIB
/* Function Pointers for the Ixf3502 Chip */
CommonFunctionPointers_t ixf3502CommonFuncPtrs = 
{
    Ixf3502InitChip,               /* InitChip */
        Ixf3502InitAllocMemory,        /* InitAllocMemory */
        Ixf3502DeAllocMemory,          /* DeAllocMemory */
        NULL,                          /* GetStatus */
        NULL,                          /* GetOpMode */
        NULL,                          /* SetOpMode */
        Ixf3502Reset,                  /* Reset */
        NULL,                          /* InitAlarmCallback */
        NULL,                          /* ChipIsr */
        NULL,                          /* CfgTest */
        Ixf3502GetCounters,            /* GetCounters */
        Ixf3502GetChipInfo,            /* GetChipInfo */
        NULL,                          /* SetChipOnline */
        NULL,                          /* SetChipOffline */
        Ixf3502SetAlarmCfg,            /* SetAlarmCfg */
        Ixf3502GetBuildVersion,        /* GetBuildVersion */
        Ixf3502Read,                   /* Read */
        Ixf3502Write,                  /* Write */
        Ixf3502GetCfg,                 /* GetCfg */
        Ixf3502SetCfg                  /* SetCfg */
};

ChipFunctionPointers_t ixf3502FuncPtrs = 
{
    &ixf3502CommonFuncPtrs,         /* COMMON routines */
        NULL,                           /* SONET routines*/
        NULL,                           /* XGMAC routines */
        NULL,                           /* DS1 routines */
        NULL,                           /* DS3 routines */
        NULL,                           /* E1 routines */
        NULL                            /* E3 routines */
#if 0
        NULL,                           /* SPI4 routines */
        NULL,                           /* POS routines */
        NULL,                           /* GFP routines */
        NULL                            /* PCS routines */
#endif
};
#endif /* INCLUDE_3502_LIB */


/*****************************************************************************
****                             IXF1110                                  ****
*****************************************************************************/

#ifdef INCLUDE_1110_LIB

/* Function Pointers for the Ixf1110 Chip */
CommonFunctionPointers_t ixf1110CommonFuncPtrs = 
{
    	Ixf1110InitChip,               	/* InitChip */
        Ixf1110InitAllocMemory,        /* InitAllocMemory */
        Ixf1110DeAllocMemory,          /* DeAllocMemory */
        Ixf1110GetStatus,              /* GetStatus */
        NULL,			               /* GetOpMode */
        NULL,           			   	/* SetOpMode */
        Ixf1110Reset,                  /* Reset */
        Ixf1110InitAlarmCallback,      /* InitAlarmCallback */
        Ixf1110ChipIsr,                /* ChipIsr */
        NULL,			               /* CfgTest */
        Ixf1110GetCounters,            /* GetCounters */
        Ixf1110GetChipInfo,            /* GetChipInfo */
        Ixf1110SetChipOnline,          /* SetChipOnline */
        Ixf1110SetChipOffline,         /* SetChipOffline */
        Ixf1110SetAlarmCfg,            /* SetAlarmCfg */
        Ixf1110GetBuildVersion,        /* GetBuildVersion */
        Ixf1110Read,                   /* Read */
        Ixf1110Write,                  /* Write */
        Ixf1110GetCfg,                 /* GetCfg */
        Ixf1110SetCfg                  /* SetCfg */
};

XgmacFunctionPointers_t ixf1110XgmacFuncPtrs = 
{
	Ixf1110XgmacGetAddress,        /* GetMACAddress */
     Ixf1110XgmacSetAddress         /* SetMACAddress */    
};

ChipFunctionPointers_t ixf1110FuncPtrs = 
{
    	&ixf1110CommonFuncPtrs,        /* COMMON routines */
        &ixf1110XgmacFuncPtrs,         /* ETHERNET routines*/
};

#endif /*INCLUDE_1110_LIB*/


/*****************************************************************************
***********        CHIP ARRAY FUNCTION POINTER DEFINITION            *********
*****************************************************************************/


/* Struct the holds the data type and the function pointer
this is the building block for a table of Chip types recognized
and function pointers.*/
typedef struct
{
    bb_ChipType_e chipType;
    ChipFunctionPointers_t *funcPtr;
} ChipFuncTableItem_t;


/* This is the table of the function pointers, is an associative array the */
/* will hold the chip type and the function pointer.  If none of the       */
/* INCLUDE_<chip>_LIB compile flags are used,                                  */

ChipFuncTableItem_t chipFuncTable[] = 
{
#ifdef INCLUDE_6048_LIB
    {bb_6048_CHIP, &ixf6048FuncPtrs},
#endif
        
#ifdef INCLUDE_6192_LIB
    {bb_6192_CHIP, &ixf6192FuncPtrs},
#endif
    
#ifdef INCLUDE_18101_LIB
    {bb_18101_CHIP, &ixf18101FuncPtrs},
#ifdef INCLUDE_1810X_8023WIS_LIB
    {bb_18101_8023WIS_CHIP, &ixf18101FuncPtrs}, 
#endif
#endif /* INCLUDE_18101_LIB */
    
#ifdef INCLUDE_18102_LIB
    {bb_18102_CHIP, &ixf18102FuncPtrs},
#endif
    
#ifdef INCLUDE_18103_LIB
    {bb_18103_CHIP, &ixf18103FuncPtrs},
#ifdef INCLUDE_1810X_8023WIS_LIB
    {bb_18103_8023WIS_CHIP, &ixf18103FuncPtrs}, 
#endif
#endif /* INCLUDE_18103_LIB */
    
#ifdef INCLUDE_18104_LIB
    {bb_18104_CHIP, &ixf18104FuncPtrs},
#endif
    
#ifdef INCLUDE_16121_LIB
    {bb_16121_CHIP, &ixf16121FuncPtrs},
#endif
    
#ifdef INCLUDE_30007_LIB
    {bb_IXF30007_CHIP, &ixf30007FuncPtrs},
#endif
    
#ifdef INCLUDE_30009_LIB
    {bb_IXF30009_CHIP, &ixf30009FuncPtrs},
#endif
    
#ifdef INCLUDE_30025_LIB
    {bb_IXF30025_CHIP, &ixf300x5FuncPtrs},
#endif
    
#ifdef INCLUDE_30005_LIB
    {bb_IXF30005_CHIP, &ixf300x5FuncPtrs},
#endif
    
#ifdef INCLUDE_3502_LIB
    {bb_IXF3502_CHIP, &ixf3502FuncPtrs},
#endif
    
#ifdef INCLUDE_3503_LIB
    {bb_IXF3503_CHIP, &ixf3503FuncPtrs},
#endif

#ifdef INCLUDE_1110_LIB
    {bb_1110_CHIP, &ixf1110FuncPtrs},
#endif
    
#ifdef INCLUDE_GENERIC_LIB    
    {bb_GENERIC_CHIP, &genericFuncPtrs},
#endif    
    /* this is the end of table marker and therefore must be the */
    /* last entry in the table.                                  */
    {bb_MAX_NUM_CHIPS, NULL}
};


/******************************************************************
* Function : getChipFuncPtr
*-----------------------------------------------------------------
* Description  : This routine returns the Function pointers 
*                table for an specified chip type, if the chip
*                type is not recognized, the function returns a
*                NULL pointer.
*-----------------------------------------------------------------
* Inputs  : chipType, The ID of the chip which we are looking for 
*           its function table.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : The function pointer table of the Chip specified. If 
*           the chip type is not recognized, a NULL pointer will 
*           be returned
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/

ChipFunctionPointers_t *getChipFuncPtr(bb_ChipType_e chipType)
{
    ChipFuncTableItem_t *pFuncTable = chipFuncTable;
    ChipFunctionPointers_t *pFuncPtr;
    int i = 0, generic = 0;
    
    while (pFuncTable[i].chipType != bb_MAX_NUM_CHIPS)
    {
        if (pFuncTable[i].chipType == chipType)
            break;
        else
        {
            /* Look for generic chip as we may need this later */
            if (pFuncTable[i].chipType == bb_GENERIC_CHIP)
                generic = i;
            i++;
        }
    }
    if (pFuncTable[i].chipType == bb_MAX_NUM_CHIPS)
    {
        /* If we get to the end of the table (bb_MAX_NUM_CHIPS), return     */
        /* generic driver function pointer if included in the build, else   */
        /* return NULL pointer                                              */
#ifdef INCLUDE_GENERIC_LIB    
        pFuncPtr = pFuncTable[generic].funcPtr;
#else
        pFuncPtr = NULL;
#endif         
    }
    else
        pFuncPtr = pFuncTable[i].funcPtr;
    
    return (pFuncPtr);
}

/******************************************************************
* Function : checkChipType
*-----------------------------------------------------------------
* Description  : This routine checks if the Chip specified by 
*                chipType is recognized by the Software.
*-----------------------------------------------------------------
* Inputs  : chipType, The ID of the chip that we want to check.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : The same chip type, if recognized. If the chip type is 
*           not recognized, bb_MAX_NUM_CHIPS is returned
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments :
******************************************************************/

bb_ChipType_e checkChipType(bb_ChipType_e chipType)
{
    int i = 0;
    
    while (chipFuncTable[i].chipType != bb_MAX_NUM_CHIPS)
    {
        if (chipFuncTable[i].chipType == chipType)
            break;
        else
            i++;
    }
    return (chipFuncTable[i].chipType);   
}

