/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110.h $
*   WORKFILE:       $Workfile: ixf1110.h $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: This file contains function prototypes for device related
*          routines
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 3     10/10/03 5:01p Tepulkow
 * added setchiponlione and setchipoffline prototypes
 * 
 * 2     10/07/03 2:03p Tepulkow
 * General cleanup for code review
 * 
* 
* --------------------------------------------------------------------------
*/
#ifndef __IXF1110_H__
#define __IXF1110_H__

/******************************************************************************
* Function : Ixf1110GetChipInfo
*------------------------------------------------------------------------------
* Description : Retrieves chip ID and version.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           pChipInfo : Pointer to Chip info structure where chip ID and
*                       version will be stored
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110GetChipInfo (bb_ChipData_t *pChipData, bb_ChipInfo_t *pChipInfo);

/******************************************************************************
* Function : Ixf1110GetCounters
*------------------------------------------------------------------------------
* Description : Retrieves one/all counters (from one/all) blocks of 
*                the chip Ixf1110.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to retrieve counters
*           eCounter  : Specifies which counter
*           pCounters : buffer to write the counters.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_BLOCK_OPERATION
*           bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110GetCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                    bb_SelCounters_e eCounter, void *pCounters);

                    /******************************************************************************
                    * Function : Ixf1110Reset
                    *------------------------------------------------------------------------------
                    * Description : Reset (chip or a section of the chip) and then 
                    *                reconfigure it.
                    *------------------------------------------------------------------------------
                    * Inputs  : pChipData : Initialized Chip Data
                    *           section   : Determines the section(block) to reset
                    *           resetType : Specifies type of Reset
                    * Outputs : None.
                    *------------------------------------------------------------------------------
                    * Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHIP_TYPE
                    *------------------------------------------------------------------------------
                    * Access Globals : None.
                    *------------------------------------------------------------------------------
                    * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110Reset(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
              bb_SelResetType_e resetType);

              /******************************************************************************
              * Function : Ixf1110CfgTest
              *------------------------------------------------------------------------------
              * Description : Run Tests
              *------------------------------------------------------------------------------
              * Inputs  : pChipData : Validated Chip Data
              *           section   : Determines the section(block) to run the tests on.
              *           testType  : The type of test to run.
              * Outputs : None.
              *------------------------------------------------------------------------------
              * Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
              *------------------------------------------------------------------------------
              * Access Globals : None.
              *------------------------------------------------------------------------------
              * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110CfgTest(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                bb_TestType_e testType, void *pTestCfg);

                /******************************************************************************
                * Function : Ixf1110SetCfg
                *------------------------------------------------------------------------------
                * Description : Configures one/all block(s) of IXF1810x device
                *------------------------------------------------------------------------------
                * Inputs  : *pChipData : pointer to the chip information.
                *           *pChipSeg   : pointer to chip segment structure
                *           SelCfg      : enum value defining what config needs to be set
                * Outputs : None.
                *------------------------------------------------------------------------------
                * Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
                *------------------------------------------------------------------------------
                * Access Globals : None.
                *------------------------------------------------------------------------------
                * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110SetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg);

               /******************************************************************************
               * Function : Ixf1110GetCfg
               *------------------------------------------------------------------------------
               * Description : Reads the configuration of one/all block(s) of IXF1810x device
               *------------------------------------------------------------------------------
               * Inputs  : *pChipData : pointer to the chip information.
               *           *pChipSeg   : pointer to chip segment structure
               *           SelCfg      : enum value defining what configuration
               *                         needs to be retrieved
               * Outputs : None.
               *------------------------------------------------------------------------------
               * Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
               *------------------------------------------------------------------------------
               * Access Globals : None.
               *------------------------------------------------------------------------------
               * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110GetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg);

               /******************************************************************************
               * Function : Ixf1110GetStatus
               *------------------------------------------------------------------------------
               * Description : Reads the status of one/all block(s) of IXF1810x device
               *------------------------------------------------------------------------------
               * Inputs  : pChipData : Initialized Chip Data
               *           section   : Determines the section(block) to get the
               *                       status from.
               *           selStatus : Type of Status: 
               * Outputs : pStatus : place to put Status.
               *------------------------------------------------------------------------------
               * Returns : bb_Error_e  Returns Error Condition if Not successful
               *------------------------------------------------------------------------------
               * Access Globals : None.
               *------------------------------------------------------------------------------
               * Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110GetStatus(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                  bb_SelStatus_e selStatus, void *pStatus);

                  /******************************************************************************
                  * Function : Ixf1110GetBuildVersion
                  *------------------------------------------------------------------------------
                  * Description : Reads the build version for the IXF1810x device driver
                  *------------------------------------------------------------------------------
                  * Inputs  : pChipData : Initialized Chip Data
                  *           section   : Determines the section(block) to get the
                  *                       status from.
                  *           selStatus : Type of Status: 
                  * Outputs : pStatus   : place to put Status.
                  *------------------------------------------------------------------------------
                  * Returns : bb_NO_ERROR
                  *------------------------------------------------------------------------------
                  * Access Globals : None.
                  *------------------------------------------------------------------------------
                  * Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110GetBuildVersion(bb_ChipData_t *pChipData, char *drvName, char *date,
                        ushort *buildVer, ushort *buildRev);

                        /******************************************************************************
                        * Function : Ixf1110Read
                        *------------------------------------------------------------------------------
                        * Description : Reads n number of one/two/four byte values.
                        *------------------------------------------------------------------------------
                        * Inputs  : pChipData : Initialized Chip Data
                        *           wordSize  : One/two/four byte word size
                        *           address   : Starting address to read from
                        *           length    : Number of words to read
                        *           buffer    : Location where words are read into
                        * Outputs : None.
                        *------------------------------------------------------------------------------
                        * Returns : bb_Error_e  Returns Error Condition if Not successful
                        *------------------------------------------------------------------------------
                        * Access Globals : None.
                        *------------------------------------------------------------------------------
                        * Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110Read(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
             ushort length, void *buffer);

             /******************************************************************************
             * Function : Ixf1110Write
             *------------------------------------------------------------------------------
             * Description : Writes n number of one/two/four byte values to the device
             *------------------------------------------------------------------------------
             * Inputs  : pChipData : Initialized Chip Data
             *           wordSize  : One/two/four byte word size
             *           address   : Starting address to write to
             *           length    : Number of words to write
             *           buffer    : Location where words are written from
             * Outputs : None.
             *------------------------------------------------------------------------------
             * Returns : bb_Error_e  Returns Error Condition if Not successful
             *------------------------------------------------------------------------------
             * Access Globals : None.
             *------------------------------------------------------------------------------
             * Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110Write(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
              ushort length, void *buffer);

/******************************************************************************
* Function : Ixf1110GetMacAddress
*------------------------------------------------------------------------------
* Description : Gets the Mac Address of the device.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
* Outputs : address   : Pointer to the Mac Address structure
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110GetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address);

                      /******************************************************************************
                      * Function : Ixf1110SetMacAddress
                      *------------------------------------------------------------------------------
                      * Description : Sets the Mac Address of the device.
                      *------------------------------------------------------------------------------
                      * Inputs  : pChipData : Initialized Chip Data
                      *           section   : Determines the section(block) of the chip
                      *           address   : Pointer to the Mac Address structure
                      * Outputs : None.
                      *------------------------------------------------------------------------------
                      * Returns : bb_Error_e  Returns Error Condition if Not successful
                      *------------------------------------------------------------------------------
                      * Access Globals : None.
                      *------------------------------------------------------------------------------
                      * Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110SetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address);

/******************************************************************************
* Function : Ixf1110SetAlarmCfg
*------------------------------------------------------------------------------
* Description : Configures the interrupt masks
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
*           AlarmType : Type of Alarm 
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110SetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                    bb_AlarmType_e AlarmType, void *pAlarmCfg);

                    /******************************************************************************
                    * Function : Ixf1110InitAlarmCallback
                    *------------------------------------------------------------------------------
                    * Description : Initialize the alarm callback function with the IXF30007
                    *               device driver
                    *------------------------------------------------------------------------------
                    * Inputs  : pChipData      : Validated Chip Data
                    *           pAlarmCallback : Alarm Callback
                    * Outputs : None.
                    *------------------------------------------------------------------------------
                    * Returns : bb_NO_ERROR
                    *------------------------------------------------------------------------------
                    * Access Globals : None.
                    *------------------------------------------------------------------------------
                    * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110InitAlarmCallback(bb_ChipData_t *pChipData,
                          AlarmCallBack pAlarmCallbackArg);

                          /******************************************************************************
                          * Function : Ixf1110ChipIsr
                          *------------------------------------------------------------------------------
                          * Description : The ISR services the interrupts that the user has selected to
                          *               monitor
                          *------------------------------------------------------------------------------
                          * Inputs  : pChipData : Validated Chip Data
                          * Outputs : None.
                          *------------------------------------------------------------------------------
                          * Returns : 
                          *------------------------------------------------------------------------------
                          * Access Globals : None.
                          *------------------------------------------------------------------------------
                          * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110ChipIsr(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf1110InitAllocMemory
*------------------------------------------------------------------------------
* Description : Allocate the memory space for IXF1810x device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110InitAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf1110DeAllocMemory
*------------------------------------------------------------------------------
* Description : Deallocate the memory space for IXF1810x device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NO_CHIP_CFG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110DeAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf1110InitChip
*------------------------------------------------------------------------------
* Description : Initialize the ixf1810x chip via OCM UI generated
*               data structure
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           pTable : Table to be committed to the device.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);

/******************************************************************************
* Function : Ixf1110GetOpMode
*------------------------------------------------------------------------------
* Description : This function returns the operational mode that the chip is
*               configured in.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section: Chip Segment
*           pModeCfg: NULL
* Outputs : opMode: Operational Mode
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_STM_MODE_MISMATCH, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : We have only one enum value for GFP over FIBER Mode 
* and GFP  Client Map Mode. The reason being that the configuration
* value for this register in these two modes is identical.
******************************************************************************/
extern bb_Error_e
Ixf1110GetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_OperMode_e *opMode, void *pModeCfg);

                  /******************************************************************************
                  * Function : Ixf1110SetOpMode
                  *------------------------------------------------------------------------------
                  * Description : Configures the Chip Configuration structure to 
                  *               operate in a certain mode.
                  *------------------------------------------------------------------------------
                  * Inputs  : pChipData : Validated Chip Data
                  *           section: Chip Segment
                  *           opMode: Operational Mode
                  *           pModeCfg: NULL
                  * Outputs : None.
                  *------------------------------------------------------------------------------
                  * Returns : bb_NO_ERROR, bb_INV_PARAMETER
                  *------------------------------------------------------------------------------
                  * Access Globals : None.
                  *------------------------------------------------------------------------------
                  * Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110SetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_OperMode_e opMode, void *pModeCfg);

/******************************************************************************
* Function : Ixf1110IxfApiSetChipOffline
*------------------------------------------------------------------------------
* Description : 
*               
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section : Chip segment
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110SetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section);


/******************************************************************************
* Function : Ixf1110IxfApiSetChipOnline
*------------------------------------------------------------------------------
* Description : 
*               
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section : Chip segment
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110SetChipOnline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section);

#endif /* __IXF1110_H__ */
