/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_global.h $
*   WORKFILE:       $Workfile: ixf1110_global.h $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: This file contains function prototypes for global block related
*          routines
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_global.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 6     10/13/03 4:57p Tepulkow
 * added prototypes for setchiponline and offline
 * 
 * 5     10/07/03 2:03p Tepulkow
 * General cleanup for code review
 * 
 * 4     9/26/03 2:56p Rnballar
 * added reset prototypes
 * 
 * 3     9/18/03 1:31p Rnballar
 * 
 * 2     9/17/03 3:41p Rnballar
 * updated headers to match _d.h files
 * 
 * 1     8/04/03 3:49p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* --------------------------------------------------------------------------
*/

#ifndef IXF1110_GLOBAL_H
#define IXF1110_GLOBAL_H

/******************************************************************************
* Function : Ixf1110GlobalSetCfg
*------------------------------------------------------------------------------
* Description : Global configuration functions, the mode combinations are 
*               mutually exclusive.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : Pointer to values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf1110GlobalSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
			 ixf1110_GlbStatusCfg_t *pCnfg);

/******************************************************************************
* Function : Ixf1110GlobalGetCfg
*------------------------------------------------------------------------------
* Description : This function returns the mode configuration.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : The pointer of the structure to return values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110GlobalGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
			 		ixf1110_GlbStatusCfg_t *pCnfg);
/******************************************************************************
* Function : Ixf1110MacSftReset
*------------------------------------------------------------------------------
* Description : This function resets a mac port control registers
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ushort channel -- port to resset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHANNEL, bb_GENERAL_ERROR 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110MacSftReset(bb_RegPointer_type BaseAddress, ushort channel);
/******************************************************************************
* Function : Ixf1110MacSftResetAll
*------------------------------------------------------------------------------
* Description : This function resets all mac channels(ports)
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110MacSftResetAll(bb_RegPointer_type BaseAddress);
/******************************************************************************
* Function : Ixf1110CoreClkSftReset
*------------------------------------------------------------------------------
* Description : This function resets mac core clock system
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110CoreClkSftReset(bb_RegPointer_type BaseAddress); 

extern bb_Error_e
Ixf1110GlobalGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e selStatus, void* pStatus);

/******************************************************************************
* Function : Ixf1110GlobalSetChipOffline
*------------------------------------------------------------------------------
* Description : This function stes the specified channel off line.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to set off line.
* Outputs : 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GlobalSetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection);

/******************************************************************************
* Function : Ixf1110GlobalSetChipOnline
*------------------------------------------------------------------------------
* Description : This function stes the specified channel on line.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to set on line.
* Outputs : 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GlobalSetChipOnline(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection);

#endif
