/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_rx.h $
*   WORKFILE:       $Workfile: ixf1110_rx.h $
*
* --------------------------------------------------------------------------
* Project: ixf1110 device API
*
* Purpose: This file contains rx block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_rx.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 5     9/26/03 2:56p Rnballar
 * added reset prototypes
 * 
 * 4     9/19/03 9:32a Rnballar
 * 
 * 3     9/18/03 2:42p Rnballar
 * 
 * 2     9/18/03 2:19p Rnballar
 * 
 * 1     9/18/03 12:28p Tepulkow
 * Created for ixf1110 project
 * 
 * 3     9/18/03 12:03p Tepulkow
 * added get counter function
 * 
 * 2     9/16/03 5:00p Tepulkow
 * Created function framework.
 * 
* --------------------------------------------------------------------------
*/
#ifndef _IXF1110_RX_H__
#define _IXF1110_RX_H__
/******************************************************************************
* Function : Ixf1110RxSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the Rx block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptRxCfg : Pointer to the values to set int the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_RxCfg_t *ptRxCfg);

/******************************************************************************
* Function : Ixf1110RxGetCfg
*------------------------------------------------------------------------------
* Description : This function gets Rx block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptRxCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
			 ixf1110_RxCfg_t *ptRxCfg);

/******************************************************************************
* Function : Ixf1110RxGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 32 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110RxGetStatus(bb_RegPointer_type BaseAddress,
                        bb_SelStatus_e selStatus, void* pStatus);

/******************************************************************************
* Function : Ixf1110RxGetCounters
*------------------------------------------------------------------------------
* Description  : The function reads the counter register  values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/

extern bb_Error_e
Ixf1110RxGetCounters(bb_RegPointer_type BaseAddress, bb_SelCounters_e SelCntr,
						 void *ptCounters);

/******************************************************************************
* Function : Ixf1110RxFifoReset
*------------------------------------------------------------------------------
* Description : This function resets an individual mac fifo
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ushort channel -- port to resset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHANNEL, bb_GENERAL_ERROR 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110RxFifoReset(bb_RegPointer_type BaseAddress, ushort channel);
/******************************************************************************
* Function : Ixf1110RxFifoResetAll
*------------------------------------------------------------------------------
* Description : This function resets all rx fifos
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf1110RxFifoResetAll(bb_RegPointer_type BaseAddress);

#endif /*_IXF1110_RX_H__*/
