/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_xgmac.h $
*   WORKFILE:       $Workfile: ixf1110_xgmac.h $
*
* --------------------------------------------------------------------------
* Project: ixf18100 family device API
*
* Purpose: 
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/include/ixf1110_xgmac.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 4     10/01/03 1:46p Rnballar
 * changed last param of get status proto
 * 
 * 3     9/18/03 12:40p Rnballar
 * 
 * 2     9/17/03 11:07a Tepulkow
 * updated prototypes for getcfg, setcfg, getstatus, getcounters.
 * 
 * 1     8/04/03 3:50p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 
* --------------------------------------------------------------------------
*/
#ifndef __IXF1110_XGMAC_H__
#define __IXF1110_XGMAC_H__

/******************************************************************************
* Function : Ixf1110XgmacSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the XGMAC block in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf1110_XgmacCfg_t *ptXgmacCfg: A pointer a structure containing
*                                          chip configuration settings.
* Outputs : Writes to registers to configure XGMAC settings on the chip.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ushort channel, ixf1110_XgmacCfg_t *ptXgmacCfg);

/******************************************************************************
* Function : Ixf1110XgmacGetCfg
*------------------------------------------------------------------------------
* Description : This function reads the XGMAC configuration settings into a 
*                data structure.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*			unshort channel: Selects channel to write status to.
*           ixf1110_XgmacCfg_t *ptXgmacCfg: A pointer to a structure used to
*                                          contain configuration information
*                                          obtained from the XGMAC registers.
* Outputs : A filled data structure containing XGMAC configuration information.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ushort channel, ixf1110_XgmacCfg_t *ptXgmacCfg);

/******************************************************************************
* Function : Ixf1110XgmacGetStatus
*------------------------------------------------------------------------------
* Description : This function reads the status registers of the XGMAC register
*               block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelStatus_e SelStat: Selects the status to be retrieved.
*			unshort channel: Selects channel to read status from.
*           void* pStatus: 
*                                                   structure
* Outputs : A filled data structure containing XGMAC status information. 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                       ushort channel, void* pStatus);

/******************************************************************************
* Function : Ixf1110XgmacGetCounters
*------------------------------------------------------------------------------
* Description : This function reads counter values in the XGMAC register
*                block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*			unshort channel: Selects channel to read counters from.
*           bb_SelCounters_e SelCntr: Selects the counters to be read.
*           void *ptCounters: A pointer to a structure used to
*                                          store counter values.
* Outputs : A filled data structure containing counter values.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetCounters(bb_RegPointer_type BaseAddress, ushort channel, 
                         bb_SelCounters_e SelCntr, void *ptCounters);

/******************************************************************************
* Function : Ixf1110XgmacSetAddress
*------------------------------------------------------------------------------
* Description :  Function that sets th MAC address for a ixf1110 device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: The channel number is derived from section.
*           pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the address.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacSetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress);

/******************************************************************************
* Function : Ixf1110XgmacGetAddress
*------------------------------------------------------------------------------
* Description : This function reads the Mac Address from the device.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: The channel number is derived from section.
* Outputs : pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the MAC address.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress);

#endif /*__IXF1110_XGMAC_H__*/
