/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110.c $
*   WORKFILE:       $Workfile: ixf1110.c $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: This file contains the device specific API's called by the IXF API
*          layer.
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 19    10/21/03 10:35a Tepulkow
 * consilidated the edit alarm functions for rx, tx, and mod_def into a
 * single edit alarm function
 * 
 * 18    10/13/03 4:58p Tepulkow
 * added calls to setchiponline and offline
 * 
 * 16    10/10/03 4:30p Tepulkow
 * coded out the getcounters support for RX and TX blocks
 * 
 * 15    9/30/03 5:38p Tepulkow
 * Removed "&" from in front of pointer passed from chipisr
 * 
 * 14    9/26/03 2:54p Rnballar
 * added reset functionailty
 * 
 * 13    9/24/03 3:49p Rnballar
 * adding callback to chipisr
 * 
 * 12    9/23/03 4:02p Rnballar
 * fixed linking
 * 
 * 11    9/23/03 10:42a Rnballar
 * 
 * 10    9/23/03 10:20a Rnballar
 * added ixf1110 alarms
 * 
 * 9     9/22/03 4:05p Rnballar
 * 
 * 8     9/19/03 4:53p Tepulkow
 * 
 * 7     9/18/03 3:42p Tepulkow
 * Modified 1110_read and 1110write functions for long word accesses
 * 
 * 6     9/18/03 12:04p Rnballar
 * 
 * 5     9/17/03 11:05a Tepulkow
 * modified xgmac_setcfg, xgmac_getcfg, xgmac_getstatus to indicate
 * channel selection.
 * 
 * 4     9/16/03 4:55p Tepulkow
 * Ported Ixf1110GetChipInfo, Ixf1110GetCounters, Ixf1110Reset,
 * Ixf11100SetCfg, Ixf11100GetCfg, Ixf1110GetStatus to the ixf1110.
 * Deleted unsupported functions gettestcfg, settestcfg.
 * 
 * 3     9/15/03 11:18a Tepulkow
 * Added kernel ifdefs
 * 
 * 2     9/09/03 4:46p Tepulkow
 * Created and modified many header files for IXF GI creation
 * 
 * 1     8/04/03 3:51p Tepulkow
 * 
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#include "msgQLib.h"
#endif

#ifdef IXF_LINUX
#ifndef __KERNEL__
#include <semaphore.h>
#endif
#endif

#ifndef __KERNEL__
#include <string.h>
#include <stdlib.h>
#else
#include <linux/string.h>
#endif
#include "common_types.h"
#include "common_def.h"

#ifdef __KERNEL__
#include "linuxkern.h"
#else
#include "os.h"
#endif

#include "ixf_api_d.h"
#include "ixf1110_d.h"

#include "ixf1110_global_d.h"  /* needed so ixf_api.h can be unchanged...*/
#include "ixf_api_spi4_d.h"     /* needed so ixf_api.h can be unchanged...*/

#include "ixf_api_ether_d.h"
#include "ixf1110_xgmac_d.h"
#include "ixf1110_xgmac.h"

#include "ixf1110_global.h"
#include "ixf1110.h"
#include "ixf1110_spi4_d.h"
#include "ixf1110_spi4.h"
#include "ixf_api.h"
#include "bld1110.h"
#include "ixf1110_d.h"
#include "ixf1110_cnfg_d.h"
#include "ixf1110_alarms_d.h"
#include "ixf1110_alarms.h"
#include "ixf1110_global_b.h"
#include "ixf1110_rx.h"
#include "ixf1110_tx.h"
#include "ixf1110_serdes.h"
#include "ixf1110_gbic.h"
#include "hal.h"
#include "ixf_perf_metric.h"

/******************************************************************************
* Function : Ixf1110GetChipInfo
*------------------------------------------------------------------------------
* Description : Retrieves chip ID and version.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           pChipInfo : Pointer to Chip info structure where chip ID and
*                       version will be stored
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GetChipInfo (bb_ChipData_t *pChipData, bb_ChipInfo_t *pChipInfo)
{
    if (pChipInfo != NULL && pChipData != NULL)
    {
        /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
        {
            return bb_INV_CHIP_TYPE;
        }
        
        pChipInfo->ChipId = (ulong)(HAL_IXF1110_READ(pChipData->BaseAddress
            + ixf1110_GLB_CHIPID_OFFSET) & ixf1110_mCHIP_ID);
        
        pChipInfo->ChipVersion
            = (ulong)((HAL_IXF1110_READ(pChipData->BaseAddress
            + ixf1110_GLB_CHIPID_OFFSET) & ixf1110_mCHIP_VER) 
            >> ixf1110_sCHIP_VER);
        
        return bb_NO_ERROR;
    }
    
    return bb_NULL_ARG;
}

/******************************************************************************
* Function : Ixf1110GetCounters
*------------------------------------------------------------------------------
* Description : Retrieves one/all counters (from one/all) blocks of 
*                the chip Ixf1110.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to retrieve counters
*           eCounter  : Specifies which counter
*           pCounters : buffer to write the counters.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_BLOCK_OPERATION
*           bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GetCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                    bb_SelCounters_e eCounter, void *pCounters)
{
    bb_Error_e rc = bb_NO_ERROR;
    
    if (pCounters == NULL)
        return bb_NULL_ARG;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    switch (ptSegment->block.block)
    {
    case ixf_eXGMAC:
        rc = Ixf1110XgmacGetCounters(pChipData->BaseAddress, 
        	ptSegment->group.channel, eCounter, pCounters);
        break;
        
    case ixf_eGLBRX:
        rc = Ixf1110RxGetCounters(pChipData->BaseAddress, 
        	eCounter, pCounters);
        break;

    case ixf_eGLBTX:
        rc = Ixf1110TxGetCounters(pChipData->BaseAddress, 
        	eCounter, pCounters);
        break;

	default:
        rc = bb_INV_PARAMETER;
    }
    
    return rc;
}

/******************************************************************************
* Function : Ixf1110Reset
*------------------------------------------------------------------------------
* Description : Reset (chip or a section of the chip) and then 
*                reconfigure it.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to reset
*           resetType : Specifies type of Reset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110Reset(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
              bb_SelResetType_e resetType)
{
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

	switch(resetType)  {
	case bb_RESET_RX_FIFO:
		rc = Ixf1110RxFifoReset(pChipData->BaseAddress, ptSegment->group.channel);
		break;
	case bb_RESET_TX_FIFO:
		rc = Ixf1110RxFifoResetAll(pChipData->BaseAddress);
		break;
	case bb_RESET_XGMAC:
		rc = Ixf1110MacSftReset(pChipData->BaseAddress, ptSegment->group.channel);
		break;
	case bb_RESET_CORE_CLK:
		rc = Ixf1110CoreClkSftReset(pChipData->BaseAddress);
		break;
	case bb_RESET_XGMAC_ALL:
		rc = Ixf1110MacSftResetAll(pChipData->BaseAddress);
		break;
	case bb_RESET_RX_FIFO_ALL:
		rc = Ixf1110RxFifoResetAll(pChipData->BaseAddress);
		break;
	case bb_RESET_TX_FIFO_ALL:
		rc = Ixf1110TxFifoResetAll(pChipData->BaseAddress);
		break;
	default:
		rc = bb_INV_RESET_TYPE;
	}
      
    return rc;
}

/******************************************************************************
* Function : Ixf1110SetCfg
*------------------------------------------------------------------------------
* Description : Configures one/all block(s) of IXF1110 device
*------------------------------------------------------------------------------
* Inputs  : *pChipData : pointer to the chip information.
*           *pChipSeg   : pointer to chip segment structure
*           SelCfg      : enum value defining what config needs to be set
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110SetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg)
{
    bb_Error_e rc = bb_NO_ERROR;
    ixf1110_ChipCfg_t *p1110ChipCfg
        = (ixf1110_ChipCfg_t *)pChipData->pChipCfg;
    
    if (pChipData->pChipCfg == NULL)
        return bb_NULL_ARG;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    switch (ptSegment->block.block)
    {
    case ixf_eGLOBAL:
        rc = Ixf1110GlobalSetCfg(pChipData->BaseAddress, SelCfg,
            &(p1110ChipCfg->GlbStatusCfg));
        break;
        
    case ixf_eSPI4:
        rc = Ixf1110Spi4SetCfg(pChipData->BaseAddress, SelCfg,
            &(p1110ChipCfg->Spi4Cfg));
        break;
        
    case ixf_eXGMAC:
        rc = Ixf1110XgmacSetCfg(pChipData->BaseAddress, SelCfg, 
        	ptSegment->group.channel, &(p1110ChipCfg->XgmacCfg));
        break;

    case ixf_eGLBRX:
        rc = Ixf1110RxSetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->RxCfg));
        break;

    case ixf_eGLBTX:
        rc = Ixf1110TxSetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->TxCfg));
        break;

    case ixf_eSERDES:
        rc = Ixf1110SerdesSetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->SerDesCfg));
        break;

    case ixf_eGBIC:
        rc = Ixf1110GbicSetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->GbicCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }
    return rc;
}

/******************************************************************************
* Function : Ixf1110GetCfg
*------------------------------------------------------------------------------
* Description : Reads the configuration of one/all block(s) of IXF1110 device
*------------------------------------------------------------------------------
* Inputs  : *pChipData : pointer to the chip information.
*           *pChipSeg   : pointer to chip segment structure
*           SelCfg      : enum value defining what configuration
*                         needs to be retrieved
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg)
{
    bb_Error_e rc = bb_NO_ERROR;
    ixf1110_ChipCfg_t *p1110ChipCfg
        = (ixf1110_ChipCfg_t *)pChipData->pChipCfg;
    
    if (NULL == pChipData->pChipCfg)
        return bb_NULL_ARG;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    switch (ptSegment->block.block)
    {
    case ixf_eGLOBAL:
        rc = Ixf1110GlobalGetCfg(pChipData->BaseAddress, SelCfg,
            &(p1110ChipCfg->GlbStatusCfg));
        break;
        
    case ixf_eSPI4:
        rc = Ixf1110Spi4GetCfg(pChipData->BaseAddress, SelCfg,
            &(p1110ChipCfg->Spi4Cfg));
        break;
        
    case ixf_eXGMAC:
        rc = Ixf1110XgmacGetCfg(pChipData->BaseAddress, SelCfg,
        	ptSegment->group.channel, &(p1110ChipCfg->XgmacCfg));
        break;

    case ixf_eGLBRX:
        rc = Ixf1110RxGetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->RxCfg));
        break;

    case ixf_eGLBTX:
        rc = Ixf1110TxGetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->TxCfg));
        break;

    case ixf_eSERDES:
        rc = Ixf1110SerdesGetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->SerDesCfg));
        break;

    case ixf_eGBIC:
        rc = Ixf1110GbicGetCfg(pChipData->BaseAddress, SelCfg, 
        	&(p1110ChipCfg->GbicCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }
    
    return rc;
}

/******************************************************************************
* Function : Ixf1110GetStatus
*------------------------------------------------------------------------------
* Description : Reads the status of one/all block(s) of IXF1110 device
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the status from.
*           selStatus : Type of Status: 
* Outputs : pStatus : place to put Status.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110GetStatus(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                  bb_SelStatus_e selStatus, void *pStatus)
{
    bb_Error_e rc = bb_NO_ERROR;
    
    if (NULL == pStatus)
        return bb_NULL_ARG;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    switch (ptSegment->block.block)
    {
    case ixf_eGLOBAL:
        rc = Ixf1110GlobalGetStatus(pChipData->BaseAddress, selStatus,
            pStatus);

        break;
        
    case ixf_eSPI4:
        rc = Ixf1110Spi4GetStatus(pChipData->BaseAddress, selStatus,
            pStatus);
        break;
        
    case ixf_eXGMAC:
        rc = Ixf1110XgmacGetStatus(pChipData->BaseAddress, selStatus, 
        	ptSegment->group.channel, pStatus);
        break;
        
    case ixf_eGLBRX:
        rc = Ixf1110RxGetStatus(pChipData->BaseAddress, selStatus, 
        	pStatus);
        break;

    case ixf_eGLBTX:
        rc = Ixf1110TxGetStatus(pChipData->BaseAddress, selStatus, 
        	pStatus);
        break;

    case ixf_eSERDES:
        rc = Ixf1110SerdesGetStatus(pChipData->BaseAddress, selStatus, 
        	pStatus);
        break;

    case ixf_eGBIC:
        rc = Ixf1110GbicGetStatus(pChipData->BaseAddress, selStatus, 
        	pStatus);
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }
    return rc;
}

/******************************************************************************
* Function : Ixf1110GetBuildVersion
*------------------------------------------------------------------------------
* Description : Reads the build version for the IXF1110 device driver
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           selStatus : Type of Status: 
* Outputs : pStatus   : place to put Status.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110GetBuildVersion(bb_ChipData_t *pChipData, char *drvName, char *date,
                        ushort *buildVer, ushort *buildRev)
{
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    strcpy(drvName, IXF1110_BUILD_DRIVER_NAME);
    strcpy(date, IXF1110_BUILD_DATE);
    *buildVer = IXF1110_BUILD_VERSION;
    *buildRev = IXF1110_BUILD_REV;
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110Read
*------------------------------------------------------------------------------
* Description : Reads n number of one/two/four byte values.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           wordSize  : One/two/four byte word size
*           address   : Starting address to read from
*           length    : Number of words to read
*           buffer    : Location where words are read into
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110Read(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
             ushort length, void *buffer)
{
    bb_Error_e rc = bb_NO_ERROR;
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress + address;
    int i;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    switch(wordSize)
    {
    case ONE_BYTE:
        for (i = 0; i < length; i++)
            *((uchar*)buffer + i)
            = (uchar)HAL_IXF1110_READ(BaseAddress + i);
        break;
        
    case TWO_BYTES:
        for (i = 0; i < length; i++)
            *((ushort*)buffer + i)
            = (ushort)HAL_IXF1110_READ(BaseAddress + i);
        break;
        
    case FOUR_BYTES:
        for (i = 0; i < length; i++)
            *((ulong*)buffer + i)
            = (ulong)HAL_IXF1110_READ(BaseAddress + i);
        break;
        
    case EIGHT_BYTES:
    default:
        rc = bb_WORD_SIZE_UNSUPPORTED;
    }
    
    return rc;
}
/******************************************************************************
* Function : Ixf1110Write
*------------------------------------------------------------------------------
* Description : Writes n number of one/two/four byte values to the device
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           wordSize  : One/two/four byte word size
*           address   : Starting address to write to
*           length    : Number of words to write
*           buffer    : Location where words are written from
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110Write(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
              ushort length, void *buffer)
{
    bb_Error_e rc = bb_NO_ERROR;
    bb_RegPointer_type BaseAddress = pChipData->BaseAddress + address;
    int i;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
   
    switch(wordSize)
    {
    case ONE_BYTE:
        for (i = 0; i < length; i++)
            HAL_IXF1110_WRITE(BaseAddress + i, *((uchar*)buffer + i));
        break;
        
    case TWO_BYTES:
        for (i = 0; i < length; i++)
            HAL_IXF1110_WRITE(BaseAddress + i, *((ushort*)buffer + i));
        break;
        
    case FOUR_BYTES:
        for (i = 0; i < length; i++)
            HAL_IXF1110_WRITE(BaseAddress + i, *((ulong*)buffer + i));
        break;
        
    case EIGHT_BYTES:
    default:
        rc = bb_WORD_SIZE_UNSUPPORTED ;
    }
    
    return rc;
}

/******************************************************************************
* Function : Ixf1110GetMacAddress
*------------------------------------------------------------------------------
* Description : Gets the Mac Address of the device.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
* Outputs : address   : Pointer to the Mac Address structure
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110GetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address)
{
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    return Ixf1110XgmacGetAddress(pChipData, section, 
        (IxfApi_MacAddress_t *)address);
}

/******************************************************************************
* Function : Ixf1110SetMacAddress
*------------------------------------------------------------------------------
* Description : Sets the Mac Address of the device.
*----------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
*           address   : Pointer to the Mac Address structure
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110SetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address)
{
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    return Ixf1110XgmacSetAddress(pChipData, section, 
        (IxfApi_MacAddress_t *)address);
}

/******************************************************************************
* Function : Ixf1110SetAlarmCfg
*------------------------------------------------------------------------------
* Description : Configures the interrupt masks
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
*           AlarmType : Type of Alarm 
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
bb_Error_e
Ixf1110SetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                    bb_AlarmType_e AlarmType, void *pAlarmCfg)
{
    bb_Error_e rc = bb_NO_ERROR;
    ixf1110_AlarmCfg_t *ptAlarmChipCfg;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    ptAlarmChipCfg = (ixf1110_AlarmCfg_t*)pChipData->pAlarmCfg;
    
    /* Error if Alarm structure not initialized */
    if (pAlarmCfg == NULL)
        return bb_NO_ALARM_CFG; /* We Have a Problem */

  
    /* Configure alarms based upon type */
    switch(AlarmType)
    {
    case bb_1110_RX_LOS_ALARM:
    case bb_1110_TX_FAULT_ALARM:
    case bb_1110_MOD_DEF_ALARM:
        Ixf1110Edit_Alarms(pChipData->BaseAddress,
            (ulong *)&(ptAlarmChipCfg->pAlarmCfg->Gbic_Control.ulReg));
        break;
        
    default:
        rc = bb_INV_PARAMETER;
        break;
    }
    
    return rc;
}

/******************************************************************************
* Function : Ixf1110InitAlarmCallback
*------------------------------------------------------------------------------
* Description : Initialize the alarm callback function with the IXF30007
*               device driver
*------------------------------------------------------------------------------
* Inputs  : pChipData      : Validated Chip Data
*           pAlarmCallback : Alarm Callback
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110InitAlarmCallback(bb_ChipData_t *pChipData,
                          AlarmCallBack pAlarmCallbackArg)
{
    ixf1110_AlarmCfg_t *AlarmConfigPtr
        = (ixf1110_AlarmCfg_t *) pChipData->pAlarmCfg;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    /* Error if we have not initialized our Alarm structure */
    if (AlarmConfigPtr == NULL)
        return bb_NO_ALARM_CFG;
    
    /* Assign the Alarm Callback Function Pointer */
    AlarmConfigPtr->pAlarmCallback = pAlarmCallbackArg;
    
    /* Indicate that the driver is using Alarms */
    AlarmConfigPtr->bAlarmsUsed = TRUE;
    
    /* Set flag indicating we are complete with Callback assigment */
    /* Alarm Callback Function Pointer has been assigned */
    (*AlarmConfigPtr).bAlarmCallbackAssigned = TRUE;
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110ChipIsr
*------------------------------------------------------------------------------
* Description : The ISR services the interrupts that the user has selected to
*               monitor
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110ChipIsr(bb_ChipData_t *pChipData)
{
    ixf1110_AlarmCfg_t *AlarmConfigPtr = (ixf1110_AlarmCfg_t *) pChipData->pAlarmCfg;
    
    OS_CMD_METRIC_EVENT(IXF_EVENT_CHIPISR_ENTRY, 0,0);
    
    if (AlarmConfigPtr == NULL)
        return bb_NO_ALARM_CFG;
    
    /* Validate ChipType */
    if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

	/*ensure callback assigned*/
    if (AlarmConfigPtr->bAlarmCallbackAssigned)
    	/*call callback*/
    	(*AlarmConfigPtr->pAlarmCallback)(pChipData, bb_1110_XGMAC_ALARM,
    										(AlarmConfigPtr->pAlarmCfg));	    
    else
        return bb_NO_ALARM_CFG;
    
    OS_CMD_METRIC_EVENT(IXF_EVENT_CHIPISR_EXIT, 0,0);
   
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110InitAllocMemory
*------------------------------------------------------------------------------
* Description : Allocate the memory space for IXF1110 device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110InitAllocMemory(bb_ChipData_t *pChipData)
{
    ixf1110_AlarmCfg_t *pAlarmCfg;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    /* Allocate memory for ChipCfg */
    pChipData->pChipCfg
        = (ixf1110_ChipCfg_t *) OS_CMD_ALLOC_MEM(sizeof(ixf1110_ChipCfg_t));
    
    /* Allocate memory for AlarmCfg */
    pChipData->pAlarmCfg
        = (ixf1110_AlarmCfg_t *)
        OS_CMD_ALLOC_MEM(sizeof(ixf1110_AlarmCfg_t));
    
    /* Cast the void pointer */
    pAlarmCfg = (ixf1110_AlarmCfg_t *) pChipData->pAlarmCfg;
    
    pAlarmCfg->pAlarmCfg
        = (ixf1110_AlarmCfgSet_t *)
        OS_CMD_ALLOC_MEM(sizeof(ixf1110_AlarmCfgSet_t));
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110DeAllocMemory
*------------------------------------------------------------------------------
* Description : Deallocate the memory space for IXF1110 device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NO_CHIP_CFG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110DeAllocMemory(bb_ChipData_t *pChipData)
{
    ixf1110_AlarmCfg_t* pAlarmCfg;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    /* Deallocate memory for ChipCfg */
    if (pChipData->pChipCfg) 
    {       
        OS_CMD_DEALLOC_MEM(pChipData->pChipCfg);
    }
    else 
    {
        /* Exit with error. Chip configuration parameter is not initialized */
        return bb_NO_CHIP_CFG;
    }
    
    /* Deallocate memory for AlarmCfg */
    if (pChipData->pAlarmCfg) 
    {
        pAlarmCfg = (ixf1110_AlarmCfg_t *) pChipData->pAlarmCfg;
        OS_CMD_DEALLOC_MEM(pAlarmCfg->pAlarmCfg);
        OS_CMD_DEALLOC_MEM(pChipData->pAlarmCfg);
    } 
    else 
    {
        /* Exit with error. Chip configuration parameter is not initialized */
        return bb_NO_ALARM_CFG;
    }
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110InitChip
*------------------------------------------------------------------------------
* Description : Initialize the ixf1110 chip via OCM UI generated
*               data structure
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           pTable : Table to be committed to the device.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable)
{
    bb_Error_e rc = bb_NO_ERROR;
    InitRegTable_t *pRegPtr;
    
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }
    
    if (NULL == pTable)
    {
        return bb_NULL_ARG;
    }
    
    /* Initialize the register pointer */
    pRegPtr = pTable;
    
    /* Loop through the array commiting the data down to the chip */
    while ((pRegPtr->offset != (bb_RegPointer_type)0xFFFF) &&
        (pRegPtr->value  != IXF_INIT_DATA_DELIMITER))
    {
        HAL_IXF1110_WRITE((pChipData->BaseAddress + (ulong) pRegPtr->offset),
            pRegPtr->value);
        
        pRegPtr++;
    }
    
    return rc;
}

/******************************************************************************
* Function : Ixf1110IxfApiSetChipOnline
*------------------------------------------------------------------------------
* Description : 
*               
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section : Chip segment
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110SetChipOnline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section)
{
    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }


    return Ixf1110GlobalSetChipOnline(pChipData, section);

}

/******************************************************************************
* Function : Ixf1110IxfApiSetChipOffline
*------------------------------------------------------------------------------
* Description : 
*               
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section : Chip segment
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110SetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section)
{

    /* Validate ChipType */
        if (pChipData->ChipType != bb_1110_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }


    return Ixf1110GlobalSetChipOffline(pChipData, section);

}


