/*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_alarms_d.h $
*   WORKFILE:       $Workfile: ixf1110_alarms_d.h $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: 
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_alarms_d.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 5     9/24/03 2:59p Rnballar
 * adding change struct to alarmsetcfg struct
 * 
 * 4     9/23/03 10:21a Rnballar
 * added ixf1110 alarms
 * 
 * 3     9/22/03 4:05p Rnballar
 * 
 * 2     9/09/03 4:46p Tepulkow
 * Created and modified many header files for IXF GI creation
 * 
 * 1     8/04/03 3:51p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 13    3/06/03 1:58p Pkjain2
* 
* 12    1/30/03 4:40p Pkjain2
* Made changes for PTQ release. Added Getstatus calls for SONET and
* GLOBAL block.
* 
* 11    10/17/02 11:33a Jaalliso
* Replacing tab characters with spaces
* 
* 10    8/28/02 11:23a Kssaepha
* Added Counter Callback types
* 
* 9     3/14/02 3:05p Srodrig1
* 
* 8     3/04/02 9:51a Srodrig1
* 
* 7     3/01/02 2:09p Srodrig1
* 
* 6     3/01/02 10:48a Srodrig1
* 
* 5     1/25/02 11:06a Srodrig1
* 
* 4     1/23/02 2:28a Srodrig1
* 
* 3     1/21/02 9:41p Srodrig1
* 
* 
* 1     12/15/01 9:58p Srodrig1
* Initial check in - file headers only
* 
* --------------------------------------------------------------------------
*/
#ifndef IXF1110_ALARMS_D_H
#define IXF1110_ALARMS_D_H

#include "ixf_bitfield_macros.h"

/* Interrupt Hierarchy:
The Alarm are set in the following hierarchy 

  Chip Interrupts ___ Global Clock
  |
  |___ Watch Dog
  |
  |___ Global Interrupt
  |
  |___ APS/WIS ___ Rx LOH    
  |           |
  |           |___ Framer Ohp
  |
  |
  |____ SONET/APS ___ Rx POH 
  |              |
  |              |___ Rx J1
  |              |
  |              |___ Rx Pointer
  |              |
  |              |___ Framer SOH 
  |              |
  |              |___ Framer Alarm
  |              |
  |              |___ Tx Tim (Loss of system and/or Pulse Clock)
  |              |
  |              |___ Serial Alarm
  |              |
  |              |___ Transmit Transport Serial Alarm
  | 
  |___ GFP
  |
  |___ PCS
  |
  |___ MAC 1
  |
  |___ MAC 2
  |
  |___ SIF/SPI4
  |
  |___ POS
*/



/******************************************************************************
* Alarm configuration structures:
*     These alarm configuration structures will be used to enable\disable the 
* desired alarms.
*    The structures are divided in a bitfield structure and a register, so every 
* member of structure is really a direct mapping of one of the bits of the 
* register, so we can manipulate individual bits without the use of shifts 
* and/or masks.
*     We are using a union that make us possible read/write the contents as a whole 
* register, or read/write individual bits. Example:
*
*   typedef union
*    {
*        uchar uiRegbyte;  --This is our 8 bits register
*        struct {
*           uchar bit0     :1; -- Least significant bit mapped
*           uchar bit1     :1;
*           uchar bit2     :1;
*           uchar bit3     :1;
*           uchar bit4to7  :4; -- The four most significant bits
*         } tBitField;
*     } regByte_t;
*
*   .
*   .
*   .
*   regByte_t register;
*
*   register.uiRegByte = 0x00;
*   register.tBitField.bit0 = 1; -- Now uiRegByte is 0x01
*   register.tBitField.bit4to7 = 0xF; -- Now uiRegByte is 0xF1;
*   register.tBitFiled.bit3 = 1; -- uiRegByte is 0xF9... etc.
*
******************************************************************************/ 


/*Possible states of the  Interrupt Status*/
enum 
{
    ixf1110_eINTERRUPT_INACTIVE = 0,
        ixf1110_eINTERRUPT_ACTIVE
} ixf1110_eIntrptStatus_e;


/*Possible states of the  Interrupt configuration*/
enum { 
    ixf1110_eINTERRUPT_DISABLE = 0,
        ixf1110_eINTERRUPT_ENABLE
}ixf1110_eIntrEnabling_e;


/*	GBIC Status Register Ports 0-9
	3 interrupts -- rx_los, tx_fault & mod_def*/
typedef union {

	ulong ulReg;
	struct {
		BITFIELD_4(		
			uint uiMod_Def		:10,	
			uint uiTx_Fault		:10,
			uint uiRx_Los		:10,
			uint				:2			
		);
	} tCnfg;

} ixf1110_Gbic_Status_t;

/*struct to indicate gbic int change
	1 = bit changed, 0 = no change */
typedef union {

	ulong ulReg;
	struct {
		BITFIELD_4(		
			uint uiMod_Def_chg		:10,	
			uint uiTx_Fault_chg		:10,
			uint uiRx_Los_chg		:10,
			uint					:2			
		);
	} tCnfg;

} ixf1110_Gbic_Change_t;




/*GBIC Control Register Ports 0-9 Register
3 interrupts -- rx_los, tx_fault & mod_def*/
typedef union {

	ulong ulReg;
	struct {
		BITFIELD_5(
			uint uiTx_Disable		:10,
			uint uiMod_Def_En		:1,
			uint uiTx_Fault_En		:1,
			uint uiRx_Los_En		:1,
			uint 		  			:19
		);
	} tCnfg;
	
} ixf1110_Gbic_Control_t;



typedef  struct /* Alarm Configuration & Status */
{   
	ixf1110_Gbic_Status_t 	Gbic_Status;
	ixf1110_Gbic_Change_t	Gbic_Change;
	ixf1110_Gbic_Control_t	Gbic_Control;
    
} ixf1110_AlarmCfgSet_t;


/* Collected Alarm Config - for chip */
typedef struct 
{
    boolean                  bAlarmsUsed;
    boolean                  bAlarmCallbackAssigned;
    AlarmCallBack            pAlarmCallback;
    ixf1110_AlarmCfgSet_t*  pAlarmCfg; 
}  ixf1110_AlarmCfg_t;



#endif

