/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_global.c $
*   WORKFILE:       $Workfile: ixf1110_global.c $
*
* --------------------------------------------------------------------------
* Project: ixf18100 family device API
*
* Purpose: This file contains Global block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_global.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 8     10/13/03 4:58p Tepulkow
 * added setchiponline and offline functions
 * 
 * 6     10/02/03 3:26p Rnballar
 * fixed read/write, removed redundant offset
 * 
 * 5     9/26/03 2:54p Rnballar
 * added reset functionailty
 * 
 * 4     9/26/03 9:56a Rnballar
 * added get/set cfg
 * 
 * 3     9/18/03 1:30p Rnballar
 * 
 * 2     9/09/03 4:46p Tepulkow
 * Created and modified many header files for IXF GI creation
 * 
 * 1     8/04/03 3:51p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf1110_global_d.h"
#include "ixf1110_global.h"
#include "ixf1110_global_b.h"
#include "ixf1110_xgmac_b.h"
#include "ixf1110_util.h"
#include "ixf1110.h"
#include "ixf1110_d.h"
#include "hal.h"


/******************************************************************************
* Function : Ixf1110GlobalSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the global block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the values to set int he device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110GlobalSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_GlbStatusCfg_t *pCnfg)
{
	if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }


	/*Port Enable Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_PORT_EN_OFFSET),
					pCnfg->Glb_PortEn.ulReg);

   	/*Link LED Enable Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_LINKLED_EN_OFFSET),
					pCnfg->Glb_LinkLEDEn.ulReg);


	/*CPU Interface Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_CPU_XFACE_OFFSET),
					pCnfg->Glb_CPUXFace.ulReg);

	/*LED Control Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_LED_CONTROL_OFFSET),
					pCnfg->Glb_LEDControl.ulReg);

	/*LED Flash Rate Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_LED_FLASH_RATE_OFFSET),
					pCnfg->Glb_LEDFlashRate.ulReg);

	/*LED Fault Disable Register*/
	HAL_IXF1110_WRITE((BaseAddress + ixf1110_LED_FAULT_DIS_OFFSET),
					pCnfg->Glb_LEDFaultDis.ulReg);

	    
    return bb_NO_ERROR;
}


/******************************************************************************
* Function : Ixf1110GlobalGetCfg
*------------------------------------------------------------------------------
* Description : This function gets global block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110GlobalGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_GlbStatusCfg_t *pCnfg)
{

	if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }


	/*Port Enable Register*/
	pCnfg->Glb_PortEn.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_PORT_EN_OFFSET);

   	/*Link LED Enable Register*/
	pCnfg->Glb_LinkLEDEn.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_LINKLED_EN_OFFSET);


	/*CPU Interface Register*/
	pCnfg->Glb_CPUXFace.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_CPU_XFACE_OFFSET);

	/*LED Control Register*/
	pCnfg->Glb_LEDControl.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_LED_CONTROL_OFFSET);

	/*LED Flash Rate Register*/
	pCnfg->Glb_LEDFlashRate.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_LED_FLASH_RATE_OFFSET);

	/*LED Fault Disable Register*/
	pCnfg->Glb_LEDFaultDis.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_LED_FAULT_DIS_OFFSET);



    return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf1110GlobalGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GlobalGetStatus(bb_RegPointer_type BaseAddress,
                        bb_SelStatus_e selStatus, void* pStatus)
{
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110MacSftReset
*------------------------------------------------------------------------------
* Description : This function resets a mac port control registers
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ushort channel -- port to resset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHANNEL, bb_GENERAL_ERROR 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110MacSftReset(bb_RegPointer_type BaseAddress, ushort channel)
{
	/*validate channel*/
	if(channel > ixf1110_XGMAC_CH_CNT - 1)
		return bb_INV_CHANNEL;
	
	/*set reset bit active for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_MACRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_MACRESET_OFFSET) |
				(1 << channel)));
				
	/*set reset bit inactive for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_MACRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_MACRESET_OFFSET) &
				~(1 << channel)));  

	return bb_NO_ERROR;	
}
/******************************************************************************
* Function : Ixf1110MacSftResetAll
*------------------------------------------------------------------------------
* Description : This function resets all mac channels(ports)
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110MacSftResetAll(bb_RegPointer_type BaseAddress)
{
	/*set all reset bits active for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_MACRESET_OFFSET,
				ixf1110_mMACRESET_ALL);
				
	/*set all reset bits inactive for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_MACRESET_OFFSET,
				~ixf1110_mMACRESET_ALL);
				

	return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf1110CoreClkSftReset
*------------------------------------------------------------------------------
* Description : This function resets mac core clock system
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110CoreClkSftReset(bb_RegPointer_type BaseAddress)
{
	/*set all reset bits active for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_CORCLKSFTRESET_OFFSET,
				ixf1110_mCORCLKSFTRST);
				
	/*set all reset bits inactive for xgmac channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_CORCLKSFTRESET_OFFSET,
				~ixf1110_mCORCLKSFTRST);
	
	return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf1110GlobalSetChipOffline
*------------------------------------------------------------------------------
* Description : This function stes the specified channel off line.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to set off line.
* Outputs : 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GlobalSetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection)
{
    	bb_Error_e rc = bb_NO_ERROR;
    	uint data;

	/*validate channel*/
	if(pSection->group.channel > ixf1110_XGMAC_CH_CNT - 1)
		return bb_INV_CHANNEL;

    	data = HAL_IXF1110_READ(pChipData->BaseAddress + ixf1110_PORT_EN_OFFSET);
	data &= ~(1 << pSection->group.channel);
   	HAL_IXF1110_WRITE(pChipData->BaseAddress + ixf1110_PORT_EN_OFFSET,data);

    	return rc;
}

/******************************************************************************
* Function : Ixf1110GlobalSetChipOnline
*------------------------------------------------------------------------------
* Description : This function stes the specified channel on line.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to set on line.
* Outputs : 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110GlobalSetChipOnline(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection)
{
    	bb_Error_e rc = bb_NO_ERROR;
    	uint data;

	/*validate channel*/
	if(pSection->group.channel > ixf1110_XGMAC_CH_CNT - 1)
		return bb_INV_CHANNEL;

    	data = HAL_IXF1110_READ(pChipData->BaseAddress + ixf1110_PORT_EN_OFFSET);
	data |= (1 << pSection->group.channel);
   	HAL_IXF1110_WRITE(pChipData->BaseAddress + ixf1110_PORT_EN_OFFSET,data);

    	return rc;
}

