/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_rx.c $
*   WORKFILE:       $Workfile: ixf1110_rx.c $
*
* --------------------------------------------------------------------------
* Project: ixf1110 device API
*
* Purpose: This file contains rx block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_rx.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 10    10/10/03 4:30p Tepulkow
 * coded out the getcounters support for RX and TX blocks
 * 
 * 9     10/10/03 3:01p Tepulkow
 * added include for xgmac_b.h and changed code to use the mac_cnt macro
 * defined therein
 * 
 * 8     10/01/03 1:32p Rnballar
 * added get status
 * 
 * 7     9/26/03 2:55p Rnballar
 * added reset functionailty
 * 
 * 6     9/23/03 1:26p Rnballar
 * added functionality set & get config
 * 
 * 5     9/18/03 2:18p Rnballar
 * 
 * 4     9/18/03 2:09p Tepulkow
 * Updated function header for get_counters
 * 
 * 3     9/18/03 12:03p Tepulkow
 * added get counter function
 * 
 * 2     9/16/03 5:00p Tepulkow
 * Created function framework.
 * 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf1110_rx_d.h"
#include "ixf1110_rx.h"
#include "ixf1110_rx_b.h"
#include "ixf1110_xgmac_b.h"
#include "ixf1110.h"
#include "ixf1110_d.h"
#include "hal.h"

/******************************************************************************
* Function : Ixf1110RxSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the Rx block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptRxCfg : Pointer to the values to set int the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_RxCfg_t *ptRxCfg)
{
    
    /* Sanity Check */
    if(ptRxCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    /* Set RX config fill out the register mapping and write
    the proper register */
	/*Rx FIFO Hi water*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER0_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_0);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER1_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_1);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER2_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_2);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER3_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_3); 
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER4_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_4);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER5_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_5);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER6_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_6);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER7_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_7);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER8_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_8);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXHIWATER9_OFFSET,
						ptRxCfg->RxHiWaterMrk.ulPort_9);
	/*Rx FIFO Low water*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER0_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_0);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER1_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_1);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER2_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_2);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER3_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_3);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER4_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_4);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER5_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_5);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER6_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_6);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER7_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_7);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER8_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_8);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXLOWATER9_OFFSET,
						ptRxCfg->RxLowWaterMrk.ulPort_9);

	/*Rx FIFO err frame en*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXERRFRMDROPEN_OFFSET,
						ptRxCfg->RxErrFrmDrpEn.ulReg);
    
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110RxGetCfg
*------------------------------------------------------------------------------
* Description : This function gets Rx block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptRxCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_RxCfg_t *ptRxCfg)
{
    /* Sanity Check */
    if(ptRxCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    /* Read the registers containing the Rx configuration */
	/*Rx FIFO Hi water*/
	ptRxCfg->RxHiWaterMrk.ulPort_0 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER0_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_1 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER1_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_2 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER2_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_3 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER3_OFFSET); 
	ptRxCfg->RxHiWaterMrk.ulPort_4 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER4_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_5 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER5_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_6 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER6_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_7 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER7_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_8 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER8_OFFSET);
	ptRxCfg->RxHiWaterMrk.ulPort_9 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXHIWATER9_OFFSET);
	/*Rx FIFO Low water*/
	ptRxCfg->RxLowWaterMrk.ulPort_0 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER0_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_1 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER1_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_2 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER2_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_3 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER3_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_4 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER4_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_5 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER5_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_6 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER6_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_7 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER7_OFFSET);
   	ptRxCfg->RxLowWaterMrk.ulPort_8 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER8_OFFSET);
	ptRxCfg->RxLowWaterMrk.ulPort_9 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXLOWATER9_OFFSET);

	/*Rx FIFO err frame en*/
	ptRxCfg->RxErrFrmDrpEn.ulReg =
			HAL_IXF1110_READ(BaseAddress + ixf1110_RXERRFRMDROPEN_OFFSET);

    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110RxGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 32 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110RxGetStatus(bb_RegPointer_type BaseAddress,
                        bb_SelStatus_e selStatus, void* pStatus)
{
    if(pStatus == NULL){
        return bb_NULL_ARG;
	}

	if(selStatus != ixf_eALL_STATUS) {
		return bb_INV_SEL_STATUS_TYPE;
	}


	((ixf1110_RxFIFO_Status_t *)pStatus)->RxOvrFlwEvent.ulReg = 
				HAL_IXF1110_READ(BaseAddress + ixf1110_RXOVRFLWEVENT_OFFSET);
		

    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110RxGetCounters
*------------------------------------------------------------------------------
* Description  : The function reads the counter register  values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/

extern bb_Error_e
Ixf1110RxGetCounters(bb_RegPointer_type BaseAddress, bb_SelCounters_e SelCntr,
						 void *ptCounters)
{
    if(ptCounters == NULL)
	{
        return bb_NULL_ARG;
	}

    ixf1110_RxFIFO_NumFrmRem_t *ptCounterData;

        ptCounterData = (ixf1110_RxFIFO_NumFrmRem_t *) ptCounters;

		ptCounterData->ulPort_0 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV0_OFFSET);

		ptCounterData->ulPort_1 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV1_OFFSET);

		ptCounterData->ulPort_2 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV2_OFFSET);

		ptCounterData->ulPort_3 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV3_OFFSET);

		ptCounterData->ulPort_4 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV4_OFFSET);

		ptCounterData->ulPort_5 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV5_OFFSET);

		ptCounterData->ulPort_6 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV6_OFFSET);

		ptCounterData->ulPort_7 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV7_OFFSET);

		ptCounterData->ulPort_8 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV8_OFFSET);

		ptCounterData->ulPort_9 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_RXNUMFRMREMV9_OFFSET);
    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110RxFifoReset
*------------------------------------------------------------------------------
* Description : This function resets an individual mac fifo
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ushort channel -- port to resset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHANNEL, bb_GENERAL_ERROR 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxFifoReset(bb_RegPointer_type BaseAddress, ushort channel)
{
   /*validate channel*/
	if(channel > ixf1110_XGMAC_CH_CNT - 1)
		return bb_INV_CHANNEL;
	
	/*set reset bit active for rx fifo channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET) |
				(1 << channel)));
				
	/*set reset bit inactive for rx fifo channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET) &
				~(1 << channel)));


	
	return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf1110RxFifoResetAll
*------------------------------------------------------------------------------
* Description : This function resets all rx fifos
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110RxFifoResetAll(bb_RegPointer_type BaseAddress)
{
	/*set all reset bits active for rx fifos*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET,
				ixf1110_mRXFIFOPORTRESET_ALL);
				
	/*set all reset bits inactive for rx fifo*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_RXFIFOPORTRESET_OFFSET,
				~ixf1110_mRXFIFOPORTRESET_ALL);

	return bb_NO_ERROR;
}
