/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_spi4.c $
*   WORKFILE:       $Workfile: ixf1110_spi4.c $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: This file contains SPI4 block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_spi4.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:57p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 5     10/01/03 1:15p Rnballar
 * added get status
 * 
 * 4     9/29/03 2:07p Rnballar
 * added set/get cfg
 * 
 * 3     9/18/03 12:04p Rnballar
 * 
 * 2     9/09/03 4:46p Tepulkow
 * Created and modified many header files for IXF GI creation
 * 
 * 1     8/04/03 3:51p Tepulkow
 * 
 * 1     5/16/03 1:43p Tepulkow
* 
* 20    1/20/03 2:05p Pkjain2
* beautified the code
* 
* 19    1/15/03 5:21p Jaalliso
* Detabbing
* 
* 18    7/08/02 1:16p Pkjain2
* Moved Calypso specific data stuctures into Calypso block specific data
* structure file.
* 
* 16    4/15/02 4:53p Pkjain2
* Cleaned up some if 0 code.
* 
* 15    4/12/02 10:27a Pkjain2
* 
* 14    3/06/02 8:49a Pkjain2
* Updated to EAS 5.0
* 
* 13    2/12/02 11:11a Jaalliso
* Renaming SIF loopbacks to be more generic
* 
* 12    12/17/01 2:49p Pkjain2
* Added trace macros
* 
* 3     9/05/01 5:14p Pkjain2
* updated code to meet the spec received in last week of august
* 
* 2     8/15/01 8:35a Pkjain2
* Added SPI 4 Code
* 
* 1     7/18/01 10:55a Rfranz
* Initial check in - file headers only
* 
* --------------------------------------------------------------------------
*/
#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf1110_spi4_b.h"
#include "ixf_api_spi4_d.h"
#include "ixf1110_spi4_d.h"
#include "ixf1110_d.h"
#include "hal.h"


/******************************************************************************
* Function : Ixf1110Spi4SetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be set
*           ixf1110_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set in the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110Spi4SetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
 					ixf1110_Spi4Cfg_t *ptSpi4Cfg)
{
	 /* Sanity Check */
    if(ptSpi4Cfg == NULL)
    {
        return bb_NULL_ARG;
    }

	/*SPI4-2 RX Burst Size Register*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_SIF_RX_BURST_OFFSET,
					ptSpi4Cfg->Spi4BurstSize.ulReg);
	/*SPI4-2 RX Training Register*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_SIF_RX_TRAINING_OFFSET,
					ptSpi4Cfg->Spi4Training.ulReg);
	/*SPI4-2 RX Calendar Register*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_SIF_RX_CALENDAR_OFFSET,
					ptSpi4Cfg->Spi4RxCalendar.ulReg);
	/*SPI4-2 TX Synchronization Register*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_SIF_TX_SYNCH_OFFSET,
					ptSpi4Cfg->Spi4TxSynchro.ulReg);
	 
	 	  	 
	return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110Spi4GetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be fetched
*           ixf1110_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110Spi4GetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
 				ixf1110_Spi4Cfg_t *ptSpi4Cfg)
{
	/* Sanity Check */
    if(ptSpi4Cfg == NULL)
    {
        return bb_NULL_ARG;
    }

 	/*SPI4-2 RX Burst Size Register*/
	ptSpi4Cfg->Spi4BurstSize.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_SIF_RX_BURST_OFFSET);
	/*SPI4-2 RX Training Register*/
	ptSpi4Cfg->Spi4Training.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_SIF_RX_TRAINING_OFFSET);
	/*SPI4-2 RX Calendar Register*/
	ptSpi4Cfg->Spi4RxCalendar.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_SIF_RX_CALENDAR_OFFSET);
	/*SPI4-2 TX Synchronization Register*/
	ptSpi4Cfg->Spi4TxSynchro.ulReg = 
		HAL_IXF1110_READ(BaseAddress + ixf1110_SIF_TX_SYNCH_OFFSET);

 
         
    return bb_NO_ERROR;
}
         
/******************************************************************************
* Function : Ixf1110Spi4GetStatus
*------------------------------------------------------------------------------
* Description : The function reads all the SPI4 counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf1110_Spi4Status_t *ptSpi4Status: pointer to the status 
*               structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110Spi4GetStatus(bb_RegPointer_type BaseAddress,
					bb_SelStatus_e selStatus, void* pStatus)
{
 
	if(selStatus != ixf_eALL_STATUS) {
		return bb_INV_SEL_STATUS_TYPE;
	}

	/*SPI4-2 RX Calendar Register.DIP2_Error*/
	((ixf1110_Spi4Status_t *)pStatus)->uiDIP2_Error = 
		((HAL_IXF1110_READ(BaseAddress + ixf1110_SIF_RX_CALENDAR_OFFSET) &
		ixf1110_mSIF_RX_DIP2ERR) >> ixf1110_sSIF_RX_DIP2ERR);


	return bb_NO_ERROR;
}

