/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_tx.c $
*   WORKFILE:       $Workfile: ixf1110_tx.c $
*
* --------------------------------------------------------------------------
* Project: ixf1110 device API
*
* Purpose: This file contains tx block related routines
*
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_tx.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 8     10/10/03 4:30p Tepulkow
 * coded out the getcounters support for RX and TX blocks
 * 
 * 7     10/10/03 3:01p Tepulkow
 * added include for xgmac_b.h and changed code to use the mac_cnt macro
 * defined therein
 * 
 * 6     10/01/03 1:32p Rnballar
 * added get status
 * 
 * 5     9/26/03 2:55p Rnballar
 * added reset functionailty
 * 
 * 4     9/23/03 12:43p Rnballar
 * 
 * 3     9/18/03 2:43p Rnballar
 * 
 * 2     9/18/03 2:09p Tepulkow
 * Updated function header for get_counters
 * 
 * 1     9/18/03 12:02p Tepulkow
 * Created for ixf1110
 * 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf1110_tx_d.h"
#include "ixf1110_tx.h"
#include "ixf1110_tx_b.h"
#include "ixf1110_xgmac_b.h"
#include "ixf1110_util.h"
#include "ixf1110.h"
#include "ixf1110_d.h"
#include "hal.h"

/******************************************************************************
* Function : Ixf1110TxSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the Tx block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptTxCfg : Pointer to the values to set int the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110TxSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_TxCfg_t *ptTxCfg)
{
    /* Sanity Check */
    if(ptTxCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    /* Set Tx config fill out the register mapping and write
    the proper register */
	/*Tx FIFO Hi water*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER0_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_0);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER1_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_1);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER2_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_2);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER3_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_3); 
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER4_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_4);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER5_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_5);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER6_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_6);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER7_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_7);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER8_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_8);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXHIWATER9_OFFSET,
						ptTxCfg->TxHiWaterMrk.ulPort_9);
	/*Tx FIFO Low water*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER0_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_0);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER1_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_1);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER2_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_2);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER3_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_3);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER4_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_4);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER5_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_5);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER6_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_6);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER7_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_7);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER8_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_8);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXLOWATER9_OFFSET,
						ptTxCfg->TxLowWaterMrk.ulPort_9);
	/*Tx FIFO MAC Thresh*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH0_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_0);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH1_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_1);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH2_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_2);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH3_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_3);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH4_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_4);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH5_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_5);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH6_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_6);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH7_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_7);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH8_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_8);
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXMACTHRESH9_OFFSET,
						ptTxCfg->TxMacThreshold.ulPort_9);



    return bb_NO_ERROR;
}


/******************************************************************************
* Function : Ixf1110TxGetCfg
*------------------------------------------------------------------------------
* Description : This function gets Tx block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptTxCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110TxGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf1110_TxCfg_t *ptTxCfg)
{
    /* Sanity Check */
    if(ptTxCfg == NULL)
    {
        return bb_NULL_ARG;
    }
    
    /* Read the registers containing the Tx configuration*/
	/*Tx FIFO Hi water*/
   	ptTxCfg->TxHiWaterMrk.ulPort_0 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER0_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_1 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER1_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_2 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER2_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_3 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER3_OFFSET); 
	ptTxCfg->TxHiWaterMrk.ulPort_4 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER4_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_5 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER5_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_6 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER6_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_7 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER7_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_8 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER8_OFFSET);
	ptTxCfg->TxHiWaterMrk.ulPort_9 =
   				HAL_IXF1110_READ(BaseAddress + ixf1110_TXHIWATER9_OFFSET);
	/*Tx FIFO Low water*/
	ptTxCfg->TxLowWaterMrk.ulPort_0 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER0_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_1 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER1_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_2 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER2_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_3 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER3_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_4 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER4_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_5 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER5_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_6 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER6_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_7 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER7_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_8 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER8_OFFSET);
	ptTxCfg->TxLowWaterMrk.ulPort_9 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXLOWATER9_OFFSET);
	/*Tx FIFO MAC Thresh*/
	ptTxCfg->TxMacThreshold.ulPort_0 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH0_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_1 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH1_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_2 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH2_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_3 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH3_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_4 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH4_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_5 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH5_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_6 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH6_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_7 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH7_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_8 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH8_OFFSET);
	ptTxCfg->TxMacThreshold.ulPort_9 =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXMACTHRESH9_OFFSET);

    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110TxGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 32 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110TxGetStatus(bb_RegPointer_type BaseAddress,
                        bb_SelStatus_e selStatus, void* pStatus)
{
    if(pStatus == NULL){
        return bb_NULL_ARG;
	}
	if(selStatus != ixf_eALL_STATUS) {
		return bb_INV_SEL_STATUS_TYPE;
	}

	/*TX FIFO Overflow Event Register*/
	((ixf1110_TxFIFO_Status_t *)pStatus)->TxOvrFlwEvent.ulReg =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXOVRFLWEVENT_OFFSET); 
		

	/*TX FIFO Info Out-of-Sequence Register*/
	((ixf1110_TxFIFO_Status_t *)pStatus)->TxInfoOutSeq.ulReg =
				HAL_IXF1110_READ(BaseAddress + ixf1110_TXINFOUTSEQ_OFFSET); 
			

    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110TxGetCounters
*------------------------------------------------------------------------------
* Description  : The function reads the counter register  values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/

extern bb_Error_e
Ixf1110TxGetCounters(bb_RegPointer_type BaseAddress, bb_SelCounters_e SelCntr,
						 void *ptCounters)
{
    if(ptCounters == NULL)
	{
        return bb_NULL_ARG;
	}

    ixf1110_TxFIFO_NumFrmRem_t *ptCounterData;

        ptCounterData = (ixf1110_TxFIFO_NumFrmRem_t *) ptCounters;

		ptCounterData->ulPort_0 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE0);

		ptCounterData->ulPort_1 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE1);

		ptCounterData->ulPort_2 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE2);

		ptCounterData->ulPort_3 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE3);

		ptCounterData->ulPort_4 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE4);

		ptCounterData->ulPort_5 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE5);

		ptCounterData->ulPort_6 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE6);

		ptCounterData->ulPort_7 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE7);

		ptCounterData->ulPort_8 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE8);

		ptCounterData->ulPort_9 = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_TXNUMFRMREMOVE9);

    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110TxFifoReset
*------------------------------------------------------------------------------
* Description : This function resets an individual mac fifo
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ushort channel -- port to resset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHANNEL, bb_GENERAL_ERROR 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110TxFifoReset(bb_RegPointer_type BaseAddress, ushort channel)
{
	/*validate channel*/
	if(channel > ixf1110_XGMAC_CH_CNT - 1)
		return bb_INV_CHANNEL;
	
	/*set reset bit active for tx fifo channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET) |
				(1 << channel)));
				
	/*set reset bit inactive for tx fifo channel*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET,
				(HAL_IXF1110_READ(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET) &
				~(1 << channel)));

	return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf1110TxFifoReseAll
*------------------------------------------------------------------------------
* Description : This function resets all rx fifos
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_GENERAL_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
bb_Error_e
Ixf1110TxFifoResetAll(bb_RegPointer_type BaseAddress)
{
	/*set all reset bits active for tx fifos*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET,
				ixf1110_mTXFIFOPORTRESET_ALL);
				
	/*set all reset bits inactive for tx fifo*/
	HAL_IXF1110_WRITE(BaseAddress + ixf1110_TXFIFOPORTRESET_OFFSET,
				~ixf1110_mTXFIFOPORTRESET_ALL);

	return bb_NO_ERROR;
}
