/*
*---------------------------------------------------------------------------
*
*                  I N T E L   P R O P R I E T A R Y                   
*
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*
*                        INTEL  CORPORATION                            
*
*                     2200 MISSION COLLEGE BLVD                        
*
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_xgmac.c $
*   WORKFILE:       $Workfile: ixf1110_xgmac.c $
*
* --------------------------------------------------------------------------
* Project: ixf1110 family device API
*
* Purpose: This file contains XGMAC block related routines
*
* Notes:
*
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf1110/ixf1110_xgmac.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 11    10/21/03 10:37a Tepulkow
 * removed commented out code from get_cfg function
 * 
 * 10    10/01/03 1:47p Rnballar
 * changed last param of get status to void*
 * 
 * 9     9/25/03 10:09a Rnballar
 * added get/set cfg functionality
 * 
 * 8     9/18/03 12:05p Rnballar
 * 
 * 7     9/18/03 9:58a Tepulkow
 * removed excessive "break" statements from get_counters routine
 * 
 * 5     9/17/03 5:49p Tepulkow
 * removed unused functions.  stubbed out getcfg, setcfg, getstatus.
 * ported set and get mac address to 1110. started work on getcounters,
 * need to complete "get all counters".
 * 
 * 4     9/17/03 11:45a Tepulkow
 * rev history test
* 
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf1110_xgmac_b.h"
#include "ixf_api_ether_d.h"
#include "ixf1110_xgmac_d.h"
#include "ixf1110_xgmac.h"
#include "ixf1110_d.h"
#include "hal.h"

/******************************************************************************
* Function : Ixf1110XgmacSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the XGMAC block in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf1110_XgmacCfg_t *ptXgmacCfg: A pointer a structure containing
*                                          chip configuration settings.
* Outputs : Writes to registers to configure XGMAC settings on the chip.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ushort channel, ixf1110_XgmacCfg_t *ptXgmacCfg)
{
    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }

	/*Station Address Register Low*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_STA_ADDR_LO_OFFSET),
		ptXgmacCfg->Xgmac_StationAddr.ulStationAddrLow);
		 
	/*Station Address Register High*/
	HAL_IXF1110_WRITE(BaseAddress + 
		ixf1110_XGMAC_CH_REG(channel, ixf1110_STA_ADDR_HI_OFFSET),
		ptXgmacCfg->Xgmac_StationAddr.ulStationAddrHi);
		
	/*FDFC Type Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FDFC_TYPE_OFFSET),
		ptXgmacCfg->Xgmac_FDFCType.ulReg);

	/*FC TX Timer Value Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FCTX_TIMER_OFFSET),
		ptXgmacCfg->Xgmac_FCTxTimer.ulReg);
		 
	/*FDFC Address Low Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FDFC_ADDR_LO_OFFSET),
		ptXgmacCfg->Xgmac_FDFCAddr.ulFDFCAddrLow);
		 
	/*FDFC Address High Register*/
	HAL_IXF1110_WRITE(BaseAddress + 
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FDFC_ADDR_HI_OFFSET),
		ptXgmacCfg->Xgmac_FDFCAddr.ulFDFCAddrHi);

	/*IPG Transmit Time Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_IPG_TX_TIME_OFFSET),
		ptXgmacCfg->Xgmac_IPGTxTime.ulReg);
		 
	/*Pause Threshold Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_PAUSE_THRESHLD_OFFSET),
		ptXgmacCfg->Xgmac_PauseThreshold.ulReg);

	/*Max Frame Size Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_MAX_FRAME_SZ_OFFSET),
		ptXgmacCfg->Xgmac_MaxFrameSize.ulReg);


	/*FC Enable Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FC_EN_OFFSET),
		ptXgmacCfg->Xgmac_FCEnable.ulReg);

	/*Short Runts Threshold Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_SHORT_RUNTS_THRESHLD_OFFSET),
		ptXgmacCfg->Xgmac_ShortRuntThresh.ulReg);

	/*Discard Unknown Control Frame Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_DISCARD_UNKNOWN_CTRL_FRM_OFFSET),
		ptXgmacCfg->Xgmac_DisUnkCtrlFrm.ulReg);

	/*TX Config Word Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_CONFIG_WORD_OFFSET),
		ptXgmacCfg->Xgmac_TxConfigWord.ulReg);

	/*Diverse Config Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_DIVERSE_CONFIG_OFFSET),
		ptXgmacCfg->Xgmac_DivCfgWord.ulReg);

	/*RX Packet Filter Control Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_RXPCKT_FLTR_CTRL_OFFSET),
		ptXgmacCfg->Xgmac_RxPktFilter.ulReg);

	/*Port Multicast Address Low Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_PORT_MCAST_ADDR_LO_OFFSET),
		ptXgmacCfg->Xgmac_PortMultCstAddr.ulPortMultCstAddrLow);

	/*Port Multicast Address High Register*/
	HAL_IXF1110_WRITE(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_PORT_MCAST_ADDR_HI_OFFSET),
		ptXgmacCfg->Xgmac_PortMultCstAddr.ulPortMultCstAddrHi);	


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110XgmacGetCfg
*------------------------------------------------------------------------------
* Description : This function reads the XGMAC configuration settings into a 
*                data structure.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*			unshort channel: Selects channel to write status to.
*           ixf1110_XgmacCfg_t *ptXgmacCfg: A pointer to a structure used to
*                                          contain configuration information
*                                          obtained from the XGMAC registers.
* Outputs : A filled data structure containing XGMAC configuration information.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ushort channel, ixf1110_XgmacCfg_t *ptXgmacCfg)
{
    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }

	/*Station Address Register Low*/
	ptXgmacCfg->Xgmac_StationAddr.ulStationAddrLow =
		HAL_IXF1110_READ(BaseAddress + ixf1110_XGMAC_CH_REG(channel,
											ixf1110_STA_ADDR_LO_OFFSET)); 
	/*Station Address Register High*/
	ptXgmacCfg->Xgmac_StationAddr.ulStationAddrHi =	(ixf1110_mSTA_ADDR_HI &
		HAL_IXF1110_READ(BaseAddress + ixf1110_XGMAC_CH_REG(channel,
											ixf1110_STA_ADDR_HI_OFFSET)));

	/*FDFC Type Register*/
	ptXgmacCfg->Xgmac_FDFCType.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FDFC_TYPE_OFFSET));

	/*FC TX Timer Value Register*/
	ptXgmacCfg->Xgmac_FCTxTimer.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FCTX_TIMER_OFFSET));
		 
	/*FDFC Address Low Register*/
	ptXgmacCfg->Xgmac_FDFCAddr.ulFDFCAddrLow = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FDFC_ADDR_LO_OFFSET));
		 
	/*FDFC Address High Register*/
	ptXgmacCfg->Xgmac_FDFCAddr.ulFDFCAddrHi = (ixf1110_mFDFC_ADDR_HI &
		HAL_IXF1110_READ(BaseAddress + ixf1110_XGMAC_CH_REG(channel,
										ixf1110_FDFC_ADDR_HI_OFFSET)));

	/*IPG Transmit Time Register*/
	ptXgmacCfg->Xgmac_IPGTxTime.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_IPG_TX_TIME_OFFSET));
		 
	/*Pause Threshold Register*/
	ptXgmacCfg->Xgmac_PauseThreshold.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_PAUSE_THRESHLD_OFFSET));

	/*Max Frame Size Register*/
	ptXgmacCfg->Xgmac_MaxFrameSize.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_MAX_FRAME_SZ_OFFSET));


	/*FC Enable Register*/
	ptXgmacCfg->Xgmac_FCEnable.ulReg = HAL_IXF1110_READ(BaseAddress	+
		ixf1110_XGMAC_CH_REG(channel, ixf1110_FC_EN_OFFSET));

	/*Short Runts Threshold Register*/
	ptXgmacCfg->Xgmac_ShortRuntThresh.ulReg = HAL_IXF1110_READ(BaseAddress
		+ ixf1110_XGMAC_CH_REG(channel, ixf1110_SHORT_RUNTS_THRESHLD_OFFSET));

	/*Discard Unknown Control Frame Register*/
	ptXgmacCfg->Xgmac_DisUnkCtrlFrm.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_DISCARD_UNKNOWN_CTRL_FRM_OFFSET));

	/*TX Config Word Register*/
	ptXgmacCfg->Xgmac_TxConfigWord.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_CONFIG_WORD_OFFSET));

	/*Diverse Config Register*/
	ptXgmacCfg->Xgmac_DivCfgWord.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_DIVERSE_CONFIG_OFFSET));

	/*RX Packet Filter Control Register*/
	ptXgmacCfg->Xgmac_RxPktFilter.ulReg = HAL_IXF1110_READ(BaseAddress +
		ixf1110_XGMAC_CH_REG(channel, ixf1110_RXPCKT_FLTR_CTRL_OFFSET));

	/*Port Multicast Address Low Register*/
	ptXgmacCfg->Xgmac_PortMultCstAddr.ulPortMultCstAddrLow =
		HAL_IXF1110_READ(BaseAddress + ixf1110_XGMAC_CH_REG(channel,
										ixf1110_PORT_MCAST_ADDR_LO_OFFSET));

	/*Port Multicast Address High Register*/
	ptXgmacCfg->Xgmac_PortMultCstAddr.ulPortMultCstAddrHi =
		(ixf1110_mPORT_MCAST_ADDR_HI & HAL_IXF1110_READ(BaseAddress +
			ixf1110_XGMAC_CH_REG(channel, ixf1110_PORT_MCAST_ADDR_HI_OFFSET)));	


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110XgmacGetStatus
*------------------------------------------------------------------------------
* Description : This function reads the status registers of the XGMAC register
*               block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelStatus_e SelStat: Selects the status to be retrieved.
*			unshort channel: Selects channel to read status from.
*           void* pStatus
* Outputs : A filled data structure containing XGMAC status information. 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                       ushort channel, void* pStatus)
{

	/*RX Config Word Register*/
	((ixf1110_XgmacStatus_t *)pStatus)->Xgmac_RxConfigWord.ulReg =
		HAL_IXF1110_READ(BaseAddress + ixf1110_XGMAC_CH_REG(channel,
											ixf1110_RX_CONFIG_WORD_OFFSET));


    
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110XgmacGetCounters
*------------------------------------------------------------------------------
* Description : This function reads counter values in the XGMAC register
*                block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*			unshort channel: Selects channel to read counters from.
*           bb_SelCounters_e SelCntr: Selects the counters to be read.
*           void *ptCounters: A pointer to a structure used to
*                                          store counter values.
* Outputs : A filled data structure containing counter values.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf1110XgmacGetCounters(bb_RegPointer_type BaseAddress, ushort channel, 
                         bb_SelCounters_e SelCntr, void *ptCounters)
{
    ixfApi_XgmacCounters_t *ptCounterData;

    switch(SelCntr)
    {
    case bb_XGMAC_TX_OK_OCTS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_OCT_OK_OFFSET));
        break;

    case bb_XGMAC_TX_BAD_OCTS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_OCT_BAD_OFFSET));
        break;

    case bb_XGMAC_TX_UNICAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_UC_PKTS_OFFSET));
        break;

    case bb_XGMAC_TX_OK_MLTCAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_MC_PKTS_OFFSET));
        break;

    case bb_XGMAC_TX_OK_BRDCAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_BC_PKTS_OFFSET));
        break;

    case bb_XGMAC_TX_64_OCT_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_64_OFFSET));
        break;

    case bb_XGMAC_TX_65_TO_127_OCT_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_65TO127_OFFSET));
        break;

    case bb_XGMAC_TX_128_TO_255_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_128TO255_OFFSET));
        break;

    case bb_XGMAC_TX_256_TO_511_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_256TO511_OFFSET));
        break;

    case bb_XGMAC_TX_OK_512_TO_1023_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_512TO1023_OFFSET));
        break;

    case bb_XGMAC_TX_OK_1024_TO_15XX_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_1024TO1518_OFFSET));
        break;

    case bb_XGMAC_TX_OK_15XX_TO_MAX_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_1519TOMAX_OFFSET));
        break;

    case bb_XGMAC_TX_DEFERRED_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_DEFERRED_OFFSET));
        break;

    case bb_XGMAC_TX_TOTAL_COLLISIONS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_TOTAL_COL_OFFSET));
        break;

    case bb_XGMAC_TX_SINGLE_COLLISIONS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_SINGLE_COL_OFFSET));
        break;

    case bb_XGMAC_TX_MULTIPLE_COLLISIONS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_MULT_COL_OFFSET));
        break;

    case bb_XGMAC_TX_LATE_COLLISIONS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_LATE_COL_OFFSET));
        break;

    case bb_XGMAC_TX_EXCESSIVE_COLLISIONS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_COL_ERR_OFFSET));
        break;

    case bb_XGMAC_TX_EXCESSIVE_DEFERRAL_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_DEF_ERR_OFFSET));
        break;

    case bb_XGMAC_TX_EXCESSIVE_LENGTH_DROP_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_LENGTH_DROP_OFFSET));
        break;

    case bb_XGMAC_TX_UNDERRUN_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_UNDERRUN_OFFSET));
        break;

    case bb_XGMAC_TX_TAGGED_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_TAGGED_OFFSET));
        break;

    case bb_XGMAC_TX_CRC_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_CRC_ERR_OFFSET));
        break;

    case bb_XGMAC_TX_PAUSE_CTRL_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PAUSE_CTRL_OFFSET));
        break;

    case bb_XGMAC_TX_FLOW_CTL_COLLISIONS_SEND_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_FLOW_CTRL_COL_OFFSET));
        break;

    case bb_XGMAC_RX_OK_OCTS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_OCT_OK_OFFSET));
        break;

    case bb_XGMAC_RX_BAD_OCTS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_OCT_BAD_OFFSET));
        break;

    case bb_XGMAC_RX_UNICAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_UC_PKTS_OFFSET));
        break;

    case bb_XGMAC_RX_OK_MLTCAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_MC_PKTS_OFFSET));
        break;

    case bb_XGMAC_RX_OK_BRDCAST_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_BC_PKTS_OFFSET));
        break;

    case bb_XGMAC_RX_64_OCT_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_64_OFFSET));
        break;

    case bb_XGMAC_RX_65_TO_127_OCT_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_65TO127_OFFSET));
        break;

    case bb_XGMAC_RX_128_TO_255_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_128TO255_OFFSET));
        break;

    case bb_XGMAC_RX_256_TO_511_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_256TO511_OFFSET));
        break;

    case bb_XGMAC_RX_OK_512_TO_1023_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_512TO1023_OFFSET));
        break;

    case bb_XGMAC_RX_OK_1024_TO_15XX_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_1024TO1518_OFFSET));
        break;

    case bb_XGMAC_RX_OK_15XX_TO_MAX_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_1519TOMAX_OFFSET));
        break;

    case bb_XGMAC_RX_FRM_CHK_SEQ_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_FCS_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_TAGGED_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_TAGGED_OFFSET));
        break;

    case bb_XGMAC_RX_DATA_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_DATA_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_ALIGN_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_ALIGN_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_LONG_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_LONG_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_ETHER_STATS_JABBERS_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_JABBER_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_PAUSE_CTRL_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PAUSE_CTRL_OFFSET));
        break;

    case bb_XGMAC_RX_UNKNOWN_CTL_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_UNKNOWN_CTRL_OFFSET));
        break;

    case bb_XGMAC_RX_VERYLONG_ERR_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_VERYLONG_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_RUNT_ERR_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_RUNT_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_SHORT_ERR_FRM_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SHORT_ERR_OFFSET));
        break;

    case bb_XGMAC_RX_CARRIER_EXT_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_CAR_EXT_OFFSET));
        break;

    case bb_XGMAC_RX_SEQUENCE_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SEQ_OFFSET));
        break;

    case bb_XGMAC_RX_SYMBOL_ERR_CNT:
		*(bb_RegDataValue_type *) ptCounters = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SYMBOL_OFFSET));
        break;

    case bb_XGMAC_COUNTERS:
        ptCounterData = (ixfApi_XgmacCounters_t *) ptCounters;

		ptCounterData->OctetsTxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_OCT_OK_OFFSET));

		ptCounterData->OctetsTxBad = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_OCT_BAD_OFFSET));

		ptCounterData->UnicastFramesTxOK = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_UC_PKTS_OFFSET));

		ptCounterData->MulticastFramesTxOK = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_MC_PKTS_OFFSET));

		ptCounterData->BroardcastFramesTxOK = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_BC_PKTS_OFFSET));

		ptCounterData->Counter64OctetTxFrames = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_64_OFFSET));

		ptCounterData->Counter65to127TxFrames = HAL_IXF1110_READ(BaseAddress + 
    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_65TO127_OFFSET));

		ptCounterData->Counter128to255TxFrames = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_128TO255_OFFSET));

		ptCounterData->Counter256to511TxFrames = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_256TO511_OFFSET));

		ptCounterData->Counter512to1023TxOK = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_512TO1023_OFFSET));

		ptCounterData->Counter1024to15xxTxOK = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_1024TO1518_OFFSET));

		ptCounterData->Counter15xxtoMaxTxOK = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PKTS_1519TOMAX_OFFSET));

		ptCounterData->DeferredTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_DEFERRED_OFFSET));

		ptCounterData->TotalCollisionsTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_TOTAL_COL_OFFSET));

		ptCounterData->SingleCollisionsTx = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_SINGLE_COL_OFFSET));

		ptCounterData->MultiCollisionsTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_MULT_COL_OFFSET));

		ptCounterData->LateCollisionsTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_LATE_COL_OFFSET));

		ptCounterData->ExcessCollisionsTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_COL_ERR_OFFSET));

		ptCounterData->ExcessDeferralTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_DEF_ERR_OFFSET));

		ptCounterData->ExcessLengthDropTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_EXCESS_LENGTH_DROP_OFFSET));

		ptCounterData->UnderrunTx = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_UNDERRUN_OFFSET));

		ptCounterData->TaggedTx = HAL_IXF1110_READ(BaseAddress + 
	    	ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_TAGGED_OFFSET));

		ptCounterData->CRCErrTx = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_CRC_ERR_OFFSET));

		ptCounterData->PAUSECtrlFramesTx = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_PAUSE_CTRL_OFFSET));

		ptCounterData->FlowCtlCollisionsSendTx = HAL_IXF1110_READ(BaseAddress + 
    		ixf1110_XGMAC_CH_REG(channel, ixf1110_TX_FLOW_CTRL_COL_OFFSET));

		ptCounterData->OctetsRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_OCT_OK_OFFSET));

		ptCounterData->OctetsRxBad = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_OCT_BAD_OFFSET));

		ptCounterData->UnicastFramesRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_UC_PKTS_OFFSET));

		ptCounterData->MulticastFramesRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_MC_PKTS_OFFSET));

		ptCounterData->BroardcastFramesRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_BC_PKTS_OFFSET));

		ptCounterData->Counter64OctetRxFrames = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_64_OFFSET));

		ptCounterData->Counter65to127RxFrames = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_65TO127_OFFSET));

		ptCounterData->Counter128to255RxFrames = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_128TO255_OFFSET));

		ptCounterData->Counter256to511RxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_256TO511_OFFSET));

		ptCounterData->Counter512to1023RxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_512TO1023_OFFSET));

		ptCounterData->Counter1024to15xxRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_1024TO1518_OFFSET));

		ptCounterData->Counter15xxtoMaxRxOK = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PKTS_1519TOMAX_OFFSET));

		ptCounterData->FCSErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_FCS_ERR_OFFSET));

		ptCounterData->TaggedRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_TAGGED_OFFSET));

		ptCounterData->DataErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_DATA_ERR_OFFSET));

		ptCounterData->AlignErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_ALIGN_ERR_OFFSET));

		ptCounterData->LongErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_LONG_ERR_OFFSET));

		ptCounterData->JabbersErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_JABBER_ERR_OFFSET));

		ptCounterData->PAUSEMACCtrlFramesRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_PAUSE_CTRL_OFFSET));

		ptCounterData->UnknownCtlCntRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_UNKNOWN_CTRL_OFFSET));

		ptCounterData->VeryLongErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_VERYLONG_ERR_OFFSET));

		ptCounterData->RuntErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_RUNT_ERR_OFFSET));

		ptCounterData->ShortErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SHORT_ERR_OFFSET));

		ptCounterData->CarierExtErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_CAR_EXT_OFFSET));

		ptCounterData->SequenceErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SEQ_OFFSET));

		ptCounterData->SymbolErrRx = HAL_IXF1110_READ(BaseAddress + 
			ixf1110_XGMAC_CH_REG(channel, ixf1110_RX_SYMBOL_OFFSET));
        break;
        
    default:
        return bb_INV_SEL_COUNTERS;
    }
    
    return bb_NO_ERROR;    
}

/******************************************************************************
* Function : Ixf1110XgmacSetAddress
*------------------------------------------------------------------------------
* Description :  Function that sets th MAC address for a ixf1110 device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to set.
*           pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the address.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110XgmacSetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection,
                        IxfApi_MacAddress_t *pMacAddress)
{

    /* Validate ChipType */
    if (pChipData->ChipType != bb_1110_CHIP)
    {
        return (bb_INV_CHIP_TYPE);
    }
    
    /* STA_ADDR_LO */
    HAL_IXF1110_WRITE(pChipData->BaseAddress + 
    	ixf1110_XGMAC_CH_REG(pSection->group.channel, 
    	ixf1110_STA_ADDR_LO_OFFSET) , pMacAddress->ulLowValue);

    /*STA_ADDR_HI*/
    pMacAddress->ulHighValue &= (ixf1110_mSTA_ADDR_HI);
    HAL_IXF1110_WRITE(pChipData->BaseAddress + 
    	ixf1110_XGMAC_CH_REG(pSection->group.channel, 
    	ixf1110_STA_ADDR_HI_OFFSET) , pMacAddress->ulHighValue);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf1110XgmacGetAddress
*------------------------------------------------------------------------------
* Description : This function reads the Mac Address from the device.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Selects channel to get.
* Outputs : pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the MAC address.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1110XgmacGetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSection,
                        IxfApi_MacAddress_t *pMacAddress)
{
    /* Validate ChipType */
    if (pChipData->ChipType != bb_1110_CHIP)
    {
        return (bb_INV_CHIP_TYPE);
    }

    /* STA_ADDR_LO */
    pMacAddress->ulLowValue = HAL_IXF1110_READ(pChipData->BaseAddress + 
    	ixf1110_XGMAC_CH_REG(pSection->group.channel, 
    	ixf1110_STA_ADDR_LO_OFFSET));

    /*STA_ADDR_HI*/
    pMacAddress->ulHighValue = HAL_IXF1110_READ(pChipData->BaseAddress + 
    ixf1110_XGMAC_CH_REG(pSection->group.channel, ixf1110_STA_ADDR_HI_OFFSET));
    pMacAddress->ulHighValue &= (ixf1110_mSTA_ADDR_HI);
    
    return bb_NO_ERROR;
}
