/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf_api.h $
*   WORKFILE:       $Workfile: ixf_api.h $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose: Provides the API common interface to all IXF chips.
*         
* Notes:
*         
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf_api.h $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 2     10/07/03 2:02p Tepulkow
 * General cleanup for code review
 * 
* --------------------------------------------------------------------------
*/ 

#ifndef _IXF_FUNC_H_ /* Include file contents only once per compilation */
#define _IXF_FUNC_H_


/******************************************************************
* Function : IxfApiInit
*-----------------------------------------------------------------
* Description  : This routine initializes the main data structure
*               and returns a handle to that structure.
*-----------------------------------------------------------------
* Inputs  : baseAddress : Specifies the base address of the device
*          chipType    : Specifices the type of device
*          pChipData   : A pointer to an allocated structure.
* Outputs : pChipData  : A pointer to the chip data structure.
*                        This structure is passed to most API
*                        routines.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further initiali-
*          zation.
******************************************************************/
extern bb_Error_e 
IxfApiInit(void *baseAddress, bb_ChipType_e chipType,
           bb_ChipData_t *pChipData);

/******************************************************************
* Function : IxfApiInitChip
*-----------------------------------------------------------------
* Description  : This routine commits the internal register table
*                down to the chip and initializes the bb_ChipData_t
*                structure.
*-----------------------------------------------------------------
* Inputs  : pChipData  : A pointer to an allocated structure.
*           pTable
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further initiali-
*          zation.
******************************************************************/
extern bb_Error_e 
IxfApiInitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);

/******************************************************************
* Function : IxfApiAllocDataStructureMem
*-----------------------------------------------------------------
* Description  : This routine dynamically allocates memory within
*                the "pChipData" parameter for those members that 
*                need dynamic allocation.
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure without 
*                       pointers memory allocation.
* Outputs : pChipData : A pointer to the chip data structure with 
*                       pointer memory allocation.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiInit for the completion of the initialization
******************************************************************/
extern bb_Error_e 
IxfApiAllocDataStructureMem(bb_ChipData_t *pChipData);

/******************************************************************
* Function : IxfApiDeAllocMemory
*-----------------------------------------------------------------
* Description  : This routine frees the dynamically allocated memory
*                within the "pChipData" structure
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiDeAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************
* Function : IxfApiSetCfg
*-----------------------------------------------------------------
* Description  : This routine is used to configure the chip.
*                Many different types of configurations could be
*                performed with the combination of SelCfg and the 
*                generic pointer of the ptCfg.
*-----------------------------------------------------------------
* Inputs  : pChipData : pointer to the chip information.
*           ptSegment : pointer to chip segment structure
*           SelCfg : enum value defining what configuration
*                    needs to be set
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e
IxfApiSetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
             bb_SelConfig_e  SelCfg);

/******************************************************************
* Function : IxfApiGetCfg
*-----------------------------------------------------------------
* Description  : This routine is used to get the configuration of 
*                the chip. Many different types of configurations
*                could be performed with the combination of SelCfg
*                and the sgeneric pointer of the ptCfg.
*-----------------------------------------------------------------
* Inputs  : pChipData : pointer to the chip information.
*           ptSegment : pointer to chip segment structure
*           SelCfg : enum value defining what configuration
*                    needs to be retrieved
* Outputs : None
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e
IxfApiGetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
             bb_SelConfig_e SelCfg);

 /******************************************************************
 * Function : IxfApiGetStatus
 *-----------------------------------------------------------------
 * Description : Gets the Status.
 *-----------------------------------------------------------------
 * Inputs  : pChipData : Initialized Chip Data
 *           section   : Determines the section(block) to get the
 *                       status from.
 *           selStatus : Type of Status: 
 * Outputs : pStatus   : Place to put Status.
 *-----------------------------------------------------------------
 * Returns : bb_Error_e  Returns Error Condition if Not successful
 *-----------------------------------------------------------------
 * Access Globals : None.
 *-----------------------------------------------------------------
 * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGetStatus(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_SelStatus_e selStatus, void *pStatus);

/******************************************************************
* Function : IxfApiGetOpMode
*-----------------------------------------------------------------
* Description  : Gets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
* Outputs : pModeCfg  : Place to put mode.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e *opMode, void *pModeCfg); 

/******************************************************************
* Function : IxfApiSetOpMode
*-----------------------------------------------------------------
* Description  : Sets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
*           pModeCfg  : place to put mode.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e opMode, void *pModeCfg);

/******************************************************************
* Function : IxfApiReset
*-----------------------------------------------------------------
* Description  : Reset (chip or a section of the chip) and then 
*                reconfigures it. To restart traffic, the chip or 
*                section of chip must be set back online
*-----------------------------------------------------------------
* Inputs  : pChipData  : Initialized Chip Data
*           section    : Determines the section(block) to reset
*           resetType  : Specifies type of Reset
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e : Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiReset(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
            bb_SelResetType_e resetType);


/******************************************************************
* Function : IxfApiInitAlarmCallback
*-----------------------------------------------------------------
* Description : Initialize the Driver
*-----------------------------------------------------------------
* Inputs  : pChipData      : Validated Chip Data
*           pAlarmCallback : Alarm Callback
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiInitAlarmCallback(bb_ChipData_t *pChipData, AlarmCallBack pAlarmCallbackArg);

/******************************************************************
* Function : IxfApiChipIsr
*-----------------------------------------------------------------
* Description  : Handle Interupts for Chip
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern void
IxfApiChipIsr(bb_ChipData_t *pChipData);

/******************************************************************
* Function : IxfApiGetChipInfo
*-----------------------------------------------------------------
* Description  : Get the chip information.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : pChipInfo : Place to return chip info
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiGetChipInfo(bb_ChipData_t *pChipData, bb_ChipInfo_t *pChipInfo);

/******************************************************************
* Function : IxfApiCfgTest
*-----------------------------------------------------------------
* Description  : Run Tests
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section : Determines the section(block) to run the
*                     test for.
*           testType : The type of test to run.
*           pTestCfg : Contains the test configuration data.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
extern bb_Error_e
IxfApiCfgTest(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
              bb_TestType_e testType, void *pTestCfg);

              /******************************************************************
              * Function : IxfApiGetCounters
              *-----------------------------------------------------------------
              * Description  : Retrieve a set of Counters
              *-----------------------------------------------------------------
              * Inputs  : pChipData   : Validated Chip Data
              *           section     : Determines the section(block) to get the
              *                         counters from.
              *           SelCounters : Selects the set of counters
              * Outputs : pCounters   : Place to put Counters
              *-----------------------------------------------------------------
              * Returns : bb_Error_e  : Returns Error Condition if Not successful
              *-----------------------------------------------------------------
              * Access Globals : None.
              *-----------------------------------------------------------------
              * Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiGetCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *section, 
                  bb_SelCounters_e SelCounters, void *pCounters);

                  /******************************************************************
                  * Function : IxfApiClearCounters
                  *-----------------------------------------------------------------
                  * Description  : Clear a set of Counters, or all Counters.
                  *-----------------------------------------------------------------
                  * Inputs  : pChipData   : Validated Chip Data
                  *           section     : Determines the section(block) to clear the
                  *                         counters from.
                  *           SelCounters : Selects the set of counters
                  * Outputs : pCounters   : Not used. Should be NULL.
                  *-----------------------------------------------------------------
                  * Returns : bb_NO_ERROR, bb_FN_NOT_SUPPORTED
                  *-----------------------------------------------------------------
                  * Access Globals : None.
                  *-----------------------------------------------------------------
                  * Comments : 
******************************************************************/
extern bb_Error_e 
IxfApiClearCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                    bb_SelCounters_e SelCounters, void *pCounters);

                    /******************************************************************
                    * Function : IxfApiClearAlarms
                    *-----------------------------------------------------------------
                    * Description  : Clear all alarms that are enabled for a chip.
                    *-----------------------------------------------------------------
                    * Inputs  : pChipData : Validated Chip Data
                    * Outputs : None.
                    *-----------------------------------------------------------------
                    * Returns : bb_NO_ERROR, bb_FN_NOT_SUPPORTED
                    *-----------------------------------------------------------------
                    * Access Globals : None.
                    *-----------------------------------------------------------------
                    * Comments:   Interrupts are Not reported through a Callback function.
******************************************************************/
bb_Error_e
IxfApiClearAlarms(bb_ChipData_t *pChipData);

/******************************************************************
* Function : IxfApiSetChipOnline
*-----------------------------------------------------------------
* Description  : Puts some or all of the chip online.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*          section    : Specifies which portion of the chip to 
*                       put online.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e:  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None. 
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetChipOnline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section);

/******************************************************************
* Function : IxfApiSetChipOffline
*-----------------------------------------------------------------
* Description  : Takes some or all of the chip offline.
*-----------------------------------------------------------------
* Inputs  : pChipData  : Validated Chip Data
*          section     : Specifies which portion of the chip to 
*                        take offline.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section);

/******************************************************************
* Function : IxfApiSetAlarmCfg
*-----------------------------------------------------------------
* Description  : Edits Interrupt Enabled Masks for a Specific Set
*                of Interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*          section    : Determines the section(block) of the alarm(s)
*                       to edit.
*          AlarmType  : Specifies the type of Alarm
*          pAlarmCfg  : A pointer to the alarm structure.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiSetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_AlarmType_e AlarmType, void *pAlarmCfg);

                  /******************************************************************
                  * Function : IxfApiGetAlarmCfg
                  *-----------------------------------------------------------------
                  * Description  : Gets Interrupt Enabled Masks for a Specific Set
                  *                of Interrupts.
                  *-----------------------------------------------------------------
                  * Inputs  : pChipData : Validated Chip Data
                  *          section    : Determines the section(block) of the alarm(s)
                  *                       to edit.
                  *          AlarmType  : Specifies the type of Alarm
                  *          pAlarmCfg  : A pointer to the alarm structure.
                  * Outputs : None.
                  *-----------------------------------------------------------------
                  * Returns : bb_Error_e  Returns Error Condition if Not successful
                  *-----------------------------------------------------------------
                  * Access Globals : None.
                  *-----------------------------------------------------------------
                  * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_AlarmType_e AlarmType, void *pAlarmCfg);


                  /******************************************************************
                  * Function : IxfApiGetBuildVersion
                  *-----------------------------------------------------------------
                  * Description  : This routine returns the firmware build version of 
                  *                given the particular software driver identified.
                  *-----------------------------------------------------------------
                  * Inputs  : pChipData : Validated Chip Data
                  *           swType    : Identifies the upper or lower layer driver.
                  * Outputs : drvName   : Null terminated string describing the driver
                  *           date      : The build date of the driver.
                  *           buildVer  : The version number of the driver.
                  *           buildRev  : The revision number of the driver.
                  *-----------------------------------------------------------------
                  * Returns : bb_Error_e  Returns Error Condition if Not successful
                  *-----------------------------------------------------------------
                  * Access Globals : None.
                  *-----------------------------------------------------------------
                  * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGetBuildVersion(bb_ChipData_t *pChipData, bb_SoftwareVersion_e swType,
                      char *drvName, char *date, ushort *buildVer, ushort *buildRev);


                      /******************************************************************
                      * Function : IxfApiGenericRead
                      *-----------------------------------------------------------------
                      * Description  : This routine will read starting at a particular 
                      *                location and read for N number of words.
                      *-----------------------------------------------------------------
                      * Inputs  : pChipData  : Validated Chip Data
                      *          wordSize    : Identifies the size of a word.
                      *          address     : Location where to start reading from.
                      *          length      : Number of words to read.
                      * Outputs : buffer     : The buffer to contain the read data.
                      *-----------------------------------------------------------------
                      * Returns : bb_Error_e  Returns Error Condition if Not successful
                      *-----------------------------------------------------------------
                      * Access Globals : None.
                      *-----------------------------------------------------------------
                      * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGenericRead(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize,
                  ulong address, ushort length, void *buffer);                        

                  /******************************************************************
                  * Function : IxfApiGenericWrite
                  *-----------------------------------------------------------------
                  * Description  : This routine will write starting at a particular 
                  *                      location and write for N number of words.
                  *-----------------------------------------------------------------
                  * Inputs  : pChipData  : Validated Chip Data
                  *          wordSize    : Identifies the size of a word.
                  *          address     : Location where to start writing to.
                  *          length      : Number of words to write.
                  *          buffer      : The buffer to contain the read data.
                  * Outputs : None.
                  *-----------------------------------------------------------------
                  * Returns : bb_Error_e  Returns Error Condition if Not successful
                  *-----------------------------------------------------------------
                  * Access Globals : None.
                  *-----------------------------------------------------------------
                  * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiGenericWrite(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize,
                   ulong address, ushort length, void *buffer);                        

                   /******************************************************************
                   * Function : IxfApiIndirectRead
                   *-----------------------------------------------------------------
                   * Description  : This routine will retrieve the contents of a 
                   *                particular register.  The register is accessed in
                   *               an indirect means.
                   *-----------------------------------------------------------------
                   * Inputs  : pChipData : Initialized Chip Data
                   *           pSegment  : Specifies the location of the register.
                   *           ulRegOffset  : Specifies the register offset of to which
                   *                         to read from
                   * Outputs : buffer    : Location where words are read intoNone.
                   *-----------------------------------------------------------------
                   * Returns : bb_Error_e  Returns Error Condition if Not successful
                   *-----------------------------------------------------------------
                   * Access Globals : None.
                   *-----------------------------------------------------------------
                   * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiIndirectRead(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSegment, 
                   ulong ulRegOffset, void *buffer);

                   /******************************************************************
                   * Function : IxfApiIndirectWrite
                   *-----------------------------------------------------------------
                   * Description  : This routine will write the contents of "buffer" 
                   *                to a particular register.  The register is accessed 
                   *               in an indirect means.
                   *-----------------------------------------------------------------
                   * Inputs  : pChipData : Initialized Chip Data
                   *           pSegment  : Specifies the location of the register.
                   *           ulRegOffset : Specifies the register offset to which
                   *                         to write to.
                   *           buffer    : The data to be written.
                   * Outputs : None.
                   *-----------------------------------------------------------------
                   * Returns : bb_Error_e  Returns Error Condition if Not successful
                   *-----------------------------------------------------------------
                   * Access Globals : None.
                   *-----------------------------------------------------------------
                   * Comments : None.
******************************************************************/
extern bb_Error_e 
IxfApiIndirectWrite(bb_ChipData_t *pChipData, bb_ChipSegment_t *pSegment, 
                    ulong ulRegOffset, void *buffer);

#endif   /* _IXF_FUNC_H_ */
