/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf_api_ether.c $
*   WORKFILE:       $Workfile: ixf_api_ether.c $
*
* --------------------------------------------------------------------------
* Project: IXF API
*
* Purpose: XGMAC portion of the IXF API
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/ixf_api_ether.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 3     10/07/03 2:02p Tepulkow
 * General cleanup for code review
 * 
* 
* --------------------------------------------------------------------------
*/

#ifdef VXWORKS
#include "vxWorks.h"
#elif defined(__KERNEL__)
#include <linux/config.h>
#include <asm/memory.h>
#else 
#include "memory.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "builds.h"
#include "devlib.h"
#include "ixf_api_ether_d.h"
#include "ixf_api_ether.h"


/****************************************************************************
* Function : IxfApiGetMacAddress
*----------------------------------------------------------------------------
* Description : This function returns the MAC (Media Access Control)address 
*               for the selected chip. This is a 64 bits address 
*----------------------------------------------------------------------------
* Inputs : pChipData : pointer to chip data structure.
*          section : pointer to chip segment structure. Not used, only for 
*                    possible modifications. 
*          
* Outputs : address : Data structure that will hold the MAC address.
*----------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE : Invalid Chip it doesn't have the MAC Functionality.
*           bb_NO_ERROR: Successful completion
*----------------------------------------------------------------------------
* Access Globals : None
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/

bb_Error_e IxfApiGetMacAddress(bb_ChipData_t *pChipData,
                               bb_ChipSegment_t *section,
                               IxfApi_MacAddress_t *address)
{
    XgmacFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr;        
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->GetMacAddressPtr)
        {   
            /* Call Function Pointer */
            rc = ((pChipFuncPtrs->GetMacAddressPtr)(pChipData, section,address));
        }
        else
        {
            rc =  bb_INV_CHIP_TYPE;
        }
    }
    
    return rc;
}


/****************************************************************************
* Function : IxfApiSetMacAddress
*----------------------------------------------------------------------------
* Description : This function returns the MAC (Media Access Control)address 
*               for the selected chip. This is a 64 bits address 
*----------------------------------------------------------------------------
* Inputs : pChipData : pointer to chip data structure.
*          section : pointer to chip segment structure. Not used, only for 
*                    possible modifications. 
*          address : Data structure that will hold the MAC address to be set
*                    in the chip.
*          
* Outputs : 
*----------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE : Invalid Chip it doesn't have the MAC Functionality.
*           bb_NO_ERROR: Successful completion
*----------------------------------------------------------------------------
* Access Globals : None
*----------------------------------------------------------------------------
* Comments : 
*****************************************************************************/
bb_Error_e IxfApiSetMacAddress(bb_ChipData_t * pChipData,
                               bb_ChipSegment_t *section,
                               IxfApi_MacAddress_t *address)
{
    XgmacFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tXgmacFuncPtr;        
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->SetMacAddressPtr)
        {   
            /* Call Function Pointer */
            rc = ((pChipFuncPtrs->SetMacAddressPtr)(pChipData, section,address));
        }
        else
        {
            rc =  bb_INV_CHIP_TYPE;
        }
    }
    
    return rc;
}

