/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/os_linux.c $
*   WORKFILE:       $Workfile: os_linux.c $
*
* --------------------------------------------------------------------------
* Project: Angel Island
*
* Purpose: Provides the implementation to the linux support routines.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/os_linux.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 2     10/07/03 2:02p Tepulkow
 * General cleanup for code review
 * 
* --------------------------------------------------------------------------
*/

#ifdef IXF_LINUX

#include<stdlib.h>
#include<stdio.h>
#include<unistd.h>
#include<semaphore.h>
#include"os_linux.h"

/**************************************************************/
/* Example Linux support that requires special processing to  */
/* perform some function found in the ICG OS Abstraction      */
/* Layer.                                                     */
/**************************************************************/

sem_t *
LinuxSemCreate ()
{
    sem_t *s;
    
    s = malloc (sizeof (sem_t));
    sem_init ((sem_t *) s, 0, 0);
    return (sem_t *) s;
}                               /* sem_t *LinuxSemCreate() */

int
LinuxSemTake (sem_t * id, int timeout)
{
    int interval, counter;
    
    if (sem_trywait (id) == 0)
    {
        /* Semaphore was ready to be taken */
        printf ("Semaphore acquired immediately!\n");
        return 0;
    }                           /* if */
    
    /* At this point, the semaphore was blocked last time we tried. */
    /* So we will try again 4 times, each after waiting one fourth  */
    /* of the time indicated in the timeout. Other schemes are      */
    /* possible such as an exponential wait (1,2,4, etc. ms until   */
    /* the indicated time) and may be worth trying.                 */
    interval = 250 * timeout;
    
    for (counter = 0; counter < 4; counter++)
    {
        printf ("Semaphore not acquired - waiting %dms...", interval / 1000);
        usleep (interval);
        if (sem_trywait (id) == 0)
        {
            /* Semaphore was ready to be taken */
            printf ("got it!\n");
            return 0;
        }                       /* if */
        printf ("failed.\n");
    }                           /* for */
    
    printf ("Timeout expired - giving up!\n");
    return -1;
}

void
LinuxSemDestroy (sem_t * id)
{
    sem_destroy (id);
    free (id);
}                               /* LinuxSemDestroy() */

#endif 

