/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/osalsupport.c $
*   WORKFILE:       $Workfile: osalsupport.c $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose: Provides mapping of os.h macros to Intel OSAL functions.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/osalsupport.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 2     10/07/03 2:02p Tepulkow
 * General cleanup for code review
* --------------------------------------------------------------------------
*/

#include "osalsupport.h" 
#include <string.h>

/******************************************************************
* Function : IX_Int_Lock_VxWrks_Help
*-----------------------------------------------------------------
* Description  : This routine maps the Vxworks OS command for 
*                interrupt locking to the Intel OSAL version.
*-----------------------------------------------------------------
* Inputs  : None
* Outputs : Lock ID or -1 if error
*-----------------------------------------------------------------
* Returns :   IX_NO_ERROR       - Success
*             IX_UNKNOWN_ERROR  - Failure(failed to lock interrupts)   
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
IX_U32 IX_Int_Lock_VxWrks_Help()
{
    IX_U32 lockID;
    IX_Err_t rc;
    
    rc = IX_Int_Lock( &lockID );
    
    if( rc != IX_NO_ERROR )
    {
        return -1;
    }
    
    return lockID;
    
} /* IX_Int_Lock_VxWrks_Help */

  /******************************************************************
  * Function : IX_Thread_Spawn_LnxUsrSpc_Help
  *-----------------------------------------------------------------
  * Description  : This routine maps the Linux OS command for 
  *                task spawning to the Intel OSAL version.
  *-----------------------------------------------------------------
  * Inputs  : name        : Name of the task. 
  *           priority    : Task priority
  *           options     : Ignored
  *           stackSize   : Task stack size in bytes 
  *           entryPt     : Function to execute upon completion
  *                         of the spawn.
  *           args        : Arguments to the task function (entrPt)
  * Outputs : Thread ID or -1 if error.      
  *-----------------------------------------------------------------
  * Returns :   IX_NO_ERROR       - Success
  *             IX_OUT_OF_MEMORY  - Failure(out of memory)
  *             IX_BAD_PARAMS     - Failure(invalid parameter)
  *             IX_FAILED         - Failure(failed to allocate more sem)   
  *
  *-----------------------------------------------------------------
  * Access Globals : None.
  *-----------------------------------------------------------------
  * Comments : 
******************************************************************/
IX_Thread_id IX_Thread_Spawn_LnxUsrSpc_Help( char *name, 
                                            int priority, 
                                            int options, 
                                            int stackSize, 
                                            void *entryPt, 
                                            void *args ) 
{
    
    IX_Thread_Info_t tData;
    IX_Function_Ptr_t fPtr;
    IX_Thread_id tID;
    IX_Err_t err;
    
    strcpy(tData.thread_name, name);
    tData.priority = priority;
    tData.arg = (void *)args;
    tData.stackSizeInBytes = stackSize;
    
    fPtr = entryPt;
    
    err = IX_Thread_Spawn( fPtr, &tData, &tID );
    
    if( err != IX_NO_ERROR )
    {
        return -1;  
    }
    
    return tID;  
    
} /* IX_Thread_Spawn_LnxUsrSpc_Help */
