/* 
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/05/04 2:59p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:06p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/slowport.c $
*   WORKFILE:       $Workfile: slowport.c $
*
* --------------------------------------------------------------------------
* Project: ALL
*
* Purpose:  The purpose of this module is to provide any slow port configuation 
* 			required to allow reliable read/write access to the IXD2810 media
*			card.  If the slow prto configuration needs to be changed, the 
*			original configuration must be saved and restored around any 
*			modofications.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd2810/IXF/slowport.c $
 * 
 * 1     3/05/04 2:59p Tepulkow
 * 
 * 1     3/05/04 2:58p Tepulkow
 * 
 * 1     3/05/04 2:54p Tepulkow
 * 
 * 6     10/21/03 10:30a Tepulkow
 * Added a description to the "Purpose" field in the module header.
 * 
 * 5     10/10/03 4:32p Tepulkow
 * Removed printk statements used for debug
 * 
 * 4     10/10/03 2:55p Tepulkow
 * added code to implement work around for the unreliable reads from
 * slowport on the ixd2810 media card
* --------------------------------------------------------------------------
*/ 

#include "common_types.h"
#include "common_def.h"
#include <asm/hardware.h>
#include <asm/io.h>

/* Local defines */
static ulong temp_ccr;

/*******************************************************************************
 * sp_config - 
 *
 * RETURNS: void
 */
void 
sp_config(void)
{
    ulong data;
    volatile ulong *ptr = (ulong *) IXP2000_SLOWPORT_CCR;
    temp_ccr = *ptr;
    data = ((temp_ccr & ~0xF)|2);	/*force divide by 4 for slow port clk*/
    *ptr = data;
}

/*******************************************************************************
 * restore_sp_config - 
 *
 * RETURNS: void
 */
void 
restore_sp_config(void)
{
    volatile ulong *ptr = (ulong *) IXP2000_SLOWPORT_CCR;
    *ptr = temp_ccr;
}

/*******************************************************************************
 * SlowPortRead - 
 *
 * RETURNS: data from CSR read
 */
bb_RegDataValue_type                                                                          
SlowPortRead(volatile bb_RegPointer_type addr)
{
    bb_RegDataValue_type val;
    sp_config();
    val = le32_to_cpu(readl(addr));
    restore_sp_config();
    return val;
}

/*******************************************************************************
 * SlowPortWrite - 
 *
 * RETURNS: void
 */
void
SlowPortWrite(volatile bb_RegPointer_type addr, bb_RegDataValue_type val)
{
	writel(cpu_to_le32(val),addr);
}

