/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100.h $
 *   WORKFILE:       $Workfile: ixf18100.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains function prototypes for device related
 *          routines
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 1     6/27/03 4:42p Daly
 * 
 * 27    2/20/03 3:19p Kssaepha
 * Changed ifndef logic to ifdef to allow a combination of ixf1810x's to
 * be defined
 * 
 * 26    1/10/03 2:30p Pkjain2
 * 
 * 22    5/14/02 4:41p Machambe
 * Added 2nd param to InitChip
 * 
 * 21    4/18/02 4:53p Jaalliso
 * Fixing parameter declaration for GetOpMode
 * 
 * 20    2/07/02 5:10p Skadrix
 * Added a section (ChipSegment) parameter to the Ixf18100CfgTest function
 * to keep it consistent with the IxtApiCfgTest function prototype.
 * 
 * 19    2/06/02 3:46p Srodrig1
 * 
 * 18    1/30/02 5:13p Srodrig1
 * 
 * 17    1/24/02 1:58p Srodrig1
 * 
 * 16    1/24/02 12:27p Srodrig1
 * 
 * 15    1/24/02 12:14p Srodrig1
 * 
 * 14    1/24/02 11:55a Rsood
 * 
 * 13    1/14/02 10:35p Srodrig1
 * 
 * 12    12/05/01 3:26p Pkjain2
 * 
 * 11    12/04/01 4:23p Pkjain2
 * 
 * 10    11/29/01 4:46p Srodrig1
 * 
 * 9     11/29/01 9:31a Srodrig1
 * 
 * 8     11/20/01 2:48p Srodrig1
 * 
 * 7     11/19/01 11:20a Srodrig1
 * 
 * 6     11/08/01 3:26p Pkjain2
 * 
 * 5     10/23/01 3:42p Kssaepha
 * 
 * 4     8/08/01 3:50p Rfranz
 * added support for debugging using memory block instead of Calypso
 * device or simulator.  Moved device read/write macros/functions to
 * ixf18100_util.[ch]
 * 
 * 
 * 3     7/27/01 4:04p Rfranz
 * ixf18100_sonet.c compiles without errors or warnings
 * 
 * 2     7/27/01 2:11p Rfranz
 * Check in before first attempt to compile
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */
#ifndef __IXF18100_H__
#define __IXF18100_H__

/******************************************************************************
* Function : Ixf18100GetChipInfo
*------------------------------------------------------------------------------
* Description : Retrieves chip ID and version.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           pChipInfo : Pointer to Chip info structure where chip ID and
*                       version will be stored
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GetChipInfo (bb_ChipData_t *pChipData, bb_ChipInfo_t *pChipInfo);

/******************************************************************************
* Function : Ixf18100GetCounters
*------------------------------------------------------------------------------
* Description : Retrieves one/all counters (from one/all) blocks of 
*                the chip Ixf18100.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to retrieve counters
*           eCounter  : Specifies which counter
*           pCounters : buffer to write the counters.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_BLOCK_OPERATION
*           bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GetCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                    bb_SelCounters_e eCounter, void *pCounters);

/******************************************************************************
* Function : Ixf18100Reset
*------------------------------------------------------------------------------
* Description : Reset (chip or a section of the chip) and then 
*                reconfigure it.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to reset
*           resetType : Specifies type of Reset
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_RESET_TYPE, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Reset(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
              bb_SelResetType_e resetType);

/******************************************************************************
* Function : Ixf18100CfgTest
*------------------------------------------------------------------------------
* Description : Run Tests
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section   : Determines the section(block) to run the tests on.
*           testType  : The type of test to run.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100CfgTest(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                bb_TestType_e testType, void *pTestCfg);

/******************************************************************************
* Function : Ixf18100SetCfg
*------------------------------------------------------------------------------
* Description : Configures one/all block(s) of IXF1810x device
*------------------------------------------------------------------------------
* Inputs  : *pChipData : pointer to the chip information.
*           *pChipSeg   : pointer to chip segment structure
*           SelCfg      : enum value defining what config needs to be set
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg);

/******************************************************************************
* Function : Ixf18100GetCfg
*------------------------------------------------------------------------------
* Description : Reads the configuration of one/all block(s) of IXF1810x device
*------------------------------------------------------------------------------
* Inputs  : *pChipData : pointer to the chip information.
*           *pChipSeg   : pointer to chip segment structure
*           SelCfg      : enum value defining what configuration
*                         needs to be retrieved
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_PARAMETER, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
               bb_SelConfig_e  SelCfg);

/******************************************************************************
* Function : Ixf18100GetStatus
*------------------------------------------------------------------------------
* Description : Reads the status of one/all block(s) of IXF1810x device
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           selStatus : Type of Status: 
* Outputs : pStatus : place to put Status.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100GetStatus(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
                  bb_SelStatus_e selStatus, void *pStatus);

/******************************************************************************
* Function : Ixf18100GetBuildVersion
*------------------------------------------------------------------------------
* Description : Reads the build version for the IXF1810x device driver
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           selStatus : Type of Status: 
* Outputs : pStatus   : place to put Status.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100GetBuildVersion(bb_ChipData_t *pChipData, char *drvName, char *date,
                        ushort *buildVer, ushort *buildRev);

/******************************************************************************
* Function : Ixf18100Read
*------------------------------------------------------------------------------
* Description : Reads n number of one/two/four byte values.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           wordSize  : One/two/four byte word size
*           address   : Starting address to read from
*           length    : Number of words to read
*           buffer    : Location where words are read into
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100Read(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
             ushort length, void *buffer);

/******************************************************************************
* Function : Ixf18100Write
*------------------------------------------------------------------------------
* Description : Writes n number of one/two/four byte values to the device
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           wordSize  : One/two/four byte word size
*           address   : Starting address to write to
*           length    : Number of words to write
*           buffer    : Location where words are written from
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100Write(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize, ulong address,
              ushort length, void *buffer);

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
    defined(INCLUDE_18104_LIB)
/******************************************************************************
* Function : Ixf18100GetMacAddress
*------------------------------------------------------------------------------
* Description : Gets the Mac Address of the device.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
* Outputs : address   : Pointer to the Mac Address structure
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100GetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address);

/******************************************************************************
* Function : Ixf18100SetMacAddress
*------------------------------------------------------------------------------
* Description : Sets the Mac Address of the device.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
*           address   : Pointer to the Mac Address structure
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100SetMacAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                      void *address);
#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
          defined(INCLUDE_18104_LIB) */

/******************************************************************************
* Function : Ixf18100SetAlarmCfg
*------------------------------------------------------------------------------
* Description : Configures the interrupt masks
*------------------------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) of the chip
*           AlarmType : Type of Alarm 
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100SetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                    bb_AlarmType_e AlarmType, void *pAlarmCfg);

/******************************************************************************
* Function : Ixf18100InitAlarmCallback
*------------------------------------------------------------------------------
* Description : Initialize the alarm callback function with the IXF30007
*               device driver
*------------------------------------------------------------------------------
* Inputs  : pChipData      : Validated Chip Data
*           pAlarmCallback : Alarm Callback
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100InitAlarmCallback(bb_ChipData_t *pChipData,
                          AlarmCallBack pAlarmCallbackArg);

/******************************************************************************
* Function : Ixf18100ChipIsr
*------------------------------------------------------------------------------
* Description : The ISR services the interrupts that the user has selected to
*               monitor
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : 
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100ChipIsr(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf18100InitAllocMemory
*------------------------------------------------------------------------------
* Description : Allocate the memory space for IXF1810x device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100InitAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf18100DeAllocMemory
*------------------------------------------------------------------------------
* Description : Deallocate the memory space for IXF1810x device driver
*               configuration and alarm structures.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NO_CHIP_CFG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100DeAllocMemory(bb_ChipData_t *pChipData);

/******************************************************************************
* Function : Ixf18100InitChip
*------------------------------------------------------------------------------
* Description : Initialize the ixf1810x chip via OCM UI generated
*               data structure
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           pTable : Table to be committed to the device.
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);

/******************************************************************************
* Function : Ixf18100GetOpMode
*------------------------------------------------------------------------------
* Description : This function returns the operational mode that the chip is
*               configured in.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section: Chip Segment
*           pModeCfg: NULL
* Outputs : opMode: Operational Mode
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_STM_MODE_MISMATCH, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : We have only one enum value for GFP over FIBER Mode 
* and GFP  Client Map Mode. The reason being that the configuration
* value for this register in these two modes is identical.
******************************************************************************/
extern bb_Error_e
Ixf18100GetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_OperMode_e *opMode, void *pModeCfg);

/******************************************************************************
* Function : Ixf18100SetOpMode
*------------------------------------------------------------------------------
* Description : Configures the Chip Configuration structure to 
*               operate in a certain mode.
*------------------------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section: Chip Segment
*           opMode: Operational Mode
*           pModeCfg: NULL
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_OperMode_e opMode, void *pModeCfg);

#endif /* __IXF18100_H__ */
