/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_global.h $
 *   WORKFILE:       $Workfile: ixf18100_global.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains function prototypes for global block related
 *          routines
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_global.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 1     6/27/03 4:42p Daly
 * 
 * 13    2/20/03 3:19p Kssaepha
 * Changed ifndef logic to ifdef to allow a combination of ixf1810x's to
 * be defined
 * 
 * 12    1/30/03 4:41p Pkjain2
 * Made changes for PTQ release. Added Getstatus calls for SONET and
 * GLOBAL block.
 * 
 * 11    1/10/03 2:30p Pkjain2
 * 
 * 8     7/11/02 4:21p Pkjain2
 * Corrected some headers. Added Legal notice to some files. 
 * 
 * --------------------------------------------------------------------------
 */

#ifndef IXF18100_GLOBAL_H
#define IXF18100_GLOBAL_H

/******************************************************************************
* Function : Ixf18100EditGlbCnfg
*------------------------------------------------------------------------------
* Description : Global configuration functions, the mode combinations are 
*               mutually exclusive.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : Pointer to values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
extern bb_Error_e
Ixf18100EditGlbCnfg(bb_RegPointer_type BaseAddress, ixf18100_GlbCnfg_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GetGlbCnfg
*------------------------------------------------------------------------------
* Description : This function returns the mode configuration.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : The pointer of the structure to return values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetGlbCnfg(bb_RegPointer_type BaseAddress, ixf18100_GlbCnfg_t *pCnfg);

/******************************************************************************
* Function : Ixf18100EditGlbReset
*------------------------------------------------------------------------------
* Description : This function is used to reset One/All blocks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pCnfg : Values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100EditGlbReset(bb_RegPointer_type BaseAddress,
                     ixf18100_GlbReset_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GetGlbReset
*------------------------------------------------------------------------------
* Description : Gets reset configuration for all the blocks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer of the structure to store the value
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetGlbReset(bb_RegPointer_type BaseAddress,
                    ixf18100_GlbReset_t *pCnfg);

/******************************************************************************
* Function : Ixf18100HoldSwReset
*------------------------------------------------------------------------------
* Description : This function will set the Sw reset bit to 1 to reset all the
*               blocks, to release the reset use the counterpart function
*               Ixf18100ReleaseSwReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100HoldSwReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100ReleaseSwReset
*------------------------------------------------------------------------------
* Description : This function will set the SW reset bit to 0 to release the
*               reset on all the blocks.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100ReleaseSwReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100HoldApsReset
*------------------------------------------------------------------------------
* Description : This function will set the APS reset bit to 1 to reset the APS
*               block, to release the reset use  the counterpart function
*               Ixf18100ReleaseApsReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100HoldApsReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100ReleaseApsReset
*------------------------------------------------------------------------------
* Description : This function will set the APSreset bit to 0 to release the
*               reset on the APS block.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100ReleaseApsReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100HoldLnIntfReset
*------------------------------------------------------------------------------
* Description : This function will set the Line Intf reset bit to 1 to reset
*               the Line Interface. To release the reset use the counterpart
*               function Ixf18100ReleaseLnIntfReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100HoldLnIntfReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100ReleaseLnIntfReset
*------------------------------------------------------------------------------
* Description : This function will set the Line Intf reset bit to 0 to release
*               the reset on the Line Interface.
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100ReleaseLnIntfReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100HoldSpiReset
*------------------------------------------------------------------------------
* Description : This function will set the SPI reset bit to 1 to reset the SPI
*               block. To release the reset use  the counterpart function
*               Ixf18100ReleaseSpiReset
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100HoldSpiReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100ReleaseSpiReset
*------------------------------------------------------------------------------
* Description : This function will set the SPI reset bit to release the reset
*               on the SPI block. 
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100ReleaseSpiReset(bb_RegPointer_type BaseAddress);

/******************************************************************************
* Function : Ixf18100EditWatchDogCnfg
*------------------------------------------------------------------------------
* Description : This function will enable/disable the watchdog and will set
*               the watchdog timer
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the values to be set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100EditWatchDogCnfg(bb_RegPointer_type BaseAddress,
                         ixf18100_GlbWatchDogInfo_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GetWatchDogCnfg
*------------------------------------------------------------------------------
* Description : This function will get the watchdog configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetWatchDogCnfg(bb_RegPointer_type BaseAddress,
                        ixf18100_GlbWatchDogInfo_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GetWatchDogFailAdress
*------------------------------------------------------------------------------
* Description : This function will get the address for which the watchdog 
*               timer expired
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the structure to store failure address
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetWatchDogFailAdress(bb_RegPointer_type BaseAddress,
                              ixf18100_GlbWatchDogInfo_t *pCnfg);

/******************************************************************************
* Function : Ixf18100OifSfiGetGlblLnIntf
*------------------------------------------------------------------------------
* Description : This function gets the Line Intf configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100OifSfiGetGlblLnIntf(bb_RegPointer_type BaseAddress,
                            ixf18100_GlobalLineInterfaceCnfg_t *cnfg);

/******************************************************************************
* Function : Ixf18100OifSfiSetGlblLnIntf
*------------------------------------------------------------------------------
* Description : This function sets the Line Interface configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100OifSfiSetGlblLnIntf(bb_RegPointer_type BaseAddress,
                            ixf18100_GlobalLineInterfaceCnfg_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GetClkCnfg
*------------------------------------------------------------------------------
* Description : This function gets the clock configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e 
Ixf18100GetClkCnfg(bb_RegPointer_type BaseAddress,
                   ixf18100_GlbClkCnfg_t *cnfg);

/******************************************************************************
* Function : Ixf18100SetClkCfg
*------------------------------------------------------------------------------
* Description : This function sets the clock configuration for the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e 
Ixf18100SetClkCfg(bb_RegPointer_type BaseAddress, ixf18100_GlbClkCnfg_t *cnfg);

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
/******************************************************************************
* Function : Ixf18100SetMuxCfg
*------------------------------------------------------------------------------
* Description : This function sets configuration for various Muxes
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : MuxCfg : The pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100SetMuxCfg(bb_RegPointer_type BaseAddress, bb_ChipSegment_t *ptSegment,
                  bb_SelConfig_e  SelCfg, ixf18100_ApiMuxCfg_t *MuxCfg);

/******************************************************************************
* Function : Ixf18100GetMuxCfg
*------------------------------------------------------------------------------
* Description : This function gets configuration for various Muxes
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : MuxCfg : The pointer to the structure to store the values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetMuxCfg(bb_RegPointer_type BaseAddress, bb_ChipSegment_t *ptSegment,
                  bb_SelConfig_e SelCfg, ixf18100_ApiMuxCfg_t *MuxCfg);
#endif

/******************************************************************************
* Function : Ixf18100GlobalSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the global block of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the values to set int he device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GlobalSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf18100_ChipLevelCfg_t *ptGlobalCfg);

/******************************************************************************
* Function : Ixf18100GlobalGetCfg
*------------------------------------------------------------------------------
* Description : This function gets global block configuration of the device
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : ptGlobalCfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GlobalGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                     ixf18100_ChipLevelCfg_t *ptGlobalCfg);

/******************************************************************************
* Function : Ixf18100GetGlblAps
*------------------------------------------------------------------------------
* Description : This function gets the global APS configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the structure to store values
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100GetGlblAps(bb_RegPointer_type BaseAddress,
                   ixf18100_GlobalApsCfng_t *pCnfg);

/******************************************************************************
* Function : Ixf18100SetGlblAps
*------------------------------------------------------------------------------
* Description : This functions sets global APS configuration
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*         : pCnfg : Pointer to the values to set in the device
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
extern bb_Error_e
Ixf18100SetGlblAps(bb_RegPointer_type BaseAddress,
                   ixf18100_GlobalApsCfng_t *pCnfg);

/******************************************************************************
* Function : Ixf18100GlobalSetTestCfg
*------------------------------------------------------------------------------
* Description : The function sets the test configuration
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device
*           bb_TestType_e testType: selects the test type
*           void *ptTestCfg: pointer to the test configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GlobalSetTestCfg(bb_RegPointer_type BaseAddress,
                         bb_TestType_e testType, void *ptTestCfg);

/******************************************************************************
* Function : Ixf18100GlobalGetTestCfg
*------------------------------------------------------------------------------
* Description : The function gets the test configuration set in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           testType : The type of test to run.
*           void *pTestCfg: pointer to the test configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_TEST_CNFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GlobalGetTestCfg(bb_RegPointer_type BaseAddress,
                         bb_TestType_e testType, void *pTestCfg);

/******************************************************************************
* Function : Ixf18100GlobalGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           selStatus : Type of Status: 
*			pStatus: pointer to the status structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100GlobalGetStatus(bb_RegPointer_type BaseAddress,
                       bb_SelStatus_e selStatus, void* pStatus);

#endif
