/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_pos.h $
 *   WORKFILE:       $Workfile: ixf18100_pos.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains function prototypes for POS block related
 *          routines
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_pos.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:42p Daly
 * 
 * 8     1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 5     7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 4     12/05/01 3:27p Pkjain2
 * 
 * 3     12/04/01 4:24p Pkjain2
 * 
 * 2     11/16/01 11:22a Pkjain2
 * 
 * 1     11/10/01 11:25a Pkjain2
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */


#ifndef _IXF18100_POS_H__
#define _IXF18100_POS_H__

/******************************************************************************
* Function : Ixf18100PosRxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS Rx config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosRxCfg_t *ptPosRxCfg: pointer to the values that need
*               to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosRxSetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosRxCfg_t *ptPosRxCfg);

/******************************************************************************
* Function : Ixf18100PosTxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS Tx config structure and
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosTxCfg_t *ptPosTxCfg: pointer to the values that need
*               to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosTxSetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosTxCfg_t *ptPosTxCfg);

/******************************************************************************
* Function : Ixf18100PosRxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosRxCfg_t *ptPosRxCfg: pointer to the values that need
*               to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosRxGetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosRxCfg_t *ptPosRxCfg);

/******************************************************************************
* Function : Ixf18100PosTxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosTxCfg_t *ptPosTxCfg: pointer to the values that need
*               to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosTxGetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosTxCfg_t *ptPosTxCfg);

/******************************************************************************
* Function : Ixf18100PosRxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS RX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosRxGetCounters(bb_RegPointer_type BaseAddress,
                         bb_SelCounters_e SelCntr, void *ptCounters);

/******************************************************************************
* Function : Ixf18100PosTxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS TX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosTxGetCounters(bb_RegPointer_type BaseAddress,
                         bb_SelCounters_e SelCntr, void *ptCounters);

/******************************************************************************
* Function : Ixf18100PosSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be set
*           ixf18100_PosCfg_t *ptPosCfg: pointer to the values that need to 
*               be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e  SelCfg,
                  ixf18100_PosCfg_t *ptPosCfg);

/******************************************************************************
* Function : Ixf18100PosGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be fetched
*           ixf18100_PosCfg_t *ptPosCfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e  SelCfg,
                  ixf18100_PosCfg_t *ptPosCfg);

/******************************************************************************
* Function : Ixf18100PosGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptPosCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100PosGetCounters(bb_RegPointer_type BaseAddress,
                       bb_SelCounters_e SelCntr, void *ptPosCounters);

#endif /* _IXF18100_POS_H__ */
