/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_sonet.h $
 *   WORKFILE:       $Workfile: ixf18100_sonet.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains prototypes for SONET functions that are
 *  part of the OCD API.
 *          
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_sonet.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:42p Daly
 * 
 * 16    1/30/03 4:41p Pkjain2
 * Made changes for PTQ release. Added Getstatus calls for SONET and
 * GLOBAL block.
 * 
 * 15    1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 12    5/29/02 11:15a Jaalliso
 * Code format cleanup
 * 
 * 4     9/10/01 6:05p Srodrig1
 * adding API to get and set windows size for degraded and excessive
 * error.
 * 
 * 3     7/27/01 2:11p Rfranz
 * Check in before first attempt to compile
 * 
 * 2     7/24/01 1:22p Rfranz
 * WIP
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */


#ifndef __IXF18100_SONET_H__
#define __IXF18100_SONET_H__


/******************************************************************************
* Function : Ixf18100SonetGetOhBytes
*------------------------------------------------------------------------------
* Description : This function returns the requested overhead bytes. These may
*               be either the currently programmed transmit bytes, or the
*               received bytes.  Received bytes may be filtered depending
*               on the filter configuration.
*------------------------------------------------------------------------------
* Inputs : ptChipData: pointer to chip data structure.
*          ptSegment: pointer to chip segment structure.  Only the block.block
*                   and block.flow members are used.
*          eOhBytes: which overhead bytes to set
* Outputs : pOhBytes: pointer to structure containing overhead bytes. Note that
*           the caller must allocate space for this.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals : None
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetOhBytes(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                        bb_SelOhBytes_e eOhBytes, void *pOhBytes);

/******************************************************************************
* Function : Ixf18100SonetSetOhBytes
*------------------------------------------------------------------------------
* Description : This function sets SONET overhead bytes for the ixf18100
*               family of chips.  It sets both the outgoing transmitted bytes,
*               and the expected RX C2 byte. The expected C2 RX byte is the
*               only byte on the RX side that may be set. Note that trace
*               strings are set with the function ixf18100SonetSetTrace.
*------------------------------------------------------------------------------
* Inputs  : ptChipData: pointer to chip data structure.
*           ptSegment: pointer to chip segment structure. Only the block.block
*               and block.flow members are used.
*           eOhBytes: which overhead bytes to set
*           pOhBytes: pointer to structure containing overhead bytes.
* Outputs : return value only
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals : None
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetSetOhBytes(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                        bb_SelOhBytes_e eOhBytes, ushort pOhBytes);

/******************************************************************************
* Function : Ixf18100SonetGetCounters
*------------------------------------------------------------------------------
* Description : Reads and returns one/all SONET counters
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block and RX flow
*           bb_SelCounters_e eCounter :  selects which counter(s) are to
*               be read.
* Outputs : void *pCounterData:  The counter structure is returned in this
*               buffer.   This pointer is generic to allow for different
*               counter structures to be returned in the future.  Note that
*               Caller must allocate memory for htis buffer.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS, bb_INV_CHIP_TYPE, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetCounters(bb_ChipData_t *ptChipData,
                         bb_ChipSegment_t *ptSegment,
                         bb_SelCounters_e eCounter, void *pCounterData);

/******************************************************************************
* Function : Ixf18100SonetGetTrace
*------------------------------------------------------------------------------
* Description : Reads traces (J0 or J1 string) from the device
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block
*           bb_TraceType_e eTraceType  :  selects with trace to read.
*
* Outputs : pTrace - pointer to buffer containing trace string.
*           pLength - pointer to uint16 that contains the length of the
*               string in bytes, including trailing NUL
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_PARAMETER, bb_INV_TRACE_FORMAT, bb_INV_TRACE_TYPE
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetTrace(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                      bb_TraceType_e eTraceType, char *pTrace,
                      ushort *pLength);

/******************************************************************************
* Function : Ixf18100SonetSetTrace
*------------------------------------------------------------------------------
* Description : Inserts traces in the device
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block
*           bb_TraceType_e eTraceType : selects with trace to read.
*           bb_TraceFormat_e eTraceFormat : Format of the trace to be inserted
*           pTrace - pointer to buffer containing trace string.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_TRACE_FORMAT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments :  This function can be used to configure the RX section to ignore
*       an incoming trace, so it must be able to tolerate a NULL trace pointer
*       being given.  This is checked in the Ixf18100SonetProcessTrace
*       function where a NULL pointer is only allowed for the ignore trace
*       case.
******************************************************************************/
extern bb_Error_e
Ixf18100SonetSetTrace(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                      bb_TraceType_e eTraceType,
                      bb_TraceFormat_e eTraceFormat, char *pTrace);

/******************************************************************************
* Function : Ixf18100SonetGetWindowSize
*------------------------------------------------------------------------------
* Description : Reads the window size for the degraded and excessive error
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t *pChipData: Pointer to the data structure holding
*                                     the base address for the device                   
*           bb_ChipSegment_t *section: Pointer to the data structure to 
*                                      identify the section of the chip that 
*                                      will be modify.
*           bb_WindowSizeMode_e mode: Parameter to identify the kind of
*                                     error (degraded or excessive) if the 
*                                     the window is being cleared or set.
*   
* Outputs : ulong *value: Parameter to store the value of the window size.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : 
*------------------------------------------------------------------------------
* Comments : 
*   NOTE: preemption or interrupts must be disabled around this function for
*       correct results to be guarenteed.
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetWindowSize(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                           bb_WindowSizeMode_e mode, ulong *value);

/******************************************************************************
* Function : Ixf18100SonetSetWindowSize
*------------------------------------------------------------------------------
* Description : Set the window size for the degraded and excessive error
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t *pChipData: Pointer to the data structure holding
*                                     the base address for the device                   
*           bb_ChipSegment_t *section: Pointer to the data structure to 
*                                      identify the section of the chip that 
*                                      will be modify.
*           bb_WindowSizeMode_e mode: Parameter to identify the kind of
*                                     error (degraded or excessive) if the 
*                                     the window is being cleared or set.
*           ulong *value: Parameter to store the value of the window size.
* Outputs :  none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : 
*------------------------------------------------------------------------------
* Comments : 
*   NOTE: preemption or interrupts must be disabled around this function for
*       correct results to be guarenteed.
******************************************************************************/
extern bb_Error_e
Ixf18100SonetSetWindowSize(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                           bb_WindowSizeMode_e mode, ulong value);

/******************************************************************************
* Function : Ixf18100SonetGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 8 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: BaseAddress to the device
*           bb_SelConfig_e SelCfg: Selects the config to be read from device
* Outputs : ixf18100_SonetCfg_t ptSonetCfg: pointer to the Sonet cfg structure
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_SonetCfg_t *ptSonetCfg);

/******************************************************************************
* Function : Ixf18100SonetSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from Sonet config 
* structure and appropriately writes the values into the corresponding 
* registers.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: BaseAddress to the device
*           bb_SelConfig_e SelCfg: Selects the config to be read from device
*           ixf18100_SonetCfg_t ptSonetCfg: pointer to the Sonet cfg structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e 
Ixf18100SonetSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_SonetCfg_t *ptSonetCfg);

/******************************************************************************
* Function : Ixf18100SonetGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           section   : Determines the section(block) to get the status from.
*           selStatus : Type of Status: 
*			void *ptSonetStatus: pointer to the status
*				that needs to be read from the device
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_SEL_STATUS_TYPE,
*           bb_INV_CHI_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100SonetGetStatus(bb_RegPointer_type BaseAddress,
                       bb_ChipSegment_t *ptSegment, bb_SelStatus_e selStatus,
                       void* pStatus);

#define ixf18100_TRACE_TIMEOUT  100


#endif /* __IXF18100_SONET_H__ */
