/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_spi4.h $
 *   WORKFILE:       $Workfile: ixf18100_spi4.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains function prototypes for SPI4 block related
 *          routines
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_spi4.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:42p Daly
 * 
 * 9     1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 6     7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 5     12/05/01 3:27p Pkjain2
 * 
 * 4     12/04/01 4:24p Pkjain2
 * 
 * 3     11/16/01 11:22a Pkjain2
 * 
 * 2     11/10/01 11:26a Pkjain2
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */


#ifndef _IXF18100_SPI4_H__
#define _IXF18100_SPI4_H__

/******************************************************************************
* Function : Ixf18100Spi4GlbSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 Global cfg struct and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg: pointer to the values that
*               need to be set in the device
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GlbSetCfg(bb_RegPointer_type BaseAddress,
                      ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg);

/******************************************************************************
* Function : Ixf18100Spi4RxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 RX config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4RxCfg_t *ptSpi4RxCfg: pointer to the values that
*               need to be set in the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4RxSetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4RxCfg_t *ptSpi4RxCfg);

/******************************************************************************
* Function : Ixf18100Spi4TxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 TX config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxCfg_t *ptSpi4TxCfg: pointer to the values that
*               need to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4TxSetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4TxCfg_t *ptSpi4TxCfg);

/******************************************************************************
* Function : Ixf18100Spi4GlbGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GlbGetCfg(bb_RegPointer_type BaseAddress,
                      ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg);

/******************************************************************************
* Function : Ixf18100Spi4RxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4RxCfg_t *ptSpi4RxCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4RxGetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4RxCfg_t *ptSpi4RxCfg);

/******************************************************************************
* Function : Ixf18100Spi4TxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxCfg_t *ptSpi4TxCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4TxGetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4TxCfg_t *ptSpi4TxCfg);

/******************************************************************************
* Function : Ixf18100Spi4RxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the SPI4 RX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4RxGetCounters(bb_RegPointer_type BaseAddress,
                          bb_SelCounters_e SelCntr, void *ptSpi4Counters);

/******************************************************************************
* Function : Ixf18100Spi4TxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the SPI4 Tx counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4TxGetCounters(bb_RegPointer_type BaseAddress,
                          bb_SelCounters_e SelCntr, void *ptSpi4Counters);

/******************************************************************************
* Function : Ixf18100Spi4SetTxDeskewCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg: pointer to the values 
*           that need to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4SetTxDeskewCfg(bb_RegPointer_type BaseAddress,
                           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg);

/******************************************************************************
* Function : Ixf18100Spi4GetTxDeskewCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GetTxDeskewCfg(bb_RegPointer_type BaseAddress,
                           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg);

/******************************************************************************
* Function : Ixf18100Spi4SetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be set
*           ixf18100_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set in the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4SetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                   ixf18100_Spi4Cfg_t *ptSpi4Cfg);

/******************************************************************************
* Function : Ixf18100Spi4GetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be fetched
*           ixf18100_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                   ixf18100_Spi4Cfg_t *ptSpi4Cfg);

/******************************************************************************
* Function : Ixf18100Spi4SetTestCfg
*------------------------------------------------------------------------------
* Description : The function sets the test configuration to the chip..
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_TestType_e testType: selects the test type
*           void *ptSpi4TestCfg: pointer to the test 
*           configuration that needs to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4SetTestCfg(bb_RegPointer_type BaseAddress, bb_TestType_e testType,
                       void *ptTestCfg);

/******************************************************************************
* Function : Ixf18100Spi4GetTestCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           testType : The type of test to run.
*           void *pTestCfg: pointer to the test 
*               configuration that needs to be read from the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_TEST_CFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GetTestCfg(bb_RegPointer_type BaseAddress, bb_TestType_e testType,
                       void *pTestCfg);

/******************************************************************************
* Function : Ixf18100Spi4GetCounters
*------------------------------------------------------------------------------
* Description : The function reads all the SPI4 counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GetCounters(bb_RegPointer_type BaseAddress,
                        bb_SelCounters_e SelCntr, void *ptSpi4Counters);

/******************************************************************************
* Function : Ixf18100Spi4GetStatus
*------------------------------------------------------------------------------
* Description : The function reads all the SPI4 counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4Status_t *ptSpi4Status: pointer to the status 
*               structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100Spi4GetStatus(bb_RegPointer_type BaseAddress,
                      bb_SelStatus_e selStatus, void* pStatus);

#endif /* _IXF18100_SPI4_H__ */

