/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_xgmac.h $
 *   WORKFILE:       $Workfile: ixf18100_xgmac.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: 
 *          
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/include/ixf18100_xgmac.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 15    1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 12    7/11/02 4:22p Pkjain2
 * Corrected some headers. Added Legal notice to some files. 
 * 
 * 11    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 10    1/14/02 10:35p Srodrig1
 * 
 * 9     1/08/02 11:14a Srodrig1
 * 
 * 8     12/04/01 4:25p Pkjain2
 * 
 * 7     11/29/01 4:46p Srodrig1
 * 
 * 6     11/20/01 11:34a Srodrig1
 * 
 * 5     11/19/01 2:01p Srodrig1
 * 
 * 4     8/23/01 5:43p Wrbrackn
 * Test driver that tests the sets and gets now compiles cleanly.
 * 
 * 3     8/23/01 4:12p Wrbrackn
 * Made changes suggested after review of the code with Mark.
 * 
 *
 * 
 * --------------------------------------------------------------------------
 */
#ifndef __IXF18100_XGMAC_H__
#define __IXF18100_XGMAC_H__

/******************************************************************************
* Function : Ixf18100XgmacSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the XGMAC block in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf18100_XgmacCfg_t *ptXgmacCfg: A pointer a structure containing
*                                          chip configuration settings.
* Outputs : Writes to registers to configure XGMAC settings on the chip.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_XgmacCfg_t *ptXgmacCfg);

/******************************************************************************
* Function : Ixf18100XgmacGetCfg
*------------------------------------------------------------------------------
* Description : This function reads the XGMAC configuration settings into a 
*                data structure.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf18100_XgmacCfg_t *ptXgmacCfg: A pointer to a structure used to
*                                          contain configuration information
*                                          obtained from the XGMAC registers.
* Outputs : A filled data structure containing XGMAC configuration information.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_XgmacCfg_t *ptXgmacCfg);

/******************************************************************************
* Function : Ixf18100XgmacGetStatus
*------------------------------------------------------------------------------
* Description : This function reads the status registers of the XGMAC register
*               block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelStatus_e SelStat: Selects the status to be retrieved.
*           ixf18100_XgmacStatus_t *ptXgmacStatus: pointer to the status
*                                                   structure
* Outputs : A filled data structure containing XGMAC status information. 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                       ixf18100_XgmacStatus_t *ptXgmacStatus);

/******************************************************************************
* Function : Ixf18100XgmacGetCounters
*------------------------------------------------------------------------------
* Description : This function reads counter values in the XGMAC register
*                block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelCounters_e SelCntr: Selects the counters to be read.
*           void *ptCounters: A pointer to a structure used to
*                                          store counter values.
* Outputs : A filled data structure containing counter values.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacGetCounters(bb_RegPointer_type BaseAddress, 
                         bb_SelCounters_e SelCntr, void *ptCounters);

/******************************************************************************
* Function : Ixf18100XgmacSetAddress
*------------------------------------------------------------------------------
* Description :  Function that sets th MAC address for a ixf18100 device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Not used.
*           pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the address.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacSetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress);

/******************************************************************************
* Function : Ixf18100XgmacGetAddress
*------------------------------------------------------------------------------
* Description : This function reads the Mac Address from the device.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Not used.
* Outputs : pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the MAC address.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
extern bb_Error_e
Ixf18100XgmacGetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress);

#endif /*__IXF18100_XGMAC_H__*/
