/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************64
 *     Name:    OSAL C header file for Linux User Space
 *  
 *    Type:     C header file
 * 
 *    Desc:     This file contains the #define and data structures
 *              used by OSAL primitives for Linux User Space
 *
 *
 *    File:     ix_osal_Linux.h 
 *
 *    Sid:      ix_osal_Linux.h@@/main/1 - Tue Dec 10 10:32:48 2002
 *
 *    Prg:      shabbir ali 
 *
*****************************************************************/

#ifndef __IX_OSAL_LINUX_USR_SPACE_H__
#define __IX_OSAL_LINUX_USR_SPACE_H__

#ifdef __cplusplus
extern "C" {
#endif


#ifdef IX_LINUX_USR_SPACE
/* The following defines should be removed once the 
* -ansi and -pedantic features can be used with the
* xscale_be-gcc cross compiler */ 
#define cli()        __cli()
#define sti()        __sti()
#define clf()        __clf()
#define stf()        __stf()
#define save_flags(x)      __save_flags(x)
#define restore_flags(x)   __restore_flags(x)
#define save_flags_cli(x)  __save_flags_cli(x)
/*
 * Disable IRQs
 */
#define __cli()                     \
   {                   \
      unsigned long temp;           \
   __asm__ __volatile__(               \
   "mrs  %0, cpsr    @ cli\n"    \
"  orr   %0, %0, #128\n"               \
"  msr   cpsr_c, %0"             \
   : "=r" (temp)                 \
   :                    \
   : "memory");                  \
   }
/*
 * Enable IRQs
 */
#define __sti()                     \
   {                   \
      unsigned long temp;           \
   __asm__ __volatile__(               \
   "mrs  %0, cpsr    @ sti\n"    \
"  bic   %0, %0, #128\n"               \
"  msr   cpsr_c, %0"             \
   : "=r" (temp)                 \
   :                    \
   : "memory");                  \
   }
/*
 * save current IRQ & FIQ state
 */
#define __save_flags(x)                \
   __asm__ __volatile__(               \
   "mrs  %0, cpsr    @ save_flags\n"      \
     : "=r" (x)                  \
     :                     \
     : "memory")

/*
 * restore saved IRQ & FIQ state
 */
#define __restore_flags(x)             \
   __asm__ __volatile__(               \
   "msr  cpsr_c, %0     @ restore_flags\n"   \
   :                    \
   : "r" (x)                  \
   : "memory")

/* Linux-user space specific defines */
#define IX_LOW_RES_CLOCK_FREQ 1000          /* Low Res Clock Freq */
#define IX_HIGH_RES_CLOCK_FREQ 1000         /* High Res Clock Freq */

/* Give Binary Semaphore */
#define IX_GiveBinarySem_LnxUsrSpc(s) \
	((s == NULL)?IX_UNKNOWN_HANDLE: \
	((pthread_mutex_unlock(s) == EINVAL) ? IX_UNKNOWN_HANDLE : IX_NO_ERROR) )

/* Allocate Cache-Safe Memory */
#define IX_DMA_Malloc_LnxUsrSpc(sizeInBytes) \
	malloc (sizeInBytes)

/* Deallocate Cache-Safe Memory */
#define IX_DMA_Free_LnxUsrSpc(ptr) \
	free(ptr)

/* Allocate regular Memory */
#define IX_Malloc_LnxUsrSpc(sizeInBytes,heapNumber) \
	malloc (sizeInBytes)

/* Deallocate regular Memory */
#define IX_Free_LnxUsrSpc(ptr) \
	free(ptr)

/* Exit from a Thread */
#define IX_Thread_Exit_LnxUsrSpc() \
	pthread_exit(0)

/* Get Low Resolution Clock Frequency */
#define IX_GetLowResClockFrequency_LnxUsrSpc(frequency) \
	((*frequency) = IX_LOW_RES_CLOCK_FREQ)? IX_NO_ERROR \
		: IX_NO_ERROR

/* Get High Resolution Clock Frequency */
#define IX_GetHighResClockFrequency_LnxUsrSpc(frequency) \
	((*frequency) = IX_LOW_RES_CLOCK_FREQ)? IX_NO_ERROR \
		: IX_NO_ERROR



/* Linux specific data types */
typedef struct ix_Linux_Conv IX_Linux_Conv;
struct ix_Linux_Conv
{
	IX_Function_Ptr_t f; /* Function Pointer */
	void *args;          /* Argument to function pointer */
};


/* Linux User space specific extern function defines */

/* Binary semaphore support */
extern IX_Err_t IX_CreateBinarySem_LnxUsrSpc(IX_Sem_t *s);
extern IX_Err_t IX_DestroyBinarySem_LnxUsrSpc(IX_Sem_t s);
extern IX_Err_t IX_TakeBinarySem_LnxUsrSpc(IX_Sem_t s,IX_Boolean_t block);



/* Interrup Lock/Unlock */
extern IX_Err_t IX_Int_Lock_LnxUsrSpc(IX_U32 *lockId);
extern IX_Err_t IX_Int_Unlock_LnxUsrSpc(IX_U32 lockId);


/* Task/Thread Creation/Deletion/Exit */
extern IX_Err_t IX_Thread_Spawn_LnxUsrSpc(IX_Function_Ptr_t f,
		IX_Thread_Info_t *t, IX_Thread_id *i);
extern IX_Err_t IX_Thread_Stop_LnxUsrSpc(IX_Thread_id i);

extern void * ixConversionFunc_LnxUsrSpc(void *arg);

/* Tick Counters */
extern IX_Err_t IX_GetLowResClockCount_LnxUsrSpc(IX_U32 *currentTicks);
extern IX_Err_t IX_GetHighResClockCount_LnxUsrSpc(IX_U32 *currentTicks);



#endif /* IX_LINUX_USR_SPACE */

#ifdef __cplusplus
}
#endif

#endif /* __IX_OSAL_LINUX_USR_SPACE_H__ */

/**********************************************************************
 
     ver       pat    init                  description
------------ -------- ---- ----------------------------------------------
$SID$          ---    sali 1. initial release.
  
**********************************************************************/
