/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_8023WIS.c $
 *   WORKFILE:       $Workfile: ixf18100_8023WIS.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18101/3 802.3 WIS API
 *
 * Purpose: 
 *          
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_8023WIS.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 5     7/19/02 8:37p Jaalliso
 * Change WIS reset to toggle SONET block reset
 * 
 * 4     6/27/02 11:29a Jaalliso
 * IEEE 802.3 WIS API functions
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#endif

/* IXF driver common includes */
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet_d.h"
#include "hal.h"

/* ixf18100 includes */
#include "ixf18100.h"
#include "ixf18100_d.h"
#include "ixf18100_global_d.h"
#include "ixf18100_global_b.h"
#include "ixf18100_sonet_d.h"
#include "ixf18100_sonet_b.h"
#include "ixf18100_sonet.h"
#include "ixf18100_pcs_b.h"
#include "ixf18100_util.h"

/* ixf18101/3 WIS API includes */
#include "ixf18100_8023_b.h"
#include "ixf18100_8023WIS_d.h"


/* Static IEEE 802.3 WIS API data */

/* Status of last IXF API call */
static bb_Error_e ixfStatus = bb_NO_ERROR;  

/* Status of last IEEE 802.3 WIS API function call */
static IEEE8023_Error_e ixfIEEE8023Status = IEEE8023_eNO_ERROR;

/* Flags for first read of 802.3 registers defined with LL/LH behavior */
static boolean IEEE8023_IntLos_Flag = FALSE;  
static boolean IEEE8023_ExtLos_Flag = FALSE;  
static boolean IEEE8023_Lof_Flag = FALSE;  
static boolean IEEE8023_Oof_Flag = FALSE;  
static boolean IEEE8023_ExcB2Err_Flag = FALSE;  
static boolean IEEE8023_Rdil_Flag = FALSE;  
static boolean IEEE8023_Aisl_Flag = FALSE;  
static boolean IEEE8023_Lop_Flag = FALSE;  
static boolean IEEE8023_Aisp_Flag = FALSE;  
static boolean IEEE8023_PlmpChg_Flag = FALSE;  

/* Software register array */
static ushort SWReg[] = 
{
    ixfIEEE8023_WIS_CNTL1_DFLT,             /* 2.0 WIS Control 1 */
    ixfIEEE8023_WIS_STAT1_DFLT,             /* 2.1 WIS Status 1 */
    ixfIEEE8023_DEVICE_ID_2_DFLT,           /* 2.2 Device Identifier */
    ixfIEEE8023_DEVICE_ID_3_DFLT,           /* 2.3 Device Identifier */
    ixfIEEE8023_WIS_SPD_ABLTY_DFLT,         /* 2.4 WIS Speed Ability */
    ixfIEEE8023_DEV_IN_PKG_5_DFLT,          /* 2.5 Devices in package */
    ixfIEEE8023_DEV_IN_PKG_6_DFLT,          /* 2.6 Devices in package */
    ixfIEEE8023_WIS_CNTL2_DFLT,             /* 2.7 10G WIS Control 2 */
    ixfIEEE8023_WIS_STAT2_DFLT,             /* 2.8 10G WIS Status 2 */
    0x0000,                                 /* 2.9 10G WIS test pattern error counter */
    0x0000,                                 /* 2.10 Reserved */
    0x0000,                                 /* 2.11 Reserved */
    0x0000,                                 /* 2.12 Reserved */
    0x0000,                                 /* 2.13 Reserved */
    ixfIEEE8023_PKG_ID_14_DFLT,             /* 2.14 Package identifier */
    ixfIEEE8023_PCK_ID_15_DFLT,             /* 2.15 Package identifier */
    0x0000,                                 /* 2.16 Reserved */
    0x0000,                                 /* 2.17 Reserved */
    0x0000,                                 /* 2.18 Reserved */
    0x0000,                                 /* 2.19 Reserved */
    0x0000,                                 /* 2.20 Reserved */
    0x0000,                                 /* 2.21 Reserved */
    0x0000,                                 /* 2.22 Reserved */
    0x0000,                                 /* 2.23 Reserved */
    0x0000,                                 /* 2.24 Reserved */
    0x0000,                                 /* 2.25 Reserved */
    0x0000,                                 /* 2.26 Reserved */
    0x0000,                                 /* 2.27 Reserved */
    0x0000,                                 /* 2.28 Reserved */
    0x0000,                                 /* 2.29 Reserved */
    0x0000,                                 /* 2.30 Reserved */
    0x0000,                                 /* 2.31 Reserved */
    0x0000,                                 /* 2.32 Reserved */
    ixfIEEE8023_WIS_STAT3_DFLT,             /* 2.33 10G WIS Status 3 */
    0x0000,                                 /* 2.34 Reserved */
    0x0000,                                 /* 2.35 Reserved */
    0x0000,                                 /* 2.36 Reserved */
    ixfIEEE8023_WIS_FEP_BLK_ERR_CNT_DFLT,   /* 2.37 10G WIS Far End Path Block Error Count */
    0x0000,                                 /* 2.38 Reserved */
    ixfIEEE8023_WIS_J1_TX01_DFLT,           /* 2.39 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX23_DFLT,           /* 2.40 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX45_DFLT,           /* 2.41 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX67_DFLT,           /* 2.42 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX89_DFLT,           /* 2.43 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX1011_DFLT,         /* 2.44 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX1213_DFLT,         /* 2.45 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_TX1415_DFLT,         /* 2.46 10G WIS J1 Tx */
    ixfIEEE8023_WIS_J1_RX01_DFLT,           /* 2.47 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX23_DFLT,           /* 2.48 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX45_DFLT,           /* 2.49 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX67_DFLT,           /* 2.50 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX89_DFLT,           /* 2.51 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX1011_DFLT,         /* 2.52 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX1213_DFLT,         /* 2.53 10G WIS J1 Rx */
    ixfIEEE8023_WIS_J1_RX1415_DFLT,         /* 2.54 10G WIS J1 Rx */
    ixfIEEE8023_WIS_FEL_BIP_ERR_55_DFLT,    /* 2.55 10G WIS Far End Line BIP Errors */
    ixfIEEE8023_WIS_FEL_BIP_ERR_56_DFLT,    /* 2.56 10G WIS Far End Line BIP Errors */
    ixfIEEE8023_WIS_LINE_BIP_ERR_57_DFLT,   /* 2.57 10G WIS Line BIP Errors */
    ixfIEEE8023_WIS_LINE_BIP_ERR_58_DFLT,   /* 2.58 10G WIS Line BIP Errors */
    ixfIEEE8023_WIS_PATH_BLK_ERR_CNT_DFLT,  /* 2.59 10G WIS Path Block Error Count */
    ixfIEEE8023_WIS_SECT_BIP_ERR_CNT_DFLT,  /* 2.60 10G WIS Secion BIP Count */
    0x0000,                                 /* 2.61 Reserved */
    0x0000,                                 /* 2.62 Reserved */
    0x0000,                                 /* 2.63 Reserved */
    ixfIEEE8023_WIS_J0_TX01_DFLT,           /* 2.64 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX23_DFLT,           /* 2.65 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX45_DFLT,           /* 2.66 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX67_DFLT,           /* 2.67 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX89_DFLT,           /* 2.68 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX1011_DFLT,         /* 2.69 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX1213_DFLT,         /* 2.70 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_TX1415_DFLT,         /* 2.71 10G WIS J0 Tx */
    ixfIEEE8023_WIS_J0_RX01_DFLT,           /* 2.72 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX23_DFLT,           /* 2.73 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX45_DFLT,           /* 2.74 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX67_DFLT,           /* 2.75 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX89_DFLT,           /* 2.76 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX1011_DFLT,         /* 2.77 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX1213_DFLT,         /* 2.78 10G WIS J0 Rx */
    ixfIEEE8023_WIS_J0_RX1415_DFLT          /* 2.79 10G WIS J0 Rx */
};
static void
Ixf1810xWISApiRegReset(void);

IEEE8023_Error_e
Ixf1810xWISApiGetStatus(void);

bb_Error_e
Ixf1810xWISApiGetIxfStatus(void);

/******************************************************************************
* Function : Ixf1810xWISApiGetIxfStatus
*------------------------------------------------------------------------------
* Description  : Returns the value of the static variable ixfStatus.  This
*                variable is set to the return value of each IXF API function
*                call made from withing the 802.3 WIS API. 
*------------------------------------------------------------------------------
* Inputs  : None.
*
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : Static variable ixfStatus is returned.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf1810xWISApiGetIxfStatus()
{
    return(ixfStatus);
}

/******************************************************************************
* Function : Ixf1810xWISApiGetStatus
*------------------------------------------------------------------------------
* Description  : Returns the value of the static variable ixfIEEE8023Status.  
*                This variable is set with  IEEE 8023 WIS API functions such
*                as Ixf1810xWISApiRead() that can't return  error codes. 
*------------------------------------------------------------------------------
* Inputs  : None.
*
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : Static variable ixfIEEE8023Status is returned.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
IEEE8023_Error_e
Ixf1810xWISApiGetStatus()
{
    return(ixfIEEE8023Status);
}

/******************************************************************************
* Function : Ixf1810xWISApiRegReset
*------------------------------------------------------------------------------
* Description  : Resets the 802.3 WIS API software registers (with exception
*                of 2.0.15 (Reset bit) to their default values.
*------------------------------------------------------------------------------
* Inputs  : None.
*
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : Contents of SWReg[] array reset to default values.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static void
Ixf1810xWISApiRegReset()
{
    /* For 2.0, preserve bit 15 (Reset) if it is set */
    SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] = 
        (SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] & ixfIEEE8023_mRESET) | ixfIEEE8023_WIS_CNTL1_DFLT;

    /* Reset the reset of the registers to their default values */
    SWReg[ixfIEEE8023_WIS_STAT1_OFFSET] = ixfIEEE8023_WIS_STAT1_DFLT;             
    SWReg[ixfIEEE8023_DEVICE_ID_2_OFFSET] = ixfIEEE8023_DEVICE_ID_2_DFLT;         
    SWReg[ixfIEEE8023_DEVICE_ID_3_OFFSET] = ixfIEEE8023_DEVICE_ID_3_DFLT;         
    SWReg[ixfIEEE8023_WIS_SPD_ABLTY_OFFSET] = ixfIEEE8023_WIS_SPD_ABLTY_DFLT;     
    SWReg[ixfIEEE8023_DEV_IN_PKG_5_OFFSET] = ixfIEEE8023_DEV_IN_PKG_5_DFLT;       
    SWReg[ixfIEEE8023_DEV_IN_PKG_6_OFFSET] = ixfIEEE8023_DEV_IN_PKG_6_DFLT;       
    SWReg[ixfIEEE8023_WIS_CNTL2_OFFSET] = ixfIEEE8023_WIS_CNTL2_DFLT;             
    SWReg[ixfIEEE8023_WIS_STAT2_OFFSET] = ixfIEEE8023_WIS_STAT2_DFLT;             
    SWReg[ixfIEEE8023_WIS_TP_ERR_CNT_OFFSET] = 0x0000;   
    SWReg[ixfIEEE8023_RESERVED10] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED11] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED12] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED13] = 0x0000;                                 
    SWReg[ixfIEEE8023_PKG_ID_14_OFFSET] = ixfIEEE8023_PKG_ID_14_DFLT;            
    SWReg[ixfIEEE8023_PKG_ID_15_OFFSET] = ixfIEEE8023_PCK_ID_15_DFLT;            
    SWReg[ixfIEEE8023_RESERVED16] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED17] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED18] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED19] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED20] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED21] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED22] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED23] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED24] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED25] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED26] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED27] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED28] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED29] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED30] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED31] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED32] = 0x0000;                                 
    SWReg[ixfIEEE8023_WIS_STAT3_OFFSET] = ixfIEEE8023_WIS_STAT3_DFLT;             
    SWReg[ixfIEEE8023_RESERVED34] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED35] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED36] = 0x0000;                                 
    SWReg[ixfIEEE8023_WIS_FEP_BLK_ERR_CNT_OFFSET] = ixfIEEE8023_WIS_FEP_BLK_ERR_CNT_DFLT;   
    SWReg[ixfIEEE8023_RESERVED38] = 0x0000;                                
    SWReg[ixfIEEE8023_WIS_J1_TX01_OFFSET] = ixfIEEE8023_WIS_J1_TX01_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_TX23_OFFSET] = ixfIEEE8023_WIS_J1_TX23_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_TX45_OFFSET] = ixfIEEE8023_WIS_J1_TX45_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_TX67_OFFSET] = ixfIEEE8023_WIS_J1_TX67_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_TX89_OFFSET] = ixfIEEE8023_WIS_J1_TX89_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_TX1011_OFFSET] = ixfIEEE8023_WIS_J1_TX1011_DFLT;       
    SWReg[ixfIEEE8023_WIS_J1_TX1213_OFFSET] = ixfIEEE8023_WIS_J1_TX1213_DFLT;       
    SWReg[ixfIEEE8023_WIS_J1_TX1415_OFFSET] = ixfIEEE8023_WIS_J1_TX1415_DFLT;       
    SWReg[ixfIEEE8023_WIS_J1_RX01_OFFSET] = ixfIEEE8023_WIS_J1_RX01_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_RX23_OFFSET] = ixfIEEE8023_WIS_J1_RX23_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_RX45_OFFSET] = ixfIEEE8023_WIS_J1_RX45_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_RX67_OFFSET] = ixfIEEE8023_WIS_J1_RX67_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_RX89_OFFSET] = ixfIEEE8023_WIS_J1_RX89_DFLT;           
    SWReg[ixfIEEE8023_WIS_J1_RX1011_OFFSET] = ixfIEEE8023_WIS_J1_RX1011_DFLT;       
    SWReg[ixfIEEE8023_WIS_J1_RX1213_OFFSET] = ixfIEEE8023_WIS_J1_RX1213_DFLT;       
    SWReg[ixfIEEE8023_WIS_J1_RX1415_OFFSET] = ixfIEEE8023_WIS_J1_RX1415_DFLT;       
    SWReg[ixfIEEE8023_WIS_FEL_BIP_ERR_55_OFFSET] = ixfIEEE8023_WIS_FEL_BIP_ERR_55_DFLT;    
    SWReg[ixfIEEE8023_WIS_FEL_BIP_ERR_56_OFFSET] = ixfIEEE8023_WIS_FEL_BIP_ERR_56_DFLT;    
    SWReg[ixfIEEE8023_WIS_LINE_BIP_ERR_57_OFFSET] = ixfIEEE8023_WIS_LINE_BIP_ERR_57_DFLT;  
    SWReg[ixfIEEE8023_WIS_LINE_BIP_ERR_58_OFFSET] = ixfIEEE8023_WIS_LINE_BIP_ERR_58_DFLT;  
    SWReg[ixfIEEE8023_WIS_PATH_BLK_ERR_CNT_OFFSET] = ixfIEEE8023_WIS_PATH_BLK_ERR_CNT_DFLT;
    SWReg[ixfIEEE8023_WIS_SECT_BIP_ERR_CNT_OFFSET] = ixfIEEE8023_WIS_SECT_BIP_ERR_CNT_DFLT;
    SWReg[ixfIEEE8023_RESERVED61] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED62] = 0x0000;                                 
    SWReg[ixfIEEE8023_RESERVED63] = 0x0000;                                 
    SWReg[ixfIEEE8023_WIS_J0_TX01_OFFSET] = ixfIEEE8023_WIS_J0_TX01_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_TX23_OFFSET] = ixfIEEE8023_WIS_J0_TX23_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_TX45_OFFSET] = ixfIEEE8023_WIS_J0_TX45_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_TX67_OFFSET] = ixfIEEE8023_WIS_J0_TX67_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_TX89_OFFSET] = ixfIEEE8023_WIS_J0_TX89_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_TX1011_OFFSET] = ixfIEEE8023_WIS_J0_TX1011_DFLT;      
    SWReg[ixfIEEE8023_WIS_J0_TX1213_OFFSET] = ixfIEEE8023_WIS_J0_TX1213_DFLT;      
    SWReg[ixfIEEE8023_WIS_J0_TX1415_OFFSET] = ixfIEEE8023_WIS_J0_TX1415_DFLT;      
    SWReg[ixfIEEE8023_WIS_J0_RX01_OFFSET] = ixfIEEE8023_WIS_J0_RX01_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_RX23_OFFSET] = ixfIEEE8023_WIS_J0_RX23_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_RX45_OFFSET] = ixfIEEE8023_WIS_J0_RX45_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_RX67_OFFSET] = ixfIEEE8023_WIS_J0_RX67_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_RX89_OFFSET] = ixfIEEE8023_WIS_J0_RX89_DFLT;          
    SWReg[ixfIEEE8023_WIS_J0_RX1011_OFFSET] = ixfIEEE8023_WIS_J0_RX1011_DFLT;      
    SWReg[ixfIEEE8023_WIS_J0_RX1213_OFFSET] = ixfIEEE8023_WIS_J0_RX1213_DFLT;      
    SWReg[ixfIEEE8023_WIS_J0_RX1415_OFFSET] = ixfIEEE8023_WIS_J0_RX1415_DFLT;      

    return;
}


/******************************************************************************
* Function : Ixf1810xWISApiRead
*------------------------------------------------------------------------------
* Description  : Ixf18101/3 WIS API read function.  Reads an Ixf18101/3
*                register from the IEEE 802.3 perspective.
*------------------------------------------------------------------------------
* Inputs  : *pChipData : Pointer to chip data structure
*           offset     : IEEE 802.3 WIS register offset to be written
*
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : Value read/assembled from software maintained registers and/or
*           Ixf18101/3 register(s).
*------------------------------------------------------------------------------
* Access Globals : ixfStatus : set if there is an error condition in calling
*                  an IXF API function.
*                : ixfIEEE8023Status : set if there is the WIS API is in reset
*                  or if the offset parameter is invalid, 
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
ushort
Ixf1810xWISApiRead(bb_ChipData_t *pChipData, ushort offset)
{
    ushort regval = 0, temp1 = 0, temp2 = 0, tlength = 0;
    ushort rl_soh_is = 0, rl_loh_is = 0, rl_soh_s = 0, rl_loh_s = 0;
    ushort rl_poh_is = 0, rl_pt_is = 0, rl_pt_s = 0, rl_rcv_g1 = 0;
    ushort rl_poh_s = 0;
    ushort i;
    bb_ChipSegment_t Segment;
    char trace[IEEE8023_WIS_TRACELENGTH + 1];

    /* If a WIS Reset is in progress, per 802.3 spec, only respond to reads */
    /* of registers 2.0 (returning bit 15) and 2.8 (returning bits 15:14)   */
    if (SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] & ixfIEEE8023_mRESET)
    {
        /* WIS Reset in progress */
        if (offset != ixfIEEE8023_WIS_CNTL1_OFFSET && offset != ixfIEEE8023_WIS_STAT2_OFFSET)
        {
            /* Don't allow reads from any other WIS registers, return 0 */
            ixfIEEE8023Status = IEEE8023_eWIS_IN_RESET;
            return (0);
        }
    }

    switch (offset)
    {
        case ixfIEEE8023_WIS_CNTL1_OFFSET: /* 2.0 */
            /* 2.0.15 - SW maintained bit for read.  SW bit is flag indicating */
            /* if a reset is in progress.  Note - this bit also maps to ixf18101/3 */
            /* SRESET and GLB_RST registers for writes */
            
            if (SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] & ixfIEEE8023_mRESET)
            {
                /* WIS reset in progress, only respond with 2.0.15 */
                regval = SWReg[offset] & ixfIEEE8023_mRESET;
            }
            else
            {            
                regval = (SWReg[offset] & ixfIEEE8023_mRESET);

                /* 2.0.14 - maps to ixf18101/3 GLB_LINE.lloop_en */
                temp1 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_GLB_LINE_OFFSET);
                temp1 = (temp1 & ixf18100_mLLOOP_EN) >> ixf18100_sLLOOP_EN;
                regval |= (temp1 << ixfIEEE8023_sLOOPBACK);

                /* 2.0.13 - SW maintained bit */
                temp1 = (SWReg[offset] & ixfIEEE8023_mSPD_SEL_13);
                regval |= temp1; 

                /* 2.0.12 is a reserved bit, value always 0 so nothing to do */

                /* 2.0.11 maps to inverted value of ixf18101/3 GLB_CFG1:SONET_En */
                temp1 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_GLB_CFG1_OFFSET);
                temp1 = (~temp1 & ixf18100_mSONET_EN) >> ixf18100_sSONET_EN;
                regval |= (temp1 << ixfIEEE8023_sLOW_PWR);

                /* 2.0.10:7 are reserved bits */

                /* 2.0.6 - SW maintained bit */
                temp1 = (SWReg[offset] & ixfIEEE8023_mSPD_SEL_6);
                regval |= temp1;

                /* 2.0.5:2 - SW maintained bits */
                temp1 = (SWReg[offset] & ixfIEEE8023_mSPD_SEL_52);
                regval |= temp1;

                /* 2.0.1:0 are reserved bits */
            }
            break;

        case ixfIEEE8023_WIS_STAT1_OFFSET:  /* 2.1 */
            /* 2.1.15:8 are reserved bits */

            /* 2.1.7 maps to logical OR of ixf18101/3 GLB_CLKINTSTAT.Clk_Rx_Fail_Int, */
            /* RL_SOH_IS/RL_SOH_S, and RL_LOH_IS/RL_LOH_S */

            temp1 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_GLB_CLKINTSTAT_OFFSET);  
            temp1 = (temp1 & ixf18100_mCLK_RX_FAIL_INT) >> ixf18100_sCLK_RX_FAIL_INT;

            rl_soh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_SOH_IS_OFFSET);
            rl_soh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_SOH_S_OFFSET);
            rl_loh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_LOH_IS_OFFSET);
            rl_loh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_LOH_S_OFFSET);
            rl_poh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_POH_IS_OFFSET);
            rl_poh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_POH_S_OFFSET);
            rl_pt_is  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_PT_IS_OFFSET);
            rl_pt_s   = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_PT_S_OFFSET);

            /* Check RL_SOH_IS/RL_SOH_S */
            if (IEEE8023_IntLos_Flag == FALSE && IEEE8023_ExtLos_Flag == FALSE &&
                IEEE8023_Lof_Flag == FALSE && IEEE8023_Oof_Flag == FALSE &&
                IEEE8023_ExcB2Err_Flag == FALSE)
            {
                /* No interrupt status bits for this register have ever indicated an   */
                /* error condition so check ixf18101/3 RL_SOH_IS interrupt status bits */
                if ( (rl_loh_is & ixf18100_mEXC_B2_ERR) | 
                     (rl_soh_is & (ixf18100_mINT_LOS | ixf18100_mEXT_LOS | 
                                  ixf18100_mLOF | ixf18100_mOOF)) )
                {
                    /* At least one of the mapped interrupt status bits of */
                    /* RL_SOH_IS or RL_LOH_IS is set so 2.1.7 is 0x1 */
                    temp1 = 0x1;                    
                }
                else
                    temp1 = 0x0;
            }
            else
            {
                /* At least one LL/LH bit has indicated an error condition */
                /* at some point so check ixf18101/3 RL_SOH_S and rl_loh_s */
                /* status bits */
                if ((rl_loh_s & ixf18100_mEXC_B2_ERR_ST) | 
                    (rl_soh_s & (ixf18100_mINT_LOS_ST | ixf18100_mEXT_LOS_ST | 
                                 ixf18100_mLOF_ST | ixf18100_mOOF_ST)) )
                {
                    /* At least one of the mapped interrupt bits of RL_SOH_S is set */
                    /* so 2.1.7 is 0x1 */                
                    temp1 = 0x1;
                }
                else
                    temp1 = 0x0;
            }

            /* Shift value we've determined for 2.1.7 to proper position */
            regval = temp1 << ixfIEEE8023_sLOCAL_FAULT;


            /*****************************/
            /* 2.1.6:3 are reserved bits */
            /*****************************/


            /* 2.1.2 - Note - defined in spec as Latching Low (LL) */
            if (IEEE8023_PlmpChg_Flag == FALSE && IEEE8023_Aisp_Flag == FALSE && 
                IEEE8023_Lop_Flag == FALSE && IEEE8023_Lof_Flag == FALSE && 
                IEEE8023_Aisl_Flag == FALSE)
            {
                /* All interrupt status register flags are FALSE so look at */
                /* interrupt status registers */

                /* Read appropriate interrupt status registers (that haven't */
                /* already been read */
                if ( (rl_poh_is & ixf18100_mPLMP_CHG)   | 
                     (rl_pt_is & ixf18100_mAISP)        |
                     (rl_pt_is & ixf18100_mLOP)         |
                     (rl_soh_is & ixf18100_mLOF)        |
                     (rl_loh_is & ixf18100_mAISL) )
                {
                    /* At least one of the interrupt status bits is set so */
                    /* 2.1.2 is 0x0 (not 0x1 because 2.1.2 is LL) */
                    temp1 = 0x0;                    
                }
                else
                    temp1 = 0x1;
            }
            else
            {
                /* At least one interrupt status register flag was FALSE so */
                /* look at interrupt state bits */
                if ( (rl_poh_s & ixf18100_mPLMP_ST) |
                     (rl_pt_s & ixf18100_mAISP_ST)  |
                     (rl_pt_s & ixf18100_mLOP_ST)   |
                     (rl_soh_s & ixf18100_mLOF_ST)  |
                     (rl_loh_s & ixf18100_mAISL_ST) )
                {
                    /* At least one of the interrupt bits is set, indicate */
                    /* by clearing 2.1.2 as it is LL */                
                    temp1 = 0x0;
                }
                else
                    temp1 = 0x1;
            }
            /* Or in value we've determined for 2.1.2 */
            regval |= (temp1 << ixfIEEE8023_sLINK_STAT);


            /* 2.1.1 - SW maintained bit */
            temp1 = (SWReg[offset] & ixfIEEE8023_mLOW_PWR_ABLTY);
            regval |= temp1;

            /* 2.1.0 is a reserved bit */


            /* If any mapped interrupt status bit is set, set appropriate   */
            /* flag so next read of will check current interrupt state bits */
            if (rl_soh_is & ixf18100_mINT_LOS)
                IEEE8023_IntLos_Flag = TRUE;
                     
            if (rl_soh_is & ixf18100_mEXT_LOS)
                IEEE8023_ExtLos_Flag = TRUE;
                     
            if (rl_soh_is & ixf18100_mLOF)
                IEEE8023_Lof_Flag = TRUE;
                     
            if (rl_soh_is & ixf18100_mOOF)
                IEEE8023_Oof_Flag = TRUE;     

            if (rl_loh_is & ixf18100_mEXC_B2_ERR)
                IEEE8023_ExcB2Err_Flag = TRUE;     

            if (rl_loh_is & ixf18100_mAISL)
                IEEE8023_Aisl_Flag = TRUE;

            if (rl_poh_is & ixf18100_mPLMP_CHG)
                IEEE8023_PlmpChg_Flag = TRUE;
                     
            if (rl_pt_is & ixf18100_mAISP)
                IEEE8023_Aisp_Flag = TRUE;
                     
            if (rl_pt_is & ixf18100_mLOP)
                IEEE8023_Lop_Flag = TRUE;
                     
            break;

        case ixfIEEE8023_DEVICE_ID_2_OFFSET: /* 2.2 */
            /* 2.2 is SW maintained */
            regval = SWReg[offset];

            break;

        case ixfIEEE8023_DEVICE_ID_3_OFFSET: /* 2.3 */
            /* 2.3 is SW maintained */
            regval = SWReg[offset];

            break;

        case ixfIEEE8023_WIS_SPD_ABLTY_OFFSET: /* 2.4 */
            /* 2.4.15:1 are reserved bits */

            /* 2.4.0 */
            regval = (SWReg[offset] & ixfIEEE8023_m10G_CAPABLE);
            break;

        case ixfIEEE8023_DEV_IN_PKG_5_OFFSET: /* 2.5 */
        case ixfIEEE8023_DEV_IN_PKG_6_OFFSET: /* 2.6 */
            /* All defined bits in 2.5 and 2.6 are SW maintained */
            regval = SWReg[offset];
            
            break;

        case ixfIEEE8023_WIS_CNTL2_OFFSET: /* 2.7 */
            /* 2.7.15:4 are reserved */
            /* 2.7.3 maps to WMAP_CFG.jitter_pattern_select */
            temp1 = temp2 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_WMAP_CFG_OFFSET);
            temp1 = (temp1 & ixf18100_mJITTER_PATTERN_SEL) >> ixf18100_sJITTER_PATTERN_SEL;
            regval |= (temp1 << ixfIEEE8023_sTEST_PTRN);

            /* 2.7.2 maps to WMAP_CFG.rx_jitter_test */
            temp1 = temp2;  /* Already read WMAP_CFG and saved a copy in temp1 */
            temp1 = (temp1 & ixf18100_mRX_JITTER_TEST) >> ixf18100_sRX_JITTER_TEST;
            regval |= (temp1 << ixfIEEE8023_sRX_TEST_PTRN_EN);

            /* 2.7.1 maps to WMAP_CFG.tx_jitter_test */
            temp1 = temp2;  /* Already read WMAP_CFG and saved a copy in temp1 */
            temp1 = (temp1 & ixf18100_mTX_JITTER_TEST) >> ixf18100_sTX_JITTER_TEST;
            regval |= (temp1 << ixfIEEE8023_sTX_TEST_PTRN_EN);

            /* 2.7.0 maps to ixf18101/3 GLB_CFG1:SONET_En */
            temp1 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_GLB_CFG1_OFFSET);
            temp1 = (temp1 & ixf18100_mSONET_EN) >> ixf18100_sSONET_EN;
            regval |= (temp1 << ixfIEEE8023_sPCS_TYPE_SEL);

            break;

        case ixfIEEE8023_WIS_STAT2_OFFSET:  /* 2.8 */
            /* If a WIS Rest is in progress, respond with 2.8.15:14 */
            if (SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] & ixfIEEE8023_mRESET)
            {
                /* If a WIS Reset is in process, only respond with bits 15:14 */
                /* 2.8.15:14 are SW maintained */
                regval = (SWReg[offset] & ixfIEEE8023_mDEV_PRES);
            }
            else
            {
                /* No WIS reset in progress, proceed normally */
                /* All bits are SW maintained or reserved */
                regval = SWReg[offset];
            }

            break;
 
        case ixfIEEE8023_WIS_TP_ERR_CNT_OFFSET:  /* 2.9 */
            /* 2.9 not supported as Ixf1810x doesn't support PRBS31 */
            ixfIEEE8023Status = IEEE8023_eWIS_REGNOTSUPPORTED;

            break;

        /* 2.10 - 2.13 are reserved registers */

        case ixfIEEE8023_PKG_ID_14_OFFSET:  /* 2.14 */
        case ixfIEEE8023_PKG_ID_15_OFFSET:  /* 2.15 */
            /* 2.14 and 2.15 are SW maintained */
            regval = SWReg[offset];

            break;

        /* 2.16 - 2.32 are reserved registers */

        case ixfIEEE8023_WIS_STAT3_OFFSET:  /* 2.33 */
            /* 2.33.15:12 are reserved bits */

            /* Read all interrupt status/state registers once (status */
            /* registers are CoR) */
            rl_soh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_SOH_IS_OFFSET);
            rl_soh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_SOH_S_OFFSET);
            rl_poh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_POH_IS_OFFSET);
            rl_poh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_POH_S_OFFSET);
            rl_rcv_g1 = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_RCV_G1_OFFSET);
            rl_loh_is = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_LOH_IS_OFFSET);
            rl_loh_s  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_LOH_S_OFFSET);
            rl_pt_is  = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_PT_IS_OFFSET);
            rl_pt_s   = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_PT_S_OFFSET);

            /* 2.33.11 */
            if (IEEE8023_Oof_Flag == FALSE)
            {
                /* LL/LH bit for this register have never indicated an      */
                /* error condition so check ixf18101/3 RL_SOH_IS interrupt  */
                /* status bits                                              */
                temp1 = (rl_soh_is & ixf18100_mOOF) >> ixf18100_sOOF;
            }
            else
                temp1 = (rl_soh_s & ixf18100_mOOF_ST) >> ixf18100_sOOF_ST;

            regval = (temp1 << ixfIEEE8023_sSEF);

            /* 2.33.10 - Maps to RL_POH_IS/RL_RCV_G1 */
            /* Read RL_RCV_G1 to see if the Rdi bits indicate an error */
            temp2 = (rl_rcv_g1 & ixf18100_mRDI) >> ixf18100_sRDI;
            if (temp2 == 0x2) /* PLM-P, LCD-P */
                temp1 = 0x1;  /* 2.33.10 needs to be set */
            else
                temp1 = 0x0;  /* 2.33.10 needs to be clear */

            regval |= (temp1 << ixfIEEE8023_sFAR_END_PLM_P_LCD_P);

            /* 2.33.9 - Maps to RL_POH_IS/RL_RCV_G1 */
            /* Read RL_RCV_G1 to see if the Rdi bits indicate an error */
            temp2 = (rl_rcv_g1 & ixf18100_mRDI) >> ixf18100_sRDI;
            if (temp2 == 0x5) /* AIS-P, LOP-P */
                temp1 = 0x1;  /* 2.33.10 needs to be set */
            else
                temp1 = 0x0;  /* 2.33.10 needs to be clear */

            regval |= (temp1 << ixfIEEE8023_sFAR_END_AIS_P_LOP_P);


            /* 2.33.8 - Reserved */


            /* 2.33.7 */
            if (IEEE8023_Lof_Flag == FALSE)
                temp1 = (rl_soh_is & ixf18100_mLOF) >> ixf18100_sLOF;
            else
                temp1 = (rl_soh_s & ixf18100_mLOF_ST) >> ixf18100_sLOF_ST;
            
            regval |= (temp1 << ixfIEEE8023_sLOF);


            /* 2.33.6 */
            if (IEEE8023_IntLos_Flag == FALSE)
                temp1 = (rl_soh_is & ixf18100_mINT_LOS) >> ixf18100_sINT_LOS;
            else
                temp1 = (rl_soh_s & ixf18100_mINT_LOS_ST) >> ixf18100_sINT_LOS_ST;
            
            regval |= (temp1 << ixfIEEE8023_sLOS);


            /* 2.33.5 */
            if (IEEE8023_Rdil_Flag == FALSE)
                temp1 = (rl_loh_is & ixf18100_mRDIL) >> ixf18100_sRDIL;
            else
                temp1 = (rl_loh_s & ixf18100_mRDIL_ST) >> ixf18100_sRDIL_ST;
            
            regval |= (temp1 << ixfIEEE8023_sRDI_L);

            /* 2.33.4 */
            if (IEEE8023_Aisl_Flag == FALSE)
                temp1 = (rl_loh_is & ixf18100_mAISL) >> ixf18100_sAISL;
            else
                temp1 = (rl_loh_s & ixf18100_mAISL_ST) >> ixf18100_sAISL_ST;
            
            regval |= (temp1 << ixfIEEE8023_sAIS_L);


            /* 2.33.3 - Maps to PCS_STAT1.PCS Receive Link Status */
            /* No special LL/LH handling since this PCS register is defined as LL */
            temp1 = (rl_poh_is & ixf18100_mPCS_REC_LINK_STAT) >> ixf18100_sPCS_REC_LINK_STAT;
            regval |= (temp1 << ixfIEEE8023_sLCD_P);

            /* 2.33.2 - Maps to RL_POH_IS.PimpChg */
            if (IEEE8023_PlmpChg_Flag == FALSE)
                temp1 = (rl_poh_is & ixf18100_mPLMP_CHG) >> ixf18100_sPLMP_CHG;
            else
                temp1 = (rl_poh_s & ixf18100_mPLMP_ST) >> ixf18100_sPLMP_ST;
            
            regval |= (temp1 << ixfIEEE8023_sPLM_P);


            /* 2.33.1 - Maps to RL_PT_IS.Aisp/RL_PT_S.AispSt */
            if (IEEE8023_Aisp_Flag == FALSE)
                temp1 = (rl_pt_is & ixf18100_mAISP) >> ixf18100_sAISP;
            else
                temp1 = (rl_pt_s & ixf18100_mAISP_ST) >> ixf18100_sAISP_ST;
            
            regval |= (temp1 << ixfIEEE8023_sAIS_P);


            /* 2.33.0 - Maps to RL_PT_IS.Lop/RL_PT_S.LopSt */
            if (IEEE8023_Lop_Flag == FALSE)
                temp1 = (rl_pt_is & ixf18100_mLOP) >> ixf18100_sLOP;
            else
                temp1 = (rl_pt_s & ixf18100_mLOP_ST) >> ixf18100_sLOP_ST;
            
            regval |= (temp1 << ixfIEEE8023_sLOP_P);

            /* If mapped bit is set, set IEEE8023_Oof_Flag so next read */
            /* of 2.33 will check current interrupt state bits   */
            if (rl_soh_is & ixf18100_mOOF)
                IEEE8023_Oof_Flag = TRUE;     

            if (rl_soh_is & ixf18100_mLOF)
                IEEE8023_Lof_Flag = TRUE;     

            if (rl_soh_is & ixf18100_mINT_LOS)
                IEEE8023_IntLos_Flag = TRUE;     

            if (rl_loh_is & ixf18100_mRDIL)
                IEEE8023_Rdil_Flag = TRUE;     

            if (rl_loh_is & ixf18100_mAISL)
                IEEE8023_Aisl_Flag = TRUE;     

            if (rl_poh_is & ixf18100_mPLMP_CHG)
                IEEE8023_PlmpChg_Flag = TRUE;

            if (rl_pt_is & ixf18100_mAISP)
                IEEE8023_Aisp_Flag = TRUE;

            if (rl_pt_is & ixf18100_mLOP)
                IEEE8023_Lop_Flag = TRUE;

            break;

        /* 2.34 - 2.36 are reserved registers */

        case ixfIEEE8023_WIS_FEP_BLK_ERR_CNT_OFFSET:  /* 2.37 */
            /* 2.37 maps to RL_REIPBLK_CNT */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_REIP_BLK_CNT_OFFSET);

            break;

        /* 2.38 is a reserved register */

        case ixfIEEE8023_WIS_J1_TX01_OFFSET:    /* 2.39 */
        case ixfIEEE8023_WIS_J1_TX23_OFFSET:    /* 2.40 */
        case ixfIEEE8023_WIS_J1_TX45_OFFSET:    /* 2.41 */
        case ixfIEEE8023_WIS_J1_TX67_OFFSET:    /* 2.42 */
        case ixfIEEE8023_WIS_J1_TX89_OFFSET:    /* 2.43 */
        case ixfIEEE8023_WIS_J1_TX1011_OFFSET:  /* 2.44 */
        case ixfIEEE8023_WIS_J1_TX1213_OFFSET:  /* 2.45 */
        case ixfIEEE8023_WIS_J1_TX1415_OFFSET:  /* 2.46 */
            /* 2.39 - 2.46 map to TL_J1_STRA */
            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* clear trace buffer */
                trace[i] = '\0';
            
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure TL_POH_CFG is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_TL_POH_CFG_OFFSET, 
                              (ixf18100_mTL_J1_CFG | ixf18100_mCRC7_EN), 
                              ((ixf18100_vTL_J1_CFG_16Byte << ixf18100_sTL_J1_CFG) | ixf18100_mCRC7_EN));
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_TRANSMIT_J1, trace, &tlength);
            if (ixfStatus == bb_NO_ERROR)
            {
                /* Return the two trace bytes corresponding to offset parameter.    */
                /* trace[] is character oriented, return value contains two trace   */
                /* characters.                                                      */
                i = (offset - ixfIEEE8023_WIS_J1_TX01_OFFSET) * 2;
                regval = (trace[i] << 8) | trace[i + 1];
            }
            else
                regval = 0;

            break;

        case ixfIEEE8023_WIS_J1_RX01_OFFSET:    /* 2.47 */
        case ixfIEEE8023_WIS_J1_RX23_OFFSET:    /* 2.48 */
        case ixfIEEE8023_WIS_J1_RX45_OFFSET:    /* 2.49 */
        case ixfIEEE8023_WIS_J1_RX67_OFFSET:    /* 2.50 */
        case ixfIEEE8023_WIS_J1_RX89_OFFSET:    /* 2.51 */
        case ixfIEEE8023_WIS_J1_RX1011_OFFSET:  /* 2.52 */
        case ixfIEEE8023_WIS_J1_RX1213_OFFSET:  /* 2.53 */
        case ixfIEEE8023_WIS_J1_RX1415_OFFSET:  /* 2.54 */
            /* 2.47 - 2.54 map to RL_J1_ASTRA */
            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* clear trace buffer */
                trace[i] = '\0';
            
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure RL_J1_CFG is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_RL_J1_CFG_OFFSET, 
                              ixf18100_mJ1_CFG, ixf18100_vJ1_CFG_CRC7);
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_ACCEPTED_J1, trace, &tlength);
            if (ixfStatus == bb_NO_ERROR)
            {
                /* Return the two trace bytes corresponding to offset parameter.    */
                /* trace[] is character oriented, return value contains two trace   */
                /* characters.                                                      */
                i = (offset - ixfIEEE8023_WIS_J1_RX01_OFFSET) * 2;
                regval = (trace[i] << 8) | trace[i + 1];
            }
            else
                regval = 0;
    
            break;

        case ixfIEEE8023_WIS_FEL_BIP_ERR_55_OFFSET:  /* 2.55 */
            /* 2.55 maps to RL_REIL_BIP_CNT (bits 31:16) */                                           
            /* Reading RL_REIL_BIP_CNT returns most significant 16 bits of counter and latches lower    */
            /* 16 bits at the next register address.  Note - RL_REIL_BIP_CNT is a CoR register          */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_REIL_BIP_CNT_W1_OFFSET);

            break;

        case ixfIEEE8023_WIS_FEL_BIP_ERR_56_OFFSET:  /* 2.56 */
            /* 2.56 maps to RL_REIL_BIP_CNT (bits 15:0) */
            /* Assuming that 2.55 has been read, reading RL_REIL_BIP_CNT_W1 returns least significant 16 */
            /* bits of counter.  If 2.56 has not been read, the results of this read is 0                */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_REIL_BIP_CNT_W0_OFFSET);

            break;

        case ixfIEEE8023_WIS_LINE_BIP_ERR_57_OFFSET:  /* 2.57 */
            /* 2.57 maps to RL_B2BIP_ECNT (bits 31:16) */
            /* Reading RL_B2BIP_ECNT returns most significant 16 bits of counter and latches lower */
            /* 16 bits at the next register address.  Note - RL_B2BIP_ECNT is a CoR register       */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_B2BIP_ECNT_W1_OFFSET);

            break;

        case ixfIEEE8023_WIS_LINE_BIP_ERR_58_OFFSET:  /* 2.58 */
            /* 2.58 maps to RL_B2BIP_ECNT (bits 31:16) */
            /* Assuming that 2.57 has been read, reading RL_B2BIP_ECNT_W1 returns least significant 16 */
            /* bits of counter.  If 2.57 has not been read, the results of this read is 0                */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_B2BIP_ECNT_W0_OFFSET);

            break;

        case ixfIEEE8023_WIS_PATH_BLK_ERR_CNT_OFFSET:  /* 2.59 */
            /* 2.59 maps to RL_B3BLK_ECNT. Note,  RL_B3BLK_ECNT is a CoR register */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_B3_BLK_ECNT_OFFSET);

            break;

        case ixfIEEE8023_WIS_SECT_BIP_ERR_CNT_OFFSET:  /* 2.60 */
            /* 2.59 maps to RL_B1BIP_ECNT.  Note,  RL_B1BIP_ECNT is a CoR register */
            regval = HAL_IXF18100_READ(pChipData->BaseAddress + ixf18100_RL_B1BIP_ECNT_OFFSET);

            break;

        /* 2.61 - 2.63 are reserved registers */

        case ixfIEEE8023_WIS_J0_TX01_OFFSET:    /* 2.64 */
        case ixfIEEE8023_WIS_J0_TX23_OFFSET:    /* 2.65 */
        case ixfIEEE8023_WIS_J0_TX45_OFFSET:    /* 2.66 */
        case ixfIEEE8023_WIS_J0_TX67_OFFSET:    /* 2.67 */
        case ixfIEEE8023_WIS_J0_TX89_OFFSET:    /* 2.68 */
        case ixfIEEE8023_WIS_J0_TX1011_OFFSET:  /* 2.69 */
        case ixfIEEE8023_WIS_J0_TX1213_OFFSET:  /* 2.70 */
        case ixfIEEE8023_WIS_J0_TX1415_OFFSET:  /* 2.71 */
            /* 2.64 - 2.71 map to TL_J0_STRA */

            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* clear trace buffer */
                trace[i] = '\0';
            
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure TL_TOH_CFG_ALL is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_TL_TOH_CFG_ALL_OFFSET, 
                              ixf18100_mTL_J0_CFG, ixf18100_J0_CFG_16_BYTE_TRC_ID);
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_TRANSMIT_J0, trace, &tlength);
            if (ixfIEEE8023Status == bb_NO_ERROR)
            {
                /* Return the two trace bytes corresponding to offset parameter.    */
                /* trace[] is character oriented, return value contains two trace   */
                /* characters.                                                      */
                i = (offset - ixfIEEE8023_WIS_J0_TX01_OFFSET) * 2;
                regval = (trace[i] << 8) | trace[i + 1];
            }
            else
                regval = 0;

            break;

        case ixfIEEE8023_WIS_J0_RX01_OFFSET:    /* 2.72 */
        case ixfIEEE8023_WIS_J0_RX23_OFFSET:    /* 2.73 */
        case ixfIEEE8023_WIS_J0_RX45_OFFSET:    /* 2.74 */
        case ixfIEEE8023_WIS_J0_RX67_OFFSET:    /* 2.75 */
        case ixfIEEE8023_WIS_J0_RX89_OFFSET:    /* 2.76 */
        case ixfIEEE8023_WIS_J0_RX1011_OFFSET:  /* 2.77 */
        case ixfIEEE8023_WIS_J0_RX1213_OFFSET:  /* 2.78 */
        case ixfIEEE8023_WIS_J0_RX1415_OFFSET:  /* 2.79 */
            /* 2.72 - 2.79 mao to RL_J0_STRA */

            /* If a WIS Reset is in progress, ignore this read request */
            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* clear trace buffer */
                trace[i] = '\0';
            
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure RL_J0_CFG is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_RL_J0_CFG_OFFSET, 
                              ixf18100_mRL_TRACE_CFG, ixf18100_RL_TRACE_CFG_16_CRC7);
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_ACCEPTED_J0, trace, &tlength);
            if (ixfStatus == bb_NO_ERROR)
            {
                /* Return the two trace bytes corresponding to offset parameter.    */
                /* trace[] is character oriented, return value contains two trace   */
                /* characters.                                                      */
                i = (offset - ixfIEEE8023_WIS_J0_RX01_OFFSET) * 2;
                regval = (trace[i] << 8) | trace[i + 1];
            }
            else
                regval = 0;

            break;

        default:
            /* This function can't return an error code so set ixfStatus */
            /* and return 0x0000 */
            ixfStatus = bb_INV_PARAMETER;
            ixfIEEE8023Status = IEEE8023_eWIS_INVALID_REG;
            regval = 0;
            break;

    } /* switch (offset) */

    return (regval);
}

/******************************************************************************
* Function : Ixf1810xWISApiWrite
*------------------------------------------------------------------------------
* Description  : Ixf18101/3 WIS API write function.  Writes Ixf18101/3
*                register(s) and/or software maintained register from the 
*                IEEE 802.3 WIS perspective.
*------------------------------------------------------------------------------
* Inputs  : *pChipData : Pointer to chip data structure
*           offset     : IEEE 802.3 WIS register offset to be written
*           regval     : Value to be written 
*
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : IEEE8023_eNO_ERROR, IEEE8023_eWIS_READONLY, IEEE8023_eWIS_RESERVED,
*           IEEE8023_eWIS_IN_RESET
*------------------------------------------------------------------------------
* Access Globals : ixfStatus is set if there is an error condition in calling
*                  an IXF API function.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
IEEE8023_Error_e
Ixf1810xWISApiWrite(bb_ChipData_t *pChipData, ushort offset, ushort regval)
{
    IEEE8023_Error_e status = IEEE8023_eNO_ERROR;
    ushort WISBitVal, i;
    ushort tlength = 0;
    bb_ChipSegment_t *pSegment, Segment;
    char trace[IEEE8023_WIS_TRACELENGTH + 1];

    /* If a WIS Reset is in progress, per 802.3 spec, not required to accept */
    /* writes to any registers until the reset is complete                   */
    if (SWReg[ixfIEEE8023_WIS_CNTL1_OFFSET] & ixfIEEE8023_mRESET)
    {
        /* WIS Reset in progress */
        return (IEEE8023_eWIS_IN_RESET);
    }

    switch (offset)
    {
        case ixfIEEE8023_WIS_CNTL1_OFFSET: /* 2.0 */
            /* 2.0.15 - Self clearing SW maintained bit indicating if a reset is in */
            /* progress. Note - for writes, this bit also maps to ixf18101/3        */
            /* SRESET.SRESETALL and GLB_RST */

            if (regval & ixfIEEE8023_mRESET)
            {
                /* Reset bit is set.  Set SW maintained bit indicating WIS is */
                /* in reset                                                   */
                SWReg[offset] |= ixfIEEE8023_mRESET;

                /* Reset SW Registers */
                Ixf1810xWISApiRegReset();

                /* Reset ixf18101/3 SONET block */
                /* First, set SRESET.SRESETALL */
                Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_SRESET_OFFSET,
                                  ixf18100_mSRESET_ALL, (0x1 << ixf18100_sSRESET_ALL));

                /* Then, clear SRESET.SRESETALL */
                Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_SRESET_OFFSET,
                                  ixf18100_mSRESET_ALL, (0x0 << ixf18100_sSRESET_ALL));

                /* Reset ixf18101/3 line interface */
                pSegment = NULL; /* NULL parameter for Ixf18100Reset() */
                ixfStatus = Ixf18100Reset(pChipData, pSegment, bb_RESET_LINE_INTF);
                if (ixfStatus == bb_NO_ERROR)
                    status = IEEE8023_eNO_ERROR;
                else
                    status = IEEE8023_eWIS_IXFAPI_ERR;

                /* Clear SW maintained reset bit to indicate WIS is out of reset */
                SWReg[offset] &= ~ixfIEEE8023_mRESET;

                /* Ignore any other bits */

                break;
            }

            /* 2.0.14 - maps to ixf18101/3 GLB_LINE.lloop_en */
            WISBitVal = (regval & ixfIEEE8023_mLOOPBACK) >> ixfIEEE8023_sLOOPBACK;
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_GLB_LINE_OFFSET,
                              ixf18100_mLLOOP_EN, (WISBitVal << ixf18100_sLLOOP_EN));

            /* 2.0.13 - SW maintained bit, writes ignored */

            /* 2.0.12 is a reserved bit */

            /* 2.0.11 maps to inverted ixf18101/3 GLB_CFG1:SONET_En */
            if (regval & ixfIEEE8023_mLOW_PWR)
            {
                /* Only disable SONET framer when bit 2.0.11 is set.  Per */
                /* 802.3 spec, low power mode is exited by resetting WIS  */
                /* (not by clearing 2.0.11)                               */

                WISBitVal = (~regval & ixfIEEE8023_mLOW_PWR) >> ixfIEEE8023_sLOW_PWR;
                Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_GLB_CFG1_OFFSET,
                                  ixf18100_mSONET_EN, (WISBitVal << ixf18100_sSONET_EN));

                /* If we disable SONET framer, by definition, we've changed */
                /* the ixf18101/3 opmode from WAN to LAN.  2.7.0 maps to    */
                /* non-inverted ixf18101/3 GLB_CFG1:SONET_En                */

            }
            /* 2.0.10:7 are reserved bits */
            /* 2.0.6 - SW maintained bit, writes ignored */
            /* 2.0.5:2 - SW maintained bits, writes ignored */
            /* 2.0.1:0 are reserved bits */

            status = IEEE8023_eNO_ERROR;
            break;

        case ixfIEEE8023_WIS_STAT1_OFFSET:      /* 2.1 - RO */
        case ixfIEEE8023_DEVICE_ID_2_OFFSET:    /* 2.2 - RO */
        case ixfIEEE8023_DEVICE_ID_3_OFFSET:    /* 2.3 - RO */
        case ixfIEEE8023_WIS_SPD_ABLTY_OFFSET:  /* 2.4 - RO */
        case ixfIEEE8023_DEV_IN_PKG_5_OFFSET:   /* 2.5 - RO */
        case ixfIEEE8023_DEV_IN_PKG_6_OFFSET:   /* 2.6 - RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        case ixfIEEE8023_WIS_CNTL2_OFFSET: /* 2.7 */
            /* 2.7.15:4 are reserved */

            /* 2.7.3 maps to WMAP_CFG.jitter_pattern_select and */
            /* GLB_LINE.lloop_pattern */

            WISBitVal = (regval & ixfIEEE8023_mTEST_PTRN) >> ixfIEEE8023_sTEST_PTRN;
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_WMAP_CFG_OFFSET,
                              ixf18100_mJITTER_PATTERN_SEL, 
                              ((WISBitVal) << ixf18100_sJITTER_PATTERN_SEL));
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_GLB_LINE_OFFSET,
                              ixf18100_mLLOOP_PATTERN, 
                              ((WISBitVal) << ixf18100_sLLOOP_PATTERN));

            /* 2.7.2 maps to WMAP_CFG.rx_jitter_test */

            WISBitVal = (regval & ixfIEEE8023_mRX_TEST_PTRN_EN) >> ixfIEEE8023_sRX_TEST_PTRN_EN;
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_WMAP_CFG_OFFSET,
                              ixf18100_mRX_JITTER_TEST, 
                              ((WISBitVal) << ixf18100_sRX_JITTER_TEST));

            /* 2.7.1 maps to WMAP_CFG.tx_jitter_test */
     
            WISBitVal = (regval & ixfIEEE8023_mTX_TEST_PTRN_EN) >> ixfIEEE8023_sTX_TEST_PTRN_EN;
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_WMAP_CFG_OFFSET,
                              ixf18100_mTX_JITTER_TEST, 
                              ((WISBitVal) << ixf18100_sTX_JITTER_TEST));

            /* 2.7.0 maps to ixf18101/3 GLB_CFG1:SONET_En */
            WISBitVal = (regval & ixfIEEE8023_mPCS_TYPE_SEL) >> ixfIEEE8023_sPCS_TYPE_SEL;
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_GLB_CFG1_OFFSET,
                              ixf18100_mSONET_EN, (WISBitVal << ixf18100_sSONET_EN));

            break;

        case ixfIEEE8023_WIS_STAT2_OFFSET:      /* 2.8 - RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        /* 2.9 not supported as ixf1810x doesn't support PRBS31 */
        case ixfIEEE8023_WIS_TP_ERR_CNT_OFFSET: 
            status = IEEE8023_eWIS_REGNOTSUPPORTED;
            break;

        case ixfIEEE8023_RESERVED10:
        case ixfIEEE8023_RESERVED11:
        case ixfIEEE8023_RESERVED12:
        case ixfIEEE8023_RESERVED13:
            /* 2.10 - 2.13 are reserved registers */
            status = IEEE8023_eWIS_RESERVED;
            break;

        case ixfIEEE8023_PKG_ID_14_OFFSET:  /* 2.14 */
        case ixfIEEE8023_PKG_ID_15_OFFSET:  /* 2.15 */
            /* 2.14 and 2.15 are RO SW maintained */
            status = IEEE8023_eWIS_READONLY;
            break;

        case ixfIEEE8023_RESERVED16:
        case ixfIEEE8023_RESERVED17:
        case ixfIEEE8023_RESERVED18:
        case ixfIEEE8023_RESERVED19:
        case ixfIEEE8023_RESERVED20:
        case ixfIEEE8023_RESERVED21:
        case ixfIEEE8023_RESERVED22:
        case ixfIEEE8023_RESERVED23:
        case ixfIEEE8023_RESERVED24:
        case ixfIEEE8023_RESERVED25:
        case ixfIEEE8023_RESERVED26:
        case ixfIEEE8023_RESERVED27:
        case ixfIEEE8023_RESERVED28:
        case ixfIEEE8023_RESERVED29:
        case ixfIEEE8023_RESERVED30:
        case ixfIEEE8023_RESERVED31:
        case ixfIEEE8023_RESERVED32:
            /* 2.16 - 2.32 are reserved registers */
            status = IEEE8023_eWIS_RESERVED;
            break;

        case ixfIEEE8023_WIS_STAT3_OFFSET:  /* 2.33 */
            /* 2.33 is RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        case ixfIEEE8023_RESERVED34:
        case ixfIEEE8023_RESERVED35:
        case ixfIEEE8023_RESERVED36:
            /* 2.34 - 2.36 are reserved registers */
            status = IEEE8023_eWIS_RESERVED;
            break;

        case ixfIEEE8023_WIS_FEP_BLK_ERR_CNT_OFFSET:  /* 2.37 */
            /* 2.37 is RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        case ixfIEEE8023_RESERVED38:
            /* 2.38 is a reserved register */
            status = IEEE8023_eWIS_RESERVED;
            break;

        case ixfIEEE8023_WIS_J1_TX01_OFFSET:    /* 2.39 */
        case ixfIEEE8023_WIS_J1_TX23_OFFSET:    /* 2.40 */
        case ixfIEEE8023_WIS_J1_TX45_OFFSET:    /* 2.41 */
        case ixfIEEE8023_WIS_J1_TX67_OFFSET:    /* 2.42 */
        case ixfIEEE8023_WIS_J1_TX89_OFFSET:    /* 2.43 */
        case ixfIEEE8023_WIS_J1_TX1011_OFFSET:  /* 2.44 */
        case ixfIEEE8023_WIS_J1_TX1213_OFFSET:  /* 2.45 */
        case ixfIEEE8023_WIS_J1_TX1415_OFFSET:  /* 2.46 */
            /* 2.39 - 2.46 map to TL_J1_STRA */

            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* Clear trace buffer */
                trace[i] = '\0';
                
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure TL_POH_CFG is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_TL_POH_CFG_OFFSET, 
                              (ixf18100_mTL_J1_CFG | ixf18100_mCRC7_EN), 
                              ((ixf18100_vTL_J1_CFG_16Byte << ixf18100_sTL_J1_CFG) | ixf18100_mCRC7_EN));
            /* Get the current Transmit J1 trace */
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_TRANSMIT_J1, trace, &tlength);

            if (ixfStatus == bb_NO_ERROR)
            {
                /* Insert the two trace bytes passed to us then call SetTrace() */
                i = (offset - ixfIEEE8023_WIS_J1_TX01_OFFSET) * 2;
                if (i < IEEE8023_WIS_TRACELENGTH)
                {
                    trace[i] = (regval & ixfIEEE8023_mJ1_TX_1) >> ixfIEEE8023_sJ1_TX_1;
                    trace[i + 1] = (regval & ixfIEEE8023_mJ1_TX_0);

                    /* Call SetTrace function to write the trace to the ixf18101/3 */
                    ixfStatus = Ixf18100SonetSetTrace(pChipData, &Segment, 
                                                      bb_TRANSMIT_J1, bb_16_BYTE_WITH_CRC7, 
                                                      trace);
                    status = (ixfStatus == bb_NO_ERROR ? IEEE8023_eNO_ERROR : IEEE8023_eWIS_IXFAPI_ERR);
                }
                else
                {
                    /* Should never encounter this condition as offsets are pre-defined */
                    status = IEEE8023_eWIS_INVALID_REG;
                }
            }
            else
                status = IEEE8023_eWIS_IXFAPI_ERR;

            break;

        case ixfIEEE8023_WIS_J1_RX01_OFFSET:            /* 2.47 - RO */
        case ixfIEEE8023_WIS_J1_RX23_OFFSET:            /* 2.48 - RO */
        case ixfIEEE8023_WIS_J1_RX45_OFFSET:            /* 2.49 - RO */
        case ixfIEEE8023_WIS_J1_RX67_OFFSET:            /* 2.50 - RO */
        case ixfIEEE8023_WIS_J1_RX89_OFFSET:            /* 2.51 - RO */
        case ixfIEEE8023_WIS_J1_RX1011_OFFSET:          /* 2.52 - RO */
        case ixfIEEE8023_WIS_J1_RX1213_OFFSET:          /* 2.53 - RO */
        case ixfIEEE8023_WIS_J1_RX1415_OFFSET:          /* 2.54 - RO */
        case ixfIEEE8023_WIS_FEL_BIP_ERR_55_OFFSET:     /* 2.55 - RO */
        case ixfIEEE8023_WIS_FEL_BIP_ERR_56_OFFSET:     /* 2.56 - RO */
        case ixfIEEE8023_WIS_LINE_BIP_ERR_57_OFFSET:    /* 2.57 - RO */
        case ixfIEEE8023_WIS_LINE_BIP_ERR_58_OFFSET:    /* 2.58 - RO */
        case ixfIEEE8023_WIS_PATH_BLK_ERR_CNT_OFFSET:   /* 2.59 - RO */
        case ixfIEEE8023_WIS_SECT_BIP_ERR_CNT_OFFSET:   /* 2.60 - RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        case ixfIEEE8023_RESERVED61:            /* 2.61 - Reserved */
        case ixfIEEE8023_RESERVED62:            /* 2.62 - Reserved */
        case ixfIEEE8023_RESERVED63:            /* 2.63 - Reserved */
            status = IEEE8023_eWIS_RESERVED;
            break;

        case ixfIEEE8023_WIS_J0_TX01_OFFSET:    /* 2.64 */
        case ixfIEEE8023_WIS_J0_TX23_OFFSET:    /* 2.65 */
        case ixfIEEE8023_WIS_J0_TX45_OFFSET:    /* 2.66 */
        case ixfIEEE8023_WIS_J0_TX67_OFFSET:    /* 2.67 */
        case ixfIEEE8023_WIS_J0_TX89_OFFSET:    /* 2.68 */
        case ixfIEEE8023_WIS_J0_TX1011_OFFSET:  /* 2.69 */
        case ixfIEEE8023_WIS_J0_TX1213_OFFSET:  /* 2.70 */
        case ixfIEEE8023_WIS_J0_TX1415_OFFSET:  /* 2.71 */
            /* 2.64 - 2.71 map to TL_J0_STRA */

            for (i = 0; i < IEEE8023_WIS_TRACELENGTH + 1; i++)  /* clear trace buffer */
                trace[i] = '\0';
                
            Segment.block.block = ixf_eSONET;   /* Indicate SONET block */    

            /* Ensure TL_TOH_CFG_ALL is configured for 16-byte trace w/ CRC7 */
            Ixf18100SetBits16((ushort *)pChipData->BaseAddress + ixf18100_TL_TOH_CFG_ALL_OFFSET, 
                              ixf18100_mTL_J0_CFG, ixf18100_J0_CFG_16_BYTE_TRC_ID);
            ixfStatus = Ixf18100SonetGetTrace(pChipData, &Segment, bb_TRANSMIT_J0, trace, &tlength);

            if (ixfStatus == bb_NO_ERROR)
            {
                /* Insert the two trace bytes passed to us then call SetTrace() */
                i = (offset - ixfIEEE8023_WIS_J0_TX01_OFFSET) * 2;
                if (i < IEEE8023_WIS_TRACELENGTH)
                {
                    trace[i] = (regval & ixfIEEE8023_mJ1_TX_1) >> ixfIEEE8023_sJ1_TX_1;
                    trace[i + 1] = (regval & ixfIEEE8023_mJ1_TX_0);

                    /* Call SetTrace function to write the trace to the ixf18101/3 */
                    ixfStatus = Ixf18100SonetSetTrace(pChipData, &Segment, 
                                                      bb_TRANSMIT_J0, bb_16_BYTE_WITH_CRC7, 
                                                      trace);
                    status = (ixfStatus == bb_NO_ERROR ? IEEE8023_eNO_ERROR : IEEE8023_eWIS_IXFAPI_ERR);
                }
                else
                {
                    /* Should never encounter this condition as offsets are pre-defined */
                    status = IEEE8023_eWIS_INVALID_REG;
                }
            }
            else
                status = IEEE8023_eWIS_IXFAPI_ERR;
            
            break;

        case ixfIEEE8023_WIS_J0_RX01_OFFSET:    /* 2.72 - RO */
        case ixfIEEE8023_WIS_J0_RX23_OFFSET:    /* 2.73 - RO */
        case ixfIEEE8023_WIS_J0_RX45_OFFSET:    /* 2.74 - RO */
        case ixfIEEE8023_WIS_J0_RX67_OFFSET:    /* 2.75 - RO */
        case ixfIEEE8023_WIS_J0_RX89_OFFSET:    /* 2.76 - RO */
        case ixfIEEE8023_WIS_J0_RX1011_OFFSET:  /* 2.77 - RO */
        case ixfIEEE8023_WIS_J0_RX1213_OFFSET:  /* 2.78 - RO */
        case ixfIEEE8023_WIS_J0_RX1415_OFFSET:  /* 2.79 - RO */
            status = IEEE8023_eWIS_READONLY;
            break;

        default:
            /* Invalid register offset */
            status = IEEE8023_eWIS_INVALID_REG;
            break;
        
    }
    return (status);
}

