/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_alarms.c $
 *   WORKFILE:       $Workfile: ixf18100_alarms.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: 
 *          
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_alarms.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 28    2/20/03 3:19p Kssaepha
 * Changed ifndef logic to ifdef to allow a combination of ixf1810x's to
 * be defined
 * 
 * 27    2/12/03 4:41p Pkjain2
 * Workaround for The J0/J1Mismatch Interrupt Status doesn't work properly
 * when the CRC7Err masks the J0/J1Mismatch
 * 
 * 26    1/30/03 4:40p Pkjain2
 * Made changes for PTQ release. Added Getstatus calls for SONET and
 * GLOBAL block.
 * 
 * 25    1/27/03 1:14p Pkjain2
 * Corrected the #ifdef's in MasterInterrptHandler code. Code was not
 * handling WIS interrupts for 18103.
 * 
 * 24    1/10/03 2:30p Pkjain2
 * 
 * 23    1/06/03 11:05a Pkjain2
 * 
 * 22    1/03/03 1:43p Pkjain2
 * 
 * 21    11/11/02 3:53p Jaalliso
 * Conditional compilation for ixf18103/4
 * 
 * 20    11/11/02 11:58a Jaalliso
 * Changes to support ixf18103 build
 * 
 * 19    10/17/02 11:35a Jaalliso
 * Replacing tab characters with spaces
 * 
 * 18    8/13/02 1:18p Pkjain2
 * 
 * 17    8/12/02 5:30p Machambe
 * Adding IXF1810x Family support
 * 
 * 16    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 15    4/16/02 2:44p Jaalliso
 * Renaming ixf18100_xgmac_b.h
 * 
 * 1     12/15/01 9:55p Srodrig1
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */
#ifdef VXWORKS
#include "vxworks.h"
#include "msgQLib.h"
#include "semLib.h"
#endif

#ifdef IXF_LINUX
#ifndef __KERNEL__
#include <semaphore.h>
#endif
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_perf_metric.h"
#include "ixf18100_global_b.h"
#ifdef __KERNEL__
#include "linuxkern.h"
#else
#include "os.h"
#endif

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
#include "ixf18100_sonet_b.h"
#include "ixf_api_sonet_d.h"
#endif

/* ixf18101/2 support POS and GFP, ixf18103/4 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB)
#include "ixf18100_pos_b.h"
#include "ixf_api_pos_d.h"
#include "ixf18100_gfp_b.h"
#include "ixf_api_gfp_d.h"
#include "ixf18100_gfp_d.h"
#include "ixf18100_gfp.h"
#endif

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
    defined(INCLUDE_18104_LIB)
#include "ixf18100_pcs_b.h"
#include "ixf18100_xgmac_b.h"
#include "ixf_api_pcs_d.h"
#include "ixf_api_ether_d.h"
#endif

#include "ixf18100_spi4_b.h"
#include "ixf18100_global_d.h"

#include "ixf18100_spi4_d.h"
#include "ixf18100_d.h"
#include "ixf18100_spi4.h"
#include "ixf18100_alarms_d.h"
#include "ixf18100_alarms.h"
#include "hal.h"


/* Local Functions */
static void GlobalClkInterruptHandler(bb_ChipData_t *pChipData, 
                                      AlarmCallBack pAlarmCallback);

static void GlobalInterruptHandler(bb_ChipData_t *pChipData, 
                                   AlarmCallBack pAlarmCallback);

static void WatchDogInterruptHandler(bb_ChipData_t *pChipData, 
                                     AlarmCallBack pAlarmCallback);

static void SifInterruptHandler(bb_ChipData_t *pChipData, 
                                AlarmCallBack pAlarmCallback);

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
static void WisInterruptHandler(bb_ChipData_t *pChipData, 
                                AlarmCallBack pAlarmCallback);

static void RxPohInterruptHandler(bb_ChipData_t *pChipData, 
                                  AlarmCallBack pAlarmCallback);

static void RxJ1InterruptHandler(bb_ChipData_t *pChipData, 
                                 AlarmCallBack pAlarmCallback);

static void SerAlInterruptHandler(bb_ChipData_t *pChipData,
                                  AlarmCallBack pAlarmCallback);

static void TxTimAlInterruptHandler(bb_ChipData_t *pChipData,
                                    AlarmCallBack pAlarmCallback);

static void RxLohInterruptHandler(bb_ChipData_t *pChipData,
                                  AlarmCallBack pAlarmCallback);

static void RxPtrInterruptHandler(bb_ChipData_t *pChipData,
                                  AlarmCallBack pAlarmCallback);

static void FrmSohInterruptHandler(bb_ChipData_t *pChipData,
                                   AlarmCallBack pAlarmCallback);

static void TTSerPrtInterruptHandler(bb_ChipData_t *pChipData,
                                     AlarmCallBack pAlarmCallback);

static void WisApsInterruptHandler(bb_ChipData_t *pChipData, 
                                   AlarmCallBack pAlarmCallback);

static void RxApsLohInterruptHandler(bb_ChipData_t *pChipData, 
                                     AlarmCallBack pAlarmCallback);

static void FrmApsSohInterruptHandler(bb_ChipData_t *pChipData, 
                                      AlarmCallBack pAlarmCallback);

#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
          defined(INCLUDE_18103_LIB) */

/* ixf18101/2 support POS and GFP, ixf18103/4 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB)
static void PosInterruptHandler(bb_ChipData_t *pChipData,
                         AlarmCallBack pAlarmCallback);

static void  GfpInterruptHandler(bb_ChipData_t *pChipData, 
                               AlarmCallBack pAlarmCallback);
#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) */

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
    defined(INCLUDE_18104_LIB)
static void PcsInterruptHandler(bb_ChipData_t *pChipData, 
                                AlarmCallBack pAlarmCallback);
#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
          defined(INCLUDE_18104_LIB) */

/** End of Local Functions **/

/******************************************************************************
* Function : EditGlbChipAlarms
*------------------------------------------------------------------------------
* Description : Edit Global Chip Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           GlobalChipIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void 
Ixf18100EditGlbChipAlarms(bb_RegPointer_type BaseAddress,
                          ushort *GlobalChipIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_CHP_INTEN_OFFSET,
        *GlobalChipIntAlarms);
}

/******************************************************************************
* Function : EditGlbClkAlarms
*------------------------------------------------------------------------------
* Description : Edit Global Clock Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           GlobalClkIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void 
Ixf18100EditGlbClkAlarms(bb_RegPointer_type BaseAddress,
                      ushort *GlobalClkIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_GLB_CLKINTEN_OFFSET,
        *GlobalClkIntAlarms);
}

/******************************************************************************
* Function : EditGlbAlarms
*------------------------------------------------------------------------------
* Description : Edit Global Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           GlobalIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditGlbAlarms(bb_RegPointer_type BaseAddress,
                      ushort *GlobalIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GLB_INTEN_OFFSET,
        *GlobalIntAlarms);
}

/******************************************************************************
* Function : EditSifAlarms
*------------------------------------------------------------------------------
* Description : Edit SIF Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           SifIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditSifAlarms(bb_RegPointer_type BaseAddress,
                      ushort *SifIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_IRPT_ENABLE_OFFSET,
        *SifIntAlarms);
}

/* ixf18101/2 support POS and GFP, ixf18103/4 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB)
/******************************************************************************
* Function : EditPosRcvAlarms
*------------------------------------------------------------------------------
* Description : Edit POS Receive Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           PosRcvIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditPosRcvAlarms(bb_RegPointer_type BaseAddress,
                         ushort *PosRcvIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_R_POSINTEN_OFFSET,
        *PosRcvIntAlarms);
}

/******************************************************************************
* Function : EditPosXmtAlarms
*------------------------------------------------------------------------------
* Description : Edit POS Transmit Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           PosXmtIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditPosXmtAlarms(bb_RegPointer_type BaseAddress,
                         ushort *PosXmtIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_T_POSINTEN_OFFSET,
        *PosXmtIntAlarms);
}

/******************************************************************************
* Function : EditGfpAlarms
*------------------------------------------------------------------------------
* Description : Edit GFP Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           GfpIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditGfpAlarms(bb_RegPointer_type BaseAddress,
                      ushort *GfpIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_GFP_INTEN_OFFSET, *GfpIntAlarms);
}

#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) */


/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)

/******************************************************************************
* Function : EditWisAlarms
*------------------------------------------------------------------------------
* Description : Edit WIS Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           WisIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditWisAlarms(bb_RegPointer_type BaseAddress,
                      ushort *WisIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_WIS_INT_EN_OFFSET,
        *WisIntAlarms);
}

/******************************************************************************
* Function : EditRlPohAlarms
*------------------------------------------------------------------------------
* Description : Edit Receive Line POH Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           RlPohIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditRlPohAlarms(bb_RegPointer_type BaseAddress,
                        ushort *RlPohIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_POH_IE_OFFSET,
        *RlPohIntAlarms);
}

/******************************************************************************
* Function : EditRlJ1Alarms
*------------------------------------------------------------------------------
* Description : Edit Receive Line J1 Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           RlJ1IntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditRlJ1Alarms(bb_RegPointer_type BaseAddress,
                       ushort *RlJ1IntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_J1_IE_OFFSET,
        *RlJ1IntAlarms);
}

/******************************************************************************
* Function : EditRlPtrAlarms
*------------------------------------------------------------------------------
* Description : Edit Receive Line Pointer Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           RlPtrIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditRlPtrAlarms(bb_RegPointer_type BaseAddress,
                        ushort *RlPtrIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_PT_IE_OFFSET,
        *RlPtrIntAlarms);
}

/******************************************************************************
* Function : EditRlLohAlarms
*------------------------------------------------------------------------------
* Description : Edit Receive Line Overhead Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           RlLohIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditRlLohAlarms(bb_RegPointer_type BaseAddress,
                        ushort *RlLohIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_LOH_IE_OFFSET,
        *RlLohIntAlarms);
}

/******************************************************************************
* Function : Ixf18100EditRlSohAlarms
*------------------------------------------------------------------------------
* Description : Edit Receive Line SOH Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           RlSohIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditRlSohAlarms(bb_RegPointer_type BaseAddress,
                        ushort *RlSohIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_SOH_IE_OFFSET,
        *RlSohIntAlarms);
}

/******************************************************************************
* Function : EditTxTimAlAlarms
*------------------------------------------------------------------------------
* Description : Edit Transmit Tim Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           TxTimAlIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditTxTimAlAlarms(bb_RegPointer_type BaseAddress,
                          ushort *TxTimAlIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_TX_TIM_AL_INT_EN_OFFSET,
        *TxTimAlIntAlarms);
}

/******************************************************************************
* Function : EditSerAlAlarms
*------------------------------------------------------------------------------
* Description : Edit Serial Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           SerAlIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditTTPrtAlarms(bb_RegPointer_type BaseAddress,
                        ushort *SerAlIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_TT_SER_PRT_INT_EN_OFFSET,
        *SerAlIntAlarms);
}
    
/******************************************************************************
* Function : EditTTSerPrt1Alarms
*------------------------------------------------------------------------------
* Description : Edit TT Serial Port 1 Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           TTSerPrt1IntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditTTSerPrt1Alarms(bb_RegPointer_type BaseAddress,
                      ushort *TTSerPrt1IntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_TT_SER_PRT_1_IE_OFFSET,
        *TTSerPrt1IntAlarms);
}
    
/******************************************************************************
* Function : EditTTSerPrt1Alarms
*------------------------------------------------------------------------------
* Description : Edit TT Serial Port 2 Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           TTSerPrt2IntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditTTSerPrt2Alarms(bb_RegPointer_type BaseAddress,
                            ushort *TTSerPrt2IntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_TT_SER_PRT_2_IE_OFFSET,
        *TTSerPrt2IntAlarms);
}

/******************************************************************************
* Function : Ixf18100EditApsWisAlarms
*------------------------------------------------------------------------------
* Description : Edit APS WIS Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ApsWisIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditApsWisAlarms(bb_RegPointer_type BaseAddress,
                         ushort *ApsWisAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_GLOBAL_INT_EN_OFFSET,
        *ApsWisAlarms);
}


/******************************************************************************
* Function : Ixf18100EditApsRlLohAlarms
*------------------------------------------------------------------------------
* Description : Edit APS Receive Line Overhead Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ApsRlLohIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditApsRlLohAlarms(bb_RegPointer_type BaseAddress,
                           ushort *ApsRlLohAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_RL_LOH_IE_OFFSET,
        *ApsRlLohAlarms);
}
   
/******************************************************************************
* Function : Ixf18100EditApsRlSohAlarms
*------------------------------------------------------------------------------
* Description : Edit APS Receive Line SOH Interrupts Enable Masks
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           ApsRlSohIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditApsRlSohAlarms(bb_RegPointer_type BaseAddress,
                           ushort *ApsRlSohAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_RL_SOH_IE_OFFSET,
        *ApsRlSohAlarms);
}


/******************************************************************************
* Function : WisInterruptHandler
*------------------------------------------------------------------------------
* Description : WIS Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
WisInterruptHandler(bb_ChipData_t *pChipData,
                    AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_WisIntStat_t   WisIntStat;

    WisIntStat.uiReg = 0;

    WisIntStat.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_WIS_INT_EN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_WIS_INT_OFFSET));

    if(WisIntStat.uiReg)
        (*pAlarmCallback)(pChipData, ixf18100_eWIS_ALARM,
            &(WisIntStat.uiReg));


    if (WisIntStat.tCnfg.uiRxPohIntSt == ixf18100_eINTERRUPT_ACTIVE)
        RxPohInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiRxJ1IntSt == ixf18100_eINTERRUPT_ACTIVE)
        RxJ1InterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiRxLohIntSt == ixf18100_eINTERRUPT_ACTIVE)
        RxLohInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiRxPtIntSt == ixf18100_eINTERRUPT_ACTIVE)
        RxPtrInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiFrmOhpIntSt == ixf18100_eINTERRUPT_ACTIVE)
         FrmSohInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiTxTimAlIntSt == ixf18100_eINTERRUPT_ACTIVE)
        TxTimAlInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiSerAlIntSt == ixf18100_eINTERRUPT_ACTIVE)
        SerAlInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiTtSerPrtIntSt == ixf18100_eINTERRUPT_ACTIVE)
        TTSerPrtInterruptHandler(pChipData, pAlarmCallback);
}

/******************************************************************************
* Function : RxPohInterruptHandler
*------------------------------------------------------------------------------
* Description : Receive Line POH Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
RxPohInterruptHandler(bb_ChipData_t *pChipData,
                      AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RxPohStatus_t  RxPohDataStatus;

    RxPohDataStatus.alarms.uiReg = 0;

    RxPohDataStatus.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_RL_POH_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_RL_POH_IS_OFFSET));

    if(RxPohDataStatus.alarms.tCnfg.uiB3BipOvFlwIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        RxPohDataStatus.B3BipCnt
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_B3_BIP_ECNT_OFFSET);
    }

    if(RxPohDataStatus.alarms.tCnfg.uiB3BlkOvFlwIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        RxPohDataStatus.B3BlkCnt
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_B3_BLK_ECNT_OFFSET);
    }

    if(RxPohDataStatus.alarms.tCnfg.uiReiBipOvFlwIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        RxPohDataStatus.ReipBipCnt
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_REIP_BIP_CNT_OFFSET);
    }

    if(RxPohDataStatus.alarms.tCnfg.uiReiBlkOvFlwIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        RxPohDataStatus.ReipBlkCnt
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_REIP_BLK_CNT_OFFSET);
    }

    RxPohDataStatus.Z5N1Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_Z5_N1_OFFSET));

    RxPohDataStatus.Z4K3Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_Z4_K3_OFFSET));

    RxPohDataStatus.Z3F3Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_Z3_F3_OFFSET));

    RxPohDataStatus.F2Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_F2_OFFSET));

    RxPohDataStatus.H4Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_H4_OFFSET));

    RxPohDataStatus.C2Byte = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_C2_OFFSET));

    RxPohDataStatus.RlRcvG1.reg = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_RCV_G1_OFFSET));

    RxPohDataStatus.RlPohStat.reg = (uchar)(HAL_IXF18100_READ(BaseAddress
        + ixf18100_RL_POH_S_OFFSET));

    if(RxPohDataStatus.alarms.uiReg)
    {
        (*pAlarmCallback)(pChipData, ixf18100_eRL_POH_ALARM, &RxPohDataStatus);
    }
}

/******************************************************************************
* Function : RxJ1InterruptHandler
*------------------------------------------------------------------------------
* Description : Receive J1 Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
RxJ1InterruptHandler(bb_ChipData_t *pChipData,
                     AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_GenPurposeIntStatus_t rxJ1IntStat;

    rxJ1IntStat.alarms = 0;
    rxJ1IntStat.status = 0;

    rxJ1IntStat.alarms
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J1_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J1_IS_OFFSET));

#ifdef IXF1810X_SW_WORKAROUND
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J1_CFG_OFFSET);

    if (((usValue >> ixf18100_sJ1_MM_ON_CRC7_ERR) & 0x01)
        && ((rxJ1IntStat.alarms >> ixf18100_sJ1_CRC7_ERR) & 0x01))
    {
        rxJ1IntStat.alarms &= ~(IXF_BIT(ixf18100_sJ1_MSMTCH));
    }
#endif

    if (rxJ1IntStat.alarms)
    {
        rxJ1IntStat.status
            |= HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J1_S_OFFSET);

#ifdef IXF1810X_SW_WORKAROUND
        if (((usValue >> ixf18100_sJ1_MM_ON_CRC7_ERR) & 0x01)
            && ((rxJ1IntStat.alarms >> ixf18100_sJ1_CRC7_ERR) & 0x01))
        {
            rxJ1IntStat.status &= ~(IXF_BIT(ixf18100_sJ1_MSMTCH_ST));
        }
#endif

        (*pAlarmCallback)(pChipData, ixf18100_eRL_J1_ALARM, &rxJ1IntStat);
    }
}

/******************************************************************************
* Function : SerAlInterruptHandler
*------------------------------------------------------------------------------
* Description : Serial Port Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
SerAlInterruptHandler(bb_ChipData_t* pChipData,
                      AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_GenPurposeIntStatus_t serialIntStat;

    serialIntStat.alarms = 0;
    serialIntStat.status = 0;

    serialIntStat.alarms
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_SER_AL_INT_EN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_SER_AL_INT_OFFSET));

    if(serialIntStat.alarms)
    {
        serialIntStat.status
            |= HAL_IXF18100_READ(BaseAddress + ixf18100_SER_AL_S_OFFSET);

        (*pAlarmCallback)(pChipData, ixf18100_eSER_AL_ALARM, &serialIntStat);
    }
}

/******************************************************************************
* Function : TxTimAlInterruptHandler
*------------------------------------------------------------------------------
* Description : Transmit Tim Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
TxTimAlInterruptHandler(bb_ChipData_t *pChipData,
                        AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_GenPurposeIntStatus_t TxTimAlIntStat;

    TxTimAlIntStat.alarms = 0;
    TxTimAlIntStat.status = 0;

    TxTimAlIntStat.alarms
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_TX_TIM_AL_INT_EN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_TX_TIM_AL_INT_OFFSET));

    if(TxTimAlIntStat.alarms)
    {
        TxTimAlIntStat.status |= HAL_IXF18100_READ(BaseAddress
            + ixf18100_TX_TIM_AL_INT_ST_OFFSET);

        (*pAlarmCallback)(pChipData, ixf18100_eTX_TIM_AL_ALARM,
            &TxTimAlIntStat);
    }
}

/******************************************************************************
* Function : RxLohInterruptHandler
*------------------------------------------------------------------------------
* Description : Receive Lone Overhead Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
RxLohInterruptHandler(bb_ChipData_t *pChipData,
                      AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RlLohStatus_t RxLohIntStat;

    RxLohIntStat.alarms.uiReg = 0;
    RxLohIntStat.B2BipOvflw = 0;
    RxLohIntStat.B2BlkOvflw = 0;
    RxLohIntStat.ReiBlkOvflw = 0;
    RxLohIntStat.ReilBipOvflw = 0;
    RxLohIntStat.RlLohStat.reg = 0 ;

    RxLohIntStat.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_RL_LOH_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_RL_LOH_S_OFFSET));

    if(RxLohIntStat.alarms.uiReg)
    {
        RxLohIntStat.K1K2Bytes.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_K2_K1_OFFSET);

        RxLohIntStat.S1Byte
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_S1_OFFSET);

        RxLohIntStat.B2BipOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_B2BIP_ECNT_W1_OFFSET) << 16);
        RxLohIntStat.B2BipOvflw
            |= (ushort)(HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_B2BIP_ECNT_W0_OFFSET));

        RxLohIntStat.B2BlkOvflw
            |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_B2BLK_ECNT_W1_OFFSET)) << 16);
        RxLohIntStat.B2BlkOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_B2BLK_ECNT_W0_OFFSET);
        
        
        RxLohIntStat.ReiBlkOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_REIL_BLK_CNT_W1_OFFSET) << 16);
        RxLohIntStat.ReiBlkOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_REIL_BLK_CNT_W0_OFFSET);

        RxLohIntStat.ReilBipOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_REIL_BIP_CNT_W1_OFFSET) << 16);
        RxLohIntStat.ReilBipOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_REIL_BIP_CNT_W0_OFFSET);

        RxLohIntStat.RlLohStat.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_LOH_S_OFFSET);
        
        (*pAlarmCallback)(pChipData, ixf18100_eRL_LOH_ALARM, &RxLohIntStat);
    }  
}

/******************************************************************************
* Function : RxPtrInterruptHandler
*------------------------------------------------------------------------------
* Description : Receive Pointer Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
RxPtrInterruptHandler(bb_ChipData_t *pChipData,
                      AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RlPtrStatus_t RxPtrIntStat;

    RxPtrIntStat.alarms.uiReg = 0;

    RxPtrIntStat.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_RL_PT_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_RL_PT_IS_OFFSET));

    if(RxPtrIntStat.alarms.uiReg)
    {
        RxPtrIntStat.LnNjCntOvflw
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_IN_NJC_OFFSET);

        RxPtrIntStat.LnPjCntOvflw
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_IN_PJC_OFFSET);

        RxPtrIntStat.RlPtrStat.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_PT_S_OFFSET);

        (*pAlarmCallback)(pChipData, ixf18100_eRL_PTR_ALARM, &RxPtrIntStat);
    }  
}

/******************************************************************************
* Function : FrmSohInterruptHandler
*------------------------------------------------------------------------------
* Description : Frame SOH Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.
******************************************************************************/
void
FrmSohInterruptHandler(bb_ChipData_t* pChipData,
                       AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RlSohStatus_t RxSohIntStat;

    RxSohIntStat.alarms.uiReg = 0;

    RxSohIntStat.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_RL_SOH_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_RL_SOH_IS_OFFSET));

#ifdef IXF1810X_SW_WORKAROUND
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J0_CFG_OFFSET);

    if (((usValue >> ixf18100_sJ0MM_ON_CRC7_ERR_DSBL) & 0x01)
        && (RxSohIntStat.alarms.tCnfg.uiJ0CrcErrIntSt))
    {
        RxSohIntStat.alarms.tCnfg.uiJ0MsMtchIntSt = 0;
    }
#endif

    if(RxSohIntStat.alarms.uiReg)
    {
        RxSohIntStat.B1BipOvflw
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_B1BIP_ECNT_OFFSET);

        RxSohIntStat.B1BlkOvflw
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_B1BLK_ECNT_OFFSET);

        RxSohIntStat.OofOvflw
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_OOF_ECNT_OFFSET);

        RxSohIntStat.RlSohStat.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_SOH_S_OFFSET);

#ifdef IXF1810X_SW_WORKAROUND
        if (((usValue >> ixf18100_sJ0MM_ON_CRC7_ERR_DSBL) & 0x01)
            && (RxSohIntStat.alarms.tCnfg.uiJ0CrcErrIntSt))
        {
            RxSohIntStat.RlSohStat.cnfg.J0MstMtchSt = 0;
        }
#endif

        (*pAlarmCallback)(pChipData, ixf18100_eRL_SOH_ALARM, &RxSohIntStat);
    }
}

/******************************************************************************
* Function : TTSerPrtInterruptHandler
*------------------------------------------------------------------------------
* Description : TT Serial Port Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
TTSerPrtInterruptHandler(bb_ChipData_t *pChipData,
                         AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_TTSerPrt1Status_t Prt1St;
    ixf18100_TTSerPrt2Status_t Prt2St;
    ixf18100_TtSerPrtIntStat_t PrtSelect;

    PrtSelect.uiReg = 0;

    PrtSelect.uiReg
        = (HAL_IXF18100_READ(BaseAddress + ixf18100_TT_SER_PRT_INT_EN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_TT_SER_PRT_INT_OFFSET));
    
    if (PrtSelect.uiReg != 0)
    {
         (*pAlarmCallback)(pChipData, ixf18100_eTT_SER_PRT_ALARM,
             &(PrtSelect.uiReg));
    }

    if(PrtSelect.tCnfg.uiTtSerPrt1IntSt == ixf18100_eINTERRUPT_ACTIVE)
    {
        Prt1St.alarms.uiReg = 0;
        Prt1St.alarms.uiReg
            = (HAL_IXF18100_READ(BaseAddress + ixf18100_TT_SER_PRT_1_IS_OFFSET)
                & HAL_IXF18100_READ(BaseAddress
                + ixf18100_TT_SER_PRT_1_IE_OFFSET));

        if(Prt1St.alarms.uiReg)
        {
            Prt1St.TTSerPrt1IntStat = HAL_IXF18100_READ(BaseAddress
                + ixf18100_TT_SER_PRT_1_S_OFFSET);

            (*pAlarmCallback)(pChipData, ixf18100_eTT_SER_PRT1_ALARM,
                &Prt1St);
        }
    }

    if(PrtSelect.tCnfg.uiTtSerPrt2IntSt == ixf18100_eINTERRUPT_ACTIVE)
    {
        Prt2St.alarms.uiReg = 0;

        Prt2St.alarms.uiReg
            = (HAL_IXF18100_READ(BaseAddress + ixf18100_TT_SER_PRT_2_IS_OFFSET)
                & HAL_IXF18100_READ(BaseAddress
                + ixf18100_TT_SER_PRT_2_IE_OFFSET));

        if(Prt2St.alarms.uiReg)
        {
            Prt2St.TTSerPrt2IntStat = HAL_IXF18100_READ(BaseAddress
                + ixf18100_TT_SER_PRT_2_S_OFFSET);

            (*pAlarmCallback)(pChipData, ixf18100_eTT_SER_PRT2_ALARM,
                &Prt2St);
        }
    }  
}

/******************************************************************************
* Function : WisApsInterruptHandler
*------------------------------------------------------------------------------
* Description : APS WIS Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
WisApsInterruptHandler(bb_ChipData_t *pChipData,
                       AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_ApsWisIntStat_t    WisIntStat;

    WisIntStat.uiReg = 0;

    WisIntStat.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_APS_GLOBAL_INT_EN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_APS_GLOBAL_INT_OFFSET));

    if(WisIntStat.uiReg)
        (*pAlarmCallback)(pChipData, ixf18100_eWIS_APS_ALARM,
            &(WisIntStat.uiReg));

    if (WisIntStat.tCnfg.uiRxLohIntSt == ixf18100_eINTERRUPT_ACTIVE)
        RxApsLohInterruptHandler(pChipData, pAlarmCallback);

    if (WisIntStat.tCnfg.uiFrmOhpIntSt == ixf18100_eINTERRUPT_ACTIVE)
         FrmApsSohInterruptHandler(pChipData, pAlarmCallback);
}

/******************************************************************************
* Function : RxApsLohInterruptHandler
*------------------------------------------------------------------------------
* Description : Receive APS LOH Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
RxApsLohInterruptHandler(bb_ChipData_t *pChipData,
                         AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RlLohStatus_t RxLohIntStat;

    RxLohIntStat.alarms.uiReg = 0;
    RxLohIntStat.B2BipOvflw = 0;
    RxLohIntStat.B2BlkOvflw = 0;
    RxLohIntStat.ReiBlkOvflw = 0;
    RxLohIntStat.ReilBipOvflw = 0;
    RxLohIntStat.RlLohStat.reg = 0 ;

    RxLohIntStat.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_LOH_IE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_LOH_IS_OFFSET));

    if(RxLohIntStat.alarms.uiReg)
    {
        RxLohIntStat.K1K2Bytes.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_K2_K1_OFFSET);

        RxLohIntStat.S1Byte
            = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_S1_OFFSET);

        RxLohIntStat.B2BipOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_B2BIP_ECNT_W1_OFFSET) << 16);
        RxLohIntStat.B2BipOvflw
            |= (ushort)(HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_B2BIP_ECNT_W0_OFFSET));

        RxLohIntStat.B2BlkOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_B2BLK_ECNT_W1_OFFSET) << 16);
        RxLohIntStat.B2BlkOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_B2BLK_ECNT_W0_OFFSET);

        RxLohIntStat.ReiBlkOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_REIL_BLK_CNT_W1_OFFSET) << 16);
        RxLohIntStat.ReiBlkOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_REIL_BLK_CNT_W0_OFFSET);

        RxLohIntStat.ReilBipOvflw
            |= (HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_REIL_BIP_CNT_W1_OFFSET) << 16);
        RxLohIntStat.ReilBipOvflw
            |= (ushort)HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_REIL_BIP_CNT_W0_OFFSET);

        RxLohIntStat.RlLohStat.reg
            = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_LOH_S_OFFSET);

        (*pAlarmCallback)(pChipData, ixf18100_eAPS_RL_LOH_ALARM,
            &RxLohIntStat);
    }  
}

/******************************************************************************
* Function : FrmApsSohInterruptHandler
*------------------------------------------------------------------------------
* Description : Frame APS SOH Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
FrmApsSohInterruptHandler(bb_ChipData_t *pChipData,
                          AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_RlSohStatus_t RxSohIntStat;

    RxSohIntStat.alarms.uiReg = 0;

    RxSohIntStat.alarms.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_SOH_IS_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_SOH_IE_OFFSET));

#ifdef IXF1810X_SW_WORKAROUND
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_J0_CFG_OFFSET);

    if (((usValue >> ixf18100_sJ0MM_ON_CRC7_ERR_DSBL) & 0x01)
        && (RxSohIntStat.alarms.tCnfg.uiJ0CrcErrIntSt))
    {
        RxSohIntStat.alarms.tCnfg.uiJ0MsMtchIntSt = 0;
    }
#endif

    if(RxSohIntStat.alarms.uiReg)
    {
        RxSohIntStat.B1BipOvflw = HAL_IXF18100_READ(BaseAddress
            + ixf18100_APS_RL_B1BIP_ECNT_OFFSET);

        RxSohIntStat.B1BlkOvflw = HAL_IXF18100_READ(BaseAddress
            + ixf18100_APS_RL_B1BLK_ECNT_OFFSET);

        RxSohIntStat.OofOvflw = HAL_IXF18100_READ(BaseAddress
            + ixf18100_APS_RL_OOF_ECNT_OFFSET);

        RxSohIntStat.RlSohStat.reg = HAL_IXF18100_READ(BaseAddress
            + ixf18100_APS_RL_SOH_S_OFFSET);

#ifdef IXF1810X_SW_WORKAROUND
        if (((usValue >> ixf18100_sJ0MM_ON_CRC7_ERR_DSBL) & 0x01)
            && (RxSohIntStat.alarms.tCnfg.uiJ0CrcErrIntSt))
        {
            RxSohIntStat.RlSohStat.cnfg.J0MstMtchSt = 0;
        }
#endif

        (*pAlarmCallback)(pChipData, ixf18100_eAPS_RL_SOH_ALARM,
            &RxSohIntStat);
    }  
}


#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
          defined(INCLUDE_18103_LIB) */

/* ixf18101/2 support POS and GFP, ixf18103/4 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB)
/******************************************************************************
* Function : GfpInterruptHandler
*------------------------------------------------------------------------------
* Description : GFP Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
GfpInterruptHandler(bb_ChipData_t *pChipData,
                    AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_GfpAlarmStatus_t gfpIntStat;

    gfpIntStat.counters.rxCtrlFrmCnt = 0;
    gfpIntStat.counters.rxEhecErrCnt = 0;
    gfpIntStat.counters.rxFcsErrCnt = 0;
    gfpIntStat.counters.rxOverSizeFrmCnt = 0;

    gfpIntStat.glbStatus
        = HAL_IXF18100_READ(BaseAddress + ixf18100_GFP_INT_OFFSET);

    if( gfpIntStat.glbStatus & ixf_BIT(ixf18100_sGFP_STATUS_INT))
    {
        gfpIntStat.status = 0;

        gfpIntStat.status
            |= (HAL_IXF18100_READ(BaseAddress + ixf18100_GFP_INTSTAT_OFFSET)
                & HAL_IXF18100_READ(BaseAddress + ixf18100_GFP_INTEN_OFFSET));

        if(gfpIntStat.status & ixf_BIT(ixf18100_sGFP_RX_CTRL_FRM_INT))
        {
            Ixf18100GfpRxGetCounters(BaseAddress,bb_GFP_RX_CTRL_INT_CNT,
                                     &(gfpIntStat.counters.rxCtrlFrmCnt));
        }
        if((gfpIntStat.status & ixf_BIT(ixf18100_sGFP_RX_THEC_ERR_INT)))
        {
            Ixf18100GfpRxGetCounters(BaseAddress,bb_GFP_RX_THEC_ERR_INT_CNT,
                                     &(gfpIntStat.counters.rxThecErrCnt));
        }

        if((gfpIntStat.status & ixf_BIT(ixf18100_sGFP_RX_EHEC_ERR_INT)))
        {
            Ixf18100GfpRxGetCounters(BaseAddress,bb_GFP_RX_EHEC_ERR_INT_CNT,
                                     &(gfpIntStat.counters.rxEhecErrCnt));
        }

        if((gfpIntStat.status & ixf_BIT(ixf18100_sGFP_RX_FCS_ERR_INT)))
        {
            Ixf18100GfpRxGetCounters(BaseAddress,bb_GFP_RX_FCS_ERR_INT_CNT,
                                     &(gfpIntStat.counters.rxFcsErrCnt));
        }

        if((gfpIntStat.status & ixf_BIT(ixf18100_sGFP_RX_OVERSIZE_FRM_INT)))
        {
            Ixf18100GfpRxGetCounters(BaseAddress,bb_GFP_RX_LARGE_FRM_CNT,
                                     &(gfpIntStat.counters.rxOverSizeFrmCnt));
        }

        (*pAlarmCallback)(pChipData, ixf18100_eGFP_ALARM, &gfpIntStat);
    }    
}

/******************************************************************************
* Function : PosInterruptHandler
*------------------------------------------------------------------------------
* Description : POS Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
PosInterruptHandler(bb_ChipData_t *pChipData,
                    AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100_PosRxAlarmStatus_t PosRxAlSt;
    ixf18100_PosTxAlarmStatus_t PosTxAlSt;
    ixf18100_PosIntSrc_t AlarmSrc;

    AlarmSrc.uiReg=  HAL_IXF18100_READ(BaseAddress
        + ixf18100_POS_INT_STATUS_OFFSET);

    if( AlarmSrc.tCnfg.uiXmtIntAct == ixf18100_eINTERRUPT_ACTIVE)
    {
        PosTxAlSt.counters.txByteCnt = 0;
        PosTxAlSt.counters.txFrmCnt = 0;
        PosTxAlSt.alarms.uiReg = 0;

        PosTxAlSt.alarms.uiReg
            = (HAL_IXF18100_READ(BaseAddress + ixf18100_T_POSINTEN_OFFSET)
                & HAL_IXF18100_READ(BaseAddress + ixf18100_T_POSINT_OFFSET));

        if(PosTxAlSt.alarms.tCnfg.uiXmtByteCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
             PosTxAlSt.counters.txByteCnt = HAL_IXF18100_READ(BaseAddress
                 + ixf18100_T_BYTECNT_W0_OFFSET);
             PosTxAlSt.counters.txByteCnt |= ((HAL_IXF18100_READ(BaseAddress
                 + ixf18100_T_BYTECNT_W1_OFFSET)) << 16);
        }

        if(PosTxAlSt.alarms.tCnfg.uiXmtFrmCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
             PosTxAlSt.counters.txFrmCnt = HAL_IXF18100_READ(BaseAddress
                 + ixf18100_T_FRMCNT_W0_OFFSET);
             PosTxAlSt.counters.txFrmCnt |= ((HAL_IXF18100_READ(BaseAddress
                 + ixf18100_T_FRMCNT_W1_OFFSET) & ixf18100_mT_FRMCNT_W1)
                    << 16);
        }

        if(PosTxAlSt.alarms.uiReg)
        {
            (*pAlarmCallback)(pChipData, ixf18100_ePOS_XMT_ALARM, &PosTxAlSt);
        }
    }

    if(AlarmSrc.tCnfg.uiRcvIntAct == ixf18100_eINTERRUPT_ACTIVE)
    {
        PosRxAlSt.counters.rxAfByteErrCnt = 0;
        PosRxAlSt.counters.rxAfCnt = 0;
        PosRxAlSt.counters.rxFcsByteErrCnt = 0;
        PosRxAlSt.counters.rxFcsCnt = 0;
        PosRxAlSt.counters.rxGfByteCnt = 0;
        PosRxAlSt.counters.rxGfCnt = 0;
        PosRxAlSt.counters.rxMaxPleCnt = 0;
        PosRxAlSt.counters.rxMinPleCnt = 0;
        PosRxAlSt.alarms.uiReg = 0;

        PosRxAlSt.alarms.uiReg
            = (HAL_IXF18100_READ(BaseAddress + ixf18100_R_POSINTEN_OFFSET)
                & HAL_IXF18100_READ(BaseAddress + ixf18100_R_POSINT_OFFSET));

        if(PosRxAlSt.alarms.tCnfg.uiRcvAfbByteCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxAfByteErrCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_AFBYTECNT_W0_OFFSET);
            PosRxAlSt.counters.rxAfByteErrCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_AFBYTECNT_W1_OFFSET)) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvAfcCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxAfCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_AFCNT_W0_OFFSET);
            PosRxAlSt.counters.rxAfCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_AFCNT_W1_OFFSET) & ixf18100_mR_AFCNT_W1) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvFcseByteCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxFcsByteErrCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_FCSEBYTECNT_W0_OFFSET);
            PosRxAlSt.counters.rxFcsByteErrCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_FCSEBYTECNT_W1_OFFSET)) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvFcsCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxFcsCnt= HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_FCSECNT_W0_OFFSET);
            PosRxAlSt.counters.rxFcsCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_FCSECNT_W1_OFFSET) & ixf18100_mR_FCSECNT_W1) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvGfcCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxGfCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_GFCNT_W0_OFFSET);
            PosRxAlSt.counters.rxGfCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_GFCNT_W1_OFFSET) & ixf18100_mR_GFCNT_W1) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvGfbByteIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxGfByteCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_GFBYTECNT_W0_OFFSET);
            PosRxAlSt.counters.rxGfByteCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_GFBYTECNT_W1_OFFSET)) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvMinPlcCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxMinPleCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_MINPLECNT_W0_OFFSET);
            PosRxAlSt.counters.rxMinPleCnt |= ((HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_MINPLECNT_W1_OFFSET)) << 16);
        }

        if(PosRxAlSt.alarms.tCnfg.uiRcvMaxPlcCntlIntSt
            == ixf18100_eINTERRUPT_ACTIVE)
        {
            PosRxAlSt.counters.rxMaxPleCnt = HAL_IXF18100_READ(BaseAddress
                + ixf18100_R_MAXPLECNT_OFFSET);
        }

        if(PosRxAlSt.alarms.uiReg)
        {
            (*pAlarmCallback)(pChipData, ixf18100_ePOS_RCV_ALARM, &PosRxAlSt);
        }
    }  
}

#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) */

/******************************************************************************
* Function : MasterInterruptHandler
*------------------------------------------------------------------------------
* Description : Master Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
ixf18100MasterInterruptHandler(bb_ChipData_t *pChipData,
                               ixf18100_GlbChpIntStat_t *glbChipStat,          
                               AlarmCallBack pAlarmCallback)
{
        
    /* Find out which interrupt went off, save the alarm type, */
    /* alarm, and status, call the callback with the acquired  */
    /* saved data                                              */

    OS_CMD_METRIC_EVENT(IXF_EVENT_MASTER_INTRHNDLR_ENTRY, 0,0);

    if (glbChipStat->uiReg)
            (*pAlarmCallback)(pChipData, ixf18100_eGLOBAL_CHIP_ALARM,
            glbChipStat);

    if(glbChipStat->tCnfg.uiClkIntSt ==   ixf18100_eINTERRUPT_ACTIVE)
        GlobalClkInterruptHandler(pChipData, pAlarmCallback);

    if(glbChipStat->tCnfg.uiGlbIntSt ==    ixf18100_eINTERRUPT_ACTIVE)
        GlobalInterruptHandler(pChipData, pAlarmCallback);

    if(glbChipStat->tCnfg.uiWatchIntSt ==  ixf18100_eINTERRUPT_ACTIVE)
        WatchDogInterruptHandler(pChipData, pAlarmCallback);

/* ixf18101/2 support POS and GFP, ixf18103/4 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB)

    if(glbChipStat->tCnfg.uiGfpIntSt ==  ixf18100_eINTERRUPT_ACTIVE)
        GfpInterruptHandler(pChipData, pAlarmCallback);

    if(glbChipStat->tCnfg.uiPosIntSt   ==  ixf18100_eINTERRUPT_ACTIVE)
        PosInterruptHandler(pChipData, pAlarmCallback);
#endif

/* ixf18101/2/3 support SONET, ixf18104 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18102_LIB) || \
    defined(INCLUDE_18103_LIB)
    if(glbChipStat->tCnfg.uiWisIntSt  ==   ixf18100_eINTERRUPT_ACTIVE)
        WisInterruptHandler(pChipData, pAlarmCallback);

    if(glbChipStat->tCnfg.uiWisApsIntSt == ixf18100_eINTERRUPT_ACTIVE)
        WisApsInterruptHandler(pChipData, pAlarmCallback);
#endif

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
    defined(INCLUDE_18104_LIB)

    if(glbChipStat->tCnfg.uiPcsIntSt  ==   ixf18100_eINTERRUPT_ACTIVE)
        PcsInterruptHandler(pChipData, pAlarmCallback);

#endif

    if(glbChipStat->tCnfg.uiSifIntSt   ==  ixf18100_eINTERRUPT_ACTIVE)
        SifInterruptHandler(pChipData, pAlarmCallback);

    OS_CMD_METRIC_EVENT(IXF_EVENT_MASTER_INTRHNDLR_EXIT, 0,0);
}

/******************************************************************************
* Function : GlobalClkInterruptHandler
*------------------------------------------------------------------------------
* Description : Global Clock Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
GlobalClkInterruptHandler(bb_ChipData_t *pChipData,
                          AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ushort  glbClkStat = 0;

    glbClkStat
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_GLB_CLKINTEN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_CLKINTSTAT_OFFSET));

    if(glbClkStat)
        (*pAlarmCallback)(pChipData, ixf18100_eGLOBAL_CLOCK_ALARM,
        &glbClkStat);
}

/******************************************************************************
* Function : GlobalInterruptHandler
*------------------------------------------------------------------------------
* Description : Global Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
GlobalInterruptHandler(bb_ChipData_t *pChipData,
                       AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ushort  glbIntStat = 0;

    glbIntStat
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_INTEN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_GLB_INTSTAT_OFFSET));

    if(glbIntStat)
        (*pAlarmCallback)(pChipData, ixf18100_eGLOBAL_ALARM, &glbIntStat);
}

/******************************************************************************
* Function : WatchDogInterruptHandler
*------------------------------------------------------------------------------
* Description : Watch Dog Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
WatchDogInterruptHandler(bb_ChipData_t *pChipData,
                         AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ushort watchDogFailAddress = 0;

    watchDogFailAddress
        |= HAL_IXF18100_READ(BaseAddress + ixf18100_sWATCHDG_ADDR);

    if(watchDogFailAddress)
        (*pAlarmCallback)(pChipData, ixf18100_eWATCHDOG_ALARM,
            &watchDogFailAddress);
}

/* ixf18101/3/4 support Ethernet, ixf18102 does not */
#if defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
    defined(INCLUDE_18104_LIB)

/******************************************************************************
* Function : EditPcsAlarms
*------------------------------------------------------------------------------
* Description : Edit PCS Interrupts Enable Mask
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           PcsIntAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditPcsAlarms(bb_RegPointer_type BaseAddress,
                      ushort *PcsIntAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_INTEREN_OFFSET,
        *PcsIntAlarms);
}

/******************************************************************************
* Function : Ixf18100EditXgmacAlarms
*------------------------------------------------------------------------------
* Description : Edit XGMAC Interrupts Enable Mask
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           XgmacAlarms : The value of the enable masks
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
Ixf18100EditXgmacAlarms(bb_RegPointer_type BaseAddress,
                        ixf18100_XgmacIntCfg_t *XgmacAlarms)
{
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_XGMAC_STATINTENB_W0_OFFSET,
                       XgmacAlarms->tReg1.uiReg);
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_XGMAC_STATINTENB_W1_OFFSET,
                       XgmacAlarms->tReg2.uiReg);
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_XGMAC_STATINTENB_W2_OFFSET,
                       XgmacAlarms->tReg3.uiReg);
}

/******************************************************************************
* Function : PcsInterruptHandler
*------------------------------------------------------------------------------
* Description : PCS Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
PcsInterruptHandler(bb_ChipData_t *pChipData,
                    AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type  BaseAddress = pChipData->BaseAddress;
    ushort  pcsIntStat = 0;

    pcsIntStat
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_INTEREN_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_INTR_OFFSET));

    if(pcsIntStat)
        (*pAlarmCallback)(pChipData, ixf18100_ePCS_ALARM, &pcsIntStat);
}

#endif /* defined(INCLUDE_18101_LIB) || defined(INCLUDE_18103_LIB) || \
          defined(INCLUDE_18104_LIB) */

/******************************************************************************
* Function : SifInterruptHandler
*------------------------------------------------------------------------------
* Description : SIF Interrupt Handler
*------------------------------------------------------------------------------
* Inputs  : BaseAddress : Pointer to the base address of the device
*           pAlarmCallback : User Alarm Callback Function Pointer
* Outputs : None.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : None.  
******************************************************************************/
void
SifInterruptHandler(bb_ChipData_t *pChipData,
                    AlarmCallBack pAlarmCallback)
{
    bb_RegPointer_type   BaseAddress = pChipData->BaseAddress;
    ixf18100Spi4IntStatus status;

    status.alarmsPresent.uiReg = 0;

    status.alarmsPresent.uiReg
        |= (HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_IRPT_ENABLE_OFFSET)
            & HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_IRPT_STAT_OFFSET));
    
    if(status.alarmsPresent.tCnfg.uiTxBusErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_TX_BUS_ERR_CNT,
            &(status.counters.txBusErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiTxParErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_TX_PAR_ERR_CNT,
            &(status.counters.txParErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiTxFullErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_TX_FULL_ERR_CNT,
            &(status.counters.txFullErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiTxNoEopErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_TX_NO_EOP_ERR_CNT,
            &(status.counters.txNoEopErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiRxBusErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_RX_BUS_ERR_CNT,
            &(status.counters.rxBusErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiRxFullErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_RX_FULL_ERR_CNT,
            &(status.counters.rxFullErrCnt));
    }

    if(status.alarmsPresent.tCnfg.uiRxDip2FailErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_RX_DIP2_ERR_CNT,
            &(status.counters.rxDip2FailErrCnt));
    }
     
    if(status.alarmsPresent.tCnfg.uiRxNoEopErrIntSt
        == ixf18100_eINTERRUPT_ACTIVE)
    {
        Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_RX_NO_EOP_ERR_CNT,
            &(status.counters.rxNoEopErrCnt));
    }

    if(status.alarmsPresent.uiReg)
        (*pAlarmCallback)(pChipData, ixf18100_eSIF_ALARM, &status);
}
