/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_gfp_b.h $
 *   WORKFILE:       $Workfile: ixf18100_gfp_b.h $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 API 
 *
 * Purpose: This file contains register and bit defines for the Generic 
 *          framing Procedure (GFP) block of registers in the ixf18100. 
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_gfp_b.h $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 13    12/02/02 2:22p Kssaepha
 * Fixed #define ixf18100_v values for GMODE
 * 
 * 12    5/16/02 4:58p Jaalliso
 * Adding value #defines for GFP modes
 * 
 * 11    3/07/02 11:23a Pkjain2
 * 
 * 10    1/21/02 2:57p Kssaepha
 * 
 * 9     1/17/02 11:05a Kssaepha
 * 
 * 8     11/20/01 3:59p Jjchin
 * Updated GFP
 * 
 * 7     11/02/01 5:09p Pkjain2
 * 
 * 6     10/23/01 3:46p Kssaepha
 * Modified block offsets
 * 
 * 5     8/28/01 4:32p Pkjain2
 * Removed some errors and changed some bit names
 * 
 * 4     8/28/01 11:59a Pkjain2
 * 
 * 3     8/28/01 10:11a Pkjain2
 * Updated to latest spec
 * 
 * 2     8/22/01 2:21p Pkjain2
 * Updated to new gfp spec.
 * 
 * 1     8/21/01 10:01a Rcordova
 * GFP Registers Bit definitions
 * 
 * --------------------------------------------------------------------------
 */

#ifndef ixf18100_GFP_B_H
#define ixf18100_GFP_B_H

#ifndef ixf18100_REG_SHIFT
#define ixf18100_REG_SHIFT  0x0
#endif

/* Block Offset for Generic Framing Procedure (GFP) */
#ifndef ixf18100_GFP_BASE
#define ixf18100_GFP_BASE		0x3400
#endif

#define GFP_REG_OFFSET(offset) ((ixf18100_GFP_BASE + offset) << ixf18100_REG_SHIFT)

/******************************************************************************/
/* GFP_CFG1 - GFP Configuration 1 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CFG1_OFFSET	GFP_REG_OFFSET(0x0000)
#define ixf18100_GFP_CFG1_DFLT		0x0040

/* -------- Bit, shift and mask defines.-------- */

/*TXPLI *PLI of Transmit Control Frame */
#define ixf18100_sGFP_TX_PLI_CTRL_FRM		1
#define ixf18100_mGFP_TX_PLI_CTRL_FRM		(0x03 << ixf18100_sGFP_TX_PLI_CTRL_FRM)

/*RPLISEN *Enables storing of PLI from Control Frame Received */
#define ixf18100_sGFP_RX_STORE_PLI			3

/*RPLISTR *Selects the value of Rx Control frame to store */
#define ixf18100_sGFP_RX_PLI_VAL_STORE		4
#define ixf18100_mGFP_RX_PLI_VAL_STORE		(0x03 << ixf18100_sGFP_RX_PLI_VAL_STORE)

/*GMODE *Programmed Mode */
#define ixf18100_sGFP_MODE					6
#define ixf18100_mGFP_MODE					(0x07 << ixf18100_sGFP_MODE)

#define ixf18100_vGFP_MODE_1                (0x01 << ixf18100_sGFP_MODE)
#define ixf18100_vGFP_MODE_2                (0x02 << ixf18100_sGFP_MODE)
#define ixf18100_vGFP_MODE_3                (0x03 << ixf18100_sGFP_MODE)
#define ixf18100_vGFP_MODE_4                (0x04 << ixf18100_sGFP_MODE)
#define ixf18100_vGFP_MODE_5                (0x05 << ixf18100_sGFP_MODE)

/*GTXRST *Reset GFP Trasmit Interface */
#define ixf18100_sGFP_TX_RESET				9

/*GRXRST *Reset GFP Receive Interface */
#define ixf18100_sGFP_RX_RESET				10

/*GCPURST *Reset GFP CPU Interface */
#define ixf18100_sGFP_CPU_RESET				11

/*TFCSRST *Trasmit FCS Reset */
#define ixf18100_sGFP_TX_FCS_RESET			12

/*RFCSRST *Receive FCS Reset */
#define ixf18100_sGFP_RX_FCS_RESET			13

/*RFSMRST *Receive FSM Reset */
#define ixf18100_sGFP_RX_FSM_RESET			14

/*GFPRST *Resets the complete GFP Interface */
#define ixf18100_sGFP_RESET   	  			15

/******************************************************************************/
/* GFP_CFG2 - GFP Configuration 2 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CFG2_OFFSET	GFP_REG_OFFSET(0x0001)
#define ixf18100_GFP_CFG2_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXPSCREN *Enables Trasmit Payload Scrambler */
#define ixf18100_sGFP_TX_PLD_SCRMB_EN		0

/*RXPDESCEN *Enables Receive Payload Descrambler  */
#define ixf18100_sGFP_RX_PLD_DESCRMB_EN		1

/*TXFCSEN *Enables appending calculated FCS */
#define ixf18100_sGFP_TX_APPEND_FCS_EN		2

/*TXFCSER *Enables inserting of FCS Errors for testing purposes */
#define ixf18100_sGFP_TX_FCS_ERR_EN			3

/*RXFCSEN *Enables processing of received FCS */
#define ixf18100_sGFP_RX_FCS_PROCESS_EN		4

/*TXCHSEN *Enables Tx Core Header scrambler control */
#define ixf18100_sGFP_TX_CHDR_SCRMB_EN		5

/*RXLOSEN *Enables Rx Loss of SYNC signal to WIS */
#define ixf18100_sGFP_RX_LOSS_SYNC_SIG_EN	6

/*RXLOSDIS *Enables discarding frames when in LOS State */
#define ixf18100_sGFP_RX_LOS_DISCARD_FRM	7

/*RXSEREN *Enables marking of erroneous frames sent to SIF */
#define ixf18100_sGFP_RX_MARK_ERR_FRM_EN	8

/*RXCHEC_EN *Enables the cHEC correction algorithm */
#define ixf18100_sGFP_RX_CHEC_ALGO_EN		9

/*RXTHECEN *Enables the tHEC correction algorithm */
#define ixf18100_sGFP_RX_THEC_ALGO_EN		10

/*RXCHSEN *Enables core hearder descrambler */
#define ixf18100_sGFP_RX_DESCRMB_HDR_TO_SIF_EN		11

/*RXCHDEN *Enables descrambled core header forwarded to SIF */
#define ixf18100_sGFP_RX_CORE_HDR_DESCRMB_EN		12

/*RXDSCPRE *Enables payload descramble in PRESYNC state */
#define ixf18100_sGFP_RX_PRESYNC_PLD_DESCRMB_EN		13

/*TXDAER *Inserts error in Transmit Data Stream */
#define ixf18100_sGFP_TX_DATA_STREAM_INS_ERR_EN		14

/*TXIDER *Inserts error in Transmit Idle Stream */
#define ixf18100_sGFP_TX_IDLE_STREAM_INS_ERR_EN		15

/******************************************************************************/
/* GFP_CFG3 - GFP Configuration 3 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CFG3_OFFSET	GFP_REG_OFFSET(0x0002)
#define ixf18100_GFP_CFG3_DFLT		0x1040

/* -------- Bit, shift and mask defines.-------- */

/*TXCFRM *Trasmit Control Frame Select */
#define ixf18100_sGFP_TX_CTRL_FRM_SEL		    0

/*GFPTXEN *Enables GFP Trasmitter */
#define ixf18100_sGFP_TX_EN					    3

/*GFPRXEN *Enables GFP Receiver */
#define ixf18100_sGFP_RX_EN					    4

/*RXMAXFRMEN *Enables marking frames erroneous when greater than max frame length */
#define ixf18100_sGFP_RX_FRM_SIZE_CHK_EN	    5

/*TXENSIFR *Enables reading from SIF FIFO */
#define ixf18100_sGFP_TX_READ_SIF_FIFO_EN	    6

/*TXCCRCEN *Enables appending of CRC to control frame */
#define ixf18100_sGFP_TX_CRC_CTRL_FRM_EN	    7

/*RXCCRCEN *Enables CRC processing of Control Frames */
#define ixf18100_sGFP_RX_CRC_CTRL_FRM_EN	    11

/*TXHCDESEN *Enables transmit core header descrambler */
#define ixf18100_sGFP_TX_CORE_HDR_DESCRMB_EN	12

/******************************************************************************/
/* GFP_CFG4 - GFP Configuration 4 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CFG4_OFFSET	GFP_REG_OFFSET(0x0003)
#define ixf18100_GFP_CFG4_DFLT		0x0101

/* -------- Bit, shift and mask defines.-------- */

/*THECERRINTPROG *Programmed number of Rx tHEC failures before generating an interrupt */
#define ixf18100_sGFP_RX_THECERRINTPROG			0
#define ixf18100_mGFP_RX_THECERRINTPROG			(0x7F << ixf18100_sGFP_RX_THECERRINTPROG)

/*THECSERREN *Include single bit tHEC corrected failures in count above */
#define ixf18100_sGFP_RX_THECSERREN				7

/*EHECERINTPROG *Programmed number of Rx eHEC failures before generating an interrupt */
#define ixf18100_sGFP_RX_EHECERINTPROG			8
#define ixf18100_mGFP_RX_EHECERINTPROG			(0xFF << ixf18100_sGFP_RX_EHECERINTPROG)

/******************************************************************************/
/* GFP_CFG5 - GFP Configuration 5 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CFG5_OFFSET	GFP_REG_OFFSET(0x0004)
#define ixf18100_GFP_CFG5_DFLT		0x0101

/* -------- Bit, shift and mask defines.-------- */

/*CONERRINTPROG *Number of control frames received before causing an interrupt */
#define ixf18100_sGFP_RX_CONERRINTPROG			0
#define ixf18100_mGFP_RX_CONERRINTPROG			(0xFF << ixf18100_sGFP_RX_CONERRINTPROG)

/*FCSERRINTPROG *Programmed number of Rx FCS failures before causing an interrupt */
#define ixf18100_sGFP_RX_FCSERRINTPROG			8
#define ixf18100_mGFP_RX_FCSERRINTPROG			(0xFF << ixf18100_sGFP_RX_FCSERRINTPROG)

/******************************************************************************/
/* GFP_STAT - GFP Status Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_STAT_OFFSET	GFP_REG_OFFSET(0x0005)
#define ixf18100_GFP_STAT_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXFSM *FSM State */
#define ixf18100_sGFP_RX_FSM_STATE_STATUS		0
#define ixf18100_mGFP_RX_FSM_STATE_STATUS		(0x03 << ixf18100_sGFP_RX_FSM_STATE_STATUS)

/*RXAIS *AIS Indication */
#define ixf18100_sGFP_RX_AIS_INDICATION			2

/*RXCSFC *Control byte of Client Signal Fail user frame */
#define ixf18100_sGFP_RX_CTRL_BYTE_CSF_FRM		8
#define ixf18100_mGFP_RX_CTRL_BYTE_CSF_FRM		(0xFF << ixf18100_sGFP_RX_CTRL_BYTE_CSF_FRM)

/******************************************************************************/
/* GFP_RXOAM1 - GFP Rx OA&M 1 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXOAM1_OFFSET	GFP_REG_OFFSET(0x0006)
#define ixf18100_GFP_RXOAM1_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXOAM1 *Bits [15:0] of received control frame */
#define ixf18100_sGFP_RX_OAM_BYTES12		0
#define ixf18100_mGFP_RX_OAM_BYTES12		(0xFFFF << ixf18100_sGFP_RX_OAM_BYTES12)

/******************************************************************************/
/* GFP_RXOAM2 - GFP Rx OA&M 2 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXOAM2_OFFSET	GFP_REG_OFFSET(0x0007)
#define ixf18100_GFP_RXOAM2_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXOAM2 *Bits [31:16] of received control frame */
#define ixf18100_sGFP_RX_OAM_BYTES34		0
#define ixf18100_mGFP_RX_OAM_BYTES34		(0xFFFF << ixf18100_sGFP_RX_OAM_BYTES34)

/******************************************************************************/
/* GFP_RXOAM3 - GFP Rx OA&M 3 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXOAM3_OFFSET	GFP_REG_OFFSET(0x0008)
#define ixf18100_GFP_RXOAM3_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXOAM3 *Bits [47:32] of received control frame */
#define ixf18100_sGFP_RX_OAM_BYTES56		0
#define ixf18100_mGFP_RX_OAM_BYTES56		(0xFFFF << ixf18100_sGFP_RX_OAM_BYTES56)

/******************************************************************************/
/* GFP_TXPL - GFP Tx PL Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXPL_OFFSET	GFP_REG_OFFSET(0x0009)
#define ixf18100_GFP_TXPL_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXLASTPPLI *Value of last transmitted PLI */
#define ixf18100_sGFP_TX_LAST_PLI		0
#define ixf18100_mGFP_TX_LAST_PLI		(0xFFFF << ixf18100_sGFP_TX_LAST_PLI)

/******************************************************************************/
/* GFP_RXPL - GFP Rx PL Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXPL_OFFSET	GFP_REG_OFFSET(0x000A)
#define ixf18100_GFP_RXPL_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXLASTPPLI *Value of last received PLI */
#define ixf18100_sGFP_RX_LAST_PLI		0
#define ixf18100_mGFP_RX_LAST_PLI		(0xFFFF << ixf18100_sGFP_RX_LAST_PLI)

/******************************************************************************/
/* GFP_TXPLTYPE - GFP Tx Payload type Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXPLTYPE_OFFSET	GFP_REG_OFFSET(0x000B)
#define ixf18100_GFP_TXPLTYPE_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXLASTPTY *Value of last transmitted payload type */
#define ixf18100_sGFP_TX_LAST_PL_TYPE		0
#define ixf18100_mGFP_TX_LAST_PL_TYPE		(0xFFFF << ixf18100_sGFP_TX_LAST_PL_TYPE)

/******************************************************************************/
/* GFP_RXPLTYPE - GFP Rx Payload type Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXPLTYPE_OFFSET	GFP_REG_OFFSET(0x000C)
#define ixf18100_GFP_RXPLTYPE_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXLASTPTY *Value of last received payload type */
#define ixf18100_sGFP_RX_LAST_PL_TYPE		0
#define ixf18100_mGFP_RX_LAST_PL_TYPE		(0xFFFF << ixf18100_sGFP_RX_LAST_PL_TYPE)

/******************************************************************************/
/* GFP_RXFSMDEL - GFP Rx FSM Delta Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFSMDEL_OFFSET	GFP_REG_OFFSET(0x000D)
#define ixf18100_GFP_RXFSMDEL_DFLT		0x0001

/* -------- Bit, shift and mask defines.-------- */

/*RXDELTA *Rx cHEC frame delineation DELTA value */
#define ixf18100_sGFP_RX_FSM_DELTA			0
#define ixf18100_mGFP_RX_FSM_DELTA			(0x0F << ixf18100_sGFP_RX_FSM_DELTA)

/******************************************************************************/
/* GFP_INTSTAT - GFP Interrupt Status Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_INTSTAT_OFFSET	GFP_REG_OFFSET(0x000E)
#define ixf18100_GFP_INTSTAT_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXCONFRM_INT *Configured number of GFP control frames received */
#define ixf18100_sGFP_RX_CTRL_FRM_INT			0

/*RXTHECERR_INT *Configured number of tHEC errors received */
#define ixf18100_sGFP_RX_THEC_ERR_INT			1

/*RXEHECERR_INT *Configured number of eHEC errors received */
#define ixf18100_sGFP_RX_EHEC_ERR_INT			2

/*RXFCSERR_INT *Configured number of FCS failures received */
#define ixf18100_sGFP_RX_FCS_ERR_INT			3

/*RXLODSYN_INT *Loss of delineation in SYNC State */
#define ixf18100_sGFP_RX_LOD_SYNC_INT			4

/*RXLODPRE_INT *Loss of delineation in PRESYNC State */
#define ixf18100_sGFP_RX_LOD_PRESYNC_INT		5

/*TXOAMCOM_INT *OA&M control frame transmission complete */
#define ixf18100_sGFP_TX_OAM_CTRL_FRM_INT		6

/*RXOAMCOM_INT *OA&M control frame reception complete */
#define ixf18100_sGFP_RX_OAM_CTRL_FRM_INT		7

/*MAXFRMEX_INT *Maximum Rx frame size exceeded */
#define ixf18100_sGFP_RX_OVERSIZE_FRM_INT		8

/******************************************************************************/
/* GFP_INTEN - GFP Interrupt Enable Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_INTEN_OFFSET	GFP_REG_OFFSET(0x000F)
#define ixf18100_GFP_INTEN_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXCONFRM_INT *Configured number of GFP control frames received */
#define ixf18100_sGFP_RX_CTRL_FRM_INT_EN			0

/*RXTHECERR_EN *Configured number of tHEC errors received */
#define ixf18100_sGFP_RX_THEC_ERR_INT_EN			1

/*RXEHECERR_EN *Configured number of eHEC errors received */
#define ixf18100_sGFP_RX_EHEC_ERR_INT_EN			2

/*RXFCSERR_EN *Configured number of FCS failures received */
#define ixf18100_sGFP_RX_FCS_ERR_INT_EN			    3

/*RXLODSYN_EN *Loss of delineation in SYNC State */
#define ixf18100_sGFP_RX_LOD_SYNC_INT_EN			4

/*RXLODPRE_EN *Loss of delineation in PRESYNC State */
#define ixf18100_sGFP_RX_LOD_PRESYNC_INT_EN		    5

/*TXOAMCOM_EN *OA&M control frame transmission complete */
#define ixf18100_sGFP_TX_OAM_CTRL_FRM_INT_EN		6

/*RXOAMCOM_EN *OA&M control frame reception complete */
#define ixf18100_sGFP_RX_OAM_CTRL_FRM_INT_EN		7

/*MAXFRMEX_EN *Maximum Rx frame size exceeded */
#define ixf18100_sGFP_RX_OVERSIZE_FRM_INT_EN		8

/******************************************************************************/
/* GFP_CNTOVR_INTEN - GFP Interrupt Enable Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CNTOVR_INTEN_OFFSET	GFP_REG_OFFSET(0x0010)
#define ixf18100_GFP_CNTOVR_INTEN_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFP_RXFRM_SBCHECFAIL_INTEN *Enable interrupt when GFP_RXFRM_SBCHECFAIL overflows  */
#define ixf18100_sGFP_RXFRM_SBCHECFAIL_INT_EN			1

/*GFP_RXFRM_MBCHEC_INTEN *Enable interrupt when GFP_RXFRM_MBCHEC overflows  */
#define ixf18100_sGFP_RXFRM_MBCHEC_INT_EN				2

/*GFP_RXFRM_SBTHECFAIL_INTEN *Enable interrupt when GFP_RXFRM_SBTHECFAIL overflows  */
#define ixf18100_sGFP_RXFRM_SBTHECFAIL_INT_EN			3

/*GFP_RXFRM_MBTHEC_INTEN *Enable interrupt when GFP_RXFRM_MBTHEC overflows  */
#define ixf18100_sGFP_RXFRM_MBTHEC_INT_EN				4

/*GFP_RXFRM_EHEC_INTEN *Enable interrupt when GFP_RXFRM_EHEC overflows  */
#define ixf18100_sGFP_RXFRM_EHEC_INT_EN					5

/*GFP_RXFRM_CCRC_INTEN *Enable interrupt when GFP_RXFRM_CCRC overflows  */
#define ixf18100_sGFP_RXFRM_CCRC_INT_EN					6

/*GFP_RXFRM_FCS_INTEN *Enable interrupt when GFP_RXFRM_FCS overflows  */
#define ixf18100_sGFP_RXFRM_FCS_INT_EN					7

/*GFP_RXCNTLFRM_INTEN *Enable interrupt when GFP_RXCNTLFRM overflows  */
#define ixf18100_sGFP_RXCNTLFRM_INT_EN					8

/*GFP_RXFRM_MAX_INTEN *Enable interrupt when GFP_RXFRM_MAX overflows  */
#define ixf18100_sGFP_RXFRM_MAX_INT_EN					9

/*GFP_RX_GDFRM_INTEN *Enable interrupt when GFP_RX_GDFRM overflows  */
#define ixf18100_sGFP_RX_GDFRM_INT_EN					10

/*GFP_TX_GDFRM_INTEN *Enable interrupt when GFP_TX_GDFRM overflows  */
#define ixf18100_sGFP_TX_GDFRM_INT_EN					11

/*GFP_TXFRMER_INTEN *Enable interrupt when GFP_TXFRMER overflows  */
#define ixf18100_sGFP_TXFRMER_INT_EN					12

/*0 Reserved */

/******************************************************************************/
/* GFP_TXOAM1 - GFP Tx OA&M 1 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXOAM1_OFFSET	GFP_REG_OFFSET(0x0011)
#define ixf18100_GFP_TXOAM1_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXOAM1 *Bits [15:0] of transmitted control frame */
#define ixf18100_sGFP_TX_OAM_BYTES12		0
#define ixf18100_mGFP_TX_OAM_BYTES12		(0xFFFF << ixf18100_sGFP_TX_OAM_BYTES12)

/******************************************************************************/
/* GFP_TXOAM2 - GFP Tx OA&M 2 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXOAM2_OFFSET	GFP_REG_OFFSET(0x0012)
#define ixf18100_GFP_TXOAM2_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXOAM2 *Bits [31:16] of transmitted control frame */
#define ixf18100_sGFP_TX_OAM_BYTES34		0
#define ixf18100_mGFP_TX_OAM_BYTES34		(0xFFFF << ixf18100_sGFP_TX_OAM_BYTES34)

/******************************************************************************/
/* GFP_TXOAM3 - GFP Tx OA&M 3 Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXOAM3_OFFSET	GFP_REG_OFFSET(0x0013)
#define ixf18100_GFP_TXOAM3_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*TXOAM3 *Bits [47:32] of transmitted control frame */
#define ixf18100_sGFP_TX_OAM_BYTES56		0
#define ixf18100_mGFP_TX_OAM_BYTES56		(0xFFFF << ixf18100_sGFP_TX_OAM_BYTES56)

/******************************************************************************/
/* GFP_FRMMAX - GFP Frm Max Size Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_FRMMAX_OFFSET	GFP_REG_OFFSET(0x0014)
#define ixf18100_GFP_FRMMAX_DFLT	0x0640

/* -------- Bit, shift and mask defines.-------- */

/*GFPMAXPKT *Maximum size of packet supported */
#define ixf18100_sGFP_MAX_FRM_SIZE		0
#define ixf18100_mGFP_MAX_FRM_SIZE		(0xFFFF << ixf18100_sGFP_MAX_FRM_SIZE)

/******************************************************************************/
/* GFP_CNTSTAT - GFP Counter Status Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_CNTSTAT_OFFSET	GFP_REG_OFFSET(0x0015)
#define ixf18100_GFP_CNTSTAT_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFP_RXFRM_SBCHECFAIL_RO *GFP_RXFRM_SBCHECFAIL rollover bit */
#define ixf18100_sGFP_RXFRM_SBCHECFAIL_RO			1

/*GFP_RXFRM_MBCHEC_RO *GFP_RXFRM_MBCHEC rollover bit */
#define ixf18100_sGFP_RXFRM_MBCHEC_RO				2

/*GFP_RXFRM_SBTHECFAIL_RO *GFP_RXFRM_SBTHECFAIL rollover bit */
#define ixf18100_sGFP_RXFRM_SBTHECFAIL_RO			3

/*GFP_RXFRM_MBTHEC_RO *GFP_RXFRM_MBTHEC rollover bit */
#define ixf18100_sGFP_RXFRM_MBTHEC_RO				4

/*GFP_RXFRM_EHEC_RO *GFP_RXFRM_EHEC rollover bit */
#define ixf18100_sGFP_RXFRM_EHEC_RO					5

/*GFP_RXFRM_CCRC_RO *GFP_RXFRM_CCRC rollover bit */
#define ixf18100_sGFP_RXFRM_CCRC_RO					6

/*GFP_RXFRM_FCS_RO *GFP_RXFRM_FCS rollover bit */
#define ixf18100_sGFP_RXFRM_FCS_RO					7

/*GFP_RXCNTLFRM_RO *GFP_RXCNTLFRM rollover bit */
#define ixf18100_sGFP_RXCNTLFRM_RO					8

/*GFP_RXFRM_MAX_RO *GFP_RXFRM_MAX rollover bit */
#define ixf18100_sGFP_RXFRM_MAX_RO					9

/*GFP_RX_GDFRM_RO *GFP_RX_GDFRM rollover bit */
#define ixf18100_sGFP_RX_GDFRM_RO					10

/*GFP_TX_GDFRM_RO *GFP_TX_GDFRM rollover bit */	
#define ixf18100_sGFP_TX_GDFRM_RO					11

/*GFP_TXFRMER_RO *GFP_TXFRMER rollover bit */
#define ixf18100_sGFP_TXFRMER_RO					12

/*0 Reserved */

/******************************************************************************/
/* GFP_EXIL34 - GFP EXI 3 and 4 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL34_OFFSET	GFP_REG_OFFSET(0x0016)
#define ixf18100_GFP_EXIL34_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI4 *EXI Length when EXI in GFP frame set to 4 */
#define ixf18100_sGFP_EXI4_LENGTH			4
#define ixf18100_mGFP_EXI4_LENGTH			(0x3F << ixf18100_sGFP_EXI4_LENGTH)

/*GFPEXI3 *EXI Length when EXI in GFP frame set to 3 */
#define ixf18100_sGFP_EXI3_LENGTH			10
#define ixf18100_mGFP_EXI3_LENGTH			(0x3F << ixf18100_sGFP_EXI3_LENGTH)

/******************************************************************************/
/* GFP_EXIL56 - GFP EXI 5 and 6 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL56_OFFSET	GFP_REG_OFFSET(0x0017)
#define ixf18100_GFP_EXIL56_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI6 *EXI Length when EXI in GFP frame set to 6 */
#define ixf18100_sGFP_EXI6_LENGTH			4
#define ixf18100_mGFP_EXI6_LENGTH			(0x3F << ixf18100_sGFP_EXI6_LENGTH)

/*GFPEXI5 *EXI Length when EXI in GFP frame set to 5 */
#define ixf18100_sGFP_EXI5_LENGTH			10
#define ixf18100_mGFP_EXI5_LENGTH			(0x3F << ixf18100_sGFP_EXI5_LENGTH)

/******************************************************************************/
/* GFP_EXIL78 - GFP EXI 7 and 8 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL78_OFFSET	GFP_REG_OFFSET(0x0018)
#define ixf18100_GFP_EXIL78_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI8 *EXI Length when EXI in GFP frame set to 8 */
#define ixf18100_sGFP_EXI8_LENGTH			4
#define ixf18100_mGFP_EXI8_LENGTH			(0x3F << ixf18100_sGFP_EXI8_LENGTH)

/*GFPEXI7 *EXI Length when EXI in GFP frame set to 7 */
#define ixf18100_sGFP_EXI7_LENGTH			10
#define ixf18100_mGFP_EXI7_LENGTH			(0x3F << ixf18100_sGFP_EXI7_LENGTH)

/******************************************************************************/
/* GFP_EXIL910 - GFP EXI 9 and 10 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL910_OFFSET	GFP_REG_OFFSET(0x0019)
#define ixf18100_GFP_EXIL910_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI10 *EXI Length when EXI in GFP frame set to 10 */
#define ixf18100_sGFP_EXI10_LENGTH			4
#define ixf18100_mGFP_EXI10_LENGTH			(0x3F << ixf18100_sGFP_EXI10_LENGTH)

/*GFPEXI9 *EXI Length when EXI in GFP frame set to 9 */
#define ixf18100_sGFP_EXI9_LENGTH			10
#define ixf18100_mGFP_EXI9_LENGTH			(0x3F << ixf18100_sGFP_EXI9_LENGTH)

/******************************************************************************/
/* GFP_EXIL1112 - GFP EXI 11 and 12 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL1112_OFFSET	GFP_REG_OFFSET(0x001A)
#define ixf18100_GFP_EXIL1112_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI12 *EXI Length when EXI in GFP frame set to 12 */
#define ixf18100_sGFP_EXI12_LENGTH			4
#define ixf18100_mGFP_EXI12_LENGTH			(0x3F << ixf18100_sGFP_EXI12_LENGTH)

/*GFPEXI11 *EXI Length when EXI in GFP frame set to 11 */
#define ixf18100_sGFP_EXI11_LENGTH			10
#define ixf18100_mGFP_EXI11_LENGTH			(0x3F << ixf18100_sGFP_EXI11_LENGTH)

/******************************************************************************/
/* GFP_EXIL1314 - GFP EXI 13 and 14 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL1314_OFFSET	GFP_REG_OFFSET(0x001B)
#define ixf18100_GFP_EXIL1314_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI14 *EXI Length when EXI in GFP frame set to 14 */
#define ixf18100_sGFP_EXI14_LENGTH			4
#define ixf18100_mGFP_EXI14_LENGTH			(0x3F << ixf18100_sGFP_EXI14_LENGTH)

/*GFPEXI13 *EXI Length when EXI in GFP frame set to 13 */
#define ixf18100_sGFP_EXI13_LENGTH			10
#define ixf18100_mGFP_EXI13_LENGTH			(0x3F << ixf18100_sGFP_EXI13_LENGTH)

/******************************************************************************/
/* GFP_EXIL15 - GFP EXI 15 Length Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_EXIL15_OFFSET	GFP_REG_OFFSET(0x001C)
#define ixf18100_GFP_EXIL15_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*GFPEXI15 *EXI Length when EXI in GFP frame set to 15 */
#define ixf18100_sGFP_EXI15_LENGTH			10
#define ixf18100_mGFP_EXI15_LENGTH			(0x3F << ixf18100_sGFP_EXI15_LENGTH)

/******************************************************************************/
/* GFP_INT - GFP Interrupt Register */

/*Offset/Default definition for register.*/
#define ixf18100_GFP_INT_OFFSET	    GFP_REG_OFFSET(0x001D)
#define ixf18100_GFP_INT_DFLT		0x0000

/* -------- Bit, shift and mask defines.-------- */
/*CNTROVRFLW_INT *Counter Overflow Indication */
#define ixf18100_sGFP_CNTR_OVRFLW_INT			0

/*GPF_STATUS_INT *Status Interrupt Indication */
#define ixf18100_sGFP_STATUS_INT			    1

/******************************************************************************/
/* GFP_RXFRM_SBCHECFAIL Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_SBCHECFAIL_W0_OFFSET	GFP_REG_OFFSET(0x0040)
#define ixf18100_GFP_RXFRM_SBCHECFAIL_W1_OFFSET	GFP_REG_OFFSET(0x0041)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_SBCHECFAIL_CNT *Single Bit cHEC failure count */
#define ixf18100_sGFP_RXFRM_SBCHECFAIL_W0		0
#define ixf18100_mGFP_RXFRM_SBCHECFAIL_W0		(0xFFFF << ixf18100_sGFP_RXFRM_SBCHECFAIL_W0)
#define ixf18100_sGFP_RXFRM_SBCHECFAIL_W1		0
#define ixf18100_mGFP_RXFRM_SBCHECFAIL_W1		(0xFFFF << ixf18100_sGFP_RXFRM_SBCHECFAIL_W1)

/******************************************************************************/
/* GFP_RXFRM_MBCHEC Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_MBCHEC_W0_OFFSET	GFP_REG_OFFSET(0x0042)
#define ixf18100_GFP_RXFRM_MBCHEC_W1_OFFSET	GFP_REG_OFFSET(0x0043)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_MBCHEC_CNT *Multi bit cHEC frame count */
#define ixf18100_sGFP_RXFRM_MBCHEC_W0		0
#define ixf18100_mGFP_RXFRM_MBCHEC_W0		(0xFFFF << ixf18100_sGFP_RXFRM_MBCHEC_W0)
#define ixf18100_sGFP_RXFRM_MBCHEC_W1		0
#define ixf18100_mGFP_RXFRM_MBCHEC_W1		(0xFFFF << ixf18100_sGFP_RXFRM_MBCHEC_W1)

/******************************************************************************/
/* GFP_RXFRM_SBTHECFAIL Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_SBTHECFAIL_W0_OFFSET	GFP_REG_OFFSET(0x0044)
#define ixf18100_GFP_RXFRM_SBTHECFAIL_W1_OFFSET	GFP_REG_OFFSET(0x0045)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_SBTHECFAIL_CNT *Single Bit THEC failure count */
#define ixf18100_sGFP_RXFRM_SBTHECFAIL_W0		0
#define ixf18100_mGFP_RXFRM_SBTHECFAIL_W0		(0xFFFF << ixf18100_sGFP_RXFRM_SBTHECFAIL_W0)
#define ixf18100_sGFP_RXFRM_SBTHECFAIL_W1		0
#define ixf18100_mGFP_RXFRM_SBTHECFAIL_W1		(0xFFFF << ixf18100_sGFP_RXFRM_SBTHECFAIL_W1)

/******************************************************************************/
/* GFP_RXFRM_MBTHEC Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_MBTHEC_W0_OFFSET	GFP_REG_OFFSET(0x0046)
#define ixf18100_GFP_RXFRM_MBTHEC_W1_OFFSET	GFP_REG_OFFSET(0x0047)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_MBTHEC_CNT *Multi bit THEC frame count */
#define ixf18100_sGFP_RXFRM_MBTHEC_W0		0
#define ixf18100_mGFP_RXFRM_MBTHEC_W0		(0xFFFF << ixf18100_sGFP_RXFRM_MBTHEC_W0)
#define ixf18100_sGFP_RXFRM_MBTHEC_W1		0
#define ixf18100_mGFP_RXFRM_MBTHEC_W1		(0xFFFF << ixf18100_sGFP_RXFRM_MBTHEC_W1)

/******************************************************************************/
/* GFP_RXFRM_EHEC Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_EHEC_W0_OFFSET	GFP_REG_OFFSET(0x0048)
#define ixf18100_GFP_RXFRM_EHEC_W1_OFFSET	GFP_REG_OFFSET(0x0049)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_EHEC_CNT *EHEC failure count */
#define ixf18100_sGFP_RXFRM_EHEC_W0		0
#define ixf18100_mGFP_RXFRM_EHEC_W0		(0xFFFF << ixf18100_sGFP_RXFRM_EHEC_W0)
#define ixf18100_sGFP_RXFRM_EHEC_W1		0
#define ixf18100_mGFP_RXFRM_EHEC_W1		(0xFFFF << ixf18100_sGFP_RXFRM_EHEC_W1)

/******************************************************************************/
/* GFP_RXFRM_CCRC Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_CCRC_W0_OFFSET	GFP_REG_OFFSET(0x004A)
#define ixf18100_GFP_RXFRM_CCRC_W1_OFFSET	GFP_REG_OFFSET(0x004B)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_CCRC_CNT *Control frame CRC failure count */
#define ixf18100_sGFP_RXFRM_CCRC_W0		0
#define ixf18100_mGFP_RXFRM_CCRC_W0		(0xFFFF << ixf18100_sGFP_RXFRM_CCRC_W0)
#define ixf18100_sGFP_RXFRM_CCRC_W1		0
#define ixf18100_mGFP_RXFRM_CCRC_W1		(0xFFFF << ixf18100_sGFP_RXFRM_CCRC_W1)

/******************************************************************************/
/* GFP_RXFRM_FCS Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_FCS_W0_OFFSET	GFP_REG_OFFSET(0x004C)
#define ixf18100_GFP_RXFRM_FCS_W1_OFFSET	GFP_REG_OFFSET(0x004D)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_FCS_CNT *Frame with FCS failure count */
#define ixf18100_sGFP_RXFRM_FCS_W0		0
#define ixf18100_mGFP_RXFRM_FCS_W0		(0xFFFF << ixf18100_sGFP_RXFRM_FCS_W0)
#define ixf18100_sGFP_RXFRM_FCS_W1		0
#define ixf18100_mGFP_RXFRM_FCS_W1		(0xFFFF << ixf18100_sGFP_RXFRM_FCS_W1)

/******************************************************************************/
/* GFP_RXCNTLFRM Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXCNTLFRM_W0_OFFSET	GFP_REG_OFFSET(0x004E)
#define ixf18100_GFP_RXCNTLFRM_W1_OFFSET	GFP_REG_OFFSET(0x004F)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXCNTLFRM_CNT *Control frames received count */
#define ixf18100_sGFP_RXCNTLFRM_W0		0
#define ixf18100_mGFP_RXCNTLFRM_W0		(0xFFFF << ixf18100_sGFP_RXCNTLFRM_W0)
#define ixf18100_sGFP_RXCNTLFRM_W1		0
#define ixf18100_mGFP_RXCNTLFRM_W1		(0xFFFF << ixf18100_sGFP_RXCNTLFRM_W1)

/******************************************************************************/
/* GFP_RXFRM_MAX Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFRM_MAX_W0_OFFSET	GFP_REG_OFFSET(0x0050)
#define ixf18100_GFP_RXFRM_MAX_W1_OFFSET	GFP_REG_OFFSET(0x0051)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RXFRM_MAX_CNT *Number of frames greater than configured maximum frame size received */
#define ixf18100_sGFP_RXFRM_MAX_W0		0
#define ixf18100_mGFP_RXFRM_MAX_W0		(0xFFFF << ixf18100_sGFP_RXFRM_MAX_W0)
#define ixf18100_sGFP_RXFRM_MAX_W1		0
#define ixf18100_mGFP_RXFRM_MAX_W1		(0xFFFF << ixf18100_sGFP_RXFRM_MAX_W1)

/******************************************************************************/
/* GFP_RX_GDFRM Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RX_GDFRM_W0_OFFSET	GFP_REG_OFFSET(0x0052)
#define ixf18100_GFP_RX_GDFRM_W1_OFFSET	GFP_REG_OFFSET(0x0053)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*RX_GDFRM_CNT *Number of good frames received */
#define ixf18100_sGFP_RX_GDFRM_W0		0
#define ixf18100_mGFP_RX_GDFRM_W0		(0xFFFF << ixf18100_sGFP_RX_GDFRM_W0)
#define ixf18100_sGFP_RX_GDFRM_W1		0
#define ixf18100_mGFP_RX_GDFRM_W1		(0xFFFF << ixf18100_sGFP_RX_GDFRM_W1)

/******************************************************************************/
/* GFP_TX_GDFRM Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TX_GDFRM_W0_OFFSET	GFP_REG_OFFSET(0x0054)
#define ixf18100_GFP_TX_GDFRM_W1_OFFSET	GFP_REG_OFFSET(0x0055)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*TX_GDFRM_CNT *Number of good frames transmitted */
#define ixf18100_sGFP_TX_GDFRM_W0		0
#define ixf18100_mGFP_TX_GDFRM_W0		(0xFFFF << ixf18100_sGFP_TX_GDFRM_W0)
#define ixf18100_sGFP_TX_GDFRM_W1		0
#define ixf18100_mGFP_TX_GDFRM_W1		(0xFFFF << ixf18100_sGFP_TX_GDFRM_W1)

/******************************************************************************/
/* GFP_TXFRMER Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_TXFRMER_W0_OFFSET	GFP_REG_OFFSET(0x0056)
#define ixf18100_GFP_TXFRMER_W1_OFFSET	GFP_REG_OFFSET(0x0057)
/*Default value not applicable to this register*/

/* -------- Bit, shift and mask defines.-------- */

/*TXFRMER_CNT *Number of frames transmitted, marked with error */
#define ixf18100_sGFP_TXFRMER_W0		0
#define ixf18100_mGFP_TXFRMER_W0		(0xFFFF << ixf18100_sGFP_TXFRMER_W0)
#define ixf18100_sGFP_TXFRMER_W1		0
#define ixf18100_mGFP_TXFRMER_W1		(0xFFFF << ixf18100_sGFP_TXFRMER_W1)

/******************************************************************************/
/* GFP_RXCNTL_CNTI Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXCNTL_CNTI_OFFSET	GFP_REG_OFFSET(0x0058)
#define ixf18100_GFP_RXCNTL_CNTI_DFLT	0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXCON_INT_CNT *Generates interupt when the received control frame count equals the number in GFP_CFG5 */
#define ixf18100_sGFP_RX_CTRL_FRM_CNT_INT		0
#define ixf18100_mGFP_RX_CTRL_FRM_CNT_INT		(0xFF << ixf18100_sGFP_RX_CTRL_FRM_CNT_INT)

/******************************************************************************/
/* GFP_RXTHECER_CNTI Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXTHECER_CNTI_OFFSET	GFP_REG_OFFSET(0x0059)
#define ixf18100_GFP_RXTHECER_CNTI_DFLT	    0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXTHECER_INT_CNT *Generates interupt when the THEC failure count equals the number in GFP_CFG4 */
#define ixf18100_sGFP_RX_THEC_ERR_CNT_INT		0
#define ixf18100_mGFP_RX_THEC_ERR_CNT_INT		(0xFF << ixf18100_sGFP_RX_THEC_ERR_CNT_INT)

/******************************************************************************/
/* GFP_RXEHECER_CNTI Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXEHECER_CNTI_OFFSET	GFP_REG_OFFSET(0x005A)
#define ixf18100_GFP_RXEHECER_CNTI_DFLT	    0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXEHECER_INT_CNT *Generates interupt when the EHEC failure count equals the number in GFP_CFG4 */
#define ixf18100_sGFP_RX_EHEC_ERR_CNT_INT		0
#define ixf18100_mGFP_RX_EHEC_ERR_CNT_INT		(0xFF << ixf18100_sGFP_RX_EHEC_ERR_CNT_INT)

/******************************************************************************/
/* GFP_RXFCSER_CNTI Register*/

/*Offset/Default definition for register.*/
#define ixf18100_GFP_RXFCSER_CNTI_OFFSET	GFP_REG_OFFSET(0x005B)
#define ixf18100_GFP_RXFCSER_CNTI_DFLT	    0x0000

/* -------- Bit, shift and mask defines.-------- */

/*RXFCSER_INT_CNT *Generates interupt when the FCS failure count equals the number in GFP_CFG4 */
#define ixf18100_sGFP_RX_FCS_ERR_CNT_INT		0
#define ixf18100_mGFP_RX_FCS_ERR_CNT_INT		(0xFF << ixf18100_sGFP_RX_FCS_ERR_CNT_INT)

/******************************************************************************/
#endif /* ixf18100_GFP_B_H */

