/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_pcs.c $
 *   WORKFILE:       $Workfile: ixf18100_pcs.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains GFP block related routines
 *
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_pcs.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 15    1/10/03 2:30p Pkjain2
 * 
 * 14    7/11/02 4:22p Pkjain2
 * Corrected some headers. Added Legal notice to some files. 
 * 
 * 13    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 12    5/29/02 9:38a Jaalliso
 * Changing bb_18100_CHIP to bb_18101_CHIP, adding 18102/3/4 chips
 * 
 * 11    4/26/02 3:39p Jaalliso
 * Fixed incorrect comment and reformatted some code
 * 
 * 3     11/15/01 9:25a Machambe
 * Changing bb_util.h to ixf_api_d.h
 * 
 * 2     8/23/01 4:12p Wrbrackn
 * Made changes suggested after review of the code with Mark.
 * 
 *
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf18100_d.h"
#include "ixf_api_pcs_d.h"
#include "ixf18100_pcs_d.h"
#include "ixf18100_pcs_b.h"
#include "ixf18100_pcs.h"
#include "hal.h"

/******************************************************************************
* Function : Ixf18100PcsSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the PCS register block by writing 
*               to every writable bit and bit-field in the PCS registers. 
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SelCfg: Selects the part of the block to be configured.
*          *ptPcsCfg: A pointer to a structure containing settings for the 
*                     PCS register block.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                  ixf18100_PcsCfg_t *ptPcsCfg)
{
    ushort usValue = 0x0;

    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }
    /* PCS_CNTL1 REGISTER
    * There is only one allowed setting on the speed select bits and 
    * bit field; the 10gbps setting.
    */

    /* The Resest condition will be the changes not affective the reset must
    be treated in a different way */

    usValue |= ((ptPcsCfg->ucLoopbackModeEn & 0x01)
        << ixf18100_sPCS_LOOPBACK);

    usValue |= ((ptPcsCfg->ucSpeedSelection1 << ixf18100_sSPEED_SELECTION1)
        & ixf18100_mSPEED_SELECTION1);

    usValue |= (ushort)((ptPcsCfg->ucPcsPowerdownEn & 0x01)
        << ixf18100_sPCS_POWERDOWN);

    usValue |= ((ptPcsCfg->ucSpeedSelection2 & 0x01)
        << ixf18100_sSPEED_SELECTION2);

    usValue |= ((ptPcsCfg->ucSpeedSelection3 & 0x01)
        << ixf18100_sSPEED_SELECTION3);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_CNTL1_OFFSET, usValue);


    /* PCS_10GCNTL2 REGISTER */
    usValue = (ushort)((ptPcsCfg->ucJitterTestPattModeEn & 0x01)
        << ixf18100_sPCS_JITTER_TEST_MODE);

    /* Port Type select bits (1:0) are read only in this register. */
    usValue |= ((ptPcsCfg->ucPortTypeSel << ixf18100_sPCS_PORT_TYPE_SEL)
        & ixf18100_mPCS_PORT_TYPE_SEL);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_10GCNTL2_OFFSET, usValue);


    /* PCS_SEED REGISTERS */
    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDA0_OFFSET,
        ptPcsCfg->uiJittTestSeedA0);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDA1_OFFSET,
        ptPcsCfg->uiJittTestSeedA1);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDA2_OFFSET,
        ptPcsCfg->uiJittTestSeedA2);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDA3_OFFSET,
        ptPcsCfg->uiJittTestSeedA3);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDB0_OFFSET,
        ptPcsCfg->uiJittTestSeedB0);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDB1_OFFSET,
        ptPcsCfg->uiJittTestSeedB1);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDB2_OFFSET,
        ptPcsCfg->uiJittTestSeedB2);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_SEEDB3_OFFSET,
        ptPcsCfg->uiJittTestSeedB3);


    /* PCS_JITCTL REGISTER */
    usValue = (ushort)((ptPcsCfg->ucTxJittTestPattEn & 0x01)
        << ixf18100_sTX_JITTER_ENABLE);

    usValue |= ((ptPcsCfg->ucRxJittTestPattEn & 0x01)
        << ixf18100_sRX_JITTER_ENABLE);

    usValue |= ((ptPcsCfg->ucJittTestPattSel_SW1_PR0 & 0x01)
        << ixf18100_sJITTER_TEST_PATTERN);

    usValue |= ((ptPcsCfg->ucJittDataPattSel_Z1_LF0 & 0x01)
        << ixf18100_sJITTER_DATA_PATTERN);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_JITCTL_OFFSET, usValue);


    /* PCS_CALY1 REGISTER */
    usValue = (ushort)((ptPcsCfg->ucPcsInitDoneSignal & 0x01)
        << ixf18100_sPCS_INIT_DONE);

    usValue |= ((ptPcsCfg->ucPcsTxLatency << ixf18100_sPCS_TX_LATENCY)
        & ixf18100_mPCS_TX_LATENCY);

    usValue |= ((ptPcsCfg->ucPcsJitCntMaxID & 0x01)
        << ixf18100_sPCS_JIT_CNT_MAX_ID);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_CALY1_OFFSET, usValue);


    /* PCS_CALY3 REGISTER */
    usValue = ((ptPcsCfg->ucPcsRxThreshold << ixf18100_sPCS_RX_THRESHOLD)
        & ixf18100_mPCS_RX_THRESHOLD);

    usValue |= ((ptPcsCfg->ucPcsRxPtrGap << ixf18100_sPCS_RX_PTR_GAP)
        & ixf18100_mPCS_RX_PTR_GAP);

    usValue |= ((ptPcsCfg->ucPcsRxFifoLoad & 0x01)
        << ixf18100_sPCS_RX_FIFO_LOAD);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_CALY3_OFFSET, usValue);


    /* PCS_CALY4 REGISTER */
    usValue = ((ptPcsCfg->ucPcsTxThreshold << ixf18100_sPCS_TX_THRESHOLD)
        & ixf18100_mPCS_TX_THRESHOLD);

    usValue |= ((ptPcsCfg->uiPcsTxPtrGap << ixf18100_sPCS_TX_PTR_GAP)
        & ixf18100_mPCS_TX_PTR_GAP);

    usValue |= ((ptPcsCfg->ucPcsTxFifoLoad & 0x01)
        << ixf18100_sPCS_TX_FIFO_LOAD);

    usValue |= ((ptPcsCfg->ucPcsTxLanIpgVal & 0x01)
        << ixf18100_s_PCS_TX_LAN_IPG_VAL);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_PCS_CALY4_OFFSET, usValue);


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsGetCfg
*------------------------------------------------------------------------------
* Description : This function gets the configuration settings of the PCS block
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SelCfg: Selects the part of the block to be configured.
*          *ptPcsCfg: A pointer to a structure that will be filled with the
*           PCS register block configuration settings.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                  ixf18100_PcsCfg_t *ptPcsCfg)
{
    ushort usValue;

    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }
    /* PCS_CNTL1 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_CNTL1_OFFSET);

    ptPcsCfg->ucResetPcsRegisters
        = (uchar)((usValue >> ixf18100_sPCS_RESET) & 0x01);

    ptPcsCfg->ucLoopbackModeEn
        = (uchar)((usValue >> ixf18100_sPCS_LOOPBACK) & 0x01);

    ptPcsCfg->ucSpeedSelection1
        = (uchar)((usValue & ixf18100_mSPEED_SELECTION1)
            >> ixf18100_sSPEED_SELECTION1);

    ptPcsCfg->ucSpeedSelection2
        = (uchar)((usValue >> ixf18100_sSPEED_SELECTION2) & 0x01);

    ptPcsCfg->ucPcsPowerdownEn
        = (uchar)((usValue >> ixf18100_sPCS_POWERDOWN) & 0x01);

    ptPcsCfg->ucSpeedSelection3
        = (uchar)((usValue >> ixf18100_sSPEED_SELECTION3) & 0x01);


    /* PCS_10GCNTL2 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_10GCNTL2_OFFSET);

    ptPcsCfg->ucJitterTestPattModeEn
        = (uchar)((usValue >> ixf18100_sPCS_JITTER_TEST_MODE) & 0x01);
 
    ptPcsCfg->ucPortTypeSel
        = (uchar)((usValue & ixf18100_mPCS_PORT_TYPE_SEL)
            >> ixf18100_sPCS_PORT_TYPE_SEL);


    /* PCS_SEED REGISTERS */
    ptPcsCfg->uiJittTestSeedA0 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDA0_OFFSET);

    ptPcsCfg->uiJittTestSeedA1 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDA1_OFFSET);

    ptPcsCfg->uiJittTestSeedA2 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDA2_OFFSET);

    ptPcsCfg->uiJittTestSeedA3 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDA3_OFFSET);

    ptPcsCfg->uiJittTestSeedB0 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDB0_OFFSET);

    ptPcsCfg->uiJittTestSeedB1 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDB1_OFFSET);

    ptPcsCfg->uiJittTestSeedB2 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDB2_OFFSET);

    ptPcsCfg->uiJittTestSeedB3 = HAL_IXF18100_READ(BaseAddress
        + ixf18100_PCS_SEEDB3_OFFSET);


    /* PCS_JITCTL REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_JITCTL_OFFSET);

    ptPcsCfg->ucTxJittTestPattEn
        = (uchar)((usValue >> ixf18100_sTX_JITTER_ENABLE) & 0x01);

    ptPcsCfg->ucRxJittTestPattEn
        = (uchar)((usValue >> ixf18100_sRX_JITTER_ENABLE) & 0x01);

    ptPcsCfg->ucJittTestPattSel_SW1_PR0
        = (uchar)((usValue >> ixf18100_sJITTER_TEST_PATTERN) & 0x01);

    ptPcsCfg->ucJittDataPattSel_Z1_LF0
        = (uchar)((usValue >> ixf18100_sJITTER_DATA_PATTERN) & 0x01);


    /* PCS_CALY1 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_CALY1_OFFSET);

    ptPcsCfg->ucPcsJitCntMaxID
        = (uchar)((usValue >> ixf18100_sPCS_JIT_CNT_MAX_ID) & 0x01);

    ptPcsCfg->ucPcsTxLatency = (uchar)((usValue & ixf18100_mPCS_TX_LATENCY)
        >> ixf18100_sPCS_TX_LATENCY);

    ptPcsCfg->ucPcsInitDoneSignal
        = (uchar)((usValue >> ixf18100_sPCS_INIT_DONE) & 0x01);


    /* PCS_CALY3 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_CALY3_OFFSET);

    ptPcsCfg->ucPcsRxThreshold = (uchar)((usValue & ixf18100_mPCS_RX_THRESHOLD)
        >> ixf18100_sPCS_RX_THRESHOLD);

    ptPcsCfg->ucPcsRxPtrGap = (uchar)((usValue & ixf18100_mPCS_RX_PTR_GAP)
        >> ixf18100_sPCS_RX_PTR_GAP);

    ptPcsCfg->ucPcsRxFifoLoad
        = (uchar)((usValue >> ixf18100_sPCS_RX_FIFO_LOAD) & 0x01);


    /* PCS_CALY4 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_CALY4_OFFSET);

    ptPcsCfg->ucPcsTxThreshold
        = (uchar)((usValue & ixf18100_mPCS_TX_THRESHOLD)
            >> ixf18100_sPCS_TX_THRESHOLD);

    ptPcsCfg->uiPcsTxPtrGap = (ushort)((usValue & ixf18100_mPCS_TX_PTR_GAP)
        >> ixf18100_sPCS_TX_PTR_GAP);

    ptPcsCfg->ucPcsTxFifoLoad
        = (uchar)((usValue >> ixf18100_sPCS_TX_FIFO_LOAD) & 0x01);

    ptPcsCfg->ucPcsTxLanIpgVal
        = (uchar)((usValue >> ixf18100_s_PCS_TX_LAN_IPG_VAL) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsSetTestCfg
*------------------------------------------------------------------------------
* Description : This function sets the bits and bit fields within the PCS 
*               register block that are associated with testing functionality
*               of the device.
*------------------------------------------------------------------------------
* Inputs : *ptChipData: A pointer to information about the device
*          *ptPcsTestCfg: A pointer to a structure that contains test 
*                         configuration settings for the device.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsSetTestCfg(bb_ChipData_t *ptChipData,
                      ixf18100_PcsTestCfg_t *ptPcsTestCfg)
{
    ushort usValue;
    ushort uiMask;
    ushort uiOriginalValue;

    /* Loopback, PCS_CNTRL1 Register */
    uiOriginalValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_CNTL1_OFFSET);

    uiMask = (0x01 << ixf18100_sPCS_LOOPBACK);

    usValue = ((ptPcsTestCfg->ucPCSLoopbackTxRx & 0x01)
        << ixf18100_sPCS_LOOPBACK);

    usValue |= (uiOriginalValue & (~uiMask));

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_CNTL1_OFFSET,
        usValue);


    /* Jitter Test Mode, PCS_10GCNTL2 Register */
    uiOriginalValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_10GCNTL2_OFFSET);

    uiMask = (0x01 << ixf18100_sPCS_JITTER_TEST_MODE);

    usValue = ((ptPcsTestCfg->ucPCSJitterTestMode & 0x01)
        << ixf18100_sPCS_JITTER_TEST_MODE);

    usValue |= (uiOriginalValue & (~uiMask));

    HAL_IXF18100_WRITE(ptChipData->BaseAddress
        + ixf18100_PCS_10GCNTL2_OFFSET, usValue);


    /* Seed Values, PCS_SEEDA Registers */
    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDA0_OFFSET,
        ptPcsTestCfg->uiPCSSeedA0);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDA1_OFFSET,
        ptPcsTestCfg->uiPCSSeedA1);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDA2_OFFSET,
        ptPcsTestCfg->uiPCSSeedA2);

    usValue = (ushort)(ptPcsTestCfg->uiPCSSeedA3 & ixf18100_mPCS_SEED_A3);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDA3_OFFSET,
        usValue);


    /* Seed Values, PCS_SEEDB Registers */
    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDB0_OFFSET,
        ptPcsTestCfg->uiPCSSeedB0);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDB1_OFFSET,
        ptPcsTestCfg->uiPCSSeedB1);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDB2_OFFSET,
        ptPcsTestCfg->uiPCSSeedB2);

    usValue = (ushort)(ptPcsTestCfg->uiPCSSeedB3 & ixf18100_mPCS_SEED_B3);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_SEEDB3_OFFSET,
        usValue);


    /* Tx and Rx Jitter Enable, Test and Data Jitter Pattern,
    PCS_JITCTL_OFFSET Register */
    uiOriginalValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_JITCTL_OFFSET);

    uiMask = (ushort)(0x01 << ixf18100_sTX_JITTER_ENABLE)
        | (0x01 << ixf18100_sRX_JITTER_ENABLE)
        | (0x01 << ixf18100_sJITTER_TEST_PATTERN)
        | (0x01 << ixf18100_sJITTER_DATA_PATTERN);

    usValue = (ushort)((ptPcsTestCfg->ucPcsTxJitterEnable & 0x01)
        << ixf18100_sTX_JITTER_ENABLE);

    usValue |= ((ptPcsTestCfg->ucPcsRxJitterEnable & 0x01)
        << ixf18100_sRX_JITTER_ENABLE);

    usValue |= ((ptPcsTestCfg->ucJittTestPattSel_SW1_PR0 & 0x01)
        << ixf18100_sJITTER_TEST_PATTERN);

    usValue |= ((ptPcsTestCfg->ucJittDataPattSel_Z1_LF0 & 0x01)
        << ixf18100_sJITTER_DATA_PATTERN);

    usValue |= (uiOriginalValue & (~uiMask));

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_JITCTL_OFFSET,
        usValue);


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsGetTestCfg
*------------------------------------------------------------------------------
* Description  : This function gets the test configuration settings of the 
*                PCS register block and stores them in a structure.
*------------------------------------------------------------------------------
* Inputs  : *ptChipData: A pointer to information about the device
*           *ptPcsTestCfg: A pointer to a structure that will be filled with 
*                          the settings used to configure the testing options 
*                          in the PCS register block. 
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsGetTestCfg(bb_ChipData_t *ptChipData,
                      ixf18100_PcsTestCfg_t *ptPcsTestCfg)
{
    ushort usValue;

    /* Loopback, PCS_CNTRL1 Register */    
    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_CNTL1_OFFSET);

    ptPcsTestCfg->ucPCSLoopbackTxRx
        = (uchar)((usValue >> ixf18100_sPCS_LOOPBACK) & 0x01);


    /* Jitter Test Mode, PCS_10GCNTL2 Register */
    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_10GCNTL2_OFFSET);

    ptPcsTestCfg->ucPCSJitterTestMode
        = (uchar)((usValue >> ixf18100_sPCS_JITTER_TEST_MODE) & 0x01);


    /* Seed Values, PCS_SEEDA Registers */
    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDA0_OFFSET);

    ptPcsTestCfg->uiPCSSeedA0 = (ushort)((usValue >> ixf18100_sPCS_SEED_A0)
        & ixf18100_mPCS_SEED_A0);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDA1_OFFSET);

    ptPcsTestCfg->uiPCSSeedA1 = (ushort)((usValue >> ixf18100_sPCS_SEED_A1)
        & ixf18100_mPCS_SEED_A1);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDA2_OFFSET);

    ptPcsTestCfg->uiPCSSeedA2 = (ushort)((usValue >> ixf18100_sPCS_SEED_A2)
        & ixf18100_mPCS_SEED_A2);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDA3_OFFSET);

    ptPcsTestCfg->uiPCSSeedA3 = (ushort)((usValue >> ixf18100_sPCS_SEED_A3)
        & ixf18100_mPCS_SEED_A3);


    /* Seed Values, PCS_SEEDB Registers */
    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDB0_OFFSET);

    ptPcsTestCfg->uiPCSSeedB0 = (ushort)((usValue >> ixf18100_sPCS_SEED_B0)
        & ixf18100_mPCS_SEED_B0);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDB1_OFFSET);

    ptPcsTestCfg->uiPCSSeedB1 = (ushort)((usValue >> ixf18100_sPCS_SEED_B1)
        & ixf18100_mPCS_SEED_B1);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDB2_OFFSET);

    ptPcsTestCfg->uiPCSSeedB2 = (ushort)((usValue >> ixf18100_sPCS_SEED_B2)
        & ixf18100_mPCS_SEED_B2);

    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_SEEDB3_OFFSET);

    ptPcsTestCfg->uiPCSSeedB3 = (ushort)((usValue >> ixf18100_sPCS_SEED_B3)
        & ixf18100_mPCS_SEED_B3);


    /* Tx and Rx Jitter Enable, Test and Data Jitter Pattern,
    PCS_JITCTL_OFFSET Register */
    usValue = HAL_IXF18100_READ(ptChipData->BaseAddress
        + ixf18100_PCS_JITCTL_OFFSET);

    ptPcsTestCfg->ucPcsTxJitterEnable
        = (uchar)((usValue >> ixf18100_sTX_JITTER_ENABLE) & 0x01);

    ptPcsTestCfg->ucPcsRxJitterEnable
        = (uchar)((usValue >> ixf18100_sRX_JITTER_ENABLE) & 0x01);

    ptPcsTestCfg->ucJittTestPattSel_SW1_PR0
        = (uchar)((usValue >> ixf18100_sJITTER_TEST_PATTERN) & 0x01);

    ptPcsTestCfg->ucJittDataPattSel_Z1_LF0
        = (uchar)((usValue >> ixf18100_sJITTER_DATA_PATTERN) & 0x01);


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsGetStatus
*------------------------------------------------------------------------------
* Description : This function gets the status configurations of the PCS 
*               register block and stores them in a structure.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SelStat: Selects the part of the block to get the status
*          *ptPcsStatus: A pointer to a structure used to store the status 
*                        information for the PCS register block.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                     ixf18100_PcsStatus_t *ptPcsStatus)
{
    ushort    usValue;

    /* PCS_STAT1 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_STAT1_OFFSET);

    ptPcsStatus->ucLocalFault
        = (uchar)((usValue >> ixf18100_sPCS_LOCAL_FAULT) & 0x01);

    ptPcsStatus->ucPcsRxLinkupEn
        = (uchar)((usValue >> ixf18100_sPCS_REC_LINK_STAT) & 0x01);

    ptPcsStatus->ucPowerdownAbility
        = (uchar)((usValue >> ixf18100_sPCS_POWERDOWN_ABILITY) & 0x01);


    /* PCS_ID1 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_ID1_OFFSET);

    ptPcsStatus->uiPcsID1
        = (ushort)((usValue & ixf18100_mPCS_ID1) >> ixf18100_sPCS_ID1);


    /* PCS_ID2 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_ID2_OFFSET);

    ptPcsStatus->uiPcsID2
        = (ushort)((usValue & ixf18100_mPCS_ID2) >> ixf18100_sPCS_ID2);


    /* PCS_SPEED_ABIL REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_SPEED_ABIL_OFFSET);

    ptPcsStatus->ucPcs10gCapable
        = (uchar)((usValue >> ixf18100_s10G_CAPABLE) & 0x01);


    /* PCS_DEVICES1 */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_DEVICES1_OFFSET);

    ptPcsStatus->ucPcsVendorSpecDevicePresent
        = (uchar)((usValue >> ixf18100_sPCS_VENDOR_SPEC) & 0x01);


    /* PCS_DEVICES2 */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_DEVICES2_OFFSET);

    ptPcsStatus->ucDTE_Present
        = (uchar)((usValue >> ixf18100_sDTE_PRESENT) & 0x01);

    ptPcsStatus->ucPHY_Present
        = (uchar)((usValue >> ixf18100_sPHY_PRESENT) & 0x01);

    ptPcsStatus->ucPcsPresent
        = (uchar)((usValue >> ixf18100_sPCS_PRESENT) & 0x01);

    ptPcsStatus->ucWIS_Present
        = (uchar)((usValue >> ixf18100_sWIS_PRESENT) & 0x01);

    ptPcsStatus->ucPMD_PMA_Present
        = (uchar)((usValue >> ixf18100_sPMD_PMA_PRESENT) & 0x01);

    ptPcsStatus->ucClause22Present
        = (uchar)((usValue >> ixf18100_sCLAUSE22_PRESENT) & 0x01);


    /* PCS_10GSTAT2 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_10GSTAT2_OFFSET);

    ptPcsStatus->ucDevicePresent = (uchar)((usValue
        & ixf18100_mDEVICE_PRESENT) >> ixf18100_sDEVICE_PRESENT);

    ptPcsStatus->ucTxLocalFaultDetected
        = (uchar)((usValue >> ixf18100_sTRANSMIT_LOCAL_FAULT) & 0x01);

    ptPcsStatus->ucRxLocalFaultDetected
        = (uchar)((usValue >> ixf18100_sRECEIVE_LOCAL_FAULT) & 0x01);

    ptPcsStatus->uc10gBaseW_Supported
        = (uchar)((usValue >> ixf18100_s10GBASE_W_CAPABLE) & 0x01);

    ptPcsStatus->uc10gBaseX_Supported
        = (uchar)((usValue >> ixf18100_s10GBASE_X_CAPABLE) & 0x01);

    ptPcsStatus->uc10gBaseR_Supported
        = (uchar)((usValue >> ixf18100_s10GBASE_R_CAPABLE) & 0x01);


    /* PCS_10GBRSTAT1 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_10GBRSTAT1_OFFSET);

    ptPcsStatus->uc10gBaseR_RxLinkStatus
        = (uchar)((usValue >> ixf18100_s10GBASE_R_RCV_LINK_STAT) & 0x01);    

    ptPcsStatus->uc10gBaseR_HiBER_Reported
        = (uchar)((usValue >> ixf18100_s10GBASE_R_PCS_HIGH_BER) & 0x01);

    ptPcsStatus->uc10gBaseR_PcsLocked
        = (uchar)((usValue >> ixf18100_s10GBASE_R_PCS_LOCK) & 0x01);


    /* PCS_10GBRSTAT2 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_10GBRSTAT2_OFFSET);

    ptPcsStatus->ucBlockLockOccurred
        = (uchar)((usValue >> ixf18100_sSTAT2_LATCHED_BLK_LOCK) & 0x01);

    ptPcsStatus->ucHiBER_Entered
        = (uchar)((usValue >> ixf18100_sSTAT2_LATCHED_HIGH_BER) & 0x01);


    /* PCS_CALY2 REGISTER */
    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PCS_CALY2_OFFSET);

    ptPcsStatus->ucWAN_ModeSignalDetect
        = (uchar)((usValue >> ixf18100_sPCS_WIS_SIG_DET) & 0x01);

    ptPcsStatus->ucLAN_ModeSignalDetect
        = (uchar)((usValue >> ixf18100_sPCS_IO_SIG_DET) & 0x01);


    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsGetCounters
*------------------------------------------------------------------------------
* Description : This function gets the PCS counter values and stores them in 
*               a structure.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SelCntr: Selctes the counter(s) to be read.
*          *ptCounters: A pointer to a structure filled with the PCS 
*                          counter values.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsGetCounters(bb_RegPointer_type BaseAddress,
                       bb_SelCounters_e SelCntr, void *ptCounters)
{
    ushort usValue;
    uchar *uByteValue;
    ushort *uWordValue;
    ixfApi_PcsCounters_t *ptPcsCounters;

    if(ptCounters == NULL)
    {
        return bb_NULL_ARG;
    }

    switch(SelCntr)
    {
        case bb_PCS_COUNTERS:
            ptPcsCounters = (ixfApi_PcsCounters_t *) ptCounters;

            usValue = HAL_IXF18100_READ(BaseAddress
                + ixf18100_PCS_10GBRSTAT2_OFFSET);

            ptPcsCounters->ucBER_Counter
                = (uchar)((usValue & ixf18100_mSTAT2_BER_CNTR)
                    >> ixf18100_sSTAT2_BER_CNTR);

            ptPcsCounters->ucErrorredBlocksCounter
                = (uchar)((usValue & ixf18100_mSTAT2_ERR_BLK_CNTR)
                    >> ixf18100_sSTAT2_ERR_BLK_CNTR);

            ptPcsCounters->uiJitterErrorCount = HAL_IXF18100_READ(BaseAddress
                + ixf18100_PCS_JIT_COUNT_OFFSET);
            break;

        case bb_PCS_ERR_BLK_CNT:
            uByteValue = (uchar *) ptCounters;

            usValue = HAL_IXF18100_READ(BaseAddress
                + ixf18100_PCS_10GBRSTAT2_OFFSET);

            *uByteValue = (uchar)((usValue & ixf18100_mSTAT2_ERR_BLK_CNTR)
                >> ixf18100_sSTAT2_ERR_BLK_CNTR);
            break;

        case bb_PCS_BER_CNT:
            uByteValue = (uchar *) ptCounters;

            usValue = HAL_IXF18100_READ(BaseAddress
                + ixf18100_PCS_10GBRSTAT2_OFFSET);

            *uByteValue = (uchar)((usValue & ixf18100_mSTAT2_BER_CNTR)
                >> ixf18100_sSTAT2_BER_CNTR);
            break;

         case bb_PCS_JITTER_ERR_CNT:
            uWordValue = (ushort *) ptCounters;

            *uWordValue = HAL_IXF18100_READ(BaseAddress
                + ixf18100_PCS_JIT_COUNT_OFFSET);
            break;

         default:
             return bb_INV_SEL_COUNTERS;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PcsReset
*------------------------------------------------------------------------------
* Description : This function resets the PCS block
*------------------------------------------------------------------------------
* Inputs : ptChipData : Pointer to information about the device
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PcsReset(bb_ChipData_t *ptChipData)
{
    ushort usValue = (ushort)((0x01) << ixf18100_sPCS_RESET);

    HAL_IXF18100_WRITE(ptChipData->BaseAddress + ixf18100_PCS_CNTL1_OFFSET,
        usValue);

    return bb_NO_ERROR;
}
