/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *
 *                        INTEL  CORPORATION                            
 *
 *                     2200 MISSION COLLEGE BLVD                        
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_pos.c $
 *   WORKFILE:       $Workfile: ixf18100_pos.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains POS block related routines
 *
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_pos.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 15    1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 14    1/09/03 9:58a Jaalliso
 * Detabbing
 * 
 * 13    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 12    4/15/02 4:52p Pkjain2
 * Cleaned up some if 0 code.
 * 
 * 2     8/17/01 2:34p Pkjain2
 * Updated Code to conform to spec as of 8/17
 * 
 * 1     8/15/01 8:35a Pkjain2
 * POS Code
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf18100_pos_b.h"
#include "ixf_api_pos_d.h"
#include "ixf18100_pos_d.h"
#include "ixf18100_d.h"
#include "hal.h"


ulong Ixf18100PosRead32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb);


/******************************************************************************
* Function : Ixf18100PosRxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS Rx config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosRxCfg_t *ptPosRxCfg: pointer to the values that need
*               to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosRxSetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosRxCfg_t *ptPosRxCfg)
{

    ushort      uiValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosRxSetCfg");

    uiValue = ((ptPosRxCfg->ucPosRxHdlcFrmDescrambleEn & 0x01)
        << ixf18100_sPOS_RX_DESCRABLER_EN);

    uiValue |= ((ptPosRxCfg->ucPosWriteHdlcAddrInRxFifoEn & 0x01)
        << ixf18100_sPOS_RX_AC_PASS_EN);

    uiValue |= (ixf18100_mPOS_RX_FCS_CNF & (ptPosRxCfg->ucPosRxFcsType
        << ixf18100_sPOS_RX_FCS_CNF));

    uiValue |= ((ptPosRxCfg->ucPosRxCheckHdlcAddrEn & 0x01)
        << ixf18100_sPOS_RX_AC_CHECK_EN);

    uiValue |= ((ptPosRxCfg->ucPosRxDiscardSmallPkt & 0x01)
        << ixf18100_sPOS_RX_MIN_PLD_EN);

    uiValue |= ((ptPosRxCfg->ucPosRxDiscardLargePkt & 0x01)
        << ixf18100_sPOS_RX_MAX_PLD_EN);

    uiValue |= ((ptPosRxCfg->ucPosRxDescrmbAfterByteDestuffingEn & 0x01)
        << ixf18100_sPOS_RX_TR_DESCRAMBLER_EN);

    uiValue |= ((ptPosRxCfg->ucPosWriteHdlcFcsFieldToRxFifo & 0x01)
        << ixf18100_sPOS_RX_FCS_PASS_EN);

    uiValue |= ((ptPosRxCfg->ucPosRxMarkFrameWithFcsErrorsEn & 0x01)
        << ixf18100_sPOS_RX_FCS_ERR_EN);

    uiValue |= ((ptPosRxCfg->ucPosRxReadPktFromWIS & 0x01)
        << ixf18100_sPOS_RX_READ_EN);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_R_PHCCNF_OFFSET, uiValue);


    uiValue = (ixf18100_mPOS_RX_MIN_PKT_LEN & (ptPosRxCfg->ucPosRxMinPktLength
        << ixf18100_sPOS_RX_MIN_PKT_LEN));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_R_MINPL_OFFSET, uiValue);


    uiValue = (ixf18100_mPOS_RX_MAX_PKT_LEN & (ptPosRxCfg->uiPosRxMaxPktLength
        << ixf18100_sPOS_RX_MAX_PKT_LEN));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_R_MAXPL_OFFSET, uiValue);

    IXF_FUNC_EXIT_TRC("Ixf18100PosRxSetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}


/******************************************************************************
* Function : Ixf18100PosTxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS Tx config structure and
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosTxCfg_t *ptPosTxCfg: pointer to the values that need
*               to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosTxSetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosTxCfg_t *ptPosTxCfg)
{
    ushort      uiValue;
    bb_Error_e  rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosTxSetCfg");

    uiValue = ((ptPosTxCfg->ucPosTxHdlcFrmScrambleEn & 0x01)
        << ixf18100_sPOS_TX_SCRABLER_EN);

    uiValue |= ((ptPosTxCfg->ucPosTxHdlcAddrCtrlEn & 0x01)
        << ixf18100_sPOS_TX_AC_PASS_EN);

    uiValue |= (ixf18100_mPOS_TX_FCS_CNF & (ptPosTxCfg->ucPosTxFcsType
        << ixf18100_sPOS_TX_FCS_CNF));

    uiValue |= ((ptPosTxCfg->ucPosTxInsertFcsErrEn & 0x01)
        << ixf18100_sPOS_TX_FCS_ERR_EN);

    uiValue |= ((ptPosTxCfg->ucPosTxFcsErrInAllFrmEn & 0x01)
        << ixf18100_sPOS_TX_FCS_ERR_CNF_EN);

    uiValue |= ((ptPosTxCfg->ucPosTxFifoPktReadEn & 0x01)
        << ixf18100_sPOS_TX_PKT_READ_EN);

    uiValue |= ((ptPosTxCfg->ucPosTxScrmDataBeforeByteStuffingEn & 0x01)
        << ixf18100_sPOS_TX_TR_SCRAMBLER_EN);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_T_PHCCNF_OFFSET, uiValue);

    /*Inter packet relative gap has 3 bits to be set in the register based on 
    **the enumeration provided in the config structure. If the enumerations 
    **says Disable Relative IPG, then we set the Enable bit to 0 and don't
    **care about the rest two fields. Otherwise we set the Enable bit to 1 
    ** and set the other two fields.*/
    if (e_REL_PKT_GAP_DISABLE == ptPosTxCfg->ePosTxInterPktRelativeGap)
    {

        uiValue = (0x00 << ixf18100_sPOS_TX_IPG_REL_EN);
    }
    else
    {

        uiValue = (0x01 << ixf18100_sPOS_TX_IPG_REL_EN);

        if ((ptPosTxCfg->ePosTxInterPktRelativeGap >= e_PKT_LEN_MULT_1) 
            && (ptPosTxCfg->ePosTxInterPktRelativeGap <= e_PKT_LEN_MULT_128))
        {

            uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_REL_CNF);
        }

        switch(ptPosTxCfg->ePosTxInterPktRelativeGap)
        {

            case e_PKT_LEN_MULT_1:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x00
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_2:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x01
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_4:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x02
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_8:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x03
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_16:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x04
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_32:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x05
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_64:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x06
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_MULT_128:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x07
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_MAX_PKTLEN_1:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x00
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_2:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x01
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_4:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x02
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_8:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x03
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_16:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x04
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_32:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x05
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_64:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x06
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            case e_PKT_LEN_DIV_128:
                uiValue |= (ixf18100_mPOS_TX_IPG_REL & (0x07
                    << ixf18100_sPOS_TX_IPG_REL));
                break;

            default:

                rc =  bb_INV_PARAMETER;
        }
    }

    /*Absolute inter packet gap can have valus form 0 to 65536. If the value 
    **specified is 0 then it means that absolute IPG is disabled. If the value 
    ** specified is between 1 to 255, we store it as it is. Value above 256 
    **upto 65536, are stored in multiples of 256. Any value above 256 which 
    **is not a multiple will be converted to a lower multiple value of 256.
    **The value of 65536 is stored as 0 in the 8 bits.*/
    if (0 == ptPosTxCfg->ulPosTxInterPktAbsoluteGap)
    {

        uiValue |= (0x00 << ixf18100_sPOS_TX_IPG_ABS_EN);
    }
    else if ((ptPosTxCfg->ulPosTxInterPktAbsoluteGap >= 1)
                && (ptPosTxCfg->ulPosTxInterPktAbsoluteGap <= 255))
    {

        uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_ABS_EN);

        uiValue |= (ixf18100_mPOS_TX_IPG_ABS 
                        & (ptPosTxCfg->ulPosTxInterPktAbsoluteGap 
                                << ixf18100_sPOS_TX_IPG_ABS));

        uiValue |= (0x00 << ixf18100_sPOS_TX_IPG_ABS_CNF);
    }
    else if ((ptPosTxCfg->ulPosTxInterPktAbsoluteGap >= 256)
                && (ptPosTxCfg->ulPosTxInterPktAbsoluteGap <= 65535))
    {

        uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_ABS_EN);

        uiValue |= (ixf18100_mPOS_TX_IPG_ABS
            & ((ptPosTxCfg->ulPosTxInterPktAbsoluteGap >> 0x08)
                << ixf18100_sPOS_TX_IPG_ABS));

        uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_ABS_CNF);
    }
    else if (ptPosTxCfg->ulPosTxInterPktAbsoluteGap == 65536)
    {

        uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_ABS_EN);

        uiValue |= (ixf18100_mPOS_TX_IPG_ABS & (0x00
            << ixf18100_sPOS_TX_IPG_ABS));

        uiValue |= (0x01 << ixf18100_sPOS_TX_IPG_ABS_CNF);
    }
    else if (ptPosTxCfg->ulPosTxInterPktAbsoluteGap > 65536)
    {

        rc =  bb_INV_PARAMETER;
    }

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_T_IPGCTRL_OFFSET, uiValue);

    IXF_FUNC_EXIT_TRC("Ixf18100PosTxSetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosRxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosRxCfg_t *ptPosRxCfg: pointer to the values that need
*               to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosRxGetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosRxCfg_t *ptPosRxCfg)
{

    ushort      uiValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosRxGetCfg");

    uiValue = HAL_IXF18100_READ(BaseAddress + ixf18100_R_PHCCNF_OFFSET);

    ptPosRxCfg->ucPosRxHdlcFrmDescrambleEn
        = ((uiValue >> ixf18100_sPOS_RX_DESCRABLER_EN) & 0x01);

    ptPosRxCfg->ucPosWriteHdlcAddrInRxFifoEn
        = ((uiValue >> ixf18100_sPOS_RX_AC_PASS_EN) & 0x01);

    ptPosRxCfg->ucPosRxFcsType = (uchar)((uiValue & ixf18100_mPOS_RX_FCS_CNF)
        >> ixf18100_sPOS_RX_FCS_CNF);

    ptPosRxCfg->ucPosRxCheckHdlcAddrEn
        = ((uiValue >> ixf18100_sPOS_RX_AC_CHECK_EN) & 0x01);

    ptPosRxCfg->ucPosRxDiscardSmallPkt
        = ((uiValue >> ixf18100_sPOS_RX_MIN_PLD_EN) & 0x01);

    ptPosRxCfg->ucPosRxDiscardLargePkt
        = ((uiValue >> ixf18100_sPOS_RX_MAX_PLD_EN) & 0x01);

    ptPosRxCfg->ucPosRxDescrmbAfterByteDestuffingEn
        = ((uiValue >> ixf18100_sPOS_RX_TR_DESCRAMBLER_EN) & 0x01);

    ptPosRxCfg->ucPosWriteHdlcFcsFieldToRxFifo
        = ((uiValue >> ixf18100_sPOS_RX_FCS_PASS_EN) & 0x01);

    ptPosRxCfg->ucPosRxMarkFrameWithFcsErrorsEn
        = ((uiValue >> ixf18100_sPOS_RX_FCS_ERR_EN) & 0x01);

    ptPosRxCfg->ucPosRxReadPktFromWIS
        = ((uiValue >> ixf18100_sPOS_RX_READ_EN) & 0x01);


    uiValue = HAL_IXF18100_READ(BaseAddress + ixf18100_R_MINPL_OFFSET);

    ptPosRxCfg->ucPosRxMinPktLength = (uchar)((uiValue
        & ixf18100_mPOS_RX_MIN_PKT_LEN) >> ixf18100_sPOS_RX_MIN_PKT_LEN);


    uiValue = HAL_IXF18100_READ(BaseAddress + ixf18100_R_MAXPL_OFFSET);

    ptPosRxCfg->uiPosRxMaxPktLength
        = (ushort)((uiValue & ixf18100_mPOS_RX_MAX_PKT_LEN)
            >> ixf18100_sPOS_RX_MAX_PKT_LEN);

    IXF_FUNC_EXIT_TRC("Ixf18100PosRxGetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100PosTxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_PosTxCfg_t *ptPosTxCfg: pointer to the values that need
*               to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosTxGetCfg(bb_RegPointer_type BaseAddress,
                    ixf18100_PosTxCfg_t *ptPosTxCfg)
{

    ushort      uiValue;
    uchar       ucPosTxIPGAbs;
    uchar       ucPosTxIPGAbsCnf;
    uchar       ucPosTxIPGAbsEn;
    uchar       ucPosTxIPGRel;
    uchar       ucPosTxIPGRelCnf;
    uchar       ucPosTxIPGRelEn;
    bb_Error_e  rc = bb_NO_ERROR;


    IXF_FUNC_ENTRY_TRC("Ixf18100PosTxGetCfg");

    uiValue = HAL_IXF18100_READ(BaseAddress + ixf18100_T_PHCCNF_OFFSET);

    ptPosTxCfg->ucPosTxHdlcFrmScrambleEn
        = ((uiValue >> ixf18100_sPOS_TX_SCRABLER_EN) & 0x01);

    ptPosTxCfg->ucPosTxHdlcAddrCtrlEn
        = ((uiValue >> ixf18100_sPOS_TX_AC_PASS_EN) & 0x01);

    ptPosTxCfg->ucPosTxFcsType
        = (uchar)((uiValue & ixf18100_mPOS_TX_FCS_CNF)
            >> ixf18100_sPOS_TX_FCS_CNF);

    ptPosTxCfg->ucPosTxInsertFcsErrEn
        = ((uiValue >> ixf18100_sPOS_TX_FCS_ERR_EN) & 0x01);

    ptPosTxCfg->ucPosTxFcsErrInAllFrmEn
        = ((uiValue >> ixf18100_sPOS_TX_FCS_ERR_CNF_EN) & 0x01);

    ptPosTxCfg->ucPosTxFifoPktReadEn
        = ((uiValue >> ixf18100_sPOS_TX_PKT_READ_EN) & 0x01);

    ptPosTxCfg->ucPosTxScrmDataBeforeByteStuffingEn
        = ((uiValue >> ixf18100_sPOS_TX_TR_SCRAMBLER_EN) & 0x01);


    uiValue = HAL_IXF18100_READ(BaseAddress + ixf18100_T_IPGCTRL_OFFSET);

    ucPosTxIPGAbs = (uchar)((uiValue & ixf18100_mPOS_TX_IPG_ABS)
        >> ixf18100_sPOS_TX_IPG_ABS);

    ucPosTxIPGAbsCnf = ((uiValue >> ixf18100_sPOS_TX_IPG_ABS_CNF) & 0x01);

    ucPosTxIPGAbsEn = ((uiValue >> ixf18100_sPOS_TX_IPG_ABS_EN) & 0x01);

    ucPosTxIPGRel = (uchar)((uiValue & ixf18100_mPOS_TX_IPG_REL)
        >> ixf18100_sPOS_TX_IPG_REL);

    ucPosTxIPGRelCnf = ((uiValue >> ixf18100_sPOS_TX_IPG_REL_CNF) & 0x01);

    ucPosTxIPGRelEn = ((uiValue >> ixf18100_sPOS_TX_IPG_REL_EN) & 0x01);

    if (0x01 == ucPosTxIPGRelEn)
    {
        if (0x00 == ucPosTxIPGRelCnf)
        {
            switch(ucPosTxIPGRel)
            {
                case 0x00:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_MAX_PKTLEN_1;
                    break;

                case 0x01:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_2;
                    break;

                case 0x02:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_4;
                    break;

                case 0x03:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_8;
                    break;

                case 0x04:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_16;
                    break;

                case 0x05:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_32;
                    break;

                case 0x06:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_64;
                    break;

                case 0x07:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_DIV_128;
                    break;

                default:
                    rc = bb_INV_PARAMETER;
                    break;
            }
        }
        else if (0x01 == ucPosTxIPGRelCnf)
        {
            switch(ucPosTxIPGRel)
            {
                case 0x00:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_1;
                    break;

                case 0x01:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_2;
                    break;

                case 0x02:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_4;
                    break;

                case 0x03:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_8;
                    break;

                case 0x04:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_16;
                    break;

                case 0x05:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_32;
                    break;

                case 0x06:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_64;
                    break;

                case 0x07:
                    ptPosTxCfg->ePosTxInterPktRelativeGap = e_PKT_LEN_MULT_128;
                    break;

                default:
                    rc = bb_INV_PARAMETER;
                    break;
            }
        }
    }
    else if (0x00 == ucPosTxIPGRelEn)
    {
        ptPosTxCfg->ePosTxInterPktRelativeGap = e_REL_PKT_GAP_DISABLE;
    }

    if (0x00 == ucPosTxIPGAbsEn)
    {
        ptPosTxCfg->ulPosTxInterPktAbsoluteGap = 0;
    }

    if (0x00 == ucPosTxIPGAbsCnf)
    {
        ptPosTxCfg->ulPosTxInterPktAbsoluteGap = ucPosTxIPGAbs;
    }
    else if (0x01 == ucPosTxIPGAbsCnf)
    {
        if (0 == ucPosTxIPGAbs)
        {
            ptPosTxCfg->ulPosTxInterPktAbsoluteGap = 0x10000;
        }
        else
        {
            ptPosTxCfg->ulPosTxInterPktAbsoluteGap = (ucPosTxIPGAbs << 0x08);
        }
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosTxGetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosRxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS RX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosRxGetCounters(bb_RegPointer_type BaseAddress,
                         bb_SelCounters_e SelCntr, void *ptCounters)
{
    bb_Error_e          rc = bb_NO_ERROR;
    ulong               *ulCounter = (ulong *)ptCounters;
    ushort              *usCounter = (ushort *)ptCounters;
    ixfApi_PosRxCntr_t  *ptPosRxCounters = (ixfApi_PosRxCntr_t *)ptCounters;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosRxGetCounters");

    switch (SelCntr)
    {
    case bb_POS_RX_GOOD_FRM_CNT:
        *ulCounter 
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_GFCNT_W0_OFFSET, 
                ixf18100_mR_GFCNT_W1, ixf18100_mR_GFCNT_W0);
        break;

    case bb_POS_RX_GOOD_BYTE_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_GFBYTECNT_W0_OFFSET, 
                ixf18100_mR_GFBYTECNT_W1, ixf18100_mR_GFBYTECNT_W0);
        break;

    case bb_POS_RX_ABORTED_FRM_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_AFCNT_W0_OFFSET, 
                ixf18100_mR_AFCNT_W1, ixf18100_mR_AFCNT_W0);
        break;

    case bb_POS_RX_ABORTED_BYTE_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_AFBYTECNT_W0_OFFSET, 
                ixf18100_mR_AFBYTECNT_W1, ixf18100_mR_AFBYTECNT_W0);
        break;

    case bb_POS_RX_FCS_ERR_FRM_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_FCSECNT_W0_OFFSET, 
                ixf18100_mR_FCSECNT_W1, ixf18100_mR_FCSECNT_W0);
        break;

    case bb_POS_RX_FCS_ERR_BYTE_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_FCSEBYTECNT_W0_OFFSET, 
                ixf18100_mR_FCSEBYTECNT_W1, ixf18100_mR_FCSEBYTECNT_W0);
        break;

    case bb_POS_RX_MIN_PLE_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_MINPLECNT_W0_OFFSET,
                ixf18100_mR_MINPLECNT_W1, ixf18100_mR_MINPLECNT_W0);
        break;

    case bb_POS_RX_MAX_PLE_CNT:
        *usCounter
            = HAL_IXF18100_READ(BaseAddress + ixf18100_R_MAXPLECNT_OFFSET);
        break;

    case bb_POS_RX_COUNTERS:
        ptPosRxCounters->ulPosRxGoodFrmCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_GFCNT_W0_OFFSET,
                ixf18100_mR_GFCNT_W1, ixf18100_mR_GFCNT_W0);

        ptPosRxCounters->ulPosRxGoodFrmByteCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_GFBYTECNT_W0_OFFSET,
                ixf18100_mR_GFBYTECNT_W1, ixf18100_mR_GFBYTECNT_W0);

        ptPosRxCounters->ulPosRxAbortedFrmCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_AFCNT_W0_OFFSET,
                ixf18100_mR_AFCNT_W1, ixf18100_mR_AFCNT_W0);

        ptPosRxCounters->ulPosRxAbortedFrmByteCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_AFBYTECNT_W0_OFFSET,
                ixf18100_mR_AFBYTECNT_W1, ixf18100_mR_AFBYTECNT_W0);

        ptPosRxCounters->ulPosRxFcsErrCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_FCSECNT_W0_OFFSET,
                ixf18100_mR_FCSECNT_W1, ixf18100_mR_FCSECNT_W0);

        ptPosRxCounters->ulPosRxFcsErrByteCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_FCSEBYTECNT_W0_OFFSET,
                ixf18100_mR_FCSEBYTECNT_W1, ixf18100_mR_FCSEBYTECNT_W0);

        ptPosRxCounters->ulPosRxMinPktLenErrCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_R_MINPLECNT_W0_OFFSET,
                ixf18100_mR_MINPLECNT_W1, ixf18100_mR_MINPLECNT_W0);

        ptPosRxCounters->ulPosRxMaxPktLenErrCnt
            = HAL_IXF18100_READ(BaseAddress + ixf18100_R_MAXPLECNT_OFFSET);
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosRxGetCounters", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosTxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS TX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosTxGetCounters(bb_RegPointer_type BaseAddress,
                         bb_SelCounters_e SelCntr, void *ptCounters)
{
    bb_Error_e          rc = bb_NO_ERROR;
    ulong               *ulCounter = (ulong *)ptCounters;
    ixfApi_PosTxCntr_t  *ptPosTxCounters = (ixfApi_PosTxCntr_t *)ptCounters;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosTxGetCounters");

    switch (SelCntr)
    {
    case bb_POS_TX_FRM_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_T_FRMCNT_W0_OFFSET,
                ixf18100_mT_FRMCNT_W1, ixf18100_mT_FRMCNT_W0);
        break;

    case bb_POS_TX_BYTE_CNT:
        *ulCounter
            = Ixf18100PosRead32(BaseAddress + ixf18100_T_BYTECNT_W0_OFFSET,
                ixf18100_mT_BYTECNT_W1, ixf18100_mT_BYTECNT_W0);
        break;

    case bb_POS_TX_COUNTERS:
        ptPosTxCounters->ulPosTxFrmCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_T_FRMCNT_W0_OFFSET,
                ixf18100_mT_FRMCNT_W1, ixf18100_mT_FRMCNT_W0);
        ptPosTxCounters->ulPosTxByteCnt
            = Ixf18100PosRead32(BaseAddress + ixf18100_T_BYTECNT_W0_OFFSET,
                ixf18100_mT_BYTECNT_W1, ixf18100_mT_BYTECNT_W0);
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosTxGetCounters", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from POS config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be set
*           ixf18100_PosCfg_t *ptPosCfg: pointer to the values that need to 
*               be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e  SelCfg,
                  ixf18100_PosCfg_t *ptPosCfg)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosSetCfg");

    switch (SelCfg)
    {
    case e_RX_CONFIG:
        rc = Ixf18100PosRxSetCfg(BaseAddress, &(ptPosCfg->PosRxCfg));
        break;

    case e_TX_CONFIG:
        rc = Ixf18100PosTxSetCfg(BaseAddress, &(ptPosCfg->PosTxCfg));
        break;

    case e_ALL_CONFIG:
        rc = Ixf18100PosRxSetCfg(BaseAddress, &(ptPosCfg->PosRxCfg));
        rc = Ixf18100PosTxSetCfg(BaseAddress, &(ptPosCfg->PosTxCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosSetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be fetched
*           ixf18100_PosCfg_t *ptPosCfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e  SelCfg,
                  ixf18100_PosCfg_t *ptPosCfg)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosGetCfg");

    switch (SelCfg)
    {
    case e_RX_CONFIG:
        rc = Ixf18100PosRxGetCfg(BaseAddress, &(ptPosCfg->PosRxCfg));
        break;

    case e_TX_CONFIG:
        rc = Ixf18100PosTxGetCfg(BaseAddress, &(ptPosCfg->PosTxCfg));
        break;

    case e_ALL_CONFIG:
        rc = Ixf18100PosRxGetCfg(BaseAddress, &(ptPosCfg->PosRxCfg));
        rc = Ixf18100PosTxGetCfg(BaseAddress, &(ptPosCfg->PosTxCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosGetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100PosGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the POS counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptPosCounters: pointer to the counter structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100PosGetCounters(bb_RegPointer_type BaseAddress,
                       bb_SelCounters_e SelCntr, void *ptPosCounters)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosGetCounters");

    switch (SelCntr)
    {
    case bb_POS_RX_GOOD_FRM_CNT:
    case bb_POS_RX_GOOD_BYTE_CNT:
    case bb_POS_RX_ABORTED_FRM_CNT:
    case bb_POS_RX_ABORTED_BYTE_CNT:
    case bb_POS_RX_FCS_ERR_FRM_CNT:
    case bb_POS_RX_FCS_ERR_BYTE_CNT:
    case bb_POS_RX_MIN_PLE_CNT:
    case bb_POS_RX_MAX_PLE_CNT:
    case bb_POS_RX_COUNTERS:
        rc = Ixf18100PosRxGetCounters(BaseAddress, SelCntr, ptPosCounters);
        break;

    case bb_POS_TX_FRM_CNT:
    case bb_POS_TX_BYTE_CNT:
    case bb_POS_TX_COUNTERS:
        rc = Ixf18100PosTxGetCounters(BaseAddress, SelCntr, ptPosCounters);
        break;

    case bb_POS_COUNTERS:
        rc = Ixf18100PosRxGetCounters(BaseAddress, bb_POS_RX_COUNTERS,
            (void *)&(((ixfApi_PosCntr_t *)ptPosCounters)->tPosRxCounters));
        rc = Ixf18100PosTxGetCounters(BaseAddress, bb_POS_TX_COUNTERS,
            (void *)&(((ixfApi_PosCntr_t *)ptPosCounters)->tPosTxCounters));
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100PosGetCounters", rc);

    return rc;
}


/*****************************INTERNAL ROUTINES*****************************/


/******************************************************************************
* Function : Ixf18100PosRead32
*------------------------------------------------------------------------------
* Description : Reads 32 bit values (counters) from the POS block
*------------------------------------------------------------------------------
* Inputs  : ushort *puiLsb: pointer to the least significant bits of the
*                   32 bit value to be read.
*           ushort uiMaskMsb: Mask for the upper 16 bits
*           ushort uiMaskLsb: Mask for the lower 16 bits
* Outputs : none
*------------------------------------------------------------------------------
* Returns : 32 bit value containing properly masked value.
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
ulong
Ixf18100PosRead32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb)
{

    ushort uiBitsHigh, uiBitsLow;

    IXF_FUNC_ENTRY_TRC("Ixf18100PosRead32");

    /* The high counter bits are stored at the higher address, and must
    ** be read second.  The function takes the address of the word
    ** containing the least significant bits of the counter, so we 
    ** add 1 to get the address of the word containing the most
    ** significant bits. */
    uiBitsLow = HAL_IXF18100_READ(puiLsb);
    uiBitsHigh = HAL_IXF18100_READ(puiLsb + 1);

    IXF_FUNC_EXIT_TRC("Ixf18100PosRead32", 0);

    return(((uiBitsHigh & uiMaskMsb) << 16) | (uiBitsLow & uiMaskLsb));
}
