/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y                   
 *
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *
 *                        INTEL  CORPORATION                            
 *
 *                     2200 MISSION COLLEGE BLVD                        
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_sonet.c $
 *   WORKFILE:       $Workfile: ixf18100_sonet.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains SONET block related routines
 *
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_sonet.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 1     6/27/03 4:43p Daly
 * 
 * 50    3/26/03 1:10p Kssaepha
 * Fixed Validate ChipType for some functions
 * 
 * 49    1/30/03 4:40p Pkjain2
 * Made changes for PTQ release. Added Getstatus calls for SONET and
 * GLOBAL block.
 * 
 * 48    1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 47    12/12/02 9:49p Machambe
 * Adding IXF_LINUX
 * 
 * 46    12/11/02 1:24p Jaalliso
 * Fixing chiptype error checking for ixf18103 & SONET functions
 * 
 * 45    8/26/02 2:01p Kssaepha
 * Added support for Path Counters under case bb_SONET_COUNTERS
 * 
 * 44    7/19/02 8:39p Jaalliso
 * Proper error handling for Ixf18100SonetGetTrace
 * 
 * 43    6/13/02 3:20p Srodrig1
 * Correcting Potential indeterminate results from SonetGetTrace
 * 
 * 42    5/29/02 9:40a Jaalliso
 * Changing bb_18100_CHIP to bb_18101_CHIP, adding 18102/3/4 chips
 * 
 * 41    5/23/02 3:19p Kssaepha
 * Modified Ixf18100SonetPadTrace to not pad with '0'
 * 
 * 40    5/17/02 3:11p Machambe
 * Added msgQLib.h header file.
 * 
 * 39    5/17/02 1:53p Machambe
 * Adding () to PREEMPTION calls.
 * 
 * 38    5/17/02 1:48p Machambe
 * 
 * 37    4/25/02 1:58p Kssaepha
 * Changed lf_cr to cr_lf for bb_64_BYTE_WITH_LF_CR
 * 
 * 36    4/19/02 11:04a Kssaepha
 * Fixed SonetGetTrace for case bb_TRANSMIT_J0
 * 
 * 35    4/17/02 10:37a Kssaepha
 * Fixed Load J1 Compose
 * 
 * 34    3/21/02 9:44a Srodrig1
 * 
 * 33    2/26/02 4:54p Kssaepha
 * 
 * 32    2/13/02 3:58p Machambe
 * 
 * 31    2/08/02 2:52p Kssaepha
 * Added case bb_K1_K2
 * 
 * 30    2/08/02 9:25a Kssaepha
 * 
 * 29    2/07/02 5:13p Kssaepha
 * Removed bb_TRANSMIT_J1_TEST
 * 
 * 28    2/07/02 4:00p Srodrig1
 * 
 * 27    2/07/02 12:50p Rsood
 * Updated trace related functions for APS
 * 
 * 26    2/05/02 3:55p Srodrig1
 * 
 * 25    2/05/02 2:19p Srodrig1
 * 
 * 24    2/01/02 10:58a Srodrig1
 * 
 * 23    1/30/02 5:13p Srodrig1
 * 
 * 22    1/29/02 2:45p Srodrig1
 * 
 * 21    1/28/02 2:44p Srodrig1
 * 
 * 20    1/28/02 2:14p Srodrig1
 * 
 * 19    1/24/02 6:18p Srodrig1
 * 
 * 18    1/24/02 2:48p Srodrig1
 * 
 * 17    1/11/02 10:30a Srodrig1
 * 
 * 16    12/17/01 11:51a Rsood
 * Updated APS registers
 * 
 * 15    12/05/01 2:42p Rsood
 * Added APS Section
 * 
 * 14    12/04/01 4:24p Pkjain2
 * 
 * 13    12/04/01 9:00a Srodrig1
 * 
 * 12    12/03/01 3:32p Pkjain2
 * 
 * 11    11/20/01 11:34a Srodrig1
 * 
 * 10    11/16/01 8:46p Pkjain2
 * 
 * 9     11/09/01 11:09a Srodrig1
 * 
 * 8     9/14/01 12:21p Srodrig1
 * 
 * 7     9/10/01 6:07p Srodrig1
 * Implementation for functions to set and get the windows size for the
 * degraded and excessive errors.
 *  also the get and set OHT bytes is bytewise instead of using blocks or
 * data structure to return the OHT bytes.
 * 
 * 6     8/15/01 11:25a Rfranz
 * Added comments
 * 
 * 5     8/08/01 3:50p Rfranz
 * added support for debugging using memory block instead of Calypso
 * device or simulator.  Moved device read/write macros/functions to
 * ixf18100_util.[ch]
 * 
 * 
 * 4     7/27/01 4:04p Rfranz
 * ixf18100_sonet.c compiles without errors or warnings
 * 
 * 3     7/27/01 2:11p Rfranz
 * Check in before first attempt to compile
 * 
 * 2     7/24/01 1:22p Rfranz
 * work in progress
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#include "msgQLib.h"
#endif

#ifdef IXF_LINUX
#ifndef __KERNEL__
#include <semaphore.h>
#endif
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#ifdef __KERNEL__
#include "linuxkern.h"
#else
#include "os.h"
#endif
#include "ixf_api_sonet_d.h"
#include "ixf18100_sonet_d.h"
#include "ixf18100_sonet_b.h"
#include "ixf18100_sonet.h"
#include "ixf18100.h"
#include "ixf18100_d.h"
#include "ixf18100_util.h"
#include "ixf18100_d.h"
#include "hal.h"



/* Prototypes for utility functions used by this module */
static bb_Error_e
Ixf18100SonetGetRxTraceLength(ushort *puiTraceCfgReg, uchar *pucTraceLength);

static bb_Error_e
Ixf18100SonetReadRxTraceString(volatile ushort *puiTraceReg, uchar *pucTrace,
                               uchar ucLength);

static ulong
Ixf18100SonetRead32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb);

static bb_Error_e
Ixf18100SonetReadTxTraceString(volatile ushort *puiTraceReg, uchar *pucTrace,
                               uchar ucLength);

static bb_Error_e
Ixf18100SonetProcessTrace(uchar *pucUserArg, uchar *pucProcessedArg,
                          bb_TraceFormat_e eTraceFormat,
                          uchar *pucTraceLength);

static bb_Error_e
Ixf18100SonetWriteRxTraceString(volatile ushort *puiTraceReg, uchar *pucTrace,
                                uchar ucLength);

static bb_Error_e
Ixf18100SonetSetRxTraceFormat(bb_TraceFormat_e eTraceFormat,
                              ushort *piTraceCfg);

static bb_Error_e
Ixf18100SonetWriteTxTraceString(volatile ushort *piTraceReg, uchar *pcTrace,
                                uchar ucLength);

static bb_Error_e
Ixf18100GetPathOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_PathOhBytes_t *ptPathOh);

static bb_Error_e
Ixf18100SetPathOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_PathOhBytes_t *ptPathOh);

static bb_Error_e
Ixf18100GetLineOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_RstMstBytes_t *ptPathOh);

static bb_Error_e
Ixf18100SetLineOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_RstMstBytes_t *ptPathOh);

static bb_Error_e
Ixf18100SonetGetMiscStatus(bb_RegPointer_type BaseAddress,
                           ixf18100_SonetMiscStatus_t *SonetMiscStat);

static bb_Error_e
Ixf18100SonetGetRxSOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxSectionOhStatus_t *RxSOHStat);

static bb_Error_e
Ixf18100SonetGetRxAPSSOHStatus(bb_RegPointer_type BaseAddress,
                ixf18100_SonetAPSRxSectionOhStatus_t *RxAPSSOHStat);

static bb_Error_e
Ixf18100SonetGetRxLOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxLineOhStatus_t *RxLOHStat);

static bb_Error_e
Ixf18100SonetGetRxAPSLOHStatus(bb_RegPointer_type BaseAddress,
                               ixf18100_SonetAPSRxLineOhStatus_t *RxAPSLOHStat);

static bb_Error_e
Ixf18100SonetGetRxPOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxPathOhStatus_t *RxPOHStat);

static bb_Error_e
Ixf18100SonetGetRxPtrStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxPtrTrackerStatus_t *RxPtrStat);

static bb_Error_e
Ixf18100SonetGetTxPtrStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetTxPtrTrackerStatus_t *TxPtrStat);

/******************************************************************************
* Function : Ixf18100SonetGetOhBytes
*------------------------------------------------------------------------------
* Description : This function returns the requested overhead bytes. These may
*               be either the currently programmed transmit bytes, or the
*               received bytes.  Received bytes may be filtered depending
*               on the filter configuration.
*------------------------------------------------------------------------------
* Inputs : ptChipData: pointer to chip data structure.
*          ptSegment: pointer to chip segment structure.  Only the block.block
*                   and block.flow members are used.
*          eOhBytes: which overhead bytes to set
* Outputs : pOhBytes: pointer to structure containing overhead bytes. Note that
*           the caller must allocate space for this.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals : None
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100SonetGetOhBytes(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                        bb_SelOhBytes_e eOhBytes, void *pOhBytes)
{
    uchar *ohtByte;
    ushort uiTemp;
    bb_Error_e rc;

    /* Validate ChipType */
    if (ptChipData->ChipType != bb_18101_CHIP &&
        ptChipData->ChipType != bb_18102_CHIP &&
        ptChipData->ChipType != bb_18103_CHIP &&
        ptChipData->ChipType != bb_18101_8023WIS_CHIP &&
        ptChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate arguments */
    if (((TX != ptSegment->block.flow) && (RX != ptSegment->block.flow))
        || ((ixf_eSONET != ptSegment->block.block)
            && (ixf_eAPS != ptSegment->block.block)))
    {
        return bb_INV_CHIP_SEGMENT;
    }

    switch (eOhBytes)
    {
        case bb_J1:
            if(ptSegment->block.block == ixf_eSONET)
            {           
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_POH_BVA_J1_OFFSET)
                                & ixf18100_mTL_POH_BVA)
                                    >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    /* J1 only available through string access on RL side */
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                /* J1 only available through string access on RL side */
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RECEIVED_C2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    return bb_INV_SEL_OH_BYTES;
                }
                else
                {
                    /* Return the Received C2 here.
                    Expected is handled separately */
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_C2_OFFSET) & ixf18100_mC2)
                            >> ixf18100_sC2);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_TX_C2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_C2_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_B3:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_B3_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_G1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_G1_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_G1_OFFSET)
                            & ixf18100_mG1) >> ixf18100_sG1);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_F2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_F2_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_F2_OFFSET)
                            & ixf18100_mF2) >> ixf18100_sF2);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_H4:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_H4_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_H4_OFFSET)
                            & ixf18100_mH4) >> ixf18100_sH4);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_Z3_F3:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z3_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_Z3_F3_OFFSET)
                            & ixf18100_mZ3_F3) >> ixf18100_sZ3_F3);
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_K3_Z4:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z4_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_Z4_K3_OFFSET)
                            & ixf18100_mZ4_K3) >> ixf18100_sZ4_K3);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_Z5_N1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z5_OFFSET)
                            & ixf18100_mTL_POH_BVA) >> ixf18100_sTL_POH_BVA);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_RCV_Z5_N1_OFFSET)
                            & ixf18100_mZ5_N1) >> ixf18100_sZ5_N1);
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_K1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK1)
                        >> ixf18100_sK1);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK1)
                        >> ixf18100_sK1);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK1)
                        >> ixf18100_sK1);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK1)
                        >> ixf18100_sK1);
                }
            }
            break;

        case bb_K2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK2)
                        >> ixf18100_sK2);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_RL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK2)
                        >> ixf18100_sK2);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK2)
                        >> ixf18100_sK2);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mK2)
                        >> ixf18100_sK2);
                }
            }
            break;

        case bb_K1_K2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    *(ushort *)pOhBytes
                        = (ushort)(HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_K1_K2_OFFSET));
                }
                else
                {
                    *(ushort *)pOhBytes
                        = (ushort)(HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_RL_K1_K2_OFFSET));
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {           
                if (TX == ptSegment->block.flow)
                {
                    *(ushort *)pOhBytes
                        = (ushort)(HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_APS_TL_K1_K2_OFFSET));
                }
                else
                {
                    *(ushort *)pOhBytes
                        = (ushort)(HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_APS_RL_K1_K2_OFFSET));
                }
            }
            break;

        case bb_S1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_S1_OFFSET)
                                & ixf18100_mS1) >> ixf18100_sS1);
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_RL_S1_OFFSET)
                                & ixf18100_mS1) >> ixf18100_sS1);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                ohtByte = (uchar *)pOhBytes;
                if (TX == ptSegment->block.flow)
                {
                    return bb_INV_SEL_OH_BYTES;
                }
                else
                {
                    *ohtByte
                        = (uchar)((HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_APS_RL_S1_OFFSET)
                                & ixf18100_mS1) >> ixf18100_sS1);
                }
            }
            break;

        case bb_M0:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mM0)
                        >> ixf18100_sM0);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_M1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET);

                    *ohtByte = (uchar)((uiTemp & ixf18100_mM1)
                        >> ixf18100_sM1);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_M0_M1:
            if(ptSegment->block.block == ixf_eSONET)
            {               
                if (TX == ptSegment->block.flow)
                {
                    *(ushort *)pOhBytes
                        = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_M0_M1_OFFSET);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU1)
                            >> ixf18100_sNU1);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU1)
                            >> ixf18100_sNU1);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU1)
                            >> ixf18100_sNU1);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU1)
                            >> ixf18100_sNU1);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU2)
                            >> ixf18100_sNU2);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU2)
                            >> ixf18100_sNU2);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU2)
                            >> ixf18100_sNU2);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET);

                        *ohtByte = (uchar)((uiTemp & ixf18100_mNU2)
                            >> ixf18100_sNU2);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU1_RNU2:
            if(ptSegment->block.block == ixf_eSONET)
            {               
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        *(ushort *)pOhBytes
                            = HAL_IXF18100_READ(ptChipData->BaseAddress
                                + ixf18100_TL_NU1_2_0_OFFSET);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        *(ushort *)pOhBytes
                            = HAL_IXF18100_READ(ptChipData->BaseAddress
                                + ixf18100_TL_NU1_2_1_OFFSET);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        *(ushort *)pOhBytes
                            = HAL_IXF18100_READ(ptChipData->BaseAddress
                                + ixf18100_TL_NU1_2_2_OFFSET);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        *(ushort *)pOhBytes
                            = HAL_IXF18100_READ(ptChipData->BaseAddress
                                + ixf18100_TL_NU1_2_3_OFFSET);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU9:
            if(ptSegment->block.block == ixf_eSONET)
            {
                ohtByte = (uchar *)pOhBytes;
                
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        *ohtByte = (uchar)((HAL_IXF18100_READ(
                            ptChipData->BaseAddress + ixf18100_TL_NU9_0_OFFSET)
                                & ixf18100_mNU9) >> ixf18100_sNU9);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        *ohtByte = (uchar)((HAL_IXF18100_READ(
                            ptChipData->BaseAddress + ixf18100_TL_NU9_1_OFFSET)
                                & ixf18100_mNU9) >> ixf18100_sNU9);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        *ohtByte = (uchar)((HAL_IXF18100_READ(
                            ptChipData->BaseAddress + ixf18100_TL_NU9_2_OFFSET)
                                & ixf18100_mNU9) >> ixf18100_sNU9);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        *ohtByte = (uchar)((HAL_IXF18100_READ(
                            ptChipData->BaseAddress + ixf18100_TL_NU9_3_OFFSET)
                                & ixf18100_mNU9) >> ixf18100_sNU9);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_PATH_OH_BYTES:
            rc = Ixf18100GetPathOhBytes(ptChipData->BaseAddress,
                ptSegment->block.block, ptSegment->block.flow,
                (bb_PathOhBytes_t *)pOhBytes);

            if (rc != bb_NO_ERROR)
            {
                return rc;
            }
            break;

        case bb_LINE_OH_BYTES:
            rc = Ixf18100GetLineOhBytes(ptChipData->BaseAddress,
                ptSegment->block.block, ptSegment->block.flow,
                (bb_RstMstBytes_t *)pOhBytes);

            if (rc != bb_NO_ERROR)
            {
                return rc;
            }
            break;

        case bb_EXPECTED_C2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    return bb_INV_SEL_OH_BYTES;
                }
                else
                {
                    *(uchar *)pOhBytes = (uchar)((HAL_IXF18100_READ(
                        ptChipData->BaseAddress + ixf18100_RL_EXP_C2_OFFSET)
                            & ixf18100_mEXP_C2) >> ixf18100_sEXP_C2);
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        default:
            return bb_INV_SEL_OH_BYTES;
            break;
    }  /* switch(eOhBytes) */

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetSetOhBytes
*------------------------------------------------------------------------------
* Description : This function sets SONET overhead bytes for the ixf18100
*               family of chips.  It sets both the outgoing transmitted bytes,
*               and the expected RX C2 byte. The expected C2 RX byte is the
*               only byte on the RX side that may be set. Note that trace
*               strings are set with the function ixf18100SonetSetTrace.
*------------------------------------------------------------------------------
* Inputs  : ptChipData: pointer to chip data structure.
*           ptSegment: pointer to chip segment structure. Only the block.block
*               and block.flow members are used.
*           eOhBytes: which overhead bytes to set
*           pOhBytes: pointer to structure containing overhead bytes.
* Outputs : return value only
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals : None
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100SonetSetOhBytes(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                        bb_SelOhBytes_e eOhBytes, ushort pOhBytes)
{
    ushort uiTemp;

    /* Validate ChipType */
    if (ptChipData->ChipType != bb_18101_CHIP &&
        ptChipData->ChipType != bb_18102_CHIP &&
        ptChipData->ChipType != bb_18103_CHIP &&
        ptChipData->ChipType != bb_18101_8023WIS_CHIP &&
        ptChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate arguments */
    if (((TX != ptSegment->block.flow) && (RX != ptSegment->block.flow))
        || ((ixf_eSONET != ptSegment->block.block)
            && (ixf_eAPS != ptSegment->block.block)))
    {
        return bb_INV_CHIP_SEGMENT;
    }

    switch (eOhBytes)
    {
        case bb_J1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_J1_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    /* J1 only available through string access on RL side */
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_TX_C2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_C2_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_B3:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_B3_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_G1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_G1_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_F2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_F2_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_H4:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_H4_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_Z3_F3:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z3_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if (ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_K3_Z4:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z4_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_Z5_N1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_Z5_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sTL_POH_BVA));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_K1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET);

                    uiTemp &= ixf18100_mK2;

                    uiTemp |= (pOhBytes << ixf18100_sK1);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET, uiTemp);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET);

                    uiTemp &= ~ixf18100_mK1;

                    uiTemp |= (pOhBytes << ixf18100_sK1);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET, uiTemp);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET);

                    uiTemp &= ~ixf18100_mK1;

                    uiTemp |= (pOhBytes << ixf18100_sK1);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET, uiTemp);
                }
            }
            break;

        case bb_K2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET);

                    uiTemp &= ~ixf18100_mK2;

                    uiTemp |= (pOhBytes << ixf18100_sK2);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET, uiTemp);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET);

                    uiTemp &= ~ixf18100_mK2;

                    uiTemp |= (pOhBytes << ixf18100_sK2);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET, uiTemp);
                }
                else
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET);

                    uiTemp &= ~ixf18100_mK2;

                    uiTemp |= (pOhBytes << ixf18100_sK2);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_APS_RL_K1_K2_OFFSET, uiTemp);
                }
            }
            break;

        case bb_K1_K2:
            if(ptSegment->block.block == ixf_eSONET)
            {                   
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_K1_K2_OFFSET, pOhBytes);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {                               
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_APS_TL_K1_K2_OFFSET, pOhBytes);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            break;

        case bb_S1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_S1_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sS1));
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_M0:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET);

                    uiTemp |= (pOhBytes << ixf18100_sM0);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET, uiTemp);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_M1:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET);

                    uiTemp &= ~ixf18100_mM1;

                    uiTemp |= (pOhBytes << ixf18100_sM1);

                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET, uiTemp);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_M0_M1:
            if(ptSegment->block.block == ixf_eSONET)
            { 
                if (TX == ptSegment->block.flow)
                {
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_M0_M1_OFFSET, pOhBytes);
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU1:
            if(ptSegment->block.block == ixf_eSONET)
            {               
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET);

                        uiTemp &= ~ixf18100_mNU1;

                        uiTemp |= (pOhBytes << ixf18100_sNU1);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET, uiTemp);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET);

                        uiTemp &= ~ixf18100_mNU1;

                        uiTemp |= (pOhBytes << ixf18100_sNU1);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET, uiTemp);
                    } 
                    else if (2 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET);

                        uiTemp &= ~ixf18100_mNU1;

                        uiTemp |= (pOhBytes << ixf18100_sNU1);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET, uiTemp);
                    } 
                    else if (3 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET);

                        uiTemp &= ~ixf18100_mNU1;

                        uiTemp |= (pOhBytes << ixf18100_sNU1);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET, uiTemp);
                    } 
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        case bb_RNU2:
            if(ptSegment->block.block == ixf_eSONET)
            {                   
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET);

                        uiTemp &= ~ixf18100_mNU2;

                        uiTemp |= (pOhBytes << ixf18100_sNU2);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET, uiTemp);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET);

                        uiTemp &= ~ixf18100_mNU2;

                        uiTemp |= (pOhBytes << ixf18100_sNU2);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET, uiTemp);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET);

                        uiTemp &= ~ixf18100_mNU2;

                        uiTemp |= (pOhBytes << ixf18100_sNU2);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET, uiTemp);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET);

                        uiTemp &= ~ixf18100_mNU2;

                        uiTemp |= (pOhBytes << ixf18100_sNU2);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET, uiTemp);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break; 

        case bb_RNU1_RNU2:
            if(ptSegment->block.block == ixf_eSONET)
            {  
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {
                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_0_OFFSET, pOhBytes);
                    }
                    else if (1 == ptSegment->group.group)
                    {
                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_1_OFFSET, pOhBytes);
                    }
                    else if (2 == ptSegment->group.group)
                    {
                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_2_OFFSET, pOhBytes);
                    }
                    else if (3 == ptSegment->group.group)
                    {
                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU1_2_3_OFFSET, pOhBytes);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;
            
        case bb_RNU9:
            if(ptSegment->block.block == ixf_eSONET)
            {                 
                if (TX == ptSegment->block.flow)
                {
                    if (0 == ptSegment->group.group)
                    {                       
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_0_OFFSET);

                        uiTemp &= ~ixf18100_mNU9;

                        uiTemp |= (pOhBytes << ixf18100_sNU9);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_0_OFFSET, uiTemp);
                    }
                    else if (1 == ptSegment->group.group)
                    {                       
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_1_OFFSET);

                        uiTemp &= ~ixf18100_mNU9;

                        uiTemp |= (pOhBytes << ixf18100_sNU9);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_1_OFFSET, uiTemp);
                    }
                    else if (2 == ptSegment->group.group)
                    {                       
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_2_OFFSET);

                        uiTemp &= ~ixf18100_mNU9;

                        uiTemp |= (pOhBytes << ixf18100_sNU9);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_2_OFFSET, uiTemp);
                    }
                    else if (3 == ptSegment->group.group)
                    {                       
                        uiTemp = HAL_IXF18100_READ(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_3_OFFSET);

                        uiTemp &= ~ixf18100_mNU9;

                        uiTemp |= (pOhBytes << ixf18100_sNU9);

                        HAL_IXF18100_WRITE(ptChipData->BaseAddress
                            + ixf18100_TL_NU9_3_OFFSET, uiTemp);
                    }
                    else
                    {
                        return bb_INV_SEL_OH_BYTES;
                    }
                }
                else
                {
                    return bb_INV_SEL_OH_BYTES;
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break; 

        case bb_EXPECTED_C2:
            if(ptSegment->block.block == ixf_eSONET)
            {
                if (TX == ptSegment->block.flow)
                {
                    return bb_INV_SEL_OH_BYTES;
                }
                else
                {
                     HAL_IXF18100_WRITE(ptChipData->BaseAddress
                         + ixf18100_RL_EXP_C2_OFFSET,
                            (ushort)(pOhBytes << ixf18100_sEXP_C2));
                }
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_OH_BYTES;
            }
            break;

        default:
            return bb_INV_SEL_OH_BYTES;
            break;

    }  /* switch(eOhBytes) */

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetCounters
*------------------------------------------------------------------------------
* Description : Reads and returns one/all SONET counters
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block and RX flow
*           bb_SelCounters_e eCounter :  selects which counter(s) are to
*               be read.
* Outputs : void *pCounterData:  The counter structure is returned in this
*               buffer.   This pointer is generic to allow for different
*               counter structures to be returned in the future.  Note that
*               Caller must allocate memory for htis buffer.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS, bb_INV_CHIP_TYPE, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100SonetGetCounters(bb_ChipData_t *ptChipData,
                         bb_ChipSegment_t *ptSegment,
                         bb_SelCounters_e eCounter, void *pCounterData)
{
    bb_RxSonetCounters_t *ptCounters;
    ushort uiValue16;
    ushort *counter16;
    ulong *counter32;

    /* Validate ChipType */
    if (ptChipData->ChipType != bb_18101_CHIP &&
        ptChipData->ChipType != bb_18102_CHIP &&
        ptChipData->ChipType != bb_18103_CHIP &&
        ptChipData->ChipType != bb_18101_8023WIS_CHIP &&
        ptChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate arguments */
    if ((RX != ptSegment->block.flow) || 
        ((ixf_eSONET != ptSegment->block.block) && 
         (ixf_eAPS != ptSegment->block.block) ))
    {
        return bb_INV_CHIP_SEGMENT;
    }
    if (NULL == pCounterData)
    {
        return bb_NULL_ARG;
    }

    /* Cast buffer to pointer structure type.  Note that caller must ensure
    * that the buffer is large enough.
    */
    ptCounters = (bb_RxSonetCounters_t *)pCounterData;

    switch(eCounter){
        case bb_OOF_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_OOF_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mOOF_CNT)
                    >> ixf18100_sOOF_CNT);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_OOF_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mOOF_CNT)
                    >> ixf18100_sOOF_CNT);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_B1_BIP_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B1BIP_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                    >> ixf18100_sB1_ERR_CNT);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B1BIP_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                    >> ixf18100_sB1_ERR_CNT);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_B1_BLOCK_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B1BLK_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                    >> ixf18100_sB1_ERR_CNT);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B1BLK_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                    >> ixf18100_sB1_ERR_CNT);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_B2_BIP_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_RL_B2BIP_ECNT_W0_OFFSET,
                        ixf18100_mB2_BIP_CNT_W1, ixf18100_mB2_BIP_CNT_W0);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B2BIP_ECNT_W0_OFFSET,
                    ixf18100_mB2_BIP_CNT_W1, ixf18100_mB2_BIP_CNT_W0);
            }

            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_B2_BLOCK_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_RL_B2BLK_ECNT_W0_OFFSET,
                    ixf18100_mB2_BLK_CNT_W1, ixf18100_mB2_BLK_CNT_W0);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B2BLK_ECNT_W0_OFFSET,
                    ixf18100_mB2_BLK_CNT_W1, ixf18100_mB2_BLK_CNT_W0);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_MST_REI_BIP_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_RL_REIL_BIP_CNT_W0_OFFSET,
                    ixf18100_mREIL_BIP_CNT_W1, ixf18100_mREIL_BIP_CNT_W0);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_APS_RL_REIL_BIP_CNT_W0_OFFSET,
                    ixf18100_mREIL_BIP_CNT_W1, ixf18100_mREIL_BIP_CNT_W0);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_MST_REI_BLOCK_ERR_CNT:      
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_RL_REIL_BLK_CNT_W0_OFFSET,
                    ixf18100_mREIL_BLK_CNT_W1, ixf18100_mREIL_BLK_CNT_W0);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                counter32 = (ulong *)pCounterData;

                *counter32 = Ixf18100SonetRead32(ptChipData->BaseAddress
                    + ixf18100_APS_RL_REIL_BLK_CNT_W0_OFFSET,
                    ixf18100_mREIL_BLK_CNT_W1, ixf18100_mREIL_BLK_CNT_W0);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;
    
        case bb_IN_AU_NCNT:             
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {           
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_IN_NJC_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mIN_NJ_CNT)
                    >> ixf18100_sIN_NJ_CNT);                
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_IN_AU_PCNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_IN_PJC_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mIN_PJ_CNT)
                    >> ixf18100_sIN_PJ_CNT);
            }    
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_HPM_REI_BLOCK_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_REIP_BLK_CNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mREIP_BLK_CNT)
                    >> ixf18100_sREIP_BLK_CNT);
            }    
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_HPM_REI_BIP_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_REIP_BIP_CNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mREIP_BIP_CNT)
                    >> ixf18100_sREIP_BIP_CNT);
            }    
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;
        
        case bb_B3_BLOCK_ERR_CNT:
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B3_BLK_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB3_BLK_CNT)
                    >> ixf18100_sB3_BLK_CNT);
            }    
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_B3_BIP_ERR_CNT :
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                counter16 = (ushort *)pCounterData;

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B3_BIP_ECNT_OFFSET);

                *counter16 = (ushort)((uiValue16 & ixf18100_mB3_BIP_CNT)
                    >> ixf18100_sB3_BIP_CNT);
            }    
            else if(ptSegment->block.block == ixf_eAPS)
            {
                return bb_INV_SEL_COUNTERS;
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case bb_SONET_COUNTERS:
        /* Fill counters structure with SONET counters. */

        /* Disable preemption so a consistent snapshot of counters is taken.
        ** Note that if counters are also read at interrupt level, interrupts
        ** will have to be disabled as well.
        */
            OS_CMD_DISABLE_PREEMPTION();
            if(ptSegment->block.block == ixf_eSONET)
            {
                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_OOF_ECNT_OFFSET);

                ptCounters->tRxRstCounters.OofCnt
                    = (ushort)((uiValue16 & ixf18100_mOOF_CNT)
                        >> ixf18100_sOOF_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B1BIP_ECNT_OFFSET);

                ptCounters->tRxRstCounters.B1ErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                        >> ixf18100_sB1_ERR_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B1BLK_ECNT_OFFSET);

                ptCounters->tRxRstCounters.B1BlockErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB1_BLK_CNT)
                        >> ixf18100_sB1_BLK_CNT);

                /* Mst/Line counters are > 16 bits, so each must be read from
                ** two registers in the correct sequence. */

                ptCounters->tRxMstCounters.B2BlockErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_RL_B2BLK_ECNT_W0_OFFSET,
                        ixf18100_mB2_BLK_CNT_W1, ixf18100_mB2_BLK_CNT_W0);

                ptCounters->tRxMstCounters.B2BitErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_RL_B2BIP_ECNT_W0_OFFSET,
                        ixf18100_mB2_BIP_CNT_W1, ixf18100_mB2_BIP_CNT_W0);

                ptCounters->tRxMstCounters.ReiBlockErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_RL_REIL_BLK_CNT_W0_OFFSET,
                        ixf18100_mREIL_BLK_CNT_W1, ixf18100_mREIL_BLK_CNT_W0);

                ptCounters->tRxMstCounters.ReiBitErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_RL_REIL_BIP_CNT_W0_OFFSET,
                        ixf18100_mREIL_BIP_CNT_W1, ixf18100_mREIL_BIP_CNT_W0);

                /* Path Counters */
                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B3_BIP_ECNT_OFFSET);
                ptCounters->tRxPathCounters.B3BitErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB3_BIP_CNT)
                        >> ixf18100_sB3_BIP_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_B3_BLK_ECNT_OFFSET);
                ptCounters->tRxPathCounters.B3BlockErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB3_BLK_CNT)
                    >> ixf18100_sB3_BLK_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_REIP_BIP_CNT_OFFSET);
                ptCounters->tRxPathCounters.ReiBitCnt
                    = (ushort)((uiValue16 & ixf18100_mREIP_BIP_CNT)
                        >> ixf18100_sREIP_BIP_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_REIP_BLK_CNT_OFFSET);
                ptCounters->tRxPathCounters.ReiBlockCnt
                    = (ushort)((uiValue16 & ixf18100_mREIP_BLK_CNT)
                        >> ixf18100_sREIP_BLK_CNT);

                /* Pointer Counters */
                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_IN_PJC_OFFSET);
                ptCounters->tPointerCounters.InAuPosJustCnt
                    = (ushort)((uiValue16 & ixf18100_mIN_PJ_CNT)
                        >> ixf18100_sIN_PJ_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_RL_IN_NJC_OFFSET);
                ptCounters->tPointerCounters.InAuNegJustCnt
                    = (ushort)((uiValue16 & ixf18100_mIN_NJ_CNT)
                        >> ixf18100_sIN_NJ_CNT);
            }
            else if(ptSegment->block.block == ixf_eAPS)
            {
                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_OOF_ECNT_OFFSET);
                ptCounters->tRxRstCounters.OofCnt
                    = (ushort)((uiValue16 & ixf18100_mOOF_CNT)
                        >> ixf18100_sOOF_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B1BIP_ECNT_OFFSET);
                ptCounters->tRxRstCounters.B1ErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB1_ERR_CNT)
                        >> ixf18100_sB1_ERR_CNT);

                uiValue16 = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_APS_RL_B1BLK_ECNT_OFFSET);
                ptCounters->tRxRstCounters.B1BlockErrCnt
                    = (ushort)((uiValue16 & ixf18100_mB1_BLK_CNT)
                        >> ixf18100_sB1_BLK_CNT);

                /* Mst/Line counters are > 16 bits, so each must be read from
                ** two registers in the correct sequence. */

                ptCounters->tRxMstCounters.B2BlockErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_APS_RL_B2BLK_ECNT_W0_OFFSET,
                        ixf18100_mB2_BLK_CNT_W1, ixf18100_mB2_BLK_CNT_W0);

                ptCounters->tRxMstCounters.B2BitErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_APS_RL_B2BIP_ECNT_W0_OFFSET,
                        ixf18100_mB2_BIP_CNT_W1, ixf18100_mB2_BIP_CNT_W0);

                ptCounters->tRxMstCounters.ReiBlockErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_APS_RL_REIL_BLK_CNT_W0_OFFSET,
                        ixf18100_mREIL_BLK_CNT_W1, ixf18100_mREIL_BLK_CNT_W0);

                ptCounters->tRxMstCounters.ReiBitErrCnt
                    = Ixf18100SonetRead32(ptChipData->BaseAddress
                        + ixf18100_APS_RL_REIL_BIP_CNT_W0_OFFSET,
                        ixf18100_mREIL_BIP_CNT_W1, ixf18100_mREIL_BIP_CNT_W0);

                /* These Counters are not valid for APS section */
                ptCounters->tPointerCounters.InAuPosJustCnt = 0;
                ptCounters->tPointerCounters.InAuNegJustCnt = 0;
            }
            OS_CMD_ENABLE_PREEMPTION();

            /* Output pointer adjustment counters not supported for ixf18100
            ** family, so set to 0 */
            ptCounters->tPointerCounters.OutAuPosJustCnt = 0;
            ptCounters->tPointerCounters.OutAuNegJustCnt = 0;
            break;

        default:
            return bb_INV_SEL_COUNTERS;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetTrace
*------------------------------------------------------------------------------
* Description : Reads traces (J0 or J1 string) from the device
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block
*           bb_TraceType_e eTraceType  :  selects with trace to read.
*
* Outputs : pTrace - pointer to buffer containing trace string.
*           pLength - pointer to uint16 that contains the length of the
*               string in bytes, including trailing NUL
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_PARAMETER, bb_INV_TRACE_FORMAT, bb_INV_TRACE_TYPE
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e 
Ixf18100SonetGetTrace(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                      bb_TraceType_e eTraceType, char *pTrace,
                      ushort *pLength)
{
    uchar ucTraceLen;
    bb_Error_e eReturn = bb_NO_ERROR;
    ushort uiValue, uiCfgOffset, uiAstraOffset, uiEstraOffset;

    /* Validate ChipType */
    if (ptChipData->ChipType != bb_18101_CHIP &&
        ptChipData->ChipType != bb_18102_CHIP &&
        ptChipData->ChipType != bb_18103_CHIP &&
        ptChipData->ChipType != bb_18101_8023WIS_CHIP &&
        ptChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate block ptSegment->block.block */
    if (ptSegment->block.block != ixf_eSONET
        && ptSegment->block.block != ixf_eAPS)
    {
       return bb_INV_CHIP_SEGMENT;    
    }
      
    switch (eTraceType)
    {
        /* The Rx strings are fairly straightforward - we just need to check
        ** the trace config bits, and then read the appropriate number of 
        ** characters from the trace buffer.
        */
        case bb_ACCEPTED_J1:
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_PARAMETER;
            else
            {
                eReturn
                    = Ixf18100SonetGetRxTraceLength(ptChipData->BaseAddress
                        + ixf18100_RL_J1_CFG_OFFSET, &ucTraceLen);

                if (eReturn == bb_NO_ERROR)
                {
                    eReturn = Ixf18100SonetReadRxTraceString(
                        ptChipData->BaseAddress + ixf18100_RL_J1_ASTRA_OFFSET,
                        pTrace, ucTraceLen);
                }
                *pLength = ucTraceLen;
            }
            break;

        case bb_EXPECTED_J1:
            if (ptSegment->block.block != ixf_eSONET)
            {
                eReturn = bb_INV_CHIP_SEGMENT;
            }
            else
            {
                eReturn
                    = Ixf18100SonetGetRxTraceLength(ptChipData->BaseAddress
                        + ixf18100_RL_J1_CFG_OFFSET, &ucTraceLen);

                if (eReturn == bb_NO_ERROR)
                {
                    eReturn = Ixf18100SonetReadRxTraceString(
                        ptChipData->BaseAddress + ixf18100_RL_J1_ESTRA_OFFSET,
                        pTrace, ucTraceLen);
                }

                *pLength = ucTraceLen;
            }
            break;

        case bb_ACCEPTED_J0:
            if (ptSegment->block.block == ixf_eSONET)
            {
                uiCfgOffset = ixf18100_RL_J0_CFG_OFFSET;
                uiAstraOffset = ixf18100_RL_J0_ASTRA_OFFSET;
            }
            else
            {
                if (ptSegment->block.block == ixf_eAPS)
                {
                    uiCfgOffset = ixf18100_APS_RL_J0_CFG_OFFSET;
                    uiAstraOffset = ixf18100_APS_RL_J0_ASTRA_OFFSET;    
                }    
                else
                {
                    /* should never reach this condition as segment is
                    checked upon function entry */
                    eReturn = bb_INV_CHIP_SEGMENT;
                    break;
                }
            }
            eReturn = Ixf18100SonetGetRxTraceLength(ptChipData->BaseAddress
                + uiCfgOffset, &ucTraceLen);

            if (eReturn == bb_NO_ERROR)
            {
                eReturn = Ixf18100SonetReadRxTraceString(
                    ptChipData->BaseAddress + uiAstraOffset,
                    pTrace, ucTraceLen);
            }

            if (eReturn == bb_NO_ERROR)
            {
                *pLength = ucTraceLen;
            }
            break;

        case bb_EXPECTED_J0:
            if (ptSegment->block.block == ixf_eSONET)
            {
                uiCfgOffset = ixf18100_RL_J0_CFG_OFFSET;
                uiEstraOffset = ixf18100_RL_J0_ESTRA_OFFSET;
            }
            else
            {
                if (ptSegment->block.block == ixf_eAPS)
                {
                    uiCfgOffset = ixf18100_APS_RL_J0_CFG_OFFSET;
                    uiEstraOffset = ixf18100_APS_RL_J0_ESTRA_OFFSET;    
                }    
                else
                {
                    /* should never reach this condition as segment is checked
                    upon function entry */
                    eReturn = bb_INV_CHIP_SEGMENT;
                    break;
                }
            }

            eReturn = Ixf18100SonetGetRxTraceLength(ptChipData->BaseAddress
                + uiCfgOffset, &ucTraceLen);

            if (eReturn == bb_NO_ERROR)
            {
                eReturn = Ixf18100SonetReadRxTraceString(
                    ptChipData->BaseAddress + uiEstraOffset, pTrace,
                        ucTraceLen);
            }

            *pLength = ucTraceLen;
            break;

        case bb_TRANSMIT_J0:
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_CHIP_SEGMENT;
            else
            {
                uiValue = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_TL_TOH_CFG_0_OFFSET);

                uiValue &= ixf18100_mTL_J0_CFG;
                switch (uiValue)
                {
                    case ixf18100_J0_CFG_1_BYTE_TRC_ID:
                        *pLength = 1;
                        eReturn = Ixf18100SonetReadTxTraceString(
                            ptChipData->BaseAddress
                                + ixf18100_TL_J0_STRA_OFFSET, pTrace,
                                (uchar)*pLength);
                        break;

                    case ixf18100_J0_CFG_16_BYTE_TRC_ID:
                        *pLength = 16;
                        eReturn = Ixf18100SonetReadTxTraceString(
                            ptChipData->BaseAddress
                                + ixf18100_TL_J0_STRA_OFFSET, pTrace,
                                (uchar)*pLength);
                        break;

                    case ixf18100_J0_CFG_64_BYTE_FORMAT:
                        *pLength = 64;
                        eReturn = Ixf18100SonetReadTxTraceString(
                            ptChipData->BaseAddress
                                + ixf18100_TL_J0_STRA_OFFSET, pTrace,
                                (uchar)*pLength);
                        break;

                    default:
                        eReturn = bb_INV_TRACE_FORMAT;
                        break;
                }
            }
            break;

        case bb_TRANSMIT_J1_COMPOSE:
        case bb_TRANSMIT_J1:
            if (ptSegment->block.block != ixf_eSONET)
            {
                eReturn = bb_INV_CHIP_SEGMENT;
            }
            else
            {
                uiValue = HAL_IXF18100_READ(ptChipData->BaseAddress
                    + ixf18100_TL_POH_SCA_J1_OFFSET);

                if ((uiValue & ixf18100_mTL_POH_SRC)
                    == ixf18100_POH_SRC_MICRO_PROC)
                {
                    /* J1 is sourced from the single byte register,
                    so read it and return it */
                    uiValue = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_J1_OFFSET);

                    pTrace[0] = (uchar)((uiValue & ixf18100_mTL_POH_BVA)
                        >> ixf18100_sTL_POH_BVA);

                    pTrace[1] = 0;  /* Nul terminate to make C string */
                    *pLength = 1;
                    eReturn = bb_NO_ERROR;
                }
                else 
                {
                    ushort *puiTraceReg;

                    /* We are sending a multi-byte string, so determine it's
                    length, location (compose or active) and read it. */
                    if (bb_TRANSMIT_J1_COMPOSE == eTraceType)
                        puiTraceReg = ptChipData->BaseAddress
                            + ixf18100_TL_J1_CMPA_OFFSET;
                    else
                        puiTraceReg = ptChipData->BaseAddress
                            + ixf18100_TL_J1_STRA_OFFSET;
            
                    uiValue = HAL_IXF18100_READ(ptChipData->BaseAddress
                        + ixf18100_TL_POH_CFG_OFFSET);

                    /* Get length from config register Bit set
                    -> 64 byte string, else 16 byte */
                    *pLength
                        = (uiValue & ixf_BIT(ixf18100_sTL_J1_CFG)) ? 64 : 16;

                    /* Now read the string */
                    eReturn = Ixf18100SonetReadTxTraceString(puiTraceReg,
                        pTrace, (uchar)*pLength);
                }
            }
            break;

        default:
            /* We read an invalid trace format from the device when
            determining the trace format, so return an error. */
            eReturn = bb_INV_TRACE_TYPE;
            break;
    }
    return eReturn;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxTraceLength
*------------------------------------------------------------------------------
* Description : Reads the trace length from the device.
*------------------------------------------------------------------------------
* Inputs  : ushort *puiTraceCfgReg : pointer to trace configuration register.
*
* Outputs : uchar *pucTraceLength : pointer to variable to store trace length. 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_TRACE_FORMAT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxTraceLength(ushort *puiTraceCfgReg, uchar *pucTraceLength)
{
    ushort uiTraceCfg;

    uiTraceCfg = HAL_IXF18100_READ(puiTraceCfgReg);
    uiTraceCfg &= ixf18100_mRL_TRACE_CFG;

    switch (uiTraceCfg)
    {
        case ixf18100_RL_TRACE_CFG_FRAMED_64_LF_CR:
        case ixf18100_RL_TRACE_CFG_FREE_64:
            *pucTraceLength = 64;
            break;

        case ixf18100_RL_TRACE_CFG_16_CRC7:
            *pucTraceLength = 16;
            break;

        case ixf18100_RL_TRACE_CFG_1_BYTE:  
            *pucTraceLength = 1;
            break;

        case ixf18100_RL_TRACE_CFG_NO_TRACE:
            *pucTraceLength = 0;
            break;

        default:
            /* Unrecognized value read from device, return error */
            return bb_INV_TRACE_FORMAT;
            break;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetSetTrace
*------------------------------------------------------------------------------
* Description : Inserts traces in the device
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t* ptChipData : pointer to chip data structure.
*           bb_ChipSegment_t *ptSegment : Must select SONET block
*           bb_TraceType_e eTraceType : selects with trace to read.
*           bb_TraceFormat_e eTraceFormat : Format of the trace to be inserted
*           pTrace - pointer to buffer containing trace string.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_TRACE_FORMAT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments :  This function can be used to configure the RX section to ignore
*       an incoming trace, so it must be able to tolerate a NULL trace pointer
*       being given.  This is checked in the Ixf18100SonetProcessTrace
*       function where a NULL pointer is only allowed for the ignore trace
*       case.
******************************************************************************/
bb_Error_e
Ixf18100SonetSetTrace(bb_ChipData_t *ptChipData, bb_ChipSegment_t *ptSegment,
                      bb_TraceType_e eTraceType,
                      bb_TraceFormat_e eTraceFormat, char *pTrace)
{
    uchar ucTraceString[65];  /* 65 so we can add nul and make it printable */
    uchar ucTraceLength;
    bb_Error_e eReturn;
    ushort uiCfgOffset, uiEstraOffset;

    /* Validate ChipType */
    if (ptChipData->ChipType != bb_18101_CHIP &&
        ptChipData->ChipType != bb_18102_CHIP &&
        ptChipData->ChipType != bb_18103_CHIP &&
        ptChipData->ChipType != bb_18101_8023WIS_CHIP &&
        ptChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate block ptSegment->block.block */
    if (ptSegment->block.block != ixf_eSONET
        && ptSegment->block.block != ixf_eAPS)
    {
       return bb_INV_CHIP_SEGMENT;
    }

    /* Process trace string if needed */
    /* We assume that the trace string is a stream of bytes. 
    ** We will add the CR/LF and CRC as requested.
    ** We will also terminate the string with a NUL character.
    ** This function also validates the trace string pointer, allowing
    ** a null pointer in the case of ignored RX traces.
    ** All following functions use the ucTraceString array, which is
    ** always valid. */
    eReturn = Ixf18100SonetProcessTrace(pTrace, ucTraceString, eTraceFormat,
        &ucTraceLength);

    if (eReturn != bb_NO_ERROR)
        return eReturn;

    /* The trace string has been formatted as required, now write it to
    ** the device, and configure the proper trace format
    */
    switch (eTraceType)
    {
        case bb_ACCEPTED_J1:
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_PARAMETER;
            else
            {
                eReturn = Ixf18100SonetWriteRxTraceString(
                    ptChipData->BaseAddress + ixf18100_RL_J1_ASTRA_OFFSET,
                    ucTraceString, ucTraceLength);

                Ixf18100SonetSetRxTraceFormat(eTraceFormat,
                    ptChipData->BaseAddress + ixf18100_RL_J1_CFG_OFFSET);
            }
            break;

        case bb_EXPECTED_J1:
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_PARAMETER;
            else
            {
                eReturn = Ixf18100SonetWriteRxTraceString(
                    ptChipData->BaseAddress + ixf18100_RL_J1_ESTRA_OFFSET,
                    ucTraceString, ucTraceLength);

                Ixf18100SonetSetRxTraceFormat(eTraceFormat,
                    ptChipData->BaseAddress + ixf18100_RL_J1_CFG_OFFSET);
            }
            break;

        case bb_ACCEPTED_J0:
            if (ptSegment->block.block == ixf_eSONET)
            {
                eReturn = Ixf18100SonetWriteRxTraceString(
                    ptChipData->BaseAddress + ixf18100_RL_J0_ASTRA_OFFSET,
                    ucTraceString, ucTraceLength);

                Ixf18100SonetSetRxTraceFormat(eTraceFormat,
                    ptChipData->BaseAddress + ixf18100_RL_J0_CFG_OFFSET);
            }
            else
            {
                if (ptSegment->block.block == ixf_eAPS)
                {
                    eReturn = Ixf18100SonetWriteRxTraceString(
                        ptChipData->BaseAddress + ixf18100_APS_RL_J0_ASTRA_OFFSET,
                        ucTraceString, ucTraceLength);

                    Ixf18100SonetSetRxTraceFormat(eTraceFormat,
                        ptChipData->BaseAddress + ixf18100_APS_RL_J0_CFG_OFFSET);
                }
                else
                {
                    /* should never reach this condition as segment is
                    checked upon function entry */
                    eReturn = bb_INV_CHIP_SEGMENT;
                    break;
                }
            }
            break;

        case bb_EXPECTED_J0:
            if (ptSegment->block.block == ixf_eSONET)
            {
                uiCfgOffset = ixf18100_RL_J0_CFG_OFFSET;
                uiEstraOffset = ixf18100_RL_J0_ESTRA_OFFSET;
            }
            else
            {
                if (ptSegment->block.block == ixf_eAPS)
                {
                    uiCfgOffset = ixf18100_APS_RL_J0_CFG_OFFSET;
                    uiEstraOffset = ixf18100_APS_RL_J0_ESTRA_OFFSET;    
                }    
                else
                {
                    /* should never reach this condition as segment is
                    checked upon function entry */
                    eReturn = bb_INV_CHIP_SEGMENT;
                    break;
                }
            }
            eReturn = Ixf18100SonetWriteRxTraceString(ptChipData->BaseAddress
                + uiEstraOffset, ucTraceString, ucTraceLength);

            if (eReturn == bb_NO_ERROR)
                eReturn = Ixf18100SonetSetRxTraceFormat(eTraceFormat,
                    ptChipData->BaseAddress + uiCfgOffset);
           break;

            /* These three cases are very similar, and allow writing of the two
            ** different J1 transmit memories in different ways.
            ** bb_TRANSMIT_J1 -  For testing purposes, use this to write the
            ** string directly to the J1 transmit line string (TL_J1_STRA)
            ** bb_TRANSMIT_J1_COMPOSE - This writes to the compose memory
            ** before copying to the active memory.
            */
        case bb_TRANSMIT_J1_COMPOSE:
        case bb_TRANSMIT_J1:
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_CHIP_SEGMENT;
            else
            {
                if (eTraceFormat == bb_1_BYTE
                    || eTraceFormat == bb_DEFAULT_TX_TRACE)
                {   
                    /* Use oh byte rather than trace for single byte values */
                    HAL_IXF18100_WRITE(ptChipData->BaseAddress
                        + ixf18100_TL_POH_BVA_J1_OFFSET,
                        (ushort)(ucTraceString[0] & ixf18100_mTL_POH_BVA));
    
                    /* set source for J1 data */
                    Ixf18100SetBits16(ptChipData->BaseAddress
                        + ixf18100_TL_POH_SCA_J1_OFFSET, ixf18100_mTL_POH_SRC,
                        ixf18100_POH_SRC_MICRO_PROC);

                    /* Trace buffer not used in this case,
                    so settings not changed */
                }
                else 
                {
                    if (eTraceFormat == bb_64_BYTE_WITH_LF_CR   ||
                        eTraceFormat == bb_64_BYTE_FREE_FORM    ||
                        eTraceFormat == bb_16_BYTE_WITH_CRC7)
                    {
                        if (eTraceType == bb_TRANSMIT_J1_COMPOSE)
                        {
                            /* Write to compose string memory */
                            eReturn = Ixf18100SonetWriteTxTraceString(
                                ptChipData->BaseAddress
                                + ixf18100_TL_J1_CMPA_OFFSET, ucTraceString,
                                ucTraceLength);
                        }
                        else
                        {               
                            /* Write to active string memory */
                            /* NOTE: This is for testing ONLY!!! */
                            eReturn = Ixf18100SonetWriteTxTraceString(
                                ptChipData->BaseAddress
                                + ixf18100_TL_J1_STRA_OFFSET, ucTraceString,
                                ucTraceLength);
                        }

                        if (eReturn == bb_NO_ERROR)
                        {
                            /* set source for J1 data to be string buffer */
                            Ixf18100SetBits16(ptChipData->BaseAddress
                                + ixf18100_TL_POH_SCA_J1_OFFSET,
                                ixf18100_mTL_POH_SRC,
                                ixf18100_POH_SRC_HARDWARE);

                            if (eTraceFormat == bb_16_BYTE_WITH_CRC7)
                            {
                                /* Clear bit to indicate 16 byte string */
                                Ixf18100SetBits16(ptChipData->BaseAddress
                                    + ixf18100_TL_POH_CFG_OFFSET,
                                    ixf_BIT(ixf18100_sTL_J1_CFG), 0); 
                            }
                            else
                            {
                                /* Configure for 64 byte string */
                                Ixf18100SetBits16(ptChipData->BaseAddress
                                    + ixf18100_TL_POH_CFG_OFFSET,
                                    ixf_BIT(ixf18100_sTL_J1_CFG),
                                    ixf_BIT(ixf18100_sTL_J1_CFG));
                            }

                            if (eTraceType == bb_TRANSMIT_J1_COMPOSE)
                            {
                                /* Copy trace to active output buffer.  Only
                                ** do this if we are writing the active memory
                                ** via the compose memory. This is the normal
                                ** mode of operation for setting the transmit
                                ** trace. */
                                Ixf18100SetBits16(ptChipData->BaseAddress
                                    + ixf18100_TL_POH_CFG_OFFSET,
                                    ixf_BIT(ixf18100_sLOAD_J1_CMP),
                                    ixf_BIT(ixf18100_sLOAD_J1_CMP));
                            }
                        }
                    }
                    else
                        eReturn = bb_INV_TRACE_FORMAT;
                }
            }
            break;

        case bb_TRANSMIT_J0:
            /* Write string to device */
            if (ptSegment->block.block != ixf_eSONET)
                eReturn = bb_INV_CHIP_SEGMENT;
            else
            {
                eReturn = Ixf18100SonetWriteTxTraceString(
                    ptChipData->BaseAddress + ixf18100_TL_J0_STRA_OFFSET,
                    ucTraceString, ucTraceLength);

                /* set source for J0 data to microprocessor register */
                Ixf18100SetBits16(ptChipData->BaseAddress
                    + ixf18100_TL_SC1_TOH_0_OFFSET, ixf18100_mJ0_SRC,
                    ixf18100_J0_SRC_TL_J0_STRA);

                /* Set trace length */
                if (eTraceFormat == bb_1_BYTE || eTraceFormat == bb_DEFAULT_TX_TRACE)
                {
                    Ixf18100SetBits16(ptChipData->BaseAddress
                        + ixf18100_TL_TOH_CFG_0_OFFSET, ixf18100_mTL_J0_CFG,
                        ixf18100_J0_CFG_1_BYTE_TRC_ID);
                }
                else 
                {
                    if (eTraceFormat == bb_64_BYTE_WITH_LF_CR
                        || eTraceFormat == bb_64_BYTE_FREE_FORM)
                    {
                        Ixf18100SetBits16(ptChipData->BaseAddress
                            + ixf18100_TL_TOH_CFG_0_OFFSET,
                            ixf18100_mTL_J0_CFG,
                            ixf18100_J0_CFG_64_BYTE_FORMAT);
                    }
                    else 
                    {
                        if (eTraceFormat == bb_16_BYTE_WITH_CRC7)
                        {
                            Ixf18100SetBits16(ptChipData->BaseAddress
                                + ixf18100_TL_TOH_CFG_0_OFFSET,
                                ixf18100_mTL_J0_CFG,
                                ixf18100_J0_CFG_16_BYTE_TRC_ID);
                        }
                        else
                            eReturn = bb_INV_TRACE_FORMAT;
                    }
                }
            }
            break;

        default:
            eReturn = bb_INV_TRACE_FORMAT;
            break;

    }   /* switch (eTraceType) */

    return eReturn;
}

/******************************************************************************
* Function : Ixf18100SonetSetRxTraceFormat
*------------------------------------------------------------------------------
* Description : Configures the trace format type.
*------------------------------------------------------------------------------
* Inputs  : bb_TraceFormat_e eTraceFormat : trace format type
*           ushort *piTraceCfg : pointer to trace configuration register
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_TRACE_FORMAT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetSetRxTraceFormat(bb_TraceFormat_e eTraceFormat,
                              ushort *piTraceCfg)
{
    /* Set configuration of trace format */
    switch (eTraceFormat)
    {
        case bb_64_BYTE_WITH_LF_CR:
            Ixf18100SetBits16(piTraceCfg, ixf18100_mRL_TRACE_CFG,
                ixf18100_RL_TRACE_CFG_FRAMED_64_LF_CR);
            break;

        case bb_64_BYTE_FREE_FORM:
            Ixf18100SetBits16(piTraceCfg, ixf18100_mRL_TRACE_CFG,
                ixf18100_RL_TRACE_CFG_FREE_64);
            break;

        case bb_16_BYTE_WITH_CRC7:
            Ixf18100SetBits16(piTraceCfg, ixf18100_mRL_TRACE_CFG,
                ixf18100_RL_TRACE_CFG_16_CRC7);
            break;

        case bb_1_BYTE:
        case bb_DEFAULT_TX_TRACE:  
            Ixf18100SetBits16(piTraceCfg, ixf18100_mRL_TRACE_CFG,
                ixf18100_RL_TRACE_CFG_1_BYTE);
            break;

        case bb_IGNORE_RX_TRACE:
            Ixf18100SetBits16(piTraceCfg, ixf18100_mRL_TRACE_CFG,
                ixf18100_RL_TRACE_CFG_NO_TRACE);
            break;

        default:
            return bb_INV_TRACE_FORMAT;
            break;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetRead32
*------------------------------------------------------------------------------
* Description : Reads 32 bit values (counters) from the SONET block
*------------------------------------------------------------------------------
* Inputs  : ushort *puiLsb: pointer to the least significant bits of the
*                   32 bit value to be read.
*           ushort uiMaskMsb: Mask for the upper 16 bits
*           ushort uiMaskLsb: Mask for the lower 16 bits
* Outputs : none
*------------------------------------------------------------------------------
* Returns : 32 bit value containing properly masked value.
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
*   NOTE: preemption or interrupts must be disabled around this function for
*       correct results to be guarenteed.
******************************************************************************/
static ulong
Ixf18100SonetRead32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb)
{

    ulong value32 = 0;
    ushort uiBitsHigh, uiBitsLow;

    /* The high counter bits are stored at the lower address, and must
    ** be read first.  The function takes the address of the word
    ** containing the least significant bits of the counter, so we 
    ** subtract 1 to get the address of the word containing the most
    ** significant bits. */
    uiBitsHigh = HAL_IXF18100_READ(puiLsb - 1);  /* must read this first */
    uiBitsLow = HAL_IXF18100_READ(puiLsb);
    value32 |=  (uiBitsHigh & uiMaskMsb);
    value32 <<= 16;
    value32 |= (uiBitsLow & uiMaskLsb);

    return value32;
}

/******************************************************************************
* Function : Ixf18100SonetWriteTxTraceString
*------------------------------------------------------------------------------
* Description : This functions writes a Tx trace string to the device.  The
*       trace to be written is determined by piTraceReg.
*------------------------------------------------------------------------------
* Inputs  : ushort *piTraceReg : pointer to trace register
*           uchar *pcTrace : trace string
*           uchar ucLength : length of trace string
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetWriteTxTraceString(volatile ushort *piTraceReg, uchar *pcTrace,
                                uchar ucLength)
{
    uchar ucAddress;
    ushort trcValue;
    
    if (NULL == piTraceReg || NULL == pcTrace || ucLength > 64)
    {
        return bb_INV_PARAMETER;
    }

    /* Update address on every write to register */
    for (ucAddress = 0; ucAddress < ucLength; ucAddress++)
    {
        trcValue = ((ucAddress << ixf18100_sTL_TRACE_ADDR)
            & ixf18100_mTL_TRACE_ADDR) | *pcTrace++;  

        HAL_IXF18100_WRITE((bb_RegPointer_type) piTraceReg, trcValue);
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetReadTxTraceString
*------------------------------------------------------------------------------
* Description : This functions reads a Tx trace string from the device.  The
*               trace to be read is determined by puiTraceReg.
*------------------------------------------------------------------------------
* Inputs  : ushort *piTraceReg : pointer to trace register
*           uchar ucLength : length of trace string
* Outputs : uchar *pucTrace : trace string
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments :
******************************************************************************/
static bb_Error_e
Ixf18100SonetReadTxTraceString(volatile ushort *puiTraceReg, uchar *pucTrace,
                               uchar ucLength)
{
    ushort trcValue;
    uchar ucAddress;

    if (NULL == puiTraceReg || NULL == pucTrace || ucLength > 64)
    {
        return bb_INV_PARAMETER;
    }
    /* Put register in read mode, and set address to zero */

    trcValue = ixf_BIT(ixf18100_sDSBL_TL_TRACE_DATA_WR);
    HAL_IXF18100_WRITE((bb_RegPointer_type) puiTraceReg, trcValue);
    
    /* Read each byte, register autoincrements after each read */
    for (ucAddress = 0; ucAddress < ucLength; ucAddress++)
    {
        pucTrace[ucAddress] = (uchar)((HAL_IXF18100_READ((bb_RegPointer_type) puiTraceReg)
            & ixf18100_mTL_TRACE_DATA) >> ixf18100_sTL_TRACE_DATA);
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetWriteRxTraceString
*------------------------------------------------------------------------------
* Description : This functions writes a Rx trace string to the device.  The
*               trace to be written is determined by puiTraceReg.
*------------------------------------------------------------------------------
* Inputs  : ushort *piTraceReg : pointer to trace register
*           uchar *pucTrace : trace string
*           uchar ucLength : length of trace string
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER, bb_TRACE_ACCESS_FAIL
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments :
******************************************************************************/
static bb_Error_e
Ixf18100SonetWriteRxTraceString(volatile ushort *puiTraceReg,
                                uchar *pucTrace, uchar ucLength)
{
    uchar ucAddress;
    ushort uiTimeout;
    ushort trcValue;

    if (NULL == puiTraceReg || NULL == pucTrace || ucLength > 64)
    {
        return bb_INV_PARAMETER;
    }

    /* Update address on every write to register */
    for (ucAddress = 0; ucAddress < ucLength; ucAddress++)
    {

        /* Initiate write */
        trcValue = ixf_BIT(ixf18100_sRL_TRACE_WRITE)
            | ((ucAddress << ixf18100_sRL_TRACE_ADDR)
                & ixf18100_mRL_TRACE_ADDR) | *pucTrace++;
   
        HAL_IXF18100_WRITE((bb_RegPointer_type) puiTraceReg, trcValue);
        /* Wait for write to complete */
        uiTimeout = ixf18100_TRACE_TIMEOUT;

        while (uiTimeout-- > 0 && (HAL_IXF18100_READ((bb_RegPointer_type) puiTraceReg)
            & ixf_BIT(ixf18100_sRL_TRACE_WRITE)))
        {
            /* empty loop */
        }

        if (0 == uiTimeout)
        {
            /* The timeout has expired, return error */
            return bb_TRACE_ACCESS_FAIL;
        }
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetReadRxTraceString
*------------------------------------------------------------------------------
* Description : This functions reads a Rx trace string from the device.  The
*               trace to be read is determined by puiTraceReg.
*------------------------------------------------------------------------------
* Inputs  : ushort *piTraceReg : pointer to trace register
*           uchar ucLength : length of trace string
* Outputs : uchar *pucTrace : trace string
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER, bb_TRACE_ACCESS_FAIL
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetReadRxTraceString(volatile ushort *puiTraceReg,
                               uchar *pucTrace, uchar ucLength)
{
    uchar ucAddress;
    ushort uiTimeout;
    ushort uiTraceVal;
    ushort trcValue;

    if (NULL == puiTraceReg || NULL == pucTrace || ucLength > 64)
    {
        return bb_INV_PARAMETER;
    }

    /* Update address on register access */
    for (ucAddress = 0; ucAddress < ucLength; ucAddress++)
    {

        /* Initiate read by setting read bit and address */
        trcValue = (ushort)(ixf_BIT(ixf18100_sRL_TRACE_READ)
            | ((ucAddress << ixf18100_sRL_TRACE_ADDR)
                & ixf18100_mRL_TRACE_ADDR));

        HAL_IXF18100_WRITE((bb_RegPointer_type) puiTraceReg, trcValue);
        /* Wait for read to complete */
        uiTimeout = ixf18100_TRACE_TIMEOUT;

        while (uiTimeout-- > 0
            && ((uiTraceVal = HAL_IXF18100_READ((bb_RegPointer_type) puiTraceReg))
                & ixf_BIT(ixf18100_sRL_TRACE_READ)))
        {
            /* empty loop */
        }

        if (0 == uiTimeout)
        {
            /* The timeout has expired, return error */
            return bb_TRACE_ACCESS_FAIL;
        }
        else
        {
            /* The read has completed, extract trace value from register */
            pucTrace[ucAddress] = (uchar)((uiTraceVal
                & ixf18100_mRL_TRACE_DATA) >> ixf18100_sRL_TRACE_DATA);
        }

    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetPadTrace
*------------------------------------------------------------------------------
* Description : Takes a user's trace string, and copies it to internal buffer.
*------------------------------------------------------------------------------
* Inputs  : uchar *pucUserArg : user's trace string
*           uchar ucLength : length of trace string to copy
* Outputs : uchar *pucProcessedArg : internal buffer
*------------------------------------------------------------------------------
* Returns : None
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
void
Ixf18100SonetPadTrace(uchar *pucUserArg, uchar *pucProcessedArg,
                      uchar ucLength)
{
    uchar ucCount;
    uchar *pucUser = pucUserArg;
    uchar *pucProcessed = pucProcessedArg;

    ucCount = 0;

    /* copy string up to max length */
    while (ucCount++ < ucLength) 
    {
        *pucProcessed++ = *pucUser++;
    }
}


/******************************************************************************
* Function : Ixf18100SonetProcessTrace
*------------------------------------------------------------------------------
* Description : Processes the trace string according to the trace format.
*------------------------------------------------------------------------------
* Inputs  : uchar *pucUserArg : user trace string
*           bb_TraceFormat_e eTraceFormat : trace format type
* Outputs : uchar *pucProcessedArg : formatted traces tring
*           uchar *pucTraceLength : length of trace string
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_TRACE_FORMAT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments :
******************************************************************************/
static bb_Error_e
Ixf18100SonetProcessTrace(uchar *pucUserArg, uchar *pucProcessedArg,
                          bb_TraceFormat_e eTraceFormat,
                          uchar *pucTraceLength)
{
    if ((NULL == pucUserArg && eTraceFormat != bb_IGNORE_RX_TRACE)
        || NULL == pucProcessedArg || NULL == pucTraceLength)
    {
        return bb_NULL_ARG;
    }

    switch (eTraceFormat)
    {
        case bb_64_BYTE_WITH_LF_CR:
            Ixf18100SonetPadTrace(pucUserArg,pucProcessedArg, 64);
            /* Add CR and LF*/
            pucProcessedArg[62] = '\r';
            pucProcessedArg[63] = '\n';
            pucProcessedArg[64] = 0;  /* End with null to make printable */
            *pucTraceLength = 64;
            break;

        case bb_64_BYTE_FREE_FORM:
            Ixf18100SonetPadTrace(pucUserArg,pucProcessedArg, 64);
            pucProcessedArg[64] = 0;  /* End with null to make printable */
            *pucTraceLength = 64;
            break;

        case bb_16_BYTE_WITH_CRC7:
            Ixf18100SonetPadTrace(pucUserArg,pucProcessedArg, 16);
            /* TBD! compute CRC */
            pucProcessedArg[16] = 0;  /* End with null to make printable */
            *pucTraceLength = 16;
            break;

        case bb_1_BYTE:
            Ixf18100SonetPadTrace(pucUserArg,pucProcessedArg, 1);
            pucProcessedArg[1] = 0;  /* End with null to make printable */
            *pucTraceLength = 1;
            break;

        case bb_DEFAULT_TX_TRACE:
            pucProcessedArg[0] = 0x01;  /* set to default value */
            pucProcessedArg[1] = 0;  /* End with null to make printable */
            *pucTraceLength = 1;
            break;

        case bb_IGNORE_RX_TRACE:
            pucProcessedArg[0] = 0;  /* End with null to make printable */
            *pucTraceLength = 0;
            break;

        default:
            return bb_INV_TRACE_FORMAT;
            break;
    }
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetWindowSize
*------------------------------------------------------------------------------
* Description : Reads the window size for the degraded and excessive error
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t *pChipData: Pointer to the data structure holding
*                                     the base address for the device                   
*           bb_ChipSegment_t *section: Pointer to the data structure to 
*                                      identify the section of the chip that 
*                                      will be modify.
*           bb_WindowSizeMode_e mode: Parameter to identify the kind of
*                                     error (degraded or excessive) if the 
*                                     the window is being cleared or set.
*   
* Outputs : ulong *value: Parameter to store the value of the window size.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT, bb_NULL_ARG
*------------------------------------------------------------------------------
* Access Globals : 
*------------------------------------------------------------------------------
* Comments : 
*   NOTE: preemption or interrupts must be disabled around this function for
*       correct results to be guarenteed.
******************************************************************************/
bb_Error_e 
Ixf18100SonetGetWindowSize(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                           bb_WindowSizeMode_e mode, ulong *value)
{
    /* Validate ChipType */
    if (pChipData->ChipType != bb_18101_CHIP &&
        pChipData->ChipType != bb_18102_CHIP &&
        pChipData->ChipType != bb_18103_CHIP &&
        pChipData->ChipType != bb_18101_8023WIS_CHIP &&
        pChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate arguments */
    if ((RX != section->block.flow) || ( (ixf_eSONET != section->block.block)
        && (ixf_eAPS != section->block.block)))
    {
        return bb_INV_CHIP_SEGMENT;
    }

    if (NULL == value)
    {
        return bb_NULL_ARG;
    }

    *value = 0;

    switch(mode)
    {
        case CLEARING_DEG_B2:
            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                *value = Ixf18100SonetRead32(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_CDEGB2_W0_OFFSET,
                    ixf18100_mDEG_B2_CLR_WIN_SZ_W1,
                    ixf18100_mDEG_B2_CLR_WIN_SZ_W0);
            }
            else if(section->block.block == ixf_eAPS)
            {
                *value = Ixf18100SonetRead32(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_CDEGB2_W0_OFFSET,
                    ixf18100_mDEG_B2_CLR_WIN_SZ_W1,
                    ixf18100_mDEG_B2_CLR_WIN_SZ_W0);
            }

            OS_CMD_ENABLE_PREEMPTION();
            break;

        case SETTING_DEG_B2:
            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                *value = Ixf18100SonetRead32(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_SDEGB2_W0_OFFSET,
                    ixf18100_mDEG_B2_SET_WIN_SZ_W1,
                    ixf18100_mDEG_B2_SET_WIN_SZ_W0);
            }
            else if(section->block.block == ixf_eAPS)
            {
                *value = Ixf18100SonetRead32(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_SDEGB2_W0_OFFSET,
                    ixf18100_mDEG_B2_SET_WIN_SZ_W1,
                    ixf18100_mDEG_B2_SET_WIN_SZ_W0);
            }

            OS_CMD_ENABLE_PREEMPTION();
            break;

        case CLEARING_EXC_B2:
            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                *value = (ulong)HAL_IXF18100_READ(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_CB2_OFFSET)
                        & ixf18100_mEXC_B2_CLR_WIN_SZ;
            }
            else if(section->block.block == ixf_eAPS)
            {
                *value = (ulong)HAL_IXF18100_READ(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_CB2_OFFSET)
                        & ixf18100_mEXC_B2_CLR_WIN_SZ;
            }

            OS_CMD_ENABLE_PREEMPTION();
            break;

        case SETTING_EXC_B2:
            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                *value = (ulong)HAL_IXF18100_READ(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_SB2_OFFSET)
                        & ixf18100_mEXC_B2_SET_WIN_SZ;
            }
            else if(section->block.block == ixf_eAPS)
            {
                *value = (ulong)HAL_IXF18100_READ(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_SB2_OFFSET)
                        & ixf18100_mEXC_B2_SET_WIN_SZ;
            }

            OS_CMD_ENABLE_PREEMPTION();
            break;

        default:
            return bb_INV_PARAMETER;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetSetWindowSize
*------------------------------------------------------------------------------
* Description : Set the window size for the degraded and excessive error
*------------------------------------------------------------------------------
* Inputs  : bb_ChipData_t *pChipData: Pointer to the data structure holding
*                                     the base address for the device                   
*           bb_ChipSegment_t *section: Pointer to the data structure to 
*                                      identify the section of the chip that 
*                                      will be modify.
*           bb_WindowSizeMode_e mode: Parameter to identify the kind of
*                                     error (degraded or excessive) if the 
*                                     the window is being cleared or set.
*           ulong *value: Parameter to store the value of the window size.
* Outputs :  none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE, bb_INV_CHIP_SEGMENT,
*           bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : 
*------------------------------------------------------------------------------
* Comments : 
*   NOTE: preemption or interrupts must be disabled around this function for
*       correct results to be guarenteed.
******************************************************************************/
bb_Error_e 
Ixf18100SonetSetWindowSize(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                           bb_WindowSizeMode_e mode, ulong value)
{
    ushort tempLsb, tempMsb;

    /* Validate ChipType */
    if (pChipData->ChipType != bb_18101_CHIP &&
        pChipData->ChipType != bb_18102_CHIP &&
        pChipData->ChipType != bb_18103_CHIP &&
        pChipData->ChipType != bb_18101_8023WIS_CHIP &&
        pChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return bb_INV_CHIP_TYPE;
    }

    /* Validate arguments */
    if ((RX != section->block.flow) ||( (ixf_eSONET != section->block.block)
        && (ixf_eAPS != section->block.block)))
    {
        return bb_INV_CHIP_SEGMENT;
    }
    
    switch(mode)
    {
        case CLEARING_DEG_B2:
            tempMsb = (ushort)((value>>16) & ixf18100_mDEG_B2_CLR_WIN_SZ_W1);
            tempLsb = (ushort)(value & ixf18100_mDEG_B2_CLR_WIN_SZ_W0);  

            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_CDEGB2_W0_OFFSET, tempLsb);

                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_CDEGB2_W1_OFFSET, tempMsb);
            }
            else if(section->block.block == ixf_eAPS)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_CDEGB2_W0_OFFSET, tempLsb);

                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_CDEGB2_W1_OFFSET, tempMsb);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case SETTING_DEG_B2:
            tempMsb = (ushort)((value>>16) & ixf18100_mDEG_B2_SET_WIN_SZ_W1);
            tempLsb = (ushort)(value & ixf18100_mDEG_B2_SET_WIN_SZ_W0);

            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_SDEGB2_W0_OFFSET, tempLsb);

                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_SDEGB2_W1_OFFSET, tempMsb);
            }
            else if(section->block.block == ixf_eAPS)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_SDEGB2_W0_OFFSET, tempLsb);

                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_SDEGB2_W1_OFFSET, tempMsb);
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case CLEARING_EXC_B2:
            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_CB2_OFFSET,
                        (ushort)(value & ixf18100_mEXC_B2_CLR_WIN_SZ));
            }
            else if(section->block.block == ixf_eAPS)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_CB2_OFFSET,
                        (ushort)(value & ixf18100_mEXC_B2_CLR_WIN_SZ));
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        case SETTING_EXC_B2:

            OS_CMD_DISABLE_PREEMPTION();
            if(section->block.block == ixf_eSONET)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_RL_WINSZ_SB2_OFFSET,
                        (ushort)(value & ixf18100_mEXC_B2_SET_WIN_SZ));
            }
            else if(section->block.block == ixf_eAPS)
            {
                HAL_IXF18100_WRITE(pChipData->BaseAddress
                    + ixf18100_APS_RL_WINSZ_SB2_OFFSET,
                        (ushort)(value & ixf18100_mEXC_B2_SET_WIN_SZ));
            }
            OS_CMD_ENABLE_PREEMPTION();
            break;

        default:
            return bb_INV_PARAMETER;

    }
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 8 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: BaseAddress to the device
*           bb_SelConfig_e SelCfg: Selects the config to be read from device
* Outputs : ixf18100_SonetCfg_t ptSonetCfg: pointer to the Sonet cfg structure
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100SonetGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_SonetCfg_t *ptSonetCfg)
{
    ushort uTemp;

    switch(SelCfg)
    {
        case e_OHT_CONFIG:

            Ixf18100GetPathOhBytes(BaseAddress,ixf_eSONET,TX,
                                   &(ptSonetCfg->PathTxOhBytes));

            Ixf18100GetPathOhBytes(BaseAddress,ixf_eSONET,RX,
                                   &(ptSonetCfg->PathRxOhBytes));

            Ixf18100GetLineOhBytes(BaseAddress,ixf_eSONET,TX,
                                   &(ptSonetCfg->RstTxMstBytes));

            Ixf18100GetLineOhBytes(BaseAddress,ixf_eSONET,RX,
                                   &(ptSonetCfg->RstRxMstBytes));

            /*APS OHT Bytes */
            Ixf18100GetPathOhBytes(BaseAddress,ixf_eAPS,TX,
                &(ptSonetCfg->SonetApsPathOhBytes.PathTxOhBytes));

            Ixf18100GetPathOhBytes(BaseAddress,ixf_eAPS,RX,
                &(ptSonetCfg->SonetApsPathOhBytes.PathRxOhBytes));
            break;

        case e_POINTER_CONFIG:
            uTemp = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_PT_CFG_OFFSET);

            ptSonetCfg->PointerCfg.Ais2pAisEn
                = (uTemp & ixf_BIT(ixf18100_sAIS2P_AIS_EN))
                    >> ixf18100_sAIS2P_AIS_EN;

            ptSonetCfg->PointerCfg.Aisc2pAisEn
                = (uTemp & ixf_BIT(ixf18100_sAISC2P_AIS_EN))
                    >> ixf18100_sAISC2P_AIS_EN;

            ptSonetCfg->PointerCfg.Lopc2pAisEn
                = (uTemp & ixf_BIT(ixf18100_sLOPC2P_AIS_EN))
                    >> ixf18100_sLOPC2P_AIS_EN;

            ptSonetCfg->PointerCfg.Lop2pAisEn
                = (uTemp & ixf_BIT(ixf18100_sLOP2P_AIS_EN))
                    >> ixf18100_sLOP2P_AIS_EN;

            ptSonetCfg->PointerCfg.PtrJustCnfg
                = (uTemp & ixf_BIT(ixf18100_sPNTR_JUST_CFG))
                    >> ixf18100_sPNTR_JUST_CFG;

            ptSonetCfg->PointerCfg.PtrSsEn
                = (uTemp & ixf_BIT(ixf18100_sPNTR_SS_EN))
                    >> ixf18100_sPNTR_SS_EN;

            ptSonetCfg->PointerCfg.ExpcPtrSs
                = (uTemp & ixf18100_mEXPC_PNTR_SS)
                    >> ixf18100_sEXPC_PNTR_SS;
            break;

        case e_SERIAL_PORT_CONFIG:/*TBD*/
            break;

        default:
            return bb_INV_PARAMETER;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from Sonet config 
* structure and appropriately writes the values into the corresponding 
* registers.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: BaseAddress to the device
*           bb_SelConfig_e SelCfg: Selects the config to be read from device
*           ixf18100_SonetCfg_t ptSonetCfg: pointer to the Sonet cfg structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e 
Ixf18100SonetSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_SonetCfg_t *ptSonetCfg)
{
   ushort uTemp;

    switch(SelCfg)
    {
        case e_OHT_CONFIG:
            Ixf18100SetPathOhBytes(BaseAddress,ixf_eSONET, TX,
                                   &(ptSonetCfg->PathTxOhBytes));

            Ixf18100SetPathOhBytes(BaseAddress,ixf_eSONET, RX,
                                   &(ptSonetCfg->PathRxOhBytes));

            Ixf18100SetLineOhBytes(BaseAddress,ixf_eSONET, TX,
                                   &(ptSonetCfg->RstTxMstBytes));

            Ixf18100SetLineOhBytes(BaseAddress,ixf_eSONET, RX,
                                   &(ptSonetCfg->RstRxMstBytes));

            /*APS OHT Bytes */
            Ixf18100SetPathOhBytes(BaseAddress,ixf_eAPS, TX,
                &(ptSonetCfg->SonetApsPathOhBytes.PathTxOhBytes));

            Ixf18100SetPathOhBytes(BaseAddress,ixf_eAPS, RX,
                &(ptSonetCfg->SonetApsPathOhBytes.PathRxOhBytes));
            break;

        case e_POINTER_CONFIG:
            uTemp = 0;

            uTemp |= (ptSonetCfg->PointerCfg.Ais2pAisEn
                << ixf18100_sAIS2P_AIS_EN);

            uTemp |= (ptSonetCfg->PointerCfg.Aisc2pAisEn
                << ixf18100_sAISC2P_AIS_EN);

            uTemp |= (ptSonetCfg->PointerCfg.Lopc2pAisEn
                << ixf18100_sLOPC2P_AIS_EN);

            uTemp |= (ptSonetCfg->PointerCfg.Lop2pAisEn
                << ixf18100_sLOP2P_AIS_EN);

            uTemp |= (ptSonetCfg->PointerCfg.PtrJustCnfg
                << ixf18100_sPNTR_JUST_CFG);

            uTemp |= (ptSonetCfg->PointerCfg.PtrSsEn
                << ixf18100_sPNTR_SS_EN);

            uTemp |= ((ptSonetCfg->PointerCfg.ExpcPtrSs
                & ixf18100_mEXPC_PNTR_SS) << ixf18100_sEXPC_PNTR_SS);

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_RL_PT_CFG_OFFSET,
                uTemp);
            break;

        default:
            return bb_INV_PARAMETER;
    }
    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetStatus
*------------------------------------------------------------------------------
* Description  : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the device.
*           section   : Determines the section(block) to get the status from.
*           selStatus : Type of Status: 
*			void *ptSonetStatus: pointer to the status
*				that needs to be read from the device
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_NULL_ARG, bb_INV_SEL_STATUS_TYPE,
*           bb_INV_CHI_SEGMENT
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100SonetGetStatus(bb_RegPointer_type BaseAddress,
                       bb_ChipSegment_t *ptSegment, bb_SelStatus_e selStatus,
                       void* pStatus)
{
    bb_Error_e  rc = bb_NO_ERROR;

    ixf18100_SonetRxSectionOhStatus_t       *RxSOHStat;
    ixf18100_SonetRxLineOhStatus_t          *RxLOHStat;
    ixf18100_SonetRxPathOhStatus_t          *RxPOHStat;
    ixf18100_SonetRxPtrTrackerStatus_t      *RxPtrStat;
    ixf18100_SonetAPSRxSectionOhStatus_t    *RxAPSSOHStat;
    ixf18100_SonetAPSRxLineOhStatus_t       *RxAPSLOHStat;
    ixf18100_SonetRxStatus_t                *RxAllStat;
    ixf18100_SonetTxPtrTrackerStatus_t      *TxPtrStat;
    ixf18100_SonetTxStatus_t                *TxAllStat;
    ixf18100_SonetStatus_t                  *SonetStat;

    if (NULL == pStatus)
    {
        return bb_NULL_ARG;
    }

    switch (ptSegment->block.flow)
    {
    case RX:
        switch (selStatus)
        {
        case ixf_eSECTION_OH_STATUS:
            RxSOHStat = (ixf18100_SonetRxSectionOhStatus_t *)pStatus;
            RxAPSSOHStat = (ixf18100_SonetAPSRxSectionOhStatus_t *)pStatus;

            rc = Ixf18100SonetGetRxSOHStatus(BaseAddress, RxSOHStat);
            rc = Ixf18100SonetGetRxAPSSOHStatus(BaseAddress, RxAPSSOHStat);
            break;

        case ixf_eLINE_OH_STATUS:
            RxLOHStat = (ixf18100_SonetRxLineOhStatus_t *)pStatus;
            RxAPSLOHStat = (ixf18100_SonetAPSRxLineOhStatus_t *)pStatus;

            rc = Ixf18100SonetGetRxLOHStatus(BaseAddress, RxLOHStat);
            rc = Ixf18100SonetGetRxAPSLOHStatus(BaseAddress, RxAPSLOHStat);
            break;

        case ixf_ePATH_OH_STATUS:
            RxPOHStat = (ixf18100_SonetRxPathOhStatus_t *)pStatus;

            rc = Ixf18100SonetGetRxPOHStatus(BaseAddress, RxPOHStat);
            break;

        case ixf_ePOINTER_STATUS:
            RxPtrStat = (ixf18100_SonetRxPtrTrackerStatus_t *)pStatus;

            rc = Ixf18100SonetGetRxPtrStatus(BaseAddress, RxPtrStat);
            break;

        case ixf_eALL_STATUS:
            RxAllStat = (ixf18100_SonetRxStatus_t *)pStatus;

            rc = Ixf18100SonetGetRxSOHStatus(BaseAddress,
                &(RxAllStat->SOHStat));
            rc = Ixf18100SonetGetRxAPSSOHStatus(BaseAddress,
                &(RxAllStat->APSSOHStat));
            rc = Ixf18100SonetGetRxLOHStatus(BaseAddress,
                &(RxAllStat->LOHStat));
            rc = Ixf18100SonetGetRxAPSLOHStatus(BaseAddress,
                &(RxAllStat->APSLOHStat));
            rc = Ixf18100SonetGetRxPOHStatus(BaseAddress,
                &(RxAllStat->POHStat));
            rc = Ixf18100SonetGetRxPtrStatus(BaseAddress,
                &(RxAllStat->PtrStat));
            break;

        default:
            rc = bb_INV_SEL_STATUS_TYPE;
            break;
        }
        break;

    case TX:
        switch (selStatus)
        {
        case ixf_ePOINTER_STATUS:
            TxPtrStat = (ixf18100_SonetTxPtrTrackerStatus_t *)pStatus;

            rc = Ixf18100SonetGetTxPtrStatus(BaseAddress, TxPtrStat);
            break;

        case ixf_eALL_STATUS:
            TxAllStat = (ixf18100_SonetTxStatus_t *)pStatus;

            rc = Ixf18100SonetGetTxPtrStatus(BaseAddress,
                &(TxAllStat->PtrStat));
            break;

        default:
            rc = bb_INV_SEL_STATUS_TYPE;
            break;
        }
        break;

    case BOTH_FLOWS:
        SonetStat = (ixf18100_SonetStatus_t *)pStatus;

        switch (selStatus)
        {
        case ixf_eMISC_STATUS:
            rc = Ixf18100SonetGetMiscStatus(BaseAddress,
                &(SonetStat->SonetMiscStat));
            break;

        case ixf_eSECTION_OH_STATUS:
            rc = Ixf18100SonetGetRxSOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.SOHStat));
            rc = Ixf18100SonetGetRxAPSSOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.APSSOHStat));
            break;

        case ixf_eLINE_OH_STATUS:
            rc = Ixf18100SonetGetRxLOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.LOHStat));
            rc = Ixf18100SonetGetRxAPSLOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.APSLOHStat));
            break;

        case ixf_ePATH_OH_STATUS:
            rc = Ixf18100SonetGetRxPOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.POHStat));
            break;

        case ixf_ePOINTER_STATUS:
            rc = Ixf18100SonetGetRxPtrStatus(BaseAddress,
                &(SonetStat->SonetRxStat.PtrStat));
            rc = Ixf18100SonetGetTxPtrStatus(BaseAddress,
                &(SonetStat->SonetTxStat.PtrStat));
            break;

        case ixf_eALL_STATUS:
            rc = Ixf18100SonetGetMiscStatus(BaseAddress,
                &(SonetStat->SonetMiscStat));

            rc = Ixf18100SonetGetRxSOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.SOHStat));
            rc = Ixf18100SonetGetRxAPSSOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.APSSOHStat));

            rc = Ixf18100SonetGetRxLOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.LOHStat));
            rc = Ixf18100SonetGetRxAPSLOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.APSLOHStat));

            rc = Ixf18100SonetGetRxPOHStatus(BaseAddress,
                &(SonetStat->SonetRxStat.POHStat));

            rc = Ixf18100SonetGetRxPtrStatus(BaseAddress,
                &(SonetStat->SonetRxStat.PtrStat));
            rc = Ixf18100SonetGetTxPtrStatus(BaseAddress,
                &(SonetStat->SonetTxStat.PtrStat));
            break;

        default:
            rc = bb_INV_SEL_STATUS_TYPE;
            break;
        }
        break;

    default:
        rc = bb_INV_CHIP_SEGMENT;
        break;
    }

	return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100GetPathOhBytes
*------------------------------------------------------------------------------
* Description : Gets the Sonet path overhead bytes from the device.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          ptSegment: Pointer to chip segment structure.
*          flow: flow of the device
* Outputs : ptPathOh: Pointer to the POH structure
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100GetPathOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_PathOhBytes_t *ptPathOh)
{
    if(block == ixf_eSONET)
    {                     
        if (TX == flow)
        {
            ptPathOh->ucJ1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_J1_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucC2 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_C2_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucB3 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_B3_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucG1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_G1_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucF2 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_F2_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucH4 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_H4_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucZ3F3 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_Z3_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucZ4K3 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_Z4_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);

            ptPathOh->ucZ5N1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_POH_BVA_Z5_OFFSET) & ixf18100_mTL_POH_BVA)
                    >> ixf18100_sTL_POH_BVA);
        }
        else
        {
            /* J1 only available through string access on RL side */
            ptPathOh->ucJ1 = 0;

            /* Return the Received C2 here.  Expected is handled separately */
            ptPathOh->ucC2 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_C2_OFFSET) & ixf18100_mC2) >> ixf18100_sC2);

            ptPathOh->ucB3 = 0;  /* B3 not available on RL side. */

            ptPathOh->ucG1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_G1_OFFSET) & ixf18100_mG1) >> ixf18100_sG1);

            ptPathOh->ucF2 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_F2_OFFSET) & ixf18100_mF2) >> ixf18100_sF2);

            ptPathOh->ucH4 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_H4_OFFSET) & ixf18100_mH4) >> ixf18100_sH4);

            ptPathOh->ucZ3F3 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_Z3_F3_OFFSET) & ixf18100_mZ3_F3)
                    >> ixf18100_sZ3_F3);

            ptPathOh->ucZ4K3 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_Z4_K3_OFFSET) & ixf18100_mZ4_K3)
                    >> ixf18100_sZ4_K3);

            ptPathOh->ucZ5N1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_RCV_Z5_N1_OFFSET) & ixf18100_mZ5_N1)
                    >> ixf18100_sZ5_N1);
        }
    }
    else if(block == ixf_eAPS)
    {
        ptPathOh->ucJ1 = 0;
        ptPathOh->ucC2 = 0;
        ptPathOh->ucB3 = 0;
        ptPathOh->ucG1 = 0;
        ptPathOh->ucF2 = 0;
        ptPathOh->ucH4 = 0;
        ptPathOh->ucZ3F3 = 0;
        ptPathOh->ucZ4K3 = 0;
        ptPathOh->ucZ5N1 = 0;
    }
    else 
    {
        return bb_INV_CHIP_SEGMENT;
    }

    return bb_NO_ERROR;
}
            
/******************************************************************************
* Function : Ixf18100SetPathOhBytes
*------------------------------------------------------------------------------
* Description : Sets the path overhead bytes on the device.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          ptSegment: Pointer to chip segment structure.
*          flow: flow of the device
*          ptPathOh: Pointer to the POH structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/            
bb_Error_e
Ixf18100SetPathOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_PathOhBytes_t *ptPathOh)
{
    if(block == ixf_eSONET)
    {
        if (TX == flow)
        {
            /* Structure members not masked because all are byte types, and */
            /* fields are all one byte in length */
            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_J1_OFFSET,
                (ushort)(ptPathOh->ucJ1 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_C2_OFFSET,
                (ushort)(ptPathOh->ucC2 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_B3_OFFSET,
                (ushort)(ptPathOh->ucB3 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_G1_OFFSET,
                (ushort)(ptPathOh->ucG1 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_F2_OFFSET,
                (ushort)(ptPathOh->ucF2 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_H4_OFFSET,
                (ushort)(ptPathOh->ucH4 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_Z3_OFFSET,
                (ushort)(ptPathOh->ucZ3F3 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_Z4_OFFSET,
                (ushort)(ptPathOh->ucZ4K3 << ixf18100_sTL_POH_BVA));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_POH_BVA_Z5_OFFSET,
                (ushort)(ptPathOh->ucZ5N1 << ixf18100_sTL_POH_BVA));
                
            return bb_NO_ERROR;
        }
        else
        {
            /* No bytes to set on Rx side, except expected C2,
            which is set separately */
            return bb_INV_SEL_OH_BYTES;
        }
    }
    else if(block == ixf_eAPS)
    {
        return bb_INV_SEL_OH_BYTES;
    }
    else 
    {
        return bb_INV_CHIP_SEGMENT;
    }
}

/******************************************************************************
* Function : Ixf18100GetLineOhBytes
*------------------------------------------------------------------------------
* Description : Gets the line overhead bytes from the device.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          ptSegment: Pointer to chip segment structure.
*          flow: flow of the device
* Outputs : ptLineOh: Pointer to the Line OH structure
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100GetLineOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_RstMstBytes_t *ptLineOh)
{
    ushort uiTemp;

    if(block == ixf_eSONET)
    {   
        if (TX == flow)
        {
            uiTemp = HAL_IXF18100_READ(BaseAddress + ixf18100_TL_K1_K2_OFFSET);

            ptLineOh->K1 = (uchar)((uiTemp & ixf18100_mK1) >> ixf18100_sK1);

            ptLineOh->K2 = (uchar)((uiTemp & ixf18100_mK2) >> ixf18100_sK2);

            ptLineOh->S1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_S1_OFFSET) & ixf18100_mS1) >> ixf18100_sS1);


            uiTemp = HAL_IXF18100_READ(BaseAddress + ixf18100_TL_M0_M1_OFFSET);

            ptLineOh->MNU = (uchar)((uiTemp & ixf18100_mM0) >> ixf18100_sM0);

            ptLineOh->M1 = (uchar)((uiTemp & ixf18100_mM1) >> ixf18100_sM1);


            uiTemp = HAL_IXF18100_READ(BaseAddress + ixf18100_TL_NU1_2_0_OFFSET);

            ptLineOh->RNU1 = (uchar)((uiTemp & ixf18100_mNU1) >> ixf18100_sNU1);

            ptLineOh->RNU2 = (uchar)((uiTemp & ixf18100_mNU2) >> ixf18100_sNU2);

            ptLineOh->RNU9 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_TL_NU9_0_OFFSET) & ixf18100_mNU9) >> ixf18100_sNU9);
        }
        else
        {
            /* Read rx side line oh */
            uiTemp = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_K1_K2_OFFSET);

            ptLineOh->K1 = (uchar)((uiTemp & ixf18100_mK1) >> ixf18100_sK1);

            ptLineOh->K2 = (uchar)((uiTemp & ixf18100_mK2) >> ixf18100_sK2);

            ptLineOh->S1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_RL_S1_OFFSET) & ixf18100_mS1) >> ixf18100_sS1);

            /* No other bytes available on Receive side, so set to zero */
            ptLineOh->M1 = 0;
            ptLineOh->MNU = 0;
            ptLineOh->RNU1 = 0;
            ptLineOh->RNU2 = 0;
        }
    }
    else if(block == ixf_eAPS)
    { 
        if (TX == flow)
        {
            uiTemp = HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_TL_K1_K2_OFFSET);

            ptLineOh->K1 = (uchar)((uiTemp & ixf18100_mK1) >> ixf18100_sK1);

            ptLineOh->K2 = (uchar)((uiTemp & ixf18100_mK2) >> ixf18100_sK2);

            ptLineOh->S1   = 0;
            ptLineOh->MNU  = 0;
            ptLineOh->M1   = 0;
            ptLineOh->RNU1 = 0;
            ptLineOh->RNU2 = 0;
            ptLineOh->RNU9 = 0;
        }
        else
        {
            /* Read rx side line oh */
            uiTemp = HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_K1_K2_OFFSET);

            ptLineOh->K1 = (uchar)((uiTemp & ixf18100_mK1) >> ixf18100_sK1);

            ptLineOh->K2 = (uchar)((uiTemp & ixf18100_mK2) >> ixf18100_sK2);

            ptLineOh->S1 = (uchar)((HAL_IXF18100_READ(BaseAddress
                + ixf18100_APS_RL_S1_OFFSET) & ixf18100_mS1) >> ixf18100_sS1);

            /* No other bytes available on Receive side, so set to zero */
            ptLineOh->M1 = 0;
            ptLineOh->MNU = 0;
            ptLineOh->RNU1 = 0;
            ptLineOh->RNU2 = 0;
        }
    }
    else
    {
        return bb_INV_CHIP_SEGMENT;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SetLineOhBytes
*------------------------------------------------------------------------------
* Description : Sets the line overhead bytes in the device.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          ptSegment: Pointer to chip segment structure.
*          flow: flow of the device
*          ptLineOh: Pointer to the Line OH structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_SEGMENT, bb_INV_SEL_OH_BYTES
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SetLineOhBytes(bb_RegPointer_type BaseAddress, bb_eChipBlock_t block,
                       bb_ChipFlow_t flow, bb_RstMstBytes_t *ptLineOh)
{
    ushort uiTemp;

    if (block == ixf_eSONET)
    {
        if (TX == flow)
        {
            /* Structure members not masked because all are byte types,
            and fields are all one byte in length  */
            uiTemp = (ptLineOh->K1 << ixf18100_sK1)
                | (ptLineOh->K2 << ixf18100_sK2);

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_K1_K2_OFFSET, uiTemp);

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_S1_OFFSET,
                (ushort)(ptLineOh->S1 << ixf18100_sS1));

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_NU9_0_OFFSET,
                (ushort)(ptLineOh->RNU9 << ixf18100_sNU9));

            uiTemp = (ptLineOh->MNU << ixf18100_sM0) | (ptLineOh->M1
                << ixf18100_sM1);

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_M0_M1_OFFSET, uiTemp);

            uiTemp = (ptLineOh->RNU1 << ixf18100_sNU1) | (ptLineOh->RNU2
                << ixf18100_sNU2);

            HAL_IXF18100_WRITE(BaseAddress + ixf18100_TL_NU1_2_0_OFFSET, uiTemp);
        }
        else
        {
            /* No bytes to set on Rx side. */
            return bb_INV_SEL_OH_BYTES;
        }
    }
    else if(block == ixf_eAPS)
    {
    if (TX == flow)
    {
        /* Structure members not masked because all are byte types,
        and fields are all one byte in length  */
        uiTemp = (ptLineOh->K1 << ixf18100_sK1) | (ptLineOh->K2
            << ixf18100_sK2);

        HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_TL_K1_K2_OFFSET, uiTemp);
    }
    else
    {
        uiTemp = (ptLineOh->K1 << ixf18100_sK1) | (ptLineOh->K2
            << ixf18100_sK2);

        HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_RL_K1_K2_OFFSET, uiTemp);

        HAL_IXF18100_WRITE(BaseAddress + ixf18100_APS_RL_S1_OFFSET,
            (ushort)(ptLineOh->S1 << ixf18100_sS1));
        }               
    }
    else
    {
        return bb_INV_CHIP_SEGMENT;
    }

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetMiscStatus
*------------------------------------------------------------------------------
* Description : Reads the miscellaneous status of the SONET block.
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          SonetMiscStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetMiscStatus(bb_RegPointer_type BaseAddress,
                           ixf18100_SonetMiscStatus_t *SonetMiscStat)
{
    ushort  usValue =0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_PRBS_SYNC_STATE_OFFSET);

    SonetMiscStat->ucPRBSSyncState
        = (uchar)((usValue >> ixf18100_sPRBS_STATE) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxSOHStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the SONET Section OH
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxSOHStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxSOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxSectionOhStatus_t *RxSOHStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_SOH_S_OFFSET);

    RxSOHStat->ucOOFDetected = (uchar)((usValue >> ixf18100_sOOF_ST) & 0x01);

    RxSOHStat->ucLOFDetected = (uchar)((usValue >> ixf18100_sLOF_ST) & 0x01);

    RxSOHStat->ucExtLOSDetected
        = (uchar)((usValue >> ixf18100_sEXT_LOS_ST) & 0x01);

    RxSOHStat->ucIntLOSDetected
        = (uchar)((usValue >> ixf18100_sINT_LOS_ST) & 0x01);

    RxSOHStat->ucJ0CRC7ErrDetected
        = (uchar)((usValue >> ixf18100_sJ0_CRC7_ERR_ST) & 0x01);

    RxSOHStat->ucJ0Mismatch
        = (uchar)((usValue >> ixf18100_sJ0_MSMTCH_ST) & 0x01);

    RxSOHStat->ucJ0RxUnstableTrace
        = (uchar)((usValue >> ixf18100_sJ0_UNSTABLE_ST) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxAPSSOHStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the APS Section OH
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxAPSSOHStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxAPSSOHStatus(bb_RegPointer_type BaseAddress,
                ixf18100_SonetAPSRxSectionOhStatus_t *RxAPSSOHStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_SOH_S_OFFSET);

    RxAPSSOHStat->ucOOFDetected = (uchar)((usValue >> ixf18100_sOOF_ST) & 0x01);

    RxAPSSOHStat->ucLOFDetected = (uchar)((usValue >> ixf18100_sLOF_ST) & 0x01);

    RxAPSSOHStat->ucIntLOSDetected
        = (uchar)((usValue >> ixf18100_sINT_LOS_ST) & 0x01);

    RxAPSSOHStat->ucJ0CRC7ErrDetected
        = (uchar)((usValue >> ixf18100_sJ0_CRC7_ERR_ST) & 0x01);

    RxAPSSOHStat->ucJ0Mismatch
        = (uchar)((usValue >> ixf18100_sJ0_MSMTCH_ST) & 0x01);

    RxAPSSOHStat->ucJ0RxUnstableTrace
        = (uchar)((usValue >> ixf18100_sJ0_UNSTABLE_ST) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxLOHStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the SONET Line OH
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxLOHStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxLOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxLineOhStatus_t *RxLOHStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_LOH_S_OFFSET);

    RxLOHStat->ucAISLineDetected
        = (uchar)((usValue >> ixf18100_sAISL_ST) & 0x01);

    RxLOHStat->ucExcessiveBERDetected
        = (uchar)((usValue >> ixf18100_sEXC_B2_ERR_ST) & 0x01);

    RxLOHStat->ucRDILineDetected
        = (uchar)((usValue >> ixf18100_sRDIL_ST) & 0x01);

    RxLOHStat->ucSigFailDetected
        = (uchar)((usValue >> ixf18100_sSF_ST) & 0x01);

    RxLOHStat->ucDegSigDetected
        = (uchar)((usValue >> ixf18100_sDEG_B2_ERR_ST) & 0x01);

    RxLOHStat->ucK2UnstableDeteted
        = (uchar)((usValue >> ixf18100_sK2_UNSTABLE_ST) & 0x01);

    RxLOHStat->ucK1UnstableDeteted
        = (uchar)((usValue >> ixf18100_sK1_UNSTABLE_ST) & 0x01);

    RxLOHStat->ucS1UnstableDeteted
        = (uchar)((usValue >> ixf18100_sS1_UNSTABLE_ST) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxAPSLOHStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the APS Line OH
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxAPSLOHStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxAPSLOHStatus(bb_RegPointer_type BaseAddress,
                               ixf18100_SonetAPSRxLineOhStatus_t *RxAPSLOHStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_APS_RL_LOH_S_OFFSET);

    RxAPSLOHStat->ucAISLineDetected
        = (uchar)((usValue >> ixf18100_sAISL_ST) & 0x01);

    RxAPSLOHStat->ucExcessiveBERDetected
        = (uchar)((usValue >> ixf18100_sEXC_B2_ERR_ST) & 0x01);

    RxAPSLOHStat->ucRDILineDetected
        = (uchar)((usValue >> ixf18100_sRDIL_ST) & 0x01);

    RxAPSLOHStat->ucSigFailDetected
        = (uchar)((usValue >> ixf18100_sSF_ST) & 0x01);

    RxAPSLOHStat->ucDegSigDetected
        = (uchar)((usValue >> ixf18100_sDEG_B2_ERR_ST) & 0x01);

    RxAPSLOHStat->ucK2UnstableDeteted
        = (uchar)((usValue >> ixf18100_sK2_UNSTABLE_ST) & 0x01);

    RxAPSLOHStat->ucK1UnstableDeteted
        = (uchar)((usValue >> ixf18100_sK1_UNSTABLE_ST) & 0x01);

    RxAPSLOHStat->ucS1UnstableDeteted
        = (uchar)((usValue >> ixf18100_sS1_UNSTABLE_ST) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxPOHStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the SONET Path OH
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxPOHStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxPOHStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxPathOhStatus_t *RxPOHStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_POH_S_OFFSET);

    RxPOHStat->ucVCAISDetected
        = (uchar)((usValue >> ixf18100_sVC_AIS_ST) & 0x01);

    RxPOHStat->ucPldLabelMismatch
        = (uchar)((usValue >> ixf18100_sPLMP_ST) & 0x01);

    RxPOHStat->ucUnequipped
        = (uchar)((usValue >> ixf18100_sUN_EQP_ST) & 0x01);

    RxPOHStat->ucC2UnstableDetected
        = (uchar)((usValue >> ixf18100_sC2_UNSTABLE_ST) & 0x01);

    RxPOHStat->ucPDIPAlarmActive
        = (uchar)((usValue >> ixf18100_sPDIP_ST) & 0x01);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_J1_S_OFFSET);

    RxPOHStat->ucJ1CRC7ErrDetected
        = (uchar)((usValue >> ixf18100_sJ1_CRC7_ERR_ST) & 0x01);

    RxPOHStat->ucJ1Mismatch
        = (uchar)((usValue >> ixf18100_sJ1_MSMTCH_ST) & 0x01);

    RxPOHStat->ucJ1RxUnstableTrace
        = (uchar)((usValue >> ixf18100_sJ1_UNSTABLE_ST) & 0x01);

    RxPOHStat->ucJ1RxTraceAllZero
        = (uchar)((usValue >> ixf18100_sJ1_ALLZERO_ST) & 0x01);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100SonetGetRxPtrStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the SONET Rx Pointer Tracker
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          RxPtrStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetRxPtrStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetRxPtrTrackerStatus_t *RxPtrStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_RL_PT_S_OFFSET);

    RxPtrStat->ucNonConcatAISPDetected
        = (uchar)((usValue >> ixf18100_sAISP_ST) & 0x01);

    RxPtrStat->ucNonConcatLossOfPointer
        = (uchar)((usValue >> ixf18100_sLOP_ST) & 0x01);

    RxPtrStat->ucConcatAISPDetected
        = (uchar)((usValue >> ixf18100_sAISPC_ST) & 0x01);

    RxPtrStat->ucConcatLossOfPointer
        = (uchar)((usValue >> ixf18100_LOPC_ST) & 0x01);

    return bb_NO_ERROR;
}
/******************************************************************************
* Function : Ixf18100SonetGetTxPtrStatus
*------------------------------------------------------------------------------
* Description : Reads the status of the SONET Tx Pointer Tracker
*------------------------------------------------------------------------------
* Inputs : BaseAddress: BaseAddress of the device
*          TxPtrStat: Pointer to the status structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals :
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
static bb_Error_e
Ixf18100SonetGetTxPtrStatus(bb_RegPointer_type BaseAddress,
                            ixf18100_SonetTxPtrTrackerStatus_t *TxPtrStat)
{
    ushort  usValue = 0;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_TL_H1H2_STA_OFFSET);

    TxPtrStat->usOutgoingPtrValue = ((usValue & ixf18100_mPOINTER_VAL)
        >> ixf18100_sPOINTER_VAL);

    TxPtrStat->ucOutgoingSyncStatus
        = (uchar)((usValue & ixf18100_mSS_VAL) >> ixf18100_sSS_VAL);

    TxPtrStat->ucOutgoingNewDataFlag
        = (uchar)((usValue & ixf18100_mNDF_VAL) >> ixf18100_sNDF_VAL);

    return bb_NO_ERROR;
}
