/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_spi4.c $
 *   WORKFILE:       $Workfile: ixf18100_spi4.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains SPI4 block related routines
 *
 * Notes:
 *          
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_spi4.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 20    1/20/03 2:05p Pkjain2
 * beautified the code
 * 
 * 19    1/15/03 5:21p Jaalliso
 * Detabbing
 * 
 * 18    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 16    4/15/02 4:53p Pkjain2
 * Cleaned up some if 0 code.
 * 
 * 15    4/12/02 10:27a Pkjain2
 * 
 * 14    3/06/02 8:49a Pkjain2
 * Updated to EAS 5.0
 * 
 * 13    2/12/02 11:11a Jaalliso
 * Renaming SIF loopbacks to be more generic
 * 
 * 12    12/17/01 2:49p Pkjain2
 * Added trace macros
 * 
 * 3     9/05/01 5:14p Pkjain2
 * updated code to meet the spec received in last week of august
 * 
 * 2     8/15/01 8:35a Pkjain2
 * Added SPI 4 Code
 * 
 * 1     7/18/01 10:55a Rfranz
 * Initial check in - file headers only
 * 
 * --------------------------------------------------------------------------
 */
#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf18100_spi4_b.h"
#include "ixf_api_spi4_d.h"
#include "ixf18100_spi4_d.h"
#include "ixf18100_d.h"
#include "hal.h"


ulong
Ixf18100Spi4Read32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb);

/******************************************************************************
* Function : Ixf18100Spi4GlbSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 Global cfg struct and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg: pointer to the values that
*               need to be set in the device
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GlbSetCfg(bb_RegPointer_type BaseAddress,
                      ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg)
{
    ushort      usValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GlbSetCfg");

    usValue = (ixf18100_mSIF_MAC_THRESHOLD1
        & (ptSpi4GlbCfg->uiSIFLowerThresholdForMacPause
            << ixf18100_sSIF_MAC_THRESHOLD1));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_MAC_THRESHOLD1_OFFSET,
        usValue);


    usValue = (ixf18100_mSIF_MAC_THRESHOLD2
        & (ptSpi4GlbCfg->uiSIFUpperThresholdForMacPause
            << ixf18100_sSIF_MAC_THRESHOLD2));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_MAC_THRESHOLD2_OFFSET,
        usValue);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GlbSetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4RxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 RX config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4RxCfg_t *ptSpi4RxCfg: pointer to the values that
*               need to be set in the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4RxSetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4RxCfg_t *ptSpi4RxCfg)
{
    ushort      usValue;
    ushort      usTempValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4RxSetCfg");

    usValue = (ixf18100_mSIF_RX_MAXBURST1 & (ptSpi4RxCfg->uiSIFRxMaxBurst1
        << ixf18100_sSIF_RX_MAXBURST1));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_MAXBURST1_OFFSET,
        usValue);


    usValue = (ixf18100_mSIF_RX_MAXBURST2 & (ptSpi4RxCfg->uiSIFRxMaxBurst2
        << ixf18100_sSIF_RX_MAXBURST2));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_MAXBURST2_OFFSET,
        usValue);


    usTempValue = ((ushort)ptSpi4RxCfg->ulSIFRxMaxTrainInterval & 0x00FF);

    usValue = (ixf18100_mSIF_RX_MAXTLS & (usTempValue
        << ixf18100_sSIF_RX_MAXTLS));

    usValue |= (ixf18100_mSIF_RX_ALPHA
        & (ptSpi4RxCfg->ucSIFRxTrainPatternRepeats
            << ixf18100_sSIF_RX_ALPHA));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_CFG1_OFFSET, usValue);


    usTempValue = (ushort)((ptSpi4RxCfg->ulSIFRxMaxTrainInterval >> 0x08)
        & 0xFFFF);

    usValue = (ixf18100_mSIF_RX_MAXTMS & (usTempValue
        << ixf18100_sSIF_RX_MAXTMS));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_CFG2_OFFSET, usValue);


    usValue = (ixf18100_mSIF_RX_SPI4_ADDR & (ptSpi4RxCfg->ucSIFRxSpi4Address
        << ixf18100_sSIF_RX_SPI4_ADDR));

    usValue |= ((ptSpi4RxCfg->ucSIFRxReverseBit & 0x01)
        << ixf18100_sSIF_RX_BIT_REV);

    usValue |= ((ptSpi4RxCfg->ucSIFRxErasePktOnParallelBusErr & 0x01)
        << ixf18100_sSIF_RX_ERROR_ERASE);

    usValue |= ((ptSpi4RxCfg->ucSIFRxPreventSpi4Err & 0x01)
        << ixf18100_sSIF_RX_ERROR_MASK);

    usValue |= ((ptSpi4RxCfg->ucSIFIgnoreParityOnDip2Input & 0x01)
        << ixf18100_sSIF_RX_DIP2_MASK);

    usValue |= ((ptSpi4RxCfg->ucSIFRxNumTrainingPatternRepeats & 0x01)
        << ixf18100_sSIF_RX_FSTAT_REPEATS);

    usValue |= ((ptSpi4RxCfg->ucSIFRxFifoInvClkEn & 0x01)
        << ixf18100_sSIF_RX_FSTAT_CLK_INV);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_INTFCFG_OFFSET, usValue);


    usValue = (ixf18100_mSIF_RX_BURSTLEN & (ptSpi4RxCfg->ucSIFRxBurstLen
        << ixf18100_sSIF_RX_BURSTLEN));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_BURSTLEN_OFFSET, usValue);


    usValue = (ixf18100_mSIF_RX_POLARITY & (ptSpi4RxCfg->uiSIFRxBitWisePolarity
        << ixf18100_sSIF_RX_POLARITY));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_RX_BIT_POLARITY_OFFSET,
        usValue);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4RxSetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4TxSetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 TX config structure
* and masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxCfg_t *ptSpi4TxCfg: pointer to the values that
*               need to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4TxSetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4TxCfg_t *ptSpi4TxCfg)
{
    ushort      usValue;
    ushort      usTxSyncVal;
    ushort      usTxNoSyncVal;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4TxSetCfg");

    usValue = (ixf18100_mSIF_TX_MAXBURST1 & (ptSpi4TxCfg->uiSIFTxMaxBurst1
        << ixf18100_sSIF_TX_MAXBURST1));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_MAXBURST1_OFFSET,
        usValue);


    usValue = (ixf18100_mSIF_TX_MAXBURST2 & (ptSpi4TxCfg->uiSIFTxMaxBurst2
        << ixf18100_sSIF_TX_MAXBURST2));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_MAXBURST2_OFFSET,
        usValue);


    if (ptSpi4TxCfg->ucSIFTxReqDip4PassToGainLock == 16)
    {
        usTxSyncVal = 0;
    }
    else
    {
        usTxSyncVal = ptSpi4TxCfg->ucSIFTxReqDip4PassToGainLock;
    }

    if (ptSpi4TxCfg->ucSIFTxReqDip4FailToLoseLock == 16)
    {
        usTxNoSyncVal = 0;
    }
    else
    {
        usTxNoSyncVal = ptSpi4TxCfg->ucSIFTxReqDip4FailToLoseLock;
    }

    usValue = (ixf18100_mSIF_TX_SYNC & (usTxSyncVal << ixf18100_sSIF_TX_SYNC));

    usValue |= (ixf18100_mSIF_TX_NOSYNC & (usTxNoSyncVal
        << ixf18100_sSIF_TX_NOSYNC));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_SYNC_OFFSET, usValue);


    usValue = ((ptSpi4TxCfg->ucSIFDip4ModeEn & 0x01)
        << ixf18100_sSIF_TX_DIP4_MODE);

    usValue |= ((ptSpi4TxCfg->ucSIFTxTestFifoStatus & 0x01)
        << ixf18100_sSIF_TX_FSTAT_TEST);

    usValue |= ((~(ptSpi4TxCfg->ucSIFTxFifoStatusChnEn) & 0x01)
        << ixf18100_sSIF_TX_FSTAT_DIS);

    usValue |= ((ptSpi4TxCfg->ucSIFTxErasePktOnParityErr & 0x01)
        << ixf18100_sSIF_TX_PARITY_ERASE);

    usValue |= ((ptSpi4TxCfg->ucSIFTxErasePktOnSpi4Err & 0x01)
        << ixf18100_sSIF_TX_ERROR_ERASE);

    usValue |= ((ptSpi4TxCfg->ucSIFTxPreventParallelBusErr & 0x01)
        << ixf18100_sSIF_TX_ERROR_MASK);

    usValue |= ((ptSpi4TxCfg->ucSIFTxAddHoldTimeModeEn & 0x01)
        << ixf18100_sSIF_TX_LVC_STAT_MODE);

    usValue |= ((ptSpi4TxCfg->ucSIFTxNumTrainingPatternRepeats & 0x01)
        << ixf18100_sSIF_TX_FSTAT_REPEATS);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_INTFCFG_OFFSET, usValue);


    usValue = ((ptSpi4TxCfg->ucSIFTxDeskewEnable & 0x01)
        << ixf18100_sSIF_TX_DSK_ENABLE);

    usValue |= ((ptSpi4TxCfg->ucSIFTxBitReverseEn & 0x01)
        << ixf18100_sSIF_TX_BIT_REVERSE);

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_DSK_DISABLE_OFFSET,
        usValue);


    usValue = (ixf18100_mSIF_TX_POLARITY
        & (ptSpi4TxCfg->uiSIFTxBitWisePolarity << ixf18100_sSIF_TX_POLARITY));

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_BIT_POLARITY_OFFSET,
        usValue);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4TxSetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4GlbGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GlbGetCfg(bb_RegPointer_type BaseAddress,
                      ixf18100_Spi4GlbCfg_t *ptSpi4GlbCfg)
{
    ushort      usValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GlbGetCfg");

    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_MAC_THRESHOLD1_OFFSET);

    ptSpi4GlbCfg->uiSIFLowerThresholdForMacPause
        = (ushort)((usValue & ixf18100_mSIF_MAC_THRESHOLD1)
            >> ixf18100_sSIF_MAC_THRESHOLD1);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_MAC_THRESHOLD2_OFFSET);

    ptSpi4GlbCfg->uiSIFUpperThresholdForMacPause
        = (ushort)((usValue & ixf18100_mSIF_MAC_THRESHOLD2)
            >> ixf18100_sSIF_MAC_THRESHOLD2);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GlbGetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4RxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4RxCfg_t *ptSpi4RxCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4RxGetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4RxCfg_t *ptSpi4RxCfg)
{
    ushort      usValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4RxGetCfg");

    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_RX_MAXBURST1_OFFSET);

    ptSpi4RxCfg->uiSIFRxMaxBurst1
        = (ushort)((usValue & ixf18100_mSIF_RX_MAXBURST1)
            >> ixf18100_sSIF_RX_MAXBURST1);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_RX_MAXBURST2_OFFSET);

    ptSpi4RxCfg->uiSIFRxMaxBurst2
        = (ushort)(( usValue & ixf18100_mSIF_RX_MAXBURST2)
            >> ixf18100_sSIF_RX_MAXBURST2);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_RX_CFG1_OFFSET);

    ptSpi4RxCfg->ulSIFRxMaxTrainInterval
        = (ulong)((usValue & ixf18100_mSIF_RX_MAXTLS)
            >> ixf18100_sSIF_RX_MAXTLS);

    ptSpi4RxCfg->ucSIFRxTrainPatternRepeats
        = (uchar)((usValue & ixf18100_mSIF_RX_ALPHA)
            >> ixf18100_sSIF_RX_ALPHA);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_RX_CFG2_OFFSET);

    ptSpi4RxCfg->ulSIFRxMaxTrainInterval
        |= (((usValue & ixf18100_mSIF_RX_MAXTMS) >> ixf18100_sSIF_RX_MAXTMS)
            << 0x08);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_RX_INTFCFG_OFFSET);

    ptSpi4RxCfg->ucSIFRxSpi4Address
        = (uchar)((usValue & ixf18100_mSIF_RX_SPI4_ADDR)
            >> ixf18100_sSIF_RX_SPI4_ADDR);

    ptSpi4RxCfg->ucSIFRxReverseBit
        = ((usValue >>  ixf18100_sSIF_RX_BIT_REV) & 0x01);

    ptSpi4RxCfg->ucSIFRxErasePktOnParallelBusErr
        = ((usValue >> ixf18100_sSIF_RX_ERROR_ERASE) & 0x01);

    ptSpi4RxCfg->ucSIFRxPreventSpi4Err
        = ((usValue >> ixf18100_sSIF_RX_ERROR_MASK) & 0x01);

    ptSpi4RxCfg->ucSIFIgnoreParityOnDip2Input
        = ((usValue >> ixf18100_sSIF_RX_DIP2_MASK) & 0x01);

    ptSpi4RxCfg->ucSIFRxNumTrainingPatternRepeats
        = ((usValue >> ixf18100_sSIF_RX_FSTAT_REPEATS) & 0x01);

    ptSpi4RxCfg->ucSIFRxFifoInvClkEn
        = (uchar)((usValue >> ixf18100_sSIF_RX_FSTAT_CLK_INV) & 0x01);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_RX_BURSTLEN_OFFSET);

    ptSpi4RxCfg->ucSIFRxBurstLen
        = (uchar)((usValue & ixf18100_mSIF_RX_BURSTLEN)
            >> ixf18100_sSIF_RX_BURSTLEN);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_RX_BIT_POLARITY_OFFSET);

    ptSpi4RxCfg->uiSIFRxBitWisePolarity
        = (ushort)((usValue & ixf18100_mSIF_RX_POLARITY)
            >> ixf18100_sSIF_RX_POLARITY);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4RxGetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4TxGetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxCfg_t *ptSpi4TxCfg: pointer to the values that
*               need to be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4TxGetCfg(bb_RegPointer_type BaseAddress,
                     ixf18100_Spi4TxCfg_t *ptSpi4TxCfg)
{
    ushort      usValue;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4TxGetCfg");

    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_TX_MAXBURST1_OFFSET);

    ptSpi4TxCfg->uiSIFTxMaxBurst1
        = (ushort)((usValue & ixf18100_mSIF_TX_MAXBURST1)
            >> ixf18100_sSIF_TX_MAXBURST1);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_TX_MAXBURST2_OFFSET);

    ptSpi4TxCfg->uiSIFTxMaxBurst2
        = (ushort)((usValue & ixf18100_mSIF_TX_MAXBURST2)
            >> ixf18100_sSIF_TX_MAXBURST2);


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TX_SYNC_OFFSET);

    ptSpi4TxCfg->ucSIFTxReqDip4PassToGainLock
        = (uchar)((usValue & ixf18100_mSIF_TX_SYNC) >> ixf18100_sSIF_TX_SYNC);

    ptSpi4TxCfg->ucSIFTxReqDip4FailToLoseLock
        = (uchar)((usValue & ixf18100_mSIF_TX_NOSYNC)
            >> ixf18100_sSIF_TX_NOSYNC);

    if (ptSpi4TxCfg->ucSIFTxReqDip4PassToGainLock == 0)
    {
        ptSpi4TxCfg->ucSIFTxReqDip4PassToGainLock = 16;
    }

    if (ptSpi4TxCfg->ucSIFTxReqDip4FailToLoseLock == 0)
    {
        ptSpi4TxCfg->ucSIFTxReqDip4FailToLoseLock = 16;
    }


    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TX_INTFCFG_OFFSET);

    ptSpi4TxCfg->ucSIFDip4ModeEn
        = ((usValue >> ixf18100_sSIF_TX_DIP4_MODE) & 0x01);

    ptSpi4TxCfg->ucSIFTxTestFifoStatus
        = ((usValue >> ixf18100_sSIF_TX_FSTAT_TEST) & 0x01);

    ptSpi4TxCfg->ucSIFTxFifoStatusChnEn
        = (~(usValue >> ixf18100_sSIF_TX_FSTAT_DIS) & 0x01);

    ptSpi4TxCfg->ucSIFTxErasePktOnParityErr
        = ((usValue >> ixf18100_sSIF_TX_PARITY_ERASE) & 0x01);

    ptSpi4TxCfg->ucSIFTxErasePktOnSpi4Err
        = ((usValue >> ixf18100_sSIF_TX_ERROR_ERASE) & 0x01);

    ptSpi4TxCfg->ucSIFTxPreventParallelBusErr
        = ((usValue >> ixf18100_sSIF_TX_ERROR_MASK) & 0x01);

    ptSpi4TxCfg->ucSIFTxAddHoldTimeModeEn
        = ((usValue >> ixf18100_sSIF_TX_LVC_STAT_MODE) & 0x01);

    ptSpi4TxCfg->ucSIFTxNumTrainingPatternRepeats
        = ((usValue >> ixf18100_sSIF_TX_FSTAT_REPEATS) & 0x01);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_TX_DSK_DISABLE_OFFSET);

    ptSpi4TxCfg->ucSIFTxDeskewEnable
        = ((usValue >> ixf18100_sSIF_TX_DSK_ENABLE) & 0x01);

    ptSpi4TxCfg->ucSIFTxBitReverseEn
        = ((usValue >> ixf18100_sSIF_TX_BIT_REVERSE) & 0x01);


    usValue = HAL_IXF18100_READ(BaseAddress
        + ixf18100_SIF_TX_BIT_POLARITY_OFFSET);

    ptSpi4TxCfg->uiSIFTxBitWisePolarity
        = (ushort)((usValue & ixf18100_mSIF_TX_POLARITY)
            >> ixf18100_sSIF_TX_POLARITY);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4TxGetCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4RxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the SPI4 RX counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4RxGetCounters(bb_RegPointer_type BaseAddress,
                          bb_SelCounters_e SelCntr, void *ptSpi4Counters)
{
    bb_Error_e          rc = bb_NO_ERROR;
    ulong               *ulCounter = (ulong *)ptSpi4Counters;
    ixfApi_Spi4RxCntr_t *ptSpi4RxCounters
        = (ixfApi_Spi4RxCntr_t *)ptSpi4Counters;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4RxGetCounters");

    switch (SelCntr)
    {
    case bb_SPI4_RX_BUS_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_BUSERR_W0_OFFSET, ixf18100_mSIF_RX_BUSERR_W1,
            ixf18100_mSIF_RX_BUSERR_W0);
        break;

    case bb_SPI4_RX_FULL_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_FULLERR_W0_OFFSET, ixf18100_mSIF_RX_FULLERR_W1,
            ixf18100_mSIF_RX_FULLERR_W0);
        break;

    case bb_SPI4_RX_DIP2_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_DIP2ERR_W0_OFFSET, ixf18100_mSIF_RX_DIP2ERR_W1,
            ixf18100_mSIF_RX_DIP2ERR_W0);
        break;

    case bb_SPI4_RX_NO_EOP_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_NOEOPERR_W0_OFFSET, ixf18100_mSIF_RX_NOEOPERR_W1,
            ixf18100_mSIF_RX_NOEOPERR_W0);
        break;

    case bb_SPI4_RX_COUNTERS:
        ptSpi4RxCounters->ulSifRxBusErrCnt = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_BUSERR_W0_OFFSET, ixf18100_mSIF_RX_BUSERR_W1,
            ixf18100_mSIF_RX_BUSERR_W0);

        ptSpi4RxCounters->ulSifRxFifoFullErrCnt
            = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_FULLERR_W0_OFFSET, ixf18100_mSIF_RX_FULLERR_W1,
            ixf18100_mSIF_RX_FULLERR_W0);

        ptSpi4RxCounters->ulSifRxDip2ErrCnt = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_DIP2ERR_W0_OFFSET, ixf18100_mSIF_RX_DIP2ERR_W1,
            ixf18100_mSIF_RX_DIP2ERR_W0);

        ptSpi4RxCounters->ulSifRxNoEndOfPktErrCnt
            = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_RX_NOEOPERR_W0_OFFSET, ixf18100_mSIF_RX_NOEOPERR_W1,
            ixf18100_mSIF_RX_NOEOPERR_W0);
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4RxGetCounters", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4TxGetCounters
*------------------------------------------------------------------------------
* Description : The function reads one/all the SPI4 Tx counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4TxGetCounters(bb_RegPointer_type BaseAddress,
                          bb_SelCounters_e SelCntr, void *ptSpi4Counters)
{
    bb_Error_e          rc = bb_NO_ERROR;
    ulong               *ulCounter = (ulong *)ptSpi4Counters;
    ixfApi_Spi4TxCntr_t *ptSpi4TxCounters
        = (ixfApi_Spi4TxCntr_t *)ptSpi4Counters;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4TxGetCounters");

    switch (SelCntr)
    {
    case bb_SPI4_TX_BUS_ERR_CNT:
        *ulCounter  = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_BUSERR_W0_OFFSET, ixf18100_mSIF_TX_BUSERR_W1,
            ixf18100_mSIF_TX_BUSERR_W0);
        break;

    case bb_SPI4_TX_PAR_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_PARERR_W0_OFFSET, ixf18100_mSIF_TX_PARERR_W1,
            ixf18100_mSIF_TX_PARERR_W0);
        break;

    case bb_SPI4_TX_FULL_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_FULLERR_W0_OFFSET, ixf18100_mSIF_TX_FULLERR_W1,
            ixf18100_mSIF_TX_FULLERR_W0);
        break;

    case bb_SPI4_TX_NO_EOP_ERR_CNT:
        *ulCounter = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_NOEOPERR_W0_OFFSET, ixf18100_mSIF_TX_NOEOPERR_W1,
            ixf18100_mSIF_TX_NOEOPERR_W0);
        break;

    case bb_SPI4_TX_COUNTERS:
        ptSpi4TxCounters->ulSifTxBusErrCnt = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_BUSERR_W0_OFFSET, ixf18100_mSIF_TX_BUSERR_W1,
            ixf18100_mSIF_TX_BUSERR_W0);

        ptSpi4TxCounters->ulSifTxParityErrCnt = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_PARERR_W0_OFFSET, ixf18100_mSIF_TX_PARERR_W1,
            ixf18100_mSIF_TX_PARERR_W0);

        ptSpi4TxCounters->ulSifTxFifoFullErrCnt
            = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_FULLERR_W0_OFFSET, ixf18100_mSIF_TX_FULLERR_W1,
            ixf18100_mSIF_TX_FULLERR_W0);

        ptSpi4TxCounters->ulSifTxNoEndOfPktErrCnt
            = Ixf18100Spi4Read32(BaseAddress
            + ixf18100_SIF_TX_NOEOPERR_W0_OFFSET, ixf18100_mSIF_TX_NOEOPERR_W1,
            ixf18100_mSIF_TX_NOEOPERR_W0);
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4TxGetCounters", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4SetTxDeskewCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg: pointer to the values 
*           that need to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4SetTxDeskewCfg(bb_RegPointer_type BaseAddress,
                           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg)
{
    ushort      usValue;
    uchar           Index;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4SetTxDeskewCfg");

    for (Index = 0; Index < 17; Index++)
    {
        usValue = (ixf18100_mSIF_TX_DSK_WRITE_PHASE_D0
            & (ptSpi4TxDskCfg->ucSIFTxDeskewWritePhase[Index]
                << ixf18100_sSIF_TX_DSK_WRITE_PHASE_D0));

        usValue |= (ixf18100_mSIF_TX_DSK_OFFSET_D0 
            & (ptSpi4TxDskCfg->ucSIFTxDeskewOffset[Index] 
                << ixf18100_sSIF_TX_DSK_OFFSET_D0));

        HAL_IXF18100_WRITE(BaseAddress + 
            ixf18100_SIF_TX_DSK_DAT_0_OFFSET + Index, usValue);
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4SetTxDeskewCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4GetTxDeskewCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GetTxDeskewCfg(bb_RegPointer_type BaseAddress,
                           ixf18100_Spi4TxDskCfg_t *ptSpi4TxDskCfg)
{
    ushort      usValue;
    uchar           Index;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GetTxDeskewCfg");

    for (Index = 0; Index < 17; Index++)
    {
        usValue = HAL_IXF18100_READ(BaseAddress
            + ixf18100_SIF_TX_DSK_DAT_0_OFFSET
            + (Index << ixf18100_REG_SHIFT));

        ptSpi4TxDskCfg->ucSIFTxDeskewWritePhase[Index] 
            = (uchar)((usValue & ixf18100_mSIF_TX_DSK_WRITE_PHASE_D0) 
                >> ixf18100_sSIF_TX_DSK_WRITE_PHASE_D0);

        ptSpi4TxDskCfg->ucSIFTxDeskewOffset[Index] 
            = (uchar)((usValue & ixf18100_mSIF_TX_DSK_OFFSET_D0) 
                >> ixf18100_sSIF_TX_DSK_OFFSET_D0);
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GetTxDeskewCfg", bb_NO_ERROR);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100Spi4SetCfg
*------------------------------------------------------------------------------
* Description : The function takes the values from SPI4 config structure and 
* masks it appropriately and writes the 16 bit value into the register.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be set
*           ixf18100_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set in the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4SetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                   ixf18100_Spi4Cfg_t *ptSpi4Cfg)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4SetCfg");

    switch (SelCfg)
    {
    case e_GLOBAL_CONFIG:
        rc = Ixf18100Spi4GlbSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4GlbCfg));
        break;

    case e_RX_CONFIG:
        rc = Ixf18100Spi4RxSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4RxCfg));
        break;

    case e_TX_CONFIG:
        rc = Ixf18100Spi4TxSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4TxCfg));
        break;

    case e_TX_DSK_CONFIG:
        rc = Ixf18100Spi4SetTxDeskewCfg(BaseAddress,
            &(ptSpi4Cfg->Spi4TxDskCfg));
        break;

    case e_ALL_CONFIG:
        rc = Ixf18100Spi4GlbSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4GlbCfg));
        rc = Ixf18100Spi4RxSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4RxCfg));
        rc = Ixf18100Spi4TxSetCfg(BaseAddress, &(ptSpi4Cfg->Spi4TxCfg));
        rc = Ixf18100Spi4SetTxDeskewCfg(BaseAddress,
            &(ptSpi4Cfg->Spi4TxDskCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4SetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4GetCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelConfig_e  SelCfg: enum value defining what configuration
*               needs to be fetched
*           ixf18100_Spi4Cfg_t *ptSpi4Cfg: pointer to the values that need to 
*               be set from the current chip configuration
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                   ixf18100_Spi4Cfg_t *ptSpi4Cfg)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GetCfg");

    switch (SelCfg)
    {
    case e_GLOBAL_CONFIG:
        rc = Ixf18100Spi4GlbGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4GlbCfg));
        break;

    case e_RX_CONFIG:
        rc = Ixf18100Spi4RxGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4RxCfg));
        break;

    case e_TX_CONFIG:
        rc = Ixf18100Spi4TxGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4TxCfg));
        break;

    case e_TX_DSK_CONFIG:
        rc = Ixf18100Spi4GetTxDeskewCfg(BaseAddress,
            &(ptSpi4Cfg->Spi4TxDskCfg));
        break;

    case e_ALL_CONFIG:
        rc = Ixf18100Spi4GlbGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4GlbCfg));
        rc = Ixf18100Spi4RxGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4RxCfg));
        rc = Ixf18100Spi4TxGetCfg(BaseAddress, &(ptSpi4Cfg->Spi4TxCfg));
        rc = Ixf18100Spi4GetTxDeskewCfg(BaseAddress,
            &(ptSpi4Cfg->Spi4TxDskCfg));
        break;

    default:
        rc = bb_INV_PARAMETER;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GetCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4SetTestCfg
*------------------------------------------------------------------------------
* Description : The function sets the test configuration to the chip..
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_TestType_e testType: selects the test type
*           void *ptSpi4TestCfg: pointer to the test 
*           configuration that needs to be set in the chip
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_INV_CHIP_TYPE, bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4SetTestCfg(bb_RegPointer_type BaseAddress, bb_TestType_e testType,
                       void *ptTestCfg)
{
    ushort              usValue;
    ulong               ulTempVal = 0;
    bb_Error_e          rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4SetTestCfg");

    ulTempVal = (ulong)ptTestCfg;

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TEST_OFFSET);

    switch (testType)
    {
    case bb_SYSTEM_LOCAL_LOOPBACK:
        usValue &= ~(IXF_BIT(ixf18100_sSIF_RX_PI_LOOPBACK));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sSIF_RX_PI_LOOPBACK);
        break;

    case bb_SYSTEM_REMOTE_LOOPBACK:
        usValue &= ~(IXF_BIT(ixf18100_sSIF_RX_LOOPBACK));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sSIF_RX_LOOPBACK);
        break;

    case bb_SPI4_TX_LOCK_DIS:
        usValue &= ~(IXF_BIT(ixf18100_sSIF_TX_LOCK_DISABLE));
        usValue |= ((ulTempVal & 0x01) << ixf18100_sSIF_TX_LOCK_DISABLE);
        break;

    default:
        break;
    }

    HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TEST_OFFSET, usValue);

    switch (testType)
    {
    case bb_SPI4_TX_FIFO_STATUS:
        usValue = HAL_IXF18100_READ(BaseAddress
            + ixf18100_SIF_TX_INTFCFG_OFFSET);

        usValue &= ~(IXF_BIT(ixf18100_sSIF_TX_FSTAT_TEST));

        usValue |= ((ulTempVal & 0x01) << ixf18100_sSIF_TX_FSTAT_TEST);

        HAL_IXF18100_WRITE(BaseAddress + ixf18100_SIF_TX_INTFCFG_OFFSET,
            usValue);
        break;

    default:
        rc = bb_INV_TEST_CNFG;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4SetTestCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4GetTestCfg
*------------------------------------------------------------------------------
* Description : The function reads the 16 bit register values and sets the
* value in the structure members appropriately.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           testType : The type of test to run.
*           void *pTestCfg: pointer to the test 
*               configuration that needs to be read from the chip
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_TEST_CFG
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GetTestCfg(bb_RegPointer_type BaseAddress, bb_TestType_e testType,
                       void *pTestCfg)
{
    ushort              usValue;
    ulong               ulRetVal = 0;
    bb_Error_e          rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GetTestCfg");

    switch (testType)
    {
    case bb_SPI4_TX_FIFO_STATUS:
        usValue = HAL_IXF18100_READ(BaseAddress
            + ixf18100_SIF_TX_INTFCFG_OFFSET);

        ulRetVal = ((usValue >> ixf18100_sSIF_TX_FSTAT_TEST) & 0x01);
        break;

    case bb_SYSTEM_LOCAL_LOOPBACK:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TEST_OFFSET);
        ulRetVal = (ulong)((usValue >> ixf18100_sSIF_RX_PI_LOOPBACK) & 0x01);
        break;

    case bb_SYSTEM_REMOTE_LOOPBACK:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TEST_OFFSET);
        ulRetVal = (ulong)((usValue >> ixf18100_sSIF_RX_LOOPBACK) & 0x01);
        break;

    case bb_SPI4_TX_LOCK_DIS:
        usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_TEST_OFFSET);
        ulRetVal = (ulong)((usValue >> ixf18100_sSIF_TX_LOCK_DISABLE) & 0x01);
        break;

    default:
        rc = bb_INV_TEST_CNFG;
        break;
    }

    *((ulong *)pTestCfg) = ulRetVal;

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GetTestCfg", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4GetCounters
*------------------------------------------------------------------------------
* Description : The function reads all the SPI4 counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           bb_SelCounters_e SelCntr: selects coutner to be returned
*           void *ptSpi4Counters: pointer to the counter 
*               structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GetCounters(bb_RegPointer_type BaseAddress,
                        bb_SelCounters_e SelCntr, void *ptSpi4Counters)
{
    bb_Error_e      rc = bb_NO_ERROR;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GetCounters");

    switch (SelCntr)
    {
    case bb_SPI4_RX_BUS_ERR_CNT:
    case bb_SPI4_RX_FULL_ERR_CNT:
    case bb_SPI4_RX_DIP2_ERR_CNT:
    case bb_SPI4_RX_NO_EOP_ERR_CNT:
    case bb_SPI4_RX_COUNTERS:
        rc = Ixf18100Spi4RxGetCounters(BaseAddress, SelCntr, ptSpi4Counters);
        break;

    case bb_SPI4_TX_BUS_ERR_CNT:
    case bb_SPI4_TX_PAR_ERR_CNT:
    case bb_SPI4_TX_FULL_ERR_CNT:
    case bb_SPI4_TX_NO_EOP_ERR_CNT:
    case bb_SPI4_TX_COUNTERS:
        rc = Ixf18100Spi4TxGetCounters(BaseAddress, SelCntr, ptSpi4Counters);
        break;

    case bb_SPI4_COUNTERS:
        rc = Ixf18100Spi4RxGetCounters(BaseAddress, bb_SPI4_RX_COUNTERS,
            (void *)&(((ixfApi_Spi4Cntr_t *)ptSpi4Counters)->Spi4RxCntr));
        rc = Ixf18100Spi4TxGetCounters(BaseAddress, bb_SPI4_TX_COUNTERS,
            (void *)&(((ixfApi_Spi4Cntr_t *)ptSpi4Counters)->Spi4TxCntr));
        break;

    default:
        rc = bb_INV_SEL_COUNTERS;
        break;
    }

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GetCounters", rc);

    return rc;
}

/******************************************************************************
* Function : Ixf18100Spi4GetStatus
*------------------------------------------------------------------------------
* Description : The function reads all the SPI4 counters
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: Base address of the chip.
*           ixf18100_Spi4Status_t *ptSpi4Status: pointer to the status 
*               structure
* Outputs : none
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100Spi4GetStatus(bb_RegPointer_type BaseAddress,
                      bb_SelStatus_e selStatus, void* pStatus)
{
    ushort          usValue;
    ixf18100_Spi4Status_t   *ptSpi4Status = (ixf18100_Spi4Status_t *)pStatus;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4GetStatus");

    usValue = HAL_IXF18100_READ(BaseAddress + ixf18100_SIF_STATUS_OFFSET);

    ptSpi4Status->ucFifoStatChnRcvdMsg
        = ((usValue >> ixf18100_sSIF_RX_FSTAT_ON) & 0x01);

    ptSpi4Status->ucTxTrainDetected
        = ((usValue >> ixf18100_sSIF_TX_TRAIN_DETECTED) & 0x01);

    ptSpi4Status->ucTxDeskewLockEn
        = ((usValue >> ixf18100_sSIF_TX_DESKEW_LOCKED) & 0x01);

    ptSpi4Status->uiTxFifoStatus
        = (ushort)((usValue & ixf18100_mSIF_TX_FIFO_STATUS)
            >> ixf18100_sSIF_TX_FIFO_STATUS);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4GetStatus", bb_NO_ERROR);

    return bb_NO_ERROR;
}



/*****************************INTERNAL ROUTINES*****************************/

/******************************************************************************
* Function : Ixf18100Spi4Read32
*------------------------------------------------------------------------------
* Description : Reads 32 bit values (counters) from the SPI4 block
*------------------------------------------------------------------------------
* Inputs  : ushort *puiLsb: pointer to the least significant bits of the
*                   32 bit value to be read.
*           ushort uiMaskMsb: Mask for the upper 16 bits
*           ushort uiMaskLsb: Mask for the lower 16 bits
* Outputs : none
*------------------------------------------------------------------------------
* Returns : 32 bit value containing properly masked value.
*------------------------------------------------------------------------------
* Access Globals : none
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
ulong
Ixf18100Spi4Read32(ushort *puiLsb, ushort uiMaskMsb, ushort uiMaskLsb)
{

    ushort uiBitsHigh, uiBitsLow;

    IXF_FUNC_ENTRY_TRC("Ixf18100Spi4Read32");

    /* The high counter bits are stored at the higher address, and must
    ** be read second.  The function takes the address of the word
    ** containing the least significant bits of the counter, so we 
    ** add 1 to get the address of the word containing the most
    ** significant bits. */
    uiBitsLow = HAL_IXF18100_READ(puiLsb);
    uiBitsHigh = HAL_IXF18100_READ(puiLsb + 1);

    IXF_FUNC_EXIT_TRC("Ixf18100Spi4Read32", 0);

    return (((uiBitsHigh & uiMaskMsb) << 16) | (uiBitsLow & uiMaskLsb));
}



