/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y                   
 *
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *
 *                        INTEL  CORPORATION                            
 *
 *                     2200 MISSION COLLEGE BLVD                        
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:57p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_xgmac.c $
 *   WORKFILE:       $Workfile: ixf18100_xgmac.c $
 *
 * --------------------------------------------------------------------------
 * Project: ixf18100 family device API
 *
 * Purpose: This file contains XGMAC block related routines
 *
 * Notes:
 *
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf1810x/ixf18100_xgmac.c $
 * 
 * 1     3/08/04 1:57p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 20    1/30/03 4:40p Pkjain2
 * Made changes for PTQ release. Added Getstatus calls for SONET and
 * GLOBAL block.
 * 
 * 19    1/20/03 2:06p Pkjain2
 * beautified the code
 * 
 * 18    12/06/02 4:25p Jaalliso
 * Fixing incorrect comment re: xgmacGet/SetAddress
 * 
 * 17    11/20/02 11:41a Jaalliso
 * Fixing problem with Ixf18100XgmacSetCfg() and XGMAC_CNTL2
 * 
 * 16    11/12/02 2:26p Jaalliso
 * XGMAC_CNTL/XGMAC_CNTL2 B0 silicon changes
 * 
 * 15    7/08/02 1:16p Pkjain2
 * Moved Calypso specific data stuctures into Calypso block specific data
 * structure file.
 * 
 * 14    5/29/02 9:42a Jaalliso
 * Changing bb_18100_CHIP to bb_18101_CHIP, adding 18102/3/4 chips
 * 
 * 13    5/17/02 9:09a Jaalliso
 * Removing PrependValue from Get/SetCfg()
 * 
 * 12    4/16/02 2:45p Jaalliso
 * Renaming  ixf_api_xgmac
 * 
 * 10    1/17/02 12:20p Srodrig1
 * Added comments to functions Get and Set mac address.
 * 
 * 4     8/24/01 3:17p Wrbrackn
 * Removed case structure in XGMAC per Mark's request.
 * 
 * 3     8/23/01 4:12p Wrbrackn
 * Made changes suggested after review of the code with Mark.
 * 
 *
 * 
 * --------------------------------------------------------------------------
 */

#ifdef VXWORKS
#include "vxWorks.h"
#endif

#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf18100_xgmac_b.h"
#include "ixf_api_ether_d.h"
#include "ixf18100_xgmac_d.h"
#include "ixf18100_xgmac.h"
#include "ixf18100_d.h"
#include "hal.h"


static
ulong ixf18100XgmacRead32(ushort *Address);

static void
ixf18100XgmacWrite32(ushort *Address, ulong Value);

static void
ixf18100XgmacRead64(ushort *Address, uint64Struct_t *ptReturnedValue);

/******************************************************************************
* Function : ixf18100XgmacRead32
*------------------------------------------------------------------------------
* Description : This function reads two consecutive 16bit registers, placing 
*                the contents of the lower address register in the less
*                significant word of 32bits and the contents of the higher
*                address in the more significant word of 32bits.
*------------------------------------------------------------------------------
* Inputs  : ushort *Address: A 16bit register address containing the lower 
*           word of a 32bit value to be returned as the result of the 
*           read.
* Outputs : See Returns.
*------------------------------------------------------------------------------
* Returns : A 32bit unsigned integer with the contents of two consecutive
*           16bit registers.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
ulong
ixf18100XgmacRead32(ushort *Address)
{
    ushort uiHighBits;
    ushort uiLowBits;
    ulong ulThirtyTwoBits;

    uiLowBits = HAL_IXF18100_READ(Address);
    uiHighBits = HAL_IXF18100_READ(Address + 1);

    ulThirtyTwoBits = uiHighBits;
    ulThirtyTwoBits = ((ulThirtyTwoBits << 16) | uiLowBits);

    return ulThirtyTwoBits;
}

/******************************************************************************
* Function : ixf18100XgmacWrite32
*------------------------------------------------------------------------------
* Description : This function writes two 16bit unsigned integer values to 
*                two consecutive 16bit registers. The low word is written
*                to the low address and the high word is written to the 
*                high address.
*------------------------------------------------------------------------------
* Inputs  : ushort *Address: A pointer to the lower of two 16bit registers
*                            that are written to.
*           ulong ulValue: The 32bit value that will be split between the 
*                          higher and lower 16bit registers.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
void
ixf18100XgmacWrite32(ushort *Address, ulong ulValue)
{  
    ushort uiHighBits;
    ushort uiLowBits;

    uiHighBits = (ushort)((ulValue >> 0x10) & 0xffff);
    uiLowBits = (ushort)(ulValue & 0xffff);

    HAL_IXF18100_WRITE(Address, uiLowBits);
    HAL_IXF18100_WRITE((Address + 1), uiHighBits);
}

/******************************************************************************
* Function : ixf18100XgmacRead64
*------------------------------------------------------------------------------
* Description : This function fills a structure used to store 64 bits of data.
*------------------------------------------------------------------------------
* Inputs  : ushort *Address: A pointer to the lowest address of four 
*               consecutive 16bit registers used to store 64 bits of counter
*               data.
*           uint64Struct_t *ptReturnedValue: A pointer to the structure used 
*               to store the 64 bits of data.
* Outputs : The function fills the structure pointed to by the input parameter
*           ptReturnedValue.
*------------------------------------------------------------------------------
* Returns : None.
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : A structure is used to store counter data as there is no native
*            64bit data type.
******************************************************************************/
void
ixf18100XgmacRead64(ushort *Address, uint64Struct_t *ptReturnedValue)
{
    ulong ulHigh32Bits;
    ulong ulLow32Bits;

    ulLow32Bits = ixf18100XgmacRead32(Address);
    ulHigh32Bits = ixf18100XgmacRead32(Address + 2);

    ptReturnedValue->ulHighValue = ulHigh32Bits;
    ptReturnedValue->ulLowValue = ulLow32Bits;
}

/******************************************************************************
* Function : Ixf18100XgmacSetCfg
*------------------------------------------------------------------------------
* Description : This function configures the XGMAC block in the device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf18100_XgmacCfg_t *ptXgmacCfg: A pointer a structure containing
*                                          chip configuration settings.
* Outputs : Writes to registers to configure XGMAC settings on the chip.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100XgmacSetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_XgmacCfg_t *ptXgmacCfg)
{
    ulong ulValue;

    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }

    /* XGMAC_CNTL */
    ulValue = ((ptXgmacCfg->ucRxPauseFrameAddr
        << ixf18100_sRX_PAUSE_FRAME_ADDR) & ixf18100_mRX_PAUSE_FRAME_ADDR);

    ulValue |= ((ptXgmacCfg->ucCntlFrmFilterEn & 0x01)
        << ixf18100_sCNTL_FRM_FILTER_EN);

    ulValue |= ((ptXgmacCfg->ucCont_MDC_En & 0x01) << ixf18100_sCONT_MDC);

    ulValue |= ((ptXgmacCfg->ucMDC_SpdSel & 0x01) << ixf18100_sMDC_SPD_SEL);

    ulValue |= ((ptXgmacCfg->ucLoopback & 0x01) << ixf18100_sLOOPBACK);

    ulValue |= ((ptXgmacCfg->ucRxPauseEn & 0x01) << ixf18100_sRX_PAUSE_EN);

    ulValue |= ((ptXgmacCfg->ucRxEn & 0x01) << ixf18100_sRX_EN);

    ulValue |= ((ptXgmacCfg->ucTxPauseEn & 0x01) << ixf18100_sTX_PAUSE_EN);

    ulValue |= ((ptXgmacCfg->ucTxEn & 0x01) << ixf18100_sTX_EN);

    ulValue |= ((ptXgmacCfg->ucTxPadFrmEn & 0x01) << ixf18100_sTX_PAD_FRM_EN);

    ulValue |= ((ptXgmacCfg->ucSftRst & 0x01) << ixf18100_sSFT_RST);

    ulValue |= ((ptXgmacCfg->ucMngmntRst & 0x01) << ixf18100_sMNGMNT_RST);

    ulValue |= ((ptXgmacCfg->ucRxRst & 0x01) << ixf18100_sRX_RST);

    ulValue |= ((ptXgmacCfg->ucRxCRC_Rst & 0x01) << ixf18100_sRX_CRC_RST);

    ulValue |= ((ptXgmacCfg->ucTxRst & 0x01) << ixf18100_sTX_RST);

    ulValue |= ((ptXgmacCfg->ucTxCRC_Rst & 0x01) << ixf18100_sTX_CRC_RST);

    ulValue |= ((ptXgmacCfg->ucJumFrmEn & 0x01) << ixf18100_sJUM_FRM_EN);

    ulValue |= ((ptXgmacCfg->ucRxCRC_Strip & 0x01) << ixf18100_sRX_CRC_STRIP);

    ulValue |= ((ptXgmacCfg->ucTxCRC_En & 0x01) << ixf18100_sTX_CRC_EN);

    ixf18100XgmacWrite32(BaseAddress + ixf18100_XGMAC_CNTL_W0_OFFSET,
        ulValue);

    /* XGMAC_CNTL2 */
    ulValue = ((ptXgmacCfg->ucRxCrcErrorDropEn & 0x01)
        << ixf18100_sRX_CRC_ERROR_DROP_EN);

    ulValue |= ((ptXgmacCfg->ucRxOversizeDropEn & 0x01)
        << ixf18100_sRX_OVERSIZE_DROP_EN);

    ulValue |= ((ptXgmacCfg->ucRxUndersizeDropEn & 0x01)
        << ixf18100_sRX_UNDERSIZE_DROP_EN);

    ixf18100XgmacWrite32(BaseAddress + ixf18100_XGMAC_CNTL2_W0_OFFSET,
        ulValue);

    /* XGMAC_PP */
    if((ptXgmacCfg->eTxPace > 0xf) | ((ptXgmacCfg->eTxPace > 0x9)
        && (ptXgmacCfg->eTxPace < 0xf)))
    {
        return bb_INV_PARAMETER;
    }

    ulValue = ((ptXgmacCfg->eTxPace << ixf18100_sTX_PACE) & ixf18100_mTX_PACE);

    ulValue |= ((ptXgmacCfg->uiTxPauseCnt << ixf18100_sTX_PAUSE_CNT)
        & ixf18100_mTX_PAUSE_CNT);

    ixf18100XgmacWrite32(BaseAddress + ixf18100_XGMAC_PP_W0_OFFSET, ulValue);

    /* XGMAC_ADDLO */
    ulValue = ptXgmacCfg->MAC_Addr.ulLowValue;
    
    ixf18100XgmacWrite32(BaseAddress + ixf18100_XGMAC_ADDLO_W0_OFFSET,
        ulValue);

    /* XGMAC_ADDHIJUM */
    ulValue = ((ptXgmacCfg->uiMaxFrmSize << ixf18100_sMAX_FRM_SIZE)
        & ixf18100_mMAX_FRM_SIZE);

    ulValue |= ((ptXgmacCfg->MAC_Addr.ulHighValue << ixf18100_sMAC_ADDR_HIGH)
        & ixf18100_mMAC_ADDR_HIGH);

    ixf18100XgmacWrite32(BaseAddress + ixf18100_XGMAC_ADDHIJUM_W0_OFFSET,
        ulValue);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100XgmacGetCfg
*------------------------------------------------------------------------------
* Description : This function reads the XGMAC configuration settings into a 
*                data structure.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelConfig_e SelCfg: Selects the part of the block to be
*                                   configured.
*           ixf18100_XgmacCfg_t *ptXgmacCfg: A pointer to a structure used to
*                                          contain configuration information
*                                          obtained from the XGMAC registers.
* Outputs : A filled data structure containing XGMAC configuration information.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e 
Ixf18100XgmacGetCfg(bb_RegPointer_type BaseAddress, bb_SelConfig_e SelCfg,
                    ixf18100_XgmacCfg_t *ptXgmacCfg)
{
    ulong ulValue;
    uchar ucTxPaceSetting;
        
    if (SelCfg != e_ALL_CONFIG)
    {
        return bb_INV_PARAMETER;
    }

    /* XGMAC_CNTL */
    ulValue = ixf18100XgmacRead32(BaseAddress
        + ixf18100_XGMAC_CNTL_W0_OFFSET);

    ptXgmacCfg->ucRxPauseFrameAddr
        = (uchar)((ulValue & ixf18100_mRX_PAUSE_FRAME_ADDR)
            >> ixf18100_sRX_PAUSE_FRAME_ADDR);

    ptXgmacCfg->ucCntlFrmFilterEn
        = (uchar)((ulValue >> ixf18100_sCNTL_FRM_FILTER_EN) & 0x01);

    ptXgmacCfg->ucCont_MDC_En
        = (uchar)((ulValue >> ixf18100_sCONT_MDC) & 0x01);

    ptXgmacCfg->ucMDC_SpdSel
        = (uchar)((ulValue >> ixf18100_sMDC_SPD_SEL) & 0x01);

    ptXgmacCfg->ucLoopback = (uchar)((ulValue >> ixf18100_sLOOPBACK) & 0x01);

    ptXgmacCfg->ucRxPauseEn
        = (uchar)((ulValue >> ixf18100_sRX_PAUSE_EN) & 0x01);

    ptXgmacCfg->ucRxEn = (uchar)((ulValue >> ixf18100_sRX_EN) & 0x01);

    ptXgmacCfg->ucTxPauseEn
        = (uchar)((ulValue >> ixf18100_sTX_PAUSE_EN) & 0x01);

    ptXgmacCfg->ucTxEn = (uchar)((ulValue >> ixf18100_sTX_EN) & 0x01);

    ptXgmacCfg->ucTxPadFrmEn
        = (uchar)((ulValue >> ixf18100_sTX_PAD_FRM_EN) & 0x01);

    ptXgmacCfg->ucSftRst = (uchar)((ulValue >> ixf18100_sSFT_RST) & 0x01);

    ptXgmacCfg->ucMngmntRst
        = (uchar)((ulValue >> ixf18100_sMNGMNT_RST) & 0x01);

    ptXgmacCfg->ucRxRst = (uchar)((ulValue >> ixf18100_sRX_RST) & 0x01);

    ptXgmacCfg->ucRxCRC_Rst
        = (uchar)((ulValue >> ixf18100_sRX_CRC_RST) & 0x01);

    ptXgmacCfg->ucTxRst = (uchar)((ulValue >> ixf18100_sTX_RST) & 0x01);

    ptXgmacCfg->ucTxCRC_Rst
        = (uchar)((ulValue >> ixf18100_sTX_CRC_RST) & 0x01);

    ptXgmacCfg->ucJumFrmEn = (uchar)((ulValue >> ixf18100_sJUM_FRM_EN) & 0x01);

    ptXgmacCfg->ucRxCRC_Strip
        = (uchar)((ulValue >> ixf18100_sRX_CRC_STRIP) & 0x01);

    ptXgmacCfg->ucTxCRC_En = (uchar)((ulValue >> ixf18100_sTX_CRC_EN) & 0x01);

    /* XGMAC_CNTL2 */
    ulValue = ixf18100XgmacRead32(BaseAddress
        + ixf18100_XGMAC_CNTL2_W0_OFFSET);

    ptXgmacCfg->ucRxCrcErrorDropEn
        = (uchar)((ulValue >> ixf18100_sRX_CRC_ERROR_DROP_EN) & 0x01);

    ptXgmacCfg->ucRxOversizeDropEn
        = (uchar)((ulValue >> ixf18100_sRX_OVERSIZE_DROP_EN) & 0x01);

    ptXgmacCfg->ucRxUndersizeDropEn
        = (uchar)((ulValue >> ixf18100_sRX_UNDERSIZE_DROP_EN) & 0x01);

    /* XGMAC_PP */
    ulValue = ixf18100XgmacRead32(BaseAddress + ixf18100_XGMAC_PP_W0_OFFSET);

    ucTxPaceSetting = (uchar)((ulValue & ixf18100_mTX_PACE)
        >> ixf18100_sTX_PACE);

    ptXgmacCfg->eTxPace = (eIxf18100_TxPace_t)ucTxPaceSetting;

    ptXgmacCfg->uiTxPauseCnt = (ushort)((ulValue & ixf18100_mTX_PAUSE_CNT)
        >> ixf18100_sTX_PAUSE_CNT);

    /* XGMAC_ADDLO */
    ptXgmacCfg->MAC_Addr.ulLowValue
        = ixf18100XgmacRead32(BaseAddress + ixf18100_XGMAC_ADDLO_W0_OFFSET);

    /*XGMAC_ADDHIJUM*/
    ulValue = ixf18100XgmacRead32(BaseAddress
        + ixf18100_XGMAC_ADDHIJUM_W0_OFFSET);
    
    ptXgmacCfg->uiMaxFrmSize = (ushort)((ulValue & ixf18100_mMAX_FRM_SIZE)
        >> ixf18100_sMAX_FRM_SIZE);

    ptXgmacCfg->MAC_Addr.ulHighValue = (ushort)((ulValue
        & ixf18100_mMAC_ADDR_HIGH) >> ixf18100_sMAC_ADDR_HIGH);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100XgmacGetStatus
*------------------------------------------------------------------------------
* Description : This function reads the status registers of the XGMAC register
*               block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelStatus_e SelStat: Selects the status to be retrieved.
*           ixf18100_XgmacStatus_t *ptXgmacStatus: pointer to the status
*                                                   structure
* Outputs : A filled data structure containing XGMAC status information. 
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100XgmacGetStatus(bb_RegPointer_type BaseAddress, bb_SelStatus_e SelStat,
                       ixf18100_XgmacStatus_t *ptXgmacStatus)
{
    ulong ulValue;

    /* XGMAC_STATUS */
    ulValue = ixf18100XgmacRead32(BaseAddress
        + ixf18100_XGMAC_STATUS_W0_OFFSET);

    ptXgmacStatus->ucRxLinkFail
        = (uchar)((ulValue >> ixf18100_sRX_LINK_FAIL) & 0x01);

    ptXgmacStatus->ucRxRemoteFault
        = (uchar)((ulValue >> ixf18100_sRX_REMOTE_FAULT) & 0x01);

    ptXgmacStatus->ucRxLocalFault
        = (uchar)((ulValue >> ixf18100_sRX_LOCAL_FAULT) & 0x01);

    ptXgmacStatus->ucRxIdleError
        = (uchar)((ulValue >> ixf18100_sRX_IDLE_ERROR) & 0x01);

    ptXgmacStatus->ucRxIllegalSymbol
        =(uchar)((ulValue >> ixf18100_sRX_ILLEGAL_SYM) & 0x01);

    ptXgmacStatus->ucRxErrorSymbol
        = (uchar)((ulValue >> ixf18100_sRX_ER_SYM) & 0x01);

    ptXgmacStatus->ucAutoscanInProgress
        = (uchar)((ulValue >> ixf18100_sAUTOSCAN) & 0x01);

    ptXgmacStatus->ucRevisionID
        = (uchar)((ulValue & ixf18100_mREV_ID) >> ixf18100_sREV_ID);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100XgmacGetCounters
*------------------------------------------------------------------------------
* Description : This function reads counter values in the XGMAC register
*                block.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           bb_SelCounters_e SelCntr: Selects the counters to be read.
*           void *ptCounters: A pointer to a structure used to
*                                          store counter values.
* Outputs : A filled data structure containing counter values.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_SEL_COUNTERS
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100XgmacGetCounters(bb_RegPointer_type BaseAddress, 
                         bb_SelCounters_e SelCntr, void *ptCounters)
{
    ixfApi_XgmacCounters_t *ptCounterData;
    uint64Struct_t *uDWordValue;

    switch(SelCntr)
    {
    case bb_XGMAC_TX_OK_OCTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_OCTETS_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_OK_MLTCAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MULTICAST_FRAMES_TX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_TX_OK_BRDCAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_BROADCAST_FRAMES_TX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_TX_OK_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_FRAMES_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_64_OCT_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_64OCTET_TX_FRAMES_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_65_TO_127_OCT_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_65TO127_TX_FRAMES_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_128_TO_255_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_128TO255_TX_FRAMES_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_TX_256_TO_511_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_256TO511_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_OK_512_TO_1023_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_512TO1023_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_OK_1024_TO_15XX_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_1024TO15XX_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_OK_15XX_TO_MAX_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_15XXTOMAX_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_VLAN_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_VLAN_FRAMES_TX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_TX_PAUSE_CTRL_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_PAUSE_CTRL_FRAMES_TX_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_TX_UNICAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_UNICAST_FRAMES_TX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_TX_MAC_CTRL_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MAC_CNTRL_FRAMES_TX_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_OK_OCTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_OCTETS_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_OK_MLTCAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MULTICAST_FRAMES_RX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_OK_BRDCAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_BROADCAST_FRAMES_RX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_OK_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_FRAMES_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_64_OCT_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_64OCTET_RX_FRAMES_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_65_TO_127_OCT_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_65TO127_RX_FRAMES_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_128_TO_255_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_128TO255_RX_FRAMES_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_256_TO_511_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_256TO511_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_OK_512_TO_1023_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_512TO1023_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_OK_1024_TO_15XX_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_1024TO15XX_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_OK_15XX_TO_MAX_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_15XXTOMAX_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_VLAN_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_VLAN_FRAMES_RX_OK_W0_OFFSET,
            uDWordValue);
        break;

    case bb_XGMAC_RX_PAUSE_CTRL_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_PAUSE_MAC_CTRL_FRAMES_RX_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_UNICAST_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_UNICAST_FRAMES_RX_OK_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_MAC_CTRL_FRM_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MAC_CNTRL_FRAMES_RX_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_ETHER_STATS_USIZE_PKTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_UNDERSIZE_PKTS_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_ETHER_STATS_OSIZE_PKTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_OVERSIZE_PKTS_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_ETHER_STATS_OCTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_OCTECTS_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_RX_ETHER_STATS_PKTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress + ixf18100_ETHER_STATS_PKTS_W0_OFFSET,
            uDWordValue);
        break;
    
    case bb_XGMAC_RX_ETHER_STATS_FRGMNTS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_FRAGMENTS_W0_OFFSET, uDWordValue);
        break;
    
    case bb_XGMAC_RX_ETHER_STATS_JABBERS_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_JABBERS_W0_OFFSET, uDWordValue);
        break;
    
    case bb_XGMAC_RX_FRM_CHK_SEQ_ERR_CNT:
        uDWordValue = (uint64Struct_t *) ptCounters;
        ixf18100XgmacRead64(BaseAddress
            + ixf18100_FRAME_CHECK_SEQ_ERRS_W0_OFFSET, uDWordValue);
        break;

    case bb_XGMAC_COUNTERS:
        ptCounterData = (ixfApi_XgmacCounters_t *) ptCounters;

        ixf18100XgmacRead64(BaseAddress + ixf18100_OCTETS_TX_OK_W0_OFFSET,
            &ptCounterData->OctetsTxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MULTICAST_FRAMES_TX_OK_W0_OFFSET,
                &ptCounterData->MulticastFramesTxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_BROADCAST_FRAMES_TX_OK_W0_OFFSET,
                &ptCounterData->BroadcastFramesTxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_FRAMES_TX_OK_W0_OFFSET,
            &ptCounterData->FramesTxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_64OCTET_TX_FRAMES_W0_OFFSET,
            &ptCounterData->Counter64OctetTxFrames);

        ixf18100XgmacRead64(BaseAddress + ixf18100_65TO127_TX_FRAMES_W0_OFFSET,
            &ptCounterData->Counter65to127TxFrames);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_128TO255_TX_FRAMES_W0_OFFSET,
                &ptCounterData->Counter128to255TxFrames);

        ixf18100XgmacRead64(BaseAddress + ixf18100_256TO511_TX_OK_W0_OFFSET,
            &ptCounterData->Counter256to511TxFrames);

        ixf18100XgmacRead64(BaseAddress + ixf18100_512TO1023_TX_OK_W0_OFFSET,
            &ptCounterData->Counter512to1023TxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_1024TO15XX_TX_OK_W0_OFFSET,
            &ptCounterData->Counter1024to15xxTxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_15XXTOMAX_TX_OK_W0_OFFSET,
            &ptCounterData->Counter15xxtoMaxTxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_VLAN_FRAMES_TX_OK_W0_OFFSET,
            &ptCounterData->VLANFramesTxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_PAUSE_CTRL_FRAMES_TX_W0_OFFSET,
                &ptCounterData->PAUSECtrlFramesTx);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_UNICAST_FRAMES_TX_OK_W0_OFFSET,
                &ptCounterData->UnicastFramesTxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MAC_CNTRL_FRAMES_TX_W0_OFFSET,
                &ptCounterData->MACCtrlFramesTx);

        ixf18100XgmacRead64(BaseAddress + ixf18100_OCTETS_RX_OK_W0_OFFSET,
            &ptCounterData->OctetsRxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MULTICAST_FRAMES_RX_OK_W0_OFFSET,
                &ptCounterData->MulticastFramesRxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_BROADCAST_FRAMES_RX_OK_W0_OFFSET,
                &ptCounterData->BroadcastFramesRxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_FRAMES_RX_OK_W0_OFFSET,
            &ptCounterData->FramesRxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_64OCTET_RX_FRAMES_W0_OFFSET,
            &ptCounterData->Counter64OctetRxFrames);

        ixf18100XgmacRead64(BaseAddress + ixf18100_65TO127_RX_FRAMES_W0_OFFSET,
            &ptCounterData->Counter65to127RxFrames);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_128TO255_RX_FRAMES_W0_OFFSET,
                &ptCounterData->Counter128to255RxFrames);

        ixf18100XgmacRead64(BaseAddress + ixf18100_256TO511_RX_OK_W0_OFFSET,
            &ptCounterData->Counter256to511RxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_512TO1023_RX_OK_W0_OFFSET,
            &ptCounterData->Counter512to1023RxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_1024TO15XX_RX_OK_W0_OFFSET,
            &ptCounterData->Counter1024to15xxRxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_15XXTOMAX_RX_OK_W0_OFFSET,
            &ptCounterData->Counter15xxtoMaxRxOK);

        ixf18100XgmacRead64(BaseAddress + ixf18100_VLAN_FRAMES_RX_OK_W0_OFFSET,
            &ptCounterData->VLANFramesRxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_PAUSE_MAC_CTRL_FRAMES_RX_W0_OFFSET,
                &ptCounterData->PAUSEMACCtrlFramesRx);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_UNICAST_FRAMES_RX_OK_W0_OFFSET,
                &ptCounterData->UnicastFramesRxOK);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_MAC_CNTRL_FRAMES_RX_W0_OFFSET,
                &ptCounterData->MACCtrlFramesRx);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_UNDERSIZE_PKTS_W0_OFFSET,
                &ptCounterData->EtherStatsUndersizePkts);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_OVERSIZE_PKTS_W0_OFFSET,
                &ptCounterData->EtherStatsOversizePkts);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_OCTECTS_W0_OFFSET,
                &ptCounterData->EtherStatsOctets);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_PKTS_W0_OFFSET,
                &ptCounterData->EtherStatsPkts);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_FRAGMENTS_W0_OFFSET,
                &ptCounterData->EtherStatsFragments);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_ETHER_STATS_JABBERS_W0_OFFSET,
                &ptCounterData->EtherStatsJabbers);

        ixf18100XgmacRead64(BaseAddress
            + ixf18100_FRAME_CHECK_SEQ_ERRS_W0_OFFSET,
                &ptCounterData->FrameCheckSeqErr);
        break;

    default:
        return bb_INV_SEL_COUNTERS;
    }

    return bb_NO_ERROR;    
}

/******************************************************************************
* Function : Ixf18100XgmacSetAddress
*------------------------------------------------------------------------------
* Description :  Function that sets th MAC address for a ixf18100 device
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Not used.
*           pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the address.
* Outputs : None
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100XgmacSetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress)
{
    ulong ulValue;

    /* Validate ChipType */
    if (pChipData->ChipType != bb_18101_CHIP &&
        pChipData->ChipType != bb_18103_CHIP &&
        pChipData->ChipType != bb_18104_CHIP &&
        pChipData->ChipType != bb_18101_8023WIS_CHIP &&
        pChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return (bb_INV_CHIP_TYPE);
    }

    /* XGMAC_ADDLO */
    ixf18100XgmacWrite32(pChipData->BaseAddress
        + ixf18100_XGMAC_ADDLO_W0_OFFSET, pMacAddress->ulLowValue); 

    /*XGMAC_ADDHIJUM*/
    ulValue = ixf18100XgmacRead32(pChipData->BaseAddress
        + ixf18100_XGMAC_ADDHIJUM_W0_OFFSET);

    pMacAddress->ulHighValue &= (ixf18100_mMAC_ADDR_HIGH);

    ulValue &= ~(ixf18100_mMAC_ADDR_HIGH);
    
    ixf18100XgmacWrite32(pChipData->BaseAddress
        + ixf18100_XGMAC_ADDHIJUM_W0_OFFSET, ulValue);

    return bb_NO_ERROR;
}

/******************************************************************************
* Function : Ixf18100XgmacGetAddress
*------------------------------------------------------------------------------
* Description : This function reads the Mac Address from the device.
*------------------------------------------------------------------------------
* Inputs  : bb_RegPointer_type BaseAddress: The base address of the chip.
*           section: Not used.
* Outputs : pMacAddress: A 64 bits structure divide in 2 32 bits unsigned 
*                        integers which contains the MAC address.
*------------------------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_CHIP_TYPE
*------------------------------------------------------------------------------
* Access Globals : None.
*------------------------------------------------------------------------------
* Comments : 
******************************************************************************/
bb_Error_e
Ixf18100XgmacGetAddress(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                        IxfApi_MacAddress_t *pMacAddress)
{
    /* Validate ChipType */
    if (pChipData->ChipType != bb_18101_CHIP &&
        pChipData->ChipType != bb_18103_CHIP &&
        pChipData->ChipType != bb_18104_CHIP &&
        pChipData->ChipType != bb_18101_8023WIS_CHIP &&
        pChipData->ChipType != bb_18103_8023WIS_CHIP)
    {
        return (bb_INV_CHIP_TYPE);
    }

    /* XGMAC_ADDLO */
    pMacAddress->ulLowValue = ixf18100XgmacRead32(pChipData->BaseAddress
        + ixf18100_XGMAC_ADDLO_W0_OFFSET);

    /* XGMAC_ADDHIJUM */
    pMacAddress->ulHighValue = ixf18100XgmacRead32(pChipData->BaseAddress
        + ixf18100_XGMAC_ADDHIJUM_W0_OFFSET);

    pMacAddress->ulHighValue &= (ixf18100_mMAC_ADDR_HIGH);

    return bb_NO_ERROR;
}
