/*
*---------------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------------
*
*
*   AUTHOR:         $Author: Tepulkow $
*   DATE:           $Date: 3/08/04 1:56p $
*   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
*   REV#:           $Revision: 1 $
*   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf_api.c $
*   WORKFILE:       $Workfile: ixf_api.c $
*
* --------------------------------------------------------------------------
* Project: IXF6192
*
* Purpose: Provides the implementation to the common API for IXF chips.
*          
* Notes:
*          
* --------------------------------------------------------------------------
* CompilerFlags: 
* --------------------------------------------------------------------------
* --------------------------------------------------------------------------
*   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/ixf_api.c $
 * 
 * 1     3/08/04 1:56p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 46    11/07/02 11:06a Jaalliso
 * IxfApiInit return bb_INV_CHIP_TYPE if chip function table empty,
 * renamed IXF driver name labels
 * 
 * 45    5/14/02 4:14p Machambe
 * Added InitChip 2nd Param.
 * 
 * 44    4/24/02 2:03p Rcordova
 * Code changes for debugging  generic driver code
 * 
 * 43    4/24/02 10:39a Srodrig1
 * 
 * 42    4/18/02 11:31a Jaalliso
 * Opmode parameter for GetOpmode should be passed by reference
 * 
 * 41    4/05/02 12:34p Machambe
 * Adding more pointer checking.
* 
* 40    4/03/02 9:50a Machambe
* Adding include <string.h> to non-VxWorks build
* 
* 39    2/14/02 2:51p Jaalliso
* Changing return code bb_INV_CHIP_TYPE to bb_FN_NOT_SUPPORTED
* 
* 38    2/07/02 5:02p Skadrix
* Added a section (ChipSegment) parameter to the TestCfg API function.
* 
* 37    12/05/01 3:26p Pkjain2
* 
* 36    12/04/01 4:22p Pkjain2
* 
* 35    11/28/01 12:03p Machambe
* 
* 34    11/18/01 10:48p Machambe
* 
* 33    11/16/01 8:45p Pkjain2
* 
* 32    11/15/01 6:40p Machambe
* 
* 31    11/14/01 3:26p Machambe
* Fixing common function pointers
* 
* 29    11/14/01 2:11p Machambe
* 
* 28    11/14/01 1:44p Machambe
* 
* 27    11/14/01 1:42p Machambe
* 
* 26    11/14/01 1:38p Machambe
* 
* 25    11/14/01 12:07p Machambe
* 
* 24    11/14/01 11:50a Machambe
* 
* 23    11/14/01 11:38a Machambe
* Modifying to the new devlib.h file
* 
* 22    7/20/01 9:11a Machambe
* Adding IxfApiGetBuildVersion
* 
* 21    6/25/01 10:54a Machambe
* 
* 20    6/11/01 4:11p Machambe
* 
* 19    5/03/01 3:54p Machambe
* 
* --------------------------------------------------------------------------
*/ 

#ifdef VXWORKS
#include "vxWorks.h"
#elif defined(__KERNEL__)
#include <linux/config.h>
#include <linux/module.h>
#include <linux/string.h>
#include <asm/memory.h>
#else 
#include "memory.h"
#include "string.h"
#endif
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h"
#include "ixf_api.h"
#include "builds.h"

#include "devlib.h"

#ifdef __KERNEL__
int init_module(void)
{
	return 0;
}
void cleanup_module(void)
{
}
#endif
/******************************************************************
* Function : IxfApiInit
*-----------------------------------------------------------------
* Description  : This routine initializes the main data structure
*                and returns a handle to that structure.
*-----------------------------------------------------------------
* Inputs  : baseAddress : Specifies the base address of the device
*           chipType    : Specifices the type of device
*           pChipData   : A pointer to an allocated structure.
* Outputs : pChipData   : A pointer to the chip data structure.
*                         This structure is passed to most API
*                         routines.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further 
*            initialization.
******************************************************************/
bb_Error_e 
IxfApiInit(void *baseAddress, bb_ChipType_e chipType,
           bb_ChipData_t *pChipData)
{
    
    /* Store the input parameters in our data structures. */
    pChipData->BaseAddress = (bb_RegPointer_type) baseAddress;
    pChipData->ChipType = chipType;
    
    /* Initialize the chip configuration structure */
    pChipData->pChipCfg = NULL;
    
    /* Initialize the chip alarm structure */
    pChipData->pAlarmCfg = NULL;
    
    /* Initialize the chip's api routines */
    pChipData->funcPtr = (void *) getChipFuncPtr(chipType);
    
    /* If getChipFuncPtr returns NULL pointer, no entry was found */
    /* in chip function table so return bb_INV_CHIP_TYPE.  Else,  */
    /* return bb_NO_ERROR                                         */ 
    return (pChipData->funcPtr == NULL ? bb_INV_CHIP_TYPE : bb_NO_ERROR);
}

/******************************************************************
* Function : IxfApiInitChip
*-----------------------------------------------------------------
* Description  : This routine commits the internal register table
*                down to the chip and initializes the bb_ChipData_t
*                structure.
*-----------------------------------------------------------------
* Inputs  : pChipData  : A pointer to an allocated structure.
*           pTable : Table to be committed to the device.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiAllocDataStructureMem for further 
*            initialization.
******************************************************************/
bb_Error_e 
IxfApiInitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->InitChipFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->InitChipFuncPtr)(pChipData, pTable);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiAllocDataStructureMem
*-----------------------------------------------------------------
* Description  : This routine dynamically allocates memory within
*                the "pChipData" parameter for those members that 
*                need dynamic allocation.
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure without 
*                       pointers memory allocation.
* Outputs : pChipData : A pointer to the chip data structure with 
*                       pointer memory allocation.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : See IxfApiInit for the completion of the initialization
******************************************************************/
bb_Error_e 
IxfApiAllocDataStructureMem(bb_ChipData_t *pChipData)
{  
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->AllocFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->AllocFuncPtr)(pChipData);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiDeAllocMemory
*-----------------------------------------------------------------
* Description  : This routine frees the dynamically allocated memory
*                within the "pChipData" structure
*-----------------------------------------------------------------
* Inputs  : pChipData : A pointer to the chip data structure.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : Status
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
IxfApiDeAllocMemory(bb_ChipData_t *pChipData)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->DeAllocFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->DeAllocFuncPtr)(pChipData);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiSetCfg
*-----------------------------------------------------------------
* Description  : This routine is used to configure the chip.
*                Many different types of configurations could be
*                performed with the combination of SelCfg and the 
*                generic pointer of the ptCfg.
*-----------------------------------------------------------------
* Inputs  : pChipData : pointer to the chip information.
*           ptSegment : pointer to chip segment structure
*           SelCfg    : enum value defining what configuration
*                       needs to be set
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e
IxfApiSetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
             bb_SelConfig_e SelCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->SetCfgFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->SetCfgFuncPtr)(pChipData, ptSegment,
                SelCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}



/******************************************************************
* Function : IxfApiGetCfg
*-----------------------------------------------------------------
* Description  : This routine is used to get the configuration of 
*                the chip. Many different types of configurations
*                could be performed with the combination of SelCfg
*                and the sgeneric pointer of the ptCfg.
*-----------------------------------------------------------------
* Inputs  : pChipData : pointer to the chip information.
*           ptSegment : pointer to chip segment structure
*           SelCfg    : enum value defining what configuration
*                       needs to be retrieved
* Outputs : None
*-----------------------------------------------------------------
* Returns : bb_NO_ERROR, bb_INV_PARAMETER
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e
IxfApiGetCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *ptSegment,
             bb_SelConfig_e SelCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;    
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->GetCfgFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->GetCfgFuncPtr)(pChipData, ptSegment,
                SelCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}


/******************************************************************
* Function : IxfApiGetStatus
*-----------------------------------------------------------------
* Description  : Gets the Status.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           selStatus : Type of Status: 
* Outputs : pStatus   : place to put Status.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
IxfApiGetStatus(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_SelStatus_e selStatus, void *pStatus)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->GetStatusFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->GetStatusFuncPtr)(pChipData, section,
                selStatus,  pStatus);
        } 
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiGetOpMode
*-----------------------------------------------------------------
* Description  : Gets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
* Outputs : pModeCfg  : Place to put mode.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
IxfApiGetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e *opMode, void *pModeCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->GetOpModeFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->GetOpModeFuncPtr)(pChipData, section,
                  opMode, pModeCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiSetOpMode
*-----------------------------------------------------------------
* Description  : Sets the operation mode of the chip.
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to get the
*                       status from.
*           opMode    : The mode of operation.
*           pModeCfg  : place to put mode.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
IxfApiSetOpMode(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                bb_OperMode_e opMode, void *pModeCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->SetOpModeFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->SetOpModeFuncPtr)(pChipData, section,
                opMode, pModeCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}


/******************************************************************
* Function : IxfApiGetCounters
*-----------------------------------------------------------------
* Description  : Retrieve a set of Counters
*-----------------------------------------------------------------
* Inputs  : pChipData   : Validated Chip Data
*           section     : Determines the section(block) to get the
*                         counters from.
*           SelCounters : Selects the set of counters
* Outputs : pCounters   : Place to put Counters
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiGetCounters(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_SelCounters_e SelCounters, void *pCounters)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->GetCountersFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->GetCountersFuncPtr)(pChipData,
                section, SelCounters, pCounters);  
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiInitAlarmCallback
*-----------------------------------------------------------------
* Description  : Initialize the Driver
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           pAlarmCallback : Alarm Callback
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiInitAlarmCallback(bb_ChipData_t *pChipData, AlarmCallBack pAlarmCallbackArg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->InitAlarmCallbackFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->InitAlarmCallbackFuncPtr)(pChipData,
                pAlarmCallbackArg);    
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiChipIsr
*-----------------------------------------------------------------
* Description  : Handle Interupts for Chip
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : None.
*-----------------------------------------------------------------
* Returns : None.
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments:   Interrupts are reported through a Callback function.
*
*   Transmit and receive Interrupts act independantly of each another.
*
*   Interrupts have been prioritized to be handled according to the severity 
*   of the effecting interupt. See the MasterInterruptHandler function for 
*   specifics
******************************************************************/
void
IxfApiChipIsr(bb_ChipData_t *pChipData)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ChipIsrFuncPtr)
        {
            /* Call Function Pointer */
            (pChipFuncPtrs->ChipIsrFuncPtr)(pChipData);
        }
    }
    
    return;
}


/******************************************************************
* Function : IxfApiGetChipInfo
*-----------------------------------------------------------------
* Description  : Get the chip information.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
* Outputs : pChipInfo : Place to return chip info
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiGetChipInfo(bb_ChipData_t *pChipData, bb_ChipInfo_t *pChipInfo)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ChipInfoFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->ChipInfoFuncPtr)(pChipData, pChipInfo);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}


/******************************************************************
* Function : IxfApiSetChipOnline
*-----------------------------------------------------------------
* Description  : Puts some or all of the chip online.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section   : Specifies which portion of the chip to 
*                       put online.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None. 
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiSetChipOnline(bb_ChipData_t *pChipData,
                    bb_ChipSegment_t *section)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ChipOnlineFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->ChipOnlineFuncPtr)(pChipData, section);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiSetChipOffline
*-----------------------------------------------------------------
* Description  : Takes some or all of the chip offline.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section  :  Specifies which portion of the chip to 
*                       take offline.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiSetChipOffline(bb_ChipData_t *pChipData, bb_ChipSegment_t *section)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ChipOfflineFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->ChipOfflineFuncPtr)(pChipData, section);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiReset
*-----------------------------------------------------------------
* Description  : Reset (chip or a section of the chip) and then 
*                reconfigure it. To restart traffic, the chip or 
*                section of chip must be set back online
*-----------------------------------------------------------------
* Inputs  : pChipData : Initialized Chip Data
*           section   : Determines the section(block) to reset
*           resetType : Specifies type of Reset
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiReset(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
            bb_SelResetType_e resetType)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ResetFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->ResetFuncPtr)(pChipData, section, resetType);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiCfgTest
*-----------------------------------------------------------------
* Description  : Run Tests
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section   : Determines the section(block) to run the
*                       test for.
*           testType : The type of test to run.
*           pTestCfg : Contains the test configuration data.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e
IxfApiCfgTest(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
              bb_TestType_e testType, void *pTestCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->CfgTestFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->CfgTestFuncPtr)(pChipData, section,
                testType, pTestCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiSetAlarmCfg
*-----------------------------------------------------------------
* Description  : Edits Interrupt Enabled Masks for a Specific Set
*                of Interrupts.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           section   : Determines the section(block) of the alarm(s)
*                       to edit.
*           AlarmType : Specifies the type of Alarm
*           pAlarmCfg : A pointer to the alarm structure.
* Outputs : 
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiSetAlarmCfg(bb_ChipData_t *pChipData, bb_ChipSegment_t *section,
                  bb_AlarmType_e AlarmType, void *pAlarmCfg)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->SetAlarmCfgFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->SetAlarmCfgFuncPtr)(pChipData, section,
                AlarmType, pAlarmCfg);
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}

/******************************************************************
* Function : IxfApiGetBuildVersion
*-----------------------------------------------------------------
* Description  : This routine returns the firmware build version of 
*                given the particular software driver identified.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           swType    : Identifies the upper or lower layer driver.
* Outputs : drvName   : Null terminated string describing the driver
*           date      : The build date of the driver.
*           buildVer  : The version number of the driver.
*           buildRev  : The revision number of the driver.
*-----------------------------------------------------------------
* Returns : bb_Error_e  Returns Error Condition if Not successful
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : None.
******************************************************************/
bb_Error_e 
IxfApiGetBuildVersion(bb_ChipData_t *pChipData, bb_SoftwareVersion_e swType,
                      char *drvName, char *date, ushort *buildVer, ushort *buildRev)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        if (swType == bb_SW_VERSION_CHIP_TYPE)
        {
            
            /* If function pointer is valid */
            if (pChipFuncPtrs->GetBuildVersionFuncPtr)
            {
                /* Call Function Pointer */
                rc = (pChipFuncPtrs->GetBuildVersionFuncPtr)(pChipData,
                    drvName, date, buildVer, buildRev);
            }
            else
            {
                rc = bb_FN_NOT_SUPPORTED;
            }
        }
        else if (swType == bb_SW_VERSION_API_TYPE)
        {
            /* Get the information for the API layer */
            strcpy(drvName, IXF_DRIVER_NAME);
            strcpy(date, IXF_BUILD_DATE);
            *buildVer = IXF_BUILD_VERSION;
            *buildRev = IXF_BUILD_REV;
            
            rc = bb_NO_ERROR;
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}


/******************************************************************
* Function : IxfApiGenericRead
*-----------------------------------------------------------------
* Description  : This routine will read starting at a particular 
*                location and read for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           wordSize  : Identifies the size of a word.
*           address   : Location where to start reading from.
*           length    : Number of words to read.
* Outputs : buffer    : The buffer to contain the read data.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiGenericRead(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize,
                  ulong address, ushort length, void *buffer)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->ReadFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->ReadFuncPtr)(pChipData, wordSize,
                address, length, buffer);          
        } 
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}


/******************************************************************
* Function : IxfApiGenericWrite
*-----------------------------------------------------------------
* Description  : This routine will write starting at a particular 
*                location and write for N number of words.
*-----------------------------------------------------------------
* Inputs  : pChipData : Validated Chip Data
*           wordSize  : Identifies the size of a word.
*           address   : Location where to start writing to.
*           length    : Number of words to write.
*           buffer    : The buffer to contain the read data.
* Outputs : None.
*-----------------------------------------------------------------
* Returns : 
*-----------------------------------------------------------------
* Access Globals : None.
*-----------------------------------------------------------------
* Comments : 
******************************************************************/
bb_Error_e 
IxfApiGenericWrite(bb_ChipData_t *pChipData, bb_Word_Size_t wordSize,
                   ulong address, ushort length, void *buffer)
{
    CommonFunctionPointers_t *pChipFuncPtrs;
    bb_Error_e rc = bb_NO_ERROR;
    
    /* Check to see if the specific chip's function pointer table
    has be initialized */
    if(((ChipFunctionPointers_t *) pChipData->funcPtr) != NULL)
    {
        if(((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr ==
            NULL)
        {
            /* Set Return Code */
            rc =  bb_FN_NOT_SUPPORTED;
        }
    }
    else
    {
        /* Set Return Code */
        rc =  bb_FN_NOT_SUPPORTED;     
    }
    
    /* Assign the chip's COMMON function pointers */
    if (rc != bb_FN_NOT_SUPPORTED)
    {
        pChipFuncPtrs = 
            ((ChipFunctionPointers_t *) pChipData->funcPtr)->tCommonFuncPtr;
        
        /* If function pointer is valid */
        if (pChipFuncPtrs->WriteFuncPtr)
        {
            /* Call Function Pointer */
            rc = (pChipFuncPtrs->WriteFuncPtr)(pChipData, wordSize,
                address, length, buffer);          
        }
        else
        {
            rc = bb_FN_NOT_SUPPORTED;
        }
    }
    
    return rc;
}
