/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: Tepulkow $
 *   DATE:           $Date: 3/08/04 1:56p $
 *   LAST MODIFIED:  $Modtime: 3/05/04 2:07p $
 *   REV#:           $Revision: 1 $
 *   LOGFILE:        $Logfile: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/os.h $
 *   WORKFILE:       $Workfile: os.h $
 *
 * --------------------------------------------------------------------------
 * Project: ALL
 *
 * Purpose: Provides the specfication for the eiffel alarm, status, & counters
 *          handler routines.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: /IXDP2800_BoardSupport_3.5_FCS_310D/linux/MediaCardDrivers/ixd28192/IXF/os.h $
 * 
 * 1     3/08/04 1:56p Tepulkow
 * 
 * 1     6/27/03 4:44p Daly
 * 
 * 20    3/26/03 9:53a Rsood
 * Made changes for default OS macros OS_CMD_GET_TICKS and  TICKS_PER_SEC
 * 
 * 19    3/25/03 1:36p Bbpillai
 * make change to taskdelay macro for OS not defined section
 * 
 * 18    3/25/03 12:28p Bbpillai
 * add #defs for ticks
 * 
 * 17    2/19/03 8:25a Gcain
 * Updated to map into ICG wide OSAL code.
 * 
 * 16    1/03/03 1:54p Pkjain2
 * 
 * 15    1/03/03 1:43p Pkjain2
 * 
 * 14    12/13/02 1:38p Machambe
 * Adding Task Delay back in
 * 
 * 13    12/12/02 9:49p Machambe
 * Adding Linux support
 *   Revision 1.1  2002/10/30 13:00:02  machambe
 *   Initial CVS checkin
 *
 * 
 * 11    12/03/01 9:57a Jaalliso
 * Added CR/LF to end of file to get rid of GNU C compiler warning
 * 
 * 10    12/01/01 6:34p Machambe
 * 
 * 9     11/06/01 10:41a Machambe
 * Added open, read, write, close, & pipe macros
 * 
 * 8     7/24/01 11:47a Machambe
 * 
 * 7     7/12/01 10:31a Machambe
 * 
 * 6     7/06/01 3:53p Machambe
 * 
 * 5     7/03/01 4:09p Machambe
 * 
 * 4     5/03/01 3:54p Machambe
 * 
 * --------------------------------------------------------------------------
 */

#ifndef __OS_H_
#define __OS_H_

#include "ix_osal.h"			/* Intel standard OSAL */
#include "osalsupport.h"		/* Mapping of os.h to OSAL */


#ifdef VXWORKS


/***************************************************************************
 *               Define OS abstraction layer for VxWorks OS                * 
****************************************************************************/
#define OS_CMD_ALLOC_MEM(size) (IX_Malloc(size,0))
#define OS_CMD_DEALLOC_MEM(ptr) (IX_Free(ptr))
#define OS_CMD_GET_TASK_ID() (taskIdSelf())
#define OS_CMD_ENABLE_PREEMPTION() (taskUnlock())
#define OS_CMD_DISABLE_PREEMPTION() (taskLock())
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) \
		(taskSpawn(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10))

#define OS_CMD_CREATE_SEMAPHORE(opt) (semMCreate(opt))
#define OS_CMD_GIVE_MUTEX_SEMAPHORE(id) (semGive(id))
#define OS_CMD_TAKE_MUTEX_SEMAPHORE(id, timeOut) (semTake(id, timeOut))

#define OS_CMD_ENABLE_INTERRUPTS(irq) (IX_Int_Unlock(irq))
#define OS_CMD_DISABLE_INTERRUPTS() (IX_Int_Lock_VxWrks_Help())
#define OS_CMD_VERIFY_TASK_ID(tId)  (taskIdVerify(tId))
#define OS_CMD_TASK_DELAY(ticks)    (taskDelay(ticks))

#define OS_CMD_GET_TICKS() (tickGet())
#define OS_CMD_INCREMENT_TICKS()    (tickAnnounce()) 
#define TICKS_PER_SEC 60

/* defines */

#define MAX_MSG_Q_MSGS (10)
#define MSG_Q_PRI 1

#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) (msgQCreate (maxMsg, maxMsgLen, priority))
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) (msgQSend (id, msg, size, flag,priority))
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	(msgQReceive(id, msg, size, flag))

#define OS_CMD_TASK_DELETE(tId) (IX_Thread_Stop(tId))

#define OS_CMD_CREATE_PIPE(name, num, size) (pipeDevCreate(name, num, size))
#define OS_CMD_OPEN(name, flag, mode)  (open(name, flag, mode))
#define OS_CMD_READ(fd, buffer, size)  (read(fd, buffer, size))
#define OS_CMD_WRITE(fd, buffer, size) (write(fd, buffer, size))
#define OS_CMD_CLOSE(fd) (close(fd))

/* Global variable for Msg Q's */
extern MSG_Q_ID ixfMsgQId;

#ifdef IXF_PERF_METRIC_MEASUREMENT
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen) (wvEvent(eventID, evBuffer, evBufLen))
#else
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen) 
#endif

#elif IXF_LINUX

#include "os_linux.h"
/***************************************************************************
 *     Define OS abstraction layer for when no embedded OS is defined      * 
****************************************************************************/

#define OS_CMD_ALLOC_MEM(size) (IX_Malloc(size, 0))
#define OS_CMD_DEALLOC_MEM(ptr) (IX_Free(ptr))
#define OS_CMD_GET_TASK_ID() (getpid())
#define OS_CMD_ENABLE_PREEMPTION()
#define OS_CMD_DISABLE_PREEMPTION()
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, args) \
        		(IX_Thread_Spawn_LnxUsrSpc_Help(name, priority, options, \
                                                stackSize, entryPt, args ) )

#define OS_CMD_CREATE_SEMAPHORE(opt) (LinuxSemCreate())
#define OS_CMD_GIVE_MUTEX_SEMAPHORE(id) (LinuxSemDestroy(id))
#define OS_CMD_TAKE_MUTEX_SEMAPHORE(id, timeOut) (LinuxSemTake(id, timeOut))

#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) (mssget())
#define OS_CMD_DELETE_MSG_Q(id, cmd, buf) (msgctl(id, cmd, buf))
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) (msgsnd(id, msg, size, flag))
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	(msgrcv(id,msg,size,0,flag))
#define OS_CMD_VERIFY_TASK_ID(tId)  (IX_Thread_Stop(tId))
#define OS_CMD_TASK_DELETE(tId) (IX_Thread_Stop(tId))

#define OS_CMD_CREATE_PIPE(name, num, size) ()
#define OS_CMD_OPEN(name, flag, mode)  (open(name, flag, mode))
#define OS_CMD_READ(fd, buffer, size)  (read(fd, buffer, size))
#define OS_CMD_WRITE(fd, buffer, size) (write(fd, buffer, size))
#define OS_CMD_CLOSE(fd)               (close(fd))   
#define OS_CMD_TASK_DELAY(ticks)    (sleep(ticks))

#define OS_CMD_GET_TICKS()
#define TICKS_PER_SEC
#define OS_CMD_INCREMENT_TICK()

#ifdef IXF_PERF_METRIC_MEASUREMENT
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen)
#else
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen)
#endif

#else

/***************************************************************************
 *     Define OS abstraction layer for when no embedded OS is defined      * 
****************************************************************************/

#define OS_CMD_ALLOC_MEM(size) (malloc(size))
#define OS_CMD_DEALLOC_MEM(ptr) (free(ptr))
#define OS_CMD_GET_TASK_ID()
#define OS_CMD_ENABLE_PREEMPTION()
#define OS_CMD_DISABLE_PREEMPTION()
#define OS_CMD_FORK_TASK(name,priority, options, stackSize, entryPt, arg1, \
						 arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10) ()

#define OS_CMD_CREATE_SEMAPHORE()
#define OS_CMD_GIVE_MUTEX_SEMAPHORE()
#define OS_CMD_TAKE_MUTEX_SEMAPHORE()

#define OS_CMD_ENABLE_PREEMPTION()
#define OS_CMD_DISABLE_PREEMPTION()

#define OS_CMD_CREATE_MSG_Q(maxMsg, maxMsgLen, priority) ()
#define OS_CMD_SEND_MSG_Q(id,msg,size,flag,priority) ()
#define OS_CMD_RECV_MSG_Q(id,msg,size,flag)	()
#define OS_CMD_VERIFY_TASK_ID(tId)  ()
#define OS_CMD_TASK_DELETE(tId) ()

#define OS_CMD_CREATE_PIPE(name, num, size) ()
#define OS_CMD_OPEN(name, flag, mode)  ()
#define OS_CMD_READ(fd, buffer, size)  ()
#define OS_CMD_WRITE(fd, buffer, size) ()
#define OS_CMD_CLOSE(fd)               ()
#define OS_CMD_TASK_DELAY(ticks)    
#define OS_CMD_GET_TICKS()	0      /* Temporary Value */
#define TICKS_PER_SEC	    0      /* Temporary Value */
#define OS_CMD_INCREMENT_TICK()


#ifdef IXF_PERF_METRIC_MEASUREMENT
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen)
#else
#define OS_CMD_METRIC_EVENT(eventID, evBuffer, evBufLen)
#endif

#endif


#endif /* __OS_H_ */
