/*
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *   AUTHOR:         $Author: lagarwal $
 *   DATE:           $Date: 2003/07/23 17:36:45 $
 *   LAST MODIFIED:  $Modtime: 5/17/02 1:58p $
 *   REV#:           $Revision: 1.4 $
 *   LOGFILE:        $Logfile: /Embedded/source/drivers/ixf/ixf6048/bkcompat.c $
 *   WORKFILE:       $Workfile: bkcompat.c $
 *
 * --------------------------------------------------------------------------
 * Project: IXF6048
 *
 * Purpose: Provides the API for Ixf6048 Specific routines.  This
 *          is meant to glue the new and old api together for 
 *          backward compatibilty.
 *			
 * Notes:
 *			
 * --------------------------------------------------------------------------
 * CompilerFlags: 
 * --------------------------------------------------------------------------
 * --------------------------------------------------------------------------
 *   $Log: ixf6048.c,v $
 *   Revision 1.4  2003/07/23 17:36:45  lagarwal
 *   no message
 *
 *   Revision 1.3  2003/07/22 18:53:57  lagarwal
 *   no message
 *
 *   Revision 1.2  2003/05/01 20:42:51  lagarwal
 *   no message
 *
 *   Revision 1.1.1.1  2002/11/15 17:51:11  lagarwal
 *   rename dir
 *
 *   Revision 1.9  2002/10/27 20:00:11  machambe
 *   no message
 *
 *   Revision 1.10  2002/09/09 21:49:21  cmah1
 *   no message
 *
 *   Revision 1.8  2002/07/23 16:12:58  machambe
 *   no message
 * --------------------------------------------------------------------------
 */ 

#ifdef VXWORKS
#include "vxWorks.h"
#endif

/* #include <ansi_c.h> */       /* C Standard Library interface */
#include <string.h>             /* C Standard String Operations Library Interface */

/* --------------------------	--------------------------	
	Header Files To include for Project
   --------------------------	--------------------------	*/
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h" /* BroadBand Utility Definitions and Function headers */
#include "ixf_api_sonet_d.h" /* Include Sonet data definitions */
#include "ixf6048b.h" /* Include additional Amazon-A Bit definitions */
#include "ixf6048d.h" /* Include additional Amazon-A data definitions */
#include "ixf6048f.h" /* Function Headers, Build Flags and Project Includes*/
#include "hal.h"


#if 0
#define SetBit(Reg, Bit) ((Reg) = (Bit | Reg))
#define ClrBit(Reg, Bit) ((Reg) = (~(Bit) & Reg))
#endif

#define Ixf6048SetBitVal(Reg, Bit) ((Reg) = (Bit | Reg))
#define Ixf6048ClrBitVal(Reg, Bit) ((Reg) = (~(Bit) & Reg))

#define Ixf6048SetBit(RegAddr, Bit) (WRITE_TO_HW((bb_RegPointer_type)RegAddr, (Bit | READ_FROM_HW((bb_RegPointer_type)RegAddr))))
#define Ixf6048ClrBit(RegAddr, Bit) (WRITE_TO_HW((bb_RegPointer_type)RegAddr, (~(Bit) & READ_FROM_HW((bb_RegPointer_type)RegAddr))))

/***********************************************************************
 * Static data: C runtime inits to 0
 **********************************************************************/
/* Pointer to the Alarm Callback Function */
static ixf6048_pAlarmCallback   pAlarmCallback = NULL;

static boolean AlarmsUsed = FALSE;  /* Only Enable Alarms if this is TRUE */

static ushort _TxUtopiaAlarms;     /* Used for Utopia Tx Interrupt Register*/
static ushort _RxUtopiaAlarms;     /* Used for Utopia Rx Interrupt Register*/
static ushort _PRBSAlarms;         /* Used for PRBS Interrupt Register*/
static ushort _TxAlarmBusAlarms;   /* Used for TxAlarmBus Interrupt Register*/
static ushort _LineSideParityAlarm;/* Used for LSP Interrupt Register*/

/* Define the Channels Available */
#define ixf6048_CHANEL_0    0
#define ixf6048_CHANEL_1    1
#define ixf6048_CHANEL_2    2
#define ixf6048_CHANEL_3    3

#define IXF6048_VALIDATE_CHANNEL(chan) ((chan > 3) ? FALSE : TRUE)

/*QMode related Constants */
#define SINGLE_CHANNEL_MODE 1
#define QUAD_CHANNEL_MODE   4

const int IXF6048_CHANNEL_MASK_ALARM_ARRAY[MAX_NUM_CHANNELS] = 
{
    ixf6048_mCHAN0_ALARM,  /* 0x1111 Mask for channel 0 alarms */ 
    ixf6048_mCHAN1_ALARM,  /* 0x2222 Mask for channel 1 alarms */
    ixf6048_mCHAN2_ALARM,  /* 0x4444 Mask for channel 2 alarms */
    ixf6048_mCHAN3_ALARM   /* 0x8888 Mask for channel 3 alarms */
};

/* Used to Convert the RcvJn_Cfg bits to bb_TraceFormat_e */
const bb_TraceFormat_e IXF6048_RCV_TRACE_FORMAT_ARR[] = 
{
   bb_1_BYTE,                 /* 00x 1 byte */
   bb_1_BYTE,                 /* 00x 1 byte */
   bb_IGNORE_RX_TRACE,        /* 01X Ignore trace on Rx */
   bb_IGNORE_RX_TRACE,        /* 01X Ignore trace on Rx */
   bb_16_BYTE_WITH_CRC7,      /* 10X 16 bytes, starting with CRC7 (SDH) */
   bb_16_BYTE_WITH_CRC7,      /* 10X 16 bytes, starting with CRC7 (SDH) */
   bb_64_BYTE_FREE_FORM,      /* 110 64 bytes, free form */
   bb_64_BYTE_WITH_LF_CR,     /* 111 64 bytes with LF,CR at end */ 
};

/* Used to Convert the XmtJn_Cfg bits to bb_TraceFormat_e */
const bb_TraceFormat_e IXF6048_TX_TRACE_FORMAT_ARR[] = 
{
   bb_1_BYTE,                   /* 00 1 byte */
   bb_DEFAULT_TX_TRACE,         /* 01 1 byte (0x01) */
   bb_16_BYTE_WITH_CRC7,        /* 10 Ignore trace on Rx */
   bb_64_BYTE_FREE_FORM         /* 11 Ignore trace on Rx */
};

/* ---------------------------------------------------------------------
   Used to Convert ixf6048_LcdFilter_e members to Actual LCD Filter values
      -- Note that the Value is configured as 2 16 byte values to correspond 
         to the 2 registers that make up the LCD filter */
#define NUM_OF_TIME_PERIODS   4    /* 1ms, 2ms, 3ms, 4ms */
#define NUM_OF_OC_TYPES       4    /* OC-1, OC-3, OC-12, OC-48 */
#define NUM_REGISTERS         2    /* Register (1cc)23H and (1cc)22H */

      /* Corresponding Array Offsets */
#define LCD_FILTER_MSB_OFFSET  0
#define LCD_FILTER_LSB_OFFSET  1

#define OC1_INTERFACE   0
#define OC3_INTERFACE   1
#define OC12_INTERFACE  2
#define OC48_INTERFACE  3

const ushort LCD_FILTER_CONFIGURATION_TABLE[NUM_OF_OC_TYPES][NUM_OF_TIME_PERIODS][NUM_REGISTERS] =
{/*  MSB LSB    MSB LSB    MSB LSB    MSB LSB */
   { {0,0x0654},{0,0x0CA8},{0,0x12FC},{0,0x1950} },    /* OC 1 */
   { {0,0x12FC},{0,0x25F8},{0,0x38F4},{0,0x4BF0} },    /* OC 3 */
   { {0,0x4BF0},{0,0x97E0},{0,0xE3D0},{1,0x2FC0} },    /* OC 12*/
   { {1,0x2FC0},{2,0x5F81},{4,0x8F41},{4,0xBF02} }     /* OC 48*/
};

/* --------------- Miscelaneous Constants ------------------- */
/* Used for HDLC Flow Control */
#define HDLC_FLOW_CONTROL_SCALING_FACTOR_256    256

/***********************************************************************
 *
 * Writable area reserved for Driver
 *
 **********************************************************************/

typedef enum
{
    eINT_EFFECT_NONE,
        eINT_EFFECT_MINOR,
        eINT_EFFECT_MAJOR,
        eINT_EFFECT_CATASTROPHIC
} ActiveInterruptEffect_e;

/***********************************************************************
 *
 * Static Function Prototypes -- Private Methods
 *
 **********************************************************************/
/* ------------------ Initialization ------------------ */
#if 0
static bb_Error_e InitChanCfg(ushort*             BaseAddress,
    ixf6048_Side_e      Side,
    ixf6048_ChipCfg_t*  pChipCfg);

/* ------------------ Edit Chip Level Configuration ------------------ */
static bb_Error_e EditChipLevelCfg(ushort*      		   BaseAddress,
    ixf6048_Side_e 		   Side,
    ixf6048_ChipLevelCfg_t* pCfg);

static bb_Error_e EditUtopiaChipCfg(ushort*                  BaseAddress,
    ixf6048_Side_e       	 Side,
    ixf6048_UtopiaChipCfg_t* pCfg);

static bb_Error_e EditPosChipCfg(ushort*               BaseAddress,
    ixf6048_Side_e        Side,
    ixf6048_PosChipCfg_t* pPosChipfg);
#endif
static void SetPosWatermarks( ushort*                  BaseAddress,
    uint                     ChanNum, 
    ixf6048_Side_e           Side,
    ixf6048_PosWatermarks_t* pPosWatermarks);


/* ------------------ Edit the channel configuration  ------------------ */
static bb_Error_e EditChanCfg (ushort*            BaseAddress,
    int                ChanNum,
    ixf6048_Side_e     Side,
    ixf6048_ChanCfg_t* pCfg);

static bb_Error_e EditChanIfCfg(ushort*              BaseAddress,
    uint                 ChanNum,
    ixf6048_Side_e       Side ,
    ixf6048_ChanMode_e   ChanMode,
    ixf6048_ChanIfCfg_t* pCfg);

static bb_Error_e EditOhtChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_OhtChanCfg_t* pCfg);

static bb_Error_e EditAtmChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_AtmChanCfg_t* pCfg);

static bb_Error_e EditPosChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_PosChanCfg_t* pCfg);

/* --------------------- Edit Alarm Configuration ---------------------- */
      /* Enables/Disables for Channel specific Alarms */
static bb_Error_e EditAlarms (ushort*             BaseAddress,
    ixf6048_Side_e      Side,
    ixf6048_ChipCfg_t*  pChipCfg,
    ixf6048_AlarmCfg_t* pAlarmCfg);

bb_Error_e  EditOhtChanAlarms(ushort*                    BaseAddress, 
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_OhtChanAlarmCfg_t* pOhtChanAlms);

bb_Error_e  EditUtopiaChanAlarms(ushort*                   BaseAddress, 
    uint                      ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_UtopiaAlarmsCfg_t* pUtopiaChanAlms);

bb_Error_e  EditAtmChanAlarms(ushort*                    BaseAddress, 
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_AtmChanAlarmCfg_t* pAtmChanAlms);

bb_Error_e  EditPosChanAlarms(ushort*                    BaseAddress, 
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_PosChanAlarmCfg_t* pPosChanAlms);

bb_Error_e  EditMiscChanAlarms(ushort*                 BaseAddress, 
    uint                     ChanNum, 
    ixf6048_Side_e           Side,
    ixf6048_MiscAlarmsCfg_t* pMiscChanAlms);


/* ------------------ Alarm Handler Functions ------------------ */
static void MasterInterruptHandler( ushort*           BaseAddress,  
    ixf6048_GlobalAlarmCfg_t *pGlobAlmCfg);

static void SdhInterruptHandler(ushort*  BaseAddress,   /* Base address of chip */
    ushort ChanNum,
    ushort GlobalAlmChan,
    ActiveInterruptEffect_e* EffectingInterrupt);

static void AtmInterruptHandler(ushort*  BaseAddress,   /* Base address of chip */
    ushort ChanNum,
    ushort GlobalAlmChan,
    ActiveInterruptEffect_e* EffectingInterrupt);

static void PosInterruptHandler(ushort*  BaseAddress,   /* Base address of chip */
    ushort ChanNum,
    ushort GlobalAlmChan,
    ActiveInterruptEffect_e* EffectingInterrupt);

static void UtopiaInterruptHandler(ushort*  BaseAddress,   /* Base address of chip */
    ushort   ChanNum,
    ushort   GlobalAlm,
    boolean* IntsRead,	  /* Enforce a Single Read */
    ActiveInterruptEffect_e* EffectingInterrupt);

static void MiscInterruptHandler(ushort*   BaseAddress,   /* Base address of chip */
    ushort   ChanNum,
    ushort   GlobalAlm,
    boolean* IntsRead,		/* Enforce a Single Read*/
    ActiveInterruptEffect_e* EffectingInterrupt);


/* ---------- Functions Related to reading/writing Trace Bytes ------------------  */
static bb_Error_e SetRcvJnPathTrace(ushort*           pReg,
    ushort*           pJnCfgMask,
    bb_TraceFormat_e  TraceFormat,
    char*             pTrace);

static bb_Error_e SetTxJnPathTrace(ushort*          pReg,
    ushort*          pJnCfgMask,
    bb_TraceFormat_e TraceFormat,
    char*            pTrace);

static boolean CfgOhtJnString(ushort*  pReg,
    char*   pWorkString, 
    int      StringLen);

static bb_Error_e GetRcvJnPathTrace(ushort*         pReg,
    ushort          pJnCfgMask,
    char*           pTrace, ushort* pLength);

static bb_Error_e GetTxJnPathTrace(ushort*         pReg,
    ushort          pJnCfgMask,
    char*          pTrace,         ushort*         pLength);

static boolean GetOhtJnString(ushort*  pReg,
    char*    pString, 
    int      StringLen);

/* ------------------ Functions Relating to Counters ------------------ */
static void GetRstCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             CntrsBuffFlag,
    bb_RxRstCounters_t* pRxRst);

static void GetMstCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             CntrsBuffFlag,
    bb_RxMstCounters_t* pRxMst);

static void GetHptCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             CntrsBuffFlag,
    bb_RxHptCounters_t* pRxHpt);

static void GetMsaCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             CntrsBuffFlag,
    bb_AuPtrCounters_t* pRxMsa);

static void GetAtmCounters(ushort*               BaseAddress,
    uint                  ChanNum,    
    ixf6048_AtmCounters_t* pAtm);

static void GetPosCounters(ushort*               BaseAddress,
    uint                  ChanNum,    
    ixf6048_PosCounters_t* pPos);

/* ------------------ Test Specific Functions ------------------  */
#if 0
static void TestChanSdh(ushort*                BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg);
static void TestChanPtrs(ushort*                BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg);
static void TestChanAtm(ushort*                BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg);
static void TestChanPos(ushort*                BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg);
static bb_Error_e TestChanLoopbacks(ushort*    BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg);
#endif
/* ------------------ Oht Configuration Functions  ------------------ */
      /* RECEIVE Side OHT Configuration */
static bb_Error_e EditRxRstCfg(ushort*              BaseAddress, 
    uint                 ChanNum,
    ixf6048_RxRstCfg_t* pRstCfg);

static bb_Error_e EditRxMstCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_RxMstCfg_t* pMstCfg);

static bb_Error_e EditRxMsaCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_RxMsaCfg_t* pMsaCfg);

static bb_Error_e EditRxHptCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_RxHptCfg_t* pHptCfg);

      /* TRANSMIT Side OHT Configuration */
static bb_Error_e EditTxRstCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxRstCfg_t* pRstCfg);

static bb_Error_e EditTxMstCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxMstCfg_t* pMstCfg);

static bb_Error_e EditTxMsaCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxMsaCfg_t* pMsaCfg);

static bb_Error_e EditTxHptCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxHptCfg_t* pHptCfg);


/* ------------------ Local Utility Functions ---------------------*/
void SetRegBits(ushort *RegValue, ushort BitValue, 
    ushort BitMask, ushort Shift);
unsigned long int ReadCounterLong(ushort* BaseAddress,
    ushort CounterMSBAddress,
    ushort CounterLSBAddress,
    uint  bCountersBufferedFlag);

unsigned long int ReadCounterShort( ushort* BaseAddress,
    ushort CounterAddress,
    uint  bCountersBufferedFlag);

/* ------------------ Utility Functions ---------------------*/
/***********************************************************************
 * Procedure Name:   SetRegBits
 *
 * Description: Set Register Bit Value using a Shift and a Mask
 *
 * Conditions for Use:
 *    		Local Method Only
 * Notes:
 *    
 *
 * End
 **********************************************************************/
void SetRegBits(ushort *RegValue, ushort BitValue, 
    ushort BitMask, ushort Shift)
{
    ushort value;

    value = READ_FROM_HW(RegValue);
    WRITE_TO_HW(RegValue, (value & ~BitMask));

    value = READ_FROM_HW(RegValue);
    WRITE_TO_HW(RegValue, (value | ((BitValue << Shift) & BitMask)));

#ifdef TAKEMEOUT
    *RegValue &= ~BitMask;
    *RegValue |= (BitValue << Shift) & BitMask;
#endif
}

/***********************************************************************
 * Procedure Name:   ReadCounterLong
 *
 * Description: Returns the Value contained in 2 Registers
 *
 * Conditions for Use:
 *    		Local Method Only
 * Notes:
 *    
 *
 * End
 **********************************************************************/
unsigned long int ReadCounterLong(ushort* BaseAddress,
    ushort CounterMSBAddress, 
    ushort CounterLSBAddress,
    uint   bCntrsBuffered)
{
    unsigned long int RegValueMSB = 0;
    unsigned long int RegValueLSB = 0;
    unsigned long int RegValue;

    if(!bCntrsBuffered)
        WRITE_TO_HW((BaseAddress + CounterMSBAddress), 0);

    RegValueMSB = READ_FROM_HW(BaseAddress + CounterMSBAddress);
    RegValueLSB = READ_FROM_HW(BaseAddress + CounterLSBAddress);
    RegValue = (RegValueMSB << 16) + RegValueLSB;
    return (RegValue);
}
/***********************************************************************
 * Procedure Name:   ReadCounterShort
 *
 * Description: Returns the Value contained in a Registers
 *
 * Conditions for Use:
 *    		Local Method Only
 * Notes:
 *    
 *
 * End
 **********************************************************************/
unsigned long int ReadCounterShort( ushort* BaseAddress, 
    ushort CounterAddress,
    uint   bCntrsBuffered)
{
    ushort RegValue;

    if(!bCntrsBuffered)
        WRITE_TO_HW((BaseAddress + CounterAddress), 0);

    RegValue = READ_FROM_HW(BaseAddress + CounterAddress);
    return ((ulong)RegValue);
}

/***********************************************************************
 * Procedure Name:   ixf6048_InitpAlarmCallback
 *
 * Description: Assign a pointer to an Alarm Callback Method.
 *
 * Conditions for Use:
 *    Called by ixf6048_InitpAlarmCallback
 * Notes:
 *    Assign A pointer to user defined the Alarm Callback method.
 *
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_InitAlarmCallback (ixf6048_pAlarmCallback  pAlarmCallbackArg)

{
    /* Assign the Alarm Callback Function Pointer */
    pAlarmCallback = pAlarmCallbackArg;

    return bb_NO_ERROR;
}

/***********************************************************************
 * Procedure Name: ixf6048_InitChip
 *
 * Description:Initialize Chip Data
 *
 * Conditions for Use:
 *    none
 *
 * Note:
 *    ChipData is validated by bb_InitChip:
 *       pChipData Is Non Null
 *       ChipType = 6048
 *       pChipCfg is non-NULL
 *       pAlarmCfg is non-NULL, if Polled or Interrupt-driver Alarms used
 *    bb_InitDriver - has completed OK
 *
 *    Common config is validated each time - to be sure.
 *    If it has changed - new values used (whether consistent or not)
 *
 * End
 **********************************************************************/
#if 0
bb_Error_e ixf6048_InitChip (bb_ChipData_t *pChipData)
{
    bb_Error_e               rc = bb_NO_ERROR;
    ixf6048_ChipCfg_t*       pChipCfg= NULL;      /* -> OHT Chip cfg */
    ixf6048_ChipLevelCfg_t*  pChipLevelCfg=NULL;  /* -> Common cfg */
    ixf6048_AlarmCfg_t*      pAlarmCfg= NULL;     /* -> All alarm cfg */
    register ushort*         BaseAddress;

    /* Convert each void* into valid, typed struct ptr */
    pChipCfg    = (ixf6048_ChipCfg_t*)(pChipData->pChipCfg);
    pAlarmCfg   = pChipData->pAlarmCfg;
    BaseAddress = pChipData->BaseAddress;

    if (pChipCfg->pChipLevelCfg)
        pChipLevelCfg = pChipCfg->pChipLevelCfg;
    else
    {
        return bb_NO_CHIP_LEVEL_CFG;
    }

    /* Ensure that the chip is Off-Line Prior to Changing any Global Settings */
    ixf6048_SetChipOffline(pChipData);

    /* Set/Clear the QMode Bit (Quad or Single Channel Mode) */
    if(pChipCfg->NumChanCfg == ixf6048_eSINGLE_CHAN)
    {
        Ixf6048ClrBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bQUAD_MODE);
    }
    else
    {
        /* Must be Quad Channel Mode */
        Ixf6048SetBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bQUAD_MODE);
    }

    /* Edit the Chip Level Configuration */
    rc = EditChipLevelCfg(BaseAddress, ixf6048_eSIDE_BOTH, pChipLevelCfg);

    if (rc != bb_NO_ERROR)
        return rc;

    /* Now Set the data for the specific Channel information.
    All structures are assumed to have been validated */
    rc = InitChanCfg(BaseAddress, ixf6048_eSIDE_BOTH, pChipCfg);

    if (rc != bb_NO_ERROR)
        return rc;

    /* Edit alarms & create MaskedGlobalIntSources, if Alarms used */
    if (pAlarmCfg) /* If alarms used */
    {
        AlarmsUsed = TRUE;
        rc = EditAlarms(BaseAddress, ixf6048_eSIDE_BOTH, pChipCfg, pAlarmCfg);
        if (rc != bb_NO_ERROR)
            return rc;
    }

    /* Set The chip Online */
    ixf6048_SetChipOnline(pChipData);  /* Set OHT online */

#ifdef UNIT_TEST_6048
    printf("Appl Online => SONET/SDH at 0x%x set Online\n",
        pChipData->BaseAddress);
#endif


    return bb_NO_ERROR;

}
/***********************************************************************
 * Procedure Name:   EditChipLevelCfg
 *
 * Description:      Edit Chip Level config
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *    This Function will write all of the Chip Level Config data
 *    to the Chip. 
 *    Assumes that the passed pointers have been validated prior to this call.
 *    Assumes that all data has been validated prior to calling this function.
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditChipLevelCfg(ushort*        BaseAddress,
    ixf6048_Side_e  Side,
    ixf6048_ChipLevelCfg_t* pCfg)
{
    bb_Error_e              rc           = bb_NO_ERROR;
    register ushort*        pReg;
    register ushort         RegVal =0;
    register ushort         macroRegVal =0;

    /* PSN_TODO -- Get a copy of the Release Draft of the Spec Sheet and
    -- Check for Changes in the Global Config. */

    /* Configure the Byte Change updates --Used primarily for Testing */
    /* PSN_ADDED B1 Release Changes Note RegVal Changes */
    if (pCfg->ByteChgUpdDsbl == ixf6048_eBYTE_CHG_UPDATE_ENABLE)
        Ixf6048SetBit((BaseAddress + ixf6048_MACNF_OFFSET), ixf6048_bBYTE_CHG_UPD_DSBL);
    else
        Ixf6048ClrBit((BaseAddress + ixf6048_MACNF_OFFSET), ixf6048_bBYTE_CHG_UPD_DSBL);

    /* I/O Pin Config and PECL MSB config */
    /* Clear Related bits in Reg Val */
    Ixf6048ClrBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bGEN_IO_VAL |
        ixf6048_bGEN_IO_MODE |
        ixf6048_bXMT_PECL_MSB_CNF|
        ixf6048_bRCV_PECL_MSB_CNF);
    /* Set the GENIO Input/output Config bits */
    if (pCfg->GenIOVal == ixf6048_eGEN_IO_VAL_HIGH)
        Ixf6048SetBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bGEN_IO_VAL);
    if (pCfg->GenIOMode == ixf6048_eGEN_IO_MODE_OUTPUT)
        Ixf6048SetBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bGEN_IO_MODE);
    /* Set the PECL MSB Bit order for Rx and Tx */
    if (pCfg->XmtPeclMsbCnf == ixf6048_ePECL_MSB_CNF_BIT_PN_0)
        Ixf6048SetBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bXMT_PECL_MSB_CNF);
    if (pCfg->RcvPeclMsbCnf == ixf6048_ePECL_MSB_CNF_BIT_PN_0)
        Ixf6048SetBit((BaseAddress + ixf6048_GOCNF_OFFSET), ixf6048_bRCV_PECL_MSB_CNF);

    /* PSN_END B1 Release Changes */

    rc = EditUtopiaChipCfg(BaseAddress, Side, &pCfg->UtopiaChipCfg);

    /* Set the Orderwire Configuration bits */
    RegVal = 0;

    if(pCfg->OrderWirePulseByteLocn == ixf6048_eMSB_SAME_CLK_CYCLE_AS_PULSE)
        Ixf6048SetBitVal(RegVal,ixf6048_bOW_PlS_CNF);

    if(pCfg->OverheadPortMode == ixf6048_eOH_PORT_LOGICAL_INTERFACE_NUM3)
        Ixf6048SetBitVal(RegVal, ixf6048_bOH_PORT_MODE);

    /* Set the bits to the Register */
    WRITE_TO_HW((BaseAddress + ixf6048_OHPCNF_OFFSET), RegVal);

    /* Set the Rx and Tx Frame Pulse Bytes */
    pReg = BaseAddress + ixf6048_R_FPCNF_OFFSET;
    WRITE_TO_HW(pReg, (pCfg->RxFramePulseByteNum & ixf6048_mRCV_FPI_CNF));

    RegVal = 0;
    pReg = BaseAddress + ixf6048_T_FPCNF_OFFSET;
    macroRegVal = READ_FROM_HW(pReg);
    SET_XMT_FRAME_PULSE_CNF(macroRegVal, pCfg->TxFramePulseOutByteNum, 
        pCfg->TxFramePulseInByteNum);

    /* Set the Clock Polarity bits to the Registers */
    pReg = BaseAddress + ixf6048_OCPCNF_OFFSET;
    WRITE_TO_HW(pReg, (pCfg->OutClockPolarity & ixf6048_mOCPCNF));

    pReg = BaseAddress + ixf6048_ICPCNF1_OFFSET;
    WRITE_TO_HW(pReg, (pCfg->InClockPolarity1));

    pReg = BaseAddress + ixf6048_ICPCNF2_OFFSET;
    WRITE_TO_HW(pReg, (pCfg->InClockPolarity2 & ixf6048_OCPCNF2_MASK));

    /* Build the Register Setting for the Non Concat Config */
    /* Set the Non-Concatenation Mode Configuration  -- Set Regardless of Concat mode */
    pReg = BaseAddress + ixf6048_NCMODECNF_OFFSET;
    RegVal = 0;
    if (pCfg->NonconcatModeCnfg.J0MuxConfig == ixf6048_eC0_BYTES_OVER_WRITTEN)
        RegVal |= ixf6048_bJ0_MUX_CNF;

    if (pCfg->NonconcatModeCnfg.BWMuxConfig == ixf6048_eBYTE_INTERLEAVED)
        RegVal |= ixf6048_bB_W_MUX_CNF;

    if (pCfg->NonconcatModeCnfg.BWDeMuxConfig == ixf6048_eBYTE_INTERLEAVED)
        RegVal |= ixf6048_bB_W_DMX_CNF;

    /* Set the new Cnfg Value to the Register */
    WRITE_TO_HW(pReg, RegVal);

    /* Set the Line Side Parity Configuration */
    RegVal = 0;
    if(pCfg->LineSideParityConfig.PrtyModeConfig == ixf6048_ePARITY_PROTECTS_PARALLEL_DATA_BUS_AND_FRAME_PULSE) 
        Ixf6048SetBitVal(RegVal, ixf6048_bPRTY_MODE_CNF);
    if(pCfg->LineSideParityConfig.RxParityConfig == ixf6048_eEVEN_PARITY)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_PRTY_CNF);
    if(pCfg->LineSideParityConfig.TxParityConfig == ixf6048_eEVEN_PARITY)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_PRTY_CNF);
    /* Now Set the New Config Values to the LineSideParity Register */
    pReg = BaseAddress + ixf6048_LSPCNF_OFFSET;
    WRITE_TO_HW(pReg, RegVal);

    /* Always Set the Pos Chip Configuration. 
    -- if No POS mode being used then these should be set to Defaults (0) */
    rc = EditPosChipCfg( BaseAddress, Side, &pCfg->PosChipCfg);

#ifdef UNIT_TEST_6048
    printf("Edit of Chip Level Cfg at 0x%x - PASSED OK\n", pCfg);
#endif

    return rc;       /* should be bb_NO_ERROR */
} /* End EditChipLevelCfg */

/***********************************************************************
 * Procedure Name: EditUtopiaChipCfg  
 *
 * Description: This function sets the Configuration for the Chip Level
 *              Utopia interface.
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *    The user must ensure that the  ixf6048_UtopiaCfg_t contains 
 *    valid data.
 *    This application only uses "Side" Param ixf6048_eSIDE_BOTH, 
 *    but it is included for future flexibility
 *    
 *
 * End
 **********************************************************************/
static bb_Error_e EditUtopiaChipCfg(ushort*             BaseAddress,
    ixf6048_Side_e       Side,
    ixf6048_UtopiaChipCfg_t *pCfg)
{
    ixf6048_UtopiaMultiPhy_e MultiDeviceCheck;
    boolean UtopiaWidthIs8Bit = FALSE;
    register ushort* pUicnfReg;
    register ushort* pGocnfReg;

    /* PSN_ADDED -- Note the *pReg Changes */

    pGocnfReg = BaseAddress + ixf6048_GOCNF_OFFSET;
    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        /* Set the Utopia Interface Configuration for Rx */
        pUicnfReg = BaseAddress + ixf6048_R_UICNF_OFFSET;
        switch(pCfg->RxUtopiaInterface)
        {
        case ixf6048_eUTOPIA_L3_64_BIT_USING_TTL_PINS: /* U64Mode = 0 */
        case ixf6048_eUTOPIA_L3_64_BIT_USING_SOH_PINS: /* U64Mode = 1 */
            /* Set the Rcv Utopia data bus widths to 64 Bit */
            /*  and Set the Rcv QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_SINGLE_64BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_SINGLE_64BIT));

            /* Clr/Set the U64Mode Bit in GOCNF Reg */
            if( pCfg->RxUtopiaInterface == ixf6048_eUTOPIA_L3_64_BIT_USING_TTL_PINS)
                Ixf6048ClrBit(pGocnfReg, ixf6048_bU64_MODE);
            else /* Must be ixf6048_eUTOPIA_L3_64_BIT_USING_SOH_PINS */
                Ixf6048SetBit(pGocnfReg, ixf6048_bU64_MODE);

            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;
        case ixf6048_eUTOPIA_L3_32_BIT: /* Level 3 with 32-bit data bus */
            /* Set the Rcv  Utopia data bus widths to 32 Bit */
            /*  and Set the Rcv QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_SINGLE_32BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_SINGLE_32BIT));
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L3_4_X_8_BIT:  /* Level 3 with 4 independent 8-bit buses */
            /* Set the Rcv Utopia data bus widths to 8 Bit */
            /*  and Set the Rcv QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            UtopiaWidthIs8Bit = TRUE;
            /* Always set the Utopia Interface to a Quad Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        case ixf6048_eUTOPIA_L2_16_BIT:     /* Level 2 with 16-bit data bus */
            /* Set the Rcv and Tx Utopia data bus widths to 16 Bit */
            /*  and Set the Rcv/Tx QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /*ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_16BIT);*/
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_16BIT));
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L2_4_X_16_BIT: /* Level 2 with 4 independant 16-bit data buses */
            /* Set the Rcv Utopia data bus widths to 16 Bit */
            /*  and Set the Rcv QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_16BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_16BIT));
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        case ixf6048_eUTOPIA_L2_8_BIT:     /* Level 2 with 8-bit data bus */
            /* Set the Rcv  Utopia data bus widths to 8 Bit */
            /*  and Set the Rcv QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /*ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT);*/
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            UtopiaWidthIs8Bit = TRUE;
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L2_4_X_8_BIT: /* Level 2 with 4 - 8-bit data buses */
            /* Set the Rcv  Utopia data bus widths to 8 Bit */
            /*  and Set the Rcv QuadMode bits to 0 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            UtopiaWidthIs8Bit = TRUE;
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;
        case ixf6048_eUTOPIA_L1_4_X_8_BIT: /* Level 1 with 4 independent 8-bit buses */
            /* Set the Rcv Utopia data bus widths to 8 Bit */
            /*  and Set the Rcv QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            UtopiaWidthIs8Bit = TRUE;
            /* Always set the Utopia Interface to a Quad Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        default:
            return bb_INV_UTOPIA_INTERFACE_USED;
        } /* end Switch -- RxUtopiaInterface */

        if (UtopiaWidthIs8Bit == TRUE)
        {   /* In 8 Bit Mode RcvAtmHEC is used to configure Extra Byte information */
            if(pCfg->RcvAtmHecConfig == ixf6048_eEXTRA_BYTE_IS_STATUS)
                Ixf6048SetBit(pUicnfReg, ixf6048_bRCV_ATM_HEC);
            else
                Ixf6048ClrBit(pUicnfReg, ixf6048_bRCV_ATM_HEC);
        }
        if(pCfg->RxUtopiaSmallMemCfg == ixf6048_eSMALL_MEM_CHAN_0_FIFO_2K)
            Ixf6048SetBit(pUicnfReg, ixf6048_bRCV_SMALL_MEM);
        else
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRCV_SMALL_MEM);

        /* Ensure that the MultiDevice bit is Configured Correctly. */
        if(pCfg->RxUtopiaMultiPhy != MultiDeviceCheck)
        {
            pCfg->RxUtopiaMultiPhy = MultiDeviceCheck;
        }
        /* Configure how RXPFA is driven */
        if (pCfg->RcvMPhyDevCnf == ixf6048_eOUTPUT_ONLY_DRIVEN_WHEN_SELECTED)
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_M_PHY_DEV_CNF);
        else  /* Can only be used if the Interface is NOT shared */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_M_PHY_DEV_CNF);

        /* Set the behaviour of unused pins */
        if(pCfg->UnusedUtopiaPinsImp == ixf6048_eUNUSED_UTOPIA_PINS_HIGH_IMP)
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRCV_TEST_O_EN);
        else /* (pCfg->UnusedUtopiaPinsImp == ixf6048_eUNUSED_UTOPIA_PINS_NOT_HIGH_IMP) */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRCV_TEST_O_EN);

        /* PSN_ADDED for B1 Release */
        /* Set the Behavior for Tristating the Rx UTOPIA interface */
        if(pCfg->RcvUOutEnCnf == ixf6048_eUTOPIA_OUT_EN_CNF_NOT_DISABLED_INDEPENDANTLY)
            Ixf6048SetBit(pGocnfReg, ixf6048_bRCV_U_OUT_EN_CNF);
        else
            Ixf6048ClrBit(pGocnfReg, ixf6048_bRCV_U_OUT_EN_CNF);
        /* PSN_END B1 Release Changes */

    } /* End if( Receive or Both Side )*/

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        /* Set the Utopia Interface Configuration for Rx */
        pUicnfReg = BaseAddress + ixf6048_T_UICNF_OFFSET;
        switch(pCfg->TxUtopiaInterface)
        {

        case ixf6048_eUTOPIA_L3_64_BIT_USING_TTL_PINS: /* U64Mode = 0 */
        case ixf6048_eUTOPIA_L3_64_BIT_USING_SOH_PINS: /* U64Mode = 1 */

            /* Set the Tx Utopia data bus widths to 64 Bit */
            /*  and Set the Rcv/Tx QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_SINGLE_64BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_SINGLE_64BIT));

            /* Clr/Set the U64Mode Bit in GOCNF Reg */
            if( pCfg->TxUtopiaInterface == ixf6048_eUTOPIA_L3_64_BIT_USING_TTL_PINS)
                Ixf6048ClrBit(pGocnfReg, ixf6048_bU64_MODE);
            else /* Must be ixf6048_eUTOPIA_L3_64_BIT_USING_SOH_PINS */
                Ixf6048SetBit(pGocnfReg, ixf6048_bU64_MODE);

            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L3_32_BIT: /* Level 3 with 32-bit data bus */
            /* Set the Tx Utopia data bus widths to 32 Bit */
            /*  and Set the Tx QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_SINGLE_32BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_SINGLE_32BIT));

            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L3_4_X_8_BIT:  /* Level 3 with 4 independent 8-bit buses */
            /* Set the Tx Utopia data bus widths to 8 Bit */
            /*  and Set the Tx QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /*ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT);*/
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));

            /* Always set the Utopia Interface to a Quad Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        case ixf6048_eUTOPIA_L2_16_BIT:     /* Level 2 with 16-bit data bus */
            /* Set the Tx Utopia data bus widths to 16 Bit */
            /*  and Set the Tx QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_16BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_16BIT));

            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;

        case ixf6048_eUTOPIA_L2_4_X_16_BIT: /* Level 2 with 4 independant 16-bit data buses */
            /* Set the Tx Utopia data bus widths to 16 Bit */
            /*  and Set the Tx QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_16BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_16BIT));
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        case ixf6048_eUTOPIA_L2_8_BIT:     /* Level 2 with 8-bit data bus */
            /* Set the Tx Utopia data bus widths to 8 Bit */
            /*  and Set the Tx QuadMode bits to 0 */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eSINGLE_UTOPIA_DEV;
            break;
        case ixf6048_eUTOPIA_L2_4_X_8_BIT: /* Level 2 with 4 - 8-bit data buses */
            /* Set the Rcv  Utopia data bus widths to 8 Bit */
            /*  and Set the Rcv QuadMode bits to 0 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));
            UtopiaWidthIs8Bit = TRUE;
            /* Always set the Utopia Interface to a Single Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        case ixf6048_eUTOPIA_L1_4_X_8_BIT: /* Level 1 with 4 independent 8-bit buses */
            /* Set the Tx Utopia data bus widths to 8 Bit */
            /*  and Set the Tx QuadMode bits to 1 */
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_U_QUAD);
            /* ixf6048_SET_U_WIDTH(*pUicnfReg, ixf6048_U_WIDTH_QUAD_SINGLE_8BIT); */
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) & ~ixf6048_mRX_TX_U_WIDTH));
            WRITE_TO_HW(pUicnfReg, (READ_FROM_HW(pUicnfReg) | ixf6048_U_WIDTH_QUAD_SINGLE_8BIT));

            /* Always set the Utopia Interface to a Quad Device for this mode */
            MultiDeviceCheck = ixf6048_eMULTIPLE_UTOPIA_DEVS;
            break;

        default:
            return bb_INV_UTOPIA_INTERFACE_USED;
        } /* end Switch */

        if(pCfg->TxUtopiaSmallMemCfg == ixf6048_eSMALL_MEM_CHAN_0_FIFO_2K)
            Ixf6048SetBit(pUicnfReg, ixf6048_bXMT_SMALL_MEM);
        else
            Ixf6048ClrBit(pUicnfReg, ixf6048_bXMT_SMALL_MEM);

        /* Ensure that the MultiDevice bit is Configured Correctly. */
        if(pCfg->TxUtopiaMultiPhy != MultiDeviceCheck)
        {
            pCfg->TxUtopiaMultiPhy = MultiDeviceCheck;
        }

        /* Configure how RXPFA is driven */
        if (pCfg->XmtMPhyDevCnf == ixf6048_eOUTPUT_ONLY_DRIVEN_WHEN_SELECTED)
            Ixf6048SetBit(pUicnfReg, ixf6048_bRX_TX_M_PHY_DEV_CNF);
        else  /* Can only be used if the Interface is NOT shared */
            Ixf6048ClrBit(pUicnfReg, ixf6048_bRX_TX_M_PHY_DEV_CNF);

        /* PSN_ADDED for B1 Release */
        /* Set the Behavior for Tristating the Rx UTOPIA interface */
        if(pCfg->XmtUOutEnCnf == ixf6048_eUTOPIA_OUT_EN_CNF_NOT_DISABLED_INDEPENDANTLY)
            Ixf6048SetBit(pGocnfReg, ixf6048_bXMT_U_OUT_EN_CNF);
        else
            Ixf6048ClrBit(pGocnfReg, ixf6048_bXMT_U_OUT_EN_CNF);

        /* PSN_END B1 Release Changes */

    } /* End if( Transmit or Both Side )*/

    /* Always Set the Utopia Base Address regardless of Side*/
    /* Mask Address Bits then write new Address */
    Ixf6048ClrBit(pGocnfReg, ixf6048_mUTOPIA_ADDR_BASE);
    Ixf6048SetBit(pGocnfReg, (pCfg->UtopiaDevBaseAddress & ixf6048_mUTOPIA_ADDR_BASE));

    return bb_NO_ERROR;
}

/***********************************************************************
 * Procedure Name: EditPosChipCfg  
 *
 * Description: This function sets the Configuration for the Utopia
 *              interface when in POS mode. 
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pPosChipCfg is non-NULL
 *
 * Notes:
 *    The bits are Always initialized regardless of whether POS
 *    mode is selected for any of the channels. 
 *    The user must ensure that the  ixf6048_PosChipCfg_t contains 
 *    valid data.
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditPosChipCfg (ushort*               BaseAddress,
    ixf6048_Side_e        Side,
    ixf6048_PosChipCfg_t* pPosChipfg)
{
    bb_Error_e          rc = bb_NO_ERROR;
    register ushort*    pRxReg;
    register ushort*    pTxReg;
    register ushort     RxRegVal = 0;
    register ushort     TxRegVal = 0;

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        /* Point to the Receive UTOPIA Configuration Register */
        pRxReg = BaseAddress +  ixf6048_R_UICNF_OFFSET;
        /* Get the Current Config Values */
        RxRegVal = READ_FROM_HW(pRxReg);

        /* clear all Rx Configuration bits */
        Ixf6048ClrBitVal(RxRegVal, (ixf6048_bRCV_VAL2_CNF |
            ixf6048_bRCV_XMT_SEL_MODE |
            ixf6048_bRX_TX_DIR_STAT_CNF |
            ixf6048_bRCV_VAL_CNF |
            ixf6048_bRCV_FIF_EMPT_EOF));

        /* Configure the Rx Selection Mode Bit 0*/
        if(pPosChipfg->RxPosUtopiaSelMode == ixf6048_ePOS_UTOPIA_MEMORY_MAPPED)
            Ixf6048SetBitVal(RxRegVal,ixf6048_bRCV_XMT_SEL_MODE);
        /* Configure the Rx Direct Status Indication config - Bit5*/
        if (pPosChipfg->RxFaStatus == ixf6048_eFA_MULTIPLEXED_STATUS_POLLING)
            Ixf6048SetBitVal(RxRegVal,ixf6048_bRX_TX_DIR_STAT_CNF);
        /* Configure the Receive Value Output (Rx-Bit7)*/
        if (pPosChipfg->RxValDeassertAtEop == ixf6048_eRXVAL_DEASSERT_ON_FIFO_EMPTY)
            Ixf6048SetBitVal(RxRegVal, ixf6048_bRCV_VAL_CNF);
        /* Configure the Assertion Condition for the Receive FIFO  (Rx-Bit6)*/
        if ( pPosChipfg->RxFaDeassertOnLow == ixf6048_eRXFA_DEASSERT_ON_LOW_IF_NO_EOF)
            Ixf6048SetBitVal(RxRegVal, ixf6048_bRCV_FIF_EMPT_EOF);
        /* PSN_ADDED B1 Release */
        /* Configure the Blocking of the REceive Interface */
        if(pPosChipfg->RxVal2Cnf == ixf6048_eRCV_VAL2_CNF_READ_BLOCKED_IF_EOF_ONLY)
            Ixf6048SetBitVal(RxRegVal, ixf6048_bRCV_VAL2_CNF);
        /* PSN_END B1 Release Changes */

        /* Now write the new Configuration values */
        WRITE_TO_HW(pRxReg, RxRegVal);
        /* Now set the Rx Fifo Restart levels */
        pRxReg  = BaseAddress + ixf6048_R_UIIML_OFFSET;
        WRITE_TO_HW(pRxReg, (pPosChipfg->RxFifoRestartLevel & ixf6048_mR_UIIML));
    }

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        /* Transmit Side Config */
        pTxReg  = BaseAddress +  ixf6048_T_UICNF_OFFSET;
        TxRegVal= READ_FROM_HW(pTxReg);

        /* clear all Tx Configuration bits */
        Ixf6048ClrBitVal(TxRegVal, (ixf6048_bRCV_XMT_SEL_MODE |
            ixf6048_bRX_TX_DIR_STAT_CNF |
            ixf6048_XMT_FIF_EMPT_EOF) );

        /* Configure the Tx Selection Mode Bit 0*/
        if(pPosChipfg->TxPosUtopiaSelMode == ixf6048_ePOS_UTOPIA_MEMORY_MAPPED)
            Ixf6048SetBitVal(TxRegVal, ixf6048_bRCV_XMT_SEL_MODE);

        /* Configure the Tx Direct Status Indication config - Bit5*/
        if (pPosChipfg->TxFaStatus == ixf6048_eFA_MULTIPLEXED_STATUS_POLLING)
            Ixf6048SetBitVal(TxRegVal, ixf6048_bRX_TX_DIR_STAT_CNF);
        /* Configure when the Tx FIFO can start Tx of a new packet (Tx-Bit6)*/
        if ( pPosChipfg->TxEofEmpty == ixf6048_eTX_AT_GT_MIN_LEVEL_ONLY)
            Ixf6048SetBitVal(TxRegVal, ixf6048_XMT_FIF_EMPT_EOF);

        /* Write new configuration values to Registers */
        WRITE_TO_HW(pTxReg, TxRegVal);

        /* Now set the Rx and Tx Fifo Restart levels */
        pTxReg  = BaseAddress + ixf6048_T_UIIML_OFFSET;
        WRITE_TO_HW(pTxReg, (pPosChipfg->TxFifoRestartLevel & ixf6048_mT_UIIML));
    }

#ifdef UNIT_TEST_6048
    printf("Edit of POS Chip Cfg at 0x%x - PASSED OK\n", pPosChipfg);
#endif

    return rc;       /* should be bb_NO_ERROR */
}  /* End EditPosChipCfg */

/***********************************************************************
 * Procedure Name: InitChanCfg
 *
 * Description:Initialize Chip
 *
 * Conditions for Use:
 *    All Data passed to function is Assumed Valid. 
 *
 * Note:
 *    bb_InitDriver - has completed OK
 *
 *
 * End
 **********************************************************************/
static bb_Error_e InitChanCfg(ushort*            BaseAddress,
    ixf6048_Side_e     Side,
    ixf6048_ChipCfg_t* pChipCfg)
{
    bb_Error_e         rc       = bb_NO_ERROR;
    ixf6048_ChanCfg_t* pChanCfg = NULL;      /* -> Specific cfg */
    int                iMaxNumChans = SINGLE_CHANNEL_MODE;
    int                ChanNum;

    /* Check for Qmode to Determine how many channels are assigned. */
    if(pChipCfg->NumChanCfg != ixf6048_eSINGLE_CHAN)
        iMaxNumChans = QUAD_CHANNEL_MODE;

    for (ChanNum=0; ChanNum < iMaxNumChans; ++ChanNum)
    {
        if (pChipCfg->pChanCfg[ChanNum])
        {
            pChanCfg =  pChipCfg->pChanCfg[ChanNum];
            /* Edit the Configuration of Each Assigned Channel Both Rx and Tx*/
            rc =  EditChanCfg(BaseAddress, ChanNum, Side, pChanCfg);
            if (rc != bb_NO_ERROR)
                return rc;
        }
        else if (ChanNum == 0)
        {  /* If Channel 0 fails then there is a Problem so Exit */
            return bb_INVALID_CHAN_CFG;
        }
        else /* Must be Channel 1-3, with No Config so Simply Disable */
        {

        }
    }
    return rc;
}
#endif
/***********************************************************************
 * Procedure Name:   EditChanCfg
 *
 * Description:      Edit Channel configuration
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *    pCfg->ChanMode is the Same Value for both Tx and Rx Sides.
 *      
 *    Definitions of Structures used in this function
 *      ixf6048_ChanIfCfg_t*  pChanIfCfg;                                     
 *      ixf6048_OhtChanCfg_t* pOhtChanCfg;
 *      ixf6048_AtmChanCfg_t* pAtmChanCfg;    NULL if not ATM
 *      ixf6048_PosChanCfg_t* pPosChanCfg;    NULL if not POS
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditChanCfg (ushort*            BaseAddress, 
    int                iChanNum,
    ixf6048_Side_e     Side,
    ixf6048_ChanCfg_t* pCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;
    ushort			 RegVal=0;
    /* Set the ChanMode for this Channel -- Rx and Tx */
    if( (Side == ixf6048_eSIDE_RECEIVE) ||
        (Side == ixf6048_eSIDE_BOTH   ) )
    {
        /* PSN_ADDED for B1 Release  -- NOTE: RegVal Addition*/
        pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(iChanNum);
        RegVal = READ_FROM_HW(pReg);
        Ixf6048ClrBitVal(RegVal, ixf6048_mCHAN_MODE); /* Clear ChanMode bits */
        Ixf6048SetBitVal(RegVal, (pCfg->ChanMode & ixf6048_mCHAN_MODE));
        if(pCfg->TrScrEn == ixf6048_eTR_SCR_EN_SCRAMBLER_ENABLED)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_TRANS_DESCR_EN);
        else
            Ixf6048ClrBitVal(RegVal, ixf6048_bRCV_TRANS_DESCR_EN);

        WRITE_TO_HW(pReg, RegVal);
        /* PSN_END B1 RElease Changes*/
    }
    if( (Side == ixf6048_eSIDE_TRANSMIT) ||
        (Side == ixf6048_eSIDE_BOTH    ) )
    {  /* Tx Side Channel mode */
        /* PSN_ADDED for B1 Release  -- NOTE: RegVal Changes*/
        pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(iChanNum);
        RegVal = READ_FROM_HW(pReg);
        Ixf6048ClrBitVal(RegVal,ixf6048_mCHAN_MODE); /* Clear ChanMode bits */
        Ixf6048SetBitVal(RegVal, (pCfg->ChanMode & ixf6048_mCHAN_MODE));
        if(pCfg->TrScrEn == ixf6048_eTR_SCR_EN_SCRAMBLER_ENABLED)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_TRANS_SCR_EN);
        else
            Ixf6048ClrBitVal(RegVal, ixf6048_bXMT_TRANS_SCR_EN);

        WRITE_TO_HW(pReg, RegVal);
        /* PSN_END B1 Release Changes*/
    }
    /* If In repeater mode, then the payload is passed directly, so 
    the configuration is ignored (below) */
    if(pCfg->RepeaterMode == ixf6048_eCHANNEL_IN_REPEATER_MODE)
        Ixf6048SetBit((BaseAddress + ixf6048_COCNF_CHAN_OFFSET(iChanNum)),
            ixf6048_bCHAN_REPEATER_MODE);
    else /* Must be in Normal Mode */
        Ixf6048ClrBit((BaseAddress + ixf6048_COCNF_CHAN_OFFSET(iChanNum)),
            ixf6048_bCHAN_REPEATER_MODE);

    /* Init the Channel Interface Configuration */
    if (pCfg->pChanIfCfg)
        rc = EditChanIfCfg( BaseAddress, iChanNum,Side, 
            pCfg->ChanMode, pCfg->pChanIfCfg);
    else
    {
        return bb_INV_CHAN_IF_CFG_USED;
    }

    /* Edit the OHT for each Channel */
    if (pCfg->pOhtChanCfg)
        rc = EditOhtChanCfg(BaseAddress, iChanNum, Side, pCfg->pOhtChanCfg);
    else
    {
        return bb_INV_SONET_SDH_CHAN_CFG_USED;
    }
    /* Configure the ATM or POS section if Used and NOT in Repeater Mode */
    if( (pCfg->RepeaterMode != ixf6048_eCHANNEL_IN_REPEATER_MODE ) &&
        (pCfg->ChanMode == ixf6048_eATM_MODE_CHAN) )
    {
        if (pCfg->pAtmChanCfg)
            rc = EditAtmChanCfg(BaseAddress, iChanNum, Side, pCfg->pAtmChanCfg);
        else
        {
            return bb_INV_ATM_CHAN_CFG_USED;
        }

    }
    else if( (pCfg->RepeaterMode != ixf6048_eCHANNEL_IN_REPEATER_MODE ) &&
        (pCfg->ChanMode == ixf6048_ePOS_MODE_CHAN) )
    {
        if (pCfg->pPosChanCfg)
            rc = EditPosChanCfg(BaseAddress, iChanNum, Side, pCfg->pPosChanCfg);
        else
        {
            return bb_INV_POS_CHAN_CFG_USED;
        }
    }

#ifdef UNIT_TEST_6048
    printf("Edit of Channel Cfg at 0x%x - PASSED OK\n", pCfg);
#endif

    return rc;       /* should be bb_NO_ERROR */
}   /* end EditChanCfg */


/***********************************************************************
 * Procedure Name:   EditChanIfCfg
 *
 * Description:      Edit Channel If configuration 
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *    This Sets up the Channel Config that is Common to Rx and Tx Sides
 *
 *
 * PDL:
 *
 * End
 **********************************************************************/
static bb_Error_e EditChanIfCfg(ushort*             BaseAddress,
    uint                ChanNum,
    ixf6048_Side_e      Side ,
    ixf6048_ChanMode_e  ChanMode,
    ixf6048_ChanIfCfg_t *pCfg)
{
    bb_Error_e        rc = bb_NO_ERROR;
    register ushort   RegVal=0;
    volatile register bb_RegPointer_type  pGlobalCnfReg;
    volatile register bb_RegPointer_type  pChanCnfReg;
    volatile register bb_RegPointer_type  pReg;
    register ushort         macroRegVal =0;
    /* Set the Line Speed configuration -- Common to Tx and Rx. */
    if(Side == ixf6048_eSIDE_BOTH)
    {
        pGlobalCnfReg = BaseAddress + ixf6048_GOCNF_OFFSET;
        pChanCnfReg   = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum);

        /* Clear the Channel Rate Bits Prior to Write  */
        Ixf6048ClrBit(pChanCnfReg, ixf6048_mCHAN_RATE);
        switch (pCfg->LineSideSpeed)
        {
        case ixf6048_eSINGLE_STS_48C: /* Must be Concat,Single Mode */
        case ixf6048_eSINGLE_STM_16C:
            Ixf6048SetBit(pGlobalCnfReg, ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC48);
            break;
        case ixf6048_eSINGLE_STS_48:  /* Must be NonConcat, Single Mode */
        case ixf6048_eSINGLE_STM_16:
            Ixf6048ClrBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC48);
            break;
        case ixf6048_eSINGLE_STS_12:  /* Must be NonConcat, Single Mode */
        case ixf6048_eSINGLE_STM_4:
            Ixf6048ClrBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC12);
            break;
        case ixf6048_eSINGLE_STS_12C:   /* Concatenated , Single Mode */
            if(ChanNum == 0)
            {	/* Assign Channel 1 to be the Same as Channel 2 -- See Erata */
                pReg = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(1);
                /* Note: The Following MUST be done for ixf6012 Bug.. Chan(1) Must be the Same as Chan(0) */
                Ixf6048ClrBit(pReg, ixf6048_mCHAN_RATE);
                Ixf6048SetBit(pReg, ixf6048_bCHAN_RATE_OC12);
            }
            Ixf6048SetBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC12);
            break;
        case ixf6048_eSINGLE_STS_3C:   /* Must be NonConcat, Single Mode */
            Ixf6048SetBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC3);
            /* Note: The Following MUST be done for ixf6012 Bug.. Chan(1) Must be the Same as Chan(0) */
            if(ChanNum == 0)
            {	/* Assign Channel 1 to be the Same as Channel 2 -- See Erata */
                pReg = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(1);
                /* Note: The Following MUST be done for ixf6012 Bug.. Chan(1) Must be the Same as Chan(0) */
                Ixf6048ClrBit(pReg, ixf6048_mCHAN_RATE);
                Ixf6048SetBit(pReg, ixf6048_bCHAN_RATE_OC3);
            }
            break;
        case ixf6048_eSINGLE_STS_3:   /* Must be NonConcat, Single Mode */
            Ixf6048ClrBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC3);
            break;
        case ixf6048_eQUAD_STS_12C:   /* Must be Concat, Quad Mode */
        case ixf6048_eQUAD_STM_4C:
            Ixf6048SetBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC12);
            break;
        case ixf6048_eQUAD_STS_3C:    /* Must be Concat,Quad Mode */
        case ixf6048_eQUAD_STM_1:
            Ixf6048SetBit(pGlobalCnfReg,ixf6048_bC_MODE);
            Ixf6048SetBit(pChanCnfReg, ixf6048_bCHAN_RATE_OC3);
            break;
        case ixf6048_eQUAD_STS_1:     /* Must be Concat, Quad Mode */
            Ixf6048SetBit(pGlobalCnfReg,ixf6048_bC_MODE);
            /* Already Set to OC-1 Above */
            break;
        default:
            return bb_INV_LINE_SPEED_USED;
        }
    }

    /* Initialize Each Side -- Rx and Tx individually*/
    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(ChanNum);
        /* Get A copy of the Register to work with */
        RegVal = READ_FROM_HW(pReg);
        WRITE_TO_HW(pReg, 0); /* Clear Register */
        /* Set the RcvLockCnf */
        if(pCfg->RcvLockConfig == ixf6048_eSWITCH_TO_BLUE_CLOCK_ON_ACTIVE_HIGH)
            Ixf6048SetBitVal(RegVal,ixf6048_bRCV_LOCK_CNF);
        else
            Ixf6048ClrBitVal(RegVal,ixf6048_bRCV_LOCK_CNF);
        /* Set the New If Mode */
        /* ixf6048_SET_IF_MODE(RegVal,pCfg->RxLineSideInterface); */

        RegVal =  (uint)(RegVal & ~ixf6048_mIF_MODE    );
        RegVal |= ((pCfg->RxLineSideInterface<< 2)& ixf6048_mIF_MODE);

        /* Set the Clock Divide */
        ixf6048_SET_CO_CNF(RegVal, pCfg->RxTtlClockDivide);
        /* Write the new Value to the Register */
        WRITE_TO_HW(pReg, RegVal);

        /* Set the UTOPIA Configuration Only if POS or ATM Mode */
        /* Set the UTOPIA Configuration Only if POS or ATM Mode */
        if( (ChanMode == ixf6048_eATM_MODE_CHAN) ||
            (ChanMode == ixf6048_ePOS_MODE_CHAN) )
        {
            /* Set the Common Configuration */
            pReg = BaseAddress + ixf6048_R_UICHCNF_CHAN_OFFSET(ChanNum);
            RegVal = READ_FROM_HW(pReg);
            /* Utopia Parity */
            if (pCfg->RxUtopiaParity == ixf6048_eUTOPIA_ODD_PARITY)
                Ixf6048ClrBitVal(RegVal,ixf6048_bPRTY_CNF);
            else
                Ixf6048SetBitVal(RegVal,ixf6048_bPRTY_CNF);

            /* Decode Response Delay */
            if(pCfg->RxUtopiaResponseDelay == ixf6048_eONE_CYCLE_DECODE_RESPONSE_DELAY)
                Ixf6048ClrBitVal(RegVal, ixf6048_bD_R_CNF);
            else
                Ixf6048SetBitVal(RegVal, ixf6048_bD_R_CNF);

            if (pCfg->RxFaPolarity == ixf6048_eTXFA_ACTIVE_HIGH)
                Ixf6048ClrBitVal(RegVal, ixf6048_bF_A_CNF);
            else
                Ixf6048SetBitVal(RegVal, ixf6048_bF_A_CNF);

            /* Set the ATM Specific Configuration */
            if(ChanMode == ixf6048_eATM_MODE_CHAN)
            {  /* RcvBurstCnf Bit */
                if(pCfg->UtopiaAtmChanCfg.RxCellBurstCnf == ixf6048_eCELL_BURST_PASS_THROUGH_MODE)
                    Ixf6048SetBitVal(RegVal, ixf6048_bBURST_CNF);
                else
                    Ixf6048ClrBitVal(RegVal, ixf6048_bBURST_CNF);

                if(pCfg->UtopiaAtmChanCfg.RxDummyHecWord == ixf6048_ePASS_NULL_HEC_WORD)
                    Ixf6048SetBitVal(RegVal, ixf6048_bCELL_STRUCT);
                else
                    Ixf6048ClrBitVal(RegVal, ixf6048_bCELL_STRUCT);
                /* Set the Cell Available Deassertion level */
                ixf6048_SET_C_A_DEASSERT(RegVal, pCfg->UtopiaAtmChanCfg.RxCellAvailDeassertByteNum);
            }
            else /*  (ChanMode == ixf6048_ePOS_MODE_CHAN) */
                SetPosWatermarks(BaseAddress, ChanNum, ixf6048_eSIDE_TRANSMIT, &pCfg->PosWatermarks);

            /* Write the new Value to the R_UICHCNF Register */
            WRITE_TO_HW(pReg, RegVal);
        }
    }

    /* Channel Configuration for the TRANSMIT Side */
    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(ChanNum);
        /* Get A copy of the Register to work with */
        RegVal = READ_FROM_HW(pReg);
        WRITE_TO_HW(pReg, 0); /* Clear the Register */

        /* Set the output PECL Clock Config */
        if (pCfg->XmtPClkOutConfig == ixf6048_eXMT_PECL_CLK_DISABLED)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_PCLK_OUT);
        else   /* Enable the output Clock -- This is An Active Low Bit*/
            Ixf6048ClrBitVal(RegVal, ixf6048_bXMT_PCLK_OUT);
        /* Set the New If Mode */
        /* ixf6048_SET_IF_MODE(RegVal,pCfg->TxLineSideInterface); */
        RegVal =  (uint)(RegVal & ~ixf6048_mIF_MODE);
        RegVal |= ((pCfg->TxLineSideInterface<< 2)& ixf6048_mIF_MODE);
        /* Set the new Clock Divide */
        /* PSN_DEBUG */
        /* ixf6048_SET_CO_CNF(RegVal, pCfg->TxTtlClockDivide); */
        RegVal &= ~ixf6048_mCO_CNF;
        RegVal |= (pCfg->TxTtlClockDivide << 5) & ixf6048_mCO_CNF;
        ixf6048_SET_XMT_TIM_REF_CLK_SRC(RegVal, pCfg->TxTimingSource);
        /* Write the new Value to the Register */
        WRITE_TO_HW(pReg, RegVal);

        /* Set the UTOPIA Configuration Only if POS or ATM Mode */
        if( (ChanMode == ixf6048_eATM_MODE_CHAN) ||
            (ChanMode == ixf6048_ePOS_MODE_CHAN) )
        {
            /* Set the Common Configuration */
            pReg = BaseAddress + ixf6048_T_UICHCNF_CHAN_OFFSET(ChanNum);
            RegVal = READ_FROM_HW(pReg);

            /* Utopia Parity */
            if (pCfg->TxUtopiaParity == ixf6048_eUTOPIA_ODD_PARITY)
                Ixf6048ClrBitVal(RegVal,ixf6048_bPRTY_CNF);
            else
                Ixf6048SetBitVal(RegVal,ixf6048_bPRTY_CNF);
            /* Decode Response Delay */
            if(pCfg->TxUtopiaResponseDelay == ixf6048_eONE_CYCLE_DECODE_RESPONSE_DELAY)
                Ixf6048ClrBitVal(RegVal, ixf6048_bD_R_CNF);
            else
                Ixf6048SetBitVal(RegVal, ixf6048_bD_R_CNF);

            if (pCfg->TxFaPolarity == ixf6048_eTXFA_ACTIVE_HIGH)
                Ixf6048ClrBitVal(RegVal, ixf6048_bF_A_CNF);
            else
                Ixf6048SetBitVal(RegVal, ixf6048_bF_A_CNF);

            /* Set the ATM Specific Configuration */
            if(ChanMode == ixf6048_eATM_MODE_CHAN)
            {  /* XmtBurstCnf bit */
                if(pCfg->UtopiaAtmChanCfg.TxCellBurstCnf == ixf6048_eCELL_BURST_PASS_THROUGH_MODE)
                    Ixf6048SetBitVal(RegVal, ixf6048_bBURST_CNF);
                else
                    Ixf6048ClrBitVal(RegVal, ixf6048_bBURST_CNF);

                if(pCfg->UtopiaAtmChanCfg.TxDummyHecWord == ixf6048_ePASS_NULL_HEC_WORD)
                    Ixf6048SetBitVal(RegVal ,ixf6048_bCELL_STRUCT);
                else
                    Ixf6048ClrBitVal(RegVal ,ixf6048_bCELL_STRUCT);

                /* Set the Cell Available Deassertion level*/
                ixf6048_SET_C_A_DEASSERT(RegVal, pCfg->UtopiaAtmChanCfg.TxCellAvailDeassertByteNum);

                pChanCnfReg = BaseAddress + ixf6048_T_UIFDP_CHAN_OFFSET(ChanNum);
                /* Set the Transmit Fifo Depth */
                macroRegVal = READ_FROM_HW(pChanCnfReg);
                if (ChanNum == 0)
                    ixf6048_SET_XMT_F_D_CNF(macroRegVal, pCfg->UtopiaAtmChanCfg.TxFifoDepth, 
                        ixf6048_mXMT_F_D_CNF_CHAN_0);
                else
                    ixf6048_SET_XMT_F_D_CNF(macroRegVal, pCfg->UtopiaAtmChanCfg.TxFifoDepth, 
                        ixf6048_mXMT_F_D_CNF_CHANS_1_3);
            }
            else /*  (ChanMode == ixf6048_ePOS_MODE_CHAN) */
                SetPosWatermarks(BaseAddress, ChanNum, ixf6048_eSIDE_TRANSMIT, &pCfg->PosWatermarks);
            /* Write the new Value to the T_UICHCNF Register */
            WRITE_TO_HW(pReg, RegVal);
        }
    }

#ifdef UNIT_TEST_6048
    printf("Edit of Channel If Cfg at 0x%x - PASSED OK\n", pCfg);
#endif

    return rc;       /* should be bb_NO_ERROR */

}   /* end EditChanIfCfg */

/***********************************************************************
 * Procedure Name:   SetPosWatermarks
 *
 * Description:      Set the POS Watermark Configuration 
                     -- Called for the Rx, Tx or Both Sides
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 *
 * PDL:
 *
 * End
 **********************************************************************/
static void SetPosWatermarks(ushort*                  BaseAddress,
    uint                     ChanNum, 
    ixf6048_Side_e           Side,
    ixf6048_PosWatermarks_t* pPosWatermarks)
{
    ushort           uWMarkMask;
    register ushort* pReg;
    register ushort         macroRegVal =0;

    if (ChanNum == ixf6048_CHANEL_0)
        /* Set the Watermark Mask  --  Watermarks bits[11:0]*/
        uWMarkMask = ixf6048_mWATERMARK_CHAN_0;
    else /* Channel 1,2,3 so Set Watermark Mask to bits[8:0] */
        uWMarkMask = ixf6048_mWATERMARK_CHAN_1_3;

    /* Set the Receive Watermark Cnfg */
    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        /* Set Receive Programable Watermark */
        pReg = BaseAddress + ixf6048_R_PWM_CHAN_OFFSET(ChanNum);
        macroRegVal = READ_FROM_HW(pReg);
        ixf6048_SET_WATERMARK(macroRegVal, pPosWatermarks->RcvPWM, 
            uWMarkMask);
    }

    /* Set the TRANSMIT Watermark Config */
    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        /* Set the Tx Near Full Programable Watermark */
        pReg = BaseAddress + ixf6048_T_NFPWM_CHAN_OFFSET(ChanNum);
        macroRegVal = READ_FROM_HW(pReg);
        ixf6048_SET_WATERMARK(macroRegVal, pPosWatermarks->XmtNFPWM, 
            uWMarkMask);

        /* Set the Near Empty Programmable Watermark */
        pReg = BaseAddress + ixf6048_T_NEPWM_CHAN_OFFSET(ChanNum);
        macroRegVal = READ_FROM_HW(pReg);
        ixf6048_SET_WATERMARK(macroRegVal, pPosWatermarks->XmtNEPWM, 
            uWMarkMask);
    }
}

/***********************************************************************
 * Procedure Name:   EditOhtChanCfg
 *
 * Description:      Edit Sonet SDH Channel configuration
 *                   
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditOhtChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_OhtChanCfg_t* pCfg)
{
    bb_Error_e rc = bb_NO_ERROR;

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        rc = EditRxRstCfg(BaseAddress, ChanNum, &pCfg->RxRstCfg);
        if(rc == bb_NO_ERROR)
            rc = EditRxMstCfg(BaseAddress, ChanNum,&pCfg->RxMstCfg);
        if(rc == bb_NO_ERROR)
            rc = EditRxMsaCfg(BaseAddress, ChanNum,&pCfg->RxMsaCfg);
        if(rc == bb_NO_ERROR)
            rc = EditRxHptCfg(BaseAddress, ChanNum, &pCfg->RxHptCfg);
    }

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        if(rc == bb_NO_ERROR)
            rc = EditTxRstCfg(BaseAddress, ChanNum, &pCfg->TxRstCfg);
        if(rc == bb_NO_ERROR)
            rc = EditTxMstCfg(BaseAddress, ChanNum, &pCfg->TxMstCfg);
        if(rc == bb_NO_ERROR)
            rc = EditTxMsaCfg(BaseAddress, ChanNum, &pCfg->TxMsaCfg);
        if(rc == bb_NO_ERROR)
            rc = EditTxHptCfg(BaseAddress, ChanNum, &pCfg->TxHptCfg);
    }


#ifdef UNIT_TEST_6048
    printf("Edit of Sonet SDH Chan Cfg at 0x%x - NO VERIFICATION\n", pCfg);
#endif

    return rc;       /* should be bb_NO_ERROR */
}   /* end EditOhtChanCfg */

/***********************************************************************
 * Procedure Name:  EditRxRstCfg 
 *
 * Description:      Edit the RST Oht configuration
 *
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 * End
 **********************************************************************/
static bb_Error_e EditRxRstCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_RxRstCfg_t* pRstCfg)
{
    bb_Error_e        rc = bb_NO_ERROR;
    bb_TraceFormat_e  TraceFormat;
    register ushort*  pReg;
    register ushort   RegVal = 0;
    ushort            JnCfgMask;

    /* Build All bits in the R_RSTC Register */
    if (pRstCfg->RxRstScrmblCnfg== ixf6048_eENABLE_RX_RST_2E7_DESCRAMBLING)/* Bit 15 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_SCRMBL_CNFG);
    if(pRstCfg->RxRstFBaDsbl == ixf6048_eRX_RST_USE_RFPI)/* Bit 14 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_F_BA_DSBL);
    if(pRstCfg->RxRstRcvFwdOffCnfg == ixf6048_eFWD_OFF_CNFG_SHORT_FORMAT) /* Bit 13 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_FWD_OFF_CNFG);
    if(pRstCfg->RxRstRcvFwdOnCnfg == ixf6048_eFWD_OFF_CNFG_SHORT_FORMAT) /* Bit 12 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_FWD_ON_CNFG);
    if(pRstCfg->B1CntrCfg == ixf6048_eCOUNT_B1_BLOCK_ERRORS)/* Bit 11 */
        Ixf6048SetBitVal(RegVal, ixf6048_bCNFG_B1_CNTR);

    ixf6048_SET_RST_RCV_LOS_CNFG(RegVal, pRstCfg->RxRstLos);  /* Bits[10:9] */

    if(pRstCfg->BlueClkOnLosLoc == ixf6048_eENABLE_BLUE_CLOCK_ON_LOS_LOC) /* Bit 8 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRST_CLK_LOCK_EN);
    if(pRstCfg->RxRstAisOnLof == ixf6048_eENABLE_RX_RST_AIS_ON_LOF) /* Bit 7 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRST_AIS_LOF_EN);
    if(pRstCfg->RxRstAisOnLos == ixf6048_eENABLE_RX_RST_AIS_ON_LOS) /* Bit 6 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRST_AIS_LOS_EN);
    if(pRstCfg->RxRstAisOnLoc == ixf6048_eENABLE_RX_RST_AIS_ON_LOC) /* Bit 5 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRST_AIS_LOCK_EN);
    /* Note: Bit 4 is Used for Testing so leave Clear */
    if(pRstCfg->RxRstAisOnJ0 == ixf6048_eENABLE_RX_RST_AIS_ON_J0) /* Bit 3 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRST_AIS_TIM_EN);

    ixf6048_SET_LOCK_ITG(RegVal, pRstCfg->RxRstLocFiltering);   /* Bits[2:1]  */

    if(pRstCfg->NdfFrameAcqCfg == ixf6048_eSTM0_NDF_ROBUST_FRAME_ACQUISITION)/* Bit 0 */
        Ixf6048SetBitVal(RegVal, ixf6048_bCNFG_FRM_ACQ);

    /* Set the New Config Values to  R_RSTC register */
    pReg   = BaseAddress + ixf6048_R_RSTC_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Clear the Register value and Set for LOF_LMN Register bits*/
    RegVal = 0;
    if (pRstCfg->RcvOofCnfg == ixf6048_eOOF_CNFG_5_FRAMES)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_OOF_CNFG); /* Reg LOF_LMN Bit 15 */

    ixf6048_STORE_L(RegVal,pRstCfg->LofStateMachineParms.L);   /* Reg LOF_LMN */
    ixf6048_STORE_M(RegVal,pRstCfg->LofStateMachineParms.M);   /* Reg LOF_LMN */
    ixf6048_STORE_N(RegVal,pRstCfg->LofStateMachineParms.N);   /* Reg LOF_LMN *//* Set the New Config Values to LOF_LMN*/
    /* Write to the LMN Register for this channel */
    pReg   = BaseAddress + ixf6048_LOF_LMN_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Clear the Reg Value and Set for J0_RSTC Configuration Register */
    RegVal = 0;
    if(pRstCfg->J0Crc7Mismatch == ixf6048_eJ0_CRC7_MASKS_MISMATCH)/* Reg J0_RSTC Bit 5 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRS_TIM_ON_CRC7_ERR_DSB);
    if(pRstCfg->J0UnstableMismatch ==ixf6048_eJ0_UNSTABLE_SETS_MISMATCH )  /* Reg J0_RSTC Bit 4 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRS_TIM_ON_UNSTABLE_EN);
    if(pRstCfg->J0StableNum == ixf6048_eJ0_STABLE_AFTER_5) /* Reg J0_RSTC Bit 3 */
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_J0_STABLE_CNFG);

    ixf6048_SET_RCV_J0_CNF_FORMAT(RegVal, pRstCfg->ExpectedJ0Format); /* Reg J0_RSTC Bits[2:0] */

    /* Set the New Config Values to J0_RSTC*/
    pReg   = BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Now Set the Expected J0 Trace String (If there is one to Set) */
    if(pRstCfg->pExpectedJ0String != NULL)
    {
        pReg = BaseAddress + ixf6048_EXPECTED_J0_CHAN_OFFSET(ChanNum);
        /* Map the ixf6048_RxJ0Format_e Val to the generic bb_TraceFormat_e */
        TraceFormat = IXF6048_RCV_TRACE_FORMAT_ARR[pRstCfg->ExpectedJ0Format];
        /* Set the J0_RSTC bits 2:0 to TraceFormat -- note that JnCfgMask is Ignored */
        rc = SetRcvJnPathTrace( pReg, &JnCfgMask, TraceFormat, pRstCfg->pExpectedJ0String);
    }
    return rc;
}
/***********************************************************************
 * Procedure Name:  EditRxMstCfg 
 *
 * Description:     Rx Multiplexer Section Termination Configuration
 *                   
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 * End
 **********************************************************************/
static bb_Error_e EditRxMstCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_RxMstCfg_t* pMstCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort  RegVal;
    register ushort* pReg;

    /* Set the B2 Excessive Error Configuration Values */
    ixf6048_SET_EXC_B2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Set.WindowSize);
    ixf6048_SET_EXC_B2_NUM_ERRS(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Set.NumExcessiveErrs);
    ixf6048_SET_EXC_B2_NUM_WINDOWS(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Set.NumExcessiveWindows);

    /* Settings to the Clear B2 Excessive Error Configuration  */
    ixf6048_SET_CLR_EXC_B2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Clear.WindowSize);
    ixf6048_SET_CLR_EXC_B2_NUM_ERRS(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Clear.NumNonexcessiveErrs);
    ixf6048_SET_CLR_EXC_B2_NUM_WINDOWS(BaseAddress,ChanNum,
        pMstCfg->B2ExcErrCfg.Clear.NumNonexcessiveWindows);

    /* Set the B2 Degraded Error Configuration Values */
    ixf6048_SET_LSB_SDEGB2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Set.WindowSize);
    ixf6048_SET_MSB_SDEGB2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Set.WindowSize);

    ixf6048_SET_DEG_B2_ERRORS(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Set.NumDegradedErrs);
    ixf6048_SET_DEG_B2_WINDOWS(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Set.NumDegradedWindows);

    /* Settings to Clear Degraded B2 Errors */
    ixf6048_SET_CLR_LSB_DEGB2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Clear.WindowSize);
    ixf6048_SET_CLR_MSB_DEGB2_WINDOW_SIZE(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Clear.WindowSize);

    ixf6048_SET_CLEAR_DEG_B2_ERRORS(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Clear.NumNondegradedErrs);

    ixf6048_SET_CLEAR_DEG_B2_WINDOWS(BaseAddress,ChanNum,
        pMstCfg->B2DegErrCfg.Clear.NumNondegradedWindows);

    /* Now set the Config Values for the R_MST_C Register */
    RegVal = 0;
    if (pMstCfg->K1K2Filtering == ixf6048_eK1K2_COMBINED)/* Bit13 */
        Ixf6048SetBitVal(RegVal, ixf6048_bMST_APS_FILT_CNFG);

    if (pMstCfg->K2AisDetect == ixf6048_eK2_AIS_DETECT_IN_5_FRAMES)/* Bit10 */
        Ixf6048SetBitVal(RegVal, ixf6048_bMST_AIS_DET_CNT);

    /* Bits[9:8] */
    ixf6048_SET_K2_RDI_DETECT(RegVal,pMstCfg->K2RdiDetect);

    if (pMstCfg->AisToMsaOnExcB2Err == ixf6048_eENABLE_AIS_TO_MSA_ON_EXC_B2_ERR)/* Bit7 */
        Ixf6048SetBitVal(RegVal, ixf6048_bAIS_ON_EXC_B2_EN);

    if (pMstCfg->AisToMsa == ixf6048_eENABLE_AIS_TO_MSA)/* Bit5 */
        Ixf6048SetBitVal(RegVal, ixf6048_MST_AIS_EN);

    if (pMstCfg->TxReiBipBlk == ixf6048_eTX_REI_BLK_ERRS)/* Bit4 */
        Ixf6048SetBitVal(RegVal, ixf6048_MST_GEN_REI_CNF);

    if (pMstCfg->RxReiBipBlk == ixf6048_eRX_REI_BLK_ERRS) /* Bit3 */
        Ixf6048SetBitVal(RegVal, ixf6048_MST_RCV_REI_CNF);

    if (pMstCfg->MstRdiOnExcB2Err == ixf6048_eENABLE_MST_RDI_ON_EXC_B2_ERR) /* Bit2 */
        Ixf6048SetBitVal(RegVal, ixf6048_MST_RDI_ON_EXC_B2_EN);

    if (pMstCfg->BlksPerConcatFrame == ixf6048_eMULT_2430_BYTE_BLKS_PER_FRAME) /* Bit0 */
        Ixf6048SetBitVal(RegVal, ixf6048_MST_BLK_ERR_CNFG);

    /* Now set the New Value to the Register */
    pReg = BaseAddress + ixf6048_R_MST_C_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    return rc;
}
/***********************************************************************
 * Procedure Name:  EditRxMsaCfg 
 *
 * Description:     Rx Multiplexer Section Adaptation Configuration 
 *                      - Reg R_MSA_C 
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditRxMsaCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_RxMsaCfg_t* pMsaCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort  RegVal;
    register ushort* pReg;

    /* Assign all of the Register Configuration Values R_MSA_C */
    RegVal = 0;
    /* Bit9 Configure the AuAisc detection*/
    if(pMsaCfg->UseAuAiscDetection == ixf6048_eAU_AISC_DETECTION_ENABLE) 
        Ixf6048SetBitVal(RegVal, ixf6048_bDAUAISC_EN);
    /* Bit8 */
    if(pMsaCfg->LopcOnServerDefect == ixf6048_eHP_RDI_GEN_DURING_LOPC_ENABLE) 
        Ixf6048SetBitVal(RegVal, ixf6048_bLOPC_ON_SV_DEF_EN);
    /* Bit7*/
    if(pMsaCfg->LopcOnMsaAis == ixf6048_eAIS_GEN_DURING_LOPC_ENABLE)
        Ixf6048SetBitVal(RegVal, ixf6048_bLOPC_ON_MSA_AIS_EN);
    /* Bit6*/
    if(pMsaCfg->ConcPtDetCnfg_e == ixf6048_eVERIFY_CONCAT_PTR_ON_SOME_H1H2_PAIRS)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_CONC_PT_DET_CNFG);
    /* Bit5*/
    if(pMsaCfg->UseSonetSdhAuPtrs == ixf6048_eUSE_SONET_AU_PTRS)
        Ixf6048SetBitVal(RegVal, ixf6048_bAU_PTR_JUST_CNFG);
    /* Bit2*/
    if(pMsaCfg->AisToHpt == ixf6048_eENABLE_AIS_TO_HPT)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_MSA_AIS_EN);
    /* Bit0*/
    if(pMsaCfg->UseAuPtrSsBits == ixf6048_eUSE_AU_PTR_SS_BITS)
    {
        Ixf6048SetBitVal(RegVal, ixf6048_bAU_PTR_SS_EN);
        /* Set SS bits R_MSA_C Bits [4:3] only if Using AU Pointer SS Bits Configured */
        ixf6048_STORE_EXPC_AU_PNTR_SS_BITS(RegVal,pMsaCfg->ExpectedAuPtrSsBits);
    }

    /* Now set the Updated Register Value */
    pReg  = BaseAddress + ixf6048_R_MSA_C_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    return rc;
}
/***********************************************************************
 * Procedure Name: EditRxHptCfg  
 *
 * Description:      Rx High Order Path Termination Configuration
 *                      Regs: R_HPT_C1, R_HPT_C2
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditRxHptCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_RxHptCfg_t* pRxHptCfg)
{
    bb_Error_e        rc = bb_NO_ERROR;
    bb_TraceFormat_e  TraceFormat;
    ushort   RegVal;
    register ushort*  pReg;
    ushort            JnCfgMask;

    /* Set the New R_HPT_C1 Config Bits */
    RegVal = 0;
    /* R_HPT_C1 Bits[15:14]*/
    ixf6048_SET_HPT_RDI_DET_CNT(RegVal, pRxHptCfg->HptRdiDetect);

    /* R_HPT_C1 Bit 13 */
    if(pRxHptCfg->DetectEnhancedHptRdi == ixf6048_eDETECT_ENHANCED_HPT_RDI)
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_RDI_DET_ENH_CNFG);

    /* R_HPT_C1 Bit 12 */
    if(pRxHptCfg->HptSlMismatchCnt == ixf6048_eHPT_SL_MISMATCH_ON_5_C2)
        Ixf6048SetBitVal(RegVal, ixf6048_bC2_MS_MTCH_CNT);

    /* R_HPT_C1 Bit 11  */
    if(pRxHptCfg->HpSlmOnC2Config == ixf6048_eFORCE_HPT_SLM_ALARM_ON_C2_UNSTABLE)
        Ixf6048SetBitVal(RegVal, ixf6048_bHP_SLM_ON_C2_UNSTABLE_EN);

    /* R_HPT_C1 Bit 10 */
    if(pRxHptCfg->B3CntrCfg == ixf6048_eCOUNT_B3_BLOCK_ERRORS)           
        Ixf6048SetBitVal(RegVal, ixf6048_bB3_CNTR_CNFG);

    /* R_HPT_C1 Bit 9 */
    if(pRxHptCfg->HptReiCntrCfg == ixf6048_eCOUNT_HPT_REI_BLOCK_ERRORS)       
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_REI_CNTR_CNFG);

    /* R_HPT_C1 Bit 8 */
    if(pRxHptCfg->Cols30_59 == ixf6048_eCOLS_30_59_STUFF)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_STUFF_CFG);

    /* R_HPT_C1 Bit 7 */
    if(pRxHptCfg->HptRdiOnAtmLcd == ixf6048_eENABLE_HPT_RDI_ON_ATM_LCD)     
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_RDI_ON_LCD_EN);

    /* R_HPT_C1 Bits[6:5] -- Set Behaviour of RDI on Hpt Signal Label Mismatch*/
    ixf6048_SET_HPT_RDI_ON_SLM_EN(RegVal, pRxHptCfg->HptRdiOnHptSlm);

    /* R_HPT_C1 Bit 4 */
    if(pRxHptCfg->HptRdiOnJ1Mismatch == ixf6048_eENABLE_HPT_RDI_ON_J1_MISMATCH)
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_RDI_ON_TIM_EN);

    /* R_HPT_C1 Bit 3 */
    if(pRxHptCfg->HptRdiOnHptUneq == ixf6048_eENABLE_HPT_RDI_ON_HPT_UNEQ)
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_RDI_ON_UNEQP_EN);

    /* R_HPT_C1 Bits[2:1]*/
    ixf6048_SET_HPT_RDI_GEN_ENH_CNFG(RegVal,pRxHptCfg->GenHptRdiCfg);

    /* R_HPT_C1 Bit 0 */
    if(pRxHptCfg->G1SpareBit == ixf6048_eG1_SPARE_BIT_IS_1)
        Ixf6048SetBitVal(RegVal, ixf6048_bG1_SP_ON_RPAL_CNFG);

    /* Write the New Value to the Register */
    pReg = BaseAddress + ixf6048_R_HPT_C1_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Set the New R_HPT_C2 Config Bits */

    /* R_HPT_C2 Bit 13 */
    if(pRxHptCfg->J1Crc7Mismatch == ixf6048_eJ1_CRC7_MASKS_MISMATCH)     
        Ixf6048SetBitVal(RegVal, ixf6048_bHP_TIM_ON_CRC7_ERR_DSB);

    /* R_HPT_C2 Bit 12 */
    if(pRxHptCfg->J1UnstableMismatch == ixf6048_eJ1_UNSTABLE_SETS_MISMATCH) 
        Ixf6048SetBitVal(RegVal, ixf6048_bHP_TIM_ON_UNSTABLE_EN);

    /* R_HPT_C2 Bit 11 */
    if(pRxHptCfg->J1StableCnt == ixf6048_eJ1_STABLE_AFTER_5)        
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_J1_STABLE_CNFG);

    /* R_HPT_C2 Bits[10:8]*/
    /*ixf6048_SET_RCV_J1_CNF_FORMAT(RegVal,
    pRxHptCfg->ExpectedJ1Format);*/
    SetRegBits(&RegVal, (ushort) pRxHptCfg->ExpectedJ1Format, 
        (ushort) ixf6048_mRCV_J1_CNF, (ushort) ixf6048_BIT_SHIFT_8);

    /* R_HPT_C2 Bits[7:6] Unused */

    /* R_HPT_C2 Bit 5 -- Used in Testing */
    if(pRxHptCfg->RcvHptAisFrc == ixf6048_eFORCE_AIS_TO_HPT)  
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_HPT_AIS_FRC);

    /* R_HPT_C2 Bit 4 */
    if(pRxHptCfg->RcvHptAisSlmEnbl == ixf6048_eENABLE_AIS_TO_HPA)  
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_HPT_AIS_SLM_ENBL);

    /* R_HPT_C2 Bit 3 */
    if(pRxHptCfg->RcvHptAisUneqEnbl == ixf6048_eENABLE_AIS_TO_HPA) 
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_HPT_AIS_UNEQ_ENBL);

    /* R_HPT_C2 Bit 2 */
    if(pRxHptCfg->RcvHptAisTimEnbl == ixf6048_eENABLE_AIS_TO_HPA)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_HPT_AIS_TIM_ENBL);

    /* R_HPT_C2 Bits[1:0] Unused */

    /* Expected HPT Signal Label */
    ixf6048_PUT_EXP_C2(BaseAddress,ChanNum,pRxHptCfg->ExpectedC2);

    /* -> Expected J1 string */

    /* Now Set the Expected J1 Trace String -- If there is one to Set */
    if (pRxHptCfg->pExpectedJ1String != NULL)
    {
        pReg = BaseAddress + ixf6048_EXPECTED_J1_CHAN_OFFSET(ChanNum);

        /* Map the ixf6048_RxJ0Format_e Val to the generic bb_TraceFormat_e */
        TraceFormat = IXF6048_RCV_TRACE_FORMAT_ARR[pRxHptCfg->ExpectedJ1Format];
        /* Now Set the J1 Path Trace to the Register -- Note that JnCfgMask is Ignored */
        rc = SetRcvJnPathTrace(pReg, &JnCfgMask, TraceFormat, pRxHptCfg->pExpectedJ1String);
    }
    return rc;
}

/***********************************************************************
 * Procedure Name: EditTxRstCfg  
 *
 * Description: Tx Regenerator Section Termination Configuration
 *
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 * End
 **********************************************************************/
static bb_Error_e EditTxRstCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_TxRstCfg_t* pTxRstCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    bb_TraceFormat_e TraceFormat;
    ushort           JnCfgMask;
    ushort  		 RegVal;
    register ushort* pReg;

    /* Clear the Register Value and set the new Config Values 
    for the T_RMST_OP Config Register */
    RegVal = 0;

    /* T_RMST_OP bit 15 */
    if(pTxRstCfg->TxRstScrambling == ixf6048_eENABLE_TX_RST_2E7_SCRAMBLING)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_SCRMBL_CNFG);

    /* T_RMST_OP bit 14 */
    if(pTxRstCfg->UnusedTxOh == ixf6048_eUNUSED_TX_OH_ONES)       
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_UN_BY_CNFG);

    /* T_RMST_OP bits[13:12] */
    ixf6048_SET_TX_TZ0_BY_LINE_SPEED_EN(RegVal, pTxRstCfg->TxTzo);

    /* T_RMST_OP bits[11:10] */
    ixf6048_SET_TX_SZ0_BY_LINE_SPEED_EN(RegVal, pTxRstCfg->TxSzo);

    /* T_RMST_OP bits[9:8] */
    ixf6048_SET_TX_FZ0_BY_LINE_SPEED_EN(RegVal, pTxRstCfg->TxFzo);

    /* T_RMST_OP bits[7:6] */
    /* ixf6048_SET_TX_J0_CNFG(RegVal, pTxRstCfg->TxJ0Format); */
    SetRegBits(&RegVal, (ushort) pTxRstCfg->TxJ0Format, 
        (ushort) ixf6048_mXMT_J0_CNFG, (ushort) ixf6048_BIT_SHIFT_6);

    /* T_RMST_OP bits[5:1] Used for Testing*/
    /* Now set the New Config Value to the Register */
    pReg = BaseAddress + ixf6048_T_RMST_OP_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Clear the Register Value and set the new Config Values 
    for the T_SC_RSOH config Register */
    RegVal = 0;

    /* PSN_ADDED for B1 Release */
    /* T_SC_RSOH bit[15] */
    if(pTxRstCfg->TSOHInsEna == ixf6048_eOH_INS_ENABLED)
        Ixf6048SetBitVal(RegVal, ixf6048_bTSOH_INS_ENA);
    /* PSN_END B1 Release Changes */

    /* T_SC_RSOH bit 10 (ignored if STM-0/STS-1) */
    if(pTxRstCfg->TxUnSource == ixf6048_eTX_UN_RX_BYTE)    
        Ixf6048SetBitVal(RegVal, ixf6048_bMT_UN_RSOH_SRC);

    /* T_SC_RSOH bits 9:8 */
    ixf6048_SET_XMT_D1D3_SRC(RegVal,pTxRstCfg->TxD1D3Source);

    /* T_SC_RSOH bit 7 (ignored if STM-0/STS-1) */
    if(pTxRstCfg->TxNu2Source == ixf6048_eTX_NU2_RX_BYTE)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_NU_2_SRC);

    /* T_SC_RSOH bits 6:5 */
    ixf6048_SET_XMT_F1_SRC(RegVal, pTxRstCfg->TxF1Source);

    /* T_SC_RSOH bits 4:2 */
    ixf6048_SET_XMT_E1_SRC(RegVal, pTxRstCfg->TxE1Source);

    /*T_SC_RSOH bit 1 (ignored if STM-0/STS-1) */
    if(pTxRstCfg->TxNu1Z0Source == ixf6048_eTX_NU1Z0_RX_BYTE)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_NU_1_Z0_SRC);

    /* T_SC_RSOH bit 0 */
    if(pTxRstCfg->TxJ0Source == ixf6048_eTX_J0_RX_BYTE)    
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_J0_SRC);

    /* Now set the New Config Value to the Register */
    pReg = BaseAddress + ixf6048_T_SC_RSOH_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    if(pTxRstCfg->pJ0String)  /* -> J0 string, if from CPU */
    {
        pReg = BaseAddress + ixf6048_TX_J0_CHAN_OFFSET(ChanNum);
        /* Map the TxJn Trace format to Generic bb_TraceFormat_e */
        TraceFormat = IXF6048_TX_TRACE_FORMAT_ARR[pTxRstCfg->TxJ0Format];
        /* Set the String to the Register. Note: JnCfgMask is Ignored */
        rc = SetTxJnPathTrace( pReg, &JnCfgMask, TraceFormat, pTxRstCfg->pJ0String);
    }
    return rc;
}
/***********************************************************************
 * Procedure Name: EditTxMstCfg  
 *
 * Description: Tx Multiplexer Section Termination Configuration -
 *                  (Not needed for a Repeater)
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 * End
 **********************************************************************/
static bb_Error_e EditTxMstCfg(ushort*             BaseAddress, 
    uint                ChanNum,
    ixf6048_TxMstCfg_t* pMstCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort  RegVal;
    register ushort* pReg;

    /* Clear the Reg Value and Set the new config for the T_SC_MSOH Register */
    RegVal = 0;

    /* T_SC_MSOH bits[15:14] */
    ixf6048_SET_MST_RDI_ON_HYST_CNFG(RegVal, pMstCfg->TxMstRdiHysteresis);

    /* T_SC_MSOH bits[13:12] */
    ixf6048_SET_MST_REI_SRC(RegVal, pMstCfg->TxMstReiSource);

    /* T_SC_MSOH bits[11:10] */
    ixf6048_SET_MST_RDI_SRC(RegVal, pMstCfg->TxMstRdiSource);

    /* T_SC_MSOH bit 9 */
    if(pMstCfg->TxK2ApsSource == ixf6048_eTX_K2_APS_FROM_TSAL)   
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_K2_APS_SRC);

    /* T_SC_MSOH bit 8 */
    if(pMstCfg->TxK1Source == ixf6048_eTX_K1_FROM_TSAL)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_K1_SRC);

    /* T_SC_MSOH bit 7 */
    if(pMstCfg->TxInSectionAlarmBus == ixf6048_eTX_IN_SECTION_ALM_BUS_CODIR)
        Ixf6048SetBitVal(RegVal, ixf6048_bTSAL_BUS_CNFG);

    /* T_SC_MSOH bit 6 */
    if(pMstCfg->TxMsoh_Source == ixf6048_eTX_MSOH_RX_BYTE)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_PRC_MSOH_SRC);

    /* T_SC_MSOH bits 5:3 */
    ixf6048_SET_E2_SRC(RegVal, pMstCfg->TxE2Source);

    /* T_SC_MSOH bits 2:1 */
    ixf6048_SET_XMT_D4D1_SRC(RegVal, pMstCfg->TxD4D12Source);

    /* T_SC_MSOH bit 0 */
    if(pMstCfg->TxUnMsoh_Source == ixf6048_eTX_UN_MSOH_RX_BYTE)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_UN_MSOH_SRC);

    pReg  = BaseAddress + ixf6048_T_SC_MSOH_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* See Regs MP_TK2K1 and MP_TS1 for K1,K2,S1 */
    ixf6048_SET_TX_K1_BYTE(BaseAddress, ChanNum, pMstCfg->K1);   /* if from CPU */
    ixf6048_SET_TX_K2_BYTE(BaseAddress, ChanNum, pMstCfg->K2);   /* If MST RDI or K2 APS from CPU */
    ixf6048_SET_TX_S1_BYTE(BaseAddress, ChanNum, pMstCfg->S1);

    return rc;
}
/***********************************************************************
 * Procedure Name: EditTxMsaCfg  
 *
 * Description:   Tx Multiplexer Section Adaptation Configuration
 *
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditTxMsaCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxMsaCfg_t* pTxMsaCfg)
{
    register ushort* pReg;
    register ushort  RegVal;

    /* Set a pointer to the Tx AU pointer Config Register */
    pReg = BaseAddress + ixf6048_T_AU_PTS_CHAN_OFFSET(ChanNum);
    /* Read the Reg Value one Time*/
    RegVal = READ_FROM_HW(pReg);

    /* Now set the new config Values to the Temp Register Value */
    /* Initial Tx AU Pointer (0-782) - reg T_AU_PTS bits 9:0 */
    ixf6048_SET_XMT_AU_PT_VAL(RegVal, pTxMsaCfg->InitialTxAuPtr);

    /* Tx SS bits, in Tx AU pointer - reg T_AU_PTS bits 11:10 */
    ixf6048_SET_XMT_AU_PT_SS(RegVal, pTxMsaCfg->TxSsBits);

    /* Now Write the new T_AU_PTS Configuration register Value*/
    WRITE_TO_HW(pReg, RegVal);

    return bb_NO_ERROR;
}
/***********************************************************************
 * Procedure Name: EditTxHptCfg  
 *
 * Description: Tx High Order Path Termination Configuration
 *
 * Conditions for Use:
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditTxHptCfg(ushort*            BaseAddress, 
    uint               ChanNum,
    ixf6048_TxHptCfg_t* pTxHptCfg)
{
    bb_Error_e       rc = bb_NO_ERROR;
    bb_TraceFormat_e TraceFormat;
    ushort           JnCfgMask;
    register ushort  RegVal;
    register ushort* pReg;
    register ushort  macroRegVal;

    RegVal = 0;
    /* PSN_ADDED for B1 Release */
    /* T_HPT_C Bit[15]  */
    if(pTxHptCfg->TPOHInsEna == ixf6048_eOH_INS_ENABLED)
        Ixf6048SetBitVal(RegVal, ixf6048_bTPOH_INS_ENA);
    /* PSN_END B1 Release Changes */
    /*  T_HPT_C bit 14 */
    if(pTxHptCfg->Cols30_59 == ixf6048_eCOLS_30_59_STUFF)      
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_STUF_CNFG);
    /*  T_HPT_C bit 13 */
    if(pTxHptCfg->TxAuPtrSource == ixf6048_eTX_AU_PTR_FROM_RX_BYTE)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_AU_PNTR_SRC);
    /*  T_HPT_C bit 12 */
    if(pTxHptCfg->TxPohSource == ixf6048_eTX_POH_RX_BYTES)
        Ixf6048SetBitVal(RegVal, ixf6048_XMT_POH_SRC);
    /* T_HPT_C bit 11 Unused */
    /*  T_HPT_C bit 10 */
    if(pTxHptCfg->TxH4Value == ixf6048_eTX_H4_IS_ONES)      
        Ixf6048SetBitVal(RegVal, ixf6048_XMT_H4_CNFG);
    /*  T_HPT_C bits[9:8] */
    ixf6048_SET_XMT_F2_SRC(RegVal, pTxHptCfg->TxF2Source);
    /*  T_HPT_C bits[7:6] */
    ixf6048_SET_XMT_F3_SRC(RegVal, pTxHptCfg->TxF3Source);
    /* T_HPT_C bits[5:4] Unused */
    /*  T_HPT_C bits[3:2] */
    ixf6048_SET_HPT_REI_SRC(RegVal, pTxHptCfg->HptReiSource);
    /*  T_HPT_C bits[1:0] */
    ixf6048_SET_HPT_RDI_SRC(RegVal, pTxHptCfg->HptRdiSource);

    /* Now set the New T_HPT_C Register value to the Register */
    pReg  = BaseAddress + ixf6048_T_HPT_C_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* Clear the RegVal and Setup the Config Values for the T_HPT_OPC register */
    RegVal = 0;
    /* T_HPT_OPC bit 15 */
    if(pTxHptCfg->TxInPathAlarmBus == ixf6048_eTX_IN_PATH_ALM_BUS_CODIR)  
        Ixf6048SetBitVal(RegVal, ixf6048_bTPAL_BUS_CNFG);
    /*T_HPT_OPC bits[14:13] unused */
    /* T_HPT_OPC bit 12 */
    if(pTxHptCfg->InvH1ConcInd == ixf6048_eINV_H1_CONC_IND_INVERT_FOREVER)      
        Ixf6048SetBitVal(RegVal, ixf6048_bINV_H1_CONC_IND);
    /* T_HPT_OPC bits[11:10] */
    ixf6048_SET_HPT_RDI_XMT_HYST_CNFG(RegVal, pTxHptCfg->TxHptRdiHysteresis);
    /* T_HPT_OPC bit 9 */
    if(pTxHptCfg->TxEnhancedHptRdi == ixf6048_eTX_ENHANCED_HPT_RDI) 
        Ixf6048SetBitVal(RegVal, ixf6048_bHPT_RDI_XMT_ENH_CNFG);
    /* T_HPT_OPC bit 8*/
    if(pTxHptCfg->TxVcUneq == ixf6048_eENABLE_VC_UNEQ)          
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_UNEQP);
    /* T_HPT_OPC bit 7*/
    if(pTxHptCfg->TxJ1UnequippedValue == ixf6048_eTX_J1_SEND_ZEROS_ON_HPT_UNEQ)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_UNEQP_J1_CNFG);
    /* T_HPT_OPC bit 6*/
    if(pTxHptCfg->TxN1UnequippedValue == ixf6048_eTX_N1_SEND_ZEROS_ON_HPT_UNEQ)
        Ixf6048SetBitVal(RegVal, ixf6048_bXMT_UNEQP_N1_CNFG);
    /* T_HPT_OPC bits[5:4] */
    ixf6048_SET_TX_J1_CNFG(RegVal, pTxHptCfg->TxJ1Format);
    /* T_HPT_OPC Bits[3:0] used for Testing */

    /* Now set the New  T_HPT_C Register value to the Register */
    pReg  = BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /*  C2 Byte reg MP_TC2     */
    pReg = BaseAddress + ixf6048_MP_TC2_CHAN_OFFSET(ChanNum);
    macroRegVal = READ_FROM_HW(pReg);
    ixf6048_SET_XMT_C2(macroRegVal, pTxHptCfg->C2);

    /* Clear the RegVal and set the G1 RDI Bits */
    RegVal = 0;
    /*  G1[3:1] reg MP_THPTRDI Bits[3:0]*/
    ixf6048_SET_XMT_HPT_RDI(RegVal, pTxHptCfg->HptRdi);

    /*  G1[0] Reg MP_THPTRDI Bit 0*/
    Ixf6048SetBitVal(RegVal, (pTxHptCfg->XmtG1SpBit & ixf6048_bXMT_G1_SP_BIT));

    /* Set the G1[3:0] bits to the MP_THPTRDI Register */
    pReg = BaseAddress + ixf6048_MP_THPTRDI_CHAN_OFFSET(ChanNum);
    WRITE_TO_HW(pReg, RegVal);

    /* -> J1 string, if used */
    if(pTxHptCfg->pTxJ1String != NULL)
    {
        pReg = BaseAddress + ixf6048_TX_J1_CHAN_OFFSET(ChanNum);
        /* Map the TxJn Trace format to Generic bb_TraceFormat_e */
        TraceFormat = IXF6048_TX_TRACE_FORMAT_ARR[pTxHptCfg->TxJ1Format];
        /* Set the String to the Register. Note: JnCfgMask is Ignored */
        rc = SetTxJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTxHptCfg->pTxJ1String);
    }

    return rc;
}

/***********************************************************************
 * Procedure Name:   EditAtmChanCfg
 *
 * Description: Edit ATM Channel configuration
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 *
 * End
 **********************************************************************/
static bb_Error_e EditAtmChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_AtmChanCfg_t* pCfg)
{
    ixf6048_RxAtmChanCfg_t*   pRxAtm;
    ixf6048_TxAtmChanCfg_t*   pTxAtm;
    ixf6048_IdleCellFilter_t* pCellFltr;
    register ushort*          pReg;
    ushort           		  RegVal;
    ushort                    LineRate = 0;

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pRxAtm =  &pCfg->RxAtmChanCfg;

        /* Clear the Register Value and Load the new R_ACPCNF config Settings */
        RegVal = 0;

        /* R_ACPCNF bit 8 */
        if(pRxAtm->DropPassIdleCell == ixf6048_eDROP_IDLE_CELLS)    
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_IU_FLTR_EN);
        /* R_ACPCNF bit 7 */
        if(pRxAtm->DropCorSingleBitHecErrs == ixf6048_eCORRECT_SINGLE_BIT_HEC_ERRS)     
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_CORRECT_EN);

        /* R_ACPCNF bits[6:5] */
        /* ixf6048_SET_RCV_CORR_DET_CNF(RegVal, pRxAtm->RxCorDetect); */
        SetRegBits (&RegVal, (ushort) pRxAtm->RxCorDetect, 
            (ushort) ixf6048_mRCV_CORR_DET_CNF, (ushort) ixf6048_BIT_SHIFT_5);
        /* R_ACPCNF bit 4 */
        if(pRxAtm->DropPassLcdCells == ixf6048_eDROP_LCD_CELLS)            
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_LCD_CNF);

        /* R_ACPCNF bit 3 */
        if(pRxAtm->DropPassUncorCellsInSync == ixf6048_ePASS_UNCOR_CELLS_IN_SYNC)    
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_SYNC_CNF);

        /* R_ACPCNF bit 2 */
        if(pRxAtm->DropPassOkCellsInPresync == ixf6048_ePASS_OK_CELLS_IN_PRESYNC)    
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_PRESYNC_CNF);

        /* R_ACPCNF bit 1 */
        if(pRxAtm->HecXor == ixf6048_eENABLE_HEC_XOR)                      
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_HEC_ADD);

        /* R_ACPCNF bit 0 */
        if(pRxAtm->SpeCellDescrambling == ixf6048_eENABLE_SPE_CELL_DESCRAMBLING)         
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_DESCR_EN);
        /* Now Set the New Configuration to the Config Register 
        -- Note: This Register has Clear on Read Bits, so Simply Overwrite with 
        the latest settings */
        pReg  = BaseAddress +  ixf6048_R_ACPCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, RegVal);

        /* Set the LCD filter According to Table 24 in the Data sheet
        -- Register R_LCDFLTR   
        -- Must First Determine the Channel Rate */
        pReg = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum);
        LineRate = READ_FROM_HW(pReg) & ixf6048_mCHAN_RATE; /* OC1, OC3, OC12, OC48 */

        pReg = BaseAddress + ixf6048_R_LCDFLTR_CHAN_LSB_OFFSET(ChanNum);
        RegVal = LCD_FILTER_CONFIGURATION_TABLE[LineRate][pRxAtm->LcdFilter][LCD_FILTER_LSB_OFFSET];
        WRITE_TO_HW(pReg, RegVal); /*LCD_FILTER_CONFIGURATION_TABLE[LineRate][pRxAtm->LcdFilter][LCD_FILTER_LSB_OFFSET]; */

        pReg = BaseAddress + ixf6048_R_LCDFLTR_CHAN_MSB_OFFSET(ChanNum);
        RegVal = LCD_FILTER_CONFIGURATION_TABLE[LineRate][pRxAtm->LcdFilter][LCD_FILTER_MSB_OFFSET];
        WRITE_TO_HW(pReg, RegVal);

        /* Setup the Idle/Unassigned Cell Filter Register R_IUCFLTR   */
        pCellFltr = &pRxAtm->IdleCellFilter;
        RegVal = 0;

        /* bits 15:12  are mask for GFC bits */
        /* ixf6048_SET_RCV_GFC_MASK(RegVal, pCellFltr->GfcMask); */
        SetRegBits (&RegVal, pCellFltr->GfcMask, 
            ixf6048_mRCV_GFC_MASK, ixf6048_BIT_SHIFT_12);

        /* bits 11:9 are mask for PTI bits */
        /*  ixf6048_SET_RCV_PTI_MASK(RegVal, pCellFltr->PtiMask); */
        SetRegBits (&RegVal, pCellFltr->PtiMask, 
            ixf6048_mRCV_PTI_MASK, ixf6048_BIT_SHIFT_9);

        /* bit 8  is the mask for CLP bit */
        /* ixf6048_SET_RCV_CLP_MASK(RegVal, pCellFltr->ClpMask); */
        SetRegBits (&RegVal, pCellFltr->ClpMask, 
            ixf6048_bRCV_CLP_MASK, ixf6048_BIT_SHIFT_8);

        /* bits 7:4 are masked Pattern of GFC bits */
        /* ixf6048_SET_RCV_GFC_PATT(RegVal, pCellFltr->GfcPattern); */
        SetRegBits (&RegVal, pCellFltr->GfcPattern, 
            ixf6048_mRCV_GFC_PATT, ixf6048_BIT_SHIFT_4);

        /* bits 3:1 are masked value of PTI bits */
        /* ixf6048_SET_RCV_PTI_PATT(RegVal, pCellFltr->PtiPattern);*/
        SetRegBits (&RegVal, pCellFltr->PtiPattern, 
            ixf6048_mRCV_PTI_PATT, ixf6048_BIT_SHIFT_1);

        /* bit 0 is the masked value of CLP bit */
        /* ixf6048_SET_RCV_CLP_PATT(RegVal, pCellFltr->ClpPattern); */
        SetRegBits (&RegVal, pCellFltr->ClpPattern, 
            ixf6048_bRCV_CLP_PATT, ixf6048_BIT_SHIFT_0);

        /* Now set the New Config Values to the Register */
        pReg  = BaseAddress +  ixf6048_R_IUCFLTR_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, RegVal);
    }

    if( (Side == ixf6048_eSIDE_BOTH) ||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pTxAtm = & pCfg->TxAtmChanCfg;

        RegVal = 0;

        /* PSN_ADDED B1 Release Changes  -- Set Tx Idle cell to be  VPI=32 */
        RegVal |= (pTxAtm->XmtTestVPI==ixf6048_eXMT_TEST_VPI_DEBUG_MODE)?ixf6048_bXMT_TEST_VPI:0;
        /* PSN_END B1 Release Changes*/
        /* Masks bit 12:11 */
        SetRegBits (&RegVal, (ushort) pTxAtm->HecInsertion, 
            (ushort) ixf6048_mXMT_EXT_HEC, (ushort) ixf6048_BIT_SHIFT_11);
        /* Setup the Role of the GFC */
        if(pTxAtm->GfcRole == ixf6048_eGFC_CONTROLLER) /* this chip's GFC role */
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_GFC_CONTROLLER);
        else if (pTxAtm->GfcRole == ixf6048_eGFC_CONTROLLED)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_GFC_CONTROLLED);
        else  /* (ixf6048_eGFC_NOT_ACTIVE) */
            Ixf6048ClrBitVal(RegVal, (ixf6048_bXMT_GFC_CONTROLLER | ixf6048_bXMT_GFC_CONTROLLED)) ;
        /* Masks bits 8:6 */
        SetRegBits (&RegVal,  (ushort) pTxAtm->GfcHaltRatio, 
            (ushort) ixf6048_mXMT_GFC_CNT, (ushort) ixf6048_BIT_SHIFT_6);
        /* Bit 5 */
        if(pTxAtm->XmtReadEn == ixf6048_eREAD_EN_NORMAL_MODE)           /* Bit 5 */
            Ixf6048SetBitVal(RegVal,ixf6048_bXMT_ATM_READ_EN);

        /* Note that the XmtHErrCnf and XmtHErrEn Members Bits [4:3], 2 are 
        		Used in Test Mode Only */

        if(pTxAtm->HecXor == ixf6048_eENABLE_HEC_XOR)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_HEC_ADD);
        if (pTxAtm->SpeCellScrambling == ixf6048_eENABLE_SPE_CELL_SCRAMBLING);
        Ixf6048SetBitVal(RegVal,ixf6048_bXMT_SCR_EN);

        /* Set the new config Values to the T_ACPCNF Register */
        pReg = BaseAddress + ixf6048_T_ACPCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, RegVal);

        /* Clear the Register and Set the T_ICELLP Config Bits */
        pReg  = BaseAddress + ixf6048_T_ICELLP_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, 0);

        RegVal = 0;
        /* Tx Idle cell Structure -- Note: Values Set Directly to Register */
        /* Bits[15:8] Value for 48-byte ATM payload */
        SetRegBits (&RegVal, pTxAtm->TxIdleCellDef.PayloadValue, 
            ixf6048_mTX_PAYLOAD_PATTERN, ixf6048_BIT_SHIFT_8);
        /* Value in [7:4] sent in GFC bits of ATM header */
        SetRegBits (&RegVal, pTxAtm->TxIdleCellDef.GfcValue, 
            ixf6048_mTX_GFC_PATTERN, ixf6048_BIT_SHIFT_4);
        /* Value in [3:1] sent in PTI bits of ATM header */
        SetRegBits (&RegVal, pTxAtm->TxIdleCellDef.PtiValue, 
            ixf6048_mTX_PTI_PATTERN, ixf6048_BIT_SHIFT_1);
        /* Value in bit 0 sent in CLP bit  of ATM header */
        SetRegBits (&RegVal, pTxAtm->TxIdleCellDef.ClpValue, 
            ixf6048_mTX_CLP_PATTERN, ixf6048_BIT_SHIFT_0);

        /* Now Set New Config to Register */
        WRITE_TO_HW(pReg, 0);
    }


#ifdef UNIT_TEST_6048
    printf("Edit of ATM Chan Cfg at 0x%x - NO VERIFICATION\n", pCfg);
#endif

    return bb_NO_ERROR;       /* should be bb_NO_ERROR */

}   /* end EditAtmChanCfg */

/***********************************************************************
 * Procedure Name:   EditPosChanCfg
 *
 * Description:      Edit POS Channel configuration
 *
 * Conditions for Use:
 *    Driver is initialized OK
 *    pCfg is non-NULL
 *
 * Notes:
 *
 *
 *
 * PDL:
 *
 * End
 **********************************************************************/
static bb_Error_e EditPosChanCfg(ushort*               BaseAddress,
    uint                  ChanNum, 
    ixf6048_Side_e        Side,
    ixf6048_PosChanCfg_t* pCfg)
{
    ixf6048_RxPosChanCfg_t* pRxPos;
    ixf6048_TxPosChanCfg_t* pTxPos;
    register ushort*        pReg;
    ushort         		    RegVal;
    uchar                   XmtIpgRel=0;
    uchar                   TempN=0;
    uchar                   NumFlagsAbs=0;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pRxPos = &pCfg->RxPosChanCfg;

        /* Clear the Register value and set config bits */
        RegVal = 0;

        /* Reg R_PHCCNF Bit 12 */
        if(pRxPos->RxPosLineOrTrafficMon == ixf6048_eRX_POS_TRAFFIC_MONITORING)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_CNT_WR_FR);

        /* Reg R_PHCCNF Bit 11 */
        if(pRxPos->RxPosByteCountCfg == ixf6048_eRX_POS_COUNT_OK_BYTES)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_A_BYTE_CNT_EN);

        /* Reg R_PHCCNF Bit 10 */
        if(pRxPos->FcsErrMarkedInFifo == ixf6048_eFCS_ERR_MARKED_IN_FIFO)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_FCS_ERR);

        /* Reg R_PHCCNF Bit 9, 1*/
        if (pRxPos->RxPosFifoFormat == ixf6048_eRX_POS_FIFO_IS_PPP_AC )
        Ixf6048SetBitVal(RegVal,ixf6048_bRCV_A_C_PASS); /* Bit1 = 1, Bit9=0 */
        else if (pRxPos->RxPosFifoFormat == ixf6048_eRX_POS_FIFO_IS_PPP_FCS)
        Ixf6048SetBitVal(RegVal, ixf6048_bRCV_FCS_PASS); /* Bit1 = 0, Bit9= X */
        else /* (pRxPos->RxPosFifoFormat == ixf6048_eRX_POS_FIFO_IS_PPP) */
            Ixf6048ClrBitVal(RegVal, ixf6048_bRCV_A_C_PASS |
        ixf6048_bRCV_FCS_PASS); /* Bit1 = 0, Bit9=0 */

        /* Reg R_PHCCNF Bit 8 */
        if(pRxPos->FrameDescrambling == ixf6048_eENABLE_FRAME_DESCRAMBLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_TR_DESCR_EN);

        /* Reg R_PHCCNF Bit 0 */
        if(pRxPos->SpeFrameDescrambling == ixf6048_eENABLE_SPE_FRAME_DESCRAMBLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_DESCR_EN);

        /* Reg R_PHCCNF Bit 7 */
        if(pRxPos->FrameIntrafilling == ixf6048_eENABLE_FRAME_INTRAFILLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_FIF_EN);

        /* Reg R_PHCCNF Bit 6 */
        if(pRxPos->LongFrameHandling == ixf6048_eLONG_FRAMES_MARKED_ERR)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_MAX_PLD_EN);

        /* Reg R_PHCCNF Bit 5 */
        if(pRxPos->ShortFrameHandling == ixf6048_eSHORT_FRAMES_MARKED_ERR)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_MIN_PLD_EN);

        /* Reg R_PHCCNF Bit 4 */
        if(pRxPos->RxPosAcCheck == ixf6048_eRX_POS_AC_CHK)
            Ixf6048SetBitVal(RegVal, ixf6048_bRCV_A_C_CHK);

        /* Reg R_PHCCNF Bits[3:2]  */
        SetRegBits (&RegVal, (ushort) pRxPos->RxFcs, 
            (ushort) ixf6048_mRCV_FCS_CNF, (ushort) ixf6048_BIT_SHIFT_2);

        /* Now set the new Register Value to the POS Config Register */
        pReg = BaseAddress + ixf6048_R_PHCCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, 0);

        RegVal = 0;
        /* Min Packet Len -- Register R_MINPL*/
        ixf6048_SET_MIN_PACKET_LENGTH(BaseAddress, ChanNum,
            pRxPos->RxShortFrameLen);
        /* Max Packet Len -- Register R_MAXPL*/
        ixf6048_SET_MAX_PACKET_LENGTH(BaseAddress, ChanNum, 
            pRxPos->RxLongFrameLen);
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pTxPos = &pCfg->TxPosChanCfg;
        /* Clear then Load new Register Config to  RegVal */
        RegVal = 0;

        /* Reg T_PHCCNF bit 1 */
        if(pTxPos->TxPosGenAcceptAc == ixf6048_eTX_POS_ACCEPT_AC)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_A_C_PASS);

        /* Reg T_PHCCNF bit 0 */
        if(pTxPos->SpeFrameScrambling == ixf6048_eENABLE_SPE_FRAME_SCRAMBLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_SCR_EN);

        /* Reg T_PHCCNF bit 8 */
        if(pTxPos->FrameScrambling == ixf6048_eENABLE_FRAME_SCRAMBLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_TR_SCR_EN);

        /* Reg T_PHCCNF bit 7 */
        if(pTxPos->FrameIntrafilling == ixf6048_eENABLE_FRAME_INTRAFILLING)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_FIF_EN);

        /* Reg T_PHCCNF bit 6. Note: Default = 1*/
        if(pTxPos->ReadTxPosFifoEnable == ixf6048_ePOS_READ_PACKETS_FROM_TX_FIFO)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_POS_READ_EN);

        /* Reg T_PHCCNF bits[3:2] */
        /* ixf6048_SET_XMT_FCS_CNF(RegVal, pTxPos->TxFcs); */
        SetRegBits (&RegVal,  (ushort) pTxPos->TxFcs, 
            (ushort) ixf6048_mXMT_FCS_CNF, (ushort) ixf6048_BIT_SHIFT_2);

        /* Now Set the New Configuration to the T_PHCCNF Config Register */
        pReg = BaseAddress +  ixf6048_T_PHCCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, 0);

        /* Clear the Temp RegVal and load the Config Values for the T_IPGCTRL Register */
        RegVal = 0;

        /* Reg T_IPGCTRL bit 14,13  -- Method 1*/
        if (pTxPos->TxPosFlowControl.TxPosProportionalFill != ixf6048_eTX_POS_NO_PROPORTIONAL_FILL)
        {
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_IPG_REL_EN);  /*Always Set Bit 14 */

            if (pTxPos->TxPosFlowControl.TxPosProportionalFill == ixf6048_eTX_POS_FILL_LEN_MULT_2_EXP_N )
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_IPG_REL_CFG); /* Set Bit 13 for Multiply Factor */
            else /* ixf6048_eTX_POS_FILL_LEN_DIV_2_EXP_N  -- Ensure that Bit is Cleared*/
            Ixf6048ClrBitVal(RegVal, ixf6048_bXMT_IPG_REL_CFG); /* Not Necessary, but put here for Consistency */

            /* Reg T_IPGCTRL bits[12:10]-- The Dividing/Multiplying Factor*/
            if ( pTxPos->TxPosFlowControl.N == 0)
            {
                pTxPos->TxPosFlowControl.N = 1;   /* Force Default to 1 NOT 0 */
            }
            /* Convert N (The Dividing/Multiplying Factor) to 3 bit Setting */
            TempN = pTxPos->TxPosFlowControl.N;    /* Where N = 1, 2, 4, 8, .., 128 */

            for(XmtIpgRel=0; XmtIpgRel<= 7; ++XmtIpgRel)
                if(TempN & 0x01)  /* Count the Shifts to determine the IPGRel Value */
                    break;
                /* Masks bits 12:10 -- The Dividing/Multiplying Factor*/
                /* ixf6048_SET_XMT_IPG_REL(RegVal, XmtIpgRel); */
                SetRegBits(&RegVal, XmtIpgRel, 
                    ixf6048_mXMT_IPG_REL, ixf6048_BIT_SHIFT_10);
        }
        /* Reg T_IPGCTRL bits 9,8 -- Method 2*/
        if ( pTxPos->TxPosFlowControl.TxPosConstantFill != ixf6048_eTX_POS_NO_CONSTANT_FILL)
        {
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_IPG_ABS_EN);  /* Always Set Bit 9 */
            if (pTxPos->TxPosFlowControl.TxPosConstantFill == ixf6048_eTX_POS_FILL_MULT_256)
            {  /* NumInterframeFlags uses a scaling factor of 256 */
                Ixf6048SetBitVal(RegVal, ixf6048_bXMT_IPG_ABS_CNF);
                if(pTxPos->TxPosFlowControl.NumInterframeFlags == 256 )
                NumFlagsAbs = 0;  /* See Data Sheet, Table 27 */
                else
                    NumFlagsAbs  = (uchar) pTxPos->TxPosFlowControl.NumInterframeFlags;
            }
            else /* ixf6048_eTX_POS_FILL_MULT_1  -- Ensure that Bit is Cleared*/
            {  /* NumInterframeFlags uses a scaling factor of 1 */
                Ixf6048ClrBitVal(RegVal, ixf6048_bXMT_IPG_ABS_CNF); /* Note: this is not necessary, but placed here for consistency */
                if(pTxPos->TxPosFlowControl.NumInterframeFlags == 65536 )
                NumFlagsAbs = 0;  /* See Data Sheet, Table 28 */
                else
                {
                    /* Ensure that we do not divide by 0 */
                    if(pTxPos->TxPosFlowControl.NumInterframeFlags == 0)
                        /* Default to Min Value -- See Data Sheet Table 28 */
                        pTxPos->TxPosFlowControl.NumInterframeFlags = 256;

                    NumFlagsAbs  = (uchar)((pTxPos->TxPosFlowControl.NumInterframeFlags / HDLC_FLOW_CONTROL_SCALING_FACTOR_256));
                }
                /* Reg T_IPGCTRL bits[7:0] */
                /* ixf6048_SET_XMT_IPG_ABS(RegVal, NumFlagsAbs); */
                SetRegBits(&RegVal, NumFlagsAbs, 
                    ixf6048_mXMT_IPG_ABS, ixf6048_BIT_SHIFT_0);
            }
        }
        /* Now Set the new T_IPGCTRL Register Config Value */
        pReg = BaseAddress + ixf6048_T_IPGCTRL_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, 0);
    }


#ifdef UNIT_TEST_6048
    printf("Edit of POS Chan Cfg at 0x%x - NO VERIFICATION\n", pCfg);
#endif

    return bb_NO_ERROR;       /* should be bb_NO_ERROR */

}   /* end EditPosChanCfg */

/***********************************************************************
 * Procedure Name:   ixf6048_SetAlarmCfg
 *
 * Description: Edit Interrupt Enabled Masks for a Specific Set of Interrupts.
 *             
 *
 * Conditions for Use:
 *    pAlarmCfg is not NULL
 *    pChipCfg is not NULL and is a Valid Chip
 * Notes:
 *    
 *    This Public Function is used to re-configure a specific
 *    Set of Alarm Enables/Disables for a Single Channel.
 *
 *
 *
 **********************************************************************/
bb_Error_e ixf6048_SetAlarmCfg(bb_ChipData_t*   pChipData,
    uint             ChanNum,
    ixf6048_Side_e   Side,
    bb_AlarmType_e   AlarmType,
    void*            pAlarmCfg)
{
    bb_Error_e                  rc=bb_NO_ERROR;
    ushort*                     BaseAddress=pChipData->BaseAddress;
    ixf6048_OhtChanAlarmCfg_t*  pOhtChanAlms;
    ixf6048_UtopiaAlarmsCfg_t*  pUtopiaAlms;  /* UTOPIA Cfg Per Channel */
    ixf6048_AtmChanAlarmCfg_t*  pAtmChanAlms; /* Ignored, if not ATM Chan */
    ixf6048_PosChanAlarmCfg_t*  pPosChanAlms; /* Ignored, if not POS Chan */
    ixf6048_MiscAlarmsCfg_t*    pMiscAlms;    /* Combination of all Channels */

    if (pAlarmCfg == NULL)
    return bb_NULL_ADDRESS_ASSIGNED; /* We Have a Problem */

    switch(AlarmType)
    {
    case bb_6048_OHT_ALARM:    /* pAlarmStatus -> ixf6048_OhtAlarmStatus_t */
        pOhtChanAlms = pAlarmCfg;
        rc =  EditOhtChanAlarms(BaseAddress,ChanNum, 
            Side, pOhtChanAlms);
        /* Edit OHT alarm */
        break;
    case bb_6048_ATM_ALARM:    /* pAlarmStatus -> ixf6048_AtmAlarmStatus_t */
        pAtmChanAlms = pAlarmCfg;
        /* Edit ATM alarm */
        rc = EditAtmChanAlarms(BaseAddress,ChanNum,
            Side, pAtmChanAlms);
        break;
    case bb_6048_POS_ALARM:    /* pAlarmStatus -> ixf6048_PosAlarmStatus_t */
        pPosChanAlms = pAlarmCfg;
        /* Edit POS alarm */
        rc = EditPosChanAlarms(BaseAddress,ChanNum,
            Side, pPosChanAlms);
        break;
    case bb_6048_UTOPIA_ALARM: /* pAlarmStatus -> ixf6048_UtopiaAlarmStatus_t */
        pUtopiaAlms = pAlarmCfg;
        /* Edit UTOPIA alarm */
        rc = EditUtopiaChanAlarms(BaseAddress,ChanNum, 
            Side,pUtopiaAlms);
        break;
    case bb_6048_MISC_ALARM:    /* pAlarmStatus -> ixf6048_MiscAlarmStatus_t */
        pMiscAlms = pAlarmCfg;
        /* Edit MISC alarm */
        rc = EditMiscChanAlarms(BaseAddress,ChanNum,
            Side, pMiscAlms);
        break;
    default:
        rc = bb_NO_ALARM_CFG;
        break;
    }
    return rc;
}
/***********************************************************************
 * Procedure Name:   EditAlarms
 *
 * Description: Edit All Interrupts Enabled Masks
 *
 * Conditions for Use:
 *    pAlarmCfg is not NULL
 *    pChipCfg is not NULL and is a Valid Chip
 * Notes:
 *    This function is generally called in the Init Cycle to '
 *    reconfigure all of the Alarms.
 *
 *    
 **********************************************************************/
static bb_Error_e EditAlarms (ushort*             BaseAddress,
    ixf6048_Side_e      Side,
    ixf6048_ChipCfg_t*  pChipCfg,
    ixf6048_AlarmCfg_t* pAlarmCfg)
{
    bb_Error_e                 rc = bb_NO_ERROR;
    ixf6048_ChanAlarmCfg_t*     pChanAlarmCfg;
    ixf6048_OhtChanAlarmCfg_t*  pOhtChanAlms;
    ixf6048_UtopiaAlarmsCfg_t*  pUtopiaAlms;  /* UTOPIA Cfg Per Channel */
    ixf6048_AtmChanAlarmCfg_t*  pAtmChanAlms; /* Ignored, if not ATM Chan */
    ixf6048_PosChanAlarmCfg_t*  pPosChanAlms; /* Ignored, if not POS Chan */
    ixf6048_MiscAlarmsCfg_t*    pMiscAlms;    /* Combination of all Channels */
    int                        iMaxNumChans = SINGLE_CHANNEL_MODE;
    int                        ChanNum;

    /* Check for Qmode to Determine how many channels are assigned. */
    if(pChipCfg->NumChanCfg != ixf6048_eSINGLE_CHAN)
        iMaxNumChans = QUAD_CHANNEL_MODE;

    for (ChanNum=0; ChanNum < iMaxNumChans; ++ChanNum)
    {

        if (pAlarmCfg->pChanAlarmCfg[ChanNum] != NULL)
        {

            /* Edit the Alarm Configuration of Each Assigned Channel Both Rx and Tx*/
            pChanAlarmCfg =  pAlarmCfg->pChanAlarmCfg[ChanNum];

            /* OHT Alarms Configuration */
            pOhtChanAlms = pChanAlarmCfg->pOhtChanAlarmCfg;
            if(pOhtChanAlms != NULL)
                rc =  EditOhtChanAlarms(BaseAddress,ChanNum, 
                    Side, pOhtChanAlms);

            /* Utopia Alarms Configuration */
            pUtopiaAlms = pChanAlarmCfg->pUtopiaAlarmsCfg;
            if( (pUtopiaAlms != NULL )&&
                (rc == bb_NO_ERROR   ) )
                rc = EditUtopiaChanAlarms(BaseAddress,ChanNum, 
                    Side,pUtopiaAlms);

            /* ATM Alarms Configuration */
            pAtmChanAlms = pChanAlarmCfg->pAtmChanAlarmCfg;
            if( (pAtmChanAlms != NULL)&&
                (rc == bb_NO_ERROR   ) )
                rc = EditAtmChanAlarms(BaseAddress,ChanNum,
                    Side, pAtmChanAlms);

            /* POS Alarms Configuration */
            pPosChanAlms = pChanAlarmCfg->pPosChanAlarmCfg;
            if( (pPosChanAlms != NULL)&&
                (rc == bb_NO_ERROR   ) )
                rc = EditPosChanAlarms(BaseAddress,ChanNum,
                    Side, pPosChanAlms);

            /* MISC Alarms Configuration */
            pMiscAlms = pChanAlarmCfg->pMiscAlarmsCfg;
            if( (pMiscAlms != NULL) &&
                (rc == bb_NO_ERROR) )
                rc = EditMiscChanAlarms(BaseAddress,ChanNum,
                    Side, pMiscAlms);

            /* If we have a configuration problem, then Exit with Return Code */
            if (rc != bb_NO_ERROR)
                return rc;
        }
        else if (ChanNum == 0)
        {  /* If Channel 0 fails then there is a Problem so Exit */
            return bb_INVALID_CHAN_CFG;
        }

    }

#ifdef UNIT_TEST_6048
    printf("Edit Alarms at 0x%x - NO VERIFICATION\n", pCfg);
#endif

    return rc;

}   /* end EditAlarms */

/***********************************************************************
 * Procedure Name: EditOhtChanAlarms  
 *
 * Description: Edit Interrupt Enable Masks
 *			    This is Done by directly Setting the Configuration Passed 
 *				into the method using pOhtChanAlms.
 *
 * Conditions for Use:
 *    pOhtChanAlms is not NULL
 * Notes:
 *
 *
 *
 * End
 **********************************************************************/
bb_Error_e  EditOhtChanAlarms(ushort*                    BaseAddress,  
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_OhtChanAlarmCfg_t* pOhtChanAlms)
{
    bb_Error_e rc = bb_NO_ERROR;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        /* Rst Interrupt Enable Register */
        WRITE_TO_HW(BaseAddress + ixf6048_IE_RG_CHAN_OFFSET(ChanNum),
            (pOhtChanAlms->RxRstAlarms & ixf6048_mIE_RG));

        /*Mst Interrupt Enable Register */
        WRITE_TO_HW(BaseAddress + ixf6048_IE_MUX_CHAN_OFFSET(ChanNum),
            (pOhtChanAlms->RxMstAlarms & ixf6048_mIE_MUX));

        /*Msa Interrupt Enable Register */
        WRITE_TO_HW(BaseAddress + ixf6048_IE_ADP_CHAN_OFFSET(ChanNum),
            (pOhtChanAlms->RxMsaAlarms & ixf6048_mIE_ADP));

        /*Hpt Interrupt Enable Register */
        WRITE_TO_HW(BaseAddress + ixf6048_IE_HPT_CHAN_OFFSET(ChanNum),
            (pOhtChanAlms->RxHptAlarms & ixf6048_mIE_HPT));
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        /* Do Nothing -- Placed here for documentation only*/
    }

    return rc;
}
/***********************************************************************
 * Procedure Name: EditUtopiaChanAlarms  
 *
 * Description: Edit Interrupt Enable Masks on a Per Channel Basis:
 *
 * Conditions for Use:
 *    pUtopiaChanAlms is not NULL
 * Notes:
 *    This method has been written on a Per Channel Basis to  
 *    remain consistent with other functions. This function  
 *    would be more efficient if Called once for All Channels Combined.
 *    
 *    The Current Config MUST be cleared from the register prior to Setting 
 *	  the Config Passed using pUtopiaChanAlms.
 *
 *
 * End
 **********************************************************************/
bb_Error_e EditUtopiaChanAlarms(ushort*                    BaseAddress,  
    uint                       ChanNum,
    ixf6048_Side_e             Side,
    ixf6048_UtopiaAlarmsCfg_t* pUtopiaChanAlms)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pReg = BaseAddress + ixf6048_R_UTOINTEN_OFFSET;
        /* Mask and then Set the Rx Utopia Interrupt Enable Bits */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~(ixf6048_GET_R_UTOPIA_INT_CHAN_MASK(ChanNum))));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | (pUtopiaChanAlms->RxAlarms & ixf6048_GET_R_UTOPIA_INT_CHAN_MASK(ChanNum))));
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pReg = BaseAddress + ixf6048_T_UTOINTEN_OFFSET;
        /* Mask and then Set the Tx Utopia Interrupt Enable Bits */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~(ixf6048_GET_T_UTOPIA_INT_CHAN_MASK(ChanNum))));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | (pUtopiaChanAlms->TxAlarms & ixf6048_GET_T_UTOPIA_INT_CHAN_MASK(ChanNum))));
    }

    return rc;
}
/***********************************************************************
 * Procedure Name: EditAtmChanAlarms  
 *
 * Description: Edit Interrupt Enable Masks
 *
 * Conditions for Use:
 *    pAtmChanAlms is not NULL
 * Notes:
 *
 *
 *
 * End
 **********************************************************************/
bb_Error_e  EditAtmChanAlarms(ushort*                    BaseAddress,  
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_AtmChanAlarmCfg_t* pAtmChanAlms)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pReg = BaseAddress + ixf6048_R_ATMINTEN_CHAN_OFFSET(ChanNum);
        /* Set the Receive Atm Interrupt Enable Bits */
        WRITE_TO_HW(pReg, pAtmChanAlms->RxAlarms & ixf6048_mR_ATM_INTERRUPT_EN);
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pReg = BaseAddress + ixf6048_T_ATMINTEN_CHAN_OFFSET(ChanNum);
        /* Set the Transmit Atm Interrupt Enable Bits */
        WRITE_TO_HW(pReg, pAtmChanAlms->TxAlarms & ixf6048_mT_ATM_INTERRUPT);

    }

    return rc;
}
/***********************************************************************
 * Procedure Name: EditPosChanAlarms  
 *
 * Description: Edit Interrupt Enable Masks
 *
 * Conditions for Use:
 *    pPosChanAlms is not NULL
 * Notes:
 *
 *
 *
 * End
 **********************************************************************/
bb_Error_e  EditPosChanAlarms(ushort*                    BaseAddress,  
    uint                       ChanNum, 
    ixf6048_Side_e             Side,
    ixf6048_PosChanAlarmCfg_t* pPosChanAlms)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pReg = BaseAddress + ixf6048_R_POSINTEN_CHAN_OFFSET(ChanNum);
        /* Mask and then Set the Receive Pos Interrupt Enable Bits */
        WRITE_TO_HW(pReg, pPosChanAlms->RxAlarms & ixf6048_mRX_POS_INTERRUPT);
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pReg = BaseAddress + ixf6048_T_POSINTEN_CHAN_OFFSET(ChanNum);
        /* Mask and then Set the Transmit Pos Interrupt Enable Bits */
        WRITE_TO_HW(pReg, pPosChanAlms->TxAlarms & ixf6048_mTX_POS_INTERRUPT);
    }

    return rc;
}
/***********************************************************************
 * Procedure Name: EditMiscChanAlarms  
 *
 * Description: Edit Interrupt Enable Masks
 *
 * Conditions for Use:
 *    pMiscChanAlms is not NULL
 * Notes:
 *    Although these interupt are located in the Global Sources register,
 *    they are edited on a channel basis to remain consistent with
 *    the other Alarm Enable functions. These could be done more efficiently
 *    using a single Edit function call. 
 *
 *
 * End
 **********************************************************************/
bb_Error_e  EditMiscChanAlarms(ushort*                  BaseAddress,  
    uint                     ChanNum, 
    ixf6048_Side_e           Side,
    ixf6048_MiscAlarmsCfg_t* pMiscChanAlms)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;

    if( (Side == ixf6048_eSIDE_BOTH   )||
        (Side == ixf6048_eSIDE_RECEIVE) )
    {
        pReg = BaseAddress + ixf6048_PRBSINTEN_OFFSET;
        /*Mask Bits and then Set --  Reg PRBSI 3:0  */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_bPRBSI(ChanNum)));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | (pMiscChanAlms->PRBSAlarms & ixf6048_bPRBSI(ChanNum))));

        /* Receive Parity Bit only Set on Channel 0 */
        if (ChanNum == 0)
        {
            pReg = BaseAddress + ixf6048_LSPINTEN_OFFSET;
            /* Set/Clear Interrupt Enable bit -- LSPINT Bit 0 */
            if (pMiscChanAlms->LineSideParityAlarm & ixf6048_mLSP_INT)
                Ixf6048SetBit(pReg,ixf6048_mLSP_INT);
            else
                Ixf6048ClrBit(pReg,ixf6048_mLSP_INT);
        }
    }

    if( (Side == ixf6048_eSIDE_BOTH    )||
        (Side == ixf6048_eSIDE_TRANSMIT) )
    {
        pReg = BaseAddress + ixf6048_TALBINTEN_OFFSET;
        /* Mask Bits and then Set -- Reg TALBINT 15:0*/
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~(ixf6048_GET_TALBINTEN_CHAN_INT_MASK(ChanNum))));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | (pMiscChanAlms->TransmitAlarmBusAlarms & ixf6048_GET_TALBINTEN_CHAN_INT_MASK(ChanNum))));
    }

    return rc;
}

/***********************************************************************
 * Procedure Name: ixf6048_ChipIsr
 *
 * Description: Amazon-A Interrupt Service Routine
 *
 * Conditions for Use:
 *    BaseAddress must be a valid chip Address
 *
 * Notes:
 *   Interrupts are reported through a Callback function.
 *
 *	 Transmit and receive Interrupts act independantly of each another.
 *
 *   Interrupts have been prioritized to be handled according to the severity 
 *   of the effecting interupt. See the MasterInterruptHandler function for 
 *	 specifics
 *
 *
 * End
 **********************************************************************/
void ixf6048_ChipIsr(bb_ChipData_t *pChipData)
{
    ixf6048_GlobalAlarmCfg_t GlobalAlarmCfg;
    register ushort*  pReg;
    register ushort*  BaseAddress = pChipData->BaseAddress;

    if (pAlarmCallback == NULL)
    {
        return;
    }

    /* Check to see if Master Interrupt bit is enabled */
    pReg =  BaseAddress + ixf6048_MACNF_OFFSET;
    /* Don't look at the Master Interrupt Enable for now */
#if 0
    if (!(READ_FROM_HW(pReg) & ixf6048_bMAS_INT_EN))
        return;  /* Interrupts have been disabled, so do nothing */
#endif

    /* Check the Global Interrupt Bits to determine if any Interrupts are active. */

    /* Begin with the SDH Global Interrupt Source Register */
    GlobalAlarmCfg.OhtGlobalAlarms = READ_FROM_HW(BaseAddress + ixf6048_SDH_GIS_OFFSET);

    /* Check the ATM and POS Global Interrupt Source Register */
    GlobalAlarmCfg.AtmPosGlobalAlarms = READ_FROM_HW(BaseAddress + ixf6048_ATMPOS_GIS_OFFSET);

    /* Check the MISC Global Interrupt Source Register */
    GlobalAlarmCfg.MiscGlobalAlarms = READ_FROM_HW(BaseAddress + ixf6048_MISC_GIS_OFFSET);

    if( (GlobalAlarmCfg.OhtGlobalAlarms   )||
        (GlobalAlarmCfg.AtmPosGlobalAlarms)||
        (GlobalAlarmCfg.MiscGlobalAlarms  ) )
    {
        /* There Are Alarms Active */
        MasterInterruptHandler(BaseAddress, &GlobalAlarmCfg);
    }
    /* else do nothing */
}

/***********************************************************************
 * Procedure Name:  MasterInterruptHandler
 *
 * Description: 
 *      Determine which Interrupts are active and return Status
 *      bit to the Caller.
 *
 * Conditions for Use:
 *      An Interrupt has been indicated in one or more 
 *      Global Int Source registers.
 *      pAlarmCallback must be a Valid Function Pointer       
 *
 * Notes:
 *      Cycle Through Each Channel and
 *      only Pass the Status/Alarm Bits for Enabled Interrupts
 *
 *      R_COCNF and T_COCNF Rcv/Xmt ChMode bits hold the 
 *      operational mode per channel however the bits are not checked as 
 *      the structures are NULLed out if not used. 
 *
 *      The ActiveInterruptEffect holds the severity of an Interrupt that 
 *      has been indentified. If a Catastrophic Interrupt Occurs, then 
 *      Do not process other Alarms that are caused in the Domino effect.
 *         -- This only applies to the receive side.
 * End
 **********************************************************************/
static void MasterInterruptHandler( ushort*           BaseAddress,
    ixf6048_GlobalAlarmCfg_t *pGlobalAlarmCfg)
{
    ixf6048_OhtGlobalAlarms    *pOhtGlobalAlarms;     /* Global SDH Alarms Indicator */
    ixf6048_AtmPosGlobalAlarms *pAtmPosGlobalAlarms;  /* Global ATM Alarms Indicator */
    ixf6048_MiscGlobalAlarms   *pMiscGlobalAlarms;    /* Global POS Alarms Indicator */

    ushort                     ChanNum;
    ushort                     NumChannels;
    ushort                     GlobalAlmChan = 0;
    ActiveInterruptEffect_e     EffectingInterrupt = eINT_EFFECT_NONE;
    ushort                     ChanEna = 0;
    boolean                    ReadUtopiaIntsFlag =TRUE;
    boolean                    ReadMiscIntsFlag =TRUE;

    /* Use Local members for Global Alarm Indications */
    pOhtGlobalAlarms    = &pGlobalAlarmCfg->OhtGlobalAlarms;
    pAtmPosGlobalAlarms = &pGlobalAlarmCfg->AtmPosGlobalAlarms;
    pMiscGlobalAlarms   = &pGlobalAlarmCfg->MiscGlobalAlarms;

    /* Check the GOCNF QMode Bit to determine the Number of Channels (4 or 1)*/
    NumChannels = (READ_FROM_HW(BaseAddress + ixf6048_GOCNF_OFFSET) & ixf6048_bQUAD_MODE) ?
        MAX_NUM_CHANNELS: 1;

    for (ChanNum=0; ChanNum < NumChannels; ++ChanNum)
    {
        /* Check the COCNF Register to determine if this Channel is enabled (ChEna Bit) */
        ChanEna = READ_FROM_HW(BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum)) & ixf6048_bCH_ENA;
        if(ChanEna)
        {
            EffectingInterrupt = eINT_EFFECT_NONE; /* Reset Per Channel */
            /* Are there any OHT alarms Active */
            if(*pOhtGlobalAlarms)
            {   /* Determine if there are SDH Alarms Active on This Channel */
                GlobalAlmChan = *pOhtGlobalAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum];
                if (GlobalAlmChan)
                    SdhInterruptHandler(BaseAddress, ChanNum, GlobalAlmChan, &EffectingInterrupt);
            }

            /* Are there any ATM Alarms Active */
            if(*pAtmPosGlobalAlarms & ixf6048_mATM_ALARMS)
            {   /* Determine if there is an ATM Alarm Active on This Channel */
                GlobalAlmChan = *pAtmPosGlobalAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum];
                if (GlobalAlmChan)
                    AtmInterruptHandler(BaseAddress, ChanNum, GlobalAlmChan, &EffectingInterrupt);
            }

            /* Are there any POS Alarms Active */
            if(*pAtmPosGlobalAlarms & ixf6048_mPOS_ALARMS)
            {   /* Determine if there are POS Alarms Active on This Channel */
                GlobalAlmChan = *pAtmPosGlobalAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum];
                if (GlobalAlmChan)
                    PosInterruptHandler(BaseAddress, ChanNum, GlobalAlmChan, &EffectingInterrupt);
            }

            /* Note that the MISC and Utopia Alarms require special handling to fit 
            into the above Channel based reporting scheme. */
            /* Are there any MISC/UTOPIA Alarms Active -- */
            if (*pMiscGlobalAlarms)
            {   /* Check for any Active Utopia Alarms Per Channel */
                if (*pMiscGlobalAlarms & ixf6048_mUTOPIA_ALARMS)
                {
                    UtopiaInterruptHandler(BaseAddress, 
                        ChanNum,
                        *pMiscGlobalAlarms, 
                        &ReadUtopiaIntsFlag,
                        &EffectingInterrupt);
                }
                if (*pMiscGlobalAlarms & ixf6048_mMISC_ALARMS)
                {
                    MiscInterruptHandler(BaseAddress, 
                        ChanNum,
                        *pMiscGlobalAlarms, 
                        &ReadMiscIntsFlag,
                        &EffectingInterrupt);
                }
            }
        }
    }
}
/***********************************************************************
 * Procedure Name: SdhInterruptHandler
 *
 * Description: handles Interrupts for a Single Channel.
 *
 * Conditions for Use:
 *    GlobalAlmChan contains the Global Interrupt 
 *    Indication for a Single Channel (ChanNum). 
 *    
 *
 * Notes:
 *    OhtAlarms will contain the current condition of all 
 *    active SDH Alarms Status Bits for a single Channel.
 *    Mask Int Bits with IntEn Bits to determine which Alarms 
 *    we are interested in. Load the Corresponding status Bits.
 *
 * Returns: OhtAlarms to the AlarmCallback Function
 *          *EffectingInterrupt  Does this interupt Cause a Cascade of Interrupts? 
 *                        
 * End
 **********************************************************************/
static void SdhInterruptHandler(ushort*  BaseAddress,   /* Base address of chip */
    ushort   ChanNum,
    ushort   GlobalAlmChan,
    ActiveInterruptEffect_e* EffectingInterrupt)
{
    ixf6048_OhtAlarmStatus_t OhtAlarms;   /* Alarm Source/Status Bits for All OHT INTs */
    register ushort          k1k2RegVal;
    ushort                   uAlarmsStatus;
    register ushort          uS_AIS_Reg;

    /* Clear OhtAlarms Block */
    memset(&OhtAlarms, 0, sizeof(ixf6048_OhtAlarmStatus_t));

    /* Indicate the channel */
    OhtAlarms.Channel = ChanNum;

    /* Read the AIS Status Register and Mask for This Channel */
    uS_AIS_Reg = READ_FROM_HW(BaseAddress + ixf6048_S_AIS_OFFSET) & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum];
    /*RST Alarms */
    if(GlobalAlmChan & ixf6048_mRCV_RG_INT)    /* RST */
    {
        OhtAlarms.RxRst.Alarms =
            READ_FROM_HW(BaseAddress + ixf6048_IS_RG_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_IE_RG_CHAN_OFFSET(ChanNum)) &
            ixf6048_mIS_RG;
        /* Read the entire Status Register */
        OhtAlarms.RxRst.Status = READ_FROM_HW(BaseAddress + ixf6048_S_RG_CHAN_OFFSET(ChanNum)) &
            ixf6048_mS_RG;

        uAlarmsStatus = OhtAlarms.RxRst.Alarms & OhtAlarms.RxRst.Status;
        /* Now Check for the status of the Effecting Interrupt. If there is an
        effect-causing alarm, then do not process any other Interrupts for this
        Channel  */
        if ( (uAlarmsStatus & ixf6048_eRX_RST_LOS_ALM   )||   /* Loss Of Signal */
            (uAlarmsStatus & ixf6048_eRX_RST_LO_CLK_ALM)||   /* Loss of Clocks */
            (uAlarmsStatus & ixf6048_eRX_RST_LOF_ALM   ) )   /* Loss of Frame  */
            *EffectingInterrupt = eINT_EFFECT_CATASTROPHIC;
        else
        {
            /* Get Interrupt Effecting Counters */
            if (OhtAlarms.RxRst.Alarms & ixf6048_eRX_RST_OOF_OVFL_ALM)
            {   /* Write to Register to Load Counter*/
                WRITE_TO_HW(BaseAddress + ixf6048_OOF_ECNT_CHAN_OFFSET(ChanNum), 0);
                OhtAlarms.RxRst.OofCntr = ixf6048_GET_OOF_CNT(BaseAddress,ChanNum);
            }
            if (OhtAlarms.RxRst.Alarms & ixf6048_eRX_RST_B1_OVFL_ALM)
            {   /* Write to Register to Load Counter*/
                WRITE_TO_HW(BaseAddress + ixf6048_B1_ERRCNT_CHAN_OFFSET(ChanNum), 0);
                OhtAlarms.RxRst.B1Cntr  = ixf6048_GET_B1_ERRCNT(BaseAddress, ChanNum);
            }
            /* Check the GenRstAis status bit for this Channel. If active then 
            do not process any other Interrupts for this Channel */
            if (uS_AIS_Reg & ixf6048_GEN_RST_AIS_ST)
            {
                OhtAlarms.AisStatus = uS_AIS_Reg & ixf6048_GEN_RST_AIS_ST;
                *EffectingInterrupt = eINT_EFFECT_MAJOR;
            }
        }
    }
    /* MST Alarms Mask*/
    if( (*EffectingInterrupt == eINT_EFFECT_NONE)&&
        (GlobalAlmChan & ixf6048_mRCV_MUX_INT   ) )  /* MST */
    {
        OhtAlarms.RxMst.Alarms =
            READ_FROM_HW(BaseAddress + ixf6048_IS_MUX_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_IE_MUX_CHAN_OFFSET(ChanNum))&
            ixf6048_mIS_MUX;
        OhtAlarms.RxMst.Status = READ_FROM_HW(BaseAddress + ixf6048_S_MUX_CHAN_OFFSET(ChanNum)) &
            ixf6048_mS_MUX;

        /* Get Interrupt Effecting Counters */
        if (OhtAlarms.RxMst.Alarms & (ixf6048_eRX_MST_K1_ALM | ixf6048_eRX_MST_K2_ALM) )
        {
            k1k2RegVal = ixf6048_GET_K1K2(BaseAddress,ChanNum);
            OhtAlarms.RxMst.MstBytes.K1 = ixf6048_GET_K1(k1k2RegVal);
            OhtAlarms.RxMst.MstBytes.K2 = ixf6048_GET_K2(k1k2RegVal);
        }

        if (OhtAlarms.RxMst.Alarms & ixf6048_eRX_MST_S1_ALM)
            OhtAlarms.RxMst.MstBytes.S1 = ixf6048_GET_S1(BaseAddress,ChanNum);

        if (OhtAlarms.RxMst.Alarms & ixf6048_eRX_MST_B2_BIT_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_B2_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            OhtAlarms.RxMst.B2BitCntr = ixf6048_GET_B2_BIPCNT(BaseAddress,ChanNum);
        }
        if (OhtAlarms.RxMst.Alarms & ixf6048_eRX_MST_REI_BIT_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_MR_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            OhtAlarms.RxMst.ReiBitCntr = ixf6048_GET_MST_REI_BIPCNT(BaseAddress,ChanNum);
        }
        if (OhtAlarms.RxMst.Alarms & ixf6048_eRX_MST_B2_BLK_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_B2_BLKCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            OhtAlarms.RxMst.B2BlkCntr = ixf6048_GET_B2_BLKCNT(BaseAddress, ChanNum);
        }
        if (OhtAlarms.RxMst.Alarms & ixf6048_eRX_MST_REI_BLK_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_MR_BLKCNT_CHAN_LSB_OFFSET(ChanNum), 0);
            OhtAlarms.RxMst.ReiBlkCntr = ixf6048_GET_MST_REI_BLKCNT(BaseAddress,ChanNum);
        }
        /* Check the GenMstAis status bit for this Channel. If active then 
        do not process any other Interrupts for this Channel */
        if (uS_AIS_Reg & ixf6048_GEN_MST_AIS_ST)
        {
            OhtAlarms.AisStatus = uS_AIS_Reg & ixf6048_GEN_MST_AIS_ST;
            *EffectingInterrupt = eINT_EFFECT_MAJOR;
        }
    }

    /* MSA Alarms Mask*/
    if( (*EffectingInterrupt == eINT_EFFECT_NONE)&&
        (GlobalAlmChan & ixf6048_mRCV_ADP_INT   ) )  /* MSA */
    {
        OhtAlarms.RxMsa.Alarms =
            READ_FROM_HW(BaseAddress + ixf6048_IS_ADP_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_IE_ADP_CHAN_OFFSET(ChanNum)) &
            ixf6048_mIS_ADP;
        OhtAlarms.RxMsa.Status = READ_FROM_HW(BaseAddress + ixf6048_S_ADP_CHAN_OFFSET(ChanNum)) &
            ixf6048_mS_ADP;

        uAlarmsStatus = OhtAlarms.RxRst.Alarms & OhtAlarms.RxRst.Status;
        /* Now Check for the status of the Effecting Interrupt. If there is an 
        effect-causing alarm, then do not process any other Interrupts for this
        Channel  */
        if( (uAlarmsStatus & ixf6048_eRX_MSA_LOP_ALM    ) ||
            (uAlarmsStatus & ixf6048_eRX_MSA_AU_AIS_ALM ) )
            *EffectingInterrupt = eINT_EFFECT_CATASTROPHIC;
        else
        {
            /* Get Interrupt Effecting Counters */
            if (OhtAlarms.RxMsa.Alarms & ixf6048_eRX_MSA_AU_NEG_OVFL_ALM)
            {   /* Write to Register to Load Counter*/
                WRITE_TO_HW(BaseAddress + ixf6048_R_AU_NCNT_CHAN_OFFSET(ChanNum), 0);
                OhtAlarms.RxMsa.AuNegJustCntr = ixf6048_GET_R_AU_NCNT(BaseAddress,ChanNum);
            }
            if (OhtAlarms.RxMsa.Alarms & ixf6048_eRX_MSA_AU_POS_OVFL_ALM)
            {   /* Write to Register to Load Counter*/
                WRITE_TO_HW(BaseAddress + ixf6048_R_AU_PCNT_CHAN_OFFSET(ChanNum), 0);
                OhtAlarms.RxMsa.AuPosJustCntr = ixf6048_GET_R_AU_PCNT(BaseAddress,ChanNum);
            }
        }
        /* Check the GenMsaAis status bit for this Channel. If active then 
        do not process any other Interrupts for this Channel */
        if (uS_AIS_Reg & ixf6048_GEN_MSA_AIS_ST)
        {
            OhtAlarms.AisStatus = uS_AIS_Reg & ixf6048_GEN_MSA_AIS_ST;
            *EffectingInterrupt = eINT_EFFECT_MAJOR;
        }
    }

    /* Rx HPT Alarms Mask*/
    if( (*EffectingInterrupt == eINT_EFFECT_NONE)&&
        (GlobalAlmChan & ixf6048_mRCV_HPT_INT   ) )
    {
        OhtAlarms.RxHpt.Alarms =
            READ_FROM_HW(BaseAddress + ixf6048_IS_HPT_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_IE_HPT_CHAN_OFFSET(ChanNum))&
            ixf6048_mIS_HPT;
        /* Read the entire Status Register */
        OhtAlarms.RxHpt.Status = READ_FROM_HW(BaseAddress + ixf6048_S_HPT_CHAN_OFFSET(ChanNum)) &
            ixf6048_mS_HPT;

        /* Only Read the Interrupt-Causing Registers */
        if (OhtAlarms.RxHpt.Alarms & ixf6048_eRX_HPT_RDI_ALM)
            OhtAlarms.RxHpt.HptBytes.Rdi= ixf6048_GET_RX_HPT_RDI(BaseAddress,ChanNum);

        if (OhtAlarms.RxHpt.Alarms & ixf6048_eRX_HPT_C2_ALM)
            OhtAlarms.RxHpt.HptBytes.C2 = ixf6048_GET_RX_C2(BaseAddress,ChanNum);

        if (OhtAlarms.RxHpt.Alarms & ixf6048_eRX_HPT_B3_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_B3_ECNT_CHAN_OFFSET(ChanNum), 0);
            OhtAlarms.RxHpt.B3Cntr = ixf6048_GET_B3_ERRCNT(BaseAddress,ChanNum);
        }
        if (OhtAlarms.RxHpt.Alarms & ixf6048_eRX_HPT_REI_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_HPTREI_CNT_CHAN_OFFSET(ChanNum), 0);
            OhtAlarms.RxHpt.ReiCntr = ixf6048_GET_HPT_REI_CNT(BaseAddress,ChanNum);
        }
        /* Check the GenHptAis status bit for this Channel. If active then 
        do not process any other Interrupts for this Channel */
        if (uS_AIS_Reg & ixf6048_GEN_HPT_AIS_ST)
        {
            OhtAlarms.AisStatus = uS_AIS_Reg & ixf6048_GEN_HPT_AIS_ST;
            *EffectingInterrupt = eINT_EFFECT_MAJOR;
        }
    }

    /*There are Alarms active, so use the Callback on a Per Channel Basis */
    pAlarmCallback(BaseAddress, bb_6048_OHT_ALARM, &OhtAlarms);
}
/***********************************************************************
 * Procedure Name: AtmInterruptHandler
 *
 * Description: handles Interrupts for a Single Channel.
 *
 * Conditions for Use:
 *    GlobalAlmChan contains the Global Interrupt 
 *    Indication for a Single Channel (ChanNum). 
 *    
 *
 * Notes:
 *    AtmAlarms will contain the current condition of all 
 *    active ATM Alarms Status Bits for a single Channel.
 *    Mask Int Bits with IntEn Bits to determine which Alarms 
 *    we are interested in. Load the Corresponding status Bits.
 *
 * Returns: AtmAlarms to the AlarmCallback Function
 *          *EffectingInterrupt  Does this interupt Cause a Cascade of Interrupts? 
 *
 * End
 **********************************************************************/
static void AtmInterruptHandler(ushort*  BaseAddress,
    ushort   ChanNum,
    ushort   GlobalAlmChan,
    ActiveInterruptEffect_e* EffectingInterrupt)

{
    ixf6048_AtmAlarmStatus_t AtmAlarm;   /* Alarm Source/Status Bits for All ATM INTs */
    register ushort          uRegVal;
    boolean                  InterruptsToReport = FALSE;

    /* Clear the Atm Alarms Structure */
    memset(&AtmAlarm,0,sizeof(ixf6048_AtmAlarmStatus_t));
    /* Indicate the channel */
    AtmAlarm.Channel = ChanNum;

    /* Rx ATM Alarms Mask -- Only Process if No Catastrophic Ints have occured*/
    if( (*EffectingInterrupt == eINT_EFFECT_NONE) &&
        (GlobalAlmChan & ixf6048_mRCV_ATM_INT   ) )
    {
        InterruptsToReport = TRUE;
        /* Read Interrupt Reg once */
        uRegVal= READ_FROM_HW(BaseAddress + ixf6048_R_ATMINT_CHAN_OFFSET(ChanNum));
        AtmAlarm.RxAlarms = uRegVal &
            READ_FROM_HW(BaseAddress + ixf6048_R_ATMINTEN_CHAN_OFFSET(ChanNum)) &
            ixf6048_mR_ATM_INTERRUPT_EN;
        /* Load all of the Status Bits */
        AtmAlarm.RxStatus = uRegVal & ixf6048_mRCV_ATM_INT_STATUS_BITS;

        /* Only Load Interrupt Causing Counters */
        /* Rx Accepted ATM Cell counter */
        if (AtmAlarm.RxAlarms & ixf6048_eRX_ATM_CELL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_ACELLCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            AtmAlarm.RxAtmCellCntr = ixf6048_GET_RX_ATM_CELL_CNT(BaseAddress,ChanNum);
        }
        /* Rx Idle Cell counter */
        if (AtmAlarm.RxAlarms & ixf6048_eRX_ATM_IDLE_CELL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_ICELLCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            AtmAlarm.RxIdleCellCntr = ixf6048_GET_RX_IDLE_CELL_CNT(BaseAddress,ChanNum);
        }
        /* Rx Correctable HEC error counter */
        if (AtmAlarm.RxAlarms & ixf6048_eRX_ATM_COR_HEC_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_CHECNT_CHAN_OFFSET(ChanNum), 0);
            AtmAlarm.RxCorHecCntr = ixf6048_GET_COR_HEC_CNT(BaseAddress,ChanNum);
        }
        /* Rx Uncorrectable HEC error counter */
        if (AtmAlarm.RxAlarms & ixf6048_eRX_ATM_UNCOR_HEC_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_UHECNT_CHAN_OFFSET(ChanNum), 0);
            AtmAlarm.RxUncorHecCntr = ixf6048_GET_UNCOR_HEC_CNT(BaseAddress,ChanNum);
        }
        /* Cnt of OK Rx Cells lost due to Rx FIFO Ovfl */
        if (AtmAlarm.RxAlarms & ixf6048_eRX_ATM_FIFO_OVFL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_CFOCNT_CHAN_OFFSET(ChanNum), 0);
            AtmAlarm.RxFifoOvflCntr = ixf6048_GET_FIFO_OFLOW_CNT(BaseAddress,ChanNum);
        }
    }

    /* Tx ATM Alarms Mask  -- Always Process Tx Interrupts*/
    if(GlobalAlmChan & ixf6048_mXMT_ATM_INT)
    {
        InterruptsToReport = TRUE;
        AtmAlarm.TxAlarms = READ_FROM_HW(BaseAddress + ixf6048_T_ATMINT_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_T_ATMINTEN_CHAN_OFFSET(ChanNum))&
            ixf6048_mT_ATM_INTERRUPT;

        /* Only Load Interrupt Causing Counters */

        /* Tx ATM Cell counter */
        if (AtmAlarm.TxAlarms &ixf6048_eTX_ATM_CELL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_ACELLCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            AtmAlarm.TxAtmCellCntr = ixf6048_GET_TX_ATM_CELL_CNT(BaseAddress,ChanNum);
        }
        /* Tx Idle Cell counter */
        if (AtmAlarm.TxAlarms & ixf6048_eTX_ATM_IDLE_CELL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_ICELLCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            AtmAlarm.TxIdleCellCntr= ixf6048_GET_TX_IDLE_CELL_CNT(BaseAddress,ChanNum);
        }
    }

    if (InterruptsToReport == TRUE)
        /* there are Alarms active, so use the Callback on a Per Channel Basis */
        pAlarmCallback(BaseAddress, bb_6048_ATM_ALARM, &AtmAlarm);

}
/***********************************************************************
 * Procedure Name: PosInterruptHandler
 *
 * Description: handles Interrupts for a Single Channel.
 *
 * Conditions for Use:
 *    GlobalAlmChan contains the Global Interrupt 
 *    Indication for a Single Channel (ChanNum). 
 *    
 *
 * Notes:
 *    PosAlarms will contain the current condition of all 
 *    active POS Alarms Status Bits for a single Channel.
 *    Mask Int Bits with IntEn Bits to determine which Alarms 
 *    we are interested in. Load the Corresponding status Bits.
 *
 * Returns: PosAlarms to the AlarmCallback Function
 *          *EffectingInterrupt  Does this interupt Cause a Cascade of Interrupts? 
 *
 * End
 **********************************************************************/
static void PosInterruptHandler(ushort*  BaseAddress,
    ushort   ChanNum,
    ushort   GlobalAlmChan, 
    ActiveInterruptEffect_e* EffectingInterrupt)

{
    ixf6048_PosAlarmStatus_t PosAlarm;   /* Alarm Source/Status Bits for All POS INTs */
    boolean                  InterruptsToReport = FALSE;

    /* Clear the Pos Alarms Structure */
    memset(&PosAlarm,0,sizeof(ixf6048_PosAlarmStatus_t));
    /* Indicate the channel */
    PosAlarm.Channel = ChanNum;

    /* Rx ATM Alarms Mask -- Only Process if No Catastrophic Alarms have occured*/
    if( ( *EffectingInterrupt == eINT_EFFECT_NONE) &&
        ( GlobalAlmChan & ixf6048_mRCV_POS_INT   ) )
    {
        InterruptsToReport = TRUE;
        /* Read Interrupt Reg once */
        PosAlarm.RxAlarms = READ_FROM_HW(BaseAddress + ixf6048_R_POSINT_CHAN_OFFSET(ChanNum))&
            READ_FROM_HW(BaseAddress + ixf6048_R_POSINTEN_CHAN_OFFSET(ChanNum)) &
            ixf6048_mRX_POS_INTERRUPT;
        /* Load the Associated Status Registers */
        /* Note: Only Load Interrupt Causing Counters */

        /* Rx Accepted Frame counter */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_FRAME_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_FRMCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.RxFrameCntr    = ixf6048_GET_RX_FRAME_CNT(BaseAddress,ChanNum);
        }
        /* Rx Accepted Byte counter */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_BYTE_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_BYTECNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.RxByteCntr     = ixf6048_GET_RX_BYTE_CNT(BaseAddress,ChanNum);
        }
        /* Rx Aborted Frame counter */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_ABORT_FRAME_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_AFCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.RxAbortFrameCntr=ixf6048_GET_RX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
        }
        /* Rx FCS Error counter */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_FCS_ERR_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_FCSECNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.RxFcsErrCntr   = ixf6048_GET_RX_FCS_ERROR_CNT(BaseAddress,ChanNum);
        }
        /* Cnt of Rx Frames lost to Rx FIFO Ovfl */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_FIFO_OVFL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_PFOCNT_CHAN_OFFSET(ChanNum), 0);
            PosAlarm.RxFifoOvflCntr = ixf6048_GET_RX_FIFO_OVERFLOW_COUNTER(BaseAddress,ChanNum);
        }
        /* Cnt of Rx Frames shorter than Min Len */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_MIN_PKT_LEN_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_MINPLECNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.RxShortFrameCntr=ixf6048_GET_RX_MIN_PACKET_LEN_ERR_CNT(BaseAddress,ChanNum);
        }
        /* Cnt of Rx Frames longer than Max len */
        if(PosAlarm.RxAlarms & ixf6048_eRX_POS_MAX_PKT_LEN_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_R_MAXPLECNT_CHAN_OFFSET(ChanNum), 0);
            PosAlarm.RxLongFrameCntr= ixf6048_GET_RX_MAX_PACKET_LEN_ERR_CNT(BaseAddress, ChanNum);
        }
    }

    /* Tx ATM Alarms Mask -- Always Process Tx Interrupts*/
    if(GlobalAlmChan & ixf6048_mXMT_POS_INT)
    {
        InterruptsToReport = TRUE;
        /* Read Interrupt Reg once */
        PosAlarm.TxAlarms = READ_FROM_HW(BaseAddress + ixf6048_T_POSINT_CHAN_OFFSET(ChanNum)) &
            READ_FROM_HW(BaseAddress + ixf6048_T_POSINTEN_CHAN_OFFSET(ChanNum))&
            ixf6048_mTX_POS_INTERRUPT;
        /* Load the Associated Status Registers */
        /* Note: Only Load Interrupt Causing Counters */
        /* Tx Frame counter */
        if(PosAlarm.TxAlarms & ixf6048_eTX_POS_FRAME_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_FRMCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.TxFrameCntr    = ixf6048_GET_TX_FRAME_CNT(BaseAddress,ChanNum);
        }
        /* Tx Byte counter */
        if(PosAlarm.TxAlarms & ixf6048_eTX_POS_BYTE_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_BYTECNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.TxByteCntr     = ixf6048_GET_TX_BYTE_CNT(BaseAddress,ChanNum);
        }
        /* Tx Aborted (by Link Layer) Frame counter */
        if(PosAlarm.TxAlarms & ixf6048_eTX_POS_ABORT_FRAME_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_AFCNT_CHAN_MSB_OFFSET(ChanNum), 0);
            PosAlarm.TxAbortFrameCntr=ixf6048_GET_TX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
        }
        /* Cnt of Tx Frames Aborted on Tx FIFO Underfl */
        if(PosAlarm.TxAlarms & ixf6048_eTX_POS_FIFO_UNFL_CNTR_OVFL_ALM)
        {   /* Write to Register to Load Counter*/
            WRITE_TO_HW(BaseAddress + ixf6048_T_PFUCNT_CHAN_OFFSET(ChanNum), 0);
            PosAlarm.TxFifoUnflCntr = ixf6048_GET_TX_FIFO_UFLOW_COUNT(BaseAddress, ChanNum);
        }
    }

    if(InterruptsToReport == TRUE)
        /* there are Alarms active, so use the Callback on a Per Channel Basis */
        pAlarmCallback(BaseAddress, bb_6048_POS_ALARM, &PosAlarm);

}
/***********************************************************************
 * Procedure Name: UtopiaInterruptHandler
 *
 * Description: handles Interrupts for a Single Channel.
 *
 * Conditions for Use:
 *    MiscGlobalAlarms contains the Global Interrupt Indication
 *    that a Utopia Related Alarm Has Occured
 *    
 *
 * Notes:
 *    UtopiaAlarmStatus will contain the current condition of all 
 *    active Utopia Alarms Status Bits.
 *    This Function is NOT channel Based
 *
 * Returns: UtopiaAlarms        Structure is Passed to Alarm Callback Func      
 *          ReadIntFlag         Has the Utopia Interrupt Register been Read?
 *          *EffectingInterrupt  Does this interupt Cause a Cascade of Interrupts? 
 * End
 **********************************************************************/
static void UtopiaInterruptHandler(ushort*  BaseAddress,   
    ushort   ChanNum,
    ushort   MiscGlobalAlarms,
    boolean *ReadIntFlag,
    ActiveInterruptEffect_e* EffectingInterrupt)

{
    /* Alarm Source/Status Bits for All UTOPIA Interrupts */
    ixf6048_UtopiaAlarmStatus_t UtopiaAlarms;
    boolean                    ChannelAlarmsActive = FALSE;

    /* Note the Special Handling of the register to Conform to Channel 
    based Interrupt Handling.*/
    if (*ReadIntFlag == TRUE)
    { /* Read Interrupt Registers only one time: The 1st time this func is Called!*/
        _TxUtopiaAlarms = (READ_FROM_HW(BaseAddress + ixf6048_T_UTOINT_OFFSET))&
            (READ_FROM_HW(BaseAddress + ixf6048_T_UTOINTEN_OFFSET))&
            ixf6048_mT_UTOINT;

        _RxUtopiaAlarms = (READ_FROM_HW(BaseAddress + ixf6048_R_UTOINT_OFFSET))&
            (READ_FROM_HW(BaseAddress + ixf6048_R_UTOINTEN_OFFSET))&
            ixf6048_mR_UTOINT;
        *ReadIntFlag = FALSE;
    }
    /* clear the Utiopia Alarms Struct */
    memset(&UtopiaAlarms, 0, sizeof(ixf6048_UtopiaAlarmStatus_t));

    /* Are there any Tx Utopia Alarms Active? */
    if (MiscGlobalAlarms & ixf6048_mGLOBAL_XMT_UTOPIA_INT)
    {  /* Are they Active on this Channel? -- Always Handle Tx Alarms that are active */
        if (_TxUtopiaAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum])
        {
            ChannelAlarmsActive = TRUE;
            UtopiaAlarms.Channel = ChanNum;
            UtopiaAlarms.TxAlarms = _TxUtopiaAlarms;
        }
    }
    /* Are there any Rx Utopia Alarms Active? */
    if(MiscGlobalAlarms & ixf6048_mGLOBAL_RCV_UTOPIA_ALARM)
    {  /* Are they Active on this Channel? -- Only Process if NO Catastrophic Alarms have occured*/
        if( (_RxUtopiaAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum])&&
            (*EffectingInterrupt == eINT_EFFECT_NONE             ) )
        {
            ChannelAlarmsActive = TRUE;
            UtopiaAlarms.Channel = ChanNum;
            UtopiaAlarms.RxAlarms = _RxUtopiaAlarms;
        }

    }
    /* Only use the Callback for Active Channels */
    if (ChannelAlarmsActive)
        /* there are Alarms active, so use the Callback */
        pAlarmCallback(BaseAddress, bb_6048_UTOPIA_ALARM, &UtopiaAlarms);

}
/***********************************************************************
 * Procedure Name: MiscInterruptHandler
 *
 * Description: handles Interrupts for a Single Channel.
 *
 * Conditions for Use:
 *    MiscGlobalAlarms contains the Global Interrupt Indication
 *    that a Utopia Related Alarm Has Occured
 *    
 *
 * Notes:
 *    MiscAlarmStatus will contain the current condition of all 
 *    active Miscelaneous Alarm Status Bits
 *    This function Calls The Alarm Callback on a Channel basis, 
 *    however the Structure that is passed contains All Active Interrupts.
 *
 *
 * Returns: MiscAlarmStatus     Structure is Passed to Alarm Callback Func
 *          ReadIntFlag         Has the Utopia Interrupt Register been Read?
 *          EffectingInterrupt   Does this interupt Cause a Cascade of Interrupts? 
 * End
 **********************************************************************/
static void MiscInterruptHandler(ushort*  BaseAddress,
    ushort   ChanNum,
    ushort   MiscGlobalAlarms,
    boolean *ReadIntFlag,
    ActiveInterruptEffect_e* EffectingInterrupt)

{
    /* Alarm Source/Status Bits for All MISC Interrupts */
    ixf6048_MiscAlarmStatus_t MiscAlarms;
    boolean                   ChannelAlarmsActive = FALSE;

    /* Note the Special Handling of the register to Conform to Channel 
    based Interrupt Handling. */
    if (*ReadIntFlag == TRUE)
    { /* Read Interrupt Registers only one time: The 1st time this func is Called!*/
        _PRBSAlarms = (READ_FROM_HW(BaseAddress + ixf6048_PRBSINT_OFFSET))&
            (READ_FROM_HW(BaseAddress + ixf6048_PRBSINTEN_OFFSET))&
            ixf6048_mPRBS_ALARMS;

        _TxAlarmBusAlarms =
            (READ_FROM_HW(BaseAddress + ixf6048_TALBINT_OFFSET))&
            (READ_FROM_HW(BaseAddress + ixf6048_TALBINTEN_OFFSET)) &
            ixf6048_mTALB_INT;

        _LineSideParityAlarm =
            (READ_FROM_HW(BaseAddress + ixf6048_LSPINT_OFFSET))&
            (READ_FROM_HW(BaseAddress + ixf6048_LSPINTEN_OFFSET)) &
            ixf6048_mLSP_INT;

        *ReadIntFlag = FALSE;
    }

    /* Clear the Misc Alarms Struct */
    memset(&MiscAlarms, 0, sizeof(ixf6048_MiscAlarmStatus_t));
    /* Set the Channel Number to the Return Struct */
    MiscAlarms.Channel = ChanNum;

    /* Are there any PRBS Alarms Active? */
    if (MiscGlobalAlarms & ixf6048_mGLOBAL_PRBS_ANLYZER_INT)
    {    /* Are they Active on this Channel? */
        if (_PRBSAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum])
        {
            MiscAlarms.PRBSAlarms = _PRBSAlarms;
            ChannelAlarmsActive = TRUE;
        }
    }
    /* Are there any Tx Alarm Bus Alarms Active? */
    if (MiscGlobalAlarms & ixf6048_mGLOBAL_XMT_ALARM_BUS_INT)
    {    /* Are they Active on this Channel? */
        if (_TxAlarmBusAlarms & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum])
        {
            MiscAlarms.TxAlarmBusAlarms = _TxAlarmBusAlarms;
            ChannelAlarmsActive = TRUE;
        }
    }
    /* Are there any Line Side Parity Alarms Active? */
    if (MiscGlobalAlarms & ixf6048_mGLOBAL_LINE_SIDE_PARITY_INT)
    {    /* Are they Active on this Channel? -- Only Possible for Chan 0 */
        if (_LineSideParityAlarm & IXF6048_CHANNEL_MASK_ALARM_ARRAY[ChanNum])
        {
            MiscAlarms.LineSideParityAlarm = _LineSideParityAlarm;
            ChannelAlarmsActive = TRUE;
        }
    }

    if(ChannelAlarmsActive == TRUE)
        /* there are Alarms active on this channel, so use the Callback */
        pAlarmCallback(BaseAddress, bb_6048_MISC_ALARM, &MiscAlarms);

}
/***********************************************************************
 * Procedure Name: ixf6048_GetChipInfo
 *
 * Description:  Get chip read only data -- Version and Id 
 *
 * Conditions for Use:
 *    none
 *
 * Returns:
 *        ixf6048_ChipInfo_t*  pChipInfo
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetChipInfo(bb_ChipData_t *pChipData,
    ixf6048_ChipInfo_t*  pChipInfo)
{
    register ushort  RegVal;
    register ushort* BaseAddress = pChipData->BaseAddress;

    RegVal = READ_FROM_HW(BaseAddress + ixf6048_CHIP_ID_OFFSET);
    pChipInfo->ChipVersion = (RegVal & ixf6048_mCHIP_VER) >> 8;  /* Chip Version */
    pChipInfo->ChipId = (RegVal & ixf6048_mCHIP_ID);             /* Chip ID      */

    return bb_NO_ERROR;
}


/***********************************************************************
 * Procedure Name:   ixf6048_SetChipOnline
 *
 * Description:   Set Chip Online
 *
 * Conditions for Use:
 *    BaseAddress is known good for open Chip
 *    All Setups Have been completed (Utopia Interface etc..)
 *
 * Notes:
 *
 *
 *
 * PDL:
 *          If Alarms are Interrupt Driven
 *             Enable interrupts for chip
 *          Endif
 *          Enable I/O Busses
 * End
 **********************************************************************/
void ixf6048_SetChipOnline (bb_ChipData_t* pChipData)
{
    register ushort* pReg;
    register ushort* BaseAddress = pChipData->BaseAddress;

    /* Enable Interrupts if Alarms are being used by this driver*/
    if (AlarmsUsed == TRUE)
    {
        pReg = BaseAddress + ixf6048_MACNF_OFFSET;
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bMAS_INT_EN));
    }

    /* Enable I/O busses */
    pReg = BaseAddress + ixf6048_GOCNF_OFFSET;
    Ixf6048SetBit(pReg, ixf6048_CHIP_ENABLE);
}

/***********************************************************************
 * Procedure Name:   ixf6048_SetChipOffline
 *
 * Description:  Take the Chip Offline
 *
 * Conditions for Use:
 *    BaseAddress is known good for open OHT
 *
 * Notes:
 *
 *
 *
 * PDL:
 *    Disable interrupts for chip
 *    Disable I/O Busses
 * End
 **********************************************************************/
void ixf6048_SetChipOffline(bb_ChipData_t* pChipData)
{
    register ushort* pReg;
    register ushort* BaseAddress = pChipData->BaseAddress;

    /* Disable interrupts */
    pReg = BaseAddress + ixf6048_MACNF_OFFSET;
    Ixf6048ClrBit(pReg, ixf6048_bMAS_INT_EN);

    /* Disable I/O busses */
    pReg = BaseAddress + ixf6048_GOCNF_OFFSET;
    WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ixf6048_CHIP_DISABLE));
}
/***********************************************************************
 * Procedure Name: SetChanOnline
 *
 * Description: Set a single Channel On-line
 *                                                         
 * Conditions for Use:
 *    The Chip Configuration must be set prior to turning 
 *    a channel On-Line.
 *
 * Notes:
 *    Mode (ATM/POS/REPEATER). This Global Structure is 
 *    set at Init, and whenever there are changes to the 
 *    Xmt/Rcv ChMode bits. 
 *
 *   bb_ChipData_t*   pChipData,    Initialized Chip Config Data  
 *   uint             ChanNum,      Channel 0-3
 *
 *
 * End
 **********************************************************************/
#if 0
bb_Error_e ixf6048_SetChanOnline(bb_ChipData_t* pChipData, 
    uint           ChanNum)
{
    bb_Error_e         rc = bb_NO_ERROR;
    register ushort*   pReg;
    register ushort*	 BaseAddress = pChipData->BaseAddress;
    ixf6048_ChipCfg_t* pChipCfg;

    pChipCfg = pChipData->pChipCfg;

    /* Validate that we are able to set this channel online. */
    if (ChanNum > 0)
    {
        pReg = BaseAddress + ixf6048_GOCNF_OFFSET;
        /* check that we are in Quad Mode */
        if (!(READ_FROM_HW(pReg) & ixf6048_bQUAD_MODE))
        {
            rc =  bb_INVALID_NUM_CHAN_CFG;
        }
    }

    /* All modes are OK so Enable Channel */
    if (rc == bb_NO_ERROR)
    {
        /* Get the Config for this Channel ATM/POS or OHT and Set
        the Rcv/Xmt Channel mode */

        /* Set the OpMode for the Chip  -- Rcv Side*/
        pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mCHAN_MODE));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | pChipCfg->pChanCfg[ChanNum]->ChanMode));

        /* Set the OpMode for the Chip  -- Tx Side*/
        pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(ChanNum);
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mCHAN_MODE));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | pChipCfg->pChanCfg[ChanNum]->ChanMode));

        /* Now Enable the Channel */
        pReg = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum);
        Ixf6048SetBit(pReg, ixf6048_bCH_ENA);
    }

    return rc;
}

/***********************************************************************
 * Procedure Name: SetChanOffline
 *
 * Description: Sets a single Channel Off Line
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 *   ushort*       BaseAddress,    Base address of chip
 *   uint          ChanNum,        Channel 0-3
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_SetChanOffline(bb_ChipData_t* pChipData, 
    uint           ChanNum)
{
    register ushort* pReg;
    register ushort* BaseAddress = pChipData->BaseAddress;

    pReg = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum);
    Ixf6048ClrBit(pReg, ixf6048_bCH_ENA);

    return bb_NO_ERROR;
}

/***********************************************************************
 * Procedure Name: SetChanTestMode
 *
 * Description: Set a single Channel to Test Mode
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *    This Function Sets the Chip into Test mode for both
 *    Receive and Transmit. These could be divided into 
 *    separate functions for User specific tests.
 *  
 *    Once placed into Test mode, no data is Read/Written, 
 *    To/From, the UTOPIA FIFOs. 
 *
 *    bb_ChipData_t* pChipData,    Chip config information
 *    uint           ChanNum,        Channel 0-3
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_SetChanTestMode(bb_ChipData_t* pChipData, 
    uint           ChanNum)
{
    bb_Error_e       rc = bb_NO_ERROR;
    register ushort* pReg;
    register ushort* BaseAddress = pChipData->BaseAddress;

    /* Validate that we are able to Alter this channel configuration. */
    if (ChanNum > 0)
    {
        pReg = BaseAddress + ixf6048_GOCNF_OFFSET;
        /* check that we are in Quad Mode */
        if (!(READ_FROM_HW(pReg) & ixf6048_bQUAD_MODE))
        {
            rc =  bb_INVALID_NUM_CHAN_CFG;
        }
    }

    /* No problems so set chip into Test mode -- Tx and Rx */
    if (rc == bb_NO_ERROR)
    {
        pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(ChanNum);
        /* Set the Receive side ChanMode bit */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mCHAN_MODE));    /* ixf6048_bCHAN_MODE_TEST =00 */

        pReg = BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(ChanNum);
        /* Set the Transmit Side ChanMode bit */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mCHAN_MODE));    /* ixf6048_bCHAN_MODE_TEST =00 */
    }

    return rc;
}
#endif
/***********************************************************************
 * Procedure Name: ixf6048_GetChanOpMode
 *
 * Description:  Get the Chan mode
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *   bb_ChipData_t *pChipData,   -> BaseAddress of Chip
 *   uint          ChanNum,       Channel 0-3
 *   bb_ChanMode_e*  pChanMode);      place to return Op Mode for chan 
 *
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetChanOpMode (bb_ChipData_t*       pChipData,
    uint                 ChanNum,
    ixf6048_ChanMode_e*  pChanMode)
{
    register ushort* BaseAddress = pChipData->BaseAddress;
    register ushort* pReg;
    /* PSN_TODO Read from hardware */

    pReg = BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(ChanNum);

    *pChanMode = (READ_FROM_HW(pReg) & ixf6048_mCHAN_MODE);

    return bb_NO_ERROR;

}
#if 0
/***********************************************************************
 * Procedure Name:  ixf6048_Test
 *
 * Description: Run a Test on a Channel
 *
 * Conditions for Use:
 *    None.
 *
 * Notes:
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
bb_Error_e ixf6048_Test(bb_ChipData_t*         pChipData,
    ixf6048_ChanTestCfg_t* pTestCfg)
{
    bb_Error_e          rc = bb_NO_ERROR;
    ixf6048_ChanMode_e  ChanMode;
    register ushort*    BaseAddress = pChipData->BaseAddress;

    switch(pTestCfg->ChanTests)
    {
    case ixf6048_eCHAN_SDH_TESTS:  /* To introduce SONET/SDH errors */
        TestChanSdh(BaseAddress, pTestCfg);
        break;
    case ixf6048_eCHAN_PTR_TESTS: /* To test AU pointers and/or SS bits */
        TestChanPtrs(BaseAddress, pTestCfg);
        break;

    case ixf6048_eCHAN_ATM_TESTS: /* To introduce ATM (HEC) errors */
        rc = ixf6048_GetChanOpMode (pChipData, pTestCfg->ChanNum, &ChanMode);
        /* Is this Channel in ATM mode? */
        if( (rc       == bb_NO_ERROR          )&&
            (ChanMode == ixf6048_eATM_MODE_CHAN) )
            TestChanAtm(BaseAddress, pTestCfg);
        else
            rc = bb_INV_ATM_CHAN_CFG_USED;
        break;

    case ixf6048_eCHAN_POS_TESTS: /* To introduce POS (FCS) errors */
        rc = ixf6048_GetChanOpMode (pChipData,pTestCfg->ChanNum, &ChanMode);
        /* Is this Channel in POS mode? */
        if( (rc       == bb_NO_ERROR          )&&
            (ChanMode == ixf6048_ePOS_MODE_CHAN) )
            TestChanPos(BaseAddress, pTestCfg);
        else
            rc = bb_INV_POS_CHAN_CFG_USED;
        break;

    case ixf6048_eCHAN_LOOPBACKS: /* To do local or line loopback */
        rc = TestChanLoopbacks(BaseAddress, pTestCfg);
        break;

    default:
        rc = bb_INV_CHAN_TEST;
        break;
    }

    return rc;
}

/***********************************************************************
 * Procedure Name:  TestChanSdh
 *
 * Description: Run an SDH related Test on a Specific Channel
 *
 * Conditions for Use:
 *    All Addresses have been Validated
 *
 * Notes:
 *    The channel must be in the test mode to call this function.
 *
 *    Multiple Tests Can be Activated/Deactived by passing Multiple
 *    Bits;
 *    
 *    FORCE/CLEAR TX_MST_RDI will Also Force/Clear the RX_MST_RDI. See
 *    the T_SC_MSOH Register, XmtMstRdiSrc[1:0] Bits. To Force TX_MST_RDI
 *    XmtMstRdiSrc is Set to Hardware Supplied (Internal Feedback from 
 *    the receiver) and then RDI is Forced on the Rx MST RDI. 
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
static void TestChanSdh(ushort*         BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg)
{
    ushort                  RegBitMask  = 0;
    register ushort*        pReg;
    ixf6048_ChanRxSdhTest_e RxTest;
    ixf6048_ChanTxSdhTest_e TxTest;
    uint                    ChanNum = pTestCfg->ChanNum;

    if(pTestCfg->u.ChanSdhTest.RxSdhTest)
    {
        RxTest = pTestCfg->u.ChanSdhTest.RxSdhTest;

        /* Force/Clear  Counter Tests */
        if (RxTest & (ixf6048_eFORCE_CNTR_OVFL | ixf6048_eCLEAR_CNTR_OVFL))
        {
            pReg = BaseAddress + ixf6048_MACNF_OFFSET;
            if (RxTest & ixf6048_eFORCE_CNTR_OVFL)
                Ixf6048SetBit(pReg, ixf6048_bCNTR_TEST_EN);
            else
                Ixf6048ClrBit(pReg, ixf6048_bCNTR_TEST_EN);
        }
        /* Force/Clear Rx RST AIS */
        if (RxTest & (ixf6048_eFORCE_RX_RST_AIS | ixf6048_eCLEAR_RX_RST_AIS))
        {
            pReg = BaseAddress + ixf6048_R_RSTC_CHAN_OFFSET(ChanNum);
            if (RxTest & ixf6048_eFORCE_RX_RST_AIS)
                Ixf6048SetBit(pReg, ixf6048_bRST_AIS_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_bRST_AIS_FRC);
        }
        /* Force/Clear MST AIS */
        if (RxTest & (ixf6048_eFORCE_RX_MST_AIS | ixf6048_eCLEAR_RX_MST_AIS))
        {
            pReg = BaseAddress + ixf6048_R_MST_C_CHAN_OFFSET(ChanNum);
            if (RxTest & ixf6048_eFORCE_RX_MST_AIS)
                Ixf6048SetBit(pReg, ixf6048_MST_AIS_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_MST_AIS_FRC);
        }
        /* Force/Clear Rx MST RDI */
        if (RxTest & (ixf6048_eFORCE_RX_MST_RDI | ixf6048_eCLEAR_RX_MST_RDI))
        {
            pReg = BaseAddress + ixf6048_R_MST_C_CHAN_OFFSET(ChanNum);
            if (RxTest & ixf6048_eFORCE_RX_MST_RDI)
                Ixf6048SetBit(pReg, ixf6048_MST_RDI_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_MST_RDI_FRC);
        }
        /* Force/Clear Rx MSA AIS */
        if (RxTest & (ixf6048_eFORCE_RX_MSA_AIS | ixf6048_eCLEAR_RX_MSA_AIS))
        {
            pReg = BaseAddress + ixf6048_R_MSA_C_CHAN_OFFSET(ChanNum);
            if (RxTest & ixf6048_eFORCE_RX_MSA_AIS)
                Ixf6048SetBit(pReg, ixf6048_bRCV_MSA_AIS_FORCE);
            else
                Ixf6048ClrBit(pReg, ixf6048_bRCV_MSA_AIS_FORCE);
        }
        /* Force/Clear Rx HPT AIS */
        if (RxTest & (ixf6048_eFORCE_RX_HPT_AIS | ixf6048_eCLEAR_RX_HPT_AIS))
        {
            pReg = BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum);
            if (RxTest & ixf6048_eFORCE_RX_HPT_AIS)
                Ixf6048SetBit(pReg, ixf6048_bRCV_HPT_AIS_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_bRCV_HPT_AIS_FRC);
        }

    } /* end if (pTestCfg->u.ChanSdhTest.RxSdhTest) */

    if(pTestCfg->u.ChanSdhTest.TxSdhTest)
    {
        TxTest = pTestCfg->u.ChanSdhTest.TxSdhTest;
        /* Force/Clear Tx MST RDI */
        if (TxTest & (ixf6048_eFORCE_TX_MST_RDI | ixf6048_eCLEAR_TX_MST_RDI))
        {  /* Note: Special Case Handling For this Register */
            pReg = BaseAddress + ixf6048_T_SC_MSOH_CHAN_OFFSET(ChanNum);
            if (TxTest & ixf6048_eFORCE_TX_MST_RDI)
            {
                Ixf6048SetBit(pReg, ixf6048_mXMT_MST_RDI_SRC);
                /* Must Set the XmtMstRdiSrc[1:0] bits to be Hardware Supplied */
                Ixf6048ClrBit(pReg,ixf6048_mXMT_MST_RDI_SRC);

                /* Now Force RDI From RxMstRdi */
                pReg = BaseAddress + ixf6048_R_MST_C_CHAN_OFFSET(ChanNum);
                Ixf6048SetBit(pReg,ixf6048_MST_RDI_FRC);
            }
            else
            {
                /* Halt RDI Generation by clearing  Rx Mst Rdi Force Bit*/
                pReg = BaseAddress + ixf6048_R_MST_C_CHAN_OFFSET(ChanNum);
                Ixf6048ClrBit(pReg, ixf6048_MST_RDI_FRC);
                /* Note: Caller should Re-Set XmtMstRdiSrc[1:0] bits to
                The Value it was Prior to Forcing Tx Rdi */
            }
        }
        /* Force/Clear Tx MSA AIS */
        if (TxTest &(ixf6048_eFORCE_TX_MSA_AIS | ixf6048_eCLEAR_TX_MSA_AIS))
        {
            pReg = BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum);
            if (TxTest & ixf6048_eFORCE_TX_MSA_AIS)
                Ixf6048SetBit(pReg, ixf6048_bXMT_MSA_AIS_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_bXMT_MSA_AIS_FRC);
        }
        /* Force/Clear Tx HPT AIS */
        if (TxTest &(ixf6048_eFORCE_TX_HPT_AIS | ixf6048_eCLEAR_TX_HPT_AIS))
        {
            pReg = BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum);
            if (TxTest & ixf6048_eFORCE_TX_HPT_AIS)
                Ixf6048SetBit(pReg, ixf6048_bXMT_HPT_AIS_FRC);
            else
                Ixf6048ClrBit(pReg, ixf6048_bXMT_HPT_AIS_FRC);
        }

        /* Register offset for A1, B1 and B3 Errors */
        RegBitMask= 0;
        if (TxTest & ixf6048_eFORCE_4_A1_ERRS)
            RegBitMask |= ixf6048_bFORCE_4_A1_ERRS;

        if (TxTest & ixf6048_eFORCE_A1_ERRS)
            RegBitMask |= ixf6048_bFORCE_A1_ERRS;

        if (TxTest & ixf6048_eFORCE_ONE_B1_ERR)
            RegBitMask |= ixf6048_bFORCE_ONE_B1_ERR;

        if (TxTest & ixf6048_eFORCE_B1_ERRS)
            RegBitMask |= ixf6048_bFORCE_B1_ERRS;

        if (TxTest & ixf6048_eFORCE_ONE_B2_ERR)
            RegBitMask |= ixf6048_bFORCE_ONE_B2_ERR;

        if (TxTest & ixf6048_eFORCE_B2_ERRS)
            RegBitMask |= ixf6048_bFORCE_B2_ERRS;

        /* Clr/Set Reg 1 time only if specified */
        if (RegBitMask)
        {
            pReg = BaseAddress + ixf6048_T_RMST_OP_CHAN_OFFSET(ChanNum);
            /* Always Clear and then Set the Selected Bits*/
            Ixf6048ClrBit(pReg, RegBitMask);
            Ixf6048SetBit(pReg, RegBitMask);
        }

        RegBitMask = 0;
        if (TxTest & ixf6048_eFORCE_ONE_B3_ERR)
            RegBitMask |= ixf6048_bINVERT_B3_BYTE_FOR_1_FRAME;
        if (TxTest & ixf6048_eFORCE_B3_ERRS)
            RegBitMask |= ixf6048_bINVERT_B3_BYTE_FOREVER;

        if (RegBitMask)
        {
            pReg = BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum);
            /* Always Clear and then Set */
            Ixf6048ClrBit(pReg, RegBitMask);
            Ixf6048SetBit(pReg, RegBitMask);
        }
    }  /* end if(pTestCfg->u.ChanSdhTest.RxSdhTest) */

}
/***********************************************************************
 * Procedure Name:  TestChanPtrs
 *
 * Description: Run a Pointers related Test on a Specific Channel
 *
 * Conditions for Use:
 *    All Addresses have been Validated
 *
 * Notes:
 *    The channel must be in the test mode to call this function.
 *
 *    Multiple Tests Can be Activated/Deactived by passing Multiple
 *    Bits;
 *
 *    Cannot Perform The Combination of SET_PTR or SET_PTR_SS with a 
 *    Posative/Negative Pointer adjustment.
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
static void TestChanPtrs(ushort*        BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg)
{
    ixf6048_ChanPtrTest_e ChanPtrTest;   /* AU Ptr tests */
    ushort                InitialPtr;    /* Initial Ptr val, if needed */
    uchar                 SsBits;        /* SS bits, if needed */
    register ushort*      pReg;
    register ushort       RegVal = 0;

    ChanPtrTest = pTestCfg->u.ChanPtrTestCfg.ChanPtrTest;
    pReg = BaseAddress + ixf6048_T_AU_PTS_CHAN_OFFSET(pTestCfg->ChanNum);

    /* Set initial AU Ptr value */
    if(ChanPtrTest & ixf6048_eSET_AU_PTR_TEST)
    {
        InitialPtr =  pTestCfg->u.ChanPtrTestCfg.InitialPtr;
        RegVal  |= ixf6048_bXMT_SET_AU_PT_VAL |
            (ixf6048_mXMT_AU_PT_VAL & InitialPtr);
    }
    /* Set SS bits value */
    if (ChanPtrTest & ixf6048_eSET_AU_PTR_SS_TEST)
    {
        SsBits =  pTestCfg->u.ChanPtrTestCfg.SsBits;
        RegVal |= ixf6048_bXMT_SET_AU_PT_VAL |
            (ixf6048_mXMT_AU_PT_SS & SsBits);
    }

    /* Increment AU Ptr */
    if (ChanPtrTest & ixf6048_eINC_AU_PTR_TEST)
        RegVal |= ixf6048_bXMT_SET_AU_POS;

    /* Decrement AU Ptr */
    if (ChanPtrTest & ixf6048_eDEC_AU_PTR_TEST)
        RegVal |= ixf6048_bXMT_SET_AU_NEG;

    /* Send NDF -- 1001 is inserted in the Tx H1 */
    if (ChanPtrTest & ixf6048_eSET_NDF_PTR_TEST)
        RegVal  |= ixf6048_bXMT_SET_NDF;

    /* Now Set the Test Configuration to the Reg */
    WRITE_TO_HW(pReg, RegVal);
}
/***********************************************************************
 * Procedure Name: TestChanAtm 
 *
 * Description: Run an ATM  related Test on a Specific Channel
 *
 * Conditions for Use:
 *    All Addresses have been Validated
 *
 * Notes:
 *    The channel must be in the test mode to call this function.
 *   
 *    The XmtHErrEn bit is Automatically Cleared when inserting a 
 *    Single HEC error, however if in continuous mode, then the
 *    user will have to Manually Clear this bit (see Data Sheet.)
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
static void TestChanAtm(ushort*                 BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg)
{
    ixf6048_ChanAtmTest_e AtmTest;
    register ushort*      pReg;

    AtmTest = pTestCfg->u.ChanAtmTest;
    pReg = BaseAddress + ixf6048_T_ACPCNF_CHAN_OFFSET(pTestCfg->ChanNum);

    /* Stop ATM errors */
    if (AtmTest == ixf6048_eSTOP_ATM_ERRS)        
    WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_bXMT_H_ERR_EN));   /* Force-Clr HEC Error Enable Bit */

    /* Force 1 correctable error */
    if (AtmTest & ixf6048_eFORCE_1_COR_ERR)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mXMT_H_ERR_CNF)); /* Clear HErrCnf */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_H_ERR_EN));   /* Set Enable Bit */
    }

    /* Force correctable errors */
    if (AtmTest & ixf6048_eFORCE_COR_ERRS)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) &  ~ixf6048_mXMT_H_ERR_CNF)); /* Clear HErrCnf */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) |  ixf6048_INSERT_HEC_COR_CELLS_EN));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) |  ixf6048_bXMT_H_ERR_EN));   /* Set Enable Bit */
    }

    /* Force 1 uncorrectable error */
    if (AtmTest & ixf6048_eFORCE_1_UNCOR_ERR)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mXMT_H_ERR_CNF)); /* Clear HErrCnf */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_INSERT_1_HEC_UNCOR_CELL_EN));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_H_ERR_EN));   /* Set Enable Bit */
    }

    /* Force uncorrectable errors */
    if (AtmTest & ixf6048_eFORCE_UNCOR_ERRS)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_mXMT_H_ERR_CNF)); /* Clear HErrCnf */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_INSERT_HEC_UNCOR_CELLS_EN));
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_H_ERR_EN));   /* Set Enable Bit */
    }
}
/***********************************************************************
 * Procedure Name: TestChanPos
 *
 * Description: Run a POS related Test on a Specific Channel
 *
 * Conditions for Use:
 *    All Addresses have been Validated
 *
 * Notes:
 *    The channel must be in the test mode to call this function.
 *
 *    The FCSErrCnf bit is Automatically Cleared when inserting a 
 *    Single FCS error, however this bit Must be cleared Manually
 *    if configured for Continuous Mode (See Data Sheet.)
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
static void TestChanPos(ushort*                 BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg)
{
    ixf6048_ChanPosTest_e PosTest;
    register ushort*      pReg;


    PosTest = pTestCfg->u.ChanPosTest;
    pReg = BaseAddress + ixf6048_T_PHCCNF_CHAN_OFFSET(pTestCfg->ChanNum);

    /* Stop POS HDLC FCS errors */
    if (PosTest == ixf6048_eSTOP_POS_ERRS)
    WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_bXMT_FCS_ERR_CNF)); /* Clear the FCSErrCnf Bit */

    /* Force 1 FCS error */
    if (PosTest & ixf6048_eFORCE_1_FCS_ERR)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) & ~ixf6048_bXMT_FCS_ERR));     /* Clear the XMT_FCS error Bit */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_FCS_ERR_CNF)); /* Set the FCSErrCnf Bit */
    }

    /* Force FCS errors */
    if (PosTest & ixf6048_eFORCE_FCS_ERRS)
    {
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_FCS_ERR));     /* Set the XMT_FCS error Bit */
        WRITE_TO_HW(pReg, (READ_FROM_HW(pReg) | ixf6048_bXMT_FCS_ERR_CNF)); /* Set the FCSErrCnf Bit */
    }
}
/***********************************************************************
 * Procedure Name: TestChanLoopbacks 
 *
 * Description: Run a Loopback related Test on a Specific Channel
 *
 * Conditions for Use:
 *    All Addresses have been Validated
 *
 * Notes:
 *    The channel must be in the test mode to call this function.
 *
 *    Line Loopback can only be enabled if RcvIFMode[2:0] (Reg R_COCNF) and
 *    XmtIFMode[2:0] (reg T_COCNF) Values Match;
 *
 *    ixf6048_ChanTestCfg_t  -> Test cfg -- for Channel 
 *    BaseAddress           --> Must be a Valid Chip Base Address
 *
 **********************************************************************/
static bb_Error_e TestChanLoopbacks(ushort*       BaseAddress,
    ixf6048_ChanTestCfg_t*  pTestCfg)
{
    bb_Error_e             rc = bb_NO_ERROR;
    ixf6048_ChanLoopback_e LpbkTest;
    register ushort*       pReg;
    register ushort        RcvRegVal;
    register ushort        XmtRegVal;
    uint                   ChanNum;

    LpbkTest = pTestCfg->u.ChanLoopback;
    ChanNum  = pTestCfg->ChanNum;

    pReg  = BaseAddress + ixf6048_COCNF_CHAN_OFFSET(ChanNum);

    /* Stop all loopback(s) */
    if(LpbkTest == ixf6048_eCHAN_LOOPBACK_OFF)
    {
        Ixf6048ClrBit(pReg,ixf6048_bCHAN_SYS_LOOPBACK); /* Clear Sys Loopback Mode bit */
        Ixf6048ClrBit(pReg, ixf6048_bLINE_LOOP_BACK);   /* Clear Line Loopback Bit*/
    }

    /* Do local channel loopback, inside chip */
    if( (LpbkTest == ixf6048_eCHAN_LOCAL_LOOPBACK)||
        (LpbkTest == ixf6048_eCHAN_BOTH_LOOPBACKS) )
    Ixf6048SetBit(pReg, ixf6048_bCHAN_SYS_LOOPBACK); /* Set Loopback Mode bit */

    /* Loop Rx SDH data back to SDH network */
    if( (LpbkTest == ixf6048_eCHAN_LINE_LOOPBACK )||
        (LpbkTest == ixf6048_eCHAN_BOTH_LOOPBACKS) )
    {
        RcvRegVal = READ_FROM_HW(BaseAddress + ixf6048_R_COCNF_CHAN_OFFSET(ChanNum));
        XmtRegVal = READ_FROM_HW(BaseAddress + ixf6048_T_COCNF_CHAN_OFFSET(ChanNum));
        RcvRegVal = (ushort) ixf6048_GET_IF_MODE(RcvRegVal);
        XmtRegVal = (ushort) ixf6048_GET_IF_MODE(XmtRegVal);

        /* Check that Rcv side Matches Tx side */
        if(RcvRegVal != XmtRegVal)
        {
            rc = bb_LOOPBACK_MODE_RX_TX_MISMATCH;
        }
        else
            Ixf6048SetBit(pReg, ixf6048_bLINE_LOOP_BACK); /*Set Line Loopback Bit*/
    }

    return rc;
}
#endif
/***********************************************************************
 * Procedure Name: ixf6048_SetTrace
 *
 * Description: Set Expected/Tx J0/J1 Trace
 *
 * Conditions for Use:
 *    none
 *
 * Inputs:
 *  (bb_ChipData_t*   pChipData,   -> Initialized Chip Data 
 *   bb_TraceType_e   TraceType,   Type of Trace: Expected/Tx J0/J1 
 *   uint             ChanNum,     Channel 0-3 
 *   bb_TraceFormat_e TraceFormat, Specifies the Trace Format 
 *   uchar*           pTrace);     -> null-terminated Trace String (data only)
 *                                   Driver adds CRC7, or LF/CR, if needed 
 * Notes:
 *   The bb_TraceType_e value for  
 *          bb_RX_J0 and bb_RX_J1
 *   must be Set to "Ignore Jn Trace (No Trace)". 
 *   This Condition is Forced in this Function.
 *    --> see R_Jn_ESTRA Registers
 *
 * Returns:
 *    Sets the new Trace String in the Associated Storage Structure 
 *    Sets the new TraceFormat in the Associated Storage Structure
 *    Error Condition if Unsuccessful
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_SetTrace(bb_ChipData_t*   pChipData,
    uint              ChanNum,
    bb_TraceType_e    TraceType,
    bb_TraceFormat_e  TraceFormat,
    char*             pTrace)
{
    bb_Error_e            rc = bb_NO_ERROR;
    register ushort*      pReg;
    register ushort*      BaseAddress;
    ushort                JnCfgMask;
    ixf6048_ChipCfg_t*    pChipCfg;
    ixf6048_ChanCfg_t*    pChanCfg;
    ixf6048_OhtChanCfg_t* pOhtChanCfg;

    BaseAddress = pChipData->BaseAddress;
    pChipCfg    = pChipData->pChipCfg;
    pChanCfg    = pChipCfg->pChanCfg[ChanNum];

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    pOhtChanCfg = pChanCfg->pOhtChanCfg;

    switch (TraceType)
    {
    case bb_EXPECTED_J0:
        pReg = BaseAddress + ixf6048_EXPECTED_J0_CHAN_OFFSET(ChanNum);
        /* Set the J0_RSTC bits 2:0 to TraceFormat */
        rc = SetRcvJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTrace);
        if (rc == bb_NO_ERROR )
        {
            /* ixf6048_SET_RCV_J0_CNF_FORMAT(*(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)), 
            JnCfgMask); */
            WRITE_TO_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum),
                READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)) & ~ixf6048_mRCV_J0_CNF);
            WRITE_TO_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum),
                READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)) | JnCfgMask);
            /* strcpy(pOhtChanCfg->RxRstCfg.pExpectedJ0String, pTrace); */
            pOhtChanCfg->RxRstCfg.ExpectedJ0Format = TraceFormat;
        }
        break;
    case bb_EXPECTED_J1:
        pReg = BaseAddress + ixf6048_EXPECTED_J1_CHAN_OFFSET(ChanNum);
        /* Set the J1_HPT_C2 bits [10:8] to TraceFormat */
        rc = SetRcvJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTrace);
        if (rc == bb_NO_ERROR )
        {
            /* ixf6048_SET_RCV_J1_CNF_FORMAT(*(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)), 
            JnCfgMask); */
            WRITE_TO_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum),
                READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)) & ~ixf6048_mRCV_J1_CNF);
            WRITE_TO_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum),
                READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)) | ixf6048_SHIFT_RCV_J1_CNF_FORMAT(JnCfgMask));

            /* strcpy(pOhtChanCfg->RxHptCfg.pExpectedJ1String, pTrace); */
            pOhtChanCfg->RxHptCfg.ExpectedJ1Format = TraceFormat;
        }
        break;
    case bb_RX_J0:   /* For Testing Purposes Only */
        pReg = BaseAddress + ixf6048_RX_J0_CHAN_OFFSET(ChanNum); /* R_ASTRA */
        /* Must set the J0_RSTC bits 2:0 to Ignore Trace (01X) prior to Writing ASTRA */
        /* ixf6048_SET_RCV_J0_CNF_FORMAT(*(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)),
        ixf6048_IGNORE_RX_JN_EN); */
        WRITE_TO_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum),
            READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)) & ~ixf6048_mRCV_J0_CNF);
        WRITE_TO_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum),
            READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)) | ixf6048_IGNORE_RX_JN_EN);
        /* Note JnCfgMask Ignored */
        rc = SetRcvJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTrace);

        if (rc == bb_NO_ERROR )
        {
            TraceFormat = bb_IGNORE_RX_TRACE; /* Force this Trace Format */
            pOhtChanCfg->RxRstCfg.ExpectedJ0Format = bb_IGNORE_RX_TRACE;
            /* strcpy(pOhtChanCfg->RxRstCfg.pExpectedJ0String,pTrace); */
        }
        break;
    case bb_RX_J1:   /* For Testing Purposes Only */
        pReg = BaseAddress + ixf6048_RX_J1_CHAN_OFFSET(ChanNum);
        /* Set the R_HPT_C2 bits [10:8] to Ignore Trace (01X) */
        /* ixf6048_SET_RCV_J1_CNF_FORMAT(*(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)), 
        ixf6048_IGNORE_RX_JN_EN); */
        WRITE_TO_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum),
            READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)) & ~ixf6048_mRCV_J1_CNF);
        WRITE_TO_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum),
            READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)) | ixf6048_SHIFT_RCV_J1_CNF_FORMAT(ixf6048_IGNORE_RX_JN_EN));
        /* Note JnCfgMask Ignored */
        rc = SetRcvJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTrace);

        if (rc == bb_NO_ERROR )
        {
            TraceFormat = bb_IGNORE_RX_TRACE; /* Force this Trace Format */
            /* strcpy(pOhtChanCfg->RxHptCfg.pExpectedJ1String, pTrace); */
            pOhtChanCfg->RxHptCfg.ExpectedJ1Format = TraceFormat;
        }
        break;
    case bb_TX_J0:
        pReg = BaseAddress + ixf6048_TX_J0_CHAN_OFFSET(ChanNum);
        /* Use the T_RMST_OP bits [7:6] to  set the TraceFormat */
        /* Get the T_SC_RSOH bit 0 for J0 Src (uP or Received byte ) -- Does Not Effect this */
        rc = SetTxJnPathTrace(pReg,&JnCfgMask, TraceFormat, pTrace);
        if (rc == bb_NO_ERROR )
        {
            /* ixf6048_SET_TX_J0_CNFG(*(BaseAddress + ixf6048_T_RMST_OP_CHAN_OFFSET(ChanNum)),
            JnCfgMask);  */
            SetRegBits(pReg, JnCfgMask, ixf6048_mXMT_J0_CNFG, ixf6048_BIT_SHIFT_6);

            pOhtChanCfg->TxRstCfg.TxJ0Format = TraceFormat;
            /* strcpy(pOhtChanCfg->TxRstCfg.pJ0String,pTrace); */
        }
        break;
    case bb_TX_J1:
        pReg = BaseAddress + ixf6048_TX_J1_CHAN_OFFSET(ChanNum);
        /* Check T_HPT_C Bit 12 -- Pass through ALL POH bytes */
        /* Use the T_HPT_OPC bits [5:4] for TraceFormat/StrgLen */
        /* Get the T_SC_RSOH bit 0 for J0 Src (uP or Received byte ) */
        rc = SetTxJnPathTrace(pReg, &JnCfgMask, TraceFormat, pTrace);
        if (rc == bb_NO_ERROR )
        {
            /* ixf6048_SET_TX_J1_CNFG(*(BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum)), 
            JnCfgMask); */
            SetRegBits(pReg, JnCfgMask, ixf6048_mXMT_J1_CNFG, ixf6048_BIT_SHIFT_4);
            pOhtChanCfg->TxHptCfg.TxJ1Format = TraceFormat;
            /* strcpy(pOhtChanCfg->TxHptCfg.pTxJ1String,pTrace); */
        }
        break;
    default :
        rc = bb_INV_TRACE_TYPE;
        break;
    }

    return rc;
}

/***********************************************************************
 * Procedure Name: SetRcvJnPathTrace                                   
 *                                                                      
 * Description: Set the Rcv J0/J1 Path Trace Bytes
 *                                                                      
 * Conditions for Use:                                                  
 *      None                                                            
 *                                                                      
 * Inputs:  
 *          pReg        Pointer to the Rx Path Trace Register                                                                      
 *          TraceFormat The String length and type of Trace String
 *          pTrace      The String so Set into the Trace Byte
 *
 * Returns:
 *          pJnCgMask   Mask to set the RxJn_Cfg bits. Bit May be ignored if
 *                      This function is not Called from ixf6048_SetTrace
 *          bb_Error_e  Returns Error Condition if Not successful
 * 
 *
 * End                                                                  
 **********************************************************************/
static bb_Error_e SetRcvJnPathTrace(ushort*          pReg,
    ushort*          pJnCfgMask,
    bb_TraceFormat_e TraceFormat,
    char*            pTrace)
{
    bb_Error_e        rc = bb_NO_ERROR;
    uchar*            pWorkString=NULL;
    uchar*            pChar;
    uint              StringLen, TraceLen;
    boolean           CfgOK = TRUE;

    switch(TraceFormat)
    {
    case bb_64_BYTE_WITH_LF_CR:     /* 64 bytes with LF,CR at end */
        *pJnCfgMask = ixf6048_RX_JN_64_WITH_LF_CR_EN;
        pWorkString = bb_Make64ByteWorkString(pTrace);

        StringLen = 62;   /* Max length of trace, outside of LF,CR */
        TraceLen = strlen(pWorkString);  /* Get length of trace chars */
        if (TraceLen > 62)
        {
            TraceLen = 62; /* truncate trace, if too long */
        }
        pChar = pWorkString + TraceLen;  /* -> Null term at end */
        *pChar++ = '\n';
        *pChar   = '\r';  /* Add LF,CR at end of trace */

        CfgOK = CfgOhtJnString( pReg, pWorkString, 64);
        break;

    case bb_64_BYTE_FREE_FORM:      /* 64 bytes, free form */
        StringLen = 64;             /* Max length of trace, if free-form */
        *pJnCfgMask = ixf6048_RX_JN_64_FREE_FORM_EN;
        pWorkString = bb_Make64ByteWorkString(pTrace);
        CfgOK = CfgOhtJnString(pReg, pWorkString, 64);
        break;
    case bb_16_BYTE_WITH_CRC7:      /* 16 bytes, starting with CRC7 (SDH) */
        *pJnCfgMask = ixf6048_RX_JN_16_WITH_CRC7_EN;
        pWorkString = bb_Make16ByteWorkString(pTrace);
        StringLen = 15;

        bb_InsertCRC7(pWorkString); /* Add CRC7  */
        CfgOK = CfgOhtJnString(pReg, pWorkString, 16);

        break;
    case bb_IGNORE_RX_TRACE:        /* Ignore trace on Rx */
        *pJnCfgMask = ixf6048_IGNORE_RX_JN_EN;
        pWorkString = (uchar *) pTrace;
        /* Simply Set the Status bit */
        break;
    case bb_1_BYTE:                 /* 1 byte */
        *pJnCfgMask = ixf6048_RX_JN_1_BYTE_EN;
        /* 1st Byte of pTrace is the only byte of the 1 byte String */
        CfgOK = CfgOhtJnString(pReg,pTrace, 1);
        break;
    case bb_DEFAULT_TX_TRACE:       /* Not Valid format for Expected Trace */
    default :
        /* Nothing supported Here */
        return bb_INV_EXP_JN_FMT;
    }

    if (CfgOK)
    strcpy(pTrace, pWorkString); /* Copy the Formatted String Over the Actual String */
    else
    {
        rc = bb_JN_TRACE_WRITE_FAIL;
    }
    return rc;

}
/***********************************************************************
 * Procedure Name: SetTxJnPathTrace                                   
 *                                                                      
 * Description: Set the Rcv J0/J1 Path Trace Bytes
 *                                                                      
 * Conditions for Use:                                                  
 *      None                                                            
 *                                                                      
 * Inputs:  
 *          pReg        Pointer to the Tx Path Trace Register                                                                      
 *          TraceFormat The String length and type of Trace String
 *          pTrace      The String so Set into the Trace Byte
 *
 * Returns:
 *          pJnCgMask   Mask to set the TxJn_Cfg bits
 *          bb_Error_e  Returns Error Condition if Not successful
 * End                                                                  
 **********************************************************************/
static bb_Error_e SetTxJnPathTrace(ushort*          pReg,
    ushort*          pJnCfgMask,
    bb_TraceFormat_e TraceFormat,
    char*            pTrace)
{
    bb_Error_e rc = bb_NO_ERROR;
    char*      pWorkString=NULL;
    char*      pChar;
    uint       StringLen, TraceLen;
    boolean    CfgOK = TRUE;

    switch(TraceFormat)
    {
    case bb_64_BYTE_WITH_LF_CR:           /* 64 bytes with LF,CR at end String is  */
        *pJnCfgMask = ixf6048_TX_JN_64_EN; /* Built here, but set as 64 Byte String */
        pWorkString = bb_Make64ByteWorkString(pTrace);

        StringLen = 62;   /* Max length of trace, outside of LF,CR */
        TraceLen = strlen(pWorkString);  /* Get length of trace chars */
        if (TraceLen > 62)
        {
            TraceLen = 62; /* truncate trace, if too long */
        }
        pChar = pWorkString + TraceLen;  /* -> Null term at end */
        *pChar++ = '\n';
        *pChar   = '\r';  /* Add LF,CR at end of trace */

        CfgOK = CfgOhtJnString(pReg,pWorkString, 64);
        break;
    case bb_64_BYTE_FREE_FORM:      /* 64 bytes, free form */
        *pJnCfgMask = ixf6048_TX_JN_64_EN;
        StringLen = 64;             /* Max length of trace */
        pWorkString = bb_Make64ByteWorkString(pTrace);
        CfgOK = CfgOhtJnString(pReg, pWorkString, 64);
        break;
    case bb_16_BYTE_WITH_CRC7:      /* 16 bytes, starting with CRC7 (SDH) */
        *pJnCfgMask = ixf6048_TX_JN_16_EN;
        pWorkString = bb_Make16ByteWorkString(pTrace);
        StringLen = 15;

        bb_InsertCRC7(pWorkString); /* Add CRC7 for Tx */
        CfgOK = CfgOhtJnString(pReg,pWorkString, 16);

        break;
    case bb_1_BYTE:                 /* 1 byte */
        *pJnCfgMask = ixf6048_TX_JN_1_BYTE_EN;
        /* 1st Byte of pTrace is the only byte of the 1 byte String */
        CfgOK = CfgOhtJnString(pReg,pTrace, 1);
        break;
    case bb_DEFAULT_TX_TRACE:       /* 01H (Former C2 Byte) */
        *pJnCfgMask = ixf6048_TX_JN_DEFAULT_EN;
        *pTrace = 0x01; /* Ensure that the 1st byte of the Trace String is 01H */
        CfgOK = CfgOhtJnString(pReg,pTrace, 1);
        break;
    case bb_IGNORE_RX_TRACE:        /* Not supported for Tx Trace */
    default :
        /* Nothing supported Here */
        return bb_INV_EXP_JN_FMT;
    }

    if (CfgOK)
    strcpy(pTrace, pWorkString); /* Copy the Formatted String Over the Actual String */
    else
    {
        rc = bb_JN_TRACE_WRITE_FAIL;
    }
    return rc;

}
/***********************************************************************
 * Procedure Name: CfgOhtJnString                                   
 *                                                                      
 * Description: Configure the J0/J1 Trace 
 *                                                                      
 * Conditions for Use:                                                  
 *    none
 *
 *
 * PDL:                                                                 
 *                                                                      
 * End                                                                  
 **********************************************************************/
static boolean CfgOhtJnString(ushort*  pReg,
    char*    pWorkString, 
    int      StringLen)
{
    boolean  bMatch = TRUE;
    int      iByteIndex;
    int      iWaitTest;
    uchar*   pTraceByte = pWorkString;

#ifdef UNIT_TEST_6048
    if (StringLen == 1) 
        /* Jn is a Single Byte */
        printf("CfgOhtJnString: 1-byte WorkString \n");
    else if (StringLen == 16)
        /* CRC7, or space for it, at start of string */
        printf("CfgOhtJnString: 16-byte WorkString with CRC7 at start \n");
    else
        printf("CfgOhtJnString: 64-byte WorkString\n");
#endif
    /* Setup to Write to Trace Register */
    /* See Reg R_J0_ESTRA in Data Sheet for the Write/Read process */
    for (iByteIndex=0; iByteIndex<StringLen; ++ iByteIndex)
    {  /* Wait for Reg Access */
        iWaitTest = 0;
        while(!(ixf6048_CAN_WRITE_TRACE(pReg)))
        {  /* Sanity Check */
            if(iWaitTest++ == 100)
            {
                bMatch = FALSE;  /* Report that an Error has occured to calling Function */
                break;
            }
        }

        ixf6048_WRITE_TRACE(pReg, iByteIndex, pTraceByte++);
    }
    /* PSN_TODO -- Should I Read back the Bytes to Validate what I wrote? */

    return bMatch;
}
/***********************************************************************
 * Procedure Name: ixf6048_GetTrace
 *
 * Description: Get The Path Trace for the Selected ChanNum and Trace Type
 *
 * Conditions for Use:
 *          pTrace      Must point to A Valid Empty String
 *
 * Notes:
 *    Function to get the Rx J0/J1 Trace
 *
 * Inputs:
 *          pChipData      Initialized Chip Data 
 *          TraceType      Type of Trace: Rx J0/J1
 *          ChanNum        Chan 0-3 
 *          pTrace         place to return null-terminated
 *                         Rx Trace String.
 *                         Note: at least 65 bytes long
 * Outputs:
 *          pLength         The length of pTrace.
 *
 * Returns  bb_Error_e     Returns error Condition if Unsucessful
 *          pTrace         The Formatted Path Trace String
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetTrace(bb_ChipData_t*   pChipData,
    uint             ChanNum,    
    bb_TraceType_e   TraceType,  
    char*            pTrace,
    ushort*          pLength)
{
    bb_Error_e            rc = bb_NO_ERROR;
    register ushort*      pReg;
    register ushort*      BaseAddress;
    ushort                JnCfgMask;
    ixf6048_ChipCfg_t*    pChipCfg;
    ixf6048_ChanCfg_t*    pChanCfg;
    ixf6048_OhtChanCfg_t* pOhtChanCfg;

    BaseAddress = pChipData->BaseAddress;
    pChipCfg    = pChipData->pChipCfg;
    pChanCfg    = pChipCfg->pChanCfg[ChanNum];

    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    pOhtChanCfg = pChanCfg->pOhtChanCfg;

    switch (TraceType)
    {
    case bb_EXPECTED_J0:
        pReg = BaseAddress + ixf6048_EXPECTED_J0_CHAN_OFFSET(ChanNum);
        /* Check the J0_RSTC bits 2:0 For TraceFormat */
        JnCfgMask = ixf6048_GET_RCV_J0_CNF_FORMAT(READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)));
        rc = GetRcvJnPathTrace(pReg,JnCfgMask, pTrace, pLength);
        break;

    case bb_EXPECTED_J1:
        pReg = BaseAddress + ixf6048_EXPECTED_J1_CHAN_OFFSET(ChanNum);
        /* Set the J1_HPT_C2 bits [10:8] to TraceFormat */
        JnCfgMask = ixf6048_GET_RCV_J1_CNF_FORMAT(READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)));
        rc = GetRcvJnPathTrace(pReg, JnCfgMask, pTrace, pLength);
        break;
    case bb_RX_J0:   /* For Testing Purposes Only */
        pReg = BaseAddress + ixf6048_RX_J0_CHAN_OFFSET(ChanNum); /* R_ASTRA */

        /* Must Get the J0_RSTC bits 2:0 prior to looking at ASTRA */
        JnCfgMask = ixf6048_GET_RCV_J0_CNF_FORMAT(READ_FROM_HW(BaseAddress + ixf6048_J0_RSTC_CHAN_OFFSET(ChanNum)));
        rc = GetRcvJnPathTrace(pReg, JnCfgMask, pTrace, pLength);
        break;
    case bb_RX_J1:   /* For Testing Purposes Only */
        pReg = BaseAddress + ixf6048_RX_J1_CHAN_OFFSET(ChanNum);
        /* Get the R_HPT_C2 bits [10:8] prior to looking at ASTRA) */
        JnCfgMask = ixf6048_GET_RCV_J1_CNF_FORMAT(READ_FROM_HW(BaseAddress + ixf6048_R_HPT_C2_CHAN_OFFSET(ChanNum)));
        rc = GetRcvJnPathTrace(pReg,JnCfgMask, pTrace, pLength);
        break;
    case bb_TX_J0:
        pReg = BaseAddress + ixf6048_TX_J0_CHAN_OFFSET(ChanNum);
        /* Use the T_RMST_OP bits [7:6] to  Get the TraceFormat */
        JnCfgMask = ixf6048_GET_TX_J0_CNFG(READ_FROM_HW(BaseAddress + ixf6048_T_RMST_OP_CHAN_OFFSET(ChanNum)));

        rc = GetTxJnPathTrace(pReg,JnCfgMask, pTrace, pLength);
        break;
    case bb_TX_J1:
        pReg = BaseAddress + ixf6048_TX_J1_CHAN_OFFSET(ChanNum);
        /* Use the T_HPT_OPC bits [5:4] for TraceFormat/StrgLen */
        JnCfgMask = ixf6048_GET_TX_J1_CNFG(READ_FROM_HW(BaseAddress + ixf6048_T_HPT_OPC_CHAN_OFFSET(ChanNum)));

        rc = GetTxJnPathTrace(pReg,JnCfgMask, pTrace, pLength);
        break;
    default:
        rc = bb_INV_TRACE_TYPE;
        break;
    }

    return rc;

}

/***********************************************************************
 * Procedure Name: GetRcvJnPathTrace                                   
 *                                                                      
 * Description: Get the Rcv J0/J1 Path Trace Bytes
 *                                                                      
 * Conditions for Use:                                                  
 *          pTrace      Must point to A Valid Empty String
 *                                                                      
 * Inputs:  
 *          pReg        Pointer to the Rx Path Trace Register                                                                      
 *          pJnCgMask   Mask Taken from the RxJn_Cfg bits
 * Outputs:
 *          pLength         The length of pTrace.
 *
 * Returns:
 *          bb_Error_e  Returns Error Condition if Not successful
 *          pTrace      The String the load from the Trace Bytes
 *
 * End                                                                  
 **********************************************************************/
static bb_Error_e GetRcvJnPathTrace(ushort* pReg,
    ushort  pJnCfgMask,
    char*   pTrace,
    ushort* pLength)
{
    bb_Error_e        rc = bb_NO_ERROR;
    boolean           CfgOK = TRUE;

    /* IXF6048_RCV_TRACE_FORMAT_ARR Converts the Actual RcvJn_Cfg bits to bb_TraceFormat_e */
    switch(IXF6048_RCV_TRACE_FORMAT_ARR[pJnCfgMask])
    {
    case bb_64_BYTE_WITH_LF_CR:     /* 64 bytes with LF,CR at end */
    case bb_64_BYTE_FREE_FORM:      /* 64 bytes, free form */
        CfgOK = GetOhtJnString(pReg,pTrace,64);
        *pLength = 64;
        break;
    case bb_16_BYTE_WITH_CRC7:      /* 16 bytes, starting with CRC7 (SDH) */
        CfgOK = GetOhtJnString(pReg,pTrace,16);
        *pLength = 16;
        break;
    case bb_IGNORE_RX_TRACE:        /* Ignore trace on Rx */
        pTrace = NULL;
        CfgOK = TRUE;
        *pLength = 64;
        break;
    case bb_1_BYTE:                 /* 1 byte */
        CfgOK = GetOhtJnString(pReg,pTrace,1);
        *pLength = 1;
        break;
    case bb_DEFAULT_TX_TRACE:       /* Not Valid format for Expected Trace */
    default :
        *pLength = 0;
        /* Nothing supported Here */
        return bb_INV_EXP_JN_FMT;
    }

    if (!CfgOK)
    {
        rc = bb_JN_TRACE_WRITE_FAIL;
    }

    return rc;
}
/***********************************************************************
 * Procedure Name: GetTxJnPathTrace                                   
 *                                                                      
 * Description: Get the Rcv J0/J1 Path Trace Bytes
 *                                                                      
 * Conditions for Use:                                                  
 *          pTrace      Must point to A Valid Empty String
 *                                                                      
 * Inputs:  
 *          pReg        Pointer to the Tx Path Trace Register                                                                      
 *          pJnCgMask   Mask Taken from the TxJn_Cfg bits
 *
 * Returns:
 *          bb_Error_e  Returns Error Condition if Not successful
 *          pTrace      The String to load from the Trace Bytes
 *
 * End                                                                  
 **********************************************************************/
static bb_Error_e GetTxJnPathTrace(ushort* pReg,
    ushort  pJnCfgMask,
    char*   pTrace,
    ushort* pLength)
{
    bb_Error_e  rc = bb_NO_ERROR;
    boolean     CfgOK = TRUE;

    /* IXF6048_TX_TRACE_FORMAT_ARR Converts the Actual XmtJnStrgLen bits to bb_TraceFormat_e */
    switch(IXF6048_TX_TRACE_FORMAT_ARR[pJnCfgMask])
    {
    case bb_64_BYTE_WITH_LF_CR:     /* 64 bytes with LF,CR at end -- Not an Readible Tx format setting*/
    case bb_64_BYTE_FREE_FORM:      /* 64 bytes, free form */
        CfgOK = GetOhtJnString(pReg,pTrace,64);
        *pLength = 64;
        break;
    case bb_16_BYTE_WITH_CRC7:      /* 16 bytes, starting with CRC7 (SDH) */
        CfgOK = GetOhtJnString(pReg,pTrace,16);
        *pLength = 16;
        break;
    case bb_1_BYTE:                 /* 1 byte */
        CfgOK = GetOhtJnString(pReg,pTrace,1);
        *pLength = 1;
        break;
    case bb_DEFAULT_TX_TRACE:       /* Default Tx Trace String (0x01) */
        CfgOK = GetOhtJnString(pReg,pTrace,1);   /* Should be 01H */
        *pLength = 1;
        if (*pTrace != 0x01)
            CfgOK = FALSE;
        break;
    case bb_IGNORE_RX_TRACE:        /*  Not Valid format for Expected Trace */
    default :
        /* Nothing supported Here */
        return bb_INV_EXP_JN_FMT;
    }

    if (!CfgOK)
    {
        rc = bb_JN_TRACE_WRITE_FAIL;
    }
    return rc;


}

/***********************************************************************
 * Procedure Name: GetOhtJnString                                   
 *                                                                      
 * Description: Configure the J0/J1 Trace 
 *                                                                      
 * Conditions for Use:                                                  
 *          pWorkString     Must be pointer to Valid Empty String of StringLen
 *
 *
 * Returns: bMatch          TRUE if Successful
 *          pWorkingString  Copy of the Stored String     
 *                                                                      
 * End                                                                  
 **********************************************************************/
static boolean GetOhtJnString(ushort* pReg,
    char*   pString, 
    int     StringLen)
{
    boolean  bMatch = TRUE;
    int      iByteIndex;
    int      iWaitTest;
    char     CharByte;
    char*    pWorkString = pString;

#ifdef UNIT_TEST_6048
    if (StringLen == 1) 
        /* Jn is a Single Byte */
        printf("GetOhtJnString: 1-byte WorkString \n");
    else if (StringLen == 16)
        /* CRC7, or space for it, at start of string */
        printf("GetOhtJnString: 16-byte WorkString with CRC7 at start \n");
    else
        printf("GetOhtJnString: 64-byte WorkString\n");
#endif
    /* Setup to Read From Trace Register */
    /* See Reg R_J0_ESTRA in Data Sheet for the Write/Read process */
    for (iByteIndex=0; iByteIndex<StringLen; ++ iByteIndex)
    {  /* Wait for Reg Access */
        iWaitTest = 0;
        /* Initiate the Read */
        ixf6048_READ_TRACE(pReg, iByteIndex);
        /* Wait for the Read Access */
        while(!(ixf6048_READ_TRACE_DONE(READ_FROM_HW(pReg))))
        {  /* Sanity Check */
            if(iWaitTest++ == 100)
            {
                bMatch = FALSE;  /* Report that an Error has occured to calling Function */
                break;
            }
        }
        CharByte = ixf6048_GET_TRACE_BYTE(READ_FROM_HW(pReg));
        *pWorkString++ = CharByte;
    }
    return bMatch;
}
/***********************************************************************
 * Procedure Name: ixf6048_GetCounters
 *
 * Description: Get a set of Rx/Tx Counters for a selected OHT type, ATM or POS 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *  (bb_ChipData_t *  pChipData,   -> BaseAddress of chip
 *   bb_SelCounters_e SelCounters, to select the set of counters
 *   uint             ChanNum,     Channel 0-3
 *   void*            pCounters);  -> place to put Counters
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetCounters(bb_ChipData_t*   pChipData,
    uint             ChanNum,    
    bb_SelCounters_e SelCounters,
    void*            pCounters)
{
    bb_Error_e               rc = bb_NO_ERROR;
    ushort*                  usCounter;      /* 16 bit Counter */
    ulong*                   ulCounter;      /* 32 bit Counter */
    bb_RxRstCounters_t*      pRxRst;         /* RST Counters*/
    bb_RxMstCounters_t*      pRxMst;         /* MST Counters*/
    bb_RxHptCounters_t*      pRxHpt;         /* HPT Counters*/
    bb_AuPtrCounters_t*      pRxMsa;         /* MSA Counters*/
    ixf6048_OhtCounters_t*   pOhtCounters;   /* Combincation of Above Counters */
    ixf6048_AtmCounters_t*   pAtm;           /* ATM Counters*/
    ixf6048_PosCounters_t*   pPos;           /* POS Counters */
    ixf6048_AllCounters_t*   pAll;           /* All Counters */
    boolean                  bCntrsBuffered= FALSE;
    ushort                   ChanMask=0;
    register ushort*         BaseAddress = pChipData->BaseAddress;

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    usCounter = (ushort *) pCounters;
    ulCounter = (ulong *) pCounters;

    switch(SelCounters)
    {
    case bb_OOF_CNT:
        WRITE_TO_HW(BaseAddress + ixf6048_OOF_ECNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = (ushort) ixf6048_GET_OOF_CNT(BaseAddress,ChanNum);
        break;
    case bb_B1_BIP_ERR_CNT:
        /* B1 error count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_B1_ERRCNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = (ushort) ixf6048_GET_B1_ERRCNT(BaseAddress,ChanNum);
        break;
    case bb_B1_BLOCK_ERR_CNT:
        /* B1 error count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_B1_ERRCNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = (ushort) ixf6048_GET_B1_ERRCNT(BaseAddress,ChanNum);
        break;
    case bb_RX_RST_COUNTERS:
        pRxRst = pCounters;
        GetRstCounters(BaseAddress, ChanNum, bCntrsBuffered, pRxRst);
        break;

    case bb_B2_BIP_ERR_CNT:
        /* Number B2  Bit errors */
        WRITE_TO_HW(BaseAddress + ixf6048_B2_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        *ulCounter   = (ushort) ixf6048_GET_B2_BIPCNT(BaseAddress,ChanNum);
        break;
    case bb_MST_REI_BIP_ERR_CNT:
        /* Number REI Bit errors */
        WRITE_TO_HW(BaseAddress + ixf6048_MR_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        *ulCounter   = (ushort) ixf6048_GET_MST_REI_BIPCNT(BaseAddress,ChanNum);
        break;
    case bb_B2_BLOCK_ERR_CNT:
        /* Number B2  Block errors */
        WRITE_TO_HW(BaseAddress + ixf6048_B2_BLKCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        *ulCounter   = (ushort) ixf6048_GET_B2_BLKCNT(BaseAddress, ChanNum);
        break;
    case bb_MST_REI_BLOCK_ERR_CNT:
        /* Number REI Block errors */
        WRITE_TO_HW(BaseAddress + ixf6048_MR_BLKCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        *ulCounter   = (ushort) ixf6048_GET_MST_REI_BLKCNT(BaseAddress,ChanNum);
        break;
    case bb_RX_MST_COUNTERS:
        pRxMst = pCounters;
        GetMstCounters(BaseAddress, ChanNum, bCntrsBuffered, pRxMst);
        break;

    case bb_HPM_REI_BIP_CNT:
        /* HPT REI count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_HPTREI_CNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_HPT_REI_CNT(BaseAddress,ChanNum);
        break;
    case bb_HPM_REI_BLOCK_CNT:
        /* HPT REI count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_HPTREI_CNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_HPT_REI_CNT(BaseAddress,ChanNum);
        break;
    case bb_B3_BIP_ERR_CNT:
        /* B3 Error count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_B3_ECNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_B3_ERRCNT(BaseAddress,ChanNum);
        break;
    case bb_B3_BLOCK_ERR_CNT:
        /* B3 Error count: Bit or Block errs, as cfg'd */
        WRITE_TO_HW(BaseAddress + ixf6048_B3_ECNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_B3_ERRCNT(BaseAddress,ChanNum);
        break;
    case bb_RX_HPT_COUNTERS:
        pRxHpt = pCounters;
        GetHptCounters(BaseAddress, ChanNum, bCntrsBuffered, pRxHpt);
        break;

    case bb_IN_AU_NCNT:
        /* Count of Negative AU Ptr Justifications */
        WRITE_TO_HW(BaseAddress + ixf6048_R_AU_NCNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_R_AU_NCNT(BaseAddress,ChanNum);
        break;
    case bb_IN_AU_PCNT:
        /* Count of Positive AU Ptr Justifications */
        WRITE_TO_HW(BaseAddress + ixf6048_R_AU_PCNT_CHAN_OFFSET(ChanNum), 0);
        *usCounter   = ixf6048_GET_R_AU_PCNT(BaseAddress,ChanNum);
        break;
    case bb_RX_MSA_COUNTERS:
        pRxMsa = pCounters;
        GetMsaCounters(BaseAddress, ChanNum, bCntrsBuffered, pRxMsa);
        break;

    case bb_OHT_COUNTERS:   /* Combination of all Counters Above */
        pOhtCounters = pCounters;

        ChanMask = ixf6048_MASK_OHT_CHAN_CNTRS(ChanNum);
        ixf6048_BUFFER_COUNTERS(BaseAddress, ChanMask);
        bCntrsBuffered = TRUE;

        GetRstCounters(BaseAddress, ChanNum, 
            bCntrsBuffered, &pOhtCounters->RxRst);
        GetMstCounters(BaseAddress, ChanNum, 
            bCntrsBuffered, &pOhtCounters->RxMst);
        GetHptCounters(BaseAddress, ChanNum, 
            bCntrsBuffered, &pOhtCounters->RxHpt);
        GetMsaCounters(BaseAddress, ChanNum, 
            bCntrsBuffered, &pOhtCounters->RxMsa);
        break;

    case bb_ATM_RX_CELL_CNT:
        /* 24-bit count of accepted Rx ATM cells */
        *ulCounter   = ReadCounterLong(BaseAddress,
            (ushort) ixf6048_R_ACELLCNT_CHAN_MSB_OFFSET(ChanNum), 
            (ushort) ixf6048_R_ACELLCNT_CHAN_LSB_OFFSET(ChanNum),
            (uint) bCntrsBuffered);
        break;
    case bb_ATM_RX_IDLE_CELL_CNT:
        /* 24-bit count of idle/unassigned cells */
        *ulCounter   = ReadCounterLong(BaseAddress,
            (ushort) ixf6048_R_ICELLCNT_CHAN_MSB_OFFSET(ChanNum),
            (ushort) ixf6048_R_ICELLCNT_CHAN_LSB_OFFSET(ChanNum),
            (uint) bCntrsBuffered );

        break;
    case bb_ATM_RX_COR_HEC_ERR_CNT:
        /* 16-bit count of correctable HEC errors */
        *usCounter   = (ushort)ReadCounterShort(BaseAddress,
            (ushort) ixf6048_R_CHECNT_CHAN_OFFSET(ChanNum),
            (uint) bCntrsBuffered);
        break;
    case bb_ATM_RX_UNCOR_HEC_ERR_CNT:
        /* 16-bit count of uncorrectable HEC errors */
        *usCounter   = (ushort)ReadCounterShort(BaseAddress,
            (ushort) ixf6048_R_UHECNT_CHAN_OFFSET(ChanNum),
            (uint) bCntrsBuffered );
        break;

    case bb_ATM_RX_OVFL_CELL_CNT:
        /* 16-bit count of cells lost to FIFO overrun */
        *usCounter   = (ushort)ReadCounterShort(BaseAddress,
            (ushort) ixf6048_R_CFOCNT_CHAN_OFFSET(ChanNum),
            (uint) bCntrsBuffered );
        break;

    case bb_ATM_TX_CELL_CNT:
        /* 24-bit count of Tx ATM cells */
        *ulCounter   = ReadCounterLong(BaseAddress,
            (ushort) ixf6048_T_ACELLCNT_CHAN_MSB_OFFSET(ChanNum),
            (ushort) ixf6048_T_ACELLCNT_CHAN_LSB_OFFSET(ChanNum),
            (uint) bCntrsBuffered );
        break;
    case bb_ATM_TX_IDLE_CELL_CNT:
        /* 24-bit count of idle/unassigned cells sent */
        *ulCounter   = ReadCounterLong(BaseAddress,
            (ushort) ixf6048_T_ICELLCNT_CHAN_MSB_OFFSET(ChanNum),
            (ushort) ixf6048_T_ICELLCNT_CHAN_LSB_OFFSET(ChanNum),
            (uint) bCntrsBuffered  );
        break;

    case bb_ATM_COUNTERS:
        pAtm = pCounters;
        GetAtmCounters(BaseAddress,ChanNum,pAtm );
        break;

    case bb_POS_RX_GOOD_FRM_CNT:
        /* 27-bit count of accepted Rx frames */
        *ulCounter = ixf6048_GET_RX_FRAME_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_GOOD_BYTE_CNT:
        /* 29-bit count of Rx Bytes (in OK frames, or in all frames, as configured by
        RxPosByteCountConfig in RxPosChanConfig) */
        *ulCounter = ixf6048_GET_RX_BYTE_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_ABORTED_FRM_CNT:
        /* 20-bit count of Aborted frames */
        *ulCounter = ixf6048_GET_RX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_FCS_ERR_FRM_CNT:
        /* 20-bit count of frames with FCS errors */
        *ulCounter = ixf6048_GET_RX_FCS_ERROR_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_FIFO_OVERFLOW_CNT:
        /* 16-bit count of frames lost to FIFO overflow*/
        *usCounter = ixf6048_GET_RX_FIFO_OVERFLOW_COUNTER(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_MIN_PLE_CNT:
        /* 27-bit count of short frames received */
        *ulCounter = ixf6048_GET_RX_MIN_PACKET_LEN_ERR_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_RX_MAX_PLE_CNT:
        /* 16-bit count of long frames received */
        *usCounter = ixf6048_GET_RX_MAX_PACKET_LEN_ERR_CNT(BaseAddress, ChanNum);
        break;
    case bb_POS_TX_FRM_CNT:
        /* 27-bit count of (non-aborted) frames sent */
        *ulCounter = ixf6048_GET_TX_FRAME_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_TX_BYTE_CNT:
        /* 29-bit count of all Tx bytes put into SPE */
        *ulCounter = ixf6048_GET_TX_BYTE_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_TX_ABORTED_FRAME_CNT:
        /* 20-bit count of frames aborted by link layer*/
        *ulCounter = ixf6048_GET_TX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
        break;
    case bb_POS_TX_FIFO_UFLOW_CNT:
        /* 16-bit count of frames aborted by underflow */
        *usCounter = ixf6048_GET_TX_FIFO_UFLOW_COUNT(BaseAddress, ChanNum);
        break;

    case bb_POS_COUNTERS:
        pPos = pCounters;
        GetPosCounters(BaseAddress,ChanNum,pPos);
        break;
    case bb_ALL_6048_COUNTERS:   /* All Counters */
        pAll = pCounters;

        ChanMask = ixf6048_MASK_OHT_CHAN_CNTRS(ChanNum);
        ixf6048_BUFFER_COUNTERS(BaseAddress,ChanMask);
        bCntrsBuffered = TRUE;

        GetRstCounters(BaseAddress, ChanNum, 
            bCntrsBuffered, &pAll->OhtCounters.RxRst);
        GetMstCounters(BaseAddress, ChanNum,
            bCntrsBuffered, &pAll->OhtCounters.RxMst);
        GetHptCounters(BaseAddress, ChanNum,
            bCntrsBuffered, &pAll->OhtCounters.RxHpt);
        GetMsaCounters(BaseAddress, ChanNum,
            bCntrsBuffered, &pAll->OhtCounters.RxMsa);
        if(pAll->ChanMode ==  ixf6048_eATM_MODE_CHAN)
            GetAtmCounters(BaseAddress, ChanNum, &pAll->u.Atm );
        /* Must be POS */
        else if (pAll->ChanMode ==  ixf6048_ePOS_MODE_CHAN)
            GetPosCounters(BaseAddress, ChanNum,&pAll->u.Pos);

        rc = bb_INV_CHANEL_MODE_USED;
        break;
    default :
        rc = bb_INV_SEL_COUNTERS;
        break;
    }
    return rc;
}
/***********************************************************************
 * Procedure Name: GetRstCounters
 *
 * Description: Get the set of RST Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 * 
 * Inputs:
 *     bCntrsbuffered   TRUE if the OHT counters have been loaded 
 *                      prior to Calling this function (using the 
 *                      BUF_ACNTS register -- Buff All Counters 
 *                      Global register. 
 *
 *     
 *
 * End
 **********************************************************************/
static void GetRstCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             bCntrsbuffered,
    bb_RxRstCounters_t* pRxRst)
{
    if(!bCntrsbuffered)
    {  /* If the Buffers have not been loaded then trigger the individual Regs */
        WRITE_TO_HW(BaseAddress + ixf6048_OOF_ECNT_CHAN_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_B1_ERRCNT_CHAN_OFFSET(ChanNum), 0);
    }
    /* Out of Frame Event Counter */
    pRxRst->OofCnt   = ixf6048_GET_OOF_CNT(BaseAddress,ChanNum);
    /* B1 error count: Bit or Block errs, as cfg'd */
    pRxRst->B1ErrCnt = ixf6048_GET_B1_ERRCNT(BaseAddress,ChanNum);
}

/***********************************************************************
 * Procedure Name: GetMstCounters
 *
 * Description: Get the set of MST Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 * Inputs:
 *     bCntrsbuffered   TRUE if the OHT counters have been loaded 
 *                      prior to Calling this function (using the 
 *                      BUF_ACNTS register -- Buff All Counters 
 *                      Global register. 
 *
 *
 *
 * End
 **********************************************************************/
static void GetMstCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             bCntrsbuffered,
    bb_RxMstCounters_t* pRxMst)
{
    if(!bCntrsbuffered)
    {  /* If the Buffers have not been loaded then trigger the individual Count Regs */
        WRITE_TO_HW(BaseAddress + ixf6048_B2_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_MR_BIPCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_B2_BLKCNT_CHAN_MSB_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_MR_BLKCNT_CHAN_MSB_OFFSET(ChanNum), 0);
    }
    /* Number B2  Bit errors */
    pRxMst->B2BitErrCnt  = ixf6048_GET_B2_BIPCNT(BaseAddress,ChanNum);
    /* Number REI Bit errors */
    pRxMst->ReiBitErrCnt = ixf6048_GET_MST_REI_BIPCNT(BaseAddress,ChanNum);
    /* Number B2  Block errors */
    pRxMst->B2BlockErrCnt= ixf6048_GET_B2_BLKCNT(BaseAddress, ChanNum);
    /* Number REI Block errors */
    pRxMst->ReiBlockErrCnt=ixf6048_GET_MST_REI_BLKCNT(BaseAddress,ChanNum);
}

/***********************************************************************
 * Procedure Name: GetHptCounters
 *
 * Description: Get the set of HPT Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 * Inputs:
 *     bCntrsbuffered   TRUE if the OHT counters have been loaded 
 *                      prior to Calling this function (using the 
 *                      BUF_ACNTS register -- Buff All Counters 
 *                      Global register. 
 *
 *
 * End
 **********************************************************************/
static void GetHptCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             bCntrsbuffered,
    bb_RxHptCounters_t* pRxHpt)
{
    if(!bCntrsbuffered)
    {  /* If the Buffers have not been loaded then trigger the individual Count Regs */
        WRITE_TO_HW(BaseAddress + ixf6048_HPTREI_CNT_CHAN_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_B3_ECNT_CHAN_OFFSET(ChanNum), 0);
    }
    /* HPT REI count: Bit or Block errs, as cfg'd */
    pRxHpt->ReiCnt = ixf6048_GET_HPT_REI_CNT(BaseAddress,ChanNum);
    /* B3 Error count: Bit or Block errs, as cfg'd */
    pRxHpt->B3ErrCnt=ixf6048_GET_B3_ERRCNT(BaseAddress,ChanNum);
}

/***********************************************************************
 * Procedure Name: GetMsaCounters
 *
 * Description: Get the set of MSA Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 * Inputs:
 *     bCntrsbuffered   TRUE if the OHT counters have been loaded 
 *                      prior to Calling this function (using the 
 *                      BUF_ACNTS register -- Buff All Counters 
 *                      Global register. 
 *
 *
 * End
 **********************************************************************/
static void GetMsaCounters(ushort*             BaseAddress,
    uint                ChanNum,    
    boolean             bCntrsbuffered,
    bb_AuPtrCounters_t* pRxMsa)
{
    if(!bCntrsbuffered)
    {  /* If the Buffers have not been loaded then trigger the individual Count Regs */
        WRITE_TO_HW(BaseAddress + ixf6048_R_AU_NCNT_CHAN_OFFSET(ChanNum), 0);
        WRITE_TO_HW(BaseAddress + ixf6048_R_AU_PCNT_CHAN_OFFSET(ChanNum), 0);
    }
    /* Count of Negative AU Ptr Justifications */
    pRxMsa->InAuNegJustCnt =ixf6048_GET_R_AU_NCNT(BaseAddress,ChanNum);
    /* Count of Positive AU Ptr Justifications */
    pRxMsa->InAuPosJustCnt =ixf6048_GET_R_AU_PCNT(BaseAddress,ChanNum);
}

/***********************************************************************
 * Procedure Name: GetAtmCounters
 *
 * Description: Get the set of ATM Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 *
 *
 * End
 **********************************************************************/
static void GetAtmCounters(ushort*                BaseAddress,
    uint                   ChanNum,    
    ixf6048_AtmCounters_t* pAtm)
{
    ushort ChanMask = ixf6048_MASK_ATM_POS_CHAN_CNTRS(ChanNum);

    /* Load all ATM Counters for this Channel */
    /* ixf6048_BUFFER_COUNTERS(BaseAddress, ChanMask); */
    WRITE_TO_HW(BaseAddress + ixf6048_BUF_ACNTS_OFFSET, ChanMask);

    /* 24-bit count of accepted Rx ATM cells */
    /*pAtm->RxCellCnt = ixf6048_GET_RX_ATM_CELL_CNT(BaseAddress,ChanNum);*/
    pAtm->RxCellCnt = ReadCounterLong(BaseAddress,
        (ushort) ixf6048_R_ACELLCNT_CHAN_MSB_OFFSET(ChanNum), 
        (ushort) ixf6048_R_ACELLCNT_CHAN_LSB_OFFSET(ChanNum),
        (uint) TRUE);

    /* 24-bit count of idle/unassigned cells */
    pAtm->RxIdleCellCnt = ReadCounterLong(BaseAddress,
        (ushort) ixf6048_R_ICELLCNT_CHAN_MSB_OFFSET(ChanNum),
        (short) ixf6048_R_ICELLCNT_CHAN_LSB_OFFSET(ChanNum),
        (uint) TRUE );

    /* 16-bit count of correctable HEC errors */
    pAtm->RxCorHecErrCnt= (ushort) ReadCounterShort( BaseAddress,
        (ushort) ixf6048_R_CHECNT_CHAN_OFFSET(ChanNum), (uint) TRUE );

    /* 16-bit count of uncorrectable HEC errors */
    pAtm->RxUncorHecErrCnt = (ushort)ReadCounterShort( BaseAddress, 
        (ushort) ixf6048_R_UHECNT_CHAN_OFFSET(ChanNum), (uint) TRUE );

    /* 16-bit count of cells lost to FIFO overrun */
    pAtm->RxOvflCellCnt = (ushort)ReadCounterShort( BaseAddress,
        (ushort) ixf6048_R_CFOCNT_CHAN_OFFSET(ChanNum), (uint) TRUE );

    /*-------- Tx ATM Counters --------*/
    /* 24-bit count of Tx ATM cells */
    pAtm->TxCellCnt = ReadCounterLong(BaseAddress, 
        (ushort) ixf6048_T_ACELLCNT_CHAN_MSB_OFFSET(ChanNum),
        (ushort) ixf6048_T_ACELLCNT_CHAN_LSB_OFFSET(ChanNum),
        (uint) TRUE );

    /* 24-bit count of idle/unassigned cells sent */
    pAtm->TxIdleCellCnt = ReadCounterLong(BaseAddress, 
        (ushort) ixf6048_T_ICELLCNT_CHAN_MSB_OFFSET(ChanNum),
        (ushort) ixf6048_T_ICELLCNT_CHAN_LSB_OFFSET(ChanNum),
        (uint) TRUE );
}
/***********************************************************************
 * Procedure Name: GetPosCounters
 *
 * Description: Get the set of POS Counters 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *    
 *
 *
 * End
 **********************************************************************/
static void GetPosCounters(ushort*                BaseAddress,
    uint                   ChanNum,    
    ixf6048_PosCounters_t* pPos)
{
    ushort ChanMask = ixf6048_MASK_ATM_POS_CHAN_CNTRS(ChanNum);

    /* Load all POS Counters for this Channel */
    ixf6048_BUFFER_COUNTERS(BaseAddress, ChanMask);

    /* 27-bit count of accepted Rx frames */
    pPos->RxFrameCnt =ixf6048_GET_RX_FRAME_CNT(BaseAddress,ChanNum);
    /* 29-bit count of Rx Bytes (in OK frames, or in all frames, as configured by
    RxPosByteCountConfig in RxPosChanConfig) */
    pPos->RxByteCnt =ixf6048_GET_RX_BYTE_CNT(BaseAddress,ChanNum);
    /* 20-bit count of Aborted frames */
    pPos->RxAbtFrameCnt =ixf6048_GET_RX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
    /* 20-bit count of frames with FCS errors */
    pPos->RxFcsErrCnt =ixf6048_GET_RX_FCS_ERROR_CNT(BaseAddress,ChanNum);
    /* 16-bit count of frames lost to FIFO overflow*/
    pPos->RxOvflFrameCnt =ixf6048_GET_RX_FIFO_OVERFLOW_COUNTER(BaseAddress,ChanNum);
    /* 27-bit count of short frames received */
    pPos->RxShortFrameCnt =ixf6048_GET_RX_MIN_PACKET_LEN_ERR_CNT(BaseAddress,ChanNum);
    /* 16-bit count of long frames received */
    pPos->RxLongFrameCnt =ixf6048_GET_RX_MAX_PACKET_LEN_ERR_CNT(BaseAddress, ChanNum);

    /*-------- Tx POS Counters  --------*/
    /* 27-bit count of (non-aborted) frames sent */
    pPos->TxFrameCnt =ixf6048_GET_TX_FRAME_CNT(BaseAddress,ChanNum);
    /* 29-bit count of all Tx bytes put into SPE */
    pPos->TxByteCnt =ixf6048_GET_TX_BYTE_CNT(BaseAddress,ChanNum);
    /* 20-bit count of frames aborted by link layer*/
    pPos->TxAbtFrameCnt =ixf6048_GET_TX_ABORTED_FRAME_CNT(BaseAddress,ChanNum);
    /* 16-bit count of frames aborted by underflow */
    pPos->TxUnflFrameCnt =ixf6048_GET_TX_FIFO_UFLOW_COUNT(BaseAddress, ChanNum);
}

/***********************************************************************
 * Procedure Name: ixf6048_SetOhByte
 *
 * Description: Function to Set an Overhead Byte: 
 *                      K1,K2, Expected/Tx C2, HPT RDI in G1 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *   bb_ChipData_t*   pChipData,   -> Initialized Chip Data 
 *   bb_SelOhByte_e   SelOhByte,   to select a particular Overhead byte 
 *   int              ChanNum,     Channel Num = 0-3 
 *   uchar            OhByte       value of selected Overhead byte 
 *
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_SetOhByte  (bb_ChipData_t*   pChipData, 
    uint             ChanNum,
    bb_SelOhByte_e   SelOhByte,
    ushort            OhByte)
{
    uint             RegOffset;
    char*            pRegTitle;
    register ushort  RegVal = 0;
    register ushort* BaseAddress= pChipData->BaseAddress;

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    switch(SelOhByte)
    {
    case bb_TX_K1:
        RegOffset = ixf6048_MP_TK2K1_CHAN_OFFSET(ChanNum);
        RegVal    = (ixf6048_GET_TX_K2_BYTE(BaseAddress, ChanNum) << 8) | OhByte;
        pRegTitle = "Tx_K1";
        break;
    case bb_TX_K2:
        RegOffset   = ixf6048_MP_TK2K1_CHAN_OFFSET(ChanNum);
        ;
        RegVal    = ixf6048_GET_TX_K1_BYTE(BaseAddress, ChanNum) | (OhByte << 8);
        pRegTitle   = "Tx_K2";
        break;
    case bb_K1_K2:
        RegOffset = ixf6048_MP_TK2K1_CHAN_OFFSET(ChanNum);
        RegVal    = OhByte;
        pRegTitle   = "Tx_K2";
        break;
    case bb_TX_S1:
        RegOffset = ixf6048_MP_TS1_CHAN_OFFSET(ChanNum);
        RegVal    |= OhByte;
        pRegTitle = "Tx_S1";
        break;
    case bb_RECEIVED_C2:
        RegOffset = ixf6048_R_C2_CHAN_OFFSET(ChanNum);
        RegVal    |= OhByte;
        pRegTitle = "RECEIVED_C2";
        break;
    case bb_EXPECTED_C2:
        RegOffset = ixf6048_EXP_C2_CHAN_OFFSET(ChanNum);
        RegVal    |= OhByte;
        pRegTitle   = "EXP_C2";
        break;
    case bb_TX_C2:
        RegOffset = ixf6048_MP_TC2_CHAN_OFFSET(ChanNum);
        RegVal   |= OhByte;
        pRegTitle = "Tx_C2";
        break;
    case bb_TX_HPT_RDI_IN_G1:
        RegOffset = ixf6048_MP_THPTRDI_CHAN_OFFSET(ChanNum);
        RegVal    |= (OhByte & (ixf6048_mXMT_HPT_RDI | ixf6048_bXMT_G1_SP_BIT) );
        pRegTitle = "Tx_HPT_RDI_IN_G1";
        break;
    case bb_TX_K3:
    default:
        return (bb_INV_SEL_OH_BYTE);
    }

    /* Set OH byte in Reg */
    WRITE_TO_HW(BaseAddress + RegOffset, RegVal);

    return bb_NO_ERROR;
}

/***********************************************************************
 * Procedure Name: ixf6048_GetOhBytes
 *
 * Description: Function to Get Overhead Bytes: 
 *                      Rx MST, Rx HPT 
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *   bb_ChipData_t*   pChipData,   -> Initialized Chip Data 
 *   bb_SelOhBytes_e  SelOhBytes,  to select the Overhead Bytes 
 *   int              ChanNum,     Channel Num = 0-3 
 *   void*            pOhBytes     -> place to put Overhead Bytes, of type:
 *                                      bb_RxMstBytes_t, or
 *                                      ixf6048_RxHptBytes_t
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetOhBytes (bb_ChipData_t*   pChipData,
    uint             ChanNum,   
    bb_SelOhBytes_e  SelOhBytes,
    void*            pOhBytes)
{
    bb_Error_e            rc = bb_NO_ERROR;
    bb_RxMstBytes_t*      pRxMstBytes;
    ixf6048_RxHptBytes_t* pRxHptBytes;
    register ushort       k1k2RegVal;
    register ushort*      BaseAddress= pChipData->BaseAddress;
    uchar *ucByte;
    ushort *usByte;

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    switch(SelOhBytes)
    {
    case bb_K1:
        ucByte = pOhBytes;
        /* Read the entire K1K2 Register and then split Kn bytes */
        k1k2RegVal = ixf6048_GET_K1K2(BaseAddress,ChanNum);
        *ucByte = ixf6048_GET_K1(k1k2RegVal);
        break;
    case bb_K2:
        ucByte = pOhBytes;
        /* Read the entire K1K2 Register and then split Kn bytes */
        k1k2RegVal = ixf6048_GET_K1K2(BaseAddress,ChanNum);
        *ucByte = ixf6048_GET_K2(k1k2RegVal);
        break;
    case bb_K1_K2:
        usByte = pOhBytes;
        /* Read the entire K1K2 Register and then split Kn bytes */
        k1k2RegVal = ixf6048_GET_K1K2(BaseAddress,ChanNum);
        *usByte = (ixf6048_GET_K2(k1k2RegVal) << 8) | ixf6048_GET_K1(k1k2RegVal);
        break;
    case bb_RECEIVED_C2:
        ucByte = pOhBytes;
        *ucByte  = ixf6048_GET_RX_C2(BaseAddress,ChanNum);
        break;
    case bb_EXPECTED_C2:
        ucByte = pOhBytes;
        *ucByte  = (uchar) READ_FROM_HW(BaseAddress + ixf6048_EXP_C2_CHAN_OFFSET(ChanNum));
        break;
    case bb_TX_C2:
        ucByte = pOhBytes;
        *ucByte  = (uchar) READ_FROM_HW(BaseAddress + ixf6048_MP_TC2_CHAN_OFFSET(ChanNum));
        break;
    case bb_S1:
        ucByte = pOhBytes;
        *ucByte = ixf6048_GET_S1(BaseAddress,ChanNum);
        break;
    case bb_RX_HPT_RDI_IN_G1:
        ucByte = pOhBytes;
        *ucByte = ixf6048_GET_RX_HPT_RDI(BaseAddress,ChanNum);
        break;
    case bb_TX_HPT_RDI_IN_G1:
        ucByte = pOhBytes;
        *ucByte = (uchar) READ_FROM_HW(BaseAddress + ixf6048_MP_THPTRDI_CHAN_OFFSET(ChanNum));
        break;
    case bb_RX_MST_BYTES:
        pRxMstBytes = pOhBytes;
        /* Read the entire K1K2 Register and then split Kn bytes */
        k1k2RegVal = ixf6048_GET_K1K2(BaseAddress,ChanNum);

        pRxMstBytes->K1 = ixf6048_GET_K1(k1k2RegVal);
        pRxMstBytes->K2 = ixf6048_GET_K2(k1k2RegVal);
        pRxMstBytes->S1 = ixf6048_GET_S1(BaseAddress,ChanNum);
        break;
    case bb_RX_HPT_BYTES:
        pRxHptBytes = pOhBytes;

        pRxHptBytes->C2  = ixf6048_GET_RX_C2(BaseAddress,ChanNum);
        pRxHptBytes->Rdi = ixf6048_GET_RX_HPT_RDI(BaseAddress,ChanNum);
        break;
    case bb_RX_NU_BYTES:
    default:
        rc = bb_INV_SEL_OH_BYTES;
        break;
    }

    return rc;
}

/***********************************************************************
 * Procedure Name: ixf6048_Reset
 *
 * Description:  
 *          Function to Reset (chip or channel) and then reconfigure it.
 *          To restart traffic, the chip or channel must be set back online,
 *          using ixf6048_SetChanOnline or ixf6048_SetChipOnline.
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *
 *  (bb_ChipData_t*      pChipData,  -> Initialized Chip Data
 *   ixf6048_ResetType_e  ResetType,  Specifies type of Chip/Chan Reset
 *   int                 ChanNum);   Channel Num = 0-3, if applicable
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_Reset (bb_ChipData_t*      pChipData,
    uint                ChanNum,
    ixf6048_ResetType_e ResetType)
{
    register ushort*         BaseAddress;
    uint                     iSetClr;
    register ushort*         pReg;
    ixf6048_ChipCfg_t*       pChipCfg= NULL;      /* -> OHT Chip cfg */
    ixf6048_ChanCfg_t*       pChanCfg = NULL;      /* -> Specific cfg */
    ixf6048_AlarmCfg_t*      pAlarmCfg= NULL;     /* -> All alarm cfg */
    bb_Error_e               rc = bb_NO_ERROR;
    ixf6048_Side_e           Side = ixf6048_eSIDE_BOTH;
    /* Convert each void* into valid, typed struct ptr */
    pChipCfg    = (ixf6048_ChipCfg_t*)(pChipData->pChipCfg);
    pAlarmCfg   = pChipData->pAlarmCfg;

    BaseAddress = pChipData->BaseAddress;

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    switch(ResetType)
    {
    case ixf6048_eRESET_CHIP:    /* Reset entire chip; then reconfig chip */
    case bb_RESET_CHIP:
        pReg = BaseAddress + ixf6048_SW_RESET_OFFSET;
        /* Set Bit 8 of Register */
        Ixf6048SetBit(pReg, ixf6048_bSW_RESET_CHIP);  /* Hold in RESET   */
        Ixf6048ClrBit(pReg, ixf6048_bSW_RESET_CHIP);  /* Clear RESET Bit */
        break;
    case ixf6048_eRESET_RX_CHAN: /* Reset Rx side of Channel; then re-cfg Rx Side */
    case bb_RESET_RX_CHAN:
        Side = ixf6048_eSIDE_RECEIVE;
        /* Set the Corresponding Channel Bits */
        iSetClr = 1;   /* Hold in Reset */
        ixf6048_RESET_RCV_CH(BaseAddress, ChanNum, iSetClr);
        iSetClr = 0;   /* End the Reset */
        ixf6048_RESET_RCV_CH(BaseAddress, ChanNum, iSetClr);

        /* Now, Re-initialize the Channel that was Reset on the RCV side */
        if (pChipCfg->pChanCfg[ChanNum])
        {
            pChanCfg =  pChipCfg->pChanCfg[ChanNum];
            /* Edit the Configuration of the Assigned Channel */
            rc = EditChanCfg(BaseAddress, ChanNum, Side, pChanCfg);
        }
        else
        {  /* If there is no Channel Pointer, then there is a problem */
            return bb_INVALID_CHAN_CFG;
        }

        if (pAlarmCfg) /* If alarms used */
            rc = EditAlarms(BaseAddress, Side, pChipCfg, pAlarmCfg);

        break;
    case ixf6048_eRESET_TX_CHAN: /* Reset Tx side of Channel; then recfg Tx Side */
    case bb_RESET_TX_CHAN:
        Side = ixf6048_eSIDE_TRANSMIT;
        /* Set the Corresponding Channel Bits */
        iSetClr = 1;   /* Hold in Reset */
        ixf6048_RESET_XMT_CH(BaseAddress, ChanNum, iSetClr);
        iSetClr = 0;   /* End the Reset */
        ixf6048_RESET_XMT_CH(BaseAddress, ChanNum, iSetClr);

        /* Now, Re-initialize the Channel that was Reset on the XMT side */
        if (pChipCfg->pChanCfg[ChanNum])
        {
            pChanCfg =  pChipCfg->pChanCfg[ChanNum];
            /* Edit the Configuration of the Assigned Channel*/
            rc =  EditChanCfg(BaseAddress, ChanNum, Side, pChanCfg);
        }
        else
        {  /* If there is no Channel Pointer, then there is a problem */
            return bb_INVALID_CHAN_CFG;
        }
        if (pAlarmCfg) /* If alarms used */
            rc = EditAlarms(BaseAddress, Side, pChipCfg,pAlarmCfg);

        break;
    case ixf6048_eRESET_CHAN:    /* Reset both sides of Channel; then recfg chan */
    case bb_RESET_CHAN:
        Side = ixf6048_eSIDE_BOTH;
        /* Set the Corresponding Channel Bits */
        iSetClr = 1;   /* Hold in Reset */
        ixf6048_RESET_CHANNEL(BaseAddress, ChanNum, iSetClr);
        iSetClr = 0;   /*End the Reset */
        ixf6048_RESET_CHANNEL(BaseAddress, ChanNum, iSetClr);

        /* Now re-initialize the data */
        if (pChipCfg->pChanCfg[ChanNum])
        {
            pChanCfg =  pChipCfg->pChanCfg[ChanNum];
            /* Edit the Configuration of Each Assigned Channel*/
            rc = EditChanCfg(BaseAddress, ChanNum,Side, pChanCfg);
        }
        else
        {  /* If there is no Channel Pointer, then there is a problem */
            return bb_INVALID_CHAN_CFG;
        }
        if (pAlarmCfg) /* If alarms used */
            rc = EditAlarms(BaseAddress, Side, pChipCfg, pAlarmCfg);

        break;

    case ixf6048_eRESET_RX_FIFO:
    case bb_RESET_RL_MSA_FIFO:
        pReg = BaseAddress + ixf6048_R_UICHCNF_CHAN_OFFSET(ChanNum);
        Ixf6048SetBit(pReg,ixf6048_bFIFO_RST);
        /* Pause for Several uSecs */
        for (iSetClr=0; iSetClr < 5; ++ iSetClr);
        Ixf6048ClrBit(pReg,ixf6048_bFIFO_RST);
        break;
    case ixf6048_eRESET_TX_FIFO:
    case bb_RESET_TT_MSA_FIFO:
        pReg = BaseAddress + ixf6048_T_UICHCNF_CHAN_OFFSET(ChanNum);
        Ixf6048SetBit(pReg,ixf6048_bFIFO_RST);
        /* Pause for Several uSecs */
        for (iSetClr=0; iSetClr < 5; ++ iSetClr);
        Ixf6048ClrBit(pReg,ixf6048_bFIFO_RST);
        break;
    default :
        return (bb_INV_RESET_TYPE);
    }
    return rc;
}

/***********************************************************************
 * Procedure Name: ixf6048_GetStatus
 *
 * Description:  
 * Function to Get Status: Rx AIS, Input Clock Activity, Far-end GFC Role
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 * The test for the far-end GFC Role depends upon having active ATM traffic.
 *
 * The far end is identified as a GFC controller if an ATM cell has been
 * received with a GFC Halt, SET-A, or SET-B command, since this function
 * was last called.
 *
 * The far end is identified as a GFC controlled device if an ATM cell has
 * been received with the GFC Controlled bit set, since this function
 * was last called.
 *
 *  (bb_ChipData_t*      pChipData,  -> Initialized Chip Data 
 *   ixf6048_SelStatus_e  SelStatus,  To select type of status to get
 *   int                 ChanNum,    Channel Num = 0-3 
 *   void*               pStatus);   -> place to put Status, of type
 *                                     ushort (for Rx AIS Status: Reg 0x7), or
 *                                     ixf6048_InClkStatus_t, or
 *                                     ixf6048_FarEndGfcRole_e
 *
 *
 * PDL:
 *
 * End
 **********************************************************************/
bb_Error_e ixf6048_GetStatus(bb_ChipData_t*      pChipData,
    uint                ChanNum,
    ixf6048_SelStatus_e SelStatus,
    void*               pStatus)
{
    register ushort*         pRxAisStat;
    register ushort          RegVal;
    ixf6048_InClkStatus_t*   pInClkStatus;
    ixf6048_FarEndGfcRole_e* pFarEndGfcRole;
    register ushort*         BaseAddress= pChipData->BaseAddress;

    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    switch(SelStatus)
    {
    case ixf6048_eRX_AIS_STATUS:    /* pStatus -> ushort: Reg 0x7) */
    case bb_RX_AIS_STATUS:
        pRxAisStat = pStatus;
        *pRxAisStat = (READ_FROM_HW(BaseAddress + ixf6048_S_AIS_OFFSET)) &
            ixf6048_GEN_S_AIS_ST(ChanNum);
        break;
    case ixf6048_eIN_CLK_STATUS:    /* pStatus -> ixf6048_InClkStatus_t */
    case bb_IN_CLK_STATUS:
        pInClkStatus = pStatus;
        pInClkStatus->Icmr1Reg = READ_FROM_HW(BaseAddress + ixf6048_ICMR1_OFFSET);
        pInClkStatus->Icmr2Reg = (READ_FROM_HW(BaseAddress + ixf6048_ICMR2_OFFSET)) &
            ixf6048_mICMR2;
        break;
    case ixf6048_eFAR_END_GFC_ROLE: /* pStatus -> ixf6048_FarEndGfcRole_e */
    case bb_FAR_END_GFC_ROLE:
        pFarEndGfcRole = pStatus;
        /* Read Clr-on-Read Reg 1 time */
        RegVal = READ_FROM_HW(BaseAddress + ixf6048_R_ACPCNF_CHAN_OFFSET(ChanNum));
        /*Mask the Value into the Return Pointer */
        *pFarEndGfcRole = RegVal &
            (ixf6048_bRCV_GFC_CONTROLLER | ixf6048_bRCV_GFC_CONTROLLED);
        break;
    default :
        return (bb_INV_SEL_STATUS_TYPE);
    }
    return bb_NO_ERROR;
}

/***********************************************************************
 * Procedure Name: ixf6048_SetCfg
 *
 * Description:  
 * Function to Set Config: POS Watermarks, 
 *                         POS Flow Control, 
 *                         Chip's GFC Role
 *
 * Conditions for Use:
 *    none
 *
 * Notes:
 *    
 *   bb_ChipData_t*   pChipData,  ->Initialized Chip Data 
 *   bb_6048_SelCfg_e  SelCfg,    To select type of config to set 
 *   int              ChanNum,    Channel Num = 0-3 
 *   void*            pCfg        -> configuration structure, of type
 *                                   ixf6048_PosWatermarks_t, or
 *                                   ixf6048_TxPosFlowControl_t, or
 *                                   ixf6048_GfcRole_t 
 *
 * End
 **********************************************************************/
#if 0
bb_Error_e ixf6048_SetCfg (bb_ChipData_t*    pChipData,
    uint              ChanNum,
    bb_6048_SelCfg_e  SelCfg,
    void*             pCfg)
{
    ixf6048_PosWatermarks_t*    pPosWatermarks;
    ixf6048_TxPosFlowControl_t* pTxPosFlowControl;
    ixf6048_GfcRole_t*          pGfcRole;
    register ushort             RegVal = 0;
    register ushort*            pReg;
    register ushort*            BaseAddress= pChipData->BaseAddress;


    /* Validate Channel number */
    if (IXF6048_VALIDATE_CHANNEL(ChanNum) == FALSE)
    {
        return bb_INVALID_NUM_CHAN_CFG;
    }

    switch(SelCfg)
    {
    case ixf6048_eCFG_POS_WATERMARKS: /*ixf6048_PosWatermarks_t */
        pPosWatermarks = pCfg;

        SetPosWatermarks(BaseAddress,
            ChanNum, 
            ixf6048_eSIDE_BOTH,
            pPosWatermarks);

        break;

    case ixf6048_eCFG_POS_FLOW_CONTROL:/* ixf6048_TxPosFlowControl_t */
        pTxPosFlowControl = pCfg;

        /* build the entire Register. 
        Note: Both methods 1 and 2 can be active simultaneously */

        /* Method 1. max(1,L/m) OR max(1,L*m) */
        if (pTxPosFlowControl->TxPosProportionalFill == ixf6048_eTX_POS_FILL_LEN_DIV_2_EXP_N)
        {
            Ixf6048SetBitVal(RegVal,ixf6048_bXMT_IPG_REL_EN); /*IPGRelCfg = 0 */
            RegVal |= ((pTxPosFlowControl->N <<10) & ixf6048_mXMT_IPG_REL);
        }
        else if ( pTxPosFlowControl->TxPosProportionalFill == ixf6048_eTX_POS_FILL_LEN_MULT_2_EXP_N)
        {
            RegVal |= (ixf6048_bXMT_IPG_REL_EN | ixf6048_bXMT_IPG_REL_CFG);
            RegVal |= ((pTxPosFlowControl->N <<10) & ixf6048_mXMT_IPG_REL);
        }

        /* Method 2. max(1, M) -- Where M is Constant Value defined below */
        if (pTxPosFlowControl->TxPosConstantFill == ixf6048_eTX_POS_FILL_MULT_1)
        {
            RegVal |= ixf6048_bXMT_IPG_ABS_EN;   /* IPGAbsCfg = 0 */
            RegVal |= (pTxPosFlowControl->NumInterframeFlags | ixf6048_mXMT_IPG_ABS);
        }
        else if (pTxPosFlowControl->TxPosConstantFill == ixf6048_eTX_POS_FILL_MULT_256)
        {
            RegVal |= (ixf6048_bXMT_IPG_ABS_EN | ixf6048_bXMT_IPG_ABS_CNF);
            RegVal |= (pTxPosFlowControl->NumInterframeFlags | ixf6048_mXMT_IPG_ABS);
        }

        /* Set the Register */
        WRITE_TO_HW((BaseAddress + ixf6048_T_IPGCTRL_CHAN_OFFSET(ChanNum)), RegVal);
        break;

    case ixf6048_eCFG_GFC_ROLE: /* ixf6048_GfcRole_t  -- */
        pGfcRole = pCfg;
        pReg = BaseAddress + ixf6048_T_ACPCNF_CHAN_OFFSET(ChanNum);
        /* Clear Controller/Controlled Bits Prior to Configure */
        Ixf6048ClrBit(pReg, (ixf6048_bXMT_GFC_CONTROLLER | ixf6048_bXMT_GFC_CONTROLLED ));

        if (pGfcRole->GfcRole == ixf6048_eGFC_CONTROLLER) /* this chip's GFC role */
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_GFC_CONTROLLER);
        else if (pGfcRole->GfcRole == ixf6048_eGFC_CONTROLLED)
            Ixf6048SetBitVal(RegVal, ixf6048_bXMT_GFC_CONTROLLED);
        else  /* (ixf6048_eGFC_NOT_ACTIVE) */
            Ixf6048ClrBitVal(RegVal, (ixf6048_bXMT_GFC_CONTROLLER | ixf6048_bXMT_GFC_CONTROLLED)) ;

        /* Clear Halt Ratio Bits Prior to Configure */
        Ixf6048ClrBit(pReg, ixf6048_mXMT_GFC_CNT);

        ixf6048_SET_HALT_RATIO(RegVal,pGfcRole->GfcHaltRatio); /* GFC Halt Ratio (if controller) (0-7) */

        /* Set Bits into T_ACPCNF Reg */
        WRITE_TO_HW(pReg, READ_FROM_HW(pReg) | RegVal);
        break;
    default :
        return (bb_INV_SEL_CFG_TYPE);
    }
    return bb_NO_ERROR;
}
#endif
